/*  Fix the negative pB at the poles.

    This code was written by Alice Lecinski, September 2001.

    It is modeled after David Elmore's mk4 calibraton
    routines written as idl procedures.  David developed these
    procedures in fall of 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>

#include "mk4v2.h"

int FixNegpBatP( Img, FixedImg , xdim, ydim )
    float *Img;
    float *FixedImg;
    int    xdim,ydim;
{
    int   ii,jj, hxdim , hsamplesz, samplesz;
    float *pFix, *pF;
    float Sum, ovCount, Avg;

    /* Sample the region +-10 azimuths near the
       poles.  If the avg is less than zero,
       fix the entire height by subtracting the negative avg.
    */

    hxdim     = xdim     >>1;
    samplesz  = 20          ;
    hsamplesz = samplesz >>1;
    ovCount   = 1. / ((float)(samplesz * 2));

    VVcopy(Img,FixedImg,xdim*ydim);

    for(jj=0;jj<ydim;jj++){

	Sum  = 0.;
	pFix = FixedImg + jj*xdim;

	pF   = pFix;
	for(ii=0;ii<hsamplesz;ii++) {
	    Sum += *pF;
	    pF++;
	}
	pF = pFix + hxdim - hsamplesz;
	for(ii=0;ii< samplesz;ii++) {
	    Sum += *pF;
	    pF++;
	}
	pF = pFix +  xdim - hsamplesz;
	for(ii=0;ii<hsamplesz;ii++) {
	    Sum += *pF;
	    pF++;
	}

	if(Sum< 0.) {
	    Avg  = Sum * ovCount;
	    Vminus(pFix,Avg,xdim);
	    /*
	    fprintf(stderr,"Bad height %d Avg %f \n",jj,Avg);
	    */
	}
    }


}
