/*  This file contains various Matrix/Vector operations.

    This code was written by Alice Lecinski, September 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>

/* The vector should be of dimension "ydim" */

int MVplus(A,B,C,xdim,ydim)  /* A+B = C */
    float *A,*B,*C;
    int xdim,ydim;
{
    int ii,jj;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(jj=0;jj<ydim;jj++){   
	for(ii=0;ii<xdim;ii++){   
	    *pC = *pA + *pB ;
	    pA++; 
	    pC++; 
	}
	pB++;
    } 
}

/* The vector should be of dimension "ydim" */

int MVminus(A,B,C,xdim,ydim)  /* A-B = C */
    float *A,*B,*C;
    int xdim,ydim;
{
    int ii,jj;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(jj=0;jj<ydim;jj++){   
	for(ii=0;ii<xdim;ii++){   
	    *pC = *pA - *pB ;
	    pA++; 
	    pC++; 
	}
	pB++;
    } 
}




/* The vector should be of dimension "ydim" */

int MVmult(A,B,C,xdim,ydim)  /* A*B = C */
    float *A,*B,*C;
    int xdim,ydim;
{
    int ii,jj;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(jj=0;jj<ydim;jj++){   
	for(ii=0;ii<xdim;ii++){   
	    *pC = *pA * (*pB);
	    pA++; 
	    pC++; 
	}
	pB++;
    } 
}


/* The vector should be of dimension "ydim" */

int MVdivide(A,B,C,xdim,ydim)  /* A/B = C */
    float *A,*B,*C;
    int xdim,ydim;
{
    int ii,jj;
    float *pA,*pB,*pC , ftemp;

    pA=A;
    pB=B;
    pC=C;
    for(jj=0;jj<ydim;jj++){   
	ftemp = ( *pB != 0. ) ? (1./(*pB)) : 0. ;
	for(ii=0;ii<xdim;ii++){   
	    *pC = *pA * ftemp;
	    pA++; 
	    pC++; 
	}
	pB++;
    } 
}





/* The vector can be either of dimension "xdim" or "ydim" */


int MVplusReplace(A,B,xdim,ydim , vdim)  /* A = A+B  */
    float *A,*B;
    int xdim,ydim;
    int vdim;
{
    int ii,jj;
    float *pA,*pB;
    float C;

    if(vdim==ydim) {  /* Vector is of dimension "ydim". */
	pA=A;
	pB=B;
	for(jj=0;jj<ydim;jj++){   
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA + *pB ;
		*pA = C;
		pA++; 
	    }
	    pB++;
	} 
    }
    else {           /* Vector is of dimension "xdim" */
	pA=A;
	for(jj=0;jj<ydim;jj++){   
	    pB=B;
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA + *pB ;
		*pA = C;
		pA++; 
		pB++;
	    }
	} 
    }
}

/* The vector can be either of dimension "xdim" or "ydim" */

int MVminusReplace(A,B,xdim,ydim, vdim)  /* A = A+B  */
    float *A,*B;
    int xdim,ydim;
    int vdim;
{
    int ii,jj;
    float *pA,*pB;
    float C;

    if(vdim==ydim) {  /* Vector is of dimension "ydim". */
	pA=A;
	pB=B;
	for(jj=0;jj<ydim;jj++){   
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA - *pB ;
		*pA = C;
		pA++; 
	    }
	    pB++;
	} 
    }
    else {           /* Vector is of dimension "xdim" */
	pA=A;
	for(jj=0;jj<ydim;jj++){   
	    pB=B;
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA - *pB ;
		*pA = C;
		pA++; 
		pB++;
	    }
	} 
    }
}



/* The vector can be either of dimension "xdim" or "ydim" */


int MVmultReplace(A,B,xdim,ydim, vdim)  /* A = A*B */
    float *A,*B;
    int xdim,ydim;
    int vdim;
{
    int ii,jj;
    float *pA,*pB;
    float C;

    if(vdim==ydim) {  /* Vector is of dimension "ydim". */
	pA=A;
	pB=B;
	for(jj=0;jj<ydim;jj++){   
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA * (*pB);
		*pA = C;
		pA++; 
	    }
	    pB++;
	} 
    }
    else {           /* Vector is of dimension "xdim" */
	pA=A;
	for(jj=0;jj<ydim;jj++){   
	    pB=B;
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA * (*pB);
		*pA = C;
		pA++; 
		pB++;
	    }
	} 
    }
}


/* The vector can be either of dimension "xdim" or "ydim" */

int MVdivideReplace(A,B,xdim,ydim, vdim)  /* A = A/B  */
    float *A,*B;
    int xdim,ydim;
    int vdim;
{
    int ii,jj;
    float *pA,*pB, C, ftemp;

    if(vdim==ydim) {  /* Vector is of dimension "ydim". */
	pA=A;
	pB=B;
	for(jj=0;jj<ydim;jj++){   
	    ftemp = ( *pB != 0. ) ? (1./(*pB)) : 0. ;
	    for(ii=0;ii<xdim;ii++){   
		C   = *pA * ftemp;
		*pA = C;
		pA++; 
	    }
	    pB++;
	} 
    }
    else {           /* Vector is of dimension "xdim" */
	pA=A;
	for(jj=0;jj<ydim;jj++){   
	    pB=B;
	    for(ii=0;ii<xdim;ii++){   
		/* Very stinkin slow way to do it. */
		ftemp = ( *pB != 0. ) ? (1./(*pB)) : 0. ;
		C   = *pA * ftemp;
		*pA = C;
		pA++; 
		pB++;
	    }
	} 
    }
}



