function azel,ra,d,stg,lat,lon
; azel gives the azimuth and elevation for a celestial
; object, given
; ra = right ascension in decimaldegrees
; d = declination in decimal degrees
; stg = Sidereal Time at Greenwich
; lat = lattitude of site, decimal degrees, positive north
; lon = longitude of site, decimal degrees, positive west of Greenwich
; az = different than in Meeus, 0 is north and is positive
;     if measured east, all I do is add 180deg
; el = 0 is horizon, negative below, positive above

ra = double(ra)
d = double(d)
stg = double(stg)
lat = double(lat)
lon = double(lon)

dr = !pi/180.0d
rd = 180.0d/!pi
h = 15.0d*stg - lon - ra   ; in decimal degrees

; do the tan(az) think to get the quadrant right
sinaz = sin(h*dr)
cosaz = cos(h*dr)*sin(lat*dr) - tan(d*dr)*cos(lat*dr)
az = rd*atan(sinaz/cosaz)

negatcosaz = where(cosaz lt 0.0d)
if negatcosaz(0) ge 0L then az(negatcosaz) = az(negatcosaz) + 180.0

negataz = where(az lt 0.0d)
if negataz(0) ge 0L then az(negataz) = 360.0 + az(negataz)

az = ((az + 180.0d) mod 360.0d)

el = rd*asin(sin(lat*dr)*sin(d*dr) + cos(lat*dr)*cos(d*dr)*cos(h*dr))
output = [az,el]
return, output
end
