pro solmerx,yr,month,day
; calculates the solar meridonal crossing to the nearest second
; & prints out the Az-El & RA-Dec or the crossing for Boulder, CO
;
; & prints out the Az-El & RA-Dec or the crossing for Dunlod, India.
;
; & prints out the Az-El & RA-Dec or the crossing for Darchan, Mongolia.
;
; DST flags Daylight Saving Time

if keyword_set(DST) then local2ut = 6.0d else local2ut = 7.0d

; latitude and longitude of Boulder, CO
; lat =  40.035100d
; lon = 105.242717d
; local2ut = 7.0d
; latitude and longitude of Dunlod, India
; lat = 27.901667d
; lon = 360.0d - 75.226667
; local2ut = -5.5d
; latitude and longitude of Darchan, Mongolia
lat = 49.4667d
lon = 360.0d - 105.9333
local2ut = -8.0d
localstart = 11.0d + (40.0d / 60.0d)
localend = 12.0d + (20.0d / 60.0d)
deltaT = 1.0d/3600.0d/3.0d ; (calculate every 3rd of a second)

yr = double(yr)
month = double(month)
day = double(day)
localstart = double(localstart)
localend= double(localend)
deltaT = double(deltaT)

ntimes = long((localend - localstart)/deltaT)+1L

UT = dindgen(1,ntimes)*deltaT + localstart + local2ut
nextday = where(UT ge 24.0d)

; create the date array
date = dblarr(4,ntimes)
date(0,*) = date(0,*)+yr
date(1,*) = date(1,*)+month
date(2,*) = date(2,*)+day

ldate=date
ldate(3,*) = dindgen(1,ntimes)*deltaT + localstart

if nextday(0) ge 0L then begin
	UT(nextday) = UT(nextday)-24.0d
	date(2,nextday) = date(2,nextday)+1.0d
endif

; need a month/year filter for month/year boundaries

date(3,*) = date(3,*)+UT

sazel = sunazel(date, lat, lon)

; find the meridonal crossing (where az = 180deg)
xc = abs(sazel(0,*) - 180.0d)
xci = where(xc eq min(xc))

strdate = strtrim(fix(month),2)+'/'+strtrim(fix(day),2)+'/'+strtrim(fix(yr),2)

print,' '
;print, 'The '+strdate+' meridonal crossing (Boulder, CO):'
;print, 'The '+strdate+' meridonal crossing (Dunlod, India):'
print, 'The '+strdate+' meridonal crossing (Darchan, Mongolia):'

; convert the decimal hrs to hr:mm:ss.sss
xchr = fix(UT(xci))
xcm = fix(60.0d*(UT(xci)-xchr))
xcs = 60.0d*(60.0d*(UT(xci)-xchr) - xcm)

sxcm = strtrim(xcm,2)
if xcm(0) lt 10 then sxcm = '0'+strtrim(xcm,2)

sxcs= strtrim(xcs,2)
if xcs(0) lt 10 then sxcs = '0'+sxcs
sxcs = strmid(sxcs,0,5)

; convert the RA degrees to hr:mm:ss.sss
rr = sazel(2,xci)/15.0d
rhr = fix(rr)
rm = fix(60.0d*(rr-rhr))
rs = 60.0d*(60.0d*(rr-rhr) - rm)

srm = strtrim(rm,2)
if rm(0) lt 10 then srm = '0'+strtrim(rm,2)

srs= strmid(strtrim(rs,2),0,6)
if rs(0) lt 10 then srs = '0'+srs
srs = strmid(srs,0,5)


print,'   UT: '+strdate+' '+strtrim(xchr,2)+':'+sxcm+':'+sxcs
print,'   Solar Position: RA: '+strtrim(rhr,2)+':'+srm+':'+srs
print,'                   Dec:'+strtrim(sazel(3,xci),2)
print,'                   Az: '+strtrim(sazel(0,xci),2)
print,'                   El: '+strtrim(sazel(1,xci),2)

return
end
