/*  This file contains various Vector operations.

    This code was written by Alice Lecinski, September 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>

int VVplus(A,B,C,n)  /* A+B = C */
    float *A,*B,*C;
    int n;
{
    int ii;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(ii=0;ii<n;ii++){   
	*pC = *pA + *pB ;
	pA++; 
	pB++;
	pC++; 
    } 
}

int VVminus(A,B,C,n)  /* A-B = C */
    float *A,*B,*C;
    int n;
{
    int ii;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(ii=0;ii<n;ii++){   
	*pC = *pA - *pB ;
	pA++; 
	pB++;
	pC++; 
    } 
}

int VVmult(A,B,C,n)  /* A*B = C */
    float *A,*B,*C;
    int n;
{
    int ii;
    float ftmp;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(ii=0;ii<n;ii++){   
	ftmp  = *pA;
	ftmp *= *pB;
	*pC   = ftmp;
	pA++; 
	pB++;
	pC++; 
    } 
}

int VVdivide(A,B,C,n)  /* A/B = C */
    float *A,*B,*C;
    int n;
{
    int ii;
    float *pA,*pB,*pC;

    pA=A;
    pB=B;
    pC=C;
    for(ii=0;ii<n;ii++){   
	*pC = (*pB == 0.) ? 0. : *pA / *pB ;
	pA++; 
	pB++;
	pC++; 
    } 
}

int VVcopy(A,B,n)  /* Copy A to B  */
    float *A,*B;
    int n;
{
    int ii;
    float *pA,*pB;

    pA=A;
    pB=B;
    for(ii=0;ii<n;ii++){   
	*pB = *pA ;
	pA++; 
	pB++;
    }
}
int Vplus(A,value,n)  /* A = A + value  */
    float *A;
    float value;
    int n;
{
    int ii;
    float *pA;

    pA=A;
    for(ii=0;ii<n;ii++){   
	*pA += value;
	pA++; 
    }
}
int Vminus(A,value,n)  /* A = A - value  */
    float *A;
    float value;
    int n;
{
    int ii;
    float *pA;

    pA=A;
    for(ii=0;ii<n;ii++){   
	*pA -= value;
	pA++; 
    }
}
int Vmult(A,value,n)  /* A = A * value  */
    float *A;
    float value;
    int n;
{
    int ii;
    float *pA;

    pA=A;
    for(ii=0;ii<n;ii++){   
	*pA *= value;
	pA++; 
    }
}

int Vover(A,n)  /* A = 1. / A */
    float *A;
    int n;
{
    int ii;
    float value, ovvalue;
    float *pA;

    pA=A;
    for(ii=0;ii<n;ii++){   
	value = *pA;
	ovvalue = ( value != 0. ) ? 1./value : 0. ;
	*pA = ovvalue;
	pA++; 
    }
}


int Vinitialize(A,value,n)  /* A = value  */
    float *A;
    float value;
    int n;
{
    int ii;
    float *pA;

    pA=A;
    for(ii=0;ii<n;ii++){   
	*pA = value;
	pA++; 
    }
}
int iVminus(A,value,n)  /* A = A - value  */
    int *A;
    int value;
    int n;
{
    int ii;
    int *pA;

    pA=A;
    for(ii=0;ii<n;ii++){   
	*pA -= value;
	pA++; 
    }
}
