/*	abcd_lat.c	spherical trigonometry at SITE_LAT		*/
/*									*/
/*	Implements the fundamental spherical trigonometry equations of	*/
/* Smart: Text-Book on Spherical Astronomy, eq. A, B, C, and D.		*/
/*									*/
/*	These equations can be used to convert between coordinate	*/
/* systems, as follows:							*/
/*									*/
/* 1)   Let b=hour angle		2)   Let b=azimuth		*/
/*          c=declination			 c=elevation		*/
/*     Then x=azimuth			    Then x=hour angle		*/
/*          y=elevation				 y=declination		*/
/*	    z=parallatic angle			 z=parallatic angle	*/
/*									*/
/*	08 FEB 85: phw							*/
/************************************************************************/

#ifdef	VxWorks
#include "vxWorks.h"
#else
#include "astro.h"
#endif

#ifdef VxWorks
#include	"math.h"
#include	"site.h"
#include	"astro.h"
#endif

/*
void abcd_lat( ANGLE	b, ANGLE c, ANGLE *x, ANGLE *y, ANGLE *z)

	Convert from Ha & Dec to Az, El, Beta

	abcd_lat(Ha, Dec, &Az, &El, &Beta);


	2 sin, 2 cos, 2 atan2, asin, 9 mul, 3 add or subs,
*/

void
abcd_lat( b, c, x, y, z)
ANGLE	b, c, *x, *y, *z;
{
	FAST AMP	sina = SIN_LAT, cosa = COS_LAT;
	FAST AMP	sinb, cosb, sinc, cosc;
	FAST AMP	cosz;

  	sinb = SIN(b);	cosb = COS(b);
	sinc = SIN(c);	cosc = COS(c);
	*x = ATAN2(-cosc*sinb,(sinc*cosa - cosc*cosb*sina));
	cosz = sina*cosc - sinc*cosa*cosb;
	*z = ATAN2(cosa*sinb,cosz);
	*y = ASIN(sinc*sina + cosc*cosb*cosa);
	}
