;+
; NAME
;	dpmsq
;
; PURPOSE
;	convert ACOS/DPM images to 800x800 pixels.
;
; SYNTAX
;	dpmsq, 'dpmimages.ls'
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   16 Dec 1998
;-

PRO dpmsq, filelist

   nx = 800
   ny = 800

   file = ''

   GET_LUN, lun
   OPENR, lun, filelist
   WHILE (NOT EOF (lun)) DO			$
   BEGIN
      READF, lun, file, FORMAT = '(A)'
      PRINT, 'Read  file: ', file
      img = readfits (file, hdu)
      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      xcen     = fxpar (hdu, 'CRPIX1')
      ycen     = fxpar (hdu, 'CRPIX2')
      cdelt1   = fxpar (hdu, 'CDELT1')
      cdelt2   = fxpar (hdu, 'CDELT2')
      crradius = fxpar (hdu, 'CRRADIUS')
      pang     = fxpar (hdu, 'SOLAR_P0')
      datamin  = fxpar (hdu, 'DATAMIN')
      datamax  = fxpar (hdu, 'DATAMAX')
      dispmin  = fxpar (hdu, 'DISPMIN')
      dispmax  = fxpar (hdu, 'DISPMAX')

      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax
      xbeg = (xcen - nx / 2)
      ybeg = (ycen - ny / 2)
      xcen = (nx / 2) - 0.5
      ycen = (ny / 2) - 0.5 
      imgsq = intarr (nx, ny)
      imgsq = img (xbeg : xbeg +  nx - 1, ybeg : ybeg +  ny - 1)
      nxcen = FIX (xcen + 0.5)
      nycen = FIX (ycen + 0.5)
      mag = 1.0
      roll = pang
      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, INTERP=Cubic)
      minsq = min (imgsq)
      maxsq = max (imgsq)
;      imglo = rebin (imgsq, nx / 2, ny / 2)
      imglo = imgsq
;      xcen = (nx / 2) - 0.5 
;      ycen = (ny / 2) - 0.5
;      xcen = (xcen - xbeg) / 2
;      ycen = (ycen - ybeg) / 2
      minlo = min (imglo)
      maxlo = max (imglo)
      PRINT, 'minsq/maxsq: ', minsq, maxsq
      PRINT, 'minlo/maxlo: ', minlo, maxlo

      fxaddpar, hdu, 'CRPIX1', xcen
      fxaddpar, hdu, 'CRPIX2', ycen
      fxaddpar, hdu, 'CDELT1', cdelt1 * 2.0
      fxaddpar, hdu, 'CDELT2', cdelt2 * 2.0
      fxaddpar, hdu, 'CRRADIUS', crradius / 2.0

      extpos = strpos (file, '.fts')
      outfile = strmid (file, 0, extpos) + '.sq.fts'
      PRINT, 'Write file: ', outfile
      writefits, outfile, imglo, hdu
   END
   CLOSE, LUN
   
END
