;+
; fixchipbsh.pro   IDL procedure
;
; PURPOSE
;	Read a CHIP FITS file and fix the TYPE-OBS parameter.
;	For 2001 DOY 001 - 017, it was incorrectly set to "BLUE".
;	Change it to "NEAR BLUE".
;
; SYNTAX
;	fixchipbsh, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   HAO/NCAR   25 January 2001
;-

pro fixchipbsh, fits_list

fits_name = ''				; Define FITS file name variable.
wave_code = 'BSH     '
wave_name = 'NEAR BLUE'
comment1 = ' TYPE OF OBSERVATION [CHARACTER CODE]'
comment2 = ' TYPE OF OBSERVATION [DESCRIPTION]'

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.

      print, fits_name

      ;--- Change CODE-OBS parameter.

      fxhmodify, fits_name, 'CODE-OBS',  wave_code, comment1
      fxhmodify, fits_name, 'TYPE-OBS',  wave_name, comment2
   END
CLOSE, 1

 END
