;+
; NAME	fts2srfvid.pro
;
; PURPOSE	Label & Convert FITS images to Sun Rasterfile format.
;
; SYNTAX	fts2srfvid, 'fits_list'
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   25 January 2000
;-

pro fts2srfvid, fts_list

smonth = strarr (12)
smonth [0] = 'Jan'
smonth [1] = 'Feb'
smonth [2] = 'Mar'
smonth [3] = 'Apr'
smonth [4] = 'May'
smonth [5] = 'Jun'
smonth [6] = 'Jul'
smonth [7] = 'Aug'
smonth [8] = 'Sep'
smonth [9] = 'Oct'
smonth [10] = 'Nov'
smonth [11] = 'Dec'

CLOSE, 1

red = bindgen (256)
green = red
blue  = red
fts_name = ''
lct, '/home/stanger/color/bwy.lut'	; color table.
tvlct, red, green, blue, /get

dxvid = 640
dyvid = 480
dxv = 560
dyv = 400

;--- File list loop.

loop = 0
OPENR, 1, fts_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{
   loop = loop + 1

   ;--- Read GIF image.

   READF, 1, fts_name					; Get file name.
print, 'fts_name: ', fts_name
   fts_img = readfits (fts_name, hdu)

   dx = fxpar (hdu, 'NAXIS1')
   dy = fxpar (hdu, 'NAXIS2')
   xcen = fxpar (hdu, 'CRPIX1')
   ycen = fxpar (hdu, 'CRPIX2')
   bitpix = fxpar (hdu, 'BITPIX')
   fts_min = fxpar (hdu, 'DISPMIN')
   fts_max = fxpar (hdu, 'DISPMAX')
   obs_date = fxpar (hdu, 'OBS-DATE')
   obs_time = fxpar (hdu, 'OBS-TIME')

   byt_img = bytscl (fts_img, min=fts_min, max=fts_max)

   nbypix = bitpix / 8
   xmag = dxv / dx
   ymag = dyv / dy
   if (xmag LT ymag) then mag = xmag else mag = ymag

   mag_img = rot (byt_img, 0.0, mag, xcen, ycen)
   mag_size = size (mag_img)
   dxm = mag_size [1]
   dym = mag_size [2]
   dsp_img = bytscl (mag_img, min=fts_min, max=fts_max)
   x1 = (dxv - dxm) / 2
   y1 = (dyv - dym) / 2
   vid_img = bytarr (dxvid, dyvid)
   vid_img [x1:x1+dxm-1, y2:y1+dym-1] = disp_img

   ;--- Find dimensions of image.
   ;--- Set label location as 10% up and centered horizontally.

   sizimg = size (vid_img)
   xdim = sizimg [1]
   ydim = sizimg [2]
   xloc1 = xdim / 2
   yloc1 = ydim - 40
   xloc2 = xdim / 2
   yloc2 = ydim - 60
   print, 'xloc1, yloc1: ', xloc1, yloc1
   print, 'xloc2, yloc2: ', xloc2, yloc2

;   IF (loop EQ 1) THEN	$
;   BEGIN
;      window, xsize=xdim, ysize=ydim
;      tvlct, red, green, blue				; Write color map.
;   END

   ;--- Find intensity min/max.

;   GIF_min = MIN (fts_img, max=fts_max)
   print, FORMAT = '("fts_min/max: ", E10.3, E10.3)', fts_min, fts_max

   ;--- Compute square root of image.

;   new_img = sqrt (fts_img)				; Compute square root.

;   new_min = min (new_img, max=new_max)		; Get image min/max.
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max
;   IF (new_min LT 0.0) THEN	$			; Symmetric about zero.
;      IF (-new_min LT new_max) THEN new_min = -new_max  ELSE new_max = -new_min
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   new_min = -5.00e-8
;   new_max =  5.00e-8
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

   ;--- Convert to byte pixels.
   ;--- Display image.

;   byt_img = bytscl (new_img, min=new_min, max=new_max, top=249)
;   tv, byt_img						; Display image.

   tv, fts_img
;   year   = strmid (fts_name,  0, 2)
;   month  = strmid (fts_name,  2, 2)
;   day    = strmid (fts_name,  4, 2)
;   hour   = strmid (fts_name,  7, 2)
;   minute = strmid (fts_name,  9, 2)
;   second = strmid (fts_name, 11, 2)

   year    = strmid (obs_date, 0, 2)
   month   = strmid (obs_date, 3, 2)
   day     = strmid (obs_date, 6, 2)

   hour    = strmid (obs_time, 0, 2)
   minute  = strmid (obs_time, 3, 2)
   second  = strmid (obs_time, 6, 2)

   monthname = smonth [fix (month) - 1]
   date = day + ' ' + monthname + ' ' + '19' + year
   time = hour + ':' + minute + ':' + second + ' ' + 'UT'
   datetime = date + '   ' + time

   print, 'date: ', date
   print, 'time: ', time
   print, 'datetime: ', datetime
;   xyouts, xloc, yloc, datetime, charthick=2.0, /device, alignment=0.5
   xyouts, xloc1, yloc1, date, /device, alignment=0.5, charthick=2.0
   xyouts, xloc2, yloc2, time, /device, alignment=0.5, charthick=2.0

   new_img = tvrd ()

   ;--- Write newference image to disk.

   sloc = RSTRPOS (fts_name, '.fts')
   new_name = strmid (fts_name, 0, sloc) + '.vid.sun'
   print, 'Write output image : ', new_name
   new_img = reverse (vid_img, 2)			; Invert for WRITE_SRF.
   write_srf, new_name, new_img, red, green, blue	; Write new image.
END	;}

CLOSE, 1
END
