;+
; NAME	fts2vid.pro
;
; PURPOSE	Label & Convert FITS images to Sun Rasterfile format.
;
; SYNTAX	fts2vid, 'fits_list'
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   25 January 2000
;-

pro fts2vid, fts_list

smonth = strarr (12)
smonth [0] = 'Jan'
smonth [1] = 'Feb'
smonth [2] = 'Mar'
smonth [3] = 'Apr'
smonth [4] = 'May'
smonth [5] = 'Jun'
smonth [6] = 'Jul'
smonth [7] = 'Aug'
smonth [8] = 'Sep'
smonth [9] = 'Oct'
smonth [10] = 'Nov'
smonth [11] = 'Dec'

CLOSE, 1

red = bindgen (256)
green = red
blue  = red
fts_name = ''
;lct, '/home/stanger/color/bwy.lut'	; color table.
lct, '/home/stanger/color/qualy.lut'	; color table.
;lct, '/home/stanger/color/qual.lut'	; color table.
tvlct, red, green, blue, /get

dxvid = 640
dyvid = 480
dxv = 560
dyv = 400
window, xsize=dxvid, ysize=dyvid

;--- File list loop.

loop = 0
OPENR, 1, fts_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{
   loop = loop + 1

   ;--- Read GIF image.

   READF, 1, fts_name					; Get file name.
print, 'fts_name: ', fts_name
   fts_img = readfits (fts_name, hdu)

   dx = fxpar (hdu, 'NAXIS1')
   dy = fxpar (hdu, 'NAXIS2')
   xcen = fxpar (hdu, 'CRPIX1')
   ycen = fxpar (hdu, 'CRPIX2')
   bitpix = fxpar (hdu, 'BITPIX')
   fts_min = fxpar (hdu, 'DISPMIN')
   fts_max = fxpar (hdu, 'DISPMAX')
   obs_date = fxpar (hdu, 'DATE-OBS')
   obs_time = fxpar (hdu, 'TIME-OBS')
   print, 'dx, dy: ', dx, dy
   print, 'fts_min, fts_max: ', fts_min, fts_max

;   byt_img = bytscl (fts_img, min=fts_min, max=fts_max, top=249)
   byt_img = bytscl (fts_img, min=fts_min, max=fts_max, top=255)
;   tv, byt_img

   nbypix = bitpix / 8
   xmag = float(dxv) / float(dx)
   ymag = float(dyv) / float (dy)
   if (xmag LT ymag) then mag = xmag else mag = ymag
   print, 'dxv, dyv: ', dxv, dyv
   print, 'mag: ', mag
   print, 'xcen, ycen: ', xcen, ycen

;   mag_img = rot (byt_img, 0.0, mag, xcen, ycen)
   IF (dxv LT dyv) THEN		$
      mag_img = congrid (byt_img, dxv, dxv)	$
   ELSE						$
      mag_img = congrid (byt_img, dyv, dyv)	

;   tv, mag_img

   mag_size = size (mag_img)
   dxm = mag_size [1]
   dym = mag_size [2]
   print, 'dxm, dym: ', dxm, dym
;   dsp_img = bytscl (mag_img, min=fts_min, max=fts_max)
   dsp_img = mag_img
   x1 = (dxvid - dym) / 2
   y1 = (dyvid - dym) / 2
   vid_img = bytarr (dxvid, dyvid)
   vid_img [x1:x1+dxm-1, y1:y1+dym-1] = dsp_img [*, *]

   ;--- Find dimensions of image.
   ;--- Set label location as 10% up and centered horizontally.

   sizimg = size (vid_img)
   xdim = sizimg [1]
   ydim = sizimg [2]
   xloc1 = xdim / 2
   yloc1 = ydim - 40
   xloc2 = xdim / 2
   yloc2 = ydim - 60
   print, 'xloc1, yloc1: ', xloc1, yloc1
   print, 'xloc2, yloc2: ', xloc2, yloc2

;   IF (loop EQ 1) THEN	$
;   BEGIN
;      window, xsize=dxvid, ysize=dyvid
;      tvlct, red, green, blue				; Write color map.
;   END

   ;--- Find intensity min/max.

;   GIF_min = MIN (fts_img, max=fts_max)
   print, FORMAT = '("fts_min/max: ", E10.3, E10.3)', fts_min, fts_max

   ;--- Compute square root of image.

;   new_img = sqrt (fts_img)				; Compute square root.

;   new_min = min (new_img, max=new_max)		; Get image min/max.
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max
;   IF (new_min LT 0.0) THEN	$			; Symmetric about zero.
;      IF (-new_min LT new_max) THEN new_min = -new_max  ELSE new_max = -new_min
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   new_min = -5.00e-8
;   new_max =  5.00e-8
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

   ;--- Convert to byte pixels.
   ;--- Display image.

;   byt_img = bytscl (new_img, min=new_min, max=new_max, top=249)
;   tv, byt_img						; Display image.

   tv, vid_img
;   year   = strmid (fts_name,  0, 2)
;   month  = strmid (fts_name,  2, 2)
;   day    = strmid (fts_name,  4, 2)
;   hour   = strmid (fts_name,  7, 2)
;   minute = strmid (fts_name,  9, 2)
;   second = strmid (fts_name, 11, 2)

   year    = strmid (obs_date, 0, 4)
   month   = strmid (obs_date, 5, 2)
   day     = strmid (obs_date, 8, 2)
   print, 'obs_date: ', obs_date
   print, 'year, month, day: ', year, month, day

   hour    = strmid (obs_time, 0, 2)
   minute  = strmid (obs_time, 3, 2)
   second  = strmid (obs_time, 6, 2)
   print, 'obs_time: ', obs_time
   print, 'hour, minute, second: ', hour, minute, second

   monthname = smonth [fix (month) - 1]
   date = day + ' ' + monthname + ' ' + year
   time = hour + ':' + minute + ':' + second + ' ' + 'UT'
   datetime = date + '   ' + time

   print, 'date: ', date
   print, 'time: ', time
   print, 'datetime: ', datetime
;   xyouts, xloc, yloc, datetime, charthick=2.0, /device, alignment=0.5
;   xyouts, xloc1, yloc1, date, /device, alignment=0.5, charthick=2.0, color=255
;   xyouts, xloc2, yloc2, time, /device, alignment=0.5, charthick=2.0, color=255

   new_img = tvrd ()

   ;--- Write newference image to disk.

   sloc = RSTRPOS (fts_name, '.fts')
   new_name = strmid (fts_name, 0, sloc) + '.vid.sun'
   print, 'Write output image : ', new_name
   new_img = reverse (new_img, 2)			; Invert for WRITE_SRF.
   write_srf, new_name, new_img, red, green, blue	; Write new image.
END	;}

CLOSE, 1
END
