;+
; ftsaddcrlong.pro   IDL procedure
;
; PURPOSE
;	Read a FITS file and add heliographic Carrington rotation & longitude.
;
; SYNTAX
;	ftsaddhg, 'fits_list'
;
; PROCEDURES USED
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   HAO/NCAR  16 December 2002
;-

pro ftsaddcrlong, fits_list

fits_name = ''				; Define FITS file name variable.
syear   = ''
smonth  = ''
sday    = ''
shour   = ''
sminute = ''
ssecond = ''

CLOSE, 11

print, 'fits_list: ', fits_list

OPENR, 11, fits_list			; Open list file.

WHILE (NOT EOF (11)) DO	$		; File processing loop.
   BEGIN
      READF, 11, fits_name		; Read FITS file name from list file.

      print, fits_name

      openr,   12, fits_name
      fxhread, 12, hdu			; Read FITS header.
      close,   12

      date = fxpar (hdu, 'DATE-OBS')
      time = fxpar (hdu, 'TIME-OBS')
      solar_lo = fxpar (hdu, 'SOLAR_L0')

      syear   = strmid (date, 0, 4)
      smonth  = strmid (date, 5, 2)
      sday    = strmid (date, 8, 2)

      shour   = strmid (time, 0, 2)
      sminute = strmid (time, 3, 2)
      ssecond = strmid (time, 6, 2)

      year   = FIX (syear)
      month  = FIX (smonth)
      day    = FIX (sday)

      hour   = FIX (shour)
      minute = FIX (sminute)
      second = FIX (ssecond)

      jd_crot, year, month, day, hour, minute, second, cr=crot, long=Lo, lat=Bo

      print, 'SOLAR_L0: ', solar_lo, ' Lo: ', Lo, ' crot: ', crot

      ;--- Add/Update Carrington Rotation number & Longitude in FITS header.

      fxhmodify, fits_name, 'CARR_ROT', crot, 'CARRINGTON ROTATION NUMBER'
      fxhmodify, fits_name, 'SOLAR_L0', Lo,   'HELIOGRAPHIC LONGITUDE'
   END
CLOSE, 11

 END
