;+
; NAME	ftsavg.pro
;
; PURPOSE	Perform square root scaling on FITS images.
;
; SYNTAX	sqrtfts, list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   29 November 1999
;-

pro ftsavg, fts_list

red = bindgen (256)
green = red
blue  = red
cme_name = ''
fts_name = ''
avg_name = 'avg.fts'
lct,'/home/stanger/color/bwy.lut'	; color table.
tvlct, red, green, blue, /get
counter = 0 ;
bzero  = 0.0
bscale = 1.0

;--- File list loop.

CLOSE, 1
OPENR, 1, fts_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fts_name				; Get file name.
   PRINT, 'fts_name: ', fts_name
   fts_img = readfits (fts_name, hdu, /noscale)
   fimg = float (fts_img)

   counter = counter + 1
   if (counter EQ 1) then	$
      BEGIN
	 xdim = fxpar (hdu, 'NAXIS1')
	 ydim = fxpar (hdu, 'NAXIS2')
	 bzero = fxpar (hdu, 'BZERO')
	 bscale = fxpar (hdu, 'BSCALE')
	 print, 'bzero, bscale: ', bzero, bscale
	 nbitpix = fxpar (hdu, 'BITPIX')
	 if (nbitpix EQ  8)  then sum_img = bytarr (xdim, ydim)
	 if (nbitpix EQ 16)  then sum_img = intarr (xdim, ydim)
	 if (nbitpix EQ 32)  then sum_img = lonarr (xdim, ydim)
	 if (nbitpix EQ -32) then sum_img = fltarr (xdim, ydim)
      END

   ;--- Extract information from header.

   disp_min = fxpar (hdu, 'DISPMIN') / bscale
   disp_max = fxpar (hdu, 'DISPMAX') / bscale
   tv, bytscl (fimg, min=disp_min, max=disp_max, top=249)

   fts_min = MIN (fimg, max=fts_max)
   print, FORMAT = '("fts_min/max: ", E10.3, E10.3)', fts_min, fts_max

   ;--- Compute image average.

   sum_img = sum_img + fimg				; Add image to sum.

   new_min = min (sum_img, max=new_max)			; Get image min/max.
   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max
;   IF (new_min LT 0.0) THEN	$			; Symmetric about zero.
;      IF (-new_min LT new_max) THEN new_min = -new_max  ELSE new_max = -new_min
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   new_min = -5.00e-8
;   new_max =  5.00e-8
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

   ;--- Convert to byte pixels.
   ;--- Display image.

;   byt_img = bytscl (sum_img, min=new_min, max=new_max, top=249)
;   tv, byt_img						; Display image.
END	;}

CLOSE, 1

   ;--- Compute average.

   avg_img = sum_img / float (counter)
   avg_img = FIX (avg_img)
;   avg_img = FIX ( (avg_img - bzero) / bscale )
;   fxaddpar, hdu, 'BZERO', bzero
;   fxaddpar, hdu, 'BSCALE', bscale
   
   ;--- Write newference image to disk.

;   sloc = RSTRPOS (fits_name, '.fts')
;   new_name = strmid (fits_name, 0, sloc) + '.sqrt.fts'
   print, 'Write average image : ', avg_name
   writefits, avg_name, avg_img, hdu			; Write new image.

END
