;+
; NAME	ftsdif.pro
;
; PURPOSE	Compute the difference between two FITS images.
;
; SYNTAX	ftsdif, fits_reference_file, fits_image_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   10 Sep 1998
;-

pro ftsdif, ref_name, cme_list

cme_name = ''
ref_hdu = ''
cme_hdu = ''
tscope = 'sun'
print, 'ref_name: ', ref_name
lct,'/home/stanger/color/dif.lut'	; load difference color table.

;--- Read reference image.

;ref_img    = rfits (ref_name, header=ref_hdu, /scale)
ref_img    = readfits (ref_name, ref_hdu)

ref_telescop = fxpar (ref_hdu, "TELESCOP")
ref_instrume = fxpar (ref_hdu, "INSTRUME")
ref_detector = fxpar (ref_hdu, "DETECTOR")
ref_date     = fxpar (ref_hdu, "DATE-OBS")
ref_time     = fxpar (ref_hdu, "TIME-OBS")
ref_hour     = strmid (ref_time, 0, 2)
ref_minute   = strmid (ref_time, 3, 2)

ref_min = MIN (ref_img, max=ref_max)
print, FORMAT = '("ref_min/max: ", E10.3, E10.3)', ref_min, ref_max

IF (STRPOS (ref_telescop, "SMM C/P") NE -1)	THEN  tscope = 'smmcp'
IF (STRPOS (ref_telescop, "MK3") NE -1)		THEN  tscope = 'mk3'
IF (STRPOS (ref_instrume, "SXT") NE -1)		THEN  tscope = 'sxt'
IF (STRPOS (ref_telescop, "SOHO") NE -1)	THEN	$	; SOHO
   BEGIN
   IF (STRPOS (ref_instrume, "LASCO") NE -1)	THEN	$
      BEGIN
      IF (STRPOS (ref_detector, "C1") NE -1)  THEN tscope = 'c1'
      IF (STRPOS (ref_detector, "C2") NE -1)  THEN tscope = 'c2'
      IF (STRPOS (ref_detector, "C3") NE -1)  THEN tscope = 'c3'
      END
   IF (STRPOS (ref_instrume, "EIT") NE -1)  THEN tscope = 'eit'
   END

;--- File list loop.

CLOSE, 1
OPENR, 1, cme_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, cme_name				; Get file name.
print, 'cme_name: ', cme_name
;   cme_img = rfits (cme_name, header=cme_hdu, /scale)	; Read CME image.
   cme_img = readfits (cme_name, cme_hdu)	; Read CME image.

   ;--- Extract information from header.

   cme_date   = fxpar  (cme_hdu, "DATE-OBS")
   cme_time   = fxpar  (cme_hdu, "TIME-OBS")
   cme_day    = strmid (cme_date, 0, 2)
   cme_month  = strmid (cme_date, 3, 2)
   cme_year   = strmid (cme_date, 6, 2)
   cme_hour   = strmid (cme_time, 0, 2)
   cme_minute = strmid (cme_time, 3, 2)
   dif_name = tscope + '.' + cme_year + cme_month + cme_day + '.' +	$
		       cme_hour + cme_minute + '-' +		$
		       ref_hour + ref_minute + '.fts'

   cme_min = MIN (cme_img, max=cme_max)
   print, FORMAT = '("cme_min/max: ", E10.3, E10.3)', cme_min, cme_max

   ;--- Compute image difference.

   dif_img = cme_img - ref_img				; Compute difference.

   dif_min = min (dif_img, max=dif_max)			; Get image min/max.
   print, FORMAT = '("dif_min/max: ", E10.3, E10.3)', dif_min, dif_max
   IF (dif_min LT 0.0) THEN	$			; Symmetric about zero.
      IF (-dif_min LT dif_max) THEN dif_min = -dif_max  ELSE dif_max = -dif_min
   print, FORMAT = '("dif_min/max: ", E10.3, E10.3)', dif_min, dif_max

   dif_min = -5.00e-8
   dif_max =  5.00e-8
   print, FORMAT = '("dif_min/max: ", E10.3, E10.3)', dif_min, dif_max

   ;--- Convert to byte pixels.
   ;--- Display image.

   byt_img = bytscl (dif_img, min=dif_min, max=dif_max, top=249)
   tv, byt_img						; Display image.

   ;--- Update FITS header.

   dif_hdu = cme_hdu
   fxaddpar, dif_hdu, "DATAMIN", dif_min, "BSUN UNITS"
   fxaddpar, dif_hdu, "DATAMAX", dif_max, "BSUN UNITS"
   fxaddpar, dif_hdu, "DISPMIN",   0.0
   fxaddpar, dif_hdu, "DISPMAX", 249.0
   fxaddpar, dif_hdu, "BSCALE",    1.0
   fxaddpar, dif_hdu, "BZERO",     0.0
   dif_comment = 'IMAGE AT ' + string (cme_hour) + ':' + string (cme_minute) + $
		 'WAS SUBTRACTED FROM EVENT FRAME.'
   fxaddpar, dif_hdu, "HISTORY", dif_comment

   ;--- Write difference image to disk.

   print, 'Write Difference image : ', dif_name
   writefits, dif_name, byt_img, dif_hdu		; Write diff. image.
END	;}

CLOSE, 1
END
