;+
; NAME	picstemp.pro
;
; PURPOSE	Plot air temperature on PICS instrument.
;
; SYNTAX	picstemp, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   2 May 2000
;-

pro picstemp, fits_list

CLOSE, 1
CLOSE, 2
fits_name = ''
data_name = ''
plot_name = ''
xdim_prev = 0
ydim_prev = 0

monthtab = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',	$
	    'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']

;red = bindgen (256)
;green = red
;blue  = red
;lct,'/home/stanger/color/bwy.lut'	; color table.
;tvlct, red, green, blue, /get

picstemp  = fltarr (220)
picshours = fltarr (220)
i     = 0

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
   BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
;   print, 'fits_name: ', fits_name

;   img = readfits (fits_name, hdu)
   hdu = headfits (fits_name, EXTEN=0)
   ext = headfits (fits_name, EXTEN=1)

   ;--- Extract information from primary header.

;   fits_min = MIN (img, max=fits_max)

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   fits_min = fxpar (hdu, 'DISPMIN')
   fits_max = fxpar (hdu, 'DISPMAX')
   date_obs = fxpar (hdu, 'DATE-OBS')
   time_obs = fxpar (hdu, 'TIME-OBS')

   print, 'date_obs: ', date_obs
   print, 'time_obs: ', time_obs

   ;--- Extract day, month & year from date.

   dashpos  = strpos (date_obs, '-', 0)
   slashpos = strpos (date_obs, '/', 0)

   size_date = strlen (date_obs)
   syear = ''

   if (slashpos GT 0) then	$
      BEGIN
      sday    = strmid (date_obs, 0, 2)
      smonth  = strmid (date_obs, slashpos + 1, 2)
      yearpos = strpos (date_obs, '/', slashpos + 1 ) + 1
      size_year = size_date - yearpos
      syear = strmid (date_obs, yearpos, size_year)
      END			$
   else				$
   if (dashpos GT 0) then	$
      BEGIN
      yearpos = 0
      size_year = size_date - 6
      syear  = strmid (date_obs, yearpos, size_year)
      smonth = strmid (date_obs, dashpos + 1, 2)
      daypos = strpos (date_obs, '-', dashpos + 1) + 1
      sday   = strmid (date_obs, daypos, 2)
      END			

   year  = fix (syear)
   month = fix (smonth)
   day   = fix (sday)

;   year   = strmid (date_obs, 0, 2)
;   month  = strmid (date_obs, 3, 2)
;   day    = strmid (date_obs, 6, 2)

   month_name = monthtab [month - 1]
   plot_title = sday + '-' + month_name + '-' + syear +		$
	        ' PICS Temperature vs. Time'
   if (i EQ 0) then						$
      BEGIN
      data_name = sday + '-' + month_name + '-' + syear + '.picstemp.dat'
      plot_name = sday + '-' + month_name + '-' + syear + '.picstemp.ps'
      openw, 2, data_name
      SET_PLOT, 'PS'
      DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = plot_name, $
         XOFFSET = 1.27, XSIZE = 19.05, $        ; 0.5" left   margin, 7.5" wide
         YOFFSET = 5.08, YSIZE = 19.05           ; 2.0" bottom margin, 7.5" high
      END

   hour   = strmid (time_obs, 0, 2)
   minute = strmid (time_obs, 3, 2)
   second = strmid (time_obs, 6, 2)

   picshours [i] = hour + minute / 60.0 + second / 3600.0

   ;--- Extract temperature from extension header.

   airtemp = fxpar (ext, 'ANALOG01')
   picstemp [i] = airtemp

;   fits_min = -1000
;   fits_max = 5000
;   print, FORMAT = '("fits_min/max: ", E10.3, E10.3)', fits_min, fits_max
;   print, 'fits_min/fits_max: ', fits_min, fits_max

   ;--- Convert to byte pixels.
   ;--- Display image.

;   IF (xdim NE xdim_prev OR ydim NE ydim_prev) THEN	$
;      window, xsize=xdim, ysize=ydim
;   byt_img = bytscl (img, min=fits_min, max=fits_max, top=249)
;   tv, byt_img						; Display image.

   print, fits_name, '  hours: ', picshours [i],	$
	  '   temp: ', picstemp [i]
   printf, 2, fits_name, '  hours: ', picshours [i],	$
	      '   temp: ', picstemp [i]

   i = i + 1
   xdim_prev = xdim
   ydim_prev = ydim
   END	;}

CLOSE, 1
CLOSE, 2

numpts = i
hours = fltarr (numpts)
temp  = fltarr (numpts)

hours [*] = picshours [0 : numpts-1]
temp  [*] = picstemp  [0 : numpts-1]

PLOT, hours, temp,					$
      psym=3,					$
      yrange = [16.0, 24.0],				$
      xrange = [17.0, 23.0],				$
      title = plot_title,				$
      xtitle = 'Hours',				$
      ytitle = 'Air Temperature [degrees Celsius]'

DEVICE, /CLOSE
SET_PLOT, 'X'
END
