;+
; NAME	sqrtfts2gif.pro
;
; PURPOSE	Perform square root scaling on FITS images.
;
; SYNTAX	sqrtfts2gif, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   25 January 2000
;-

pro sqrtfts2gif, fits_list

CLOSE, 1
fits_name = ''

red = bindgen (256)
green = red
blue  = red
lct,'/home/stanger/color/bwy.lut'	; color table.
tvlct, red, green, blue, /get

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
print, 'fits_name: ', fits_name
;   READ_GIF, fits_name, fits_img			; Read CME image.

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

;   fits_min = MIN (fits_img, max=fits_max)

   fits_min = fxpar (hdu, 'DISPMIN')
   fits_max = fxpar (hdu, 'DISPMAX')
;   fits_min = -1000
;   fits_max = 5000
;   print, FORMAT = '("fits_min/max: ", E10.3, E10.3)', fits_min, fits_max
   print, 'fits_min/fits_max: ', fits_min, fits_max

   ;--- Compute image difference.

   IF (fits_min LT 0)  THEN	$
   BEGIN
      new_img = fits_img - fits_min
      new_img = new_img > 0
      fits_min = 0
      fits_max = fits_max - fits_min
   END
   new_img = sqrt (fits_img)				; Compute square root.

;   new_min = min (new_img, max=new_max)		; Get image min/max.

   new_min = 0
   new_max = sqrt (fits_max)
   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   IF (new_min LT 0.0) THEN	$			; Symmetric about zero.
;      IF (-new_min LT new_max) THEN new_min = -new_max  ELSE new_max = -new_min
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   new_min = -5.00e-8
;   new_max =  5.00e-8
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

   ;--- Convert to byte pixels.
   ;--- Display image.

   byt_img = bytscl (new_img, min=new_min, max=new_max, top=249)
   tv, byt_img						; Display image.

   ;--- Write newference image to disk.

   sloc = RSTRPOS (fits_name, '.fts')
   new_name = strmid (fits_name, 0, sloc) + '.sqrt.gif'
   print, 'Write sqrt image : ', new_name
   write_gif, new_name, byt_img, red, green, blue	; Write new image.
END	;}

CLOSE, 1
END
