#include <stdio.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>
#include <errno.h>
#include <fcntl.h>

struct opdef {
  int op;
  char *text;
  int count;
} ops[] = {
  MTWEOF, "Write EOF", 1,
  MTFSF, "Forward Skip File", 1,
  MTBSF, "Backward Skip File", 1,
  MTFSR, "Forward Skip Record", 1,
  MTBSR, "Backward Skip Record", 1,
  MTREW, "Rewind", 0,
  MTOFFL, "Offline", 0,
  MTNOP, "No Operation", 0,
  MTRETEN, "Retension", 0,
  MTERASE, "Erase", 0,
  MTEOM, "Position to EOM", 0,
  MTNBSF, "New Backward Skip File", 1,
  -1, "Exit", 1
};

main (argc, argv)

int	argc;
char	*argv[];
{
  int i, j, n, mt;
  char line[120];

  if (argc < 2) {
    fprintf (stderr, "Usage: mttest dev\n");
    exit (0);
  } 
  if (strcmp (argv[1], "-") == 0) {
    mt = dup(1);
  } else {
    mt = open (argv[1], O_RDWR|O_CREAT|O_TRUNC, 0666);
  }
  if (mt < 0) {
    fprintf (stderr, "Open of device %s failed\n", argv[1]);
    exit (-1);
  }
  n = sizeof (ops) / sizeof (struct opdef);

  while (1) {
    fprintf (stderr, "\nPlease type in number of tape operation you would like next\n");
    for (i = 0; i < n; i++) {
      fprintf (stderr, "%5d  %s\n", i, ops[i].text);
    }
    gets (line);
    sscanf (line, "%d", &i);
/*    fprintf (stderr, "You chose i = %d\n", i);  */
    if ((i < 0) || (i >= n)) exit (0);
/*    fprintf (stderr, "That corresponds to operation %d or %s\n",
	    ops[i].op, ops[i].text);  */
    if (ops[i].op < 0) exit (0);
    if ((j = tapeop (mt, i)) != 0) {
      fprintf (stderr, "Tapeop returned %d\n", j);
      if (j < 0) {
	fprintf (stderr, "Tape error was %d ", errno);
	perror (" : ");
      }
    }
    mtiocget(mt);
  }
}

/* * * * * * * *
 * M T I O C G E T
 * * * * * * * *
 */
mtiocget(mt)
int mt;
{
  static struct mtget mt_status;
  int rval;

  rval = ioctl (mt, MTIOCGET, (char *)&mt_status);
  fprintf (stderr, "tape status: \n");
  fprintf (stderr, "   residual= %d   file no= %d   block no= %d\n",
	mt_status.mt_resid,mt_status.mt_fileno,mt_status.mt_blkno);
  return (rval);
}

/* * * * * * * *
 * T A P E O P
 * * * * * * * *
 */
int tapeop(mt, oper)
int mt, oper;
{
  struct mtop tapeoper;
  char line[120];

  tapeoper.mt_op = ops[oper].op;
  tapeoper.mt_count = 1;
  if (ops[oper].count) {
    fprintf (stderr, "Count : ");
    gets (line);
    sscanf (line, "%d", &tapeoper.mt_count);
  }

  fprintf (stderr, "About to issue %s", ops[oper].text);
  if (ops[oper].count) fprintf (stderr, " with count of %d", tapeoper.mt_count);
  fprintf (stderr, "\n");
  return (ioctl (mt, MTIOCTOP, (char *)&tapeoper));
}

