/* abcd_lat.c	spherical trigonometry at SITE_LAT
 *------------------------------------------------------------------------------
 * Implements the fundamental spherical trigonometry equations of
 * Smart: "Text-Book on Spherical Astronomy", eq. A, B, C, and D.
 *
 * These equations can be used to convert between coordinate systems,
 * as follows:	
 *
 * 1) Let  b = hour angle		2) Let  b = azimuth
 *         c = declination		        c = elevation
 *    Then x = azimuth			   Then x = hour angle
 *         y = elevation			y = declination
 *         z = parallatic angle			z = parallatic angle
 *
 * 08 FEB 1985: phw
 *------------------------------------------------------------------------------
 * Author:  David Elmore     HAO/NCAR    3 Aug 1998
 * Updates: Andrew Stanger   HAO/NCAR
 *  3 Dec 2002: Remove VxWorks directive.
 *------------------------------------------------------------------------------
 */

#include "astro.h"

/*
 * void abcd_lat( ANGLE	b, ANGLE c, ANGLE *x, ANGLE *y, ANGLE *z)
 *
 *	Convert from [Ha & Dec] to [Az, El, Beta]
 *
 *	abcd_lat (Ha, Dec, &Az, &El, &Beta);
 *
 *	2 sin, 2 cos, 2 atan2, asin, 9 mul, 3 add or subs,
 */

void
abcd_lat (b, c, x, y, z)

ANGLE  b, c;
ANGLE  *x, *y, *z;
{
    FAST AMP	sina = SIN_LAT, cosa = COS_LAT;
    FAST AMP	sinb, cosb, sinc, cosc;
    FAST AMP	cosz;

    sinb = SIN (b);
    cosb = COS (b);
    sinc = SIN (c);
    cosc = COS (c);

    *x   = ATAN2 (-cosc * sinb, (sinc * cosa - cosc * cosb * sina));
    cosz = sina * cosc - sinc * cosa * cosb;
    *z   = ATAN2 (cosa * sinb,cosz);
    *y   = ASIN (sinc * sina + cosc * cosb * cosa);
}
