/*
 * astro.h - local astro library defines
 */
#include	<math.h>

#ifndef INClclastro
#define INClclastro

#include <math.h>

#define FAST register
#define IMPORT extern
#define LOCAL static

#define AMP	double
#define ANGLE double
#define OK		 0
#define ERROR	-1

#define abs(x)		( (x) < 0 ? -(x) : (x) )
#define min(x,y)	( x < y ? x : y )
#define max(x,y)	( x > y ? x : y )
/*
 * 3600*24 seconds/day = 360 degrees 10*24 or 240 seconds/degree 
 */
#define	S2A(x)		fd2r((AMP)(x)/(AMP)240.)	/* sec of time -> ANGLE */
#define	TWO_PI		(2*PI)
#define	TWO_HUNDRED	(200*PI)

/*
 * the equation of time can be found in the astronomical almanac section B,
 * page 5-7, etc. 
 */

#define TICKS_PER_SEC	1		/* clock rate, user is 1 = 1 sec */
#define	ONE_DAY		(86400*TICKS_PER_SEC)	/* # clock ticks in a day */
#define	TO_SIDEREAL	((ANGLE) (.00273790934*2*PI))

#define PI		(double) 3.14159265358979323846264338327
#define	SMALL		1.E-100		/* small number */

#define	ATAN2(x,y)	atan2(x,y)
#define	ATAN(x)		atan(x)
#define	ASIN(x)		asin(x)
#define	SQRT(x)		sqrt(x)
#define	SIN(x)		sin(x)
#define	COS(x)		cos(x)

#ifndef VxWorks
/*
 * frem(a,b) makes returns a in the range +- b/2
 *
 * fd2r(a) degrees to radians
 *
 * fr2d(a) radians to degrees 
 */
/****#define frem(a,b)		( remainder(a,b) ) ***/
#define frem(a,b)		( fmod(a,b) )
#define fd2r(a)			( (a) * PI / (double) 180. )
#define fr2d(a)			( (a) * (double)180. / PI )
#define hmstosec(h,m,s)		( (s)+60*((m)+60*(h)) )
#endif

/*
 * astro goodies
 */

/* Hawaii MLSO coordinates */

#define SITE_LAT ((ANGLE) (19.538505556*PI/180.)) 	/* 19d 32m 18.62s */
#define SITE_LON ((ANGLE) (155.579294445*PI/180.))	/* 155d 34m 45.46s */
#define SIN_LAT 0.334440285 			 	/* sin(SITE_LAT) */
#define COS_LAT 0.942416944			 	/* cos(SITE_LAT) */

/* Coordinates of Sunspot VTT */

/*#define SITE_LAT ((ANGLE) (32.78722222*PI/180)) */	/* 32d 47m 14s */
/*#define NEW_LAT  ((ANGLE) (32.61222222*PI/180)) */	/* 32d 36m 44s */
/*#define SITE_LON ((ANGLE) (105.8202777778*PI/180)) */	/* 105d 49m 13s */

/*#define SIN_LAT 0.53895103101618891	 */	/* sin(NEW_LAT) */
/*#define COS_LAT 0.84233709770292498	 */	/* cos(NEW_LAT) */

/* Coordinates of Foothills Lab of NCAR */

/*#define SITE_LAT ((ANGLE) (40.07 * PI / 180))	*/	/* 40d 4m 12s */
/*#define SITE_LON ((ANGLE) (105.242777778 * PI / 180)) */ /* 105d 14m 34s */

/*#define	SIN_LAT 0.643723030		*/	/* sin(SITE_LAT) */
/*#define	COS_LAT	0.765258558		*/	/* cos(SITE_LAT) */

#define	EPOCH		2000	/* base year for icc date and time */
#define JD_EPOCH	2451545	/* is 2000 Jan 1.5 */

#define MSD (fd2r (1922.4 / 3600))	/* mean solar diameter, arc seconds */

IMPORT ANGLE last ();

#endif
