/* rs.c
 *
 * Compute Rsun (solar radius).
 *
 */

#include <stdio.h>
#include <math.h>

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern long jday () ;
    extern double jdate () ;
    int  year, month, day ;
    int  hour, minute, second ;

    double degrad  = 0.0 ;		/* degrees / radian. */
    double elong   = 0.0 ;		/* Ecliptic longitude. [radians] */
    double elongdeg= 0.0 ;		/* Ecliptic longitude. [degrees] */
    double eobliq  = 0.0 ;		/* Obliquity of ecliptic. */
    double fracday = 0.0 ;		/* Fraction of a day. */
    double g       = 0.0 ;		/* Mean anomaly. */
    double jd      = 0.0 ;		/* Julian date.  */
    double L       = 0.0 ;		/* Mean solar longitude. */
    double nday    = 0.0 ;		/* # days since EPOCH. */
    double pi      = 3.14159265358979323846264338327 ;
    double rau     = 0.0 ;		/* Earth-Sun distance (AU). */
    double rsun    = 0.0 ;		/* Solar radius. */
    double raddeg  = 0.0 ;		/* radians / degree. */
    double RA      = 0.0 ;		/* Right Ascension. */
    double DEC     = 0.0 ;		/* Declination. */

    sscanf (argv [1], "%d", &year) ;
    sscanf (argv [2], "%d", &month) ;
    sscanf (argv [3], "%d", &day) ;
    sscanf (argv [4], "%d", &hour) ;
    sscanf (argv [5], "%d", &minute) ;
    sscanf (argv [6], "%d", &second) ;

    raddeg = pi / 180.0 ;
    degrad = 180.0 / pi ;

    /*** Julian Day. ***/

    jd = jdate (year, month, day, hour, minute, second) ;

    fracday = hour / 24.0 + minute / 1440.0 + second / 86400.0 ;

    nday = jd - 2451544.5 + fracday ;

    /*** Mean solar Longitude, corrected for abberation. ***/

    L = (280.460 + 0.9856474 * nday) * raddeg ;

    /*** Mean anomaly. ***/

    g = (357.528 + 0.9856003 * nday) * raddeg ;

    /*** Ecliptic Longitude. ***/

    elong = L + (1.915 * sin (g) + 0.02 * sin (2.0 * g)) * raddeg ;
    elongdeg = elong * degrad ;
    while (elongdeg >  360.0)  elongdeg -= 360.0 ;
    while (elongdeg < -360.0)  elongdeg += 360.0 ;

    /*** Obliquity of ecliptic. ***/

    eobliq = (23.439 - 0.0000004 * nday) * raddeg ;

    /*** Right Ascension. ***/

    RA = atan (cos (eobliq) * tan (elong)) ;

    /*** Declination. ***/

    DEC = asin (sin (eobliq) * sin (elong)) ;

    /*** Earth-Sun distance [AU]. ***/

    rau = 1.00014 - 0.01671 * cos (g) - 0.00014 * cos (2.0 * g) ;

    /*** Solar radius. ***/

    rsun = 0.2666 / rau ;		/* Degrees. */

    printf ("year month day: %4d-%02d-%02d\n", year, month, day) ;
    printf ("hour min sec  : %02d:%02d:%02d\n", hour, minute, second) ;
    printf ("jd:     %9.4f\n", jd) ;
    printf ("nday:   %9.4f\n", nday) ;
    printf ("L:      %9.4f radians\n", L) ;
    printf ("g:      %9.4f radians\n", g) ;
    printf ("elong:  %9.4f radians   elong: %9.4f degrees\n", elong, elongdeg) ;
    printf ("eobliq: %9.4f radians\n", eobliq) ;
    printf ("RA:     %9.4f radians   RA:  %9.4f degrees\n", RA,  RA  * degrad) ;
    printf ("DEC:    %9.4f radians   DEC: %9.4f degrees\n", DEC, DEC * degrad) ;
    printf ("rau:    %9.4f AU\n", rau) ;
    printf ("rsun:   %9.4f degrees   rsun: %9.4f arcsec\n", rsun, rsun*3600.0) ;

    exit (0) ;
}

/* -------------------------------------------------------------------------- */

double
jdate (year, month, day, hour, minute, second)

int year ;
int month ;
int day ;
int hour ;
int minute ;
int second ;
{
    extern int day_of_year () ;
    int doy       = 0 ;
    int  i        = 0 ;
    long leap     = 0 ;
    long EPOCH    = 2000 ;
    double JD_EPOCH = 2451544.5 ;		/* 2000 Jan 1 00:00:00 */
    double jd ;

    if (year > EPOCH)
        {
        for (i = EPOCH; i <= year; i++)
	   if (isleap (i)) leap++ ;
        }
    else
	{
	for (i = year; i < EPOCH; i++)
	    if (isleap (i)) leap-- ;
        }

    doy = day_of_year (year, month, day) ;
    printf ("doy: %d\n", doy) ;
    jd = JD_EPOCH + (year - EPOCH) * 365.0 + leap + doy - 1.0 ;
    jd += (double) hour / 24.0 + (double) minute / 14400.0
	+ (double) second / 86400.0 ;

    return (jd) ;
}

/* -------------------------------------------------------------------------- */

int
day_of_year (year, month, day)

int year ;
int month ;
int day ;
{
    extern int isleap () ;
    int doy ;
    int leap ;
    static int mday [2][13] =
                   {
		   0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365,
		   0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
		   };

    if (isleap (year))
	leap = 1 ;
    else
	leap = 0 ;

    doy = day ;
    doy += mday [leap][month - 1] ;

    return (doy) ;
}
/* -------------------------------------------------------------------------- */

int
isleap (year)                     /* Gregorian leap year calculation */

int year;
{
    if ((year & 3) != 0)
	return (0);             /* not divisible by 4, not leap */
    if ((year % 100) != 0)
	return (1);             /* not divisible by 100 but by 4, a leap */
    if ((year % 400) != 0)
	return (0);             /* not divisible by 400 but by 100, not leap */
    if ((year % 4000) != 0)
	return (1);             /* not divisible by 4000, a leap */

    return (0);                 /* divisible by 4000, not a leap */
}

/* -------------------------------------------------------------------------- */
