#include "astro.h"
/*
 * astrotime.c - astronomical time calculations in terms
 * of julian day and gmt.
 *
 * jday(yy,mm,dy) - return julian day for integer year, mont day.
 *
 * doy(yy, mm, dy) - return day of year for ...
 *
 * ANGLE last(jday, gmt) - return radians for local siderial time.
 *							24 hours = 2 * PI
 */

static short    mtab[] = {
		0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365,
		0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366};

/*
 * jday() 
 *
 * RETURNS: julian day for this yymmdd.
 */
jday(yy,mm,dy)
int yy, mm, dy;
{			
	int             leap = -1, j;
	int	ddd, Day;

	if (yy == mm && yy == dy && yy == 0)
	{
		printf("jday yy, mm, dy with respect to jd EPOCH %d, year %d\n",
				JD_EPOCH, EPOCH);
		return ERROR;
	}
	if (yy < 100)
		yy += 1900;	/* make year like 1985 */
	if (yy > EPOCH) {
		for (j = EPOCH; j <= yy; j++)
			if (!(j % 4))
				leap++;
	} else {
		for (j = yy; j < EPOCH; j++)
			if (!(j % 4))
				leap--;
	}
	ddd = doy(yy,mm,dy);
	Day = (yy - EPOCH) * 365 + leap + ddd + JD_EPOCH;/* # days since epoch */
	return Day;
}

/*
 * for this date, return day of the year
 */
doy(yy, mm, dy)
int yy, mm, dy;
{
int ddd = dy, i = 0;
int	j;
	if(yy < 100) yy += 1900;
	if(!(yy % 4)) i = 13;	/* skip to leap months */
	ddd += mtab[mm+i-1];
	return ddd;
}

/*
 * the equation of time can be found in the astronomical almanac section B,
 * page 5-7, etc. 
 */

/*
 * for this julian day and gmt in  seconds, return
 * local siderial time in radians, 2Pi being 24 hours
 */
ANGLE
last(jday, gmt)
int jday, gmt;
{
FAST ANGLE Tu;
FAST ANGLE dt;

	if(abs(jday) > 10000)
		jday = jday - JD_EPOCH;

#ifdef DEBUG
	printf("jday is %d\n",jday);
#endif

	Tu = ((AMP) jday * 2. - 1.)/(36525. * 2);	/* Last on reference
								 * date, 0h UT */
	Tu = Tu * (Tu * (-Tu * S2A(6.2E-06) + S2A(.093104)) + S2A(184.812866))
		+ 200. * PI * Tu
		+ S2A(24110.54841)	/* Greenwich (mean!) sidereal time */
		-SITE_LON /* Local (mean!) sidereal time */ ;

	dt = (ANGLE) (gmt % ONE_DAY) / ((ANGLE) ONE_DAY);
	Tu = Tu /* Last at ref. date/time set in intcoord */
		+ (gmt / ONE_DAY) * TO_SIDEREAL	/* integral days since ref.
						 * time */
		+ dt * 2. * PI + dt * TO_SIDEREAL;	/* fractional day */

	return Tu;
}
