/* bprms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bright point removal.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 7 September 1985
 * 26 December 1985: Search for brightest point within a box centered around
 *                   approximate location of bright point.
 * 30 July 1986
 * 7 March 1988: convert to function instead of main program.
 * 17 July 1992: SGI GL version.
 *  7 June 1993: UNIX version; no screen display.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define NBOX 10 
#define NSEARCH 20 
#define NINT 12

bprms (imgbuf, nbypx, xdim, ydim)

unsigned char *imgbuf;		/* Image buffer. */
int nbypx;			/* # bytes per pixel. */
int xdim;			/* X-dimension. */
int ydim;			/* Y-dimension. */
{
   static unsigned char pixval [NSEARCH][NSEARCH];

   static char dir2 [] = "ru";

   int bpx, bpy;
   int dxx = 20;
   int dyy = 16;
   int i;
   int ibox;
   int ierr;
   int level = -1;
   int pixmax;
   int pval;
   int x, y;
   int xmin = 0;
   int ymin = 0;
   int xp, yp;
   int xul = 91;
   int yul = 358;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;
   static int xx [NBOX] = {208, 226, 215, 253, 250, 371, 494, 497, 325, 414};
   static int yy [NBOX] = {504, 504, 493, 494, 418, 340, 303, 228, 159, 130};
   static int xb [NBOX];
   static int yb [NBOX];
   static int xbrt [NBOX];
   static int ybrt [NBOX];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   ierr = 0;
   dxs = NSEARCH;
   dys = NSEARCH;
   for (i = 0; i < NBOX; i++)
      {
      xb [i] = xx [i] -  63;
      yb [i] = 510 - yy [i];
      if (xb [i] < 0)  xb [i] = 0;
      if (yb [i] < 0)  yb [i] = 0;
      }

   /*** Bright Point Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {

      /*** Read box centered about estimated location of bright point. ***/

      xmin = xb [ibox] - dxs/2;
      ymin = yb [ibox] - dys/2;
      if (xmin < 0)  xmin = 0;
      if (ymin < 0)  ymin = 0;
      ierr = mr_box (imgbuf, nbypx, xdim, ydim, pixval,
		     xmin, ymin, dxs, dys, dir2);
      if (ierr < 0)
	 {
	 fprintf (stderr, "bprms> mr_box error: %d  x:%3d  y:%3d\n",
		  ierr, xmin, ymin);
         return (ierr);
	 }

      /*** Initial estimate of bright point location and intensity. ***/

      bpx = xb [ibox];
      bpy = yb [ibox];
      pixmax = pixval [yb [ibox]] [xb [ibox]];

      /*** Search each box for brightest pixel. ***/

      for (y = 0; y < dys; y++)
         {
         yp = ymin + y;
         if (yp >= 0 && yp < ydim)

            for (x = 0; x < dxs; x++)
               {
               xp = xmin + x;
               if ((xp >= 0 && xp < xdim) && (pixval [y][x] > pixmax))
                  {
		  pixmax = pixval [y][x];
		  bpx = xp;
		  bpy = yp;
		  }
               }
         }

   /* Redefine lower left corner of box so that it is centered about the
    * brightest pixel found.
    */

      xbrt [ibox] = bpx - dx/2;
      ybrt [ibox] = bpy - dy/2;
      if (xbrt [i] < 0)  xbrt [i] = 0;
      if (ybrt [i] < 0)  ybrt [i] = 0;

/*
 *      fprintf (stdout, "box: %2d   pixmax: %3d   bpx: %3d   bpy:%3d   ",
 *              ibox,pixmax,bpx,bpy);
 *      fprintf (stdout, "xbrt: %3d   ybrt: %3d\n", xbrt [ibox], ybrt [ibox]);
 */
      }

   /*** Perform horizontal interpolation for first four bright points. ***/

   ierr = hint (imgbuf, nbypx, xdim, ydim, level,
		     2, xbrt, ybrt, dx, dy, dir2);
   if (ierr < 0)
      {
      fprintf (stderr, "bprms> hint  error: %d\n", ierr);
      return (ierr);
      }

   /*** Perform vertical interpolation for remaining bright points. ***/

   ierr = vint (imgbuf, nbypx, xdim, ydim, level,
		nbox-4, &xbrt [4], &ybrt [4], dx, dy, dir2);
   if (ierr < 0)
      {
      fprintf (stderr, "bprms> vint  error: %d\n", ierr);
      return (ierr);
      }

   /*** Vertical interpolation for upper left black/white spot. ***/

   xul =  38;
   yul = 154;
   dxx = 24;
   dyy = 24;

   xmin = xul - 15;
   ymin = yul - 15;
   dxs = 20;
   dys = 20;
   pixmax = -1;
  ierr = mr_box (imgbuf, nbypx, xdim, ydim, pixval,
		 xmin, ymin, dxs, dys, dir2);

      for (y = 0; y < dys; y++)
         {
         yp = ymin + y;
         if (yp >= 0 && yp < ydim)

            for (x = 0; x < dxs; x++)
               {
               xp = xmin + x;
               if ((xp >= 0 && xp < xdim) && (pixval [y][x] > pixmax))
                  {
		  pixmax = pixval [y][x];
		  bpx = xp;
		  bpy = yp;
		  }
               }
         }

   xul = bpx - dxx / 2;
   yul = bpy - dyy / 2;
   ierr = vints (imgbuf, nbypx, xdim, ydim, level,
		      1, &xul, &yul, &dxx, &dyy, dir2);
   if (ierr < 0)
      {
      fprintf (stderr, "bprms> vints error: %d\n", ierr);
      return (ierr);
      }

   return (0);
}
