/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * bytsrf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert byte image to a Sun Rasterfile format file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "rasterfile.h"

bytsrf (namsrf, imgbuf, dx, dy, r, g, b)

char *namsrf;
unsigned char *imgbuf;
int dx, dy;
unsigned char *r, *g, *b;
{
   FILE *fp;
   int i;
   char *tmp, *end;
   unsigned char defrgb [256];
   struct rasterfile ras;

   /*** Default RGB table: linear greyscale. ***/

   for (i = 0; i < 256; i++)  defrgb [i] = i;

   /*** Open new file on disk. ***/
 
   if ( (fp = fopen (namsrf, "w")) == NULL )
     {
      perror ("bytsrf> fopen error:");
      return (-1);
      }

   /*** Fill Sun rasterfile structure elements. ***/

   ras.ras_magic = RAS_MAGIC;
   ras.ras_width = ( dx % 2 == 0 ) ? dx : dx + 1;
   ras.ras_height = dy;
   ras.ras_depth = 8;
   ras.ras_length = ras.ras_width * ras.ras_height;
   ras.ras_type = RT_STANDARD;
   ras.ras_maptype = RMT_EQUAL_RGB;
   ras.ras_maplength = 3*256;

   /*** Write rasterfile header. ***/

   if ( fwrite ((char *) (&ras), sizeof (ras), 1, fp) != 1 )
      {
      perror("bytsrf> header fwrite error:");
      return(-1);
      }

   /*** Write color table values. ***/

   if ( fwrite (((r == NULL)?defrgb:r), 256, 1, fp) != 1 )
      {
      perror("bytsrf> colormap fwrite error:");
      return (-1);
      }

   if ( fwrite (((g == NULL)?defrgb:g), 256, 1, fp) != 1 )
      {
      perror ("bytsrf> colormap fwrite error:");
      return (-1);
      }

   if ( fwrite (((b == NULL)?defrgb:b), 256, 1, fp) != 1 )
      {
      perror ("bytsrf> colormap fwrite error:");
      return (-1);
      }

   /*** Write image pixel data. ***/

   if ( ras.ras_width == dx )
      {
      if ( fwrite ((char *) imgbuf, dx*dy, 1, fp) != 1 )
	 {
	 perror ("bytsrf> image fwrite error:");
	 return (-1);
         }
      }
   else 
      {
      end = (char *) imgbuf + dx*dy;
      for (tmp = (char *) imgbuf; tmp < end; tmp += dx)
         {
         if ( fwrite (tmp, ras.ras_width, 1, fp) != 1 )
	    {
            perror ("bytsrf> image fwrite error:");
            return (-1);
            }
         }
    }

 fclose (fp);
 return (0);
}
