c#######################################################################
      function spb (rho, eta, tmin, tmax, nt)
c
c-----------------------------------------------------------------------
c
c ****** Calculate polarization brightness [adapted from F. Bagenal].
c
c-----------------------------------------------------------------------
c
c ****** RHO is the plane-of-sky radial distance
c ****** in units of R_sun.
c
c ****** ETA is the plane-of-sky polar angle in radians
c ****** (solar North pole ==> ETA=0 and ETA=2*pi).
c ****** Angles with ETA between 0 and pi specify the
c ****** right limb,  and angles with ETA between pi and 2*pi
c ****** specify the left limb.
c
c ****** EMU is the central meridian longitude (Carrington
c ****** rotation longitude) in radians.
c
c ****** TMIN and TMAX are the limits in the angle tau for which
c ****** to integrate along the line of sight.
c ****** They should not specify an interval outside the
c ****** interval (-pi/2 to pi/2).
c
c ****** NT is the number of points to use in this
c ****** integration (with uniform tau interval).
c
c-----------------------------------------------------------------------
c
      parameter (pi=3.14159265358979)
c
c-----------------------------------------------------------------------
c
      dtau = (tmax - tmin) / nt
c
      tau = tmin - 0.5 * dtau
      pb = 0.0
      do i = 1, nt
        tau = tau + dtau
        ct = cos (tau)
        r = rho / ct
        theta = acos (cos (eta) * ct)
        se = sin (eta)
        if (se .eq. 0.0) then
          if (tau .ge. 0.0) then
            zeta = 0.5 * pi
          else
            zeta = -0.5 * pi
          end if
        else
          tt = tan (tau)
          zeta = atan2 (tt, se)
        end if
        phi = 0.5 * pi + zeta
        phi = amod (phi, 2.0 * pi)
        if (phi .lt. 0.0) phi = phi + 2.0 * pi
        rhov = rhoval (r, theta, phi)
        pb = pb + rho * rhov * cfun (r) * dtau
      enddo
c
      spb = pb
c
      return
      end
c#######################################################################
