/* den.c
 * *****************************************************************************
 * Acquired from Sarah Gibson   [pBB3d.pro]
 * Conversion to C-language: Andrew L. Stanger  18 Aug 2004
 * *****************************************************************************
 */

float *
dens1 (float r_pb, float theta_pb, float phi_pb)

{
  float cpar = 1.0e+8;		/* amplitude */
  float dpar = 2.0;		/* slope     */
  float apar = 5.0e+8;
  float bpar = 2.0;

  density = cpar * pow (r_pb, (-1 * dpar) );

  theta_pb = theta_pb + 30;

  /*
   * test = where (theta_pb gt 80 and thetapb lt 110 and phipb gt 80
   *               and phipb lt 120)
   * if (min (test) ne -1) then density [test] = apar*(r_pb [test]^(-1 * bpar))
   */

  /*
   * test = where (theta_pb gt 130 and thetapb lt 150 and
   *               phipb gt 10 and phipb lt 30)
   * if (min (test) ne -1) then density [test] = apar*(r_pb [test]^(-1 * bpar))
   */

return (density):
}

;************************************************************************

float *
dens2 (float r_pb, float theta_pb, float phi_pb)

{
  float cpar = 1.0e+8;		/* amplitude */
  float dpar = 2.0;		/* slope     */
  float apar = 5.0e+8;
  float bpar = 2;

  density = cpar * (r_pb^(-1 * dpar))

 /*
  * test = where (r_pb * abs ((cos (theta_pb * !pi / 180.0)))^0.5 lt 1.0)
  * if (min (test) ne -1) then density [test] = apar*(r_pb [test]^(-1 * bpar))
  */

  return (density);
}

;************************************************************************

float *
dens3 (float r_pb, float theta_pb, float phi_pb)

{
  float cpar = 1e8;		/* amplitude */
  float dpar = 2.0;		/* slope     */

  density = cpar * (r_pb^(-1 * dpar));

  /*
   * test = where (r_pb * abs ((cos (theta_pb * !pi / 180.)))^.5 lt 1.
   *               and phipb gt 180.)
   * if (min (test) ne -1) then density [test] =
   *     cpar * (r_pb [test]^(-1 * dpar)) * ((cos (phi_pb [test]
   *     * !pi / 180.0))^2)^(1.0 / 3.0)
   */

  return (density);
}

;************************************************************************
float *
dens4 (float r_pb, float theta_pb, float phi_pb)

{
/* common edge,       coneedge, onedconeedge */

  float d2r= 1.745329e-2;

  /*** BACKGROUND DENSITY. ***/
  /* ;PARAMETERS amplitude (cpar) and slope (dpar) */

  float cpar = 1e8;
  float dpar = 3.0;

  dens_back = cpar * pow (r_pb, (-1 * dpar));

  /*** ENHANCED MODEL DENSITY. ***/
  /* ;   PARAMETERS amplitude (cmod) and slope (dmod) */

  /*** CASE 1 ***/
  cmod = 1e09;

  /*** CASE 2 ***/
  /* ; cmod = 2e09 */

  dmod = 2.0;

  dens_mod = cmod * pow (r_pb, (-1 * dmod));

  /*** PUT in the background density. ***/

  density = dens_back;

  /*** PUT IN ICE CREAM CONE. ***/

  /* PARAMETERS angles. */

  /*** CASE 1 ***/
  alpha_o = 35.0;
  alpha_i = 30.0;

  /*** CASE 2 ***/
  /* ; alpha_o = 55.0 */
  /* ; alpha_i = 50.0 */

  /*Parameter R_c  --> distance from origin to center of the ice cream scoop. */

  /*** CASE 1 ***/
  R_c =  4.0;

  /*** CASE 2 ***/
  /* ; R_c =  2.7 */

  /* ;-- PARAMETERS rho_i and rho_o inner and outer radius of the scoop. */

  rho_i = R_c * sin (alpha_i*d2r);
  rho_o = R_c * sin (alpha_o*d2r);

  /*** PARAMETERS sectionheights - s1_ht, s2_ht, s3_ht, s4_ht, s5_ht. ***/

  s1_ht = 1.0;
  s2_ht = R_c-rho_o * sin (alpha_o * d2r);
  s3_ht = R_c-rho_i * sin (alpha_i * d2r);
  s4_ht = R_c + rho_i;
  s5_ht = R_c + rho_o;

  /* Define relationship between cone angle and spherical coordinate angles. */

  sinalpha =  sqrt (((sin (theta_pb * d2r))^2) * ((cos (phi_pb * d2r))^2) + (cos (theta_pb * d2r)^2));

  /* Define variation of inner and outer curves in the scoop. */

  alpha_o_scoop = (atan (sqrt ((sin (alpha_o * d2r)^2) - (r_pb / R_c - 1.0)^2.0) / (r_pb / R_c))) / d2r;
  alpha_i_scoop = (atan (sqrt ((sin (alpha_i * d2r)^2) - (r_pb / R_c - 1.0)^2.0) / (r_pb / R_c))) / d2r;

  /*
  test = where (alpha_o_scoop le 0.0 or alpha_o_scoop ge 90.0);
  if (min (test) ne -1) then begin
    print, alpha_o_scoop, 'outer: out of first quadrant'
    stop
  endif
   */

  /*
  test = where (alpha_i_scoop le 0.0 or alpha_i_scoop ge 90.0);
  if (min (test) ne -1) then begin
    print, alpha_i_scoop, 'inner: out of first quadrant'
    stop
  endif
   */

  /*** Section 1 ***/

  /* Legs of cone, cone angles constant. */

  section1 = where (r_pb gt s1_ht and r_pb le s2_ht);

  if (min (section1) ne -1)
    {
    shell_1 = where (sinalpha lt sin (alpha_o*d2r) and sinalpha ge sin (alpha_i * d2r) and r_pb gt s1_ht and r_pb le s2_ht and sin (phipb * d2r) ge 0.0);
    insidecone_1 = where (sinalpha lt sin (alpha_i*d2r) and r_pb gt s1_ht and r_pb le s2_ht and sin (phipb * d2r) ge 0.0);

    if (min (shell_1) ne -1) then density [shell_1] = dens_mod [shell_1];
    if (min (insidecone_1) ne -1)
        density (insidecone_1) = dens_back (insidecone_1);

    /*;  if (min (shell_1 ne -1)) then stop */
    }

  /*** Section 2 ***/

  section2 = where (r_pb gt s2_ht and r_pb le s3_ht);

  if (min (section2) ne -1)
    {
    /*** Above the legs - outer cone angle varies with height. ***/

    shell_2 = where (sinalpha lt sin (alpha_o_scoop * d2r) and sinalpha ge sin (alpha_i * d2r) and r_pb gt s2_ht and r_pb le s3_ht and sin (phipb * d2r) ge 0.0);
    insidecone_2 = where (sinalpha lt sin (alpha_i * d2r) and r_pb gt s2_ht and r_pb le s3_ht and sin (phipb * d2r) ge 0.0);

    if (min (shell_2) ne -1)  density [shell_2] = dens_mod [shell_2];
    if (min (insidecone_2) ne -1)
        density [insidecone_2] = dens_back [insidecone_2];
    }

  /*** Section 3 ***/

  section3 = where (r_pb gt s3_ht and r_pb le s4_ht);

  if (min (section3) ne -1)
    {

    /*** In the middle of the scoop. ***/

    shell_3 = where (sinalpha lt sin (alpha_o_scoop * d2r) and sinalpha ge sin (alpha_i_scoop*d2r) and r_pb gt s3_ht and r_pb le s4_ht and sin (phipb * d2r) ge 0.0);
    insidecone_3 = where (sinalpha lt sin (alpha_i_scoop * d2r) and r_pb gt s3_ht and r_pb le s4_ht and sin (phipb * d2r) ge 0.0);

    if (min (shell_3) ne -1) then density [shell_3] = dens_mod [shell_3];
    if (min (insidecone_3) ne -1) then density [insidecone_3] = dens_back [insidecone_3];
    }

  /*** Section 4 ***/

  section4 = where (r_pb gt s4_ht and r_pb le s5_ht);

  if (min (section4) ne -1)
    {

    /*** At the of the scoop. ***/
    /*** never inside cone ***/

    shell_4 = where (sinalpha lt sin (alpha_o_scoop * d2r) and r_pb gt s4_ht and r_pb le s5_ht and sin (phipb * d2r) ge 0.0);

    if (min (shell_4) ne -1) then density [shell_4] = dens_mod [shell_4];
    }

  coneedge = where (density ne dens_back);

}

;************************************************************************
float *
dens5 (float r_pb, float theta_pb, float phi_pb)

{
/*
common newresults, density
common results,    Dens, Pres, bstrengthphys, brphys, bthphys, bphphys
common prams,      apar, xo, thetao, phio, rbub, ao, Pio, alpha, eta, $
                   velmult, cpar, dpar, alnot, sig1, out
common shiftcoord, Rcap, Thcap, Phcap
 */

  /*** First set up shell. ***/
  /*** PARAMETER out will determine how thick the shell is. ***/


  out = 22. ;

  denscalc, r_pb, theta_pb, phi_pb;
  
/* ; density = dens */
/* ; goto, label1 */

  /*** PUT in the shell density. ***/
  /*** (everything but the background, cavity, and core) ***/
  /*** PARAMETERS amplitude (cshell) and slope (dshell) ***/

  cshell = 4.0e+8;
  dshell = 3.0;

  dens_shell = cshell * pow (r_pb, (-1 * dshell));

  density = dens_shell;

  /*** PUT in the cavity density. ***/
  /*** PARAMETERS amplitude (ccav) and slope (dcav) ***/

  ccav = 0.0;
  dcav = 3.0;

  dens_cav = ccav * pow (r_pb, (-1 * dcav));

  cavity = where (dens lt 0.0);
  if (min (cavity) ne -1) then density [cavity] = dens_cav [cavity];

  /*** PUT in the background density. ***/
  /*** PARAMETERS amplitude (cback) and slope (dback). ***/

  cback = 5.0e+7;
  dback = 4.0;

  dens_back = cback * (r_pb^(-1 * dback));

  background = where (Rcap ge rbub);
  if (min (background) ne -1) then	$
    density [background] = dens_back [background];

  /* PUT in the MODEL CORE DENSITY.
   * PARAMETER amplitude coremult
   * (out=0 means no front so pos. stuff in bubble core only)
   */

  out = 0.0;

  denscalc, r_pb, theta_pb, phi_pb;

  coremult = 1.0;

  core = where (Rcap lt rbub and dens ge 0.0);
  if (min (core) ne -1) then density [core] = coremult * dens [core];

  /* NOTE this could be changed, e.g. replaced
   * by a power law or some other function.
   */

label1:

}

;************************************************************************

float *
dens6 (float r_pb, float theta_pb, float phi_pb)

{
/*
common params1,    Aprofhole, AprofNE, awidth, bwidth, thcs, phcs, alpha, m, w
common gridpiv3d,  rpiv, thpiv, phpiv
common newresults, density
common Bangle,     tilt
 */

  th = theta_pb * !dtor;
  ph = phi_pb   * !dtor;

  tilt = 1;

  getgauss, density, r_pb, th, ph;

}

/* ************************************************************************** */
