/* pbb3d.v2.c		[converted to C from IDL procedure: pbb3d.pro]
 * *****************************************************************************
 * Input: 3D electron number density array.
 * Output: pB, B.
 * *****************************************************************************
 * Acquired from Sarah Gibson
 * C conversion: Andrew L. Stanger   completed 17 Aug 2004
 * *****************************************************************************
 */

/*** Global variables. ***/

int full     = 0;
int idens    = 1;
int limbread = 0;
int nx       = 256;
int ny       = 256;
int ntau     = 0;;
int onedconeedge = 0;
int test  = 0;
int test2 = 0;
int tlen  = 0;

float *pbimg;
float *brightimg;
float *thsave;
float x;
float y;
float *bright2d;
float *pb2d;

float bang      =  0.0;
float cmer      =  0.0;
float coneedge  =  0.0;
float density   =  0.0;
float lat       =  0.0;
float maxheight =  1.5;			/* Rsun */
float pb        =  0.0;			/* pB */
float bright    =  0.0;			/* brightness */
float mult      =  1.0;
float tauint    =  0.1;
float taumin    = -1.57;
float taumax    =  1.57;

/* -------------------------------------------------------------------------- */

main (int argc, char *argv [])

{
  void inputs ();
  int pbint (float r, float theta, float phi);

  int count;
  int endrot = 1;
  int ix;
  int iy;
  int ntau;

  float r;
  float phi;
  float theta;

  float dx        =  0.0;
  float maxpb     =  0.0;
  float minpb     =  0.0;
  float pi        =  3.14159265;
  float rwid      =  0.0;
  float thetwid   =  0.0;

  ntau = (int) ((taumax - taumin) / tauint + 1.0);

  /*** Get inputs. ***/

  /* inputs (); */

  /* The plane of sky coordinates are calculated (r, theta, phi).
   *
   * I decided to do these evenly spaced in x-y so that we could
   * then display a 2D image (I knew there was a reason for an x-y grid!).
   */

  pbimg        = (float *) malloc (nx * ny * sizeof (float));
  brightimg    = (float *) malloc (nx * ny * sizeof (float));
  thsave       = (float *) malloc (nx * ny * sizeof (float));

  bright2d  = (float *) malloc (512 * 512        * sizeof (float));
  pb2d      = (float *) malloc (512 * 512        * sizeof (float));

  dx     = 2.0 * maxheight / (nx - 1.0);
  count  = 0;

  for (iy = 0; iy < ny; iy++)
    {
    for (ix = 0; ix < nx; ix++)
      {
      x = -maxheight + ix * (2 * maxheight / (nx - 1.0));
      y = -maxheight + iy * (2 * maxheight / (ny - 1.0));

      /*** Now convert to r, theta (which is what the pb integral wants). ***/

      r      = sqrt (x * x  + y * y );
      theta  = acos (y  / r);
  /*
   *      neg   = where (x lt 0.0);
   *      if (min (neg) != -1)  theta [neg] = 2.0 * pi - theta [neg];
   */
      if (x != -1) theta = 2.0 * pi - theta;

      /*** And your phi is the central meridian of the "day" you want. ***/

      phi = cmer + 10.0;

      phi = phi * pi / 180.0;        /* Convert to radians. */

      /*** Now calculate pb and B!. ***/

      limbread = 1;
      pbint (r, theta, phi);

      *(pbimg     + ix + iy*nx) = alog10 (pb     );
      *(brightimg + ix + iy*nx) = alog10 (bright );
      }

    /*** Check the apparent width. ***/

    if (idens == 4)
      {
/*
      if (min (onedconeedge) != -1)
        {
        thetwid = theta [onedconeedge];
        rwid    = r     [onedconeedge];
        print, rwid
        test = where (abs (rwid - 1.0) lt sqrt (2.1) * dx);
        stop

        if (min (test) != -1)
          {
          tlen = size (test);
          tlen = tlen (1);
          thsave [count:count + tlen - 1] = thetwid (test);
          count = count + tlen;
          }
        }
 */
      }

    /*** Block out the solar disks. ***/

/*    test = where (r lt 1.0); */
    if (min (test) != -1)  pbimg     [test, iy] = -100.0;
    if (min (test) != -1)  brightimg [test, iy] = -100.0;
    }

  /*** Width. ***/

  if (idens == 4)
    {
    if (min (onedconeedge) != -1)
      {
      /* thsavefin = where (thsave != 0.0); */
      /*
       * print, 'thetas are', thsave [thsavefin] / !dtor;
       * print, 'width is',
       *    max (thsave [thsavefin] / !dtor) - min (thsave [thsavefin] / !dtor);
       */
      }
    }

  /* test2 = where (pbimg != -100); */
/*
 * minpb = min (pbimg [test2]);
 * maxpb = max (pbimg [test2]);
 * test2 = 0;
 */

  /*** Normalization for CASE 1 vs. CASE 2. ***/

  minpb = -2.5;
  maxpb =  2.5;

  /*** rebin. ***/

  /*
   * pbimg     = rebin (pbimg,     512, 512)
   * brightimg = rebin (brightimg, 512, 512)
   */

  /*** Display the pb image. ***/

  /*
   * window, 2,    $
   * xs=512,  ys=512,  retain=3, title='pb model'
   */

  for (iy = 0; iy < ny; iy++)
    for (ix = 0; ix < nx; ix++)
      {
      *(pb2d + ix + iy*nx) = *(pbimg + ix + iy*nx);
      }

  /*** And plot. ***/

    /* tv, bytscl (pb2d, min=minpb, max=maxpb) */

  /*** And plot. ***/

  /*
   * ;    print, 'type .c to continue to modeled brightness image'
   * ;    stop
   */

  /*** Show the brightness movie. ***/

  /* window, 3, xs=512, ys=512, retain=3, title='brightness model' */

  for (iy = 0; iy < ny; iy++)
    for (ix = 0; ix < nx; ix++)
      *(bright2d + ix + iy*nx) = *(brightimg + ix + iy*nx);

  /* tv, bytscl (bright2d, min=minpb, max=maxpb) */

  exit (0);
}

/* -------------------------------------------------------------------------- */

/* pro inputs */
    
void
inputs ()

{
  int nsymm = 0;
  int ntau  = 0;

  float time      = 0.0;
  float accel     = 0.0;
  float maxheight = 0.0;
  float tauint    = 0.0;
  float taumax    = 0.0;
  float taumin    = 0.0;
  float mult      = 1.0;

  /*
   * read, 'What density routine? dens1 - 1, dens2 - 2, dens3 - 3, dens4 (ice
   * cream cone) - 4, G&L model dens - 5 , or Spartan 3d  -- 6 ', idens
   */

  /*** Read in prams for G&L model. ***/

  if (idens == 5)
    {
    /* read, 'What time step? in hours (0. default) ', time */
    time = time * 3600.0;
    accel = 0;

  /*
   * openr, 5, 'pramsset.d'
   * readprams
   * close, 5
   */

  /* read, 'What orientation? along l.o.s. (90.), perp to l.o.s. (0.) ', sig1 */
    }

  /*** Set up for Spartan. ***/

  if (idens == 6) params ();

  /*** How far out do you extend? ***/

  /* read, 'To what height do you wish the image to go? (Rsun) ', maxheight */

  /*** What grid spacing do you want? ***/
       
  /* read, 'Input grid dimensions (e.g. 256X256, type 256) ', nx */
  ny =  nx;

  /* The l.o.s. integrating angle tau goes basically from -pi/2 to pi/2.
   * tauint = .1 is the integration step I've been using, but you could
   * them finer or coarser if you wish.
   */
  /*read, 'Maximum l.o.s integrating angle? (type 1.57 for default) ', taumax */
  /*read, 'l.o.s integrating angle step-size? (type .1 for default) ', tauint */

  /* IF the model you are using is azimuthally symmetric, then may as
   * well only integrate from 0 to pi/2.  Here's a check for that:
   */

  /* read, 'Azimuthally symmetric model? (yes = 1, no = 0) ', nsymm */

  if (nsymm == 0)
     {
     taumin = -1 * taumax;
     mult   = 1.0;
     }

  if (nsymm == 1)
     {
     taumin = 0.0;
     mult   = 2.0;
     }

  ntau = fix ((taumax - taumin) / tauint + 1.0);

  /* What central meridian phi do you want?   (radians)
   * (You may want to step through these do see your structure
   * rotate round if it's not azimuthally symmetric).
   */

  cmer = 0.0;
  /* if (nsymm eq 0)  read, 'What central meridian longitude (degrees)? (0
puts structures at west limb for CME models) ', cmer */

  /*** Also position in latitude. ***/

  lat = 0.0;
  /* read, 'What change in latitude (degrees)? (0 returns structure centered
at equator, +/-  is below/above ', lat */

  bang = 0.0;
  /* read, 'What change in viewing angle (i.e. B angle tilt) (degrees)? (0
is 0 B angle, +/-  is below/above ', bang */

  full = 0;
  /* if (nsymm eq 0)  read, 'Do you want a single image (0), a full rotation
(1), or three STEREO images separated by 50 degrees (2) ', full */
}

/* -------------------------------------------------------------------------- */

/* Calculates integrated line-of-sight polarized brightness pb (r, theta, phi)
 * and brightness bright (r, theta, phi) from density produced by
 * denscalc (rpb, thetapb, phipb) and integrated over tau
 * (line of sight angle).
 */

/* NOTE:  coordinates r, theta, phi are plane of sky coordinates
 * rpb, thetapb, phipb are "physical" spherical 3-d coordinates.
 */

int
pbint (float r, float theta, float phi)

{
  int bangrot (float r_pb, float theta_pb, float phi_pb,
               float *theta_bang, float *phi_bang);
  int throt   (float r_pb, float theta_bang, float phi_bang,
               float *theta_tang, float *phi_tang);
  float bfun (float r);
  float cfun (float r);

  int count    = 0;
  int limbread = 0;

  float alfa;
  float alfap;

  float *bff;
  float *cff;
  float *fgt;
  float *phi_pb;
  float *phi_brot;
  float *phi_rot;
  float *r_pb;
  float *r2;
  float *r3;
  float *sumb;
  float *sumpb;
  float *tnorm;
  float *tau;
  float *theta_pb;
  float *theta_brot;
  float *theta_rot;
  float *zeta;

  float d2r          = 0.0;
  float mult         = 1.0;
  float onedcone     = 0.0;
  float onedconeedge = 0.0;
  float onedtemp     = 0.0;
  float phi_bang;
  float phi_tang;
  float pi           = 3.14159265;
  float pid2         = 0.0;
  float tauint       = 0.0;
  float taumax       = 0.0;
  float taumin       = 0.0;
  float tauminslope  = 0.0;
  float tauslope     = 0.0;
  float theta_bang;
  float theta_tang;
  float widthflag    = 0.0;

  int itau;

  pb      = 0.0;
  bright  = 0.0;

  zeta       = (float *) malloc (ntau * sizeof (float));
  theta_pb   = (float *) malloc (ntau * sizeof (float));
  r_pb       = (float *) malloc (ntau * sizeof (float));
  r2         = (float *) malloc (ntau * sizeof (float));
  r3         = (float *) malloc (ntau * sizeof (float));
  phi_pb     = (float *) malloc (ntau * sizeof (float));
  fgt        = (float *) malloc (ntau * sizeof (float));
  tnorm      = (float *) malloc (ntau * sizeof (float));
  phi_brot   = (float *) malloc (ntau * sizeof (float));
  theta_brot = (float *) malloc (ntau * sizeof (float));
  phi_rot    = (float *) malloc (ntau * sizeof (float));
  theta_rot  = (float *) malloc (ntau * sizeof (float));

  /*** Degrees to radian. ***/

  d2r = 1.745329e-2;

  /*** pi/2 ***/

  pid2 = pi / 2.0;
 
  for (itau = 0; itau < ntau; itau++)
    fgt [itau] = (float) itau;

  /* Both limbs will be represented in the plane of the sky:
   * the angle phi is the central meridian angle
   */

  /* alfa +ve for West (right) limb
   * alfa -ve for East (left) limb
   */ 

  alfa = sin (theta) / abs (sin (theta));
  if (limbread != 0)  alfap = alfa * limbread;
  if (limbread == 0)  alfap = alfa;
 
  tauslope    = (taumax - taumin) - tauint * (ntau - 1);
  tauminslope = taumin + tauslope / 2.0;

  for (itau = 0; itau < ntau; itau++)
    {
    tau   [itau] = tauminslope + (tauint * itau);
    tnorm [itau] = (1 + itau) / (1 + itau);
    }

  for (itau = 0; itau < ntau; itau++)
    {
    zeta     [itau] = atan (tan (tau [itau]) * (1 / sin (theta)));
    r2       [itau] = tnorm [itau] * r;
    r_pb     [itau] = (1 / cos (tau [itau])) * r;
    theta_pb [itau] = acos (cos (tau [itau]) * cos (theta)) / d2r;
    phi_pb   [itau] = (tnorm [itau] * alfap * pid2 - alfap * zeta [itau]) / d2r;
    }

  /*** First add Bangle if necessary. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    bangrot (r_pb [itau], theta_pb [itau], phi_pb [itau],
             &theta_bang, &phi_bang);
    theta_brot [itau] = theta_bang;
    phi_brot   [itau] = phi_bang;
    }

  for (itau = 0; itau < ntau; itau++)
    phi_brot [itau] = phi_brot [itau] + tnorm [itau] * phi / d2r;

  /*** Now put in model structure rotation if necessary. ***/

  /*
   * test = where (phi_bang > 360.0);
   * if (min (test) != -1)  phi_bang [test] = phi_bang [test] - 360.0;
   * test = where (phi_bang < 0.0);
   * if (min (test) != -1)  phi_bang [test] = phi_bang [test] + 360.0;
   */

  for (itau = 0; itau < ntau; itau++)
    {
    throt (r_pb [itau], theta_brot [itau], phi_brot [itau],
           &theta_tang, &phi_tang);
    theta_rot [itau] = theta_tang;
    phi_rot   [itau] = phi_tang;
    }

  /*
   * test = where (phi_rot > 360.0);
   * if (min (test) != -1)  phi_rot [test] = phi_rot [test] - 360.0;
   * test = where (phi_rot < 0.0);
   * if (min (test) != -1)  phi_rot [test] = phi_rot [test] + 360.0;
   */

  if (idens == 1)  dens1 (r_pb, theta_rot, phi_rot);
  if (idens == 2)  dens2 (r_pb, theta_rot, phi_rot);
  if (idens == 3)  dens3 (r_pb, theta_rot, phi_rot);
  if (idens == 4)  dens4 (r_pb, theta_rot, phi_rot);
  if (idens == 5)  dens5 (r_pb, theta_rot, phi_rot);
  if (idens == 6)  dens6 (r_pb, theta_rot, phi_rot);

  /*** Get width info. ***/

/*
 * if (idens == 4)
 *   {
 *   widthflag = density * 0.0;
 *   if (min (coneedge) != -1)  widthflag [coneedge] = 1.0;
 *   }
 */

  /*** First calculate pb. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    cff [itau] = cfun (r_pb [itau]);
    }

  sumpb = (float *) malloc (ntau * sizeof (float));

  for (itau = 0; itau < ntau; itau++)
    {
    sumpb [itau] = cff [itau] * r2 [itau] * tauint;
    sumpb [itau] = density * sumpb [itau];
    }

  /*** Now calculate bright. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    bff [itau] = bfun (r_pb [itau]);
    }

  sumb   = (float *) malloc (ntau * sizeof (float));

  for (itau = 0; itau < ntau; itau++)
    {
    sumb [itau]  = bff [itau] * tauint * r_pb [itau] * r_pb [itau] / r2 [itau];
    sumb [itau] = 2.0 * density * sumb [itau];
    }

  /*** Now integrate both and set up width array. ***/

  count = 0;

  for (itau = 1; itau < ntau; itau++)
    {
    pb     = pb     + sumpb [itau];
    bright = bright + sumb  [itau] - sumpb [itau];
    }

/*
 * if (idens == 4)
 *   {
 *    onedtemp = where (widthflag [i, *] eq 1.);
 *   if (min (onedtemp) != -1)
 *     {
 *     onelen   = size (onedtemp);
 *     onelen   = onelen (1);
 *     onedcone = onedtemp;
 *     count    = count + onelen;
 *     }
 *   }
 */

/*
 * if (idens == 4)
 *   {
 *   test = where (onedcone != 0.0);
 *   if (min (test) != -1)  onedconeedge = onedcone;
 *   if (min (test) == -1)  onedconeedge = -1;
 *   }
 */

  pb     = mult * pb;
  bright = mult * bright;

  /*** And add in plane of sky info for the symmetric case. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    pb     = pb                   + sumpb [itau];
    bright = bright + sumb [itau] - sumpb [itau];
    }

  return (0);
}

/* -------------------------------------------------------------------------- */

/*** Calculates scattering function needed for brightness calculation. ***/

float
bfun (float r)

{
  float a1, a2, a3;
  float cfunc, df, dfunc, bf;

/*
  float *cfunc ;
  float *df ;
  float *dfunc ;
  float *bf ;
 */

  a1 = 8.69e-7;
  a2 = 0.37;
  a3 = 0.63;
 
  /*** Allocate storage. ***/

/*
  cfunc = (float *) malloc (ntau * sizeof (float));
  df    = (float *) malloc (ntau * sizeof (float));
  dfunc = (float *) malloc (ntau * sizeof (float));
  bf    = (float *) malloc (ntau * sizeof (float));
 */
 
  cfunc = 4.0 / 3.0 - sqrt (1.0 - 1.0 / r / r)
          - pow ((1.0 - 1.0 / r / r),(1.5)) / 3.0;
  df    = (r - 1.0 / r) * (5 - 1.0/ r / r)
          * alog ((1.0 + 1.0 / r) / sqrt (1.0 - 1.0 / r / r));
  dfunc = 1.0 / 8.0 * (5.0 + 1.0 / r / r - df);
  bf    = a1 * (a2 * cfunc + a3 * dfunc);
 
/*
  free (cfunc);
  free (df) ;
  free (dfunc);
 */

  return (bf);
}

/* -------------------------------------------------------------------------- */

/* Calculates scattering function needed for brightness calculation
 * and also polarized brightness.
 */

float
cfun (float r)

{
  float a1, a2, a3;
  float afunc, bf, bfunc, cf;

  a1 = 8.69e-7;
  a2 = 0.37;
  a3 = 0.63;
 
  /*** Allocate storage. ***/

/*
  afunc = (float *) malloc (ntau * sizeof (float));
  bf    = (float *) malloc (ntau * sizeof (float));
  bfunc = (float *) malloc (ntau * sizeof (float));
  cf    = (float *) malloc (ntau * sizeof (float));
 */
 
  afunc = sqrt (1.0 - 1.0 / r / r) * (1.0 / r / r);
  bf    = (r - 1.0 / r) * (1 + 3.0 / r / r)
           * alog ((1.0 + 1.0 / r) / sqrt (1.0 - 1.0 / r / r));
  bfunc = -1.0 / 8 * (1 - 3 / r / r - bf);
  cf     = a1 * (a2 * afunc + a3 * bfunc);
 
/*
  free (afunc) ;
  free (bf) ;
  free (bfunc) ;
 */

  return (cf);
}

/* -------------------------------------------------------------------------- */

int
bangrot (float r_pb, float  theta_pb,   float  phi_pb,
                     float *theta_brot, float *phi_brot)

{
  float theta_rot;
  float phi_rot;
  float xx, yy, zz;
  float xp, yp, zp;
  float d2r = 1.745329e-2;

  xx = r_pb * sin (theta_pb * d2r) * sin (phi_pb * d2r);
  yy = r_pb * sin (theta_pb * d2r) * cos (phi_pb * d2r);
  zz = r_pb * cos (theta_pb * d2r);

  /* Change line of sight due to B angle  tilt.
   * Arfkin p 199
   */

  xp = xx;
  yp = yy * cos (bang * d2r) - zz * sin (bang * d2r);
  zp = yy * sin (bang * d2r) + zz * cos (bang * d2r);

  theta_rot = acos (zp / r_pb);
  phi_rot   = theta_rot * 0.e0;

/*
 * test = where (yp != 0.d0);
 * if (min (test) != -1)  phi_rotB [test] = atan (xp [test] / yp [test]);
 * test = where (yp eq 0.d0 and xp gt 0.d0);
 * if (min (test) != -1)  phi_rotB [test] = !pi / 2.d0;
 * test = where (yp eq 0.d0 and xp lt 0.d0);
 * if (min (test) != -1)  phi_rotB [test] = -!pi / 2.d0;
 * if (min (test) != -1)  stop;
 * test = where (yp < 0.0d0);
 * if (min (test) != -1)
 *   phi_rot [test] = phi_rot [test] + double (!pi);
 */

  /*** Convert radians to degrees. ***/

  *theta_brot = theta_rot / d2r;
  *phi_brot   = phi_rot   / d2r;
}

/* -------------------------------------------------------------------------- */

int
throt (float r_pb, float  theta_pb,   float  phi_pb,
                   float *theta_trot, float *phi_trot)

{
  int itau;
  float theta_rot;
  float phi_rot;
  float d2r = 1.745329e-2;
  float xx, yy, zz;
  float xp, yp, zp;

  xx = r_pb * sin (theta_pb * d2r) * sin (phi_pb * d2r);
  yy = r_pb * sin (theta_pb * d2r) * cos (phi_pb * d2r);
  zz = r_pb * cos (theta_pb * d2r);

  /* Rotate dens structure at limb by theta = lat.
   * Arfkin p 199
   */

  yp =   yy;
  xp =   xx * cos (lat * d2r) + zz * sin (lat * d2r);
  zp = - xx * sin (lat * d2r) + zz * cos (lat * d2r);

  theta_rot = acos (zp) / r_pb;
  phi_rot   = theta_rot * 0.e0;

/*
 * test = where (yp != 0.d0);
 * if (min (test) != -1)  phi_rot [test] = atan (xp [test] / yp [test]);
 * test = where (yp == 0.d0 and xp gt 0.d0);
 * if (min (test) != -1)  phi_rot [test] =  !pi / 2.d0;
 * test = where (yp == 0.d0 and xp lt 0.d0);
 * if (min (test) != -1)  phi_rot [test] = -!pi / 2.d0;
 * test = where (yp < 0.0d0);
 * if (min (test) != -1)  phi_rot [test] = phi_rot [test] + double (!pi);
 */

  /*** Convert radians to degrees. ***/

  *theta_trot = theta_rot / d2r;
  *phi_trot   = phi_rot   / d2r;
}

/* -------------------------------------------------------------------------- */
