/* pbb3d.v4.c		[converted to C from IDL procedure: pbb3d.pro]
 * *****************************************************************************
 * Input:  3D electron number density array.
 * Output: 2D pB, B [integrated along the line-of-sight].
 * *****************************************************************************
 * "pbb3d.pro" acquired from Sarah Gibson.
 * C conversion: Andrew L. Stanger   completed 17 Aug 2004
 * *****************************************************************************
 */
#include <stdio.h>
#include <math.h>

float cosf   (float);
float sinf   (float);
float tanf   (float);
float acosf  (float);
float asinf  (float);
float atanf  (float);
float logf   (float);
float log10f (float);
float powf   (float, float);
float sqrtf  (float);

/*** Global variables. ***/

int limbread = 0;
int nx       = 256;
int ny       = 256;
int ntau     = 0;
int test     = 0;

float *pbimg;
float *brightimg;
float *thsave;
float *bright2d;
float *pb2d;

float bang      =  0.0;
float bright    =  0.0;			/* brightness */
float cmer      =  0.0;
float degrad    =  0.0;
float density   =  1.0;
float lat       =  0.0;
float maxheight =  1.5;			/* Rsun */
float mult      =  1.0;
float pb        =  0.0;			/* pB */
float tauint    =  0.1;
float taumin    = -1.57;
float taumax    =  1.57;
float x;
float y;

/* -------------------------------------------------------------------------- */

main (int argc, char *argv [])

{
  void inputs ();
  int pbint (float r, float theta, float phi);

  int ix;
  int iy;

  float r;
  float phi;
  float theta;

  float dx        =  0.0;
  float maxpb     =  0.0;
  float minpb     =  0.0;
  float pi        =  3.14159265;
  float rwid      =  0.0;
  float thetwid   =  0.0;

  tauint = 0.5;
  ntau = (int) ((taumax - taumin) / tauint + 1.0);


  degrad = 180.0 / pi;

  /*** Get inputs. ***/

  /* inputs (); */

  /* The plane of sky coordinates are calculated (r, theta, phi).
   *
   * I decided to do these evenly spaced in x-y so that we could
   * then display a 2D image (I knew there was a reason for an x-y grid!).
   */

  pbimg        = (float *) malloc (nx * ny * sizeof (float));
  brightimg    = (float *) malloc (nx * ny * sizeof (float));
  thsave       = (float *) malloc (nx * ny * sizeof (float));

  bright2d  = (float *) malloc (512 * 512        * sizeof (float));
  pb2d      = (float *) malloc (512 * 512        * sizeof (float));

  dx     = 2.0 * maxheight / (nx - 1.0);

  for (iy = 0; iy < ny; iy++)
    {
    for (ix = 0; ix < nx; ix++)
      {
      x = -maxheight + ix * (2 * maxheight / (nx - 1.0));
      y = -maxheight + iy * (2 * maxheight / (ny - 1.0));

      /*** Now convert to r, theta (which is what the pb integral wants). ***/

      r      = sqrtf (x * x  + y * y );
      theta  = acosf (y  / r);

      /*
       * neg   = where (x lt 0.0);
       * if (min (neg) != -1)  theta [neg] = 2.0 * pi - theta [neg];
       */

      if (x != -1) theta = 2.0 * pi - theta;

      /*** And your phi is the central meridian of the "day" you want. ***/

      phi = cmer + 10.0;

      phi = phi * pi / 180.0;        /* Convert to radians. */

      if (x == -1000)
	  printf ("ix/iy:%3d  x:%7.2f  y:%7.2f  r:%7.2f  theta:%7.2f  phi:%7.2f\n",
		   ix, x, y, r, theta * degrad, phi * degrad);

      /*** Now calculate pb and B!. ***/

      limbread = 1;
      pbint (r, theta, phi);

      if (ix == iy)
      printf ("ix/iy:%3d  pb: %8.4e  bright: %8.4e\n", ix, pb, bright);

      *(pbimg     + ix + iy*nx) = (float) log10f (pb );
      *(brightimg + ix + iy*nx) = (float) log10f (bright );
      }

    /*** Block out the solar disks. ***/

/*    test = where (r lt 1.0);
 *    if (min (test) != -1)  pbimg     [iy][test] = -100.0;
 *    if (min (test) != -1)  brightimg [iy][test] = -100.0;
 */
    }

  /*** Normalization for CASE 1 vs. CASE 2. ***/

  minpb = -2.5;
  maxpb =  2.5;

  /*** rebin. ***/

  /*
   * pbimg     = rebin (pbimg,     512, 512)
   * brightimg = rebin (brightimg, 512, 512)
   */

  /*** Display the pb image. ***/

  /*
   * window, 2,    $
   * xs=512,  ys=512,  retain=3, title='pb model'
   */

  for (iy = 0; iy < ny; iy++)
    for (ix = 0; ix < nx; ix++)
      {
      *(pb2d + ix + iy*nx) = *(pbimg + ix + iy*nx);
      }

  /*** And plot. ***/

  /* tv, bytscl (pb2d, min=minpb, max=maxpb) */

  /*** And plot. ***/

  /*
   * ;    print, 'type .c to continue to modeled brightness image'
   * ;    stop
   */

  /*** Show the brightness movie. ***/

  /* window, 3, xs=512, ys=512, retain=3, title='brightness model' */

  for (iy = 0; iy < ny; iy++)
    for (ix = 0; ix < nx; ix++)
      *(bright2d + ix + iy*nx) = *(brightimg + ix + iy*nx);

  /* tv, bytscl (bright2d, min=minpb, max=maxpb) */

  exit (0);
}

/* -------------------------------------------------------------------------- */

void
inputs ()

{
  /*** How far out do you extend? ***/

  /* read, 'To what height do you wish the image to go? (Rsun) ', maxheight */

  /*** What grid spacing do you want? ***/
       
  /* read, 'Input grid dimensions (e.g. 256X256, type 256) ', nx */
  ny =  nx;

  /* The l.o.s. integrating angle tau goes basically from -pi/2 to pi/2.
   * tauint = .1 is the integration step I've been using, but you could
   * them finer or coarser if you wish.
   */
  /*read, 'Maximum l.o.s integrating angle? (type 1.57 for default) ', taumax */
  /*read, 'l.o.s integrating angle step-size? (type .1 for default) ', tauint */

  /* What central meridian phi do you want?   (radians)
   * (You may want to step through these do see your structure
   * rotate round if it's not azimuthally symmetric).
   */

  cmer = 0.0;
  /* read, 'What central meridian longitude (degrees)? (0
puts structures at west limb for CME models) ', cmer */

  /*** Also position in latitude. ***/

  lat = 0.0;
  /* read, 'What change in latitude (degrees)? (0 returns structure centered
at equator, +/-  is below/above ', lat */

  bang = 0.0;
  /* read, 'What change in viewing angle (i.e. B angle tilt) (degrees)? (0
is 0 B angle, +/-  is below/above ', bang */

}

/* -------------------------------------------------------------------------- */

/* Calculates integrated line-of-sight polarized brightness pB (r, theta, phi)
 * and brightness bright (r, theta, phi) from density and integrated over tau
 * (line of sight angle).
 */

/* NOTE:  coordinates r, theta, phi are plane of sky coordinates.
 * r_pb, theta_pb, phi_pb are "physical" spherical 3-d coordinates.
 */

int
pbint (float r, float theta, float phi)

{
  /*** Function prototypes. ***/

  int bangrot (float r_pb, float theta_pb, float phi_pb,
               float *theta_bang, float *phi_bang);
  int throt   (float r_pb, float theta_bang, float phi_bang,
               float *theta_tang, float *phi_tang);
  float bfun (float r);
  float cfun (float r);

  /*** Local variables. ***/

  int itau;
  int limbread = 0;

  float alfa;
  float alfap;

  float *bff;
  float *cff;
  float *phi_pb;
  float *phi_brot;
  float *phi_rot;
  float *r_pb;
  float *r2;
  float *r3;
  float *sumb;
  float *sumpb;
  float *tnorm;
  float *tau;
  float *theta_pb;
  float *theta_brot;
  float *theta_rot;
  float *zeta;

  float d2r          = 1.745329e-2;	/* Degrees to radians. */
  float mult         = 1.0;
  float onedcone     = 0.0;
  float onedtemp     = 0.0;
  float phi_bang;
  float phi_tang;
  float pi           = 3.14159265;	/* pi */
  float pid2         = 0.0;
  float tauminslope  = 0.0;
  float tauslope     = 0.0;
  float theta_bang;
  float theta_tang;
  float widthflag    = 0.0;

/*
 *  printf ("r: %10.4e  theta: %10.4e  phi: %10.4e\n", r, theta, phi);
 */

  /*** Initialize variables. ***/

  pid2    = pi / 2.0;
  pb      = 0.0;
  bright  = 0.0;

  /*** Allocate storage. ***/

  zeta       = (float *) malloc (ntau * sizeof (float));
  theta_pb   = (float *) malloc (ntau * sizeof (float));
  r_pb       = (float *) malloc (ntau * sizeof (float));
  r2         = (float *) malloc (ntau * sizeof (float));
  r3         = (float *) malloc (ntau * sizeof (float));
  phi_pb     = (float *) malloc (ntau * sizeof (float));
  tnorm      = (float *) malloc (ntau * sizeof (float));
  phi_brot   = (float *) malloc (ntau * sizeof (float));
  theta_brot = (float *) malloc (ntau * sizeof (float));
  phi_rot    = (float *) malloc (ntau * sizeof (float));
  theta_rot  = (float *) malloc (ntau * sizeof (float));
  sumb       = (float *) malloc (ntau * sizeof (float));
  sumpb      = (float *) malloc (ntau * sizeof (float));
  tau        = (float *) malloc (ntau * sizeof (float));
  bff        = (float *) malloc (ntau * sizeof (float));
  cff        = (float *) malloc (ntau * sizeof (float));
 
  /* Both limbs will be represented in the plane of the sky:
   * the angle phi is the central meridian angle.
   */

  /* alfa +ve for West (right) limb
   * alfa -ve for East (left)  limb
   */ 

  alfa = sinf (theta) / abs (sinf (theta));
  if (limbread != 0)  alfap = alfa * limbread;
  if (limbread == 0)  alfap = alfa;
 
  tauslope    = (taumax - taumin) - tauint * (ntau - 1);
  tauminslope = taumin + tauslope / 2.0;

  for (itau = 0; itau < ntau; itau++)
    {
    tau   [itau] = tauminslope + (tauint * itau);
    tnorm [itau] = (1 + itau) / (1 + itau);
    }

  for (itau = 0; itau < ntau; itau++)
    {
    zeta     [itau] = atanf (tan (tau [itau]) * (1 / sinf (theta)));
    r2       [itau] = tnorm [itau] * r;
    r_pb     [itau] = (1 / cosf (tau [itau])) * r;
    theta_pb [itau] = acosf (cosf (tau [itau]) * cosf (theta)) / d2r;
    phi_pb   [itau] = (tnorm [itau] * alfap * pid2 - alfap * zeta [itau]) / d2r;
/*
 *    printf ("itau: %3d  r_pb: %10.4e\n", itau, r_pb [itau]);
 */
    }

  /*** First add Bangle if necessary. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    bangrot (r_pb [itau], theta_pb [itau], phi_pb [itau],
             &theta_bang, &phi_bang);
    theta_brot [itau] = theta_bang;
    phi_brot   [itau] = phi_bang;
/*
 *    printf ("itau:%3d  theta_brot:%7.2f  phi_brot:%7.2f\n",
 *	     itau, theta_brot [itau], phi_brot [itau]);
 */
    }

  for (itau = 0; itau < ntau; itau++)
    {
    phi_brot [itau] = phi_brot [itau] + tnorm [itau] * phi / d2r;
/*
 *    printf ("itau:%3d  phi_brot:%7.2f\n", itau, phi_brot [itau]);
 */
    }

  /*** Now put in model structure rotation if necessary. ***/

  /*
   * test = where (phi_bang > 360.0);
   * if (min (test) != -1)  phi_bang [test] = phi_bang [test] - 360.0;
   * test = where (phi_bang < 0.0);
   * if (min (test) != -1)  phi_bang [test] = phi_bang [test] + 360.0;
   */

  for (itau = 0; itau < ntau; itau++)
    {
    throt (r_pb [itau], theta_brot [itau], phi_brot [itau],
           &theta_tang, &phi_tang);
    theta_rot [itau] = theta_tang;
    phi_rot   [itau] = phi_tang;
/*
 *    printf ("itau:%3d  theta_rot:%7.2f  phi_rot:%7.2f\n",
 *	     itau, theta_rot [itau], phi_rot [itau]);
 */

    }

  /*
   * test = where (phi_rot > 360.0);
   * if (min (test) != -1)  phi_rot [test] = phi_rot [test] - 360.0;
   * test = where (phi_rot < 0.0);
   * if (min (test) != -1)  phi_rot [test] = phi_rot [test] + 360.0;
   */

  /*** First calculate pB. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    cff   [itau] = cfun (r_pb [itau]);
    sumpb [itau] = density * cff [itau] * r2 [itau] * tauint;

/*
    printf ("itau: %3d  cff: %10.4e  sumpb: %10.4e\n",
	     itau, *(cff + itau), *(sumpb + itau));
 */

    }

  /*** Now calculate brightness. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    bff  [itau] = bfun (r_pb [itau]);
    sumb [itau] = 2.0 * density * bff [itau] * tauint * r_pb [itau]
                      * r_pb [itau] / r2 [itau];
    }

  /*** Now integrate both and set up width array. ***/

  for (itau = 1; itau < ntau; itau++)
    {
    pb     = pb     + sumpb [itau];
    bright = bright + sumb  [itau] - sumpb [itau];
    }

  pb     = mult * pb;
  bright = mult * bright;

  /*** And add in plane of sky info for the symmetric case. ***/

  for (itau = 0; itau < ntau; itau++)
    {
    pb     = pb                   + sumpb [itau];
    bright = bright + sumb [itau] - sumpb [itau];
    }

  return (0);
}

/* -------------------------------------------------------------------------- */

/*** Calculates scattering function needed for brightness calculation. ***/

float
bfun (float r)

{
  float a1, a2, a3;
  float cfunc, df, dfunc, bf;

  a1 = 8.69e-7;
  a2 = 0.37;
  a3 = 0.63;
 
  if (fabs (r) < 1.0)  return (0.0);

  cfunc = 4.0 / 3.0 - sqrtf (1.0 - 1.0 / r / r)
          - powf ((1.0 - 1.0 / r / r), (1.5)) / 3.0;

  df    = (r - 1.0 / r) * (5 - 1.0/ r / r)
          * logf ((1.0 + 1.0 / r) / sqrtf (1.0 - 1.0 / r / r));

  dfunc = 1.0 / 8.0 * (5.0 + 1.0 / r / r - df);

  bf    = a1 * (a2 * cfunc + a3 * dfunc);

  return (bf);
}

/* -------------------------------------------------------------------------- */

/* Calculates scattering function needed for brightness calculation
 * and also polarized brightness.
 */

float
cfun (float r)

{
  float a1, a2, a3;
  float afunc, bf, bfunc, cf;

  a1 = 8.69e-7;
  a2 = 0.37;
  a3 = 0.63;
 
  if (fabs (r) < 1.0)  return (0.0);

  afunc = sqrtf (1.0 - 1.0 / r / r) * (1.0 / r / r);

  bf    = (r - 1.0 / r) * (1 + 3.0 / r / r)
           * logf ((1.0 + 1.0 / r) / sqrtf (1.0 - 1.0 / r / r));

  bfunc = -1.0 / 8 * (1 - 3 / r / r - bf);

  cf     = a1 * (a2 * afunc + a3 * bfunc);

/*
 *  printf ("r:%7.2f  cfun> afunc:%7.2f  bfunc:%7.2f  cf:%10.2e\n",
 *	   r, afunc, bfunc, cf);
 */

  return (cf);
}

/* -------------------------------------------------------------------------- */

/*** Latitude rotation. ***/

int
bangrot (float r_pb, float  theta_pb,   float  phi_pb,
                     float *theta_brot, float *phi_brot)

{
  float theta_rot;
  float phi_rot;
  float xx, yy, zz;
  float xp, yp, zp;
  float d2r = 1.745329e-2;

  xx = r_pb * sinf (theta_pb * d2r) * sinf (phi_pb * d2r);
  yy = r_pb * sinf (theta_pb * d2r) * cosf (phi_pb * d2r);
  zz = r_pb * cosf (theta_pb * d2r);

  /*** Change line of sight due to B angle tilt.  [Arfkin p 199] ***/

  xp = xx;
  yp = yy * cosf (bang * d2r) - zz * sinf (bang * d2r);
  zp = yy * sinf (bang * d2r) + zz * cosf (bang * d2r);

  theta_rot = acosf (zp / r_pb);
  phi_rot   = theta_rot * 0.e0;

  /*
   * test = where (yp != 0.d0);
   * if (min (test) != -1)  phi_rotB [test] = atanf (xp [test] / yp [test]);
   * test = where (yp eq 0.d0 and xp gt 0.d0);
   * if (min (test) != -1)  phi_rotB [test] = !pi / 2.d0;
   * test = where (yp eq 0.d0 and xp lt 0.d0);
   * if (min (test) != -1)  phi_rotB [test] = -!pi / 2.d0;
   * if (min (test) != -1)  stop;
   * test = where (yp < 0.0d0);
   * if (min (test) != -1)
   *   phi_rot [test] = phi_rot [test] + double (!pi);
   */

  /*** Convert radians to degrees. ***/

  *theta_brot = theta_rot / d2r;
  *phi_brot   = phi_rot   / d2r;

  return (0);
}

/* -------------------------------------------------------------------------- */

/*** Longitude rotation. ***/

int
throt (float r_pb, float  theta_pb,   float  phi_pb,
                   float *theta_trot, float *phi_trot)
{
  float theta_rot;
  float phi_rot;
  float d2r = 1.745329e-2;
  float xx, yy, zz;
  float xp, yp, zp;

  xx = r_pb * sinf (theta_pb * d2r) * sinf (phi_pb * d2r);
  yy = r_pb * sinf (theta_pb * d2r) * cosf (phi_pb * d2r);
  zz = r_pb * cosf (theta_pb * d2r);

/*
 *  printf ("r_pb:%6.2f  theta_pb:%6.2f  phi_pb:%6.2f  xx:%6.2f  yy:%6.2f  zz:%6.2f\n",
 *	   r_pb, theta_pb, phi_pb, xx, yy, zz);
 */

  /*** Rotate dens structure at limb by theta = lat.  [Arfkin p 199] ***/

  yp =   yy;
  xp =   xx * cosf (lat * d2r) + zz * sinf (lat * d2r);
  zp = - xx * sinf (lat * d2r) + zz * cosf (lat * d2r);

/*
 *  printf ("xp:%7.2f  yp:%7.2f  zp:%7.2f\n", xp, yp, zp);
 */

  theta_rot = acosf (zp / r_pb);
  phi_rot   = theta_rot * 0.e0;

/*
 *  printf ("theta_rot:%7.2f  phi_rot:%7.2f\n", theta_rot, phi_rot);
 */

  /*
   * test = where (yp != 0.d0);
   * if (min (test) != -1)  phi_rot [test] = atanf (xp [test] / yp [test]);
   * test = where (yp == 0.d0 and xp gt 0.d0);
   * if (min (test) != -1)  phi_rot [test] =  !pi / 2.d0;
   * test = where (yp == 0.d0 and xp lt 0.d0);
   * if (min (test) != -1)  phi_rot [test] = -!pi / 2.d0;
   * test = where (yp < 0.0d0);
   * if (min (test) != -1)  phi_rot [test] = phi_rot [test] + double (!pi);
   */

  /*** Convert radians to degrees. ***/

  *theta_trot = theta_rot / d2r;
  *phi_trot   = phi_rot   / d2r;

  return (0);
}

/* -------------------------------------------------------------------------- */

float
logf (float x)

{
    return ( (float) log ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
log10f (float x)

{
    return ( (float) log10 ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
sinf (float x)

{
    return ( (float) sin ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
cosf (float x)

{
    return ( (float) cos ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
tanf (float x)

{
    return ( (float) tan ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
asinf (float x)

{
    return ( (float) asin ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
acosf (float x)

{
    return ( (float) acos ( (double) x ) );
}

/* -------------------------------------------------------------------------- */

float
atanf (float x)

{
    return ( (float) atan ((double) x) );
}

/* -------------------------------------------------------------------------- */

float powf (float x, float y)

{
    return ( (float) pow ( (double) x, (double) y ) );
}

/* -------------------------------------------------------------------------- */

