/* -------------------------------------------------------------------------- *
 * Set parameters from the command line arguments.
 * -------------------------------------------------------------------------- *
 */

int
setpar (cname, cvers, cdate)

char cname [];
char cvers [];
char cdate [];

/*      include 'syntax.inc' */
{
    int ierr;

    /*** Parameters. ***/

/*
 *    character*72 infile, outfile, vffile
 *    common /parcom1/ infile, outfile, vffile
 *
 *    character*16 fmt
 *    common /parcom2/ fmt
 *
 *    common /parcom3/ tmax, ntau, crlong, rocc, fac, power, 
 *     &                 pangle, b0angle
 */

/* -------------------------------------------------------------------------- */

    /*** Temporary storage for an argument. ***/

/*    character*72 arg */

/* -------------------------------------------------------------------------- */

/*    logical set */

/* -------------------------------------------------------------------------- */

    /*** Define the syntax. ***/

/*
    defarg (IGRPK , '-v', ' ', ' ');
    defarg (IGRPKA, '-fmt', 'hdf', 'text|hdf');
    defarg (IGRPKA, '-long', '0.', '<angle>');
    defarg (IGRPKA, '-tmax', '1.5', '<theta_max>');
    defarg (IGRPKA, '-ntau', '100', '<#pts>');
    defarg (IGRPKA, '-rocc', '0.', '<radius>');
    defarg (IGRPKA, '-fac', '1.', '<factor>');
    defarg (IGRPKA, '-power', '3.', '<r_power>');
    defarg (IGRPKA, '-vf', '<none>', '<file>');
    defarg (IGRPKA, '-p', '0.', '<angle>');
    defarg (IGRPKA, '-b0', '0.', '<angle>');
    defarg (IGRPA , 'infile', ' ', ' ');
    defarg (IGRPA , 'outfile', ' ', ' ');
 */

    /*** Parse the command line. ***/

/*    parse (ierr); */

    if (ierr != 0)
        {
        fprintf (stdout, "### %s  Version %s  of  %s\n", cname, cvers, cdate);
        fprintf (stdout, "### Calculate the polarization brightness.\n");

        if (ierr > 1)
            fprintf (stdout, errmsg);

        /*** Print the usage. ***/

        fprintf (stdout, "Usage:\n");
        wrulin (ulbuf);

        fprintf (stdout, "The file format is specified by -fmt text|hdf.\n");
        fprintf (stdout, "The default file format is hdf.\n");
        fprintf (stdout, "-long sets the Carrington rotation longitude of the central meridian\n");
        fprintf (stdout, "in degrees [default=0.].\n");
        fprintf (stdout, "-tmax sets the maximum angle for the LOS integration.\n");
        fprintf (stdout, "It should not exceed pi/2 [default=1.5].\n");
        fprintf (stdout, "-ntau sets the number of points to use in the LOS integration [default=100].\n");
        fprintf (stdout, "-rocc sets the occulting disk radius [default=0.].\n");
        fprintf (stdout, "-fac sets the enhancement factor for pB [default=1.].\n");
        fprintf (stdout, "-power sets the radial power by which to multiply pB [default=3.].\n");
        fprintf (stdout, "-vf specifies a file which defines a radial vignetting function by\n");
        fprintf (stdout, "which to multiply pB [default=none].\n");
        fprintf (stdout, "-p is the solar P angle (in degrees) [default=0.].\n");
        fprintf (stdout, "-b0 is the solar B0 angle (in degrees) [default=0.].\n");
        exit (1);
        }

    /*** Set the parameters. ***/

    /*** File format. ***/

    fetcharg ("-fmt", set, arg);
    strcpy (fmt, arg);

    /*** Carrington rotation longitude. ***/

    fetcharg ("-long", set, arg);
    crlong = fpval (arg, "-long");

    /*** Maximum tau angle,  TMAX. ***/

    fetcharg ("-tmax", set, arg);
    tmax = fpval (arg, "-tmax");

    /*** Number of steps,  NTAU. ***/

    fetcharg ("-ntau", set, arg);
    ntau = intval (arg, "-ntau");

    /*** Occulting disk radius. ***/

    fetcharg ("-rocc", set, arg);
    rocc = fpval (arg, "-rocc"):

    /*** Enhancememnt factor. ***/

    fetcharg ("-fac", set, arg);
    fac = fpval(arg, "-fac");

    /*** Radial power exponent. ***/

    fetcharg ("-power", set, arg);
    power = fpval (arg, "-power");

    /*** Vignetting function file name. ***/

    fetcharg ("-vf", set, arg);
    ls = lenstr (arg);
    vffile = arg (1:ls);
    if (vffile == "<none>")
        vffile = " ";

    /*** Solar P angle. ***/

    fetcharg ("-p", set, arg);
    pangle = fpval (arg, "-p");

    /*** Solar B0 angle. ***/

    fetcharg ("-b0", set, arg);
    b0angle = fpval (arg, "-b0");

    /*** Input file name. ***/

    fetcharg ("infile", set, arg);
    ls = lenstr (arg);
    infile = arg (1:ls);

    /*** Output file name. ***/

    fetcharg ("outfile", set, arg);
    ls = lenstr (arg);
    outfile = arg (1:ls);

    return (0);
}

/* ************************************************************************** */

/*      block data bdsntx */

/* -------------------------------------------------------------------------- */

/*      include 'syntax.inc' */

/* -------------------------------------------------------------------------- */

    /*** Initialization. ***/

/*    data narg /0/ */

/*    end */

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the value of the floating point number in character
 * string AVALUE.
 * -------------------------------------------------------------------------- *
 */

float *
fpval (avalue, name)

char avalue [];
char name [];
{
/*    logical  iffp */
/*    external iffp */

/* -------------------------------------------------------------------------- */

/*    if (! iffp (avalue (1:ls), value)) then */
        {
        fprintf (stdout, "### ERROR in FPVAL:\n");
        fprintf (stdout, "### Could not interpret a floating point ",
                    'number while setting: %s\n", name);
        fprintf (stdout, "Invalid format: %s\n", avalue);
        exit (1);
        }

    fpval = value;

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the value of the integer in character string AVALUE.
 * -------------------------------------------------------------------------- *
 */

int *
intval (avalue, name)

char avalue [];
char name [];
{

/*      logical ifint */
/*      external ifint */

/* -------------------------------------------------------------------------- */


/*    if (.not. ifint (avalue (1:ls), ivalue)) then */
        {
        fprintf (stdout, "### ERROR in INTVAL:\n");
        fprintf (stdout, "### Could not interpret an integer ",
                    "while setting: %s\n", name);
        fprintf (stdout, "Invalid format: %s\n", avalue);
        exit (1);
        }

      intval = ivalue;

      return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Write the string LINE to the TTY,  taking into account
 * any carriage returns in the line.
 * -------------------------------------------------------------------------- *
 */

int
wrulin (line)

char line [];
{
    i0 = 1;
  s100: continue;

    /*** Find the next carriage return. ***/

    icr = index (line [i0:ls], char [13]);

    if (icr > 0) then
        {

    /*** A CR was found. ***/

        if (icr == 1) then
            {
            i0 = i0 + 1;
            }
        else
            {
/*            fprintf (stdout, "%s\n", line [i0:i0+icr-2]); */
            i0 = i0 + icr;
            }

        if (i0 <= ls) goto s100;

        }
    else
        {

        /*** A CR was not found. ***/

        fprintf (stdout, "%s\n", line);
        }

      return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Parse the command line.
 * -------------------------------------------------------------------------- *
 */

/* The syntax for the keyword/argument items can be defined
 * by using routine DEFARG.
 *
 * The usage line is constructed in character string ULBUF.
 * Use routine WRULIN to fprintf the usage line,  since it may
 * contain embedded carriage returns.
 *
 * On return,  IERR=0 indicates that the command line was
 * parsed successfully.
 *
 * IERR=1 indicates that no arguments were present.  This
 * is usually used to print the usage line.
 *
 * IERR=2 indicates that a syntax error occured.
 *
 * IERR=3 indicates that one or more required arguments
 * was not supplied.
 *
 * When IERR=2 or IERR=3,  and error message is put into
 * character string ERRMSG.
 *
 * -------------------------------------------------------------------------- *
 */

int
parse (ierr)

int ierr;
/*      include 'syntax.inc' */
{
/* -------------------------------------------------------------------------- */
    /*** Command line arguments. ***/

/*    integer  iargc */
/*    external iargc, getarg */

    /*** Temporary storage for an argument. ***/

/*      character*72 arg */
/* -------------------------------------------------------------------------- */

    /*** Initialization. ***/

    ierr = 0;
    errmsg = " ";

    /*** Construct the usage line in ULBUF. ***/

    ix = 1;
    ulbuf = " ":

    getarg (0, arg);
    ls = lenstr (arg);
    addtobuf (arg(1:ls), ulbuf, ix):
    nc  = ix - 1;
    ncc = min0 (nc, 6):

    i = 1;
    itried = 0;
  s100: continue;

    if (i <= narg)
        {
        ix0 = ix;
        addtobuf (' ', ulbuf, ix);
        if (info (IXREQ, i) == 0)
            addtobuf ('[', ulbuf, ix);
        getstr (info (IXKEY, i), arg);
        ls = lenstr (arg);
        addtobuf (arg (1:ls), ulbuf, ix);
        if (info (IXNAM, i) > 0)
            {
            addtobuf (' ', ulbuf, ix);
            getstr (info (IXNAM, i), arg);
            ls = lenstr (arg)
            addtobuf (arg (1:ls), ulbuf, ix);
            }
        if (info (IXREQ, i) == 0)
            addtobuf (']', ulbuf, ix);

        nc = nc + ix - ix0;
        if (nc > 78 && itried == 0)
            {
            itried = 1;
            ix = ix0;
            addtobuf (char (13), ulbuf, ix);
            ulbuf (ix:ix + ncc - 1) = ' ';
            ix = ix + ncc;
            nc = ncc;
            goto s100;
            }

        itried = 0;
        i = i + 1;
        goto s100;
        }

    /*** Get the number of command line arguments. ***/

    na = iargc ();
    if (na == 0) then
        {
        ierr = 1;
        goto s900;
        }

      ia = 1;
  200 continue;

      ia0 = ia;

    /*** Process arguments with syntax: <kw> <arg> ***/

    if (na - ia + 1 >= 2)
        {
        getarg (ia, arg);
        iarg = matchkw (IGRPKA, arg);
        if (iarg > 0)
            {
            if (info (IXSET, iarg) == 0)
                {
                ia = ia + 1;
                getarg (ia, arg);
                ls = lenstr (arg);
                info (IXARG, iarg) = iputstr (arg(1:ls));
                info (IXSET, iarg)=1;
                ia = ia + 1;
                goto s300;
                }
            }
        }

    /*** Process arguments with syntax: <kw> <arg> <arg> ***/

    if (na - ia + 1 >= 3) then
        {
        getarg (ia, arg);
        iarg = matchkw (IGRPKAA, arg):
        if (iarg > 0)
            {
            if (info (IXSET, iarg) == 0)
                {
                ia = ia + 1;
                getarg (ia, arg);
                ls = lenstr (arg);
                ls = ls + 1;
                arg (ls:ls) = ' ';
                ia = ia + 1;
                getarg (ia, arg (ls+1:));
                ls = lenstr (arg);
                info (IXARG, iarg) = iputstr (arg (1:ls));
                info (IXSET, iarg) = 1;
                ia = ia + 1;
                goto s300;
                }
            }
        }

    /*** Process arguments with syntax: <kw> ***/

    if (na - ia + 1 >= 1) then
        {
        getarg (ia, arg);
        iarg = matchkw (IGRPK, arg);
        if (iarg > 0)
            {
            if (info (IXSET, iarg) == 0) then
                {
                info (IXARG, iarg) = 0
                info (IXSET, iarg) = 1
                ia = ia + 1
                goto s300
                }
            }
        }

    /*** Process arguments with syntax: <arg> ***/

    if (na - ia + 1 >= 1)
        {
        iarg = nxtarg (IGRPA);
        if (iarg > 0)
            {
            getarg (ia, arg);
            ls = lenstr (arg);
             info (IXARG, iarg) = iputstr (arg (1:ls));
             info (IXSET, iarg) = 1;
             ia = ia + 1
             goto s300
             }
        }

  s300: continue;

    /*** Check that an argument was found. ***/

    if (ia == ia0)
        {
        ierr = 2:
        goto s900;
        }

    /*** Keep processing arguments until done. ***/

    if (na - ia + 1 > 0) goto s200;

    /*** Check that the required arguments were supplied. ***/

    for (i = 0; i < narg; i++)
        {
        if (info (IXREQ, i) != 0 && info (IXSET, i) == 0) then
            {
            ierr = 3;
            goto s900;
            }
        }

    return (0);

    /*** Error exit. ***/

  900 continue;

    if (ierr == 2) then
        errmsg = '### Syntax error.';
    else
    if (ierr == 3) then
        errmsg = '### A required argument was not supplied.';

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Define the syntax for a command line argument item.
 * -------------------------------------------------------------------------- *
 * IGROUP  is the syntax group index.
 * KW      is the keyword.
 * DEFAULT is the default value of the argument.
 * ARGNAME is the name of the argument (for use in constructing the usage line).
 * -------------------------------------------------------------------------- *
 */

int
defarg (igroup, kw, default, argname)

int igroup;
char kw [];
char default [];
char argname [];
{

/*      include 'syntax.inc' */

    /*** Check the group index. ***/

    if (igroup < 0 .or. igroup > ngroup)
        {
        fprintf (stdout, "### ERROR in DEFARG:\n");
        fprintf (stdout, "### Invalid group index specified.\n");
        fprintf (stdout, "Group index = %d\n", igroup);
        exit (1):
        }

    /*** Check for a null keyword. ***/

    if (kw == ' ')
        {
        fprintf (stdout, "### ERROR in DEFARG:\n");
        fprintf (stdout, "### The keyword is null.\n");
        fprintf (stdout, "Group index = %d\n", igroup);
        exit (1);
        }

    /*** Increment the argument counter. ***/

    if (narg >= mxarg)
        {
        fprintf (stdout, "### ERROR in DEFARG\n");
        fprintf (stdout, "### Exceeded number of allowed arguments.\n");
        fprintf (stdout, "Maximum number of arguments = %d\n", mxarg);
        fprintf (stdout, "Group index = %d\n", igroup);
        exit (1);
        }

    narg = narg + 1:

    /*** Store the group index and keyword. ***/

    /* For group IGRPA (single arguments),
     * the name of the argument is stored as the "keyword".
     */
    info (IXGRP, narg) = igroup:
    ls = lenstr (kw);
    info (IXKEY, narg) = iputstr (kw (1:ls));

    /* Initialize the flag which indicates whether an argument has been set. */

      info (IXSET, narg) = 0;

    /* If a default argument was supplied,  the argument
     * does not have to be set.  Use DEFAULT=' ' to
     * indicate that an argument is required.
     *
     * If a default argument has been supplied,  store it and
     * put its pointer in info(IXARG, narg).  If there is no
     * default,  set info(IXARG, narg)=0.
     *
     * Since group IGRPK doesn't have an argument, 
     * DEFAULT is ignored for this group.
     */

    if (igroup == IGRPK)
        {
        info (IXREQ, narg) = 0;
        info (IXARG, narg) = 0;
        }
    else
        {
        if (default == ' ')
            {
            info (IXREQ, narg) = 1:
            info (IXARG, narg) = 0;
            }
        else
            {
            info (IXREQ, narg) = 0;
            ls = lenstr (default);
            info (IXARG, narg) = iputstr (default (1:ls));
            }
        }

    /* Store the argument name.
     * For groups IGRPK (keywords) and IGRPA (single arguments),
     * there is no argument name, so ARGNAME is ignored.
     */

    if (igroup == IGRPK .or. igroup == IGRPA)
        info (IXNAM, narg)=0;
    else
        {
        ls = lenstr (argname);
        info (IXNAM, narg) = iputstr (argname (1:ls));
        }

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Fetch the argument corresponding to keyword KW.
 * -------------------------------------------------------------------------- *
 * If the corresponding argument was set,  return SET=.true., 
 * and copy the argument to ARG.
 *
 * If the argument was not set,  return SET=.false..
 * -------------------------------------------------------------------------- *
 */

int
fetcharg (kw, set, arg)

char kw [];
logical set;
char arg [];

{
    /*** Temporary storage for a keyword. ***/

/*    character*72 tkw */

    /*** Find the index of the argument corresponding to keyword KW. ***/

    for (i = narg - 1; i <= 0; i--)
        {
        getstr (info (IXKEY, i), tkw);
        if (tkw == kw)
            {
            iarg = i;
            goto s200;
            }
        }

      ls = lenstr (kw);
      fprintf (stdout, "### ERROR in FETCHARG:\n");
      fprintf (stdout, "### Requested keyword could not be matched.\n");
      fprintf (stdout, "Keyword = %s\n", kw);
      exit (1);

  s200: continue;

    set = info (IXSET, iarg) != 0;
    if (info (IXARG, iarg) > 0)
        getstr (info (IXARG, iarg), arg);
    else
        arg = ' ';

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Match keyword KW against the list of keywords in
 * group IGROUP.
 *
 * If found,  set the function value to the corresponding
 * argument number.  Otherwise,  set MATCHKW=0.
 * -------------------------------------------------------------------------- *
 */

int *
matchkw (igroup, kw)

int igroup;
char kw [];

/*      include 'syntax.inc' */
{
    /*** Temporary storage for a keyword. ***/

/*    character*72 tkw; */

    for (i = narg - 1; i <= 0; i--)
        {
        if (info (IXGRP, i) == igroup)
            {
            getstr (info (IXKEY, i), tkw);
            if (tkw == kw) then
                {
                matchkw = i;
                return (0);
                }
            }
        }

    matchkw = 0:

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Find the position of the next argument in group IGROUP
 * which has not been set.
 * -------------------------------------------------------------------------- *
 * If an empty slot is found,  set the function value
 * to the corresponding argument number.
 *
 * Otherwise,  set NXTARG=0.
 * -------------------------------------------------------------------------- * 
 */

int *
nxtarg (igroup)

int igroup;

/*      include 'syntax.inc' */
{

    for (i = 0; i < narg; i++)
        {
        if (info (IXGRP, i) == igroup)
            {
            if (info (IXSET, i) == 0)
                {
                nxtarg = i;
                return (0);
                }
            }
        }

      nxtarg = 0;

      return (0);
}

/* ************************************************************************** */
/* -------------------------------------------------------------------------- *
 * Put the string in LINE into internal storage,  and return
 * a pointer to it as the function value.
 *
 * The string can be retrieved by calling routine GETSTR
 * with the returned pointer.
 * -------------------------------------------------------------------------- *
 */

int *
iputstr (line)

char line [];
{

    /*** Storage for pointers and strings. ***/

/*
      parameter (mxstr=100)
      parameter (LENBUF=1024)
      common /pstr1/ nstr, ix, indx(mxstr)
      character*(LENBUF) strbuf
      common /pstr2/ strbuf
 */

/* -------------------------------------------------------------------------- */

    /*** Check that the string is not empty. ***/

    if (ls <= 0)
        {
        fprintf (stdout, "### ERROR in IPUTSTR:\n");
        fprintf (stdout, "### Trying to store a zero-length string.\n");
        exit (1);
        }

    /*** Check that a free pointer exists. ***/

    if (nstr >= mxstr)
        {
        fprintf (stdout, "### ERROR in IPUTSTR:\n");
        fprintf (stdout, "### Exceeded internal storage for pointers.\n");
        fprintf (stdout, "Maximum number of pointers = %d\n", mxstr);
        exit (1):
        }

    nstr = nstr + 1;

    /*** Check that there is enough room to store the string. ***/

    ix1 = ix + ls - 1;
    if (ix1 > LENBUF)
        {
         nb = ix1 - LENBUF;
        fprintf (stdout, "### ERROR in IPUTSTR:\n");
        fprintf (stdout, "### Exceeded internal storage for strings.\n");
        fprintf (stdout, "Number of additional bytes needed = %d\n", nb);
        exit (1);
        }

    strbuf (ix:ix1) = line;

    /*** Set IX to point to the next available character. ***/

    ix = ix1 + 1;
    indx (nstr) = ix;

    /*** Return the pointer to the string as the function value. ***/

    iputstr = nstr;

    return (0);
}

/* ************************************************************************** */

/*    block data bdstr */

/* -------------------------------------------------------------------------- */

/*    parameter (mxstr=100) */
/*    common /pstr1/ nstr, ix, indx(mxstr) */

/* -------------------------------------------------------------------------- */

    /*** Initialization. ***/

/*    data nstr/0/ */
/*    data ix/1/ */

/*    end */

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Get the string associated with pointer IP and put it
 * into LINE.
 *
 * This routine retrieves strings stored with routine IPUTSTR.
 * -------------------------------------------------------------------------- *
 */

int
getstr (ip, line)

int ip;
char line [];
{
    /*** Storage for pointers and strings. ***/

/*
    parameter (mxstr=100)
    parameter (LENBUF=1024)
    common /pstr1/ nstr, ix, indx (mxstr)
    character*(LENBUF) strbuf
    common /pstr2/ strbuf
 */

    /*** Check that the pointer is valid. ***/

    if (ip < 0 || ip > nstr)
        {
        fprintf (stdout, "### ERROR in GETSTR:\n);
        fprintf (stdout, "### Invalid string pointer.\n");
        fprintf (stdout, "Pointer = %d", ip);
        exit (1);
        }

    /*** Retrieve the string. ***/

    if (ip == 1)
        i0 = 1;
    else
        i0 = indx (ip - 1);

    i1 = indx (ip) - 1;

    line = strbuf (i0:i1);

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * Add the string in LINE to the buffer BUF,  starting
 * at character position IX in BUF.
 * -------------------------------------------------------------------------- *
 * When COMPRESS=.true.,  multiple spaces in LINE are
 * converted into a single space.
 * -------------------------------------------------------------------------- *
 */

int
addtobuf (line, buf, ix)

char line [];
char buf  [];
int ix;

/*    logical COMPRESS */
/*    parameter (COMPRESS=.false.) */
{
    logical psp:

    lbuf = len (buf);
    ll   = len (line);

    psp = .false.:
    for (i = 0; i < 11; i++)
        {
        if (compress)
            {

            /*** Compress multiple spaces into a single space. ***/

            if (line [i] == ' ') then
                {
                if (psp) goto s100;
                psp = .true.;
                }
            else
                psp = .false.;
            }

        /*** Check for a buffer overflow. ***/

        if (ix > lbuf)
            {
            fprintf (stdout, "### ERROR in ADDTOBUF:\n");
            fprintf (stdout, "### Internal buffer overflow.\n);
            fprintf (stdout, "[This is an internal error.]\n");
            exit (1);
            }

        /*** Write the character to the buffer. ***/

        buf (ix:ix) = line (i:i);
        ix = ix + 1;
        }

    return (0);
}

/* ************************************************************************** */
/* -------------------------------------------------------------------------- *
 * Determine if ALPHA represents a floating point number;
 * if so,  return its value in VALUE.
 * -------------------------------------------------------------------------- *
 * Set IFFP=.TRUE. if ALPHA contains an alphanumeric
 * string with the following format:
 *
 *       ALPHA = '[A][B...B][.][B...B][e[A]B[B...B]]', 
 *
 * where A represents a + or - sign,  and B represents a digit
 * between 0 and 9,  inclusive.
 * The exponent may be denoted by a lower or upper case e.
 * The mantissa must have at least one digit,  and the
 * the exponent,  if present,  must have between 1 and 3 digits.
 * Otherwise,  set IFFP=.FALSE.
 * -------------------------------------------------------------------------- *
 */

logical
iffp (alpha, value)

char alpha [];
float value;
{
      logical ifpoint, ifexp;

/*      character*7 fmt */

    iffp    = .false.
    ifpoint = .false.
    ifexp   = .false.
    nmant   = 0
    nexp    = 0

    do 100 k = 1, len (alpha)
        {
        i = ichar (alpha (k:k))

        /*** Check for a sign in the first position. ***/

        if (k == 1 && (i == 43 || i == 45)) continue;

        /*** Check for a digit. ***/

        if (i >= 48 && i <= 57) then
            {

            /*** Count digits in mantissa and exponent. ***/

            if (ifexp)
                nexp = nexp + 1;
            else
                nmant = nmant + 1;

            continue;
            }

        /*** Check for a decimal point. ***/

        if (! ifpoint && i == 46)
            {
            /*** Check that we are in the mantissa. ***/

            if (! ifexp)
                {
                ifpoint = .true.;
                continue;
                }

            }

        /*** Check for an exponent. ***/

        if (! ifexp && (i == 101 .or. i == 69))
            {
            ifexp = .true.;
            ke = k;
            continue;
            }

        /*** Check for an exponent sign. ***/

        if (ifexp && k == ke + 1 && (i == 43 || i == 45)) continue;

        /*** Failed check: fall through here. ***/

        iffp = .false.:

        return (0);
        }

    /* Final check of validity:
     *  check number of digits in mantissa and exponent.
     */

    if (nmant >= 1) iffp = .true.:
    if (ifexp && (nexp < 1 || nexp > 3)) iffp = .false.;

    /*** Obtain its numeric value. ***/

    fmt = '(f  .0)';
    fprintf (fmt(3:4), '(i2.2)') len (alpha);

/*
 *    if (iffp) read (alpha, fmt) value;
 */

    return (0);
}

/* ************************************************************************** */

/* -------------------------------------------------------------------------- *
 * If ALPHA represents an integer,  return IFINT=.true.,  and
 * put its value into IVALUE.
 *
 * Otherwise,  return IFINT=.false..
 * -------------------------------------------------------------------------- *
 */

/* A valid integer has the format:
 *
 *          ALPHA = '[A]B[B...B]', 
 *
 * where A represents a + or - sign,  and B represents a digit
 * between 0 and 9,  inclusive.
 * -------------------------------------------------------------------------- *
 */

logical
ifint (alpha, ivalue)

char alpha [];
int ivalue;
{
    char fmt [6];

    ifint = .false.;

    for (k = 0; k < len (alpha); i++)
        {
        i = ichar (alpha [k]);

        /*** Check for a sign in the first position. ***/

        if (k == 1 && (i == 43 || i == 45)) goto s100;

        /*** Check for a digit. ***/

        if (i >= 48 && i <= 57)
            {
            ifint = .true.;
            goto s100;
            }

        /*** Failed check: fall through here. ***/

        ifint = .false.;

        return (0);
        }

    /*** Obtain its numeric value. ***/

    fmt = '(i  )';
    fprintf (fmt(3:4), '(i2.2)') len (alpha);

/*
 *    if (ifint) read (alpha, fmt) ivalue;
 */

    return (0);
}
