/* lcm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write colormap entries from a disk file to SGI/IRIS color LUT.
 * User specifies a LUT file and [optionally] the start index level.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   31 October 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
typedef unsigned char  u_char ; 
typedef unsigned short u_short ;

main (argc, argv)

int argc ;
char *argv [] ;

{
   FILE *MAP ;

   static char namemap [80] ;	/* Name of disk file. */
   static char pathmap [80] ;	/* "/home/stanger/color" + file name. */

   static u_short red [256], green [256], blue [256] ;

   u_short rval, gval, bval ;	/* R, G, B color intensitites. */

   int i ;
   unsigned int index ;		/* Color index. */
   int min_level = 512 ;		/* Starting colormap index level. */
   int num_level = 256 ;		/* # colormap levels to write. */
   int offset = 0 ;		/* offset to be subtracted for output file. */

   if (argc == 3)
      {
      strcpy (namemap, argv [1]) ;
      sscanf (argv [2], "%d", &min_level) ;
      }
   else
   if (argc == 2)
      strcpy (namemap, argv [1]) ;
   else
      {
      fprintf (stderr, "usage: %s color_map_filename [start_level]\n",
               argv [0]) ;
      exit (-1) ;
      }

/*
 *   printf ("argc: %d\n", argc) ;
 *   printf ("min_level: %d   num_level: %d   namemap: %s\n",
 *            min_level, num_level, namemap) ;
 */

   /*** If min_level < 0,
    ***  normalize color tables so that index starts with zero.
    ***/


   if (min_level < 0)
      {
      min_level = -min_level ;
      offset = min_level ;
      strcat (namemap, ".map") ;	    /* suffix is ".lut" for normalized map.*/
      }
   else
      {
      offset = 0 ;
      strcat (namemap, ".lut") ;
      }

/*
 *   printf ("min_level: %d   num_level: %d   namemap: %s  offset: %d\n",
 *            min_level, num_level, namemap, offset) ;
 */

   if (num_level <= 0)
      {
      fprintf (stderr, "how_many_levels < 0\n") ;
      exit (-2) ;
      }

   /*** Allocate memory for color tables. ***/

   /*** Read color index, red intensity, green intesity, & blue intensity. ***/

   if ((MAP = fopen (namemap, "r")) == NULL)
      {
      sprintf (pathmap, "/home/stanger/color/%s", namemap) ;
      if ((MAP = fopen (pathmap, "r")) == NULL)
	 {
         fprintf (stderr, "Unable to open file %s\n", pathmap) ;
         exit (-3) ;
	 }
      }

   for (i = 0 ; i < num_level; i++)
      {
      fscanf (MAP, "%u %hu %hu %hu", &index, &rval, &gval, &bval) ;
/*
 *      printf ("i:%3d  r:%3d  g:%3d  b:%3d\n", index, rval, gval, bval) ;
 */
      *(red   + i) = rval ;
      *(green + i) = gval ;
      *(blue  + i) = bval ;
      }
   fclose (MAP) ;

   /*** Write color index + RGB values to SGI LUT. ***/

   put_scmap (min_level, num_level, red, green, blue) ;

/*
 *   fprintf (stdout, "Color map [levels %4d -> %4d] restored from file %s\n",
 *            min_level, min_level + num_level - 1, namemap) ;
 */

   exit (0) ;
}
