/* restore_cmap.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write colormap entries from a disk file to SGI/IRIS color LUT.
 * User specifies a LUT file and the start index level.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  7 September 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
typedef unsigned char  u_char; 
typedef unsigned short u_short;

main (argc, argv)

int argc;
char *argv [];

{
   FILE *MAP;

   static char nammap [80];	/* Name of disk file. */

   static u_short red [256], green [256], blue [256];

   u_short rval, gval, bval;	/* R, G, B color intensitites. */
/*
 *   u_short *red;
 *   u_short *green;
 *   u_short *blue;
 */

   int i;
   unsigned int index;
   int min_level;		/* Starting colormap index level. */
   int num_level = 256;		/* # colormap levels to write. */
   int offset = 0;		/* offset to be subtracted for output file. */

   if (argc == 3)
      {
      sscanf (argv [1], "%d", &min_level);
      strcpy (nammap, argv [2]);
      }
   else
      {
      fprintf (stderr, "usage: %s start_level color_map_filename\n",
               argv [0]);
      exit (-1);
      }

printf ("argc: %d\n", argc);
printf ("min_level: %d   num_level: %d   nammap: %s\n",
         min_level, num_level, nammap);

   /*** If min_level < 0,
    ***  normalize color tables so that index starts with zero.
    ***/


   if (min_level < 0)
      {
      min_level = -min_level;
      offset = min_level;
      strcat (nammap, ".map");	    /* suffix is ".lut" for normalized map.*/
      }
   else
      {
      offset = 0;
      strcat (nammap, ".lut");
      }

printf ("min_level: %d   num_level: %d   nammap: %s  offset: %d\n",
         min_level, num_level, nammap, offset);

   if (num_level <= 0)
      {
      fprintf (stderr, "how_many_levels < 0\n");
      exit (-2);
      }

   /*** Allocate memory for color tables. ***/

/*
 *   red   = (u_short *)  malloc (num_level);
 *   green = (u_short *)  malloc (num_level);
 *   blue  = (u_short *)  malloc (num_level);
 */

   /*** Read color index, red intensity, green intesity, & blue intensity. ***/

   if ((MAP = fopen (nammap, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %d\n", nammap);
      exit (-3);
      }

   for (i = 0; i < num_level; i++)
      {
      fscanf (MAP, "%u %hu %hu %hu", &index, &rval, &gval, &bval);
/*
 *      printf ("i:%3d  r:%3d  g:%3d  b:%3d\n", index, rval, gval, bval);
 */
      *(red   + i) = rval;
      *(green + i) = gval;
      *(blue  + i) = bval;
      }
   fclose (MAP);

   /*** Write color index + RGB values to SGI LUT. ***/

   put_scmap (min_level, num_level, red, green, blue);

/*
   rval = *(red   + 249);
   gval = *(green + 249);
   bval = *(blue  + 249);
   put_scmap (min_level, 1, &rval, &gval, &bval);
 */

   fprintf (stdout, "Color map [levels %4d -> %4d] restored from file %s\n",
            min_level, min_level + num_level - 1, nammap);

   exit (0);
}
