/* save_cmap.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Store SGI/IRIS colormap entries into a disk file.
 * User specifies a range of table entries using start/stop index levels.
 * If start level is negative, it is to be used as an offset level; i.e.,
 * the color level index written to disk will begin with zero.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 17 August 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
typedef unsigned char  u_char; 
typedef unsigned short u_short;

main (argc, argv)

int argc;
char *argv [];

{
   FILE *MAP;

   static char nammap [80];	/* Name of disk file. */

   u_char *red;		/* red   color table. */
   u_char *green;	/* green color table. */
   u_char *blue;	/* blue  color table. */

   int i;
   int min_level;
   int num_level;
   int offset = 0;		/* offset to be subtracted for output file. */

   if (argc == 4)
      {
      sscanf (argv [1], "%d", &min_level);
      sscanf (argv [2], "%d", &num_level);
      strcpy (nammap, argv [3]);
      }
   else
      {
      fprintf (stderr, "usage: %s start_level how_many_levels map_name\n",
               argv [0]);
      exit (-1);
      }

printf ("argc: %d\n", argc);
printf ("min_level: %d   num_level: %d   nammap: %s\n",
         min_level, num_level, nammap);

   /*** If min_level < 0,
    ***  normalize color tables so that index starts with zero.
    ***/


   if (min_level < 0)
      {
      min_level = -min_level;
      offset = min_level;
      strcat (nammap, ".map");	    /* suffix is ".lut" for normalized map.*/
      }
   else
      {
      offset = 0;
      strcat (nammap, ".lut");
      }

printf ("min_level: %d   num_level: %d   nammap: %s  offset: %d\n",
         min_level, num_level, nammap, offset);
   if (num_level <= 0)
      {
      fprintf (stderr, "how_many_levels < 0\n");
      exit (-2);
      }

   /*** Allocate memory for color tables. ***/

   red   = (u_char *)  malloc (num_level);
   green = (u_char *)  malloc (num_level);
   blue  = (u_char *)  malloc (num_level);

   /*** Fetch color table intensities for specified range of levels. ***/

   get_scmap ((u_short) min_level, (u_short) num_level, red, green, blue);

   if ((MAP = fopen (nammap, "w")) == NULL)
      {
      fprintf (stderr, "Unable to create file %d\n", nammap);
      exit (-3);
      }

   for (i = 0; i < num_level; i++)
      fprintf (MAP, "%4d  %3d  %3d  %3d\n",
               i + offset, *(red + i), *(green + i), *(blue + i));
   fclose (MAP);

   fprintf (stdout, "Color map [levels %4d -> %4d] saved in file %s\n",
            min_level, min_level + num_level - 1, nammap);

   exit (0);
}
