; cptape.pro
;+=============================================================================+
; Reads a C/P archive tape (Exabyte in atar format) and stores data to a file.
;+-----------------------------------------------------------------------------+
; written by Dr. C. St.Cyr (HAO) 1991
;+=============================================================================+
pro getdata, drive=drive, x=x, size=size, request=request, ioff=ioff, $
	     taperr=taperr

request = bytarr(size)
ioff = ioff mod 51200
remainder = 51200 - ioff
taperr = 0 & !err = 0

on_ioerror, set_taperr

set_taperr:
   if (!err lt 0) then taperr=1

case 1 of

 (remainder ge size): begin
	request(0)=x(ioff:ioff+size-1) & ioff=ioff+size
	if (ioff eq 51200) then taprd, x, drive		; present x is empty
	end

 (remainder lt size) and ((size-remainder) lt 51200): begin
	request(0) = x(ioff:*)
	joff=remainder &  ioff=0
	taprd, x, drive				; &  print, 'last joff ', joff
	request(joff) = x(0:size-joff-1)
	ioff = size - joff
	if ((ioff eq 0) or (ioff eq 51200)) then taprd, x, drive
	end

 (remainder lt size) and ((size-remainder) ge 51200): begin
	request(0) = x(ioff:*)
	joff = remainder &  ioff=0
	repeat begin
	taprd, x, drive				; & print, 'joff ', joff
	request(joff) = x & joff = joff + 51200
	endrep until joff ge (size-51200)

	taprd, x, drive				; &  print, 'last joff ', joff
	request(joff) = x(0:size-joff-1)
	ioff = size - joff
	if ((ioff eq 0) or (ioff eq 51200)) then taprd, x, drive
	end

else: begin
	print, 'arrrghhh! Hosed in GETDATA!'
	end
endcase
end

;+========================== MAIN PROGRAM =====================================+

on_ioerror, tryagain
TRYAGAIN:
  ans = ''  &  read, 'What is the tape drive number? ', ans
  drive = fix(ans)
on_ioerror, null

mount, drive
                                         
image = 0        &  imcnt = 1
ioff = 0         &  x = bytarr(51200)  		; x stores one tape block.
taperr = 0       & extend = ''
hkp = bytarr(224)
sec = ['SW', 'SS', 'SE', 'EE', 'NE', 'NN', 'NW', 'WW', 'CM']

;>>> Request year & DOY from user.

 on_ioerror, NEWDAY
NEWDAY:
 read, 'Enter last 2 digits of year to search for: ', year  & year = fix(year)
 read, 'Enter DOY to search for: ', doy  &  doy = fix(doy)

 on_ioerror, null
 if (year lt 80 and year gt 89) then goto, EXIT
 if (doy eq 0) then goto, EXIT

;>>> Create sub-directory name : "yy_ddd", where yy = year, ddd = DOY.

 sub_dir = strtrim (year, 2) + '_' + string (doy, '(i3.3)')

 cd, current = current_dir			; Save pwd.
 disk =  'sys$data4:[cp'   &    delim = ']'
 topdata = disk + delim
 print, 'Changing to data directory ', topdata
 print, '      ---------------------------------------------------------'
 print, '     |						 		|
 print, '     |  If the program terminates without successfully		|
 print, '     |	completing, you will still be in that directory.	|
 print, '     |								|
 print, '      ---------------------------------------------------------'

 cd, topdata			; Move to directory specified by "topdata".

;>>> Search for directory specified by "sub_dir".

 dir_string = findfile (sub_dir + '.dir')

;>>> If desired directory doesn't exist, create it.

 if (dir_string(0) eq '') then begin
      print, 'Creating the correct subdirectory [.' + sub_dir + '] in ' +topdata
      target = '[.' + sub_dir + ']'
      spawn, 'create /dir ' + target
 endif 

 direct = disk + '.' + sub_dir + ']'
 print, 'Files will be written to directory ' + direct
 
read, 'Write files to disk? ([n]/y)', ans
if (strupcase (strtrim (ans, 2)) ne 'Y') then wrt = 0 else wrt = 1

;>>> Read tape until requested year/DOY is found.

repeat begin     	; until we get to day requested
 taprd, x, drive  &  yr = fix (string (x(0:1)))  &  day = fix (string (x(2:4)))
 if (day lt doy) then begin 
    print, 'Now skipping YR ', strtrim(yr, 2), ' DOY ', strtrim(day, 2)
    skipf, drive, 1
 endif else if (day gt doy) then begin
      print, 'DOY ', strtrim(doy, 2), ' YR ', strtrim(yr, 2), ' is not on this tape.'
      print, 'Possible data gap, check catalog.'
      print, 'Rewinding tape...'  &  rewind, drive 
      goto, EXIT
    endif                              
endrep until (day eq doy)

; ========================= NOW AT CORRECT DOY =====================

print, 'The data being retrieved is:', ' YR ', strtrim (yr, 2), $
       ' DOY ', strtrim (day, 2)

repeat begin

;>>> Read atar header record.

getdata, drive=drive, x=x, size=512, request=hdr, ioff=ioff, taperr=taperr

filename = string (hdr(0:10))

;>>> Read C/P housekeeping & decode selected items.

getdata, drive=drive, x=x, size=224, request=hkp, ioff=ioff, taperr=taperr

tor = ishft (hkp(14) and '3c'x, -3)				; sector mirror
reso = ishft ((hkp(3) and '4'x), -2)				; resolution
ihiv = 0 & for k = 0, 7 do ihiv = ihiv + (ishft (hkp(9), -k) and 1)	; IH/IV
cal = ishft (hkp(15) and '20'x, -5)				; cal mirror
if (reso eq 0) then extend = 'H'				; High res
if (cal eq 1) then tor = 8					; cal mirror in
if (ihiv ne 8) then extend = 'P'

case 1 of

 (reso eq 1 and ihiv eq 8) or (reso eq 0 and ihiv eq 4): begin

	imsiz = 448L			; length of a image side
	totdata = 200704  		; 448*448 number of pixels
	hskp = 448                      ; size of housekeeping
	bitmap = 448		       	; size of bitmap before data
	postfill = 128			; size of filler after data
	end

 (reso eq 0 and ihiv eq 8): begin

	imsiz = 896			; length of one horizontal scan line
	totdata = 802816 		; 896*896 number of pixels
	hskp = 896			; size of housekeeping
	bitmap = 896			; size of bitmap before data
	postfill = 256			; size of filler after data
	end

 (reso eq 1 and ihiv eq 6): begin

	imsiz = 336L			; length of a image side
	totdata = 112896  		; 336*336 number of pixels
	hskp = 336                      ; size of housekeeping
	bitmap = 336		       	; size of bitmap before data
	postfill = 96			; size of filler after data
	end

 (reso eq 1 and ihiv eq 4) or (reso eq 0 and ihiv eq 2): begin

	imsiz = 224L			; length of a image side
	totdata = 50176   		; 224*224 number of pixels
	hskp = 224                      ; size of housekeeping
	bitmap = 224		       	; size of bitmap before data
	postfill = 64			; size of filler after data
	end

 (reso eq 0 and ihiv eq 6): begin

	imsiz = 672L			; length of a image
	totdata = 451584  		; 672*672 number of pixels
	hskp = 672                      ; size of housekeep
	bitmap = 672		       	; size of bitmap before data
	postfill = 192			; size of filler after data
	end

else: begin 
   print, 'Final image on this day (or unknown image type encountered).'
   taprd, x, drive       ; to get into end-of-file gap
   exit & end
endcase

;>>> Read bitmap.

getdata, drive=drive, x=x, size=(hskp-224+bitmap), request=dummy, ioff=ioff, $
	 taperr=taperr

;>>> Read pixel data from tape.

getdata, drive=drive, x=x, size=totdata, request=a, ioff=ioff, taperr=taperr

;>>> Write pixel data to disk (reversing order of scan lines).
;>>> 1st record is housekeeping.
;>>> 2nd record is last scan line of pixels.
;>>> last record is first scan line of pixels.

print, 'Now processing image number: ', imcnt
imcnt = imcnt+1
if wrt then  get_lun, unit  		; Fetch a LUN for use.
if wrt then  openw, unit, direct + filename + sec(tor) + extend, imsiz
if wrt then  w = assoc (unit, bytarr(imsiz))  
if wrt then  w(0) = hkp 
if wrt then  for i = imsiz, 1, -1 do begin 
     w(long(imsiz)-i+1) = a(((long(i)-1)*imsiz):((long(i)*imsiz)-1))
     end
print, 'Wrote file ' + direct + filename + sec(tor) + extend
if wrt then  free_lun, unit
a = 0				; clear memory of array A
extend = ''                     ; clear filename extender

getdata, drive=drive, x=x, size=postfill, request=dummy, ioff=ioff, $
	 taperr=taperr

endrep until (taperr eq 1)

EXIT:

ans = ''  &  read, 'Do you want to dismount the tape?: ([N]/Y) ', ans
if (strtrim (strupcase (ans), 2) eq 'Y') then dismount, drive

end
;+=============================================================================+
