;+
;
; NAME: 
;	CPHELP1
;
; PURPOSE:
;       Return a string array containing the requested help information.
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;       result = CPHELP(VALUE)
;
; CALLED BY:
;       CPHELP
;
; CALLS TO:
;	none
;
; INPUTS:
;       VALUE	:  the string value of the help item requested
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       Return a string array with help information
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	The value passed is the exact wording of the topic as it appears
;	in the help widget.  A change to the wording of the help topic
;	(when the widget is created - in CPHELP) or to the widget event
;	handler case value (in CPHELP_EV) or to the case values in CPHELPn
;	(where n is a number) will require the same changes occur in all 3
;	locations.
;
; PROCEDURE:
;	Return the help information associated with the input value.
;
; MODIFICATION HISTORY:
;	Oct 93 - Elaine Einfalt (HSTX)
;-


function cphelp1, value

case value of 
    'Getting C/P data to disk' : hval = [                          $
        'Getting C/P data to disk                               ', $
        '                                                       ', $
        '1. Load the C/P archive tape, containing the desired   ', $
        '    data into a tape drive.                            ', $
        '2. Login into an account that has the priviledge to    ', $
        '    write C/P data to the data location used.          ', $
        '3. Enter IDL by typing IDL.                            ', $
        '4. Start the program by typing CPTAPE.                 ', $
        '5. Answer the program prompts as needed.               ', $
        '6. Once the tape has dismounted, return tape to rack.  ', $
        '                                                       ']  

    'Getting started' : hval = [         $
        'Getting started                                        ', $
	'                                                       ', $
 	' REQUIREMENTS :  IDL Version 2 with widgets, or better.', $
	'                 X windows graphics device.            ', $
	'                                                       ', $
	'The access the Solar Data Analysis Center software,    ', $
	'acquire the IDL program that sets up the SDAC tree in  ', $
	'your IDL path.  This routine is called SOLAR_DAC.PRO,  ', $
	'and may be obtained from Solar Data Analysis Center    ', $
	'personnel. In you IDL startup file run this routine:   ', $
	'                                                       ', $
	'        .RUN disk:[subdir]SOLAR_DAC                    ', $
	'        SOLAR_DAC                                      ', $
	'                                                       ', $
	'If you decide to acquire all the individual files,     ', $
	'they are the following:                                ', $
	'                                                       ', $
	'Programs for C/P have the naming convention of CP*.PRO:', $
	'                                                       ', $
	'   CP.PRO, CPCOLOR.PRO, CPDISK.PRO, CPDISPLAY.PRO,     ', $
	'   CPHOUSE.PRO, CPINTERANIMATE.PRO, CPMOVIE.PRO,       ', $
	'   CPONE.PRO, CPSUNCEN.PRO, CPHELP.PRO,CPHELP1.PRO,    ', $
	'   CPHELP2.PRO, CPHELP3.PRO                            ', $
	'                                                       ', $
	'Common block for these programs are named CP*.COMMON : ', $
	'   CPCOLORS.COMMON, CPDISPFILES.COMMON,                ', $
	'   CPGET_R_T.COMMON, CPKEYWORDS.COMMON,                ', $
	'   CPLOGIC_ENVRON.COMMON, CPMOVIDISPLAY.COMMON,        ', $
	'   CPMOVIETIME.COMMON, CPMOV_DIR.COMMON,               ', $
	'   CPNEW_WINDOW.COMMON, CPONE_DIR.COMMON,              ', $
	'   CPPROLOCAL.COMMON, CPUSERINP.COMMON,                ', $
	'   CPWIDGBASES.COMMON                                  ', $
	'                                                       ', $
	'Required program that don''t use the naming convention ', $
	'of CP*.* :                                             ', $
	'                                                       ', $
	'   any widget library routines, ACKNOWLEDGE.PRO,       ', $
	'   OPSYS_LGCL.PRO, PSPLOT.PRO, RESPOND_WIDG.PRO,       ', $
	'   WINUP.PRO	                                        ', $
	'                                                       ', $
	'                                                       ', $
	'                                                       ', $
	'In order to run the C/P display software:              ', $
	'                                                       ', $
	'   Enter IDL to get the IDL> prompt.                   ', $
	'   Enter, to the IDL> prompt, CP                       ', $
	'   The C/P widget interface will appear.               ', $
	'   For help, click the left most mouse key over the    ', $
        '       HELP button.                                    ', $
	'                                                       ']  

    'Output display legend' : hval = [                             $
        'Output display legend                                  ', $
	'                                                       ', $
	'The following describes the legend for C/P displays:   ', $
	'                                                       ', $
	'DATE:	YY:DDD		YY = Year-1900                  ', $
	'			DDD = Day of Year               ', $
        '                                                       ', $
        'TIME:	HH:MM:SS	GMT of exposure                 ', $
        '                                                       ', $
	'CALENDAR DATE                                          ', $
        '                                                       ', $
	'FILTER: Almost all of the C/P images were exposed      ', $
	'        through the wideband GREEN filter, but         ', $
	'        throughout the mission a few images were       ', $
	'        exposed through other settings:                ', $
        '                                                       ', $
	'        WideBand	 	(4435-5075 Angstroms)   ', $
	'        Clear	 	 (4000-7000 Angstroms)          ', $
	'        Green Line	 	(5300-5306 Angstroms)   ', $
	'        Green Continuum 	(5296-5302 Angstroms)   ', $
	'        H-Alpha	 	(6543-6583 Angstroms)   ', $
	'        Red		 	(5990-6464 Angstroms)   ', $
	'        GREEN	 	(5000-5350 Angstroms)           ', $
	'        Blue		 	(4440-4775 Angstroms)   ', $
        '                                                       ', $
	'POLAROID: There were three polarizing filters that     ', $
	'          were occasionally used by C/P.  The direction', $
	'          of the transmitted plane of polarization was ', $
	'          with respect to the North pole of the Sun    ', $
	'          when the spacecraft roll angle was zero (0)  ', $
	'          degrees.                                     ', $
	'                                                       ', $
	'        CLR (no polaroid)                              ', $
	'        +60 degrees                                    ', $
	'          0 degrees                                    ', $
	'        -60 degrees                                    ', $
	'                                                       ', $
	'SECTOR: The C/P used a sector mirror to image quadrants', $
	'        of the solar corona.  There were eight (8)     ', $
	'        possible settings for the sector mirror        ', $
	'        position, named for the corresponding limb of  ', $
	'        the Sun when the spacecraft roll angle was zero', $
	'        (0) degrees.                                   ', $
	'                                                       ', $
	'        N, NE, E, SE, S, SW, W, NW                     ', $
	'                                                       ', $
	'EXPOSURE: This is the duration when the shutter was    ', $
	'          open, measured in seconds.                   ', $
	'                                                       ', $
	'RESOLUTION:  Each pixel corresponded to either 6.4 arc ', $
	'             seconds (HIGH res) or 12.8 arc seconds    ', $
	'             (LOW). A full-frame C/P image was either  ', $
	'             896x896 pixels (for HIGH resolution) or   ', $
	'             448x448 pixels (LOW res).                 ', $
	'                                                       ', $
	'ROLL:  Spacecraft roll angle in degrees.  Note that    ', $
	'       this information is as important as the Sector  ', $
	'       Mirror position to determine what limb has been ', $
	'       imaged.                                         ', $
	'                                                       ']

   'Output modes' : hval = [ $
        'Output modes                                           ', $
	'                                                       ', $
	'There are 2 display modes and 2 data modes.  Which     ', $
	'pair combination you choose determines your output.    ', $
	'You select one Data Mode then determine the Display    ', $
	'Mode.                                                  ', $
	'                                                       ', $
	'                                                       ', $
	'Display modes :                                        ', $
	' Clicking on one of the two display modes causes a menu', $
	' to appear.  From this menu you will select the desired', $
	' data.                                                 ', $
	'                                                       ', $
	' "Display one image" :  When selected the output will  ', $
	'                        be a single image.             ', $
	' "Show a movie"      :  When selected the output will  ', $
	'                        be a movie of selected images. ', $
	'Data modes :                                           ', $
	' Clicking on one of the two Data modes will select the ', $
	' set the data manipulation condition.  Only one of the ', $
	' Data modes will be selected at a time.  If you "turn  ', $
	' off" both mode, then the last selected mode is used.  ', $
	'                                                       ', $
	' "Normal display"    :  When selected the output will  ', $
	'                        represent the data in the file.', $
	' "Make a difference" :  When selected the output will  ', $
	'                        represent 2 images differenced.', $
	'                                                       ']
   'Output mode menues' : hval = [ $
        'Output mode menues                                     ', $
	'                                                       ', $
	'There are 4 different output combinations, each with   ', $
	'it''s own menu.  In general the user will click on the ', $
	'desired subdirectory, then the specific file, and the  ', $
	'image will appear. See the next 4 topics for more help ', $
	'on each output mode combination.                       ', $
	'                                                       ', $
	'Note : You may not change to any other mode combination', $
	'       while another menu is being displayed. A dialog ', $
	'       box will appear, asking you to first cancel the ', $
	'       current menu.                                   ', $
	'                                                       ']
    'Display one image / Normal' : hval = [ $
        'Display one image / Normal                             ', $ 
	'                                                       ', $
	'Use this menu to select a single image to be displayed.', $
	'                                                       ', $
	'- Select one of the subdirectories.  Subdirectoried    ', $
	'  use the designation YY_DOY to represent which day''s   ', $
	'  data is contained in that subdirectory.  Other       ', $
	'  subdirectory designation represent specific data     ', $
	'  sets.  For example: SM_001 contains SMM 1 comet      ', $
	'  images; MO_VIE contains CP movies that have been     ', $
	'  saved.                                               ', $
	'                                                       ', $
	'- Once you click on a subdirectory name, all the files ', $
	'  contained in that subdirectory will be listed in the ', $
	'  file window.                                         ', $
	'                                                       ', $
	'- Click on the file you want to view and the image will', $
	'  appear in an IDL window.  If you have a specific file', $
	'  that is not in the standard data location, you may   ', $
	'  enter the full filename into the text widget and     ', $
	'  press RETURN.                                        ', $
	'                                                       ', $
	'- "Adjust color palette" :                             ', $
	'      This is a pull down menu that gives you control  ', $
	'      over the color table.  Feel free to play with the', $
	'      colors, to get back to the default color table   ', $
	'      just press on "Use appropriate C/P color table". ', $
	'                                                       ', $
	'- "Hardcopy" :                                         ', $
	'      This is a pull down menu that allow you to make  ', $
	'      a print of a the last displayed image.  The print', $
	'      processing begins as soon as you release the     ', $
	'      selected button.                                 ', $
	'                                                       ', $
	'        "Black and White print process" will send the  ', $
	'            image to a greyscale printer.  Characters  ', $
	'            will be black on white.                    ', $
	'        "Color print process" will send the image to a ', $
	'            color printer.  Characters will be black on', $
	'            white.                                     ', $
	'        "Copy screen to Black/White printer (slow)"    ', $
	'            sends the image to a greyscale printer.    ', $
	'            This differs from the above in that the    ', $
	'            image window will be read directly from    ', $
	'            the screen. This means the letters will    ', $
	'            be the colors seen on the screen and the   ', $
	'            background will be black.  This reading    ', $
	'            take a little time.                        ', $
	'        "Copy screen to color printer (slow)"          ', $
	'            sends the image to a color printer.        ', $
	'            This differs from the above in that the    ', $
	'            image window will be read directly from    ', $
	'            the screen. This means the letters will    ', $
	'            be the colors seen on the screen and the   ', $
	'            background will be black.  This reading    ', $
	'            take a little time.                        ', $
	'                                                       ', $
	'- "Redisplay" Allows you to redisplay images after you ', $
	'            have modified control (workbench) menu     ', $
	'            options.  It is not necessary to "CANCEL"  ', $
	'            then selection menu.                       ', $
	'                                                       ', $
	'- "Cancel"  will return you to the Workbench menu.     ', $
	'                                                       ', $
	'                                                       ']
    'Display one image / Difference' : hval = [ $
        'Display one image / Difference                         ', $ 
	'                                                       ', $
	'Use this menu to select a two images to be differenced ', $
	'                                                       ', $
	'- Select one subdirectory for each the base image and  ', $
	'  the subraction image. Subdirectoried use the         ', $
	'  designation YY_DOY to represent which day''s data is  ', $
	'  contained in that subdirectory.                      ', $
	'  Other subdirectory designations represent specific   ', $
	'  data sets.  For example: SM_001 contains SMM 1 comet ', $
	'  images; MO_VIE contains CP movies that have been     ', $
	'  saved.                                               ', $
	'                                                       ', $
	'- Once you click on a subdirectory name, all the files ', $
	'  contained in that subdirectory will be listed in the ', $
	'  file windows.                                        ', $
	'                                                       ', $
	'- Click on a base file and a subtraction file.  Once   ', $
	'  have clicked on one of each the difference image will', $
	'  appear in an IDL window.  If you have specific files ', $
	'  that are not in the standard data location, you may  ', $
	'  enter the full filenames into the text widget and    ', $
	'  press RETURN.                                        ', $
	'                                                       ', $
	'- "Adjust color palette" :                             ', $
	'      This is a pull down menu that gives you control  ', $
	'      over the color table.  Feel free to play with the', $
	'      colors, to get back to the default color table   ', $
	'      just press on "Use appropriate C/P color table". ', $
	'                                                       ', $
	'- "Hardcopy" :                                         ', $
	'      This is a pull down menu that allow you to make  ', $
	'      a print of a the last displayed image.  The print', $
	'      processing begins as soon as you release the     ', $
	'      selected button.                                 ', $
	'                                                       ', $
	'        "Black and White print process" will send the  ', $
	'            image to a greyscale printer.  Characters  ', $
	'            will be black on white.                    ', $
	'        "Color print process" will send the image to a ', $
	'            color printer.  Characters will be black on', $
	'            white.                                     ', $
	'        "Copy screen to Black/White printer (slow)"    ', $
	'            sends the image to a greyscale printer.    ', $
	'            This differs from the above in that the    ', $
	'            image window will be read directly from    ', $
	'            the screen. This means the letters will    ', $
	'            be the colors seen on the screen and the   ', $
	'            background will be black.  This reading    ', $
	'            take a little time.                        ', $
	'        "Copy screen to color printer (slow)"          ', $
	'            sends the image to a color printer.        ', $
	'            This differs from the above in that the    ', $
	'            image window will be read directly from    ', $
	'            the screen. This means the letters will    ', $
	'            be the colors seen on the screen and the   ', $
	'            background will be black.  This reading    ', $
	'            take a little time.                        ', $
	'                                                       ', $
	'- "Redisplay" Allows you to redisplay images after you ', $
	'            have modified control (workbench) menu     ', $
	'            options.  It is not necessary to "CANCEL"  ', $
	'            then selection menu.                       ', $
	'                                                       ', $
	'- "Cancel"  will return you to the Workbench menu.     ', $
	'                                                       ', $
	'                                                       ']
    'Show a movie      / Normal' : hval = [ $
        'Show a movie / Normal                                  ', $
	'                                                       ', $
	'Use this menu to select the images that will be used   ', $
	'as movie frames.   Unless you use save mode, you will  ', $
	'need to recreate each movie once it has been run.      ', $
	'                                                       ', $
	'- Select one of the subdirectories.  Subdirectoried    ', $
	'  use the designation YY_DOY to represent which day''s   ', $
	'  data is contained in that subdirectory.  Other       ', $
	'  subdirectory designation represent specific data     ', $
	'  sets.  For example: SM_001 contains SMM 1 comet      ', $
	'  images; MO_VIE contains CP movies that have been     ', $
	'  saved.                                               ', $
	'                                                       ', $
	'- Once you click on a subdirectory name, all the files ', $
	'  contained in that subdirectory will be listed in the ', $
	'  file window.                                         ', $
	'                                                       ', $
	'- Click on the files you want to use in your movie, in ', $
	'  in correct order.  If you have specific files that   ', $
	'  are not in the standard data location, you may enter ', $
	'  the full filename into the text widget and press     ', $
	'  RETURN.                                              ', $
	'                                                       ', $
	'  NOTE:  To run a pre-existing movie, select that one  ', $
	'         movie file from the file window.              ', $
	'                                                       ', $
	'- "Run movie" :                                        ', $
	'      Once all the movie frames have been selected,    ', $
	'      then press the "Run movie" button.  The movie    ', $
	'      will be loaded into another widget display.      ', $
	'      Use the movie menu to control the display.       ', $
	'                                                       ', $
	'- "Adjust color palette" :                             ', $
	'      This is a pull down menu that gives you control  ', $
	'      over the color table.  Feel free to play with the', $
	'      colors, to get back to the default color table   ', $
	'      just press on "Use appropriate C/P color table". ', $
	'                                                       ', $
	'- "Normal mode" :                                      ', $
	'      When this button is set, then the movie will only', $
	'      be run once then discarded when you exit the     ', $
	'      movie.                                           ', $
	'                                                       ', $
	'- "Save mode" :                                        ', $
	'      When this button is set, then the next movie to  ', $
	'      be run will be saved to a file.  A widget will   ', $
	'      appear asking for you to name your movie.        ', $
	'                                                       ', $
	'      Note : The movie''s name MUST end with ".MOV",    ', $
	'             or else the program will recognize it.    ', $
	'             The program looks for *.MOV file in the   ', $
	'             CP data subdirectory .MO_VIE and the      ', $
	'             current working directory.                ', $
	'                                                       ', $
	'- "Cancel"  will return you to the Workbench menu.     ', $
	'                                                       ', $
	'                                                       ', $
	'Note: Use this output combination to reshow any movie  ', $
	'      that has already been created.  Even difference  ', $
	'      image movies.                                    ', $
	'                                                       ', $
	'                                                       ']

    'Show a movie      / Difference' : hval = [ $
        'Show a movie / Difference                              ', $
	'                                                       ', $
	'Use this menu to select the images that will be used   ', $
	'as movie frames.   Unless you use save mode, you will  ', $
	'need to recreate each movie once it has been run.      ', $
	'                                                       ', $
	'- Select one subdirectory for each the base image and  ', $
	'  the subraction image. Subdirectoried use the         ', $
	'  designation YY_DOY to represent which day''s data is  ', $
	'  contained in that subdirectory.                      ', $
	'  Other subdirectory designations represent specific   ', $
	'  data sets.  For example: SM_001 contains SMM 1 comet ', $
	'  images; MO_VIE is not a valid subdirectory for this  ', $
	'  output mode combination.                             ', $
	'                                                       ', $
	'- Once you click on a subdirectory name, all the files ', $
	'  contained in that subdirectory will be listed in the ', $
	'  file windows.                                        ', $
	'                                                       ', $
	'- Click on pairs of files (base and subtraction) that  ', $
	'  you want to use in your movie, in in correct order.  ', $
	'  If you have specific files that are not in the       ', $
	'  standard data location, you may enter the full name  ', $
	'  into the text widget and press RETURN.               ', $
	'                                                       ', $
	'- "Run movie" :                                        ', $
	'      Once all the movie frames have been selected,    ', $
	'      then press the "Run movie" button.  The movie    ', $
	'      will be loaded into another widget display.      ', $
	'      Use the movie menu to control the display.       ', $
	'                                                       ', $
	'- "Adjust color palette" :                             ', $
	'      This is a pull down menu that gives you control  ', $
	'      over the color table.  Feel free to play with the', $
	'      colors, to get back to the default color table   ', $
	'      just press on "Use appropriate C/P color table". ', $
	'                                                       ', $
	'- "Normal mode" :                                      ', $
	'      When this button is set, then the movie will only', $
	'      be run once then discarded when you exit the     ', $
	'      movie.                                           ', $
	'                                                       ', $
	'- "Save mode" :                                        ', $
	'      When this button is set, then the next movie to  ', $
	'      be run will be saved to a file.  A widget will   ', $
	'      appear asking for you to name your movie.        ', $
	'                                                       ', $
	'      Note : The movie''s name MUST end with ".MOV",    ', $
	'             or else the program will recognize it.    ', $
	'             The program looks for *.MOV file in the   ', $
	'             CP data subdirectory .MO_VIE and the      ', $
	'             current working directory.                ', $
	'                                                       ', $
	'- "Cancel"  will return you to the Workbench menu.     ', $
	'                                                       ', $
	'                                                       ', $
	'Note: If you select this combination, then you may not ', $
	'      reshow an already created movie.  Even if the    ', $
	'      movie is a difference movie that you just made.  ', $
	'      Select "Normal display", the subdirectory MO_VIE ', $
	'      then the movie.                                  ', $
	'                                                       ', $
	'                                                       ', $
	'                                                       ']

  else : hval = 'Topic ' + value + ' is not documented in CPHELP1.PRO'
endcase

return, hval
end
