;+ -----------------------------------------------------------------------------
;  cphk_decode.pro : IDL procedure
;+ -----------------------------------------------------------------------------
;  Decode selected housekeeping items.
;+ -----------------------------------------------------------------------------
;  Andrew L. Stanger   HAO/NCAR
;  24 March 1993
;- -----------------------------------------------------------------------------

pro cphk_decode, cphk=cphk, imgndx=imgndx

@cp_img.common
;COMMON cp_img, nbypx, npx, npy, ifw, ipw, ism, iexp, ires, icm, iroll, iframe,$
;               xcen, ycen, pixrs

@cp_date.common
;COMMON cp_date, iyear, idoy, imonth, iday, ihour, iminute, isecond

   ii = imgndx

   iyear   (ii) = cphk (0) and 'ff'x
   idoy    (ii) = (cphk (1) and 'ff'x) * 256 + (cphk (2) and 'ff'x)
   ihour   (ii) = cphk (20) and 'ff'x
   iminute (ii) = cphk (21) and 'ff'x
   isecond (ii) = cphk (24) and 'ff'x

   ; print, 'cphk_decode> imgndx:', imgndx
   ; print, '   year:doy:hour:min:sec =', $
   ;	iyear (ii), idoy (ii), ihour (ii), iminute (ii), isecond (ii)

   ; Get the day and month from day-of-year and year.

   month = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', $
            'Oct', 'Nov', 'Dec']

   nday = intarr (12)
   nday ( [0, 2, 4, 6, 7, 9, 11] ) = 31  &  nday ( [3, 5, 8, 10] ) = 30
   ly = float (iyear (ii)) / 4.0
   if ((ly - fix (ly)) eq 0.0) then nday (1) = 29 else nday (1) = 28
   ndt = 0
   for i = 0, 11 do begin
      ndt = ndt + nday (i)  &  if (ndt ge idoy (ii)) then goto, label1
   endfor

label1:  imonth (ii) = i  &  iday (ii) = idoy (ii) - ndt + nday (i)

   ; Image width & height.

   npx (ii) = ((cphk (110) and 'ff'x) * 256) + (cphk (111) and 'ff'x)
   npy (ii) = npx (ii)

   ; # bytes/pixel.

   nbypx (ii) = (cphk (103) and 'ff'x) / 8			; # bytes/pixel

   ; Spectral Filter.
   ; cfw = ['RED','H-alpha', 'OPEN', 'BLUE', 'WIDE BD', 'GREEN']

   ifw (ii) = ishft (cphk (15) and 'e'x, -1) - 1

   ; Polaroid Filter.
   ; cpw  = ['CLR', '+60 deg',  '-60 deg', '0 deg']

   ipw (ii) = ishft (cphk (3) and 'c0'x, -6) 

   ; Resolution: HIGH or LOW.

   ires (ii) = ishft ((cphk (3) and '04'x), -2)		; resolution: 0=HI,1=LO

   ; Geometric scale: pixels/Rsun.

   if (ires (ii) eq 0)  then $
      pixrs (ii) = 961.0 / 5.933	$	; pixel/Rsun: High resolution.
   else					$
      pixrs (ii) = 961.0 / 11.866		; pixels/Rsun: Low resolution.

   ; Sector Mirror.
   ; csm = ['SW', 'S', 'SE', 'E', 'NE', 'N', 'NW', 'W']

   jsm = ishft (cphk (14) and '3c'x, -2)		; sector mirror
   if (jsm eq 0)  then ism (ii) = 8  else ism (ii) = (jsm - 1) / 2
   if (ism (ii) eq 8)  then pixrs (ii) = pixrs (ii) / 2.0   ; reduce scale for
							    ; N+E+S+W images.

   ; Exposure duration (seconds).

   a = ishft ((cphk (10) and 'f0'x), -4)
   b = ishft ((cphk (10) and 'f'x), 4) + ishft ((cphk (11) and 'f0'x), -4)
   iexp (ii) = 1.024 * (256*a) + b

   ; Spacecraft roll.  Range: 1-360 degrees.

   iroll (ii) = (cphk (88) and 'ff'x) * 256 + (cphk (89) and 'ff'x)
   ; if (iroll (ii) gt 180) then iroll (ii) = iroll (ii) - 360

   ; Calibration mirror position.

   icm (ii) = ishft ((cphk (15) and '30'x), -4);

   ; Frame number (sequence counter: 0-4095).

   iframe (ii) = ((cphk (30) and 'ff'x) * 16) + $
		 ishft ((cphk (31) and 'f0'x), -4);

   ; The coordinates for sun center of each sector mirror (high res values) :
   ;        SW     S      SE     E      NE     N      NW     W      composite

   x_cen = [ 36.3, 213.1, 539.9, 820.0, 892.4, 718.6, 386.7, 104.3, 447.0]
   y_cen = [499.8, 193.6,   3.3, 87.0,  361.5, 679.3, 856.4, 792.1, 447.0]

   factor = 0.5			; Center coordinates are in High res units, but
				; display window is in Low res units.

   y_cen  = 894 - y_cen     ; put Y coordinate 0 at bottom (CP 0,0 is top left)
   xcen (ii) = x_cen (ism (ii)) * factor
   ycen (ii) = y_cen (ism (ii)) * factor

   ; print, 'cphk_decode> xcen:', xcen (ii), '   ycen:', ycen (ii)
RETURN
END
