/* cpov.c
 * usage: cpov [image_filename] [ovfile]
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write area perimeter using overlay.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Works only for calibrated (short integer) C/P images.
 * Generates its own greyscale color table (saves & restores pre-existing LUT).
 * Opens a temporary GL window for image display.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 29 October 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>

/* Global variables. */

#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/whip/grscale.h"
#define Red 1
#define Green 2
#define Blue 3
#define ESC 27

int DSP = 1;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

extern glcp_view ();		/* Display image in GL window. */

FILE   *LIST, *OV;
static char flag [8];
static char namimg [80];	/* Image file name. */
static char namlst [80];	/* Name of file containing image list. */
static char namov [80];		/* Overlay file name. */
static char nampgm [80];	/* Program file name. */

int Cval = 2;		/* Color index value. */
int i;
int ierr;		/* Error variable. */
int linit = 1;		/* Initialization flag for glcp_view. */
int nval;		/* # values in scan. */
int ovcnt;		/* Overlay file counter. */
int pmode = 0;		/* Print mode for glcp_view. */
int xl = 760;		/* Specification of GL window position & size. */
int yl = 480;
int xu = 1271;
int yu = 991;
int xpos, ypos;
int xwin, ywin;
int xwin1, ywin1;
int xwin2, ywin2;

   /*** Process input parameter(s). ***/

   strcpy (nampgm, argv [0]);
   sprintf (namov, "cpov.pix");

   fprintf (stderr, "%s: draw specified pixels in overlay\n",
            nampgm);

   if (argc == 1)		/* No arguments. */
      {
      fprintf (stderr, "usage: %s [filename_list] [color]\n", argv [0]);
      fprintf (stderr, "Enter name of file containing file names : ");
      fscanf  (stdin,  "%s", namlst);
      }
   else
   if (argc == 2)		/* 1 argument: must be image filename_list. */
      {
      strcpy (namlst, argv [1]);
      }
   else
   if (argc == 3)		/* 3 arguments: 1st must be filename_list. */
      {
      strcpy (namlst, argv [1]);		/* Input image file. */
      sscanf (argv [2], "%d", &Cval);		/* Color index level. */
      }

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %s\n", namlst);
      exit (-1);
      }

   if (fscanf (LIST, "%s", namimg) == EOF)
      {
      fprintf (stderr, "End of file: %s\n", namlst);
      goto quit;
      }

   /*** GL window initialization. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen (nampgm);
   gl_wclear (BLACK);

   /*** Display C/P image in GL window. ***/

   if ((ierr = glcp_view (namimg, linit, pmode)) != 0)
      {
      fprintf (stderr, "Error (%d) in glcp_view.\n", ierr);
      exit (-3);
      }

   /*** Overlay set-up. ***/

   overlay (2);  gconfig ();	/* Establish two bitplanes for overlay. */
   drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
   mapcolor (0,   0,   0,   0);	/* color 0 -> black. */
   mapcolor (1, 255,   0,   0);	/* color 0 -> red.   */
   mapcolor (2,   0, 255,   0);	/* color 0 -> green. */
   mapcolor (3,   0,   0, 255);	/* color 0 -> blue.  */
   gl_wclear (0);

   /*** Read overlay file & write a dot for each coordinate pair. ***/

   while (fscanf (LIST, "%s", namov) != EOF)
      {
      if ((OV = fopen (namov, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open overlay file: %s\n", namov);
         exit (-2);
         }

      /*** Read overlay file & write a dot for each coordinate pair. ***/

      ovcnt = 0;
      while (fscanf (OV, "%d %d", &xpos, &ypos) != EOF)
         {
/*
 *         xwin = xpos + 63;
 *         ywin = 510 - ypos;
 */
         xwin = xpos;
         ywin = ypos;
         if (ovcnt++ == 0)  {xwin1 = xwin;  ywin1 = ywin;  continue;}

         xwin2 = xwin;
         ywin2 = ywin;
         gl_wvec (xwin1, ywin1, xwin2, ywin2, Cval);

         printf ("x/ywin1: %3d %3d   x/ywin2: %3d %3d\n",
                 xwin1, ywin1, xwin2, ywin2);

         xwin1 = xwin2;
         ywin1 = ywin2;
         }
      fclose (OV);
      }

   fclose (LIST);

quit:

   /*** Window remains on screen until terminated by mouse. ***/

   qdevice (ESCKEY);
   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   for (;;)
      {
      short val;
      switch (qread (&val))
         {
         case REDRAW:
            glcp_view (namimg, linit);
            break;
         case ESCKEY:
         case WINCLOSE:
         case WINQUIT:
            drawmode (OVERDRAW);
            gl_wclear (0);
            drawmode (NORMALDRAW);
            onemap ();  gconfig ();   /* Set single-map mode for color table. */
            greset ();
            gexit ();
            exit (0);
         }
      }
}
