/* cpovi.c
 * usage: cpov [image_filename] [ovfile] [color_index];
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write pixels (specified by input file coordinates) into image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Works only for calibrated (short integer) C/P images.
 * Generates its own greyscale color table (saves & restores pre-existing LUT).
 * Opens a temporary GL window for image display.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 13 July 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include <math.h>

/* Global variables. */

#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/whip/grscale.h"
#include "/usr/people/smm/lib/src/whip/cp_color.h"
#include "cp_color.h"
#define Black 0
#define Red 1
#define Green 2
#define Yellow 3
#define Blue 4
#define Magenta 5
#define Cyan 6
#define White 7
#define ESC 27

int DSP = 1;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

   extern char *rdimg ();	/* Read image from disk to memory. */

   FILE   *LIST, *OV;
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Name of file containing image list. */
   static char namov [80];	/* Overlay file name. */
   static char nampgm [80];	/* Program file name. */
   static char namsrf [80];	/* Sun rasterfile file name. */

   static short r [512], g [512], b [512];
   short icol;
   unsigned short cindex;
   int Cval;
   int i, ioff;
   float offset, slope;

   int ierr;		/* Error variable. */
   int linit = 1;	/* Image label flag [0=off,1=on]. */
   int nval;		/* # values in scan. */
   int ovcnt;		/* Overlay file line counter. */
   int ovfcnt;		/* Overlay file counter. */
   int pmode = 0;	/* Print mode. */
   int seq = 0;		/* Flag to indicate sequence of images. */
   int xdim, ydim;	/* X/Y window dimensions. */
   int xl = 760;	/* Specification of GL window position & size. */
   int yl = 480;
   int xu = 1271;
   int yu = 991;
   int xpos, ypos;
   int xwin, ywin;
   int xwin1, ywin1;
   int xwin2, ywin2;

   long afh;
   static char font1 [] = "Screen";
   static char font4 [] = "Charter-Black";
   static char afont [32];			/* Small font. */
   int asize = 7;

   /*** Process input parameter(s). ***/

   strcpy (nampgm, argv [0]);
   sprintf (namov, "cpov.pix");
   Cval = 253;

   fprintf (stderr, "%s: draw specified pixels in overlay\n", nampgm);

   if (argc == 1)		/* No arguments. */
      {
      fprintf (stderr, "usage: %s [filename_list] [color]\n", argv [0]);
      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);
      fprintf (stderr, "Enter name of file containing image & OV file names : ");
      fscanf  (stdin, "%s", namlst);
      fprintf (stderr, "Enter color index level [1=red,2=green,3=yellow,4=blue,5=magenta,6=cyan,7=white]: ");
      fscanf  (stdin,  "%d", &Cval);
      }
   else
   if (argc == 2)		/* 1 argument: must be image filename. */
      {
      strcpy (namlst, argv [1]);
      }
   else
   if (argc == 3)		/* 2 arguments: 1st must be filename_list. */
      {
      strcpy (namlst, argv [1]);		/* Input image file. */
      sscanf (argv [2], "%d", &Cval);		/* Color level. */
      }
   else
      {
      fprintf (stderr, "usage: %s [filename_list] [color]\n", argv [0]);
      exit (0);
      }

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %s\n", namlst);
      exit (-1);
      }

   if (fscanf (LIST, "%s", namimg) == EOF)
      {
      fprintf (stderr, "End of file: %s\n", namlst);
      goto quit;
      }

   /*** Decode C/P housekeeping. ***/

   cphskp (namimg);
   fprintf (stdout,"     G M T          Q(%%)  IMG  IH-IV RES  EXP  FW     PW  5303  SM     CM  ROLL\n");
   hkline (stdout);

   /*** Initialize GL software. ***/

   foreground ();
   if (H.ires == 1)			/* 512x512 window. */
      {
      xdim = ydim = 512;
      prefposition (760, 1271, 480, 991);
      }
   else					/*1024x1024 window.*/
      {
      xdim = ydim = 1024;
      prefposition (266, 1279,   0, 1023);
      }
   fminit ();			/* Initialize SGI font manager. */

   winopen (nampgm);		/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);
   if ((ierr = glcp_view2 (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      exit (-4);
      }

   sprintf (namsrf, "%s.srf", namimg);

   /*** Read overlay file name. ***/

   if (H.ires == 0) 
      {
      asize = 10;
      strcpy (afont, font4);
      }
   else
      {
      asize = 7;
      strcpy (afont, font1);
      }
   afh = gl_setfont (afont, asize);	/* Small font & size. */
   ovfcnt = 0;

   while (fscanf (LIST, "%s", namov) != EOF)
      {
      if ((OV = fopen (namov, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open overlay file: %s\n", namov);
         exit (-2);
         }
      gl_wstr (namov, 3 + ovfcnt*125, 45, cp_grey);
      ovfcnt++;

      /*** Read overlay file & write a dot for each coordinate pair. ***/

      ovcnt = 0;
      while (fscanf (OV, "%d %d", &xpos, &ypos) != EOF)
         {
         xwin = xpos + 63;
         ywin = 510 - ypos;
         if (ovcnt++ == 0)  { xwin1 = xwin; ywin1 = ywin; continue; }

         xwin2 = xwin;
         ywin2 = ywin;
         gl_wvec (xwin1, ywin1, xwin2, ywin2, Cval);

         xwin1 = xwin2;
         ywin1 = ywin2;
         }
      fclose (OV);
      }

   fclose (LIST);

   /*** Create sun rasterfile. ***/

   glsrf (namsrf, xdim, ydim);

quit:

   gexit ();
   exit (0);
}
