/* cpwo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen (over WHIP FB).
 * A WHIP frame buffer (#0) is also used to display image & polygon perimeter.
 * WHIP Region package software is used to process polygon area.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * 10 October 1991: add ability to read cursor position.
 * 23 October 1991: read mouse position only when left button is pressed.
 * 29 October 1991: write all information into output file.
 * 6 November 1991: fill polygon region using WHIP region tools.
 * 18 June 1992: Display difference image for excess mass computation.
 *  7 August 1992: change polygon vertex extension to ".pvX"; X = region number.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/gl/img_scale.h"
#include "/usr/people/smm/lib/src/whip/cp_color.h"
#include "/usr/people/smm/lib/src/whip/ov_color.h"
#include "cp_color.h"
#include "cp_mass.h"

#include <stdio.h>
#include <whip.h>
#include <fmclient.h>
#include <gl/gl.h>
#include <gl/device.h>
#define maxline 84
int DSP = 1;
int	ref_hour, ref_min;	/* Reference hour & minute. */
FILE *pgmlog;			/* Mass log file. */
int regnum = 0;			/* Region index number. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   extern short *polyfill ();		/* Polygon fill function. */
   extern int glcp_view ();		/* Image display function. */
   extern char *rindex ();		/* Index function. */

   static char answer [2];		/* Response to scaling question. */
   static char hkcmp [80];		/* Housekeeping summary for ref image.*/
   static char hkref [80];		/* Housekeeping summary for cmp image.*/
   static char line [maxline];		/* Array to store keyboard input. */
   static char namcmp [80];		/* Comparison image file name. */
   static char namepv [80];		/* Polynomial vertex file name. */
   static char namimg [80];		/* Image file name. */
   static char namlog [80];		/* Output file name. */
   static char namopt [80];		/* Option argument. */
   static char namout [80];		/* Polygon output file name basis. */
   static char namreg [80];		/* Program file name. */
   static char nampgm [80];		/* Program file name. */
   static char namref [80] = "none";	/* Reference image file name. */
   static char namsrf [80];		/* Sun Rasterfile name. */

   char *it;				/* File name pointer. */
   char *parg;				/* Argument pointer. */

   long afh, bfh, cfh;
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "Charter-Black";
   static char afont [32];			/* Small font. */
   static char bfont [32];			/* Medium font. */
   static char cfont [32];			/* Large font. */
   int asize = 7;
   int bsize = 10;
   int csize = 12;

   static int poly [100][2];		/* Polynomial vertices. */
   short *wpos;				/* Pixel array pointer. */
   int FB = 0;
   int action;			/* action to be taken. */
   int fast = 1;		/* Parameter for fb_clear. */
   int i;			/* Index. */
   int idot;			/* Index for '.' character in filename. */
   int ierr;			/* Error status from cpview(). */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int	mx;			/* Mass: =0 Absolute; else, Excess CMP - REF. */
   int nver;			/* # coordinate pairs in array poly. */
   static int nwp;		/* # coordinate pairs inside polygon. */
   int pmode = 0;		/* Print mode: =0 minimal; =1 verbose. */
   int polynum = 0;		/* polygon index.*/
   int ovfcnt = 0;		/* polygon counter. */
   int xocp = 63, yocp = 510;	/* C/P image origin in window coordinates. */
   int xo, yo;			/* x/y origin of current window */
   int xs, ys;			/* x/y size of current window   */
   int xmaxscrn, ymaxscrn;	/* maximum size of screen in x and y       */

   float bkgcol = 0.0;	/* Grey level for fb_clear. */
   float cbth = 0.0;	/* Coronal brightness threshold. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   i = 1;
   igr = 1;			/* Display option on by default. */
   lcc  = 0;			/* Munro correction off by default. */
   lstr = 0;
   lvig = 0;
   lfc  = 0;
   lsv  = 0;
   mx = 0;
   
   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-cd] [namref] [namcmp]\n", nampgm);

      fprintf (stderr,
              "Do you wish to apply Munro calibration correction [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lcc = 1;  else lcc = 0;

      fprintf (stderr, "Do you wish to display image [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  igr = 1;  else igr = 0;

      fprintf (stderr,
             "Enter reference C/P image file name ['none' if non-existent]:  ");
      fgets (line, maxline, stdin);
      sscanf  (line, "%s", namref);

      fprintf (stderr, "Enter comparison C/P image file name:  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", namcmp);
      }
   else
   if (argc == 2)	/* One argument: must be namcmp. */
      {
      igr = 1;
      mx = 0;
      strcpy (namcmp, argv [1]);
      }
   else
   if (argc == 3)	/* Two arguments: namref & namcmp or -d & namcmp. */
      {
      strcpy (namref, argv [1]);
      if (namref [0] == '-')
         {
         if (namref [1] == 'c' || namref [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namref [1] == 'd' || namref [2] == 'd')  igr = 0;  else igr = 1;
         strcpy (namref, "none");
         mx = 0;
         }
      else
         mx = 1;
      strcpy (namcmp, argv [2]);
      }
   else
   if (argc == 4)	/* Three arguments: -cd namref namcmp. */
      {
      strcpy (namopt, argv [1]);
      if (namopt [0] == '-')
         {
         if (namopt [1] == 'c' || namopt [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namopt [1] == 'd' || namopt [2] == 'd')  igr = 0;  else igr = 1;
         }
      strcpy (namref, argv [2]);
      if (strncmp (namref, "none", 4) == 0)  mx = 0;  else mx = 1;
      strcpy (namcmp, argv [3]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-cd] [namref] [namcmp]\n", nampgm);
      exit (-1);
      }

   /*** Decode SMM C/P housekeeping. ***/

   if (strncmp (namref, "none", 4) == 0)  mx = 0;  else mx = 1;

   if (mx)
      {
      cphskp (namref);
      ref_hour = H.hour;
      ref_min  = H.minute;
      hkstr (hkref);
      }

   cphskp (namcmp);
   hkstr (hkcmp);

   /*** Open log file. ***/

   if (mx)
      sprintf (namlog, "%02d%03d.%02d%02d-%02d%02d.xm",
               H.year, H.doy, H.hour, H.minute, ref_hour, ref_min);
   else
      sprintf (namlog, "%02d%03d%02d%02d.xm",
               H.year, H.doy, H.hour, H.minute);

   if ((pgmlog = fopen (namlog, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namlog);
      exit (-1);
      }

   fprintf (pgmlog, "--- %s : SMM C/P coronal mass program ---\n", nampgm);

   if (mx  ==  0)
      fprintf (pgmlog, "\n*** absolute mass computation : %s ***\n\n", namcmp);
   if (mx  >  0)
      fprintf (pgmlog,
              "\n*** excess mass computation : %s - %s ***\n", namcmp, namref);
   fprintf (pgmlog, "--------------------------------------------------------------------------------\n");
   if (mx)
      {
      fprintf (pgmlog, "bkg : %s\n", hkref);
      }
   fprintf (pgmlog, "CME : %s\n", hkcmp);

   fprintf (pgmlog, "--------------------------------------------------------------------------------\n");


   if (H.ires == 0)		/* Change window image offsets for Hi res. */
      {
      xocp *= 2;
      yocp *= 2;
      }

   /*** Create GL window, define & activate cursor. ***/

   win_init (nampgm, H.ires, &xo, &yo, &xs, &ys, &xmaxscrn, &ymaxscrn);

   /*** For Excess mass, display image subtraction. ***/

   if (mx)
      cpsubimg (namcmp, namref, namimg);
   else
      strcpy (namimg, namcmp);

   /*** Create basis for polygon disk files. ***/

   idot = -1;
   for (i = 0; i < strlen (namimg); i++)  if (namimg [i] == '.')  idot = i;
   if (idot > 0)
      strncpy (namout, namimg,  idot);
   else
      strcpy  (namout, namimg);

   sprintf (namsrf, "%s.srf", namout);

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s in GL window\n", namimg);
   if ((ierr = glcp_view2 (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "glcp_view error: %d\n", ierr);
      exit (-4);
      }
   fprintf (stdout, "xwin ywin  xcp  ycp    r      th\n");

   /*** Initialize WHIP software. ***/

   fb_open ();
   openpl ();
   fb_setfb (FB);
   fb_clear (FB, &bkgcol, fast);

   /*** Invoke function to transfer image into frame buffer FB. ***/

   fprintf (stdout, "Displaying image: %s in FB #%d\n", namimg, FB);
   if ((ierr = cp_view (namimg, FB, linit)) < 0)
      {
      fprintf (stderr, "cp_view error: %d\n", ierr);
      exit (-4);
      }
   fb_touch ();
   if (mx) unlink (namimg);

   /*** Overlay set-up. ***/

   ov_init ();

   strcpy (afont, font1);
   afh = gl_setfont (afont, asize);	/* Small font & size. */

next:
   ierr = 0;
   fprintf (stderr, "Keyboard input to define  wedge   [k]\n");
   fprintf (stderr, "Mouse cursor   to define  wedge   [c]\n");
   fprintf (stderr, "Mouse cursor   to define  polygon [p]\n");
   fprintf (stderr, "Read disk file to define  polygon [f]\n");
   fprintf (stderr, "Quit program                      [q]\n");
   fprintf (stderr, "Type appropriate key to initiate action: ");

   fgets (line, maxline, stdin);
   sscanf (line, "%s", answer);
   action = (int) answer [0];

   switch (action)
      {
      case 'k':
                gl_wclear (ov_black);
                kb_wedge (xocp, yocp, &xo, &yo, &xs, &ys,
                          xmaxscrn, ymaxscrn, poly, &nver);
                fprintf (pgmlog,
                   "keyboard-selected wedge used to define mass region.\n");
                break;
      case 'c':
                gl_wclear (ov_black);
                mouse_wedge (xocp, yocp, &xo, &yo, &xs, &ys,
                             xmaxscrn, ymaxscrn, poly, &nver);
                fprintf (pgmlog,
                   "mouse-selected wedge used to define mass region.\n");
                break;
      case 'p':
                gl_wclear (ov_black);
                ierr = mouse_poly (xocp, yocp, &xo, &yo, &xs, &ys,
                                   xmaxscrn, ymaxscrn, poly, &nver);
                fprintf (pgmlog,
                        "mouse-selected polygon used to define mass region.\n");
                break;
      case 'f':
                fprintf (stderr, "Enter name of polygon vertex file: ");
                fscanf  (stdin,  "%s", namepv);
                fflush  (stderr);
                fgets (line, maxline, stdin);
                ierr = file_poly (namepv, poly, &nver);
                fprintf (pgmlog,
                   "file %s polygon used to define mass region.\n", namepv);
                break;
      case 'q':
      quit    :
                drawmode (OVERDRAW);	/* Clean up overlay bitplanes. */
                gl_wclear (ov_black);

                drawmode (NORMALDRAW);
                onemap ();	      /* Set single-map mode for color table. */
                gconfig ();

                /* Write sun rasterfile. */

/*
 *                fbsrfs (FB, namsrf, 512, 512);
 */

                closepl ();
                fb_close ();
                gexit ();
                exit (0);
      default:
                fprintf (stderr, "Invalid action.  Try again.\n");
                goto next;
      }

   if (ierr != 0)  goto quit;

   regnum++;

   /*** Polygon fill. ***/

   wpos = polyfill (namout, xocp, yocp, poly, nver, color3, &nwp, ierr);
   if (wpos == NULL)
      {
      fprintf (stderr, "polyfill error: %d\n", ierr);
      goto next;
      }

   if (answer [0] == 'p' || answer [0] == 'f')
      {
      sprintf (namreg, "%s.pv%d", namout, regnum);
      fprintf (pgmlog, "Polygon vertices stored in file: %s.pv%d\n",
               namout, regnum);
      if (ovfcnt < 5)
         {
         gl_wstr (namreg, 3 + ovfcnt*130, 45, cp_grey);
         wstr    (namreg, 3 + ovfcnt*126, 52, 1, cp_grey);
         }
      ovfcnt++;
      }


   /*** Mass integration. ***/

   cpmassp (namref, namcmp, mx, cbth, pgmlog, wpos, nwp);

   /*** Release storage for interior pixel coordinates of polygon. ***/

   free (wpos);

   goto next;
}
