/* cpwo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * 10 October 1991: add ability to read cursor position.
 * 23 October 1991: read mouse position only when left button is pressed.
 * 29 October 1991: write all information into output file.
 * 6 November 1991: fill polygon region using WHIP region tools.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/gl/img_scale.h"
#include "/usr/people/smm/lib/src/whip/cp_color.h"
#include "/usr/people/smm/lib/src/whip/ov_color.h"
#include "cp_mass.h"

#include <stdio.h>
#include <whip.h>
#include <fmclient.h>
#include <gl/gl.h>
#include <gl/device.h>
#define maxline 84
int DSP = 1;
int	ref_hour, ref_min;	/* Reference hour & minute. */
FILE *pgmlog;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   extern short *polyfill ();		/* Polygon fill function. */
   extern int glcp_view ();		/* Image display function. */
   extern char *rindex ();		/* Index function. */

   static char answer [2];		/* Response to scaling question. */
   static char line [maxline];		/* Array to store keyboard input. */
   static char namcmp [80];		/* Comparison image file name. */
   static char namref [80] = "none";	/* Reference image file name. */
   static char nampgm [80];		/* Program file name. */
   static char namlog [80];		/* Output file name. */
   static char namopt [80];		/* Option argument. */
   static char namepv [80];		/* Polynomial vertex file name. */

   char *it;				/* File name pointer. */
   char *parg;				/* Argument pointer. */

   static int poly [100][2];		/* Polynomial vertices. */
   short *wpos;				/* Pixel array pointer. */
   int FB = 0;
   int action;			/* action to be taken. */
   int fast = 1;		/* Parameter for fb_clear. */
   int i;			/* Index. */
   int ierr;			/* Error status from cpview(). */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int	mx;			/* Mass: =0 Absolute; else, Excess CMP - REF. */
   int nver;			/* # coordinate pairs in array poly. */
   int nwp;			/* # coordinate pairs inside polygon. */
   int polynum = 0;		/* polygon index.*/
   int region = 0;			/* Region index number. */
   int xocp = 63, yocp = 510;	/* C/P image origin in window coordinates. */
   int xo, yo;			/* x/y origin of current window */
   int xs, ys;			/* x/y size of current window   */
   int xmaxscrn, ymaxscrn;	/* maximum size of screen in x and y       */

   float bkgcol = 0.0;	/* Grey level for fb_clear. */
   float cbth = 0.0;	/* Coronal brightness threshold. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   i = 1;
   igr = 1;			/* Display option on by default. */
   lcc  = 0;			/* Munro correction off by default. */
   lstr = 0;
   lvig = 0;
   lfc  = 0;
   lsv  = 0;
   mx = 0;
   
   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-cd] [namref] [namcmp]\n", nampgm);

      fprintf (stderr,
              "Do you wish to apply Munro calibration correction [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lcc = 1;  else lcc = 0;

      fprintf (stderr, "Do you wish to display image [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  igr = 1;  else igr = 0;

      fprintf (stderr,
             "Enter reference C/P image file name ['none' if non-existent]:  ");
      fgets (line, maxline, stdin);
      sscanf  (line, "%s", namref);

      fprintf (stderr, "Enter comparison C/P image file name:  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", namcmp);
      }
   else
   if (argc == 2)	/* One argument: must be namcmp. */
      {
      igr = 1;
      mx = 0;
      strcpy (namcmp, argv [1]);
      }
   else
   if (argc == 3)	/* Two arguments: namref & namcmp or -d & namcmp. */
      {
      strcpy (namref, argv [1]);
      if (namref [0] == '-')
         {
         if (namref [1] == 'c' || namref [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namref [1] == 'd' || namref [2] == 'd')  igr = 0;  else igr = 1;
         strcpy (namref, "none");
         mx = 0;
         }
      else
         mx = 1;
      strcpy (namcmp, argv [2]);
      printf ("argc=3   namref: %s   namcmp: %s   mx: %d\n", namref, namcmp, mx);
      }
   else
   if (argc == 4)	/* Three arguments: -cd namref namcmp. */
      {
      strcpy (namopt, argv [1]);
      if (namopt [0] == '-')
         {
         if (namopt [1] == 'c' || namopt [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namopt [1] == 'd' || namopt [2] == 'd')  igr = 0;  else igr = 1;
         }
      strcpy (namref, argv [2]);
      if (strncmp (namref, "none", 4) == 0)  mx = 0;  else mx = 1;
      strcpy (namcmp, argv [3]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-cd] [namref] [namcmp]\n", nampgm);
      exit (-1);
      }

   /*** Decode SMM C/P housekeeping. ***/

   if (strncmp (namref, "none", 4) == 0)  mx = 0;  else mx = 1;

   printf ("namref: %s   namcmp: %s   mx: %d\n", namref, namcmp, mx);

   if (mx)
      {
      cphskp (namref);
      ref_hour = H.hour;
      ref_min  = H.minute;
      }

   cphskp (namcmp);

   if (H.ires == 0)		/* Change window image offsets for Hi res. */
      {
      xocp *= 2;
      yocp *= 2;
      }

   /*** Open log file. ***/

   if (mx)
      sprintf (namlog, "%02d%03d.%02d%02d-%02d%02d.xm",
               H.year, H.doy, H.hour, H.minute, ref_hour, ref_min);
   else
      sprintf (namlog, "%02d%03d%02d%02d.xm",
               H.year, H.doy, H.hour, H.minute);

   if ((pgmlog = fopen (namlog, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namlog);
      exit (-1);
      }

   fprintf (pgmlog, "--- %s : SMM C/P coronal mass program ---\n", nampgm);

   if (mx  ==  0)
      fprintf (pgmlog, "\n*** absolute mass computation : %s ***\n\n", namcmp);
   if (mx  >  0)
      fprintf (pgmlog,
              "\n*** excess mass computation : %s - %s ***\n", namcmp, namref);
   fprintf (pgmlog, "--------------------------------------------------------------------------------\n");
   if (mx) 
      {
      fprintf (pgmlog, "bkg   : ");
      hkline (pgmlog);
      }
   fprintf (pgmlog, "event : ");
   hkline (pgmlog);
   fprintf (pgmlog, "--------------------------------------------------------------------------------\n");

   /*** Create GL window, define & activate cursor. ***/

   win_init (nampgm, H.ires, &xo, &yo, &xs, &ys, &xmaxscrn, &ymaxscrn);

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namcmp);
   if ((ierr = glcp_view (namcmp, linit)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      exit (-4);
      }
   fprintf (stdout, "xwin ywin  xcp  ycp    r      th\n");

   /*** Initialize WHIP software. ***/

   fb_open ();
   openpl ();
   fb_setfb (FB);
   fb_clear (FB, &bkgcol, fast);

   /*** Invoke function to transfer image into frame buffer FB. ***/

   fprintf (stdout, "Displaying image: %s\n", namcmp);
   if ((ierr = cp_view (namcmp, FB, linit)) < 0)
      {
      fprintf (stderr, "cp_view error: %d\n", ierr);
      exit (-4);
      }
   fb_touch ();

   /*** Overlay set-up. ***/

   ov_init ();

next:
   ierr = 0;
   fprintf (stderr, "Keyboard input to define  wedge   [k]\n");
   fprintf (stderr, "Mouse cursor   to define  wedge   [c]\n");
   fprintf (stderr, "Mouse cursor   to define  polygon [p]\n");
   fprintf (stderr, "Read disk file to define  polygon [f]\n");
   fprintf (stderr, "Quit program                      [q]\n");
   fprintf (stderr, "Type appropriate key to initiate action: ");

   fgets (line, maxline, stdin);
   sscanf (line, "%s", answer);
   action = (int) answer [0];

   switch (action)
      {
      case 'k':
                gl_wclear (ov_black);
                kb_wedge (xocp, yocp, &xo, &yo, &xs, &ys,
                          xmaxscrn, ymaxscrn, poly, &nver);
                break;
      case 'c':
                gl_wclear (ov_black);
                mouse_wedge (xocp, yocp, &xo, &yo, &xs, &ys,
                             xmaxscrn, ymaxscrn, poly, &nver);
                break;
      case 'p':
                gl_wclear (ov_black);
                ierr = mouse_poly (xocp, yocp, &xo, &yo, &xs, &ys,
                                   xmaxscrn, ymaxscrn, poly, &nver);
                printf ("cpwo> nver: %d\n", nver);
                break;
      case 'f':
                fprintf (stderr, "Enter name of polygon vertex file: ");
                fscanf  (stdin,  "%s", namepv);
                fflush  (stderr);
                fgets (line, maxline, stdin);
                ierr = file_poly (namepv, poly, &nver);
                break;
      case 'q':
      quit    :
                drawmode (OVERDRAW);	/* Clean up overlay bitplanes. */
                gl_wclear (ov_black);

                drawmode (NORMALDRAW);
                onemap ();	      /* Set single-map mode for color table. */
                gconfig ();
                gexit ();
                closepl ();
                fb_close ();
                exit (0);
      default:
                fprintf (stderr, "Invalid action.  Try again.\n");
                goto next;
      }

   if (ierr != 0)  goto quit;

   region++;
   wpos = polyfill (nampgm, xocp, yocp, poly, nver, color4, &nwp, region);
   cpmassp (namref, namcmp, mx, cbth, pgmlog, wpos, nwp);
   free (wpos);

   goto next;
}
