/* dencp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *** Absolute/Excess image density program for SMM C/P images.
 * Computes a floating point density image from a C/P intensity image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Outputs a floating point image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 17 December 1986
 * 8 June 1990: conversion from F77 to C.
 * 20 June 1990: Housekeeping enhancements completed.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 */
#include <stdio.h>
#define nbymax 1792 
#include "/u/smm/lib/src/cp/cplib.h"
int lelec;

main ()
{
extern float *exden ();
float *imgout;

static char type[4];
static char namref [40];
static char namcmp [40];
static char namimg [40] = "dencp.den";
static char title [40];
static char typtit [12];
static char dtitle [] = "Density";
static char ititle [] = "Intensity";
static char buf [nbymax];

int i;
int idot, lastch;
int iryear, irdoy, irhour, irmin, irsec;
int luncmp, lunimg;
int nbyimg;
int nx, ny;
float xmin, xmax;

int nlink, nread, nwrite, nclose;
/*. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   lcc = 1;
   lsv = 0;
   lfc = 0;
   lstr = 0;
   lvig = 0;
   ldif = 0;
   lelec = 0;
   lden = 0;

   fprintf (stderr, "Program dencp: Excess/Absolute Brightness/Density\n");
   fprintf (stderr, "Density [d] or Brightness [b] ? ");
   fscanf  (stdin,  "%s", type);
   if (strcmp (type, "d") == 0)  lelec = 1; else lelec = 0;

   fprintf (stderr, "Enter reference image file name (type 'none' to omit): ");
   fscanf  (stdin,  "%s", namref);
   if (strcmp (namref, "none") == 0)  ldif = 0; else ldif = 1;

   fprintf (stderr, "Enter comparison image file name: ");
   fscanf  (stdin, "%s", namcmp);

/*** Decode Housekeeping. ***/

   if (ldif == 1)
      {
      cphskp (namref);
      iryear = H.year;
      irdoy  = H.doy;
      irhour = H.hour;
      irmin  = H.minute;
      irsec  = H.second;
      }

/*** Convert intensity to columnar density (electrons per cm^2). ***/

   if (lelec == 1)
      fprintf (stderr, "Computing density . . . please wait.\n");
   else
      fprintf (stderr, "Computing intensity . . . please wait.\n");
      
   cphskp (namcmp);
   imgout = exden (namref, namcmp, &nx, &ny, &xmin, &xmax);

/*** Create output file names for density image and contour metacode. ***/

   if (lelec == 1)
      strcpy (typtit, dtitle);
   else
      strcpy (typtit, ititle);

   if (ldif == 0)
      sprintf (title, "%s 19%2d %03d %02d:02d$",
	       typtit, H.year, H.doy, H.hour, H.minute);
   if (ldif == 1)
      sprintf (title, "%s 19%2d %03d %02d:02d - %03d %02d:%02d$",
	       typtit, H.year, H.doy, H.hour, H.minute,
		       iryear, irdoy, irhour, irmin);

   lastch = 0;
   idot = 0;
   strcpy (namimg, namcmp);
   for (i=0; i<strlen(namcmp); i++)
      {
      if (namcmp [i] != ' ')  lastch = i;
      if (namcmp [i] == '.')  idot = i;
      }
   if (idot == 0)
      {
      idot = lastch+1;
      namimg [idot] = '.';
      }
   namimg [idot+1] = '\0';

   if (lelec == 1)
      strcat (namimg, "den");
   else
      strcat (namimg, "brt");
   fprintf (stdout, "Output image file: %s\n", namimg);

   nlink = unlink (namimg);

/*** Write image to disk file. ***/

   luncmp = open (namcmp, 0);

   lunimg = creat (namimg, 0644);
   nclose = close (lunimg);
   lunimg = open (namimg, 1);
   if (lunimg <= 0)
      {
      fprintf (stderr, "Invalid image file name: %s\n", lunimg);
      fprintf (stdout, "Invalid image file name: %s\n", lunimg);
      exit (-1);
      }

   nbyimg = nx * ny * sizeof (float);

/*** Copy housekeeping & bitmap. ***/

   for (i=0; i<nbymax; i++)  buf[i] = '\0';
   nread = read (luncmp, buf, npix*2);	/* Read Housekeeping. */
   buf [103] = 32;			/* Floating point image. */
   buf [110] = (nx & 0177400) / 256;	/* # pixels per line. */
   buf [111] = (nx &   0377);

   buf [220] = 0;			/* photometric scaling exponent = 0. */
   if (lelec)  buf [221] = 8;		/* Set electron density flag. */
   if (ldif)  buf [221] = 16;		/* Set delta B flag. */
   buf [222] = 0;
   buf [223] = buf [223] & 004;		/* Retain disk registration bit. */
   buf [223] = (buf [223] | lstr | lvig*2 | lfc*8);

   if (ldif == 1)
      {
      buf [224] = iryear;
      buf [225] = (irdoy & 0177400)/256;
      buf [226] = (irdoy &   0377);
      buf [227] = irhour;
      buf [228] = irmin;
      buf [229] = irsec;
      }

   nwrite = write (lunimg, buf, npix*2);	/* Write Housekeeping. */
   nread  = read  (luncmp, buf, npix*2);	/* Read Bit-map. */
   nwrite = write (lunimg, buf, npix*2);	/* Write Bit-map. */

/*** Write image data to disk file. ***/

   nwrite = write (lunimg, imgout, nbyimg);
   nclose = close (lunimg);
}
