/* ebsun.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Returns # electrons per 1.0 B/Bsun brightness in solar corona.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 11 July 1990
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
double
ebsun (r)

float r;		/* radius in units of Rsun (mean solar radius). */
{
int irad;
double ebval;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * # electrons per square cm per B/Bsun brightness in solar corona 
 * as a function of radius (1.25 -> 8.5 Rsun in increments of 0.25 Rsun.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
static double etab [30] =
   {           8.917e+24, 1.448e+25, 2.100e+25,
    2.851e+25, 3.702e+25, 4.653e+25, 5.705e+25, 
    6.857e+25, 8.109e+25, 9.461e+25, 1.091e+26,
    1.247e+26, 1.412e+26, 1.587e+26, 1.772e+26,
    1.968e+26, 2.173e+26, 2.389e+26, 2.614e+26,
    2.849e+26, 3.095e+26, 3.350e+26, 3.616e+26,
    3.891e+26, 4.177e+26, 4.472e+26, 4.778e+26,
    5.093e+26, 5.419e+26, 5.754e+26};

   if (r <= 1.25)  return (0.0);
   irad = (int) ((r - 1.25) / 0.25);
   ebval = ( etab [irad] + ( ( r - irad*0.25 - 1.25 ) / 0.25 )
         * ( etab [irad+1] - etab [irad] ) );
   return (ebval);
}
