/* jack2.c
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Program to calculate total brightness, the radial and tangential
 * components of brightness, polarization and pB for lines of sight at
 * specific radii for angles zero to fifty degrees.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 */
#include <stdio.h>
#include <math.h>
#define nrad 71

double log(), pow(), cos(), sqrt();
float the[nrad], bt[nrad], br[nrad], b[nrad], pol[nrad];
float pb[nrad], pol1[nrad], b1[nrad];
float rad[nrad], x[nrad], ebsun[nrad], bt1[nrad];
float pb1[nrad], br1[nrad];
main()
{
float dt, u, pi, degrad, conval, rho2;
float t, r, s2, c2, s, c, g;
float ar, cr, bf, dr;
float tt, tr, q, w, ten;
int nang;
int i, l;
FILE *lun;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
/*
 * Set up constants for delta angle, number of angles, limb darkening
 * coefficient u, pi, degree to radian conversion and pi*sigma/(2*(1.-u/3))
 */
   dt = 2.0;
   nang = 1;
   u = 0.63;
   pi = acos(-1.0);
   degrad = pi / 180.0;
   conval = 1.2478e-25 / (1.0 - u / 3.);
/*
 * Open output file.
 */
   lun = fopen ("jack2.dat", "w");
   fprintf (lun,"    R/Rsun    B/Bsun     Elec/Bsun        pol        pB\n");

/*
 * Read the number of radii to be done and those radii.
 */
   x[0] = 1.5;
   for (i=1; i<nrad; i++) x[i] = x[i-1] + 0.1;
/*
 * Do calculation for each radius.
 */
   for (l = 0; l<nrad; l++)
      {
      rho2 = (float) pow ((double) x[l], (double) 2.);
/*
 * Initialize angle.
 */
      t = 0.0;
/*
 * Do calculation for each angle
 */
      for (i=0; i<nang; i++)
         {
/*
 * Calculate actual distance.
 */
         r = x[l] / (float) cos ((double) (t*degrad));
         s = 1.0 / r;
         s2 = s * s;
         c2 = 1.0 - s2;
         rad[i] = r;
         c = (float) sqrt ((double) c2);
         g = (float) log ((double) ((1.0 + s) / c));
/*
 * Calculate coefficients.
 */
         ar = c * s2;
         cr = 4.0 / 3.0 - c - (c * c2 / 3.0);
         bf = -(1.0 - 3.0 * s2 - c2 * (1.0 + 3.0 * s2) * g / s) / 8.0;
         dr = (5.0 + s2 - c2 * (5.0 - s2) * g / s) / 8.0;
/*
 * Calculate b(tangential), b(radial), angle, pb, total brightness,
 *    and polarization normalized.
 */
         tt = (1.0 - u) * cr + u * dr;
         tr = rho2 * ((1.0 - u) * ar + u * bf) / (r * r);
         q = tt - tr;
         w = tt + q;
         if (t <= 0.)  ten = w;
         ten = w;
         the[i] = t;
        
         bt[i] = tt / ten;
         pb[i] = tr / ten;
         br[i] = q / ten;
         b[i] = w / ten;
         pol[i] = tr / w;
/*
 * Calculate the above un-normalized.
 */
         tt = conval * tt;
         tr = conval * tr;
         q = tt - tr;
         w = tt + q;
         bt1[i] = tt;
         br1[i] = q;
         pb1[i] = tr;
         b1[i] = w;
         ebsun[i] = 1.0 / w;
         pol1[i] = tr / w;
         t = t + dt;

         fprintf (lun, "%8.2f   %10.3e   %10.3e   %9.4f   %10.3e\n",
                  rad[i], b1[i], ebsun[i], pol1[i], pb1[i]);
         }

      }

   fclose(lun);
}
