/* jack3.c
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Program to calculate total brightness, the radial and tangential
 * components of brightness, polarization and pB for lines of sight at
 * specific radii for angles zero to fifty degrees.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Original program written by Richard Munro
 * C conversion written by Andrew L. Stanger 21 Nov 86.
 * 7 June 1990
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 */
#include <stdio.h>
#include <math.h>
#define nang 2 
#define nrad 15

double log(), pow(), cos(), sqrt();
static float phi [nang], bt [nang], br [nang], b [nang], pol [nang];
static float pb [nang], pol1 [nang], b1 [nang];
static float rad [nang], x [nrad], ebsun [nang], bt1 [nang];
static float pb1 [nang], br1 [nang];
main()
{
float pstart = 0.0;
float ph, dp;
float u, pi, degrad, conval, rho2;
float r, s2, c2, s, c, g;
float ar, cr, bf, dr;
float tt, tr, q, w, ten;
int ip, ir;
FILE *lun;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
/*
 * Set up constants for delta angle, number of angles, limb darkening
 * coefficient u, pi, degree to radian conversion and pi*sigma/(2*(1.-u/3))
 */
   dp = 45.0;			/* Delta Phi. */
   u = 0.63;
   pi = (float) acos (-1.0);
   degrad = pi / 180.0;
   conval = 1.2478e-25 / (1.0 - u / 3.);
/*
 * Open output file.
 */
   lun = fopen ("jack3.dat", "w");
   fprintf (lun, "jack3: compute r, B, e/Bsun, pol, pB\n\n");
/*
 * Compute radii.
 */
   x[0] = 1.5;
   for (ir=1; ir<nrad; ir++)  x [ir] = x [ir-1] + 0.5;
/*
 * Do calculation for each radius.
 */
   for (ir = 0; ir<nrad; ir++)
      {
      fprintf (lun, "     ---------- Impact parameter: %6.2f R/Rsun ----------\n",
	       x [ir]);
      fprintf (lun," phi (deg)   r (R/Rsun)   B (B/Bsun)   Elec/Bsun        pol    pB (B/Bsun)\n");
      rho2 = (float) pow ((double) x [ir], (double) 2.0);
/*
 * Do calculation for each angle
 */
      for (ip=0; ip<nang; ip++)
         {
         ph = pstart + (float) ip * dp; 
/*
 * Calculate actual distance.
 */
         r = x [ir] / (float) cos ((double) (ph * degrad));
         s = 1.0 / r;
         s2 = s * s;
         c2 = 1.0 - s2;
         rad [ip] = r;
         c = (float) sqrt ((double) c2);
         g = (float) log ((double) ((1.0 + s) / c));
/*
 * Calculate coefficients.
 */
         ar = c * s2;
         cr = 4.0 / 3.0 - c - (c * c2 / 3.0);
         bf = -(1.0 - 3.0 * s2 - c2 * (1.0 + 3.0 * s2) * g / s) / 8.0;
         dr = (5.0 + s2 - c2 * (5.0 - s2) * g / s) / 8.0;
/*
 * Calculate b(tangential), b(radial), angle, pb, total brightness,
 *    and polarization normalized.
 */
         tt = (1.0 - u) * cr + u * dr;
         tr = rho2 * ((1.0 - u) * ar + u * bf) / (r * r);
         q = tt - tr;
         w = tt + q;
         if (ph <= 0.0)  ten = w;
         ten = w;
         phi [ip] = ph;
        
         bt  [ip] = tt / ten;
         pb  [ip] = tr / ten;
         br  [ip] = q / ten;
         b   [ip] = w / ten;
         pol [ip] = tr / w;
/*
 * Calculate the above un-normalized.
 */
         tt = conval * tt;
         tr = conval * tr;
         q = tt - tr;
         w = tt + q;
         bt1   [ip] = tt;
         br1   [ip] = q;
         pb1   [ip] = tr;
         b1    [ip] = w;
         ebsun [ip] = 1.0 / w;
         pol1  [ip] = tr / w;

         fprintf (lun, "   %6.2f      %6.3f    %10.3e   %10.3e   %9.4f   %10.3e\n",
                  phi [ip], rad [ip], b1 [ip], ebsun [ip], pol1 [ip], pb1 [ip]);
         }
      fprintf (lun, "\n");
      }
   fclose(lun);
}
