/* jack44pb.c
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Program to calculate total brightness, the radial and tangential
 * components of brightness, polarization and pB for lines of sight at
 * specific radii for angles zero to fifty degrees.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Original program written by Richard Munro
 * C conversion written by Andrew L. Stanger 21 Nov 86.
 * 7 June 1990
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 */
#include <stdio.h>
#include <math.h>
#define nang 1 
#define nrad 101

double log(), pow(), cos(), sqrt();
static double phi [nang], bt [nang], br [nang], b [nang], pol [nang];
static double pb [nang], pol1 [nang], b1 [nang];
static double rad [nang], x [nrad], ebsun [nang], bt1 [nang];
static double pb1 [nang], br1 [nang];
main()
{
float pstart = 0.0;
float ph, dp;
float u, pi, degrad, conval, rho2;
float r, s2, c2, s, c, g;
float ar, cr, bf, dr;
float tt, tr, q, w, ten;
int ip, ir;
FILE *lun;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/
   fprintf (stderr, "Enter phi angle: ");
   fscanf  (stdin, "%f%", &pstart);
/*
 * Set up constants for delta angle, number of angles, limb darkening
 * coefficient u, pi, degree to radian conversion and pi*sigma/(2*(1.-u/3))
 */
   dp = 1.0;			/* Delta Phi. */
   u = 0.63;
   pi = (float) acos (-1.0);
   degrad = pi / 180.0;
   conval = 1.2478e-25 / (1.0 - u / 3.);
/*
 * Open output file.
 */
   lun = fopen ("jack44pb.dat", "w");
   fprintf (lun, "jack44pb: compute r, B, e/Bsun, pol, pB\n\n");
/*
   fprintf (lun,"phi (deg)   x (Rsun)   r (Rsun)      B (Bsun)  Ne/Bsun-cm2      pol    pB (Bsun)\n\n");
 */
   fprintf (lun,"pB/Bsun-cm2\n\n") ;
/*
 * Compute radii.
 */
   x[0] = 1.0001;
   for (ir=1; ir<nrad; ir++)  x [ir] = x [ir-1] + 0.10;
/*
 * Do calculation for each radius.
 */
   for (ir = 0; ir<nrad; ir++)
      {
      rho2 = (float) pow ((double) x [ir], (double) 2.0);
/*
 * Do calculation for each angle
 */
      for (ip=0; ip<nang; ip++)
         {
         ph = pstart + (float) ip * dp; 
/*
 * Calculate actual distance.
 */
         r = x [ir] / (float) cos ((double) (ph * degrad));
         s = 1.0 / r;
         s2 = s * s;
         c2 = 1.0 - s2;
         rad [ip] = r;
         c = (float) sqrt ((double) c2);
         g = (float) log ((double) ((1.0 + s) / c));
/*
 * Calculate coefficients.
 */
         ar = c * s2;
         cr = 4.0 / 3.0 - c - (c * c2 / 3.0);
         bf = -(1.0 - 3.0 * s2 - c2 * (1.0 + 3.0 * s2) * g / s) / 8.0;
         dr = (5.0 + s2 - c2 * (5.0 - s2) * g / s) / 8.0;
/*
 * Calculate b(tangential), b(radial), angle, pb, total brightness,
 *    and polarization normalized.
 */
         tt = (1.0 - u) * cr + u * dr;
         tr = rho2 * ((1.0 - u) * ar + u * bf) / (r * r);
         q = tt - tr;
         w = tt + q;
         if (ph <= 0.0)  ten = w;
         ten = w;
         phi [ip] = ph;
        
         bt  [ip] = tt / ten;
         pb  [ip] = tr / ten;
         br  [ip] = q / ten;
         b   [ip] = w / ten;
         pol [ip] = tr / w;
/*
 * Calculate the above un-normalized.
 */
         tt = conval * tt;
         tr = conval * tr;
         q = tt - tr;
         w = tt + q;
         bt1   [ip] = tt;
         br1   [ip] = q;
         pb1   [ip] = tr;
         b1    [ip] = w;
         ebsun [ip] = 1.0 / w;
         pol1  [ip] = tr / w;

/*
         fprintf (lun, "   %6.2f     %6.3f     %6.3f    %10.3e   %10.3e   %6.3f   %10.3e\n",
                  phi [ip], x [ir], rad [ip], b1 [ip], ebsun [ip], pol1 [ip], pb1 [ip]);
 */
         fprintf (lun, "%10.3e\n", 1.0 / pb1 [ip]) ;
         }
/*
 *      fprintf (lun, "\n");
 */
      }
   fclose (lun);
}
