c * imgden.f
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c * Convert image intensity to columnar density .
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c * Solar Maximum Mission   Coronagraph/Polarimeter
c * Use ATM/WLC F-corona function.
c * Compute electrons/cm^2 (instead of elec/pixel);
c * 25 September 1986
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      subroutine imgden (image, bk, nx, ny, bmin, bmax)
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c * Solar radius (cm).
      parameter (cmsun = 6.96e+10)
      parameter (npmax = 896, ncmax = 112)
c
      common /center/ ixcen, iycen
      common /const/  pi, raddeg, degrad, pixr, rpix, rsun
      common /hskdat/ ires, ip, ir, lp, lr, ifw, ism, ipw
      common /size/   npix, npix4, nbypix, idtype
      common /sun/    degnor, kcen, lcen
      common /cor/ lvs, lfc
c
c * Image array.
      integer image(npmax,npmax)

c * Output density array.
      real bk(112,112)

c * Function which applies calibration correction.
      external calcor

c * Function which removes stray light.
      external stray

c * Function which removes vignetting.
      external vignet

c * Function which removes F-corona.
      external fcor
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      real bmin, bmax
      real r, th
      real cc
      integer ix, iy
      integer ierr
      integer irad
      integer nx, ny

c * Stray light & vignetting values.
      real str, vig

c * area per pixel.
      real cm2pix

c * # pixels per cm^2.
      real pixcm2

c * # electrons / 1.e-11 B/Bsun.
      real ebsun
      real elec(21)

c * coronal brightness.
      real cbrite

c * Scaling factor for calibrated intensities.
      data Cexp /1.e-11/

c * Inverse scaling factor.
      data Cinv /1.e+11/

c * Mass per light-scattering electron.
      data emass /2.0e-24/

c * # electrons per 1.e-11 B/Bsun brightness at 0.25 Rsun intervals in corona.

      data elec
     .         /1.79e14,1.79e14,2.35e14,2.85e14,3.70e14,4.65e14,5.70e14,
     .          6.86e14,8.10e14,9.34e14,1.09e15,1.25e15,1.42e15,1.59e15,
     .          1.78e15,1.97e15,2.18e15,2.39e15,2.62e15,2.85e15,2.85e15/
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

c * Initialize stray light & vignetting variables.

      vig = 1.
      str = 0.
      cc = 1.
      bmin = 1.e+30
      bmax = 0.
      kcen = ixcen / ires
      lcen = iycen / ires

c * Compute the cross-sectional area (square cm) for one pixel.

      cm2pix = (cmsun*rpix) * (cmsun*rpix)
      pixcm2 = 1. / cm2pix
      inc = npix / 112

c * ....... Image computation loop .......

      iiy = 113
      do 200 iy = 1, npix, inc
      iiy = iiy - 1
      iix = 0
c
      do 100 ix = 1, npix, inc
      iix = iix + 1
 
c * Coordinate transformation :  (pixel,line) --> (theta,angle).

      call coord(r,th,ix,iy,-1,ierr)

      if(ierr .ne. 0)  then
         write (*,*) 'imgden> coord error:', ierr
         return
      endif

c * Use external stray/vignet functions.

      if(lvs .eq. 1)  then
         vig = vignet(r)
         str = stray(r)
	 cc = calcor (r)
      endif
      irad = ((r - 1.25)/0.25)
      ebsun = ( elec(irad) + ( (r - irad*0.25 - 1.25) / 0.25)
     .       * (elec(irad+1) - elec(irad)) )
c
c--- Average image over 3x3 pixel box.
c
      cbrite = 0.
      do 90 j = iy, iy+2
         do 80 i = ix, ix+2
            cbrite = cbrite + (image(i,j) * cc - str) * vig
   80    continue
   90 continue
      cbrite = cbrite / 9.
c
      if (lfc .eq. 1)  cbrite = cbrite - fcor(r) * Cinv
      if (cbrite .lt. 0.)  cbrite = 0.
      bk(iix,iiy) = ebsun * cbrite
      if (bmin .gt. bk(iix,iiy))  bmin = bk(iix,iiy)
      if (bmax .lt. bk(iix,iiy))  bmax = bk(iix,iiy)
c      if (iix .eq. 30)  then
c         write (*,*) 'line:',iy,'  image:',image(ix,iy),
c     .               '  bk(30,iiy):',bk(30,iiy)
c         write (*,*) 'r:',r,' str:',str,' vig:',vig
c         write (*,*) 'ebsun:',ebsun,' cbrite:',cbrite,' cc:',cc
c      endif
  100 continue
  200 continue
      nx = iix
      ny = iix
c
      return
      end
