      program munro
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c Program to calculate total brightness, the radial and tangential
c components of brightness, polarization and pB for lines of sight at
c specific radii for angles zero to fifty degrees.
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      parameter (nrad=71)
      real the(nrad), bt(nrad), br(nrad), b(nrad), pol(nrad),
     .       pb(nrad), pol1(nrad), b1(nrad)
      real rad(nrad), x(nrad), the1(nrad), bt1(nrad),
     .       pb1(nrad), br1(nrad)
      data lun/10/
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c
c--- Set up constants for delta angle, number of angles, limb darkening
c    coefficient u, pi, degree to radian conversion and pi*sigma/(2*(1.-u/3))
c
      dt = 2.0
      nang = 1
      u = 0.63
      pi = acos(-1.0)
      degrad = pi / 180.0
      const = 1.2478e-25 / (1.0 - u / 3.)
c
c--- Open output file.
c
      open (unit=lun,file='munro.dat',status='unknown')
      write (lun,1)
    1 format(/4x,'angle',5x,'b(tan)',3x,'b(rad)',6x,'b',7x,'pol',
     . 6x,'pb',5x,'distance',5x,'b(tan)',7x,'b(rad)',9x,'b',
     . 9x,'pol',9x,'pb')
c
c--- Read the number of radii to be done and those radii.
c
      x(1) = 1.5
      do 2 i = 2, nrad
         x(i) = x(i-1) + 0.1
    2 continue
c
c--- Do calculation for each radius.
c
      do 8 l = 1, nrad
c      write (lun,3)
c    3 format (///)
      rho2 = x(l)**2
c
c--- Initialize angle.
c
      t = 0.0
c
c--- Do calculation for each angle
c
      do 5 i = 1, nang
c
c--- Calculate actual distance.
c
      r = x(l) / cos(t*degrad)
      s = 1.0 / r
      s2 = s * s
      c2 = 1.0 - s2
      rad(i) = r
      c = sqrt(c2)
      g = alog((1.0 + s) / c)
c
c--- Calculate coefficients.
c
      ar = c * s2
      cr = 4.0 / 3.0 - c - (c * c2 / 3.0)
      bf = -(1.0 - 3.0 * s2 - c2 * (1.0 + 3.0 * s2) * g / s) / 8.0
      dr = (5.0 + s2 - c2 * (5.0 - s2) * g / s) / 8.0
c
c--- Calculate b(tangential), b(radial), angle, pb, total brightness,
c    and polarization normalized.
c
      tt = (1.0 - u) * cr + u * dr
      tr = rho2 * ((1.0 - u) * ar + u * bf) / (r * r)
      q = tt - tr
      w = tt + q
      if (t .gt. 0.0)  go to 4
      ten = w
    4 the(i) = t
      bt(i) = tt / ten
      pb(i) = tr / ten
      br(i) = q / ten
      b(i) = w / ten
      pol(i) = tr / w
c
c--- Calculate the above un-normalized.
c
      tt = const * tt
      tr = const * tr
      q = tt - tr
      w = tt + q
      bt1(i) = tt
      br1(i) = q
      pb1(i) = tr
      b1(i) = w
      pol1(i) = tr / w
      t = t + dt
    5 continue
c
      write (lun,7) (the(i),bt(i),br(i),b(i),pol(i),pb(i),
     .       rad(i),bt1(i),br1(i), b1(i),pol1(i),pb1(i),i=1,nang)
    7 format(1x,f10.5,4f9.5,f10.5,f10.2,5e12.3)
c
    8 continue
c
      close(lun)
      stop
      end
