c * xden.f
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c * Convert image intensity (absolute or excess) to columnar density .
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c * Solar Maximum Mission   Coronagraph/Polarimeter
c * Use ATM/WLC F-corona function.
c * Compute electrons/cm^2 or corrected brightness.
c * 14 November 1986
c * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
      subroutine xden (imgref, imgcmp, den, nx, ny, dmin, dmax)
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c * Solar radius (cm).
      parameter (cmsun = 6.96e+10)
      parameter (npmax = 896, ncmax = 112)
c
      common /center/ ixcen, iycen
      common /const/  pi, raddeg, degrad, pixr, rpix, rsun
      common /hskdat/ ires, ip, ir, lp, lr, ifw, ism, ipw
      common /size/   npix, npix4, nbypix, idtype
      common /sun/    degnor, kcen, lcen
      common /opt/    lsv, lfc, lden, ldif

c * Image array.
      integer*2 imgref(npmax,npmax)
      integer*2 imgcmp(npmax,npmax)

c * Output density array.
      real den(ncmax,ncmax)

c * Function which applies calibration correction.
      external calcor

c * Functions which remove stray light, vignetting & F-corona.
      external stray
      external vignet
      external fcor
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
      real dmin, dmax
      real r, th
      real cc
      integer ix, iy
      integer ierr
      integer irad
      integer nx, ny

c * Stray light & vignetting values.
      real str, vig

c * area per pixel.
      real cm2pix

c * # pixels per cm^2.
      real pixcm2

c * # electrons / 1.e-11 B/Bsun.
      real ebsun
      real elec(28)

c * coronal brightness.
      real rbrite
      real cbrite

c * Scaling factor for calibrated intensities.
      data Cexp /1.e-11/

c * Mass per light-scattering electron.
      data emass /2.0e-24/

c * # electrons per B/Bsun brightness at 0.25 Rsun intervals in corona.

      data elec
     .         /8.93e24,1.45e25,2.10e25,2.85e25,3.70e25,4.65e25,5.71e25,
     .          6.86e25,8.11e25,9.46e25,1.09e26,1.25e26,1.41e26,1.59e26,
     .          1.77e26,1.97e26,2.17e26,2.39e26,2.61e26,2.85e26,3.10e26,
     .          3.35e26,3.62e26,3.89e26,4.18e26,4.47e26,4.78e26,5.09e26/
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c
c * Initialize variables.
c
      vig = 1.
      str = 0.
      cc = 1.
      ebsun = 1.
      dmin = 1.e+30
      dmax = -1.e+30
      kcen = ixcen / ires
      lcen = iycen / ires

      if (lsv .eq. 1) write (*,*) 'Stray light & vignetting removed.'

c * Compute the cross-sectional area (square cm) for one pixel.

      cm2pix = (cmsun*rpix) * (cmsun*rpix)

c * ....... Image computation loop .......

      inc = npix / ncmax 
      iiy = ncmax + 1
      do 200 iy = 1, npix, inc
      iiy = iiy - 1
      iix = 0
c
      do 100 ix = 1, npix, inc
      iix = iix + 1
 
c * Coordinate transformation :  (pixel,line) --> (theta,angle).

      call coord(r,th,ix,iy,-1,ierr)

      if(ierr .ne. 0)  then
         write (*,*) 'xden> coord error:', ierr
         return
      endif

c * Use external stray/vignet functions.

      if(lsv .eq. 1)  then
         vig = vignet(r)
         str = stray(r) * Cexp
	 cc = calcor (r) * Cexp
      endif

      if (lden .eq. 1)  then
         irad = ((r - 1.0)/0.25)
         ebsun = ( elec(irad) + ( (r - irad*0.25 - 1.0) / 0.25)
     .         * (elec(irad+1) - elec(irad)) )
      endif
c
c--- Average image over 'inc' X 'inc' sized box.
c
c--- Reference image.
c
      rbrite = 0.
      if (ldif .eq. 1)  then
         do 70 j = iy, iy+inc-1
            do 60 i = ix, ix+inc-1
               rbrite = rbrite + (imgref(i,j) * cc - str) * vig
   60       continue
   70    continue
      rbrite = rbrite / float (inc*inc)
      if (lfc .eq. 1)  rbrite = rbrite - fcor(r) * Cexp
      if (rbrite .lt. 0.)  rbrite = 0.
      endif
c
c--- Comparison image.
c
      cbrite = 0.
      do 90 j = iy, iy+inc-1
         do 80 i = ix, ix+inc-1
            cbrite = cbrite + (imgcmp(i,j) * cc - str) * vig
   80    continue
   90 continue
      cbrite = cbrite / float (inc*inc)
      if (lfc .eq. 1)  cbrite = cbrite - fcor(r) * Cexp
      if (cbrite .lt. 0.)  cbrite = 0.
c
      den(iix,iiy) = ebsun * (cbrite - rbrite)
      if (dmin .gt. den(iix,iiy))  dmin = den(iix,iiy)
      if (dmax .lt. den(iix,iiy))  dmax = den(iix,iiy)
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
c      if (iix .eq. 30)  then
c         write (*,*) 'line:',iy,'  image:',imgcmp(ix,iy),
c     .               '  den(30,iiy):',den(30,iiy)
c         write (*,*) 'r:',r,' str:',str,' vig:',vig
c         write (*,*) 'ebsun:',ebsun,' cbrite:',cbrite,' cc:',cc
c      endif
c . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
  100 continue
  200 continue
      nx = iix
      ny = iix
c
      return
      end
