/* dnadj88.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adjust DN (raw vidicon intensity) for predicted value at DOY 063.   *
 * The transfer curve for year 1988 is on DOY 063.                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	               *
 * High Altitude Observatory (HAO)			               *
 * National Center for Atmospheric Research (NCAR) 	               *
 * SGI 4D/25 G   UNIX IRIX 5.2   C programming language                *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Andrew L. Stanger				 	               *
 * 12 January 1996                                                     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dnadj88 (fdoy, dn, tcdn)

float fdoy ;			/* Image day-of-year (includes fraction). */
int dn ;			/* pixel DN intensity value in coronal image. */
int *tcdn ;			/* pixel intensity for transfer curve date. */

{
   /*** offset and slope of image average DN level for Cal.Mir. exposures.
    *** Corresponding Exposure durations:
    *** min,0,1,2,3,4,5,6,8,10,12,14,17,20,25,max
    ***/

   static float dnoff [15]   = {   0.0,  30.8,  55.5,  78.4,  99.1,
				 116.6, 130.9, 153.2, 167.8, 182.1,
				 193.2, 206.0, 214.4, 221.5, 256.0} ;

   static float dnslope [15] = {  0.0150, -0.0150, -0.0185, -0.0210, -0.0246,
				 -0.0280, -0.0269, -0.0217, -0.0157, -0.0143,
				 -0.0138, -0.0149, -0.0138, -0.0047, -0.0047} ;

   float fdn ;
   float dnf, dno, dns ;
   float tcdn1, tcdn2 ;
   float tcdoy = 63.5 ;		/*** DOY date of 1988 transfer curve. ***/

   int i, idn1, idn2 ;

   fdn = (float) dn ;

   if (dn <= 0 || fdoy >= 367.0)
      {
/*      printf ("dn: %3d\n", dn) ; */
      *tcdn = dn ;
      return (-1) ;
      }

   /*** Find two table entries between which the vidicon DN level falls. ***/

   idn1 = 0 ;
   idn2 = 1 ;
   for (i = 0; i < 15; i++)
      {
      tcdn1 = dnslope [i] * fdoy + dnoff [i] ;		/* calmir DN for fdoy.*/
      tcdn2 = dnslope [i+1] * fdoy + dnoff [i+1] ;
      if (fdn > tcdn1  && fdn < tcdn2)  idn1 = i ;
      }
   idn2 = idn1 + 1 ;

   /*** Interpolate to derive DN offset and slope for a given image date. ***/

   /*** Interpolate to determine DN slope and intercept. ***/
   /*** Use input DN and interpolate between two calmir curves (DN vs. DOY).***/

   dnf = (fdn - dnoff [idn1]) / (dnoff [idn2] - dnoff [idn1]) ;
   dno = dnoff [idn1] + dnf * (dnoff [idn2] - dnoff [idn1]) ;
   dns = dnslope [idn1] + dnf * (dnslope [idn2] - dnslope [idn1]) ;

/*
 *   dn1 = slope * t1 + off
 *   dn2 = slope * t2 + off
 *   ----------------------
 *   dn2 = dn1 + slope * (t2 - t1)
 */

   /*** Now, compute predicted value of DN level for the date
    *** of the calibration mirror transfer curve images.
    ***/

   *tcdn = (int) (fdn + dns * (tcdoy - fdoy)) ;

/*
   printf ("idn1/2: %2d %2d  dnf: %7.2f  dns: %7.4f  fdoy: %7.2f  dn: %3d  tcdn: %3d\n",
	    idn1, idn2, dnf, dns, fdoy, dn, *tcdn) ;
 */

   return (0) ;
}
