/*
    This does a simple linear least squares fit.
    There is a version for floats and one for doubles.

    Written by Alice Lecinski. (2000-2001)

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the 
    code.  So if you do edit it, insure results pre and post
    actually match.

*/
#include <stdio.h>
#include <math.h>

int
do_lst(a,b,R,xarry,yarry,nn,printme)
    float *a,*b,*R,*xarry,*yarry;
    int nn,printme;
{
    float sumx,sumy,sumx2,sumy2,sumxy,SXY,SXX,SYY,Num;
    float xlst,ylst,temp;
    float *xx , *yy ;
    double sqrt();
    int ii,diode;

    diode=printme;
    sumx=sumy=sumx2=sumy2=sumxy=SXY=SXX=SYY=Num=0.;

    Num = (float)nn;
    xx=xarry; yy=yarry;

    for(ii=0;ii<nn;ii++){
        xlst = *xx++;
        ylst = *yy++;
        sumx = sumx + xlst;            /* Fill up least squares sums */
        sumy = sumy + ylst;
        sumx2 = sumx2 + xlst * xlst;
        sumy2 = sumy2 + ylst * ylst;
        sumxy = sumxy + xlst * ylst;
    }

    SXY = Num * sumxy - sumx * sumy;
    SXX = Num * sumx2 - sumx * sumx;
    SYY = Num * sumy2 - sumy * sumy;
    /*   Then, in the equation y = a  + b*x,                             *
    *         a = sumy/Num - b*sumx/Num                                 *
    *         b = SXY/SXX,                                              *
    *         and the correlation coefficient is  SXY/sqrt(SXX*SYY).    */
    *b = SXY / SXX;
    *a = sumy/Num  - (*b)*sumx/Num;
    temp = SXX * SYY;                   if(temp<0.0) temp *= -1.0;
    temp = SXY / sqrt((double)temp);    if(temp<0.0) temp *= -1.0;
    *R = temp;

    if(printme!=0)
        fprintf(stderr,"\t\t\t%d, a=%.2e b=%.2e R=%.2f\n",diode,*a,*b,*R);
}

int
Ddo_lst(a,b,R,xarry,yarry,nn,printme)
    double *a,*b,*R,*xarry,*yarry;
    int nn,printme;
{
    double sumx,sumy,sumx2,sumy2,sumxy,SXY,SXX,SYY,Num;
    double xlst,ylst,temp;
    double *xx , *yy ;
    double sqrt();
    int ii,diode;

    diode=printme;
    sumx=sumy=sumx2=sumy2=sumxy=SXY=SXX=SYY=Num=0.;

    Num = (double)nn;
    xx=xarry; yy=yarry;

    for(ii=0;ii<nn;ii++){
        xlst = *xx++;
        ylst = *yy++;
        sumx = sumx + xlst;            /* Fill up least squares sums */
        sumy = sumy + ylst;
        sumx2 = sumx2 + xlst * xlst;
        sumy2 = sumy2 + ylst * ylst;
        sumxy = sumxy + xlst * ylst;
    }

    SXY = Num * sumxy - sumx * sumy;
    SXX = Num * sumx2 - sumx * sumx;
    SYY = Num * sumy2 - sumy * sumy;
    /*   Then, in the equation y = a  + b*x,                            *
    *         a = sumy/Num - b*sumx/Num                                 *
    *         b = SXY/SXX,                                              *
    *         and the correlation coefficient is  SXY/sqrt(SXX*SYY).    */
    *b = SXY / SXX;
    *a = sumy/Num  - (*b)*sumx/Num;
    temp = SXX * SYY;                   if(temp<0.0) temp *= -1.0;
    temp = SXY / sqrt((double)temp);    if(temp<0.0) temp *= -1.0;
    *R = temp;

    if(printme!=0)
        fprintf(stderr,"\t\t\t%d, a=%.2e b=%.2e R=%.2f\n",diode,*a,*b,*R);
}
