/************************************************************************
*                         W H I P . H                                   *
*                                                                       *
*           Copyright (C) 1984-90, GW Hannaway & Associates             *
*                                                                       *
*  These coded instructions, statements, and computer programs  contain *
*  unpublished  proprietary  information of  GW Hannaway & Assoc.,  and *
*  are protected by Federal copyright law.  They  may  not be disclosed *
*  to  third  parties  or copied or duplicated in any form, in whole or *
*  in part, without the prior written consent of GW Hannaway & Assoc.   *
*                                                                       *
*       Author: William R. Bishop      09/84                            *
*                                                                       *
************************************************************************/
/*
 * Tip - when using vi, set ts=4
*/

# include <fbconfig.h>

/*
 * Entries consist of 2 byte integer short words.
 * A variable length comments section may follow this header and
 * before the image data (lcom specifies length).
*/
struct pic_hdr 
{
        short   id; 	/* Image Identifier; BYTE, SHORT, etc	*/
        short   lcom; 	/* Length of comments before image data */
        short   type;	/* Type of pixel encoding; ROW_MODE, etc*/
        short   dims; 	/* Dimensions (default = 2) 			*/
        short   hpls;	/* hyperplanes (default = 1) 			*/
        short   plns;	/* Planes (default = 1) 				*/
        short   rows; 	/* Number of rows 						*/
        short   cols; 	/* Number of columns 					*/
        short   bp;   	/* Bits/pixel - must be multiple of 8	*/
        short   sbp;  	/* Significant bits/pixel				*/
        short   xstrt;	/* X starting address of image			*/
        short   ystrt;	/* Y starting address of image			*/
};
typedef struct pic_hdr pichdr;

/*
* Immediately following the header structure in
* oldest-to-newest-order, are ASCII title and comments 
* consisting of lines of less than 80 characters
* each ended by a newline (012).
* New comments would be added after the last comment 
* and the comment length field of the header would be incremented.
*/
/*
* The image data which follows is deemed to be in row mode if 
* the image data was acquired by columns, and proceeds right across
* the image.  Image data taken in column mode is assumed to be interlaced.
* Row mode is the common mode which rasters across the top row from left
* to right and then downward by rows.
* The type of acquisition is store in pic_hdr.type.
*/

/*
 * Definitions for different image formats for pic_hdr.type
*/
#define NO_MODE         -1
#define ROW_MODE        0
#define COL_MODE        1
#define RUNLEN          2
#define CONOGRAPH       3
#define VECTOR          4
#define SCANLINE		5

#define Z_MODE			6	/* Special value for 3D framebuffers */

/* 
 * Defines for pic_hdr.id (and framebuffer kind)
*/
#define	BIT			000011	/* Bit images treated as byte	*/
#define SIGNED_BYTE	000012	/* For those needing signed math*/
#define BYTE    	044520	/* 4950 hex, or "PI" in ASCII	*/
#define	SHORT		000001	/* 16 Bit signed data			*/
#define	LONG		000010	/* 32 bit signed data			*/
#define	FLOAT		000007	/* 32 bit floating point data	*/
#define	CMPLX		046503	/* 4D43 hex, Type "CM" (complex)*/
#define	RGB			000002	/* RGB data, 8 bits/component	*/
#define	RGBA		000003	/* RGBA data, 8 bits/component	*/
#define	ABGR		000004	/* ABGR data, 8 bits/component	*/
#define	RGBZ		000005	/* RGBZ data, 24 bits + 32bits	*/
#define	RGBAZ		000006	/* RGBAZ data, 32 bits + 32bits	*/

/* 
 * Local RAM structures, created and maintained by image(3) subroutines.
*/
struct str
{
	struct	str	*nx;
	char		*p;
};

struct image
{
	struct image *nx;			/* Struct maint, must be first */
	struct image *pr;

	struct	pic_hdr	hdr;		/* Image header			*/
	struct	str		*com[2];	/* Image Comments		*/
	short			type;		/* Type of image data	*/
	union
	{
		unsigned char	*cp;	/* Char (8 bit) data	*/
		short			*sp;	/* Short (16 bit) data	*/
		long			*lp;	/* Long (32 bit) data	*/
		float			*fp;	/* Float (32 bit) data	*/
	} d;
};

/*
 * Current Types of framebuffers. Note that some may or may not exist
 * in your system depending upon the configuration purchased.
*/
#define    FB_MONO	 0	/* Monochrome frame buffer		*/
#define    FB_RGB	 1	/* RGB frame buffer				*/
#define    FB_IRIS	 2	/* Iris frame buffer			*/
#define    FB_IRISGL 3	/* Iris in single buffer mode	*/
#define    FB_150	 4	/* 150 series frame buffer		*/
#define    FB_FILE	 5	/* Frame buffer is a data file 	*/
#define    FB_SHMEM	 6	/* Frame buffer is shared mem fb*/
#define    FB_RGBA	 7	/* Frame buffer is RGBA fb		*/
#define    FB_COMP	 8	/* Multi-component Framebuffer	*/
#define    FB_CMPLX	 9	/* Two component floating pt. fb*/
#define    FB_MEM	 10	/* Local memory RAM framebuffer	*/
#define    FB_PXM	 11	/* Pixel machine framebuffer	*/
#define    FB_3D     12 /* 3D Shared memory framebuffer	*/
#define    FB_SCSI   13 /* Raw SCSI framebuffer			*/

/*
 * Some global variables.  I should probably hide them and make
 * access to them via subroutine calls (sigh).
*/
extern int NUMALU;		/* Number of ALU's in system 			*/
extern int NUMFBS;		/* Number of frame buffers				*/
extern int NUMAPS;		/* Number of AP's						*/
extern int NUMRTC;		/* Number of RTC's						*/
extern int NUMHF;		/* Number of HF's						*/
extern int fb_xmax;		/* Max x of current fb					*/
extern int fb_ymax;		/* Max y of current fb					*/
extern int fb_zmax;		/* Max z of current fb					*/
extern int stepmode;	/* Rows/cols, etc						*/
extern char errbuf[];	/* Tmp buffer for error msg generation	*/

/*
 * For those crummy compilers that can't have more than 8 character
 * function names (Toy Computers -- GRRR).
*/
# ifdef IBM_AT
# define it_putap	ITPUTAP
# define it_putalu	ITPUTALU
# define it_getap	ITGETAP
# define it_getalu	ITGETALU
# define it_setap	ITSETAP
# define it_setalu	ITSETALU
# endif

# ifdef IT_150
# include <it150.h>
# endif

# ifdef PIXEL_MACHINE
# include <pxm.h>
# endif
extern  get_surface( char * file, struct surface * m );
extern void dump_surface( struct surface * surf, FILE * fp );
extern int irisview( char * filename, int xlower, int ylower, int xupper, int yupper );
extern int iristore( char * filname, int xlower, int ylower, int xupper, int yupper );
extern void docolor( FILE * colorfile );
extern  median( int fb, int size, int xlower, int ylower, int xupper, int yupper );
extern int hipas( float * indata, float * outdata, long s, long f0, int n, float * y0n, float gain );
extern  lopas( float * indata, float * outdata, float alf, int n, float * y0n );
extern int correlate( float * cor_image, int cor_size, float * ref_image, int ref_size, int * x, int * y, float * cor );
extern int cpfb( int fromfb, int tofb );
extern char * fb_resample( int fb, int xsize, int ysize );
extern int terminit( char * tname );
extern int curpos( short row, short col );
extern int cls();
extern int clend();
extern int msg( short row, short col, char * s );
extern  fb_getfb();
extern  fb_setfb( int fb );
extern char * data_xlate( int s1, char * d1, int s2, char * d2, int len, int offset, float xlate );
extern  fb_grate( int fb, int x_space, int x_width, int y_space, int y_width, float bkgnd, float inten );
extern  fb_scanrev( int fb, int mode );
extern int fb_fe( int fb, float * feature, char * basename, int min, int max );
extern  fb_ystep( int fb );
extern  resample( int tofb, int fromfb, int xlower, int ylower, int xupper, int yupper );
extern float bilinear( float p1, float p2, float p3, float p4, float fracx, float fracy );
extern  howmany( int fb, float low, float high, int * num );
extern  fconvolve( int fb, float * k, int N, int M );
extern  iconvolve( int fb, int * k, int N, int M );
extern  ldikernel( char * name, int * k, int * N, int * M );
extern  ldfkernel( char * name, float * k, int * N, int * M );
extern  fb_getsize( int fb, int * x, int * y, int * z );
extern  fb_getcomp( int fb, int comp );
extern  fb_getnumcomp( int fb );
extern  fb_isnamedfb( char * name );
extern char * fb_getname( int fb );
extern  fb_getnamedfb( char * name );
extern  fb_getminmax( int fb, float * minv, float * maxv );
extern char * malloc_scanline( int type, int num );
extern char * fb_malloc( char * name, int s, int n );
extern  fb_create( char * name, int type, int kind, int xsize, int ysize, int zsize );
extern  pictofb( char * name, int type );
extern  fbtopic( int fb, char * name );
extern  fb_attach( char * name, int type );
extern char * fb_pointer( int fb );
extern  fb_getaoi( int fb, int * lx, int * ux, int * ly, int * uy, int * lz, int * uz );
extern  fb_setaoi( int fb, int lx, int ux, int ly, int uy, int lz, int uz );
extern  fb_touch( int fb );
extern  fb_free( int fb );
extern int cffft( struct _complex * data, int n, int forward );
extern int tracer( int x, int y, int (*inside)(), void (*report)(), long max );
extern int applyfb( int fb, int (*func)(), float * args );
extern int applyfbs( int fb1, int fb2, int (*func)(), float * args );
extern  bytes_pixel( int fb );
extern  fb_bytes( int kind );
extern int find_kernel( char * namein, char * nameout );
extern char * getoken( char * str, char * token );
extern int find_picture( char * namein, char * nameout );
extern int find_gamma( char * namein, char * nameout );
extern void basename( char * in, char * out );
extern int putcol( int fb, int indx, unsigned char * ptr );
extern int putrow( int fb, int indx, unsigned char * ptr );
extern int getpix( int fb, int x, int y, unsigned char * pix );
extern void fb_putline( int rel, int cnt, int x, int y, char * v );
extern void help( char * progname, char * text, int errn, int estat );
extern int fb_open();
extern int fb_close();
extern int fb_getbuf( int fb, int * x, int * y );
extern int fb_setdata( int fb );
extern int fb_translate( int fb );
extern int fb_kind( int fb );
extern int fb_type( int fb );
extern int fb_part( int fb );
extern int fb_setaddr( int fb, int rel, int x, int y, int z );
extern int fb_write( unsigned char * buf, int cnt );
extern int fb_getpix( int rel, int x, int y, unsigned char * v );
extern int gename( char * name, char * new, char * suffix );
extern float * fb_gamgen( FILE * fp, int * n );
extern void prandom( int count, int new, int seed, char * array );
extern int picopen( char * name, int * hdr, int mode );
extern FILE * fpicopen( char * name, int * hdr, int mode );
extern void cpichdr( int * ho, int * hi );
extern int readrow( FILE * fp, int row, char * data, int * hdr );
extern int readcol( FILE * fp, int col, char * data, int * hdr );
extern int divpic( int fb, char * filename, int xlower, int ylower, int xupper, int yupper, float mult );
extern int chgval( int fb, float constant, int xlower, int ylower, int xupper, int yupper, float level );
extern int matpic( int fb, char * filename, int xlower, int ylower, int xupper, int yupper, float level, int dirflag );
extern int avgpic( int fb, char * filename, int xlower, int ylower, int xupper, int yupper );
extern int subval( int fb, float constant, int xlower, int ylower, int xupper, int yupper );
extern int subpic( int fb, char * filename, int xlower, int ylower, int xupper, int yupper );
extern int addpic( int fb, char * filename, int xlower, int ylower, int xupper, int yupper, float offset, float mult );
extern int lutgam( int lut, int number, float * convrt );
extern int erase( int greyval );
extern int openpl();
extern int closepl();
extern int fb_move( int xval, int yval );
extern int absmove( int xval, int yval );
extern int space( int x0, int y0, int x1, int y1 );
extern int linemod( char * s );
extern int fb_point( int xval, int yval, int zval );
extern int cont( int xval, int yval, int zval );
extern int fb_line( int x1, int y1, int x2, int y2, int zval );
extern int label( char * s, int size, int zval );
extern int fb_circle( int xval, int yval, int radius, int zval );
extern int fb_box( int x0, int y0, int x1, int y1, int zval );
extern int fb_arc( int x, int y, int x0, int y0, int x1, int y1, int zval );
extern int gambld( char * filename );
extern int fb_gamma( int fb, float * convrt, int n, int xl, int yl, int xu, int yu );
extern int winchk( int argc, char ** argv, int * xlower, int * ylower, int * xupper, int * yupper );
extern int differ( int fb, char * filename, float dval, int xlower, int ylower, int xupper, int yupper );
extern int view( int fb, char * filename, int xlower, int ylower, int xupper, int yupper );
extern int store( int fb, char * filename, int mode, int xlower, int ylower, int xupper, int yupper );
extern int fb_clear( int fb, float * vals, int fast );
