/* dpmfits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Mark-III image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 February 1994
 *  1 Mar 1994: add additional extension file names (namext1, namext2, namext3).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/dpmlib.h"
#include "/home/stanger/lib/include/dpm_optdev.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   char imgbuf ;
   static char linbuf [84] ;
   static char name [80] ;
   static char namext1 [80] = "\0" ;
   static char namext2 [80] = "\0" ;
   static char namext3 [80] = "\0" ;
   static char namimg [80] ;
   static char namlst [80] ;
   FILE *LIST, *IN, *OUT ;
   int hduflag = 0 ;		/* HDU (header data unit) flag. */
   				/* 1: primary image; no extension.        */
				/* 2: primary image; extension exists.    */
				/* 3: extension image; append to namfits. */
   int numext = 0 ;		/* # extensions. */
   int pmode = 0 ;		/* Print mode: 0=silent, 1=verbose. */

   if (argc == 2)
      {
      strcpy (name, argv [1]) ;
      numext = 0 ;		/* No extensions. */
      }
   else
   if (argc == 3)
      {
      strcpy (name, argv [1]) ;
      strcpy (namext1, argv [2]) ;
      numext = 1 ;		/* 1 extension. */
      }
   else
   if (argc == 4)
      {
      strcpy (name, argv [1]) ;
      strcpy (namext1, argv [2]) ;
      strcpy (namext2, argv [3]) ;
      numext = 2 ;		/* 2 extensions. */
      }
   else
   if (argc == 5)
      {
      strcpy (name, argv [1]) ;
      strcpy (namext1, argv [2]) ;
      strcpy (namext2, argv [3]) ;
      strcpy (namext3, argv [4]) ;
      numext = 3 ;		/* 3 extensions. */
      }
   else
      {
      fprintf (stderr, "usage: %s image_file [ext_file1] [ext_file2] [ext_file3] | @image_file_list\n",
	       argv [0]) ;
      fprintf (stderr, "image_file_list:	file containing a list of raw MK3 filenames\n") ;
      fprintf (stderr, "			each line may contain 1 to 3 filenames\n") ;
      exit (-1) ;
      }

   if (name [0] == '@')		/* Read list file. */
      {
      strcpy (namlst, &name [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
	 {
	 fprintf (stderr, "Unable to open file: %s\n", namlst) ;
	 exit (-2) ;
	 }

      while (fgets (linbuf, 80, LIST) != NULL)
	 {
	 sscanf (linbuf, "%s %s %s %s", namimg, namext1, namext2, namext3) ;
	 fprintf (stdout, "%s %s %s %s\n", namimg, namext1, namext2, namext3) ;

         if (strlen (namimg)  == 0)
	    break ;

         if (strlen (namext1) == 0)
	    hduflag = 1 ;
	 else
	    hduflag = 2 ;
         dpmtofits (namimg, namext1, hduflag, pmode) ;

         if (strlen (namext1) == 0)
	    continue ;
	 else
	    hduflag = 3 ;
         dpmtofits (namimg, namext1, hduflag, pmode) ;

         if (strlen (namext2) == 0)
	    continue ;
	 else
	    hduflag = 3 ;
         dpmtofits (namimg, namext2, hduflag, pmode) ;

         if (strlen (namext3) == 0)
	    continue ;
	 else
	    hduflag = 3 ;
         dpmtofits (namimg, namext3, hduflag, pmode) ;

         strcpy (namimg,  NULL) ;
         strcpy (namext1, NULL) ;
         strcpy (namext2, NULL) ;
         strcpy (namext3, NULL) ;
	 }
      fclose (LIST) ;
      }
   else
      {
      strcpy (namimg, name) ;
      if (strlen (namext1) == 0)
         hduflag = 1 ;
      else
         hduflag = 2 ;
      dpmtofits (namimg, namext1, hduflag, pmode) ;

      if (strlen (namext1) == 0)
         exit (0) ;
      else
         hduflag = 3 ;
      dpmtofits (namimg, namext1, hduflag, pmode) ;

      if (strlen (namext2) == 0)
         exit (0) ;
      else
         hduflag = 3 ;
      dpmtofits (namimg, namext2, hduflag, pmode) ;

      if (strlen (namext3) == 0)
         exit (0) ;
      else
         hduflag = 3 ;
      dpmtofits (namimg, namext3, hduflag, pmode) ;
      }

   exit (0) ;
}
