/* dpmtofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Digital Prominence Monitor (DPM) image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   1 March 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "~cordyn/mlsofits/lib/dpm/dpmlib.ext"

dpmtofits (namimg, namext, hduflag, pmode)

   char namimg [];		/* Name of primary file. */

   char namext [];		/* Name of extension file. */

   int hduflag ;		/* 1: primary image; no extension.        */
				/* 2: primary image; extension exists.    */
				/* 3: extension image; append to namfits. */

   int pmode;			/* Print mode. */


{
   extern char *malloc ();		/* Function to allocate memory. */
   extern unsigned char *rddpm ();	/* Function to read DPM image. */
   char *padbuf, *qq;
   unsigned char *dpmbuf;		/* MLSO DPM file  array pointer. */
   unsigned char *imgbuf;		/* MLSO DPM image array pointer. */
   unsigned char *hkbuf;		/* MLSO DPM housekeeping pointer.*/

   static char kairmass  [12] = "AIRMASS ";
   static char kangnorth [12] = "ANGNORTH";
   static char kazimuth  [12] = "AZIMUTH ";
   static char kbangle   [12] = "BANGLE  ";
   static char kbitpix   [12] = "BITPIX  ";
   static char kbscale   [12] = "BSCALE  ";
   static char kbzero    [12] = "BZERO   ";
   static char kbunit    [12] = "BUNIT   ";
   static char kcdelt1   [12] = "CDELT1  ";
   static char kcdelt2   [12] = "CDELT2  ";
   static char kcrota1   [12] = "CROTA1  ";
   static char kcrota2   [12] = "CROTA2  ";
   static char kcrpix1   [12] = "CRPIX1  ";
   static char kcrpix2   [12] = "CRPIX2  ";
   static char kcrval1   [12] = "CRVAL1  ";
   static char kcrval2   [12] = "CRVAL2  ";
   static char kctype1   [12] = "CTYPE1  ";
   static char kctype2   [12] = "CTYPE2  ";
   static char kdate     [12] = "DATE    ";
   static char kdate_cal [12] = "DATE-CAL";
   static char kdate_obs [12] = "DATE-OBS";
   static char kdataform [12] = "DATAFORM";
   static char kdatamax  [12] = "DATAMAX ";
   static char kdatamin  [12] = "DATAMIN ";
   static char kdatatype [12] = "DATATYPE";
   static char kdispmin  [12] = "DISPMIN ";
   static char kdispmax  [12] = "DISPMAX ";
   static char kdpmver   [12] = "DPMVER  ";
   static char kelev     [12] = "ELEV    ";
   static char kend      [12] = "END     ";
   static char kexpdur   [12] = "EXPDUR  ";
   static char kgcount   [12] = "GCOUNT  ";
   static char khangle   [12] = "HANGLE  ";
   static char klangle   [12] = "LANGLE  ";
   static char knaxis    [12] = "NAXIS   ";
   static char knaxis1   [12] = "NAXIS1  ";
   static char knaxis2   [12] = "NAXIS2  ";
   static char koptdev1  [12] = "OPTDEV1 ";
   static char koptdev2  [12] = "OPTDEV2 ";
   static char koptdev3  [12] = "OPTDEV3 ";
   static char koptdev4  [12] = "OPTDEV4 ";
   static char koptdev5  [12] = "OPTDEV5 ";
   static char koptdev6  [12] = "OPTDEV6 ";
   static char koptdev7  [12] = "OPTDEV7 ";
   static char koptdev8  [12] = "OPTDEV8 ";
   static char koptpos1  [12] = "OPTPOS1 ";
   static char koptpos2  [12] = "OPTPOS2 ";
   static char koptpos3  [12] = "OPTPOS3 ";
   static char koptpos4  [12] = "OPTPOS4 ";
   static char koptpos5  [12] = "OPTPOS5 ";
   static char koptpos6  [12] = "OPTPOS6 ";
   static char koptpos7  [12] = "OPTPOS7 ";
   static char koptpos8  [12] = "OPTPOS8 ";
   static char knimavg   [12] = "NIMAVG  ";
   static char kpangle   [12] = "PANGLE  ";
   static char kpcount   [12] = "PCOUNT  ";
   static char kpixrs    [12] = "PIXRS   ";
   static char krsun     [12] = "RSUN    ";
   static char ktmin     [12] = "TMIN    ";
   static char ktmax     [12] = "TMAX    ";
   static char ktime     [12] = "TIME    ";
   static char ktime_obs [12] = "TIME-OBS";
   static char kxcen     [12] = "XCEN    ";
   static char kycen     [12] = "YCEN    ";

   static char atype1 [24] ;			/* Coord type axis 1. */
   static char atype2 [24] ;			/* Coord type axis 2. */
   static char bunit  [32];			/* Brightness unit. */
   static char ctype1 [24] ;			/* Coord type axis 1. */
   static char ctype2 [24] ;			/* Coord type axis 2. */
   static char cunit1 [24] ;			/* Coord units axis 1. */
   static char cunit2 [24] ;			/* Coord units axis 2. */
   static char datcom [72];			/* Data min/max comment. */
   static char date   [12]   = "'DD/MM/YY'";	/* File creation date. */
   static char date_cal [12] = "'DD/MM/YY'";	/* Calibration date. */
   static char date_obs [12] = "'DD/MM/YY'";	/* Observation date. */
   static char dform  [32];			/* Data form. */
   static char eq [4] = "= ";
   static char fitbuf [36][80];		/* FITS header buffer. */
   static char namfit [80];		/* FITS file name. */
   static char object [72];			/* Object. */
   static char outbuf [84];
   static char pixcom [48];			/* Data type comment. */
   static char refcom [48];			/* Coord ref comment. */
   static char time   [12]   = "'HH.MM.SS'";	/* File creation time. */
   static char time_obs [12] = "'HH.MM.SS'";	/* Observation time. */

   int bitpix;				/* # bits per pixel. */
   int gcount = 0;			/* gcount. */
   int iyear;				/* year - 1900. */
   int i, ix, iy;			/* Indices. */
   int ival ;				/* Integer value. */
   int ndim = 0;			/* # dimensions for image (2=XY). */
   int nx, ny;				/* # pixels for X & Y axes. */
   int nbyimg;				/* # bytes in image array. */
   int nbyfits = 2880;			/* # bytes per FITS logical record. */
   int npximg;				/* # pixels in image. */
   int npad;				/* # bytes to pad file. */
   int pcount = 0;			/* pcount. */
   int nrec;				/* # fits records for image. */
   int nelwr  = 0;
   int smin = 0;			/* display min value. */
   int smax = 0;			/* display max value. */

   float angnorth =  90.0 ;		/* solar north w.r.t. +X axis. */
   float bscale   =   1.0 ;		/* Brightness intensity scale factor. */
   float bzero    =   0.0 ;		/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float crpix1, crpix2 ;		/* Reference pixel location. */
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin, dmax ;			/* Data min/max values. */
   float dpmver   =   1.0 ;		/* DPM FITS version number. */
   FILE *FF;				/* FITS file ID. */

   sprintf (namfit, "%s.fits", namimg);

   if (pmode)
      {
      fprintf (stdout, "dpmtofits> namimg: %s  namext: %s   hduflag: %d\n",
	       namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- dpmtofits> %s -> %s -------\n\n",
	       namimg, namfit);
      }

   /*** Read MLSO DPM image/data from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2)
      dpmbuf = rddpm (namimg, pmode);	/* Read DPM dataset into memory. */
   else
      dpmbuf = rddpm (namext, pmode);	/* Read DPM dataset into memory. */

   nx = DPMI->xdim;			/* X dimension. */
   ny = DPMI->ydim;			/* Y dimension. */
   npximg = nx * ny;			/* # pixels in image. */
   nbyimg = npximg * DPMI->nbypx;	/* # bytes in image. */
   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			nx, ny, DPMI->nbypx, npximg, nbyimg) ;

   imgbuf = dpmbuf;			/* Start of pixel data. */

   iyear = DPMI->year;
   if (DPMI->year > 1900)  iyear = DPMI->year - 1900;

   sprintf (date_obs, "'%02d/%02d/%02d'", DPMI->day, DPMI->month, iyear) ;
   sprintf (time_obs, "'%02d.%02d.%02d'", DPMI->hour, DPMI->minute, DPMI->second) ;

   iyear = DPMI->cal_year ;
   if (DPMI->cal_year > 1900)  iyear = DPMI->cal_year - 1900;
   sprintf (date_cal, "'%02d/%02d/%02d'", DPMI->cal_day, DPMI->cal_month, iyear) ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (DPMI->xdim > 0)
      ndim++;
   if (DPMI->ydim > 1)
      ndim++;

   /* DPM data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   for (i = 0; i < 47; i++)
      pixcom [i] = ' ';
   strcpy (datcom, "CCD");
   strcpy (atype1, "X") ;
   strcpy (atype2, "Y") ;
   strcpy (cunit1, "ARCSEC") ;
   strcpy (cunit2, "ARCSEC") ;
   strcpy (refcom, "SUN CENTER") ;
   crpix1 = (float) ((nx - 1) / 2) + 0.5 ;	/* Sun center (X pixels). */
   crpix2 = (float) ((ny - 1) / 2) + 0.5 ;	/* Sun center (Y pixels). */
   crval1 = 0.0 ;				/* Sun center (X arcsec). */
   crval2 = 0.0 ;				/* Sun center (Y arcsec). */
   cdelt1 = 3.006 ;				/* arcsec/pixel. */
   cdelt2 = 3.006 ;				/* arcsec/pixel. */
   dmin =    0.0 ;				/* Minimum scaled data value. */
   dmax = 8192.0 ;				/* Maximum scaled data value. */

   /*** Set object.
    *** Set coordinate type(s).
    *** Set bitpix.
    *** Set display scaling.
    ***/

   switch (DPMI->dtype)
      {
      case  0: /* other */
               strcpy (object, "'DPM IMAGE: OTHER'");   
               strcpy (dform,  "'OTHER'") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1, "'RADIUS'") ;
               strcpy (ctype2, "'THETA'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  1: /* disc */
               strcpy (object, "'DPM SOLAR DISC IMAGE: H-ALPHA'");   
               strcpy (dform,  "'DISC'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  2: /* limb */
               strcpy (object, "'DPM SOLAR LIMB IMAGE: H-ALPHA'");   
               strcpy (dform,  "'LIMB'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax =  5000 ;
               break;
      case  3: /* ref */
               strcpy (object, "'DPM REFERENCE IMAGE'");   
               strcpy (dform,  "'REF'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  4: /* diffuser */
               strcpy (object, "'DPM DIFFUSER IMAGE'");   
               strcpy (dform,  "'DIFFUSER'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit, "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  5: /* disc_dark */
               strcpy (object, "'DPM DISC IMAGE: DARK'");   
               strcpy (dform,  "'DISC_DARK'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  6: /* limb_dark */
               strcpy (object, "'DPM LIMB IMAGE: DARK'");   
               strcpy (dform,  "'LIMB_DARK'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'RADIUS'") ;
               strcpy (ctype2, "'THETA'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  7: /* diffuser_dark */
               strcpy (object, "'DPM DIFFUSER IMAGE: DARK'");   
               strcpy (dform,  "'DIFFUSER_DARK'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      case  8: /* ref_dark */
               strcpy (object, "'DPM REFERENCE IMAGE: DARK'");   
               strcpy (dform,  "'REF_DARK'");
               strcpy (pixcom, "2-BYTE SIGNED PIXELS");
               strcpy (ctype1, "'X'") ;
               strcpy (ctype2, "'Y'") ;
	       strcpy (bunit,  "'CCD'") ;
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
               break;
      default:
               strcpy (object, "'UNKNOWN'");   
               strcpy (dform, "'UNKNOWN'");
               bitpix =  16;			/* short integer image. */
               smin =     0 ;   smax = 8192 ;
	       fprintf (stderr, "Invalid dtype: %d\n", DPMI->dtype);
	       exit (-1);
      }
   DPMI->smin = smin ;
   DPMI->smax = smax ;

   /*** Create FITS file. ***/

   if (hduflag == 3)				/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit);
         exit (-2);
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit);
         exit (-2);
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0; iy < 36; iy++)
      for (ix = 0; ix < 80; ix++)
	 fitbuf [iy][ix] = ' ';

   if (pmode)
      printf (" --- PRIMARY HDU ---\n");

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   strcpy  (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: MLSO DIGITAL PROMINENCE MONITOR");
   else
   sprintf (&fitbuf [iy++][0],  "XTENSION= 'IMAGE   '           / MLSO DPM %s EXTENSION", dform);

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom);
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim);
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, nx, ctype1);
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, ny, ctype2);
   else
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;

   /*** Extension information. ***/

   if (hduflag == 1)
      {
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;
      strcpy (&fitbuf  [iy++][0],  "                                                                                ") ;
      }
   else
   if (hduflag == 2)
      {
      strcpy  (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE");
      strcpy  (&fitbuf [iy++][0],  "                                                                                ") ;
      }
   else
   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount);
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount);
      }

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy++][0],  "OBJECT  = %s", object);
   sprintf (&fitbuf [iy++][0],  "ORIGIN  = 'HIGH ALTITUDE OBSERVATORY'");
   strcpy  (&fitbuf [iy++][0],  "TELESCOP= 'MLSO DIGITAL PROMINENCE MONITOR'");
   strcpy  (&fitbuf [iy++][0],  "INSTRUME= 'KODAK MEGAPLUS 1.6 CCD'");

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s",     kdate_obs, eq, date_obs);
   strcpy  (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT]");
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s",     ktime_obs, eq, time_obs);
   strcpy  (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH/MM/SS [UT]");

   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s%s",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s", atype1) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / %s %s [INDEX]",
			      kcrpix1  , eq, crpix1, atype1, refcom);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom, cunit1);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1);

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%8s%2s%s",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s", atype2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [INDEX]",
			           kcrpix2, eq, crpix2, atype2, refcom);
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4e / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom, cunit2);
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4e / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2  , eq, cdelt2, atype2, cunit2);
      }

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s",     kbunit, eq, bunit);
   strcpy  (&fitbuf [iy++][31], "/ MAGNITUDE UNITS");

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / PHYSICAL = DATA * BSCALE + BZERO",
			       kbscale, eq, DPMI->bscale);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e", kbzero , eq, DPMI->bzero );

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / MINIMUM DATA VALUE [%s]",
			       kdatamin , eq, dmin, datcom);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.4e / MAXIMUM DATA VALUE [%s]",
			       kdatamax , eq, dmax, datcom);

   strcpy  (&fitbuf [iy++][0],  "          **********************************************************************") ;


   /*** All keywords after this point are unique to the MLSO DPM. ***/

   sprintf (&fitbuf [iy++][0], "%8s%2s%20d / MIN VALUE FOR DISPLAY [UNSCALED]",
			      kdispmin , eq, DPMI->smin);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20d / MAX VALUE FOR DISPLAY [UNSCALED]",
			      kdispmax , eq, DPMI->smax);

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / DPM VERSION NUMBER",
			      kdpmver,   eq, dpmver);
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", kdataform, eq, dform);
   strcpy  (&fitbuf [iy++][31], "/ DPM DATA FORM");

   sprintf (&fitbuf [iy++][0], "%8s%2s%20d / DPM DATA TYPE",
			      kdatatype, eq, DPMI->dtype);

   sprintf (&fitbuf [iy++][0], "%8s%2s%20d / NUMBER OF IMAGES AVERAGED",
			      knimavg  , eq, DPMI->navg);

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / EXPOSURE DURATION [SECONDS]",
			      kexpdur  , eq, DPMI->expdur);

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR RADIUS [ARCSEC]",
			      krsun    , eq, DPMI->rsun);

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / DIR OF SOLAR NORTH W.R.T. +X AXIS [CCW DEG]",
			      kangnorth, eq, angnorth);
/* -------------------------------------------------------------------------- */

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   for (iy = 0; iy < 36; iy++)
      for (ix = 0; ix < 80; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ';
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0; i < 36; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80);
      outbuf [80] = '\0';
      if (pmode)
         fprintf (stdout, "%s\n", outbuf);
      }

   /*** Write first record of PRIMARY FITS header. ***/

   nelwr = fwrite (fitbuf, sizeof (char), 2880, FF);

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0; iy < 36; iy++)
      for (ix = 0; ix < 80; ix++)
	 fitbuf [iy][ix] = ' ';

   /*** Fill second record of FITS header. ***/

   iy = 0 ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
			      kpangle  , eq, DPMI->pangle);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
			      kbangle  , eq, DPMI->bangle);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR L-ANGLE [DEGREES]",
			      klangle  , eq, DPMI->langle);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR H-ANGLE [DEGREES]",
			      khangle  , eq, DPMI->hangle);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR ELEVATION [DEGREES]",
			      kelev    , eq, DPMI->elev);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR AZIMUTH [DEGREES]",
			      kazimuth , eq, DPMI->azimuth);
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / AIRMASS",
			      kairmass , eq, DPMI->airmass);

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev1 ,  eq, &DPMI->optdev [0][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 1 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos1 , eq, &DPMI->optpos [0][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 1") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev2 ,  eq, &DPMI->optdev [1][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 2") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos2 , eq, &DPMI->optpos [1][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 2") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev3 ,  eq, &DPMI->optdev [2][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 3") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos3 , eq, &DPMI->optpos [2][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 3") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev4 ,  eq, &DPMI->optdev [3][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 4") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos4 , eq, &DPMI->optpos [3][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 4") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev5 ,  eq, &DPMI->optdev [4][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 5") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos5 , eq, &DPMI->optpos [4][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 5") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev6 ,  eq, &DPMI->optdev [5][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 6 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos6 , eq, &DPMI->optpos [5][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 6") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev7 ,  eq, &DPMI->optdev [6][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 7 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos7 , eq, &DPMI->optpos [6][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 7") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptdev8 ,  eq, &DPMI->optdev [7][0]);
   strcpy  (&fitbuf [iy++][31],"/ OPTICAL DEVICE 8 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%s", koptpos8 , eq, &DPMI->optpos [7][0]);
   strcpy  (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 8") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%s",     kdate_cal, eq, date_cal);
   strcpy  (&fitbuf [iy++][31],"/ DPM CALIBRATION DATE: DD/MM/YY [UT]");
   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY IS LOCATED ON THE ISLAND OF HAWAII,");
   strcpy  (&fitbuf [iy++][0],  "COMMENT   AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY, A DIVISION OF THE");
   strcpy  (&fitbuf [iy++][0],  "COMMENT   NATIONAL CENTER FOR ATMOSPHERIC RESEARCH, UNDER SPONSORSHIP OF THE");
   strcpy  (&fitbuf [iy++][0],  "COMMENT   NATIONAL SCIENCE FOUNDATION.");
   strcpy  (&fitbuf [iy++][0],  "          ----------------------------------------------------------------------") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   THE DIGITAL PROMINENCE MONITOR (DPM) OBSERVES THE SOLAR DISK & LIMB");
   strcpy  (&fitbuf [iy++][0],  "COMMENT   WITH AN APPROXIMATE 1.6 x 2.2 SOLAR RADIUS FIELD OF VIEW, USING A") ;
   strcpy  (&fitbuf [iy++][0],  "COMMENT   KODAK CCD DETECTOR (1534x1032 PIXELS, OF WHICH 1534x1029 ARE USED).");
   strcpy  (&fitbuf [iy++][0],  "COMMENT   BRIGHTNESS MEASUREMENTS ARE SPECIFIED IN UNITS OF BSUN-HALPHA,");
   strcpy  (&fitbuf [iy++][0],  "COMMENT   WHICH IS A RATIO OF PIXEL BRIGHTNESS TO BRIGHTNESS OF SOLAR DISK.") ;
   strcpy  (&fitbuf [iy++][0], "END");

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   for (iy = 0; iy < 36; iy++)
      for (ix = 0; ix < 80; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ';
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0; i < 36; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80);
      outbuf [80] = '\0';
      if (pmode)
         fprintf (stdout, "%s\n", outbuf);
      }

   /*** Write second record of PRIMARY FITS header. ***/

   nelwr = fwrite (fitbuf, sizeof (char), 2880, FF);

   /*** Write pixel data. ***/

   nelwr = fwrite (imgbuf, DPMI->nbypx, npximg, FF);
   free (dpmbuf) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++;
      npad = (nrec * nbyfits) - nbyimg;
      padbuf = malloc (npad);
      qq=padbuf;
      for (i = 0; i < npad; i++)  *qq++ = '\0';
      nelwr = fwrite (padbuf, 1, npad, FF);
      free (padbuf);
      }
   if (pmode)
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg);

   close (FF) ;
   return (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

month_day (year, doy, pmonth, pday)	/* Set month & day from day-of-year. */

int year, doy, *pmonth, *pday;

{
   static int day_tab [2][13] =
      {
      {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      };
   int i, leap;

   leap = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0);

   for (i = 1; doy > day_tab [leap][i]; i++)
      doy -= day_tab [leap][i];

   *pmonth = i;
   *pday = doy;

   return (0);
}

