/* egeo1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Geometric expansion/contraction for HAO Eclipse images (3000x2000 pixels).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 11 October 1990
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define dxin 3000
#define dyin 2000
#define dxout 960
#define dyout 640

main (argc, argv)

int argc;
char *argv[];
{
static char namin [80];
static char namout [80];
static char inbuf [dyin][dxin];
static char outbuf [dyout][dxout];
int fdin, fdout;
int nread, nwrite, nclose;
int nbyin, nbyout;
int nxin = dxin;
int nyin = dyin;
int nxout = dxout;
int nyout = dyout;
float fx, fy;
float cxin, cyin;
float cxout, cyout;

   /*** Command line argument. ***/

   if (argc > 1)
      {
      strcpy (namin, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: egeo image_name\n");
      fprintf (stderr, "Enter Eclipse file name : ");
      fscanf  (stdin,  "%s", namin);
      }

   sprintf (namout, "%s.1", namin);
   fprintf (stderr, "Output file name: %s\n", namout);

   /*** Open input eclipse image. ***/

   if ((fdin = open (namin, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open file : %s\n", namin);
      exit (-2);
      }

   /*** Read image into memory. ***/

   nbyin = nxin * nyin;
   nread = read (fdin, inbuf, nbyin);
   nclose = close (fdin);

   fx = (float) nxout / (float) nxin;
   fy = (float) nyout / (float) nyin;

   cxin = (float) nxin / 2.0;
   cyin = (float) nyin / 2.0;
   cxout = (float) nxout / 2.0;
   cyout = (float) nyout / 2.0;

   /*** Expand or contract image. ***/

   expconb (inbuf, nxin, nyin, outbuf, nxout, nyout,
	   fx, fy, cxin, cyin, cxout, cyout);

   /*** Write new image to disk file. ***/

   fdout = creat (namout, 0644);
   nbyout = nxout * nyout;
   nwrite = write (fdout, outbuf, nbyout);
   nclose = close (fdout);
   exit (0);
}
