/* expconb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Expand/Contract an image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * original author: Rose Reynolds
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * modifications by Andrew L. Stanger
 * 31 October 1990: input & output images are unsigned char 2D arrays.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
expconb (a, dxa, dya, b, dxb, dyb, fx, fy, oax, oay, obx, oby)

unsigned char *a, *b;		/* Input & output arrays. */
int dxa, dya, dxb, dyb;		/* Input/output array sizes. */
float fx, fy;			/* magnify/minify factors. */
float oax, oay, obx, oby;	/* Input/output center coordinates. */
{
   register unsigned char *rb;
   unsigned char *aj0, *aj1;
   int bi, bj, Ai, Aj;
   float ai, aj, fdxa, fdya, factorx, factory, P1, P2, P3, P4, P5, P6, P7;

   fdxa = (float) dxa;  fdya = (float) dya;
   if ( fx < 0.0 )  fx = (float) dxb / fdxa;
   if ( fy < 0.0 )  fy = (float) dyb / fdya;
   rb = b;
   factorx = oax - obx / fx;  factory = oay - oby / fy;

   for ( bj=0; bj<dyb; bj++ )
      {
      aj = (float) bj / fy + factory;
      if ( aj < 0.0 ) aj = 0.0;
      else if ( aj > fdya ) aj = fdya;
      Aj = (int) aj;
      aj0 = a + dxa * Aj;
      aj1 = aj0 + dxa;

      for ( bi=0; bi<dxb; bi++ )
	 {
         ai = (float) bi / fx + factorx;
         if ( ai < 0.0 ) ai = 0.0;
         else if ( ai > fdxa ) ai = fdxa;
         Ai = (int) ai;
         P1 = (float) (*(aj0+Ai));
         P2 = (float) (*(aj0+Ai+1));
         P3 = (float) (*(aj1+Ai));
         P4 = (float) (*(aj1+Ai+1));
         P5 = (ai - (float) Ai) * (P2-P1) + P1;
         P6 = (ai - (float) Ai) * (P4-P3) + P3;
         P7 = (aj - (float) Aj) * (P6-P5) + P5;
         *rb++ = (unsigned char) (P7 + 0.5);
         }  /* end-for  bi=0 to dxb */ 

      }  /* end-for  bj=0 to dyb */ 

   return (0);
}
