/* imgmag.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Magnify a selected portion of a binary image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 26 December 1990
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cp/cplib.h"
char *malloc ();

main ()
{
extern int newsiz ();		/* Squeeze/Stretch function. */

static char namein [80];	/* Input  image file name. */
static char namout [80];	/* Output image file name. */
char *imgin;		 /* Pointer to input image array. */
char *imgout;		 /* Pointer to output image array. */
int fdin, fdout;		/* File descriptor variables. */
int imgsiz;			/* Size of input image (bytes). */
int nread;
int nx1, ny1;			/* Dimensions of input image. */
int nx2, ny2;			/* Dimensions of output image. */
float cx1, cy1;			/* Center of input image. */
float cx2, cy2;			/* Center of output image. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   /*** Request input information. ***/

   fprintf (stderr, "Enter input image file name: ");
   fscanf  (stdin,  "%s", namein);
   sprintf (namout, "%s.i", namein);

   fprintf (stderr, "Enter input image size (x,y): ");
   fscanf  (stdin,  "%d,%d", &nx1, &ny1);
   nbypix = 1;				/* Set bytes/pixel for blint (). */
   npixl = nx1;				/* Set pixels/line for blint (). */

   fprintf (stderr, "Output file name: %s\n", namout);
   fflush (stdin);

   fprintf (stderr, "Enter output image size (x,y): ");
   fscanf  (stdin,  "%d,%d", &nx2, &ny2);

   /*** By default, set center coodinates to be at center of array. ***/ 

   cx1 = (((float) nx1) / 2.0) - 0.5;
   cx2 = (((float) nx2) / 2.0) - 0.5;
   cy1 = (((float) ny1) / 2.0) - 0.5;
   cy2 = (((float) ny2) / 2.0) - 0.5;
   printf ("1 nx1,ny1: %3d,%3d   cx1,cy1: %6.2f,%6.2f\n", nx1, ny1, cx1, cy1);
   printf ("1 nx2,ny2: %3d,%3d   cx2,cy2: %6.2f,%6.2f\n", nx2, ny2, cx2, cy2);

   /*** Read image into memory. ***/

   if ((fdin = open (namein, 0)) <= 0)
      { fprintf (stderr, "Unable to open %s\n", namein); exit (-1); }
   imgsiz = nx1 * ny1;
   imgin  = (char *) malloc (imgsiz);

   nread = read (fdin, imgin, imgsiz);
   close (fdin);
   printf ("nread: %d   imgsiz: %d\n", nread, imgsiz);

   /*** Squeeze/Stretch image. ***/

   imgsiz = nx2 * ny2;
   imgout  = (char *) malloc (imgsiz);
   newsiz (imgin, nx1, ny1, cx1, cy1, imgout, nx2, ny2, cx2, cy2); 

   /*** Write new image to a disk file. ***/

   if ((fdout = creat (namout, 0644)) <= 0)
      { fprintf (stderr, "Unable to create %s\n", namout); exit (-2); }
   write (fdout, imgout, nx2*ny2);
   close (fdout);

   free (imgin);
   free (imgout);
   exit (0);
}


/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * N E W S I Z ( )	Magnify / Minify  a portion of an image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

newsiz (imgin, nx1, ny1, cx1, cy1, imgout, nx2, ny2, cx2, cy2)

char *imgin;	/* Pointer to input image array. */
int nx1, ny1;		/* Dimensions of input image. */
float cx1, cy1;		/* Center coordinates of input image. */
char *imgout;	/* Pointer to output image array. */
int nx2, ny2;		/* Dimensions of output image. */
float cx2, cy2;		/* Center coordinates of input image. */
{
   extern float blint ();	/* Bilinear interpolator. */
   char *im1, *im2;
   int ix, iy;			/* Indices. */
   int ix1, iy1;
   int value = 0;		/* Temporary storage for pixel value. */
   int v2;
   float c2;
   float dstcen;
   float dx1, dy1;		/* Distance from center for input image. */
   float dx2, dy2;		/* Distance from center for output image. */
   float x, y;			/* Pixel coordinates. */
   float x1, y1;		/* Pointers to pixel coordinates. */
   float fx, fy;		/* Magnfy/minify factors. */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   if (nx2 < ny2)
      c2 = cx2;
   else
      c2 = cy2;
   im1 = (char *) imgin;
   im2 = (char *) imgout;
   if (nx2 == ny2)
      {
      if (nx1 >= ny1)
	 fx = fy = (float) ny1 / (float) ny2;
      else
      if (nx1 < ny1)
	 fx = fy = (float) nx1 / (float) nx2;
      }
   else
      {
      fx = (float) nx1 / (float) nx2;
      fy = (float) ny1 / (float) ny2;
      }
   printf ("3 nx1,ny1: %3d,%3d   cx1,cy1: %6.2f,%6.2f\n", nx1, ny1, cx1, cy1);
   printf ("3 nx2,ny2: %3d,%3d   cx2,cy2: %6.2f,%6.2f\n", nx2, ny2, cx2, cy2);
   printf ("fx: %f   fy: %f\n", fx, fy);

   /*** Use bi-linear interpolation to create pixel values for output image.***/

   for (iy=0; iy<ny2; iy++)
      {
      dy2 = (float) iy - cy2;
      dy1 = dy2 * fy;
      y1 = cy1 + dy1;
      iy1 = (int) y1;
      printf ("cy2: %6.2f   iy: %3d   dy2: %6.2f   dy1: %6.2f   y1: %6.2f\n",
               cy2, iy, dy2, dy1, y1);

      for (ix=0; ix<nx2; ix++)
         {
	 dx2 = (float) ix - cx2;
	 dx1 = dx2 * fx;
	 x1 = cx1 + dx1;
	 ix1 = (int) x1;
	 dstcen = (float) sqrt ((double) ((dy2*dy2) + (dx2*dx2)));
	 if (dstcen > c2)	/* Set pixel to 0 if radius is too large. */
	    value = 0.0;
         else
            value = (int) blint (im1, &x1, &y1);
	 v2 = *(im1 + ix1 + iy1*nx1);
         *(im2 + ix + iy*nx2) = (char) (value & 0377);
	 if (ix == iy)  printf ("   line:%3d   %3d   %3d\n", ix, value, v2);   
         }
      }
   return (0);
}
