/* imgsq.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average pixels to produce a smaller, square image.
 * Discard pixels outside centered square sub-image within original image array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 January 1991
 * 29 May 1991: added command line arguments.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

main (argc, argv)

int argc;
char *argv [];
{
   char *malloc ();
   static char namimg [80];
   static char namout [80];
   static char nampgm [80];
   short *aline;
   short *simg;
   short *sline;
   short *sbuf;
   int dx, dy;
   int fdin, fdout;
   int iax, iay;
   int ix, iy;
   int ixoff;
   int imgsiz;
   int nbypix;
   int nx, ny;
   int nave = 2;
   int sumx;

   /*** Command line arguments. ***/

   if (argc == 3)
      {
      strcpy (nampgm, argv [0]);
      strcpy (namimg, argv [1]);
      sscanf (argv [2], "%d", &nave);
      }
   else
      {
      fprintf (stderr, "usage: %s [ecl_name][nave]\n", argv [0]);
      fprintf (stderr, "Enter short integer eclipse image name: ");
      fscanf  (stdin,  "%s", namimg);
      fprintf (stderr, "Enter # pixels to average [integer]: ");
      fscanf  (stdin,  "%d", &nave);
      }

   sprintf (namout, "%s.sq%d", namimg, nave);
   dx = 3000;			/* Eclipse image. */
   dy = 2000;
   nbypix = 2;
   nx = ny = dy / nave;
   ixoff = (dx - dy) / 2;

   /*** Allocate memory. ***/

   imgsiz = nx * ny * nbypix;
   simg = (short *) malloc (imgsiz);
   sbuf = (short *) malloc (nave * dx * nbypix);
   sline = (short *) malloc (nave * dx * nbypix);
   aline = (short *) malloc (nave * nx * nbypix);

   /*** Open input & output files. ***/

   if ((fdin = open (namimg, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open %s\n", namimg);
      exit (-1);
      }
   if ((fdout = creat (namout, 0644)) <= 0)
      {
      fprintf (stderr, "Unable to create %s\n", namout);
      exit (-2);
      }

   /*** Read one line at a time from input image file. ***/

   for (iy = 0; iy < ny;  iy++)
      {
      for (ix = 0; ix < nx; ix++)  *(simg + ix + iy*nx) = 0;	/* Zero line. */

      for (iay = 0; iay < nave; iay++)		/* line average loop.*/
	 {
	 read (fdin, sbuf, dx*nbypix);		/* Read one line into sline. */
	 swab (sbuf, sline, dx*nbypix);		/* Swap bytes. */
	 for (ix = 0; ix < nx; ix++)
	    {
            sumx = 0;
	    for (iax = 0; iax < nave; iax ++)
	       sumx += *(sline + ix*nave + iax + ixoff);/* X-axis addition. */
            *(aline + ix) = sumx;
	    *(simg+ix+iy*nx) += *(aline + ix);		/* Y-axis addition. */
	    }
         for (ix = 0; ix < nx; ix++)
	    *(simg + ix + iy*nx) = (short) ((float) *(simg+ix+iy*nx) / (float) (nave*nave));
	 }
      }
   close (fdin);

   /*** Write square image to output file. ***/

   write (fdout, simg, imgsiz);
   close (fdout);
   exit (0);
}
