/* invecl.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Invert image:
 *    Transpose intensity levels.
 *    Invert left to right.
 *    Invert top to bottom.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 11 October 1990
 *  2 January 1991: Geometric inversions are optional.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

main (argc, argv)

int argc;
char *argv[];
{
static char answer [80];
static char namin [80];
static char namout [80];
static char inbuf [2000][3000];
static char outbuf [2000][3000];
int fdin, fdout;
int lhor, lver;			/* Horizontal & vertical inversion flags. */
int nread, nwrite, nclose;
int nbyin, nbyout;
int nxin = 3000;
int nyin = 2000;
int nxout = 3000;
int nyout = 2000;

   if (argc > 1)
      {
      strcpy (namin, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: invecl image_name\n");
      fprintf (stderr, "Enter Eclipse file name : ");
      fscanf  (stdin,  "%s", namin);
      }
   fprintf (stderr, "Flip left to right ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)
      lhor = 1;   else lhor = 0;

   fprintf (stderr, "Flip top to bottom ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)
      lver = 1;   else lver = 0;

   sprintf (namout, "%s.inv", namin);
   fprintf (stderr, "Output file name: %s\n", namout);

   if ((fdin = open (namin, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open file : %s\n", namin);
      exit (-2);
      }

   nbyin = nxin * nyin;
   nread = read (fdin, inbuf, nbyin);
   nclose = close (fdin);

   invimg (inbuf, outbuf, nxin, nyin, lhor, lver);

   fdout = creat (namout, 0644);
   nbyout = nxout * nyout;
   nwrite = write (fdout, outbuf, nbyout);
   nclose = close (fdout);
}


/* invimg
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Invert image top to bottom, left to right, min to max intensities.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

invimg (inbuf, outbuf, nx, ny, lhor, lver)

char *inbuf, *outbuf;
int nx, ny;
int lhor, lver;

{
int dx, dy;
int ix, iy, iix, iiy;

   dx = nx;
   dy = ny;


   for (iy=0; iy<ny; iy++)
      {
      if (lver == 0)  iiy = iy;  else iiy = dx - iy;
      for (ix=0; ix<nx; ix++)
         {
	 if (lhor == 0)  iix = ix;  else iix = dx - ix;
         *(outbuf + ix + iy*nx) =
            (char) (255 - (int) *(inbuf + iix + iiy*nx));
         }
      }
   return (0);
}
