/* To convert from bangle to Position Angle, apply the following:
   PA = ba + phase - pangle
*/

/*  Based on the barrel angles, compute a spline fit
    for the i q u data to go from 0 to 360 degrees.

    This code was written and updated by Alice Lecinski, September 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/


#define FLOATFUNCTIONTYPE       double  /* sparc */
#define ASSIGNFLOAT(x,y)        { union {double _d; float _f;} _kluge; _kluge._d = (y); x = _kluge._f; }

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>
#include "mk4v2.h"
#include "mk4headv2.h"


float svec[VECTOR*2];
float Spline[CY_IMG], yp[CY_IMG], temp[CY_IMG];

float sigma=1.,slp1=0.,slpn=0.;
int   islpsw=0,ierr;
extern FLOATFUNCTIONTYPE curv2_();
void  curv1_();


int
fcyspline  ( hd, Img , fba, NeedToFixBA)
    register FH hd ;
    float *Img;
    float *fba;
    int NeedToFixBA;
{

    int ii,jj;
    int nray,imsz;
    int doprint=0;
    float myang,myI;
    float *pS, *pIm;
    int fixba();

    pS = Spline; for(ii=0;ii<CY_IMG;ii++){ *pS = 0.; pS++; }
    pS = yp    ; for(ii=0;ii<CY_IMG;ii++){ *pS = 0.; pS++; }
    pS = temp  ; for(ii=0;ii<CY_IMG;ii++){ *pS = 0.; pS++; }

    if(NeedToFixBA==1)
	(void)fixba(hd,fba,0);     /* Convert the ba to radians */

    /* Spline fit. */
    nray=hd->Xdim;
    pIm =Img;
    pS  =Spline;
    for(ii=0;ii<NPX;ii++){ 
	pIm  = Img    +ii*nray;
	pS   = Spline +ii*NAZ;

	curv1_ (&nray,fba,pIm,&slp1,&slpn,&islpsw,yp,temp,&sigma,&ierr);
	for(jj=0;jj<NAZ;jj++){
	    myang = -180.*d2r +(float)jj*AngSep*d2r; 
	    myang = (myang<fba[0]) ? myang+twopi
				   : (myang>fba[nray-1]) ? myang-twopi
						         : myang         ;

	    if( (myang<fba[0]) || (myang>fba[nray-1])) myang=fba[0];

	    ASSIGNFLOAT(myI,curv2_ (&myang,&nray,fba,pIm,yp,&sigma));

	    *pS = myI;
	    pS++;
	}
	if((ii==60)&&(doprint==1)) {
	    pS   = Spline +ii*NAZ;
	    for(jj=0;jj<40;jj++)
		fprintf(stderr,"%d %f %f\n",jj,fba[jj]*r2d,*pS++);
	}
    }

    /* Copy the Spline image to the original image. */
    imsz= NPX*NAZ;
    pIm = Img;
    pS  = Spline;
    for(ii=0;ii<imsz;ii++){
	*pIm = *pS;
	pS++;
	pIm++;
    }

}

int
fixba (hd,pfba,doprint)
    register FH hd ;
    float *pfba;
    int doprint;
{
    int ii;
    float *pX;
    float *pf, prevba, diffba, ftemp;
    float thoff;

    int   do_do_lst;
    float a,b,R;

    do_do_lst = 1 ;  /* Replace the barrel angles with
		      * a least squares fit 
		     */

    thoff =  0. ;
    thoff =  0.00033 * d2r;

    /* Convert to radians. */
    /* Avg bangle increment is 17. */
    pX=svec;
    pf=pfba;
    prevba   = (float)(*pfba)*ba2rad + thoff + -17.*ba2rad;
    for(ii=0;ii<hd->Xdim;ii++){ 
        ftemp  =      (*pf)  *ba2rad + thoff ;
        diffba = ftemp - prevba;
        if(diffba > 35.*ba2rad ) {
            fprintf(stderr,"At ba %d ba way too increasing!\n",ii);
        }
        if(diffba < 0. ) {
            fprintf(stderr,"At ba %d ba not increasing!\n",ii);
        }
        *pf      = ftemp;
        prevba   = ftemp;
	if(doprint==1) fprintf(stderr,"BA %d %f\n",ii,*pf * r2d);
        pf++;
	*pX = (float)ii;
	pX++;
    }

    if(do_do_lst==1) {
	pX=svec;
	pf=pfba;
	do_lst (&a,&b,&R, pX, pf, hd->Xdim , 1);
	for(ii=0;ii<hd->Xdim;ii++){
	    *pf = a + (*pX) * b;
	    pX++;
	    pf++;
	}
    }
}

int
fshiftimg(im,angoff)
    float *im;
    double angoff;           /* angoff is in degrees. */
{                            /* The image must be a theta by R array,
                                and already spline fit to the std 
				azimuthal size. */
    int dd,ii,jj;
    int shift,start;
    float *pq, *pu;

    pq=im;

    shift = (int)((angoff+0.5*AngSep)*ovAngSep);

    for(dd=0;dd<NPX;dd++){

	start = ( shift >= 0 ) ? shift : NAZ + shift ;

	pu=svec;        pq=im +dd*NAZ; for(ii=0;ii<NAZ;ii++)  *pu++ = *pq++;
	pu=svec+NAZ;    pq=im +dd*NAZ; for(ii=0;ii<NAZ;ii++)  *pu++ = *pq++;

	pu=svec+start;  pq=im +dd*NAZ; for(ii=0;ii<NAZ;ii++)  *pq++ = *pu++;

    }
}
