/* fibrms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fiber removal (1989: upper right).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 16 October 1992
 *  7 June 1993: UNIX version; no screen display.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

#define NBOX 1 
#define NSEARCH 50
#define NINT 20 

fibrms (imgbuf, nbypx, xdim, ydim)

unsigned char *imgbuf;		/* Image buffer. */
int nbypx;			/* # bytes per pixel. */
int xdim;			/* X-dimension. */
int ydim;			/* Y-dimension. */
{
   static char dir2 [] = "ru";
   static unsigned char pixval [NSEARCH][NSEARCH];

   float pixsum = 0.0;
   int pixave = 0;
   int pixcnt = 0;

   int darkpx, darkpy;
   int dpx, dpy;
   int ibox;
   int ierr;
   int level = -1;
   int pixmin;
   int x, y;
   int xmin = 0;
   int ymin = 0;
   int xp, yp;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;

   static int xd [NBOX] = {340};	/* Lower left corner of box. */
   static int yd [NBOX] = {350};
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = 0;

   /*** Fiber Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {

      /* Lower left corner of box. */

      xmin = xd [ibox] -  63;
      ymin = 510 - yd [ibox];

      if (xmin + dxs > 447)  xmin = 447 - dxs;
      if (ymin + dys > 447)  ymin = 447 - dys;
      if (xmin < 0)  xmin = 0;
      if (ymin < 0)  ymin = 0;

      /*** Read box centered about estimated fiber location. ***/

      ierr = mr_box (imgbuf, xdim, ydim, pixval, xmin, ymin, dxs, dys, dir2);
      if (ierr < 0)
	 {
	 fprintf (stderr, "fibms> mr_box error: %d  x:%3d  y:%3d\n",
		  ierr, xmin, ymin);
         return (ierr);
	 }

      pixsum = 0.0;
      pixave = 0;
      pixcnt = 0;

      /*** Find average pixel intensity within box. ***/

      for (y = 0; y < dys; y++)
         {
         yp = ymin + y;
         if (yp > 62 && yp < 511)

            for (x = 0; x < dxs; x++)
               {
               xp = xmin + x;
               if (xp > 62 && xp < 511)
                  {
                  pixsum += (float) pixval [y][x];
                  pixcnt++;
                  }
              }
         }

      pixave = (int) (pixsum / (float) pixcnt);
      level = pixave;
      printf ("pixave: %d   pixcnt: %d   pixsum: %6.2f\n",
              pixave, pixcnt, pixsum);

      /*** Replace any pixel whose value is less than pixave with pixave. ***/

      for (y = 0; y < dys; y++)
         {
         yp = ymin + y;
         if (yp > 62 && yp < 511)

            for (x = 0; x < dxs; x++)
               {
               xp = xmin + x;
               if (xp > 62 && xp < 511)
                  {
                  if (*(imgbuf + xp + yp*xdim) < pixave)
                     *(imgbuf + xp + yp*xdim) = pixave;
                  }
               }
         }

      /*** Perform horizontal interpolation for each box. ***/

/*
      ierr = hint (imgbuf, xdim, ydim, level, nbox, xd, yd, dxs, dys, dir2); */
      if (ierr < 0)
	 {
	 fprintf (stderr, "fibrms> hint error: %d   x:%3d   y:%3d\n",
		  ierr, xd, yd);
         return (ierr);
         }

      }
   return (0);
}
