/* findcen
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find solar disk center & diameter.
 *
 * Adapted from "find_shift.c" in "/home/puka/mrchip/chipcal/src".
 * Calls Alice's subroutine findshift () to find shift values for given image.
 *
 * Adapted by Andrew L. Stanger   15 March 1996
 *
 * Andy was using doing a couple of things which I fixed.
 *  1) taking out the housekeeping line, then shoving it back in
 *  2) Using the 20 lines of padding and then not compensating for it
 *     in the center calculations.
 *  So, I took 1) out  and made padding and option, and called my routine
 *  "findshift" directly and calculate the centers properly from that.
 *  --Alice 4/22/96
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */


#include        <stdio.h>
#include        <stdlib.h>
#include        <errno.h>
#include        <math.h>

#include "/home/cordyn/mlsofits/lib/acos/acos.h"

int fts=1;

findcen (imgbuf, xdim, ydim, xcenter, ycenter, diam, scode)

short *imgbuf ;
int xdim, ydim ;
float *xcenter, *ycenter, *diam ;
int scode;
{
    int            findshift () ;
    int            ii, ix, iy, code, read (), write () ;
    short          *img ;
    float          XCEN, YCEN, xshi, yshi , radius;
    register short *op ;
    register short *ip ;
    int            nrow, ncol, record ;
    int            dopad,pad,hpad;

    dopad=1;pad=20;hpad=10;  /* Add padding to the y-dimension just incase
				disk is too close to the edge.  */
    dopad=0;pad= 0;hpad= 0;  /* Don't need no stinkin' padding! */

    ncol = xdim ;
    nrow = ydim ;

    XCEN = (xdim-1.)*0.5;
    YCEN = (ydim-1.)*0.5;


    /*
     * fprintf (stderr, "find_shift: XDIM %d YDIM %d RECORD %d\n",
     *          ncol, nrow, record) ;
     */

    if ( dopad ) 
	{
	img = (short *) malloc ( sizeof (short) * ncol * (nrow + pad) ) ;

	ip = img ;
	ip += hpad*ncol ;

	for (iy = 0 ; iy < nrow ; iy++)
	   for (ix = 0 ; ix < ncol ; ix++)
	       {
	       *(ip++) = *(imgbuf + ix + iy * xdim) ;
	       }

	/*** Pad top and bottom in case Y is too close to edge ***/

	for ( iy = 0 ; iy < hpad ; iy++ )
	    {
	    ip = img ;
	    ip += hpad * ncol ;
	    op = img ;
	    op += iy * ncol ;
	    for ( ix = 0 ; ix < ncol ; ix++ )   *op++ = *ip++;
	    }

	for ( iy = nrow - hpad ; iy < nrow ; iy++ )
	    {
	    ip = img ;
	    ip += (nrow - 11) * ncol ;
	    op = img ;
	    op += iy * ncol ;
	    for ( ix = 0 ; ix < ncol ; ix++ )   *op++ = *ip++;
	    }

	}
    else
	{
	img = imgbuf;
	}

    /* fprintf ( stderr, "find_shift calling findshift\n" ) ; */

    /*
    if (ncol ==  767) *diam = 335.0 ;
    if (ncol == 1534) *diam = 335.0 * 2.0 ;
    */

    if(scode == SOURCE_l) {
	ii = findshiftL(img+3*ncol, ncol-6, nrow+pad, &xshi, &yshi, diam) ;
    }
    else {
	ii = findshift (img+3*ncol, ncol-6, nrow+pad, &xshi, &yshi, diam) ;
    }

    if(ii==1) {                  /* findshift could do it's thing. */
	*xcenter = XCEN - xshi;
	*ycenter = YCEN - yshi;
    }
    else {                       /* findshift couldn't do it's thing. */
	if(scode == SOURCE_l) {
	    ii = findshiftL_sobel
		 (img+3*ncol, ncol-6, nrow+pad, &xshi, &yshi, diam) ;
	    if(ii!=1) {
		ii = findshiftL_max
		     (img+3*ncol, ncol-6, nrow+pad, &xshi, &yshi, diam) ;
		     /* fprintf(stderr,"ii is %d\n",ii); */
	    }
	    if(ii!=1) {
		ii =  dPcenter_nofile(img,1,&XCEN,&YCEN,&radius);
		*diam    = radius * 2.;
		xshi=yshi=0.;
	    }
	}
	if(scode == SOURCE_s) {
	    ii =      dPcenter_nofile(img,0,&XCEN,&YCEN,&radius);
	    *diam    = radius * 2.;
	    xshi=yshi=0.;
	}
	if(ii==1) {  /* one of the above findshifts could do it's thing. */
	    *xcenter = XCEN - xshi;
	    *ycenter = YCEN - yshi;
	}
	else {
	    *xcenter = -999999.;
	    *ycenter = -999999.;
	}
    }

    if(dopad) free (img) ;

}
