/* mk3fits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Mark-III image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 February 1994
 * 17 Feb 1994: add additional extension file names (namext1, namext2, namext3).
 * 28 Mar 1994: use fgets instead of fscanf in while loop.
 * 30 May 1997: add optional photometric scaling parameters to command line.
 *  3 Mar 98: Scaling from 16-bits to 8-bits now an option (bscale = 1).
 * 31 Mar 99: version 1.05: add CRRADIUS to header.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/cordyn/mlsofits/lib/mk3/mk3lib.h"
#include "/home/cordyn/mlsofits/img_scale.h"
int nbypix ;                             /* Ouput image: # bytes/pixel. */

main (argc, argv)

int argc ;
char *argv [] ;

{
   char imgbuf ;
   static char linbuf [84] ;
   static char nampgm [80] ;
   static char name [80] ;
   static char namext1 [80] = "\0" ;
   static char namext2 [80] = "\0" ;
   static char namext3 [80] = "\0" ;
   static char namimg [80] ;
   static char namlst [80] ;
   char *parg;			/* Argument pointer. */

   FILE *LIST, *IN, *OUT ;
   int hduflag = 0 ;		/* HDU (header data unit) flag. */
   				/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */
   int numext = 0 ;		/* # extensions. */
   int pmode = 0 ;		/* Print mode: 0=silent, 1=verbose. */

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   strcpy (nampgm, argv [0]) ;

   if (argc == 1)
      {
      usage (nampgm) ;
      exit (0);
      }

   while (--argc)
      {
      if (*(parg = *++argv) == '-')		/* Check for flag arguments */
	 {
	 switch (parg [1])
	    {
            case 'b':  nbypix = 1 ;	/* Output image: 1 byte / pixel. */
                       --argc ; parg = *++argv ;
                       break ;
	    case 's':  lscale = 1 ;	/* specify intensity scaling. */
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmin) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmax) ;
		       break;
            default:   fprintf (stderr," unknown flag %s\n", *argv);
		       usage (nampgm); exit (-2);
            }
         }
         if (argc == 2)
	    {
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", name) ;
	    }
         else
	 if (argc == 3)
	    {
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", name) ;
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", namext1) ;
	    }
         else
	 if (argc == 4)
	    {
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", name) ;
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", namext1) ;
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", namext2) ;
	    }
         else
	 if (argc == 5)
	    {
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", name) ;
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", namext1) ;
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", namext2) ;
	    --argc ;  parg = *++argv ;  sscanf (parg, "%s", namext3) ;
	    }
      else					/* Get the image filename. */
         {
         if (*parg == '?') { usage (nampgm); exit (-3); }
         strcpy (name, parg) ; 
	 }
      }

   if (name [0] == '@')		/* Read list file. */
      {
      strcpy (namlst, &name [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
	 {
	 fprintf (stderr, "Unable to open file: %s\n", namlst) ;
	 exit (-2) ;
	 }

      while (fgets (linbuf, 80, LIST) != NULL)
	 {
	 sscanf (linbuf, "%s %s %s %s", namimg, namext1, namext2, namext3) ;
	 fprintf (stdout, "%s %s %s %s\n", namimg, namext1, namext2, namext3) ;

         if (strlen (namimg)  == 0)
	    break ;

         if (strlen (namext1) == 0)
	    hduflag = 1 ;
	 else
	    hduflag = 2 ;
         mk3tofits (namimg, namext1, hduflag, pmode) ;

         if (strlen (namext1) == 0)
	    continue ;
	 else
	    hduflag = 3 ;
         mk3tofits (namimg, namext1, hduflag, pmode) ;

         if (strlen (namext2) == 0)
	    continue ;
	 else
	    hduflag = 3 ;
         mk3tofits (namimg, namext2, hduflag, pmode) ;

         if (strlen (namext3) == 0)
	    continue ;
	 else
	    hduflag = 3 ;
         mk3tofits (namimg, namext3, hduflag, pmode) ;

         strcpy (namimg,  NULL) ;
         strcpy (namext1, NULL) ;
         strcpy (namext2, NULL) ;
         strcpy (namext3, NULL) ;
	 }
      fclose (LIST) ;
      }
   else
      {
      strcpy (namimg, name) ;
      if (strlen (namext1) == 0)
         hduflag = 1 ;
      else
         hduflag = 2 ;
      mk3tofits (namimg, namext1, hduflag, pmode) ;

      if (strlen (namext1) == 0)
         exit (0) ;
      else
         hduflag = 3 ;
      mk3tofits (namimg, namext1, hduflag, pmode) ;

      if (strlen (namext2) == 0)
         exit (0) ;
      else
         hduflag = 3 ;
      mk3tofits (namimg, namext2, hduflag, pmode) ;

      if (strlen (namext3) == 0)
         exit (0) ;
      else
         hduflag = 3 ;
      mk3tofits (namimg, namext3, hduflag, pmode) ;
      }

   exit (0) ;
}

usage (nampgm)

char nampgm [] ;
{
   fprintf (stderr, "Usage: %s [-b][-s oldmin oldmax] image_file [ext_file1] [ext_file2] [ext_file3] | @image_file_list\n",
	       nampgm) ;
   fprintf (stderr, "   -b     FITS output will be 8 bits/pixel.\n");
   fprintf (stderr, "   -s     Override default intensity scaling.\n");
   fprintf (stderr, "          oldmin, oldmax: min -> max image values;\n");
   fprintf (stderr, "                          will be displayed as 0 -> 250.\n");
   fprintf (stderr, "          This flag only applies if -b is specified.\n");
   fprintf (stderr, "image_file:	MK3 native format filename.\n") ;
   fprintf (stderr, "ext_file1 :	MK3 extension 1 filename.\n") ;
   fprintf (stderr, "ext_file2 :	MK3 extension 2 filename.\n") ;
   fprintf (stderr, "ext_file3 :	MK3 extension 3 filename.\n") ;
   fprintf (stderr, "image_file_list:	file containing a list of raw MK3 filenames\n") ;
   fprintf (stderr, "			each line may contain 1 to 3 filenames\n") ;
   return (0);
}
