/* hs.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Histogram Specification Scaling.
 * From Winter 1997 CCD Astronomy article,
 * "An Optimized Stretch for Deep-Sky Images", * by Blaise Canzain.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 April 1997
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

unsigned char *
hs (imgbuf, xdim, ydim, nbypx)

char *imgbuf ;				/* Image array. */
int xdim, ydim ;			/* Image dimensions. */
int nbypx ;				/* # bytes per pixel. */
{
   unsigned char *ibuf ;
   unsigned char *cbuf ;
   unsigned char *gbuf ;
   short *sbuf ;
   float *fbuf ;

   short smin, smax ;
   float slope, offset ;
   float fmin, fmax ;

   int fgauss [256] ;			/* Gaussian array (float). */
   int gauss [256] ;			/* Gaussian array (int). */
   int gausscum [256];			/* Gaussian  cumulative integral. */
   int gausscuminv [256] ;
   int gsum ;				/* Gaussian  sum. */
   int hist [256] ;			/* Histogram array. */
   int histcum [256];			/* Histogram cumulative integral. */
   int hsum ;				/* Histogram sum. */
   int nelem ;				/* # pixels in image. */
   int nlevels = 256;			/* # Histogram Levels. */

   float ndif ;
   float nzero = 127.5 ;
   float s     = 256.0 / 6.0 ;

   nelem = xdim * ydim ;

   ibuf = (unsigned char *) malloc (nelem) ;
   gbuf = (unsigned char *) malloc (nelem) ;

   /*** Convert input array into a byte array. ***/

   case (nbypx)
      {

      /*** Byte Array. ***/

      1: cbuf = (unsigned char *) imgbuf ;
	 for (i = 0; i < nelem; i++)
	    *(ibuf + i) = *(cbuf + i) ;
	 break ;

      /*** Short Int Array. ***/

      2: sbuf = (short *)         imgbuf ;
	 for (i = 0; i < nelem; i++)
	    {
	    smin =  32767 ;
	    smax = -32767 ;
	    if (*(sbuf + i) < smin)  smin = *(sbuf + i) ;
	    if (*(sbuf + i) > smax)  smax = *(sbuf + i) ;
	    }
         slope = 256.0 / (smax - smin) ;
	 for (i = 0; i < nelem; i++)
	    {
	    *(ibuf = i) = (unsigned char) ( (*(sbuf + i) - smin) * slope ) ;
	    }
         break ;

      /*** Floating Point Array. ***/

      4: fbuf = (float *)         imgbuf ;
	 for (i = 0; i < nelem; i++)
	    {
	    smin =  32767 ;
	    smax = -32767 ;
	    if (*(fbuf + i) < fmin)  fmin = *(sbuf + i) ;
	    if (*(fbuf + i) > fmax)  fmax = *(sbuf + i) ;
	    }
         slope = 256.0 / (fmax - fmin) ;
	 for (i = 0; i < nelem; i++)
	    {
	    *(ibuf = i) = (unsigned char) ( (*(fbuf + i) - fmin) * slope ) ;
	    }
      }
   
   /*** Clear Historgram to Zero. ***/

   for (i = 0; i < nelem; i++)
      {
      hist [ i ] += 1 ;
      }

   /*** Compute Historgram. ***/

   for (i = 0; i < nelem; i++)
      {
      hist [*(ibuf + i)] += 1 ;
      }

   /*** Normalize Histogram. ***/

   for (i = 0; i < nelem; i++)
      {
      hist [i] /= nlevels ;
      }

   /*** Compute Cumulative Histogram Function. ***/

   for (i = 0 ; i < nlevels; i++)
      {
      hsum += hist [i] ;		/* Histogram Integral. */
      histcum [i] = hsum ;		/* Cumulative Histogram Integral. */
      }

   /*** Compute Gaussian Function. ***/

   for (i = 0 ; i < nelem; i++)
      {
      ndif = (float) *(ibuf + i) - nzero ;
      fgauss [*(ibuf + i)] = ( fexp ( - (ndif * ndif) / (2 * s * s) ) ) ;
      }

   /*** Normalize Gaussian. ***/

   for (i = 0; i < nelem; i++)
      {
      gauss [i] = (int) (fgauss [i] / nlevels) ;
      }

   /*** Compute Cumulative Gaussian Integral. ***/

   for (i = 0 ; i < nlevels; i++)
      {
      gsum += gauss [i] ;		/* Gaussian Integral. */
      gausscum [i] = gsum ;		/* Cumulative Gaussian Integral. */
      }

   /*** Compute Inverse Cumulative Gaussian Integral. ***/

   for (i = 0 ; i < nlevels; i++)
      {
      gdif = i - gausscum [il ;
      gausscuminv [i] = i + gdif ;
      }

   for (i = 0; i < nelem; i++)
      {
      n = *(ibuf + i) ;
      *(gbuf + i) = gausscuminv [ histcum [n] ] ;
      }
   return (gbuf) ;
}
