#include <stdio.h>
#include "mk4.h"

/**********************  misc.c       ******************************/


char *
prday (i)

register int i;
{
    static char c [10];
    register char *p = c;

    jdate (i + 2415021, &mon, &day, &year);
    sprintf (p,"%2d/%02d/%2d",mon, day, year - 1900);
    return (p);
}


char *
prtime (i)

register unsigned i;
{
    /* i is in Hz */

    static char t [12];
    register char *p = t;
    int hr, min;
    float sec;

    get_hrminsec(i,&hr,&min,&sec);
    sprintf (t,"%02d_%02d_%05.2f",hr,min,sec);
    return (p);
}


int 
get_hrminsec (i,hr,min,sec)

register unsigned i; int *hr,*min; float *sec;
{
    /* i is in Hz */

    *hr  = i / (60 * 60 * 60 );
    *min = i / (60 * 60 )       -   *hr * 60  ;
    *sec = ((float)i / 60. )    - ((*hr * 60 + *min) * 60.0);
}


double
pangle (i)

register int i;
{
    return (i * pi / (180 * 60));
}


double
phi (i)

register int i;
{
    return (i * pi / 32768);
}


/*	doy = idoy (mon, day, year);  */

int 
idoy (m, d, yr)

int m, d, yr ;
{
    static int day_tab [2][13] =
	{ 
	{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, 
	{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
        };

    int ii , leap ;

    leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0 ;

    for ( ii=1 ; ii<m ; ii++ ) 
	d += day_tab [leap][ii] ;
    return (d) ;
}

int
get_doy (m, d, yr)

int m, d, yr ; 
{
    static int day_tab[2][13] =
	{ 
	{0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}, 
	{0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
        };
    int ii , leap ;

    leap = yr%4 == 0 && yr%100 != 0 || yr%400 == 0 ;
    for ( ii=1 ; ii<m ; ii++ ) d += day_tab [leap][ii] ;
    return (d) ;
}


/************************** /hao/src/lib/misc *****************/

/*
 * purpose:
 * -------
 *	    This algorithm converts a correctly given calendar date,
 *     given on the Julian calendar on and before Oct 4, 1582 and on
 *     the  Gregorian calendar on and following Oct 15, 1582 through
 *     Feb 28, 4000 AD, to the correct Julian date.
 *
 * parameters:
 * ----------
 * year    -->   to input year BC, subtract year from 0 to get its
 *		 astronomical  representation.  e.g.,  4713 BC  is
 *		 input as -4713.
 * month   -->   month (as integer 1 to 12)
 * day     -->   day of month
 * jday    <--   julian date (begins at 12:00 noon GMT of input date;
 *			      12 noon, Jan 1, 4713 BC = JD 0)
 *
 * references:
 * ----------
 *
 * 1) CACM 11 (Oct 1968) 657.
 * 2) Abell, George - Exploration of the Universe, 2nd ed. (1969) p.147-151.
 *
 * notes:
 * -----
 *
 * 1) The function as presented in reference article assumes  the  Gregorian
 *    calendar indefinitely back in time, so it will not give proper results
 *    for input date prior to Oct 15, 1582.  This routine has been  modified
 *    to correct this error.
 *
 * 2) Input of the dates Oct 5 thru 14, 1582, which did not  exist,
 *    will  cause  return  of  Julian date for date 10 days before.
 *    (e.g., output for Oct 9, 1582 will be correct Julian date for
 *    Sept 29 1582).  An error message is printed.
 *
 * 3) By  1582,  the  Julian  calendar  was  10  days fast.  Pope Gregory XIII
 *    reformed the calendar by proclaiming the day following  Oct 4,  1582  to
 *    be  Oct 15, and  by  decreeing that only century years divided by 400 to
 *    be leap years.  The reform was adopted immediately by Catholic countries
 *    but  resisted  by Eastern Church and Protestant countries.  This routine
 *    corrects for the missing dates of  1582 ( notes 1 and 2 ) but  does  not
 *    correct  for  the missing dates when other nations made the change.  For
 *    example, England and America  adopted  the  new  style calendar in 1752.
 *    By parliamentary decree Sept 2, 1752 was followed by Sept 14.  Moreover,
 *    the start of the year was moved back from Mar 25 to Jan 1, thus  in  the
 *    English domains 1751 contained no Jan, Feb, or Mar 1-24.
 *
 * 4) The Gregorian calendar  has now been modified to obtain closer  conformity
 *    with the tropical year by making common years those years divided by 4000.
 *    Thus 4000 AD is not a leap year.  This  modification  makes  the  calendar
 *    accurate to 1 day in about 20,000 years.
 */

int
jday (month, day, year)

register int month, day, year;
{
    register int i;

    i = (month - 14) / 12;	/* i = month <= 2 ? 1 : 0 */
    if (year < 0) year += 1;
    year += i;

    day += 1461 * (year + 4716) / 4 - 1432
	  + 367 * (month - 2 - i * 12) / 12;

    if (day >= 2299161) day -= (3 * ((year - 1500) / 100)) / 4 + 10;

    return (day);
}


jday_ (month, day, year)			/* Fortran-callable version. */

int *month, *day, *year;
{
    return (jday (*month, *day, *year));
}


/*
 * The inverse of jday but it only works for dates after Oct 4, 1582
 *						   -----
 */

jdate_ (jday, month, day, year)

long *jday; int *month, *day, *year;
{
    jdate (*jday, month, day, year);
}


int
jdate (jday, month, day, year)

long jday; int *month, *day, *year;
{
    register int l, n;
	
    l = jday + 68569;
    n = (4 * l) / 146097;
    *year = 100 * (n - 49) ;
    l -= (146097 * n + 3) / 4;
    n = (4000 * (l + 1)) / 1461001;
    l -= (1461 * n) / 4 - 31;
    *year += n;
    n = 80 * l / 2447;
    *day = l - (2447 * n) / 80;
    l = n / 11;
    *month = n + 2 - 12 * l;
    *year += l;
    return;
}


/*
 * The inverse of iday
 */

idate_ (doy, month, day, year)

int *doy, *month, *day, *year;
{
    idate (*doy, month, day, *year);
}


int
idate (doy, month, day, year)

register int doy, *month, *day, year;
{
    register int d;

    d = isleap (year);
    d = doy + 91 + (doy > 59 + d ? 2 - d : 0);
    doy = d / 30.55;
    *day = d - (int)(doy * 30.55);
    *month = doy - 2;
}


/* Gregorian leap year calculation */

int
isleap (yr)

int yr;
{
    if ((yr & 3) != 0)
	return (0);	/* not divisible by 4, not leap */
    if ((yr % 100) != 0)
	return (1);	/* not divisible by 100 but by 4, a leap */
    if ((yr % 400) != 0)
	return (0);	/* not divisible by 400 but by 100, not a leap */
    if ((yr % 4000) != 0)
	return (1);	/* not divisible by 4000, a leap */
    return (0);		/* divisible by 4000, not a leap */
}


isleap_ (yr)				/* Fortran-callable version */

int *yr;
{
    return (isleap (*yr));
}
