/* pcoord.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Polar <---> Cartesian Coordinate Transformation.
 * pcoord (radius, theta, x, y, isign, roll)
 * isign = 1 :  (radius,theta) --> (x,y).
 *       =-1 :  (x,y) --> (radius,theta).
 * Image coordinate origin (x=0,y=0): lower left corner.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   5 April 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include "/home/cordyn/mk4/src/mk4.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

pcoord (radius, theta, x, y, xdim, ydim, xcen, ycen, rspix, roll, isign)

float	*radius;	/* Radius (Rsun). */
float	*theta;		/* Position Angle (CCW degrees from solar north). */
float	*x, *y;		/* Cartesian coordinates:  (0,0)=lower left.*/
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
float	xcen ;		/* X-axis center. */
float	ycen ;		/* Y-axis center. */
float	pixrs ;		/* pixels/Rsun. */
float	roll;		/* Roll angle (degrees). */
int	isign;		/* =1: (r,th) --> (x,y). otherwise, vice-versa.*/

{
   int   ierr;			/* error indicator. */
   float degnor;		/* angular position of north w.r.t. +x axis. */
   float deltax, deltay;	/* x/y distance from sun center. */
   float xang;			/* angle in degrees w.r.t. +x axis. */
   double angle;		/* angle in radians. */
   double r2;			/* radius^2 */
   double degrad = 57.2995780;	/* degress/radian. */
   double raddeg = 0.01745329;	/* radians/degree. */

   degnor = 90.0 + (float) roll;/* Orientation of solar north. */
   ierr = 0;

   if (isign == 1)		/*** (radius,theta) --> (x,y) ***/
      {
      xang  = *theta + degnor;
      while (xang  > 360.0)  xang  -= 360.0;
      angle = (float) (raddeg * xang);
         *x = (float) (xcen + *radius * pixrs * cos (angle));
         *y = (float) (ycen + *radius * pixrs * sin (angle));

      if ((*x < 0 || *x >= xdim-1) || (*y < 0 || *y >= ydim-1))
         ierr = -1;
      }
   else				/*** (x,y) --> (r,theta) ***/
      {
      deltax = *x - xcen;
      deltay = *y - ycen;
      r2 = (double) (deltax * deltax + deltay * deltay);

      *radius = (float) (sqrt (r2) / pixrs);
      *theta  = (float) (atan2 (deltay, deltax) * degrad - (double) degnor);
      while (*theta < 0.0)  *theta += 360.0;
      }

   return (ierr);
}
