/* rdmk4.c
 *
 * Read a mk4 header and image data from the file "namimg".
 *
 * "img" is assumed to be a short array.
 */

#include <stdio.h>
#include "mk4.h"
#include "mk4head.h"
#include "crunch.h"
#include "telscp4.h"
#include "mk4fits.h"

/*** rdmk4head: Read MLSO/Mark-IV image header. ***/

int
rdmk4head (namimg, hd, doprint) 

    char *namimg; 
    struct mk4_header *hd; 
    int doprint;
{
    int ii, hdsz;
    FILE *fid;

    if (doprint > 0) fprintf (stderr, "\tReading header from: %s \n", namimg);

    /*** Open file. ***/

    if ( ( fid = fopen (namimg, "r") ) == NULL  )
	{
	fprintf (stderr, "Error opening %s for reading.\n", namimg);
	exit (-90);
        }

    /*** Read header. ***/

    hdsz = sizeof (mk4head);

    if ( ( ii = fread (hd, 1, hdsz, fid)) != hdsz )
	{
	fprintf (stderr, "Header read error:\n\
	    Only read %d bytes from %s\nExpected %d", ii, namimg, hdsz);
	fclose (fid);
	exit (-92);
        }

    /*** Fill FITS information structure. ***/

    mk4hdu (hd) ;

    fclose (fid);
}


/*** rdmk4: Read MLSO/Mark-IV image. ***/

int
rdmk4 (namimg, hd, img, doprint) 

    char *namimg; 
    struct mk4_header *hd; 
    short *img; 
    int doprint;
{
    int ii, imsz, hdsz;
    FILE *fid;

    if (doprint > 0) fprintf (stderr, "\tReading %s\n", namimg);

    /*** Open file. ***/

    if ( ( fid = fopen (namimg, "r") ) == NULL  )
	{
	fprintf (stderr, "Error opening %s for reading.\n", namimg);
	exit (-90);
        }

    /*** Read header. ***/

    hdsz = sizeof (mk4head);

    if ( ( ii = fread (hd, 1, hdsz , fid)) != hdsz )
	{
	fprintf (stderr, "Header read error:\n\
	Only read %d bytes from %s\nExpected %d", 
		        ii, namimg, hdsz);
	fclose (fid);
	exit (-92);
        }

    /*** Fill FITS information structure. ***/

    mk4hdu (hd) ;

    /*** Read image data. ***/

    imsz = (hd->Xdim) * (hd->Ydim);

    if ( ( ii = fread (img, sizeof (short), imsz, fid)) != imsz )
	{
	fprintf (stderr, "Image read error:\n\
	Only read %d shorts from %s\nExpected %d", 
		        ii, namimg, imsz);
	fclose (fid);
	exit (-94);
        }

    fclose (fid);

    if (doprint == 1)			/* Print out the sumry info. */
	{
	printf ("\t    File: %s\n", namimg);
	printf ("\t        %s %s \n",       prday (hd->Image.Juld), 
				      prtime(hd->Image.Time));

	switch (hd->DataType)
	    {  
	    case IS_i:  printf ("\t\tThis image is i data.\n") ; break;
	    case IS_q:  printf ("\t\tThis image is q data.\n") ; break;
	    case IS_u:  printf ("\t\tThis image is u data.\n") ; break;
	    case IS_ba: printf ("\t\tThis image is ba data.\n") ; break;
	    case IS_sk: printf ("\t\tThis image is sk data.\n") ; break;
	    case IS_st: printf ("\t\tThis image is st data.\n") ; break;
	    default:    printf ("\t\tThis image is UNKNOWN.\n") ;
	    } 
        }
}


