#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "mk4v2.h"

/* -------------------------------------------------------------------------- */

int FindCinI ( Img, FixedImg , Vec, xdim, ydim ,
	       sinth, costh, Theta)

    float *Img;
    float *FixedImg;
    float *Vec;      /* Free space for manipulations. */
    int   xdim,ydim;
    float *sinth;
    float *costh;
    double *Theta;
{
    int   ii;
    float *pImg, *pFix;
    float MeanSky ;

    VVcopy (Img,FixedImg,xdim*ydim);

    /* Get and subtract meansky */

    pFix     = FixedImg;
    for (ii=0; ii<ydim; ii++)
	{
	get_mean_fast (pFix, xdim   , &MeanSky);
	Vminus        (pFix, MeanSky, xdim);
	pFix += xdim;
        }

    SkyTrend       (FixedImg, Vec, xdim, ydim);
    MVminusReplace (FixedImg, Vec, xdim, ydim,  xdim);

    SinTrend       (FixedImg, Vec, xdim, ydim,  sinth, costh, Theta);

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int SkyTrend ( FixImg, Trend, xdim, ydim )

    float *FixImg;
    float *Trend;    /* On exit, this will contain the azimuthal SkyTrend  */
    int   xdim,ydim;
{
    int ii, jj;
    int StartPix, EndPix, Range;
    float *pF, *pT;
    float ovRange;
    float Mean, Sum;

    StartPix = 150;
    EndPix   = 250 + 1;
    Range    = EndPix - StartPix;
    ovRange  = 1.0 / ((float) Range);

    pT = Trend;
    for (ii=0; ii<xdim; ii++)
	{
	pF  = FixImg + StartPix * xdim + ii;
	Sum = 0.0;
	for (jj=StartPix; jj<EndPix; jj++)
	    {
	    Sum += *pF;
	    pF  += xdim;
	    }
	Mean = Sum * ovRange;
	*pT  = Mean;
	pT++;
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

int SinTrend (FixImg, Vec, xdim, ydim, sinth, costh, Theta)

    float  *FixImg;
    float  *Vec;      /* Free space for manipulations. */
    int    xdim,ydim;
    float  *sinth;
    float  *costh;
    double *Theta;
{
    int    ii, jj;
    int    pixStart, pixEnd;
    float  *pF, *pV, *pW;
    float  sprod,cprod;
    float  Trendi;
    double *pTh;
    double Sph, MyAng, Phi, Amp, Smean, Cmean, SumSq, mpiov4;
    double atan2 (), sqrt ();

    mpiov4   = -1.0 * piov2 * 0.5;
		     
    pixStart = 0;
    pixEnd   = ydim;
    
    pF    = FixImg + pixStart * xdim;
    for (jj=pixStart; jj<pixEnd; jj++)
	{

        pW = sinth;
        pV = Vec;
        VVmult        (pF, pW, pV, xdim);
        get_mean_fast (        pV, xdim, &sprod);
 
	pW = costh;
	pV = Vec;
	VVmult        (pF, pW, pV, xdim);
	get_mean_fast (        pV, xdim, &cprod);
 
	Smean = (double) sprod;
	Cmean = (double) cprod;
 
	SumSq = Smean*Smean + Cmean*Cmean;
 
	Phi = atan2 (Smean, Cmean);
	Amp = 2.0 * sqrt (SumSq);
 
	if ( Phi < 0.0 ) Phi += twopi ;

 
	pTh = Theta;
	for (ii=0; ii<xdim; ii++)
	    {
	    MyAng  = *pTh - Phi;
	    Sph    = cos (MyAng);
	    Trendi = (float) (Sph * Amp);
	    *pF   -= Trendi;

	    pTh++;
	    pF++;
	    }
 
        }

    return (0) ;
}

/* -------------------------------------------------------------------------- */

