/* Obtains the inverse of any square matrix.
   The matrix elements are computed across, and then down. */

#include <stdio.h>
#include <math.h>

int
InvMatrix( pm , pinvm , xsz,ysz)
    float *pm , *pinvm;
    int  xsz,ysz;
{
    float *TempMatrix;
    float *y,*col;
    float d;
    int i,j, *indx;
    void ludcmp();
    void lubksub();

    TempMatrix = (float  *)malloc(sizeof(float)*xsz*ysz) ;
    col        = (float  *)malloc(sizeof(float)*xsz    ) ;
    indx       = (int    *)malloc(sizeof(int  )*xsz    ) ;
    VVcopy(pm,TempMatrix,xsz*ysz);

    y = pinvm;

    ludcmp(TempMatrix,xsz,indx,&d);
    for(j=0;j<xsz;j++){
	for(i=0;i<xsz;i++) { *(col+i) = 0.; }
	*(col+j) = 1.;
	lubksub(TempMatrix,xsz,indx,col);
	for(i=0;i<xsz;i++) *(y+j*xsz+i) = *(col+i);
    }

     free(TempMatrix);
     free(col);
     free(indx);

}

/*  The routines below were taken from "Numerical Recipies in C".
    They have been substantially rewritten by Alice Lecinski,
    as the originals were in Fortran-ese and didn't work. 
*/

#define TINY 1.0e-20;

void ludcmp(a,n,indx,d)
float *a;
int   n;
int   *indx;
float *d;
{
	int i,imax,j,k;
	float big,dum,sum,temp;
	float *vv;
	void nrerror();

	vv=(float  *)malloc(sizeof(float)*n) ;

	*d=1.0;
	for (i=0;i<n;i++) {
		big=0.0;
		for (j=0;j<n;j++)
			if ((temp=fabs(*(a+j*n+i))) > big) big=temp;
		if (big == 0.0) {
		    fprintf(stderr,"Singular matrix in routine LUDCMP");
		}
		*(vv+i)=1.0/big;
	}
	for (j=0;j<n;j++) {
		for (i=0;i<j;i++) {
			sum= *(a+j*n+i);
			for (k=0;k<i;k++) sum -= *(a+k*n+i) * *(a+j*n+k);
			*(a+j*n+i) = sum;
		}
		big=0.0;
		for (i=j;i<n;i++) {
			sum= *(a+j*n+i);
			for (k=0;k<j;k++) {
			        sum -= *(a+k*n+i) *   *(a+j*n+k); 
			}
			*(a+j*n+i) = sum;
			if ( (dum= *(vv+i) * fabs(sum)) >= big) {
				big=dum;
				imax=i;
			}
		}
		if (j != imax) {
			for (k=0;k<n;k++) {
				dum = *(a+k*n+imax);
				*(a+k*n+imax) = *(a+k*n+j) ;
				*(a+k*n+j) = dum;
			}
			*d = -(*d);
			*(vv+imax) = *(vv+j) ;
		}
		*(indx+j)=imax;
		if ( *(a+j*n+j) == 0.0) *(a+j*n+j) = TINY;
		if (j != n-1) {
			dum=1.0/(*(a+j*n+j));
			for (i=j+1;i<n;i++) *(a+j*n+i) *= dum;
		}
	}
	free(vv);
}

#undef TINY


void lubksub(a,n,indx,b)
float *a;
int   n;
int   *indx;
float *b;
{
	int i,ii,ip,j;
	float sum;

	ii= -1;

	for (i=0;i<n;i++) {
		ip= *(indx+i);
		sum= *(b+ip);
		*(b+ip) = *(b+i);
		if (ii != -1){
			for (j=ii;j<=i-1;j++) sum -= *(a+j*n+i) * *(b+j);
		}
		else if (sum) ii=i;
		*(b+i) = sum;
	}
	for (i=n-1;i>=0;i--) {
		sum = *(b+i);
		for (j=i+1;j<n;j++) sum -= *(a+j*n+i)  *  *(b+j);
		*(b+i)  =  sum/ *(a+i*n+i) ;
	}
}
