/*
    Below are several simple utilities that find:
        minimum  of an array  (and where)
        maximum  of an array  (and where)
        median   of a  vector (and where)
        median2d of an array  (and where)
        mean and stddev of an array
        mean of an array
        convert float to short
        convert short to float
        total an array
        get an avg radial vector from a 2d array
        get an avg radial vector from a 2d array, culling points above and
            below one stddev.
        get a radial vector that is mean-stddev of a 2d array

	replace all values <= BadVal with GoodVal

	get a sub image from an image

    All arrays are expected to be floats, (except float to short
    and short to float).
    
    All the were written by Alice Lecinski.  (2000-2001)

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the 
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>
#include <math.h>
#include <ieeefp.h>

#define TRUE        (1) 
#define FALSE       (0) 

float  mm2d[50];  /* For fmedian2d */
float  mmmm[50],    mmmC[50];  /* For fmedian */

/* Gets the minimum of an array, and where it is. */
int
iget_min(img, imsz, mini, spot)
        int   *img;
        int   imsz;
        int   *mini;
        int   *spot;
{
    int  ii;
    register int *ip;
    
    *mini =  65535;
    *spot = -1;
    ip = img;
    for(ii=0;ii<imsz;ii++) { 
        if ( *ip <= *mini ) {
            *mini = *ip;
            *spot = ii ;
        }
        ip++; 
    }

    return (0);
}

/* Gets the minimum of an array, and where it is. */
int
get_min(img, imsz, mini, spot)
        float *img;
        int   imsz;
        float *mini;
        int   *spot;
{
    int  ii;
    register float *ip;
    
    *mini =  9999999.;
    *spot = -1;
    ip = img;
    for(ii=0;ii<imsz;ii++) { 
        if ( *ip <= *mini ) {
            *mini = *ip;
            *spot = ii ;
        }
        ip++; 
    }

    return (0);
}





/* Gets the maximum of an array, and where it is. */
int
get_max(img, imsz, maxi, spot)
        float *img;
        int imsz;
        float *maxi;
        int *spot;
{
    int  ii;
    register float *ip;
    
    *maxi = -9999999.;
    *spot = -1;
    ip = img;
    for(ii=0;ii<imsz;ii++) { 
        if ( *ip >= *maxi ) {
            *maxi = *ip;
            *spot = ii ;
        }
        ip++; 
    }

    return (0);
}


/* Gets the maximum of an array, and where it is. */
int
iget_max(img, imsz, maxi, spot)
        int *img;
        int imsz;
        int *maxi;
        int *spot;
{
    int  ii;
    register int *ip;
    
    *maxi = -65535;
    *spot = -1;
    ip = img;
    for(ii=0;ii<imsz;ii++) { 
        if ( *ip >= *maxi ) {
            *maxi = *ip;
            *spot = ii ;
        }
        ip++; 
    }

    return (0);
}






/* Gets the median for a floating point array. . */
/* This is really stupid and slow! */
int 
fmedian(xx, n, xmed, med_pos, doprint)
    float *xx; 
    int n;   /* The dimension of the vector */
    float *xmed;
    int   *med_pos;
    int doprint;
{
    float mini;
    float *px,*pm, *pmC;
    int ii, kk, min_pos, hsz;

    px =xx;
    pm =mmmm;
    pmC=mmmC;
    for(ii=0;ii<n;ii++) { *pm = *px; *pmC = *px ; px++; pm++; pmC++; }

    hsz = (n-1)>>1;
    get_min    (mmmm, n, &mini, &min_pos);
    for(ii=1;ii<=hsz;ii++){
        /* Delete the minimum from the array */
        pmC=mmmC;
        pm =mmmm;
        for(kk=0;kk<min_pos;  kk++) { *pm = *pmC; pmC++; pm++; }
        pmC++;
        for(kk=min_pos+1;kk<n;kk++) { *pm = *pmC; pmC++; pm++; }

        /* Recopy the array */
        pmC=mmmC;
        pm =mmmm;
        for(kk=0;kk<n-ii;  kk++) { *pmC = *pm; pmC++; pm++; }

        get_min    (mmmm, n-ii, &mini, &min_pos);
    }

    *xmed    = mini;
    *med_pos = min_pos;

    if(doprint==1) {
        fprintf(stderr,"ii %d  xmed %f med_pos %d\n", ii,*xmed, *med_pos);
        px=xx;
        for(ii=0;ii<n;ii++){ fprintf(stderr,"ii %d data %f\n",ii,*xx++); }
        fprintf(stderr,"\n\n");
    }

    return (0);
}

/* Gets the median for a integer array. . */
/* This is really stupid and slow! */
int 
imedian(xx, n, xmed, med_pos, doprint)
    int *xx; 
    int n;   /* The dimension of the vector */
    int *xmed;
    int   *med_pos;
    int doprint;
{
    int mini;
    int *px,*pm, *pmC;
    int ii, kk, min_pos, hsz;

    px =xx;
    pm =(int *)mmmm;
    pmC=(int *)mmmC;
    for(ii=0;ii<n;ii++) { *pm = *px; *pmC = *px ; px++; pm++; pmC++; }

    hsz = (n-1)>>1;
    iget_min    ((int *)mmmm, n, &mini, &min_pos);
    for(ii=1;ii<=hsz;ii++){
        /* Delete the minimum from the array */
        pmC=(int *)mmmC;
        pm =(int *)mmmm;
        for(kk=0;kk<min_pos;  kk++) { *pm = *pmC; pmC++; pm++; }
        pmC++;
        for(kk=min_pos+1;kk<n;kk++) { *pm = *pmC; pmC++; pm++; }

        /* Recopy the array */
        pmC=(int *)mmmC;
        pm =(int *)mmmm;
        for(kk=0;kk<n-ii;  kk++) { *pmC = *pm; pmC++; pm++; }

        iget_min    ((int *)mmmm, n-ii, &mini, &min_pos);
    }

    *xmed    = mini;
    *med_pos = min_pos;

    if(doprint==1) {
        fprintf(stderr,"ii %d  xmed %d med_pos %d\n", ii,*xmed, *med_pos);
        px=xx;
        for(ii=0;ii<n;ii++){ fprintf(stderr,"ii %d data %d\n",ii,*xx++); }
        fprintf(stderr,"\n\n");
    }

    return (0);
}





int get_mean(img, imsz, mean, stddev)
    float *img;
    int imsz;
    float *mean;
    float *stddev;
{
    int ii;
    double sumX, sumXsq, dtmp, num;
    double ovnum, ovnumMm1;

    sumX=sumXsq=0.;
    num = (double)(imsz);
    ovnum    = 1./num;
    ovnumMm1 = 1./(num*(num-1.));

    for(ii=0;ii<imsz;ii++) {
        dtmp    = (double)*img;
        sumX   += dtmp;
        sumXsq += dtmp * dtmp;
        img++;
    }
 
    *mean   =  (float)(sumX * ovnum) ;
    dtmp    = (num * sumXsq  -  sumX*sumX ) * ovnumMm1 ;
    *stddev = (float)(sqrt(dtmp));
 
    return (0);
}

int get_mean_fast(img, imsz, mean)
    float *img;
    int    imsz;
    float *mean; 
{
    int ii;
    double sumX,  num;
    double dtmp,  ovnum;
 
    sumX  = 0.;
    num   = (double)(imsz);
    ovnum = 1./num;
 
    for(ii=0;ii<imsz;ii++) {
        dtmp    = (double)*img;
        sumX   += dtmp;
        img++;
    }
 
    *mean   =  (float)(sumX * ovnum) ;
 
    return (0);
}


int Dget_mean(img, imsz, mean, stddev)
    double *img;
    int imsz;
    double *mean;
    double *stddev;
{
    int ii;
    double sumX, sumXsq, dtmp, num;
    double ovnum, ovnumMm1;

    sumX=sumXsq=0.;
    num = (double)(imsz);
    ovnum    = 1./num;
    ovnumMm1 = 1./(num*(num-1.));

    for(ii=0;ii<imsz;ii++) {
        dtmp    = *img;
        sumX   += dtmp;
        sumXsq += dtmp * dtmp;
        img++;
    }
 
    *mean   = (sumX * ovnum) ;
    dtmp    = (num * sumXsq  -  sumX*sumX ) * ovnumMm1 ;
    *stddev = (sqrt(dtmp));
 
    return (0);
}

int Dget_mean_fast(img, imsz, mean)
    double *img;
    int    imsz;
    double *mean; 
{
    int ii;
    double sumX,  num;
    double dtmp,  ovnum;
 
    sumX  = 0.;
    num   = (double)(imsz);
    ovnum = 1./num;
 
    for(ii=0;ii<imsz;ii++) {
        dtmp    = *img;
        sumX   += dtmp;
        img++;
    }
 
    *mean   =  (sumX * ovnum) ;
 
    return (0);
}


int Float2Short(Floaty,Shorty,numx,numy)
    float *Floaty;
    short *Shorty;
    int numx;
    int numy;
{
    int ii, sz;
    short *pS;
    short stemp;
    float *pF;

    sz=numx*numy;

    pF=Floaty;
    pS=Shorty;
    for(ii=0;ii<sz;ii++){
        stemp = (short) (*pF);
        *pS   = stemp;
        pS++;
        pF++;
    }
}

int Short2Float(Shorty,Floaty,numx,numy)
    short *Shorty;
    float *Floaty;
    int numx;
    int numy;
{
    int ii, sz;
    short *pS;
    float *pF;
    float ftemp;

    sz=numx*numy;

    pF=Floaty;
    pS=Shorty;
    for(ii=0;ii<sz;ii++){
        ftemp = (float) (*pS);
        *pF   = ftemp;
        pS++;
        pF++;
    }
}

int Total(Vect,sz,sum)
    float *Vect;
    int   sz;
    float *sum;
{
    int ii;
    float *pV;

    *sum=0.;
    pV = Vect;
    for (ii=0;ii<sz;ii++){
        *sum += *pV;
        pV++;
    }
}

int
Im2AvgRadVect( Img, Vect,  numx, numy)
    float *Img;
    float *Vect;
    int   numx;
    int   numy;
{
    int ii,jj;
    float *pI, *pV, ftemp, ovnumX;

    pI = Img;
    pV = Vect;
    ovnumX = 1.0 / (float)numx ;

    for(jj=0;jj<numy;jj++){
        ftemp = 0.;
        for(ii=0;ii<numx;ii++){
            ftemp += *pI;
            pI++;
        }
        ftemp *= ovnumX;
        *pV    = ftemp;
        pV++ ;
    }

}

int
Im2AvgRadVect_Culled( Img, Vect,  numx, numy)
    float *Img;
    float *Vect;
    int   numx;
    int   numy;
{
    int ii,jj;
    float *pI, *pV, ovnumX , ftemp, fsum, value, mini, maxi, mean, stddev;

    pI = Img;
    pV = Vect;
    ovnumX = 1.0 / (float)numx ;

    for(jj=0;jj<numy;jj++){
        get_mean(pI , numx , &mean , &stddev );
        mini = mean - stddev;
        maxi = mean + stddev;
        fsum = 0.;
        for(ii=0;ii<numx;ii++){
            ftemp  = *pI;
            value  = (ftemp < mini) ? mini  : (ftemp > maxi) ? maxi : ftemp ;
            fsum  += value;
            pI++;
        }
        fsum *= ovnumX;
        *pV   = fsum;
        pV++ ;
    }

}

/* This writes over the image with a filtered version, so send it a copy
   of your image if you don't want it mangled. */
int
get_min_Vect( Img, Vect,  numx, numy)
    float *Img;
    float *Vect;
    int   numx;
    int   numy;
{
    int jj, spot, wrap;
    float *pI, *pV, mini;
    float mean, stddev;

    wrap=1;

    pI = Img;
    pV = Vect;

    for(jj=0;jj<numy;jj++){
        /*
        This is the part that writes over the image.
        If it isn't used -- no worries!
        LeeFilt (pI, numx, 5, wrap);
        get_min (pI, numx,    &mini , &spot);
        */
        /* This does not write over your image. */
        get_mean(pI, numx, &mean , &stddev);

        mini = mean - stddev;

        *pV = mini;
        pV++ ;
        pI += numx;
    }

}


int
ReplaceLE ( AnImg , xdim , ydim , BadVal, GoodVal, doprint)
    float *AnImg;
    int   xdim,ydim;
    float BadVal, GoodVal;
    int   doprint;
{
    int ii, imsz;
    float *pf, ftemp;

    if(doprint==1)
    fprintf(stderr,"\t\tReplacing data <= %f with %f\n",BadVal, GoodVal );

    imsz = xdim*ydim;
    pf   = AnImg;
    for(ii=0;ii<imsz;ii++){
	ftemp = ( *pf <= BadVal ) ? GoodVal : *pf;
	*pf = ftemp;
	pf++;
    }
}

int
ReplaceGE ( AnImg , xdim , ydim , BadVal, GoodVal, doprint)
    float *AnImg;
    int   xdim,ydim;
    float BadVal, GoodVal;
    int   doprint;
{
    int ii, imsz;
    float *pf, ftemp;

    if(doprint==1)
    fprintf(stderr,"\t\tReplacing data >= %f with %f\n",BadVal, GoodVal );

    imsz = xdim*ydim;
    pf   = AnImg;
    for(ii=0;ii<imsz;ii++){
	ftemp = ( *pf >= BadVal ) ? GoodVal : *pf;
	*pf = ftemp;
	pf++;
    }
}

int
ReplaceNaN ( AnImg , xdim , ydim , GoodVal, doprint)
    float *AnImg;
    int   xdim,ydim;
    float GoodVal;
    int   doprint;
{
    int ii, imsz, isbad;
    float *pf, ftemp;

    if(doprint==1)
    fprintf(stderr,"\t\tReplacing NaN data with %f\n", GoodVal );

    imsz = xdim*ydim;
    pf   = AnImg;
    for(ii=0;ii<imsz;ii++){
	ftemp = *pf;
	isbad = isnanf(ftemp);
	if (isbad==TRUE)  ftemp = GoodVal;
	else              ftemp =     *pf;
	*pf = ftemp;
	pf++;
    }
}

int
ReplaceNonFinite ( AnImg , xdim , ydim , GoodVal, doprint)
    float *AnImg;
    int   xdim,ydim;
    float GoodVal;
    int   doprint;
{
    int ii, imsz, isgood;
    float *pf, ftemp;
    double  dtemp;

    if(doprint==1)
    fprintf(stderr,"\t\tReplacing NonFinite data with %f\n", GoodVal );

    imsz = xdim*ydim;
    pf   = AnImg;
    for(ii=0;ii<imsz;ii++){
	dtemp  = (float)(*pf);
	isgood = finite(dtemp);
	if (isgood==FALSE)  ftemp = GoodVal;
	else                ftemp =     *pf;
	*pf = ftemp;
	pf++;
    }
}



/* This routine gets the signal present in the 1st
   dimension based on the average of a number of pixels
   in the 2nd dimension.
*/
int GetSignal(An_array,An_avg,lp,hp,xdim,ydim)
    float *An_array;
    float *An_avg;
    int lp, hp;
    int xdim,ydim;
{
    int ii,jj;
    double ovPixrange,mytotal,dtmp;
    float *pavg,*pA, *pAStart;

    ovPixrange = 1./((double)(1+hp-lp));

    pavg    = An_avg;
    pAStart = An_array + lp*xdim;
    for(ii=0;ii<xdim;ii++) {
	pA = pAStart + ii;
	mytotal = 0.;
	for(jj=lp;jj<=hp;jj++){
	    dtmp     = (double)(*pA);
	    mytotal += dtmp;
	    pA += xdim;
	}
	*pavg = (float)(mytotal * ovPixrange);
	pavg++;
    }
}

int
GetSubImgf( Img, Sub, xdim, ydim, subxdim, subydim , xstart, ystart)
    float *Img;
    float *Sub;
    int xdim,ydim,subxdim,subydim,xstart,ystart;
{
    int ii,jj;
    int spot;
    float *pI, *pS;

    if((xdim*(ystart+subydim) + subxdim) > xdim*ydim-1) {
	fprintf(stderr,"GetSubImgf: array bounds overrun, exitting...");
	exit(-10);
    }

    pI = Img + xdim*ystart + xstart;
    pS = Sub;
    for(jj=0;jj<subydim;jj++){
	for(ii=0;ii<subxdim;ii++){
	    *pS = *pI;
	    pS++;
	    pI++;
	}
	pI -= subxdim;
	pI += xdim;
    }
}
