/* mk4hduv2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill MLSO/Mk4 FITS structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 August 1999
 * 10 Sep 1999: Add tscpfile.
 *  7 Mar 2000: Modified for revised Mk4 calibration.
 *  8 Mar 2000: Add reffile & drftfile.
 * 10 Mar 2000: Add xscale & yscale.
 * 15 Mar 2000: Modify bscale & bzero settings.
 * 19 Oct 2001: Add new items for Mk4 version 2 calibration.
 *  1 Nov 2001: MK4F->nbypx = abs (hd->ImgDataType) / 8.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "mk4v2.h"
#include "mk4headv2.h"
#include "mk4fitsv2.h"
#include "mk4InsDesc.h"
#define HIZERO NPX - 11

mk4hdu (hd, pmode)

struct mk4_header *hd ;
int pmode ;
{
   float rmin, rmax ;

   /*** Fill Mk4 FITS structure with appropriate information. ***/

   MK4F->dtype  = hd->DataType ;/* IS_i, IS_q, IS_u, IS_ba, IS_sk, IS_st. */
   MK4F->otype  = hd->ObsType ;	/* IS_cal0, IS_cal15, IS_dark, IS_corona. */
   MK4F->ptype  = hd->ProcSteps ;	/* IS_raw..., IS_cal..., IS_UNKNOWN */
   MK4F->xdim   = hd->Xdim ;	/* Horizontal dimension. */
   MK4F->ydim   = hd->Ydim ;	/* Vertical   dimension. */

/*
 *   if (MK4F->ptype == IS_cal_rect_byte)
 *      MK4F->nbypx = 1 ;
 *   else
 *      MK4F->nbypx = 2 ;
 */
   MK4F->nbypx = abs (hd->ImgDataType) / 8 ;	/* Bytes/pixel. */

   MK4F->dmin   = 0 ;			/* Data    minimum value. */
   MK4F->dmax   = 0 ;			/* Data    maximum value. */
   MK4F->smin   = 0 ;			/* Display minimum value. */
   MK4F->smax   = 0 ;			/* Display maximum value. */
   MK4F->year   = hd->Image.Year ;	/* Image year.*/
   MK4F->month  = hd->Image.Month ;	/* Image month. */
   MK4F->day    = hd->Image.Day ;	/* Image day-of-month. */
   MK4F->doy    = hd->Image.Doy ;	/* Image day-of-year.*/
   MK4F->hour   = hd->Image.Hour ;	/* Image hour. */
   MK4F->minute = hd->Image.Minute ;	/* Image minute. */
   MK4F->second = hd->Image.CentiSec / 100 ;	  /* Image second. */
   MK4F->cal_year   = hd->Date_calibrated.Year ;  /* Calibration year.*/
   MK4F->cal_month  = hd->Date_calibrated.Month ; /* Calibration month. */
   MK4F->cal_day    = hd->Date_calibrated.Day ;	  /* Calibration day-of-month.*/
   strncpy (MK4F->tscpfile, hd->tscpfile,  64) ;  /* Telescope info file name.*/
   strncpy (MK4F->reffile,  hd->reffile,   64) ;  /* Reference file name.*/
   strncpy (MK4F->cversion, hd->version,   64) ;  /* Mk4 calibration version.*/

   /*** Set brightness scaling factor. ***/

/*
 *   if (hd->data_scale == -9999.0)
 *      MK4F->bscale = 1.0;
 *   else
 *      MK4F->bscale = 1.0 / hd->data_scale ;
 *
 *   if (hd->data_offset == -9999.0)
 *      MK4F->bzero  = 0.0 ;
 *   else
 *      MK4F->bzero  = - hd->data_offset * MK4F->bscale ;
 */

   switch (MK4F->dtype)
      {
      case IS_i  :
      case IS_q  :
      case IS_u  :
         switch (MK4F->ptype)
            {
	    case IS_raw          :
	    case IS_raw_spln     :
	    case IS_raw_rect     :
	    case IS_raw_avg      :
	    case IS_raw_avg_spln :
	    case IS_raw_avg_rect :

	       			MK4F->bscale = 1.0 ;		/* CCD units */
				MK4F->bzero  = 0.0 ;
				break ;
	    case IS_cal :
	    case IS_cal_rect :
	    case IS_cal_rect_byte :
	    case IS_cal_avg :
	    case IS_cal_avg_rect :

			MK4F->bscale = 1.0 / hd->data_scale ;
			MK4F->bzero  = - hd->data_offset * MK4F->bscale ;
/*		        MK4F->bzero  = 0.0 ; */
			break ;

	    case IS_cal_slit :
	    case IS_cal_rect_slit :
	    case IS_cal_avg_slit :
	    case IS_cal_avg_rect_slit:

			MK4F->bscale =  1.0e-10 ;	/* Bsun units */
			MK4F->bzero  = -1.0e-7 ;
			MK4F->bscale = 1.0 ;
		        MK4F->bzero  = 0.0 ;
			break ;

            default : MK4F->bscale = 1.0 ;
		      MK4F->bzero  = 0.0 ;
            }	break ;

      case IS_ba : MK4F->bscale = 1.0 ;		/* degrees */
		   MK4F->bzero  = 0.0 ;
		   break ;

      case IS_sk : MK4F->bscale = 1.0 ;		/* sky brightness */
		   MK4F->bzero  = 0.0 ;
		   break ;

      case IS_st : MK4F->bscale = 1.0 ;		/* scan times */
		   MK4F->bzero  = 0.0 ;
		   break ;

      default    : MK4F->bscale = 1.0 ;
		   MK4F->bzero  = 0.0 ;
      }

   if (pmode >= 2)
   fprintf (stdout, "mk4hdu> ptype: %d  bscale: %e\n",
	    MK4F->ptype, MK4F->bscale) ;

   MK4F->scndir = hd->Direction ;	/* +1 => CCW; -1 => CW. */

   if (pmode >= 3)
   fprintf (stdout, "mk4hdu> stop1\n") ;
   fflush  (stdout) ;


   /*** Scan duration (seconds). ***/

   MK4F->scndur = (float) (hd->Duration & 0177777) / 60.0 ;

   if (pmode >= 3)
   fprintf (stdout, "mk4hdu> stop2\n") ;
   fflush  (stdout) ;

   switch (MK4F->dtype)
      {
      case IS_i  :
      case IS_q  :
      case IS_u  :
         switch (MK4F->ptype)
	    {
	    /* Polar (cylindrical) coordinates. */

            case IS_raw          :
            case IS_raw_avg      :
	    case IS_raw_avg_spln :
	    case IS_cal          :
            case IS_cal_slit     :
	    case IS_cal_avg      :
	    case IS_cal_avg_slit :

                MK4F->xscale = hd->Xdim / (hd->theta_max - hd->theta_min) ;
                MK4F->yscale = hd->Radius / (hd->ID.dr * 960.0) ;
		MK4F->pixrs  = hd->Radius / (hd->ID.dr * 960.0) ;
		MK4F->xcen   = 0.0 ;	/* Not applicable. */
		if (hd->theta_max > 0.0)
		 MK4F->xcen = (MK4F->xdim) - (hd->theta_max * MK4F->xscale) ;
		MK4F->ycen  = -(hd->r0 * MK4F->pixrs) ;/* Sun center (pixels) */
		break ;

	    /* Rectangular coordinates. */

	    case IS_raw_rect          :
	    case IS_cal_rect          :
	    case IS_cal_rect_slit     :
	    case IS_cal_rect_byte     :
	    case IS_cal_avg_rect      :
	    case IS_cal_avg_rect_slit :
	    case IS_raw_avg_rect      :
            default :

                MK4F->xscale = hd->pixscl ;	/* # pixels / Rsun */
                MK4F->yscale = hd->pixscl ;	/* # pixels / Rsun */
		MK4F->pixrs  = hd->pixscl ;	/* # pixels / Rsun */
		MK4F->xcen   = hd->Xcen ;	/* X-axis sun center. */
		MK4F->ycen   = hd->Ycen ;	/* Y-axis sun center. */
            }   break ;
      }

   if (pmode >= 2)
      {
      printf ("MK4F->xscale: %f\n", MK4F->xscale) ;
      printf ("MK4F->yscale: %f\n", MK4F->yscale) ;
      }

   if (pmode >= 3)
   fprintf (stdout, "mk4hdu> stop3\n") ;
   fflush  (stdout) ;

   if (pmode >= 2)
   printf ("r0: %f  pixrs: %f  xcen: %f\n", hd->r0, MK4F->pixrs, MK4F->xcen) ;
   fflush  (stdout) ;

   if (pmode >= 3)
   fprintf (stdout, "mk4hdu> stop4\n") ;
   fflush  (stdout) ;

   MK4F->phase  = hd->Phase ;	/* Mk4 offset angle w.r.t. spar. */
   MK4F->pangle = hd->Pangle ;	/* Solar P-angle [degrees]. */
   MK4F->bangle = hd->Bangle ;	/* Solar B-angle [degrees]. */
   MK4F->rsun   = hd->Radius ;	/* Solar Radius  [arcsec].  */
   MK4F->nangle = 0.0 ;		/* Angle (CCW) of north w.r.t. +Y axis. */

   if (pmode >= 3)
   fprintf (stdout, "mk4hdu> stop5\n") ;
   fflush  (stdout) ;

   /*** Inner/Outer FOV radii. ***/
   /*** rmin = inner radius.   rmax = outer radius. ***/

   MK4F->rmin = hd->ID.goodRmin ;
   MK4F->rmax = hd->ID.goodRmax ;

   /*** Start/Stop azimuth ***/

   MK4F->tmin   = hd->theta_min ;	/* Begin barrel angle [degrees]. */
   MK4F->tmax   = hd->theta_max ;	/* End   barrel angle [degrees]. */

   if (pmode >= 3)
   fprintf (stdout, "mk4hdu> stop6\n") ;
   fflush  (stdout) ;

   /*** Mk4 binary header contents. ***/

   if (pmode >= 1)
   {
   fprintf (stdout, "hd->Image.Year:            %d\n", hd->Image.Year) ;
   fflush (stdout) ;
   fprintf (stdout, "hd->Image.Month:           %d\n", hd->Image.Month) ;
   fprintf (stdout, "hd->Image.Day:             %d\n", hd->Image.Day) ;
   fprintf (stdout, "hd->Image.Hour:            %d\n", hd->Image.Hour) ;
   fprintf (stdout, "hd->Image.Minute:          %d\n", hd->Image.Minute) ;
   fprintf (stdout, "hd->Image.CentiSec:        %d\n", hd->Image.CentiSec) ;
   fprintf (stdout, "hd->Image.Juld:            %d\n", hd->Image.Juld) ;
   fprintf (stdout, "hd->Image.Doy:             %d\n", hd->Image.Doy) ;
   fprintf (stdout, "hd->Image.Time:            %d\n", hd->Image.Time) ;
   fprintf (stdout, "hd->Xdim:                  %d\n", hd->Xdim) ;
   fprintf (stdout, "hd->Ydim:                  %d\n", hd->Ydim) ;
   fprintf (stdout, "hd->Duration:              %d\n", hd->Duration) ;
   fprintf (stdout, "hd->JuldRef:               %d\n", hd->JuldRef) ;
   fprintf (stdout, "hd->ImgDataType:           %d\n", hd->ImgDataType) ;
   fprintf (stdout, "hd->Fill1 [0]:             %d\n", hd->Fill1 [0]) ;
   fprintf (stdout, "hd->Fill1 [1]:             %d\n", hd->Fill1 [1]) ;
   fprintf (stdout, "hd->Pangle:                %f\n", hd->Pangle) ;
   fprintf (stdout, "hd->Bangle:                %f\n", hd->Bangle) ;
   fprintf (stdout, "hd->Langle:                %f\n", hd->Langle) ;
   fprintf (stdout, "hd->RA:                    %f\n", hd->RA) ;
   fprintf (stdout, "hd->Dec:                   %f\n", hd->Dec) ;
   fprintf (stdout, "hd->HA:                    %f\n", hd->HA) ;
   fprintf (stdout, "hd->Elevation:             %f\n", hd->Elevation) ;
   fprintf (stdout, "hd->Azimuth:               %f\n", hd->Azimuth) ;
   fprintf (stdout, "hd->Amass:                 %f\n", hd->Amass) ;
   fprintf (stdout, "hd->Radius:                %f\n", hd->Radius) ;
   fprintf (stdout, "hd->SkyTx:                 %f\n", hd->SkyTx) ;
   fprintf (stdout, "hd->InstrPhs:              %f\n", hd->InstrPhs) ;
   fprintf (stdout, "hd->spare1:                %f\n", hd->spare1) ;
   fprintf (stdout, "hd->data_scale:            %f\n", hd->data_scale) ;
   fprintf (stdout, "hd->data_offset:           %f\n", hd->data_offset) ;
   fprintf (stdout, "hd->spare2:                %f\n", hd->spare2) ;
   fprintf (stdout, "hd->orighairspot [0]:      %f\n", hd->orig_hairspot [0]) ;
   fprintf (stdout, "hd->orighairspot [1]:      %f\n", hd->orig_hairspot [1]) ;
   fprintf (stdout, "hd->hairspot [0]:          %f\n", hd->hairspot [0]) ;
   fprintf (stdout, "hd->hairspot [1]:          %f\n", hd->hairspot [1]) ;

   fprintf (stdout, "hd->Xcen:                  %f\n", hd->Xcen) ;
   fprintf (stdout, "hd->Ycen:                  %f\n", hd->Ycen) ;
   fprintf (stdout, "hd->pixscl:                %f\n", hd->pixscl) ;
   fprintf (stdout, "hd->r0:                    %f\n", hd->r0) ;
   fprintf (stdout, "hd->dr:                    %f\n", hd->dr) ;
   fprintf (stdout, "hd->rmax:                  %f\n", hd->rmax) ;
   fprintf (stdout, "hd->theta_min:             %f\n", hd->theta_min) ;
   fprintf (stdout, "hd->theta_max:             %f\n", hd->theta_max) ;
   fprintf (stdout, "hd->raw_theta_min:         %f\n", hd->raw_theta_min) ;
   fprintf (stdout, "hd->raw_theta_max:         %f\n", hd->raw_theta_max) ;
   fprintf (stdout, "hd->backlash:              %f\n", hd->backlash) ;
   fprintf (stdout, "hd->Phase:                 %f\n", hd->Phase) ;
   fprintf (stdout, "hd->ObsType:               %d\n", hd->ObsType) ;
   fprintf (stdout, "hd->DataType:              %d\n", hd->DataType) ;
   fprintf (stdout, "hd->ProcSteps:             %d\n", hd->ProcSteps) ;
   fprintf (stdout, "hd->Direction:             %d\n", hd->Direction) ;
   fprintf (stdout, "hd->Nray:                  %d\n", hd->Nray) ;
   fprintf (stdout, "hd->Pixels:                %d\n", hd->Pixels) ;
   fprintf (stdout, "hd->Fill2 [0]:             %d\n", hd->Fill2 [0]) ;
   fprintf (stdout, "hd->Date_calibrated.Year:  %d\n", hd->Date_calibrated.Year) ;
   fprintf (stdout, "hd->Date_calibrated.Month: %d\n", hd->Date_calibrated.Month) ;
   fprintf (stdout, "hd->Date_calibrated.Day:   %d\n", hd->Date_calibrated.Day) ;
   fprintf (stdout, "hd->Date_calibrated.Hour:  %d\n", hd->Date_calibrated.Hour) ;
   fprintf (stdout, "hd->Date_calibrated.Minute:%d\n", hd->Date_calibrated.Minute) ;
   fprintf (stdout, "hd->Date_calibrated.CentiSec:%d\n", hd->Date_calibrated.CentiSec) ;
   fprintf (stdout, "hd->Date_calibrated.Juld:  %d\n", hd->Date_calibrated.Juld) ;
   fprintf (stdout, "hd->Date_calibrated.Doy:   %d\n", hd->Date_calibrated.Doy) ;
   fprintf (stdout, "hd->Date_calibrated.Time:  %d\n", hd->Date_calibrated.Time) ;
   fprintf (stdout, "hd->tscpfile:              %s\n", hd->tscpfile) ;
   fprintf (stdout, "hd->reffile:               %s\n", hd->reffile) ;
   fprintf (stdout, "hd->version:               %s\n", hd->version) ;
   fprintf (stdout, "hd->AvgTimes:              %s\n", hd->AvgTimes) ;

   fprintf (stdout, "hd->ID.JuldRange [0]:      %f\n", hd->ID.JuldRange [0]) ;
   fprintf (stdout, "hd->ID.JuldRange [1]:      %f\n", hd->ID.JuldRange [1]) ;
   fprintf (stdout, "hd->ID.Bopal:              %f\n", hd->ID.Bopal) ;
   fprintf (stdout, "hd->ID.trt:                %f\n", hd->ID.trt) ;
   fprintf (stdout, "hd->ID.fpt:                %f\n", hd->ID.fpt) ;
   fprintf (stdout, "hd->ID.IDspare:            %f\n", hd->ID.IDspare) ;
   fprintf (stdout, "hd->ID.kappa:              %f\n", hd->ID.kappa) ;
   fprintf (stdout, "hd->ID.BarrelOffset:       %f\n", hd->ID.BarrelOffset) ;
   fprintf (stdout, "hd->ID.hairspot [0]:       %f\n", hd->ID.hairspot [0]) ;
   fprintf (stdout, "hd->ID.hairspot [1]:       %f\n", hd->ID.hairspot [1]) ;
   fprintf (stdout, "hd->ID.r0:                 %f\n", hd->ID.r0) ;
   fprintf (stdout, "hd->ID.dr:                 %f\n", hd->ID.dr) ;
   fprintf (stdout, "hd->ID.goodRmin:           %f\n", hd->ID.goodRmin) ;
   fprintf (stdout, "hd->ID.goodRmax:           %f\n", hd->ID.goodRmax) ;
   fprintf (stdout, "hd->ID.biasspot [0]:       %f\n", hd->ID.biasspot [0]) ;
   fprintf (stdout, "hd->ID.biasspot [1]:       %f\n", hd->ID.biasspot [1]) ;
   fprintf (stdout, "hd->ID.ZeroSky:            %f\n", hd->ID.ZeroSky) ;
   fprintf (stdout, "hd->ID.VenusCalFac:        %f\n", hd->ID.VenusCalFac) ;
   fprintf (stdout, "hd->ID.ExtraCalFac:        %f\n", hd->ID.ExtraCalFac) ;
   fprintf (stdout, "hd->ID.DiffractFac:        %f\n", hd->ID.DiffractFac) ;
   fprintf (stdout, "hd->ID.PhiPix:             %d\n", hd->ID.PhiPix) ;
   fprintf (stdout, "hd->ID.PolarityCkPix:      %d\n", hd->ID.PolarityCkPix) ;
   fprintf (stdout, "hd->ID.NegPix [0]:         %d\n", hd->ID.NegPix [0]) ;
   fprintf (stdout, "hd->ID.NegPix [1]:         %d\n", hd->ID.NegPix [1]) ;
   fprintf (stdout, "hd->ID.WrapPix [0]:        %d\n", hd->ID.WrapPix [0]) ;
   fprintf (stdout, "hd->ID.WrapPix [1]:        %d\n", hd->ID.WrapPix [1]) ;
   fprintf (stdout, "hd->ID.RingSearch [0]:     %d\n", hd->ID.RingSearch [0]) ;
   fprintf (stdout, "hd->ID.RingSearch [1]:     %d\n", hd->ID.RingSearch [1]) ;
   fprintf (stdout, "hd->ID.RingFixPix:         %d\n", hd->ID.RingFixPix) ;
   fprintf (stdout, "hd->ID.CorrectFile0:       %s\n", hd->ID.CorrectFile0) ;
   fprintf (stdout, "hd->ID.CorrectFile1:       %s\n", hd->ID.CorrectFile1) ;
/*
 *   fflush (stdout) ;
 */
   }

   return (0) ;
}
