/* mk4namev2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create file name for Mk4 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Author:  Alice Lecinski   HAO/NCAR   dofnm4.c
 * Updates: Andrew Stanger   HAO/NCAR   mk4name.c
 * 28 Nov 2000 : Replace "slit" with "vig".
 *  8 Nov 2001 : version 2.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <ctype.h>

#include "mk4v2.h"
#include "mk4headv2.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

char *
mk4name (hd)

struct mk4_header *hd ;
{
    static char file [80] ;
    static char months[] = "janfebmaraprmayjunjulaugsepoctnovdec" ;
    char *fp ;

    int howfar ;
    int who ;

    fp = file ;
    who    = hd->DataType ;
    howfar = hd->ProcSteps ;

    printf ("mk4name> howfar: %d  who: %d\n", howfar, who) ;
    printf ("mk4name> Image.Juld: %d\n", hd->Image.Juld) ;

    if ((hd->ProcSteps >= IS_cal_avg) &&
	(hd->ProcSteps <= IS_cal_avg_rect_slit))
	timeflag = 0 ;

    printf ("mk4name> dayflag:  %d\n", dayflag) ;
    printf ("mk4name> doyflag:  %d\n", doyflag) ;
    printf ("mk4name> timeflag: %d\n", timeflag) ;
    printf ("mk4name> Secflag:  %d\n", Secflag) ;

    if (dayflag)
	{
	jdate (hd->Image.Juld + 2415021, &mon, &day, &year) ;
	printf ("mk4name> mon: %d  day: %d  year: %d\n", mon, day, year) ;

	if (!doyflag)
	    {
	    sprintf (fp, "%04d%02d%02d.", year, mon, day) ;
/*
 *	    sprintf (fp, "%02d%3.3s%02d.",
 *				       (year-1900>=100)?year-2000:year-1900,
 *				       months+(mon-1)*3,day) ;
 *	    fp += 8 ; 
 */
	    fp += 9 ;
	    }
	else
	    {
	    doy = get_doy (mon, day, year) ;
	    sprintf (fp, "%02dd%03d.",
				   (year-1900>=100)?year-2000:year-1900,
				   doy) ;
	    fp += 7 ; 
	    }
	}

    if (timeflag && !Secflag )
	{
/*
 *	sprintf (fp,"%5.5s.", prtime (hd->Image.Time)) ;
 *	fp += 6 ;
 */
	sprintf (fp, "%02d%02d.", hd->Image.Hour, hd->Image.Minute) ;
	fp += 5 ;
	}

    if (timeflag && Secflag )
	{
/*
 *	sprintf (fp, "%8.8s.", prtime (hd->Image.Time)) ;
 *	fp += 9 ;
 */
	sprintf (fp, "%02d%02d%02d.",
		 hd->Image.Hour, hd->Image.Minute, hd->Image.CentiSec / 100) ;
	fp += 7 ;
	}
     
    /*** Add the "mk4" for ALL file types. ***/

    *fp++ = 'm' ;
    *fp++ = 'k' ;
    *fp++ = '4' ;
    *fp++ = '.' ;


    /* Have to go through "howfar" twice since some happens at the front
     * of the filename, and some at the end...
     */

    switch (howfar)
	{
	case IS_raw:
	case IS_raw_avg:
				switch (who)
				    {
				    case IS_i:
				    case IS_q:
				    case IS_u:
						*fp++ = 'c';	break;
				    default:    ;
				    }
	case IS_cal:
	case IS_cal_slit:
	case IS_cal_avg:
	case IS_cal_avg_slit:
					*fp++ = 'c';	break;

	case IS_raw_spln:		*fp++ = 'C';	break;

	case IS_raw_rect:
	case IS_cal_rect:
	case IS_cal_rect_slit:
	case IS_cal_rect_byte:
	case IS_cal_avg_rect:
	case IS_cal_avg_rect_slit:
	case IS_raw_avg_rect:
					*fp++ = 'r';	break;

	default:   ;
        }

    switch (howfar)
	{
	case IS_raw:
	case IS_raw_spln:
	case IS_raw_rect:
	    switch (who)
		{
		case IS_i:		*fp++ = 'i' ;	break;

		case IS_q:		*fp++ = 'q' ;	break;

		case IS_u:		*fp++ = 'u' ;	break;

		case IS_ba:		*fp++ = 'b';
					*fp++ = 'a';	break;

		case IS_st:		*fp++ = 's';
					*fp++ = 't';	break;

		case IS_sk:		*fp++ = 's';
					*fp++ = 'k';	break;

		default:		*fp++ = '_';
		}
	     break;

	case IS_cal:
	case IS_cal_avg:
	case IS_cal_slit:
	case IS_cal_avg_slit:
	case IS_cal_rect:
	case IS_cal_rect_slit:
	case IS_cal_rect_byte:
	case IS_cal_avg_rect:
	case IS_cal_avg_rect_slit:
	     switch (who)
		{
		case IS_i:		*fp++ = 'I' ;	break;

		case IS_q:		*fp++ = 'p' ;
					*fp++ = 'b' ;	break;

		case IS_u:		*fp++ = 'U' ;	break;

		case IS_tscp:		*fp++ = 't' ;
					*fp++ = 's' ;
					*fp++ = 'c' ;
					*fp++ = 'p' ;	break;

		case IS_drift:		*fp++ = 'd' ; 
					*fp++ = 'r' ;
					*fp++ = 'i' ;
					*fp++ = 'f' ;
					*fp++ = 't' ;	break;

		default:		*fp++ = '_';
		}
	     break;

	default: ;
        }

    switch (howfar)
	{
	case IS_cal_avg:
	case IS_cal_avg_slit:
	case IS_cal_avg_rect:
	case IS_cal_avg_rect_slit:
					*fp++ = '.';
					*fp++ = 'a';
					*fp++ = 'v';
					*fp++ = 'g';	break;
	default: ;
	}

    switch (howfar)
	{
	case IS_cal_slit:
	case IS_cal_avg_slit:
	case IS_cal_rect_slit:
	case IS_cal_avg_rect_slit:
	         			*fp++ = '.';
		 			*fp++ = 'v';
		 			*fp++ = 'i';
		 			*fp++ = 'g';	break;
	default: ;
	}

    switch (howfar)
	{
	case IS_cal_rect_byte:
	         			*fp++ = '.';
		 			*fp++ = 'b';
		 			*fp++ = 'y';
		 			*fp++ = 't';
		 			*fp++ = 'e';	break;
	default: ;
	}

    *fp = '\0' ;

    return (file) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

