/* read_mk4_image.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a Mk4 header and image data from the file "namimg".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   1 Nov 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>

#include "mk4v2.h"
#include "mk4headv2.h"
#include "mk4fitsv2.h"

/* -------------------------------------------------------------------------- */

unsigned char *
read_mk4_image (namimg, pmode) 

char *namimg ;			/* Mk4 file name (native format). */
int pmode ;			/* Print mode. */
{
    unsigned char *mk4_image ;
    int nread ;
    int nbyhead ;
    int npximg ;
    int nbypix ;
    FILE *fid ;

    if (pmode > 0)
	fprintf (stderr, "read_mk4_image> read image %s\n", namimg) ;

   fprintf (stderr, "read_mk4_image> read image %s\n", namimg) ;

    /*** Open file. ***/

    if ( ( fid = fopen (namimg, "r") ) == NULL  )
	{
	fprintf (stderr, "read_mk4_image> %s fopen error\n", namimg) ;
	exit (-1) ;
        }

    /*** Read header. ***/

    nbyhead = sizeof (mk4head) ;

    if ( (nread = fread (mk4h, 1, nbyhead, fid) ) != nbyhead )
	{
	fprintf (stderr, "read_mk4_head> %s nread=%d nbyhead=%d\n",
		 namimg, nread, nbyhead) ;
	fclose (fid) ;
	exit (-2) ;
        }

    /*** Fill FITS information structure. ***/

    mk4hdu (mk4h, pmode) ;

    /*** Read image data. ***/

    nbypix = abs (mk4h->ImgDataType) / 8 ;		/* # bytes/pixel. */
    npximg = (mk4h->Xdim) * (mk4h->Ydim) ;		/* # pixels/image. */

    /*** Allocate storage for image array. ***/

    mk4_image = (unsigned char *) malloc (npximg * nbypix) ;

    /*** Read image array. ***/

    if ( ( nread = fread (mk4_image, nbypix, npximg, fid)) != npximg )
	{
	fprintf (stderr, "read_mk4_image> %s nread:%d pixels != npximg:%d\n", 
		        namimg, nread, npximg) ;
	fclose (fid) ;
	exit (-3) ;
        }

    /*** Close file. ***/

    fclose (fid) ;

    if (pmode == 1)			/* Print out the sumry info. */
	{
	printf ("File: %s\n", namimg) ;
	printf ("%s %s\n", prday (mk4h->Image.Juld), prtime(mk4h->Image.Time)) ;

	switch (mk4h->DataType)
	    {  
	    case IS_i:  fprintf (stdout, "DataType: i\n") ; break ;
	    case IS_q:  fprintf (stdout, "DataType: q\n") ; break ;
	    case IS_u:  fprintf (stdout, "DataType: u\n") ; break ;
	    case IS_ba: fprintf (stdout, "DataType: ba\n") ; break ;
	    case IS_sk: fprintf (stdout, "DataType: sk\n") ; break ;
	    case IS_st: fprintf (stdout, "DataType: st\n") ; break ;
	    default:    fprintf (stdout, "DataType: UNKNOWN\n") ;
	    } 
        }

   return (mk4_image) ;
}

/* -------------------------------------------------------------------------- */

