/* mk4_coord.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO Mark-4 coordinate transformation.
 * rcoord (radius, theta, x, y, isign, roll)
 * isign = 1 :  (radius,theta) --> (x,y).
 *       =-1 :  (x,y) --> (radius,theta).
 * Image coordinate origin (x=0,y=0): lower left corner.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Mauna Loa Solar Observatory   Mark-4 K-Coronameter
 * Andrew L. Stanger   HAO/NCAR   10 January 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include "/home/cordyn/fits/lib/mk4/mk4fits.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

mk4_coord (radius, theta, x, y, isign, roll)

float	*radius, *theta;/* Radius (Rsun) & Position Angle (degrees). */
float	*x, *y;		/* Cartesian coordinates:  (0,0)=upper left.*/
int	isign;		/* =1: (r,th) --> (x,y). otherwise, vice-versa.*/
int	roll;		/* Roll angle (degrees). */

{
   int   ierr;			/* error indicator. */
   float degnor;		/* angular position of north w.r.t. +x axis. */
   float deltax, deltay;	/* x/y distance from sun center. */
   float  mk4th;		/* angle in degrees w.r.t. +x axis. */
   double angle;		/* angle in radians. */
   double r2;			/* radius^2 */
   double degrad = 57.2995780;	/* degress/radian. */
   double raddeg = 0.01745329;	/* radians/degree. */

   degnor = 90.0 + (float) roll;/* Orientation of solar north. */
   ierr = 0;

   if (isign == 1)		/*** (radius,theta) --> (x,y) ***/
      {
      mk4th = *theta + degnor;
      while (mk4th > 360.0)  mk4th -= 360.0;
      angle = (float) (raddeg * mk4th);
         *x = (float) (MK4F->xcen + *radius * MK4F->pixrs * cos (angle));
         *y = (float) (MK4F->ycen + *radius * MK4F->pixrs * sin (angle));

      if ((*x < 0 || *x >= MK4F->xdim) || (*y < 0 || *y >= MK4F->ydim))
         ierr = -1;
      }
   else				/*** (x,y) --> (r,theta) ***/
      {
      deltax = *x - MK4F->xcen;
      deltay = *y - MK4F->ycen;
      r2 = (double) (deltax * deltax + deltay * deltay);

      *radius = (float) (sqrt (r2) / MK4F->pixrs);
      *theta  = (float) (atan2 (deltay, deltax) * degrad - (double) degnor);
      while (*theta < 0.0)  *theta += 360.0;
      }

   return (ierr);
}
