/* mk4fitsv2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Mark-III image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 February 1994
 * 17 Feb 1994: add additional extension file names (namext1, namext2, namext3).
 * 28 Mar 1994: use fgets instead of fscanf in while loop.
 * 30 May 1997: add optional photometric scaling parameters to command line.
 *  3 Mar 1998: Scaling from 16-bits to 8-bits now an option (bscale = 1).
 * 31 Mar 1999: version 1.05: add CRRADIUS to header.
 * 11 Aug 1999: adapted from mk3fits.c for Mark-IV.
 *  7 Mar 2000: version 1.10: modified for revised Mk4 calibration.
 *  4 Feb 2002: Add Instrument Description items for Mk4 version 2 calibration.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#define START
#include <stdio.h>
#include "lib/mk4v2/mk4v2.h"
#include "lib/mk4v2/mk4headv2.h"
#include "lib/mk4v2/mk4fitsv2.h"
#include "img_scale.h"
int nbypix ;                             /* Ouput image: # bytes/pixel. */

main (argc, argv)

int argc ;
char *argv [] ;

{
   char imgbuf ;
   static char linbuf  [84] ;
   static char nampgm  [80] ;
   static char name    [80] ;
   static char namext1 [80] = "\0" ;
   static char namext2 [80] = "\0" ;
   static char namext3 [80] = "\0" ;
   static char namimg  [80] ;
   static char namlst  [80] ;
   char *parg;			/* Argument pointer. */

   FILE *LIST, *IN, *OUT ;
   int hduflag = 1 ;		/* HDU (header data unit) flag. */
   				/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */
   int numext = 0 ;		/* # extensions. */
   int pmode = 0 ;		/* Print mode: 0=silent, 1=verbose. */
   int ii = 0 ;

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   strcpy (nampgm, argv [0]) ;

   if (argc == 1)
      {
      usage (nampgm) ;
      exit (0);
      }

   while (--argc)
      {
      parg = *++argv ;

      if (*(parg) == '?')		/* Check for flag arguments */
	 {
	 usage (nampgm) ;
	 exit (1) ;
	 }

      if (*(parg) == '-')		/* Check for flag arguments */
	 {
	 switch (parg [1])
	    {
            case 'b':  nbypix = 1 ;	/* Output image: 1 byte / pixel. */
                       break ;
	    case 's':  lscale = 1 ;	/* specify intensity scaling. */
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmin) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmax) ;
		       break;
            default:   fprintf (stderr," unknown flag %s\n", *argv);
		       usage (nampgm); exit (-2);
            }
         continue ;
         }
      else
	 {

         /* Get the filename. */

         strcpy (name, parg) ; 

         if (name [0] == '@')		/* Read list file. */
            {
            strcpy (namlst, &name [1]) ;
            if ((LIST = fopen (namlst, "r")) == NULL)
	       {
	       fprintf (stderr, "Unable to open file: %s\n", namlst) ;
	       exit (-2) ;
	       }

	    while (fscanf (LIST, "%s", namimg) != EOF)
	       { 
               mk4tofits (namimg, namext1, hduflag, pmode) ;
	       }
            fclose (LIST) ;
            }
         else
            {
            strcpy (namimg, name) ;
            mk4tofits (namimg, namext1, hduflag, pmode) ;
            }
         }
      }

   exit (0) ;
}

usage (nampgm)

char nampgm [] ;
{
   fprintf (stderr, "Usage: %s [-b][-s oldmin oldmax] image_file[s] | @image_file_list\n",
	       nampgm) ;
   fprintf (stderr, "   -b     FITS output will be 8 bits/pixel.\n");
   fprintf (stderr, "   -s     Override default intensity scaling.\n");
   fprintf (stderr, "          oldmin, oldmax: min -> max image values;\n");
   fprintf (stderr, "                          will be displayed as 0 -> 250.\n");
   fprintf (stderr, "          This flag only applies if -b is specified.\n");
   fprintf (stderr, "image_file:	MK4 native format filename.\n") ;
   fprintf (stderr, "image_file_list:	file containing a list of raw MK4 filenames\n") ;
   fprintf (stderr, "			each line may contain 1 to 3 filenames\n") ;
   return (0);
}
