/* polcarf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert Mk4 Polar coordinate image to Cartesian coordinates.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Original Author: Alice Lecinski
 * Modifications by Andrew L. Stanger [ALS]
 * 15 Mar 2000: [ALS] Use BIGSCALE to output a larger cartesian coorinate image.
 *  1 Apr 2000: [ALS] Use simple coordinate conversion computation.
 * 23 May 2000: [ALS] Eliminate SQ & TRY variables.  Add -d flag to specify
 *                    output dimensions.
 * 29 Nov 2000: Replace "dofnm" with "dofname4" (replace "slit" with "vig").
 * 13 Jun 2000: Crop inner FOV edge (-e option).
 * 26 Sep 2001: Adjust r0 & dr (based on 960 arcsec/Rsun) using mk4h->Radius.
 *  6 Nov 2001: Use FITS format (instead of native Mk4 file format).
 *  9 Nov 2001: Output is the same pixel type as the input file.
 *		Previously, in 6 Nov 2001 version, the output was floating pt.
 *  9 Jan 2002: Adjust r0 & dr if image calibration is version 1.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define START

#include "mk4v2.h"
#include "mk4headv2.h"
#include "mk4fitsv2.h"

#include "fitsio.h"
#include "longnam.h"


char *fts_file ;

char warnbyte [] = "\
ALL of the images will be scaled from 0-255\n\n" ;

char warndim [] = "\
Dimensions of output image:" ;


/* NAZ     = 720					*/
/* NPX     = 384					*/
/* CY_IMG  = 1080 * 384					*/
/* VECTOR  = 1080					*/
/* LOWPIX  = 70						*/
/* HIGHPIX = 250					*/
/* RZERO   = 0.868645					*/
/* RDELT   = 0.00546721					*/
/* LOZERO  = 48						*/
/* HIZERO  = NPX - 1 - 10 = 373				*/
/* BOTPIX  = 38						*/
/* TOPPIX  = NPX = 384					*/


double RR [NPX] ;
double *prr, r0, dr;
double xcen, ycen ;			/* Solar disk center. */

fitsfile *fpol, *frec ;

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{
    extern void printerror () ;
    extern float blint () ;
    extern char *ctime () ;
    extern char *mk4_name () ;

    register char *w ;
    register FH hd ;

    static char card      [FLEN_CARD] ;
    static char rec_file  [80] ;
    static char comment   [48] ;
    static char cbitpix   [48] ;
    static char cnaxis1   [48] ;
    static char cnaxis2   [48] ;
    static char ccoordnam [48] ;
    static char cdataform [48] ;
    static char cctype1   [48] ;
    static char cctype2   [48] ;
    static char ccrpix1   [48] ;
    static char ccrpix2   [48] ;
    static char ccrval1   [48] ;
    static char ccrval2   [48] ;
    static char ccdelt1   [48] ;
    static char ccdelt2   [48] ;
    static char ccrota1   [48] ;
    static char ccrota2   [48] ;
    static char ccrradius [48] ;
    static char ctype_obs [48] ;

    static char coordnam  [24] ;
    static char ctype1    [24] ;
    static char ctype2    [24] ;
    static char dataform  [24] ;
    static char date_obs  [12] ;
    static char extname   [12] ;
    static char instrume  [24] ;
    static char telescop  [24] ;
    static char time_obs  [12] ;
    static char type_obs  [24] ;
    static char type_obsv [24] ;
    static char xtension  [12] ;

    float bkgval = 0.0 ;
    float bscale ;
    float bzero ;
    float crradius ;
    float crpix1, crpix2 ;
    float crval1, crval2 ;
    float cdelt1, cdelt2 ;
    float crota1, crota2 ;
    float datamin ;
    float datamax ;
    float dispmin ;
    float dispmax ;
    float diskval = 0.0 ;
    float fmin, fmax ;
    float fval ;
    float rdp, adp ;
    float scale ;
    float val ;
    float zero ;

    double angle ;
    double angoff ;
    double dang = 0.5 ;			/* degrees/pixel [polar coordinates]. */
    double myang ;
    double pdif ;
    double radmax = 3.00 ;
    double rad1  = 1.0  ;
    double radlo = 1.07 ;
    double radhi = 2.91 ;
    double rd ;
    double rdif ;
    double rs ;
    double rspix ;
    double xd, yd ;
    double xx, yy ;

    int anynull ;
    int bitpix ;
    int bitpixe ;
    int edge = 0 ;
    int extdim = 1024 ;
    int hdunum ;
    int hdutype ;
    int howfar ;
    int ia, ir ;
    int i ;
    int ii, jj ;
    int ix, iy ;
    int ihour, iminute, isecond ;
    int iyear, imonth, iday ;
    int morekeys = 0 ;
    int nkeys ;
    int keypos ;
    int nbypix ;
    int nbypixe ;
    int npxext ;
    int npximg ;
    int npximgc ;
    int nullval = 0 ;
    int pixlo, pixhi ;
    int pmode = 0 ;			/* =0 silent, =1 verbose print mode. */
    int proc_code ;
    int px, py ;
    int rescale_byte ;
    int status = 0 ;
    int HaveU, FixBA ;
    int xdim ;
    int ydim ;
    int xdimc = 960 ;
    int ydimc = 960 ;
    int who ;

    static int naxis [2] ;
    static long fpiximg [2] ;
    long fpixext = 1 ;
    static long naxes [2] ;

    long begtime, endtime ;
    long btime, etime ;

    unsigned char *xbuf ;
    unsigned char *cimg ;
    short	  *simg ;
    int		  *iimg ;
    float	  *fimg ;
    double	  *dimg ;

    float *pimg ;
    float *rimg ;

    (void) time (&begtime) ;

    rescale_byte = HaveU = FixBA= 0 ;

    mk4h  = (FH) malloc (sizeof (mk4head)) ;
    hd    = (FH) malloc (sizeof (mk4head)) ;

    timeflag = 1 ;

    if ( argc <= 1 ) { prsum () ; exit (3) ; }

    /*** Process command line arguments. ***/

    while (--argc)
	{	/* Read in the files and perform the coordinate conversion */
        w = *++argv ;
        if (*w == '?') { prsum () ; exit (2) ; }
	if (*w == '-')
	    {
	    switch (w [1])
		{

        	case 'b':   fprintf (stderr, warnbyte) ;
			    rescale_byte = 1 ; 
			    break ;

		case 'c':   diskval = -32000.0 ;
			    break ;

		case 'd':   argc--; w = *++argv;
			    sscanf (w, "%d", &xdimc) ;
			    ydimc = xdimc ;
			    fprintf (stderr, "Output dimensions: %4d %4d\n",
				     xdimc, ydimc) ;
			    break ;

		case 'e':   edge = 1 ;
			    break ;

		default: ;
	        }
	    continue ;
	    }
        fts_file = w ;

	npximgc = xdimc * ydimc ;
	
        xcen = (xdimc - 1.0) * 0.5 ;
        ycen = (ydimc - 1.0) * 0.5 ;

	(void) time (&btime) ;

        /*** Check on the file naming convention used on the .q file ***/

        if ((*w == '0') || (*w >= '8' && *w <= '9') || *(w + 2) == 'd' )
	    {
            if ( *(w+3) >= '0' && *(w + 3) <='9' )  doyflag++ ;
            else                                    nameflag++ ;
            }
        if (nameflag || doyflag) { dayflag++ ; timeflag = 1 ; }

	/* ------------------------------------------------------------------ */

	printf ("fts_file: %s   ", fts_file) ;
	fflush (stdout) ;

	/*** Open polar coordinate FITS file. ***/

	if ( fits_open_file (&fpol, fts_file, READONLY, &status) )
	    printerror (status) ;

	if (fits_get_num_hdus (fpol, &hdunum, &status))  printerror (status) ;
	if (fits_get_hdu_num  (fpol, &hdunum))           printerror (status) ;
	if (fits_get_hdu_type (fpol, &hdutype, &status)) printerror (status) ;

	if (pmode)
	    {
	    fprintf (stdout, "#hdus: %d\n", hdunum) ;
	    fprintf (stdout, "hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;	
	    }


        /*** Get selected keyword parameters from the primary FITS header. ***/

	if (fits_read_key (fpol, TINT, "BITPIX", &bitpix,
			   comment,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TINT, "NAXIS1", &xdim,
			   cnaxis1,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TINT, "NAXIS2", &ydim,
			   cnaxis2,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TSTRING, "DATE-OBS", date_obs,
			   comment,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TSTRING, "TIME-OBS", time_obs,
			   comment,   &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "TELESCOP", telescop,
			   comment,   &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "INSTRUME", instrume,
			   comment,   &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "TYPE-OBS", type_obs,
			   comment,   &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "DATAFORM", dataform,
			   cdataform, &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "COORDNAM", coordnam,
			   ccoordnam, &status))  printerror (status) ;

        if (fits_read_key (fpol, TSTRING, "CTYPE1", ctype1,
			   cctype1, &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CRPIX1", &crpix1,
			   ccrpix1,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CDELT1", &cdelt1,
			   ccdelt1,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CROTA1", &crota1,
			   ccrota1,   &status))  printerror (status) ;

        if (fits_read_key (fpol, TSTRING, "CTYPE2", ctype2,
			   cctype2, &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CRPIX2", &crpix2,
			   ccrpix2,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CDELT2", &cdelt2,
			   ccdelt2,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CROTA2", &crota2,
			   ccrota2,   &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "CRRADIUS", &crradius,
			   ccrradius, &status))  printerror (status) ;

	nbypix = abs (bitpix) / 8 ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;

	if (pmode)
	    {
	    fprintf (stdout, "[1] xdim/ydim/bitpix: %d %d %d\n",
		     xdim, ydim, bitpix) ;

	    fprintf (stdout, "iyear: %d  imonth: %d  iday: %d\n",
		             iyear, imonth, iday) ;

	    fprintf (stdout, "ihour: %d  iminute: %d  isecond: %d\n",
			     ihour, iminute, isecond) ;

	    fprintf (stdout, "telescop: %s\n", telescop) ;
	    fprintf (stdout, "instrume: %s\n", instrume) ;
	    fprintf (stdout, "type_obs: %s\n", type_obs) ;
	    fprintf (stdout, "dataform: %s\n", dataform) ;
	    fprintf (stdout, "coordnam: %s\n", coordnam) ;
	    }

	/* ------------------------------------------------------------------ */

	/*** Read FITS image data [polar coordinates]. ***/

        npximg  = xdim * ydim ;
	nullval = 0 ;          /* Don't check for null values in the image. */

	/*** Allocate storage for image arrays. ***/

	pimg = (float *) malloc (xdim  * ydim  * sizeof (float)) ;
	rimg = (float *) malloc (xdimc * ydimc * sizeof (float)) ;

/*
	if (rescale_byte)
	    val = 0.0 ;
	else
	    val = -30000.0 ;
	for (ii = 0 ; ii < npximg ; ii++) *rimg++ = val ;
 */
	

	xbuf = (unsigned char *) malloc (xdim * ydim * nbypix) ;

	/*** Override automatic intensity scaling. ***/
	/*** i.e., read unscaled pixel data. ***/

	scale = 1.0 ;
	zero  = 0.0 ;
	if (fits_set_bscale (fpol, scale, zero, &status)) printerror (status) ;

	fpiximg [0] = 1 ;
	fpiximg [1] = 1 ;

	/*** Read primary image array. ***/

	switch (bitpix)
	    {
	    case   8:
			printf ("case c\n") ;
			cimg = (unsigned char *) xbuf ;
			if (fits_read_img (fpol, TBYTE,   fpiximg [0], npximg,
			    &nullval, cimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npximg; i++)
			    *(pimg + i) = (float) *(cimg + i) ;
			break ;
	    case  16:
			printf ("case s\n") ;
			simg = (short  *) xbuf ;
			if (fits_read_img (fpol, TSHORT,  fpiximg [0], npximg,
			    &nullval, simg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npximg; i++)
			    *(pimg + i) = (float) *(simg + i) ;
			break ;
	    case  32:
			printf ("case i\n") ;
			iimg = (int    *) xbuf ;
			if (fits_read_img (fpol, TINT,    fpiximg [0], npximg,
			    &nullval, iimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npximg; i++)
			    *(pimg + i) = (float) *(iimg + i) ;
			break ;
	    case -32:
			printf ("case f\n") ;
			fimg = (float  *) xbuf ;
			if (fits_read_img (fpol, TFLOAT,  fpiximg [0], npximg,
			    &nullval, fimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npximg; i++)
			    *(pimg + i) = (float) *(fimg + i) ;
			break ;
	    case -64:
			printf ("case d\n") ;
			dimg = (double *) xbuf ;
			if (fits_read_img (fpol, TDOUBLE, fpiximg [0], npximg,
			    &nullval, dimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npximg; i++)
			    *(pimg + i) = (float) *(dimg + i) ;
			break ;

	    default:
			printf ("case default\n") ;
			fprintf (stderr, "Invalid bitpix: %d\n", bitpix) ;
			goto next ;
	    }

	free (xbuf) ;

	if (pmode)
	    {
	    ix = 0 ;
	    for (iy = 0; iy < ydim; iy += 10)
	        printf ("ix/iy: %3d %3d  pImg: %8.2e\n",
		        ix, iy, *(pimg + ix + iy*xdim));
	    }

	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;
	if (fits_get_hdu_num  (fpol, &hdunum))  printerror (status) ;
	if (fits_get_hdu_type (fpol, &hdutype, &status)) printerror (status) ;

	if (pmode)
	    {
	    fprintf (stdout, "fits_movabs_hdu [2] . . . status: %d\n", status) ;
	    fprintf (stdout, "hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;
	    }


	/*** Read keywords from FITS extension header. ***/

	if (fits_read_key (fpol, TINT, "BITPIX", &bitpixe,
			   comment, &status))  printerror (status) ;
	if (fits_read_key (fpol, TINT, "NAXIS1", &extdim,
			   cnaxis1, &status))  printerror (status) ;

	nbypixe = abs (bitpixe) / 8 ;

	if (pmode)
	    fprintf (stdout, "[2] extdim/bitpix: %d %d\n", extdim, bitpixe);


	/*** Read Mk4 binary header (stored as a FITS extension). ***/

	fpixext = 1 ;
        if ( fits_read_img (fpol, TBYTE, fpixext, extdim, &nullval, mk4h,
			    &anynull, &status) )
	    printerror ( status ) ;

	mk4hdu (mk4h, pmode) ;		/* Fill Mk4 FITS structure. */

	/* ------------------------------------------------------------------ */

	/*** Get geometric scaling information. ***/

	if (strncmp (mk4h->version, "Version", 7) == 0)
	    {
	    r0 = (double) mk4h->r0 ;
	    dr = (double) mk4h->dr ;
	    }
	else				/* Version 1.0 calibration. */
	    {
	    r0 = (double) (mk4h->r0 * (960.0 / mk4h->Radius)) ;
	    dr = (double) (mk4h->dr * (960.0 / mk4h->Radius)) ;

	    fprintf (stderr, "Warning: version 1 calibration.\n") ;
	    fprintf (stderr, "Adjusting r0 & dr.\n") ;
	    fprintf (stderr, "960.0 arcsec sun   r0: %12.6g  dr: %12.6g\n",
		     mk4h->r0, mk4h->dr) ;
	    fprintf (stderr, "%5.1f arcsec sun   r0: %12.6g  dr: %12.6g\n",
		     mk4h->Radius, r0, dr) ;
	    }

	if (pmode)
	    {
	    printf ("mk4h->r0: %10.6f  mk4h->dr: %10.6f  Radius:  960.00\n",
		    mk4h->r0, mk4h->dr);
	    printf ("r0 [adj]: %10.6f  dr [adj]: %10.6f  Radius: %7.2f\n",
		    r0, dr, mk4h->Radius) ;
	    }

	/*** Zero out lower and upper radii pixels. ***/

	if (edge == 1)
	    {
    	    if (pmode)
    	        printf ("pixlo (%4.2f Rsun): %d   pixhi (%4.2f Rsun): %d\n",
    		        radlo, pixlo, radhi, pixhi) ;

	    /*** Find pixel corresponding to radlo Rsun. ***/

	    rdif  = radlo - r0 ;
	    pdif  = rdif / dr ;
	    pixlo = (int) ((rdif / dr) + 0.5) ;
	    pixlo = 45 ;

    	    /*** Find pixel corresponding to radhi Rsun. ***/
    
    	    rdif  = radhi - r0 ;
    	    pdif  = rdif / dr ;
    	    pixhi = (int) ((rdif / dr) + 0.5) ;
	    pixhi = NPX - 1 ;
    
    	    if (pmode)
    	        printf ("pixlo (%4.2f Rsun): %d   pixhi (%4.2f Rsun): %d\n",
    		        radlo, pixlo, radhi, pixhi) ;
    
            for (ii = 0 ; ii < NAZ ; ii++)
    	        for (jj = 0 ; jj < pixlo ; jj++)
    		    {
    		    pimg [ii + jj * NAZ] = -30000.0 ;
    		    }
    
            for (ii = 0 ; ii < NAZ ; ii++)
    	        for (jj = pixhi ; jj < NPX ; jj++)
    		    {
    		    pimg [ii + jj * NAZ] = -30000.0 ;
    		    }
	    }


	rspix = radmax * 2.0 / xdimc ;			/* Rsun/pixel */

	/*** Convert from Polar to Cartesian coordinates. ***/

	for (iy = 0 ; iy < ydimc ; iy++)
	    {
	    yd = iy - ycen ;		/* y distance from center. */
	    for (ix = 0 ; ix < xdimc ; ix++)
		{
		xd = ix - xcen ;		  ; /* x dist. from center */
		rd = sqrt (xd * xd + yd * yd) ; /* radial dist.: pixels */
		rs = rd * rspix ;		/* radial distance: Rsun */
		rdp = (rs - r0) / dr ;	/* radial distance: pixels */

		angle = atan2 ((double) yd, (double) xd) * r2d - 90.0 ;
		while (angle < 0.0) angle += 360.0 ;
		while (angle > 360.0)  angle -= 360.0 ;
		adp = angle / dang ;	/* angular "distance": pixels.*/

		ir = (int) rdp ;
		ia = (int) adp ;

		if (rs < rad1)
		    fval = diskval ;
		else
		    if (rs <= radhi)
			{
		        if (ir > NPX-1 || ia > NAZ -1)
		            fval = -30000.0 ;
		        else
			    {
		            fval = *(pimg + ia + ir * NAZ) ;


			    fval = blint (pimg, 4, mk4h->Xdim, mk4h->Ydim,
					  adp, rdp) ;


			    }
			}
		    else
			fval = bkgval ;

		*(rimg + ix + iy * xdimc) = fval ;
		}
	    }

	free (pimg) ;


	/*** Update mk4h ***/

	mk4h->Xcen   = xcen ;
	mk4h->Ycen   = ycen ;
	mk4h->Xdim   = xdimc ;
	mk4h->Ydim   = ydimc ;
	mk4h->r0     = r0 ;
	mk4h->dr     = dr ;
	mk4h->pixscl = (float) 1.0 / rspix ;
	switch ( mk4h->ProcSteps )
	    {
	    case IS_raw :	mk4h->ProcSteps = IS_raw_rect ;
				break ;
	    case IS_cal :	mk4h->ProcSteps = (rescale_byte == 0) 
						? IS_cal_rect 
						: IS_cal_rect_byte ;
				break ;
	    case IS_cal_slit :	mk4h->ProcSteps = (rescale_byte == 0) 
						? IS_cal_rect_slit
						: IS_cal_rect_byte ;
				break ;
	    case IS_cal_avg :	mk4h->ProcSteps = IS_cal_avg_rect ;
				break ;
	    case IS_cal_avg_slit: mk4h->ProcSteps = (rescale_byte == 0) 
						  ? IS_cal_avg_rect_slit
						  : IS_UNKNOWN ;
				break ;
	    default :		mk4h->ProcSteps = IS_UNKNOWN ;
	    }

	howfar = mk4h->ProcSteps ;

	/*** Draw a polar grid on top of occulting disk region. ***/

/*
 *	suncir4 (cbuf, mk4h->Xdim, mk4h->Ydim, xcen, ycen, 1.0/rspix, 0.0) ;
 */


        /*** Create output file name. ***/

	dayflag = 1 ;
	doyflag = 0 ;

        strcpy (rec_file, mk4_name (mk4h, doyflag)) ;

	fprintf (stdout, "rec_file: %s   ", rec_file) ;
	fflush (stdout) ;

	remove (rec_file) ;		/* Delete pre-existing file. */

	/*** Create output file. ***/

	if ( fits_create_file (&frec, rec_file, &status) )
	    printerror ( status ) ;

	/* ------------------------------------------------------------------ */

	/*** Move to FITS primary HDU [input file]. ***/

	hdunum = 1 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;

	/*** Copy Primary FITS header to output file. ***/

	if (fits_get_hdrpos (fpol, &nkeys, &keypos, &status))
	    printerror (status) ;

	for (ii = 1; ii <= nkeys; ii++)
	    {
	    if (fits_read_record (fpol, ii, card, &status))
		printerror (status) ;
	    if (fits_write_record (frec, card, &status))
		printerror (status) ;
	    }

	/* ------------------------------------------------------------------ */

	/*** Update keywords in primary HDU header. ***/

	if (fits_update_key (frec, TINT, "BITPIX", &bitpix, 0, &status))
	     printerror ( status );

	if (fits_update_key (frec, TINT, "NAXIS1", &xdimc, 0, &status))
	     printerror ( status );

	if (fits_update_key (frec, TINT, "NAXIS2", &ydimc, 0, &status))
	     printerror ( status );

	sprintf (dataform, "CARTESIAN") ;
	if (fits_update_key (frec, TSTRING, "DATAFORM", dataform, cdataform,
	    &status))  printerror ( status );

	sprintf (coordnam, "HELIO-CENTRIC") ;
	if (fits_update_key (frec, TSTRING, "COORDNAM", coordnam, ccoordnam,
	    &status))  printerror ( status );

	sprintf (ctype1, "X") ;
	sprintf (ctype2, "Y") ;
	crpix1 = xdimc/2 - 0.5 ;
	crpix2 = ydimc/2 - 0.5 ;
	crval1 = 0.0 ;
	crval2 = 0.0 ;
	crota1 = 0.0 ;
	crota2 = 0.0 ;
	cdelt1 = MK4F->rsun * rspix ;
	cdelt2 = MK4F->rsun * rspix ;
	crradius = 1.0 / rspix ;		/* pixels/Rsun */

	sprintf (cctype1, "AXIS 1 TYPE: X") ;
	sprintf (cctype2, "AXIS 2 TYPE: Y") ;
	sprintf (ccrpix1, "X SUN CENTER [PIXEL]") ;
	sprintf (ccrpix2, "Y SUN CENTER [PIXEL]") ;
	sprintf (ccrval1, "X SUN CENTER [ARCSEC]") ;
	sprintf (ccrval2, "Y SUN CENTER [ARCSEC]") ;
	sprintf (ccdelt1, "X COORD INCREMENT [ARCSEC/PIXEL]") ;
	sprintf (ccdelt2, "Y COORD INCREMENT [ARCSEC/PIXEL]") ;
	sprintf (ccrota1, "X ROTATION [DEG.] FROM REFERENCE") ;
	sprintf (ccrota2, "Y ROTATION [DEG.] FROM REFERENCE") ;

	if (fits_update_key (frec, TSTRING, "CTYPE1", ctype1, cctype1, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CRPIX1", &crpix1, ccrpix1, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CRVAL1", &crval1, ccrval1, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CDELT1", &cdelt1, ccdelt1, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CROTA1", &crota1, ccrota1, &status))
	     printerror ( status );

	if (fits_update_key (frec, TSTRING, "CTYPE2", ctype2, cctype2, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CRPIX2", &crpix2, ccrpix2, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CRVAL2", &crval2, ccrval2, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CDELT2", &cdelt2, ccdelt2, &status))
	     printerror ( status );
	if (fits_update_key (frec, TFLOAT, "CROTA2", &crota2, ccrota2, &status))
	     printerror ( status );

	if (fits_update_key (frec, TFLOAT, "CRRADIUS", &crradius, ccrradius, &status))
	     printerror ( status );

/*
	bscale = 1.0 ;
	bzero  = 0.0 ;
	if (fits_update_key (frec, TFLOAT, "BSCALE", &bscale, 0, &status))
	     printerror ( status );

	if (fits_update_key (frec, TFLOAT, "BZERO", &bzero, 0, &status))
	     printerror ( status );

        datamin = fmin ;
	if (fits_update_key (frec, TFLOAT, "DATAMIN", &datamin, 0, &status))
	     printerror( status );
	     printerror( status );

	datamax = fmax ;
	if (fits_update_key (frec, TFLOAT, "DATAMAX", &datamax, 0, &status))
	     printerror( status );

	dispmin = fmin ;
	dispmin = -100.0 ;
	if (fits_update_key (frec, TFLOAT, "DISPMIN", &dispmin, 0, &status))
	     printerror( status );

	dispmax = fmax ;
	dispmax = 650.0 ;
	if (fits_update_key (frec, TFLOAT, "DISPMAX", &dispmax, 0, &status))
	     printerror( status );
 */

	if (ffgidt (frec, &bitpix, &status))  printerror (status) ; ;
	if (ffgidm (frec, naxis,   &status))  printerror (status) ; ;

	if (pmode)
	    {
	    printf ("frec: bitpix = %d\n", bitpix) ;
	    printf ("frec: naxis = %d %d\n", naxis [0], naxis [1]) ;
	    }

	/*** Write cartesian image array into the FITS data array. ***/

	fpiximg [0] = 1 ;
	fpiximg [1] = 1 ;

/*
	if (pmode)
	    {
	    ix = 0 ;
	    for (iy = 0; iy < ydimc; iy += 10)
	        printf ("ix/iy: %3d %3d  cbuf: %8.2e\n",
		        ix, iy, *(cbuf + ix + iy*xdimc));
	    }
 */

	scale = 1.0 ;
	zero  = 0.0 ;
	if (fits_set_bscale (frec, scale, zero, &status)) printerror (status) ;


	/* ------------------------------------------------------------------ */

	switch (bitpix)
	    {
	    case   8:
			printf ("case c\n") ;
			strcpy (cbitpix, "1-BYTE UNSIGNED CHARACTERS") ;
			cimg = (unsigned char *) malloc (npximgc * sizeof (unsigned char)) ;
			for (i = 0 ; i < npximgc; i++)
			    *(cimg + i) = (unsigned char) *(rimg + i) ;
			if (fits_write_img (frec, TBYTE, fpiximg [0], npximgc,
			    cimg, &status) ) printerror ( status ) ;
			free (cimg) ;
			break ;
	    case  16:
			printf ("case s  ") ;
			fflush (stdout) ;
			strcpy (cbitpix, "2-BYTE SIGNED INTEGERS") ;
			simg = (short *) malloc (npximgc * sizeof (short)) ;
			for (i = 0 ; i < npximgc; i++)
			    *(simg + i) = (short) *(rimg + i) ;
			if (fits_write_img (frec, TSHORT, fpiximg [0], npximgc,
			    simg, &status) ) printerror ( status ) ;
			free (simg) ;
			break ;
	    case  32:
			printf ("case i\n") ;
			strcpy (cbitpix, "4-BYTE SIGNED INTEGERS") ;
			iimg = (int *) malloc (npximgc * sizeof (int)) ;
			for (i = 0 ; i < npximgc; i++)
			    *(iimg + i) = (int) *(rimg + i) ;
			if (fits_write_img (frec, TINT, fpiximg [0], npximgc,
			    iimg, &status) ) printerror ( status ) ;
			free (iimg) ;
			break ;
	    case -32:
			printf ("case f\n") ;
			strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
			fimg = (float *) malloc (npximgc * sizeof (float)) ;
			for (i = 0 ; i < npximgc; i++)
			    *(fimg + i) = (float) *(rimg + i) ;
			if (fits_write_img (frec, TFLOAT, fpiximg [0], npximgc,
			    fimg, &status) ) printerror ( status ) ;
			free (fimg) ;
			break ;
	    case -64:
			printf ("case d\n") ;
			strcpy (cbitpix, "8-BYTE DOUBLE PRECISION") ;
			dimg = (double *) malloc (npximgc * sizeof (double)) ;
			for (i = 0 ; i < npximgc; i++)
			    *(dimg + i) = (double) *(rimg + i) ;
			if (fits_write_img (frec, TDOUBLE, fpiximg [0], npximgc,
			    dimg, &status) ) printerror ( status ) ;
			free (dimg) ;
			break ;
	    default:
			printf ("case default\n") ;
			bitpix = -32 ;
			strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
			fimg = (float *) malloc (npximgc * sizeof (float)) ;
			for (i = 0 ; i < npximgc; i++)
			    *(fimg + i) = (unsigned char) *(rimg + i) ;
			if (fits_write_img (frec, TFLOAT, fpiximg [0], npximgc,
			    fimg, &status) ) printerror ( status ) ;
			free (fimg) ;
	    }

	free (rimg) ;


	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;

	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;

	if (pmode)
	    fprintf (stdout, "fits_movabs_hdu [2] . . . status: %d\n", status) ;

	/*** Copy HDU2 to output file. ***/

	morekeys = 1 ;
	if (fits_copy_hdu (fpol, frec, morekeys, &status) )
	    printerror (status);

	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [output file]. ***/

	sprintf (xtension, "IMAGE   ") ;
	if (fits_update_key (frec, TSTRING, "XTENSION", &xtension, 0, &status))
	     printerror ( status );

	if (fits_update_key (frec, TINT, "PROCSTEP", &howfar, 0, &status))
	     printerror ( status );

	sprintf (extname, "MK4_HSKP") ;
	if (fits_update_key (frec, TSTRING, "EXTNAME", &extname, 0, &status))
	     printerror ( status );

	/*** Write binary header into the FITS extension [output file]. ***/

	if (pmode)
	    printf ("fits_write_img [frec, hdu2]\n") ;
	npxext = 1024 ;
	if ( fits_write_img (frec, TBYTE, fpixext, npxext, mk4h, &status) )
	    printerror ( status ) ;

	/*** Close both input & output FITS files. ***/

	if (fits_close_file (fpol, &status) ||
	    fits_close_file (frec, &status))		/* close files */
	    printerror ( status );

next:
	free (xbuf) ;
	free (pimg) ;
	free (rimg) ;

	(void) time (&etime) ;
	fprintf (stderr,
	"time : %ld sec\n", etime - btime) ;
	fflush (stderr) ;
        }

    (void) time (&endtime) ;
    fprintf (stderr,"Total time :    %ld seconds\n", endtime - begtime) ;

    exit (0) ;
}

/* -------------------------------------------------------------------------- */

prsum ()
{
    static char help [] = "\
polcarf [-b] [-c] [-d dim] [polar coord file1] [polar coord file2] ...\n\
\n\
Convert a Mk4 FITS polar coordinate (r,theta) image to a cartesian image.\n\
\n\
Example:  polcarf 20010518.180017.mk4.cpbf.fts 20010518.180313.mk4.cpbf.fts  \n\
\n\
Option Flags: \n\
\n\
-c		Solar disk pixels will be set to -32000.0 \n\
-d xdim		Specify output image size (default is 960x960). \n\
-e		Crop edges of field-of-view. \n\
\n\
" ;

       fputs (help, stderr) ;

       return (0) ;
}

/* -------------------------------------------------------------------------- */

    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/

void
printerror ( int status)
{

    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit( status );    /* terminate the program, returning error status */
        }
    return;
}

/* -------------------------------------------------------------------------- */

/* blint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * blint returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float blint ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, or floating-point images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 29 March 2000: Add nbypix parameter (instead of include statement).
 *                Add xdim & ydim parameters.
 *                Replace "npixl" with "xdim".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .  */

float
blint (image, nbypix, xdim, ydim, x, y)

char	*image;		/* Pointer to image array. */
int	nbypix ;	/* # bytes per pixel. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
float	x, y;		/* Coordinates of scan point in image. */

{
register char	*im;
register short	*ims;
register float	*imf;
int	ix, iy;
float	fx, fy;
float	f00, f10, f01, f11;
float	fx0, fx1, fxy;

   if (x < 0 || x > xdim)
      {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
      return (-1.0) ;
      }

   if (y < 0 || y > ydim)
      {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
      return (-2.0) ;
      }

   im  = (char *) image;
   ims = (short *) image;
   imf = (float *) image;

   ix = (int) x;
   iy = (int) y;

   fx = x - (float) ix;
   fy = y - (float) iy;

   /*** Select 4 nearest neighbor elements in 'image' array. ***/

   /*** 1 byte per pixel. ***/

   if (nbypix == 1)
      {
      f00 = (float) (*(im +  ix    +  iy*xdim) & 0377);
      f10 = (float) (*(im + (ix+1) +  iy*xdim) & 0377);
      f01 = (float) (*(im +  ix    + (iy+1)*xdim) & 0377);
      f11 = (float) (*(im + (ix+1) + (iy+1)*xdim) & 0377);
      }
   else

   /*** 2 bytes per pixel (short integer). ***/

   if (nbypix == 2)
      {
      f00 = (float) *(ims +  ix    +  iy*xdim);
      f10 = (float) *(ims + (ix+1) +  iy*xdim);
      f01 = (float) *(ims +  ix    + (iy+1)*xdim);
      f11 = (float) *(ims + (ix+1) + (iy+1)*xdim);
      }
   else

   /*** 4 bytes per pixel (floating point). ***/

   if (nbypix == 4)
      {
      f00 = (float) *(imf +  ix    +  iy*xdim);
      f10 = (float) *(imf + (ix+1) +  iy*xdim);
      f01 = (float) *(imf +  ix    + (iy+1)*xdim);
      f11 = (float) *(imf + (ix+1) + (iy+1)*xdim);
      }

   /*** Bilinear interpolation. ***/

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}

/* -------------------------------------------------------------------------- */

