/* polvigf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Apply an artificial 'vignetting' to polar coordinate mk4 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adapted from 'cy_slit4.c', authored by Alice Lecinski
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   4 April 2000
 * 29 Nov 2000: Replace "dofn" with dofname4 (use "vig" instead of "slit").
 * 16 May 2001: Add David Elmore's May 2001 model.
 * 19 Oct 2001: Use FITS format (instead of native Mk4 format).
 *  2 Nov 2001: Use version 2 include files.
 *  6 Nov 2001: Manually copy primary header records (instead of fits_copy_hdu).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define START

#include "fitsio.h"
#include "longnam.h"
#include "mk4v2.h"
#include "mk4headv2.h"
#include "telscp4v2.h"
#include "model4.h"
#include "model4_May2001.h"
#include "crunch.h"
#include "mk4fitsv2.h"

#define MODEL  1
#define CORONA 2
#define TSCP   3
#define MAY2001MODEL  4

char *fits_file, picnm [64], *piceos ;
char chd [1024] ;
char vig_file [80] ;
char tscpfile [80] ;

fitsfile *fpol, *fvig ;
void prsum () ;
void fixLO () ;
void fixHI () ;
void model_RadVect_table () ;
void model_RadVect () ;
void model_Resid_orig () ;
void model_Resid_orig2 () ;
void model_Resid () ;

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;
{
    FILE *fid ;
    void printerror () ;
    extern char *mk4_name () ;

    register char *w ;
    static char pixtype   [4] ;
    static char card      [FLEN_CARD] ;
    static char cbitpix   [48] ;
    static char cbscale   [48] ;
    static char cbzero    [48] ;
    static char ccoordnam [48] ;
    static char cdatamin  [48] ;
    static char cdatamax  [48] ;
    static char cdispmin  [48] ;
    static char cdispmax  [48] ;
    static char comment   [48] ;
    static char coordnam  [24] ;
    static char cnaxis1   [48], cnaxis2 [48] ;
    static char date_obs  [24] ;
    static char time_obs  [24] ;
    static char instrume  [12] ;
    static char telescop  [12] ;
    static char code_obs  [12] ;
    static char lcode_obs [12] ;
    static char type_obs  [24] ;
    static char type_obsv [24] ;
    static char xtension  [12] ;
    static char extname   [12] ;
    static char anynullc ;

    long begtime, endtime ;
    long btime, etime ;

    int anynull ;
    int bitpix ;
    int dotest ;
    int extdim = 1024 ;
    int howfar ;
    int hdunum ;
    int hdutype ;
    int i ;
    int ii ;
    int ival ;
    int ix, iy ;
    int iyear, imonth, iday ;
    int ihour, iminute, isecond ;
    int method ;
    int doelmore ;
    int keypos ;
    int lozero ;
    int lspot ;
    int morekeys = 0 ;
    int nbypix ;
    int nbyimg ;
    int ncode ;
    int ndims = 2 ;
    int nkeys ;
    int npix ;
    int npxext ;
    int nullval = 0 ;
    int pmode = 0 ;			/* print mode: 0=silent, 1=verbose. */
    int status = 0 ;
    int xdim, ydim ;

    static int naxis [2] ;
    static long fpiximg [2] ;
    long fpixext = 1 ;
    static long naxes [2] ;

    float *pf, ftemp ;
    float MyOffset ;
    float bscale ;
    float bzero ;
    float bscale1 ;
    float bzero1 ;
    float datamin ;
    float datamax ;
    float dispmin ;
    float dispmax ;
    float fmin, fmax ;
    float  r5 ;
    float udatamin ;			/* unscaled datamin. */
    float udatamax ;			/* unscaled datamax. */
    float udispmin ;			/* unscaled dispmin. */
    float udispmax ;			/* unscaled dispmax. */

    double dtmp ;
    double scale = 1.0 ;
    double zero  = 0.0 ;
    double dr, r0 ;
    double radius ;

    struct tm *T, *localtime () ;
    char *ctime_r () ;
    static char buf [30] ;

    float *aimg ;
    float *aveq ;
    float *pimg ;
    float *vimg ;

    unsigned char *xbuf ;
    unsigned char *cimg ;
    short  *simg ;
    int    *iimg ;
    float  *fimg ;
    double *dimg ;

    if ( argc <= 1 ) { prsum () ; exit (3) ; }

    (void) time (&begtime) ;
    T = (struct tm *) malloc (sizeof (struct tm)) ;
    T = localtime (&begtime) ;
    T->tm_mon += 1 ;

    (void) ctime_r (&begtime, &buf, 30) ;
/*
 *    fprintf (stderr, "Started polvigf: %s\n", buf) ;
 */

    timeflag = 1 ;
    dotest = 0 ;
    method = MODEL ;
    lspot = 32 ;
    strcpy (pixtype, "s") ;

    while (--argc)
	{		/* Read in the files and perform the calibration */
	w = *++argv ;
	if (pmode)
	    printf ("argc: %d  w: %s\n", argc, w) ;
	if (*w == '?') { prsum () ; exit (2) ; }
	if (*w == '-')
	    {				/* Check for flag arguments. */
	    switch (w [1])
		{
		case 'L': lspot = BOTPIX ;
			  break ;
		case 'C': method = CORONA ; 
			  break ;
/*
		case 'T': method = TSCP ; 
			  argc-- ;   w = *++argv ;   sscanf (w, "%s", tscpfile);
			  readtscp (tscpfile, thd, 1) ;
			  pf = thd->DriftRefVect ;
			  ftemp  = *(pf + 45) ;
			  ftemp += 400.0 ;
			  for (ii = 0; ii < 45; ii++)
			      {
			      *pf++ = ftemp ;
			      }
			  break ;
 */
		case 'M': method = MAY2001MODEL ;
			  break ;
		case 'O': argc-- ; w = *++argv ; sscanf (w, "%f", &MyOffset) ;
			  break ;
		case 'p': 
			  pixtype [0] = w [2] ;
			  pixtype [1] = '\0' ;
			  if (pmode)
			      printf ("pixtype: %s  argc: %d\n", pixtype, argc) ;
			  w = *++argv ;
			  break ;
                default:  fprintf (stderr, " unknown flag %s\n", *argv) ;
                          prsum () ; exit (1) ;
                }
	    continue ;
	    }

	fits_file = w ;

	printf ("fits_file: %s\n", fits_file) ;

	(void) time (&btime) ;


	/* Check on the file naming convention used on the file */

	if ((*w == '0') || (*w >= '8' && *w <= '9') || *(w + 2) == 'd' )
	    {
	    if ( *(w + 3) >='0' && *(w+3) <='9' )  doyflag++ ;
	    else                                   nameflag++ ;
	    }
	if (nameflag || doyflag) { dayflag++ ; timeflag=1 ; }

	/* ------------------------------------------------------------------ */

        /*** Open FITS file. ***/

	if ( fits_open_file (&fpol, fits_file, READONLY, &status) )
	    printerror (status) ;

	if (fits_get_num_hdus (fpol, &hdunum,  &status))  printerror (status) ;
	if (fits_get_hdu_num  (fpol, &hdunum))            printerror (status) ;
	if (fits_get_hdu_type (fpol, &hdutype, &status))  printerror (status) ;

	if (pmode)
	    {
	    printf ("#hdus: %d\n", hdunum) ;
	    printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;
	    }


	/*** Get selected keyword parameters from the primary FITS header. ***/

	if (fits_read_key (fpol, TINT, "BITPIX", &bitpix,
			   comment, &status))  printerror (status) ;
	if (fits_read_key (fpol, TINT, "NAXIS1", &xdim,
			   cnaxis1, &status))  printerror (status) ;
	if (fits_read_key (fpol, TINT, "NAXIS2", &ydim,
			   cnaxis2, &status))  printerror (status) ;
	if (fits_read_key (fpol, TSTRING, "DATE-OBS", date_obs,
			   comment, &status))  printerror (status) ;
	if (fits_read_key (fpol, TSTRING, "TIME-OBS", time_obs,
			   comment, &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "TELESCOP", telescop,
			   comment, &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "INSTRUME", instrume,
			   comment, &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "TYPE-OBS", type_obs,
			   comment, &status))  printerror (status) ;
        if (fits_read_key (fpol, TSTRING, "COORDNAM", coordnam,
			   ccoordnam, &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "BSCALE", &bscale1,
			   cbscale, &status))  printerror (status) ;
	if (fits_read_key (fpol, TFLOAT, "BZERO",  &bzero1, 
			   cbzero,  &status))  printerror (status) ;

	nbypix = abs (bitpix) / 8 ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;

	if (pmode)
	    {
	    printf ("nbypix: %d\n", nbypix) ;
	    printf ("[1] xdim/ydim/bitpix: %d %d %d\n",
		 xdim, ydim, bitpix) ;
	    printf ("iyear: %d  imonth: %d  iday: %d\n",
		 iyear, imonth, iday) ;
	    printf ("ihour: %d  iminute: %d  isecond: %d\n",
			 ihour, iminute, isecond) ;
	    printf ("telescop: %s\n", telescop) ;
	    printf ("instrume: %s\n", instrume) ;
	    printf ("type_obs: %s\n", type_obs) ;
	    printf ("coordnam: %s\n", coordnam) ;
	    printf ("bscale1: %8.2g  bzero1: %8.2g\n", bscale1, bzero1) ;
	    }


	/*** Read FITS image data. ***/

        npix    = xdim * ydim ;
	nullval = 0 ;          /* Don't check for null values in the image. */

	/*** Allocate storage for image arrays. ***/

	aimg = (float *) malloc (xdim * ydim * sizeof (float)) ;
	aveq = (float *) malloc (xdim * ydim * sizeof (float)) ;
	pimg = (float *) malloc (xdim * ydim * sizeof (float)) ;
	vimg = (float *) malloc (xdim * ydim * sizeof (float)) ;

	xbuf = (unsigned char *) malloc (xdim * ydim * nbypix) ;

	/*** Override automatic intensity scaling. ***/
	/*** i.e., read unscaled pixel data. ***/

	scale = 1.0 ;
	zero  = 0.0 ;
	if (fits_set_bscale (fpol, scale, zero, &status)) printerror (status) ;

	fpiximg [0] = 1 ;
	fpiximg [1] = 1 ;

	/*** Read primary image array. ***/

	switch (bitpix)
	    {
	    case   8:
			cimg = (unsigned char *) xbuf ;
			if (fits_read_img (fpol, TBYTE,   fpiximg [0], npix,
			    &nullval, cimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npix; i++)
			    *(pimg + i) = (float) *(cimg + i) ;
			break ;
	    case  16:
			simg = (short  *) xbuf ;
			if (fits_read_img (fpol, TSHORT,  fpiximg [0], npix,
			    &nullval, simg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npix; i++)
			    *(pimg + i) = (float) *(simg + i) ;
			break ;
	    case  32:
			iimg = (int    *) xbuf ;
			if (fits_read_img (fpol, TINT,    fpiximg [0], npix,
			    &nullval, iimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npix; i++)
			    *(pimg + i) = (float) *(iimg + i) ;
			break ;
	    case -32:
			fimg = (float  *) xbuf ;
			if (fits_read_img (fpol, TFLOAT,  fpiximg [0], npix,
			    &nullval, fimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npix; i++)
			    *(pimg + i) = (float) *(fimg + i) ;
			break ;
	    case -64:
			dimg = (double *) xbuf ;
			if (fits_read_img (fpol, TDOUBLE, fpiximg [0], npix,
			    &nullval, dimg, &anynull, &status) )
			    printerror ( status ) ;
			for (i = 0 ; i < npix; i++)
			    *(pimg + i) = (float) *(dimg + i) ;
			break ;

	    default:	fprintf (stderr, "Invalid bitpix: %d\n", bitpix) ;
			goto next ;
	    }

	free (xbuf) ;

	/*** Copy pimg into vimg. ***/

	/*** Note: Alice's "vignetting" software assumes the inclusion
	 *** of an additive offset of 1000.  Since the Floating Point
	 *** files do NOT include this offset, it must be added here
	 *** in order for them to work properly.
	 ***/

	if (pmode)
	    printf ("bzero/bscale: %f\n", bzero1/bscale1) ;

	if (bscale1 == 1.0e-10 && bzero1 == 0.0)
	for (i = 0 ; i < npix; i++)
	    *(vimg + i) =  *(pimg + i) + 1000 ;
	else
	for (i = 0 ; i < npix; i++)
	    *(vimg + i) =  *(pimg + i) ;

	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;

	if (pmode)
	    printf ("fpol fits_movabs_hdu [2] ... status: %d\n", status) ;

	if (fits_get_hdu_num  (fpol, &hdunum))            printerror (status) ;
	if (fits_get_hdu_type (fpol, &hdutype, &status))  printerror (status) ;

	if (pmode)
	    printf ("fpol hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;


	/*** Read Mk4 binary header (stored as a FITS extension). ***/

	fpixext = 1 ;
        if ( fits_read_img (fpol, TBYTE, fpixext, extdim, &nullval, mk4h,
			    &anynull, &status) )
	    printerror ( status ) ;

	mk4hdu (mk4h, pmode) ;		/* Fill in MK4 FITS structure. */

	/* ------------------------------------------------------------------ */

	/*** Apply "vignetting" based upon a power of radial distance. ***/

/*
	dr = mk4h->dr ;
	r0 = mk4h->r0 ;
	for (iy = 0; iy < ydim; iy++)
	    {
	    radius = iy * dr + r0 ;
	    r5 = (float) powl (radius, 1.10) ;
	    printf ("radius: %7.2f  r5: %7.2f\n", radius, r5) ;
	    for (ix = 0; ix < xdim; ix++)
		*(vimg + ix + iy*xdim) = *(vimg + ix + iy*xdim) * r5 ;
	    }

	goto done ;
 */

/*
 * 	HandleNeg_pB (vimg, xdim, ydim, lspot) ;
 */


	/*** Apply radial "vignetting" to image. ***/

	switch (method)
	    {
	    case MAY2001MODEL:
		VVcopy (May2001model, aveq, NPX);
		ftemp = MyOffset ;
		Vplus (aveq, ftemp, NPX);
		ftemp = mk4h->data_offset;
		Vminus (vimg, ftemp, xdim * ydim);
		break ;
	    case MODEL:
		model_RadVect (aveq) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi_preM", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi_pre write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (aveq, sizeof (float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
		break ;
	    case CORONA:
		VVcopy (mymodel, aveq, NPX) ;
		ftemp = mk4h->data_offset - 100.0 ;
/*
		ftemp = mk4h->data_offset - 50.0 ;
		Vminus (aveq, ftemp, NPX) ;
		ftemp = mk4h->data_offset - 100.0 ;
 */
		Vminus (vimg, ftemp, xdim * ydim) ;
		break ;
/*
	    case TSCP:
		VVcopy (thd->DriftRefVect, aveq, NPX) ;
		ftemp = mk4h->data_offset - 10.0 ;
		Vminus (vimg, ftemp, xdim * ydim) ;
		break ;
 */
	    default:
		Im2AvgRadVect_Culled (vimg, aveq, xdim, ydim) ;
		fixLO (aveq, 8000.0, -1.0) ;
		fixHI (aveq,  200.0) ;
		smoothie (aveq, mk4h->Ydim, 13) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi_pre", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi_pre write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (aveq, sizeof (float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
	    }
	MVdivide (vimg, aveq, vimg, xdim, ydim) ;


	switch (method)
	    {
	    case MODEL:
		model_Resid_orig2 (aveq) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfiM", "w"))<(FILE*)0)
			{
			fprintf (stderr, "barfi write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (aveq, sizeof (float), mk4h->Ydim, fid) ;
		    fclose (fid) ;
		    }
		break ;
	    case CORONA:
		Vinitialize (aveq, 0.0, NPX) ;
		break ;
/*
	    case TSCP:
		Vinitialize (aveq, 0.0, NPX) ;
		break ;
 */
	    case MAY2001MODEL:
		Vinitialize (aveq, 0.0, NPX) ;
		break ;
	    default:
		VVcopy       (vimg, aimg,  xdim * ydim) ;
		get_min_Vect (aimg, aveq, xdim, ydim) ;
		fixLO (aveq, 0.0, 0.6) ;
		fixHI (aveq, 0.9) ;
		smoothie (aveq, mk4h->Ydim, 13) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (aveq, sizeof(float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
	    }
	MVminus (vimg, aveq, vimg, xdim, ydim) ;

	switch (method)
	    {
	    case CORONA:
		Vinitialize (aveq, 100.0, NPX) ;
		break ;
/*
	    case TSCP:
		Vinitialize (aveq, 180.0, NPX) ;
		break ;
 */
	    case MODEL:
	    default:
			/* Put a little extra zing into the outer field. */
		pf = aveq ;
		for (ii = 0; ii < mk4h->Ydim; ii++)
		    {
		    dtmp  = (double) ii ;
		    ftemp = (float) (pow (dtmp, 1.35)) ;
		    ftemp = (float) (pow (dtmp, 1.20)) ;
		    ftemp = (float) (pow (dtmp, 1.01)) ;
/*
		    if (ii > 180)
			{
			dtmp  -= 280.0 ;
			ftemp -= (float) (pow (dtmp, 1.60)) ;
			dtmp  -= 180.0 ;
			ftemp -= (float) (pow (dtmp, 1.30)) ;
			dtmp  -= 180.0 ;
			ftemp -= (float) (pow (dtmp, 1.00)) ;
		        }
 */
		    ftemp = 100.0 + 2.0 * ((float) ii) ;
		    ftemp = 100.0 + 1.0 * ((float) ii) ;
		    if (ii > 372) ftemp = 0.01 ;
		    *pf++ = ftemp ;
		    }
	    }
	MVmult (vimg, aveq, vimg, xdim, ydim) ;

done:
	/*** Set pixels inside 1.0 Rsun to a minimum value. ***/

        lozero = BOTPIX ;
        zminlo (vimg, xdim, lozero) ;

	/*** Find min/max intensities. ***/

	fmin =  1.e+30 ;
	fmax = -1.e+30 ;
	for (iy = 0; iy < ydim; iy++)
	    for (ix = 0; ix < xdim; ix++)
		{
		if (*(vimg + ix + iy*xdim) < fmin &&
		    *(vimg + ix + iy*xdim) != -30000.0) 
		    fmin = *(vimg + ix + iy*xdim) ;
		if (*(vimg + ix + iy*xdim) > fmax) 
		    fmax = *(vimg + ix + iy*xdim) ;
		}

	fmin =  1.e+30 ;
	fmax = -1.e+30 ;
	for (iy = 30; iy < 300; iy++)
	    for (ix = 0; ix < xdim; ix++)
		{
		if (*(vimg + ix + iy*xdim) < fmin) 
		    fmin = *(vimg + ix + iy*xdim) ;
		if (*(vimg + ix + iy*xdim) > fmax) 
		    fmax = *(vimg + ix + iy*xdim) ;
		}

	if (pmode)
	    printf ("fmin/fmax: %8.2e  %8.2e\n", fmin, fmax) ;


	/*** Fix and fill in the headers with new info. ***/

	switch (mk4h->ProcSteps)
	    {
	    case IS_cal               : howfar = IS_cal_slit ; break ;
	    case IS_cal_avg           : howfar = IS_cal_avg_slit ; break ;
	     
            default                   : howfar = IS_UNKNOWN ; break ;

	    }
	mk4h->ProcSteps   = howfar ;
	mk4h->data_scale  = badval ;
	mk4h->data_offset = badval ;

	if (pmode)
	    printf ("howfar: %d\n", howfar) ;

	if (pmode >= 2)
	    mk4hdp (mk4h) ;

        /*** Create output file name. ***/

	dayflag = 1 ;
	doyflag = 0 ;

        strcpy (vig_file, mk4_name (mk4h, doyflag)) ;

	fprintf (stdout, "vig_file: %s   ", vig_file) ;

	remove (vig_file) ;		/* Delete pre-existing file. */

	/* ------------------------------------------------------------------ */

	/*** Create output file. ***/

	if ( fits_create_file (&fvig, vig_file, &status) )
	    printerror ( status ) ;


	/* ------------------------------------------------------------------ */

	/*** Move to FITS primary HDU [input file]. ***/

	hdunum = 1 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;

	/*** Copy Primary FITS header to output file. ***/

	if (fits_get_hdrpos (fpol, &nkeys, &keypos, &status))
	    printerror (status) ;

	for (ii = 1; ii <= nkeys; ii++)
	    {
	    if (fits_read_record (fpol, ii, card, &status))
		printerror (status) ;
	    if (pmode >= 2)
	        printf ("%s\n", card) ;		/* Print keyword "card". */
	    if (fits_write_record (fvig, card, &status))
		printerror (status) ;
	    }

	/*** Update keywords in primary HDU header. ***/

	bitpix = -32 ;
	sprintf (type_obsv, "%s-VIG", type_obs) ;
	bscale = 1.0 ;
	bzero  = 0.0 ;
        datamin = fmin ;
	datamax = fmax ;

	udatamin = datamin ;
	udatamax = datamax ;
	sprintf (cdatamin, "MINIMUM DATA VALUE (%8.2e)", udatamin) ;
	sprintf (cdatamax, "MAXIMUM DATA VALUE (%8.2e)", udatamax) ;

	dispmin = fmin ;
	dispmin = -100.0 ;
	dispmin = -100.0 * bscale1 + bzero1 ;
	if (bscale1 == 1.0) dispmin = -1.0e-8 ;
	dispmin = 0.0 ;

	dispmax = fmax ;
	dispmax = 750.0 * bscale1 + bzero1 ;
	if (bscale1 == 1.0) dispmax = 7.5e-8 ;
	dispmax = 750.0 ;
	dispmax = fmax ;

	udispmin = dispmin ;
	udispmax = dispmax ;
	sprintf (cdispmin, "MIN. DISPLAY VALUE (%8.2e)", udispmin) ;
	sprintf (cdispmax, "MAX. DISPLAY VALUE (%8.2e)", udispmax) ;

	switch (pixtype [0])
	    {
	    case 'c':	bitpix =   8 ;
			printf ("case c\n") ;
			strcpy (cbitpix, "1-BYTE UNSIGNED CHARACTERS") ;
			break ;
	    case 's':	bitpix =  16 ;
			printf ("case s\n") ;
			strcpy (cbitpix, "2-BYTE SIGNED INTEGERS") ;
			break ;
	    case 'i':	bitpix =  32 ;
			printf ("case i\n") ;
			strcpy (cbitpix, "4-BYTE SIGNED INTEGERS") ;
			break ;
	    case 'f':	bitpix = -32 ;
			printf ("case f\n") ;
			strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
			break ;
	    case 'd':	bitpix = -64 ;
			printf ("case d\n") ;
			strcpy (cbitpix, "8-BYTE DOUBLE PRECISION") ;
			break ;
	    default :	bitpix = -32 ;
			printf ("case default [f]\n") ;
			strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
	    }

	if (fits_update_key (fvig, TINT,    "BITPIX",   &bitpix,   cbitpix,
			     &status)) printerror ( status );
	if (fits_update_key (fvig, TSTRING, "TYPE-OBS", type_obsv, 0,
			     &status)) printerror ( status );
	if (fits_update_key (fvig, TFLOAT,  "BSCALE",   &bscale,   0,
			     &status)) printerror ( status );
	if (fits_update_key (fvig, TFLOAT,  "BZERO",    &bzero,    0,
			     &status)) printerror ( status );
	if (fits_update_key (fvig, TFLOAT,  "DATAMIN",  &datamin,  cdatamin,
			     &status)) printerror ( status );
	if (fits_update_key (fvig, TFLOAT,  "DATAMAX",  &datamax,  cdatamax,
			     &status)) printerror ( status );
	if (fits_update_key (fvig, TFLOAT,  "DISPMIN",  &dispmin,  cdispmin,
			     &status)) printerror( status );
	if (fits_update_key (fvig, TFLOAT,  "DISPMAX",  &dispmax,  cdispmax,
			     &status)) printerror( status );

	/*** Get BITPIX & NAXIS from output file. ***/

	if (ffgidt (fvig, &bitpix, &status)) printerror (status) ; ;
	if (ffgidm (fvig, naxis, &status)) printerror (status) ; ;

	if (pmode)
	    {
	    printf ("fvig: bitpix = %d\n", bitpix) ;
	    printf ("fvig: naxis  = %d  %d\n", naxis [0], naxis [1]) ;
	    }

	if (pmode >= 3)
	    {
	    ix = 0 ;
	    for (iy = 0; iy < ydim; iy += 10)
	        printf ("ix/iy: %3d %3d  pimg: %8.2e  vimg: %8.2e\n",
		        iy, ix, *(pimg + ix + iy*xdim), *(vimg + ix + iy*xdim));
	    }

	/*** Override automatic output scaling. ***/

	fpiximg [0] = 1 ;
	fpiximg [1] = 1 ;

	scale = 1.0 ;
	zero  = 0.0 ;
	if (fits_set_bscale (fvig, scale, zero, &status)) printerror (status) ;

	/*** Convert floating point image to specified output pixel type. ***/
	/*** Write vignetted image array into the FITS data array. ***/

	free (aimg) ;
	free (aveq) ;
	free (pimg) ;

	switch (pixtype [0])
	    {
	    case 'c':
			printf ("case c\n") ;
			bitpix = 8 ;
			strcpy (cbitpix, "1-BYTE UNSIGNED CHARACTERS") ;
			cimg = (unsigned char *) malloc (npix * sizeof (unsigned char)) ;
			for (i = 0 ; i < npix; i++)
			    *(cimg + i) = (unsigned char) *(vimg + i) ;
			if (fits_write_img (fvig, TBYTE, fpiximg [0], npix,
			    cimg, &status) ) printerror ( status ) ;
			free (cimg) ;
			break ;
	    case 's':
			printf ("case s\n") ;
			bitpix = 16 ;
			strcpy (cbitpix, "2-BYTE SIGNED INTEGERS") ;
			simg = (short *) malloc (npix * sizeof (short)) ;
			for (i = 0 ; i < npix; i++)
			    *(simg + i) = (short) *(vimg + i) ;
			if (fits_write_img (fvig, TSHORT, fpiximg [0], npix,
			    simg, &status) ) printerror ( status ) ;
			free (simg) ;
			break ;
	    case 'i':
			printf ("case i\n") ;
			bitpix = 32 ;
			strcpy (cbitpix, "4-BYTE SIGNED INTEGERS") ;
			iimg = (int *) malloc (npix * sizeof (int)) ;
			for (i = 0 ; i < npix; i++)
			    *(iimg + i) = (int) *(vimg + i) ;
			if (fits_write_img (fvig, TINT, fpiximg [0], npix,
			    iimg, &status) ) printerror ( status ) ;
			free (iimg) ;
			break ;
	    case 'f':
			printf ("case f\n") ;
			bitpix = -32 ;
			strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
			fimg = (float *) malloc (npix * sizeof (float)) ;
			for (i = 0 ; i < npix; i++)
			    *(fimg + i) = (float) *(vimg + i) ;
			if (fits_write_img (fvig, TFLOAT, fpiximg [0], npix,
			    fimg, &status) ) printerror ( status ) ;
			free (fimg) ;
			break ;
	    case 'd':
			printf ("case d\n") ;
			bitpix = -64 ;
			strcpy (cbitpix, "8-BYTE DOUBLE PRECISION") ;
			dimg = (double *) malloc (npix * sizeof (double)) ;
			for (i = 0 ; i < npix; i++)
			    *(dimg + i) = (double) *(vimg + i) ;
			if (fits_write_img (fvig, TDOUBLE, fpiximg [0], npix,
			    dimg, &status) ) printerror ( status ) ;
			free (dimg) ;
			break ;
	    default:
			printf ("case default: floating point\n") ;
			bitpix = -32 ;
			strcpy (cbitpix, "4-BYTE FLOATING POINT") ;
			for (i = 0 ; i < npix; i++)
			    *(fimg + i) = (unsigned char) *(vimg + i) ;
			if (fits_write_img (fvig, TFLOAT, fpiximg [0], npix,
			    fimg, &status) ) printerror ( status ) ;
			free (fimg) ;
	    }


	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpol, hdunum, &hdutype, &status))
	    printerror (status) ;

	/*** Copy HDU2 to output file. ***/

	morekeys = 1 ;
	if (fits_copy_hdu (fpol, fvig, morekeys, &status) )
	    printerror (status);

	if (fits_get_hdu_num  (fpol, &hdunum))  printerror (status) ;
	if (fits_get_hdu_type (fpol, &hdutype, &status)) printerror (status) ;

	if (pmode)
	    printf ("fpol hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;


	/*** Read keywords from FITS extension header. ***/

	if (fits_read_key (fpol, TINT, "BITPIX", &bitpix,
			   comment, &status))  printerror (status) ;
	nbypix = abs (bitpix) / 8 ;

	if (fits_read_key (fpol, TINT, "NAXIS1", &extdim,
			   cnaxis1, &status))  printerror (status) ;

	if (pmode)
	    printf ("fpol [2] extdim/bitpix: %d %d\n", extdim, bitpix);

	/* ------------------------------------------------------------------ */

	/*** Move to FITS extension HDU [output file]. ***/

	if (pmode)
	    printf ("Update keywords [fvig, hdu2]\n") ;

	sprintf (xtension, "IMAGE   ") ;
	sprintf (extname, "MK4_HSKP") ;

	hdunum = 2 ;
	if (fits_movabs_hdu (fvig, hdunum, &hdutype, &status))
	    printerror (status) ;

	if (fits_update_key (fvig, TSTRING, "XTENSION", &xtension, 0, &status))
	     printerror ( status );
	if (fits_update_key (fvig, TINT,    "PROCSTEP", &howfar,   0, &status))
	     printerror ( status );
	if (fits_update_key (fvig, TSTRING, "EXTNAME", &extname, 0, &status))
	     printerror ( status );

	/*** Write binary header into the FITS extension [output file]. ***/

	npxext = 1024 ;

	if ( fits_write_img (fvig, TBYTE, fpixext, npxext, mk4h, &status) )
	    printerror ( status ) ;

	/*** Close both input & output FITS files. ***/

	if (fits_close_file (fpol, &status) ||
	    fits_close_file (fvig, &status))		/* close files */
	    printerror ( status );

next:

	free (aveq) ;
	free (aimg) ;
	free (pimg) ;
	free (vimg) ;
	free (xbuf) ;

	(void) time (&etime) ;
	fprintf (stderr, 
	"Single image time used: %ld seconds\n", etime - btime) ;
        }

    (void) time (&endtime) ;
    fprintf (stderr, "Total time used  :    %ld seconds\n", endtime-begtime) ;

    exit (0) ;
}

/* -------------------------------------------------------------------------- */

void
prsum ()

{

    static char help [] = "\
polvigf [-L] [-C] [-M [-O offset]] [-p{type}] [polar file1] ... [polar fileN]\n\
\n\
Apply a \"vignetting\" function to polar coordinate Mk4 FITS image[s]. \n\
\n\
Example: polvigf 20010621.180052.mk4.cpb.fts \n\
Example: polvigf -L -M -O 100. `ls -1 *.mk4.cpB.fts`\n\
\n\
Flags          Description\n\
-----          -----------\n\
-L		Only fixes neg pB values down to pixel BOTPIX instead of\n\
		pixel 32 (the default).\n\
-C		Will use a coronal model as the vignetting function\n\
-M		Will use David Elmore\'s May 2001 coronal model for the\n\
		vignetting function.\n\
-O offset	The offset to use in the May 2001 model, where:\n\
		pBvig = pB / (May_2001_model + offset).\n\
-pc		ouput pixel type: unsigned char   ( 8 bits) \n\
-ps		ouput pixel type:   signed int    (16 bits) \n\
-pi		ouput pixel type:   signed int    (32 bits) \n\
-pf		ouput pixel type:   signed float  (32 bits) \n\
-pd		ouput pixel type:   signed double (64 bits) \n\
\n\
" ;

/*
    static char help [] = "\
polvigf [-L] [-C] [-T tscpfile] [-M [-O offset]] [polar file1] [polar file2] ... [polar fileN]\n\
\n\
Example: polvigf 92d004.17_28.mk4.pB  92d004.17_32.mk4.pB\n\
Example: polvigf -L -M -O 100. `ls -1 *.mk4.pB`\n\
\n\
\n\
Flags          Description\n\
-----          -----------\n\
-L		Only fixes neg pB values down to pixel BOTPIX instead of\n\
		pixel 32 (the default).\n\
-C		Will use a coronal model as the vignetting function\n\
-T tscpfile	Will use the DriftRefVect from the tscp file at the\n\
		vignetting function.\n\
-M		Will use David Elmore\'s May 2001 coronal model for the\n\
		vignetting function.\n\
-O offset	The offset to use in the May 2001 model, where:\n\
		pBvig = pB / (May_2001_model + offset).\n\
		default: float (32 bits) \n\
\n\
" ;
 */

       fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void
fixLO (Vect, flim, frepla)

float *Vect ;
float flim ;
float frepla ;
{
    int ii ;
    float *pf, fval ;

    ii   = 50 ;
    pf   = Vect+ii ;
    while ((ii > 0) && (*pf > flim)) { ii-- ; fval = *pf ; pf-- ; }

    if (frepla > 0.0) fval = frepla ;

    while (ii > 0)                { ii-- ; *pf = fval ; pf-- ; }


}

/* -------------------------------------------------------------------------- */

void
fixHI (Vect, flim)

float *Vect ;
float flim ;
{
    int ii ;
    float *pf, fval ;

    ii   = 260 ;
    pf   = Vect + ii ;
    while ((ii < 384) && (*pf > flim)) { ii++ ; fval = *pf ; pf++ ; }

    while (ii < 384)                { ii++ ; *pf = fval ; pf++ ; }

}

/* -------------------------------------------------------------------------- */

void
model_RadVect_table (Vect)

float *Vect ;
{
    int ii ;
    float *pf, *pm, fval ;
    float hlim, llim ;

    hlim = 10000. ;
    llim = 1040. ;

    pf = Vect ;
    pm = mymodel ;
    for (ii = 0; ii < NPX ; ii++)
	{
	fval  = *pm ;
	fval *= 0.3 ;
	if     (fval <=    0.0) fval = hlim ;
	else if (fval >  hlim ) fval = hlim ;
	else if (fval <  llim ) fval = llim ;
	else ;
	*pf = fval ;
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

void
model_RadVect (Vect)

float *Vect ;
{
    int ii ;
    float *pf, fval, tval, slp, ycpt, y2, y1, x2, x1 ;
    float hlim, llim ;

    hlim = 10000.0 ;
    llim = 1040.0 ;
    llim = 1000.0 ;
    llim = 1040.0 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	tval = (float) ii ;
	fval = tval * 0.1 - 4.2 ;
	fval = 500.0 + 12000.0 / fval ;
	if (ii >= 40 && ii <= 70) fval *= 1.0 * ( tval) / ( 70.0) ;
	if      (fval <=    0.0) fval = hlim ;
	else if (fval >   hlim ) fval = hlim ;
	else if (fval <   llim ) fval = llim ;
	else ;
	*pf = fval ;
	pf++ ;
        }


    x1 = 220.0 ;
    ii = (int) x1 ;
    y2   = *(Vect + 383) ;   x2 = 383.0 ;
    y1   = *(Vect + ii ) +10.0 ;
    slp  = (y2 - y1) / (x2 - x1) ;
    ycpt = y1 - slp*x1 ;
    ii = (int) x1 ;
    pf = Vect + ii ;
    for (ii = (int) x1; ii < NPX; ii++)
	{
	tval = (float) ii ;
	fval = slp * tval + ycpt ;
	fval = (fval - (tval-x1) * 1.00) ;
	if (fval < llim ) fval = llim ;
	*pf = fval ;
	pf++ ;
        }
}


/* -------------------------------------------------------------------------- */

void
model_Resid_orig (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    double dval ;
    double dlim, hlim, ov600 ;

    hlim= 1.0 ;
    ov600 = 1.0 / 600.0 ;
    ov600 = 1.0 / 900.0 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	dval = (float) ii * pi * ov600 ;
	/* dval = sin (dval) * 0.5 + 0.4 ; */
	dval = sin (dval) * 0.6 + 0.5 ;
	if (dval > hlim ) dval = hlim ;
	*pf = (float) dval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

void
model_Resid_orig2 (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    float fval ;


    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	fval = ((float) ii -380.0) * 0.0025 ;
	fval = ((float) ii -400.0) * 0.0025 ;
	fval = 1.0 - fval * fval ;
	*pf = fval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

void
model_Resid (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    float fval ;
    float hlim ;

    hlim = 1.2 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	fval = (float) ii * 0.0015 + 0.5 ;
	if (fval > hlim ) fval = hlim ;
	*pf = fval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/

void
printerror (status)

int status ;

{
    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit( status );    /* terminate the program, returning error status */
        }
    return;
}

/*--------------------------------------------------------------------------*/
