#include <stdio.h>
#include <string.h>	
#include <stdlib.h>
#include <ctype.h>

	/*	FITS Product Conformance Tester  -  driver	*/
		/*	Basic FITS Test Prototype		*/
		/*		12 April 1993		*/

/****************************************************************
 *                                                              *
 * 		Have you read the instructions in               *
 *		FITS_TESTER_INSTRUCTIONS.TXT?                   *
 *                                                              *
 *       If not, read them first before attempting to use       *
 *       		this software package.                  *
 *                                                              *
 ****************************************************************/


#define TRUE          1
#define FALSE         0 

#define GOOD          0
#define UNDEFINED     1

#define TOOFLEX       2
#define FAIR	      3
#define FLAWED        4
#define INDOUBT       5
#define NONSTAND      6
#define BADHEDR       7
#define BADREC        8
#define NOTFITS       9

#define BADSPEC       5
#define NOFILE        6

#define DATAFULL      5
#define KEYFULL       8

#define FPCTERR       9
#define NO_OUT        7

#define RECLEN     2880
#define MAXAXES     999
#define CARDLEN      80
#define KEYLEN        8

#define TOOFAR    32767

#define NOREC        -1

#define LOC_SIMPLE    0
#define LOC_BITPIX    1
#define LOC_NAXIS     2

#define COLUMN_LOGICAL 29

#define I4	     32
#define I2 	     16
#define BYT	      8
#define FLT         -32
#define DBL         -64

#define INDST        65
#define INDEND      128

#define MREC          0

/*****************************************************************/ 
/*	Adjust these typedefs as appropriate to ensure           */
/*	 types on given machine are as below			 */
typedef short Int2;	/* 	Two-byte integer		 */
typedef long Int4;	/*	Four-byte integer		 */
typedef float Flt32;	/*	32-bit floating point		 */
typedef double Dbl64;   /*	64-bit floating point		 */
/*****************************************************************/

int bitpix,naxis[MAXAXES+1];	/* values for BITPIX,NAXIS,NAXISn	*/
char *template[MAXAXES+5];   /* locates template for required keyword names */
struct arr4
  {
  int head;
  int infile;
  int space;
  int datafile;
  }; 
int settempl(char *baselist[]);
int getrec(FILE *filename, char record[]);
int checkord(char keyw[], int sarr[], long cardno, int *locord);
void numstr(char strname[], int numadd, int locstart);
int chekblnk(char *cardname, char card[], int startcol, int endcol);
int cheksimp(char card[]);
int ckbitpix(char card[], char bpval[]);
int chknaxis (char card[]);
int chkaxisn(char card[], int axisno);
int summkey(int keyarr[]);
int getdata(FILE *testname, char *array, long numbmems); 
int getspec(char *args[], int inspecs[]);
int creatout(char *array, char *name, long size, int specs[]);
int chkdfill(char *arrayloc, int bitpix);
short flip2(short vaxorder);
long flip4(long vaxorder);
int summary(struct arr4 errinds);


int main(int argc,char *argv[])
{

char record[RECLEN+1], card[CARDLEN+1], keyword[KEYLEN+1];
char bpval[4];
char *cardbeg, *array;
int noinrec,ncard,n,nrecs,ktfill=0;
int cardlist=FALSE, incards=TRUE;
int keystat,simpstat,bpstat,bpok,naxstat,axnstat,fillstat,outstat;
int headstat,datastat,specstat;
long mmembs,nbytes,mspace;
int order,filla,fillz,locseq;
int kt,kn,nx,nmtx,kx,ikc,minax,iast;
int keychek[MAXAXES+3],abitpix,inspecs[40];
int *ordloc;
long nctot=0;
size_t nlocs;
struct arr4 errlev = {UNDEFINED,GOOD,GOOD,UNDEFINED};
FILE *testfile;

char *tempbase[5] =
    {"SIMPLE  ","BITPIX  ","NAXIS   ","EXTEND  ","END     "};
char *fillseq =
    {"no.    fill"};
	/* set up list of standard keywords */
if((errlev.space = settempl(tempbase)) != GOOD)
  {	
  printf("\nUnable to allocate space for standard keyword list.\n");
  printf("FITS Product Conformance Tester cannot be run.\n");
  return(summary(errlev));
  }
ordloc = &order;    /*	identifies card name from template	*/
for (ikc=0;ikc<MAXAXES+3;ikc++)
    keychek[ikc] = FALSE; 	/*	BITPIX -->NAXISn	*/
printf("Input data file name is (maximum 30 characters recommended)\n");
printf("%s\n\n",argv[1]);
printf("Output data file name is \n");
printf("%s\n\n",argv[2]);
if(atoi(argv[3]) > 0)
  {
  cardlist = TRUE;
  printf("Non blank header card images will be listed.\n");
  }
if((testfile = fopen(argv[1],"r")) == NULL)
  {
  errlev.infile = NOFILE;
  printf("\n\nNo input file found.\n");
  return(summary(errlev));
  }
while(incards==TRUE)				/*	Check header	*/
  {
  if((noinrec=getrec(testfile,record)) == RECLEN) /* obtain header record */
    {					/*	proper 2880-byte length	*/
    cardbeg = &record[0];
    for(ncard=0;ncard<36;ncard++)
      {
      strncpy(card,cardbeg,CARDLEN);    	/* extract card images */    
      card[CARDLEN] = '\0';
      if (incards == TRUE)		/*    still reading keywords	*/
        {
        strncpy(keyword,card,KEYLEN);	 	/*   extract keyword   */
	keyword[KEYLEN] = '\0';
	if(cardlist==TRUE)  		  
	  printf("%s\n",card);		
	if(strncmp(template[MAXAXES+4],keyword,4) != 0) /* at END? */
	  {				/*	Not at end	*/
  	  if((keystat=checkord(keyword,keychek,nctot,ordloc)) > 
	      errlev.head)
	    errlev.head=keystat;
	  if(*ordloc <= MAXAXES+LOC_NAXIS)
	    {
	    if (card[KEYLEN] != '=')	 /*  check column 9 for "="   */
	      {
	      printf("\n    ***   No equal sign in column 9"); 
	      printf(" after %s keyword.   ***\n\n",keyword);
	      if(errlev.head < FLAWED)
		errlev.head = FLAWED;
	      }
            if (card[KEYLEN+1] != ' ')   	/*  check column 10 for blank  */
	      {
	      printf("\n   ***   Unexpected %c found in ",card[9]);
	      printf("column 10 after %s keyword.   ***\n",keyword);
	      printf("     ***   Column should have"); 
	      printf(" been blank.   ***\n\n");
	      if(errlev.head < FLAWED)
		errlev.head = FLAWED;
	      }
					/*	extract value	*/
	    if(*ordloc==LOC_SIMPLE)		/*     SIMPLE=T?    */
	      {
	      if((simpstat=cheksimp(card)) > errlev.head)
		errlev.head = simpstat;
	      if (simpstat>=NONSTAND)
                {
		printf("   ***  Tests will continue, but");
		printf(" diagnostics may not be meaningful.   ***   \n\n");
		}
	      }
	    else if(*ordloc==LOC_BITPIX)		/*     BITPIX    */
	      {
	      if((bpstat=ckbitpix(card,bpval)) > errlev.head)
		errlev.head=bpstat;
	      if(bpstat < BADHEDR)      	/*	get value	*/
		{
		bpok=FALSE;
		bitpix = atoi(bpval);
		for (kt=1; kt<=4; kt *= 2)
		  if(bitpix == 8*kt)
		    bpok=TRUE;
		for (kn=1; kn<=2; kn *= 2)
		  if((-bitpix) == 32*kn)
		    bpok=TRUE;
		if(bpok == FALSE)
		  {
		  printf("\n   ***   Invalid value"); 
		  printf(" of BITPIX: %d;",bitpix);
		  printf(" unable to read data array.   ***\n\n"); 
		  errlev.head = BADHEDR;
		  }
		}
	      }
       	    else if(*ordloc==LOC_NAXIS)		/*    NAXIS    */
	      {
	      if((naxstat=chknaxis(card)) > errlev.head)
		errlev.head = naxstat;
 	      if(naxis[0] == 0)
		printf("\n     No primary data array.\n\n");
	      }
	    else if(*ordloc != LOC_NAXIS+MAXAXES)	/*    NAXISn    */
	      {
	      n = *ordloc - LOC_NAXIS;
	      if((axnstat=chkaxisn(card,n)) > errlev.head)
		errlev.head = axnstat;
	      if(axnstat >= BADHEDR)
		{
		printf("\n   ***   Unable to read length for"); 
		printf(" axis %d;\n  will not be able to",n); 
		printf(" read data.   ***\n\n");	
		}
	      }
	    }
          ++nctot; 
	  }         
	else				      /*	at END	*/
	  {
	  if (cardlist != TRUE)
	    printf("END card image reached.\n");
	  filla=4;
	  fillz=CARDLEN-1;
	  if((fillstat= chekblnk(template[MAXAXES+4],card,filla,fillz))
	      > errlev.head)
	    errlev.head = fillstat;
	  incards=FALSE;
	  }
	}
      else				 /*	reading fill		*/
	{ 
	ktfill++;
	filla=0;
	fillz=CARDLEN-1;
	locseq=4;
	numstr(fillseq,ktfill,locseq);
	if((fillstat=chekblnk(fillseq,card,filla,fillz)) > errlev.head)
	  errlev.head=fillstat;
	}
      cardbeg += CARDLEN;
      }
    }
  else	    {	
    nrecs = (nctot+1)/36;
    printf("\n ");
    for (iast=1;iast<=7;iast++)
      printf("********");
    printf("****\n");
    if(noinrec == NOREC)
      printf(" *       Premature end of file");
    else
      printf(" * Header record of %4d bytes",noinrec);  	
    printf(" encountered after %d records.   *\n",nrecs);
    printf(" *                    Processing terminated.");		
    printf("\n ");
    for (iast=1;iast<=7;iast++)
      printf("********");
    printf("****\n\n");
    errlev.head = BADREC;
    incards = FALSE;
    }
  }
if((headstat=summkey(keychek)) > errlev.head) 
  errlev.head = headstat; 
if (errlev.head==UNDEFINED)
  errlev.head = GOOD;
	/* 	End header processing	 
		Begin data processing	*/
if(errlev.head < BADHEDR)
  {
  minax = 1;		/*  check for NAXIS or NAXISn equal to zero  */
  for (kx=0; kx<=naxis[0]; kx++)
    if(naxis[kx] < minax) 
  minax = naxis[kx];
  if(minax > 0)	/*	List properties of data array from header	*/
    {
    if (errlev.head == NONSTAND)
      {
      printf("\nWARNING:\n");
      printf("SIMPLE = F; file departs from standard FITS in some"); 
      printf(" significant way.\n");
      printf("An attempt will be made to read the data but \n");
      printf(" departures from standard FITS may cause failure.\n\n");
      }
    if (errlev.head == INDOUBT)
      {
      printf("\nWARNING:\n");
      printf("Severe problems were encountered in reading header"); 
      printf(" values.\n");
      printf("Description of array is probably incorrect.\n");
      printf("Run will continue but results may not be meaningful.\n\n");
      }
    if (errlev.head == FLAWED)
      {
      printf("\nAdvisory:\n");
      printf("Header syntax differs from standard.\n"); 
      printf("An error in the array description is possible.\n\n");
      }
    abitpix = abs(bitpix);
    printf("Properties of primary data array inferred from header are\n");
    if(bitpix < 0)
      printf("%d-bit floating point values.\n",abitpix);
    else
      {
      printf("%d-bit ",bitpix); 
      if(bitpix==BYT)
	printf("unsigned ");
      printf("integer values.\n");
      }
    for (nx=1; nx<= naxis[0]; nx++)
      printf("Length of axis %d is %d elements.\n",nx,naxis[nx]);
		/* 	Get matrix size 	*/ 
    mmembs=1;
    for (nmtx=1; nmtx<=naxis[0]; nmtx++)
      mmembs *= naxis[nmtx];
    nbytes = abitpix/8;
    nlocs = mspace = mmembs*nbytes;		/*	allocate array	*/
    if ((array = (char *) malloc(nlocs)) != NULL)
      {
      if(bitpix>0)
	{
	errlev.datafile = datastat=getdata(testfile,array,mspace);
    	if((specstat = getspec(argv,inspecs)) < BADSPEC) 
	  {
          if(datastat < BADREC)   	/*	Organize output	*/
	    {
	    if (inspecs[10] != 0 && 
 	      ((outstat = creatout(array,argv[2],mspace,inspecs))
		> errlev.datafile))
		    errlev.datafile = outstat;
	    }
	  else 
  	    {		
	    printf("   ***   Data record of"); 		
	    printf(" improper size encountered   ***.\n");  	
	    printf("     ***   Processing terminated.  ***\n");
	    }
   	  }
        else
          {
          printf("PROBLEM IN SPECIFICATIONS FOR OUTPUT.\n");
          printf("No output array will be produced.\n");
          errlev.infile = specstat;
          }
        }
      else
	{
	printf("\nSoftware not yet able to handle");
	printf(" floating point data.\n");
	}
      }
    else 
      {
      errlev.space = DATAFULL;
      printf("\nUnable to allocate space for output array.\n");
      printf("Data file will not be read.\n\n");
      }
    }
  else 
    printf("\nHeader specifies no primary data array.\n");
  }
else
  {
  printf(" *****************************************\n");
  printf(" *      Critical error in header.        *\n"); 
  printf(" *     Data array will not be read.      *\n");
  printf(" *****************************************\n");
  }
free(array);
return(summary(errlev));
}
/*				--O--				*/

/* settempl: set up array of keyword templates			*/
int settempl(char *base[])
{
extern char *template[];
char name[KEYLEN+1], namen[KEYLEN+1];
int i,j,n,locno,tempstat=GOOD;

for (i=0;i<=LOC_NAXIS&&tempstat==GOOD;i++) 	/*  SIMPLE - NAXIS	*/
  {
  strncpy(name,base[i],KEYLEN);
  name[KEYLEN] = '\0';
  if((template[i] = (char *) malloc(sizeof(name)))!= NULL)   
    strncpy(template[i],name,KEYLEN+1);
  else
    tempstat=KEYFULL;
  }
for (n=1;n<=MAXAXES&&tempstat==GOOD;n++)
  {				/*	NAXISn		*/
  strncpy(namen,base[2],KEYLEN); 	
  namen[KEYLEN] = '\0';	
  locno=5; 
  numstr(namen,n,locno);
  if((template[n+LOC_NAXIS] = (char *) malloc(sizeof(namen))) != NULL)   
    strncpy(template[n+LOC_NAXIS],namen,KEYLEN+1);
  else
    tempstat=KEYFULL;
  }
for (j=3;j<=4&&tempstat==GOOD;j++) 		 /*  EXTEND and END	*/
  {
  strncpy(name,base[j],KEYLEN);
  name[KEYLEN] = '\0';
  if((template[j+MAXAXES] = (char *) malloc(sizeof(name))) != NULL)   
    strncpy(template[j+MAXAXES],name,KEYLEN+1);
  else 
    tempstat=KEYFULL; 
  }
return tempstat;
}
/*				-o-				*/

/* getrec: read record, return length 				*/
int getrec (FILE *fitsfile, char record[])
{
int c, length=0;

while (length < RECLEN && (c=getc(fitsfile)) !=EOF)
    record[length++] = c;
if(c == EOF && length == 0)
    length = NOREC;
else
    record[RECLEN] = '\0';
return length;
}

/*				-o-				*/

/* checkord: identify required keywords	and check for proper order*/
int checkord(char keyword[],int keyind[],long nocard,int *ordloc)
{
extern char *template[];
extern int naxis[];
char standard[KEYLEN+1],expected[KEYLEN+1];
char *unk = {"another "};
char *past = {"optional"};
int keystat=GOOD,indnaxis;
int ktreq=0;
int found=FALSE, match;
*ordloc = TOOFAR;	 
indnaxis = keyind[LOC_NAXIS];                      
				/*  identify required keywords	*/
while (found==FALSE && ktreq <= MAXAXES+LOC_NAXIS)  
  {		/* loop through template */
  strncpy(standard,template[ktreq],KEYLEN);
  standard[KEYLEN] = '\0';
  if((match = strncmp(standard,keyword,KEYLEN)) == 0) /* compare with each */	
    {
    found = TRUE;			/* required keyword identified*/
    *ordloc = ktreq;
    }
  else
    ktreq++;						/* keep trying */
  }
if(nocard==0 && *ordloc!=LOC_SIMPLE)	/*	if checking first line	*/ 
  {
  printf("\n   ***   First eight bytes do not"); 
  printf(" contain SIMPLE  .   ***\n");
  keystat = NOTFITS;
  }
if(*ordloc <= MAXAXES+LOC_NAXIS)	/* 	match was found		*/
  {			
  if(keyind[ktreq] == FALSE)  	
    keyind[ktreq] = TRUE;	/*     note presence	*/
  else				/*	or duplication	*/
    {
    *ordloc = TOOFAR;
    printf("\n   ***  %s keyword appears ",keyword);
    printf("more than once   ***\n");
    printf("        ***             First value used.   ***\n"); 
    keystat = FLAWED;
    }
  if(ktreq != nocard)			/* if card out of order	*/
    { 			/*  where required keyword is expected	*/
    if (keystat < FLAWED) 
	keystat = FLAWED;
    if(nocard <= 1001)
      {
      if(nocard<= 2)
	strncpy(expected,template[nocard],KEYLEN);
      else  
	{
	if(indnaxis==TRUE)		/*   have value for NAXIS   */	  
	  {
	  if((nocard<=(naxis[0]+2)))
	    strncpy(expected,template[nocard],KEYLEN);
	  else
	    strncpy(expected,past,KEYLEN);
	  }
	else			/*	value of NAXIS unknown	*/ 
	  strncpy(expected,unk,KEYLEN);
	}
      } 
    else 
      strncpy(expected,past,KEYLEN);
    expected[KEYLEN] = '\0';
    printf("\n   ***         %s keyword found where ",keyword);
    printf("%s would be expected.   ***\n\n",expected); 
    }
  }
else 			/*	match was not found	*/
  {		
  if((nocard < 3) || ((keyind[2]==TRUE) && (nocard < naxis[0]+3))) 
    {		/*	required keyword was expected   */
    strncpy(expected,template[nocard],8);
    expected[KEYLEN] = '\0';
    printf("\n   ***         %s keyword found where ",keyword);
    printf("%s would be expected.   ***\n\n",expected); 
    if (keystat < FLAWED) 
      keystat = FLAWED;
    }
  }  
return keystat;
}

/*				-o-				*/

/* numstr: Add a number to text as a string 			*/
void numstr (char basetext[], int idigits, int inx)
{
int idigp, idig100, idig10;

idigp = idigits;
if(idigits > 99)
     {
     idig100 = idigits/100;
     basetext[inx++] = '0' + idig100;
     idigp -= 100*idig100;
     }
if(idigp > 9)
     {
     idig10 = idigp/10;
     basetext[inx++] = '0' + idig10;
     idigp -= 10*idig10;
     }
else if(idigp != idigits)
     basetext[inx++] = '0';
basetext[inx] = '0' + idigp;
return;
}

/*				-o-		 		*/

/* cheksimp: checks value of SIMPLE keyword				*/ 	
int cheksimp(char card[ ])
{
extern char *template[];
int blnka=KEYLEN+2, blnkz=COLUMN_LOGICAL-1;
int status=GOOD, blnkstat;
int nval;

if((blnkstat=chekblnk(template[0],card,blnka,blnkz)) > status)
  status = blnkstat;
if((nval=card[COLUMN_LOGICAL]) != 'T')
  {
  if (nval == 'F')
    {
    printf("\nSIMPLE=F; FITS-like file.\n");
    printf("Not true FITS.\n");
    printf("Cannot guarantee data will be read properly.\n\n");
    if(status < NONSTAND) 
      status = NONSTAND;
    }
  else
    {
    printf("\n   ***   SIMPLE does not have value T;");
    printf(" value is %c   ***\n\n", nval);
    if(status < NOTFITS) 
      status = NOTFITS;
    }
  }
return status;
}

/*				-o-				*/

/*	ckbitpix:  check for permitted BITPIX values		*/
int ckbitpix(char card[], char bpval[])
{
extern char *template[];
char sb,v10;
int blnkstat,blnka=KEYLEN+2,blnkz=26;
int bpstat = GOOD;
if((blnkstat=chekblnk(template[1],card,blnka,blnkz)) > GOOD)
  bpstat = blnkstat;
if((sb = bpval[0] = card[27]) !=' ' && sb != '-')   /*   sign   */ 
  {    
  printf("\n   ***   Unexpected %c in column 28 of", card[27]);
  printf(" BITPIX line.   ***\n      Assumed positive.\n\n");
  bpval[0] =' '; 
  if(bpstat < INDOUBT) 
    bpstat = INDOUBT;
  }
if(!isdigit(v10 = bpval[1]=card[28]) && v10 !=' ')	/*    number    */
  {
  printf("\n   ***   Unexpected %c in column 29 of", card[28]);
  printf(" BITPIX line.   ***\n");
  printf("      ***   Will be unable to read data array.   ***\n\n");
  if(bpstat < BADHEDR) 
    bpstat = BADHEDR;
  }
if(!isdigit(bpval[2]=card[29]))
  {
  printf("\n   ***   Unexpected %c in column 30 of", card[29]);
  printf(" BITPIX line.   ***\n");
  printf("      ***   Will be unable to read data array.   ***\n\n");
  if(bpstat < BADHEDR) 
    bpstat = BADHEDR;
  }
bpval[3] = '\0';
return bpstat;
}

/*				-o-				*/

/*	chknaxis: check NAXIS card image and get value		*/
int chknaxis (char card[])
{
extern char *template[];
char value[4], dig;
extern int naxis[];
int k,cardcol;
int blnkstat;
int naxstat=GOOD, invalue=FALSE;
int blnka=KEYLEN+2, blnkz=26;

if((blnkstat=chekblnk(template[2], card, blnka, blnkz)) > GOOD)
  naxstat = blnkstat;
for (k=27; k<30; k++)
  {
  dig = value[k-27] = card[k];
  if(invalue==FALSE)  	 			/* not in number */
    {
    if(isdigit(dig))
      invalue=TRUE; 
    else if(dig != ' ')
      {
      value[k-27] = ' ';     
      cardcol = k+1; 	
      printf("\n   ***   Unexpected %c in column",dig); 
      printf(" %d of NAXIS card image   ***\n\n", cardcol); 
      printf("   ***   Value will be assumed to start after current"); 
      printf(" column,but it may be incorrect.   ***\n\n");
      if(naxstat < FLAWED) 
	naxstat = FLAWED;
      }
    }
  else						/* in number	*/
    {
    if(!isdigit(dig))
      {
      cardcol = k+1; 	
      printf("\n   ***   Unexpected %c in column",dig); 
      printf(" %d of NAXIS card image.   ***\n", cardcol); 
      printf("           Value of NAXIS cannot be determined.\n");
      printf("      ***   Will be unable to read data array.   ***\n\n");
      if(naxstat < BADHEDR) 
        naxstat = BADHEDR;
      }
    }
  }
if(naxstat < BADHEDR)
  {
  value[3] = '\0';
  naxis[0] = atoi(value);
  }
return naxstat;
}

/*				-o-					*/

/*	chkaxisn: Check value for NAXISn card images			*/
int chkaxisn(char card[], int n)
{
char vnaxisn[21],valdig;
extern int naxis[]; 
int status = GOOD;
int invalue = FALSE;
int iref,nchar,cardcol; 
for (nchar=KEYLEN+2; nchar<30; nchar++)
  {
  valdig = vnaxisn[nchar-10] = card[nchar];
  if(invalue==FALSE)  	 			/* not in number */
    {
    if(isdigit(valdig))
      invalue=TRUE; 
    else if(valdig != ' ')
      {
      cardcol = nchar+1; 	
      printf("\n   ***   Unexpected %c in column",valdig); 
      printf(" %d of NAXIS%d card image   ***\n\n", cardcol,n); 
	if(status < FLAWED) 
	  status=FLAWED;
      }
    }
  else						/* in number	*/
    {
    if(!isdigit(valdig))
      {
      cardcol = nchar+1; 	
      printf("\n   ***   Unexpected %c in column",valdig); 
      printf(" %d of NAXIS%d card image.\n", cardcol,n); 
      printf("         Value will be assumed to start after current"); 
      printf(" column,but it may be incorrect.   ***\n\n");
      if(status < INDOUBT) 
	status = INDOUBT;
      for (iref=10;iref<=nchar;iref++)
	vnaxisn[iref-10] = ' ';
      }
    }	
  }
if(invalue == TRUE)
  {
  vnaxisn[20] = '\0'; 
  naxis[n] = atoi(vnaxisn);
  }
else
  status = BADHEDR;
return status;
}

/*			-o-					*/

/* chekblnk:check header where ASCII blanks are	expected	*/
int chekblnk(char *cardname, char card[],int initblnk, int lastblnk)
{
int kblank,kerr,blnkstat=GOOD;
char blankexp;
for (kblank=initblnk; kblank<=lastblnk; kblank++)
  {
  if((blankexp = card[kblank]) != ' ')
    {
    kerr = kblank+1;
    printf("\nImproper content '%c' found in position",blankexp); 
    printf(" %d of %s card.\n",kerr,cardname);
    printf("Should be blank.\n");
    if(blnkstat <FAIR)
      blnkstat=FAIR;
    }	
  }
return blnkstat;
}

/*			-o-			*/

/* summkey: check for presence of all required keywords		*/ 
int summkey (int keylis[])
{
extern int naxis[];
extern char *template[];
int status=GOOD;
int iaa,iab;
char standard[KEYLEN+1];

printf("\n          INVENTORY OF REQUIRED KEYWORDS\n");
printf("      (values were checked when keyword read)\n\n");
if(keylis[LOC_SIMPLE] == FALSE)	
  {	
  printf("SIMPLE keyword absent.\n      Is this a FITS file?\n");
  status = NOTFITS;
  }
if(keylis[LOC_BITPIX] == FALSE)	
  {	
  printf("BITPIX keyword absent.\n      Data cannot be read.\n");
  if(status < BADHEDR)
    status = BADHEDR;
  }
if(keylis[LOC_NAXIS] == FALSE)	
  {	
  printf("NAXIS keyword absent.\n");
  printf("    Data cannot be read.\n");
  printf("    Number of axes cannot be verified.\n");
  if(status < BADHEDR)
    status = BADHEDR;
  }
else			/* check for proper number of axes */
  { 
  for(iaa=1;iaa<=naxis[0];iaa++)
    {
    if(keylis[iaa+LOC_NAXIS] == FALSE)   /*	missing NAXISn	*/ 
      {
      strncpy(standard,template[iaa+2],8);
      standard[KEYLEN] = '\0';
      printf("%s keyword is absent.\n",standard);
      printf("    Data cannot be read.\n");
      if(status < BADHEDR)
	status = BADHEDR;
      }
    }
    for(iab=naxis[0]+1;iab<=MAXAXES;iab++)
    {
    if(keylis[iab+LOC_NAXIS] == TRUE)   /*	extra NAXISn	*/
      {
      strncpy(standard,template[iab+2],KEYLEN);
      standard[KEYLEN] = '\0';
      printf("%s keyword is present",standard);
      printf(" but there are only %d axes.\n",naxis[0]); 
      if (status < TOOFLEX) 
	status = TOOFLEX;
      }
    }
  }
if(status == GOOD)
  printf("Basic required keywords are all present.");
printf("\n\n");
return status;
}

/*				-o-				*/

/* getspec: specifications of formatted output array 		*/
int getspec(char *argv[], int spec[])
{
int status,inc,ip,lx,indlen,inclast;
int maxax,axlim[5];
status = GOOD;
spec[0] = atoi(argv[4]);  /* 1 is MSB first, 0 is MSB last */
spec[1] = atoi(argv[5]); /* ones (1) or twos (2) complement */ 
if(spec[0] == 1)
  printf("Most significant byte is first.\n");
else if(spec[0] == 0)
  printf("Most significant byte is last.\n");
else 
  {	
  printf("PROBLEM WITH SPECIFICATIONS: ");
  printf("spec[0] (endian indicator) is %d\n",spec[0]);
  status = BADSPEC;    
  }
if(spec[1] == 2)
  printf("Twos complement.\n");
else 
  {
  if(spec[1] == 1)
    printf("Software cannot yet handle ones-complement.\n");
  else    
    printf("spec[1] (n's complement) has unexpected value %d\n",spec[1]);
  printf("PROBLEM WITH SPECIFICATIONS\n");
  status = BADSPEC;    
  }
if(atoi(argv[6]) != 0)
  {
  maxax = (naxis[0]>10) ? 10:naxis[0]; 
  inclast = 9 + 3*maxax;
  for (inc=10;inc<=inclast;inc++)       /* read output array specifications */
    {                               /* each axis in turn */
    spec[inc] = atoi(argv[inc-4]);      /* order: first, last, interval */
    }
  printf("\nUser-defined specifications of output array:\n");
  printf("axis    from      to  every\n");
  for (ip=1;ip<=maxax;ip++)
    {
    lx=3*ip + 7;
    printf("%4d%8d%8d%7d\n",ip,spec[lx],spec[lx+1],spec[lx+2]);	
    axlim[ip-1] = spec[lx+1];
    }
  printf("\n");
  for (indlen=1;indlen<=maxax;indlen++)	/* Check specifications */
    {
    if(axlim[indlen-1] > naxis[indlen])
      {
      printf("Axis %d has length %d\n",indlen,naxis[indlen]); 
      printf("Listing of cell %d was requested.\n\n",axlim[indlen-1]);
      status = BADSPEC;
      }
    }
  }
else
  { 
  spec[10] = 0;
  printf("\n User specified that no sample data array be produced.\n\n");
  }
return status;
}

/*				--O--				*/

/*	getdata: extract data matrix from data records	*/
int getdata(FILE *testfile, char *array, long nsize) 
{
char record[RECLEN+1];
extern int bitpix;
int status=GOOD, indata=TRUE;
int noinrec, kbyterec,nbytes; 
int stfill;
char *locfill;
char *recloc;
long nobyte=0;
nbytes = abs(bitpix)/8;
while (indata==TRUE)
  {
  if((noinrec=getrec(testfile,record))==RECLEN)
    {
    recloc = &record[0];
    kbyterec=0;
    while (kbyterec<RECLEN)
      {
      if(indata == TRUE)		/*	data values for matrix	*/
	{	
	*(array+nobyte) = *(recloc+kbyterec);
	if(++nobyte == nsize)
	  {
	  indata=FALSE;
	  printf("\nEnd of data; start fill.\n"); 
	  }
	  kbyterec++;
	}
      else			/*	fill values	*/
	{
 	locfill = (char *) (record+kbyterec);
	if((stfill=chkdfill(locfill,bitpix)) > status)
	  status=stfill;
	if (stfill == FAIR)
	  {
	  printf("Improper fill value in byte %d",kbyterec); 
	  printf(" of last record.\n"); 
	  }
	kbyterec += nbytes;
	}
      }
    }
  else
    {
    if(noinrec == NOREC)
      printf("\n   ***   Premature end of file.   ***\n\n");
    else  
      printf("\nData record of %d bytes.\n\n",noinrec);  	
    status=BADREC;
    indata = FALSE;
    }
  }    
return status;
}

/*				-o-				*/

/*	chkdfill: check fill from data array                    */

int chkdfill(char *location, int bitpix)
{
Dbl64 dubfill;
Flt32 flotfill;
Int4 i4fill;
Int2 i2fill;
char bytefill;
Dbl64 *posdubf;
Flt32 *posflotf;
Int4 *posi4f;
Int2 *posi2f;
int status = GOOD;

if(bitpix == BYT)
  {
  bytefill = *location;
    if(bytefill != 0)
      status = FAIR;
  } 	
else if(bitpix == I2)
  {
  posi2f = (Int2 *) location;
  i2fill = *posi2f;
  if(i2fill != 0)
    status = FAIR;
  } 	
else if(bitpix == I4)
  {
  posi4f = (Int4 *) location;
  i4fill = *posi4f;
  if(i4fill != 0)
    status = FAIR;
  } 	
else if(bitpix == FLT)
  {
  posflotf = (Flt32 *) location;
  flotfill = *posflotf;
  if(flotfill != 0.)
    status = FAIR;
  } 	
else if(bitpix == DBL)
  {
  posdubf = (Dbl64 *) location;
    dubfill = *posdubf;
    if(dubfill != 0.)
      status = FAIR;
  } 	      
else
  {
  status = FPCTERR;	
  printf("\n*******************************************************\n");
  printf("* ERROR IN THIS SOFTWARE; NOFITY FITS SUPPORT OFFICE. *\n");
  printf("*          BITPIX value has been mangled.             *\n");
  printf("*                Value was %3d.          ",bitpix);
  printf("              *\n");
  printf("*******************************************************\n\n");
  printf("\n");
  }
return status;
}

/*				-o-				*/

/* creatout: write formatted array to separate file */
int creatout(char *array,char *outname,long msize,int inspecs[])
{
FILE *listout;
extern int bitpix, naxis[];
int normord,nbyte;
int increate,status=GOOD;
int ks,kt,incx,kchk,limax;
long naccum,increm,laspos,narray;
Int4 *posi4;
Int2 *posi2;
Flt32 *posf4;
Dbl64 *posf8;
int axpos[10],axst[10],axend[10],axinc[10];
long axsiz[11]; 
unsigned char item;
Int2 memi2, holdi2;
Int4 memi4, holdi4; 
Flt32 elemsgl;
Dbl64 elemdbl;

if((listout = fopen(outname,"w")) != NULL) 
  {
  normord = inspecs[0];
  nbyte = abs(bitpix)/8;
  naccum = 0;
  limax = (naxis[0]>10) ? 10:naxis[0];
  increate=TRUE;
  axsiz[0] = 1;
  for (ks=1; ks<= limax; ks++)	     /* initial locations and */
    {				    /*  members per axis index*/  
    axpos[ks-1] = axst[ks-1] = inspecs[3*ks+7];
    axend[ks-1] = inspecs[3*ks+8]; 
    axinc[ks-1] = inspecs[3*ks+9];	
    axsiz[ks] = naxis[ks]*axsiz[ks-1];
    }
  laspos = 1;
  for (kchk=0;kchk<limax;kchk++)	/* Check specifications */
    laspos+= (axend[kchk]-1)*axsiz[kchk];
  narray = msize/nbyte;
  if(laspos > narray)
    {
    increate = FALSE;
    status = BADSPEC;
    printf("\nNo data array will be written.\n");
    printf("Last element requested is %d; size is only %d\n\n",laspos,narray);
    }
  while (increate==TRUE)
    {
    increm = axpos[0]-1;
    for (kt=1;kt<limax;kt++)		/* find 1D sequence */
      increm+= (axpos[kt]-1)*axsiz[kt];		
    increm*= nbyte;	
    naccum++;
    if (bitpix==BYT)		/*	choose format	*/ 
      {
      item = *(array+increm);
      fprintf(listout,"%d ",item);
      }
    else if (bitpix==I2)
      {
      posi2 = (Int2 *) (array+increm);
      memi2 = *posi2;
      if(normord == FALSE)
        {
        holdi2 = memi2;
        memi2 = flip2(holdi2);
        }
      fprintf(listout,"%d ",memi2);
      }
    else if (bitpix==I4)
      {
      posi4 = (Int4 *) (array+increm);
      memi4 = *posi4;
      if(normord == FALSE)
	{
	holdi4 = memi4;
	memi4 = flip4(holdi4);
	}
      fprintf(listout,"%d ",memi4);
      }
    else if (bitpix==FLT)
      {
      posf4 = (Flt32 *) (array+increm);
      elemsgl = *posi4;
      fprintf(listout,"%g ",elemsgl);
      }
    else if (bitpix==DBL)
      {
      posf8 = (Dbl64 *) (array+increm);
      elemdbl = *posf8;
      fprintf(listout,"%g ",elemdbl);
      }
    else
      {
      printf("\n*************************************************\n");
      printf("*       PROGRAM ERROR: NOTIFY FITS OFFICE.       \n");
      printf("* BITPIX mangled on way to ARRAY WRITE routine. *\n"); 
      printf("*               Value was %3d.          ",bitpix);
      printf("      *\n");
      printf("*       Unable to produce formatted array.      *\n");
      printf("*************************************************\n\n");
      status = FPCTERR;
      increm = msize;
      }
    axpos[0]+= axinc[0];	/* find next output array member */
    incx = 0;
    while (incx<limax && axpos[incx]>axend[incx]) 
      {
      if(incx < limax-1)
	{ 
	axpos[incx] = axst[incx];
	incx++;
	axpos[incx]+= axinc[incx];
	}
      else
        {
        increate = FALSE;    
        incx++;
        }
      }
    }
  fclose(listout);
  }
else
  {
  printf("\nUnable to open output file; none written.\n");
  status = NO_OUT;
  }
return status;
}

/* flip2: flip bytes of I*2 array */
short flip2(short inval)

{
short value, vaxval, inhold;

union splitval
	{
	short ival;
	char valchar[2];
	};

union splitval dataitem;
value = inval;
dataitem.ival = value;				/*   start flip   */
inhold = dataitem.valchar[0];
dataitem.valchar[0] = dataitem.valchar[1];
dataitem.valchar[1] = inhold;
vaxval = dataitem.ival;
return vaxval;	
}

/* flip4: convert I*4 from standard to VAX format */
long flip4(long inval)
{

long value, vaxval, inhold;

union breakval
	{
	long ival;
	char valchar[4];
	};

union breakval dataitem;

value = inval;
dataitem.ival = value;		/*   start flip   */
inhold = dataitem.valchar[0];
dataitem.valchar[0] = dataitem.valchar[3];
dataitem.valchar[3] = inhold;
inhold = dataitem.valchar[1];
dataitem.valchar[1] = dataitem.valchar[2];
dataitem.valchar[2] = inhold;
vaxval = dataitem.ival;
return vaxval;	
}

/*                   	--o--				*/

/*	summary:	Summarize evaluation	*/
int summary(struct arr4 errcode)
{
int status = 0;
printf("\n\n			SUMMARY\n");
printf("\n                        Header\n");
printf("                        ------\n");
switch(errcode.head)
  {
  case NOTFITS:
    printf("First 30 bytes do not contain FITS signature SIMPLE=T.\n");
    printf("Check if this is really a FITS file.\n");
    status = status + NOTFITS;
    break;
  case BADREC:
    printf("Problems with header record structure; unable to read.\n");
    status = status + BADREC;
    break;
  case BADHEDR:
    printf("Cannot read data because of header errors.\n");
    status = status + BADHEDR;
    break;
  case NONSTAND:
    printf("File with SIMPLE=F. \n");
    printf("	Diagnostic messages may not be meaningful\n"); 
    printf("because standard FITS rules may not apply.\n");
    status = status + NONSTAND;
    break;
  case INDOUBT:
    printf("     Serious errors in keyword syntax will probably \n"); 
    printf("produce incorrect results when data are read.\n");
    status = status + INDOUBT;
    break;
  case FLAWED:
    printf("     Departures from standard keyword syntax may \n");
    printf("lead to incorrect results if data are read.\n");
    status = status + FLAWED;
    break;
  case FAIR:    
    printf("Minor departures from standard keyword format.\n");
    printf("File can probably still be read.\n");
    status = status + FAIR;
    break;    
  case TOOFLEX:
    printf("   Required keywords conform to FITS standard but \n");
    printf("there are departures from recommended practice.\n");
    status = status + TOOFLEX;
    break;
  case UNDEFINED:
    printf("Header evaluation not completed.\n");
    status = status + UNDEFINED;
    break;
  case GOOD:
    printf("Required keywords conform to FITS standard ");
    printf("and recommended practices.\n");
    break;
  default:
    printf("Improper header status value: %d\n",errcode.head);
    printf( " * Software error - notify FITS office of details. *\n");
    status = status + 9;
    break;
  }
printf("\n                      Input file\n");
printf("                      ----------\n");
switch(errcode.infile)
  {
  case NOFILE:
    printf("No input file; unable to continue run.\n");
    status = status + 10*NOFILE;
    break;
  case BADSPEC:    
    printf("Improper specifications in input file.\n");
    printf("No data array could be written.\n");
    status = status + 10*BADSPEC;
    break;    
  case GOOD:
    printf("No problems in reading input file.\n");
    break;
  default:
    printf("Improper input file status: %d\n",errcode.infile);
    printf( " * Software error - notify FITS office of details. *\n");
    status = status + 90;
    break;
  }

printf("\n                   Space allocation\n");
printf("                   ----------------\n");
switch(errcode.space)
  {
  case KEYFULL:    
    printf("Unable to allocate space for keyword template.\n");
    printf("File evaluation could not be run.\n");
    status = status + 100*KEYFULL;
    break;    
  case DATAFULL: 
    printf("Unable to allocate space to hold data.\n");
    printf("Data could not be read.\n");
    status = status + 100*DATAFULL;
    break;
  case GOOD:
    printf("No problems allocating space.\n");
    break;
  default:
    printf("Improper space allocation status: %d\n",errcode.space);
    printf( " * Software error - notify FITS office of details. *\n");
    status = status + 900;
    break;
  }
printf("\n                          Data\n");
printf("                          ----\n");
switch(errcode.datafile)
  {
  case FPCTERR:
    printf( " * Error in data evaluation section of FPCT * \n");
    printf(" *     Notify FITS office of details. *\n");
    status = status + 9000;
    break;
  case BADREC:
    printf("Problems with data record structure; unable to read.\n");
    status = status + 1000*BADREC;
    break;
  case NO_OUT:
    printf("Unable to allocate space for output data.\n");
    printf("None will be written.\n");
    status = status + 1000*NOTFITS;
    break;
  case FAIR:    
    printf("Non zero content found in data fill.\n");
    printf("It could be an accidental character or...\n ");
    printf("   the header specifications and ");
    printf("the data array might be inconsistent.\n");
    status = status + 1000*FAIR;
    break;    
  case UNDEFINED:
    printf("Data evaluation not completed; probably not performed.\n");
    status = status + 1000*UNDEFINED;
    break;
  case GOOD:
    printf("No problems encountered in reading data and ");
    printf("writing output file.\n");
    break;
  default:
    printf("Improper data status value: %d\n",errcode.datafile);
    printf( " * Software error - notify FITS office of details. *\n");
    status = status + 9000;
    break;
  }
return status;
}
