;+
; NAME: fw.pro
;
; PURPOSE: Fits Widget Viewer
;
; CATEGORY: ACOS Analysis
;
; CALLING SEQUENCE:
;	fw1
;
; INPUTS: none
;
; KEYWORD PARAMETERS: none
;
; PROCEDURE:
;	Display FITS images using "exptv"
;	Adjust min/max/gamma to obtain suitable contrast & brightness.
;	Apply min/max/gamma to byte array version of image.
;	Save display version of image.
;	Use later to make composites with other images.
;
; MODIFICATION HISTORY:
;	15 April 1996: created by Andrew L. Stanger, HAO/NCAR
;	17 April 1996: replace min/max/gamma inputs with sliders
;			which modify the color map only.  Apply button may
;			then be used to alter the byte array image itself.
;-

;-------------------------------------------------------------------------------
; Display Image.
;-------------------------------------------------------------------------------
PRO fw_display, image, MAX = cur_max, MIN = cur_min, relative = $
                   relative, _extra=_extra, csi=csi
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;  The call to EXPTV
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;   noexact = (relative NE 1.0)
   exptv, image, MAX = cur_max, MIN = cur_min, relative = relative, $
      noexact = noexact, xalign = 0.65, yalign = 0.8, _extra = _extra
;   get_tv_scale, sx, sy, mx, my, jx, jy
;   rx = FLOAT(sx)/FLOAT(mx)
;   ry = FLOAT(sy)/FLOAT(my)
;   temp = {xd0:jx, yd0:jy, mx:mx, my:my}
;   copy_struct, temp, csi
;   csi = rep_tag_value(csi, rx, 'RX')
;   csi = rep_tag_value(csi, ry, 'RY')
END

;-------------------------------------------------------------------------------
; Annotate image.
;-------------------------------------------------------------------------------
PRO fw_house, hdu

@fw_common

   img_date_obs = ''
   img_doy      = ''
   img_time_obs = ''
   img_origin   = ''
   img_telescop = ''
   img_object   = ''
   img_type_obs = ''
   img_type_dp  = ''
   img_data_min =   0.0
   img_data_max = 255.0
   img_disp_min =   0.0
   img_disp_max = 255.0

   fx_date_obs = fxpar (hdu, 'DATE-OBS')
   IF (!err NE -1) THEN BEGIN					$

      smonth = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 	$
                'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
      iday   = fix (strmid (fx_date_obs, 0, 2))
      imonth = fix (strmid (fx_date_obs, 3, 2)) - 1
      iyear  = fix (strmid (fx_date_obs, 6, 2))
      nday = intarr (12)
      nday ( [0, 2, 4, 6, 7, 9, 11] ) = 31
      nday ( [3, 5, 8, 10] ) = 30
      ly = float (iyear) / 4.0
      if ((ly - fix (ly)) eq 0.0) then nday (1) = 29 else nday (1) = 28
      idoy = 0

      if (imonth ne 0) then					$
         for i = 0, imonth - 1 do begin
            idoy = idoy + nday (i)
         endfor
      idoy = idoy + iday
      img_doy = string (idoy)

      img_date_obs = string (iday, FORMAT = '(I2)') + '-' +	$
	             smonth (imonth) +	'-' +			$
	             string (iyear, FORMAT = '(i2)')
   ENDIF

   fx_time_obs = fxpar (hdu, 'TIME-OBS')
   IF (!err NE -1) THEN img_time_obs = fx_time_obs
   fx_origin   = fxpar (hdu, 'ORIGIN')
   IF (!err NE -1) THEN img_origin = fx_origin
   fx_telescop = fxpar (hdu, 'TELESCOP')
   IF (!err NE -1) THEN img_telescop = fx_telescop
   fx_object   = fxpar (hdu, 'OBJECT')
   IF (!err NE -1) THEN img_object = fx_object
   fx_type_obs = fxpar (hdu, 'TYPE-OBS')
   IF (!err NE -1) THEN img_type_obs = fx_type_obs
   fx_type_dp  = fxpar (hdu, 'TYPE-DP')
   IF (!err NE -1) THEN img_type_dp = fx_type_dp

   fx_data_min = fxpar (hdu, 'DATAMIN')
   IF (!err NE -1) THEN						$
   BEGIN
      img_data_min = fx_data_min
   ENDIF							$
   ELSE								$
      img_data_min = MIN (img_dbuf)
   fx_data_max = fxpar (hdu, 'DATAMAX')
   IF (!err NE -1) THEN						$
   BEGIN
      img_data_max = fx_data_max
   ENDIF							$
   ELSE								$
      img_data_max = MAX (img_dbuf)

   fx_disp_min = fxpar (hdu, 'DISPMIN')
   IF (!err NE -1) THEN						$
   BEGIN
      img_disp_min = fx_disp_min
   ENDIF							$
   ELSE								$
      img_disp_min = MIN (img_dbuf)
   fx_disp_max = fxpar (hdu, 'DISPMAX')
   IF (!err NE -1) THEN						$
   BEGIN
      img_disp_max = fx_disp_max
   ENDIF							$
   ELSE								$
      img_disp_max = MAX (img_dbuf)

   data_min = img_data_min
   data_max = img_data_max

   disp_min   = img_disp_min
   disp_max   = img_disp_max
   disp_gamma = 1.0

   tv_min   = disp_min
   tv_max   = disp_max
   tv_gamma = disp_gamma

   sav_min   = disp_min
   sav_max   = disp_max
   sav_gamma = disp_gamma

;   print, 'fw_house>data_min/data_max: ', data_min, data_max
;   print, 'fw_house>disp_min/disp_max: ', disp_min, disp_max

   WIDGET_CONTROL, tw_date_obs, set_value = img_date_obs
   WIDGET_CONTROL, tw_doy,      set_value = img_doy
   WIDGET_CONTROL, tw_time_obs, set_value = img_time_obs
   WIDGET_CONTROL, tw_origin,   set_value = img_origin
   WIDGET_CONTROL, tw_telescop, set_value = img_telescop
   WIDGET_CONTROL, tw_object,   set_value = img_object
   WIDGET_CONTROL, tw_type_obs, set_value = img_type_obs
   WIDGET_CONTROL, tw_type_dp,  set_value = img_type_dp

END


;-------------------------------------------------------------------------------
PRO FW_Event, Event
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; Event processing for fw procedure.
;
@fw_common

  WIDGET_CONTROL, Event.Id, GET_UVALUE=Ev

;  IF (Event.Id EQ LCT_LIST) THEN BEGIN
;      LOADCT, SILENT=silent, event.index, FILE=lctfile, NCOLORS=ncolors, $
;              BOTTOM=cbot
;      goto, set_gamma
;  ENDIF

  IF (Event.Id EQ LUT_LIST) THEN BEGIN
;      print, 'LUT_LIST event', '   event.index: ', event.index, $
;	     '   ', lutnames (event.index)
;      print, 'lutpaths: '
;      print, lutpaths
;      print, 'lutpaths (event.index): ', lutpaths (event.index)

      fw_lct, lutpaths (event.index)
;      tvlct, r_orig, g_orig, b_orig, /GET

;      print, 'fw_lct>size (red): ', size (red)
;      print, 'fw_lct> red: '
;      print, red
      goto, set_gamma
  ENDIF

  CASE Ev OF 

  'BUTTON1': BEGIN
;      Print, 'Event for Quit'
      WIDGET_CONTROL, event.top, /destroy               ; Kill all widgets.
      END

  'BUTTON2': BEGIN					; Load image.
;      Print, 'Event for Load Image'
      img_file = pickfile (/READ, FILTER = '*.fts *.fits')

      img_ibuf = readfits (img_file, img_hdu)	; Read image into input buffer.
      img_dbuf = img_ibuf			; Display buffer.

      fw_house, img_hdu

      fw_display, img_dbuf, MIN=disp_min, MAX=disp_max

;      print, 'Load> min (img_dbuf): ', min (img_dbuf),	$
;	     '   max (img_dbuf): ', max (img_dbuf)
      END

  'BUTTON3': BEGIN					; Restore image.
;     Print, 'Event for Restore Image'
      img_ibuf = readfits (img_file, img_hdu)	; Read image into input buffer.
      img_dbuf = img_ibuf			; Display buffer.
      fw_house, img_hdu
      fw_display, img_dbuf, MIN=disp_min, MAX=disp_max

;      print, 'restore> min (img_dbuf): ', min (img_dbuf),	$
;	     '   max (img_dbuf): ', max (img_dbuf)
      END

  'BUTTON4': BEGIN					; Save Image: FITS.
      Print, 'Event for Save Image: FITS'

      ;--- Create output file name.

      extloc = strpos (img_file, '.fts')
      if (extloc EQ -1) then extloc = strpos (img_file, '.fits')
      if (extloc NE -1) then				$
         out_file = strmid (img_file, 0, extloc)	$
      else						$
         out_file = img_file
      out_file = out_file + '.fw.fts'

      ;--- Modify FITS header.

      out_hdu = img_hdu

      sdbuf = size (img_dbuf)
      dtype = sdbuf (3)					; data type

      CASE dtype OF
      0: BEGIN
	    nbitpix = 0
	 END
      1: BEGIN
	    nbitpix = 8
	 END
      2: BEGIN
	    nbitpix = 16 
	 END
      3: BEGIN
	    nbitpix = 32 
	 END
      4: BEGIN
	    nbitpix = -32 
	 END
      5: BEGIN
	    nbitpix = -32 
	 END
      6: BEGIN
	    nbitpix = -32 
	 END
      7: BEGIN
	    nbitpix = -8
	 END
      8: BEGIN
	    nbitpix = -1 
	 END
      ENDCASE
;      print, 'dtype: ', dtype, '   nbitpix: ', nbitpix

      fxaddpar, out_hdu, 'BITPIX', nbitpix
      fxaddpar, out_hdu, 'DATAMIN', disp_min
      fxaddpar, out_hdu, 'DATAMAX', disp_max
      fxaddpar, out_hdu, 'DISPMIN', disp_min
      fxaddpar, out_hdu, 'DISPMAX', disp_max
      fxaddpar, out_hdu, 'COMMENT', $
			 '   -->IMAGE HAS BEEN MODIFIED BY IDL PROCEDURE FW.PRO'
      fxaddpar, out_hdu, 'COMMENT', $
			 '   -->DATAMIN CORRESPONDS TO ' + string (sav_min, FORMAT = '(E8.2)')
      fxaddpar, out_hdu, 'COMMENT', $
			 '   -->DATAMAX CORRESPONDS TO ' + string (sav_max, FORMAT = '(E8.2)')
      fxaddpar, out_hdu, 'COMMENT', $
			 '   -->GAMMA CORRECTION (' + string (sav_gamma, FORMAT = '(F6.3)') + ') HAS BEEN APPLIED.'

;      print, 'save image>out_file: ', out_file
;      print, 'save image>data_min/max: ', data_min, data_max
;      print, 'save image>sav_min/max: ', sav_min, sav_max
;      print, 'save image>sav_gamma: ', sav_gamma

      ;--- Write file to disk.

      print, 'Writing FITS file ', out_file, ' to disk.'
;      print, 'size (img_dbuf): ', size (img_dbuf)
;      print, 'min (img_dbuf): ', min (img_dbuf),	$
;	     '   max (img_dbuf): ', max (img_dbuf)
      writefits, out_file, img_dbuf, out_hdu
      END

  'BUTTON5': BEGIN					; Save Sun Rasterfile.
      Print, 'Event for Save Image: Sun Raster'

      ;--- Create output file name.

      extloc = strpos (img_file, '.fts')
      if (extloc EQ -1) then extloc = strpos (img_file, '.fits')
      if (extloc NE -1) then				$
         out_file = strmid (img_file, 0, extloc)	$
      else						$
         out_file = img_file
      out_file = out_file + '.fw.sun'

      sdbuf = size (img_dbuf)
      dtype = sdbuf (3)					; data type

      IF (dtype EQ 0) THEN	$
	 BEGIN
	    print, 'Invalid image type: ', dtype
	 END			$
      ELSE			$
	 BEGIN
            img_dbuf = bytscl (img_dbuf, MIN=tv_min, MAX=tv_max, TOP=tv_top)
	 END

;      tvlct, red, green, blue, /get			; Read color LUT.

      print, 'Writing Sun Rasterfile: ', out_file
;      print, 'size (img_dbuf): ', size (img_dbuf)
;      print, 'min (img_dbuf): ', min (img_dbuf),	$
;	     '   max (img_dbuf): ', max (img_dbuf)
;      print, 'write_ras>size (red): ', size (red)
;      print, 'write_ras> red: '
;      print, red
      img_dbuf = reverse (img_dbuf, 2)			; Flip top to bottom.
      write_ras, out_file, img_dbuf, red, green, blue

      END

  'BUTTON6': BEGIN					; Lat/Long Grid.
      Print, 'Event for Grid'
      END

  'BUTTON7': BEGIN					; FITS header.
      Print, 'Event for FITS Header'
      END

  'MINIMUM': BEGIN					; Data min value.
;      Print, 'Event for Data Minimum'
      WIDGET_CONTROL, MIN_SLIDER, GET_VALUE = vbot
      frac_min = vbot / 100.0
      tv_min   = frac_min * (disp_max - disp_min) + disp_min
      GOTO, set_gamma
      END

  'MAXIMUM': BEGIN					; Data max value.
;      Print, 'Event for Data Maximum'
      WIDGET_CONTROL, MAX_SLIDER, GET_VALUE = vtop
      frac_max = vtop / 100.0
      tv_max   = frac_max * (disp_max - disp_min) + disp_min

   set_gamma:
	if (use_values) then nc = 256 else nc = ncolors
	s = (nc - 1) / 100.0
	x0 = vbot * s
	x1 = vtop * s
	if (x0 NE x1) then s = (nc - 1.0) / (x1 - x0) else s = 1.0
	int = -s * x0
	if (gammaval EQ 1.0) then s = round (findgen (nc) * s + int > 0.0) $
	else s = ((findgen (nc) * (s / nc) + (int / nc) > 0.0) ^ gammaval) * nc

	if (chop NE 0) then begin
	    too_high = where (s GE nc, n)
	    if (n GT 0) then s (too_high) = 0L
	    endif

	if (use_values) then begin
	    s = s < 255L
	    l = lindgen (ncolors) + cbot
	    r_curr (cbot) = (r = s (r_orig (l)))
	    g_curr (cbot) = (g = s (g_orig (l)))
	    b_curr (cbot) = (b = s (b_orig (l)))
	endif else begin
	    s = s + cbot
	    r_curr (cbot) = (r = r_orig (s))
	    g_curr (cbot) = (g = g_orig (s))
	    b_curr (cbot) = (b = b_orig (s))
 	endelse

	tvlct, r, g, b, cbot

      END

  "GAMMA": BEGIN					; Gamma value.
;      Print, 'Event for Gamma Correction'
      WIDGET_CONTROL, GAMMA_SLIDER, GET_VALUE = gammaval
      gammaval = 10^((gammaval/50.0) - 1)
      tv_gamma = gammaval
      WIDGET_CONTROL, GAMMA_LABEL, $
		      SET_VALUE = STRING (gammaval, format='(f6.3)')
      goto, set_gamma
      END

  'MIN_MAX_GAM': BEGIN					; Apply min/max/gamma.
;      Print, 'Event for Apply min/max/gamma'
;      print, 'Apply data_min/data_max: ', data_min, data_max
;      print, 'Apply disp_min/disp_max: ', disp_min, disp_max
;      print, 'Apply tv_min  /tv_max:   ', tv_min, tv_max

      ;--- Apply linear photometric scaling using data_min & data_max.

      img_dbuf = bytscl (img_dbuf, MIN=tv_min, MAX=tv_max, TOP=tv_top)
;      print, 'size (img_dbuf): ', size (img_dbuf)

      ;--- Apply gamma correction.

      tv_gamma = 1.0 / gammaval
      img_dbuf = gamtran (img_dbuf, tv_gamma)
;      print, 'size (img_dbuf): ', size (img_dbuf)

      ;--- Convert to byte array.

      img_dbuf = byte (img_dbuf)
;      print, 'size (img_dbuf): ', size (img_dbuf)

      dmin = MIN (img_dbuf)
      dmax = MAX (img_dbuf)
;      print, 'Apply dmin/dmax: ', dmin, dmax

      sav_min   = tv_min
      sav_max   = tv_max
      sav_gamma = tv_gamma

      disp_min   =   0
      disp_max   = 255
      disp_gamma = 1.0

      tv_min   = disp_min
      tv_max   = disp_max
      tv_gamma = disp_gamma

      vbot =   0
      vtop = 100
      gammaval = 1.0

;      gammaval = 10^((gammaval/50.0) - 1)
      WIDGET_CONTROL, MIN_SLIDER, SET_VALUE = string (vbot, FORMAT='(I3)')
      WIDGET_CONTROL, MAX_SLIDER, SET_VALUE = string (vtop, FORMAT='(I3)')
      WIDGET_CONTROL, GAMMA_LABEL, $
		      SET_VALUE = STRING (gammaval, format='(f6.3)')

      fw_display, img_dbuf, MIN=disp_min, MAX=disp_max
      goto, set_gamma
      END

  ENDCASE
END



;PRO fw, GROUP=Group

;-------------------------------------------------------------------------------
PRO FW, PICK_ONE=pick_one, SILENT=silent_f, GROUP=group, FILE=file, $
	LUTDIR=lutdir, LUTLIST=lutlist, $
	USE_CURRENT=use_current, NCOLORS=nc, BOTTOM=bottom

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; FITS widget analysis software
;
; PURPOSE
;   Create widgets for FW functions.
;
; USAGE
;   fw
;
; HISTORY
;   Author: Andrew L. Stanger   17 April  1996
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

@fw_common

  xslide = 20
  yslide = 200
  lock = 0
  chop = 0
  vbot = 0
  vtop = 100
  gammaval = 1.0
  use_values = 0

  disp_min   =   0
  disp_max   = 255
  disp_gamma = 1.0

  tv_min   =   0
  tv_max   = 255
  tv_top   = 249
  tv_gamma = 1.0

  sav_min   = disp_min
  sav_max   = disp_max
  sav_gamma = disp_gamma

  lutnames = ''

  IF (n_elements (lutdir) NE 0) THEN		$
     lutdirname = lutdir			$
  ELSE						$
     lutdirname = '/riker/usr/people/stanger/color/'

  IF (n_elements (lutlist) NE 0) THEN		$
     lutlistname = lutlist			$
   ELSE						$
      lutlistname = 'lut.list'
     
  GET_LUN, lun
;  print, 'lun: ', lun
;  lutlistname  = 'lut.list'
  cmname = ''

  lutpathname = lutdirname + lutlistname
  openr, lun, lutpathname
  ntables = 0b

  while (NOT EOF (lun)) DO    $
  BEGIN
     readf, lun, FORMAT = '(A)', cmname
     ntables = ntables + 1
;     print, 'ntables: ', ntables, '   cmname: ', cmname
  END

;  print, 'ntables: ', ntables
  lutpaths = strarr (ntables)
  lutnames = strarr (ntables)

  POINT_LUN, lun, 0					; Reset pointer to BOF.
  FOR i = 0, ntables - 1 DO	$
  BEGIN
     readf,  lun, FORMAT = '(A)', cmname
     lutnames (i) = cmname
  END

  lutpaths = lutdirname + lutnames

;  print, 'lutpaths: '
;  print, lutpaths

  IF (N_ELEMENTS (group) EQ 0) THEN GROUP=0

  junk   = { CW_PDMENU_S, flags:0, name:'' }

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

   BASE0 = WIDGET_BASE (GROUP_LEADER=group, $		; Root base.
      ROW=1, $
      MAP=1, $
      TITLE='FW', $
      UVALUE='BASE0')

  BASE1 = WIDGET_BASE (BASE0, $				; Base for left column.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE1')

  BASE1T = WIDGET_BASE (BASE1, $			; Base for buttons.
      ROW=1, $
      MAP=1, $
      UVALUE='BASE1T')

  BASE1TL = WIDGET_BASE (BASE1T, $			; Quit/Load/Save base.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE1TL')

  BASE1TR = WIDGET_BASE (BASE1T, $			; Base: col 1 top right.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE1TR')

  BUTTON1 = WIDGET_BUTTON (BASE1TL, $			; Quit button.
      UVALUE='BUTTON1', $
      VALUE='Quit')

  BUTTON2 = WIDGET_BUTTON (BASE1TL, $			; Load Image button.
      UVALUE='BUTTON2', $
      VALUE='Load Image')

  BUTTON3 = WIDGET_BUTTON (BASE1TL, $			; Restore Image button.
      UVALUE='BUTTON3', $
      VALUE='Restore Image')

  BUTTON4 = WIDGET_BUTTON (BASE1TR, $			; Save Image button.
      UVALUE='BUTTON4', $
      VALUE='Save Image: FITS')

  BUTTON5 = WIDGET_BUTTON (BASE1TR, $			; Save Image button.
      UVALUE='BUTTON5', $
      VALUE='Save Image: Sun Raster')

  BUTTON6 = WIDGET_BUTTON (BASE1TR, $			; Color Map button.
      UVALUE='BUTTON6', $
      VALUE='Grid')

  WIDGET_CONTROL, BUTTON6, SENSITIVE=0			; Make insensitive.

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  BASE1A = WIDGET_BASE (BASE1, $			; Apply base.
     /COLUMN, $
     /MAP, $
     UVALUE='BASEA')

  LABELB1A = WIDGET_LABEL (BASE1A, $
      UVALUE='LABELB1A', $
      VALUE='Image Intensity')

  APPLY_BUTTON = WIDGET_BUTTON (BASE1A, $
      UVALUE='MIN_MAX_GAM', $
      VALUE='Apply min/max/gamma')

  BASE1M = WIDGET_BASE (BASE1, $			; Modify LUT base.
     /COLUMN, $
     /MAP, $
     UVALUE='BASE1M')

  LABELB1M = WIDGET_LABEL (BASE1M, $
      UVALUE='LABELB1M', $
      VALUE='LUT Intensity')

  BASE1MS = WIDGET_BASE (BASE1M, $			; Modify LUT base.
     /ROW, $
     /MAP, $
     UVALUE='BASE1MS')

  BASE1MS1 = WIDGET_BASE (BASE1MS, $			; Min slider base.
     XSIZE=80, $
     /COLUMN, $
     /MAP, $
     UVALUE='BASE1MS1')

  MIN_SLIDER = WIDGET_SLIDER (BASE1MS1, TITLE = "Min", $
      /VERTICAL, $
     MINIMUM = 0, MAXIMUM = 100, VALUE = 0, $
     /DRAG, UVALUE = "MINIMUM", XSIZE=xslide, YSIZE=yslide)

  BASE1MS2 = WIDGET_BASE (BASE1MS, $			; Max slider base.
     XSIZE=80, $
     /COLUMN, $
     /MAP, $
     UVALUE='BASE1MS2')

  MAX_SLIDER = WIDGET_SLIDER (BASE1MS2, TITLE = "Max", $
      /VERTICAL, $
     MINIMUM = 0, MAXIMUM = 100, VALUE = 100, $
     /DRAG, UVALUE = "MAXIMUM", XSIZE=xslide, YSIZE=yslide)

  BASE1MS3 = WIDGET_BASE (BASE1MS, $			; Gamma slider base.
     XSIZE=80, $
     /COLUMN, $
     /MAP, $
     UVALUE='BASE1MS3')

  GAMMA_SLIDER = WIDGET_SLIDER (BASE1MS3, TITLE = "Gamma", $
      /VERTICAL, $
        MINIMUM = 0, MAXIMUM = 100, VALUE = 50, $
	/DRAG, UVALUE = "GAMMA", /SUPPRESS_VALUE, XSIZE=xslide, YSIZE=yslide)

  GAMMA_LABEL = WIDGET_LABEL (BASE1MS3, VALUE = STRING (1.0))

  BASE1C = WIDGET_BASE (BASE1, $			; Modify Colormap base.
     /COLUMN, $
     /MAP, $
     UVALUE='BASE1C')

  DEVICE, GET_SCREEN = junk
  IF (junk (1) LE 768) THEN junk = 4 ELSE junk = 8 

  LUT_LIST = WIDGET_LIST (BASE1C, VALUE = lutnames, ysize = junk)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  BASE2 = WIDGET_BASE (BASE0, $				; Annotation base.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE2')

  BUTTON7 = WIDGET_BUTTON (BASE2, $
      UVALUE='BUTTON7', $
      VALUE='FITS Header')

  WIDGET_CONTROL, BUTTON7, SENSITIVE=0			; Make insensitive.

  LABEL4 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL4', $
      VALUE='Date')

  TextVal4 = [ 'YY/MM/DD' ]
  TEXT4 = WIDGET_TEXT (BASE2, VALUE=TextVal4, $
      EDITABLE=1, $
      UVALUE='TEXT4', $
      YSIZE=1)
  tw_date_obs = TEXT4

  LABEL5 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL5', $
      VALUE='Time (UT)')

  TextVal5 = [ 'HH:MM:SS' ]
  TEXT5 = WIDGET_TEXT (BASE2, VALUE=TextVal5, $
      EDITABLE=1, $
      UVALUE='TEXT5', $
      YSIZE=1)
  tw_time_obs = TEXT5

  LABEL6 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL6', $
      VALUE='Day Of Year')

  TextVal6 = [ '001-366' ]
  TEXT6 = WIDGET_TEXT (BASE2, VALUE=TextVal6, $
      EDITABLE=1, $
      UVALUE='TEXT6', $
      YSIZE=1)
  tw_doy = TEXT6

  LABEL7 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL7', $
      VALUE='Origin')

  TextVal7 = [ 'Observatory' ]
  TEXT7 = WIDGET_TEXT (BASE2, VALUE=TextVal7, $
      EDITABLE=1, $
      UVALUE='TEXT7', $
      YSIZE=1)
  tw_origin = TEXT7

  LABEL8 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL8', $
      VALUE='Telescope')

  TextVal8 = [ 'Name' ]
  TEXT8 = WIDGET_TEXT (BASE2, VALUE=TextVal8, $
      EDITABLE=1, $
      UVALUE='TEXT8', $
      YSIZE=1)
  tw_telescop = TEXT8

  LABEL9 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL9', $
      VALUE='Object')

  TextVal9 = [ 'Sun' ]
  TEXT9 = WIDGET_TEXT (BASE2, VALUE=TextVal9, $
      EDITABLE=1, $
      UVALUE='TEXT9', $
      YSIZE=1)
  tw_object = TEXT9

  LABEL10 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL10', $
      VALUE='Type-Obs')

  TextVal10 = [ 'Filter Type' ]
  TEXT10 = WIDGET_TEXT (BASE2, VALUE=TextVal10, $
      EDITABLE=1, $
      UVALUE='TEXT10', $
      YSIZE=1)
  tw_type_obs = TEXT10

  LABEL11 = WIDGET_LABEL (BASE2, $
      UVALUE='LABEL11', $
      VALUE='Type-DP')

  TextVal11 = [ 'Raw Image' ]
  TEXT11 = WIDGET_TEXT (BASE2, VALUE=TextVal11, $
      EDITABLE=1, $
      UVALUE='TEXT11', $
      YSIZE=1)
  tw_type_dp =TEXT11

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  BASE3 = WIDGET_BASE (BASE0, $				; Image Display base.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE3')

  BASE3T = WIDGET_BASE (BASE3, $			; Image Window base.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE3T')

  ximg = 520
  yimg = 520
  DRAW3T = WIDGET_DRAW (BASE3T, $			; Image window.
      RETAIN=2, $
      UVALUE='DRAW3T', $
      XSIZE=ximg, $
      YSIZE=yimg)

  BASE3B = WIDGET_BASE (BASE3, $			; Color bar base.
      COLUMN=1, $
      MAP=1, $
      UVALUE='BASE3B')

  xct = 512
  yct = 20
  DRAW3B = WIDGET_DRAW (BASE3B, $			; Color bar window.
      RETAIN=2, $
      UVALUE='DRAW3B', $
      XSIZE=xct, $
      YSIZE=yct)

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  WIDGET_CONTROL, BASE0, /REALIZE
;  WIDGET_CONTROL, state.draw, GET_VALUE=tmp

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  if (n_elements (bottom) gt 0) then cbot = bottom else cbot = 0
  ncolors = !d.table_size - cbot
  if (n_elements (nc) gt 0) then ncolors = ncolors < nc
  if (ncolors le 0) then message,'Number of colors is 0 or negative'

;--- If no common, use current colors

  IF (KEYWORD_SET (use_current) or N_ELEMENTS (r_orig) LE 0) THEN BEGIN
	TVLCT, r_orig, g_orig, b_orig, /GET
	r_curr = r_orig
	b_curr = b_orig
	g_curr = g_orig
  ENDIF

  r0 = r_curr			; Save original colors
  g0 = g_curr
  b0 = b_curr

  color = ncolors + cbot -1
  cps = [0, ncolors-1]
  tfun = cps
  pnt = -1

  index = bindgen (256)
  red   = bindgen (256)
  green = bindgen (256)
  blue  = bindgen (256)

  r = red 
  g = green
  b = blue

  IF (ncolors NE 256) THEN			$
     BEGIN
        p = (LINDGEN (ncolors) * 255) / (ncolors - 1)
        r = red (p)
        g = green (p)
        b = blue (p)
     END

  r_orig (cbot) = r
  g_orig (cbot) = g
  b_orig (cbot) = b

  r_curr = r_orig
  g_curr = g_orig
  b_curr = b_orig

  tvlct, r_orig, g_orig, b_orig, cbot

;  print, 'fw> size (r_orig): ', size (r_orig)
;  print, 'fw> r_orig: '
;  print, r_orig

;  print, '!d.table_size: ', !D.TABLE_SIZE

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

  ;--- Get drawable window index.

  COMMON DRAW3T_Comm, DRAW7_Id
  WIDGET_CONTROL, DRAW3T, GET_VALUE=DRAW3T_Id

   WIDGET_CONTROL, DRAW3B, get_value=tmp1
   WSET, tmp1
      TVSCL, BYTSCL (INDGEN (xct) # REPLICATE (1, yct), top = !d.n_colors-1)

  WSET, DRAW3T_Id

   if (n_elements (file) NE 0) then		$
      img_file = file				$
   else						$
      img_file = '/riker/usr/people/stanger/idl/lib/acos/801050544.c4q.fts'

   BEGIN
      img_ibuf = readfits (img_file, img_hdu)
      img_dbuf = img_ibuf
      fw_house, img_hdu
      fw_display, img_dbuf, MIN=disp_min, MAX=disp_max
;      print, 'min (img_dbuf): ', min (img_dbuf),	$
;	     '   max (img_dbuf): ', max (img_dbuf)
   END

  XMANAGER, 'FW', BASE0
END

