;+
; NAME
;	gamtran
;
; PURPOSE
;	Transform an 8-bit/pixel image by applying gamma function.
;
; CALLING SEQUENCE
;	newimg = gamtran (oldimg, gammaval)
;
; INPUT PARAMETERS
;	oldimg:		Input image array.
;	gammaval:	Gamma function value: <1 darken, >1 lighten.
;
; OUTPUT PARAMETER
;	newimg:		Transformed image array.
;
; PROCEDURES USED
;	make_gammatab
;
; MODIFICATION HISTORY
;	17 August 1995, created by Andrew L. Stanger, HAO/NCAR
;-

function gamtran, bufinp, gammaval

;--- Check size info for input array.

sizinp = size (bufinp)
;print, sizinp
ndim   = sizinp (0)
xdim   = sizinp (1)
ydim   = sizinp (2)
nbypix = sizinp (3)

if (ndim ne 2) then	$
   begin
      print, 'Input image must be 2D array . . . size : ', sizinp
      return, bufinp
   end

if (nbypix ne 1) then	$
   begin
      print, 'Input image must be a byte array . . . size : ', sizinp
      return, bufinp
   end

;--- Did user supply gamma value ?

;print, 'n_elements (gammaval): ', n_elements (gammaval)

if (n_elements (gammaval) eq 0) then	$
   begin
      print, 'No gamma value provided.  Exit function.'
      return, bufinp
   end

print, 'GAMTRAN: apply gamma function with value: ', gammaval

;--- Create output image array.

bufout = bytarr (xdim, ydim)

;--- Create gamma table.

gamtab = make_gamma (gammaval)

;--- Peform gamma look-up table transformation.

bufout = gamtab (bufinp)

return, bufout

end
