/*  Gets the "hairline" positions in a mk4 image or vector.

    This code was written by Alice Lecinski, September 2001.

    It is modeled after David Elmore's mk4 calibraton
    routines written as idl procedures.  David developed these
    procedures in fall of 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>
#include <math.h>

#include "mk4v2.h"

float  XX  [VECTOR];
float  YY  [VECTOR];
float  Yfit[VECTOR];


int getHair(Img,xdim,lo,hi)
    float *Img;
    int xdim;
    float *lo, *hi;
{
    int ii;
    float *pX, *pY, *pYfit, *pI;
    float ftmp;
    float mean ;
    float A[10];

    pY = YY;
    pI = Img;
    pX = XX;
    for (ii=0;ii<NPX;ii++){
	*pX = (float)ii;
	get_mean_fast( pI , xdim , &mean );
	*pY = mean;
	pX++;
	pY++;
	pI += xdim;
    }

    pYfit  = Yfit;

    pX     = XX;
    pY     = YY;
    pX += 85;
    pY += 85;
    gaussfit(pX,pY,pYfit,A,41);
    *lo = A[1];

    pX     = XX;
    pY     = YY;
    pX += 255;
    pY += 255;
    gaussfit(pX,pY,pYfit,A,41);
    *hi = A[1];

    fprintf(stderr,"\t\tHairs found at pixels %.2f %.2f\n",*lo,*hi);

}

int getHairVec(myV,lo,hi)
    float *myV, *lo, *hi;
{
    int ii;
    float *pX, *pY, *pYfit, *pV;
    float ftmp;
    float A[10];

    pY = YY;
    pV = myV;
    pX = XX;
    for (ii=0;ii<NPX;ii++){
	*pX = (float)ii;
	*pY = *pV;
	pX++;
	pY++;
	pV++;
    }

    pYfit  = Yfit;

    pX     = XX;
    pY     = YY;
    pX += 85;
    pY += 85;
    gaussfit(pX,pY,pYfit,A,41);
    *lo = A[1];

    pX     = XX;
    pY     = YY;
    pX += 255;
    pY += 255;
    gaussfit(pX,pY,pYfit,A,41);
    *hi = A[1];

    fprintf(stderr,"\t\tHairs found at pixels %.2f %.2f\n",*lo,*hi);

}
