/* cpwlook.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a sequence of SMM C/P image on the SGI IRIS screen
 * using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * 12 March 1992: adapted from "cpw.c".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/img_scale.h"
int DSP = 1;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
static char nampgm [80];	/* Program file name. */
static char namimg [80];	/* Image file name. */
static char namlst [80];	/* Image file list. */
static char answer [2];		/* Response to scaling question. */
char *parg;			/* Argument pointer. */

int glcp_view();		/* Image display function. */
int i;				/* Index. */
int ians;			/* KB response. */
int ierr;			/* Error status from cpview(). */
int imgcnt = 0;			/* Image counter. */
int larg;			/* Length of scaling/disk flag argument. */
int linit = 1;			/* Image label flag [0=off,1=on]. */
int pmode = 0;			/* Print mode. */

FILE *LIST;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [file_list_name]\n", argv [0]);

      fprintf (stderr, "Enter name of file containing C/P image names: ");
      fscanf  (stdin,  "%s", namlst);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      lscale = 1;

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      lscale = 0;
      strcpy (namlst, argv [1]);
      }

   /*** Two arguments. ***/

   if (argc == 3)
      {
      if ((strncmp (argv [1], "-", 1) == 0) || (strncmp (argv [1], "s", 1) == 0)
       || (strncmp (argv [1], "d", 1) == 0))
         {
         parg = argv [1];
         larg = strlen (parg);
         if (*parg == '-')
	    {
            parg++;
            larg--;
            }
         if (*parg == 's')  lscale = 1;
         if (*parg == 'd')  ldisk = 1;
         if (larg == 2)
            {
            parg++;
            if (*parg == 's')  lscale = 1;
            if (*parg == 'd')  ldisk = 1;
            }
         }
      else
         {
         ldisk = 0;
         lscale = 0;
         }
      strcpy (namlst, argv [2]);
      }

   /*** Three arguments. ***/

   if (argc == 4)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [file_list_name]\n", argv [0]);
      exit (-1);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax, clipmin, clipmax: ");
      fscanf  (stdin,  "%f,%f,%f,%f,%f,%f",
               &foldmin, &foldmax, &fnewmin, &fnewmax, &fclipmin, &fclipmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
/*
 *  for (i=0; i<argc; i++)
 *	printf ("argv[%d]: %s\n", i, argv[i]);
 *  printf ("lscale: %d   ldisk: %d\n", lscale, ldisk);
 */

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namlst);
      exit (-3);
      }

/*
 *   fprintf (stderr, "After each image is displayed,\n");
 *   fprintf (stderr, "Enter `q <return>' to quit, `<return>' to continue.\n");
 */
   while ((fscanf (LIST, "%s", namimg)) != EOF)
      {
      cphskp (namimg);

      /*** Initialize GL software. ***/

      if (imgcnt++ == 0)
         {
         foreground ();
         if (H.ires == 1)
         prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
         else
         prefposition (266, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/
         fminit ();			/* Initialize SGI font manager. */

         winopen ("cpwl");		/* Create GL window. */
         gl_wclear (BLACK);		/* Clear window to black. */
         doublebuffer ();		/* Double buffer mode. */
         gconfig ();
         }

   /*** Display image on IRIS screen in GL window. ***/

      fprintf (stdout, "Displaying image: %s\n", namimg);
      if ((ierr = glcp_view (namimg, linit, pmode)) < 0)
         {
         fprintf (stderr, "cpview error: %d\n", ierr);
         exit (-4);
         }
      swapbuffers ();

/*
 *      fprintf (stderr, "> ");
 *      ians = getchar ();
 *      if (ians == 0161)  exit (0);
 */
      }
   gexit ();
   exit (0);
}
