/* cpws.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 *  7 December 1992: add sequence + forward/backward options.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/img_scale.h"
int DSP = 1;
#define MAXIMG 250

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
   FILE *LIST;

   static char flist [MAXIMG][80];
   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Namelist file name. */
   static char answer [2];	/* Response to scaling question. */
   char *rindex ();		/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int glcp_view ();		/* Image display function. */
   int findex = 0;
   int i;			/* Index. */
   int ians;			/* KB response. */
   int ierr;			/* Error status from cpview(). */
   int larg;			/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int numfil;				/* # images in file list. */
   int pmode = 0;		/* Print mode. */
   int seq = 0;			/* Sequence indicator. */
   int wrap = 0;		/* Wrap-around sequence indicator. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [[-]sd] [filename | @namelist | ~namelist]\n", argv [0]);

      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      lscale = 1;

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      lscale = 0;
      ldisk  = 0;
      strcpy (namimg, argv[1]);
      }

   /*** Two arguments. ***/

   else
   if (argc == 3)
      {
      if ((strncmp (argv[1], "-", 1) == 0) || (strncmp (argv[1], "s", 1) == 0)
       || (strncmp (argv[1], "d", 1) == 0))
         {
         parg = argv[1];
         larg = strlen (parg);
         if (*parg == '-')
	    {
            parg++;
            larg--;
            }
         if (*parg == 's')  lscale = 1;
         if (*parg == 'd')  ldisk = 1;
         if (larg == 2)
            {
            parg++;
            if (*parg == 's')  lscale = 1;
            if (*parg == 'd')  ldisk = 1;
            }
         }
      else
         {
         ldisk = 0;
         lscale = 0;
         }
      strcpy (namimg, argv[2]);
      }

   /*** Invalid number of command line arguments. ***/

   else
      {
      fprintf (stderr, "usage: %s [[-]sd] [filename | @namelist | ~namelist]\n", argv [0]);
      exit (-1);
      }

   /*** Sequence of images ? ***/

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
         }
      numfil = i;
      fclose (LIST);
      }


   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax, clipmin, clipmax: ");
      fscanf  (stdin,  "%f,%f,%f,%f,%f,%f",
               &foldmin, &foldmax, &fnewmin, &fnewmax, &fclipmin, &fclipmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
/*
 *  for (i=0; i<argc; i++)
 *	printf ("argv[%d]: %s\n", i, argv[i]);
 *  printf ("lscale: %d   ldisk: %d\n", lscale, ldisk);
 */

   /*** Initialize GL software. ***/

   cphskp (namimg);
   foreground ();
   if (H.ires == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (266, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/
   fminit ();			/* Initialize SGI font manager. */

   winopen ("cpw");		/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

   doublebuffer ();
   gconfig ();

   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "      `  <return>' to go forward.\n");
      fprintf (stderr, "      `b <return>' to go backward.\n");
      fprintf (stderr, "Enter `q <return>' to quit\n");
      }

loop:

   if (seq)
      {
      if (findex > numfil) 
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }
      strcpy (namimg, &flist [findex][0]);
      }

   /*** Display image on IRIS screen in GL window. ***/

   if ((ierr = glcp_view (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      gexit ();
      exit (-2);
      }

   swapbuffers ();

   if (seq)
      {
      fflush (stdin);
      fprintf (stderr, "%s > ", namimg);
      ians = getc (stdin);
      if (ians == 0161)			/* "q" */
         goto quit;
      else
      if (ians == 0142)			/* "b" */
         findex--;
      else
         findex++;

      if (findex < 0)
         {
         fprintf (stderr, "*** Unable to back up prior to start of list.\n");
         findex = 0;
         }
      if (findex >= numfil)
         {
         fprintf (stderr, "*** Unable to advance beyond end of list.\n");
         findex = numfil - 1;
         }
      goto loop;
      }

   else		/*** Window remains on screen until terminated by mouse. ***/
      {
      /*** Window remains on screen until terminated by mouse. ***/

      qdevice (ESCKEY);
      qdevice (WINCLOSE);
      qdevice (WINQUIT);
      for (;;)
         {
         short val;
         switch (qread (&val))
            {
            case REDRAW:
               glcp_view (namimg, linit);
               break;
            case ESCKEY:
            case WINCLOSE:
            case WINQUIT:
               gexit ();
               exit (0);
            }
         }
      }

quit:
   gexit ();
   exit (0);
}
