/* cpw.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 *  7 December 1992: add sequence + forward/backward options.
 *  8 December 1992: use mouse instead of keyboard; add wrap-around option (^).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/img_scale.h"
int DSP = 1;
#define MAXIMG 250

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
   FILE *LIST;
   short val;

   static char flist [MAXIMG][80];
   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Namelist file name. */
   static char namwin [80];	/* Window name. */
   static char answer [2];	/* Response to scaling question. */
   char *rindex ();		/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int glcp_view ();		/* Image display function. */
   int gwid = -1 ;		/* SGI/GL window ID. */
   int findex = 0;
   int i;			/* Index. */
   int ians;			/* KB response. */
   int ierr;			/* Error status from cpview(). */
   int larg;			/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int lastimg = 0;		/* Last image index value. */
   int pmode = 0;		/* Print mode. */
   int prevres = -1;		/* image resolution. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [[-]sd] [filename | @namelist | ^namelist]\n", argv [0]);

      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      lscale = 1;

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   else
   if (argc == 2)
      {
      lscale = 0;
      ldisk  = 0;
      strcpy (namimg, argv [1]);
      }

   /*** Two arguments. ***/

   else
   if (argc == 3)
      {
      if ((strncmp (argv[1], "-", 1) == 0) || (strncmp (argv[1], "s", 1) == 0)
       || (strncmp (argv[1], "d", 1) == 0))
         {
         parg = argv[1];
         larg = strlen (parg);
         if (*parg == '-')
	    {
            parg++;
            larg--;
            }
         if (*parg == 's')  lscale = 1;
         if (*parg == 'd')  ldisk = 1;
         if (larg == 2)
            {
            parg++;
            if (*parg == 's')  lscale = 1;
            if (*parg == 'd')  ldisk = 1;
            }
         }
      else
         {
         ldisk = 0;
         lscale = 0;
         }
      strcpy (namimg, argv [2]);
      }

   /*** Invalid number of command line arguments. ***/

   else
      {
      fprintf (stderr, "usage: %s [[-]sd] [filename | @namelist | ^namelist]\n", argv [0]);
      exit (-1);
      }

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^')
      {
      if (namimg [0] == '@')
         seq = 1;
      else
      if (namimg [0] == '^')
	 seq = 2;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
         }
      lastimg = i;
      fclose (LIST);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax, clipmin, clipmax: ");
      fscanf  (stdin,  "%f,%f,%f,%f,%f,%f",
               &foldmin, &foldmax, &fnewmin, &fnewmax, &fclipmin, &fclipmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
   if (seq)
      cphskp (&flist [0][0]);
   else
      cphskp (namimg);
   prevres = H.ires;

   /*** Initialize GL software. ***/

/*   noborder () ; */
   foreground ();
   if (H.ires == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (256, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/
   fminit ();			/* Initialize SGI font manager. */

   sprintf (namwin, "%s\n", nampgm) ;

   gwid = winopen (namwin);	/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

/*   doublebuffer ();*/		/* Use double buffer mode. */
   gconfig ();

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left  mouse' to go forward.\n");
      fprintf (stderr, "click `right mouse' to go backward.\n");
      }
   fprintf (stderr,    "type  `ESC'         to quit.\n");

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]);
      cphskp (namimg) ;

   if (prevres != H.ires)
      {
      winclose (gwid) ;
      if (H.ires == 1)		/* 512x512 window in upper right corner.*/
         prefposition (760, 1271, 480, 991);
      else			/*1024x1024 window: upper right corner.*/
         prefposition (266, 1279,   0, 1023);
      gconfig ();
      gflush ();
      gwid = winopen (namwin) ;
      }
   prevres = H.ires;

   /*** Display image on IRIS screen in GL window. ***/

   if ((ierr = glcp_view (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      gexit ();
      exit (-2);
      }

/*   swapbuffers (); */

   /*** Window remains on screen until terminated by mouse. ***/

   /*** Declare devices. ***/

   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);
   qdevice (RIGHTMOUSE);

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.\n");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               goto loop;
               }
            break;

         case RIGHTMOUSE:
            if (val == 1 && seq > 0)	/* Right mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.\n");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               goto loop;
               }
            break;

         case MIDDLEMOUSE:		/* Middle mouse button is down. */
         case REDRAW:
            glcp_view (namimg, linit, pmode);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

quit:
   gexit ();
   exit (0);
}
