/* fgcp_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  Display an SMM C/P image ('filename') in IRIS frame buffer `FB'.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 19 March 1985
 * 19 March 1991: adapt for SGI/IRIS WHIP software.
 * 16 July 1991: Add FB argument to `cp_wimgfb' calls.
 * 16 September 1992: add logic to handle pic header (if present).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/grscale.ext"
#include "/home/stanger/lib/include/cp_color.h"

int CP_imgno, CP_dim;
int gpixrs;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

fgcp_view (filename, FB, linit)

char *filename;		/* Pointer to filename array. */
int FB;			/* WHIP Frame Buffer. */
int linit;		/* Image label flag; ON if =1. */

{
  extern int cp_scalers ();	/* Function to draw solar radius scale. */
  extern int cp_north ();	/* Function to draw pointer at sun center. */
  extern int cp_suncir ();	/* Function to draw circle at 1.0 Rsun. */
  extern int cp_label ();	/* Function to label SMM C/P frame. */
  extern int cp_annot ();	/* Function to write SMM C/P annotation. */
  extern int wdisk ();		/* Function to write black disk. */
  extern int wdiskb ();		/* Function to write black disk. */
  extern int srevbyt ();	/* Reverses bytes (short int images)*/
  extern char *malloc ();

  char *imgbuf;

  int cx, cy, radius;
  int fast = 1;
  int lbottom = 0;
  int lfull = 1;
  int nx = 448, ny = 448;
  int fin, x, y, len;
  int imin = 0, imax = 0;

  float dmin = 0.0, dmax = 0.0;
  float bkgcol = 0.0;
  float level  = 0.0;

   if ( (fin = cp_open (filename, &x, &y)) == -1 )	/* Open C/P image. */
      {
      fprintf (stderr, "cp_view> Unable to open file: %s\n", filename);
      return (-2);
      }

   /*** Read image into memory. ***/

   len = CP_dim * CP_dim * H.nbypix;
   imgbuf = malloc ( len );
   if (read (fin, imgbuf, len) <= 0 )
      {
      fprintf (stderr, "cp_view> read error.");
      return (-3);
      }
   close ( fin );

   /*
    * Clear area below image.
    * If image is high resolution, convert to low resolution.
    */

/*   fprintf (stderr, "cp_view> nbypix: %d   bytord: %d\n", nbypix, bytord); */

   /*** Erase frame buffer. ***/

   if (linit) fb_clear (FB, &bkgcol, fast);

   /*** Convert image to low resolution. ***/

   if ( H.ires == 0 ) cp_avg (imgbuf);
/*   printf ("cp_view> ires: %d  CP_dim: %d\n", H.ires, CP_dim); */

   /*** Compute radius (in pixels) for black disk to overlay occulting disk.***/

   cx = (int) xcen / nres;
   cy = (int) ycen / nres;
   level = 0;
   gpixrs = pixrs / nres;
   radius = (int) (1.4 * gpixrs);
/*
 *   fprintf (stderr, "cp_view> cx: %3d   cy: %3d   xcen: %3d   ycen: %3d   nres: %d  radius: %d\n",
 *           cx, cy, (int) xcen, (int) ycen, nres, radius);
 */

   if (H.icm == 1 && ldisk == 0)
      {
      wdisk (imgbuf, CP_dim, CP_dim, nbypix, cx, cy, radius, level);
      }

   /*** Write image to frame buffer. ***/

   if ( H.nbypix == 1 )		/* 1 byte per pixel. */
      {
      wboxb (FB, imgbuf, x, y, CP_dim, CP_dim, "rd");
      }

   else if (H.nbypix == 2)	/* 2 bytes per pixel (short integer). */
      {
      if (idtype == 3 && bytord == 1 && H.ires == 1)
	 srevbyt (imgbuf, nbypix, CP_dim*CP_dim); /* Reverse bytes if needed. */

      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 dmin = (float) foldmin;
	 dmax = (float) foldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
         if (H.ipw != 0)	/* Polaroid filter. */
         {
	 dmin =   0.0;
	 dmax = 200.0;
	 imin =   0;
	 imax = 250;
         }
      else			/* Default short integer intensity scaling. */
         {
	 dmin =   0.0;
	 dmax = 500.0;
	 imin =   0;
	 imax = 250;
         }
      cp_wimgfb (FB, imgbuf, x, y, CP_dim, CP_dim, nbypix, dmin, dmax, imin, imax);
      }

   else if (H.nbypix == 4)	/* 4 bytes per pixel (floating point). */
      {
      if (lscale == 1)
	 {
	 dmin = (float) foldmin;
	 dmax = (float) foldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
	 {
	 if (idtype == 4)   {dmin =  0.0;     dmax = 5.0e+15;} /* elec/cm2 */
	 if (idtype == 5)   {dmin = -2.e-9;   dmax = 2.0e-9;}  /* B2 - B1 */
	 if (idtype == 6)   {dmin =  0.0;     dmax = 2.5e-9;}  /* Bk */
	 if (idtype == 7)   {dmin = -1.0;     dmax = 1.0;}     /* (Bk-B1)/Bk */
	 if (idtype == 8)   {dmin = -1.0;     dmax = 1.0;}     /* (B2-B1)/B1 */
	 if (idtype == 9)   {dmin =  0.0;     dmax = 1.0;}     /* polarization*/
	 if (idtype == 10)  {dmin =  0.0;     dmax = 5.0e-9;}  /* pB */
	 if (idtype == 11)  {dmin =  0.0;     dmax = 5.0e-9;}  /* Br+t */
	 if (idtype == 12)  {dmin =  0.0;     dmax = 5.0e-9;}  /* Bt */
	 if (idtype == 13)  {dmin =  0.0;     dmax = 5.0e-9;}  /* Br */
	 if (idtype == 14)  {dmin =  0.0;     dmax = 2.5e-9;}  /* Bf + Bs */
	 if (idtype == 15)  {dmin =  0.0;     dmax = 2.0;}     /* B2 / B1 */
	 if (idtype == 16)  {dmin =  0.0;     dmax = 2.0e-9;}  /* B misc */
	 imin = 0;
	 imax = 250;
	 }
/*
 *      printf ("lscale: %d  foldmin: %g   foldmax: %g\n",
 *              lscale, foldmin, foldmax);
 */
      cp_wimgfb (FB, imgbuf, x, y, CP_dim, CP_dim, nbypix, dmin, dmax, imin, imax);
      }

   else
      {
      fprintf (stderr, "cp_view> Invalid image . . . nbypix: %d\n", nbypix);
      return (-4);
      }

   free (imgbuf);

   /*** Compute disk center in FB coordinates. ***/

   cx = (int) xcen / nres + gMINROW;
   cy = gMAXCOL - ((int) ycen / nres);
/*
 *   fprintf (stderr, "cp_view> cx: %3d   cy: %3d   xcen: %3d   ycen: %3d   nres: %d\n",
 *           cx, cy, (int) xcen, (int) ycen, nres);
 */

   /*** Draw solar radius scale. ***/
   /*** Draw pointer to mark center of sun & indicate north direction. ***/

   nres = 1;
   if (H.ires == 0)  nres = 2;
   x = (int) xcen / nres + gMINROW;
   y = gMAXCOL - ((int) ycen / nres);
   if (H.icm == 1 && ldisk == 0)  
      {
      cp_north (cx, cy, 10, 10, cp_yellow);
      cp_scalers (cp_yellow);
      cp_suncir (cp_yellow);
      }

   /*** Write annotation labels if linit = 1. ***/
   /*** Write annotation information. ***/

   if (linit)					/* Add labels for annotation. */
      cp_label (FB, lfull, lbottom, cp_grey, cp_green, cp_yellow);
    glcp_label (lfull, lbottom, cval2, cval3, cval4)


   cp_annot (FB, lbottom, cp_red, cp_grey, cp_green, cp_yellow); /* Annotate image. */

   return (0);
}


/* cp_open
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Open SMM C/P image file &  decommutate selected items from housekeeping.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (Extracted from 'smmsubs.c'.)
 * 29 August 1984
 * 1 November 1984: set 0-255 -> 0-255 scaling for calibrated polaroid images.
 * 9 November 1984: use hskp.h structure and cphskp.c function.
 * 15 January 1987: adapt to handle floating-point images.
 * 19 March 1991: insert logic to set xul & yul coordinates.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#define size_pichdr 24

cp_open (filename, xul, yul)

char *filename;			/* Image file name. */
int *xul, *yul;			/* Upper left corner of image in FB. */

{
int cphskp ();			/* Function to decode housekeeping. */
int fdimg;
int nrechd;
int i;
int idot;
long offset;
static char cbuf [size_pichdr];

   cphskp (filename);	/* Decommutate selected housekeeping items. */

   /*** Open image disk file. ***/

   if ((fdimg = open (filename, 0)) == -1)
      {
      fprintf (stderr, "cp_open> can't open file: %s\n", filename);
      return (-1);
      }

   if ( cp_dim (H.ih) != cp_dim (H.iv) )
      {
      CP_dim = 0;
      CP_imgno = 0;
      return (-2);
      }
   else
      {
      CP_dim = cp_dim (H.ih);
      if ( H.ires == 1 ) CP_dim = CP_dim /2;

      switch (H.iv)
         {
         case 017: CP_imgno = 1; break;
         case 016: switch (H.ih)
	    {
            case 016: CP_imgno = 2; break;
            case 07:  CP_imgno = 3; break;
            }
         break;
         case 07: switch (H.ih)
	    {
            case 016: CP_imgno = 4; break;
            case 07:  CP_imgno = 5; break;
            }
         break;
         case 014: switch (H.ih)
	    {
            case 014: CP_imgno = 6; break;
            case 06:  CP_imgno = 7; break;
            case 03:  CP_imgno = 8; break;
            }
         break;
         case 06:  switch (H.ih)
	    {
            case 014: CP_imgno = 9; break;
            case 06:  CP_imgno = 10; break;
            case 03:  CP_imgno = 11; break;
            }
         break;
         case 03:  switch (H.ih)
	    {
            case 014: CP_imgno = 12; break;
            case 06:  CP_imgno = 13; break;
            case 03:  CP_imgno = 14; break;
            }
         break;
         case 010: switch (H.ih)
	    {
            case 010: CP_imgno = 15; break;
            case 04:  CP_imgno = 16; break;
            case 02:  CP_imgno = 17; break;
            case 01:  CP_imgno = 18; break;
            }
         break;
         case 04:  switch (H.ih)
	    {
            case 010: CP_imgno = 19; break;
            case 04:  CP_imgno = 20; break;
            case 02:  CP_imgno = 21; break;
            case 01:  CP_imgno = 22; break;
            }
         break;
         case 02:  switch (H.ih)
	    {
            case 010: CP_imgno = 23; break;
            case 04:  CP_imgno = 24; break;
            case 02:  CP_imgno = 25; break;
            case 01:  CP_imgno = 26; break;
            }
         break;
         case 01:  switch (H.ih)
	    {
            case 010: CP_imgno = 27; break;
            case 04:  CP_imgno = 28; break;
            case 02:  CP_imgno = 29; break;
            case 01:  CP_imgno = 30; break;
            }
         break;
         default:  CP_imgno = 0;  return (-3);  break;
         }
      }

   idot = -1;
   for (i=0; i<strlen (filename);  i++)
      if (filename [i] == '.')  idot = i;

   if (idot < 0)
      {
      H.nbypix = -1;
      CP_dim = 0;
      CP_imgno = 0;
      return (-4);
      }

   /*** Position file pointer at beginning of image data. ***/

/*   fprintf (stderr, "cp_view> idtype: %d  nbypix: %d\n", H.idtype, H.nbypix);
 */
   if (nbypix == 4)
      nrechd = 1;
   else
      nrechd = 2;
   offset = (long) CP_dim * H.nbypix * nrechd;

   read (fdimg, cbuf, size_pichdr);
   if (ispic (cbuf))  offset += size_pichdr;

   if ( lseek (fdimg, offset, 0) == -1 )
      {
      fprintf (stderr,"cp_open> seek error.\n");
      return (-5);
      }

   /*** Set origin coordinates of image. ***/

   switch (CP_imgno)
      {
      case 1: *xul=63;   *yul=510; break;
      case 2: *xul=63;   *yul=510; break;
      case 3: *xul=175;  *yul=510; break;
      case 4: *xul=63;   *yul=398; break;
      case 5: *xul=175;  *yul=398; break;
      case 6: *xul=63;   *yul=510; break;
      case 7: *xul=175;  *yul=510; break;
      case 8: *xul=287;  *yul=510; break;
      case 9: *xul=63;   *yul=398; break;
      case 10: *xul=175; *yul=398; break;
      case 11: *xul=287; *yul=398; break;
      case 12: *xul=63;  *yul=286; break;
      case 13: *xul=175; *yul=286; break;
      case 14: *xul=287; *yul=286; break;
      case 15: *xul=63;  *yul=510; break;
      case 16: *xul=175; *yul=510; break;
      case 17: *xul=287; *yul=510; break;
      case 18: *xul=399; *yul=510; break;
      case 19: *xul=63;  *yul=398; break;
      case 20: *xul=175; *yul=398; break;
      case 21: *xul=287; *yul=398; break;
      case 22: *xul=399; *yul=398; break;
      case 23: *xul=63;  *yul=286; break;
      case 24: *xul=175; *yul=286; break;
      case 25: *xul=287; *yul=286; break;
      case 26: *xul=399; *yul=286; break;
      case 27: *xul=63;  *yul=174; break;
      case 28: *xul=175; *yul=174; break;
      case 29: *xul=287; *yul=174; break;
      case 30: *xul=399; *yul=174; break;
      }
   return (fdimg);
}


/* cp_avg
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert an SMM C/P image from high to low resolution.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c'.)
 * 29 August 1984
 * 19 March 1991: modify to accomodate floating point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

cp_avg (imgbuf)

char *imgbuf;	/* Pointer to image array. */

{
register char *bl, *bl1, *rbim;
register unsigned short  *il, *il1, *rim;
register float  *fl, *fl1, *rfim;
int i, j, size;
int temp;

   /*** One byte per pixel. ***/

   if ( H.nbypix == 1 )
      {
      bl = imgbuf;
      rbim = imgbuf;
      for (i=0; i<CP_dim; i+=2)
         {
         bl1 = bl + CP_dim;
         for (j=0; j<CP_dim; j+=2)
            {
            temp = ((*bl++)&0377) + ((*bl1++)&0377) +
                   ((*bl++)&0377) + ((*bl1++)&0377);
            *rbim++ =  temp/4 ;
            }
         bl = bl1;
         }
      }

   /*** 2 bytes per pixel. ***/

   else 
   if ( H.nbypix == 2 )
      {
      size = CP_dim * CP_dim;
      if (bytord == 1)  srevbyt (imgbuf, nbypix, size);
      il = (unsigned short *) imgbuf;
      rim = (unsigned short *) imgbuf;
      for (i=0; i<CP_dim; i+=2)
         {
         il1 = il + CP_dim;
         for (j=0; j<CP_dim; j+=2)
            {
            *rim++ = (*il++ + *il++ + *il1++ + *il1++)/4;
            }
         il = il1;
         }
      }

   /*** 4 bytes per pixel. ***/

   else
   if ( H.nbypix == 4 )
      {
      size = CP_dim * CP_dim;
      if (bytord == 1)  srevbyt (imgbuf, nbypix, size);
      fl = (float *) imgbuf;
      rfim = (float *) imgbuf;
      for (i=0; i<CP_dim; i+=2)
         {
         fl1 = fl + CP_dim;
         for (j=0; j<CP_dim; j+=2)
            {
            *rfim++ = (*fl++ + *fl++ + *fl1++ + *fl1++)/4;
            }
         fl = fl1;
         }
      }
   CP_dim /= 2;
   return ( 0 );
}


/* cp_dim (ihv)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Return the dimension of image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c'.)
 * 29 August 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

cp_dim (ihv)

int ihv;
{
   switch (ihv)
      {
      case 0:
      case 5:
      case 9:
      case 10:
      case 11:
      case 13: return (0);
      case 1:
      case 2:
      case 4:
      case 8: return (224);
      case 3:
      case 6:
      case 12: return (448);
      case 7:
      case 14: return (672);
      case 15: return (896);
      default : return (-1);
      }
}
