/* gl_wimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rescale intensities for an image to the range 0 -> 250.
 * Write rescaled image in SGI IRIS GL window.
 * Supports byte, short, and floating point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   Associate Scientist   HAO/NCAR
 * 15 January 1987
 * 23 October 1990: allow user to specify intensity scaling.
 * 11 April 1991: modify for SGI IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

gl_wimg (imgbuf, xs, ys, dx, dy, bytpix, dmin, dmax, imin, imax, pmode)

char *imgbuf;		/* Address of 2D image array. */
int xs, ys;		/* image origin coordinates. */
int dx, dy;		/* Image size. */
int bytpix;		/* # bytes per pixel. */
float dmin, dmax;	/* Data minimum & maximum. */
int imin, imax;		/* Image minimum & maximum. */
int pmode;		/* Print mode. */
{
extern int gl_wbox ();

unsigned short *glimg;
unsigned char *cimg;
unsigned short *simg;
float *fimg;
static char dir2 [] = "rd";

/*
 * int cx, cy, level;
 * int gpixrs;
 * int radius;
 */
int i;
int ix, iy;
int nbyimg;
int nelem;
int npxlin;

float offset = 0.0;
float slope = 1.e-16;
float fdmin = 1.e+20;			/* Initial user input minimum value. */
float fdmax = -1.e+20;			/* Initial user input maximum value. */
float fgmin = 1.0, fgmax = 249.0;	/* Min/Max limits allowed for display.*/
float fimin, fimax;			/* User min/max range for display. */
float value;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   if (pmode > 0)
      fprintf (stderr,
               "gl_wimg> dx: %d   dmin: %g   dmax: %g   imin: %d   imax: %d\n",
		dx, dmin, dmax, imin, imax);
   npxlin = dx;
   nelem = dx * dy;
   nbyimg = nelem * sizeof (short);

   /*** Set image min/max levels. ***/

   if (imin == 0 && imax == 0)
      {
      fimin = 0.0;
      fimax = 250.0;
      }
   else
      {
      if (imin < 0)
         fimin = 0.0;
      else
         fimin = (float) imin;

      if (imax < 0)
         fimax = 250.0;
      else
         fimax = (float) imax;
      }

   /*** Assign appropriate type of pointer to image buffer. ***/

   if (bytpix == 1)  cimg = (unsigned char *) imgbuf;
   else
   if (bytpix == 2)  simg = (unsigned short *) imgbuf;
   else
   if (bytpix == 4)  fimg = (float *) imgbuf;
   else
      {
      fprintf (stderr, "gl_wimg> invalid image: bytpix = %d\n", bytpix);
      return (-2);
      }

   /*** Allocate temporary storage buffer. ***/

   glimg = (unsigned short *) malloc (nbyimg);

   /*** Find min & max values in user input image array. ***/

   if (dmin == 0.0 && dmax == 0.0)      
      {
      if (bytpix == 1)
         for (i=0; i<nelem; i++)
            {
            if ((float) *(cimg + i) < fdmin)  fdmin = (float) *(cimg + i);
            if ((float) *(cimg + i) > fdmax)  fdmax = (float) *(cimg + i);
            }
      else
      if (bytpix == 2)
         for (i=0; i<nelem; i++)
            {
            if ((float) *(simg + i) < fdmin)  fdmin = (float) *(simg + i);
            if ((float) *(simg + i) > fdmax)  fdmax = (float) *(simg + i);
            }
      else
      if (bytpix == 4)
         for (i=0; i<nelem; i++)
            {
            if (*(fimg + i) < fdmin)  fdmin = *(fimg + i);
            if (*(fimg + i) > fdmax)  fdmax = *(fimg + i);
            }
      }
   else
      {
      fdmin = dmin;
      fdmax = dmax;
      }

   if (fdmax == fdmin)
      {
      fprintf (stderr, "gl_wimg> error: fdmax = fdmin\n");
      return (-1);
      }

/*   if (fdmin == -1.0)  fdmin = 0.0; */

   /*** If fdmin is negative, set fdmin & fdmax to be symmetric about zero. ***/

   if (fdmin < 0.0)
      {
      if (-fdmin > fdmax)  fdmax = -fdmin;
      else fdmin = -fdmax;
      }

   offset = fdmin;
   slope = (fimax - fimin) / (fdmax - fdmin);

/*
 *   fprintf (stderr, "gl_wimg> Rescaling intensities for IRIS display...please wait.\n");
 */

   /*** Perform intensity scaling for IRIS. ***/

   if (bytpix == 1)			/* Byte image. */
      for (iy = 0; iy < npxlin; iy++)
         for (ix = 0; ix < npxlin; ix++)
            {
            value = (float) (*(cimg + ix + iy*npxlin) - offset) * slope;
            if (value < fgmin)  value = fgmin;
            if (value > fgmax)  value = fgmax;
            *(glimg + ix + iy*npxlin) = ((unsigned short) value) & 0377;
            }
   else
   if (bytpix == 2)			/* Short integer image. */
      for (iy = 0; iy < npxlin; iy++)
         for (ix = 0; ix < npxlin; ix++)
            {
            value = (float) (*(simg + ix + iy*npxlin) - offset) * slope;
            if (value < fgmin)  value = fgmin;
            if (value > fgmax)  value = fgmax;
            *(glimg + ix + iy*npxlin) = ((unsigned short) value) & 0377;
            }
   else
   if (bytpix == 4)			/* Floating point image. */
      for (iy = 0; iy < npxlin; iy++)
         for (ix = 0; ix < npxlin; ix++)
            {
            value = (*(fimg + ix + iy*npxlin) - offset) * slope;
            if (value < fgmin)  value = fgmin;
            if (value > fgmax)  value = fgmax;
            *(glimg + ix + iy*npxlin) = ((unsigned short) value) & 0377;
            }

   /*** Write black disk into occulting disk. ***/

/*
 *   cx = (int) xcen / nres;
 *   cy = (int) ycen / nres;
 *   level = 0;
 *   gpixrs = pixrs / nres;
 *   radius = (int) (1.5 * gpixrs);
 *   if (H.icm == 1)
 *      {
 *      wdisk (glimg, npxlin, npxlin, cx, cy, radius, level);
 *      }
 */

   /*** Write image to IRIS GL window. ***/

   gl_wbox (glimg, 2, xs, ys, dx, dy, dir2, pmode);

   free (glimg);
   return (0);
}
