/* glcp_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  Display an SMM C/P image ('filename') in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 19 March 1985
 * 11 April 1991: adapt for SGI/IRIS GL software.
 * 26 April 1991: use onemap color mode.
 * 18 December 1992: change CP_dim, CP_imgno, gpixrs from global to local.
 * 20 July 1995: add intensity scaling for equivalent second data.
 * 12 May 1999: identify data type 16 as Mass (grams).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/gl_color.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glcp_view (filename, lfull, pmode)

char *filename;		/* Pointer to filename array. */
int lfull;		/* Image label flag; ON if =1. */
int pmode;		/* Print mode. */

{
  extern int glcp_scalers ();	/* Function to draw solar radius scale. */
  extern int glcp_north ();	/* Function to draw pointer at sun center. */
  extern int glcp_suncir ();	/* Function to draw circle at 1.0 Rsun. */
  extern int glcp_label ();	/* Function to label SMM C/P frame. */
  extern int glcp_annot ();	/* Function to write SMM C/P annotation. */
  extern int gl_wimg ();	/* Function to write image to GL window. */
  extern int wdisk ();		/* Function to write black disk. */
  extern int wdiskb ();		/* Function to write black disk. */
  extern int srevbyt ();	/* Reverses bytes (short int images)*/
  extern int DSP;		/* Grinnell usage flag: =0 off, =1 on. */
  extern char *malloc ();

  int cp_dim;			/* C/P x/y dimension. */
  int cx, cy, radius;
  int fast = 1;
  int fin, xs, ys, nbyimg;
  int gpixrs;
  int imin = 0, imax = 0;
  float dmin = 0.0, dmax = 0.0;
  float bkgcol = 0.0;
  char *imgbuf;
  static char dir2 [] = "rd";

   if (!DSP)
      {
      fprintf (stderr, "glcp_view> display option is not selected.\n");
      return (-1);
      }

   /*** Open C/P image. ***/

   if ( (fin = glcp_open (filename, &xs, &ys, &cp_dim, pmode)) == -1 )
      {
      fprintf (stderr, "glcp_view> Unable to open file: %s\n", filename);
      return (-2);
      }
   if (pmode > 0)
      fprintf (stderr, "glcp_view> xs: %d   ys: %d\n", xs, ys);

   /*** Read image into memory. ***/

   nbyimg = cp_dim * cp_dim * H.nbypix;
   imgbuf = malloc ( nbyimg );
   if (read (fin, imgbuf, nbyimg) <= 0 )
      {
      fprintf (stderr, "glcp_view> read error.");
      return (-3);
      }
   close ( fin );

   /*** Clear area below image. ***/

   if (pmode > 0)
      fprintf (stderr, "glcp_view> nbypix: %d   bytord: %d\n", nbypix, bytord);
   if (H.ih < 15)
      {
      color (BLACK);
      rectfi (63, 63, 510, 510);
      }

   /*** Erase GL window. ***/

   if (lfull) gl_wclear (BLACK);

   /*** If image is high resolution, convert to low resolution. ***/

/*   if( H.ires == 0 ) glcp_avg (imgbuf); */

   if ( H.nbypix == 1 )		/* 1 byte per pixel. */
      {
      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 dmin = (float) oldmin;
	 dmax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
         {
	 dmin =   0.0;
	 dmax = 250.0;
	 imin =   0;
	 imax = 250;
         }
      gl_wimg (imgbuf, xs, ys, cp_dim, cp_dim, nbypix, dmin, dmax, imin, imax,
               pmode);
      }

   else if (H.nbypix == 2)	/* 2 bytes per pixel (short integer). */
      {
      if (idtype == 3 && bytord == 1 && H.ires == 1)
	 srevbyt (imgbuf, nbypix, cp_dim*cp_dim); /* Reverse bytes if needed. */

      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 dmin = (float) oldmin;
	 dmax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
         if (H.ipw != 0)	/* Polaroid filter. */
         {
	 dmin =   0.0;
	 dmax = 200.0;
	 imin =   0;
	 imax = 250;
         }
      else
      if (idtype == 2)		/* Equivalent second scaling. */
	 {
	 dmin =   0.0;
	 dmax = 250.0;
	 imin =   0;
	 imax = 250;
	 }
      else			/* Default short integer intensity scaling. */
         {
	 dmin =   0.0;
	 dmax = 500.0;
	 imin =   0;
	 imax = 250;
         }

      gl_wimg (imgbuf, xs, ys, cp_dim, cp_dim, nbypix, dmin, dmax, imin, imax,
                pmode);
      }

   else if (H.nbypix == 4)	/* 4 bytes per pixel (floating point). */
      {
      if (lscale == 1)
	 {
	 dmin = (float) oldmin;
	 dmax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
	 {
	 if (idtype == 4)   {dmin =  0.0;     dmax = 5.0e+15;} /* elec/cm2 */
	 if (idtype == 5)   {dmin = -2.0e-9;  dmax = 2.0e-9;}  /* B2 - B1 */
	 if (idtype == 6)   {dmin =  0.0;     dmax = 2.5e-9;}  /* Bk */
	 if (idtype == 7)   {dmin = -1.0;     dmax = 1.0;}     /* (Bk-B1)/Bk */
	 if (idtype == 8)   {dmin = -1.0;     dmax = 1.0;}     /* (B2-B1)/B1 */
	 if (idtype == 9)   {dmin =  0.0;     dmax = 1.0;}     /* polarization*/
	 if (idtype == 10)  {dmin =  0.0;     dmax = 5.0e-9;}  /* pB */
	 if (idtype == 11)  {dmin =  0.0;     dmax = 5.0e-9;}  /* Br+t */
	 if (idtype == 12)  {dmin =  0.0;     dmax = 5.0e-9;}  /* Bt */
	 if (idtype == 13)  {dmin =  0.0;     dmax = 5.0e-9;}  /* Br */
	 if (idtype == 14)  {dmin =  0.0;     dmax = 2.5e-9;}  /* Bf + Bs */
	 if (idtype == 15)  {dmin =  0.0;     dmax = 2.0;}     /* B2 / B1 */
	 if (idtype == 16)  {dmin =  0.0;     dmax = 5.0e+12;} /* Mass(grams) */
	 imin = 0;
	 imax = 250;
	 }
      gl_wimg (imgbuf, xs, ys, cp_dim, cp_dim, nbypix, dmin, dmax, imin, imax,
               pmode);
      }

   else
      {
      fprintf (stderr, "glcp_view> Invalid image . . . nbypix: %d\n", nbypix);
      return (-4);
      }

   free (imgbuf);

   /*** Compute disk center in GL window coordinates. ***/

   cx = (int) ((xcen / nres) + 0.5) + gMINROW;
   cy = gMAXCOL - (int) ((ycen / nres) + 0.5);
   if (pmode > 0)
      fprintf (stderr, "glcp_view> cx: %3d   cy: %3d   xcen: %3d   ycen: %3d   nres: %d\n",
           cx, cy, (int) xcen, (int) ycen, nres);

   /*** Compute radius (in pixels) for black disk to overlay occulting disk.***/
   /*** Draw black filled circle at occulting disk position. ***/

   gpixrs = pixrs / nres;
   radius = (int) (1.5 * gpixrs);

   if (H.icm == 1 && ldisk == 0)
      {
      color (BLACK);
      circfi (cx, cy, radius);
      }

   /*** Draw solar radius scale. ***/
   /*** Draw pointer to mark center of sun & indicate north direction. ***/

   xs = (int) xcen / nres + gMINROW;
   ys = gMAXCOL - ((int) ycen / nres);
   if (H.icm == 1 && ldisk == 0)  
      {
      glcp_north (cx, cy, 0.1, 0.3, gl_yellow);
      glcp_scalers (gl_yellow);
      glcp_suncir (0.4, 1.5, 0.1, 90.0, 1.0, 1.5, 0.2, 30.0, gl_yellow);
      }

   /*** Write information labels if lfull = 1. ***/
   /*** Write configuration information. ***/

   glcp_label (lfull, ldisk, gl_grey, gl_green, gl_yellow);

   glcp_annot (ldisk, gl_red, gl_grey, gl_green, gl_yellow);

   return (0);
}


/* glcp_open.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Open SMM C/P image file &  decommutate selected items from housekeeping.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (Extracted from 'smmsubs.c'.)
 * 29 August 1984
 * 1 November 1984: set 0-255 -> 0-255 scaling for calibrated polaroid images.
 * 9 November 1984: use hskp.h structure and cphskp.c function.
 * 15 January 1987: adapt to handle floating-point images.
 * 19 March 1991: insert logic to set xul & yul coordinates.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

glcp_open (filename, xul, yul, cp_dim, pmode)

char *filename;			/* Image file name. */
int *xul, *yul;			/* Upper left corner of image in FB. */
int *cp_dim;			/* C/P x/y dimension. */

{
   extern int cphskp ();	/* Function to decode housekeeping. */

   int CP_dim;			/* C/P x/y dimension. */
   int CP_imgno;		/* C/P image index. */
   int fdimg;
   int nrechd;
   int i;
   int idot;
   long offset;

   cphskp (filename);	/* Decommutate selected housekeeping items. */

   /*** Open image disk file. ***/

   if ((fdimg = open (filename,0)) == -1)
      {
      fprintf (stderr, "glcp_open> can't open file: %s\n", filename);
      return (-1);
      }

   if ( glcp_dim (H.ih) != glcp_dim (H.iv) )
      {
      CP_dim = 0;
      CP_imgno = 0;
      return (-2);
      }
   else
      {
      CP_dim = glcp_dim (H.ih);
      if ( H.ires == 1 ) CP_dim = CP_dim /2;	/* Low resolution image. */

      switch (H.iv)
         {
         case 017: CP_imgno = 1; break;
         case 016: switch (H.ih)
	    {
            case 016: CP_imgno = 2; break;
            case 07:  CP_imgno = 3; break;
            }
         break;
         case 07: switch (H.ih)
	    {
            case 016: CP_imgno = 4; break;
            case 07:  CP_imgno = 5; break;
            }
         break;
         case 014: switch (H.ih)
	    {
            case 014: CP_imgno = 6; break;
            case 06:  CP_imgno = 7; break;
            case 03:  CP_imgno = 8; break;
            }
         break;
         case 06:  switch (H.ih)
	    {
            case 014: CP_imgno = 9; break;
            case 06:  CP_imgno = 10; break;
            case 03:  CP_imgno = 11; break;
            }
         break;
         case 03:  switch (H.ih)
	    {
            case 014: CP_imgno = 12; break;
            case 06:  CP_imgno = 13; break;
            case 03:  CP_imgno = 14; break;
            }
         break;
         case 010: switch (H.ih)
	    {
            case 010: CP_imgno = 15; break;
            case 04:  CP_imgno = 16; break;
            case 02:  CP_imgno = 17; break;
            case 01:  CP_imgno = 18; break;
            }
         break;
         case 04:  switch (H.ih)
	    {
            case 010: CP_imgno = 19; break;
            case 04:  CP_imgno = 20; break;
            case 02:  CP_imgno = 21; break;
            case 01:  CP_imgno = 22; break;
            }
         break;
         case 02:  switch (H.ih)
	    {
            case 010: CP_imgno = 23; break;
            case 04:  CP_imgno = 24; break;
            case 02:  CP_imgno = 25; break;
            case 01:  CP_imgno = 26; break;
            }
         break;
         case 01:  switch (H.ih)
	    {
            case 010: CP_imgno = 27; break;
            case 04:  CP_imgno = 28; break;
            case 02:  CP_imgno = 29; break;
            case 01:  CP_imgno = 30; break;
            }
         break;
         default:  CP_imgno = 0;  return (-3);  break;
         }
      }

   idot = -1;
   for (i=0; i<strlen (filename);  i++)
      if (filename [i] == '.')  idot = i;

   if (idot < 0)
      {
      H.nbypix = -1;
      CP_dim = 0;
      CP_imgno = 0;
      return (-4);
      }
   *cp_dim = CP_dim;

   /*** Position file pointer at beginning of image data. ***/

   if (pmode > 0)
      fprintf (stderr, "glcp_open> idtype: %d  nbypix: %d\n",
               H.idtype, H.nbypix);
   if (nbypix == 4)
      nrechd = 1;
   else
      nrechd = 2;
   offset = (long) CP_dim * H.nbypix * nrechd;
   if ( lseek (fdimg, offset, 0) == -1 )
      {
      fprintf (stderr,"glcp_open> seek error.\n");
      return (-5);
      }

   /*** Set origin coordinates of image. ***/

   switch (CP_imgno)
      {
      case 1: *xul=63;   *yul=510; break;
      case 2: *xul=63;   *yul=510; break;
      case 3: *xul=175;  *yul=510; break;
      case 4: *xul=63;   *yul=398; break;
      case 5: *xul=175;  *yul=398; break;
      case 6: *xul=63;   *yul=510; break;
      case 7: *xul=175;  *yul=510; break;
      case 8: *xul=287;  *yul=510; break;
      case 9: *xul=63;   *yul=398; break;
      case 10: *xul=175; *yul=398; break;
      case 11: *xul=287; *yul=398; break;
      case 12: *xul=63;  *yul=286; break;
      case 13: *xul=175; *yul=286; break;
      case 14: *xul=287; *yul=286; break;
      case 15: *xul=63;  *yul=510; break;
      case 16: *xul=175; *yul=510; break;
      case 17: *xul=287; *yul=510; break;
      case 18: *xul=399; *yul=510; break;
      case 19: *xul=63;  *yul=398; break;
      case 20: *xul=175; *yul=398; break;
      case 21: *xul=287; *yul=398; break;
      case 22: *xul=399; *yul=398; break;
      case 23: *xul=63;  *yul=286; break;
      case 24: *xul=175; *yul=286; break;
      case 25: *xul=287; *yul=286; break;
      case 26: *xul=399; *yul=286; break;
      case 27: *xul=63;  *yul=174; break;
      case 28: *xul=175; *yul=174; break;
      case 29: *xul=287; *yul=174; break;
      case 30: *xul=399; *yul=174; break;
      }
   if (H.ires == 0)
      {
      *xul *= 2;
      *yul *= 2;
      pixrs *= 2;
      rspix /= 2;
      xcen *= 2.0;
      ycen *= 2.0;
      gMINROW = 127;
      gMAXROW = 1022;
      gMINCOL = 127;
      gMAXCOL = 1022;
      }
   else
      {
      gMINROW =  63;
      gMAXROW = 510;
      gMINCOL =  63;
      gMAXCOL = 510;
      }
   return (fdimg);
}


/* glcp_avg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert an SMM C/P image from high to low resolution.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c'.)
 * 29 August 1984
 * 19 March 1991: modify to accomodate floating point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

glcp_avg (imgbuf, cp_dim)

char *imgbuf;		/* Pointer to image array. */
int *cp_dim;		/* C/P x/y dimension. */

{
register char *bl, *bl1, *rbim;
register unsigned short  *il, *il1, *rim;
register float  *fl, *fl1, *rfim;

int CP_dim;		/* C/P x/y dimension. */
int i, j, size;
int temp;

   CP_dim = *cp_dim;

   /*** One byte per pixel. ***/

   if ( H.nbypix == 1 )
      {
      bl = imgbuf;
      rbim = imgbuf;
      for (i=0; i<CP_dim; i+=2)
         {
         bl1 = bl + CP_dim;
         for (j=0; j<CP_dim; j+=2)
            {
            temp = ((*bl++)&0377) + ((*bl1++)&0377) +
                   ((*bl++)&0377) + ((*bl1++)&0377);
            *rbim++ =  temp/4 ;
            }
         bl = bl1;
         }
      }

   /*** 2 bytes per pixel. ***/

   else 
   if ( H.nbypix == 2 )
      {
      size = CP_dim * CP_dim;
      if (bytord == 1)  srevbyt (imgbuf, nbypix, size);
      il = (unsigned short *) imgbuf;
      rim = (unsigned short *) imgbuf;
      for (i=0; i<CP_dim; i+=2)
         {
         il1 = il + CP_dim;
         for (j=0; j<CP_dim; j+=2)
            {
            *rim++ = (*il++ + *il++ + *il1++ + *il1++)/4;
            }
         il = il1;
         }
      }

   /*** 4 bytes per pixel. ***/

   else
   if ( H.nbypix == 4 )
      {
      size = CP_dim * CP_dim;
      if (bytord == 1)  srevbyt (imgbuf, nbypix, size);
      fl = (float *) imgbuf;
      rfim = (float *) imgbuf;
      for (i=0; i<CP_dim; i+=2)
         {
         fl1 = fl + CP_dim;
         for (j=0; j<CP_dim; j+=2)
            {
            *rfim++ = (*fl++ + *fl++ + *fl1++ + *fl1++)/4;
            }
         fl = fl1;
         }
      }
   CP_dim /= 2;
   *cp_dim = CP_dim;
   return ( 0 );
}


/* glcp_dim.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Return the dimension of image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c'.)
 * 29 August 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

glcp_dim (ihv)

int ihv;
{
   switch (ihv)
      {
      case 0:
      case 5:
      case 9:
      case 10:
      case 11:
      case 13: return (0);
      case 1:
      case 2:
      case 4:
      case 8: return (224);
      case 3:
      case 6:
      case 12: return (448);
      case 7:
      case 14: return (672);
      case 15: return (896);
      default : return (-1);
      }
}
