/* glcp_annot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write SMM C/P image annotation to Iris GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger  HAO/NCAR   SMM C/P
 *  3 April 1991
 * 26 April 1991: use onemap color mode.
 * 6 August 1992: modify format to accomodate difference images.
 * 14 October 1992: change bfont=font3, bsize=11 for low res images (IRIX 4.0.1)
 * 23 July  1993: 1024x1024 pixel display window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/gl_color.h"

glcp_annot (lbottom, cval1, cval2, cval3, cval4)

int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int cval1;	/* Color level for annotation. */
int cval2;	/* Color level for labels. */
int cval3;	/* Color level for HAO logo. */
int cval4;	/* Color level for miscellaneous labels. */

{
   int month_day ();		/* Function to compute month & day from DOY. */
   char *month_name ();		/* Function to look up ASCII month name. */
   static char month [4];
   static char wbuf [12];
   static char imop [4];

   /* `Screen' font sizes:
    *  7 =>  6 x  7
    * 11,13,15 =>  8 x 15
    *
    * `Screen-Bold' font sizes:
    * 13,15 =>  9 x 15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11 x 13
    * 10 => 14 x 15
    * 12 => 18 x 20
    * 14 => 20 x 22
    * 18 => 27 x 29
    * 24 => 37 x 38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12 x 13
    * 10 => 14 x 15
    * 12 => 16 x 17
    * 14 => 22 x 23
    * 19 => 28 x 30
    * 25 => 38 x 40
    *
    * `Times-Roman'		`Times-Bold' font sizes:
    *  8 => 11 x 12		8 => 11 x 13
    *  9 => 12 x 15		9 => 13 x 15
    * 10 => 13 x 16		10 => 14 x 16
    * 11 => 15 x 18		11 => 16 x 18
    * 12 => 15 x 18		12 => 17 x 19
    * 14 => 19 x 20		14 => 19 x 19
    * 16 => 21 x 23		16 => 22 x 22
    * 18 => 24 x 28		18 => 27 x 28
    * 20 => 26 x 31		20 => 26 x 31
    * 24 => 35 x 36		24 => 35 x 37
    *
    * `Fixed' font sizes:	`Fixed-Bold' font sizes:
    *  6 =>  5 x  8		 6 =>  4 x  8
    *  8 =>  6 x 11		 8 =>  6 x 11
    *  9 =>  6 x 12		 9 => 13 x 15
    * 10 =>  7 x 13		10 => 14 x 16
    * 11 =>  9 x 15		11 => 16 x 18
    * 12 => 10 x 16		12 => 17 x 19
    * 14 =>  9 x 19		14 => 19 x 19
    * 16 => 11 x 21		16 => 22 x 22
    * 18 => 12 x 24		18 => 27 x 28
    * 20 => 13 x 26		20 => 26 x 31
    * 24 => 16 x 32		24 => 35 x 37
    */
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
   static char font5 [] = "Times";
   static char font6 [] = "Times-Bold";

   static char afont [32];
   static char bfont [32];
  
   static int afx, afy, bfx, bfy;
   int annot_state = 1;
   int charheight = 43;
   int lref;
   int offset = -21;
   int xstart = 2;
   int ystart = 485;
   int xc, yc;			/* height & width for character annotation. */
   int xs, ys;			/* Start position for character annotation. */
   int year, doy;
   int imonth, iday;
   int asize, bsize;


   /* Low resolution. */
/*
   if (H.ires == 1)
      {
      ystart = 510;
      xs = xstart;
      ys = ystart;
      yc = charheight;
      strcpy (afont, font1);
      strcpy (bfont, font3);
      asize =  7;
      bsize = 10;
      afx =  6;
      afy =  7;
      bfx = 14;
      bfy = 15;
      }
   else
   if (H.ires == 0)
 */
   /* High resolution. */
      {
      ystart = 1022;
      xs = xstart * 2;
      ys = ystart * 2;
      yc = charheight * 2;
      strcpy (afont, font6);
      strcpy (bfont, font6);
      asize = 10;
      bsize = 18;
      afx = 14;
      afy = 16;
      bfx = 27;
      bfy = 28;
      }
   ys = ystart - afy - 1;

   gl_setfont (bfont, bsize);		/* Select font. */

   /*** Define image operator (if applicable). ***/

   switch (idtype)
      {
      case 5:  strcpy (imop, "- "); lref = 1; break;
      case 7:  strcpy (imop, "-/"); lref = 1; break;
      case 8:  strcpy (imop, "-/"); lref = 1; break;
      case 15: strcpy (imop, " /"); lref = 1; break;
      default: strcpy (imop, "  "); lref = 0; break;
      }

   /*** Day and month. ***/

   year = H.year + 1900;
   doy = H.doy;
   (void) month_day (year, doy, &imonth, &iday);
   strcpy (month, month_name (imonth));
   sprintf (wbuf, "%d %s ", iday, month);
   gl_wstr (wbuf, xs, ys - 1*bfy, cval1);
   sprintf (wbuf, "%d", year);
   gl_wstr (wbuf, xs, ys - 2*bfy, cval1);

   /*** Year and Day-Of-Year: ***/

   (void) sprintf (wbuf, "%02d: %03d", H.year, H.doy);
   gl_wstr (wbuf, xs, ys - 2*afy - 4*bfy, cval1);

   /*** Reference image year & DOY (if applicable). ***/

   (void) sprintf (wbuf, "         ");
   if (lref)
      {
      if (H.year != year_zero)
         (void) sprintf (wbuf, "%s%02d:%03d", imop, year_zero, doy_zero);
      else
      if (H.year == year_zero && H.doy != doy_zero)
         (void) sprintf (wbuf, "   %s %03d ", imop, doy_zero);
      else
         (void) sprintf (wbuf, "         ");
      }
   gl_wstr (wbuf, xs, ys - 2*afy - 5*bfy, cval1);

   /*** Time: ***/

   (void) sprintf (wbuf, "%02d:%02d:%02d", H.hour, H.minute, H.second);
   gl_wstr (wbuf, xs, ys - 4*afy - 7*bfy, cval1);

   /*** Reference image time (if applicable). ***/

   if (lref)
      {
      (void) sprintf (wbuf, "    %s", imop);
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 8*bfy), cval1);
      (void) sprintf (wbuf, "%02d:%02d:%02d",
                      hour_zero, minute_zero, second_zero);
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 9*bfy), cval1);
      }
   else
      {
      (void) sprintf (wbuf, "         ");
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 8*bfy), cval1);
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 9*bfy), cval1);
      }

   /*** Color filter: ***/

   switch (H.ifw)
      {
      case 2: sprintf (wbuf,"OPEN  "); break;
      case 6: sprintf (wbuf,"Fe-XIV"); break;
      case 0: sprintf (wbuf,"RED   "); break;
      case 1: sprintf (wbuf,"Halpha"); break;
      case 3: sprintf (wbuf,"BLUE  "); break;
      case 4: sprintf (wbuf,"WIDEBD"); break;
      case 5: sprintf (wbuf,"GREEN "); break;
      default: sprintf (wbuf,"******"); break;
      }
   if (H.iglstw == 1)  sprintf (wbuf,"%s  ", H.glc);
   gl_wstr (wbuf, xs, ys - 6*afy - 11*bfy, cval1);

   /*** Polaroid filter: ***/

   switch (H.ipw)
      {
      case 0: sprintf (wbuf,"CLEAR "); break;
      case 1: sprintf (wbuf,"+60deg"); break;
      case 2: sprintf (wbuf,"-60deg"); break;
      case 3: sprintf (wbuf," 0 deg"); break;
      default: sprintf (wbuf,"***"); break;
      }
   gl_wstr (wbuf, xs, ys - 7*afy - 13*bfy, cval1);

   /*** Sector mirror: ***/

   switch (H.ism)
      {
      case 5:  sprintf (wbuf, "N   "); break;
      case 4:  sprintf (wbuf, "NE  "); break;
      case 3:  sprintf (wbuf, "E   "); break;
      case 2:  sprintf (wbuf, "SE  "); break;
      case 1:  sprintf (wbuf, "S   "); break;
      case 0:  sprintf (wbuf, "SW  "); break;
      case 7:  sprintf (wbuf, "W   "); break;
      case 6:  sprintf (wbuf, "NW  "); break;
      case 8:  sprintf (wbuf, "NESW"); break;
      default: sprintf (wbuf, "****"); break;
      }
   gl_wstr (wbuf, xs, ys - 8*afy - 15*bfy, cval1);

   /*** Exposure (1.024 second units) : ***/

   (void) sprintf (wbuf, "%-4u", H.exp);
   gl_wstr (wbuf, xs, ys - 9*afy - 17*bfy, cval1);

   /*** Resolution : ***/

   if( H.ires == 0 ) (void) sprintf (wbuf, "HIGH");
   else (void) sprintf (wbuf, "LOW ");
   gl_wstr (wbuf, xs, ys - 10*afy - 19*bfy, cval1);

   /*** S/C Roll : ***/

   if (H.icm == 1)
      {
      if (annot_state) 
         {
         glcp_label (1, lbottom, cval2, cval3, cval4);
         }
      (void) sprintf (wbuf, "%-4d", scroll);
      gl_setfont (bfont, bsize);	/* Select font. */
      gl_wstr (wbuf, xs, ys - 11*afy - 21*bfy, cval1);

      annot_state = 1;
      }

   /*** Calibration Mirror: ***/

   else
      {
      switch (H.icm)
         {
         case 0: (void) sprintf (wbuf, "MOVE"); break;
         case 1: (void) sprintf (wbuf, "OUT "); break;
         case 2: (void) sprintf (wbuf, "IN  "); break;
         case 3: (void) sprintf (wbuf, "BAD "); break;
         default: (void) sprintf (wbuf, "****"); break;
         }
      if (annot_state) 
         {
         glcp_label (1, lbottom, cval2, cval3, cval4);
         }
      gl_setfont (bfont, bsize);	/* Select font. */
      gl_wstr (wbuf, xs, ys - 11*afy - 21*bfy, cval1);
      annot_state = 0;
      }

   /*** Image count (frame number) : ***/

   (void) sprintf (wbuf, "%-4u", H.frame);
   gl_wstr (wbuf, xs, ys - 12*afy - 23*bfy, cval1);

   return (0);
}
