/* cplib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Global variables for cplib.a library.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*** cpgraf.h ***/

int	igterm;	/* Graphics terminal ?  =0 no, =1 yes. */
int	lcc;  /* Calibration correction flag. */
int	lden; /* =0  => Intensity;  =1  => Density. */
int	lfc;  /* F-corona removal? =0 no, =1 yes. */
int	lstr; /* Stray light removal ? =0 no, =1 yes. */
int	lsv;  /* Vignetting/Stray Light removal? =0 no, =1 yes.*/
int	lvig; /* Vignetting removal ? =0 no, =1 yes. */
int	lreg; /* Do disk registration? =0 no, =1 yes. */
int	ldif; /* Do image difference ? =0 no, =1 yes. */
int	modeg;	/* Plot mode: =0 linear, =1 log. */
int	rawtocal; /* Convert raw data to calibrated values ? =0 no, =1 yes. */

/*** cpgrin.h ***/

int	igr;	/* Grinnell usage flag: =0 no, =1 yes. */
int	ngr;	/* Grinnell device [0,1]. */
int	nres;	/* Inverse resolution flag: =1 Lo, =2 Hi. */
int	imgch;	/* Image channel [0,1,2,3]. */
int	ovrch;	/* Overlay channel [8,9,10,11,12]. */
float	gres;	/* Grinnell inverse resolution for non-standard size images. */

/*** cpscale.h ***/

float	pixrs;	/* # Pixels per solar radius. */
float	rspix;	/* # Solar radii per pixel. */
float	degrad;	/* # Degrees per solar radius. */
float	raddeg; /* # Solar radii per degree. */

/*** cpscan.h ***/

#define	MAXSCN	3600

/*** cpsize.h ***/

int	bytord;			/* byte order: 0 -> Sun-4; 1-> DEC. */
int	npixl;			/* actual # pixels per vidicon line. */
int	npix;			/* original # pixels per vidicon line. */
int	nbypix;			/* # bytes per pixel. */
int	ip, ir, lp, lr;		/* Pixel & line limits for image. */
int	idtype;			/* Data type: 1 = raw video
					      2 = Equivalent seconds
					      3 = 10^-11 B/Bsun */
int	nave;			/* # points to average in scan. */
int	icyear;			/* Year of calibrated file creation. */
int	icdoy;			/* DOY of calibrated file creation. */
float	pscale;			/* Scale factor for pixel magnitude. */

/*** cpsun.h ***/

int	scroll;		/* Adjusted S/C roll. */
float	degnor;		/* CCW angle of north w.r.t. up vertical axis. */
float	xcen;		/* x-axis coordinate of sun center: hi/lo res pixels.*/
float	ycen;		/* y-axis coordinate of sun center: hi/lo res lines. */

/*** hskp.h :   housekeeping structure for cphskp.h  ***/

struct hskp
   {
   char  time [22], filename [18];
   char  sec [6], res [3], fil [7], glc [5], pol [4], cal [4];
   int   year, doy, hour, minute, second;	/* Julian date & time. */
   int   frame, quan, roll;	/* frame counter, quantity, S/C roll. */
   int   exp, ih, iv, ires;	/* exposure, hor/ver, resolution. */
   int   ifw, ipw, ism, icm;	/* filter, polaroid, sector, cal mir. */
   int   iglpos, iglstw;	/* 5303 mechanism position + stow status. */
   int   ip, ir, lp, lr;	/* first & last pixel & row. */
   int   nbypix, idtype;	/* #bytes/pixel, data type. */
   int   str;			/* Stray light removed ? =0 no, =1 yes. */
   int   vig;			/* Vignetting removed ?  =0 no, =1 yes. */
   int   reg;			/* Disk registered ?     =0 no, =1 yes. */
   int   f;			/* F-corona removed ?    =0 no, =1 yes. */
   int   k;			/* K-corona removed ?    =0 no, =1 yes. */
   int   bit5;			/* not used. */
   int   bit6;			/* not used. */
   int   cc;			/* Munro cal.cor. applied? =0 no, =1 yes. */
   float xcen, ycen;		/* Disk center (x,y). */
   float degnor;		/* CCW angle of north w.r.t.up vertical axis. */
   float pixrs;			/* # pixels per solar radius. */
   float pscale;		/* Photometric scaling factor for pixel data. */
   };

struct hskp H;

/*** cpzero.h ***/

int year_zero;
int doy_zero;
int hour_zero;
int minute_zero;
int second_zero;
