/* cpw.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/gl/img_scale.h"
int DSP = 1;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
static char nampgm[80];	/* Program file name. */
static char namimg[80];	/* Image file name. */
static char answer[2];	/* Response to scaling question. */
char *rindex();		/* Index function. */
char *it;		/* File name pointer. */
char *parg;		/* Argument pointer. */

int glcp_view();		/* Image display function. */
int i;			/* Index. */
int ierr;		/* Error status from cpview(). */
int larg;		/* Length of scaling/disk flag argument. */
int linit = 1;		/* Image label flag [0=off,1=on]. */
int pmode = 0;		/* Print mode. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [filename]\n", argv [0]);

      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      lscale = 1;

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      lscale = 0;
      strcpy (namimg, argv[1]);
      }

   /*** Two arguments. ***/

   if (argc == 3)
      {
      if ((strncmp (argv[1], "-", 1) == 0) || (strncmp (argv[1], "s", 1) == 0)
       || (strncmp (argv[1], "d", 1) == 0))
         {
         parg = argv[1];
         larg = strlen (parg);
         if (*parg == '-')
	    {
            parg++;
            larg--;
            }
         if (*parg == 's')  lscale = 1;
         if (*parg == 'd')  ldisk = 1;
         if (larg == 2)
            {
            parg++;
            if (*parg == 's')  lscale = 1;
            if (*parg == 'd')  ldisk = 1;
            }
         }
      else
         {
         ldisk = 0;
         lscale = 0;
         }
      strcpy (namimg, argv[2]);
      }

   /*** Three arguments. ***/

   if (argc == 4)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [filename]\n", argv [0]);
      exit (-1);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax, clipmin, clipmax: ");
      fscanf  (stdin,  "%f,%f,%f,%f,%f,%f",
               &foldmin, &foldmax, &fnewmin, &fnewmax, &fclipmin, &fclipmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
/*
 *  for (i=0; i<argc; i++)
 *	printf ("argv[%d]: %s\n", i, argv[i]);
 *  printf ("lscale: %d   ldisk: %d\n", lscale, ldisk);
 */

   /*** Initialize GL software. ***/

   cphskp (namimg);
   foreground ();
   if (H.ires == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (266, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/
   fminit ();			/* Initialize SGI font manager. */

   winopen ("cpw");		/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);
   if ((ierr = glcp_view (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      gexit ();
      exit (-2);
      }

   /*** Window remains on screen until terminated by mouse. ***/

   qdevice (ESCKEY);
   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   for (;;)
      {
      short val;
      switch (qread (&val))
         {
         case REDRAW:
            glcp_view (namimg, linit);
            break;
         case ESCKEY:
         case WINCLOSE:
         case WINQUIT:
            gexit ();
            exit (0);
         }
      }
}
