/* cpwe.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * Draw perimeter of area-of-interest.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * 10 October 1991: add ability to read cursor position.
 * 23 October 1991: read mouse position only when left button is pressed.
 * 29 October 1991: write all information into output file.
 * 31 October 1991: use curve editor to draw area outline.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
#include "/usr/people/smm/lib/include/ov_color.h"

#include <stdio.h>
#include <fmclient.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "curved.h"
#define XY 2
#define X 0
#define Y 1
Device mdev [XY];
short mval [XY];
int DSP = 1;
int xo, yo;                 /* x/y origin of current window */
int xs, ys;                 /* x/y size of current window   */
char nampgm [80];	/* Program file name. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   int glcp_view ();		/* Image display function. */

   static char namimg [80];	/* Image file name. */
   static char nampos [80];	/* Position measurement file name. */
   static char answer [2];	/* Response to scaling question. */

   char *rindex();		/* Index function. */
   char *it;		/* File name pointer. */
   char *parg;		/* Argument pointer. */

   int i;			/* Index. */
   int ierr;		/* Error status from cpview(). */
   int larg;		/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int measure = 0;		/* Counter for # of measurements. */
   int xocp = 63, yocp = 510;	/* C/P image origin in window coordinates. */

   short val;
   short xwin, ywin, num;
   long dev;
   long xscrnsize;              /* size of screen in x used to set globals */
   long xmaxscrn, ymaxscrn;     /* maximum size of screen in x and y       */
   Colorindex dummy;            /* for strict ANSI C prototyping */

   float xcp, ycp;
   float r, th;

   FILE *fdpos;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   xscrnsize = getgdesc (GD_XPMAX);
   if (xscrnsize == 1280)
      {
      xmaxscrn = 1279;
      ymaxscrn = 1023;
      }
   else if (xscrnsize = 1023)
      {
      xmaxscrn = 1023;
      ymaxscrn = 767;
      }
   else
      {
      fprintf(stderr, "Something's EXTREMELY wrong:  ");
      fprintf(stderr, "xscrnsize=%d\n", xscrnsize);
      exit(-1) ;
      }

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [filename]\n", argv [0]);

      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      lscale = 1;

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      lscale = 0;
      strcpy (namimg, argv[1]);
      }

   /*** Two arguments. ***/

   if (argc == 3)
      {
      if ((strncmp (argv[1], "-", 1) == 0) || (strncmp (argv[1], "s", 1) == 0)
       || (strncmp (argv[1], "d", 1) == 0))
         {
         parg = argv[1];
         larg = strlen (parg);
         if (*parg == '-')
	    {
            parg++;
            larg--;
            }
         if (*parg == 's')  lscale = 1;
         if (*parg == 'd')  ldisk = 1;
         if (larg == 2)
            {
            parg++;
            if (*parg == 's')  lscale = 1;
            if (*parg == 'd')  ldisk = 1;
            }
         }
      else
         {
         ldisk = 0;
         lscale = 0;
         }
      strcpy (namimg, argv[2]);
      }

   /*** Three arguments. ***/

   if (argc == 4)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [filename]\n", argv [0]);
      exit (-2);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax, clipmin, clipmax: ");
      fscanf  (stdin,  "%f,%f,%f,%f,%f,%f",
               &foldmin, &foldmax, &fnewmin, &fnewmax, &fclipmin, &fclipmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
/*
 *  for (i=0; i<argc; i++)
 *	printf ("argv[%d]: %s\n", i, argv[i]);
 *  printf ("lscale: %d   ldisk: %d\n", lscale, ldisk);
 */

   /*** Decode SMM C/P housekeeping. ***/

   cphskp (namimg);
   strcpy (nampos, namimg);
   strcat (nampos, ".pos");
   if ((fdpos = fopen (nampos, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", nampos);
      exit (-3);
      }
   else
      {
      fprintf (stderr, "Cursor measurement file: %s\n", nampos);
      }


   /*** Initialize GL software. ***/

   makeframe (nampgm, H.ires, &xo, &yo, &xs, &ys);
   init_dev ();

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);
   if ((ierr = glcp_view (namimg, linit)) < 0)
      {
      fprintf (stderr, "glcp_view error: %d\n", ierr);
      exit (-4);
      }

   while (1)  getinput ();
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * makeframe: create GL window.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 31 October 1991
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

makeframe (nampgm, ires, xo, yo, xs, ys)

char *nampgm;
int ires;
int *xo, *yo;
int *xs, *ys;

{
   foreground ();

   if (ires == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (266, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/

   fminit ();			/* Initialize SGI font manager. */

   winopen (nampgm);		/* Create GL window. */
   getsize (xs, ys);		/* Get window size. */
   getorigin (xo, yo);	/* Get window origin (lower left corner). */

   return (0);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * init_dev: initialize devices.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 31 October 1991
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

init_dev ()

{
   int dummy;
   static unsigned short cursor [128];
   static Cursor plus = {0x0100, 0x0100, 0x0100, 0x0100,
                         0x0100, 0x0100, 0x0100, 0xFEFE,
                         0x0100, 0x0100, 0x0100, 0x0100,
                         0x0100, 0x0100, 0x0100, 0x0000};
 
   static Cursor plus1 = {0x0100, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0380, 0xFEFE,
                          0x0380, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0100, 0x0000};
 

   qdevice (MOUSEX);
   qdevice (MOUSEY);
   qdevice (LEFTMOUSE);
   qdevice (MENUBUTTON);
   qdevice (KEYBD);
   qdevice (ESCKEY);

   mdev [X] = MOUSEX;
   mdev [Y] = MOUSEY;

   curstype (CCROSS);			/* Crosshair cursor. */
   defcursor (1, cursor);
   curorigin (1,0,0);

   curstype (C16X1);			/* Small crosshair cursor. */
   defcursor (2, plus);
   curorigin (2,7,7);

   curstype (C16X1);			/* Modified small crosshair cursor. */
   defcursor (3, plus1);
   curorigin (3,7,7);

   setcursor (1, dummy, dummy);

   gl_wclear (BLACK);		/* Clear window to black. */

   return (0);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

getinput ()
{

   Device dev;
   short val;
   int sel;

   do
      {
      if (!qtest ())
         mouseevent (2, mx, my);
      dev = qread (&val);
      switch (dev)
         {
         case MENUBUTTON:
              if (val)
                 {
	         sel = dopup (menu);
                 if (sel > 0) 
 	 	    curmode = sel;
                 }
              font (1);
              break;
         case LEFTMOUSE: 
              printf ("LEFT> val: %d   mx: %6.2f   my: %6.2f\n", val, mx, my);
              mouseevent (val, mx, my);
              break;
         case MOUSEX:
/*              mx = ( (100.0 * ((val) - xo)) / xs ); */
              break;
         case MOUSEY: 
/*              my = ( (100.0 * ((val) - yo)) / xs ); */
              break;
         case KEYBD: 
              switch (val)
                 {
                 case 'a': 
                      break;
                 case 'i': 
                      curmode = INSERTPOINT;
                      break;
                 case 'm': 
                      curmode = MOVEPOINT;
                      break;
                 case 'd': 
                      curmode = DELETEPOINT;
                      break;
                 case 'c': 
                      curmode = CHANGEPOINT;
                      break;
                 }
              break;
         case REDRAW: 
              reshapeviewport ();
              makeframe (nampgm, H.ires, &xo, &yo, &xs, &ys);
              break;
         case ESCKEY: 
              gexit ();
              exit (0);
              break;
         }
      } while (qtest ());
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

int curpoint;
int moving;

mouseevent (state, x, y)

int state; 
float x, y;
{
   int nextpoint;

   switch (curmode)
      {
      case ADDPOINT:
           if (state == 1)
              {
              curpoint = duppoint (points);
              geom [curpoint][0] = x;
              geom [curpoint][1] = y;
              pt [curpoint] = SQUARE;
              drawline (LINE);
              }
           break;
      case MOVEPOINT:
           if (state == 1)
              {
              curpoint = findpoint (x, y); 
              moving = 1;
              }
           else if (state == 2)
              {
              if (moving)
                 {
                 drawline (BACKGROUND);
                 geom [curpoint][0] = x;
                 geom [curpoint][1] = y;
                 drawline (LINE);
                 }
              }
           else if (state == 0) 
              moving = 0;
           break;
      case INSERTPOINT:
           if (state == 1)
              {
              curpoint = findpoint (x, y); 
              if (curpoint < 0) 
                 curpoint = duppoint (points);
              else
                 curpoint = duppoint (curpoint);
              drawline (BACKGROUND);
              geom [curpoint][0] = x;
              geom [curpoint][1] = y;
              pt [curpoint] = SQUARE;
              drawline (LINE);
              }
           break;
      case DELETEPOINT:
           if (state == 1)
              {
              curpoint = findpoint (x, y); 
              if (curpoint >= 0)
                 {
                 drawline (BACKGROUND);
                 delpoint (curpoint);
                 drawline (LINE);
                 }
              }
           break;
      case CHANGEPOINT:
           if (state == 1)
              {
              curpoint = findpoint (x, y); 
              if (curpoint >= 0)
                 {
                 drawline (BACKGROUND);
                 if (pt [curpoint] == ROUND)
                    pt [curpoint] = SQUARE;
                 else
                    pt [curpoint] = ROUND;
                    drawline(LINE);
                 }
              }
           break;
      }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

float ppdist (x1, y1, x2, y2)

float x1, y1, x2, y2;
{
   register float dx, dy;

   dx = x2 - x1;
   if (dx < 0) dx = -dx;

   dy = y2 - y1;
   if (dy < 0) dy = -dy;

   return dx+dy;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

float pldist (x, y, x1, y1, x2, y2)

float x, y, x1, y1, x2, y2;
{
   register float dx, dy, c;
    
   dx = x2 - x1;
   dy = y2 - y1;
   c = dy*x1 - dx*y1;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

findpoint (x, y)

float x, y;
{
   register float mindist;
   register int minpnt;
   register int i;
   float dist;

   mindist = 100000.0;
   minpnt = -1;
   for (i=0; i<points; i++)
      {
      dist = ppdist (geom [i][0], geom [i][1], x, y);
      if (dist < mindist)
         {
         mindist = dist;
         minpnt = i;
         }
      }
   return minpnt;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

findedge (x, y)

float x, y;
{
   register float mindist;
   register int minpnt;
   register int i;
   float dist;

   mindist = 100000.0;
   minpnt = -1;
   for (i=0; i<points; i++)
      {
      dist = ppdist(geom[i][0],geom[i][1],x,y);
      if (dist<mindist)
         {
         mindist = dist;
         minpnt = i;
         }
      }
   return minpnt;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

duppoint (pnt)

int pnt;
{
   register int i;

   for (i=points; i>pnt; i--)
      {
      geom [i][0] = geom [i-1][0];
      geom [i][1] = geom [i-1][1];
      pt [i] = pt [i-1];
      }
   points++;
   return pnt;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

delpoint (pnt)

int pnt;
{
   register int i;

   for (i=pnt; i<points; i++)
      {
      geom [i][0] = geom [i+1][0];
      geom [i][1] = geom [i+1][1];
      pt [i] = pt [i+1];
      }
   points--;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

drawline (c)

int c;
{
   register int i, j;

   pt [0] = SQUARE;
   pt [points-1] = SQUARE;
   color (c);
   if (c == BACKGROUND) 
      clear();
   else
      {
      setlinestyle (1);
      move2 (geom [0][0], geom [0][1]);
      for (i=0; i<points; i++)
         {
         draw2 (geom [i][0], geom [i][1]);
         putsym (i);
         }
      setlinestyle (0);
      move2 (geom [0][0], geom [0][1]);
      for (i=1; i<points; i++)
         {
         if (pt [i] == SQUARE)
            {
            if (pt [i-1] == ROUND)
               {
               for (j=0; j<4; j++)
                  {
                  endgeom [j][0] = geom [i-2+j][0];
                  endgeom [j][1] = geom [i-2+j][1];
                  }
               endgeom [3][0] = endgeom [2][0];
               endgeom [3][1] = endgeom [2][1];
               curvebasis (BSPLINE);
               crv (endgeom); 
               draw2 (geom [i][0], geom [i][1]);
               }
            else
               {
               draw2 (geom [i][0], geom [i][1]);
               }
            }
         else
            {
            if (pt [i-1] == SQUARE)
               {
               for (j=0; j<4; j++)
                  {
                  endgeom [j][0] = geom [i-2+j][0];
                  endgeom [j][1] = geom [i-2+j][1];
                  }
               endgeom [0][0] = endgeom [1][0];
               endgeom [0][1] = endgeom [1][1];
               curvebasis (BSPLINE);
               crv (endgeom); 
               }
            else
               {
               curvebasis (BSPLINE);
               crv ((Coord (*)[3]) geom [i-2]);
               }
            }
         }
      }
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */

putsym (i)

register int i;
{
   char buf [2];

   cmov2 (geom [i][0], geom [i][1]);
   if (pt [i] == SQUARE)
      buf [0] = 1;
   else
      buf [0] = 2;
   buf [1] = 0;
   charstr (buf);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
