/* cpwf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * 10 October 1991: add ability to read cursor position.
 * 23 October 1991: read mouse position only when left button is pressed.
 * 29 October 1991: write all information into output file.
 * 6 November 1991: fill polygon region.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"

#include <stdio.h>
#include <fmclient.h>
#include <gl/gl.h>
#include <gl/device.h>
#define XY 2
#define X 0
#define Y 1
int DSP = 1;
Device mdev [XY];
short mval [XY];
int measure = 0;		/* Counter for # of measurements. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   int glcp_view ();		/* Image display function. */

   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char nampos [80];	/* Position measurement file name. */
   static char answer [2];	/* Response to scaling question. */

   char *rindex();		/* Index function. */
   char *it;		/* File name pointer. */
   char *parg;		/* Argument pointer. */

   int i;			/* Index. */
   int ierr;		/* Error status from cpview(). */
   int larg;		/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int xo, yo;			/* x/y origin of current window */
   int xs, ys;			/* x/y size of current window   */
   int xmaxscrn, ymaxscrn;	/* maximum size of screen in x and y       */

   FILE *fdpos;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [filename]\n", argv [0]);

      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      lscale = 1;

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      lscale = 0;
      strcpy (namimg, argv[1]);
      }

   /*** Two arguments. ***/

   if (argc == 3)
      {
      if ((strncmp (argv[1], "-", 1) == 0) || (strncmp (argv[1], "s", 1) == 0)
       || (strncmp (argv[1], "d", 1) == 0))
         {
         parg = argv[1];
         larg = strlen (parg);
         if (*parg == '-')
	    {
            parg++;
            larg--;
            }
         if (*parg == 's')  lscale = 1;
         if (*parg == 'd')  ldisk = 1;
         if (larg == 2)
            {
            parg++;
            if (*parg == 's')  lscale = 1;
            if (*parg == 'd')  ldisk = 1;
            }
         }
      else
         {
         ldisk = 0;
         lscale = 0;
         }
      strcpy (namimg, argv[2]);
      }

   /*** Three arguments. ***/

   if (argc == 4)
      {
      fprintf (stderr, "Usage: %s [[-]sd] [filename]\n", argv [0]);
      exit (-2);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax, clipmin, clipmax: ");
      fscanf  (stdin,  "%f,%f,%f,%f,%f,%f",
               &foldmin, &foldmax, &fnewmin, &fnewmax, &fclipmin, &fclipmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
/*
 *  for (i=0; i<argc; i++)
 *	printf ("argv[%d]: %s\n", i, argv[i]);
 *  printf ("lscale: %d   ldisk: %d\n", lscale, ldisk);
 */

   /*** Decode SMM C/P housekeeping. ***/

   cphskp (namimg);
   strcpy (nampos, namimg);
   strcat (nampos, ".pos");
   if ((fdpos = fopen (nampos, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", nampos);
      exit (-3);
      }
   else
      {
      fprintf (stderr, "Cursor measurement file: %s\n", nampos);
      }

   /*** Create GL window, define & activate cursor. ***/

   win_init (nampgm, H.ires, &xo, &yo, &xs, &ys, &xmaxscrn, &ymaxscrn);

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);
   if ((ierr = glcp_view (namimg, linit)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      exit (-4);
      }
   fprintf (stdout, "xwin ywin  xcp  ycp    r      th\n");

   /*** Get input from keyboard or mouse. ***/

   get_input (nampos, fdpos, &xo, &yo, &xs, &ys, xmaxscrn, ymaxscrn);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * win_init : create GL window, define & activate cursor.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
win_init (nampgm, ires, xo, yo, xs, ys, xmaxscrn, ymaxscrn)

char nampgm [];			/* Program name. */
int ires;			/* C/P image resolution: 0=HI, 1=LO. */
int *xo, *yo;			/* Window origin. */
int *xs, *ys;			/* Window size. */
int *xmaxscrn, *ymaxscrn;	/* Screen size. */

{
   Colorindex dummy;		/* for strict ANSI C prototyping */
   int xscrnsize;		/* size of screen in x used to set globals */
   static unsigned short cursor [128];
   static Cursor plus = {0x0100, 0x0100, 0x0100, 0x0100,
                         0x0100, 0x0100, 0x0100, 0xFEFE,
                         0x0100, 0x0100, 0x0100, 0x0100,
                         0x0100, 0x0100, 0x0100, 0x0000};
 
   static Cursor plus1 = {0x0100, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0380, 0xFEFE,
                          0x0380, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0100, 0x0000};

   xscrnsize = getgdesc (GD_XPMAX);
   if (xscrnsize == 1280)
      {
      *xmaxscrn = 1279;
      *ymaxscrn = 1023;
      }
   else if (xscrnsize = 1023)
      {
      *xmaxscrn = 1023;
      *ymaxscrn = 767;
      }
   else
      {
      fprintf(stderr, "Something's EXTREMELY wrong:  ");
      fprintf(stderr, "xscrnsize=%d\n", xscrnsize);
      return (-1) ;
      }

   /*** Initialize GL software. ***/

   foreground ();
   if (ires == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (266, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/
   fminit ();			/* Initialize SGI font manager. */

   winopen (nampgm);		/* Create GL window. */
   getsize (xs, ys);		/* Get window size. */
   getorigin (xo, yo);	/* Get window origin (lower left corner). */
   qdevice (ESCKEY);
   qdevice (ZEROKEY);
   qdevice (ONEKEY);
   qdevice (TWOKEY);
   qdevice (THREEKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   mdev [X] = MOUSEX;
   mdev [Y] = MOUSEY;

   curstype (CCROSS);			/* Crosshair cursor. */
   defcursor (1, cursor);
   curorigin (1,0,0);

   curstype (C16X1);			/* Small crosshair cursor. */
   defcursor (2, plus1);
   curorigin (2,7,7);

   curstype (C16X1);			/* Modified small crosshair cursor. */
   defcursor (3, plus1);
   curorigin (3,7,7);

   setcursor (1, dummy, dummy);

   gl_wclear (BLACK);		/* Clear window to black. */
   return (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * get_input : get input from keyboard or mouse.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
get_input (nampos, fdpos, xo, yo, xs, ys, xmaxscrn, ymaxscrn)

char nampos [];			/* Cursor measurement file. */
FILE *fdpos;			/* File descriptor: cursor measurement file. */
int *xo, *yo;			/* Window origin. */
int *xs, *ys;			/* Window size. */
int xmaxscrn, ymaxscrn;		/* Screen size. */

{
   short val;
   Colorindex dummy;		/* for strict ANSI C prototyping */
   int boxsize = 3;		/* size of box to indicate cursor points. */
   int dev;
   static int poly [100][2];
   int ip = 0;
   int xocp = 63, yocp = 510;	/* C/P image origin in window coordinates. */
   int xwin, ywin;		/* x/y window coordinates. */

   float xcp, ycp;		/* x/y coordinates for C/P image file. */
   float r, th;			/* radius & position angle for C/P image. */

   while (1)
      {
      while (qtest ())
         {
         dev = qread (&val);
         switch (dev)
            {
            case ESCKEY:	/* Terminate program. */
                 setvaluator (MOUSEX, (short)(*xs/2), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(*ys/2), 0, ymaxscrn);
                 fclose (fdpos);
                 if (measure == 0)  unlink (nampos);
                 gexit ();
                 exit (0);
                 break;

            case REDRAW:	/* Clear window to black. */
                 reshapeviewport ();
                 getsize (*xs, *ys);
                 getorigin (*xo, *yo);
                 color (BLACK);
                 clear ();
                 break;

            case ZEROKEY:	/* Fill polygon. */
                 polyfill (poly, ip, BLUE);
                 break;

            case ONEKEY:	/* cursor associated with 1-key. */
                 setcursor (1, dummy, dummy);
                 setvaluator (MOUSEX, (short)(*xo + (0.5 * *xs)), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(*yo + (0.5 * *ys)), 0, ymaxscrn);
                 break;

            case TWOKEY:	/* cursor associated with 2-key. */
                 setcursor (2, dummy, dummy);
                 setvaluator (MOUSEX, (short)(*xo + (0.5 * *xs)), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(*yo + (0.5 * *ys)), 0, ymaxscrn);
                 break;

            case THREEKEY:	/* cursor associated with 3-key. */
                 setcursor (3, dummy, dummy);
                 setvaluator (MOUSEX, (short)(*xo + (0.5 * *xs)), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(*yo + (0.5 * *ys)), 0, ymaxscrn);
                 break;

            case LEFTMOUSE:	/* Left Mouse Button. */
                 measure++;
                 if (val == 1)	/* Read position when left key depressed. */
                    {
                    getdev (XY, mdev, mval);
                    xwin = mval [X] - *xo;		/* x window position. */
                    ywin = mval [Y] - *yo;		/* y window position. */
                    xcp = (float) (xwin - xocp); 	/* x C/P position. */
                    ycp = (float) (yocp - ywin); 	/* y C/P position. */
                    rcoord (&r, &th, &xcp, &ycp, -1, scroll);	/* Get r & th.*/
                    fprintf (stdout, " %3d  %3d  %3.0f  %3.0f  %5.2f  %6.2f\n",
                             xwin, ywin, xcp, ycp, r, th);
                    fprintf (fdpos, "%3d %3d\n", xwin, ywin);
/*
 *                    fprintf (fdpos,  " %3d  %3d  %3.0f  %3.0f  %5.2f  %6.2f\n",
 *                             xwin, ywin, xcp, ycp, r, th);
 */
                    wglbox (xwin, ywin, boxsize, RED);	/* Draw small box. */
                    poly [ip][0] = xwin;
                    poly [ip][1] = ywin;
                    ip++;
                    }
                 break;

            default:
                 break;
            }
         }
      }
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * wglbox : write box centered at cursor position.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
wglbox (xw, yw, size, cval)

int xw, yw;		/* x/y window position (pixel units). */
int size;		/* box size (pixel units). */
int cval;		/* color map index. */

{
   static int vlb [2];
   static int vrb [2];
   static int vrt [2];
   static int vlt [2];
   int xl, xr;
   int yb, yt;

   color (cval);

   xl = xw - size/2;
   xr = xl + size - 1;
   yb = yw - size/2;
   yt = yb + size - 1;

   vlb [0] = xl;
   vlb [1] = yb;
   vrb [0] = xr;
   vrb [1] = yb;
   vrt [0] = xr;
   vrt [1] = yt;
   vlt [0] = xl;
   vlt [1] = yt;

   bgnpolygon ();
      v2i (vlb);
      v2i (vrb);
      v2i (vrt);
      v2i (vlt);
   endpolygon ();

   return (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * polyfill : fill polygon.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
polyfill (poly, nval, cmap)

int poly [][2];
int nval;
int cmap;

{
   int i;

   color (cmap);

   bgnpolygon ();
      for (i=0; i<nval; i++)  v2i (poly [i]);
      v2i (poly [0]);
   endpolygon ();

   return (0);
}
