/* glcp_annot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write SMM C/P image annotation to Iris GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger  HAO/NCAR   SMM C/P
 *  3 April 1991
 * 26 April 1991: use onemap color mode.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/gl/gl_color.h"

glcp_annot (lbottom, cval1, cval2, cval3)

int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int cval1;	/* Color level for annotation. */
int cval2;	/* Color level for labels. */
int cval3;	/* Color level for HAO logo. */

{
   int month_day();		/* Function to compute month & day from DOY. */
   char *month_name();		/* Function to look up ASCII month name. */
   static char month [4];
   static char wbuf [10];

   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `Charter-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    */
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "Charter-Black";
   static char afont [32];
   static char bfont [32];
  
   int afx, afy, bfx, bfy;
   int annot_state = 1;
   int charheight = 43;
   int offset = -21;
   int xstart = 2;
   int ystart = 485;
   int xc, yc;			/* height & width for character annotation. */
   int xs, ys;			/* Start position for character annotation. */
   int year, doy;
   int imonth, iday;
   int asize, bsize;

   if (H.ires == 1)			/* Low resolution. */
      {
      ystart = 510;
      xs = xstart;
      ys = ystart;
      yc = charheight;
      strcpy (afont, font1);
      strcpy (bfont, font4);
      asize =  7;
      bsize = 10;
      afx =  6;
      afy =  7;
      bfx = 14;
      bfy = 15;
      }
   else
   if (H.ires == 0)			/* High resolution. */
      {
      ystart = 1022;
      xs = xstart * 2;
      ys = ystart * 2;
      yc = charheight * 2;
      strcpy (afont, font4);
      strcpy (bfont, font4);
      asize = 10;
      bsize = 19;
      afx = 14;
      afy = 15;
      bfx = 28;
      bfy = 30;
      }
   ys = ystart - afy;

   gl_setfont (bfont, bsize);		/* Charter-Black font. */

   /*** Year and Day-Of-Year: ***/

   (void) sprintf (wbuf, "%d: %03d", H.year, H.doy);
   wbuf [7] = '\0';
   gl_wstr (wbuf, xs, ys - 1*bfy, cval1);

   /*** Time: ***/

   (void) sprintf (wbuf, "%02d:%02d:%02d\0", H.hour, H.minute, H.second);
   gl_wstr (wbuf, xs, ys - 2*afy - 3*bfy, cval1);

   /*** Day and month. ***/

   year = H.year + 1900;
   doy = H.doy;
   (void) month_day (year, doy, &imonth, &iday);
   strcpy (month, month_name (imonth));
   sprintf (wbuf, "%d %s ", iday, month);
   gl_wstr (wbuf, xs, ys - 4*afy - 5*bfy, cval1);
   sprintf (wbuf, "%d", year);
   gl_wstr (wbuf, xs, ys - 4*afy - 6*bfy, cval1);

   /*** Spectral filter: ***/

   switch (H.ifw)
      {
      case 2: sprintf (wbuf,"OPEN  "); break;
      case 6: sprintf (wbuf,"Fe-XIV"); break;
      case 0: sprintf (wbuf,"RED   "); break;
      case 1: sprintf (wbuf,"Halpha"); break;
      case 3: sprintf (wbuf,"BLUE  "); break;
      case 4: sprintf (wbuf,"WIDEBD"); break;
      case 5: sprintf (wbuf,"GREEN "); break;
      default: sprintf (wbuf,"******"); break;
      }
   if (H.iglstw == 1)  sprintf (wbuf,"%s  ", H.glc);
   gl_wstr (wbuf, xs, ys - 6*afy - 8*bfy, cval1);

   /*** Polaroid filter: ***/

   switch (H.ipw)
      {
      case 0: sprintf (wbuf,"CLEAR "); break;
      case 1: sprintf (wbuf,"+60deg"); break;
      case 2: sprintf (wbuf,"-60deg"); break;
      case 3: sprintf (wbuf," 0 deg"); break;
      default: sprintf (wbuf,"***"); break;
      }
   gl_wstr (wbuf, xs, ys - 8*afy - 10*bfy, cval1);

   /*** Sector mirror: ***/

   switch (H.ism)
      {
      case 5:  sprintf (wbuf, "N   "); break;
      case 4:  sprintf (wbuf, "NE  "); break;
      case 3:  sprintf (wbuf, "E   "); break;
      case 2:  sprintf (wbuf, "SE  "); break;
      case 1:  sprintf (wbuf, "S   "); break;
      case 0:  sprintf (wbuf, "SW  "); break;
      case 7:  sprintf (wbuf, "W   "); break;
      case 6:  sprintf (wbuf, "NW  "); break;
      case 8:  sprintf (wbuf, "NESW"); break;
      default: sprintf (wbuf, "****"); break;
      }
   gl_wstr (wbuf, xs, ys - 10*afy - 12*bfy, cval1);

   /*** Exposure (1.024 second units) : ***/

   (void) sprintf (wbuf, "%-4u", H.exp);
   gl_wstr (wbuf, xs, ys - 12*afy - 14*bfy, cval1);

   /*** Resolution : ***/

   if( H.ires == 0 ) (void) sprintf (wbuf, "HIGH");
   else (void) sprintf (wbuf, "LOW ");
   gl_wstr (wbuf, xs, ys - 14*afy - 16*bfy, cval1);

   /*** S/C Roll : ***/

   if (H.icm == 1)
      {
      if (annot_state) 
         {
         glcp_label (1, lbottom, cval2, cval3);
         }
      (void) sprintf (wbuf, "%-4d", scroll);
      gl_setfont (bfont, bsize);	/* Charter-Black font. */
      gl_wstr (wbuf, xs, ys - 16*afy - 18*bfy, cval1);

      annot_state = 1;
      }

   /*** Calibration Mirror: ***/

   else
      {
      switch (H.icm)
         {
         case 0: (void) sprintf (wbuf, "MOVE"); break;
         case 1: (void) sprintf (wbuf, "OUT "); break;
         case 2: (void) sprintf (wbuf, "IN  "); break;
         case 3: (void) sprintf (wbuf, "BAD "); break;
         default: (void) sprintf (wbuf, "****"); break;
         }
      if (annot_state) 
         {
         glcp_label (1, lbottom, cval2, cval3);
         }
      gl_setfont (bfont, bsize);	/* Charter-Black font. */
      gl_wstr (wbuf, xs, ys - 16*afy - 18*bfy, cval1);
      annot_state = 0;
      }

   /*** Image count (frame number) : ***/

   (void) sprintf (wbuf, "%-4u", H.frame);
   gl_wstr (wbuf, xs, ys - 18*afy - 20*bfy, cval1);

   return (0);
}
