/* glcp_label.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label SMM C/P image in SGI/IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  3 April 1991
 * 26 April 1991: use onemap color mode.
 * 10 July 1991: Write border around bottom portion AFTER color bar is written.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <fmclient.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/gl/img_scale.ext"
#include "/usr/people/smm/lib/src/gl/gl_color.h"

glcp_label (lfull, lbottom, cval2, cval3)

int lfull;	/* Flag for full redrawing of labels. */
int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int cval2;	/* Color level for labels. */
int cval3;	/* Color level for HAO logo. */

{
   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `Charter-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    */
   long afh, bfh, cfh;
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "Charter-Black";
   static char afont [32];			/* Small font. */
   static char bfont [32];			/* Medium font. */
   static char cfont [32];			/* Large font. */
  
   static char logo [] = " HAO/SMM   CORONAGRAPH / POLARIMETER ";
   static short *cbar;		/* Color bar array. */
   int charheight = 43;
   int asize, bsize, csize;		/* Font point sizes. */
   int afx, afy, bfx, bfy, cfx, cfy;	/* Font x/y sizes. */
   int afx15;
   int xstr;
   int xstart = 1;
   int ystart = 501;
   int begrow, begcol, endrow, endcol, ix, iy;
   int step;
   int colormax = 255;			/* Maximum color value. */
   int barlength = 512, barwidth = 12;	/* color bar dimensions. */
   int ppc = 2;		/* # pixels per color in color bar. */
   int cby0, cby1;	/* color bar y-axis locations. */
   int startcol;
   int xc, yc;
   int xs, ys;
   int x1 = 0, x2 = 62, x3 = 511;
   int y1 = 0, y2 = 62, y3 = 511;
   int xwl, xil, xwr;
   int ywb, yib, ywt;

   charheight = 43;

   if (H.ires == 1)		/* Low resolution. */
      {
      ystart = 501;
      xs = xstart;
      ys = ystart;
      strcpy (afont, font1);
      asize = 7;
      strcpy (bfont, font4);
      bsize = 10;
      strcpy (cfont, font4);
      csize = 12;
      afx =  6;   afy =  7;
      bfx = 14;   bfy = 15;
      cfx = 16;   cfy = 17;
      xwl = x1;  xil = x2;  xwr = x3;
      ywb = y1;  yib = y2;  ywt = y3;
      startcol = 1;
      ppc = 2;
      barlength = 512;
      barwidth = 12;
      }
   else
   if (H.ires == 0)		/* High resolution. */
      {
      ystart = 1008;
      xs = xstart * 2;
      ys = ystart * 2;
      strcpy (afont, font4);
      asize = 10;
      strcpy (bfont, font4);
      bsize = 19;
      strcpy (cfont, font4);
      csize = 25;
      afx = 14;   afy = 15;
      bfx = 28;   bfy = 30;
      cfx = 38;   cfy = 40;
      xwl = x1 * 2;  xil = x2 * 2;  xwr = x3 * 2;
      ywb = y1 * 2;  yib = y2 * 2;  ywt = y3 * 2;
      ppc = 4;
      startcol = 2;
      barlength = 1024;
      barwidth = 24;
      }
   ys = ywt - afy;
   yc = (int) (bfy * 3);
   begcol = 0;
   endcol = begcol + barlength - 1;

   if (lfull)
      {		/*** Draw box around image window. ***/

      gl_wvec (xwl, yib, xwr, yib, cval2);
      gl_wvec (xwr, yib, xwr, ywt, cval2);
      gl_wvec (xwr, ywt, xwl, ywt, cval2);
      gl_wvec (xwl, ywt, xwl, yib, cval2);
      gl_wvec (xil, yib, xil, ywt, cval2);

      /*** Write annotation labels into frame buffer. ***/

      afh = gl_setfont (afont, asize);	/* Small font & size. */
      gl_wstr ("YEAR:DOY",   xs, ys,                   cval2);
      gl_wstr ("TIME",       xs, ys -  2*afy -  2*bfy, cval2);
      gl_wstr ("DATE",       xs, ys -  4*afy -  4*bfy, cval2);
      gl_wstr ("FILTER",     xs, ys -  6*afy -  7*bfy, cval2);
      gl_wstr ("POLAROID",   xs, ys -  8*afy -  9*bfy, cval2);
      gl_wstr ("SECTOR",     xs, ys - 10*afy - 11*bfy, cval2);
      gl_wstr ("EXPOSURE",   xs, ys - 12*afy - 13*bfy, cval2);
      gl_wstr ("RESOLUTION", xs, ys - 14*afy - 15*bfy, cval2);
      }

   gl_setfont (afont, asize);	/* Small font & size. */
   if (H.icm == 2)
      gl_wstr ("CAL MIRROR", xs, ys - 16*afy - 17*bfy, cval2);
   else
      gl_wstr ("ROLL      ", xs, ys - 16*afy - 17*bfy, cval2);

   if (lfull)
      {
      gl_wstr ("FRAME  ", xs, ys - 18*afy - 19*bfy, cval2);

      /*** Write HAO logo & cval2 bar only if 'lbottom = 0'. ***/

      if (lbottom == 1)  return (0);

      /*** Add annotation for the color bar ***/

      gl_setfont (afont, asize);	/* Small font & size. */
      cby0 = 1 * nres;
      afx15 = (int) ((afx * 1.5) + 0.5);
      gl_wstr ("0"  , (startcol+1),               cby0, cval2);
      gl_wstr ("64" , (startcol + 64 *ppc-afx),   cby0, cval2);
      gl_wstr ("128", (startcol + 128*ppc-afx15), cby0, cval2);
      gl_wstr ("192", (startcol + 192*ppc-afx15), cby0, cval2);
      gl_wstr ("255", (endcol-afx*3),             cby0, cval2);

      /*** Add tick marks to color bar ***/

      cby0 += afy + 2;
      cby1 = cby0 + 4 * nres;
      gl_wvec (startcol,         cby0, (startcol),         cby1, cval2);
      gl_wvec (startcol+32*ppc , cby0, (startcol+32*ppc),  cby1, cval2);
      gl_wvec (startcol+64*ppc , cby0, (startcol+64*ppc),  cby1, cval2);
      gl_wvec (startcol+96*ppc , cby0, (startcol+96*ppc),  cby1, cval2);
      gl_wvec (startcol+128*ppc, cby0, (startcol+128*ppc), cby1, cval2);
      gl_wvec (startcol+160*ppc, cby0, (startcol+160*ppc), cby1, cval2);
      gl_wvec (startcol+192*ppc, cby0, (startcol+192*ppc), cby1, cval2);
      gl_wvec (startcol+224*ppc, cby0, (startcol+224*ppc), cby1, cval2);
      gl_wvec (startcol+255*ppc, cby0, (startcol+255*ppc), cby1, cval2);

      /*** Draw the color bar. ***/

      barlength = (colormax + 1) * ppc;
      cbar = (short *) malloc (barlength * barwidth * sizeof (short));

      for (iy = 0; iy < barwidth; iy++)
         for (step = 0; step <= colormax; step++)
            for (ix = 0; ix < ppc; ix++)
               *(cbar + (step*ppc+ix) + (iy*barlength)) =
                (short) (step & 0377) + cmapoff;

      begrow = cby1 + 1;
      endrow = begrow + barwidth - 1;
      rectwrite (begcol, begrow, endcol, endrow, cbar);
      free (cbar);

      /*** Write HAO ID. ***/

      cfh = gl_setfont (cfont, csize);	/* Large font & size. */
      xstr = fmgetstrwidth (cfh, logo);
      gl_wstr (logo, xil + ((xwr - xil + 1) - (xstr)) / 2,
               endrow + 5*nres, cval3);

     /*** Write border around lower portion of window. ***/

      gl_wvec (xwl, yib, xwl, ywb, cval2);
      gl_wvec (xwl, ywb, xwr, ywb, cval2);
      gl_wvec (xwr, ywb, xwr, yib, cval2);
      }
   return (0);
}
