/* grnw.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a Grinnell byte image on the SGI IRIS screen
 * using GL library software.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 June 1991: use onemap color mode.
 * 18 September 1991: add mapoff argument to glimg_view function.
 *  8 December 1992: use mouse instead of keyboard; add forward/backward seq.
 * 29 January 1993: adapt from 'imgw.c'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/img_scale.h"
#define MAXIMG 250

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
   FILE *LIST;
   short val;

   static char answer [2];		/* Response to scaling question. */
   static char flist [MAXIMG][80];
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Name of file containing list of images. */
   static char nampgm [80];	/* Program file name. */
   static char pixtyp [12];	/* Pixel type: byte, short, or float. */
   static char cdx [8], cdy [8];	/* Character strings for dx & dy. */
   char *rindex();			/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int wbox ();			/* Image transfer function. */
   int dx, dy;			/* Image dimensions. */
   int findex = 0;		/* File index. */
   int i;				/* Index. */
   int ierr;			/* Error status. */
   int lastimg;			/* Index of last image. */
   int mapoff = 512;		/* Use color map #2. */
   int nbypx;			/* # bytes/pixel. */
   int hdrsiz = 0;			/* # bytes to skip at BOF. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */
   int xll, yll;		/* lower left corner window coordinates. */
   int xur, yur;		/* upper right corner window coordinates. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);
   strcpy (pixtyp, "byte");		/* Default pixel type: byte. */
   hdrsiz = 0;				/* Default hdrsiz: 0. */
   dx = dy = 512;			/* Default dimensions: 512x512. */

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [filename | @filename | ^filename]\n",
               argv [0]);

      fprintf (stderr, "Enter image file name: ");
      fscanf  (stdin,  "%s", namimg);
      }

   /*** Single argument: must be filename. ***/

   else
   if (argc == 2)
      {
      strcpy (namimg, argv[1]);
      }

   /*** Default. ***/

   else
      {
      fprintf (stderr, "usage: %s [filename | @namelist | ^namelist]\n",
               argv [0]);
      exit (-1);
      }

   if (pixtyp [0] == 'b')  nbypx = 1;
   else
   if (pixtyp [0] == 's')  nbypx = 2;
   else
   if (pixtyp [0] == 'f')  nbypx = 4;
   else
      {
      fprintf (stderr, "Invalid pixel type; must be byte, short, or float.\n");
      exit (-2);
      }
/*
 *   fprintf (stderr, "nbypx: %d   dx: %d  dy: %d  namimg: %s\n",
 *            nbypx, dx, dy, namimg);
 */

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^')
      {
      if (namimg [0] == '@')
         seq = 1;
      else
      if (namimg [0] == '^')
	 seq = 2;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }

      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
         }
      lastimg = i;
      fclose (LIST);
      }

   /*** Initialize GL software. ***/

   if (dx <= 1270)  xur = 1270; else xur = 1279;
   if (dy <=  995)  yur =  995; else yur = 1023;
   xll = xur - dx + 1;
   yll = yur - dy + 1;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n");
      exit (-1);
      }

   foreground ();
   fminit ();				/* Initialize SGI font manager. */
   prefposition (xll, xur, yll, yur);
   winopen (nampgm);
   gl_wclear (BLACK);

   doublebuffer ();		/* Use double buffer mode. */
   gconfig ();

   /*** Queue devices. ***/

   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left mouse'   to go forward.\n");
      fprintf (stderr, "click `middle mouse' to go backward.\n");
      }
   fprintf (stderr, "type  `ESC'          to quit.\n");

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]);

   /*** Invoke function to transfer image into frame buffer FB. ***/

   if ((ierr = glimg_view (namimg, hdrsiz, nbypx, dx, dy, mapoff)) < 0)
      {
      fprintf (stderr, "glimg_view error: %d\n", ierr);
      exit (-4);
      }

   swapbuffers ();

   fprintf (stderr, "%s > ", namimg);

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               fprintf (stderr, "\n");
               goto loop;
               }
            break;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               fprintf (stderr, "\n");
               goto loop;
               }
            break;

         case REDRAW:
            glimg_view (namimg, hdrsiz, nbypx, dx, dy, mapoff);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

}
