/* imgwi.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a byte image on the SGI IRIS screen using GL library software.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 June 1991: use onemap color mode.
 * 18 September 1991: add mapoff argument to glimg_view function.
 *                    use map #0 for SGI Indigo.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/img_scale.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
static char answer [2];		/* Response to scaling question. */
static char namimg [80];	/* Image file name. */
static char nampgm [80];	/* Program file name. */
static char pixtyp [12];	/* Pixel type: byte, short, or float. */
static char cdx [8], cdy [8];	/* Character strings for dx & dy. */
char *rindex();			/* Index function. */
char *it;			/* File name pointer. */
char *parg;			/* Argument pointer. */

int wbox();			/* Image transfer function. */
int dx, dy;			/* Image dimensions. */
int i;				/* Index. */
int ierr;			/* Error status. */
int mapoff = 0;			/* Use color map #0. */
int nbypx;			/* # bytes/pixel. */
int xll, yll;			/* lower left corner window coordinates. */
int xur, yur;			/* upper right corner window coordinates. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Usage: %s [pixel_type xdim ydim] [filename]\n",
               argv [0]);

      fprintf (stderr, "Enter pixel type [byte, short, float]: ");
      fscanf  (stdin,  "%s", pixtyp);

      fprintf (stderr, "Enter x-dimension,y-dimension: ");
      fscanf  (stdin,  "%d,%d", &dx, &dy);

      fprintf (stderr, "Enter image file name: ");
      fscanf  (stdin,  "%s", namimg);
      }

   /*** Single argument: must be filename. ***/

   else
   if (argc == 2)
      {
      fprintf (stderr, "Usage: %s [pixel_type xdim ydim] [filename]\n",
               argv [0]);

      strcpy (namimg, argv[1]);

      fprintf (stderr, "Enter pixel type [byte, short, float]: ");
      fscanf  (stdin,  "%s", pixtyp);

      fprintf (stderr, "Enter x-dimension,y-dimension: ");
      fscanf  (stdin,  "%d,%d", &dx, &dy);
      }

   /*** four arguments. ***/

   else
   if (argc == 5)
      {
      strcpy (pixtyp, argv [1]);
      strcpy (cdx, argv [2]);
      strcpy (cdy, argv [3]);
      sscanf (cdx, "%d", &dx);
      sscanf (cdy, "%d", &dy);
      strcpy (namimg, argv[4]);
      }

   /*** Default. ***/

   else
      {
      fprintf (stderr, "Usage: %s [pixel_type xdim ydim] [filename]\n",
               argv [0]);
      exit (-1);
      }

   if (pixtyp [0] == 'b')  nbypx = 1;
   else
   if (pixtyp [0] == 's')  nbypx = 2;
   else
   if (pixtyp [0] == 'f')  nbypx = 4;
   else
      {
      fprintf (stderr, "Invalid pixel type; must be byte, short, or float.\n");
      exit (-2);
      }
/*
 *   fprintf (stderr, "nbypx: %d   dx: %d  dy: %d  namimg: %s\n",
 *            nbypx, dx, dy, namimg);
 */

   /*** Initialize GL software. ***/

   if (dx <= 1270)  xur = 1270; else xur = 1279;
   if (dy <=  995)  yur =  995; else yur = 1023;
   xll = xur - dx + 1;
   yll = yur - dy + 1;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n");
      exit (-1);
      }

   foreground ();
   fminit ();				/* Initialize SGI font manager. */
   prefposition (xll, xur, yll, yur);
   winopen (nampgm);
   gl_wclear (BLACK);
/*
 *   multimap ();
 *   gconfig ();
 *   setmap (2);
 */

   /*** Display image in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);
   if ((ierr = glimg_view (namimg, nbypx, dx, dy, mapoff)) < 0)
      {
      fprintf (stderr, "imgw error: %d\n", ierr);
      exit (-4);
      }
   fprintf (stderr, "Type any character + <CR> to exit.\n");
   fscanf  (stdin,  "%s", answer);
/*
 *   sleep (15);
 */
   gexit ();
   exit (0);
}
