/* cpw.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an SMM C/P image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 *  7 December 1992: add sequence + forward/backward options.
 *  8 December 1992: use mouse instead of keyboard; add wrap-around option (^).
 * 19 January 1994: modify command line argument processing.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
int DSP = 1;
#define MAXIMG 250

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
   FILE *LIST;
   short val;

   static char flist [MAXIMG][80];
   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Namelist file name. */
   static char answer [2];	/* Response to scaling question. */
   char *rindex ();		/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int glcp_view ();		/* Image display function. */
   int findex = 0;
   int i;			/* Index. */
   int ians;			/* KB response. */
   int ierr;			/* Error status from cpview(). */
   int larg;			/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int lastimg = 0;		/* Last image index value. */
   int pmode = 0;		/* Print mode. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      usage (nampgm) ;
      exit (0);
      }

   if ( argc < 2 ) { usage (nampgm) ; exit (-1) ; }

   while (--argc)
      {
      if (*(parg = *++argv) == '-')		/* Check for flag arguments */
	 {
	 switch (parg [1])
	    {
	    case 's':  lscale = 1 ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmin) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmax) ;
		       break;
            case 'd':  ldisk = 1 ; break ;
            default:   fprintf (stderr," unknown flag %s\n", *argv);
		       usage (nampgm); exit (-2);
            }
         }
      else					/* Get the image filename. */
         {
         if (*parg == '?') { usage (nampgm); exit (-3); }
         strcpy (namimg, parg) ; 
	 }
      }

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^')
      {
      if (namimg [0] == '@')
         seq = 1;
      else
      if (namimg [0] == '^')
	 seq = 2;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
         }
      lastimg = i;
      fclose (LIST);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
   if (seq)
      cphskp (&flist [0][0]);
   else
      cphskp (namimg);

   /*** Initialize GL software. ***/

	/* OGLXXX
	 * noborder not supported -- See Window Manager
	 * noborder ()
	 */

	/* OGLXXX
	 * foreground not supported -- see fork man page
	 * foreground ()
	 */

   if (H.ires == 1)
	/* OGLXXX
	 * prefposition not supported -- See Window Manager
	 * prefposition (760, 1271, 480, 991)
	 */
   /* 512x512 window in upper right corner.*/
   else
	/* OGLXXX
	 * prefposition not supported -- See Window Manager
	 * prefposition (256, 1279,   0, 1023)
	 */
   /*1024x1024 window: upper right corner.*/
   fminit ();			/* Initialize SGI font manager. */

	/* OGLXXX
	 * winopen not supported -- See Window Manager
	 * winopen (argv [0])
	 */

   /* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

	/* OGLXXX doublebuffer: use GLX_DOUBLEBUFFER in attriblist */
   glxChooseVisual(*dpy, screen, *attriblist);		/* Use double buffer mode. */
	/* OGLXXX
	 * gconfig not supported:
	 * 	collect glxChooseVisual attributes into one list for visual selection
	 * gconfig ()
	 */
   /*DELETED*/;

   /*** Declare devices. ***/

	/* OGLXXX
	 * qdevice not supported -- See Events
	 * qdevice (WINCLOSE)
	 */

	/* OGLXXX
	 * qdevice not supported -- See Events
	 * qdevice (WINQUIT)
	 */

	/* OGLXXX
	 * qdevice not supported -- See Events
	 * qdevice (ESCKEY)
	 */
	/* OGLXXX

	 * qdevice not supported -- See Events
	 * qdevice (LEFTMOUSE)
	 */

	/* OGLXXX
	 * qdevice not supported -- See Events
	 * qdevice (MIDDLEMOUSE)
	 */

	/* OGLXXX
	 * qdevice not supported -- See Events
	 * qdevice (RIGHTMOUSE)
	 */

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left  mouse' to go forward.\n");
      fprintf (stderr, "click `right mouse' to go backward.\n");
      }
   fprintf (stderr,    "type  `ESC'         to quit.\n");

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]);

   /*** Display image on IRIS screen in GL window. ***/

   if ((ierr = glcp_view (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
	/* OGLXXX
	 * gexit not supported -- See Window Manager
	 * 	There's bound to be something you have to do!
	 * gexit ()
	 */
      exit (-2);
      }

	/* OGLXXX swapbuffers: replace display and window */
   glXSwapBuffers(*display, window);

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
	/* OGLXXX
	 * qtest not supported -- See Events
	 * qtest ()
	 */
   while (/*DELETED*/)		/* Check device queue. */
      {
	/* OGLXXX
	 * qread not supported -- See Events
	 * qread (&val)
	 */
      switch (/*DELETED*/)	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.\n");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               goto loop;
               }
            break;

         case RIGHTMOUSE:
            if (val == 1 && seq > 0)	/* Right mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.\n");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               goto loop;
               }
            break;

         case MIDDLEMOUSE:		/* Middle mouse button is down. */
         case REDRAW:
            glcp_view (namimg, linit, pmode);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
	/* OGLXXX
	 * gexit not supported -- See Window Manager
	 * 	There's bound to be something you have to do!
	 * gexit ()
	 */
               exit (0);
               }
         }
	/* OGLXXX
	 * qreset not supported -- See Events
	 * qreset ()
	 */
      }
   }

quit:
	/* OGLXXX
	 * gexit not supported -- See Window Manager
	 * 	There's bound to be something you have to do!
	 * gexit ()
	 */
   exit (0);
}



usage (nampgm)

char nampgm [] ;
{
   fprintf (stderr, "Usage: %s [-s oldmin oldmax] [-d] cp_image | @cp_list | ^cp_list\n", nampgm);
   fprintf (stderr, "   -s     Override default intensity scaling.\n");
   fprintf (stderr, "          oldmin, oldmax: min -> max image values;\n");
   fprintf (stderr, "                          will be displayed as 0 -> 250.\n");
   fprintf (stderr, "   -d     Eliminate color bar & solar disk outline.\n");
   fprintf (stderr, "cp_image  SMM C/P image filename.\n");
   fprintf (stderr, "cp_list   File containing list of SMM C/P image filenames.\n");
   fprintf (stderr, "@         Single-step sequencing through list of images.\n");
   fprintf (stderr, "          Stops at beginning or end of sequence.\n");
   fprintf (stderr, "^         Single-step sequencing through list of images.\n");
   fprintf (stderr, "          Adds capability of wrapping around begin/end.\n");
   return (0);
}
