/* glcp_label.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label SMM C/P image in SGI/IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  3 April 1991
 * 26 April 1991: use onemap color mode.
 * 10 July 1991: Write border around bottom portion AFTER color bar is written.
 * 6 August 1992: modify format to accomodate difference images.
 * 16 November 1993: minor cosmetic adjustments.
 * 19 May 1995: Change CharterBT-Black font to Times-Bold.
 *              Use Fixed-Bold font for pixval.
 *              Use H.idtype to set contents of pixval.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/img_scale.ext"
#include "/usr/people/smm/lib/include/gl_color.h"

glcp_label (lfull, lbottom, cval2, cval3, cval4)

int lfull ;	/* Flag for full redrawing of labels. */
int lbottom ;	/* Bottom flag: =0 normal ; =1 no information below image. */
int cval2 ;	/* Color level for labels. */
int cval3 ;	/* Color level for HAO logo. */
int cval4 ;	/* Color level for miscellaneous labels. */

{
	/* `Screen' font sizes:
    *  7 =>  6 x  7
    * 11,13,15 =>  8 x 15
    *
    * `Screen-Bold' font sizes:
    * 13,15 =>  9 x 15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11 x 13
    * 10 => 14 x 15
    * 12 => 18 x 20
    * 14 => 20 x 22
    * 18 => 27 x 29
    * 24 => 37 x 38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12 x 13
    * 10 => 14 x 15
    * 12 => 16 x 17
    * 14 => 22 x 23
    * 19 => 28 x 30
    * 25 => 38 x 40
    *
    * `Times-Roman'		`Times-Bold' font sizes:
    *  8 => 11 x 12		8 => 11 x 13
    *  9 => 12 x 15		9 => 13 x 15
    * 10 => 13 x 16		10 => 14 x 16
    * 11 => 15 x 18		11 => 16 x 18
    * 12 => 15 x 18		12 => 17 x 19
    * 14 => 19 x 20		14 => 19 x 19
    * 16 => 21 x 23		16 => 22 x 22
    * 18 => 24 x 28		18 => 27 x 28
    * 20 => 26 x 31		20 => 26 x 31
    * 24 => 35 x 36		24 => 35 x 37
    *
    * `Fixed' font sizes:	`Fixed-Bold' font sizes:
    *  6 =>  5 x  8		 6 =>  4 x  8
    *  7 =>  6 x  9		 7 =>  5 x  9
    *  8 =>  6 x 11		 8 =>  6 x 11
    *  9 =>  6 x 12		 9 => 13 x 15
    * 10 =>  7 x 13		10 => 14 x 16
    * 11 =>  9 x 15		11 => 16 x 18
    * 12 => 10 x 16		12 => 17 x 19
    * 14 =>  9 x 19		14 => 19 x 19
    * 16 => 11 x 21		16 => 22 x 22
    * 18 => 12 x 24		18 => 27 x 28
    * 20 => 13 x 26		20 => 26 x 31
    * 24 => 16 x 32		24 => 35 x 37
    */
	extern fmfonthandle gl_setfont () ;
	fmfonthandle afh, bfh, cfh, dfh ;
	static char font1 [] = "Screen" ;
	static char font2 [] = "Screen-Bold" ;
	static char font3 [] = "Helvetica-Bold" ;
	static char font4 [] = "CharterBT-Black" ;
	static char font5 [] = "Times" ;
	static char font6 [] = "Times-Bold" ;
	static char font7 [] = "Fixed" ;
	static char font8 [] = "Fixed-Bold" ;
	static char afont [32] ;			/* Font : Labels. */
	static char bfont [32] ;			/* Font : Parameters. */
	static char cfont [32] ;			/* Font : Brightness Units. */
	static char dfont [32] ;			/* Font : C/P Logo. */
	static char efont [32] ;			/* Font : C/P Logo. */

	static char logo [] = " HAO / SMM   CORONAGRAPH / POLARIMETER " ;
	static char pixval [12] ;			/* Pixel value mode string. */
	static short *cbar ;				/* Color bar array. */

	int asize, bsize, csize, dsize, esize ;	/* Font point sizes. */
	int afx, afy, bfx, bfy, cfx, cfy, dfx, dfy ;	/* Font x/y sizes. */
	int efx, efy ;
	int efx15 ;

	int i ;
	int lxstr ;
	int pxstr ;
	int xempty ;
	int xlogo ;
	int ylogo ;
	int begrow, endrow, ix, iy ;
	int step ;

	int colormax = 255 ;			/* Maximum color value. */
	int barwidth = 512, barheight = 12 ;	/* color bar dimensions. */
	int ppc = 2 ;			/* # pixels per color in color bar. */
	int cbx0, cbx1 ;			/* color bar x-axis start/stop. */
	int cby0, cby1 ;			/* color bar y-axis start/stop. */

	int xs, ys ;
	int x1 = 0, x2 = 62, x3 = 511 ;
	int y1 = 0, y2 = 62, y3 = 511 ;
	int xwl, xil, xwr ;
	int ywb, yib, ywt ;

	if (H.ires == 1)			/* Low resolution. */
	{
		strcpy (afont, font1) ;
		strcpy (bfont, font3) ;
		strcpy (cfont, font7) ;
		strcpy (dfont, font6) ;
		strcpy (efont, font7) ;
		asize =  7 ;
		bsize = 10 ;
		csize = 12 ;
		dsize = 11 ;
		esize =  7 ;
		afx =  6 ;   
		afy =  7 ;
		bfx = 14 ;   
		bfy = 15 ;
		cfx = 10 ;   
		cfy = 16 ;
		dfx = 16 ;   
		dfy = 18 ;
		efx =  6 ;   
		efy =  9 ;
		xs = 2 ;
		xwl = x1 ;  
		xil = x2 ;  
		xwr = x3 ;
		ywb = y1 ;  
		yib = y2 ;  
		ywt = y3 ;
		ppc = 2 ;
		barwidth  = 512 ;
		barheight =  12 ;
	}
	else
		if (H.ires == 0)			/* High resolution. */
		{
			strcpy (afont, font6) ;
			strcpy (bfont, font6) ;
			strcpy (cfont, font7) ;
			strcpy (dfont, font6) ;
			strcpy (efont, font7) ;
			asize =  9 ;
			bsize = 18 ;
			csize = 24 ;
			dsize = 24 ;
			esize =  9 ;
			afx = 13 ;   
			afy = 15 ;
			bfx = 27 ;   
			bfy = 28 ;
			cfx = 16 ;   
			cfy = 32 ;
			dfx = 35 ;   
			dfy = 37 ;
			efx =  6 ;   
			efy = 12 ;
			xs = 4 ;
			xwl = x1 * 2 ;  
			xil = x2 * 2 ;  
			xwr = x3 * 2 + 1 ;
			ywb = y1 * 2 ;  
			yib = y2 * 2 ;  
			ywt = y3 * 2 + 1 ;
			ppc = 4 ;
			barwidth  = 1024 ;
			barheight =   24 ;
		}
	ys = ywt - afy - 2 ;
	cbx0 = 0 ;
	cbx1 = cbx0 + barwidth - 1 ;

	/*
 *   printf ("xwl: %3d  ywt: %3d  xwr: %3d  ywb: %3d\n", xwl, ywt, xwr, ywb) ;
 *   printf ("xil: %3d  yib: %3d\n", xil, yib) ;
 */

	if (lfull)
	{
		/*** Write annotation labels into frame buffer. ***/

		afh = gl_setfont (afont, asize) ;	/* Small font & size. */
		gl_wstr ("DATE    ",   xs, ys,                   cval2) ;
		gl_wstr ("YEAR:DOY",   xs, ys -  2*afy -  3*bfy, cval2) ;
		gl_wstr ("TIME",       xs, ys -  4*afy -  6*bfy, cval2) ;

		gl_wstr ("FILTER",     xs, ys -  6*afy - 10*bfy, cval2) ;
		gl_wstr ("POLAROID",   xs, ys -  7*afy - 12*bfy, cval2) ;
		gl_wstr ("SECTOR",     xs, ys -  8*afy - 14*bfy, cval2) ;
		gl_wstr ("EXPOSURE",   xs, ys -  9*afy - 16*bfy, cval2) ;
		gl_wstr ("RESOLUTION", xs, ys - 10*afy - 18*bfy, cval2) ;
	}

	gl_setfont (afont, asize) ;	/* Small font & size. */
	if (H.icm == 2)
		gl_wstr ("CAL MIRROR", xs, ys - 11*afy - 20*bfy, cval2) ;
	else
		gl_wstr ("ROLL      ", xs, ys - 11*afy - 20*bfy, cval2) ;

	if (lfull)
	{
		gl_wstr ("FRAME  ", xs, ys - 12*afy - 22*bfy, cval2) ;

		/*** Write HAO logo & cval2 bar only if 'lbottom = 0'. ***/

		if (lbottom == 1)  return (0) ;

		/*** Add annotation for the color bar ***/

		gl_setfont (efont, esize) ;	/* Small font & size. */
		cby0 = 1 * nres ;
		efx15 = (int) ((efx * 1.5) + 0.5) ;
		gl_wstr ("0"  , (cbx0 + nres),              cby0, cval2) ;
		gl_wstr ("64" , (cbx0 +  64 * ppc - efx),   cby0, cval2) ;
		gl_wstr ("128", (cbx0 + 128 * ppc - efx15), cby0, cval2) ;
		gl_wstr ("192", (cbx0 + 192 * ppc - efx15), cby0, cval2) ;
		gl_wstr ("255", (cbx1 - efx * 3),           cby0, cval2) ;

		/*** Add tick marks to color bar ***/

		cby0 += afy + 2 ;
		cby1 = cby0 + 4 * nres ;
		gl_wvec (cbx0 + nres,      cby0, (cbx0 + nres),      cby1, cval2) ;
		gl_wvec (cbx0 +  32 * ppc, cby0, (cbx0 +  32 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 +  64 * ppc, cby0, (cbx0 +  64 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 +  96 * ppc, cby0, (cbx0 +  96 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 + 128 * ppc, cby0, (cbx0 + 128 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 + 160 * ppc, cby0, (cbx0 + 160 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 + 192 * ppc, cby0, (cbx0 + 192 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 + 224 * ppc, cby0, (cbx0 + 224 * ppc), cby1, cval2) ;
		gl_wvec (cbx0 + 255 * ppc, cby0, (cbx0 + 255 * ppc), cby1, cval2) ;

		/*** Draw the color bar. ***/

		barwidth = (colormax + 1) * ppc ;
		cbar = (short *) malloc (barwidth * barheight * sizeof (short)) ;

		for (iy = 0 ; iy < barheight ; iy++)
			for (step = 0 ; step <= colormax ; step++)
				for (ix = 0 ; ix < ppc ; ix++)
					*(cbar + (step * ppc + ix) + (iy * barwidth)) =
					    (short) (step & 0377) + cmapoff ;

		begrow = cby1 + 1 ;
		endrow = begrow + barheight - 1 ;
		rectwrite (cbx0, begrow, cbx1, endrow, cbar) ;
		free (cbar) ;

		/*** Determine Brightness Units. ***/

		for (i = 0 ; i < 12 ; i++)  pixval [i] = '\0' ;

		if (H.idtype == 1)
			strcpy (pixval, "VIDICON:DN ") ;
		else
			if (H.idtype == 2)
				strcpy (pixval, "EQUIV.SEC. ") ;
			else
				if (H.idtype == 3)
				{
					if ( H.k == 1)
					{
						if ( H.str &&  H.f &&  H.vig)  strcpy (pixval, "         ") ;
						if (!H.str && !H.f && !H.vig)  strcpy (pixval, "S+F*V    ") ;

						if ( H.str &&  H.f && !H.vig)  strcpy (pixval, "    V    ") ;
						if ( H.str && !H.f && !H.vig)  strcpy (pixval, "  F*V    ") ;
						if (!H.str &&  H.f && !H.vig)  strcpy (pixval, "S  *V    ") ;

						if (!H.str &&  H.f &&  H.vig)  strcpy (pixval, "S        ") ;
						if (!H.str && !H.f &&  H.vig)  strcpy (pixval, "S+F      ") ;

						if ( H.str && !H.f &&  H.vig)  strcpy (pixval, "  F      ") ;
					}
					else
					{
						if ( H.str &&  H.f &&  H.vig)  strcpy (pixval, "     K   ") ;
						if (!H.str && !H.f && !H.vig)  strcpy (pixval, "S+(F+K)*V") ;

						if ( H.str &&  H.f && !H.vig)  strcpy (pixval, "     K *V") ;
						if ( H.str && !H.f && !H.vig)  strcpy (pixval, "  (F+K)*V") ;
						if (!H.str &&  H.f && !H.vig)  strcpy (pixval, "S+   K *V") ;

						if (!H.str &&  H.f &&  H.vig)  strcpy (pixval, "S+   K   ") ;
						if (!H.str && !H.f &&  H.vig)  strcpy (pixval, "S+(F+K)  ") ;

						if ( H.str && !H.f &&  H.vig)  strcpy (pixval, "  (F+K)  ") ;
					}
				}
				else
				{
					switch (H.idtype)
					{
					case 4:  
						strcpy (pixval, "elec/cm2   ") ;	
						break ;
					case 5:  
						if (H.vig)  strcpy (pixval, "(B1-B2)    ") ;
						else strcpy (pixval, "(B1-B2)*V  ") ;
						break ;
					case 6:  
						if (H.vig)  strcpy (pixval, "B[k]       ") ;
						else strcpy (pixval, "B[k]*V     ") ;
						break ;
					case 7:  
						strcpy (pixval, "(B1-B2)/Bk ") ;	
						break ;
					case 8:  
						strcpy (pixval, "(B1-B2)/B2 ") ;	
						break ;
					case 9:  
						strcpy (pixval, "pol.frac.  ") ;	
						break ;
					case 10: 
						strcpy (pixval, "pB         ") ;	
						break ;
					case 11: 
						if (H.vig)  strcpy (pixval, "B[r+t]     ") ;
						else strcpy (pixval, "B[r+t]*V   ") ;
						break ;
					case 12: 
						if (H.vig)  strcpy (pixval, "B[t]       ") ;
						else strcpy (pixval, "B[r]*V     ") ;
						break ;
					case 13: 
						if (H.vig)  strcpy (pixval, "B[r]       ") ;
						else strcpy (pixval, "B[r]*V     ") ;
						break ;
					case 14: 
						strcpy (pixval, "B[f]*V+B[s]") ;	
						break ;
					case 15: 
						strcpy (pixval, "B1 / B2    ") ;	
						break ;
					case 16: 
						strcpy (pixval, "           ") ;	
						break ;
					}
				}

		/*** Write Brightness Units & C/P Logo. ***/

		cfh = gl_setfont (cfont, csize) ;		/* Large font & size. */
		dfh = gl_setfont (dfont, dsize) ;		/* Logo  font & size. */
		pxstr = fmgetstrwidth (cfh, pixval) ;
		lxstr = fmgetstrwidth (dfh, logo) ;

		xempty = (xwr - xwl) - pxstr - lxstr ;
		if (xempty > 0)  xlogo = xs + pxstr + xempty/2 ;
		ylogo = endrow + 7*nres ;

		gl_wstr (pixval, xs,    ylogo, cval4) ;
		gl_wstr (logo,   xlogo, ylogo, cval3) ;

		/*** Draw box around image window. ***/

		gl_wvec (xwr, ywb, xwr, ywt, cval2) ;	/* Vert : Right  edge window. */
		gl_wvec (xwr, ywt, xwl, ywt, cval2) ;	/* Horiz: Top    edge window. */
		gl_wvec (xwl, ywt, xwl, ywb, cval2) ;	/* Vert : Left   edge window. */
		gl_wvec (xwl, ywb, xwr, ywb, cval2) ;	/* Horiz: Bottom edge window. */
		gl_wvec (xil, yib, xil, ywt, cval2) ;	/* Vert : Left   edge image.  */
		gl_wvec (xwl, yib, xwr, yib, cval2) ;	/* Horiz: bottom edge image.  */
		gl_wvec (xwl, ys-4*afy-10*bfy, xil, ys-4*afy-10*bfy, cval2) ;

		/*** Write border around lower portion of window. ***/

		/*      gl_wvec (xwl, yib, xwl, ywb, cval2) ;*/	/* Vert : Left   edge logo.   */
		/*      gl_wvec (xwl, ywb, xwr, ywb, cval2) ;*/	/* Horiz: Bottom edge window. */
		/*      gl_wvec (xwr, ywb, xwr, yib, cval2) ;*/	/* Vert : Right  edge logo.   */
	}
	return (0) ;
}
