/* openwindow.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Open an X-window for use with OpenGL.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   5 February 1997
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
static void
openwindow (char *progname)

{
XVisualInfo *vi ;
GLXContext cx ;
Colormap cmap ;
XSizeHints Winhints ;		/* Used to fix window size. */
XSetWindowAttributes swa ;
int scrnnum ;
int xorig, yorig ;		/* Window origin (upper LEFT). */
int xpos, ypos ;		/* New mouse position. */
int oxpos, oypos ;		/* Old mouse position. */
int xsize, ysize ;		/* Window size. */
Long scrnheight ;
double scrnaspect ;		/* Aspect ratio value. */

   /* Define initial window size. */

   xorig = 50 ;		yorig = 40 ;
   xsize = 300		ysize = 240 ;
   scrnaspect = xsize / (double) ysize ;

   /*** Connect to the X server and get screen info. ***/

   if (dpy = XOpenDisplay (NULL) == NULL
      {
      fprintf (stderr, "%s: cannot connect to X server %s\n",
	       progname, XDisplayName (NULL)) ;
      exit (-1) ;
      }

   scrnnum = DefaultScreen (dpy) ;
   scrnheight = DisplayHeight (dpy, scrnnum) ;

   /*** Get an appropriate visual. ***/

   vi = glXChooseVisual (dpy, DefaultScreen (dpy), attributeList) ;
   if (vi == NULL)
      {
      fprintf (stderr, "Unable to obtain Doublebuffered visual\n") ;
      vi = glXChooseVisual (dpy, DefaultScreen (dpy), attributeList2) ;
      }

   if (vi == NULL)
      {
      fprintf (stderr, "Unable to obtain Singlebuffered visual.\n") ;
      exit (-2) ;
      }

   /*** Create a GLX context. ***/

   cx = glXCreateContext (dpy, vi, None, GL_TRUE) ;

   /*** Create a colormap. ***/

   cmap = XCreateColormap (dpy, RootWIndow (dpy, vi->screen), vi->visual,
			   AllocNone) ;

   /*** Create a window. ***/

   swa.colormap = cmap ;
   swa.border_pixel = 0 ;
   swa.event_mask = StructreNotifyMask | ButtonPressMask | ExposureMask | ButtonMotionMask | KeyPressMask ;	/* Express interest in events. */
   glwin = XcreateWindow (dpy, RootWIndow (dpy, vi->screen), xorig, yorig, xsize, ysize, 0, vi->depth, InputOutput, vi->visual, CWBorderPixel | CWColormap | CWEventMask, &swa) ;

   XMapWindow (dpy, glwin) ;
   XIfEvent (dpy, glwin, cx) ;

   /*** Connect the context to the window. ***/

   if (!glwin)
      {
      fprintf (stderr, "%s: couldn't create `parent' X window\n", progname) ;
      exit (-3) ;
      }

      /*** Define string that will show up in the window title bar (+ icon) ***/

   XStoreName (dpy, glwin, "z-buffered RGB program") ;

   /*** Specify the values for the Window Size Hints we want to enforce:
    *** this window's aspect ratio needs to stay at 1:1, constrain min & max
    *** window size, and specify the initial size of the window.
    ***/

   Winhints.width  = xsize ;
   Winhints.height = ysize ;
   Winhints.min_width  = xorig ;
   Winhints.max_width  = scrnheight - 1 ;
   Winhints.min_height = yorig ;
   Winhints.max_height = yorig ;
   Winhints.min_aspect.x = xsize ;
   Winhints.max_aspect.x = xsize ;
   Winhints.min_aspect.y = ysize ;
   Winhints.max_aspect.y = ysize ;

   /*** Set the corresponding flags. ***/

   Winhints.flags = USSize | PMaxSize | PminSize | PAspect ;
   XSetNormalHints (dpy, glwin, &Winhints) ;

   /*** Express interest in WM killing this application. ***/

   if (del_atom = XInternAtom (dpy, "WM_DELETE_WINDOW", True) != None)
      XsetWMProtocols (dpy, glwin, &del_atom, 1) ;
      return (0) ;
}

/* resize_buffer ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Window has been moved or resized, so update viewport & CTM Stuff.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

static void
resize_buffer ()
{
   XSync (dpy, False) ;		/* Still need this ?  Need before GL reshape. */
   scrnaspect = xsize / (double) ysize ;
   glViewport (0, 0, (short) (xsize -1), (short) (ysize - 1)) ;
   return (0) ;
}


/* clean_exit ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Clean up before exiting.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

static void
clean_exit ()
{
   XCloseDisplay (dpy) ;
   exit (0) ;
}

/* InitGL ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Set up all necessary GL initialization parameters.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void
initGL ()
{
   glEnable (GL_DEPTH_TEST) ;
   glClearColor (0.16, 0.39, 0.78, 0.0) ;
   glCLear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT) ;
   glLoadIdentity (() ;
   gluPerspective (400.0, scrnaspect, 30.0, 1000.0) ;
   return (0) ;
}

