/* planet.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Displays a planet going around the sun.
 * Demonstrates how to use composite modeling transformations to draw
 * translated and rotated models.
 * Pressing left/right/up/down arrow keys alters the rotation of the planet
 * around the sun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * OpenGL Porting Guide example.
 * 6 February 1997
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <GL/gl.h>
#include <GL/glu.h>
#include "aux.h"

static int year = 0 ;
static int day  = 0 ;

void
dayAdd (void)
{
   day = (day + 10) % 360 ;
}

void
daySubtract (void)
{
   day = (day - 10) % 360 ;
}

void yearAdd (void)
{
   year = (year + 5) % 360 ;
}

void
yearSubtract (void)
{
   year = (year - 5) % 360 ;
}

void
display (void)
{
   glClear (GL_COLOR_BUFFER_BIT) ;
   glColor3f (1.0, 1.0, 1.0) ;
   glPushMatrix () ;

   /* Draw sun. */

   auxWireSphere (1.0) ;

   /* Draw smaller planet. */

   glRotatef ((GLfloat) year, 0.0, 1.0, 0.0) ;
   glTranslatef (2.0, 0.0, 0.0) ;
   glRotatef ((GLfloat) day, 0.0, 1.0, 0.0) ;
   auxWireSphere (0.2) ;
   glPopMatrix () ;
   glFlush () ;
}

void
myinit (void)
{
   glShadeModel (GL_FLAT) ;
}

void
myReshape (GLsizei w, GLsizei h)
{
   glViewport (0, 0, w, h) ;
   glmatrixMode (GL_PROJECTION) ;
   glLoadIentity () ;
   gluPerspective (60.0, (GLfloat) w / (GLfloat) h, 1.0, 20.0) ;
   glMatrixMod (GL_MODELVIEW) ;
   glLoadIdentity () ;
   glTranslatef (0.0, 0.0, -5.0) ;
}

/* Main Loop
 * Open window with initial window dize, title bar, RGBA display mode,
 * and handle input events.
 */

main (int argc, *char [] argv)
{
   auxInitDisplayMode (AUX_SINGLE | AUX_RGBA) ;
   auxInitPosition (0, 0, 500, 500) ;
   auxInitWindow (argv [0]) ;

   myinit () ;

   auxKeyFUnc (AUX_LEFT,  yearSubtract) ;
   auxKeyFunc (AUX_RIGHT, yearAdd) ;
   auxKeyFunc (AUX_UP,    dayAdd) ;
   auxKeyFUnc (AUX_DOWN,  daySubtract) ;
   auxReshapeFunc (myReshape) ;
   auxMainLoop (display) ;

}
