/*   
 * overlay.c:   another overlay bitplane usage demo program... 
 */

#include <stdio.h>
#include <gl/gl.h>
#include <gl/device.h>

main () {
    Colorindex heat;
    int rampup();
    int i, xpos = 1013, ypos = 1013;
    float xspd = 0.0, yspd = 0.0, yaccel = -1.0, yacc = -.4, yreflect = -0.6;
    long xmaxscrn, ymaxscrn;         /* maximum size of screen in x and y    */
    long xscrnsize;              /* size of screen in x used to set globals  */


    xscrnsize = getgdesc(GD_XPMAX);                   /* get/set screen size */
    if (xscrnsize == 1280) {
         xmaxscrn = 1279;
         ymaxscrn = 1023;
    } else if (xscrnsize ==1024) {
         xmaxscrn = 1023;
         ymaxscrn = 767;
    } else {
        fprintf(stderr, "Something's EXTREMELY wrong:  ");
        fprintf(stderr, "xscrnsize=%d\n", xscrnsize);
        exit(-1) ;
    }


    foreground();
    prefposition(0, xmaxscrn, 0, ymaxscrn);
    winopen ("overlay");
    qdevice (ESCKEY);
    qdevice (RIGHTMOUSE);
    doublebuffer ();
    overlay (2);
    gconfig ();
    get_cmap();

    drawmode (OVERDRAW);               /* get into the overlay bitplanes */
    mapcolor (1, 255, 0, 0);           
    mapcolor (2, 0, 255, 0);
    mapcolor (3, 0, 255, 255);
    color (1);
    rectfi (200, 200, 300, 300);       /* and draw some rectangles for   */
    color (2);                         /* ball to roll underneath        */
    rectfi (500, 500, 600, 600);
    color (3);
    rectfi (800, 800, 900, 900);
    rectfi (850, 500, 950, 600);
    rectfi (750, 400, 850, 500);
    rectfi (650, 300, 750, 400);
    drawmode (NORMALDRAW);
    mapcolor (0, 0, 0, 0);
    mapcolor (1, 255, 255, 0);
    mapcolor (10, 255, 255, 255);
    rampup(12, 82, 255, 255, 0, 255, 0, 0);
    setbell(1);

    while (!getbutton (ESCKEY)) {
	for (i = 0; i < 1013 && !getbutton(ESCKEY); i = i + 3) {   
	    color (BLUE);            /* roll the ball up and to the right */
	    clear (); 
	    if ((i==210) || (i==510) || (i==810)) 
		 ringbell();                   /* ring the bell everytime */
	    if ((i>250) && (i<550))            /* the ball gets to the    */
		color (2);                     /* next rectangle, as well */
	    else if ((i>550) && (i<850))       /* change the ball's color */
		color (5);
	    else if (i>850)
		color (10);
            else
		color (1);
	    circfi (i, i, 10);
	    swapbuffers ();
	}
	yspd = 0.0;
	for (heat=82, ypos=1013; ypos>=5 && !getbutton(ESCKEY); ypos+=yspd) {
	    color (BLUE);             /* drop the ball back to the bottom */
	    clear ();                 
	    color (heat--);                    /* change the ball's color */
	    yspd += yacc;                      /* as it falls             */
	    circfi (1013, ypos, 10);
	    swapbuffers ();
        }
	yspd = -60.0;
	for (xpos=1013, ypos=10; xpos>=-0&& !getbutton(ESCKEY); xpos -= 5) {
	    if (ypos <= 10)          /* roll the ball back to the beginning */
		 yspd *= yreflect;             /* and keep updating its'   */
	    color (BLUE);                      /* bounce-ability per frame */
	    clear (); 
	    color (1);
	    ypos += yspd;
	    yspd += yaccel;
	    circfi (xpos, ypos, 10);
	    swapbuffers ();
        }
    }

    drawmode (OVERDRAW);                /* clean up the overlay bitplanes */
    color (0);
    clear();
    drawmode(NORMALDRAW);
    restore_cmap();                     /* and restore the color map      */
    greset();
    gexit ();
}

/*
 * make a color ramp                             
 * makes an interpolated ramp from the 1st arguement's index to the 2nd. 
 * 3rd and 4th are red's low and hi indices (5&6 green's, 7&8 are blue's) 
 */
#define round(n)     ((int) (n + 0.5))

rampup(first_lutv,last_lutv,minR,maxR,minG,maxG,minB,maxB)
unsigned short first_lutv, last_lutv,          /* start & end ramp values */
	       minR, maxR, minG, maxG, minB, maxB;      /* lo/hi rgb vals */
{
     unsigned short len_red, len_green, len_blue, /* length of each color */
                    i;                     /* counter for number of steps */

     short red, gre, blu;                                   /* lut values */
     float rdx, gdx, bdx,                      /* sizes of rgb increments */
           r, g, b,                             /* a position on the ramp */
           steps;                    /* # of steps along the ramp @ which */
                                    /* intensity assignments will be made */

     steps = (float) (last_lutv-first_lutv + 1);/*determine length of ramp*/

     len_red   = (maxR - minR);                /* determine length of red */
     len_green = (maxG - minG);              /* determine length of green */
     len_blue  = (maxB - minB);               /* determine length of blue */

     rdx = (float) len_red   / steps;                     /* compuke step */
     gdx = (float) len_green / steps;                     /* sizes of r g */
     bdx = (float) len_blue  / steps;                     /* and b values */
     r = minR;                                         /* assign starting */
     g = minG;                                        /* indices for each */
     b = minB;                                        /* color value      */

     for (i = first_lutv; i <= last_lutv; i++) {
 	  red = (short) round(r);                            /* round off */
	  gre = (short) round(g);                            /* given r g */
	  blu = (short) round(b);                            /* b value   */
     	  mapcolor(i, red, gre, blu);      /* assign next color map index */
	  r += rdx;                                          /* increment */
	  g += gdx;                                          /* color in- */
	  b += bdx;                                          /* dices     */
     }    
}         

#define MAXCOLI 255

static short CarrayR[MAXCOLI+1], CarrayG[MAXCOLI+1], CarrayB[MAXCOLI+1];
unsigned short index;

get_cmap() {

    short rcomp, gcomp, bcomp;

    for (index=0; index<=MAXCOLI; index++) {
	getmcolor(index,&rcomp, &gcomp, &bcomp);
	CarrayR[index] = rcomp;
	CarrayG[index] = gcomp;
	CarrayB[index] = bcomp;
    }
}

restore_cmap() {

    for (index=0; index<=MAXCOLI; index++) 
	mapcolor(index,CarrayR[index], CarrayG[index], CarrayB[index]);
}
