/* fits_date.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decode FITS DATE or DATE-OBS parameter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   12 January 1999
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

int
fits_date (fdate, doy, day, month, year)

char fdate [] ;
int *doy ;
int *day ;
int *month ;
int *year ;

{
   char cf1, cf2, cf3 ;
   static char sf1 [80], sf2 [80], sf3 [80] ;
   static char month_name [12][4] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
				     "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"} ;

   int dash ;
   int dot ;
   int fslash ;
   int idash1,  idash2 ;
   int i ;
   int idot1,   idot2 ;
   int if1, if2, if3 ;
   int islash1, islash2 ;
   int ierr ;
   int iday, imonth, iyear ;
   int lendate ;

   ierr = 0 ;

   /*** Day and month. ***/

   *doy   = 0 ;
   *day   = 0 ;
   *month = 0 ;
   *year  = 0 ;

   lendate = strlen (fdate) ;
/*   printf ("lendate: %d   fdate:|%s|\n", lendate, fdate) ; */

   dot = dash = fslash = 0 ;
   idash1  = idash2  = -1 ;
   idot1   = idot2   = -1 ;
   islash1 = islash2 = -1 ;
   if1 = 0 ;
   if2 = 0 ;
   if3 = 0 ;

   for (i = 0; i < strlen (fdate); i++)
      {
      if (fdate [i] == '.')
	 {
	 dot++    ;
         if (dot == 1)  idot1 = i ;
         if (dot == 2)  idot2 = i;
	 }
      if (fdate [i] == '-')
	 {
	 dash++   ;
         if (dash == 1)  idash1 = i ;
         if (dash == 2)  idash2 = i;
	 }
      if (fdate [i] == '/')
	 {
	 fslash++ ;
         if (fslash == 1)  islash1 = i ;
         if (fslash == 2)  islash2 = i ;
	 }
      }

   if (dot    == 2)		/* Dot delimiter */
      {
      strncpy (sf1, &fdate [0],       idot1) ;
      strncpy (sf2, &fdate [idot1+1], idot2 - idot1 - 1) ;
      strncpy (sf3, &fdate [idot2+1], strlen (fdate) - idot2 - 1) ;
      cf1 = sf1 [0] ;
      cf2 = sf2 [0] ;
      cf3 = sf3 [0] ;
      }
   else
   if (dash   == 2)		/* Dash Delimiter */
      {
      strncpy (sf1, &fdate [0],        idash1) ;
      strncpy (sf2, &fdate [idash1+1], idash2 - idash1 - 1) ;
      strncpy (sf3, &fdate [idash2+1], strlen (fdate) - idash2 - 1) ;
      cf1 = sf1 [0] ;
      cf2 = sf2 [0] ;
      cf3 = sf3 [0] ;
      }
   else
   if (fslash == 2)		/* Slash delimiter */
      {
      strncpy (sf1, &fdate [0],         islash1) ;
      strncpy (sf2, &fdate [islash1+1], islash2 - islash1 - 1) ;
      strncpy (sf3, &fdate [islash2+1], strlen (fdate) - islash2 - 1) ;
      cf1 = sf1 [0] ;
      cf2 = sf2 [0] ;
      cf3 = sf3 [0] ;
      }
   else			/* Unrecognized Date Format. */
      {
      ierr = -1 ;
      *day = *month = *year = 0 ;
      fprintf (stderr, "unrecognized date format (%s)\n", fdate) ;
      }

   /* Analyze each of the three fields. */

   if (cf1 >= '0' && cf1 <= '9')	/* 1st field is numeric. */
      {
      sscanf (sf1, "%d", &if1) ;
      }
   else					/* 1st field is a string. */
      {
      for (i = 0 ; i < strlen (sf1) ; i++)
         sf1 [i] = (char) (toupper ((int) sf1 [i])) ;
      for (i = 0 ; i < 12 ; i++)
         if (strcmp (sf1, month_name [i]) == 0)  if1 = i + 1 ;
      }

   if (cf2 >= '0' && cf2 <= '9')	/* 2nd field is numeric. */
      {
      sscanf (sf2, "%d", &if2) ;
      }
   else					/* 2nd field is a string. */
      {
      for (i = 0 ; i < strlen (sf2) ; i++)
         sf2 [i] = (char) (toupper ((int) sf2 [i])) ;
      for (i = 0 ; i < 12 ; i++)
         if (strcmp (sf2, month_name [i]) == 0)  if2 = i + 1 ;
      }

   if (cf3 >= '0' && cf3 <= '9')	/* 3rd field is numeric. */
      {
      sscanf (sf3, "%d", &if3) ;
      }
   else					/* 3rd field is numeric. */
      {
      for (i = 0 ; i < strlen (sf3) ; i++)
         sf3 [i] = (char) (toupper ((int) sf3 [i])) ;
      for (i = 0 ; i < 12 ; i++)
         if (strcmp (sf3, month_name [i]) == 0)  if3 = i + 1 ;
      }

   printf ("if1: %d  if2: %d   if3: %d\n", if1, if2, if3) ;

      *year  = if3 ;
      *month = if2 ;
      *day   = if1 ;

   if (if1 > 31)
      {
      if (if2 > 12)
         {
         *year  = if1 ;
         *month = if3 ;
         *day   = if2 ;
         }
      }
   else
   if (if2 > 31)
      {
      if (if3 > 12)
         {
         *year  = if2 ;
         *month = if1 ;
         *day   = if3 ;
         }
      }
   else
   if (if3 > 31)
      {
      if (if1 > 12)
         {
         *year  = if3 ;
         *month = if2 ;
         *day   = if1 ;
         }
      }
   else
      {
      *year  = if3 ;
      *month = if2 ;
      *day   = if1 ;
      }

   return (ierr) ;
}
