/* fitsrfspr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a FITS image on the SGI IRIS screen using GL library software.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   16 March 1994
 * 18 April 1994: add dir2 argument to glfit_view ().
 * 27 April 1994: Save image as a Sun rasterfile image.
 *  8 July  1994: set xwmax & ywmax to screen size minus border size.
 * 10 August 1994: add '=' prefix for filelist (=> seq=3).
 * 19 November 1998: version for Spartan images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/img_scale.h"
#include "/home/stanger/lib/include/cp_color.h"
#include "/home/stanger/lib/include/fitslib.h"
#define MAXIMG 250
float minfac = 0.2, maxfac = 1.2 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc ;
char *argv[] ;

{
   extern char *rindex () ;		/* Index function. */
   FILE *LIST ;

   static char answer [2] ;		/* Response to scaling question. */
   static char dir2 [4] = "ru" ;	/* Scan direction: rd or ru. */
   static char flist [MAXIMG][80] ;
   static char namsrf [80] ;
   static char namimg [80] ;		/* Image file name. */
   static char namlst [80] ;		/* File containing list of images. */
   static char nampgm [80] ;		/* Program file name. */
   static char pixtyp [12] ;		/* Pixel type: byte, short, or float. */
   static char cdx [8], cdy [8] ;	/* Character strings for dx & dy. */
   static unsigned char red [256], green [256], blue [256] ;

   unsigned char  *rawbuf ;
   unsigned short *glbuf  ;
   unsigned short sval, smin, smax ;

   char *it ;			/* File name pointer. */
   char *parg ;			/* Argument pointer. */

   short val ;

   int dx = 0, dy = 0 ;		/* Image dimensions. */
   int findex = 0 ;		/* File index. */
   int gwid = -1 ;		/* GL window ID. */
   int i ;			/* Index. */
   int ierr ;			/* Error status. */
   int ix, iy, oy ;
   int ixdim, iydim ;
   int lastimg ;		/* Index of last image. */
   int gloff = 0 ;
   int mapoff = 512 ;		/* color index offset (logical map #2). */
   int nave = 1 ;		/* # pixels to average in each dimension. */
   int nxave = 1 ;		/* # pixels to average in x-dimension. */
   int nyave = 1 ;		/* # pixels to average in y-dimension. */
   int nbypx ;			/* # bytes/pixel. */
   int onaxis1, onaxis2 ;	/* Previous naxis1 & naxis2. */
   int pmode = 1 ;		/* Print mode: 0 silent, 1 verbose. */
   int seq = 0 ;		/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */
   int xll, yll ;		/* lower left corner window coordinates. */
   int xur, yur ;		/* upper right corner window coordinates. */
   int xwmax, ywmax ;		/* Screen size. */
   int xwsize, ywsize ;		/* Window size. */
   int xbsize, ybsize ;		/* Anotation border size. */
   int xmax, ymax ;
   int x1, x2, y1, y2 ;
   int oixdim, oiydim ;

   float rotang = 0.0 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Set border size. */

   xbsize = 128 ;
   ybsize =  64 ;

   /*** Get maximum window size. ***/

   xwmax = getgdesc (GD_XPMAX) - xbsize ;
   ywmax = getgdesc (GD_YPMAX) - ybsize ;

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   strcpy (nampgm, argv [0]) ;

   /*** Request information if not enough arguments are provided. ***/

   if ( argc < 2 ) { usage (nampgm) ; exit (-1) ; }

   while (--argc)
      {
      if (*(parg = *++argv) == '-')		/* Check for flag arguments */
	 {
	 switch (parg [1])
	    {
	    case 's':  lscale = 1 ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmin) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmax) ;
		       break ;
            case 'd':  --argc ; parg = *++argv ; sscanf (parg, "%s", dir2) ;
                       break ;
	    case 'm':
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &minfac) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &maxfac) ;
		       break ;
            default:   fprintf (stderr," unknown flag %s\n", *argv) ;
		       usage (nampgm) ; exit (-2) ;
            }
         }
      else					/* Get the image filename. */
         {
         if (*parg == '?') { usage (nampgm) ; exit (-3) ; }
         strcpy (namimg, parg) ; 
	 }
      }

   /*** Set intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      oldmin =  (int) foldmin ;    oldmax  = (int) foldmax ;
      newmin =  (int) fnewmin ;    newmax  = (int) fnewmax ;
      clipmin = (int) fclipmin ;   clipmax = (int) fclipmax ;
      }
        

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^' || namimg [0] == '=')
      {
      if (namimg [0] == '@')
         seq = 1 ;
      else
      if (namimg [0] == '^')
	 seq = 2 ;
      else
      if (namimg [0] == '=')
	 seq = 3 ;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst) ;
         exit (-2) ;
         }
      i = -1 ;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++ ;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n") ;
	    exit (-1) ;
	    }
         strcpy (&flist [i][0], namimg) ;
         }
      lastimg = i ;
      fclose (LIST) ;
      }

   if (seq)
      strcpy (namimg, &flist [0][0]) ;

   foreground () ;
   fminit () ;				/* Initialize SGI font manager. */

/*   doublebuffer () ; */		/* Use double buffer mode. */
/*   gconfig () ; */

loop:
/*   printf ("findex: %d\n", findex) ; */

   if (seq)
      strcpy (namimg, &flist [findex][0]) ;

   rdhdu (namimg) ;

   rotang = crota1 ;		/* Rotation angle. */
   if (rotang != 0.0)
      {
      x1 = crpix1 ;
      x2 = naxis1 - 1 - crpix1 ;
      y1 = crpix2 ;
      y2 = naxis2 - 1 - crpix2 ;
      if (x1 < x2) xmax = x2 ; else xmax = x1 ;
      if (y1 < y2) ymax = y2 ; else ymax = y1 ;
      ixdim = (xmax + 1) * 2 ;
      iydim = (ymax + 1) * 2 ;
      if (xmax < ymax) ixdim = iydim ; else iydim = ixdim ;
      fprintf (stdout, "fitw> crota1: %7.2f  xmax/ymax: %4d %4d\n",
			crota1, xmax, ymax) ;
      fprintf (stdout, "fitw> ixdim/iydim: %4d %4d\n", ixdim, iydim) ;
      }
   else
      {
      ixdim = naxis1 ;
      iydim = naxis2 ;
      }

   /*** Initialize GL software. ***/

   if (naxis1 != onaxis1 || naxis2 != onaxis2)
      {
      dx = ixdim ;
      dy = iydim ;

      /*** Is image too large for screen ? ***/

      if (ixdim > xwmax || iydim > ywmax)
	 {
	 if (ixdim > xwmax)
	    nxave = (ixdim / xwmax) + 1 ;
	 else
	 if (iydim > ywmax)
	    nyave = (iydim / ywmax) + 1 ;

         if (nxave > nyave)
	    nave = nxave ;
	 else
	    nave = nyave ;
         dx = ixdim / nave ;         
         dy = iydim / nave ;         
	 printf ("ixdim/2: %4d %4d  dx/dy: %4d %4d  nave: %d\n",
		 ixdim, iydim, dx, dy, nave) ;
	 }
      else
	 nave = 1 ;

      xwsize = dx + xbsize ;
      ywsize = dy + ybsize ;

      if (dx <= 1270)  xur = 1270 ; else xur = 1279 ;
      if (dy <=  995)  yur =  995 ; else yur = 1023 ;

      xll = xur - xwsize + 1 ;
      yll = yur - ywsize + 1 ;
      if (xll < 0 || yll < 0)
         {
         fprintf (stderr, "Image too large for screen\n") ;
         exit (-1) ;
         }
      if (gwid > 0)
         winclose (gwid) ;
      prefposition (xll, xur, yll, yur) ;
      gwid = winopen (nampgm) ;
      gl_wclear (cp_black) ;

      /*** Queue devices. ***/

      qdevice (WINCLOSE) ;
      qdevice (WINQUIT) ;
      qdevice (ESCKEY) ;
      qdevice (LEFTMOUSE) ;
      qdevice (MIDDLEMOUSE) ;
      }

   oixdim = ixdim ;
   oiydim = iydim ;

   /*** Invoke function to transfer image into frame buffer FB. ***/

   if ((ierr = glfit_view (namimg, dir2, dx, dy, nave,
			   xwsize, ywsize, xbsize, ybsize, mapoff, pmode)) < 0)
      {
      fprintf (stderr, "glfit_view error: %d\n", ierr) ;
      exit (-4) ;
      }


   rawbuf = (unsigned char  *) malloc (xwsize * ywsize) ;
   glbuf  = (unsigned short *) malloc (xwsize * ywsize * sizeof (short)) ;
   rectread (0, 0, xwsize - 1, ywsize - 1, glbuf) ;

   fprintf (stdout, "mapoff: %d  gloff: %d\n", mapoff, gloff) ;
   smin = 32767 ;
   smax = 0 ;

   /*** Reverse image top-to-bottom for Sun Rasterfile. ***/
   /*** Subtract colormap offset. ***/

   for (iy = 0 ; iy < ywsize ; iy++)
      {
      oy = ywsize - iy - 1 ;
      for (ix = 0 ; ix < xwsize ; ix++)
         {
	 sval = *(glbuf + ix + iy * xwsize) - mapoff ;
	 if (sval < smin) smin = sval ;
	 if (sval > smax) smax = sval ;
         *(rawbuf + ix + oy * xwsize) =
	    (unsigned char) ((*(glbuf + ix + iy * xwsize) - mapoff) & 0377) ;

/*
 *      if (ix%32 == 0 && iy%32 == 0)
 *         printf ("ix:%3d iy:%3d   glbuf: %3d   rawbuf: %3d\n",
 *	         ix, iy, *(glbuf + ix + iy * xwsize),
 *		 *(rawbuf + ix + oy * xwsize) ) ;
 */

         }
      }

   fprintf (stdout, "smin: %d  smax: %d\n", smin, smax) ;

   /*** Read color table. ***/

   get_scmap (mapoff, 256, red, green, blue) ;

   /*** Create sun rasterfile and write to disk. ***/

   strcpy () ;
   sprintf (namsrf, "%s.ras", namimg) ;
   bytsrf (namsrf, rawbuf, xwsize, ywsize, red, green, blue) ;

   free (rawbuf) ;
   free (glbuf ) ;

/*   swapbuffers () ; */

   fprintf (stderr, "%s > ", namimg) ;

/*
   fprintf (stderr, "\n") ;
   findex++ ;
   if (findex > lastimg)
      {
      sleep (3) ;
      gexit () ;
      exit (0) ;
      }
   else
      {
      goto loop ;
      }
*/

   if (seq == 3)
      {
      findex++ ;
      if (findex > lastimg)
         {
         sleep (3) ;
         gexit () ;
         exit (0) ;
         }
      goto loop ;
      }

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++ ;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.") ;
                     findex = lastimg ;
		     }
                  else
		  if (seq == 2)
		     findex = 0 ;
                  }
               fprintf (stderr, "\n") ;
               goto loop ;
               }
            break ;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex-- ;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.") ;
                     findex = 0 ;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg ;
                  }
               fprintf (stderr, "\n") ;
               goto loop ;
               }
            break ;

         case REDRAW:
            glfit_view (namimg, xwsize, ywsize, xbsize, ybsize, dir2, mapoff) ;
            break ;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit () ;
               exit (0) ;
               }
         }
      qreset () ;
      }
   }

}


usage (nampgm)

char nampgm [] ;

{
   fprintf (stderr, "usage: %s [-m minfac maxfac][-s oldmin oldmax] [-d scan_direction] [@^=]filename\n",
            nampgm) ;
   fprintf (stderr, "defaults: minfac = 0.2\n") ;
   fprintf (stderr, "          maxfac = 1.2\n") ;
   fprintf (stderr, "          oldmin =   0\n") ;
   fprintf (stderr, "          oldmax = 249\n") ;
   fprintf (stderr, "          scan_direction = 'ru'\n") ;

   return (0) ;
}
