/* fitw3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a FITS image on the SGI IRIS screen using GL library software.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   16 March 1994
 * 18 April 1994: add sdir2 argument to glfit_view ().
 *  8 July  1994: set xwmax & ywmax to screen size minus border size.
 *  9 April 1997: use histogram scaling method.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/img_scale.h"
#include "/home/stanger/lib/include/fitslib.h"
#define MAXIMG 500
#include "cpgraf.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc ;
char *argv[] ;

{
   extern char *rindex () ;		/* Index function. */
   FILE *LIST ;

   static char answer [2] ;		/* Response to scaling question. */
   static char sdir2 [4] = "ru" ;	/* Scan direction: rd or ru. */
   static char flist [MAXIMG][80] ;
   static char namimg [80] ;		/* Image file name. */
   static char namlst [80] ;		/* File containing list of images. */
   static char nampgm [80] ;		/* Program file name. */
   static char pixtyp [12] ;		/* Pixel type: byte, short, or float. */
   static char cdx [8], cdy [8] ;	/* Character strings for dx & dy. */

   char *it ;			/* File name pointer. */
   char *parg ;			/* Argument pointer. */

   short val ;

   int dx = 0, dy = 0 ;		/* Image dimensions. */
   int findex = 0 ;		/* File index. */
   int gwid = -1 ;		/* GL window ID. */
   int i ;			/* Index. */
   int ierr ;			/* Error status. */
   int lastimg ;		/* Index of last image. */
   int mapoff = 512 ;		/* color index offset (logical map #2). */
   int nave = 1 ;		/* # pixels to average in each dimension. */
   int nxave = 1 ;		/* # pixels to average in x-dimension. */
   int nyave = 1 ;		/* # pixels to average in y-dimension. */
   int nbypx ;			/* # bytes/pixel. */
   int onaxis1, onaxis2 ;	/* Previous naxis1 & naxis2. */
   int pmode = 0 ;		/* Print mode: 0 silent, 1 verbose. */
   int seq = 0 ;		/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */
   int xll, yll ;		/* lower left corner window coordinates. */
   int xur, yur ;		/* upper right corner window coordinates. */
   int xwmax, ywmax ;		/* Screen size. */
   int xwsize, ywsize ;		/* Window size. */
   int xbsize, ybsize ;		/* Anotation border size. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   opngks_ () ;

   /*** Set border size. ***/

   xbsize = 128 ;
/*   ybsize =  64 ; */
   ybsize =  96 ;

   /*** Get maximum window size. ***/

   xwmax = getgdesc (GD_XPMAX) - xbsize ;
   ywmax = getgdesc (GD_YPMAX) - ybsize ;

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   strcpy (nampgm, argv [0]) ;

   /*** Request information if not enough arguments are provided. ***/

   if ( argc < 2 ) { usage (nampgm) ; exit (-1) ; }

   while (--argc)
      {
      if (*(parg = *++argv) == '-')		/* Check for flag arguments */
	 {
	 switch (parg [1])
	    {
	    case 's':  lscale = 1 ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmin) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmax) ;
		       break ;
            case 'd':  --argc ; parg = *++argv ; sscanf (parg, "%s", sdir2) ;
                       break ;
            default:   fprintf (stderr," unknown flag %s\n", *argv) ;
		       usage (nampgm) ; exit (-2) ;
            }
         }
      else					/* Get the image filename. */
         {
         if (*parg == '?') { usage (nampgm) ; exit (-3) ; }
         strcpy (namimg, parg) ; 
	 }
      }

   /*** Set intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      oldmin =  (int) foldmin ;    oldmax  = (int) foldmax ;
      newmin =  (int) fnewmin ;    newmax  = (int) fnewmax ;
      clipmin = (int) fclipmin ;   clipmax = (int) fclipmax ;
      }
        

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^')
      {
      if (namimg [0] == '@')
         seq = 1 ;
      else
      if (namimg [0] == '^')
	 seq = 2 ;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst) ;
         exit (-2) ;
         }
      i = -1 ;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++ ;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (> %d) in namelist sequence.\n",
		     MAXIMG) ;
	    exit (-1) ;
	    }
         strcpy (&flist [i][0], namimg) ;
         }
      lastimg = i ;
      fclose (LIST) ;
      }

   if (seq)
      strcpy (namimg, &flist [0][0]) ;

   foreground () ;
   fminit () ;				/* Initialize SGI font manager. */

/*   doublebuffer () ; */		/* Use double buffer mode. */
/*   gconfig () ; */

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left  mouse' to go forward.\n");
      fprintf (stderr, "click `right mouse' to go backward.\n");
      }
   fprintf (stderr,    "type  `ESC'         to quit.\n");

loop:
/*   printf ("findex: %d\n", findex) ; */

   if (seq)
      strcpy (namimg, &flist [findex][0]) ;

   rdhdu (namimg) ;

   /*** Initialize GL software. ***/

   if (naxis1 != onaxis1 || naxis2 != onaxis2)
      {
      dx = naxis1 ;
      dy = naxis2 ;

      /*** Is image too large for screen ? ***/

      if (naxis1 > xwmax || naxis2 > ywmax)
	 {
	 if (naxis1 > xwmax)
	    nxave = (naxis1 / xwmax) + 1 ;
	 else
	 if (naxis2 > ywmax)
	    nyave = (naxis2 / ywmax) + 1 ;

         if (nxave > nyave)
	    nave = nxave ;
	 else
	    nave = nyave ;
         dx = naxis1 / nave ;         
         dy = naxis2 / nave ;         
	 }
      else
	 nave = 1 ;

      xwsize = dx + xbsize ;
      ywsize = dy + ybsize ;

      if (pmode)
         printf ("naxis1/2: %4d %4d  dx/dy: %4d %4d  nave: %d\n",
		  naxis1, naxis2, dx, dy, nave) ;
      if (xwsize < 576)  xwsize = 576 ;
      if (ywsize < 576)  ywsize = 576 ;

      if (dx <= 1270)  xur = 1270 ; else xur = 1279 ;
      if (dy <=  995)  yur =  995 ; else yur = 1023 ;

      xll = xur - xwsize + 1 ;
      yll = yur - ywsize + 1 ;
      if (xll < 0 || yll < 0)
         {
         fprintf (stderr, "Image too large for screen\n") ;
         exit (-1) ;
         }
      if (gwid > 0)
         winclose (gwid) ;
      prefposition (xll, xur, yll, yur) ;
      gwid = winopen (nampgm) ;
      gl_wclear (BLACK) ;

      /*** Queue devices. ***/

      qdevice (WINCLOSE) ;
      qdevice (WINQUIT) ;
      qdevice (ESCKEY) ;
      qdevice (LEFTMOUSE) ;
      qdevice (MIDDLEMOUSE) ;
      qdevice (RIGHTMOUSE) ;

      }
   onaxis1 = naxis1 ;
   onaxis2 = naxis2 ;

   /*** Invoke function to transfer image into frame buffer FB. ***/

   if ((ierr = glfit_view (namimg, sdir2, dx, dy, nave,
			   xwsize, ywsize, xbsize, ybsize, mapoff, pmode)) < 0)
      {
      fprintf (stderr, "glfit_view error: %d\n", ierr) ;
      exit (-4) ;
      }

/*   swapbuffers () ; */

   fprintf (stderr, "%s > ", namimg) ;

/*
   fprintf (stderr, "\n") ;
   findex++ ;
   if (findex > lastimg)
      {
      sleep (3) ;
      gexit () ;
      exit (0) ;
      }
   else
      {
      goto loop ;
      }
*/

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++ ;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.") ;
                     findex = lastimg ;
		     }
                  else
		  if (seq == 2)
		     findex = 0 ;
                  }
               fprintf (stderr, "\n") ;
               goto loop ;
               }
            break ;

         case RIGHTMOUSE:
            if (val == 1 && seq > 0)	/* Right mouse button is down. */
               {
               findex-- ;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.") ;
                     findex = 0 ;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg ;
                  }
               fprintf (stderr, "\n") ;
               goto loop ;
               }
            break ;

         case MIDDLEMOUSE:		/* Middle mouse button is down. */
         case REDRAW:
            glfit_view (namimg, sdir2, dx, dy, nave,
			xwsize, ywsize, xbsize, ybsize, mapoff, pmode) ;
            break ;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
	       clsgks_ () ;
               gexit () ;
               exit (0) ;
               }
         }
      qreset () ;
      }
   }

}


usage (nampgm)

char nampgm [] ;

{
   fprintf (stderr, "usage: %s [-s oldmin oldmax] [-d scan_direction] [@^]filename\n",
            nampgm) ;
   fprintf (stderr, "defaults: oldmin =   0\n") ;
   fprintf (stderr, "          oldmax = 249\n") ;
   fprintf (stderr, "          scan_direction = 'ru'\n") ;

   return (0) ;
}
