/* glfit_annot_dp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Annotate FITS images in SGI/GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 March 1994
 * 24 May 1996: add datamin, datamax, dispmin, dispmax to annotation.
 * 17 Dec 1998: add logic for datamin/datamax, dispmin/dispmax (e vs f format).
 * 21 Sep 2000: add TYPE-DP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/gl_color.h"
#include "/home/stanger/lib/include/fitslib.ext"

glfit_annot (xwsize, ywsize, xbsize, ybsize, cval1, cval2, cval3)

int xwsize, ywsize ;		/* Window size. */
int xbsize, ybsize ;		/* Annotation border size. */
int cval1 ;			/* Color level for labels. */
int cval2 ;			/* Color level for parameters. */
int cval3 ;			/* Color level for logo. */

{
   extern fmfonthandle gl_setfont () ;
   fmfonthandle afh, bfh, cfh ;
   static char font1 [] = "Screen" ;
   static char font2 [] = "Screen-Bold" ;
   static char font3 [] = "Helvetica-Bold" ;
   static char font4 [] = "CharterBT-Black" ;
   static char afont [32] ;			/* Small  font. */
   static char bfont [32] ;			/* Medium font. */
   static char cfont [32] ;			/* Large  font. */
   static char month_name [12][4] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
			             "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"} ;
   static char cmonth [4] ;
   static char wlab [24] ;
   static char wpar [24] ;
   static char cdelim [4] ;
   static char blanks [32] = "                        ";
   static char blankx [32] = "xxxxxxxxxxxxxxxxxxxxxxxx";
   short *cbar ;

   static int afx, afy, bfx, bfy, cfx, cfy ;
   int afx15 ;
   int asize, bsize, csize ;

   int barwidth  = 512 ;
   int barheight =  12 ;
   int begcol, endcol ;
   int begrow, endrow ;
   int colormax = 255 ;		/* Maximum color level. */
   int cby0, cby1 ;		/* Color bar y-axis locations. */
   int ppc = 1 ;		/* # pixels per color in color bar. */
   int ix, iy, step ;
   int xbegin ;
   int ybegin ;
   int xb, yb ;
   int doy  ;
   int day  ;
   int month ;
   int year ;
   int hour ;
   int minute ;
   int second ;
   int ldelim ;
   int lendate ;
   int i, fslash, dot, dash ;
   int ifslash1, ifslash2, idot1, idot2, idash1, idash2 ;

   strcpy (afont, font1) ;
   strcpy (bfont, font3) ;
   strcpy (cfont, font4) ;
   asize =  7 ;
   bsize = 11 ;
   csize = 12 ;
   afx   =  6 ;
   afy   =  7 ;
   bfx   = 14 ;
   bfy   = 15 ;
   cfx   = 16 ;
   cfy   = 17 ;

   xbegin = 2 ;
   ybegin = ywsize - 3 ;

   xb = xbegin ;
   yb = ybegin - afy - 1 ;

   if (xwsize >  255)  ppc = 1 ;
   if (xwsize >  511)  ppc = 2 ;
   if (xwsize > 1023)  ppc = 4 ;

   barwidth = (colormax + 1)  * ppc ;
   begcol = xwsize - barwidth ;
   endcol = xwsize - 1 ;

   /*** Write labels. ***/

   afh = gl_setfont (afont, asize) ;	/* Small font/size. */

   gl_wstr ("DATE      ", xb, yb,                   cval1) ;
   gl_wstr ("YEAR:DOY  ", xb, yb -  2*afy -  3*bfy, cval1) ;
   gl_wstr ("TIME      ", xb, yb -  4*afy -  6*bfy, cval1) ;
   gl_wstr ("ORIGIN    ", xb, yb -  6*afy - 10*bfy, cval1) ;
   gl_wstr ("TELESCOPE ", xb, yb -  7*afy - 12*bfy, cval1) ;
   gl_wstr ("INSTRUMENT", xb, yb -  8*afy - 14*bfy, cval1) ;
   gl_wstr ("OBJECT    ", xb, yb -  9*afy - 16*bfy, cval1) ;
   gl_wstr ("TYPE-OBS  ", xb, yb - 10*afy - 18*bfy, cval1) ;
   gl_wstr ("TYPE-DP   ", xb, yb - 11*afy - 20*bfy, cval1) ;
   gl_wstr ("DATA MIN  ", xb, yb - 12*afy - 22*bfy, cval1) ;
   gl_wstr ("DATA MAX  ", xb, yb - 13*afy - 24*bfy, cval1) ;
   gl_wstr ("DISP MIN  ", xb, yb - 14*afy - 26*bfy, cval1) ;
   gl_wstr ("DISP MAX  ", xb, yb - 15*afy - 28*bfy, cval1) ;

   bfh = gl_setfont (bfont, bsize) ;	/* Medium font/size. */

   /*** Day and month. ***/

   month = 0 ;
   lendate = strlen (date_obs) ;
/*   printf ("lendate: %d   date_obs:|%s|\n", lendate, date_obs) ; */

   fslash = dot = dash = 0 ;
   idash1 = idash2 = -1 ;
   for (i = 0; i < strlen (date_obs); i++)
      {
      if (date_obs [i] == '/')  fslash++ ;
      if (date_obs [i] == '.')  dot++    ;
      if (date_obs [i] == '-')
	 {
	 dash++   ;
         if (dash == 1)  idash1 = i ;
         if (dash == 2)  idash2 = i;
	 }
      }

   if (fslash == 2)
      {
      sscanf (date_obs, "%d/%d/%d",
	      &day, &month, &year) ;

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%d/%d/%d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%d/%d/%d",
	      &year, &month, &day) ;
      }
   else
   if (dot    == 2)
      {
      sscanf (date_obs, "%d.%d.%d",
	      &day, &month, &year) ;

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%d.%d.%d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%d.%d.%d",
	      &year, &month, &day) ;
      }
   else
   if (dash   == 2)
      {
      if ((idash2 - idash1) == 3)
         sscanf (date_obs, "%d-%d-%d",
	         &day, &month, &year) ;
      else
	 {
         sscanf (date_obs, "%d-%3s-%d",
	         &day, cmonth, &year) ;
         for (iy = 0; iy < 12; iy++)
	    if (strcmp (&month_name [iy][0], cmonth) == 0)
	       month = iy + 1 ;
         }

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%d-%d-%d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%d-%d-%d",
	      &year, &month, &day) ;
      }
   else
      {
      day = month = year = 0 ;
      fprintf (stderr, "unrecognized date_obs format (%s)\n", date_obs) ;
      }

/*

   if (lendate == 6)
      {
      sscanf (date_obs, "%2d%2d%2d",
	      &day, &month, &year) ;

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%2d%2d%2d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%2d%2d%2d",
	      &year, &month, &day) ;

      }

   else
   if (lendate == 7)
      {
      sscanf (date_obs, "%2d%3s%2d",
	      &day, cdelim, cmonth, cdelim, &year) ;
      for (iy = 0; iy < 12; iy++)
	 if (strcmp (&month_name [iy][0], cmonth) == 0)
	    month = iy + 1 ;
      }

   else
   if (lendate == 8)
      {
      sscanf (date_obs, "%d%1s%d%1s%d",
	      &day, cdelim, &month, cdelim, &year) ;

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%2d%1s%2d%1s%2d",
	      &month, cdelim, &day, cdelim, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%2d%1s%2d%1s%2d",
	      &year, cdelim, &month, cdelim, &day) ;
      }

   else
   if (lendate == 9)
      {
      sscanf (date_obs, "%d%1s%3s%1s%2d",
	      &day, cdelim, cmonth, cdelim, &year) ;
      for (iy = 0; iy < 12; iy++)
	 if (strcmp (&month_name [iy][0], cmonth) == 0)
	    month = iy + 1 ;
      }
   else
      {
      day = month = year = 0 ;
      fprintf (stderr, "unrecognized date_obs format (%s)\n", date_obs) ;
      }
 */

   if (year < 50 )  year += 2000 ;
   else
   if (year < 100)  year += 1900 ;

   sprintf (wpar, "%2d  %3s", day, &month_name [month - 1][0]) ;
   gl_wstr (blanks, xb, yb -  1*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  1*bfy, cval2) ;
   sprintf (wpar, "%4d", year) ;
   gl_wstr (blanks, xb, yb -  2*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  2*bfy, cval2) ;

   /*** Year and Day-Of-Year. ***/

   doy = day_of_year (year, month, day) ;
   sprintf (wpar, "%02d: %03d", year, doy) ;
   gl_wstr (blanks, xb, yb -  2*afy - 4*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  2*afy - 4*bfy, cval2) ;

   /*** Reference image  year & DOY (if applicable). ***/

/*
   (void) sprintf (wpar, "            ") ;
   if (lref)
      {
      if (year != year0)
         (void) sprintf (wpar, "%s%02d:%03d", imop, year0, doy0) ;
      else
      if (year == year0 && doy != doy0)
	 (void) sprintf (wpar, "   %s %03d ", imop, doy0) ;
      else
	 (void) sprintf (wpar, "           ", imop, doy0) ;
       }
   gl_wstr (blanks, xb, yb -  6*afy - 11*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  6*afy - 11*bfy, cval2) ;
 */

   /*** Time. ***/

   if (strlen (time_obs) >= 8)
      {
      sscanf (time_obs, "%2d%1s%2d%1s%2d",
	      &hour, cdelim, &minute, cdelim, &second) ;
      (void) sprintf (wpar, "%02d:%02d:%02d", hour, minute, second) ;
      gl_wstr (blanks, xb,  yb -  4*afy  -  7*bfy, cval2) ;
      gl_wstr (wpar,   xb,  yb -  4*afy  -  7*bfy, cval2) ;
      }

   /*** Reference image time (if applicable). ***/

/*
   sprintf (wpar, "    %s", imiop) ;
   gl_wstr (blanks, xb, yb - 4*afy -  8*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 4*afy -  8*bfy, cval2) ;
   sprintf (wpar, "%02d:%02d:%02d", hour0, minute0, second0) ;
   gl_wstr (blanks, xb, yb -  4*afy -  9*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  4*afy -  9*bfy, cval2) ;
 */

   strcpy (wpar, "        ") ;

   /*** origin. ***/

   strcpy (wpar, origin) ;
   gl_wstr (blanks, xb, yb -  6*afy - 11*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  6*afy - 11*bfy, cval2) ;

   /*** telescope. ***/

   strcpy (wpar, telescop) ;
   gl_wstr (blanks, xb, yb -  7*afy - 13*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  7*afy - 13*bfy, cval2) ;

   /*** instrument. ***/

   strcpy (wpar, instrume) ;
   gl_wstr (blanks, xb, yb -  8*afy - 15*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  8*afy - 15*bfy, cval2) ;

   /*** object ***/

   strcpy (wpar, object) ;
   gl_wstr (blanks, xb, yb -  9*afy - 17*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb -  9*afy - 17*bfy, cval2) ;

   /*** type-obs. ***/

   strcpy (wpar, type_obs) ;
   gl_wstr (blanks, xb, yb - 10*afy - 19*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 10*afy - 19*bfy, cval2) ;

   /*** type-dp. ***/

   strcpy (wpar, type_dp) ;
   gl_wstr (blanks, xb, yb - 11*afy - 21*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 11*afy - 21*bfy, cval2) ;

   /*** Data minimum. ***/

   if ((datamin < 0.01 || datamin > 10000.0) && datamin != 0.0)
      sprintf (wpar, "%9.2e", datamin) ;
   else
      sprintf (wpar, "%9.2f", datamin) ;
   gl_wstr (blanks, xb, yb - 12*afy - 23*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 12*afy - 23*bfy, cval2) ;

   /*** Data maximum. ***/

   if ((datamax < 0.01 || datamax > 10000.0) && datamax != 0.0)
      sprintf (wpar, "%9.2e", datamax) ;
   else
      sprintf (wpar, "%9.2f", datamax) ;
   gl_wstr (blanks, xb, yb - 13*afy - 25*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 13*afy - 25*bfy, cval2) ;

   /*** Display minimum. ***/

   if ((dispmin < 0.01 || dispmin > 10000.0) && dispmin != 0.0)
      sprintf (wpar, "%9.2e", dispmin) ;
   else
      sprintf (wpar, "%9.2f", dispmin) ;
   gl_wstr (blanks, xb, yb - 14*afy - 27*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 14*afy - 27*bfy, cval2) ;

   /*** Display maximum. ***/

   if ((dispmax < 0.01 || dispmax > 10000.0) && dispmax != 0.0)
      sprintf (wpar, "%9.2e", dispmax) ;
   else
      sprintf (wpar, "%9.2f", dispmax) ;
   gl_wstr (blanks, xb, yb - 15*afy - 29*bfy, cval2) ;
   gl_wstr (wpar,   xb, yb - 15*afy - 29*bfy, cval2) ;

   /*** Add annotation for the color bar ***/

   afh = gl_setfont (afont, asize);	/* Small font & size. */
   cby0 = 1;
   afx15 = (int) ((afx * 1.5) + 0.5);
   gl_wstr ("0"  , (begcol),                 cby0, cval1);
   gl_wstr ("64" , (begcol + 64 *ppc-afx),   cby0, cval1);
   gl_wstr ("128", (begcol + 128*ppc-afx15), cby0, cval1);
   gl_wstr ("192", (begcol + 192*ppc-afx15), cby0, cval1);
   gl_wstr ("255", (endcol-afx*3),           cby0, cval1);

   /*** Add tick marks to color bar ***/

   cby0 += afy + 2;
   cby1 = cby0 + 4;
   gl_wvec (begcol,         cby0, (begcol),         cby1, cval1);
   gl_wvec (begcol+32*ppc , cby0, (begcol+32*ppc),  cby1, cval1);
   gl_wvec (begcol+64*ppc , cby0, (begcol+64*ppc),  cby1, cval1);
   gl_wvec (begcol+96*ppc , cby0, (begcol+96*ppc),  cby1, cval1);
   gl_wvec (begcol+128*ppc, cby0, (begcol+128*ppc), cby1, cval1);
   gl_wvec (begcol+160*ppc, cby0, (begcol+160*ppc), cby1, cval1);
   gl_wvec (begcol+192*ppc, cby0, (begcol+192*ppc), cby1, cval1);
   gl_wvec (begcol+224*ppc, cby0, (begcol+224*ppc), cby1, cval1);
   gl_wvec (begcol+255*ppc, cby0, (begcol+255*ppc), cby1, cval1);

   /*** Draw color bar. ***/

   cbar = (short *) malloc (barwidth * barheight * sizeof (short)) ;

   for (iy = 0; iy < barheight; iy++)
      for (step = 0; step <= colormax; step++)
	 for (ix = 0; ix < ppc; ix++)
	    *(cbar + (step * ppc + ix) + (iy * barwidth)) =
	       (short) (step & 0377) + cmapoff ;

   begrow = cby1 + 1 ;
   endrow  = begrow + barheight - 1 ;
   rectwrite (begcol, begrow, endcol, endrow, cbar) ;
   free (cbar) ;

   return (0) ;
}
