/* glfit_annot_vid.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Annotate FITS images in SGI/GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 March 1994
 * 24 May 1996: add datamin, datamax, dispmin, dispmax to annotation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/gl_color.h"
#include "/home/stanger/lib/include/fitslib.ext"

glfit_annot_vid (xwsize, ywsize, xbsize, ybsize, cval1, cval2, cval3)

int xwsize, ywsize ;		/* Window size. */
int xbsize, ybsize ;		/* Annotation border size. */
int cval1 ;			/* Color level for labels. */
int cval2 ;			/* Color level for parameters. */
int cval3 ;			/* Color level for logo. */

{

   /* `Screen' font sizes:
    *  7 =>  6 x  7
    * 11,13,15 =>  8 x 15
    *
    * `Screen-Bold' font sizes:
    * 13,15 =>  9 x 15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11 x 13
    * 10 => 14 x 15
    * 12 => 18 x 20
    * 14 => 20 x 22
    * 18 => 27 x 29
    * 24 => 37 x 38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12 x 13
    * 10 => 14 x 15
    * 12 => 16 x 17
    * 14 => 22 x 23
    * 19 => 28 x 30
    * 25 => 38 x 40
    *
    * `Times-Roman'		`Times-Bold' font sizes:
    *  8 => 11 x 12		8 => 11 x 13
    *  9 => 12 x 15		9 => 13 x 15
    * 10 => 13 x 16		10 => 14 x 16
    * 11 => 15 x 18		11 => 16 x 18
    * 12 => 15 x 18		12 => 17 x 19
    * 14 => 19 x 20		14 => 19 x 19
    * 16 => 21 x 23		16 => 22 x 22
    * 18 => 24 x 28		18 => 27 x 28
    * 20 => 26 x 31		20 => 26 x 31
    * 24 => 35 x 36		24 => 35 x 37
    *
    * `Fixed' font sizes:	`Fixed-Bold' font sizes:
    *  6 =>  5 x  8		 6 =>  4 x  8
    *  8 =>  6 x 11		 8 =>  6 x 11
    *  9 =>  6 x 12		 9 => 13 x 15
    * 10 =>  7 x 13		10 => 14 x 16
    * 11 =>  9 x 15		11 => 16 x 18
    * 12 => 10 x 16		12 => 17 x 19
    * 14 =>  9 x 19		14 => 19 x 19
    * 16 => 11 x 21		16 => 22 x 22
    * 18 => 12 x 24		18 => 27 x 28
    * 20 => 13 x 26		20 => 26 x 31
    * 24 => 16 x 32		24 => 35 x 37
    *
    * `Bookman-Demi' font sizes:
    * 11 => 18 x 16
    * 14 => 21 x 19
    * 17 => 28 x 22
    * 19 => 31 x 24
    * 25 => 43 x 40
    * 33 => 56 x 52
    */

   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
   static char font5 [] = "Times";
   static char font6 [] = "Times-Bold";
   static char font9 [] = "Bookman-Demi" ;

   extern fmfonthandle gl_setfont () ;
   fmfonthandle afh, bfh, cfh ;
   static char afont [32] ;			/* Small  font. */
   static char bfont [32] ;			/* Medium font. */
   static char cfont [32] ;			/* Large  font. */
   static char month_name [12][4] = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN",
			             "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"} ;
   static char cmonth [4] ;
   static char wlab [24] ;
   static char wpar [24] ;
   static char cdelim [4] ;
   static char blanks [32] = "                        ";
   static char blankx [32] = "xxxxxxxxxxxxxxxxxxxxxxxx";
   short *cbar ;

   static int afx, afy, bfx, bfy, cfx, cfy ;
   int afx15 ;
   int asize, bsize, csize ;

   int barwidth  = 512 ;
   int barheight =  12 ;
   int begcol, endcol ;
   int begrow, endrow ;
   int colormax = 255 ;		/* Maximum color level. */
   int ix, iy, step ;
   int doy  ;
   int day  ;
   int month ;
   int year ;
   int hour ;
   int minute ;
   int second ;
   int ldelim ;
   int lendate ;
   int i, fslash, dot, dash ;
   int ifslash1, ifslash2, idot1, idot2, idash1, idash2 ;

   int cbx0, cbx1 ;			/* color bar x-axis start/stop. */
   int cby0, cby1 ;			/* color bar y-axis start/stop. */
   int dxbar  =  12 ;
   int dybar  = 400 ;
   int xticklen = 4 ;

   int dxframe = 640 ;			/* Video frame: x dimension. */
   int dyframe = 480 ;			/* Video frame: y dimension. */
   int dximg   = 400 ;			/* Image array: x dimension. */
   int dyimg   = 400 ;			/* Image array: y dimension. */
   int xmin =   0 ;
   int xmax = 639 ;
   int ymin =   0 ;
   int ymax = 479 ;
   int dxbor = 40 ;			/* X-dimension border. */
   int dybor = 40 ;			/* Y-dimension border. */
   int dxannot = 160 ;
   int dyannot = 400 ;
   int xmin1 = 39 ;
   int xmin2 = 38 ;
   int ymin1 = 38 ;
   int ymin2 = 39 ;
   int xmax1 = 600 ;
   int xmax2 = 601 ;
   int ymax1 = 440 ;
   int ymax2 = 441 ;

   int xs, ys ;
   int x1, x2, x3;
   int y1, y2, y3;
   int xwl, xil, xwr ;
   int ywb, yib, ywt ;

   float ppc = dyannot / 256.0 ; 	/* # pixels per color in color bar. */

   xmin  = 0 ;
   ymax  = 0 ;
   xmax  = dxframe - 1 ;
   ymax  = dyframe - 1 ;
   xmin1 = dxbor - 1 ;
   xmin2 = dxbor - 2 ;
   xmax1 = dxbor + dxannot + dximg ;
   xmax2 = dxbor + dxannot + dximg + 1 ;
   ymin1 = dybor - 1 ;
   ymin2 = dybor - 2 ;
   ymax1 = dybor + dyannot ;
   ymax2 = dybor + dyannot + 1 ;

   xwl = xmin1 ;
   xwr = xmax1 ;
   ywb = ymin1 ;
   ywt = ymax1 ;

   strcpy (afont, font9) ;
   strcpy (bfont, font9) ;
   strcpy (cfont, font9) ;
   asize = 11 ;
   bsize = 14 ;
   csize = 18 ;
   afx   = 18 ;
   afy   = 16 + 3 ;
   bfx   = 21 ;
   bfy   = 19 ;
   cfx   = 27 ;
   cfy   = 28 ;

   if (xwsize >  255)  ppc = 1 ;
   if (xwsize >  511)  ppc = 2 ;
   if (xwsize > 1023)  ppc = 4 ;

   barwidth = (colormax + 1)  * ppc ;
   begcol = xwsize - barwidth ;
   endcol = xwsize - 1 ;
   xs = xmin1 + 3 ;
   ys = ywt - afy + 5 ;
   printf ("xs/ys: %4d %4d\n", xs, ys) ;


   /*** Write labels. ***/

   afh = gl_setfont (afont, asize) ;	/* Small font/size. */

   gl_wstr ("DATE      ", xs, ys,                   cval1) ;
   gl_wstr ("YEAR:DOY  ", xs, ys -  1*afy -  1*bfy, cval1) ;
   gl_wstr ("TIME      ", xs, ys -  2*afy -  2*bfy, cval1) ;
   gl_wstr ("ORIGIN    ", xs, ys -  3*afy -  4*bfy, cval1) ;
   gl_wstr ("TELESCOPE ", xs, ys -  4*afy -  5*bfy, cval1) ;
   gl_wstr ("INSTRUMENT", xs, ys -  5*afy -  6*bfy, cval1) ;
   gl_wstr ("OBJECT    ", xs, ys -  6*afy -  7*bfy, cval1) ;
   gl_wstr ("TYPE-OBS  ", xs, ys -  7*afy -  8*bfy, cval1) ;
   gl_wstr ("DISP MIN  ", xs, ys -  8*afy -  9*bfy, cval1) ;
   gl_wstr ("DISP MAX  ", xs, ys -  9*afy - 10*bfy, cval1) ;

   bfh = gl_setfont (bfont, bsize) ;	/* Medium font/size. */

   /*** Day and month. ***/

   month = 0 ;
   lendate = strlen (date_obs) ;
/*   printf ("lendate: %d   date_obs:|%s|\n", lendate, date_obs) ; */

   fslash = dot = dash = 0 ;
   idash1 = idash2 = -1 ;
   for (i = 0; i < strlen (date_obs); i++)
      {
      if (date_obs [i] == '/')  fslash++ ;
      if (date_obs [i] == '.')  dot++    ;
      if (date_obs [i] == '-')
	 {
	 dash++   ;
         if (dash == 1)  idash1 = i ;
         if (dash == 2)  idash2 = i;
	 }
      }

   if (fslash == 2)
      {
      sscanf (date_obs, "%d/%d/%d",
	      &day, &month, &year) ;

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%d/%d/%d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%d/%d/%d",
	      &year, &month, &day) ;
      }
   else
   if (dot    == 2)
      {
      sscanf (date_obs, "%d.%d.%d",
	      &day, &month, &year) ;

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%d.%d.%d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%d.%d.%d",
	      &year, &month, &day) ;
      }
   else
   if (dash   == 2)
      {
      if ((idash2 - idash1) == 3)
         sscanf (date_obs, "%d-%d-%d",
	         &day, &month, &year) ;
      else
	 {
         sscanf (date_obs, "%d-%3s-%d",
	         &day, cmonth, &year) ;
         for (iy = 0; iy < 12; iy++)
	    if (strcmp (&month_name [iy][0], cmonth) == 0)
	       month = iy + 1 ;
         }

      if (day   > 31 || month > 12 && year > 70)
      sscanf (date_obs, "%d-%d-%d",
	      &month, &day, &year) ;

      if (day   > 31 || month > 12 && year < 70)
      sscanf (date_obs, "%d-%d-%d",
	      &year, &month, &day) ;
      }
   else
      {
      day = month = year = 0 ;
      fprintf (stderr, "unrecognized date_obs format (%s)\n", date_obs) ;
      }


   sprintf (wpar, "%2d  %3s  %4d", day, &month_name [month - 1][0], year+1900) ;
   gl_wstr (blanks, xs, ys -  1*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  1*bfy, cval2) ;
/*
   sprintf (wpar, "%4d", year + 1900) ;
   gl_wstr (blanks, xs, ys -  2*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  2*bfy, cval2) ;
 */

   /*** Year and Day-Of-Year. ***/

   doy = day_of_year (year, month, day) ;
   sprintf (wpar, "%04d : %03d", year + 1900, doy) ;
   gl_wstr (blanks, xs, ys -  1*afy - 2*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  1*afy - 2*bfy, cval2) ;

   /*** Time. ***/

   if (strlen (time_obs) >= 8)
      {
      sscanf (time_obs, "%2d%1s%2d%1s%2d",
	      &hour, cdelim, &minute, cdelim, &second) ;
      (void) sprintf (wpar, "%02d:%02d:%02d", hour, minute, second) ;
      gl_wstr (blanks, xs,  ys -  2*afy  -  3*bfy, cval2) ;
      gl_wstr (wpar,   xs,  ys -  2*afy  -  3*bfy, cval2) ;
      }

   strcpy (wpar, "        ") ;

   /*** origin. ***/

   strcpy (wpar, origin) ;
   gl_wstr (blanks, xs, ys -  3*afy -  5*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  3*afy -  5*bfy, cval2) ;

   /*** telescope. ***/

   strcpy (wpar, telescop) ;
   gl_wstr (blanks, xs, ys -  4*afy -  6*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  4*afy -  6*bfy, cval2) ;

   /*** instrument. ***/

   strcpy (wpar, instrume) ;
   gl_wstr (blanks, xs, ys -  5*afy -  7*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  5*afy -  7*bfy, cval2) ;

   /*** object ***/

   strcpy (wpar, object) ;
   gl_wstr (blanks, xs, ys -  6*afy -  8*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  6*afy -  8*bfy, cval2) ;

   /*** type-obs. ***/

   strcpy (wpar, type_obs) ;
   gl_wstr (blanks, xs, ys -  7*afy -  9*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  7*afy -  9*bfy, cval2) ;

   /*** Display minimum. ***/

   sprintf (wpar, "%9.2e", dispmin) ;
   gl_wstr (blanks, xs, ys -  8*afy - 10*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  8*afy - 10*bfy, cval2) ;

   /*** Display maximum. ***/

   sprintf (wpar, "%9.2e", dispmax) ;
   gl_wstr (blanks, xs, ys -  9*afy - 11*bfy, cval2) ;
   gl_wstr (wpar,   xs, ys -  9*afy - 11*bfy, cval2) ;

   return (0) ;
}
