/* glfit_view2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a FITS image ('filename') in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   16 March 1994 
 * 18 April 1994: add dir2 argument.
 * 10 May 1994: Do intensity scaling based upon dmin & dmax.
 *  8 Jun 1994: draw frame around image.
 * 23 Mar 2001: nbypx = abs (bitpix) / 8.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/cp_color.h"
#include "/home/stanger/lib/include/fitslib.ext"
extern float minfac, maxfac ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glfit_view (filename, dir2, dx, dy, nave,
	    xwsize, ywsize, xbsize, ybsize, mapoff, pmode)

char *filename ;	/* Pointer to filename array. */
char dir2 [] ;		/* Scan direction: rd or ru. */
int dx, dy ;		/* Image width & height for display. */
int nave ;		/* # pixels to average in each dimension. */
int xwsize, ywsize ;	/* Window size. */
int xbsize, ybsize ;	/* Annotation border size. */
int mapoff ;		/* Color map offset (usually 512: color map #2). */
int pmode ;		/* Print mode: 0 silent, 1 verbose. */

{
   FILE *FD ;
   extern char *malloc () ;
   unsigned int imgsiz ;
   unsigned short *lborder, *bborder ;
   unsigned char  *imgbuf, *ibuf ;
   unsigned char  *cbuf, *cimg ;
   short	  *sbuf, *simg ;
   long		  *lbuf, *limg ;
   float	  *fbuf, *fimg ;
   double	  *dbuf, *dimg ;

   short imgval ;
   short gmin = 0 ;
   short gmax = 249 ;
   short gdif = 250 ;

   unsigned char cmin, cmax ;
   short smin, smax ; 
   long  lmin, lmax ;
   float fmin, fmax ;
   float dmin, dmax ;
   float ddmin, ddmax ;

   int fitoff ;			/* # bytes for FITS hdu(s). */
   int i, ix, iy ;
   int nbylborder, nbybborder ;
   int nellborder, nelbborder ;
   int nbypx ;			/* # bytes per pixel. */
   int nval ;
   int nx ;			/* X dimension. */
   int ny ;			/* Y dimension. */
   int nelem ;
   int nread ;
   int xbeg, xend, xsize = 100 ;
   int ybeg, yend, ysize = 100 ;
   int xinc, yinc ;		/* Index increment. */
   int xs = -1, ys = -1 ;	/* Image origin in array.  */
   int xws = -1, yws = -1 ;	/* Image origin in window. */

   float bval ;
   float dval ;

   bval = dval = 0.0 ;

   /*** Read FITS header. ***/

   fitoff = rdhdu (filename) ;
   dispmin = dispmax = 0.0 ;
/*   print_fits () ; */
   nx     = naxis1 ;
   ny     = naxis2 ;
   nbypx  = abs (bitpix) / 8 ;
   xws    = xbsize - 1 ;
   yws    = ybsize - 1 ;
   nellborder = xbsize * dy ;
   nelbborder = ybsize * (xbsize + dx) ;
   nbylborder = xbsize * dy * sizeof (unsigned short) ;
   nbybborder = ybsize * (xbsize + dx) * sizeof (unsigned short) ;
   lborder = (unsigned short *) malloc (nbylborder) ;
   bborder = (unsigned short *) malloc (nbybborder) ;
/*
   bzero (lborder, nbylborder) ;
   bzero (bborder, nbybborder) ;
 */
   for (i = 0 ; i < nellborder ; i++)
      *(lborder + i) = (unsigned short) cp_white ;
   for (i = 0 ; i < nelbborder ; i++)
      *(bborder + i) = (unsigned short) cp_white ;

   /*** Set diraxis1 = 'R' & diraxis2 = 'D' for BBSO H-alpha images. ***/ 

   if ( ( (strncmp (origin, "KBOU", 4) == 0) ||
	  (strncmp (origin, "KHMN", 4) == 0) ||
	  (strncmp (origin, "LEAR", 4) == 0) )   &&
        (type_obs [0] == 'H') )
      {
      strcpy (diraxis1, "R") ;
      strcpy (diraxis2, "D") ;
      }

   if (pmode)
      fprintf (stdout, "origin:%s  type_obs:%s  diraxis1:%s  diraxis2:%s\n",
               origin, type_obs, diraxis1, diraxis2) ;

   /*** If diraxis1 & diraxis2 == NULL, use dir2 to set scan direction. ***/

   if (diraxis1 [0] != '\0')
      dir2 [0] = diraxis1 [0] ;
   if (diraxis2 [0] != '\0')
      dir2 [1] = diraxis2 [0] ;

   for (i = 0 ; i < strlen (dir2) ; i++)
      dir2 [i] = tolower (dir2 [i]) ;

   /*** Set x/y start index. ***/

   if (dir2 [0] == 'r' || dir2 [0] == 'R')
      {
      xs   = 0 ;
      xinc = 1 ;
      }
   else
   if (dir2 [0] == 'l' || dir2 [0] == 'L')
      {
      xs   = dx - 1 ;
      xinc = -1 ;
      fprintf (stderr, "dir2: %s not supported.\n", dir2) ;
      }
   else
      {
      fprintf (stderr, "dir2: %s not supported...assume +x->right\n",
	                dir2) ;
      xs   = 0 ;
      xinc = 1 ;
      }

   if (dir2 [1] == 'u' || dir2 [1] == 'U')
      {
      ys   = 0 ;
      yinc = 1 ;
      }
   else
   if (dir2 [1] == 'd' || dir2 [1] == 'D')
      {
      ys   = dy - 1 ;
      yinc = -1 ;
      }
   else
      {
      fprintf (stderr, "dir2: %s not supported...assume +y->up\n",
			dir2) ;
      ys   = 0 ;
      yinc = 1 ;
      }

   if (pmode)
      fprintf (stdout, "dir2:%s  xs:%3d  ys:%3d  xinc:%d  yinc:%d\n",
	       dir2, xs, ys, xinc, yinc) ;

   /*** Open image file. ***/

   if ((FD = fopen (filename, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %s\n", filename) ;
      exit (-1) ;
      }
   fseek (FD, fitoff, 0) ;

   /*** Read image into memory. ***/
   /*** Convert to short integer if input image is either byte or float. ***/

   nelem  = nx * ny ;
   imgsiz = nelem * nbypx ;
   ibuf   = (unsigned char *) malloc (imgsiz) ;

   /* - - - - - - - - - - - - - char pixels - - - - - - - - - - - - - - - - - */

   if (bitpix == 8)			/* 1-byte pixels (unsigned char). */
      {
      cbuf = (unsigned char *) ibuf ;
      if ((nread = fread (cbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Average image if needed to fit in GL window. ***/

      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 cimg   = (unsigned char *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(cimg + ix + iy*dx) =
                  (*(cbuf + 2*ix     + (2*iy)     * nx) +
		   *(cbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(cbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(cbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 cimg   = (unsigned char *) imgbuf ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
            {
            cmin = 255 ;
            cmax =   0 ;
            for (i = 0; i < nelem; i++)
	       {
	       if (*(cimg + i) < cmin)  cmin = *(cimg + i) ;
	       if (*(cimg + i) > cmax && *(cimg + i) < 250) cmax = *(cimg + i) ;
	       }
            ddmin = (float) cmin ;
            ddmax = (float) cmax ;

	    /*** Compute pixel average over area near center of image. ***/

            xbeg  = (dx / 2) - xsize / 2 ;
	    xend  = xbeg + xsize ;
            ybeg  = (dy / 2) - ysize / 2 ;
	    yend  = ybeg + ysize ;
	    dval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          dval += (float) *(cimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
               dval /= (float) nval ;			/* Compute average. */

	    /*** Compute pixel average over background area. ***/

            xbeg  = dx - 24 ;
	    xend  = dx ;
            ybeg  = (dy / 2) - 32 ;
	    yend  = ybeg + 64 ;
	    bval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          bval += (float) *(cimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
            bval /= (float) nval ;			/* Compute average. */
            dmin = (dval - bval) * minfac + bval ;
            dmax = dval * maxfac ;
	    }
         else
	    {
	    ddmin = (dispmin - b_zero) / b_scale ;
	    ddmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         ddmin = foldmin ;
         ddmax = foldmax ;
         }
      }

   /* - - - - - - - - - - - - - short pixels- - - - - - - - - - - - - - - - - */

   if (bitpix == 16)			/* 2-byte pixels (short int). */
      {
      sbuf = (short *) ibuf ;
      if ((nread = fread (sbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Average image if needed to fit in GL window. ***/

      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 simg   = (short         *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(simg + ix + iy*dx) =
                  (*(sbuf + 2*ix     + (2*iy)     * nx) +
		   *(sbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(sbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(sbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 simg   = (short         *) imgbuf ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified in FITS HDU. */
	    {
	    smin =  32767 ;
	    smax = -32767 ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(simg + i) < smin)  smin = *(simg + i) ;
	       if (*(simg + i) > smax)  smax = *(simg + i) ;
	       }
            ddmin = (float) smin ;
            ddmax = (float) smax ;

	    /*** Compute pixel average over area near center of image. ***/

            xbeg  = (dx / 2) - xsize / 2 ;
	    xend  = xbeg + xsize ;
            ybeg  = (dy / 2) - ysize / 2 ;
	    yend  = ybeg + ysize ;
	    dval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          dval += (float) *(simg + ix + iy*dx) ;/* Sum pixel values. */
	          }
               dval /= (float) nval ;			/* Compute average. */

	    /*** Compute pixel average over background area. ***/

            xbeg  = dx - 24 ;
	    xend  = dx ;
            ybeg  = (dy / 2) - 32 ;
	    yend  = ybeg + 64 ;
	    bval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          bval += (float) *(simg + ix + iy*dx) ;/* Sum pixel values. */
	          }
            bval /= (float) nval ;			/* Compute average. */
            ddmin = (dval - bval) * minfac + bval ;
            ddmax = dval * maxfac ;
	    }
         else
	    {
	    ddmin = (dispmin - b_zero) / b_scale ;
	    ddmax = (dispmax - b_zero) / b_scale ;
	    }
	 }
      else
	 {
	 ddmin = foldmin ;
	 ddmax = foldmax ;
	 }
      }

   /* - - - - - - - - - - - - - long int pixels - - - - - - - - - - - - - - - */

   if (bitpix == 32)			/* 4-byte pixels (long int). */
      {
      lbuf = (long *) ibuf ;
      if ((nread = fread (lbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

/*
 *      fprintf (stdout, "ys:%3d   yinc:%3d   nx/ny: %3d %3d   nbypx: %d\n",
 *	         ys, yinc, nx, ny, nbypx) ;
 */

      /*** Average image if needed to fit in GL window. ***/

      if (nave == 2)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 limg   = (long          *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(limg + ix + iy*dx) =
                  (*(lbuf + 2*ix     + (2*iy)     * nx) +
		   *(lbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(lbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(lbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      if (nave == 3)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 limg   = (long          *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(limg + ix + iy*dx) =
                  (*(lbuf + 3*ix     + (3*iy)     * nx) +
		   *(lbuf + 3*ix     + (3*iy + 1) * nx) +
		   *(lbuf + 3*ix     + (3*iy + 2) * nx) +
                   *(lbuf + 3*ix + 1 + (3*iy)     * nx) +
                   *(lbuf + 3*ix + 1 + (3*iy + 1) * nx) +
                   *(lbuf + 3*ix + 1 + (3*iy + 2) * nx) +
                   *(lbuf + 3*ix + 2 + (3*iy)     * nx) +
                   *(lbuf + 3*ix + 2 + (3*iy + 1) * nx) +
                   *(lbuf + 3*ix + 2 + (3*iy + 2) * nx)) / 9 ;
               }
            }
	 }
      else
      if (nave == 1)
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 limg   = (long          *) imgbuf ;
	 }
      else
	 {
	 fprintf (stderr, "Image is too big: %d %d\n", nx, ny) ;
	 exit (-1) ;
	 }


      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified in FITS HDU. */
	    {
	    lmin =  32767 ;
	    lmax = -32767 ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(limg + i) < lmin)  lmin = *(limg + i) ;
	       if (*(limg + i) > lmax)  lmax = *(limg + i) ;
	       }
            ddmin = (float) lmin ;
            ddmax = (float) lmax ;
	    }
         else
	    {
	    ddmin = (dispmin - b_zero) / b_scale ;
	    ddmax = (dispmax - b_zero) / b_scale ;
	    }
	 }
      else
	 {
	 ddmin = foldmin ;
	 ddmax = foldmax ;
	 }

/*
 *      fprintf (stdout, "dispmin/max: %8.2e %8.2e\n", dispmin, dispmax) ;
 *      fprintf (stdout, "datamin/max: %8.2e %8.2e\n", datamin, datamax) ;
 *      fprintf (stdout, "lmin/lmax  : %8d %8d\n", lmin, lmax) ;
 *      fprintf (stdout, "ddmin/max:    %8.2e %8.2e\n", ddmin, ddmax) ;
 */

      }

   /* - - - - - - - - - - - - - float pixels- - - - - - - - - - - - - - - - - */

   if (bitpix == -32)			/* 4-byte pixels (float). */
      {
      fbuf = (float *) ibuf ;
      if ((nread = fread (fbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Average image if needed to fit in GL window. ***/

      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 fimg   = (float         *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(fimg + ix + iy*dx) =
                  (*(fbuf + 2*ix     + (2*iy)     * nx) +
		   *(fbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(fbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(fbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 fimg   = (float         *) imgbuf ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
         {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
	    {
	    fmin =  1e+30 ;
	    fmax = -1e+30  ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(fimg + i) < fmin)  fmin = *(fimg + i) ;
	       if (*(fimg + i) > fmax)  fmax = *(fimg + i) ;
	       }
            ddmin = fmin ;
            ddmax = fmax ;

	    /*** Compute pixel average over area near center of image. ***/

            xbeg  = (dx / 2) - xsize / 2 ;
	    xend  = xbeg + xsize ;
            ybeg  = (dy / 2) - ysize / 2 ;
	    yend  = ybeg + ysize ;
	    dval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          dval += (float) *(fimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
               dval /= (float) nval ;			/* Compute average. */

	    /*** Compute pixel average over background area. ***/

            xbeg  = dx - 24 ;
	    xend  = dx ;
            ybeg  = (dy / 2) - 32 ;
	    yend  = ybeg + 64 ;
	    bval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          bval += (float) *(fimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
            bval /= (float) nval ;			/* Compute average. */
            ddmin = (dval - bval) * minfac + bval ;
            ddmax = dval * maxfac ;
	    }
         else
	    {
	    ddmin = (dispmin - b_zero) / b_scale ;
	    ddmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         ddmin = foldmin ;
         ddmax = foldmax ;
         }
      }

   /* - - - - - - - - - - - - - double pixels - - - - - - - - - - - - - - - - */

   if (bitpix == -64)			/* 8-byte pixels (double). */
      {
      dbuf = (double *) ibuf ;
      if ((nread = fread (dbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Average image if needed to fit in GL window. ***/

      if (nave == 2)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 dimg   = (double        *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(dimg + ix + iy*dx) =
                  (*(dbuf + 2*ix     + (2*iy)     * nx) +
		   *(dbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(dbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(dbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
      if (nave == 3)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 dimg   = (double        *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(dimg + ix + iy*dx) =
                  (*(dbuf + 3*ix     + (3*iy)     * nx) +
		   *(dbuf + 3*ix     + (3*iy + 1) * nx) +
		   *(dbuf + 3*ix     + (3*iy + 2) * nx) +
                   *(dbuf + 3*ix + 1 + (3*iy)     * nx) +
                   *(dbuf + 3*ix + 1 + (3*iy + 1) * nx) +
                   *(dbuf + 3*ix + 1 + (3*iy + 2) * nx) +
                   *(dbuf + 3*ix + 2 + (3*iy)     * nx) +
                   *(dbuf + 3*ix + 2 + (3*iy + 1) * nx) +
                   *(dbuf + 3*ix + 2 + (3*iy + 2) * nx)) / 9.0 ;
               }
            }
	 }
      else
      if (nave == 1)
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 dimg   = (double        *) imgbuf ;
	 }
      else
	 {
	 fprintf (stderr, "Image is too big: %d %d\n", nx, ny) ;
	 exit (-1) ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
         {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
	    {
	    dmin =  1e+30 ;
	    dmax = -1e+30  ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(dimg + i) < dmin)  dmin = *(dimg + i) ;
	       if (*(dimg + i) > dmax)  dmax = *(dimg + i) ;
	       }
            ddmin = dmin ;
            ddmax = dmax ;
	    }
         else
	    {
	    ddmin = (dispmin - b_zero) / b_scale ;
	    ddmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         ddmin = foldmin ;
         ddmax = foldmax ;
         }

      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   if (pmode)
      {
      fprintf (stdout, "bval:%8.2e  dval:%8.2e\n", bval, dval) ;
      }

   xs += xbsize ;
   ys += ybsize ;

   if (pmode)
      {
      fprintf (stdout, "nbypx:%3d  nave:%3d  dx/dy: %4d %4d  dx/dy: %4d %4d\n",
	       nbypx, nave, dx, dy, dx, dy) ;
      fprintf (stdout, "dispmin/max: %8.2e  %8.2e  ddmin/max: %8.2e  %8.2e\n",
	       dispmin, dispmax, ddmin, ddmax) ;
      fprintf (stdout, "nbypx:%d  xs:%d  ys:%d  dx:%d  dy:%d  dir2:%s  ddmin:%8.2f  ddmax:%8.2f  mapoff:%d\n",
	       nbypx, xs, ys, dx, dy, dir2, ddmin, ddmax, mapoff) ;
      }

   /*** Write image to GL window. ***/
   
   gl_wbox (imgbuf, nbypx, xs, ys, dx, dy, dir2, ddmin, ddmax, mapoff, pmode) ;

   /*** Annotate image. ***/

   dispmin = ddmin ;
   dispmax = ddmax ;

   rectwrite (0, ybsize,    xbsize-1, dy+ybsize-1, lborder) ;
   rectwrite (0,      0, dx+xbsize-1,    ybsize-1, bborder) ;

   glfit_annot (xwsize, ywsize, xbsize, ybsize, cp_grey, cp_red, cp_green) ;

   /*** Draw frame around image. ***/

   gl_wvec (xws,      yws,      dx+xws-1, yws,      cp_grey) ;
   gl_wvec (dx+xws-1, yws,      dx+xws-1, dy+yws-1, cp_grey) ;
   gl_wvec (dx+xws-1, dy+yws-1, xws,      dy+yws-1, cp_grey) ;
   gl_wvec (xws,      dy+yws-1, xws,      yws,      cp_grey) ;

   free (ibuf) ;
   free (imgbuf) ;
   free (lborder) ;
   free (bborder) ;
   fclose (FD) ;
   return (0) ;
}
