/* glfit_view3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a FITS image ('filename') in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   16 March 1994 
 * 18 April 1994: add dir2 argument.
 * 10 May 1994: Do intensity scaling based upon dispmin & dispmax.
 * 31 May 1994: Do 2x2 pixel average if image is too large for screen.
 * 12 August 1994: Use gl_wbox to write image to GL window.
 * 23 February 1995: dir2 argument becomes sdir2.
 *  5 August 1996: if dispmin & dispmax exist in HDU, then :
 *			dmin = (dispmin - b_zero) / b_scale ;
 *	    		dmax = (dispmax - b_zero) / b_scale ;
 *  7 March 1997: Add logic to handle "disp_dir" (display direction).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/gl_color.h"
#include "/home/stanger/lib/include/fitslib.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glfit_view (filename, sdir2, dx, dy, nave,
	    xwsize, ywsize, xbsize, ybsize, mapoff, pmode)

char *filename ;	/* Pointer to filename array. */
char sdir2 [] ;		/* Scan direction: rd or ru. */
int dx, dy ;		/* Image width & height for display. */
int nave ;		/* # pixels to average in each dimension. */
int xwsize, ywsize ;	/* Window size. */
int xbsize, ybsize ;	/* Annotation border size. */
int mapoff ;		/* Color map offset (usually 512: color map #2). */
int pmode ;		/* Print mode: 0 silent, 1 verbose. */

{
   FILE *FD ;
   extern char *hs () ;
   extern char *malloc () ;
   unsigned short *lborder, *bborder ;
   unsigned char  *imgbuf, *ibuf ;
   unsigned char  *cbuf, *cimg ;
   unsigned char  *gbuf ;
   short *sbuf, *simg ;
   float *fbuf, *fimg ;
   short imgval ;
   static char dir2 [4] ;

   short imin = 0, imax = 255 ;
   unsigned char cmin, cmax ;
   short smin, smax ; 
   float fmin, fmax ;
   float dmin, dmax ;

   int cval1 = gl_grey ;	/* Annotation color: labels.     */
   int cval2 = gl_red  ;	/* Annotation color: parameters. */
   int cval3 = gl_green  ;	/* Annotation color: logo.       */
   int cval4 = gl_yellow ;	/* Annotation color: border.     */
   int fitoff ;			/* # bytes for FITS hdu(s). */
   int i, ix, iy ;
   int imgsiz ;
   int nbylborder, nbybborder ;
   int nbypx ;			/* # bytes per pixel. */
   int nx ;			/* X dimension. */
   int ny ;			/* Y dimension. */
   int nelem ;
   int nread ;
   int xinc, yinc ;		/* Index increment. */
   int xs = -1, ys = -1 ;	/* Image origin in array.  */
   int xws = -1, yws = -1 ;	/* Image origin in window. */

   float slope  = 249.0 / 255.0 ;	/* Adjust intensity range: 0 -> 255.*/
   float offset = 0.0;

   /*** Read FITS header. ***/

   fitoff = rdhdu (filename) ;
/*   print_fits () ; */
   nx     = naxis1 ;
   ny     = naxis2 ;
   nbypx  = bitpix / 8 ;
   xws    = xbsize - 1 ;
   yws    = ybsize - 1 ;
   nbylborder = xbsize * dy * sizeof (unsigned short) ;
   nbybborder = ybsize * (xbsize + dx) * sizeof (unsigned short) ;
   lborder = (unsigned short *) malloc (nbylborder) ;
   bborder = (unsigned short *) malloc (nbybborder) ;
   bzero (lborder, nbylborder) ;
   bzero (bborder, nbybborder) ;

   /*** Set diraxis1 = 'R' & diraxis2 = 'D' for BBSO H-alpha images. ***/ 

   if ( ( (strncmp (origin, "KBOU", 4) == 0) ||
	  (strncmp (origin, "KHMN", 4) == 0) ||
	  (strncmp (origin, "LEAR", 4) == 0) )   &&
        (type_obs [0] == 'H') )
      {
      strcpy (diraxis1, "R") ;
      strcpy (diraxis2, "D") ;
      }

   if (pmode)
      fprintf (stdout, "glfit_view> origin:%s  type_obs:%s  diraxis1:%s  diraxis2:%s\n",
               origin, type_obs, diraxis1, diraxis2) ;

   /*** If diraxis1 & diraxis2 == NULL, use sdir2 to set scan direction. ***/

   for (i = 0 ; i < strlen (dir2) ; i++)
      dir2 [i] = '\0' ;				/* Clear dir2 to nulls. */

   if (diraxis1 [0] != '\0')
      dir2 [0] = diraxis1 [0] ;
   else
   if (disp_dir [0] != '\0')
      dir2 [0] = disp_dir [0] ;
   else
      dir2 [0] = sdir2 [0] ;

   if (diraxis2 [0] != '\0')
      dir2 [1] = diraxis2 [0] ;
   else
   if (disp_dir [1] != '\0')
      dir2 [1] = disp_dir [1] ;
   else
      dir2 [1] = sdir2 [1] ;

   for (i = 0 ; i < strlen (dir2) ; i++)
      dir2 [i] = tolower (dir2 [i]) ;

   /*** Set x/y start index. ***/

   if (dir2 [0] == 'r' || dir2 [0] == 'R')
      {
      xs   = 0 ;
      xinc = 1 ;
      }
   else
   if (dir2 [0] == 'l' || dir2 [0] == 'L')
      {
      xs   = dx - 1 ;
      xinc = -1 ;
      fprintf (stderr, "dir2: %s not supported.\n", dir2) ;
      }
   else
      {
      fprintf (stderr, "dir2: %s not supported...assume +x->right\n",
	                dir2) ;
      xs   = 0 ;
      xinc = 1 ;
      }

   if (dir2 [1] == 'u' || dir2 [1] == 'U')
      {
      ys   = 0 ;
      yinc = 1 ;
      }
   else
   if (dir2 [1] == 'd' || dir2 [1] == 'D')
      {
      ys   = dy - 1 ;
      yinc = -1 ;
      }
   else
      {
      fprintf (stderr, "dir2: %s not supported...assume +y->up\n",
			dir2) ;
      ys   = 0 ;
      yinc = 1 ;
      }


   if (pmode)
      fprintf (stdout, "glfit_view>dir2:%s  xs:%3d  ys:%3d  xinc:%d  yinc:%d\n",
	       dir2, xs, ys, xinc, yinc) ;

   /*** Open image file. ***/

   if ((FD = fopen (filename, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %s\n", filename) ;
      exit (-1) ;
      }
   fseek (FD, fitoff, 0) ;

   /*** Read image into memory. ***/
   /*** Convert to short integer if input image is either byte or float. ***/

   nelem  = nx * ny ;
   imgsiz = nelem * nbypx ;
   ibuf   = (unsigned char *) malloc (imgsiz) ;

   if (pmode)
      fprintf (stdout, "glfit_view>nx/ny: %3d %3d   dx/dy: %3d %3d   nbypx: %d\n",
	       nx, ny, dx, dy, nbypx) ;


   /* - - - - - - - - - - - - - char pixels - - - - - - - - - - - - - - - - - */

   if (nbypx == 1)			/* 1-byte pixels (unsigned char). */
      {
      cbuf = (unsigned char *) ibuf ;
      if ((nread = fread (cbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Average image if needed to fit in GL window. ***/

      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 cimg   = (unsigned char *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(cimg + ix + iy*dx) =
                  (*(cbuf + 2*ix     + (2*iy)     * nx) +
		   *(cbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(cbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(cbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 cimg   = (unsigned char *) imgbuf ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
            {
            cmin = 255 ;
            cmax =   0 ;
            for (i = 0; i < nelem; i++)
	       {
	       if (*(cimg + i) < cmin)  cmin = *(cimg + i) ;
	       if (*(cimg + i) > cmax && *(cimg + i) < 250) cmax = *(cimg + i) ;
	       }
            dmin = (float) cmin ;
            dmax = (float) cmax ;
	    }
         else
	    {
	    dmin = (dispmin - b_zero) / b_scale ;
	    dmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         dmin = foldmin ;
         dmax = foldmax ;
         }
      }

   /* - - - - - - - - - - - - - short pixels- - - - - - - - - - - - - - - - - */

   if (nbypx == 2)			/* 2-byte pixels (short int). */
      {
      sbuf = (short *) ibuf ;
      if ((nread = fread (sbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

/*
 *      fprintf (stdout, "ys:%3d   yinc:%3d   nx/ny: %3d %3d   nbypx: %d\n",
 *	         ys, yinc, nx, ny, nbypx) ;
 */

      /*** Average image if needed to fit in GL window. ***/

      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 simg   = (short         *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(simg + ix + iy*dx) =
                  (*(sbuf + 2*ix     + (2*iy)     * nx) +
		   *(sbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(sbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(sbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 simg   = (short         *) imgbuf ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified in FITS HDU. */
	    {
	    smin =  32767 ;
	    smax = -32767 ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(simg + i) < smin)  smin = *(simg + i) ;
	       if (*(simg + i) > smax)  smax = *(simg + i) ;
	       }
            dmin = (float) smin ;
            dmax = (float) smax ;
	    }
         else
	    {
	    dmin = (dispmin - b_zero) / b_scale ;
	    dmax = (dispmax - b_zero) / b_scale ;
	    }
	 }
      else
	 {
	 dmin = foldmin ;
	 dmax = foldmax ;
	 }

/*
 *      fprintf (stdout, "dispmin/max: %8.2e %8.2e\n", dispmin, dispmax) ;
 *      fprintf (stdout, "datamin/max: %8.2e %8.2e\n", datamin, datamax) ;
 *      fprintf (stdout, "smin/smax  : %8d %8d\n", smin, smax) ;
 *      fprintf (stdout, "dmin/max:    %8.2e %8.2e\n", dmin, dmax) ;
 */

      }

   /* - - - - - - - - - - - - - float pixels- - - - - - - - - - - - - - - - - */

   if (nbypx == 4)			/* 4-byte pixels (float). */
      {
      fbuf = (float *) ibuf ;
      if ((nread = fread (fbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Average image if needed to fit in GL window. ***/

      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 fimg   = (float         *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(fimg + ix + iy*dx) =
                  (*(fbuf + 2*ix     + (2*iy)     * nx) +
		   *(fbuf + 2*ix     + (2*iy + 1) * nx) +
                   *(fbuf + 2*ix + 1 + (2*iy)     * nx) +
                   *(fbuf + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) ibuf ;
	 fimg   = (float         *) imgbuf ;
	 }

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
         {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
	    {
	    fmin =  1e+30 ;
	    fmax = -1e+30  ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(fimg + i) < fmin)  fmin = *(fimg + i) ;
	       if (*(fimg + i) > fmax)  fmax = *(fimg + i) ;
	       }
            dmin = fmin ;
            dmax = fmax ;
	    }
         else
	    {
	    dmin = (dispmin - b_zero) / b_scale ;
	    dmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         dmin = foldmin ;
         dmax = foldmax ;
         }
      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   xs += xbsize ;
   ys += ybsize ;

   if (pmode)
      {
      fprintf (stdout, "glfit_view>nbypx:%3d  nave:%3d  nx/ny: %4d %4d  dx/dy: %4d %4d\n",
	       nbypx, nave, nx, ny, dx, dy) ;
      fprintf (stdout, "glfit_view>dispmin/max: %8.2e  %8.2e  dmin/max: %8.2e  %8.2e\n",
	       dispmin, dispmax, dmin, dmax) ;
      fprintf (stdout, "glfit_view>nbypx:%d  xs:%d  ys:%d  dx:%d  dy:%d  dir2:%s  dmin:%8.2f  dmax:%8.2f  mapoff:%d\n",
	       nbypx, xs, ys, dx, dy, dir2, dmin, dmax, mapoff) ;
      }

   gl_wclear (BLACK) ;			/* Clear window. */

   /*** Do Histogram Selection Intensity Scaling. ***/

   gbuf = hs (imgbuf, dx, dy, nbypx, pmode) ;
   nbypx = 1 ;

   /*** Write image to GL window. ***/
   
   gl_wbox (gbuf, nbypx, xs, ys, dx, dy, dir2, dmin, dmax, mapoff, pmode) ;

   /*** Annotate image. ***/

   rectwrite (0, ybsize,    xbsize-1, dy+ybsize-1, lborder) ;
   rectwrite (0,      0, dx+xbsize-1,    ybsize-1, bborder) ;

   glfit_annot (xwsize, ywsize, xbsize, ybsize, cval1, cval2, cval3) ;

   /*** Draw frame around image. ***/

   gl_wvec (xws,    yws,    nx+xws, yws,    cval1) ;
   gl_wvec (nx+xws, yws,    nx+xws, ny+yws, cval1) ;
   gl_wvec (nx+xws, ny+yws, xws,    ny+yws, cval1) ;
   gl_wvec (xws,    ny+yws, xws,    yws,    cval1) ;

   free (gbuf) ;
   free (ibuf) ;
   free (imgbuf) ;
   free (lborder) ;
   free (bborder) ;
   fclose (FD) ;
   return (0) ;
}
