/* glfit_view_vid.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a FITS image ('filename') in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   16 March 1994 
 * 18 April 1994: add dir2 argument.
 * 10 May 1994: Do intensity scaling based upon dmin & dmax.
 *  8 Jun 1994: draw frame around image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/cp_color.h"
#include "/home/stanger/lib/include/fitslib.ext"
extern float minfac, maxfac ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glfit_view_vid (filename, dir2, dx, dy, nave,
	    xwsize, ywsize, xbsize, ybsize, mapoff, pmode)

char *filename ;	/* Pointer to filename array. */
char dir2 [] ;		/* Scan direction: rd or ru. */
int dx, dy ;		/* Image width & height for display. */
int nave ;		/* # pixels to average in each dimension. */
int xwsize, ywsize ;	/* Window size. */
int xbsize, ybsize ;	/* Annotation border size. */
int mapoff ;		/* Color map offset (usually 512: color map #2). */
int pmode ;		/* Print mode: 0 silent, 1 verbose. */

{
   FILE *FD ;
   extern char *malloc () ;
   extern unsigned char *magimg () ;
   unsigned int imgsiz ;
   unsigned short *lborder, *bborder ;
   unsigned char  *ibuf, *imgbuf, *vidbuf ;
   unsigned char  *cbuf, *cimg ;
   short	  *sbuf, *simg ;
   float	  *fbuf, *fimg ;
   short imgval ;
   short gmin = 0 ;
   short gmax = 249 ;
   short gdif = 250 ;

   unsigned char cmin, cmax ;
   short smin, smax ; 
   float fmin, fmax ;
   float dmin, dmax ;

   int dxvid = 400, dyvid = 400 ;
   int fitoff ;			/* # bytes for FITS hdu(s). */
   int i, ix, iy ;
   int nbylborder, nbybborder ;
   int nellborder, nelbborder ;
   int nbypx ;			/* # bytes per pixel. */
   int nval ;
   int nsx, nsy ;
   int nx ;			/* X dimension. */
   int ny ;			/* Y dimension. */
   int nelem ;
   int nread ;
   int xbeg, xend, xsize = 100 ;
   int ybeg, yend, ysize = 100 ;
   int xinc, yinc ;		/* Index increment. */
   int xs = -1, ys = -1 ;	/* Image origin in array.  */
   int xws = -1, yws = -1 ;	/* Image origin in window. */

   float bval ;
   float dval ;

   bval = dval = 0.0 ;

   /*** Read FITS header. ***/

   fitoff = rdhdu (filename) ;
/*   dispmin = dispmax = 0.0 ; */
/*   print_fits () ; */
   nx     = naxis1 ;
   ny     = naxis2 ;
   nbypx  = bitpix / 8 ;
   xws    = xbsize - 1 ;
   yws    = ybsize - 1 ;
   nellborder = xbsize * dy ;
   nelbborder = ybsize * (xbsize + dx) ;
   nbylborder = xbsize * dy * sizeof (unsigned short) ;
   nbybborder = ybsize * (xbsize + dx) * sizeof (unsigned short) ;
   lborder = (unsigned short *) malloc (nbylborder) ;
   bborder = (unsigned short *) malloc (nbybborder) ;
/*
   bzero (lborder, nbylborder) ;
   bzero (bborder, nbybborder) ;
 */
   for (i = 0 ; i < nellborder ; i++)
      *(lborder + i) = (unsigned short) cp_white ;
   for (i = 0 ; i < nelbborder ; i++)
      *(bborder + i) = (unsigned short) cp_white ;

   /*** Set diraxis1 = 'R' & diraxis2 = 'D' for BBSO H-alpha images. ***/ 

   if ( ( (strncmp (origin, "KBOU", 4) == 0) ||
	  (strncmp (origin, "KHMN", 4) == 0) ||
	  (strncmp (origin, "LEAR", 4) == 0) )   &&
        (type_obs [0] == 'H') )
      {
      strcpy (diraxis1, "R") ;
      strcpy (diraxis2, "D") ;
      }

   if (pmode)
      fprintf (stdout, "glfit_view> origin:%s  type_obs:%s  diraxis1:%s  diraxis2:%s\n",
               origin, type_obs, diraxis1, diraxis2) ;

   /*** If diraxis1 & diraxis2 == NULL, use dir2 to set scan direction. ***/

   if (diraxis1 [0] != '\0')
      dir2 [0] = diraxis1 [0] ;
   if (diraxis2 [0] != '\0')
      dir2 [1] = diraxis2 [0] ;

   for (i = 0 ; i < strlen (dir2) ; i++)
      dir2 [i] = tolower (dir2 [i]) ;

   /*** Set x/y start index. ***/

   if (dir2 [0] == 'r' || dir2 [0] == 'R')
      {
      xs   = 0 ;
      xinc = 1 ;
      }
   else
   if (dir2 [0] == 'l' || dir2 [0] == 'L')
      {
      xs   = dx - 1 ;
      xinc = -1 ;
      fprintf (stderr, "glfit_view> dir2: %s not supported.\n", dir2) ;
      }
   else
      {
      fprintf (stderr, "glfit_view> dir2: %s not supported...assume +x->right\n",
	                dir2) ;
      xs   = 0 ;
      xinc = 1 ;
      }

   if (dir2 [1] == 'u' || dir2 [1] == 'U')
      {
      ys   = 0 ;
      yinc = 1 ;
      }
   else
   if (dir2 [1] == 'd' || dir2 [1] == 'D')
      {
      ys   = dy - 1 ;
      yinc = -1 ;
      }
   else
      {
      fprintf (stderr, "glfit_view> dir2: %s not supported...assume +y->up\n",
			dir2) ;
      ys   = 0 ;
      yinc = 1 ;
      }

   if (pmode)
      fprintf (stdout, "glfit_view> dir2:%s  xs:%3d  ys:%3d  xinc:%d  yinc:%d\n",
	       dir2, xs, ys, xinc, yinc) ;

   /*** Open image file. ***/

   if ((FD = fopen (filename, "r")) == NULL)
      {
      fprintf (stderr, "glfit_view> Unable to open file %s\n", filename) ;
      exit (-1) ;
      }
   fseek (FD, fitoff, 0) ;

   /*** Read image into memory. ***/
   /*** Convert to short integer if input image is either byte or float. ***/

   nelem  = nx * ny ;
   imgsiz = nelem * nbypx ;
   ibuf   = (unsigned char *) malloc (imgsiz) ;
   fprintf (stdout, "glfit_view> nx/ny: %4d %4d\n", nx, ny) ;

   /* - - - - - - - - - - - - - char pixels - - - - - - - - - - - - - - - - - */

   if (nbypx == 1)			/* 1-byte pixels (unsigned char). */
      {
      cbuf = (unsigned char *) ibuf ;
      if ((nread = fread (cbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "glfit_view> nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Ensure that image is square. ***/

      if (nx < ny)
         {
	 imgbuf = (unsigned char *) malloc (nx * nsy) ;
	 cimg  = (unsigned char *) imgbuf ;
         nsy = ny - 1 ;
         for (iy = 0 ; iy < nsy ; iy++)
            for (ix = 0 ; ix < nx ; ix++)
              *(cimg + ix + iy*nx) = *(cbuf + ix + iy*nx) ;
	 ny = nsy ;
         }
      else
      if (nx > ny)
         {
	 imgbuf = (unsigned char *) malloc (nsx * ny) ;
	 cimg  = (unsigned char *) imgbuf ;
         nsx = nx - 1 ;
         for (iy = 0 ; iy < ny ; iy++)
            for (ix = 0 ; ix < nsx ; ix++)
               *(cimg + ix + iy*nsx) = *(cbuf + ix + iy*nx) ;
         nx = nsx ;
         }
      else
	 {
/*         cimg = cbuf ; */
	 imgbuf = (unsigned char *) malloc (nx * ny) ;
	 cimg  = (unsigned char *) imgbuf ;
         for (iy = 0 ; iy < ny ; iy++)
            for (ix = 0 ; ix < nx ; ix++)
               *(cimg + ix + iy*nx) = *(cbuf + ix + iy*nx) ;
	 }

      /*** Average image if needed to fit in GL window. ***/

/*
      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 cimg   = (unsigned char *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(cimg + ix + iy*dx) =
                  (*(cimg + 2*ix     + (2*iy)     * nx) +
		   *(cimg + 2*ix     + (2*iy + 1) * nx) +
                   *(cimg + 2*ix + 1 + (2*iy)     * nx) +
                   *(cimg + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) cimg ;
	 cimg   = (unsigned char *) imgbuf ;
	 }
 */

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
            {
            cmin = 255 ;
            cmax =   0 ;
            for (i = 0; i < nelem; i++)
	       {
	       if (*(cimg + i) < cmin)  cmin = *(cimg + i) ;
	       if (*(cimg + i) > cmax && *(cimg + i) < 250) cmax = *(cimg + i) ;
	       }
            dmin = (float) cmin ;
            dmax = (float) cmax ;

	    /*** Compute pixel average over area near center of image. ***/

            xbeg  = (dx / 2) - xsize / 2 ;
	    xend  = xbeg + xsize ;
            ybeg  = (dy / 2) - ysize / 2 ;
	    yend  = ybeg + ysize ;
	    dval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          dval += (float) *(cimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
               dval /= (float) nval ;			/* Compute average. */

	    /*** Compute pixel average over background area. ***/

            xbeg  = dx - 24 ;
	    xend  = dx ;
            ybeg  = (dy / 2) - 32 ;
	    yend  = ybeg + 64 ;
	    bval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          bval += (float) *(cimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
            bval /= (float) nval ;			/* Compute average. */
            dmin = (dval - bval) * minfac + bval ;
            dmax = dval * maxfac ;
	    }
         else
	    {
	    dmin = (dispmin - b_zero) / b_scale ;
	    dmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         dmin = foldmin ;
         dmax = foldmax ;
         }
      }

   /* - - - - - - - - - - - - - short pixels- - - - - - - - - - - - - - - - - */

   if (nbypx == 2)			/* 2-byte pixels (short int). */
      {
      sbuf = (short *) ibuf ;
      if ((nread = fread (sbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "glfit_view> nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Ensure that image is square. ***/

      if (nx < ny)
         {
	 imgbuf = (unsigned char *) malloc (nx * nsy * sizeof (short)) ;
	 simg  = (short *) imgbuf ;
         nsy = ny - 1 ;
         for (iy = 0 ; iy < nsy ; iy++)
            for (ix = 0 ; ix < nx ; ix++)
	       {
               *(simg + ix + iy*nx) = *(sbuf + ix + iy*nx) ;

	       if (ix == iy) printf ("[%3d,%3d] sbuf: %4d  simg: %4d\n",
			  ix, iy, *(sbuf + ix + iy*nx), *(simg + ix + iy*nx)) ;

	       }
	 ny = nsy ;
         }
      else
      if (nx > ny)
         {
	 imgbuf = (unsigned char *) malloc (nsx * ny * sizeof (short)) ;
	 simg  = (short *) imgbuf ;
         nsx = nx - 1 ;
         for (iy = 0 ; iy < ny ; iy++)
            for (ix = 0 ; ix < nsx ; ix++)
	       {
               *(simg + ix + iy*nsx) = *(sbuf + ix + iy*nx) ;

	       if (ix == iy)
		  {
		  printf ("[%3d,%3d] sbuf: %4d  simg: %4d\n",
			ix, iy, *(sbuf + ix + iy*nx), *(simg + ix + iy*nsx)) ;
                  }

               }
         nx = nsx ;
         }
      else
	 {
/*         simg = sbuf ; */
	 imgbuf = (unsigned char *) malloc (nx * ny * sizeof (short)) ;
	 simg  = (short *) imgbuf ;
         for (iy = 0 ; iy < ny ; iy++)
            for (ix = 0 ; ix < nx ; ix++)
               *(simg + ix + iy*nx) = *(sbuf + ix + iy*nx) ;
         }

      /*** Average image if needed to fit in GL window. ***/

/*
      if (nave > 1)
	 {
	 printf ("glfit_view_vid> average image . . . dx/dy: %4d %4d\n",
	          dx, dy) ;
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 simg   = (short         *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(simg + ix + iy*dx) =
                  (*(simg + 2*ix     + (2*iy)     * nx) +
		   *(simg + 2*ix     + (2*iy + 1) * nx) +
                   *(simg + 2*ix + 1 + (2*iy)     * nx) +
                   *(simg + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 printf ("glfit_view_vid> imgbuf = simg\n") ;
	 imgbuf = (unsigned char *) simg ;
	 simg   = (short         *) imgbuf ;
	 }
 */

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
	 {
         if (dispmin == dispmax)	/* No scaling specified in FITS HDU. */
	    {
	    smin =  32767 ;
	    smax = -32767 ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(simg + i) < smin)  smin = *(simg + i) ;
	       if (*(simg + i) > smax)  smax = *(simg + i) ;
	       }
            dmin = (float) smin ;
            dmax = (float) smax ;

	    /*** Compute pixel average over area near center of image. ***/

            xbeg  = (dx / 2) - xsize / 2 ;
	    xend  = xbeg + xsize ;
            ybeg  = (dy / 2) - ysize / 2 ;
	    yend  = ybeg + ysize ;
	    dval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          dval += (float) *(simg + ix + iy*dx) ;/* Sum pixel values. */
	          }
               dval /= (float) nval ;			/* Compute average. */

	    /*** Compute pixel average over background area. ***/

            xbeg  = dx - 24 ;
	    xend  = dx ;
            ybeg  = (dy / 2) - 32 ;
	    yend  = ybeg + 64 ;
	    bval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          bval += (float) *(simg + ix + iy*dx) ;/* Sum pixel values. */
	          }
            bval /= (float) nval ;			/* Compute average. */
            dmin = (dval - bval) * minfac + bval ;
            dmax = dval * maxfac ;
	    }
         else
	    {
	    dmin = (dispmin - b_zero) / b_scale ;
	    dmax = (dispmax - b_zero) / b_scale ;
	    }
	 }
      else
	 {
	 dmin = foldmin ;
	 dmax = foldmax ;
	 }
      }

   /* - - - - - - - - - - - - - float pixels- - - - - - - - - - - - - - - - - */

   if (nbypx == 4)			/* 4-byte pixels (float). */
      {
      fbuf = (float *) ibuf ;
      if ((nread = fread (fbuf, nbypx, nelem, FD)) != nelem)
         {
         fprintf (stderr, "glfit_view> nread: %d  != nelem: %d\n", nread, nelem) ;
         return (-1) ;
         }

      /*** Ensure that image is square. ***/

      if (nx < ny)
         {
	 imgbuf = (unsigned char *) malloc (nx * nsy * sizeof (float)) ;
	 fimg  = (float *) imgbuf ;
         nsy = ny - 1 ;
         for (iy = 0 ; iy < nsy ; iy++)
            for (ix = 0 ; ix < nx ; ix++)
              *(fimg + ix + iy*nx) = *(fbuf + ix + iy*nx) ;
	 ny = nsy ;
         }
      else
      if (nx > ny)
         {
	 imgbuf = (unsigned char *) malloc (nsx * ny * sizeof (float)) ;
	 fimg  = (float *) imgbuf ;
         nsx = nx - 1 ;
         for (iy = 0 ; iy < ny ; iy++)
            for (ix = 0 ; ix < nsx ; ix++)
               *(fimg + ix + iy*nsx) = *(fbuf + ix + iy*nx) ;
         nx = nsx ;
         }
      else
	 {
/*         fimg = fbuf ; */
	 imgbuf = (unsigned char *) malloc (nx * ny * sizeof (float)) ;
	 fimg  = (float *) imgbuf ;
         for (iy = 0 ; iy < ny ; iy++)
            for (ix = 0 ; ix < nx ; ix++)
               *(fimg + ix + iy*nx) = *(fbuf + ix + iy*nx) ;
	 }

      /*** Average image if needed to fit in GL window. ***/

/*
      if (nave > 1)
	 {
         imgbuf = (unsigned char *) malloc (dx * dy * nbypx) ;
	 fimg   = (float         *) imgbuf ;
         for (iy = 0; iy < dy; iy++)
            {
	    for (ix = 0; ix < dx; ix++)
               {
               *(fimg + ix + iy*dx) =
                  (*(fimg + 2*ix     + (2*iy)     * nx) +
		   *(fimg + 2*ix     + (2*iy + 1) * nx) +
                   *(fimg + 2*ix + 1 + (2*iy)     * nx) +
                   *(fimg + 2*ix + 1 + (2*iy + 1) * nx)) / 4;
               }
            }
	 }
      else
	 {
	 imgbuf = (unsigned char *) fimg ;
	 fimg   = (float         *) imgbuf ;
	 }
 */

      /*** Find image min/max intensities. ***/

      nelem = dx * dy ;
      if (foldmin == foldmax)		/* No user-specified scaling. */
         {
         if (dispmin == dispmax)	/* No scaling specified by FITS HDU. */
	    {
	    fmin =  1e+30 ;
	    fmax = -1e+30  ;
	    for (i = 0; i < nelem; i++)
	       {
	       if (*(fimg + i) < fmin)  fmin = *(fimg + i) ;
	       if (*(fimg + i) > fmax)  fmax = *(fimg + i) ;
	       }
            dmin = fmin ;
            dmax = fmax ;

	    /*** Compute pixel average over area near center of image. ***/

            xbeg  = (dx / 2) - xsize / 2 ;
	    xend  = xbeg + xsize ;
            ybeg  = (dy / 2) - ysize / 2 ;
	    yend  = ybeg + ysize ;
	    dval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          dval += (float) *(fimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
               dval /= (float) nval ;			/* Compute average. */

	    /*** Compute pixel average over background area. ***/

            xbeg  = dx - 24 ;
	    xend  = dx ;
            ybeg  = (dy / 2) - 32 ;
	    yend  = ybeg + 64 ;
	    bval = 0.0 ;
	    nval =   0 ;
	    for (iy = ybeg; iy < yend; iy++)
	       for (ix = xbeg; ix < xend; ix++)
	          {
	          nval += 1 ;
	          bval += (float) *(fimg + ix + iy*dx) ;/* Sum pixel values. */
	          }
            bval /= (float) nval ;			/* Compute average. */
            dmin = (dval - bval) * minfac + bval ;
            dmax = dval * maxfac ;
	    }
         else
	    {
	    dmin = (dispmin - b_zero) / b_scale ;
	    dmax = (dispmax - b_zero) / b_scale ;
	    }
         }
      else
         {
         dmin = foldmin ;
         dmax = foldmax ;
         }
      }

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   if (pmode)
      {
      fprintf (stdout, "glfit_view> bval:%9.3g  dval:%9.3g\n", bval, dval) ;
      }

   xs += xbsize ;
   ys += ybsize ;

   if (pmode)
      {
      fprintf (stdout, "glfit_view> nbypx:%3d  nave:%3d  dx/dy: %4d %4d  dx/dy: %4d %4d\n",
	       nbypx, nave, dx, dy, dx, dy) ;
      fprintf (stdout, "glfit_view> dispmin/max: %9.3g  %9.3g  dmin/max: %9.3g  %9.3g\n",
	       dispmin, dispmax, dmin, dmax) ;
      fprintf (stdout, "glfit_view> xs:%d  ys:%d  dir2:%s  mapoff:%d\n",
	       xs, ys, dir2, mapoff) ;
      }

   /*** Magnify/Minify image. ***/

   vidbuf = magimg (imgbuf, nbypx, nx, ny, dxvid, dyvid) ;

   /*** Write image to GL window. ***/
   
   gl_wclear (cp_black) ;		/* Clear frame buffer to black. */
   gl_wbox (vidbuf, nbypx, xs, ys, dxvid, dyvid, dir2, dmin, dmax, mapoff, pmode) ;

   /*** Annotate image. ***/

/*
   rectwrite (0, ybsize,    xbsize-1, dy+ybsize-1, lborder) ;
   rectwrite (0,      0, dx+xbsize-1,    ybsize-1, bborder) ;
 */

   glfit_annot_vid (xwsize, ywsize, xbsize, ybsize, cp_grey, cp_red, cp_green) ;

   /*** Draw frame around image. ***/

   gl_wvec (xws,         yws,         dxvid+xws-1, yws,         cp_grey) ;
   gl_wvec (dxvid+xws-1, yws,         dxvid+xws-1, dyvid+yws-1, cp_grey) ;
   gl_wvec (dxvid+xws-1, dyvid+yws-1, xws,         dyvid+yws-1, cp_grey) ;
   gl_wvec (xws,         dyvid+yws-1, xws,         yws,         cp_grey) ;
   gl_wvec (yws,         yws,         xws-1,       yws,         cp_grey) ;
   gl_wvec (yws,         dyvid+yws-1, xws-1,       dyvid+yws-1, cp_grey) ;
   gl_wvec (yws,         yws,         yws,         dyvid+yws-1, cp_grey) ;

   free (ibuf) ;
   free (imgbuf) ;
   free (vidbuf) ;
   free (lborder) ;
   free (bborder) ;
   fclose (FD) ;
   return (0) ;
}
