/* imgbin.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reduce image in size by "binning":
 *   averaging a pixel boxes of size nave x nave, where nave is the number
 *   of pixels to average in each dimension.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   22 July 1999
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

unsigned char *
imgbin (imgbuf, dx, dy, nbypix, nave)

unsigned char *imgbuf ;		/* Input image buffer. */
int dx, dy ;			/* Dimensions of input image. */
int nbypix ;			/* Pixel depth (bytes). */
int nave ;			/* # pixels to average in each dimension. */

{
   unsigned char *outbuf ;
   unsigned char *cbuf, *ocbuf ;
   short *sbuf, osbuf ;
   float *fbuf, ofbuf ;

   int ix, iy ;			/* Array indices. */
   int nx, ny ;			/* New image dimensions. */

   float sum = 0.0 ;

   nx = dx / nave ;
   ny = dy / nave ;
   fprintf (stdout, "imgbin> nx, ny:   %d   %d\n", nx, ny) ;

   outbuf = (unsigned char *) malloc (nx * ny * nbypix) ;

   if (nbypix == 1)
      {
      cbuf  = imgbuf ;
      ocbuf = outbuf ;
      for (iy = 0 ; iy < ny ; iy++)
         {
	 sum = 0.0 ;
         for (ay = 0; ay < nave ; ay++)
	    {
	    iiy = iy * nave + ay ; 
            for (ix = 0 ; ix < nx ; ix++)
	       {
               for (ax = 0; ax < nave ; ax++)
	          {
	          iix = ix * nave + ax ; 
	          sum += *(cbuf + iix + dx * iiy) ;
	          }
               *(ocbuf + ix + nx * iy) = (unsigned char) sum / (nave * nave) ;
	       }
	    }
         }
      }

   if (nbypix == 2)
      {
      sbuf  = imgbuf ;
      osbuf = outbuf ;
      for (iy = 0 ; iy < ny ; iy++)
         {
	 sum = 0.0 ;
         for (ay = 0; ay < nave ; ay++)
	    {
	    iiy = iy * nave + ay ; 
            for (ix = 0 ; ix < nx ; ix++)
	       {
               for (ax = 0; ax < nave ; ax++)
	          {
	          iix = ix * nave + ax ; 
	          sum += *(sbuf + iix + dx * iiy) ;
	          }
               *(osbuf + ix + nx * iy) = (short) sum / (nave * nave) ;
	       }
	    }
         }
      }

   if (nbypix == 4)
      {
      fbuf  = imgbuf ;
      ofbuf = outbuf ;
      for (iy = 0 ; iy < ny ; iy++)
         {
	 sum = 0.0 ;
         for (ay = 0; ay < nave ; ay++)
	    {
	    iiy = iy * nave + ay ; 
            for (ix = 0 ; ix < nx ; ix++)
	       {
               for (ax = 0; ax < nave ; ax++)
	          {
	          iix = ix * nave + ax ; 
	          sum += *(fbuf + iix + dx * iiy) ;
	          }
               *(ofbuf + ix + nx * iy) = (float) sum / (nave * nave) ;
	       }
	    }
         }
      }

   return (outbuf) ;
}
