/* magimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * M A G I M G ( )	Enlarge or Shrink an image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   5 Nov 1998
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

unsigned char *
magimg (imgbuf, nbypx, ixdim, iydim, oxdim, oydim)

unsigned char *imgbuf ;	/* Pointer to input image array. */
int nbypx ;		/* # bytes per pixel. */
int ixdim, iydim ;	/* Dimensions of input  image. */
int oxdim, oydim ;	/* Dimensions of output image. */
{
   extern float blint () ;	/* Bilinear interpolator. */
   unsigned char  *imgout ;	/* Pointer to output image array. */
   unsigned char  *imc1, *imc2 ;
   unsigned short *ims1, *ims2 ;
   float          *imf1, *imf2 ;

   float value = 0.0 ;		/* Temporary storage for pixel value. */
   float ixcen, iycen ;		/* Center of input  image. */
   float oxcen, oycen ;		/* Center of output image. */
   float xmag, ymag ;		/* Magnification. */
   float ox, oy ;		/* Pixel coordinates. */
   int outsiz ;			/* # elements in output image. */
   int ix, iy ;			/* Indices. */

   outsiz = oxdim * oydim * nbypx ;
   imgout = (unsigned char *) malloc (outsiz) ;

   imc1 = (unsigned char *) imgbuf ;
   imc2 = (unsigned char *) imgout ;

   ims1 = (unsigned short *) imgbuf ;
   ims2 = (unsigned short *) imgout ;

   imf1 = (float *) imgbuf ;
   imf2 = (float *) imgout ;

   /* Determine image centers for input & output images. */

   ixcen = (float) ixdim / 2.0 ;
   iycen = (float) iydim / 2.0 ;
   oxcen = (float) oxdim / 2.0 ;
   oycen = (float) oydim / 2.0 ;

   /* Compute magnification. */

   xmag = (float) (oxdim) / (float) (ixdim) ;
   ymag = (float) (oydim) / (float) (iydim) ;

   fprintf (stdout, "magimg> ixdim:%d  iydim:%d  oxdim:%d  oydim:%d\n",
		     ixdim, iydim, oxdim, oydim) ;
   fprintf (stdout, "magimg> ixcen:%6.1f iycen:%6.1f oxcen:%6.1f oycen:%6.1f\n",
                     ixcen, iycen, oxcen, oycen) ;
   fprintf (stdout, "magimg> xmag:%f  ymag:%f\n", xmag, ymag) ;

   /* Squeeze input image into output image dimensions.
    * Use bilinear interpolation.
    */

   if (nbypx == 1)
      {
      for (iy = 0 ; iy < oydim ; iy++)
         {
         oy = (iy - oycen) / ymag + iycen ;
         for (ix = 0 ; ix < oxdim ; ix++)
            {
            ox = (ix - oxcen) / xmag + ixcen ;
            value = blint (imc1, nbypx, ixdim, iydim, &ox, &oy) ;
            *(imc2 + ix + iy*oxdim) = ((unsigned char) value & 0377) ;
            }
         }
      }

   if (nbypx == 2)
      {
      for (iy = 0 ; iy < oydim ; iy++)
         {
         oy = (iy - oycen) / ymag + iycen ;
         for (ix = 0 ; ix < oxdim ; ix++)
            {
            ox = (ix - oxcen) / xmag + ixcen ;
            value = blint (ims1, nbypx, ixdim, iydim, &ox, &oy) ;
            *(ims2 + ix + iy*oxdim) = ((unsigned short) value & 0177777) ;
            }
         }
      }

   if (nbypx == 4)
      {
      for (iy = 0 ; iy < oydim ; iy++)
         {
         oy = (iy - oycen) / ymag + iycen ;
         for (ix = 0 ; ix < oxdim ; ix++)
            {
            ox = (ix - oxcen) / xmag + ixcen ;
            value = blint (imf1, nbypx, ixdim, iydim, &ox, &oy) ;
            *(imf2 + ix + iy*oxdim) = value ;
            }
         }
      }

   return (imgout) ;
}
