/* rotimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * original author: Rose Reynolds
 * Adapted by Andrew Stanger   HAO/NCAR   12 July 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <math.h>

unsigned char *
rotimg (imginp, nbypx, dx, dy, cx, cy, angdeg)

unsigned char *imginp;	/* Input image array pointer. */
int nbypx ;		/* # bytes per pixel. */
int dx, dy ;		/* Dimensions of image. */
int cx, cy ;		/* Center of rotation. */
float angdeg ;		/* Rotation angle in degrees (+ = CCW). */

{
   extern char *malloc ();
   register int nx, ny, ox, oy, xdim, ydim;
   float fox, foy, cose, sine, fnx, fny, fnysin, fnycos;
   float fcx, fcy;
   double angrad;
   unsigned char  *imgrot;
   unsigned char  *cinp, *crot ;
   unsigned short *sinp, *srot ;
   unsigned short sval, smin, smax ;
   float          *finp, *frot ;

   smin =  32000 ;
   smax = -32000 ;

   xdim = dx;   ydim = dy;
   fcx = (float) cx;   fcy = (float) cy;
   angrad = (double) angdeg * (3.14159265/180.0); /* convert deg to radians. */
   sine = (float) sin ( angrad );
   cose = (float) cos ( angrad );

   imgrot = (unsigned char *) malloc ( dx * dy * nbypx );

   if (nbypx == 1)				/* unsigned char pixels. */
   for ( ny = 0; ny < ydim; ny++ )
      {
      cinp = imginp ;
      crot = imgrot ;
      fny = (float) (ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for ( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *crot++ = 0;
         else *crot++ = *(cinp + (oy*xdim) + ox);
         }
      }

   else
   if (nbypx == 2)				/* unsigned short int pixels. */
   {
   for ( ny = 0; ny < ydim; ny++ )
      {
      sinp = (unsigned short *) imginp ;
      srot = (unsigned short *) imgrot ;
      fny = (float) (ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for ( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *srot++ = 0;
         else
	    {
	    sval = *(sinp + (oy*xdim) + ox) ;
	    if (sval < smin) smin = sval ;
	    if (sval > smax) smax = sval ;

	    *srot++ = *(sinp + (oy*xdim) + ox);
	    }
         }
      }
   }
   else
   if (nbypx == 4)				/* floating point pixels. */
   for ( ny = 0; ny < ydim; ny++ )
      {
      finp = (float *) imginp ;
      frot = (float *) imgrot ;
      fny = (float) (ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for ( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *frot++ = 0;
         else *frot++ = *(finp + (oy*xdim) + ox);
         }
      }

   return ( imgrot );
}
