/* xyplot.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   9 February 1987
 *  26 March 1990: convert to GKS.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namemc : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   igterm = 0 for non-graphics terminal, =1 for graphics terminal.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   ltype	x-axis	y-axis
 *   -----      ------  ------
 *	1	linear	linear
 *	2	linear	log
 *	3	log	linear
 *	4	log	log
 */
#include "/home/stanger/lib/include/cplib.ext"
extern int pcount;

xyplot (x, y, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg)

float	x[], y[];
int	nval;
char	namemc[];
char	title [60], labx [40], laby [40];
int	ixscale, iyscale;
int	ibkg;

{
static char ip [4];
static int igph = 5;
static int ixlab = 1, iylab= 1;
int majrx = 10;
int majry = 10;

int i;
int is = 2;
int io = 0;
int ic = 0;
int ll = 1;
int lentit;
int lenlabx;
int lenlaby;
int lp;
int ltype;
int minrx, minry;

float dxx, dxn;
float dyy, dyn;
float fl = 0.0, fr = 1.0, fb = 0.0, ft = 1.0;
float ul = 0.0, ur = 1.0, ub = 0.0, ut = 1.0;
float fval;
float px, py;
float xmin, xmax, ymin, ymax;
float chsize, angd, cntr;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   pcount ++;
/*   if (pcount == 1)  gkschnam_ (namemc); */

   /*** Set scaling type. ***/

   if (ixscale == 0 && iyscale == 0)  ltype = 1;	/* x linear, y linear */
   if (ixscale == 0 && iyscale == 1)  ltype = 2;	/* x linear, y log    */
   if (ixscale == 1 && iyscale == 0)  ltype = 3;	/* x log,    y linear */
   if (ixscale == 1 && iyscale == 1)  ltype = 4;	/* x log,    y log    */

   /*** Determine minimum & maximum values for X/Y axis ranges. ***/

   xmin = 1.e+30;
   xmax = -1.e+30;
   ymin = 1.e+30;
   ymax = -1.e+30;

   for (i=0; i<nval; i++)
      {
      if (xmin > x[i])  xmin = x[i];
      if (xmax < x[i])  xmax = x[i];
      if (ymin > y[i])  ymin = y[i];
      if (ymax < y[i])  ymax = y[i];
      }
/*
 *   printf ("xmin:%f  xmax:%f  ymin:%f  ymax:%f\n", xmin, xmax, ymin, ymax);
 */

   /*** Write title & X/Y axis labels. ***/

   lentit  = strlen (title);
   lenlabx = strlen (labx);
   lenlaby = strlen (laby);
   is = 2;     io = 0;     ic = 0;
   fl = 0.0;   fr = 1.0;   fb = 0.0;   ft = 1.0;
   ul = 0.0;   ur = 1.0;   ub = 0.0;   ut = 1.0;
   ll = 1;
   set_ (&fl, &fr, &fb, &ft, &ul, &ur, &ub, &ut, &ll);

/*
 *   px = 0.6;   py = 0.9;
 *   pwrit_ (&px, &py, title, &lentit, &is, &io, &ic);
 *
 *   px = 0.6;   py = 0.08;
 *   pwrit_ (&px, &py, labx, &lenlabx, &is, &io, &ic);
 *
 *   px = 0.1;   py = 0.5;   io = 90;
 *   pwrit_ (&px, &py, laby, &lenlaby, &is, &io, &ic);
 */

   chsize = 14.0;   angd = 0.0;   cntr = 0.0;
   px = 0.565;   py = 0.985;
   strcpy (ip, "HW");
   fval = 3.0;
   pcsetr_ (ip, &fval, strlen (ip));
   plchmq_ (&px, &py, title, &chsize, &angd, &cntr, strlen (title));
   px = 0.6;   py = 0.01;   chsize = 16.0;
   fval = 2.0;
   pcsetr_ (ip, &fval, strlen (ip));
   plchhq_ (&px, &py, labx,  &chsize, &angd, &cntr, strlen (labx));
   px = 0.01;   py = 0.5;   angd = 90.0;
   plchhq_ (&px, &py, laby,  &chsize, &angd, &cntr, strlen (laby));

   /*** Determine minor & major scaling values for axes. ***/

   nicexr (&xmin, &xmax, &majrx, &minrx, &dxx, &dxn);
/*
 *   printf ("majrx:%d  minrx:%d  dxx:%f  dxn:%f\n", majrx, minrx, dxx, dxn);
 */

   nicexr (&ymin, &ymax, &majry, &minry, &dyy, &dyn);
/*
 *   printf ("majry:%d  minry:%d  dyy:%f  dyn:%f\n", majry, minry, dyy, dyn);
 *   printf ("xmin: %g  xmax: %g   ymin: %g   ymax: %g\n", xmin, xmax, ymin, ymax);
 */

   fl = 0.0 ;   fr = 1.00;   fb = 0.0 ;   ft = 1.00;
   fl = 0.16;   fr = 1.00;   fb = 0.08;   ft = 0.92;
   set_ (&fl, &fr, &fb, &ft, &xmin, &xmax, &ymin, &ymax, &ltype);

   switch (ibkg)
      {
      case 0: break;			/* Do not change. */
      default:
      case 1: igph = 5;  break;		/* perimeter with tick marks. */
      case 2: igph = 0;  break;		/* full grid. */
      case 3: igph = 10; break;		/* half axes. */
      }
   gridal_ (&majrx, &minrx, &majry, &minry, &ixlab, &iylab, &igph, &xmin, &ymin);

   /*** Draw curve through data points. ***/

   curve_ (x, y, &nval);
   frame_ ();

   return (0);
}
