/* glimg3_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display 3 images ('filename') as an RGB image in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 24 May 1991: use onemap color mode.
 * 12 Dec 1991: add hdrsiz argument for header to skip at BOF.
 * 17 Apr 1995: adapted from 'glimg_view'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/gl_color.h"


/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glimg3_view (fname1, fname2, fname3, hdrsiz, nbypx, nx, ny, dir2)

char *fname1;		/* Pointer to filename array. */
char *fname2;		/* Pointer to filename array. */
char *fname3;		/* Pointer to filename array. */
int hdrsiz;		/* # bytes to skip at BOF. */
int nbypx;		/* # bytes per pixel. */
int nx;			/* X dimension. */
int ny;			/* Y dimension. */
char dir2 [];		/* Scan direction: rd or ru. */

{
   static char filename [80] ;
   static char fname [3][80] ;
   extern char *malloc ();
   float pbfac = 2.5 ;
   int fd;
   int i, ix, iy, iiy;
   unsigned int imgsizi, imgsizo ;
   int nelem;
   int nread;
   int ul = 0 ;			/* Upper left origin ? =0 no, =1 yes. */

   unsigned char *imgbuf;
   unsigned char *ibuf;
   unsigned char *cbuf;
   unsigned short *glbuf ;
   short *sbuf;
   float *fbuf;

   float fval ;
   float fmin, fmax ;
   float slope, offset ;

/*
 *   printf ("fname1/2/3: %s %s %s\n", fname1, fname2, fname3) ;
 *   printf ("hdrsiz: %d   nbypx: %d   nx: %d  ny: %d  dir2: %s\n",
 *           hdrsiz, nbypx, nx, ny, dir2) ;
 */

   if (strcmp (dir2, "rd") == 0)
      ul = 1 ;
   else
   if (strcmp (dir2, "ru") == 0)
      ul = 0 ;
   else
      {
      fprintf (stderr, "Unsupported scan direction (%s); must be ru or rd.\n",
	       dir2) ;
      return (-1) ;
      }

   strcpy (&fname [0][0], fname1) ;
   strcpy (&fname [1][0], fname2) ;
   strcpy (&fname [2][0], fname3) ;

   nelem = nx * ny ;
   imgsizi = nelem * nbypx;
   imgbuf = (unsigned char *)  malloc (nelem * 3) ;

for (i = 0; i < 3; i++)
   {
   ibuf = imgbuf + i * nelem;
   strcpy (filename, &fname [i][0]) ;

   /*** Open image file. ***/

   if ((fd = open (filename, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open file %s ... fd: %d\n", filename, fd);
      exit (-1);
      }
   lseek (fd, hdrsiz, 0);

   /*** Read image into memory. ***/
   /*** Convert to short integer if input image is either byte or float. ***/
   /*** Flip image top-to-bottom for rectwrite. ***/

   cbuf   = (unsigned char *)  malloc (imgsizi);

   if (nbypx == 1)
      {
      if ((nread = read (fd, cbuf, imgsizi)) != imgsizi)
         {
         fprintf (stderr, "nread: %d  != imgsizi: %d\n", nread, imgsizi);
         return (-2);
         }

      for (iy = 0; iy < ny; iy++)
	 {
	 if (ul)
	    iiy = ny - 1 - iy ;
         else
	    iiy = iy ;
         for (ix = 0; ix < nx; ix++)
            {
            *(ibuf + ix + iiy*nx) =
	       (unsigned char) *(cbuf + ix + iy*nx) ;
            }
         }
      }

   if (nbypx == 2)
      {
      sbuf = (short *) cbuf ;
      if (read (fd, cbuf, imgsizi) != imgsizi)
         {
         fprintf (stderr, "nread: %d  != imgsizi: %d\n", nread, imgsizi);
         return (-3);
         }

      for (iy = 0; iy < ny; iy++)
	 {
	 if (ul)
	    iiy = ny - 1 - iy ;
         else
	    iiy = iy ;
         for (ix = 0; ix < nx; ix++)
            {
            *(ibuf + ix + iiy*nx) =
	       (unsigned char) *(sbuf + ix + iy*nx) / 4 ;
            }
         }
      }

   if (nbypx == 4)
      {
      fmin =  1.e+30 ;
      fmax = -1.e+30 ;
      fbuf = (float *) cbuf ;
      if (read (fd, cbuf, imgsizi) != imgsizi)
         {
         fprintf (stderr, "nread: %d  != imgsizi: %d\n", nread, imgsizi);
         return (-4);
         }

      slope = 250.0 / pbfac ;
      offset = 0.0 ;

      for (iy = 0; iy < ny; iy++)
	 {
	 if (ul)
	    iiy = ny - 1 - iy ;
         else
	    iiy = iy ;

         for (ix = 0; ix < nx; ix++)
            {
	    fval = *(fbuf + ix + iiy*nx) ;
	    if (fval < fmin)  fmin = fval ;
	    if (fval > fmax)  fmax = fval ;
	    if (fval > 0.0)
	       fval = (fval -  offset) * slope ;
            else
	       fval = 0.0 ;
            *(ibuf + ix + iiy*nx) = (unsigned char) fval ;
            }
         }
      fprintf (stdout, "fmin: %8.2e  fmax: %8.2e\n", fmin, fmax) ;
      }

   free (cbuf);
   close (fd);
   }

   /*** Write image to GL window. ***/
   
   gl_wrgb (imgbuf, imgbuf + nelem, imgbuf + nelem*2, nx, ny) ;

   free (imgbuf);
   return (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * gl_wrgb: Write RGB image (standard Sun rasterfile format) to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_wrgb (rbuf, gbuf, bbuf, dxs, dys)

unsigned char *rbuf, *gbuf, *bbuf ;	/*** Red/Green/Blue image buffers. ***/
int dxs, dys ;

{
   extern char *malloc () ;
   unsigned long *imgrgb ;
   unsigned long ired, igreen, iblue ;
   int ix, iy, iiy ;

   /*** Allocate storage for GL output array. ***/

   imgrgb = (unsigned long *) malloc (dxs * dys * sizeof (unsigned long)) ;

   /*** Rewrite R/G/B data into GL lrectwrite form. ***/

   for (iy = 0 ; iy < dys; iy++)
      {
      iiy = dys - iy - 1 ;
      for (ix = 0 ; ix < dxs; ix++)
         {
         iblue  = (unsigned long) *(bbuf + ix     + iy * dxs) & 0377 ;
         igreen = (unsigned long) *(gbuf + ix + 1 + iy * dxs) & 0377 ;
         ired   = (unsigned long) *(rbuf + ix + 2 + iy * dxs) & 0377 ;

         *(imgrgb + ix + iiy * dxs) = iblue << 16 | igreen << 8 | ired ;
         }
      }

   /*** Write RGB image to GL window. ***/

   RGBmode () ;
   gconfig () ;

   lrectwrite (0, 0, dxs-1, dys-1, imgrgb) ;

   return (0) ;
}
