/* grnt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a sequence of Grinnell byte images on the SGI IRIS screen
 * using GL library software.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 June 1991: use onemap color mode.
 * 18 September 1991: add mapoff argument to glimg_view function.
 *  8 December 1992: use mouse instead of keyboard; add forward/backward seq.
 * 29 January 1993: adapt from 'imgw.c'.
 * 15 September 1993: use memory for image storage.
 * 12 October 1993: add scndir.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/img_scale.h"
#define MAXIMG 80 

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv[];

{
   FILE *LIST;
   short val;

   static char answer [2];		/* Response to scaling question. */
   static char flist [MAXIMG][80];
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Name of file containing list of images. */
   static char nampgm [80];	/* Program file name. */
   static char pixtyp [12];	/* Pixel type: byte, short, or float. */
   static char scndir [4] = "rd";	/* Scan direction: ru, rd, lu, ld. */
   char *rindex();			/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   unsigned short glimg [MAXIMG][512*512];	/* GL window display buffer. */

   int dx, dy;			/* Image dimensions. */
   int findex = 0;		/* File index. */
   int i;				/* Index. */
   int ierr;			/* Error status. */
   int lastimg;			/* Index of last image. */
   int mapoff = 512;		/* Use color map #2. */
   int nbyhdr = 0;		/* header size (bytes). */
   int nbypx;			/* # bytes/pixel. */
   int hdrsiz = 0;			/* # bytes to skip at BOF. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 * =3 continuous loop
				 */
   int xll, yll;		/* lower left corner window coordinates. */
   int xur, yur;		/* upper right corner window coordinates. */
   int nx = 512, ny = 512;	/* width & height of image in pixels. */

   int iarg;
   int narg;
   int imgdelay = 0;
   int seqdelay = 0;
   int showname = 0;

   float oldmin = 0.0;
   float oldmax = 255.0;
   float newmin = 0.0;
   float newmax = 255.0;
   float wait;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);
   strcpy (pixtyp, "byte");		/* Default pixel type: byte. */
   hdrsiz = 0;				/* Default hdrsiz: 0. */
   dx = dy = 512;			/* Default dimensions: 512x512. */
   narg = argc;
   iarg = 0;

   /*** Read  filename/listname from command line. ***/

   if (argc == 2)
      {
      strcpy (namimg, argv [1]);
      }

   /*** Default: give usage syntax. ***/

   else
   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-dimg delay] [-dseq delay] [-p scan_direction: ru,rd,lu,ld] [-s oldmin oldmax newmin newmax] [-n] {filename | {@|^|%%}listname}\n",
               argv [0]);
      exit (-1);
      }
   else
   while (iarg++ < narg)
      {
      printf ("narg:%3d   iarg:%3d   argv: %s\n", narg, iarg, argv [iarg]);
      if (strcmp (argv [iarg], "-dimg") == 0)
         {
         sscanf (argv [++iarg], "%d", &imgdelay);
         }
      else
      if (strcmp (argv [iarg], "-dseq") == 0)
         {
         sscanf (argv [++iarg], "%d", &seqdelay);
         }
      else
      if (strcmp (argv [iarg], "-p") == 0)
	 {
	 strcpy (scndir, argv [++iarg]);
	 }
      else
      if (strcmp (argv [iarg], "-s") == 0)
         {
         sscanf (argv [++iarg], "%f", &oldmin);
         sscanf (argv [++iarg], "%f", &oldmax);
         sscanf (argv [++iarg], "%f", &newmin);
         sscanf (argv [++iarg], "%f", &newmax);
         }
      else
      if (strcmp (argv [iarg], "-n") == 0)
         {
         showname = 1;
         }
      else
      if (iarg == narg -1)
         {
         strcpy (namimg, argv [iarg++]);
         }
      }

   printf ("oldmin:%g  oldmax:%g  newmin:%g  newmax:%g\n",
           oldmin, oldmax, newmin, newmax);
   printf ("showname:%3d\n", showname);
   printf ("imgdelay:%3d   seqdelay:%3d\n", imgdelay, seqdelay);

   if (pixtyp [0] == 'b')  nbypx = 1;
   else
   if (pixtyp [0] == 's')  nbypx = 2;
   else
   if (pixtyp [0] == 'f')  nbypx = 4;
   else
      {
      fprintf (stderr, "Invalid pixel type; must be byte, short, or float.\n");
      exit (-2);
      }

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^' || namimg [0] == '%')
      {
      if (namimg [0] == '@')	/* forward/backward via mouse control. */
         seq = 1;
      else
      if (namimg [0] == '^')	/* circular loop via mouse control. */
	 seq = 2;
      else
      if (namimg [0] == '%')	/* continuous loop; ESC to quit. */
	 seq = 3;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }

      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
	 fprintf (stdout, "Loading image: %s into display buffer.\n", namimg);
         loadglbuf (&glimg [i][0], nbyhdr, nx, ny, nbypx, mapoff,
		    scndir, namimg);
         }
      lastimg = i;
      fclose (LIST);
      }

   /*** Initialize GL software. ***/

   if (dx <= 1270)  xur = 1270; else xur = 1279;
   if (dy <=  995)  yur =  995; else yur = 1023;
   xll = xur - dx + 1;
   yll = yur - dy + 1;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n");
      exit (-1);
      }

   foreground ();
   fminit ();				/* Initialize SGI font manager. */
   prefposition (xll, xur, yll, yur);
   winopen (nampgm);
   gl_wclear (BLACK);

   doublebuffer ();		/* Use double buffer mode. */
   gconfig ();

   /*** Queue devices. ***/

   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq > 0 && seq < 3)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left mouse'   to go forward.\n");
      fprintf (stderr, "click `middle mouse' to go backward.\n");
      }
   fprintf (stderr, "type  `ESC'          to quit.\n");

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]);

   /*** Invoke rectwrite to transfer image into frame buffer FB. ***/

   rectwrite (0, 0, (short) (nx-1), (short)(ny-1), &glimg [findex][0]);

   swapbuffers ();

   if (seq == 3)
      {
      findex++;

      if (findex == 1)
         sleep (seqdelay);

/*       sleep (imgdelay);  */
       for (i = 0; i < imgdelay*10000; i++)		/* Delay loop. */
	  wait = sqrt ((double) i * (double) i);

       if (qtest ())
	  {
	  switch (qread (&val))
	     {
	     case ESCKEY:
             case WINCLOSE:
             case WINQUIT:
	     			gexit ();  exit (0);
	     default:		break;
	     }
	  }
       if (findex > lastimg)
	  {
/*	  sleep (seqdelay);  */
          gl_wclear (BLACK);
	  findex = 0;
	  }
       goto loop;
       }

   fprintf (stderr, "%s > ", namimg);

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               fprintf (stderr, "\n");
               goto loop;
               }
            break;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               fprintf (stderr, "\n");
               goto loop;
               }
            break;

         case REDRAW:
            rectwrite (0, 0, (short) (nx-1), (short)(ny-1), &glimg [findex][0]);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

}
