/* imgt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a sequence of byte images on the SGI IRIS screen
 * using GL library software.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 June 1991: use onemap color mode.
 * 18 September 1991: add mapoff argument to glimg_view function.
 *  8 December 1992: use mouse instead of keyboard; add forward/backward seq.
 * 29 January 1993: adapt from 'imgw.c'.
 * 15 September 1993: use memory for image storage.
 * 12 October 1993: add scndir.
 *  9 May 1995: add nbyhdr and MAXBUF.
 * 24 May 1995: add pmode.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/img_scale.h"
#define MAXBUF 24000000
#define MAXIMG 100
#define MAXX 1280 
#define MAXY 1024

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc ;
char *argv [] ;

{
   extern char *malloc () ;
   FILE *LIST ;
   short val ;

   static char answer [2] ;		/* Response to scaling question. */
   static char flist [MAXIMG][80] ;
   static char namimg [80] ;	/* Image file name. */
   static char namlst [80] ;	/* Name of file containing list of images. */
   static char nampgm [80] ;	/* Program file name. */
   static char pixtyp [12] ;	/* Pixel type: byte, short, or float. */
   static char scndir [4] = "ru" ;	/* Scan direction: ru, rd, lu, ld. */
   char *rindex () ;			/* Index function. */
   char *it ;			/* File name pointer. */
   char *parg ;			/* Argument pointer. */

   unsigned short *glimg ;	/* Pointer to GL window display buffer. */
/*   unsigned short glbuf [MAXIMG][MAXX*MAXY] ; */

   int findex = 0 ;		/* File index. */
   int i ;				/* Index. */
   int ierr ;			/* Error status. */
   int lastimg ;			/* Index of last image. */
   int mapoff = 512 ;		/* Use color map #2. */
   int nbyhdr = 0 ;		/* header size (bytes) to skip at BOF. */
   int nbypx ;			/* # bytes/pixel. */
   int nbyimg ;			/* # bytes/image. */
   int nbybuf ;			/* # bytes in display buffer. */
   int seq = 0 ;		/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */
   int xll, yll ;		/* lower left corner window coordinates. */
   int xur, yur ;		/* upper right corner window coordinates. */
   int nx = MAXX, ny = MAXY ;	/* width & height of image in pixels. */

   int iarg ;
   int narg ;
   int imgdelay = 0 ;
   int pmode = 0 ;
   int seqdelay = 0 ;
   int showname = 0 ;

   float oldmin = 0.0 ;
   float oldmax = 255.0 ;
   float newmin = 0.0 ;
   float newmax = 255.0 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   if (pmode == 1)
      {
      fprintf (stdout, "debug 1\n") ;
      fflush (stdout) ;
      }
   strcpy (nampgm, argv [0]) ;
   strcpy (pixtyp, "byte") ;		/* Default pixel type: byte. */
   narg = argc ;
   iarg = 0 ;

   /*** Default: give usage syntax. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-size xdim ydim] [-h header_size] [-dimg delay] [-dseq delay] [-p scan_direction: ru,rd,lu,ld] [-s oldmin oldmax newmin newmax] [-n] {filename | {@|^|%%}listname}\n",
               argv [0]) ;
      exit (-1) ;
      }

   /*** Read  filename/listname from command line. ***/

   else
   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }

   else
   while (iarg++ < narg)
      {
      if (pmode == 1)
         fprintf (stdout, "narg:%3d   iarg:%3d   argv: %s\n",
                  narg, iarg, argv [iarg]) ;
      if (strcmp (argv [iarg], "-size") == 0)
         {
         sscanf (argv [++iarg], "%d", &nx) ;
         sscanf (argv [++iarg], "%d", &ny) ;
         }
      else
      if (strcmp (argv [iarg], "-h") == 0)
         {
         sscanf (argv [++iarg], "%d", &nbyhdr) ;
         }
      else
      if (strcmp (argv [iarg], "-dimg") == 0)
         {
         sscanf (argv [++iarg], "%d", &imgdelay) ;
         }
      else
      if (strcmp (argv [iarg], "-dseq") == 0)
         {
         sscanf (argv [++iarg], "%d", &seqdelay) ;
         }
      else
      if (strcmp (argv [iarg], "-p") == 0)
	 {
	 strcpy (scndir, argv [++iarg]) ;
	 }
      else
      if (strcmp (argv [iarg], "-s") == 0)
         {
         sscanf (argv [++iarg], "%f", &oldmin) ;
         sscanf (argv [++iarg], "%f", &oldmax) ;
         sscanf (argv [++iarg], "%f", &newmin) ;
         sscanf (argv [++iarg], "%f", &newmax) ;
         }
      else
      if (strcmp (argv [iarg], "-n") == 0)
         {
         showname = 1 ;
         }
      else
      if (iarg == narg -1)
         {
         strcpy (namimg, argv [iarg++]) ;
         }
      }

   if (pmode == 1)
      {
      fprintf (stdout, "oldmin:%g  oldmax:%g  newmin:%g  newmax:%g\n",
               oldmin, oldmax, newmin, newmax) ;
       fprintf (stdout, "showname:%3d\n", showname) ;
       fprintf (stdout, "imgdelay:%3d   seqdelay:%3d\n", imgdelay, seqdelay) ;
       }

   if (pixtyp [0] == 'b')  nbypx = 1 ;
   else
   if (pixtyp [0] == 's')  nbypx = 2 ;
   else
   if (pixtyp [0] == 'f')  nbypx = 4 ;
   else
      {
      fprintf (stderr, "Invalid pixel type ; must be byte, short, or float.\n");
      exit (-2) ;
      }

   if (nx * ny > MAXX * MAXY)
      {
      fprintf (stderr, "Largest image  supported is MAXXxMAXY.\n") ;
      exit (-1) ;
      }

   nbyimg = nx * ny * nbypx ;
/*
 *   glimg = (unsigned short *) glbuf ;
 */
   glimg = (unsigned short *) malloc (nbyimg * MAXIMG) ;

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^' || namimg [0] == '%')
      {
      if (namimg [0] == '@')	/* forward/backward via mouse control. */
         seq = 1 ;
      else
      if (namimg [0] == '^')	/* circular loop via mouse control. */
	 seq = 2 ;
      else
      if (namimg [0] == '%')	/* continuous loop ; ESC to quit. */
	 seq = 3 ;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst) ;
         exit (-2) ;
         }
      fprintf (stderr, "Please wait for images to be loaded into memory.\n");

      i = -1 ;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++ ;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n",
		     i+1) ;
	    exit (-1) ;
	    }
	 nbybuf = (i+1) * nbyimg ;
	 if (nbybuf > MAXBUF)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n",
		     i+1) ;
	    exit (-2) ;
	    }
         strcpy (&flist [i][0], namimg) ;
         if (pmode == 1)
	    fprintf (stdout, "Loading image: %s into display buffer.\n",
		     namimg) ;
         loadglbuf (glimg + i * nbyimg, nbyhdr, nx, ny, nbypx, mapoff,
		    scndir, namimg, pmode) ;
         }
      lastimg = i ;
      fclose (LIST) ;
      }

   /*** Initialize GL software. ***/

   if (nx <= 1270)  xur = 1270 ; else xur = 1279;
   if (ny <=  995)  yur =  995 ; else yur = 1023;
   xll = xur - nx + 1 ;
   yll = yur - ny + 1 ;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n") ;
      exit (-1) ;
      }

   foreground () ;
   fminit () ;				/* Initialize SGI font manager. */
   prefposition (xll, xur, yll, yur) ;
   winopen (nampgm) ;
   gl_wclear (BLACK) ;

   doublebuffer () ;		/* Use double buffer mode. */
   gconfig () ;

   /*** Queue devices. ***/

   qdevice (WINCLOSE) ;
   qdevice (WINQUIT) ;
   qdevice (ESCKEY) ;
   qdevice (LEFTMOUSE) ;
   qdevice (MIDDLEMOUSE) ;

   fprintf (stderr, "Position pointer in image window using mouse.\n") ;
   if (seq > 0 && seq < 3)
      {
      fprintf (stderr, "After each image is displayed,\n") ;
      fprintf (stderr, "click `left mouse'   to go forward.\n") ;
      fprintf (stderr, "click `middle mouse' to go backward.\n") ;
      }
   fprintf (stderr, "type  `ESC'          to quit.\n") ;

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]) ;

   /*** Invoke rectwrite to transfer image into frame buffer FB. ***/

   rectwrite (0, 0, (short) (nx-1), (short)(ny-1), glimg + findex * nbyimg) ;

   swapbuffers () ;

    if (seq == 3)
       {
       findex++ ;
       sleep (imgdelay) ; 
       if (qtest ())
	  {
	  switch (qread (&val))
	     {
	     case ESCKEY:
             case WINCLOSE:
             case WINQUIT:
	     			gexit () ;  exit (0);
	     default:		break ;
	     }
	  }
       if (findex > lastimg)
	  {
	  sleep (seqdelay) ; 
          gl_wclear (BLACK) ;
	  findex = 0 ;
	  }
       goto loop ;
       }

   fprintf (stderr, "%s > ", namimg) ;

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++ ;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.") ;
                     findex = lastimg ;
		     }
                  else
		  if (seq == 2)
		     findex = 0 ;
                  }
               fprintf (stderr, "\n") ;
               goto loop ;
               }
            break ;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex-- ;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.") ;
                     findex = 0 ;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg ;
                  }
               fprintf (stderr, "\n") ;
               goto loop ;
               }
            break ;

         case REDRAW:
            rectwrite (0, 0, (short) (nx-1), (short)(ny-1),
		       glimg + findex * nbyimg) ;
            break ;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit () ;
               exit (0) ;
               }
         }
      qreset () ;
      }
   }

}
