/* imgwc.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a pic (WHIP) image on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 * 10 October 1991: add ability to read cursor position.
 * 23 October 1991: read mouse position only when left button is pressed.
 * 29 October 1991: write all information into output file.
 * 18 February 1992: adapt from `cpwc.c'.
 * 31 March 1994: add dir2 argument for glimg_view.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/usr/people/smm/lib/src/gl/img_scale.h"

#include <stdio.h>
#include "/usr/whip/h/whip.h"
#include <fmclient.h>
#include <gl/gl.h>
#include <gl/device.h>
#define XY 2
#define X 0
#define Y 1
int DSP = 1;
pichdr hdr;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   FILE *fpicopen ();		/* Pic image open function. */
   int glimg_view ();		/* Image display function. */

   static char dir2 [4] = "rd";	/* Scan direction: right, down. */
   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char nampos [80];	/* Position measurement file name. */
   static char answer [2];	/* Response to scaling question. */

   static unsigned short cursor [128];
   static Cursor plus = {0x0100, 0x0100, 0x0100, 0x0100,
                         0x0100, 0x0100, 0x0100, 0xFEFE,
                         0x0100, 0x0100, 0x0100, 0x0100,
                         0x0100, 0x0100, 0x0100, 0x0000};
 
   static Cursor plus1 = {0x0100, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0380, 0xFEFE,
                          0x0380, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0100, 0x0000};
   static long poly [100][2];
   int ip = 0;
 
   char *rindex();		/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int hdrsiz;			/* Header size. */
   int i;			/* Index. */
   int ierr;			/* Error status from glimg_view (). */
   int larg;			/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int measure = 0;		/* Counter for # of measurements. */
   int nbypx;			/* # bytes per pixel. */
   int nx, ny;			/* Image size. */
   int xoimg = 0, yoimg = 0;	/* C/P image origin in window coordinates. */
   int xll, xur, yll, yur;	/* Window corner coordinates. */

   static Device mdev [XY];
   static short mval [XY];
   short val;
   int boxsize = 3;		/* size of box to indicate cursor points. */
   int dev;
   int mapoff = 512;		/* Colormap offset index. */
   long xo, yo;			/* x/y origin of current window */
   long xs, ys;			/* x/y size of current window   */
   int xscrnsize;		/* size of screen in x used to set globals */
   int xmaxscrn, ymaxscrn;	/* maximum size of screen in x and y       */
   int xwin, ywin;		/* x/y window coordinates. */
   Colorindex dummy;		/* for strict ANSI C prototyping */

   float ximg, yimg;		/* x/y coordinates for image file. */
   float r = 0.0, th = 0.0;	/* radius & position angle for image. */

   int fdpic;
   FILE *PIC, *fdpos;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   xscrnsize = getgdesc (GD_XPMAX);
   if (xscrnsize == 1280)
      {
      xmaxscrn = 1279;
      ymaxscrn = 1023;
      }
   else if (xscrnsize = 1023)
      {
      xmaxscrn = 1023;
      ymaxscrn = 767;
      }
   else
      {
      fprintf(stderr, "Something's EXTREMELY wrong:  ");
      fprintf(stderr, "xscrnsize=%d\n", xscrnsize);
      exit (-1) ;
      }

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Usage: %s [filename]\n", argv [0]);

      fprintf (stderr, "Enter image file name: ");
      fscanf  (stdin,  "%s", namimg);
      }

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      lscale = 0;
      strcpy (namimg, argv[1]);
      }
        
   /*** Read PIC header. ***/

   fdpic = open (namimg, 0);
   read (fdpic, hdr, 24);
   close (fdpic);
/*   PIC = fpicopen (namimg, hdr, 0); */		/* Open pic image. */
/*   fclose (PIC); */
   nbypx = hdr.bp / 8;
   nx = hdr.cols;
   ny = hdr.rows;
   hdrsiz = sizeof (hdr) +  hdr.lcom;

   strcpy (nampos, namimg);
   strcat (nampos, ".pos");
   if ((fdpos = fopen (nampos, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", nampos);
      exit (-3);
      }
   else
      {
      fprintf (stderr, "Cursor measurement file: %s\n", nampos);
      }

   /*** Initialize GL software. ***/

   if (nx <= 1270)  xur = 1270; else xur = 1279;
   if (ny <=  995)  yur =  995; else yur = 1023;
   xll = xur - nx + 1;
   yll = yur - ny + 1;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n");
      exit (-1);
      }

   fminit ();				/* Initialize SGI font manager. */
   prefposition (xll, xur, yll, yur);
   winopen (namimg);		/* Create GL window. */
   gl_wclear (BLACK);

   getsize (&xs, &ys);		/* Get window size. */
   getorigin (&xo, &yo);	/* Get window origin (lower left corner). */
   qdevice (ESCKEY);
   qdevice (ZEROKEY);
   qdevice (ONEKEY);
   qdevice (TWOKEY);
   qdevice (THREEKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   mdev [X] = MOUSEX;
   mdev [Y] = MOUSEY;

   curstype (CCROSS);			/* Crosshair cursor. */
   defcursor (1, cursor);
   curorigin (1,0,0);

   curstype (C16X1);			/* Small crosshair cursor. */
   defcursor (2, plus1);
   curorigin (2,7,7);

   curstype (C16X1);			/* Modified small crosshair cursor. */
   defcursor (3, plus1);
   curorigin (3,7,7);

   setcursor (1, dummy, dummy);

   gl_wclear (BLACK);		/* Clear window to black. */

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);

   if ((ierr = glimg_view (namimg, hdrsiz, nbypx, nx, ny, dir2, mapoff)) < 0)
      {
      fprintf (stderr, "glimg_view error: %d\n", ierr);
      exit (-1);
      }
   fprintf (stdout,  "xwin ywin  ximg  yimg    r      th\n");
   fprintf (fdpos,   "%s : cursor positions\n", namimg);
   fprintf (fdpos,   "xwin  ywin  ximg  yimg    r      th\n");

   while (1)
      {
      while (qtest ())
         {
         dev = qread (&val);
         switch (dev)
            {
            case ESCKEY:	/* Terminate program. */
                 setvaluator (MOUSEX, (short)(xs/2), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(ys/2), 0, ymaxscrn);
                 fclose (fdpos);
                 if (measure == 0)  unlink (nampos);
                 gexit ();
                 exit (0);
                 break;

            case REDRAW:	/* Clear window to black. */
                 reshapeviewport ();
                 getsize (&xs, &ys);
                 getorigin (&xo, &yo);
                 color (BLACK);
                 clear ();
                 break;

            case ZEROKEY:	/* Fill polygon. */
                 polyfill (poly, ip, BLUE);
                 break;

            case ONEKEY:	/* cursor associated with 1-key. */
                 setcursor (1, dummy, dummy);
                 setvaluator (MOUSEX, (short)(xo+(0.5*xs)), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(yo+(0.5*ys)), 0, ymaxscrn);
                 break;

            case TWOKEY:	/* cursor associated with 2-key. */
                 setcursor (2, dummy, dummy);
                 setvaluator (MOUSEX, (short)(xo+(0.5*xs)), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(yo+(0.5*ys)), 0, ymaxscrn);
                 break;

            case THREEKEY:	/* cursor associated with 3-key. */
                 setcursor (3, dummy, dummy);
                 setvaluator (MOUSEX, (short)(xo+(0.5*xs)), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(yo+(0.5*ys)), 0, ymaxscrn);
                 break;

            case LEFTMOUSE:	/* Left Mouse Button. */
                 measure++;
                 if (val == 1)	/* Read position when left key depressed. */
                    {
                    getdev (XY, mdev, mval);
                    xwin = mval [X] - xo;		/* x window position. */
                    ywin = mval [Y] - yo;		/* y window position. */
                    ximg = (float) (xwin - xoimg); 	/* x image position. */
                    yimg = (float) (yoimg - ywin); 	/* y image position. */
/*                    rcoord (&r, &th, &ximg, &yimg, -1, scroll);*//* Get r & th.*/
                    fprintf (stdout, " %3d  %3d  %3.0f  %3.0f  %5.2f  %6.2f\n",
                             xwin, ywin, ximg, yimg, r, th);
/*
 *                    fprintf (fdpos, "%3d %3d\n", xwin, ywin);
 */
                    fprintf (fdpos,  " %3d   %3d   %3.0f   %3.0f  %5.2f  %6.2f\n",
                             xwin, ywin, ximg, yimg, r, th);

                    wglbox (xwin, ywin, boxsize, RED);	/* Draw small box. */
                    poly [ip][0] = xwin;
                    poly [ip][1] = ywin;
                    ip++;
                    }
                 break;

            default:
                 break;
            }
         }
      }
}


wglbox (xw, yw, size, cval)

int xw, yw;		/* x/y window position (pixel units). */
int size;		/* box size (pixel units). */
int cval;		/* color map index. */

{
   static long vlb [2];
   static long vrb [2];
   static long vrt [2];
   static long vlt [2];
   int xl, xr;
   int yb, yt;

   color (cval);

   xl = xw - size/2;
   xr = xl + size - 1;
   yb = yw - size/2;
   yt = yb + size - 1;

   vlb [0] = xl;
   vlb [1] = yb;
   vrb [0] = xr;
   vrb [1] = yb;
   vrt [0] = xr;
   vrt [1] = yt;
   vlt [0] = xl;
   vlt [1] = yt;

   bgnpolygon ();
      v2i (vlb);
      v2i (vrb);
      v2i (vrt);
      v2i (vlt);
   endpolygon ();

   return (0);
}


polyfill (poly, nval, cmap)

long poly [][2];
int  nval;
int  cmap;

{
   int i;

   color (cmap);

   bgnpolygon ();
      for (i=0; i<nval; i++)  v2i (poly [i]);
      v2i (poly [0]);
   endpolygon ();

   return (0);
}
