/* loadglbuf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Load image into display buffer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 15 September 1993
 * 12 October 1993: add scndir.
 * 24 May 1995: add pmode.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

loadglbuf (imgbuf, nbyhdr, nx, ny, nbypx, mapoff, scndir, namimg, pmode)

unsigned short *imgbuf;			/* GL image buffer. */
int nbyhdr;				/* # bytes to skip for header. */
int nx, ny;				/* Image width & height (pixels). */
int nbypx;				/* # bytes per pixel. */
int mapoff;				/* colormap offset. */
char scndir [];				/* scan direction: ru,rd,lu,ld. */
char *namimg;				/* Image filename. */
int pmode;				/* Print mode. */
{
   unsigned char *cbuf;
   unsigned short *sbuf;
   float *fbuf;
   FILE *IMG;

   unsigned short sval;
   int ix, iy;
   int iix, iiy;
   int left, down;
   int nbyimg;
   int nelem;

   if ((IMG = fopen (namimg, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg);
      return (-1);
      }

   /*** Set scan direction flags. ***/

   left = -1;
   down = -1;
   if (scndir [0] == 'l')  left = 1;
   if (scndir [0] == 'r')  left = 0;
   if (scndir [1] == 'd')  down = 1;
   if (scndir [1] == 'u')  down = 0;
   if (left == -1 || down == -1)
      {
      fprintf (stderr, "Scan direction: %s is not supported.\n", scndir);
      exit (-1);
      }
   if (pmode == 1)
      {
      fprintf (stdout, "loadglbuf> scndir: %s\n", scndir);
      if (down)  printf ("down = %d  (TRUE)\n", down);
      }

   /*** Skip over header, if any. ***/

   if (nbyhdr > 0)
      fseek (IMG, nbyhdr, SEEK_SET);

   nelem = (int) (nx * ny);
   nbyimg = (int) (nelem * nbypx);

   if (nbypx == 1)
      {
      cbuf = (unsigned char *) malloc (nbyimg);
      fread (cbuf, sizeof (char), nelem, IMG);
      for (iy = 0; iy < ny; iy++)
	 {
	 if (down)  iiy = ny - iy - 1;
	 else       iiy = iy;
         for (ix = 0; ix < nx; ix++)
	    {
	    if (left) iix = nx - ix - 1;
	    else      iix = ix;
            *(imgbuf + ix + iy*nx) =
             (unsigned short) *(cbuf + iix + iiy * nx) + mapoff;
            }
         }
      }

   else
   if (nbypx == 2)
      {
      sbuf = (unsigned short *) malloc (nbyimg);
      fread (sbuf, sizeof (short), nelem, IMG);
      for (iy = 0; iy < ny; iy++)
	 {
	 if (down)  iiy = ny - iy - 1;
	 else       iiy = iy;
         for (ix = 0; ix < nx; ix++)
            {
	    if (left) iix = nx - ix - 1;
	    else      iix = ix;
            sval = (unsigned short) *(sbuf + iix + iiy * nx);
            if (sval > 255) sval = 255;
            *(imgbuf + ix + iy*nx) = sval + mapoff;
            }
	 }
      }

   else
   if (nbypx == 4)
      {
      fbuf = (float *) malloc (nbyimg);
      fread (fbuf, sizeof (float), nelem, IMG);
      for (iy = 0; iy < ny; iy++)
	 {
	 if (down)  iiy = ny - iy - 1;
	 else       iiy = iy;
         for (ix = 0; ix < nx; ix++)
            {
	    if (left) iix = nx - ix - 1;
	    else      iix = ix;
            sval = (unsigned short) *(fbuf + iix + iiy* nx);
            if (sval > 255) sval = 255;
            *(imgbuf + ix + iy*nx) = sval + mapoff;
            }
	 }
      }

   return (0);
}
