/* glmk3_label.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label MLSO Mark-III image in SGI/IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   10 January 1994
 * 8 July 1994: no colorbar drawn if xdim < 256.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <fmclient.h>
#include "/home/stanger/lib/include/mk3lib.ext"
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/gl_color.h"

glmk3_label (lfull, lbottom, cval2, cval3, cval4)

int lfull;	/* Flag for full redrawing of labels. */
int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int cval2;	/* Color level for labels. */
int cval3;	/* Color level for HAO logo. */
int cval4;	/* Color level for miscellaneous labels. */

{
   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    *
    * `Times-Bold' font sizes (IRIX 5.2)
    *  6 =>  9x10
    *  8 => 11x13
    * 10 => 14x16
    * 12 => 17x19
    * 14 => 19x19
    * 16 => 22x22
    * 18 => 27x28
    * 20 => 26x31
    * 24 => 35x37
    * 28 => 40x44
    */
   extern fmfonthandle gl_setfont ();
   fmfonthandle afh, bfh, cfh;
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
   static char font5 [] = "Times-Bold";
   static char afont [32];			/* Small font. */
   static char bfont [32];			/* Medium font. */
   static char cfont [32];			/* Large font. */
  
   static char logo [] = " HAO/MLSO MARK-III K-CORONAMETER";
   static char pixval [12];		/* Pixel value mode string. */
   static short *cbar;			/* Color bar array. */

   int asize, bsize, csize;		/* Font point sizes. */
   int afx, afy, bfx, bfy, cfx, cfy;	/* Font x/y sizes. */
   int afx15;

   int i;
   int lxstr;
   int pxstr = 64;
   int xempty;
   int xlogo;
   int ylogo;
   int begrow, begcol, endrow, endcol, ix, iy;
   int step;

   int colormax = 255;			/* Maximum color value. */
   int barwidth = 512, barheight = 12;	/* color bar dimensions. */
   int ppc = 2;		/* # pixels per color in color bar. */
   int cby0, cby1;	/* color bar y-axis locations. */
   int startcol = 64;

   int xs, ys;
   int x1 = 0, x2 = gMINCOL - 1, x3 = gMAXCOL + 1;
   int y1 = 0, y2 = gMINROW - 1, y3 = gMAXROW + 1;
   int xwl, xil, xwr;
   int ywb, yib, ywt;

   xs = 2;
   strcpy (afont, font1);
   asize = 7;
/*   strcpy (bfont, font4); */
   strcpy (bfont, font1);
/*   bsize = 10; */
   bsize = 11 ;
/*   strcpy (cfont, font4); */
   strcpy (cfont, font1);
/*   csize = 12; */
   csize = 13 ;
   afx =  6;   afy =  7;
/*   bfx = 14;   bfy = 15; */
   bfx =  9;   bfy = 15;
/*   cfx = 16;   cfy = 17; */
   cfx = 10;   cfy = 17;
   xwl = x1;  xil = x2;  xwr = x3;
   ywb = y1;  yib = y2;  ywt = y3;

   if (MK3I->xdim < 256)
      ppc = 0 ;
   else
      ppc = MK3I->xdim / 256 ;

   barwidth = ppc * 256 ;
   barheight =  12;
   ys = ywt - afy - 2;
   begcol = 64;
   endcol = begcol + barwidth - 1;

   /*** Draw box around image window. ***/

   gl_wvec (xwl, yib, xwr, yib, cval2);
   gl_wvec (xwr, yib, xwr, ywt, cval2);
   gl_wvec (xwr, ywt, xwl, ywt, cval2);
   gl_wvec (xwl, ywt, xwl, yib, cval2);
   gl_wvec (xil, yib, xil, ywt, cval2);
   gl_wvec (xwl, ys-4*afy-10*bfy, xil, ys-4*afy-10*bfy, cval2); 

   /*** Write annotation labels into frame buffer. ***/

   afh = gl_setfont (afont, asize);	/* Small font & size. */
   gl_wstr ("DATE",		xs, ys,                   cval2);
   gl_wstr ("YEAR:DOY",		xs, ys -  2*afy -  3*bfy, cval2);
   gl_wstr ("TIME",		xs, ys -  4*afy -  6*bfy, cval2);

   gl_wstr ("TYPE",		xs, ys -  6*afy - 10*bfy, cval2);
   gl_wstr ("CHANNEL",		xs, ys -  7*afy - 12*bfy, cval2);
   gl_wstr ("DATA RANGE",	xs, ys -  8*afy - 14*bfy, cval2);
   gl_wstr ("DISP RANGE",	xs, ys -  9*afy - 17*bfy, cval2);
   gl_wstr ("SCAN DIR",		xs, ys - 10*afy - 20*bfy, cval2);

   gl_wstr ("SCAN DUR",		xs, ys - 11*afy - 22*bfy, cval2);

   gl_wstr ("PIXEL/Rsun",	xs, ys - 12*afy - 24*bfy, cval2);

   gl_wstr ("P-ANGLE",		xs, ys - 13*afy - 26*bfy, cval2);

   /*** Write HAO logo & cval2 bar only if 'lbottom = 0'. ***/

   if (lbottom == 1)  return (0);

   /*** Add annotation for the color bar ***/

   if (ppc > 0)
      {
      gl_setfont (afont, asize);	/* Small font & size. */
      cby0 = 1;
      afx15 = (int) ((afx * 1.5) + 0.5);
      gl_wstr ("0"  , (startcol),                 cby0, cval2);
      gl_wstr ("64" , (startcol + 64 *ppc-afx),   cby0, cval2);
      gl_wstr ("128", (startcol + 128*ppc-afx15), cby0, cval2);
      gl_wstr ("192", (startcol + 192*ppc-afx15), cby0, cval2);
      gl_wstr ("255", (endcol-afx*3),             cby0, cval2);

      /*** Add tick marks to color bar ***/

      cby0 += afy + 2;
      cby1 = cby0 + 4;
      gl_wvec (startcol,         cby0, (startcol),         cby1, cval2);
      gl_wvec (startcol+32*ppc , cby0, (startcol+32*ppc),  cby1, cval2);
      gl_wvec (startcol+64*ppc , cby0, (startcol+64*ppc),  cby1, cval2);
      gl_wvec (startcol+96*ppc , cby0, (startcol+96*ppc),  cby1, cval2);
      gl_wvec (startcol+128*ppc, cby0, (startcol+128*ppc), cby1, cval2);
      gl_wvec (startcol+160*ppc, cby0, (startcol+160*ppc), cby1, cval2);
      gl_wvec (startcol+192*ppc, cby0, (startcol+192*ppc), cby1, cval2);
      gl_wvec (startcol+224*ppc, cby0, (startcol+224*ppc), cby1, cval2);
      gl_wvec (startcol+255*ppc, cby0, (startcol+255*ppc), cby1, cval2);

      /*** Draw the color bar. ***/

      barwidth = (colormax + 1) * ppc;
      cbar = (short *) malloc (barwidth * barheight * sizeof (short));

      for (iy = 0; iy < barheight; iy++)
         for (step = 0; step <= colormax; step++)
            for (ix = 0; ix < ppc; ix++)
               *(cbar + (step*ppc+ix) + (iy*barwidth)) =
                (short) (step & 0377) + cmapoff;

      begrow = cby1 + 1;
      endrow = begrow + barheight - 1;
      rectwrite (begcol, begrow, endcol, endrow, cbar);
      free (cbar);
      }

   /*** Write HAO/MK3 logo. ***/

   cfh = gl_setfont (cfont, csize);	/* Large font & size. */
   lxstr = fmgetstrwidth (cfh, logo);

   xempty = (xwr - xwl) - pxstr - lxstr;
   if (xempty > 0)  xlogo = xs + pxstr + xempty/2;
   ylogo = endrow + 7;

   gl_wstr (logo,   xlogo, ylogo, cval3);

   /*** Write border around lower portion of window. ***/

   gl_wvec (xwl, yib, xwl, ywb, cval2);
   gl_wvec (xwl, ywb, xwr, ywb, cval2);
   gl_wvec (xwr, ywb, xwr, yib, cval2);

   return (0);
}
