/* mk3rgb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an MLSO Mark-III image on the SGI IRIS screen
 *    using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * Save image as an SGI/GL image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR    4 January 1994
 * 3 February 1994: Save image as an SGI/GL image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/img_scale.h"
#include "/home/stanger/lib/include/mk3lib.h"
int DSP = 1;
#define MAXIMG 250

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   FILE *LIST;
   short val;

   static char flist [MAXIMG][80];
   static char rawfile [80];
   static unsigned char rawbuf [576*576];
   static unsigned short glbuf [576*576];
   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Namelist file name. */
   static char answer [2];	/* Response to scaling question. */
   char *rindex ();		/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int glmk3_view ();		/* Image display function. */
   int fd;
   int findex = 0;
   int i;			/* Index. */
   int ians;			/* KB response. */
   int iarg = 0;		/* argument pointer. */
   int ierr;			/* Error status from glmk3_view(). */
   int larg;			/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int lastimg = 0;		/* Last image index value. */
   int narg;			/* # command line arguments. */
   int pmode = 0;		/* Print mode. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */
   int soff = 512;		/* color index offset. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);
   narg = argc;

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-s oldmin oldmax] [-d] [filename | @namelist | ^namelist]\n", argv [0]);

      fprintf (stderr, "Enter Mark-III image file name: ");
      fscanf  (stdin,  "%s", namimg);

      lscale = 0;
      fprintf (stderr, "Override default intensity scaling [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)

      ldisk = 0;
      fprintf (stderr, "Eliminate color bar & solar disk outline [y/n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp (answer, "y", 1) == 0 || strncmp (answer, "Y", 1) == 0)
      ldisk = 1;
      }

   /*** Single argument: must be filename. ***/

   else
   if (argc == 2)
      {
      ldisk  = 0;
      strcpy (namimg, argv [1]);
      }

   /*** More than 1 argument. ***/

   else
   while (iarg++ < narg)
      {
/*      printf ("narg:%3d   iarg:%3d   argv: %s\n", narg, iarg, argv [iarg]); */
      if (strcmp (argv [iarg], "-d") == 0)
         {
	 ldisk = 1;
         }
      else
      if (strcmp (argv [iarg], "-s") == 0)
         {
	 lscale = 1 ;
         sscanf (argv [++iarg], "%f", &foldmin);
         sscanf (argv [++iarg], "%f", &foldmax);
	 fnewmin  =   0.0 ;
	 fnewmax  = 249.0 ;
	 fclipmin =   0.0 ;
	 fclipmax = 249.0 ;
         oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
         newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
         clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
         }
      else
      if (iarg == narg -1)
         {
         strcpy (namimg, argv [iarg++]);
         }
      }

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^')
      {
      if (namimg [0] == '@')
         seq = 1;
      else
      if (namimg [0] == '^')
	 seq = 2;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
         }
      lastimg = i;
      fclose (LIST);
      }
        
   if (seq)
      mk3hskp (&flist [0][0], pmode);
   else
      mk3hskp (namimg, pmode);

   /*** Initialize GL software. ***/

   foreground ();
   prefposition (696, 1271, 416, 991);/* 576x576 window in upper right corner.*/
   fminit ();			/* Initialize SGI font manager. */

   winopen (argv [0]);		/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

   doublebuffer ();		/* Use double buffer mode. */
   gconfig ();

   /*** Declare devices. ***/

   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left mouse'   to go forward.\n");
      fprintf (stderr, "click `middle mouse' to go backward.\n");
      }
   fprintf (stderr, "type  `ESC'          to quit.\n");

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]);

   /*** Display image on IRIS screen in GL window. ***/

   if ((ierr = glmk3_view (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "glmk3_view error: %d\n", ierr);
      gexit ();
      exit (-2);
      }

   rectread (0, 0, 575, 575, glbuf);
   for (i = 0; i < 576*576; i++)
      {
      rawbuf [i] = (unsigned char) ((glbuf [i] - soff) & 0377);
/*
 *      if (i%32 == 0)
 *      printf ("i:%6d   glbuf: %3d   rawbuf: %3d\n", i, glbuf [i], rawbuf [i]);
 */
      }
   sprintf (rawfile, "%s.img", namimg);
   fd = creat (rawfile, 0644);
   write (fd, rawbuf, 576*576);
   close (fd);

   swapbuffers ();

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.\n");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               goto loop;
               }
            break;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.\n");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               goto loop;
               }
            break;

         case REDRAW:
            glmk3_view (namimg, linit, pmode);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

quit:
   gexit ();
   exit (0);
}
