/* mk3w.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an MLSO Mark-III image on the SGI IRIS screen
 *    using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   4 January 1994
 * 8 July 1994: re-open GL window if image size changes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/img_scale.h"
#include "/home/stanger/lib/include/mk3lib.h"
int DSP = 1;
#define MAXIMG 250

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   FILE *LIST;
   short val;

   static char flist [MAXIMG][80];
   static char nampgm [80];	/* Program file name. */
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Namelist file name. */
   static char answer [2];	/* Response to scaling question. */
   char *rindex ();		/* Index function. */
   char *it;			/* File name pointer. */
   char *parg;			/* Argument pointer. */

   int glmk3_view ();		/* Image display function. */
   int dx = 0 ;			/* X dimension. */
   int dy = 0 ;			/* Y dimension. */
   int findex = 0;
   int gwid = -1 ;		/* GL window ID. */
   int i;			/* Index. */
   int ians;			/* KB response. */
   int iarg = 0;		/* argument pointer. */
   int ierr;			/* Error status from glmk3_view(). */
   int larg;			/* Length of scaling/disk flag argument. */
   int linit = 1;		/* Image label flag [0=off,1=on]. */
   int lastimg = 0;		/* Last image index value. */
   int narg;			/* # command line arguments. */
   int odx = 0 ;		/* old X dimension. */
   int ody = 0 ;		/* old Y dimension. */
   int pmode = 0;		/* Print mode. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 */
   int xll, yll ;		/* lower left corner window coordinates. */
   int xur, yur ;		/* upper right corner window coordinates. */
   int xwmax, ywmax ;		/* Screen size. */
   int xwsize, ywsize ;		/* Window size. */
   int xbsize, ybsize ;		/* Anotation border size. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Set border size. ***/

   xbsize =  64 ;
   ybsize =  64 ;

   /*** Get maximum window size. ***/

   xwmax = getgdesc (GD_XPMAX) - xbsize ;
   ywmax = getgdesc (GD_YPMAX) - ybsize ;

   strcpy (nampgm, argv [0]);
   narg = argc;

   /*** Request information if not enough arguments are provided. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-s oldmin oldmax] [-d] [filename | @namelist | ^namelist]\n", argv [0]);
      exit (0) ;
      }

   /*** Single argument: must be filename. ***/

   else
   if (argc == 2)
      {
      ldisk  = 0;
      strcpy (namimg, argv [1]);
      }

   /*** More than 1 argument. ***/

   else
   while (iarg++ < narg)
      {
      printf ("narg:%3d   iarg:%3d   argv: %s\n", narg, iarg, argv [iarg]);
      if (strcmp (argv [iarg], "-d") == 0)
         {
	 ldisk = 1;
         }
      else
      if (strcmp (argv [iarg], "-s") == 0)
         {
	 lscale = 1 ;
         sscanf (argv [++iarg], "%f", &foldmin);
         sscanf (argv [++iarg], "%f", &foldmax);
	 fnewmin  =   0.0 ;
	 fnewmax  = 249.0 ;
	 fclipmin =   0.0 ;
	 fclipmax = 249.0 ;
         oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
         newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
         clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
         }
      else
      if (iarg == narg -1)
         {
         strcpy (namimg, argv [iarg++]);
         }
      }

   /*** Sequence of images ? ***/

   if (namimg [0] == '@' || namimg [0] == '^')
      {
      if (namimg [0] == '@')
         seq = 1;
      else
      if (namimg [0] == '^')
	 seq = 2;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = -1;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&flist [i][0], namimg);
         }
      lastimg = i;
      fclose (LIST);
      }
        
   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left   mouse' to go forward.\n");
      fprintf (stderr, "click `right  mouse' to go backward.\n");
      }
   fprintf (stderr, "type  `ESC'          to quit.\n");

loop:

   if (seq)
      strcpy (namimg, &flist [findex][0]);

   /*** Decode housekeeping. ***/

   mk3hskp (namimg, pmode);

   dx = MK3I->xdim ;
   dy = MK3I->ydim ;

   if (odx != dx || ody != dy)

      {
      foreground ();
      fminit () ;

      xwsize = dx + xbsize ;
      ywsize = dy + ybsize ;

      if (dx <= 1270)  xur = 1270 ; else xur = 1279 ;
      if (dy <=  995)  yur =  995 ; else yur = 1023 ;

      xll = xur - xwsize + 1 ;
      yll = yur - ywsize + 1 ;
      if (xll < 0 || yll < 0)
         {
         fprintf (stderr, "Image too large for screen\n") ;
         exit (-1) ;
         }
      if (gwid > 0)
         winclose (gwid) ;
      prefposition (xll, xur, yll, yur) ;
      gwid = winopen (nampgm) ;
      gl_wclear (BLACK) ;
/*      printf ("clear window\n") ; */

      doublebuffer ();		/* Use double buffer mode. */
      gconfig ();

      /*** Declare devices. ***/

      qdevice (WINCLOSE);
      qdevice (WINQUIT);
      qdevice (ESCKEY);
      qdevice (LEFTMOUSE);
      qdevice (MIDDLEMOUSE);
      qdevice (RIGHTMOUSE);
      }
   odx = dx ;
   ody = dy ;

   /*** Display image on IRIS screen in GL window. ***/

   if ((ierr = glmk3_view (namimg, linit, xbsize, ybsize, pmode)) < 0)
      {
      fprintf (stderr, "glmk3_view error: %d\n", ierr);
      gexit ();
      exit (-2);
      }

   swapbuffers ();

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.\n");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               goto loop;
               }
            break;

         case RIGHTMOUSE:
            if (val == 1 && seq > 0)	/* Right mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.\n");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               goto loop;
               }
            break;

         case MIDDLEMOUSE:		/* Middle mouse button is down. */
         case REDRAW:
            glmk3_view (namimg, linit, xbsize, ybsize, pmode);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

quit:
   gexit ();
   exit (0);
}
