;+
;       NAME: ar_prep
;
;
;       PURPOSE: To setup the inputs to RD_AR.
;
;
;       METHOD:  Input a reference image to use for getting location
;		 and reference time for RD_AR. Call RD_LONLAT to 
;		 interactively select the regions center. Calculate
;		 or query for other needed inputs.
;
;       CALLING SEQUENCE: ar_prep,image,index,lonlat,date,ar_siz
;					[,/close][,/table]
;
;
;       PARAMETERS: 	image	Reference image.
;			index   Index for same.
;	
;			lonlat  The longitude(0) and latitude(1).
;		 	date	the reference time.
;			ar_size The region size, [xs,ys], in FR pixels.
;
;       KEYWORDS:	region  The edges of the selected region in image
;				pixels.
;			close	If set window is closed before exitting.
;			table   If set allows option of setting the 
;				color table.
;
;	CALLS: NEW_WIN, TARGET, SXT_GRID,
;
;       HISTORY: Drafted by A.McAllister, 18-mar-93
;		 Swithed to RD_LONLAT, AMcA, 23-jun-93.
;		 Changed target call to /rough, AMcA, 23-jun-93.
;		 Set box in the correct place, AMcA, 29-jun-93.
;		 Fixed up handling of QR case, AMcA, 2-jul-93.
;-
PRO ar_prep,image,index,lonlat,dat0,ar_siz,region=region,close=close, $
					table=table

 imgsz=size(image)

 case gt_res(index) of

	0: factor=1	 	;full
	1: factor=1		;half
	2: factor=2		;quarter

 endcase

 xs=fix(imgsz(1)*factor)       
 ys=fix(imgsz(2)*factor)
 num1=new_win(xs,ys)           	     		;make new window

 trgt0=target(image,index,/rough)        	;find target first
 dat0=anytim2ex(index)

 tvscl,rebin(image,xs,ys)		     	;display image

 if keyword_set(table) then begin

   yesnox,'Change color table?',ans,'n'     	;change color table
   if ans then xloadct

 endif
                 
 sxt_grid,index,mag=2.0			       	;dislpay grid

 print,'Please select the center of the region. Last entry will be used.'

 trgt0=trgt0*factor
 lonlat=rd_lonlat(num1,image,target=trgt0,date=dat0,xy_cent=cent,/norefresh)

 pnt=n_elements(lonlat)/2-1			;take last entry

 input=' '                                      
 read, 'Enter the region size in FR pixels, [xs,ys]: ', input
 ar_siz=fix(str2arr(input, delim = ',')) 

 norm=ar_siz/(2^gt_res(index))/2*factor
 cent=cent(*,pnt)
 region=[cent(0)-norm(0),cent(0)+norm(0),cent(1)-norm(1),cent(1)+norm(1)]
 draw_boxcorn,region(0),region(2),region(1),region(3),/device
 
 if pnt ne 0 then lonlat=lonlat(0:1,pnt)

 if keyword_set(close) then wdelete,num1

end