;+
;       NAME: area_mean
;
;
;       PURPOSE: To produce a mean value pixel by pixel from a data cube
;
;
;       METHOD: The default is to sort the pixels in each column and then
;		take the mean one.
;		Alternatively a straight average can be done (IDL 3.0)
;		or gaussian fitting of histograms can be selected.
;
;
;       CALLING SEQUENCE:  mn_ar=area_mean(data,ss,[type=ave])
;
;
;       PARAMETERS:	data		input data cube
;			ss		ss string of pixels to be treated
;			mn_ar		array with results
;
;
;       KEYWORDS:	type		indicates the type of mean
;					default take mean
;					"ave":  straight average
;					"hist": use histograms
;
;       NOTES: The MODE_FIT option requires considerable time
;
;	CALLS: COORD_L2V, MODE_FIT
;
;       HISTORY: Drafted by A. McAllister 2-jun-93
;
;
;-
FUNCTION area_mean,data,ss,type=type

 mn_ar=data(*,*,0)
 mn_ar(*,*)=0.0
 xy=coord_l2v(ss,size(data(*,*,0)))

 if not keyword_set(type) then type='median'

 case type of

	'ave':  begin		;return the average in each column
		
		  imgs=float(n_elements(data(xy(0,0),xy(1,0),*)))

                  for i=long(0),n_elements(ss)-1 do begin

  		    mn_ar(xy(0,i),xy(1,i))=total(data(xy(0,i),xy(1,i),*))/imgs

                  endfor

	        end

  	'median': begin 		;second order, use a median

		  for i=long(0),n_elements(ss)-1 do begin

		    mn_ar(xy(0,i),xy(1,i))=median(data(xy(0,i),xy(1,i),*))

		  endfor

	        end

	'hist': begin		;third order, probably takes years

		  for i=long(0),n_elements(ss)-1 do begin

		    mn_ar(xy(0,i),xy(1,i))=mode_fit(data(xy(0,i),xy(1,i),*))

		  endfor

	        end

endcase

return, mn_ar

end