;+
; NAME: back2
;
; PURPOSE: Calculate the featureless background of an Halpha or white light 
;          image. This version takes the average of the half maximum median
;          points on a series of concentric circles. It is faster but gives
;          much noisier results than background.pro. 
;
; CALLING SEQUENCE: back2,image
;
;    image is assumed to have been corrected for any vertical/horizontal varience
;
; HISTORY: drafted apr. 1992, A. McAllister 
;
;-
FUNCTION back2,img

backgrd=img

dsk_loc,img,HORZCNT=cnthz,VERTCNT=cntvt,HORZRAD=rdhz,VERTRAD=rdvt

for ir=1,fix(rdhz/2)+5 do begin

  dum=crcle(cnthz,cntvt,ir,VCOORDS=ss) ;throw a ring and get pix coords
  ringdat=img(ss)                               ;store data

  hst=histogram(ringdat)                        ;do histogram around ring
  peak=where(hst gt max(hst)/2)
  szpeak=size(peak)
  totcnts=total(hst(peak(0):peak(szpeak(1)-1)))
  avrg=total((hst(peak(0):peak(szpeak(1)-1))/totcnts)*(indgen(szpeak(1)+1)+peak(0)))

  backgrd(ss)=float(avrg)                       ;pick peak and use that

endfor

return,backgrd

end