;+
; NAME: bytcon
;
;
; PURPOSE: To truncate and convert something to byte without scaling
;
;
; METHOD: Set all large values to minimum and maximum values then
;         shift to positive side of zero and convert to byte.
;
; CALLING SEQUENCE:
;                    out=bytcon(data,min=min,max=max,cent=cent)
;
; PARAMETERS:	
;		data	input data
;		out     the converted data
;
; KEYWORDS:
;              min   is a minimum value, default is -127.0.
;              max   is a maximum value, default is 127.0.
;	       cent  returns the new "zero" value. 
;
; CAUTIONS:    max and min should sum to not more than 255.
;
; RETURN TYPE:  BYTE.                                                      
;
; HISTORY: drafted 2-dec-92, by A. McAllister
;
;-
FUNCTION bytcon,data,min=min,max=max,cent=cent

if keyword_set(min) eq 0 then min = -127.0
if keyword_set(max) eq 0 then max = 127.0

;truncate and shift into byte range, no scaling. 

cent=1-min

return,byte(((data>float(min))<float(max))+float(cent))

end