;+
;       NAME: circle_fn
;
;       PURPOSE: Circle function, to generate a set of points at
;		 angles matching the input points, which lie on an
;		 circle with given parameters.  
;
;       METHOD:  Given the centroid and radius, calculate
;			x = r cos( theta )
;			y = r sin( theta )
;
;       CALLING SEQUENCE: nfits = circle_fn(a,x,y)
;
;       PARAMETERS:	a		fit parameters: centroid and radius
;			x,y		data points
;			nfits		fit points [xfit,yfit]
;
;       HISTORY: Drafted AMcA, August 1994.
;-
FUNCTION circle_fn,a,x,y

xcen = a(0)     
ycen = a(1)
rad  = a(2)
szx=size(x)
if szx(n_elements(szx)-2) eq 0 then stop
   polar = rect2pol(x-xcen,y-ycen)	                ;convert to polar
   
   x_fit = rad * cos( polar(1,*) ) + xcen		;make circle and center			
   y_fit = rad * sin( polar(1,*) ) + ycen

return,[x_fit,y_fit]

end
