;+
;       NAME:  circle_trgt
;
;       PURPOSE:  To estimate target parameters for a set of points on the
;		  circumfrance of a circle.		 
;
;       METHOD:   Use averages for x,y and convert to radial to average for r.
;
;       CALLING SEQUENCE: trgt=circle_trgt(limb)
;
;       PARAMETERS:	limb	points to fit, in [x,y] form
;			trgt	return fit parameters
;					centroid: [x,y]
;					radius.
;
;	CALLS:	RECT2POL
;
;       HISTORY: Drafted AMcA, November 1994.
;
;-
FUNCTION circle_trgt,limb

x = limb(0,*)
y = limb(1,*)

npnts = n_elements(x)

xcen = total(x)/npnts
ycen = total(y)/npnts

polar = rect2pol(x-xcen,y-ycen)

radius = total(polar(0,*))/npnts

return,[xcen,ycen,radius]

end
