;+
; NAME: 
;          compst
;
; PURPOSE: To combine two images ontop of each other by interleaving.
;          To be effective this requires having an approriate color
;          table, as each image is scaled to be in one half of the 
;          color table.
;
;          The first image is scaled to be from 0 to 113.
;          The second image is scaled to be from 114 to 227.
;
; CALLING SEQUENCE:
;                   imageout= compst(image1,image2)
;
;          imageout        is a byte array of the same size as the inputs.
;          image1, image2  are expected to be the same size, and should
;                          have an even second dimension.
;
; It is recommended that images are massaged to emphasize the intensity
; ranges of particular interest. This can be combined with creative color
; tables for best results.
;
; History: 
;          drafted by A. McAllister, Apr. 1992.
;
;-
FUNCTION compst,datin1,datin2

  dat50=bytscl(datin1,top=113)        ;crop color table of first image

  dat51=bytscl(datin2,top=113)        ;crop color table of 2nd image

  datsz=size(datin1)
  hysz=fix(datsz(2)/2)
  dat52=bytarr(datsz(1),datsz(2))     ;initialize result array



 evnln=bytarr(datsz(1),hysz) 
 oddln=evnln

 evnln(2*indgen(hysz),*)=1              ;create even line mask            
 oddln(2*indgen(hysz)+1,*)=1            ;create odd line mask

 evnar=bytarr(datsz(1),datsz(2))       
 oddar=evnar                 

 evnar(*,2*indgen(hysz))=evnln         ;create even array mask
 evnar(*,2*indgen(hysz)+1)=oddln

 oddar(*,2*indgen(hysz))=oddln         ;create odd array mask
 oddar(*,2*indgen(hysz)+1)=evnln

 dat52=evnar*dat51 + oddar*byte(dat50+114)      ;create interleaved array

 return,dat52

 end


