;+
; NAME: crcle
;
; PURPOSE: calculate a circle of pixels
;
; CALLING SEQUENCE: crcle,xc,yc,r,arsz,[,XCOORDS=x,YCOORDS=y,VCOORDS=ss]
;
;            xc   are the center x coordinates 
;            yc   are the center y coordinates 
;            r    is the radius
;	     arsz is the size of the array being worked on, (assumed square)
;
; OPTIONAL PARAMETERS:
;                      XCOORDS,YCOORDS    Theta ordered coordinates
;                      VCOORDS            A unified coordinate vector
;
; NOTE:   Be careful that these coordinates start from an unknown phase,
;         it is however consistant.                                               
;
; HISTORY: drafted apr. 1992, A. McAllister 
;          vectorized, dec. 1992. A.McA.
;
;-
FUNCTION crcle,xc,yc,r,arsz,XCOORDS=x,YCOORDS=y,VCOORDS=ss	

pi=4.0*atan(1.0)

n=fix(2*pi*r)                         ;this is always enough

x=intarr(n+1)
y=x
ss=long(x)
cirarr=intarr(arsz,arsz)

ind=lindgen(1,n)                    ;try this, if it works it will be much faster

x=fix(r*cos((ind*2*pi/n)+5)+xc)     ;x coords vector
y=fix(r*sin((ind*2*pi/n)+5)+yc)     ;y coords vector

ss=x+long(y)*long(arsz)             ;unified coords vector

cirarr(ss)=1                           ;circle image

return,cirarr

end