;+
;       NAME: dejitter
;
;       PURPOSE: Use a fixed dark feature (eg a sunspot) to dejitter a set of 
;		 images.		 
;
;       METHOD: Select a small area around the feature, find the darkest pixel
;		in each image and shift the images to align these dark pixels.
;		The factor keyword allows you to blow up the small region to get
;		sub-pixel accuracy. 
;
;       CALLING SEQUENCE: out=dejitter(cube,factor)
;
;       PARAMETERS:	cube	input cube
;			factor	factor to expand by for calculations. Should
;				not be so large as to make the images larger 
;				than the screen.
;
;	KEYWORD:	max	use a maximum rather than a minimum for the fit
;
;	CALLS:   GT_RECT,COORD_L2V,POLY_2D	
;
;       HISTORY: Drafted AMcA, November 3, 1994.
;		 Modified to take average of area around the extreme 
;		 pix. Added MAX keyword, AMcA, Dec 3, 1994.
;-
FUNCTION dejitter,cube,factor,max=max

szc=size(cube)

;select a small region through the main cube.

numb=new_win(factor*szc(1),factor*szc(2))

tvscl,rebin(cube(*,*,0),factor*szc(1),factor*szc(2))

print,'Please select the center of the region. Last entry will be used.'

selpnt,cent,/mark

input=' '
read, 'Enter the region size in FR pixels, [xs,ys]: ', input
ar_siz=fix(str2arr(input, delim = ',')) 

lft=cent(0)-factor*ar_siz(0)
rgt=cent(0)+factor*ar_siz(0)
bot=cent(1)-factor*ar_siz(1)
top=cent(1)+factor*ar_siz(1)

draw_boxcorn,lft,bot,rgt,top,/device
 
small=gt_rect(cube,lft/factor,rgt/factor,bot/factor,top/factor)

szs=size(small)

expand=rebin(small,factor*szs(1),factor*szs(2),szs(3))

xy=fltarr(2,szs(3))

;find the minimum value in the region for each frame
;what to do if the minimum is not unique?

for i=0,szs(3)-1 do begin

	if not keyword_set(max) then begin
		extrm=where(expand(*,*,i) le min(expand(*,*,i))+3 )
	endif else extrm=where(expand(*,*,i) ge max(expand(*,*,i))-3 )

	xys=coord_l2v(extrm,size(expand(*,*,0)))
	xy(0,i)=total(xys(0,*))/float(n_elements(xys(0,*)))
	xy(1,i)=total(xys(1,*))/float(n_elements(xys(1,*)))

endfor

deltx=(xy(0,*)-xy(0,0))/factor			;convert back to original scale
delty=(xy(1,*)-xy(1,0))/factor

out=cube

for i=0,szs(3)-1 do out(*,*,i)=poly_2d(cube(*,*,i),[deltx(i),0,1,0],[delty(i),1,0,0],1,missing=cube(0,0,0))

return,out

end
