;+
;       NAME: edit_pnts
;
;       PURPOSE: Let user edit a list of limb points.		 
;
;       METHOD:  Use the mouse to deselect points to be omitted.
;
;       CALLING SEQUENCE: nlimb=edit_pnts(limb)
;
;       PARAMETERS:	limb 	initial list of limb points in [x,y] form
;			nlimb	reduced set of limb points, in same form
;
;	KEYWORDS:	index	return the index into the original data
;			mode	this indicates the coordinate system to use
;				when reading from the screen. The default is
;				DEVICE.
;
;       HISTORY: Copied from SXT_CENTER, AMcA, May 1994.
;		 Added INDEX keyword, AMcA, Nov. 23, 1994.
;		 Added the MODE keyword and scaling for the "data"
;		 mode, AMcA, Nov. 28, 1994.
;		
;-
FUNCTION edit_pnts,limb,index=ss,mode=mode

xfit = limb(0,*)
yfit = limb(1,*)

if keyword_set(mode) then begin

	if mode eq "data" then begin

		xscale=!x.crange(1)-!x.crange(0)
		yscale=!y.crange(1)-!y.crange(0)

		xfit=xfit/xscale
		yfit=yfit/yscale

	endif

endif else mode="device"

print
print,"Click left button near unwanted points to erase them"
print,"Click right button when finished to continue with the fit"

ss=indgen(n_elements(xfit))

REPEAT begin
        case mode of

	"data":		begin

			cursor,x,y,3,/data

			x=x/xscale
			y=y/yscale

			end

	"device":	cursor,x,y,3,/device

	else:		cursor,x,y,3,/device		;make this default

        endcase 
   
         if !ERR LT 4 then begin
            distance = sqrt((xfit-x)^2+(yfit-y)^2)
            junk = min(distance,closest_p) ; find the closest xfit,yfit point
            plots,xfit(closest_p),yfit(closest_p),/device,psym=1,color=0

            if closest_p NE 0 and closest_p NE n_elements(xfit)-1 then begin
              xfit = [xfit(0:closest_p-1),xfit(closest_p+1:n_elements(xfit)-1)]
              yfit = [yfit(0:closest_p-1),yfit(closest_p+1:n_elements(yfit)-1)]
	      ss   = [ss(0:closest_p-1),ss(closest_p+1:n_elements(ss)-1)]
            endif else if closest_p EQ 0 then begin
              xfit = xfit(1:n_elements(xfit)-1)
              yfit = yfit(1:n_elements(yfit)-1)
	      ss = ss(1:n_elements(ss)-1)
            endif else if closest_p EQ n_elements(xfit)-1 then begin
              xfit = xfit(0:n_elements(xfit)-2)
              yfit = yfit(0:n_elements(yfit)-2)
	      ss = ss(0:n_elements(ss)-2)
            endif
         endif
endrep UNTIL !ERR GE 4

pnts=n_elements(xfit)
nlimb=fltarr(2,pnts)
nlimb(0,*) = limb(0,ss)
nlimb(1,*) = limb(1,ss)

return,nlimb 

end
