;+
;       NAME: ELL2CIR
;
;       PURPOSE: To reshape an image from an elipse to a circle.	 
;
;       METHOD:  Return is not a round number, but is square.
;
;       CALLING SEQUENCE:	circular = ell2cir(image,a,trgt,/inter)
;
;       PARAMETERS:		image	elliptical image
;				abk 	average background value
;				a	ellipse fit parameters
;				trgt	target to match too.
;
;	KEYWORDS:		inter	pass-through to DSK_LOCG
;
;	CALLS:	ROTT,CONGRID,DSK_LOCG,POLY_2D
;
;       HISTORY: Drafted AMcA, August 1994.
;		 adjusted the background treatment, AMcA, 24 Aug. 1994.
;		 again, AMcA, 26 Aug. 1994
;		 added '/interp' to the ROTT calls, AMcA, 2 Sep. 1994.
;		 fixed definitions of NEWX and NEWY, AMcA, 7 Nov. 1994. 
;-
FUNCTION ell2cir,image,abk,a,trgt,inter=inter,quiet=quiet

sz=size(image)
temp=fltarr(800,800)				;create edge buffers

temp(*,*) = abk					;intialize to background

xcen = a(0)     
ycen = a(1)
minax = a(2)
eccent = a(3)
alpha = a(4)
majax = sqrt((minax*minax)/(1 - eccent*eccent ))

lft = fix(399.5 - xcen)
rgt = fix(lft + sz(1)-1)
bot = fix(399.5 - ycen)
top = fix(bot + sz(2)-1)

fdltx=((399.5 - xcen)-lft)        		;fractional pixel difference
fdlty=((399.5 - ycen)-bot)

temp(lft:rgt,bot:top) = image			;center in temp

nxcen = xcen + lft				;exact center		
nycen = ycen + bot

;if the angle (or ecentricity) is small don't do rotations?

da = !pi - alpha					;angle with verticle
if (da lt 0 ) then da = 2*!pi - da			;ROTT requires clockwise angle

temp = rott(temp,-da*!radeg,nxcen,nycen,missing=abk,/interp)	;rotate major axis to verticle

if n_params() eq 4 then begin

	newx = fix((trgt(2)/majax*800))
	newy = fix((trgt(2)/minax*800))
	
endif else begin

	newx = fix((minax/majax*800))
	newy = 800

endelse

temp = congrid(temp,newx,newy,/cubic) > abk	;reshape. Screen for negative interpolations
 
ntrgt = dsk_locg(temp,50,/cir,inter=inter,quiet=quiet)	;find new center

temp = rott(temp,da*!radeg,ntrgt(0),ntrgt(1),missing=abk,/interp)	;rotate back

tmpsz=size(temp)

if n_params() eq 4 then begin		;reposition and resize to match target		
						
	lft = fix(ntrgt(0) - trgt(0))		;extract coordinates		
	rgt = fix(ntrgt(0) + (511-trgt(0)))					
	bot = fix(ntrgt(1) - trgt(1))	
	top = fix(ntrgt(1) + (511-trgt(1)))
	
	temp = temp(lft>0:rgt<(tmpsz(1)-1),bot>0:top<(tmpsz(2)-1))	;extract	
	
	tmpsz=size(temp)
	dlft= -lft > 0				;insert coordinates 		
	drgt= (dlft + tmpsz(1)-1) < 511			
	dbot= -bot > 0				
	dtop= (dbot + tmpsz(2)-1) < 511			
	
	temp2 = replicate(abk,512,512)
	temp2(dlft:drgt,dbot:dtop)=temp			
	
	fdltx=((ntrgt(0)-trgt(0))-lft)        		;fractional pixel difference
	fdlty=((ntrgt(1)-trgt(1))-bot)
	
	temp=poly_2d(temp2,[-fdltx,0,1,0],[-fdlty,1,0,0],1,missing=abk)

endif else begin
	
	lft = fix(ntrgt(0) - (minax + 50))	;>0
	rgt = fix(ntrgt(0)+ (minax + 50))	;<tmpsz(1)-1
	bot = fix(ntrgt(1) - (minax + 50))	;>0
	top = fix(ntrgt(1) + (minax + 49))	;<tmpsz(2)-1

	temp=congrid(temp(lft:rgt,bot:top),512,512)

endelse

return,temp

end
