;+
;       NAME: ellipse_fn
;
;       PURPOSE: Ellipse function, to generate a set of points at
;		 angles matching the input points, which line on an
;		 ellipse with given parameters.  
;
;       METHOD:  Given the centroid and major and minor axis', calculate
;			x = a cos( theta )
;			y = b sin( theta' )	theta' = atan(a/b*tan(theta))
;
;       CALLING SEQUENCE: nfits = functn(a,x,y)
;
;       PARAMETERS:	a		fit parameters, centroid,
;					minor axis', eccentricity, 
;					and rotation angle.
;			x,y		data points
;			nfits		fit points [xfit,yfit]
;
;       HISTORY: Drafted AMcA, August 1994.
;-
FUNCTION ellipse_fn,a,x,y

xcen = a(0)     
ycen = a(1)
minax = a(2)
eccent = a(3)
alpha = a(4)

majax = sqrt((minax*minax)/(1 - eccent*eccent))

   x_dat =  (x-xcen)*cos(-alpha) + (y-ycen)*sin(-alpha) ;move to origin 
   y_dat = -(x-xcen)*sin(-alpha) + (y-ycen)*cos(-alpha) ;and rotate
 
   polar = rect2pol(x_dat,y_dat)	                ;convert to polar
   
   theta = atan((majax/minax)*sin(polar(1,*)),cos(polar(1,*))) 	;get equivalent angles
   
   x_ell = majax * cos( theta ) 			;make the ellipse			
   y_ell = minax * sin( theta ) 

   x_fit =   x_ell*cos(alpha) + y_ell*sin(alpha) + xcen	;recenter and rotate
   y_fit = - x_ell*sin(alpha) + y_ell*cos(alpha) + ycen

return,[x_fit,y_fit]

end
