;+
;       NAME:
;		enh_cube
;
;       PURPOSE:
;		A front end to run enhancer on a data cube
;
;       METHOD:
;
;
;       CALLING SEQUENCE:
;
;       imageout=enhancer(imagein,index,keyword=##,keyword=##,..)
;
;       where imagein & imageout are 3d arrays, and index is the
;       index entries that correspond to imagein.
;
;       NOTES:
;		*****Keywords DO NOT function yet******
;   		This only does the default enhancing.
;
;       PARAMETERS:
;
;
;       KEYWORDS:
;
;       smooth    Size of smoothing box, integer.
;       lowdelt   Lower delta cutoff, a percentage of minimum, decimal 0.xx.
;       updelt    Upper delta cutoff, a percentage of maximum, decimal 0.xx.
;       lowint    Saturation cutoff, don't add back when signal is BELOW a
;                 given percentage of the saturation value, decimal 0.xx
;
;       If a keyword is not specified then a default value is used, based on
;       the resolution of the input data array.
;
;       HISTORY:
;		drafted by A. McAllister, May-29-92
;
;-
FUNCTION enh_cube,data,indx,smooth=smth,lowdelt=lwct,updelt=upct,lowint=stctl,upint=stctu,deltcoef=dltmlt

imgsz=size(data)

out=bytarr(imgsz(1),imgsz(2),imgsz(3))

for i=0,imgsz(3) do out(*,*,i)=enhancer(data(*,*,i),indx(i))

return,out

end