;+
;       NAME: fchisq
;
;       PURPOSE: Calculates reduced chi squared
;
;       METHOD:  
;
;       CALLING SEQUENCE: chisq = fchisq(x,y,xfit,yfit,mode,sigmay,nfree)
;
;       PARAMETERS:	x	data points for independent variable
;			y	data points for dependent variable
;			xfit 	calculated values of x
;			yfit 	calculated values of y
;			mode	determines weighting method
;				+1 (instrumental) weight(i) = 1./sigmay(i)**2
;				 0 (no wieghting) weight(i) = 1.
;				-1 (statistical)  weight(i) = 1./y(i)
;			sigmay	standard deviations of the y data points
;			nfree	number of free parameters
;
;			chisq	return chi square 	
;
;       HISTORY: Converted from Bornmann C routine, AMcA, July 1994.
;-
FUNCTION fchisq,x,y,xfit,yfit,mode,sigmay,nfree

case mode of

		1: weight = 1.0 / (sigmay*sigmay)
		
		0: weight = 1.0
		
	       -1: begin 
	       	         weight = 1.0
	           	 nonz = where(yval ne 0.)
	       		 weight(nonz)=abs(1./ yval(nonz))
	       	   end 
endcase	       	   

chisq = weight * ((y-yfit)*(y-yfit) + (x-xfit)*(x-xfit)) / nfree

return, total(chisq)

end
