;+
; NAME: gt_rect
;
;
; PURPOSE: return a rectangular sub-image(s) from input image(cube).
;
;
; METHOD:  the pixels of the sub image are calculated from the edge
;	   values, then the sub-region is returned. The input rows
;	   and columns are INCLUDED.
;
;
; CALLING SEQUENCE:  out=gt_rect(data,left,right,bottom,top)
;
; PARAMETERS: 
;		data		oringial image(s).
;		out		resultant sub image(s).
;
;		left		left boundry, in pixels.
;		right		right boundry, in pixels.
;		bottom		bottom edge, in pixels.
;		top		top edge, in pixels.
;
; RETURN TYPE: Same as the input.
;
; CALLS: COORD_L2V                                                     
;
; HISTORY: Drafted by A. McAllister, March 1993.
;	   Generalized to handle cubes as well, AMcA, 18-mar-93.
;	   Reworked to allow for memory problems, AMCA, 25-mar-93.
;
;-
FUNCTION gt_rect,data,lft,rgt,bot,top

 imgsz=size(data)

 case imgsz(0) of

	2: begin
 
 		pix=lindgen(imgsz(1),imgsz(2))
 		xy=coord_l2v(pix,imgsz)
 		pix_ss=where(xy(0,*) ge lft and xy(0,*) le rgt and $
			     xy(1,*) ge bot and xy(1,*) le top)

		out=replicate(data(0,0),rgt-lft+1,top-bot+1)
 		out(xy(0,pix_ss)-lft,xy(1,pix_ss)-bot)=data(pix_ss)

	   end

	3: begin

 		pix=lindgen(imgsz(1),imgsz(2))
		dumsz=[2,imgsz(1:2)]
 		xy=coord_l2v(pix,dumsz)
 		pix_ss=where(xy(0,*) ge lft and xy(0,*) le rgt and $
			     xy(1,*) ge bot and xy(1,*) le top)

		out=replicate(data(0,0,0),rgt-lft+1,top-bot+1,imgsz(3))

		for i=0,imgsz(3)-1 do begin	;can't see a way around this (memory too small).

		  zss=replicate(i,n_elements(pix_ss))
		  out(xy(0,pix_ss)-lft,xy(1,pix_ss)-bot,zss)=data(xy(0,pix_ss),xy(1,pix_ss),zss)

		endfor

	   end
 endcase

 return,out 

end