;+
;       NAME: limbpnts
;
;
;       PURPOSE: Find limb points for spectral images
;
;       METHOD:  Take estimated center and take rays of points.
;		 Call CUT_LEV to stimate background and subtract. Take 
;		 first non-zero values as the limb. 
;		 Tried cumulative histogram technique, but abandoned it.
;		 May need to rework to handle magnetograms.
;
;       CALLING SEQUENCE: limb=limbpnts(solar,cent,np,/quiet)
;
;
;       PARAMETERS:	solar	input image
;			cent	estimated image centroid
;			np	number of points to select
;			limb	limb points returned in [x,y] form.
;
;	KEYWORDS:	quiet	when NOT set display image and points
;
;       HISTORY:	drafted by A.McAllister nov. 1993
;
;-
FUNCTION limbpnts,solar,cent,np,quiet=quiet

image=solar						;buffer input
imgsz=size(image)

cut=cut_lev(image,imgsz)				;remove background
image=image-cut

maxlen=sqrt(imgsz(1)*imgsz(1) + imgsz(2)*imgsz(2)) 	;image diagonal

dthet=361./np
theta=findgen(np) * dthet * !dtor			;angles

;ex=cos(theta)*maxlen + cent(0)    			;end points
;ey=sin(theta)*maxlen + cent(1)

;for each line

line = lindgen(maxlen)

lx=((cos(theta)#line + cent(0)) > 0) < imgsz(1)		;get points on line
ly=((sin(theta)#line + cent(1)) > 0) < imgsz(2)

lx(where(lx le 0. or lx ge imgsz(1))) = 0		;tag out of bound points
ly(where(ly le 0. or ly ge imgsz(2))) = 0
lx(where(ly eq 0)) = 0					
ly(where(lx eq 0)) = 0

lnsize=size(lx) & linnmb=lnsize(1) & linlth=lnsize(2)
lxy=fltarr(2,linnmb*linlth)
lxy(0,*)=lx & lxy(1,*)=ly				;reorganize for conversion
linss=coord_v2l(lxy,imgsz)
linss=reform(linss,linnmb,linlth,/overwrite)		;linear coords by line

limb=intarr(2,linnmb)

if not keyword_set(quiet) then tvscl,image

for ii = 0,linnmb-1 do begin

 rlin=linss(ii,*)
 tlin=rlin(where(linss(ii,*) gt 0))			;crop outside-the-image zeros
 
 zer=where(image(tlin) le 0)
 if zer(0) eq (-1) then lind=n_elements(tlin)-1 else lind=zer(0)	;trap for missing limb
 peak=max(image)
  
 limb(*,ii)=coord_l2v(linss(ii,lind),imgsz)
; limb(*,ii)=coord_l2v(linss(ii,fix(cmhst(fix(peak)))),imgsz)	;actually 10*.1*peak
 
 ;indexing into cmhst with value that is 10% of image max...
 ;since maximum values are near 0, integrating inwards, but 
 ;plotting center first..
 
if not keyword_set(quiet) then plots,limb(0,ii),limb(1,ii),psym=1,/device,color=255
 
endfor

return, limb

end
