;+
;       NAME: linsel
;
;       PURPOSE: Select, and optionally confirm, a line of points from an
;		 image or data cube		 		 
;
;       METHOD: 	Display the image in a new window (if requested)
;			and intereactively select two points with the mouse.
;			If DRAW option selected then "draw" the line defined
;			by the two points on the image or cube.		 
;
;       CALLING SEQUENCE: linss=linsel(datain,ind,/mark,/draw,/confirm,/display)
;
;       PARAMETERS:	data		the input image/cube to operate on
;			ind		index to indicate which image in a cube
;					 display for the selection process
;	
;	KEYWORDS:	mark		pass through to SELPNT to mark points 
;					as they are selected
;			draw		if set draw the line on the input image or
;					data cube
;			newwin		if set open a new window to work in
;			nodisplay	if set do not redisplay the image
;			quiet		if set do not print instructions on first
;					pass either
;			noconfirm 	if set turn off the confirmation process
;
;	NOTES:	The input data cube will return the same images but with
;		the line drawn on them if the DRAW keyword is selected.	
;
;	CALLS:  SELPNT,OPLIN,NEW_WIN,YES_NO_F,LNPNT		
;
;       HISTORY: Drafted AMcA, Nov. 1994.
;-
FUNCTION linsel,datain,ind,mark=mark,draw=draw,noconfirm=noconfirm,   $
		newwin=newwin,nodisplay=nodisplay,quiet=quiet

if not keyword_set(quiet) then firstpass=1 else firstpass=0

data=datain

sz=size(data)

if keyword_set(newwin) then num=new_win(sz(1),sz(2))

if not keyword_set(nodisplay) then tvscl,data(*,*,ind)

sel: if firstpass then selpnt,xy,mark=mark else selpnt,xy,mark=mark,/quiet

if n_elements(xy(0,*)) ne 2 then begin

	print,"Please select exactly two points, let's try again."
	firstpass=0
        goto, sel

endif

if keyword_set(draw) then begin

color=max(data)
linss=oplin(data,xy(0,0),xy(1,0),xy(0,1),xy(1,1),color)

endif else dum=lnpnt(x0,y0,x1,y1,sz,VCOORDS=linss) 

if not keyword_set(noconfirm) then begin

	stepper,data

	input=' '
	read, 'Would you like to redo the line?', input
	if yesno_f(input) then begin

		print,"OK, let's try again"
		firstpass=0
		data=datain
		tvscl,data(*,*,ind)
		goto, sel

	endif

endif

datain=data

return,linss

end
