;+
;       NAME: mk_round
;
;       PURPOSE: To fix the aspect ratio of GBO images taken from
;		 video tape.
;
;       METHOD:  
;
;       CALLING SEQUENCE: cube=mk_round(filestr,trgti,index,/invert,/inter,times=times)
;
;       PARAMETERS:	filestr		string with file name to be processed, can
;					have wild cards in it
;			target		an SXT image to match the GBO images to
;			index		the index for the target image
;			cube		output processed images
;
;	KEYWORDS:	invert		set for inverted images
;			inter		set for interactive point selection
;			quiet		set to refrain from ploting in window
;			times		to return the times of the images
;
;	NOTES:	There is some problem with the H alpha images when they are
;		flipped over before fitting them. Fiddling with the cutoff
;		sceme (especially removing the time stamp before the cutoff
;		calculation) has improved things, but not completely solved 
;		them. So for now I invert them at the end rather than at
;		the beggining.
;
;	CALLS:  EXP_FILE,TARGET,NEW_WIN,RFITS,DSK_LOCG,ELL2CIR,INVRT
;
;       HISTORY: Drafted AMcA, August 1994.
;		 Reworked to take a target image, AMcA, Aug. 25 1994.
;		 Added keywords INTER and QUIET, AMcA, August 29, 1994.
;		 Added work around to avoid problem related to time 
;		 stamp, AMcA, August 30, 1994.
;		 Cludge to avoid problems with inverted images, Sept. 21, 1994.
;		 Changed TIMES output to INT type, Nov. 22, 1994.
;		 Added KEEP_TIME keyword, Dec. 8, 1994.
;-
FUNCTION mk_round,filestr,trgti,index,invert=invert,inter=inter,quiet=quiet, $
		  times=times,keep_time=kt

infil=exp_file(filestr)					;expand file names

if n_params() eq 3 then begin

   trgt2=target(trgti,index)

   if keyword_set(invert) then begin

     tsz=size(trgti)
     trgt2(1)=tsz(2)-trgt2(1)				;correct y center for inversion

   endif	

endif

len=n_elements(infil)

cube=fltarr(512,512,len)
times=strarr(7,len)

if not keyword_set(quiet) then nwin=new_win(800,800)	;make sure there is a window

for i = 0,len-1 do begin

  print,'working on image #,',i

  image=rfits(infil(i),head=head)
  
  imgsz=size(image)
  tot=float((fix(imgsz(1)/10)+1)*(fix(imgsz(2)/10)+1))
  abk=total(image(fix(imgsz(1)*9/10):imgsz(1)-1,fix(imgsz(2)*9/10):imgsz(2)-1))/tot	;find average background
  subck = where(image lt abk)
  if subck(0) ne -1 then image(subck) = abk		;level background  
  
if not keyword_set(kt) then begin

  time = image(0:120,0:100)
  image(0:120,0:100) = abk				;take out the time (HIDA)

endif
      
  times(*,i)=fix([strmid(head(7),11,2),strmid(head(7),14,2),strmid(head(7),17,2),0,   $
              strmid(head(8),11,2),strmid(head(8),14,2),strmid(head(8),17,2)])

  trgt1=dsk_locg(image,100,inter=inter,quiet=quiet)

  if n_params() eq 3 then begin 
   
    nimage=ell2cir(image,abk,trgt1,trgt2,inter=inter,quiet=quiet)
    
  endif else nimage=ell2cir(image,abk,trgt1,inter=inter,quiet=quiet)
  
  if keyword_set(invert) then nimage=invrt(nimage,/y)		;try this here, can't use with target

  cube(*,*,i)=nimage

if not keyword_set(kt) then begin  
  if keyword_set(invert) then begin		;reinsert time

     cube(0,0,i)=invrt(times(*,i),/y)		;part of cludge		

  endif else  cube(0,0,i)=times(*,i)
endif
  
endfor

return,cube

end
