;+
;       NAME: rt_sxtdt
;
;
;       PURPOSE: To automate a direct retrieval of sxt data from sda files.
;
;
;       METHOD: Passes string to FINDFILE, then reads the index with 
;		RD_XDA, calls SSWHERE for data selection, then calls
;		RD_XDA, again to get the data. If the INFO keyword is
;		it will create an info array with GET_INFO2.
;
;
;       CALLING SEQUENCE: rt_sxtdt,filestr,indx[,data][,/nodata][,ss=ss]
;					[,info=info_array][,infil=infil]
;
;       NOTES: 
;
;
;       PARAMETERS:	filestr		string with source file
;					may include wildcards
;			data		retrieved data
;			indx		index 
;
;       KEYWORDS:	info		optional retrieval of info array
;			infil		get expanded filelist
;			ss		output of ss into whole data set
;
;	CALLS: EXP_FILE, SSWHERE, RD_XDA, RD_ROADMAP, GET_INFO2
;
;       HISTORY: Drafted by A.McAllister  20-feb-93.
;		 Removed the initial RD_XDA for the index, AMcA, 9-jul-93.
;
;-
PRO rt_sxtdt,filestr,indx,data,info=infnm,ss=ss,nodata=nd,infil=infil 

infil=exp_file(filestr)            			;make filelist

rd_roadmap,infil,roadmap
ss=sswhere(roadmap)					;select data

if ss(0) ne -1 then begin
			
  if keyword_set(nd) eq 0 then begin

    rd_xda,infil,ss,indx,data                       	;get the selected data

  endif else begin

    rd_xda,infil,ss,indx,/nodata

  endelse

  if keyword_set(infnm) then infnm=get_info2(indx)	;make info array

endif else indx=-1

end






