;+
;       NAME: sel_region
;
;
;       PURPOSE: To allow interactive selection of a rectangular region.
;
;
;       METHOD: In input display the image (if not already displayed),
;		display it, rebined by a integer factor if desired, and
;		then call GET_BOXCORN. The units are in inout image pixels.
;		If the image is passed in then a new window is opened and
;		later closed.  
;
;       CALLING SEQUENCE: xyvec=sel_region([image][,factor][,/close][,/table])
;
;
;       PARAMETERS:	image	image you want to select region from.
;			factor  a multiplication factor for display, INT.
;
;       KEYWORDS:  	close	close the display window when finished. This only
;				works you passed in the image.
;			table	query for color table change 
;
;	CALLS: NEW_WIN, GET_BOXCORN, DRAW_BOXCORN
;
;       HISTORY: Drafted by A.Mcallister, mar-18-93.
;
;-
FUNCTION sel_region,image,factor,close=close,table=table

 if n_params() ne 2 then factor=1  

 if n_params() ge 1 then begin

   imgsz=size(image)
   xs=fix(imgsz(1)*factor)	 
   ys=fix(imgsz(2)*factor)
   num1=new_win(xs,ys)           ;make new window
   tvscl,rebin(image,xs,ys)
   if keyword_set(table) then begin

     yesnox,'Change color table?',ans,'n'    ;change color table
     if ans then xloadct

   endif

 endif  

 print,'Please select region corners with the mouse:'
 print,'Left button to select, Middle to reset, and Right to save and Exit.'

 rtn=0

 while rtn ne 4 do begin

   get_boxcorn, x0, y0, x1, y1, /device
   draw_boxcorn, x0, y0, x1, y1, /device

   xyvec=[x0,x1,y0,y1]/fix(factor)

   print,'Selected region: x = ',string(xyvec(0),format='$(i4)'),' to ', $
         string(xyvec(1),format='$(i4)'),'  y =',string(xyvec(2),format='$(i4)'), $
         ' to ',string(xyvec(3),format='$(i4)')

   cursor,x,y,/device,/down
   rtn=!err

   case rtn of

	1:    rtn=0
	2:    tvscl,rebin(image,xs,ys); rtn
        else:

 endcase

 endwhile

 if keyword_set(close) and n_params() ge 1 then wdelete,num1		;clean up 

 return,xyvec

end