;+
; NAME		flmb
;
; PURPOSE	Call Peter Bandurian's flmbiter.f routine
;		to remove limb darkening.
;
; AUTHOR	Randy Meisner   HAO/NCAR
;
; HISTORY	Updates by Andrew L. Stanger   HAO/NCAR
;-

PRO flmb, file, hdu, img, xc, yc, rx, ry, niter, resid

   niter = LONG (niter)

   img = FLOAT (img)

   xc = FLOAT (xc) & yc = FLOAT (yc)
   rx = FLOAT (rx) & ry = FLOAT (ry)

   sz = SIZE (img) & nx = sz (1) & ny = sz (2)

   rad_t = MAX ([rx, ry]) & rad_lim = rad_t-4.0

   weight = FLTARR (nx, ny)

   m1 = 2L & m2 = 2L

   ex = [1L, 3] & nex = n_elements(ex)

   dimcoef = 8L

   fit = FLTARR (nx, ny, 2)

   rms = FLTARR (niter)

   coef = FLTARR (dimcoef, niter)

   npts = LONARR (3, niter)

   envchk = strpos (strupcase (!version.os), 'IRIX')

   IF (envchk EQ 0) THEN BEGIN

      dummy = call_external ('/riker/d/acos/band/sgi/flmb.so', 'flmbext_', $
			     xc, yc, rad_t, rad_lim, img, nx, ny,	 $
			     weight, m1, m2, ex, nex, fit, niter, rms,	 $
			     coef, dimcoef, npts)

   ENDIF ELSE BEGIN

       dummy = call_external ('/riker/d/acos/band/sun/flmb.so', 'flmbext_', $
			      xc, yc, rad_t, rad_lim, img, nx, ny,	 $
			      weight, m1, m2, ex, nex, fit, niter, rms,	 $
			      coef, dimcoef, npts)
   ENDELSE

   indx_it = WHERE (rms EQ 0, n_it)

   n_iter = niter - n_it

   PRINT, '# flmb iterations = ', n_iter

   resid = img - fit (*, *, 0)

   indx_fit = WHERE (fit (*, *, 0) NE 0, n_fit)

   irr_2d = TOTAL (fit (*, *, 0))

   irr_2d = irr_2d / 1.0e6

   resid = temporary (resid) / irr_2d

   PRINT, 'Integrated image intensity =', TOTAL (img (indx_fit)) / 1.0e6
   PRINT, '  Normalization value used = ', irr_2d

   min_res = MIN (resid)

   max_res = MAX (abs (resid))

   print, 'min_res/max_res: ', min_res, max_res

   indx_zero = WHERE (fit eq 0)

   resid (indx_zero) = min_res - 0.1 * min_res

   BSCALE = max_res / 30000.0

   resid = fix ( temporary (resid) / BSCALE )

   PRINT, 'BSCALE = ', bscale

   mkhdr, hdr, resid
   hdu_resid = hdu

   sxaddpar, hdu_resid, 'CRRADIUS', (rx + ry) / 2.0
   sxaddpar, hdu_resid, 'CRPIX1', xc
   sxaddpar, hdu_resid, 'CRPIX2', yc
   sxaddpar, hdu_resid, 'TYPE-DP', 'LD REMOVED'
   sxaddpar, hdu_resid, 'DATAMIN', min_res
   sxaddpar, hdu_resid, 'DATAMAX', max_res
   sxaddpar, hdu_resid, 'DISPMIN', min_res
   sxaddpar, hdu_resid, 'DISPMAX', max_res

   sxaddpar, hdu_resid, 'BSCALE', bscale, 'DATA = BSCALE * ARRAY + BZERO', 'SUMX'
   sxaddpar, hdu_resid, 'BZERO', 0.0, 'DATA = BSCALE * ARRAY + BZERO', 'SUMX'
   sxaddpar, hdu_resid, 'IRR_2D', irr_2d, '2-D QUIET SUN IRRADIENCE ESTIMATE / 1.0e6'
   sxaddpar, hdu_resid, 'RAD_LIM', rad_lim, 'RESIDUAL RADIUS, IN PIXELS.'
   sxaddpar, hdu_resid, 'C0', coef (0,n_iter-1), 'FIT COEFF CONSTANT'
   sxaddpar, hdu_resid, 'C1', coef (1,n_iter-1), 'FIT COEFF X'
   sxaddpar, hdu_resid, 'C2', coef (2,n_iter-1), 'FIT COEFF Y'
   sxaddpar, hdu_resid, 'C3', coef (3,n_iter-1), 'FIT COEFF X^2'
   sxaddpar, hdu_resid, 'C4', coef (4,n_iter-1), 'FIT COEFF X*Y'
   sxaddpar, hdu_resid, 'C5', coef (5,n_iter-1), 'FIT COEFF Y^2'
   sxaddpar, hdu_resid, 'C6', coef (6,n_iter-1), 'FIT COEFF MU'
   sxaddpar, hdu_resid, 'C7', coef (7,n_iter-1), 'FIT COEFF MU^3'
   sxaddhist, 'RESIDUAL IMAGE IS SCALED BY IRR_2D.', hdu_resid
   sxaddhist, 'BACKGROUND = RESIDUAL MINIMUM MINUS 0.1 * MINIMUM.', hdu_resid

   extpos = strpos (file, '.fts')
   PRINT, 'extpos: ', extpos
   resfile = strmid (file, 0, extpos) + '.resid.fts'

   WRITEFITS, resfile, resid, hdu_resid

   hdufit = hdu
   min_fit = MIN (fit (*, *, 0))
   max_fit = MAX (fit (*, *, 0))
   print, 'min_fit/max_fit: ', min_fit, max_fit

   sxaddpar, hdufit, 'OBJECT', 'LIMB DARK FIT'
   sxaddpar, hdufit, 'DATAMIN', min_fit
   sxaddpar, hdufit, 'DATAMAX', max_fit
   sxaddpar, hdufit, 'DISPMIN', min_fit
   sxaddpar, hdufit, 'DISPMAX', max_fit

   extpos = strpos (file, '.fts')
   PRINT, 'extpos: ', extpos
   fitfile = strmid (file, 0, extpos) + '.ldf.fts'

   WRITEFITS, fitfile, fit (*, *, 0), hdufit

END
