      subroutine flmbiter(x0,y0,rad_t,rad_lim,z,idim1,idim2,weight,
     &         m1,m2,ex,nex, fit,maxiter, rmsarr,coef,dimcoef,npts)

c  Routine to fit a smooth function to solar data and subtract the fit thus
c  removing the solar limb darkening function.  The smooth function is a
c  quadratic polynomial in x,y and a polynomial in mu consisting of the
c  first and third order terms.

      integer idim1, idim2, nex, dimcoef, maxiter
      real x0, y0, rad_t, rad_lim, rmsarr(maxiter)
      real z(idim1,idim2), fit(idim1,idim2,2)
      real coef(dimcoef,maxiter), weight(idim1,idim2)
      integer m1, m2, ex(nex), npts(3,maxiter)

      integer mxdim1,mxdim2
      parameter (mxdim1=936, mxdim2=936)
      real x(mxdim1), y(mxdim2), dsk(mxdim1,mxdim2), mu(mxdim1,mxdim2)
      real resid(mxdim1,mxdim2)

      integer i,j,ninside,noutside,nbpts1,nbpts2,nbdchg
      integer nave, iter, nfit, it1,it2
      real rad_lsq,ysq,rsq, rr, rms_l,rms_h, ave,sumsq,rms
      real sum,sumabs, tmp
      integer itmp, t1,t2,time

      real pi,apod,rmin,rdiff, r
c     real pirmwt(mxdim1,mxdim2)
      data pi/3.1415926535/
      character*1 cr

c  Make carrage returns in call_external work.
      cr = char(13)

c  Verification:
c     write(6,'("flmbiter: x0,y0=",2f9.2," rad_t,rad_lim=",2f9.2,a)')
c    &           x0,y0,rad_t,rad_lim,cr
c     write(6,'("flmbiter: m1,m2,ex,nex=",5i4,a)') m1,m2,ex,nex,cr
c     write(6,'("flmbiter: maxiter=",i," dimcoef=",i,a)')
c    &           maxiter,dimcoef,cr

c  Check dimensions.
      if( (idim1.gt.mxdim1).or.(idim2.gt.mxdim2) )then
C        write(6,'("flmbiter: local independent arrays are too small",
C    &                   a)')cr
C        write(6,'("need",2i," have",2i,a)') idim1,idim2,
C    &                    mxdim1,mxdim2,cr
         stop
      endif

c  Create arrays of independent variables: origin at image center and
c  scaled to the true solar radius.
      do i=1,idim1
         x(i) = (float(i)-x0)/rad_t
      enddo
      do i=1,idim2
         y(i) = (float(i)-y0)/rad_t
      enddo

c  Generate disk/mask within which solar image points will be fit _and_
c  zero fitted image outside fitting range.
      ninside=0
      noutside=0
      rad_lsq = rad_lim**2
      apod = 0.9
      rmin = apod*rad_lim
      rdiff = (1.-apod)*rad_lim
      do j=1,idim2
         ysq = (float(j)-y0)**2
         do i=1,idim1
            rsq = (float(i)-x0)**2 + ysq
            if (rsq .le. rad_lsq) then
               dsk(i,j) = 1.0
               ninside  = ninside+1
c              r = sqrt(rsq)
c              if (r .le. rmin) then
c                 pirmwt(i,j) = 1.0
c              else
c                 pirmwt(i,j) = .5 * ( 1. + cos( (r-rmin)*pi/rdiff) )
c              endif
            else
               dsk(i,j) = 0.0
               z(i,j) = 0.0
c              pirmwt(i,j) = 0.0
               noutside  = noutside+1
            endif
         enddo
      enddo

c  Initialize weight.
      do j=1,idim2
      do i=1,idim1
c        weight(i,j) = dsk(i,j) * pirmwt(i,j)
         weight(i,j) = dsk(i,j)
      enddo
      enddo

c  Construct array of mu's.
      do j=1,idim2
         ysq = y(j)**2
         if (ysq.le.1) then
            do i=1,idim1
               rr = x(i)**2 + ysq
               if (rr.le.1.) then
                  mu(i,j) = sqrt(1.-rr)
               else
                  mu(i,j) = 0.0
               endif
            enddo
         else
            do i=1,idim1
               mu(i,j) = 0.0
            enddo
         endif
      enddo

c  To conserve space and decrease disk thrashing, only keep the latest two
c  iterations and keep track of which is which with the following two variables.
      it1 = 1
      it2 = 2

c  Perform the fitting.
      call lmbfit( x,y,z,idim1,idim2,m1,m2,ex,nex,weight,
     &                         mu,mxdim1,mxdim2,coef(1,1),dimcoef)

c  Evaluate the fit.
      call cmpoly( x,y,idim1,idim2,m1,m2,ex,nex,mu,mxdim1,mxdim2,
     &               dsk, coef(1,1),dimcoef, fit(1,1,it1))

c  Create the array of residuals of the fit and compute the rms variance.
      nave = 0.
      ave = 0.
      sumsq = 0.
      do j=1,idim2
      do i=1,idim1
         resid(i,j) = z(i,j) - fit(i,j,it1)
         if (dsk(i,j).eq.1) then
            nave = nave + 1
            ave = ave + resid(i,j)
            sumsq = sumsq + resid(i,j)**2
         endif
      enddo
      enddo
      ave = ave/float(nave)
      rms = sqrt( (1./float(nave-1)) * (sumsq - float(nave)*ave**2) )
      rmsarr(1) = rms

c  Mark points to be fit/not fit.
      rms_h = .25*rms
      rms_l = -3.0*rms
      nbpts1 = 0
      nfit = 0
      do j=1,idim2
      do i=1,idim1
         if( (dsk(i,j).ne.0.) .and. (resid(i,j).le.rms_h) .and. 
     &                                 (resid(i,j).ge.rms_l) )then
c           weight(i,j) = 1.0 * pirmwt(i,j)
            weight(i,j) = 1.0
            nfit = nfit+1
         else
            weight(i,j) = 0.0
            nbpts1 = nbpts1+1
         endif
      enddo
      enddo
      npts(1,1) = nfit
      npts(2,1) = nbpts1
      npts(3,1) = 0

c   Iterate a better fit!
      do iter=2,maxiter
         t1 = time()
C        write(6,'("flmbiter: iteration=",i,a)') iter,cr

         call lmbfit( x,y,z,idim1,idim2,m1,m2,ex,nex,weight,
     &                         mu,mxdim1,mxdim2,coef(1,iter),dimcoef)
      
c        Evaluate the fit.
         call cmpoly( x,y,idim1,idim2,m1,m2,ex,nex,mu,mxdim1,mxdim2,
     &               dsk, coef(1,iter),dimcoef, fit(1,1,it2))

c        Create the array of residuals of the fit and compute the rms variance.
         nave = 0.
         ave = 0.
         sumsq = 0.
         do j=1,idim2
         do i=1,idim1
            resid(i,j) = z(i,j) - fit(i,j,it2)
            if (dsk(i,j).eq.1) then
               nave = nave + 1
               ave = ave + resid(i,j)
               sumsq = sumsq + resid(i,j)**2
            endif
         enddo
         enddo
         ave = ave/float(nave)
         rms = sqrt( (1./float(nave-1)) * (sumsq - float(nave)*ave**2) )
         rmsarr(iter) = rms

c        Check for convergence.
         sum = 0.0
         sumabs = 0.0
         do j=1,idim2
         do i=1,idim1
            sum = sum + fit(i,j,it1)
            sumabs = sumabs + abs(fit(i,j,it2)-fit(i,j,it1))
         enddo
         enddo

C        write(6,'("flmbiter: fit criterion=",g,a)') sumabs/sum, cr

         if (sumabs/sum .le. 0.0001) then
c           Ensure that the final fit is in the first fit array plane.
C           write(6,'("flmtiter: successful fit!",a)') cr
            if (it2 .eq. 2) then
               do j=1,idim2
               do i=1,idim1
                  fit(i,j,1) = fit(i,j,2)
               enddo
               enddo
            endif
            return
         endif

c        Calculate weights only if not the last iteration.
         if (iter .lt. maxiter) then

c        Mark points to be fit/not fit.
         nbpts2 = 0
         nfit = 0
c        Asymmetric weights.
         if (iter .le. 5) then
            do j=1,idim2
            do i=1,idim1
               if (dsk(i,j).ne.0.0) then
                  if (resid(i,j) .gt. 0.) then
                     tmp = -1.*(resid(i,j)/rms)**2
                  else
                     tmp = resid(i,j)/rms
                  endif
                  if (tmp.gt.-6.) then
c                    weight(i,j) = exp(tmp) * pirmwt(i,j)
                     weight(i,j) = exp(tmp)
                     nfit = nfit+1
                  else
                     weight(i,j) = 0.0
                     nbpts2 = nbpts2+1
                  endif
               endif
            enddo
            enddo
         endif
c        Symmetric weights.
         if (iter .gt. 5) then
            do j=1,idim2
            do i=1,idim1
               if (dsk(i,j).ne.0.0) then
                  tmp = -1.*(resid(i,j)/rms)**2
                  if (tmp.gt.-6.) then
c                    weight(i,j) = exp(tmp) * pirmwt(i,j)
                     weight(i,j) = exp(tmp)
                     nfit = nfit+1
                  else
                     weight(i,j) = 0.0
                     nbpts2 = nbpts2+1
                  endif
               endif
            enddo
            enddo
         endif

c        Check on how the number of not-fit points changes.
         nbdchg = nbpts2-nbpts1
         npts(1,iter) = nfit
         npts(2,iter) = nbpts2
         npts(3,iter) = nbdchg
C        write(6,'("flmbiter: nbpts=",i9," nbdchg=",i9," %nbdchg=",f9.2,
C    &         " nfit=",i9,a)')nbpts2,nbdchg,
C    &                 100.*float(nbdchg)/float(ninside),nfit,cr
         nbpts1 = nbpts2

c        Interchange the location of the next iteration.
         itmp = it1
         it1 = it2
         it2 = itmp

         endif      !end only if not last iteration conditional.

         t2 = time()
C        write(6,'("lmbfit: time for current iteration=",i,a)') t2-t1,cr

      enddo     !end of iterative fit loop.

C     if(maxiter.gt.1) write(6,'("flmbiter: unable to fit in",i3,
C    &                    " iterations.",a)') maxiter,cr
      if( (it2 .eq. 2) .and. (maxiter .ge. 2) )then
         do j=1,idim2
         do i=1,idim1
            fit(i,j,1) = fit(i,j,2)
         enddo
         enddo
      endif

      return
      end
