      PROGRAM HYBSVD_DRV
C
C     ALGORITHM 581, COLLECTED ALGORITHMS FROM ACM.
C     ALGORITHM APPEARED IN ACM-TRANS. MATH. SOFTWARE, VOL.8, NO. 1,
C     MAR., 1982, P. 84.
C
C***********************************************************************
C
C       THIS FILE CONTAINS PROGRAMS AND SUBROUTINES THAT ARE RELATED
C       TO THE PAPER 'AN IMPROVED ALGORITHM FOR COMPUTING THE SINGULAR
C       VALUE DECOMPOSITION' BY T.F. CHAN, WHICH WAS SUBMITTED FOR
C       PUBLICATIONS IN ACM TOMS.
C
C       THEY ARE LISTED BELOW IN THE ORDER THAT THEY APPEAR IN THE
C       FILE:
C
C           1) DRIVER FOR THE ROUTINE HYBSVD TO COMPUTE THE SVD
C              OF A TEST MATRIX AND ITS TRANSPOSE.
C
C           2) ROUTINE HYBSVD ... HYBRID ALGORITHM ROUTINE.
C
C           3) ROUTINE MGNSVD ... HYBRID ALGORITHM, ASSUMES M .GE. N.
C
C           4) ROUTINE GRSVD  ... GOLUB-REINSCH ALGORITHM ROUTINE
C
C           5) BLAS ROUTINE   ... SSWAP, FOR SWAPPING VECTORS
C
C           6) ROUTINE SRELPR ... COMPUTES MACHINE PRECISION
C
C           7) ROUTINE SSVDC  ... LINPACK SVD ROUTINE
C
C           8) BLAS ROUTINES  ... SAXPY, SDOT, SNRM2, SROT, SROTG,
C                                 SSCAL.
C
C           9) ROUTINE SVD    ... EISPACK SVD ROUTINE
C
C          10) ROUTINE MINFIT ... EISPACK MINFIT ROUTINE
C
C       THE ROUTINES 2) TO 6) CONSTITUTE THE PACKAGE THAT IMPLEMENTS
C       THE NEW HYBRID ALGORITHM AND CAN BE USED BY THEMSELVES.
C
C       PLEASE ADDRESS COMMENTS AND SUGGESTIONS TO:
C
C           TONY CHAN
C           COMPUTER SCIENCE DEPT., YALE UNIV.,
C           BOX 2158, YALE STATION,
C           NEW HAVEN, CT 06520.
C
C***********************************************************************
C
C       DRIVER FOR HYBSVD, A ROUTINE FOR COMPUTING THE SVD OF A MATRIX.
C       RESULTS ARE COMPARED TO THOSE PRODUCED BY SSVDC FROM LINPACK,
C       SVD AND MINFIT FROM EISPACK.
C       PREPARED FOR ACM TRANS. ON MATH. SOFTWARE.
C
C       MATRIX TESTED:  A(I,J) = 1/(I+J),  1ST RHS(I) = 1
C               T                          2ND RHS(I) = I
C       (A AND A )
C
C       PROGRAMMED BY
C           TONY CHAN
C           COMPUTER SCIENCE DEPT., YALE UNIV.,
C           BOX 2158, YALE STATION,
C           NEW HAVEN, CT 06520.
C
C       LAST REVISION.. JANUARY, 1982.
C
      REAL AE(15,15), AH(10,10), AL(20,20), WORK(10), E(10)
      REAL UH(11,10), VH(12,20), Z(13,20), BH(14,5), WH(10)
      REAL UE(15,10), VE(15,10), BE(15,5), WE(10)
      REAL UL(18,10), VL(19,10), WL(10)
      LOGICAL MATU, MATV
      INTEGER NUH, NVH, NZ, NBH
      INTEGER NUE, NVE, NBE
      INTEGER NUL, NVL
      INTEGER NAE, NAH, NAL, M, N, IRHS, IERR, NOUT, JOB
      DATA NAH /10/, NUH /11/, NVH /12/, NZ /13/, NBH /14/
      DATA NUE /15/, NVE /15/, NBE /15/, NAE /15/
      DATA NUL /18/, NVL /19/, NAL /20/
C
C     THIS IS OUTPUT UNIT NUMBER.
      DATA NOUT /6/
C
C     FIRST CASE... A AND RHS B.
C
      M = 8
      N = 4
      IRHS = 2
C
      DO 50 IU=1,2
        DO 40 IV=1,2
          IF (IU.EQ.2) MATU = .TRUE.
          IF (IU.EQ.1) MATU = .FALSE.
          IF (IV.EQ.2) MATV = .TRUE.
          IF (IV.EQ.1) MATV = .FALSE.
C
C       JOB PARAMETER NEEDED BY SSVDC.
          JOB = (IU-1)*20 + IV - 1
C
C       SET UP MATRIX
C
          DO 20 I=1,M
            BH(I,1) = 1.0
            BH(I,2) = FLOAT(I)
            BE(I,1) = 1.0
            BE(I,2) = FLOAT(I)
            DO 10 J=1,N
              AH(I,J) = 1./FLOAT(I+J)
              AE(I,J) = AH(I,J)
              AL(I,J) = AH(I,J)
   10       CONTINUE
   20     CONTINUE
C
          WRITE (NOUT,99999)
99999     FORMAT (/4H A ./)
          DO 30 I=1,M
            WRITE (NOUT,99998) (AH(I,J),J=1,N)
99998       FORMAT (5E15.7)
   30     CONTINUE
C
          WRITE (NOUT,99997)
99997     FORMAT (//45H ******************* HYBSVD *****************//)
          CALL HYBSVD(NAH, NUH, NVH, NZ, NBH, M, N, AH, WH, MATU, UH,
     *     MATV, VH, Z, BH, IRHS, IERR, WORK)
          CALL PRINTS(WH, UH, VH, IERR, BH, NUH, NVH, NBH, MATU, MATV,
     *     M, N, NOUT, IRHS)
C
          WRITE (NOUT,99996)
99996     FORMAT (//45H ******************* EISPAK *****************//)
          CALL SVD(NUE, M, N, AE, WE, MATU, UE, MATV, VE, IERR, WORK)
          CALL MINFIT(NUE, M, N, AE, WE, IRHS, BE, IERR, WORK)
          CALL PRINTS(WE, UE, VE, IERR, BE, NUE, NVE, NBE, MATU, MATV,
     *     M, N, NOUT, IRHS)
C
          WRITE (NOUT,99995)
99995     FORMAT (//45H ******************* SSVDC  *****************//)
          CALL SSVDC(AL, NAL, M, N, WL, E, UL, NUL, VL, NVL, WORK, JOB,
     *     IERR)
          CALL PRINTS(WL, UL, VL, IERR, E, NUL, NVL, NBH, MATU, MATV,
     *     M, N, NOUT, IRHS)
C
          CALL DIFF(WE, WH, WL, UE, UH, UL, VE, VH, VL, BH, BE, NUE,
     *     NUH, NUL, NVE, NVH, NVL, NBH, NBE, M, N, NOUT, IRHS, MATU,
     *     MATV)
   40   CONTINUE
   50 CONTINUE
C                      T
C     SECOND CASE...  A
C
      M = 4
      N = 8
      IRHS = 2
C
      DO 100 IU=1,2
        DO 90 IV=1,2
          IF (IU.EQ.2) MATU = .TRUE.
          IF (IU.EQ.1) MATU = .FALSE.
          IF (IV.EQ.2) MATV = .TRUE.
          IF (IV.EQ.1) MATV = .FALSE.
C
C       JOB PARAMETER NEEDED BY SSVDC.
          JOB = (IU-1)*20 + IV - 1
C
C       SET UP MATRIX
C
          DO 70 I=1,M
            BH(I,1) = 1.
            BH(I,2) = FLOAT(I)
            BE(I,1) = 1.
            BE(I,2) = FLOAT(I)
            DO 60 J=1,N
              AH(I,J) = 1./FLOAT(I+J)
              AE(I,J) = AH(I,J)
              AL(I,J) = AH(I,J)
   60       CONTINUE
   70     CONTINUE
C
          WRITE (NOUT,99999)
          DO 80 I=1,M
            WRITE (NOUT,99998) (AH(I,J),J=1,N)
   80     CONTINUE
C
          WRITE (NOUT,99997)
          CALL HYBSVD(NAH, NUH, NVH, NZ, NBH, M, N, AH, WH, MATU, UH,
     *     MATV, VH, Z, BH, IRHS, IERR, WORK)
          CALL PRINTS(WH, UH, VH, IERR, BH, NUH, NVH, NBH, MATU, MATV,
     *     M, N, NOUT, IRHS)
C
          WRITE (NOUT,99996)
          CALL SVD(NUE, M, N, AE, WE, MATU, UE, MATV, VE, IERR, WORK)
          CALL MINFIT(NUE, M, N, AE, WE, IRHS, BE, IERR, WORK)
          CALL PRINTS(WE, UE, VE, IERR, BE, NUE, NVE, NBE, MATU, MATV,
     *     M, N, NOUT, IRHS)
C
          WRITE (NOUT,99995)
          CALL SSVDC(AL, NAL, M, N, WL, E, UL, NUL, VL, NVL, WORK, JOB,
     *     IERR)
          CALL PRINTS(WL, UL, VL, IERR, E, NUL, NVL, NBH, MATU, MATV,
     *     M, N, NOUT, IRHS)
          CALL DIFF(WE, WH, WL, UE, UH, UL, VE, VH, VL, BH, BE, NUE,
     *     NUH, NUL, NVE, NVH, NVL, NBH, NBE, M, N, NOUT, IRHS, MATU,
     *     MATV)
   90   CONTINUE
  100 CONTINUE
      STOP
      END
