;+
; NAME		ldf
;
; PURPOSE	Call Peter Bandurian's flmbiter.f routine
;		to compute limb darkening.
;
; AUTHOR	Randy Meisner   HAO/NCAR
;
; HISTORY	Modifications by Andrew L. Stanger   HAO/NCAR
;-

FUNCTION ldf, img, xc, yc, rx, ry, niter

   niter = LONG (niter)

   img = FLOAT (img)

   xc = FLOAT (xc) & yc = FLOAT (yc)
   rx = FLOAT (rx) & ry = FLOAT (ry)

   sz = SIZE (img) & nx = sz (1) & ny = sz (2)

   rad_t = MAX ([rx, ry]) & rad_lim = rad_t-4.0

   weight = FLTARR (nx, ny)

   m1 = 2L & m2 = 2L

   ex = [1L, 3] & nex = n_elements(ex)

   dimcoef = 8L

   fit = FLTARR (nx, ny, 2)

   rms = FLTARR (niter)

   coef = FLTARR (dimcoef, niter)

   npts = LONARR (3, niter)

   envchk = strpos (strupcase (!version.os), 'IRIX')

   IF (envchk EQ 0) THEN BEGIN

      dummy = call_external ('/riker/d/acos/chip/sgi/flmb.so', 'flmbext_', $
			     xc, yc, rad_t, rad_lim, img, nx, ny,	 $
			     weight, m1, m2, ex, nex, fit, niter, rms,	 $
			     coef, dimcoef, npts)

   ENDIF ELSE BEGIN

       dummy = call_external ('/riker/d/acos/chip/sun/flmb.so', 'flmbext_', $
			      xc, yc, rad_t, rad_lim, img, nx, ny,	 $
			      weight, m1, m2, ex, nex, fit, niter, rms,	 $
			      coef, dimcoef, npts)
   ENDELSE

   ldfit = fit (*, *, 0)
   RETURN, ldfit

END
