      subroutine lmbfit(x1,x2,z,nx1,nx2,m1,m2,ex,nex,weight,
     &                  mu,mudim1,mudim2,coef,dimcoef)

c PURPOSE:
c	Two-dimensional polynomial least squares fit with optional error
c       estimates.  This version uses SVDCMP.

c ***GIVE FITTED POLYNOMIAL HERE!!!***********
c?	The basis function for polynomials, is R(i,[j,k,l]) = x(i)^j;y(i)^k;mu^l

c INPUTS:
c	X1 = 1st independent variable vector.
c	X2 = 2nd independent variable vector.
c	 Z = dependent variable vector, should be same size as X1*X2.
c    nx1 = dimension in 1st dimension.
c    nx2 = dimension in 2nd dimension.
c	M1 = the degree of the polynomial, along 1st coord.
c	M2 = the degree of the polynomial, along 2nd coord.
c       EX = array of powers in R (MU) to be fitted.
c   WEIGHT = weight to be given for each fitted point of Z.
c       MU = array of precomputed MU's.

c OUTPUTS:
c     COEF = the fitting coefficients.
c  DIMCOEF = the dimension of fitting coefficients array.

      integer nx1,nx2,m1,m2,nex,ex(nex),dimcoef,mudim1,mudim2
      real x1(nx1), x2(nx2), z(nx1,nx2) ,weight(nx1,nx2)
      real mu(mudim1,mudim2), coef(dimcoef)

      integer adim1, adim2, mxcoef
      parameter (adim1=2048, adim2=2048, mxcoef=8)
      real a(adim1*adim2,mxcoef), b(adim1*adim2)
      real  w(mxcoef), v(mxcoef,mxcoef)

      integer x1_ord, x2_ord
      integer i,j,mx_ord,ntot,i_coef,iord,iex,ir,idifex
      integer mn,mx,nr,k,ncoef, offset

      integer irhs, ierr
      real rv1(mxcoef), zz
      logical matu, matv
      character*1 cr

c  Threshold used in editing singular values. (Is this optimal?)
      real thresh, wmin,wmax
      parameter (thresh=1.0e-9)

c  Make carrage returns in call_external work.
      cr = char(13)

c  Check dimensions.
      if( (nx1.gt.adim1).or.(nx2.gt.adim2) )then
C        write(6,'("lmbfit: need more space!")')
C        write(6,'("lmbfit: nx1,nx2=",2i," adim1,adim2=",2i)')
C    &                      nx1,nx2,adim1,adim2
         stop
      endif
      if (dimcoef.gt.mxcoef) then
C        write(6,'("lmbfit: need more space!")')
C        write(6,'("lmbfit: dimcoef=",i," mxcoef=",i)') dimcoef,mxcoef
         stop
      endif

c  Compute the number of coefficients in the bi-polynomial.
      mx=max(m1,m2)
      mn=min(m1,m2)
      nr=nex
      if (nr.eq.1) then
         if (ex(1).eq.0) nr=0
      endif
      ncoef = (mx+1)*(mx+2) - (mx+1)*(mx+2-(mn+1)) - mn*(mn+1)/2 + nr
      if (ncoef.gt.dimcoef) then
C        write(6,'("lmbfit: need more space, ncoef=",i," dimcoef=",i)')
C    &                        ncoef,dimcoef
         stop
      endif

c  Construct the polynomial design matrix.
      mx_ord = max(m1, m2)
      ntot = nx1*nx2

c  First, the x,y contribution to the design matrix.
      i_coef = 0
      do iord=0,mx_ord
         do x2_ord=0,m2
            do x1_ord=0,m1
               if ((x1_ord+x2_ord) .eq. iord) then
                  i_coef = i_coef + 1
                  do j=1,nx2
                  do i=1,nx1
                     if (weight(i,j).ne.0.) then
                        a((j-1)*nx1+i,i_coef) = 
     &                       weight(i,j) * x1(i)**x1_ord * x2(j)**x2_ord
                     else
                        a((j-1)*nx1+i,i_coef) = 0.0
                     endif
                  enddo
                  enddo
               endif
            enddo
         enddo
      enddo

c  Next calculate r**ex contribution to the design matrix.  Note that
c  if ex eq 0 then there is no r**ex contribution to be made which is
c  taken account with nex=0.
      if (nex.eq.1) then
         iex = ex(nex)
         i_coef = i_coef + 1
C        write(6,'("lmbfit: horrors, got in nex=1",a)') cr
         do j=1,nx2
         do i=1,nx1
            if (weight(i,j).ne.0.) then
               a((j-1)*nx1+i, i_coef) = weight(i,j) * mu(i,j)**iex
            else
               a((j-1)*nx1+i, i_coef) = 0.0
            endif
         enddo
         enddo
      else
         do ir=1,nex
            i_coef = i_coef + 1
            if (ir.eq.1) then 
               iex = ex(1)
               do j=1,nx2
               do i=1,nx1
                  k = (j-1)*nx1+i
                  if (weight(i,j).ne.0.) then
                     a((j-1)*nx1+i,i_coef) = weight(i,j) * mu(i,j)**iex
                  else
                     a((j-1)*nx1+i, i_coef) = 0.0
                  endif
               enddo
               enddo
            else
               if( (ex(ir)-iex) .eq. 1 )then
C                 write(6,'("lmbfit: horrors, got in ex-iex",a)') cr
                  do j=1,nx2
                  do i=1,nx1
                     if (weight(i,j).ne.0.) then
                        k = (j-1)*nx1+i
                        a(k,i_coef) = a(k,i_coef-1)*mu(i,j)
                     else
                        k = (j-1)*nx1+i
                        a(k,i_coef) = 0.0
                     endif
                  enddo
                  enddo
                  iex=ex(ir)
               else
                  idifex = ex(ir)-iex
                  do j=1,nx2
                  do i=1,nx1
                     k = (j-1)*nx1+i
                     if (weight(i,j).ne.0.) then
                        k = (j-1)*nx1+i
                        a(k,i_coef) = a(k,i_coef-1) * mu(i,j)**idifex
                     else
                        k = (j-1)*nx1+i
                        a(k,i_coef) = 0.0
                     endif
                  enddo
                  enddo
                  iex=ex(ir)
               endif
            endif
         enddo

      endif

c     write(6,'("lmbfit: completed design matrix")')
c     write(6,'("x=")')
c     write(6,'(6(x,g12.5))') (x1(i),i=251,260)
c     write(6,'("y=",g12.5)') x2(256)
c     write(6,'("lmbfit: some of a=")')
c     offset=512*255+250
c     write(6,'(6(x,g12.5))') ((a(offset+i,j),i=1,12),j=1,8)

c  Apply weights to B.
      k=0
      do j=1,nx2
      do i=1,nx1
         k=k+1
         b(k) = z(i,j)*weight(i,j)
      enddo
      enddo

c  Do the svd.
c     call svdcmp(a,ntot,ncoef,adim1*adim2,mxcoef,w,v)
      matu = .false.
      matv = .true.
      irhs = 1
      ierr = 0
      call hybsvd(adim1*adim2, adim1*adim2, mxcoef, mxcoef,
     &             adim1*adim2, nx1*nx2, ncoef, a, w, matu, a,
     &              matv, v, zz, b, irhs, ierr, rv1)
      if (ierr.ne.0) then
C        write(6,'("lmbfit: hybsvd returns ierr=",i)') ierr
         stop
      endif

c  Check for singular values, w, that may be beyond threshold.
      wmax = 0.
      do i=1,ncoef
         if (w(i).gt.wmax) wmax=w(i)
      enddo
      wmin = wmax*thresh
      do i=1,ncoef
         if (w(i).lt.wmin) then
C           write(6,'("lmbfit: found a singular value in ",i2,
C    &                   "''th coefficient.",a)') i,cr
C           write(6,'("lmbfit: singular values return from svdcmp",a)')
C    &                        cr
C           write(6,'(5f,a)') (w(j),j=1,ncoef),cr
            w(i)=0.0
         endif
c        NOTE: for HYBSVD only!
         if (w(i).ne.0.0) w(i)=1.0/w(i)
      enddo

c  Solve for coefficients.
c     call svbksb(a,w,v,ntot,ncoef,adim1*adim2,mxcoef,b,coef)
      do j=1,ncoef
         coef(j) = 0.0
         do i=1,ncoef
            coef(j) = coef(j) + v(j,i)*w(i)*b(i)
         enddo
      enddo

      return
      end
 
