      SUBROUTINE MINFIT(NM, M, N, A, W, IP, B, IERR, RV1)
C
      INTEGER I, J, K, L, M, N, II, IP, I1, KK, K1, LL, L1, M1, NM,
     * ITS, IERR
      REAL A(NM,N), W(N), B(NM,IP), RV1(N)
      REAL C, F, G, H, S, X, Y, Z, EPS, SCALE, MACHEP
      REAL SQRT, AMAX1, ABS, SIGN
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE MINFIT,
C     NUM. MATH. 14, 403-420(1970) BY GOLUB AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL II-LINEAR ALGEBRA, 134-151(1971).
C
C     THIS SUBROUTINE DETERMINES, TOWARDS THE SOLUTION OF THE LINEAR
C                                                        T
C     SYSTEM AX=B, THE SINGULAR VALUE DECOMPOSITION A=USV  OF A REAL
C                                         T
C     M BY N RECTANGULAR MATRIX, FORMING U B RATHER THAN U.  HOUSEHOLDER
C     BIDIAGONALIZATION AND A VARIANT OF THE QR ALGORITHM ARE USED.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.  NOTE THAT NM MUST BE AT LEAST
C          AS LARGE AS THE MAXIMUM OF M AND N,
C
C        M IS THE NUMBER OF ROWS OF A AND B,
C
C        N IS THE NUMBER OF COLUMNS OF A AND THE ORDER OF V,
C
C        A CONTAINS THE RECTANGULAR COEFFICIENT MATRIX OF THE SYSTEM,
C
C        IP IS THE NUMBER OF COLUMNS OF B.  IP CAN BE ZERO,
C
C        B CONTAINS THE CONSTANT COLUMN MATRIX OF THE SYSTEM
C          IF IP IS NOT ZERO.  OTHERWISE B IS NOT REFERENCED.
C
C     ON OUTPUT-
C
C        A HAS BEEN OVERWRITTEN BY THE MATRIX V (ORTHOGONAL) OF THE
C          DECOMPOSITION IN ITS FIRST N ROWS AND COLUMNS.  IF AN
C          ERROR EXIT IS MADE, THE COLUMNS OF V CORRESPONDING TO
C          INDICES OF CORRECT SINGULAR VALUES SHOULD BE CORRECT,
C
C        W CONTAINS THE N (NON-NEGATIVE) SINGULAR VALUES OF A (THE
C          DIAGONAL ELEMENTS OF S).  THEY ARE UNORDERED.  IF AN
C          ERROR EXIT IS MADE, THE SINGULAR VALUES SHOULD BE CORRECT
C          FOR INDICES IERR+1,IERR+2,...,N,
C
C                                   T
C        B HAS BEEN OVERWRITTEN BY U B.  IF AN ERROR EXIT IS MADE,
C                       T
C          THE ROWS OF U B CORRESPONDING TO INDICES OF CORRECT
C          SINGULAR VALUES SHOULD BE CORRECT,
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          K          IF THE K-TH SINGULAR VALUE HAS NOT BEEN
C                     DETERMINED AFTER 30 ITERATIONS,
C
C        RV1 IS A TEMPORARY STORAGE ARRAY.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C     ********** MACHEP IS A MACHINE DEPENDENT PARAMETER SPECIFYING
C                THE RELATIVE PRECISION OF FLOATING POINT ARITHMETIC.
C
C                **********
      MACHEP = 2.**(-26)
C
      IERR = 0
C     ********** HOUSEHOLDER REDUCTION TO BIDIAGONAL FORM **********
      G = 0.0
      SCALE = 0.0
      X = 0.0
C
      DO 220 I=1,N
        L = I + 1
        RV1(I) = SCALE*G
        G = 0.0
        S = 0.0
        SCALE = 0.0
        IF (I.GT.M) GO TO 120
C
        DO 10 K=I,M
          SCALE = SCALE + ABS(A(K,I))
   10   CONTINUE
C
        IF (SCALE.EQ.0.0) GO TO 120
C
        DO 20 K=I,M
          A(K,I) = A(K,I)/SCALE
          S = S + A(K,I)**2
   20   CONTINUE
C
        F = A(I,I)
        G = -SIGN(SQRT(S),F)
        H = F*G - S
        A(I,I) = F - G
        IF (I.EQ.N) GO TO 60
C
        DO 50 J=L,N
          S = 0.0
C
          DO 30 K=I,M
            S = S + A(K,I)*A(K,J)
   30     CONTINUE
C
          F = S/H
C
          DO 40 K=I,M
            A(K,J) = A(K,J) + F*A(K,I)
   40     CONTINUE
   50   CONTINUE
C
   60   IF (IP.EQ.0) GO TO 100
C
        DO 90 J=1,IP
          S = 0.0
C
          DO 70 K=I,M
            S = S + A(K,I)*B(K,J)
   70     CONTINUE
C
          F = S/H
C
          DO 80 K=I,M
            B(K,J) = B(K,J) + F*A(K,I)
   80     CONTINUE
   90   CONTINUE
C
  100   DO 110 K=I,M
          A(K,I) = SCALE*A(K,I)
  110   CONTINUE
C
  120   W(I) = SCALE*G
        G = 0.0
        S = 0.0
        SCALE = 0.0
        IF (I.GT.M .OR. I.EQ.N) GO TO 210
C
        DO 130 K=L,N
          SCALE = SCALE + ABS(A(I,K))
  130   CONTINUE
C
        IF (SCALE.EQ.0.0) GO TO 210
C
        DO 140 K=L,N
          A(I,K) = A(I,K)/SCALE
          S = S + A(I,K)**2
  140   CONTINUE
C
        F = A(I,L)
        G = -SIGN(SQRT(S),F)
        H = F*G - S
        A(I,L) = F - G
C
        DO 150 K=L,N
          RV1(K) = A(I,K)/H
  150   CONTINUE
C
        IF (I.EQ.M) GO TO 190
C
        DO 180 J=L,M
          S = 0.0
C
          DO 160 K=L,N
            S = S + A(J,K)*A(I,K)
  160     CONTINUE
C
          DO 170 K=L,N
            A(J,K) = A(J,K) + S*RV1(K)
  170     CONTINUE
  180   CONTINUE
C
  190   DO 200 K=L,N
          A(I,K) = SCALE*A(I,K)
  200   CONTINUE
C
  210   X = AMAX1(X,ABS(W(I))+ABS(RV1(I)))
  220 CONTINUE
C     ********** ACCUMULATION OF RIGHT-HAND TRANSFORMATIONS.
C                FOR I=N STEP -1 UNTIL 1 DO -- **********
      DO 300 II=1,N
        I = N + 1 - II
        IF (I.EQ.N) GO TO 290
        IF (G.EQ.0.0) GO TO 270
C
        DO 230 J=L,N
C     ********** DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW **********
          A(J,I) = (A(I,J)/A(I,L))/G
  230   CONTINUE
C
        DO 260 J=L,N
          S = 0.0
C
          DO 240 K=L,N
            S = S + A(I,K)*A(K,J)
  240     CONTINUE
C
          DO 250 K=L,N
            A(K,J) = A(K,J) + S*A(K,I)
  250     CONTINUE
  260   CONTINUE
C
  270   DO 280 J=L,N
          A(I,J) = 0.0
          A(J,I) = 0.0
  280   CONTINUE
C
  290   A(I,I) = 1.0
        G = RV1(I)
        L = I
  300 CONTINUE
C
      IF (M.GE.N .OR. IP.EQ.0) GO TO 330
      M1 = M + 1
C
      DO 320 I=M1,N
C
        DO 310 J=1,IP
          B(I,J) = 0.0
  310   CONTINUE
  320 CONTINUE
C     ********** DIAGONALIZATION OF THE BIDIAGONAL FORM **********
  330 EPS = MACHEP*X
C     ********** FOR K=N STEP -1 UNTIL 1 DO -- **********
      DO 460 KK=1,N
        K1 = N - KK
        K = K1 + 1
        ITS = 0
C     ********** TEST FOR SPLITTING.
C                FOR L=K STEP -1 UNTIL 1 DO -- **********
  340   DO 350 LL=1,K
          L1 = K - LL
          L = L1 + 1
          IF (ABS(RV1(L)).LE.EPS) GO TO 390
C     ********** RV1(1) IS ALWAYS ZERO, SO THERE IS NO EXIT
C                THROUGH THE BOTTOM OF THE LOOP **********
          IF (ABS(W(L1)).LE.EPS) GO TO 360
  350   CONTINUE
C     ********** CANCELLATION OF RV1(L) IF L GREATER THAN 1 **********
  360   C = 0.0
        S = 1.0
C
        DO 380 I=L,K
          F = S*RV1(I)
          RV1(I) = C*RV1(I)
          IF (ABS(F).LE.EPS) GO TO 390
          G = W(I)
          H = SQRT(F*F+G*G)
          W(I) = H
          C = G/H
          S = -F/H
          IF (IP.EQ.0) GO TO 380
C
          DO 370 J=1,IP
            Y = B(L1,J)
            Z = B(I,J)
            B(L1,J) = Y*C + Z*S
            B(I,J) = -Y*S + Z*C
  370     CONTINUE
C
  380   CONTINUE
C     ********** TEST FOR CONVERGENCE **********
  390   Z = W(K)
        IF (L.EQ.K) GO TO 440
C     ********** SHIFT FROM BOTTOM 2 BY 2 MINOR **********
        IF (ITS.EQ.30) GO TO 470
        ITS = ITS + 1
        X = W(L)
        Y = W(K1)
        G = RV1(K1)
        H = RV1(K)
        F = ((Y-Z)*(Y+Z)+(G-H)*(G+H))/(2.0*H*Y)
        G = SQRT(F*F+1.0)
        F = ((X-Z)*(X+Z)+H*(Y/(F+SIGN(G,F))-H))/X
C     ********** NEXT QR TRANSFORMATION **********
        C = 1.0
        S = 1.0
C
        DO 430 I1=L,K1
          I = I1 + 1
          G = RV1(I)
          Y = W(I)
          H = S*G
          G = C*G
          Z = SQRT(F*F+H*H)
          RV1(I1) = Z
          C = F/Z
          S = H/Z
          F = X*C + G*S
          G = -X*S + G*C
          H = Y*S
          Y = Y*C
C
          DO 400 J=1,N
            X = A(J,I1)
            Z = A(J,I)
            A(J,I1) = X*C + Z*S
            A(J,I) = -X*S + Z*C
  400     CONTINUE
C
          Z = SQRT(F*F+H*H)
          W(I1) = Z
C     ********** ROTATION CAN BE ARBITRARY IF Z IS ZERO **********
          IF (Z.EQ.0.0) GO TO 410
          C = F/Z
          S = H/Z
  410     F = C*G + S*Y
          X = -S*G + C*Y
          IF (IP.EQ.0) GO TO 430
C
          DO 420 J=1,IP
            Y = B(I1,J)
            Z = B(I,J)
            B(I1,J) = Y*C + Z*S
            B(I,J) = -Y*S + Z*C
  420     CONTINUE
C
  430   CONTINUE
C
        RV1(L) = 0.0
        RV1(K) = F
        W(K) = X
        GO TO 340
C     ********** CONVERGENCE **********
  440   IF (Z.GE.0.0) GO TO 460
C     ********** W(K) IS MADE NON-NEGATIVE **********
        W(K) = -Z
C
        DO 450 J=1,N
          A(J,K) = -A(J,K)
  450   CONTINUE
C
  460 CONTINUE
C
      GO TO 480
C     ********** SET ERROR -- NO CONVERGENCE TO A
C                SINGULAR VALUE AFTER 30 ITERATIONS **********
  470 IERR = K
  480 RETURN
C     ********** LAST CARD OF MINFIT **********
      END
