function sunradec,jd
; adapted from J. Meeus, chpter 18, "Solar Coordinates"
; given the Julian Date, calculates the solar right asc & dec
; G. Ucker   18 August 1992 1992

t = (double(jd) - 2415020.0d)/36525.0d
r2d = 180.0d/!pi
d2r = !pi/180.0d

l = 279.69688 + 36000.76892d*t + 0.0003025d*t^2
m = 358.47583d + 35999.04975d*t - 0.000150d*t^2 - 0.0000033d*t^3
ec = 0.01675104d - 0.0000418d*t - 0.000000126d*t^2
c = (1.919460d - 0.004789d*t - 0.000014d*t^2)*sin(m*d2r) + $
        (0.020094d - 0.000100d*t)*sin(2.0d*m*d2r) + $
        0.000293d*sin(3.0d*m*d2r)
theta = l + c

; sun-earth distance
v = m + c
r = 1.496d8*(1.0000002d*(1.0d - ec^2)) / (1.0d + (ec*cos(v*d2r)))
rsun = 6.96d5; solar radius (in km)
rsa = 2.0d*60.0d*r2d*tan(rsun/r)


omega = ((259.1833d - 1934.1420d*t + 0.002078d*t^2) mod 360.0d)
neg = where(omega lt 0.0d)
if (neg(0) ge 0L) then omega(neg) = omega(neg) + 360.0d

epsilon = ((23.452294d - 0.0130125d*t - 0.00000164d*t^2 + $
           0.000000503d*t^3 + 0.00256d*cos(omega*d2r)) mod 360.0d)
neg = where(epsilon lt 0.0)
if (neg(0) ge 0L) then epsilon(neg) = epsilon(neg) + 360.0d

; solar apparent longitude with corrections for perturbations
; as shown in the "Higher Accuracy" section of ch 18
thetap = ((theta - 0.00569d - 0.00479d*sin(omega*d2r) + $
0.00134d*cos((153.23d + 22518.7541d*t)*d2r) + $
0.00154d*cos((216.57d + 45037.5082d*t)*d2r) + $
0.00200d*cos((312.69d + 32964.3577d*t)*d2r) + $
0.00179d*sin((350.74d + 445267.1142d*t-0.00144d*t^2)*d2r) + $
0.00178d*sin((231.19d + 20.20d*t)*d2r)) mod 360.0d)

neg = where(thetap lt 0.0d)
if (neg(0) ge 0L) then thetap(neg) = thetap(neg) + 360.0d

ra = atan( cos( epsilon*d2r ) * sin( thetap*d2r ) / $
      cos( thetap*d2r) ) * r2d

negatcosra = where(cos( thetap*d2r ) lt 0.0d)
if negatcosra(0) ge 0L then ra(negatcosra) = ra(negatcosra) + 180.0d

negatra = where(ra lt 0.0d)
if negatra(0) ge 0L then  ra(negatra) = 360.0d + ra(negatra)

dec = asin ( sin(epsilon*d2r) * sin(thetap*d2r) ) * r2d

output = [ra,dec,rsa]
return,output
end


