function ut2apsid,jd,ut
; adapted from J. Meeus, chpter 7, "Sidereal Time at Greenwich"
; This calculates the Apparent Sidereal Time from the UT 
; and the Julian Date (ET is really what should be used;
; with a small correction for the screwy earth rotation anomolies. 
; For this to work properly, the Julian
; Date is to be for 0hrs of the date desired
; UT to be entered in decimal hours, output is also in decimal hours
; G. Ucker   20 August 1992
; converted to IDL, 9 April 1994, GU

t = (double(jd) - 2415020.0d) / 36525.d

; at 0hrs UT, in decimal hrs, then into sidereal
thetao = 24.0d*(0.276919398d + 100.0021359d*t + 0.000001075d*t^2 mod 1.0d)
thetao = thetao + ut*1.002737908d

; now for the nutation in longitude correction... apparent sidereal
; epsilon calc (obliquity of ecliptic) is from chpter 15 & 18
dr = !pi/180.0d
rd = 180.0d/!pi

omega = ((259.1833d - 1934.1420d*t + 0.002078d*t^2) mod 360.0d)
neg = where(omega lt 0.0d)
if neg(0) ge 0L then omega(neg) = omega(neg) + 360.0d

epsilon = (23.452294d - 0.0130125d*t - 0.00000164d*t^2 + $
	0.000000503d*t^3 + 0.00256d*cos(omega*dr) mod 360.0d)

neg = where(epsilon lt 0.0d)
if neg(0) ge 0L then epsilon(neg) = epsilon(neg) + 360.0d

epsilon = epsilon*dr
omega = omega*dr

l = (279.6967d + 36000.7689d*t + 0.000303d*t^2)*dr
lp = (270.4342d + 481267.8831d*t - 0.001133d*t^2)*dr
m = (358.4758d + 35999.0498d*t - 0.000150d*t^2)*dr
mp = (296.1046d + 477198.8491d*t + 0.009192d*t^2)*dr

; deltapsi is in seconds of a degree
deltapsi = -1.0d*(17.2327d +0.01737d*t)*sin(omega) - $
(1.2729d + 0.00013d*t)*sin(2d*l) + 0.2088d*sin(2d*omega) - $
0.2037d*sin(2d*lp) + (0.1261d -0.00031d*t)*sin(m) + 0.0675d*sin(mp) - $
(0.0497d -0.00012d*t)*sin(2d*l+m) - 0.0342d*sin(2d*lp-omega) - $
0.0261d*sin(2d*lp-mp) + 0.0214d*sin(2*l-m) - 0.0149d*sin(2d*l-2d*lp+mp) + $
0.0124d*sin(2d*l-omega) + 0.0114d*sin(2d*lp-mp)

; the correction deltapsi*cos(epsilon)/15 is in seconds (time)
; we wish decimal hours so dividing by 3600
sid = (thetao + (deltapsi*cos(epsilon)/15.0d)/3600.0d) mod 24.0d

return, sid
end
