PRO SPGETFMT, FMT_DEF, FMT
; reads in the format and output file specifications

; number of conversion factors contained in the fmt def files
nconv = 3

nfmts = n_elements(fmt_def)
lun = lonarr(nfmts)

; get the maximum number of items to be stripped from any one format frame
nitems = intarr(nfmts)
nitem = 0
junkit=''

for i = 0,nfmts-1 do begin
	get_lun,tlun
	lun(i) = tlun
	openr,lun(i),fmt_def(i)
	for j=0,3 do readf,lun(i), junkit
	readf,lun(i),nitem
	nitems(i) = nitem
	close,lun(i)
endfor

maxnitems=max(nitems)

; open up the files for the ingest of the formats
for i = 0,nfmts-1 do openr,lun(i),fmt_def(i)

; create the output structure
; containing the formats, the file names, the sync patterns for each 
; format, etc. 
;	spyr:	string	name prefix outputfile with particular format & flight
;	sync:	bytarr	the sync pattern values
;	flen:	lwInt	expected length of the corresponding format
;	slen:	lwInt	bytes to ignore before looking for next sync
;	nitems:	Int	number of items to be stripped and saved (#files too)
;	iname:	strarr	item mneumonic will be a part of output file name
;	ipos:	lonarr	1st byte position in the format for a particular item
;	ilen:	lonarr	number of bytes starting at ipos in format for an item
;	conv:	fltarr	the conversion factors for each data item
;	

fmt=replicate({fmt_def, spyr:'', sync:bytarr(6), flen:0L, slen:0L, nitems:0, $
	iname:strarr(maxnitems), ipos:lonarr(maxnitems)-1L, $
	ilen:lonarr(maxnitems)-1L, $
	conv:fltarr(nconv,maxnitems)}, nfmts)

; set up temp variables for read (will have same data-typing)
spyr = fmt(0).spyr 
sync = fmt(0).sync
flen = fmt(0).flen
slen = fmt(0).slen
nitems = fmt(0).nitems

for j=0,nfmts-1 do begin
	readf,lun(j),spyr
	fmt(j).spyr = spyr
	readf,lun(j),sync
	fmt(j).sync = sync
	readf,lun(j),flen
	fmt(j).flen = flen
	readf,lun(j),slen
	fmt(j).slen = slen
	readf,lun(j),nitems
	fmt(j).nitems = nitems
	iconv = fmt(j).conv

	; set up temp read arrays based on number of items delineated
	; the file item list (fmt) should be separated by commas
	names = fmt(j).iname
	ipos = fmt(j).ipos
	ilen = fmt(j).ilen
	aline=''
	bcomma = byte(',')
	for i=0,fmt(j).nitems-1 do begin
	  	readf,lun(j),aline
		baline = byte(aline)
		comma = where(baline eq bcomma(0))
		if n_elements(comma) ge 2 then begin
		 names(i)=strtrim(strmid(aline,0,comma(0)),2)
		 ipos(i)=long(strmid(aline,comma(0)+1,(comma(1)-comma(0))-1))
		 ilen(i) = fix(strmid(aline,comma(1)+1,(comma(2)-comma(1))-1))
		 for k = 0,nconv-2 do $
iconv(k,i) = float(strmid(aline,comma(k+2)+1,(comma(k+3)-comma(k+2))-1))
iconv(nconv-1,i) = float(strmid(aline,comma(nconv+1)+1,24))
		endif else begin
		  print,'FATAL ERROR: Input format file does not conform to'
		  print,'   standard: not enough comma-separated'
		  print,'   items on a line  (SPGETPARSE.PRO)'
		  print,' '
		  stop
		endelse
	endfor

	fmt(j).ipos = ipos
	fmt(j).ilen = ilen
	fmt(j).iname = names
	fmt(j).conv = iconv

endfor

for i = 0,nfmts-1 do begin
	close,lun(i)
	free_lun,lun(i)
endfor

return
end
