;+
;
; NAME: 
;	CPHELP3
;
; PURPOSE:
;       Return a string array containing the requested help information.
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;       result = CPHELP(VALUE)
;
; CALLED BY:
;       CPHELP
;
; CALLS TO:
;	none
;
; INPUTS:
;       VALUE	:  the string value of the help item requested
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       Return a string array with help information
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	The value passed is the exact wording of the topic as it appears
;	in the help widget.  A change to the wording of the help topic
;	(when the widget is created - in CPHELP) or to the widget event
;	handler case value (in CPHELP_EV) or to the case values in CPHELPn
;	(where n is a number) will require the same changes occur in all 3
;	locations.
;
; PROCEDURE:
;	Return the help information associated with the input value.
;
; MODIFICATION HISTORY:
;	Oct 93 - Elaine Einfalt (HSTX)
;-


function cphelp3, value

case value of 
   'Output R and theta'  : hval = [ $
        'Output R and theta                                     ', $
	'                                                       ', $
	'This options determine the method used to display your ', $
	'R and theta values.                                    ', $
	'                                                       ', $
        'It is only possible to select one of these options at a', $
	'time.  While "No R and theta" (see Input R and Theta)  ', $
	'is selected then none of the R and theta output modes  ', $
	'will be available.                                     ', $
	'                                                       ', $
        'If all of the selection options are turned off, then   ', $
	'the last chosen option is still in effect, as long as  ', $
	'"No R and Theta" is NOT selected.                      ', $
	'                                                       ']

   '"Draw R and theta ruler"' : hval = [ $
        '"Draw R and theta ruler"                               ', $
	'                                                       ', $
	'When set, this option draws a line from sun center     ', $
	'through your input R and theta coordinate and beyond.  ', $
	'This line is subdivided into tenth of a solar radii,   ', $
	'with larger ticks at the whole solar radii distances.  ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
   '"Mark R and theta click"' : hval = [ $
        '"Mark R and theta click"                               ', $
	'                                                       ', $
	'When set, this option draws a small mark at the        ', $
	'location of the R and theta input.                     ', $
	'                                                       ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
   '"No R and theta ruler or mark"' : hval = [ $
        '"No R and theta ruler or mark"                         ', $
	'                                                       ', $
	'When set, this option will not indicate the R and theta', $
	'location upon the image.  The location will however,   ', $
	'still be printed to the message window.                ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
   '"Plot radial scan"' : hval = [ $
        '"Plot radial scan"                                     ', $
	'                                                       ', $
	'When set, this option draws a line from sun center     ', $
	'through your input R and theta coordinate and beyond.  ', $
	'                                                       ', $
	'The intensity values that fall along this line will be ', $
	'plotted to a separate window.                          ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
   '"Plot azmuthal scan"' : hval = [ $
        '"Plot azmuthal scan"                                   ', $
	'                                                       ', $
	'When set, the option draws an azmuthal line passing    ', $
	'through the R and theta coordinate.                    ', $
	'                                                       ', $
	'The intensity values that fall along this arc will be  ', $
	'plotted to a separate window.                          ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
   'Scale options for differences' : hval = [ $
        'Scale options for differences                          ', $
	'                                                       ', $
	'These scale option only become available when the      ', $
	'"Make a difference" data mode is selected.             ', $
	'                                                       ', $
	'This option gives control over how the difference image', $
	'is mapped to the color table. Subtracting images gives', $
	'values from -255 to +255. The significant values in a  ', $
	'difference image tend to be between -65 and +65, so    ', $
	'this is the default.                                   ', $
	'                                                       ', $
	'  "Auto-Scale data" :                            ', $
	'         Automatically uses the default mapping of -65 ', $
	'         to + 65.                                      ', $
	'                                                       ', $
        '  "Input data scaling"                              ', $
	'         Presents a widget that allows you to alter the', $
	'         values used to map the red (positive) and blue', $
	'         (negative) values of a difference image.      ', $
	'                                                       ', $
	'         Choosing a smaller (closer to -255) clipping  ', $
	'         value causes the negative difference values   ', $
	'         to be mapped over a wider range of numbers.   ', $
	'         This may cause significant features about -65 ', $
	'         to appear as very faint blue.                 ', $
	'                                                       ', $
	'         Choosing a larger (closer to 0) scaling       ', $
	'         value causes the negative difference values   ', $
	'         to be mapped over a smaller range of numbers. ', $
	'         This may cause significant features about -65 ', $
	'         to all be the same shade of bright blue, with ', $
	'         out distinction.                              ', $
	'                                                       ', $
	'         The same applies to the Positive (red) scaling', $
	'         value.                                        ', $
	'                                                       ']

   'Windowing options' : hval = [ $
        'Windowing options                                      ', $
	'                                                       ', $
	'These are options used to manage the IDL windows used  ', $
	'to display the C/P images.                             ', $
	'They are discussed in the next 3 topics.               ', $
	'                                                       ']
   '"Delete windows"' : hval = [ $
        '"Delete windows"                                       ', $
	'                                                       ', $
	'Displays a list of all the active IDL windows, even    ', $
	'the non-C/P windows.                                   ', $
	'                                                       ', $
	' The DELETE ALL WINDOWS button will delete all windows ', $
	' from the current IDL session.  Even non-C/P window.   ', $
	'                                                       ', $
	' You may select an individual window to delete by      ', $
	' click on the window reference.  The list shows the    ', $
	' window''s index number and an optional title banner.   ', $
	'                                                       ', $
	' Once you have selected a window to delete press the   ', $
	' DELETE SELECTED WINDOW button.                        ', $
	'                                                       ', $
	'                                                       ']
   '"Reuse display window"' : hval = [ $
        '"Reuse display window"                                 ', $
	'                                                       ', $
	'When this option is selected then the next image is    ', $
	'displayed using the currently active C/P window.       ', $
	'This will erase the current image.  The initial state  ', $
	'of the program is to reuse windows in this manner.     ', $
	'(see "New display windows")                            ', $
	'                                                       ']
   '"New display windows"' : hval = [ $
        '"New display windows"                                  ', $
	'                                                       ', $
	'When this option is selected then the next image is    ', $
	'displayed using a new window.  The previous C/P image  ', $
	'remains.  Each successive window is alternately placed ', $
	'at the top or bottom of the monitor.  This results in  ', $
	'windows hiding windows.  Be aware of how many windows  ', $
	'you really have displayed.                             ', $
	'(see "Reuse display window")                           ', $
	'                                                       ']
   '"Save" [data to a disk file]' : hval = [ $
        '"Save" [data to a disk file]                           ', $
	'                                                       ', $
	'This button save the current data in an IDL save file  ', $
	'that may be restored outside of the CP program:        ', $
	'                                                       ', $
	'     IDL> RESTORE, "CPSAVE.DAT"                        ', $
	'                                                       ', $
	'Values save in the file:                               ', $
	'                                                       ', $
	' CPABSCISSA : The Xaxis values for up to 5 scan plots  ', $
	' CPDATA     : A 448 x 448 array containing the image.  ', $
	' CPHK       : The house keeping innformation.          ', $
	' CPIH       : FOV positional value                     ', $
	' CPIMSIZ    : The number of pixels along image side.   ', $
	' CPINTENSITY: The Yaxis values for up to 5 scan plots  ', $
	' CPIV       : FOV positional value                     ', $
	' CPMAXSCALE : The value to which the image was scaled. ', $
	' CPMIDSCALE : The mid point scaling value of the image.', $
	' CPNEGFAC   : Difference image''s negative scale value  ', $
	' CPNUM_VALS_N_SC : Number of values in the saved scans.', $
	' CPPOSFAC   : Difference image''s positive scale value  ', $
	' CPRSUN     : Array of user input radial distances.    ', $
	' CPTHETA    : Array of user input theta values.        ', $
	' LAST_DIFFNAME1 : Filename for difference base image.  ', $
	' LAST_DIFFNAME2 : Filename for subtraction image.      ', $
	' LAST_DIFFNAME1 : Filename for normal image.           ', $
	'                                                       ']
   '"Help"' : hval = [ $
        '"Help"  displays a widget listing the help topics.     ', $
	'By clicking on a topic the help information is         ', $
	'displayed in a window.                                 ', $
	'                                                       ']


  else : hval = 'Topic ' + value + ' is not documented in CPHELP3.PRO'
endcase

return, hval
end
