/* mktab.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Make SMM C/P CME table using file 'cmeXX.dat' as input,
 * where XX = 80, 84, 85, 86, 87, 88, 89.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   2 September 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

main (argc, argv)

int argc ;
char *argv [] ;

{
   static char ifile [84] ;
   static char ofile [84] ;
   static char line [84] ;
   static char d    [12] ;

   int idot = 0 ;

   int i  = 0 ;
   int n  = 0 ;
   int q  = 0 ;
   int a  = 0 ;
   int l  = 0 ;
   int w  = 0 ;
   int v  = 0 ;
   int lat = 0 ;

   float m  = 0.0 ;			/* mass [g]. */
   float k  = 0.0 ;			/* kinetic energy [joules]. */
   float p  = 0.0 ;			/* potential energy [joules]. */
   float t  = 0.0 ;			/* total energy [joules]. */
   float mcme = 0.0 ;			/* CME mass [kg]. */
   float vcme = 0.0 ;			/* velocity of CME in m/s. */
   float ke = 0.0 ;			/* computed kinetic energy. */
   float pe = 0.0 ;			/* computed potential energy. */
   float te = 0.0 ;			/* computed total energy. */
   float G = 6.672e-11 ;		/* Gravitational Constant (MKS). */
   float M = 1.99e+30 ;			/* Mass of sun (kg). */
   float R = 6.96e+8 ;			/* Radius of sun (m). */
   float dk = 0.0 ;			/* diff. between kinetic energies. */
   float dp = 0.0 ;			/* diff. between potential energies. */
   float dt = 0.0 ;			/* diff. between total energies. */
   float ek = 0.0 ;			/* tolerance for kinetic energy. */
   float ep = 0.0 ;			/* tolerance for potential energy. */
   float et = 0.0 ;			/* tolerance for total energy. */

   FILE *ifd, *ofd ;

   if (argc != 2)
      {
      fprintf (stderr, "usage: %s cmeXX.dat\n", argv [0]) ;
      exit (-1) ;
      }
   else
      strcpy (ifile, argv [1]) ;

   strcpy (ofile, ifile) ;
   idot = strlen (ofile) ;
   for (i = 0 ; i < strlen (ofile); i++)
      if (ofile [i] == '.')  idot = i ;
   ofile [idot] = '\0' ;
   strcat (ofile, ".tab") ;

   ifd = fopen (ifile, "r") ;
   ofd = fopen (ofile, "w") ;

   fgets (line, 80, ifd) ;		/* Read label line from input file. */

   fprintf (stdout, "num  DOY       mass[kg]  Q  PA   lat  wid  km/s  kinetic  potential  total[j]\n") ;

   fprintf (ofd, "num  DOY       mass[kg]  Q  PA   lat  wid  km/s  kinetic  potential  total[j]\n") ;

   i = -1 ;

   /* Line loop. */

   while (fscanf (ifd, "%d  %s  %e  %d  %d  %d  %d %d  %e  %e  %e",
		  &n, d, &m, &q, &a, &l, &w, &v, &k, &p, &t) != EOF)
      {
/*
 *      fprintf (stdout, "%d  %s  %e  %d  %d  %d  %d %d  %e  %e  %e\n",
 *		       &n, d, &m, &q, &a, &l, &w, &v, &k, &p, &t) ;
 */
      i = i + 1 ;
      mcme = m * 0.001 ;			/* Convert mass to kg. */
      vcme = v * 1000.0 ;			/* Convert velocity to m/s. */

      /* Determine latitude. */

      if (a >=   0 && a <=  90)
         lat =  90 - a ;

      if (a >   90 && a <= 180)
         lat = 90 - a ;

      if (a >  180 && a <= 270)
	 lat = a - 270 ;

      if (a >  270 && a <= 360)
         lat = a - 270 ;

      if (lat != l)
	 {
	 fprintf (stderr, "error> PA: %d  l: %d  lat: %d\n", a, l, lat) ;
	 /*
	 exit (-1) ;
	  */
	 }

      /* Compute kinetic, potential & total energy. */

      if (v > 0.0)
         {
         ke = 0.5 * mcme * vcme * vcme ;	/* Kinetic energy. */

         pe = (G * M * mcme) / R ;		/* Potential energy. */

         te = ke + pe ;				/* Total energy. */
         }
      else
         {
         ke = pe = te = 0.0 ;
	 v = 0.0 ;
         }

      dk = abs (k - ke) ;
      dp = abs (p - pe) ;
      dt = abs (t - te) ;
      ek = 0.01 * ke ;
      ep = 0.01 * pe ;
      et = 0.01 * te ;

      if (v > 0.0)
	 {
         if (dk > ek)
            fprintf (stderr, "k: %9.3e  ke: %9.3e  dk: %9.3e\n", k, ke, dk) ;
         if (dp > ep)
            fprintf (stderr, "p: %9.3e  pe: %9.3e  dp: %9.3e\n", p, pe, dp) ;
         if (dt > et)
            fprintf (stderr, "t: %9.3e  te: %9.3e  dt: %9.3e\n", t, te, dt) ;
	 }

      if (w < 0)
	 w = 0 ;

      /* Write output line. */

/*
 *      fprintf (stdout,
 *               "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2e %9.2e %9.2e\n",
 *	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
 */

      if (v > 0)
	 {
         fprintf (stdout,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2e %9.2e %9.2e\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         fprintf (ofd,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2e %9.2e %9.2e\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         }
      else
         {
         fprintf (stdout,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2g %9.2g %9.2g\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         fprintf (ofd,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2g %9.2g %9.2g\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         }
      }

   fclose (ifd) ;
   fclose (ofd) ;
   exit (0) ;
}
