/* mktab89.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Make SMM C/P CME table using file 'cme89.dat' as input.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   31 August 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

main (argc, argv)

int argc ;
char *argv [] ;

{
   static char d [12] ;
   static char line [84] ;

   int lat ;

   int i  = 0 ;
   int n  = 0 ;
   int q  = 0 ;
   int a  = 0 ;
   int l  = 0 ;
   int w  = 0 ;
   int v  = 0 ;

   float m  = 0.0 ;
   float k = 0.0 ;
   float p = 0.0 ;
   float t = 0.0 ;

   float ke = 0.0 ;
   float pe = 0.0 ;
   float te = 0.0 ;
   float mcme = 0.0 ;
   float vcme = 0.0 ;
   float G = 6.672e-11 ;		/* Gravitational Constant (MKS). */
   float M = 1.99e+30 ;			/* Mass of sun (kg). */
   float R = 6.96e+8 ;			/* Radius of sun (m). */

   FILE *ifd, *ofd ;

   ifd = fopen ("cme89.dat", "r") ;
   ofd = fopen ("cme89.tab", "w") ;

   fprintf (ofd, "num  DOY       mass[kg]  Q  PA   lat  wid  km/s  kinetic  potential  total[j]\n") ;

   fgets (line, 80, ifd) ;		/* Read label line from input file. */
/*
 *   printf ("%s", line) ;
 */

   i = -1 ;

   /* Line loop. */

   while (fscanf (ifd, "%d %s %e %d %d %d %d %d",
	  &n, d, &m, &q, &a, &l, &w, &v) != EOF)
      {
/*
 *      printf ("%3d  %-9s %8.2e  %1d  %3d  %3d  %3d  %4d\n",
 *	       n, d, m, q, a, l, w, v) ;
 */
      i = i + 1 ;
      mcme = m * 0.001 ;		/* Convert mass to kg. */
      vcme = v * 1000.0 ;		/* Convert velocity to m/s. */

      /* Determine latitude. */

      if (a >=   0 && a <=  90)
         lat =  90 - a ;

      if (a >   90 && a <= 180)
         lat = 90 - a ;

      if (a >  180 && a <= 270)
	 lat = a - 270 ;

      if (a >  270 && a <= 360)
         lat = a - 270 ;

      /* Compute kinetic, potential & total energy. */

      if (v > 0.0)
         {
         ke = 0.5 * mcme * vcme * vcme ;	/* Kinetic energy. */

         pe = (G * M * mcme) / R ;		/* Potential energy. */

         te = ke + pe ;			        /* Total energy. */
         }
      else
         {
         ke = pe = te = 0.0 ;
	 v = 0.0 ;
         }

      if (w < 0)
	 w = 0 ;

      if (v > 0)
	 {
         fprintf (stdout,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2e %9.2e %9.2e\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         fprintf (ofd,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2e %9.2e %9.2e\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         }
      else
         {
         fprintf (stdout,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2g %9.2g %9.2g\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         fprintf (ofd,
	      "%3d  %-8s  %8.2e  %d  %3d  %3d  %3d  %4d %9.2g %9.2g %9.2g\n",
	       i, d, mcme, q, a, lat, w, v, ke, pe, te) ;
         }
      }

   fclose (ifd) ;
   fclose (ofd) ;
}
