;
PRO find_center, im, minint, xc, yc, r
; calls 
;       delvarx.pro, fitcircle.pro
; from randy Meisner initgeom.pro, modified for MLSO Ha data 



ON_ERROR, 2


IF(N_PARAMS() NE 5) THEN BEGIN
  PRINT, ''
  PRINT, 'Usage:  find_center, im, minint, xc, yc, r
  PRINT, ''
  RETURN
ENDIF

; Get image size information.

isize = size(im)
nx = isize(1) & ny = isize(2)


;print threshold values
minint=220

; Create bi-level image, based on the input minimum limb intensity.

wim = im GT minint


; Create sobel edge enhanced image of bi-level image.

sim = sobel(wim)
sim = sim GT 0                  ; make sobel image bi-level

tvscl, sim

; delete bi-level image to save some memory.
delvarx, wim


; Avoid edge effects - set first 1% of edge pixels to zero.

dx = fix(nx/100.0) & dy = fix(ny/100.0)

sim(*, 0:dy) = 0.0           ; bottom
sim(0:dx, *) = 0.0           ; left
sim(nx-dx:nx-1, *) = 0.0     ; right
sim(*, ny-dy:ny-1) = 0.0     ; top


; Initialize storage arrays.

xpts = [-1.0] & ypts = [-1.0]


; Loop over all pixels in the sobel image.  If the pixel value is greater than
; zero, then record the pixel location in the image as an edge.

FOR i = 0, nx-1 DO BEGIN
   FOR j = 0, ny-1 DO BEGIN
      
      IF(sim(i, j) GT 0) THEN BEGIN
         xpts = [xpts, i]
         ypts = [ypts, j]
      ENDIF
      
   ENDFOR
ENDFOR

; delete sobel image to save some memory.
 delvarx, sim


; Transpose the edge coordinate arrays for input into fitellipse.pro.

xpts = transpose(xpts(where(xpts NE -1.0)))
ypts = transpose(ypts(where(ypts NE -1.0)))

; Perform a least-squares fit of the edge/limb points to a circle 

fitcircle, xpts, ypts, xc, yc, r
 
print, ' Circle Fit:', xc, yc, r, $
   format = '(/, a, 1x, f7.2, 1x, f7.2, 1x, f7.2)'
 
;find pixels offlimb

xx = findgen(nx,ny)mod(nx)
yy = transpose( findgen(ny,nx)mod(ny) )

r_all = sqrt( (xx-xc)^2. + (yy-yc)^2. )

; set the original image, im, as zeros outside of 5 pixel from radius 
wout=where(r_all gt r+5)
im(wout)=0.0

;repeat sobel filter fitting 

minint=200

wim = im GT minint

sim = sobel(wim)
sim = sim GT 0         

tvscl, sim 

delvarx, wim

; Initialize storage arrays and loop over pixels

xpts = [-1.0] & ypts = [-1.0]


; Loop over all pixels in the sobel image.  If the pixel value is greater than
; zero, then record the pixel location in the image as an edge.

FOR i = 0, nx-1 DO BEGIN
   FOR j = 0, ny-1 DO BEGIN
      
      IF(sim(i, j) GT 0) THEN BEGIN
         xpts = [xpts, i]
         ypts = [ypts, j]
      ENDIF
      
   ENDFOR
ENDFOR

 delvarx, sim

; Transpose the edge coordinate arrays for input into fitellipse.pro.

xpts = transpose(xpts(where(xpts NE -1.0)))
ypts = transpose(ypts(where(ypts NE -1.0)))

; Perform a least-squares fit of the edge/limb points to a circle 

fitcircle, xpts, ypts, xc, yc, r
 
print, ' Circle Fit:', xc, yc, r, $
   format = '(/, a, 1x, f7.2, 1x, f7.2, 1x, f7.2)'
 

image=im & image(*)=50
wgood = where( r_all lt r )
image(wgood)=100

tv, bytscl(image,0,255)
for i = 0, n_elements(xpts)-1 DO plots,[xpts(i)],[ypts(i)],psym=3, /dev

;plot laplacian 

;kern = [[-1., -1., -1.], [-1., 8., -1.], [-1., -1., -1.]]
 
;lap = convol(im, kern)

;plot, lap(*,400)


RETURN

END

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; End of 'initgeom.pro'.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
