;+
;  fixmk4.pro
;
;  Open fixlist to get a list of filename roots to repair.
;
;  Root is like c19_11
;  Files with .i, .q, and .u extensions to root are read.
;  The .sk file of that root is read.
;  I, Q, and U are normalized by sky transmission.
;  Wraparounds are looked for and corrected.
;  Bias offset is corrected using first 10 pixels for error
;  and scaling all pixels using gain table.
;
;  Corrector table is used to correct bias from all scans
;  by first removing gain, correcting bias, then re-applying gain.
;
;  For old camera data, a non linear correction is applied before
;  re-applying gain.
;
;  Settings are read from the 'settings' file in the current directory
;
;  Non-inear version:
;  7/24/01 revision DFE
;  7/27/01 subscripts now az,height.  Bias from reference pixels 
;	   applied to each height now the only bias removal in this scheme.  DFE
;-

;--- Define variables and read the settings file

olddalsa = 'y'
dostreach = 'y'
correctfile0 = 'fred'
correctfile1 = 'fred'

;--- Reading settings information.

CLOSE, 15 & OPENR, 15, 'settings'
READF, 15, olddalsa
olddalsa = STRTRIM (olddalsa)
READF, 15, kappa
READF, 15, phaseoffset
READF, 15, correctfile0
READF, 15, correctfile1
READF, 15, dostreach
dostreach = STRTRIM (dostreach)
READF, 15, linelow
READF, 15, linehigh
READF, 15, pixoff
READF, 15, pixscale
READF, 15, minrad
READF, 15, maxrad
READF, 15, iscale
READF, 15, ioffset
READF, 15, qscale
READF, 15, qoffset
READF, 15, uscale
READF, 15, uoffset
READF, 15, lowbias
READF, 15, highbias

NUMPIX =  384
NUMANG =  720
CORONA = 1000
CAL0   =    0
CAL15  =  150
DARK   = 9999

;--- CW/CCW offset in degrees

PSHIFT = 0.5

;--- Perhaps should be in settings file?
;    This depends upon how rawt4 is coded.

zerosky = 376.

temp = 0
corgain0 = LONARR (NUMPIX)
corgain1 = LONARR (NUMPIX)
CLOSE, 2 & OPENR, 2, correctfile0 & READU, 2, corgain0
CLOSE, 2 & OPENR, 2, correctfile1 & READU, 2, corgain1
;corgain0 = swap_endian (corgain0)
;corgain1 = swap_endian (corgain1)

filename = 'fred'
!y.range = [-32768, 32767]
loadct, 16

WINDOW, 0, xsize=983, ysize=384
WINDOW, 1, xsize=numang, ysize=384, ypos=400

;--- While there are still filenames to read from fixlist,
;    read and process them.

headib  = 0 & headib  = LONARR (16)  
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)  
headpad = 0 & headpad = bytarr (1024 - 4*16 - 4*32 - 4*16)

CLOSE, 2
OPENR, 2, 'fixlist' 
READF, 2, filename
print, filename
filei = STRING  (filename + '.i')

CLOSE, 1 & OPENR, 1, filei
    READU, 1, headib
    READU, 1, headf
    READU, 1, headie
    READU, 1, headpad

;headib = swap_endian (headib)
;headf  = swap_endian (headf)
;headie = swap_endian (headie)
;head   = swap_endian (head)

CLOSE, 2
OPENR, 2, 'fixlist' 

;--- fixlist loop:

WHILE ( NOT EOF (2) ) DO BEGIN ;{
    READF, 2, filename
    print, filename
    filei = STRING (filename + '.i')			; .i

    CLOSE, 1 & OPENR, 1, filei
	READU, 1, headib
	READU, 1, headf
	READU, 1, headie
	READU, 1, headpad

;    headib = swap_endian (headib)
;    headf  = swap_endian (headf)
;    headie = swap_endian (headie)
;    head   = swap_endian (head)

    NUMPIX = headib (10)
    NUMAZ  = headib  (9)

    azpix = FINDGEN (NUMAZ)
    datai = INTARR  (numaz)
    dataq = INTARR  (numaz)
    datau = INTARR  (numaz)

    imgi      = INTARR (NUMAZ, NUMPIX)
    imgq      = INTARR (NUMAZ, NUMPIX)
    imgu      = INTARR (NUMAZ, NUMPIX)
    skyarray  = INTARR (NUMAZ)
    starray   = INTARR (NUMAZ)
    angle     = INTARR (NUMAZ)
    biasarray = FLTARR (NUMAZ)

    type = headie (0)
    IF ( type EQ CORONA ) THEN PRINT, "corona"
    IF ( type EQ CAL0 )   THEN PRINT, "cal0"
    IF ( type EQ CAL15 )  THEN PRINT, "cal15"
    IF ( type EQ DARK )   THEN PRINT, "dark"

    i = 0
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
	READU, 1, datai
;	datai = swap_endian (datai)
	imgi (*, i) = datai (*)
	i = i + 1
    ENDWHILE ;}

    irays = i

    fileq = STRING (filename + '.q')			; .q
    CLOSE, 1 &  OPENR, 1, fileq
	READU, 1, headib
	READU, 1, headf
	READU, 1, headie
	READU, 1, headpad
    i = 0

    WHILE ( not eof(1) ) DO BEGIN
	READU, 1, dataq
;	dataq=swap_endian (dataq)
	imgq (*, i)= dataq (*)
	i = i + 1
    ENDWHILE

    fileu = STRING (filename + '.u')			; .u
    CLOSE, 1 &  OPENR, 1, fileu
	READU, 1, headib
	READU, 1, headf
	READU, 1, headie
	READU, 1, headpad
    i = 0

    WHILE ( NOT EOF (1) ) DO BEGIN ;{
	READU, 1, datau
;	datau = swap_endian (datau)
	imgu (*, i) = datau (*)
	i = i + 1

	fileang = STRING (filename + '.ba')		; .ba
	CLOSE, 1 &  OPENR, 1, fileang
	    READU, 1, headib
	    READU, 1, headf
	    READU, 1, headie
	    READU, 1, headpad
	i = 0

	WHILE ( NOT EOF (1) ) DO BEGIN ;{
	    READU, 1, temp
;	    temp = swap_endian (temp)
	    angle (i) = temp
	    i = i + 1
	ENDWHILE ;}

	angle = 360.0 * float (angle) / 16384.0

	;--- .sk not used: rawt4 puts value into header.
	;
	;    Would eventually like to have this restored with value
	;    for each ray used.
	;    A/D for sky transmission needs to be fixed first.
	;    DFE 7/01

	filesky = STRING (filename+'.sk')		; .sk
	CLOSE, 1
	OPENR, 1, filesky
	    READU, 1, headib
	    READU, 1, headf
	    READU, 1, headie
	    READU, 1, headpad
	i = 0

	WHILE ( NOT EOF (1) ) DO BEGIN ;{
	    READU, 1, temp
;;	    temp = swap_endian (temp)
	    skyarray (i) = temp
	    i = i + 1
	ENDWHILE ;}


	filest = STRING (filename + '.st')		; .st
	CLOSE, 1 &  OPENR, 1, filest
	    READU, 1, headib
	    READU, 1, headf
	    READU, 1, headie
	    READU, 1, headpad
	i = 0

	WHILE ( NOT EOF (1) ) DO BEGIN ;{
	    READU, 1, temp
;	    temp = swap_endian (temp)
	    starray (i) = temp
	    i = i + 1
	ENDWHILE ;}

	stdim = size (starray)


	;--- rawt4 as of July 2001 does a good job computing sky transmission.

	IF ( olddalsa EQ 'n' ) THEN BEGIN ;{
	    sky = skyis (skyarray, skydim (1))
	    print, 'Sky transmission', sky, '=', sky/zerosky
	ENDIF ELSE BEGIN ;}{
	    print, 'rawt4 transmission', headf (10), '=', headf(10)/zerosky
	    sky = headf (10)
	ENDELSE ;}
	numy = i

	print, numy, ' azimuths'

	WSET, 0
	TV, bytscl (imgi, -32768, 32767)
	TV, bytscl (imgu, -2000, 2000)
	TV, bytscl (imgq, -1000, 5000)

	;--- Float arrays.

	imi = float (imgi)
	imq = float (imgq)
	imu = float (imgu)

	;--- Check for and repair negative Q/U in old data

	IF ( type EQ CORONA )  THEN BEGIN ;{
	FOR ii=0, numaz-1 DO BEGIN ;{
	    IF ( imq (ii, linelow - 40) LT 0.0 ) THEN BEGIN
		imq (ii, *) = -1.0 *imq (ii, *)
		imu (ii, *) = -1.0 *imu (ii, *)
	    ENDIF ;}
	ENDFOR ;}
        tv, bytscl (imq, -1000, 5000)
    ENDIF ;}

    ;--- Fix digital wraps in intensity for all but dark.
    ;    For corona fix digital wraps at low heights, 
    ;    attempting to correct more than one wrap around.
    ;    This works well for new camera 5/01 onward.

    IF ( type NE DARK ) THEN BEGIN ;{
	fred = imi (*, 35:365)
	a = WHERE ( fred LT 0 )
	IF ( a (0) NE -1 ) THEN fred (a) = fred (a) + 65536.0
	imi (*, 35:365) = fred

	IF ( type EQ CORONA ) THEN BEGIN ;{

	    FOR jj=0, numaz-1 DO BEGIN ;{
	        wrapq = 0.0
	        wrapi = 0.0

	        IF ( olddalsa EQ 'n' ) THEN BEGIN ;{

	            FOR ii=34, 20, -1 DO BEGIN ;{
		        imq (jj, ii) = imq (jj, ii) + wrapq
		        imi (jj, ii) = imi (jj, ii) + wrapi

		        IF ( imi (jj, ii) LT 0 ) THEN BEGIN ;{
			    wrapi = wrapi + 65536.0
			    imi (jj, ii) = imi (jj, ii) + 65536.0
		        ENDIF ;}

		        IF ( imq (jj, ii) LT 0 ) THEN BEGIN ;{
			    wrapq = wrapq + 65536.0
			    imq (jj, ii) = imq (jj, ii) + 65536.0
		        ENDIF ;}

		    ENDFOR ;}

	        ENDIF ELSE BEGIN ;}{

		    FOR ii=34, 20, -1 DO BEGIN ;{

		        IF ( imi (jj, ii) LT 0 ) THEN BEGIN ;{
			    imi (jj, ii) = imi (jj, ii) + 65536.0
		        ENDIF ;}

		        IF ( imq (jj, ii) LT 0 ) THEN BEGIN ;{
			    imq (jj, ii) = imq (jj, ii) + 65536.0
		        ENDIF ;}

		    ENDFOR ;}

	        ENDELSE ;}

	    ENDFOR ;}

        ENDIF ;}

    ENDIF ;}


    ;--- Remove the corrector table gain from the image.
    ;
    ;--- Correct bias error on a ray by ray basis
    ;    using offset of specified range of low pixels.
    ;
    ;--- If old Dalsa camera, apply non linear gain.
    ;    Intensity corrected by kappa function.
    ;    Polarization corrected by derivative of intensity.
    ;
    ;--- Re-apply corrector table gain.
    
    corfac = (corgain0 + corgain1) / 2.0
    
    fac = float (highbias - lowbias + 1)

    IF ( olddalsa NE 'y' ) THEN BEGIN ;{
        print, 'Linear'

        FOR ii=0, NUMAZ-1 DO BEGIN ;{
	    imi (ii, *) = imi (ii, *) / corfac
	    biasarray (ii) = TOTAL (imi (ii, lowbias:highbias)) / fac
	    imi (ii, *) = imi (ii, *) - biasarray (ii)
	    imi (ii, *) = imi (ii, *) * corfac
        ENDFOR ;}
    ENDIF ELSE BEGIN ;}{
        print, 'Nonlinear'
        FOR ii=0, NUMAZ-1 DO BEGIN ;{
	    imi (ii, *) = imi (ii, *) / corfac
	    imq (ii, *) = imq (ii, *) / corfac
	    imu (ii, *) = imu (ii, *) / corfac

	    biasarray (ii) = TOTAL (imi (ii, lowbias:highbias)) / fac
	    imi (ii, *) = imi (ii, *) - biasarray (ii)

	    fred = kappa * (imi (ii, *) ^ (kappa- 1))
	    imq (ii, *) = imq (ii, *) * fred
	    imu (ii, *) = imu (ii, *) * fred
	    imi (ii, *) = imi (ii, *) ^ kappa

	    imi (ii, *) = imi (ii, *) * corfac
	    imq (ii, *) = imq (ii, *) * corfac
	    imu (ii, *) = imu (ii, *) * corfac
        ENDFOR ;}
    ENDELSE ;}


    ;--- Fix NaN problems.

    a = WHERE ( finite (imi) EQ 0 )
    IF ( a (0) NE -1 ) THEN BEGIN ;{
         print, 'fixing NAN'
         help, a
        imi (a) = 1.0
    ENDIF ;}

    a = WHERE ( finite (imq) EQ 0 )

    IF ( a (0) NE -1 ) THEN BEGIN ;{
        print, 'fixing NAN'
        help, a
        imq (a) = 0.0
    ENDIF ;}

    a = WHERE ( finite (imu) EQ 0 )
    IF ( a (0) NE -1 ) THEN BEGIN ;{
        print, 'fixing NAN'
        help, a
        imu (a) = 0.0
    ENDIF ;}

    ;--- Don't let intensity be 0 or negative.

    IF ( type NE DARK ) THEN BEGIN ;{
        a = WHERE ( imi le 0 )
        IF ( a (0) NE -1 ) THEN BEGIN ;{
 	    print, 'fixing negative'
	    help, a
	    imi (a) = 1.0
        ENDIF ;}
    ENDIF ;}

    ;--- Make scan go the proper direction.

    ang = float (angle (0 : numaz - 1))
    newang = FINDGEN (NUMANG) / (NUMANG / 360.0) - 180.0
    rev = 0
    IF ( starray (0) GT 8000 ) THEN rev = 1

    IF ( rev EQ 1 ) THEN BEGIN ;{
	print, 'reverse'
	SHIFT pix = (180.0 - 30.0 + PSHIFT ) * NUMANG / 360.0
     ENDIF  ELSE BEGIN ;}{
	print, 'forward'
	SHIFT pix = (180.0 - 30.0 - PSHIFT ) * NUMANG / 360.0
    ENDELSE ;}

    ;--- Attempt to remove barrel angle problems.

    absc = FINDGEN (numaz)
    coef = poly_fit (absc, ang, 1, angfit)
    a = WHERE (abs (ang - angfit) GT 0.5 )

    IF ( a (0) NE -1 ) THEN BEGIN ;{
 	sofa = size (a)
	print, "Fixing ", sofa (1), " barrel problems"
	ang (a) = angfit (a)
	coef = poly_fit (absc, ang, 1, angfit)
	a = WHERE (abs (ang - angfit) GT 0.5 )

	IF ( a (0) NE -1 ) THEN BEGIN ;{
	    ang (a) = angfit (a)
	ENDIF ;}
    ENDIF ;}

    ;--- Normalize arrays for sky transmission.

    IF ( type NE DARK ) THEN BEGIN ;{
	imi = imi * zerosky / sky
	imq = imq * zerosky / sky
	imu = imu * zerosky / sky
    ENDIF ;}

    ;--- Spline fit to -180 to 180 degrees.

    imis = FLTARR (NUMANG, NUMPIX)
    imqs = FLTARR (NUMANG, NUMPIX)
    imus = FLTARR (NUMANG, NUMPIX)

    wset, 1

    print, 'spline I'
    before = systime (1)

    FOR ii=0, NUMPIX-1 DO BEGIN ;{
	imis (*, ii) = spline (angfit, imi (*, ii), newang)
    ENDFOR ;}

    imis = SHIFT (imis, shiftpix, 0)
    tv, bytscl (imis, -1000, 60000)

    print, 'spline Q'

    FOR ii=0, NUMPIX-1 DO BEGIN
	imqs (*, ii) = spline (angfit, imq (*, ii), newang)
    ENDFOR

    imqs = SHIFT (imqs, shiftpix, 0)
    tv, bytscl (imqs, -200, 3000)

    print, 'spline U'

    FOR ii=0, NUMPIX-1 DO BEGIN ;{
	imus (*, ii) = spline (angfit, imu(*, ii), newang)
    ENDFOR ;}

    imus = SHIFT (imus, shiftpix, 0)
    tv, bytscl (imus, -1000, 5000)

    after = systime (1)
    print, after - before

    headib  (9) = numang;
    headib (10) = numpix;

    CLOSE, 1
    filei = STRING (filename + '.Ci')			; .Ci
    CLOSE, 1 & OPENW, 1, filei & WRITEU, 1, headib $
	& WRITEU, 1, headf & writeu, 1, headie $
	& WRITEU, 1, headpad & writeu, 1, imis
    CLOSE, 1
    fileq = STRING (filename + '.Cq')			; .Cq
    CLOSE, 1 & OPENW, 1, fileq & WRITEU, 1, headib $
	& WRITEU, 1, headf & writeu, 1, headie $
	& WRITEU, 1, headpad & writeu, 1, imqs
    CLOSE, 1
    fileu = STRING (filename + '.Cu')			; .Cu
    CLOSE, 1 & OPENW, 1, fileu & WRITEU, 1, headib $
		& WRITEU, 1, headf & writeu, 1, headie $
		& WRITEU, 1, headpad & writeu, 1, imus
    CLOSE, 1

    IF ( dostop NE 'n' ) THEN STOP

ENDWHILE ;}

STOP

END
