;+
; NAME		fixmk4.pro
;
; PURPOSE	Fix raw Mk4 files prior to processing by 'xmk4a.pro'.
;
;		I, Q, and U are normalized by the largest 'good'
;		sky transmission value for the scan.
;		Wraparounds are looked for and corrected.
;		Bright and dark points  and some streaks are removed.
;		Bias offset is corrected using first 10 pixels for error
;		and scaling all pixels using gain table.
;
;		Open listfile to get a list of filename roots to repair.
;		Root is like c19_11
;		Files with .i, .q, and .u extensions to root are read.
;		The .sk file of that root is read.
;
; SYNTAX	fixmk4, 'fixlist'
;
; AUTHOR	David Elmore   NCAR/HAO   9 May 2001
;
; HISTORY	14 May 2001 [ALS]: Cosmetic changes; remove swap_endian calls.
;-

PRO fixmk4, listfile

splinefit = 'y'
dostop = 'n'

NUMPIX = 384
NUMANG = 720

;--- CW/CCW offset in degrees

PSHIFT = 0.5
temp = 0
zerosky = 359.7
CORONA = 1000
CAL0 = 0
CAL15 = 150
DARK = 9999

;; correctfile0 = 'e:/elmore/mlso/data/correct/gain_0.200105041903'
;; correctfile1 = 'e:/elmore/mlso/data/correct/gain_1.200105041903'

correctfile0 = '/riker/d/mlso/mk4/correct/gain_0.200105041903'
correctfile1 = '/riker/d/mlso/mk4/correct/gain_1.200105041903'

corgain0 = LONARR (NUMPIX)
corgain1 = LONARR (NUMPIX)

CLOSE, 2
OPENR, 2, correctfile0   &   READU, 2, corgain0   &   CLOSE, 2
OPENR, 2, correctfile1   &   READU, 2, corgain1   &   CLOSE, 2

;;; corgain0 = swap_endian (corgain0)
;;; corgain1 = swap_endian (corgain1)

corgain = (corgain0 + corgain1) / 2.0

hpix = FINDGEN (16)

filename = 'fred'
!y.range = [-32768, 32767]
LOADCT, 16

WINDOW, 0, xsize = 983, ysize = 384
WINDOW, 1, xsize = 720, ysize = 384, ypos = 400
WINDOW, 2, xsize = 512, ysize = 512
WINDOW, 3, xsize = 983, ysize = 384, xpos = (1600 - 983)

;--- While there are still filenames to read from listfile, 
;--- read and process them.

OPENR, 2, listfile
WHILE ( NOT EOF (2) ) DO BEGIN ;{
    READF, 2, filename
    PRINT, filename

    filei = STRING (filename + '.i')

    head   = 0 & head   = BYTARR (1024) ;;
    headib = 0 & headib = LONARR (16)
    headf  = 0 & headf  = FLTARR (24)
    headie = 0 & headie = LONARR (16)
    pad = BYTARR (800)

    top = long (0)

    CLOSE, 1
    OPENR, 1, filei & POINT_LUN, -1, top & READU, 1, headib
                                           READU, 1, headf
                                           READU, 1, headie
                      POINT_LUN,  1, top & READU, 1, head

;;; headib = swap_endian (headib)
;;; headf  = swap_endian (headf)
;;; headie = swap_endian (headie)
;;; head   = swap_endian (head)

    type = headie [8]
    IF ( type EQ CORONA )  THEN PRINT, "corona"
    IF ( type EQ CAL0 )    THEN PRINT, "cal0"
    IF ( type EQ CAL15 )   THEN PRINT, "cal15"
    IF ( type EQ DARK )    THEN PRINT, "dark"

    NUMPIX = headib [10]
    NUMAZ  = headib  [9]
    
    azpix = FINDGEN (NUMAZ)
    datai = INTARR (NUMAZ)
    dataq = INTARR (NUMAZ)
    datau = INTARR (NUMAZ)
    imgi  = INTARR (NUMPIX, NUMAZ)
    imgq  = INTARR (NUMPIX, NUMAZ)
    imgu  = INTARR (NUMPIX, NUMAZ)
    
    skyarray  = INTARR (NUMAZ)
    starray   = INTARR (NUMAZ)
    angle     = INTARR (NUMAZ)
    biasarray = FLTARR (NUMAZ)
    
    i = 0
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
        READU, 1, datai
    ;;;	datai = swap_endian (datai)
        imgi [i, *] = (datai [*])
        i = i + 1
    ENDWHILE ;}
    CLOSE, 1
    
    irays = i
    
    fileq = STRING (filename + '.q')
    OPENR, 1, fileq
    READU, 1, head
    i = 0
    
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
        READU, 1, dataq
    ;;;	dataq = swap_endian (dataq)
        imgq [i, *] = (dataq [*])
        i = i + 1
    ENDWHILE ;}
    CLOSE, 1
    
    fileu = STRING (filename + '.u')
    OPENR, 1, fileu
    READU, 1, head
    i = 0
    
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
        READU, 1, datau
    ;;;	datau = swap_endian (datau)
        imgu [i, *] = (datau [*])
        i = i + 1
    ENDWHILE ;}
    CLOSE, 1
    
    fileang = STRING (filename + '.ba')
    
    OPENR, 1, fileang
    READU, 1, head
    i = 0
    
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
        READU, 1, temp
    ;;;	temp = swap_endian (temp)
        angle [i] = temp
        i = i + 1
    ENDWHILE ;}
    CLOSE, 1
    
    angle = 360.0 * float (angle) / 16384.0
    
    filesky = STRING (filename+'.sk')
    
    OPENR, 1, filesky
    READU, 1, head
    i = 0
    
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
        READU, 1, temp
    ;;;	temp = swap_endian (temp)
        skyarray [i] = temp
        i = i + 1
    ENDWHILE ;}
    CLOSE, 1
    
    skydim = SIZE (skyarray)
    
    filest = STRING (filename + '.st')
    
    OPENR, 1, filest
    READU, 1, head
    i = 0
    
    WHILE ( NOT EOF (1) ) DO BEGIN ;{
        READU, 1, temp
    ;;;	temp = swap_endian (temp)
        starray [i] = temp
        i = i + 1
    ENDWHILE ;}
    CLOSE, 1
    
    stdim = SIZE (starray)
    
    sky = skyis (skyarray, skydim [1])
    PRINT, 'Sky transmission', sky, '=', sky / zerosky
    numy = i
    
    PRINT, numy, ' azimuths'
    
    WSET, 0
    TV, BYTSCL (TRANSPOSE (imgi), -32768, 32767)
    TV, BYTSCL (TRANSPOSE (imgu), -2000, 2000)
    TV, BYTSCL (TRANSPOSE (imgq), -1000, 5000)
    
    ;--- Fix up digital wraps here
    ;--- This must be performed before spline
    
    ;--- Float arrays
    
    imi = FLOAT (imgi)
    imq = FLOAT (imgq)
    imu = FLOAT (imgu)
    
    ;--- Allow negative intensity at ends of the array
    
    IF ( type NE DARK ) THEN BEGIN ;{
        fred = imi [35:365, *]
        a = WHERE ( fred LT 0 )
        IF ( a [0] NE -1 ) THEN fred [a] = fred [a] + 65536.0
        imi [35:365, *] = fred
    ENDIF ;}
    
    ;--- Correct bias error on a ray by ray basis using offset
    ;--- of first 10 pixels scaled by ratio of gain of pixel
    ;--- in question relative to the gain ofthe first 10.
    
    ;--- Do not allow multiplication by large values at ends of array.
    
    corfac = corgain / 65536.0
    a = WHERE ( corfac GT 1.0 )
    corfac [a] = 1.0
    
    WSET, 3
    TV, BYTSCL (TRANSPOSE (imgi), 0, 10000)
    
    FOR ii=0, NUMAZ-1 DO BEGIN ;{
        biasarray [ii] = total (float (imi [0:9, ii])) / 10.0
        imi [*, ii] = imi [*, ii] - (biasarray [ii] * corfac)
    ENDFOR ;}
    
    WSET, 2
    !y.range = 0
    PLOT, biasarray
    WSET, 0
    TV, BYTSCL (TRANSPOSE (imgi), 0, 10000)
    
    ;--- Make scan go the proper direction.
    
    ang = FLOAT (angle [0:NUMAZ-1])
    newang = FINDGEN (NUMANG) / (NUMANG / 360.0) - 180.0
    rev = 0
    IF ( starray [0] GT 8000 ) then rev = 1
    IF ( rev EQ 1 ) THEN BEGIN ;{
        PRINT, 'reverse'
        shiftpix = (180.0 - 30.0 + PSHIFT ) * NUMANG / 360.0
    ENDIF  ELSE BEGIN ;}{
        PRINT, 'forward'
        shiftpix = (180.0 - 30.0 - PSHIFT ) * NUMANG / 360.0
    ENDELSE ;}
    
    ;--- Attempt to remove barrel angle problems.
    
    absc = FINDGEN (NUMAZ)
    coef = poly_fit (absc, ang, 1, angfit)
    a = WHERE (abs (ang - angfit) GT 1 )
    
    IF ( a [0] NE -1 ) THEN BEGIN ;{
        PRINT, "Fixing ", a [0], " barrel problems"
        ang [a] = angfit [a]
        coef = poly_fit (absc, ang, 1, angfit)
        a = WHERE (abs (ang - angfit) GT 1 )
        IF ( a [0] NE -1 ) THEN BEGIN ;{
            ang [a] = angfit [a]
        ENDIF ;}
    ENDIF ;}
    
    ;--- Normalize arrays for sky transmission.
    
    imi = imi * zerosky / sky
    imq = imq * zerosky / sky
    imu = imu * zerosky / sky
    
    
    imis = FLTARR (NUMPIX, NUMANG)
    imqs = FLTARR (NUMPIX, NUMANG)
    imus = FLTARR (NUMPIX, NUMANG)
    
    WSET, 1
    
    ;--- Data interpolation:

    IF ( splinefit EQ 'y' ) THEN BEGIN ;{

        ;--- Spline fit to -180 to 180 degrees.

        PRINT, 'spline I'
        FOR ii=0, NUMPIX-1 DO BEGIN ;{
            imis [ii, *] = SPLINE (ang, imi [ii, *], newang)
        ENDFOR ;}
    
        imis = SHIFT (imis, 0, shiftpix)
        TV, BYTSCL (TRANSPOSE (imis), -1000, 60000)
    
        PRINT, 'spline Q'
        FOR ii=0, NUMPIX-1 DO BEGIN ;{
    	imqs [ii, *] = SPLINE (ang, imq [ii, *], newang)
        ENDFOR ;}
    
        imqs = SHIFT (imqs, 0, shiftpix)
        TV, BYTSCL (TRANSPOSE (imqs), -200, 3000)
    
        PRINT, 'spline U'
        FOR ii=0, NUMPIX-1 DO BEGIN ;{
    	imus [ii, *] = SPLINE (ang, imu [ii, *], newang)
        ENDFOR ;}
    
        imus = SHIFT (imus, 0, shiftpix)
        TV, BYTSCL (TRANSPOSE (imus), -1000, 5000)
    
    ENDIF ELSE BEGIN ;}{

	;--- Congrid gives wrong azimuths but is fast.

        imis = CONGRID (imi, NUMPIX, numang, cubic = -0.5)
        imis = SHIFT (imis, 0, shiftpix)
        TV, BYTSCL (TRANSPOSE (imis), -1000, 60000)

        imqs = CONGRID (imq, NUMPIX, numang, cubic = -0.5)
        imqs = SHIFT (imqs, 0, shiftpix)
        TV, BYTSCL (TRANSPOSE (imqs), -200, 3000)

        imus = CONGRID (imu, NUMPIX, numang, cubic = -0.5)
        imus = SHIFT (imus, 0, shiftpix)
        TV, BYTSCL (TRANSPOSE (imus), -1000, 5000)
    ENDELSE ;}
    
    ;--- Probably wrong indices.
    
    headib [9] = 720;
    headie [5] = 720;
    
    ;--- Write interpolated images to disc files.

    filei = STRING (filename + '.Ci')
    CLOSE, 1 & OPENW,  1, filei & WRITEU, 1, headib $
    	 & WRITEU, 1, headf & WRITEU, 1, headie $
    	 & WRITEU, 1, pad   & WRITEU, 1, imis
    CLOSE, 1
    
    fileq = STRING (filename + '.Cq')
    CLOSE, 1 & OPENW,  1, fileq & WRITEU, 1, headib $
    	 & WRITEU, 1, headf & WRITEU, 1, headie $
    	 & WRITEU, 1, pad   & WRITEU, 1, imqs
    CLOSE, 1
    
    fileu = STRING (filename + '.Cu')
    CLOSE, 1 & OPENW,  1, fileu & WRITEU, 1, headib $
    	 & WRITEU, 1, headf & WRITEU, 1, headie $
    	 & WRITEU, 1, pad   & WRITEU, 1, imus
    CLOSE, 1
    
    IF ( dostop NE 'n' ) THEN STOP
ENDWHILE ;}

; STOP

END
