;+
; NAME		sintrend.pro
;
; PURPOSE	Remove once per scan component of intensity
;
; SYNTAX	sintrend, numpix, numaz, img, imgtrend
;
;		numpix		number of pixels (radial dimension)
;		numaz		number of angles (angle direction)
;		img		image array
;		imgtrend	
;
; AUTHOR	David Elmore   HAO/NCAR   9 May 2001
;
; HISTORY	14 May 2001 [ALS]: Cosmetic changes only.
;-

pro sintrend, numpix, numaz, img, imgtrend

ang = findgen (numaz) * 2.0 * !pi / numaz
sang = sin (ang)
cang = cos (ang)

phi = fltarr (numpix)
amp = fltarr (numpix)

;--- Find amplitude and phase of sinusoidal component.

for ii = 0, NUMPIX - 1 do begin
	sprod = total (sang * img [ii, *]) / numaz
	cprod = total (cang * img [ii, *]) / numaz
	phi [ii] = atan (sprod, cprod)
	amp [ii] = 2.0 * sqrt (sprod * sprod + cprod * cprod)
endfor

imgtrend = fltarr (numpix, numaz)

for ii = 0, numpix - 1 do begin
	sph = cos (ang - phi [ii])
	imgtrend [ii, *] = sph [*] * amp [ii]
endfor

end
