;	Xmk4.pro
;
; ;	Find the X matrix for MkIV ;
;INPUT: Already spline-fit cal0, cal15, dark
;	File names are in Xmk4list
;	File names must be in order for cal0, cal15, and dark
;Output
;	Vector by pixel height:
;		bias,X21, x22, x23, x31, x32, x33
;	2-D array
;		gain
;
;	It is assumed bias for Q and U is zero
;	It is assumed X12 and X13 are zero
;
;	Routine follows proceedure in MkIII Calibration Technote
;	Matrix convention is element ( row, column )
;
;;;;;;;;;;;;;;;;;;;;;;;;;;
numpix = 384
numaz = 720
scalepix = 150
radfac = !pi/180.
!x.ticklen=1
!y.ticklen=1
!x.style=1
!y.style=1
!y.range=0
!x.range=0

;
; This is the difference between reported barrel angle
; and that needed to minimize calibrated coronal U
; This was large before correcting barrel encoder late April, 2001
phaseoffset = -7.5
phaseoffset = 0.0
;
; Values from technote
;
; Transmission of opal
Bopal = 10.11e-6
; Fractional polarization of tilt plates at 15 degrees
fpt = 0.01543

gain = fltarr(numpix,numaz)
bias = fltarr(numpix)
x21 = fltarr(numpix)
x31 = fltarr(numpix)
x22 = fltarr(numpix)
x23 = fltarr(numpix)
x33 = fltarr(numpix)
x32 = fltarr(numpix)

listfile = 'Xmk4list'
filename = 'fred'
cal0_I = fltarr(numpix,numaz)
cal0_Q = fltarr(numpix,numaz)
cal0_U = fltarr(numpix,numaz)
cal15_I = fltarr(numpix,numaz)
cal15_Q = fltarr(numpix,numaz)
cal15_U = fltarr(numpix,numaz)
dark_I = fltarr(numpix,numaz)
dark_Q = fltarr(numpix,numaz)
dark_U = fltarr(numpix,numaz)
corona_I = fltarr(numpix,numaz)
corona_Q = fltarr(numpix,numaz)
corona_U = fltarr(numpix,numaz)
cx21 = fltarr(numpix)
cx31 = fltarr(numpix)
headl = lonarr(256)

close,1 
close, 2 & openr,2,listfile
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,cal0_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,cal0_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,cal0_U & close,1
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,cal15_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,cal15_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,cal15_U & close,1
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,dark_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,dark_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,dark_U & close,1
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,corona_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,corona_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,corona_U & close,1

;
; This is the offset of MkIV relative
; to the calibration tilt plates
; Determined by minimizing calibrated coronal U

shiftpix = phaseoffset * NUMAZ/360.

cal0_i=shift(cal0_i,0,shiftpix)
cal0_q=shift(cal0_q,0,shiftpix)
cal0_u=shift(cal0_u,0,shiftpix)
cal15_i=shift(cal15_i,0,shiftpix)
cal15_q=shift(cal15_q,0,shiftpix)
cal15_u=shift(cal15_u,0,shiftpix)
dark_i=shift(dark_i,0,shiftpix)
dark_q=shift(dark_q,0,shiftpix)
dark_u=shift(dark_u,0,shiftpix)
corona_i=shift(corona_i,0,shiftpix)
corona_q=shift(corona_q,0,shiftpix)
corona_u=shift(corona_u,0,shiftpix)

;;;;;;
;Kludge for d123
;;;;;;
;cal0_i=cal0_i-(total(cal0_i(0:9,*))/7200.)
;cal15_i=cal15_i-(total(cal15_i(0:9,*))/7200.)
;dark_i(*,*)=0.
;corona_i=corona_i-(total(corona_i(0:9,*))/7200.)
;;;;;;;;;;;;

;
; Perform bias correction
;

bias = total(dark_I,2)/float(numaz)

for ii = 0,numpix-1 do begin
	cal0_I(ii,*) = cal0_I(ii,*) - bias(ii)
	cal15_I(ii,*) = cal15_I(ii,*) - bias(ii)
	corona_I(ii,*) = corona_I(ii,*) - bias(ii)
endfor

;psym=3

; Gain is 2-D!
;
gain = cal0_I/Bopal

; Intensity to polarization 
; Use coronal Intensity to polarization if it shows
; more crosstalk than that from the opal.
;
;
X21 = total(cal0_Q,2)/total(cal0_I,2)
X31 = total(cal0_U,2)/total(cal0_I,2)

cx21img = corona_q/corona_i
cx31img = corona_u/corona_i
for ii=0,numpix-1 do begin
	mary = reform(cx21img(ii,*))
	cx21(ii) = min(leefilt(median(mary,9)))
	mary = reform(cx31img(ii,*))
	cx31(ii) = min(leefilt(median(mary,9)))
endfor

a = where ( cx21 lt x21 )
x21(a) = cx21(a)
x21(360:numpix-1) = cx21(360:numpix-1)
a = where ( cx31 lt x31 )
x31(a) = cx31(a)
x31(360:numpix-1) = cx31(360:numpix-1)

for ii=29,37 do begin
	x21(ii)=x21(ii)+cx21(ii)-cx21(38)
	x31(ii)=x31(ii)+cx31(ii)-cx31(38)
end

;
; Angles will run from -180 to 180
;

ang = (findgen(numaz) - numaz/2.)*2.*!pi/float(numaz)
sang = sin(2.*ang)
cang = cos(2.*ang)

qphi = fltarr(numpix)
uphi = fltarr(numpix)
phi = fltarr(numpix)
sprod = fltarr(numpix)
cprod = fltarr(numpix)
qamp = fltarr(numpix)
uamp = fltarr(numpix)
amp = fltarr(numpix)
rat = fltarr(numpix)
qmean = fltarr(numpix)
umean = fltarr(numpix)

; 
; Find amplitude and phase of cal 15
; at position angle of channel 4
;

fitq = fltarr(numpix,numaz)
fitu = fltarr(numpix,numaz)

;
; Compute X22 and X33
; Use fractional polarization normalizaton
;
; Remove fractional polarization due to opal alone
; from fractional polarization of cal 15
fred = cal15_q/cal15_i - cal0_q/cal0_i
mary = cal15_u/cal15_i - cal0_u/cal0_i

for ii = 0,NUMPIX-1 do begin
	sprod(ii) = total(sang*fred(ii,*))/numaz
	cprod(ii) = total(cang*fred(ii,*))/numaz
endfor

qphi = atan(sprod,cprod)
qamp = 2.*sqrt(sprod*sprod+cprod*cprod)
qmean = total(fred,2)/numaz

for ii = 0,NUMPIX-1 do begin
	sprod(ii) = total(sang*mary(ii,*))/numaz
	cprod(ii) = total(cang*mary(ii,*))/numaz
endfor
uphi = atan(sprod,cprod)
uamp = 2.*sqrt(sprod*sprod+cprod*cprod)
umean = total(mary,2)/numaz

fitu=uamp(200)*cos(2.*ang - uphi(200))
fitq=qamp(200)*cos(2.*ang - qphi(200))

amp = (qamp+uamp)/2.
phi = (qphi + (uphi + !pi/2.))/ 2.

plot,fred(200,*),psym=3
oplot,fitq
oplot,mary(200,*),psym=3
oplot,fitu
stop
;
; Find Today's corona scale factor, the average coronal brightness
; vs height at the time of the polarization calibration;

pbscale = total(corona_q(scalepix,*))/float(numaz)

;
; Signs here do not match the paper
; This is because when the barrel rotates, the orientation
; of the calibration polarizer rotates backwards.
;
;

!p.title = "X21"
!y.range=[-.3,.3]
plot,x21
oplot,cx21,color=128
stop

!p.title = "X31"
plot,x31
oplot,cx31,color=0
stop

x22 =  amp * cos(2.*phi)/fpt
!p.title = "X22"
!y.range=[-40,40]
plot,x22
stop

X23 = -amp * sin(2.*phi)/fpt
!p.title = "X23"
plot,x23
stop

x32 = amp * sin(2.*phi)/fpt
!p.title = "X32"
plot,x32
stop

x33 = amp * cos(2.*phi)/fpt
!p.title = "X33"
plot,x33
stop

close,1 & openw,1,'pbscale' & writeu,1,pbscale
print,'pbscale=',pbscale

close,1 & openw,1,'bias' & writeu,1,bias
!p.title='Bias'
!y.range=[-2000,2000]
plot,bias
stop
close,1 & openw,1,'gain' & writeu,1,gain
!y.range=[0,1.7e9]
!p.title='Gain'
plot,gain(*,200)
stop
close,1 & openw,1,'x21' & writeu,1,x21
close,1 & openw,1,'x31' & writeu,1,x31
close,1 & openw,1,'x22' & writeu,1,x22
close,1 & openw,1,'x23' & writeu,1,x23
close,1 & openw,1,'x32' & writeu,1,x32
close,1 & openw,1,'x33' & writeu,1,x33

print,'done'
stop

end
