/* dpmtofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Digital Prominence Monitor (DPM) image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   1 March 1994
 * 11 May 1994 : dispmin/dispmax changed from int to float.
 * 26 May 1994 : flip DPM image about x & y axes to put origin at lower left.
 *  1 Jun 1994 : replace DATAFORM with TYPE-OBS = ASCII data type.
 * 17 Aug 1994 : elminate angnorth.  Add DISC-OCC and GAIN-DET.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/stanger/lib/include/dpmlib.ext"

dpmtofits (namimg, namext, hduflag, pmode)

   char namimg [] ;		/* Name of primary file. */

   char namext [] ;		/* Name of extension file. */

   int hduflag ;		/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */

   int pmode ;			/* Print mode. */


{
   extern char *malloc () ;		/* Function to allocate memory. */
   extern unsigned char *rddpm () ;	/* Function to read DPM image. */
   char *padbuf ;			/* Pointer to padding buffer. */
   unsigned char *dpmbuf ;		/* MLSO DPM file  array pointer. */
   unsigned char *imgbuf ;		/* MLSO DPM image array pointer. */
   unsigned char *hkbuf ;		/* MLSO DPM housekeeping pointer.*/
   unsigned char *cibuf, *cobuf ;	/* unsigned char image pointers. */

   unsigned char cmin, cmax ;		/* Data min/max values. */
   unsigned char cval ;			/* Data value. */


   static char kairmass  [12] = "AIRMASS " ;
   static char kazimuth  [12] = "AZIMUTH " ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcode_obs [12] = "CODE-OBS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kocc_disc [12] = "OCC-DISC" ;
   static char kdisp_dir [12] = "DISP-DIR" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kdpmver   [12] = "DPMVER  " ;
   static char kelev     [12] = "ELEV    " ;
   static char kend      [12] = "END     " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kgain_det [12] = "GAIN-DET" ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char khour_ang [12] = "HOUR-ANG" ;
   static char kinstrume [12] = "INSTRUME" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char knorthang [12] = "NORTHANG" ;
   static char kobject   [12] = "OBJECT  " ;
   static char koptdev1  [12] = "OPTDEV1 " ;
   static char koptdev2  [12] = "OPTDEV2 " ;
   static char koptdev3  [12] = "OPTDEV3 " ;
   static char koptdev4  [12] = "OPTDEV4 " ;
   static char koptdev5  [12] = "OPTDEV5 " ;
   static char koptdev6  [12] = "OPTDEV6 " ;
   static char koptdev7  [12] = "OPTDEV7 " ;
   static char koptdev8  [12] = "OPTDEV8 " ;
   static char koptpos1  [12] = "OPTPOS1 " ;
   static char koptpos2  [12] = "OPTPOS2 " ;
   static char koptpos3  [12] = "OPTPOS3 " ;
   static char koptpos4  [12] = "OPTPOS4 " ;
   static char koptpos5  [12] = "OPTPOS5 " ;
   static char koptpos6  [12] = "OPTPOS6 " ;
   static char koptpos7  [12] = "OPTPOS7 " ;
   static char koptpos8  [12] = "OPTPOS8 " ;
   static char korigin   [12] = "ORIGIN  " ;
   static char knimavg   [12] = "NIMAVG  " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char krsun     [12] = "RSUN    " ;
   static char ksolar_b0 [12] = "SOLAR_B0" ;
   static char ksolar_l0 [12] = "SOLAR_L0" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ktelescop [12] = "TELESCOP" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char kversion  [12] = "VERSION " ;

   static char atype1   [24] ;			/* Coord type axis 1. */
   static char atype2   [24] ;			/* Coord type axis 2. */
   static char bunit    [32] ;			/* Brightness unit. */
   static char ctime    [12] = "HH.MM.SS" ;	/* File creation time. */
   static char ctype1   [24] ;			/* Coord type axis 1. */
   static char ctype2   [24] ;			/* Coord type axis 2. */
   static char cunit1   [24] ;			/* Coord units axis 1. */
   static char cunit2   [24] ;			/* Coord units axis 2. */
   static char datcom   [72] ;			/* Data min/max comment. */
   static char cdate    [12] = "DD/MM/YY" ;	/* File creation date. */
   static char date_cal [12] = "DD/MM/YY" ;	/* Calibration date. */
   static char date_obs [12] = "DD/MM/YY" ;	/* Observation date. */
   static char eq       [4]  = "= " ;
   static char fitbuf   [36][80] ;		/* FITS header buffer. */
   static char instrume [12] = "CCD" ;		/* Instrument name. */
   static char namfit   [80] ;			/* FITS file name. */
   static char object   [12] = "SUN" ;		/* Object. */
   static char origin   [12] = "HAO" ;		/* Origin. */
   static char outbuf   [84] ;
   static char pixcom   [48] ;			/* Data type comment. */
   static char refcom   [24] ;			/* Coord ref comment. */
   static char disp_dir [ 4] ;		/* Display direction: R/L, U/D, O/I. */
   static char telescop [12] = "DPM" ;		/* Telescope name. */
   static char time_obs [12] = "HH.MM.SS" ;	/* Observation time. */
   static char type_obs [24] ;			/* Observation type. */
   static char version  [24] ;
   static int ic [7] ;			/* Array containing date & time:
					 * year, doy, day, month,
					 * hour, minute, second.
					 */

   short *sibuf, *sobuf ;		/* Short image pointers. */
   short smin, smax ;			/* Data min/max values. */
   short sval = 0 ;			/* Data value. */

   int bitpix ;				/* # bits per pixel. */
   int doy_obs ;			/* Observation day-of-year. */
   int gcount = 0 ;			/* gcount. */
   int iyear ;				/* year - 1900. */
   int i, ix, iy ;			/* Indices. */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* Integer value. */
   int kx, ky ;				/* Image indices. */
   int ndim   = 0 ;			/* # dimensions for image (2=XY). */
   int nx, ny ;				/* # pixels for X & Y axes. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyfits = 2880 ;			/* # bytes per FITS logical record. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int pcount  = 0 ;			/* pcount. */
   int nrec ;				/* # fits records for image. */
   int nelwr   = 0 ;
   int year_obs ;			/* Observation year. */

   float *fibuf, *fobuf ;		/* Float image pointers. */

   float northdeg =  90.0 ;		/* Solar north w.r.t. +X axis. */
   float bscale   =   1.0 ;		/* Brightness intensity scale factor. */
   float bzero    =   0.0 ;		/* Brightness intensity zero offset. */
   float cdelt1, cdelt2 ;		/* Physical coordinate increments. */
   float crpix1, crpix2 ;		/* Reference pixel location. */
   float crota1, crota2 ;		/* Physical coordinate rotation angle.*/
   float crradius = 320.0 ;		/* Solar radius [pixels] = 961/3.006. */
   float crval1, crval2 ;		/* Physical coordinate values. */
   float dmin, dmax ;			/* Data min/max values. */
   float dpmver   =   1.01 ;		/* DPM FITS version number. */
   float expdur ;			/* Exposure duration (seconds). */
   float fmin, fmax ;			/* Data min/max values. */
   float fval ;				/* Data value. */
   float odisk ;			/* Occulting disk diameter (arcsec). */
   float vmin, vmax ;			/* Data min/max values. */
   float wmin =    0.0 ;		/* Display min value. */
   float wmax = 4000.0 ;		/* Display max value. */
   FILE *FF ;				/* FITS file ID. */

   sprintf (namfit, "%s.fts", namimg) ;

   if (pmode)
      {
      fprintf (stdout, "dpmtofits> namimg: %s  namext: %s   hduflag: %d\n",
	       namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- dpmtofits> %s -> %s -------\n\n",
	       namimg, namfit) ;
      }

   /*** Read MLSO DPM image/data from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2)
      dpmbuf = rddpm (namimg, pmode) ;	/* Read DPM dataset into memory. */
   else
      dpmbuf = rddpm (namext, pmode) ;	/* Read DPM dataset into memory. */

   nx = DPMI->xdim ;			/* X dimension. */
   ny = DPMI->ydim ;			/* Y dimension. */
   npximg = nx * ny ;			/* # pixels in image. */
   nbyimg = npximg * DPMI->nbypx ;	/* # bytes in image. */
   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			nx, ny, DPMI->nbypx, npximg, nbyimg) ;

   /*** Image origin is at upper right (effectively rotated 180 degrees). ***/
   /*** Flip image about both x & y axes to put image origin at lower left. ***/

   imgbuf = (unsigned char *) malloc (nbyimg) ;

   switch (DPMI->nbypx)
      {
      case 1:
	 cobuf = (unsigned char *) imgbuf ;
	 cibuf = (unsigned char *) dpmbuf ;
	 cmin  = 255 ;
	 cmax  =   0 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       kx = nx - ix - 1 ;
	       cval =
               *(cobuf + ix + iy*nx) = *(cibuf + kx + ky*nx) ;
               if (cval < cmin)  cmin = cval ;
               if (cval > cmax)  cmax = cval ;
	       }
	    }
         DPMI->dmin = (int) cmin ;
         DPMI->dmax = (int) cmax ;
	 vmin = (float) cmin ;
	 vmax = (float) cmax ;
         break ;

      case 2:
	 sobuf = (short *) imgbuf ;
	 sibuf = (short *) dpmbuf ;
	 smin  =  32767 ;
	 smax  = -32767 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       kx = nx - ix - 1 ;
	       sval =
               *(sobuf + ix + iy*nx) = *(sibuf + kx + ky*nx) ;
               if (sval < smin)  smin = sval ;
               if (sval > smax)  smax = sval ;
	       }
	    }
         DPMI->dmin = (int) smin ;
         DPMI->dmax = (int) smax ;
	 vmin = (float) smin ;
	 vmax = (float) smax ;
         break ;

      case 4:
	 fobuf = (float *) imgbuf ;
	 fibuf = (float *) dpmbuf ;
	 fmin  =  1.0e+30 ;
	 fmax  = -1.0e+30 ;
	 for (iy = 0 ; iy < ny ; iy++)
	    {
	    ky = ny - iy - 1 ;
	    for (ix = 0 ; ix < nx ; ix++)
	       {
	       kx = nx - ix - 1 ;
               *(fobuf + ix + iy*nx) = *(fibuf + kx + ky*nx) ;
               if (fval < fmin)  fmin = fval ;
               if (fval > fmax)  fmax = fval ;
	       }
	    }
         DPMI->dmin = (int) fmin ;
         DPMI->dmax = (int) fmax ;
	 vmin = fmin ;
	 vmax = fmax ;
         break ;

      default:
	 fprintf (stderr, "Invalid DPM image . . . nbypx: %d\n", DPMI->nbypx) ;
	 exit (-1) ;
      }

   iyear = DPMI->year ;
   if (DPMI->year > 1900)  iyear = DPMI->year - 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, DPMI->month, DPMI->day) ;

   sprintf (date_obs, "%02d/%02d/%02d", DPMI->day, DPMI->month, iyear) ;
   sprintf (time_obs, "%02d.%02d.%02d", DPMI->hour, DPMI->minute, DPMI->second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear	= ic [0] ;
   icdoy	= ic [1] ;
   icday	= ic [2] ;
   icmonth	= ic [3] ;
   ichour	= ic [4] ;
   icmin	= ic [5] ;
   icsec	= ic [6] ;
   sprintf (cdate, "%02d/%02d/%02d", icday, icmonth, icyear) ;
   sprintf (ctime, "%02d.%02d.%02d", ichour, icmin, icsec) ;

   iyear = DPMI->cal_year ;
   if (DPMI->cal_year > 1900)  iyear = DPMI->cal_year - 1900 ;
   sprintf (date_cal, "%02d/%02d/%02d", DPMI->cal_day, DPMI->cal_month, iyear) ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (DPMI->xdim > 0)
      ndim++ ;
   if (DPMI->ydim > 1)
      ndim++ ;

   /* DPM data types:
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   for (i = 0 ; i < 47 ; i++)
      pixcom [i] = ' ' ;
   strcpy (origin,   "HAO") ;
   strcpy (telescop, "DPM") ;
   strcpy (instrume, "CCD") ;
   strcpy (datcom,   "CCD") ;
   strcpy (atype1,   "X") ;
   strcpy (atype2,   "Y") ;
   strcpy (cunit1,   "ARCSEC") ;
   strcpy (cunit2,   "ARCSEC") ;
   strcpy (refcom,   "SUN CENTER") ;
   strcpy (disp_dir,  "RU") ;		/* Display direction: right, up. */

   crpix1 = (float) ((nx - 1) / 2) + 0.5 ;	/* Sun center (X pixels). */
   crpix2 = (float) ((ny - 1) / 2) + 0.5 ;	/* Sun center (Y pixels). */
   crval1 = 0.0 ;				/* Sun center (X arcsec). */
   crval2 = 0.0 ;				/* Sun center (Y arcsec). */
   cdelt1 = 3.006 ;				/* arcsec/pixel. */
   cdelt2 = 3.006 ;				/* arcsec/pixel. */
   crradius = DPMI->rsun / cdelt1 ;		/* Solar radius (pixels). */
   expdur = (float) DPMI->expdur / 100.0 ;
   odisk  = (float) DPMI->odisk ;

   /*** Set object.
    *** Set coordinate type(s).
    *** Set bitpix.
    *** Set display scaling.
    ***/

   switch (DPMI->dtype)
      {
      case  0: /* other */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "OTHER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  1: /* disc */
               strcpy (object,   "SUN") ;   
               strcpy (type_obs, "DISC") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 1500.0 ;
               break ;
      case  2: /* limb */
               strcpy (object,   "SUN") ;   
               strcpy (type_obs, "LIMB") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  3: /* ref */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "REF") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  4: /* diffuser */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DIFFUSER") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
               break ;
      case  5: /* disc_dark */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "DISC_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case  6: /* limb_dark */
               strcpy (object,   "SUN") ;   
               strcpy (type_obs, "LIMB_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "Y") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case  7: /* diffuser_dark */
               strcpy (object,   "SUN") ;   
	       strcpy (type_obs, "DIFFUSER_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      case  8: /* ref_dark */
               strcpy (object,   "NONE") ;   
	       strcpy (type_obs, "REF_DARK") ;
               strcpy (pixcom,   "2-BYTE SIGNED PIXELS") ;
               strcpy (ctype1,   "X") ;
               strcpy (ctype2,   "Y") ;
	       strcpy (bunit,    "CCD") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax =  500.0 ;
               break ;
      default:
               strcpy (object,   "UNKNOWN") ;   
	       strcpy (type_obs, "UNKNOWN") ;
               bitpix =  16 ;			/* short integer image. */
	       wmin =   0.0 ;   wmax = 4000.0 ;
	       fprintf (stderr, "Invalid dtype: %d\n", DPMI->dtype) ;
	       exit (-1) ;
      }

   /*** Create FITS file. ***/

   if (hduflag == 3)				/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- PRIMARY HDU ---\n") ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
   sprintf (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE: MLSO DIGITAL PROMINENCE MONITOR") ;
   else
   sprintf (&fitbuf [iy++][0],  "XTENSION= 'IMAGE   '           / MLSO DPM %s EXTENSION", type_obs) ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, ny, atype2) ;
   else
      sprintf (&fitbuf  [iy++][0],  "                                                                                ") ;

   /*** Extension information. ***/

   if (hduflag == 1)
      {
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      }
   else
   if (hduflag == 2)
      {
      sprintf (&fitbuf [iy++][0],  "EXTEND  =                    T / EXTENSION(S) POSSIBLE") ;
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      }
   else
   if (hduflag == 3)
      {
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;
      }

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ Mauna Loa Solar Observatory (MLSO)") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ High Altitude Observatory (HAO), DIV. OF NCAR") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ MLSO Digital Prominence Monitor (DPM)") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kinstrume, eq, instrume) ;
   sprintf (&fitbuf [iy++][31], "/ Kodak Megaplus 1.6 CCD") ;


   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
				doy_obs, year_obs) ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH.MM.SS [UT]") ;

   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s [EAST  -  WEST] HELIOCENTRIC", atype1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [INDEX] (DEFAULT GUESS)",
			      kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1) ;

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s [NORTH - SOUTH] HELIOCENTRIC", atype2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [INDEX] (DEFAULT GUESS)",
			           kcrpix2, eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2  , eq, cdelt2, atype2, cunit2) ;
      }

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdisp_dir,  eq, disp_dir) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION (AXIS 1 2 3): R/L  U/D  O/I") ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s%20.2f", kcrradius, eq, crradius) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR RADIUS [PIXELS] (DEFAULT GUESS)") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR RADIUS [ARCSEC]",
			      krsun    , eq, DPMI->rsun) ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / OCCULTING DISC DIAMETER [ARCSECONDS]",
			      kocc_disc, eq, odisk) ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
			      ksolar_p0, eq, DPMI->pangle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
			      ksolar_b0, eq, DPMI->bangle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE",
			      ksolar_l0, eq, DPMI->langle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR HOUR ANGLE [DEGREES] (W.R.T.ZENITH E- W+)",
			      khour_ang, eq, DPMI->hangle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR ELEVATION [DEGREES]",
			      kelev    , eq, DPMI->elev) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR AZIMUTH [DEGREES]",
			      kazimuth , eq, DPMI->azimuth) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / AIRMASS [ATMOSPHERES]",
			      kairmass , eq, DPMI->airmass) ;


   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d",
			      kcode_obs, eq, DPMI->dtype) ;
   sprintf (&fitbuf [iy++][31], "/ OBS 1:DISC  2:LIMB  3:REF  4:DIFU  5-8:1-4 DARK") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20d / NUMBER OF IMAGES AVERAGED",
			      knimavg  , eq, DPMI->navg) ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / EXPOSURE DURATION [SECONDS]",
			      kexpdur  , eq, expdur) ;

/* -------------------------------------------------------------------------- */

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < 36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write first record of PRIMARY FITS header. ***/

   nelwr = fwrite (fitbuf, sizeof (char), 2880, FF) ;

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   /*** Fill second record of FITS header. ***/

   iy = 0 ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'",     kbunit, eq, bunit) ;
   sprintf (&fitbuf [iy++][31], "/ MAGNITUDE UNITS") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / PHYSICAL = DATA * BSCALE + BZERO",
			       kbscale, eq, DPMI->bscale) ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s%20.2f", kbzero , eq, DPMI->bzero ) ;
   sprintf (&fitbuf [iy++][31], "/ PHOTOMETRIC SCALING OFFSET [SEE BSCALE]") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / MINIMUM DATA VALUE [%s]",
			       kdatamin , eq, vmin, datcom) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / MAXIMUM DATA VALUE [%s]",
			       kdatamax , eq, vmax, datcom) ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / MIN VALUE FOR DISPLAY [UNSCALED]",
			      kdispmin , eq, wmin) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / MAX VALUE FOR DISPLAY [UNSCALED]",
			      kdispmax , eq, wmax) ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev1 ,  eq, &DPMI->optdev [0][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 1 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos1 , eq, &DPMI->optpos [0][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 1") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev2 ,  eq, &DPMI->optdev [1][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 2") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos2 , eq, &DPMI->optpos [1][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 2") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev3 ,  eq, &DPMI->optdev [2][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 3") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos3 , eq, &DPMI->optpos [2][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 3") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev4 ,  eq, &DPMI->optdev [3][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 4") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos4 , eq, &DPMI->optpos [3][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 4 [NANO METERS]") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev5 ,  eq, &DPMI->optdev [4][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 5") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos5 , eq, &DPMI->optpos [4][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 5") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev6 ,  eq, &DPMI->optdev [5][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 6 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos6 , eq, &DPMI->optpos [5][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 6") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev7 ,  eq, &DPMI->optdev [6][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 7 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos7 , eq, &DPMI->optpos [6][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 7") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptdev8 ,  eq, &DPMI->optdev [7][0]) ;
   sprintf (&fitbuf [iy++][31],"/ OPTICAL DEVICE 8 [NOT USED]") ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", koptpos8 , eq, &DPMI->optpos [7][0]) ;
   sprintf (&fitbuf [iy++][31],"/ POSITION: OPTICAL DEVICE 8") ;

   sprintf (version, "DPM %4.2f", dpmver) ;
   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kversion, eq, version) ;
   sprintf (&fitbuf [iy++][31],"/ DPM FITS VERSION") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdate,     eq, cdate) ;
   sprintf (&fitbuf [iy++][31], "/ FITS FILE GENERATION DATE: DD/MM/YY") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'",     kdate_cal, eq, date_cal) ;
   sprintf (&fitbuf [iy++][31],"/ DPM CALIBRATION DATE: DD/MM/YY [UT]") ;

      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;

   sprintf (&fitbuf [iy++][0],  "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   THE DIGITAL PROMINENCE MONITOR (DPM) OBSERVES THE SOLAR DISK & LIMB") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   WITH AN APPROXIMATE 1.6 x 2.2 SOLAR RADIUS FIELD OF VIEW, USING A") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   KODAK CCD DETECTOR (1534x1032 PIXELS, OF WHICH 1534x1029 ARE USED).") ;
/*
 *   sprintf (&fitbuf [iy++][0],  "COMMENT   BRIGHTNESS MEASUREMENTS ARE SPECIFIED IN UNITS OF BSUN-HALPHA,") ;
 *   sprintf (&fitbuf [iy++][0],  "COMMENT   WHICH IS A RATIO OF OBSERVED BRIGHTNESS TO BRIGHTNESS OF SOLAR DISK.") ;
 */
   sprintf (&fitbuf [iy++][0],  "COMMENT   IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.") ;
   sprintf (&fitbuf [iy++][0], "END") ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < 36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write second record of PRIMARY FITS header. ***/

   nelwr = fwrite (fitbuf, sizeof (char), 2880, FF) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (imgbuf, DPMI->nbypx, npximg, FF) ;
   free (dpmbuf) ;
   free (imgbuf) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FF) ;
      free (padbuf) ;
      }
   if (pmode)
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg) ;

   close (FF) ;
   return (0) ;
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

month_day (year, doy, pmonth, pday)	/* Set month & day from day-of-year. */

int year, doy, *pmonth, *pday ;

{
   static int day_tab [2][13] =
      {
      {0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      {0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
      } ;
   int i, leap ;

   leap = (year % 4 == 0 && year % 100 != 0 || year % 400 == 0) ;

   for (i = 1 ; doy > day_tab [leap][i] ; i++)
      doy -= day_tab [leap][i] ;

   *pmonth = i ;
   *pday = doy ;

   return (0) ;
}
