;+
; NAME	doppler.pro
;
; PURPOSE	Determine the doppler shift profile for CHIP images.
;
; SYNTAX	doppler, fits_name
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   25 May 2000
;-

PRO doppler1, fts_name, brtpct=brtpct, getloc=getloc, saveloc=saveloc

;fts_name = 'csr.fts'
ld_name = 'chip.ldcor.img'		; Limb darkening correction image.
imgld = FLTARR (936, 936)

red = BINDGEN (256)
green = red
blue  = red
cme_name = ''
LCT,'/home/stanger/color/bwy.lut'	; color table.
TVLCT, red, green, blue, /get

   xdimsq = 936
   ydimsq = 936
   WINDOW, xsize=xdimsq, ysize=ydimsq

   ;--- Read limb darkening correction image.

   CLOSE, 1
   OPENR, 1, ld_name
   READU, 1, imgld			; 936 x 936 floating point pixels.
   CLOSE, 1
   PRINT, 'size (imgld): ', SIZE (imgld)

;   PRINT, 'imgld [*, ydimsq/2]: '
;   PRINT, imgld [*, ydimsq/2]

   TVSCL, imgld
;   WAIT, 1

;--- File list loop.

;CLOSE, 1
;OPENR, 1, fts_list
;WHILE (NOT EOF (1)) DO	$
;BEGIN	;{

   ;--- Read CHIP (B2 or "csr" filter  1082.847 nm) FITS image.

   imgb2 = readfits (fts_name, hdu)
   imgb2 [*, 0] = 0			; Set first 3 lines to zero.
   imgb2 [*, 1] = 0
   imgb2 [*, 2] = 0
   imgb2 = float (imgb2)

   ;--- Extract information from FITS header.

   fts_min = MIN (imgb2, max=fts_max)
   PRINT, FORMAT = '("fts_min/max: ", E10.3, E10.3)', fts_min, fts_max
   xdim   = fxpar (hdu, 'NAXIS1')
   ydim   = fxpar (hdu, 'NAXIS2')
   xcen   = fxpar (hdu, 'CRPIX1')
   ycen   = fxpar (hdu, 'CRPIX2')
   pangle = fxpar (hdu, 'SOLAR_P0')
   pixrs  = fxpar (hdu, 'CRRADIUS') 
   xcen   = (xdim / 2 - 0.5)
   ycen   = (ydim / 2 - 0.5)
   code_obs = fxpar (hdu, 'CODE-OBS')
   code_obs = strtrim (strlowcase(code_obs))

   PRINT, 'xdim/ydim: ', xdim, ydim
   PRINT, 'xcen/ycen: ', xcen, ycen
   PRINT, 'pangle: ', pangle
   PRINT, 'pixrs: ', pixrs

   ;--- Crop image to a square (936 x 936).

   xcensq = (xdimsq / 2 - 0.5)
   ycensq = (ydimsq / 2 - 0.5)
   ixcensq = xdimsq / 2
   iycensq = ydimsq / 2
   imgsq = FLTARR (xdimsq, ydimsq)

   ixbeg = fix (xcen - xdimsq / 2)
   iybeg = fix (ycen - ydimsq / 2)
   ixend = ixbeg + xdimsq - 1
   iyend = iybeg + ydimsq - 1
   ixdif = ixend - ixbeg + 1
   iydif = iyend - iybeg + 1

   PRINT, 'ixbeg/ixend/ixdif: ', ixbeg, ixend, ixdif
   PRINT, 'iybeg/iyend/iydif: ', iybeg, iyend, iydif

   imgsq = imgb2 [ixbeg:ixend, iybeg:iyend]
   PRINT, 'size (imgsq): ', SIZE (imgsq)

   TVSCL, imgsq
;   WAIT, 1

   ;--- Correct for limb darkening.

   imgldc = imgsq * imgld
   PRINT, 'size (imgldc): ', SIZE (imgldc)

   TVSCL, imgldc

   ;--- Rotate image to Heliocentric coordinates.

   imgrot = rot (imgldc, pangle)

   TVSCL, imgrot
;   WAIT, 1

   ;--- Find average intensity at sun center.

   ;--- First, extract a box at sun center.

   xdimcen = 40
   ydimcen = 40
   npxcen = xdimcen * ydimcen
   ixbeg = ixcensq - xdimcen
   iybeg = iycensq - ydimcen
   ixend = ixbeg + xdimcen - 1
   iyend = iybeg + ydimcen - 1

   imgcen = FLTARR (xdimcen, ydimcen)
   imgcen = imgrot [ixbeg:ixend, iybeg:iyend]

   PRINT, 'size (imgrot): ', SIZE (imgrot)

   ;--- Next, order the array according to increasing intensity.

   censort = imgcen (SORT (imgcen))	; order w.r.t. increasing brightness.

   ;--- Compute the "10% median": level at which 10% of the pixels lie above
   ;    the computed brightness.
   ;--- Then compute the average of these brightest pixels.

   nbrt90 = long (0.9 * npxcen)
   brtcen  = TOTAL (censort [nbrt90:npxcen-1]) / (npxcen-nbrt90)

   PRINT, 'brtcen: ', brtcen

   ;--- Extract a "band" (very wide box) across the equator of the disk.

   ixbeg = ixcensq - fix (pixrs * 0.9)
   ixend = ixcensq + fix (pixrs * 0.9)
   iybeg = iycensq - fix (pixrs * 0.1)
   iyend = iycensq + fix (pixrs * 0.1)
   ixdif = ixend - ixbeg + 1
   iydif = iyend - iybeg + 1

   xdimband = long (ixend - ixbeg + 1)
   ydimband = long (iyend - iybeg + 1)
   npxband  = xdimband * ydimband

   PRINT, 'ixbeg/ixend/ixdif: ', ixbeg, ixend, ixdif
   PRINT, 'iybeg/iyend/iydif: ', iybeg, iyend, iydif
   PRINT, 'xdimband/ydimband: ', xdimband, ydimband
   PRINT, 'npxband: ', npxband

   band = FLTARR (xdimband, ydimband)

   band = imgrot [ixbeg:ixend, iybeg:iyend]

   bandsort = band (SORT (band))	; order w.r.t. increasing brightness.

   ;--- Compute the "brtpct % median":
   ;    the level at which 20% of the pixels lie above the computed brightness.

   nloc = 0L
   IF (KEYWORD_SET (getloc)) THEN	$	; Read bright pixel locations.
      BEGIN
      CLOSE, 2
      OPENR, 2, getloc
      brtpct  = 100.0
      brtcenf =   0.0
      READF, 2, brtpct
      READF, 2, brtcenf
      frac = 1.0 - (brtpct / 100.0)
      sfrac = strtrim (string (fix ((1.0 - frac) * 100.00001),		$
		       FORMAT='(I4)'), 2)
      PRINT, 'frac: ', frac
      READF, 2, brtavg
      READF, 2, nloc
      brtloc = LONARR (nloc)
      FOR i = 0L, nloc - 1 DO	$
	 BEGIN
	 READF, 2, pixloc
	 brtloc [i] = pixloc
	 END
      CLOSE, 2
      END 				$
   ELSE					$
      BEGIN					; Get bright pixels in band.
      IF (KEYWORD_SET (brtpct)) THEN	$
         frac = 1.0 - (brtpct / 100.0)	$
      ELSE				$
         frac = 0.00
      sfrac = strtrim (string (fix ((1.0 - frac) * 100.00001),		$
		       FORMAT='(I4)'), 2)

      nbrtval = long (frac * npxband)	; Fraction of brightest pixels to sample
      ibrtval = npxband - nbrtval
      brtval  = bandsort (ibrtval)	; Brightness level threshold.
      brtloc  = WHERE (band GE brtval)	; Indices corresponding to bright pixels
      brtlocsize = SIZE (brtloc)
      IF (brtlocsize [0] GT 0) THEN		$
         nloc = long (brtlocsize [1])		$
      ELSE					$
         nloc = 0L

      PRINT, 'nbrtval: ', nbrtval
      PRINT, 'ibrtval: ', ibrtval
      PRINT, 'brtval:  ', brtval

      END

   brtmin = MIN   (band (brtloc))	; Minimum brightness of selected pixels.
   brtavg = TOTAL (band (brtloc)) / nloc
   brtval = brtavg
   brtcont = brtcen / 0.97		; Continuum brightness.

   PRINT, 'brtmin  : ', brtmin
   PRINT, 'brtavg  : ', brtavg
   PRINT, 'brtval  : ', brtval
   PRINT, 'brtcen  : ', brtcen 
   PRINT, 'brtcont : ', brtcont
   PRINT, 'nloc    : ', nloc

   if (KEYWORD_SET (saveloc)) THEN	$
      BEGIN
      saveloc_name = code_obs + '.brt' + sfrac + '.loc'
      PRINT, 'saveloc_name: ', saveloc_name
      CLOSE, 2
      OPENW, 2, saveloc_name			; 'doppler.brtpix.loc'
      PRINTF, 2, brtpct, ' brtpct'
      PRINTF, 2, brtcen, ' brtcen'
      PRINTF, 2, brtavg, ' brtavg'
      PRINTF, 2, nloc
      FOR i = 0L, nloc - 1 DO		$
         PRINTF, 2, brtloc [i]
      CLOSE, 2
      END

   ;--- For each box (with size xsize * xsize) within band, compute average
   ;    brightness of the "brtpct" % brightest pixels.

   xinc  = 0.05
   xsize = 0.1
   xmax  = 0.93
   nbox = (xmax * 2.0) / xinc
   eqdist = fltarr (nbox)
   boxavg = fltarr (nbox)
   xdimbox = fix (xsize * pixrs)
   ydimbox = fix (xsize * pixrs)
   box = fltarr (xdimbox, ydimbox)

   FOR i = 0, nbox - 1 DO		$
      BEGIN
      x = i * xinc - xmax
      eqdist [i] = x
      ix = ixcensq + fix (x * pixrs)
      iy = iycensq
      ixbeg = ix - xdimbox / 2
      iybeg = iy - ydimbox / 2
      ixend = ixbeg + xdimbox - 1
      iyend = iybeg + ydimbox - 1
      ixdif = ixend - ixbeg + 1
      iydif = iyend - iybeg + 1

      box = imgrot [ixbeg:ixend, iybeg:iybeg]
      boxloc = WHERE (box GE brtval)
      boxlocsize = SIZE (boxloc)

;      PRINT, 'box: ', i
;      PRINT, 'size (boxloc): ', SIZE (boxloc)

      IF (boxlocsize [0] EQ 0 AND boxloc [0]  EQ -1)  THEN		$
	 BEGIN
	 boxbrt = 0.0
	 boxavg [i] = 0.0
	 END				$
      ELSE				$
	 BEGIN
         boxbrt = box (boxloc) 
	 sizeboxbrt = SIZE (boxbrt)
         nval = sizeboxbrt [1]
	 boxavg [i] = TOTAL (boxbrt) / nval

;         PRINT, boxloc
;         PRINT, boxbrt
	 END

;      PRINT, 'i: ', i, ' x: ', x, ' boxavg: ', boxavg [i]
;      PRINT, ' ixbeg/ixend/ixdif: ', ixbeg, ixend, ixdif
;      PRINT, ' iybeg/iyend/iydif: ', iybeg, iyend, iydif

      END


pboxloc = WHERE (boxavg GT 0.0)
peqdist = eqdist (pboxloc)
pboxavg = boxavg (pboxloc)
sizepboxavg = SIZE (pboxavg)
IF (sizepboxavg [0] GT 0) THEN	$
   npbox = sizepboxavg [1]	$
ELSE				$
   npbox = 0

plot_name = code_obs + '.doppler' + sfrac + '.ps'
SET_PLOT, 'ps'
DEVICE, filename=plot_name

title1 = 'Equatorial Doppler Profile: ' + fts_name
ylabel = 'Intensity (Ave. ' + sfrac + ' % Brightest Pixels'
y1 = 0.96 * pboxavg [npbox/2]
y2 = 1.04 * pboxavg [npbox/2]

PLOT, peqdist, pboxavg, psym=2,			$
      title  = title1,				$
      xtitle = 'Equatorial Distance (Rsun)',	$
      ytitle = ylabel,				$
      yrange = [y1, y2],			$
      xrange = [-xmax, xmax]

;STOP

pboxavg = pboxavg / pboxavg [npbox/2] 
title2  = 'Normalized Equatorial Doppler Profile: ' + fts_name
ylabel  = 'Norm. Intensity (Ave. ' + sfrac + ' % Brightest Pixels'

;PLOT, peqdist, pboxavg, psym= 4,		$
;      title  = title2,				$
;      xtitle = 'Equatorial Distance (Rsun)',	$
;      ytitle = ylabel,				$
;      yrange = [0.96, 1.04],			$
;      xrange = [-xmax, xmax]

DEVICE, /close
SET_PLOT, 'x'

;END	;}

;CLOSE, 1
END
