;+
; NAME
;	dpmsqlo
;
; PURPOSE
;	convert ACOS/DPM images to 500x500 pixels.
;
; SYNTAX
;	dpmsqlo, 'dpmimages.ls'
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   15 Dec 1998
;-

PRO dpmsqlo, filelist

   nx = 1000
   ny = 1000

   file = ''

   GET_LUN, lun
   OPENR, lun, filelist
   WHILE (NOT EOF (lun)) DO			$
   BEGIN
      READF, lun, file, FORMAT = '(A)'
      PRINT, 'Read  file: ', file
      img = readfits (file, hdu)
      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')
      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')
      cdelt1 = fxpar (hdu, 'CDELT1')
      cdelt2 = fxpar (hdu, 'CDELT2')
      crradius = fxpar (hdu, 'CRRADIUS')
      datamin = fxpar (hdu, 'DATAMIN')
      datamax = fxpar (hdu, 'DATAMAX')
      dispmin = fxpar (hdu, 'DISPMIN')
      dispmax = fxpar (hdu, 'DISPMAX')
      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax

      xbeg = (xcen - nx / 2)
      ybeg = (ycen - ny / 2)
      imgsq = intarr (nx, ny)
      imgsq = img (xbeg : xbeg +  nx - 1, ybeg : ybeg +  ny - 1)
      minsq = min (imgsq)
      maxsq = max (imgsq)
      imglo = rebin (imgsq, nx / 2, ny / 2)
      xcen = (nx / 4) - 0.5
      ycen = (ny / 4) - 0.5
      minlo = min (imglo)
      maxlo = max (imglo)
      PRINT, 'minsq/maxsq: ', minsq, maxsq
      PRINT, 'minlo/maxlo: ', minlo, maxlo

      fxaddpar, hdu, 'CRPIX1', xcen
      fxaddpar, hdu, 'CRPIX2', ycen
      fxaddpar, hdu, 'CDELT1', cdelt1 * 2.0
      fxaddpar, hdu, 'CDELT2', cdelt2 * 2.0
      fxaddpar, hdu, 'CRRADIUS', crradius / 2.0

      extpos = strpos (file, '.fts')
      outfile = strmid (file, 0, extpos) + '.sqlo.fts'
      PRINT, 'Write file: ', outfile
      writefits, outfile, imglo, hdu
   END
   CLOSE, LUN
   
END
