;+
; NAME
;	dpmsqr
;
; PURPOSE
;	convert ACOS/DPM images to 800x800 pixels.
;	rotate image according to P-angle.
;
; SYNTAX
;	dpmsqr, 'dpmimages.ls'
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   16 Dec 1998
;-

PRO dpmsqr, filelist

   nx = 800
   ny = 800

   file = ''

   GET_LUN, lun
   OPENR, lun, filelist

   WHILE (NOT EOF (lun)) DO			$
   BEGIN
      READF, lun, file, FORMAT = '(A)'		; Read file name from list.
      PRINT, 'Read  file: ', file
      img = readfits (file, hdu)		; Read FITS image.

      ;--- Extract information from FITS header.

      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      xcen     = fxpar (hdu, 'CRPIX1')
      ycen     = fxpar (hdu, 'CRPIX2')
      cdelt1   = fxpar (hdu, 'CDELT1')
      cdelt2   = fxpar (hdu, 'CDELT2')
      crradius = fxpar (hdu, 'CRRADIUS')
      pang     = fxpar (hdu, 'SOLAR_P0')
      datamin  = fxpar (hdu, 'DATAMIN')
      datamax  = fxpar (hdu, 'DATAMAX')
      dispmin  = fxpar (hdu, 'DISPMIN')
      dispmax  = fxpar (hdu, 'DISPMAX')

;      fxaddpar, hdu, 'DISPMIN', 0.0
;      fxaddpar, hdu, 'DISPMAX', 0.0
;      fxaddpar, hdu, 'DISPMIN', datamin
;      fxaddpar, hdu, 'DISPMAX', datamax

      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax

      IF (xcen LT 0 OR xcen GE xdim)  THEN GOTO, next

      xbeg = (xcen - nx / 2)
      ybeg = (ycen - ny / 2)
      xcen = (nx / 2) - 0.5
      ycen = (ny / 2) - 0.5

      ;--- Extract nx*ny pixel sub-image.

      imgsq = intarr (nx, ny)
      imgsq = img (xbeg : xbeg +  nx - 1, ybeg : ybeg +  ny - 1)
      minsq = min (imgsq)
      maxsq = max (imgsq)
      PRINT, 'minsq/maxsq: ', minsq, maxsq

      ;--- Rotate image using P-angle (geocentric -> heliocentric coord.)

      nxcen = FIX (xcen + 0.5)
      nycen = FIX (ycen + 0.5)
      mag = 1.0
      roll = pang
      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, INTERP=Cubic)
      imgrot = FIX (imgrot)		; Convert image back to short integers.
      minrot = min (imgrot)
      maxrot = max (imgrot)
      PRINT, 'minrot/maxrot: ', minrot, maxrot

      ;--- Update FITS header.

      fxaddpar, hdu, 'CRPIX1', xcen
      fxaddpar, hdu, 'CRPIX2', ycen
      fxaddpar, hdu, 'CDELT1', cdelt1 * 2.0
      fxaddpar, hdu, 'CDELT2', cdelt2 * 2.0
      fxaddpar, hdu, 'CRRADIUS', crradius / 2.0
;      fxaddpar, hdu, 'DATAMIN', minrot
;      fxaddpar, hdu, 'DATAMAX', maxrot
      fxaddpar, hdu, 'DISPMIN', datamin
      fxaddpar, hdu, 'DISPMAX', datamax

      extpos = strpos (file, '.fts')
      outfile = strmid (file, 0, extpos) + '.sqr.fts'
      PRINT, 'Write file: ', outfile
      writefits, outfile, imgrot, hdu
next:
   END
   CLOSE, LUN
   
END
