;+
; fixchipcrota.pro   IDL procedure
;
; PURPOSE
;	Read a CHIP FITS file and fix the CROTA1/CROTA2 parameters.
;	For HSH data prior to 7 Nov 2000, these parameters were set
;	to -SOLAR_P0, which is incorrect, since the calibrated CHIP
;	data has been corrected for P-angle.
;
; SYNTAX
;	fixchipcrota, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   HAO/NCAR   19 December 2002
;-

pro fixchipcrota, fits_list

fits_name = ''				; Define FITS file name variable.
wave_code = 'HSH     '
wave_name = 'HELIUM-I'
comment1 = ' X [EAST->WEST ] ROTATION [DEG] FROM REFERENCE'
comment2 = ' Y [SOUTH-NORTH] ROTATION [DEG] FROM REFERENCE'
fzero = 0.0

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.

      print, fits_name

      ;--- Change CROTA1/CROAT2 parameters.

      fxhmodify, fits_name, 'CROTA1',   fzero, comment1
      fxhmodify, fits_name, 'CROTA2',   fzero, comment2
   END
CLOSE, 1

 END
