;+
; NAME	fts2gif2.pro
;
; PURPOSE	Convert FITS images to GIF format.
;
; SYNTAX	fts2gif, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   19 April 2000
;		26 Oct 2004: Add wmin & wmax keywords.
;-

pro fts2gif2, fits_list, color_lut, wmin=wmin, wmax=wmax

CLOSE, 1
fits_name = ''
xdim_prev = 0
ydim_prev = 0

red = bindgen (256)
green = red
blue  = red
if (N_ELEMENTS (color_lut) GT 0) THEN	$
    lct, color_lut			$
ELSE					$
    lct,'/home/stanger/color/bwy.lut'	; color table.

tvlct, red, green, blue, /get

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
print, 'fits_name: ', fits_name

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

;   fits_min = MIN (fits_img, max=fits_max)

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   fits_min = fxpar (hdu, 'DISPMIN')
   fits_max = fxpar (hdu, 'DISPMAX')

   dmin = fits_min + (fits_max - fits_min) * 0.25 
   dmax = fits_max * 0.9

   IF (KEYWORD_SET (wmin)) THEN dmin = wmin
   IF (KEYWORD_SET (wmax)) THEN dmax = wmax

;   fits_min = -1000
;   fits_max = 5000
;   print, FORMAT = '("fits_min/max: ", E10.3, E10.3)', fits_min, fits_max
;   print, 'fits_min/fits_max: ', fits_min, fits_max
   print, 'dmin/dmax: ', dmin, dmax

   ;--- Convert to byte pixels.
   ;--- Display image.

   IF (xdim NE xdim_prev OR ydim NE ydim_prev) THEN	$
      window, xsize=xdim, ysize=ydim
   byt_img = bytscl (fits_img, min=dmin, max=dmax, top=249)
   tv, byt_img						; Display image.

   ;--- Write new image to disk.

   sloc = RSTRPOS (fits_name, '.fts')
   new_name = strmid (fits_name, 0, sloc) + '.gif'
   print, 'Write GIF image : ', new_name
   write_gif, new_name, byt_img, red, green, blue	; Write new image.
   xdim_prev = xdim
   ydim_prev = ydim
END	;}

CLOSE, 1
END
