;+
; NAME	fts2jpg.pro
;
; PURPOSE	Convert FITS images to JPEG format.
;
; SYNTAX	fts2jpg, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   30 January 2001
;		28 Feb 2002: add wmin & wmax.
;		30 Apr 2003: add label keyword.
;-

pro fts2jpg, fits_list, color_lut, wmin=wmin, wmax=wmax, label=label

month_name = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', $
	      'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
CLOSE, 1
fits_name = ''
xdim_prev = 0
ydim_prev = 0

red = bindgen (256)
green = red
blue  = red

SET_PLOT, 'Z'
;DEVICE, set_resolution = [512, 512], set_colors = 256, z_buffering = 0
DEVICE, set_resolution = [960, 960], set_colors = 256, z_buffering = 0

if (N_ELEMENTS (color_lut) GT 0) THEN	$
    lct, color_lut			$
ELSE					$
    lct,'/home/stanger/color/quallab.lut'	; color table.

tvlct, red, green, blue, /get

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
print, 'fits_name: ', fits_name

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

;   dispmin = MIN (fits_img, max=dispmax)

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   origin   = fxpar (hdu, 'ORIGIN')
   telescop = fxpar (hdu, 'TELESCOP')
   date     = fxpar (hdu, 'DATE-OBS')
   time     = fxpar (hdu, 'TIME-OBS')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   bscale   = fxpar (hdu, 'BSCALE')
   bzero    = fxpar (hdu, 'BZERO')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')
   dispmin  = fxpar (hdu, 'DISPMIN')
   dispmax  = fxpar (hdu, 'DISPMAX')

   year  = STRMID (date, 0, 4)
   month = STRMID (date, 5, 2)
   day   = STRMID (date, 8, 2)
   iday = FIX (day)
   sday = STRTRIM (STRING (iday))
   IF (iday LT 10)  THEN sday = ' ' + sday
   imonth = FIX (month)
   smonth = month_name [imonth - 1]
   sdate = sday + ' ' +  smonth + ' ' + year
   sdate = STRTRIM (sdate, 2)

   stime = time + ' UT'

   doy = day_of_year (year, month, day)

   IF (N_ELEMENTS (wmin) GT 0) THEN	$
       imin = wmin			$
   ELSE					$
       imin = dispmin

   IF (N_ELEMENTS (wmax) GT 0) THEN	$
       imax = wmax			$
   ELSE					$
       imax = dispmax


;   dispmin = -1000
;   dispmax =  5000
;   print, FORMAT = '("dispmin/max: ", E10.3, E10.3)', dispmin, dispmax

   print, 'xdim/ydim: ', xdim, ydim
   print, 'datamin/datamax: ', datamin, datamax
   print, 'dispmin/dispmax: ', dispmin, dispmax
   print, 'imin/imax: ',imin, imax

   ;--- Convert to byte pixels.
   ;--- Display image.

;   IF (xdim NE xdim_prev OR ydim NE ydim_prev) THEN	$
;      window, xsize=xdim, ysize=ydim

   std_img = fits_img
;   WINDOW, xsize=xdim, ysize=ydim

   ; Resize to 512x512.
   ; std_img = CONGRID (fits_img, 512, 512)


   byt_img = BYTSCL (std_img, min=imin, max=imax, top=249)
   TV, byt_img						; Display image.

   print, 'xdim/ydim: ', xdim, ydim

   ;--- Label image.

   IF (KEYWORD_SET (label)) THEN	$
   BEGIN
      cs = 1.0
;      IF (!version.os EQ 'IRIX')  THEN cs = 0.7
;      IF (!version.os EQ 'sunos') THEN cs = 0.7
;      IF (!version.os EQ 'linux') THEN cs = 0.7

      sorigin   = STRTRIM (origin, 2)
      stelescop = STRTRIM (telescop, 2)
      stype_obs = STRTRIM (type_obs, 2)
      sbunit    = STRTRIM (bunit, 2)

      stitle = sorigin + ' ' + stelescop + ' ' + stype_obs 
      stitle = 'MLSO ' + stelescop + ' ' + sbunit

      sdoy  = STRTRIM (STRING (doy), 2)
      IF (doy LT 100) THEN sdoy = ' ' + sdoy
      IF (doy LT 10)  THEN sdoy = ' ' + sdoy
      labeldoy = 'DOY: ' + sdoy

      swmin = STRTRIM (STRING (imin), 2)
      swmax = STRTRIM (STRING (imax), 2)
      sscale = 'Scaling: ' + swmin + ' to ' + swmax
      dmin = imin * bscale + bzero
      dmax = imax * bscale + bzero
      sdmin = STRTRIM (STRING (dmin, FORMAT='(G9.2)'), 2)
      sdmax = STRTRIM (STRING (dmax, FORMAT='(G9.2)'), 2)
      sdmin = 'MIN: ' + sdmin
      sdmax = 'MAX: ' + sdmax

      XYOUTS, 3, ydim - 15, stitle, 	/device, charsize=cs, color=255
      XYOUTS, 3, ydim - 35, sdate,	/device, charsize=cs, color=255
      XYOUTS, 3, ydim - 55, stime,	/device, charsize=cs, color=255
      XYOUTS, 3, ydim - 75, labeldoy,	/device, charsize=cs, color=255
;      XYOUTS, 3, 4,         sscale,	/device, charsize=cs, color=255
      XYOUTS, 3, 24,        sdmin,      /device, charsize=cs, color=255
      XYOUTS, 3,  4,        sdmax,      /device, charsize=cs, color=255
   END
   byt_img = TVRD ()

   ;--- Create JPEG image [24-bits/pixel].

;   jpg_img = bytarr (3, 512, 512)
   jpg_img = bytarr (3, 960, 960)
   jpg_img [0, *, *] = red   [byt_img [*, *]]
   jpg_img [1, *, *] = green [byt_img [*, *]]
   jpg_img [2, *, *] = blue  [byt_img [*, *]]

   ;--- Write jpeg image to disk.

   sloc = RSTRPOS (fits_name, '.fts')
   new_name = strmid (fits_name, 0, sloc) + '.jpg'
   print, 'Write JPEG image : ', new_name
   write_jpeg, new_name, jpg_img, true=1, quality=75	; Write new image.
   xdim_prev = xdim
   ydim_prev = ydim
END	;}

SET_PLOT, 'x'
CLOSE, 1
END
