;+
; NAME	ftsba.pro
;
; PURPOSE	Plot Mk4 barrel angle files.
;
; SYNTAX	ftsba, list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    7 April 2003
;-

pro ftsba, fts_list

rtab = bindgen (256)
gtab = bindgen (256)
btab = bindgen (256)
fts_name = ''
ba_name = 'ba.gif'

SET_PLOT, 'Z'

LOADCT, 0
lct,'/home/stanger/color/bwcp.lut'	; color table.
tvlct, rtab, gtab, btab, /get

fcol   =   0
bcol   = 255
!P.BACKGROUND=bcol

white  = 255
red    = 254
green  = 253
blue   = 252
grey   = 251
yellow = 250
black  =   0

counter = 0
bzero  = 0.0
bscale = 1.0

;--- File list loop.

CLOSE, 1
OPENR, 1, fts_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS file header & barrel angle data.

   READF, 1, fts_name				; Get file name.
   PRINT, 'fts_name: ', fts_name
   ba = readfits (fts_name, hdu, /noscale)
   fba = float (ba)

   counter = counter + 1
   IF (counter EQ 1)  THEN	$
       BEGIN
       filedate = STRMID (fts_name, 0, 8)
       OPENW, 2, filedate + '.ba.txt'
       PRINTF, 2, 'MK4 Barrel Angle Min/Max'
       END

   ;--- Extract information from header.

   xdim = fxpar (hdu, 'NAXIS1')
   ydim = fxpar (hdu, 'NAXIS2')
   bzero = fxpar (hdu, 'BZERO')
   bscale = fxpar (hdu, 'BSCALE')
;   PRINT, 'bzero, bscale: ', bzero, bscale
   nbitpix = fxpar (hdu, 'BITPIX')
   datamin = fxpar (hdu, 'DATAMIN')
   datamax = fxpar (hdu, 'DATAMAX')
   dispmin = fxpar (hdu, 'DISPMIN')
   dispmax = fxpar (hdu, 'DISPMAX')
   PRINT, 'datamin/datamax: ', datamin, datamax

   ;--- Plot barrel angles.

   ptitle = fts_name + ' Barrel Angles'
   xlab = 'index'
   ylab = 'barrel angle'
   PLOT, fba, title=ptitle, xtitle=xlab, ytitle=ylab, color=fcol

   ba_min = MIN (fba, max=ba_max)
   PRINT, FORMAT = '("ba_min/max: ", G10.3, G10.3)', ba_min, ba_max
   PRINTF, 2, fts_name, '  ba_min: ', FIX (ba_min), '  ba_max: ', FIX (ba_max)

   ;--- Write plot image to disk.

   baimg = TVRD () ;
   sloc = RSTRPOS (fts_name, '.fts')
   ba_name = strmid (fts_name, 0, sloc) + '.gif'
;   PRINT, 'Write ba plot : ', ba_name
   WRITE_GIF, ba_name, baimg, rtab, gtab, btab		; Write ba plot.

END	;}

CLOSE, 2
CLOSE, 1

!P.BACKGROUND = 0
SET_PLOT, 'X'
END
