;+
; ftscir.pro
;
; Find the center of the solar disc in a FITS image.
;
; Andrew L. Stanger   HAO/NCAR   18 Jan 2001
;-

PRO ftscir, fits_file

img = readfits (fits_file, hdu, /noscale)

print, 'fits_file: ', fits_file

xdim = fxpar (hdu, 'NAXIS1')
ydim = fxpar (hdu, 'NAXIS2')

crpix1 = fxpar (hdu, 'CRPIX1')
crpix2 = fxpar (hdu, 'CRPIX2')

crradius = fxpar (hdu, 'CRRADIUS')

print, 'crpix1/crpix2:', crpix1, crpix2
print, 'crradius: ', crradius

xac = xdim / 2
yac = ydim / 2

xv = fltarr (xdim)
yv = fltarr (ydim)

xv = img [*,   yac]
yv = img [xac,   *]

icen = fltarr (11 * 11)
icen = img [xac-5:xac+5, yac-5:yac+5]

bcen = TOTAL (icen) / 121.0

bcen0 = img [xac, yac]
print, 'bcen0:', bcen0
print, 'bcen: ', bcen

bedge = bcen / 8.0

print, 'bedge: ', bedge

FOR i = xac - 1, 0, -1 DO	$
    BEGIN
    IF (xv [i] LE bedge) THEN	$
	BEGIN
	xvl = i
	yvl = yac
	goto, doxvr
	END
    END

doxvr:

FOR i = xac, xdim - 1 DO	$
    BEGIN
    IF (xv [i] LE bedge) THEN	$
	BEGIN
	xvr = i
	yvr = yac
	goto, doyvt
	END
    END

doyvt:

FOR i = yac, ydim - 1 DO	$
    BEGIN
    IF (yv [i] LE bedge) THEN	$
	BEGIN
	xvt = xac
	yvt = i
	goto, docir
	END
    END

docir:

cir_3pnt, [xvl, xvr, xvt], [yvl, yvr, yvt], radius, xcen, ycen

print, 'xdim/ydim: ', xdim, ydim
print, 'xvl/yvl: ', xvl, yvl
print, 'xvr/yvr: ', xvr, yvr
print, 'xvt/yvt: ', xvt, yvt

print, 'xcen/ycen: ', xcen, ycen
print, 'radius: ', radius

fxaddpar, hdu, 'CRPIX1', xcen
fxaddpar, hdu, 'CRPIX2', ycen

writefits, fits_file, img, hdu

END
