;+
; NAME	ftsdeltab.pro
;
; PURPOSE	Compute the ratio between two FITS images.
;
; SYNTAX	ftsdeltab, fits_reference_file, fits_image_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   17 Nov 1998
;-

pro ftsdeltab, ref_name, cme_list

cme_name = ''
ref_hdu = ''
cme_hdu = ''
tscope = 'sun'
print, 'ref_name: ', ref_name
lct,'/home/stanger/color/dif.lut'	; load difference color table.

;--- Read reference image.

;ref_img    = rfits (ref_name, header=ref_hdu, /scale)
ref_img    = readfits (ref_name, ref_hdu)

ref_telescop = fxpar (ref_hdu, "TELESCOP")
ref_instrume = fxpar (ref_hdu, "INSTRUME")
ref_detector = fxpar (ref_hdu, "DETECTOR")
ref_date     = fxpar (ref_hdu, "DATE-OBS")
ref_time     = fxpar (ref_hdu, "TIME-OBS")
ref_hour     = strmid (ref_time, 0, 2)
ref_minute   = strmid (ref_time, 3, 2)

ref_min = MIN (ref_img, max=ref_max)
print, FORMAT = '("ref_min/max: ", E10.3, E10.3)', ref_min, ref_max

IF (STRPOS (ref_telescop, "SMM C/P") NE -1)	THEN  tscope = 'smmcp'
IF (STRPOS (ref_telescop, "MK3") NE -1)		THEN  tscope = 'mk3'
IF (STRPOS (ref_instrume, "SXT") NE -1)		THEN  tscope = 'sxt'
IF (STRPOS (ref_telescop, "SOHO") NE -1)	THEN	$	; SOHO
   BEGIN
   IF (STRPOS (ref_instrume, "LASCO") NE -1)	THEN	$
      BEGIN
      IF (STRPOS (ref_detector, "C1") NE -1)  THEN tscope = 'c1'
      IF (STRPOS (ref_detector, "C2") NE -1)  THEN tscope = 'c2'
      IF (STRPOS (ref_detector, "C3") NE -1)  THEN tscope = 'c3'
      END
   IF (STRPOS (ref_instrume, "EIT") NE -1)  THEN tscope = 'eit'
   END

;--- File list loop.

CLOSE, 1
OPENR, 1, cme_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, cme_name				; Get file name.
print, 'cme_name: ', cme_name
;   cme_img = rfits (cme_name, header=cme_hdu, /scale)	; Read CME image.
   cme_img = readfits (cme_name, cme_hdu)	; Read CME image.

   ;--- Extract information from header.

   cme_date   = fxpar  (cme_hdu, "DATE-OBS")
   cme_time   = fxpar  (cme_hdu, "TIME-OBS")
   cme_day    = strmid (cme_date, 0, 2)
   cme_month  = strmid (cme_date, 3, 2)
   cme_year   = strmid (cme_date, 6, 2)
   cme_hour   = strmid (cme_time, 0, 2)
   cme_minute = strmid (cme_time, 3, 2)
   del_name = tscope + '.' + cme_year + cme_month + cme_day + '.' +	$
		       cme_hour + cme_minute + '-' +		$
		       ref_hour + ref_minute + '.db.fts'

   cme_min = MIN (cme_img, max=cme_max)
   print, FORMAT = '("cme_min/max: ", E10.3, E10.3)', cme_min, cme_max

   ;--- Compute image ratio.

   del_img = (cme_img - ref_img) / (ref_img > 0)	; Compute (B2-B1)/B1.

   del_min = min (del_img, max=del_max)			; Get image min/max.
;   print, FORMAT = '("del_min/max: ", E10.3, E10.3)', del_min, del_max
   IF (del_min LT 0.0) THEN	$			; Symmetric about zero.
      IF (-del_min LT del_max) THEN del_min = -del_max  ELSE del_max = -del_min

   del_min = -1.0
   del_max =  1.0
;   print, FORMAT = '("del_min/max: ", E10.3, E10.3)', del_min, del_max

;   del_min = -1.0e-7
;   del_max =  1.0e-7
   print, FORMAT = '("del_min/max: ", E10.3, E10.3)', del_min, del_max

   ;--- Convert to byte pixels.
   ;--- Display image.

   byt_img = bytscl (del_img, min=del_min, max=del_max, top=249)
   tv, byt_img						; Display image.

   ;--- Update FITS header.

   del_hdu = cme_hdu
   fxaddpar, del_hdu, "DATAMIN", del_min, "BSUN UNITS"
   fxaddpar, del_hdu, "DATAMAX", del_max, "BSUN UNITS"
   fxaddpar, del_hdu, "DISPMIN",   0.0
   fxaddpar, del_hdu, "DISPMAX", 249.0
   fxaddpar, del_hdu, "BSCALE",    1.0
   fxaddpar, del_hdu, "BZERO",     0.0
   del_comment = 'PRE-EVENT IMAGE AT ' + string (ref_hour) + ':' + string (ref_minute) + $
		 'WAS SUBTRACTED FROM EVENT FRAME,'
   del_comment = 'AND THEN THE RESULT WAS DIVIDED BY THE PRE-EVENT IMAGE.'
   fxaddpar, del_hdu, "HISTORY", del_comment

   ;--- Write ratio image to disk.

   print, 'Write ratio image : ', del_name
   writefits, del_name, byt_img, del_hdu		; Write ratio image.
END	;}

CLOSE, 1
END
