;+
; NAME	mk4hair.pro
;
; PURPOSE	Read Mk4 FITS images & plot hairline positions.
;
; SYNTAX	mk4hair, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   21 April 2000
;-

pro mk4hair, fits_list

CLOSE, 1
fits_name = ''

time = fltarr (120)
ohlo = intarr (120)
ohhi = intarr (120)
ahlo = intarr (120)
ahhi = intarr (120)
i = 0

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   print, 'fits_name: ', fits_name

   fits_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   orghair0 = fxpar (hdu, 'ORGHAIR0')
   orghair1 = fxpar (hdu, 'ORGHAIR1')
   hair0    = fxpar (hdu, 'HAIR0')
   hair1    = fxpar (hdu, 'HAIR1')

   print, 'i, orghair0, orghair1: ', i, orghair0, orghair1

   ohlo [i] = orghair0
   ohhi [i] = orghair1
   ahlo [i] = hair0
   ahhi [i] = hair1
   
   time [i] = i
   i = i + 1

END	;}

CLOSE, 1

set_plot, 'PS'
device, filename='mk4hair.ps'

plot, time, ohlo
plot, time, ohhi
plot, time, ahlo
plot, time, ahhi

set_plot,'x'

END
