;+
; rotdpmsq9.pro   IDL procedure
;
; PURPOSE
;	Read a MLSO/PICS FITS image and rotate it,
;       using CROTA1 and CRPIX1, CRPIX2.
;	Also, crop image to a square.
;
; SYNTAX
;	rotdpmsq, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   9 October 1997
;	26 May 1999: Update FITS header: COORDNAM = 'HELIOCENTRIC'
;-

pro rotdpmsq9, fits_list

fits_name = ''				; Define FITS file name variable.
xdim = 0
ydim = 0
xcen = 0
ycen = 0
roll = 0.0
mag  = 1.0

CLOSE, 1
CLOSE, 2

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.


      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')
      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')
      roll = -fxpar (hdu, 'CROTA2')
      print, 'xdim, ydim: ', xdim, ydim
      print, 'xcen, ycen: ', xcen, ycen

      ;--- Overwrite binary header line with 1st actual line of data.

      img [*,0] = img [*,1]

      ; -- Copy image into a square array in which sun is centered.

      xl = xcen + 1
      xr = xdim - (xcen + 1)
      yb = ycen + 1
      yt = ydim - (ycen + 1)
      print, 'xl, xr: ', xl, xr
      print, 'yb, yt: ', yb, yt
      xmax = MAX ([xl, xr])
      ymax = MAX ([yb, yt])
      print, 'xmax, ymax: ', xmax, ymax

      nxdim = MAX ([xmax, ymax]) * 2
      nydim = nxdim
      nxcen = nxdim / 2
      nycen = nydim / 2
      print, 'nxdim, nydim: ', nxdim, nydim
      imgsq = INTARR (nxdim, nydim)

      xbeg = nxcen - xl
      xend = xbeg + xdim - 1
      ybeg = nycen - yb
      yend = ybeg + ydim - 1
      print, 'xbeg/xend: ', xbeg, xend
      print, 'ybeg/yend: ', ybeg, yend
      imgsq (xbeg:xend, ybeg:yend) = img (0:xdim-1, 0:ydim-1)

      imgrot = ROT (imgsq, roll, mag, nxcen, nycen, MISSING=0, INTERP=Cubic)

      sdim = MIN ([xdim, ydim])
      sdim = 880
      print, 'sdim: ', sdim
      xcen = (sdim / 2) - 1
      ycen = (sdim / 2) - 1
      roll = 0.0
      print, 'new xcen, ycen: ', xcen, ycen
      print, 'sizeof (imgrot): ', sizeof (imgrot)

      imgout = intarr (sdim, sdim)
      xbeg = nxcen - xcen
      xend = xbeg + sdim - 1
      ybeg = nycen - ycen
      yend = ybeg + sdim - 1
      print, 'xbeg/xend: ', xbeg, xend
      print, 'ybeg/yend: ', ybeg, yend
      imgout (0:sdim-1, 0:sdim-1) = imgrot (xbeg:xend, ybeg:yend)

      ; -- Modify FITS header.

      xcen = xcen + 0.5
      ycen = ycen + 0.5
      fxaddpar, hdu, 'NAXIS1',   sdim
      fxaddpar, hdu, 'NAXIS2',   sdim
      fxaddpar, hdu, 'CRPIX1',   xcen,       'SUN CENTER: X AXIS [PIXELS]'
      fxaddpar, hdu, 'CRPIX2',   ycen,       'SUN CENTER: Y AXIS [PIXELS]'
      fxaddpar, hdu, 'CROTA1',   roll,       'SOLAR NORTH ANGLE W.R.T. +Y AXIS'
      fxaddpar, hdu, 'CROTA2',   roll,       'SOLAR NORTH ANGLE W.R.T. +Y AXIS'
      fxaddpar, hdu, 'COORDNAM', 'HELIOCENTRIC', 'SOLAR NORTH ALIGNED WITH +Y AXIS'
;      fxaddpar, hdu, 'HISTORY',  'IMAGE ROTATED TO ALIGN SOLAR NORTH WITH +Y AXIS.'

      ; -- Write FITS image to disk.

      sloc = RSTRPOS (fits_name, '.fts')
      rfits_name = strmid (fits_name, 0, sloc) + '.rot.sq.fts'	; Create output file name.
      print, 'rfits_name: ', rfits_name
      writefits, rfits_name, imgout, hdu

   END
CLOSE, 1

END
