;+
; NAME	sqrtfts.pro
;
; PURPOSE	Perform square root scaling on FITS images.
;
; SYNTAX	sqrtfts, list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   29 November 1999
;-

pro sqrtfts, fts_list

red = bindgen (256)
green = red
blue  = red
cme_name = ''
lct,'/home/stanger/color/bwy.lut'	; color table.
tvlct, red, green, blue, /get

;--- File list loop.

CLOSE, 1
OPENR, 1, fts_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fts_name				; Get file name.
print, 'fts_name: ', fts_name
   READ_GIF, fts_name, fts_img			; Read CME image.

   ;--- Extract information from header.

   fts_min = MIN (fts_img, max=fts_max)
   print, FORMAT = '("fts_min/max: ", E10.3, E10.3)', fts_min, fts_max

   ;--- Compute image difference.

   new_img = sqrt (fts_img)				; Compute square root.

   new_min = min (new_img, max=new_max)			; Get image min/max.
   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max
;   IF (new_min LT 0.0) THEN	$			; Symmetric about zero.
;      IF (-new_min LT new_max) THEN new_min = -new_max  ELSE new_max = -new_min
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   new_min = -5.00e-8
;   new_max =  5.00e-8
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

   ;--- Convert to byte pixels.
   ;--- Display image.

   byt_img = bytscl (new_img, min=new_min, max=new_max, top=249)
   tv, byt_img						; Display image.

   ;--- Write newference image to disk.

   sloc = RSTRPOS (fits_name, '.fts')
   new_name = strmid (fits_name, 0, sloc) + '.sqrt.fts'
   print, 'Write sqrt image : ', new_name
   write_gif, new_name, byt_img, red, green, blue	; Write new image.
END	;}

CLOSE, 1
END
