;+
; NAME		suncir.pro
;
; PURPOSE	Draw a circle at 1.0 Rsun.
;
; SYNTAX	suncir, xdim, ydim, xcen, ycen, xb, yb, rspix, roll
;
;		xdim: x-axis dimension
;		ydim: y-axis dimension
;		xcen: x-axis center
;		ycen: y-axis center
;		xb:   x-axis border for annotation
;		yb:   y-axis border for annotation
;		rspix: solar radii/pixel
;		roll:  roll angle (solar north w.r.t. +Y axis, CCW=positive).
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   22 November 2004
;-

PRO suncir2, xdim, ydim, xcen, ycen, xb, yb, rspix, roll

   white  = 255
   red    = 254
   green  = 253
   blue   = 252
   grey   = 251
   yellow = 250
   black  =   0

   r       = 1.0
   r1min   = 0.2		; radial inner limit for major lines.
   r1max   = 1.0		; radial outer limit for major lines.
   r1inc   = 0.1		; radial  increment between major lines
   ang1inc = 90.0		; angular increment between major lines.

   r2min   = 0.0		; radial inner limit for major lines.
   r2max   = 1.0		; radial outer limit for major lines.
   r2inc   = 0.2		; radial  increment between major lines
   ang2inc = 30.0		; angular increment between major lines.

   ; --- Draw "+" mark at sun center.

   xg = FIX (xcen + xb + 0.5)
   yg = FIX (ycen + yb + 0.5)

   PLOTS, xg,   yg,   /data, color=white
   PLOTS, xg+1, yg,   /data, color=white
   PLOTS, xg+2, yg,   /data, color=white
   PLOTS, xg-1, yg,   /data, color=white
   PLOTS, xg-2, yg,   /data, color=white
   PLOTS, xg,   yg+1, /data, color=white
   PLOTS, xg,   yg+2, /data, color=white
   PLOTS, xg,   yg-1, /data, color=white
   PLOTS, xg,   yg-2, /data, color=white

   ; --- Draw radial lines.

   FOR th = 0.0, 360.0, ang1inc DO		$
   BEGIN ;{
      FOR radius = r1min, r1max*1.01, r1inc DO	$
      BEGIN ;{
	 ierr = rcoord (radius, th, x, y, 1, roll, xcen, ycen, rspix)
	 xg = FIX (x + xb + 0.5)
	 yg = FIX (y + yb + 0.5)
	 PLOTS, xg, yg, color=white
      END   ;}
   END   ;}

   ; --- Draw radial lines.

   FOR th = 0.0, 360.0, ang2inc DO		$
   BEGIN ;{
      FOR radius = r2min, r2max, r2inc DO	$
      BEGIN ;{
	 ierr = rcoord (radius, th, x, y, 1, roll, xcen, ycen, rspix)
	 xg = FIX (x + xb + 0.5)
	 yg = FIX (y + yb + 0.5)
	 PLOTS, xg, yg, color=white
      END   ;}
   END   ;}

   ; --- Draw a circle at 1.0 Rsun:  one dot every 10 degrees.

   FOR th = 0, 360, 10 DO			$
   BEGIN ;{
      ierr = rcoord (r, th, x, y, 1, roll, xcen, ycen, rspix)
      xg = FIX (x + xb + 0.5)
      yg = FIX (y + yb + 0.5)
      PLOTS, xg, yg, color=white
   END   ;}

   ; --- Draw a circle at 1.0 Rsun:  one BOLD dot every 30 degrees.

   FOR th = 0, 360, 30 DO			$
   BEGIN ;{
      ierr = rcoord (r, th, x, y, 1, roll, xcen, ycen, rspix)
      xg = FIX (x + xb + 0.5)
      yg = FIX (y + yb + 0.5)
      PLOTS, xg,   yg,   color=white
      PLOTS, xg+1, yg,   color=white
      PLOTS, xg-1, yg,   color=white
      PLOTS, xg,   yg+1, color=white
      PLOTS, xg,   yg-1, color=white
   END   ;}

   RETURN
END
