;+
; NAME
;	addras
;
; PURPOSE
;	"Add" together two Sun Rasterfile images & write result to disk.
;
; CALLING SEQUENCE
;	addras, filelist1, filelist2
;
; PARAMETERS
;	filelist1		Sun Rasterfile name list
;	filelist2		Sun Rasterfile name list
;
; OPTIONAL PARAMETERS
;	outnam		Base name for output files.
;
; OUTPUT
;	composite Sun Rasterfile.
;
; HISTORY
;	23 March 1995, Andrew L. Stanger, HAO/NCAR
;-

pro addras, filelist1, filelist2, outnam=outnam

srf1 = ''
srf2 = ''

; --- Create RGB colormap (greyscale: equal brightness for all 3 colors).

map_red   = bindgen (256)

map_green = bindgen (256)

map_blue  = bindgen (256)

OPENR, 1, filelist1
OPENR, 2, filelist2

i = 0 

WHILE (NOT EOF (1)) DO BEGIN

   i = i + 1
   READF, 1, srf1				; Read SRF name
   READF, 2, srf2				; Read SRF name

   read_srf, srf1, img1				; Read SRF image
   read_srf, srf2, img2				; Read SRF image

   img_out  = img1 > img2			; composite image: maximum op.

   if (keyword_set (outnam)) then begin
      srf_out = outnam + string (format='(I2.2)', i) + '.ras'
   endif else begin
      srf_out = 'img' + string (format='(I2.2)', i) + '.ras'
   endelse

   ; --- Write composite image to disk.

   write_srf, srf_out, img_out, map_red, map_green, map_blue

ENDWHILE

CLOSE, 1
CLOSE, 2
END
