;+
; NAME
;	brtras
;
; PURPOSE
;	Brighten Sun Rasterfile images & write result to disk.
;
; CALLING SEQUENCE
;	brtras, file1
;
; PARAMETERS
;	listname	Name of file containing Rasterfile names
;
; OPTIONAL PARAMETERS
;	outnam		Base name for output files.
;
; OUTPUT
;	brightened Sun Rasterfile.
;
; HISTORY
;	24 March 1995, Andrew L. Stanger, HAO/NCAR
;-

pro brtras, listname, outnam=outnam

srfnam = ''

; --- Create RGB colormap (greyscale: equal brightness for all 3 colors).

map_red   = bindgen (256)

map_green = bindgen (256)

map_blue  = bindgen (256)

OPENR, 1, listname

i = 0 

WHILE (NOT EOF (1)) DO BEGIN

   i = i + 1
   READF, 1, srfnam				; Read SRF name

   read_srf, srfnam, img				; Read SRF image

   imgmin = MIN (img, max=imgmax)
   print, 'imgmin/max: ', imgmin, imgmax
   imgmax = 0.75 * imgmax
   print, 'imgmin/max: ', imgmin, imgmax
   img_out  = bytscl (img, min=imgmin, max=imgmax)	; brightened image.

   dotpos = strpos (srfnam, '.ras')
   if (dotpos EQ -1) then dotpos = strpos (srfnam, '.srf')
   if (dotpos EQ -1) then dotpos = strlen (srfnam)
   srfnew = strmid (srfnam, 0, dotpos)
   if (keyword_set (outnam)) then begin
      srf_out = outnam + string (format='(I2.2)', i) + '.ras'
   endif else begin
      srf_out = srfnew + '.brt.ras'
   endelse
   print, 'Output file name: ', srf_out

   ; --- Write composite image to disk.

   write_srf, srf_out, img_out, map_red, map_green, map_blue

ENDWHILE

CLOSE, 1
END
