pro chipf2fts

;+
;--- Convert MLSO/CHIP floating point image to FITS format.
;
; Andrew L. Stanger   HAO/NCAR   23 Aug 1995
;-

   xdim = 767
   ydim = 516
   xcen = xdim / 2 - 1
   ycen = ydim / 2 - 1
   sunrad  = 206
   imgbuf = fltarr (xdim, ydim) 
   chipmin = -0.19 & chipmax = -0.05	; 9000 -> 10500
 
   filename = 'chip.img'
   fileout  = 'chip.img.fts'
   listfile = 'chipf.l'

   loadct, 0
   window
   dsk = disk (512, 512, sunrad, 255.5, 255.5)		; disk w/ radius sunrad
   index_dsk = where (dsk eq 0)
 
   close, 10 & openr, 10, listfile
 
   while ( not eof (10) ) do begin ;{
      readf, 10, filename
      fileout = filename + '.fts'
      close, 11 & openr, 11, filename
      readu, 11, imgbuf & close, 11 
      bytbuf = bytscl (imgbuf, min=chipmin, max=chipmax)
      grnbuf = bytbuf (127:638, 2:513)		; Extract 512x512 image.
      bytbuf = rot (grnbuf, 138.5, 1.0)		; Rotate image.

      ;--- Set pixel value to zero if beyond sunrad distance from center.

      bytbuf (index_dsk) = 0			; Set area outside disk to zero.

;      for iy = 0, ydim - 1 do	$
;      begin
;	 ydist = float (iy - ycen)
;	 yd = ydist * ydist
;         for ix = 0, xdim -1 do	$
;	 begin
;	    xdist = float (ix - xcen)
;	    xd = xdist * xdist
;	    r  = sqrt (xd + yd)			; Distance from center.
;	    if (r gt sunrad) then	$
;	       bytbuf (ix, iy) = 0		; Set pixel =0 if r > sunrad.
;         print, 'ix,iy,xd,yd,r,bytbuf: ', ix, iy, xd, yd, r, '  ', bytbuf (ix, iy)
;         end
;      end
      tv, bytbuf

      fxhmake,  hdu, bytbuf			; Make basic FITS header.
      fxaddpar, hdu, 'ORIGIN', 'MLSO'
      fxaddpar, hdu, 'TELESCOP', 'ACOS'
      fxaddpar, hdu, 'INSTRUME', 'CHIP'
      fxaddpar, hdu, 'OBJECT', 'SUN'
      fxaddpar, hdu, 'TYPE-OBS', 'HE:1083 NM'
      fxaddpar, hdu, 'DATE-OBS', '26/06/95'
      hour   = strmid (filename, 10, 2)
      ihour  = fix (hour)
      ihour = ihour + 6
      hour = string (ihour, FORMAT = "(I2)")
      minute = strmid (filename, 12, 2)
      time_obs = hour + ':' + minute + ':00'
      fxaddpar, hdu, 'TIME-OBS', time_obs
      fxaddpar, hdu, 'COMMENT', '  FINAL CALIB IMAGES'
      fxaddpar, hdu, 'COMMENT', '  ROTATED 138.5 DEGREES CW TO ORIENT NORTH UP.'
      fileout = 'chip.950626.' + hour + minute + '.calib.fts'
      print, 'CHIP name: ', filename, '   FITS name: ', fileout
      writefits, fileout, bytbuf, hdu		; Write image as FITS file.
   endwhile ;} 
 
   close, 10

end 
