pro chipi2fts

;+
;--- Convert MLSO/CHIP short integer image to FITS format.
;
; Andrew L. Stanger   HAO/NCAR   23 Aug 1995
; 24 Aug 1995: crop to 512x512 pixels & rotate 138.5 degrees CW.
; 29 Aug 1995: invert intensity scaling of image.
;-

xdim = 767
ydim = 516
sunrad  = 201
imgbuf = intarr (xdim, ydim) 
chipmin = 9000 & chipmax = 10500
 
filename = 'chip.img'
fileout  = 'chip.img.fts'
listfile = 'chipi.l'
   loadct, 0
   window
   dsk = disk (512, 512, sunrad, 255.0, 255.0)		; disk w/ radius sunrad
   index_dsk = where (dsk eq 0)
 
   close, 10 & openr, 10, listfile
 
   while ( not eof (10) ) do begin ;{
      readf, 10, filename
      fileout = filename + '.fts'
      close, 11 & openr, 11, filename
      readu, 11, imgbuf & close, 11 
      bytbuf = bytscl (imgbuf, min=chipmin, max=chipmax)
      grnbuf = bytbuf (127:638, 2:513)		; Extract 512x512 image.
      bytbuf = rot (grnbuf, 138.5, 1.0)		; Rotate image.
      tv, bytbuf

      bytbuf = 249 - bytbuf			; Invert intensity scaline.

      ;--- Set pixel value to zero if beyond sunrad distance from center.

      bytbuf (index_dsk) = 0			; Set area outside disk to zero.
      tv, bytbuf

      fxhmake,  hdu, bytbuf			; Make basic FITS header.
      fxaddpar, hdu, 'ORIGIN', 'MLSO'
      fxaddpar, hdu, 'TELESCOP', 'ACOS'
      fxaddpar, hdu, 'INSTRUME', 'CHIP'
      fxaddpar, hdu, 'OBJECT', 'SUN'
      fxaddpar, hdu, 'TYPE-OBS', 'HE:1083 NM'
      fxaddpar, hdu, 'DATE-OBS', '26/06/95'
      hour   = strmid (filename, 11, 2)
      ihour  = fix (hour)
      ihour = ihour + 6
      hour = string (ihour, FORMAT = "(I2)")
      minute = strmid (filename, 13, 2)
      time_obs = hour + ':' + minute + ':00'
      fxaddpar, hdu, 'TIME-OBS', time_obs
      fxaddpar, hdu, 'COMMENT', '  FINAL SHIFT IMAGES'
      fxaddpar, hdu, 'COMMENT', '  ROTATED 138.5 DEGREES CW TO ORIENT NORTH UP.'
      fileout = 'chip.950626.' + hour + minute + '.shift.fts'
      print, 'CHIP name: ', filename, '   FITS name: ', fileout
      writefits, fileout, bytbuf, hdu		; Write image as FITS file.
   endwhile ;} 
 
   close, 10

end 
