;+
; dirfts.pro   IDL procedure
;
; PURPOSE
;	Read a FITS image and change the scan direction. 
;
; SYNTAX
;	dirfits, 'fits_list', 'rd'
;
;	display directions: ru (right up)
;			    rd (right down)
;			    lu (left up)
;			    ld (left down)
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   16 October 1997
;-

pro dirfts, fits_list, disp_dir

fits_name = ''				; Define FITS file name variable.

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      ; -- Modify FITS header.

      fxaddpar, hdu, 'DISP-DIR', disp_dir

      ; -- Write FITS image to disk.

      writefits, fits_name, img, hdu

   END
CLOSE, 1

END
