;+
; dskcen1.pro   IDL procedure
;
; PURPOSE
;	Read a FITS image and find disk center & radius.
;
; SYNTAX
;	dskcen, 'fits_list'
;
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;	gen_loc
;
; HISTORY
;	Author: Andrew L. Stanger   10 June 1998
;-

pro dskcen1, fits_list

fits_name = ''				; Define FITS file name variable.

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')

      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')
      crradius = fxpar (hdu, 'CRRADIUS')

      ;--- Copy line 1 into line 0 (remove binary header) for PICS/CHIP images.

      IF (xdim EQ 1534 AND ydim EQ 1030) THEN img [*, 0] = img [*, 1]

      gen_loc, img, /cir, CENTROID=centroid, HORZRAD=hdiam, VERTRAD=vdiam

      fxaddpar, hdu, 'CRPIX1', centroid [0], ' X Sun Center [pixels]'
      fxaddpar, hdu, 'CRPIX2', centroid [1], ' Y Sun Center [pixels]'
;      fxaddpar, hdu, 'CRRADIUS', hdiam / 2.0, ' Pixels per solar radius'

      PRINT, 'center  [orig]:    ', xcen, ycen
      PRINT, 'radiuis [orig]:    ', crradius
      PRINT, 'center  [compute]: ', centroid
      PRINT, 'radius  [compute]: ', hdiam / 2.0

      ; -- Write FITS image to disk.

      writefits, fits_name, img, hdu

   END
CLOSE, 1

 END
