;+
; fig78.pro
;
; 8 Mar 2002		Pseudo-color version
;
; IDL pseudo-color setup:
;
;     device, pseudo_color=8
;     window, colors = 256
;     wdelete
;     help, /device
;-

;sig3.pro
;Feb 27,2002
;final version of sig2
;IMPORTANT: do not run for np ne 1 because the formulae are sloppily 
;treated with np=1 here and there

;lofc2.pro
;Feb 14,2002, case2, lines of force of np-helical soluton

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      FUNCTION fpsi, rho

; streamfunction

	common para,omega,gamma,np,k,frs
      
      fpsi=-omega*rho*BESELJ(omega*rho,np+1) + $
            (np-gamma)*BESELJ(omega*rho,np)
      return, double(fpsi)

      END

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	FUNCTION dfpsi, rho

; first derivative, dfpsi is derivative of fpsi wrt omega*rho

	common para,omega,gamma,np,k,frs
      
      dfpsi=(gamma+(np*(np - gamma)-(omega*rho)^2)*.5/np)*$
	BESELJ(omega*rho,np+1) + $
            (np*(np - gamma)-(omega*rho)^2)*.5 $
	*BESELJ(omega*rho,np-1)/np
      return, double(dfpsi)

      END

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	FUNCTION derivs, x, y

; y(0) is kr, y(1) is phi, and y(2) is z

	common para,omega,gamma,np,k,frs

	zeta=double(y(1)-k*y(2))
	f1=fpsi(y(0)) & df1=dfpsi(y(0))*omega 

	Br=omega*.5*(-(np+gamma)*BESELJ(omega*y(0),np+1) + $
	             (np - gamma)*BESELJ(omega*y(0),np-1))* $
	             cos(zeta)

	Bphi=(gamma*y(0)- df1/f1)*frs/(1. + y(0)^2)

	Bz=(gamma +y(0)*df1/f1)*frs/(1. + y(0)^2)

	sq=sqrt((f1^2 - frs^2)/y(0)^2 + Bphi^2)

	return, double([Br/sq,Bphi/(y(0)*sq),Bz/sq])
   
  	END

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	FUNCTION  bn, x, y, z

; By with y being the vertical
; program assumes that frs is the flux function value frs consistently
; used in the field line computation

	common para,omega,gamma,np,k,frs

	rr=sqrt(x^2 + y^2)
	theta=asin(abs(y/rr))
	
	if y eq 0 then begin
	if x lt 0 then theta=!pi else theta=0.
	end else begin
	if y gt 0 then begin
	if x lt 0 then theta=!pi-theta
	end else begin
	if x lt 0 then theta=-!pi+theta else theta=-theta
	endelse
	end

	zeta=theta-k*z
	f1=fpsi(rr) & df1=dfpsi(rr)*omega 
	
	psi=f1*sin(zeta)

	Br=omega*.5*(-(np+gamma)*BESELJ(omega*rr,np+1) + $
	             (np - gamma)*BESELJ(omega*rr,np-1))* $
	             cos(zeta)

	Bphi=(gamma*rr- df1/f1)*psi/(1. + rr^2)

	return, double(Br*sin(theta)+Bphi*cos(theta))
   
  	END

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      PRO fig78p

; photosphere-induced flux surface

      common para,omega,gamma,np,k,frs
	

;%%%%%%%%%%%%%%%%%%%%%%%%%


      n=200 & jn=indgen(n+1) & psi=dblarr(n+1, n+1)
      r=fltarr(n+1) & theta=r & fr=r & dfr=r & pr=r
	pr1=r & pr2=r & tension=r
      psi0=psi & psi1=psi & xx=psi & zz=psi & by=psi

	rpz1=dblarr(3) & rpz2=rpz1 & rpz3=rpz1
	drpz1=rpz1 & drpz2=rpz1 & drpz3=rpz1 

;%%%%%%%%%%%%%%%%%%%%%%%%%

; color table

;	loadct, 2				; GRN-RED-BLU-WHT

	sred   = bytarr (12)
	sgreen = bytarr (12)
	sblue  = bytarr (12)

;	sred   = [  0, 255,   0,   0, 255,   0, 255, 255, 240, 255, 255 ]
;	sgreen = [  0,   0, 255,   0, 255, 255,   0, 230, 255,   0, 255 ]
;	sblue  = [  0,   0,   0, 255,   0, 255, 255, 230, 240, 240, 255 ]

	sred   = [  0, 255,   0,   0, 200,   0, 255, 255, 240, 255, 127, 255 ]
	sgreen = [  0,   0, 255,   0, 200, 255,   0, 230, 255,   0, 127, 255 ]
	sblue  = [  0,   0,   0, 255,   0, 255, 255, 230, 240, 240, 127, 255 ]

	black   =  0
	red     =  1
	green   =  2
	blue    =  3
	yellow  =  4
	cyan    =  5
	magenta =  6
	orange  =  7
	lgreen  =  8
	rpurple =  9
	grey    = 10
	white   = 11

	tvlct, sred, sgreen, sblue

;%%%%%%%%%%%%%%%%%%%%%%%%%

	ans = string (1) & titlea = string (25)
        mydevice = !D.NAME

;%%%%%%%%%%%%%%%%%%%%%%%%%

; the parameters

;	np=1. & gamma=-1.5 & k=1. & rin=0.763672 & rou=1.61387
;	np=1. & gamma=-1.5 & k=1. & rin=0.763673 & rou=1.61386

;case2
;	np=1. & gamma=-2 & k=1. & rin=0.573243 & rou=1.03417
;	frs=1.2 & rh0=1.7 & rmm=.9

;case1
;	np=1. & gamma=2 & k=1. & rin=1.33447 & rou=2.24102
;	frs=-1.5 & rh0=2.9 & rmm=1.7

;case3
;	np=1. & gamma=1.5 & k=1. & rin=2.03027 & rou=3.78437
;	frs=-1.2 & rh0=4.6 & rmm=3.0

;case4
;	np=1. & gamma=1.2 & k=1. & rin=3.16895 & rou=6.82227
;	frs=-0.9 & rh0=8. & rmm=5.0

;case10, real sigmoids
;	np=1. & gamma=1.2 & k=1. & rin=1.49903  & rou=7.51366
;	frs=-0.2 & rh0=8. & rmm=5.0

;case11, real sigmoids
	np=1. & gamma=1.2 & k=1. & rin=1.84083   & rou=7.41991
	frs=-0.3 & rh0=8. & rmm=5.0

;case5
;	np=1. & gamma=-1.2 & k=1. & rin=0.894532 & rou=2.95702
;	frs=0.6 & rh0=4. & rmm=2.0

;case6
;	np=1. & gamma=-1.1 & k=1. & rin=0.636719 & rou=4.80078
;	frs=0. & rh0=6. & rmm=3.0 

;case7
;	np=1. & gamma=-3. & k=1. & rin=0.0537109 & rou=0.990137
;	frs=0.3 & rh0=1.2 & rmm=.5

;case8
;	np=1. & gamma=-2. & k=1. & rin=0.0764648 & rou=1.50781
	;frs=0.2 & rh0=1.7 & rmm=.9

;case9
;	np=1. & gamma=-2. & k=1. & rin=0.155566  & rou=1.43594
;	frs=0.4 & rh0=1.7 & rmm=.9


;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

	omega=sqrt(gamma^2 - np^2) 

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	rh1=0.6*rh0

;	rh1=0.8*rh0

; range

;	print,'enter factor'
;	read,factor
;	factor=1.1
	factor=1.5


;	z1=rh1 & y0=rin*factor & x1=rh1 & y1=x1

; use this for frs=-.9 to see absence of forward S

	z1=rh1 & y0=3.16895 & x1=rh1 & y1=x1

	dxx=rh1*2/n

	for i=0,n do begin
	    for j=0,n do begin
	        xx(i,j)=-rh1+j*dxx & zz(i,j)=-rh1+i*dxx
	        by(i,j)=bn(xx(i,j),y0,zz(i,j))
	    endfor
	endfor
	
	ax=max(by) & an=min(by)
        nda=20
;   	da=(ax - an)/(nda-1) 
	da=abs(ax)*2./nda
   	alevels=an + da*indgen(nda) 

	read, 'DO YOU want HARD COPY ? ', ans

        if (ans eq 'y') then begin
            set_plot, 'ps'
            device, XSIZE=7, YSIZE=7, /inches, /portrait, /color,  $
		   filename='fig78a.ps'
	    col1 = grey
	    col2 = yellow
	    erase, 0
	    id=1
	endif else begin
	    window, 1, xsize=500, ysize=500, retain=2
	    wset,1
	    col1 = white
	    col2 = yellow
	    erase, 0
	    id=0
	endelse
  	
	
      	contour, by, zz, xx, levels=alevels, /follow, $ 
		c_linestyle=alevels lt 0.0, $
		xrange=[-rh1, rh1], yrange=[-rh1, rh1], $
		/noclip,  pos=[.1, .1, .8, .8], xtitle='z', $
		ytitle='x', xmargin=[4, 2], ymargin=[4, 2], $
		title='by',  color=black

;		title='by', color=blue

		
; 	goto,label100

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

; determine the rope axis

	ep1=0.0001

	ar1=.1*rh0 & ar2=.9*rh0

label60:
	df1=dfpsi(ar1) & df2=dfpsi(ar2)

	arm=0.5*(ar1+ar2) & dfm=dfpsi(arm)
	
	if abs(dfm) lt ep1 then goto,label61 else begin
	if df1*dfm gt 0 then ar1=arm else ar2=arm
	goto,label60
	endelse

label61:
	print,' dfm=',dfm,'    arm=',arm
	
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
; set up footpoints along neutral line

; m is number of footpoints

	m=15 & jm=indgen(m+1) & xs=dblarr(m+1) & zs=xs
	bns=xs

	jm2=indgen(2*m+2) & xsd=dblarr(2*m+2) & zsd=xsd
	bts2=xsd & ysd=xsd

; ends of lines of force

	xlf1=xsd & xlf2=xsd & zlf1=xsd & zlf2=xsd 

; distance element along neutral line

	dsn=rh1/m
	
; initialize

	xs(0)=0.0 & zs(0)=0.0 & bns(0)=0.0

	ep=0.000001
	
	for i=1,m do begin
	iq=1

	theta1=0.0
	zs1=zs(i-1)+dsn*cos(theta1) 
	xs1=xs(i-1)+dsn*sin(theta1)
	bn1=bn(xs1,y0,zs1) 

	dth=!pi/18.
	
	    for j=1,36 do begin
	        theta2=theta1+j*dth

	        zs2=zs(i-1)+dsn*cos(theta2) 
	        xs2=xs(i-1)+dsn*sin(theta2)
	        bn2=bn(xs2,y0,zs2) 

	        if bn1*bn2 lt 0 then begin
;	        print,'i=',i,' theta2=',theta2*180./!pi
	        goto,label51
	        endif
	
	    endfor

label51:
	
            if iq gt 500 then begin
                print,' no convergence at iq =',iq,'  i=',i
                goto,label100
            endif else iq=iq+1 
        
            xs1=xs(i-1)+dsn*sin(theta1)
            bn1=bn(xs1,y0,zs1) 
        
            zs2=zs(i-1)+dsn*cos(theta2) 
            xs2=xs(i-1)+dsn*sin(theta2)
            bn2=bn(xs2,y0,zs2) 
        
            thetam=.5*(theta1 + theta2)
            zsm=zs(i-1)+dsn*cos(thetam)  
            xsm=xs(i-1)+dsn*sin(thetam)
            bnm=bn(xsm,y0,zsm)

	    if abs(bnm) lt ep then goto,label50

	    if bnm*bn1 gt 0 then theta1=thetam else theta2=thetam
	    goto,label51

label50:
	    zs(i)=zsm & xs(i)=xsm & bns(i)=bnm

;	    oplot,[zs(i-1),zs(i)],[xs(i-1),xs(i)],thick=2,color=red
;	    read,ia

; i sequence

	endfor
 
	zsd=[-reverse(zs),zs]
	xsd=[-reverse(xs),xs]
	bnsd=[-reverse(bns),bns]
	ysd=double(jm2*0.0+y0)

;	oplot, zsd, xsd, thick=2
;	oplot, zsd, xsd, thick=3, color=red

	oplot, zsd, xsd, thick=8, color=red

	if id eq 1 then begin
	    device, /close
	    set_plot, mydevice
	endif

	
	
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


label10:
	print,'enter za and then xa'
	read,za
	read,xa
label9:
	read, 'DO YOU want HARD COPY ? ', ans

        if (ans eq 'y') then begin
            set_plot,'ps'
            device, XSIZE=7, YSIZE=7, /inches, /portrait, $
		    /color, filename='fig78b.ps'
	    col1 = grey
	    col2 = yellow
	    erase, 0
	    id=1
	endif else begin
	    window, 2, xsize=500, ysize=500, retain=2
	    wset,2
	    col1 = white
	    col2 = yellow
	    erase, 0
	    id=0
	endelse


;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

; set up	

	scale3,xrange=[-x1,x1],yrange=[y0,y0+2.*y1], $
	       zrange=[-z1,z1],AX=xa, AZ=za

; the axes

	plots,[0,x1],[y0,y0],[0,0],/t3d,/data,color=red
	plots,[0,0],[y0,y0+2.*y1],[0,0],/t3d,/data,color=green
	plots,[0,0],[y0,y0],[0,z1],/t3d,/data,color=blue


	plots,[-x1,-x1,x1,x1,-x1],[y0,y0,y0,y0,y0],[-z1,z1,z1,-z1,-z1], $
            /t3d, /data,color=green,thick=2

	plots,[x1,x1],[y0,y0+2.*y1],[z1,z1],/t3d, /data,color=green
	plots,[-x1,-x1],[y0,y0+2.*y1],[z1,z1],/t3d, /data,color=green
	plots,[x1,x1],[y0,y0+2.*y1],[-z1,-z1],/t3d, /data,color=green
	plots,[-x1,-x1],[y0,y0+2.*y1],[-z1,-z1],/t3d, /data,color=green

;	plots,[-x1,-x1,x1,x1,-x1],[y0+2.*y1,y0+2.*y1, $
;	y0+2.*y1,y0+2.*y1,y0+2.*y1],[-z1,z1,z1,-z1,-z1], $
;            /t3d, /data,color=green

;	plots,[-x1,-x1,-x1,-x1,-x1],[y0,y0+2.*y1,y0+2.*y1,y0,y0], $
;		[-z1,-z1,z1,z1,-z1], $
;            /t3d, /data,color=red
;	plots,[x1,x1,x1,x1,x1],[y0,y0+2.*y1,y0+2.*y1,y0,y0], $
;		[-z1,-z1,z1,z1,-z1], $
;            /t3d, /data,color=red
;	plots,[-x1,-x1,x1,x1,-x1],[y0,y0+2.*y1,y0+2.*y1,y0,y0], $
;		[-z1,-z1,-z1,-z1,-z1], $
;            /t3d, /data,color=blue
;	plots,[-x1,-x1,x1,x1,-x1],[y0,y0+2.*y1,y0+2.*y1,y0,y0], $
;		[z1,z1,z1,z1,z1], $
;            /t3d, /data,color=blue
	
;	polyfill,[-x1,-x1,x1,x1,-x1],[y0,y0,y0,y0,y0],[-z1,z1,z1,-z1,-z1], $
;           /t3d,/data,color=white

      xyouts,0,y0,Z=1.1*z1,'Z',/t3d,/data
      xyouts,1.1*x1,y0,Z=0,'X',/t3d,/data
      xyouts,0,1.1*(y0+2.*y1),Z=0,'Y',/t3d,/data

	plots,xsd,ysd,zsd,thick=3,color=red,/t3d,/data

;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

; plot the flux rope axis
	
	dz=rh0/n

	zz1=0.0 & yy1=arm & xx1=0.0 
	phi1=!pi*.5 & zeta1=phi1-zz1

	for j=1,n do begin
	zz2=zz1+dz & phi2=zeta1+zz2
	xx2=arm*cos(phi2) & yy2=arm*sin(phi2)
	if yy2 gt y0 then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],/t3d,/data,color=magenta,thick=2
	xx1=xx2 & yy1=yy2 & zz1=zz2 & phi1=phi2
	goto,label70
	endif else goto,label71
label70:
	endfor
label71:
	dz=-dz
	zz1=0.0 & yy1=arm & xx1=0.0 
	phi1=!pi*.5 & zeta1=phi1-zz1

	for j=1,n do begin
	zz2=zz1+dz & phi2=zeta1+zz2
	xx2=arm*cos(phi2) & yy2=arm*sin(phi2)
	if yy2 gt y0 then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],/t3d,/data,color=magenta,thick=2
	xx1=xx2 & yy1=yy2 & zz1=zz2 & phi1=phi2
	goto,label72
	endif else goto,label73
label72:
	endfor
label73:
	
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

;	goto,label99

; plot the field lines


    	for jj=0,2*m-1 do begin

; integration in the plus s direction
		
	zz1=zsd(jj) & xx1=xsd(jj) & yy1=ysd(jj) & rr1=sqrt(xx1^2 + yy1^2)
	
	theta1=asin(abs(yy1/rr1))

	if yy1 eq 0 then begin
	if xx1 lt 0 then theta1=!pi else theta1=0.
	endif else begin
	if yy1 gt 0 then begin
	if xx1 lt 0 then theta1=!pi-theta1
	endif else begin
	if xx1 lt 0 then theta1=-!pi+theta1 else theta1=-theta1
	endelse	
	endelse

	frs=fpsi(rr1)*sin(theta1-k*zz1)

	rpz1(0)=rr1 & rpz1(1)=theta1 & rpz1(2)=zz1 & s=0.0
	drpz1=derivs(s,rpz1)
	drpz2=drpz1 & rpz2=rpz1
	h=rh1/100.

;	print,' dy=',drpz1(0)*sin(theta1)+drpz1(1)*xx1
;	print,' bn=',bn(xx1,yy1,zz1)

label1:
	
	for js=1,10 do begin

	rpz2=rk4(rpz2,drpz2,s,0.1*h,'derivs',/double)
	s=s+0.1*h
	drpz2=derivs(s,rpz2)
	endfor

	zz2=rpz2(2) & xx2=rpz2(0)*cos(rpz2(1)) & yy2=rpz2(0)*sin(rpz2(1))

	rlimit=sqrt(xx1^2+zz1^2)

	if (yy2 gt y0) and (rlimit lt 1.5*rh1) then begin
	plots, [xx1, xx2], [yy1, yy2], [zz1, zz2], /data, /t3d, color=col2
;	plots, [xx1, xx2], [yy1, yy2], [zz1, zz2], /data, /t3d, color=yellow

	xx1=xx2 & yy1=yy2 & zz1=zz2
	rpz1=rpz2 & drpz1=drpz2
	goto, label1
	endif else begin
	print,'end of  plus-end line for jj=',jj

;	xlf1(jj)=xx2 & zlf1(jj)=zz2
	
	if yy2 lt y0 then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=blue
	endif else begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=green
	endelse
	endelse 
	
; integration in the minus s direction		

	zz1=zsd(jj) & xx1=xsd(jj) & yy1=ysd(jj) & rr1=sqrt(xx1^2 + yy1^2)

	theta1=asin(abs(yy1/rr1))

	if yy1 eq 0 then begin
	if xx1 lt 0 then theta1=!pi else theta1=0.
	endif else begin
	if yy1 gt 0 then begin
	if xx1 lt 0 then theta1=!pi-theta1
	endif else begin
	if xx1 lt 0 then theta1=-!pi+theta1 else theta1=-theta1
	endelse	
	endelse
		
;	read,ia
	
	rpz1(0)=rr1 & rpz1(1)=theta1 & rpz1(2)=zz1 & s=0.0
	drpz1=derivs(s,rpz1)
	drpz2=drpz1 & rpz2=rpz1
	h=rh1/100.

label2:
	
	for js=1,10 do begin
	rpz2=rk4(rpz2,drpz2,s,-0.1*h,'derivs',/double)
	s=s-0.1*h
	drpz2=derivs(s,rpz2)
	endfor
	
	zz2=rpz2(2) & xx2=rpz2(0)*cos(rpz2(1)) & yy2=rpz2(0)*sin(rpz2(1))

	rlimit=sqrt(xx1^2+zz1^2)

	if (yy2 gt y0) and (rlimit lt 1.5*rh1) then begin
	plots, [xx1, xx2], [yy1, yy2], [zz1, zz2], /data, /t3d, color=col2
;	plots, [xx1, xx2], [yy1, yy2], [zz1, zz2], /data, /t3d, color=yellow
	xx1=xx2 & yy1=yy2 & zz1=zz2
	rpz1=rpz2 & drpz1=drpz2
	goto, label2
	endif else begin
	print,'end of negative-end line jj=',jj
;	xlf2(jj)=xx2 & zlf2(jj)=zz2

	if yy2 lt y0 then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=blue
	endif else begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=green
	endelse
	endelse

;	read,ia
	wait,1
	endfor
; jj loop

	plots,xsd,ysd,zsd,thick=3,color=red,/t3d,/data
;	plots,xlf1,ysd,zlf1,thick=3,color=blue,/t3d,/data
;	plots,xlf2,ysd,zlf2,thick=3,color=blue,/t3d,/data


label99:
	
;%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
; plotting the internal flux rope

	np=1. & gamma=1.2 & k=1. & rin=3.16895 & rou=6.82227
	frs=-0.9 & rh0=8. & rmm=5.0

	ms=4 & rs=dblarr(2*ms) & thetas=rs

	rs(0)=rin & thetas(0)=!pi*.5

	drr=(rou-rin)/ms

	for ii=1,ms-1 do begin
	rs(2*ii)=rin+ii*drr & rs(2*ii-1)=rs(2*ii)
	thetas(2*ii)=asin(frs/fpsi(rs(2*ii))) 
	thetas(2*ii-1)=!pi-thetas(2*ii)
	endfor

	rs(2*ms-1)=rou & thetas(2*ms-1)=.5*!pi


	print,' flux-rope field lines'
	for ii=0,2*ms-1 do begin

; integration in the plus s direction
		
	zz1=0.0 & xx1=rs(ii)*cos(thetas(ii)) & yy1=rs(ii)*sin(thetas(ii))
	
	rpz1(0)=rs(ii) & rpz1(1)=thetas(ii) & rpz1(2)=zz1 & s=0.0
	drpz1=derivs(s,rpz1)
	drpz2=drpz1 & rpz2=rpz1
	h=rh1/100.

label5:
	for js=1,10 do begin

	rpz2=rk4(rpz2,drpz2,s,0.1*h,'derivs',/double)
	s=s+0.1*h
	drpz2=derivs(s,rpz2)
	endfor

	zz2=rpz2(2) & xx2=rpz2(0)*cos(rpz2(1)) & yy2=rpz2(0)*sin(rpz2(1))

	rlimit=sqrt(xx1^2+zz1^2)

	if (yy2 gt y0) and (rlimit lt 1.5*rh1) then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],/data,/t3d,color=col1
;	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],/data,/t3d,color=white

	xx1=xx2 & yy1=yy2 & zz1=zz2
	rpz1=rpz2 & drpz1=drpz2
	goto, label5
	endif else begin
	print,'end of  plus-end line for ii=',ii
;	xlf1(ii)=xx2 & zlf1(ii)=zz2
	
	if yy2 lt y0 then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=blue
	endif else begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=green
	endelse
	endelse 
	
; integration in the minus s direction		
	
	zz1=0.0 & xx1=rs(ii)*cos(thetas(ii)) & yy1=rs(ii)*sin(thetas(ii))
	
	rpz1(0)=rs(ii) & rpz1(1)=thetas(ii) & rpz1(2)=zz1 & s=0.0
	
	drpz1=derivs(s,rpz1)
	drpz2=drpz1 & rpz2=rpz1
	h=rh1/100.

label6:
	for js=1,10 do begin
	rpz2=rk4(rpz2,drpz2,s,-0.1*h,'derivs',/double)
	s=s-0.1*h
	drpz2=derivs(s,rpz2)
	endfor
	
	zz2=rpz2(2) & xx2=rpz2(0)*cos(rpz2(1)) & yy2=rpz2(0)*sin(rpz2(1))

	rlimit=sqrt(xx1^2+zz1^2)

	if (yy2 gt y0) and (rlimit lt 1.5*rh1) then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],/data,/t3d,color=col1
;	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],/data,/t3d,color=white

	xx1=xx2 & yy1=yy2 & zz1=zz2
	rpz1=rpz2 & drpz1=drpz2
	goto, label6
	endif else begin
	print,'end of negative-end line ii=',ii

;	xlf2(ii)=xx2 & zlf2(ii)=zz2

	if yy2 lt y0 then begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=blue
	endif else begin
	plots,[xx1,xx2],[yy1,yy2],[zz1,zz2],thick=6,/data,/t3d,color=green
	endelse
	endelse 

	endfor

; ii loop

	if id eq 1 then begin
	    device, /close
	    set_plot, mydevice
	endif
	
	print, 'Enter 1 to plot 3D fields and 0 to quit : '
	read, ia
	if ia eq 1 then goto, label10

label100:
	END



      

