;+
; flipfits.pro   IDL procedure
;
; PURPOSE
;	Read a FITS image and change the scan direction. 
;
; SYNTAX
;	flipfits, 'fits_list', 'hor'
;	flipfits, 'fits_list', 'ver'
;
;		flip direction: hor (horizontal)
;		flip direction: ver (vertical  )
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   16 October 1997
;-

pro flipfits, fits_list, flip_dir

fits_name = ''				; Define FITS file name variable.

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')
      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')

      dir = strmid (flip_dir, 0, 3)
      dir = STRUPCASE (dir)
      if (dir EQ 'HOR') THEN		$
	 BEGIN	
	 img2 = REVERSE (img, 1)
	 xcen = xdim - 1 - xcen	
         fxaddpar, hdu, 'CRPIX1', xcen
	 END				$
      ELSE				$
	 BEGIN
	 img2 = REVERSE (img, 2)
	 ycen = ydim - 1 - ycen	
         fxaddpar, hdu, 'CRPIX2', ycen
	 END

      ; -- Modify FITS header.

      fxaddpar, hdu, 'DISP-DIR', 'RU', 'X: LEFT-> RIGHT, Y: BOTTOM->TOP'

      ; -- Write FITS image to disk.

      writefits, fits_name, img2, hdu

   END
CLOSE, 1

 END
