;+
; fits3img.pro   IDL procedure
;
; PURPOSE
;	Read 3 FITS images and create a column composite output image.
;
; SYNTAX
;	fits3img, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   2 December 1997
;-

pro fits3img, fits_list

loadct, 0

red   = indgen (256)
green = indgen (256)
blue  = indgen (256)

fits_name = ''				; Define FITS file name variable.
xdim = 0
ydim = 0
xcen = 0
ycen = 0
roll = 0.0
mag  = 1.0

CLOSE, 1
CLOSE, 2

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

i = 0
img = intarr (1024,1024,3)

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img (i) = readfits (fits_name, hdu)	; Read FITS image & header.

      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')
      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')
      pixrs = fxpar (hdu, 'RADIUS')
      pixrs = fxpar (hdu, 'CRRADIUS')

;      xcen = (xdim / 2) - 1
;      ycen = (ydim / 2) - 1

   END
CLOSE, 1

      imgout = intarr (1024, 3072) ;

      imgout (*, 0   :1023) = img (0) ;
      imgout (*, 1024:2047) = img (1) ;
      imgout (*, 2048:3071) = img (2) ;

      ; -- Write FITS image to disk.

      sloc = RSTRPOS (fits_name, '.fts')
      tiff_name = strmid (fits_name, 0, sloc) + '.tiff'	; Create output file name.
      print, 'rfits_name: ', rfits_name
      tiff_write, tiff_name, imgbuf, red, green, blue


END
