;+
; NAME
;	make_gamma
;
; PURPOSE
;	Create a gamma intensity look-up table
;	by computing a gamma tranform function
;	as defined by the IDL procedure 'xloadct'.
;
; CALLING SEQUENCE
;	gamtab = make_gamma (gammaval)
;
; ARGUMENTS
;	gamtab		gamma table
;	gammaval	gamma value (< 1 to brighten, > 1 to darken)
;
; HISTORY
;	16 August 1995, created by Andrew L. Stanger, HAO/NCAR
;-

FUNCTION make_gamma, gammaval

;window, xsize = 512, ysize = 512, color=256

chop = 0
cbot = 0
vbot = 0
vtop = 100
gamma = 1.0
nc = 256
ncolors = 256
maxcolor = ncolors - 1

;r_orig = indgen (nc)
;g_orig = indgen (nc)
;b_orig = indgen (nc)

gamtab = bytarr (nc)

gamma = 1.0 / gammaval

    s = (nc - 1) / 100.0			; =   2.55
    x0 = vbot * s				; =   0
    x1 = vtop * s				; = 100
    if (x0 ne x1) then s = (nc - 1.0) / (x1 - x0) else s = 1.0
    int = -s * x0
    if (gamma eq 1.0) then gamtab = round (findgen (nc) * s + int > 0.0) $
    else gamtab = ((findgen (nc) * (s / nc) + (int / nc) > 0.0) ^ gamma) * nc

;    plot, gamtab

    if (chop ne 0) then begin
        too_high = where (gamtab ge nc, maxcolor)
        if (maxcolor gt 0) then gamtab (too_high) = 0L
    endif

    gamtab = gamtab < 255L
;    l = lindgen (ncolors) + cbot
;    r = gamtab (r_orig (l))
;    g = gamtab (g_orig (l))
;    b = gamtab (b_orig (l))

;    tvlct, r, g, b, cbot

;    img = bytarr (512, 512)
;    for i = 0, 511 do img (i, *) = i / 2
;    tv, img

    return, gamtab
end
