;+
; mtkcen.pro   IDL procedure
;
; PURPOSE
;	Read a MTK FITS image, find sun center, update FITS header,
;       and write FITS file with new name (containing date & time).
;
; SYNTAX
;	mtkcen, 'fits_list'
;
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   11 June 1998
;-

pro mtkcen, fits_list

fits_name = ''				; Define FITS file name variable.
new_name = ''
centroid = [0.0, 0.0]
hdiam = 0.0
vdiam = 0.0

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

;--- File Loop :

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img  = rfits (fits_name, header=hdu)	; Read FITS image & header.
      imin = MIN (img, max=imax)	; Find min/max pixel intensity range.
      print, 'image min/max: ', imin, imax
      imgb = bytscl (img, min=imin, max=imax)
      tv, imgb

      ;--- Get keyword parameters from FITS header.

      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      observat = fxpar (hdu, "OBSERVAT")
      date     = fxpar (hdu, "DATE_OBS")
      time     = fxpar (hdu, "TIME1")
      print, 'date: ', date, '   time: ', time

;      xcen = fxpar (hdu, 'CRPIX1')
;      ycen = fxpar (hdu, 'CRPIX2')

      ;--- Find sun center & radius.

;      gen_loc, imgb, /cir, CENTROID=centroid, HORZRAD=hdiam, VERTRAD=vdiam

      ;--- Update FITS header.

;      fxaddpar, hdu, 'CRPIX1',  centroid [0], ' X Sun Center [pixels]'
;      fxaddpar, hdu, 'CRPIX2',  centroid [1], ' Y Sun Center [pixels]'
;      fxaddpar, hdu, 'CRRADIUS', hdiam / 2.0, ' Pixels per solar radius'
;      fxaddpar, hdu, 'HISTORY', $
;		'CRPIX1, CRPIX2, CRRADIUS DETERMINED BY MTKCEN.PRO'

      year   = strmid (date, 0, 2)
      month  = strmid (date, 3, 2)
      day    = strmid (date, 6, 2)

      hour   = strmid (time, 0, 2)
      minute = strmid (time, 3, 2)
      second = strmid (time, 6, 2)

;      new_name = STRLOWCASE (observat) + '.' + year + month + day + '.' $
;		 + hour + minute + second + '.fts'
      new_name = STRLOWCASE (observat) + '.' + fits_name

      ; -- Write FITS image to disk.

      writefits, new_name, imgb, hdu

   END
CLOSE, 1

 END
