;+
; mtkfix.pro   IDL procedure
;
; PURPOSE
;	Read a MTK FITS image, fix string variables, update FITS header,
;       and write FITS file with new name (containing date & time).
;
; SYNTAX
;	mtkfix, 'fits_list'
;
; PROCEDURES USED
;	rfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   11 June 1998
;-

pro mtkfix, fits_list

fits_name = ''				; Define FITS file name variable.
new_name = ''
centroid = [0.0, 0.0]
hdiam = 0.0
vdiam = 0.0
b10  = '          '
b11  = '           '
b13  = '             '
b16  = '                '
bcom = '/                                                 '; start col 31

CLOSE, 1

;PRINT, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

;--- File Loop :

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img  = rfits (fits_name, header=hdu)	; Read FITS image & header.

;      PRINT, 'size (hdu): ', size (hdu)
      FOR i = 0, 35 DO	$
         BEGIN
	    keyword   = STRMID (hdu [i],  0,  8)
	    param     = STRMID (hdu [i],  9, 21)
	    comment   = STRMID (hdu [i], 31, 49)
;	    PRINT, 'index:   ', i
;	    PRINT, 'keyword: ', keyword
;	    PRINT, 'param:   ', param
;	    PRINT, 'comment: ', comment

	    IF (STRTRIM (keyword, 2) EQ 'DATE_OBS') THEN $
	       hdu [i] = "DATE_OBS= '" + STRTRIM (param, 2) + "'" + b11 + bcom
            IF (STRTRIM (keyword, 2) EQ 'TIME1')    THEN $
	       hdu [i] = "TIME1   = '" + STRTRIM (param, 2) + "'" + b11 + bcom
            IF (STRTRIM (keyword, 2) EQ 'TIME2')    THEN $
	       hdu [i] = "TIME2   = '" + STRTRIM (param, 2) + "'" + b11 + bcom
	    IF (STRTRIM (keyword, 2) EQ 'OBSERVAT') THEN $
	       hdu [i] = "OBSERVAT= '" + STRTRIM (param, 2) + "'" + b16 + bcom
	    IF (STRTRIM (keyword, 2) EQ 'TELESCOP') THEN $
	       hdu [i] = "TELESCOP= '" + STRTRIM (param, 2) + "'" + b16 + bcom
	    IF (STRTRIM (keyword, 2) EQ 'DATA_TYP') THEN $
	       hdu [i] = "DATA_TYP= '" + STRTRIM (param, 2) + "'" + b13 + bcom
	    IF (STRTRIM (keyword, 2) EQ 'BSCALE')   THEN $
	       hdu [i] = "BSCALE  =                  1.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'BZERO')    THEN $
	       hdu [i] = "BZERO   =                  0.0 " + bcom 
	    IF (STRTRIM (keyword, 2) EQ 'CRVAL1') THEN $
	       hdu [i] = "CRVAL1  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CRVAL2') THEN $
	       hdu [i] = "CRVAL2  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CRVAL3') THEN $
	       hdu [i] = "CRVAL3  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CRPIX1') THEN $
	       hdu [i] = "CRPIX1  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CRPIX2') THEN $
	       hdu [i] = "CRPIX2  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CRPIX3') THEN $
	       hdu [i] = "CRPIX3  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CDELT1') THEN $
	       hdu [i] = "CRDELT1 =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CDELT2') THEN $
	       hdu [i] = "CRDELT2 =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CDELT3') THEN $
	       hdu [i] = "CRDELT3 =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CTYPE1') THEN $
	       hdu [i] = "CTYPE1  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CTYPE2') THEN $
	       hdu [i] = "CTYPE2  =                  0.0 " + bcom
	    IF (STRTRIM (keyword, 2) EQ 'CTYPE3') THEN $
	       hdu [i] = "CTYPE3  =                  0.0 " + bcom
         END

      imin = MIN (img, max=imax)	; Find min/max pixel intensity range.
;      PRINT, 'image min/max: ', imin, imax
      imgb = BYTSCL (img, min=imin, max=imax)
;      TV, imgb

      ;--- Get keyword parameters from FITS header.

      xdim     = fxpar (hdu, 'NAXIS1')
      ydim     = fxpar (hdu, 'NAXIS2')
      observat = fxpar (hdu, "OBSERVAT")
      date     = fxpar (hdu, "DATE_OBS")
      time     = fxpar (hdu, "TIME1")
;      PRINT, 'date: ', date, '   time: ', time

      fxaddpar, hdu, "DATE-OBS", date, 'OBSERVATION DATE: [YY/MM/DD]"
      fxaddpar, hdu, "TIME-OBS", time, 'OBSERVATION TIME: [HH:MM:SS]"
      fxaddpar, hdu, "TYPE-OBS", 'H-ALPHA', ''

;      xcen = fxpar (hdu, 'CRPIX1')
;      ycen = fxpar (hdu, 'CRPIX2')

      ;--- Find sun center & radius.

      gen_loc, imgb, /cir, CENTROID=centroid, HORZRAD=hdiam, VERTRAD=vdiam

      ;--- Update FITS header.

      fxaddpar, hdu, 'CRPIX1',  centroid [0], ' X Sun Center [pixels]'
      fxaddpar, hdu, 'CRPIX2',  centroid [1], ' Y Sun Center [pixels]'
      fxaddpar, hdu, 'CRRADIUS', hdiam / 2.0, ' Pixels per solar radius'
      fxaddpar, hdu, 'HISTORY', $
		'CRPIX1, CRPIX2, CRRADIUS DETERMINED BY MTKCEN.PRO'

      year   = STRMID (date, 0, 2)
      month  = STRMID (date, 3, 2)
      day    = STRMID (date, 6, 2)

      hour   = STRMID (time, 0, 2)
      minute = STRMID (time, 3, 2)
      second = STRMID (time, 6, 2)

      new_name = STRLOWCASE (observat) + '.' + year + month + day + '.' $
		 + hour + minute + second + '.fts'
;      new_name = STRLOWCASE (STRTRIM (observat, 2)) + '.' + fits_name
      new_name = STRTRIM (new_name, 2)
      PRINT, 'new_name: ', new_name
;      PRINT, hdu

      ; -- Write FITS image to disk.

      writefits, new_name, imgb, hdu

   END
CLOSE, 1

 END
