;+
; pmftsname.pro   IDL procedure
;
; PURPOSE
;	Read a FITS image and change the file name. 
;
; SYNTAX
;	pmftsname, 'img.fts'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   10 October 1997
;-

pro pmftsname, fits_list

fits_name = ''				; Define FITS file name variable.

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      date   = fxpar (hdu, 'DATE-OBS')
      time   = fxpar (hdu, 'TIME-OBS')
      day    = strmid (date, 0, 2)
      month  = strmid (date, 3, 2)
      year   = strmid (date, 6, 2)
      hour   = strmid (time, 0, 2)
      minute = strmid (time, 3, 2)
      second = strmid (time, 6, 2)
      new_fits_name = 'pm.' + year + month + day + '.' + $
			      hour + minute + second + '.fts'
      fxaddpar, hdu, 'DATE-OBS', date, 'DAY/MONTH/YEAR'
      fxaddpar, hdu, 'TIME-OBS', time, 'HOUR:MINUTE:SECOND'

      ; -- Write FITS image to disk.

      writefits, new_fits_name, img, hdu

   END
CLOSE, 1

END
