;+
; IDL PROCEDURE
;
; NAME:
;	srfgam
;
; PURPOSE:
;	Apply a gamma function to a Sun rasterfile image.
;
; CALLING SEQUENCE:
;	srfgam, 'image.srf', gammaval
;	srfgam, 'image.srf', gammaval, /order
;	srfgam, 'image.srf', gammaval, /order, /label
;
; INPUTS:
;	image.srf:	Input Sun Rasterfile image filename
;	gammaval:	gamma function value
;
; OUTPUTS:
;	image.srf.gam:	Output Sun Rasterfile image filename
;
; PROCEDURES USED:
;	READ_SRF
;	WRITE_SRF
;	GAMTRAN
;
; MODIFICATION HISTORY:
;	14 Sep 1995: procedure created by Andrew L. Stanger, HAO/NCAR
;-

pro srfgam, inp_file, gammaval, label=label, order=order

red   = bindgen (256)
green = bindgen (256)
blue  = bindgen (256)

out_file = 'img.srf.gam'

      out_file = inp_file + '.gam'

      ;--- Read image into array & scale intensity levels to range: 0-255.

      read_srf, inp_file, imgbuf, red, green, blue ; Read first image in list.

      sizimg = size (imbuf)
      nbitpx = 8
      naxis  = sizimg (0)
      nxpix  = sizimg (1)
      nypix  = sizimg (2)

      if (nbitpx eq   8)  then imgbuf = byte   (imgbuf) else $
                               imgbuf = bytscl (imgbuf)

      ;--- If order keyword is set, reverse image top to bottom.

      if (keyword_set (order))  then imgbuf = reverse (imgbuf, 2)

;      loadct, 0
;      window, 0, xsize = nxpix, ysize = nypix
;      wset, 0
;      tv, imgbuf

      imgout = gamtran (imgbuf, gammaval)		; Transform image.

      ;--- Label image if label keyword is set.

      if (keyword_set (label))  then $
      begin
         window, 20, xsize = nxpix, ysize = nypix, color=256, /pixmap
         wset, 20

         xpos = 10
         ypos = nypix - 20
         sframe = string (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         tv, imgout
         imglabel = sframe + '   ' + inp_file
	 gamlabel = 'gamma: ' + string (gammaval, FORMAT = "(F7.3)")
         xyouts, xpos, ypos, imglabel, /device
	 xyouts, xpos, ypos - 20, gamlabel, /device
         imgout = tvrd ()
	 wdelete, 20
      end

      write_srf, out_file, imgout, red, green, blue

end 
