;+
; IDL PROCEDURE
;
; NAME:
;	xinjpg
;
; PURPOSE:
;	Animate a sequence of JPEG images.
;
; CALLING SEQUENCE:
;	xinjpg, 'jpg.ls'
;	xinjpg, 'jpg.ls', rate=25
;	xinjpg, 'jpg.ls', rate=8, /label
;	xinjpg, 'jpg.ls', rate=2, /label, cm='~/color/sunset.lut'
;
; INPUTS:
;	listfile : character string containing the names of the SUN rasterfile
;		   images to be animated.
;	rate : animation rate (range: 0-100, default=50).
;	label: if present, label each image with frame #, date, time.
;	cm: name of color map file (ASCII, each line: index, red, green, blue).
;
; OUTPUTS:
;	none
;
; PROCEDURES USED:
;	READ_GIF
;	XINTERANIMATE
;	LCT
;
; MODIFICATION HISTORY:
;	created 12 June 1996, Andrew L. Stanger, HAO/NCAR
;	13 July 2000: Adapted for JPEG format.
;-

PRO xinjpg, listfile, rate=rate, label=label, cm=cm

IF (not KEYWORD_SET (rate)) THEN rate = 100 

filename = ' '
idate    = ' '
itime    = ' '
ii = 0
winid = 20

;red   = BYTARR (256)
;green = BYTARR (256)
;blue  = BYTARR (256)

; --- Open list file, read filenames, count the number of files in list.

CLOSE, 10
OPENR, 10, listfile

WHILE (NOT EOF (10))  DO BEGIN	;{
   READF, 10, filename
   ii = ii + 1
ENDWHILE			;}
numimg = ii

CLOSE, 10

;--- Get size of last GIF image in list.

;read_jpeg, filename, imgbuf, red, green, blue	; Read last image in listfile.
read_jpeg, filename, imgbuf			; Read last image in listfile.

sizimg = SIZE (imgbuf)


IF (sizimg (0) NE 2) THEN	$
BEGIN
   PRINT, 'size (imgbuf): ', sizimg
   PRINT, '# dimensions: ', sizimg (0)
   PRINT, 'xinjpg only works currently with 8-bit/pixel images.'
   RETURN
END

nxpix  = sizimg (1)
nypix  = sizimg (2)
nbypix = sizimg (3)

;--- If label keyword set, create IDL window in which to display each image.

if (KEYWORD_SET (label))  then	$
BEGIN
   WINDOW, winid, xsize = nxpix, ysize = nypix, color=256, /pixmap
   WSET, winid
END

;--- Load color table.

;PRINT, 'SIZE (red): ', SIZE (red)
sizred = SIZE (red)
IF (KEYWORD_SET (cm))  THEN lct, cm	$	; Load specified colormap.
ELSE IF (sizred (0) NE 0) THEN tvlct, red, green, blue	$
ELSE			    LOADCT, 0		; Load greyscale colormap.
 
;--- Initialize 'xinteranimate'.

init = 'y'    ; --- init can be set to 'n'  once everything
	      ; --- has been initialized,  
	      ; --- i.e. once this routine has been run once... 

IF ( init EQ 'y') THEN BEGIN ;{ 
   XINTERANIMATE, SET = [nxpix+4, nypix+4, numimg], /SHOWLOAD
 
   CLOSE, 10 & OPENR, 10, listfile	; Open file containing list of images.
 
   ii = 0 
   WHILE ( NOT EOF (10) ) DO BEGIN ;{
      READF, 10, filename

;      read_jpeg, filename, imgbuf, red, green, blue	; Read image from list.
      read_jpeg, filename, imgbuf			; Read image from list.

      sizimg = SIZE (imgbuf)
      nxpix  = sizimg (1)
      nypix  = sizimg (2)
      nbypix = sizimg (3)

      IF (nbypix EQ  1)  THEN imgbuf = BYTE ( imgbuf) ELSE $
                              imgbuf = BYTSCL (imgbuf)		; Scale data.

      IF (KEYWORD_SET (label))  THEN $
      BEGIN
         xpos = 10
         ypos = nypix - 20
         sframe = STRING (ii, FORMAT = "(I3)")
	 sframe = 'FRAME ' + sframe
         TV, imgbuf
         imglabel = sframe + '     ' + filename
	 PRINT, 'imglabel: ', imglabel
         XYOUTS, xpos, ypos, imglabel, /device
         XINTERANIMATE, FRAME = ii, WINDOW = !d.window		; Load image
      END				$
      ELSE				$
	 XINTERANIMATE, IMAGE = imgbuf, FRAME = ii		; Load image

      ii = ii+1
   ENDWHILE ;} 
 
   CLOSE, 10

ENDIF ;}

;WDELETE, winid					; Delete temp window.

;--- Activate animation.

XINTERANIMATE, rate 

; --- When you want to stop animating,  hit ^C. 
; --- You might then have to do a "retall" in IDL.
 
END 
