;+
; NAME	brtvalpc
;
; PURPOSE	Find the brightest pixels (top brtpct%) in a polar coord image.
;
; SYNTAX	pixloc = brtvalpc (img, pixrs, brtpct,
;                                  ixbeg, ixend, iybeg, iyend)
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   6 June 2000
;
; HISTORY	22 Sep 2000: Modified to work for polar coordinate images.
;		
;-

FUNCTION brtvalpc, imgbuf, pixrs, brtpct, ixbeg, ixend, iybeg, iyend

sizeimg = size (imgbuf)
PRINT, 'brtvalpc> size (imgbuf): ', size (imgbuf)

xdim = sizeimg [1]
ydim = sizeimg [2]
npximg = xdim * ydim

xcen = (npximg / 2 - 0.5)
ycen = (npximg / 2 - 0.5)

ixcen = xdim / 2
iycen = ydim / 2

;--- Extract a "box" from the image.

ixdif = ixend - ixbeg + 1
iydif = iyend - iybeg + 1

xdimbox = long (ixend - ixbeg + 1)
ydimbox = long (iyend - iybeg + 1)
npxbox  = xdimbox * ydimbox

PRINT, 'ixbeg/ixend/ixdif: ', ixbeg, ixend, ixdif
PRINT, 'iybeg/iyend/iydif: ', iybeg, iyend, iydif
PRINT, 'xdimbox/ydimbox: ', xdimbox, ydimbox
PRINT, 'npxbox: ', npxbox

box = FLTARR (xdimbox, ydimbox)

box = imgbuf [ixbeg:ixend, iybeg:iyend]

boxsort = box (SORT (box))        ; order w.r.t. increasing brightness.

;--- Compute the "brtpct % median":
;    the level at which 20% of the pixels lie above the computed brightness.

   IF (KEYWORD_SET (brtpct)) THEN	$
      frac = 1.0 - (brtpct / 100.0)	$
   ELSE				$
      frac = 0.8
   sfrac = strtrim (string (fix ((1.0 - frac) * 100.00001),		$
		    FORMAT='(I4)'), 2)

   nbrtval = long (frac * npxbox)	; Fraction of brightest pixels to sample
   ibrtval = npxbox - nbrtval
   brtval  = boxsort (ibrtval)		; Brightness level threshold.
   brtloc  = WHERE (box GE brtval)	; Indices corresponding to bright pixels
   brtlocsize = SIZE (brtloc)
   IF (brtlocsize [0] GT 0) THEN		$
      nloc = long (brtlocsize [1])		$
   ELSE					$
      nloc = 0L

   PRINT, 'nbrtval: ', nbrtval
   PRINT, 'ibrtval: ', ibrtval
   PRINT, 'brtval:  ', brtval

RETURN, brtval
END
