;+
; NAME		cenbrt.pro
;
; PURPOSE	Determine the average disk center brightness of a solar image..
;
; MEHTOD	Extract a box located a disk center.
;		Order the subset array according to brightness.
;		Average over the brightest 10% of the pixels in this box.
;	
; SYNTAX	avgbrt = cenbrt (img)
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   5 June 2000
;
; HISTORY	 3 Oct 2000: Increase box size from 40 to 120.
;		 5 Oct 2000: Increase box size to 240
; 
;-

FUNCTION cenbrt, imgbuf

print, '******* cenbrt *******'

sizeimg = SIZE (imgbuf)

xdim = sizeimg [1]
ydim = sizeimg [2]

xcen = (xdim / 2 - 0.5)
ycen = (ydim / 2 - 0.5)
ixcen = xdim / 2
iycen = ydim / 2

print, 'xdim/ydim:   ', xdim, ydim
print, 'ixcen/iycen: ', ixcen, iycen

;--- Extract a box at sun center.

xdimcen = 240
ydimcen = 240
npxcen  = LONG (xdimcen) * LONG (ydimcen)
ixbeg   = ixcen - xdimcen / 2
iybeg   = iycen - ydimcen / 2
ixend   = ixbeg + xdimcen - 1
iyend   = iybeg + ydimcen - 1

print, 'xdimcen/ydimcen: ', xdimcen, ydimcen
print, 'npxcen: ', npxcen
print, 'ixbeg/ixend: ', ixbeg, ixend
print, 'iybeg/iyend: ', iybeg, iyend
print, 'ixend - ixbeg: ', ixend - ixbeg
print, 'iyend - iybeg: ', iyend - iybeg

imgcen = FLTARR (xdimcen, ydimcen)
imgcen = imgbuf [ixbeg:ixend, iybeg:iyend]

;--- Order the array according to increasing intensity.

censort = imgcen (SORT (imgcen))

;--- Compute the "10% median": level at which 10% of the pixels lie above
;--- the computed level. 
;--- Then compute the average of these brightest pixels.

nbrt90 = LONG (0.9 * npxcen)
print, 'npxcen: ', npxcen, '   nbrt90: ', nbrt90
brtcen = TOTAL (censort [nbrt90:npxcen-1]) / (npxcen - nbrt90)

RETURN, brtcen
END
