;+
; NAME		cencir.pro
;
; PURPOSE	Find sun center of a list of FITS images.
;		Update CRPIX1 & CRPIX2 parameters in FITS header.
;
; SYNTAX	cencir, fits_list
;
;		fits_list	file containing a list of FITS images
;
; AUTHOR	Guiliana DeToma		29 March 2001	[center.pro]
;
; HISTORY	Adapted from "center.pro" by Andrew L. Stanger   30 March 2001
;
;-

PRO cencir, fits_list

fits_name = ' '				

CLOSE, 1
CLOSE, 2
OPENR, 1, fits_list			
OPENW, 2, 'center.dat'

format='(i4, i4, i4, f8.2, 2x, 2f8.2, 1x, 3f8.2, 1x, 3f8.2, 1x, 2f8.2 )'

;--- File processing loop.

i = 0

WHILE  (NOT EOF (1)) DO	$		
    BEGIN 
    READF, 1, fits_name		
    img = readfits (fits_name, hdu, /noscale)	
    hsk = readfits (fits_name, ext, exten_no=1, /noscale)
   
    ;--- Get information from FITS header.

    xdim     = fxpar (hdu, 'NAXIS1')
    ydim     = fxpar (hdu, 'NAXIS2')
    xcen     = fxpar (hdu, 'CRPIX1')
    ycen     = fxpar (hdu, 'CRPIX2')
    crradius = fxpar (hdu, 'CRRADIUS')
    time     = fxpar (hdu, 'TIME-OBS')

    ;--- Get time from TIME-OBS parameter.

    hour = float ( strmid (time, 0, 2) )
    min  = float ( strmid (time, 3, 2) )
    sec  = float ( strmid (time, 6, 2) )
      
;    ft = hour * 60.0 + min + sec / 60.0

;    xshift = FIX ( (1534 - xdim) * 0.5 )
;    yshift = FIX ( (1030 - ydim) * 0.5 )

     
    find_center, img, minint, xc, yc, r

    IF (xdim GT 1200 OR ydim GT 1000) THEN	$
	BEGIN
	xdim2 = xdim / 2
	ydim2 = ydim / 2
	xc2   = xc / 2
	yc2   = yc / 2
	crradius2 = crradius / 2
	img2 = rebin (img, xdim2, ydim2)
        window, xsize=xdim2, ysize=ydim2
	tv, bytscl (img2, min=dispmin, max=dispmax)
        drawcir, xc2, yc2, crradius2
	END					$
    ELSE					$
	BEGIN
        window, xsize=xdim, ysize=ydim
	tv, bytscl (img, min=dispmin, max=dispmax)
        drawcir, xc, yc, crradius
	END

;     printf, 2, hour, min, sec, ft, xc+xshift, yc+yshift, xc, yc, r, $
;	      xcen, ycen, radius, xc-xcen, yc-ycen, format= format

;    printf, 2, fits_name, xc, yc, r

;    fxaddpar, hdu, 'CRPIX1', xc, ' X [EAST->WEST]   SUN CENTER [INDEX:PIXELS]'
;    fxaddpar, hdu, 'CRPIX2', yc, ' Y [NORTH->SOUTH] SUN CENTER [INDEX:PIXELS]'
;    writefits, fits_name, img, hdu
;    writefits, fits_name, hsk, ext, /APPEND


ENDWHILE

CLOSE, 1
CLOSE, 2

END
