;+ cenregs.pro
;
; Find sun center, using Guiliana Detoma's software.
; Register image by co-aligning sun center with array center.
;
; Use find_cen.pro (Giuliana DeToma).
;
; Andrew L. Stanger   HAO/NCAR   13 April 2001
;-

PRO cenregs, fits_list

;!order = 1			; Display image top to bottom.
border = 6
file_name = ''

CLOSE, 11
OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO		$
    BEGIN ;{
    READF, 11, file_name, FORMAT = '(A)'

    ;--- Read image & extension.
    
    img = readfits (file_name, hdu, /noscale)
    hsk = readfits (file_name, ext, exten_no=1, /noscale)
    
    imin = MIN (img, max=imax)
    print, 'imin/imax: ', imin, imax
    
    xdim     = fxpar (hdu, 'NAXIS1')
    ydim     = fxpar (hdu, 'NAXIS2')
    xcen     = fxpar (hdu, 'CRPIX1')
    ycen     = fxpar (hdu, 'CRPIX2')
    nsum     = fxpar (hdu, 'NIMSUM')
    crradius = fxpar (hdu, 'CRRADIUS')
    
    ;--- Compute array center.
    
    xc = (xdim - 1) * 0.5
    yc = (ydim - 1) * 0.5
    
    ; imgc = intarr (xdim - 2 * border, ydim - 2 * border)
    ; imgc = img [border:xdim-1-border, border:ydim-1-border]
    ; cmin = MIN (imgc, max=cmax)
    ; print, 'cmin/cmax: ', cmin, cmax
    
    ;--- Display image.
    
    xdim2 = xdim / 2
    ydim2 = ydim / 2
    
    img2 = REBIN (img, xdim2, ydim2)
    
    WINDOW, xsize = xdim2, ysize=ydim2
    
    TV, BYTSCL (img2, min=cmin, max=cmax)
    
    XYOUTS, 10, 10, 'ORIGINAL IMAGE', /device
    
    ;--- Find sun center (Giuliana DeToma).

    find_center, img, minint, xcen, ycen, radius

    print, file_name, ' xcen/ycen: ', xcen, ycen, ' radius: ', radius

    ;--- Translate image using average x/y shifts.
    
    xsh = xc - xcen
    ysh = yc - ycen
    print, 'xsh/ysh: ', xsh, ysh
    
    imgs = intarr (xdim, ydim)
    xsh = xc - xcen
    ysh = yc - ycen
    
    FOR iy = 0, ydim - 1 DO	$
    BEGIN
       FOR ix = 0, xdim - 1 DO	$
       BEGIN
       imgs [ix, iy] = INTERPOLATE (img, ix - xsh, iy - ysh, MISSING=0)
       ENDFOR
    ENDFOR
    
    ;--- Display registered image.
    
    imgs2 = REBIN (imgs, xdim2, ydim2)
    
    TV, BYTSCL (img2, min=cmin, max=cmax)
    XYOUTS, 10, 10, 'REGISTERED IMAGE', /device

    drawcir, xc/2, yc/2, crradius/2
    WAIT, 1
    
    ;--- Write registered image to disk.
    
    fpos = STRPOS (file_name, '.fts')
    out_name = STRMID (file_name, 0, fpos) + '.reg.fts'
    
    hdush = hdu
    fxaddpar, hdush, 'CRPIX1', xc
    fxaddpar, hdush, 'CRPIX2', yc
    fxaddpar, hdush, 'HISTORY', ' IMAGE REGISTERED WITH SUN CENTER EQUAL TO ARRAY CENTER.'
    
    writefits, out_name, img, hdush
    writefits, out_name, hsk, ext, /APPEND

    END ;}
    
CLOSE, 11
END
