;+ cenregsq.pro
;
; Find sun center, using Guiliana Detoma's software.
; Register image by co-aligning sun center with array center.
; Crop image to square size.
;
; Use find_cen.pro (Giuliana DeToma).
;
; Andrew L. Stanger   HAO/NCAR   17 April 2001
;-

PRO cenregsq, fits_list, dim=dim

file_name = ' '

IF (KEYWORD_SET (dim)) THEN	$
    BEGIN
    xdimsq = dim
    ydimsq = dim
    END 			$
ELSE				$
    BEGIN
    PRINT, 'Must specify keyword dim=value, where value=x/y dimension [output].'
    RETURN
    END

CLOSE, 11
OPENR, 11, fits_list

WHILE (NOT EOF (11)) DO		$
    BEGIN 
    READF, 11, file_name, FORMAT = '(A)'
    PRINT, file_name

    ;--- Read image & extension.
    
    img = readfits (file_name, hdu, /noscale)
    hsk = readfits (file_name, ext, exten_no=1, /noscale)
    
    imin = MIN (img, max=imax)
    print, 'imin/imax: ', imin, imax
    
    xdim     = fxpar (hdu, 'NAXIS1', comment=naxis1com)
    ydim     = fxpar (hdu, 'NAXIS2', comment=naxis2com)
    xcen     = fxpar (hdu, 'CRPIX1', comment=crpix1com)
    ycen     = fxpar (hdu, 'CRPIX2', comment=crpix2com)
    crradius = fxpar (hdu, 'CRRADIUS', comment=crradiuscom)
    radius = crradius
    
    ;--- Display image.
    
    xdim2 = xdim / 2
    ydim2 = ydim / 2
    
    img2 = REBIN (img, xdim2, ydim2)
    
    WINDOW, xsize = xdim2, ysize=ydim2
    
    TV, BYTSCL (img2, min=cmin, max=cmax)
    
    XYOUTS, 10, 10, 'ORIGINAL IMAGE', /device

    drawcir, xcen/2, ycen/2, crradius/2
    
    ;--- Find sun center (Giuliana DeToma).

    print, 'find_center'

    find_center, img, minint, xcen, ycen, radius

    print, file_name, ' xcen/ycen: ', xcen, ycen, ' radius: ', radius

    ;--- Compute array center of output image.
    
    imgsq = INTARR (xdimsq, ydimsq)
    xcensq = (xdimsq - 1) * 0.5
    ycensq = (ydimsq - 1) * 0.5
    
    ;--- Translate image using average x/y shifts.
    
    print, 'Translate image.'

    xsh = xcensq - xcen
    ysh = ycensq - ycen
    print, 'xsh/ysh: ', xsh, ysh
    
    FOR iy = 0, ydimsq - 1 DO	$
    BEGIN
       FOR ix = 0, xdimsq - 1 DO	$
       BEGIN
       imgsq [ix, iy] = INTERPOLATE (img, ix - xsh, iy - ysh, MISSING=0)
       END
    END

    ;--- Display registered image.
    
    IF (xdimsq GT 1200 OR ydimsq GT 1000)  THEN	$
	BEGIN
        xdimsq2 = xdimsq / 2
        ydimsq2 = ydimsq / 2

        imgsq2 = REBIN (imgsq, xdimsq2, ydimsq2)
    
        WINDOW, xsize = xdimsq2, ysize=ydimsq2
        TV, BYTSCL (imgsq2, min=cmin, max=cmax)
        XYOUTS, 10, 10, 'REGISTERED IMAGE', /device

        ;--- Draw a circle of radius 1.0 Rsun.

        drawcir, xcensq/2, ycensq/2, crradius/2

	WAIT, 2
        END					$
    ELSE					$
        BEGIN

        WINDOW, xsize = xdimsq, ysize=ydimsq
        TV, BYTSCL (imgsq, min=cmin, max=cmax)
        XYOUTS, 10, 10, 'REGISTERED IMAGE', /device

        ;--- Draw a circle of radius 1.0 Rsun.

        drawcir, xcensq, ycensq, crradius

	WAIT, 2
	END
    
    ;--- Write registered image to disk.
    
    fpos = STRPOS (file_name, '.fts')
    out_name = STRMID (file_name, 0, fpos) + '.regsq.fts'
    PRINT, 'out_name: ', out_name
    
    hdusq = hdu
    fxaddpar, hdusq, 'NAXIS1', xdimsq, naxis1com
    fxaddpar, hdusq, 'NAXIS2', ydimsq, naxis2com
    fxaddpar, hdusq, 'CRPIX1', xcensq, crpix1com
    fxaddpar, hdusq, 'CRPIX2', ycensq, crpix2com
    fxaddpar, hdusq, 'HISTORY', ' IMAGE REGISTERED WITH SUN CENTER EQUAL TO ARRAY CENTER.'
    
    writefits, out_name, imgsq, hdusq
    writefits, out_name, hsk, ext, /APPEND

    END

CLOSE, 11

print, 'End of cenregsq.pro'
    
END
