;+
; chipave4.pro
;
; Average 4 raw CHIP FITS images.
;
; Rename the file, adding a '0' after the WSR code.
;
; Andrew L. Stanger   HAO/NCAR   4 April 2001
;-

PRO chipave4, fits_list

fits_file = ''
CLOSE, 1

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO                  $
    BEGIN

    ;--- Read image file name.

    READF, 1, fits_file, FORMAT = '(A)'

    print, fits_file

    ;--- Read image, primary HDU, and extension HDU.

    img1 = readfits (fits_file, hdu1, /noscale)
    hsk1 = readfits (fits_file, ext1, /noscale, exten_no=1)

    date_obs = fxpar (hdu1, 'DATE-OBS')		; YYYY-MM-DD
    time_obs = fxpar (hdu1, 'TIME-OBS')		; HH:MM:SS
    code_obs = fxpar (hdu1, 'CODE-OBS')		; WSR code
    code_obs = STRTRIM (code_obs, 2)
    wsr = STRLOWCASE ( STRMID (code_obs, 0, 3) )

    print, 'date_obs : ', date_obs
    print, 'time_obs : ', time_obs
    print, 'code_obs : ', code_obs
    print, 'wsr      : ', wsr

    syear   = STRMID (date_obs, 0, 4)
    smonth  = STRMID (date_obs, 5, 2)
    sday    = STRMID (date_obs, 8, 2)

    shour   = STRMID (time_obs, 0, 2)
    sminute = STRMID (time_obs, 3, 2)
    ssecond = STRMID (time_obs, 6, 2)

    hdu0 = hdu1
    fxaddpar, hdu0, 'CODE-OBS', STRUPCASE (wsr) + '0'

    ;--- Read image file name.

    READF, 1, fits_file, FORMAT = '(A)'
    print, fits_file

    ;--- Read image, primary HDU, and extension HDU.

    img2 = readfits (fits_file, hdu2, /noscale)
    hsk2 = readfits (fits_file, ext2, /noscale, exten_no=1)

    ;--- Read image file name.

    READF, 1, fits_file, FORMAT = '(A)'
    print, fits_file

    ;--- Read image, primary HDU, and extension HDU.

    img3 = readfits (fits_file, hdu3, /noscale)
    hsk3 = readfits (fits_file, ext3, /noscale, exten_no=1)

    ;--- Read image file name.

    READF, 1, fits_file, FORMAT = '(A)'
    print, fits_file

    ;--- Read image, primary HDU, and extension HDU.

    img4 = readfits (fits_file, hdu4, /noscale)
    hsk4 = readfits (fits_file, ext4, /noscale, exten_no=1)

    ;--- Compute average.

    imgsum = float (img1) + float (img2) + float (img3) + float (img4)
    imgave = FIX (imgsum / 4.0)

    ;--- Write FITS image to disk.
    
    out_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
               '.chp.' + STRLOWCASE (wsr) + '0.fts'
    PRINT, 'out_name: ', out_name
    PRINT, ' '
    
    writefits, out_name, imgave, hdu0
    writefits, out_name, hsk1,   ext1, /APPEND
    END
CLOSE, 1
    
END
    
