;+
; NAME		chipcensq
;
; PURPOSE	Crop images to square [936x936].
;
; SYNTAX	chipcensq, fits_list
;
;		fits_list	File containing a list of 7 images, spanning
;				the wavelength range 1082.745 - 1083.315 nm.
;				Images are required to be in FITS format.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   10 May 2000
;		22 Mar 2001 : Also read & write the extension HDU.
;-

PRO chipcensq, fits_list

file1 = ''
file2 = ''
file3 = ''
file4 = ''
file5 = ''
file6 = ''
file7 = ''

print, 'fits_list: ', fits_list

;--- Read the FITS file names from the input list file.

CLOSE,  10
OPENR,  10, fits_list
READF,  10, file1
READF,  10, file2
READF,  10, file3
READF,  10, file4
READF,  10, file5
READF,  10, file6
READF,  10, file7
CLOSE, 10

print, 'file1: ', file1
print, 'file2: ', file2
print, 'file3: ', file3
print, 'file4: ', file4
print, 'file5: ', file5
print, 'file6: ', file6
print, 'file7: ', file7

;--- Read each of the images into memory.

img1 = readfits (file1, hdu1, /noscale)
img2 = readfits (file2, hdu2, /noscale)
img3 = readfits (file3, hdu3, /noscale)
img4 = readfits (file4, hdu4, /noscale)
img5 = readfits (file5, hdu5, /noscale)
img6 = readfits (file6, hdu6, /noscale)
img7 = readfits (file7, hdu7, /noscale)

;--- Read extension HDU.

hsk1 = readfits (file1, ext1, /noscale, exten_no=1)
hsk2 = readfits (file2, ext2, /noscale, exten_no=1)
hsk3 = readfits (file3, ext3, /noscale, exten_no=1)
hsk4 = readfits (file4, ext4, /noscale, exten_no=1)
hsk5 = readfits (file5, ext5, /noscale, exten_no=1)
hsk6 = readfits (file6, ext6, /noscale, exten_no=1)
hsk7 = readfits (file7, ext7, /noscale, exten_no=1)

;--- Overwrite the first line (bottom), which contains the binary header.

;img1 [*, 0] = img1 [*, 1]
;img2 [*, 0] = img2 [*, 1]
;img3 [*, 0] = img3 [*, 1]
;img4 [*, 0] = img4 [*, 1]
;img5 [*, 0] = img5 [*, 1]
;img6 [*, 0] = img6 [*, 1]
;img7 [*, 0] = img7 [*, 1]

;--- Get information from FITS header.

pixrs  = fxpar (hdu1, 'CRRADIUS')	; # Pixels / solar radius 
rsun   = fxpar (hdu1, 'RSUN')		; Solar Radius (arcsec)
xdim   = fxpar (hdu1, 'NAXIS1')
ydim   = fxpar (hdu1, 'NAXIS2')
bitpix = fxpar (hdu1, 'BITPIX')
nbypix = bitpix / 8

xdimsq = 936
ydimsq = 936

print, 'xdim/ydim:', xdim, ydim
print, 'nbypix: ', nbypix
print, 'rsun: ',  rsun
print, 'pixrs: ', pixrs
print, 'xdimsq/ydimsq: ', xdimsq, ydimsq

;--- Create storage for cropped images.

img   = fltarr (xdimsq, ydimsq)
img1sq = fltarr (xdimsq, ydimsq)
img2sq = fltarr (xdimsq, ydimsq)
img3sq = fltarr (xdimsq, ydimsq)
img4sq = fltarr (xdimsq, ydimsq)
img5sq = fltarr (xdimsq, ydimsq)
img6sq = fltarr (xdimsq, ydimsq)
img7sq = fltarr (xdimsq, ydimsq)

;--- Get the disk center locations.

xc1 = fxpar (hdu1, 'CRPIX1')
yc1 = fxpar (hdu1, 'CRPIX2')
xc2 = fxpar (hdu2, 'CRPIX1')
yc2 = fxpar (hdu2, 'CRPIX2')
xc3 = fxpar (hdu3, 'CRPIX1')
yc3 = fxpar (hdu3, 'CRPIX2')
xc4 = fxpar (hdu4, 'CRPIX1')
yc4 = fxpar (hdu4, 'CRPIX2')
xc5 = fxpar (hdu5, 'CRPIX1')
yc5 = fxpar (hdu5, 'CRPIX2')
xc6 = fxpar (hdu6, 'CRPIX1')
yc6 = fxpar (hdu6, 'CRPIX2')
xc7 = fxpar (hdu7, 'CRPIX1')
yc7 = fxpar (hdu7, 'CRPIX2')

;xc1 = xdim / 2.0 - 0.5
;yc1 = ydim / 2.0 - 0.5
;xc2 = xdim / 2.0 - 0.5
;yc2 = ydim / 2.0 - 0.5
;xc3 = xdim / 2.0 - 0.5
;yc3 = ydim / 2.0 - 0.5
;xc4 = xdim / 2.0 - 0.5
;yc4 = ydim / 2.0 - 0.5
;xc5 = xdim / 2.0 - 0.5
;yc5 = ydim / 2.0 - 0.5
;xc6 = xdim / 2.0 - 0.5
;yc6 = ydim / 2.0 - 0.5
;xc7 = xdim / 2.0 - 0.5
;yc7 = ydim / 2.0 - 0.5

;--- Get date & time.
;--- Get WSR (wavelength/Source/Reduction) code. 

date_obs1 = fxpar (hdu1, 'DATE-OBS')
date_obs2 = fxpar (hdu2, 'DATE-OBS')
date_obs3 = fxpar (hdu3, 'DATE-OBS')
date_obs4 = fxpar (hdu4, 'DATE-OBS')
date_obs5 = fxpar (hdu5, 'DATE-OBS')
date_obs6 = fxpar (hdu6, 'DATE-OBS')
date_obs7 = fxpar (hdu7, 'DATE-OBS')

time_obs1 = fxpar (hdu1, 'TIME-OBS')
time_obs2 = fxpar (hdu2, 'TIME-OBS')
time_obs3 = fxpar (hdu3, 'TIME-OBS')
time_obs4 = fxpar (hdu4, 'TIME-OBS')
time_obs5 = fxpar (hdu5, 'TIME-OBS')
time_obs6 = fxpar (hdu6, 'TIME-OBS')
time_obs7 = fxpar (hdu7, 'TIME-OBS')

code_obs1 = strtrim (strlowcase (fxpar (hdu1, 'CODE-OBS')))
code_obs2 = strtrim (strlowcase (fxpar (hdu2, 'CODE-OBS')))
code_obs3 = strtrim (strlowcase (fxpar (hdu3, 'CODE-OBS')))
code_obs4 = strtrim (strlowcase (fxpar (hdu4, 'CODE-OBS')))
code_obs5 = strtrim (strlowcase (fxpar (hdu5, 'CODE-OBS')))
code_obs6 = strtrim (strlowcase (fxpar (hdu6, 'CODE-OBS')))
code_obs7 = strtrim (strlowcase (fxpar (hdu7, 'CODE-OBS')))

year1 = strmid (date_obs1, 0, 4)
year2 = strmid (date_obs2, 0, 4)
year3 = strmid (date_obs3, 0, 4)
year4 = strmid (date_obs4, 0, 4)
year5 = strmid (date_obs5, 0, 4)
year6 = strmid (date_obs6, 0, 4)
year7 = strmid (date_obs7, 0, 4)

month1 = strmid (date_obs1, 5, 2)
month2 = strmid (date_obs2, 5, 2)
month3 = strmid (date_obs3, 5, 2)
month4 = strmid (date_obs4, 5, 2)
month5 = strmid (date_obs5, 5, 2)
month6 = strmid (date_obs6, 5, 2)
month7 = strmid (date_obs7, 5, 2)

day1 = strmid (date_obs1, 8, 2)
day2 = strmid (date_obs2, 8, 2)
day3 = strmid (date_obs3, 8, 2)
day4 = strmid (date_obs4, 8, 2)
day5 = strmid (date_obs5, 8, 2)
day6 = strmid (date_obs6, 8, 2)
day7 = strmid (date_obs7, 8, 2)

hour1 = strmid (time_obs1, 0, 2)
hour2 = strmid (time_obs2, 0, 2)
hour3 = strmid (time_obs3, 0, 2)
hour4 = strmid (time_obs4, 0, 2)
hour5 = strmid (time_obs5, 0, 2)
hour6 = strmid (time_obs6, 0, 2)
hour7 = strmid (time_obs7, 0, 2)

minute1 = strmid (time_obs1, 3, 2)
minute2 = strmid (time_obs2, 3, 2)
minute3 = strmid (time_obs3, 3, 2)
minute4 = strmid (time_obs4, 3, 2)
minute5 = strmid (time_obs5, 3, 2)
minute6 = strmid (time_obs6, 3, 2)
minute7 = strmid (time_obs7, 3, 2)

second1 = strmid (time_obs1, 6, 2)
second2 = strmid (time_obs2, 6, 2)
second3 = strmid (time_obs3, 6, 2)
second4 = strmid (time_obs4, 6, 2)
second5 = strmid (time_obs5, 6, 2)
second6 = strmid (time_obs6, 6, 2)
second7 = strmid (time_obs7, 6, 2)

;--- Create filenames for output cropped images.

file1sq = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.sq.fts'

file2sq = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.sq.fts'

file3sq = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.sq.fts'

file4sq = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.sq.fts'

file5sq = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.sq.fts'

file6sq = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.sq.fts'

file7sq = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.sq.fts'

print, 'file1sq: ', file1sq
print, 'file2sq: ', file2sq
print, 'file3sq: ', file3sq
print, 'file4sq: ', file4sq
print, 'file5sq: ', file5sq
print, 'file6sq: ', file6sq
print, 'file7sq: ', file7sq

;--- Crop each image to a square, centered at solar disk center.

x1beg = FIX ((xc1 - xdimsq / 2) + 0.5)
x1end = x1beg + xdimsq - 1
x2beg = FIX ((xc2 - xdimsq / 2) + 0.5)
x2end = x2beg + xdimsq - 1
x3beg = FIX ((xc3 - xdimsq / 2) + 0.5)
x3end = x3beg + xdimsq - 1
x4beg = FIX ((xc4 - xdimsq / 2) + 0.5)
x4end = x4beg + xdimsq - 1
x5beg = FIX ((xc5 - xdimsq / 2) + 0.5)
x5end = x5beg + xdimsq - 1
x6beg = FIX ((xc6 - xdimsq / 2) + 0.5)
x6end = x6beg + xdimsq - 1
x7beg = FIX ((xc7 - xdimsq / 2) + 0.5)
x7end = x7beg + xdimsq - 1

y1beg = FIX ((yc1 - ydimsq / 2) + 0.5)
y1end = y1beg + ydimsq - 1
y2beg = FIX ((yc2 - ydimsq / 2) + 0.5)
y2end = y2beg + ydimsq - 1
y3beg = FIX ((yc3 - ydimsq / 2) + 0.5)
y3end = y3beg + ydimsq - 1
y4beg = FIX ((yc4 - ydimsq / 2) + 0.5)
y4end = y4beg + ydimsq - 1
y5beg = FIX ((yc5 - ydimsq / 2) + 0.5)
y5end = y5beg + ydimsq - 1
y6beg = FIX ((yc6 - ydimsq / 2) + 0.5)
y6end = y6beg + ydimsq - 1
y7beg = FIX ((yc7 - ydimsq / 2) + 0.5)
y7end = y7beg + ydimsq - 1

img1sq [*, *] = img1 [x1beg:x1end, y1beg:y1end]
img2sq [*, *] = img2 [x2beg:x2end, y2beg:y2end]
img3sq [*, *] = img3 [x3beg:x3end, y3beg:y3end]
img4sq [*, *] = img4 [x4beg:x4end, y4beg:y4end]
img5sq [*, *] = img5 [x5beg:x5end, y5beg:y5end]
img6sq [*, *] = img6 [x6beg:x6end, y6beg:y6end]
img7sq [*, *] = img7 [x7beg:x7end, y7beg:y7end]

;--- Display images.

WINDOW, xsize=xdimsq, ysize=ydimsq
TVSCL, img1sq
WAIT, 1
TVSCL, img2sq
WAIT, 1
TVSCL, img3sq
WAIT, 1
TVSCL, img4sq
WAIT, 1
TVSCL, img5sq
WAIT, 1
TVSCL, img6sq
WAIT, 1
TVSCL, img7sq

;--- Modify FITS header to update the sun center information.

xcsq = (xdimsq / 2.0) - 0.5
ycsq = (ydimsq / 2.0) - 0.5

fxaddpar, hdu1, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu1, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu2, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu2, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu3, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu3, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu4, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu4, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu5, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu5, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu6, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu6, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]' 

fxaddpar, hdu7, 'CRPIX1', xcsq, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu7, 'CRPIX2', ycsq, ' Y-AXIS SUN CENTER [PIXELS]' 

;--- Write square images to disk in FITS format.

writefits, file1sq, fix (img1sq), hdu1
writefits, file2sq, fix (img2sq), hdu2
writefits, file3sq, fix (img3sq), hdu3
writefits, file4sq, fix (img4sq), hdu4
writefits, file5sq, fix (img5sq), hdu5
writefits, file6sq, fix (img6sq), hdu6
writefits, file7sq, fix (img7sq), hdu7

;--- Append extension to FITS file.

writefits, file1sq, hsk1, ext1, /append
writefits, file2sq, hsk2, ext2, /append
writefits, file3sq, hsk3, ext3, /append
writefits, file4sq, hsk4, ext4, /append
writefits, file5sq, hsk5, ext5, /append
writefits, file6sq, hsk6, ext6, /append
writefits, file7sq, hsk7, ext7, /append

END
