;+ chipcor.pro
;
; Apply Bias & LSD correction to a raw CHIP image.
;
; Andrew L. Stanger   HAO/NCAR   6 March 2001
;-

PRO chipcor, fits_file

    ;--- Read solar image file.

    img = readfits (fits_file, hdu)
    xdim = fxpar (hdu, 'NAXIS1')
    ydim = fxpar (hdu, 'NAXIS2')

    ;--- Get x/y shifts from extension.

    hdr = readfits (fits_file, hduext, /noscale, exten_no=1)

    ;--- Compute average shift.

    xshift0 = fxpar (hduext, 'XSHIFT00')
    xshift0 = fxpar (hduext, 'XSHIFT01')
    xshift0 = fxpar (hduext, 'XSHIFT02')
    xshift0 = fxpar (hduext, 'XSHIFT03')
    xshift  = (xshift0 + xshift1 + xshift2 + xshift3) / 4.0

    yshift0 = fxpar (hduext, 'YSHIFT00')
    yshift0 = fxpar (hduext, 'YSHIFT01')
    yshift0 = fxpar (hduext, 'YSHIFT02')
    yshift0 = fxpar (hduext, 'YSHIFT03')
    yshift  = (yshift0 + yshift1 + yshift2 + yshift3) / 4.0

    ;--- Create shift vectors.

    xin = INDGEN (xdim) + xshift
    yin = INDGEN (ydim) + yshift

   ;--- Create shift location arrays.

    xloc = INTARR (xdim, ydim)
    yloc = INTARR (xdim, ydim)

    FOR iy = 0, ydim - 1 DO xloc [*, iy] = xin
    FOR ix = 0, xdim - 1 DO yloc [ix, *] = yin

    ;--- Read Bias image.

    bias = readfits ('xbr.fts', hdubias)

    ;--- Read LSD image.

    lsd = readfits ('hdr.lsc.fts', hdulsd)
    dmin = MIN (lsd, max=dmax)
    PRINT, 'dmin/dmax: ', dmin, dmax

    xdim = fxpar (hdu, 'NAXIS1')
    ydim = fxpar (hdu, 'NAXIS2')
    xdim2 = xdim / 2
    ydim2 = ydim / 2

    ;--- Over-write the 3 pixels along the perimeter to avoid "shift" junk.

;    img [*, 0] = img [*, 3]
;    img [*, 1] = img [*, 3]
;    img [*, 2] = img [*, 3]

;    img [*, ydim-1] = img [*, ydim-4]
;    img [*, ydim-2] = img [*, ydim-4]
;    img [*, ydim-3] = img [*, ydim-4]

;    img [0, *] = img [3, *]
;    img [1, *] = img [3, *]
;    img [2, *] = img [3, *]

;    img [xdim-1, *] = img [xdim-4, *]
;    img [xdim-2, *] = img [xdim-4, *]
;    img [xdim-3, *] = img [xdim-4, *]

    img [*, 0] = 1.0
    img [*, 1] = 1.0
    img [*, 2] = 1.0

    img [*, ydim-1] = 1.0
    img [*, ydim-2] = 1.0
    img [*, ydim-3] = 1.0

    img [0, *] = 1.0 
    img [1, *] = 1.0
    img [2, *] = 1.0

    img [xdim-1, *] = 1.0
    img [xdim-2, *] = 1.0
    img [xdim-3, *] = 1.0

    imin = MIN (img, max=imax)
    PRINT, 'imin/imax: ', imin, imax

    ;--- Shift bias image to match solar image registration.

;    biasreg = BILINEAR (bias, xloc, yloc)
    biasreg = INTERPOLATE (bias, xloc, yloc, missing=1.0)

    ;--- Shift LSD correction to match solar image registration.

;    lsdreg = BILINEAR (lsd, xloc, yloc)
    lsdreg = INTERPOLATE (lsd, xloc, yloc, missing=1.0)

    ;--- Display diffuser correction image.

    img2 = REBIN (img, xdim2, ydim2)
    TVSCL, img2
    XYOUTS, 5, 5, 'ORIGINAL', /device
    WAIT, 3

    ;--- Apply LSD correction.

    imgc = ( img  - biasreg ) * lsdreg

    ;--- Display corrected solar image.

    cmin = MIN (imgc, max=cmax)
    PRINT, 'cmin/cmax: ', cmin, cmax

    imgc2 = REBIN (imgc, xdim2, ydim2)

    window, xsize=xdim2, ysize=ydim2
    TVSCL, imgc2
    XYOUTS, 5, 5, 'CORRECTED', /device

    file_adj = syear + smonth + sday + '.' + shour + sminute + ssecond + $
	       '.chp.' + code_obs + '.bgc.fts'
    writefits, file_adj, imgc, hdu

END

