;+ chiphv.pro
;
; Shift a raw CHIP image.
; Use XSHIFTxx and YSHIFTxx values in extension.
;
; Andrew L. Stanger   HAO/NCAR   6 March 2001
;-

PRO chiphv, file_name

!order = 1			; Display image top to bottom.
border = 6

img = readfits (file_name, hdu, /noscale)
ext = readfits (file_name, hduext, exten_no=1)

imin = MIN (img, max=imax)
print, 'imin/imax: ', imin, imax

xdim     = fxpar (hdu, 'NAXIS1')
ydim     = fxpar (hdu, 'NAXIS2')
xcen     = fxpar (hdu, 'CRPIX1')
ycen     = fxpar (hdu, 'CRPIX2')
nsum     = fxpar (hdu, 'NIMSUM')
crradius = fxpar (hdu, 'CRRADIUS')
xcen2 = xcen / 2.0
ycen2 = ycen / 2.0
crradius2 = crradius / 2.0

imgc = intarr (xdim - 2 * border, ydim - 2 * border)
imgc = img [border:xdim-1-border, border:ydim-1-border]
cmin = MIN (imgc, max=cmax)
print, 'cmin/cmax: ', cmin, cmax

;--- Compute average x/y shift using shift values in FITS EXTENSION.

xsh = fltarr (nsum)
ysh = fltarr (nsum)
xshsum = 0
yshsum = 0

FOR i = 0, nsum - 1 DO	$
BEGIN
    snum = STRTRIM (STRING (i), 2)
    IF (nsum LT 10) THEN snum = '0' + snum
    xshkey = 'XSHIFT' + snum
    print, 'xshkey: ', xshkey
    xsh [i] = fxpar (hduext, 'XSHIFT' + snum)
    print, 'xsh: ', xsh [i]
    xshsum  = xshsum + xsh [i]
    ysh [i] = fxpar (hduext, 'YSHIFT' + snum)
    yshsum  = yshsum + ysh [i]
    print, 'ysh: ', ysh [i]
END
xshave = xshsum / nsum
yshave = yshsum / nsum
print, 'xshave/yshave: ', xshave, yshave

;--- Display image.

xdim2 = xdim / 2
ydim2 = ydim / 2

WINDOW, xsize = xdim2, ysize=ydim2

img2 = rebin (img, xdim2, ydim2)
window, xsize = xdim2, ysize = ydim2
;tvscl, img2
tv, bytscl (img2, min=cmin, max=cmax)
wait, 3

;--- Translate image using average x/y shifts.

img2s = intarr (xdim2, ydim2)

FOR iy = 0, ydim2 - 1 DO	$
BEGIN
   FOR ix = 0, xdim2 - 1 DO	$
   BEGIN
   img2s [ix, iy] = INTERPOLATE (img2, ix - xshave, iy - yshave, MISSING=0)
   ENDFOR
ENDFOR

;--- Display image.

tv, bytscl (img2s, min=cmin, max=cmax)

;--- Write shifted image to disk.

out_name = 'imgsh.fts'
hdush = hdu
fxaddpar, hdush, 'CRPIX1', xcen2 + xshave
fxaddpar, hdush, 'CRPIX2', ycen2 + yshave
fxaddpar, hdush, 'CRRADIUS', crradius2
writefits, out_name, img2s, hdush

END
