;+
; NAME		chipnorm11
;
; PURPOSE	Normalize filter responses.
;
; SYNTAX	chipnorm11, fits_list
;
;		fits_list	File containing a list of 7 images, spanning
;				the wavelength range 1082.745 - 1083.315 nm.
;				Images are required to be in FITS format.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR    2 Jun 2000
;		19 Sep 2000: Use ssr limb darkening for all except dsr.
;		             Normalization: all agree at sun center.
;-

PRO chipnorm11, fits_list

file  = ''
file1 = ''
file2 = ''
file3 = ''
file4 = ''
file5 = ''
file6 = ''
file7 = ''

xdimsq = 936
ydimsq = 936

daytab = INTARR (13,2)
daytab = [[0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ],  $
          [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]]
window, xsize=xdimsq, ysize=ydimsq
lct, '/home/stanger/color/chipcol.lut'

;--- Limb darkening correction is computed by averaging radial scans
;    every 1 degree for the R2 (mid red) filter image.
;    Use chip_ldark.c program to do this (/home/stanger/mlso/chip/als).

;ld_name = 'chip.ldcor.img'              ; Limb darkening correction image.
;imgld = FLTARR (xdimsq, ydimsq)

;--- Limb darkening is computed for each image.

imgld1 = FLTARR (xdimsq, ydimsq)
;imgld2 = FLTARR (xdimsq, ydimsq)
;imgld3 = FLTARR (xdimsq, ydimsq)
;imgld4 = FLTARR (xdimsq, ydimsq)
;imgld5 = FLTARR (xdimsq, ydimsq)
imgld6 = FLTARR (xdimsq, ydimsq)
;imgld7 = FLTARR (xdimsq, ydimsq)

;--- Read the FITS file names from the input list file.
;
;    Note: Prior to using this IDL program, use the  "acosfits" c-language
;    program to convert native format CHIP images into FITS format.

CLOSE,  10
OPENR,  10, fits_list

;--- Read each of the images into memory.

FOR i = 0, 6 DO		$
    BEGIN
    READF,  10, file
    PRINT, 'file: ', file
    img = readfits (file, hdu, /noscale)
    code_obs = strtrim (strlowcase (fxpar (hdu, 'CODE-OBS')))
    wavelen = fxpar (hdu, 'WAVELEN')
    pangle = fxpar (hdu, 'SOLAR_P0')

;--- Assign image arrays according to the following labels:
;
;    img1: dsr   far  blue
;    img2: csr   mid  blue
;    img3: bsr   near blue
;    img4: hsr   on-line helium
;    img5: rsr   near red
;    img6: ssr   mid  red
;    img7: tsr   far  red

    CASE code_obs OF
        'dsr': BEGIN
	       img1 = img
	       hdu1 = hdu
	       code_obs1 = code_obs
	       file1 = file
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'csr': BEGIN
	       img2 = img
	       hdu2 = hdu
	       code_obs2 = code_obs
	       file2 = file
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'bsr': BEGIN
	       img3 = img
	       hdu3 = hdu
	       file3 = file
	       code_obs3 = code_obs
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'hsr': BEGIN
	       img4 = img
	       hdu4 = hdu
	       code_obs4 = code_obs
	       file4 = file
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'rsr': BEGIN
	       img5 = img
	       hdu5 = hdu
	       code_obs5 = code_obs
	       file5 = file
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'ssr': BEGIN
	       img6 = img
	       hdu6 = hdu
	       code_obs6 = code_obs
	       file6 = file
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        'tsr': BEGIN
	       img7 = img
	       hdu7 = hdu
	       code_obs7 = code_obs
	       file7 = file
	       print, 'code_obs: ', code_obs, '   wavelen: ', wavelen
               END
        ELSE:  BEGIN
	       PRINT, 'Invalid code-obs: ', code_obs
	       END
        ENDCASE
    END
CLOSE, 10

img = 0
hdu = 0

;--- Overwrite the first line (bottom), which contains the binary header.
;--- Replace it with the adjacent horizontal line of the image.

img1 [*, 0] = img1 [*, 1]
img2 [*, 0] = img2 [*, 1]
img3 [*, 0] = img3 [*, 1]
img4 [*, 0] = img4 [*, 1]
img5 [*, 0] = img5 [*, 1]
img6 [*, 0] = img6 [*, 1]
img7 [*, 0] = img7 [*, 1]

;--- Get information from FITS header.

pixrs  = fxpar (hdu4, 'CRRADIUS')	; # Pixels / solar radius 
rsun   = fxpar (hdu4, 'RSUN')		; Solar Radius (arcsec)
xdim   = fxpar (hdu4, 'NAXIS1')		; X-axis dimension.
ydim   = fxpar (hdu4, 'NAXIS2')		; Y-axis dimension.
bitpix = fxpar (hdu4, 'BITPIX')		; Bits per pixel.
nbypix = bitpix / 8			; Bytes per pixel.

ipixrs = fix (pixrs + 0.5)
border = 50				; "Border" beyond the limb for cropping.
xdimc = (ipixrs + border) * 2
ydimc = xdimc
xdimc = 936				; X-axis dimension for cropped image.
ydimc = 936				; Y-axis dimension for cropped image.

PRINT, 'xdim/ydim :   ', xdim, ydim, '   nbypix: ', nbypix
PRINT, 'xdimc/ydimc : ', xdimc, ydimc
PRINT, 'rsun  : ', rsun
PRINT, 'pixrs : ', pixrs

;--- Create storage for cropped images (square shape).

img1f = FLTARR (xdimc, ydimc)
img2f = FLTARR (xdimc, ydimc)
img3f = FLTARR (xdimc, ydimc)
img4f = FLTARR (xdimc, ydimc)
img5f = FLTARR (xdimc, ydimc)
img6f = FLTARR (xdimc, ydimc)
img7f = FLTARR (xdimc, ydimc)

;--- Get the disk center locations from the FITS header.

xc1 = fxpar (hdu1, 'CRPIX1')
yc1 = fxpar (hdu1, 'CRPIX2')
xc2 = fxpar (hdu2, 'CRPIX1')
yc2 = fxpar (hdu2, 'CRPIX2')
xc3 = fxpar (hdu3, 'CRPIX1')
yc3 = fxpar (hdu3, 'CRPIX2')
xc4 = fxpar (hdu4, 'CRPIX1')
yc4 = fxpar (hdu4, 'CRPIX2')
xc5 = fxpar (hdu5, 'CRPIX1')
yc5 = fxpar (hdu5, 'CRPIX2')
xc6 = fxpar (hdu6, 'CRPIX1')
yc6 = fxpar (hdu6, 'CRPIX2')
xc7 = fxpar (hdu7, 'CRPIX1')
yc7 = fxpar (hdu7, 'CRPIX2')

;--- If center information is invalid, assume disk center is array center.

IF (xc1 LT 0.0)  THEN xc1 = xdim / 2.0 - 0.5
IF (yc1 LT 0.0)  THEN yc1 = ydim / 2.0 - 0.5
IF (xc2 LT 0.0)  THEN xc2 = xdim / 2.0 - 0.5
IF (yc2 LT 0.0)  THEN yc2 = ydim / 2.0 - 0.5
IF (xc3 LT 0.0)  THEN xc3 = xdim / 2.0 - 0.5
IF (yc3 LT 0.0)  THEN yc3 = ydim / 2.0 - 0.5
IF (xc4 LT 0.0)  THEN xc4 = xdim / 2.0 - 0.5
IF (yc4 LT 0.0)  THEN yc4 = ydim / 2.0 - 0.5
IF (xc5 LT 0.0)  THEN xc5 = xdim / 2.0 - 0.5
IF (yc5 LT 0.0)  THEN yc5 = ydim / 2.0 - 0.5
IF (xc6 LT 0.0)  THEN xc6 = xdim / 2.0 - 0.5
IF (yc6 LT 0.0)  THEN yc6 = ydim / 2.0 - 0.5
IF (xc7 LT 0.0)  THEN xc7 = xdim / 2.0 - 0.5
IF (yc7 LT 0.0)  THEN yc7 = ydim / 2.0 - 0.5

PRINT, 'xc1/yc1: ', xc1, yc1

;--- Get date & time.
;--- Get WSR (wavelength/Source/Reduction) code. 

date_obs1 = fxpar (hdu1, 'DATE-OBS')
date_obs2 = fxpar (hdu2, 'DATE-OBS')
date_obs3 = fxpar (hdu3, 'DATE-OBS')
date_obs4 = fxpar (hdu4, 'DATE-OBS')
date_obs5 = fxpar (hdu5, 'DATE-OBS')
date_obs6 = fxpar (hdu6, 'DATE-OBS')
date_obs7 = fxpar (hdu7, 'DATE-OBS')

time_obs1 = fxpar (hdu1, 'TIME-OBS')
time_obs2 = fxpar (hdu2, 'TIME-OBS')
time_obs3 = fxpar (hdu3, 'TIME-OBS')
time_obs4 = fxpar (hdu4, 'TIME-OBS')
time_obs5 = fxpar (hdu5, 'TIME-OBS')
time_obs6 = fxpar (hdu6, 'TIME-OBS')
time_obs7 = fxpar (hdu7, 'TIME-OBS')

year1 = strmid (date_obs1, 0, 4)
year2 = strmid (date_obs2, 0, 4)
year3 = strmid (date_obs3, 0, 4)
year4 = strmid (date_obs4, 0, 4)
year5 = strmid (date_obs5, 0, 4)
year6 = strmid (date_obs6, 0, 4)
year7 = strmid (date_obs7, 0, 4)

month1 = strmid (date_obs1, 5, 2)
month2 = strmid (date_obs2, 5, 2)
month3 = strmid (date_obs3, 5, 2)
month4 = strmid (date_obs4, 5, 2)
month5 = strmid (date_obs5, 5, 2)
month6 = strmid (date_obs6, 5, 2)
month7 = strmid (date_obs7, 5, 2)

day1 = strmid (date_obs1, 8, 2)
day2 = strmid (date_obs2, 8, 2)
day3 = strmid (date_obs3, 8, 2)
day4 = strmid (date_obs4, 8, 2)
day5 = strmid (date_obs5, 8, 2)
day6 = strmid (date_obs6, 8, 2)
day7 = strmid (date_obs7, 8, 2)

hour1 = strmid (time_obs1, 0, 2)
hour2 = strmid (time_obs2, 0, 2)
hour3 = strmid (time_obs3, 0, 2)
hour4 = strmid (time_obs4, 0, 2)
hour5 = strmid (time_obs5, 0, 2)
hour6 = strmid (time_obs6, 0, 2)
hour7 = strmid (time_obs7, 0, 2)

minute1 = strmid (time_obs1, 3, 2)
minute2 = strmid (time_obs2, 3, 2)
minute3 = strmid (time_obs3, 3, 2)
minute4 = strmid (time_obs4, 3, 2)
minute5 = strmid (time_obs5, 3, 2)
minute6 = strmid (time_obs6, 3, 2)
minute7 = strmid (time_obs7, 3, 2)

second1 = strmid (time_obs1, 6, 2)
second2 = strmid (time_obs2, 6, 2)
second3 = strmid (time_obs3, 6, 2)
second4 = strmid (time_obs4, 6, 2)
second5 = strmid (time_obs5, 6, 2)
second6 = strmid (time_obs6, 6, 2)
second7 = strmid (time_obs7, 6, 2)

;--- Create filenames for output cropped images.

file1c = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.sq.fts'

file2c = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.sq.fts'

file3c = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.sq.fts'

file4c = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.sq.fts'

file5c = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.sq.fts'

file6c = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.sq.fts'

file7c = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.sq.fts'

;--- Find disc center.

IF (xc1 LT 0.0 OR yc1 LT 0.0) THEN	$
   BEGIN
   gen_loc, img1, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file1: ', file1
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc1 = centroid [0]
   yc1 = centroid [1]
   END

;*******************************************************************************

;--- Determine beginning & ending pixels to crop images to a square size.

;--- For now, assume all images are registered.

x1beg = fix ((xc1 - xdimsq / 2) + 0.5)
x1end = x1beg + xdimsq - 1
x2beg = x1beg
x2end = x1end
x3beg = x1beg
x3end = x1end
x4beg = x1beg
x4end = x1end
x5beg = x1beg
x5end = x1end
x6beg = x1beg
x6end = x1end
x7beg = x1beg
x7end = x1end

y1beg = fix ((yc1 - ydimsq / 2) + 0.5)
y1end = y1beg + ydimsq - 1
y2beg = y1beg
y2end = y1end
y3beg = y1beg
y3end = y1end
y4beg = y1beg
y4end = y1end
y5beg = y1beg
y5end = y1end
y6beg = y1beg
y6end = y1end
y7beg = y1beg
y7end = y1end

;x1beg = fix (xc1 + 0.5) - ipixrs - border
;x1end = x1beg + (ipixrs + border) * 2 - 1
;y1beg = yc1 - ipixrs - border
;y1end = y1beg + (ipixrs + border) * 2 - 1

x1dif = x1end - x1beg + 1
PRINT, 'x1beg/x1end: ', x1beg, x1end, ' x1dif: ', x1dif
;*******************************************************************************

IF (xc2 LT 0.0 OR yc2 LT 0.0) THEN	$
   BEGIN
   gen_loc, img2, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file2: ', file2
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc2 = centroid [0]
   yc2 = centroid [1]
   END

IF (xc3 LT 0.0 OR yc3 LT 0.0) THEN	$
   BEGIN
   gen_loc, img3, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file3: ', file3
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc3 = centroid [0]
   yc3 = centroid [1]
   END

IF (xc4 LT 0.0 OR yc4 LT 0.0) THEN	$
   BEGIN
   gen_loc, img4, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file4: ', file4
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc4 = centroid [0]
   yc4 = centroid [1]
   END

IF (xc5 LT 0.0 OR yc5 LT 0.0) THEN	$
   BEGIN
   gen_loc, img5, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file5: ', file5
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc5 = centroid [0]
   yc5 = centroid [1]
   END

IF (xc6 LT 0.0 OR yc6 LT 0.0) THEN	$
   BEGIN
   gen_loc, img6, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file6: ', file6
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc6 = centroid [0]
   yc6 = centroid [1]
   END

IF (xc7 LT 0.0 OR yc7 LT 0.0) THEN	$
   BEGIN
   gen_loc, img7, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
            inter=inter, quiet=quiet, cir=cir
   PRINT, 'file7: ', file7
   PRINT, 'hzrd/vtrd: ', hzrd, vtrd
   PRINT, 'centroid: ', centroid
   xc7 = centroid [0]
   yc7 = centroid [1]
   END

;-------------------------------------------------------------------------------

;--- Crop each image to a square, centered at solar disk center.
;--- Also, convert to floating point.

;x1beg = fix (xc1 + 0.5) - ipixrs - border
;x1end = x1beg + (ipixrs + border) * 2 - 1
;y1beg = fix (yc1 + 0.5) - ipixrs - border
;y1end = y1beg + (ipixrs + border) * 2 - 1

;x2beg = fix (xc2 + 0.5) - ipixrs - border
;x2end = x2beg + (ipixrs + border) * 2 - 1
;y2beg = fix (yc2 + 0.5) - ipixrs - border
;y2end = y2beg + (ipixrs + border) * 2 - 1

;x3beg = fix (xc1 + 0.5) - ipixrs - border
;x3end = x3beg + (ipixrs + border) * 2 - 1
;y3beg = fix (yc1 + 0.5) - ipixrs - border
;y3end = y3beg + (ipixrs + border) * 2 - 1

;x4beg = fix (xc4 + 0.5) - ipixrs - border
;x4end = x4beg + (ipixrs + border) * 2 - 1
;y4beg = fix (yc4 + 0.5) - ipixrs - border
;y4end = y4beg + (ipixrs + border) * 2 - 1

;x5beg = fix (xc5 + 0.5) - ipixrs - border
;x5end = x5beg + (ipixrs + border) * 2 - 1
;y5beg = fix (yc5 + 0.5) - ipixrs - border
;y5end = y5beg + (ipixrs + border) * 2 - 1

;x6beg = fix (xc6 + 0.5) - ipixrs - border
;x6end = x6beg + (ipixrs + border) * 2 - 1
;y6beg = fix (yc6 + 0.5) - ipixrs - border
;y6end = y6beg + (ipixrs + border) * 2 - 1

;x7beg = fix (xc7 + 0.5) - ipixrs - border
;x7end = x7beg + (ipixrs + border) * 2 - 1
;y7beg = fix (yc7 + 0.5) - ipixrs - border
;y7end = y7beg + (ipixrs + border) * 2 - 1

img1f [*, *] = float (img1 [x1beg:x1end, y1beg:y1end])
img2f [*, *] = float (img2 [x2beg:x2end, y2beg:y2end])
img3f [*, *] = float (img3 [x3beg:x3end, y3beg:y3end])
img4f [*, *] = float (img4 [x4beg:x4end, y4beg:y4end])
img5f [*, *] = float (img5 [x5beg:x5end, y5beg:y5end])
img6f [*, *] = float (img6 [x6beg:x6end, y6beg:y6end])
img7f [*, *] = float (img7 [x7beg:x7end, y7beg:y7end])

;--- Display cropped images & label with filter designation.

tvscl, img1f
xyouts, 50, 50, 'B3', /device
wait, 1
tvscl, img2f
xyouts, 50, 50, 'B2', /device
wait, 1
tvscl, img3f
xyouts, 50, 50, 'B1', /device
wait, 1
tvscl, img4f
xyouts, 50, 50, 'He', /device
wait, 1
tvscl, img5f
xyouts, 50, 50, 'R1', /device
wait, 1
tvscl, img6f
xyouts, 50, 50, 'R2', /device
wait, 1
tvscl, img7f
xyouts, 50, 50, 'R3', /device
wait, 1

;--- For cropped, square images, set sun center equal to array center.

xc1 = xdimc / 2.0 - 0.5
yc1 = ydimc / 2.0 - 0.5

xc2 = xdimc / 2.0 - 0.5
yc2 = ydimc / 2.0 - 0.5

xc3 = xdimc / 2.0 - 0.5
yc3 = ydimc / 2.0 - 0.5

xc4 = xdimc / 2.0 - 0.5
yc4 = ydimc / 2.0 - 0.5

xc5 = xdimc / 2.0 - 0.5
yc5 = ydimc / 2.0 - 0.5

xc6 = xdimc / 2.0 - 0.5
yc6 = ydimc / 2.0 - 0.5

xc7 = xdimc / 2.0 - 0.5
yc7 = ydimc / 2.0 - 0.5

;--- Now, find sun center again in the square images.

;window, xsize=xdimc, ysize=ydimc

;gen_loc, img1f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc1 = centroid [0]
;yc1 = centroid [1]

;gen_loc, img2f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc2 = centroid [0]
;yc2 = centroid [1]

;gen_loc, img3f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc3 = centroid [0]
;yc3 = centroid [1]

;gen_loc, img4f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc4 = centroid [0]
;yc4 = centroid [1]

;gen_loc, img5f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc5 = centroid [0]
;yc5 = centroid [1]

;gen_loc, img6f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc6 = centroid [0]
;yc6 = centroid [1]

;gen_loc, img7f, HORZRAD=hzrd, VERTRAD=vtrd, CENTROID=centroid, $
;         inter=inter, quiet=quiet, cir=cir
;xc7 = centroid [0]
;yc7 = centroid [1]

;--- Modify FITS header to update the sun center information.

fxaddpar, hdu1, 'CRPIX1', xc1, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu1, 'CRPIX2', yc1, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu2, 'CRPIX1', xc2, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu2, 'CRPIX2', yc2, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu3, 'CRPIX1', xc3, ' X-AXIS SUN CENTER [PIXELS]'
fxaddpar, hdu3, 'CRPIX2', yc3, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu4, 'CRPIX1', xc4, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu4, 'CRPIX2', yc4, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu5, 'CRPIX1', xc5, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu5, 'CRPIX2', yc5, ' Y-AXIS SUN CENTER [PIXELS]'

fxaddpar, hdu6, 'CRPIX1', xc6, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu6, 'CRPIX2', yc6, ' Y-AXIS SUN CENTER [PIXELS]' 

fxaddpar, hdu7, 'CRPIX1', xc7, ' X-AXIS SUN CENTER [PIXELS]' 
fxaddpar, hdu7, 'CRPIX2', yc7, ' Y-AXIS SUN CENTER [PIXELS]' 

;--- Rotate image to heliocentric coordinates.

img1f = rot (img1f, pangle)
img2f = rot (img2f, pangle)
img3f = rot (img3f, pangle)
img4f = rot (img4f, pangle)
img5f = rot (img5f, pangle)
img6f = rot (img6f, pangle)
img7f = rot (img7f, pangle)

;--- Display the square, rotated images.

tvscl, img1f
xyouts, 50, 50, 'B3 Heliocentric', /device
wait, 1
tvscl, img2f
xyouts, 50, 50, 'B2 Heliocentric', /device
wait, 1
tvscl, img3f
xyouts, 50, 50, 'B1 Heliocentric', /device
wait, 1
tvscl, img4f
xyouts, 50, 50, 'He Heliocentric', /device
wait, 1
tvscl, img5f
xyouts, 50, 50, 'R1 Heliocentric', /device
wait, 1
tvscl, img6f
xyouts, 50, 50, 'R2 Heliocentric', /device
wait, 1
tvscl, img7f
xyouts, 50, 50, 'R3 Heliocentric', /device
wait, 1

;---  Update FITS header to indicate heliocentric coordinate orientation.

fxaddpar, hdu1, 'CROTA1', 0.0
fxaddpar, hdu2, 'CROTA1', 0.0
fxaddpar, hdu3, 'CROTA1', 0.0
fxaddpar, hdu4, 'CROTA1', 0.0
fxaddpar, hdu5, 'CROTA1', 0.0
fxaddpar, hdu6, 'CROTA1', 0.0
fxaddpar, hdu7, 'CROTA1', 0.0

fxaddpar, hdu1, 'CROTA2', 0.0
fxaddpar, hdu2, 'CROTA2', 0.0
fxaddpar, hdu3, 'CROTA2', 0.0
fxaddpar, hdu4, 'CROTA2', 0.0
fxaddpar, hdu5, 'CROTA2', 0.0
fxaddpar, hdu6, 'CROTA2', 0.0
fxaddpar, hdu7, 'CROTA2', 0.0

fxaddpar, hdu1, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu2, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu3, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu4, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu5, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu6, 'COORDNAM', 'HELIOCENTRIC'
fxaddpar, hdu7, 'COORDNAM', 'HELIOCENTRIC'

;--- Write square, heliocentric images to disk.

writefits, file1c, fix (img1f), hdu1
writefits, file2c, fix (img2f), hdu2
writefits, file3c, fix (img3f), hdu3
writefits, file4c, fix (img4f), hdu4
writefits, file5c, fix (img5f), hdu5
writefits, file6c, fix (img6f), hdu6
writefits, file7c, fix (img7f), hdu7

;--- Create the limb darkening file names.

syr1 = strmid (year1, 2, 2)
syr2 = strmid (year2, 2, 2)
syr3 = strmid (year3, 2, 2)
syr4 = strmid (year4, 2, 2)
syr5 = strmid (year5, 2, 2)
syr6 = strmid (year6, 2, 2)
syr7 = strmid (year7, 2, 2)

iyear1 = FIX (year1)
leap = LONG ((iyear1 MOD 4 EQ 0) AND (iyear1 MOD 100 NE 0) OR $
             (iyear1 MOD 400 EQ 0))
idoy1 = FIX (day1)
idoy2 = FIX (day2)
idoy3 = FIX (day3)
idoy4 = FIX (day4)
idoy5 = FIX (day5)
idoy6 = FIX (day6)
idoy7 = FIX (day7)

FOR i = 0, month1 - 1 DO idoy1 = idoy1 + daytab (i, leap)
FOR i = 0, month2 - 1 DO idoy2 = idoy2 + daytab (i, leap)
FOR i = 0, month3 - 1 DO idoy3 = idoy3 + daytab (i, leap)
FOR i = 0, month4 - 1 DO idoy4 = idoy4 + daytab (i, leap)
FOR i = 0, month5 - 1 DO idoy5 = idoy5 + daytab (i, leap)
FOR i = 0, month6 - 1 DO idoy6 = idoy6 + daytab (i, leap)
FOR i = 0, month7 - 1 DO idoy7 = idoy7 + daytab (i, leap)

sdoy1 = STRTRIM (STRING (idoy1), 2)
sdoy2 = STRTRIM (STRING (idoy2), 2)
sdoy3 = STRTRIM (STRING (idoy3), 2)
sdoy4 = STRTRIM (STRING (idoy4), 2)
sdoy5 = STRTRIM (STRING (idoy5), 2)
sdoy6 = STRTRIM (STRING (idoy6), 2)
sdoy7 = STRTRIM (STRING (idoy7), 2)

IF (idoy1 LT 10)  THEN sdoy1 = '00' + sdoy1
IF (idoy2 LT 10)  THEN sdoy2 = '00' + sdoy2
IF (idoy3 LT 10)  THEN sdoy3 = '00' + sdoy3
IF (idoy4 LT 10)  THEN sdoy4 = '00' + sdoy4
IF (idoy5 LT 10)  THEN sdoy5 = '00' + sdoy5
IF (idoy6 LT 10)  THEN sdoy6 = '00' + sdoy6
IF (idoy7 LT 10)  THEN sdoy7 = '00' + sdoy7

IF (idoy1 LT 100)  THEN sdoy1 = '0' + sdoy1
IF (idoy2 LT 100)  THEN sdoy2 = '0' + sdoy2
IF (idoy3 LT 100)  THEN sdoy3 = '0' + sdoy3
IF (idoy4 LT 100)  THEN sdoy4 = '0' + sdoy4
IF (idoy5 LT 100)  THEN sdoy5 = '0' + sdoy5
IF (idoy6 LT 100)  THEN sdoy6 = '0' + sdoy6
IF (idoy7 LT 100)  THEN sdoy7 = '0' + sdoy7

ld_name1 = syr1 + 'd' + sdoy1 + '.' + hour1 + minute1 + second1 + '.' + $
	   code_obs1 + '.ldcor'
ld_name2 = syr2 + 'd' + sdoy2 + '.' + hour2 + minute2 + second2 + '.' + $
	   code_obs2 + '.ldcor'
ld_name3 = syr3 + 'd' + sdoy3 + '.' + hour3 + minute3 + second3 + '.' + $
	   code_obs3 + '.ldcor'
ld_name4 = syr4 + 'd' + sdoy4 + '.' + hour4 + minute4 + second4 + '.' + $
	   code_obs4 + '.ldcor'
ld_name5 = syr5 + 'd' + sdoy5 + '.' + hour5 + minute5 + second5 + '.' + $
	   code_obs5 + '.ldcor'
ld_name6 = syr6 + 'd' + sdoy6 + '.' + hour6 + minute6 + second6 + '.' + $
	   code_obs6 + '.ldcor'
ld_name7 = syr7 + 'd' + sdoy7 + '.' + hour7 + minute7 + second7 + '.' + $
	   code_obs7 + '.ldcor'

print, 'ld_name1: ', ld_name1
print, 'ld_name2: ', ld_name2
print, 'ld_name3: ', ld_name3
print, 'ld_name4: ', ld_name4
print, 'ld_name5: ', ld_name5
print, 'ld_name6: ', ld_name6
print, 'ld_name7: ', ld_name7

;--- Read limb darkening correction images.

;--- These images are created by the c-language program "chip_ldark2.c"
;    in the directory: /home/stanger/mlso/chip/als".

CLOSE, 1

OPENR, 1, ld_name1
READU, 1, imgld1                      ; 936 x 936 floating point pixels.
CLOSE, 1
PRINT, 'size (imgld1): ', SIZE (imgld1)

;OPENR, 1, ld_name2
;READU, 1, imgld2                      ; 936 x 936 floating point pixels.
;CLOSE, 1
;PRINT, 'size (imgld2): ', SIZE (imgld2)

;OPENR, 1, ld_name3
;READU, 1, imgld3                      ; 936 x 936 floating point pixels.
;CLOSE, 1
;PRINT, 'size (imgld3): ', SIZE (imgld3)

;OPENR, 1, ld_name4
;READU, 1, imgld4                      ; 936 x 936 floating point pixels.
;CLOSE, 1
;PRINT, 'size (imgld4): ', SIZE (imgld4)

;OPENR, 1, ld_name5
;READU, 1, imgld5                      ; 936 x 936 floating point pixels.
;CLOSE, 1
;PRINT, 'size (imgld5): ', SIZE (imgld5)

OPENR, 1, ld_name6
READU, 1, imgld6                      ; 936 x 936 floating point pixels.
CLOSE, 1
PRINT, 'size (imgld6): ', SIZE (imgld6)

;OPENR, 1, ld_name7
;READU, 1, imgld7                      ; 936 x 936 floating point pixels.
;CLOSE, 1
;PRINT, 'size (imgld7): ', SIZE (imgld7)

ldmin1 = MIN (imgld1, max=ldmax1)
;ldmin2 = MIN (imgld2, max=ldmax2)
;ldmin3 = MIN (imgld3, max=ldmax3)
;ldmin4 = MIN (imgld4, max=ldmax4)
;ldmin5 = MIN (imgld5, max=ldmax5)
ldmin6 = MIN (imgld6, max=ldmax6)
;ldmin7 = MIN (imgld7, max=ldmax7)

PRINT, 'ldmin1/ldmax1: ', ldmin1, ldmax1
;PRINT, 'ldmin2/ldmax2: ', ldmin2, ldmax2
;PRINT, 'ldmin3/ldmax3: ', ldmin3, ldmax3
;PRINT, 'ldmin4/ldmax4: ', ldmin4, ldmax4
;PRINT, 'ldmin5/ldmax5: ', ldmin5, ldmax5
PRINT, 'ldmin6/ldmax6: ', ldmin6, ldmax6
;PRINT, 'ldmin7/ldmax7: ', ldmin7, ldmax7

;--- Display the limb darkening correction images.

tvscl, imgld1
xyouts, 50, 50, 'Limb Darkening Correction B3', /device
wait, 1

;tvscl, imgld2
;xyouts, 50, 50, 'Limb Darkening Correction B2', /device
;wait, 1

;tvscl, imgld3
;xyouts, 50, 50, 'Limb Darkening Correction B1', /device
;wait, 1

;tvscl, imgld4
;xyouts, 50, 50, 'Limb Darkening Correction He', /device
;wait, 1

;tvscl, imgld5
;xyouts, 50, 50, 'Limb Darkening Correction R1', /device
;wait, 1

tvscl, imgld6
xyouts, 50, 50, 'Limb Darkening Correction R2', /device
wait, 1

;tvscl, imgld7
;xyouts, 50, 50, 'Limb Darkening Correction R3', /device
;wait, 1

;--- Doppler correction image (B3 filter only).

dc_name1 = syr1 + 'd' + sdoy1 + '.' + hour1 + minute1 + second1 + '.' + $
	   code_obs1 + '.dc'
imgdc1 = FLTARR (xdimsq, ydimsq)
CLOSE, 1
OPENR, 1, dc_name1
READU, 1, imgdc1                      ; 936 x 936 floating point pixels.
CLOSE, 1
;plot, imgdc1 [*, 468], title='B3 Equatorial doppler correction'
;wait, 3

lct, '/home/stanger/color/dif.lut'
tvscl, imgdc1
xyouts, 50, 50, 'Doppler Correction B3', /device
wait, 1

;--- Remove limb darkening.

;img1f = img1f * imgld1
;img2f = img2f * imgld2
;img3f = img3f * imgld3
;img4f = img4f * imgld4
;img5f = img5f * imgld5
;img6f = img6f * imgld6
;img7f = img7f * imgld7

;--- Use dsr limb darkening for dsr image.
;--- Use ssr limb darkening for all other images.

img1f = img1f * imgld1 + imgdc1
img2f = img2f * imgld6
img3f = img3f * imgld6
img4f = img4f * imgld6
img5f = img5f * imgld6
img6f = img6f * imgld6
img7f = img7f * imgld6

;--- Display the limb-darkening corrected images.

lct, '/home/stanger/color/chipcol.lut'
tvscl, img1f
xyouts, 50, 50, 'B3 Heliocentric LD removed', /device
wait, 1
tvscl, img2f
xyouts, 50, 50, 'B2 Heliocentric LD removed', /device
wait, 1
tvscl, img3f
xyouts, 50, 50, 'B1 Heliocentric LD removed', /device
wait, 1
tvscl, img4f
xyouts, 50, 50, 'He Heliocentric LD removed', /device
wait, 1
tvscl, img5f
xyouts, 50, 50, 'R1 Heliocentric LD removed', /device
wait, 1
tvscl, img6f
xyouts, 50, 50, 'R2 Heliocentric LD removed', /device
wait, 1
tvscl, img7f
xyouts, 50, 50, 'R3 Heliocentric LD removed', /device
wait, 1

;---  Update FITS header to indicate limb darkening correction.

fxaddpar, hdu1, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hdu2, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hdu3, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hdu4, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hdu5, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hdu6, 'HISTORY', 'Limb darkening has been removed.'
fxaddpar, hdu7, 'HISTORY', 'Limb darkening has been removed.'

;--- Create filenames for limb darkening corrected images.

file1c = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.sq.ld.fts'

file2c = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.sq.ld.fts'

file3c = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.sq.ld.fts'

file4c = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.sq.ld.fts'

file5c = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.sq.ld.fts'

file6c = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.sq.ld.fts'

file7c = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.sq.ld.fts'

;--- Write square, limb darkening-corrected images to disk in FITS format.

fxaddpar, hdu1, 'TYPE-DP', 'LD REM'
fxaddpar, hdu2, 'TYPE-DP', 'LD REM'
fxaddpar, hdu3, 'TYPE-DP', 'LD REM'
fxaddpar, hdu4, 'TYPE-DP', 'LD REM'
fxaddpar, hdu5, 'TYPE-DP', 'LD REM'
fxaddpar, hdu6, 'TYPE-DP', 'LD REM'
fxaddpar, hdu7, 'TYPE-DP', 'LD REM'

writefits, file1c, fix (img1f), hdu1
writefits, file2c, fix (img2f), hdu2
writefits, file3c, fix (img3f), hdu3
writefits, file4c, fix (img4f), hdu4
writefits, file5c, fix (img5f), hdu5
writefits, file6c, fix (img6f), hdu6
writefits, file7c, fix (img7f), hdu7

;--- Find average of brightest 10% pixels at disk center.

icen1 = cenbrt (img1f)
icen2 = cenbrt (img2f)
icen3 = cenbrt (img3f)
icen4 = cenbrt (img4f)
icen5 = cenbrt (img5f)
icen6 = cenbrt (img6f)
icen7 = cenbrt (img7f)

;--- Determine continuum brightness.

icont = icen2 / 0.97

PRINT, 'icont: ', icont

;--- Find brightest pixels (top 20%) in Helium-1 image.

brtloc = brtpix (img4f, pixrs, 20.0, ixbeg, ixend, iybeg, iyend)

sizeloc = size (brtloc)				; Vector array.
PRINT, 'size (brtloc): ', sizeloc
nloc = LONG (sizeloc [3])

;--- Extract a horizonal "band" across the equator of each image (except He-I).

xdimband = long (ixend - ixbeg + 1)
ydimband = long (iyend - iybeg + 1)
npxband  = xdimband * ydimband

band1 = FLTARR (xdimband, ydimband)
band2 = FLTARR (xdimband, ydimband)
band3 = FLTARR (xdimband, ydimband)
band4 = FLTARR (xdimband, ydimband)
band5 = FLTARR (xdimband, ydimband)
band6 = FLTARR (xdimband, ydimband)
band7 = FLTARR (xdimband, ydimband)

band1 = img1f [ixbeg:ixend, iybeg:iyend]
band2 = img2f [ixbeg:ixend, iybeg:iyend]
band3 = img3f [ixbeg:ixend, iybeg:iyend]
band4 = img4f [ixbeg:ixend, iybeg:iyend]
band5 = img5f [ixbeg:ixend, iybeg:iyend]
band6 = img6f [ixbeg:ixend, iybeg:iyend]
band7 = img7f [ixbeg:ixend, iybeg:iyend]

;--- Use brightest He-I pixel locations to compute average brightness
;--- for each filter (other than He-I).

brtavg1 = TOTAL (band1 (brtloc)) / nloc
brtavg2 = TOTAL (band2 (brtloc)) / nloc
brtavg3 = TOTAL (band3 (brtloc)) / nloc

brtavg4 = TOTAL (band4 (brtloc)) / nloc

brtavg5 = TOTAL (band5 (brtloc)) / nloc
brtavg6 = TOTAL (band6 (brtloc)) / nloc
brtavg7 = TOTAL (band7 (brtloc)) / nloc

PRINT, 'Average raw brightness: (for brightest 20% pixels)'
PRINT, 'brtavg1: ', brtavg1
PRINT, 'brtavg2: ', brtavg2
PRINT, 'brtavg3: ', brtavg3
PRINT, 'brtavg4: ', brtavg4
PRINT, 'brtavg5: ', brtavg5
PRINT, 'brtavg6: ', brtavg6
PRINT, 'brtavg7: ', brtavg7

;--- Compute normalization correction terms.

nc1 = icont - brtavg1
nc2 = icont - brtavg2
nc6 = icont - brtavg6
nc7 = icont - brtavg7

nc3 = icont - brtavg3			; Simple normalization.
nc4 = icont - brtavg4			; Simple normalization.
nc5 = icont - brtavg5			; Simple normalization.

;nc3 = 0.8 * nc2			; Theoretical normalization.
;nc4 = 0.4 * nc2 + 0.04 * nc6		; Theoretical normalization.
;nc5 = 0.2 * nc2 + 0.6  * nc6		; Theoretical normalization.

;nc3 = 0.745 * nc2			; Experimental normalization.
;nc4 = 0.74  * nc2 + 0.075 * nc6	; Experimental normalization.
;nc5 = 0.07  * nc2 + 0.270 * nc6	; Experimental normalization.

normavg1 = brtavg1 + nc1
normavg2 = brtavg2 + nc2
normavg3 = brtavg3 + nc3
normavg4 = brtavg4 + nc4
normavg5 = brtavg5 + nc5
normavg6 = brtavg6 + nc6
normavg7 = brtavg7 + nc7

PRINT, 'Average normalized brightness: (for brightest 20% pixels)'
PRINT, 'normavg1: ', normavg1
PRINT, 'normavg2: ', normavg2
PRINT, 'normavg3: ', normavg3
PRINT, 'normavg4: ', normavg4
PRINT, 'normavg5: ', normavg5
PRINT, 'normavg6: ', normavg6
PRINT, 'normavg7: ', normavg7

PRINT, 'Normalization corrections (additive): (for brightest 20% pixels)'
PRINT, 'nc1: ', nc1
PRINT, 'nc2: ', nc2
PRINT, 'nc3: ', nc3
PRINT, 'nc4: ', nc4
PRINT, 'nc5: ', nc5
PRINT, 'nc6: ', nc6
PRINT, 'nc7: ', nc7

img1f = img1f + nc1
img2f = img2f + nc2
img3f = img3f + nc3
img4f = img4f + nc4
img5f = img5f + nc5
img6f = img6f + nc6
img7f = img7f + nc7

;--- Display normalized images & label accordingly.

tvscl, img1f
xyouts, 50, 50, 'B3 Norm', /device
wait, 1
tvscl, img2f
xyouts, 50, 50, 'B2 Norm', /device
wait, 1
tvscl, img3f
xyouts, 50, 50, 'B1 Norm', /device
wait, 1
tvscl, img4f
xyouts, 50, 50, 'He Norm', /device
wait, 1
tvscl, img5f
xyouts, 50, 50, 'R1 Norm', /device
wait, 1
tvscl, img6f
xyouts, 50, 50, 'R2 Norm', /device
wait, 1
tvscl, img7f
xyouts, 50, 50, 'R3 Norm', /device
wait, 1

;--- Create filenames for output normalized images.

file1n = year1 + month1  + day1    + '.' +	$
	 hour1 + minute1 + second1 + '.' +	$
	 'chp.' + code_obs1 + '.norm.fts'

file2n = year2 + month2  + day2    + '.' +	$
	 hour2 + minute2 + second2 + '.' +	$
	 'chp.' + code_obs2 + '.norm.fts'

file3n = year3 + month3  + day3    + '.' +	$
	 hour3 + minute3 + second3 + '.' +	$
	 'chp.' + code_obs3 + '.norm.fts'

file4n = year4 + month4  + day4    + '.' +	$
	 hour4 + minute4 + second4 + '.' +	$
	 'chp.' + code_obs4 + '.norm.fts'

file5n = year5 + month5  + day5    + '.' +	$
	 hour5 + minute5 + second5 + '.' +	$
	 'chp.' + code_obs5 + '.norm.fts'

file6n = year6 + month6  + day6    + '.' +	$
	 hour6 + minute6 + second6 + '.' +	$
	 'chp.' + code_obs6 + '.norm.fts'

file7n = year7 + month7  + day7    + '.' +	$
	 hour7 + minute7 + second7 + '.' +	$
	 'chp.' + code_obs7 + '.norm.fts'

;--- Write normalized images to disk.

imin1  = MIN (img1f, max = imax1)
imin2  = MIN (img2f, max = imax2)
imin3  = MIN (img3f, max = imax3)
imin4  = MIN (img4f, max = imax4)
imin5  = MIN (img5f, max = imax5)
imin6  = MIN (img6f, max = imax6)
imin7  = MIN (img7f, max = imax7)

fxaddpar, hdu1, 'TYPE-DP', 'NORM'
fxaddpar, hdu2, 'TYPE-DP', 'NORM'
fxaddpar, hdu3, 'TYPE-DP', 'NORM'
fxaddpar, hdu4, 'TYPE-DP', 'NORM'
fxaddpar, hdu5, 'TYPE-DP', 'NORM'
fxaddpar, hdu6, 'TYPE-DP', 'NORM'
fxaddpar, hdu7, 'TYPE-DP', 'NORM'

fxaddpar, hdu1, 'DATAMIN', imin1
fxaddpar, hdu1, 'DATAMAX', imax1
fxaddpar, hdu1, 'DISPMIN', imin1
fxaddpar, hdu1, 'DISPMAX', imax1
fxaddpar, hdu2, 'DATAMIN', imin2
fxaddpar, hdu2, 'DATAMAX', imax2
fxaddpar, hdu2, 'DISPMIN', imin2
fxaddpar, hdu2, 'DISPMAX', imax2
fxaddpar, hdu3, 'DATAMIN', imin3
fxaddpar, hdu3, 'DATAMAX', imax3
fxaddpar, hdu3, 'DISPMIN', imin3
fxaddpar, hdu3, 'DISPMAX', imax3
fxaddpar, hdu4, 'DATAMIN', imin4
fxaddpar, hdu4, 'DATAMAX', imax4
fxaddpar, hdu4, 'DISPMIN', imin4
fxaddpar, hdu4, 'DISPMAX', imax4
fxaddpar, hdu5, 'DATAMIN', imin5
fxaddpar, hdu5, 'DATAMAX', imax5
fxaddpar, hdu5, 'DISPMIN', imin5
fxaddpar, hdu5, 'DISPMAX', imax5
fxaddpar, hdu6, 'DATAMIN', imin6
fxaddpar, hdu6, 'DATAMAX', imax6
fxaddpar, hdu6, 'DISPMIN', imin6
fxaddpar, hdu6, 'DISPMAX', imax6
fxaddpar, hdu7, 'DATAMIN', imin7
fxaddpar, hdu7, 'DATAMAX', imax7
fxaddpar, hdu7, 'DISPMIN', imin7
fxaddpar, hdu7, 'DISPMAX', imax7

fxaddpar, hdu1, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu2, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu3, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu4, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu5, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu6, 'HISTORY', 'NORMALIZED BRIGHTNESS'
fxaddpar, hdu7, 'HISTORY', 'NORMALIZED BRIGHTNESS'

writefits, file1n, fix (img1f), hdu1
writefits, file2n, fix (img2f), hdu2
writefits, file3n, fix (img3f), hdu3
writefits, file4n, fix (img4f), hdu4
writefits, file5n, fix (img5f), hdu5
writefits, file6n, fix (img6f), hdu6
writefits, file7n, fix (img7f), hdu7

;--- Compute He-I image by subtracting continuum from on-band image.

imghe1 = img4f - icont
he1min = MIN (imghe1, max=he1max)
hduhe1 = hdu4
fxaddpar, hduhe1, 'TYPE-DP', 'CAL'
fxaddpar, hduhe1, 'DATAMIN', he1min
fxaddpar, hduhe1, 'DATAMAX', he1max
fxaddpar, hduhe1, 'DISPMIN', he1min
fxaddpar, hduhe1, 'DISPMAX', he1max
writefits, 'he1.fts', fix (imghe1), hduhe1

tvscl, imghe1
xyouts, 50, 50, 'He-I', /device
wait, 1

;--- Compute average of all 7 normalized images.

imgavg = (img1f + img2f + img3f + img4f + img5f + img6f + img7f) / 7.0
tvscl, imgavg
xyouts, 50, 50, 'Average Normalized', /device
wait, 1

;PRINT, 'size (img): ', size (imgavg)

imin = MIN (imgavg, max=imax)

PRINT, 'average image min/max: ', imin, imax

;--- Compute velocity image.

imgdif0 = 1.0 - ( imgavg / icont)
dif0min = MIN (imgdif0, max=dif0max)
PRINT, 'dif0min/dif0max: ', dif0min, dif0max

imgdif1 = (img3f - img5f) / icont
imgdif2 = (img2f - img6f) / icont
imgdif3 = (img1f - img7f) / icont

d1min = MIN (imgdif1, max=d1max)
d2min = MIN (imgdif2, max=d2max)
d3min = MIN (imgdif3, max=d3max)

PRINT, 'd1min/d1max: ', d1min, d1max
PRINT, 'd2min/d2max: ', d2min, d2max
PRINT, 'd3min/d3max: ', d3min, d3max

;--- Display "component images" of the velocity image equation.

lct, '/home/stanger/color/dif.lut'
if (abs (dif0min) LT abs (dif0max)) then		$
   BEGIN
   rmin =   dif0min
   rmax = - dif0min
   END							$
else							$
   BEGIN
   rmin = - dif0max
   rmax =   dif0max
   END
imgb = bytscl (imgdif0, min=rmin, max=rmax, top=249)

;tvscl, imgdif0
tv, imgb
xyouts, 50, 50, '1.0 - Iave/Icont', /device
wait, 1

hdudif0 = hdu4
fxaddpar, hdudif0, 'TYPE-OBS', '1.0-IAVE/ICONT'
fxaddpar, hdudif0, 'TYPE-DP',  'NORM'
fxaddpar, hdudif0, 'DATAMIN', dif0min
fxaddpar, hdudif0, 'DATAMAX', dif0max
fxaddpar, hdudif0, 'DISPMIN', dif0min
fxaddpar, hdudif0, 'DISPMAX', dif0max
filedif0 = year4 + month4  + day4    + '.' +	$
	   hour4 + minute4 + second4 + '.' +	$
	   'chp' + '.dif0.fts'
writefits, filedif0, imgdif0, hdudif0
writefits, 'imgdif0.fts', imgdif0, hdudif0

dif1min = MIN (imgdif1, max=dif1max)
if (abs (dif1min) LT abs (dif1max)) then		$
   BEGIN
   d1min =   dif1min
   d1max = - dif1min
   END							$
else							$
   BEGIN
   d1min = - dif1max
   d1max =   dif1max
   END
imgb = bytscl (imgdif1, min=d1min, max=d1max, top=249)

tvscl, imgdif1
xyouts, 50, 50, 'B1 - R1', /device
wait, 1
hdudif1 = hdu3
fxaddpar, hdudif1, 'TYPE-OBS', '(B1-R1)/ICONT'
fxaddpar, hdudif1, 'TYPE-DP',  'NORM'
fxaddpar, hdudif1, 'DATAMIN', dif1min
fxaddpar, hdudif1, 'DATAMAX', dif1max
fxaddpar, hdudif1, 'DISPMIN', dif1min
fxaddpar, hdudif1, 'DISPMAX', dif1max
filedif1 = year3 + month3  + day3    + '.' +	$
	   hour3 + minute3 + second3 + '.' +	$
	   'chp' + '.dif1.fts'
writefits, filedif1, imgdif1, hdudif1
writefits, 'imgdif1.fts', imgdif1, hdudif1

dif2min = MIN (imgdif2, max=dif2max)
if (abs (dif2min) LT abs (dif2max)) then		$
   BEGIN
   d2min =   dif2min
   d2max = - dif2min
   END							$
else							$
   BEGIN
   d2min = - dif2max
   d2max =   dif2max
   END
imgb = bytscl (imgdif2, min=d2min, max=d2max, top=249)

;tvscl, imgdif2
tv, imgb
xyouts, 50, 50, 'B2 - R2', /device
wait, 1
hdudif2 = hdu2
fxaddpar, hdudif2, 'TYPE-OBS', '(B2-R2)/ICONT'
fxaddpar, hdudif2, 'TYPE-DP',  'NORM'
fxaddpar, hdudif2, 'DATAMIN', dif2min
fxaddpar, hdudif2, 'DATAMAX', dif2max
fxaddpar, hdudif2, 'DISPMIN', dif2min
fxaddpar, hdudif2, 'DISPMAX', dif2max
filedif2 = year2 + month2  + day2    + '.' +	$
	   hour2 + minute2 + second2 + '.' +	$
	   'chp' + '.dif2.fts'
writefits, filedif2, imgdif2, hdudif2
writefits, 'imgdif2.fts', imgdif2, hdudif2

dif3min = MIN (imgdif3, max=dif3max)
if (abs (dif3min) LT abs (dif3max)) then		$
   BEGIN
   d3min =   dif3min
   d3max = - dif3min
   END							$
else							$
   BEGIN
   d3min = - dif3max
   d3max =   dif3max
   END
imgb = bytscl (imgdif3, min=d3min, max=d3max, top=249)

;tvscl, imgdif3
tv, imgb
xyouts, 50, 50, 'B3 - R3', /device
wait, 1
hdudif3 = hdu1
fxaddpar, hdudif3, 'TYPE-OBS', '(B3-R3)/ICONT'
fxaddpar, hdudif3, 'TYPE-DP',  'NORM'
fxaddpar, hdudif3, 'DATAMIN', dif3min
fxaddpar, hdudif3, 'DATAMAX', dif3max
fxaddpar, hdudif3, 'DISPMIN', dif3min
fxaddpar, hdudif3, 'DISPMAX', dif3max
filedif3 = year1 + month1  + day1    + '.' +	$
	   hour1 + minute1 + second1 + '.' +	$
	   'chp' + '.dif3.fts'
writefits, filedif3, imgdif3, hdudif3
writefits, 'imgdif3.fts', imgdif3, hdudif3

;--- Now, put the pieces together to get the velocity image.

dif0loc = WHERE (abs (imgdif0) LT 0.01, ndif0)
sizdif0 = size (imgdif0)
print, 'size (imgdif0): ', size (imgdif0)
print, 'ndif0: ', ndif0
print, 'sizdif0 [4]: ', sizdif0 [4]
print, 'fraction of imgdif0 < 0.01 : ', float (ndif0) / float (sizdif0 [4]) 

imgvel =   ( 4.0 /  imgdif0 )			$
	 * (        imgdif1 			$
	 +   2.0 *  imgdif2 			$
	 +   3.5 *  imgdif3 			$
	 +   6.0 * (imgdif0 ^ 1.4) )

imgvel [dif0loc] = 0.0		;--- Set velocity to zero where imgdif0 < 0.01

;--- Convert to short integer.

imgvel = fix (imgvel)

;--- Display velocity image.

velmin = MIN (imgvel, max=velmax)
PRINT, 'velmin/velmax: ', velmin, velmax

if (abs (velmin) LT abs (velmax)) then			$
   BEGIN
   vmin =   velmin
   vmax = - velmin
   END							$
else							$
   BEGIN
   vmin = - velmax
   vmax =   velmax
   END
imgvelb = bytscl (imgvel, min=vmin, max=vmax, top=249)

;tvscl, imgvel
tv, imgb
xyouts, 50, 50, 'Velocity', /device
wait, 1

;--- Write velocity image to disk as a FITS file.

hduvel = hdu4
fxaddpar, hduvel, 'TYPE-OBS', 'VELOCITY'
fxaddpar, hduvel, 'TYPE-DP',  'CAL'
fxaddpar, hduvel, 'BUNIT',    'KM/SEC'
fxaddpar, hduvel, 'DATAMIN', velmin
fxaddpar, hduvel, 'DATAMAX', velmax
fxaddpar, hduvel, 'DISPMIN', velmin
fxaddpar, hduvel, 'DISPMAX', velmax
fxaddpar, hduvel, 'TYPE-OBS', 'HEI-VEL'
writefits, 'vel.fts', imgvel, hduvel

END
