;+ chipreg.pro
;
; Register image by co-aligning sun center with array center.
;
; Use CRPIX1 & CRPIX2 values for sun center.
;
; Andrew L. Stanger   HAO/NCAR   6 April 2001
;-

PRO chipreg, file_name

!order = 1			; Display image top to bottom.
border = 6

;--- Read image & extension.

img = readfits (file_name, hdu, /noscale)
hsk = readfits (file_name, ext, exten_no=1, /noscale)

imin = MIN (img, max=imax)
print, 'imin/imax: ', imin, imax

xdim     = fxpar (hdu, 'NAXIS1')
ydim     = fxpar (hdu, 'NAXIS2')
xcen     = fxpar (hdu, 'CRPIX1')
ycen     = fxpar (hdu, 'CRPIX2')
nsum     = fxpar (hdu, 'NIMSUM')
crradius = fxpar (hdu, 'CRRADIUS')

;--- Compute array center.

xc = (xdim - 1) * 0.5
yc = (ydim - 1) * 0.5

xsh = xc - xcen
ysh = yc - ycen

print, 'xsh/ysh: ', xsh, ysh

xcen2 = xcen / 2.0
ycen2 = ycen / 2.0
crradius2 = crradius / 2.0

; imgc = intarr (xdim - 2 * border, ydim - 2 * border)
; imgc = img [border:xdim-1-border, border:ydim-1-border]
; cmin = MIN (imgc, max=cmax)
; print, 'cmin/cmax: ', cmin, cmax

;--- Display image.

xdim2 = xdim / 2
ydim2 = ydim / 2

img2 = rebin (img, xdim2, ydim2)

WINDOW, xsize = xdim2, ysize=ydim2

tv, bytscl (img2, min=cmin, max=cmax)

XYOUTS, 10, 10, 'ORIGINAL IMAGE', /device

;--- Translate image using average x/y shifts.

imgs = intarr (xdim, ydim)

FOR iy = 0, ydim - 1 DO	$
BEGIN
   FOR ix = 0, xdim - 1 DO	$
   BEGIN
   imgs [ix, iy] = INTERPOLATE (img, ix - xsh, iy - ysh, MISSING=0)
   ENDFOR
ENDFOR

;--- Display image.

imgs2 = REBIN (imgs, xdim2, ydim2)

tv, bytscl (img2, min=cmin, max=cmax)
XYOUTS, 10, 10, 'REGISTERED IMAGE', /device

;--- Write shifted image to disk.

fpos = STRPOS (file_name, '.fts')
out_name = STRMID (file_name, 0, fpos) + '.reg.fts'

hdush = hdu
fxaddpar, hdush, 'CRPIX1', xc
fxaddpar, hdush, 'CRPIX2', yc

writefits, out_name, img, hdush
writefits, out_name, hsk, ext, /APPEND

END
