;+
; NAME
;	chipsqlo
;
; PURPOSE
;	convert ACOS/CHIP images to 500x500 pixels.
;
; SYNTAX
;	chipsqlo, 'chipimages.ls'
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   15 May 1996
;	6 June 1996: modify cdelt1 & cdelt2 parameter values in output header.
;-

PRO chipsqlo, filelist

   file = ''
;   print, 'filelist: ', filelist

   GET_LUN, lun
;   print, 'lun: ', lun
   OPENR, lun, filelist
   WHILE (NOT EOF (lun)) DO			$
   BEGIN
      ;--- Read file name from list.

      READF, lun, file, FORMAT = '(A)'
      PRINT, 'Read file: ', file

      ;--- Read image from FITS file.
      ;--- Get information from FITS header.

      img = readfits (file, hdu)
      xdim = fxpar (hdu, 'NAXIS1')
      ydim = fxpar (hdu, 'NAXIS2')
      xcen = fxpar (hdu, 'CRPIX1')
      ycen = fxpar (hdu, 'CRPIX2')
      cdelt1 = fxpar (hdu, 'CDELT1')
      cdelt2 = fxpar (hdu, 'CDELT2')
      crradius = fxpar (hdu, 'CRRADIUS')
      datamin = fxpar (hdu, 'DATAMIN')
      datamax = fxpar (hdu, 'DATAMAX')
      dispmin = fxpar (hdu, 'DISPMIN')
      dispmax = fxpar (hdu, 'DISPMAX')
      PRINT, 'datamin/datamax: ', datamin, datamax
      PRINT, 'dispmin/dispmax: ', dispmin, dispmax

      ;--- Crop image to 1000x1000.

      xbeg = (xdim - 1000) / 2
      ybeg = (ydim - 1000) / 2
      imgsq = intarr (1000, 1000)
      imgsq = img (xbeg : xbeg +  999, ybeg : ybeg +  999)
      minsq = min (imgsq)
      maxsq = max (imgsq)

      ;--- Reduce size to 500x500.

      imglo = rebin (imgsq, 500, 500)

      ;--- Update FITS header.

      xcen = 249.5
      ycen = 249.5
;      xcen = (xcen - xbeg) / 2
;      ycen = (ycen - ybeg) / 2
      minlo = min (imglo)
      maxlo = max (imglo)
      PRINT, 'minsq/maxsq: ', minsq, maxsq
      PRINT, 'minlo/maxlo: ', minlo, maxlo
      fxaddpar, hdu, 'CRPIX1', xcen
      fxaddpar, hdu, 'CRPIX2', ycen
      fxaddpar, hdu, 'CDELT1', cdelt1 * 2.0
      fxaddpar, hdu, 'CDELT2', cdelt2 * 2.0
      fxaddpar, hdu, 'CRRADIUS', crradius / 2.0

      ;--- Write square image to disk.

      extpos = strpos (file, '.fts')
;      PRINT, 'extpos: ', extpos
      outfile = strmid (file, 0, extpos) + '.sqlo.fts'
      PRINT, 'outfile: ', outfile
      writefits, outfile, imglo, hdu
   END
   CLOSE, LUN
   
END
