;+
; NAME
;	chipsum4.pro
;
; PURPOSE
;	Add 4 CHIP images together.
;
; SYNTAX
;	chipsum4, 'list'
;
;	list	contains a list of raw CHIP images (FITS format):
;		sr1.fts	solar image
;		sr2.fts	solar image
;		sr3.fts	solar image
;		sr4.fts	solar image
; OUPUT	sr0.fts
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR    12 April 2001
;-

PRO chipsum4, filelist

    file = ''
    file1 = ''
    file2 = ''
    file3 = ''
    file4 = ''
    nexp  = 4

;   print, 'filelist: ', filelist

    GET_LUN, lun
    OPENR, lun, filelist
    READF, lun, file1, FORMAT = '(A)'

    img1 = readfits (file1, hdu1, /noscale)
    hsk1 = readfits (file1, ext1, exten_no=1, /noscale)

    READF, lun, file2, FORMAT = '(A)'

    img2 = readfits (file2, hdu2, /noscale)
    hsk2 = readfits (file2, ext2, exten_no=1, /noscale)

    READF, lun, file3, FORMAT = '(A)'

    img3 = readfits (file3, hdu3, /noscale)
    hsk3 = readfits (file3, ext3, exten_no=1, /noscale)

    READF, lun, file4, FORMAT = '(A)'

    img4 = readfits (file4, hdu4, /noscale)
    hsk4 = readfits (file4, ext4, exten_no=1, /noscale)

    CLOSE, lun

    xdim     = fxpar (hdu1, 'NAXIS1')
    ydim     = fxpar (hdu1, 'NAXIS2')
    code_obs = fxpar (hdu1, 'CODE-OBS', comment=codecom) 
    date     = fxpar (hdu1, 'DATE-OBS')
    time     = fxpar (hdu1, 'TIME-OBS')
    nimsum   = fxpar (hdu1, 'NIMSUM',   comment=expcom)
    datamin  = fxpar (hdu1, 'DATAMIN',  comment=datamincom)
    datamax  = fxpar (hdu1, 'DATAMAX',  comment=datamaxcom)
    dispmin  = fxpar (hdu1, 'DISPMIN',  comment=dispmincom)
    dispmax  = fxpar (hdu1, 'DISPMAX',  comment=dispmaxcom)
    code_obs = STRLOWCASE (code_obs)
    wsr      = STRMID (code_obs, 0, 3)
    expnum   = STRMID (code_obs, 3, 1)

    print, 'codecom: ', codecom
    print, 'expcom : ', expcom

    ;--- Decipher date & time.
    
    syear    = STRMID (date, 0, 4)
    smonth   = STRMID (date, 5, 2)
    sday     = STRMID (date, 8, 2)
    shour    = STRMID (time, 0, 2)
    sminute  = STRMID (time, 3, 2)
    ssecond  = STRMID (time, 6, 2)
    
    print, 'syear: ', syear, ' smonth: ', smonth, ' sday: ', sday
    print, 'shour: ', shour, ' sminute: ', sminute, ' ssecond: ', ssecond
    
    ;--- Add images.

    imgs = img1 + img2 + img3 + img3
    imgs = FIX (imgs)			; Convert to short integer.

    dispmin = dispmin * 4
    dispmax = dispmax * 4
    datamin = datamin * 4
    datamax = datamax * 4

    ;--- Display summed image.

    WINDOW, xsize=xdim/2, ysize=ydim/2
    lct, '/home/stanger/color/chip1.lut'	; Load color table.

    imgs2 = REBIN (imgs, xdim/2, ydim/2)
    TV, BYTSCL (imgs2, min=dispmin, max=dispmax)
    XYOUTS, 10, 30, 'SUMMED IMAGE', /device
    WAIT, 3

;    type_dps  = 'B&G ADJ'	; bias/gain correction applied.

;    extloc   = STRPOS (file1, code_obs)
;    files  = STRMID (file1, 0, extloc) + wsr + '0.fts'

;    extloc = STRPOS (file1, 'r')
;    fext   = STRMID (file1, extloc+1, strlen (files)-extloc+1)
;    files  = files + fext
;    print, 'files: ', files

    hdus = hdu1
    exts = ext1
    hsks   = hsk1

    code_obs3 = STRMID (code_obs, 0, 3)		; Get first two characters.
    code_obs4 = STRMID (code_obs, 3, 1)		; Exposure number.
    code_obss = code_obs3 + '0'			; replace 4td char with "0".
    code_obss = STRUPCASE (STRTRIM (code_obss, 2))	; remove any blanks.

    fxaddpar, hdus, 'CODE-OBS', code_obss, codecom	; update CODE-OBS.
    fxaddpar, hdus, 'NIMSUM', nexp, expcom	; update NIMSUM.
    fxaddpar, hdus, 'DATAMIN', datamin, datamincom
    fxaddpar, hdus, 'DATAMAX', datamax, datamaxcom
    fxaddpar, hdus, 'DISPMIN', dispmin, dispmincom
    fxaddpar, hdus, 'DISPMAX', dispmax, dispmaxcom
;    fxaddpar, hdus, 'TYPE-DP',  type_dps	; update TYPE-DP.

    files = syear + smonth + sday + '.' + shour + sminute + ssecond + $
               '.chp.' + STRLOWCASE (wsr) + '0.fts'
    PRINT, 'files: ', files
    
    writefits, files, imgs, hdus
    writefits, files, hsks, exts, /append

END
