;+
; NAME		do_fit2.pro
;
; PURPOSE	Fit a curve to the limb-darkening profile.
;
; SYNTAX	do_fit2, rad_ang_file
;
; INPUT		rad_ang_file: Binary table (radius, angle) created by
;		C-language program (chip_ldark2.c).
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR
;
; HISTORY	Created 15 August 2000
;-

PRO do_fit2, rad_ang_file

nrad = 417
nang = 362

rsun = 417.79

r_dist  = findgen (nrad)
r_bright = fltarr (nrad)

b_fit   = fltarr (nrad, nang)
rad_ang = fltarr (nrad, nang)

print, 'radial scan file: ', rad_ang_file

;--- Read radial scan (2D: radius, angle).

OPENR, 11, rad_ang_file 
   READU, 11, rad_ang
CLOSE, 11

;--- Fit radial scan to limb darkening equation.

b_cen = 0.0 ;
ncen  = 0 ;

FOR iz = 0, nang - 1 DO					$
   BEGIN
   r_bright [*] = rad_ang [0:nrad-1, iz]
   b_fit [0:nrad-1, iz] = ld_fit2 (r_dist, r_bright , rsun, iz)

   b_cen = b_cen + b_fit [0, iz] + b_fit [1, iz] + b_fit [2, iz] + $
		  b_fit [3, iz] + b_fit [4, iz] + b_fit [5, iz] + $
		  b_fit [6, iz] + b_fit [7, iz] + b_fit [8, iz] + $
		  b_fit [9, iz]
   ncen = ncen + 10

   IF (iz / 30 * 30 EQ iz) THEN	$
   BEGIN
;    plot,  r_dist, r_bright,	$			; Plot radial scan.
;           title='Limb Darkening Radial Profile' +	$
;	    ' ' + strtrim (string (iz), 2) + ' deg.',	$
;           xtitle='Radial Distance [Rsun]',		$
;           ytitle='Brightness'
   WAIT, 1
   END

END

b_cen = b_cen / float (ncen)

print, 'b_cen: ', b_cen
b_cor = b_cen / b_fit 

nf = N_ELEMENTS (rad_ang_file)

OPENW, 11, rad_ang_file + '_fac'
   WRITEU, 11, b_cor
CLOSE, 11

END
