;+
; NAME		do_fit5.pro
;
; PURPOSE	Fit a curve to the limb-darkening profile.
;
; SYNTAX	do_fit5, radius_vs_angle_file
;
; INPUT		scanrad.tab : Binary table (radius, angle) created by
;		C-language program (chip_ldark2.c).
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR
;
; HISTORY	Created 31 August 2000
;-

PRO do_fit5, rad_ang_file

nrad = 414
nang = 362

rsun = 417.87
pang = -20.11
pang = 0.0

t_ang    = findgen (nang) - 1.0
t_bright = fltarr (nang)
r_dist  = findgen (nrad)
r_bright = fltarr (nrad)

b_fit   = fltarr (nrad, nang)
rad_ang = fltarr (nrad, nang)

print, 'radial scan file: ', rad_ang_file

;--- Read radial scan (2D: radius, angle).

OPENR, 11, rad_ang_file 
   READU, 11, rad_ang
CLOSE, 11

b_cen = 0.0 ;
ncen  = 0 ;

FOR iz = 0, nang - 1 DO					$
BEGIN
   b_cen = b_cen + b_fit [0, iz] + b_fit [1, iz] + b_fit [2, iz] + $
		  b_fit [3, iz] + b_fit [4, iz] + b_fit [5, iz] + $
		  b_fit [6, iz] + b_fit [7, iz] + b_fit [8, iz] + $
		  b_fit [9, iz]
   ncen = ncen + 10
END

b_cen = b_cen / float (ncen)

print, 'b_cen: ', b_cen

;--- Fit radial scan to limb darkening equation.

FOR iz = 0, nrad - 1 DO					$
   BEGIN
   t_bright [*] = rad_ang [iz, 0:nang-1]
   b_fit [iz, 0:nang-1] = ld_fit5 (t_ang, t_bright, pang, iz)

   IF (iz / 30 * 30 EQ iz) THEN	$
   BEGIN
;    plot,  r_dist, r_bright,	$			; Plot radial scan.
;           title='Limb Darkening Radial Profile' +	$
;	    ' ' + strtrim (string (iz), 2) + ' deg.',	$
;           xtitle='Radial Distance [Rsun]',		$
;           ytitle='Brightness'
   WAIT, 1
   END

END

b_cor = b_cen / b_fit 

nf = N_ELEMENTS (rad_ang_file)

OPENW, 11, rad_ang_file + '_fac'
   WRITEU, 11, b_cor
CLOSE, 11

END
