;+
; NAME		donut.pro
;
; PURPOSE	Compute equal area donuts of a circular disc.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR
;
; HISTORY	Created 12 Oct 2000
;
;-

PRO donut, img_fits_file, radius_fits_file

;--- Read FITS image.

img  = readfits (img_fits_file, hdu)
rimg = readfits (radius_fits_file, hdurad)

;--- Decode selected header items.

nx    = fxpar (hdu, 'NAXIS1')
ny    = fxpar (hdu, 'NAXIS2')
pixrs = fxpar (hdu, 'RSUN')

;--- Area of a circle:  pi * r^2
;--- Area of a donut:   pi * (r2^2 - r1^2)
;--- Area of a central (initial) disc of size 0.1 Rsun:  pi * 0.1 * pixrs

pi = 3.14159265
radius = pixrs * 0.1
area = pi * radius * radius
rsq = fix ( (area / pi) + 0.5 )
rsq = float (rsq)
rd = fltarr (100)

;--- Determine how many elements are needed for radius array.

FOR i = 0, 99 DO	$
    BEGIN
    radius = sqrt (area * (i + 1))
    IF (radius GT 1.0) THEN nrad = i
    END

;--- Declare storage for radial distance array.

rd = FLTARR (nrad)

;--- Compute radial distance limits for donuts.

FOR i = 0, nrad - 1 DO	$
    BEGIN
    rd [i] = sqrt (area * (i + 1))
    END

;--- Declare storage for donut pixel locations.

rloc = FLTARR (nrad)
rbrt = FLTARR (nrad)

;--- Determine pixel locations for each donut.

FOR i = 0, nrad - 2 DO	$
    BEGIN
    rloc = WHERE (rimg GE rd [i] AND rimg LT rd [i+1])
    print, 'size (rloc): ', size (rloc)
    size_rloc = size (rloc)
    ndonut = size_rloc [1]
    brtsort = img (rloc)		; Order according to brightness
    nbrt90  = LONG (0.9 * ndonut)
    brtdonut = TOTAL (brtsort [nbrt90:ndonut-1]) / (ndonut - nbrt90)
    rbrt [i] = brtdonut
    END

;--- Do last point as a special case.

rloc = WHERE (rimg GE rd [nrad-1] AND rimg LT 1.0)
size_rloc = size (rloc)
ndonut = size_rloc [1]
brtsort = img (rloc)
nbrt90  = LONG (0.9 * ndonut)
brtdonut = TOTAL (brtsort [nbrt90:ndonut-1]) / (ndonut - nbrt90)
rbrt [nrad - 1] = brtdonut

plot, rd, rbrt,					$
      title = 'CHIP radial profile',		$
      xtitle = 'Radial distance [Rsun]',	$
      ytitle = 'Brightness'

END
