;+
; fixcodeobsnum.pro
;
; Change the code_obs parameter in the FITS header.
; Also, rename the file.
;
; Andrew L. Stanger   HAO/NCAR   4 April 2001
;-

PRO fixcodeobsnum, fits_list


fits_file = ''
CLOSE, 11

OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO                  $
    BEGIN
    READF, 11, fits_file, FORMAT = '(A)'

    ;--- Read image, primary HDU, and extension HDU.

    img = readfits (fits_file, hdu, /noscale)
    ext = readfits (fits_file, hduext, /noscale, exten_no=1)

    ;--- Get information from primary HDU.
    
    date     = fxpar (hdu, 'DATE-OBS')
    time     = fxpar (hdu, 'TIME-OBS')
    code_obs = STRLOWCASE ( STRTRIM (fxpar (hdu, 'CODE-OBS'), 2) )
    
    print, 'date: ', date
    print, 'time: ', time
    print, 'code_obs: ', code_obs
    
    ;--- Decipher date & time.
    
    syear    = STRMID (date, 0, 4)
    smonth   = STRMID (date, 5, 2)
    sday     = STRMID (date, 8, 2)
    shour    = STRMID (time, 0, 2)
    sminute  = STRMID (time, 3, 2)
    ssecond  = STRMID (time, 6, 2)
    
    print, 'syear/smonth/sday: ', syear, smonth, sday
    print, 'shour/sminute/ssecond: ', shour, sminute, ssecond
    
    ;--- Add exposure number to code_obs.
    
    pos1 = -1
    pos2 = -1
    pos3 = -2
    pos4 = -1
    pos1 = STRPOS (fits_file, '1.fts')
    pos2 = STRPOS (fits_file, '2.fts')
    pos3 = STRPOS (fits_file, '3.fts')
    pos4 = STRPOS (fits_file, '4.fts')
    
    IF (pos1 GE 0) THEN code_obs = code_obs + '1'	; Add "1" to obs_code.
    IF (pos2 GE 0) THEN code_obs = code_obs + '2'	; Add "1" to obs_code.
    IF (pos3 GE 0) THEN code_obs = code_obs + '3'	; Add "1" to obs_code.
    IF (pos4 GE 0) THEN code_obs = code_obs + '4'	; Add "1" to obs_code.
    
    print, 'code_obs: ', code_obs
    
    ;--- Update CODE-OBS parameter in FITS header.
    
    fxaddpar, hdu, 'CODE-OBS', STRUPCASE (code_obs)	; Update 'CODE-OBS'.
    
    ;--- Write FITS image to disk.
    
    lsc_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
               '.chp.' + code_obs + '.fts'
    PRINT, 'lsc_name: ', lsc_name
    
    writefits, lsc_name, img, hdu
    writefits, lsc_name, ext, hduext, /APPEND
    END
CLOSE, 11
    
END
    
