;+
; fixhbr4.pro
;
; Change the code_obs parameter in the FITS header.
; Modify the Type code in the FITS extension.
; Also, rename the file.
;
; The "hbr.1", "hbr.2", "hbr.3", and "hbr.4" files have the
; WRONG "Type" code set in the binary header, and consequently,
; the "CODE-OBS" parameter in the FITS header is also incorrect.
;
; The "Type" is set to 241 (ddr) instead of 151 (hbr).
; This program fixes this problem.
;
; Andrew L. Stanger   HAO/NCAR   4 April 2001
;-

PRO fixhbr4, fits_list


fits_file = ''
CLOSE, 11

OPENR, 11, fits_list
WHILE (NOT EOF (1)) DO                  $
    BEGIN
    READF, 11, fits_file, FORMAT = '(A)'

    ;--- Read image, primary HDU, and extension HDU.

    img = readfits (fits_file, hdu, /noscale)
    hsk = readfits (fits_file, ext, /noscale, exten_no=1)

    ;--- Get information from primary HDU.
    
    date     = fxpar (hdu, 'DATE-OBS')
    time     = fxpar (hdu, 'TIME-OBS')
    code_obs = STRLOWCASE ( STRTRIM (fxpar (hdu, 'CODE-OBS'), 2) )
    
    print, 'date: ', date
    print, 'time: ', time
    print, 'code_obs: ', code_obs

    code_obs = "HBR"			; Fix obs_code.
    hsk [13] = 151			; Fix Type code.
    type_obs = "HELIUM-I"
    object   = "BIAS"
    
    ;--- Decipher date & time.
    
    syear    = STRMID (date, 0, 4)
    smonth   = STRMID (date, 5, 2)
    sday     = STRMID (date, 8, 2)
    shour    = STRMID (time, 0, 2)
    sminute  = STRMID (time, 3, 2)
    ssecond  = STRMID (time, 6, 2)
    
    print, 'syear/smonth/sday: ', syear, smonth, sday
    print, 'shour/sminute/ssecond: ', shour, sminute, ssecond
    
    ;--- Add exposure number to code_obs.
    
    pos1 = -1
    pos2 = -1
    pos3 = -2
    pos4 = -1
    pos1 = STRPOS (fits_file, '1.fts')
    pos2 = STRPOS (fits_file, '2.fts')
    pos3 = STRPOS (fits_file, '3.fts')
    pos4 = STRPOS (fits_file, '4.fts')
    
    IF (pos1 GE 0) THEN code_obs = code_obs + '1'	; Add "1" to obs_code.
    IF (pos2 GE 0) THEN code_obs = code_obs + '2'	; Add "1" to obs_code.
    IF (pos3 GE 0) THEN code_obs = code_obs + '3'	; Add "1" to obs_code.
    IF (pos4 GE 0) THEN code_obs = code_obs + '4'	; Add "1" to obs_code.
    
    print, 'code_obs: ', code_obs
    
    ;--- Update CODE-OBS parameter in FITS header.
    
    fxaddpar, hdu, 'CODE-OBS', STRUPCASE (code_obs)	; Update 'CODE-OBS'.
    fxaddpar, hdu, 'TYPE-OBS', STRUPCASE (type_obs)	; Update 'TYPE-OBS'.
    fxaddpar, hdu, 'OBJECT',   STRUPCASE (object  )	; Update 'OBJECT'.

    fxaddpar, ext, 'TYPE-OBS', STRUPCASE (type_obs)	; Update 'TYPE-OBS'.
    fxaddpar, ext, 'OBJECT',   STRUPCASE (object  )	; Update 'OBJECT'.
    
    ;--- Write FITS image to disk.
    
    lsc_name = syear + smonth + sday + '.' + shour + sminute + ssecond + $
               '.chp.' + STRLOWCASE (code_obs) + '.fts'
    PRINT, 'lsc_name: ', lsc_name
    
    writefits, lsc_name, img, hdu
    writefits, lsc_name, hsk, ext, /APPEND
    END
CLOSE, 11
    
END
    
