;+ fixlsdgains.pro
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MLSO/CHIP Helium-I instrument
;
;  Get extension from a diffuser file & add it to the corresponding "lsc" file.
; "list" contains the names of the files:
;     ddr.fts
;     ddr.lsc.fts
;     cdr.fts
;     cdr.lsc.fts
;     bdr.fts
;     bdr.lsc.fts
;     hdr.fts
;     hdr.lsc.fts
;     rdr.fts
;     rdr.lsc.fts
;     sdr.fts
;     sdr.lsc.fts
;     tdr.fts
;     tdr.lsc.fts
;
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; SYNTAX	fixlsdgains, 'list'
;		list: file containing a list of image names.
;		Each line contains the lsd & lsc file names.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   22 Mar 2001
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-

PRO fixlsdgains, list

numband = 7

filelsd = 'lsd.fts'
filelsc = 'lsc.fts'

;--- Open list file & read LSD bias image.

CLOSE, 11
OPENR, 11, list

LOADCT, 3

;--- Process light shaping diffuser (LSD) images.

FOR ii = 0, numband - 1 DO BEGIN
    READF, 11, filelsd
    READF, 11, filelsc
    print, 'filelsd: ', filelsd
    print, 'filelsc: ', filelsc
    lsd = readfits (filelsd, hdulsd, /noscale)
    ext = readfits (filelsd, hduext, exten_no=1, /noscale)
    lsc = readfits (filelsc, hdulsc, /noscale)

    xdim = fxpar (hdulsc, 'NAXIS1')
    ydim = fxpar (hdulsc, 'NAXIS2')

    xdim2 = xdim / 2
    ydim2 = ydim / 2
    WINDOW, xsize = xdim2, ysize = ydim2
    code_obs = strlowcase ( strtrim (fxpar (hdulsc, 'CODE-OBS'), 2) )
    print, 'code_obs: ...', code_obs, '...'

    print, 'hduext: '
    print, hduext

    ;--- Display image.

    lsd2 = REBIN (lsd, xdim2, ydim2)
    TVSCL, lsd2

    XYOUTS, 5, 5, 'LSD', /device

    ;--- Write ratio image to disk.

    writefits, filelsc, lsc, hdulsc
    writefits, filelsc, ext, hduext, /APPEND

ENDFOR
CLOSE, 11

END
