;+
; NAME	ld_fit1
;
; PURPOSE	Fit Limb darkening equation
;
; SYNTAX	f_bright = ld_fit1 (r_dist, r_bright, rsun, iz)
;
; INPUT     	r_dist: radial distance from sun center in pixels.
;		r_bright: pixel brightness.
;		rsun: number of pixels per solar radius.
;		iz: azimuthal angle
;
; OUTPUT	f_bright: pixel brightness according to equation fit.
;
; AUTHOR	Andrew L. Stanger   14 August 2000
;
; HISTORY	Equation acquired from Randy Meisner.
;-

FUNCTION ld_fit1, r_dist, r_bright, rsun, iz

rs = r_dist / rsun			; Convert to units of Rsun.

IF (iz / 30 * 30 EQ iz) THEN	$
plot,  rs, r_bright,	$			; Plot radial scan.
       title='Limb Darkening Radial Profile' +	$
	      ' ' + strtrim (string (iz), 2) + ' deg.',	$
       xtitle='Radial Distance [Rsun]',		$
       ytitle='Brightness'

nrad = N_ELEMENTS (rs)			; Number of elements in radial scan.

w = FLTARR (nrad)
w [*] = 0.5				; Assign weights.
w [ 0 :  9] = 10.0
w [10 : 19] =  5.0
w [nrad - 40 : nrad - 31] =  3.0
w [nrad - 30 : nrad - 21] =  7.0
w [nrad - 20 : nrad -  1] = 10.0

w [*] = 0.5				; Assign weights.
w [ 0 :  5] = 10.0
w [nrad -  5 : nrad -  1] = 10.0

w [*] = 1.0

a     = FLTARR (3)
sigma = FLTARR (3)

a [0] = r_bright [0]			; Initial guess for coefficients.
a [1] = 0.0
a [2] = 0.0

;--- Fit radial scan to limb-darkening equation (ld_func1.pro).

f_bright = CURVEFIT (rs, r_bright, w, a, sigma, function_name='ld_func1')

;print, 'iz: ', iz, '   coef: ', a

IF (iz / 30 * 30 EQ iz) THEN	$
BEGIN
print, 'iz: ', iz, '   A: ', a
oplot, rs, f_bright			; Overplot the fitted curve.
wait, 1
END

RETURN, f_bright
END
