;+
; NAME	ld_fit5
;
; PURPOSE	Fit Limb darkening equation
;
; SYNTAX	f_bright = ld_fit5 (r_dist, r_bright, rsun)
;
; INPUT     	t_ang: angular distance from solar north in degrees.
;		t_bright: pixel brightness.
;		rsun: number of pixels per solar radius.
;
; OUTPUT	f_bright: pixel brightness according to equation fit.
;
; AUTHOR	Andrew L. Stanger   31 August 2000
;
; HISTORY	Sine function.
;-

FUNCTION ld_fit5, t_ang, t_bright, pang, iz

ta = t_ang + pang
poff = fix (pang + 0.5)

IF (iz / 30 * 30 EQ iz) THEN	$
BEGIN
plot,  ta, t_bright,	$			; Plot angular scan.
       title='Limb Darkening Angular Profile' +	$
	      ' ' + strtrim (string (iz), 2) + ' deg.',	$
       xtitle='Angle [degrees]',		$
       ytitle='Brightness'
END

nang = N_ELEMENTS (ta)			; Number of elements in angular scan.

mom = moment (t_bright, sdev=sd)	; mean, variance, skew, kurtosis.
maxval = MAX (t_bright)
aveval = mom (0)
loval = maxval - sd
hival = aveval + sd

w = FLTARR (nang)
w [*] = 1.0						; Assign weights.
;w [ 85 + poff: 96 + poff] = 100.0			; Emphasize equator.
;w [265 + poff:276 + poff] = 100.0
wlow = WHERE (t_bright LT loval)
IF (sd GT 0.0) THEN w [wlow] = 0.0

;w [*] = 1.0

a     = FLTARR (2)
sigma = FLTARR (2)

a [0] = t_bright [0]			; Initial guess for coefficients.
a [1] = 0.0

;--- Fit radial scan to limb-darkening equation (ld_func5.pro).

f_bright = CURVEFIT (ta, t_bright, w, a, sigma, function_name='ld_func5')

;print, 'iz: ', iz, '   coef: ', a

IF (iz / 30 * 30 EQ iz) THEN	$
BEGIN
print, 'iz: ', iz, '   A: ', a
print, 'aveval: ', aveval
print, 'sd: ', sd
;print, 'wlow: '
;print, wlow
oplot, ta, f_bright			; Overplot the fitted curve.
wait, 1
END

RETURN, f_bright
END
