;+
; NAME	ld_func2
;
; PURPOSE	Limb darkening equation
;
; SYNTAX	ld_func2, r, a, b, pd
;
; AUTHOR	Andrew L. Stanger   14 August 2000
;
; HISTORY	Equation acquired from Randy Meisner.
;-

PRO ld_func2, r, a, b, pd

; B = A0 + A1*mu + A2*[1-mu*ln(1+1/mu)]
; B = A0 + A1*mu + A2*mu^3
; mu = sqrt[1-(r/R)^2]
; r=radius of pixel from center
; R = radius of solar disk.
;
; b = a [0] + a [1] * u * a [2] * (1.0 - u * ln (1.0 + 1.0 / u)
; b = a [0] + a [1] * u + a [2] * u^3
; u = sqrt (1.0 - (r / rsun) ^ 2)

pd = FLTARR (N_ELEMENTS (r), N_ELEMENTS (a))

rsun = 1.0

u = sqrt (1.0 - (r / rsun) ^ 2)

b = a [0] + a [1] * u + a [2] * (1.0 - u * alog (1.0 + 1.0 / u))

pd [*, 0] = 1.0

pd [*, 1] = u

pd [*, 2] = (1.0 - u * alog (1.0 + 1.0 / u))

END

