;+
; NAME		ldprofile.pro
;
; PURPOSE	Determine a limb-darkening profile using radius vs angle scans.
;
; SYNTAX	ldprofile, rad_ang_file
;
; INPUT		rad_ang_file : floating point table (radius, angle) created by
;		C-language program (chip_ldark11.c).
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR
;
; HISTORY	Created 22 September 2000
;-

PRO ldprofile, rad_ang_file

nrad = 414
nang = 362

nrbox = nrad / 10 + 1
rbrt = fltarr (nrbox)
rpos = fltarr (nrbox)
bfit = fltarr (nrbox)

pixrs = 414.99		; 28 Aug 2000
pang =  20.29 

t_ang    = findgen (nang) - 1.0
t_bright = fltarr (nang)
r_dist   = findgen (nrad)
r_bright = fltarr (nrad)

b_fit   = fltarr (nrad, nang)
rad_ang = fltarr (nrad, nang)

print, 'polar coordinate file: ', rad_ang_file

;--- Read radial scan (2D: radius, angle).

CLOSE, 11
OPENR, 11, rad_ang_file 
   READU, 11, rad_ang
CLOSE, 11

b_cen = 0.0 ;
ncen  = 0 ;

;--- Compute average brightness at sun center.

FOR iz = 0, nang - 1 DO					$
   BEGIN
   b_cen = b_cen + b_fit [0, iz] + b_fit [1, iz] + b_fit [2, iz] + $
		  b_fit [3, iz] + b_fit [4, iz] + b_fit [5, iz] + $
		  b_fit [6, iz] + b_fit [7, iz] + b_fit [8, iz] + $
		  b_fit [9, iz]
   ncen = ncen + 10
   END
b_cen = b_cen / float (ncen)

FOR ir = 0, nrbox - 1 DO	$
   BEGIN
   ixbeg = ir * 10
   ixend = ixbeg + 9
   IF (ixend GT nrad - 1) THEN ixend = nrad - 1
   print, 'ixbeg/ixend: ', ixbeg, ixend
   iybeg = 0
   iyend = nang - 1
   ;brtloc = brtpixpc (rad_ang, pixrs, 20.0, ixbeg, ixend, iybeg, iyend)
   ;nbrt = N_ELEMENTS (brtloc)
   ;brtave = TOTAL (rad_ang [brtloc]) / float (nbrt)
   brtave = brtvalpc (rad_ang, pixrs, 20.0, ixbeg, ixend, iybeg, iyend)
   rbrt [ir] = brtave
   rpos [ir] = (ixend - ixbeg) / 2.0 + ixbeg
   END

plot, rpos, rbrt, title='Radial Profile'

print, rbrt

;--- Fit curve to radial profile.

iz = 90
b_fit  = ldrfit (rpos, rbrt, pixrs, param)

PRINT, 'b_cen: ', b_cen

;--- Fit azimuthal scan to limb darkening equation.

;FOR iz = 0, nrad - 1 DO					$
;   BEGIN
;   t_bright [*] = rad_ang [iz, 0:nang-1]
;   b_fit [iz, 0:nang-1] = ld_fit5 (t_ang, t_bright, pang, iz)

;   IF (iz / 30 * 30 EQ iz) THEN	$
;      BEGIN
;       plot,  r_dist, r_bright,	$		; Plot radial scan.
;           title='Limb Darkening Radial Profile' +	$
;	    ' ' + strtrim (string (iz), 2) + ' deg.',	$
;           xtitle='Radial Distance [Rsun]',		$
;           ytitle='Brightness'
;      WAIT, 1
;      END
;   END

b_cor = b_cen / b_fit 

nf = N_ELEMENTS (rad_ang_file)

OPENW, 11, rad_ang_file + '_fac'
   writeu, 11, b_cor
CLOSE, 11

END
