;+
; NAME	ldrfit
;
; PURPOSE	Fit Limb darkening equation
;
; SYNTAX	f_bright = ld_fit2 (r_dist, r_bright, rsun)
;
; INPUT     	r_dist: radial distance from sun center in pixels.
;		r_bright: pixel brightness.
;		rsun: number of pixels per solar radius.
;
; OUTPUT	f_bright: pixel brightness according to equation fit.
;
; AUTHOR	Andrew L. Stanger   14 August 2000
;
; HISTORY	Equation acquired from Randy Meisner.
;-

FUNCTION ldrfit, r_dist, r_bright, rsun, param

rs = r_dist / rsun			; Convert to units of Rsun.

plot,  rs, r_bright,	$			; Plot radial scan.
       title='Limb Darkening Radial Profile',	$
       xtitle='Radial Distance [Rsun]',		$
       ytitle='Brightness'

nrad = N_ELEMENTS (rs)			; Number of elements in radial scan.

w = FLTARR (nrad)
w [*] = 0.5				; Assign weights.
w [ 0 :  9] = 100.0
w [10 : 19] = 10.0
w [20 : 29] =  5.0
w [nrad - 40 : nrad - 31] =  2.0
w [nrad - 30 : nrad - 21] =  5.0
w [nrad - 20 : nrad -  1] = 10.0

;w [*] = 1.0

param = FLTARR (3)
sigma = FLTARR (3)

param [0] = r_bright [0]		; Initial guess for coefficients.
param [1] = 0.0
param [2] = 0.0

;--- Fit radial scan to limb-darkening equation (ld_func2.pro).

f_bright = CURVEFIT (rs, r_bright, w, param, sigma, function_name='ld_func2')

print, 'N_ELEMENTS (f_bright): ', N_ELEMENTS (f_bright)
print, 'param: ', param
oplot, rs, f_bright			; Overplot the fitted curve.
wait, 1

RETURN, f_bright
END
