;+
; NAME	lsdcor.pro
;
; PURPOSE	Use CHIP light shaping diffuser images to create correction.
;
; SYNTAX	lsdcor, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   22 February 2001
;-

pro lsdcor, fits_list, color_lut

CLOSE, 11
fits_name = ''
xdim_prev = 0
ydim_prev = 0

red = bindgen (256)
green = red
blue  = red
if (N_ELEMENTS (color_lut) GT 0) THEN	$
    lct, color_lut			$
ELSE					$
    lct,'/home/stanger/color/bwy.lut'	; color table.

tvlct, red, green, blue, /get

;--- File list loop.

OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 11, fits_name				; Get file name.
print, 'fits_name: ', fits_name

   lsd_img = readfits (fits_name, hdu)

   ;--- Extract information from header.

;   fits_min = MIN (lsd_img, max=fits_max)

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   fits_min = fxpar (hdu, 'DISPMIN')
   fits_max = fxpar (hdu, 'DISPMAX')

;   fits_min = -1000
;   fits_max = 5000
;   print, FORMAT = '("fits_min/max: ", E10.3, E10.3)', fits_min, fits_max
   print, 'fits_min/fits_max: ', fits_min, fits_max

   ;--- Convert to byte pixels.
   ;--- Display image.

   IF (xdim NE xdim_prev OR ydim NE ydim_prev) THEN	$
      window, xsize=xdim, ysize=ydim
   byt_img = bytscl (lsd_img, min=fits_min, max=fits_max, top=249)
   tv, byt_img						; Display image.

   ;--- Get average brightness at center of array.

   brtavg = cenbrt (lsd_img)
   lsd_cor = fltarr (xdim, ydim)
   lsd_cor = lsd_img / brtavg		; divide by avg brightness at center.

   ;--- Write new image to disk.

   sloc = RSTRPOS (fits_name, '.fts')
   new_name = strmid (fits_name, 0, sloc) + '.lsd.fts'
   print, 'Write FITS image : ', new_name
   hducor = hdu
   datamin = MIN (lsd_cor, max=datamax)
   dispmin =  1.1
   dispmax =  0.9
   fxaddpar, hducor, 'DATAMIN', datamin
   fxaddpar, hducor, 'DATAMAX', datamax
   fxaddpar, hducor, 'DISPMIN', dispmin
   fxaddpar, hducor, 'DISPMAX', dispmax
   fxaddpar, hducor, 'OBJECT', 'LSD RATIO', ' LIGHT SHAPING DIFFUSER CORRECTION.'

   writefits, new_name, lsd_cor, hducor		; Write lsd correction image.
   xdim_prev = xdim
   ydim_prev = ydim
END	;}

CLOSE, 11
END
