; lsdfit2.pro
;
; Fit light shaping diffuser image.

PRO lsdfit2, lsd_name

;lsd_name = '20010105.200333.chp.hdr.fts'

;--- Read image.

lsd = readfits (lsd_name, hdulsd, /noscale)

;--- Get info from FITS header.

bitpix = fxpar (hdulsd, 'BITPIX')
nbypix = bitpix / 8
xdim   = fxpar (hdulsd, 'NAXIS1')
ydim   = fxpar (hdulsd, 'NAXIS2')

;--- Do surface fit.

lsdfit = sfit (lsd, 4)

;--- Compute residual.

lsddif = lsd - lsdfit

;--- Get min/max values.

fitmin = MIN (lsdfit, max=fitmax)
difmin = MIN (lsddif, max=difmax)

;--- Resize image.

xdim2 = xdim / 2
ydim2 = ydim / 2
lsdfit2 = rebin (lsdfit, xdim2, ydim2)
lsddif2 = rebin (lsddif, xdim2, ydim2)

;--- Display residual image.

window, xsize=xdim2, ysize=ydim2
tvscl, lsdfit2
wait, 3
tvscl, lsddif2

;--- Write LSD fit image to disk.

hdufit = hdulsd
fxaddpar, hdufit, 'DATAMIN', fitmin
fxaddpar, hdufit, 'DATAMAX', fitmax
fxaddpar, hdufit, 'DISPMIN', fitmin
fxaddpar, hdufit, 'DISPMAX', fitmax
fxaddpar, hdufit, 'HISTORY', 'LSD 2ND DEGREE POLYNOMIAL FIT'

ipos = strpos (lsd_name, '.fts')
lsdfit_name = strmid (lsd_name, 0, ipos)
lsdfit_name = lsddif_name + '.fit.fts'
writefits, lsdfit_name, lsdfit, hdufit

;--- Write residual image to disk.

hdudif = hdulsd
fxaddpar, hdudif, 'DATAMIN', difmin
fxaddpar, hdudif, 'DATAMAX', difmax
fxaddpar, hdudif, 'DISPMIN', difmin
fxaddpar, hdudif, 'DISPMAX', difmax
fxaddpar, hdudif, 'HISTORY', 'RESIDUAL IMAGE [LSD - 2ND DEGREE FIT]'

ipos = strpos (lsd_name, '.fts')
lsddif_name = strmid (lsd_name, 0, ipos)
lsddif_name = lsddif_name + '.resid.fts'
writefits, lsddif_name, lsddif, hdudif

END
