; lsdfit2.pro
;
; Fit light shaping diffuser image.

PRO lsdfit2, lsd_name

;lsd_name = '20010105.200333.chp.hdr.fts'

;--- Read image.

lsd = readfits (lsd_name, hdulsd, /noscale)

;--- Get info from FITS header.

bitpix = fxpar (hdulsd, 'BITPIX')
nbypix = bitpix / 8
xdim   = fxpar (hdulsd, 'NAXIS1')
ydim   = fxpar (hdulsd, 'NAXIS2')

;--- Crop the image, trimming off a few border pixels.

xdimc = xdim - 6
ydimc = ydim - 6
xdim2 = xdimc / 2
ydim2 = ydimc / 2
lsdc = intarr (xdimc, ydimc)
lsdc = lsd [3:xdim-4, 3:ydim-4]
cmin = MIN (lsdc, max=cmax)
hdulsdc = hdulsd

print, 'xdimc/ydimc: ', xdimc, ydimc
print, 'size (lsdc): ', size (lsdc)

;--- Display cropped image (after resizing to fit on screen).

lsdc2 = rebin (lsdc, xdim2, ydim2)
window, xsize=xdim2, ysize=ydim2
tvscl, lsdc2

;--- Do surface fit.

lsdfit = sfit (lsdc, 2)
print, 'size (lsdfit): ', size (lsdfit)

;--- Compute residual.

lsddif = lsdc - lsdfit

;--- Get min/max values.

fitmin = MIN (lsdfit, max=fitmax)
difmin = MIN (lsddif, max=difmax)

print, 'fitmin/fitmax: ', fitmin, fitmax
print, 'difmin/difmax: ', difmin, difmax

;--- Resize image.

lsdfit2 = rebin (lsdfit, xdim2, ydim2)
lsddif2 = rebin (lsddif, xdim2, ydim2)

;--- Display residual image.

window, xsize=xdim2, ysize=ydim2
tvscl, lsdfit2
;wait, 3
;tvscl, lsddif2

;--- Write LSD fit image to disk.

hdufit = hdulsdc
fxaddpar, hdufit, 'DATAMIN', fitmin
fxaddpar, hdufit, 'DATAMAX', fitmax
fxaddpar, hdudif, 'DISPMIN', -20.0
fxaddpar, hdudif, 'DISPMAX',  20.0
fxaddpar, hdufit, 'HISTORY', 'LSD 2ND DEGREE RESIDUAL IMAGE'

ipos = strpos (lsd_name, '.fts')
lsdfit_name = strmid (lsd_name, 0, ipos)
lsdfit_name = lsdfit_name + '.fit.fts'
writefits, lsdfit_name, lsdfit, hdufit

;--- Write residual image to disk.

hdudif = hdulsdc
fxaddpar, hdudif, 'DATAMIN', difmin
fxaddpar, hdudif, 'DATAMAX', difmax
fxaddpar, hdudif, 'DISPMIN', -20.0
fxaddpar, hdudif, 'DISPMAX',  20.0
fxaddpar, hdudif, 'HISTORY', 'RESIDUAL IMAGE [LSD - 2ND DEGREE FIT]'

ipos = strpos (lsd_name, '.fts')
lsddif_name = strmid (lsd_name, 0, ipos)
lsddif_name = lsddif_name + '.resid.fts'
writefits, lsddif_name, lsddif, hdudif

END
