; lsdfit4.pro
;
; Fit light shaping diffuser image.

PRO lsdfit4, lsd_name

;lsd_name = '20010105.200333.chp.hdr.fts'

;--- Read image.

lsd = readfits (lsd_name, hdulsd, /noscale)

;--- Get info from FITS header.

bitpix = fxpar (hdulsd, 'BITPIX')
nbypix = bitpix / 8
xdim   = fxpar (hdulsd, 'NAXIS1')
ydim   = fxpar (hdulsd, 'NAXIS2')

;--- Crop the image, trimming off a few border pixels.

border = 6
xdimc = xdim - border * 2
ydimc = ydim - border * 2
xdim2 = xdimc / 2
ydim2 = ydimc / 2
lsdc = intarr (xdimc, ydimc)
lsdc = lsd [border:xdim-1-border, border:ydim-1-border]
cmin = MIN (lsdc, max=cmax)
hdulsdc = hdulsd

print, 'xdimc/ydimc: ', xdimc, ydimc
print, 'size (lsdc): ', size (lsdc)

;--- Display cropped image (after resizing to fit on screen).

lsdc2 = rebin (lsdc, xdim2, ydim2)
window, xsize=xdim2, ysize=ydim2
tvscl, lsdc2

;--- Do surface fit.

lsdfit = sfit (lsdc, 4, kx=pc)
print, 'size (lsdfit): ', size (lsdfit)
print, 'size (pc): ', size (pc)
print, 'pc: ', pc

;--- Compute residual.

lsddif = lsdc - lsdfit

;--- Get min/max values.

fitmin = MIN (lsdfit, max=fitmax)
difmin = MIN (lsddif, max=difmax)

print, 'fitmin/fitmax: ', fitmin, fitmax
print, 'difmin/difmax: ', difmin, difmax

;--- Resize image.

xdim2 = xdimc / 2
ydim2 = ydimc / 2
lsdfit2 = rebin (lsdfit, xdim2, ydim2)
lsddif2 = rebin (lsddif, xdim2, ydim2)

;--- Display residual image.

;tvscl, lsdfit
tvscl, lsdfit2
;wait, 3
;tvscl, lsddif2

;--- Write LSD fit image to disk.

hdufit = hdulsdc
fxaddpar, hdufit, 'DATAMIN', fitmin
fxaddpar, hdufit, 'DATAMAX', fitmax
fxaddpar, hdufit, 'DISPMIN', fitmin
fxaddpar, hdufit, 'DISPMAX', fitmax
fxaddpar, hdufit, 'HISTORY', 'LSD 4TH DEGREE POLYNOMIAL FIT'

ipos = strpos (lsd_name, '.fts')
lsdfit_name = strmid (lsd_name, 0, ipos)
lsdfit_name = lsdfit_name + '.fit4.fts'
writefits, lsdfit_name, lsdfit, hdufit

;--- Write residual image to disk.

hdudif = hdulsdc
fxaddpar, hdudif, 'DATAMIN', difmin
fxaddpar, hdudif, 'DATAMAX', difmax
fxaddpar, hdudif, 'DISPMIN', difmin
fxaddpar, hdudif, 'DISPMAX', difmax
fxaddpar, hdudif, 'HISTORY', 'RESIDUAL IMAGE [LSD - 4TH DEGREE FIT]'

ipos = strpos (lsd_name, '.fts')
lsddif_name = strmid (lsd_name, 0, ipos)
lsddif_name = lsddif_name + '.res4.fts'
writefits, lsddif_name, lsddif, hdudif

END
