;+ makelsdgain.pro
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MLSO/CHIP Helium-I instrument
;
; Routine to take a bunch of fr images
; and create a new gain to pass to MLSO
;
; frlist contains the names of the files
; The first file is a bias, the remaining
; ones are for the various passbands.
;
; The mean of the lsd gain files is turned
; into a multiplicative gain file for chip
; and written out as 'gainlsd'
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; SYNTAX	makelsdgain, 'frlist'
;		frlist: file containing a list of image names.
;
; AUTHOR	David Elmore   HAO/NCAR   2 March 2001
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-

PRO makelsdgain, frlist

numband = 7
pixX = 1534
pixY = 1030

bias = INTARR (pixX, pixY)
temp = INTARR (pixX, pixY)
fr   = INTARR (numband, pixX, pixY)
filenm = 'fred'
WINDOW, xs = pixX / 2, ys = pixY / 2
LOADCT, 3

CLOSE, 15
OPENR, 15, frlist
READF, 15, filenm

CLOSE, 11
OPENR, 11, filenm
READU, 11, bias
CLOSE, 11

; Remove header and edge effects from bias

bias (*, 0) = bias (*, 2)
bias (*, 1) = bias (*, 2)
bias (*, pixy - 1) = bias (*, pixY - 2)

; Remove header and lots of edge effects from lsd images
; Subtract bias

FOR ii = 0, numband - 1 DO BEGIN
	READF, 15, filenm 
	OPENR, 11, filenm
	READU, 11, temp
        CLOSE, 11
	temp (*, 0) = temp (*, 7)
	temp (*, 1) = temp (*, 7)
	temp (*, 2) = temp (*, 7)
	temp (*, 3) = temp (*, 7)
	temp (*, 4) = temp (*, 7)
	temp (*, 5) = temp (*, 7)
	temp (*, 6) = temp (*, 7)
	temp (*, pixy-1) = temp (*, pixY-2)
	temp = temp - bias
	fr [ii, *, *] = temp
	fred = temp
	fred = REBIN (fred, pixX / 2, pixY / 2)
	TV, BYTSCL (fred, 1200, 2500)
ENDFOR
CLOSE, 15

;stop
meanfr = 1.0 * fr [0, *, *]
FOR ii= 1, numband - 1 DO BEGIN
	meanfr = meanfr + fr [ii, *, *]
ENDFOR
meanfr = meanfr / 7
meanfr = REFORM (meanfr)
;STOP

PRINT, "doing polynomial fit"
frfit = SFIT (meanfr, 4)
gain = frfit / meanfr * 8192.0
gain = FIX (gain)
fred = REBIN (gain, pixX / 2, pixY / 2)
TV, BYTSCL (fred, 1.1 * MAX (fred), 0.8 * MAX (fred))
;STOP

; Compare new vs old gain

OPENW,  11, 'gainlsd'
WRITEU, 11, gain
CLOSE,  11

OPENR, 11, 'chip_gain'
READU, 11, temp
CLOSE, 11

fred = gain - temp
fred = REBIN (fred, pixX / 2, pixY / 2)
TV, BYTSCL (fred, -400, 400)

;STOP

END
