;+
; NAME		maxbrt.pro
;
; PURPOSE	Determine the average maximum disk brightness of a solar image.
;
; MEHTOD	Use only pixels located ON the solar disc.
;		Order the subset array according to brightness.
;		Average over the brightest 5% of the pixels in this box.
;	
; SYNTAX	avgbrt = maxbrt (img)
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   4 Oct 2000
;-

FUNCTION maxbrt, imgbuf, radbuf

;PRINT, 'maxbrt: 5% brightest pixel average.'

sizeimg = SIZE (imgbuf)

xdim = sizeimg [1]
ydim = sizeimg [2]

xcen = (xdim / 2 - 0.5)
ycen = (ydim / 2 - 0.5)
ixcen = xdim / 2
iycen = ydim / 2

sizerad = SIZE (radbuf)
xrad = sizerad [1]
yrad = sizerad [2]

;PRINT, 'size (radbuf): ', size (radbuf)
;PRINT, 'xrad/yrad: ', xrad, yrad

discpix = WHERE (radbuf LT 0.8)
sizedisc = size (discpix)
ndisc = sizedisc [1]
;PRINT, 'maxbrt> size (discpix): ', size (discpix)

;--- Use only pixels located ON the solar disk.

disc = imgbuf [discpix]

;--- Order the array according to increasing intensity.

censort = disc (SORT (disc))

;--- Compute the "5% median": level at which 5% of the pixels lie above
;--- the computed level. 
;--- Then compute the average of these brightest pixels.

nbrt95 = LONG (0.95 * ndisc)
brtcen = TOTAL (censort [nbrt95:ndisc-1]) / (ndisc - nbrt95)

RETURN, brtcen
END
