;+ mklsdgain.pro
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; MLSO/CHIP Helium-I instrument
;
; Routine to take a bunch of fr images and create a new gain to pass to MLSO.
;
; frlist contains the names of the files.
; The first file is a bias, the remaining ones are for the various passbands.
;
; The mean of the lsd gain files is turned into a multiplicative gain file
; for chip and written out as 'gainlsd'
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
; SYNTAX	mklsdgain, 'frlist'
;		frlist: file containing a list of image names.
;
; AUTHOR	David Elmore   HAO/NCAR   2 March 2001
; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
;-

PRO mklsdgain, frlist

numband = 7
xdim = 1534
ydim = 1030
xdim2 = xdim / 2
ydim2 = ydim / 2

bias = INTARR (xdim, ydim)
lsd  = INTARR (xdim, ydim)
fr   = INTARR (numband, xdim, ydim)
filename = 'chip'
WINDOW, xsize = xdim2, ysize = ydim2, retain = 2
LOADCT, 3

CLOSE, 15
OPENR, 15, frlist
READF, 15, filename

CLOSE, 11
OPENR, 11, filename
READU, 11, bias
CLOSE, 11

;--- Remove header and edge effects from bias

bias [*, 0] = bias [*, 2]
bias [*, 1] = bias [*, 2]
bias [*, ydim - 1] = bias [*, ydim - 2]

;--- Remove header and lots of edge effects from lsd images
;--- Subtract bias

FOR ii = 0, numband - 1 DO BEGIN
    READF, 15, filename 
    OPENR, 11, filename
    READU, 11, lsd
    CLOSE, 11
    lsd [*, 0] = lsd [*, 7]
    lsd [*, 1] = lsd [*, 7]
    lsd [*, 2] = lsd [*, 7]
    lsd [*, 3] = lsd [*, 7]
    lsd [*, 4] = lsd [*, 7]
    lsd [*, 5] = lsd [*, 7]
    lsd [*, 6] = lsd [*, 7]
    lsd [*, ydim - 1] = lsd [*, ydim - 2]
 
    lsd = lsd - bias
    fr [ii, *, *] = lsd

    lsd2 = REBIN (lsd2, xdim2, ydim2)
    TV, BYTSCL (lsd2, 1200, 2500)
ENDFOR
CLOSE, 15

;stop

;--- Compute average bias-corrected LSD image.

meanfr = 1.0 * fr [0, *, *]
FOR ii= 1, numband - 1 DO BEGIN
    meanfr = meanfr + fr [ii, *, *]
ENDFOR
meanfr = meanfr / 7
meanfr = REFORM (meanfr)

;STOP

;--- Compute a 4th order polynomial fit to the average LSD image.

PRINT, "Doing polynomial fit"
frfit = SFIT (meanfr, 4)
gain = frfit / meanfr * 8192.0
gain = FIX (gain)

gain2 = REBIN (gain, xdim2, ydim2)
TV, BYTSCL (gain2, 1.1 * MAX (gain2), 0.8 * MAX (gain2))

;STOP

;--- Write gain image to disk.

OPENW,  11, 'gainlsd'
WRITEU, 11, gain
CLOSE,  11

;--- Compare new vs old gain.

;OPENR, 11, 'chip_gain'
;READU, 11, lsd
;CLOSE, 11

;gaindif = gain - lsd
;gaindif = REBIN (gaindif, xdim2, ydim2)
;TV, BYTSCL (gaindif, -400, 400)

;STOP

END
