; pb3d.pro
;+
;**************************************************************************
; Program Dens.to.pB
;
;  Given an analytic function denscalc (rpB, thetapB, phipB), which
;  can return an electron _number_ density given the physical
;  3-d coordinates rpB, thetapB, phipB, this program will 
;  calculate a plane-of-sky distribution of both Brightness and 
;  polarized Brightness.
;
;**************************************************************************
;-

pro pb3d 

common limbresults, pB, Bright
common quants,      ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
                    mult, full, idens, lat, Bang
common limbinfo,    limbread
common edge,        coneedge, onedconeedge

;-- Get inputs.

inputs

;-- The plane of sky coordinates are calculated (r, theta, phi).
;   I decided to do these evenly spaced in x-y so that we could
;   then plot a tv image (I knew there was a reason for an x-y
;   grid!).

;-- However- because idl runs out of memory, we have to use
;   a loop in ny.

if (full eq 0) then endrot =  1
if (full eq 1) then endrot = 36
if (full eq 2) then endrot =  3

pBarr     = fltarr (nx, ny, endrot)
Brightarr = fltarr (nx, ny, endrot)

x =  fltarr (nx)
y =  fltarr (nx)

dx = 2.0 * maxheight / (nx - 1.0)
thsave = fltarr (nx * ny)
count  = 0

for irot = 0, endrot - 1 do begin ;{
  for stepy = 0, ny - 1 do begin ;{
    x [*] =  -maxheight + findgen (nx) * (2 * maxheight / (nx - 1.0))
    y [*] =  -maxheight + stepy        * (2 * maxheight / (ny - 1.0))

    ;-- Now convert to r, theta (which is what the pB integral wants).

    r     = sqrt (x * x + y * y)
    theta = acos (y / r)
    neg   = where (x lt 0.0)
    if (min (neg) ne -1) then theta [neg] = 2. * !pi - theta [neg]

    print, 'size (x): ', size (x)
    print, 'size (y): ', size (y)
    print, 'size (r): ', size (r)
    print, 'size (theta): ', size (theta)

    ;-- And your phi is the central meridian of the "day" you want.

    phi = fltarr (nx)
    if (full ne 2) then phi [*] =         cmer + irot * 10.0
    if (full eq 2) then phi [*] = -50.0 + cmer + irot * 50.0

    phi = phi * !pi / 180.0

    ;-- Now calculate pB and B!.

    limbread = 1
    pbint, r, theta, phi

    pBarr     [*, stepy, irot] = alog10 (pB [*])
    Brightarr [*, stepy, irot] = alog10 (Bright [*])

;    pBarr     [*, stepy, irot] = pB [*]
;    Brightarr [*, stepy, irot] = Bright [*]


    ;-- Check the apparant width.

    if (idens eq 4) then begin ;{
      if (min (onedconeedge) ne -1) then begin ;{
        thetwid = theta [onedconeedge]
        rwid    = r [onedconeedge]
;        print, rwid
        test = where (abs (rwid - 1.) lt sqrt (2.1) * dx)
;        stop
        if (min (test) ne -1) then begin ;{
          tlen = size (test)
          tlen = tlen (1)
          thsave [count:count + tlen - 1] = thetwid (test)
          count = count + tlen
        endif ;}
      endif ;}
    endif ;}

;-- Block out the solar disks.

    test = where (r lt 1.0)
    if (min (test) ne -1) then pBarr     [test, stepy, irot] = -100.0
    if (min (test) ne -1) then Brightarr [test, stepy, irot] = -100.0
  endfor ;}
endfor ;}

;-- Width.

if (idens eq 4) then begin ;{
  if (min (onedconeedge) ne -1) then begin ;{
    thsavefin = where (thsave ne 0.)
    print, 'thetas are', thsave [thsavefin] / !dtor
    print, 'width is', max (thsave [thsavefin] / !dtor) - min (thsave [thsavefin] / !dtor)
  endif ;}
endif ;}

test2 = where (pBarr ne -100)
minpB = min (pBarr [test2])
maxpB = max (pBarr [test2])
test2 = 0.0

;-- Normalization for CASE 1 vs. CASE 2.

minpB = -2.5
maxpB =  2.5

;-- rebin.

pBarr     = rebin (pBarr,     512, 512, endrot)
Brightarr = rebin (Brightarr, 512, 512, endrot)

;-- Show the pB movie.

pB2d = fltarr (512, 512)

if (full ne 2) then window, 2, xs=512,  ys=512,  retain=3, title='pB model'
if (full eq 2) then window, 2, xs=1024, ys=1024, retain=3, title='pB model'

if (full eq 0) then  begin ;{
 pB2d [*, *] = pBarr [*, *, 0]

;-- And plot.

tv, bytscl (pB2d, min=minpB, max=maxpB)
endif ;}

if (full eq 1) then movie, bytscl (pBarr, min=minpB, max=maxpB)

if (full eq 2) then begin ;{
  pB2d [*, *] = pBarr [*, *, 0]
  tv, bytscl (pB2d, min=minpB, max=maxpB), 0
  pB2d [*, *] = pBarr [*, *, 1]
  tv, bytscl (pB2d, min=minpB, max=maxpB), 1
  pB2d [*, *] = pBarr [*, *, 2]
  tv, bytscl (pB2d, min=minpB, max=maxpB), 2
endif ;}

;-- And plot.

;;	print, 'type .c to continue to modeled brightness image'
;;	stop

;-- Show the Brightness movie

Bright2d = fltarr (512, 512)

window, 3, xs=512, ys=512, retain=3, title='Brightness model'


if (full eq 0) then begin ;{

  Bright2d [*, *] = Brightarr [*, *, 0]

  tv, bytscl (Bright2d, min=minpB, max=maxpB)

endif ;}

if (full eq 1) then movie, bytscl (Brightarr, min=minpB, max=maxpB)
stop

end

;***********************************************************************

pro inputs
	
common quants,      ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
                    mult, full, idens, lat, Bang
common densrequest, ndens
common sscoord,     rsquig, phiss, time
common prams,       apar, xo, thetao, phio, rbub, ao, Pio, alpha, eta, $
                    velmult, cpar, dpar, alnot, sig1, out


read, 'What density routine? dens1 - 1, dens2 - 2, dens3 - 3, dens4 (ice cream cone) - 4, G&L model dens - 5 , or Spartan 3d  -- 6 ', idens

;-- Read in prams for G&L model.

if (idens eq 5) then begin
  read, 'What time step? in hours (0. default) ', time
  time = time * 3600.0
  accel = 0

  openr, 5, 'pramsset.d'
  readprams
  close, 5

  read, 'What orientation? along l.o.s. (90.), perp to l.o.s. (0.) ', $
	sig1
endif

;-- Set up for Spartan.

if (idens eq 6) then params

;-- How far out do you extend?  

read, 'To what height do you wish the image to go? (Rsun) ', maxheight

;-- What grid spacing do you want?
       
read, 'Input grid dimensions (e.g. 256X256, type 256) ', nx
ny =  nx

;-- The l.o.s. integrating angle tau goes basically from -pi/2 to pi/2.
;   tauint = .1 is the integration step I've been using, but you could
;   them finer or coarser if you wish.
;
read, 'Maximum l.o.s integrating angle? (type 1.57 for default) ', $
      taumax
read, 'l.o.s integrating angle step-size? (type .1 for default) ', $
      tauint

;-- IF the model you are using is azimuthally symmetric, then may as
;   well only integrate from 0 to pi/2.  Here's a check for that:

read, 'Azimuthally symmetric model? (yes = 1, no = 0) ', nsymm

if (nsymm eq 0) then  begin
   taumin = -1 * taumax
   mult = 1.
endif

if (nsymm eq 1) then begin
   taumin = 0.
   mult = 2.
endif

ntau = fix ((taumax - taumin) / tauint + 1.0)

;-- What central meridian phi do you want?   (radians)
;   (you may want to step through these do see your
;   structure rotate round if its not azimuthally symmetric)

cmer = 0.0
if (nsymm eq 0) then read, 'What central meridian longitude (degrees)? (0 puts structures at west limb for CME models) ', cmer

;-- Also position in latitude.

lat = 0.0
read, 'What change in latitude (degrees)? (0 returns structure centered at equator, +/-  is below/above ', lat

Bang = 0.0
read, 'What change in viewing angle (i.e. B angle tilt) (degrees)? (0 is 0 B angle, +/-  is below/above ', Bang

full = 0
if (nsymm eq 0) then read, 'Do you want a single image (0), a full rotation (1), or three STEREO images separated by 50 degrees (2) ', full
end

;************************************************************************

pro pbint, r, theta, phi

common newresults,  density
common quants,      ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
                    mult, full, idens, lat, Bang
common limbresults, pB, Bright
common densrequest, ndens
common limbinfo,    limbread
common edge,        coneedge, onedconeedge
 
;-- Calculates integrated line-of-sight polarized Brightness pB (r, theta, phi)
;   and Brightness Bright (r, theta, phi) from density produced by 
;   denscalc (rpB, thetapB, phipB) and integrated over tau
;   (line of sight angle).

;   NOTE:  coordinates r, theta, phi are plane of sky coordinates
;    rpB, thetapB, phipB are "physical" spherical 3-d coordinates.

;-- Degrees to radian.

d2r = 1.745329e-2

;-- pi/2

pid2 = !pi / 2.0
 
;-- Both limbs will be represented in the plane of the sky:
;   the angle phi is the central meridian angle

; alfa +ve for West (right) limb
; alfa -ve for East (left) limb

alfa = fltarr (nx)
alfa = sin (theta) / abs (sin (theta))
	if (limbread ne 0) then alfap = alfa * limbread
	if (limbread eq 0) then alfap = alfa
 
zeta    = fltarr (ntau, nx)
thetapB = fltarr (ntau, nx)
rpB     = fltarr (ntau, nx)
r2      = fltarr (ntau, nx)
r3      = fltarr (ntau, nx)
phipB   = fltarr (ntau, nx)

tauslop    = (taumax - taumin) - tauint * (ntau - 1)
tauminslop = taumin + tauslop / 2.0
tau        = tauminslop + tauint * (findgen (ntau))

tnorm = (1 + findgen (ntau)) / (1 + findgen (ntau))

for i = 0, nx - 1 do begin
  zeta    [*, i] = atan (tan (tau) * (1 / sin (theta [i])))
  rpB     [*, i] = (1 / cos (tau)) * r [i]
  r2      [*, i] = tnorm * r [i]
  thetapB [*, i] = acos (cos (tau) * cos (theta [i])) / d2r
  phipB   [*, i] = (tnorm * alfap [i] * pid2 - alfap [i] * zeta [*, i]) / d2r
endfor

;-- First add Bangle if necessary.

bangrot, rpB, thetapB, phipB, thetauseB, phiuseB

for i = 0, nx - 1 do	$
  phiuseB [*, i] = phiuseB [*, i] + tnorm * phi [i] / d2r

;-- Now put in model structure rotation if necessary.

test = where (phiuseB gt 360.)
if (min (test) ne -1) then phiuseB [test] = phiuseB [test] - 360.0
test = where (phiuseB lt 0.0)
if (min (test) ne -1) then phiuseB [test] = phiuseB [test] + 360.0

throt, rpB, thetauseB, phiuseB, thetause, phiuse

test = where (phiuse gt 360.)
if (min (test) ne -1) then phiuse [test] = phiuse [test] - 360.0
test = where (phiuse lt 0.)
if (min (test) ne -1) then phiuse [test] = phiuse [test] + 360.0

if (idens eq 1) then dens1, rpB, thetause, phiuse
if (idens eq 2) then dens2, rpB, thetause, phiuse
if (idens eq 3) then dens3, rpB, thetause, phiuse
if (idens eq 4) then dens4, rpB, thetause, phiuse
if (idens eq 5) then dens5, rpB, thetause, phiuse
if (idens eq 6) then dens6, rpB, thetause, phiuse

;-- Get width info.

if (idens eq 4) then begin
  widthflag = density * 0.0
  if (min (coneedge) ne -1) then widthflag [coneedge] = 1.0
endif

;-- First calculate pB.

cff = cfun (rpB)

sumpB = fltarr (ntau, nx)
sumpB =  cff * r2 * tauint
sumpB = density * sumpB
pB = fltarr (nx)

;-- Now calculate Bright.

bff = bfun (rpB)

sumB   = fltarr (ntau, nx)
sumB   = bff * tauint * rpB * rpB / r2
sumB   = 2.0 * density * sumB
Bright = fltarr (nx)

;-- Now integrate both and set up width array.

onedcone = fltarr (nx)
count = 0

for i = 1, ntau-1 do begin
  pB = pB + sumpB [i, *] 
  Bright = Bright + sumB [i, *] - sumpB [i, *]

  if (idens eq 4) then begin
    onedtemp = where (widthflag [i, *] eq 1.)

    if (min (onedtemp) ne -1) then begin
      onelen = size (onedtemp)
      onelen = onelen (1)
      onedcone [count:count + onelen - 1] = onedtemp
      count = count + onelen
    endif

  endif

endfor

if (idens eq 4) then begin
  test = where (onedcone ne 0.0)
  if (min (test) ne -1) then onedconeedge = onedcone [test]
  if (min (test) eq -1) then onedconeedge = -1
endif

pB     = mult * pB
Bright = mult * Bright

;-- And add in plane of sky info for the symmetric case.

pB     = pB     + sumpB [0, *]
Bright = Bright + sumB  [0, *] - sumpB [0, *]

end

;************************************************************************

pro bangrot, rpB, thetapB, phipB, thetauseB, phiuseB

common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
               mult, full, idens, lat, Bang

d2r = 1.745329e-2

x = rpB * sin (d2r * thetapB) * sin (d2r * phipB)
y = rpB * sin (d2r * thetapB) * cos (d2r * phipB)
z = rpB * cos (d2r * thetapB)

;-- Change line of sight due to B angle  tilt.
;   Arfkin p 199

xp = x
yp = y * cos (Bang * d2r) - z * sin (Bang * d2r)
zp = y * sin (Bang * d2r) + z * cos (Bang * d2r)

thetauseB = acos (zp / rpB)
phiuseB   = thetauseB * 0.d0

test = where (yp ne 0.d0) 
if (min (test) ne -1) then phiuseB [test] = atan (xp [test] / yp [test])
test = where (yp eq 0.d0 and xp gt 0.d0)
if (min (test) ne -1) then phiuseB [test] =  !pi / 2.d0
test = where (yp eq 0.d0 and xp lt 0.d0)
if (min (test) ne -1) then phiuseB [test] = -!pi / 2.d0
if (min (test) ne -1) then stop
test = where (yp lt 0.0d0) 
if (min (test) ne -1) then	$
    phiuseB [test] = phiuseB [test] + double (!pi)

thetauseB = thetauseB / d2r
phiuseB   = phiuseB   / d2r

end

;************************************************************************

function bfun, r

;-- Calculates scattering function needed for Brightness calculation.
 
common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
               mult, full, idens, lat, Bang

a1 = 8.69d-7
a2 = 0.37
a3 = 0.63
 
cfunc = fltarr (ntau, nx)
df    = fltarr (ntau, nx)
dfunc = fltarr (ntau, nx)
bf    = fltarr (ntau, nx)
 
cfunc = 4. / 3. - sqrt (1. - 1./r/r) - ((1. - 1./r/r)^(1.5)) / 3.
df    = (r - 1./r) * (5 - 1./r/r) * alog ((1. + 1./r) / sqrt(1.-1./r/r))
dfunc = 1. / 8 * (5. + 1./r/r - df)
bf    = a1 * (a2 * cfunc + a3 * dfunc)
 
return, bf
end

;***********************************************************************

function cfun, r
 
;-- Calculates scattering function needed for Brightness calculation
;    and also polarized Brightness.

common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
               mult, full, idens, lat, Bang

a1 = 8.69d-7
a2 = 0.37
a3 = 0.63
 
afunc = fltarr (ntau, nx)
bf    = fltarr (ntau, nx)
bfunc = fltarr (ntau, nx)
cf    = fltarr (ntau, nx)
 
afunc = sqrt (1.0 - 1.0 / r / r) * (1.0 / r / r)
bf    = (r - 1.0 / r) * (1 + 3.0 / r / r) *				$
	alog ((1.0 + 1.0 / r) / sqrt (1.0 - 1.0 / r / r))
bfunc = -1.0 / 8 * (1 - 3 / r / r - bf)
cf    = a1 * (a2 * afunc + a3 * bfunc)
 
return, cf
end

;************************************************************************

pro throt, rpB, thetauseB, phiuseB, thetause, phiuse

common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, $
               mult, full, idens, lat, Bang

d2r= 1.745329e-2

x = rpB * sin (d2r * thetauseB) * sin (d2r * phiuseB)
y = rpB * sin (d2r * thetauseB) * cos (d2r * phiuseB)
z = rpB * cos (d2r * thetauseB)

;-- Rotate dens structure at limb by theta = lat.
;   Arfkin p 199

yp = y
xp =  x * cos (lat * d2r) + z * sin (lat * d2r)
zp = -x * sin (lat * d2r) + z * cos (lat * d2r)

thetause = acos (zp / rpB)
phiuse   = thetause * 0.d0

test = where (yp ne 0.d0) 
if (min (test) ne -1) then phiuse [test] = atan (xp [test] / yp [test])
test = where (yp eq 0.d0 and xp gt 0.d0)
if (min (test) ne -1) then phiuse [test] =  !pi / 2.d0
test = where (yp eq 0.d0 and xp lt 0.d0)
if (min (test) ne -1) then phiuse [test] = -!pi / 2.d0
test = where (yp lt 0.0d0) 
if (min (test) ne -1) then phiuse [test] = phiuse [test] + double (!pi)

thetause = thetause / d2r
phiuse   = phiuse   / d2r

end

;************************************************************************

pro dens1, rpB, thetapB, phipB

common newresults, density
 
;-- Parameters amplitude (cpar) and slope (dpar).

cpar = 1e8
dpar = 2.0

density = cpar * (rpB^(-1 * dpar))

thetapB = thetapB + 30

apar = 5e8
bpar = 2
test = where (thetapB gt 80 and thetapb lt 110 and phipb gt 80 and phipb lt 120)
if (min (test) ne -1) then	$
    density [test] = apar * (rpB [test]^(-1 * bpar))

apar = 5e8
bpar = 2
test = where (thetapB gt 130 and thetapb lt 150 and phipb gt 10 and phipb lt 30)
if (min (test) ne -1) then	$
    density [test] = apar * (rpB [test]^(-1 * bpar))

end

;************************************************************************

pro dens2, rpB, thetapB, phipB

common newresults, density

;-- Parameters amplitude (cpar) and slope (dpar).

cpar = 1e8
dpar = 2.0

density = cpar * (rpB^(-1 * dpar))

apar = 5e8
bpar = 2
test = where (rpB * abs ((cos (thetapB * !pi / 180.0)))^0.5 lt 1.0)
if (min (test) ne -1) then	$
    density [test] = apar * (rpB [test]^(-1 * bpar))

end

;************************************************************************

pro dens3, rpB, thetapB, phipB

common newresults, density

;-- Parameters amplitude (cpar) and slope (dpar).

cpar = 1e8
dpar = 2.0

density = cpar * (rpB^(-1 * dpar))

test = where (rpB * abs ((cos (thetapB * !pi / 180.)))^.5 lt 1. and phipb gt 180.)
if (min (test) ne -1) then density [test] = cpar * (rpB [test]^(-1 * dpar)) * ((cos (phipB [test] * !pi / 180.0))^2)^(1.0 / 3.0)

end

;************************************************************************
pro dens4, rpB, thetapB, phipB

common newresults, density
common edge,       coneedge, onedconeedge

d2r= 1.745329e-2

;-- BACKGROUND DENSITY.
;   PARAMETERS amplitude (cpar) and slope (dpar)

cpar = 1e8
dpar = 3.0

dens_back = cpar * (rpB^(-1 * dpar))

;-- ENHANCED MODEL DENSITY.
;   PARAMETERS amplitude (cmod) and slope (dmod)

;-- CASE 1
cmod = 1e09

;-- CASE 2
; cmod = 2e09

dmod = 2.0

dens_mod = cmod * (rpB^(-1 * dmod))

;-- PUT in the background density.

density = dens_back

;-- PUT IN ICE CREAM CONE.

;-- PARAMETERS angles.

;-- CASE 1
alpha_o = 35.0
alpha_i = 30.0

;-- CASE 2
; alpha_o = 55.0
; alpha_i = 50.0

;-- Parameter R_c  --> distance from origin to center of the ice cream scoop.

;-- CASE 1
R_c =  4.0

;-- CASE 2
; R_c =  2.7

;-- PARAMETERS rho_i and rho_o inner and outer radius of the scoop.

rho_i = R_c * sin (alpha_i*d2r)
rho_o = R_c * sin (alpha_o*d2r)

;-- PARAMETERS sectionheights - s1_ht, s2_ht, s3_ht, s4_ht, s5_ht.

s1_ht = 1.0
s2_ht = R_c-rho_o * sin (alpha_o * d2r)
s3_ht = R_c-rho_i * sin (alpha_i * d2r)
s4_ht = R_c + rho_i
s5_ht = R_c + rho_o

;-- Define relationship between cone angle and spherical coordinate angles.

sinalpha =  sqrt (((sin (thetapB * d2r))^2) * ((cos (phipB * d2r))^2) + (cos (thetapB * d2r)^2))

;-- Define variation of inner and outer curves in the scoop.

alpha_o_scoop = (atan (sqrt ((sin (alpha_o * d2r)^2) - (rpB / R_c - 1.0)^2.0) / (rpB / R_c))) / d2r
alpha_i_scoop = (atan (sqrt ((sin (alpha_i * d2r)^2) - (rpB / R_c - 1.0)^2.0) / (rpB / R_c))) / d2r

test = where (alpha_o_scoop le 0.0 or alpha_o_scoop ge 90.0)
if (min (test) ne -1) then begin
  print, alpha_o_scoop, 'outer: out of first quadrant'
  stop
endif

test = where (alpha_i_scoop le 0.0 or alpha_i_scoop ge 90.0)
if (min (test) ne -1) then begin
  print, alpha_i_scoop, 'inner: out of first quadrant'
  stop
endif

;-- Section 1

;-- Legs of cone, cone angles constant.

section1 = where (rpB gt s1_ht and rpB le s2_ht)

if (min (section1) ne -1) then begin

  shell_1 = where (sinalpha lt sin (alpha_o*d2r) and sinalpha ge sin (alpha_i * d2r) and rpB gt s1_ht and rpB le s2_ht and sin (phipb * d2r) ge 0.0)
  insidecone_1 = where (sinalpha lt sin (alpha_i*d2r) and rpB gt s1_ht and rpB le s2_ht and sin (phipb * d2r) ge 0.0)

  if (min (shell_1) ne -1) then density [shell_1] = dens_mod [shell_1]
  if (min (insidecone_1) ne -1) then	$
      density (insidecone_1) = dens_back (insidecone_1)

;  if (min (shell_1 ne -1)) then stop

endif

;-- Section 2

section2 = where (rpB gt s2_ht and rpB le s3_ht)

if (min (section2) ne -1) then begin

;-- Above the legs - outer cone angle varies with height.

  shell_2 = where (sinalpha lt sin (alpha_o_scoop * d2r) and sinalpha ge sin (alpha_i * d2r) and rpB gt s2_ht and rpB le s3_ht and sin (phipb * d2r) ge 0.0)
  insidecone_2 = where (sinalpha lt sin (alpha_i * d2r) and rpB gt s2_ht and rpB le s3_ht and sin (phipb * d2r) ge 0.0)

  if (min (shell_2) ne -1) then density [shell_2] = dens_mod [shell_2]
  if (min (insidecone_2) ne -1) then	$
      density [insidecone_2] = dens_back [insidecone_2]

endif

;-- Section 3

section3 = where (rpB gt s3_ht and rpB le s4_ht)

if (min (section3) ne -1) then begin

;-- In the middle of the scoop.

  shell_3 = where (sinalpha lt sin (alpha_o_scoop * d2r) and sinalpha ge sin (alpha_i_scoop*d2r) and rpB gt s3_ht and rpB le s4_ht and sin (phipb * d2r) ge 0.0)
  insidecone_3 = where (sinalpha lt sin (alpha_i_scoop * d2r) and rpB gt s3_ht and rpB le s4_ht and sin (phipb * d2r) ge 0.0)

  if (min (shell_3) ne -1) then density [shell_3] = dens_mod [shell_3]
  if (min (insidecone_3) ne -1) then density [insidecone_3] = dens_back [insidecone_3]

endif

;-- Section 4

section4 = where (rpB gt s4_ht and rpB le s5_ht)

if (min (section4) ne -1) then begin

;-- At the of the scoop.
;   never inside cone

  shell_4 = where (sinalpha lt sin (alpha_o_scoop * d2r) and rpB gt s4_ht and rpB le s5_ht and sin (phipb * d2r) ge 0.0)

  if (min (shell_4) ne -1) then density [shell_4] = dens_mod [shell_4]
endif

coneedge = where (density ne dens_back)

end

;************************************************************************
pro dens5, rpB, thetapB, phipB

common newresults, density
common results,    Dens, Pres, bstrengthphys, brphys, bthphys, bphphys
common prams,      apar, xo, thetao, phio, rbub, ao, Pio, alpha, eta, $
                   velmult, cpar, dpar, alnot, sig1, out
common shiftcoord, Rcap, Thcap, Phcap

;-- First set up shell.
;   PARAMETER out will determine how thick the shell is.

out = 22. 

denscalc, rpB, thetapB, phipB
  
; density = dens
; goto, label1

;-- PUT in the shell density.
;  (everything but the background, cavity, and core)
;  PARAMETERS amplitude (cshell) and slope (dshell)

cshell = 4e08
dshell = 3.0

dens_shell = cshell * (rpB^(-1 * dshell))

density = dens_shell

;-- PUT in the cavity density.
;   PARAMETERS amplitude (ccav) and slope (dcav)

ccav = 0.0
dcav = 3.0

dens_cav = ccav * (rpB^(-1 * dcav))

cavity = where (dens lt 0.0)
if (min (cavity) ne -1) then density [cavity] = dens_cav [cavity]

;-- PUT in the background density.
;   PARAMETERS amplitude (cback) and slope (dback).

cback = 5e7
dback = 4.0

dens_back = cback * (rpB^(-1 * dback))

background = where (Rcap ge rbub)
if (min (background) ne -1) then	$
  density [background] = dens_back [background]

;-- PUT in the MODEL CORE DENSITY.
;   PARAMETER amplitude coremult
;   (out=0 means no front so pos. stuff in bubble core only)

out = 0.0

denscalc, rpB, thetapB, phipB

coremult = 1.0

core = where (Rcap lt rbub and dens ge 0.0)
if (min (core) ne -1) then density [core] = coremult * dens [core]

;-- NOTE this could be changed, e.g. replaced
;   by a power law or some other function.

label1:

end

;************************************************************************

pro dens6, rpB, thetapB, phipB

common params1,    Aprofhole, AprofNE, awidth, bwidth, thcs, phcs, alpha, m, w
common gridpiv3d,  rpiv, thpiv, phpiv
common newresults, density
common Bangle,     tilt

th = thetapB * !dtor
ph = phipB   * !dtor

tilt = 1

getgauss, density, rpB, th, ph

end

;************************************************************************
