/*
 * pbb3d.pro
 * *****************************************************************************
 * Program Dens.to.pb
 *
 *  Given an analytic function denscalc (rpb, thetapb, phipb), which
 *  can return an electron _number_ density given the physical
 *  3-d coordinates rpb, thetapb, phipb, this program will 
 *  calculate a plane-of-sky distribution of both brightness and 
 *  polarized brightness.
 *
 * *****************************************************************************
 */

/* ************************************************************************** */

/*** Global variables. ***/

int idens = 1;
int nx = 256 ;
int ny = 256 ;
int ntau ;
int onedconeedge = 0;
int test  = 0;
int test2 = 0;
int tlen  = 0;

float *pbimg;
float *brightimg;
float *thsave;
float x;
float y;
float *bright2d;
float *pb2d;

/*
common limbresults, pb, bright
common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, mult, full, idens, lat, Bang
common limbinfo, limbread
common edge, coneedge, onedconeedge
 */

main (int argc, char *argv [])

{
  int count;
  int endrot = 1;
  int full = 0;
  int ix;
  int iy;
  int limbread;
  int ntau;

  float pb;
  float bright;
  float r;
  float phi;
  float theta;

  float cmer      =  0.0;
  float dx        =  0.0;
  float maxheight =  1.5;		/* Rsun */
  float pi        =  3.14159265;
  float rwid      =  0.0;
  float taumin    = -1.57;
  float taumax    =  1.57;
  float tauint    =  0.1;
  float thetwid   =  0.0;

  ntau = (int) ((taumax - taumin) / tauint + 1.0);

  /*** Get inputs. ***/

  /* inputs (); */

  /* The plane of sky coordinates are calculated (r, theta, phi).
   *
   * I decided to do these evenly spaced in x-y so that we could
   * then display a 2D image (I knew there was a reason for an x-y grid!).
   */

  /*** However, since idl runs out of memory, we have to use a loop in ny. **/

  if (full == 0)  endrot =  1;
  if (full == 1)  endrot = 36;
  if (full == 2)  endrot =  3;

  pbimg        = (float *) malloc (nx * ny * endrot * sizeof (float));
  brightimg    = (float *) malloc (nx * ny * endrot * sizeof (float));
  thsave       = (float *) malloc (nx * ny          * sizeof (float));

  bright2d  = (float *) malloc (512 * 512        * sizeof (float));
  pb2d      = (float *) malloc (512 * 512        * sizeof (float));

  dx     = 2.0 * maxheight / (nx - 1.0);
  count  = 0;

  for (iy = 0; iy < ny; iy++)
    {
    for (ix = 0; ix < nx; ix++)
      {
      x = -maxheight + ix * (2 * maxheight / (nx - 1.0));
      y = -maxheight + iy * (2 * maxheight / (ny - 1.0));

      /*** Now convert to r, theta (which is what the pb integral wants). ***/

      r      = sqrt (x * x  + y * y );
      theta  = acos (y  / r );
  /*
   *      neg   = where (x lt 0.0);
   *      if (min (neg) != -1)  theta [neg] = 2.0 * pi - theta [neg];
   */
      if (x != -1) theta = 2.0 * pi - theta;

      /*** And your phi is the central meridian of the "day" you want. ***/

      phi = cmer + 10.0;

      phi = phi * pi / 180.0;		/* Convert to radians. */

      /*** Now calculate pb and B!. ***/

      limbread = 1;
      pbint (r , theta, phi);

      *(pbimg     + ix + iy*nx) = alog10 (pb     );
      *(brightimg + ix + iy*nx) = alog10 (bright );
      }

    /*** Check the apparent width. ***/

    if (idens == 4) 
      {
      if (min (onedconeedge) != -1)
        {
        thetwid = theta [onedconeedge];
        rwid    = r     [onedconeedge];
/*       print, rwid */
/*        test = where (abs (rwid - 1.0) lt sqrt (2.1) * dx); */
/*       stop */

        if (min (test) != -1)
	  {
          tlen = size (test);
          tlen = tlen (1);
          thsave [count:count + tlen - 1] = thetwid (test);
          count = count + tlen;
          }

        }
      }

    /*** Block out the solar disks. ***/

/*    test = where (r lt 1.0); */
    if (min (test) ne -1) then pbimg     [test, iy] = -100.0;
    if (min (test) ne -1) then brightimg [test, iy] = -100.0;
    }

  /*** Width. ***/

  if (idens == 4)
    {
    if (min (onedconeedge) != -1)
      {
      thsavefin = where (thsave ne 0.);
      print, 'thetas are', thsave [thsavefin] / !dtor;
      print, 'width is', max (thsave [thsavefin] / !dtor) - min (thsave [thsavefin] / !dtor);
      }
    }

  /* test2 = where (pbimg ne -100); */
  minpb = min (pbimg [test2]);
  maxpb = max (pbimg [test2]);
  test2 = 0;

  /*** Normalization for CASE 1 vs. CASE 2. ***/

  minpb = -2.5;
  maxpb =  2.5;

  /*** rebin. ***/

  /*
  pbimg     = rebin (pbimg,     512, 512, endrot)
  brightimg = rebin (brightimg, 512, 512, endrot)
   */

  /*** Display the pb image. ***/

  /*
  window, 2,	$
  xs=512,  ys=512,  retain=3, title='pb model'
   */

  /*
  if (full eq 2) then window, 2,	$
      xs=1024, ys=1024, retain=3, title='pb model'
   */ 

  for (iy = 0; iy < ny; iy++)
    for (ix = 0; ix < nx; ix++)
      {
      *(pb2d + ix + iy*nx) = *(pbimg + ix + iy*nx);
      }

  /*** And plot. ***/

    /* tv, bytscl (pb2d, min=minpb, max=maxpb) */

  /*** And plot. ***/

  /*
   ;	print, 'type .c to continue to modeled brightness image'
   ;	stop
   */

  /*** Show the brightness movie. ***/

  /* window, 3, xs=512, ys=512, retain=3, title='brightness model' */

  for (iy = 0; iy < ny; iy++)
    for (ix = 0; ix < nx; ix++)
      *(bright2d + ix + iy*nx) = *(brightimg + ix + iy*nx);

  /* tv, bytscl (bright2d, min=minpb, max=maxpb) */

  exit (0);
}

/* ************************************************************************** */

/* pro inputs */
	
/*
common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, mult, full, idens, lat, Bang
common densrequest, ndens
common sscoord, rsquig, phiss, time
common prams, apar, xo, thetao, phio, rbub, ao, Pio, alpha, eta, velmult, cpar, dpar, alnot, sig1, out
 */

void
inputs ()

{

/*
read, 'What density routine? dens1 - 1, dens2 - 2, dens3 - 3, dens4 (ice cream cone) - 4, G&L model dens - 5 , or Spartan 3d  -- 6 ', idens
 */

  ;-- Read in prams for G&L model.

  if (idens == 5)
    {
    /* read, 'What time step? in hours (0. default) ', time */
    time = time * 3600.
    accel = 0

  /*
    openr, 5, 'pramsset.d'
    readprams
    close, 5
   */

  /* read, 'What orientation? along l.o.s. (90.), perp to l.o.s. (0.) ', sig1 */
    }

  /*** Set up for Spartan. ***/

  if (idens == 6) params ();

  /*** How far out do you extend? ***/

  /* read, 'To what height do you wish the image to go? (Rsun) ', maxheight */

  /*** What grid spacing do you want? ***/
       
  /* read, 'Input grid dimensions (e.g. 256X256, type 256) ', nx */
  ny =  nx

  /* The l.o.s. integrating angle tau goes basically from -pi/2 to pi/2.
   * tauint = .1 is the integration step I've been using, but you could
   * them finer or coarser if you wish.
   */
  /*read, 'Maximum l.o.s integrating angle? (type 1.57 for default) ', taumax */
  /*read, 'l.o.s integrating angle step-size? (type .1 for default) ', tauint */

  /* IF the model you are using is azimuthally symmetric, then may as
   * well only integrate from 0 to pi/2.  Here's a check for that:
   */

  /* read, 'Azimuthally symmetric model? (yes = 1, no = 0) ', nsymm */

  if (nsymm == 0)
     {
     taumin = -1 * taumax;
     mult   = 1.0;
     }

  if (nsymm == 1)
     {
     taumin = 0.0;
     mult   = 2.0;
     }

  ntau = fix ((taumax - taumin) / tauint + 1.);

  /* What central meridian phi do you want?   (radians)
   * (You may want to step through these do see your structure
   * rotate round if it's not azimuthally symmetric).
   */

  cmer = 0.0;
  /* if (nsymm eq 0) then read, 'What central meridian longitude (degrees)? (0 puts structures at west limb for CME models) ', cmer */

  /*** Also position in latitude. ***/

  lat = 0.0;
  /* read, 'What change in latitude (degrees)? (0 returns structure centered at equator, +/-  is below/above ', lat */

  Bang = 0.0;
  /* read, 'What change in viewing angle (i.e. B angle tilt) (degrees)? (0 is 0 B angle, +/-  is below/above ', Bang */

  full = 0;
  /* if (nsymm eq 0) then read, 'Do you want a single image (0), a full rotation (1), or three STEREO images separated by 50 degrees (2) ', full */
}

/* ************************************************************************** */

/* Calculates integrated line-of-sight polarized brightness pb (r, theta, phi)
 * and brightness bright (r, theta, phi) from density produced by 
 * denscalc (rpb, thetapb, phipb) and integrated over tau
 * (line of sight angle).
 */

/* NOTE:  coordinates r, theta, phi are plane of sky coordinates
 * rpb, thetapb, phipb are "physical" spherical 3-d coordinates.
 */

/* pro pbint, r, theta, phi */

/*
common newresults, density
common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, mult, full, idens, lat, Bang
common limbresults, pb, bright
common densrequest, ndens
common limbinfo, limbread
common edge, coneedge, onedconeedge
 */
 
int
pbint (float r, float theta, float phi)

{

  float *bfun (float r [], int ntau, int nx);
  float *cfun (float r [], int ntau, int nx);

  float *alfa;
  float *alfap;
  float *zeta;
  float *thetapb
  float *rpb;
  float *r2;
  float *r3;
  float *phipb;
  float *fgt;

  float d2r;
  float pi = 3.14159265;
  float pid2;

  int itau;

  alfa    = malloc (nx * sizeof (float));
  alfap   = malloc (nx * sizeof (float));
  zeta    = malloc (ntau * nx * sizeof (float));
  thetapb = malloc (ntau * nx * sizeof (float));
  rpb     = malloc (ntau * nx * sizeof (float));
  r2      = malloc (ntau * nx * sizeof (float));
  r3      = malloc (ntau * nx * sizeof (float));
  phipb   = malloc (ntau * nx * sizeof (float));
  fgt     = malloc (ntau *      sizeof (float));

  /*** Degrees to radian.

  d2r = 1.745329e-2;

  /*** pi/2 ***/

  pid2 = pi / 2.0;
 
  for (itau = 0; itau < ntau; itau++)
    fgt [itau] = (float) itau;

  /* Both limbs will be represented in the plane of the sky:
   * the angle phi is the central meridian angle
   */

  /* alfa +ve for West (right) limb
   * alfa -ve for East (left) limb
   */ 

  alfa = sin (theta) / abs (sin (theta));
  if (limbread != 0)  alfap = alfa * limbread;
  if (limbread == 0)  alfap = alfa;
 
  tauslop    = (taumax - taumin) - tauint * (ntau - 1);
  tauminslop = taumin + tauslop / 2.0;

  for (itau = 0; itau < ntau; itau++)
    {
    tau   [itau] = tauminslop + tauint * (fgt [itau]);
    tnorm [itau] = (1 + fgt (itau)) / (1 + fgt (ntau));
    }

  for (itau = 0; itau < ntau; itau++)
    {
    zeta    [itau] = atan (tan (tau [itau]) * (1 / sin (theta)));
    rpb     [itau] = (1 / cos (tau [itau])) * r;
    r2      [itau] = tnorm * r;
    thetapb [itau] = acos (cos (tau [itau]) * cos (theta)) / d2r;
    phipb   [itau] = (tnorm * alfap * pid2 - alfap * zeta [*]) / d2r;
    }

  /*** First add Bangle if necessary. ***/

  Bangrot (rpb, thetapb, phipb, thetauseB, phiuseB);

  for (itau = 0; itau < ntau; itau++)
    phiuseB [itau] = phiuseB [itau] + tnorm * phi [itau] / d2r;

  /*** Now put in model structure rotation if necessary. ***/

  test = where (phiuseB gt 360.);
  if (min (test) != -1)  phiuseB [test] = phiuseB [test] - 360.;
  test = where (phiuseB < 0.);
  if (min (test) != -1)  phiuseB [test] = phiuseB [test] + 360.;

  throt (rpb, thetauseB, phiuseB, thetause, phiuse);

  test = where (phiuse > 360.);
  if (min (test) != -1)  phiuse [test] = phiuse [test] - 360.;
  test = where (phiuse < 0.);
  if (min (test) != -1)  phiuse [test] = phiuse [test] + 360.;

  if (idens == 1)  dens1 (rpb, thetause, phiuse);
  if (idens == 2)  dens2 (rpb, thetause, phiuse);
  if (idens == 3)  dens3 (rpb, thetause, phiuse)'
  if (idens == 4)  dens4 (rpb, thetause, phiuse);
  if (idens == 5)  dens5 (rpb, thetause, phiuse);
  if (idens == 6)  dens6 (rpb, thetause, phiuse);

  /*** Get width info. ***/

  if (idens == 4)
    {
    widthflag = density * 0.;
    if (min (coneedge) != -1)  widthflag [coneedge] = 1.0;
    }

  /*** First calculate pb. ***/

  cff = cfun (rpb, ntau, nx);

  sumpb = malloc (ntau * nx * sizeof (float));

  sumpb = cff * r2 * tauint;
  sumpb = density * sumpb;
  pb    = malloc (nx * sizeof (float));

  /*** Now calculate bright. ***/

  bff = bfun (rpb, ntau, nx);

  bright = malloc (nx * sizeof (float));
  sumB   = malloc (ntau * nx * sizeof (float));

  sumB   = bff * tauint * rpb [itau] * rpb [itau] / r2;
  sumB   = 2.0 * density * sumB;

  /*** Now integrate both and set up width array. ***/

  onedcone = malloc (nx * sizeof (float));
  count = 0;

  for (itau = 1; itau < ntau; itau++)
    {
    pb     = pb     + sumpb [itau];
    bright = bright + sumB  [itau] - sumpb [itau];
    }

  if (idens == 4)
    {
    onedtemp = where (widthflag [i, *] eq 1.);
    if (min (onedtemp) != -1)
      {
      onelen = size (onedtemp);
      onelen = onelen (1);
      onedcone [count:count + onelen - 1] = onedtemp;
      count = count + onelen;
      }
    }

  if (idens == 4)
    {
    test = where (onedcone ne 0.);
    if (min (test) != -1)  onedconeedge = onedcone [test];
    if (min (test) == -1)  onedconeedge = -1;
    }

  pb = mult * pb;
  bright = mult * bright;

  /*** And add in plane of sky info for the symmetric case. ***/

  pb = pb + sumpb [0, *];
  bright = bright + sumB [0, *] - sumpb [0, *];

}

/* ************************************************************************** */

/* function bfun, r */

float *
bfun (float r, int ntau, int nx)

{
  ;-- Calculates scattering function needed for brightness calculation.
 
  float a1, a2, a3;
  float *cfunc ;
  float df ;
  float dfunc ;
  float bf ;

  a1 = 8.69d-7;
  a2 = 0.37;
  a3 = 0.63;
 
  /*** Allocate storage. ***/

  cfunc = malloc (ntau * nx * sizeof (float));
  df    = malloc (ntau * nx * sizeof (float));
  dfunc = malloc (ntau * nx * sizeof (float));
  bf    = malloc (ntau * nx * sizeof (float));
 
  cfunc = 4.0 / 3.0 - sqrt (1.0 - 1.0/r/r) - ((1.0 - 1.0/r/r)^(1.5)) / 3.0;
  df    = (r - 1.0/r) * (5 - 1.0/r/r) * alog ((1.0 + 1.0/r) / sqrt(1.0-1.0/r/r));
  dfunc = 1.0 / 8 * (5.0 + 1.0/r/r - *df);
  bf    = a1 * (a2 * *cfunc + a3 * *dfunc);
 
  free (cfunc);
  free (df) ;
  free (dfunc);

  return (bf);
}

/* ************************************************************************** */

/* function cfun, r */

float *
cfun (float r, int ntau, int nx)

{
 
  /* Calculates scattering function needed for brightness calculation
   * and also polarized brightness.
   */

  float a1, a2, a3;
  float *afunc, *bf, *bfunc, *cf;

  a1 = 8.69d-7;
  a2 = 0.37;
  a3 = 0.63;
 
  /*** Allocate storage. ***/

  afunc = malloc (ntau * nx * sizeof (float));
  bf    = malloc (ntau * nx * sizeof (float));
  bfunc = malloc (ntau * nx * sizeof (float));
  cf    = malloc (ntau * nx * sizeof (float));
 
  *afunc = sqrt (1.0 - 1.0/r/r) * (1.0/r/r);
  *bf    = (r - 1.0/r) * (1 + 3.0/r/r) * alog ((1.0 + 1.0/r) / sqrt(1.0-1.0/r/r));
  *bfunc = -1.0 / 8 * (1 - 3/r/r - *bf);
  cf    = a1 * (a2 * *afunc + a3 * *bfunc);
 
  free (afunc) ;
  free (bf) ;
  free (bfunc) ;

  return (cf);
}

/* ************************************************************************** */

/* pro Bangrot, rpb, thetapb, phipb, thetauseB, phiuseB */

/*
common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, mult, full, idens, lat, Bang
 */

int
Bangrot (float rpb, float thetapb, float phiB, float thetauseB, float phiuseB)

{

  float *phiuseB;
  float *thetauseB;

  phiuseB   = malloc (ntau * nx * sizeof (float));
  thetauseB = malloc (ntau * nx * sizeof (float));

  d2r = 1.745329e-2;

  for (itau = 0; itau < ntau; itau++)
    {
    x = rpb [itau] *
        sin (d2r * thetapb [itau]) *
        sin (d2r * phipb   [itau]);
    y = rpb [itau] *
        sin (d2r * thetapb [itau]) *
        cos (d2r * phipb   [itau]);
    z = rpb [itau] * cos (d2r * thetapb [itau]);
    }

  /* Change line of sight due to B angle  tilt.
   * Arfkin p 199
   */

  for (itau = 0; itau < ntau; itau++)
    {
    xp [itau] = x [itau];
    yp [itau] = y [itau] * cos (Bang * d2r) - z * sin (Bang * d2r);
    zp [itau] = y [itau] * sin (Bang * d2r) + z * cos (Bang * d2r);

    thetauseB [itau] = acos (zp / rpb [itau]);
    phiuseB   [itau] = thetauseB [itau] * 0.d0;
    }

  test = where (yp != 0.d0);
  if (min (test) != -1)  phiuseB [test] = atan (xp [test] / yp [test]);
  test = where (yp eq 0.d0 and xp gt 0.d0);
  if (min (test) != -1)  phiuseB [test] = !pi / 2.d0;
  test = where (yp eq 0.d0 and xp lt 0.d0);
  if (min (test) != -1) then phiuseB [test] = -!pi / 2.d0;
  if (min (test) != -1) then stop;
  test = where (yp < 0.0d0);
  if (min (test) != -1) 
    phiuseB [test] = phiuseB [test] + double (!pi);

  for (itau = 0; itau < ntau; itau++)
    {
    thetauseB [itau] = thetauseB [itau] / d2r;
    phiuseB   [itau] = phiuseB   [itau] / d2r;
    }
}

/* ************************************************************************** */

/*pro throt, rpb, thetauseB, phiuseB, thetause, phiuse */

/*
common quants, ntau, nx, ny, cmer, maxheight, taumin, taumax, tauint, mult, full, idens, lat, Bang
 */

int
throt (rpb, thetauseB, phiuseB, thetause, phiuse)

float rpb;
float thetauseB;
float phiuseB;
float thetause;
float phiuse;
{

  float d2r= 1.745329e-2;

  for (itau = 0; itau < ntau; itau++)
    {
    x [itau] = rpb [itau] *
		   sin (d2r * thetauseB [itau]) *
		   sin (d2r * phiuseB [itau]);
    y [itau] = rpb [itau] *
		   sin (d2r * thetauseB [itau]) *
		   cos (d2r * phiuseB [itau]);
    z [itau] = rpb [itau] * cos (d2r * thetauseB [itau]);
    }

  /* Rotate dens structure at limb by theta = lat. ***/
   * Arfkin p 199
   */

  for (itau = 0; itau < ntau; itau++)
    {
    yp [itau] =  y [itau];
    xp [itau] =  x [itau] * cos (lat * d2r) +
		     z [itau] * sin (lat * d2r);
    zp [itau] = -x [itau] * sin (lat * d2r) +
		     z [itau] * cos (lat * d2r);

    thetause [itau] = acos (zp [itau] / rpb [itau]);
    phiuse   [itau] = thetause [itau] * 0.d0;
    }

  test = where (yp != 0.d0);
  if (min (test) != -1)  phiuse [test] = atan (xp [test] / yp [test]);
  test = where (yp == 0.d0 and xp gt 0.d0);
  if (min (test) != -1)  phiuse [test] =  !pi / 2.d0;
  test = where (yp == 0.d0 and xp lt 0.d0);
  if (min (test) != -1)  phiuse [test] = -!pi / 2.d0;
  test = where (yp < 0.0d0);
  if (min (test) != -1)  phiuse [test] = phiuse [test] + double (!pi);

  for (itau = 0; itau < ntau; itau++)
    {
    thetause [itau] = thetause [itau] / d2r;
    phiuse   [itau] = phiuse   [itau] / d2r;
    }
}

/* ************************************************************************** */

/* pro dens1, rpb, thetapb, phipb */

/*
common newresults, density
 */
 
int
dens1 (float rpb [][], float thetapb, float phipb)

{

  /*** Parameters amplitude (cpar) and slope (dpar). ***/

  float cpar = 1e8;
  float dpar = 2.0;

  for (itau = 0; itau < ntau; itau++)
    {
    density [itau] = cpar * (rpb [itau]^(-1 * dpar));

    thetapb [itau] = thetapb [itau] + 30;
    }

  apar = 5e8;
  bpar = 2;
  test = where (thetapb gt 80 and thetapb lt 110 and phipb gt 80 and phipb lt 120);
  if (min (test) != -1)
    density [test] = apar * (rpb [test]^(-1 * bpar));

  apar = 5e8;
  bpar = 2;
  test = where (thetapb gt 130 and thetapb lt 150 and phipb gt 10 and phipb lt 30);
  if (min (test) != -1)
    density [test] = apar * (rpb [test]^(-1 * bpar));
}

/* ************************************************************************** */

/* pro dens2, rpb, thetapb, phipb */

/*
common newresults, density
*/

int
dens2 (float rpb [][], float thetapb, float phipb)

{

/*** Parameters amplitude (cpar) and slope (dpar). ***/

float cpar = 1e8;
float dpar = 2.0;

  for (itau = 0; itau < ntau; itau++)
    density [itau] = cpar * (rpb [itau]^(-1 * dpar));

float apar = 5e8;
float bpar = 2;

test = where (rpb [][] * abs ((cos (thetapb * !pi / 180.)))^.5 lt 1.);
if (min (test) != -1)
    density [test] = apar * (rpb [test]^(-1 * bpar));

}

;************************************************************************

/* pro dens3, rpb, thetapb, phipb */

/*
common newresults, density
*/

int
dens3 (float rpb [][], float thetapb, float phipb)

{

/*** Parameters amplitude (cpar) and slope (dpar). ***/

cpar = 1e8;
dpar = 2.0;

  for (itau = 0; itau < ntau; itau++)
    density [itau]= cpar * (rpb [itau]^(-1 * dpar));

test = where (rpb [][] * abs ((cos (thetapb * !pi / 180.)))^.5 lt 1. and phipb gt 180.);
if (min (test) ne -1) then density [test] = cpar * (rpb [test]^(-1 * dpar)) * ((cos (phipb [test] * !pi / 180.))^2)^(1. / 3.);

}

;************************************************************************
/* pro dens4, rpb, thetapb, phipb */

/*
common newresults, density
common edge, coneedge, onedconeedge
*/

int
dens4 (float rpb [][], float thetapb, float phipb)

{

float d2r= 1.745329e-2;

/* BACKGROUND DENSITY.
 * PARAMETERS amplitude (cpar) and slope (dpar)
 */

float cpar = 1e8;
float dpar = 3.0;

  for (itau = 0; itau < ntau; itau++)
    dens_back [itau] = cpar * (rpb [itau]^(-1 * dpar));

/* ENHANCED MODEL DENSITY.
 * PARAMETERS amplitude (cmod) and slope (dmod)
 */

/*** CASE 1 ***/
float cmod = 1e09;

/*** CASE 2 ***/
/* ; cmod = 2e09 */

float dmod = 2.0;

  for (itau = 0; itau < ntau; itau++)
    dens_mod [itau]= cmod * (rpb [itau]^(-1 * dmod));

/*** PUT in the background density. ***/

  for (itau = 0; itau < ntau; itau++)
    density [itau] = dens_back [itau];

/*** PUT IN ICE CREAM CONE. ***/


/*** PARAMETERS angles. ***/

/*** CASE 1 ***?
float alpha_o = 35.0;
float alpha_i = 30.0;

/*** CASE 2 ***/
/* ; alpha_o = 55.0 */
/* ; alpha_i = 50.0 */

/* Parameter R_c  --> distance from origin to center of the ice cream scoop. */

/*** CASE 1 ***/
float R_c =  4.0;

/*** CASE 2 ***/
/* ; R_c =  2.7 */

/*** PARAMETERS rho_i and rho_o inner and outer radius of the scoop. ***/

float rho_i = R_c * sin (alpha_i*d2r);
float rho_o = R_c * sin (alpha_o*d2r);

/*** PARAMETERS sectionheights - s1_ht, s2_ht, s3_ht, s4_ht, s5_ht. ***/

float s1_ht = 1.0;
float s2_ht = R_c-rho_o * sin (alpha_o * d2r);
float s3_ht = R_c-rho_i * sin (alpha_i * d2r);
float s4_ht = R_c + rho_i;
float s5_ht = R_c + rho_o;

/** Define relationship between cone angle and spherical coordinate angles. **/

  for (itau = 0; itau < ntau; itau++)
    sinalpha [itau] =  sqrt (((sin (thetapb [itau] * d2r))^2) * ((cos (phipb [itau] * d2r))^2) + (cos (thetapb [itau] * d2r)^2));

/*** Define variation of inner and outer curves in the scoop. ***/

alpha_o_scoop = (atan (sqrt ((sin (alpha_o * d2r)^2) - (rpb [itau] / R_c - 1.)^2.) / (rpb [itau] / R_c))) / d2r;
alpha_i_scoop = (atan (sqrt ((sin (alpha_i * d2r)^2) - (rpb [itau] / R_c - 1.)^2.) / (rpb [itau] / R_c))) / d2r;

test = where (alpha_o_scoop le 0. or alpha_o_scoop ge 90.) ;
if (min (test) != -1)
  {
  print, alpha_o_scoop, 'outer: out of first quadrant';
  stop
  }

test = where (alpha_i_scoop le 0. or alpha_i_scoop ge 90.) ;
if (min (test) != -1)
  {
  print, alpha_i_scoop, 'inner: out of first quadrant';
  stop
  }

/*** Section 1 ***/

/*** Legs of cone, cone angles constant. ***/

section1 = where (rpb gt s1_ht and rpb le s2_ht);

if (min (section1) != -1)
  {
  shell_1 = where (sinalpha lt sin (alpha_o*d2r) and sinalpha ge sin (alpha_i * d2r) and rpb gt s1_ht and rpb le s2_ht and sin (phipb * d2r) ge 0.);
  insidecone_1 = where (sinalpha lt sin (alpha_i*d2r) and rpb gt s1_ht and rpb le s2_ht and sin (phipb * d2r) ge 0.);

  if (min (shell_1) != -1)  density [shell_1] = dens_mod [shell_1];
  if (min (insidecone_1) != -1)
      density (insidecone_1) = dens_back (insidecone_1);

/* ;  if (min (shell_1 ne -1)) then stop */

  }

/*** Section 2 ***/

section2 = where (rpb gt s2_ht and rpb le s3_ht);

if (min (section2) != -1)
  {
  /*** Above the legs - outer cone angle varies with height. ***/

  shell_2 = where (sinalpha lt sin (alpha_o_scoop * d2r) and sinalpha ge sin (alpha_i * d2r) and rpb gt s2_ht and rpb le s3_ht and sin (phipb * d2r) ge 0.);
  insidecone_2 = where (sinalpha lt sin (alpha_i * d2r) and rpb gt s2_ht and rpb le s3_ht and sin (phipb * d2r) ge 0.);

  if (min (shell_2) != -1)  density [shell_2] = dens_mod [shell_2];
  if (min (insidecone_2) != -1) 
      density [insidecone_2] = dens_back [insidecone_2];

  }

/*** Section 3 ***/

section3 = where (rpb gt s3_ht and rpb le s4_ht);

if (min (section3) != -1)
  {
  /*** In the middle of the scoop. ***/

  shell_3 = where (sinalpha lt sin (alpha_o_scoop * d2r) and sinalpha ge sin (alpha_i_scoop*d2r) and rpb gt s3_ht and rpb le s4_ht and sin (phipb * d2r) ge 0.);
  insidecone_3 = where (sinalpha lt sin (alpha_i_scoop * d2r) and rpb gt s3_ht and rpb le s4_ht and sin (phipb * d2r) ge 0.);

  if (min (shell_3) ne -1) then density [shell_3] = dens_mod [shell_3];
  if (min (insidecone_3) ne -1) then density [insidecone_3] = dens_back [insidecone_3];
  }

/*** Section 4 ***/

section4 = where (rpb gt s4_ht and rpb le s5_ht);

if (min (section4) != -1)
  {

  /* At the of the scoop.
   * never inside cone.
   */

  shell_4 = where (sinalpha lt sin (alpha_o_scoop * d2r) and rpb gt s4_ht and rpb le s5_ht and sin (phipb * d2r) ge 0.);

  if (min (shell_4) != -1)  density [shell_4] = dens_mod [shell_4];
  }

coneedge = where (density ne dens_back);

}

;************************************************************************
/* pro dens5, rpb, thetapb, phipb */

/*
common newresults, density
common results, Dens, Pres, bstrengthphys, brphys, bthphys, bphphys
common prams, apar, xo, thetao, phio, rbub, ao, Pio, alpha, eta, velmult, cpar, dpar, alnot, sig1, out
common shiftcoord, Rcap, Thcap, Phcap
 */

int
dens5 (float rpb [][], float thetapb, float phipb)

{

/* First set up shell.
 * PARAMETER out will determine how thick the shell is.
 */

out = 22.0 ;

denscalc (rpb, thetapb, phipb);
  
/*
; density = dens
; goto, label1
 */


/* PUT in the shell density.
 * (everything but the background, cavity, and core)
 * PARAMETERS amplitude (cshell) and slope (dshell)
 */

float cshell = 4e08;
float dshell = 3.0;

  for (itau = 0; itau < ntau; itau++)
    {
    dens_shell [itau] = cshell * (rpb [itau]^(-1 * dshell));

    density [itau] = dens_shell [itau];
    }

/* PUT in the cavity density.
 * PARAMETERS amplitude (ccav) and slope (dcav)
 */

float ccav = 0.0;
float dcav = 3.0;

  for (itau = 0; itau < ntau; itau++)
    dens_cav [x][itau] = ccav * (rpb [itau]^(-1 * dcav));

cavity = where (dens lt 0.0);
if (min (cavity) != -1)  density [cavity] = dens_cav [cavity];

/* PUT in the background density.
 * PARAMETERS amplitude (cback) and slope (dback).
 */

float cback = 5e7;
float dback = 4.0;

  for (itau = 0; itau < ntau; itau++)
    dens_back [itau] = cback * (rpb [itau]^(-1 * dback));

background = where (Rcap ge rbub);
if (min (background) != -1) 
  density [background] = dens_back [background];

/* PUT in the MODEL CORE DENSITY.
 * PARAMETER amplitude coremult
 * (out=0 means no front so pos. stuff in bubble core only)
 */

float out = 0.0;

denscalc (rpb, thetapb, phipb);

float coremult = 1.0;

core = where (Rcap lt rbub and dens ge 0.);
if (min (core) != -1) density [core] = coremult * dens [core];

/* NOTE this could be changed, e.g. replaced
 * by a power law or some other function.
 */

label1:

}

;************************************************************************

/* pro dens6, rpb, thetapb, phipb */

/*
common params1, Aprofhole, AprofNE, awidth, bwidth, thcs, phcs, alpha, m, w
common gridpiv3d, rpiv, thpiv, phpiv
common newresults, density
common Bangle, tilt
 */

int
dens6 (float rpb [][], float thetapb, float phipb)

{

  for (itau = 0; itau < ntau; itau++)
    {
    th [itau] = thetapb [itau] * !dtor;
    ph [itau] = phipb   [itau] * !dtor;
    }

tilt = 1;

getgauss (density, rpb, th, ph);

}

;************************************************************************
