;+
; NAME
;	mk4avegif
;
; PURPOSE
;	Annotates & scales mk4_best/*.slit file.
;	Stores GIF image in ~ftp/mk4/daily_images/CCYY/NN_MON.
;
; SYNTAX
;	mk4avegif, filename
;
; INPUT
;	filename	File name of *.rpB.avg.slit image.
;
; HISTORY
;	Modified from "mkqual.auto.pro", written by Alice Lecinski.
;	Andrew L. Stanger   HAO/NCAR   16 September 1999
;-

PRO mk4avegif, filename

@mk4h.pro		; Mk4 binary header structure.

daytab = INTARR (13, 2)
daytab = [[0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ],  $
          [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]]

montab = STRARR (12)
montab = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', $
	  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
imontb = STRARR (12)
imontb = ['01_', '02_', '03_', '04_', '05_', '06_', $
	  '07_', '08_', '09_', '10_', '11_', '12_']

seegif = 0
fixscl = 1
dotest = seegif

IF (seegif) THEN BEGIN ;{
    gif = 0
    gif = BYTARR (512,512)
    window, 0, xsize = 512, ysize = 512
    window, 2, xsize = 512, ysize = 512
ENDIF;}

tscpfile = ''
avgtimes = ''
numimg = 1

npix   = 384		; # pixels in reticon array.
nray   = 1080 		; # radial scans. */
nraym1 = nray - 1

acirc  = !PI * 2.0 / FLOAT (nray)
dp     = FINDGEN (nray) * acirc  +  acirc * (0.25 * (FLOAT (nray)))
dpx    = INTARR  (nray)
dpy    = INTARR  (nray)
xcen   = 255.5
ycen   = 255.5
mk4rad =  89.81			; # pixels / Rsun.
radius = 1.00
rnder  = 0.5005
dpx    = FIX ( cos (dp) * mk4rad * radius + xcen + rnder )  ; For disk size
dpy    = FIX ( sin (dp) * mk4rad * radius + ycen + rnder )  ; For disk size

;;;  r0 = 1.016486
;;;  dr = (255.5 / mk4rad - r0) / 384.0
;;;  rmx =  2.45

rmin   = 1.016486	; height of lowest pixel (Rsun @ 960 arcsec/radius). */
r0     = 1.25
dr     = (255.5 / mk4rad - rmin) / FLOAT (npix) 
rmx    =  2.00
rpix   = FIX ((rmx - r0) / dr) + 1
rpixm1 = rpix - 1

rimg   = FLTARR (rpix, nray)
fpix   = FLOAT (nray) * FLOAT (rpix)

!order = 0

rpB = 0
rpB = INTARR (512, 512)
                  
SET_PLOT, 'z'
DEVICE, set_resolution = [512, 512], set_colors = 256, z_buffering = 0

;--- Load color table

CLOSE, 11
red = BYTARR (256)  &  green = BYTARR (256)  &  blue = BYTARR (256)

OPENR, 11, '/home/cordyn/color_idl/qual.rgb' 
   READU, 11, red  &  READU, 11, green  &  READU, 11, blue
CLOSE, 11

green [0] = 5

TVLCT, red, green, blue 
                  
;--- Request file name (if not provided on command line).

mn = -800 & mx = 4000
IF ( N_ELEMENTS (filename) EQ 0) THEN BEGIN ;{
    filename = ' '
    PRINT,	$
      'Enter a filename (*.rpB.avg.slit) : '
    PRINT, 'The name should include the entire path to the image.'
    READ, filename
ENDIF

   ;--- Read input image.

    CLOSE, 1
    OPENR, 1, filename
        READU, 1, mk4h1		; Read Mk4 binary header (part 1).
;        READU, 1, mk4h2		; Read Mk4 binary header (part 2).

        POINT_LUN, 1, 224
        READF, 1, tscpfile	; Read telescope file name from header.

        POINT_LUN, 1, 288
        READF, 1, avgtimes	; Read image times for average image.

        POINT_LUN, 1, 1024
        READU, 1, rpB		; Read Mk4 image pixel data.
    CLOSE, 1

;    PRINT, 'tscpfile: ', tscpfile
    PRINT, 'avgtimes: ', avgtimes

    ;--- Write image times to a temporary file.

    OPENW,  2, 'mk4.avg.times'
    PRINTF, 2, avgtimes
    CLOSE,  2

    ;--- View image option.

    IF (seegif) THEN BEGIN ;{
	SET_PLOT, 'z'
	DEVICE, set_resolution = [512, 512], set_colors = 256, z_buffering = 0
    ENDIF;}

    ;--- Fixed scaling option.

    IF (fixscl) THEN BEGIN ;{
	FOR ii = 0, rpixm1, 1 DO BEGIN ;{
	    RR = r0 + dr * FLOAT (ii)
	    Rpx = FIX ( cos (dp) * mk4rad * RR + xcen + rnder )
	    Rpy = FIX ( sin (dp) * mk4rad * RR + ycen + rnder )
	    rimg (ii, 0:nraym1) = rpB (Rpx, Rpy)
	ENDFOR ;}

	avg   = TOTAL (rimg) / fpix
	diff  = rimg - avg 
	sigma = sqrt ( TOTAL (diff * diff) / fpix)
	 
	mn = FIX (FIX (avg /  50.0) *  50.0 - 2.5 * FIX (sigma /  50.0) *  50.0)
	mx = FIX (FIX (avg / 100.0) * 100.0 + 6.0 * FIX (sigma / 100.0) * 100.0)

	PRINT, 'Raw avg, sigma:', avg, sigma, '  min/max: ', mn, mx

	;--- Cull out the way too big and little values.

	rimg =  rimg * ((rimg GT mn) AND (rimg LE mx))

	;--- Recompute the avg and sigma.

	avg   = TOTAL (rimg) / fpix
	diff  = rimg - avg 
	sigma = sqrt ( TOTAL (diff * diff) / fpix)
	 
	PRINT, 'Cleaned avg,  sigma:', avg, sigma
	mn = FIX (FIX (avg / 50.0)* 50.0 - 1.4 * FIX (sigma / 50.0) * 50.0)
	mx = FIX (FIX (avg / 100.0) * 100.0 + 4.5 * FIX (sigma / 100.0) * 100.0)
	mn = FIX (FIX ((avg - 1.3 * sigma) / 50.0) * 50.0)  

;;;     if (mn GT 0) then mn = 0

	mx = FIX (FIX ((avg + 4.2 * sigma) / 100.0) * 100.0)

	;--- Cull out the way too big and little values.

;       rimg =  rimg * ((rimg GT mn) AND (rimg LE mx))

    ENDIF ;}

    ;--- Get the times (hours & minutes) used for average image.

;    cmnd = "set mk4tmp = `grep '^time' "+filename+$
;         " | sed 's/time //'` ; echo $#mk4tmp > bzbzbz; echo $mk4tmp >> bzbzbz"

    cmnd = "set mk4tmp = `cat mk4.avg.times` ;" + $
	   " echo $#mk4tmp > bzbzbz; echo $mk4tmp >> bzbzbz"
    SPAWN, cmnd

    CLOSE, 1
    OPENR, 1, 'bzbzbz'
    READF, 1, numimg 
;    PRINT, 'numimg: ', numimg

	IF (numimg GE 60) THEN numimg = 59
	       times = INTARR (numimg)
	       hrs   = FLTARR (numimg)

	IF (numimg EQ 1) THEN BEGIN ;{
	       ihrs = ' '
	       READF, 1, ihrs
	       CLOSE, 1 
	       times (0) = 100 * FIX (STRMID (ihrs, 0, 2))		$
			       + FIX (STRMID (ihrs, 3, 2))
	ENDIF ELSE BEGIN ;}{
	       READF, 1, times
	       CLOSE, 1 
	ENDELSE ;}

;    PRINT, 'times: ', times


    ;--- Convert hours & minutes to time in floating point hour units.

    hrs = 1.0 * FIX (times / 100.0)			; hours
    hrs = hrs + (times - (hrs * 100.0)) / 60.0		; hours + min / 60.0

    ;--- Add 24 hours (effectively, the next day) if hours < 10.

    FOR ii = 0, numimg - 1 DO BEGIN
	IF ( hrs (ii) LE 10 ) THEN hrs (ii) = hrs (ii) + 24.0
    ENDFOR

    ;--- Compute average time.

        ftime = TOTAL (hrs) / (FLOAT (numimg))
        myhr  = FIX (ftime)
	mymin = FIX ((ftime - myhr * 1.0) * 60.0)
        avtime = STRCOMPRESS (STRING (myhr) + ':' + STRING (mymin), /remove_all)
        IF (mymin LT 10) THEN $
           avtime = STRCOMPRESS (STRING (myhr) + ':0'	$
		  + STRING (mymin), /remove_all)

    ;--- Compute beginning time.

        myhr  = FIX  (MIN (hrs))
	mymin = FIX ((MIN (hrs) - myhr * 1.0) * 60.0)
        btime = STRCOMPRESS (STRING (myhr) + ':' + STRING (mymin), /remove_all)
        IF (mymin LT 10) THEN $
            btime = STRCOMPRESS (STRING (myhr) + ':0'	$
		  + STRING (mymin), /remove_all)

    ;--- Compute ending time.

        myhr  = FIX  (MAX (hrs))
	mymin = FIX ((MAX (hrs) - myhr * 1.0) * 60.0)

	IF (myhr GE 24 ) THEN myhr = myhr - 24
	ssmyhr = STRING (myhr)
	IF ( myhr LT 10 )  THEN ssmyhr = '0' + STRING (myhr)
	smymin = STRING (mymin)
	IF ( mymin LT 10 ) THEN smymin = '0' + STRING (mymin)
        etime = STRCOMPRESS (ssmyhr + ':' + smymin, /remove_all)
    
    ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;    PRINT, 'Image presently displayed is: ', filename

    year = mk4h1.Year
    syear = STRCOMPRESS (STRING (year), /remove_all)
    iyr = FIX (year)

    doy = mk4h1.Doy
    sdoy = STRCOMPRESS (STRING (doy), /remove_all)
    leap = LONG ((iyr MOD 4 EQ 0) AND (iyr MOD 100 NE 0) OR (iyr MOD 400 EQ 0))

    yrDdoy = syear + 'd' + sdoy
    yrDdoy = STRCOMPRESS (yrDdoy, /remove_all)

;    PRINT, 'syear: ', syear
;    PRINT, 'sdoy:  ', sdoy

    ;--- Determine Month & Day-Of-Month from Day-Of-Year.

    d = doy
    m = 0

    WHILE (d GT 0) DO BEGIN ;{
        m = m + 1
        d = d - daytab (m, leap)
    ENDWHILE ;}

    resid = d + daytab (m, leap)
    m = m - 1
    IF (m LT 0) THEN m = 0

    month  = STRLOWCASE (montab (m))		; 3-character month name.
    imonth = imontb (m)				; 2-character month number + "_"

    IF (resid LT 10) THEN $
         date = ' ' + STRCOMPRESS (STRING (resid), /remove_all) + ' '	$
		   + montab (m) + ' ' + syear $
    ELSE date =       STRCOMPRESS (STRING (resid), /remove_all) + ' '	$
		   + montab (m) + ' ' + syear

    IF      ( doy LT 10  ) THEN adoy  = 'DOY   ' + sdoy $
    ELSE IF ( doy LT 100 ) THEN adoy  = 'DOY  '  + sdoy $
    ELSE                        adoy  = 'DOY '   + sdoy

    ;--- Create output file name.

    qualname = '/jabba/ftp/mk4/daily_images/' + syear + '/'	$
	     + imonth + month + '/' + yrDdoy + '.mk4.gif'
    qualname = STRCOMPRESS (qualname, /remove_all)

    ;--- Test option.

    IF (dotest) THEN BEGIN ;{
	Oqualname = qualname
	qualname = '/riker/d/acos/mk4/qualtest/' + yrDdoy + '.mk4.gif'
    ENDIF ;}

    PRINT, 'qualname: ', qualname

    tmp = BYTSCL (rpB, MIN = mn, MAX = mx, TOP = 254)
    tmp (dpx, dpy) = 255

    ERASE, 0

    ;--- Annotate image.

    TV, tmp
    XYOUTS,   2, 496, 'Coronameter', size = 1.3, /dev
    XYOUTS,   2, 476, 'MLSO-Mk4', size = 1.3, /dev
    XYOUTS, 412, 496, date, size = 1.1, /dev
    XYOUTS, 445, 476, adoy,  size = 1.1, /dev
    XYOUTS, 438, 456, '~' + avtime + ' UT',   size = 1.1, /dev
    XYOUTS,   2,   5, 'North: straight up', size = 1.1, /dev

    qualcom = STRCOMPRESS ( STRING (numimg) + ' images' ) 
    IF (numimg LT 10) THEN qualcom = ' ' + qualcom

    XYOUTS, 435, 45, qualcom, size = 1.0, /dev
    XYOUTS, 403, 25, btime + '-' + etime + ' UT', size = 1.0, /dev

    sclcomm = 'Scaling: ' + strtrim (STRING (mn), 2) + '-'		$
	                  + strtrim (STRING (mx), 2)

    XYOUTS, 375,  5, sclcomm, size = 1.0, /dev

    zzz = TVRD ()				; Read annotated image.
    WRITE_GIF, qualname, zzz, red, green, blue 	; Write GIF file.

    ;--- Delete temporary files.

    cmnd = '/bin/rm mk4.avg.times'
    SPAWN, cmnd

    cmnd = '/bin/rm bzbzbz'
    SPAWN, cmnd

;    cmnd = '/bin/rm ' + filename			; Delete input file.
;    SPAWN, cmnd

    IF (seegif) THEN BEGIN ;{
	DEVICE, /CLOSE
	SET_PLOT, 'x'

	WSET, 0
	TV, zzz

	READ_GIF, Oqualname, gif
	WSET, 2
	TV, gif
	PRINT, 'Click mouse in window 0 when ready to continue.'

	WSET, 0
	CURSOR, x, y, /NORMAL
    ENDIF ;}

    
CLOSE, 13

END
