;+
; NAME
;	mk4dirlist
;
; PURPOSE
;	Reads a list of Mk4 FITS images.
;	Outputs two lists: forwards scans & backward scans.
;
; SYNTAX
;	mk4dirlist, listfile
;
; INPUT
;	listfile	contains a list of *.rpB.avg.slit images
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   4 May 2000
;-

PRO mk4dirlist, listfile

daytab = INTARR (13, 2)
daytab = [[0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ],  $
          [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]]

montab = STRARR (12)
montab = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', $
	  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
imontb = STRARR (12)
imontb = ['01_', '02_', '03_', '04_', '05_', '06_', $
	  '07_', '08_', '09_', '10_', '11_', '12_']

filename = ''
xdimprev = 0
ydimprev = 0
imgcnt = 0
                  
;--- Load color table

CLOSE, 10
red = BYTARR (256)  &  green = BYTARR (256)  &  blue = BYTARR (256)

OPENR, 10, '/home/cordyn/color_idl/qual.rgb' 
   READU, 10, red  &  READU, 10, green  &  READU, 10, blue
CLOSE, 10

TVLCT, red, green, blue 
                  
CLOSE, 10 & OPENR, 10, listfile
CLOSE, 11 & OPENW, 11, 'fwd.ls'
CLOSE, 12 & OPENW, 12, 'rev.ls'
                  
;--- Image loop:

WHILE ( NOT EOF (10) )  DO BEGIN ;{

    READF, 10, filename
    img = readfits  (filename, hdu)
    ext = headfits (filename, exten=1)

    xdim = fxpar (hdu, 'NAXIS1')
    ydim = fxpar (hdu, 'NAXIS2')
    print, 'xdim/ydim: ', xdim, ydim
;    IF (xdim NE xdimprev OR ydim NE ydimprev) THEN	$
    IF (Imgcnt EQ 1) THEN				$
	window, xsize=xdim, ysize=ydim

    dispmin = fxpar (hdu, 'DISPMIN')
    dispmax = fxpar (hdu, 'DISPMAX')
    tv, bytscl (img, min=dispmin, max=dispmax, top=249)
    
    bardir = fxpar (ext, 'BAR_DIR')
    IF (bardir EQ -1) THEN	$
	printf, 11, filename	$
    ELSE			$
	printf, 12, filename
    
    imgcnt = imgcnt + 1
    xdimprev = xdim
    ydimprev = ydim
ENDWHILE ;}
                  
CLOSE, 10
CLOSE, 11
CLOSE, 12

END
