;+
; MLSO Mark-4 binary header structure.
;
; HISTORY
;	Andrew L. Stanger   HAO/NCAR   3 September 1999
;	3 May 2000: Update binary header contents.
;-

mk4h1 =			$
    {mk4head1,		$
    Year:	0L,	$
    Month:	0L,	$
    Day:	0L,	$
    Hour:	0L,	$
    Minute:	0L,	$
    CentiSec:  	0L,	$
    Juld:	0L,	$ ; Julian day - 2415021
    Doy:	0L,	$
    Time:	0L,	$ ; Time in Hz
    Xdim:	0L,	$
    Ydim:	0L,	$
    Duration:	0L,	$ ; Duration of entire scan, in Hz.
    Fill1:	lonarr (4),	$
    Pangle:	0.0,	$ ; P angle in degrees
    Bangle:	0.0,	$ ; B angle in degrees
    Langle:	0.0,	$ ; L angle in degrees
    RA:		0.0,	$ ; Solar Right Ascension
    Dec:	0.0,	$ ; Solar Declination
    HA:		0.0,	$ ; Hour angle in degrees
    Elevation:	0.0,	$ ; Solar elevation in degrees
    Azimuth:	0.0,	$ ; Solar azimuth in degrees
    Amass:	0.0,	$ ; Airmass
    Radius:	0.0,	$ ; Solar radius in arc-seconds.
    SkyTx:	0.0,	$ ; Max. reasonable sky transmission from "sk" file.
    InstrPhs:	0.0,	$ ; Instrument coordinate frame phase offset. (degrees)

    spare1:		0.0,	$
    data_scale:		0.0,	$
    data_offset:	0.0,	$
    pB_ReferenceScale:	0.0,	$
    orig_hairspot:	fltarr (2),	$
    hairspot:		fltarr (2),	$

    Xcen:	0.0,	$ ; Center  of the image if in rect. coordinates.
    Ycen:	0.0,	$ ; Center  of the image if in rect. coordinates.
    pixscl:	0.0,	$ ; Scaling of the image if in rect. coordinates.
    r0:		0.0,	$ ; In present image.
    dr:		0.0,	$ ; In present image.
    rmax:	0.0,	$ ; In present image.
    theta_min:	0.0,	$ ; In present image.
    theta_max:	0.0,	$ ; In present image.
    raw_theta_min:	0.0,	$ ; In original raw data image. Uncorrected for
    raw_theta_max:	0.0,	$ ;             direction and Pangle and phase.
    backlash:	0.0,	$ ; Backlash correction applied, in degrees.
    Phase   :	0.0,	$ ; Phase offset, in degrees.

    ObsType:	0L,	$ ; IS_cal0, IS_cal15, IS_dark, IS_coronal
    DataType:	0L,	$ ; IS_i , IS_q , IS_u
    ProcSteps:	0L,	$ ; Process steps, IS_raw through IS_cal_avg_rect_slit
    Direction:	0L,	$ ; "direction" of original raw data. -1==rev, +1 fwd.
    Nray:	0L,	$ ; Number of barrel angle positions (rays) in raw data.
    Pixels:	0L,	$ ; Number of pixels in original raw data.
    Fill2:	0L,	$
    cYear:	0L,	$ ; Date calibrated.
    cMonth:	0L,	$
    cDay:	0L,	$
    cHour:	0L,	$
    cMinute:	0L,	$
    cCentiSec: 	0L,	$
    cJuld:	0L,	$ ; Julian day - 2415021
    cDoy:	0L,	$
    cTime:	0L	$ ; Time in Hz
    }

mk4h2 =			$
    {mk4head2,	$
    tscpfile:   bytarr (64),    $ ; filename of tscp file.
    reffile:    bytarr (64),    $ ; filename of reference file.
    driftfile:  bytarr (64),    $ ; filename of drift file.
    AvgTimes:   bytarr (160),   $ ; If avg img, these are the img times
				  ;   used to make the avg.
    space:      bytarr (416)    $ ; Fill header to 1024 bytes (416 = 1024 - 608)
    }


;--- Constants:

    naz      = 1080		; # azimuthal positions (# radial scans).
    ovnaz    = 1.0 / naz	;
    az2deg   = 360.0 / naz	; angular separation (degrees).
    AngSep   = 360.0D / naz	; angular separation (degrees).
    ovAngSep = naz / 360.0D	; # radial scans per degree.
