;+
; NAME	mk4hairlo.pro
;
; PURPOSE	Read Mk4 .pB images & plot hairline positions.
;
; SYNTAX	mk4hairlo, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   21 April 2000
;-
@mk4h.pro	; Define Mk4 binary header structure.

pro mk4hairlo, pb_list

CLOSE, 1
pb_name = ''
plot_name = ''

month_name = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',		$
	      'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
time = fltarr (120)
ohlo = fltarr (120)
ohhi = fltarr (120)
ahlo = fltarr (120)
ahhi = fltarr (120)
i = 0
;mk4h = fltarr (256)

;--- File list loop.

OPENR, 1, pb_list
OPENW, 3, 'mk4hairlo.txt'

WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, pb_name				; Get file name.
   print, 'pb_name: ', pb_name

;   fits_img = readfits (fits_name, hdu)
   OPENU, 2, pb_name
;   READU, 2, mk4h
   READU, 2, mk4h1
   CLOSE, 2

   ;--- Extract information from mk4 binary header.

;   orghair0 = mk4h [32]
;   orghair1 = mk4h [33]
;   hair0    = mk4h [34]
;   hair1    = mk4h [35]

   year   = mk4h1.Year
   month  = mk4h1.Month
   day    = mk4h1.Day
   hour   = mk4h1.Hour
   minute = mk4h1.Minute
   second = mk4h1.CentiSec / 10
   date   = string (year) + '-' + month_name [month-1] + '-' + string (day)
   print, 'date: ', date
   hours  = float (hour) + minute / 60.0 + second / 3600.0

   orghair0 = mk4h1.orig_hairspot [0]
   orghair1 = mk4h1.orig_hairspot [1]
   hair0 = mk4h1.hairspot [0]
   hair1 = mk4h1.hairspot [1]

   ;--- Extract information from FITS header.

;   dateobs  = fxpar (hdu, 'DATE-OBS')
;   timeobs  = fxpar (hdu, 'TIME-OBS')
;   orghair0 = fxpar (hdu, 'ORGHAIR0')
;   orghair1 = fxpar (hdu, 'ORGHAIR1')
;   hair0    = fxpar (hdu, 'HAIR0')
;   hair1    = fxpar (hdu, 'HAIR1')

   print, 'i, orghair0, orghair1: ', i, orghair0, orghair1
   printf, 3, 'i, orghair0, orghair1: ', i, orghair0, orghair1

   ohlo [i] = orghair0
   ohhi [i] = orghair1
   ahlo [i] = hair0
   ahhi [i] = hair1
   
   time [i] = hours
   i = i + 1

END	;}

CLOSE, 1
CLOSE, 3


orglo = fltarr (i)
orghi = fltarr (i)
timi  = fltarr (i)

orglo [0:i-1] = ohlo [0:i-1]
orghi [0:i-1] = ohhi [0:i-1]
timi  [0:i-1] = time [0:i-1]
titlelo = pb_name + ' orig. hairline [low]'
titlehi = pb_name + ' orig.hairline [high]'

set_plot, 'PS'
;device, filename='mk4hairlo.ps'
DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'mk4hairlo.ps', $
	XOFFSET = 1.27, XSIZE = 17.50, $        ; 1" left margin, 7.0" W
	YOFFSET = 12.70, YSIZE = 12.70          ; 5" bottom margin, 5" H
;!P.MULTI = [0, 0, 0, 0, 0]

plot, timi, orglo,			$
       yrange=[ 95, 110],		$
      title=titlelo, xtitle='index', ytitle='pixel'

;plot, timi, orghi,			$
;      yrange=[270, 285],		$
;      title=titlehi, xtitle='index', ytitle='pixel'
;plot, time, ahlo
;plot, time, ahhi

device, /close
set_plot,'x'

END
