;+
; mk4height.pro
;
; Geometric scale determination for Mark-4.
;
; HISTORY:
;	checkheights4.pro
;	David Elmore   HAO/NCAR   September 1999
;-

PRO mk4height, filename

nslots = 3

;heights = [1.24, 1.804, 2.287]   ?
;heights = [1.224, 1.804, 2.287, 2.818]
;heights = [1.224, 1.804, 2.287]

IF (nslots EQ 3) THEN BEGIN			$
    heights = [1.224, 1.804, 2.287] 
END						$
ELSE IF (nslots EQ 4) THEN BEGIN		$
    heights = [1.224, 1.804, 2.287, 2.818]
END						$
ELSE BEGIN					$
   PRINT, 'nslots must be 3 or 4.'
   RETURN
END

minpix = [40, 120, 220, 300]
maxpix = [120, 200, 300, 380]
x = FLTARR (2)
y = FLTARR (2)
numaz = 985
numaz = 984
img = INTARR (384, numaz)
slot = FLTARR (4, numaz)
col = FINDGEN (81)

; **** Uncomment out the next four lines to do it the old way - Tony
;filename = '/slug/d/cordyn/mk3/94d215/DATA/99d036/mk4/99d036.19_15.ch0.i'
;OPENR, 1, filename
;READU, 1, img
;CLOSE, 1

;
;===============================================================================
; Stuff added by Tony to correctly read in a mk4 image.  I was getting
;	wraparound the other way.  I think the size of the header changed
;	since this was last done.
;

CLOSE, 1
head   = 0 & head   = BYTARR (1024)
headib = 0 & headib = LONARR (16)  
headf  = 0 & headf  = FLTARR (24)
headie = 0 & headie = LONARR (16)  
scl = 2
scl = 1
pixx = 384  & pixy = 1200
pixx = 1100 & pixy = 1100
xsz = (pixx / scl)
ysz = (pixy / scl)
xcen = pixx / 2  & ycen = pixy / 2

;filename = '/puka/d/cordyn/Data/mk4/MASK/99d239.18_43.mk4.i'
;filename = '/thisbe/e/elmore/MkIV/Height/99d274.22_44.mk4.i'

top  = LONG (0)
fpos = 0
fpos = LONARR (5000)
ftmp = LONG (0)
ii = 0

CLOSE, 1
OPENR, 1, filename
POINT_LUN, -1, top
READU, 1, headib
READU, 1, headf
READU, 1, headie

POINT_LUN, 1, top
READU, 1, head

pixx = headib (9) & pixy = headib (10)
PRINT, 'xdim:', pixx, '  ydim:', pixy
img = 0
img  = INTARR (pixx, pixy) 

READU, 1, img
CLOSE, 1

img = 1.0 * img
a = WHERE ( img LT 0 )
IF ( a (0) > 0 ) THEN img (a) = img (a) + 65536.0

;
; End of stuff added by Tony
;===============================================================================
;

;minx = 32
minx = 40
!y.style = 1
!x.style = 1
!x.range = [minx, 383]

doplot = 1
IF ( doplot EQ 1 ) THEN BEGIN
	SET_PLOT, 'ps'
        DEVICE, /color, /portrait,  $
	       ; /inches, xsize = 8.0, ysize = 5.0, $
               ; xoffset = 0.5, yoffset = 1.5, bits_per_pixel = 8, $
                filename = 'mk4height.' + filename + '.ps'
ENDIF ELSE BEGIN
	WINDOW, ysize = 384, xsize = numaz
	LOADCT, 0
	TV, BYTSCL (transpose (img), -50, 20000)
	; STOP
ENDELSE

!y.range = 0
!p.title = filename + '   Intensity vs Radius'
!x.title = 'Pixel Height'
!y.title = 'Intensity'
PLOT, img (*, numaz / 2)

;STOP

; 99d239:
   r2min =  2.7
   r2max = 10.0
   ngterms = 6

; 99d274:
   r2min =  2.0
   r2max = 35.0
   ngterms = 4

FOR ii = 0, numaz - 1 DO BEGIN
    FOR jj = 0, nslots - 1 DO BEGIN
        fit = GAUSSFIT (col, img (minpix (jj) : maxpix (jj), ii),	$
			r, nterms = ngterms)
        IF ( (r (2) GT r2min) and (r (2) LT r2max) ) THEN BEGIN
            slot (jj, ii) = r (1) + minpix (jj)
        ENDIF ELSE BEGIN
            str = STRING (' bad r (2) = ', r(2))
            PRINT, ii, str
            IF ( ii EQ 0 ) THEN BEGIN
                slot (jj, ii) = (minpix (jj) + maxpix (jj) / 2.0)
            ENDIF ELSE BEGIN
                slot (jj, ii) = slot (jj, ii-1)
            ENDELSE
        ENDELSE
    ENDFOR

;    PRINT, ii
ENDFOR

meanpix = FLTARR (4)

!p.title = filename + '  ' + 'r[2] min/max: '				$
		    + string (r2min, FORMAT='(F4.1)') + ' ' $
		    + string (r2max, FORMAT='(F4.1)') + '  nterms: '	$
		    + string (ngterms, FORMAT='(I2)')
!y.title = 'Pixel Number'
!x.title = 'Scan Azimuth'
!x.range = [0, numaz]

FOR jj = 0, nslots - 1 DO BEGIN
	!y.range = [slot (jj, numaz / 2) - 5, slot (jj, numaz / 2) + 5]
	!y.title = 'Slot # ' + STRTRIM (STRING (jj + 1, FORMAT='(I)'), 2)   $
		 + '   Pixel Number'
	PLOT, slot (jj, *), psym = 1
	m = moment (slot (jj, *))
	meanpix (jj) = m (0)
	str = STRING (m (0), '+/-', sqrt (m (1)))
	XYOUTS,  numaz / 4, slot (jj, numaz / 2) - 2, str
	str = STRING (heights (jj), ' Rsun')
	XYOUTS,  numaz / 4, slot (jj, numaz / 2) - 3, str
	; STOP
ENDFOR

IF ( meanpix (0) LT 60 ) THEN heights (0) = 1.185

slopes  = FLTARR (numaz)
offs    = FLTARR (numaz)
curves  = FLTARR (numaz)
limbpix = FLTARR (numaz)
toppix  = limbpix

FOR ii = 0, numaz - 1 DO BEGIN
	IF (nslots EQ 3) THEN				$
	    res = POLY_FIT (slot (0:2, ii), heights, 1)	$
	ELSE IF (nslots EQ 4) THEN			$
	    res = POLY_FIT (slot (0:3, ii), heights, 1)
	slopes (ii) = res (1)
	offs (ii) = res (0)
ENDFOR

limbpix =  minx * slopes + offs
toppix = 383.0 * slopes + offs

!y.range = [0.98 * limbpix (numaz / 2), 1.02 * limbpix (numaz / 2)]
!y.title = 'R0 Lowest Corona'
PLOT, limbpix, psym = 2

; STOP

!y.range = [0.98 * toppix (numaz / 2 ), 1.02 * toppix (numaz / 2)]
!y.title = 'R0 Highest Corona'
PLOT, toppix, psym = 1

; STOP

m = moment (offs)
off = m (0)

m = moment (slopes)
slope = m (0)

;m = moment (curves)
;curvature = m (0)

pixels = FLTARR (384)
numbers = FINDGEN (384)
pixels = off + numbers * slope

!x.style = 1
!x.range = [minx, 383]
!y.range = [1, 3]
!x.title = 'Pixel Number'
PLOT, pixels
str = STRING ('height  = ', off, ' + pixel *', slope)
PRINT, str
XYOUTS, 44, 2.75, str

;FOR ii = 0, 3 DO BEGIN
FOR ii = 0, nslots - 1 DO BEGIN
	x (0) = meanpix (ii)
	x (1) = x (0)
	y (0) = heights (ii) - 0.1
	y (1) = heights (ii) + 0.1
	PLOTS, x, y

	x (0) = meanpix (ii) - 5
	x (1) = meanpix (ii) + 5
	y (0) = heights (ii)
	y (1) = y (0)
	PLOTS, x, y
ENDFOR

; STOP

IF ( doplot EQ 1 ) THEN BEGIN
	device, /CLOSE
ENDIF

PRINT, 'Done'

RETURN
END

