;+
; NAME	rdmk3map.pro
;
; SYNTAX  
;	rdmk3map, 'mk3.e1943.r125.df'
;	rdmk3map, 'mk3.w1943.r125.df', map=mk3map, limb=mk3limb, radius=mk3rad
;
; VARIABLES RETURNED:
;	map: floating point array (360,181) containing synoptic map intensity 
;	limb: string indicating the limb ('e' or 'w')
;	radius: floating point value representing the radius in Rsun units.
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   14 Nov 2002
;-

PRO rdmk3map, mk3mapfile, map=map, limb=limb, radius=radius

limb = ''
map = FLTARR (360, 181)

;--- Read Mk3 synoptic map file.

CLOSE, 1
OPENR, 1, mk3mapfile
READU, 1, map
CLOSE, 1

;--- Get limb designation from file name.

limb    = STRMID (mk3mapfile, 4, 1)

;--- Get radius from file name.

sradius = STRMID (mk3mapfile, 11, 3)

radius = FIX (sradius)
radius = radius / 100.0

;--- Display image.

WINDOW, xs=360, ys=181
TVSCL, map
TV, bytscl (map, min=0, max=9e-7)

;--- Print out information.

PRINT, 'file: ', mk3mapfile
PRINT, 'limb: ', limb
PRINT, 'radius: ', radius

fmin = MIN (map, max=fmax)
PRINT, 'fmin/fmax: ', fmin, fmax

END
