;+
; NAME	mk3_tscan.pro
;
; PURPOSE	Read Mk3 FITS images & plot theta scan.
;
; SYNTAX	mk3_tscan, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    28 June 2001
;-

pro mk3_tscan0, fits_list, radius

CLOSE, 1
fits_name = ''
plotname = ''
set_plot, 'PS'

posang = findgen (720) / 2.0		; Position Angle array.

;--- File list loop.

OPENR, 1, fits_list
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 1, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')
   plotname = STRMID (fits_name, 0, ftspos)
   plotname = plotname + '.ps'
   print, plotname
   device, filename = plotname

   fits_img = readfits (fits_name, hdu)
;   ext = readfits (fits_name, hduext, exten_no=1)

   ;--- Extract information from header.

   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')
   crpix1   = fxpar (hdu, 'CRPIX1')
   cdelt1   = fxpar (hdu, 'CDELT1')
   r0       = - (crpix1 * cdelt1)
   dr       = cdelt1

   print, 'xdim: ', xdim, '   ydim: ', ydim
   print, 'r0: ', r0, '   dr: ', dr

;   r0       = fxpar (hduext, 'R0')
;   dr       = fxpar (hduext, 'DR')
   ir = (radius - r0) / dr			; Radial index.
   print, 'radius: ', radius, '   ir: ', ir
   ir = fix (ir + 0.5)

   sradius = STRING (radius, FORMAT='(F4.2)')
   ititle = fits_name + ' Theta Scan @' + sradius + ' Rsun'
   xlab = 'Position Angle [deg]'
   ylab = 'Intensity'

   plot, posang, fits_img [ir, *],	$
	 title=ititle,		$
	 xtitle = xlab,		$
	 ytitle = ylab,		$
	 xrange = [0.0, 360.0]
   
   device, /close
END	;}

CLOSE, 1

set_plot,'x'

END
