;+
; NAME		mk3fts.pro
; 
; PURPOSE	Convert binary Mk3 files to FITS
;		when no binary tailer is present.
;
; SYNTAX	mk3fts, 'mk3_list'
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   17 May 2002
;-

PRO mk3fts, mk3_file

daytab = INTARR (13, 2)
daytab = [[0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ],  $
	  [0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]]

smonth  = ''
sday    = ''
syear   = ''
shour   = ''
sminute = ''

;mk3_file = ''
img = INTARR (512, 512)

;CLOSE, 11
;OPENR, 11, mk3_list
;READF, 11, mk3_file

syear   = STRMID (mk3_file, 0, 2)
sdoy    = STRMID (mk3_file, 3, 3)
shour   = STRMID (mk3_file, 7, 2)
sminute = STRMID (mk3_file, 10, 2)

year   = FIX (syear)
doy    = FIX (sdoy)
minute = FIX (sminute)
IF (year LT 1900) THEN		$
BEGIN
    year = year + 1900
    syear = '19' + syear
END

;--- Determine Month & Day-Of-Month from Day-Of-Year.

leap = LONG ((year MOD 4 EQ 0) AND (year MOD 100 NE 0) OR (year MOD 400 EQ 0))
d = doy
month = 0
	
WHILE (d GT 0) DO BEGIN ;{
month = month + 1
d = d - daytab (month, leap)
ENDWHILE ;}
				
day = d + daytab (month, leap)

print, 'doy: ', doy, '  month: ', month, '  day: ', day

smonth = STRTRIM (STRING (month), 2) ;
sday   = STRTRIM (STRING (day)  , 2) ;

IF (month LT 10) THEN smonth = '0' + smonth
IF (day   LT 10) THEN sday   = '0' + sday

OPENR, 12, mk3_file
READU, 12, img
CLOSE, 12

dmin = MIN (img, max=dmax)
fmin = FLOAT (dmin)
fmax = FLOAT (dmax)

fxhmake,  hdu, img
fxaddpar, hdu, 'ORIGIN',   'MLSO'
fxaddpar, hdu, 'TELESCOP', 'MK3'
fxaddpar, hdu, 'INSTRUME', 'RETICON'
fxaddpar, hdu, 'OBJECT',   'SUN'
fxaddpar, hdu, 'TYPE-OBS', 'Q'
fxaddpar, hdu, 'BUNIT',    'Q RAW'
fxaddpar, hdu, 'COORDNAM', 'HELIO-CENTRIC'
fxaddpar, hdu, 'CTYPE1',   'X',        'AXIS 1 TYPE: X'
fxaddpar, hdu, 'CRPIX1',   255.5,      'X SUN CENTER [PIXEL]'
fxaddpar, hdu, 'CRVAL1',   0.0,        'X SUN CENTER [Rsun]'
fxaddpar, hdu, 'CDELT1',   9.4664E-03, 'X COORD INCREMENT [Rsun/PIXEL]'
fxaddpar, hdu, 'CROTA1',   0.0,        'X ROTATION [DEGREES] FROM REFERENCE'
fxaddpar, hdu, 'CTYPE2',   'Y',        'AXIS 2 TYPE: Y'
fxaddpar, hdu, 'CRPIX2',   255.5,      'Y SUN CENTER [PIXEL]'
fxaddpar, hdu, 'CRVAL2',   0.0,        'Y SUN CENTER [Rsun]'
fxaddpar, hdu, 'CDELT2',   9.4664E-03, 'Y COORD INCREMENT [Rsun/PIXEL]'
fxaddpar, hdu, 'CROTA2',   0.0,        'Y ROTATION [DEGREES] FROM REFERENCE'
fxaddpar, hdu, 'CRRADIUS', 105.64,     'PIXELS PER SOLAR RADIUS'
fxaddpar, hdu, 'RSUN',     970.95,     'SOLAR RADIUS [ARCSEC]'
fxaddpar, hdu, 'SOLAR_P0', -18.08,     'SOLAR P-ANGLE [DEGREES]'
fxaddpar, hdu, 'SOLAR_B0', -6.96,      'SOLAR P-ANGLE [DEGREES]'
fxaddpar, hdu, 'BOPAL',    1.48E-5,    'OPAL BRIGHTNESS [BSUN] MEASURED'
fxaddpar, hdu, 'BOPALC',   1.011E-5,   'OPAL BRIGHTNESS [BSUN] IN CALIBRATION'
fxaddpar, hdu, 'TILTPOL',  1.641E-2,   'TILT POLARIZATION [FRACTION] MEASURED'
fxaddpar, hdu, 'TILTPOLC', 1.641E-2,   'TILT POLARIZATION [FRACTION] IN CAL'
fxaddpar, hdu, 'BSCALE',   1.5569E-10, 'PHYSICAL = DATA * BSCALE + BZERO'
fxaddpar, hdu, 'BZERO',    0.0
fxaddpar, hdu, 'DIRAXIS1', 'RIGHT',    'DISPLAY DIRECTION FOR AXIS 1 [X]'
fxaddpar, hdu, 'DIRAXIS2', 'UP',       'DISPLAY DIRECTION FOR AXIS 2 [Y]'
fxaddpar, hdu, 'VERSION',  'Mk3 2.01', 'MK3 version number'

fxaddpar, hdu, 'DATE-OBS', syear + '-' + smonth + '-' + sday
fxaddpar, hdu, 'TIME-OBS', shour + ':' + sminute + ':00'
fxaddpar, hdu, 'DATAMIN',  fmin,       'DATA MINIMUM'
fxaddpar, hdu, 'DATAMAX',  fmax,       'DATA MAXIMUM'
fxaddpar, hdu, 'DISPMIN',   0.0,       'DISPLAY MINIMUM'
fxaddpar, hdu, 'DISPMAX', 800.0,       'DISPLAY MAXIMUM'

ofile = mk3_file + '.fts'
print, 'ofile: ', ofile

writefits, ofile, img, hdu

;CLOSE, 11

END

