;+
; NAME	mk3synlong.pro
;
; PURPOSE	Longitude scan of Mk3 synoptic map.
;
; SYNTAX	mk3synlong, mk3_synoptic_file latitude
;
;		mk3_synoptic_file	binary file 360x181 shorts, e or w
;		latitude
;
;		mk3synlong, '1943e1.25', 0
;		
; AUTHOR	Andrew L. Stanger   HAO/NCAR    4 Nov 2002
;-

PRO mk3synlong, mapfile, lat

map = INTARR (360, 181)

CLOSE, 1
OPENR, 1, mapfile
READU, 1, map
CLOSE, 1

SET_PLOT, 'Z'
LOADCT, 0				; Load greyscale color map.
fcol =   0
bcol = 255
!P.BACKGROUND = bcol

map = map * 1.0e-10			; Convert to pB Bsun units.
xlab   = 'Longitude'
ylab   = 'pB [Bsun]'

ilat = FIX (lat) + 90
slat = STRING (FIX (lat), FORMAT='(i3)')
IF (lat LT 0) THEN hemi = 's' ELSE hemi = 'n'
IF (lat EQ 0) THEN hemi = ''
ptitle = 'Mk3 C.R.' + mapfile + ' Latitude ' + slat
ymin = -1.0e-8
ymax = MAX (map [*, ilat])

PLOT, map [*, ilat],		$
      title  = ptitle,		$
      xtitle = xlab,		$
      ytitle = ylab,		$
      yrange = [ymin, ymax],	$
      xmargin = [14,3],		$
      ymargin = [3, 2],		$
      color  = fcol

slat = STRTRIM (STRING (ABS (FIX (lat)), FORMAT='(I)'), 2)
gifname = mapfile + '.lat' + slat + hemi + '.gif'
plotimg = TVRD ()
WRITE_GIF, gifname, plotimg
SET_PLOT, 'X'

END
