;+
; NAME		avgpb.pro
;
; PURPOSE	Routine to average a list of images
;
; AUTHOR	David Elmore   13 Nov 2001
;
; HISTORY	Andrew L. Stanger
;		 3 Jan 2002: Cosmetic changes only.
;-

;--- Set up display values.

!y.title    = 'Daily Average pB'
!x.title    = 'Height (R-Sun)'
!p.title    = 'Average pB vs. Height'
!p.font     = 1
!p.thick    = 1.6
!p.ticklen  = 1
!p.charsize = 1.6
!x.style    = 1
!y.style    = 1
!x.ticks    = 10
!x.range    = [1.10,   2.8]
!y.range    = [1.e-10, 1.e-6]

is_cal_avg  = 18
azplace     = 220

;--- Read settings file.

olddalsa     = 'n'
dostreach    = 'n'
correctfile0 = 'fred'
correctfile1 = 'fred'

CLOSE, 15 & OPENR, 15, 'settings'
READF, 15, olddalsa
olddalsa = STRTRIM (olddalsa)
READF, 15, kappa
READF, 15, phaseoffset
READF, 15, correctfile0
READF, 15, correctfile1
READF, 15, dostreach
dostreach = STRTRIM (dostreach)
READF, 15, linelow
READF, 15, linehigh
READF, 15, pixoff
READF, 15, pixscale
READF, 15, minrad
READF, 15, maxrad
READF, 15, iscale
READF, 15, ioffset
READF, 15, qscale
READF, 15, qoffset
READF, 15, uscale
READF, 15, uoffset
READF, 15, lowbias
READF, 15, highbias


;--- List of files to average is in avglist.

listname = 'avglist'
filename = 'fred'


;--- Read a header to set some variables.

headib  = 0 & headib  = LONARR (16)  
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)  
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

CLOSE, 2
OPENR, 2, 'avglist' 
READF, 2, filename
CLOSE, 2

CLOSE, 1
OPENR, 1, filename
    READU, 1, headib
    READU, 1, headf
    READU, 1, headie
    READU, 1, headpad
CLOSE, 1

PRINT, headib
PRINT, headf
PRINT, headie

numaz    = headib  [9]				; Xdim
numpix   = headib [10]				; Ydim
pixoff   = headf  [23]				; r0
pixscale = headf  [24]				; dr
PRINT, numaz, numpix, pixscale, pixoff


;--- Heights from header of first file.

height = FINDGEN (numpix) * pixscale + pixoff

inimg  = INTARR (NUMAZ, NUMPIX)
avgimg = FLTARR (NUMAZ, NUMPIX)

CLOSE, 2
OPENR, 2, listname
READF, 2, filename
numimg = 0

WHILE ( NOT EOF (2) ) DO BEGIN
    numimg = numimg + 1
    READF, 2, filename
ENDWHILE

rays = FLTARR (numimg, NUMPIX)

CLOSE, 2
OPENR, 2, listname
READF, 2, filename

WINDOW, 0, xs=numaz, ys=numpix
WINDOW, 1, xs=numaz, ys=numpix
imgnum = 0

WHILE ( NOT EOF (2) ) DO BEGIN
    PRINT, filename

    CLOSE, 1
    OPENR, 1, filename 
    READU, 1, headib 
    READU, 1, headf 
    READU, 1, headie 
    READU, 1, headpad
    READU, 1, inimg

    scale  = headf [13]				; data_scale
    offset = headf [14]				; data_offset
    img = (FLOAT (inimg) - offset) / scale 
    avgimg = avgimg + img
    rays [imgnum, *] = img [azplace, *] 
    imgnum = imgnum + 1
    WSET, 0
    TV, BYTSCL (ALOG10 (img + 2.e-9), -9, -5)
    WSET, 1
    TV, BYTSCL (ALOG10 ((avgimg / imgnum) + 2.e-9), -9, -5)
    READF, 2, filename
ENDWHILE

avgimg = avgimg / numimg 
TV, BYTSCL (ALOG10 (avgimg + 2.e-9), -9, -5)

CLOSE, 1
OPENW, 1, STRMID (filename, 0, 6) + '_avg.CalpB'
headie [2] = is_cal_avg					; ProcSteps

WRITEU, 1, headib
WRITEU, 1, headf
WRITEU, 1, headie
WRITEU, 1, headpad
WRITEU, 1, FIX (avgimg * scale + offset)
CLOSE,  1


WINDOW, 2, xs=512, ys=512

;   Joan, this is all you really need
;	feed these lines of code with the average
;	image of the day.
; What is mean and std dev for each height?
;   an indicator of coronal structure

avgheight      = FLTARR (numaz)
avgheightplus  = FLTARR (numaz)
avgheightminus = FLTARR (numaz)
avgheightmax   = FLTARR (numaz)
avgheightmin   = FLTARR (numaz)

FOR ii = 0, numpix-1 DO BEGIN
    res = MOMENT (avgimg [*, ii], sdev=derf)
    avgheight [ii]      = res [0]
    avgheightplus [ii]  = res [0] + derf
    avgheightminus [ii] = res [0] - derf
    avgheightmax [ii]   = max (avgimg [*, ii])
    avgheightmin [ii]   = min (avgimg [*, ii])
ENDFOR

CLOSE, 1
OPENW, 1, 'corona_model'
WRITEU, 1, avgheight
CLOSE, 1


;--- Create azimuthally binned data.
;    Again, this will work on average image of the day.

binsize = 20
binimg = FLTARR (numaz / binsize, numpix)

FOR jj = 0, numaz-1 DO BEGIN
	kk = jj / binsize
	binimg [kk, *] = binimg [kk, *] + img [jj, *]
ENDFOR

FOR kk = 0, numaz/binsize - 1 DO binimg [kk, *] = binimg [kk, *] / binsize
	
PRINT, "Set plot device"
STOP

plot_oo, height, avgheight
OPLOT, height, avgheightplus, psym=3
OPLOT, height, avgheightminus, psym=3

;OPLOT, height, avgheightmax, psym=3
;OPLOT, height, avgheightmin, psym=3

STOP

;--- What is the stdev for each height over the day?
;
; This requires all the images going into the average
; image of the day to get RMS at each height
; over the day.

daystdev  = FLTARR (NUMPIX)
daymean   = FLTARR (NUMPIX)
polystdev = FLTARR (NUMPIX)
imgnum    = FINDGEN (numimg)

FOR jj = 0, numpix-1 DO BEGIN
	res  = MOMENT (rays [*, jj], sdev=derf)
	coef = POLY_FIT (imgnum, rays [*, jj], 2, yfit, yband, sigma)
	daystdev  [jj] = derf
	daymean   [jj] = res [0]
	polystdev [jj] = sigma
ENDFOR

!p.title='Mean and Standard Deviation at One Azimuth'
!y.title='pB'

plot_oo, height, daymean
OPLOT, height, daystdev, psym=3
OPLOT, height, polystdev, psym=1
STOP

!p.title='pB:  10 degree binned azimuths'

plot_oo, height, binimg [0, *], psym=3
FOR kk = 1, binsize - 1 DO BEGIN
	OPLOT, height, binimg [kk, *], psym=3
ENDFOR
END

