;+
; NAME		avgpb_plot.pro
;
; PURPOSE	Plot pB versus height.
;
; SYNTAX	avgpb_plot, 'mk4.cpb.fts'
;
; AUTHOR	David Elmore   13 Nov 2001
;
; HISTORY	21 Dec 2001 : [ALS] Adapt for use with average FITS image input.
;_

PRO avgpb_plot, fits_file

!y.title='Daily Average pB'
!x.title='Height (R-Sun)'
!p.title = 'Average pB vs. Height'
!p.font=1
!p.thick=1.6
!p.ticklen=1
!p.charsize=1.6
!p.charsize=1.2
!x.style=1
!y.style=1
!x.ticks=10
!x.range=[1.10,2.8]
!y.range=[1.e-10,1.e-6]

;--- Read Mk4 pB image.

avgimg = readfits (fits_file, hdu)	; Read average Mk4 FITS image.
					; Image is in polar coordinate form.

numimg = 1
numaz  = fxpar (hdu, 'NAXIS1')
numpix = fxpar (hdu, 'NAXIS2')

print, 'numaz/numpix: ', numaz, numpix

bhead    = readfits (fits_file, hduext, exten_no=1, /noscale)
pixscale = fxpar (hduext, 'DR')
pixoff   = fxpar (hduext, 'R0')
print, 'pixscale: ', pixscale, '   pixoff: ', pixoff

;--- Heights from header.

height = FINDGEN (numpix) * pixscale + pixoff

avgheight      = FLTARR (numaz)
avgheightplus  = FLTARR (numaz)
avgheightminus = FLTARR (numaz)
avgheightmax   = FLTARR (numaz)
avgheightmin   = FLTARR (numaz)

FOR ii = 0, numpix-1 DO BEGIN
    res = MOMENT (avgimg [*, ii], sdev=derf)
    avgheight      [ii] = res [0]
    avgheightplus  [ii] = res [0] + derf
    avgheightminus [ii] = res [0] - derf
    avgheightmax   [ii] = max (avgimg [*, ii])
    avgheightmin   [ii] = min (avgimg [*, ii])
ENDFOR

CLOSE,  1
OPENW,  1, 'corona_model'
WRITEU, 1, avgheight
CLOSE,  1

;--- Create azimuthally binned data.
;    Again, this will work on average image of the day.

binsize = 20
binimg = FLTARR (numaz / binsize, numpix)

FOR jj = 0, numaz - 1 DO BEGIN
    kk = jj / binsize
    binimg [kk, *] = binimg [kk, *] + avgimg [jj, *]
ENDFOR

for kk = 0, numaz / binsize - 1 do binimg [kk, *] = binimg [kk, *] / binsize

;print, binimg

;plot, binimg
;print, height 
	
print, "Set plot device"
set_plot, 'ps'
;STOP

PLOT_OO, height, avgheight
;PLOT,    height, avgheight, xrange = [1.1, 2.8], yrange = [1.e-10, 1.e-6], $
;	 /xlog, /ylog
OPLOT,   height, avgheightplus,  psym=3
OPLOT,   height, avgheightminus, psym=3

;OPLOT, height, avgheightmax, psym=3
;OPLOT, height, avgheightmin, psym=3

device, /close
set_plot, 'x'
STOP

;--- What is the stdev for each height over the day?
;
; This requires all the images going into the average
; image of the day to get RMS at each height
; over the day.

daystdev  = FLTARR (NUMPIX)
daymean   = FLTARR (NUMPIX)
polystdev = FLTARR (NUMPIX)
imgnum    = FINDGEN (numimg)

FOR jj = 0, numpix-1 DO BEGIN
    res  = MOMENT (rays [*, jj], sdev=derf)
    coef = POLY_FIT (imgnum, rays [*, jj], 2, yfit, yband, sigma)
    daystdev  [jj] = derf
    daymean   [jj] = res [0]
    polystdev [jj] = sigma
ENDFOR

!p.title='Mean and Standard Deviation at One Azimuth'
!y.title='pB'

PLOT_OO, height, daymean
OPLOT,   height, daystdev,  psym=3
OPLOT,   height, polystdev, psym=1
STOP

!p.title='pB:  10 degree binned azimuths'

PLOT_OO, height, binimg [0, *], psym=3
FOR kk = 1, binsize - 1 DO BEGIN
    OPLOT, height, binimg [kk, *], psym=3
ENDFOR

END

