;+
; NAME	fixmk4_gscale.pro
;
; PURPOSE	Read Mk4 FITS images & replace "DR" & "R0" FITS parameters.
;
; SYNTAX	fixmk4_gscale, fits_list
;
; HISTORY	Andrew L. Stanger   HAO/NCAR    9 July 2001
;		21 Sep 2001: recompute rmin & rmax.
;-

pro fixmk4_gscale, fits_list

;--- Mk4 Geometric scale factors from height mask test 28 April 2001 :

r0 = 0.00526475
dr = 0.938592

comment_dr = ''
comment_r0 = ''
comment_rmin = ''
comment_rmax = ''
comment_rsun = ''
comment_cdelt1 = ''
comment_cdelt2 = ''

CLOSE, 11
fits_name = ''

;--- File list loop.

OPENR, 11, fits_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

   ;--- Read FITS image header & pixel data.

   READF, 11, fits_name				; Get file name.
   print, 'fits_name: ', fits_name
   ftspos = STRPOS (fits_name, '.fts')

   img = readfits (fits_name, hdu, /noscale)		; Read image.
   ext = readfits (fits_name, hduext, exten_no=1)	; Read extension.

   ;--- Extract information from header.

   dateobs  = fxpar (hdu,    'DATE-OBS')
   timeobs  = fxpar (hdu,    'TIME-OBS')
   dataform = fxpar (hdu,    'DATAFORM')
   rmin     = fxpar (hdu,    'RMIN',     comment_rmin)		; Rsun units
   rmax     = fxpar (hdu,    'RMAX',     comment_rmax)		; Rsun units
   rsun     = fxpar (hdu,    'RSUN',     comment_rsun)		; arcsec/Rsun
   cdelt1   = fxpar (hdu,    'CDELT1',   comment_cdelt1)	; arcsec/pixel
   cdelt2   = fxpar (hdu,    'CDELT2',   comment_cdelt2)	; arcsec/pixel

   fr0      = fxpar (hduext, 'R0',       comment_r0)	; Height [Rsun] @pixel#0
   fdr      = fxpar (hduext, 'DR',       comment_dr)	; Scale: Rsun / pixel

   ;--- Re-compute pixel scale.

   rspix = dr * (960.0 / rsun)		; Corrected scale [Rsun / pixel]
   pixrs = 1.0 / rspix			; Corrected scale [pixel / Rsun]

   ;--- Modify cdelt1 & cdelt2.

   dataform = STRTRIM (dataform, 2)

   IF (dataform EQ 'CARTESIAN')  THEN	$
   BEGIN
      cdelt1 = rsun / pixrs		; arcsec/pixel
      cdelt2 = rsun / pixrs		; arcsec/pixel
   ENDIF

   IF (dataform EQ 'POLAR')  THEN	$
   BEGIN
      cdelt1 = 0.5			; degrees/pixel
      cdelt2 = rsun / pixrs		; arcsec/pixel
   ENDIF

   fxaddpar, hdu,    'CDELT1', cdelt1, comment_cdelt1
   fxaddpar, hdu,    'CDELT2', cdelt2, comment_cdelt2

   ;--- Adjust rmin & rmax for actual solar radius size.

   rmin = r0 * 960.0 / rsun
   rmax = (383.0 * dr + r0) * 960.0 / rsun

   fxaddpar, hdu,    'RMIN', rmin, comment_rmin
   fxaddpar, hdu,    'RMAX', rmax, comment_rmax

   ;--- Update R0 & RMIN.

;   fxaddpar, hduext, 'DR', dr, comment_dr
;   fxaddpar, hduext, 'R0', r0, comment_r0

   ;--- Write modified FITS file to disk.

   writefits, fits_name, img, hdu
   writefits, fits_name, ext, hduext, /APPEND

END	;}

CLOSE, 11

END
