;+
; NAME		mk4mapfts.pro
;
; PURPOSE	Convert Mk4 binary synoptic map into FITS format.
;
; SYNTAX	mk4mapfts, 'mk4_map_list'
;
; AUTHOR	Andrew L. Stanger   HAO/NCAR   18 Nov 2002
;
;-

PRO mk4mapfts, mk4_map_list

mapname  = ''
scrot    = ''
sradius  = ''
begdoy   = 0L
begyear  = 0L
enddoy   = 0L
endyear  = 0L
carr_rot = 0L
limb     = '    '

begmonth = 0L
begyear  = 0L

map = FLTARR (360, 181)

CLOSE, 11
OPENR, 11, mk4_map_list
WHILE (NOT EOF (11)) DO	$
BEGIN	;{

    ;--- Read synoptic map file name.

    READF, 11, mapname
    PRINT, 'map  name: ', mapname

    CLOSE, 12
    OPENR, 12, mapname
    READU, 12, map
    READU, 12, begdoy, begyear, enddoy, endyear
    READU, 12, carr_rot, limb
    CLOSE, 12

    print, 'begdoy/begyear: ', begdoy, begyear
    print, 'enddoy/endyear: ', enddoy, endyear

    fbegdoy = FLOAT (begdoy) / 100.0
    fenddoy = FLOAT (enddoy) / 100.0
    begdoy  = begdoy / 100
    enddoy  = enddoy / 100

    print, 'fbegdoy:     ', fbegdoy
    print, ' begdoy:     ',  begdoy

    ;--- Create beginning date for synoptic map.

    IF (begyear LT 50) THEN begyear = begyear + 2000
    IF (begyear GT 50 AND begyear LT 1900)  THEN begyear = begyear + 1900

    month_day, begdoy, begyear, begmonth, begday

    print, 'After month_day:'
    print, 'begdoy:   ', begdoy
    print, 'begyear:  ', begyear
    print, 'begmonth: ', begmonth
    print, 'begday:   ', begday

    sbegyear  = STRTRIM (STRING (begyear,  FORMAT='(I4)'), 2)
    sbegmonth = STRTRIM (STRING (begmonth, FORMAT='(I2)'), 2)
    sbegday   = STRTRIM (STRING (begday,   FORMAT='(I2)'), 2)
    IF (begmonth LT 10) THEN sbegmonth = '0' + sbegmonth
    IF (begday   LT 10) THEN sbegday   = '0' + sbegday
    begdate = sbegyear + '-' + sbegmonth + '-' + sbegday

    print, 'begdate: ', begdate

    ;--- Create beginning time for synoptic map.

    fbegdoyfrac = fbegdoy - begdoy
    fenddoyfrac = fenddoy - enddoy
    fbeghour = fbegdoyfrac * 24.0
    beghour  = FIX (fbeghour)

;    print, 'fbegdoyfrac: ', fbegdoyfrac
;    print, 'fbeghour:    ', fbeghour
;    print, ' beghour:    ',  beghour

    fbegminute = (fbeghour - beghour) * 60.0
    begminute  = FIX (fbegminute)
    fbegsecond = (fbegminute - begminute) * 60.0
    begsecond  = FIX (fbegsecond)

    sbeghour   = STRTRIM (STRING (beghour,   FORMAT='(I2)'), 2)
    sbegminute = STRTRIM (STRING (begminute, FORMAT='(I2)'), 2)
    sbegsecond = STRTRIM (STRING (begsecond, FORMAT='(I2)'), 2)
    IF (beghour   LT 10)  THEN sbeghour   = '0' + sbeghour
    IF (begminute LT 10)  THEN sbegminute = '0' + sbegminute
    IF (begsecond LT 10)  THEN sbegsecond = '0' + sbegsecond
    begtime = sbeghour + ':' + sbegminute + ':' + sbegsecond

;    print, 'sbeghour: ', sbeghour
    print, 'begtime: ', begtime

    ;--- file syntax: mk4.e1234.r123.df
    ;                           1
    ;                 01234567890123456
    
    ;--- Extract information from file name.

    slimb    = STRMID (mapname,  4,  1)
    scrot    = STRMID (mapname,  5,  4)
    sradius  = STRMID (mapname, 11,  3)
    basename = STRMID (mapname,  0, 14)
    fitsname = basename + '.fts'

    print, 'fits name: ', fitsname

    IF (slimb EQ 'e')  THEN crlimb = 'East'	$
    ELSE					$
    IF (slimb EQ 'w')  THEN crlimb = 'West'	$
    ELSE					$
			    crlimb = 'Unknown'

    type_obs = STRUPCASE (slimb) + ' cr' + scrot + ' [pb]'

    crot  = FIX (scrot)
    iradius = FIX (sradius)
    radius = iradius / 100.0
    sradius = STRTRIM (STRING (radius, FORMAT='(F5.2)'), 2)
    object  = 'Kcorona ' + sradius + 'Ro'

    datamin = MIN (map)
    datamax = MAX (map)
    dispmin = 0
    dispmax = datamax

    ;--- Make basic FITS header.

    fxhmake, hdu, map

    ;--- Add additional keywords & parameters to FITS header.

    fxaddpar, hdu, 'ORIGIN',   'MLSO',        'Mauna Loa Solar Observatory'
    fxaddpar, hdu, 'TELESCOP', 'Mk4',         'Mark-IV K-coronameter'
    fxaddpar, hdu, 'INSTRUME', 'Dalsa',       'Dalsa linear array CCD'
    fxaddpar, hdu, 'OBJECT',   object,        'Solar K-corona'
    fxaddpar, hdu, 'TYPE-OBS', type_obs,      'Mk4 Synoptic Map'
    fxaddpar, hdu, 'DATAFORM', 'synoptic map'
    fxaddpar, hdu, 'COORDNAM', 'Carrington'
    fxaddpar, hdu, 'CARR_ROT', crot,          'Carrington Rotation number'
    fxaddpar, hdu, 'SCAN_RAD', radius,        'Radius [Rsun] (solar radius units)'
    fxaddpar, hdu, 'LIMB',     crlimb,        'Limb of sun from which map was made'
    fxaddpar, hdu, 'DATE-OBS', begdate,       'Beginning image date for synoptic map'
    fxaddpar, hdu, 'TIME-OBS', begtime,       'Beginning image time for synoptic map'

    fxaddpar, hdu, 'CTYPE1',   'Longitude',   'Carrington Longitude [degrees]'
    fxaddpar, hdu, 'CRVAL1',   0.0,           'Longitude reference'
    fxaddpar, hdu, 'CDELT1',   1.0,           'degrees'
    fxaddpar, hdu, 'CROTA1',   0.0,           'degrees'
    fxaddpar, hdu, 'CTYPE2',   'Latitude',    'Latitude [degrees]'
    fxaddpar, hdu, 'CRVAL2',   0.0,           'Latitude reference'
    fxaddpar, hdu, 'CDELT2',   1.0,           'degrees'
    fxaddpar, hdu, 'CROTA2',   0.0,           'degrees'
    fxaddpar, hdu, 'BUNIT',    'Bsun',        'Brightness w.r.t. solar disk'
    fxaddpar, hdu, 'BSCALE',   1.0
    fxaddpar, hdu, 'BZERO',    0.0
    fxaddpar, hdu, 'DATAMIN',  datamin
    fxaddpar, hdu, 'DATAMAX',  datamax
    fxaddpar, hdu, 'DISPMIN',  dispmin
    fxaddpar, hdu, 'DISPMAX',  dispmax
    fxaddpar, hdu, 'DIRAXIS1', 'RIGHT',       'X-axis left   -> right'
    fxaddpar, hdu, 'DIRAXIS2', 'UP',          'Y-axis bottom -> top'
    fxaddpar, hdu, 'VERSION',  'mk4syn.v1.0', 'Software version'
;    fxaddpar, hdu, 'COMMENT', 'Mk4 SYNOPTIC MAP'
;    fxaddpar, hdu, 'COMMENT', 'Array origin [lower left corner] is at longitude 0'
;    fxaddpar, hdu, 'COMMENT', 'latitude -90 [= 90 degrees south].'
;    fxaddpar, hdu, 'COMMENT', 'Mauna Loa Solar Observatory is located on the island of Hawaii.'
;    fxaddpar, hdu, 'COMMENT', 'MLSO is operated by the High Altitude Observatory [HAO],'
;    fxaddpar, hdu, 'COMMENT', 'a division of the National Center for Atmospheric Research'

    ;--- Write FITS file to disk.

    writefits, fitsname, map, hdu

END	;}

CLOSE, 11

END
