;+
; NAME	mk4pbfix.pro
;
; PURPOSE	Zero out inner diodes for Mk4 pB images.
;
; SYNTAX	mk4pbfix, pbfitslist
;
; INPUT		Images in native format.
; OUTPUT	Images in native format.
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   8 December 1999
;-

pro mk4pbfix, rpb_list

header = bytarr (1024)
img    = intarr (384, 1080)
red = bindgen (256)
green = red
blue  = red
rpb_name = ''
close,1
lct, '/home/stanger/color/qual.lut'		; color table.
lct, '/home/stanger/color/bwy.lut'		; color table.
tvlct, red, green, blue, /get
window, xsize=1080, ysize=384

;--- File list loop.

CLOSE, 1
OPENR, 1, rpb_list				; Open list file.
WHILE (NOT EOF (1)) DO	$
BEGIN	;{

   ;--- Read image header & pixel data.

   READF, 1, rpb_name				; Get FITS file name.
   print, 'rpb_name: ', rpb_name

;   rpb_img = readfits (rpb_name, hdu)		; Read FITS image.
   CLOSE, 2
   OPENR, 2, rpb_name
   READU, 2, header
   READU, 2, img
   CLOSE, 2

   ;--- Extract information from header.

;   rpb_min = MIN (rpb_img, max=rpb_max)

;   rpb_min = fxpar (hdu, 'DISPMIN')
;   rpb_max = fxpar (hdu, 'DISPMAX')
;   img_min = fxpar (hdu, 'DATAMIN')
;   img_max = fxpar (hdu, 'DATAMAX')

;   offset = 5.0E-7				; Mk3
;   offset = 1.2E-6				; Mk4
;   i_min = 0.0
;   i_max = (img_max - rpb_max) / 2.0 + rpb_max
;   i_max = rpb_max + offset

   ;--- Zero out lower diodes.

   new_img = img
   new_img [0:40, *] = 0

;   new_min = MIN (new_img, max=new_max)		; Get image min/max.
;   new_min = SQRT (i_min)
;   new_max = SQRT (i_max)

;   print, FORMAT = '("rpb_min/max: ", E10.3, E10.3)', rpb_min, rpb_max
;   print, FORMAT = '("img_min/max: ", E10.3, E10.3)', img_min, img_max
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   IF (new_min LT 0.0) THEN	$			; Symmetric about zero.
;      IF (-new_min LT new_max) THEN new_min = -new_max  ELSE new_max = -new_min
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

;   new_min = -5.00e-8
;   new_max =  5.00e-8
;   print, FORMAT = '("new_min/max: ", E10.3, E10.3)', new_min, new_max

   ;--- Display image.

   byt_img = bytscl (new_img, min=0, max=17500, top=255)
   TV, transpose (byt_img)						; Display image.

   ;--- Write newference image to disk.

   fpos = strpos (rpb_name, '.pB')
   new_name = strmid (rpb_name, 0, fpos) + '.disk.pB'
   print, 'Write fixed pB image : ', new_name
;   writefits, new_name, new_img, hdu			; Write new image.
   OPENW,  2, new_name
   WRITEU, 2, header
   WRITEU, 2, new_img
   CLOSE,  2
END	;}

CLOSE, 1
END
