;+
; NAME		mk4plotgif1.pro
;
; PURPOSE	Routine to average a list of images
;
; AUTHOR	David Elmore   13 Nov 2001   [avgmk4.pro]
;
; HISTORY	Andrew L. Stanger
;		 3 Jan 2002: Cosmetic changes only.
;		 4 Jan 2002: Plots go to postscript file.
;			     Input & output files are in FITS format.
;			     Update AVGTIME* keywords in FITS header.
;		 7 Jan 2002: Update PROCSTEP.
;		 9 Jan 2002: use NATIVE file format.
;		10 Jan 2002: Add time plots.
;			     Save plots as GIF files.
;		17 May 2002: Set fixed y-axis range for plots.
;		21 May 2001: Set fixed x-axis range [16-24] for plots.
;-

PRO mk4plotgif1, file_list

;--- Initialize variables.

avgtime1 = ''
avgtime2 = ''
avgtime3 = ''
avgtime4 = ''
avgtime5 = ''
avgtime6 = ''
avgtime7 = ''
avgtime8 = ''

cavgtime1 = ''
cavgtime2 = ''
cavgtime3 = ''
cavgtime4 = ''
cavgtime5 = ''
cavgtime6 = ''
cavgtime7 = ''
cavgtime8 = ''

is_cal_avg  = 18
azplace     = 220

filename = 'mk4.fts'


;--- Read a header to set some variables.

CLOSE, 2
OPENR, 2, file_list 
READF, 2, filename
CLOSE, 2
filename = STRTRIM (filename, 2)

headib  = 0 & headib  = LONARR (16)
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

CLOSE, 1
OPENR, 1, filename
READU, 1, headib
READU, 1, headf
READU, 1, headie
READU, 1, headpad
CLOSE, 1

;PRINT, headib
;PRINT, headf
;PRINT, headie

numaz    = headib  [9]				; Xdim
numpix   = headib [10]				; Ydim
pixoff   = headf  [23]				; r0
pixscale = headf  [24]				; dr

;PRINT, numaz, numpix, pixscale, pixoff

;img  = readfits (filename, hdu0, /noscale)
;hdr0 = readfits (filename, ext0, /noscale, exten_no=1)

;numaz    = fxpar (hdu0, 'NAXIS1')		; Xdim
;numpix   = fxpar (hdu0, 'NAXIS2')		; Ydim
;date     = fxpar (hdu0, 'DATE-OBS')
;pixoff   = fxpar (ext0, 'R0')			; r0
;pixscale = fxpar (ext0, 'DR')			; dr

;avgtime1 = STRTRIM (fxpar (ext0, 'AVGTIME1', cavgtime1), 2)
;avgtime2 = STRTRIM (fxpar (ext0, 'AVGTIME2', cavgtime2), 2)
;avgtime3 = STRTRIM (fxpar (ext0, 'AVGTIME3', cavgtime3), 2)
;avgtime4 = STRTRIM (fxpar (ext0, 'AVGTIME4', cavgtime4), 2)
;avgtime5 = STRTRIM (fxpar (ext0, 'AVGTIME5', cavgtime5), 2)
;avgtime6 = STRTRIM (fxpar (ext0, 'AVGTIME6', cavgtime6), 2)
;avgtime7 = STRTRIM (fxpar (ext0, 'AVGTIME7', cavgtime7), 2)
;avgtime8 = STRTRIM (fxpar (ext0, 'AVGTIME8', cavgtime8), 2)

;syear  = STRMID (date, 0, 4)
;smonth = STRMID (date, 5, 2)
;sday   = STRMID (date, 8, 2)

year    = headib [0]			; Image.Year
month   = headib [1]			; Image.Month
day     = headib [2]			; Image.Day

IF (year LT 1900) THEN year = year + 1900

syear   = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
smonth  = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
sday    = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)

IF (month LT 10) THEN smonth = '0' + smonth
IF (day   LT 10) THEN sday   = '0' + sday

date = syear + '-' + smonth + '-' + sday

;PRINT, 'syear: ', syear, ' smonth: ', smonth, ' sday: ', sday

plotfile = syear + smonth + sday + '.mk4.cpb.plot.ps'

;PRINT, 'numaz/numpix: ', numaz, numpix
;PRINT, 'pixscale/pixoff: ', pixscale, pixoff


;--- Specify low, middle & high radius sample heights.

radlo  = 1.2			; Rsun units
radmid = 1.5
radhi  = 1.8
pixlo  = FIX ((radlo  - pixoff) / pixscale + 0.5)
pixmid = FIX ((radmid - pixoff) / pixscale + 0.5)
pixhi  = FIX ((radhi  - pixoff) / pixscale + 0.5)

spixlo  = STRTRIM (STRING (pixlo), 2)
spixmid = STRTRIM (STRING (pixmid), 2)
spixhi  = STRTRIM (STRING (pixhi), 2)
sradlo  = STRTRIM (STRING (radlo,  FORMAT='(F3.1)'), 2)
sradmid = STRTRIM (STRING (radmid, FORMAT='(F3.1)'), 2)
sradhi  = STRTRIM (STRING (radhi,  FORMAT='(F3.1)'), 2)

;PRINT, 'pixlo/pixmid/pixhi: ', pixlo, pixmid, pixhi

;--- Heights from header of first file.

height = FINDGEN (numpix) * pixscale + pixoff

;--- Determine number of images to average.

CLOSE, 2
OPENR, 2, file_list
numimg = 0

WHILE ( NOT EOF (2) ) DO BEGIN
    numimg = numimg + 1
    READF, 2, filename
ENDWHILE

;--- Define arrays.

;PRINT, 'numimg: ', numimg

inimg  = INTARR (NUMAZ, NUMPIX)
avgimg = FLTARR (NUMAZ, NUMPIX)

rays   = FLTARR (numimg, NUMPIX)
pblo   = FLTARR (numimg)
pbmid  = FLTARR (numimg)
pbhi   = FLTARR (numimg)
hours  = FLTARR (numimg)

;--- Open list file.

CLOSE, 2
OPENR, 2, file_list

;WINDOW, 0, xs=numaz, ys=numpix
;WINDOW, 1, xs=numaz, ys=numpix
imgnum = 0

;--- File loop.

WHILE ( NOT EOF (2) ) DO BEGIN	;{
    READF, 2, filename
    filename = STRTRIM (filename, 2)
    PRINT, filename

    CLOSE, 1
    OPENR, 1, filename				; Open binary image file.

    READU, 1, headib				; 32-bit integers
    READU, 1, headf				; floats
    READU, 1, headie				; 32-bit integers
    READU, 1, headpad				; char

    READU, 1, inimg				; Read image array.

    ;--- Extract items from binary header.

    scale   = headf [13]			; data_scale
    offset  = headf [14]			; data_offset
    hour    = headib [3]			; image hour
    minute  = headib [4]			; image minute
    shour   = STRTRIM (STRING (hour),   2)
    sminute = STRTRIM (STRING (minute), 2)
    if (hour LT 10)   then shour   = '0' + shour
    if (minute LT 10) then sminute = '0' + sminute
    imgtime = shour + sminute
    hours [imgnum] = FLOAT (hour) + FLOAT (minute) / 60.0

    ;--- Read FITS image.

;    inimg = readfits (filename, hdu, /noscale)
;    hdr   = readfits (filename, ext, /noscale, exten_no=1)

;    itime = fxpar (hdu, 'TIME-OBS')
;    imgtime   = STRMID (itime, 0, 5)
;    scale   = fxpar (ext, 'DATASCL')		; data_scale
;    offset  = fxpar (ext, 'DATAOFF')		; data_offset

    ;--- Apply photometric scaling.
    ;--- Accumulate image sum.

    img = (FLOAT (inimg) - offset) / scale 

    ;--- Compute azimuthal averages at low, middle & high radius positions.

    pblo  [imgnum] = TOTAL (img [*, pixlo])  / numaz
    pbmid [imgnum] = TOTAL (img [*, pixmid]) / numaz
    pbhi  [imgnum] = TOTAL (img [*, pixhi])  / numaz
;    PRINT, 'pblo/mid/hi: ', pblo [imgnum], pbmid [imgnum], pbhi [imgnum]

    ;--- Accumulate image sum for average image.

    avgimg = avgimg + img

    ;--- Store radial scan at one azimuth (azplace).

    rays [imgnum, *] = img [azplace, *] 

    imgnum = imgnum + 1

    ;--- Display image & average image.

;    WSET, 0
;    TV, BYTSCL (ALOG10 (img + 2.e-9), -9, -5)
;    WSET, 1
;    TV, BYTSCL (ALOG10 ((avgimg / imgnum) + 2.e-9), -9, -5)

    ;--- Concatenate image times to be stored in header.

    inum1 = (imgnum - 1) / 4
    inum2 = (imgnum - 1) - inum1 * 4
;    PRINT, 'imgnum/inum1/inum2: ', imgnum, inum1, inum2, ' imgtime: ', imgtime

;    CASE inum1 OF	;{
;    0: BEGIN						$
;       avgtime1 = avgtime1 + imgtime + ' '
;       PRINT, 'avgtime1: ', avgtime1
;       fxaddpar, ext0, 'AVGTIME1', avgtime1, cavgtime1
;       END
;    1: BEGIN						$
;       avgtime2 = avgtime2 + imgtime + ' '
;       PRINT, 'avgtime2: ', avgtime2
;       fxaddpar, ext0, 'AVGTIME2', avgtime2, cavgtime2
;       END
;    2: BEGIN						$
;       avgtime3 = avgtime3 + imgtime + ' '
;       PRINT, 'avgtime3: ', avgtime3
;       fxaddpar, ext0, 'AVGTIME3', avgtime3, cavgtime3
;       END
;    3: BEGIN						$
;       avgtime4 = avgtime4 + imgtime + ' '
;       PRINT, 'avgtime4: ', avgtime4
;       fxaddpar, ext0, 'AVGTIME4', avgtime4, cavgtime4
;       END
;    4: BEGIN						$
;       avgtime5 = avgtime5 + imgtime + ' '
;       PRINT, 'avgtime5: ', avgtime5
;       fxaddpar, ext0, 'AVGTIME5', avgtime5, cavgtime5
;       END
;    5: BEGIN						$
;       avgtime6 = avgtime6 + imgtime + ' '
;       PRINT, 'avgtime6: ', avgtime6
;       fxaddpar, ext0, 'AVGTIME6', avgtime6, cavgtime6
;       END
;    6: BEGIN						$
;       avgtime7 = avgtime7 + imgtime + ' '
;       PRINT, 'avgtime7: ', avgtime7
;       fxaddpar, ext0, 'AVGTIME7', avgtime7, cavgtime7
;       END
;    7: BEGIN						$
;       avgtime8 = avgtime8 + imgtime + ' '
;       PRINT, 'avgtime8: ', avgtime8
;       fxaddpar, ext0, 'AVGTIME8', avgtime8, cavgtime8
;       END
;
;    ELSE: PRINT, 'imgnum: ', imgnum, ' greater than maximum allowed'
;    ENDCASE	;}

ENDWHILE	;}

;--- Compute average image.

avgimg = avgimg / numimg 
;TV, BYTSCL (ALOG10 (avgimg + 2.e-9), -9, -5)

;--- Write average image to disk NATIVE format.

headie [2] = is_cal_avg

avgtime = avgtime1 + avgtime2 + avgtime3 + avgtime4 + avgtime5 + avgtime6 $
	+ avgtime7 + avgtime8

;PRINT, 'avgtime: '
;PRINT, avgtime

;WHILE (STRLEN (avgtime) LT 160) DO avgtime = avgtime + ' '

;lenavgtime = STRLEN (avgtime)
;PRINT, 'lenavgtime: ', lenavgtime

;bavgtime = BYTARR (lenavgtime)
;READS, avgtime, bavgtime, format='(160I1)'
;PRINT, 'bavgtime: '
;PRINT, bavgtime

;headpad [192:351] = bavgtime


;avgchar = ''
;FOR i = 0, lenavgtime DO	$
;BEGIN
;    avgchar = STRMID (avgtime, i, 1) 
;    READS, avgchar, headpad [192+i]
;    headpad [192 + i] = FIX (avgchar)
;    PRINT, 'i: ', i, ' avgchar: ', avgchar
;END


;headpad [192:211] = avgtime1
;headpad [212:231] = avgtime2
;headpad [232:251] = avgtime3
;headpad [252:271] = avgtime4
;headpad [272:291] = avgtime5
;headpad [292:311] = avgtime6
;headpad [312:331] = avgtime7
;headpad [332:351] = avgtime8

outfile = STRMID (filename, 0, 6) + '.mk4.cpb.avg'

;CLOSE, 1
;OPENW, 1, outfile
;    WRITEU, 1, headib
;    WRITEU, 1, headf
;    WRITEU, 1, headie
;    WRITEU, 1, headpad
;    WRITEU, 1, FIX (avgimg * scale + offset)
;CLOSE, 1

;fxaddpar, ext0, 'PROCSTEP', is_cal_avg			; ProcSteps
;outfile = STRMID (filename, 0, 6) + '.mk4.cpb.avg.fts'
;writefits, outfile, FIX (avgimg * scale + offset), hdu0
;writefits, outfile, hdr0, ext0, /append


;WINDOW, 2, xs=512, ys=512

; What is mean and std dev for each height?
; An indicator of coronal structure.

avgheight      = FLTARR (numaz)
avgheightplus  = FLTARR (numaz)
avgheightminus = FLTARR (numaz)
avgheightmax   = FLTARR (numaz)
avgheightmin   = FLTARR (numaz)

FOR ii = 0, numpix-1 DO BEGIN
    res = MOMENT (avgimg [*, ii], sdev=derf)
    avgheight [ii]      = res [0]
    avgheightplus [ii]  = res [0] + derf
    avgheightminus [ii] = res [0] - derf
    avgheightmax [ii]   = max (avgimg [*, ii])
    avgheightmin [ii]   = min (avgimg [*, ii])
ENDFOR

;CLOSE,  1
;OPENW,  1, 'corona_model'
;WRITEU, 1, avgheight
;CLOSE,  1

;writefits, 'corona_model', avgheight


;--- Create azimuthally binned (i.e., averaged) data.
;    Again, this will work on average image of the day.

binsize = 20			; Remember that azimuth increment is 0.5 degree.
binimg = FLTARR (numaz / binsize, numpix)

FOR jj = 0, numaz-1 DO BEGIN
	kk = jj / binsize
	binimg [kk, *] = binimg [kk, *] + img [jj, *]
ENDFOR

FOR kk = 0, numaz/binsize - 1 DO binimg [kk, *] = binimg [kk, *] / binsize
	

;--- Write plots to GIF files.

SET_PLOT, 'Z'				; Use Z-buffer for plots.

;PRINT, !version.os
;IF (!version.os EQ 'IRIX') THEN cs = 0.8 ELSE cs = 1.6
cs = 1.0

LOADCT, 0				; Load greyscale color map
fcol =   0
bcol = 255
!P.BACKGROUND=bcol

;PRINT, "Set plot device"
;SET_PLOT, 'PS'
;DEVICE, filename = plotfile

;WINDOW, xsize=640, ysize=480

;STOP

;--- Plot average pB vs. time at 3 different heights.

;PRINT, 'hours: '
;PRINT, hours

;FOR i = 0, numimg - 1 DO	$
;BEGIN
;    PRINT, 'hours: ', hours [i], ' pblo/mid/hi: ',	$
;	    pblo [i], pbmid [i], pbhi [i]
;END

xmin = MIN (hours)
xmax = MAX (hours)
xmin = 16.0
xmax = 24.0
;PRINT, 'xmin/xmax: ', xmin, xmax

ymin = MIN (pblo)
ymax = MAX (pblo)
;ymin = 2.0e-7
;ymax = 5.0e-7
;PRINT, 'ymin/ymax: ', ymin, ymax

ptitle = 'Average pB @' + sradlo + ' Rsun vs. Time' + '   Mk4 ' + date
;!p.title = 'Average pB at pixel 100 vs. Time' + '   Mk4 ' + date
;!x.title = 'Time [Hours]'
;!y.title = 'pB'
;!x.style = 0
;!y.style = 0

PLOT, hours, pblo,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle='pB',		$
      charsize=cs, ticklen=1, color=fcol, 			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.r' + sradlo + '.gif'
write_gif, gifname, plotimg

;STOP

ymin = MIN (pbmid)
ymax = MAX (pbmid)
;ymin = 2.0e-8
;ymax = 1.2e-7
;ymin = 4.0e-8
;ymax = 1.0e-7
;PRINT, 'ymin/ymax: ', ymin, ymax

ptitle = 'Average pB @' + sradmid + ' Rsun vs. Time' + '   Mk4 ' + date

PLOT, hours, pbmid,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle='pB',		$
      charsize=cs, ticklen=1, color=fcol,			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.r' + sradmid + '.gif'
write_gif, gifname, plotimg

ymin = MIN (pbhi)
ymax = MAX (pbhi)
;ymin = 0.0
;ymax = 4.0E-8
;ymin = 1.0E-8
;ymax = 3.0E-8
;PRINT, 'ymin/ymax: ', ymin, ymax

ptitle = 'Average pB @' + sradhi + ' Rsun vs. Time' + '   Mk4 ' + date
;!p.title = 'Average pB at pixel 200 vs. Time' + '   Mk4 ' + date
;!x.title = 'Time [Hours]'
;!y.title = 'pB'

PLOT, hours, pbhi,						$
      xrange=[xmin, xmax], yrange=[ymin, ymax],			$
      title=ptitle, xtitle='Time [Hours]', ytitle='pB',		$
      charsize=cs, ticklen=1, color=fcol,			$
      xmargin=[11,3], ymargin=[5,3], ytickformat='(E9.1)'

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.r' + sradhi + '.gif'
write_gif, gifname, plotimg

;--- Average radial profile.

ptitle    = 'Average pB vs. Height' + '   Mk4 ' + date

PLOT, height, avgheight, /xlog, /ylog,				$
      title=ptitle, xtitle='Height [Rsun]', ytitle='pB',	$
      xrange=[1.1, 2.5], yrange=[1.0e-10, 1.0e-5], xticks=6,	$
      charsize=cs, ticklen=1, color=fcol, xmargin=[10,3], ymargin=[5,3]
OPLOT, height, avgheightplus,  psym=3, color=fcol
OPLOT, height, avgheightminus, psym=3, color=fcol

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.radave.gif'
write_gif, gifname, plotimg

;OPLOT, height, avgheightmax, psym=3, color=fcol
;OPLOT, height, avgheightmin, psym=3, color=fcol

;STOP

;--- What is the stdev for each height over the day?
;
; This requires all the images going into the average
; image of the day to get RMS at each height
; over the day.

;--- Plot mean & standard deviation.

daystdev  = FLTARR (NUMPIX)
daymean   = FLTARR (NUMPIX)
polystdev = FLTARR (NUMPIX)
imgnum    = FINDGEN (numimg)

FOR jj = 0, numpix-1 DO BEGIN
	res  = MOMENT (rays [*, jj], sdev=derf)
	coef = POLY_FIT (imgnum, rays [*, jj], 2, yfit, yband, sigma)
	daystdev  [jj] = derf
	daymean   [jj] = res [0]
	polystdev [jj] = sigma
ENDFOR

sdaz = STRING (azplace / 2, FORMAT='(I)')	; azimuthal increment = 0.5 deg.
sdaz = STRTRIM (azplace, 2)
ptitle = 'Mean & Std.Dev. @' + sdaz + ' degrees vs. Height' + '   Mk4 ' + date

PLOT, height, daymean, /xlog, /ylog,				$
      title=ptitle, xtitle='Height [Rsun]', ytitle='pB',	$
      xrange=[1.1, 2.5], yrange=[1.0e-10, 1.0e-5], xticks=6,	$
      charsize=cs, ticklen=1, color=fcol, xmargin=[10,3], ymargin=[5,3]
OPLOT, height, daystdev,  psym=3, color=fcol
;OPLOT, height, polystdev, psym=1, color=fcol

XYOUTS, 0.55, 0.85, 'Mean     -', /NORMAL, charsize=cs, color=fcol
XYOUTS, 0.55, 0.80, 'Std Dev  .', /NORMAL, charsize=cs, color=fcol
;XYOUTS, 0.55, 0.75, 'Std Dev Fit +', /NORMAL, charsize=cs, color=fcol

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.raddev.gif'
write_gif, gifname, plotimg


;STOP

;--- Plot azimuthal averages.

ptitle = '10-degree binned azimuths vs. Height' + '   Mk4 ' + date

PLOT, height, binimg [0, *], psym=3, /xlog, /ylog,		$
      title=ptitle, xtitle='Height [Rsun]', ytitle='pB',	$
      xrange=[1.1, 2.5], yrange=[1.0e-10, 1.0e-5], xticks=6,	$
      charsize=cs, ticklen=1, color=fcol, xmargin=[10,3], ymargin=[5,3]
FOR kk = 1, binsize - 1 DO BEGIN
	OPLOT, height, binimg [kk, *], psym=3, color=fcol
ENDFOR

plotimg = TVRD ()
gifname = syear + smonth + sday + '.mk4.radbin.gif'
write_gif, gifname, plotimg

!P.BACKGROUND=0
;DEVICE, /close
SET_PLOT, 'X'

END

