;+
; NAME		mk4ploth.pro
;
; PURPOSE	Routine to plot hair line position vs. time.
;
; AUTHOR	Andrew L. Stanger    4 Oct 2002
;-

PRO mk4ploth, file_list

filename = 'mk4.fts'
hlo = FLTARR (200)
hhi = FLTARR (200)
hourtab = FLTARR (200)
red   = BYTARR (256)
green = BYTARR (256)
blue  = BYTARR (256)

;--- Open list file.

CLOSE, 2
OPENR, 2, file_list

SET_PLOT, 'Z'
;WINDOW, xs=640, ys=480
lct, '/home/stanger/color/bwcp.lut'
TVLCT, red, green, blue, /get

imgnum = 0

;--- File loop.

WHILE ( NOT EOF (2) ) DO BEGIN	;{
    READF, 2, filename
    filename = STRTRIM (filename, 2)
    PRINT, filename

    ;--- Read FITS image.

    inimg = readfits (filename, hdu, /noscale)
    hdr   = readfits (filename, ext, /noscale, exten_no=1)

    if (imgnum EQ 0) THEN	$
    BEGIN
        sdate   = fxpar (hdu, 'DATE-OBS')
        syear   = STRMID (sdate, 0, 4)
        smonth  = STRMID (sdate, 5, 2)
        sday    = STRMID (sdate, 8, 2)
    END

    stime   = fxpar (hdu, 'TIME-OBS')
    shour   = STRMID (stime, 0, 2)
    sminute = STRMID (stime, 3, 2)
    ssecond = STRMID (stime, 6, 2)
    ihour   = FIX (shour)
    iminute = FIX (sminute)
    isecond = FIX (ssecond)
    hourtab [imgnum] = FLOAT (ihour) + FLOAT (iminute) / 60.0 + $
		                       FLOAT (isecond) / 3600.0

    hlo [imgnum] = fxpar (ext, 'ORGHAIR0')
    hhi [imgnum] = fxpar (ext, 'ORGHAIR1')

    IF (hlo [imgnum] GT 0 AND hhi [imgnum] GT 0)  THEN	imgnum = imgnum + 1

ENDWHILE	;}

hairlo = hlo [0:imgnum-1]
hairhi = hhi [0:imgnum-1]
hours = hourtab [0:imgnum-1]
hairdif = FLTARR (imgnum)
hairdif = hairhi - hairlo

hlomin  = MIN (hairlo,  max=hlomax)
hhimin  = MIN (hairhi,  max=hhimax)
hdifmin = MIN (hairdif, max=hdifmax)

print, 'hairlo: '
print, hairlo
print, ' '
print, 'hairhi: '
print, hairhi
print, ' '
print, 'hairdif: '
print, hairdif
print, ' '
print, 'hours: '
print, hours

ptitle = sdate + ' Mk4 Hairline Positions vs. Time'
xlab   = 'HOURS'
ylab   = 'Height [pixels]'

PLOT,	hours, hairlo,			$
	title=ptitle,			$
	xtitle=xlab,			$
	ytitle=ylab,			$
	xrange = [17.0,  24.0],		$
	yrange = [90.0, 300.0],		$
	/nodata

OPLOT,  hours, hairlo, color=254
OPLOT,  hours, hairhi, color=252

XYOUTS, 0.85, 0.90, 'HAIR-HIGH', /NORMAL, color=252
XYOUTS, 0.85, 0.85, 'HAIR-LOW',  /NORMAL, color=254

hairplot = TVRD ()
gifname = syear + smonth + sday + '.hairlines.gif'
;write_gif, gifname, hairplot, red, green, blue

ptitle = sdate + ' Mk4 Low Hairline Position vs. Time'
PLOT,	hours, hairlo,			$
	title=ptitle,			$
	xtitle=xlab,			$
	ytitle=ylab,			$
	xrange = [17.0,  24.0],		$
;	yrange = [90.0, 130.0],		$
	yrange = [hlomin, hlomax],	$
	color=255

hairplot = TVRD ()
gifname = syear + smonth + sday + '.hairlo.gif'
write_gif, gifname, hairplot, red, green, blue

ptitle = sdate + ' Mk4 High Hairline Position vs. Time'
PLOT,	hours, hairhi,			$
	title=ptitle,			$
	xtitle=xlab,			$
	ytitle=ylab,			$
	xrange = [17.0,  24.0],		$
;	yrange = [250.0, 300.0],		$
	yrange = [hhimin, hhimax],	$
	color=255

hairplot = TVRD ()
gifname = syear + smonth + sday + '.hairhi.gif'
write_gif, gifname, hairplot, red, green, blue

ptitle = sdate + ' Diff [Mk4 HairHi - HairLo] vs. Time'
PLOT,	hours, hairdif,			$
	title=ptitle,			$
	xtitle=xlab,			$
	ytitle=ylab,			$
	xrange = [17.0,  24.0],		$
;	yrange = [150.0, 200.0],	$
	yrange = [hdifmin, hdifmax],	$
	color=255

hairplot = TVRD ()
gifname = syear + smonth + sday + '.hairdif.gif'
write_gif, gifname, hairplot, red, green, blue

!P.BACKGROUND=0
SET_PLOT, 'X'

END

