;+
; NAME		mk4radave.pro
;
; PURPOSE	Routine to average a list of images
;
; AUTHOR	David Elmore   13 Nov 2001   [avgmk4.pro]
;
; HISTORY	Andrew L. Stanger
;		 3 Jan 2002: Cosmetic changes only.
;		 4 Jan 2002: Plots go to postscript file.
;			     Input & output files are in FITS format.
;			     Update AVGTIME* keywords in FITS header.
;		 7 Jan 2002: Update PROCSTEP.
;		 9 Jan 2002: use NATIVE file format.
;-

PRO mk4radave, file_list

SET_PLOT, 'X'

avgtime1 = ''
avgtime2 = ''
avgtime3 = ''
avgtime4 = ''
avgtime5 = ''
avgtime6 = ''
avgtime7 = ''
avgtime8 = ''

cavgtime1 = ''
cavgtime2 = ''
cavgtime3 = ''
cavgtime4 = ''
cavgtime5 = ''
cavgtime6 = ''
cavgtime7 = ''
cavgtime8 = ''

;--- Set up display values.

!y.title    = 'Daily Average pB'
!x.title    = 'Height (R-Sun)'
!p.title    = 'Average pB vs. Height'
!p.font     = 1
!p.thick    = 1.6
!p.ticklen  = 1
;!p.charsize = 1.6
!p.charsize = 1.2
!x.style    = 1
!y.style    = 1
!x.ticks    = 10
!x.range    = [1.10,     2.8]
!y.range    = [1.e-10, 1.e-6]

is_cal_avg  = 18
azplace     = 220

filename = 'mk4.fts'


;--- Read a header to set some variables.

CLOSE, 2
OPENR, 2, file_list 
READF, 2, filename
CLOSE, 2
filename = STRTRIM (filename, 2)

headib  = 0 & headib  = LONARR (16)
headf   = 0 & headf   = FLTARR (32)
headie  = 0 & headie  = LONARR (16)
headpad = 0 & headpad = BYTARR (1024 - 4*16 - 4*32 - 4*16)

CLOSE, 1
OPENR, 1, filename
READU, 1, headib
READU, 1, headf
READU, 1, headie
READU, 1, headpad
CLOSE, 1

PRINT, headib
PRINT, headf
PRINT, headie

numaz    = headib  [9]
numpix   = headib [10]
pixoff   = headf  [23]
pixscale = headf  [24]

;avgtime1 = STRING (headpad [192:211])
;avgtime2 = STRING (headpad [212:231])
;avgtime3 = STRING (headpad [232:251])
;avgtime4 = STRING (headpad [252:271])
;avgtime5 = STRING (headpad [272:291])
;avgtime6 = STRING (headpad [292:311])
;avgtime7 = STRING (headpad [312:331])
;avgtime8 = STRING (headpad [332:351])

PRINT, numaz, numpix, pixscale, pixoff

;img  = readfits (filename, hdu0, /noscale)
;hdr0 = readfits (filename, ext0, /noscale, exten_no=1)

;numaz    = fxpar (hdu0, 'NAXIS1')	; Xdim
;numpix   = fxpar (hdu0, 'NAXIS2')	; Ydim
;date     = fxpar (hdu0, 'DATE-OBS')
;pixoff   = fxpar (ext0, 'R0')		; r0
;pixscale = fxpar (ext0, 'DR')		; dr

;avgtime1 = STRTRIM (fxpar (ext0, 'AVGTIME1', cavgtime1), 2)
;avgtime2 = STRTRIM (fxpar (ext0, 'AVGTIME2', cavgtime2), 2)
;avgtime3 = STRTRIM (fxpar (ext0, 'AVGTIME3', cavgtime3), 2)
;avgtime4 = STRTRIM (fxpar (ext0, 'AVGTIME4', cavgtime4), 2)
;avgtime5 = STRTRIM (fxpar (ext0, 'AVGTIME5', cavgtime5), 2)
;avgtime6 = STRTRIM (fxpar (ext0, 'AVGTIME6', cavgtime6), 2)
;avgtime7 = STRTRIM (fxpar (ext0, 'AVGTIME7', cavgtime7), 2)
;avgtime8 = STRTRIM (fxpar (ext0, 'AVGTIME8', cavgtime8), 2)

;syear  = STRMID (date, 0, 4)
;smonth = STRMID (date, 5, 2)
;sday   = STRMID (date, 8, 2)

year    = headib [0]
month   = headib [1]
day     = headib [2]

IF (year LT 1900) THEN year = year + 1900

syear   = STRTRIM (STRING (year,  FORMAT='(I4)'), 2)
smonth  = STRTRIM (STRING (month, FORMAT='(I2)'), 2)
sday    = STRTRIM (STRING (day,   FORMAT='(I2)'), 2)

IF (month LT 10) THEN smonth = '0' + smonth
IF (day   LT 10) THEN sday   = '0' + sday

date = syear + '-' + smonth + '-' + sday

PRINT, 'syear: ', syear, ' smonth: ', smonth, ' sday: ', sday

plotfile = syear + smonth + sday + '.mk4.cpb.plot.ps'

PRINT, numaz, numpix, pixscale, pixoff


;--- Heights from header of first file.

height = FINDGEN (numpix) * pixscale + pixoff

inimg  = INTARR (NUMAZ, NUMPIX)
avgimg = FLTARR (NUMAZ, NUMPIX)

CLOSE, 2
OPENR, 2, file_list

;--- Determine number of images to average.

numimg = 0
WHILE ( NOT EOF (2) ) DO BEGIN
    numimg = numimg + 1
    READF, 2, filename
ENDWHILE

print, 'numimg: ', numimg

rays = FLTARR (numimg, NUMPIX)

;--- Open list file.

CLOSE, 2
OPENR, 2, file_list

WINDOW, 0, xs=numaz, ys=numpix
WINDOW, 1, xs=numaz, ys=numpix
imgnum = 0

;--- File loop.

WHILE ( NOT EOF (2) ) DO BEGIN
    READF, 2, filename
    filename = STRTRIM (filename, 2)
    PRINT, filename

    CLOSE, 1
    OPENR, 1, filename
    READU, 1, headib
    READU, 1, headf
    READU, 1, headie
    READU, 1, headpad

    READU, 1, inimg

    scale   = headf [13]			; data_scale
    offset  = headf [14]			; data_offset
    hour    = headib [3]			; image hour
    minute  = headib [4]			; image minute
    shour   = STRTRIM (STRING (hour),   2)
    sminute = STRTRIM (STRING (minute), 2)
    if (hour LT 10)   then shour   = '0' + shour
    if (minute LT 10) then sminute = '0' + sminute
    imgtime = shour + sminute

    ;--- Read FITS image.

;    inimg = readfits (filename, hdu, /noscale)
;    hdr   = readfits (filename, ext, /noscale, exten_no=1)

;    itime = fxpar (hdu, 'TIME-OBS')
;    imgtime   = STRMID (itime, 0, 5)
;    scale   = fxpar (ext, 'DATASCL')		; data_scale
;    offset  = fxpar (ext, 'DATAOFF')		; data_offset

    ;--- Apply photometric scaling.
    ;--- Accumulate image sum.

    img = (FLOAT (inimg) - offset) / scale 
    avgimg = avgimg + img
    rays [imgnum, *] = img [azplace, *] 

    imgnum = imgnum + 1

    ;--- Display image & average image.

    WSET, 0
    TV, BYTSCL (ALOG10 (img + 2.e-9), -9, -5)
    WSET, 1
    TV, BYTSCL (ALOG10 ((avgimg / imgnum) + 2.e-9), -9, -5)

    ;--- Add image time to extension header.

    inum1 = (imgnum - 1) / 4
    inum2 = (imgnum - 1) - inum1 * 4
;    print, 'imgnum/inum1/inum2: ', imgnum, inum1, inum2, ' imgtime: ', imgtime

    CASE inum1 OF
    0: BEGIN						$
       avgtime1 = avgtime1 + imgtime + ' '
       PRINT, 'avgtime1: ', avgtime1
;       fxaddpar, ext0, 'AVGTIME1', avgtime1, cavgtime1
       END
    1: BEGIN						$
       avgtime2 = avgtime2 + imgtime + ' '
       PRINT, 'avgtime2: ', avgtime2
;       fxaddpar, ext0, 'AVGTIME2', avgtime2, cavgtime2
       END
    2: BEGIN						$
       avgtime3 = avgtime3 + imgtime + ' '
       PRINT, 'avgtime3: ', avgtime3
;       fxaddpar, ext0, 'AVGTIME3', avgtime3, cavgtime3
       END
    3: BEGIN						$
       avgtime4 = avgtime4 + imgtime + ' '
       PRINT, 'avgtime4: ', avgtime4
;       fxaddpar, ext0, 'AVGTIME4', avgtime4, cavgtime4
       END
    4: BEGIN						$
       avgtime5 = avgtime5 + imgtime + ' '
       PRINT, 'avgtime5: ', avgtime5
;       fxaddpar, ext0, 'AVGTIME5', avgtime5, cavgtime5
       END
    5: BEGIN						$
       avgtime6 = avgtime6 + imgtime + ' '
       PRINT, 'avgtime6: ', avgtime6
;       fxaddpar, ext0, 'AVGTIME6', avgtime6, cavgtime6
       END
    6: BEGIN						$
       avgtime7 = avgtime7 + imgtime + ' '
       PRINT, 'avgtime7: ', avgtime7
;       fxaddpar, ext0, 'AVGTIME7', avgtime7, cavgtime7
       END
    7: BEGIN						$
       avgtime8 = avgtime8 + imgtime + ' '
       PRINT, 'avgtime8: ', avgtime8
;       fxaddpar, ext0, 'AVGTIME8', avgtime8, cavgtime8
       END

    ELSE: PRINT, 'imgnum: ', imgnum, ' greater than maximum allowed'
    ENDCASE

ENDWHILE

;--- Compute average image.

avgimg = avgimg / numimg 
TV, BYTSCL (ALOG10 (avgimg + 2.e-9), -9, -5)

;--- Write average image to disk NATIVE format.

headie [2] = is_cal_avg

avgtime = avgtime1 + avgtime2 + avgtime3 + avgtime4 + avgtime5 + avgtime6 $
	+ avgtime7 + avgtime8

print, 'avgtime: '
print, avgtime

;WHILE (STRLEN (avgtime) LT 160) DO avgtime = avgtime + ' '

;lenavgtime = STRLEN (avgtime)
;PRINT, 'lenavgtime: ', lenavgtime

;bavgtime = BYTARR (lenavgtime)
;READS, avgtime, bavgtime, format='(160I1)'
;print, 'bavgtime: '
;print, bavgtime

;headpad [192:351] = bavgtime


;avgchar = ''
;FOR i = 0, lenavgtime DO	$
;BEGIN
;    avgchar = STRMID (avgtime, i, 1) 
;    READS, avgchar, headpad [192+i]
;    headpad [192 + i] = FIX (avgchar)
;    PRINT, 'i: ', i, ' avgchar: ', avgchar
;END


;headpad [192:211] = avgtime1
;headpad [212:231] = avgtime2
;headpad [232:251] = avgtime3
;headpad [252:271] = avgtime4
;headpad [272:291] = avgtime5
;headpad [292:311] = avgtime6
;headpad [312:331] = avgtime7
;headpad [332:351] = avgtime8

outfile = STRMID (filename, 0, 6) + '.mk4.cpb.avg'

CLOSE, 1
OPENW, 1, outfile
    WRITEU, 1, headib
    WRITEU, 1, headf
    WRITEU, 1, headie
    WRITEU, 1, headpad
    WRITEU, 1, FIX (avgimg * scale + offset)
CLOSE, 1

;fxaddpar, ext0, 'PROCSTEP', is_cal_avg			; ProcSteps
;outfile = STRMID (filename, 0, 6) + '.mk4.cpb.avg.fts'
;writefits, outfile, FIX (avgimg * scale + offset), hdu0
;writefits, outfile, hdr0, ext0, /append


WINDOW, 2, xs=512, ys=512

; What is mean and std dev for each height?
; An indicator of coronal structure.

avgheight      = FLTARR (numaz)
avgheightplus  = FLTARR (numaz)
avgheightminus = FLTARR (numaz)
avgheightmax   = FLTARR (numaz)
avgheightmin   = FLTARR (numaz)

FOR ii = 0, numpix-1 DO BEGIN
    res = MOMENT (avgimg [*, ii], sdev=derf)
    avgheight [ii]      = res [0]
    avgheightplus [ii]  = res [0] + derf
    avgheightminus [ii] = res [0] - derf
    avgheightmax [ii]   = max (avgimg [*, ii])
    avgheightmin [ii]   = min (avgimg [*, ii])
ENDFOR

CLOSE,  1
OPENW,  1, 'corona_model'
WRITEU, 1, avgheight
CLOSE,  1

;writefits, 'corona_model', avgheight


;--- Create azimuthally binned data.
;    Again, this will work on average image of the day.

binsize = 20
binimg = FLTARR (numaz / binsize, numpix)

FOR jj = 0, numaz-1 DO BEGIN
	kk = jj / binsize
	binimg [kk, *] = binimg [kk, *] + img [jj, *]
ENDFOR

FOR kk = 0, numaz/binsize - 1 DO binimg [kk, *] = binimg [kk, *] / binsize
	

;--- Write plots to a Postscript file.

PRINT, "Set plot device"
SET_PLOT, 'PS'
DEVICE, filename = plotfile

;STOP

!p.title    = 'Average pB vs. Height' + '   Mk4 ' + date
plot_oo, height, avgheight
OPLOT, height, avgheightplus, psym=3
OPLOT, height, avgheightminus, psym=3

;OPLOT, height, avgheightmax, psym=3
;OPLOT, height, avgheightmin, psym=3

;STOP

;--- What is the stdev for each height over the day?
;
; This requires all the images going into the average
; image of the day to get RMS at each height
; over the day.

daystdev  = FLTARR (NUMPIX)
daymean   = FLTARR (NUMPIX)
polystdev = FLTARR (NUMPIX)
imgnum    = FINDGEN (numimg)

FOR jj = 0, numpix-1 DO BEGIN
	res  = MOMENT (rays [*, jj], sdev=derf)
	coef = POLY_FIT (imgnum, rays [*, jj], 2, yfit, yband, sigma)
	daystdev  [jj] = derf
	daymean   [jj] = res [0]
	polystdev [jj] = sigma
ENDFOR

;!p.title = 'Mean and Standard Deviation at One Azimuth'
!p.title = 'Mean & Std Deviation at One Azimuth' + '   Mk4 ' + date
!y.title = 'pB'

plot_oo, height, daymean
OPLOT, height, daystdev, psym=3
OPLOT, height, polystdev, psym=1
;STOP

;!p.title = 'pB:  10 degree binned azimuths'
!p.title = 'pB: 10 degree binned azimuths' + '   Mk4 ' + date

plot_oo, height, binimg [0, *], psym=3
FOR kk = 1, binsize - 1 DO BEGIN
	OPLOT, height, binimg [kk, *], psym=3
ENDFOR

DEVICE, /close
SET_PLOT, 'X'

END

