;+
; NAME	mk4synlong3.pro
;
; PURPOSE	Longitude scan of Mk4 synoptic map.
;
; SYNTAX	mk4synlong3, mk4_synoptic_file latitude_index
;
;		mk4_synoptic_file	binary file 360x181 shorts, e or w
;
;		mk4synlong3, '1943e1.25'
;		
; AUTHOR	Andrew L. Stanger   HAO/NCAR    4 Nov 2002
;-

PRO mk4synlong3, mapfile

map = INTARR (360, 181)

CLOSE, 1
OPENR, 1, mapfile
READU, 1, map
CLOSE, 1

map = (map * 1.0e-10) - 1.0e-7			; Convert to pB Bsun units.

plotfile = mapfile + '.plot.ps'
xlab   = 'Longitude'
ylab   = 'pB [Bsun]'

ymin = 0.0
ymax = MAX (map)

SET_PLOT, 'PS'
DEVICE, file=plotfile

lat = -30
ilat = lat + 90
slat = STRING (lat, FORMAT='(i3)')
ptitle = 'Mk4 C.R.' + mapfile + ' Latitude ' + slat

PLOT, map [*, ilat],		$
      title=ptitle,		$
      xtitle=xlab,		$
      ytitle=ylab,		$
      yrange = [ymin, ymax]

lat = 0
ilat = lat + 90
slat = STRING (lat, FORMAT='(i3)')
ptitle = 'Mk4 C.R.' + mapfile + ' Latitude ' + slat

PLOT, map [*, ilat],		$
      title=ptitle,		$
      xtitle=xlab,		$
      ytitle=ylab,		$
      yrange = [ymin, ymax]

lat = 30
ilat = lat + 90
slat = STRING (lat, FORMAT='(i3)')
ptitle = 'Mk4 C.R.' + mapfile + ' Latitude ' + slat

PLOT, map [*, ilat],		$
      title=ptitle,		$
      xtitle=xlab,		$
      ytitle=ylab,		$
      yrange = [ymin, ymax]

DEVICE, /CLOSE
SET_PLOT, 'X'
END
