;+
; NAME	tvmovie.pro
;
; PURPOSE	Animate a sequence of images using the TV command.
;
; AUTHOR	Atle Borsholm   Technical Support Engineer RSI   8 Sep 2003
;		
;-

PRO tvmovie_event, ev

  ;{

  ; --- Get the information structure.

  WIDGET_CONTROL, ev.top, get_uvalue=s

  ; --- What kind of event is this ?

  evname = TAG_NAMES (ev, /STRUCTURE_NAME)

  ; --- Code for button events.

  IF (evname EQ 'WIDGET_BUTTON') THEN	$
  BEGIN ;{
    CASE ev.id OF ;{

      ; --- START button.

      s.start: BEGIN ;{

	; --- If stop flag is 0, then diusplay next frame.

	IF (s.stopflag EQ 0) THEN BEGIN ;{
	  WSET, s.win
          TV, (*s.img3d) [*, *, *s.n]
        ENDIF ;}

        ; --- Update frame number.

        *s.n = (*s.n + 1) MOD s.nframes

	; --- Set a time event to get back into this event handler.

	WIDGET_CONTROL, ev.id, TIMER=0

	; --- Update stop flag.

	s.stopflag = 0
      END ;} of START button case.

      ; --- RESTART button.

      s.restart: BEGIN ;{

        ; --- If stop flag is 0, then display next frame.

        IF (s.stopflag EQ 0) THEN BEGIN ;{
	  WSET, s.win
          TV, (*s.img3d) [*, *, *s.n]
        ENDIF ;}

	; --- Update the frame number.

        *s.n = (*s.n + 1) MOD s.nframes

      END ;} of RESTART button case.

      ; --- STOP button.

      s.stop: BEGIN ;{
        s.stopflag = 1
        *s.n = *s.n - 1				; Subtract a frame.
      END ;} of STOP button case.

      ; --- QUIT button.

      s.quit: WIDGET_CONTROL, ev.top, /DESTROY

    ENDCASE ;}

  ENDIF ;} of BUTTON code.

  ; --- Code for TIMER events.

  IF (evname EQ 'WIDGET_TIMER') THEN	$
  BEGIN ;{
    WIDGET_CONTROL, ev.top, timer=1d0/(*s.fps)

    IF (s.stopflag EQ 0) THEN BEGIN ;{
      WSET, s.win
      IF (s.true) THEN DEVICE, decomposed=0

      ; --- Delay for a moment.

      WAIT, s.delay

      ; --- Display the image.

      TV, (*s.img3d) [*, *, *s.n]

      ; --- Update the frame number.

      *s.n = (*s.n + 1) MOD s.nframes

      ; --- Set a timer event to get back into this event handler.

      IF (s.stopflag EQ 0) THEN WIDGET_CONTROL, ev.id, TIMER=0

    END ;} of STOP flag.

  ENDIF ;} of WIDGET_TIMER code.

  ; --- Put information structure back.

  WIDGET_CONTROL, ev.top, set_uvalue=s, /NO_COPY

;  WIDGET_CONTROL, ev.top, timer=1d0/(*s.fps)
;  WSET, s.win
;  TV, (*s.img3d) [*, *, *s.n]
;  *s.n = (*s.n + 1) MOD s.nframes

END ;}

; ------------------------------------------------------------------------------

PRO ct_event, ev

;  LOADCT, ev.index, /silent

  XCOLORS, Group=ev.top, Title='xcolors', NotifyID=[ev.id, ev.top], /drag

END

; ------------------------------------------------------------------------------

PRO speed_event, ev

  WIDGET_CONTROL, ev.top, get_uvalue=s
  *s.fps=ev.value

END

; ------------------------------------------------------------------------------

PRO tvmovie_cleanup, tlb

  WIDGET_CONTROL, tlb, get_uvalue=s
  HEAP_FREE, s

END

; ------------------------------------------------------------------------------

PRO tvmovie, img3d, no_copy=no_copy

  compile_opt idl2

  ; --- 8-bit color or something else ?

  WINDOW, xsize=10, ysize=10, /free
  WDELETE, !D.WINDOW
  IF (!D.N_COLORS GE 256) THEN	$
  BEGIN
    true = 1
    DEVICE, decomposed=0
  END				$
  ELSE				$
    true = 0

;  LOADCT, 0, get_names=ct

  IF (N_ELEMENTS (img3d) eq 0) THEN $
  BEGIN
    PRINT, '3D Data cube must be supplied to tvmovie.'
    RETURN
  END

  title = 'tvmovie'

  dim    = SIZE (img3d, /dim)
  xdim   = dim [0]
  ydim   = dim [1]
  zdim   = dim [2]
  print, 'xdim/ydim/zdim: ', xdim, ydim, zdim

  ; --- Create the widgets.

  tlb     = WIDGET_BASE   (COLUMN=2, TITLE=title)

  bbase   = WIDGET_BASE   (tlb, ROW=6)
  ibase   = WIDGET_BASE   (tlb)

  start   = WIDGET_BUTTON (bbase, value='Start Animation')
  restart = WIDGET_BUTTON (bbase, value='Restart Animation')
  stop    = WIDGET_BUTTON (bbase, value='Stop Animation')
  
  colors  = WIDGET_BUTTON (bbase, value='Modify Color Table',	$
				  event_pro='ct_event')
  quit    = WIDGET_BUTTON (bbase, value='Exit Program')
  delayid = WIDGET_SLIDER (bbase, min=5, max=50, value=30, $
			          event_pro='speed_event')  ; frames per second
  drawid  = WIDGET_DRAW   (ibase, xsize=xdim, ysize=ydim)

  ; --- Realize widgets.

  WIDGET_CONTROL, tlb,  /realize

  WIDGET_CONTROL, drawid, get_value=win
  s = { img3d:ptr_new (img3d, no_copy=co_copy),	$ ;
	n:ptr_new (0),		$ ; Current animation frame.
	xsize:xdim,		$ ; X-axis dimension of image.
	ysize:ydim,		$ ; y-axis dimension of image.
	nframes:zdim,		$ ; Number of animation frames.
	start:start,		$ ; ID of START button.
	restart:restart,	$ ; ID of RESTART button.
	stop:stop,		$ ; ID of STOP button.
	quit:quit,		$ ; ID of QUIT button.
	win:win,		$ ; ID of graphics window.
	drawid:drawid,		$ ; ID of draw widget.
	true:true,		$ ; Truecolor [0=no, 1-yes].
	delay:0.25,		$ ; Delay time (sec) between frames
	time:systime (1),	$ ;
	fps:ptr_new (30),	$ ;
	stopflag:0}		  ; Stop animation flag.


;  WIDGET_CONTROL, ibase,  set_uvalue= $
;                          {img3d:ptr_new(img3d,no_copy=no_copy), n:ptr_new(0),$
;                           time:systime(1), win:win, dim:dim, fps:ptr_new(30)}

  WIDGET_CONTROL, tlb, set_uvalue=s, /no_copy
  WIDGET_CONTROL, ibase, timer=0.1

  XMANAGER, 'tvmovie', tlb,  /no_block, cleanup='tvmovie_cleanup'

END

; ------------------------------------------------------------------------------

