; Program to break up the Spartan files (segmented) into separate files
; for ease of loading into IDL and analysis. All data is time tagged via the
; MET (Mission Elapsed Time).
;
;
;	Author: Greg Ucker, NCAR/HAO
;	Date:	20 Oct 1993
;
; MODIFICATIONS: 
;
;
; define preambles
print,' '
print,'NOTE: this must be run from the flight data sp201-n directory'
print,'		with the tape data in sp201-n/tapes & a sp201-n/dbase
print,'		directory set up as the repository'
print,'	Any other place will result in an error...'
print,' '


fmtloc = '/local/opt/projects/spartan/programs/'
dbloc = 'dbase/'
tapeloc = 'tapes/'

spyr = 0
read,'Enter Year of Flight (eq 93) ',spyr

spyr = strtrim(spyr,2)

; for use in the spgetfmt procedure:
; list of the files containing the format specifications and the 
; items to be stripped, file names to contain the stripped items
fmt_defiles = [fmtloc+'sptmfmt'+spyr+'.lis', fmtloc+'spimfmt'+spyr+'.lis', $
		fmtloc+'spmsfmt'+spyr+'.lis']

first_sync = 1b

repeat begin
	apans = ''
	read,'Is data to be appended to existant files (Y/N): ',apans
	apans=strupcase(strtrim(apans,2))
endrep until((apans eq 'Y') or (apans eq 'N'))

; get the file containing the list of raw files to be parsed
iflist = ''
get_lun, iflun
on_ioerror, FILE_NOT_FOUND

if 0 then begin
	FILE_NOT_FOUND:
	print,' '
	print,'File not found....
endif

read,'Enter name of file containing the raw file list: ',iflist
iflist = strtrim(iflist,2)
openr,iflun,iflist

on_ioerror,NULL


; get the formats & number of them, and the items to be individually
; stripped from the each of the fmts from the definiton files
spgetfmt, fmt_defiles, fmt

; if update then append new data, else create new files
if apans eq 'N' then spcreateouts, fmt, dbloc

; set up the parsing buffers
spmakebufrs, fmt, ibuf, obuf, dbuf, ifstat

; now time to start ingesting data from the files
istreamf = ''
get_lun,ilun
get_lun,olun
nbzeros = 0L

while not eof(iflun) do begin

;	get the input stream file name
	on_ioerror, IFILE_NOT_FOUND
	readf,iflun,istreamf
	readf,iflun,nbzeros
	istreamf = tapeloc+strtrim(istreamf,2)
	openr,ilun,istreamf
	if not eof(ilun) then ifstat.eofflag = 0L
	if 0 then begin
		IFILE_NOT_FOUND:
		print,' '
		print,'SPARTAN_PARSE: Input stream file not found....'
		print,'	filename: '+istreamf
		print,'		Procedure exiting to main level...'
		print,' '
		close,/all
		free_lun,iflun

		; clean up the remaining data at the end of the 'mid-parsing'
		if (ibuf.ibvalid-ibuf.ibpos) gt 0L then begin
		case strtrim(strupcase(!version.os),2) of
			'SUNOS': 	openw,ilun,tapeloc+'spartan_tmp.dat'
			'VMS':	openw,ilun,tapeloc+'spartan_tmp.dat',/stream
			else:		openw,ilun,tapeloc+'spartan_tmp.dat'
		endcase

		writeu,ilun,ibuf.ib(ibuf.ibpos:ibuf.ibvalid-1L)
		close,ilun
	print,'SPARTAN_PARSE: Writing spartan_tmp.dat file to hold data'
	print,'		not ingested into the database... remember to make'
	print,'		this the 1st filename in the master list for the'
	print,'		next round of ingest.'
		endif	; cleanup
		
		free_lun,ilun
	endif

	print,' '
	print,'SPARTAN_PARSE: Ingesting input data from: '+strtrim(istreamf,2)
	print,' '

	; get the data piped from the istream file into ibuf.ib
	; until one of the obuf.dat buffers is full or until the 
	; eof of istream file is found, then write the extracted data into
	; the separate files specified by the fmt
	open_newfile = 0b
	nothings_left = 0b

	while not(open_newfile or nothings_left) do begin

		; if needed, read in a chunk of data from the input file
		if ((ibuf.tbrd gt 0L) and not(ifstat.eofflag)) then begin
			print,'SPARTAN_PARSE:	Reading input data...'
			spread, ilun, ibuf, ifstat, nbzeros
		endif

		; finding sync and parsing into the obuf(i).dat
		spdecom, fmt, ibuf, obuf, dbuf, first_sync

		; check if the output buffer is full or if eof encountered
		; if so then write it out and flush the output buffer

		full_obuf = total(obuf(*).nmax eq obuf(*).ndat) gt 0

		no_files = (eof(iflun) and ifstat.eofflag)

		if (ibuf.syncfnd ge 0L) then begin

			nothings_left = (no_files and $
		((ibuf.ibvalid-ibuf.ibpos) lt fmt(ibuf.syncfnd).flen))

			open_newfile = (ifstat.eofflag and $
		((ibuf.ibvalid-ibuf.ibpos) lt fmt(ibuf.syncfnd).flen))
		
		endif else begin

			nothings_left = (no_files and $
		((ibuf.ibvalid-ibuf.ibpos) lt min(fmt(*).flen)))

			open_newfile = (ifstat.eofflag and $
		((ibuf.ibvalid-ibuf.ibpos) lt min(fmt(*).flen)))
		endelse

		if (full_obuf or nothings_left) then begin
			print,'SPARTAN_PARSE:	Writing parsed data...'
			spwrite, fmt, obuf, olun
		endif

	endwhile

	close,ilun

endwhile

; clean up the remaining data at the end of the 'mid-parsing'
if ((ibuf.ibvalid - ibuf.ibpos) gt 0L) then begin

	case strtrim(strupcase(!version.os),2) of
		'SUNOS': 	openw,ilun,'spartan_tmp.dat'
		'VMS':		openw,ilun,'spartan_tmp.dat',/stream
		else:		openw,ilun,'spartan_tmp.dat'
	endcase

	writeu,ilun,ibuf.ib(ibuf.ibpos:ibuf.ibvalid-1L)
	close,ilun
	print,'SPARTAN_PARSE: Writing spartan_tmp.dat file to hold data'
	print,'		not ingested into the database... remember to make'
	print,'		this the 1st filename in the master list for the'
	print,'		next round of ingest.'
endif	; cleanup

close,iflun
free_lun,iflun
free_lun,ilun

end

