;+
; NAME		spdisp.pro
;
; PURPOSE	Display Spartan sp201 image.
;
; SYNTAX	spdisp, 'image.fits', xdim_prev, ydim_prev, /gif
;
; EXAMPLES	spdisp, 'image.fts'
;		spdisp, 'image.fts', xdim_prev, ydim_prev
;		spdisp, 'image.fts', xdim_prev, ydim_prev, /gif
;
; EXTERNAL
; PROCEDURES	spartan_annotate.pro
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   11 September 2001
;		20 Sep 2001: Add wmax keyword
;-

PRO spdisp, fits_name, xdim_prev, ydim_prev, gif=gif, wmax=wmax

   xb = 160		; X-axis border for annotation.
   yb =  88		; Y-axis border for annotation.

   ftspos = STRPOS (fits_name, '.fts')
   basename = STRMID (fits_name, 0, ftspos)

;   print, 'basename: ', basename

   img = readfits (fits_name, hdu)

   ;--- Extract information from header.

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')

   xdim2 = xdim * 2
   ydim2 = ydim * 2

   ;--- "Erase" left annotation area.

   leftborder = BYTARR (xb, ydim2 + yb)
   leftborder [*, *] = 0
   TV, leftborder

;   window, xsize=xdim2, ys=ydim2

   ;--- Resize window if it has changed.

   if (xdim NE xdim_prev OR ydim NE ydim_prev)  THEN	$
       window, xsize=xdim2+xb, ys=ydim2+yb

;   print, 'xdim2 + xb: ', xdim2 + xb
;   print, 'ydim2 + yb: ', ydim2 + yb

   xdim_prev = xdim
   ydim_prev = ydim

   ;--- Annotate image.

   spartan_annotate, hdu, xdim2, ydim2, xb, yb

   ;--- Get information from FITS header.

   orbit_id = fxpar (hdu, 'ORBIT-ID')
   image_id = fxpar (hdu, 'IMAGE-ID')
   telescop = fxpar (hdu, 'TELESCOP')
   instrume = fxpar (hdu, 'INSTRUME')
   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')
   dispmin  = fxpar (hdu, 'DISPMIN')
   dispmax  = fxpar (hdu, 'DISPMAX')

   telescop = STRTRIM (telescop, 2)
   IF (telescop NE 'SPARTAN 201' AND instrume NE 'WLC')  THEN         $
   BEGIN
      PRINT, 'TELESCOP: ', telescop, ' should be "SPARTAN 201".'
      PRINT, 'INSTRUME: ', instrume, ' should be "WLC".'
      RETURN
   END

   ;--- Expand image (double its size).

   img2 = rebin (img, xdim2, ydim2)

   ;--- Display image.

   dmin = datamin
   dmax = datamax
   dmin = dispmin
   dmax = dispmax
   IF (KEYWORD_SET (wmax))  THEN dmax = wmax

   TV, BYTSCL (img2, min=dmin, max=dmax, top=249), xb, yb

   ;--- Write displayed image as a GIF file (if "gif" keyword is set).

   IF (KEYWORD_SET (gif))  THEN		$
   BEGIN  ;{
      gif_file = basename + '.gif'
      img_gif = TVRD ()
      WRITE_GIF, gif_file, img_gif, red, green, blue
   ENDIF  ;}

   RETURN

END

