;+
; NAME		sv.pro
;
; PURPOSE	Display Spartan sp201 image.
;
; SYNTAX	sv, img, hdu
;
; EXAMPLE	sv, img, hdu
;
; HISTORY	Andrew L. Stanger   HAO/NCAR   11 September 2001
;-

PRO sv, img, hdu

   xb = 160 
   yb =  88
   xdim_prev = 0
   ydim_prev = 0

   ;--- Extract information from header.

   xdim     = fxpar (hdu, 'NAXIS1')
   ydim     = fxpar (hdu, 'NAXIS2')

   xdim2 = xdim * 2
   ydim2 = ydim * 2

;   window, xsize=xdim2, ys=ydim2

   if (xdim NE xdim_prev OR ydim NE ydim_prev)  THEN	$
       window, xsize=xdim2+xb, ys=ydim2+yb

   print, 'xdim2 + xb: ', xdim2 + xb
   print, 'ydim2 + yb: ', ydim2 + yb

   xdim_prev = xdim
   ydim_prev = ydim

   spartan_annotate, hdu, xdim2, ydim2, xb, yb

   orbit_id = fxpar (hdu, 'ORBIT-ID')
   image_id = fxpar (hdu, 'IMAGE-ID')
   telescop = fxpar (hdu, 'TELESCOP')
   instrume = fxpar (hdu, 'INSTRUME')
   dateobs  = fxpar (hdu, 'DATE-OBS')
   timeobs  = fxpar (hdu, 'TIME-OBS')
   type_obs = fxpar (hdu, 'TYPE-OBS')
   dataform = fxpar (hdu, 'DATAFORM')
   rsun     = fxpar (hdu, 'RSUN')
   srsun    = STRING (rsun, FORMAT='(F7.2)')
   bunit    = fxpar (hdu, 'BUNIT')
   datamin  = fxpar (hdu, 'DATAMIN')
   datamax  = fxpar (hdu, 'DATAMAX')

   telescop = STRTRIM (telescop, 2)
      IF (telescop NE 'SPARTAN 201' AND instrume NE 'WLC')  THEN         $
      BEGIN
       PRINT, 'TELESCOP: ', telescop, ' should be "SPARTAN 201".'
       PRINT, 'INSTRUME: ', instrume, ' should be "WLC".'
       RETURN
      END

   img2 = rebin (img, xdim2, ydim2)
   tv, bytscl (img2, min=datamin, max=datamax, top=249), xb, yb
;   tv, bytscl (img2, min=datamin, max=datamax, top=249)

   RETURN

END

