pro  heliolm, listfile, soho=soho, stepsize=stepsize, lat=lat,	$
	      rmax=rmax, minpix=minpix, bkg=bkg

; IDL> helio, 'list.helium', step=0.2d

; PROGRAM ASSUMES THAT THE IMAGES ARE ALREADY ROTATED FOR P ANGLE
; PROGRAM ASSUMES THAT HEADER COORDINATES ARE CORRECT, DOES NOT CHECK THEM 


; new March 27, 2003:  adj_lon,  step default set to 1.0,  cmlong,
; adjustment, function:nearest_gridcell, 

T = SYSTIME (1)

IF (NOT KEYWORD_SET (rmax))	THEN rmax   = 0.99
IF (NOT KEYWORD_SET (minpix))	THEN minpix = 3L
IF (NOT KEYWORD_SET (bkg))	THEN bkg    = 0.0

minpix = LONG (minpix)

;---------------------
; Find if soho or not.
;---------------------
if not keyword_set(soho) then soho = 0

;-------------------
; Use sine latitude?
;-------------------
;sinlat = 1 ; 0 false, 1 true

;---------------------
; 1st longitude value.
;---------------------
lonstart = -90.0D

;---------------------------------
; Grid step size 0, 0.2, 0.4, ...
;---------------------------------
;if not keyword_set (stepsize) then stepsize = 0.2d0
if not keyword_set (stepsize) then stepsize = 1.0d0

;------------------------------------
; Find number of images in file list.
;------------------------------------
file = ' '
num = 0l

openr, lun, listfile, /get_lun
while (not eof (lun))  do begin 
       readf, lun, file
       num = num + 1
endwhile                       

	
;-----------------------------------
; Define new arrays in for stepsize.
;-----------------------------------
newsize = FIX (180.d0 / double (stepsize) + 0.5)

new_image = fltarr (newsize,newsize)

;--------------------------------------------------------
; This is in equal steps of stepsize degrees in longitude
;--------------------------------------------------------
genarr = dindgen (newsize, newsize) mod (newsize)

;------------------------------------------------
; Use double (stepsize) * 0.5d0 to get cell centers.
; Use lonstart to shift it -90.0 degrees.
;------------------------------------------------
newlon = genarr * double (stepsize) + lonStart + double (stepsize) * 0.5d0


;twoDlon = dindgen (newsize)
;testlon = testarr * double (stepsize) + lonStart + double (stepsize) * 0.5d0
;twoDlon = twoDlon + 1.
;twoDlon = testlon * twoDlon <-- get same as newlon

;stop

;--------------------
; Convert to radians - keep in degrees to add adjustment later!!
;--------------------     
;newlon = temporary (newlon) / !radeg

;---------------
; Set latitude.
;---------------
if keyword_set (lat) then begin

    ;--------------------------------------------------------------------
    ; Define latitude in equal steps in latitude if keywork is set to lat.
    ; old equation : newlat= transpose(dindgen(901,901)mod(901)/5. ) -90
    ;--------------------------------------------------------------------
    ;newlat = transpose (genarr * double (step)) - 90.d0 + double (step) * 0.5d0
    ;newlat=  temporary (newlat) / !radeg

    ;------------------------------------------------------------------
    ; This is in equal steps of stepsize degrees in latitude in radians.
    ;------------------------------------------------------------------
    newlat = transpose (newlon)
    newlat = temporary (newlat) / !radeg

endif else begin

    l = transpose (genarr - newsize * 0.5d0 + 0.5d0) / (newsize * 0.5d0) 
    newlat =  asin (l)

endelse ; sinlat 

   
;------------------
; Rewind file list.
;------------------
point_lun, lun, 0

file = ' '

;----------------------------------------------------
; Read images and convert to heliographic coordinates.
;----------------------------------------------------

FOR i = 0, num-1 do begin 

     readf, lun, file
     print, 'file = ', file

     image = readfits (file, hdu)
     image = float (image)

     ;---------------
     ; Read extension.
     ;---------------
     if (not soho)  then temp  = readfits (file, exten,/exten)

   
     xsize = (size (image))[1]
     ysize = (size (image))[2]

     if (xsize ne ysize) then print, '!!!!!!!! WARNING: IMAGE NOT SQUARE  !!!!!!!!!' 
 
     ;------------------------
     ; Read header information.
     ;------------------------

     get_date_time, soho, hdu, $ ; inputs
                    date, t_ave  ; output
  
     ;----------------
     ; Find julian day.
     ;----------------
     jd = anytim2jd (date)

     ;--------------------------------------------------
     ; Get carrington rotation # and central meridian L0.
     ;--------------------------------------------------
     jul_day = double (jd.int) + jd.frac
     jd_carr, jul_day, carr, l0

     ;-----------------------------------
     ; Find nearest center gridcell to L0.
     ;-----------------------------------
     cmlong = nearest_gridcell (L0, stepsize)

     adjustment = cmlong - L0
     print,'L0, cmlong, adjustment = ',L0, cmlong, adjustment

     ;-----------------------------
     ; Adjust longitude grid center.
     ; Convert to radians.
     ;-----------------------------
     adj_lon = newlon + adjustment
;stop
     adj_lon = temporary (adj_lon) / !radeg

     
     ;--------------------------------
     ; Get center of map, arcsec/pixel.
     ;--------------------------------
     xcen   =  fxpar (hdu, 'CRPIX1')      ; x center in pixel 
     ycen   =  fxpar (hdu, 'CRPIX2')      ; y center in pixel 
     dx     =  fxpar (hdu, 'CDELT1')      ; arcsec/pixel
     dy     =  fxpar (hdu, 'CDELT2')      ; arcsec/pixel

     ;--------------------------
     ; Find time of observations.
     ;--------------------------
     obs_time = utc2tai (anytim2utc (date))
     
     ;-----------------------------------------------------------------
     ; Find solar p,b angles and radius at time of the image using pb0r.
     ; pb0r is more reliable than the values in the header for MLSO !! 
     ;-----------------------------------------------------------------
     angles = pb0r(obs_time, soho=soho)
     
     p_ang = angles(0)
     b_ang = angles(1)
     sunr_min = angles(2) ; sun radius in arcmin

     ;-----------------------------------------------
     ; angles(2) = radius?, convert to arcsec, 
     ; divide by arcsec/pixel
     ; to get size of radius in terms of pixel size
     ;-----------------------------------------------
     radius_pix = sunr_min * 60.0d / float (dx)  ; about 420 pixels from center

     ;----------------------------
     ; Compute B angle in radians.
     ;----------------------------
     b0 = angles (1) / !radeg


     ;--------------------------------
     ; Compute sun's radius in radians.
     ;--------------------------------
     sunr = sunr_min / 60.0d0 / !radeg
      
     print, ' p_ang, b_ang, and radius_pix :  ' , p_ang, b_ang, radius_pix
 
     IF (not soho) then begin 

        solar_l0 = fxpar (hdu, 'SOLAR_L0')
        if (solar_l0 lt 0) then solar_l0 = solar_l0 + 360.
  
     ENDIF

     ;-----------------------------------------------
     ; Find pixels within and outside the solar disk.
     ;-----------------------------------------------
     disk_edge = 0.995D
     disk_edge = double (rmax)
     if (soho) then  disk_edge = 0.99D

     ;----------------------------------------------------------
     ; Define coordinates for interpolation on the disk =
     ; regular array centered at image center in units of radius.
     ;----------------------------------------------------------
     genarr = dindgen(xsize,ysize)mod(xsize)
    
     ;xx = genarr - xcen + 0.5
     ;yy = transpose (dindgen (ysize, xsize) mod (ysize) ) - ycen + 0.5

     ;---------------------------
     ; Don't add 0.5 : incorrect?
     ;---------------------------
     xx = genarr - xcen 
     yy = transpose (dindgen (ysize, xsize) mod (ysize) ) - ycen 

     xx = xx / radius_pix
     yy = yy / radius_pix

     ;---------------------------------------
     ; Set off-disk values on image to zero.
     ;---------------------------------------
     zero_offdisk, disk_edge, xx, yy, image


     ;---------------------------------------------------------------
     ; Transform sin(latitude) (newlat) and longitude (newlon) in 
     ; new x and y coordinates on the disk.
     ; First find 3D cartesian coordinates.
     ;---------------------------------------------------------------     
     newzz  = sin (newlat)
     newyy  = cos (newlat) * sin (adj_lon)
     newxx  = cos (newlat) * cos (adj_lon)
     
     ;--------------------------------------------------------------
     ; Transform sin(latitude) (newlat) and longitude (adj_lon) in 
     ; new x and y coordinates on the disk.
     ; Correct for B angle.
     ;--------------------------------------------------------------
     dd = 1.0 / tan (sunr)
     rr =  dd - ( COS (b0) * newxx + SIN (b0) * newzz )
     newx = atan ( newyy, rr)  ; returns the angle whose tangent is newyy/rr
     newy = atan ( (-SIN (b0) * newxx + COS (b0) * newzz), rr)

     ;----------------------------------
     ; Divide by sun's radius in radians
     ; if B0 = 0,
     ; 0.,-90. maps to 0.,-1.
     ; 0.,0.  maps to 0.,0.
     ; 0.,90. maps to 0.,1.
     ;----------------------------------
     newx = temporary (newx) / sunr
     newy = temporary (newy) / sunr

;stop
     ;--------------------------------------------------------------------------
     ; Interpolate original image onto the new coordinate grid, newx, newy,
     ; in equally spaced in latitude and longitude in stepsize degree increments
     ;--------------------------------------------------------------------------

     ; Andy's routine ==========================================================

      ;------------------------------------------------------------------
      ; Make integers LONG and FLOATing point arrays DOUBLE precision.
      ;------------------------------------------------------------------

      image      = FLOAT  (image)
      xsize      = LONG   (xsize)
      ysize      = LONG   (ysize)
      xx         = FLOAT  (xx)
      yy         = FLOAT  (yy)
      xcen       = FLOAT  (xcen)
      ycen       = FLOAT  (ycen)
      radius_pix = FLOAT  (radius_pix)
      newx       = FLOAT  (newx)
      newy       = FLOAT  (newy)
      newxsize   = LONG   (newsize)
      newysize   = LONG   (newsize)

      ;------------------------------------------------------------------
      ; Convert angle coordinates to pixel coordinates for interpolation.
      ;------------------------------------------------------------------
    
      xslope = (MAX (xx) - MIN (xx)) / (xsize - 1)
      yslope = (MAX (yy) - MIN (yy)) / (ysize - 1)
      xint = (newx - MIN (xx)) / xslope
      yint = (newy - MIN (yy)) / yslope

      xslope = FLOAT (xslope)
      yslope = FLOAT (yslope)
      xint   = FLOAT (xint)
      yint   = FLOAT (yint)

      PRINT, 'newxsize/newysize:', newxsize, newysize
      PRINT, 'rmax:             ', rmax
      PRINT, 'minpix:           ', minpix
      PRINT, 'bkg:              ', bkg
      PRINT, 'xslope/yslope: ', xslope, yslope

;      PRINT, ''
;      PRINT, 'xx [*, ysize/2]:'
;      PRINT, xx [*, ysize/2]
;      PRINT, ''
;      PRINT, 'newx [*, newysize/2]:'
;      PRINT, newx [*, newysize/2]
;      PRINT, ''
;      PRINT, 'xint [*, newysize/2]:'
;      PRINT, xint [*, newysize/2]
      PRINT, 'rmax:             ', rmax
      PRINT, 'minpix:           ', minpix
      PRINT, 'bkg:              ', bkg

      PRINT, ''
      PRINT, 'size (xint):      ', size (xint)
      PRINT, 'size (yint):      ', size (yint)
      PRINT, ''
      PRINT, 'xsize/ysize:      ', xsize, ysize
      PRINT, 'size (xx):        ', size (xx)
      PRINT, 'size (yy):        ', size (yy)
      PRINT, 'size (newx):      ', size (newx)
      PRINT, 'size (newy):      ', size (newy)
      PRINT, ''
      PRINT, 'size (image):     ', size (image)
      PRINT, 'size (xsize):     ', size (xsize)
      PRINT, 'size (ysize):     ', size (ysize)
      PRINT, 'size (xcen):      ', size (xcen)
      PRINT, 'size (ycen):      ', size (ycen)
      PRINT, 'size (radius_pix):', size (radius_pix)
      PRINT, 'size (rmax):      ', size (rmax)
      PRINT, 'size (new_image): ', size (new_image)
      PRINT, 'size (newx):      ', size (newx)
      PRINT, 'size (newy):      ', size (newy)
      PRINT, 'size (newxsize):  ', size (newxsize)
      PRINT, 'size (newysize):  ', size (newysize)
      PRINT, 'size (minpix):    ', size (minpix)
      PRINT, 'size (bkg):       ', size (bkg)

      ;------------------------------------------------------------------
      ; Call C-language routine to do 2D weighted interpolation.
      ;------------------------------------------------------------------

      cobj = 'fwint2d'
      IF (!version.os EQ 'linux') THEN				$
	  clib = '/home/stanger/lib/linux/libfwint2d.so'	$
      ELSE IF (!version.os EQ 'IRIX') THEN			$
	   clib = '/home/stanger/lib/sgi/libfwint2d.so'		$
      ELSE							$
	  BEGIN
	  PRINT, 'Only linux & IRIX are currently supported.'
	  RETURN
	  END

      iext = CALL_EXTERNAL (clib, cobj,					$
			    image, xsize, ysize, xcen, ycen,		$
			    radius_pix, Rmax,				$
			    new_image, xint, yint, newxsize, newxsize,	$
			    minpix, bkg)

;      PRINT, 'iext: ', iext
;      PRINT, 'size (new_image): ', size (new_image)

      imin = MIN (new_image, max=imax)
      PRINT, 'imin/imax: ', imin, imax
   
     ;------------------------------------------------------------------------
     ; Andy this is where I call the bilinear interpolation.
     ; This is a solarsoft routine that calls the IDL routine interpolate.pro
     ; I suspect interpolate.pro calls or is a c routine.
     ;------------------------------------------------------------------------   
     ;new_image = interp2d (image, xx, yy, newx, newy, missing=-9999.99)  

    ;------------------------------------------
    ; Trim data near the limb to avoid bad data.
    ; We shall use 0.99 or less.
    ; I m experimenting with bigger number.
    ;------------------------------------------
     new_edge = 0.994D
     new_edge = double (rmax)
     ;new_radius = sqrt ( newx ^ 2.0 + newy ^ 2.0 )
     ;off_disk_new = where( new_radius gt new_edge)
     ;new_image(off_disk_new) = 0

     zero_offdisk, new_edge, newx, newy, new_image
 
    ;-----------------------------------------------------------------
    ; Add keywords about carrington totations and time of observation.
    ; Find carrington number and longitude for central meridian.
    ;-----------------------------------------------------------------
  
      fxaddpar, hdu, 'COORDNAM', 'HELIOCENTRIC', $
		     ' SOLAR NORTH ALIGNED WITH +Y AXIS'
      fxaddpar, hdu, 'OBS_L0',  l0, ' SOLAR LONGITUDE L0' 
      fxaddpar, hdu, 'OBS_CR',  carr, ' CARRINGTON ROTATION'
      ;---------------------------------
      ; Write out adjusted center of map.
      ;---------------------------------
      fxaddpar, hdu, 'CMLONG',  cmlong, ' ADJUSTED CENTRAL MERIDIAN' 
      fxaddpar, hdu, 'T_REC', t_ave, 'AVERAGE TIME OF IMAGE'
      if (not soho) then $
         fxaddpar, hdu, 'SOLAR_L0',solar_l0, ' SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE'


   ;----------------------------------
   ; Write new square images in files.
   ;----------------------------------

   if (not soho) then begin

       ;flen = strlen(file)-3
       newfile = strmid (file, 0, strlen (file)- 3) + 'heliolm.fts'
       writefits, newfile, new_image, hdu
       writefits, newfile, temp , exten, /append

   endif else begin
   
       newfile = strmid (file, 0, 23) + 'heliolm.fts' 
       writefits, newfile, new_image, hdu

   endelse

ENDFOR

close, lun
free_lun, lun
close, /all

PRINT, SYSTIME (1) - T, 'Seconds'

END

