;+
; acos_hdu.pro
;
; PURPOSE
;	Create a FITS header,
;	given an ACOS binary file (including binary header).
;
; SYNTAX
;	acos_hdu, acos_image_array
;
; OUTPUT
;	FITS header.
;
;
;-

FUNCTION acos_hdu, acos_img

fxhmake, hdu, acos_img

fxaddpar, hdu, 'ORIGIN',   'MLSO'
fxaddpar, hdu, 'TELESCOP', 'LOWK'
fxaddpar, hdu, 'INSTRUME', 'CCD'
fxaddpar, hdu, 'CODE-OBS', 'KLB'
fxaddpar, hdu, 'TYPE-OBS', 'KCORONA'
fxaddpar, hdu, 'OBJECT',   'LIMB'
fxaddpar, hdu, 'TYPE-DP',  'BRIGHTNESS'
fxaddpar, hdu, 'WAVELEN',  656.28
fxaddpar, hdu, 'WAVEWID',  0.80
fxaddpar, hdu, 'CTYPE1',   'X'
fxaddpar, hdu, 'CRPIX1',   -99999.00
fxaddpar, hdu, 'CRVAL1',   0.0
fxaddpar, hdu, 'CDELT1,    2.29
fxaddpar, hdu, 'CTYPE2',   'Y'
fxaddpar, hdu, 'CRPIX2',   -99999.00
fxaddpar, hdu, 'CRVAL2',   0.0
fxaddpar, hdu, 'CDELT2,    2.29
fxaddpar, hdu, 'COORDNAM', 'GEOCENTRIC'
fxaddpar, hdu, 'CRRADIUS', 0.0
fxaddpar, hdu, 'RSUN',     0.0
fxaddpar, hdu, 'OCC-DISC', 
fxaddpar, hdu, 'SOLAR_P0', 
fxaddpar, hdu, 'SOLAR_B0', 
fxaddpar, hdu, 'SOLAR_L0', 
fxaddpar, hdu, 'SOLAR_RA', 
fxaddpar, hdu, 'SOLARDEC', 
fxaddpar, hdu, 'HOUR-ANG', 
fxaddpar, hdu, 'ELEV', 
fxaddpar, hdu, 'AZIMUTH',
fxaddpar, hdu, 'AIRMASS',
fxaddpar, hdu, 'GAIN-DET',
fxaddpar, hdu, 'NIMSUM',
fxaddpar, hdu, 'EXPDUR',
fxaddpar, hdu, 'BUNIT',
fxaddpar, hdu, 'BSCALE',
fxaddpar, hdu, 'BZERO',
fxaddpar, hdu, 'DATAMIN',
fxaddpar, hdu, 'DATAMAX',
fxaddpar, hdu, 'DISPMIN',
fxaddpar, hdu, 'DISPMAX',
fxaddpar, hdu, 'DISP-DIR', 'RU'
fxaddpar, hdu, 'VERSION', 'ACOS V1.1'
fxaddpar, hdu, 'DATE'
fxaddpar, hdu, 'DATE-CAL'
fxaddpar, hdu, 'COMMENT', 'THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND OF'
fxaddpar, hdu, 'HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),'
fxaddpar, hdu, 'A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARHC (NCAR),'
fxaddpar, hdu, 'UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).'
fxaddpar, hdu, 'THE ADVANCED CORONAL OBSERVING SYSTEM (ACOS) CONSISTS OF 3 SOLAR'
fxaddpar, hdu, 'INSTRUMENTS (ALL LOCATED AT MLSO):'
fxaddpar, hdu, '   DIGITAL PROMINENCE MONITOR (DPM), LOW K-CORONA POLARIMETER (LOWK),'
fxaddpar, hdu, '   & CHROMOSPHERIC HELIUM-I IMAGING PHOTOMETER (CHIP).'
fxaddpar, hdu, 'IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.'
fxaddpar, hdu, 'END'

RETURN, hdu
END
