;+
; NAME		carr_jd.pro
;
; PURPOSE	Convert Carrington Rotation to Julian Date.
;
; AUTHOR	Nick Arge   NOAA   20 Nov 2002
;-

PRO carr_jd, ca,lon

;Convert from Carrington Longitude and Latitude to (Julian Date -2440000)
;Modified Nov. 20 2002

carrot = 0.0D0
lonmax = 0.0D0

JD_low = 0.0D0

carrot = DOUBLE (ca)
lonmax = DOUBLE (lon)

IF (lonmax EQ 0.0D0) THEN BEGIN
  lonmax = 360.0D0
  carrot = carrot + 1.0D0
ENDIF

incl = 7.25D0

JD_DD = 0.0D0
Lo    = 0.0D0

JD_orig = 0.0D0

JD_DD = carrot + (360.0D0 - lonmax) / 360.0D0
JD_DD = JD_DD - 1750.0D0
JD_DD = JD_DD * 27.27530D0
JD_DD = JD_DD + 5871.91D0

JD_orig = JD_DD

JD_low = JD_DD + 3.0D0
JD_DD  = JD_DD - 3.0D0

; For each of the dates labeling the bottom x-axis of the 1st plot.

incr = DOUBLE (0.1)

FOR MM = LONG (0), 10000 DO BEGIN
   
  JD_DD = JD_DD + incr

  ;nn = JD_DD - 2451545.0
  ; 2440000 - 2451545 = 11545

  nn = JD_DD - DOUBLE (11545.0)
  LL = 280.472D0 + 0.9856474D0 * nn
  gg = 357.528D0 + 0.9856003D0 * nn
  sun = LL + 1.915D0 * SIN (gg * !dpi / 180.0D0) $
	   + 0.020D0 * SIN (2.0D0 * gg * !dpi / 180.0D0)

; Explains where the number 43243.25 comes from
; omega = 73.0 + 40.0/60.0+ 50.25*((JD_DD+2440000.0 - $
; 2396756.75)/365.25)/3600.0

  omega = 73.0D0 + 40.0D0 / 60.0D0 + $
          50.25D0 * ((JD_DD + 43243.25D0) / 365.25D0) / 3600.0D0

; Explains where the number 41780.0 comes from
; ww = 360.0/25.38*(JD_DD+2440000.0 - 2398220.0)

  ww = 360.0D0 / 25.38D0 * (JD_DD + 41780.0D0)
  ww = (ww / 360.0D0 - LONG (ww / 360.0D0)) * 360.0D0

  Bosin = SIN ((sun-omega) * !dpi / 180.0D0) * SIN (incl * !dpi / 180.0D0)
  Bocos = SQRT (1.0D0 - Bosin^2)

  Bo = ATAN (Bosin, Bocos) * 180.0D0 / !dpi

  ;print,Bo

  LoWsin = COS (incl * !dpi / 180.0D0) * SIN ((omega - sun) * !dpi / 180.0D0) $
	 / COS (Bo * !dpi / 180.0D0)
  LoWcos = -COS ((omega - sun) * !dpi / 180.0D0) / COS (Bo * !dpi / 180.0D0)
  LoW    = ATAN (LoWsin, LoWcos) * 180.0D0 / !dpi

  if (LoW LT 0.0D0) THEN LoW = LoW + 360.0D0
  Lo = LoW - ww

  IF (Lo LT 0.0D0) THEN Lo = Lo + 360.0D0

  LONG_diff = Lo - lonmax

  ;print, incr, JD_DD, Lo, lonmax

  IF ( (LONG_diff LT 0.0D0) AND (abs (LONG_diff) LT 100.0D0)) THEN BEGIN 

      JD_DD = JD_DD - incr
      incr = incr / 10.0D0
      IF (incr LT 1.0D-6) THEN GOTO, DONE

  ENDIF

  IF ( (LONG_diff GT 0.0D0) AND (ABS (LONG_diff) GT 100.0D0)) THEN BEGIN

      JD_DD = JD_DD - incr
      incr = incr / 10.0D0
      IF (incr LT 1.0D-6) THEN GOTO, DONE

  ENDIF

ENDFOR

DONE:

caldat, (JD_DD + 2440000.0D0), mmm, ddd, yyy
;print,'------------------'
dayfr = JD_DD - LONG (JD_DD) + 0.5D0
IF (dayfr GE 1.0D0) THEN dayfr = dayfr - 1.0D0

print, JD_DD, JD_DD+2440000.0D0, mmm, ddd, yyy, dayfr, FORMAT='(f14.7, 1x, f14.5, 1x, i2, 1x, i2, 1x, i4, 1x, f10.3)'
print,'------------------'


END
