;+
; NAME		jd_carr.pro
;
; PURPOSE	Convert Julian Date to Carrington Rotation.
;
; AUTHOR	Nick Arge   NOAA   20 Nov 2002
;-

PRO jd_carr, JD_temp

  JD = DOUBLE (string (JD_temp) + 'D0') - 2440000.0D0
  
  JD_CR0 = -41859.899931D0
  num_rot = (JD - JD_CR0) / 27.2753D0
  print, 'num_rot: ', num_rot
  
  IF (num_rot GE 1000.0D0) THEN carr_increm = 1000.0D0 else $
  IF (num_rot GE  100.0D0) THEN carr_increm =  100.0D0 else $
  IF (num_rot GE   10.0D0) THEN carr_increm =   10.0D0 else $
  IF (num_rot GE    1.0D0) THEN carr_increm =    1.0D0 else $
  CARR_CM = 0.0D0

  print, 'carr_increm: ', carr_increm
  
  carr_test = carr_increm
  JD_test = carr_jd_fn (DOUBLE (carr_test), 0.0D0)
  
  print, 'carr_test: ', carr_test, '  JD_test: ', JD_test
  
  WHILE (JD_test LE JD) DO	$
  BEGIN
    num_rot = (DOUBLE (JD) - DOUBLE (JD_test)) / DOUBLE (27.2753) 
    ;print, 'num_rot: ', num_rot
  
    IF (num_rot GE 1000.0D0) THEN carr_increm = 1000.0D0 else $
    IF (num_rot GE  100.0D0) THEN carr_increm =  100.0D0 else $
    IF (num_rot GE   10.0D0) THEN carr_increm =   10.0D0 else $
    IF (num_rot GE    0.0D0) THEN carr_increm =    1.0D0 
    ;print, 'carr_increm: ', carr_increm
    
    carr_test = carr_test + carr_increm
    JD_test   = carr_jd_fn (DOUBLE (carr_test), 0.0D0)
    print, 'carr_test: ', carr_test, '  JD_test: ', JD_test
  ENDWHILE
  
  Lo = jd_lo_fn (JD)
  Bo = jd_bo_fn (JD)
  pred_carr = carr_jd_fn (carr_test,Lo)
  
  del_JD = ABS (JD - pred_carr)
  IF (del_JD GT 20.0) THEN BEGIN 
  
    IF (JD - pred_carr LT 0.0D0) THEN carr_test = carr_test - 1.0D0 ELSE $
    carr_test = carr_test + 1.0D0
  
  ENDIF
  
  PRINT, 'carr_test: ', carr_test, ' Lo: ', Lo, ' Bo: ', Bo
  
  caldat, (JD + 2440000.0D0), mmm, ddd, yyy
  dayfr = JD - LONG (JD) + 0.5D0
  IF (dayfr GE 1.0D0) THEN dayfr = dayfr - 1.0D0
  
  PRINT, 'JD              JD+offset   month day year   dayfr:'
  PRINT, JD, JD + 2440000.0D0, mmm, ddd, yyy, dayfr, $
         FORMAT='(f14.7, 1x, f14.5, 1x, i2, 1x, i2, 1x, i4, 1x, f10.3)'

END
