;+
; NAME		jd_lo_fn.pro
;
; PURPOSE	Convert Julian Date to Heliographic Longitude.
;
; AUTHOR	Nick Arge   NOAA   20 Nov 2002
;-

FUNCTION jd_lo_fn, xx

; Modified April 27, 2001
;Converts from (Julian date -2440000) to Helographic longitude

  incl = DOUBLE (7.25)

  JD_DD = DOUBLE (xx) 

  ;nn = JD_DD - 2451545.0
  ; 2440000 - 2451545 = 11545

  nn = JD_DD - double (11545.0)
  LL = 280.472D0 + 0.9856474D0 * nn
  gg = 357.528D0 + 0.9856003D0 * nn
  sun = LL + 1.915D0 * SIN (gg * !dpi / 180.0D0) $
      + 0.020D0 * SIN (2.0D0 * gg * !dpi / 180.0D0)

; Explains where the number 43243.25 comes from
; omega = 73.0 + 40.0/60.0+ 50.25*(JD_DD + 2440000.0 - $
; 2396756.75)/365.25)/3600.0

  omega = 73.0D0 + 40.0D0 / 60.0D0+$
  50.25D0 * ((JD_DD + 43243.25D0) / 365.25D0) / 3600.0D0

; Explains where the number 41780.0 comes from
; ww = 360.0/25.38*(JD_DD+2440000.0 - 2398220.0)

  ww = 360.0D0 / 25.38D0 * (JD_DD + 41780.0D0)
  ww = (ww / 360.0D0 - long (ww / 360.0D0)) * 360.0D0

  Bosin = SIN ((sun - omega) * !dpi / 180.0D0) * SIN (incl * !dpi / 180.0D0)
  Bocos = SQRT (1.0D0 - Bosin^2)

  Bo = ATAN (Bosin, Bocos) * 180.0D0 / !dpi

  LoWsin = COS (incl * !dpi / 180.0D0) * SIN ((omega - sun) * !dpi / 180.0D0) $
	 / COS (Bo * !dpi / 180.0D0)
  LoWcos = -COS ((omega - sun) * !dpi / 180.0D0) / COS (Bo * !dpi / 180.0D0)
  LoW = atan (LoWsin, LoWcos) * 180.0D0 / !dpi

  IF (LoW LT 0.0D0) THEN LoW = LoW + 360.0D0
  Lo = LoW - ww

  IF (Lo  LT 0.0D0) THEN Lo  = Lo  + 360.0D0

  RETURN, Lo

END
