;
; Routine to calibrate MkIV images
;
; INPUT: The root name for a coronal image file
;	contained in the file callist
;	This file has +I, _Q, and _U appended
;	to read the three coronal stokes files
;	Also read are
;	gain, X21, X31, X22, X23, X32, X33
;	and Pangle
;
;	All files must have been run through a spline fit
;	to have NUMPIX x NUMAZ elements and to be
;	in the instrument coordinate frame.
;
; OUTPUT:
;	Calibrated corona files in heliocentric coordinates
;	called cali, calq, and calu
;
; David Elmore	11/5/98
;

scalepix = 150
filename = 'callist'
close,2 & openr,2,'callist' 
readf,2,filename & close,2
head =0 & head =bytarr(1024) ;;
headib=0 & headib=lonarr(16)  
headf =0 & headf =fltarr(24)
headie=0 & headie=lonarr(16)  


top =long(0)

close,1 & openr,1,filename+'.Ci' & point_lun,-1,top & readu,1,headib
                                                  readu,1,headf
                                                  readu,1,headie
                               point_lun, 1,top & readu,1,head
close,1

;;;;;;;;;;;;;;;;;;;;;;;;;
numaz = headib(10)
numpix = headib(9)
pangle = headf(0)
print,numaz,numpix,pangle
;;;;;;;;;;;;;;;;;;;;;;;;;
loadct,16
window,1,xsize=numaz,ysize=numpix-53,title='U'
window,2,xsize=numaz,ysize=numpix-53,title='I'
window,0,xsize=numaz,ysize=numpix-53,title='Q'
window,3,xsize=numaz,ysize=numpix-53

meanq = fltarr(numpix,100)
meani = fltarr(numpix,100)

corona_I = fltarr(numpix,numaz)
corona_Q = fltarr(numpix,numaz)
corona_U = fltarr(numpix,numaz)

gain = fltarr(numpix,numaz)
bias = fltarr(numpix)
X21 =  fltarr(numpix)
X31 =  fltarr(numpix)
X22 =  fltarr(numpix)
X23 =  fltarr(numpix)
X32 =  fltarr(numpix)
X33 =  fltarr(numpix)
Xinv11 =  fltarr(numpix)
Xinv12 =  fltarr(numpix)
Xinv13 =  fltarr(numpix)
Xinv21 =  fltarr(numpix)
Xinv22 =  fltarr(numpix)
Xinv23 =  fltarr(numpix)
Xinv31 =  fltarr(numpix)
Xinv32 =  fltarr(numpix)
Xinv33 =  fltarr(numpix)

drift = fltarr(numpix)
pbscale = 0.
str = filename
tmpstr = 'fred'

;
; Read in calibration matrix elements
;

close,1 & openr,1,'pbscale' & readu,1,pbscale
close,1 & openr,1,'drift' & readu,1,drift
close,1 & openr,1,'bias.fit' & readu,1,bias
close,1 & openr,1,'gain.fit' & readu,1,gain
close,1 & openr,1,'x21.fit' & readu,1,X21
close,1 & openr,1,'x31.fit' & readu,1,X31
close,1 & openr,1,'x22.fit' & readu,1,X22
close,1 & openr,1,'x23.fit' & readu,1,X23
close,1 & openr,1,'x32.fit' & readu,1,X32
close,1 & openr,1,'x33.fit' & readu,1,X33
;
; Create inverse polarimeter response matrix
;

det = 1./(x22*x33 - x23*x32)
xinv11(*,*) = 1.
xinv12(*,*) = 0. 
xinv13(*,*) = 0. 
xinv21 = det*(x23*x31 - x21*x33)
xinv22 = det*(x33)
xinv23 = det*(-x23)
xinv31 = det*(x21*x32 - x31*x22)
xinv32 = det*(-x32)
xinv33 = det*(x22)

;
; Loop through coronal image files applying
; calibration and writing out result
; Depol is applied to calibrated pB
;

close,2 & openr,2,'callist' 
first = 1
!y.range = [-1.e-8,1.e-7]
!y.style=1
!x.style=1
!x.ticklen=1
!y.ticklen=1

ifile=0
while ( not eof(2) ) do begin
	readf,2,filename
	print,filename

	str = string(filename+'.Ci')
	close,1 & openr,1,str & readu,1,head & readu,1,corona_I

	str = string(filename+'.Cq')
	close, 1 & openr,1,str &  readu,1,head & readu,1,corona_Q

	str = string(filename+'.Cu')
	close, 1 & openr,1,str &  readu,1,head & readu,1,corona_U

	;
	cali = fltarr(numpix,numaz)
	calq = fltarr(numpix,numaz)
	calu = fltarr(numpix,numaz)

; Remove rings from I, Q, and U
;

; Remove fine gain structure from Q and U and bias from I
; Scale gain by changes in coronal q during the day
; Use drift function and pbscale (from xmk4).


	rawmeani = fltarr(numpix)
	for ii=0,numpix-1 do begin
		corona_i(ii,*) = corona_i(ii,*) - float(bias(ii))
		rawmeani(ii) = total(corona_i(ii,*))/float(numaz) 
	endfor
	smoothi = leefilt(median(rawmeani,7))
	qmagic = total(corona_q(scalepix,*))/numaz
	for jj=0,numaz-1 do begin
		corona_q(*,jj) = smoothi(*)*corona_q(*,jj)/rawmeani(*)
		corona_u(*,jj) = smoothi(*)*corona_u(*,jj)/rawmeani(*)
		corona_i(*,jj) = corona_i(*,jj) * ( 1. - drift * $
			( qmagic/pbscale - 1.) )
		corona_q(*,jj) = corona_q(*,jj) * ( 1. - drift * $
			( qmagic/pbscale - 1.) )
		corona_u(*,jj) = corona_u(*,jj) * ( 1. - drift * $
			( qmagic/pbscale - 1.) )
	endfor
	;
	; Apply inverse X matrix to measured data
	;
	for ii=0,NUMAZ-1 do begin
cali(*,ii)=xinv11*corona_i(*,ii)+xinv12*corona_q(*,ii)+xinv13*corona_u(*,ii)
calq(*,ii)=xinv21*corona_i(*,ii)+xinv22*corona_q(*,ii)+xinv23*corona_u(*,ii)
calu(*,ii)=xinv31*corona_i(*,ii)+xinv32*corona_q(*,ii)+xinv33*corona_u(*,ii)
	endfor


	calq = calq/gain
	calu = calu/gain
	cali = cali/gain

	depol ,numpix,numaz,cali,calq,calu,skyq,skyu

	calq = calq - skyq
	calu = calu - skyu

	depolscan ,numpix,numaz,cali,calq,calu,skyq,skyu

	calq = calq - skyq
	calu = calu - skyu

;
; Shift for pangle
;

pixshift = -pangle*NUMAZ/360.
	cali = shift(cali,0,pixshift)
	calq = shift(calq,0,pixshift)
	calu = shift(calu,0,pixshift)


	str = string(filename+'.CalI')
	close,1 & openw,1,str & writeu,1,head & writeu,1,(cali*1.e9)

	str = string(filename+'.CalpB')
	close,1 & openw,1,str & writeu,1,head & writeu,1,(calq*1.e10)

	str = string(filename+'.CalU')
	close,1 & openw,1,str & writeu,1,head & writeu,1,(calu*1.e10)

	wset,0
	tv,bytscl(transpose(alog10(calq(40:370,*)+5.e-9)),-9,-5.7)
	wset,1
	tv,bytscl(transpose(calu(40:370,*)),-1.e-8,1.e-8)
	wset,2
	tv,bytscl(transpose(cali(40:370,*)),2.e-6,5.e-5)
	wset,3
	if ( first eq 1 ) then plot,calq(*,200)
	first=0
	oplot,calq(*,200)

	for ii=0,numpix-1  do  begin
	meanq(ii,ifile)=total(calq(ii,*))/720.
	meani(ii,ifile)=total(cali(ii,*))/720.
	endfor
	ifile=ifile+1
;stop
endwhile
print,'done'
!y.range=[1.e-9,1.e-6]
!p.title = 'Average pB vs. Scan # for Various Coronal Heights'
!y.title = "Average Calibrated pB"
!x.title = "Scan Number"
plot_io,meanq(275,*),psym=1
for ii=50,250,25 do oplot,meanq(ii,*),psym=1
stop


end

