;+
;  fixmk4s.pro
;
;  Open fixlist to get a list of filename roots to repair
;  Root is like c19_11
;  Files with .i, .q, and .u extensions to root are read.
;  The .sk file of that root is read.
;  I, Q, and U are normalized by the largest 'good' sky transmission
;	value for the scan.
;  Wraparounds are looked for and corrected.
;	(not needed after 98d333 1900)
;  Bright and dark points  and some streaks are removed.
;  The same files are writen out haven been fixed.
;
;  Note: first three images in "fixlist" must be: cal0, cal15, cal dark.
;  Note: swap_endian is used to swap bytes for PC or IBM or DEC computers.
;
;  David Elmore   HAO/NCAR   27 July 1999
;-
pro fixmk4s

nowrap = 1

NUMPIX = 384
NUMAZ = 1080
NUMANG = 720
PSHIFT = 0.5
temp = 0
zerosky = 359.7
CORONA = 1000
CAL0 = 0
CAL15 = 150
DARK = 9999

hpix = findgen(16)

filename='fred'
!y.range=[-32768,32767]
loadct,16

;
; While there are still filenames to read from fixlist,
; read and process them.
;
close,2
openr,2,'fixlist'				; Open list file.

while ( not eof(2) ) do begin
	readf,2,filename
	print,filename

	hdr=fltarr(256)
	imgi=intarr(NUMPIX,NUMAZ)
	imgq=intarr(NUMPIX,NUMAZ)
	imgu=intarr(NUMPIX,NUMAZ)
	datai=intarr(NUMPIX)
	dataq=intarr(NUMPIX)
	datau=intarr(NUMPIX)
	skyarray=intarr(NUMAZ)
	starray=intarr(NUMAZ)
	angle=intarr(NUMAZ)

	filei = string(filename+'.i')

head =0 & head =bytarr(1024) ;;
headib=0 & headib=lonarr(16)  
headf =0 & headf =fltarr(24)
headie=0 & headie=lonarr(16)  
pad = bytarr(800)

top =long(0)

;--- Read binary image header.

close,1 & openr,1,filei & point_lun,-1,top & readu,1,headib
                                                  readu,1,headf
                                                  readu,1,headie
                               point_lun, 1,top & readu,1,head

;headib=swap_endian(headib)
;headf=swap_endian(headf)
;headie=swap_endian(headie)
;head=swap_endian(head)

print,headib(0:4)
type = headie(0)
if ( type eq CORONA )  then print,"corona"
if ( type eq CAL0 )  then print,"cal0"
if ( type eq CAL15 )  then print,"cal15"
if ( type eq DARK )  then print,"dark"

;--- Read data records & store into a 2D array (radius, angle).
;--- Do this for i, q, u, ba, sk, st files.

	i=0
	while ( not eof(1) ) do begin
		readu,1,datai
;		datai=swap_endian(datai)
		imgi(*,i)=(datai(*))
		i=i+1
	endwhile
	irays = i

	fileq = string(filename+'.q')
	close,1
	openr,1,fileq
	readu,1,head
	i = 0
	while ( not eof(1) ) do begin
		readu,1,dataq
;		dataq=swap_endian(dataq)
		imgq(*,i)=(dataq(*))
		i=i+1
	endwhile

	fileu = string(filename+'.u')
	close,1
	openr,1,fileu
	readu,1,head
	i = 0
	while ( not eof (1) ) do begin
		readu,1,datau
;		datau=swap_endian(datau)
		imgu(*,i)=(datau(*))
		i = i + 1
	endwhile

	fileang = string(filename+'.ba')
	close,1
	openr,1,fileang
	readu,1,head
	i = 0
	while ( not eof(1) ) do begin
		readu,1,temp
;		temp=swap_endian(temp)
		angle(i) = temp
		i=i+1
	endwhile
	angle = 360. * float(angle) / 16384.

	filesky = string(filename+'.sk')
	close,1
	openr,1,filesky
	readu,1,head
	i = 0
	while ( not eof(1) ) do begin
		readu,1,temp
;		temp=swap_endian(temp)
		skyarray(i) = temp
		i=i+1
	endwhile
	skydim = size(skyarray) 

	filest = string(filename+'.st')
	close,1
	openr,1,filest
	readu,1,head
	i = 0
	while ( not eof(1) ) do begin
		readu,1,temp
;		temp=swap_endian(temp)
		starray(i) = temp
		i=i+1
	endwhile
	stdim = size(starray)
	;
	;
	sky = skyis(skyarray,skydim(1))
	print,'Sky transmission',sky,'=',sky/zerosky
	numy = i 

window,0,xsize=numy,ysize=384
window,1,xsize=720,ysize=384,ypos=400
print,numy,' azimuths'

wset,0
tv,bytscl(transpose(imgi),-32768,32767)
tv,bytscl(transpose(imgu),-2000,2000)
tv,bytscl(transpose(imgq),-1000,5000)

;
; Fix up digital wraps here
; This must be performed before spline
;
; Intensity is unsigned above pixel 32, signed otherwise

	imi = float(imgi(*,0:numy-1))
	imq = float(imgq(*,0:numy-1))
	imu = float(imgu(*,0:numy-1))

if ( type ne DARK ) then begin
	fred = imi(32:383,*)
	a = where ( fred lt 0 )
	if ( a(0) ne -1 ) then fred(a)=fred(a)+65536
	imi(32:383,*)=fred
endif

if ( type eq CORONA ) then begin
	for ii=0,numy-1 do begin
		if ( imq(60,ii) lt 0. ) then begin
			imq(*,ii) = -imq(*,ii)
			imu(*,ii) = -imu(*,ii)
		endif
	end

	a = where ( imq lt -4000. )
	if ( a(0) ne -1 ) then imq(a)=imq(a)+65536.
endif

;
; Make scan go the proper direction
;
	ang = float(angle(0:numy-1))
	newang = findgen(NUMANG)/(NUMANG/360.) - 180.
	rev = 0
	if ( starray(0) gt 8000 ) then rev = 1
	if ( rev eq 1 ) then begin
		shiftpix = (180. - 30. + PSHIFT)*NUMANG/360.
	endif  else begin
		shiftpix = (180. - 30. - PSHIFT)*NUMANG/360.
	endelse

;
; Attempt to remove barrel angle problems
;
	absc = findgen(numy)
	coef=poly_fit(absc,ang,1,angfit)
	a = where(abs(ang-angfit) gt 1 )
	if ( a(0) ne -1 ) then begin
		ang(a) = angfit(a)
		coef=poly_fit(absc,ang,1,angfit)
		a = where(abs(ang-angfit) gt 1 )
		if ( a(0) ne -1 ) then begin
			ang(a)=angfit(a)
		endif
	endif

;
; Normalize arrays for sky transmission
;

	imi = imi*zerosky/sky
	imq = imq*zerosky/sky
	imu = imu*zerosky/sky


;	interpi = (avgi(minpix+4) - avgi(minpix-4)) / 8.
;	interpq = (avgq(minpix+4) - avgq(minpix-4)) / 8.
;	interpu = (avgu(minpix+4) - avgu(minpix-4)) / 8.
;	for kk = -3,3 do begin
;		avgi(minpix+kk) = avgi(minpix-4) + interpi * (kk+4)
;		avgq(minpix+kk) = avgq(minpix-4) + interpq * (kk+4)
;		avgu(minpix+kk) = avgu(minpix-4) + interpu * (kk+4)
;	end
;		
;	minpix = 269
;	mini = avgi(minpix)
;	for kk=minpix+1,minpix+15 do begin
;		if ( avgi(kk) lt mini ) then begin
;			mini = avgi(kk)
;			minpix = kk
;		endif
;	end
;	interpi = (avgi(minpix+4) - avgi(minpix-4)) / 8.
;	interpq = (avgq(minpix+4) - avgq(minpix-4)) / 8.
;	interpu = (avgu(minpix+4) - avgu(minpix-4)) / 8.
;	for kk = -3,3 do begin
;		avgi(minpix+kk) = avgi(minpix-4) + interpi * (kk+4)
;		avgq(minpix+kk) = avgq(minpix-4) + interpq * (kk+4)
;		avgu(minpix+kk) = avgu(minpix-4) + interpu * (kk+4)
;	end
;;;
;;; Spline fit to -180 to 180 degrees
;;;
	imis = fltarr(NUMPIX,NUMANG)
	imqs = fltarr(NUMPIX,NUMANG)
	imus = fltarr(NUMPIX,NUMANG)

	wset,1

	print,'spline I'
	for ii=0,NUMPIX-1 do begin
		imis(ii,*) = spline(angfit,imi(ii,*),newang)
	endfor
	imis = shift(imis,0,shiftpix)
	tv,bytscl(transpose(imis),-1000,60000)

	print,'spline Q'
	for ii=0,NUMPIX-1 do begin
		imqs(ii,*) = spline(angfit,imq(ii,*),newang)
	endfor
	imqs = shift(imqs,0,shiftpix)
	tv,bytscl(transpose(imqs),-1000,5000)

	print,'spline U'
	for ii=0,NUMPIX-1 do begin
		imus(ii,*) = spline(angfit,imu(ii,*),newang)
	endfor
	imus = shift(imus,0,shiftpix)
	tv,bytscl(transpose(imus),-1000,5000)

;
headib(10) = 720;
headie(5) = 720;

	close,1
	filei = string(filename+'.Ci')
	close,1 & openw,1,filei & writeu,1,headib $
		& writeu,1,headf & writeu,1,headie $
		& writeu,1,pad & writeu,1,imis 
	close,1
	fileq = string(filename+'.Cq')
	close,1 & openw,1,fileq & writeu,1,headib $
		& writeu,1,headf & writeu,1,headie $
		& writeu,1,pad & writeu,1,imqs 
	close,1
	fileu = string(filename+'.Cu')
	close,1 & openw,1,fileu & writeu,1,headib $
		& writeu,1,headf & writeu,1,headie $
		& writeu,1,pad & writeu,1,imus 
	close,1

;stop
endwhile

;stop

end
