;+ mk4s.pro
;
;	Find the X matrix for MkIV ;
;
;INPUT:
;	Already spline-fit cal0, cal15, dark
;	File names are in Xmk4list
;	File names must be in order for cal0, cal15, and dark
;OUTPUT:
;	Vector by pixel height:
;		bias,X21, x22, x23, x31, x32, x33
;	2-D array
;		gain
;
;	It is assumed bias for Q and U is zero
;	It is assumed X12 and X13 are zero
;
;	Routine follows proceedure in MkIII Calibration Technote
;	Matrix convention is element ( row, column )
;
; David Elmore   HAO/NCAR   3 November 1999
;-

pro xmk4s

numpix = 384
numaz = 720
scalepix = 150
radfac = !pi/180.
!x.ticklen=1
!y.ticklen=1
!x.style=1
!y.style=1
!y.range=0
!x.range=0
;
; Values from technote
;
; Transmission of opal

Bopal = 10.11e-6

; Fractional polarization of tilt plates at 15 degrees

fpt = 0.01543

; Half-Width of noise on meanq signal

x21noise = 0.02

gain = fltarr(numpix,numaz)
bias = fltarr(numpix)
x21 = fltarr(numpix)
x31 = fltarr(numpix)
x22 = fltarr(numpix)
x23 = fltarr(numpix)
x33 = fltarr(numpix)
x32 = fltarr(numpix)

listfile = 'Xmk4list'
filename = 'fred'
cal0_I = fltarr(numpix,numaz)
cal0_Q = fltarr(numpix,numaz)
cal0_U = fltarr(numpix,numaz)
cal15_I = fltarr(numpix,numaz)
cal15_Q = fltarr(numpix,numaz)
cal15_U = fltarr(numpix,numaz)
dark_I = fltarr(numpix,numaz)
dark_Q = fltarr(numpix,numaz)
dark_U = fltarr(numpix,numaz)
corona_I = fltarr(numpix,numaz)
corona_Q = fltarr(numpix,numaz)
corona_U = fltarr(numpix,numaz)
cx21 = fltarr(numpix)
headl = lonarr(256)

close,1 
close, 2 & openr,2,listfile
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,cal0_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,cal0_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,cal0_U & close,1
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,cal15_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,cal15_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,cal15_U & close,1
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,dark_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,dark_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,dark_U & close,1
readf,2,filename
print,filename
openr,1,filename+'.Ci' & readu,1,headl & readu,1,corona_I & close,1
openr,1,filename+'.Cq' & readu,1,headl & readu,1,corona_Q & close,1
openr,1,filename+'.Cu' & readu,1,headl & readu,1,corona_U & close,1

;
; This is the offset of MkIV relative
; to the calibration tilt plates
; Determined by minimizing calibrated coronal U

shiftpix = -7.0 * NUMAZ/360.

cal0_i=shift(cal0_i,0,shiftpix)
cal0_q=shift(cal0_q,0,shiftpix)
cal0_u=shift(cal0_u,0,shiftpix)
cal15_i=shift(cal15_i,0,shiftpix)
cal15_q=shift(cal15_q,0,shiftpix)
cal15_u=shift(cal15_u,0,shiftpix)
dark_i=shift(dark_i,0,shiftpix)
dark_q=shift(dark_q,0,shiftpix)
dark_u=shift(dark_u,0,shiftpix)
corona_i=shift(corona_i,0,shiftpix)
corona_q=shift(corona_q,0,shiftpix)
corona_u=shift(corona_u,0,shiftpix)

; Perform bias correction

for ii = 0,numpix-1 do begin
	bias(ii) = total(dark_I(ii,*))/float(numaz)
endfor

for ii = 0,numpix-1 do begin
	cal0_I(ii,*) = cal0_I(ii,*) - bias(ii)
	cal15_I(ii,*) = cal15_I(ii,*) - bias(ii)
	corona_I(ii,*) = corona_I(ii,*) - bias(ii)
endfor

; Gain is 2-D!

gain = cal0_I/Bopal

; Intensity to polarization 

for  ii=0, numpix-1 do begin
	x21(ii) = total(cal0_Q(ii,*))/total(cal0_I(ii,*))
	x31(ii) = total(cal0_U(ii,*))/total(cal0_I(ii,*))
endfor

; Adjust x21 for coronal crosstalk, not opal

for ii=0,numpix-1 do begin
	cx21(ii) = min((corona_q(ii,*))/corona_i(ii,*))
endfor
calratio = (total(cx21(280:290))+11.*x21noise)/total(x21(280:290))
x21 = x21 * calratio


; Angles will run from -180 to 180

ang = (findgen(numaz) - numaz/2.)*2.*!pi/float(numaz)
sang = sin(2.*ang)
cang = cos(2.*ang)

qphi = fltarr(numpix)
uphi = fltarr(numpix)
phi = fltarr(numpix)
qamp = fltarr(numpix)
uamp = fltarr(numpix)
amp = fltarr(numpix)
rat = fltarr(numpix)
qmean = fltarr(numpix)
umean = fltarr(numpix)

; Find amplitude and phase of cal 15
; at position angle of channel 4

fitq = fltarr(numpix,numaz)
fitu = fltarr(numpix,numaz)

; Compute X22 and X33
; Use fractional polarization normalizaton
;
; Remove fractional polarization due to opal alone
; from fractional polarization of cal 15

fred = cal15_q/cal15_i - cal0_q/cal0_i
mary = cal15_u/cal15_i - cal0_u/cal0_i

for ii=0,NUMPIX-1 do begin
	sprod = total(sang*fred(ii,*))/numaz
	cprod = total(cang*fred(ii,*))/numaz
	qphi(ii) = atan(sprod,cprod)
	qamp(ii) = 2.*sqrt(sprod*sprod+cprod*cprod)
	fitq(ii,*)=qamp(ii)*cos(2.*ang - qphi(ii))
	qmean(ii) = total(fred(ii,*))/numaz

	sprod = total(sang*mary(ii,*))/numaz
	cprod = total(cang*mary(ii,*))/numaz
	uphi(ii) = atan(sprod,cprod)
	uamp(ii) = 2.*sqrt(sprod*sprod+cprod*cprod)
	fitu(ii,*)=uamp(ii)*cos(2.*ang - uphi(ii))
	rat(ii) = total(cal15_i(ii,*)/cal0_i(ii,*))/numaz
	umean(ii) = total(mary(ii,*))/numaz

endfor
amp = (qamp+uamp)/2.
phi = (qphi + (uphi + !pi/2.))/ 2.

plot,fred(200,*),psym=3
oplot,fitq(200,*)
oplot,mary(200,*),psym=3
oplot,fitu(200,*)
stop

; Find Today's corona scale factor, the average coronal brightness
; vs height at the time of the polarization calibration;

pbscale = total(corona_q(scalepix,*))/float(numaz)

; Signs here do not match the paper
; This is because when the barrel rotates, the orientation
; of the calibration polarizer rotates backwards.

!p.title = "X21"
!y.range=[-.2,.2]
plot,x21
stop

!p.title = "X31"
plot,x31
stop

x22 =  amp * cos(2.*phi)/fpt
!p.title = "X22"
!y.range=[-40,40]
plot,x22
stop

X23 = -amp * sin(2.*phi)/fpt
!p.title = "X23"
plot,x23
stop

x32 = amp * sin(2.*phi)/fpt
!p.title = "X32"
plot,x32
stop

x33 = amp * cos(2.*phi)/fpt
!p.title = "X33"
plot,x33
stop

close,1 & openw,1,'pbscale' & writeu,1,pbscale
print,'pbscale=',pbscale

close,1 & openw,1,'bias' & writeu,1,bias
!p.title='Bias'
!y.range=[-500,500]
plot,bias
stop

close,1 & openw,1,'gain' & writeu,1,gain
!y.range=[0,1.e9]
!p.title='Gain'
plot,gain(*,200)
stop

close,1 & openw,1,'x21' & writeu,1,x21
close,1 & openw,1,'x31' & writeu,1,x31
close,1 & openw,1,'x22' & writeu,1,x22
close,1 & openw,1,'x23' & writeu,1,x23
close,1 & openw,1,'x32' & writeu,1,x32
close,1 & openw,1,'x33' & writeu,1,x33
close,1

print,'done'

end
