;**************************************************************************
;NAME       : f4data (procedure)
;FUNCTION   : reformat f4 v & s. data
;		+ read dark/flat/data images
;		+ subtract dark
;		+ make flat correction
;		+ calculate sbtraction & average of data
;		+    "      polarization dgree in unit of 10-4
;		+ save results (in *.p or .s .v files)
;    filename should be following
;       dark   --- fd** 					--> read dark
;       flat   --- ff**.v, .s    	3 files shuld exist	--> read flat
;       data   --- ****.v, .s	          "
;MODIFICATION HISTORY :
;	K.Ichimoto, 92/10/25   direct read, add delete keyword
;==========================================================================
pro f4data,fnam,h,img,s,v,hd,drk,hf,flt,save=save,psave=psave,dir=dir, $
	noshift=noshift,ver=ver,delete=delete

; fnam    : file name without '.v','.s'
;     	    if 'fd*'  read dark in drk
;     	    if 'ff*'  read 2-flats in flt
;     	    others (usualy 'f1*' or 'fb*') read 3-images 
;		and make dark & flat corrections if drk and flt exist
;  h(4)	  : header for img,q,u,v
;    drk      : dark image
;    hd       : header of dark data
;    flat     : flat image (*,*,4)
;    hf       : header of flat data (4)
;    /save    : save results
;    /psave   : save results in a unique file of '.p'
;    /noshift : don't make co-alignment correction fot q,u images
;    /ver     : display part images for co-arignments
;    /delete  : delete .q .u .v files after psave

if not keyword_set(dir) then dir=''
rfact=10000.
change=0

case strmid(fnam,strpos(fnam,'/')+1,2) of

    'fd': begin		; ============== dark  ==============
	nkrget,dir+fnam,hd,drk
	if keyword_set(save) then $
			nkrsave,dir+fnam,hd,drk,/nocheck
	end

    'ff': begin		; ============== flat  ==============
	nkrgeth,dir+fnam+'.s',h
	if h.nbyt eq 2 then flt=intarr(h.nx,h.ny,4)
	if h.nbyt eq 1 then flt=bytarr(h.nx,h.ny,4)
	h1=nkrhead()
	hf=replicate(h1,4)
	ext=['.s','.s','.v','.v']
	imn=[1,2,1,2]
	for i=0,3 do begin
		nkrget,dir+fnam+ext(i),h1,img1,img_no=imn(i)
		if h1.correct eq 1 then begin
			sbtdrk,h1,img1,hd,drk
			change=1
		endif
		hf(i)=h1
		flt(*,*,i)=img1
	endfor
	if keyword_set(save) and (change eq 1) then begin
		nkrsave,dir+fnam+'.s',hf(0),flt(*,*,0),/nocheck
		nkrsave,dir+fnam+'.s',hf(1),flt(*,*,1),/nocheck,/append
		nkrsave,dir+fnam+'.v',hf(2),flt(*,*,2),/nocheck
		nkrsave,dir+fnam+'.v',hf(3),flt(*,*,3),/nocheck,/append
	endif
	end

    else: begin		; ============== data  ==============
	if not keyword_set(drk) then drk=0
	if not keyword_set(hd) then hd=0
	if not keyword_set(hf) then hf1=0
	h1=nkrhead()
	h2=replicate(h1,2)	; header for i and q,u,v
	h=replicate(h1,3)	; header for i,s,v
	ff=findfile(dir+fnam,count=count)
	if count eq 0 then exist=0 else exist=1
	ff=findfile(dir+fnam+'.*',count=count1)
	ff=[ff,findfile(dir+fnam,count=count2)]
	count=count1+count2
	if count eq 0 then begin
		print,'file: '+dir+fnam+'.* not found in "f4data" !'
		return
	endif else begin	; examine the kind of file '.p' or '.v,q,u'
		p_exist=0	& v_exist=0
		for i=0,count-1 do begin
		    if strmid(ff(i),strlen(ff(i))-2,2) eq '.v' then v_exist=1
		    if strmid(ff(i),strlen(ff(i))-2,2) eq '.p' then p_exist=1
		endfor
		if v_exist and (exist or p_exist) then begin
			ans=''
			print,'"" & "*.v" both files exist.'
			read,'    Which do you use (*<ret>/v) ? : ',ans
			if ans ne 'v' then v_exist=0
		endif
		if not v_exist then begin
			if exist then begin
				fext=''
			endif else begin
				fext='.p'
			endelse
			nkrget,dir+fnam+fext,h1,img,img_no=1	& h(0)=h1
			nkrget,dir+fnam+fext,h1,s,img_no=2	& h(1)=h1
			nkrget,dir+fnam+fext,h1,v,img_no=3	& h(2)=h1
			goto,endlab
		endif
	endelse
		
	; ------------  s  -------------
	if keyword_set(hf) then hf1=hf(0)
	if keyword_set(flt) then $
		ftgetar,dir+fnam+'.s',h2,img,s,dark=drk,hd=hd,	$
					flat=flt(*,*,0:1),hf=hf1,info=info $
	else	ftgetar,dir+fnam+'.s',h2,img,s,dark=drk,hd=hd,info=info
	if info eq -1 then begin
		print,dir+fnam+'.s   not found!'
		return
	endif else begin
	    h(0)=h2(0)
	    h(1)=h2(1)	&	h(1).value='S'
	    if keyword_set(save) and info eq 1 then begin
		nkrsave,dir+fnam+'.s',h(1),s,/nocheck
		nkrsave,dir+fnam+'.s',h(0),img,/nocheck,/append
	    endif
	endelse

	; ------------  v  -------------
	if keyword_set(hf) then hf1=hf(2)
	if keyword_set(flt) then $
		ftgetar,dir+fnam+'.v',h2,img0,v,dark=drk,hd=hd,	$
					flat=flt(*,*,2:3),hf=hf1,info=info $
	else	ftgetar,dir+fnam+'.v',h2,img0,v,dark=drk,hd=hd,info=info
	if info eq -1 then begin
		print,dir+fnam+'.v   not found!'
	endif else begin
	    h(2)=h2(1)	&	h(2).value='V'
	    if keyword_set(save) and info eq 1 then begin
		nkrsave,dir+fnam+'.v',h(2),v,/nocheck
		nkrsave,dir+fnam+'.v',h2(0),img0,/nocheck,/append
	    endif
	    if not keyword_set(noshift) then begin
		displace,img,img0,dx,dy,ver=ver
		v=rshift(v,dx,dy,missing=0)
	    endif
	endelse

	if keyword_set(psave) then begin
	    if strmid(fnam,strlen(fnam)-2,2) eq '.p' then fext='' $
	    else fext='.p'
	    nkrsave,dir+fnam+fext,h(0),img,/nocheck
	    nkrsave,dir+fnam+fext,h(1),s,/append
	    nkrsave,dir+fnam+fext,h(2),v,/append
	    if keyword_set(delete) then begin
		spawn,'rm '+dir+fnam+'.s'
		spawn,'rm '+dir+fnam+'.v'
	    endif
	endif

endlab:
	if n_params() eq 2 then begin
		s=size(img)
		tmp=img
		img=make_array(s(1),s(2),3,type=s(0))
		img(*,*,0)=tmp
		img(*,*,1)=s
		img(*,*,2)=v
	endif

	end

endcase
end
