;********************************************************************
;NAME       : fdisp_p (procedure)
;FUNCTION   : display flare telescope data, catalog to NAOJ data base
;MODIFICATION HISTORY :
;	K.Ichimoto, '92/06/28
;	K.Ichimoto, '92/09/20
;	K.Ichimoto, 1992/10/23	magcont,/v,  add catalog keyword
;	   "      , 1992/10/24	add calibration procedure on cataloging
;	   "      , '93/04/10	t4sample keyword
;	   "      , '93/04/28	fb* catalog
;====================================================================
pro fdisp_p,date,datalog=datalog,catalog=catalog,nocalib=nocalib,	$
	t4sample=t4sample
; filename is assumed to be as follow
;   F1 data:    'f1***.v,q,u' or 'fb***.v,q,u'
;   F4 data:    'f4***.v,s '  or 'fv***.v,s'
; date    ---  'yymmdd'
; KEYWORD:
;   /datalog  -- use datalog file to display menu
;		  datalog file is /data/ftobs/datalog/yymmdd.log
;   catalog   -- return catalogued filename
;   nocalib   -- do not make calibration
;   t4sample  -- if 1, sample t4 data also

ver=1
if n_elements(date) eq 0 then begin
	date=''
	read,'Enter date : ',date
endif
dir='/data2/ftobs/'+date+'/'
sample_dir='/data/ftobs/sample/'
catalog_dir='/solar/flare/'
cd,dir
dir=''
window,0
ctlg=0
mode=1	; 0-magcont, 1-magtv1, 2-magtv2 
if keyword_set(datalog) then menu_mode=1 $
else menu_mode=0	; 0-only filename,  1-use datalog
if keyword_set(t4sample) then t4sample=1 else t4sample=0

fnam0=''
while 1 do begin
	case mode of
	   0:	ret=['<red-blue>','<spot+cont>','<printout>']
	   1:	ret=['<contour>','<spot+cont>']
	   2:	ret=['<contour>','<red-blue>']
	endcase
	if fnam0 ne '' then ret=[ret,'[catalog]']
	ret=[ret,'[menu-mode]','[exit]']
	if menu_mode eq 0 then $
		fnam=ftlsmenu(dir,ret=ret,op='-tr',lsstr=lsstr)
	if menu_mode eq 1 then $
		fnam=ftlsmenu(dir,ret=ret,datalog=date)

	;  ---------  exit  ------------
	if fnam eq '[exit]' then return

	;  ---------  menu  ------------
	if fnam eq '[menu-mode]' then begin
		case menu_mode of
		   0:	begin
			m_mode=1
			end
		   1:	begin
			m_mode=0
			end
		endcase
		menu_mode=m_mode
	endif

	;  ---------  F1 data  ---------
	if strmid(fnam,0,2) eq 'f1' or strmid(fnam,0,2) eq 'fb' then begin
		f1data,fnam,h,img,q,u,v,dir=dir,ver=ver
		wset,0
		case mode of
		   0:	magcont,img,q,u,v,h=h
		   1:	magtv1,img,q,u,v,h=h
		   2:	magtv2,img,q,u,v,h=h
		endcase
		fnam0=fnam
	endif

	;  ---------  F4 data  ---------
	if strmid(fnam,0,2) eq 'f4' or strmid(fnam,0,2) eq 'fv' then begin
		f4data,fnam,h,img,s,v,dir=dir
		wset,0
		veltv1,img,s,h=h
		fnam0=fnam
	endif

	;  ---------  sample  ----------
	if fnam eq '[sample]' then begin
	    ; save 'f1' and 'f4' data to sample_dir
	    ; with a filename of fbyymmdd.a##.p   (f1)
	    ;                 or fvyymmdd.a##.p   (f4) 
	    kind=strmid(fnam0,0,2)
	    if (kind eq 'f1') or (kind eq 'f4') then begin
	    	if dir eq '' then begin
		    print,'unable to sample the data'
		    cd,'..'
		endif
		seq=strmid(fnam0,2,strlen(fnam0)-2)
		case kind of
		    'f1': outfnam=sample_dir+'fb'+date+'.'+seq
		    'f4': outfnam=sample_dir+'fv'+date+'.'+seq
		    endcase
	    endif else begin
		outfnam=sample_dir+fnam0
	    endelse
	    print,'save "',fnam0,'" to "',outfnam,'.p"'
	    if (kind eq 'f1') or (kind eq 'fb') then begin
	    	nkrsave,outfnam+'.p',h(0),img,/nocheck
	    	nkrsave,outfnam+'.p',h(1),q,/append
	   	nkrsave,outfnam+'.p',h(2),u,/append
	    	nkrsave,outfnam+'.p',h(3),v,/append
	    endif
	    if (kind eq 'f4') or (kind eq 'fv') then begin
	    	nkrsave,outfnam+'.p',h(0),img,/nocheck
	    	nkrsave,outfnam+'.p',h(1),s,/append
	   	nkrsave,outfnam+'.p',h(2),v,/append
	    endif
	endif

	;  ---------  catalog  ----------
	if fnam eq '[catalog]' then begin
	    ; save 'f1' and 'f4' data to catalog_dir
	    ; with a filename of fbyymmdd.hhmm.p   (f1)
	    ;                 or fvyymmdd.hhmm.p   (f4) 
	    kind=strmid(fnam0,0,2)
	    if (kind eq 'f1') or (kind eq 'f4') $ 
		or (kind eq 'fb') or (kind eq 'fv') then begin
	    	;if dir eq '' then begin
		;    print,'unable to sample the data'
		;    cd,'..'
		;endif
		seq=strmid(fnam0,2,strlen(fnam0)-2)
		case kind of
		    'f1': begin
			  samplename=sample_dir+'fb'+date+'.'+seq+'.p'
	    		  print,'save "',fnam0,'" to "',samplename
	    		  nkrsave,samplename,h(0),img,/nocheck
	    		  nkrsave,samplename,h(1),q,/append
	   	  	  nkrsave,samplename,h(2),u,/append
	    		  nkrsave,samplename,h(3),v,/append
			;-- copy closest T4 data to sample --
			  if t4sample eq 1 then begin
			      fnam4=findtime(date,h(0).time1,'f4')
			      seq4=strmid(fnam4,2,strlen(fnam4)-2)
			      samplename4=sample_dir+'fv'+date+'.'+seq4+'.p'
			      com='cp '+fnam4+'.p '+samplename4
			      print,com
			      spawn,com
			  endif
			  if not keyword_set(nocalib) then begin ; calibration
				f1calibf,samplename,/ver,	$
					outfile='/data/ftobs/temp',bin=2
				samplename='/data/ftobs/temp'
				;f4calibf,samplename4,/ver,	$
				;	outfile='/data/ftobs/temp4',bin=2
				;samplename4='/data/ftobs/temp4'
			  endif
			  end
		    'fb': begin	; catalog sampled data
			  if not keyword_set(nocalib) then begin ; calibration
				print,fnam0
				f1calibf,sample_dir+fnam0,/ver,	$
					outfile='/data/ftobs/temp',bin=2
				samplename='/data/ftobs/temp'
			  endif
			  end
		    'f4': begin
			  samplename=sample_dir+'fv'+yymmdd+'.'+seq
	    		  print,'save "',fnam0,'" to "',samplename,'.p"'
	    		  nkrsave,samplename,h(0),img,/nocheck
	    		  nkrsave,samplename,h(1),s,/append
	   		  nkrsave,samplename,h(2),v,/append
			  end
		    endcase
	    endif else begin
		samplename=sample_dir+fnam0
	    endelse
	    catalog,samplename,filename=ctlgfile1
	    ;catalog,samplename4,filename=ctlgfile4
	    if ctlg eq 0 then catalog=ctlgfile1 	$
		else catalog=[catalog,ctlgfile1]
	    ctlg=1
	endif

	;  ---------  change display mode  ----------
	if fnam eq '<contour>' then begin
		magcont,img,q,u,v,h=h
		mode=0
	endif
	if fnam eq '<red-blue>' then begin
		magtv1,img,q,u,v,h=h
		mode=1
	endif
	if fnam eq '<spot+cont>' then begin
		magtv2,img,q,u,v,h=h
		mode=2
	endif
	if fnam eq '<printout>' then begin
		magcont,img,q,u,v,h=h,/printout
	endif

endwhile

end
