;********************************************************************
;NAME       : ftgetar (procedure)
;FUNCTION   : get sum and subtraction/sum images from F.T. raw file
;			K.I. '92/04/03
;			K.I. '92/11/21 t_parm(0,1)
;			K.I. '93/02/06 100% pol. 20000 -> 10000
;====================================================================
pro ftgetar,file,h,add,r,dark=dark,hd=hd,flat=flat,hf=hf, $
	help=help,save=save,info=info,one=one

;input:
;   file   : input (raw) file
;   dark   : dark file name or dark image
;   hd     : header of dark data
;          if dark is image and hd not exist, 
;		integd should be same with integ of data
;   	   if dark is filename, dark image*integ/integd is stored in dark
;   flat   : flat file name or flat image (*,*,2)
;   hf     : header of flat data (1-element common)
;   save   : if exist save results ( save=savefile )
;   one    : if set, 100% pol. is 10000. else is 20000.
;output:
;   h	: header structure
;   add	: summension of 2-images
;   r   : ratio between subtraction and image * rfact
;   info: return status   -1 -- file not found
;                          0 -- already reduced data
;			   1 -- currently reduce the data
rfact=10000.
ans=''
info=0

if keyword_set(help) then begin
	print,'>> ftgetar <<'
	print,'FUNCTION:'
	print,'   obtain polarization degree and intensity image '
	print,'   from Flare Telescope (T1 & T4) row data and save it to file '
	print,'USAGE: '
	print,'   ftgetar,file,h,img,r,[dark=dark][,flat=flat],[save=svefile]'
	print,'     file --- raw data filename'
	return
endif

print,'ftgetar: '
!error=0
nkrget,file,h1,a1,img_no=1
if !error eq 1 then begin
	info=-1
	return	; file not found
endif
nkrget,file,h2,a2,img_no=2
if ( h1.nx ne h2.nx ) or $
   ( h1.ny ne h2.ny ) or $
   ( h1.nbyt ne h2.nbyt) or $
   ( h1.integ ne h2.integ ) then begin
	print,'data type not compatible between img1 & img2'
	stop
endif
h=replicate(nkrhead(),2)

if(h1.correct eq 4) then begin	; ----- reduced data
	r=a1	& h(1)=h1
	add=a2	& h(0)=h2
	info=0

endif else begin		; ----- not corrected data
    if keyword_set(dark) and (h2.correct eq 1) then begin
	if(n_elements(dark) le 3) then $ 
		nkrget,dark,hd,dark,img_no=1	; dark filename --> dark array
	if not keyword_set(hd) then hd=h2
	if h1.correct eq 1 then sbtdrk,h1,a1,hd,dark
	sbtdrk,h2,a2,hd,dark
    endif

    if keyword_set(flat) and (h2.correct le 2) then begin
	print,'  flat correction'
	if(n_elements(flat) le 3) then begin ; flat = flat filename
		nkrget,flat,hf,flt1,img_no=1
		nkrget,flat,hf,flt2,img_no=2
		if hf.correct eq 1 then begin
			if not keyword_set(dark) then begin
				print,'flat needs dark data !!'
				stop
			endif
			sbtdrk,hf,flt1,hd,dark
			sbtdrk,hf,flt2,hd,dark
		endif
		flat=intarr(hf.nx,hf.ny,2)
		flat(*,*,0)=flt1
		flat(*,*,1)=flt2
	endif else begin
		flt1=flat(*,*,0)
		flt2=flat(*,*,1)
	endelse
	if(arrcmp(a1,flt1) eq -1) then begin
		read,'flat not correctly defined! continue ? : ',ans
		if(ans eq 'n') then stop
		print,'continue without flat correction'
	endif else begin
		sf=size(flt1)
		npix=float(sf(1))*float(sf(2))
		avflt1=total(flt1)/npix
		avflt2=total(flt2)/npix
		fact=(avflt1+avflt2)/2.
		a1=fix(float(a1)/float(flt1>1)*fact)
		a2=fix(float(a2)/float(flt2>1)*fact)
		h1.correct=3	& h2.correct=3
		h1.t_parm(0)=fix(avflt1) &	h1.t_parm(1)=fix(avflt2)
		h2.t_parm(0)=fix(avflt1) &	h2.t_parm(1)=fix(avflt2)
	endelse
    endif

    add=fix((long(a1)+long(a2))/2)
    r=fix(long(a1-a2)*rfact/(add>1))
    if keyword_set(one) then r=r/2	; <== 93/02/06
    h(0)=h2
    h(1)=h1
    h(0).value='I'
    h(0).zero='0'
    h(1).correct=4
    h(1).value='P'
    h(1).zero='0'
    h(1).scale='10000.'
    if keyword_set(save) then begin
	ss=strcompress(string(save),/remove_all)
	if ss eq '1' then outfile=file	$
	else outfile=save
	print,'    save to "',outfile,'"'
	nkrsave,outfile,h(1),r,/nocheck
	nkrsave,outfile,h(0),add,/nocheck,/append
    endif
    info=1

endelse

return
end
