;********************************************************************
;NAME       : ftstatus (function)
;FUNCTION   : define status structure of flare telescope
;						K.I. '92/03/28
;====================================================================
function ftstatus
; present status of the flare telescope and observation	
; written by f0-f4 in n:\flare\cntl\ftstat
fst = { ftstatus,	$
; -----------    header of this status    -------------- $
	fst_id: 	'',	$ ; status identifier "FTST_1"
	one: 		1,	$ ; one in this system
	date:		'',	$ ; date 	"yy/mm/dd"
	time:		'',	$ ; time	"hh:mm:ss"
; -----------    telescope common   -------------- $
	l0:   	      	0,	$ ; solar brighteness
	gx:		0,	$ ; guiding table x-position
	gy:             0,	$ ; guiding table y-position
	posih: 		0l,	$ ; hour angle
	posid:		0l,	$ ; declination
	g_balx:         0,	$ ; guiding balance  of x
	g_baly:         0,	$ ; guiding balance  of y
	tst:     bytarr(4),	$ ; telescope status from Nikon controler
		; tst[0]  bit 0 - /DECLMT_N		$
		;             1 - /DECLMT_P		$
		;             2 - /RALMT_N 		$
		;             3 - /RALMT_P 		$
		;             4 - /DEC_ALMOUT		$
		;             5 - /RA_ALMOUT		$
		; tst[1]  bit 0 - /DEC_SVOUT		$
		;             1 - /RA_SVOUT		$
		;             2 - /SOLAR_ON		$
		;             3 - /SIDEREAL_ON		$
		;             4 - /POST1_OUT		$
		;             5 - /POST2_OUT		$
		;             6 - /HNDST_OUT		$
		;             7 - /PANEL_OUT		$
		; tst[2]  bit 0 - DEC_TRK_OUT		$
		;             1 - RA_TRK_OUT		$
		;             3 - /SRVON_OUT		$
		;             4 - SRV_ON_OUT		$
		;             5 - DEC_FINE_OUT		$
		;             6 - RA_FINE_OUT		$
		;             7 - CAPTURE_OUT		$
		; tst[3]  bit 0 - /DEC_RDY_OUT		$
		; 	      1 - /RA_RDY_OUT		$
		; 	      2 - /STRTRK_OUT		$
		; 	      3 - /SUNTRK_OUT		$
		;	      4 - C-OPN_MON (Y11)    ?	$
		;             6 - M-DISP_MON(Y12)    ?	$
	f_flag:    byte(0),	$ ; flag of flare occurence
	kdp_power: byte(0),	$ ; KDP power (15V) off(0) / on(1)
	temp:	 intarr(2),	$ ; temperature
; ---------------    each telescope   --------------- $
	cap:	  byte(0),	$ ; cap    open/close
			; bit 0 - cap of T1  open(0)/close(1)		$
			;     1 -        T2         "			$
			;     2 -        T3         "			$
			;     3 -        T4         "			$
	calib:    byte(0),	$ ; calib. in/out, shutter open/close
			; bit 0 - calibration optics of T1 out(0)/in(1)	$
			;     1 -          "            T2      "	$
			;     2 -          "            T3      "	$
			;     3 -          "            T4      "	$
			;     4 - shutter of T1  open(0)/close(1)	$
			;     5 -     "      T2        "		$
			;     6 -     "      T3        "		$
			;     7 -     "      T4        "		$
	focus:  intarr(4),	$ ; focus position 1-4
	exp:    bytarr(4),	$ ; ccd exposure 
	prog:   strarr(5),	$ ; running program 
	ost:    bytarr(5),	$ ; operation mode of f0-f4
		;          0 - sleep	 				$
		;          1 - standby	 				$
		;          2 - pause		   			$
		;          4 - taking dark			    	$
		;          5 - taking flat				$
		;          9 - obsend    				$
		;         10 - continuous observation mode		$
		;         11 - step observation mode			$
		;         20 - flare mode				$
	o_table:  bytarr(5),	$ ; observation table no. in use
	errst:    bytarr(5),	$ ; error status of f0-f4
		;           0 - no error               			$
		; est[3] bit 0 - LV full				$
	dummy:      byte(0),	$ ; keep even number of char
; ------  t1 special  -------- $
	t1_filter:	0,	$ ; 6303 filter w-l position
	t1_ftemp:	0,	$ ; filter temperature
	t1_kdp:		0,	$ ; voltage of kdp1
	t1_turret:  	0,	$ ; turret position
; ------  t2 special  -------- $
	t2_turret:  	0,	$ ; turret position
	t2_lvadr:	0,	$ ; laser video address	
; ------  t3 special  -------- $
	t3_filter:	0,	$ ; H-alpha filter w-l position
	t3_lvadr:	0,	$ ; laser video address	
; ------  t4 special  -------- $
	t4_filter:	0,	$ ; 6374 filter w-l position
	t4_ftemp:	0,	$ ; filter temperature
	t4_kdp1:	0,	$ ; voltage of kdp1
	t4_kdp2:	0,	$ ; voltage of kdp2
	est:		''	$ ; "EST:" end mark
	}

;--------------------------------------------------------------------------
fst.fst_id 	= string(' ',format='(A8)')
fst.date 	= string(' ',format='(A8)')
fst.time 	= string(' ',format='(A8)')
fst.prog	= replicate(string(' ',format='(A8)'),5)
fst.est	 	= string(' ',format='(A4)')
return,fst
end
