;**************************************************************************
;+
; NAME       : magtv1m (procedure)
; PURPOSE    : display multi images of T1
; CALLING SEQUENCE :
;	magtv1m,files,size=size,info=info
; INPUTS :
;     files	-- filename array
; KEYWORD PARAMETERS :
;     size	-- size factor
;     info	-- information array
; OUTPUTS :
;     none
; SIDE EFFECTS : 
; 	display
; MODIFICATION HISTORY :
;	K.Ichimoto, 1993/01/29
;-
;==========================================================================
pro magtv1m,files,size=size,info=info,pos_info=pos_info,title=title

pause='y'
new_window=0
if keyword_set(size) then size=size else size=1
if keyword_set(title) then new_window=1	else title='IDL: magtv1m'
; --------------- open window ---------------- 
wx0=1 	;x-origin of basic window
wy0=80  ;y-origin of basic window
wx=1140 ;x-width of basic window
wy=800	;y-width of basic window
dx=5	;x-interval bitween images
dy=5	;y-interval bitween images

if (!d.window eq -1) or new_window then begin
	window,0,xsize=17,ysize=56
	device,get_window_pos=pw
	wx=pw(0)	&	wy=pw(1)
	window,0,xpos=wx0,ypos=wy0,xsize=wx,ysize=wy,title=title
endif
wx=!d.x_size
wy=!d.y_size
erase

nn=n_elements(files)
print,nn,' files will be displayed'

nkrgeth,files(0),h
nx=fix(size*h(0).nx)	&	ny=fix(size*h(0).ny)
kx=wx/(nx+dx)
ky=wy/(ny+dy)
print,kx,'*',ky,' images displayed'

ans=''
for i=0,nn-1 do begin
	f1data,files(i),h,img,bx,by,bl,dir=dir
	x0 = dx + (nx+dx) * ( i-i/kx*kx ) 
	y0 = wy-(ny+dy) - (ny+dy) * ( i/kx - (i/kx)/ky*ky )
	magtv1,img,bx,by,bl,h=h,x0=x0,y0=y0,/notitle,mag=size
	if keyword_set(info) then begin
		if keyword_set(pos_info) then begin
			x=x0+pos_info(0)	&	y=y0+pos_info(1)
		endif else begin
			x=x0+3	&	y=y0+3
		endelse
		xyouts,x,y,info(i),/dev,charsize=1,color=230
	endif
	if(i+1-(i+1)/(kx*ky)*(kx*ky) eq 0) and (pause eq 'y') then begin
		ret=''
		read,'pause;  hit <RET> to continue (e <RET> to escape) : ',ret
		if( ret eq 'e' ) then goto,endl
		erase
	endif
endfor

endl:
end
