;**************************************************************************
; NAME       : sctlevel (function)
; PURPOSE    : determine scatter light level from img interactively
; CALLING SEQUENCE :
;	scatter=sctlevel(img,spot=spot)
; INPUTS :
;     img
; KEYWORD PARAMETERS :
;     spot 	: spot darkness relative to normal region
; OUTPUTS :
;     sacttered light level
; SIDE EFFECTS : 
;	im g displayed on screen
; 
; MODIFICATION HISTORY :
;	K.Ichimoto, 1992/10/25
;==========================================================================
;
function getspot,img	; return spot minimum intensity
print,'++++ click sunspot area ...'
box_cursor, x0,y0,wx,wy,INIT=0, MESSAGE = 1
spot=min(img(x0:x0+wx-1,y0:y0+wy-1),posi)
x=x0+posi-posi/wx*wx
y=y0+posi/wx
xyouts,x,y,'!I!9n',size=2,alignment=0.5,color=255,/dev
print,'spot intensity=',spot,' (',string(x,y,format='(I3,",",I3)'),')'
return,spot
end

function getbox,img	; return return average normal photosphere intensity
box_cursor, x0,y0,wx,wy,INIT=0, MESSAGE = 1
return,total(img(x0:x0+wx-1,y0:y0+wy-1))/wx/wy
end


function sctlevel,img,q,u,v,spot=spot,dwl=dwl,choice=choice

!order=0
loadct,0
tvscl,img
rspot=0.15
print,'++++++  determin scatter light level  ++++++++'

if keyword_set(spot) then begin
    rspot=spot
    choice=1
endif else begin
    menustr=['Method to Determin Scattered Light Level',	$
	'Assume maximum V as the saturation level for dwl=dwl',	$
	'Assume spot darkness as 15% of the normal region',	$
	'Use sky as scattered light level',	$
	'Enter the ratio of scattered light and the normal region I ',	$
	'Enter the value of scattered light',	$
	'No scatter correction']
    choice=wmenu(menustr,title=0,init=1)
endelse

case choice of 
    1: begin
	if n_params() eq 1 then begin
		print,'V not defined!!'
		return,-1
	endif
	if n_elements(dwl) eq 0 then begin
		read,'Enter wave length position of Lyot filter (mA) : ',dwl
	endif
	end
    2: begin
	spot=getspot(img)
	print,'++++ click normal area ...'
	normal=getbox(img)
	print,'normal photosphere=',normal
	scatter=fix(float(spot-normal*rspot)/(1.-rspot))
	end
    3: begin
	print,'++++ click sky area ...'
	scatter=getbox(img)
	end
    4: begin
	print,'++++ click normel area ...'
	scatter=getbox(img)
	print,'scatter=',scatter
	normal=getbox(img)
	print,'normal photosphere=',normal
	read,'++++ Enter ratio of scatter and normal region => ',rsct
	scatter=fix(normal*rsct)
	end
    5: begin
	read,'++++ Enter scatter light level => ',scatter
	end
    6: begin
	scatter=0
	end
endcase

print,'scatter level =',scatter
imin=min(img)
if imin lt scatter then begin
	print,'scatter < min(img) : replaceed by min(img) '
	scatter=imin
endif

return,scatter

end
