;**************************************************************************
; NAME       : shear (procedure)
; PURPOSE    : get magnetic shear angle from bx,by,bl
; CALLING SEQUENCE :
;	angle = shear( bx,by,bl,bxp=bxp,byp=byp )
; INPUTS :
;     bx,by,bl   -- pol. degree *10000 (q,u,v already devided by img)
; KEYWORD PARAMETERS :
;     bxp,byp    -- potential field
; OUTPUTS :
;     angle  -- angle between (bx,by) and (bxp,byp)
; 
; MODIFICATION HISTORY :
;	K.Ichimoto, 	'93/03/12
;==========================================================================
;function shear,bx,by,bl,bxp=bxp,byp=byp

s=size(bl)
nx=s(1)	&	ny=s(2)
pfact=nx/64
cff1n,rebin(bl,nx/pfact,ny/pfact),bxp,byp
bxp=rebin(bxp,nx,ny)
byp=rebin(byp,nx,ny)

ii=where( bx eq 0 and by eq 0 , count )
if count ne 0 then bx(ii)=1
ii=where( bxp eq 0 and byp eq 0 , count )
if count ne 0 then bxp(ii)=1
bt=sqrt(float(bx)^2+float(by)^2)
btp=sqrt(float(bxp)^2+float(byp)^2)
cs=(bx*bxp+by*byp)/bt/btp
angle=acos(cs<1.0>(-1.0))*180./!pi
;angle(where(bt lt 3000.))=0.
es=angle*bt	; energy for sheared mag. field
;return,angle

end
