;********************************************************************
;NAME       : veltv1 (procedure)
;FUNCTION   : display velocity field and draw intensity contour
;						K.I. '92/07/27
;====================================================================
pro veltv1,img0,s,h=h,spline=spline,order=order,save=save,cal=cal,seq=seq, $
	trend=trend
; img0    : image (*,*) or i,s as (*,*,2)
;  h      : header array
; spline  : if exist draw spline contour
; order   : if exist, up-down direction is opposit
; save=filename  : save tvrd to file 
; /cal	  : calibrated data	Bl=>q,  Bt=>u, th=>v
; /seq	  : display seq.no
; /trend  : remove linear trend

;-------------------   parameters   --------------------
pcmax=0.15 	; max. of dopp. signel for display
pix1=pix_size('ft1')	; arc sec. for 1 pixel
nstep=10	; step of linear pol. bar plot
lfact=0.4	; factor for length of linear pol. bar
rfact=10000.	; ratio-fact  q,u,v -> q,u,v/rfact
imgmin=0.2	; minimum level to display pol.
cmpfact=2	; compress factor for drawing contour


if keyword_set(spline) then spline=1 else spline=0
if n_params() eq 0 then begin
	icmax=!d.table_size-4
	lct_rb,icmax=icmax
	return
endif
ss=size(img0)
nx=ss(1)	& ny=ss(2)
if ss(0) eq 2 then img=img0 else img=img0(*,*,0)
if n_params() eq 1 then begin	; case only img0 given
	if ss(0) eq 2 then begin	; case img0 is 2-dimensional
		img=img0
	endif else begin	; case img0 is 3-dimensional
		img=img0(*,*,0)
		if ss(3) eq 2 then s=img0(*,*,1)
	endelse
endif


;--------------------   image display   ---------------------
x0=50
y0=50
wwx=nx+2*x0	;expected window size
wwy=ny+y0+40
if(wwx gt !d.x_size) or (wwy gt !d.y_size) then window,!d.window,	$
	xsize=wwx,ysize=wwy

icmax=!d.table_size-6
frame_color=!d.table_size-1
contour_color=!d.table_size-2
lpol_color=!d.table_size-3
sky_color=!d.table_size-4
tvlct,0,0,0,frame_color		; color of frame in icontour
tvlct,0,255,0,contour_color	; color of image contour
tvlct,0,0,0,lpol_color		; color of linear pol.
tvlct,0,0,0,sky_color		; color of sky
tvlct,255,255,255,0		; back ground
lct_rb,icmax=icmax
pos=[x0,y0,x0+nx,y0+ny]

!order=1
if keyword_set(order) then !order=0

if keyword_set(cal) then begin 
	;magtv1c,img,q,u,v,h=h,spline=spline
endif else begin

;--------------------   draw   -----------------------------   
erase
if keyword_set(trend) then 	$
	s = s-rebin(surface_fit(rebin(s,nx/4,ny/4),2),nx,ny)
;rb=byte( icmax*(s/rfact+pcmax)/(2*pcmax) )
smean=total(s)/nx/ny
rb=byte( icmax*((s-smean)/rfact+pcmax)/(2*pcmax) )
imgmax=max(img)
sky=where(img lt imgmax*imgmin, nsky)
if(nsky ne 0l) then rb(sky)=lpol_color
tv,rb,x0,y0,/dev

img=rebin(img,nx/cmpfact,ny/cmpfact)
if !order eq 1 then mirror,img
iav=(total(img(*,0))+total(img(*,ny/cmpfact-1)))/(nx/cmpfact)/2
contour,smooth(img,2)/float(iav),level=[0.3,0.5,0.7,0.9],	$		
	pos=pos,/dev,xstyle=1+4,ystyle=1+4,color=contour_color,	$
	xrange=[0.,nx/cmpfact],yrange=[0.,ny/cmpfact],/noerase,	$
	thick=1,spline=spline

endelse

;--------------------   frame display   ---------------------
if keyword_set(h) then begin
	if n_elements(h) gt 1 then begin
		timeint,h,time1,time2,date1,date2
		time_sys=h(0).time_sys
		g_x=h(0).g_x
		g_y=h(0).g_y
		seqno=h(0).seqno
		hql=h(0)
	endif else begin
		date1=h.date1
		time1=h.time1
		date2=h.date2
		time2=h.time2
		time_sys=h.time_sys
		g_x=h.g_x
		g_y=h.g_y
		seqno=h.seqno
		hql=h
	endelse
	if keyword_set(save) then begin
		hql.nbyt=1
		s=size(img)
		hql.nx=s(1)	& hql.ny=s(2)
		hql.date1=date1
		hql.date2=date2
		hql.time1=time1
		hql.time2=time2
		hql.nset=1
		hql.value='map'
		wshow
		rd=tvrd(x0,y0,wx,wy)
		nkrsave,save,hql,rd,/nocheck
		print,'display saved in '+save
	endif
	if time_sys eq 'JST' then begin
		jst2ut,date1,time1
		jst2ut,date2,time2
	endif
	title1='!N!3Solar Flare Telescope (MTK) : velocity field'
	title2=date1+'  '+strmid(time1,0,8)+'-'+strmid(time2,0,8)+' UT' $
		+'    '+str_gpos(g_x,g_y)
endif else begin
	title1=''
	title2='!N!3Solar Flare Telescope (MTK) : velocity field'
endelse

xyouts,x0,y0+ny+20,title1,/dev,charsize=1.7
xyouts,x0,y0+ny+5,title2,/dev,charsize=1.5
if keyword_set(seq) then $
	xyouts,x0+nx-35,y0+ny+5,	$
	'('+string(seqno,format='(i3)')+')',/dev,charsize=1.5
xrange=[0.,pix1*(nx-1)]
yrange=[0.,pix1*(ny-1)]
tickv=[0,100,200,300,400,500,600]
plot,[0,0],[0,0],/nodata,pos=pos,/dev,/noerase,color=frame_color,	$
	xrange=xrange,yrange=yrange,xstyle=1,ystyle=1,	$
        xticks=fix(nx*pix1/100.),xtickv=tickv,xminor=10,xticklen=0.03, $ 
        yticks=fix(ny*pix1/100.),ytickv=tickv,yminor=10,yticklen=0.03, $ 
	xtitle='arc sec.',ytitle='arc sec.'

end
