;+
;
;NAME       : mtv (pro)
; PURPOSE :
;	multi tv
; CATEGORY :
;	idl/lib/nkr
; CALLING SEQUENCE :
;	mtv,data,data2,info=info,step=step,size=size,start=start,dx=dx,dy=dy,	$
;		scale=scale,v=v,pos_info=pos_info,order=order,continue=continue, $
;		img_no=img_no,get=get,image=image,h=h,title=title,next=next,	$
;		charsize=charsize,title1=title1
; INPUTS :
;	data    : data(*,*,*) or NKR type filenames
; OPTIONAL INPUT PARAMETERS : 
;	data2
; KEYWORD PARAMETERS :
;	info	: comment array
;	pos_info: position of info (default=[0,0])
;	step	:
;	size	: magnification fact
;	start	: starting no.
;	dx,dy	: space between images
;	scale	: scaling, 1,2 or 3
;			1. tvscl, linear
;			2. tvscl, log
;			3. tv
;			4. ..
;			else. tv,a/scale
;	v	: display direction to vertical
;	order	: same as tv or tvscl procedure
;	continue: do not pause at full screen
;	img_no	: image no. for nkrget when data is filename array
;	get	: get image array when data is filename array
;	image	: image array to get nkr images
;	h	: header array to get nkr headers
;	title	: title of new window 
;	title1	: title in the window 
;	next	: give strting position (count)
;	charsize: size of information characters
; OUTPUTS :
;	none
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;	If title keyword is set, new full window is always created.
; MODIFICATION HISTORY :
;	K.I. '93/01/07	
;	K.I. '93/01/10	; pos_info
;	K.I. '93/01/25	; v, continue, order keyywords
;	K.I. '93/01/26	; 2 kind of images can be displayed
;	K.I. '93/01/29	; fact --> size
;	K.I. '93/01/30	; title keyword, full window
;	K.I. '93/05/09	; /get -> nng
;	K.I. '93/06/21	; dx,dy keyword
;	K.I. '93/08/14	; next keyword
;	K.I. '93/08/21	; charsize keyword
;	K.I. '93/08/25	; scale
;	K.I. '94/01/25	; files1,files2
;-
;----------------------------------------------------------------------
pro mtv,data,data2,info=info,step=step,size=size,start=start,dx=dx,dy=dy,	$
	scale=scale,v=v,pos_info=pos_info,order=order,continue=continue,	$
	img_no=img_no,get=get,image=image,h=h,title=title,next=next,	$
	charsize=charsize,title1=title1

new_window=0
if keyword_set(step) then step=step else step=1
if keyword_set(size) then size=size else size=1
if keyword_set(start) then start=start else start=0
if keyword_set(scale) then scale=scale else scale=1
if keyword_set(v) then direc='v' else direc='h'
if keyword_set(order) then order=order else order=!order
if keyword_set(continue) then pause='n' else pause='y'
if not keyword_set(img_no) then img_no=1
if keyword_set(title) then new_window=1	else title='IDL: mtv'
if n_elements(dx) ne 0 then dx=dx	else dx=5
if n_elements(dy) ne 0 then dy=dy	else dy=5
if not keyword_set(charsize) then charsize=1

dmin=0
dmax=0
if scale eq 4 then begin
	fact=10000
	amax=0.2
	icmin=1
	icmax=!d.table_size
	lct_rb,icmax=icmax
endif

; --------------- open window ---------------- 
wx0=1 	;x-origin of basic window
wy0=80 ;y-origin of basic window
wx=1140 ;x-width of basic window
wy=820	;y-width of basic window
ddx=2	;x-interval bitween images in a set
ddy=2	;y-interval bitween images in a set

if (!d.window eq -1) or new_window then fullwindow,0,title=title
wx=!d.x_size
wy=!d.y_size
if keyword_set(title1) then wy=wy-fix(charsize*20)

; ----------------  determin nx & ny  ---------------
s=size(data)
if s(s(0)+1) eq 7 then begin
	nkrgeth,data(0),h
	nx=fix(h.nx*size)	&	ny=fix(h.ny*size)	&	nn=s(3)
	nng=(nn-start-1)/step+1
	mode='file'
	if keyword_set(get) then begin
	    if h.nbyt eq 1 then begin
		image=bytarr(nx,ny,nng) & type='BYTARR'
	    endif else if h.nbyt eq 2 then begin
		image=intarr(nx,ny,nng)	& type='INTARR'
	    endif else if h.nbyt eq 4 then begin
		if h.data_type eq 3 then begin
			image=fltarr(nx,ny,nng)	& type='FLTARR'
		endif else begin
			image=lonarr(nx,ny,nng)	& type='LONARR'
		endelse
	    endif else if h.nbyt eq 8 then begin
		image=dblarr(nx,ny,nng)	& type='DBLARR'
	    endif
	    h=replicate(h,nng)
	endif
endif else begin
	nx=fix(s(1)*size)	&	ny=fix(s(2)*size)
	if s(0) ge 3 then nn=s(3) else nn=1
	mode='data'
endelse
sx=nx+dx	&	sy=ny+dy
if n_params() eq 2 then begin
	s=size(data2)
	if s(s(0)+1) eq 7 then begin
		nkrgeth,data2(0),h2
		nx2=fix(h2.nx*size)	&	ny2=fix(h2.ny*size)	&	nn2=s(3)
	endif else begin
		nx2=fix(s(1)*size)	&	ny2=fix(s(2)*size)	&	nn2=s(3)
	endelse
	;if s(0) ge 3 then nn2=s(3) else nn2=1
	if(direc eq 'h') then begin
		sx=max([nx,nx2])+dx
		sy=ny+ny2+ddy+dy
	endif else begin
		sx=nx+nx2+ddx+dx
		sy=max([ny,ny2])+dy
	endelse
	nn=min([nn,nn2])
endif else begin
	nx2=0
	ny2=0
endelse
kx=wx/(sx)
ky=wy/(sy)
if not keyword_set(next) then begin
	erase
	print,kx,' *',ky,' images displayed'
endif

; ----------  display routine  --------------
if keyword_set(next) then count=next else count=1
if keyword_set(title1) then xyouts,dx,wy,title1,size=charsize*1.5,/dev

for n=start,nn-1,step do begin
	;print,n
	if mode eq 'data' then  a=data(*,*,n)  $
	else nkrget,data(n),h1,a,img_no=img_no(0)
	count0=count-1
	if(direc eq 'v') then begin
		x0 = dx + sx * ( count0/ky - (count0/ky)/kx*kx ) 
		y0 = wy-sy - sy * ( count0-count0/ky*ky )
	endif else begin
		x0 = dx + sx * ( count0-count0/kx*kx ) 
		y0 = wy-sy - sy * ( count0/kx - (count0/kx)/ky*ky )
	endelse
	if(size ne 1) then a=congrid(a,nx,ny,/Interp)
	if mode eq 'file' and keyword_set(get) then begin
		image(*,*,count0)=a
		h(count0)=h1	&	h(count0).nx=nx	&	h(count0).ny=ny
	endif
	if dmin eq 0 and dmax eq 0 then begin
	    case scale of
		 1: 	tvscl,a,x0,y0,order=order
		 2: 	tvscl,alog10(a>1),x0,y0,order=order
		 3: 	tv,a,x0,y0,order=order
		 4: begin
			rb=byte( (icmax-icmin)	$	
				*(((float(-a)/fact)>(-amax)<amax)+amax)  $
				/(2.*amax) ) + icmin
			tv,rb,x0,y0,order=order
		    end
		else: begin
			tv,a/scale,x0,y0,order=order
		    end
	    endcase
	endif else begin
	    b=(float(a-dmin)/(dmax-dmin))>0<1
	    case scale of
		 1: 	tv,b*255,x0,y0
	    endcase
	endelse

	; -------  info display: chose your favourit ----
	if keyword_set(info) then begin
		if keyword_set(pos_info) then begin
			x=x0+pos_info(0)	&	y=y0+pos_info(1)
		endif else begin
			x=x0+3	&	y=y0+3
		endelse
		xyouts,x,y,info(n),size=charsize,/device
	endif

	if n_params() ge 2 then begin
	    if mode eq 'data' then  a=data2(*,*,n)  $
	    else nkrget,data2(n),h1,a,img_no=img_no(0)
 	    if(direc eq 'v') then x0 = x0+nx+ddx else y0 = y0+ny+ddy
	    if(size ne 1) then a=congrid(a,nx2,ny2,/Interp)
	    if dmin eq 0 and dmax eq 0 then begin
	      case scale of
		 1: 	tvscl,a,x0,y0,order=order
		 2: 	tvscl,alog10(a>1),x0,y0,order=order
		 3: 	tv,a,x0,y0,order=order
	      endcase
	    endif else begin
	      b=(float(a-dmin)/(dmax-dmin))>0<1
	      case scale of
		 1: 	tv,b*255,x0,y0
	      endcase
	    endelse
	endif
	
	if(count-count/(kx*ky)*(kx*ky) eq 0) and (pause eq 'y') then begin
		ret=''
		read,'pause;  hit <RET> to continue (q <RET> to escape) : ',ret
		if( ret eq 'q' ) then goto,endl
		if( ret eq 'p' ) then begin
			print,'take hard copy of screen.....'
			wwx=dx+sx*kx
			wwy=dy+(ny+dy)*ky
			a=tvrd(0,wy-wwy,wwx,wwy)
			set_plot,'ps'
			wxps=23
			wyps=float(wwy)/float(wwx)*wxps
			device,/landscape,filename='../tmp.ps',	$
				xsize=wxps,ysize=wyps
			tvscl,a
			date=h.date1
			xyouts,0,1.02,'lookm: '+date,/normal,size=1.5
			device,/close
			spawn,'lpr -h ../tmp.ps'
			set_plot,'x'
		endif
		erase
		if keyword_set(title1) then 	$
			xyouts,dx,wy,title1,size=charsize*1.5,/dev
	endif
	count=count+1

endfor
next=count

endl:

end
