;+
;NAME       : ncongrid (pro)
; PURPOSE :
;	shrink/expand multiple images in norikura format
; CATEGORY :
;	idl/lib/nkr
; CALLING SEQUENCE :
;	ncongrid,files,xfact=xfact,yfact=yfact,nx=nx,ny=ny,	$
;		outfiles=outfiles,fact=fact
; INPUTS :
;	files    : NKR type filenames
; OPTIONAL INPUT PARAMETERS : 
; KEYWORD PARAMETERS :
; OUTPUTS :
;	none
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;	If title keyword is set, new full window is always created.
; MODIFICATION HISTORY :
;	'91/01/29   k.i.
;	'92/11/12   k.i.
;	'92/11/28   k.i.
;	'93/02/04   k.i.
;	'93/02/06   k.i.   'temp' --> outfiles(i)
;-
;----------------------------------------------------------------------
pro ncongrid,files,xfact=xfact,yfact=yfact,nx=nx,ny=ny,	$
	outfiles=outfiles,fact=fact

nn=n_elements(files)
if nn eq 0 then begin
	file=''
	ofile=''
	read,'enter (first) file name : ',file
	read,'how many files do you shrink/expand ? : ',nn
	files=file
	for i=0,nn-1 do files=[files,nextfile(files(i),next=1)]
endif
if not keyword_set(xfact) and not keyword_set(yfact) and $
	not keyword_set(fact) and $
	(not keyword_set(nx) or not keyword_set(ny) ) then begin
	print,'Do you enter  1. x-factor & y-factor '
	read, '	      2. x-size   & y-size    ==> ',mode
	nx=1	&	ny=1
	case mode of
	    1: 	read,'enter x-factor & y-factor : ',xfact,yfact
	    2: 	read,'enter nx & ny : ',nx,ny
	endcase
endif else if keyword_set(fact) then begin
	xfact=fact	&	yfact=fact	& mode=1
endif else if keyword_set(xfact) and keyword_set(yfact) then begin
	mode=1
endif else begin
	mode=0
endelse

if not keyword_set(outfiles) then begin
	outfile=''
	read,'enter (first) output file name ( if same <RET> ) : ',outfile
	if( outfile eq '' ) then begin
		outfiles=files
	endif else begin
		outfiles=outfile
		for i=0,nn-1 do outfiles=[outfiles,nextfile(outfiles(i),next=1)]
	endelse
endif

count=1
for i=0,nn-1 do begin
    nkrget,files(i),img_no=-1,nset=nset
    for n=1,nset do begin
	nkrget,files(i),h,a,img_no=n
	print,files(i)
	print,'nx=',h.nx,'     ny=',h.ny,'     nf=',h.nf,'    nbyt=',h.nbyt
	ysize=h.ny*h.nf
	if mode eq 1 then begin
		nx=fix(h.nx*xfact)
		ny=fix(h.ny*yfact)
	endif
	if h.nx ne nx or h.ny ne ny then begin
	    if (nx-nx/h.nx*h.nx eq 0) or (h.nx-h.nx/nx*nx eq 0) and $
	       (ny-ny/h.ny*h.ny eq 0) or (h.ny-h.ny/ny*ny eq 0) then begin
		print,'REBIN used'
		b=rebin(a,nx,ny)
	    endif else begin
		print,'CONGRID used'
		b=congrid(a,nx,ny,/interp)
	    endelse
	    h.nx=nx
	    h.ny=ny
	    if nset gt 1 then	begin
		if outfiles(i) eq files(i) then begin
		    nkrsave,'temp',h,b,/nocheck,/append
		endif else begin
		    nkrsave,outfiles(i),h,b,/nocheck,/append
		endelse
	    endif else begin
		nkrsave,outfiles(i),h,b,/nocheck
	    endelse
	endif
    endfor
    if nset gt 1 and outfiles(i) eq files(i) then $
	spawn,'mv temp '+outfiles(i)
endfor

end
