pro nkr3fits, dir, select = sel, option = op, scale = scale
;+
;********************************************************************
;NAME       : nkr3fits (procedure)
;FUNCTION   : Read norikura formatted data & write out as FITS files.
;PROGRAMMER : Andrew L. Stanger   9 May 1994
;====================================================================
; dir : directory
; sel : file selection wild card
; op  : option of ls command  (for ex. -tr   -- time sequence)
; scale : option to expand scale (min -> max expand to 0 -> 249).
;-

; Set current directory as the default if 'dir' is not specified.

if (keyword_set (dir))  then dir0 = dir  else dir0 = '.'

if (strmid (dir0, strlen (dir0) - 1, 1) eq '/') $
   then dir1 = strmid (dir0, 0, strlen (dir0) - 1) $
   else dir1 = dir0

; Save current directory and move to specified directory.

spawn, 'pwd', dir_old	&	dir_old = dir_old (0)
cd, dir0

; List contents of directory.

com = '/bin/ls -F '
if (keyword_set (op)) then com = com + op + ' '

;com = com + dir1

if (keyword_set (sel)) then  com = com + sel
print, com
spawn, com, files

; List data file header information.

; Write image to disk as a FITS file.

nn = n_elements (files)
hlist = 'filename   date   time1    time2   sys   EW   NS   g0'
nkr = 1

for i = 0, nn - 1 do begin
   ;nkrgeth, dir1 + '/' + files (i), h, nkr = nkr
   nkrget, dir1 + '/' + files (i), nkrhdr, img
   on_ioerror, notnkr
   if (nkr eq 1) then begin
      hlist1 = files (i) + '  ' + nkrhdr.date1 + ' '	$
             + strmid (nkrhdr.time1, 0, 8) + '-'		$
             + strmid (nkrhdr.time2, 0, 8) + ' '		$
             + nkrhdr.time_sys
      if (strmid (nkrhdr.telescope, 0, 2) eq 'ft') then $
      hlist1 = hlist1 + '   ' + str_gpos (h.g_x, h.g_y) + string (h.l0)
      hlist  = [hlist, hlist1]

      namfits = files (i) + '.fts'
      date_obs = nkrhdr.date1
      time_obs = nkrhdr.time1
      origin   = nkrhdr.observatory
      telescop = nkrhdr.telescope
      instrume = strcompress (string (nkrhdr.icam), /remove_all)
      object   = 'sun'
      type_obs = 'H-alpha'
      comment  = 'created using IDL procedure nkr2fits.pro'


      ; odcenter, img, cx, cy, radius			; find disk center.
      ; print,'cx/cy/radius: ', cx, cy, radius
      ; sx = nkrhdr.nx / 2 - cx				; compute x shift
      ; sy = nkrhdr.ny / 2 - cy				; compute y shift
      ; img = shift (img, sx, sy)			; center disk in array.

      ; scale intensities.

      if (keyword_set (scale)) then $
      begin
         minval = min (img)				; get min intensity
         maxval = max (img)				; get max intensity
         range   = maxval - minval
         ; lowval  = minval + fix (0.10 * range)
         lowval  = minval
         highval = maxval
         clipval = 249
         bzero = float (lowval)
         bscale = float (highval - lowval) / float (clipval)
	 print, 'min/max: ', minval, maxval, '   low/high: ', lowval, highval
         img = bytscl (img, min = lowval, max = highval, top = clipval)

         write_fits, img, namfits, date_obs = date_obs, time_obs = time_obs,$
                     bscale = bscale, bzero = bzero,$
                     origin = origin, telescop = telescop, instrume = instrume,$
		     object = object, type_obs = type_obs, history = history,$
		     comment = comment
      end  $
      else $
         write_fits, img, namfits, date_obs = date_obs, time_obs = time_obs,$
                     origin = origin, telescop = telescop, instrume = instrume,$
		     object = object, type_obs = type_obs, history = history,$
		     comment = comment

   endif
notnkr:
endfor
print, hlist

cd, dir_old

end
