;+
;
; NAME       : nkrhead (function)
; PURPOSE    :
;	define norikura type header structure
; CATEGORY :
;	idl/lib/nkr
; CALLING SEQUENCE :
;	h = nkrhead()
; INPUTS :
; MODIFICATION HISTORY :
;	K.I. '93/01/31	
;-

function nkrhead
; ================  define Norikura header structure  ===============
h = { header,			$
; -----  header & file informaiton  ----- $
	hdr_id: 	'',	$ ; header identifier "HDR:0100"
	one: 		1,	$ ; one in this system		
	offset:		0,	$ ; header offset 0 or 512
	file_name:	'',	$ ; file name
	file_form:	'',	$ ; file format
; -----  data type information  -----	$
	nset:		1,	$ ; number of frame set in file
	nbyt:		1,	$ ; byte of data	 1,2,4 or 8
	data_type:	1,	$ ; 1.signed, 2.unsigned, 3.new pip4, 4.float
	nx:		512,	$ ; size of data	in x-direc.
	ny:		480,	$ ;      "	in y-direc.
	nf:		1,	$ ; frame number in the frame set
;  -----  observational basic parameters----	$
	observatory:	'',	$ ; "MTK ", "NKR ",,,
	telescope:	'',	$ ; telescope
	icam:		0,	$ ; camera no.
	obs_mode:	0,	$ ; 1.data, 2.dark, 3.flat
	time_sys:	'',	$ ; "JST", "UT",,,
	date1:		'',	$ ; date 	"yy/mm/dd"
	time1:		'',	$ ; start time	"hh:mm:ss.sss"
	date2:		'',	$ ; date	"yy/mm/dd"
	time2:		'',	$ ; end time	"hh:mm:ss.sss"	
	seqno:		1,	$ ; sequence no.
	exp:		1,	$ ; exposuer time (msec)
	integ:		1,	$ ; total integration no.
	integ1:		0,	$ ; integration no. for 1set
	prog:		'',	$ ; program name & version
	obs_id:		'',	$ ; observation id
	o_dummy:	'',	$ ; dummy
;  -----  explanation of the data  -----	$
	correct:	1,	$ ; status of data correction
	value:		'',	$ ; [4] physical quantity
	scale:		'',	$ ; [14] scale   (formatted real)
	zero:		'',	$ ; [14] ( " ) true=scale*data-zero
	unit:		'',	$ ; [10] unit of above value
	x_ax:		'',	$ ; [10] quantity of x-axis
	x1:		'',	$ ; [14] x-value at i=1 (formatted real)
	dx:		'',	$ ; [14] x-increment
	y_ax:		'',	$ ; [10] quantity of y-axis
	y1:		'',	$ ; [14] y-value at j=1 (formatted real)
	dy:		'',	$ ; [14] y-increment
	com:		'',	$ ; [50] comment about the data	
;  -----  device dependent parameters  ---	$
	pix_x1:		0,	$ ; starting x-pixel on CCD
	pix_y1:		0,	$ ; starting x-pixel on CCD
	d_parm: intarr(23),	$ ; parameters for devices
;  -----  telescope dependent parameters ----	$
	hangle:		0L,	$ ; hour angle
	delta:		0L,	$ ; declination
	l0:		0,	$ ; central brightness of sun
	g_on:		0,	$ ; guide on/off  0;off, 1;on
	g_balx:		0,	$ ; guide balance x
	g_baly:		0,	$ ; guide balance y
	g_x:		0,	$ ; guide table x-position
	g_y:		0,	$ ; guide table y-position
	t_parm: intarr(10),	$ ; telescope parameter
	t_dummy:	'',	$ ; [10] dummy
;  -----  parameters for multi frames in frame set  ----- $
	z_ax:		'',	$ ; [10] frame step (z) "TIME", "X"
	z1:		'',	$ ; [14] z-value of first frame	
	dz:		'',	$ ; [14] z-increment
	fs_parm:  intarr(6),	$ ; frame set parameters
	fs_com:		'',	$ ; [50] frame set comment
	dummy:		'',	$ ; [8] dummy
	eoh:		''	$ ; [4] end of file "EOH:"
	}

;----------------------------------------------------------------------
; <<<  remarks  >>>
; "frame" means an image data whose size is nx*ny*nbyt
; "frame set" may containes one or many frames, but each frame has 	
;         common nx, ny, data_type etc. Each frame set has a header 	
;	   on the top.							
; "file" may containes one or many frame set				
;                 							
; one:    1 in the computer system which write this header.	 	
;	   If one=256, byte reversal is necessary.			
; offset: If the header is over written on the top of the data, 	
;	   offset=HLENGTH (512),  else offset=0.			
; nset:   It has meaning only when it appears in the first header 	
;	   in the file							
; data_type:  	1. signed int/char,  2. unsigned int/char  3. real
;	   If data_type=-1 or -2 and nbyt=2, byte reversal is necessary 
;	   for the data							
; correct: 1. raw data, 2. dark level is subtracted, 			
;	   3. after flat field correction
;	   4. image ratio (polarization degree)
;	   >100. magnetic calibrated data			
;----------------------------------------------------------------------
;		     <10 cm>	   <25 cm>		  <flare 1,4>	
;	d_parm[0] -- turret1	  integ_mode(get1p)	  integ_mode
;	d_parm[1] -- turret2	  pm1-pos1( " )		  kdp1v1
;	d_parm[2] -- focus_o	  pm1-pos2( " )		  kdp1v2
;	d_parm[3] -- focus_c  	  pm1-offset( " )	  kdp1offset
;	d_parm[4] -- setting no.  pm2-pos1( " )		  kdp2v1
;	d_parm[5] -- obj.potentio pm2-pos2( " )		  kdp2v2
;	d_parm[6] -- ccd pos(dumy)pm2-offset( " )	  kdp2offset
;	d_parm[7] --              image sizel(get1 etc.)  kdpwait(ms)
;	d_parm[8] --              			  turret (f1)
;	d_parm[10]--					  dwl   (ftcalib)
;	d_parm[11]--					  scatter     (")
;	d_parm[12]--					  kdpfact*100 (")
;	d_parm[13]--					  kdpfact2*100(")
;	d_parm[14]--					  bmag        (")
;	d_parm[15]--					  blfact*1000 (")
;	t_parm[0] -- sky				  avr(flt1)
;	t_parm[1] -- i0				  	  avr(flt2)
;	t_parm[2] --  				  	  temp0 *100 (C)
;	t_parm[3] -- 				  	  temp1     "
;	t_parm[4] -- 				  	  t1_ftemp  "
;	t_parm[5] -- 				  	  t4_ftemp  "
;	fs_parm[0] -              x-step(raster)
; ====================================================================	

h.hdr_id 	= string(' ',format='(A8)')
h.file_name 	= string(' ',format='(A12)')
h.file_form 	= string(' ',format='(A4)')
h.observatory 	= string(' ',format='(A4)')
h.telescope	= string(' ',format='(A4)')
h.time_sys 	= string(' ',format='(A4)')
h.date1 	= string(' ',format='(A8)')
h.time1 	= string(' ',format='(A12)')
h.date2 	= string(' ',format='(A8)')
h.time2 	= string(' ',format='(A12)')
h.prog 		= string(' ',format='(A10)')
h.obs_id 	= string(' ',format='(A10)')
h.o_dummy 	= string(' ',format='(A6)')
h.value 	= string(' ',format='(A4)')
h.scale 	= string(' ',format='(A14)')
h.zero 		= string(' ',format='(A14)')
h.unit 		= string(' ',format='(A10)')
h.x_ax 		= string(' ',format='(A10)')
h.x1 		= string(' ',format='(A14)')
h.dx 		= string(' ',format='(A14)')
h.y_ax 		= string(' ',format='(A10)')
h.y1 		= string(' ',format='(A14)')
h.dy 		= string(' ',format='(A14)')
h.com 		= string(' ',format='(A50)')
h.t_dummy 	= string(' ',format='(A10)')
h.z_ax 		= string(' ',format='(A10)')
h.z1 		= string(' ',format='(A14)')
h.dz 		= string(' ',format='(A14)')
h.fs_com 	= string(' ',format='(A50)')
h.dummy 	= string(' ',format='(A8)')
h.eoh	 	= string(' ',format='(A4)')
return,h
end
