
;+
;
; NAME       : pix_size (function)
; PURPOSE    :
;	return pix1 scale (arc sec.) for the observational data
; CATEGORY :
;	idl/lib/nkr
; CALLING SEQUENCE :
;	pix1 = pix_size(tel)
; INPUTS :
;	tel   : telescope name, 'ft1', 'ft3', 'c10', 'sxt', 'c25b'
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :	none
; OUTPUTS :		none
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
; MODIFICATION HISTORY :
;	K.I. '93/01/22	
;	K.I. '93/05/06	append monochro	
;	K.I. '93/09/03	append c25b	
;-

;---------------------------------------------------------------------
function pix_size,tel

pix1_ft1=0.660		; determined by stopping telescope tracker
pix1_ft3=0.608		;    "        on 22-Jan.1993
pix1_c10=5.23
pix1_sxt=2.46020
pix1_mon=4.47262	; monochro, determined for 'ha921102.0431'
pix1_c25b=1.00		; norikura project_b by hair line

if n_elements(tel) le 0 then begin
	print,'   1. Flare Telescope Mag. Data'
	print,'   2. Flare Telescope Ha Data'
	print,'   3. Norikura 10-cm coronagraph'
	print,'   4. Monochro full disk Ha'
	print,'   5. SXT full resolution'
	print,'   6. Norikura 25-cm, project-B'
	read,'               Enter Telescope No. : ',n
	case n of
	    1 : tel='ft1'
	    2 : tel='ft3'
	    3 : tel='c10'
	    4 : tel='mon'
	    5 : tel='sxt'
	    6 : tel='c25b'
	endcase
endif
case tel of
	"ft1":	pix1=pix1_ft1
	"ft3":	pix1=pix1_ft3
	"c10":	pix1=pix1_c10
	"mon":	pix1=pix1_mon
	"sxt":	pix1=pix1_sxt
	"c25b":	pix1=pix1_c25b
	else : begin
		print,'Telescope ',tel,' not defined in pix_size.pro !!'
		pix1=0
		end
endcase

return,pix1

end
