pro shownkr, dir0
;+
;
; NAME       : shownkr (main program)
; PURPOSE    :
;	display NAOJ catalog data in directory 'dir'.
; CATEGORY :
;	~stanger/idl/lib/naoj/nkr
; CALLING SEQUENCE :
;       shownkr
;	shownkr, dir0
; INPUTS :
; OPTIONAL INPUT PARAMETERS : 
;	dir0 :	main directory path where NAOJ images are located.
; KEYWORD PARAMETERS :
; OUTPUTS :
;	none
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
; MODIFICATION HISTORY :
;	K.I. '92/05/11
;	K.I. '92/05/11		minor change 
;	K.I. '92/06/30		include okayama
;	K.I. '93/01/22		pix1 = pix_size ('ft1')
;	K.I. '93/04/10		lsmenu ( nnmax=310)
;	K.I. '93/05/22		lsmenu ( nnmax=310)
;	K.I. '93/11/20		lsmenu ( nnmax=nnmax) for monochro
;	T.S. '93/12/21		OAO magnetogram by magplot
;	Andrew L. Stanger	HAO/NCAR, Boulder, Colorado
;				add 'dir' parameter.
;-

;====================================================================
if (keyword_set (dir0)) then dir0 = dir0 + '/'  else  dir0 = './'

;pix1 = 0.6738    ; arc sec. for 1 pixel
pix1 = pix_size ('ft1')				; arc sec. for 1 pixel

ff = findfile ('/home/guest/credit.doc',count=count)
if (count ne 0) then $
   spawn,'clear; cat /home/guest/credit.doc'

top:

; dir0 = '/picard/d/stanger/yoh/naoj/'
dirtmp = '/picard/d/tmp'
dir = ''
spawn, 'pwd', dir_old	&	dir_old = dir_old (0)
menustr = ['display NAOJ optical data',	$
	'magnetic data of flare telescope',	$
	'Ha full disk image of Mitaka',	$
	'Norikura data (HeI10830, corona)',	$
	'Okayama magnetic data',	$
	'exit']
kind = wmenu (menustr, title=0)
case kind of

1: begin		; --------------  flare telescope  --------------------
   cd, dir0 + 'flare/'
   mode = 1				; 0-magcont, 1-magtv1, 2-magtv2 
   fnam0 = ''
   while 1 do begin
      case mode of
         0:   ret = ['<red-blue>', '<spot+cont>', '<printout>', '<PostScript>']
         1:   ret = ['<contour>', '<spot+cont>', '<PostScript>']
         2:   ret = ['<contour>', '<red-blue>', '<PostScript>']
      endcase
      ret = [ret, '[return]', '[exit]']
      device, get_screen_size = ws
      if (ws (0) eq 1024 and ws (1) eq 768) then nnmax = 308 else nnmax = 700
      fnam = lsmenu_nkr (dir, ret = ret, x = 2, y = 85, nnmax = nnmax)

      ;  ---------  exit  ------------

      case fnam of
         '[exit]': begin
            if h(1).correct ge 100 then begin
               bx = q	&	by = u	&	bl = v
               print, '>>>>> Magnetic images are stored in img,  bx,  by,  bl '
            endif else begin
               print, '>>>>> Magnetic images are stored in img,  q,  u,  v '
            endelse
               print, '>>>>> Information are in Norikura type header "h(4)"'
               print, '>> Electric currents can be seen by typing ".run jnft1" <<'
               file = dir0 + 'flare/' + fnam0
               cd, dir_old
               stop
               end

         '[return]': begin
            cd, dir0
            goto, top
            end

         ;  ---------  change display mode  ----------

         '<contour>' : begin
            if (fnam0 ne '') then $
	       magcont, img, q, u, v, h = h, mag = magnify, bin = bin
            mode = 0
            end

        '<red-blue>' : begin
           if (fnam0 ne '') then $
	      magtv1, img, q, u, v, h = h, magnify = magnify, bin = bin
           mode = 1
           end

         '<spot+cont>' : begin
            if (fnam0 ne '') then $
	       magtv2, img, q, u, v, h = h, magnify = magnify, bin = bin
            mode = 2
            end

         '<printout>' : begin
            if (fnam0 ne '') then $
	       magcont, img, q, u, v, h = h, mag = magnify, bin = bin, $
			/printout $
            else print, 'Please select data before printing !'
            end

         '<PostScript>' : begin
            if (fnam0 ne '') then $
	       magcont, img, q, u, v, h = h, mag = magnify, $
                        bin = bin, save_ps = dirtmp + '/magcont.ps'$
		else print, 'Please select data before printing !'
		end
	    else: begin

            ;  ---------  F1 data  ---------

            if strmid (fnam, 0, 2) eq 'f1' or strmid (fnam, 0, 2) eq 'fb' then $
	    begin
               print, fnam
               h1 = nkrhead ()
               h = replicate (h1, 4)	; header for i, q, u, v
               nkrget, dir + fnam, h1, img, img_no = 1	& h (0) = h1
               nkrget, dir + fnam, h1, q, img_no = 2	& h (1) = h1
               nkrget, dir + fnam, h1, u, img_no = 3	& h (2) = h1
               nkrget, dir + fnam, h1, v, img_no = 4	& h (3) = h1
               if (strpos (h(0).dx, '.') ne -1) then begin
                  magnify = fix (float (h(0).dx) / pix1)
                  bin =     fix (float (h(0).dx) / pix1)
            endif else begin
               magnify = 1	& bin = 1
            endelse
            case mode of
               0:   magcont, img, q, u, v, h = h, mag = magnify, bin = bin
               1:   magtv1, img, q, u, v, h = h, magnify = magnify, bin = bin
               2:   magtv2, img, q, u, v, h = h, magnify = magnify, bin = bin
            endcase
            fnam0 = fnam
         endif

         ;  ---------  F4 data  ---------

         if (strmid (fnam, 0, 2) eq 'f4' or $
             strmid (fnam, 0, 2) eq 'fv') then begin
            f4data, fnam, h, img, s, v, dir = dir
            magtv1, img, dumy, s, v, h = h
            fnam0 = fnam
         endif
         end
      endcase

   endwhile
   end

2: begin		; --------------  monochro data  ------------------
   ; dir1 = dir0 + 'monochro/1993/jan/'
   ; cd, dir0 + 'monochro/1993/jan/'
   dir1 = dir0
   cd, dir1
   ff = findfile (dir1)
   ; print, dir1
   ; print, ff
   loadct, 3
   !order = 1
   wh = 20
   while 1 do begin
      device, get_screen_size = ws
      if (ws (0) eq 1024 and ws (1) eq 768) then nnmax = 308 else nnmax = 632
      file1 = lsmenu_nkr (dir, ret = ['[return]', '[exit]'], x = 2, y = 85, $
			 nnmax = nnmax)

      ;  ---------  exit  ------------

      case file1 of
         '[exit]': begin
            print, '>>>>> Image is stored in "img"'
            print, '>>>>> Information is in Norikura type header "h"'
            cd, dir_old
            stop
            end

         '[return]': begin
            cd, dir0
            goto, top
            end

         else: begin
            nkrget, file1, h, img
            wwx = h.nx	& wwy = h.ny + wh
            if (wwx ne !d.x_size) or (wwy ne !d.y_size) then $
               window, xsize = wwx, ysize = wwy
            erase
            tvscl, img
            xyouts, 10, h.ny + wh - 17,  $
               '!8' + h.observatory + '!6:  ' + h.date1 + '  ' $
               + strmid (h.time1, 0, 8) + '-' +  strmid (h.time2, 0, 8) $
               + '  ' + file1, $
	       size = 1.5, /device
         endelse
      endcase
	
   endwhile
   end

3: begin		; --------------  norikura data  ------------------
   cd, dir0 + 'norikura'
   loadct, 0
   !order = 0
   dir = ''
   wh = 20
   while 1 do begin
ld1:  file1 = lsmenu_nkr (dir, ret = ['<scale>', '[return]', '[exit]'], $
			   x = 2, y = 85)

      ;  ---------  exit  ------------

      case file1 of
         '[exit]': begin
            print, '>> Image is stored in "img" <<'
            print, '>> Information is in Norikura type header "h" <<'
            cd, dir_old
            stop
            end

         '[return]': begin
            cd, dir0
            goto, top
            end

         '<scale>': begin
            print, 'current min & max = ', lowlimit, uplimit
            read, 'enter new values : ', lowlimit, uplimit
            tvscl, img > lowlimit < uplimit
            xyouts, 10, h.ny + wh - 27, $
		    '!8' + 'NKR' + '!6:  ' + h.date1 + '  ' $
                    + strmid (h.time1, 0, 8) + '-' +  strmid (h.time2, 0, 8), $
                    size = 2.5, /device
            goto, ld1
            end

         else: begin
            nkrget, dir + file1, h, img
            wwx = h.nx	& wwy = h.ny + wh
            if (wwx gt !d.x_size) or (wwy gt !d.y_size) then $
            window, xsize = wwx, ysize = wwy
            erase
            if (strmid (file1, 0, 2) eq 'he') then begin
               xm = h.nx/2	& ym = h.ny/2	& ww = h.nx/6
               lowlimit = min (img (xm-ww:xm+ww, ym-ww:ym+ww))
               uplimit  = max (img (xm-ww:xm+ww, ym-ww:ym+ww))
               tvscl, img > lowlimit < uplimit
            endif else begin
               tvscl, img
            endelse
            xyouts, 10, h.ny + wh - 27, $
		    '!8' + 'NKR' + '!6:  ' + h.date1 + '  ' $
                    + strmid (h.time1, 0, 8) + '-' +  strmid (h.time2, 0, 8), $
                    size = 2.5, /device
         endelse
      endcase

   endwhile	
   end

4: begin		; --------------  okayama data  ------------------
   cd, dir0 + 'okayama'
   loadct, 0
   !order = 0
   year = lsmenu_nkr (dir, x = 2, y = 85, /dirret)
   cd, year
   dir = ''
   while 1 do begin
      file1 = lsmenu_nkr (dir, ret = ['[return]', '[exit]'], x = 2, y = 85, $
			 select = '*.D')

      ;  ---------  exit  ------------

      case file1 of
         '[exit]': begin
            print, '>> Data are stored in "rdata" <<'
            print, '>> Information is in "mparam" <<'
            print, '>> Electric currents can be seen by typing ".run jn" <<'

            cd, dir_old
            stop
            end

         '[return]': begin
            cd, dir0
            wdelete,  0,  2
            goto, top
            end

         else: begin
            magplot,  mparam, rdata,  filename = file1,  $
            printdir = dirtmp,  /nomenu
         endelse
      endcase
   endwhile	
   end

5: begin
      cd, dir_old
      stop
   end

endcase

end
