; NAME :	dispm
; PURPOSE :
;	display magnetograms
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	dispm, mparam, rdata
; INPUTS :
;	mparam, rdata
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	oaodefault : for 'show' program on spot@mtk.nao.ac.jp
;		plot i in gray scale, BL in contours, BT by bars
;		if oaodefault is not specified, interactive mode will be invoked
;	whitebg : white background (default is dark background)
;	noask :	if it is not set, you are asked to repeat plotting
; OUTPUTS :
;	creates magnetogram plot
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;
;------------------------------------------------------------------------
;
pro dispm, mparam, rdata, $
	oaodefault=oaodefault, whitebg=whitebg, noask=noask, $
	switch=switch

	switch={switch1, $
		nh:0, nc:0, nv:0, arrow:0, lof:0, region:0}

	setcolor, bandwidth, w_start, rb_start, g_start, color8, $
			whitebg = whitebg

	if keyword_set(whitebg) then begin
		fgcolor = color8.black 
		bgcolor = color8.white
	endif else begin
		fgcolor = color8.white
		bgcolor = color8.black
	endelse

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	xe = xs + dx *(nx-1)
	ye = ys + dy *(ny-1)

	title1 = strtrim( mparam.inst+': ', 1) + strtrim( mparam.title, 1)
			; trim leading blanks

	if ( mparam.kindm lt 0 ) then pol2b, mparam, rdata
	kindm = mparam.kindm
	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz

	x = xs + indgen(nx)*dx
	y = ys + indgen(ny)*dy

	intlevels = [ 0.05, 0.5, 0.6, 0.7, 0.8, 0.85, 0.9 ]
	dummylevels = - rotate(intlevels, 2)
;
;	bplevels = [ 18.,36.,91.,182.,370.,1030.,2800. ]
;		this gives the same contours as in old calibration with
;		levels=10,20,50,100,200,500,1000 for OAO magnetogram
;
	bplevels = [ 10.,20.,50.,100.,200.,500.,1000. ]	
	bmlevels = - rotate(bplevels, 2)
	dplevels = [200.,400.,600.,800.,1000.]
	dmlevels = - rotate(dplevels, 2)
	jplevels  = [100.0, 200.0, 500.0, 1000.0, 2000.0, 5000.0, 10000.0]
	jmlevels = - rotate(jplevels, 2)
	fplevels = [100.0, 200.0, 500.0, 1000.0, 2000.0, 5000.0, 10000.0]
	fmlevels = - rotate(fplevels, 2)

top:
	nh=0
	kh=-1
	nc=0
	kc=-1
	kx=-1
	ky=-1
	nv=0
	chrh = ' '
	chrc = ' '
	arrow=1
	title2 = ''
	title3 = ''
	
	if keyword_set (oaodefault) then begin
		kh = mparam.status.i
		kc = mparam.status.bl
		kx = mparam.status.bx
		ky = mparam.status.by
		chrh = 'I'
		chrc = 'B'
		strx = [ 'BX' ]
		stry = [ 'BY' ]
		strz = [ 'I', 'BL' ]
		nh = 1
		nc = 2
		nv = 1
	endif else begin
		if ( nz gt 0 ) then begin
			print,format='("halftone map: 0=skip",$)
			for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strz(i-1)
			repeat read,nh until ( nh ge 0 and nh le nz )
			if ( nh ne 0 ) then begin
				kh = kzlist(nh-1)
				chrh = strmid( strz(nh-1), 0,1)
			endif

			print,format='("contour map : 0=skip",$)
			for i=1, nz do print,format='(", ",i1,"=",a5,$)', i,strz(i-1)
			repeat read,nc until ( nc ge 0 and nc le nz )
			if ( nc ne 0 ) then begin
				kc = kzlist(nc-1)
				chrc = strmid( strz(nc-1), 0,1)
			endif
		endif
		if ( nxy gt 0 ) then begin
			print,format='("vector map  : 0=skip",$)'
			for i=1, nxy do print, $
			format='(", ",i1,"=(",a5,",",a5,")",$)', i,strx(i-1),stry(i-1)
			repeat read,nv until ( nv ge -nxy and nv le nxy )
			if ( nv lt 0 ) then begin
				nv = -nv
				arrow = 0
			endif
			if ( nv ne 0 ) then begin
				kx = kxlist(nv-1)
				ky = kylist(nv-1)
			endif
		endif	
	endelse
		
; set switches for printer output
	switch.nh= nh
	switch.nc= nc
	switch.nv= nv
	switch.arrow= arrow

; magplot sets !d.x_size = !d.x_vsize = 640,
;              !d.y_size = !d.y_vsize = 600

	xwsize = !d.x_vsize
	ywsize = !d.y_vsize
;	xpixwidth= fix(xwsize) - 100
;	ypixwidth= fix(ywsize) - 80
      xpixwidth= fix(0.8*xwsize)
      ypixwidth= fix(0.8*ywsize)
; xpixwidth,ypixwidth = size of plot window in device coordinates

;	xcorner = 50
;	ycorner = 40
	xcorner = fix(0.1*xpixwidth)
	ycorner = fix(0.1*ypixwidth)
; xcorner,ycorner = lower left corner of plot window in device coordinates

; determine magnification factor
	mag = fix(min( [ xpixwidth/nx, ypixwidth/ny ] )) > 1

; lower left and upper right corners of contour plot in device coordinates
	mapposition = [xcorner, ycorner, $
	               xcorner+mag*(nx-1), ycorner+mag*(ny-1) ]
; locations of title strings
	h = fix(0.04*ywsize)
	x3 = xcorner & y3 = ycorner + mag*ny
	x2 = x3      & y2 = y3 + h
	x1 = x3      & y1 = y2 + h

	erase, bgcolor

; halftone map

	if ( kh ge 0 ) then begin
		if ( chrh eq 'I' ) then begin
			color = w_start
			bw = bandwidth/2	; max. brightness is 128
			bytebuf = bytscl( rdata(*,*,kh), min=0.3, max=1.0 )
		endif else begin
			color = rb_start
			bw = 2*bandwidth
			valmax = max(abs(rdata(*,*,kh)))
			if chrh eq 'J' then valmin=0.05*valmax $
					else valmin=0.01*valmax
			if ( chrh eq 'V' ) then $
				bytebuf = bytscl( $
					loglevel( -rdata(*,*,kh), valmin, valmax), $
					max=1.0, min=-1.0 ) $
			else $
				bytebuf = bytscl( $
					loglevel( rdata(*,*,kh), valmin, valmax), $
					max=1.0, min=-1.0 ) 
		endelse

		if mag gt 1 then bytebuf = rebin(bytebuf, mag*nx,mag*ny)
		tv, $
		color + fix( bytebuf( 0:(mag*(nx-1)), 0:(mag*(ny-1)) ) $
		/255.0*(bw-1) ), $
		xcorner, ycorner, order=0

		title3 = strmid('HALFTONE:'+strz(nh-1)+'                    ',0,20)
	endif 

; contour map

	if ( kc ge 0 ) then begin
		case chrc of
		'B':	begin
			plevels = bplevels & mlevels = bmlevels
			end
		'V':  begin
			plevels = dplevels & mlevels = dmlevels
			end
		'I':	begin
			plevels = intlevels & mlevels = dummylevels
			end
		'J':	begin
			plevels = jplevels & mlevels = jmlevels
			end
		'F':	begin
			plevels = fplevels & mlevels = fmlevels
			end
		endcase

     		sz = size(plevels)
     		nlevels = fix(sz(1))
     		plinestyle = fix(plevels)
     		mlinestyle = fix(mlevels)
     		pcolors = plinestyle
     		mcolors = mlinestyle
     		plabels=plinestyle
     		plabels(*)=0
     		mlabels=plabels

		plinestyle(*) = 0 ; solid
;		mlinestyle(*) = 1 ; dotted --  no good
		mlinestyle(*) = 2 ; dashed
		case chrc of
		'I':	begin
			pcolors(*) = color8.yellow
			mcolors(*) = color8.yellow ; dummy
			end
		'V':	begin
			pcolors(*) = color8.blue
			mcolors(*) = color8.red
			end
		else:	begin
			pcolors(*) = color8.red
			mcolors(*) = color8.blue
			end
		endcase

		title2 = strmid( $
			'CONTOURS:'+strz(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels)

		contour, rdata(*,*,kc), x,y, $
		/spline, $
		/noerase, $
		levels=[mlevels,plevels], $
		c_linestyle = [mlinestyle,plinestyle], $
		c_color = [mcolors, pcolors], $
		c_labels = [mlabels, plabels], $
		xstyle=1, ystyle=1, /follow, $
		color = fgcolor, $
		position = mapposition, /device, $
		xtitle='!6X (10!E4!Nkm)', ytitle='!6Y (10!E4!Nkm)'
	endif

	if kc lt 0 then begin
		contour, bytarr(nx,ny), x,y, $
		/noerase, /nodata, $
		xstyle=1, ystyle=1, $
		color = fgcolor, $
		position = mapposition, /device, $
		xtitle='!6X (10!E4!Nkm)', ytitle='!6Y (10!E4!Nkm)'
	endif

; vector map

	if ( kx ge 0 and ky ge 0 ) then begin

;		btmax = 1000.0
		btmax = max ( sqrt( rdata(*,*,kx)^2 + rdata(*,*,ky)^2 ) ) 
		rbtmin=0.1
		btmin=rbtmin*btmax
		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx gt 18 and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected

		nbin = max([nx,ny])/100 +1
		xb=fltarr(2)
		yb=xb
		xa1=xb
		xa2=xb
		ya1=xb
		ya2=xb
		ds0 = 0.5* nbin * sqrt( dx*dx + dy*dy )
		dsa = 0.5* ds0 ; size of arrow
		fa = 0.3
		for j = 0, ny-1, nbin do begin
		for i = 0, nx-1, nbin do begin
			bt = sqrt(rdata(i,j,kx)^2 +rdata(i,j,ky)^2)
			if ( bt eq 0.0) then goto, endloop
			ds = ds0 * loglevel( bt, btmin, btmax)
			xb(0) = xs + dx*i + ds*rdata(i,j,kx)/bt
			yb(0) = ys + dy*j + ds*rdata(i,j,ky)/bt
			xb(1) = xs + dx*i - ds*rdata(i,j,kx)/bt
			yb(1) = ys + dy*j - ds*rdata(i,j,ky)/bt
			if arrow eq 0 then begin
				if (ds lt 0.5*dsa) then goto, endloop
;				if (ds le 0.0) then goto, endloop
				oplot, xb, yb, $
				color = fgcolor, thick=2
			endif else begin
				if (ds lt 0.5*dsa) then goto, endloop
				oplot, xb, yb, $
				color = fgcolor, thick=1
				xa1(0)=xb(0) & xa2(0)=xb(0)
				ya1(0)=yb(0) & ya2(0)=yb(0)
				xa1(1)=xa1(0)-dsa/bt*(rdata(i,j,kx)+fa*rdata(i,j,ky))
				xa2(1)=xa2(0)-dsa/bt*(rdata(i,j,kx)-fa*rdata(i,j,ky))
				ya1(1)=ya1(0)-dsa/bt*(rdata(i,j,ky)-fa*rdata(i,j,kx))
				ya2(1)=ya2(0)-dsa/bt*(rdata(i,j,ky)+fa*rdata(i,j,kx))
				oplot, xa1, ya1, $
					color = fgcolor, thick=1
				oplot, xa2, ya2, $
					color = fgcolor, thick=1
			endelse
		endloop:
		endfor
		endfor
		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif


	font = 0	; hardware font
	xyouts, x1,y1, /device, font=font, title1, color=fgcolor
	xyouts, x2,y2, /device, font=font, title2, color=fgcolor
	xyouts, x3,y3, /device, font=font, title3, color=fgcolor
;	xyouts, x1,y1, /device, title1, color=fgcolor
;	xyouts, x2,y2, /device, title2, color=fgcolor
;	xyouts, x3,y3, /device, title3, color=fgcolor

	yn = ' '
	if keyword_set(oaodefault) then return

	if not keyword_set(noask) then begin
		print, format='("repeat? (y/n) > ",$)
		repeat read, yn until $
			 (yn eq 'y' or yn eq 'Y' or yn eq 'n' or yn eq 'N') 

		if ( yn eq 'y' or yn eq 'Y' ) then goto, top
	endif

	return
end

