; NAME :	plotm
; PURPOSE :
;	plot magnetograms on a line-drawing device
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	plotm, mparam, rdata
; INPUTS :
;	mparam, rdata
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	switch : a structure specifying items to be plotted
;	color : if set, plot output is made in color
; OUTPUTS :
;	creates magnetogram plot
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1993
;
;------------------------------------------------------------------------
;
pro plotm, mparam, rdata, $
	switch=switch, color=color

	if not keyword_set( color ) then begin
		fgcolor = !p.color
	endif else begin
		setcolor, bandwidth, w_start, rb_start, g_start, color8
		fgcolor = color8.white
	endelse

	xs=mparam.xs
	ys=mparam.ys
	nx=mparam.nx
	ny=mparam.ny
	dx=mparam.dx
	dy=mparam.dy
	xe = xs + dx *(nx-1)
	ye = ys + dy *(ny-1)

	title1 = strtrim( mparam.inst+': ', 1) + strtrim( mparam.title, 1)
			; trim leading blanks

	if ( mparam.kindm lt 0 ) then pol2b, mparam, rdata
	kindm = mparam.kindm
	mklist, mparam, nxy,nz,kxlist,kylist,kzlist,strx,stry,strz

	x = xs + indgen(nx)*dx
	y = ys + indgen(ny)*dy

	intlevels = [ 0.05, 0.5, 0.6, 0.7, 0.8, 0.85, 0.9 ]
	dummylevels = - rotate(intlevels, 2)
;
;	bplevels = [ 18.,36.,91.,182.,370.,1030.,2800. ]
;		this gives the same contours as in old calibration with
;		levels=10,20,50,100,200,500,1000 for OAO magnetogram
;
	bplevels = [ 10.,20.,50.,100.,200.,500.,1000. ]	
	bmlevels = - rotate(bplevels, 2)
	dplevels = [200.,400.,600.,800.,1000.]
	dmlevels = - rotate(dplevels, 2)
	jplevels  = [100.0, 200.0, 500.0, 1000.0, 2000.0, 5000.0, 10000.0]
	jmlevels = - rotate(jplevels, 2)
	fplevels = [100.0, 200.0, 500.0, 1000.0, 2000.0, 5000.0, 10000.0]
	fmlevels = - rotate(fplevels, 2)

      kc=-1
      kx=-1
      ky=-1
      nv=0
      chrh = ' '
      chrc = ' '
	nc=switch.nc
	nv=switch.nv
	if ( nc ne 0 ) then begin
		kc = kzlist(nc-1)
		chrc = strmid( strz(nc-1), 0,1)
	endif
	if ( nv ne 0 ) then begin
		kx = kxlist(nv-1)
		ky = kylist(nv-1)
	endif
	arrow=switch.arrow
	title2 = ''
	title3 = ''
	
; magplot sets !d.x_size = !d.x_vsize = 640,
;              !d.y_size = !d.y_vsize = 600

	xwsize = !d.x_vsize
	ywsize = !d.y_vsize
;	xpixwidth= fix(xwsize) - 100
;	ypixwidth= fix(ywsize) - 80
	xpixwidth= fix(0.8*xwsize)
	ypixwidth= fix(0.8*ywsize)
; xpixwidth,ypixwidth = size of plot window in device coordinates
	xcorner = fix(0.1*xpixwidth)
	ycorner = fix(0.1*ypixwidth)
; xcorner,ycorner = lower left corner of plot window in device coordinates

; determine magnification factor
	mag = fix(min( [ xpixwidth/nx, ypixwidth/ny ] )) > 1

; lower left and upper right corners of contour plot in device coordinates
	mapposition = [xcorner, ycorner, $
	               xcorner+mag*(nx-1), ycorner+mag*(ny-1) ]
; locations of title strings
	h = fix(0.04*ywsize)
	x3 = xcorner & y3 = ycorner + mag*ny
	x2 = x3      & y2 = y3 + h
	x1 = x3      & y1 = y2 + h

	erase

; contour map

	if ( kc ge 0 ) then begin
		case chrc of
		'B':	begin
			plevels = bplevels & mlevels = bmlevels
			end
		'V':  begin
			plevels = dplevels & mlevels = dmlevels
			end
		'I':	begin
			plevels = intlevels & mlevels = dummylevels
			end
		'J':	begin
			plevels = jplevels & mlevels = jmlevels
			end
		'F':	begin
			plevels = fplevels & mlevels = fmlevels
			end
		endcase

     		sz = size(plevels)
     		nlevels = fix(sz(1))
     		plinestyle = fix(plevels)
     		mlinestyle = fix(mlevels)
     		pcolors = plinestyle
     		mcolors = mlinestyle
     		plabels=plinestyle
     		plabels(*)=0
     		mlabels=plabels

		plinestyle(*) = 0 ; solid
;		mlinestyle(*) = 1 ; dotted --  no good
		mlinestyle(*) = 2 ; dashed

		if not keyword_set( color ) then begin
			pcolors(*) = fgcolor
			mcolors(*) = fgcolor
		endif else begin

		case chrc of
		'I':	begin
			pcolors(*) = color8.yellow
			mcolors(*) = color8.yellow ; dummy
			end
		'V':	begin
			pcolors(*) = color8.blue
			mcolors(*) = color8.red
			end
		else:	begin
			pcolors(*) = color8.red
			mcolors(*) = color8.blue
			end
		endcase
		endelse

		title2 = strmid( $
			'CONTOURS:'+strz(nc-1) +'                    ',0,20) + $
  			'LEVELS='
		title2 = title2 + strlevel( plevels)

		contour, rdata(*,*,kc), x,y, $
		/spline, $
		/noerase, $
		levels=[mlevels,plevels], $
		c_linestyle = [mlinestyle,plinestyle], $
		c_color = [mcolors, pcolors], $
		c_labels = [mlabels, plabels], $
		xstyle=1, ystyle=1, /follow, $
		color = fgcolor, $
		position = mapposition, /device, $
		xtitle='!6X (10!E4!Nkm)', ytitle='!6Y (10!E4!Nkm)'
	endif

	if kc lt 0 then begin
		contour, bytarr(nx,ny), x,y, $
		/noerase, /nodata, $
		xstyle=1, ystyle=1, $
		color = fgcolor, $
		position = mapposition, /device, $
		xtitle='!6X (10!E4!Nkm)', ytitle='!6Y (10!E4!Nkm)'
	endif

; vector map

	if ( kx ge 0 and ky ge 0 ) then begin

;		btmax = 1000.0
		btmax = max ( sqrt( rdata(*,*,kx)^2 + rdata(*,*,ky)^2 ) ) 
		rbtmin=0.1
		btmin=rbtmin*btmax
		kindmb = kindm mod 10
		if ( arrow gt 0 and kindmb gt 2 ) then arrow = 1 else arrow = 0
		if ( kx gt 18 and kindmb eq 6 ) then arrow =0
;		kindmb=6: data are pseudo B and Btp, and pseudo B is selected

		nbin = max([nx,ny])/80 +1
		xb=fltarr(2)
		yb=xb
		xa1=xb
		xa2=xb
		ya1=xb
		ya2=xb
		ds0 = 0.5* nbin * sqrt( dx*dx + dy*dy )
		dsa = 0.5* ds0 ; size of arrow
		fa = 0.3
		for j = 0, ny-1, nbin do begin
		for i = 0, nx-1, nbin do begin
			bt = sqrt(rdata(i,j,kx)^2 +rdata(i,j,ky)^2)
			if ( bt eq 0.0) then goto, endloop
			ds = ds0 * loglevel( bt, btmin, btmax)
			xb(0) = xs + dx*i + ds*rdata(i,j,kx)/bt
			yb(0) = ys + dy*j + ds*rdata(i,j,ky)/bt
			xb(1) = xs + dx*i - ds*rdata(i,j,kx)/bt
			yb(1) = ys + dy*j - ds*rdata(i,j,ky)/bt
			if arrow eq 0 then begin
				if (ds lt 0.5*dsa) then goto, endloop
;				if (ds le 0.0) then goto, endloop
				oplot, xb, yb, $
				color = fgcolor, thick=2
			endif else begin
				if (ds lt 0.5*dsa) then goto, endloop
				oplot, xb, yb, $
				color = fgcolor, thick=1
				xa1(0)=xb(0) & xa2(0)=xb(0)
				ya1(0)=yb(0) & ya2(0)=yb(0)
				xa1(1)=xa1(0)-dsa/bt*(rdata(i,j,kx)+fa*rdata(i,j,ky))
				xa2(1)=xa2(0)-dsa/bt*(rdata(i,j,kx)-fa*rdata(i,j,ky))
				ya1(1)=ya1(0)-dsa/bt*(rdata(i,j,ky)-fa*rdata(i,j,kx))
				ya2(1)=ya2(0)-dsa/bt*(rdata(i,j,ky)+fa*rdata(i,j,kx))
				oplot, xa1, ya1, $
					color = fgcolor, thick=1
				oplot, xa2, ya2, $
					color = fgcolor, thick=1
			endelse
		endloop:
		endfor
		endfor
		title3 = title3 + $
			strmid( 'VECTORS :' + strx(nv-1) + ', ' + stry(nv-1) + $
			'          ', 0, 20 ) + $
			'MAX,MIN=' + string(btmax, btmin, format='(2f6.0)')
	endif


	font = 0	; hardware font
;	xyouts, x1,y1, /device, font=font, title1, color=fgcolor
;	xyouts, x2,y2, /device, font=font, title2, color=fgcolor
;	xyouts, x3,y3, /device, font=font, title3, color=fgcolor
	xyouts, x1,y1, /device, title1, color=fgcolor
	xyouts, x2,y2, /device, title2, color=fgcolor
	xyouts, x3,y3, /device, title3, color=fgcolor

	return
end

