; NAME :	setcolor
; PURPOSE :
;	set color table for magnetograms and field line plot
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;
;	setcolor, bandwidth, w_start, rb_start, g_start, $
;			color8, whitebg=whitebg
; INPUTS :
;	none
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	whitebg = white background ( default is dark background )
; OUTPUTS :
;	generated color table is:
;	<--bandwidth--> <----2*bandwidth----> <--bandwidth-->
;	black,,,,white, blue,,,,black,,,,red, black,,,,green, color8
;	|               |                     |
;	w_start         rb_start              g_start
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	color table will be modified
; RESTRICTIONS :	X-window environment is assumed
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;
;------------------------------------------------------------------------
;
pro setcolor, bandwidth, w_start, rb_start, g_start, color8, whitebg=whitebg
;
	color8 = { clr8, white:0, red:0, green:0, blue:0, $
			yellow:0, cyan:0, magenta:0, black:0 }

	maxcolor = fix( !d.n_colors )
	if (maxcolor ge 255 ) then maxcolor=230
	bw = ( maxcolor-8 ) / 4
	f = 255.0 / (bw-1)

	incr = fix( f*indgen(bw) )
	decr = fix( f*(bw-1-indgen(bw)) )
	zero = intarr(bw)
	full = replicate( 255, bw )

	if keyword_set(whitebg) then begin
; white background
		rtable=[ incr, incr, full, zero, intarr(maxcolor - 4*bw) ]
		gtable=[ incr, incr, decr, incr, intarr(maxcolor - 4*bw) ]
		btable=[ incr, full, decr, zero, intarr(maxcolor - 4*bw) ]
	endif else begin
; dark  background
		rtable=[ incr, zero, incr, zero, intarr(maxcolor - 4*bw) ]
		gtable=[ incr, zero, zero, incr, intarr(maxcolor - 4*bw) ]
		btable=[ incr, decr, zero, zero, intarr(maxcolor - 4*bw) ]
	endelse
	

	w_start	= 0
	rb_start	= w_start + bw
	g_start	= rb_start + bw +bw

	white	= maxcolor - 1
	red	= white - 1
	green	= red - 1
	blue	= green - 1
	yellow	= blue - 1
	cyan	= yellow - 1
	magenta	= cyan - 1
	black	= magenta - 1

	rtable( black:white )	= 127*[0, 2, 0, 2, 0, 1, 2, 2]
	gtable( black:white )	= 127*[0, 0, 2, 2, 1, 2, 1, 2]
	btable( black:white )	= 127*[0, 2, 2, 0, 2, 1, 0, 2]

	tvlct,rtable,gtable,btable

	color8.white	= white
	color8.red	= red			; red contains 33% green
	color8.green	= green	; green contains 25% red and 25% blue
	color8.blue	= blue		; blue contains 25% green
	color8.yellow	= yellow
	color8.cyan	= cyan
	color8.magenta	= magenta
	color8.black	= black
	bandwidth = bw
	
	return
end
