;
;  for a specified location (x,y),
;  get a new mesh location (i,j) and fractional coordinates (xr,yr)
;  old mesh : origin at (xs,ys), increment=(dx,dy)
;
function getij, nx,ny,xs,ys,dx,dy,x,y,i,j,xr,yr
;
	eps=0.01
	bordercheck = 0
;
	i=fix( (x-xs)/dx )
	j=fix( (y-ys)/dy )
	xr=(x-xs)/dx-i
	yr=(y-ys)/dy-j

	if (bordercheck eq 0) then goto, skip

	loc = where( i eq -1 and xr gt 1.0-eps )
	if (loc(0) ge 0) then begin
		i(loc)=0
		xr(loc)=xr(loc)-1.0
	endif
	loc = where( j eq -1 and yr gt 1.0-eps)
	if (loc(0) ge 0) then begin
		j(loc)=0
		yr(loc)=yr(loc)-1.0
	endif
	loc = where( i eq nx-1 and xr lt eps)
	if (loc(0) ge 0) then begin
		i(loc)=nx-2
		xr(loc)=xr(loc)+1.0
	endif
	loc = where( j eq ny-1 and yr lt eps )
	if (loc(0) ge 0) then begin
		j(loc)=ny-2
		yr(loc)=yr(loc)+1.0
	endif

skip:
	loc = where( i ge 0 and i lt nx and j ge 0 and j lt ny)
	return, loc
end
;
function umake, kindm
	m=intarr(5)
	m(4) = kindm/10000 mod 10
	m(3) = kindm/1000 mod 10
	m(2) = kindm/100 mod 10
	m(1) = kindm/10 mod 10
	m(0) = kindm mod 10
	return, m
end

@transc

pro transm, mparam, rdata, kindmo=kindm2, mesho=mesh2
; for the moment, only coordinate transform is made
; vector component transform will be installed later
;
	kindm1 = mparam.kindm
	mesh1  = mparam.mesh
;
print,'kindm,mesh=',kindm1,mesh1

;  transform of grids
;  data are first transformed to D-grid if necessary, and then
;  transformed to the final grid specified by mesh2
;
	ufl=1.0e-15
	pi=3.141592
;
	if keyword_set(kindmo) then begin
		if kindm2 eq 0 then kindm2=kindm1
	endif else $
		kindm2=kindm1

	if (kindm1 lt 0 or kindm2 lt 0) and kindm1 ne kindm2 then begin
		print, '(proc.transm) incompatible specification,', $
		'kindm1,kindm2=', kindm1,kindm2
		return
	endif

	oldm = umake(kindm1)
	newm = umake(kindm2)

	if  oldm(4) gt 3  or  newm(4) gt 3 then begin
		print, '(proc.transm) incompatible specification,', $
		'kindm1,kindm2=', kindm1,kindm2
		return
	endif

	if  mesh2 eq mesh1 then goto, vectortrans
	if  mesh1 eq -1 and mesh2 eq 0 then begin
		print,'(proc.transm) change mesh1 from -1 to 0'
		mesh1=mesh2
		goto, vectortrans
	endif

	dimdata = size(rdata)
	if dimdata(0) eq 3L then nframe0 = dimdata(3) else nframe0=1

	xs = mparam.xs
	ys = mparam.ys
	dx = mparam.dx
	dy = mparam.dy
	nx = mparam.nx
	ny = mparam.ny
	rsp = mparam.rsp
	kmemarr = kmem(mparam,rdata)
	nframe = mparam.nframe
	print, '  old parameters (xs,ys)=',xs,ys
	print, '                 (dx,dy)=',dx,dy
	print, '                 (nx,ny)=',nx,ny

	mesh12 = 4*abs(mesh1) + mesh2
	coord_d = setdu ( mparam )
	coord_t = settu ( mparam )
	coord_s = setas ( mparam )
	coord_r = setar ( mparam )
	ximesh=[xs, xs+dx*(nx-1), xs+dx*(nx-1), xs, xs, xs+dx*(nx-1)]
	yimesh=[ys, ys, ys+dy*(ny-1), ys+dy*(ny-1), 0,0]
	zimesh=[0,0,0,0,0,0]
	if (yimesh(0)*yimesh(3) gt 0.0) then yimesh(4)=yimesh(0)
	if (yimesh(1)*yimesh(2) gt 0.0) then yimesh(5)=yimesh(1)

	case mesh12 of

	0:
	5:
	10:
	15:	; no operation

	1:begin	; mesh1=0 and mesh2=1
		dtoc, ximesh,yimesh,zimesh, xmesh,ymesh,zmesh, coord_d
 		projt, xmesh,ymesh,zmesh, xomesh,yomesh, coord_d, coord_t
	end

	2:begin	; mesh1=0 and mesh2=2
		dtoc, ximesh,yimesh,zimesh, xmesh,ymesh,zmesh, coord_d
		projs, xmesh,ymesh,zmesh, xomesh,yomesh, coord_d, coord_s
	end

	3:begin	; mesh1=0 and mesh2=3
		dtoc, ximesh,yimesh,zimesh, xmesh,ymesh,zmesh, coord_d
		projr, xmesh,ymesh,zmesh, xomesh,yomesh, coord_d, coord_r
;		if (xomesh(1) lt xomesh(0)) then xomesh(1)=xomesh(1)+2*pi
;		if (xomesh(3) lt xomesh(0)-pi) then xomesh(3)=xomesh(3)+2*pi
;		if (xomesh(3) gt xomesh(0)+pi) then xomesh(3)=xomesh(3)-2*pi
;		if (xomesh(4) lt xomesh(0)-pi) then xomesh(4)=xomesh(4)+2*pi
;		if (xomesh(4) gt xomesh(0)+pi) then xomesh(4)=xomesh(4)-2*pi
;		if (xomesh(2) lt xomesh(3)) then xomesh(2)=xomesh(2)+2*pi
;		if (xomesh(5) lt xomesh(4)) then xomesh(5)=xomesh(5)+2*pi
	end

	4:begin	; mesh1=1 and mesh2=0
		ttod, ximesh,yimesh,zimesh, xomesh,yomesh,zomesh, $
			coord_t, coord_d
	end

	8:begin	; mesh1=2 and mesh2=0
		yimesh=alofy(yimesh)
		stod, ximesh, yimesh, zimesh, xomesh, yomesh, zomesh, $
			coord_s, coord_d
	end

	12:begin	; mesh1=3 and mesh2=0
		yimesh=alofy(yimesh)
		rtod, ximesh, yimesh, zimesh, xomesh, yomesh, zomesh, $
			coord_r, coord_d
	end
	endcase

	if (mesh2 ge 2) then begin
      		ysnew=yofal(min( yomesh ))
      		yenew=yofal(max( yomesh ))
      		dxnew=abs(dx)/rsp
      		dynew=yofal(abs(dy)/rsp)
	endif else begin
		ysnew=min( yomesh )
		yenew=max( yomesh )
		dxnew=abs(dx)
		dynew=abs(dy)
	endelse
	xsnew=min( xomesh )
	xenew=max( xomesh )
	nxnew = fix((xenew - xsnew )/dxnew +1.5)
	nynew = fix((yenew - ysnew )/dynew +1.5)

	newdata = fltarr(nxnew,nynew, nframe0)

	for km=0, nframe-1 do begin
		k=kmemarr(km)
		xo = fltarr(nxnew,nynew)
		yo = fltarr(nxnew,nynew)
		zo = fltarr(nxnew,nynew)
		for jnew=0,nynew-1 do xo(*,jnew)=xsnew+dxnew*indgen(nxnew)
		for inew=0,nxnew-1 do yo(inew,*)=ysnew+dynew*indgen(nynew)
		if (mesh2 ge 2) then yo=alofy(yo)

		case mesh12 of
		1:begin	; mesh1=0 and mesh2=1
			ttod, xo,yo,zo, xi,yi,zi, coord_t, coord_d
		end
		2:begin	; mesh1=0 and mesh2=2
			stod, xo,yo,zo, xi,yi,zi, coord_s, coord_d
		end
		3:begin	; mesh1=0 and mesh2=3
			rtod, xo,yo,zo, xi,yi,zi, coord_r, coord_d
		end
		4:begin	; mesh1=1 and mesh2=0
			dtoc, xo,yo,zo, x,y,z, coord_d
			projt, x,y,x, xi,yi, coord_d, coord_t
		end
		8:begin	; mesh1=2 and mesh2=0
			dtoc, xo,yo,zo, x,y,z, coord_d
			projs, x,y,z, xi,yi, coord_d, coord_s
		end
		12:begin	; mesh1=3 and mesh2=0
			dtoc, xo,yo,zo, x,y,z, coord_d
			projr, x,y,x, xi,yi, coord_d, coord_r
		end
		endcase

		loc =  getij (nx,ny,xs,ys,dx,dy,xi,yi,ii,jj,xr,yr)
		b1=shift(rdata(*,*,k), 0, 0)
		b2=shift(rdata(*,*,k),-1, 0)
		b3=shift(rdata(*,*,k), 0,-1)
		b4=shift(rdata(*,*,k),-1,-1)
		b0 = ufl+fltarr(nxnew,nynew)
		if (loc(0) ge 0) then begin
			i=ii(loc)
			j=jj(loc)
			b0(loc)= b1(i,j)+(b2(i,j)-b1(i,j))*xr(i,j)+ $
				(b4(i,j)-b1(i,j))*yr(i,j)+ $
			(b3(i,j)-b4(i,j)-b2(i,j)+b1(i,j))*xr(i,j)*yr(i,j)
		endif else stop
		newdata(*,*,k)=b0
	endfor
;
	rdata = newdata
	mparam.xs=xsnew
	mparam.ys=ysnew
	mparam.nx=nxnew
	mparam.ny=nynew
	mparam.dx=dxnew
	mparam.dy=dynew
	print, '  new parameters (xs,ys)=',xsnew,ysnew
	print, '                 (dx,dy)=',dxnew,dynew
	print, '                 (nx,ny)=',nxnew,nynew


;
vectortrans:
; not ready
;
	mparam.kindm = kindm2
	mparam.mesh = mesh2
     	print, '(proc.transm) end: updated kindm,mesh=',kindm2,mesh2
	return
	end
