; NAME :	writem
; PURPOSE :
;	write magnetogram data to a file
; CATEGORY :
; 	solar magnetic field computation package (MAGPACK2)
; CALLING SEQUENCE :
;	writem, mparam, rdata
; INPUTS :
;	mparam, rdata
; OPTIONAL INPUT PARAMETERS : 
;	none
; KEYWORD PARAMETERS :
;	filename = magnetogram file name
;	if filename is not specified, it should be typed in from keyboard
; OUTPUTS :
;	data are written to a file
; COMMON BLOCKS : 	none
; SIDE EFFECTS : 	none
; RESTRICTIONS :	none
; PROCEDURE :
;
; MODIFICATION HISTORY :
;	T.Sakurai, 1992
;
;------------------------------------------------------------------------
;
pro writem, mparam,rdata, filename=filename
;
	if keyword_set(filename) then begin
		magfile = filename
	endif else begin
		magfile = ' '
		read,'(proc.writem) enter output file name > ',magfile
	endelse
	openw, unitm, magfile, /get_lun

	specs = defspecs(nspec)
	nbytem = mparam.nbytem
	nx = mparam.nx
	ny = mparam.ny

	writemh1, mparam, unitm

	for i=0, nspec-1 do begin
		k = mparam.status.(i)
		if k ne -1 then begin
			spec = specs(i)
			factor = mparam.factors.(i)
			mhdr2 = defmh2( spec=spec, factor=factor )
			print,'spec,factor=',mhdr2.spec,mhdr2.factor

			writeu, unitm, mhdr2
			if mparam.nbytem eq 2 then $
				writeu, unitm, fix(rdata(*,*,k)/factor) $
			else $
				writeu, unitm, long(rdata(*,*,k)/factor)

			nlast = (long(nbytem)*long(nx)*long(ny)) mod 80
			if ( nlast ne 0 ) then begin
				padstr = replicate(' ', 80 - nlast )
				writeu, unitm, padstr
			endif
		endif
	endfor

	stat = fstat(unitm)
	print, 'file=',magfile,', data size in bytes = ',stat.cur_ptr
	free_lun, unitm
	return
end
