;+
;
; FILE CP.PRO  the display program for Coronagraph/Polarimeter data.
;
;
; REQUIREMENTS :  IDL Version 2 with widgets, or better.
;		  X windows graphics device.
;			
;   To test to see if widgets are available :
;
;   IDL> if (!D.FLAGS AND 65536) NE 0 THEN PRINT, 'WIDGETS ARE AVAILABLE.'
;		
;
; The CP data must be written from the CP achive media using the IDL
; routine CPTAPE.PRO.  This is a VMS specific procedure and will place
; the data into subdirectories, one per day.  These file can then be copied
; to a UNIX machine, if desired. 
;
; - The program has two main modes: "Display one image" (single image) mode or 
;   "Show a movie" (movie) mode.  
; - The mode is selected from the main options widget, but first select the
;   desired option for "Normal display" (not a difference image) and 
;   "Make a difference" (difference image).  
; - Once single or movie modes are selected, each will create it's own
;   widget menues.
; - These menues are designed to display either normal or difference images
; - The remain option are independant and may be selected interactively 
;   during processing.
; NOTE: All widget menues are placed at thge top left corner of the screen.
;	This may result in smaller menues, such as the main options menu,
; 	being completely hidden.  Move the offending menu to as you would
;	any window on your system.
;
;
;                         /--- CPONE---CPONE_EV*-------------------\
;    CP--------CP_EVENT---                                         ---**
;    (cpcolor)            \--- CPMOVIE--CPMOVIE_EV*---CPMOVIE_DO---/   
;
;                                                                * returns to 
;            **---CPDISPLAY---------------------CPTV---CPSUNCEN    cpone_ev or
;                 (cpdisk)   \----CPDIFF---/                       to cpmovie_ev
;
;
;    Programs for C/P have the naming convention of :  CP*.PRO
;    Common block for these programs are named      :  CP*.COMMON
;
;    Required program that don't use 
;    the naming convention of CP*.PRO :	any widget library routines
;					ACKNOWLEDGE.PRO
;					OPSYS_LGCL.PRO
;					PSPLOT.PRO
;					WINUP.PRO	
;
;
;-                                                     
                                                    

;+
;
; NAME:  		
;	CP_EVENT
;
; PURPOSE: 
;	The event handler for main CP option menu widget
;
; CATEGORY: 	
;	CP display
;
; CALLING SEQUENCE: 	
;	CP_EVENT, EVENT
;
; CALLED BY: 	
;	CP.PRO
;
; CALLS TO:  		
;	WINUP, ACKNOWLEDGE, CPONE, CPMOVIE
;
; INPUTS:
;	EV : the event structure, variable depending on type of item clicked
;	also, values in commons
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS: 
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 	
;	CPCOLORS 	to set color tables states
;	CPDISPFILES 	to pass printer status
;	CPKEYWORDS 	for current options conditions
;	CPNEW_WINDOW 	for reuse state
;	CPPROLOCAL 	for the widget IDs of option buttons
;	CPWIDGBASES 	for the widget IDs of parents and informational 
;
; SIDE EFFECTS: 		
;	none
;              	
; RESTRICTIONS: 		
;	none
;
; PROCEDURE: 		
;	Process the events from the main options menu widget.  Continue 
;	recieving events until "Cancel program" is clicked, thus sending 
;	return control to calling routine.  Or until "Display one image" or
;	"Show a movie" are selected. Each of these will call the appropriate
;	routines to contine processing.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;-

pro cp_event, event

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@cpprolocal.common
;common toggles, norm, diff, snorth, sdisk, rulenone, rulerit

@cpcolors.common 
;common cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset

@cpkeywords.common 
;common keywords, norm_diff, donorth, dodisk, doradii, doradial, $
;		 ruletimes, rulecondition, posfac, negfac, sin_mov

@cpdispfiles.common 
;common dispfiles, filename, diffname1, diffname2, onewin, colo

;e 2/19/93 @cplogic_envron.common 
;common logic_envron, dataplace

@cpnew_window.common 
;common new_window, lastmark, sc, current_window, reuse, first_pass

 widget_control, event.id, get_uvalue=input	; which widget was clicked
 if n_elements(input) eq 0 then input = ''
 type = strmid(tag_names(event, /structure_name),7,1000)    ; get widget type

 case type of 
    'BUTTON' : case input of
	'QUITPRO': widget_control, event.top, /destroy 	; kill all widgets
        'KILLSCL': begin
	             widget_control, event.top, /destroy  ; kill top widget
                     widget_control, base, sensitive=1    ; can use BASE again
                   end
	'DELWIN' : begin
		     ; widget to display active window and delete
		     winup, group_leader=event.top, message=message
		     if n_elements(message) ne 0 then $
		               widget_control, blurb, set_value=message
		     device, window=any_left  ; list of active windows
		     any_left = where(any_left ne 0)
		     still_there = where(any_left eq current_window)
		     if still_there(0) eq -1 then first_pass=1  ; deleted
								; currently
								; active window
		   end

	'REUSE'	  : reuse = 1
	'NOREUSE' : reuse = 0

	; the user set options
	'NORMAL'   :  if event.select ne 0 then begin	; not turning off

                         if norm_diff eq 1 then begin	; difference was active
			    if xregistered('cpone') or $	  ; have wrong
				xregistered('cpmovie') then begin ;   option
								  ;   display 
								  ;   active
			        acknowledge, message=([ $
				        'Cancel active top widget,',$
					'either "Display one image"',$
					'or "Show a movie". ', $
					'It is incompatible ', $
					'with "Normal display.'])

				; undo normal event, put back to difference
			        widget_control, norm, set_button=0   ; undo 
			        widget_control, diff, set_button=1   ;   event
				return
			    endif

;			    colreset=1  	; reset colors
			 endif

			 coltyp=0  &  norm_diff=0
			 widget_control, scal, sensitive=0  ; not applicable
		      endif

	'DIFFERE'  :  if event.select ne 0 then begin	; not turning off

                         if norm_diff eq 0 then begin	; normal was active
			    if xregistered('cpone') or $	  ; have wrong
				xregistered('cpmovie') then begin ;   option
								  ;   display 
								  ;   active
			        acknowledge, message=([ $
				        'Cancel active top widget,',$
					'either "Display one image"',$
					'or "Show a movie". ', $
					'It is incompatible ', $
					'with "Make a difference".'])


				; undo difference event, put back to normal
			        widget_control, norm, set_button=1  
			        widget_control, diff, set_button=0 
				return
			    endif

;			    colreset=1  	; reset colors
			 endif


			 coltyp=1  &  norm_diff=1  
			 widget_control, scal, sensitive=1  ; now an option
		      endif

	'SOLNORTH' :  if event.select eq 0 then donorth=0 else donorth=1
	'SOLDISK'  :  if event.select eq 0 then dodisk=0  else dodisk=1
	'SOLRADII' :  if event.select ne 0 then doradii=1 else doradii=0
	'SOLRADIAL':  if event.select ne 0 then doradial=1 else doradial=0

	'RULNONE'  :  if event.select ne 0 then begin
			ruletimes = 0
		        widget_control, ruleopt2, sensitive=0
                      endif
	'RULONCE'  :  if event.select ne 0 then begin
			ruletimes = 1
		        widget_control, ruleopt2, sensitive=1
                      endif
	'RULMANY'  :  if event.select ne 0 then begin
			ruletimes = 2
		        widget_control, ruleopt2, sensitive=1
                      endif
	'RULINPT'  :  if event.select ne 0 then begin
			ruletimes = 3
		        widget_control, ruleopt2, sensitive=1
                      endif



	'RULIT'     :  if event.select ne 0 then rulecondition=0
	'MARKIT'    :  if event.select ne 0 then rulecondition=1
	'NORULMARK' :  if event.select ne 0 then rulecondition=2

        'AUTOSCL'   :  if event.select ne 0 then begin   ; default clip
                         negfac = 65.  &  posfac = 65.
		       endif 	
	'MANUSCL'   :  if event.select ne 0 then begin
			widget_control, base, sensitive=0 ; no BASE inputs now
                        negfac = 65.  &  posfac = 65.   	; default clip

                        ; make a widget to ask for the scaling factors
          		inpscl = widget_base(title='Enter difference factor',$
				 /column, space=10, xpad=20, ypad=20)

	    		go_on = widget_button(inpscl, value='READY', $
					              uvalue='KILLSCL')
	    		lab = widget_label(inpscl, value='Defaults are 65')
	    		tex = widget_slider(inpscl, title='Positive (red) '+$
			      'clipping value', minimum=1, maximum=254, $
			      value=65, xsize=700, uvalue = 'POSINP') 
		    	tex = widget_slider(inpscl, title='Negative (blue) '+$
			      'clipping value', minimum=1, maximum=254, $
			      value=65, xsize=700, uvalue = 'NEGINP')

	    		widget_control, inpscl, /realize
	    		xmanager,'cp', inpscl, /modal, group_leader=event.top
		       endif           

	'ONEIMAG' : begin
           sin_mov=0				; single image
	   onewin = -99				; no cp display window yet
	   cpone, group_leader=event.top 		; go display single image
	   end

        'MOVIEIM' : begin
	   sin_mov=1				; doing a movie
	   cpmovie, group_leader=event.top 		; go make a movie
           end
	else : widget_control, blurb, set_value='Invalid BUTTON value in CP:'+$
					         input, /append
	endcase   	; of input            

    'SLIDER': case input of 		; only if diff and manual scale factor
	'POSINP' : begin
	              widget_control, event.id, get_value=value
		      posfac = float(value) 
                   end
	'NEGINP' : begin
	              widget_control, event.id, get_value=value
		      negfac = float(value)
                   end                            
	else : widget_control, blurb, set_value='Invalid SLIDER value'+input
	endcase   	; of input


    else : widget_control, blurb, set_value= 'Invalid type '+$
    					    'of widget in CP: ' + type, /append

    endcase      	; of type
end

;==============================================================================

;+
;
; NAME: 
;	CP
;
; PURPOSE:
;	Create the main options menu widget
;
; CATEGORY:  
;	CP display
;
; CALLING SEQUENCE:
;	CP, [GROUP_LEADER=GROUP]
;
; CALLED BY: 
;	none
;
; CALLS TO: 
;	CP_event (via XMANAGER)
;
; INPUTS: 
;	none
;
; OPTIONAL INPUTS:
;	GROUP :  The widget ID of the parent widget.
;
; OUTPUTS: 
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	CPCOLORS     	assigns the CP color tables
;	CPKEYWORDS  	assigns the initial values for option conditions
;	CPLOGIC_ENVRON	assigns the location of data
;	CPNEW_WINMDOW  	assigns initial values for window management
;	CPPROLOCAL 	creates the widget IDs of menu buttons
;	CPWIDGBASES 	creates the widget IDs of parents and informational 
;
; SIDE EFFECTS:  
;	none
;
; RESTRICTIONS:  
;	none
;
; PROCEDURE:
;	Verifiy data loction and create CP color tables then create the 
;	main options menu widget. 
;
; MODIFICATION HISTORY:
; 	1991 - Elaine Einfalt (HSTX)
;
;-


pro cp,group_leader=group

@cpwidgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal
@cpprolocal.common
;common toggles, norm, diff, snorth, sdisk, rulenone, rulerit

@cpcolors.common
;common cpcolors, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset
@cpkeywords.common
;common keywords, norm_diff, donorth, dodisk, doradii, doradial, $
;		 ruletimes, rulecondition, posfac, negfac, sin_mov
@cplogic_envron.common
;common logic_envron, dataplace
@cpnew_window.common
common new_window, lastmark, sc, current_window, reuse, first_pass
 
 if xregistered('cp') then return	 ; only want one running at a time

 ; see if logical/environment name is defined
 opsys_lgcl, part1='CP', discern=1, part2='DATA', separate=1, 		      $
	dataplace=dataplace, ok=ok, 					      $
  	instructme=['Enter your designation for the data location and return',$
  		    '     VMS    ex: NEWMAX::CP$DATA, $1$DIA4:[CP]',          $
  		    '     ULTRIX ex: $TEST_DATA, /usr/users/Data']


 if not(ok) then begin
	acknowledge, message=(['Progam is returning', $
			       'because of cancel'])
	return		; no input given to look for the data
 endif

; windowing setup
  device, get_screen_size=sc
  lastmark = 0
  reuse = 1
  first_pass = 1

; C/P color table

cpcolor, diff_r=diff_r, diff_g=diff_g, diff_b=diff_b, $
	 norm_r=norm_r, norm_g=norm_g, norm_b=norm_b

; initialize keywords
coltyp=0     &  colreset = 1 
norm_diff=0  &  donorth=1  &  dodisk=1  &  doradii=0   &  doradial=0
ruletimes=0  &  rulecondition=0         &  posfac=65.  &  negfac=65.

base = widget_base(title='C/P image display routine', /column, space=20, $
					xpad=20, ypad=20)
 dispway = widget_base(base, /row, space=20)
   oneim = widget_button(dispway, value='Display one image', uvalue='ONEIMAG')
   movie = widget_button(dispway, value='Show a movie', uvalue='MOVIEIM')
 
 normdiffopt = widget_base(base, /column, /frame, /exclusive)
   norm = widget_button(normdiffopt, value='Normal display ', uvalue='NORMAL')
   diff = widget_button(normdiffopt, value='Make a difference', uvalue='DIFFERE')

 drawopt = widget_base(base, /column, /frame, /nonexclusive)
   snorth = widget_button(drawopt, value='Draw solar north', uvalue='SOLNORTH')
   sdisk = widget_button(drawopt, value='Draw solar disk', uvalue='SOLDISK')
   sradii = widget_button(drawopt, value='Draw solar radii lines', $
						uvalue='SOLRADII')
   sradial = widget_button(drawopt, value='Draw radial lines', $
						 uvalue='SOLRADIAL')

 rulopts = widget_base(base, /column, /frame)
 ruleopt1 = widget_base(rulopts, /column, /frame, /exclusive)
   rulenone = widget_button(ruleopt1, value='No R and theta', uvalue='RULNONE')
   ruleone = widget_button(ruleopt1, value='Pick one R and theta', $
				     uvalue='RULONCE')
   rulemulti = widget_button(ruleopt1, value='Pick multiple R and theta', $
				     uvalue='RULMANY')
   ruleinput = widget_button(ruleopt1, value='User input R and theta', $
				     uvalue='RULINPT')

 ruleopt2 = widget_base(rulopts, /column, /frame, /exclusive)
   rulerit = widget_button(ruleopt2, value='Draw R and theta ruler', $
				     uvalue='RULIT')
   markit = widget_button(ruleopt2, value='Mark R and theta click', $
				     uvalue='MARKIT')
   norule = widget_button(ruleopt2, value='No R and theta ruler or mark', $
				     uvalue='NORULMARK')
   
 scal = widget_base(base, /column, /frame, /exclusive)
   auto = widget_button(scal, value='Scale data to maximum', uvalue='AUTOSCL')
   inscl = widget_button(scal, value='Input data scale maximum', $
					uvalue='MANUSCL')
 
 inf = widget_base(base, /frame, /column)
  orm = widget_label(inf, value='Informational comments')
  blurb = widget_text(inf, /scroll, ysize=3)

 xpdmenu, ['"Windowing" {', $
		'"Delete windows" DELWIN', $
		'"Reuse display window" REUSE',$
		'"New display windows" NOREUSE', '}' ], base

 endit = widget_button(base, value='Cancel program', uvalue='QUITPRO')

 widget_control, base, /realize

 widget_control, norm, set_button=1
 widget_control, snorth, set_button=1
 widget_control, sdisk, set_button=1
 widget_control, rulenone, set_button=1
 widget_control, ruleopt2, sensitive=0
 widget_control, rulerit, set_button=1
 widget_control, scal, sensitive=0
 widget_control, auto, set_button=1
 

 xmanager,'cp', base, group_leader=group

 first_pass = 1

end

