;+
;
; NAME:  
;	CPCOLOR
;
; PURPOSE:   
;	Creates arrays for CP black and white and CP differcence color tables
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;	CPCOLOR, DIFF_R=DIFF_R, DIFF_G=DIFF_G, DIFF_B=DIFF_B, $
;	         NORM_R=NORM_R, NORM_G=NORM_G, NORM_B=NORM_B
;
; CALLED BY:  
;	CP.PRO
;
; CALLS TO:  
;	none
;
; INPUTS: 
;	none
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS: 
;	DIFF_R = red color array for difference images
;	DIFF_G = green array for difference images
;	DIFF_B = blue  array for difference images
;	NORM_R = red color array for normal (not difference) images
;	NORM_G = green color array for normal (not difference) images
;	NORM_B = blue color array for normal (not difference) images
;
; OPTIONAL OUTPUTS: none
;
; COMMON BLOCKS: 
;	none, but once the outputs are returned, they are in common CPCOLORS.
;
; SIDE EFFECTS: 
;	Temporarily creates a small IDL window. Users will see it "blip"
;	on the screen.  Necessary to obtain a corrrect value in !D.N_COLORS.
;
; RESTRICTIONS:
;	Must have X window system graphic device.
;
; PROCEDURE:
;	Create color array of size !D.N_COLORS. For normal (not difference)
;	images, the color table is linear black to white, with white at the
;	top index and red one index below that. For difference images the color 
;	table goes from blue to black (mid point index) to red to yellow, with
;	black at lowest index and white at maximun index.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;-

pro cpcolor, diff_r=diff_r, diff_g=diff_g, diff_b=diff_b, $
	     norm_r=norm_r, norm_g=norm_g, norm_b=norm_b


 window,/free,xsize=10,ysize=10 & wdelete,!d.window  ; get # colors the system

; Normal color table, black and white with white line at max color index 
; and red line at max index minus 1

 norm_r=indgen(!d.n_colors)  &   norm_g=norm_r  &  norm_b=norm_r   ; colors to max available

 ; white line at max value
 norm_r(!p.color)=255  &  norm_g(!p.color)= 255  &  norm_b(!p.color)=255  

 ; red line at max value minus 1, used for plotting highlights
 norm_r(!p.color-1)=255  &  norm_g(!p.color-1)=0  &  norm_b(!p.color-1)=0

; savbwr=bwr  &  savbwg=bwg  &  savbwb=bwb           ; save b/w colors


; Difference color table, intense blue to black at mid-index value(s) to
; red to yellows. At max index is a white line.

 diff_r=intarr(!d.n_colors)    &  diff_g=diff_r  &  diff_b=diff_r    ; colors to max available
   
 mid=(!p.color)/2.                      ; middle value of blue/red scale
 ; if not integer make two black at middle to center arrays
 if (mid mod 1) ne 0 then begin lomid=fix(mid)  &  himid=lomid+1
     	       endif else begin lomid=mid       &  himid=mid      & endelse

 reds = indgen(lomid) * 2                ; array values from 0 to ~max
 blues=max(reds) - reds 	 	 ; array values from ~max to 0
 reds=reds<255 & blues=blues<255
 diff_b(0) = blues  &  diff_r(himid+1)=reds    ; fill in color gun arrays

 greennum = 20                           ; top indexs will be shades of yellow
 startgreen= fix(!p.color - greennum)
 green = (indgen(greennum)+1) * (255./greennum)
 diff_g(startgreen) = green                 ; fill in color gun arrays

 ; white line at max value
 diff_r(!p.color) = 255   &  diff_g(!p.color)= 255  &  diff_b(!p.color)=255 
 
 ; black line at min value
 diff_r(0) = 0  &  diff_g(0)= 0  &  diff_b(0)=0 
 
; savbrr=brr  &  savbrg=brg  &  savbrb=brb          ; save difference colors
end

