;+
;
; NAME: 
;	CPHELP2
;
; PURPOSE:
;       Return a string array containing the requested help information.
;
; CATEGORY:
;	CP display
;
; CALLING SEQUENCE:
;       result = CPHELP(VALUE)
;
; CALLED BY:
;       CPHELP
;
; CALLS TO:
;	none
;
; INPUTS:
;       VALUE	:  the string value of the help item requested
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;       Return a string array with help information
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	none
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	The value passed is the exact wording of the topic as it appears
;	in the help widget.  A change to the wording of the help topic
;	(when the widget is created - in CPHELP) or to the widget event
;	handler case value (in CPHELP_EV) or to the case values in CPHELPn
;	(where n is a number) will require the same changes occur in all 3
;	locations.
;
; PROCEDURE:
;	Return the help information associated with the input value.
;
; MODIFICATION HISTORY:
;	Oct 93 - Elaine Einfalt (HSTX)
;-


function cphelp2, value

case value of 
    '"Quit"' : hval = [ $
	'QUIT cause the C/P display program to terminate and    ',$
	'removes any C/P menues that are still active.  Any C/P ', $
	'image windows will remain.                             ', $
	'                                                       ']
    'Output accessories' : hval = [ $
        'Output accessories                                     ', $ 
	'                                                       ', $
	'All, none or any of this options may be selected at a  ', $
	'time.  They provide helpful measuring guide lines.     ', $
	'The next 4 topics discuss these options.               ', $
	'                                                       ']
    '"Draw solar north"' : hval = [ $
        '"Draw solar north"                                     ', $
	'                                                       ', $
        'When this item is set, a line will be drawn on the     ', $
	'occulting disk representing solar north.               ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
    '"Draw solar disk"' : hval = [ $
        '"Draw solar disk"                                      ', $
	'                                                       ', $
        'When this item is set, a circle will be drawn on the   ', $
	'occulting disk representing solar disk.                ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
    '"Draw solar radii lines"' : hval = [ $
        '"Draw solar radii lines"                               ', $
	'                                                       ', $
        'When this item is set, arc will be drawn on the image  ', $
	'representing solar radii.  Solid line represent whole  ', $
	'value radii and dash lines represent 0.5 radii.        ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']
    '"Draw radial lines"' : hval = [ $
        '"Draw radial lines"                                    ', $
	'                                                       ', $
        'When this item is set, lines will be drawn on the image', $
	'representing radial lines from sun center.  Each line  ', $
	'has 15 dergrees difference.                            ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ']

    'Input Radius and position angle' : hval = [ $
	'Input Radius and position angle                        ', $
	'                                                       ', $
	'Allows you to identify the radius and postion angle for', $
	'an input.  This input is most often a mouse click that ', $
        'shows the R and Theta.  But you may type in the R and  ', $
	'Theta to show it''s position.   The methods used to    ', $
	'show R and Theta are discussed in "Output R and Theta" ', $
      	'                                                       ', $
        'It is only possible to select one of these options at a', $
	'time.  While "No R and theta" is selected then none of ', $
	'the R and theta output modes (see Output R and Theta)  ', $
	'will be available.                                     ', $
      	'                                                       ', $
        'If all of the selection options are turned off, then   ', $
	'the last chosen option is still in effect.             ', $
	'                                                       ']
    '"No R and theta"' : hval = [ $
        '"No R and theta"                                       ', $
	'                                                       ', $
	'When set, this option prevents the program from asking ', $
	'for or displaying any radius or position angle data.   ', $
	'                                                       ', $
	'Note: When this option is selected then R and theta    ', $
	'      output mode are not available.                   ', $
	'                                                       ']
    '"Pick one R and theta"' : hval = [ $
        '"Pick one R and theta"                                 ', $
	'                                                       ', $
	'When set, this option will request that you use the    ', $
	'mouse to click on the image at the desired location.   ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'                                                       ', $
	'The values for the radius and position angle will be   ', $
	'printed to the message window.  Also the appropriate   ', $ 
	'output action will be preformed (see Output R and      ', $
	'theta).                                                ', $
	'                                                       ', $
	'If you decide to cancel the R and theta request, after ', $
	'the program prompts you for the input, then click the  ', $
	'mouse at sun center.                                   ', $
	'                                                       ']
    '"Pick multiple R and theta"' : hval = [ $
        '"Pick multiple R and theta"                            ', $
	'                                                       ', $
	'When set, this option will request that you use the    ', $
	'mouse to click on the image at the desired location.   ', $
	'The program will continue to ask for cursor clicks     ', $
	'until you click at sun center.                         ', $
	'                                                       ', $
	'If the desired image is already displayed then you need', $
	'redisplay the image.                                   ', $
	'This is only necessary one time, not between   ', $
	'each of the multiple inputs.                           ', $
	'                                                       ', $
	'The values for the radius and position angle will be   ', $
	'printed to the message window.  Also the appropriate   ', $ 
	'output action will be preformed (see Output R and      ', $
	'theta).                                                ', $
	'                                                       ', $
	'If you decide to cancel the R and theta request, after ', $
	'the program prompts you for the input, then click the  ', $
	'mouse at sun center.                                   ', $
	'                                                       ']
    '"User input R and theta"' : hval = [ $
        '"User input R and theta"                               ', $
	'                                                       ', $
	'This option is used when you know the R and theta and  ', $
	'want to see it displayed on the image.                 ', $
	'                                                       ', $
	'A widget will appear requesting the radius and position', $
	'angle.  Click in the text window to input the values,  ', $
	'and remember to press RETURN.                          ', $
	'                                                       ', $
	'After every pair of values you input you must press the', $
	'READY button.  The select R and theta pair will be     ', $
	'display immediately, there is no need to redisplay the ', $
	'image.                                                 ', $
	'                                                       ', $
	'After the program displays the current R and Theta pair', $
	'according to the output mode (see Output R and theta), ', $
        'the program redisplays the input widget and will       ', $
	'continue to ask for input until you terminate by       ', $
	'entering values of 0 and 0 or press the CANCEL button. ', $
	'                                                       ', $
	'NOTE: The R and theta values are erased from memory if ', $
	'      the "CANCEL" button is selection in the "User    ', $
	'      input R and theta" option (see "Use R and theta  ', $
	'      in memory").                                     ', $
	'                                                       ']
   '"Use R and theta in memory"' : hval = [ $
        '"Use R and theta in memory"                            ', $
	'                                                       ', $
	'When this option is selected, the last select set of   ', $
	'R and theta pairs are recalled from memory and used as ', $
	'the current input values.  The set of values may be    ', $
	'a single pair or multiple pairs, depending on your     ', $
	'last input series.                                     ', $
	'                                                       ', $
	'The image displayed may be newly selected, redisplayed ', $
	'or new movies.                                         ', $
	'                                                       ', $
	'NOTE: The R and theta values are erased from memory if ', $
	'      the "CANCEL" button is selection in the "User    ', $
	'      input R and theta" option.                       ', $
	'                                                       ']

  else : hval = 'Topic ' + value + ' is not documented in CPHELP2.PRO'
endcase

return, hval
end
