pro cmehp, quan, quanname, quanunit, plotlab, yearlab, xtype, ytype

;+
; Plot SMM C/P CME Histograms.
; quan: fltarr of values.
; quanname: name of quantity (e.g., MASS, KINETIC ENERGY, etc.)
; quanunit: units of quan array.
; plotlab: plot title.
; yearlab: string label for plot describing years covered.
; xtype: 0 = linear, 1 = log
; ytype: 0 = linear, 1 = log
;
; Andrew L. Stanger   HAO/NCAR   19 August 1994
;-

;print, 'quanname: ', quanname
;print, 'quanunit: ', quanunit
print, 'plotlab   ', plotlab
;print, 'yearlab   ', yearlab

; CME excess mass (no jupiter correction; coefficient of 1.e+14 grams).

squan = size (quan)
nquan = squan (1)
ncme  = nquan

;print, 'nquan: ', nquan
;print, 'quan: '
;print, quan

; Compute log (quan).

lquan = fltarr (nquan)
lquan = alog10 (quan)				; log (quan)

;for i = 0, ncme - 1 do $
;   print, i, quan (i), lquan (i)

; Compute average CME quan.

quantot = 0.0
for i = 0, ncme - 1 do quantot = quantot + quan (i)
quanave = quantot / ncme 
lquanave = alog10 (quanave)
lyave    = alog10 (0.2)
;print, 'total CME quan: ', quantot, '   Average CME quan: ', quanave

; Create histogram bins.

IF (xtype eq 1) THEN $				; x logarithmic.
BEGIN
   xmin = float (fix (min (lquan)))
   xmax = fix (max (lquan)) + 1.0
   xinc = 0.2					; Increment exponent of 10.

   nbin = (xmax - xmin) / xinc + 1
;   print, 'xmax-xmin: ', xmax-xmin, ' xinc: ', xinc, ' nbin: ', nbin
   bin = intarr (nbin + 1)
   binval = fltarr (nbin + 1)

   binexp0 = xmin
   for i = 0, nbin do $
      begin
         binexp = binexp0 + i * xinc
         binval (i) = binexp
      end

   ; Fill bins.

   for i = 0, ncme - 1 do $
      for n = 0, nbin - 1 do $
         if (lquan (i) gt binval (n) and lquan (i) le binval (n+1)) then $
            bin (n) = bin (n) + 1

   dxmin = xmin 
   dxmax = xmax 

END $
ELSE $						; x linear.
BEGIN
   xmin = 0.0
   lmax = fix (max (lquan)) + 1
   xmax = 10.0 ^ lmax
   xinc = xmax / 100.0 

   nbin = ((xmax - xmin) / xinc) + 2
   bin    = intarr (nbin + 1)
   binval = fltarr (nbin + 1)

   bin0 = xmin
   for i = 0, nbin do $
      begin
	 binval (i) = bin0 + i * xinc
      end

   ; Fill bins.

   for i = 0, ncme - 1 do $
      for n = 0, nbin - 1 do $
         if (quan (i) gt binval (n) and quan (i) le binval (n+1)) then $
            bin (n) = bin (n) + 1

   dxmin = xmin 
   dxmax = xmax 

END

;for i = 0, nbin - 1 do $
;   print, 'bin: ', i, '   binval: ', binval (i), '   binnum: ', bin (i)  

sum = 0 
for i = 0, nbin - 1   do sum = sum + bin (i)
;print, 'ncme: ', ncme, '   sum (histogram): ', sum

; Plot histogram.

xval = binval + xinc / 2.0
yval = (bin / float (ncme)) * 100.0
sxval = size (xval)
nxval = sxval (1)
syval = size (yval)
nyval = syval (1)

;print, 'nxval/nyval: ', nxval, nyval

;plotlab = ' CME Mass Histogram   SMM C/P'

IF (xtype eq 0 and ytype eq 0) THEN $		; x linear, y linear.
   BEGIN
   xlab = quanname + '[' + quanunit + ']'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = min (yval) 
   ymax = max (yval) 
;   print, 'xval:'
;   print, xval
;   print, 'yval:'
;   print, yval
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
         TITLE  = plotlab, $
         XTITLE = xlab, $
         YTITLE = ylab, $
;	 POSITION = [0.10, 0.00, 0.9, 0.50], $
         XTHICK = 5.0, YTHICK = 5.0, $
	 XTICKLEN = -0.02, YTICKLEN = -0.02, $
         PSYM   = 10    			; Histogram plot mode.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, quanave, 2.0, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.8
   PLOTS, [quanave, quanave], [ymin, ymax], LINESTYLE = 2
   END $
ELSE $
IF (xtype eq 1 and ytype eq 0) THEN $		; x log, y linear.
   BEGIN
   xlab = 'log (' + quanname + ') [' + quanunit + ']'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
;   dxmin = 27.0
;   dxmax = 33.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
         TITLE  = plotlab, $
         XTITLE = xlab, $
         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
	 XTICKLEN = -0.02, YTICKLEN = -0.02, $
         PSYM   = 10    			; Histogram plot mode.
;	 /XTYPE         			; X axis logarithmic.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0 
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, lquanave, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.8
   PLOTS, [lquanave, lquanave], [ymin, ymax], LINESTYLE = 2
   END $
ELSE $
IF (xtype eq 0 and ytype eq 1) THEN $		; x linear, y log.
   BEGIN
   xlab = quanname + ' [' + quanunit + ']'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme) ) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
         TITLE  = plotlab, $
         XTITLE = xlab, $
         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
	 XTICKLEN = -0.02, YTICKLEN = -0.02, $
         PSYM   = 10,   $			; Histogram plot mode.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, quanave, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.8
   PLOTS, [quanave, quanave], [ymin, ymax], LINESTYLE = 2
   END $
ELSE $
IF (xtype eq 1 and ytype eq 1) THEN $		; x log, y log.
   BEGIN
   xlab = 'log (' + quanname + ') [' + quanunit + ']'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 1.0e-1 
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
         TITLE  = plotlab, $
         XTITLE = xlab, $
         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
	 XTICKLEN = -0.02, YTICKLEN = -0.02, $
         PSYM   = 10,   $			; Histogram plot mode.
;	 /XTYPE,        $			; X axis logarithmic.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, lquanave, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.8
   PLOTS, [lquanave, lquanave], [ymin, ymax], LINESTYLE = 2
   END

;OPLOT, xval, yval, PSYM   = 5			; overplot with triangles.

;for i = 0, nbin - 1   do $
;   box, xval (i), !Y.CRANGE (0), xval (i) + xval (i+1), yval (i), 45 

; Write year label on plot.

if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.85, yearlab, /NORMAL, CHARSIZE = 1.0 $
else  $
   xyouts, 0.21, 0.85, yearlab, /NORMAL, CHARSIZE = 1.0

; Write # CMEs measured on plot.

cmelab = string (ncme, FORMAT = '(I3)') + ' CMEs measured'
if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.75, cmelab, /NORMAL, CHARSIZE = 0.8 $
else $
   xyouts, 0.21, 0.75, cmelab, /NORMAL, CHARSIZE = 0.8

; Write average quan on plot.

avelab = 'mean = ' + string (quanave, FORMAT = '(E8.2)') + ' ' + quanunit
print, 'quanave: ', quanave
if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.70, avelab, /NORMAL, CHARSIZE = 0.8 $
else $
   xyouts, 0.21, 0.70, avelab, /NORMAL, CHARSIZE = 0.8

;FOR i = 0, nbin - 1 DO $
;   print, i, '   binval: ', binval (i), $
;	     '   xval: ', xval (i), '   yval: ', yval (i)
end
 
; --- Histogram fill procedure -------------------------------------------------

pro box, x0, y0, x1, y1, line
;   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0], ORIENTATION = line 
   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0], COLOR = 191
end
