pro cmelat    

;+
; Scatter plots: width, speed, mass, energy versus latitude. 
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   14 December 1994
;-

@cpmass.common

rdcpmass

; Plot to postscript file.

SET_PLOT, 'PS'
!P.FONT = 0

!P.MULTI = [0, 0, 4, 0, 0]		; four plots per page.

;XSIZE = 15.24, YSIZE = 12.70, YOFFSET = 12.70	; 6.0"W, 5"H, 5" bottom margin

DEVICE, /HELVETICA, /BOLD, FONT_SIZE = 18, FILENAME = 'cmelat.ps', $
XSIZE = 8.00, XOFFSET = 11.30, YSIZE = 23.00, YOFFSET = 1.90	; 8cm W, 23cm H, 1.9cm bottom margin

;DEVICE, /TIMES, /BOLD, /COLOR, FONT_SIZE = 18, FILENAME = 'eh.ps'
;loadct, 0			; B-W
;loadct, 1			; BLUE-WHITE
;loadct, 2			; GRN-RED-BLU-WHT
;loadct, 3			; RED TEMPERATURE
;loadct, 4			; BLUE/GREEN/RED/YELLOW
;loadct, 5			; STD GAMMA-II
;loadct, 6			; PRISM
;loadct, 7			; RED-PURPLE
;loadct, 8			; GREEN/WHITE LINEAR
;loadct, 9			; GRN/WHT EXPONENTIAL
;loadct, 10 			; GREEN-PINK
;loadct, 11			; BLUE-RED
;loadct, 12			; 16 LEVEL
;loadct, 13			; RAINBOW

np = 7
A = (FINDGEN (np) + 0) * (!PI * 2 / float (np-1))
;USERSYM, COS (A), SIN (A), /FILL
USERSYM, COS (A), SIN (A)
;x = [0,1,1,0,0]				; small square
;y = [0,0,1,1,0]
x = [-0.5,    0,  0.5,    0, -0.5]		; small diamond
y = [   0,  0.5,    0, -0.5,    0]
USERSYM, x, y

; Scatter Plot: latitude vs. width

poslat = abs (lat)
sizeposlat = size (poslat)
nposlat = sizeposlat (1)

sizewidth = size (width)
nwidth = sizewidth (1)

print, 'nposlat: ', nposlat, ' nwidth: ', nwidth
print, width
for i=0, nwidth-1 do if (width (i) GT 200)  then $
   print, 'i: ', i, ' width: ', width (i)

ymin = min (width)
ymax = max (width)
print, 'min(width): ', ymin, ' max(width): ', ymax
PLOT, poslat, width, /YNOZ, PSYM = 8, $
     YRANGE = [ymin, ymax], $
;     TITLE='Latitude vs Width', $
     XTITLE='LATITUDE  [degrees]', $
     YTITLE = 'WIDTH  [degrees]', $
     XTHICK = 5.0, YTHICK = 5.0, $
     XTICKLEN = -0.02, YTICKLEN = -0.02

rlw = CORRELATE (poslat, width)
srlw = string (rlw, FORMAT = '(F6.3)')
print, 'correlation (poslat, width): ', rlw
;xyouts, 0.21, 0.86, '1980, 1984-1988', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.82, 'corr. coeff = ' + srlw, /NORM, CHARSIZE = 0.8

; Scatter Plot: latitude vs. speed

PLOT, poslat, speed, /YNOZ, PSYM = 8, $
;     YRANGE = [29.0, 33.0], $
;     TITLE='Latitude vs Speed', $
     XTITLE='LATITUDE  [degrees]', $
     YTITLE = 'SPEED  [km/sec]', $
     XTHICK = 5.0, YTHICK = 5.0, $
     XTICKLEN = -0.02, YTICKLEN = -0.02

rls = CORRELATE (poslat, speed)
srls = string (rls, FORMAT = '(F6.3)')
print, 'correlation (poslat, speed): ', rls
;xyouts, 0.21, 0.86, '1980, 1984-1988', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.82, 'corr. coeff = ' + srls, /NORM, CHARSIZE = 0.8

; Scatter Plot: latitude vs. mass

xm = xmass * 1000.0				; mass [kg -> grams]
lxmass = alog10 (xmass * 1000.0)		; log (mass) [grams].

ymin = 13.5
ymax = 17.0

PLOT, poslat, lxmass, /YNOZ, PSYM = 8, $
     YRANGE = [ymin, ymax], $
;     TITLE='Latitude vs Mass', $
     XTITLE='Latitude [degrees]', $
     YTITLE = 'Log (Mass)   [grams]', $
     XTHICK = 5.0, YTHICK = 5.0, $
     XTICKLEN = -0.02, YTICKLEN = -0.02

rlm = CORRELATE (poslat, lxmass)
srlm = string (rlm, FORMAT = '(F6.3)')
print, 'correlation (poslat, lxmass): ', rlm
;xyouts, 0.21, 0.86, '1980, 1984-1988', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.82, 'corr. coeff = ' + srlm, /NORM, CHARSIZE = 0.8

; Convert joules to ergs.

ke = ke * 1.e+7
pe = pe * 1.e+7
te = te * 1.e+7

ke80 = ke80 * 1.e+7
ke84 = ke84 * 1.e+7
ke85 = ke85 * 1.e+7
ke86 = ke86 * 1.e+7
ke87 = ke87 * 1.e+7
ke88 = ke88 * 1.e+7

pe80 = pe80 * 1.e+7
pe84 = pe84 * 1.e+7
pe85 = pe85 * 1.e+7
pe86 = pe86 * 1.e+7
pe87 = pe87 * 1.e+7
pe88 = pe88 * 1.e+7

te80 = te80 * 1.e+7
te84 = te84 * 1.e+7
te85 = te85 * 1.e+7
te86 = te86 * 1.e+7
te87 = te87 * 1.e+7
te88 = te88 * 1.e+7

; Get size of arrays.

sizlat = size (lat)
nlat   = sizlat (1)
poslat = abs (lat)

sizke  = size (ke)
sizpe  = size (pe)
sizte  = size (te)

sizeke80 = size (ke80)
sizeke84 = size (ke84)
sizeke85 = size (ke85)
sizeke86 = size (ke86)
sizeke87 = size (ke87)
sizeke88 = size (ke88)

sizepe80 = size (pe80)
sizepe84 = size (pe84)
sizepe85 = size (pe85)
sizepe86 = size (pe86)
sizepe87 = size (pe87)
sizepe88 = size (pe88)

sizete80 = size (te80)
sizete84 = size (te84)
sizete85 = size (te85)
sizete86 = size (te86)
sizete87 = size (te87)
sizete88 = size (te88)

nke   = sizke  (1)
npe   = sizpe  (1)
nte   = sizte  (1)
nval  = nke

nke80 = sizeke80 (1)
nke84 = sizeke84 (1)
nke85 = sizeke85 (1)
nke86 = sizeke86 (1)
nke87 = sizeke87 (1)
nke88 = sizeke88 (1)

npe80 = sizepe80 (1)
npe84 = sizepe84 (1)
npe85 = sizepe85 (1)
npe86 = sizepe86 (1)
npe87 = sizepe87 (1)
npe88 = sizepe88 (1)

nte80 = sizete80 (1)
nte84 = sizete84 (1)
nte85 = sizete85 (1)
nte86 = sizete86 (1)
nte87 = sizete87 (1)
nte88 = sizete88 (1)

;print, 'nxmass: ', nxmass, '   nlat: ', nlat

;for i = 0, nxmass - 1  do $
;   print, 'i: ', i, '   xmass: ', xmass (i), '   lat: ', lat (i)

; Create new arrays containing non-zero elements of energy arrays.

ki = WHERE (ke, npke)
pi = WHERE (pe, nppe)
ti = WHERE (te, npte)

poske = ke (ki)
pospe = pe (pi)
poste = te (ti)

ki80 = WHERE (ke80, npke80)
ki84 = WHERE (ke84, npke84)
ki85 = WHERE (ke85, npke85)
ki86 = WHERE (ke86, npke86)
ki87 = WHERE (ke87, npke87)
ki88 = WHERE (ke88, npke88)

pi80 = WHERE (pe80, nppe80)
pi84 = WHERE (pe84, nppe84)
pi85 = WHERE (pe85, nppe85)
pi86 = WHERE (pe86, nppe86)
pi87 = WHERE (pe87, nppe87)
pi88 = WHERE (pe88, nppe8)

ti80 = WHERE (te80, npte80)
ti84 = WHERE (te84, npte84)
ti85 = WHERE (te85, npte85)
ti86 = WHERE (te86, npte86)
ti87 = WHERE (te87, npte87)
ti88 = WHERE (te88, npte88)

pke80 = ke80 (ki80)
pke84 = ke84 (ki84)
pke85 = ke85 (ki85)
pke86 = ke86 (ki86)
pke87 = ke87 (ki87)
pke88 = ke88 (ki88)

ppe80 = pe80 (pi80)
ppe84 = pe84 (pi84)
ppe85 = pe85 (pi85)
ppe86 = pe86 (pi86)
ppe87 = pe87 (pi87)
ppe88 = pe88 (pi88)

pte80 = te80 (ti80)
pte84 = te84 (ti84)
pte85 = te85 (ti85)
pte86 = te86 (ti86)
pte87 = te87 (ti87)
pte88 = te88 (ti88)

; Create subset arrays for measurements in which both K.E. and P.E. are > 0.

ip  = 0
poske  = fltarr (nval)
pospe  = fltarr (nval)
poslat = intarr (nval)

for i = 0, nval - 1  do $
begin
   if (ke (i) gt 0.0 and pe (i) gt 0.0) then $
   begin
      poske (ip) = ke (i)
      pospe (ip) = pe (i)
      poslat (ip) = abs (lat (i))
      ip = ip + 1
   endif
endfor

np = ip
print, 'np: ', np

pke = poske (0: np - 1)
ppe = pospe (0: np - 1)
pte = pke + ppe
plat = poslat (0: np - 1)

; log (ke), log (pe), log (te)

lke = alog10 (pke)			; log (K.E.) [ergs].
lpe = alog10 (ppe)			; log (P.E.) [ergs].
lte = alog10 (pte)			; log (T.E.) [ergs].

ske1 = 0			; Sum of kinetic energy with  0 < lat < 20
ske2 = 0			; Sum of kinetic energy with 20 < lat < 40
ske3 = 0			; Sum of kinetic energy with 20 < lat < 90

spe1 = 0			; Sum of potential energy with  0 < lat < 20
spe2 = 0			; Sum of potential energy with 20 < lat < 40
spe3 = 0			; Sum of potential energy with 20 < lat < 90

ste1 = 0			; Sum of total energy with  0 < lat < 20
ste2 = 0			; Sum of total energy with 20 < lat < 40
ste3 = 0			; Sum of total energy with 20 < lat < 90

nke1 = 0			; Number of ke measurements  0 < lat < 20
nke2 = 0			; Number of ke measurements 20 < lat < 40
nke3 = 0			; Number of ke measurements 40 < lat < 90

npe1 = 0			; Number of pe measurements  0 < lat < 20
npe2 = 0			; Number of pe measurements 20 < lat < 40
npe3 = 0			; Number of pe measurements 40 < lat < 90

nte1 = 0			; Number of te measurements  0 < lat < 20
nte2 = 0			; Number of te measurements 20 < lat < 40
nte3 = 0			; Number of te measurements 40 < lat < 90

ke1 = fltarr (nval)		; Kinetic Energy values with  0 < lat < 20
ke2 = fltarr (nval)		; Kinetic Energy values with 20 < lat < 40
ke3 = fltarr (nval)		; Kinetic Energy values with 40 < lat < 90

pe1 = fltarr (nval)		; Potential Energy values with  0 < lat < 20
pe2 = fltarr (nval)		; Potential Energy values with 20 < lat < 40
pe3 = fltarr (nval)		; Potential Energy values with 40 < lat < 90

te1 = fltarr (nval)		; Total Energy values with  0 < lat < 20
te2 = fltarr (nval)		; Total Energy values with 20 < lat < 40
te3 = fltarr (nval)		; Total Energy values with 40 < lat < 90

i1 = 0
i2 = 0
i3 = 0

for i = 0, np - 1 do $
begin
   if (plat (i) le 20) then $
   begin
      nke1 = nke1 + 1
      ske1 = ske1 + pke (i) 
      spe1 = spe1 + ppe (i) 
      ste1 = ste1 + pte (i) 
      ke1 (i1) = pke (i)
      pe1 (i1) = ppe (i)
      te1 (i1) = pte (i)
      i1 = i1 + 1
   endif

   if (plat (i) gt 20) and plat (i) le 40 then $
   begin
      nke2 = nke2 + 1
      ske2 = ske2 + pke (i)
      spe2 = spe2 + ppe (i)
      ste2 = ste2 + pte (i)
      ke2 (i2) = pke (i)
      pe2 (i2) = ppe (i)
      te2 (i2) = pte (i)
      i2 = i2 + 1
   endif

   if (plat (i) gt 40) then $
   begin
      nke3 = nke3 + 1
      ske3 = ske3 + pke (i)
      spe3 = spe3 + ppe (i)
      ste3 = ste3 + pte (i)
      ke3 (i3) = pke (i)
      pe3 (i3) = ppe (i)
      te3 (i3) = pte (i)
      i3 = i3 + 1
   endif
endfor

npe1 = nke1
npe2 = nke2
npe3 = nke3

nte1 = nke1
nte2 = nke2
nte3 = nke3

k1 = ke1 (0: nke1 - 1)
k2 = ke2 (0: nke2 - 1)
k3 = ke3 (0: nke3 - 1)

p1 = pe1 (0: npe1 - 1)
p2 = pe2 (0: npe2 - 1)
p3 = pe3 (0: npe3 - 1)

t1 = te1 (0: nte1 - 1)
t2 = te2 (0: nte2 - 1)
t3 = te3 (0: nte3 - 1)

nke = nke1 + nke2 + nke3
ake1 = ske1 / nke1
ake2 = ske2 / nke2 
ake3 = ske3 / nke3
ake = (ske1 + ske2 + ske3) / nke

npe = npe1 + npe2 + npe3
ape1 = spe1 / npe1
ape2 = spe2 / npe2 
ape3 = spe3 / npe3
ape = (spe1 + spe2 + spe3) / npe

nte = nke1 + nke2 + nke3
ate1 = ste1 / nte1
ate2 = ste2 / nte2 
ate3 = ste3 / nte3
ate = (ste1 + ste2 + ste3) / nte

print, '<K.E.> ( 0 < lat < 20) = ', ake1
print, '<K.E.> (20 < lat < 40) = ', ake2
print, '<K.E.> (40 < lat < 90) = ', ake3   
print, '<K.E.>                 = ', ake

print, '<P.E.> ( 0 < lat < 20) = ', ape1
print, '<P.E.> (20 < lat < 40) = ', ape2
print, '<P.E.> (40 < lat < 90) = ', ape3   
print, '<P.E.>                 = ', ape

print, '<M.E.> ( 0 < lat < 20) = ', ate1
print, '<M.E.> (20 < lat < 40) = ', ate2
print, '<M.E.> (40 < lat < 90) = ', ate3   
print, '<M.E.>                 = ', ate

syearlab = '1980, 1984-1988'			; year label.
xtype = 1					; log   : energy.
ytype = 0					; linear: # in bin.
plab = 'Histogram'
kelab = 'Kinetic Energy'			; K.E. label.
pelab = 'Potential Energy'			; P.E. label.
telab = 'Mechanical Energy'	; T.E. label.
elab = 'Energy'
pkelab = plab + ': ' + kelab
ppelab = plab + ': ' + pelab
ptelab = plab + ': ' + telab
eunits = 'ergs'				; Energy units.

pkelab = kelab
ppelab = pelab
ptelab = telab

; Hisograms for Kinetic, Potential & Total Energy; All measurements.

;cmehp, pke, elab, eunits, pkelab, syearlab, xtype, ytype
;cmehp, ppe, elab, eunits, ppelab, syearlab, xtype, ytype
;cmehp, pte, elab, eunits, ptelab, syearlab, xtype, ytype

; Histograms for Kinetic, Potential & Total Energy: by latitude range.

;cmehp, k1, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, ' 0 < lat < 20 deg.', /NORMAL, CHARSIZE = 1.0
;cmehp, k2, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, '20 < lat < 40 deg.', /NORMAL, CHARSIZE = 1.0
;cmehp, k3, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, '40 < lat < 90 deg.', /NORMAL, CHARSIZE = 1.0

;cmehp, p1, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, ' 0 < lat < 20 deg.', /NORMAL, CHARSIZE = 1.0
;cmehp, p2, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, '20 < lat < 40 deg.', /NORMAL, CHARSIZE = 1.0
;cmehp, p3, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, '40 < lat < 90 deg.', /NORMAL, CHARSIZE = 1.0

;cmehp, t1, elab, eunits, ptelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, ' 0 < lat < 20 deg.', /NORMAL, CHARSIZE = 1.0
;cmehp, t2, elab, eunits, ptelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, '20 < lat < 40 deg.', /NORMAL, CHARSIZE = 1.0
;cmehp, t3, elab, eunits, ptelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.8, '40 < lat < 90 deg.', /NORMAL, CHARSIZE = 1.0

; Scatter Plot: latitude vs. mechanical energy

PLOT, plat, lte, /YNOZ, PSYM = 8, $
     YRANGE = [29.0, 33.0], $
;     TITLE='Latitude vs Mechanical Energy', $
     XTITLE='Latitude [degrees]', $
     YTITLE = 'Log (Energy)   [ergs]', $
     XTHICK = 5.0, YTHICK = 5.0, $
     XTICKLEN = -0.02, YTICKLEN = -0.02

rle = CORRELATE (plat, lte)

print, 'correlation (plat, lte): ', rle

sr = string (rle, FORMAT = '(F6.3)')
sate1 = string (ate1, FORMAT = '(E8.2)')
sate2 = string (ate2, FORMAT = '(E8.2)')
sate3 = string (ate3, FORMAT = '(E8.2)')

;xyouts, 0.21, 0.86, '1980, 1984-1988', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.82, 'corr. coeff = ' + sr, /NORM, CHARSIZE = 0.8
;xyouts, 0.50, 0.86, '<M.E.> ( 0<lat<20) = ' + sate1, /NORM, CHARSIZE = 0.8
;xyouts, 0.50, 0.82, '<M.E.> (20<lat<40) = ' + sate2, /NORM, CHARSIZE = 0.8
;xyouts, 0.50, 0.78, '<M.E.> (40<lat<90) = ' + sate3, /NORM, CHARSIZE = 0.8

nspot = [ 92.5, 155.4, 154.6, 140.4, 115.9, $
	  66.6,  45.9,  17.9,  13.4,  29.4, $
	 100.2, 157.6, 142.6, 145.7,  94.5]
yspot = indgen (15) + 1978

ake80 = avg (pke80)
ake84 = avg (pke84)
ake85 = avg (pke85)
ake86 = avg (pke86)
ake87 = avg (pke87)
ake88 = avg (pke88)

ape80 = avg (ppe80)
ape84 = avg (ppe84)
ape85 = avg (ppe85)
ape86 = avg (ppe86)
ape87 = avg (ppe87)
ape88 = avg (ppe88)

print, '1984 positive K.E. :'
print, pke84
print, '1984 positive P.E. :'
print, ppe84
print, '1984 positive M.E. :'
print, pte84

ate80 = avg (pte80)
ate84 = avg (pte84)
ate85 = avg (pte85)
ate86 = avg (pte86)
ate87 = avg (pte87)
ate88 = avg (pte88)

lake80 = alog10 (ake80)
lake84 = alog10 (ake84)
lake85 = alog10 (ake85)
lake86 = alog10 (ake86)
lake87 = alog10 (ake87)
lake88 = alog10 (ake88)

lape80 = alog10 (ape80)
lape84 = alog10 (ape84)
lape85 = alog10 (ape85)
lape86 = alog10 (ape86)
lape87 = alog10 (ape87)
lape88 = alog10 (ape88)

late80 = alog10 (ate80)
late84 = alog10 (ate84)
late85 = alog10 (ate85)
late86 = alog10 (ate86)
late87 = alog10 (ate87)
late88 = alog10 (ate88)

ycme  = [1980, 1984, 1985, 1986, 1987, 1988]
syear = size (ycme)
nyear = syear (1)

;ayke  = [ ake80,  ake84,  ake85,  ake86,  ake87,  ake88]
;aype  = [ ape80,  ape84,  ape85,  ape86,  ape87,  ape88]
;ayte  = [ ate80,  ate84,  ate85,  ate86,  ate87,  ate88]

;layke = [lake80, lake84, lake85, lake86, lake87, lake88]
;laype = [lape80, lape84, lape85, lape86, lape87, lape88]
;layte = [late80, late84, late85, late86, late87, late88]

;print, ycme 
;print, ayke
;print, aype
;print, ayte

; Define circle symbol for plotting.

a = FINDGEN (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), SIN (a), /FILL

; Sunspot number versus time.

;PLOT, yspot, nspot, $
;      PSYM = 8, $					; filled circle
;      XTHICK = 5.0, YTHICK = 5.0, $
;      XTICKLEN = -0.02, YTICKLEN = -0.02, $
;      XRANGE = [1980, 1992], $
;      TITLE = "Sunspot Number vs. Time", $
;      XTITLE = "YEAR", $
;      YTITLE = "Average Monthly Sunspot Number"
;OPLOT, yspot, nspot, LINESTYLE = 0			; solid line

; Energy versus time.

;PLOT, ycme,  ayte, $
;      PSYM = 6, $					; square
;      XTHICK = 5.0, YTHICK = 5.0, $
;      XTICKLEN = -0.02, YTICKLEN = -0.02, $
;      XRANGE = [1980, 1992], $
;      YRANGE = [0, 1.4E+31], $
;      TITLE = "Energy vs. Time", $
;      XTITLE = "YEAR", $
;      YTITLE = "Average CME Energy [ergs]"

;OPLOT, ycme, ayte, LINESTYLE = 0			; solid line
;OPLOT, ycme, ayke, PSYM = 2				; asterisk
;OPLOT, ycme, ayke, LINESTYLE = 1			; dotted line
;OPLOT, ycme, aype, PSYM = 5				; triangle
;OPLOT, ycme, aype, LINESTYLE = 2			; dashed line
;xyouts, 1988.25, ayke (nyear-1), 'KINETIC',    CHARSIZE = 0.5
;xyouts, 1988.25, aype (nyear-1), 'POTENTIAL',  CHARSIZE = 0.5
;xyouts, 1988.25, ayte (nyear-1), 'MECHANICAL', CHARSIZE = 0.5

; Log (Energy) versus time.

;PLOT, ycme,  layte, $
;      PSYM = 6, $					; square
;      XTHICK = 5.0, YTHICK = 5.0, $
;      XTICKLEN = -0.02, YTICKLEN = -0.02, $
;      XRANGE = [1980, 1992], $
;      YRANGE = [29.0, 31.5], $
;      TITLE = "Log (Energy) vs. Time", $
;      XTITLE = "YEAR", $
;      YTITLE = "Log (Ave. CME Energy) [ergs]"

;OPLOT, ycme, layte, LINESTYLE = 0			; solid line
;OPLOT, ycme, layke, PSYM = 2				; asterisk
;OPLOT, ycme, layke, LINESTYLE = 1			; dotted line
;OPLOT, ycme, laype, PSYM = 5				; triangle
;OPLOT, ycme, laype, LINESTYLE = 2			; dashed line
;xyouts, 1988.25, layke (nyear-1), 'KINETIC',    CHARSIZE = 0.5
;xyouts, 1988.25, laype (nyear-1), 'POTENTIAL',  CHARSIZE = 0.5
;xyouts, 1988.25, layte (nyear-1), 'MECHANICAL', CHARSIZE = 0.5

!P.MULTI = [0, 0, 0, 0, 0]		; one plot per page.

DEVICE, /CLOSE
SET_PLOT, 'X'

end
