pro em

;+
; Scatter Plot: energy versus mass
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   2 April 1996 
;-

@cpmassa.common

rdcpmass2

ke = ke * 1.0e+7				; kinetic energy [ergs]
pe = pe * 1.0e+7				; potential energy [ergs]
me = ke + pe					; mechanical energy [ergs]
xmass = xmass * 1.0e+3				; mass [grams]

ske = size (ke)
spe = size (pe)
me = ke + pe
ske = size (ke)
sxmass = size (xmass)

nval= ske (1)
nke = ske (1)
nxmass = sxmass  (1)
;print, 'nxmass: ', nxmass, '   nxmass: ', nxmass

;for i = 0, nke - 1  do $
;   print, 'i: ', i, ' ke: ', ke (i), ' pe: ', pe (i), ' me: ', me (i), $
;	  ' xmass: ', xmass (i)

;--- Assemble only those non-zero energies for which xmass > 0.0

ip = 0
pxmass = fltarr (nval)
pke    = fltarr (nval)
ppe    = fltarr (nval)
pme    = fltarr (nval)
for i = 0, nke - 1  do $
begin
   if (me (i) gt 0.0 and xmass (i) gt 0.0) then	$
   begin
      pxmass (ip) = xmass (i)
      pke (ip) = ke (i)
      ppe (ip) = pe (i)
      pme (ip) = me (i)
      ip = ip + 1
   end				
end

nval = ip

;--- Resize arrays.

pxmass = pxmass (0 : nval - 1)
pke    = pke    (0 : nval - 1)
ppe    = ppe    (0 : nval - 1)
pme    = pme    (0 : nval - 1)

;for i = 0, nval - 1  do $
;   print, 'i: ', i, ' pke: ', pme (i), ' ppe: ', ppe (i), ' pme: ', pme (i), $
;	  ' pxmass: ', pxmass (i)

lpke = alog10 (pke)				; log (energy) [ergs].
lppe = alog10 (ppe)				; log (energy) [ergs].
lpme = alog10 (pme)				; log (energy) [ergs].
lpxmass = alog10 (pxmass)

SET_PLOT, 'PS'
!P.FONT = 0
DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'em.ps', $
	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

A = FINDGEN(16) * (!PI*2/16.)
;USERSYM, COS(A), SIN(A), /FILL
USERSYM, COS(A), SIN(A)

tke1 = 0
tke2 = 0
tke3 = 0
tke4 = 0
tke5 = 0

tpe1 = 0
tpe2 = 0
tpe3 = 0
tpe4 = 0
tpe5 = 0

tme1 = 0
tme2 = 0
tme3 = 0
tme4 = 0
tme5 = 0

nke1 = 0
nke2 = 0
nke3 = 0
nke4 = 0
nke5 = 0

ke1 = fltarr (nval)
ke2 = fltarr (nval)
ke3 = fltarr (nval)
ke4 = fltarr (nval)
ke5 = fltarr (nval)

pe1 = fltarr (nval)
pe2 = fltarr (nval)
pe3 = fltarr (nval)
pe4 = fltarr (nval)
pe5 = fltarr (nval)

me1 = fltarr (nval)
me2 = fltarr (nval)
me3 = fltarr (nval)
me4 = fltarr (nval)
me5 = fltarr (nval)

i1 = 0
i2 = 0
i3 = 0
i4 = 0
i5 = 0

m1 = 5.0e+14
m2 = 1.0e+15
m3 = 5.0e+15
m4 = 1.0e+16

for i = 0, nval - 1 do $
begin
   if (pxmass (i) le m1) then $
   begin
      nke1 = nke1 + 1
      tke1 = tke1 + pke (i) 
      ke1 (i1) = pke (i)
      tpe1 = tpe1 + ppe (i) 
      pe1 (i1) = ppe (i)
      tme1 = tme1 + pme (i) 
      me1 (i1) = pme (i)
      i1 = i1 + 1
   endif

   if (pxmass (i) gt m1 and pxmass (i) le m2) then $
   begin
      nke2 = nke2 + 1
      tke2 = tke2 + pke (i)
      ke2 (i2) = pke (i)
      tpe2 = tpe2 + ppe (i)
      pe2 (i2) = ppe (i)
      tme2 = tme2 + pme (i)
      me2 (i2) = pme (i)
      i2 = i2 + 1
   endif

   if (pxmass (i) gt m2 and pxmass (i) le m3) then $
   begin
      nke3 = nke3 + 1
      tke3 = tke3 + pke (i)
      ke3 (i3) = pke (i)
      tpe3 = tpe3 + ppe (i)
      pe3 (i3) = ppe (i)
      tme3 = tme3 + pme (i)
      me3 (i3) = pme (i)
      i3 = i3 + 1
   endif

   if (pxmass (i) gt m3 and pxmass (i) le m4) then $
   begin
      nke4 = nke4 + 1
      tke4 = tke4 + pke (i)
      ke4 (i4) = pke (i)
      tpe4 = tpe4 + ppe (i)
      pe4 (i4) = ppe (i)
      tme4 = tme4 + pme (i)
      me4 (i4) = pme (i)
      i4 = i4 + 1
   endif

   if (pxmass (i) gt m4) then $
   begin
      nke5 = nke5 + 1
      tke5 = tke5 + pke (i)
      ke5 (i5) = pke (i)
      tpe5 = tpe5 + ppe (i)
      pe5 (i5) = ppe (i)
      tme5 = tme5 + pme (i)
      me5 (i5) = pme (i)
      i5 = i5 + 1
   endif
endfor

ke1 = ke1 (0: nke1 - 1)
ke2 = ke2 (0: nke2 - 1)
ke3 = ke3 (0: nke3 - 1)
ke4 = ke4 (0: nke4 - 1)
ke5 = ke5 (0: nke5 - 1)

pe1 = pe1 (0: nke1 - 1)
pe2 = pe2 (0: nke2 - 1)
pe3 = pe3 (0: nke3 - 1)
pe4 = pe4 (0: nke4 - 1)
pe5 = pe5 (0: nke5 - 1)

me1 = me1 (0: nke1 - 1)
me2 = me2 (0: nke2 - 1)
me3 = me3 (0: nke3 - 1)
me4 = me4 (0: nke4 - 1)
me5 = me5 (0: nke5 - 1)

num = nke1 + nke2 + nke3 + nke4 + nke5

avgke1 = tke1 / nke1
avgke2 = tke2 / nke2 
avgke3 = tke3 / nke3
avgke4 = tke4 / nke4
avgke5 = tke5 / nke5
avgke = (tke1 + tke2 + tke3 + tke4 + tke5) / num

avgpe1 = tpe1 / nke1
avgpe2 = tpe2 / nke2 
avgpe3 = tpe3 / nke3
avgpe4 = tpe4 / nke4
avgpe5 = tpe5 / nke5
avgpe = (tpe1 + tpe2 + tpe3 + tpe4 + tpe5) / num

avgme1 = tme1 / nke1
avgme2 = tme2 / nke2 
avgme3 = tme3 / nke3
avgme4 = tme4 / nke4
avgme5 = tme5 / nke5
avgme = (tme1 + tme2 + tme3 + tme4 + tme5) / num

savgke1 = string (avgke1, FORMAT = '(E8.2)')
savgke2 = string (avgke2, FORMAT = '(E8.2)')
savgke3 = string (avgke3, FORMAT = '(E8.2)')
savgke4 = string (avgke4, FORMAT = '(E8.2)')
savgke5 = string (avgke5, FORMAT = '(E8.2)')

savgpe1 = string (avgpe1, FORMAT = '(E8.2)')
savgpe2 = string (avgpe2, FORMAT = '(E8.2)')
savgpe3 = string (avgpe3, FORMAT = '(E8.2)')
savgpe4 = string (avgpe4, FORMAT = '(E8.2)')
savgpe5 = string (avgpe5, FORMAT = '(E8.2)')

savgme1 = string (avgme1, FORMAT = '(E8.2)')
savgme2 = string (avgme2, FORMAT = '(E8.2)')
savgme3 = string (avgme3, FORMAT = '(E8.2)')
savgme4 = string (avgme4, FORMAT = '(E8.2)')
savgme5 = string (avgme5, FORMAT = '(E8.2)')

print, '<K.E.> (     0 < xmass < 5.e+14) = ', avgke1
print, '<K.E.> (5.e+14 < xmass < 1.e+15) = ', avgke2
print, '<K.E.> (1.e+15 < xmass < 5.e+15) = ', avgke3
print, '<K.E.> (5.e+15   xmass < 1.e+16) = ', avgke4
print, '<K.E.> (         xmass > 1.e+16) = ', avgke5

print, '<P.E.> (     0 < xmass < 5.e+14) = ', avgpe1
print, '<P.E.> (5.e+14 < xmass < 1.e+15) = ', avgpe2
print, '<P.E.> (1.e+15 < xmass < 5.e+15) = ', avgpe3
print, '<P.E.> (5.e+15   xmass < 1.e+16) = ', avgpe4
print, '<P.E.> (         xmass > 1.e+16) = ', avgpe5

print, '<M.E.> (     0 < xmass < 5.e+14) = ', avgme1
print, '<M.E.> (5.e+14 < xmass < 1.e+15) = ', avgme2
print, '<M.E.> (1.e+15 < xmass < 5.e+15) = ', avgme3
print, '<M.E.> (5.e+15   xmass < 1.e+16) = ', avgme4
print, '<M.E.> (         xmass > 1.e+16) = ', avgme5

print, 'num: ', num, '   K.E. ave: ', avgke, ' P.E. ave: ', avgpe, $
		       ' M.E. ave: ', avgme
snum = string (num, FORMAT = '(I4)')
savgke = string (avgke, FORMAT = '(E8.2)')
savgpe = string (avgpe, FORMAT = '(E8.2)')
savgme = string (avgme, FORMAT = '(E8.2)')

print, 'size (pxmass) :', size (pxmass)
print, 'size (pke   ) :', size (pke   )
print, 'size (ppe   ) :', size (ppe   )
print, 'size (pme   ) :', size (pme   )
rke = CORRELATE (lpxmass, lpke)
rpe = CORRELATE (lpxmass, lppe)
rme = CORRELATE (lpxmass, lpme)

srke = string (rke, FORMAT = '(F6.3)')
srpe = string (rpe, FORMAT = '(F6.3)')
srme = string (rme, FORMAT = '(F6.3)')

print, 'correlation: ke vs. xmass: ', rke
print, 'correlation: pe vs. xmass: ', rpe
print, 'correlation: me vs. xmass: ', rme

;plot, xmass, lpke

ymin = 27.0 
ymax = 34.0

PLOT, lpxmass, lpke, /YNOZ, PSYM = 8,		$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Kinetic Energy vs Mass',		$
     XTITLE = 'Log (Mass)   [grams]',		$
     YTITLE = 'Log (Energy)   [ergs]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.86, '1980, 1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.82, 'corr. coeff = ' + srke, /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.78, snum + ' CMEs measured.', /NORM, CHARSIZE = 0.6
xyouts, 0.21, 0.74, '<K.E.>=' + savgke + ' ergs', /NORM, CHARSIZE = 0.6
xyouts, 0.50, 0.88, '<K.E.> (     0 < xmass < 5.e+14) = ' + savgke1, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.84, '<K.E.> (5.e+14 < xmass < 1.e+15) = ' + savgke2, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.80, '<K.E.> (1.e+15 < xmass < 5.e+15) = ' + savgke3, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.76, '<K.E.> (5.e+15   xmass < 1.e+16) = ' + savgke4, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.72, '<K.E.> (         xmass > 1.e+16) = ' + savgke5, /NORM, $
	CHARSIZE=0.6

ymin = 29.0 
ymax = 33.0

PLOT, lpxmass, lppe, /YNOZ, PSYM = 8,		$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Potential Energy vs Mass',	$
     XTITLE = 'Log (Mass)   [grams]',			$
     YTITLE = 'Log (Energy)   [ergs]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.86, '1980, 1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.82, 'corr. coeff = ' + srpe, /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.78, snum + ' CMEs measured.', /NORM, CHARSIZE = 0.6
xyouts, 0.21, 0.74, '<P.E.>=' + savgpe + ' ergs', /NORM, CHARSIZE = 0.6
xyouts, 0.50, 0.88, '<P.E.> (     0 < xmass < 5.e+14) = ' + savgpe1, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.84, '<P.E.> (5.e+14 < xmass < 1.e+15) = ' + savgpe2, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.80, '<P.E.> (1.e+15 < xmass < 5.e+15) = ' + savgpe3, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.76, '<P.E.> (5.e+15   xmass < 1.e+16) = ' + savgpe4, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.72, '<P.E.> (         xmass > 1.e+16) = ' + savgpe5, /NORM, $
	CHARSIZE=0.6

ymin = 29.0 
ymax = 33.0

PLOT, lpxmass, lpme, /YNOZ, PSYM = 8,		$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Mechanical Energy vs Mass',	$
     XTITLE = 'Log (Mass)   [grams]',			$
     YTITLE = 'Log (Energy)   [ergs]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.86, '1980, 1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.82, 'corr. coeff = ' + srme, /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.78, snum + ' CMEs measured.', /NORM, CHARSIZE = 0.6
xyouts, 0.21, 0.74, '<M.E.>=' + savgme + ' ergs', /NORM, CHARSIZE = 0.6
xyouts, 0.50, 0.88, '<M.E.> (     0 < xmass < 5.e+14) = ' + savgme1, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.84, '<M.E.> (5.e+14 < xmass < 1.e+15) = ' + savgme2, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.80, '<M.E.> (1.e+15 < xmass < 5.e+15) = ' + savgme3, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.76, '<M.E.> (5.e+15   xmass < 1.e+16= ' + savgme4, /NORM, $
	CHARSIZE=0.6
xyouts, 0.50, 0.72, '<M.E.> (         xmass > 1.e+16) = ' + savgme5, /NORM, $
	CHARSIZE=0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

syearlab = '1980, 1984-1989'			; year label.
xtype = 1					; log   : mass.
ytype = 0					; linear: # in bin.

; --- Hisograms for Kinetic, Potential & Total Energy; All measurements.

elab = 'Energy'
eunits = 'ergs'
pkelab = 'Histogram Kinetic Energy'
ppelab = 'Histogram Potential Energy'
pmelab = 'Histogram Mechanical Energy'
syearlab = '1980, 1984-1989'

;cmehp, pke, elab, eunits, pkelab, syearlab, xtype, ytype
;cmehp, ppe, elab, eunits, ppelab, syearlab, xtype, ytype
;cmehp, pme, elab, eunits, pmelab, syearlab, xtype, ytype

;--- Kinetic Energy Histograms by mass range:

;cmehp, ke1, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '  0 < xmass < 250 km/s', /NORMAL
;cmehp, ke2, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '250 < xmass < 500 km/s', /NORMAL
;cmehp, ke3, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '500 < xmass < 750 km/s', /NORMAL
;cmehp, ke4, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '750 < xmass < 1000 km/s', /NORMAL
;cmehp, ke5, elab, eunits, pkelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '      xmass > 1000 km/s', /NORMAL

;--- Potential Energy Histograms by mass range:

;cmehp, pe1, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '  0 < xmass < 250 km/s', /NORMAL
;cmehp, pe2, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '250 < xmass < 500 km/s', /NORMAL
;cmehp, pe3, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '500 < xmass < 90 km/s', /NORMAL
;cmehp, pe4, elab, eunits, ppelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '750 < xmass < 1000 km/s', /NORMAL

;--- Mechanical Energy Histograms by mass range:

;cmehp, me1, elab, eunits, pmelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '  0 < xmass < 2500 km/s', /NORMAL
;cmehp, me2, elab, eunits, pmelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '250 < xmass < 500 km/s', /NORMAL
;cmehp, me3, elab, eunits, pmelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '500 < xmass < 750 km/s', /NORMAL
;cmehp, me4, elab, eunits, pmelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '750 < xmass < 1000 km/s', /NORMAL
;cmehp, me5, elab, eunits, pmelab, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '      xmass > 1000 km/s', /NORMAL

DEVICE, /CLOSE
SET_PLOT, 'X'

end
