pro fke2

;+
; Scatter Plot: GOES flare versus kinetic energy
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.0e+3				; mass [grams]
ke    = ke * 1.0e+7				; energy [ergs]
sxmass = size (xmass)
sgx    = size (gx)

nval = sxmass (1)
ncme = sxmass (1)
ngx  = sgx    (1)
;print, 'ncme: ', ncme, '   ngx: ', ngx

for i = 0, ncme - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pke    = fltarr (nval)
tlgx   = 0.0
tke    = 0.0
tlke   = 0.0

ieq = 0
ile = 0
ilt = 0

neq = 0
nle = 0
nlt = 0

eqgx = fltarr (nval)
legx = fltarr (nval)
ltgx = fltarr (nval)

eqke  = fltarr (nval)
leke  = fltarr (nval)
ltke  = fltarr (nval)

for i = 0, ncme - 1  do $
begin
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pke   (ip)  = ke (i)
      tlgx = tlgx + alog10 (gx (i))
      tke = tke + ke (i)
      tlke = tlke + alog10 (ke (i))
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqke  (ieq)  = ke (i)
         ieq = ieq + 1
      endif			$
      else			$
      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         leke  (ile)  = ke (i)
         ile = ile + 1
      endif			$
      else			$
      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltke  (ilt)  = ke (i)
         ilt = ilt + 1
      endif			$
      else			$
      begin
	 print, 'year/doy: ', year (i), ' ', doy (i), ' ', gb (i)
      end
   end				
end

npgx = ip
neq  = ieq
nle  = ile
nlt  = ilt
spnum = string (npgx, FORMAT = '(I4)')

lgxave = tlgx / npgx
slgxave = string (lgxave, FORMAT = '(F7.2)')
print, 'average (log (goes magnitude)) : ', lgxave
keave = tke / npgx
lkeave = tlke / npgx
skeave = string (keave, FORMAT = '(E8.2)')
slkeave = string (lkeave, FORMAT = '(F7.2)')
print, 'average (ke)) : ', keave, '   average (log (ke)): ', lkeave

neqlelt = neq + nle + nlt
print, 'npgx: ', npgx, '   neqlelt: ', neqlelt
print, ' neq: ', neq, '   nle: ', nle, '   nlt: ', nlt

eqgx = eqgx (0 : neq - 1)
legx = legx (0 : nle - 1)
ltgx = ltgx (0 : nlt - 1)

eqke = eqke (0 : neq - 1)
leke = leke (0 : nle - 1)
ltke = ltke (0 : nlt - 1)

leqgx = alog10 (eqgx)
llegx = alog10 (legx)
lltgx = alog10 (ltgx)

leqke = alog10 (eqke)
lleke = alog10 (leke)
lltke = alog10 (ltke)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pke   = pke   (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pke: ', pke (i), '   pgx: ', pgx (i)

lpke    = alog10 (pke)			; log (energy) [ergs].
lpgx    = alog10 (pgx)			; log (GOES mag) [w/m^2].

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fke2.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fke2.ps', $
;	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin,   7.5" Wide
;	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" High

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fke2.ps', $
	XOFFSET = 11.43, XSIZE =  7.62, $	; 4.5" left margin,   3.0" Wide
	YOFFSET = 17.78, YSIZE =  7.62	 	; 7.0" bottom margin, 3.0" High
!P.MULTI = [0, 0, 0, 0, 0]

tke1 = 0.0
tke2 = 0.0
tke3 = 0.0
tke4 = 0.0
tke5 = 0.0

nke1 = 0
nke2 = 0
nke3 = 0
nke4 = 0
nke5 = 0

ke1 = fltarr (npgx)
ke2 = fltarr (npgx)
ke3 = fltarr (npgx)
ke4 = fltarr (npgx)
ke5 = fltarr (npgx)

im1 = 0
im2 = 0
im3 = 0
im4 = 0
im5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      ke1 (im1) = pke (i)
      tke1 = tke1 + pke (i)
      im1 = im1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      ke2 (im2) = pke (i)
      tke2 = tke2 + pke (i)
      im2 = im2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      ke3 (im3) = pke (i)
      tke3 = tke3 + pke (i)
      im3 = im3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      ke4 (im4) = pke (i)
      tke4 = tke4 + pke (i)
      im4 = im4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      ke5 (im5) = pke (i)
      tke5 = tke5 + pke (i)
      im5 = im5 + 1
   endif

endfor

;-------------------------------------------------------------------------------
nke1 = im1
nke2 = im2
nke3 = im3
nke4 = im4
nke5 = im5

ke1 = ke1 (0: nke1 - 1)
ke2 = ke2 (0: nke2 - 1)
ke3 = ke3 (0: nke3 - 1)
ke4 = ke4 (0: nke4 - 1)
ke5 = ke5 (0: nke5 - 1)

gnum = nke1 + nke2 + nke3 + nke4 + nke5

avg1 = tke1 / nke1
avg2 = tke2 / nke2 
avg3 = tke3 / nke3
avg4 = tke4 / nke4
avg5 = tke5 / nke5
keavg = (tke1 + tke2 + tke3 + tke4 + tke5) / gnum

savg1 = string (avg1, FORMAT = '(E8.2)')
savg2 = string (avg2, FORMAT = '(E8.2)')
savg3 = string (avg3, FORMAT = '(E8.2)')
savg4 = string (avg4, FORMAT = '(E8.2)')
savg5 = string (avg5, FORMAT = '(E8.2)')

print, '<Energy> (A < flare < B) = ', avg1
print, '<Energy> (B < flare < C) = ', avg2
print, '<Energy> (C < flare < M) = ', avg3
print, '<Energy> (M < flare < X) = ', avg4
print, '<Energy> (X < flare    ) = ', avg5

print, 'gnum: ', gnum, '   ke ave: ', keavg
sgnum = string (gnum, FORMAT = '(I4)')
skeavg = string (keavg, FORMAT = '(E8.2)')

rke = CORRELATE (lpgx, lpke)
srke = string (rke, FORMAT = '(F6.3)')
print, 'correlation: flare magnitude vs. kinetic energy: ', rke

;-------------------------------------------------------------------------------
;plot, pgx, lpke

xmin = -8.0 
xmax = -3.0
ymin = 28.0 
ymax = 34.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

;PLOT, lpgx, lpke,   /YNOZ, PSYM = 4,		$
;     XRANGE = [xmin, xmax],			$
;     YRANGE = [ymin, ymax],			$
;     TITLE = 'Flare Intensity vs. CME Kinetic Energy',	$
;     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
;     YTITLE = 'Log (CME Kinetic Energy)   [ergs]',		$
;     XTHICK = 5.0, YTHICK = 5.0,		$
;     XTICKLEN = -0.02, YTICKLEN = -0.02

;xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.90, 'SMM   1984-1989', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.87, spnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
;xyouts, 0.21, 0.84, 'correlation coefficient = ' + srke, /NORM, CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<log (m.e.)>= ' + slkeave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.86, '<m.e.> (A < flare < B ) = ' + savg1, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.84, '<m.e.> (B < flare < C ) = ' + savg2, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.82, '<m.e.> (C < flare < M ) = ' + savg3, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.80, '<m.e.> (M < flare < X) = ' + savg4, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.78, '<m.e.> (X < flare    ) = ' + savg5, /NORM, $
;	CHARSIZE = 0.6

;-------------------------------------------------------------------------------
USERSYM, COS(A), SIN(A), /FILL

;eqlegx    = fltarr (neq + nle)
;eqleltgx  = fltarr (neq + nle + nlt)
;eqleke   = fltarr (neq + nle)
;eqleltke = fltarr (neq + nle + nlt)

;eqlegx   (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqlegx   (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqleltgx (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (neq + nle : neq + nle + nlt - 1) = ltgx (0 : nlt - 1)

;eqleke   (0         : neq - 1)             = eqke (0 : neq - 1)
;eqleke   (neq       : neq + nle - 1)       = leke (0 : nle - 1)
;eqleltke (0         : neq - 1)             = eqke (0 : neq - 1)
;eqleltke (neq       : neq + nle - 1)       = leke (0 : nle - 1)
;eqleltke (neq + nle : neq + nle + nlt - 1) = ltke (0 : nlt - 1)

eqlegx    = [eqgx,legx]
eqleltgx  = [eqgx,legx,ltgx]
eqleke    = [eqke,leke]
eqleltke  = [eqke,leke,ltke]

leqlegx   = alog10 (eqlegx)
leqleltgx = alog10 (eqleltgx)
leqleke   = alog10 (eqleke)
leqleltke = alog10 (eqleltke)

;print, 'size (eqgx):     ', size (eqgx)
;print, 'size (eqlegx):   ', size (eqlegx)
;print, 'size (eqleltgx): ', size (eqleltgx)

req     = CORRELATE (leqgx, leqke)
reqle   = CORRELATE (leqlegx, leqleke)
reqlelt = CORRELATE (leqleltgx, leqleltke)
rle     = CORRELATE (llegx, lleke)
rlt     = CORRELATE (lltgx, lltke)

sreq     = string (req, FORMAT = '(F6.3)')
sreqle   = string (reqle, FORMAT = '(F6.3)')
sreqlelt = string (reqlelt, FORMAT = '(F6.3)')
srle     = string (rle, FORMAT = '(F6.3)')
srlt     = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ]       vs. kinetic energy: ', req
print, 'correlation: flare magnitude [EQ,LE]    vs. kinetic energy: ', reqle
print, 'correlation: flare magnitude [EQ,LE,lT] vs. kinetic energy: ', reqlelt
print, 'correlation: flare magnitude [LE]       vs. kinetic energy: ', rle
print, 'correlation: flare magnitude [LT]       vs. kinetic energy: ', rlt

ymin = 28.0 
ymax = 33.0

PLOT, leqgx, leqke, /YNOZ, PSYM = 8,		$
     CHARSIZE = 0.5,				$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Kinetic Energy',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'Log (CME Kinetic Energy)   [ergs]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, lleke,  PSYM = 5
OPLOT, lltgx, lltke,  PSYM = 1

;PLOTS, [0.21, 0.765], /NORM, PSYM = 8 
;PLOTS, [0.41, 0.765], /NORM, PSYM = 5 
;PLOTS, [0.61, 0.765], /NORM, PSYM = 1 
;xyouts, 0.23, 0.76, 'flare EQ value', /NORM, CHARSIZE = 0.6
;xyouts, 0.43, 0.76, 'flare LE value', /NORM, CHARSIZE = 0.6
;xyouts, 0.63, 0.76, 'flare LT value', /NORM, CHARSIZE = 0.6

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
;xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0

;xyouts, 0.21, 0.90, 'SMM   1984-1989', /NORM, CHARSIZE = 1.0
;xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8

xyouts, 0.24, 0.88, 'SMM   1984-1989', /NORM, CHARSIZE = 0.5
xyouts, 0.24, 0.84, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.4

;xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.81, 'correlation coefficient [EQ,LE]= ' + sreqle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.78, 'correlation coefficient [EQ,LE,LT]= ' + sreqlelt, /NORM, $
	CHARSIZE = 0.8

;;xyouts, 0.21, 0.75, 'correlation coefficient [LE]= ' + srle, /NORM, $
;;	CHARSIZE = 0.8
;;xyouts, 0.21, 0.72, 'correlation coefficient [LT]= ' + srlt, /NORM, $
;;	CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<log (m.e.)>= ' + slkeave, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

DEVICE, /CLOSE
SET_PLOT, 'X'

end
