pro fl

;+
; Scatter Plot: GOES flare versus latitude
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.e+3				; mass [grams]
sxmass = size (xmass)
sgx    = size (gx)

nval   = sxmass (1)
nxmass = sxmass (1)
ngx    = sgx     (1)
;print, 'nxmass: ', nxmass, '   ngx: ', ngx

for i = 0, nxmass - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pxmass = fltarr (nval)
plat   = fltarr (nval)
tlgx    = 0.0
tlat    = 0.0

ieq = 0
ile = 0
ilt = 0

neq = 0
nle = 0
nlt = 0

eqgx = fltarr (nval)
legx = fltarr (nval)
ltgx = fltarr (nval)

eqmass = fltarr (nval)
lemass = fltarr (nval)
ltmass = fltarr (nval)

eqlat  = fltarr (nval)
lelat  = fltarr (nval)
ltlat  = fltarr (nval)

for i = 0, nxmass - 1  do $
begin
;   if (gx (i) gt 0.0 and lat (i) ne 0.0) then	$	; GOES mag > 0.0
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pxmass (ip) = xmass (i)
      plat   (ip) = abs (lat (i))
      tlgx = tlgx + alog10 (gx (i))
      tlat = tlat + abs (lat (i))
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqmass (ieq) = xmass (i)
         eqlat  (ieq) = abs (lat (i))
         ieq = ieq + 1
      endif			$
      else			$
      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         lemass (ile) = xmass (i)
         lelat  (ile) = abs (lat (i))
         ile = ile + 1
      endif			$
      else			$
      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltmass (ilt) = xmass (i)
         ltlat  (ilt) = abs (lat (i))
         ilt = ilt + 1
      endif			$
      else			$
      begin
	 print, 'year/doy: ', year (i), ' ', doy (i), ' ', gb (i)
      end
   end				
end

npgx = ip
neq  = ieq
nle  = ile
nlt  = ilt
spnum = string (npgx, FORMAT = '(I4)')

lgxave = tlgx / npgx
slgxave = string (lgxave, FORMAT = '(F7.2)')
print, 'average (log (goes magnitude)) : ', lgxave
latave = tlat / npgx
slatave = string (latave, FORMAT = '(F7.2)')
print, 'average (latitude)) : ', latave

neqlelt = neq + nle + nlt
print, 'npgx: ', npgx, '   neqlelt: ', neqlelt
print, ' neq: ', neq, '   nle: ', nle, '   nlt: ', nlt

eqgx = eqgx (0 : neq - 1)
legx = legx (0 : nle - 1)
ltgx = ltgx (0 : nlt - 1)

eqmass = eqmass (0 : neq - 1)
lemass = lemass (0 : nle - 1)
ltmass = ltmass (0 : nlt - 1)

eqlat = eqlat (0 : neq - 1)
lelat = lelat (0 : nle - 1)
ltlat = ltlat (0 : nlt - 1)

leqgx = alog10 (eqgx)
llegx = alog10 (legx)
lltgx = alog10 (ltgx)

;leqmass = alog10 (eqmass)
;llemass = alog10 (lemass)
;lltmass = alog10 (ltmass)

;leqlat = alog10 (eqlat)
;llelat = alog10 (lelat)
;lltlat = alog10 (ltlat)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pxmass = pxmass (0 : npgx - 1)
plat   = plat   (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pxmass: ', pxmass (i), '   pgx: ', pgx (i)

lpxmass = alog10 (pxmass)			; log (mass) [grams].
lpgx    = alog10 (pgx   )			; log (GOES mag) [w/m^2].
;lplat   = alog10 (plat)

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fl.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fl.ps', $
	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin, 7.5" wide
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" high
!P.MULTI = [0, 0, 0, 0, 0]

tmass1 = 0.0
tmass2 = 0.0
tmass3 = 0.0
tmass4 = 0.0
tmass5 = 0.0

tlat1 = 0.0
tlat2 = 0.0
tlat3 = 0.0
tlat4 = 0.0
tlat5 = 0.0

nmass1 = 0
nmass2 = 0
nmass3 = 0
nmass4 = 0
nmass5 = 0

nlat1 = 0
nlat2 = 0
nlat3 = 0
nlat4 = 0
nlat5 = 0

mass1 = fltarr (npgx)
mass2 = fltarr (npgx)
mass3 = fltarr (npgx)
mass4 = fltarr (npgx)
mass5 = fltarr (npgx)

lat1 = fltarr (npgx)
lat2 = fltarr (npgx)
lat3 = fltarr (npgx)
lat4 = fltarr (npgx)
lat5 = fltarr (npgx)

im1 = 0
im2 = 0
im3 = 0
im4 = 0
im5 = 0

il1 = 0
il2 = 0
il3 = 0
il4 = 0
il5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      nmass1 = nmass1 + 1
      tmass1 = tmass1 + pxmass (i) 
      mass1 (im1) = pxmass (i)
      im1 = im1 + 1
      lat1 (im1) = plat (i)
      tlat1 = tlat1 + plat (i)
      il1 = il1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      nmass2 = nmass2 + 1
      tmass2 = tmass2 + pxmass (i)
      mass2 (im2) = pxmass (i)
      im2 = im2 + 1
      lat2 (im2) = plat (i)
      tlat2 = tlat2 + plat (i)
      il2 = il2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      nmass3 = nmass3 + 1
      tmass3 = tmass3 + pxmass (i)
      mass3 (im3) = pxmass (i)
      im3 = im3 + 1
      lat3 (im3) = plat (i)
      tlat3 = tlat3 + plat (i)
      il3 = il3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      nmass4 = nmass4 + 1
      tmass4 = tmass4 + pxmass (i)
      mass4 (im4) = pxmass (i)
      im4 = im4 + 1
      lat4 (im4) = plat (i)
      tlat4 = tlat4 + plat (i)
      il4 = il4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      nmass5 = nmass5 + 1
      tmass5 = tmass5 + pxmass (i)
      mass5 (im5) = pxmass (i)
      im5 = im5 + 1
      lat5 (im5) = plat (i)
      tlat5 = tlat5 + plat (i)
      il5 = il5 + 1
   endif

endfor

;m1 = mass1 (0: nmass1 - 1)
;m2 = mass2 (0: nmass2 - 1)
;m3 = mass3 (0: nmass3 - 1)
;m4 = mass4 (0: nmass4 - 1)
;m5 = mass5 (0: nmass5 - 1)

;gnum = nmass1 + nmass2 + nmass3 + nmass4 + nmass5

;avg1 = tmass1 / nmass1
;avg2 = tmass2 / nmass2 
;avg3 = tmass3 / nmass3
;avg4 = tmass4 / nmass4
;avg5 = tmass5 / nmass5
;avg = (tmass1 + tmass2 + tmass3 + tmass4 + tmass5) / gnum

;savg1 = string (avg1, FORMAT = '(E8.2)')
;savg2 = string (avg2, FORMAT = '(E8.2)')
;savg3 = string (avg3, FORMAT = '(E8.2)')
;savg4 = string (avg4, FORMAT = '(E8.2)')
;savg5 = string (avg5, FORMAT = '(E8.2)')

;print, '<Mass> (A < flare < B) = ', avg1
;print, '<Mass> (B < flare < C) = ', avg2
;print, '<Mass> (C < flare < M) = ', avg3
;print, '<Mass> (M < flare < X) = ', avg4
;print, '<Mass> (X < flare    ) = ', avg5

;print, 'gnum: ', gnum, '   mass ave: ', avg
;sgnum = string (gnum, FORMAT = '(I4)')
;savg = string (avg, FORMAT = '(E8.2)')

;r = CORRELATE (lpgx, lpxmass)
;sr = string (r, FORMAT = '(F6.3)')
;print, 'correlation: flare magnitude vs. mass: ', r

;-------------------------------------------------------------------------------
nlat1 = il1
nlat2 = il2
nlat3 = il3
nlat4 = il4
nlat5 = il5

lat1 = lat1 (0: nlat1 - 1)
lat2 = lat2 (0: nlat2 - 1)
lat3 = lat3 (0: nlat3 - 1)
lat4 = lat4 (0: nlat4 - 1)
lat5 = lat5 (0: nlat5 - 1)

gnum = nlat1 + nlat2 + nlat3 + nlat4 + nlat5

avg1 = tlat1 / nlat1
avg2 = tlat2 / nlat2 
avg3 = tlat3 / nlat3
avg4 = tlat4 / nlat4
avg5 = tlat5 / nlat5
latavg = (tlat1 + tlat2 + tlat3 + tlat4 + tlat5) / gnum

savg1 = string (avg1, FORMAT = '(F6.2)')
savg2 = string (avg2, FORMAT = '(F6.2)')
savg3 = string (avg3, FORMAT = '(F6.2)')
savg4 = string (avg4, FORMAT = '(F6.2)')
savg5 = string (avg5, FORMAT = '(F6.2)')

print, '<Lat> (A < flare < B) = ', avg1
print, '<Lat> (B < flare < C) = ', avg2
print, '<Lat> (C < flare < M) = ', avg3
print, '<Lat> (M < flare < X) = ', avg4
print, '<Lat> (X < flare    ) = ', avg5

print, 'gnum: ', gnum, '   lat ave: ', latavg
sgnum = string (gnum, FORMAT = '(I4)')
slatavg = string (latavg, FORMAT = '(F6.2)')

rlat = CORRELATE (pgx, plat)
srlat = string (rlat, FORMAT = '(F6.3)')
print, 'correlation: flare magnitude vs. lat: ', rlat

;-------------------------------------------------------------------------------
;plot, pgx, lplat

xmin = -8.0 
xmax = -3.0
ymin =   0.0 
ymax = 120.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

PLOT, lpgx, plat,   /YNOZ, PSYM = 4,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Magnitude vs. Latitude',	$
     XTITLE = 'Log (Flare Magnitude) [w/m^2]',	$
     YTITLE = 'Latitude   [degrees]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, spnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, 'correlation coefficient = ' + srlat, /NORM, CHARSIZE = 0.8

xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.88, '<lat>= '        + slatave, /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.86, '<lat> (A < flare < B ) = ' + savg1, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.84, '<lat> (B < flare < C ) = ' + savg2, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.82, '<lat> (C < flare < M ) = ' + savg3, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.80, '<lat> (M < flare < X) = ' + savg4, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.78, '<lat> (X < flare    ) = ' + savg5, /NORM, $
	CHARSIZE = 0.6

;-------------------------------------------------------------------------------
USERSYM, COS(A), SIN(A), /FILL

;eqlegx    = fltarr (neq + nle)
;eqleltgx  = fltarr (neq + nle + nlt)
;eqlelat   = fltarr (neq + nle)
;eqleltlat = fltarr (neq + nle + nlt)

;eqlegx   (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqlegx   (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqleltgx (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (neq + nle : neq + nle + nlt - 1) = ltgx (0 : nlt - 1)

;eqlelat   (0         : neq - 1)             = eqlat (0 : neq - 1)
;eqlelat   (neq       : neq + nle - 1)       = lelat (0 : nle - 1)
;eqleltlat (0         : neq - 1)             = eqlat (0 : neq - 1)
;eqleltlat (neq       : neq + nle - 1)       = lelat (0 : nle - 1)
;eqleltlat (neq + nle : neq + nle + nlt - 1) = ltlat (0 : nlt - 1)

eqlegx    = [eqgx,legx]
eqleltgx  = [eqgx,legx,ltgx]
eqlelat   = [eqlat,lelat]
eqleltlat = [eqlat,lelat,ltlat]

;print, 'size (eqgx):     ', size (eqgx)
;print, 'size (eqlegx):   ', size (eqlegx)
;print, 'size (eqleltgx): ', size (eqleltgx)

req     = CORRELATE (eqgx, eqlat)
reqle   = CORRELATE (eqlegx, eqlelat)
reqlelt = CORRELATE (eqleltgx, eqleltlat)
rle     = CORRELATE (legx, lelat)
rlt     = CORRELATE (ltgx, ltlat)

sreq     = string (req, FORMAT = '(F6.3)')
sreqle   = string (reqle, FORMAT = '(F6.3)')
sreqlelt = string (reqlelt, FORMAT = '(F6.3)')
srle     = string (rle, FORMAT = '(F6.3)')
srlt     = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ]       vs. latitude: ', req
print, 'correlation: flare magnitude [EQ,LE]    vs. latitude: ', reqle
print, 'correlation: flare magnitude [EQ,LE,lT] vs. latitude: ', reqlelt
print, 'correlation: flare magnitude [LE]       vs. latitude: ', rle
print, 'correlation: flare magnitude [LT]       vs. latitude: ', rlt

PLOT, leqgx, eqlat, /YNOZ, PSYM = 8,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Magnitude vs. Latitude',	$
     XTITLE = 'Log (Flare Magnitude) [w/m^2]',	$
     YTITLE = 'Latitude   [degrees]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, lelat,  PSYM = 5
OPLOT, lltgx, ltlat,  PSYM = 1

PLOTS, [0.21, 0.765], /NORM, PSYM = 8           
PLOTS, [0.41, 0.765], /NORM, PSYM = 5           
PLOTS, [0.61, 0.765], /NORM, PSYM = 1           
xyouts, 0.23, 0.76, 'flare EQ value', /NORM, CHARSIZE = 0.6
xyouts, 0.43, 0.76, 'flare LE value', /NORM, CHARSIZE = 0.6
xyouts, 0.63, 0.76, 'flare LT value', /NORM, CHARSIZE = 0.6

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
	CHARSIZE = 0.8
xyouts, 0.21, 0.81, 'correlation coefficient [EQ,LE]= ' + sreqle, /NORM, $
	CHARSIZE = 0.8
xyouts, 0.21, 0.78, 'correlation coefficient [EQ,LE,LT]= ' + sreqlelt, /NORM, $
	CHARSIZE = 0.8

;xyouts, 0.21, 0.75, 'correlation coefficient [LE]= ' + srle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.72, 'correlation coefficient [LT]= ' + srlt, /NORM, $
;	CHARSIZE = 0.8

xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.88, '<lat>= ' + slatave, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

DEVICE, /CLOSE
SET_PLOT, 'X'

end
