pro fm1

;+
; Scatter Plot: GOES flare versus mass
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.e+3				; mass [grams]
sxmass = size (xmass)
sgx    = size (gx)

nval   = sxmass (1)
nxmass = sxmass (1)
ngx    = sgx     (1)
;print, 'nxmass: ', nxmass, '   ngx: ', ngx

for i = 0, nxmass - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pxm    = fltarr (nval)
tlgx   = 0.0
txm    = 0.0
tlxm   = 0.0

ieq = 0
ile = 0
ilt = 0

neq = 0
nle = 0
nlt = 0

eqgx = fltarr (nval)
legx = fltarr (nval)
ltgx = fltarr (nval)

eqxm  = fltarr (nval)
lexm  = fltarr (nval)
ltxm  = fltarr (nval)

for i = 0, nxmass - 1  do $
begin
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pxm   (ip)  = xmass (i)
      tlgx = tlgx + alog10 (gx (i))
      txm = txm + xmass (i)
      tlxm = tlxm + alog10 (xmass (i))
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqxm  (ieq)  = xmass (i)
         ieq = ieq + 1
      endif			$
      else			$
      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         lexm  (ile)  = xmass (i)
         ile = ile + 1
      endif			$
      else			$
      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltxm  (ilt)  = xmass (i)
         ilt = ilt + 1
      endif			$
      else			$
      begin
	 print, 'year/doy: ', year (i), ' ', doy (i), ' ', gb (i)
      end
   end				
end

npgx = ip
neq  = ieq
nle  = ile
nlt  = ilt
spnum = string (npgx, FORMAT = '(I4)')

lgxave = tlgx / npgx
slgxave = string (lgxave, FORMAT = '(F7.2)')
print, 'average (log (goes magnitude)) : ', lgxave
xmave = txm / npgx
lxmave = tlxm / npgx
sxmave = string (xmave, FORMAT = '(E8.2)')
slxmave = string (lxmave, FORMAT = '(F7.2)')
print, 'average (xm)) : ', xmave, '   average (log (xm)): ', lxmave

neqlelt = neq + nle + nlt
print, 'npgx: ', npgx, '   neqlelt: ', neqlelt
print, ' neq: ', neq, '   nle: ', nle, '   nlt: ', nlt

eqgx = eqgx (0 : neq - 1)
legx = legx (0 : nle - 1)
ltgx = ltgx (0 : nlt - 1)

eqxm = eqxm (0 : neq - 1)
lexm = lexm (0 : nle - 1)
ltxm = ltxm (0 : nlt - 1)

leqgx = alog10 (eqgx)
llegx = alog10 (legx)
lltgx = alog10 (ltgx)

leqxm = alog10 (eqxm)
llexm = alog10 (lexm)
lltxm = alog10 (ltxm)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pxm   = pxm   (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pxm: ', pxm (i), '   pgx: ', pgx (i)

lpxm    = alog10 (pxm)			; log (mass) [grams].
lpgx    = alog10 (pgx)			; log (GOES mag) [w/m^2].

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fm1.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fm1.ps', $
	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin, 7.5" xme
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" high
!P.MULTI = [0, 0, 0, 0, 0]

txm1 = 0.0
txm2 = 0.0
txm3 = 0.0
txm4 = 0.0
txm5 = 0.0

nxm1 = 0
nxm2 = 0
nxm3 = 0
nxm4 = 0
nxm5 = 0

xm1 = fltarr (npgx)
xm2 = fltarr (npgx)
xm3 = fltarr (npgx)
xm4 = fltarr (npgx)
xm5 = fltarr (npgx)

im1 = 0
im2 = 0
im3 = 0
im4 = 0
im5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      xm1 (im1) = pxm (i)
      txm1 = txm1 + pxm (i)
      im1 = im1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      xm2 (im2) = pxm (i)
      txm2 = txm2 + pxm (i)
      im2 = im2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      xm3 (im3) = pxm (i)
      txm3 = txm3 + pxm (i)
      im3 = im3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      xm4 (im4) = pxm (i)
      txm4 = txm4 + pxm (i)
      im4 = im4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      xm5 (im5) = pxm (i)
      txm5 = txm5 + pxm (i)
      im5 = im5 + 1
   endif

endfor

;-------------------------------------------------------------------------------
nxm1 = im1
nxm2 = im2
nxm3 = im3
nxm4 = im4
nxm5 = im5

xm1 = xm1 (0: nxm1 - 1)
xm2 = xm2 (0: nxm2 - 1)
xm3 = xm3 (0: nxm3 - 1)
xm4 = xm4 (0: nxm4 - 1)
xm5 = xm5 (0: nxm5 - 1)

gnum = nxm1 + nxm2 + nxm3 + nxm4 + nxm5

avg1 = txm1 / nxm1
avg2 = txm2 / nxm2 
avg3 = txm3 / nxm3
avg4 = txm4 / nxm4
avg5 = txm5 / nxm5
xmavg = (txm1 + txm2 + txm3 + txm4 + txm5) / gnum

savg1 = string (avg1, FORMAT = '(E8.2)')
savg2 = string (avg2, FORMAT = '(E8.2)')
savg3 = string (avg3, FORMAT = '(E8.2)')
savg4 = string (avg4, FORMAT = '(E8.2)')
savg5 = string (avg5, FORMAT = '(E8.2)')

print, '<Mass> (A < flare < B) = ', avg1
print, '<Mass> (B < flare < C) = ', avg2
print, '<Mass> (C < flare < M) = ', avg3
print, '<Mass> (M < flare < X) = ', avg4
print, '<Mass> (X < flare    ) = ', avg5

print, 'gnum: ', gnum, '   xm ave: ', xmavg
sgnum = string (gnum, FORMAT = '(I4)')
sxmavg = string (xmavg, FORMAT = '(F6.2)')

rxm = CORRELATE (lpgx, lpxm)
srxm = string (rxm, FORMAT = '(F6.3)')
print, 'correlation: flare magnitude vs. xm: ', rxm

;-------------------------------------------------------------------------------
;plot, pgx, lpxm

xmin = -8.0 
xmax = -3.0
ymin = 14.0 
ymax = 17.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

PLOT, lpgx, lpxm,   /YNOZ, PSYM = 4,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Excess Mass',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'Log (CME Mass)   [grams]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, spnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
;xyouts, 0.21, 0.84, 'correlation coefficient = ' + srxm, /NORM, CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<log (mass)>= ' + slxmave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.86, '<mass> (A < flare < B ) = ' + savg1, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.84, '<mass> (B < flare < C ) = ' + savg2, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.82, '<mass> (C < flare < M ) = ' + savg3, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.80, '<mass> (M < flare < X) = ' + savg4, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.78, '<mass> (X < flare    ) = ' + savg5, /NORM, $
;	CHARSIZE = 0.6

;-------------------------------------------------------------------------------
USERSYM, COS(A), SIN(A), /FILL

;eqlegx    = fltarr (neq + nle)
;eqleltgx  = fltarr (neq + nle + nlt)
;eqlexm   = fltarr (neq + nle)
;eqleltxm = fltarr (neq + nle + nlt)

;eqlegx   (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqlegx   (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqleltgx (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (neq + nle : neq + nle + nlt - 1) = ltgx (0 : nlt - 1)

;eqlexm   (0         : neq - 1)             = eqxm (0 : neq - 1)
;eqlexm   (neq       : neq + nle - 1)       = lexm (0 : nle - 1)
;eqleltxm (0         : neq - 1)             = eqxm (0 : neq - 1)
;eqleltxm (neq       : neq + nle - 1)       = lexm (0 : nle - 1)
;eqleltxm (neq + nle : neq + nle + nlt - 1) = ltxm (0 : nlt - 1)

eqlegx    = [eqgx,legx]
eqleltgx  = [eqgx,legx,ltgx]
eqlexm    = [eqxm,lexm]
eqleltxm  = [eqxm,lexm,ltxm]

leqlegx   = alog10 (eqlegx)
leqleltgx = alog10 (eqleltgx)
leqlexm   = alog10 (eqlexm)
leqleltxm = alog10 (eqleltxm)

;print, 'size (eqgx):     ', size (eqgx)
;print, 'size (eqlegx):   ', size (eqlegx)
;print, 'size (eqleltgx): ', size (eqleltgx)

req     = CORRELATE (leqgx, leqxm)
reqle   = CORRELATE (leqlegx, leqlexm)
reqlelt = CORRELATE (leqleltgx, leqleltxm)
rle     = CORRELATE (llegx, llexm)
rlt     = CORRELATE (lltgx, lltxm)

sreq     = string (req, FORMAT = '(F6.3)')
sreqle   = string (reqle, FORMAT = '(F6.3)')
sreqlelt = string (reqlelt, FORMAT = '(F6.3)')
srle     = string (rle, FORMAT = '(F6.3)')
srlt     = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ]       vs. mass: ', req
print, 'correlation: flare magnitude [EQ,LE]    vs. mass: ', reqle
print, 'correlation: flare magnitude [EQ,LE,lT] vs. mass: ', reqlelt
print, 'correlation: flare magnitude [LE]       vs. mass: ', rle
print, 'correlation: flare magnitude [LT]       vs. mass: ', rlt

PLOT, leqgx, leqxm, /YNOZ, PSYM = 8,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Excess Mass',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'Log (CME Mass)   [grams]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, llexm,  PSYM = 5
OPLOT, lltgx, lltxm,  PSYM = 1

PLOTS, [0.21, 0.845], /NORM, PSYM = 8           
PLOTS, [0.41, 0.845], /NORM, PSYM = 5           
PLOTS, [0.61, 0.845], /NORM, PSYM = 1           
xyouts, 0.23, 0.84, 'flare EQ value', /NORM, CHARSIZE = 0.6
xyouts, 0.43, 0.84, 'flare LE value', /NORM, CHARSIZE = 0.6
xyouts, 0.63, 0.84, 'flare LT value', /NORM, CHARSIZE = 0.6

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
;xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.81, 'correlation coefficient [EQ,LE]= ' + sreqle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.78, 'correlation coefficient [EQ,LE,LT]= ' + sreqlelt, /NORM, $
;	CHARSIZE = 0.8
;
;xyouts, 0.21, 0.75, 'correlation coefficient [LE]= ' + srle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.72, 'correlation coefficient [LT]= ' + srlt, /NORM, $
;	CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<log (mass)>= ' + slxmave, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

DEVICE, /CLOSE
SET_PLOT, 'X'

end
