pro fme

;+
; Scatter Plot: GOES flare versus mechanical energy
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.0e+3				; mass [grams]
me    = me * 1.0e+7				; energy [ergs]
sxmass = size (xmass)
sgx    = size (gx)

nval = sxmass (1)
ncme = sxmass (1)
ngx  = sgx    (1)
;print, 'ncme: ', ncme, '   ngx: ', ngx

for i = 0, ncme - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pme    = fltarr (nval)
tlgx   = 0.0
tme    = 0.0
tlme   = 0.0

ieq = 0
ile = 0
ilt = 0

neq = 0
nle = 0
nlt = 0

eqgx = fltarr (nval)
legx = fltarr (nval)
ltgx = fltarr (nval)

eqme  = fltarr (nval)
leme  = fltarr (nval)
ltme  = fltarr (nval)

for i = 0, ncme - 1  do $
begin
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pme   (ip)  = me (i)
      tlgx = tlgx + alog10 (gx (i))
      tme = tme + me (i)
      tlme = tlme + alog10 (me (i))
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqme  (ieq)  = me (i)
         ieq = ieq + 1
      endif			$
      else			$
      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         leme  (ile)  = me (i)
         ile = ile + 1
      endif			$
      else			$
      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltme  (ilt)  = me (i)
         ilt = ilt + 1
      endif			$
      else			$
      begin
	 print, 'year/doy: ', year (i), ' ', doy (i), ' ', gb (i)
      end
   end				
end

npgx = ip
neq  = ieq
nle  = ile
nlt  = ilt
spnum = string (npgx, FORMAT = '(I4)')

lgxave = tlgx / npgx
slgxave = string (lgxave, FORMAT = '(F7.2)')
print, 'average (log (goes magnitude)) : ', lgxave
meave = tme / npgx
lmeave = tlme / npgx
smeave = string (meave, FORMAT = '(E8.2)')
slmeave = string (lmeave, FORMAT = '(F7.2)')
print, 'average (me)) : ', meave, '   average (log (me)): ', lmeave

neqlelt = neq + nle + nlt
print, 'npgx: ', npgx, '   neqlelt: ', neqlelt
print, ' neq: ', neq, '   nle: ', nle, '   nlt: ', nlt

eqgx = eqgx (0 : neq - 1)
legx = legx (0 : nle - 1)
ltgx = ltgx (0 : nlt - 1)

eqme = eqme (0 : neq - 1)
leme = leme (0 : nle - 1)
ltme = ltme (0 : nlt - 1)

leqgx = alog10 (eqgx)
llegx = alog10 (legx)
lltgx = alog10 (ltgx)

leqme = alog10 (eqme)
lleme = alog10 (leme)
lltme = alog10 (ltme)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pme   = pme   (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pme: ', pme (i), '   pgx: ', pgx (i)

lpme    = alog10 (pme)			; log (energy) [ergs].
lpgx    = alog10 (pgx)			; log (GOES mag) [w/m^2].

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fme.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fme.ps', $
	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin, 7.5" mee
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" high
!P.MULTI = [0, 0, 0, 0, 0]

tme1 = 0.0
tme2 = 0.0
tme3 = 0.0
tme4 = 0.0
tme5 = 0.0

nme1 = 0
nme2 = 0
nme3 = 0
nme4 = 0
nme5 = 0

me1 = fltarr (npgx)
me2 = fltarr (npgx)
me3 = fltarr (npgx)
me4 = fltarr (npgx)
me5 = fltarr (npgx)

im1 = 0
im2 = 0
im3 = 0
im4 = 0
im5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      me1 (im1) = pme (i)
      tme1 = tme1 + pme (i)
      im1 = im1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      me2 (im2) = pme (i)
      tme2 = tme2 + pme (i)
      im2 = im2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      me3 (im3) = pme (i)
      tme3 = tme3 + pme (i)
      im3 = im3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      me4 (im4) = pme (i)
      tme4 = tme4 + pme (i)
      im4 = im4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      me5 (im5) = pme (i)
      tme5 = tme5 + pme (i)
      im5 = im5 + 1
   endif

endfor

;-------------------------------------------------------------------------------
nme1 = im1
nme2 = im2
nme3 = im3
nme4 = im4
nme5 = im5

me1 = me1 (0: nme1 - 1)
me2 = me2 (0: nme2 - 1)
me3 = me3 (0: nme3 - 1)
me4 = me4 (0: nme4 - 1)
me5 = me5 (0: nme5 - 1)

gnum = nme1 + nme2 + nme3 + nme4 + nme5

avg1 = tme1 / nme1
avg2 = tme2 / nme2 
avg3 = tme3 / nme3
avg4 = tme4 / nme4
avg5 = tme5 / nme5
meavg = (tme1 + tme2 + tme3 + tme4 + tme5) / gnum

savg1 = string (avg1, FORMAT = '(E8.2)')
savg2 = string (avg2, FORMAT = '(E8.2)')
savg3 = string (avg3, FORMAT = '(E8.2)')
savg4 = string (avg4, FORMAT = '(E8.2)')
savg5 = string (avg5, FORMAT = '(E8.2)')

print, '<Energy> (A < flare < B) = ', avg1
print, '<Energy> (B < flare < C) = ', avg2
print, '<Energy> (C < flare < M) = ', avg3
print, '<Energy> (M < flare < X) = ', avg4
print, '<Energy> (X < flare    ) = ', avg5

print, 'gnum: ', gnum, '   me ave: ', meavg
sgnum = string (gnum, FORMAT = '(I4)')
smeavg = string (meavg, FORMAT = '(E8.2)')

rme = CORRELATE (lpgx, lpme)
srme = string (rme, FORMAT = '(F6.3)')
print, 'correlation: flare magnitude vs. mech energy: ', rme

;-------------------------------------------------------------------------------
;plot, pgx, lpme

mein = -8.0 
meax = -3.0
ymin = 28.0 
ymax = 34.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

PLOT, lpgx, lpme,   /YNOZ, PSYM = 4,		$
     XRANGE = [mein, meax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Magnitude vs. Mechanical Energy',	$
     XTITLE = 'Log (Flare Magnitude) [w/m^2]',	$
     YTITLE = 'Log (Energy)   [ergs]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, spnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, 'correlation coefficient = ' + srme, /NORM, CHARSIZE = 0.8

xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.88, '<log (m.e.)>= ' + slmeave, /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.86, '<m.e.> (A < flare < B ) = ' + savg1, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.84, '<m.e.> (B < flare < C ) = ' + savg2, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.82, '<m.e.> (C < flare < M ) = ' + savg3, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.80, '<m.e.> (M < flare < X) = ' + savg4, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.78, '<m.e.> (X < flare    ) = ' + savg5, /NORM, $
	CHARSIZE = 0.6

;-------------------------------------------------------------------------------
USERSYM, COS(A), SIN(A), /FILL

;eqlegx    = fltarr (neq + nle)
;eqleltgx  = fltarr (neq + nle + nlt)
;eqleme   = fltarr (neq + nle)
;eqleltme = fltarr (neq + nle + nlt)

;eqlegx   (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqlegx   (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqleltgx (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (neq + nle : neq + nle + nlt - 1) = ltgx (0 : nlt - 1)

;eqleme   (0         : neq - 1)             = eqme (0 : neq - 1)
;eqleme   (neq       : neq + nle - 1)       = leme (0 : nle - 1)
;eqleltme (0         : neq - 1)             = eqme (0 : neq - 1)
;eqleltme (neq       : neq + nle - 1)       = leme (0 : nle - 1)
;eqleltme (neq + nle : neq + nle + nlt - 1) = ltme (0 : nlt - 1)

eqlegx    = [eqgx,legx]
eqleltgx  = [eqgx,legx,ltgx]
eqleme    = [eqme,leme]
eqleltme  = [eqme,leme,ltme]

leqlegx   = alog10 (eqlegx)
leqleltgx = alog10 (eqleltgx)
leqleme   = alog10 (eqleme)
leqleltme = alog10 (eqleltme)

;print, 'size (eqgx):     ', size (eqgx)
;print, 'size (eqlegx):   ', size (eqlegx)
;print, 'size (eqleltgx): ', size (eqleltgx)

req     = CORRELATE (leqgx, leqme)
reqle   = CORRELATE (leqlegx, leqleme)
reqlelt = CORRELATE (leqleltgx, leqleltme)
rle     = CORRELATE (llegx, lleme)
rlt     = CORRELATE (lltgx, lltme)

sreq     = string (req, FORMAT = '(F6.3)')
sreqle   = string (reqle, FORMAT = '(F6.3)')
sreqlelt = string (reqlelt, FORMAT = '(F6.3)')
srle     = string (rle, FORMAT = '(F6.3)')
srlt     = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ]       vs. mech energy: ', req
print, 'correlation: flare magnitude [EQ,LE]    vs. mech energy: ', reqle
print, 'correlation: flare magnitude [EQ,LE,lT] vs. mech energy: ', reqlelt
print, 'correlation: flare magnitude [LE]       vs. mech energy: ', rle
print, 'correlation: flare magnitude [LT]       vs. mech energy: ', rlt

PLOT, leqgx, leqme, /YNOZ, PSYM = 8,		$
     XRANGE = [mein, meax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Magnitude vs. Mechanical Energy',	$
     XTITLE = 'Log (Flare Magnitude) [w/m^2]',	$
     YTITLE = 'Log (Energy)   [ergs]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, lleme,  PSYM = 5
OPLOT, lltgx, lltme,  PSYM = 1

PLOTS, [0.21, 0.765], /NORM, PSYM = 8           
PLOTS, [0.41, 0.765], /NORM, PSYM = 5           
PLOTS, [0.61, 0.765], /NORM, PSYM = 1           
xyouts, 0.23, 0.76, 'flare EQ value', /NORM, CHARSIZE = 0.6
xyouts, 0.43, 0.76, 'flare LE value', /NORM, CHARSIZE = 0.6
xyouts, 0.63, 0.76, 'flare LT value', /NORM, CHARSIZE = 0.6

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
	CHARSIZE = 0.8
xyouts, 0.21, 0.81, 'correlation coefficient [EQ,LE]= ' + sreqle, /NORM, $
	CHARSIZE = 0.8
xyouts, 0.21, 0.78, 'correlation coefficient [EQ,LE,LT]= ' + sreqlelt, /NORM, $
	CHARSIZE = 0.8

;xyouts, 0.21, 0.75, 'correlation coefficient [LE]= ' + srle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.72, 'correlation coefficient [LT]= ' + srlt, /NORM, $
;	CHARSIZE = 0.8

xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.88, '<log (m.e.)>= ' + slmeave, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

DEVICE, /CLOSE
SET_PLOT, 'X'

end
