pro fq

;+
; Scatter Plot: GOES flare versus mass, latitude, speed, mech energy
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.e+3				; mass [grams]
sxmass = size (xmass)
sgx    = size (gx)

nval   = sxmass (1)
nxmass = sxmass (1)
ngx    = sgx     (1)
;print, 'nxmass: ', nxmass, '   ngx: ', ngx

for i = 0, nxmass - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pxmass = fltarr (nval)
plat   = fltarr (nval)
pwidth = fltarr (nval)
pspeed = fltarr (nval)
pme    = fltarr (nval)

ilt = 0
ile = 0
ieq = 0
nlt = 0
nle = 0
neq = 0
ltgx = fltarr (nval)
legx = fltarr (nval)
eqgx = fltarr (nval)
ltmass = fltarr (nval)
lemass = fltarr (nval)
eqmass = fltarr (nval)

for i = 0, nxmass - 1  do $
begin
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pxmass (ip) = xmass (i)
      pspeed (ip) = speed (i)
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqmass (ieq) = xmass (i)
         ieq = ieq + 1
      endif

      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         lemass (ile) = xmass (i)
         ile = ile + 1
      endif

      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltmass (ilt) = xmass (i)
         ilt = ilt + 1
      endif
   end				
end

npgx = ip
nlt  = ilt
nle  = ile
neq  = ieq
print, 'neq: ', neq, ' nle: ', nle, ' nlt: ', nlt
ltmass = ltmass (0 : nlt - 1)
lemass = lemass (0 : nle - 1)
eqmass = eqmass (0 : neq - 1)
ltgx = ltgx (0 : nlt - 1)
legx = legx (0 : nle - 1)
eqgx = eqgx (0 : neq - 1)
lltmass = alog10 (ltmass)
llemass = alog10 (lemass)
leqmass = alog10 (eqmass)
lltgx = alog10 (ltgx)
llegx = alog10 (legx)
leqgx = alog10 (eqgx)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pxmass = pxmass (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pxmass: ', pxmass (i), '   pgx: ', pgx (i)

lpxmass = alog10 (pxmass)			; log (mass) [grams].
lpgx    = alog10 (pgx   )			; log (GOES mag) [w/m^2].

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fm.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fm.ps', $
	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin, 7.5" wide
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" high
!P.MULTI = [0, 0, 0, 0, 0]

tmass1 = 0
tmass2 = 0
tmass3 = 0
tmass4 = 0
tmass5 = 0

nmass1 = 0
nmass2 = 0
nmass3 = 0
nmass4 = 0
nmass5 = 0

mass1 = fltarr (npgx)
mass2 = fltarr (npgx)
mass3 = fltarr (npgx)
mass4 = fltarr (npgx)
mass5 = fltarr (npgx)

i1 = 0
i2 = 0
i3 = 0
i4 = 0
i5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      nmass1 = nmass1 + 1
      tmass1 = tmass1 + pxmass (i) 
      mass1 (i1) = pxmass (i)
      i1 = i1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      nmass2 = nmass2 + 1
      tmass2 = tmass2 + pxmass (i)
      mass2 (i2) = pxmass (i)
      i2 = i2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      nmass3 = nmass3 + 1
      tmass3 = tmass3 + pxmass (i)
      mass3 (i3) = pxmass (i)
      i3 = i3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      nmass4 = nmass4 + 1
      tmass4 = tmass4 + pxmass (i)
      mass4 (i4) = pxmass (i)
      i4 = i4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      nmass5 = nmass5 + 1
      tmass5 = tmass5 + pxmass (i)
      mass5 (i5) = pxmass (i)
      i5 = i5 + 1
   endif

endfor


m1 = mass1 (0: nmass1 - 1)
m2 = mass2 (0: nmass2 - 1)
m3 = mass3 (0: nmass3 - 1)
m4 = mass4 (0: nmass4 - 1)
m5 = mass5 (0: nmass5 - 1)

gnum = nmass1 + nmass2 + nmass3 + nmass4 + nmass5

avg1 = tmass1 / nmass1
avg2 = tmass2 / nmass2 
avg3 = tmass3 / nmass3
avg4 = tmass4 / nmass4
avg5 = tmass5 / nmass5
avg = (tmass1 + tmass2 + tmass3 + tmass4 + tmass5) / gnum

savg1 = string (avg1, FORMAT = '(E8.2)')
savg2 = string (avg2, FORMAT = '(E8.2)')
savg3 = string (avg3, FORMAT = '(E8.2)')
savg4 = string (avg4, FORMAT = '(E8.2)')
savg5 = string (avg5, FORMAT = '(E8.2)')

print, '<Mass> (0 < flare < A) = ', avg1
print, '<Mass> (A < flare < B) = ', avg2
print, '<Mass> (B < flare < C) = ', avg3
print, '<Mass> (C < flare < M) = ', avg4
print, '<Mass> (M < flare < X) = ', avg5

print, 'gnum: ', gnum, '   mass ave: ', avg
sgnum = string (gnum, FORMAT = '(I4)')
savg = string (avg, FORMAT = '(E8.2)')

r = CORRELATE (lpgx, lpxmass)
sr = string (r, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude vs. mass: ', r

;plot, pgx, lpxmass

xmin = -8.0 
xmax = -3.0
ymin = 14
ymax = 17.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

PLOT, lpgx, lpxmass, /YNOZ, PSYM = 8,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Magnitude vs. Mass',	$
     XTITLE = 'Log (Flare Magnitude) [w/m^2]',	$
     YTITLE = 'Log (Mass)   [grams]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.87, sgnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, 'correlation coefficient = ' + sr, /NORM, CHARSIZE = 0.8

xyouts, 0.60, 0.90, '<mass>=' + savg + 'grams', /NORM, CHARSIZE = 0.6
xyouts, 0.60, 0.88, '<mass> (A < flare < B ) = ' + savg1, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.86, '<mass> (B < flare < C ) = ' + savg2, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.84, '<mass> (C < flare < M ) = ' + savg3, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.82, '<mass> (M < flare < X) = ' + savg4, /NORM, $
	CHARSIZE = 0.6
xyouts, 0.60, 0.80, '<mass> (    flare > X) = ' + savg5, /NORM, $
	CHARSIZE = 0.6

USERSYM, COS(A), SIN(A), /FILL

req = CORRELATE (leqgx, leqmass)
rle = CORRELATE (llegx, llemass)
rlt = CORRELATE (lltgx, lltmass)
sreq = string (req, FORMAT = '(F6.3)')
srle = string (rle, FORMAT = '(F6.3)')
srlt = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ] vs. mass: ', req
print, 'correlation: flare magnitude [LE] vs. mass: ', rle
print, 'correlation: flare magnitude [LT] vs. mass: ', rlt

PLOT, leqgx, leqmass, /YNOZ, PSYM = 8,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Magnitude vs. Mass',	$
     XTITLE = 'Log (Flare Magnitude) [w/m^2]',	$
     YTITLE = 'Log (Mass)   [grams]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, llemass,  PSYM = 5
OPLOT, lltgx, lltmass,  PSYM = 1

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
	CHARSIZE = 0.8
xyouts, 0.21, 0.81, 'correlation coefficient [LE]= ' + srle, /NORM, $
	CHARSIZE = 0.8
xyouts, 0.21, 0.78, 'correlation coefficient [LT]= ' + srlt, /NORM, $
	CHARSIZE = 0.8

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

syearlab = '1980, 1984-1989'			; year label.
xtype = 1					; log   : mass.
ytype = 0					; linear: # in bin.

;mhp0, m1, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, 'A < flare < B', /NORMAL
;mhp0, m2, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, 'B < flare < C', /NORMAL
;mhp0, m3, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, 'C < flare < M', /NORMAL
;mhp0, m4, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, 'M < flare < X', /NORMAL
;mhp0, m5, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '    flare > X', /NORMAL

DEVICE, /CLOSE
SET_PLOT, 'X'

end
