pro fs1

;+
; Scatter Plot: GOES flare versus speed
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.e+3				; mass [grams]
sxmass = size (xmass)
sgx    = size (gx)

nval   = sxmass (1)
nxmass = sxmass (1)
ngx    = sgx     (1)
;print, 'nxmass: ', nxmass, '   ngx: ', ngx

for i = 0, nxmass - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pxmass = fltarr (nval)
pspd   = fltarr (nval)
tlgx    = 0.0
tspd    = 0.0

ieq = 0
ile = 0
ilt = 0

neq = 0
nle = 0
nlt = 0

eqgx = fltarr (nval)
legx = fltarr (nval)
ltgx = fltarr (nval)

eqmass = fltarr (nval)
lemass = fltarr (nval)
ltmass = fltarr (nval)

eqspd  = fltarr (nval)
lespd  = fltarr (nval)
ltspd  = fltarr (nval)

for i = 0, nxmass - 1  do $
begin
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pxmass (ip) = xmass (i)
      pspd   (ip) = speed (i)
      tlgx = tlgx + alog10 (gx (i))
      tspd = tspd + speed (i)
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqmass (ieq) = xmass (i)
         eqspd  (ieq) = speed (i)
         ieq = ieq + 1
      endif			$
      else			$
      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         lemass (ile) = xmass (i)
         lespd  (ile) = speed (i)
         ile = ile + 1
      endif			$
      else			$
      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltmass (ilt) = xmass (i)
         ltspd  (ilt) = speed (i)
         ilt = ilt + 1
      endif			$
      else			$
      begin
	 print, 'year/doy: ', year (i), ' ', doy (i), ' ', gb (i)
      end
   end				
end

npgx = ip
neq  = ieq
nle  = ile
nlt  = ilt
spnum = string (npgx, FORMAT = '(I4)')

lgxave = tlgx / npgx
slgxave = string (lgxave, FORMAT = '(F7.2)')
print, 'average (log (goes magnitude)) : ', lgxave
spdave = tspd / npgx
sspdave = string (spdave, FORMAT = '(F7.2)')
print, 'average (speed)) : ', spdave

neqlelt = neq + nle + nlt
print, 'npgx: ', npgx, '   neqlelt: ', neqlelt
print, ' neq: ', neq, '   nle: ', nle, '   nlt: ', nlt

eqgx = eqgx (0 : neq - 1)
legx = legx (0 : nle - 1)
ltgx = ltgx (0 : nlt - 1)

eqmass = eqmass (0 : neq - 1)
lemass = lemass (0 : nle - 1)
ltmass = ltmass (0 : nlt - 1)

eqspd = eqspd (0 : neq - 1)
lespd = lespd (0 : nle - 1)
ltspd = ltspd (0 : nlt - 1)

leqgx = alog10 (eqgx)
llegx = alog10 (legx)
lltgx = alog10 (ltgx)

;leqmass = alog10 (eqmass)
;llemass = alog10 (lemass)
;lltmass = alog10 (ltmass)

;leqspd = alog10 (eqspd)
;llespd = alog10 (lespd)
;lltspd = alog10 (ltspd)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pxmass = pxmass (0 : npgx - 1)
pspd   = pspd   (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pxmass: ', pxmass (i), '   pgx: ', pgx (i)

lpxmass = alog10 (pxmass)			; log (mass) [grams].
lpgx    = alog10 (pgx   )			; log (GOES mag) [w/m^2].
;lpspd   = alog10 (pspd)

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fs1.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fs1.ps', $
	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin,   7.5" Wide
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" High
!P.MULTI = [0, 0, 0, 0, 0]

tmass1 = 0.0
tmass2 = 0.0
tmass3 = 0.0
tmass4 = 0.0
tmass5 = 0.0

tspd1 = 0.0
tspd2 = 0.0
tspd3 = 0.0
tspd4 = 0.0
tspd5 = 0.0

nmass1 = 0
nmass2 = 0
nmass3 = 0
nmass4 = 0
nmass5 = 0

nspd1 = 0
nspd2 = 0
nspd3 = 0
nspd4 = 0
nspd5 = 0

mass1 = fltarr (npgx)
mass2 = fltarr (npgx)
mass3 = fltarr (npgx)
mass4 = fltarr (npgx)
mass5 = fltarr (npgx)

spd1 = fltarr (npgx)
spd2 = fltarr (npgx)
spd3 = fltarr (npgx)
spd4 = fltarr (npgx)
spd5 = fltarr (npgx)

im1 = 0
im2 = 0
im3 = 0
im4 = 0
im5 = 0

il1 = 0
il2 = 0
il3 = 0
il4 = 0
il5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      nmass1 = nmass1 + 1
      tmass1 = tmass1 + pxmass (i) 
      mass1 (im1) = pxmass (i)
      im1 = im1 + 1
      spd1 (im1) = pspd (i)
      tspd1 = tspd1 + pspd (i)
      il1 = il1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      nmass2 = nmass2 + 1
      tmass2 = tmass2 + pxmass (i)
      mass2 (im2) = pxmass (i)
      im2 = im2 + 1
      spd2 (im2) = pspd (i)
      tspd2 = tspd2 + pspd (i)
      il2 = il2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      nmass3 = nmass3 + 1
      tmass3 = tmass3 + pxmass (i)
      mass3 (im3) = pxmass (i)
      im3 = im3 + 1
      spd3 (im3) = pspd (i)
      tspd3 = tspd3 + pspd (i)
      il3 = il3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      nmass4 = nmass4 + 1
      tmass4 = tmass4 + pxmass (i)
      mass4 (im4) = pxmass (i)
      im4 = im4 + 1
      spd4 (im4) = pspd (i)
      tspd4 = tspd4 + pspd (i)
      il4 = il4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      nmass5 = nmass5 + 1
      tmass5 = tmass5 + pxmass (i)
      mass5 (im5) = pxmass (i)
      im5 = im5 + 1
      spd5 (im5) = pspd (i)
      tspd5 = tspd5 + pspd (i)
      il5 = il5 + 1
   endif

endfor

;m1 = mass1 (0: nmass1 - 1)
;m2 = mass2 (0: nmass2 - 1)
;m3 = mass3 (0: nmass3 - 1)
;m4 = mass4 (0: nmass4 - 1)
;m5 = mass5 (0: nmass5 - 1)

;gnum = nmass1 + nmass2 + nmass3 + nmass4 + nmass5

;avg1 = tmass1 / nmass1
;avg2 = tmass2 / nmass2 
;avg3 = tmass3 / nmass3
;avg4 = tmass4 / nmass4
;avg5 = tmass5 / nmass5
;avg = (tmass1 + tmass2 + tmass3 + tmass4 + tmass5) / gnum

;savg1 = string (avg1, FORMAT = '(E8.2)')
;savg2 = string (avg2, FORMAT = '(E8.2)')
;savg3 = string (avg3, FORMAT = '(E8.2)')
;savg4 = string (avg4, FORMAT = '(E8.2)')
;savg5 = string (avg5, FORMAT = '(E8.2)')

;print, '<Mass> (A < flare < B) = ', avg1
;print, '<Mass> (B < flare < C) = ', avg2
;print, '<Mass> (C < flare < M) = ', avg3
;print, '<Mass> (M < flare < X) = ', avg4
;print, '<Mass> (X < flare    ) = ', avg5

;print, 'gnum: ', gnum, '   mass ave: ', avg
;sgnum = string (gnum, FORMAT = '(I4)')
;savg = string (avg, FORMAT = '(E8.2)')

;r = CORRELATE (lpgx, lpxmass)
;sr = string (r, FORMAT = '(F6.3)')
;print, 'correlation: flare magnitude vs. mass: ', r

;-------------------------------------------------------------------------------
nspd1 = il1
nspd2 = il2
nspd3 = il3
nspd4 = il4
nspd5 = il5

spd1 = spd1 (0: nspd1 - 1)
spd2 = spd2 (0: nspd2 - 1)
spd3 = spd3 (0: nspd3 - 1)
spd4 = spd4 (0: nspd4 - 1)
spd5 = spd5 (0: nspd5 - 1)

gnum = nspd1 + nspd2 + nspd3 + nspd4 + nspd5

avg1 = tspd1 / nspd1
avg2 = tspd2 / nspd2 
avg3 = tspd3 / nspd3
avg4 = tspd4 / nspd4
avg5 = tspd5 / nspd5
spdavg = (tspd1 + tspd2 + tspd3 + tspd4 + tspd5) / gnum

savg1 = string (avg1, FORMAT = '(F7.2)')
savg2 = string (avg2, FORMAT = '(F7.2)')
savg3 = string (avg3, FORMAT = '(F7.2)')
savg4 = string (avg4, FORMAT = '(F7.2)')
savg5 = string (avg5, FORMAT = '(F7.2)')

print, '<Speed> (A < flare < B) = ', avg1
print, '<Speed> (B < flare < C) = ', avg2
print, '<Speed> (C < flare < M) = ', avg3
print, '<Speed> (M < flare < X) = ', avg4
print, '<Speed> (X < flare    ) = ', avg5

print, 'gnum: ', gnum, '   spd ave: ', spdavg
sgnum = string (gnum, FORMAT = '(I4)')
sspdavg = string (spdavg, FORMAT = '(F7.2)')

rspd = CORRELATE (pgx, pspd)
srspd = string (rspd, FORMAT = '(F6.3)')
print, 'correlation: flare magnitude vs. speed: ', rspd

;-------------------------------------------------------------------------------
;plot, pgx, lpspd

print, 'max (pspd): ', max (pspd)

xmin = -8.0 
xmax = -3.0
ymin =   0.0 
ymax = 2500.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

PLOT, lpgx, pspd,   /YNOZ, PSYM = 4,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Speed',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'CME Speed   [km/s]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, spnum + ' CMEs measured', /NORM, CHARSIZE = 0.8

;xyouts, 0.21, 0.84, 'correlation coefficient = ' + srspd, /NORM, CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<speed>= ' + sspdave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.86, '<speed> (A < flare < B ) = ' + savg1, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.84, '<speed> (B < flare < C ) = ' + savg2, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.82, '<speed> (C < flare < M ) = ' + savg3, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.80, '<speed> (M < flare < X) = ' + savg4, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.78, '<speed> (X < flare    ) = ' + savg5, /NORM, $
;	CHARSIZE = 0.6

;-------------------------------------------------------------------------------
USERSYM, COS(A), SIN(A), /FILL

;eqlegx    = fltarr (neq + nle)
;eqleltgx  = fltarr (neq + nle + nlt)
;eqlespd   = fltarr (neq + nle)
;eqleltspd = fltarr (neq + nle + nlt)

;eqlegx   (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqlegx   (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqleltgx (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (neq + nle : neq + nle + nlt - 1) = ltgx (0 : nlt - 1)

;eqlespd   (0         : neq - 1)             = eqspd (0 : neq - 1)
;eqlespd   (neq       : neq + nle - 1)       = lespd (0 : nle - 1)
;eqleltspd (0         : neq - 1)             = eqspd (0 : neq - 1)
;eqleltspd (neq       : neq + nle - 1)       = lespd (0 : nle - 1)
;eqleltspd (neq + nle : neq + nle + nlt - 1) = ltspd (0 : nlt - 1)

eqlegx    = [eqgx,legx]
eqleltgx  = [eqgx,legx,ltgx]
eqlespd   = [eqspd,lespd]
eqleltspd = [eqspd,lespd,ltspd]

;print, 'size (eqgx):     ', size (eqgx)
;print, 'size (eqlegx):   ', size (eqlegx)
;print, 'size (eqleltgx): ', size (eqleltgx)

req     = CORRELATE (eqgx, eqspd)
reqle   = CORRELATE (eqlegx, eqlespd)
reqlelt = CORRELATE (eqleltgx, eqleltspd)
rle     = CORRELATE (legx, lespd)
rlt     = CORRELATE (ltgx, ltspd)

sreq     = string (req, FORMAT = '(F6.3)')
sreqle   = string (reqle, FORMAT = '(F6.3)')
sreqlelt = string (reqlelt, FORMAT = '(F6.3)')
srle     = string (rle, FORMAT = '(F6.3)')
srlt     = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ]       vs. speed: ', req
print, 'correlation: flare magnitude [EQ,LE]    vs. speed: ', reqle
print, 'correlation: flare magnitude [EQ,LE,lT] vs. speed: ', reqlelt
print, 'correlation: flare magnitude [LE]       vs. speed: ', rle
print, 'correlation: flare magnitude [LT]       vs. speed: ', rlt

PLOT, leqgx, eqspd, /YNOZ, PSYM = 8,		$
;     CHARSIZE = 0.5,				$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Speed',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'CME Speed   [km/s]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, lespd,  PSYM = 5
OPLOT, lltgx, ltspd,  PSYM = 1

PLOTS, [0.21, 0.845], /NORM, PSYM = 8           
PLOTS, [0.41, 0.845], /NORM, PSYM = 5           
PLOTS, [0.61, 0.845], /NORM, PSYM = 1           
xyouts, 0.23, 0.84, 'flare EQ value', /NORM, CHARSIZE = 0.6
xyouts, 0.43, 0.84, 'flare LE value', /NORM, CHARSIZE = 0.6
xyouts, 0.63, 0.84, 'flare LT value', /NORM, CHARSIZE = 0.6

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
;xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0

xyouts, 0.21, 0.90, 'SMM   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8

;xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.81, 'correlation coefficient [EQ,LE]= ' + sreqle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.78, 'correlation coefficient [EQ,LE,LT]= ' + sreqlelt, /NORM, $
;	CHARSIZE = 0.8

;xyouts, 0.21, 0.75, 'correlation coefficient [LE]= ' + srle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.72, 'correlation coefficient [LT]= ' + srlt, /NORM, $
;	CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<speed>= ' + sspdave, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

DEVICE, /CLOSE
SET_PLOT, 'X'

end
