pro fwidth1

;+
; Scatter Plot: GOES flare versus width
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   25 April 1996 
;-

@cpmassc.common

rdcpmass3

xmass = xmass * 1.e+3				; mass [grams]
sxmass = size (xmass)
sgx    = size (gx)

nval   = sxmass (1)
nxmass = sxmass (1)
ngx    = sgx     (1)
;print, 'nxmass: ', nxmass, '   ngx: ', ngx

for i = 0, nxmass - 1  do $
   if (gx (i) gt 1.0) then $
   print, 'i: ', i, 'year/doy: ', year (i), doy (i), ' xmass: ', xmass (i), ' gb: ', gb (i), ' gx: ', gx (i), $
   ' hf: ', hf (i)

;--- Assemble only those masses for which GOES X-ray > 0.0

ip = 0
pgb    = strarr (nval)
pgx    = fltarr (nval)
pxmass = fltarr (nval)
pwid   = fltarr (nval)
tlgx    = 0.0
twid    = 0.0

ieq = 0
ile = 0
ilt = 0

neq = 0
nle = 0
nlt = 0

eqgx = fltarr (nval)
legx = fltarr (nval)
ltgx = fltarr (nval)

eqmass = fltarr (nval)
lemass = fltarr (nval)
ltmass = fltarr (nval)

eqwid  = fltarr (nval)
lewid  = fltarr (nval)
ltwid  = fltarr (nval)

for i = 0, nxmass - 1  do $
begin
   if (gx (i) gt 0.0) then	$		; GOES mag > 0.0
   begin
      pgb (ip)    = gb (i)
      pgx (ip)    = gx (i)
      pxmass (ip) = xmass (i)
      pwid   (ip) = width (i)
      tlgx = tlgx + alog10 (gx (i))
      twid = twid + width (i)
      ip = ip + 1

      if (gb (i) eq 'EQ') then $		; GOES mag = value > 0.0
      begin
         eqgx (ieq)   = gx (i)
         eqmass (ieq) = xmass (i)
         eqwid  (ieq) = width (i)
         ieq = ieq + 1
      endif			$
      else			$
      if (gb (i) eq 'LE') then $		; GOES mag < or = value > 0.0
      begin
         legx (ile)   = gx (i)
         lemass (ile) = xmass (i)
         lewid  (ile) = width (i)
         ile = ile + 1
      endif			$
      else			$
      if (gb (i) eq 'LT') then $		; GOES mag < value > 0.0
      begin
         ltgx (ilt)   = gx (i)
         ltmass (ilt) = xmass (i)
         ltwid  (ilt) = width (i)
         ilt = ilt + 1
      endif			$
      else			$
      begin
	 print, 'year/doy: ', year (i), ' ', doy (i), ' ', gb (i)
      end
   end				
end

npgx = ip
neq  = ieq
nle  = ile
nlt  = ilt
spnum = string (npgx, FORMAT = '(I4)')

lgxave = tlgx / npgx
slgxave = string (lgxave, FORMAT = '(F7.2)')
print, 'Average (log (GOES Intensity)) : ', lgxave
widave = twid / npgx
swidave = string (widave, FORMAT = '(F7.2)')
print, 'Average (width)) : ', widave

neqlelt = neq + nle + nlt
print, 'npgx: ', npgx, '   neqlelt: ', neqlelt
print, ' neq: ', neq, '   nle: ', nle, '   nlt: ', nlt

eqgx = eqgx (0 : neq - 1)
legx = legx (0 : nle - 1)
ltgx = ltgx (0 : nlt - 1)

eqmass = eqmass (0 : neq - 1)
lemass = lemass (0 : nle - 1)
ltmass = ltmass (0 : nlt - 1)

eqwid = eqwid (0 : neq - 1)
lewid = lewid (0 : nle - 1)
ltwid = ltwid (0 : nlt - 1)

leqgx = alog10 (eqgx)
llegx = alog10 (legx)
lltgx = alog10 (ltgx)

;leqmass = alog10 (eqmass)
;llemass = alog10 (lemass)
;lltmass = alog10 (ltmass)

;leqwid = alog10 (eqwid)
;llewid = alog10 (lewid)
;lltwid = alog10 (ltwid)

;--- Resize arrays.

pgb    = pgb    (0 : npgx - 1)
pgx    = pgx    (0 : npgx - 1)
pxmass = pxmass (0 : npgx - 1)
pwid   = pwid   (0 : npgx - 1)

;for i = 0, npgx - 1  do $
;   print, 'i: ', i, '   pxmass: ', pxmass (i), '   pgx: ', pgx (i)

lpxmass = alog10 (pxmass)			; log (mass) [grams].
lpgx    = alog10 (pgx   )			; log (GOES mag) [w/m^2].
;lpwid   = alog10 (pwid)

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fwidth1.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'fwidth1.ps', $
	XOFFSET = 1.27, XSIZE = 19.05, $	; 0.5" left margin, 7.5" wide
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" high
!P.MULTI = [0, 0, 0, 0, 0]

tmass1 = 0.0
tmass2 = 0.0
tmass3 = 0.0
tmass4 = 0.0
tmass5 = 0.0

twid1 = 0.0
twid2 = 0.0
twid3 = 0.0
twid4 = 0.0
twid5 = 0.0

nmass1 = 0
nmass2 = 0
nmass3 = 0
nmass4 = 0
nmass5 = 0

nwid1 = 0
nwid2 = 0
nwid3 = 0
nwid4 = 0
nwid5 = 0

mass1 = fltarr (npgx)
mass2 = fltarr (npgx)
mass3 = fltarr (npgx)
mass4 = fltarr (npgx)
mass5 = fltarr (npgx)

wid1 = fltarr (npgx)
wid2 = fltarr (npgx)
wid3 = fltarr (npgx)
wid4 = fltarr (npgx)
wid5 = fltarr (npgx)

im1 = 0
im2 = 0
im3 = 0
im4 = 0
im5 = 0

il1 = 0
il2 = 0
il3 = 0
il4 = 0
il5 = 0

for i = 0, npgx - 1 do $
begin
   if (pgx (i) gt 1.0e-8 and pgx (i) le 1.0e-7) then $
   begin
      nmass1 = nmass1 + 1
      tmass1 = tmass1 + pxmass (i) 
      mass1 (im1) = pxmass (i)
      im1 = im1 + 1
      wid1 (im1) = pwid (i)
      twid1 = twid1 + pwid (i)
      il1 = il1 + 1
   endif

   if (pgx (i) gt 1.0e-7 and pgx (i) le 1.0e-6) then $
   begin
      nmass2 = nmass2 + 1
      tmass2 = tmass2 + pxmass (i)
      mass2 (im2) = pxmass (i)
      im2 = im2 + 1
      wid2 (im2) = pwid (i)
      twid2 = twid2 + pwid (i)
      il2 = il2 + 1
   endif

   if (pgx (i) gt 1.0e-6 and pgx (i) le 1.0e-5) then $
   begin
      nmass3 = nmass3 + 1
      tmass3 = tmass3 + pxmass (i)
      mass3 (im3) = pxmass (i)
      im3 = im3 + 1
      wid3 (im3) = pwid (i)
      twid3 = twid3 + pwid (i)
      il3 = il3 + 1
   endif

   if (pgx (i) gt 1.0e-5 and pgx (i) le 1.0e-4) then $
   begin
      nmass4 = nmass4 + 1
      tmass4 = tmass4 + pxmass (i)
      mass4 (im4) = pxmass (i)
      im4 = im4 + 1
      wid4 (im4) = pwid (i)
      twid4 = twid4 + pwid (i)
      il4 = il4 + 1
   endif

   if (pgx (i) gt 1.0e-4) then $
   begin
      nmass5 = nmass5 + 1
      tmass5 = tmass5 + pxmass (i)
      mass5 (im5) = pxmass (i)
      im5 = im5 + 1
      wid5 (im5) = pwid (i)
      twid5 = twid5 + pwid (i)
      il5 = il5 + 1
   endif

endfor

;m1 = mass1 (0: nmass1 - 1)
;m2 = mass2 (0: nmass2 - 1)
;m3 = mass3 (0: nmass3 - 1)
;m4 = mass4 (0: nmass4 - 1)
;m5 = mass5 (0: nmass5 - 1)

;gnum = nmass1 + nmass2 + nmass3 + nmass4 + nmass5

;avg1 = tmass1 / nmass1
;avg2 = tmass2 / nmass2 
;avg3 = tmass3 / nmass3
;avg4 = tmass4 / nmass4
;avg5 = tmass5 / nmass5
;avg = (tmass1 + tmass2 + tmass3 + tmass4 + tmass5) / gnum

;savg1 = string (avg1, FORMAT = '(E8.2)')
;savg2 = string (avg2, FORMAT = '(E8.2)')
;savg3 = string (avg3, FORMAT = '(E8.2)')
;savg4 = string (avg4, FORMAT = '(E8.2)')
;savg5 = string (avg5, FORMAT = '(E8.2)')

;print, '<Mass> (A < flare < B) = ', avg1
;print, '<Mass> (B < flare < C) = ', avg2
;print, '<Mass> (C < flare < M) = ', avg3
;print, '<Mass> (M < flare < X) = ', avg4
;print, '<Mass> (X < flare    ) = ', avg5

;print, 'gnum: ', gnum, '   mass ave: ', avg
;sgnum = string (gnum, FORMAT = '(I4)')
;savg = string (avg, FORMAT = '(E8.2)')

;r = CORRELATE (lpgx, lpxmass)
;sr = string (r, FORMAT = '(F6.3)')
;print, 'correlation: flare magnitude vs. mass: ', r

;-------------------------------------------------------------------------------
nwid1 = il1
nwid2 = il2
nwid3 = il3
nwid4 = il4
nwid5 = il5

wid1 = wid1 (0: nwid1 - 1)
wid2 = wid2 (0: nwid2 - 1)
wid3 = wid3 (0: nwid3 - 1)
wid4 = wid4 (0: nwid4 - 1)
wid5 = wid5 (0: nwid5 - 1)

gnum = nwid1 + nwid2 + nwid3 + nwid4 + nwid5

avg1 = twid1 / nwid1
avg2 = twid2 / nwid2 
avg3 = twid3 / nwid3
avg4 = twid4 / nwid4
avg5 = twid5 / nwid5
widavg = (twid1 + twid2 + twid3 + twid4 + twid5) / gnum

savg1 = string (avg1, FORMAT = '(F6.2)')
savg2 = string (avg2, FORMAT = '(F6.2)')
savg3 = string (avg3, FORMAT = '(F6.2)')
savg4 = string (avg4, FORMAT = '(F6.2)')
savg5 = string (avg5, FORMAT = '(F6.2)')

print, '<Width> (A < flare < B) = ', avg1
print, '<Width> (B < flare < C) = ', avg2
print, '<Width> (C < flare < M) = ', avg3
print, '<Width> (M < flare < X) = ', avg4
print, '<Width> (X < flare    ) = ', avg5

print, 'gnum: ', gnum, '   wid ave: ', widavg
sgnum = string (gnum, FORMAT = '(I4)')
swidavg = string (widavg, FORMAT = '(F6.2)')

rwid = CORRELATE (pgx, pwid)
srwid = string (rwid, FORMAT = '(F6.3)')
print, 'correlation: flare magnitude vs. width: ', rwid

;-------------------------------------------------------------------------------
;plot, pgx, lpwid

ymax = max (pwid)
print, 'max (pwid): ', ymax

xmin = -8.0 
xmax = -3.0
ymin =   0.0 
ymax = 180.0

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A)

PLOT, lpgx, pwid,   /YNOZ, PSYM = 4,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Width',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'CME Width   [degrees]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, spnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
;xyouts, 0.21, 0.84, 'correlation coefficient = ' + srwid, /NORM, CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<width>= '      + swidave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.86, '<width> (A < flare < B ) = ' + savg1, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.84, '<width> (B < flare < C ) = ' + savg2, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.82, '<width> (C < flare < M ) = ' + savg3, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.80, '<width> (M < flare < X) = ' + savg4, /NORM, $
;	CHARSIZE = 0.6
;xyouts, 0.60, 0.78, '<width> (X < flare    ) = ' + savg5, /NORM, $
;	CHARSIZE = 0.6

;-------------------------------------------------------------------------------
USERSYM, COS(A), SIN(A), /FILL

;eqlegx    = fltarr (neq + nle)
;eqleltgx  = fltarr (neq + nle + nlt)
;eqlewid   = fltarr (neq + nle)
;eqleltwid = fltarr (neq + nle + nlt)

;eqlegx   (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqlegx   (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (0         : neq - 1)             = eqgx (0 : neq - 1)
;eqleltgx (neq       : neq + nle - 1)       = legx (0 : nle - 1)
;eqleltgx (neq + nle : neq + nle + nlt - 1) = ltgx (0 : nlt - 1)

;eqlewid   (0         : neq - 1)             = eqwid (0 : neq - 1)
;eqlewid   (neq       : neq + nle - 1)       = lewid (0 : nle - 1)
;eqleltwid (0         : neq - 1)             = eqwid (0 : neq - 1)
;eqleltwid (neq       : neq + nle - 1)       = lewid (0 : nle - 1)
;eqleltwid (neq + nle : neq + nle + nlt - 1) = ltwid (0 : nlt - 1)

eqlegx    = [eqgx,legx]
eqleltgx  = [eqgx,legx,ltgx]
eqlewid   = [eqwid,lewid]
eqleltwid = [eqwid,lewid,ltwid]

;print, 'size (eqgx):     ', size (eqgx)
;print, 'size (eqlegx):   ', size (eqlegx)
;print, 'size (eqleltgx): ', size (eqleltgx)

req     = CORRELATE (eqgx, eqwid)
reqle   = CORRELATE (eqlegx, eqlewid)
reqlelt = CORRELATE (eqleltgx, eqleltwid)
rle     = CORRELATE (legx, lewid)
rlt     = CORRELATE (ltgx, ltwid)

sreq     = string (req, FORMAT = '(F6.3)')
sreqle   = string (reqle, FORMAT = '(F6.3)')
sreqlelt = string (reqlelt, FORMAT = '(F6.3)')
srle     = string (rle, FORMAT = '(F6.3)')
srlt     = string (rlt, FORMAT = '(F6.3)')

print, 'correlation: flare magnitude [EQ]       vs. width: ', req
print, 'correlation: flare magnitude [EQ,LE]    vs. width: ', reqle
print, 'correlation: flare magnitude [EQ,LE,lT] vs. width: ', reqlelt
print, 'correlation: flare magnitude [LE]       vs. width: ', rle
print, 'correlation: flare magnitude [LT]       vs. width: ', rlt

PLOT, leqgx, eqwid, /YNOZ, PSYM = 8,		$
     XRANGE = [xmin, xmax],			$
     YRANGE = [ymin, ymax],			$
     TITLE = 'Flare Intensity vs. CME Width',	$
     XTITLE = 'Log (Flare Intensity) [w/m^2]',	$
     YTITLE = 'CME Width   [degrees]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

OPLOT, llegx, lewid,  PSYM = 5
OPLOT, lltgx, ltwid,  PSYM = 1

PLOTS, [0.21, 0.845], /NORM, PSYM = 8           
PLOTS, [0.41, 0.845], /NORM, PSYM = 5           
PLOTS, [0.61, 0.845], /NORM, PSYM = 1           
xyouts, 0.23, 0.84, 'flare EQ value', /NORM, CHARSIZE = 0.6
xyouts, 0.43, 0.84, 'flare LE value', /NORM, CHARSIZE = 0.6
xyouts, 0.63, 0.84, 'flare LT value', /NORM, CHARSIZE = 0.6

bnum = neq + nle + nlt
sbnum = string (bnum, FORMAT = '(I4)')
xyouts, 0.21, 0.90, 'SMM C/P   1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, sbnum + ' CMEs measured', /NORM, CHARSIZE = 0.8
;xyouts, 0.21, 0.84, 'correlation coefficient [EQ]= ' + sreq, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.81, 'correlation coefficient [EQ,LE]= ' + sreqle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.78, 'correlation coefficient [EQ,LE,LT]= ' + sreqlelt, /NORM, $
;	CHARSIZE = 0.8

;xyouts, 0.21, 0.75, 'correlation coefficient [LE]= ' + srle, /NORM, $
;	CHARSIZE = 0.8
;xyouts, 0.21, 0.72, 'correlation coefficient [LT]= ' + srlt, /NORM, $
;	CHARSIZE = 0.8

;xyouts, 0.60, 0.90, '<log (goes)>= ' + slgxave, /NORM, CHARSIZE = 0.6
;xyouts, 0.60, 0.88, '<width>= '      + swidave, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

DEVICE, /CLOSE
SET_PLOT, 'X'

end
