pro mh2a

;+
; Generate SMM C/P Excess Mass histograms for period 1980, 1984-1989.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   29 August 1994
; revised 28 June 1995 to include year 1989.  Also use rdcpmass2.
; 2 August 1996: Plot Size: 5"w x 4" h.
;-

@cpmassa.common

rdcpmass2				; Read SMM C/P mass data.

xmass = xmass * 1000.0			; convert from kg to grams.
xm80  = xm80  * 1000.0
xm84  = xm84  * 1000.0
xm85  = xm85  * 1000.0
xm86  = xm86  * 1000.0
xm87  = xm87  * 1000.0
xm88  = xm88  * 1000.0
xm89  = xm89  * 1000.0

sxmass = size (xmass)
ncme = sxmass (1)
xmp = fltarr (ncme)
ip = 0
for i = 0, ncme - 1 do			$
   if (xmass (i) le 1.0e+16) then	$
   begin
       xmp (ip) = xmass (i)
       ip = ip + 1
   end
nxmp = ip
xmp = xmp (0 : nxmp - 1)

SET_PLOT, 'PS'
!P.FONT = 0
DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'mh2a.ps', $
        XOFFSET = 2.54, XSIZE = 12.70, $	; 1" left margin,   5.0" wide
	YOFFSET = 5.08, YSIZE = 10.16		; 2" bottom margin, 4.0" high
!P.MULTI = [0, 0, 0, 0, 0]

xmmin = min (xmass)
xmmax = max (xmass)
xmdif = xmmax - xmmin
binsiz = xmdif / 20.0

;print, 'min (xmass): ', xmmin, '   max (xmass): ', xmmax
;print, 'xmdif: ', xmdif, '   binsiz: ', binsiz

; Define year variables

year80 = 1980
year84 = 1984
year85 = 1985
year86 = 1986
year87 = 1987
year88 = 1988
year89 = 1989

syear80 = string (year80, FORMAT = '(I4)')
syear84 = string (year84, FORMAT = '(I4)')
syear85 = string (year85, FORMAT = '(I4)')
syear86 = string (year86, FORMAT = '(I4)')
syear87 = string (year87, FORMAT = '(I4)')
syear88 = string (year88, FORMAT = '(I4)')
syear89 = string (year89, FORMAT = '(I4)')

plab = 'SMM C/P   CME Mass Histogram'

; Plot mass histograms.

 syearlab = '1980, 1984-1989'

; mhp0, xm80,  syear80,  0, 0
; mhp0, xm84,  syear84,  0, 0
; mhp0, xm85,  syear85,  0, 0
; mhp0, xm86,  syear86,  0, 0
; mhp0, xm87,  syear87,  0, 0
; mhp0, xm88,  syear88,  0, 0
; mhp0, xm89,  syear89,  0, 0

; mhp0, xm80,  syear80,  1, 0
; mhp0, xm84,  syear84,  1, 0
; mhp0, xm85,  syear85,  1, 0
; mhp0, xm86,  syear86,  1, 0
; mhp0, xm87,  syear87,  1, 0
; mhp0, xm88,  syear88,  1, 0
; mhp0, xm89,  syear89,  1, 0

; mhp0, xm80,  syear80,  1, 1
; mhp0, xm84,  syear84,  1, 1
; mhp0, xm85,  syear85,  1, 1
; mhp0, xm86,  syear86,  1, 1
; mhp0, xm87,  syear87,  1, 1
; mhp0, xm88,  syear88,  1, 1
; mhp0, xm89,  syear89,  1, 1

xinc = 2.0e+14
xtype = 0
ytype = 0
; mhp0, xmass, syearlab, 1, 1
;--- mhp2a, xmass, syearlab, 0, 0
; cmehp2, xmass,  'mass', 'grams', plab, syearlab, xinc, xtype, ytype
xtype = 1
ytype = 0
 mhp2a, xmass, syearlab, 1, 0
; cmehp2, xmass,  'mass', 'grams', plab, syearlab, xinc, xtype, ytype

 ;print, '!P.POSITION: ', !P.POSITION
 ;print, '!P.REGION:   ', !P.REGION
 ;print, '!P.THICK:    ', !P.THICK
 ;print, '!P.XTICKLEN: ', !P.TICKLEN

 ;print, '!X.WINDOW:   ', !X.WINDOW
 ;print, '!Y.WINDOW:   ', !Y.WINDOW
 ;print, '!X.REGION:   ', !X.WINDOW
 ;print, '!Y.REGION:   ', !Y.WINDOW

 ;print, '!D.X_SIZE:   ', !D.X_SIZE
 ;print, '!D.Y_SIZE:   ', !D.Y_SIZE
 ;print, '!D.X_VSIZE:  ', !D.X_VSIZE
 ;print, '!D.Y_VSIZE:  ', !D.Y_VSIZE

DEVICE, /CLOSE					; flush buffer to PS file.
SET_PLOT, 'X'					; reset output to X windows.

end
