pro mhp, mass, yearlab

;+
; Plot SMM C/P excess mass for total period.
; mass: fltarr of excess mass values [grams].
; yearlab: string label for plot describing years covered.
;
; Andrew L. Stanger   HAO/NCAR   19 August 1994
;-

; CME excess mass (no jupiter correction; coefficient of 1.e+14 grams).

smass = size (mass)
nmass = smass (1)
ncme  = nmass - 1

; Compute mass (grams) and log (mass).

lmass = fltarr (nmass)
lmass (0) = 0.0
lmass (1:ncme) = alog10 (mass (1:ncme))		; log10 (mass)

;for i = 0, ncme do $
;   print, i, mass (i), lmass (i)

; Create histogram bins.

xmin = 10.0					; Minimum exponent of 10.
xmax = 14.0					; Maximum exponent of 10.
xinc = 0.25					; Increment exponent of 10.

nbin = (xmax - xmin) / xinc + 1
bin = intarr (nbin)
binval = fltarr (nbin + 1)

binexp0 = xmin - xinc
for i = 0, nbin do $
   begin
      binexp = binexp0 + i * xinc
      binval (i) = binexp
   end

; Fill bins.

for i = 1, ncme do $
   for n = 0, nbin - 1 do $
      if (lmass (i) gt binval (n) and lmass (i) le binval (n+1)) then $
         bin (n) = bin (n) + 1

;for i = 0, nbin - 1 do $
;   print, 'bin: ', i, '   binval: ', binval (i), '   binnum: ', bin (i)  

sum = 0 
for i = 0, 11 do sum = sum + bin (i)
;print, 'ncme: ', ncme, '   sum (histogram): ', sum

; Plot histogram.

XVAL = binval + xinc / 2.0
YVAL = bin / float (ncme)

plab = ' CME Mass Histogram   SMM C/P'
ylab = 'FRACTION OF EVENTS PER BIN'
xlab = 'log exponent (MASS) [kg]'

;for i = 0, nbin - 1 do $
;   print, i, '   binval: ', binval (i), $
;	     '   xval: ', xval (i), '   yval: ', yval (i)

ymin = 0.0
ymax = 0.3

dxmin = 10.0
dxmax = 14.0

PLOT, XVAL, YVAL, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
      TITLE  = plab, $
      XTITLE = xlab, $
      YTITLE = ylab, $
      PSYM   = 10				; Histogram plot mode.

;OPLOT, XVAL, YVAL, PSYM   = 5			; overplot with triangles.

;for i = 0, 11 do $
;   box, xval (i), !Y.CRANGE (0), xval (i) + xval (i+1), yval (i), 45 

; Write year label on plot.

xyouts, 0.2, 0.85, yearlab, /NORMAL

; Write # CMEs measured on plot.

cmelab = string (ncme, FORMAT = '(I3)') + ' CMEs measured'
xyouts, 0.2, 0.8, cmelab, /NORMAL

; Compute average CME mass.

masstot = 0.0
for i = 1, ncme do masstot = masstot + mass (i)
massave = masstot / ncme 
;print, 'total CME mass: ', masstot, '   Average CME mass: ', massave

; Write average mass on plot.

avelab = 'mean = ' + string (massave, FORMAT = '(E8.2)') + ' kg'
xyouts, 0.2, 0.75, avelab, /NORMAL

end

 
; --- Histogram fill procedure -------------------------------------------------

pro box, x0, y0, x1, y1, line
   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0],  ORIENTATION = line 
end
