pro mhp1, mass, yearlab, xtype, ytype

;+
; Plot SMM C/P excess mass for total period.
; mass: fltarr of excess mass values [grams].
; yearlab: string label for plot describing years covered.
; xtype: 0 = linear, 1 = log
; ytype: 0 = linear, 1 = log
;
; Andrew L. Stanger   HAO/NCAR   19 August 1994
;-

; CME excess mass (no jupiter correction; coefficient of 1.e+14 grams).

smass = size (mass)
nmass = smass (1)
ncme  = nmass

;print, 'nmass: ', nmass
;print, 'mass: '
;print, mass

; Compute log (mass).

lmass = fltarr (nmass)
lmass = alog10 (mass)				; log (mass)

;for i = 0, ncme - 1 do $
;   print, i, mass (i), lmass (i)

; Compute average CME mass.

masstot = 0.0
for i = 0, ncme - 1 do masstot = masstot + mass (i)
massave = masstot / ncme 
lmassave = alog10 (massave)
lyave    = alog10 (0.2)
;print, 'total CME mass: ', masstot, '   Average CME mass: ', massave

; Create histogram bins.

IF (xtype eq 1) THEN $				; x logarithmic.
BEGIN
   xmin = 12.0					; Minimum exponent of 10.
   xmax = 17.0					; Maximum exponent of 10.
   xinc = 0.25					; Increment exponent of 10.

   nbin = (xmax - xmin) / xinc + 1
   bin = intarr (nbin)
   binval = fltarr (nbin + 1)

   binexp0 = xmin
   for i = 0, nbin do $
      begin
         binexp = binexp0 + i * xinc
         binval (i) = binexp
      end

   ; Fill bins.

   for i = 0, ncme - 1 do $
      for n = 0, nbin - 1 do $
         if (lmass (i) gt binval (n) and lmass (i) le binval (n+1)) then $
            bin (n) = bin (n) + 1

   dxmin = 13.0
   dxmax = 17.0

END $
ELSE $						; x linear.
BEGIN
   xmin = 0.0
   xmax = 2.0e+16
   xinc = 1.0e+15

   nbin = ((xmax - xmin) / xinc) + 2
   bin = intarr (nbin)
   binval = fltarr (nbin + 1)

   bin0 = xmin
   for i = 0, nbin do $
      begin
	 binval (i) = bin0 + i * xinc
      end

   ; Fill bins.

   for i = 0, ncme - 1 do $
      for n = 0, nbin - 1 do $
         if (mass (i) gt binval (n) and mass (i) le binval (n+1)) then $
            bin (n) = bin (n) + 1

   dxmin = 1.0e+13
   dxmax = 2.0e+16 

END

;for i = 0, nbin - 1 do $
;   print, 'bin: ', i, '   binval: ', binval (i), '   binnum: ', bin (i)  

sum = 0 
for i = 0, 11 do sum = sum + bin (i)
;print, 'ncme: ', ncme, '   sum (histogram): ', sum

; Plot histogram.

xval = binval + xinc / 2.0

plab = ' CME Mass Histogram   SMM C/P'

IF (xtype eq 0 and ytype eq 0) THEN $		; x linear, y linear.
   BEGIN
   xlab = 'MASS [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
;         TITLE  = plab, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
;	 POSITION = [0.10, 0.00, 0.9, 0.50], $
         XTHICK = 5.0, YTHICK = 5.0, $
;	 XTICKLEN = 0.02, YTICKLEN = 0.02, $
         PSYM   = 10    			; Histogram plot mode.
   xyouts, massave, 2.0, 'average', /DATA, ORIENTATION=90
   plots, [massave, massave], [ymin, ymax], LINESTYLE = 1
   END $
ELSE $
IF (xtype eq 1 and ytype eq 0) THEN $		; x log, y linear.
   BEGIN
   xlab = 'log (MASS) [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
;         TITLE  = plab, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10    			; Histogram plot mode.
;	 /XTYPE         			; X axis logarithmic.
   xyouts, lmassave, 2.0, 'average', /DATA, ORIENTATION=90
   plots, [lmassave, lmassave], [ymin, ymax], LINESTYLE = 1
   END $
ELSE $
IF (xtype eq 0 and ytype eq 1) THEN $		; x linear, y log.
   BEGIN
;   xlab = 'MASS [grams]'
;   ylab = 'PERCENTAGE OF EVENTS PER BIN'
;   yval = (bin / float (ncme) ) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
         TITLE  = plab, $
         XTITLE = xlab, $
         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
	 /YTYPE         			; Y axis logarithmic.
   xyouts, massave, 0.2, 'average', /DATA, ORIENTATION = 90
   plots, [massave, massave], [ymin, ymax], LINESTYLE = 1
   END $
ELSE $
IF (xtype eq 1 and ytype eq 1) THEN $		; x log, y log.
   BEGIN
   xlab = 'log (MASS) [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 1.0e-1 
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
;         TITLE  = plab, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
;	 /XTYPE,        $			; X axis logarithmic.
	 /YTYPE         			; Y axis logarithmic.
   xyouts, lmassave, 0.2, 'average', /DATA, ORIENTATION = 90
   plots, [lmassave, lmassave], [ymin, ymax], LINESTYLE = 1
   END

;OPLOT, xval, yval, PSYM   = 5			; overplot with triangles.

;for i = 0, 11 do $
;   box, xval (i), !Y.CRANGE (0), xval (i) + xval (i+1), yval (i), 45 

; Write year label on plot.

if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.85, yearlab, /NORMAL $
else $
   xyouts, 0.21, 0.85, yearlab, /NORMAL

; Write # CMEs measured on plot.

cmelab = string (ncme, FORMAT = '(I3)') + ' CMEs measured'
if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.8, cmelab, /NORMAL $
else $
   xyouts, 0.21, 0.8, cmelab, /NORMAL

; Write average mass on plot.

avelab = 'mean = ' + string (massave, FORMAT = '(E8.2)') + ' g'
print, 'massave: ', massave
if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.75, avelab, /NORMAL $
else $
   xyouts, 0.21, 0.75, avelab, /NORMAL

;FOR i = 0, nbin - 1 DO $
;   print, i, '   binval: ', binval (i), $
;	     '   xval: ', xval (i), '   yval: ', yval (i)
end
 
; --- Histogram fill procedure -------------------------------------------------

pro box, x0, y0, x1, y1, line
   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0],  ORIENTATION = line 
end
