pro mhp33, mass1, mass2, mass3, yearlab1, yearlab2, yearlab3, xtype, ytype

;+
; Plot SMM C/P excess mass for total period.
; mass1: fltarr of excess mass values [grams].
; mass2: fltarr of excess mass values [grams].
; mass3: fltarr of excess mass values [grams].
; yearlab1: string label for plot describing year covered.
; yearlab2: string label for plot describing year covered.
; yearlab3: string label for plot describing year covered.
; xtype: 0 = linear, 1 = log
; ytype: 0 = linear, 1 = log
;
; Andrew L. Stanger   HAO/NCAR   23 September 1994
;-

; CME excess mass

smass1 = size (mass1)
nmass1 = smass1 (1)
ncme1  = nmass1

smass2 = size (mass2)
nmass2 = smass2 (1)
ncme2  = nmass2

smass3 = size (mass3)
nmass3 = smass3 (1)
ncme3  = nmass3

;print, 'nmass1: ', nmass1
;print, 'mass1: '
;print, mass1

; Compute log (mass).

lmass1 = fltarr (nmass1)
lmass1 = alog10 ( mass1)				; log (mass1)

lmass2 = fltarr (nmass2)
lmass2 = alog10 ( mass2)				; log (mass2)

lmass3 = fltarr (nmass3)
lmass3 = alog10 ( mass3)				; log (mass3)

;for i = 0, ncme1 - 1 do $
;   print, i, mass1 (i), lmass1 (i)

; Compute average CME mass.

masstot1 = 0.0
masstot2 = 0.0
masstot3 = 0.0
for i = 0, ncme1 - 1 do masstot1 = masstot1 + mass1 (i)
for i = 0, ncme2 - 1 do masstot2 = masstot2 + mass2 (i)
for i = 0, ncme3 - 1 do masstot3 = masstot3 + mass3 (i)
massave1 = masstot1 / ncme1 
massave2 = masstot2 / ncme2 
massave3 = masstot3 / ncme3 
lmassave1 = alog10 (massave1)
lmassave2 = alog10 (massave2)
lmassave3 = alog10 (massave3)
lyave    = alog10 (0.2)
print, yearlab1, '  Average CME mass:', massave1
print, yearlab2, '  Average CME mass:', massave2
print, yearlab3, '  Average CME mass:', massave3

; Create histogram bins.

IF (xtype eq 1) THEN $				; x logarithmic.
BEGIN
   xmin = 12.0					; Minimum exponent of 10.
   xmax = 17.0					; Maximum exponent of 10.
   xinc = 0.25					; Increment exponent of 10.

   nbin1 = (xmax - xmin) / xinc + 1
   nbin2 = (xmax - xmin) / xinc + 1
   nbin3 = (xmax - xmin) / xinc + 1
   bin1    = intarr (nbin1 + 1)
   binval1 = fltarr (nbin1 + 1)
   bin2    = intarr (nbin2 + 1)
   binval2 = fltarr (nbin2 + 1)
   bin3    = intarr (nbin3 + 1)
   binval3 = fltarr (nbin3 + 1)

   binexp0 = xmin
   for i = 0, nbin1 do $
      begin
         binexp = binexp0 + i * xinc
         binval1 (i) = binexp
      end

   for i = 0, nbin2 do $
      begin
         binexp = binexp0 + i * xinc
         binval2 (i) = binexp
      end

   for i = 0, nbin3 do $
      begin
         binexp = binexp0 + i * xinc
         binval3 (i) = binexp
      end


   ; Fill bins.

   for i = 0, ncme1 - 1 do $
      for n = 0, nbin1 - 1 do $
         if (lmass1 (i) gt binval1 (n) and lmass1 (i) le binval1 (n+1)) then $
            bin1 (n) = bin1 (n) + 1

   for i = 0, ncme2 - 1 do $
      for n = 0, nbin2 - 1 do $
         if (lmass2 (i) gt binval2 (n) and lmass2 (i) le binval2 (n+1)) then $
            bin2 (n) = bin2 (n) + 1

   for i = 0, ncme3 - 1 do $
      for n = 0, nbin3 - 1 do $
         if (lmass3 (i) gt binval3 (n) and lmass3 (i) le binval3 (n+1)) then $
            bin3 (n) = bin3 (n) + 1

   dxmin = 13.0
   dxmax = 17.0

END $
ELSE $						; x linear.
BEGIN
   xmin = 0.0
   xmax = 2.0e+16
   xinc = 1.0e+15

   nbin1 = ((xmax - xmin) / xinc) + 2
   bin1    = intarr (nbin1 + 1)
   binval1 = fltarr (nbin1 + 1)

   nbin2 = ((xmax - xmin) / xinc) + 2
   bin2    = intarr (nbin2 + 1)
   binval2 = fltarr (nbin2 + 1)

   nbin3 = ((xmax - xmin) / xinc) + 2
   bin3    = intarr (nbin3 + 1)
   binval3 = fltarr (nbin3 + 1)

   bin0 = xmin
   for i = 0, nbin1 do $
      begin
	 binval1 (i) = bin0 + i * xinc
      end

   bin0 = xmin
   for i = 0, nbin2 do $
      begin
	 binval2 (i) = bin0 + i * xinc
      end

   bin0 = xmin
   for i = 0, nbin3 do $
      begin
	 binval3 (i) = bin0 + i * xinc
      end

   ; Fill bins.

   for i = 0, ncme1 - 1 do $
      for n = 0, nbin1 - 1 do $
         if (mass1 (i) gt binval1 (n) and mass1 (i) le binval1 (n+1)) then $
            bin1 (n) = bin1 (n) + 1

   for i = 0, ncme2 - 1 do $
      for n = 0, nbin2 - 1 do $
         if (mass2 (i) gt binval2 (n) and mass2 (i) le binval2 (n+1)) then $
            bin2 (n) = bin2 (n) + 1

   for i = 0, ncme3 - 1 do $
      for n = 0, nbin3 - 1 do $
         if (mass3 (i) gt binval3 (n) and mass3 (i) le binval3 (n+1)) then $
            bin3 (n) = bin3 (n) + 1

   dxmin = 1.0e+13
   dxmax = 2.0e+16 

END

;for i = 0, nbin1 do $
;   print, 'bin1: ', i, '   binval1: ', binval1 (i), '   binnum: ', bin1 (i)  

sum1 = 0 
sum2 = 0 
sum3 = 0 
for i = 0, nbin1 do sum1 = sum1 + bin1 (i)
;print, 'ncme1: ', ncme1, '   sum1 (histogram): ', sum1

; Plot histogram.

xval1 = binval1 + xinc / 2.0
sxval1 = size (xval1)
nxval1 = sxval1 (1)

xval2 = binval2 + xinc / 2.0
sxval2 = size (xval2)
nxval2 = sxval2 (1)

xval3 = binval3 + xinc / 2.0
sxval3 = size (xval3)
nxval3 = sxval3 (1)

;plab = ' CME Mass Histogram   SMM C/P'

;!P.REGION = [0.10, 0.65, 0.90, 0.90]
;!P.REGION = [0.10, 0.40, 0.90, 0.65]
;!P.REGION = [0.10, 0.15, 0.90, 0.40]
;print, '!P.REGION: ', !P.REGION

!P.MULTI = [0, 0, 3, 0, 0]			; Do 3 plots vertically

IF (xtype eq 0 and ytype eq 0) THEN $		; x linear, y linear.
   BEGIN
   xlab = 'MASS [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval1 = (bin1 / float (ncme1)) * 100.0
   ymin = 0.0
   ymax = max (yval1) 
;!P.REGION = [0.10, 0.65, 0.90, 0.90]
   PLOT, xval1, yval1, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab1, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
;	 POSITION = [0.10, 0.00, 0.9, 0.50], $
         XTHICK = 5.0, YTHICK = 5.0, $
;	 XTICKLEN = 0.02, YTICKLEN = 0.02, $
         PSYM   = 10    			; Histogram plot mode.

   for i = 0, nxval1 - 1 do $
   begin
      if (yval1 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval1 (i) - xinc/2, ybmin, xval1 (i) + xinc/2, yval1 (i), 45
   endfor

   OPLOT, xval1, yval1, PSYM = 10

   xyouts, massave1, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.50
   plots, [massave1, massave1], [ymin, ymax], LINESTYLE = 0

   yval2 = (bin2 / float (ncme2)) * 100.0
   ymin = 0.0
   ymax = max (yval2) 
;!P.REGION = [0.10, 0.40, 0.90, 0.65]
   PLOT, xval2, yval2, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab2, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
;	 POSITION = [0.10, 0.00, 0.9, 0.50], $
         XTHICK = 5.0, YTHICK = 5.0, $
;	 XTICKLEN = 0.02, YTICKLEN = 0.02, $
         PSYM   = 10    			; Histogram plot mode.

   for i = 0, nxval2 - 1 do $
   begin
      if (yval2 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval2 (i) - xinc/2, ybmin, xval2 (i) + xinc/2, yval2 (i), 45
   endfor

   OPLOT, xval2, yval2, PSYM = 10

   xyouts, massave2, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.50
   plots, [massave2, massave2], [ymin, ymax], LINESTYLE = 0

   yval3 = (bin3 / float (ncme3)) * 100.0
   ymin = 0.0
   ymax = max (yval3) 
;!P.REGION = [0.10, 0.15, 0.90, 0.40]
   PLOT, xval3, yval3, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab3, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
;	 POSITION = [0.10, 0.00, 0.9, 0.50], $
         XTHICK = 5.0, YTHICK = 5.0, $
;	 XTICKLEN = 0.02, YTICKLEN = 0.02, $
         PSYM   = 10    			; Histogram plot mode.

   for i = 0, nxval3 - 1 do $
   begin
      if (yval3 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval3 (i) - xinc/2, ybmin, xval3 (i) + xinc/2, yval3 (i), 45
   endfor

   OPLOT, xval3, yval3, PSYM = 10

   xyouts, massave3, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.50
   plots, [massave3, massave3], [ymin, ymax], LINESTYLE = 0

   END $

ELSE $
IF (xtype eq 1 and ytype eq 0) THEN $		; x log, y linear.

   BEGIN
   xlab = 'log (MASS) [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval1 = (bin1 / float (ncme1)) * 100.0
   ymin = 0.0
   ymax = max (yval1) 
;!P.REGION = [0.10, 0.65, 0.90, 0.90]
   PLOT, xval1, yval1, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab1, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10    			; Histogram plot mode.
;	 /XTYPE         			; X axis logarithmic.

   for i = 0, nxval1 - 1 do $
   begin
      if (yval1 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval1 (i) - xinc/2, ybmin, xval1 (i) + xinc/2, yval1 (i), 45
   endfor

   OPLOT, xval1, yval1, PSYM = 10

   xyouts, lmassave1, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.50
   plots, [lmassave1, lmassave1], [ymin, ymax], LINESTYLE = 0

   yval2 = (bin2 / float (ncme2)) * 100.0
   ymin = 0.0
   ymax = max (yval2) 
;!P.REGION = [0.10, 0.40, 0.90, 0.65]
   PLOT, xval2, yval2, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab2, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10    			; Histogram plot mode.
;	 /XTYPE         			; X axis logarithmic.

   for i = 0, nxval2 - 1 do $
   begin
      if (yval2 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval2 (i) - xinc/2, ybmin, xval2 (i) + xinc/2, yval2 (i), 45
   endfor

   OPLOT, xval2, yval2, PSYM = 10

   xyouts, lmassave2, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.50
   plots, [lmassave2, lmassave2], [ymin, ymax], LINESTYLE = 0

   yval3 = (bin3 / float (ncme3)) * 100.0
   ymin = 0.0
   ymax = max (yval3) 
;!P.REGION = [0.10, 0.15, 0.90, 0.40]
   PLOT, xval3, yval3, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab3, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10    			; Histogram plot mode.
;	 /XTYPE         			; X axis logarithmic.

   for i = 0, nxval3 - 1 do $
   begin
      if (yval3 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval3 (i) - xinc/2, ybmin, xval3 (i) + xinc/2, yval3 (i), 45
   endfor

   OPLOT, xval3, yval3, PSYM = 10

   xyouts, lmassave3, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.50
   plots, [lmassave3, lmassave3], [ymin, ymax], LINESTYLE = 0

   END $

ELSE $
IF (xtype eq 0 and ytype eq 1) THEN $		; x linear, y log.

   BEGIN
   xlab = 'MASS [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval1 = (bin1 / float (ncme1) ) * 100.0
   ymin = 0.0
   ymax = max (yval1) 
;!P.REGION = [0.10, 0.65, 0.90, 0.90]
   PLOT, xval1, yval1, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab1, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval1 - 1 do $
   begin
      if (yval1 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval1 (i) - xinc/2, ybmin, xval1 (i) + xinc/2, yval1 (i), 45
   endfor

   OPLOT, xval1, yval1, PSYM = 10

   xyouts, massave1, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.50
   plots, [massave1, massave1], [ymin, ymax], LINESTYLE = 0

   yval2 = (bin2 / float (ncme2) ) * 100.0
   ymin = 0.0
   ymax = max (yval2) 
;!P.REGION = [0.10, 0.40, 0.90, 0.65]
   PLOT, xval2, yval2, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab2, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval2 - 1 do $
   begin
      if (yval2 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval2 (i) - xinc/2, ybmin, xval2 (i) + xinc/2, yval2 (i), 45
   endfor

   OPLOT, xval2, yval2, PSYM = 10

   xyouts, massave2, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.50
   plots, [massave2, massave2], [ymin, ymax], LINESTYLE = 0

   yval3 = (bin3 / float (ncme3) ) * 100.0
   ymin = 0.0
   ymax = max (yval3) 
;!P.REGION = [0.10, 0.15, 0.90, 0.40]
   PLOT, xval3, yval3, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab3, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval3 - 1 do $
   begin
      if (yval3 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval3 (i) - xinc/2, ybmin, xval3 (i) + xinc/2, yval3 (i), 45
   endfor

   OPLOT, xval3, yval3, PSYM = 10

   xyouts, massave3, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.50
   plots, [massave3, massave3], [ymin, ymax], LINESTYLE = 0

   END $

ELSE $
IF (xtype eq 1 and ytype eq 1) THEN $		; x log, y log.

   BEGIN
   xlab = 'log (MASS) [grams]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval1 = (bin1 / float (ncme1)) * 100.0
   ymin = 1.0e-1 
   ymax = max (yval1) 
;!P.REGION = [0.10, 0.65, 0.90, 0.90]
   PLOT, xval1, yval1, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab1, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
;	 /XTYPE,        $			; X axis logarithmic.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval1 - 1 do $
   begin
      if (yval1 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval1 (i) - xinc/2, ybmin, xval1 (i) + xinc/2, yval1 (i), 45
   endfor

   OPLOT, xval1, yval1, PSYM = 10

   xyouts, lmassave1, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.50
   plots, [lmassave1, lmassave1], [ymin, ymax], LINESTYLE = 0

   yval2 = (bin2 / float (ncme2)) * 100.0
   ymin = 1.0e-1 
   ymax = max (yval2) 
;!P.REGION = [0.10, 0.40, 0.90, 0.65]
   PLOT, xval2, yval2, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab2, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
;	 /XTYPE,        $			; X axis logarithmic.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval2 - 1 do $
   begin
      if (yval2 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval2 (i) - xinc/2, ybmin, xval2 (i) + xinc/2, yval2 (i), 45
   endfor

   OPLOT, xval2, yval2, PSYM = 10

   xyouts, lmassave2, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.50
   plots, [lmassave2, lmassave2], [ymin, ymax], LINESTYLE = 0

   yval3 = (bin3 / float (ncme3)) * 100.0
   ymin = 1.0e-1 
   ymax = max (yval3) 
;!P.REGION = [0.10, 0.15, 0.90, 0.40]
   PLOT, xval3, yval3, $
	 /YNOZERO, XRANGE = [dxmin, dxmax], YRANGE = [ymin, ymax], $
	 CHARSIZE = 1.0, $
         TITLE  = yearlab3, $
;         XTITLE = xlab, $
;         YTITLE = ylab, $
         XTHICK = 5.0, YTHICK = 5.0, $
         PSYM   = 10,   $			; Histogram plot mode.
;	 /XTYPE,        $			; X axis logarithmic.
	 /YTYPE         			; Y axis logarithmic.

   for i = 0, nxval3 - 1 do $
   begin
      if (yval3 (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval3 (i) - xinc/2, ybmin, xval3 (i) + xinc/2, yval3 (i), 45
   endfor

   OPLOT, xval3, yval3, PSYM = 10

   xyouts, lmassave3, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.50
   plots, [lmassave3, lmassave3], [ymin, ymax], LINESTYLE = 0

   END

;OPLOT, xval1, yval1, PSYM   = 5		; overplot with triangles.

;for i = 0, nbin do $
;   box, xval (i), !Y.CRANGE (0), xval (i) + xval (i+1), yval (i), 45 

; Write year label on plot.

;if (xtype eq 0 and ytype eq 0) then $
;   xyouts, 0.59, 0.85, yearlab, /NORMAL, CHARSIZE = 1.0 $
;else $
;   xyouts, 0.21, 0.85, yearlab, /NORMAL, CHARSIZE = 1.0

; Write # CMEs measured on plot.

;cmelab = string (ncme, FORMAT = '(I3)') + ' CMEs measured'
;if (xtype eq 0 and ytype eq 0) then $
;   xyouts, 0.59, 0.75, cmelab, /NORMAL, CHARSIZE = 0.8 $
;else $
;   xyouts, 0.21, 0.75, cmelab, /NORMAL, CHARSIZE = 0.8

; Write average mass on plot.

;avelab = 'mean = ' + string (massave, FORMAT = '(E8.2)') + ' g'
;print, 'massave: ', massave
;if (xtype eq 0 and ytype eq 0) then $
;   xyouts, 0.59, 0.70, avelab, /NORMAL, CHARSIZE = 0.8 $
;else $
;   xyouts, 0.21, 0.70, avelab, /NORMAL, CHARSIZE = 0.8

;FOR i = 0, nbin DO $
;   print, i, '   binval: ', binval (i), $
;	     '   xval: ', xval (i), '   yval: ', yval (i)

;!P.MULTI = [0, 0, 1, 0, 0]			; Do 1 plot vertically
end
 
; --- Histogram fill procedure -------------------------------------------------

pro box, x0, y0, x1, y1, line
;   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0],  ORIENTATION = line 
   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0], COLOR = 191
end
