pro ml1

;+
; Scatter Plot: mass versus latitude
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   6 September 1994
;-

@cpmass.common

rdcpmass

poslat = abs (lat)

sxmass = size (xmass)
slat   = size (lat)

nval   = sxmass (1)
nxmass = sxmass (1)
nlat   = slat  (1)
;print, 'nxmass: ', nxmass, '   nlat: ', nlat

;for i = 0, nxmass - 1  do $
;   print, 'i: ', i, '   xmass: ', xmass (i), '   lat: ', lat (i)

xm = xmass * 1000.0				; mass [grams]
lxmass = alog10 (xmass * 1000.0)		; log (mass) [grams].
SET_PLOT, 'PS'
!P.FONT = 0
DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'ml1.ps', $
	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

A = FINDGEN(16) * (!PI*2/16.)
USERSYM, COS(A), SIN(A), /FILL

;plot, poslat, lxmass

ymin = 13.5
ymax = 17.0

PLOT, poslat, lxmass, /YNOZ, PSYM = 2, $
     YRANGE = [ymin, ymax], $
     TITLE='Scatter Plot: Mass vs Latitude', XTITLE='Latitude [degrees]', $
     YTITLE = 'Log (Mass)   [grams]', $
     XTHICK = 5.0, YTHICK = 5.0, $
     XTICKLEN = -0.02, YTICKLEN = -0.02

r = CORRELATE (poslat, lxmass)
sr = string (r, FORMAT = '(F6.3)')

print, r

tmass1 = 0
tmass2 = 0
tmass3 = 0

nmass1 = 0
nmass2 = 0
nmass3 = 0

mass1 = fltarr (nval)
mass2 = fltarr (nval)
mass3 = fltarr (nval)

i1 = 0
i2 = 0
i3 = 0

for i = 0, nval - 1 do $
begin
   if (poslat (i) le 20) then $
   begin
      nmass1 = nmass1 + 1
      tmass1 = tmass1 + xm (i) 
      mass1 (i1) = xm (i)
      i1 = i1 + 1
   endif

   if (poslat (i) gt 20) and poslat (i) le 40 then $
   begin
      nmass2 = nmass2 + 1
      tmass2 = tmass2 + xm (i)
      mass2 (i2) = xm (i)
      i2 = i2 + 1
   endif

   if (poslat (i) gt 40) then $
   begin
      nmass3 = nmass3 + 1
      tmass3 = tmass3 + xm (i)
      mass3 (i3) = xm (i)
      i3 = i3 + 1
   endif
endfor

m1 = mass1 (0: nmass1 - 1)
m2 = mass2 (0: nmass2 - 1)
m3 = mass3 (0: nmass3 - 1)

num = nmass1 + nmass2 + nmass3

avg1 = tmass1 / nmass1
avg2 = tmass2 / nmass2 
avg3 = tmass3 / nmass3
avg = (tmass1 + tmass2 + tmass3) / num

savg1 = string (avg1, FORMAT = '(E8.2)')
savg2 = string (avg2, FORMAT = '(E8.2)')
savg3 = string (avg3, FORMAT = '(E8.2)')

print, '<Mass> ( 0 < lat < 20) = ', avg1
print, '<Mass> (20 < lat < 40) = ', avg2
print, '<Mass> (40 < lat < 90) = ', avg3   

print, 'num: ', num, '   mass ave: ', avg

xyouts, 0.21, 0.86, '1980, 1984-1988', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.78, 'corr. coeff = ' + sr, /NORM, CHARSIZE = 0.8
xyouts, 0.50, 0.86, '<Mass> ( 0<lat<20) = ' + savg1, /NORM, CHARSIZE = 0.8
xyouts, 0.50, 0.82, '<Mass> (20<lat<40) = ' + savg2, /NORM, CHARSIZE = 0.8
xyouts, 0.50, 0.78, '<Mass> (40<lat<90) = ' + savg3, /NORM, CHARSIZE = 0.8

syearlab = '1980, 1984-1988'			; year label.
xtype = 1					; log   : mass.
ytype = 0					; linear: # in bin.

mhp0, m1, syearlab, xtype, ytype
xyouts, 0.21, 0.8, ' 0 < lat < 20 deg.', /NORMAL
mhp0, m2, syearlab, xtype, ytype
xyouts, 0.21, 0.8, '20 < lat < 40 deg.', /NORMAL
mhp0, m3, syearlab, xtype, ytype
xyouts, 0.21, 0.8, '40 < lat < 90 deg.', /NORMAL

DEVICE, /CLOSE
SET_PLOT, 'X'

end
