pro mswh3

;+
; Generate SMM C/P Excess Mass, Speed & Width histograms for
; the period 1980, 1984-1989.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   8 April 1996
;-

@cpmassa.common

rdcpmass2				; Read SMM C/P mass data.

SET_PLOT, 'PS'
!P.FONT = 0
DEVICE, /HELVETICA, /BOLD, FONT_SIZE = 18, FILENAME = 'mswh3.ps', $
XSIZE = 16.51, YOFFSET = 2.54, YSIZE = 22.86	; 6.5" wide, 1" bottom , 9" high

!P.MULTI = [0, 0, 3, 0, 0]			; Do 3 plots vertically

label = ''
d = ''
s80 = size (xm80)
s84 = size (xm84)
s85 = size (xm85)
s86 = size (xm86)
s87 = size (xm87)
s88 = size (xm88)
s89 = size (xm89)
scme = size (xmass)

n80 = s80 (1)
n84 = s84 (1)
n85 = s85 (1)
n86 = s86 (1)
n87 = s87 (1)
n88 = s88 (1)
n89 = s89 (1)
ncme = scme (1)

print, 'n80:  ', n80
print, 'n84:  ', n84
print, 'n85:  ', n85
print, 'n86:  ', n86
print, 'n87:  ', n87
print, 'n88:  ', n88
print, 'ncme: ', ncme

;--- Convert to grams.

xm80 = xm80 * 1.0e+3
xm84 = xm84 * 1.0e+3
xm85 = xm85 * 1.0e+3
xm86 = xm86 * 1.0e+3
xm87 = xm87 * 1.0e+3
xm88 = xm88 * 1.0e+3
xm89 = xm89 * 1.0e+3
xmass = xmass * 1.0e+3

pwidth = fltarr (ncme)
pspeed = fltarr (ncme)

ip = 0
for i = 0, ncme-1 do		$
begin
   if (width (i) > 0.0) then	$
   begin
      pwidth (ip) = width (i)
      ip = ip + 1
   end
end
nwid = ip

ip = 0
for i = 0, ncme-1 do		$
begin
   if (speed (i) > 0.0) then	$
   begin
      pspeed (ip) = speed (i)
      ip = ip + 1
   end
end
nspd = ip

pwidth = pwidth (0 : nwid - 1)
pspeed = pspeed (0 : nspd - 1)
print, 'ncme: ', ncme, ' nspd: ', nspd, ' nwid: ', nwid

;for i = 0, nwid-1 do print, i, pwidth (i)
;for i = 0, nspd-1 do print, i, pspeed (i)

;--- Create a single array of mass values for years 1980, 1984-1988.

xmmin = min (xmass)
xmmax = max (xmass)
xmdif = xmmax - xmmin
binsiz = xmdif / 20.0

;print, 'min (xmass): ', xmmin, '   max (xmass): ', xmmax
;print, 'xmdif: ', xmdif, '   binsiz: ', binsiz

; Define year variables

year80 = 1980
year84 = 1984
year85 = 1985
year86 = 1986
year87 = 1987
year88 = 1988
year89 = 1989

syear80 = string (year80, FORMAT = '(I4)')
syear84 = string (year84, FORMAT = '(I4)')
syear85 = string (year85, FORMAT = '(I4)')
syear86 = string (year86, FORMAT = '(I4)')
syear87 = string (year87, FORMAT = '(I4)')
syear88 = string (year88, FORMAT = '(I4)')
syear89 = string (year89, FORMAT = '(I4)')
syear   = '1980, 1984-1989'

; Plot histograms.

syearlab = '1980, 1984-1989'

wlab = 'Width'
slab = 'Speed'
mlab = 'Mass'

wtit = ''
stit = ''
mtit = ''
;wtit = 'Width Histogram' + '   ' + syearlab
;stit = 'Speed Histogram' + '   ' + syearlab
;mtit = 'Mass Histogram ' + '   ' + syearlab

sdeg = 'degrees'
skms = 'km/s'
sgrams = 'grams'

;-------------------------------------------------------------------------------
;pro cmehp0, quan, quanname, quanunit, plotlab, yearlab, xinc, xtype, ytype
;+
; Plot SMM C/P CME Histograms.
; quan: fltarr of values.
; quanname: name of quantity (e.g., MASS, KINETIC ENERGY, etc.)
; quanunit: units of quan array.
; plotlab: plot title.
; yearlab: string label for plot describing years covered.
; xinc: x bin size.
; xtype: 0 = linear, 1 = log
; ytype: 0 = linear, 1 = log
;
;-------------------------------------------------------------------------------

xinc = 5.0
xtype = 0
ytype = 0
cmehp00, pwidth, wlab, sdeg,   wtit, syearlab, xinc, xtype, ytype

xinc = 100.0
xtype = 0
ytype = 0
;cmehp0, pspeed, slab, skms,   stit, syearlab, xinc, xtype, ytype
cmehp00, pspeed, slab, skms,   stit, syearlab, xinc, xtype, ytype

xinc = 0.1
xtype = 1
ytype = 0
;cmehp0, xmass,  mlab, sgrams, mtit, syearlab, xinc, xtype, ytype
cmehp00, xmass,  mlab, sgrams, mtit, syearlab, xinc, xtype, ytype

;whp0, width, wlab, syear, xtype, ytype
;shp0, speed, slab, syear, xtype, ytype
;mhp0, xmass, mlab, syear, xtype, ytype

!P.MULTI = [0, 0, 1, 0, 0]			; Do 1 plot vertically

DEVICE, /CLOSE					; flush buffer to PS file.
SET_PLOT, 'X'					; reset output to X windows.

end
