pro mt

;+
; Generate SMM C/P plots:Excess Mass vs. Time for the period 1980, 1984-1989.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   29 June 1995
;  8 Jan 96 : cme89.tab now includes Jupiter calibration correction.
; 11 Mar 96 : cme88.tab now includes vidicon sensitivity calibration adjustment.
;-

@cpmassa.common

rdcpmass2				; Read SMM C/P mass data.

xmass = xmass * 1000.0			; convert from kg to grams.
xm80  = xm80  * 1000.0
xm84  = xm84  * 1000.0
xm85  = xm85  * 1000.0
xm86  = xm86  * 1000.0
xm87  = xm87  * 1000.0
xm88  = xm88  * 1000.0
xm89  = xm89  * 1000.0

;xm89  = xm89  * 1000.0 * 2.0		; cme89.tab does NOT include
					; Jupiter calibration correction.

xm = [xm80, xm84, xm85, xm86, xm87, xm88, xm89]

sizvec = size (xm80)
n80 = sizvec (1)
sizvec = size (xm84)
n84 = sizvec (1)
sizvec = size (xm85)
n85 = sizvec (1)
sizvec = size (xm86)
n86 = sizvec (1)
sizvec = size (xm87)
n87 = sizvec (1)
sizvec = size (xm88)
n88 = sizvec (1)
sizvec = size (xm89)
n89 = sizvec (1)

print, 'n80/84/85/86/87/88/89: ', n80, n84, n85, n86, n87, n88, n89

sum = 0.0
for i = 0, n80 - 1 do sum  = sum + xm80 (i)
am80 = sum / n80

sum = 0.0
for i = 0, n84 - 1 do sum  = sum + xm84 (i)
am84 = sum / n84

sum = 0.0
for i = 0, n85 - 1 do sum  = sum + xm85 (i)
am85 = sum / n85

sum = 0.0
for i = 0, n86 - 1 do sum  = sum + xm86 (i)
am86 = sum / n86

sum = 0.0
for i = 0, n87 - 1 do sum  = sum + xm87 (i)
am87 = sum / n87

sum = 0.0
for i = 0, n88 - 1 do sum  = sum + xm88 (i)
am88 = sum / n88

n88a = n88 / 8
sum = 0.0
for i = 0, n88a - 1 do sum  = sum + xm88 (i)
am88a = sum / n88a

sum = 0.0
for i = n88a, 2*n88a - 1 do sum  = sum + xm88 (i)
am88b = sum / n88a

sum = 0.0
for i = 2*n88a, 3*n88a - 1 do sum  = sum + xm88 (i)
am88c = sum / n88a

sum = 0.0
for i = 3*n88a, 4*n88a - 1 do sum  = sum + xm88 (i)
am88d = sum / n88a

sum = 0.0
for i = 4*n88a, 5*n88a - 1 do sum  = sum + xm88 (i)
am88e = sum / n88a

sum = 0.0
for i = 5*n88a, 6*n88a - 1 do sum  = sum + xm88 (i)
am88f = sum / n88a

sum = 0.0
for i = 6*n88a, 7*n88a - 1 do sum  = sum + xm88 (i)
am88g = sum / n88a

sum = 0.0
for i = 7*n88a, 8*n88a - 1 do sum  = sum + xm88 (i)
am88h = sum / n88a

print, 'ave [xm88] parts 1-8: ', am88a, am88b, am88c, am88d, $
				 am88e, am88f, am88g, am88h

sum = 0.0
for i = 0, n89 - 1 do sum  = sum + xm89 (i)
am89 = sum / n89

n89a = n89 / 8
sum = 0.0
for i = 0, n89a - 1 do sum  = sum + xm89 (i)
am89a = sum / n89a

sum = 0.0
for i = n89a, 2*n89a - 1 do sum  = sum + xm89 (i)
am89b = sum / n89a

sum = 0.0
for i = 2*n89a, 3*n89a - 1 do sum  = sum + xm89 (i)
am89c = sum / n89a

sum = 0.0
for i = 3*n89a, 4*n89a - 1 do sum  = sum + xm89 (i)
am89d = sum / n89a

sum = 0.0
for i = 4*n89a, 5*n89a - 1 do sum  = sum + xm89 (i)
am89e = sum / n89a

sum = 0.0
for i = 5*n89a, 6*n89a - 1 do sum  = sum + xm89 (i)
am89f = sum / n89a

sum = 0.0
for i = 6*n89a, 7*n89a - 1 do sum  = sum + xm89 (i)
am89g = sum / n89a

sum = 0.0
for i = 7*n89a, 8*n89a - 1 do sum  = sum + xm89 (i)
am89h = sum / n89a

print, 'ave [xm89] parts 1-8: ', am89a, am89b, am89c, am89d, $
				 am89e, am89f, am89g, am89h

amt = [am80, am84, am85, am86, am87, am88, am89]

print, 'am80: ', am80
print, 'am84: ', am84
print, 'am85: ', am85
print, 'am86: ', am86
print, 'am87: ', am87
print, 'am88: ', am88
print, 'am89: ', am89

;SET_PLOT, 'X'					; reset output to X windows.
SET_PLOT, 'PS'
!P.FONT = 0
DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'mt.ps', $
        XOFFSET = 2.54,  XSIZE = 16.51, $	; 1" left margin, 6.5" wide
	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" high
!P.MULTI = [0, 0, 0, 0, 0]

xmmin = min (xmass)
xmmax = max (xmass)

;print, 'min (xmass): ', xmmin, '   max (xmass): ', xmmax
;print, 'xmdif: ', xmdif, '   binsiz: ', binsiz

; Define year variables

year80 = 1980
year84 = 1984
year85 = 1985
year86 = 1986
year87 = 1987
year88 = 1988
year89 = 1989

syear80 = string (year80, FORMAT = '(I4)')
syear84 = string (year84, FORMAT = '(I4)')
syear85 = string (year85, FORMAT = '(I4)')
syear86 = string (year86, FORMAT = '(I4)')
syear87 = string (year87, FORMAT = '(I4)')
syear88 = string (year88, FORMAT = '(I4)')
syear89 = string (year89, FORMAT = '(I4)')

time = [1980, 1984, 1985, 1986, 1987, 1988, 1989] ;

; --- Define circle symbol for plotting.

a = FINDGEN (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), SIN (a), /FILL

; Plot mass vs. Time.

 syearlab = 'Mass vs. Time'

PLOT, time, amt,				$
      PSYM = 8,					$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Time',			$
      XTITLE = 'Y E A R',			$
      YTITLE = 'M A S S   [g r a m s]'
OPLOT, time, amt, LINESTYLE = 0

PLOT, doy80, xm80, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1980]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

PLOT, doy84, xm84, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1984]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

PLOT, doy85, xm85, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1985]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

PLOT, doy86, xm86, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1986]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

PLOT, doy87, xm87, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1987]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

PLOT, doy88, xm88, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1988]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

PLOT, doy89, xm89, PSYM = 2,			$
      YRANGE = [1.E+13, 1.E+17],		$
      /YNOZERO, /YTYPE,				$
      CHARSIZE = 0.8,				$
      TITLE  = 'Mass vs. Day-Of-Year [1989]',	$
      XTITLE = 'D A Y   O F   Y E A R',		$
      YTITLE = 'M A S S   [g r a m s]'

d80 = fltarr (n80)
for i = 0, n80 - 1 do d80 (i) = 1980 + doy80 (i) / 366.0

d84 = fltarr (n84)
for i = 0, n84 - 1 do d84 (i) = 1984 + doy84 (i) / 366.0

d85 = fltarr (n85)
for i = 0, n85 - 1 do d85 (i) = 1985 + doy85 (i) / 365.0

d86 = fltarr (n86)
for i = 0, n86 - 1 do d86 (i) = 1986 + doy86 (i) / 365.0

d87 = fltarr (n87)
for i = 0, n87 - 1 do d87 (i) = 1987 + doy87 (i) / 365.0

d88 = fltarr (n88)
for i = 0, n88 - 1 do d88 (i) = 1988 + doy88 (i) / 366.0

d89 = fltarr (n89)
for i = 0, n89 - 1 do d89 (i) = 1989 + doy89 (i) / 365.0

yd = [d80, d84, d85, d86, d87, d88, d89]

print, 'size (yd): ', size (yd)
print, 'size (xm): ', size (xm)

a = findgen (16) * (!PI * 2 / 16.0)
USERSYM, COS (a), SIN (a)
;PLOT, yd, xm, PSYM = 8,	$
PLOT, yd, xm, PSYM = 2,					$
      YRANGE = [1.E+13, 1.E+17],			$
      /YNOZERO, /YTYPE,					$
      CHARSIZE = 0.8,					$
      TITLE  = 'Mass vs. Time [1980, 1984-1989]',	$
      XTITLE = 'Y E A R',				$
      YTITLE = 'M A S S   [g r a m s]'

;DEVICE, /CLOSE					; flush buffer to PS file.
SET_PLOT, 'X'					; reset output to X windows.

end
