pro mw

;+
; Scatter Plot: mass versus width
; SMM C/P CMEs.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   1 April 1996 
;-

@cpmassa.common

rdcpmass2

xmass = xmass * 1.e+3			; mass [grams]
sxmass = size (xmass)
swidth = size (width)

nval   = sxmass (1)
nxmass = sxmass (1)
nwidth = swidth  (1)
;print, 'nxmass: ', nxmass, '   nwidth: ', nwidth

;for i = 0, nxmass - 1  do $
;   print, 'i: ', i, '   xmass: ', xmass (i), '   width: ', width (i)

;--- Assemble only those masses for which width > 0.0

ip = 0
pwidth = fltarr (nval)
pxmass = fltarr (nval)
for i = 0, nxmass - 1  do $
begin
   if (width (i) gt 0.0) then	$
   begin
      pwidth (ip) = width (i)
      pxmass (ip) = xmass (i)
      ip = ip + 1
   end				
end

nval = ip

;--- Resize arrays.

pwidth = pwidth (0 : nval - 1)
pxmass = pxmass (0 : nval - 1)

;for i = 0, nval - 1  do $
;   print, 'i: ', i, '   pxmass: ', pxmass (i), '   pwidth: ', pwidth (i)

lpxmass = alog10 (pxmass)			; log (mass) [grams].
lpwidth = alog10 (pwidth)			; log (width) [degrees].

SET_PLOT, 'PS'
!P.FONT = 0
;DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'mw.ps', $
;	XOFFSET = 1.27, XSIZE = 17.50, $	; 1" left margin, 7.0" W
;	YOFFSET = 12.70, YSIZE = 12.70		; 5" bottom margin, 5" H

DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'mw.ps', $
	XOFFSET = 2.54,  XSIZE = 17.78, $	; 1.0" left margin, 7.0" wide
	YOFFSET = 5.08, YSIZE = 19.05		; 2.0" bottom margin, 7.5" high
!P.MULTI = [0, 0, 0, 0, 0]

A = FINDGEN(16) * (!PI*2/16.)
;USERSYM, COS(A), SIN(A), /FILL
USERSYM, COS(A), SIN(A)

tmass1 = 0
tmass2 = 0
tmass3 = 0
tmass4 = 0

nmass1 = 0
nmass2 = 0
nmass3 = 0
nmass4 = 0

mass1 = fltarr (nval)
mass2 = fltarr (nval)
mass3 = fltarr (nval)
mass4 = fltarr (nval)

i1 = 0
i2 = 0
i3 = 0
i4 = 0

for i = 0, nval - 1 do $
begin
   if (pwidth (i) le 30) then $
   begin
      nmass1 = nmass1 + 1
      tmass1 = tmass1 + pxmass (i) 
      mass1 (i1) = pxmass (i)
      i1 = i1 + 1
   endif

   if (pwidth (i) gt 30) and pwidth (i) le 60 then $
   begin
      nmass2 = nmass2 + 1
      tmass2 = tmass2 + pxmass (i)
      mass2 (i2) = pxmass (i)
      i2 = i2 + 1
   endif

   if (pwidth (i) gt 60 and pwidth (i) le 90) then $
   begin
      nmass3 = nmass3 + 1
      tmass3 = tmass3 + pxmass (i)
      mass3 (i3) = pxmass (i)
      i3 = i3 + 1
   endif

   if (pwidth (i) gt 90) then $
   begin
      nmass4 = nmass4 + 1
      tmass4 = tmass4 + pxmass (i)
      mass4 (i4) = pxmass (i)
      i4 = i4 + 1
   endif
endfor

m1 = mass1 (0: nmass1 - 1)
m2 = mass2 (0: nmass2 - 1)
m3 = mass3 (0: nmass3 - 1)
m4 = mass4 (0: nmass4 - 1)

num = nmass1 + nmass2 + nmass3 + nmass4

avg1 = tmass1 / nmass1
avg2 = tmass2 / nmass2 
avg3 = tmass3 / nmass3
avg4 = tmass4 / nmass4
avg = (tmass1 + tmass2 + tmass3 + tmass4) / num

savg1 = string (avg1, FORMAT = '(E8.2)')
savg2 = string (avg2, FORMAT = '(E8.2)')
savg3 = string (avg3, FORMAT = '(E8.2)')
savg4 = string (avg4, FORMAT = '(E8.2)')

print, '<Mass> ( 0 < width < 30) = ', avg1
print, '<Mass> (30 < width < 60) = ', avg2
print, '<Mass> (60 < width < 90) = ', avg3
print, '<Mass> (     width > 90) = ', avg4

print, 'num: ', num, '   mass ave: ', avg
snum = string (num, FORMAT = '(I4)')
savg = string (avg, FORMAT = '(E8.2)')

r = CORRELATE (lpwidth, lpxmass)
;r = CORRELATE (pwidth, pxmass)
sr = string (r, FORMAT = '(F6.3)')

print, 'correlation: width vs. mass: ', r

;plot, pwidth, lpxmass

ymin = 13.5
ymax = 17.0

;PLOT, pwidth, lpxmass, /YNOZ, PSYM = 4,		$
PLOT, pwidth, pxmass, /YNOZ, PSYM = 4,		$
;     YRANGE = [ymin, ymax],			$
     TITLE = 'Mass vs Width',	$
     XTITLE = 'Width [degrees]',		$
     YTITLE = 'Log (Mass)   [grams]',		$
     XTHICK = 5.0, YTHICK = 5.0,		$
     XTICKLEN = -0.02, YTICKLEN = -0.02

xyouts, 0.21, 0.90, 'SMM C/P   1980, 1984-1989', /NORM, CHARSIZE = 1.0
xyouts, 0.21, 0.87, 'correlation coefficient = ' + sr, /NORM, CHARSIZE = 0.8
xyouts, 0.21, 0.84, snum + ' CMEs measured', /NORM, CHARSIZE = 0.8
;xyouts, 0.21, 0.81, '<mass>=' + savg + ' grams', /NORM, CHARSIZE = 0.8
;xyouts, 0.50, 0.88, '<mass> ( 0 < width < 30) = ' + savg1, /NORM, CHARSIZE = 0.6
;xyouts, 0.50, 0.84, '<mass> (30 < width < 60) = ' + savg2, /NORM, CHARSIZE = 0.6
;xyouts, 0.50, 0.80, '<mass> (60 < width < 90) = ' + savg3, /NORM, CHARSIZE = 0.6
;xyouts, 0.50, 0.76, '<mass> (     width > 90) = ' + savg4, /NORM, CHARSIZE = 0.6

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

syearlab = '1980, 1984-1989'			; year label.
xtype = 1					; log   : mass.
ytype = 0					; linear: # in bin.

;mhp0, m1, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, ' 0 < width < 30 deg.', /NORMAL
;mhp0, m2, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '30 < width < 60 deg.', /NORMAL
;mhp0, m3, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '60 < width < 90 deg.', /NORMAL
;mhp0, m4, syearlab, xtype, ytype
;xyouts, 0.21, 0.80, '     width > 90 deg.', /NORMAL

DEVICE, /CLOSE
SET_PLOT, 'X'

end
