pro mwh

;+
; Generate SMM C/P Excess Mass / Width histograms for period 1980, 1984-1989.
;
; IDL procedure
; Andrew L. Stanger   HAO/NCAR   8 April 1996
;-

@cpmassa.common

rdcpmass2				; Read SMM C/P mass data.

s80 = size (xm80)
s84 = size (xm84)
s85 = size (xm85)
s86 = size (xm86)
s87 = size (xm87)
s88 = size (xm88)
s89 = size (xm89)
sxmass = size (xmass)

n80 = s80 (1)
n84 = s84 (1)
n85 = s85 (1)
n86 = s86 (1)
n87 = s87 (1)
n88 = s88 (1)
n89 = s89 (1)
ncme = sxmass (1)

xmass = xmass * 1000.0			; convert from kg to grams.
xm80  = xm80  * 1000.0
xm84  = xm84  * 1000.0
xm85  = xm85  * 1000.0
xm86  = xm86  * 1000.0
xm87  = xm87  * 1000.0
xm88  = xm88  * 1000.0
xm89  = xm89  * 1000.0

mw80 = fltarr (n80)
mw84 = fltarr (n84)
mw85 = fltarr (n85)
mw86 = fltarr (n86)
mw87 = fltarr (n87)
mw88 = fltarr (n88)
mw89 = fltarr (n89)
mwxx = fltarr (ncme)

SET_PLOT, 'PS'
!P.FONT = 0
DEVICE, /HELVETICA, FONT_SIZE = 18, FILENAME = 'mwh.ps', $
        XOFFSET = 2.54,  XSIZE = 17.78, $	; 1" left margin,   7.0" wide
	YOFFSET = 5.08,  YSIZE = 17.78		; 2" bottom margin, 7.0" high
!P.MULTI = [0, 0, 0, 0, 0]

ip = 0
for i = 0, n80 - 1 do		$
begin
   if (wid80 (i) gt 0) then	$
   begin
      mw80 (ip) = xm80 (i) / wid80 (i)
      ip = ip  + 1
   end
end
nw80 = ip

ip = 0
for i = 0, n84 - 1 do		$
begin
   if (wid84 (i) gt 0) then	$
   begin
      mw84 (ip) = xm84 (i) / wid84 (i)
      ip = ip  + 1
   end
end
nw84 = ip

ip = 0
for i = 0, n85 - 1 do		$
begin
   if (wid85 (i) gt 0) then	$
   begin
      mw85 (ip) = xm85 (i) / wid85 (i)
      ip = ip  + 1
   end
end
nw85 = ip

ip = 0
for i = 0, n86 - 1 do		$
begin
   if (wid86 (i) gt 0) then	$
   begin
      mw86 (ip) = xm86 (i) / wid86 (i)
      ip = ip  + 1
   end
end
nw86 = ip

ip = 0
for i = 0, n87 - 1 do		$
begin
   if (wid87 (i) gt 0) then	$
   begin
      mw87 (ip) = xm87 (i) / wid87 (i)
      ip = ip  + 1
   end
end
nw87 = ip

ip = 0
for i = 0, n88 - 1 do		$
begin
   if (wid88 (i) gt 0) then	$
   begin
      mw88 (ip) = xm88 (i) / wid88 (i)
      ip = ip  + 1
   end
end
nw88 = ip

ip = 0
for i = 0, n89 - 1 do		$
begin
   if (wid89 (i) gt 0) then	$
   begin
      mw89 (ip) = xm89 (i) / wid89 (i)
      ip = ip  + 1
   end
end
nw89 = ip

ip = 0
for i = 0, ncme - 1 do		$
begin
   if (width (i) gt 0) then	$
   begin
      mwxx (ip) = xmass (i) / width (i)
      ip = ip  + 1
   end
end
nw = ip

mw80 = mw80 (0 : nw80 - 1)
mw84 = mw84 (0 : nw84 - 1)
mw85 = mw85 (0 : nw85 - 1)
mw86 = mw86 (0 : nw86 - 1)
mw87 = mw87 (0 : nw87 - 1)
mw88 = mw88 (0 : nw88 - 1)
mw89 = mw89 (0 : nw89 - 1)
mwxx = mwxx (0 : nw - 1)

mwmin = min (xmass)
mwmax = max (xmass)
mwdif = mwmax - mwmin
binsiz = mwdif / 20.0

;print, 'min (xmass): ', xmmin, '   max (xmass): ', xmmax
;print, 'xmdif: ', xmdif, '   binsiz: ', binsiz

; Define year variables

year80 = 1980
year84 = 1984
year85 = 1985
year86 = 1986
year87 = 1987
year88 = 1988
year89 = 1989

syear80 = string (year80, FORMAT = '(I4)')
syear84 = string (year84, FORMAT = '(I4)')
syear85 = string (year85, FORMAT = '(I4)')
syear86 = string (year86, FORMAT = '(I4)')
syear87 = string (year87, FORMAT = '(I4)')
syear88 = string (year88, FORMAT = '(I4)')
syear89 = string (year89, FORMAT = '(I4)')

; Plot mass/width histograms.

 plab = 'SMM C/P   Mass/Width Histogram'
 syearlab = '1980, 1984-1989'

; mwhp0, mw80, plab, syear80,  0, 0
; mwhp0, mw84, plab, syear84,  0, 0
; mwhp0, mw85, plab, syear85,  0, 0
; mwhp0, mw86, plab, syear86,  0, 0
; mwhp0, mw87, plab, syear87,  0, 0
; mwhp0, mw88, plab, syear88,  0, 0
; mwhp0, mw89, plab, syear89,  0, 0

; mwhp0, mw80, plab, syear80,  1, 0
; mwhp0, mw84, plab, syear84,  1, 0
; mwhp0, mw85, plab, syear85,  1, 0
; mwhp0, mw86, plab, syear86,  1, 0
; mwhp0, mw87, plab, syear87,  1, 0
; mwhp0, mw88, plab, syear88,  1, 0
; mwhp0, mw89, plab, syear89,  1, 0

; mwhp0, mw80, plab, syear80,  1, 1
; mwhp0, mw84, plab, syear84,  1, 1
; mwhp0, mw85, plab, syear85,  1, 1
; mwhp0, mw86, plab, syear86,  1, 1
; mwhp0, mw87, plab, syear87,  1, 1
; mwhp0, mw88, plab, syear88,  1, 1
; mwhp0, mw89, plab, syear89,  1, 1

mwp = fltarr (nw)
ip = 0
for i = 0, nw - 1 do	$
   if (mwxx (i) le 1.5e+14) then		$
   begin
      mwp (ip) = mwxx (i)
      ip = ip + 1
   end
nwp = ip
mwp = mwp (0 : nwp - 1)

 xinc = 5.0e+12
 xtype = 0
 ytype = 0
 cmehp0, mwxx, 'mass/width', 'grams/degree', plab, syearlab, xinc, xtype, ytype
; cmehp0, mwp,  'mass/width', 'grams/degree', plab, syearlab, xinc, xtype, ytype
 xinc = 0.1
 xtype = 1
 ytype = 0
; cmehp0, mwxx, 'mass/width', 'grams/degree', plab, syearlab, xinc, xtype, ytype
 xinc = 0.1
 xtype = 1
 ytype = 1
; cmehp0, mwxx, 'mass/width', 'grams/degree', plab, syearlab, xinc, xtype, ytype

; mwhp0, mwxx, plab, syearlab, 1, 1
; mwhp0, mwxx, plab, syearlab, 0, 0
; mwhp0, mwxx, plab, syearlab, 1, 0

 ;print, '!P.POSITION: ', !P.POSITION
 ;print, '!P.REGION:   ', !P.REGION
 ;print, '!P.THICK:    ', !P.THICK
 ;print, '!P.XTICKLEN: ', !P.TICKLEN

 ;print, '!X.WINDOW:   ', !X.WINDOW
 ;print, '!Y.WINDOW:   ', !Y.WINDOW
 ;print, '!X.REGION:   ', !X.WINDOW
 ;print, '!Y.REGION:   ', !Y.WINDOW

 ;print, '!D.X_SIZE:   ', !D.X_SIZE
 ;print, '!D.Y_SIZE:   ', !D.Y_SIZE
 ;print, '!D.X_VSIZE:  ', !D.X_VSIZE
 ;print, '!D.Y_VSIZE:  ', !D.Y_VSIZE

DEVICE, /CLOSE					; flush buffer to PS file.
SET_PLOT, 'X'					; reset output to X windows.

end
