pro mwhp0, xmw, plab, yearlab, xtype, ytype

;+
; Plot SMM C/P excess mass / width for total period.
; mass: fltarr of excess mass values [grams].
; title: string: plot title.
; yearlab: string label for plot describing years covered.
; xtype: 0 = linear, 1 = log
; ytype: 0 = linear, 1 = log
;
; Andrew L. Stanger   HAO/NCAR   19 August 1994
;-

;--- CME excess mass/width.

sxmw  = size (xmw)
nxmw  = sxmw (1)
ncme = nxmw

print, 'size (xmw): ', size (xmw)
;print, 'nxmw: ', nxmw
;print, 'xmw: '
;print, xmw

;--- Compute log (mass).

lxmw = fltarr (nxmw)
lxmw = alog10 (xmw)				; log (mass)

;for i = 0, ncme - 1 do $
;   print, i, xmw (i), lxmw (i)

;--- Compute average CME mass/width.

xmwtot = 0.0
for i = 0, ncme - 1 do xmwtot = xmwtot + xmw (i)
xmwave = xmwtot / ncme 
lxmwave = alog10 (xmwave)
lyave    = alog10 (0.1)
; print, yearlab, '  Average CME xmw:', xmwave

;--- Create histogram bins.

IF (xtype eq 1) THEN $				; x logarithmic.
BEGIN
   xmin = 12.0					; Minimum exponent of 10.
   xmax = 16.0					; Maximum exponent of 10.
   xinc = 0.1					; Increment exponent of 10.

   nbin = (xmax - xmin) / xinc + 1
   bin    = intarr (nbin + 1)
   binval = fltarr (nbin + 1)

   binexp0 = xmin
   for i = 0, nbin do $
      begin
         binexp = binexp0 + i * xinc
         binval (i) = binexp
      end

   ; Fill bins.

   for i = 0, ncme - 1 do $
      for n = 0, nbin - 1 do $
         if (lxmw (i) gt binval (n) and lxmw (i) le binval (n+1)) then $
            bin (n) = bin (n) + 1

   dxmin = 12.0
   dxmax = 16.0

END $
ELSE $						; x linear.
BEGIN
   xmin = 0.0
   xmax = 1.0e+16
   xinc = 1.0e+14

   nbin = ((xmax - xmin) / xinc) + 2
   bin    = intarr (nbin + 1)
   binval = fltarr (nbin + 1)

   bin0 = xmin
   for i = 0, nbin do $
      begin
	 binval (i) = bin0 + i * xinc
      end

   ; Fill bins.

   for i = 0, ncme - 1 do $
      for n = 0, nbin - 1 do $
         if (xmw (i) gt binval (n) and xmw (i) le binval (n+1)) then $
            bin (n) = bin (n) + 1

   dxmin = 1.0e+12
   dxmax = 1.0e+16 

END

;for i = 0, nbin do $
;   print, 'bin: ', i, '   binval: ', binval (i), '   binnum: ', bin (i)  

sum = 0 
for i = 0, nbin do sum = sum + bin (i)
;print, 'ncme: ', ncme, '   sum (histogram): ', sum

;--- Plot histogram.

xval = binval + xinc / 2.0
sxval = size (xval)
nxval = sxval (1)

;plab = ' CME Mass Histogram   SMM C/P'

IF (xtype eq 0 and ytype eq 0) THEN $		; x linear, y linear.
   BEGIN
   xlab = 'MASS / WIDTH [grams/degree]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO,			$
         XRANGE = [dxmin, dxmax],		$
	 YRANGE = [ymin, ymax],			$
         TITLE  = plab,				$
         XTITLE = xlab,				$
         YTITLE = ylab,				$
;	 POSITION = [0.10, 0.00, 0.9, 0.50],	$
         CHARSIZE = 0.8,			$
;         XCHARSIZE = 0.8,			$
;         YCHARSIZE = 0.8,			$
         XTHICK = 5.0, YTHICK = 5.0,		$
	 XTICKLEN = -0.02, YTICKLEN = -0.02,	$
         PSYM   = 10    			; Histogram plot mode.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, xmwave, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.8
   plots, [xmwave, xmwave], [ymin, ymax], LINESTYLE = 2
   END $
ELSE $
IF (xtype eq 1 and ytype eq 0) THEN $		; x log, y linear.
   BEGIN
   xlab = 'log (MASS/WIDTH) [grams/degree]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO,			$
   XRANGE = [dxmin, dxmax],			$
   YRANGE = [ymin, ymax],			$
         TITLE  = plab,				$
         XTITLE = xlab,				$
         YTITLE = ylab,				$
         CHARSIZE = 0.8,			$
;         XCHARSIZE = 0.8,			$
;         YCHARSIZE = 0.8,			$
         XTHICK = 5.0, YTHICK = 5.0,		$
	 XTICKLEN = -0.02, YTICKLEN = -0.02,	$
         PSYM   = 10    				; Histogram plot mode.
;	 /XTYPE         				; X axis logarithmic.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, lxmwave, 2.0, 'AVERAGE', /DATA, ORIENTATION=90, CHARSIZE = 0.8
   plots, [lxmwave, lxmwave], [ymin, ymax], LINESTYLE = 2
   END $
ELSE $
IF (xtype eq 0 and ytype eq 1) THEN $		; x linear, y log.
   BEGIN
   xlab = 'MASS/WIDTH [grams/degree]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme) ) * 100.0
   ymin = 0.0
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO,			$
         XRANGE = [dxmin, dxmax],		$
         YRANGE = [ymin, ymax],			$
         TITLE  = plab,				$
         XTITLE = xlab,				$
         YTITLE = ylab,				$
         CHARSIZE = 0.8,			$
;         XCHARSIZE = 0.8,			$
;         YCHARSIZE = 0.8,			$
         XTHICK = 5.0, YTHICK = 5.0,		$
	 XTICKLEN = -0.02, YTICKLEN = -0.02,	$
         PSYM   = 10,				$	; Histogram plot mode.
	 /YTYPE         				; Y axis logarithmic.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, xmwave, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.8
   plots, [xmwave, xmwave], [ymin, ymax], LINESTYLE = 2
   END $
ELSE $
IF (xtype eq 1 and ytype eq 1) THEN $		; x log, y log.
   BEGIN
   xlab = 'log (MASS/WIDTH) [grams/degree]'
   ylab = 'PERCENTAGE OF EVENTS PER BIN'
   yval = (bin / float (ncme)) * 100.0
   ymin = 1.0e-1 
   ymax = max (yval) 
   PLOT, xval, yval, /YNOZERO,			$
         XRANGE = [dxmin, dxmax],		$
	 YRANGE = [ymin, ymax],			$
         TITLE  = plab,				$
         XTITLE = xlab,				$
         YTITLE = ylab,				$
         CHARSIZE = 0.8,			$
;         XCHARSIZE = 0.8,			$
;         YCHARSIZE = 0.8,			$
         XTHICK = 5.0, YTHICK = 5.0,		$
	 XTICKLEN = -0.02, YTICKLEN = -0.02,	$
         PSYM   = 10,				$	; Histogram plot mode.
;	 /XTYPE,				$	; X axis logarithmic.
	 /YTYPE         				; Y axis logarithmic.

   for i = 0, nxval - 1 do $
   begin
      if (yval (i) eq 0) then ybmin = !Y.CRANGE (0)  else ybmin = 0.0
      box, xval (i) - xinc/2, ybmin, xval (i) + xinc/2, yval (i), 45
   endfor

   OPLOT, xval, yval, PSYM = 10

   xyouts, lxmwave, 0.2, 'AVERAGE', /DATA, ORIENTATION = 90, CHARSIZE = 0.8
   plots, [lxmwave, lxmwave], [ymin, ymax], LINESTYLE = 2
   END

;OPLOT, xval, yval, PSYM   = 5			; overplot with triangles.

;for i = 0, nbin do $
;   box, xval (i), !Y.CRANGE (0), xval (i) + xval (i+1), yval (i), 45 

;--- Write year label on plot.

if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.85, yearlab, /NORMAL, CHARSIZE = 1.0 $
else $
   xyouts, 0.21, 0.85, yearlab, /NORMAL, CHARSIZE = 1.0

;--- Write # CMEs measured on plot.

cmelab = string (ncme, FORMAT = '(I3)') + ' CMEs measured'
if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.75, cmelab, /NORMAL, CHARSIZE = 0.8 $
else $
   xyouts, 0.21, 0.75, cmelab, /NORMAL, CHARSIZE = 0.8

;--- Write average xmw on plot.

avelab = 'mean = ' + string (xmwave, FORMAT = '(E8.2)') + ' g'
;print, 'xmwave: ', xmwave
if (xtype eq 0 and ytype eq 0) then $
   xyouts, 0.59, 0.70, avelab, /NORMAL, CHARSIZE = 0.8 $
else $
   xyouts, 0.21, 0.70, avelab, /NORMAL, CHARSIZE = 0.8

;FOR i = 0, nbin DO $
;   print, i, '   binval: ', binval (i), $
;	     '   xval: ', xval (i), '   yval: ', yval (i)
end
 
; --- Histogram fill procedure -------------------------------------------------

pro box, x0, y0, x1, y1, line
;   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0],  ORIENTATION = line 
   polyfill, [x0, x0, x1, x1], [y0, y1, y1, y0], COLOR = 191
end
