;
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; NAME		si.pro
;
; PURPOSE	Extract Spartan 201-3 images from a raw data file.
; 
; SYNTAX	si, 'tape1.raw'
;
; HISTORY	Created 13 September 1996 by Andrew L. Staner, HAO/NCAR
;
;+ + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +


;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
FUNCTION int2, bbuf			; Convert byte array to 16-bit integers.

n_byte = N_ELEMENTS (bbuf)
n_elem = n_byte / 2
b = BYTARR  (2, n_elem)
b (0) = bbuf (0 : 2 * n_elem - 1)
a = REFORM (b (0,*) * 256 + b (1,*))

RETURN, a
END
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
FUNCTION int4, bbuf			; Convert byte array to 32-bit integers.

n_byte = N_ELEMENTS (bbuf)
n_elem = n_byte / 4
b = BYTARR  (4, n_elem)
b (0) = bbuf (0 : 4 * n_elem - 1)
a = REFORM (b (0,*) * 16777216L + b (1,*) * 65536L + b (2,*) * 256L + b (3,*))

RETURN, a
END
;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
PRO si, ifile

;-- Define byte arrays.

hdr  = BYTARR (10)
ibuf = BYTARR (8200)

;--- Define UART structure array.

us = REPLICATE ({uart, blknum : 0L, bytnum : 0, err : 0B, z : 0B}, 512)

;--- Open input file & read header information.

OPENR, ilun, ifile, /GET_LUN
READU, ilun, hdr		; Read first 10 bytes of tape header.

print, 'hdr (0:3) : ', hdr (0), hdr (1), hdr (2), hdr (3), FORMAT = '(O3)'

;--- Convert header information from bytes to integers.

nblocks = int4 (hdr (0:3))	; Number of blocks.
blksize = int2 (hdr (4:5))	; Block size.
tapenum = int2 (hdr (6:7))	; Tape number.
uerrcnt = int2 (hdr (8:9))	; UART error count.

print, 'nblocks: ', nblocks
print, 'blksize: ', blksize
print, 'tapenum: ', tapenum
print, 'uerrcnt: ', uerrcnt

;--- Read UART structure.

READU, ilun, us			; Read in UART structure (next part of header).

POINT_LUN, ilun, 8200		; Position file after tape header.

;--- Read data blocks.

FOR i = 0, nblocks - 1 DO	$
BEGIN
   READU, ilun, bbuf
   blknum  = intr (bbuf (0:3))
   berrcnt = int4 (bbuf (4:7))
   data = bbuf (9:8199) 	; Data array size : 16 * 512 = 8192
   print, 'blknum : ', blknum, '   berrcnt : ', berrcnt
END

CLOSE, ilun

END

