;
; Checkheights4
;   Routine to evaluate height mask data from Mk4
;
;   David Elmore and Tony Darnell
;
;   File name of rawt extracted intensity file is hard
;   wired into code.
;
;   25 Sep 2001: [ALS] Cosmetic changes only.

heights = [1.224, 1.804, 2.287] 

; Adjust the following settings to isolate actual peaks

minpix = [40, 120, 220, 300]
maxpix = [120, 200, 300, 380]
x     = FLTARR (2)
y     = FLTARR (2)
numaz = 985
numaz = 984
img   = INTARR (384, numaz)
slot  = FLTARR (4, numaz)
col   = FINDGEN (81)

filename = '/thisbe/e/elmore/MkIV/Height/99d274.22_44.mk4.i'

gterms = 4
;
;============================================================================= 
; Stuff added by Tony to correctly read in a mk4 image.  I was getting
;	wraparound the other way.  I think the size of the header changed
;	since this was last done.
;

CLOSE, 1
head   = 0
head   = BYTARR (1024)
headib = 0
headib = LONARR (16)  
headf  = 0
headf  = FLTARR (24)
headie = 0
headie = LONARR (16)  
scl    = 2
scl    = 1
pixx   =  384
pixy   = 1200
pixx   = 1100
pixy   = 1100
xsz    = (pixx / scl)
ysz    = (pixy / scl)
xcen   = pixx / 2
ycen   = pixy / 2

top  = LONG (0)
fpos = 0
fpos = LONARR (5000)
ftmp = LONG (0)
ii   = 0

;--- Read input image.

CLOSE, 1

OPENR, 1, filename
   POINT_LUN, -1, top
   READU, 1, headib
   READU, 1, headf
   READU, 1, headie

   POINT_LUN, 1, top
   READU, 1, head

   pixx = headib  [9]		; X dimension
   pixy = headib [10]		; Y dimension

   print, pixx, pixy

   img = 0
   img = INTARR (pixx, pixy) 
   READU, 1, img
CLOSE, 1

    img = 1.0 * img
    a = WHERE ( img LT 0 )
    IF ( a [0] > 0 ) THEN img [a] = img [a] + 65536.0

;
; End of stuff added by Tony
;============================================================================= 
;

; This may change with realignments
;
;minx = 32

minx = 40
!y.style = 1
!x.style = 1
!x.range = [minx, 383]

doplot = 1
IF ( doplot EQ 1 ) THEN BEGIN
    SET_PLOT, 'ps'
    DEVICE, /color, /portrait,  $ ;/inches, xsize=8.0, ysize=5.0, $
                                  ;xoffset=0.5, yoffset=1.5, bits_per_pixel=8, $
            filename = 'mk4heights' + STRTRIM (STRING (gterms, FORMAT='(I2)'), 2) $
		     + '.ps'
ENDIF ELSE BEGIN
    WINDOW, ysize=384, xsize=numaz
    LOADCT, 0
    TV, BYTSCL (TRANSPOSE (img), -50, 20000)
;    STOP
ENDELSE

!y.range = 0
!p.title = 'Intensity vs Radius'
!x.title = 'Pixel Height'
!y.title = 'Intensity'
PLOT, img [*, numaz / 2]
;STOP

;--- Do gaussian fits.

FOR ii = 0, numaz - 1 DO BEGIN
    FOR jj = 0, 2 DO BEGIN
	fit = GAUSSFIT (col, img (minpix [jj]:maxpix [jj], ii), r, nterms=gterms)

	;IF ( (r [2] GT 2.7) AND (r [2] LT 10) ) THEN BEGIN

	IF ( (r [2] GT 2.0) AND (r [2] LT 35) ) THEN BEGIN
		slot [jj, ii] = r [1] + minpix [jj]
	ENDIF ELSE BEGIN
		str = STRING ('bad r [2] = ', r [2])
		print, str
		IF ( ii EQ 0 ) THEN BEGIN
			slot [jj, ii] = (minpix [jj] + maxpix [jj] / 2.0)
		ENDIF ELSE BEGIN
			slot [jj, ii] = slot [jj, ii - 1]
		ENDELSE
	ENDELSE
    ENDFOR
;    print, ii
ENDFOR

meanpix = FLTARR (4)

!p.title = filename
!y.title = 'Pixel Number'
!x.title = 'Scan Azimuth'
!x.range = [0, numaz]

;--- Plot slot position versus azimuth.

;FOR jj = 0, 3 DO BEGIN

FOR jj = 0, 2 DO BEGIN
	!y.range = [slot [jj, numaz / 2] - 5, slot [jj, numaz / 2] + 5]
	PLOT, slot [jj, *], psym=1
	m = MOMENT (slot [jj, *])
	meanpix [jj] = m [0]
	str = STRING (m [0], '+/-', SQRT (m [1]))
	XYOUTS,  numaz / 4, slot [jj, numaz / 2] - 2, str
	str = STRING (heights [jj], ' R-sun')
	XYOUTS,  numaz / 4, slot [jj, numaz / 2] - 3, str
;	STOP
ENDFOR

;IF ( meanpix (0) LT 60 ) THEN heights [0] = 1.185

slopes  = FLTARR (numaz)
offs    = FLTARR (numaz)
curves  = FLTARR (numaz)
limbpix = FLTARR (numaz)
toppix  = limbpix

;--- Perform straight line fit to slot positions.

FOR ii = 0, numaz - 1 DO BEGIN
	;res = POLY_FIT (slot [*, ii], heights, 1)

	res = POLY_FIT (slot [0:2, ii], heights, 1)
	slopes [ii] = res [1]
	offs [ii] = res [0]
ENDFOR

limbpix = minx  * slopes + offs
toppix  = 383.0 * slopes + offs

!y.range = [0.98 * limbpix (numaz / 2), 1.02 * limbpix (numaz / 2)]
!y.title = 'R0 Lowest Corona'
PLOT, limbpix, psym=2
;STOP

!y.range = [0.98 * toppix (numaz / 2), 1.02 * toppix (numaz / 2)]
!y.title = 'R0 Highest Corona'
PLOT, toppix, psym=1

;STOP

m = MOMENT (offs)
off = m [0]			; Average offset [r0].

m = MOMENT (slopes)
slope = m [0]			; Average slope  [dr].

;m = MOMENT (curves)
;curvature = m [0]

pixels  = FLTARR (384)
numbers = FINDGEN (384)
pixels  = off + numbers * slope

;--- Plot straight line fit.

!x.style = 1
!x.range = [minx, 383]
!y.range = [1, 3]
!x.title = 'Pixel Number'
PLOT, pixels
str = STRING ('height =', off, ' + pixel *', slope)
print, str
XYOUTS, 44, 2.05, str

;FOR ii = 0, 3 DO BEGIN

FOR ii = 0, 2 DO BEGIN
	x [0] = meanpix [ii]
	x [1] = x [0]
	y [0] = heights [ii] - 0.1
	y [1] = heights [ii] + 0.1
	PLOTS, x, y
	x [0] = meanpix [ii] - 5
	x [1] = meanpix [ii] + 5
	y [0] = heights [ii]
	y [1] = y [0]
	PLOTS, x, y
ENDFOR

print, 'offset: ', off, '   slope: ', slope, '    gterms: ', gterms
;STOP

IF ( doplot EQ 1 ) THEN BEGIN
	DEVICE, /CLOSE
ENDIF

print, 'Done'

END

