
/*
 *	dpmon_header.h
 *
 *	Header structure for digital Prominence Monitor data records
 *
 *	A dPMon "Image Record" is an array of short integers (16 bit)
 *	1534 rows (x subscript rapidly varying) by 
 *	1030 columns (y subscript, slowly varying).
 *
 *	The first row, 3068 bytes, contains this header structure.
 *	The next 1029 rows each contain 1534 unsigned short integers
 *		of camera image.
 *
 *	The physical format on the tape:
 *		Image Records writen as 103 blocks each 30680 bytes
 *	(2 bytes/short integer) in length.
 *		Image records follow image records.
 *		Before every calibration sequence there is a filemark.
 *		At the end of the day there are two filemarks.
 *
 *	Image normalization:
 *		I recommend normalizing all data to 13 bits (0-8192).
 *	This is the best possible signal to noise achievable with a 
 *	10-bit camera summed 64 times to fill a 16 bit word.
 *	To do the normalization FIRST multiply each pixel value by 8
 *	then divide by the number of exposures summed,the .Nsum element
 *	of the header structure.  The Nsum value will normally be a power
 *	of two so that shifts can be used (check first).
 *
 *	David Elmore
 *
 *	Thu Jan 13 20:01:17 GMT 1994
 */

struct	dpmon_header
	{
	int		Version;	/* Program version number */

	int		Year;		/* Date and time UT */
	int		Month;
	int		Day;
	int		Hour;
	int		Minute;
	int		Sec;

	int		Xdim;		/* # of pixels in X */
	int		Ydim;		/* # of pixels in Y */
	int		Nsum;		/* # of exposures summed to form one */
	int		Exposure;	/* Exposure time in hundredths of sec*/
	int		Gain;		/* Camera controller gain factor */
	int		Odisk;		/* O disk diameter in arc-seconds */
	int		Type;		/* DARK, DISK, LIMB, or FLAT */

	int		Optics[8];	/* Positions of all 8 optical devs */
					/* see dev_defs.h for definitions */

	float	Pangle;			/* P angle in degrees */
	float	Bangle;			/* B angle in degrees */
	float	Langle;			/* L angle in degrees */
	float	HA;			/* Hour angle in degrees */
	float	Elevation;		/* Solar elevation in degrees */
	float	Azimuth;		/* Solar azimuth in degrees */
	float	Amass;			/* Airmass */
	float	Radius;			/* Solar radius in arc-seconds 30*4*/

	char	dummy[1534-122];	/* set so that length <= 1536 */
	};

#ifdef DPMON
	struct	dpmon_header	*Image_header;
#else
	extern	struct	dpmon_header	*Image_header;
#endif
