/* fitskey.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * FITS key words.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 March 1994
 * 22 April 1994: add keywords: TYPE-OBS, etc., for disk images (BBSO, KPNO)
 * 2 June 1994: change:     BANGLE   -> SOLAR_B0, HANGLE   -> SOLAR_H0,
 *                          LANGLE   -> SOLAR_L0, PANGLE   -> SOLAR_P0
 * 18 August 1994: change : SOLAR_H0 -> HOUR-ANG                      
 *                          SCNDIR   -> SCAN-DIR, SCNDUR   -> SCAN-DUR
 *                          ANGNORTH -> NORTHANG, SCROLL   -> SC-ROLL
 *		   add    : CODE-OBS, CRRADIUS, DISP-DIR, OCC-DISC
 *                 remove : CPVER, DPMVER, MK3VER
 * 22 August 1994: add    : CODE-CP, CODE-DPM, CODE-MK3
 * 16 November 1998: OPTDEV1 -> OPTDEV8 becomes OPTDEV00 -> OPTDEV07.
 *                   OPTPOS1 -> OPTPOS8 becomes OPTPOS00 -> OPTPOS07.
 *                   Add OPTDEV08 -> OPTDEV12 and OPTPOS08 -> OPTPOS12.
 *                   Add detector, image_id, orbit_id keywords.
 * 21 Sep 2000: Add TYPE-DP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define numkey 170

char fitskey [numkey][12] =
   {
   "",			/*   0 */

   "AIRMASS",		/*   1 */
   "ANGLE",		/*   2 */
   "AUTHOR",		/*   3 */
   "AZIMUTH",		/*   4 */

   "BIAS",		/*   5 */
   "BITPIX",		/*   6 */
   "BLANK",		/*   7 */
   "BLOCKED",		/*   8 */
   "BSCALE",		/*   9 */
   "BUNIT",		/*  10 */
   "BZERO",		/*  11 */

   "CALIBRAT",		/*  12 */
   "CALMIR",		/*  13 */
   "CAMERA",		/*  14 */
   "CDELT1",		/*  15 */
   "CDELT2",		/*  16 */
   "CDELT3",		/*  17 */
   "CDELT4",		/*  18 */
   "CODE-CP",		/*  19 */
   "CODE-DPM",		/*  20 */
   "CODE-MK3",		/*  21 */
   "CODE-OBS",		/*  22 */
   "COLORFIL",		/*  23 */
   "COMMENT",		/*  24 */
   "CROTA1",		/*  25 */
   "CROTA2",		/*  26 */
   "CROTA3",		/*  27 */
   "CROTA4",		/*  28 */
   "CRPIX1",		/*  29 */
   "CRPIX2",		/*  30 */
   "CRPIX3",		/*  31 */
   "CRPIX4",		/*  32 */
   "CRRADIUS",		/*  33 */
   "CRVAL1",		/*  34 */
   "CRVAL2",		/*  35 */
   "CRVAL3",		/*  36 */
   "CRVAL4",		/*  37 */
   "CTYPE1",		/*  38 */
   "CTYPE2",		/*  39 */
   "CTYPE3",		/*  40 */
   "CTYPE4",		/*  41 */

   "DATAFORM",		/*  42 */
   "DATAMAX",		/*  43 */
   "DATAMIN",		/*  44 */
   "DATE",		/*  45 */
   "DATE-CAL",		/*  46 */
   "DATE-OBS",		/*  47 */
   "DATE-OPC",		/*  48 */
   "DELTAT",		/*  49 */
   "DETECTOR",		/*  50 */
   "DIRAXIS1",		/*  51 */
   "DIRAXIS2",		/*  52 */
   "DIRAXIS3",		/*  53 */
   "DIRAXIS4",		/*  54 */
   "DISP-DIR",		/*  55 */
   "DISPMAX",		/*  56 */
   "DISPMIN",		/*  57 */

   "ELEV",		/*  58 */
   "END",		/*  59 */
   "EPOCH",		/*  60 */
   "EQUINOX",		/*  61 */
   "EXPDUR",		/*  62 */
   "EXPUNITS",		/*  63 */
   "EXTEND",		/*  64 */
   "EXTLEVEL",		/*  65 */
   "EXTNAME",		/*  66 */
   "EXTVER",		/*  67 */

   "FILTER",		/*  68 */
   "FRAME",		/*  69 */
   "FRAMES",		/*  70 */

   "GAIN",		/*  71 */
   "GCOUNT",		/*  72 */
   "GLFILTER",		/*  73 */

   "HISTORY",		/*  74 */
   "HOUR_ANG",		/*  75 */

   "IMAGE-ID",		/*  76 */
   "INSTRUME",		/*  77 */
   "INT-MODE",		/*  78 */

   "NAXIS",		/*  79 */
   "NAXIS1",		/*  80 */
   "NAXIS2",		/*  81 */
   "NAXIS3",		/*  82 */
   "NAXIS4",		/*  83 */
   "NIMAVG",		/*  84 */
   "NORTHANG",		/*  85 */

   "OBJECT",		/*  86 */
   "OBSERVER",		/*  87 */
   "OCC-DISC", 		/*  88 */
   "OPTDEV00",		/*  89 */
   "OPTDEV01",		/*  90 */
   "OPTDEV02",		/*  91 */
   "OPTDEV03",		/*  92 */
   "OPTDEV04",		/*  93 */
   "OPTDEV05",		/*  94 */
   "OPTDEV06",		/*  95 */
   "OPTDEV07",		/*  96 */
   "OPTDEV08",		/*  97 */
   "OPTDEV09",		/*  98 */
   "OPTDEV10",		/*  99 */
   "OPTDEV11",		/* 100 */
   "OPTDEV12",		/* 101 */
   "OPTPOS00",		/* 102 */
   "OPTPOS01",		/* 103 */
   "OPTPOS02",		/* 104 */
   "OPTPOS03",		/* 105 */
   "OPTPOS04",		/* 106 */
   "OPTPOS05",		/* 107 */
   "OPTPOS06",		/* 108 */
   "OPTPOS07",		/* 109 */
   "OPTPOS08",		/* 110 */
   "OPTPOS08",		/* 111 */
   "OPTPOS08",		/* 112 */
   "OPTPOS08",		/* 113 */
   "OPTPOS08",		/* 114 */
   "ORBIT-ID",		/* 115 */
   "ORIGIN",		/* 116 */

   "PCOUNT",		/* 117 */
   "PHI",		/* 118 */
   "POLAROID",		/* 119 */
   "PUNIT",		/* 120 */

   "QUALITY",		/* 121 */

   "RADIUS",		/* 122 */
   "RANGVALU",		/* 123 */
   "REFERENC",		/* 124 */
   "RMAX",		/* 125 */
   "RMIN",		/* 126 */
   "RSUN",		/* 127 */
   "RUNIT",		/* 128 */

   "SECTOR",		/* 129 */
   "SEEING",		/* 130 */
   "SCAN-DIR",		/* 131 */
   "SCAN-DUR",		/* 132 */
   "SC-ROLL",		/* 133 */
   "SIMPLE",		/* 134 */
   "SKY",		/* 135 */
   "SOLAR_B0",		/* 136 */
   "SOLAR_L0",		/* 137 */
   "SOLAR_P0",		/* 138 */

   "TELESCOP",		/* 139 */
   "THETA",		/* 140 */
   "TIME",		/* 141 */
   "TIME-CAL",		/* 142 */
   "TIME-OBS",		/* 143 */
   "TIME-OPC",		/* 144 */
   "TILT",		/* 145 */
   "TMAX",		/* 146 */
   "TMIN",		/* 147 */
   "TUNIT",		/* 148 */
   "TYPE-CP",		/* 149 */
   "TYPE-DP",		/* 150 */
   "TYPE-DPM",		/* 151 */
   "TYPE-MK3",		/* 152 */
   "TYPE-OBS",		/* 153 */
   "TYPE1",		/* 154 */
   "TYPE2",		/* 155 */
   "TYPE3",		/* 156 */
   "TYPE4",		/* 157 */
   "TYPE5",		/* 158 */
   "TYPE6",		/* 159 */
   "TYPE7",		/* 160 */
   "TYPE8",		/* 161 */

   "UTSTART",		/* 162 */
   "UTSTOP",		/* 163 */

   "VERSION",		/* 164 */

   "WAVE   ",		/* 165 */
   "WAVELEN",		/* 166 */
   "WAVELNTH",		/* 167 */

   "XTENSION",		/* 168 */

   "ZEROVALU"		/* 169 */
   } ;
