/* fitslib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * FITS keyword variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 March 1994
 * 22 April 1994: add variables: type_obs, etc., for disk images (BBSO, KPNO)
 * 2 June 1994: change:     bangle   -> solar_b0, hangle   -> solar_H0,
 *                          langle   -> solar_l0, pangle   -> solar_p0
 * 18 August 1994: change : solar_h0 -> hour_ang                      
 *                          scndir   -> scan_dir, scndur   -> scan_dur
 *                          angnorth -> northang, scroll   -> sc_roll
 *		   add    : code_obs, crradius, disp_dir, occ_disc
 *                 remove : cpver, dpmver, mk3ver
 * 22 August 1994: add    : CODE-CP, CODE-DPM, CODE-MK3
 * 19 June 1997: change "end" to "end_fts" & "time" to "time_fts".
 * 22 December 1997: Initialize naxis1=naxis2=naxis3=naxis4=1 (instead of zero).
 * 13 November 1998: calmir, colorfil, glfilter, polaroid, sector
 *                   changed from short to char [16].
 *                   optdev1 -> optdev8 changed to optdev01 -> optdev08.
 *                   optpos1 -> optpos8 changed to optpos01 -> optpos08.
 * 16 November 1998: optdev01 -> optdev08 replaced by optdev00 -> optdev12.
 *                   optpos01 -> optpos08 replaced by optpos00 -> optpos12.
 *                   Add detector, image_id, orbit_id variables.
 * 21 Sep 2000: Add type_dp.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int	simple   = 0 ;	/* SIMPLE ? = 1, T; = 0, F; = -1, not defined. */
int	extend   = 0 ;	/* EXTEND ? = 1, T, = 0, F; = -1, not defined. */
int	blocked  = 0 ;	/* =0, NO; =1, YES. */
int	end_fts  = 0 ;	/* END encountered ? =0, NO; =1, YES. */

int	bitpix   = 0 ;	/* # bitx/pixel: 8,16,32: int; -16,-32: float. */
int	naxis    = 0 ;		/* # axes. */
int	naxis1   = 1 ;		/* # elements along axis 1. */
int	naxis2   = 1 ;		/* # elements along axis 2. */
int	naxis3   = 1 ;		/* # elements along axis 3. */
int	naxis4   = 1 ;		/* # elements along axis 4. */

int	pcount   = 0 ;		/* pcount. */
int	gcount   = 0 ;		/* gcount. */

int	image_id = 0 ;		/* image ID. */
int	orbit_id = 0 ;		/* oribt ID. */

char	xtension [12] ;		/* Extension name. */

char	date     [12] = "" ;	/* DD/MM/YY : file created */
char	date_obs [12] = "" ;	/* DD/MM/YY : observation */
char	date_cal [12] = "" ;	/* DD/MM/YY : calibration */
char	date_opc [12] = "" ;	/* DD/MM/YY : MK3 cal 0 */
char    detector [12] = "" ;	/* Detector designation. */

char	time_fts [12] = "" ;	/* HH/MM/SS : file created */
char	time_obs [12] = "" ;	/* HH/MM/SS : observation */
char	time_cal [12] = "" ;	/* HH/MM/SS : calibration */
char	time_opc [12] = "" ;	/* HH/MM/SS : MK3 cal 0 */

char	origin   [72] = "" ;	/* Institution name. */
char	author   [72] = "" ;	/* Compilation author's name. */
char	referenc [72] = "" ;	/* Reference where assoc. data are published. */

char	telescop [72] = "" ;	/* Telescope   name. */
char	instrume [72] = "" ;	/* Instrument  name. */
char	observer [72] = "" ;	/* Observer's  name. */
char	object   [72] = "" ;	/* Object      name. */

float	equinox  = 0 ;	/* Equinox (year) for celestial coord system. */
float	epoch    = 0 ;	/* Epoch   (year) for celestial coord system. */

char	extname  [72] = "" ;	/* Extension name.	*/
int	extver   = 0 ;		/* Extension version.	*/
int	extlevel = 0 ;		/* Extension level.	*/

int	blank    = 0 ;		/* Blank value. */

float	b_scale  = 1.0 ;	/* Multiplicative scaling value. */
float	b_zero   = 0.0 ;	/* Additive scaling value. */

char	bunit    [12] = "" ;	/* Name of brightness (magnitude) units. */

char	ctype1   [72] = "" ;	/* Name of coordinate axis 1. */
char	ctype2   [72] = "" ;	/* Name of coordinate axis 2. */
char	ctype3   [72] = "" ;	/* Name of coordinate axis 3. */
char	ctype4   [72] = "" ;	/* Name of coordinate axis 4. */

float	crpix1   = 0.0 ;/* Location of reference point axis 1 [index]. */
float	crpix2   = 0.0 ;/* Location of reference point axis 2 [index]. */
float	crpix3   = 0.0 ;/* Location of reference point axis 3 [index]. */
float	crpix4   = 0.0 ;/* Location of reference point axis 4 [index]. */

float	crradius = 0.0 ;/* Rsun [pixels]. */

float	crval1   = 0.0 ;/* Location of reference point axis 1 [coord 1]. */
float	crval2   = 0.0 ;/* Location of reference point axis 2 [coord 2]. */
float	crval3   = 0.0 ;/* Location of reference point axis 3 [coord 3]. */
float	crval4   = 0.0 ;/* Location of reference point axis 4 [coord 4]. */

float	cdelt1   = 0.0 ;/* Increment between pixels axis 1 [coord 1]. */
float	cdelt2   = 0.0 ;/* Increment between pixels axis 2 [coord 2]. */
float	cdelt3   = 0.0 ;/* Increment between pixels axis 3 [coord 3]. */
float	cdelt4   = 0.0 ;/* Increment between pixels axis 4 [coord 4]. */

float	crota1   = 0.0 ;/* Rotation angle of axis 1 [coord 1]. */
float	crota2   = 0.0 ;/* Rotation angle of axis 2 [coord 2]. */
float	crota3   = 0.0 ;/* Rotation angle of axis 3 [coord 3]. */
float	crota4   = 0.0 ;/* Rotation angle of axis 4 [coord 4]. */

float	datamin  = 0.0 ;/* Minimum physical value in data array. */
float	datamax  = 0.0 ;/* Maximum physical value in data array. */

float	dispmin  = 0.0 ;/* Minimum display value in data array. */
float	dispmax  = 0.0 ;/* Maximum display value in data array. */

char	disp_dir [4] ;	/* Display direction: R/L U/D O/I */

char    diraxis1 [8] = "" ;	/* Scan direction axis 1: "left" or "right". */
char    diraxis2 [8] = "" ;	/* Scan direction axis 2: "up"   or "down".  */
char    diraxis3 [8] = "" ;	/* Scan direction axis 3. "in"   or "out"    */
char    diraxis4 [8] = "" ;	/* Scan direction axis 4. "plus" or "minus"  */

char	dataform [72] = "" ;	/* Data type description. */

int	code_cp  = 0 ;		/* SMM C/P data code. */
int	code_dpm = 0 ;		/* MLSO DPM data code. */
int	code_mk3 = 0 ;		/* MLSO MK3 data code. */
int	code_obs = 0 ;		/* observation code. */

char	type_cp  [72] ;		/* SMM C/P data type. */
char	type_dp  [72] ;		/* Type of data processing. */
char	type_dpm [72] ;		/* MLSO DPM data type. */
char	type_mk3 [72] ;		/* MLSO MK3 data type. */
char	type_obs [72] ;		/* Type of observation. */
char	type1 [72] = "" ;	/* Type 1. */
char	type2 [72] = "" ;	/* Type 2. */
char	type3 [72] = "" ;	/* Type 3. */
char	type4 [72] = "" ;	/* Type 4. */
char	type5 [72] = "" ;	/* Type 5. */
char	type6 [72] = "" ;	/* Type 6. */
char	type7 [72] = "" ;	/* Type 7. */
char	type8 [72] = "" ;	/* Type 8. */

int	nimavg   = 0 ;		/* # images averaged. */
float	expdur   = 0 ;		/* Exposure duration. */
char	expunits [12] = "" ;	/* Exposure duration units. */

float	rmin     = 0.0 ;	/* Minimum radius. */
float	rmax     = 0.0 ;	/* Maximum radius. */
float	rsun     = 0.0 ;	/* pixels per solar radius. */

float	tmin     = 0.0 ;	/* Minimum angle. */
float	tmax     = 0.0 ;	/* Maximum angle. */

float	scan_dur   = 0.0 ;	/* MLSO MK3 scan duration. */
char	scan_dir [4] = "" ;	/* MLSO MK3 scan direction: CW,CCW. */

float	northang = 0.0 ;	/* Angle of solar north w.r.t. +X axis. */
float	angle    = 0.0 ;	/* Angle. */

float	bias     = 0.0 ;	/* Bias. */
float	gain     = 0.0 ;	/* Gain. */
float	deltat   = 0.0 ;	/* Delta Time. */
float	phi      = 0.0 ;	/* Phi angle. */
float	theta    = 0.0 ;	/* Theta angle. */
float	radius   = 0.0 ;	/* Radius. */
float	rangvalu = 0.0 ;	/* Range value. */
float	zerovalu = 0.0 ;	/* Zero value. */
float	tilt     = 0.0 ;	/* Tilt angle. */
float	wave     = 0.0 ;	/* Wavelength. */
float	wavelen  = 0.0 ;	/* Wavelength. */
float	wavelnth = 0.0 ;	/* Wavelength. */

char	calibrat  [4] = "" ;	/* Calibrated. */
char	camera    [4] = "" ;	/* Camera name. */
char	filter   [24] = "" ;	/* Filter name. */
char	int_mode [24] = "" ;	/* Integration mode. */
char	runit    [24] = "" ;	/* Radius units. */
char	punit    [24] = "" ;	/* Phi angle units. */
char	tunit    [24] = "" ;	/* Theta angle units. */
char	seeing   [24] = "" ;	/* Seeing.  */
char	quality  [24] = "" ;	/* Quality. */
char	sky      [24] = "" ;	/* Sky.     */
char	utstart  [24] = "" ;	/* UT start time. */
char	utstop   [24] = "" ;	/* UT stop  time. */
char	version  [24] = "" ;	/* Version name.  */

int	frames   = 0 ;		/* # frames. */

float	occ_disc = 0.0 ;	/* occulting disk diameter [arcsec]. */

float	solar_b0 = 0.0 ;	/* B-angle. */
float	solar_l0 = 0.0 ;	/* L-angle. */
float	solar_p0 = 0.0 ;	/* P-angle. */
float	hour_ang = 0.0 ;	/* H-angle. */

float	elev     = 0.0 ;	/* Elevation (degrees). */
float	azimuth  = 0.0 ;	/* Azimuth   (degrees). */
float	airmass  = 0.0 ;	/* Airmass. */

char	optdev00 [16] = "" ;	/* optical device #  0 */
char	optdev01 [16] = "" ;	/* optical device #  1 */
char	optdev02 [16] = "" ;	/* optical device #  2 */
char	optdev03 [16] = "" ;	/* optical device #  3 */
char	optdev04 [16] = "" ;	/* optical device #  4 */
char	optdev05 [16] = "" ;	/* optical device #  5 */
char	optdev06 [16] = "" ;	/* optical device #  6 */
char	optdev07 [16] = "" ;	/* optical device #  7 */
char	optdev08 [16] = "" ;	/* optical device #  8 */
char	optdev09 [16] = "" ;	/* optical device #  9 */
char	optdev10 [16] = "" ;	/* optical device # 10 */
char	optdev11 [16] = "" ;	/* optical device # 11 */
char	optdev12 [16] = "" ;	/* optical device # 12 */

char	optpos00 [16] = "" ;	/* optical position #  0 */
char	optpos01 [16] = "" ;	/* optical position #  1 */
char	optpos02 [16] = "" ;	/* optical position #  2 */
char	optpos03 [16] = "" ;	/* optical position #  3 */
char	optpos04 [16] = "" ;	/* optical position #  4 */
char	optpos05 [16] = "" ;	/* optical position #  5 */
char	optpos06 [16] = "" ;	/* optical position #  6 */
char	optpos07 [16] = "" ;	/* optical position #  7 */
char	optpos08 [16] = "" ;	/* optical position #  8 */
char	optpos09 [16] = "" ;	/* optical position #  9 */
char	optpos10 [16] = "" ;	/* optical position # 10 */
char	optpos11 [16] = "" ;	/* optical position # 11 */
char	optpos12 [16] = "" ;	/* optical position # 12 */

char	calmir   [16] = "" ;	/* SMM C/P calibration mirror position.	*/
char	colorfil [16] = "" ;	/* SMM C/P color filter position.	*/
char	glfilter [16] = "" ;	/* SMM C/P Green Line filter position.	*/
char	polaroid [16] = "" ;	/* SMM C/P polaroid filter position.	*/
char	sector   [16] = "" ;	/* SMM C/P sector mirror position.	*/
int	frame	= 0 ;		/* SMM C/P frame number.		*/
int	sc_roll	= 0 ;		/* SMM C/P spacecraft roll (degrees).	*/
