/* smmcplib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Global variables for cplib.a library.
 *  1 Apr 1999: add rsun to hskp structure.
 * 25 Sep 2002: add variables for complete list housekeeping items.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/*** cpscan.h ***/

#define	MAXSCN	3600

struct cpheader
{
int year ;			/* Image year [-1900]. */
int doy ;			/* Image DOY. */
int pwpos ;			/* Polaroid Wheel Position. */
int hvstat ;			/* HV Command Status. */
int smmid ;			/* SMM Instrument ID. */
int resolut ;			/* Image Resolution. */
int datamode ;			/* Data Mode. */
int hkfrozen ;			/* Housekeeping Frozen. */
int segcnt ;			/* Segment Count. */
int expinh ;			/* Exposure Inhibit. */
int readinh ;			/* Read Inhibit. */
int prexpinh ;			/* Premature Exposure Inhibit. */
int met ;			/* Mission Elapsed Time. */
int ih ;			/* IH horizontal image parameter. */
int iv ;			/* Iv vertical   image parameter. */
int expose ;			/* Exposure Duration. */
int glpos ;			/* Green Line Position. */
int glstow ;			/* Green Line Stow status. */
int smpos ;			/* Sector Mirror Position. */
int smmrollr ;			/* Spacecraft Roll [2.e-9 radian units]. */
int ispos ;			/* Internal Shutter Position. */
int adpos ;			/* Aperture Door Position. */
int cmpos ;			/* Calibration Mirror Position. */
int fwpos ;			/* Filter WHeel Position. */

int pes1a ;			/* PES #1. */
int pes2a ;			/* PES #2. */
int pes3a ;			/* PES #3. */
int pes4a ;			/* PES #4. */

int hour ;			/* Image hour. */
int minute ;			/* Image minute. */
int pcsyoff ;			/* PCS Y Offset. */
int pcszoff ;			/* PCS Z Offset. */
int second ;			/* Image second. */
int metconv ;			/* MET -> GMT Conversion Index. */
int pcsth ;			/* PCS Threshold. */
int dasth ;			/* DAS Threshold. */
int pcsovr ;			/* PCS Override Statud. */
int dasovr ;			/* DAS Overrride Status. */
int pcsthexc ;			/* PCS Threshold Exceeded. */
int dasthexc ;			/* DAS Threshold Exceeded. */
int stat8v ;			/* 8 Volt Status. */
int pcsen ;			/* PCS Enable/Disable Status. */
int secerase ;			/* SEC Erase Mode. */
int secexpos ;			/* SEC Expose Mode. */
int secread ;			/* SEC Read Mode. */
int imsizovr ;			/* Image Size Override. */
int flood ;			/* Flood Status. */
int mebseq ;			/* MEB Sequencer Reset. */
int isredun ;			/* Internal Shutter Redundant Enable. */
int adredun ;			/* Aperture Door Redundant Enable. */
int cmredun ;			/* Calibration Mirror Redundant Enable. */
int glredun ;			/* 5303 Redundant Enable. */
int flpower ;			/* Flood Lamp Power. */
int imgcnt ;			/* Image Counter. */

int pes1b ;			/* PES #1. */
int pes2b ;			/* PES #2. */
int pes3b ;			/* PES #3. */
int pes4b ;			/* PES #4. */

char refvolid [8] ;		/* Reformatted Volume ID. */
char refdsid  [8] ;		/* Reformatted Dataset ID. */

int pes1c ;			/* PES #1. */
int pes2c ;			/* PES #2. */
int pes3c ;			/* PES #3. */
int pes4c ;			/* PES #4. */

int bytorder ;			/* Byte Order [0=sun, 1=DEC. */
int scbvp15 ;			/* SCB +15v. */
int scbvm15 ;			/* SCB -15v. */
int refyear ;			/* Reformatted Year (year - 1900). */
int scbvp10 ;			/* SCB +10v. */
int scbvp12 ;			/* SCB +12v. */
int pcbvp15 ;			/* PCB +15v. */
int pcbvm15 ;			/* PCB -15v. */
int scbvp22 ;			/* SCB +22v. */
int scbvm22 ;			/* SCB -22v. */
int refdoy ;			/* Reformatted data DOY. */

int pes1d ;			/* PES #1. */
int pes2d ;			/* PES #2. */
int pes3d ;			/* PES #3. */
int pes4d ;			/* PES #4. */

int pcsypos ;			/* PCS Y Position. */
int pcszpos ;			/* PCS Z Position. */
int src_cpu ;			/* Source Processor. */
int mcbvp5 ;			/* MCB +5v. */
int dasypos ;			/* DAS Y Position. */
int daszpos ;			/* DAS Z Position. */
int mebvp15 ;			/* MEB +15v. */
int mebvm15 ;			/* MEB -15v. */
int mebvp12 ;			/* MEB +12v. */
int mebvp5 ;			/* MEB +5v. */
int mebvp64;			/* MEB +6.4v. */

int pes1e ;			/* PES #1. */
int pes2e ;			/* PES #2. */
int pes3e ;			/* PES #3. */
int pes4e ;			/* PES #4. */

int imgquan ;			/* Image Quantity [0-100%]. */
int sdibvp15 ;			/* SDIB +15v. */
int sdibvm15 ;			/* SDIB -15v. */
int sdibvp5 ;			/* SDIB +5v. */
int sectemp ;			/* SEC Temperature. */
int smmrolld ;			/* S/C Roll [degrees]. */
int vpbvp64 ;			/* VPB +6.4v. */
int vpbvp26 ;			/* VPB +26v. */
int vpbvp20 ;			/* VPB +20v. */
int vpbvm10 ;			/* VPB -10v. */
int filvolt ;			/* Filament Volts. */
int filcur ;			/* Filament Current. */

int pes1f ;			/* PES #1. */
int pes2f ;			/* PES #2. */
int pes3f ;			/* PES #3. */
int pes4f ;			/* PES #4. */

int pcathv ;			/* Photocathode Volts. */
int g1v ;			/* G1 Volts. */
int g2v ;			/* G2 Volts. */
int nbitpix ;			/* Bits per pixel. */
int cathvh ;			/* Cathode Volts [high]. */
int cathvl ;			/* Cathode Volts [high]. */
int g3vh ;			/* G3 high Volts. */
int g3vl ;			/* G3 low  Volts. */
int g4vh ;			/* G4 high Volts. */
int g4vl ;			/* G4 low  Volts. */
int npixrow ;			/* # pixels / line. */

int pes1g ;			/* PES #1. */
int pes2g ;			/* PES #2. */
int pes3g ;			/* PES #3. */
int pes4g ;			/* PES #4. */

int secalcv ;			/* SEC Alignment Current [vertical]. */
int secalch ;			/* SEC Alignment Current [horizaontal].*/
int secalv ;			/* SEC Alignment Voltage.*/
char fileid   [20] ;		/* File Name Identifier [18 char]. */
char calvolid [8] ;		/* Calibrated volume name. */
char caldsid  [8] ;		/* Calibrated dataset name. */
int calyear ;			/* Year when image was calibrated [year-1900].*/
int caldoy ;			/* Day-of-Year when image was calibrated. */
char namea0  [8] ;		/* NAMA0 [6 char]. */
char namea1  [8] ;		/* NAMA1 [6 char]. */
char namea2  [8] ;		/* NAMA2 [6 char]. */
char namea3  [8] ;		/* NAMA3 [6 char]. */
char namecor [8] ;		/* Geometric Correction Array Name. */
char namestr [8] ;		/* Stray Light mask name. */
char namevig [8] ;		/* Vignetting mask name. */
int fwmag ;			/* FW magnification [x10,000]. */
int pwcorx ;			/* Polaroid Shift [x10]. */
int pwcory ;			/* Polaroid Shift [x10]. */
int smcorx ;			/* Sector Mirror Shift [x10]. */
int smcory ;			/* Sector Mirror Shift [x10]. */
int fwcorx ;			/* Filter Shift [x10]. */
int fwcory ;			/* Filter Shift [x10]. */
int cxcen ;			/* Computed Disk Center [x10]. */
int cycen ;			/* Computed Disk Center [x10]. */
int dxcen ;			/* Standard Disk Center [x10]. */
int dycen ;			/* Standard Disk Center [x10]. */
int sxcen ;			/* Standard Sun  Center [x10]. */
int sycen ;			/* Standard Sun  Center [x10]. */
int escorr ;			/* Equivalent Second Factor [x10,000]. */
int radfac ;			/* Calflux Radiance Factor, x1.e+11. */
int calexp ;			/* Calibration scale Factor, exp of 10 [=11]. */
int datatype ;			/* Data Type. */
int lcc ;			/* Calibration Correction flag. */
int lfc ;			/* F-Corona Flag. */
int lreg ;			/* Registration Flag. */
int lvig ;			/* Vignetting Flag. */
int lstr ;			/* Stray Light Flag. */
};

/*** hskp.h :   housekeeping structure for cphskp.h  ***/

struct hskp
   {
   char  time [22], filename [18];
   char  sec [6], res [3], fil [7], glc [5], pol [4], cal [4];
   int   year, doy, month, day, hour, minute, second;	/* date & time. */
   int   frame, quan, roll;	/* frame counter, quantity, S/C roll. */
   int   exp, ih, iv, ires;	/* exposure, hor/ver, resolution. */
   int   ifw, ipw, ism, icm;	/* filter, polaroid, sector, cal mir. */
   int   iglpos, iglstw;	/* 5303 mechanism position + stow status. */
   int   ip, ir, lp, lr;	/* first & last pixel & row. */
   int   nbypix, idtype;	/* #bytes/pixel, data type. */
   int   str;			/* Stray light removed ? =0 no, =1 yes. */
   int   vig;			/* Vignetting removed ?  =0 no, =1 yes. */
   int   reg;			/* Disk registered ?     =0 no, =1 yes. */
   int   f;			/* F-corona removed ?    =0 no, =1 yes. */
   int   k;			/* K-corona removed ?    =0 no, =1 yes. */
   int   bit5;			/* not used. */
   int   bit6;			/* not used. */
   int   cc;			/* Munro cal.cor. applied? =0 no, =1 yes. */
   float xcen, ycen;		/* Disk center (x,y). */
   float degnor;		/* CCW angle of north w.r.t.up vertical axis. */
   float pixrs;			/* # pixels per solar radius. */
   float rsun;			/* Solar radius (arcsec). */
   float pscale;		/* Photometric scaling factor for pixel data. */
   };


#ifdef START

struct hskp H;
struct cpheader CPH ;

/*** cpgraf.h ***/

int	igterm;	/* Graphics terminal ?  =0 no, =1 yes. */
int	lcc;  /* Calibration correction flag. */
int	ljc;  /* Jupiter calibration correction flag. */
int	lden; /* =0  => Intensity;  =1  => Density. */
int	lfc;  /* F-corona removal? =0 no, =1 yes. */
int	lstr; /* Stray light removal ? =0 no, =1 yes. */
int	lsv;  /* Vignetting/Stray Light removal? =0 no, =1 yes.*/
int	lvig; /* Vignetting removal ? =0 no, =1 yes. */
int	lreg; /* Do disk registration? =0 no, =1 yes. */
int	ldif; /* Do image difference ? =0 no, =1 yes. */
int	modeg;	/* Plot mode: =0 linear, =1 log. */
int	rawtocal; /* Convert raw data to calibrated values ? =0 no, =1 yes. */

/*** cpgrin.h ***/

int	igr;	/* Grinnell usage flag: =0 no, =1 yes. */
int	ngr;	/* Grinnell device [0,1]. */
int	nres;	/* Inverse resolution flag: =1 Lo, =2 Hi. */
int	imgch;	/* Image channel [0,1,2,3]. */
int	ovrch;	/* Overlay channel [8,9,10,11,12]. */
float	gres;	/* Grinnell inverse resolution for non-standard size images. */

/*** cpscale.h ***/

float	pixrs;	/* # Pixels per solar radius. */
float	rspix;	/* # Solar radii per pixel. */
float	degrad;	/* # Degrees per solar radius. */
float	raddeg; /* # Solar radii per degree. */

/*** cpsize.h ***/

int	bytord;			/* byte order: 0 -> Sun-4; 1-> DEC. */
int	npixl;			/* actual # pixels per vidicon line. */
int	npix;			/* original # pixels per vidicon line. */
int	nbypix;			/* # bytes per pixel. */
int	ip, ir, lp, lr;		/* Pixel & line limits for image. */
int	idtype;			/* Data type: 1 = raw video
					      2 = Equivalent seconds
					      3 = 10^-11 B/Bsun */
int	nave;			/* # points to average in scan. */
int	icyear;			/* Year of calibrated file creation. */
int	icdoy;			/* DOY of calibrated file creation. */
float	pscale;			/* Scale factor for pixel magnitude. */

/*** cpsun.h ***/

int	scroll;		/* Adjusted S/C roll. */
float	degnor;		/* CCW angle of north w.r.t. up vertical axis. */
float	xcen;		/* x-axis coordinate of sun center: hi/lo res pixels.*/
float	ycen;		/* y-axis coordinate of sun center: hi/lo res lines. */

/*** cpzero.h ***/

int year_zero;
int doy_zero;
int hour_zero;
int minute_zero;
int second_zero;

#else

extern struct hskp H;
extern struct cpheader CPH ;

/*** cpgraf.h ***/

extern int	igterm;	/* Graphics terminal ?  =0 no, =1 yes. */
extern int	lcc;	/* Calibration correction flag. */
extern int	ljc;	/* Jupiter calibration correction flag. */
extern int	lden;	/* =0  => Intensity;  =1  => Density. */
extern int	lfc;	/* F-corona removal? =0 no, =1 yes. */
extern int	lstr;	/* Stray light removal ? =0 no, =1 yes. */
extern int	lsv;	/* Vignetting/Stray Light removal? =0 no, =1 yes.*/
extern int	lvig;	/* Vignetting removal ? =0 no, =1 yes. */
extern int	lreg;	/* Do disk registration? =0 no, =1 yes. */
extern int	ldif;	/* Do image difference ? =0 no, =1 yes. */
extern int	modeg;	/* Plot mode: =0 linear, =1 log. */
extern int	rawtocal; /* Convert raw data to cal. values ? =0 no, =1 yes. */

/*** cpgrin.h ***/

extern int	igr;	/* Grinnell usage flag: =0 no, =1 yes. */
extern int	ngr;	/* Grinnell device [0,1]. */
extern int	nres;	/* Inverse resolution flag: =1 Lo, =2 Hi. */
extern int	imgch;	/* Image channel [0,1,2,3]. */
extern int	ovrch;	/* Overlay channel [8,9,10,11,12]. */
extern float	gres;	/* Grinnell inverse resolution [non-std size images]. */

/*** cpscale.h ***/

extern float	pixrs;	/* # Pixels per solar radius. */
extern float	rspix;	/* # Solar radii per pixel. */
extern float	degrad;	/* # Degrees per solar radius. */
extern float	raddeg; /* # Solar radii per degree. */

/*** cpsize.h ***/

extern int	bytord;		/* byte order: 0 -> Sun-4; 1-> DEC. */
extern int	npixl;		/* actual # pixels per vidicon line. */
extern int	npix;		/* original # pixels per vidicon line. */
extern int	nbypix;		/* # bytes per pixel. */
extern int	ip, ir, lp, lr;	/* Pixel & line limits for image. */
extern int	idtype;		/* Data type: 1 = raw video
				      2 = Equivalent seconds
				      3 = 10^-11 B/Bsun */
extern int	nave;		/* # points to average in scan. */
extern int	icyear;		/* Year of calibrated file creation. */
extern int	icdoy;		/* DOY of calibrated file creation. */
extern float	pscale;		/* Scale factor for pixel magnitude. */

/*** cpsun.h ***/

extern int	scroll;	/* Adjusted S/C roll. */
extern float	degnor;	/* CCW angle of north w.r.t. up vertical axis. */
extern float	xcen;	/* x-axis coordinate of sun center: hi/lo res pixels.*/
extern float	ycen;	/* y-axis coordinate of sun center: hi/lo res lines. */

/*** cpzero.h ***/

extern int year_zero;
extern int doy_zero;
extern int hour_zero;
extern int minute_zero;
extern int second_zero;

#endif
