/* acoshskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill MLSO/ACOS housekeeping structure
 * using information in CHIP header structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    22 November 1995
 * 15 February 1996: add ra & dec.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
/* #define	ACOS */
/*#include "acoslib.ext" */

#include "acos.h"
#include "acoslib.h"

acoshskp (namimg, pmode)

char namimg [] ;			/* CHIP filename. */
int pmode ;				/* Print mode: 0=silent, 1=verbose. */

{
   FILE *FD ;
   int doy ;
   int i ;
   int ipos ;
   int idrv ;
   int wcode, scode, rcode ;
   float bandpass = 0.0 ;

   static char namdev [13][12] =
      {"FILSTAGE","O2-STAGE","CHIPCOVR","DIFFUSER","CHIPSTRB",
       "PMLKSTRB","OCC-LIM","PMLKCOVR","POLSTAGE","FIL-WHL",
       "POL-WHL","OBJ-WHL","OCC-WHL"} ;

   static char nampos [13][8][12] =
      {
      "STOP",       "IN",         "OUT",        "ERROR",
      "?",          "?",          "?",          "?",
      "UNKNOWN",    "LOWK",       "DPMON",      "ERROR", 
      "?",          "?",          "?",          "?",
      "UNKNOWN",    "IN",         "OUT",        "ERROR",
      "?",          "?",          "?",          "?",
      "UNKNOWN",    "IN",         "OUT",        "ERROR",
      "?",          "?",          "?",          "?",
      "NONE",       "CHIP",       "ILLEGAL",    "ILLEGAL",
      "?",          "?",          "?",          "?",
      "NONE",       "DPMON/LOWK", "ILLEGAL",    "ILLEGAL",
      "?",          "?",          "?",          "?",
      "UNKNOWN",    "CW LIMIT",   "CCW LIMIT",  "ERROR",
      "?",          "?",          "?",          "?",
      "UNKNOWN",    "IN",         "OUT",        "ERROR",
      "?",          "?",          "?",          "?",
      "UNKNOWN",    "IN",         "OUT",        "ERROR",
      "?",          "?",          "?",          "?",
      "658.1",      "654.0",      "DARK",       "DARK",
      "DARK",       "655.5",      "656.3",      "657.1",
      "0",          "45",         "90",         "135",
      "180",        "225",        "270",        "315",
      "CLEAR",      "DIFFUSER",   "OPAL",       "POLARIZER",
      "ERROR",      "ERROR",      "ERROR",      "ERROR",
      "WEDGE",      "LOWK",       "CLEAR",      "OFFSET",
      "RETICULE",   "OCCULT",     "ERROR",      "ERROR" 
      } ;

   static char namdrv [13][4][12] =
      {
      "STOP",  "IN",    "OUT",   "STOP",
      "STOP",  "LOWK",  "DPMON", "STOP",
      "STOP",  "IN",    "OUT",   "STOP",
      "STOP",  "IN",    "OUT",   "STOP",
      "?",     "?",     "?",     "?",   
      "?",     "?",     "?",     "?",   
      "?",     "?",     "?",     "?",   
      "STOP",  "IN",    "OUT",   "STOP",
      "STOP",  "IN",    "OUT",   "STOP",
      "STOP",  "CW",    "CCW",   "STOP",
      "STOP",  "CW",    "CCW",   "STOP",
      "STOP",  "CW",    "CCW",   "STOP",
      "STOP",  "CW",    "CCW",   "STOP"
      } ;

   /*** Open ACOS file. ***/

   if ((FD = fopen (namimg, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      return (-1) ;
      }

   /*** Read ACOS header. ***/

/*   if (fread ((char *) ACOSH, sizeof (ACOSH), 1, FD) == NULL) */

   if (fread ((char *) &ACOSHS, sizeof (ACOSHS), 1, FD) == NULL)
      {
      fprintf (stderr, "Unable to read header structure in file %s\n", namimg) ;
      return (-1) ;
      }

   /*** Print header. ***/

   if (pmode)
      {
      fprintf (stdout, "ACOSH->Version:		%10d\n",	ACOSH->Version) ;
      fprintf (stdout, "ACOSH->Year   :		%10d\n",	ACOSH->Year   ) ;
      fprintf (stdout, "ACOSH->Month  :		%10d\n",	ACOSH->Month  ) ;
      fprintf (stdout, "ACOSH->Day    :		%10d\n",	ACOSH->Day    ) ;
      fprintf (stdout, "ACOSH->Hour   :		%10d\n",	ACOSH->Hour   ) ;
      fprintf (stdout, "ACOSH->Minute :		%10d\n",	ACOSH->Minute ) ;
      fprintf (stdout, "ACOSH->Sec    :		%10d\n",	ACOSH->Sec    ) ;
      fprintf (stdout, "ACOSH->Xdim   :		%10d\n",	ACOSH->Xdim   ) ;
      fprintf (stdout, "ACOSH->Ydim   :		%10d\n",	ACOSH->Ydim   ) ;
      fprintf (stdout, "ACOSH->Nsum   :		%10d\n",	ACOSH->Nsum   ) ;
      fprintf (stdout, "ACOSH->Exposure:	%10d\n",	ACOSH->Exposure) ;
      fprintf (stdout, "ACOSH->Gain   :		%10d\n",	ACOSH->Gain   ) ;
      fprintf (stdout, "ACOSH->Odisk  :		%10d\n",	ACOSH->Odisk  ) ;
      fprintf (stdout, "ACOSH->Type   :		%10d\n",	ACOSH->Type   ) ;
      fprintf (stdout, "ACOSH->Wave   :		%10.4f\n",	ACOSH->Wave   ) ;
      fprintf (stdout, "ACOSH->Pangle :		%10.4f\n",	ACOSH->Pangle ) ;
      fprintf (stdout, "ACOSH->Bangle :		%10.4f\n",	ACOSH->Bangle ) ;
      fprintf (stdout, "ACOSH->Langle :		%10.4f\n",	ACOSH->Langle ) ;
      fprintf (stdout, "ACOSH->RA     :		%10.4f\n",	ACOSH->RA     ) ;
      fprintf (stdout, "ACOSH->Dec    :		%10.4f\n",	ACOSH->Dec    ) ;
      fprintf (stdout, "ACOSH->HA     :		%10.4f\n",	ACOSH->HA     ) ;
      fprintf (stdout, "ACOSH->Elevation:	%10.4f\n",	ACOSH->Elevation) ;
      fprintf (stdout, "ACOSH->Azimuth:		%10.4f\n",	ACOSH->Azimuth) ;
      fprintf (stdout, "ACOSH->Amass  :		%10.4f\n",	ACOSH->Amass  ) ;
      fprintf (stdout, "ACOSH->Radius :		%10.4f\n",	ACOSH->Radius ) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Xshift [%d]:	%10.3f\n",
                           i, ACOSH->Xshift [i]) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Yshift [%d]:	%10.3f\n",
                           i, ACOSH->Yshift [i]) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Diam [%d]:	%10.3f\n",
                           i, ACOSH->Diam [i]) ;
      for (i = 0; i < 32; i++)
         fprintf (stdout, "ACOSH->Pos [%d]:	%10d\n",
                           i, ACOSH->Pos [i]) ;
      for (i = 0; i < 32; i++)
         fprintf (stdout, "ACOSH->Drive [%d]:	%10d\n",
                           i, ACOSH->Drive [i]) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Analog [%d]:	%10.3f\n",
                           i, ACOSH->Analog [i]) ;
      for (i = 0; i <  8; i++)
         fprintf (stdout, "ACOSH->Lcvr [%d]:	%10.3f\n",
                           i, ACOSH->Lcvr [i]) ;
      for (i = 0; i <  6; i++)
         fprintf (stdout, "ACOSH->Mic [%d]:	%10.3f\n",
                           i, ACOSH->Mic [i]) ;
      fprintf (stdout, "\n") ;
      }

   /*** Fill housekeeping structure. ***/

   bandpass = 0.0 ;				/* Nanometers. */

   wcode =  ACOSI->dtype / 100 ;
   scode = (ACOSI->dtype - wcode * 100) / 10 ;
   rcode =  ACOSI->dtype - wcode * 100 - scode * 10 ;

   if (ACOSH->Version == IS_DPMON)
      {
      if (scode == 2)
         bandpass = 0.80 ;			/* Solar Limb [Nanometers]. */
      else
         bandpass = 0.05 ;			/* Solar Disk [Nanometers]. */
      }
   else
   if (ACOSH->Version == IS_LOWK)
      bandpass = 0.80 ;				/* Nanometers. */
   else
   if (ACOSH->Version == IS_CHIP)
      bandpass = 0.14 ;				/* Nanometers. */

   ACOSI->swvers        = 1.00 ;
   ACOSI->ttype         = ACOSH->Version ;
   ACOSI->dtype         = ACOSH->Type ;
   ACOSI->xdim		= ACOSH->Xdim ;
   ACOSI->ydim		= ACOSH->Ydim ;
   ACOSI->zdim		= 1 ;
   ACOSI->nbypx		= sizeof (short) ;
   ACOSI->dmin		= 0 ;
   ACOSI->dmax		= 0 ;
   ACOSI->smin		= 0 ;
   ACOSI->smax		= 0 ;
   ACOSI->year		= ACOSH->Year ;
   if (ACOSI->year < 1900)   ACOSI->year += 1900 ;
   ACOSI->month		= ACOSH->Month ;	/*0=Jan in ACOSH->Month*/
   ACOSI->day		= ACOSH->Day ;
   doy = day_of_year (ACOSI->year, ACOSI->month, ACOSI->day) ;
   ACOSI->doy		= doy ;
   ACOSI->hour		= ACOSH->Hour ;
   ACOSI->minute	= ACOSH->Minute ;
   ACOSI->second	= ACOSH->Sec ;
   ACOSI->cal_year	= 0 ;
   ACOSI->cal_month	= 0 ;
   ACOSI->cal_day	= 0 ;
   ACOSI->navg		= ACOSH->Nsum ;
   ACOSI->expdur	= (float) ACOSH->Exposure / 1000.0 ;	/* seconds. */
   ACOSI->gain		= (float) ACOSH->Gain ;
   ACOSI->odisk		= (float) ACOSH->Odisk ;
   ACOSI->bscale	= 1.0 ;
   ACOSI->bzero		= 0.0 ;
   ACOSI->pixrs		= 0.0 ;
   ACOSI->pangle	= ACOSH->Pangle ;
   ACOSI->bangle	= ACOSH->Bangle ;
   ACOSI->langle	= ACOSH->Langle ;
   ACOSI->ra		= ACOSH->RA ;
   ACOSI->dec		= ACOSH->Dec ;
   ACOSI->hangle	= ACOSH->HA ;
   ACOSI->nangle	= 90.0 ;
   ACOSI->elev		= ACOSH->Elevation ;
   ACOSI->azimuth	= ACOSH->Azimuth ;
   ACOSI->airmass	= ACOSH->Amass ;
   ACOSI->rsun		= ACOSH->Radius ;	/* Solar Radius (arcsec). */
   ACOSI->xcen		= 0.0 ;
   ACOSI->ycen		= 0.0 ;
   ACOSI->rmin		= 0.0 ;
   ACOSI->rmax		= 0.0 ;
   ACOSI->wavelen       = ACOSH->Wave ;		/* Central Wavelength (nm). */
   ACOSI->wavewid       = bandpass ;		/* Filter bandpass (nm). */

   for (i = 0; i < 13; i++)
      {
      ipos = ACOSH->Pos [i] ;
      idrv = ACOSH->Drive [i] ;

      strcpy (&ACOSI->optdev [i][0], &namdev [i][0]) ;		/* Dev. Name. */
      ACOSI->optdev [i][15] = '\0';
      if (ipos >= 0 && ipos <= 7)
         strcpy (&ACOSI->optpos [i][0], &nampos [i][ipos][0]) ;	/* Position. */
      else
	 strcpy (&ACOSI->optpos [i][0], "???") ;
      ACOSI->optpos [i][15] = '\0';

      if (idrv >= 0 && idrv <= 3)
         strcpy (&ACOSI->optdrv [i][0], &namdrv [i][idrv][0]);	/* Drive Req. */
      else
         strcpy (&ACOSI->optdrv [i][0], "???");
      ACOSI->optdrv [i][15] = '\0';
      }

   /*** Print housekeeping. ***/

   if (pmode)
      {
      fprintf (stdout, "ACOSI->swvers: 		%10d\n",	ACOSI->swvers) ;
      fprintf (stdout, "ACOSI->dtype:		%10d\n",	ACOSI->dtype) ;
      fprintf (stdout, "ACOSI->xdim:		%10d\n",	ACOSI->xdim) ;
      fprintf (stdout, "ACOSI->ydim:		%10d\n",	ACOSI->ydim) ;
      fprintf (stdout, "ACOSI->zdim:		%10d\n",	ACOSI->zdim) ;
      fprintf (stdout, "ACOSI->nbypx:		%10d\n",	ACOSI->nbypx) ;
      fprintf (stdout, "ACOSI->dmin:		%10d\n",	ACOSI->dmin) ;
      fprintf (stdout, "ACOSI->dmax:		%10d\n",	ACOSI->dmax) ;
      fprintf (stdout, "ACOSI->smin:		%10d\n",	ACOSI->smin) ;
      fprintf (stdout, "ACOSI->smax:		%10d\n",	ACOSI->smax) ;
      fprintf (stdout, "ACOSI->year:		%10d\n",	ACOSI->year) ;
      fprintf (stdout, "ACOSI->month:		%10d\n",	ACOSI->month) ;
      fprintf (stdout, "ACOSI->day:		%10d\n",	ACOSI->day) ;
      fprintf (stdout, "ACOSI->doy:		%10d\n",	ACOSI->doy) ;
      fprintf (stdout, "ACOSI->hour:		%10d\n",	ACOSI->hour) ;
      fprintf (stdout, "ACOSI->minute:		%10d\n",	ACOSI->minute) ;
      fprintf (stdout, "ACOSI->second:		%10d\n",	ACOSI->second) ;
      fprintf (stdout, "ACOSI->cal_year:	%10d\n",	ACOSI->cal_year) ;
      fprintf (stdout, "ACOSI->cal_month:	%10d\n",	ACOSI->cal_month) ;
      fprintf (stdout, "ACOSI->cal_day:		%10d\n",	ACOSI->cal_day) ;
      fprintf (stdout, "ACOSI->navg:		%10d\n",	ACOSI->navg) ;
      fprintf (stdout, "ACOSI->expdur:		%10.4f\n",	ACOSI->expdur) ;
      fprintf (stdout, "ACOSI->gain:		%10.4f\n",	ACOSI->gain) ;
      fprintf (stdout, "ACOSI->odisk:		%10.4f\n",	ACOSI->odisk) ;
      fprintf (stdout, "ACOSI->bscale:		%10.4f\n",	ACOSI->bscale) ;
      fprintf (stdout, "ACOSI->bzero:		%10.4f\n",	ACOSI->bzero) ;
      fprintf (stdout, "ACOSI->pixrs:		%10.4f\n",	ACOSI->pixrs) ;
      fprintf (stdout, "ACOSI->pangle:		%10.4f\n",	ACOSI->pangle) ;
      fprintf (stdout, "ACOSI->bangle:		%10.4f\n",	ACOSI->bangle) ;
      fprintf (stdout, "ACOSI->langle:		%10.4f\n",	ACOSI->langle) ;
      fprintf (stdout, "ACOSI->hangle:		%10.4f\n",	ACOSI->hangle) ;
      fprintf (stdout, "ACOSI->nangle:		%10.4f\n",	ACOSI->nangle) ;
      fprintf (stdout, "ACOSI->elev:		%10.4f\n",	ACOSI->elev) ;
      fprintf (stdout, "ACOSI->azimuth:		%10.4f\n",	ACOSI->azimuth) ;
      fprintf (stdout, "ACOSI->airmass:		%10.4f\n",	ACOSI->airmass) ;
      fprintf (stdout, "ACOSI->rsun:		%10.4f\n",	ACOSI->rsun) ;
      fprintf (stdout, "ACOSI->xcen:		%10.4f\n",	ACOSI->xcen) ;
      fprintf (stdout, "ACOSI->ycen:		%10.4f\n",	ACOSI->ycen) ;
      fprintf (stdout, "ACOSI->rmin:		%10.4f\n",	ACOSI->rmin) ;
      fprintf (stdout, "ACOSI->rmax:		%10.4f\n",	ACOSI->rmax) ;
      fprintf (stdout, "ACOSI->wavelen:		%10.4f\n",	ACOSI->wavelen) ;
      fprintf (stdout, "ACOSI->wavewid:		%10.4f\n",	ACOSI->wavewid) ;

/*
      for (i = 0; i < 13; i++)
	 {
         fprintf (stdout, "ACOSI->optdev[%d]: %-16s	ACOSI->optpos[%d]: %s	ACOSI->optdrv[%d]: %s\n",
		  i, &ACOSI->optdev [i][0], i, &ACOSI->optpos[i][0]), &ACOSI->optdrv[i][0] ;
	 }
 */
      }

   return (0) ;
}
