/*------------------------------------------------------------------------------
 * I S S O M E T H I N G . C  - Collection of routines useful in discovering the
 *				identity of c strings and characters.
 *
 *	isalpha(c)   - return true if input character (c) is alphabetic.
 *	isascii(c)   - return true if input character (c) is ascii ( < 0x80 ).
 *	iscntrl(c)   - return true if input character (c) is control character.
 *	islower(c)   - return true if input character (c) is lower case.
 *	isdigit(c)   - return true if input character (c) is a digit.
 *	isspace(cc)  - return true if input character is space, newline or tab.
 *      lower(s)     - converts string s to lower case.
 *      upper(s)     - converts string s to upper case.
 *      toupper(c)   - converts character c to upper case.
 *      tolower(c)   - converts character c to lower case.
 *------------------------------------------------------------------------------
 */


/* Return true if input character is alphabetic. */

isalpha (c)

unsigned char c;
{
    return (('A'<=c && c<='Z') || ('a'<=c && c<='z'));
}

/* Return true if input character is less than 0x80. */

isascii (c)

unsigned char c;
{
    return (c < 0x80);
}

/* Return true if input character is a a control character. */

iscntrl (c)

unsigned char c;
{
    return (0177 == c || c <= 037);
}

/* Return true (!= 0)  if input character is a digit. */

isdigit (c)

unsigned char c;
{
    return ('0' <= c && c <= '9');
}

/* Return true (!= 0) if input character is lower case alphabetic. */

islower (c)

unsigned char c;
{
  return ('a' <= c && c <= 'z');
}

/* Returns true if chararcter is a blank,tab or newline. */

isspace (cc)

char cc;
{
  return (cc == ' ' || cc == '\t' || cc == '\n');
}

/* Return true if input character is upper case alphabetic. */

isupper (c)

unsigned char c;
{
  return ('A' <= c && c <= 'Z');
}

/* Convert a string to lower case. */

unsigned char *lower (cp)

unsigned char *cp;
{
  unsigned char *ct;

  for (ct = cp; *ct; ++ct)
      if (*ct >= 'A' && *ct <= 'Z')
	  *ct += ('a' - 'A');
  return cp;
}

/* Convert character to lowercase if upper case. */

tolower (c)

char c;
{
  if ('A' <= c && c <= 'Z')
      c += ('a' - 'A');
  return c;
}

/* Convert character to uppercase if lower case. */

toupper (c)

char c;
{
  if ('a' <= c && c <= 'z')
      c += ('A' - 'a');
  return c;
}

/* Convert a string to upper case. */

unsigned char *upper (cp)

unsigned char *cp;
{
  unsigned char *ct;

  for (ct = cp; *ct; ++ct)
      if (*ct >= 'a' && *ct <= 'z') * ct += ('A' - 'a');
  return cp;
}

