/* cpclean.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Interpolate over reseau dots and bright points in a sequence of C/P images.
 * Also interpolate over beam blemishes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  7 June 1993: UNIX version; no screen display.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/local/home/stanger/lib/include/cplib.h"

main (argc, argv)

int argc;
char *argv [];

{
   extern char *malloc ();
   extern unsigned char *rdfimg ();
   extern char *outname ();
   FILE *LIST;

   static char namimg [80];	/* Input image name. */
   static char namlst [80];	/* Input filelist name. */
   static char namout [80];	/* Output image name. */
   static char perimtype [12] = "none";	/* Perimeter type (ASCII). */

   unsigned char *cpbuf;	/* array containing image file. */
   unsigned char *ibuf;		/* array containing image pixels only. */

   int fd;			/* file descriptor. */
   int i;			/* index. */
   int lblem = 1;		/* blemish removal flag. */
   int lradius = 30;		/* Default radius. */
   int lwidth = 30;		/* Default width. */
   int nbyimg;			/* # bytes in image. */
   int nbypx;			/* # bytes per pixel. */
   int nhdrec;			/* # logical records in header. */
   int npximg;			/* # pixels in image. */
   int nwrite;			/* # bytes written to file. */
   int nx, ny;			/* width & height of image. */
   int ptype = 2;		/* Perimeter type:1=horiz, 2=vert, 3=pinwheel */
   int rh = -1;			/* Hole radius (pixel units). */
   int seq = 0;			/* Flag to indicate sequence of images. */
   int width = -1;		/* Width of vertical blemish (pixels). */
   int xh = 133, yh = 342 ;	/* Hole center coordinates (pixel units). */

   if (argc == 2)
      strcpy (namimg, argv [1]) ;
   else
   if ((argc > 2) && *(argv [1]) == '-')
      {
      for (i  = 1; i < strlen (argv [1]); i++)
	 {
	 if (*(argv [1] + i) == 'n')
	    lblem = 0;
         else
	 if (*(argv [1] + i) == 'w')
	    {
            sscanf (argv [1+i], "%d", &width) ;
	    }
	 else
	 if (*(argv [1] + i) == 'p')
	    {
            strcpy (perimtype, argv [1+i]);
	    }
	 else
	 if (*(argv [1] + i) == 'r')
            {
            sscanf (argv [1+i], "%d", &rh) ;
	    }
	 }
      strcpy (namimg, argv [argc-1]) ;
      }
   else
      {
      fprintf (stderr, "%s: program to clean up SMM C/P images (no screen display provided).\n", argv [0]);
      fprintf (stderr, "Removes dark reseau dots, pinholes (bright points), and beam blemishes.\n");
      fprintf (stderr, "\n");
      fprintf (stderr, "usage: %s [-nwpr] [line_width] [perim_type] [hole_radius] cp_filename | @cp_filelist\n", argv [0]);
      fprintf (stderr, "\n");
      fprintf (stderr, "NO beam blemish removal option:\n");
      fprintf (stderr, "   -n              do NOT interpolate over beam blemishes.\n");
      fprintf (stderr, "\n");
      fprintf (stderr, "beam blemish removal options:\n");
      fprintf (stderr, "   -w line_width   width of vertical line (pixels)\n");
      fprintf (stderr, "   -p perim_type   bullet hole interpolation type\n");
      fprintf (stderr, "                   h [horizontal]\n");
      fprintf (stderr, "                   v [vertical]\n");
      fprintf (stderr, "                   p [pinwheel]\n");
      fprintf (stderr, "   -r hole_radius  radius of bullet hole (pixels)\n");
      exit (0) ;
      }

   /*** Set defaults if options not specified. ***/

   if (rh < 0)     rh = lradius;
   if (width < 0)  width = lwidth;
   if (strcmp (perimtype, "none") == 0) ptype = 2;

   if (perimtype [0] == 'h')  ptype = 1;
   if (perimtype [0] == 'v')  ptype = 2;
   if (perimtype [0] == 'p')  ptype = 3;

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

loop:

   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }
   strcpy (namout, outname (namimg));
   fprintf (stdout, "%s\n", namout);

   /*** Decode C/P image housekeeping & read image into array 'cpbuf'. ***/

   cpbuf = rdfimg (namimg);	/* Read hskp + bitmap + image data. */
   nbypx = nbypix;
   nx = npix;
   ny = npix;
   npximg = npix * npix;

   if (nbypix != 1)
      {
      fprintf (stderr, "*** Only raw C/P images are allowed. ***\n");
      exit (-1);
      }

   /*** 'ibuf' points to beginning of image pixel data. ***/

   if (nbypx == 4)
      nhdrec = 1;
   else
      nhdrec = 2;

   ibuf = cpbuf + npix * nhdrec;

   /*** Remove dark points (reseau dots). ***/

   dkrms (ibuf, nbypx, nx, ny);

   /*** Remove bright points (pin holes). ***/

   bprms (ibuf, nbypx, nx, ny);

   /*** Remove beam blemishses (vertical & diagonal lines). ***/

   if (lblem == 1)
      blrms (ibuf, nbypx, nx, ny, width);

   /*** Perform circular interpolation over bullet hole. ***/

   cirint (ibuf, nbypx, nx, ny, rh, xh,   yh, ptype);
   cirint (ibuf, nbypx, nx, ny, rh, xh+1, yh, ptype);

   cirint (ibuf, nbypx, nx, ny, 15, 158, 343, ptype);
   cirint (ibuf, nbypx, nx, ny, 15, 159, 343, ptype);

   /*** Write image to disk. ***/

   nbyimg = (nx + nhdrec) * ny * nbypix;
   fd = creat (namout, 0644);
   nwrite = write (fd, cpbuf, nbyimg);
   if (nwrite != nbyimg)
      {
      fprintf (stderr, "%s: Write error; nwrite (%d) != nbyimg (%d)\n",
	       namimg, nwrite, nbyimg);
      exit (-2);
      }
   close (fd);
   free (cpbuf);

   if (seq)  goto loop;

quit:

   exit (0);

}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

char *
outname (namfil)

char namfil [];

{
   static char newname [80];
   int i, idot;

   strcpy (newname, namfil);

   i = -1;
   for (i = 0; i < strlen (namfil); i++)
      {
      if (namfil [i] == '.')
	 {
	 idot = i;
	 break;
	 }
      }

   if (idot < 0)
      {
      strcat (namfil, ".");
      idot = strlen (namfil) - 1;
      }

   newname [idot + 1] = 'g';

   return (newname);
}
