/* fixcal.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fixcal (rsun, nr, data, ndata)                                    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Applies Munro calibration correction to array 'data'.
 *----------------------------------------------------------------------
 * rsun  = array of radial values (units of Rsun). 
 * nr = # elements in rsun array (=1 for radial scan, =ndata for theta scan).
 * data  = array of intensity values (units of 10**-11 B/Bsun).
 * ndata = # elements in array data.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 26 July 1984
 *----------------------------------------------------------------------
 */

#include <stdio.h>

fixcal (rsun, nr, data, ndata)

int nr, ndata;
float rsun[];
float data[];

{

int i;
int ierr ;
float r;
float cc;
extern float calcor ();

   if (nr > 1 && nr < ndata)
      {
      fprintf (stderr,"# rsun values:%d less than # data values:%d\n",
               nr, ndata);
      ierr = -1 ;
      }

   else

      {

      /*** Radial scans. ***/

      if (nr > 1)
         {
         for (i=0; i<ndata; i++)
             {
             data [i] *= calcor (rsun [i]);
             }
         ierr = 0 ;
         }

      else

      /*** Theta scans. ***/

         if (nr == 1)
            {
            r = rsun [0];
            for (i=0; i<ndata; i++)  data [i] *= calcor (r);
            ierr = 0 ;
            }
      }

   return (ierr) ;
}
