/* cphskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decodes housekeeping and puts result into stucture H as defined
 * in file "hskp.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Originally written by Rainer Illing   13 December 1983
 * Revised by Andrew Stanger  28 December 1983
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Updated to compute ip,ir,lp,lr and look up disk center:  27 January 1984
 * Add definition of scaling variables:  30 January 1984.
 * Open and close image locally:  22 February 1984.
 * Remove call to 'hkshow':  24 February 1984
 * Rename include files:  27 February 1984.
 * Read into character array, then copy to integer array: 20 March 1984.
 * Change open, read, close to  fopen, fread, fclose: 25 April 1984.
 * Revise housekeeping structure: 25 October 1984.
 * 28 November 1984: Add roll.
 * 3 December 1984: Modify green line configuration (H.glc).
 *                  NESW sector designation.
 * 20 March 1985: add extension to 'H.filename'.
 * 21 May 1985: extract icyear and icdoy and add to file 'cpsize.h'.
 * 3 June 1985: Use "malloc" to allocate temporary storage for 'h[]'.
 *              Execute "free (c)" after 'c[]' is copied to 'h[]'.
 * 5 June 1985: If icyear < 80, set icyear = 0, icdoy = 0;
 * 7 November 1986: Determine nbypix from nbtpix (h[103]).
 * 11 November 1986: Extract 'npixl' from bytes 110 & 111.
 *                   Adjust disk center and geometric scale factor accordingly.
 * 14 November 1986: Add 'gres' for display of non-standard size images.
 * 17 November 1986: Add 'pscale': pixel magnitude scale factor.
 * 20 November 1986: idtype = 4 represents electron density data.
 * 5 December 1986: Compute 'scroll': roll adjusted to closest 10 degrees.
 * 11 December 1986: Set variables in 'cpgraf.h' by decoding hskp byte 221.
 * 17 December 1986: idtype=4 => floating point data.  lden=2 => density units.
 * 18 December 1986: For CRAY-calibrated data, set idtype from file name ext.
 *  6 September 1989: H.filename: "yydddhhmm.vxx".
 * 19 September 1989: Roll is computed differently for 1980 & 1984-89.
 *  1 December 1989: Use bytes 88 and 89 for roll (if they are not 127 or 128);
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define	MAXHSK	224
#define	ZC	'\0'

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

cphskp (namimg)	/* SMM C/P Housekeeping Decoder */

char	namimg[];	/* Image file name. */

{

/*** Arrays for housekeeping entries. ***/

extern float pow10();
static	char	namres[] = "HI\0LO\0";
static	char	namfil[] = "RED   \0HALPHA\0CLEAR \0BLUE  \0WIDEBD\0GREEN \0";
static	char	namsec[] =
		"sw   \0s    \0se   \0e    \0ne   \0n    \0nw   \0w    \0";
static	char	namsmf[8][3] = {"sw","ss","se","ee","ne","nn","nw","ww"};
static	char	nampol[] = "CLR\0+60\0-60\00dg\0";
static	char	namgls[] = "m\0a\0s\0f\0";
static	char	namcal[] = "MOV\0OUT\0IN \0BAD\0";
static	char	cglstw[2];	/* Storage for green line stow status. */
static	int	lih[4], liv[4];	/* Binary components of IH & IV. */
static	char	namtype[4] = {'v', 't', 'c', '\0'};

/*** Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). ***/
/* (No geometric correction applied.) */

static	float	vidcen[9][2] = { 36.3, 499.8,	/* sw */
				213.1, 193.6,	/* s  */
				539.9,   3.3,	/* se */
				820.0,  87.0,	/* e  */
				892.4, 361.5,	/* ne */
				718.6, 679.3,	/* n  */
				386.7, 856.4,	/* nw */
				104.3, 792.1,	/* w  */
				447.0, 447.0};	/* n+e+s+w */

/*** Disk centers (vs. Sector Mirror) for calibrated data (C34520). ***/
/* (Processed with geometric correction WW6COR). */

static	float	ww6cen[9][2] = { 47.9, 507.5,	/* sw */
				222.2, 205.6,	/* s  */
				529.7,  40.3,	/* se */
				814.6,  85.4,	/* e  */
				884.0, 353.4,	/* ne */
				714.8, 671.1,	/* n  */
				393.2, 824.1,	/* nw */
				113.6, 786.4,	/* w  */
				447.0, 447.0};	/* n+e+s+w */

static	float	secrs = 961.;	/* arcseconds per solar radius. */
static	float	secpix = 5.933; /* arcseconds per pixel (Hi res). */

float	pi = 3.141592654;
float	radrol, rollwt;

int	icode;
int	iexp;
int	i, k;
int	ires, ipw, ifw, ism, icm, iglstw, iglpos;
int	roll1, roll2, roll, isign;
int	jsm;
int	kres, ihs, ivs;
int	lenimg, iext;
int	lres, nbtpix;

int	*h;		/* Pointer to housekeeping array (integer).*/
char	*c;		/* Pointer to housekeeping array (character). */
char	*malloc();	/* system allocation routine. */

FILE	*imgfd;		/* File pointer for image disk file. */
FILE	*fopen();	/* Type declaration for system functions. */
FILE	*fclose();	/* Type declaration for system functions. */
int	fread();	/* Type declaration for system function. */


/*** Open image file and read housekeeping information. ***/

   if ((imgfd = fopen (namimg,"r")) == NULL)  { fclose(imgfd); return(-1); }

/*** Allocate storage for housekeeping array. ***/

   if (((h = (int *) calloc (MAXHSK,sizeof(int))) == NULL) ||
      (c = malloc (MAXHSK)) == NULL) 
      {
      fprintf (stderr,"cphskp> malloc error.\n") ;
      fflush (stderr);
      fclose (imgfd);
      return (-2);
      }

/*** Read housekeeping from C/P image file into character array 'c'. ***/

   if (fread (c, sizeof(char), MAXHSK, imgfd) == NULL)
      {
      fclose (imgfd);
      free (c);
      free (h);
      return (-3);
      }
   fclose(imgfd);

/*** Copy housekeeping from character*1 to integer*4 array. ***/

   for (i = 0; i < MAXHSK; i++)  *(h + i) = (int) *(c + i) & 0377;
   iexp = (int) *(c+220);
   free (c);

/* IDTYPE: data type.
 *		=1 raw vidicon data (DN units, 8 bits).
 *		=2 Equivalent seconds * 5, 8 bits.
 *		=3 Calibrated data (10^-11 B/Bsun units, 16 bits). 
 * Data type is inferred from the first letter of the filename extension.
 *
 * A C/P filename has the following format :
 * ydddhhmms.vxx :	type 1
 *          .txx :	     2
 *          .cxx :	     3
 *          .dxx :           4
 * where y = year -1980.
 *       ddd = day-of-year, hh = hour, mm = minute, s = sec/10.
 *       x = arbitrary character.
 */

   lenimg = strlen (namimg);
   iext = -1;
   for (i=0; i<lenimg; i++) if (namimg[i] == '.') iext = i+1;

   nbypix = 2;
   idtype = 3;
   if (namimg[iext] == 'v' || namimg[iext] == 'V')  {idtype = 1; nbypix=1;}
   if (namimg[iext] == 't' || namimg[iext] == 'T')  {idtype = 2; nbypix=2;}
   if (namimg[iext] == 'd' || namimg[iext] == 'D')  {idtype = 4; nbypix=4;}

/*** Year and Day-of-Year. ***/

   H.year = *h;		/* year */

   H.doy = *(h+1)<<8;
   H.doy += *(h+2);	/* day-of-year */

/*** Polaroid. ***/

   ipw = (*(h+3) & 0300)>>6;
   H.ipw = ipw;
   for (i=0; i<4; i++) *(H.pol+i)= *(nampol+4*ipw+i);

/*** Resolution. ***/

   ires = (*(h+3) & 04)>>2;
   H.ires = ires;
   for (i=0; i<3; i++) *(H.res+i)= *(namres+3*ires+i);

/*** IH,IV. ***/

   H.ih = (*(h+9) & 0360)>>4;
   H.iv = (*(h+9) & 017);

   lih[0] = (H.ih & 010) >> 3;
   lih[1] = (H.ih &  04) >> 2;
   lih[2] = (H.ih &  02) >> 1;
   lih[3] = (H.ih &  01);

   liv[0] = (H.iv & 010) >> 3;
   liv[1] = (H.iv &  04) >> 2;
   liv[2] = (H.iv &  02) >> 1;
   liv[3] = (H.iv &  01);

/*** Exposure. ***/

   H.exp = (*(h+10) & 0377)<<4;
   H.exp += (*(h+11) & 0360)>>4;

/*** S/C roll. ***/

   roll1 = (int) (*(h+11) & 017);
   roll2 = (int) (*(h+12) & 0377);
   isign =  (roll1 & 8) / 8;

   /***... Least significant roll bit (smm#2) = 7.16197 degrees. ...***/

   if (H.year > 80)
      {
      roll = (int) ((roll1 & 7) * 4 + (roll2 & 0300) / 64);
      radrol = (float) roll;
      if (isign == 1)  radrol = - (float) (((!(roll)+1) & 037));
      rollwt = 7.16197;
     }

   /***... Least significant roll bit (smm#1) = .111906 degrees. ...***/

   if (H.year == 80)
      {
      roll = (int) ((roll1 & 7) * 256 + (roll2 & 0377));
      radrol = (float) roll;
      if (isign == 1)  radrol = - (float) (((!(roll)+1) & 03777));
      rollwt = 0.111906;
      }

   roll = (int) (radrol * rollwt);
   H.roll = roll;

/*** Set scroll to closest 10 degrees. ***/

   if (H.year > 80)
      {
      scroll = roll;
      if (scroll > 0)  scroll = roll + 7;
      scroll = (scroll / 10) * 10;
      }
   else
      scroll = roll;

   /*** Check bytes 88 and 89.
    *** If not 127 or 128, they contain roll in degree units (+/- 1 deg).
    ***/

   roll1 = (int) (*(h+88) & 0377);
   roll2 = (int) (*(h+89) & 0377);
   if ((roll1 != 127 && roll2 != 127) &&
       (roll1 != 128 && roll2 != 127) &&
       (roll1 != 128 && roll2 != 128) &&
       (roll1 != 127 && roll2 != 128))
      {
      roll = roll1 * 256 + roll2;
      if (roll > 360)  roll -= 360;
      if (roll <   0)  roll += 360;
      if (roll > 180)  roll -= 360;
      H.roll = roll;
      scroll = roll;
      }

/*** Green line position and stow. ***/

   iglpos = *(h+13) & 0377;
   H.iglpos = iglpos;

   iglstw = (*(h+14) & 0300)>>6;
   H.iglstw = iglstw;
   for (i=0; i<2; i++) *(cglstw+i)= *(namgls + 2*iglstw + i);

/*** Sector mirror. ***/

   jsm = (*(h+14) & 074)>>2;
   ism = (jsm-1)/2;		/* Decode to array position. */
   if (jsm == 0)  ism = 8;	/* Special case for NESW. */
   H.ism = ism;
   for (i=0; i<6; i++) *(H.sec+i)= *(namsec+6*ism+i);

/*** Calibration mirror. ***/

   icm = (*(h+15) & 060)>>4;
   H.icm = icm;
   for (i=0; i<4; i++) *(H.cal+i)= *(namcal+4*icm+i);

/*** Filter wheel. ***/

   ifw = (*(h+15) & 016)>>1;
   ifw = ifw - 1;			/* Decode to array position. */
   H.ifw = ifw;
   for (i=0; i<7; i++) *(H.fil+i)= *(namfil+7*ifw+i);

/*** Hours and minutes. ***/

   H.hour = *(h+20);
   H.minute = *(h+21);
   H.second = *(h+24);

/*** Image count (12-bit operational status). ***/

   H.frame = *(h+30) * 16 + (*(h+31) & 0360) / 16;

/*** Byte order: 0 -> Sun-4;  1 -> DEC. ***/

   bytord = 0;
   if (*(h+52) > 0)  bytord = 1;

/*** Quantity. ***/

   H.quan = *(h+79) & 0377;

/*** Bits per pixel. ***/

   if (H.year > 80)
      {
      nbtpix = *(h+103);
      nbypix = nbtpix / 8;
      }

/*** Actual # pixels per line. ***/

   npixl = (*(h+110) & 0377) * 256 + (*(h+111) & 0377);

/*** Convert date & time into characters [yydddhhmm]. ***/

   sprintf (H.filename, "%02d%03d%02d%02d",
            H.year, H.doy, H.hour, H.minute);
   sprintf (H.glc, "%03d%s", iglpos, cglstw);

/*** Compute start/end pixel/line coordinates for C/P image. ***/

   nres = 1;
   if (ires == 0)  nres = 2;
   kres = ires + 1;
   ip = 0;
   lp = 895 / kres;
   ir = 0;
   lr = 895 / kres;
   ihs = 0;
   ivs = 0;

   for (i=0; i<4; i++)
      {
      if (lih[i] == 0 && ihs == 0)  ip += 224 / kres;
      if (liv[i] == 0 && ivs == 0)  ir += 224 / kres;
      if (lih[i] == 1)  ihs = 1;
      if (liv[i] == 1)  ivs = 1;
      if (lih[i] == 0 && ihs == 1)  lp -= 224 / kres;
      if (liv[i] == 0 && ivs == 1)  lr -= 224 / kres;
      }

   H.ip = ip;
   H.ir = ir;
   H.lp = lp;
   H.lr = lr;
   npix = lp - ip + 1;
   lres = -1;
   gres = -1;
   if (npixl < 1 || npixl > 896)  npixl = npix;
   if (npixl > 0)
      {
      lres = npix / npixl;
      if (lres > 0)  gres = (float) nres / (float) lres;
      else {gres = 1.;   lres = 1;}
      }

/*** Decipher numerical code. ***/

   lstr = 0;
   lvig = 0;
   lreg = 0;
   lden = 0;
   icode = *(h+221) & 0377;
   if (idtype != 1 && H.year != 80)
      {
      if (icode & 1)  idtype = 1;
      if (icode & 2)  idtype = 2;
      if (icode & 4)  idtype = 3;
      if (icode & 8)  lstr = 2;
      if (icode & 16) lvig = 2;
      if (icode & 32) lreg = 2;
      if (icode & 64) idtype = 4;
      if (icode & 128) lfc = 2;
      if (lstr == 2 && lvig == 2)  lsv = 2;
      }
/*	printf ("cphskp> icode: %o   idtype: %d\n", icode, idtype); */
   H.idtype = idtype;

/*
 *   nbypix = 2;
 *   if(idtype == 1 || idtype == 2)  nbypix = 1;
 */
   H.nbypix = nbypix;

   strncat (H.filename, ".", 1);
   strncat (H.filename, &namtype[idtype-1], 1);
   strncat (H.filename, &namsmf[ism][0], 2);
   lenimg = strlen (H.filename);
   H.filename[lenimg] = '\0';

/*** Look up disk center location according to sector mirror. ***/

/*** Raw DN images : (no geometric distortion correction) ***/

   if (idtype == 1)
      {
      xcen = vidcen[ism][0] / (float) (kres*lres);
      ycen = vidcen[ism][1] / (float) (kres*lres);
      pscale = 1.0;
      }

/* Equivalent Second and Calibrated images :
 * (SMMCOR or WW6COR geometric corrections)
 */

   if (idtype == 2 || idtype == 3 || idtype == 4)
      {
      xcen = ww6cen[ism][0] / (float) (kres*lres);
      ycen = ww6cen[ism][1] / (float) (kres*lres);
      icyear = *(h+141);
      icdoy = (*(h+142)) * 256 + *(h+143);
      if (icyear < 80)
         {
         icyear = 0;
         icdoy = 0;
         }
      pscale = pow10 (-1 * iexp);
      }
   H.xcen = xcen;
   H.ycen = ycen;

/*** Set up scale variables. ***/

   raddeg = pi / 180.0;
   degrad = 180.0 / pi;
   pixrs = secrs / (secpix * kres * lres);
   if (ism == 8)  pixrs *= 0.5;
   rspix = 1.0 / pixrs;

   free (h);
   return (0);
}
