/* dpmhskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill MLSO Digital Prominence Monitor housekeeping structure
 * using information in DPM header structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   15 February 1994
 * 31 May 1994: discovered that Month=0 in DPM header for January.
 *              Therefore DPMI->month = DPMH->Month + 1.
 * 10 Sep 1997: Add DPMI->wcode, DPMI->scode, DPMI->rcode.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "dpmlib.h"
#include "dpm_optdev.h"
#include "optics_names.h"

dpmhskp (namimg, pmode)

char namimg [] ;			/* DPM filename. */
int pmode ;				/* Print mode: 0=silent, 1=verbose. */

{
   FILE *FD ;
   int doy ;
   int i ;

   /*** Open DPM file. ***/

   if ((FD = fopen (namimg, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      return (-1) ;
      }

   /*** Read DPM header. ***/

   if (fread ((char *) &DPMHS, sizeof (DPMHS), 1, FD) == NULL)
      {
      fprintf (stderr, "Unable to read header structure in file %s\n", namimg) ;
      return (-1) ;
      }

   /*** Print header. ***/

   if (pmode)
      {
      fprintf (stdout, "DPMH->Version:		%10d\n",	DPMH->Version) ;
      fprintf (stdout, "DPMH->Year   :		%10d\n",	DPMH->Year   ) ;
      fprintf (stdout, "DPMH->Month  :		%10d\n",	DPMH->Month  ) ;
      fprintf (stdout, "DPMH->Day    :		%10d\n",	DPMH->Day    ) ;
      fprintf (stdout, "DPMH->Hour   :		%10d\n",	DPMH->Hour   ) ;
      fprintf (stdout, "DPMH->Minute :		%10d\n",	DPMH->Minute ) ;
      fprintf (stdout, "DPMH->Sec    :		%10d\n",	DPMH->Sec    ) ;
      fprintf (stdout, "DPMH->Xdim   :		%10d\n",	DPMH->Xdim   ) ;
      fprintf (stdout, "DPMH->Ydim   :		%10d\n",	DPMH->Ydim   ) ;
      fprintf (stdout, "DPMH->Nsum   :		%10d\n",	DPMH->Nsum   ) ;
      fprintf (stdout, "DPMH->Exposure:		%10d\n",	DPMH->Exposure) ;
      fprintf (stdout, "DPMH->Gain   :		%10d\n",	DPMH->Gain   ) ;
      fprintf (stdout, "DPMH->Odisk  :		%10d\n",	DPMH->Odisk  ) ;
      fprintf (stdout, "DPMH->Type   :		%10d\n",	DPMH->Type   ) ;
      for (i = 0; i < 8; i++)
         fprintf (stdout, "DPMH->Optics [%d]:	%10d\n",
                           i, DPMH->Optics [i]) ;
      fprintf (stdout, "DPMH->Pangle :		%10.4f\n",	DPMH->Pangle ) ;
      fprintf (stdout, "DPMH->Bangle :		%10.4f\n",	DPMH->Bangle ) ;
      fprintf (stdout, "DPMH->Langle :		%10.4f\n",	DPMH->Langle ) ;
      fprintf (stdout, "DPMH->HA     :		%10.4f\n",	DPMH->HA     ) ;
      fprintf (stdout, "DPMH->Elevation:	%10.4f\n",	DPMH->Elevation) ;
      fprintf (stdout, "DPMH->Azimuth:		%10.4f\n",	DPMH->Azimuth) ;
      fprintf (stdout, "DPMH->Amass  :		%10.4f\n",	DPMH->Amass  ) ;
      fprintf (stdout, "DPMH->Radius :		%10.4f\n",	DPMH->Radius ) ;
      fprintf (stdout, "\n") ;
      }

   /*** Fill housekeeping structure. ***/

   DPMI->version 	= DPMH->Version		;
   DPMI->dtype   	= DPMH->Type		;
   DPMI->xdim		= DPMH->Xdim		;
   DPMI->ydim		= DPMH->Ydim		;
   DPMI->zdim		= 1			;
   DPMI->nbypx		= sizeof (short)	;
   DPMI->dmin		= 0			;
   DPMI->dmax		= 0			;
   DPMI->smin		= 0			;
   DPMI->smax		= 0			;
   DPMI->year		= DPMH->Year    	;
   DPMI->month		= DPMH->Month + 1      	;	/*0=Jan in DPMH->Month*/
   DPMI->day		= DPMH->Day		;
   doy = day_of_year (DPMI->year, DPMI->month, DPMI->day) ;
   DPMI->doy		= doy			;
   DPMI->hour		= DPMH->Hour		;
   DPMI->minute		= DPMH->Minute		;
   DPMI->second		= DPMH->Sec		;
   DPMI->cal_year	= 0			;
   DPMI->cal_month	= 0			;
   DPMI->cal_day	= 0			;
   DPMI->nsum		= DPMH->Nsum		;
   DPMI->expdur		= DPMH->Exposure	;
   DPMI->gain		= DPMH->Gain		;
   DPMI->odisk		= DPMH->Odisk		;
   DPMI->bscale		= 1.0			;
   DPMI->bzero		= 0.0			;
   DPMI->pixrs		= 0.0			;
   DPMI->pangle		= DPMH->Pangle		;
   DPMI->bangle		= DPMH->Bangle		;
   DPMI->langle		= DPMH->Langle		;
   DPMI->hangle		= DPMH->HA		;
   DPMI->nangle		= 90.0			;
   DPMI->elev		= DPMH->Elevation	;
   DPMI->azimuth	= DPMH->Azimuth		;
   DPMI->airmass	= DPMH->Amass	 	;
   DPMI->rsun		= DPMH->Radius   	;
   DPMI->xcen		= 0.0			;
   DPMI->ycen		= 0.0			;
   DPMI->rmin		= 0.0			;
   DPMI->rmax		= 0.0			;

   switch (DPMI->dtype)
      {
      case 0: /* other          */
	   DPMI->wcode = 9 ;              /* WAVELENGTH_s */
           DPMI->scode = 0 ;              /* SOURCE_x     */
           DPMI->rcode = 0 ;              /* REDUCTION_x  */
	   break ;

      case 1: /* disc           */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 1 ;              /* SOURCE_s     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 2: /* limb           */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 2 ;              /* SOURCE_l     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 3: /* reference      */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 3 ;              /* SOURCE_o     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 4: /* diffuser       */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 4 ;              /* SOURCE_d     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 5: /* dark disc      */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 5 ;              /* SOURCE_b     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 6: /* dark limb      */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 5 ;              /* SOURCE_b     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 7: /* dark reference */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 5 ;              /* SOURCE_b     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      case 8: /* dark diffuser  */
	   DPMI->wcode = 5 ;              /* WAVELENGTH_a */
           DPMI->scode = 5 ;              /* SOURCE_b     */
           DPMI->rcode = 1 ;              /* REDUCTION_r  */
	   break ;

      default:
	   DPMI->wcode = 0 ;              /* WAVELENGTH_x */
           DPMI->scode = 0 ;              /* SOURCE_x     */
           DPMI->rcode = 0 ;              /* REDUCTION_x  */
      }

   for (i = 0; i < 8; i++)
      {
      strcpy (&DPMI->optdev [i][0], &devnam [i][0]) ;
      DPMI->optdev [i][15] = '\0';
      strcpy (&DPMI->optpos [i][0], &devpos [i][DPMH->Optics [i]][0]);
      DPMI->optpos [i][15] = '\0';
      }

   /*** Print housekeeping. ***/

   if (pmode)
      {
      fprintf (stdout, "DPMI->version:		%10d\n",	DPMI->version) ;
      fprintf (stdout, "DPMI->dtype:		%10d	%s\n",	DPMI->dtype,
                       type_str [DPMI->dtype]) ;
      fprintf (stdout, "DPMI->wcode:		%10d	%s\n",	DPMI->dtype,
	               wcode_str [DPMI->wcode]) ;
      fprintf (stdout, "DPMI->scode:		%10d	%s\n",	DPMI->dtype,
	               scode_str [DPMI->scode]) ;
      fprintf (stdout, "DPMI->rcode:		%10d	%s\n",	DPMI->dtype,
	               rcode_str [DPMI->rcode]) ;
      fprintf (stdout, "DPMI->xdim:		%10d\n",	DPMI->xdim) ;
      fprintf (stdout, "DPMI->ydim:		%10d\n",	DPMI->ydim) ;
      fprintf (stdout, "DPMI->zdim:		%10d\n",	DPMI->zdim) ;
      fprintf (stdout, "DPMI->nbypx:		%10d\n",	DPMI->nbypx) ;
      fprintf (stdout, "DPMI->dmin:		%10d\n",	DPMI->dmin) ;
      fprintf (stdout, "DPMI->dmax:		%10d\n",	DPMI->dmax) ;
      fprintf (stdout, "DPMI->smin:		%10d\n",	DPMI->smin) ;
      fprintf (stdout, "DPMI->smax:		%10d\n",	DPMI->smax) ;
      fprintf (stdout, "DPMI->year:		%10d\n",	DPMI->year) ;
      fprintf (stdout, "DPMI->month:		%10d\n",	DPMI->month) ;
      fprintf (stdout, "DPMI->day:		%10d\n",	DPMI->day) ;
      fprintf (stdout, "DPMI->doy:		%10d\n",	DPMI->doy) ;
      fprintf (stdout, "DPMI->hour:		%10d\n",	DPMI->hour) ;
      fprintf (stdout, "DPMI->minute:		%10d\n",	DPMI->minute) ;
      fprintf (stdout, "DPMI->second:		%10d\n",	DPMI->second) ;
      fprintf (stdout, "DPMI->cal_year:		%10d\n",	DPMI->cal_year) ;
      fprintf (stdout, "DPMI->cal_month:	%10d\n",	DPMI->cal_month) ;
      fprintf (stdout, "DPMI->cal_day:		%10d\n",	DPMI->cal_day) ;
      fprintf (stdout, "DPMI->nsum:		%10d\n",	DPMI->nsum) ;
      fprintf (stdout, "DPMI->expdur:		%10.4\n",	DPMI->expdur) ;
      fprintf (stdout, "DPMI->gain:		%10.4\n",	DPMI->gain) ;
      fprintf (stdout, "DPMI->odisk:		%10.4\n",	DPMI->odisk) ;
      fprintf (stdout, "DPMI->bscale:		%10.4f\n",	DPMI->bscale) ;
      fprintf (stdout, "DPMI->bzero:		%10.4f\n",	DPMI->bzero) ;
      fprintf (stdout, "DPMI->pixrs:		%10.4f\n",	DPMI->pixrs) ;
      fprintf (stdout, "DPMI->pangle:		%10.4f\n",	DPMI->pangle) ;
      fprintf (stdout, "DPMI->bangle:		%10.4f\n",	DPMI->bangle) ;
      fprintf (stdout, "DPMI->langle:		%10.4f\n",	DPMI->langle) ;
      fprintf (stdout, "DPMI->hangle:		%10.4f\n",	DPMI->hangle) ;
      fprintf (stdout, "DPMI->nangle:		%10.4f\n",	DPMI->nangle) ;
      fprintf (stdout, "DPMI->elev:		%10.4f\n",	DPMI->elev) ;
      fprintf (stdout, "DPMI->azimuth:		%10.4f\n",	DPMI->azimuth) ;
      fprintf (stdout, "DPMI->airmass:		%10.4f\n",	DPMI->airmass) ;
      fprintf (stdout, "DPMI->rsun:		%10.4f\n",	DPMI->rsun) ;
      fprintf (stdout, "DPMI->xcen:		%10.4f\n",	DPMI->xcen) ;
      fprintf (stdout, "DPMI->ycen:		%10.4f\n",	DPMI->ycen) ;
      fprintf (stdout, "DPMI->rmin:		%10.4f\n",	DPMI->rmin) ;
      fprintf (stdout, "DPMI->rmax:		%10.4f\n",	DPMI->rmax) ;
      for (i = 0; i < 8; i++)
	 {
         fprintf (stdout, "DPMI->optdev[%d]: %-16s	DPMI->optpos[%d]: %s\n",
		  i, &DPMI->optdev [i][0], i, &DPMI->optpos[i][0]) ;
	 }
      }

   return (0) ;
}
