/* hdu_decode.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decode FITS Header Data Unit record.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 March 1994
 * 22 April 1994: add keywords: TYPE-OBS, etc., for disk images (BBSO, KPNO)
 * 2 June 1994: change names: bangle->solar_b0, hangle->solar_h0,
 *                            langle->solar_l0, pangle->solar_p0.
 * 18 August 1994: change : SOLAR_H0 -> HOUR-ANG
 *                          SCNDIR   -> SCAN-DIR, SCNDUR   -> SCAN-DUR
 *                          ANGNORTH -> NORTHANG, SCROLL   -> SC-ROLL
 *		   add    : CODE-OBS, CRRADIUS, DISP-DIR, NIMAVG, OCC-DISC
 *                 remove : CPVER, DPMVER, MK3VER
 * 22 August 1994: add    : CODE-CP, CODE-DPM, CODE-MK3
 * 13 November 1998: calmir, colorfil, glfilter, polaroid & sector
 *                   changed from sparam to cparam.
 *                   optdev1 -> optdev8 changed to optdev01 -> optdev08.
 *                   optpos1 -> optpos8 changed to optpos01 -> optpos08.
 * 16 November 1998: optdev01 -> optdev08 replaced by optdev00 -> optdev12.
 *                   optpos01 -> optpos08 replaced by optpos00 -> optpos12.
 *                   Add detector, image-id, orbit-id keywords.
 * 21 Sep 2000: add type_dp.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/fitslib.ext"
#include "/home/stanger/lib/include/fitskey.h"

hdu_decode (linbuf)

char *linbuf ;

{
   int i ;
   int ibeg ;
   int ikey ;				/* Keyword index. */
   static char key [12] ;		/* Keyword [ASCII string]. */
   static char cbuf [84];		/* line buffer. */
   static char cparam  [72] ;		/* Parameter [ASCII string]. */
   static char cparam1 [72] ;		/* Parameter [ASCII string]. */
   short sparam ;			/* Short parameter. */
   long  lparam ;			/* Long  parameter. */
   float fparam ;			/* Float parameter. */

   strncpy (cbuf, linbuf, 80) ;
   cbuf [80] = '\0' ;			/* Terminate string. */
   strncpy (cparam, &cbuf [10], 70) ;	/* Character parameter. */
   cparam [70] = '\0' ;			/* Terminate string. */

   /*** Discard comment. ***/

   for (i = 20; i < strlen (cparam) - 1; i++)
/*      if (cparam [i] == '/' && cparam [i+1] == ' ') */
      if (cparam [i] == '/')
	 {
	 cparam [i] = '\0' ;
	 break ;
	 }

   squeeze (cparam, 047) ;		/* Remove all quotes from cparam. */

   /*** Replace trailing blanks with NULL. ***/

   for (i = strlen (cparam) - 1; i >= 0; i--)
      {
      if (cparam [i] != ' ')  break ;
      cparam [i] = '\0' ;
      }

/*
   for (i = 0; i = strlen (cparam) - 1; i++)
      {
      if (cparam [i] != ' ')  
	 {
	 ibeg = i ;
	 break ;
	 }
      }
   if (ibeg > 0)
      {
      strcpy (cparam1, cparam + ibeg) ;
      strcpy (cparam, cparam1) ;
      }
 */

/*   squeeze (cparam, ' ') ; */		/* Remove all blanks from cparam. */

   sparam = (short) atoi (cparam) ;		/* Short int parameter. */
   lparam = (int)   atol (cparam) ;		/* Long  int parameter. */
   fparam = (float) atof (cparam) ;		/* Floating point parameter. */

   sscanf (cparam, "%hd", &sparam) ;
   sscanf (cparam, "%d", &lparam) ;
   sscanf (cparam, "%e", &fparam) ;

   strncpy (key, cbuf, 8) ;
   squeeze (key, ' ') ;			/* remove all blanks from key. */

   ikey = -1 ;
   for (i = 0; i < numkey; i++)
      {
      if (strcmp (key, &fitskey [i][0]) == 0)
         ikey = i ;
      }
/* fprintf (stdout, "key:|%s|cparam:|%s|ikey:%3d\n", key, cparam, ikey) ; */

   switch (ikey)
      {
      case   0:	/*          */
						break ;
      case   1:	/* AIRMASS  */
		airmass  = fparam ;		break ;	/* MLSO DPM  */
      case   2:	/* ANGLE    */
		angle    = fparam ;		break ;	/*           */
      case   3:	/* AUTHOR   */
		strcpy (author,   cparam) ;	break ;
      case   4:	/* AZIMUTH  */
		azimuth  = fparam ;		break ;	/* MLSO DPM  */

      case   5:	/* BIAS     */
		bias     = fparam ;		break ;	/* KPNO      */
      case   6:	/* BITPIX   */
		bitpix   = sparam ;		break ;
      case   7:	/* BLANK    */
		blank    = lparam ;		break ;
      case   8:	/* BLOCKED  */
		blocked  = 1 ;			break ;
      case   9:	/* BSCALE   */
		b_scale   = fparam ;		break ;
      case  10:	/* BUNIT    */
		strcpy (bunit,    cparam) ;	break ;
      case  11:	/* BZERO    */
		b_zero    = fparam ;		break ;

      case  12:	/* CALIBRAT */
		strcpy (calibrat, cparam) ;	break ;
      case  13:	/* CALMIR   */
		strcpy (calmir, cparam) ;	break ;
      case  14:	/* CAMERA   */
		strcpy (camera,   cparam) ;	break ;
      case  15:	/* CDELT1   */
		cdelt1   = fparam ;		break ;
      case  16:	/* CDELT2   */
		cdelt2   = fparam ;		break ;
      case  17:	/* CDELT3   */
		cdelt3   = fparam ;		break ;
      case  18:	/* CDELT4   */
		cdelt4   = fparam ;		break ;
      case  19:	/* CODE-CP  */
		code_cp  = sparam ;		break ;	/* SMM C/P data code. */
      case  20:	/* CODE-DPM */
		code_dpm = sparam ;		break ;	/* MLSO DPM data code.*/
      case  21:	/* CODE-MK3 */
		code_mk3 = sparam ;		break ;	/* MLSO MK3 data code.*/
      case  22:	/* CODE-OBS */
		code_obs = sparam ;		break ;	/* observation code. */
      case  23:	/* COLORFIL */
		strcpy (colorfil, cparam) ;	break ;
      case  24:	/* COMMENT  */
						break ;
      case  25:	/* CROTA1   */
		crota1   = fparam ;		break ;
      case  26:	/* CROTA2   */
		crota2   = fparam ;		break ;
      case  27:	/* CROTA3   */
		crota3   = fparam ;		break ;
      case  28:	/* CROTA4   */
		crota4   = fparam ;		break ;
      case  29:	/* CRPIX1   */
		crpix1   = fparam ;		break ;
      case  30:	/* CRPIX2   */
		crpix2   = fparam ;		break ;
      case  31: /* CRPIX3   */
		crpix3   = fparam ;		break ;
      case  32:	/* CRPIX4   */
		crpix4   = fparam ;		break ;
      case  33: /* CRRADIUS */
		crradius = fparam ;		break ;	/* Rsun [pixels]. */
      case  34:	/* CRVAL1   */
		crval1   = fparam ;		break ;
      case  35:	/* CRVAL2   */
		crval2   = fparam ;		break ;
      case  36:	/* CRVAL3   */
		crval3   = fparam ;		break ;
      case  37:	/* CRVAL4   */
		crval4   = fparam ;		break ;
      case  38:	/* CTYPE1   */
		strcpy (ctype1,   cparam) ;	break ;
      case  39:	/* CTYPE2   */
		strcpy (ctype2,   cparam) ;	break ;
      case  40:	/* CTYPE3   */
		strcpy (ctype3,   cparam) ;	break ;
      case  41:	/* CTYPE4   */
		strcpy (ctype4,   cparam) ;	break ;

      case  42:	/* DATAFORM */
		strcpy (dataform, cparam) ;	break ; /* HAO       */
      case  43:	/* DATAMAX  */
		datamax  = fparam ;		break ;
      case  44:	/* DATAMIN  */
		datamin  = fparam ;		break ;
      case  45:	/* DATE     */
		strcpy (date,     cparam) ;	break ;
      case  46:	/* DATE-CAL */
		strcpy (date_cal, cparam) ;	break ;	/* HAO       */
      case  47:	/* DATE-OBS */
		strcpy (date_obs, cparam) ;	break ;	/* HAO       */
      case  48:	/* DATE-OPC */
		strcpy (date_opc, cparam) ;	break ;	/* MLSO MK3  */
      case  49:	/* DELTAT   */
		deltat   = fparam ;		break ;
      case  50:	/* DETECTOR */
		strcpy (detector, cparam) ;	break ;	/* detector name */
      case  51:	/* DIRAXIS1 */
		strcpy (diraxis1, cparam) ;	break ;	/* direction axis 1 */
      case  52:	/* DIRAXIS2 */
		strcpy (diraxis2, cparam) ;	break ;	/* direction axis 2 */
      case  53:	/* DIRAXIS3 */
		strcpy (diraxis3, cparam) ;	break ;	/* direction axis 3 */
      case  54:	/* DIRAXIS4 */
		strcpy (diraxis4, cparam) ;	break ;	/* direction axis 4 */
      case  55: /* DISP-DIR */
		strcpy (disp_dir, cparam) ;	break ;	/* display direction. */
      case  56:	/* DISPMAX  */
		dispmax  = fparam ;		break ;	/* HAO       */
      case  57:	/* DISPMIN  */
		dispmin  = fparam ;		break ;	/* HAO       */

      case  58:	/* ELEV     */
		elev     = fparam ;		break ;	/* MLSO DPM  */
      case  59:	/* END      */
		end_fts  = 1 ;			break ;
      case  60: /* EPOCH    */
		epoch    = fparam ;		break ;
      case  61:	/* EQUINOX  */
		equinox  = fparam ;		break ;
      case  62:	/* EXPDUR   */
		expdur   = fparam ;		break ;	/* HAO       */
      case  63:	/* EXPUNITS */
		strcpy (expunits, cparam) ;	break ;	/* HAO       */
      case  64:	/* EXTEND   */
		extend   = 1 ;			break ;
      case  65:	/* EXTLEVEL */
		extlevel = lparam ;		break ;
      case  66:	/* EXTNAME  */
		strcpy (extname,  cparam) ;	break ;
      case  67:	/* EXTVER   */
		extver   = lparam ;		break ;

      case  68:	/* FILTER   */
		strcpy (filter,   cparam) ;	break ;
      case  69:	/* FRAME    */
		frame    = lparam ;		break ;
      case  70:	/* FRAMES   */
		frames   = lparam ;		break ;

      case  71:	/* GAIN     */
		gain     = fparam ;		break ;
      case  72:	/* GCOUNT   */
		gcount   = lparam ;		break ;
      case  73:	/* GLFILTER */
		strcpy (glfilter, cparam) ;	break ;

      case  74:	/* HISTORY  */
						break ;
      case  75:	/* HOUR-ANG */
		hour_ang   = fparam ;		break ;	/* MLSO DPM  */

      case  76: /* IMAGE-ID */
		image_id   = lparam ;		break ; /* SPARTAN 201 */
      case  77:	/* INSTRUME */
		strcpy (instrume, cparam) ;	break ;
      case  78:	/* INT-MODE */
		strcpy (int_mode, cparam) ;	break ;

      case  79:	/* NAXIS    */
		naxis    = sparam ;		break ;
      case  80:	/* NAXIS1   */
		naxis1   = sparam ;		break ;
      case  81:	/* NAXIS2   */
		naxis2   = sparam ;		break ;
      case  82:	/* NAXIS3   */
		naxis3   = sparam ;		break ;
      case  83:	/* NAXIS4   */
		naxis4   = sparam ;		break ;
      case  84:	/* NIMAVG   */
		nimavg   = sparam ;		break ;	/* # images averaged. */
      case  85:	/* NORTHANG */
		northang = fparam ;		break ;	/* HAO       */

      case  86:	/* OBJECT   */
		strcpy (object,   cparam) ;	break ;
      case  87:	/* OBSERVER */
		strcpy (observer, cparam) ;	break ;
      case  88:	/* OCC-DISC */
		occ_disc = fparam ;		break ;	/* MLSO DPM  */
      case  89:	/* OPTDEV00  */
		strcpy (optdev00,  cparam) ;	break ; /* MLSO DPM  */
      case  90:	/* OPTDEV01  */
		strcpy (optdev01,  cparam) ;	break ; /* MLSO DPM  */
      case  91:	/* OPTDEV02  */
		strcpy (optdev02,  cparam) ;	break ; /* MLSO DPM  */
      case  92:	/* OPTDEV03  */
		strcpy (optdev03,  cparam) ;	break ; /* MLSO DPM  */
      case  93:	/* OPTDEV04  */
		strcpy (optdev04,  cparam) ;	break ; /* MLSO DPM  */
      case  94:	/* OPTDEV05  */
		strcpy (optdev05,  cparam) ;	break ; /* MLSO DPM  */
      case  95:	/* OPTDEV06  */
		strcpy (optdev06,  cparam) ;	break ; /* MLSO DPM  */
      case  96:	/* OPTDEV07  */
		strcpy (optdev07,  cparam) ;	break ; /* MLSO DPM  */
      case  97:	/* OPTDEV08  */
		strcpy (optdev08,  cparam) ;	break ; /* MLSO DPM  */
      case  98:	/* OPTDEV09  */
		strcpy (optdev09,  cparam) ;	break ; /* MLSO DPM  */
      case  99:	/* OPTDEV10  */
		strcpy (optdev10,  cparam) ;	break ; /* MLSO DPM  */
      case 100:	/* OPTDEV11  */
		strcpy (optdev11,  cparam) ;	break ; /* MLSO DPM  */
      case 101:	/* OPTDEV12  */
		strcpy (optdev12,  cparam) ;	break ; /* MLSO DPM  */
      case 102:	/* OPTPOS00  */
		strcpy (optpos00,  cparam) ;	break ; /* MLSO DPM  */
      case 103:	/* OPTPOS01  */
		strcpy (optpos01,  cparam) ;	break ; /* MLSO DPM  */
      case 104:	/* OPTPOS02  */
		strcpy (optpos02,  cparam) ;	break ; /* MLSO DPM  */
      case 105:	/* OPTPOS03  */
		strcpy (optpos03,  cparam) ;	break ; /* MLSO DPM  */
      case 106:	/* OPTPOS04  */
		strcpy (optpos04,  cparam) ;	break ; /* MLSO DPM  */
      case 107:	/* OPTPOS05  */
		strcpy (optpos05,  cparam) ;	break ; /* MLSO DPM  */
      case 108:	/* OPTPOS06  */
		strcpy (optpos06,  cparam) ;	break ; /* MLSO DPM  */
      case 109:	/* OPTPOS07  */
		strcpy (optpos07,  cparam) ;	break ; /* MLSO DPM  */
      case 110:	/* OPTPOS08  */
		strcpy (optpos08,  cparam) ;	break ; /* MLSO DPM  */
      case 111:	/* OPTPOS09  */
		strcpy (optpos09,  cparam) ;	break ; /* MLSO DPM  */
      case 112:	/* OPTPOS10  */
		strcpy (optpos10,  cparam) ;	break ; /* MLSO DPM  */
      case 113:	/* OPTPOS11  */
		strcpy (optpos11,  cparam) ;	break ; /* MLSO DPM  */
      case 114:	/* OPTPOS12  */
		strcpy (optpos12,  cparam) ;	break ; /* MLSO DPM  */
      case 115:	/* ORBIT-ID  */
		orbit_id = lparam ;		break ;	/* SPARTAN 201 */
      case 116:	/* ORIGIN   */
		strcpy (origin,   cparam) ;	break ;

      case 117:	/* PCOUNT   */
		pcount   = lparam ;		break ;
      case 118:	/* PHI      */
		phi      = fparam ;		break ;
      case 119:	/* POLAROID */
		strcpy (polaroid, cparam) ;	break ;	/* HAO SMM C/P  */
      case 120:	/* PUNIT    */
		strcpy (punit,    cparam) ;	break ;

      case 121:	/* QUALITY  */
		strcpy (quality,  cparam) ;	break ;

      case 122:	/* RADIUS   */
		radius   = fparam ;		break ;
      case 123:	/* RANGVALU */
		rangvalu = fparam ;		break ;
      case 124:	/* REFERENC */
		strcpy (referenc, cparam) ;	break ;
      case 125:	/* RMAX     */
		rmax     = fparam ;		break ;	/* MLSO MK3  */
      case 126:	/* RMIN     */
		rmin     = fparam ;		break ;	/* MLSO MK3  */
      case 127:	/* RSUN     */
		rsun     = fparam ;		break ;	/* HAO       */
      case 128:	/* RUNIT    */
		strcpy (runit,    cparam) ;	break ;

      case 129:	/* SECTOR   */
		strcpy (sector, cparam) ;	break ;	/* HAO SMM C/P  */
      case 130:	/* SEEING   */
		strcpy (seeing,   cparam) ;	break ;
      case 131:	/* SCAN-DIR */
		strcpy (scan_dir, cparam) ;	break ; /* MLSO MK3  */
      case 132:	/* SCAN_DUR */
		scan_dur = fparam ;		break ;	/* MLSO MK3  */
      case 133:	/* SC_ROLL  */
		sc_roll  = sparam ;		break ;	/* HAO SMM C/P */
      case 134:	/* SIMPLE   */
		if (cparam [0] == 'T') simple =1; else simple = 0; break;
      case 135:	/* SKY      */
		strcpy (sky,      cparam) ;	break ; /*           */
      case 136:	/* SOLAR_B0 */
		solar_b0   = fparam ;		break ;	/* MLSO DPM  */
      case 137:	/* SOLAR_L0 */
		solar_l0   = fparam ;		break ;	/* MLSO DPM  */
      case 138:	/* SOLAR_P0   */
		solar_p0   = fparam ;		break ;	/* MLSO DPM  */

      case 139:	/* TELESCOP */
		strcpy (telescop, cparam) ;	break ;
      case 140:	/* THETA    */
		theta    = fparam ;		break ;	/* MLSO MK3  */
      case 141:	/* TIME     */
		strcpy (time_fts, cparam) ;	break ;
      case 142:	/* TIME-CAL */
		strcpy (time_cal, cparam) ;	break ;	/* HAO       */
      case 143:	/* TIME-OBS */
		strcpy (time_obs, cparam) ;	break ;
      case 144:	/* TIME-OPC */
		strcpy (time_opc, cparam) ;	break ;	/* MLSO MK3  */
      case 145:	/* TILT     */
		tilt     = fparam ;		break ;	/*           */
      case 146:	/* TMAX     */
		tmax     = fparam ;		break ;	/* MLSO MK3  */
      case 147:	/* TMIN     */
		tmin     = fparam ;		break ;	/* MLSO MK3  */
      case 148:	/* TUNIT    */
		strcpy (tunit,    cparam) ;	break ;	/*           */
      case 149:	/* TYPE-CP */
		strcpy (type_cp,  cparam) ;	break ;	/* SMM  C/P  */
      case 150:	/* TYPE-DP */
		strcpy (type_dp,  cparam) ;	break ;	/*           */
      case 151:	/* TYPE-DPM */
		strcpy (type_dpm, cparam) ;	break ;	/* MLSO DPM  */
      case 152:	/* TYPE-MK3 */
		strcpy (type_mk3, cparam) ;	break ;	/* MLSO MK3  */
      case 153:	/* TYPE-OBS */
		strcpy (type_obs, cparam) ;	break ;	/*           */
      case 154:	/* TYPE1    */
		strcpy (type1,    cparam) ;	break ;	/*           */
      case 155:	/* TYPE2    */
		strcpy (type2,    cparam) ;	break ;	/*           */
      case 156:	/* TYPE3    */
		strcpy (type3,    cparam) ;	break ;	/*           */
      case 157:	/* TYPE4    */
		strcpy (type4,    cparam) ;	break ;	/*           */
      case 158:	/* TYPE5    */
		strcpy (type5,    cparam) ;	break ;	/*           */
      case 159:	/* TYPE6    */
		strcpy (type6,    cparam) ;	break ;	/*           */
      case 160:	/* TYPE7    */
		strcpy (type7,    cparam) ;	break ;	/*           */
      case 161:	/* TYPE8    */
		strcpy (type8,    cparam) ;	break ;	/*           */

      case 162:	/* UTSTART  */
		strcpy (utstart,  cparam) ;	break ;	/*           */
      case 163:	/* UTSTOP   */
		strcpy (utstop,   cparam) ;	break ;	/*           */

      case 164:	/* VERSION  */
		strcpy (version,  cparam) ;	break ;	/*           */

      case 165:	/* WAVE     */
		wave     = fparam ;		break ;	/* wavelength */
      case 166:	/* WAVELEN  */
		wavelen  = fparam ;		break ;	/* wavelength */
      case 167:	/* WAVELNTH */
		wavelnth = fparam ;		break ;	/* wavelength */

      case 168:	/* XTENSION */
		strcpy (xtension, cparam) ;	break ;	/* FITS extension */

      case 169:	/* ZEROVALU */
		zerovalu = fparam ;		break ;	/*            */
      }
   return (0) ;
}
