/* print_fits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print FITS keyword variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 March 1994
 * 2 June 1994: change names: bangle->solar_b0, hangle->solar_h0,
 *                            langle->solar_l0, pangle->solar_p0.
 * 18 August 1994: change : SOLAR_H0 -> HOUR-ANG
 *                          SCNDIR   -> SCAN-DIR, SCNDUR   -> SCAN-DUR
 *                          ANGNORTH -> NORTHANG, SCROLL   -> SC-ROLL
 *		   add    : CODE-OBS, CRRADIUS, DISP-DIR, NIMAVG, OCC-DISC
 *                 remove : CPVER, DPMVER, MK3VER
 * 21 Sep 2000: add TYPE-DP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/fitslib.ext"

print_fits ()

{

   fprintf (stdout, "\n") ;
   fprintf (stdout, "simple   : %20d\n",   simple  ) ;
   fprintf (stdout, "bitpix   : %20d\n",   bitpix  ) ;
   fprintf (stdout, "naxis    : %20d\n",   naxis   ) ;
   fprintf (stdout, "naxis1   : %20d\n",   naxis1  ) ;
   fprintf (stdout, "naxis2   : %20d\n",   naxis2  ) ;
   fprintf (stdout, "\n") ;
   fprintf (stdout, "extend   : %20d\n",   extend  ) ;
   fprintf (stdout, "\n") ;
   fprintf (stdout, "pcount   : %20d\n",   pcount  ) ;
   fprintf (stdout, "gcount   : %20d\n",   gcount  ) ;
   fprintf (stdout, "\n") ;
   fprintf (stdout, "date     : %s\n",     date    ) ;
   fprintf (stdout, "time     : %s\n",     time_fts) ;
   fprintf (stdout, "date_obs : %s\n",     date_obs) ;
   fprintf (stdout, "time_obs : %s\n",     time_obs) ;
   fprintf (stdout, "date_cal : %s\n",     date_cal) ;
   fprintf (stdout, "time_cal : %s\n",     time_cal) ;
   fprintf (stdout, "date_opc : %s\n",     date_opc) ;
   fprintf (stdout, "time_opc : %s\n",     time_opc) ;

   fprintf (stdout, "origin   : %s\n",     origin  ) ;
   fprintf (stdout, "author   : %s\n",     author  ) ;
   fprintf (stdout, "referenc : %s\n",     referenc) ;
   fprintf (stdout, "telescop : %s\n",     telescop) ;
   fprintf (stdout, "instrume : %s\n",     instrume) ;
   fprintf (stdout, "observer : %s\n",     observer) ;
   fprintf (stdout, "object   : %s\n",     object  ) ;

   fprintf (stdout, "equinox  : %20.0f\n", equinox ) ;
   fprintf (stdout, "epoch    : %20.0f\n", epoch   ) ;

   fprintf (stdout, "extname  : %s\n",     extname ) ;
   fprintf (stdout, "extver   : %20d\n",   extver  ) ;
   fprintf (stdout, "extlevel : %20d\n",   extlevel) ;

   fprintf (stdout, "blank    : %20d\n",   blank   ) ;

   fprintf (stdout, "b_scale  : %20.4e\n", b_scale  ) ;
   fprintf (stdout, "b_zero   : %20.4e\n", b_zero   ) ;
   fprintf (stdout, "bunit    : %s\n",     bunit   ) ;

   fprintf (stdout, "ctype1   : %s\n",     ctype1  ) ;
   fprintf (stdout, "ctype2   : %s\n",     ctype2  ) ;
   fprintf (stdout, "ctype3   : %s\n",     ctype3  ) ;

   fprintf (stdout, "crpix1   : %20.4e\n", crpix1  ) ;
   fprintf (stdout, "crpix2   : %20.4e\n", crpix2  ) ;
   fprintf (stdout, "crpix3   : %20.4e\n", crpix3  ) ;

   fprintf (stdout, "crval1   : %20.4e\n", crval1  ) ;
   fprintf (stdout, "crval2   : %20.4e\n", crval2  ) ;
   fprintf (stdout, "crval3   : %20.4e\n", crval3  ) ;

   fprintf (stdout, "crval1   : %20.4e\n", crval1  ) ;
   fprintf (stdout, "crval2   : %20.4e\n", crval2  ) ;
   fprintf (stdout, "crval3   : %20.4e\n", crval3  ) ;

   fprintf (stdout, "cdelt1   : %20.4e\n", cdelt1  ) ;
   fprintf (stdout, "cdelt2   : %20.4e\n", cdelt2  ) ;
   fprintf (stdout, "cdelt3   : %20.4e\n", cdelt3  ) ;

   fprintf (stdout, "crota1   : %20.4e\n", crota1  ) ;
   fprintf (stdout, "crota2   : %20.4e\n", crota2  ) ;
   fprintf (stdout, "crota3   : %20.4e\n", crota3  ) ;

   fprintf (stdout, "crradius : %20.4e\n", crradius) ;
   fprintf (stdout, "disp_dir : %s\n", disp_dir) ;

   fprintf (stdout, "datamin  : %20.4e\n", datamin ) ;
   fprintf (stdout, "datamax  : %20.4e\n", datamax ) ;

   fprintf (stdout, "dispmin  : %20.4e\n", dispmin ) ;
   fprintf (stdout, "dispmax  : %20.4e\n", dispmax ) ;

   fprintf (stdout, "version  : %s\n", version ) ;

   fprintf (stdout, "dataform : %20s\n", dataform) ;

   fprintf (stdout, "code_cp  : %20d\n", code_cp ) ;
   fprintf (stdout, "code_dpm : %20d\n", code_dpm) ;
   fprintf (stdout, "code_mk3 : %20d\n", code_mk3) ;
   fprintf (stdout, "code_obs : %20d\n", code_obs) ;

   fprintf (stdout, "type_cp  : %s\n", type_cp ) ;
   fprintf (stdout, "type_dp  : %s\n", type_dp ) ;
   fprintf (stdout, "type_dpm : %s\n", type_dpm) ;
   fprintf (stdout, "type_mk3 : %s\n", type_mk3) ;
   fprintf (stdout, "type_obs : %s\n", type_obs) ;

   fprintf (stdout, "nimavg   : %20d\n", nimavg  ) ;

   fprintf (stdout, "expdur   : %20.4e\n", expdur  ) ;
   fprintf (stdout, "expunits : %s\n",     expunits) ;

   fprintf (stdout, "rmin     : %20.4e\n", rmin    ) ;
   fprintf (stdout, "rmax     : %20.4e\n", rmax    ) ;

   fprintf (stdout, "tmin     : %20.4e\n", tmin    ) ;
   fprintf (stdout, "tmax     : %20.4e\n", tmax    ) ;

   fprintf (stdout, "scan_dur : %20.4e\n", scan_dur) ;
   fprintf (stdout, "scan_dir : %s\n",     scan_dir) ;

   fprintf (stdout, "northang : %20.4e\n", northang) ;
   fprintf (stdout, "rsun     : %20.4e\n", rsun    ) ;
   fprintf (stdout, "occ_disc : %20.4e\n", occ_disc) ;

   fprintf (stdout, "solar_b0 : %20.4e\n", solar_b0) ;
   fprintf (stdout, "solar_l0 : %20.4e\n", solar_l0) ;
   fprintf (stdout, "solar_p0 : %20.4e\n", solar_p0) ;
   fprintf (stdout, "hour_ang : %20.4e\n", hour_ang) ;

   fprintf (stdout, "elev     : %20.4e\n", elev    ) ;
   fprintf (stdout, "azimuth  : %20.4e\n", azimuth ) ;
   fprintf (stdout, "airmass  : %20.4e\n", airmass ) ;

   fprintf (stdout, "optdev01  : %s\n",     optdev01 ) ;
   fprintf (stdout, "optdev02  : %s\n",     optdev02 ) ;
   fprintf (stdout, "optdev03  : %s\n",     optdev03 ) ;
   fprintf (stdout, "optdev04  : %s\n",     optdev04 ) ;
   fprintf (stdout, "optdev05  : %s\n",     optdev05 ) ;
   fprintf (stdout, "optdev06  : %s\n",     optdev06 ) ;
   fprintf (stdout, "optdev07  : %s\n",     optdev07 ) ;
   fprintf (stdout, "optdev08  : %s\n",     optdev08 ) ;

   fprintf (stdout, "optpos01  : %s\n",     optpos01 ) ;
   fprintf (stdout, "optpos02  : %s\n",     optpos02 ) ;
   fprintf (stdout, "optpos03  : %s\n",     optpos03 ) ;
   fprintf (stdout, "optpos04  : %s\n",     optpos04 ) ;
   fprintf (stdout, "optpos05  : %s\n",     optpos05 ) ;
   fprintf (stdout, "optpos06  : %s\n",     optpos06 ) ;
   fprintf (stdout, "optpos07  : %s\n",     optpos07 ) ;
   fprintf (stdout, "optpos08  : %s\n",     optpos08 ) ;

   fprintf (stdout, "calmir   : %20s\n", calmir  ) ;
   fprintf (stdout, "colorfil : %20s\n", colorfil) ;
   fprintf (stdout, "glfilter : %20s\n", glfilter) ;
   fprintf (stdout, "polaroid : %20s\n", polaroid) ;
   fprintf (stdout, "sector   : %20s\n", sector  ) ;
   fprintf (stdout, "frame    : %20d\n", frame   ) ;
   fprintf (stdout, "sc_roll  : %20d\n", sc_roll ) ;

   return (0) ;
}
