/* rdhdu.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read FITS Header Data Unit records.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 March 1994
 * 21 Feb 95: use hdu_clear to reset all parameters to default values.
 * 19 Jun 97: Replace "end" with "end_fts".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/fitslib.ext"
#define NCHLIN 80
#define NLINES 36
#define NBYFIT 2880

rdhdu (namfit)

char namfit [] ;		/* File name. */

{
   FILE *FD ;			/* File descriptor. */
   static char hdu [NLINES][NCHLIN] ;	/* HDU buffer. */
   static char keyword [12] ;	/* FITS keyword (8 characters maximum). */
   static char param [72] ;	/* FITS parameter. */

   char  *cparam ;		/* Pointer to character string parameter. */
   short *sparam ;		/* Pointer to short int parameter. */
   int   *iparam ;		/* Pointer to long int parameter. */
   float *fparam ;		/* Pointer to floating point parameter. */

   int i ;
   int nread ;
   int numhdu ;

   numhdu  = 0 ;
   end_fts = 0 ;
   simple  = 0 ;
   extend  = 0 ;
   naxis = naxis1 = naxis2 = bitpix = 0 ;

   /*** Set all FITS parameters to default values. ***/

   hdu_clear () ;

   /*** Open FITS file. ***/

   if ((FD = fopen (namfit, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open FITS file: %s\n", namfit) ;
      exit (-1) ;
      }

   while (end_fts != 1)
      {
      nread = fread (hdu, NBYFIT, 1, FD) ;	/* Read HDU from file. */

      if (feof (FD) != 0)			/* Check for EOF. */
	 {
         fprintf (stderr, "rdhdu> nread: %d\n", nread) ;
         fprintf (stderr, "%s\n", &hdu [0][0]) ;
	 fprintf (stderr, "EOF encountered in file: %s\n", namfit) ;
	 exit (-2) ;
	 }

      if (ferror (FD) != 0)			/* Check for read error. */
	 {
	 fprintf (stderr, "Read error encountered in file: %s\n", namfit) ;
	 exit (-3) ;
	 }

      numhdu++ ;
      if (numhdu == 1 && strncmp (hdu, "SIMPLE", 6) != 0)
	 {
	 fprintf (stderr, "File (%s) is not a FITS file !\n", namfit) ;
	 exit (-3) ;
	 }

      for (i = 0; i < NLINES; i++)
         {
	 hdu_decode (&hdu [i][0]) ;		/* Decode HDU ASCII line. */
         }
      }

   fclose (FD) ;
   return (numhdu * NBYFIT) ;
}
