/*-----------------------------------------------------------------------------
 * G L C P _ N O R T H ()   - Draws a North pointer with center
 *                           (screen coordinates) at (x,y),
 *                           circle radius cirrad, tip radius tiprad.
 *
 * If the coordinates fall outside the displayable screen area, the 
 * coordinates are clipped to the screen boundary.
 *
 * externals required;
 *    gl_wvec()	- draw a vector on the display screen.
 *    gMINROW, gMAXROW  - integers. Minimum and maximum vertical pixels.
 *    gMINCOL, gMAXCOL  - integers. Minimum and maximum horizontal pixels.
 *-----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 5 June 1989
 * 21 August 1990: Remove 2 degree offset previously used.
 *  4 March 1991: Make inner circle 0.1 Rsun.
 *  4 April 1991: SGI/GL version.
 *-----------------------------------------------------------------------------
 */
#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"

glcp_north (x, y, cirrad, tiprad, cindex)

int x, y;			/* cross hair center position. */
float cirrad;			/* Circle radius (Rsun). */
float tiprad;			/* Pointer tip radius (Rsun). */
int cindex;			/* color LUT index. */
{
   float r, th;
   float xx, yy;
   int xg, yg;		/* Inner circle. */
   int xcn, ycn;	/* North of center. */
   int xcs, ycs;	/* South of center. */
   int xce, yce;	/* East of center. */
   int xcw, ycw;	/* West of center. */
   int xpn, ypn;	/* North pointer tip. */
   int xwe, ywe;	/* Left wing tip. */
   int xww, yww;	/* Right wing tip. */

   /*** North pointer tip coordinates. ***/

   r = tiprad;
   th = 0.0;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xpn = ((xx / nres) + 0.5) + gMINROW;
   ypn = gMAXCOL - ((yy / nres) + 0.5);

   /*** East wing tip coordinates. ***/

   r = tiprad;
   th = 90.0;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xwe = ((xx / nres) + 0.5) + gMINROW;
   ywe = gMAXCOL - ((yy / nres) + 0.5);

   r = cirrad;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xce = ((xx / nres) + 0.5) + gMINROW;
   yce = gMAXCOL - ((yy / nres) + 0.5);

   /*** West wing tip coordinates. ***/

   r = tiprad;
   th = 270.0;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xww = ((xx / nres) + 0.5) + gMINROW;
   yww = gMAXCOL - ((yy / nres) + 0.5);

   r = cirrad;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xcw = ((xx / nres) + 0.5) + gMINROW;
   ycw = gMAXCOL - ((yy / nres) + 0.5);

   /*** Draw dots surrounding disk center pixel. ***/

   r = 0.0;
   th = 0.0;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xg = ((xx / nres) + 0.5) + gMINROW;
   yg = gMAXCOL - ((yy / nres) + 0.5);
   if (xg  >= gMINCOL && xg  <= gMAXCOL && yg  >= gMINROW && yg  <= gMAXROW)
      {
      gl_wvec (xg-1, yg,   xg-1, yg,   cindex);
      gl_wvec (xg+1, yg,   xg+1, yg,   cindex);
      gl_wvec (xg,   yg-1, xg,   yg-1, cindex);
      gl_wvec (xg,   yg+1, xg,   yg+1, cindex);
      }
/*
 *   fprintf (stdout, "glcp_north> xg: %d   yg: %d\n", xg, yg);
 */

   /*** Draw circle around disk center. ***/

   r = cirrad;
   for (th=0.0; th<=360.0; th += 5.0)
      {
      rcoord (&r, &th, &xx, &yy, 1, scroll);
      xg = ((xx / nres) + 0.5) + gMINROW;
      yg = gMAXCOL - ((yy / nres) + 0.5);
      if (xg  >= gMINCOL && xg  <= gMAXCOL && yg >= gMINROW && yg <= gMAXROW)
         gl_wvec (xg, yg, xg, yg, cindex);
      }

   /*** Draw line from circle to north pointer tip. ***/

   r = cirrad;
   th = 0.0;
   rcoord (&r, &th, &xx, &yy, 1, scroll);
   xg = ((xx / nres) + 0.5) + gMINROW;
   yg = gMAXCOL - ((yy / nres) + 0.5);
/*
 *   if (xpn >= gMINCOL && xpn <= gMAXCOL && ypn >= gMINROW && ypn <= gMAXROW &&
 *       xg  >= gMINCOL && xg  <= gMAXCOL && yg  >= gMINROW && yg  <= gMAXROW)
 *      gl_wvec (xg, yg, xpn, ypn, cindex);
 */
 
   /*** Draw north pointer. ***/

   /* North pointer tip to east wing tip. */

   if (xpn >= gMINCOL && xpn <= gMAXCOL && ypn >= gMINROW && ypn <= gMAXROW &&
       xwe >= gMINCOL && xwe <= gMAXCOL && ywe >= gMINROW && ywe <= gMAXROW)
      gl_wvec (xpn, ypn, xwe, ywe, cindex);

   /* Circle to east wing tip. */

   if (xce >= gMINCOL && xce <= gMAXCOL && yce >= gMINROW && yce <= gMAXROW &&
       xwe >= gMINCOL && xwe <= gMAXCOL && ywe >= gMINROW && ywe <= gMAXROW)
      gl_wvec (xce, yce, xwe, ywe, cindex);

   /* North pointer tip to west wing tip. */

   if (xpn >= gMINCOL && xpn <= gMAXCOL && ypn >= gMINROW && ypn <= gMAXROW &&
       xww >= gMINCOL && xww <= gMAXCOL && yww >= gMINROW && yww <= gMAXROW)
      gl_wvec (xpn, ypn, xww, yww, cindex);

   /* Circle to west wing tip. */

   if (xcw >= gMINCOL && xcw <= gMAXCOL && ycw >= gMINROW && ycw <= gMAXROW &&
       xww >= gMINCOL && xww <= gMAXCOL && yww >= gMINROW && yww <= gMAXROW)
      gl_wvec (xcw, ycw, xww, yww, cindex);

   return (0);
}
