/* blint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language   UNIX 4.2 bsd operating system   VAX-11/750
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * blint returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float blint();
 * in the calling routine.
 * 
 * This routine will accept byte, short integer, or floating-point images. 
 * The file `iproc.ext' contains the `image->bppx' which specifies whether
 * the image is a byte (bppx=1), short integer (bppx=2), or
 * floating-point (bppx=4) array. 
 * `iproc.ext' also contains the `image->dx', which specifies the number
 * of pixels per horizontal line in the image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * HAO/NCAR   Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 14 February 1990: adapt blint.c from cplib.a for use with IMAGE structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/local/home/stanger/lib/include/iproc.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
blint (image, x, y)

struct IMAGE *image;	/* Pointer to image array. */
float *x, *y;		/* Coordinates of scan point in image. */

{
register unsigned char	*im;
register unsigned short	*ims;
register float	*imf;
int	ix, iy;
float	fx, fy;
float	f00, f10, f01, f11;
float	fx0, fx1, fxy;

   im  = (unsigned char *) image->addr;
   ims = (unsigned short *) image->addr;
   imf = (float *) image->addr;

   ix = (int) *x;
   iy = (int) *y;

   fx = *x - (float) ix;
   fy = *y - (float) iy;

/* Select 4 nearest neighbor elements in 'image' array. */

/* 1 byte per pixel. */

   if (image->bppx == 1)
      {
      f00 = (float) (*(im +  ix    +  iy    * image->dx) & 0377);
      f10 = (float) (*(im + (ix+1) +  iy    * image->dx) & 0377);
      f01 = (float) (*(im +  ix    + (iy+1) * image->dx) & 0377);
      f11 = (float) (*(im + (ix+1) + (iy+1) * image->dx) & 0377);
      }
   else

/* 2 bytes per pixel (short integer). */

   if (image->bppx == 2)
      {
      f00 = (float) *(ims +  ix    +  iy    * image->dx);
      f10 = (float) *(ims + (ix+1) +  iy    * image->dx);
      f01 = (float) *(ims +  ix    + (iy+1) * image->dx);
      f11 = (float) *(ims + (ix+1) + (iy+1) * image->dx);
      }
   else

/* 4 bytes per pixel (floating point). */

   if (image->bppx == 4)
      {
      f00 = (float) *(imf +  ix    +  iy    * image->dx);
      f10 = (float) *(imf + (ix+1) +  iy    * image->dx);
      f01 = (float) *(imf +  ix    + (iy+1) * image->dx);
      f11 = (float) *(imf + (ix+1) + (iy+1) * image->dx);
      }

/* Bilinear interpolation. */

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}
