/* coord.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P coordinate transformation.
 * isign = 1 :  (radius,theta) --> (x,y).
 *       =-1 :  (x,y) --> (radius,theta).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <math.h>
#include "iproc.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

coord (image, radius, theta, x, y, isign)
struct IMAGE *image;
float	*radius, *theta;  /* Radius (Rsun) & Position Angle (degrees). */
float	*x, *y;		  /* Cartesian coordinates:  (0,0)=upper left.*/
int	isign;		  /* =1: (x,y) --> (r,th). otherwise, vice-versa.*/
{
   int ierr;
   float xc, yc, smmth;
   float raddeg = 1.7453293e-2;	/* Degrees to radians  */
   float degrad = 57.29577951;	/* Radians to degrees  */
   double angle;

   /*** Get the center position for this image. ***/

   xc = image->xcen;
   yc = image->ycen;

   ierr = 0;


   if (isign == 1)	/*** (radius, theta) --> (x,y) ***/

      {
      smmth = *theta + image->degnor;
      if (smmth > 360.0) smmth -= 360.0;
      angle = (double) (raddeg * smmth);
      *x = (float) (xc + *radius * image->pixsr * cos(angle));
      *y = (float) (yc - *radius * image->pixsr * sin(angle));
      }

   else			/*** (x,y) --> (radius, theta) ***/

      {
      *radius = (float) sqrt ((double) ((*x-xc)*(*x-xc) + (*y-yc)*(*y-yc)))
              * 1.0 / (image->pixsr);
      *theta = (float) atan2((double)(yc-*y), (double)(*x-xc))*degrad
             - image->degnor;
      if (*theta < 0.0) *theta += 360.0;
      }

   return (ierr);
}
