/* grnimg.c
 *---------------------------------------------------------------------------
 * A package of Grinnell utilities for program 'scan.c'.
 *---------------------------------------------------------------------------
 * 12 June 1989
 * 3 January 1990: Insert smm_avg call into addimg routine.
 * 22 February 1990: Remove line which had previously copied Grinnell-scaled
 *                   image data back into the array fcb->addr.
 *                   The effect of this line was to produce scans based upon
 *                   Grinnell-scaled intensities rather than being based upon
 *                   the unmanipulated data from the C/P image disk file.
 *---------------------------------------------------------------------------
 */

#include <stdio.h>
#include "iproc.ext"

/*---------------------------------------------------------------------------
 * A D D I M G ( )	- Write the image at address fcb->addr to the default
 *			  Grinnell channel.
 *---------------------------------------------------------------------------
 * 25 Jan 91: Add data types 14 & 15.
 *---------------------------------------------------------------------------
 */
addimg (fcb)

struct IMAGE *fcb;
{
   char *malloc();	/* System allocation function. */
   int imgsiz;		/* # bytes per image. */
   int img_dx, img_dy;	/* Display image dimensions. */
   int img_ip, img_ir;	/* Display image origin. */
   register int i;	/* Array index. */
   register char *img;	/* Pointer to image array. */

   /*** Copy image array to a temporary storage for Grinnell rescaling. ***/

   if (fcb->ires == 0)
      {
      img_dx = fcb->dx / 2;
      img_dy = fcb->dy / 2;
      img_ip = fcb->ip / 2;
      img_ir = fcb->ir / 2;
      smm_avg (fcb);
      }
   else
      {
      img_dx = fcb->dx;
      img_dy = fcb->dy;
      img_ip = fcb->ip;
      img_ir = fcb->ir;
      }
   imgsiz = (img_dx * img_dy * fcb->bppx);
   if ((img = malloc (imgsiz)) == NULL)
      {
      fprintf (stderr,"addimg> malloc error.");
      die ();
      }
   for (i=0; i<imgsiz; i++)  *(img + i) = *(fcb->addr + i);

   /*** Write 'byte' image to Grinnell channel. ***/

   if (fcb->bppx == 1)
       wboxb (img, _GD.x + img_ip, _GD.y - img_ir, img_dx, img_dy, "rd");

   /*** Write 'integer' image to Grinnell channel. ***/

   else
   if (fcb->bppx == 2)
      {
      if (strcmp (fcb->type, "SMM") == 0)
         {
	 if (fcb->bytord == 1)  swab (fcb->addr, img, imgsiz); /* byte order. */

	 if (fcb->dtype == 3 || fcb->dtype == 11 || fcb->dtype == 12)
      	    set_resc_parms (0, 500, 0, 250, 0, 249);  /* Btotal, Btan or Brt. */
         else
	 if (fcb->dtype == 10)
            set_resc_parms (0, 500, 0, 250, 0, 249);	/* pB. */
         else
	 if (fcb->dtype == 13)
            set_resc_parms (0, 250, 0, 250, 0, 249);	/* Bradial. */
         else
            set_resc_parms (0, 250, 0, 250, 0, 249);	/* default. */

	 /*** If this is a polaroid, rescaling is slightly different. ***/

      	 if (fcb->ipw != 0)  set_resc_parms (0, 255, 0, 249, 0, 249);
      	 gr_resc (img, fcb->dx * fcb->dx);
/*         for (i=0; i<imgsiz; i++)  *(fcb->addr + i) = *(img + i); */
         }
      else
         for (i=0; i<imgsiz; i++)  *(img + i) = *(fcb->addr + i);
      wbox (img, _GD.x + img_ip, _GD.y - img_ir, img_dx, img_dy, "rd");
      }

   else
   if (fcb->bppx == 4)
      {
      if (fcb->dtype == 5)	/* Difference image. */
         sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx, -5.e-9, 5.e-9);
      else
      if (fcb->dtype == 6)	/* Bk [K corona]. */
         sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx, 0.0, 5.e-9);
      else
      if (fcb->dtype == 7)	/* (Bcme - Bbkg) / Bk. */
         sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx, -1.0, 1.0);
      else
      if (fcb->dtype == 8)	/* (Bcme - Bbkg) / Bbkg. */
         sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx, -0.5, 0.5);
      else
      if (fcb->dtype == 9)	/* Polarization Fraction. */
         sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx,  0.0, 1.0);
      else
      if (fcb->dtype == 14)	/* F + Stray mask. */
	 sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx, 0.0, 2.5e-9);
      else
      if (fcb->dtype == 15)	/* Ratio: B1 / B2. */
	 sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx, 0.0, 2.0);
      else
         sfimg (fcb->addr, _GD.x + img_ip, _GD.y - img_ir, img_dx,  0.0, 0.0);
      }

/*
 *   else
 *     {
 *     sprintf (stderr, "Invalid image. bytes/pixel: %d\n", fcb->bppx);
 *     }
 */

   /*** Relinquish storage for 'img' array. ***/

   free (img);
   return (0);
}

/*---------------------------------------------------------------------------
 * W A D D I M G ( )	- Write the image at address fcb->addr to the Grinnell
 *			  'win', performing bilinear interpolation to expand
 *			  or shrink the image, thereby forcing it to fit.
 *---------------------------------------------------------------------------
 */
wgrnimg (win, fcb)

struct IMAGE *fcb;
struct GRINNELL *win;
{
    bilint (win, fcb);
    return (0);
}


/*----------------------------------------------------------------------------
 * E R A I M G ( )	- Erase an image from the default Grinnell channel.
 *----------------------------------------------------------------------------
 */
eraimg (fcb)

struct IMAGE *fcb;
{
   if (fcb->ires == 1)
      ebox (_GD.x, _GD.y, fcb->dx, fcb->dy, "rd");
   else
      ebox (_GD.x/2, _GD.y/2, fcb->dx/2, fcb->dy/2, "rd");
}


/*----------------------------------------------------------------------------
 * W E R A I M G ( ) 	- Erase the Grinnell image area defined by 'win'.
 *----------------------------------------------------------------------------
 */
weraimg (win, fcb)

struct GRINNELL *win;
struct IMAGE *fcb;
{
   ebox (win->x, win->y, win->dx, win->dy, "rd");
}


/*----------------------------------------------------------------------------
 * G E T I M G ( )	- Get an image from the default Grinnell channel and
 *			  store at the location pointed to by 'fcb->addr'.
 *----------------------------------------------------------------------------
 */
getimg (fcb)

struct IMAGE *fcb;
{
   if (fcb->ires == 1)
      rbox (fcb->addr, _GD.x, _GD.y, fcb->dx, fcb->dy, "rd");
   else
      rbox (fcb->addr, _GD.x/2, _GD.y/2, fcb->dx/2, fcb->dy/2, "rd");
}

/*----------------------------------------------------------------------------
 * W G E T I M G ( )	- Get an image from the Grinnell window 'win' and store 
 *			  it at the address pointed to by fcb->addr.
 *----------------------------------------------------------------------------
 */
wgetimg (win, fcb)

struct GRINNELL *win;
struct IMAGE *fcb;
{
   if (fcb->ires == 1)
      rbox(fcb->addr, win->x, win->y, win->dx, win->dy, "rd");
   else
      rbox(fcb->addr, win->x/2, win->y/2, win->dx/2, win->dy/2, "rd");
} 

/*----------------------------------------------------------------------------
 * S A V I M G ( )	- Get an image from the default Grinnell channel and
 *			  write it to the file filename.
 *----------------------------------------------------------------------------
 */
savimg (win, filename)

struct GRINNELL *win;
char *filename;
{
   chsave (filename, 0, _GD.x, _GD.y, _GD.dx, _GD.dy, "rd");
}


/*----------------------------------------------------------------------------
 * W S A V I M G ( )	- Get an image from the Grinnell window defined by 'win'
 *			  and write it to the file filename.
 *----------------------------------------------------------------------------
 */
wsavimg (win, filename)

struct GRINNELL *win;
char *filename;
{
   chsave (filename, 0, win->x, win->y, win->dx, win->dy,"rd");
}
	

/*---------------------------------------------------------------------------
 * G D E L I M G ( )	- Free an image control structure and the associated
 *			  image, returning the space to the heap.
 *---------------------------------------------------------------------------
 */
gdelimg (img)

struct IMAGE *img;
{
   if (img->addr != NULL)
      free ( img->addr );
   free ( (char *) img->name);

   free ( (struct IMAGE *) img);
}


/*----------------------------------------------------------------------------
 * G D E L W I N ( )	- Free a screen window structure, returning the 
 *			  space to the heap.
 *----------------------------------------------------------------------------
 */
gdelwin (win)

struct GRINNELL *win;
{
   free ((struct GRINNELL *) win);
}
