#include <stdio.h>
#include "iproc.ext"

/* 
 *
 * Read ATM image from disk file into memory array.
 *
 * Alice Lecinski
 * 1 May 1984
 *
 *  Modified 01/17/85 by Kevin Cole.
 *    Made compatible with the image processing library iproc.a.
 */

char	*malloc();

IMAGE *makeatm(namimg)
char *namimg;	
{ 
    FILE *imgfd, *fopen();
    struct IMAGE *ifcb;
    int   imgsiz, nread;
    static char itype[] = "ATM";
	
   register   char  *image;	/* Pointer to image array. */
   register   int   imgfd;	/* File descriptor for image disk file. */
   register   int   imgsiz;     /* Image size (bytes) (excluding hskp). */
   register   long  nread;

    /* Allocate space for the image file control block
     */
    ifcb = (struct IMAGE *) malloc(sizeof(struct IMAGE));
    ifcb->name = (char *) malloc(strlen(filename) + 1);
    ifcb->type = (char *) malloc(strlen(itype) + 1);
    ifcb->addr = NULL;

    strcpy(ifcb->type, itype);
    strcpy(ifcb->name, filename);
    ifcb->degnor = 00.0;		/* offset of polar axis relative to */
					/* screen vertical   		    */
    /* Open image file. 
     */
    if((imgfd = fopen(filename, "r")) == NULL)
	return(NULL);

    /* Decommutate Housekeeping. 
     */
       atmhk(namimg);


    /* Position I/O pointer to beginning of video data in image disk file. 
     */
    if (lseek(fileno(imgfd), 172L, 0) == -1)
    {
	fprintf(stderr,"makeatm(): seek error.\n");  
	return(NULL); 
    }
       
    imgsiz = ifcb->dx * ifcb->dy * ifcb->bppx;  

    /* allocate space for the image */
    if((ifcb->addr = (unsigned char *) malloc(imgsiz)) == NULL)
    { 
	fprintf(stderr,"makeatm(): memory allocation error.");
	return(NULL);
    }
    if((nread = read(fileno(imgfd),ifcb->addr,imgsiz)) < imgsiz)
    {
	fprintf(stderr,"Read returned nread = %d\n", nread);
       	return(NULL); 
    }

    if (fclose(imgfd) == -1)
    { 
	fprintf(stderr,"rdimg> file close error.%s\n", ifcb->name); 
	return(NULL);
    }

    /* Define the default display area on a 512x512 image display screen    */
    _GD.y  = 510;		/* initial y pixel on the display screen    */
    _GD.x  = 63;		/* initial x pixel on the display screen    */
    _GD.dy = 448;		/* image extent in x pixels 		    */
    _GD.dy = 448;		/* image extent in x pixels 		    */

    return(image);
}


/* atmhk(namimg)
 *
 * Routine to decode ATM housekeeping.
 *
 * (Somehow, somewhere, someway) written by Alice Lecinski  8 May 1984
 */

#define	MAXHSK	172
#include <stdio.h>
extern struct hskp H;

static float pixrs=40.0;	/* # Pixels per solar radius. */
static float rspix=0.025;	/* # Solar radii per pixel. */
float degrad;           	/* # Degrees per solar radius. */
float raddeg;           /* # Solar radii per degree. */
int npix;           	/* # pixels per vidicon line. */
int nrec;               /* # number of records in the image */
int nbypix;           	/* # bytes per pixel. */
static int idtype=0;	/* Data type: 0 = ATM */
static int nave=1;	/* # points to average in scan. */
float roll;

/* $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ $ */
              /* ATM Housekeeping Decoder */ 
atmhk(ifc, imgfd)	
struct IMAGE *ifc;		/* Image file name.      */
FILE *imgfd;			/* Image file descriptor */
{
     short int mark, ystart, xstart, start, words, recsize, records;
     short int iroll, xcenM, ycenM;
     int n, ii, jj, kk;
     static float secrad = 960.;	/* arcseconds per solar radius. */
     static float secpix = 24.0;    /* arcseconds per pixel (50micron scan). */
     char ans[2];
     float pi = 3.141592654;
     char *c;		/* Housekeeping array (character). */
     char *malloc();	/* system allocation routine. */
     FILE *imgfd, *fopen(), *fclose();
     int fread();	/* Type declaration for system function. */
     
        /* Open image file and read housekeeping information. */
        if((imgfd = fopen(namimg,"r")) == NULL)  
               return(-1); 

        /* Allocate storage for housekeeping array. 
	 */
        if((c = malloc(MAXHSK)) == NULL) {
	   fprintf(stderr,"atmhk> malloc error.\n") ;
	   fflush(stderr);
           return(-2);
	}
        if(fread(c, sizeof(char), 172, imgfd) == NULL)
	{ 
	   fclose(imgfd); return(-1); 
        }
        fclose(imgfd);

	mark = (( *(c+1) & 0377 )<<8) + (*(c+0) & 0377);
	ystart = (( *(c+3) & 0377)<<8) +  (*(c+2) & 0377);
	xstart = (( *(c+5) & 0377)<<8) +  (*(c+4) & 0377);
	start =  (( *(c+7) & 0377)<<8) +  (*(c+6) & 0377);
	ifc->dx =     (( *(c+9) & 0377)<<8) +  (*(c+8) & 0377);
	ifc->dy =     (( *(c+11) & 0377)<<8) + (*(c+10) & 0377);
	words =  (( *(c+13) & 0377)<<8) + (*(c+12) & 0377);
	recsize= (( *(c+15) & 0377)<<8) + (*(c+14) & 0377);
	records= (( *(c+17) & 0377)<<8) + (*(c+16) & 0377);
	iroll =   (( *(c+157) & 0377)<<8) + (*(c+156) & 0377);
	xcenM =  (( *(c+159) & 0377)<<8) + (*(c+158) & 0377);
	ycenM =  (( *(c+161) & 0377)<<8) + (*(c+160) & 0377);
     
        roll = (float) iroll/60.0;
	ifc->xcen = (float) (xcenM)/((float)dx);
	ifc->ycen = (float) (ycenM)/((float)dx);  /* this is a test dx==dy */

	ifc->year = -1;	  	/* year not defined		  */
	ifc->doy = -1;	  	/* day of year not defined 	  */
	ifc->hour = -1;	  	/* hours not defined		  */
	ifc->minute = -1;  	/* minute			  */
	ifc->sec = -1;	  	/* seconds			  */
        ifc->frameno = -1; 	/* Frame number	not defined       */ 
	ifc->ism = -1;		/* sector mirror not defined      */
	ifc->icm = -1;		/* calibration mirror not defined */
	ifc->ipw = -1;		/* polaroid wheel not defined	  */
	ifc->quan = 100.;	/* image quantity is 100 %	  */
  
	ifc->pixsr = secrad;
	ifc->bppx = 2;			/* 2 bytes per pixel */
	ifc->bytord = 0;		/* Sun-4 byte order. */
     
        /* Compute start/end pixel/line coordinates for ATM image. 
	 */
 	ifc->ip = 0;
 	ifc->lp = 895;
 	ifc->ir = 0;
 	ifc->lr = 895;
 	ifc->ires = 0;

        return(0);
     }
