#include "iproc.ext"

/* rscan(image, rmin, rmax, rinc, theta, scan, scandx, ns)
 *
 * Radial Scan.
 *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984
 *
 * Include structure definitions from iproc.a 10/30/84 - Kevin Cole
 * Changed scntyp from pointer to value       12/10/84 - Kevin Cole
 * added to iproc.a         		       1/11/84 - Kevin Cole
 *-----------------------------------------------------------------------------
 */

rscan(icb, rmin, rmax, rinc, theta, scntyp, scan, scandx, ns)
struct IMAGE  *icb;     /* Pointer to image structure.			*/
float   rmin, rmax;     /* Solar radius limits for radial scan. 	*/
float   rinc;           /* Radial increment for radial scan. 		*/
float   theta;          /* Theta (degrees). 				*/
int     scntyp;         /* Pointer to scan type (1=radial,2=line). 	*/
float  scan[];          /* Dependent axis scan array. 			*/
float  scandx[];        /* Independent axis scan array. 		*/
int     *ns;            /* Number of values in scan. 			*/
{
    extern float    blint();
    float    x, y, r, th, dstsun, radinc, temp;
    register int  ir;
    int      ix, iy, ixg, iyg, ierr;
    int      MAXSCN = 1296;

/* Radial increment: set to 0.1 Rsun if 'rinc' <= 0. */

   radinc = rinc;
   if(rinc <= 0.)  radinc = 0.1;
   if(rmax < rmin)
   {
	temp = rmax;
	rmax = rmin;
	rmin = temp;
   }
   th = theta;

   dstsun = -radinc;

    /* ****** Radial Scan ****** 
     */
   for (ir = 0; ir < MAXSCN; ir++)  *(scan + ir) = *(scandx + ir) = 0.;

    /* ....... Radial scan loop ....... 
     */
   *ns = 0;
   ir = -1;
   for (r = rmin; r <= rmax; r += radinc)
     {
      ir++;
 
     /* Coordinate transformation :  (theta,angle) --> (pixel,line). 
      */
      if((ierr = coord(icb,&r,&th,&x,&y,1)) != 0)  return(ierr);

    /* Perform bilinear interpolation to compute scan point value from image. 
     * Use 4 nearest neighbors. 
     */
      dstsun += radinc;
      if(scntyp == 1)  *(scandx + ir) = (float) r;
      else if(scntyp == 2)  *(scandx + ir) = (float) dstsun;

      ix = (int) (x + 0.5);
      iy = (int) (y + 0.5);

    /* convert to frame coordinates 
     */
    x = x - icb->ip;  y = y - icb->ir;

    /* clip the coordinates 
     */
    if((x >= 0.0) && (x <= icb->dx) && (y >= 0.0) && (y <= icb->dy))
        *(scan+ir)=(float) (blint(icb,&x,&y)*icb->scale);
    else
    	*(scan+ir)= 0.0;

    /*  Divide scan values by 5 for equivalent second data type.
     *  ( numbers  in  file are  5 * equivalent second  units. ) 
     */
      if(icb->dtype == 2) *(scan + ir) /= 5.;
   }
   *ns = ir + 1;
   return(0);
}


