/* sfimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rescale intensities for floating-point image to the range 0 -> 250.
 * Display rescaled C/P image on Grinnell-274.
 * Image is assumed to be square.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   Associate Scientist   HAO/NCAR
 * 15 January 1987
 * 12 April 1990: add arguments to allow specification of fmin & fmax.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

sfimg (image, x1, y1, dx, fimin, fimax)

char *image;			/* Floating point image array address. */
int x1, y1;			/* (x,y) coordinates of image origin. */
int dx;				/* # pixels per horizontal line. */
float fimin, fimax;		/* Minimum & maxiumum value range of interest.*/
{
int min = 4096;
int max = -1;
register short *ims;
register float *fimage;
int i;
int nelem;
int nbytes;
register int ix, iy;
register int npline;
float offset = 0.;
float slope = 1.e-16;
float fmin = 1.e+20;
float fmax = -1.e+20;
float gmax = 249.0;
float gmin =   0.0;
register float value;
FILE *ifd;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   npline = dx;
   nelem = npline * npline;
   nbytes = nelem * sizeof (float);

/* Allocate storage and read image into memory array. */

   fimage = (float *) image;
   ims = (short *) malloc (nbytes/2);

/* Find min & max values in image. */

   if (fimin == 0.0 && fimax == 0.0)
      for (i=0; i<nelem; i++)
         {
         if (*(fimage + i) < fmin)  fmin = *(fimage + i);
         if (*(fimage + i) > fmax)  fmax = *(fimage + i);
         }
   else
      {
      fmin = fimin;
      fmax = fimax;
      }
/*
 *      fprintf (ifd, "fmin:   %g   fmax:  %g\n", fmin, fmax);
 */

   if (fmax == fmin)
      {
      fprintf (stderr, "dfimg> error: fmax = fmin");
      return (-1);
      }

   ifd = fopen ("img.dat", "w");

/*   if (fmin == -1.0)  fmin = 0.; */

   if (fmin < 0.)
      {
      if (-fmin > fmax)  fmax = -fmin;
      else fmin = -fmax;
      }

   offset = fmin;
   slope = 250.0 / (fmax - fmin);

   fprintf (ifd, "fmin:   %g   fmax:  %g\n", fmin, fmax);
   fprintf (ifd, "offset: %g   slope: %g\n", offset, slope);
/*
 *   fprintf (ifd, "Rescaling intensities for Grinnell display...please wait.\n");
 */

/* Perform intensity scaling for Grinnell. */

   for (iy=0; iy<npline; iy++)
      for (ix=0; ix<npline; ix++)
         {
         value = (*(fimage + ix + iy*npline) - offset) * slope;
         if (value > gmax)  value = gmax;
         if (value < gmin)  value = gmin;
         *(ims + ix + iy*npline) = ((short) value) & 0377;
         }

   wbox (ims, x1, y1, npline, npline, "rd");

   free (ims);
   fclose (ifd);
   return (0);
}
