#include "iproc.ext"
#define MAXSCN 1296

/*------------------------------------------------------------------------------
 * tscan(image, thmin, thmax, thinc, radius, scan, scandx, ns)
 *
 * Theta Scan.
 *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984
 * 
 * Modified 10/30/84  To use image structures from iproc.a    - Kevin Cole
 * Modified  1/11/85  Added to image library iproc.a	      - Kevin Cole
 *------------------------------------------------------------------------------
 */

tscan(image, thmin,thmax,thinc,radius, scan, scandx, ns)
struct IMAGE *image;    /* Pointer to image array. 			*/
float   thmin, thmax;   /* Solar position angle limits for theta scan. 	*/
float   thinc;          /* Angular increment for theta scan. 		*/
float   radius;         /* Radius (R/Rsun units). 			*/
float   scan[];         /* Dependent axis scan array. 			*/
float   scandx[];       /* Independent axis scan array. 		*/
int     *ns;            /* Number of values in scan. 			*/
{
    extern float blint();
    float   x, y;
    float   r, th;
    float   tinc;
    register int     it;
    int     ierr;
    float raddeg = 1.7453293e-2;	/* Degrees to radians  */
    float degrad = 57.29577951;		/* Radians to degrees  */

   /* Theta increment: set default to angle separating two adjacent pixels. 
    */
   tinc = thinc;
   if(thinc <= 0.)  tinc = degrad / (radius*image->pixsr);

   r = radius;

   /* Clear scan arrays. 
    */
   for (it = 0; it < MAXSCN; it++)  *(scan + it) = *(scandx+ it) = 0.;
 
   /* ******* Theta scan loop ******* 
    */
   if(thmax < thmin)  thmax += 360.;

   *ns = 0;
   it = -1;
   for (th = thmin; th <= thmax; th += tinc)
   {
      it++;
 
      /* Coordinate transformation :  (radius,angle) --> (pixel,line). 
       */
      if((ierr = coord(image, &r,&th,&x,&y,1)) != 0)  return(ierr);

    /* Perform bilinear interpolation to compute scan point from image. 
     */
    *(scandx + it) = (float) th;
    x = x - image->ip;  y = y - image->ir;
    if((x >= 0.0) && (x <= image->dx) && (y >= 0.0) && (y <= image->dy))
        *(scan+it)=(float) (blint(image,&x,&y)*image->scale);
    else
	*(scan+it) = 0.0;
   }
   *ns = it + 1;
 
    /*   Divide scan values by 5 for equivalent second data type.
     * ( numbers  in file  are   5 * equivalent second   units. ) 
     */
   if(image->dtype == 2)  for(it = 0; it < *ns; it++) scan[it] /= 5.;

   return(0);
}
