#include <sys/time.h>
#include <sys/types.h>

/*
 * This routine converts time as follows.
 * The epoch is 0000 Jan 1 1970 GMT.
 * The argument time is in seconds since then.
 * The localtime(t) entry returns a pointer to an array
 * containing
 *  seconds (0-59)
 *  minutes (0-59)
 *  hours (0-23)
 *  day of month (1-31)
 *  month (0-11)
 *  year-1970
 *  weekday (0-6, Sun is 0)
 *  day of the year
 *  daylight savings flag
 *
 * The routine calls the system to determine the local
 * timezone and whether Daylight Saving Time is permitted locally.
 * (DST is then determined by the current local rules)
 *
 * The routine LOCAL_TIME corrects for daylight saving time and will work in
 * any time zone provided "TIME_ZONE" is adjusted to the difference between
 * Greenwich and local standard time (measured in seconds).  In places like
 * Michigan "DAY_LIGHT" must be initialized to 0 to prevent the conversion
 * to daylight time.
 *
 * The routine does not work
 * in Saudi Arabia which runs on Solar time.
 *
 * asctime (tvec))
 * where tvec is produced by localtime
 * returns a ptr to a character string
 * that has the ascii time in the form
 *       Thu Jan 01 00:00:00 1970n0\\
 *       01234567890123456789012345
 *       0         1         2
 *
 * ctime(t) just calls localtime, then asctime.
 */

#define UNIXEPIC 2440588	/* 1/1/1970 */

static char cbuf[26];
int dmsize[12] = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
int TIME_ZONE = 7*60*60;  /* changed from est to mst */
int DAY_LIGHT = 1;     /* USA daylight conversion */

char *
ctime (at) long *at; {
    extern char *asctime();
    extern struct tm *localtime();

    return (asctime (localtime (at)));
}

struct tm *
localtime (tim) long *tim; {
    struct timeval systime[1];
    struct timezone zone[1];
    extern struct tm *LOCAL_TIME();

    gettimeofday (systime, zone);
    TIME_ZONE = zone->tz_minuteswest * 60;
    DAY_LIGHT = zone->tz_dsttime;

    return (LOCAL_TIME (tim));
}

struct tm *
LOCAL_TIME (tim) long *tim; {
    register int doy, yr;
    register struct tm *ct;
    register int begdls, enddls;
    extern struct tm *gmtime();
    static long copyt;

    copyt = *tim - TIME_ZONE;
    ct = gmtime(&copyt);
    doy = ct->tm_yday;
    yr = ct->tm_year + 1900;

    switch (DAY_LIGHT) {  /* Berzerkeley likes unreadable tables, I don't */
    case DST_USA:
	if (yr == 1974) {	/* 1/6/74 - 11/24/74 == doy 6 - 328 */
	    begdls = 6;    enddls = 328; }
	else if (yr == 1975) {	/* 2/23/75 - 10/26/75 == doy 54 - 299 */
	    begdls = 54;    enddls = 299; }
	else if (yr < 1988) {	/* last Sun in Apr to Last Sun in Oct. */
	    begdls = iday (4,30,yr) - izlr (4,30,yr);
	    enddls = iday (10,31,yr) - izlr (10,31,yr); }
	else {   /* First Sun in Apr to Last Sun in Oct.  */
	    begdls = iday (4,7,yr) -izlr (4,7,yr);
	    enddls = iday (10,31,yr) - izlr (10,31,yr); }
fixdls:
	if ( (doy > begdls || (doy == begdls && ct->tm_hour >= 2))
	  && (doy < enddls || (doy == enddls && ct->tm_hour <  1))) {
	    copyt += 3600;
	    ct = gmtime (&copyt);
	    ct->tm_isdst++; }
	break;
    case DST_AUST:
	if (yr <= 1970) break;
	else if (yr == 1971) {	/* 10/31/71 - 12/31/71 == doy 304 - 365 */
	    begdls = 304;    enddls = 0; }
	else if (yr == 1972) {	/* 1/1 - 2/27 && 10/29 - 12/31 == 58 - 303 */
	    begdls = 303;    enddls = 58; }
	else {	/* last Sun Oct to first Sun in Mar ???. */
	    enddls = iday (4,7,yr) - izlr (4,30,yr);
	    begdls = iday (10,31,yr) - izlr (10,31,yr); }
	if ( (doy > begdls || (doy == begdls && ct->tm_hour >= 2))
	  || (doy < enddls || (doy == enddls && ct->tm_hour <  2))) {
	    copyt += 3600;
	    ct = gmtime (&copyt);
	    ct->tm_isdst++; }
	break;
/*
 * The European tables ... based on hearsay
 * Believed correct for:
 *	WE:	Great Britain, Ireland, Portugal
 *	ME:	Belgium, Luxembourg, Netherlands, Denmark, Norway,
 *		Austria, Poland, Czechoslovakia, Sweden, Switzerland,
 *		DDR, DBR, France, Spain, Hungary, Italy, Jugoslavia
 * Eastern European dst is unknown, we'll make it ME until someone speaks up.
 *	EE:	Bulgaria, Finland, Greece, Rumania, Turkey, Western Russia
 */
    case DST_WET:
	if (yr <= 1982) break;
	else {	/* last Sun in March to Last Sun in Oct. */
	    begdls = iday (3,31,yr) - izlr (3,31,yr);
	    enddls = iday (10,31,yr) - izlr (10,31,yr); }
	goto fixdls;
    case DST_EET:
    case DST_MET:
	if (yr <= 1982) break;
	else {	/* last Sun in March to Last Sun in Sept. */
	    begdls = iday (3,31,yr) - izlr (3,31,yr);
	    enddls = iday (9,30,yr) - izlr (9,30,yr); }
	goto fixdls;
    }
    return (ct);
}

struct tm *
gmtime (tim) long *tim; {
    register long hms, day;
    static struct tm xtime[1];

    /*
     * break initial number into days
     */

    hms = *tim % 86400;
    day = *tim / 86400;
    if (hms < 0) { hms += 86400; day -= 1; }

    /*
     * generate hours:minutes:seconds
     */

    { register int d, *tp = (int *) xtime;
      *tp++ = hms % 60;
      d = hms / 60;
      *tp++ = d % 60;
      d /= 60;
      *tp++ = d;
    }

    /*
     * day is the day number.
     * generate day of the week.
     * The addend is 4 mod 7 (1/1/1970 was Thursday)
     */

    xtime->tm_wday = (day + 7340036) % 7;

    /*
     * year number
     */

    jdate (day + UNIXEPIC, &xtime->tm_mon, &xtime->tm_mday, &xtime->tm_year);
    xtime->tm_yday = iday (xtime->tm_mon, xtime->tm_mday, xtime->tm_year) - 1;

    xtime->tm_mon--;
    xtime->tm_year -= 1900;
    xtime->tm_isdst = 0;
    return (xtime);
}

static char *
ct_numb (cp, n) register char *cp; {
         
         cp++;
         if (n >= 10) *cp++ = (n / 10) % 10 + '0';
         else *cp++ = ' ';
         *cp++ = n % 10 + '0';
         return (cp);
}

char *
asctime (t) register struct tm *t; {
    register char *cp, *ncp;
    register int *tp;

    cp = cbuf;
    for (ncp = "Day Mon 00 00:00:00 1900\n"; *cp++ = *ncp++;);
    ncp = &"SunMonTueWedThuFriSat"[3*t->tm_wday];
    cp = cbuf;
    *cp++ = *ncp++; *cp++ = *ncp++; *cp++ = *ncp++; cp++;
    tp = &t->tm_mon;
    ncp = &"JanFebMarAprMayJunJulAugSepOctNovDec"[(*tp)*3];
    *cp++ = *ncp++; *cp++ = *ncp++; *cp++ = *ncp++;
    cp = ct_numb (cp, *--tp); cp = ct_numb (cp, *--tp+100);
    cp = ct_numb (cp, *--tp+100); cp = ct_numb (cp, *--tp+100);
    if (t->tm_year >= 100) {
	    cp[1] = '2';
	    cp[2] = '0';
    }
    cp += 2;
    cp = ct_numb (cp, t->tm_year+100);
    return (cbuf);
}
