#include <sys/time.h>

/*
 * time_t
 * unixtime (mo, day, yr, hr, min, sec)
 *
 *	0 <= mo  <= 11
 *	1 <= day <= 31
 *	? <= yr  <= 206	relative to 1900
 *	0 <= hr  <= 23
 *	0 <= min <= 59
 *	0 <= sec <= 59
 *
 * return the unix time.
 *
 * Bugs:  1-2AM is ambiguous converting back to standard time.
 *	  Treated as std time.
 */

#define SECPERDAY 86400		/* 24*60*60 */
#define UNIXEPIC 2440588	/* 1/1/1970 */
#define _1AM 3600
#define _2AM 7200

extern int DAY_LIGHT;	/* set if correcting for daylight time */
extern int TIME_ZONE;	/* seconds to Greenwich time */

long
unixtime (mo, day, yr, hr, min, sec) register int yr; {
    register long t;
    register int doy, tz;
    register int begdls, enddls;
    extern int jday(), iday(), izlr();

    yr += 1900;    mo++;
    tz = TIME_ZONE;
    t = (((hr * 60) + min) * 60) + sec;

    /* daylight savings time correction */

    if (DAY_LIGHT) {
	doy = iday (mo, day, yr);
	switch (DAY_LIGHT) {  /* Berzerkeley likes unreadable tables, I don't */
	case DST_USA:
	    if (yr == 1974) {	/* 1/6/74 - 11/24/74 == doy 6 - 328 */
		begdls = 6;    enddls = 328; }
	    else if (yr == 1975) {	/* 2/23/75 - 10/26/75 == doy 54 - 299 */
		begdls = 54;    enddls = 299; }
	    else if (yr < 1988) {	/* last Sun in Apr to Last Sun in Oct. */
		begdls = iday (4,30,yr) - izlr (4,30,yr);
		enddls = iday (10,31,yr) - izlr (10,31,yr); }
	    else {    /* First Sunday in Apr to Last Sun in Oct. */
	      begdls = iday (4,7,yr) - izlr (4,7,yr);
	      enddls = iday (10,31,yr) - izlr (10,31,yr); }
    fixdls:
	    if ( (doy > begdls || (doy == begdls && t >= _2AM))
	      && (doy < enddls || (doy == enddls && t <  _1AM)))
		tz -= 3600;
	    break;
	case DST_AUST:
	    if (yr <= 1970) break;
	    else if (yr == 1971) { /* 10/31/71 - 12/31/71 == doy 304 - 365 */
		begdls = 304;    enddls = 0; }
	    else if (yr == 1972) { /* 1/1 - 2/27 && 10/29 - 12/31 == 58 - 303 */
		begdls = 303;    enddls = 58; }
	    else {	/* last Sun Oct to first Sun in Mar ???. */
		enddls = iday (4,7,yr) - izlr (4,30,yr);
		begdls = iday (10,31,yr) - izlr (10,31,yr); }
	    if ( (doy > begdls || (doy == begdls && t >= _2AM))
	      || (doy < enddls || (doy == enddls && t <  _2AM)))
		tz -= 3600;
	    break;
/*
 * The European tables ... based on hearsay
 * Believed correct for:
 *	WE:	Great Britain, Ireland, Portugal
 *	ME:	Belgium, Luxembourg, Netherlands, Denmark, Norway,
 *		Austria, Poland, Czechoslovakia, Sweden, Switzerland,
 *		DDR, DBR, France, Spain, Hungary, Italy, Jugoslavia
 * Eastern European dst is unknown, we'll make it ME until someone speaks up.
 *	EE:	Bulgaria, Finland, Greece, Rumania, Turkey, Western Russia
 */
	case DST_WET:
	    if (yr <= 1982) break;
	    else {	/* last Sun in March to Last Sun in Oct. */
		begdls = iday (3,31,yr) - izlr (3,31,yr);
		enddls = iday (10,31,yr) - izlr (10,31,yr); }
	    goto fixdls;
	case DST_EET:
	case DST_MET:
	    if (yr <= 1982) break;
	    else {	/* last Sun in March to Last Sun in Sept. */
		begdls = iday (3,31,yr) - izlr (3,31,yr);
		enddls = iday (9,30,yr) - izlr (9,30,yr); }
	    goto fixdls;
	}
    }

    t += (jday (mo, day, yr) - UNIXEPIC) * SECPERDAY + tz;
    return (t);
}

unixtime_(mo,day,yr,hr,min,sec) /* fortran callable version */
int *mo,*day,*yr,*hr,*min,*sec;
	{
	return (unixtime(*mo,*day,*yr,*hr,*min,*sec));
	}
