/* glmk3_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  Display an MLSO Mark-III image ('filename') in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P   4 January 1994
 * 25 Feb 94: default scaling uses MK3I->smin & MK3I->smax.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/mk3lib.ext"
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/gl_color.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glmk3_view (filename, lfull, pmode)

char *filename;		/* Pointer to filename array. */
int lfull;		/* Image label flag; ON if =1. */
int pmode;		/* Print mode. */

{
  extern int glmk3_scalers ();	/* Function to draw solar radius scale. */
  extern int glmk3_north ();	/* Function to draw pointer at sun center. */
  extern int glmk3_suncir ();	/* Function to draw circle at 1.0 Rsun. */
  extern int glmk3_label ();	/* Function to label Mark-III frame. */
  extern int glmk3_annot ();	/* Function to write Mark-III annotation. */
  extern int gl_wimg ();	/* Function to write image to GL window. */
  extern int wdisk ();		/* Function to write black disk. */
  extern int wdiskb ();		/* Function to write black disk. */
  extern int srevbyt ();	/* Reverses bytes (short int images)*/
  extern int DSP;		/* Grinnell usage flag: =0 off, =1 on. */
  extern char *malloc ();

  int mk3_dim;			/* Mark-III x/y dimension. */
  int cx, cy, radius;
  int fast = 1;
  int fin, xs, ys, nbyimg;
  int gpixrs;
  int idtype ;
  int imin = 0, imax = 0;
  int nbypix;			/* # bytes per pixel. */
  int xdim, ydim;		/* Image size. */
  float smin = 0.0, smax = 0.0;
  float bkgcol = 0.0;
  float xcen, ycen;		/* Image center. */
  char *imgbuf;
  static char dir2 [] = "ru";

   if (!DSP)
      {
      fprintf (stderr, "glmk3_view> display option is not selected.\n");
      return (-1);
      }

   /*** Open C/P image. ***/

   if ( (fin = glmk3_open (filename, &nbypix, &xdim, &ydim, &xs, &ys,
			   &xcen, &ycen, pmode)) == -1 )
      {
      fprintf (stderr, "glmk3_view> Unable to open file: %s\n", filename);
      return (-2);
      }

   idtype = MK3I->dtype ;

   if (pmode > 0)
      {
      fprintf (stdout, "glmk3_view> xs: %d   ys: %d   idtype: %d\n",
	       xs, ys, idtype);
      }

   /*** Read image into memory. ***/

   nbyimg = xdim * ydim * nbypix;
   imgbuf = malloc ( nbyimg );
   if (read (fin, imgbuf, nbyimg) <= 0 )
      {
      fprintf (stderr, "glmk3_view> read error.");
      return (-3);
      }
   close ( fin );

   /*** Clear area below image. ***/

   if (pmode > 0)
      fprintf (stdout, "glmk3_view> nbypix: %d\n", nbypix);

   color (gl_black);
   rectfi (xs, ys, xs + xdim - 1, ys + ydim - 1);

   /*** Erase GL window. ***/

   if (lfull) gl_wclear (gl_black);

   /*** Display image in GL window. ***/

   if ( nbypix == 1 )		/* 1 byte per pixel. */
      {
      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 smin = (float) oldmin;
	 smax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
         {
	 smin =   0.0;
	 smax = 250.0;

	 if (idtype ==  2)   {smin =     0.0; smax =   250.0;}  /* srq  */
	 if (idtype ==  5)   {smin =     0.0; smax =   250.0;}  /* sri  */
	 if (idtype == 11)   {smin =     0.0; smax =   250.0;}  /* srpB */

         smin = (float) MK3I->smin ;
         smax = (float) MK3I->smax ;
	 imin =   0;
	 imax = 250;
         }
      MK3I->smin = smin;
      MK3I->smax = smax;
      gl_wimg (imgbuf, xs, ys, xdim, ydim, nbypix, smin, smax, imin, imax,
               dir2);
      }

   else if (nbypix == 2)	/* 2 bytes per pixel (short integer). */
      {
      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 smin = (float) oldmin;
	 smax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else			/* Default short integer intensity scaling. */
         {
	 smin =   0.0;
	 smax = 500.0;

	 if (idtype ==  0)   {smin = - 500.0; smax =  5000.0;}  /*   q  */
	 if (idtype ==  1)   {smin = - 500.0; smax =  5000.0;}  /*  rq  */

	 if (idtype ==  3)   {smin = - 500.0; smax =  5000.0;}  /*   i  */
	 if (idtype ==  4)   {smin = - 500.0; smax =  5000.0;}  /*  ri  */

	 if (idtype ==  6)   {smin = -5000.0; smax =  5000.0;}  /*   u  */
	 if (idtype ==  7)   {smin = -5000.0; smax =  5000.0;}  /*  ru  */

	 if (idtype ==  9)   {smin = - 250.0; smax =  2500.0;}  /*  pB */
	 if (idtype == 10)   {smin = - 500.0; smax =  7500.0;}  /*  rpB */

         smin = (float) MK3I->smin ;
         smax = (float) MK3I->smax ;
	 imin =   0;
	 imax = 250;
	 if (pmode)
	    fprintf (stdout, "glmk3_view> smin/max: %7.2f %7.2f  imin/max: %3d %3d\n",
		     smin, smax, imin, imax);
         }
      MK3I->smin = smin;
      MK3I->smax = smax;
      gl_wimg (imgbuf, xs, ys, xdim, ydim, nbypix, smin, smax, imin, imax,
	       dir2);
      }

   else if (nbypix == 4)	/* 4 bytes per pixel (floating point). */
      {
      if (lscale == 1)
	 {
	 smin = (float) oldmin;
	 smax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
	 {
         smin = MK3I->smin ;
         smax = MK3I->smax ;
	 imin = 0;
	 imax = 250;
	 }
      MK3I->smin = smin;
      MK3I->smax = smax;
      gl_wimg (imgbuf, xs, ys, xdim, ydim, nbypix, smin, smax, imin, imax,
               dir2);
      }

   else
      {
      fprintf (stderr, "glmk3_view> Invalid image . . . nbypix: %d\n", nbypix);
      return (-4);
      }

   free (imgbuf);

   /*** Compute disk center in GL window coordinates. ***/

   cx = (int) ((xcen) + 0.5) + gMINROW;
   cy = (int) ((ycen) + 0.5) + gMINCOL;
   if (pmode > 0)
      {
      fprintf (stdout, "glmk3_view> gMINROW:%4d  gMINCOL:%4d  gMAXROW:%4d  gMAXCOL:%4d\n",
	       gMINROW, gMINCOL, gMAXROW, gMAXCOL);
      fprintf (stdout, "glmk3_view> cx: %3d   cy: %3d   xcen: %3d   ycen: %3d   \n",
           cx, cy, (int) xcen, (int) ycen);
      }

   /*** Compute radius (in pixels) for black disk to overlay occulting disk.***/
   /*** Draw black filled circle at occulting disk position. ***/

   gpixrs = (int) MK3I->pixrs;
   radius = (int) (1.0 * gpixrs);

   color (gl_black);
   circfi (cx, cy, radius);

   /*** Draw solar radius scale. ***/
   /*** Draw pointer to mark center of sun & indicate north direction. ***/

   if (ldisk == 0)  
      {
      glmk3_north (cx, cy, 0.1, 0.3, gl_yellow);
      glmk3_scalers (gl_yellow);
      glmk3_suncir (0.4, 1.0, 0.1, 90.0, 0.4, 0.8, 0.2, 30.0, gl_yellow);
      }

   /*** Write information labels if lfull = 1. ***/
   /*** Write configuration information. ***/

   glmk3_label (lfull, ldisk, gl_grey, gl_green, gl_yellow);

   glmk3_annot (ldisk, gl_red, gl_grey, gl_green, gl_yellow);

   return (0);
}


/* glmk3_open.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Open MLSO Mark-III image file & decommutate selected items from housekeeping.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P   4 January 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

glmk3_open (filename, nbypix, xdim, ydim, xs, ys, xcen, ycen, pmode)

char *filename;			/* Image file name. */
int *nbypix;			/* # bytes per pixel. */
int *xdim, *ydim;		/* Image size. */
int *xs, *ys;			/* Image origin coordinates in window. */
float *xcen, *ycen;		/* Image center coordinates. */
int pmode;			/* Print mode. */

{
   extern int mk3hskp ();	/* Function to decode housekeeping. */

   int MK3_dim;			/* MK3 x/y dimension. */
   int MK3_imgno;		/* MK3 image index. */
   int fdimg;
   int nrechd;
   int i;
   int idot;
   long offset;

   mk3hskp (filename, pmode);	/* Decommutate selected housekeeping items. */
   *nbypix = MK3I->nbypx ;

   /*** Open image disk file. ***/

   if ((fdimg = open (filename,0)) == -1)
      {
      fprintf (stderr, "glmk3_open> can't open file: %s\n", filename);
      return (-1);
      }

      MK3_dim = 512;

   idot = -1;
   for (i = 0; i < strlen (filename);  i++)
      if (filename [i] == '.')  idot = i;

   if (idot < 0)
      {
      *nbypix = -1;
      MK3_dim = 0;
      MK3_imgno = 0;
      return (-4);
      }
   *xdim = *ydim = MK3_dim;

   /*** Position file pointer at beginning of image data. ***/

   if (pmode > 0)
      fprintf (stdout, "glmk3_open> MK3I->dtype: %d  MK3I->nbypx: %d\n",
               MK3I->dtype, MK3I->nbypx);
   offset = (long) 0;
   if ( lseek (fdimg, offset, 0) == -1 )
      {
      fprintf (stderr,"glmk3_open> seek error.\n");
      return (-5);
      }

   /*** Set origin coordinates of image. ***/

   gMINROW =  63;
   gMAXROW = 574;
   gMINCOL =  63;
   gMAXCOL = 574;

   *xs = gMINCOL;
   *ys = gMINROW;

   *xcen = 255.5;
   *ycen = 255.5;

   return (fdimg);
}
