/* glmk4_suncir.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Draw a dotted circular pattern at 1.0 Rsun.
 * For MLSO Mark-III images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   10 January 1994
 * 24 Nov 1998: adapt for Mk4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/stanger/lib/include/mk4lib.ext"
#include "/home/stanger/lib/include/img_scale.ext"

glmk4_suncir (r1min, r1max, r1inc, ang1inc,
	      r2min, r2max, r2inc, ang2inc, cindex)

int cindex;		/* Color LUT index. */

float r1min, r1max;	/* radial limits for major dotted lines. */
float r1inc;		/* radial increment for major dotted lines. */
float ang1inc;		/* angular increment between major lines.  */
float r2min, r2max;	/* radial limits for minor dotted lines. */
float r2inc;		/* radial increment for minor dotted lines. */
float ang2inc;		/* angular increment between minor lines.  */
{
   extern int nres;
   float x, y;		/* (x,y) coordinates corresponding to (r,th). */
   float r = 1.0;		/* Radius in R/Rsun units. */
   float th;		/* Position angle in degrees. */
   int xg, yg;		/* Grinnell coordinates. */
   int xg1, yg1, xg2, yg2;

   float radius;
   float x1, y1, x2, y2;
   float r1, r2;

   /*** Draw radial scans (r=r1min -> r1max) every ang1inc degrees. ***/

   for (th = 0.0; th < 360.0; th += ang1inc)
      {
/*      if (th == 180.0)  r1min = 0.2;  else r1min = 0.4; */
      for (radius = r1min; radius <= r1max; radius += r1inc)
         {
	 mk4_coord (&radius, &th, &x, &y, 1, 0);
         xg = (x + 0.0) + gMINROW;
         yg = (y + 0.0) + gMINCOL;
         if (xg >= gMINCOL && xg <= gMAXCOL && yg >= gMINROW && yg <= gMAXROW)
            {
            gl_wvec (xg,   yg-1, xg,   yg+1, cindex);
            gl_wvec (xg-1, yg,   xg+1, yg,   cindex);
            }
	 }
      }

   /*** Draw radial scans (r=r2min -> r2max) every ang2inc degrees. ***/

   for (th = 0.0; th < 360.0; th += ang2inc)
      for (radius = r2min; radius <= r2max; radius += r2inc)
         {
	 mk4_coord (&radius, &th, &x, &y, 1, 0);
         xg = (x + 0.0) + gMINROW;
         yg = (y + 0.0) + gMINCOL;
         if (xg >= gMINCOL && xg <= gMAXCOL && yg >= gMINROW && yg <= gMAXROW)
            {
            gl_wvec (xg,   yg-1, xg,   yg+1, cindex);
            gl_wvec (xg-1, yg,   xg+1, yg,   cindex);
            }
	 }

   /*** Draw sun circle at 1.0 Rsun.  One dot every 10 degrees. ***/

   for (th = 0; th < 360; th += 10)
      {
      mk4_coord (&r, &th, &x, &y, 1, 0);
      xg = (x + 0.0) + gMINROW;
      yg = (y + 0.0) + gMINCOL;
      if (xg >= gMINCOL && xg <= gMAXCOL && yg >= gMINROW && yg <= gMAXROW)
         {
         gl_wvec (xg,   yg-1, xg,   yg+1, cindex);
         gl_wvec (xg-1, yg,   xg+1, yg,   cindex);
         }
      }

   return (0);
}
