/* mk4lib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO Mark-III library information.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   23 February 1994
 * 24 Nov 1998: adapt for Mk4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

struct mk4head			/* MLSO Mark-III image header structure. */
   {
   short flagch	;		/* Flags & channel number	*/
   short juld	;		/* "julian" date		*/
   long  time	;		/* image time			*/
   short phase	;		/* 				*/
   short bright	;		/*				*/
   short pangle	;		/* P-angle			*/
   short nray	;		/* # radial rays		*/
   short dmax	;		/* Data maximum value		*/
   short dmin	;		/* Data minimum value		*/
   short didxx	;		/*				*/
   short didxn	;		/*				*/
   short exdate	;		/*				*/
   short scndur	;		/* Scan duration (Hertz)	*/
   long  ipixscl;		/* pixels/Rsun * 10000		*/
   long  ir0	;		/*				*/
   long  idr	;		/*				*/
   };

struct mk4info			/* MLSO Mark-III housekeeping info. */
   {
   int dtype	;		/* Data type [1-15]		*/
   int channel	;		/* Data channel [0,1]		*/
   int xdim	;		/* Image width			*/
   int ydim	;		/* Image height			*/
   int zdim	;		/* Image depth			*/
   int nbypx	;		/* # bytes/pixel		*/
   int dmin	;		/* Data minimum value		*/
   int dmax	;		/* Data maximum value		*/
   int smin	;		/* Display min value		*/
   int smax	;		/* Display max value		*/
   int year	;		/* Image year			*/
   int month	;		/* Image month			*/
   int day	;		/* Image day (of month)		*/
   int doy	;		/* Image day-of-year		*/
   int hour	;		/* Image hour			*/
   int minute	;		/* Image minute			*/
   int second	;		/* Image second			*/
   int cal_year ;		/* Calibration year.		*/
   int cal_month;		/* Calibration month.		*/
   int cal_day  ;		/* Calibration day.		*/
   int opc_year ;		/* Opal Cal 0 year.		*/
   int opc_month;		/* Opal Cal 0 month.		*/
   int opc_day  ;		/* Opal Cal 0 day.		*/
   int opc_hour ;		/* Opal Cal 0 hour.		*/
   int opc_minute;		/* Opal Cal 0 minute.		*/
   int opc_second;		/* Opal Cal 0 second.		*/
   int scndir	;		/* +1 => CCW; -1 => CW		*/
   float scndur	;		/* Scan duration (sec)		*/
   float bscale ;		/* Brightness scaling factor.	*/
   float bzero  ;		/* Brightness scaling offset.	*/
   float pixrs	;		/* # pixels/Rsun		*/
   float phase 	;		/* Channel angle w.r.t. geo north. */
   float pangle	;		/* P-angle			*/
   float nangle ;		/* Angle (CCW) of north w.r.t. +X axis. */
   float xcen	;		/* X-axis sun center		*/
   float ycen	;		/* Y-axis sun center		*/
   float rmin   ;		/* Inner FOV radius (Rsun)      */
   float rmax   ;		/* Outer FOV radius (Rsun)      */
   float tmin   ;		/* Begin barrel angle (degrees) */
   float tmax   ;		/* End   barrel angle (degrees) */
   } ;

struct mk4head MK4HS ;
struct mk4head *MK4H = &MK4HS ;

struct mk4info MK4IS ;
struct mk4info *MK4I = &MK4IS ;

