/* rdmk4.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read MLSO Mark-4 image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Mauna Loa Solar Observatory   Mark-4 K-coronameter
 * Andrew L. Stanger   3 February 1994
 * 24 Nov 1998: adapt for Mk4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/mk4lib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char *
rdmk4 (namimg, pmode)

char  *namimg ;		/* Pointer to image disk file name. */
int pmode ;		/* Print mode: 0=silent, 1=verbose. */

{
register unsigned char *imgbuf;	/* Pointer to image array. */
register int	imgfd;		/* File descriptor for image disk file. */
register int	imgsiz;		/* Image size (bytes) (excludes hskp/tailer). */
register long	nread;
int nrechd;			/* # records for header. */
FILE *imgFD ;			/* File structure for image disk file. */

   /*** Decommutate Housekeeping. ***/

   mk4hskp (namimg, pmode);	

   /*** Open image file. ***/

/*   if ((imgfd = open (namimg, 0)) < 0) */
   if ((imgFD = fopen (namimg, "r")) == NULL)
      {
      fprintf (stderr, "rdmk4> Can't open file %s\n", namimg);
      return (NULL);
      }

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

/*   if (lseek (imgfd, (long) 0, 0) == -1) *?
   if (fseek (imgFD, (long) 0, 0) == -1)
      {
      fprintf (stderr, "rdmk4> seek error.\n");
      return (NULL);
      }

/*   fprintf (stdout, "rdmk4> xdim: %d   ydim: %d   nbypx: %d\n",
 *            MK4I->xdim, MK4I->ydim, MK4I->nbypx);
 */

   imgsiz = (MK4I->xdim * MK4I->ydim * MK4I->nbypx);
   if ((imgbuf = (unsigned char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr, "rdmk4> malloc error.");
      exit (1);
     }

/*   if ((nread = read (imgfd, imgbuf, imgsiz)) <= 0) */
   if ((nread = fread (imgbuf, sizeof (char), imgsiz, imgFD)) <= 0)
      {
      fprintf (stderr, "rdmk4> image read error.\n");
      return (NULL);
      }

   if (nread != imgsiz)
      {
      fprintf (stderr, "rdmk4> nread:%d < imgsiz:%d\n", nread, imgsiz);
      return (NULL);
      }

/*   if (close (imgfd) == -1) */
   if (fclose (imgFD) != 0)
      {
      fprintf (stderr, "rdmk4> file close error.%s\n", namimg);
      return (NULL);
      }
   
  return (imgbuf);
}
