/* appcal4.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO/Mark-IV K-coronameter
 * Perform calibration.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define START

#include "mk4.h"
#include "mk4head.h"
#include "crunch.h"
#include "telscp4.h"

int bz, by ; 

char *filenm, picnm [64], *piceos ; 
char tscpfile [64] ; 

short Iim [CY_IMG], Qim [CY_IMG], Uim [CY_IMG] ; 
short  ba [VECTOR] ; 
float Fim [CY_IMG], FUim [CY_IMG], fba [VECTOR] ; 
float phase [NDS], 
      biasi [NDS], biasq  [NDS], biasu [NDS],
      ang0  [NDS],   fp0  [NDS], 
      XM    [NDS*9], XinvM [NDS*9] ; 


main (an, av)

int an ; 
char *av [] ;  
{
    register char *w, *ww ; 
    FH  hdi, hdq, hdu, hd ; 

    long begtime, endtime ; 
    int ii ; 
    int gotcalib = 0 ; 
    int dotest = 0 ; 
    int rerot = 0 ; 
    int polrm = 0, haze = 0, depolU = 0 ; 
    short *pqq, *qq ; 
    unsigned short appdate ; 
    float *pfba ; 
    float *Gain, *ovGain ; 
    float sky ; 
    double angoff ; 
    double pphase ; 
    struct tm *T, *localtime () ; 
    char *ctime_r (), buf [30] ; 

    (void) time (&begtime) ; 
    T  =  (struct tm *) malloc (sizeof (struct tm)) ; 
    T  =  localtime (&begtime) ; 
    T->tm_mon +=  1 ; 

    (void) ctime_r (&begtime, &buf, 30) ; 
    fprintf (stdout, "Started appcal4:      %s\n", buf) ; 

    by = 900 ;   /* bangle to print out, usually 200 */
    bz = 900 ;   /* height to print out, usually 50 */

    hd  = (FH) malloc (sizeof(mk4head)) ; 
    hdi = (FH) malloc (sizeof(mk4head)) ; 
    hdq = (FH) malloc (sizeof(mk4head)) ; 
    hdu = (FH) malloc (sizeof(mk4head)) ; 

    timeflag = 1 ; 

    if ( an <= 2 ) { prsum () ;  exit (3) ;  }

    while (--an)
	{		/*** Read in the files and perform the calibration ***/
	w = *++av ; 
	if (*w == '?') { prsum() ;  exit(2) ;  }
	if (*w == '-')
	    {			/*** Check for flag arguments. ***/
	    switch (w [1])
		{
                default:
                        fprintf (stderr, " unknown flag %s\n", *av) ; 
                        prsum () ;  exit (1) ; 
                }
	    continue ; 
	    }
	filenm = w ; 

	if ( *(w + strlen (filenm) - 1) == 'p' )
	    {
	    gotcalib = 1 ;  sprintf (tscpfile, "%s", filenm) ; 
	    readtscp (filenm, thd, 1) ; 
	    Fillparam (thd, phase, biasi, biasq, biasu, ang0, fp0, XM) ; 
	    Gain = thd->Gain ; 
            ovGain = Gain ; 
	    SetupM ( Gain,  XM, 
	             ovGain, XinvM) ; 

	    fprintf (stdout, "\nCalibration was created: %s\n", 
		     prday (thd->tdysdate)) ; 
	    continue ; 
	    }

	if (!gotcalib)
	    {
	    fprintf (stderr, 
	    "Sorry, the calibration \".tscp\" file must come first.\n") ; 
	    fprintf (stderr, "Exitting\n") ; 
	    exit (-10) ; 
	    }


	/*** Check on the file naming convention used on the file ***/

	if ((*w=='0') || (*w >= '8' && *w <= '9') || *(w+2)=='d' )
	    {
	    if ( *(w+3) >='0' && *(w+3) <='9' )  doyflag++ ; 
	    else                                 nameflag++ ; 
	    }
	if (nameflag || doyflag) { dayflag++ ;  timeflag=1 ;  }

	(void) time (&begtime) ; 

	readmk4head (filenm, mk4h, 1) ; 
	sky = (SKYBRI - SKYOFF)  / (mk4h->SkyTx - SKYOFF) ; 


        /*** Get the i q u data. ***/

        dopicnm (mk4h) ; 
        dofnm (IS_i,  mk4h->ProcSteps) ;	readmk4 (picnm, hdi, Iim, 0) ; 
        dofnm (IS_q,  mk4h->ProcSteps) ;	readmk4 (picnm, hdq, Qim, 0) ; 
        dofnm (IS_u,  mk4h->ProcSteps) ;	readmk4 (picnm, hdu, Uim, 0) ; 
        dofnm (IS_ba, IS_raw) ;			readmk4 (picnm, hd,  ba,  0) ; 

	fprintf (stdout, "\n\tComputing splines\n") ; 

        /*** Spline it, save it as an theta by R array. ***/

        if (dotest == 0)
	    {
            cyspline(0, mk4h, Iim, ba) ; 
            cyspline(0, mk4h, Qim, ba) ; 
            cyspline(0, mk4h, Uim, ba) ; 
            }


	/*** Calibrate ! ***/

	Recrunch (   biasi, biasq, biasu, 
		     ovGain, XinvM, 
		     Iim, Qim, Uim) ; 

        if ( polrm )
	    {
            fprintf (stdout, "\tRemoving residual polarization (depol).\n") ; 

            /*** Get the bangles to spline values. ***/

            pfba = fba ; 
	    for (ii = 0 ;  ii < NAZ ;  ii++) *pfba++ =  (float) ii * AngSep * d2r ; 

            /* Find the pol. phase of the U/I data, fit it to a cos wave, 
             * shift the pol phase of the fitted cos wave by 45deg
	     */

            getphsNfit (Iim, Uim, Fim, FUim, fba, NAZ, &pphase, haze) ; 

            /*** Multiply the Fit by I and subtract from the pB data. ***/

            do_subt (Qim, Iim, Fim, NAZ) ; 
            if ( depolU ) do_subt (Uim, Iim, FUim, NAZ) ; 
            }

        fprintf (stdout, 
        "\n\tChecking for residual coronal in U (inaccurate TILT phase).\n") ; 
        ckTphase (Qim, Uim) ; 


	/*** Adjust for sky brightness ***/

	pqq = Iim ; 
	for (ii = 0 ;  ii < CY_IMG ;  ii++)
	    { *pqq = (short) (sky * (float) (*pqq)) ;   pqq++ ;  }
	pqq = Qim ; 
	for (ii = 0 ;  ii < CY_IMG ;  ii++)
	    { *pqq = (short) (sky * (float) (*pqq)) ;   pqq++ ;  }
	pqq = Uim ; 
	for (ii = 0 ;  ii < CY_IMG ;  ii++)
	    { *pqq = (short) (sky * (float) (*pqq)) ;   pqq++ ;  }


	/*** Get to heliocentric coordinates. ***/

	angoff = (double) (hdi->Pangle + hdi->Phase + hdi->backlash) ; 
        shiftimg (Iim, angoff) ; 
        shiftimg (Qim, angoff) ; 
        shiftimg (Uim, angoff) ; 

	/*** Rotate back to an R, theta array. ***/

	rotbackcy (NAZ, Iim) ; 
	rotbackcy (NAZ, Qim) ; 
	rotbackcy (NAZ, Uim) ; 


	/*** Fix and fill in the headers with new info. ***/

	hdi->ProcSteps = hdq->ProcSteps = hdu->ProcSteps = IS_cal ; 
	hdi->theta_min = hdq->theta_min = hdu->theta_min =     0.0 ; 
	hdi->theta_max = hdq->theta_max = hdu->theta_max =   360.0 ; 
	hdi->Xdim      = hdq->Xdim      = hdu->Xdim      =    NDS ; 
	hdi->Ydim      = hdq->Ydim      = hdu->Ydim      =    NAZ ; 

        sprintf (hdi->tscpfile, "%s", tscpfile) ; 
        sprintf (hdq->tscpfile, "%s", tscpfile) ; 
        sprintf (hdu->tscpfile, "%s", tscpfile) ; 

	hdi->Date_calibrated.Year   = 
	hdq->Date_calibrated.Year   = 
	hdu->Date_calibrated.Year   = T->tm_year+1900 ; 
	hdi->Date_calibrated.Month  = 
	hdq->Date_calibrated.Month  = 
	hdu->Date_calibrated.Month  = T->tm_mon ; 
	hdi->Date_calibrated.Day    = 
	hdq->Date_calibrated.Day    = 
	hdu->Date_calibrated.Day    = T->tm_mday ; 
	hdi->Date_calibrated.Doy    = 
	hdq->Date_calibrated.Doy    = 
	hdu->Date_calibrated.Doy    = T->tm_yday ; 
	hdi->Date_calibrated.Hour   = 
	hdq->Date_calibrated.Hour   = 
	hdu->Date_calibrated.Hour   = T->tm_hour ; 
	hdi->Date_calibrated.Minute = 
	hdq->Date_calibrated.Minute = 
	hdu->Date_calibrated.Minute = T->tm_min ; 
	hdi->Date_calibrated.CentiSec = 
	hdq->Date_calibrated.CentiSec = 
	hdu->Date_calibrated.CentiSec = T->tm_sec * 100 ; 

	hdi->Date_calibrated.Juld = 
	hdq->Date_calibrated.Juld = 
	hdu->Date_calibrated.Juld = jday(T->tm_mon, T->tm_mday, T->tm_year+1900) 
				    - 2415021 ; 

	hdi->Date_calibrated.Time = 
	hdq->Date_calibrated.Time = 
	hdu->Date_calibrated.Time =  T->tm_hour * 60*60*60
				    +T->tm_min  * 60*60
				    +T->tm_sec  * 60      ; 


	dopicnm (hdi) ; 
	dofnm (hdi->DataType, hdi->ProcSteps) ; 
	writemk4 (picnm, hdi, Iim, 1) ; 

	dopicnm (hdq) ; 
	dofnm (hdq->DataType, hdq->ProcSteps) ; 
	writemk4 (picnm, hdq, Qim, 1) ; 

	dopicnm (hdu) ; 
	dofnm (hdu->DataType, hdu->ProcSteps) ; 
	writemk4 (picnm, hdu, Uim, 1) ; 

	(void) time (&endtime) ; 
	fprintf (stdout, "\nTime used:    %ld seconds\n", endtime-begtime) ; 

        }

    exit (0) ; 


}


prsum ()

{
    static char help[] = "\
appcal [calfile] [cylindrical file1] [cylindrical file2] ... [cylindrical fileN]\n\
\n\
Example: appcal 92d004.20_04.ch0.tscp 92d004.17_28.ch0.q  92d004.17_32.ch0.q\n\
\n\
\n\
Note:  This progam requires the \".i\" \".q\" \".u\" and \".ba\" files \n\
       for a given image time.\n\
       The program automatically builds the filenames of \n\
       the \".i\" \".q\" \".u\" and \".ba\" files \n\
       files using the .q data filenames as a template.\n\
\n\
Flags          Description\n\
-----          -----------\n\
\n\
" ; 

       fputs (help, stderr) ; 
}



/* This computes the matricies that are constant for any given calibration... */

extern    int Invmatrix() ; 
extern int bz, by ; 

int
SetupM (    Gain,    XM,   ovGain,  XinvM)

     float *Gain,   *XM,  *ovGain, *XinvM ; 
{
    int ii, dd, anerror ; 
    float tmp,  *pGain ; 
    float *pXM, *pXinvM ; 
    float *pf ; 

    /*** Compute the variables and matricies that vary with height... ***/

    pGain = Gain ;    
    pXM   = XM  ;    pXinvM= XinvM  ; 

    for (dd = 0 ;  dd < NDS ;  dd++)
	{
       
	for (ii = 0 ;  ii < NAZ ;  ii++)
	    {					/* Gain is an image. */
	    tmp = ( *pGain > 0.0) ? 1.0/(*pGain) : 0.0 ; 
	    if (dd == bz)
		fprintf (stderr, "Gain %3f ovGain %3f\n", *pGain, tmp) ; 
	    *pGain = tmp ; 
	    pGain++ ; 
	    }

	/*** X inverse matrix... ***/

	if ((anerror = Invmatrix (pXM, pXinvM)) != 0)
	    {
	    if (dd > BOTGDI)
		fprintf (stderr, 
		"At diode %d, error in the inversion of the X matrix!!\n", dd) ; 
	    pf = pXinvM ; 
	    for (ii = 0 ;  ii < 9 ;  ii++) *pf++=0.0 ; 
	    }
	pXM += 9 ;  pXinvM += 9 ; 
        }
}


int
Recrunch ( biasi, biasq, biasu, 
	   ovGain, XinvM, 
	   Iim, Qim, Uim)

float  *biasi, *biasu, *biasq ; 
float *ovGain, *XinvM ; 
short *Iim, *Qim, *Uim ; 
{
    int ii, dd , mee ; 
    float *pV ,  iquVec [3] ; 
    short *pIim, *pQim , *pUim ; 
    float *povGain ; 
    float *pXinvM ; 
    float calI, calQ, calU ; 
    float *bI, *bQ, *bU ; 

    mee = 90 ; 

    pIim = Iim ;       bI=biasi ; 
    pQim = Qim ;       bQ=biasq ; 
    pUim = Uim ;       bU=biasu ; 

    povGain = ovGain ; 
    pXinvM = XinvM ; 

    /*** Recall the data is still rotated... ***/

    for (dd = 0 ;  dd < NDS ;  dd++)
	{  

	for (ii = 0 ;  ii < NAZ ;  ii++)
	    {

	    pV = iquVec ; 
	    *pV++ = (float) *pIim ; 
	    *pV++ = (float) *pQim ; 
	    *pV   = (float) *pUim ; 

	    pV = iquVec ; 
	    *pV++ -= *bI ; 
	    *pV++ -= *bQ ; 
	    *pV   -= *bU ; 

	    pV = iquVec ; 
	    *pV++ *= (*povGain) ; 
	    *pV++ *= (*povGain) ; 
	    *pV   *= (*povGain) ; 
	    if (dd == bz && ii == by)
		printf ("ovg  %.3f %8.5f %8.5f\n", 
			 iquVec [0], iquVec [1], iquVec [2]) ; 

	    Mvmult (iquVec, pXinvM) ; 

	    if (dd == bz && ii == by)
		printf ("Xin  %.3f %8.5f %8.5f\n", 
			iquVec [0], iquVec [1], iquVec [2]) ; 

	    pV = iquVec ; 
	    calI= *pV++ ; 
	    calQ= *pV++ ; 
	    calU= *pV ; 
	    *pIim = (unsigned short) (calI  * 1.e+2) ; 
	    *pQim = (short) (         calQ  * 1.e+4) ; 
	    *pUim = (short) (         calU  * 1.e+4) ; 

	    if (dd == bz && ii == by)
		printf ("%d %d %d\n", *pIim, *pQim, *pUim) ; 


	    pIim++ ;   pQim++ ;   pUim++ ;   povGain++ ; 

	    }
	pXinvM += 9 ; 
	bI++ ;   bQ++ ;   bU++ ; 
        }
}



int
ckTphase (  Qim,  Uim)

short      *Qim, *Uim ; 

{
    short *qq , *uu ; 
    int ii, mydi , beg, end ; 
    double avgqq, avguu, diffi ; 
    double asin (), fabs (), rat, Tphserr ; 

    avgqq = avguu = 0.0 ; 

    mydi = 100 ;  beg = 60*3 ;  end = 120*3 ;      diffi = (end-beg) +1. ; 

    uu = Uim + NAZ * mydi + beg ; 
    qq = Qim + NAZ * mydi + beg ; 
    for (ii = beg ;  ii < end ;  ii++)
	{
	avgqq += *qq++ ; 
	avguu += *uu++ ; 
        }
    rat = avguu / avgqq ; 
    Tphserr = asin (rat) ; 
    Tphserr *= 0.5 * r2d ; 
    fprintf (stdout, "\t\tTilt phase error is %5.1f deg", Tphserr) ; 
    if (fabs (Tphserr) > 5.0) 
        fprintf (stdout, 
	"\t****TOO BIG!\n\t\t\t\t\t\t****Please let Alice know.****\n") ; 
    else
        fprintf (stdout, "\tThis is reasonable.\n") ; 


}


int
fill_constantI ( Iim , Ival )
 
unsigned short *Iim, Ival ; 

{
    unsigned short *pI ; 
    int ii ; 

    pI = Iim ; 
    for (ii = 0 ;  ii < NAZ*NDS ;  ii++)
        { *pI++ = Ival ;  }
}
