/* mk4fits.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO Mark-IV FITS information.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 August 1999
 * 10 September 1999: add tscpfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

struct mk4_fits			/* MLSO Mark-IV FITS info. */
   {
   int dtype	;		/* Data type [0-6]			*/
   int otype	;		/* Observation type [0,150,9999,1000]	*/
   int ptype    ;		/* Processing Level [10-19,24-26]	*/
   int xdim	;		/* Image width			*/
   int ydim	;		/* Image height			*/
   int zdim	;		/* Image depth			*/
   int nbypx	;		/* # bytes/pixel		*/
   int dmin	;		/* Data minimum value		*/
   int dmax	;		/* Data maximum value		*/
   int smin	;		/* Display min value		*/
   int smax	;		/* Display max value		*/
   int year	;		/* Image year			*/
   int month	;		/* Image month			*/
   int day	;		/* Image day (of month)		*/
   int doy	;		/* Image day-of-year		*/
   int hour	;		/* Image hour			*/
   int minute	;		/* Image minute			*/
   int second	;		/* Image second			*/
   int cal_year ;		/* Calibration year.		*/
   int cal_month;		/* Calibration month.		*/
   int cal_day  ;		/* Calibration day.		*/
   char tscpfile [64] ;		/* tscp file name.		*/
   int scndir	;		/* +1 => CCW; -1 => CW		*/
   float scndur	;		/* Scan duration (sec)		*/
   float bscale ;		/* Brightness scaling factor.	*/
   float bzero  ;		/* Brightness scaling offset.	*/
   float pixrs	;		/* # pixels/Rsun		*/
   float phase 	;		/* Channel angle w.r.t. geo north. */
   float pangle	;		/* P-angle			*/
   float bangle	;		/* B-angle			*/
   float rsun  	;		/* Solar Radius (arcsec)	*/
   float nangle ;		/* Angle (CCW) of north w.r.t. +X axis. */
   float xcen	;		/* X-axis sun center		*/
   float ycen	;		/* Y-axis sun center		*/
   float rmin   ;		/* Inner FOV radius (Rsun)      */
   float rmax   ;		/* Outer FOV radius (Rsun)      */
   float tmin   ;		/* Begin barrel angle (degrees) */
   float tmax   ;		/* End   barrel angle (degrees) */
   } ;

#ifdef START
   struct mk4_fits MK4_FITS ;
   struct mk4_fits *MK4F = &MK4_FITS ;
#else
   extern struct mk4_fits MK4_FITS ;
   extern struct mk4_fits *MK4F ;
#endif



