/* mk4hdu.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill MLSO/Mk4 FITS structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 August 1999
 * 10 September 1999: Add tscpfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/cordyn/fits/lib/mk4/mk4.h"
#include "/home/cordyn/fits/lib/mk4/mk4head.h"
#include "/home/cordyn/fits/lib/mk4/mk4fits.h"

mk4hdu (hd)

struct mk4_header *hd ;
{
   int pmode = 1 ;
   float rmin, rmax ;

   /*** Fill Mk4 FITS structure with appropriate information. ***/

   MK4F->dtype  = hd->DataType ;/* IS_i, IS_q, IS_u, IS_ba, IS_sk, IS_st. */
   MK4F->otype  = hd->ObsType ;	/* IS_cal0, IS_cal15, IS_dark, IS_corona. */
   MK4F->ptype  = hd->ProcSteps ;	/* IS_raw..., IS_cal..., IS_UNKNOWN */
   MK4F->xdim   = hd->Xdim ;	/* Horizontal dimension. */
   MK4F->ydim   = hd->Ydim ;	/* Vertical   dimension. */

   if (MK4F->ptype == IS_cal_rect_byte)
      MK4F->nbypx = 1 ;		/* # bytes per pixel (byte). */
   else
      MK4F->nbypx = 2 ;		/* # bytes per pixel (short integer). */
   MK4F->dmin   = 0 ;		/* Data    minimum value. */
   MK4F->dmax   = 0 ;		/* Data    maximum value. */
   MK4F->smin   = 0 ;		/* Display minimum value. */
   MK4F->smax   = 0 ;		/* Display maximum value. */
   MK4F->year   = hd->Image.Year ;	/* Image year.*/
   MK4F->month  = hd->Image.Month ;	/* Image month. */
   MK4F->day    = hd->Image.Day ;	/* Image day-of-month. */
   MK4F->doy    = hd->Image.Doy ;	/* Image day-of-year.*/
   MK4F->hour   = hd->Image.Hour ;	/* Image hour. */
   MK4F->minute = hd->Image.Minute ;	/* Image minute. */
   MK4F->second = hd->Image.CentiSec / 100 ;	  /* Image second. */
   MK4F->cal_year   = hd->Date_calibrated.Year ;  /* Calibration year.*/
   MK4F->cal_month  = hd->Date_calibrated.Month ; /* Calibration month. */
   MK4F->cal_day    = hd->Date_calibrated.Day ;	  /* Calibration day-of-month.*/
   strncpy (MK4F->tscpfile, hd->tscpfile, 64) ;   /* Telescope info file name.*/

   /*** Set brightness scaling factor. ***/

   switch (MK4F->dtype)
      {
      case IS_i  :
      case IS_q  :
      case IS_u  :
         switch (MK4F->ptype)
            {
	    case IS_raw          :
	    case IS_raw_spln     :
	    case IS_raw_rect     :
	    case IS_raw_avg      :
	    case IS_raw_avg_spln :
	    case IS_raw_avg_rect :
	       			MK4F->bscale = 1.0 ;     break ; /* CCD units */
	    case IS_cal :
	    case IS_cal_rect :
	    case IS_cal_rect_slit :
	    case IS_cal_rect_byte :
	    case IS_cal_avg :
	    case IS_cal_avg_rect :
	    case IS_cal_avg_rect_slit:
				MK4F->bscale = 1.0e-10 ; break ; /* Bsun units*/
            default : MK4F->bscale = 1.0 ;
            }	break ;
      case IS_ba : MK4F->bscale = 1.0 ;	break ;		/* degrees */
      case IS_sk : MK4F->bscale = 1.0 ;	break ;		/* sky brightness */
      case IS_st : MK4F->bscale = 1.0 ;	break ;		/* scan times */
      default    : MK4F->bscale = 1.0 ;
      }
   if (pmode)
   fprintf (stdout, "mk4hdu> ptype: %d  bscale: %e\n",
	    MK4F->ptype, MK4F->bscale) ;

   MK4F->bzero  = 0.0 ;		/* Brightness scaling offset. */
   MK4F->scndir = hd->Direction ;	/* +1 => CCW; -1 => CW. */

   if (pmode)
   fprintf (stdout, "mk4hdu> stop1\n") ;
   fflush  (stdout) ;

   /*** Scan duration (seconds). ***/

   MK4F->scndur = (float) (hd->Duration & 0177777) / 60.0 ;

   if (pmode)
   fprintf (stdout, "mk4hdu> stop2\n") ;
   fflush  (stdout) ;

   switch (MK4F->dtype)
      {
      case IS_i  :
      case IS_q  :
      case IS_u  :
         switch (MK4F->ptype)
	    {
	    case IS_raw          :	/* polar (or cylindrical) coordinates */
	    case IS_raw_avg      :
	    case IS_raw_avg_spln :
	    case IS_cal          :
	    case IS_cal_avg      :
		MK4F->pixrs = hd->Radius / (hd->dr * 960.0) ;
		MK4F->xcen  = -(hd->r0 * MK4F->pixrs) ;/* Sun center (pixels) */
		MK4F->ycen  = 0.0 ;	/* Not applicable. */
		break ;
            default : MK4F->pixrs = hd->pixscl ;	/* # pixels / Rsun */
		      MK4F->xcen  = hd->Xcen ;		/* X-axis sun center. */
		      MK4F->ycen  = hd->Ycen ;		/* Y-axis sun center. */
            } break ;
      }

   if (pmode)
   fprintf (stdout, "mk4hdu> stop3\n") ;
   fflush  (stdout) ;

printf ("r0: %f  pixrs: %f  xcen: %f\n", hd->r0, MK4F->pixrs, MK4F->xcen) ;

   if (pmode)
   fprintf (stdout, "mk4hdu> stop4\n") ;
   fflush  (stdout) ;

   MK4F->phase  = hd->Phase ;	/* Mk4 offset angle w.r.t. spar. */
   MK4F->pangle = hd->Pangle ;	/* Solar P-angle [degrees]. */
   MK4F->bangle = hd->Bangle ;	/* Solar B-angle [degrees]. */
   MK4F->rsun   = hd->Radius ;	/* Solar Radius  [arcsec].  */
   MK4F->nangle = 0.0 ;		/* Angle (CCW) of north w.r.t. +Y axis. */

   if (pmode)
   fprintf (stdout, "mk4hdu> stop5\n") ;
   fflush  (stdout) ;

   /*** Inner/Outer FOV radii. ***/
   /*** rmin = inner radius.   rmax = outer radius. ***/
   /*** r0 & dr values are based upon 960 arcsec/Rsun. ***/

   rmin = hd->r0 ;			/* Rsun units based on 960 arcsec/Rsun*/
   rmin *= 960.0 ;				/* Convert to arcsec. */
   MK4F->rmin = rmin / MK4F->rsun ;		/* Rsun units. */
   rmax = hd->dr * 383.0 + MK4F->rmin ;		/* Rs units*/
   rmax *= 960.0 ;				/* Convert to arcsec. */
   MK4F->rmax = rmax / MK4F->rsun ;		/* Rsun units. */

   MK4F->tmin   = hd->raw_theta_min ;	/* Begin barrel angle [degrees]. */
   MK4F->tmax   = hd->raw_theta_max ;	/* End   barrel angle [degrees]. */

   fprintf (stdout, "mk4hdu> stop6\n") ;
   fflush  (stdout) ;

   /*** Mk4 binary header contents. ***/

   if (pmode)
   {
   fprintf (stdout, "hd->Image.Year:            %d\n", hd->Image.Year) ;
   fflush (stdout) ;
   fprintf (stdout, "hd->Image.Month:           %d\n", hd->Image.Month) ;
   fprintf (stdout, "hd->Image.Day:             %d\n", hd->Image.Day) ;
   fprintf (stdout, "hd->Image.Hour:            %d\n", hd->Image.Hour) ;
   fprintf (stdout, "hd->Image.Minute:          %d\n", hd->Image.Minute) ;
   fprintf (stdout, "hd->Image.CentiSec:        %d\n", hd->Image.CentiSec) ;
   fprintf (stdout, "hd->Image.Juld:            %d\n", hd->Image.Juld) ;
   fprintf (stdout, "hd->Image.Doy:             %d\n", hd->Image.Doy) ;
   fprintf (stdout, "hd->Image.Time:            %d\n", hd->Image.Time) ;
   fprintf (stdout, "hd->Xdim:                  %d\n", hd->Xdim) ;
   fprintf (stdout, "hd->Ydim:                  %d\n", hd->Ydim) ;
   fprintf (stdout, "hd->Duration:              %d\n", hd->Duration) ;
   fprintf (stdout, "hd->Fill1 [0]:             %d\n", hd->Fill1 [0]) ;
   fprintf (stdout, "hd->Fill1 [1]:             %d\n", hd->Fill1 [1]) ;
   fprintf (stdout, "hd->Fill1 [2]:             %d\n", hd->Fill1 [2]) ;
   fprintf (stdout, "hd->Fill1 [3]:             %d\n", hd->Fill1 [3]) ;
   fprintf (stdout, "hd->Pangle:                %f\n", hd->Pangle) ;
   fprintf (stdout, "hd->Bangle:                %f\n", hd->Bangle) ;
   fprintf (stdout, "hd->Langle:                %f\n", hd->Langle) ;
   fprintf (stdout, "hd->RA:                    %f\n", hd->RA) ;
   fprintf (stdout, "hd->Dec:                   %f\n", hd->Dec) ;
   fprintf (stdout, "hd->HA:                    %f\n", hd->HA) ;
   fprintf (stdout, "hd->Azimuth:               %f\n", hd->Azimuth) ;
   fprintf (stdout, "hd->Amass:                 %f\n", hd->Amass) ;
   fprintf (stdout, "hd->SkyTx:                 %f\n", hd->SkyTx) ;
   fprintf (stdout, "hd->InstrPhs:              %f\n", hd->InstrPhs) ;
   fprintf (stdout, "hd->Xcen:                  %f\n", hd->Xcen) ;
   fprintf (stdout, "hd->Ycen:                  %f\n", hd->Ycen) ;
   fprintf (stdout, "hd->pixscl:                %f\n", hd->pixscl) ;
   fprintf (stdout, "hd->r0:                    %f\n", hd->r0) ;
   fprintf (stdout, "hd->dr:                    %f\n", hd->dr) ;
   fprintf (stdout, "hd->rmax:                  %f\n", hd->rmax) ;
   fprintf (stdout, "hd->theta_min:             %f\n", hd->theta_min) ;
   fprintf (stdout, "hd->theta_max:             %f\n", hd->theta_max) ;
   fprintf (stdout, "hd->raw_theta_min:         %f\n", hd->raw_theta_min) ;
   fprintf (stdout, "hd->raw_theta_max:         %f\n", hd->raw_theta_max) ;
   fprintf (stdout, "hd->backlash:              %f\n", hd->backlash) ;
   fprintf (stdout, "hd->Phase:                 %f\n", hd->Phase) ;
   fprintf (stdout, "hd->ObsType:               %d\n", hd->ObsType) ;
   fprintf (stdout, "hd->DataType:              %d\n", hd->DataType) ;
   fprintf (stdout, "hd->ProcSteps:             %d\n", hd->ProcSteps) ;
   fprintf (stdout, "hd->Direction:             %d\n", hd->Direction) ;
   fprintf (stdout, "hd->Diodes:                %d\n", hd->Diodes) ;
   fprintf (stdout, "hd->Fill2 [0]:             %d\n", hd->Fill2 [0]) ;
   fprintf (stdout, "hd->Date_calibrated.Year:  %d\n", hd->Date_calibrated.Year) ;
   fprintf (stdout, "hd->Date_calibrated.Month: %d\n", hd->Date_calibrated.Month) ;
   fprintf (stdout, "hd->Date_calibrated.Day:   %d\n", hd->Date_calibrated.Day) ;
   fprintf (stdout, "hd->Date_calibrated.Hour:  %d\n", hd->Date_calibrated.Hour) ;
   fprintf (stdout, "hd->Date_calibrated.Minute:%d\n", hd->Date_calibrated.Minute) ;
   fprintf (stdout, "hd->Date_calibrated.CentiSec:%d\n", hd->Date_calibrated.CentiSec) ;
   fprintf (stdout, "hd->Date_calibrated.Juld:  %d\n", hd->Date_calibrated.Juld) ;
   fprintf (stdout, "hd->Date_calibrated.Doy:   %d\n", hd->Date_calibrated.Doy) ;
   fprintf (stdout, "hd->Date_calibrated.Time:  %d\n", hd->Date_calibrated.Time) ;
   fprintf (stdout, "hd->tscpfile:              %s\n", hd->tscpfile) ;
   fprintf (stdout, "hd->AvgTimes:              %s\n", hd->AvgTimes) ;
   fflush (stdout) ;
   }

   return (0) ;
}
