/* cpmassb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * ****** Absolute/Excess mass calculation: polygon version *******
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * NCAR/HAO   SMM C/P
 * 26 September 1984
 * 21 December 1984: Add coronal brightness threshold.
 * 31 October 1985: Apply Munro calibration correction.
 *  1 November 1985: Use cphskp & hkline.
 * 12 July 1990: Remove stray light, vignetting, & F-corona (if required).
 * 20 July 1990: For excess mass, no F-corona subtraction is needed.
 * 15 November 1991: input is an array of pixel coordinates.
 *                   elminate cbthresh.
 *  8 August 1994: add (Bcme - Bref) / Bcme computation.
 *- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/ov_color.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

cpmassb (namref, namcme, mx, cbth, pgmout, winpos, np)

char	namref [] ;	/* REF image file name. */
char	namcme [] ;	/* CME image file name. */
int	mx ;		/* Mass: =1: CME-REF ; otherwise, Absolute: CME. */
float	cbth ;		/* Coronal brightness threshold (B/Bsun).*/
FILE *pgmout ;		/* Output destination. */
short winpos [][2] ;	/* Window x/y coordinate positions of pixels in AOI. */
int np ;		/* # pixels in AOI. */

{
#define	MAXPIX	896
#define RSUN 6.96e+10	/* Solar radius (cm). */
extern	char *malloc() ;
extern	char *rdimg () ;
extern  int    cphskp(), hkline() ;/* Housekeeping decommutation & display. */
extern	int    min (), max () ;	/* Integer min & max functions. */
extern	float  xmin (), xmax () ;/* Specialized float min & max functions. */
extern	float  stray () ;	/* Declare stray light function. */
extern	float  vignet () ;	/* Declare vignetting function. */
extern	float  fcorona () ;	/* F-corona function. */
extern	float  calcor () ;	/* Munro calibration correction function. */
extern	double medsun () ;	/* Mass density (grams/cm2) per 1.0 Bsun. */

static char	answer [2] ;	/* Storage for y/n response. */
char	*rbuf, *cbuf ;

short *irbuf ;		/* Image array for REF image. */
short *icbuf ;		/* Image array for CME image. */

int	cmefd, reffd ;	/* File descriptors for CME & REF images. */
int	ctype ;		/* Data type for CME image. */
int	i ;		/* General loop index. */
int	icp, irp ;	/* Array indices for icbuf and irbuf. */
int	iix, iiy ;	/* Index variables for pixel and line. */
int	iixb ;		/* X-axis index for REF image. */
int	iiyb ;		/* Y-axis index for REF image. */
int	ipix ;
int	ipc,irc,lpc,lrc ;/* Limits for CME image. */
int	ipb,irb,lpb,lrb ;/* Limits for REF image. */
int	irad ;		/* Index for elec array. */
int	ixs, iys ;	/* Sun center shift of REF image relative to CME. */
int	ixcm, iycm ;	/* Center of mass coordinates. */
int	ixcmg, iycmg ;	/* Center of mass coordinates for display. */
int	ixg, iyg ;	/* Display coordinates of integrated pixel. */
int	ix11,ix12,ix21,ix22 ;	/* Pixel limits for wedge. */
int	iy11,iy12,iy21,iy22 ;	/* Line limits for wedge. */
int	mv ;		/* Vignetting flag: =1 vignetting has been removed. */
int	n ;		/* temporary debug variable. */
int	nbyrec ;	/* Number of bytes per record. */
int	num, numun ;	/* Number of values in kmass & unmass. */
int	numneg ;	/* Number of negative values in integration. */
int	numth ;		/* Number of values above 'cpth' brightness threshold.*/
int	rtype ;		/* Data type for REF image. */
int	xocp = 63 ;	/* Window x origin. */
int	yocp = 510 ;	/* Window y origin. */

float	*xbbuf ;	/* Data buffer: excess brightness. */
float	*xmbuf ;	/* Data buffer: excess mass. */
float	bcme ;		/* Brightness of CME image pixel. */
float	bref ;		/* Brightness of REF image pixel. */
float	cbrite ;	/* Coronal mass. */
float	cbsum ;		/* Integrated coronal brightness. */
float	cbthresh ;	/* Threshold level (B/Bsun). */
float	cc ;		/* Munro calibration correction factor. */
float	cm2pix ;	/* Area (cm**2) per pixel. */
float	cmebrt ;	/* Integrated coronal mass. */
float	cmebrt0 ;	/* Integrated coronal mass above a threshold level. */
float	cmemass ;	/* CME image mass. */
float	crrbrt ;	/* Integrated brightness: (CME - REF) / REF. */
float	cxcen, cycen ;	/* Sun center for CME image. */
float	fx, fy ;	/* Floating variables for iix & iiy. */
float	kbrite ;	/* Integrated absolute or excess brightness value. */
float	kmass ;		/* Integrated absolute or excess mass value. */
float	masspct ;	/* % mass change. */
float	negmass ;	/* Negative mass. */
float	negmassp ;	/* Negative mass percentage. */
float	negpct ;		/* % Mass which is negative. */
float	p11,p12,p21,p22 ;	/* Pixel limits for wedge. */
float	r11,r12,r21,r22 ;	/* Line limits for wedge. */
float	r, theta ;	/* Radius & theta. */
float	rcm, thcm ;	/* (r,th) coordinates of center-of-mass. */ 
float	refbrt ;	/* Integrated coronal mass. */
float	refbrt0 ;	/* Integrated coronal mass above a threshold level. */
float	refmass ;	/* REF image mass. */
float	rxcen, rycen ;	/* Sun center for REF image. */
float	str, vig, fc ;	/* Stray light, vignetting & F-corona  values. */
float	xcm, ycm ;	/* (x,y) coordinates of center-of-mass. */
float	unmass ;	/* Mass from unreliably low differences. */
float	unmassp ;	/* % Mass from unreliably low differences. */
float	unpct ;		/* Percentage from unreliably low differences. */
float	wtheta ;	/* Wedge theta coordinate. */

double	gcm2 ;		/* grams per cm**2 along line of sight. */
double	Ne ;		/* # electrons. */

static float difunr = 5.0e-10 ;	/* Threshold for unreliable differences. */
static double emass = 2.0e-24 ;	/* Mass per light-scattering electron. */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   cbthresh = cbth ;
   ixs = 0 ;
   iys = 0 ;

   /*** Open REF image file. ***/

   if (mx > 0)
      {
      if ((reffd = open (namref, 0)) < 0)
         {
	 fprintf (stderr,"cpmass> Can't open file %s\n", namref) ;
	 return (-1) ;
	 }
      close (reffd) ;

      /*** Decommutate housekeeping for REF image. ***/

      cphskp (namref) ;
      rxcen = xcen ;
      rycen = ycen ;
      rtype = idtype ;
      nbyrec = npix * nbypix ;
      ipb = ip ;
      irb = ir ;
      lpb = lp ;
      lrb = lr ;
      }

   /*** Decommutate housekeeping for CME image. ***/

   cphskp (namcme) ;

   cxcen = xcen ;
   cycen = ycen ;
   ctype = idtype ;
   nbyrec = npix * nbypix ;
   ipc = ip ;
   irc = ir ;
   lpc = lp ;
   lrc = lr ;

/*
 *   fprintf(stderr, "cxcen:%-6.2f cycen:%-6.2f nbypix:%1d npix:%3d\n",
 *                   cxcen, cycen, nbypix, npix) ;
 */

   /*** Verify that the image has been calibrated. ***/

   if (((mx > 1) && (rtype != 3)) || (ctype != 3 && ctype !=6))
      {
      fprintf (stderr,
              "cpmass> Image(s) not calibrated ... excess mass not done.\n") ;
      fprintf (pgmout,
              "cpmass> Image(s) not calibrated ... excess mass not done.\n") ;
      return (-2) ;
      }

   /*** Request sun center coordinates for CME image. ***/

   if (cxcen <= 0.0 || cycen <= 0.0)
     {
      fprintf (stderr, "Enter sun center coordinates (HIGH res) for CME image : ") ;
      fscanf  (stdin,  "%f,%f", &cxcen, &cycen) ;
      cxcen = cxcen / nres ;
      cycen = cycen / nres ;
     }

   /*** Request sun center coordinates for REF image. ***/

   if (mx > 0)
      {
      if (rxcen <= 0.0 || rycen <= 0.0)
        {
         fprintf (stderr,
                  "Enter sun center coordinates (HIGH res) for REF image : ") ;
         fscanf  (stdin,  "%f,%f", &rxcen, &rycen) ;
         rxcen = rxcen / nres ;
         rycen = rycen / nres ;
         if (rxcen <= 0.0)  rxcen = cxcen ;
         if (rycen <= 0.0)  rycen = cycen ;
        }
      ixs = cxcen - rxcen ;
      iys = cycen - rycen ;
      }

   /*** Stray Light & Vignetting removal required ? ***/

   if (lstr == 2 && lvig == 2)  mv = 1 ;  else mv = 0 ;

   fprintf (stderr, "Please wait ... mass computation in progress.\n") ;

   /*** Open CME image file. ***/

   if ((cmefd = open (namcme, 0)) < 0)
      { fprintf (stderr,"cpmass> Can't open file %s\n", namcme) ;  return (-3) ; }
   close (cmefd) ;

   /*** Compute the cross-sectional area (square cm) for one pixel. ***/

   cm2pix = (RSUN*rspix) * (RSUN*rspix) ;

   /*** Initialize variables. ***/

   cc      = pscale ;
   fc      = 0.0 ;
   str     = 0.0 ;
   vig     = 1.0 ;
   Ne      = 0.0 ;
   cmemass = 0.0 ;
   kmass   = 0.0 ;
   negmass = 0.0 ;
   refmass = 0.0 ;
   unmass  = 0.0 ;
   xcm     = 0.0 ;
   ycm     = 0.0 ;

   num     = 0 ;
   numneg  = 0 ;
   numun   = 0 ;
   numth   = 0 ;

   fprintf (stdout, "ixs: %3d iys: %3d\n", ixs, iys) ;
   fprintf (stdout, "np: %8d   winpos [beg]: %3d %3d   winpos [end]: %3d %3d\n",
            np, winpos[0][0], winpos[0][1], winpos[np-1][0], winpos[np-1][1]) ;

   /*** Clear data buffer. ***/

   xbbuf  = (float *) malloc (np * sizeof (float)) ;
   xmbuf  = (float *) malloc (np * sizeof (float)) ;
   for (i=0 ; i<np ; i++)  xbbuf [i] = xmbuf [i] = 0.0 ;

   /*** Read REF and CME images into memory arrays. ***/

   if (mx > 0)
      irbuf = (short *) rdimg (namref) ;
   icbuf = (short *) rdimg (namcme) ;
   rbuf = (char *) irbuf ;
   cbuf = (char *) icbuf ;

   /* ******* Pixel Loop ******* */

   i       = -1 ;
   num     = 0 ;
   numth   = 0 ;
   cbsum   = 0.0 ;
   refbrt  = 0.0 ;
   refbrt0 = 0.0 ;
   cmebrt  = 0.0 ;
   cmebrt0 = 0.0 ;
   crrbrt  = 0.0 ;

   for (ipix = 0 ; ipix < np ; ipix++)
      {
/*      fprintf (stderr, "acpmass...pixel # %3d\n", ipix) ; */

      ixg = winpos [ipix][0] ;		/* Window x-coordinate. */
      iyg = winpos [ipix][1] ;		/* Window y-coordinate. */

      iix = ixg - xocp ;			/* Image x-coordinate. */
      iiy = yocp - iyg ;			/* Image y-coordinate. */

      fx = (float) iix ;			/* Float image x-coordinate. */
      fy = (float) iiy ;			/* Float image y-coordinate. */

      if (rcoord (&r, &theta, &fx, &fy, -1, scroll) != 0)  goto oops ;

/*      fprintf (stdout,
 *               "fx:%6.2f fy:%6.2f  r:%6.2f theta:%6.2f\n", fx, fy, r, theta) ;
 */

      /*** Draw dot to indicate pixel has been integrated. ***/

      gl_wvec (ixg, iyg, ixg, iyg, ov_blue) ;

/* fprintf (stdout, "fx:%6.2f fy:%6.2f . . . passed r,th test.\n", fx, fy) ; */

      i++ ;
      num++ ;
      xbbuf [i] = xmbuf [i] = 0.0 ;

      irad = (int) ((r - 1.25) / 0.25) ;
      gcm2 = medsun (r) ;

      /*** Use external stray/vignet functions ***/

      if (!H.vig)  vig = vignet (r) ;	/* Dale Gary's vignetting. */
      if (!H.str)  str = stray (r) ;	/* Adam Csoeke-Poeckh's stray light. */
      if (!H.f  )  fc  = fcorona (r) ;	/* ATM F-corona. */
      if (lcc)     cc  = calcor (r) * pscale ;	/* Munro calib. correction. */
      if (ljc)     cc  = 2.0 * pscale ;		/* Jupiter calib. correction. */

      /*** Absolute mass: Remove stray light from CME image. ***/

      if (mx  ==  0)
         {
         icp = iix + iiy * npix ;
         cbrite  = (((float) *(icbuf + icp) - str) * cc * vig) - fc ;
         bcme    = (((float) *(icbuf + icp) - str) * cc * vig) - fc ;
         cmebrt += bcme ;
/*
 *       fprintf (stdout,
 *          "   icbuf[%3d][%3d]:%3d   str:%8.2e   vig:%8.2e   cbrite: %8.2e\n",
 *          iix, iiy, *(icbuf + icp), str, vig, cbrite) ;
 */
         cbsum  += cbrite ;
/*
 *         if (cbrite > cbthresh)
 */
            {
            numth++ ;
            cmebrt0 += cbrite ;
            cmemass += cbrite * gcm2 * cm2pix * cbrite ;
            }
         xbbuf [i] = cbrite ;
         xmbuf [i] = (float) (cbrite * gcm2 * cm2pix) ;
/*
 *         fprintf (stdout,
 *           "gcm2: %8.2e   cm2pix: %8.2e   cbrite: %8.2e   data [%d]: %8.2e\n",
 *            gcm2, cm2pix, cbrite, i, data [i]) ;
 */
         }

      /*** Excess mass: compute pixel coordinate for REF image. ***/

      else
         {
         iiyb = iiy - iys ;
         iixb = iix - ixs ;

         icp = iix + iiy*npix ;
         irp = iixb + iiyb*npix ;
         cbrite = (float) (*(icbuf + icp)  - *(irbuf + irp)) * cc * vig ;
         bref   = (((float) *(irbuf + irp) - str) * cc * vig) - fc ;
         bcme   = (((float) *(icbuf + icp) - str) * cc * vig) - fc ;
         refbrt += bref ;
         cmebrt += bcme ;
         cbsum  += cbrite ;
/*
 *         if (cbrite > cbthresh)
 */
            {
            numth++ ;
            refbrt0 += bref ;
            cmebrt0 += bcme ;
            refmass += bref * gcm2 * cm2pix ;
            cmemass += bcme * gcm2 * cm2pix ;
            }
         xbbuf [i] = cbrite ;
         xmbuf [i] = (float) (cbrite * gcm2 * cm2pix) ;
         }

/*
 * fprintf (stdout, "gcm2:%g cm2pix:%g v:%g\n", gcm2, cm2pix, v) ;
 * fprintf (stdout, "icbuf:%-5d irbuf:%-5d\n",
 *          *(icbuf + icp), *irbuf + irp)) ;
 * fprintf (stdout, "xmbuf [%3d][%3d]: %g\n", iix, iiy, xmbuf [iiy][iix]) ;
 */

      kbrite += xbbuf [i] ;
      kmass  += xmbuf [i] ;

      if (mx > 0)
         {
         if (xbbuf [i] < difunr && xbbuf [i] > -difunr)
            {
            unmass += xmbuf [i] ;
            numun++ ;
            }
         if (xmbuf [i] < 0.0)
            {
            negmass += xmbuf [i] ;
            numneg++ ;
            }
         }

      ycm += xmbuf [i] * fy ;
      xcm += xmbuf [i] * fx ;

pixend:    n++ ;		/* fprintf (stdout, "pixend...iix%3d\n", iix) ; */


recend: n++ ;		/* fprintf (stdout, "recend...iiy:%3d\n", iiy) ; */

      } /* End pixel loop. */

   fprintf (pgmout, "# pixels : %d\n\n", num) ;

/*   fprintf (pgmout, "F-corona has NOT been removed.\n") ; */

   if (lcc)
      fprintf (pgmout,
              ">>> NOTE: Munro calibration correction has been applied. <<<\n\n") ;
   if (ljc)
      fprintf (pgmout,
              ">>> NOTE: Jupiter calibration correction has been applied. <<<\n\n") ;

   if (refbrt > 0.0)
      crrbrt = (cmebrt - refbrt) / refbrt ;
   else
      crrbrt = 0.0 ;

   Ne = (double) kmass / emass ;

   /*** Center of mass. ***/

   if (kmass  !=  0.0)
      {
      ixcm = (int) (xcm / kmass) ;
      iycm = (int) (ycm / kmass) ;
      }

   xcm = (float) ixcm ;
   ycm = (float) iycm ;

   if (rcoord (&rcm, &thcm, &xcm, &ycm, -1, scroll) != 0)  goto oops ;
   fprintf (stdout, "rcm: %7.2f   thcm: %7.2f   xcm: %7.2f   ycm: %7.2f   scroll: %d\n",
	            rcm, thcm, xcm, ycm, scroll) ;

   /*** Draw "+" on display to indicate center of mass location. ***/
   /*** Draw lines connecting 4 wedge corners. ***/

   if (igr == 1)
      {
      ixcmg = ixcm/nres + xocp ;
      iycmg = yocp - iycm/nres ;
      gl_wvec (ixcmg-5, iycmg, ixcmg+5, iycmg, ov_red) ;
      gl_wvec (ixcmg, iycmg-5, ixcmg, iycmg+5, ov_red) ;

      ix11 = (int) p11 / nres + xocp ;
      ix12 = (int) p12 / nres + xocp ;
      ix21 = (int) p21 / nres + xocp ;
      ix22 = (int) p22 / nres + xocp ;
      iy11 = yocp - (int) r11 / nres ;
      iy12 = yocp - (int) r12 / nres ;
      iy21 = yocp - (int) r21 / nres ;
      iy22 = yocp - (int) r22 / nres ;
      }

   if (mx == 0)			/* Absolute brightness/mass. */
      {
      fprintf (pgmout, "[CME brightness] (%d pixels) :   %9.2e B/Bsun\n",
              numth, cmebrt) ;
/*
 *      if (cbthresh > 0.0)
 */
      fprintf (pgmout, "[Brightness > %9.2e B/Bsun] (%d) :  %9.2e B/Bsun\n\n",
               cbthresh, numth, cmebrt0) ;

      fprintf (pgmout,
               "[Integrated absolute brightness] : %9.2e B/Bsun\n\n",
               cbsum) ;

      fprintf (pgmout,
        "[Total integrated absolute mass] :%9.2e grams <->%9.2e electrons.\n\n",
               kmass, Ne) ;
      }
   else
   if (mx  > 0)			/* Excess brightness/mass. */
      {
      fprintf (pgmout, "[CME brightness] (%6d pixels) :%9.2e B/Bsun\n",
              numth, cmebrt) ;
      fprintf (pgmout, "[REF brightness] (%6d pixels) :%9.2e B/Bsun\n",
               numth, refbrt) ;
      fprintf (pgmout, "[CME - REF brightness] :          %9.2e B/Bsun (excess brightness)\n",
               cmebrt - refbrt) ;
      fprintf (pgmout, "[(CME - REF) / REF brightness] :  %9.2e        (delta B / B)\n\n",
	       crrbrt) ;

      masspct = ((cmemass - refmass) / refmass) * 100.0 ;
      fprintf (pgmout, "CME image mass :          %9.2e grams\n",
               cmemass) ;
      fprintf (pgmout, "REF image mass :          %9.2e grams\n",
               refmass) ;
      fprintf (pgmout,
      "[Integrated excess mass] :%9.2e grams <-> %9.2e electrons (%6.2f%%)\n\n",
               kmass, Ne, masspct) ;

      unpct    = ( (float) numun  / (float) num ) * 100.0 ;
      negpct   = ( (float) numneg / (float) num ) * 100.0 ;
      unmassp  = (unmass / kmass) * 100.0 ;
      negmassp = (negmass / kmass) * 100.0 ;
      fprintf (pgmout,
              "Unreliably low points [%5.2f%% (%d/%d) with -%9.2e < B <%9.2e]\n",
               unpct, numun, num, difunr, difunr) ;
      fprintf (pgmout, "contributed %9.2e grams [%5.2f%%] to the total.\n\n",
               unmass, unmassp) ;

      fprintf (pgmout,
              "Negative mass points [%5.2f%% (%d/%d) with B < 0.0]\n",
               negpct, numneg, num) ;
      fprintf (pgmout, "contributed %9.2e grams [%5.2f%%] to the total.\n\n",
               negmass, negmassp) ;

      }

   fprintf (pgmout, "The center of mass within this area is at %5.2f Rsun   %7.2f degrees.\n",
      rcm, thcm) ;
   fprintf (pgmout, "--------------------------------------------------------------------------------\n") ;

   if (mx > 0)  close (reffd) ;
   close (cmefd) ;
   free (xbbuf) ;
   free (xmbuf) ;
   return (0) ;

   /*** Coordinate out-of-range error. ***/

oops:
   fprintf (stderr, "Invalid range: radius or theta") ;
   free (xbbuf) ;
   free (xmbuf) ;
   return (-10) ;
}

/* xmin (x1, x2, x3, x4)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find minimum of 4 floating point variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * 1 March 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

float
xmin (x1, x2, x3, x4)

float x1, x2, x3, x4 ;

{
float x ;

   x = x1 ;
   if (x > x2)  x = x2 ;
   if (x > x3)  x = x3 ;
   if (x > x4)  x = x4 ;

   return (x) ;
}

/* xmax (x1, x2, x3, x4)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find maximum of 4 floating point variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger
 * 1 March 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

float
xmax (x1, x2, x3, x4)

float x1, x2, x3, x4 ;

{
float x ;

   x = x1 ;
   if (x < x2)  x = x2 ;
   if (x < x3)  x = x3 ;
   if (x < x4)  x = x4 ;

   return (x) ;
}
