/* ellip.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Plot the distance of earth from sun (in AU units) versus angle.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    3 Aug 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

main (argc, argv)

int argc ;
char *argv [] ;

{
   static char namemc [] = "ellip.cgm" ;
   static char title [60] ;
   static char labx  [40] ;
   static char laby  [40] ;

   static float day   [3660] ;		/* Distance array. */
   static float dist  [3660] ;		/* Distance array. */
   static float eanom [3660] ;		/* Eccentric anomaly. */
   static float lum   [3660] ;		/* Lumens array. */
   static float manom [3660] ;		/* Mean anomaly. */
   static float th    [3660] ;		/* Theta array. */

   float doy ;
   float EA ;				/* Eccentric anomaly */
   float MA ;				/* Mean anomaly */
   float R ;				/* Distance from Sun to Earth. */

   double a = 1.0 ;			/* Mean distance Earth-Sun in AU. */
   double e = 0.0167 ;			/* Eccentricity of Earth's orbit. */
   double theta ;			/* Angle ccw (0 degrees at perihelion.*/
   double degrad ;			/* Degrees per radian. */
   double raddeg ;			/* Radians per degree. */
   double PI = 3.14159265 ;		/* Pi. */

   int i = 0 ;				/* Index. */
   int idoy = 0 ;			/* Day-of-year. */
   int ihour = 0 ;			/* Hour. */
   int iyear = 80 ;			/* Year. */
   int nval ;				/* # values in plotting arrays. */
   int ixscale = 0 ;			/* X axis: linear. */
   int iyscale = 0 ;			/* Y axis: linear. */
   int ibkg = 1 ;

   raddeg = PI / 180.0 ;
   degrad = 180.0 / PI ;
   printf ("raddeg: %f\n", raddeg) ;

   opngks_ () ;

   /*** Time loop. ***/

   i = 0 ;
   for (doy = 1.0 ; doy < 367.0 ; doy += 0.25)
      {
      idoy = (int) doy ;
      ihour = (doy - (float) idoy) * 24.0 ;

      /*** Compute R, MA, EA. ***/

      earthorb (iyear, idoy, ihour, &R, &MA, &EA) ;

      day   [i] = doy ;			/* Day of Year. */
      dist  [i] = (float) R ;		/* Earth-Sun distance (A.U.). */
      eanom [i] = EA ;			/* Eccentric anomaly [degrees]. */
      manom [i] = MA ;			/* Mean      anomaly [degrees]. */
      lum   [i] = (float) 1 / (R * R) ;	/* Luminosity (w.r.t. mean). */
      th    [i] = (float) EA ;		/* Circular angle w.r.t. perihelion. */

      fprintf (stdout, "iyear:%2d  idoy:%03d  ihour:%02d  R:%7.4f  MA:%7.2f  EA:%7.2f\n",
	       iyear, idoy, ihour, R, MA, EA) ;
/*
      printf ("i: %4d   d: %9.4f  th: %7.2f  cos(th): %7.2f\n",
	       i, d [i], th [i], cos (th [i])) ;
 */
      i++ ;
      }

   nval = i ;

   sprintf (title, "Earth-Sun Distance (A.U.) vs. DOY [19%02d]", iyear) ;
   sprintf (labx,  "Day Of Year [19%02d]", iyear) ;
   sprintf (laby,  "Earth-Sun Distance [A.U.]") ;
   xyplot  (day, dist, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg) ;

   sprintf (title, "Solar Luminosity vs. DOY [19%02d]", iyear) ;
   sprintf (labx,  "Day Of Year [19%02d]", iyear) ;
   sprintf (laby,  "Solar Luminosity [w.r.t. mean]") ;
   xyplot  (day, lum, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg) ;

   sprintf (title, "Eccentric Anomaly vs. DOY [19%02d]", iyear) ;
   sprintf (labx,  "Day Of Year [19%02d]", iyear) ;
   sprintf (laby,  "Eccentric Anomaly [degrees]") ;
   xyplot  (day, eanom, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg) ;

   sprintf (title, "Mean Anomaly vs. DOY [19%02d]", iyear) ;
   sprintf (labx,  "Day Of Year [19%02d]", iyear) ;
   sprintf (laby,  "Mean Anomaly [degrees]") ;
   xyplot  (day, manom, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg) ;

   sprintf (title, "Mean Anomaly vs. Eccentric Anomaly") ;
   sprintf (labx,  "Eccentric Anomaly [degrees]") ;
   sprintf (laby,  "Mean Anomaly [degrees]") ;
   xyplot  (eanom, manom, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg) ;


   clsgks_ () ;
   exit (0) ;
}
