/* ellipse.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Plot the distance of earth from sun (in AU units) versus angle.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   28 Oct 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

main (argc, argv)

int argc;
char *argv [];

{
   static char namemc [] = "ellipse.cgm";
   static char title [60] = "Earth-Sun Distance";
   static char labx [40] = "Angle [degrees] w.r.t. Perihelion";
   static char laby [40] = "Distance [AU]";

   static float d [3620];		/* Distance array. */
   static float lum [3620];		/* Lumens array. */
   static float th [3620];		/* Theta array. */

   double a = 1.0;			/* Mean distance Earth-Sun in AU. */
   double e = 0.017;			/* Eccentricity of Earth's orbit. */
   double r;				/* Distance from Sun to Earth. */
   double theta;			/* Angle ccw (0 degrees at perihelion.*/
   double raddeg;			/* Radians per degree. */
   double PI = 3.14159;			/* Pi. */

   int i = 0;
   int nval;				/* # values in plotting arrays. */
   int ixscale = 0;			/* X axis: linear. */
   int iyscale = 0;			/* Y axis: linear. */
   int ibkg = 1;

   raddeg = PI / 180.0;
   printf ("raddeg: %f\n", raddeg);

   opngks_ ();

   for (theta = 0.0; theta <= 360.0; theta += 0.1)
      {
      r = (a * (1.0 - e * e)) / (1.0 + e * cos (raddeg * theta));
      d [i] = (float) r;			/* Earth-Sun distance. */
      lum [i] = (float) 1 / (r * r);		/* Luminosity. */
      th [i] = (float) theta;			/* Angle w.r.t. perihelion. */
/*
      printf ("i: %4d   d: %9.4f  th: %7.2f  cos(th): %7.2f\n", i, d[i], th[i], cos(th[i]));
 */
      i++;
      }

   nval = i;
   xyplot (th, d, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg);

   strcpy (title, "Solar Luminosity vs. Earth's Orbital Angle");
   strcpy (laby, "Solar Luminosity");
   xyplot (th, lum, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg);

   clsgks_ ();
   exit (0);
}
