;+ =========================================================================== +
;
; FILE MK3.PRO  the display program for Mauna Loa Mark-III K-coronameter images.
;
;
; REQUIREMENTS :  IDL Version 2 with widgets, or better.
;		  X windows graphics device.
;			
;   To test to see if widgets are available :
;
;   IDL> if (!D.FLAGS AND 65536) NE 0 THEN PRINT, 'WIDGETS ARE AVAILABLE.'
;		
;
; The MK3 data must be written from the MK3 achive media to disk, placing
; the data into subdirectories, one per day.
;
; - The program has two main modes: "Display one image" (single image) mode or 
;   "Show a movie" (movie) mode.  
; - The mode is selected from the main options widget, but first select the
;   desired option for "Normal display" (not a difference image) and 
;   "Make a difference" (difference image).  
; - Once single or movie modes are selected, each will create it's own
;   widget menus.
; - These menus are designed to display either normal or difference images
; - The remain option are independant and may be selected interactively 
;   during processing.
; NOTE: All widget menus are placed at thge top left corner of the screen.
;	This may result in smaller menus, such as the main options menu,
; 	being completely hidden.  Move the offending menu to as you would
;	any window on your system.
;
;
;                         /--- MK3ONE---MK3ONE_EV*-------------------\
;    MK3--------MK3_EVENT---                                         ---**
;    (mk3color)            \--- MK3MOVIE--MK3MOVIE_EV*---MK3MOVIE_DO---/   
;
;                                                                * returns to 
;            **---MK3DISPLAY-------------------MK3TV---MK3SUNCEN   mk3one_ev or
;                (mk3disk)   \----MK3DIFF---/                      to mk3movie_ev
;
;
;    Programs for MK3 have the naming convention of :  MK3*.PRO
;    Common block for these programs are named      :  MK3*.COMMON
;
;    Required program that don't use 
;    the naming convention of MK3*.PRO : any widget library routines
;					 ACKNOWLEDGE.PRO
;					 OPSYS_LGCL.PRO
;					 PSPLOT.PRO
;					 WINUP.PRO	
;
;- =========================================================================== -
                                                    

;+ =========================================================================== +
;
; NAME:  		
;	MK3_EVENT
;
; PURPOSE: 
;	The event handler for main MK3 option menu widget
;
; CATEGORY: 	
;	MK3 display
;
; CALLING SEQUENCE: 	
;	MK3_EVENT, EVENT
;
; CALLED BY: 	
;	MK3.PRO
;
; CALLS TO:  		
;	WINUP, ACKNOWLEDGE, MK3ONE, MK3MOVIE
;
; INPUTS:
;	EV : the event structure, variable depending on type of item clicked
;	also, values in commons
;
; OPTIONAL INPUTS: 
;	none
;
; OUTPUTS: 
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS: 	
;	MK3COLORS 	to set color tables states
;	MK3DISPFILES 	to pass printer status
;	MK3KEYWORDS 	for current options conditions
;	MK3NEW_WINDOW 	for reuse state
;	MK3PROLOCAL 	for the widget IDs of option buttons
;	MK3WIDGBASES 	for the widget IDs of parents and informational 
;
; SIDE EFFECTS: 		
;	none
;              	
; RESTRICTIONS: 		
;	none
;
; PROCEDURE: 		
;	Process the events from the main options menu widget.  Continue 
;	receiving events until "Cancel program" is clicked, thus sending 
;	return control to calling routine.  Or until "Display one image" or
;	"Show a movie" are selected. Each of these will call the appropriate
;	routines to contine processing.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cp_event" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3_event, event

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@mk3prolocal.common
;common toggles, norm, diff, snorth, sdisk, rulenone, rulerit

@mk3colors.common 
;common mk3colors, maxcolor, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset

@mk3keywords.common 
;common keywords, norm_diff, donorth, dodisk, doradii, doradial, $
;		 ruletimes, rulecondition, posfac, negfac, sin_mov

@mk3dispfiles.common 
;common dispfiles, filename, diffname1, diffname2, onewin, colo

@mk3new_window.common 
;common new_window, lastmark, sc, current_window, reuse, first_pass

  widget_control, event.id, get_uvalue=input	; which widget was clicked ?
  if (n_elements (input) eq 0) then input = ''
  type = strmid (tag_names (event, /structure_name), 7, 1000)  ; get widget type

  case type of 
    'BUTTON' : case input of

      'QUITPRO': widget_control, event.top, /destroy 	; kill all widgets

      'KILLSCL': begin
		   widget_control, event.top, /destroy  ; kill top widget
                   widget_control, base, sensitive=1    ; can use BASE again
                   end

      'DELWIN' : begin		; Widget to display active window and delete.

		   winup, group_leader=event.top, message=message
		   if (n_elements (message) ne 0) then $
		   widget_control, blurb, set_value=message
		   device, window=any_left  ; list of active windows
		   any_left = where (any_left ne 0)
		   still_there = where (any_left eq current_window)
		   if (still_there (0) eq -1) then $
		      first_pass = 1	; deleted currently active window
		   end

      'REUSE'	: reuse = 1
      'NOREUSE' : reuse = 0

      ; The user set options :

      'NORMAL'	: if (event.select ne 0) then begin	; not turning off

		    if (norm_diff eq 1) then begin	; Difference was active.
		      if (xregistered ('mk3one') or $	    ; Have wrong option
			  xregistered ('mk3movie')) then begin ; display active.
			acknowledge, message = $
					(['Cancel active top widget,', $
					  'either "Display one image"', $
					  'or "Show a movie". ', $
					  'It is incompatible ', $
					  'with "Normal display.'])

			; Undo normal event, put back to difference.

			widget_control, norm, set_button=0   ; undo 
			widget_control, diff, set_button=1   ;   event
			return
		      endif

;			colreset = 1			; reset colors

		    endif

		    coltyp=0  &  norm_diff=0
		    widget_control, scal, sensitive=0  ; not applicable
		  endif

      'DIFFERE'	: if (event.select ne 0) then begin	; not turning off

		    if (norm_diff eq 0) then begin	    ; Normal was active.
		      if (xregistered ('mk3one') or $	     ; Have wrong option
		          xregistered ('mk3movie')) then begin ; display active.
			acknowledge, message = $
				(['Cancel active top widget,',$
				  'either "Display one image"',$
				  'or "Show a movie". ', $
				  'It is incompatible ', $
				  'with "Make a difference".'])

			; undo difference event, put back to normal

			widget_control, norm, set_button=1  
			widget_control, diff, set_button=0 
			return
		      endif

;			colreset = 1  	; reset colors
		    endif

		    coltyp=1  &  norm_diff=1  
		    widget_control, scal, sensitive=1  ; now an option
		  endif

      'SOLNORTH': if (event.select eq 0) then donorth = 0   else donorth = 1

      'SOLDISK'	: if (event.select eq 0) then dodisk = 0    else dodisk = 1

      'SOLRADII': if (event.select ne 0) then doradii = 1   else doradii = 0

      'SOLRADIAL': if (event.select ne 0) then doradial = 1 else doradial = 0

      'RULNONE'	: if (event.select ne 0) then begin
		    ruletimes = 0
		    widget_control, ruleopt2, sensitive=0
                  endif

      'RULONCE'	: if (event.select ne 0) then begin
		    ruletimes = 1
		    widget_control, ruleopt2, sensitive=1
		  endif

      'RULMANY'	: if (event.select ne 0) then begin
		    ruletimes = 2
		    widget_control, ruleopt2, sensitive=1
		  endif

      'RULINPT'	: if (event.select ne 0) then begin
		    ruletimes = 3
		    widget_control, ruleopt2, sensitive=1
		  endif

      'RULIT'	: if (event.select ne 0) then rulecondition = 0

      'MARKIT'	: if (event.select ne 0) then rulecondition = 1

      'NORULMARK' : if (event.select ne 0) then rulecondition = 2

      'AUTOSCL'	: if (event.select ne 0) then begin		; default clip
		    negfac = 65.0  &  posfac = 65.0
		  endif 	

      'MANUSCL'	: if (event.select ne 0) then begin
		    widget_control, base, sensitive=0 ; no BASE inputs now
		    negfac = 65.0  &  posfac = 65.0		; default clip

		    ; Make a widget to ask for the scaling factors.

		    inpscl = widget_base (title='Enter difference factor', $
					/column, space=10, xpad=20, ypad=20)

		    go_on = widget_button (inpscl, value='READY', $
					              uvalue='KILLSCL')
		    lab = widget_label (inpscl, value = 'Defaults are 65')
		    tex = widget_slider (inpscl, title = 'Positive (red) ' + $
				'clipping value', minimum = 1, maximum = 254, $
				value = 65, xsize = 700, uvalue = 'POSINP') 
		    tex = widget_slider (inpscl, title = 'Negative (blue) ' + $
				'clipping value', minimum = 1, maximum = 254, $
				value = 65, xsize = 700, uvalue = 'NEGINP')

		    widget_control, inpscl, /realize
		    xmanager, 'mk3', inpscl, /modal, group_leader=event.top
		  endif           

      'ONEIMAG'	: begin
		    sin_mov = 0			; single image
		    onewin = -99		; no mk3 display window yet
		    mk3one, group_leader=event.top ; go display single image
		  end

      'MOVIEIM'	: begin
		    sin_mov = 1			; doing a movie
		    mk3movie, group_leader=event.top ; go make a movie
		  end

      else	: widget_control, blurb, $
			set_value='Invalid BUTTON value in MK3:' + input, /append
    endcase   	; of input            

    'SLIDER': case input of 		; only if diff and manual scale factor

      'POSINP'	: begin
		    widget_control, event.id, get_value=value
		    posfac = float (value) 
		  end

      'NEGINP'	: begin
		    widget_control, event.id, get_value=value
		    negfac = float (value)
		  end                            

      else	: widget_control, blurb, $
			set_value = 'Invalid SLIDER value' + input
    endcase   	; of input


    else : widget_control, blurb, set_value = 'Invalid type ' + $
					'of widget in MK3: ' + type, /append

  endcase      	; of type
end
;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3
;
; PURPOSE:
;	Create the main options menu widget.
;
; CATEGORY:  
;	MK3 display
;
; CALLING SEQUENCE:
;	MK3, [GROUP_LEADER=GROUP]
;
; CALLED BY: 
;	none
;
; CALLS TO: 
;	MK3_event (via XMANAGER)
;
; INPUTS: 
;	none
;
; OPTIONAL INPUTS:
;	GROUP :  The widget ID of the parent widget.
;
; OUTPUTS: 
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS: 
;	none
;
; COMMON BLOCKS:
;	MK3COLORS     	assigns the MK3 color tables
;	MK3KEYWORDS  	assigns the initial values for option conditions
;	MK3LOGIC_ENVRON	assigns the location of data
;	MK3NEW_WINMDOW  	assigns initial values for window management
;	MK3PROLOCAL 	creates the widget IDs of menu buttons
;	MK3WIDGBASES 	creates the widget IDs of parents and informational 
;
; SIDE EFFECTS:  
;	none
;
; RESTRICTIONS:  
;	none
;
; PROCEDURE:
;	Verifiy data location and create MK3 color tables,
;	then create the main options menu widget. 
;
; MODIFICATION HISTORY:
; 	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cp" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3, group_leader=group

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@mk3prolocal.common
;common toggles, norm, diff, snorth, sdisk, rulenone, rulerit

@mk3colors.common
;common mk3colors, maxcolor, diff_r, diff_g, diff_b, norm_r, norm_g, norm_b, $
;	         coltyp, colreset

@mk3keywords.common
;common keywords, norm_diff, donorth, dodisk, doradii, doradial, $
;		 ruletimes, rulecondition, posfac, negfac, sin_mov

@mk3logic_envron.common
;common logic_envron, dataplace

@mk3new_window.common
common new_window, lastmark, sc, current_window, reuse, first_pass
 
  if (xregistered ('mk3')) then return	 ; only want one running at a time

  ; See if logical/environment name is defined.

  opsys_lgcl, part1='MK3', discern=1, part2='DATA', separate=1,		       $
	dataplace=dataplace, ok=ok, 					       $
  	instructme=['Enter your designation for the data location and return', $
  		    '     VMS    ex: NEWMAX::MK3$DATA, $1$DIA4:[MK3]',         $
  		    '     ULTRIX ex: $TEST_DATA, /usr/users/Data',             $
		    '     UNIX   ex: /yoda/d/stanger/mk3img']

  if (not (ok)) then begin
    acknowledge, message = (['Progam is returning', $
			     'because of cancel'])
    return				; no input given to look for the data
  endif

  ; Windowing setup.

  device, get_screen_size = sc
  lastmark = 0
  reuse = 1
  first_pass = 1

  print, 'sc: ', sc

  ; Create C/P color table arrays.

  mk3color, maxcolor=maxcolor, diff_r=diff_r, diff_g=diff_g, diff_b=diff_b, $
			       norm_r=norm_r, norm_g=norm_g, norm_b=norm_b

  ; Initialize keywords.

  coltyp = 0     &  colreset = 1 
  norm_diff=0  &  donorth=1  &  dodisk=1  &  doradii=0    &  doradial=0
  ruletimes=0  &  rulecondition=0         &  posfac=65.0  &  negfac=65.0

  ; Create display control window.

  base = widget_base (title='MK3 image display routine', /column, space=20, $
		      xpad=20, ypad=20)
  dispway = widget_base (base, /row, space=20)
  oneim = widget_button (dispway, value='Display one image', uvalue='ONEIMAG')
  movie = widget_button (dispway, value='Show a movie', uvalue='MOVIEIM')
 
  normdiffopt = widget_base (base, /column, /frame, /exclusive)
  norm = widget_button (normdiffopt, value='Normal display ', uvalue='NORMAL')
  diff = widget_button (normdiffopt, value='Make a difference', $
			uvalue='DIFFERE')

  drawopt = widget_base   (base, /column, /frame, /nonexclusive)
  snorth  = widget_button (drawopt, value='Draw solar north', uvalue='SOLNORTH')
  sdisk   = widget_button (drawopt, value='Draw solar disk', uvalue='SOLDISK')
  sradii  = widget_button (drawopt, value='Draw solar radii lines', $
					uvalue='SOLRADII')
  sradial  = widget_button (drawopt, value='Draw radial lines', $
					uvalue='SOLRADIAL')

  rulopts  = widget_base   (base, /column, /frame)
  ruleopt1 = widget_base   (rulopts, /column, /frame, /exclusive)
  rulenone = widget_button (ruleopt1, value='No R and theta', uvalue='RULNONE')
  ruleone  = widget_button (ruleopt1, value='Pick one R and theta', $
				     uvalue='RULONCE')
  rulemulti = widget_button (ruleopt1, value='Pick multiple R and theta', $
				     uvalue='RULMANY')
  ruleinput = widget_button (ruleopt1, value='User input R and theta', $
				     uvalue='RULINPT')

  ruleopt2 = widget_base   (rulopts, /column, /frame, /exclusive)
  rulerit  = widget_button (ruleopt2, value='Draw R and theta ruler', $
				     uvalue='RULIT')
  markit   = widget_button (ruleopt2, value='Mark R and theta click', $
				     uvalue='MARKIT')
  norule   = widget_button (ruleopt2, value='No R and theta ruler or mark', $
				     uvalue='NORULMARK')
   
  scal  = widget_base   (base, /column, /frame, /exclusive)
  auto  = widget_button (scal, value='Scale data to maximum', uvalue='AUTOSCL')
  inscl = widget_button (scal, value='Input data scale maximum', $
					uvalue='MANUSCL')
 
  inf   = widget_base  (base, /frame, /column)
  orm   = widget_label (inf, value='Informational comments')
  blurb = widget_text  (inf, /scroll, ysize=3)

  xpdmenu, ['"Windowing" {', $
		'"Delete windows" DELWIN', $
		'"Reuse display window" REUSE',$
		'"New display windows" NOREUSE', '}' ], base

  endit = widget_button (base, value='Cancel program', uvalue='QUITPRO')

  widget_control, base, /realize

  widget_control, norm, set_button=1
  widget_control, snorth, set_button=1
  widget_control, sdisk, set_button=1
  widget_control, rulenone, set_button=1
  widget_control, ruleopt2, sensitive=0
  widget_control, rulerit, set_button=1
  widget_control, scal, sensitive=0
  widget_control, auto, set_button=1
 
  xmanager, 'mk3', base, group_leader=group

  first_pass = 1

end

;+ =========================================================================== +
