;+ =========================================================================== +
;
; file MK3SUNCEN.PRO - includes MK3TVCOPY, MK3CURSOR, MK3GET_R_T,
;			       MK3SUNCEN_EV, MK3SUNCEN
;+ =========================================================================== +
;
;
; NAME: 
;	MK3TVCOPY
;
; PURPOSE:
;	Output current window to selected printer
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
;	MK3TVCOPY, COLO=COLO, NORM_DIFF=NORM_DIFF
;
; CALLED BY:
;	MK3ONE
;
; CALLS TO:
;	PSPLOT
;
; INPUTS:
;	COLO : -1 means no hard copy allowed
;		1 means hard copy device is color
;		0 means hard copy device is color
;	NORM_DIFF :  0 means do a normal image
;                    1 means do a difference image
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	MK3WIDGBASES for the widget IDs of the informational windows.
;
; SIDE EFFECTS: 
;	Prints a file.
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Read (TVRD) a window into an array and print it to the selected device.
;
; MODIFICATION HISTORY:
; 	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cptvcopy" for MLSO Mark-III images.
;- =========================================================================== -


pro mk3tvcopy, colo=colo, norm_diff=norm_diff

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

  ; Copy the current window, then output it to desired device.
  ; colo=0 is black/white, colo=1 is color.

  widget_control, comment, set_value='Copying IDL window', /append
  snap = byte (tvrd (0, 0, !d.x_size, !d.y_size)) 
                                    
  oxsiz = 21.0  &   oysiz = oxsiz * (!d.y_size / float (!d.x_size))
  yoff  = 24.5  &   if (norm_diff eq 0) then xoff = 3.0 else xoff = 4.5    	

  if (colo eq 0) then  pfile = 'mk3bw.ps'  else  pfile = 'mk3color.ps'
  set_plot, 'ps', /copy   
  device, /landscape, bits_per_pixel=8, xsize=oxsiz, ysize=oysiz, $
          xoffset=xoff, yoffset=yoff, color=colo, filename=pfile

  tv, snap

  device, /close  &  set_plot, 'x' 
  widget_control, comment, set_value = 'Wrote window to file: ' + pfile, /append
 
;  if (colo eq 0) then spawn, 'print/setup=psi/delete/notify idl.ps' $
;	          else spawn, 'print/que=color$script/notify/delete idl.ps'

;25 Feb 93  psplot, filename = 'mk3.ps', color = colo, /delete

   if (colo eq 0) then $
      print, 'Wrote window to B/W postscript file:   ' + pfile $ 
   else if (colo eq 1) then $
      print, 'Wrote window to color postscript file: ' + pfile

  widget_control, onebase, sensitive=1

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3CURSOR
;
; PURPOSE:
; 	Draw user's requested R and theta.
;	
; CATEGORY:
;	MK3DISPLAY
;
; CALLING SEQUENCE:
;	MK3CURSOR, XCUR=XCUR, YCUR=YCUR, SC_SIZ=SC_SIZ, SIN_MOV=SIN_MOV, $
;			  RULECONDITION=RULECONDITION, RULETIMES=RULETIMES, $
;			  RADIUS=RADIUS, THETA=THETA, HGHLIT=HGHLIT, RD=R, $
;			  RADIALMAX=RADIALMAX, COLO=COLO, ROLL=ROLL
;
; CALLED BY:
;	MK3SUNCEN_EV 
;
; CALLS TO:
;	none
;
; INPUTS:
;	XCUR :  Radius value unit of solar radii
;	YCUR :  Position angle value unit of degrees
;	SC_SIZ : radius of area to be marked as suncenter
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;			  
;	RADIUS :  radius array for solar disk
;	THETA  :  array of all degree in a circle (360) convert to radians
;	HGHLIT :  the color with which the lines are drawn
;		  white for (top index) for difference images and B/W printer
;		  red (top index-1) for normal images, except to B/W printer
;	R : the sun's radius in pixel, (and a terrible variable name)
;	RADIALMAX : the largest radial extent for this image
;	COLO  : -1 means no hard copy allowed
;		 1 means hard copy device is color
;		 0 means hard copy device is color
;	ROLL : the angle of solar north in radians, from up
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	MK3NEW_WINDOW  for value of CURRENT_WINDOW
;	MK3WIDGBASES   for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Gets user input (cursor click) and turns it into R and theta.
;	Draw over top of the already displayed image R/Theta using option
;	requested by the user.  
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cpcursor" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3cursor, xcur=xcur, ycur=ycur, sc_siz=sc_siz, sin_mov=sin_mov, $
			  rulecondition=rulecondition, ruletimes=ruletimes, $
			  radius=radius, theta=theta, hghlit=hghlit, rd=r, $
			  radialmax=radialmax, colo=colo, roll=roll

@mk3new_window.common

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

   repeat begin	; keep doing R and theta until user clicks near sun center,
		; for multi-ruler case.

   if (ruletimes ne 3 and colo eq -1) then begin	; user clicked on screen
      rcur = xcur^2 + ycur^2  &  rcur = sqrt (rcur)	; radius of cursor click
      explain = 'Radius (solar radii)          : ' + strtrim(rcur/r)
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain,/append $
      else widget_control, news, set_value=explain, /append 
   
      Tclick = atan (ycur, xcur)		; theta (radians) of click
      pos = Tclick / !dtor			; theta in degrees
      if (pos lt 0) then pos = 360 + pos	; adjust to subscript values

;2/2/93      realpos = pos - 90 - 0		; correct for polar and Mark-III

      realpos = pos - roll			; roll corrects for polar 
						; and Mark-III 

      if (realpos lt 0) then realpos = realpos + 360	; if neg. position angle
							; c.c.w. from "+X axis"

      explain = 'Position angle (degrees) : ' + strtrim (realpos)
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain,/append $ 
      else widget_control, news, set_value=explain, /append
 
   endif else begin			; user input R and theta
      rcur = xcur * r			; radii times pixels per radii

      pos = (roll + ycur) mod 360	; add to polar adjusted roll
      tclick = pos * !dtor

   endelse

   if (rulecondition eq 1) then $			; mark cursor click
      oplot, [rcur], [Tclick], psym=4, color=hghlit, /polar	

   if (rulecondition eq 0) then begin			; make a ruler

      ; Line from sun center outward thru click.

      oplot, [0, radialmax], [Tclick, Tclick], /polar, color=hghlit     

      ; Each of the ruler tick marks.

      pos = fix (pos + 0.5)		; round pos to nearest integer

      thru_R  = 5.0			; ticks out to thru_R + 1 radii
      tick = 5				; degree length of long tick
      by = 1.0				; ticks every "by" of a solar radius

      for long_short=0,1 do begin	; long ticks then short ticks

        if ((pos ge tick) and (pos le 360-tick)) then $   	

         ; plot long ruler ticks at 1 thru 5 solar radii

         for i = 0.0, thru_R, by  do $
	    oplot, radius (pos - tick:pos+tick) + (r*i),  $
			   theta (pos-tick:pos+tick), /polar, color=hghlit $

         else begin		; just in case the ticks cross the "+X axis"

         ; How much above "+X" axis ?

         if (pos ge 0 and pos lt tick*2) then above = tick + pos $
					 else above = tick - (360-pos)
         below = 360 - ((tick*2)-above)		; how much below "+X" axis ?

         for i = 0.0, thru_R, by do begin	

	   ; From +X axis upward.

	   oplot, radius (0:above) + (r*i), theta (0:above), /polar, $
		  color=hghlit

	   ; From +X axis down.

	   oplot, radius (below:360) + (r*i), theta (below:360), /polar, $
		  color=hghlit
         endfor
      endelse 

        tick = 2			; degree length of short tick
        by = 0.1			; ticks every "by" of a solar radius
     endfor

   endif

   if (ruletimes eq 2 and colo eq -1) then begin  ; multiple R & theta to screen
      explain = 'Click cursor at desired position | click sun center to stop'
      if (sin_mov eq 0) then $
           widget_control, comment, set_value=explain,/append $
      else widget_control, news, set_value=explain, /append

      wshow, current_window
      cursor, xcur,  ycur, /down
   endif
   
   endrep until  $
      ((abs (xcur) le sc_siz) and (abs (ycur) le sc_siz)) or     $
       (ruletimes ne 2)  or                                    $
       (colo ne -1) 
                                                             
   explain = 'R and theta done, continue processing.'
   if (sin_mov eq 0) then $
	 widget_control, comment, set_value=explain, /append $
    else widget_control, news, set_value=explain, /append 

end
;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3GET_R_T
;
; PURPOSE:
; 	Make widget that will allow use input of R and Theta.
;
; CATEGORY:
;	Mark-III display	
;
; CALLING SEQUENCE:
;	MK3GET_R_T
;
; CALLED BY:
;	MK3SUNCEN
;
; CALLS TO:
;	MK3SUNCEN_EV (via XMANAGER)
;
; INPUTS:
;	none
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	MK3GET_R_T   widget IDs for R and Theta manual input and input status
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Create the widget which will prompt user to input R and Theta values.
;
; MODIFICATION HISTORY :
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cpget_r_t" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3get_r_t

@mk3get_r_t.common
;common get_r_t, doingit, r_ok, t_ok

  r_ok=0  &  t_ok=0		; initialize

  inpget = widget_base (title='Enter Radius and Theta', /column, $
			space=20, xpad=20, ypad=20)
   doingit = widget_button (inpget, value='Ready', uvalue='READY')

   get_r_t = widget_base (inpget, /column, space=20)
   get_r   = widget_base (get_r_t, /column, space=10)
   junkr = widget_label (get_r, $
			 value='Enter Radius (solar radii) and hit RETURN')
   r_it = widget_text  (get_r, /editable, uvalue='RADII')
   get_t = widget_base (get_r_t, /column, space=10)   
   junkt= widget_label (get_t, $
			value='Enter position angle (degrees) and hit RETURN')
   t_it = widget_text (get_t, /editable, uvalue='THETA')

   canit = widget_button (inpget, value='Cancel', uvalue='CAN_R_T')

   widget_control, inpget, /realize

   widget_control, doingit, sensitive=0 

   xmanager, 'mk3suncen', inpget, event_handler='mk3suncen_ev', /modal

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3SUNCEN_EV 
;
; PURPOSE :
;	The event handler for manual R and Theta input widget
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
;	MK3SUNCEN_EV, EV
;
; CALLED BY:
;	MK3GET_R_T (via XMANAGER)
;
; CALLS TO:
;	MK3CURSOR
;
; INPUTS:
;	EV: the event structure, variable depending on type of item selected
;	also, values in commons
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	MK3USERINP	for passing to MK3CURSOR
;	MK3DISPFILES 	for passing COLO to MK3CURSOR
;	MK3GET_R_T   	widget IDs for R/Theta manual input and input status
;	MK3RULE		for passing to MK3CURSOR
;	MK3WIDGBASES 	for the widget IDs of informational widgets
;	
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Process events representing user input of R and Theta until the 
;	buttons "Ready" or "Cancel" are clicked. At that point control is
;	passed to MK3CURSOR or back to calling routine.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cpsuncen_ev" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3suncen_ev, ev

; handle the widget that is asking for the user to input R and theta

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@mk3userinp.common
;common userinp, xcur, ycur, radialmax, sc_siz, radius, theta, hghlit, r, roll

@mk3rule.common
;common rule, times, condition, state

@mk3dispfiles.common
;common dispfiles, filename, diffname1, diffname2, onewin, colo

@mk3get_r_t.common
;common get_r_t, doingit, r_ok, t_ok

  widget_control, ev.id, get_uvalue=input	; which widget was clicked ?
  if (n_elements (input) eq 0) then input = ''
  type = strmid (tag_names (ev, /structure_name), 7, 1000)   ; get widget type

  case type of 

    'BUTTON' : case input of

      'READY': begin
		 widget_control, ev.top, /destroy 	; kill top widgets

		 if (colo ne -1) then set_plot,'ps'	; back to print device

		 mk3cursor, xcur=xcur, ycur=ycur, sc_siz=sc_siz, roll=roll, $
			   rulecondition=condition, ruletimes=times, $
			   radius=radius, theta=theta, hghlit=hghlit, rd=r, $
			   radialmax=radialmax, sin_mov=state, colo=colo 
	       end 
                                         
      'CAN_R_T': begin
		   widget_control, ev.top, /destroy   ; kill top widgets
	 	   if (colo ne -1) then set_plot,'ps' ; back to print device
                 end

      else : begin
		explain ='Invalid BUTTON value in MK3SUNCEN: ' + input
		if (state eq 0) then $
		     widget_control, comment, set_value=explain, /append $
		else widget_control, news, set_value=explain, /append 
	      end
    endcase			; of input

    'TEXT' : $
      begin
        widget_control, ev.id, get_value=value, set_value=''

        case input of 

          'RADII' : $
	    begin       ; get and test input radius
              on_ioerror, error_jump              
	      explain = 'Entered radius of '+ value(0) +' solar radii'
	      if (state eq 0) then $
                   widget_control, comment, set_value=explain, /append $
              else widget_control, news, set_value=explain, /append 

              xcur = float (value (0))
              on_ioerror, null

              xcur = xcur < 10.0 	; keep it reasonable, but
					; still may not be visible
              r_ok = 1
              if (r_ok and t_ok) then widget_control, doingit, sensitive=1
            end

          'THETA' : $
	    begin       ; get and test input theta
	      on_ioerror, error_jump
	      explain ='Entered postion angle of ' + value(0) + ' degrees'
	      if (state eq 0) then $
		   widget_control,comment,set_value=explain, /append $
              else widget_control, news, set_value=explain, /append 

              ycur = float (value (0)) mod 360
              if (ycur lt 0) then ycur = ycur + 360
              on_ioerror, null

              t_ok=1
	      if (r_ok and t_ok) then widget_control, doingit, sensitive=1
	    end

	  else : begin
		   explain = 'Invalid TEXT value in MK3SUNCEN: ' + input
	           if (state eq 0) then $
			widget_control,comment, set_value=explain, /append $
          	   else widget_control, news, set_value=explain, /append 
                 end

        endcase 		; of input
      end			; of text    

    else : begin
	     explain = 'Invalid widget type in MK3SUNCEN: '  + type
	     if (state eq 0) then $
		  widget_control,comment, set_value=explain, /append $
             else widget_control, news, set_value=explain,/append 
           end

  endcase			; of type

  error_jump:

end

;+ =========================================================================== +


;+ =========================================================================== +
;
; NAME: 
;	MK3SUNCEN
;
; PURPOSE:
;	Draw solar disk, solar north, and radial grid, as requested by user.
;
; CATEGORY:
;	Mark-III display
;
; CALLING SEQUENCE:
;	MK3SUNCEN, IMSIZ=IMSIZ, HK=HK,  DONORTH=DONORTH, DODISK=DODISK, $
;		DORADII=DORADII, DORADIAL=DORADIAL, COLO=COLO, $
;	  	RULETIMES=RULETIMES, RULECONDITION=RULECONDITION, $
;		SIN_MOV=SIN_MOV, NORM_DIFF=NORM_DIFF, SAVPCOLOR=SAVPCOLOR, $
;		XPOS=XPOS, YPOS=YPOS, X_SIZ=X_SIZ, Y_SIZ=Y_SIZ,HOUSEKP=HOUSEKP
;
; CALLED BY:
; 	MK3DISPLAY, MK3MOVIE 
;
; CALLS TO:
;	MK3CURSOR, MK3GET_R_T
;
; INPUTS:
;	IMSIZ : the size of the MK3 image in pixels
;	HK    : Array of size (224 x N) where N is 1 for single images
;               and 2 for difference images. Contains the housekeeping
;               information for the current image(s)
;	DONORTH : 0 means don't draw solar north
;		  1 means do draw solar north on image
;	DODISK  : 0 means don't draw solar disk
;		  1 means do draw solar disk
;	DORADII : 0 means don't draw radius lines
;		  1 means do draw radius lines
;	DORADIAL : 0 means don't draw radial lines
;		   1 means do draw radial lines
;	COLO  : -1 means no hard copy allowed
;		 1 means hard copy device is color
;		 0 means hard copy device is color
;	RULETIMES : 0 means don't do pick any R/theta
;		    1 means pick 1 R/theta
;		    2 means pick multiple R/theta
;		    3 means enter R/theta
;	RULECONDITION : 0 means draw ruler s/c thru R/theta
;			1 means mark R/theta click w/ circle
;			2 means don't draw ruler or mark
;	SIN_MOV  : 0 means current display mode is for single images
;	           1 means current display mode is for movies
;	NORM_DIFF : 0 means do a normal image
;                   1 means do a difference image
;	SAVPCOLOR : the value in !P.COLOR, which is the max color index
;	XPOS  : X direction screen coordinate (device pixels), to place window
;	YPOS  : Y direction screen coordinate (device pixels), to place window
;	X_SIZ : Image X size
;	Y_SIZ : Image Y size
;	HOUSEKP : X direction size reserved for house keeping information
;
; OPTIONAL INPUTS:
;	none
;
; OUTPUTS:
;	none explicit, only through commons
;
; OPTIONAL OUTPUTS:
;	none
;
; COMMON BLOCKS:
;	MK3NEW_WINDOW  for value of CURRENT_WINDOW
;	MK3RULE	    for passing through the event handler destined for MK3CURSOR
;	MK3USERINP  for passing through the event handler destined for MK3CURSOR
;	MK3WIDGBASES for the widget IDs of the informational windows
;
; SIDE EFFECTS:
;	none
;
; RESTRICTIONS:
;	none
;
; PROCEDURE:
;	Convert solar and MK3 parameters for display over an already existing
;	MK3 image.  Add solar disk, solar north, and radial grid, as 
;	requested by user.
;
; MODIFICATION HISTORY:
;	1991 - Elaine Einfalt (HSTX)
;
;	23 March 1993 - Andrew L. Stanger
;	Adapted from "cpsuncen" for MLSO Mark-III images.
;- =========================================================================== -

pro mk3suncen, imsiz=imsiz, hk=hk,  donorth=donorth, dodisk=dodisk, $
		doradii=doradii, doradial=doradial, colo=colo, $
		ruletimes=ruletimes, rulecondition=rulecondition, $
		sin_mov=sin_mov, norm_diff=norm_diff, savpcolor=savpcolor, $
		xpos=xpos, ypos=ypos, x_siz=x_siz, y_siz=y_siz,housekp=housekp

@mk3new_window.common

@mk3widgbases.common
;common widgbases, base, onebase, movbase, blurb, comment, news, ruleopt2, scal

@mk3userinp.common
;common userinp, xcur, ycur, radialmax, sc_siz, radius, theta, hghlit, r, roll

@mk3rule.common
;common rule, times, condition, state

@mk3_img.common
; COMMON mk3_img, iyear, idoy, imonth, iday, ihour, iminute, isecond, $
;                 dtype, nbypx, npx, npy, channel, dmin, dmax, smin, smax, $
;                 scndir, scndur, pixrs, pangle

   ; Ignore calibration path (CM = in) image.

   if (ishft (hk (15) and '20'x, -5) eq 1) then return

   r = pixrs				; sun's radius in pixels
   radius = fltarr (361) + r		; radius array
   theta = findgen (361) * !dtor	; all theta converted to radians
   range = findgen (imsiz)		; the X and Y scale in pixels (square)
   radialmax = sqrt (imsiz^2.0 + imsiz^2.0)	; max possible radial length

   ; Highlight color is red for normal (unless b/w printer),
   ; white for differences.

   if (colo eq 0) then	$			; B/W printer.
      begin
	 hghlit = savpcolor			; white
	 suncol = savpcolor			; white
	 lincol = savpcolor			; white
	 norcol = savpcolor			; white
      end		$
   else			$
   if (norm_diff eq 0) then $			; Normal (direct) images.
      begin
         hghlit = !p.color - 4			; red
	 suncol = !p.color - 3			; yellow 
	 lincol = !p.color - 2			; green
	 norcol = !p.color - 1			; blue
      end		$
   else			$
   if (norm_diff eq 1) then $			; Difference images.
      begin
         hghlit = !p.color - 4			; red
	 suncol = !p.color - 3			; yellow 
	 lincol = savpcolor			; white
	 norcol = !p.color - 1			; blue
      end

   xcen = 255.5   &   ycen = 255.5

   ; Set up an invisible coordinate system with (0,0) at sun center.

   ter = ishft (hk (14) and '3c'x, -3)			; image sector mirror
   fov_x = range - xcen    &   fov_y = range - ycen	; coordinate system

   fov_pos_x0 = housekp       &   fov_pos_y0 = 0	; plotting start pos.
   fov_pos_x1 = housekp+x_siz &   fov_pos_y1 = y_siz	; plotting end pos.

   plot, fov_x, fov_y, /nodata, xstyle=5, ystyle=5, /noerase, $ 	
	 position=[fov_pos_x0,fov_pos_y0,fov_pos_x1,fov_pos_y1] 

;   axis, xax=0, 0, 0, xstyle=1  &  axis,yax=0,0,0 ,ystyle=1    ; for debugging

   if (dodisk) then $				; draw solar disk
      oplot, radius, theta, /polar, linestyle=1, color=suncol	

   if (doradii) then begin			; draw solar radius lines
      oplot, radius,     theta, /polar, linestyle=3, color=suncol   ; 1.0 Rsun
      oplot, radius*1.5, theta, /polar, linestyle=4, color=lincol   ; 1.5 Rsun
      oplot, radius*2,   theta, /polar, linestyle=3, color=lincol   ; 2.0 Rsun

      inc = 0.1
      for i = 1, 4 do $					; 0.1 Rsun increments.
      begin
         oplot, radius * (1.0 + i * inc), theta, /polar, linesty=1, color=lincol
         oplot, radius * (1.5 + i * inc), theta, /polar, linesty=1, color=lincol
         oplot, radius * (2.0 + i * inc), theta, /polar, linesty=1, color=lincol
      end
   endif

   percent = 0.10		; % of solar radius to mark with center circle
   sc_siz = r * percent 	; radius of area marked as suncenter (float)

   ; Get value of north for this image.

   roll = 0
   roll = (roll + 90) mod 360		; adjust for polar coordinates.

   north = roll * !dtor

   if (donorth) then begin		; draw line indicating north

      ; Draw circle about sun center.

      center = radius * percent    ; radius of area marked as suncenter (fltarr)
      oplot, center, theta, /polar, color=lincol 	; small circle about s/c

      ; Draw line pointing north.

      oplot, /polar, [sc_siz, r], [north, north], thick=3, color=norcol

      if (not (doradial)) then begin
         east  = (roll +  90) * !dtor
         south = (roll + 180) * !dtor
         west  = (roll + 270) * !dtor
         oplot, /polar, [sc_siz, r], [north, north], linesty=4, color=lincol
         oplot, /polar, [sc_siz, r], [east,  east ], linesty=4, color=lincol
         oplot, /polar, [sc_siz, r], [south, south], linesty=4, color=lincol
         oplot, /polar, [sc_siz, r], [west,  west ], linesty=4, color=lincol
      endif
   endif

   ; Cross hairs @ sun center are always drawn (a must if using multiple rulers)

   oplot, [-(sc_siz), sc_siz], [0, 0], color=lincol
   oplot, [0, 0], [-(sc_siz), sc_siz], color=lincol

   if (doradial) then begin

      ; Draw radial grid on image every 30 degrees.

      deg_sep = 30.0			   ; every 30 degrees
      three60 = 360 * !dtor  &  space = deg_sep * !dtor
      degctr = - fix (deg_sep)
      for i = north, north + three60, space do $		; radial line 
	 begin
	 degctr = degctr + fix (deg_sep)
	 if ((degctr mod 90) ne 0) then $
            oplot, /polar, [percent, radialmax], [i, i], linesty=1, color=lincol
	 end

      ; Draw radial grid on image every 90 degrees.

      deg_sep = 90.0			   ; every 90 degrees
      three60 = 360 * !dtor  &  space = deg_sep * !dtor
      for i = north, north + three60, space do $		; radial line 
         oplot, /polar, [percent, radialmax], [i, i], linesty=4, color=lincol
   endif

   if (ruletimes ne 0) then begin	; some sort of R and theta is wanted

      if (ruletimes lt 3) then begin	; user wants to click in a R and theta
                                             
         if (colo eq -1) then begin	; output to screen
            explain = 'Click cursor on desired position'
            if (sin_mov eq 0) then $
	         widget_control, comment, set_value=explain, /append $
	    else widget_control, news, set_value=explain, /append 
            wshow, current_window
            cursor, xcur, ycur, /down
            mk3cursor, xcur=xcur, ycur=ycur, sc_siz=sc_siz, sin_mov=sin_mov, $
			  rulecondition=rulecondition, ruletimes=ruletimes, $
			  radius=radius, theta=theta, hghlit=hghlit, rd=r, $
			  radialmax=radialmax, colo=colo, roll=roll

         endif else begin 		; can't cursor to device printer
            explain = 'Cursor invalid with printer, enter R and theta'
            if (sin_mov eq 0) then $
	         widget_control, comment, set_value=explain,/append $
	    else widget_control, news, set_value=explain, /append
            set_plot, 'X'		; so widget can ask for R and theta
            times = ruletimes  &  condition = rulecondition  &  state = sin_mov
            mk3get_r_t
         endelse

      endif else begin                       ; user wants to input R and theta
         times = ruletimes  &  condition = rulecondition  &  state = sin_mov
         set_plot, 'X'		; so widget can ask for R and theta
         mk3get_r_t 				; make widget to ask for input
      endelse

   endif 

end

;+ =========================================================================== +
