;+ =========================================================================== +
; NAME:
;        PSPLOT
; PURPOSE:
;        print a PostScript plot file
; CALLING SEQUENCE:
;        PSPLOT [,VERSION][DELETE=DELETE][,FILENAME=FILENAME]
;               [,queue=queue][,color=color]
; INPUTS:
;        none
; OPTIONAL INPUT PARAMETERS:
;        version = An integer version number will cause the plotting of that
;        particular idl.ps file.
; KEYWORD PARAMETERS:
;        DELETE - delete the plot file after printing
;	 FILENAME - Other than the default, idl.ps
;        QUEUE    - Default queue is NET$PRINT
;        COLOR    - send to color printer
; SIDE EFFECTS:
;	 The plot file is queued on the Talaris T1590-T printstation with
;        /SETUP = PSI, unless color is specified
; RESTRICTIONS:
;        There must be a file idl.ps on the default directory.
; PROCEDURE:
;	 A command line is spawned.        
; MODIFICATION HISTORY:
;	 D.M. fecit, 25 July, 1985.
;        Modified for system print queue, 8 December, 1987.
;        Modified for IDL V2 and PostScript, 2 May, 1990.
;	 Modified for optional filenames, 9 Nov., 1991 by RAS
;        DMZ (ARC) July'92 -- Made UNIX compatible, and introduced /DELETE keyword
;	25 Feb 93: Andrew L. Stanger   (HAO/NCAR)
;	Changed color printer to 'haocolor'.
;	Removed host_name call.  All B/W hardcopy goes to default printer.
;	
;- =========================================================================== -

PRO PSPLOT, version, delete=delete, filename=file,  queue=queue, color=color

   if (n_elements (file) eq 0) then file = 'idl.ps'

   if (!version.os eq 'vms') then vms = 1 else vms = 0
   if (vms and (n_elements (version) ne 0)) then $
      file = file + ';' + strtrim (string (version, '(i2)'), 2)

   ;-- File present? 

   f = findfile (file, count=count)
   if (count eq 0) then begin
      message, 'cannot locate ' + file, /cont
      return
   endif else file = f (0)

   ;-- File open?

   if (!d.unit ne 0) then begin
      status = fstat (!d.unit)
      if (file eq status.name) then device, /close
   endif

   ;-- Delete after print.

   if (vms and keyword_set (delete)) then delete = ' /delete' else delete = ''

   ;-- Different keywords for different cases.

   if (n_elements (queue) eq 0) then begin

      if (keyword_set (color)) then begin		; Color postscript.
         if (vms) then begin				; VMS
            printcom = 'print' & queue = ' /queue=solar$color' & qualifier = ''
         endif else begin				; UNIX
            printcom = 'lpr' & queue = '' & qualifier = ' -Phaocolor'
         endelse
      endif $

      else begin					; B/W postscript.
         if (vms) then begin				; VMS
            printcom = 'print' & queue = ' /queue=net$print' & $
		        qualifier = ' /setup=psi'
         endif $
	 else begin					; UNIX
	 printcom = 'lpr'
;25feb93            host_name, lnode
;25feb93            if (strupcase (lnode) eq 'UMBRA') then $
;25feb93	    printcom = 'print' else printcom = 'lpr'
            queue = '' & qualifier = ''
         endelse
      endelse

   endif else begin
      printcom = 'print' & queue = ' /queue= '+queue & qualifier = ' /setup=psi'
   endelse

   ;-- Spawn print.

   com_line = printcom + ' ' + qualifier + ' ' + file + queue + delete
   message, com_line, /contin   &   spawn, com_line

   ;-- Spawn delete.

   if (keyword_set (delete) and (not vms)) then begin
       delcom = 'rm ' + file
      message, delcom, /info 
      spawn, delcom
   endif

   return
end

;+ =========================================================================== +
