/*  Returns the date and time for a "mk4InsDesc" julian day.

    This code was written by Alice Lecinski, November 2001.
    I've used some code written by Jay Chalmers (jwc) and
    some written by Barry Knapp.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>
#include <ctype.h>

int mon, day, doy, year;
double   juldref;


char use[] = "\
 Returns the date and time for a \"mk4InsDesc\" julian day \n\
 mk4IDjd2datetime mk4IDjuld.decimal	\n\
 mk4IDjd2datetime 36109.7868   \n\
 \n";

main (an,av) register int an; register char *av[]; {

    long   myjuld;
    long   myfrac;
    double true_juld;
    double fracday;
    double decpl;
    char   ccstr[40];
    char   fracpart[20], daypart[20];
    char   *pv, *ps;
    int    hr,minute,sec;

    decpl   = 1.;
    juldref = 2415021.;  /* Jan 1, 1900 */


    if (an == 2) { 
	sprintf(ccstr,"%s",av[1]);
	pv = ccstr;
	ps = daypart;
	while( *pv != '.')  *ps++ = *pv++;
	*ps = '\0';
	pv++;
	ps = fracpart;
	while( *pv != '\0') { *ps++ = *pv++;   decpl *= 10; }
	*ps = '\0';

	myjuld  = atoi(daypart);
	myfrac  = atoi(fracpart);
	fracday = (double)myfrac / decpl;

	true_juld = (double)myjuld +fracday + juldref - 0.5;
	knappjdate(true_juld,&doy,&year);
	mon_day(doy,year,&mon,&day);

	hr     =  (int)(fracday * 24.) ;
	minute =  (int)(fracday * 24.*60. +0.500005)     - hr*60;

	fprintf(stderr,"For mk4InsDesc.h JuldRange value %s\n",ccstr);
	fprintf(stderr,"\t month  %2d day %2d year %4d \n", mon,day,year); 
	fprintf(stderr,"\t doy    %3d \n", doy); 
	fprintf(stderr,"\t hour   %2d minute %2d \n", hr,minute); 
	fprintf(stderr,"\t TrueJulianDay %.4f\n", true_juld ); 
	}

    else printf (use);

}




/* idoy and mon_day are jwc's */

static int day_tab[2][13] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
                              0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
			    } ;

idoy(mon, day, year) int mon, day, year ;
{
    int ii , leap ;

    leap = year%4 == 0 && year%100 != 0 || year%400 == 0 ;

    for ( ii=1 ; ii<mon ; ii++ ) 
	day += day_tab[leap][ii] ;
    return(day) ;
}


mon_day(doy, year , pmon , pday ) int doy , year , *pmon , *pday ;
{
    int ii , leap ;

    leap = year%4 == 0 && year%100 != 0 || year%400 == 0 ;

    for ( ii=1 ; doy > day_tab[leap][ii]  ; ii++ ) 
	doy -= day_tab[leap][ii] ;
    *pmon = ii ;
    *pday = doy ;
    return(0) ;
}

/*  The below is basically Barry Knapp's meeus library jd2yd.pro
;
; Given Julian date (Julian Day Number and fraction), returns
; Gregorian calendar date of the form yyyyddd.dd.  (Year "1 B.C."
; is Gregorian year 0, "2 B.C." is Gregorian year -1, etc.)
;
; B. Knapp, 95.09.25
;
; Note: This function should not be used to obtain dates for historical
; events prior to the adoption of the Gregorian calendar (1582 to 1918,
; depending on the country); instead, for such dates a JD #-to-*Julian*-
; calendar-date service should be used.
;
*/

int
knappjdate (jday, doy, year) double jday; int *doy;  int *year; {

    double aa,bb,jj,dd,mm;
    double jd;
    int    nn,yy;

    jd = (double)jday;

    /*  ; Add one million years (to handle negative years) and subtract JD
	; number for day 1.0, year 0, Gregorian proleptic calendar
    */
    jj = jd + 365242500.0 - 1721059.5 ;

    /*  ; What portion of a 400-year cycle do we have?
    */
    aa = (double)((int)jj % 146097) ;

    /*  ; How many (non-leap year) centuries have elapsed in this cycle?
    */
    if ( aa < 36890.0 ) {
	 mm = 0. ;
    }
    else {
	mm = (double)((int)( (aa-366.0)/36524.25 )) ;
    }

    /*  ; How many days in the current 4-year cycle have elapsed?
    */
    bb = (double)((int)(aa+mm) % 1461 )  ;

    /* ; Determine the number of years elapsed in the 4-year cycle, and the
       ; number of days in the year
    */
    if ( bb < 366.0 ) {
	nn = 0;
	dd = bb+1. ;
    }
    else {
	nn = (int)( (bb-1.)/365.0 ) ;
	dd = bb-nn*365.0 ;
    }

    /*  ; Compute the year
	; Take back the one-million-year offset
    */
    yy = (int)( jj/146097.0 )*400 + (int)( (aa+mm)/1461.0 )*4 
	+ nn - 1000000 ;

    /*
    fprintf(stderr,"yy  = %d\n",yy);
    fprintf(stderr,"dd  = %f\n",dd);
    */

    *doy   = (int)dd ;
    *year  = (int)yy ;

}
