/*  The mk4 header after rawt4v2 

    Written by Alice Lecinski in fall of 2001.

    I strongly suggest *not* editting this file at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    file.  So if you do edit it, insure results pre and post
    actually match.


*/

struct   TimeDate   { /* 36 bytes == 4*9 */
    int    Year;
    int    Month;
    int    Day;
    int    Hour;
    int    Minute;
    int    CentiSec;  
    int    Juld;     /* Julian day - JuldRef */
    int    Doy;
    int    Time;     /* Time in Hz */
};

struct   InstrDescrip {  /* 256 bytes */

    double JuldRange[2]; /* These are the first and last dates
			    (inclusive) that the following instrument
			    description and parameters are valid.

			    The JuldRange is a bit non-standard.
			    A standard Julian day begins at noon.
			    But here, I (Alice Lecinski) begin the
			    Julian day at midnight.
			    I also use an offset of 2415021 (Jan 1, 1900)

			    So, Standard_JulianDay = JuldRange + 2415021 - 0.5

			    To convert JuldRange numbers to
			    a date and time, use the program:
			        mk4IDjd2datetime
			    e.g.
			        mk4IDjd2datetime 36109.7868
			 */
    float  Bopal;        /* The transmission of the opal*1.e-6

			    There is a new value for Bopal
			    based on Elmore, Nov 2001.
			    For 750 nm wavelength,
			    the Nov 2001 value for Bopal is:
				13.8e-6
			    The previous value was:
				10.11e-6
			 */
    float  trt;          /* The transmission of the tiltplate.
			 */
    float  fpt;          /* The fractional polarization of the tiltplate.
                            New as of 1 Sept 2001.
			    New value     fpt = 0.01641
                            Old value was fpt = 0.01543
                            The old was computed for just
                            a single ray of light coming down
                            the center line and wavelength of 820nm.
                            The new accounts for all rays
                            and appropriate wavelength of 775nm.
			 */
    float  IDspare;

    float  kappa;        /* Non-linear (OldDalsa) camera constant.
			    Here is how it is used:
				fred = kappa*(imi^(kappa-1))
				imq=imq*fred
				imu=imu*fred
				imi=imi^kappa
			    For specifics, see routine DoKappa in
			    fixmk4v2.c
			 */
    float  BarrelOffset; /* The InstrPhs angle, calculated from
			    the tilt plate data, corrects for the
			    majority angular
			    errors in the barrel angle data.
			    The BarrelOffset is the *residual* correction
			    needed to align mk4 with Lasco C2.
			    David found this to be 2.85 degrees
			    when eclipse day (1998) was compared to C2.
			    Each time the "top-hat" is opened, this 
			    activity may change "BarrelOffset".
			    There are 4 such known dates. 98d302,
			    99d113, 99d271/99d274, 01d118.
			    This parameter may need to be updated
			    as needed in the mk4InsDesc.h file
			    as more x-comparisions between mk4
			    and C2 are done.
                         */

    float  hairspot [2]; /* The hairspots are taken from a gaussian fit to
                            the hairlines in a regular image
                            closest to when a mask test and mask images
                            were gathered.
                            Which mask data and hence which hairspots
                            to use are date dependent.
			    These are the "standard" hairspots for
			    a given date.
                         */

    float  r0;         
    float  dr;           /* r0 and dr are a least squares fit between the
                            measured pixel positions of the slots in the
                            mask images, and the physical positions of the
                            slots.
                            The pixel positions are measured with
                            a gaussian fit.
                            The heights for the MASK
                            slots are: 
                                [1.224,1.804,2.287]
                            all in Rsun, assuming a Rsun
			    of 960 arc-seconds. 
                         */

    float  goodRmin;
    float  goodRmax;     /* goodRmin and goodRmax are the
			    lowest and highest
                            Rsun heights (inclusive) where the data
                            is "good".
                          */

    float  biasspot [2]; /* These are the pixel positions of
                            lowest and highest pixels (inclusive)
                            between which the bias is computed.
                         */
    float  ZeroSky;      /* Largest possible Sky value.  This would
			    be the value from this instrument if there
			    were no atmosphere.
			    Prior to May 1, 2001, the camera refurbishment,
			    the largest this value could be 359.7 or 376
			    depending on how the instrument might have
			    been behaving on this day.
			    The SkyTx instrument did not give reliable
			    values until May 14, 2001.
			    Thus, the SkyTx for this early data is calculated
			    from the AirMass. And, since the slope and
			    y-intercept were calculated from the
			    post May 14, 2001 data, the pre May 14
			    data actually has the same ZeroSky as the
			    post May 14, 2001 data, 376.  
			    The value is recorded in the header
			    to hopefully avoid any confusion.
		 	 */
    float  VenusCalFac;  /* This is a multiplicative factor that was
			    required to bring Venus up to its
			    expected brightness in the 9 and 12 June
			    2000 data.
			 */
    float  ExtraCalFac;  /* This is just a place holder for any 
			    other factor that might be needed.
			 */
    float  DiffractFac;  /* This is the factor to attempt to correct
			    for polarized diffraction around occulter.
			 */
    int    DiffractPix;  /* Highest pixel (inclusive) to correct
			    for polarized diffraction around occulter.
			 */
    int    PhiPix;       /* The pixel to check for a phase error 
			    in the cal15 q and u data. 
			    "We know the tilt plates are bolted to the
			    spar at 0 deg geocentric. (So) Offset in
			    phase is error in barrel angle."
			 */
    int    PolarityCkPix;  /* Pixel offset from lowest hairline
			      to check for reverse polarity
			      (swapped sign) in the q and u data.
			 */
    int    NegPix[2];    /* Lowest and highest pixels (inclusive)
			    to apply a single 65536 additive fix to
			    negative "i" data. (Single negative wrap).
			 */
    int    WrapPix[2];   /* Lowest and highest pixels (inclusive)
			    to apply and check for multiple negative
			    wraps in "i" "q" and "u" data.
			    WrapPix[1] == NegIPix[0] - 1
			 */

    int   RingSearch[2];  /* Pixels where to start and end the search
			     for the Ring center */

    int   RingFixPix;     /* Pixels heights of RingFixPix and 
			     above are fixed for rings.
                             "RingFixPix-1" is used as a reference
			     height in fixing the rings.  
			  */




    char   CorrectFile0  [64];  
    char   CorrectFile1  [64]; /* CorrectFile0 and CorrectFile1
				  are the Gain0 and Gain1
                                  correct files that are appropriate
                                  for this data.  Again, this is date
				  dependent.
                                */
};


struct   mk4_header {
    struct TimeDate    Image;
    int    Xdim;
    int    Ydim;
    int    Duration;   /* Duration of entire scan, in Hz. */
    int    JuldRef;    /* 2415021 , Jan 1 1900 */ 
    int    ImgDataType; /*  16 => 16 bit integers  (shorts)
			   -32 => floating point
			*/
    int    Fill1[2];
    /*16 ints */
    float  Pangle;     /* P angle in degrees */
    float  Bangle;     /* B angle in degrees */
    float  Langle;     /* L angle in degrees */
    float  RA;         /* Solar Right Ascension */
    float  Dec;        /* Solar Declination */
    float  HA;         /* Hour angle in degrees */
    float  Elevation;  /* Solar elevation in degrees */
    float  Azimuth;    /* Solar azimuth in degrees */
    float  Amass;      /* Airmass */
    float  Radius;     /* Solar radius in arc-seconds. */
    float  SkyTx;      /* Max. reasonable sky transmission from "sk" file. */
    float  InstrPhs;   /* Instrument coordinate frame phase offset. (degrees)
                          This angular offset gets applied to all data.
                          It is calculated in getcal4v2 from the tiltplate
                          data at pixel 192 (ID.PhiPix).
			  This corrects for the the majority of the
			  angular offset between C2 and mk4 (this
			  varies up to 10 degrees).
			  To quote from David's 20 August 2001
			  mk4 IDL calibration memo:
			     "The polarimeter response matrix computation
			      now corrects for barrel offset error
			      comapred to the calibration tilt
			      plates.  This should fix the many
			      degree errors before May, 2001.
			      Still there is a small phase
			      offset between this value and the
			      corona as seen in C2.  About 2.35
			      degrees of correction is needed
			      both before and after May, 2001."
			 David said that since the tilt plates
			 were known to be bolted in a known and
			 certain configuration relative to
			 geo-centric north,  one can use this
			 information and data gleaned from the 
			 tilt plate data at pixel 192 to
			 automatically compute the difference in
			 the angular orientation of the mk4
			 and true north.

			 If you have questions, please see
			 David Elmore.
                       */

    float  spare1;
    float  data_scale;
    float  data_offset;
    float  spare2;
    float  orig_hairspot[2];
    float  hairspot     [2];

    float  Xcen;       /* Center  of the image if in rect. coordinates. */
    float  Ycen;       /* Center  of the image if in rect. coordinates. */
    float  pixscl;     /* Scaling of the image if in rect. coordinates. */
    float  r0;         /* In present image. */
    float  dr;         /* In present image. */
    float  rmax;       /* In present image. */
    float  theta_min;  /* In present image. */
    float  theta_max;  /* In present image. */
    float  raw_theta_min;  /* In original raw data image. Uncorrected for */
    float  raw_theta_max;  /*             direction and Pangle and phase. */
    float  backlash;   /* Backlash correction applied, in degrees. */
    float  Phase   ;   /* Phase offset, in degrees. */
    /*16+32=48*/

    int    ObsType;    /* IS_cal0, IS_cal15, IS_dark, IS_coronal */
    int    DataType;   /* IS_i , IS_q , IS_u                        */
    int    ProcSteps;  /* Process steps, IS_raw through IS_cal_avg_rect_slit */
    int    Direction;  /* "direction" of original raw data 
                          -1==reverse, +1 forward.*/
    int    Nray;       /* Number of barrel angle positions (rays)
                          in original raw data. */
    int    Pixels;     /* Number of pixels in original raw data. */
    int    Fill2[1];

    struct TimeDate Date_calibrated;

    /*16+32+16=64  64*4=256*/

    char            tscpfile [64];   /* filename of tscp file. */
    char            reffile  [64];   /* filename of reference file. */
    char            version  [64];   /* Version number of the code. */

    char            AvgTimes[160];   /* If avg img, these are the img times
                                        used to make the avg. */
    /* 256+3*64+160 = 608 */

    struct InstrDescrip ID;          /* 256 bytes, 608+256=864 */

    char            space[1024-864]; /* Even up to 1024. */
};


typedef struct mk4_header * (FH);


#ifdef START
    int   naz      = NAZ ;
    float ovNAZ    = 1./NAZ ;
    float az2deg   = 360./NAZ ;
    double AngSep  = 360./NAZ ;
    double ovAngSep= NAZ/360. ;
    struct mk4_header mk4head;
    struct mk4_header *mk4h = &mk4head;
#else
    extern int      naz     ;
    extern float    ovNAZ   ;
    extern float    az2dec  ;
    extern double   AngSep  ;
    extern double ovAngSep  ;
    extern struct mk4_header  mk4head , *mk4h ;
#endif
