;
; This IDL code will read in a MK3 and MK4 pB image and plot a radial and
;	azimuthal scan for each image.
; Here is how to use it:
;	1. Copy the mk3 and mk3 pB images into a directory you can
;	   write to.
;	2. Copy the corresponding barrel angle files over (*.ba).
;	3. Enter the proper filenames below.
;	4. Enter a height and desired angle to make comparisons from.
;	5. Run the program in the directory where files are located.
;
;
PRO mk4scans

;
; ---------- Here are the user changable parameters ----------  
;
mk4_bafilenm='99d183.21_59.mk4.ba'
mk3_bafilenm='99d183.21_59.ba'
mk4_filenm='99d183.21_59.mk4.pB'
mk3_filenm='99d183.21_59.ch0.pB
height=1.35   	;height to take sample from each image (solar_radii)
angle = 10	;angle to take sample from each image (degrees)

;image scaling
mk3_mn=-100
mk3_mx=2000
mk4_mn=-1000
mk4_mx=4700

;
;---------- End of user changable parameters ----------  
;


;--- These arrays were taken from Alice's seemk4.pro code
headib=0 & headib=lonarr(16) ;contains date/time, xdim and ydim
headf =0 & headf =fltarr(24) ;contains ephemeris info, pangle, etc.
headie=0 & headie=lonarr(16) ;contains image type (iqu,st,ba) and related info
head =0 & head =bytarr(1024)  ;looks like a spacer to get readu to right spot

;
; height = r0+pixel*dr
;
mk4_r0=1.0018
mk4_dr=0.005035

mk3_r0=1.1224
mk3_dr=0.01042

ba2deg=0.021973 ;; 360./16384. ;; 45.51111111 ba to 1 degree
                               ;; 3640.888888 = ba of 80.

;
;---------- Read in mk4 barrel angle image ----------   
;
  close,1 & openr,1,mk4_bafilenm & point_lun,-1,top & readu,1,headib
                                                readu,1,headf
                                                readu,1,headie
                             point_lun, 1,top & readu,1,head
  bxsz=headib(9)
  bysz=headib(10)
  bhr=headib(3)
  bmn=headib(4)
 
  mk4_bimg=0 & mk4_bimg=INTARR(bxsz,bysz)
  readu,1,mk4_bimg & close,1
  tmp=long(mk4_bimg(0,*))
  mk4_angles=tmp*ba2deg

;
;---------- Read in MK4 image  ----------  
;

  close,1 & openr,1,mk4_filenm & point_lun,-1,top & readu,1,headib
                                                readu,1,headf
                                                readu,1,headie
                             point_lun, 1,top & readu,1,head
  mk4_xsz=headib(9)
  mk4_ysz=headib(10)
  hr=headib(3)
  mn=headib(4)

mk4_img=INTARR(mk4_xsz,mk4_ysz)
readu,1,mk4_img & close,1

mk4_xsample=FIX((height-mk4_r0)/mk4_dr)
;mk4_ysample=450
mk4_ysample=WHERE(FIX(mk4_angles) EQ angle)

; --- make an array of heights using mk4 scaling
index=indgen(mk4_xsz)
mk4_height_arr=mk4_r0+((index+1)*mk4_dr)
mk4_inten1=mk4_img(*,mk4_ysample[0])
mk4_inten2=mk4_img(mk4_xsample,*)

;
;---------- Read in MK3 barrel angle image ----------  
;
bxsz=720
mk3_bimg=intarr(bxsz)
openr,1,mk3_bafilenm & readu,1,mk3_bimg & close,1
mk3_angles=mk3_bimg*ba2deg
mk3_angles=mk3_angles-30

;
;---------- Read in MK3 image  ---------- 
;
mk3_xsz=128
mk3_ysz=720
mk3_img=intarr(mk3_xsz,mk3_ysz)
close,1 & openr,1,mk3_filenm & readu,1,mk3_img & close,1
mk3_xsample=FIX((height-mk3_r0)/mk3_dr)
;mk3_ysample=300
mk3_ysample=WHERE(FIX(mk3_angles) EQ angle)
index=indgen(mk3_xsz)
mk3_height_arr=mk3_r0+((index+1)*mk3_dr)
mk3_inten1=mk3_img(*,mk3_ysample[0])
mk3_inten2=mk3_img(mk3_xsample,*)

;
;---------- Make plots ----------  
;
set_plot,'PS'
device,filename='coronalscans.ps',/inches,xsize=8,ysize=10,xoffset=0.25, $
	yoffset=0.5
!p.multi=[0,1,2]
!p.subtitle='Height='+strcompress(string(height))+' Rsun'

;  Azimuth plots
plot,mk4_angles,mk4_inten2,title=mk4_filenm, $
	xtitle='Angle (degrees)',ytitle='Intensity (pB units)'
oplot,mk3_angles,mk3_inten2,psym=1  ;linestyle=1
!p.subtitle=''

; Radial plots
plot,mk4_height_arr,mk4_inten1,title=mk3_filenm, $
	xtitle='Height (Solar radius)', ytitle='Intensity (pB Units)'
oplot,mk3_height_arr,mk3_inten1,linestyle =1
device,/close

;
;---------- Insert good colortable ---------- 
;
_a=bytarr(256)  &  _b=bytarr(256)  &  _c=bytarr(256)
openr,11,'/home/cordyn/color_idl/qual.rgb'
readu,11,_a  &  readu,11,_b  &  readu,11,_c  &  close,11
tvlct,_a,_b,_c

;
;---------- Make gif images and save them ----------  
;
set_plot,'X'
mk4_img(mk4_xsample,*)=max(mk4_img)
mk4_img(*,mk4_ysample)=max(mk4_img)

mk3_img(mk3_xsample,*)=max(mk3_img)
mk3_img(*,mk3_ysample)=max(mk3_img)

window,2,xsize=mk4_xsz,ysize=mk4_ysz
window,3,xsize=mk3_xsz,ysize=mk3_ysz
wset,2
tvlct,_a,_b,_c
tv,bytscl(mk4_img,min=mk4_mn,max=mk4_max)
xyouts,100,100,mk4_filenm,size=2,/device
write_gif,mk4_filenm+'.gif',tvrd(),_a,_b,_c

wset,3
tvlct,_a,_b,_c
tv,bytscl(mk3_img,min=mk3_mn,max=mk3_mx)
xyouts,.0,.85,mk3_filenm,size=2,/NORMAL
write_gif,mk3_filenm+'.gif',tvrd(),_a,_b,_c

END
