/* fts2jpg.c
 * -----------------------------------------------------------------------------
 * Convert FITS image(s) into JPEG format.
 * -----------------------------------------------------------------------------
 * Requires cfitsio and ImageMagick libraries.
 * -----------------------------------------------------------------------------
 * Tony Darnell   HAO/NCAR   5 Mar 2002
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
/*#include <getopt.h> */
#include <sys/types.h>

#include "fitsio.h"
#include "magick/api.h"

int scaleimg  (int *iimg, int xdim, int ydim, int imin, int imax, int top);
int scalechip (int *iimg, int xdim, int ydim, int imin, int imax, int top,
               float crradius, float xcen, float ycen);

int main (int argc, char *argv [])
{

  FILE     *fptr;
  fitsfile *fileptr;
  int      *iimg;

  static char filename  [80];
  static char outname   [80];
  static char tmparr    [80];
  static char origin    [20], instrument [20] ;
  static char dateobs   [20], timeobs    [20], typeobs [20] ;
  static char object    [20], typedata   [20];
  static char colorpath [24] = "/home/cordyn/color/";
  static char *geometry;

  static int index [255], red [255], green [255], blue [255], ind, r, g, b;

  int status = 0, mystat;
  int i, iy, ix, ky, fpixel=1, nelem, xdim, ydim;
  int indx, c, loflag, hiflag, topflag, colorflag, cropflag=0, scalelimb = 1;
  int newcalflag = 0;
  int xpos, ypos, newxdim, newydim;
  int nullval, anynull = -1;
  int scalemin, scalemax, top;

  int year, mon, day, ii, leap, doy = 0;
  int day_tab [2][13] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
	                  0, 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

  float crradius, xcen, ycen;

  unsigned char *bimg;
  unsigned char *cImg;
  unsigned char *rgbimg;

  /* ImageMagick library structures */

  Image *jpegimage;
  ImageInfo *image_info;
  DrawInfo *info;
  ExceptionInfo exception;
  PixelPacket textcolor;

  /*** Parse command line ***/

  loflag = hiflag = topflag = colorflag = 1;
  top    = 249;
  opterr =   0;

  while ((c = getopt (argc, argv, "l:h:t:c:s:n")) != -1)
    switch (c)
      {
      case 'l':
        scalemin = atoi (optarg);
	loflag = 0;
        break;
      case 'h':
	scalemax = atoi (optarg);
	hiflag = 0;
	break;
      case 't':
	top = atoi (optarg);
	topflag = 0;
	break;
      case 'c':
        fptr = fopen (strcat (colorpath, optarg), "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
	colorflag = 0;
	break;
      case 's':
	geometry = optarg;
	cropflag = 1;
	break;
      case 'n':
	newcalflag = 1;
	break;
      default:
	abort ();
      }

  /*** File Loop. ***/

  for (indx = optind; indx < argc; indx++)
    {

    /*
     *  printf ("Start of loop\n");
     *  fflush (stdout);
     */

    /*** Open fits file and get image info. ***/

    strcpy (filename, argv [indx]);

    fits_open_file (&fileptr, filename, READONLY, &status);

    fits_read_key  (fileptr, TINT,    "NAXIS1",   &xdim,       NULL, &status);
    fits_read_key  (fileptr, TINT,    "NAXIS2",   &ydim,       NULL, &status);
    fits_read_key  (fileptr, TSTRING, "ORIGIN",   &origin,     NULL, &status);
    fits_read_key  (fileptr, TSTRING, "TELESCOP", &instrument, NULL, &status);
    fits_read_key  (fileptr, TSTRING, "DATE-OBS", &dateobs,    NULL, &status);
    fits_read_key  (fileptr, TSTRING, "TIME-OBS", &timeobs,    NULL, &status);
    fits_read_key  (fileptr, TSTRING, "TYPE-OBS", &typeobs,    NULL, &status);
    fits_read_key  (fileptr, TSTRING, "OBJECT",   &object,     NULL, &status);

    if (strstr (instrument, "CHIP"))
      {
      fits_read_key  (fileptr, TSTRING, "TYPE-DP",  &typedata, NULL, &status);
      fits_read_key  (fileptr, TFLOAT,  "CRRADIUS", &crradius, NULL, &status);
      fits_read_key  (fileptr, TFLOAT,  "CRPIX1",   &xcen,     NULL, &status);
      fits_read_key  (fileptr, TFLOAT,  "CRPIX2",   &ycen,     NULL, &status);
      }

    /* Compute the DOY from dateobs. */

    fits_str2date (dateobs, &year, &mon, &day, &status);
    leap = year%4 == 0 && year%100 != 0 || year%400 == 0 ;
    doy = 0;
    for (ii = 1; ii < mon; ii++)
      doy += day_tab [leap][ii];
    doy += day;

    /*
     *  printf ("instrument: %s\n", instrument) ;
     *  fflush (stdout) ;
     */

    nelem = xdim * ydim;
    iimg = (int *) malloc (nelem * sizeof (int));

    /* fits_read_img (fileptr, TINT, fpixel, nelem, &nullval, iimg, 
     *                &anynull,&status);
     */

    if (fits_read_img (fileptr, TINT, fpixel, nelem, &nullval, iimg, 
		       &anynull, &status))
    fits_report_error (stderr, status);

    fits_close_file (fileptr, &status);

    /* fits_report_error (stderr, status); */

    /*** Assign color table according to instrument. ***/

    if (strstr (instrument, "CHIP"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/RED_TEMPERATURE.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }
      if (strstr (typedata, "VEL"))
        {
        textcolor.red   = MaxRGB;
        textcolor.green = MaxRGB;
        textcolor.blue  = MaxRGB;
        scalelimb       = MaxRGB;
        }
      else
        {
        textcolor.red   = 0;
        textcolor.green = 0;
        textcolor.blue  = 0;
        }
      if (loflag) { scalemin = -1200;}
      if (hiflag) { scalemax = 50;}
      }
    else
    if (strstr (instrument, "MK4"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/qual.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }

      textcolor.red   = MaxRGB;
      textcolor.green = MaxRGB;
      textcolor.blue  = MaxRGB;
      if (loflag) { scalemin = -350; }
      if (hiflag) { scalemax = 1200; }
      }
    else
    if (strstr (instrument, "MK3"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/qual.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
	  {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }

      textcolor.red   = MaxRGB;
      textcolor.green = MaxRGB;
      textcolor.blue  = MaxRGB;
      if (loflag) { scalemin = -1100; }
      if (hiflag) { scalemax = 4000; }
      }
    else
    if (strstr (instrument, "DPM"))
      {
      if (colorflag)
        {
        fptr = fopen ("/home/cordyn/color/RED_TEMPERATURE.lut", "r");
        i = 0;
        while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
          {
          red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
          i++;
          }
        fclose (fptr);
        }

      textcolor.red   = MaxRGB;
      textcolor.green = MaxRGB;
      textcolor.blue  = MaxRGB;

      if (loflag)
        {
        if (strstr (object, "SUN"))        { scalemin = 1; } 
        else if (strstr (object, "LIMB"))  { scalemin = 1; }
        }
      if (hiflag)
        {
        if (strstr (object,"SUN"))	{ scalemax = 600; } 
        else if (strstr (object,"LIMB"))	{ scalemax = 200; }
        }
      }

    /*** Perform image scaling and create image buffers. ***/

    if ((strstr (instrument, "CHIP") && scalelimb) && (newcalflag))
      scalechip (iimg, xdim, ydim, scalemin, scalemax, top,
	         crradius, xcen, ycen);
    else 
      scaleimg (iimg, xdim, ydim, scalemin, scalemax, top);

    bimg   = (unsigned char *) malloc (nelem * sizeof (unsigned char)    );
    cimg   = (unsigned char *) malloc (nelem * sizeof (unsigned char)    );
    rgbimg = (unsigned char *) malloc (nelem * sizeof (unsigned char) * 3);

    /*** Convert int array to byte array. ***/

    for (i = 0; i < nelem; i++)
      { 
      if (iimg [i] < 0)
	cimg [i] = 0;
      else
      if (iimg [i] > 255)
	cimg [i] = top;
      else
        cimg [i] = (unsigned char) iimg [i];
      }

    /*** Invert image top to bottom. ***/

    for (iy = 0; iy < ydim; iy++)
      {
      ky = ydim - iy - 1 ;
      for (ix = 0; ix < xdim; ix++)
        *(bimg + ix + (iy*xdim)) = *(cimg + ix + (ky*xdim));
      }

    /*** RGB interlacing. ***/

    for (i = 0; i < nelem; i++)
      {
      rgbimg [i*3    ] = (unsigned char) red   [bimg [i]];
      rgbimg [i*3 + 1] = (unsigned char) green [bimg [i]];
      rgbimg [i*3 + 2] = (unsigned char) blue  [bimg [i]];
      }

    /*** Create ImageMagick image, annotate, apply filters and resize. ***/

    GetExceptionInfo (&exception);
    image_info = CloneImageInfo  ((ImageInfo *) NULL);
    jpegimage  = ConstituteImage (xdim, ydim, "RGB", CharPixel, rgbimg,
	                          &exception);
    info       = CloneDrawInfo   (image_info, (DrawInfo *) NULL);

    if (!(cropflag))
      {
      if (xdim == ydim)
        {
        newxdim = 512; newydim = 512;
        jpegimage = ScaleImage (jpegimage, 512, 512, &exception);
        xpos = 256; ypos = 256;
        }
      else
        {
        newxdim = xdim / 2;
        newydim = ydim / 2;

        jpegimage = MinifyImage (jpegimage, &exception);

        xpos = newxdim / 2;
        ypos = newydim / 2;
        }
      }
    else
      {
      printf ("Cropping image...\n"); 
      newxdim = 512; newydim = 512;
      xpos = 256; ypos = 256;
      TransformImage (&jpegimage, geometry, "512x512");
      }

/*
    ContrastImage (jpegimage, 1);
    NormalizeImage (jpegimage);
    jpegimage = EnhanceImage (jpegimage, &exception);
    info.stroke = textcolor;
    GetDrawInfo (image_info, &info);
    info.fill = textcolor;
    info.box = textcolor;
    info.text_antialias = 1;
    info.pointsize = 18;
    info.border_color = textcolor;
    info.box = textcolor;
    info.primitive = tmparr;
    info.primitive = tmparr;
    info.primitive = tmparr;
    status = DrawImage (jpegimage, info);
    mystat = AnnotateImage (jpegimage, info);
    printf ("%d\n", mystat);
    info->pointsize = 18;
    info->font = "-*-clean-medium-r-*-*-16-*-*-*-*-*-*";
    info->font = "-*-fixed-medium-r-*-*-18-*-*-*-*-iso8859-1";
    info->font = "-*-courier-medium-r-*-*-17-*-*-*-*-*-iso8859-1";
 */

    /*** Annotate image. ***/

    info->fill.opacity = 255;
    info->fill = textcolor;

    sprintf (tmparr, "text 3,20 '%s %s' text 3,40 '%s' text 3,60 '%s' text 3,100 'DOY: %d' ",
             origin, instrument, typeobs, dateobs, doy);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimage, info); 

    sprintf (tmparr, "text 3, 80 '%s UT' text %d, 22 'N' text %d, %d 'W'", 
             timeobs, xpos, newxdim-24, ypos);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimage, info);

    sprintf (tmparr, "text %d,%d 'S' text 12,%d 'E'",
             xpos, newydim-12, ypos);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimage, info);

    sprintf (tmparr, "text 3,%d 'Scaling: %d to %d'",
	     newydim-10, scalemin, scalemax);
    info->primitive = tmparr;
    mystat = DrawImage (jpegimage, info);

    /*** Save image as desired filetype. ***/

    strcpy (outname, filename);
    strcpy (strstr (outname, "fts"), "jpg");
    printf ("%s\n", outname);
    (void) strcpy (jpegimage->filename, outname);

    WriteImage (image_info, jpegimage);

/*
 *  printf ("End of loop\n");
 *  fflush (stdout);
 */

    free (bimg);
    free (cimg);
    free (rgbimg);
    free (iimg);
    }  /* End File Loop */
}  /* End Main */


/* -------------------------------------------------------------------------- */

/*
 * scaleimg applies a linear scaling to the pixel data
 * from supplied data min, max values as well as a top.
 * Works just like IDL's bytscl routine.
 */

int scaleimg (int *iimg, int xdim, int ydim, int imin, int imax, int top)

{
  int i; 
  int nelem;
  int omin   = 0
  int omax   = top;
  int offset = imin;

  float bscale;
  float newpixel;

  /*
   *  printf ("scaleimg> xdim:%4d  ydim:%4d  imin:%5d  imax:%5d  top:%5d\n",
   *          xdim, ydim, imin, imax, top) ;
   */

  /*** Image size. ***/

  nelem = xdim * ydim;

  /*** Perform intensity scaling. ***/

  bscale = (float) (omax - omin) / (float) (imax - imin);
  for (i = 0; i < nelem; i++)
      {
      if (*(iimg + i) <= -30000)
	  {
          *(iimg + i) = 1;
          }
      else if (*(iimg + i) == -3000)
	  {
          *(iimg + i) = 1;
          }
      else if (*(iimg + i) == 2000)
	  {
          *(iimg + (i-1)) = 250;
          *(iimg +  i   ) = 250;
          *(iimg + (i+1)) = 250;
          }
      else
          {
          newpixel = (float) ( *(iimg + i) - offset ) * bscale;
          if (newpixel > omax) { newpixel = omax;}
          if (newpixel < omin) { newpixel = omin + 1;}
          *(iimg + i) = (int) newpixel;
          }
      }

  return (0);
}


/* -------------------------------------------------------------------------- */

/*
 * scalechip applies different scaling to the limb than it does to the
 * disk because we see the disk in absorption and the limb in emmission.
 * The scaling values are currently hard-coded in and may need to be
 * added to the command line at some point in the future.
 *
 */

int scalechip (int *iimg, int xdim, int ydim, int imin, int imax, int top, 
	       float crradius, float xcen, float ycen)

{

  int omin = 0, omax = top;
  int offset = imin; 
  int limb_imin = 0, limb_imax = 350, limb_offset = limb_imin;
  int i, ix, iy; 
  float xdist, ydist, radius, bscale, limb_bscale, newpixel;

  bscale =      (float) (imax - imin) / (float) (omax - omin);
  limb_bscale = (float) (limb_imax - limb_imin) / (float) (omax - omin);

  for (iy = 0; iy < ydim; iy++)
    for (ix = 0; ix < xdim; ix++)
      {
      xdist = (float) ix - xcen ;
      ydist = (float) iy - ycen ;
      radius = sqrt ((xdist * xdist) + (ydist * ydist));
      if ((radius / crradius) < 1.0)
	{
	if (*(iimg + ix + iy * xdim) < imin)
	    *(iimg + ix + iy * xdim) = imin;
	if (*(iimg + ix + iy * xdim) > imax)
	    *(iimg + ix + iy * xdim) = imax;

        newpixel = (float) (*(iimg + ix + iy * xdim) - offset) / bscale;
        if (newpixel > omax) { newpixel = omax;}
        *(iimg + ix + iy * xdim) = (int) newpixel;
        }
      else
        {
	if (*(iimg + ix + iy * xdim) == -1000.0)
	    {*(iimg + ix + iy * xdim) = limb_imax;}
        newpixel = (float) (*(iimg + ix + iy * xdim) - limb_offset)/limb_bscale;
        if (newpixel > omax) { newpixel = omax;}
        *(iimg + ix + iy * xdim) = (int) newpixel;
        }

      }
  return (0);
}

/* -------------------------------------------------------------------------- */

