/*
 * Laplacian.c
 *
 * Written April 7, 2000 by Randy Meisner, HAO/NCAR, Boulder, CO
 * (meisner@ucar.edu)
 *
 * PROJECT:  SunRISE/PSPT
 *
 * PURPOSE:  To create a Laplacian filtered image.
 *
 * INPUTS:   float **im - the image to be filtered (im[1-xdim][1-ydim])
 *           long   xdim - size of the input image in the x dimension.
 *           long   ydim - size of the input image in the y dimension.
 *
 * OUTPUTS:  float **lim - the Laplacian filtered image.
 *
 * CALLS:    None.
 *
 *****************************************************************************/

#include "reduction.h"

void Laplacian(float **im, long xdim, long ydim, float **lim){

  float kern[3][3];

  long i, j;


  /* Zero Laplacian image */

  for(i = 1; i <= xdim; i++)
    for(j = 1; j <= ydim; j++)
      lim[i][j] = 0.0;


  /* Define Laplacian kernal */

  for(i = 0; i < 3; i++)
    for(j = 0; j < 3; j++)
      kern[i][j] = -1.0;

  kern[1][1] = 8.0;

  /* Convolve the Laplacian kernal with the input image */

  for(i = 2; i < xdim; i++)
    for(j = 2; j < ydim; j++){

      lim[i][j] = (im[i-1][j-1]* kern[0][0] + 
		  im[i-1][j]   * kern[0][1] +
		  im[i-1][j+1] * kern[0][2] +
		  im[i][j-1]   * kern[1][0] +
		  im[i][j]     * kern[1][1] +
		  im[i][j+1]   * kern[1][2] +
		  im[i+1][j-1] * kern[2][0] +
		  im[i+1][j]   * kern[2][1] +
		  im[i+1][j+1] * kern[2][2]);

    }

}
