/* crop_img.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Crop image.
 * The result of the function is a pointer to the output cropped image.
 *
 * usage: imgout = crop_img (img, nx, ny, nbypix, xshift, yshift, nxout, nyout)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   22 June 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

unsigned char *
crop_img (imginp, xdiminp, ydiminp, nbypix, xshift, yshift, xdimout, ydimout)

unsigned char *imginp ;		/* Input image. */
int xdiminp, ydiminp ;		/* Input image dimensions. */
int nbypix ;			/* Pixel depth of input image. */
float xshift, yshift ;		/* X/Y image shift. */
int xdimout, ydimout ;		/* Output image dimensions. */

{
    extern float blint () ;
    unsigned char *imgout ;   

    unsigned char *cibuf ;
    unsigned char *cobuf ;
    short *sibuf ;
    short *sobuf ;
    float *fibuf ;
    float *fobuf ;

    int ix, iy ;
    float fval ;
    float xx, yy ;

    imgout = (unsigned char *) malloc (xdimout * ydimout * nbypix) ;

    /*** Perform translation using bilinear interpolation. ***/

    if (nbypix == 1)
	{
	cibuf = (unsigned char *) imginp ;
	cobuf = (unsigned char *) imgout ;
        for (iy = 0 ; iy < ydimout ; iy++)
	    {
            yy = (float) iy + yshift ;
	    for (ix = 0 ; ix < xdimout ; ix++)
	        {
	        xx = (float) ix + xshift ;
	        fval = blint (cibuf, nbypix, xdiminp, ydiminp, xx, yy) ;
	        if (fval < 0)
	            *(cobuf + ix + iy*xdimout) = 0 ;
                else
	            *(cobuf + ix + iy*xdimout) = (unsigned char) fval ;
	        }
	    }
	}
    else
    if (nbypix == 2)
	{
	sibuf = (short *) imginp ;
	sobuf = (short *) imgout ;
        for (iy = 0 ; iy < ydimout ; iy++)
	    {
            yy = (float) iy + yshift ;
	    for (ix = 0 ; ix < xdimout ; ix++)
	        {
	        xx = (float) ix + xshift ;
	        fval = blint (sibuf, nbypix, xdiminp, ydiminp, xx, yy) ;
	        if (fval < 0)
	            *(sobuf + ix + iy*xdimout) = 0.0 ;
                else
	            *(sobuf + ix + iy*xdimout) = (short) fval ;
	        }
	    }
	}
    else
    if (nbypix == 4)
	{
	fibuf = (float *) imginp ;
	fobuf = (float *) imgout ;
        for (iy = 0 ; iy < ydimout ; iy++)
	    {
            yy = (float) iy + yshift ;
	    for (ix = 0 ; ix < xdimout ; ix++)
	        {
	        xx = (float) ix + xshift ;
	        fval = blint (fibuf, nbypix, xdiminp, ydiminp, xx, yy) ;
	        if (fval < 0)
	            *(fobuf + ix + iy*xdimout) = 0.0 ;
                else
	            *(fobuf + ix + iy*xdimout) = (short) fval ;
	        }
	    }
	}
    else
	{
	fprintf (stderr, "Invalid data type . . . nbypix = %d\n", nbypix) ;
	return (NULL) ;
	}

    return (imgout) ;
}
