/* rd_fits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a FITS image and determine the minimum and maximum pixel values.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   4 January 2001
 * Adapted from "cookbook.c", distributed with cfitsio library.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "/opt/share/PUBLIC/include/fitsio.h"

float *rd_fits ( char filename [], int *nx, int *ny ) ;
void printerror ( int status) ;

/*--------------------------------------------------------------------------*/
float *
rd_fits ( char filename [], int *nx, int *ny )

{
    fitsfile *fptr ;	/* Pointer to the FITS file, defined in fitsio.h */
    int status, nfound, anynull ;
    int nbyimg ;
    long naxes [2], fpixel, npixels, ii ;
    float datamin, datamax, nullval ;
    float fval ;
    float *imgbuf ;

    status = 0 ;

    /* Open fits file. */

    if ( fits_open_file (&fptr, filename, READONLY, &status) )
         printerror ( status ) ;

    /* Read the NAXIS1 and NAXIS2 keyword to get image size. */

    if ( fits_read_keys_lng (fptr, "NAXIS", 1, 2, naxes, &nfound, &status) )
         printerror ( status ) ;

    *nx = naxes [0] ;
    *ny = naxes [1] ;
    npixels  = naxes [0] * naxes [1] ;	/* number of pixels in the image */
    nbyimg   = npixels * sizeof (float) ;
    imgbuf   = malloc (nbyimg) ;
    fpixel   = 1 ;
    nullval  = 0 ;               /* Don't check for null values in the image. */
    datamin  =  1.0E30 ;
    datamax  = -1.0E30 ;
    printf ("npixels: %d\n", npixels) ;

    /* Note that even though the FITS images contains unsigned integer */
    /* pixel values (or more accurately, signed integer pixels with    */
    /* a bias of 32768),  this routine is reading the values into a    */
    /* float array.   Cfitsio automatically performs the datatype      */
    /* conversion in cases like this.                                  */

    if ( fits_read_img (fptr, TFLOAT, fpixel, npixels, &nullval,
                        imgbuf, &anynull, &status) )
         printerror ( status ) ;

    for (ii = 0 ; ii < npixels; ii++)  
      {
      fval = (float) imgbuf [ii] ;
      if ( fval < datamin )  datamin = fval ;
      if ( fval > datamax )  datamax = fval ;
      }

    printf ("\nMin and max image pixels =  %f, %f\n", datamin, datamax) ;

    if ( fits_close_file (fptr, &status) )
         printerror ( status ) ;

    return (imgbuf) ;
}

/*--------------------------------------------------------------------------*/
