/* sg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find sun center & radius of a solar disc image in FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Uses cfitsio library for FITS operations.
 * Uses Randy Meisner's f90 routines: limb_int.f90 & solargeom.f90.
 * Caution: array indices from f90 routines use [1,1] as the origin, not [0,0].
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   4 Jan 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/opt/share/PUBLIC/include/fitsio.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern float *rd_fits () ;
   static char namimg [80] ;
   float *imgbuf ;
   int xdim, ydim ;
   float minint ;
   float xrad, yrad ;
   float xcen, ycen ;

   /*** Get FITS file name from command line. ***/

   strcpy (namimg, argv [1]) ;


   /*** Read FITS image into memory.  [Use "cfitsio" library.] ***/

   imgbuf = rd_fits (namimg, &xdim, &ydim) ;

   printf ("xdim: %4d  ydim: %4d\n", xdim, ydim) ;


   /*** Compute limb intensity value for solargeom routine. ***/

   limb_int_ (imgbuf, &xdim, &ydim, &minint) ;

   printf ("minint: %f\n", minint) ;


   /*** Compute sun center & radius. ***/

   solargeom_(imgbuf, &minint, &xdim, &ydim, &xrad, &yrad, &xcen, &ycen);

   printf ("xcen/ycen: %7.2f %7.2f  xrad/yrad: %7.2f %7.2f\n",
	    xcen, ycen, xrad, yrad) ;


   exit (0) ;
}

