/*
 * sobel.c
 *
 * Written April 5, 2000 by Randy Meisner, HAO/NCAR, Boulder, CO
 * (meisner@ucar.edu)
 *
 * PROJECT:  SunRISE/PSPT
 *
 * PURPOSE:  To create a Sobel filtered image to detect edges.
 *
 * INPUTS:   im - the image to be filtered.
 *         xdim - size of the image in the x dimension.
 *         ydim - size of the image in the y dimension.
 *
 * OUTPUTS:  Replaces the input image (im) with the Sobel filtered image.
 *
 *****************************************************************************/

#include "reduction.h"

void sobel(float **im, long xdim, long ydim){

  long i, j;
  float c, cc, **sim;


  sim = matrix(1, xdim, 1, ydim);

  for(i = 1; i <= xdim; i++)
    for(j = 1; j <= ydim; j++)
      sim[i][j] = 0.0;

  for(i = 2; i < xdim; i++)
    for(j = 2; j < ydim; j++){

      cc = -1.0*im[i-1][j-1]-2.0*im[i-1][j]-im[i-1][j+1];
 
      cc = cc+(im[i+1][j-1]+2.0*im[i+1][j]+im[i+1][j+1]);
 
      c = fabs(cc);
 
      cc = -1.0*im[i-1][j-1]-2.0*im[i][j-1]-im[i+1][j-1];
 
      cc = cc+(im[i-1][j+1]+2.0*im[i][j+1]+im[i+1][j+1]);
 
      c = c+fabs(cc);
 
      sim[i][j] = c;
 
    }

  for(i = 1; i <= xdim; i++)
    for(j = 1; j <= ydim; j++)
      im[i][j] = sim[i][j];

  free_matrix(sim, 1, xdim, 1, ydim);
}
