!
! NAME:  sobel.f90
!
! PROJECT:  SunRISE/PSPT
!
! PURPOSE:  To create a Sobel filtered image to detect edges.
!
! INPUTS:   im - the image to be filtered.
!         xdim - size of the image in the x dimension.
!         ydim - size of the image in the y dimension.
!
! OUTPUTS:  Replaces the input image (im) with the Sobel filtered image.
!
! LIBRARIES:  None.
!
! HISTORY:  Written January 14, 1999 by Randy Meisner, HAO/NCAR
!           (meisner@ucar.edu).
!
! COMPILE:  f90 -c -O sobel.f90
!
! COMPILE NOTE:  April 12, 2000, Randy Meisner, HAO/NCAR
!                For some reason, compiling this routine with the -parallel
!                flag set and running it on a machine with 2+ processors
!                causes a segmentation fault.  It may have something to do
!                with the Y2K patches that were installed last fall.  Prior
!                to the installation of the patches, I had used this code in
!                another program, with the -parallel flag set, and have not
!                had any problems with that program.
!
!******************************************************************************

SUBROUTINE sobel(im, xdim, ydim)

  ! Input variables

  INTEGER, INTENT(in) :: xdim, ydim

  REAL, INTENT(inout) :: im(xdim, ydim)

  ! Local variables

  INTEGER :: i, j
  REAL    :: c, cc

  REAL, ALLOCATABLE :: sim(:,:)

  ALLOCATE(sim(xdim, ydim))

  sim = 0.0

  DO i = 2, xdim-1
     DO j = 2, ydim-1

        cc = -1.0*im(i-1, j-1)-2.0*im(i-1, j)-im(i-1, j+1)

        cc = cc+(im(i+1, j-1)+2.0*im(i+1, j)+im(i+1, j+1))

        c = ABS(cc)

        cc = -1.0*im(i-1, j-1)-2.0*im(i, j-1)-im(i+1, j-1)

        cc = cc+(im(i-1, j+1)+2.0*im(i, j+1)+im(i+1, j+1))

        c = c+ABS(cc)

        sim(i, j) = c

     END DO

  END DO

  im = sim

  DEALLOCATE(sim)

END SUBROUTINE sobel
