/* blint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * blint returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float blint ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, or floating-point images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 29 March 2000: Add nbypix parameter (instead of include statement).
 *                Add xdim & ydim parameters.
 *                Replace "npixl" with "xdim".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
blint (image, nbypix, xdim, ydim, x, y)

char	*image;		/* Pointer to image array. */
int	nbypix ;	/* # bytes per pixel. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
float	x, y;		/* Coordinates of scan point in image. */

{
register char	*im;
register short	*ims;
register float	*imf;
int	ix, iy;
float	fx, fy;
float   fxdim, fydim ;
float	f00, f10, f01, f11;
float	fx0, fx1, fxy;

   fxdim = (float) xdim ;
   if (x < 0.0 || x > fxdim)
      {
      fprintf (stderr, "Invalid X-axis coordinate: %f   xdim: %d\n", x, xdim) ;
      return (-1.0) ;
      }

   fydim = (float) ydim ;
   if (y < 0.0 || y > fydim)
      {
      fprintf (stderr, "Invalid Y-axis coordinate: %f   ydim: %d\n", y, ydim) ;
      return (-2.0) ;
      }

   im  = (char *) image;
   ims = (short *) image;
   imf = (float *) image;

   ix = (int) x;
   iy = (int) y;

   fx = x - (float) ix;
   fy = y - (float) iy;

   /*** Select 4 nearest neighbor elements in 'image' array. ***/

   /*** 1 byte per pixel. ***/

   if (nbypix == 1)
      {
      f00 = (float) (*(im +  ix    +  iy*xdim) & 0377);
      f10 = (float) (*(im + (ix+1) +  iy*xdim) & 0377);
      f01 = (float) (*(im +  ix    + (iy+1)*xdim) & 0377);
      f11 = (float) (*(im + (ix+1) + (iy+1)*xdim) & 0377);
      }
   else

   /*** 2 bytes per pixel (short integer). ***/

   if (nbypix == 2)
      {
      f00 = (float) *(ims +  ix    +  iy*xdim);
      f10 = (float) *(ims + (ix+1) +  iy*xdim);
      f01 = (float) *(ims +  ix    + (iy+1)*xdim);
      f11 = (float) *(ims + (ix+1) + (iy+1)*xdim);
      }
   else

   /*** 4 bytes per pixel (floating point). ***/

   if (nbypix == 4)
      {
      f00 = (float) *(imf +  ix    +  iy*xdim);
      f10 = (float) *(imf + (ix+1) +  iy*xdim);
      f01 = (float) *(imf +  ix    + (iy+1)*xdim);
      f11 = (float) *(imf + (ix+1) + (iy+1)*xdim);
      }

   /*** Bilinear interpolation. ***/

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}
