/* boxbrt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute the average of the pixels whose magnitude is greater than brtmin.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   24 May 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

float
boxbrt (imsq, xdimsq, ydimsq, ixb, iyb, xdimbox, ydimbox, brtmin) 

float *imsq ;			/* Image array. */
int xdimsq, ydimsq ;		/* Dimensions of array "imsq". */
int ixb, iyb ;			/* Coordinates of box center. */
int xdimbox, ydimbox ;		/* Size of box. */
float brtmin ;			/* Minimum brightness for average. */

{
   int ix, iy ;
   int ixx, iyy ;
   int ixbeg ;
   int iybeg ;
   int nval ;
   float pixval ;
   float sum ;
   float avg ;

   ixbeg = ixb - xdimbox / 2 ;
   iybeg = iyb - ydimbox / 2 ;
   nval = 0 ;
   sum  = 0.0 ;

   /*** Sum up the pixels whose intensities exceed or equal "brtmin". ***/

   for (iy = 0 ; iy < ydimbox ; iy++)
      {
      iyy = iy + iybeg ;
      for (ix = 0 ; ix < xdimbox ; ix++)
	 {
	 ixx = ix + ixbeg ;
	 pixval = *(imsq + ixx + iyy * xdimsq) ; 
/*
 *	 if (pixval != 0.0)
 *	    printf ("boxbrt> ixx/iyy: %4d %4d  pixval: %10.2f\n",
 *		    ixx, iyy, pixval) ;
 */
	 if (pixval >= brtmin)
	    {
	    sum += pixval ;
            nval += 1 ;
	    }
	 }
      }

   /*** Compute average. ***/

   if (nval > 0)
      avg = sum / (float) nval ;	
   else
      avg = 0.0 ;

   printf ("boxbrt> ixb/iyb: %4d %4d  nval: %3d  avg: %10.2f\n",
	    ixb, iyb,  nval, avg) ;

   return (avg) ;
}
