/* chip_doppler.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Determine the doppler correction for CHIP images.
 * Use "B2" filter (middle blue).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   22 May 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#define ACOS

#include "/home/cordyn/mlsofits/mechanism_defs.h"
#include "/home/cordyn/mlsofits/acos.h"
#include "/home/cordyn/mlsofits/chip.h"

#include "/home/cordyn/mlsofits/device.h"
#include "/home/cordyn/mlsofits/analog.h"
#include "/home/cordyn/mlsofits/lcvr.h"
#include "/home/cordyn/mlsofits/mic.h"

#include "/home/stanger/lib/src/acos/acoslib.h"
#include "chipscan.h"
#define NBOX 45

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern unsigned char *rotimg () ;	/* Rotate an image. */
   extern unsigned short *srotimg () ;	/* Rotate an image. */
   extern int xyplot ();        /* XY plot using NCAR system plot package. */
   extern int xysplot ();       /* XY plot using NCAR system plot package. */
   extern float boxbrt () ;		/* Average of brightest pixels in box.*/
   extern float selip () ;		/* Select brightest pixels in array. */
   extern int acoshskp () ;		/* Function to read ACOS header. */
   extern unsigned short *rdacos () ;	/* Function to read ACOS image. */
   extern float lint () ;		/* Linear interpolation. */

   static char gksout [80];
   static char labx   [40];		/* Label for X-axis. */
   static char laby   [40];		/* Label for Y-axis. */
   static char namemc [80];     	/* Metacode file name. */
   static char namimg [80] ;		/* ACOS file name. */
   static char namout [80] ;		/* ACOS file name. */
   static char namdop [80] ;		/* ACOS file name. */
   static char title  [60];		/* Graph title. */
   static char labtyp [30];		/* Data type description. */

   static float boxx [NBOX] ;
   static float boxv [NBOX] ;

   float *band ;
   float *boxcen ;
   float *imsq ;
   float *ldsq ;

   unsigned short *sbuf ;
   unsigned short *srot ;
   unsigned short *obuf ;

   unsigned char *acosbuf ;             /* MLSO ACOS file  array pointer. */
   unsigned char *acosrot ;             /* MLSO ACOS file  array pointer. */
   unsigned char *padbuf ;

   unsigned long kk, nn ;

   unsigned short sval ;

   float angle ;
   float brtmin  = 0.0 ;
   float brtcont = 0.0 ;
   float cenmin  = 0.0 ;
   float cdelt1 ;
   float degnorth = 0.0 ;
   float degrad = 57.295780 ;		/* Degrees/radian. */
   float ffix ;
   float ldcor ;
   float limbdark ;
   float ldmin, ldmax ;
   float pangle ;
   float pixrs ;
   float radius ;
   float radinc = 0.1 ;
   float radmin, radmax ;
   float rmin, rmax ;
   float rinc ;
   float favg ;
   float theta ;
   float tmin =   0.0 ;
   float tmax = 360.0 ;
   float xbeg ;
   float xcen, ycen ;
   float xcensq, ycensq ;
   float xinc ;
   float xx, yy ;

   int ib ;
   int ipixrs ;
   int ir ;
   int it ;
   int ix, iy ;
   int ixb, iyb ;
   int ixscale = 0;     /* x-axis scaling type; 0=linear. */
   int iyscale = 0;     /* y-axis scaling type; 0=linear. */
   int ibkg = 1;        /* Background flag; 0=tick marks only. */
   int ixx, iyy ;
   int ixsq, iysq ;
   int ixbeg, ixend ;
   int ixcen, iycen ;
   int ixcensq, iycensq ;
   int iybeg, iyend ;
   int navg ;
   int nbox = NBOX ;
   int nbypix ;
   int nbyimg ;
   int nang ;
   int npad ;
   int npximg ;
   int npxband ;
   int npxcen ;
   int npxbox ;
   int npxsq ;
   int ns ;
   int pmode = 1 ;
   int scntyp = 1 ;
   int xdim, ydim ;
   int xdimsq, ydimsq ;
   int xdimband, ydimband ;
   int xdimcen, ydimcen ;
   int xdimbox, ydimbox ;
   int xdif, ydif ;

   unsigned short sfix ;

   FILE *FD, *FDOUT, *FDCOR, *FDDOP ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s chip_filename\n", argv [0]) ;
      exit (-1) ;
      }

/*
   sprintf (title, "Doppler Profile %s\n", namimg) ;
   sprintf (labx,  "Equatorial Distance (Rsun)") ;
   sprintf (laby,  "Bin value (20%% brightest avg)") ;
   sprintf (namemc, "%s.cgm", namimg) ;
   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc) ;
   putenv (gksout) ;
   opngks_ () ;
 */

   /* Read header. */

   acoshskp (namimg, 0) ;

   /* Get items from header. */

   xdim = ACOSI->xdim ;
   ydim = ACOSI->ydim ;
   nbypix = ACOSI->nbypx ;
   xcen = (float) (ACOSH->Xdim - 1) * 0.5 ;
   ycen = (float) (ACOSH->Ydim - 1) * 0.5 ;
   xcen = (float) (xdim - 1) * 0.5 ;		/* Sun center = array center. */
   ycen = (float) (ydim - 1) * 0.5 ;
   ixcen = (int) xcen ;
   iycen = (int) ycen ;
   cdelt1 = 2.29 ;				/* arcsec / pixel */
   pixrs = ACOSI->rsun / cdelt1 ;		/* pixels / solar radius. */
   ipixrs = (int) pixrs ;
   pangle = ACOSI->pangle ;			/* rotation angle. */

   printf ("xdim: %d  ydim: %d\n", xdim, ydim) ;
   printf ("xcen: %7.2f ycen: %7.2f\n", xcen, ycen) ;
   printf ("pixrs: %7.2f  pangle: %7.2f\n", pixrs, pangle) ;

   /*** Read image. ***/

/*
   acosbuf = rdacos (namimg, pmode) ;
 */

   FD = fopen (namimg, "r") ;   
   if (FD == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      exit (-2) ;
      }

   /*** Allocate storage for image array. ***/

   npximg = ACOSH->Xdim * ACOSH->Ydim ;         /* # pixels in image. */
   nbypix = sizeof (unsigned short) ;           /* # bytes  in image. */
   nbyimg = npximg * nbypix ;
   sbuf = (unsigned short *) malloc (nbyimg) ;  /* Allocate memory for image. */
   printf ("nbypix: %d\n", nbypix) ;
   
   /*** Read pixel data. ***/

   fread (sbuf, nbypix, npximg, FD) ;

   fclose (FD) ;

   /*** Set first 3 lines of image to zero. ***/

   for (iy = 0; iy < 3; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(sbuf + ix + iy * xdim) = 0 ;
	 }

   /*** Rotate image to convert to Heliographic coordinates. ***/

   acosbuf = (unsigned char *) sbuf ;
/*   acosrot = rotimg (acosbuf, nbypix, xdim, ydim, ixcen, iycen, -pangle) ; */
   srot = srotimg (sbuf, xdim, ydim, ixcen, iycen, -pangle) ;
   free (sbuf) ;
   
   /*** Read limb darkening correction "image". ***/
   
   xdimsq = 936 ;
   ydimsq = 936 ;
   npxsq  = xdimsq * ydimsq ;
   xcensq = (float) (xdimsq - 1) * 0.5 ;
   ycensq = (float) (ydimsq - 1) * 0.5 ;
   ixcensq = (int) xcensq ;
   iycensq = (int) ycensq ;
   ldsq = (float *) malloc (npxsq * sizeof (float)) ;
   printf ("xdimsq/ydimsq: %4d %4d  xcensq/ycensq: %7.2f %7.2f\n",
	   xdimsq, ydimsq, xcensq, ycensq) ;

   FDCOR = fopen ("chip.ldcor.img", "r") ;
   fread  (ldsq, 4, npxsq, FDCOR) ;
   fclose (FDCOR) ;

   /*** Crop image to a square size (936x936). ***/
   /*** Multiply by limb darkening correction image. ***/

   imsq = (float *) malloc (npxsq * sizeof (float)) ;
   obuf = (unsigned short *) malloc (npxsq * sizeof (unsigned short));

   ixbeg = (xdim - xdimsq) / 2 ;
   iybeg = (ydim - ydimsq) / 2 ;

   for (iy = 0; iy < ydimsq; iy++)
      {
      iyy = iy + iybeg ;
      for (ix = 0; ix < xdimsq; ix++)
	 {
	 ixx = ix + ixbeg ;
	 *(imsq + ix + iy * xdimsq) = (float) *(srot + ixx + iyy * xdim) ;
	 *(imsq + ix + iy * xdimsq) *= *(ldsq + ix + iy * xdimsq) ;
	 }
      }
   free (srot) ;

   /*** Extract a box located at sun center. ***/

   xdimcen = 40 ;
   ydimcen = 40 ;
   npxcen  = xdimcen * ydimcen ;
   boxcen = (float *) malloc (npxcen * sizeof (float)) ;

   fprintf (stdout, "xdimcen/ydimcen: %d %d  npxcen: %d\n",
	    xdimcen, ydimcen, npxcen) ;

   ixbeg = ixcensq - xdimcen / 2 ;
   ixend = ixbeg   + xdimcen ;
   iybeg = iycensq - ydimcen / 2 ;
   iyend = iybeg   + ydimcen ;

   iyb = -1 ;

   for (iy = iybeg; iy <= iyend; iy++)
      {
      iyb += 1 ;
      ixb = -1 ;
      for (ix = ixbeg; ix <= ixend; ix++)
	  {
	  ixb += 1 ;
	  *(boxcen + ixb + iyb * xdimcen) = *(imsq + ix + iy * xdimsq) ;
	  }
      }

   /*** Determine the level for the brightest 10% of the pixels. ***/

   kk = (unsigned long) (0.9 * npxcen) ;
   nn = (unsigned long) (npxcen) ;
   fprintf (stdout, "boxcen kk: %d   nn: %d\n", kk, nn) ;

   cenmin = selip (kk, nn, boxcen) ;
   printf ("cenmin: %10.2f\n", cenmin) ;

   /*** Find average intensity of brightest 10% of pixels at disk center. ***/

   favg = 0.0 ;
   navg = 0 ;
   for (iy = iybeg; iy <= iyend; iy++)
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 if (*(imsq + ix + iy * xdim) >= cenmin)
	    {
	    favg += *(imsq + ix + iy * xdim) ;
	    navg++ ;
	    }
	 }
   favg = (favg / navg) ;
   brtcont = favg / 0.97 ;		/* Continuum brightness. */
   printf ("favg: %7.2f   brtcont: %7.2f\n", favg, brtcont) ;

   ix = (int) xcen ;
   iy = (int) ycen ;
   printf ("imsq [center]: %7.2f\n", *(imsq + ixcensq + iycensq * xdimsq)) ;

   /*** Extract a horizontal "band" centered about the equator. ***/
   /*** Band is 0.2 Rsun high, slightly less than 2.0 Rsun wide. ***/
   /*** Find the brightest 20% pixels. ***/

   radius =   0.1 ;
   angle =    0.0 ;
   crcoord (&radius, &angle, &xx, &yy, 1, degnorth, xcensq, ycensq, pixrs) ;
   iyend = (int) yy ;

   printf ("xx/yy: %7.2f  %7.2f\n", xx, yy) ;

   angle = asin ((yy - ycensq)  / (ipixrs - 2)) ;

   printf ("angle: %7.2f\n", angle * degrad) ;

   ixbeg = ixcen - (int) (cos (angle) * pixrs) ;
   ixend = ixcen + (int) (cos (angle) * pixrs) ;

   radius =   0.1 ;
   angle =  180.0 ;
   crcoord (&radius, &angle, &xx, &yy, 1, degnorth, xcensq, ycensq, pixrs) ;
   iybeg = (int) yy ;

   fprintf (stdout, "xx/yy: %7.2f  %7.2f\n", xx, yy) ;

   printf ("ixbeg/ixend: %4d %4d\n", ixbeg, ixend) ;
   fprintf (stdout, "iybeg/iyend: %4d %4d\n", iybeg, iyend) ;

   xdimband = ixend - ixbeg + 1 ;
   ydimband = iyend - iybeg + 1 ;
   npxband  = xdimband * ydimband ;

   fprintf (stdout, "xdimband/ydimband: %4d %4d  npxband: %6d\n",
	    xdimband, ydimband, npxband) ;
   fflush (stdout) ;

   fprintf (stdout, "check #1\n") ;
   fflush (stdout) ;

   band = (float *) malloc (npxband * sizeof (float)) ;

   fprintf (stdout, "check #2\n") ;
   fflush (stdout) ;

   iyb = -1 ;
   for (iy = iybeg; iy <= iyend; iy++)
      {
      iyb += 1 ;
      ixb = -1 ;
      printf ("iy: %4d\n", iy) ;
      for (ix = ixbeg; ix <= ixend; ix++)
	  {
	  ixb += 1 ;
	  fprintf (stdout, "ix/iy: %4d %4d   ixb/iyb: %4d %4d\n",
		   ix, iy, ixb, iyb) ;
          fflush (stdout) ;

	  *(band + ixb + iyb * xdimband) = *(imsq + ix + iy * xdimsq) ;

	  }
      }

   kk = (unsigned long) (0.8 * npxband) ;
   nn = (unsigned long) (npxband) ;
   fprintf (stdout, "band kk: %d   nn: %d\n", kk, nn) ;
   fflush (stdout) ;
/*
 *   brtmin = selip ((unsigned long) (0.8 * npxband), (unsigned long) npxband,
 *		    band) ;
 */
/*
   brtmin = selip (kk, nn, band) ;
   printf ("brtmin: %10.2f\n", brtmin) ;
 */

   /*** Boxes will be of size 0.1 Rsun square. ***/

   xdimbox = ydimband ;
   ydimbox = ydimband ;
   xbeg  = -1.10 ;
   ixbeg = (xcensq - (0.85 * pixrs)) ;
   xinc = 0.05 ;

   sprintf (namdop, "%s.doppler.dat", namimg) ;
   FDDOP = fopen (namdop, "w") ;
   printf ("namdop: %s\n", namdop) ;
   fprintf (FDDOP, "%d\n", nbox) ;
   fprintf (FDDOP, "%s\n", namimg) ;

   for (ib = 0; ib < nbox; ib++)
      {
      radius = (ib * xinc + xbeg) ;
      ixb = (int) ((radius * pixrs) + xcensq) ;
      iyb = ydimsq / 2 ;
      xx = ixb ;
      yy = iyb ;
      boxx [ib] = radius ;
      boxv [ib] = boxbrt (imsq, xdimsq, ydimsq, ixb, iyb, xdimbox, ydimbox,
			  brtmin) ;
      printf ("ib: %3d  ixb/iyb: %4d %4d  boxx: %10.2f  boxv: %10.2f\n",
               ib, ixb, iyb, boxx [ib], boxv [ib]) ;
      fprintf (FDDOP, "%5.2f %10.2f\n", boxx [ib], boxv [ib]) ;
      }

   fclose (FDDOP) ;

/*
 *   xyplot (boxx, boxv, nbox, namemc, title, labx, laby,
 *		       ixscale, iyscale, ibkg);
 */

   /*** Write limb-darkening corrected image to disk. ***/

   for (iy = 0; iy < ydimsq; iy++)
      for (ix = 0; ix < xdimsq; ix++)
	 *(obuf + ix + iy * xdimsq) = (short) *(imsq + ix + iy * xdimsq) ;
	  
   sprintf (namout, "%s.ld", namimg) ;
   npad = xdimsq * nbypix - sizeof (ACOSHS) ;
   padbuf = (unsigned char *) malloc (npad) ;
   ACOSH->Xdim = 936 ;
   ACOSH->Ydim = 936 ;
   FDOUT = fopen (namout, "w") ;

   fwrite ((char *) &ACOSHS, sizeof (ACOSHS), 1, FDOUT) ;
   fwrite (padbuf, sizeof (unsigned char), npad,  FDOUT) ;
   fwrite (obuf,   sizeof (unsigned short), npxsq,  FDOUT) ;
   fclose (FDOUT) ;

   free (padbuf) ;
   free (sbuf) ;
   free (srot) ;
   free (imsq) ;
   free (ldsq) ;
   free (obuf) ;
   free (boxcen) ;
   free (band) ;

/*   clsgks_ (); */
   exit (0) ;
}
