/* chip_rdfits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read CHIP FITS image.
 *
 * Uses the "cfitsio" library to extract information from the FITS headers.
 * NOTE: The direction, scan time are obtained from the extension HDU.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   mk4time.c   26 January 2001
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <ctype.h>
#include "math.h"
#include "fitsio.h"

/* -------------------------------------------------------------------------- */

main (an, av)

int an ;
char *av [] ;
{

    float get_frac_offset () ;
    void prsum () ;
    void printerror () ;

    char *w ;		/* pointer to command line arguments */
    static char comment [80] ;
    static char fits_file [80] ;
    static char code_obs [24] ;
    static char date_obs [24] ;		/* Observation date. */
    static char time_obs [24] ;		/* Observation time. */

    int ihour, iminute, isecond ;	/* Begin scan time. */
    int iyear, imonth,  iday    ;	/* Begin date. */
    int status ;
    int xdim, ydim ;

    float crradius ;			/* Solar Radius (pixels). */
    float solar_p0 ;			/* P-angle. */

    fitsfile *fptr ;

    if ( an <= 2 ) { prsum () ; exit (-4) ; }
    while (--an)
	{
	*(w = *++av) ;
	if (*w == '?') { prsum () ; exit (-2) ; }
	fits_file = w ;

	/*** Open FITS file. ***/

	if ( fits_open_file (&fptr, fits_file, READONLY, &status) )
	    printerror ( status );

	/*** Primary FITS header. ***/

	if (fits_read_key (fptr, TINT, "NAXIS1", &xdim,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TINT, "NAXIS2", &ydim,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TSTRING, "DATE-OBS", date_obs,
	    comment, &status))  printerror (status) ;
	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	pyear  = byear ;
	pmonth = bmonth ;
	pday   = bday ;

	if (fits_read_key (fptr, TSTRING, "TIME-OBS", time_obs,
	    comment, &status))  printerror (status) ;
	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;

	if (fits_read_key (fptr, TSTRING, "CODE-OBS", code_obs,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "CRRADIUS", &crradius,
	    comment, &status))  printerror (status) ;

	if (fits_read_key (fptr, TFLOAT, "SOLAR_P0", &solar_p0,
	    comment, &status))  printerror (status) ;

	/*** Close FITS file. ***/

	if ( fits_close_file (fptr, &status) )  printerror (status) ;

        }
   exit (0) ;
}

/* -------------------------------------------------------------------------- */


/* -------------------------------------------------------------------------- */

/* prsum ()
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print usage summary.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void
prsum ()
{
    static char help [] = "\
chip_rdfits imagename1 imagename2 imagename3 . . . \n\
\n\
Example:              chip_rdfits  20010101.123456.mk4.rpb.fts \n\
Example:              chip_rdfits  20010101.123456.dpm.dsk.fts \n\
Example:              chip_rdfits  20010101.123456.chp.hsr.fts \n\
\n\
Flags	    Description\n\
-----	    -----------\n\
\n\
" ;

       fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void 
printerror ( int status)
{
    /*****************************************************/
    /* Print out cfitsio error messages and exit program */
    /*****************************************************/


    if (status)
        {
        fits_report_error (stderr, status); /* print error report */

        exit ( status );    /* terminate the program, returning error status */
        }

    return;
}

/* -------------------------------------------------------------------------- */

