/* chip_ldark.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Perform radial scans on a CHIP image.
 * Determine limb darkening correction by averaging radial scans.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   16 May 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#define ACOS

#include "/home/cordyn/mlsofits/mechanism_defs.h"
#include "/home/cordyn/mlsofits/acos.h"
#include "/home/cordyn/mlsofits/chip.h"

#include "/home/cordyn/mlsofits/device.h"
#include "/home/cordyn/mlsofits/analog.h"
#include "/home/cordyn/mlsofits/lcvr.h"
#include "/home/cordyn/mlsofits/mic.h"

#include "/home/stanger/lib/src/acos/acoslib.h"
#include "chipscan.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern int acoshskp () ;		/* Function to read ACOS header. */
   extern unsigned short *rdacos () ;	/* Function to read ACOS image. */
   extern float lint () ;		/* Linear interpolation. */

   static char namimg [80] ;		/* ACOS file name. */
   static char namout [80] ;		/* ACOS file name. */

   static float scanavg [MAXSCN] ;
   static float scan    [MAXSCN] ;
   static float scandx  [MAXSCN] ;
   float *ldbuf, *ldsqr ;
   unsigned char *acosbuf ;             /* MLSO ACOS file  array pointer. */
   unsigned short *sbuf ;
   unsigned char *padbuf ;

   unsigned short sval ;

   float angle ;
   float cdelt1 ;
   float degnorth = 0.0 ;
   float ffix ;
   float ldcor ;
   float limbdark ;
   float ldmin, ldmax ;
   float pangle ;
   float pixrs ;
   float radius ;
   float radmin, radmax ;
   float rmin, rmax ;
   float rinc ;
   float savg ;
   float theta ;
   float tmin =   0.0 ;
   float tmax = 360.0 ;
   float xcen, ycen ;
   float xx, yy ;

   int ir ;
   int it ;
   int ix, iy ;
   int ixsq, iysq ;
   int ixbeg, ixend ;
   int ixcen, iycen ;
   int iybeg, iyend ;
   int navg ;
   int nbypix ;
   int nbyimg ;
   int nang ;
   int npad ;
   int npximg ;
   int ns ;
   int pmode = 1 ;
   int scntyp = 1 ;
   int xdim, ydim ;
   int xdimsq, ydimsq ;
   int xdif, ydif ;

   unsigned short sfix ;

   FILE *FD, *FDOUT, *FDCOR ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s chip_filename\n", argv [0]) ;
      exit (-1) ;
      }

   sprintf (namout, "%s.ff", namimg) ;

   /* Read header. */

   acoshskp (namimg, 0) ;

   /* Get items from header. */

   xdim = ACOSI->xdim ;
   ydim = ACOSI->ydim ;
   nbypix = ACOSI->nbypx ;
   xcen = (float) (ACOSH->Xdim - 1) * 0.5 ;
   ycen = (float) (ACOSH->Ydim - 1) * 0.5 ;
   ixcen = (int) xcen ;
   iycen = (int) ycen ;
   cdelt1 = 2.29 ;				/* arcsec / pixel */
   pixrs = ACOSI->rsun / cdelt1 ;		/* pixels / solar radius. */
   pangle = ACOSI->pangle ;			/* rotation angle. */
   printf ("xdim: %d  ydim: %d\n", xdim, ydim) ;
   printf ("xcen: %7.2f ycen: %7.2f\n", xcen, ycen) ;
   printf ("pixrs: %7.2f  pangle: %7.2f\n", pixrs, pangle) ;

   /*** Read image. ***/

/*
   acosbuf = rdacos (namimg, pmode) ;
 */

   FD = fopen (namimg, "r") ;   
   if (FD == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      exit (-2) ;
      }

   /*** Allocate storage for image array. ***/

   npximg = ACOSH->Xdim * ACOSH->Ydim ;         /* # pixels in image. */
   nbypix = sizeof (unsigned short) ;           /* # bytes  in image. */
   nbyimg = npximg * nbypix ;
   sbuf = (unsigned short *) malloc (nbyimg) ;  /* Allocate memory for image. */
   ldbuf = (float *) malloc (npximg * sizeof (float)) ;
   
   /*** Read pixel data. ***/

   fread (sbuf, nbypix, npximg, FD) ;

   fclose (FD) ;

   /*** Set first 3 lines of image to zero. ***/

   for (iy = 0; iy < 3; iy++)
      for (ix = 0; ix < xdim; ix++)
	 {
	 *(sbuf + ix + iy * xdim) = 0 ;
	 }
   acosbuf = (unsigned char *) sbuf ;

   /*** Find average intensity near disk center. ***/

   ixbeg = ixcen - 5 ;
   ixend = ixbeg + 10 ;
   iybeg = iycen - 5 ;
   iyend = iybeg + 10 ;

   savg = 0.0 ;
   navg   = 0 ;
   for (iy = iybeg; iy <= iyend; iy++)
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 savg += (float) *(sbuf + ix + iy * xdim) ;
	 navg++ ;
	 }
   savg = (savg / navg) ;

   ix = (int) xcen ;
   iy = (int) ycen ;
   printf ("sbuf [center]: %d\n", *(sbuf + ix + iy * xdim)) ;

   /*** Perform radial scan. ***/

   for (ir = 0; ir < MAXSCN; ir++) scanavg [ir] = 0.0 ;

   rmin = 0.0 ;
   rmax = 1.00001 ;
   rinc = 1.0 / pixrs ;
   nang = 0 ;
   for (it = tmin; it < tmax; it += 1.0)
      {
      theta = it ;
      nang = nang + 1 ;

      crscan (acosbuf, pangle, nbypix, xdim, xcen, ycen, pixrs,
	      rmin, rmax, rinc,
	      theta, &scntyp, scan, scandx, &ns) ;

      for (ir = 0; ir < ns; ir++)
	 scanavg [ir] = scanavg [ir] + scan [ir] ;
      }

   /*** Compute average radial scan. ***/

   for (ir = 0; ir < ns; ir++)
      scanavg [ir] /= (float) nang ;

   /*** Remove limb darkening. ***/

   ldmin  =  1.0e+6 ;
   ldmax  = -1.0e+6 ;
   radmin =  10.0 ;
   radmax = -10.0 ;
   printf ("xdim: %d  ydim: %d\n", xdim, ydim) ;
   printf ("sbuf [center]: %d\n", *(sbuf + ix + iy * xdim)) ;

   for (iy = 1; iy < ydim; iy++)
      {
      yy = (float) iy ;
      for (ix = 0; ix < xdim; ix++)
	 {
	 xx = (float) ix ;

	 /* Get solar radius. */

         crcoord (&radius, &angle, &xx, &yy, -1, degnorth, xcen, ycen, pixrs) ;

         if (iy == 515)
	    printf ("ix/iy: %4d  xx/yy: %7.2f %7.2f  radius: %5.2f  sbuf: %6d\n",
		     ix, xx, yy, radius, *(sbuf + ix + iy * xdim)) ;

         if (radius < radmin) radmin = radius ;
	 if (radius > radmax) radmax = radius ;

         ldcor = 0.0 ;
         if (radius < 1.0)
	    {
	    limbdark = lint (radius, scandx, scanavg, ns) ;
	    ldcor = savg / limbdark ;
	    if (ldcor < 0.0) ldcor = 0.0 ;
	    sval = *(sbuf + ix + iy * xdim) ;
	    ffix = (*(sbuf + ix + iy * xdim) * ldcor + 0.5) ;
	    sfix = (unsigned short) ffix ;
	    *(sbuf + ix + iy * xdim) = sfix ;

/*
         if (iy == 515)
	    printf ("radius: %5.2f  angle: %7.2f  limbdark: %7.2f  ldcor: %5.2f  sval: %6d\n",
		     radius, angle, limbdark, ldcor, sval) ;
 */

/*
	    printf ("xx/yy: %7.2f %7.2f  r/th: %7.2f %7.2f sbuf: %5d  sfix: %5d\n", xx, yy, radius, angle, *(sbuf + ix + iy * xdim), sfix) ;
 */
	    }

         *(ldbuf + ix + iy * xdim) = ldcor ;
         if (ldcor < ldmin) ldmin = ldcor ;
         if (ldcor > ldmax) ldmax = ldcor ;
	 }
      }

   printf ("ldmin:  %8.2f  ldmax:  %8.2f\n", ldmin, ldmax) ;
   printf ("radmin: %8.2f  radmax: %8.2f\n", radmin, radmax) ;

   /*** Write corrected image to disk. ***/

   npad = xdim * nbypix - sizeof (ACOSHS) ;
   padbuf = (unsigned char *) malloc (npad) ;
   FDOUT = fopen (namout, "w") ;
   fwrite ((char *) &ACOSHS, sizeof (ACOSHS), 1, FDOUT) ;
   fwrite (padbuf, npad, sizeof(unsigned char), FDOUT) ;
   fwrite (sbuf,   xdim * ydim, sizeof (unsigned short), FDOUT) ;
   fclose (FDOUT) ;

   /*** Write limb darkening correction image to disk. ***/

   xdimsq = 936 ;
   ydimsq = 936 ;

   ixbeg = (int) (((ixcen - xdimsq) / 2) + 0.5) ;
   ixend = ixbeg + xdimsq ;
   iybeg = (int) (((iycen - ydimsq) / 2) + 0.5) ;
   iyend = iybeg + ydimsq ;

   xdif = (int) (((xdim - xdimsq) / 2) + 0.5) ;
   ydif = (int) (((ydim - ydimsq) / 2) + 0.5) ;

   ldsqr = (float *) malloc (xdimsq * ydimsq * sizeof (float)) ;

   for (iy = iybeg; iy <= iyend; iy++)
      {
      iysq = iy - ydif ;
      for (ix = ixbeg; ix <= ixend; ix++)
	 {
	 ixsq = ix - xdif ;
	 *(ldsqr + ixsq + iysq * xdimsq) = *(ldbuf + ix + iy * xdim) ;
	 }
      }

   FDCOR = fopen ("chip.ldcor.img", "w") ;
   fwrite (ldsqr, xdimsq * ydimsq, 4, FDCOR) ;
   fclose (FDCOR) ;

   free (padbuf) ;
   free (sbuf) ;

   exit (0) ;
}
