/* crcoord.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * cylindrical <---> rectangular coordinate transformation.
 * crcoord (radius, theta, x, y, isign, degnorth, xcen, ycen, pixrs)
 * isign = 1 :  (radius,theta) --> (x,y).
 *       =-1 :  (x,y) --> (radius,theta).
 * Image coordinate origin (x=0,y=0): lower left corner.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   15 May 2000   Adapted from rcoord.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

crcoord (radius, theta, x, y, isign, roll, xcen, ycen, pixrs)

float	*radius, *theta ;/* Radius (Rsun) & Position Angle (degrees). */
float	*x, *y ;		/* Cartesian coordinates:  (0,0)=lower left.*/
int	isign ;		/* =1: (r,th) --> (x,y). otherwise, vice-versa.*/
float	roll ;		/* roll w.r.t. solar north. */
float	xcen, ycen ;	/* Sun center location. */
float	pixrs ;		/* # Pixels per solar radius. */

{
int	ierr ;			/* error indicator. */
float	xang ;			/* angle in degrees w.r.t. +x axis. */
double	angle ;			/* angle in radians. */
double deltax, deltay ;
double degnorth ;
double degrad = 57.295780 ;	/* degrees/radian. */
double raddeg =  0.017453 ;	/* radians/degree. */
double r2 ;			/* radius^2 */

   degnorth = roll + 90.0 ;	/* Orientation of solar north. */
   ierr = 0 ;

   if (isign == 1)		/*** (radius,theta) --> (x,y) ***/
      {
      xang = *theta + degnorth ;
      while (xang > 360.0)  xang -= 360.0 ;
      angle = (float) (raddeg * xang) ;
         *x = (float) (xcen + *radius * pixrs * cos (angle)) ;
         *y = (float) (ycen + *radius * pixrs * sin (angle)) ;

/*
      if ((*x < ip || *x > lp) || (*y < ir || *y > lr))
         ierr = -1 ;
 */
      }
   else				/*** (x,y) --> (r,theta) ***/
      {
      deltax = *x - xcen ;
      deltay = *y - ycen ;
      r2 = deltax * deltax + deltay * deltay ;

      *radius = sqrt (r2) / pixrs ;
      *theta = atan2 (deltay, deltax) * degrad - degnorth ;
      while (*theta < 0.0)  *theta += 360.0 ;
      }
   return (ierr) ;
}
