/* crscan2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Radial Scan.   Adapted from /home/stanger/smm/cp/prscan.c
 * Andrew L. Stanger   HAO/NCAR   15 May 2000
 * 9 Aug 2000 : Add nave to argument list.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

/*
 * #include "/home/stanger/lib/include/cplib.ext"
 * #include "/home/stanger/lib/include/img_scale.ext"
 */
#include "/home/stanger/lib/include/ov_color.h"
#include "chipscan.h"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

crscan2 (image, pangle, nbypix, xdim, xcen, ycen, pixrs, rmin, rmax, rinc,
	theta, scntyp, scan, scandx, ns, nave)

unsigned char	*image ;		/* Pointer to image array. */
float	pangle ;		/* Solar P-angle */
int	nbypix ;		/* # bytes per pixel in image array. */
int	xdim ;		/* Size of X-axis dimension. */
float	xcen, ycen ;	/* Sun center location. */
float	pixrs ;		/* # pixels / solar radius. */
float	rmin, rmax ;	/* Solar radius limits for radial scan. */
float	rinc ;		/* Radial increment for radial scan. */
float	theta ;		/* Theta (degrees). */
int	*scntyp ;	/* Pointer to scan type (1=radial,2=line). */
float	scan [] ;	/* Dependent axis scan array. */
float	scandx [] ;	/* Independent axis scan array. */
int	*ns ;		/* Number of values in scan. */
int	nave ;		/* Number of points to average in scan. */

{
extern float bilint () ;
float	dist, dstinc, dstsun ;
float	radinc ;
float	cval ;
float	x, y ;
float	r ;
float	th ;

register   int	ir ;
int	igr = 0 ;
int	igterm = 0 ;
int	ixg, iyg ;
int	ierr ;
int	nscan ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Radial increment: set to 0.1 Rsun if 'rinc' <= 0.0 ***/

   radinc = rinc ;
   if (rinc <= 0.0)  radinc = 0.1 ;
   th = theta ;

/* if (*scntyp == 2) */
      {
      dist = rmax - rmin ;
      dstsun = -radinc ;
      }

   /* ****** Radial Scan ****** */

   for (ir = 0 ; ir < MAXSCN ; ir++)
      *(scan + ir) = *(scandx + ir) = 0.0 ;
 
   /* ....... Radial scan loop ....... */

   *ns = 0 ;
   ir = -1 ;
   for (r = rmin ; r <= rmax ; r += radinc)
      {
      ir++ ;
 
      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = crcoord (&r, &th, &x, &y, 1, pangle, xcen, ycen, pixrs)) != 0)
         return (ierr) ;

      /*** Draw "X" to mark scan point on IRIS display. ***/
 
/*
      if (igr > 0)
         {
         ixg = (int) x + gMINCOL ;
         iyg = (int) y + gMAXROW ;
         gl_wvec (ixg, iyg, ixg+1, iyg+1, ov_red) ;
         gl_wvec (ixg+1, iyg, ixg, iyg+1, ov_red) ;
         }
 */
      /* 
       * Perform bilinear interpolation to compute scan point value from image.
       * Use 4 nearest neighbors.
       */
      dstsun += radinc ;
      if (*scntyp == 1)  *(scandx + ir) = (float) r ;
      else if (*scntyp == 2)  *(scandx + ir) = (float) dstsun ;

      cval = (bilint (image, nbypix, xdim, &x, &y)) ;
      *(scan + ir) = cval ;
      }

count:   *ns = ir + 1 ;

   /*** Smooth data using 'nave' point average. ***/
 
   flaverage (nave, scan, MAXSCN, *ns) ;
 
   nscan = *ns ;
/*
   if (igterm == 0)
	for (ir = 0 ; ir < *ns ; ir++)
           fprintf (stdout, "r:%7.3f  th:%7.2f  scan:%9.2e\n",
                    scandx [ir], th, scan [ir]) ;
 */
   return (0) ;
}
