/* flaverage (nave, buf, nval)
 *--------------------------------------------------------------------
 *** Averaging routine for a vector array of floating point numbers.
 *--------------------------------------------------------------------
 *    Andrew L. Stanger   HAO/NCAR   4 March 1983
 *    6 August 1984: converted from 11/34 (f4p) to VAX 750 (C).
 *    8 February 1985: 'buf' array changed to float.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *    nave = # elements to average.
 *    buf  = floating point array.
 *    nval = number of elements in array buf.
 *--------------------------------------------------------------------
 */
#include <stdio.h>
#include <sys/types.h>
/*
 * #include "/home/stanger/lib/include/cplib.ext"
 */
#include "chipscan.h"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

flaverage (nave, buf, nval)

int nave;	/* # points to average. */
float buf [];	/* vector array. */
int nval;	/* # values in array buf. */

{

   float temp [MAXSCN];			/* Temporary storage array. */
   float sum;
   int   i, is, navehalf;
   float *ps;

   if (nave <= 1)
      {
      fprintf (stderr, "flaverage> No averaging done, since nave=1.\n") ;
      return (0) ;
      }

   if (nval > MAXSCN)
      {
      fprintf (stderr, "flaverage> Array too large for smoothing routine.\n");
      return (0);
      }

   ps = (float *) buf;
   for (i = 0; i < nval; i++)  temp [i] = (float) *(ps + i);

   /*** Average over 'nave' points. ***/

   navehalf = nave / 2;
   for (i = navehalf; i < (nval - navehalf); i++)
      {
      sum = 0.0;
      for (is = 0; is < nave; is++)  sum = sum + temp [i - navehalf + is];
      *(ps + i) = (float) (sum / (float) nave);
      }

   return (0);
}
