/* img2fts.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert a binary image into a fits array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   10 Aug 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#define MAXLIN 180

int
img2fts (namfts, imgbuf, nbypix, xdim, ydim)

char *namfts ;
char *imgbuf ;
int nbypix ;
int xdim ;
int ydim ;

{
   char *padbuf ;
   static char kbitpix [12] = "BITPIX" ;
   static char knaxis  [12] = "NAXIS" ;
   static char knaxis1 [12] = "NAXIS1" ;
   static char knaxis2 [12] = "NAXIS2" ;
   static char kcrpix1 [12] = "CRPIX1" ;
   static char kcrpix2 [12] = "CRPIX1" ;
   static char kcrval1 [12] = "CRVAL1" ;
   static char kcrval2 [12] = "CRVAL2" ;
   static char kcdelt1 [12] = "CDELT1" ;
   static char kcdelt2 [12] = "CDELT2" ;
   static char kcrota1 [12] = "CROTA1" ;
   static char kcrota2 [12] = "CROTA2" ;
   static char kctype1 [12] = "CTYPE1" ;
   static char kctype2 [12] = "CTYPE2" ;

   static char atype1 [] = "X-AXIS." ;
   static char atype2 [] = "Y-AXIS." ;
   static char ctype1  [12] = "X" ;
   static char ctype2  [12] = "Y" ;
   static char cunit1  [12] = "PIXELS" ;
   static char cunit2  [12] = "PIXELS" ;
   static char eq [4] = "= " ;
   static char fitbuf [MAXLIN][80] ;
   static char outbuf [84] ;
   static char pixcom [48] = "" ;
   static char refcom [12] = "" ;

   int bitpix ;
   int i ;
   int ix, iy ;
   int ival ;
   int iynum ;
   int ndim = 2 ;
   int npad ;
   int nrec ;
   int nbyfits = 2880 ;
   int nbyimg ;
   int nhdu ;
   int nwrite ;
   int pmode = 0 ;
   int swrite ;                         /* # bytes written to disk. */

   float crpix1 = 0.0 ;
   float crpix2 = 0.0 ;
   float crval1 = 0.0 ;
   float crval2 = 0.0 ;
   float cdelt1 = 1.0 ;
   float cdelt2 = 1.0 ;
   float htilt  = 0.0 ;

   FILE *FF ;

   nbyimg = xdim * ydim * nbypix ;        /* # bytes in image. */
   iy = 0 ;
   bitpix = nbypix * 8 ;
   nwrite = 0 ;
   swrite = 0 ;

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         fitbuf [iy][ix] = ' ' ;

   /*** Create FITS header. ***/

   strcpy  (&fitbuf [iy++][0],  "SIMPLE  =                    T / FITS FILE") ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	    knaxis, eq, ndim) ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	    knaxis1, eq, xdim, atype1) ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	    knaxis2, eq, ydim, atype2) ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s", atype1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [PIXELS]",
            kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [%s]",
            kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s COORD INCREMENT [%s/PIXEL]",
            kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f", kcrota1, eq, htilt) ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype2, eq, ctype2) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s", atype2) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [PIXELS]",
            kcrpix2, eq, crpix2, atype2, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s %s [%s]",
            kcrval2, eq, crval2, atype2, refcom, cunit2) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f / %s COORD INCREMENT [%s/PIXEL]",
            kcdelt2, eq, cdelt2, atype2, cunit2) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.3f", kcrota2, eq, htilt) ;

   strcpy  (&fitbuf [iy++][0], "END") ;

   /*** Open output file. ***/

   fprintf (stdout, "namfts: %s\n", namfts) ;
   if ((FF = fopen (namfts, "w")) == NULL)
      {
      fprintf (stderr, "Unable to create file: %s\n", namfts) ;
      exit (-2) ;
      }

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   iynum = iy + 1 ;
   iynum = iy ;
   nhdu = iynum / 36 ;
   printf ("nhdu: %d\n", nhdu) ;
   if (nhdu * 36 != iynum) nhdu++ ;

   fprintf (stdout, "fitbuf:\n") ;
   fprintf (stdout, "%s\n", fitbuf) ;

   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         {
         if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
         ival = (int) fitbuf [iy][ix] ;
         fitbuf [iy][ix] = (char) toupper (ival) ;
         }

   for (i = 0 ; i < nhdu*36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write PRIMARY FITS header to disk file. ***/

   nwrite = fwrite (fitbuf, sizeof (char), nhdu*2880, FF) ;
   swrite += nwrite ;
   printf ("swrite: %d\n", swrite) ;
   nwrite = 0 ;

   /*** Write pixel data. ***/

   for (iy = 0 ; iy < ydim ; iy++)
      {
      nwrite  = fwrite (imgbuf + iy * xdim * nbypix, nbypix, xdim, FF) ;
      swrite += nwrite ;
      }

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = (char *) malloc (npad) ;
      for (i = 0 ; i < npad ; i++)  *(padbuf + i) = '\0' ;
      nwrite  = fwrite (padbuf, 1, npad, FF) ;
      swrite += nwrite ;
      free (padbuf) ;
      }

   fclose (FF) ;
   return (0) ;
}
