      SUBROUTINE NLLST2(X,Y,C,SIG,FIX,NA,M,CHISQ,IER,CHIO,W)
C
C     NON-LINEAR LEAST SQUARES FITTING
C
C     INPUT
C        X        =ARRAY OF INDEPENDENT QUANTITIES
C        Y        =ARRAY OF DEPENDENT QUANTITIES
C        C        =ESTIMATES OF THE  FITTING PARAMETERS
C        FIX      -LOGICAL ARRAY
C                  IF FIX(I)=.TRUE. THEN C(I) IS NOT CORRECTED
C        NA       =NUMBER OF PARAMETERS IN FIT (INCLUDING THOSE FIXED)
C        M        =NUMBER OF DATA POINTS
C        W        =WEIGHT OF DEPENDENT VARIABLE
C
C     USER SUPPLIED SUBROUTINES
C        FUNC(X,C,NA)        FUNC=VALUE AT X GIVEN PARAMETERS C
C        DERIV(X,C,NA,F,DF)  F=VALUE AT X GIVEN PARAMETERS C
C                            DF(I) = DF/DC(I)
C
C     OUTPUT
C        C        =LEAST SQUARES FITTED PARAMETERS
C        SIG      =ESTIMATE OF STANDARD ERROR IN C.
C        CHISQ    =ESTIMATE OF UNIT WEIGHT VARIANCE OF FIT
C        IER      =1 --> FIT IS OK
C                 =2 --> FIT IS SUSPICIOUS
C                 =3 --> MORE THAN 25 ITERATIONS REQUIRED
C                 =4 --> DISASTER, NO FIT FOUND
C
C     CALLS:SYMRED,SYMSOL,SYMINV
C
C     COMMENTS:   1) IF MORE THAN 6 PARAMETERS ARE USED BE SURE TO CHANGE
C                    THE DIMENSION STATEMENTS.
C                 2) WHEN DEBUGGING COMPILE WITH THE /DE SWITCH TO TYPE
C                    CONVERGENCE INFORMATION.
C                 3) TO CHANGE THE CONVERGENCE CRITERION ALTER TOL IN DATA.
C
      DIMENSION FIX(NA),X(M),Y(M),A(6,6),A0(6,6),B(6),DC(6),
     1C1(6),INDX(6),C(6),SIG(6),DF(6),W(M)
      LOGICAL FIX,SING,CONV
      DATA NMAX, ITRMAX, TOL / 6,  25,  1.0E-3/
C
C     IS THERE SUFFICIENT SCRATCH?
      IER = 4
      IF(NA.GT.NMAX) RETURN
      IER = 1
C
C     NON-LINEAR LEAST SQUARES
      FAC = 1.0E-3
      N = 0
      ITER=0
      DO 99 I = 1, NA
         C1(I) = C(I)
         IF(FIX(I)) GO TO 99
         N = N + 1
         INDX(N) = I
   99 CONTINUE
      IF(N.EQ.0) RETURN
C     SET UP THE LINEARIZED MATRIX
  100 DO 2 I = 1, N
         DO 1 J = I, N
            A0(I,J) = 0.0
    1    CONTINUE
         B(I) = 0.0
    2 CONTINUE
      CHI0 = 0.0
      DO 5 K = 1, M
         CALL DERIV(X(K),C,NA,F,DF,W(K))
         DO 6 I = 1, N
            J=INDX(I)
            DF(I)=DF(J)
    6    CONTINUE
         DO 4 I = 1, N
            B(I) = B(I) + (Y(K)-F)*DF(I)
            DO 3 J = I, N
               A0(I,J) = A0(I,J) + DF(I)*DF(J)
    3       CONTINUE
    4    CONTINUE
         CHI0 = CHI0 + (Y(K)-F)**2
    5 CONTINUE
C     TYPE 500, ITER, CHI0
C 500 FORMAT(' ITERATION ',I3,' SUM OF RESIDUALS SQUARED ',1PE11.4)
C     APPLY A CORRECTION FACTOR TO AVOID ROUNDOFF PROBLEMS IN TEST AGAINST CHI1
      CHI0 = 1.001*CHI0
C     MULTIPLY THE DIAGONAL BY A SCALE FACTOR
  200 DO 12 I = 1, N
         DO 11 J = I, N
            A(I,J) = A0(I,J)
   11    CONTINUE
         A(I,I) = A(I,I)*(1.0+FAC)
   12 CONTINUE
C     FIND THE CORRECTIONS TO C
      CALL SYMRED(A,SING,N,6)
      IF(.NOT.SING) GO TO 113
C     WRITE(5,112)
C 112 FORMAT(' MATRIX IS SINGULAR, ABANDON ALL HOPE')
      IER = 4
      RETURN
  113 CALL SYMSOL(A,B,DC,N,6)
      CONV = .TRUE.
      DO 13 I = 1, N
         J=INDX(I)
         C1(J)=C(J)+DC(I)
         CONV = CONV.AND.(ABS(DC(I)).LE.TOL*(1.+ABS(C1(J))))
   13 CONTINUE
C     EVALUATE NEW CHI SQUARED
      CHI1 = 0.0
      DO 14 K = 1, M
         CHI1 = CHI1 + (Y(K)-FUNC(X(K),C1,NA,W(K)))**2
   14 CONTINUE
C     TYPE 501, (DC(I),I=1,N)
C 501 FORMAT(' DC ',1P6E11.4)
      IF(CHI1.LT.CHI0) GO TO 300
C     INCREASE THE DAMPING FACTOR
      FAC = 10.0*FAC
      IF(FAC.LE.1.0E5) GO TO 200
C     LAST VALUES WERE OK OR THERE IS NO REASON FOR INCREASING FAC. 
  300 FAC = 0.1*FAC
      DO 21 I = 1, NA
         C(I) = C1(I)
   21 CONTINUE
      IF(CONV) GO TO 318
      ITER = ITER + 1
      IF(ITER.LT.ITRMAX) GO TO 100
      IF(FAC.GT.1.0E-3) IER = 2
      IF(ITER.GE.ITRMAX) IER = 3
  318 CHISQ = CHI1/FLOAT(M-N+1)
      CALL SYMRED(A0,SING,N,6)
      CALL SYMINV(A0,N,6)
      DO 320 I=1,NA
      SIG(I)=0.0
  320 CONTINUE
      DO 330 J=1,N
         I=INDX(J)
         SIG(I)=SQRT(CHISQ*A0(J,J))
  330 CONTINUE
      RETURN
      END
      SUBROUTINE SYMRED(A,SING,N,NR)
      LOGICAL SING
      DIMENSION A(NR,NR)
      DO 7 I = 1, N
         DO 6 J = 1, I
            X = A(J,I)
            IF(I.NE.J) GO TO 3
            IF(J.EQ.1) GO TO 2
            DO 1 KK = 2, J
               K = J + 1 - KK
               Y = A(I,K)
               Z = Y*A(K,K)
               A(I,K) = Z
               X = X - Y*Z
    1       CONTINUE
    2       IF(X.EQ.0.0E0) GO TO 8
            A(I,I) = 1.0E0/X
            GO TO 6
    3       IF(J.EQ.1) GO TO 5
            DO 4 KK = 2, J
               K = J + 1 - KK
               X = X - A(I,K)*A(J,K)
    4       CONTINUE
    5       A(I,J) = X
    6    CONTINUE
    7 CONTINUE
      SING = .FALSE.
      RETURN
    8 SING = .TRUE.
      RETURN
      END
      SUBROUTINE SYMSOL(A,B,X,N,NR)
      DIMENSION A(NR,NR),B(NR),X(NR)
      DO 3 I = 1, N
         Y = B(I)
         IF(I.EQ.1) GO TO 2
         DO 1 KK = 2, I
            K = I + 1 - KK
            Y = Y - A(I,K)*X(K)
    1    CONTINUE
    2    X(I) = Y
    3 CONTINUE
      DO 6 II = 1, N
         I = N + 1 - II
         Y = X(I)*A(I,I)
         IF(I.EQ.N) GO TO 5
         IP1 = I + 1
         DO 4 K = IP1, N
            Y = Y - A(K,I)*X(K)
    4    CONTINUE
    5    X(I) = Y
    6 CONTINUE
      RETURN
      END
      SUBROUTINE SYMINV(A,N,NR)
      DIMENSION A(NR,NR)
      DO 4 I = 2, N
         IM1 = I - 1
         DO 3 J = 1, IM1
            X = - A(I,J)
            IF(J.EQ.IM1) GO TO 2
            JP1 = J + 1
            DO 1 K = JP1, IM1
               X = X - A(I,K)*A(K,J)
    1       CONTINUE
    2       A(I,J) = X
    3    CONTINUE
    4 CONTINUE
      DO 16 J = 1, N
         DO 15 I = J, N
            X = A(I,J)
            IF(I.EQ.N) GO TO 14
            IP1 = I + 1
            IF(I.EQ.J) GO TO 12
            DO 11 K = IP1, N
               X = X + A(K,I)*A(K,J)
   11       CONTINUE
            GO TO 14
   12       DO 13 K = IP1, N
               Z = A(K,J)
               A(K,J) = Z*A(K,K)
               X = X + Z*A(K,J)
   13       CONTINUE
   14       A(I,J) = X
   15    CONTINUE
   16 CONTINUE
      RETURN
      END
