/* shell.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Sorts an array a [1 ... n] into ascending order by Shell's method
 * (diminishing increment sort).
 * n is input;
 * a is replaced on output by its sorted rearrangement.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * "Numerical Recipes: The Art of Scientific Computing" 2nd Edition
 * William H. Press, William T. Vetterling, Saul A. Teukolsky, Brian P. Flannery
 * page 332
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   22 May 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

void shell (n, a)

unsigned long n ;
float a [] ;

{
    unsigned long i, j, inc ;
    float v ;

    inc = 1 ;
    do
	{
	inc *= 3 ;
	inc++ ;
	} while (inc <= n) ;

    do  {
        inc /= 3 ;
	for (i = inc + 1; i <= n; i++)
	    {
	    v = a [i] ;
	    j = i ;
	    if (j <= inc)  break ;
	    }
        a [j] = v ;
        } while (inc > 1) ;

}
