/* srotimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an unsigned short image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * original author: Rose Reynolds
 * Adapted by Andrew Stanger   HAO/NCAR   12 July 1994
 * 24 May 2000: unsigned short int version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>

unsigned short *
srotimg (imginp, dx, dy, cx, cy, angdeg)

unsigned short *imginp;	/* Input image array pointer. */
int dx, dy ;		/* Dimensions of image. */
int cx, cy ;		/* Center of rotation. */
float angdeg ;		/* Rotation angle in degrees (+ = CCW). */

{
   extern char *malloc ();
   register int nx, ny, ox, oy, xdim, ydim;
   float fox, foy, cose, sine, fnx, fny, fnysin, fnycos;
   float fcx, fcy;
   double angle;
   unsigned short *imgrot;
   unsigned short *sinp, *srot ;

   printf ("srotimg> dx/dy:%3d %3d  cx/cy:%3d %3d\n",
	    dx, dy, cx, cy) ;

   xdim = dx;   ydim = dy;
   fcx = (float) cx;   fcy = (float) cy;
   angle = (double) angdeg * (3.14159265 / 180.0); /* convert deg to radians. */
   sine = (float) sin ( angle );
   cose = (float) cos ( angle );
   printf ("angdeg: %6.2f   angle: %f   sine: %6.2f   cose: %6.2f\n",
            angdeg, angle, sine, cose);
   imgrot = (unsigned short *) malloc ( dx * dy * sizeof (unsigned short) );

   sinp = (unsigned short *) imginp ;
   srot = (unsigned short *) imgrot ; 

   for ( ny = 0; ny < ydim; ny++ )
      {
      fny = (float) (ny) - fcy;
      fnysin = fny * sine - fcx;
      fnycos = fny * cose + fcy;

      for ( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *srot++ = 0;
         else
	    *srot++ = *(sinp + (oy*xdim) + ox);
         }
      }

   return ( imgrot );
}
