/*
 *	Accumulate
 *
 *	Task to take data from camera memory
 *	optionally apply dark correction, gain correction and centering,
 *	and write into image memory.
 *
 *	This task is started with the "Run" command.
 *	It accepts commands via the SendAccumCommand either
 *	automatically from the Program script or typed at the keyboard.
 *
 *	David Elmore
 *
 *	Open shared image memory
 *	Read in dark and gain files
 *	Loop
 *		Receive a command on the accumulate command queue
 *		Execute the command
 *		Send out done status
 *  end loop
 *
 * mod 6/14/96 for limb pointer incrementation when not good.
 * Thu Oct 16 09:57:14 MDT 1997
 * Revs for do_shift and to set special camera numbers for 
 * k-corona buffers (4-7).
 */

#include	<stdio.h>
#include	<ctype.h>
#include	<sys/mman.h>
#include	<fcntl.h>
#include 	<netinet/in.h>
#include 	<time.h>

#define		MESSAGE
#define		DEVICE
#define		ACOS
#define		START

#include	<sdvlib.h>
#include 	<camera.h>

#include	<acos.h>
#include	<acos_header.h>
#include	<message.h>
#include	<net.h>
#define		IMAGE
#include	<image.h>
/* Initial diamter for findshift */
#define		PICS_DIAM	670.
#define		CHIP_DIAM	840.
#define		KBUF0	CCD_BUF

/* This is no little deal here.  There must be room for 6 Meg or so */
/* If centering is specified, another 6meg will be malloc ed! */

short	gain [NUM_ROW*NUM_COL];
short	dark [NUM_ROW*NUM_COL];
short	img  [NUM_ROW*NUM_COL];

extern		void	exit();

main (argc,argv)

int	argc;
char	*argv[];
{
FILE	*fid;

caddr_t address,openimagememory();
caddr_t cam_address,opencameramemory();
caddr_t hdr_address[2],openheadermemory();
caddr_t cb,Camera_buf[2*CCD_BUF],Accum_buf[4];
size_t	size ;
int	i,j,ii,jj,nrow,ncol,imsz,obuf;
int	code,iam,nshort,in_buf,out_buf,do_add,do_dark,do_gain,do_center,do_shift;
char	*instr,*receivemessage(),outstr[MSG_NBYTES];
float	xshi,yshi,diam;
extern	void	findshift(),doshift();
int		isum[2] ={ 1,1 };

long curtime;
char *ctime_r(), timebuf[30];

/* Open shared memory */

if ( (address = openimagememory()) < (caddr_t)0 )
	{
	fprintf ( stderr,"Accumulate: memory open error\n" );
	exit (-1);
	}

if ( (cam_address = opencameramemory()) < (caddr_t)0 )
	{
	fprintf ( stderr,"Accumulate: memory open error\n" );
	exit (-1);
	}

if ( (hdr_address[0] = openheadermemory()) < (caddr_t)0 )
	{
	fprintf ( stderr,"Accumulate: memory open error\n" );
	exit (-1);
	}
hdr_address[1]=hdr_address[0]+NUM_COL*2;
Image_header[0] = (struct acos_header*) hdr_address[0];
Image_header[1] = (struct acos_header*) hdr_address[1];

for ( i=0 ; i<CCD_BUF ; i++ )
	{
	Camera_buf[i] = cam_address + i * CCD_BUF_SIZE ;
	fprintf ( stdout,"Camera_buf[%d] = %x\n",i,Camera_buf[i] );
	}
for ( i=0 ; i<CCD_BUF ; i++ )
	{
	Camera_buf[i+KBUF0] = Camera_buf[i]+CCD_BUF_SIZE/4;
	fprintf ( stdout,"Camera_buf[%d] = %x\n",i+KBUF0,Camera_buf[i+KBUF0] );
	}

Accum_buf[0] = address ;
Accum_buf[2] = address + NUM_ROW*(NUM_COL);
Accum_buf[1] = address + NUM_ROW*(NUM_COL)*2;
Accum_buf[3] = address + NUM_ROW*(NUM_COL)*3;

/* Clear out any 86s */

instr = receivemessage ( ACCUM_COMMAND,IPC_NOWAIT );
code = (int)instr;
while ( code > 0 )
	{
	fprintf ( stdout,"Accumulate received %s\n",instr );
	instr = receivemessage ( ACCUM_COMMAND,IPC_NOWAIT );
	code = (int)instr;
	}

/* Who are we? */

	iam = Image_header[0]->Version;
	if ( iam == IS_CHIP )
		openstatout(KII);
	else
		openstatout(NENE);

/* Set up instrument dependent variables */

	nrow=num_row[iam];
	ncol=num_col[iam];
	nshort = nrow*ncol;

	fprintf(stdout,"%s",dark_name[iam] );
	if ( ! (fid = fopen(dark_name[iam],"r"))){
	fprintf (stderr,"%s cannot read %s\n",dark_name[iam]); exit (1); }
	jj=fread(dark,2,nshort,fid);
	fprintf(stdout,"\n" );
	fclose ( fid );

	fprintf(stdout,"%s",gain_name[iam] );
	if ( ! (fid = fopen(gain_name[iam],"r"))){
	fprintf (stderr,"%s cannot read %s\n",gain_name[iam]); exit (1); }
	jj=fread(gain,2,nshort,fid);
	fprintf(stdout,"\n" );
	fclose ( fid );

/* Loop parsing messages and commanding the camera */

instr = receivemessage ( ACCUM_COMMAND,0 );
code = (int)instr;
sscanf ( instr,"%d %d %d %d %d %d",
	&in_buf,&out_buf,&do_add,&do_dark,&do_gain,&do_center,&do_shift );
fprintf ( stdout,"Accumulate: got %d %d %d %d %d %d %d\n",
		in_buf,out_buf,do_add,do_dark,do_gain,do_center,do_shift );

while ( (code > 0) && (in_buf != 86) )
	{

	if ( (in_buf < 0) || (in_buf > 2*CCD_BUF-1 ) )
		in_buf = 0;
	if ( (out_buf < 0) || (out_buf > 3) )
		out_buf = 0;
	nrow = num_row[iam];
	if ( in_buf >= KBUF0 )
		nrow = num_row[iam]/2;
fprintf ( stdout,"Accumulate: %d x %d \n",ncol,nrow );

	obuf = out_buf%2;

 /* Send a busy to Status */

	stat_buf[0] = ACCUM_STATUS;
	stat_buf[1] = DEV_BUSY;
	stat_buf[2] = in_buf;
	stat_buf[3] = out_buf;
	sendstat();


	if ( do_add )
		{
		isum[obuf] = isum[obuf] + 1;
		if ( isum[obuf] > 16 )
			isum[obuf] = 1;
		}
	else			/* this is move, initialize sum counter */
		{
		isum[obuf] = 1;
		for ( j=0 ; j<16 ; j++ )
			{
			Image_header[obuf]->Xshift[j] = 0.;
			Image_header[obuf]->Yshift[j] = 0.;
			Image_header[obuf]->Diam[j] = 0.;
			}
		}



#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	start moveimage   : %s\n",timebuf);
#endif

/* In all cases copy camera buffer to closely pointed to image */

	moveimage ( Camera_buf[in_buf],img,ncol,nrow);

#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	end moveimage   : %s\n",timebuf);
#endif



/* If dark desired, do that */

	if ( do_dark )
		subdark ( img,dark,ncol,nrow );

#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	end subdark   : %s\n",timebuf);
#endif



/* If gain desired, do that */

	if ( do_gain )
		timesgain ( img,gain,ncol,nrow );

#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	end timesgain   : %s\n",timebuf);
#endif



/* If centering desired, do that */

/* Set shift to other buffer */
/* If we do_center, then values of xshi and yshi */
/* get updated. */
/* If not, we can still do_shift with the other buffer */
/* If there is no do_shift, then set shift value to 0 */

	xshi = Image_header[obuf]->Xshift[0];
	yshi = Image_header[obuf]->Yshift[0];

	if ( do_center )
		{
		iam = Image_header[0]->Version;
		if ( iam == IS_CHIP )
			diam = CHIP_DIAM;
		else
			diam = PICS_DIAM;
		findshift(img,ncol,nrow,&xshi,&yshi,&diam);
	fprintf ( stdout,"%f %f %f\n",xshi,yshi,diam);

#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	end findshift   : %s\n",timebuf);
#endif
		}

	if ( do_shift )
		{
		doshift(img,ncol,nrow,xshi,yshi);

#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	end doshift   : %s\n",timebuf);
#endif
		}
	else
		{
		xshi = 0.;
		yshi = 0.;
		diam = 0.;
		}


	if ( do_add )
		addimage ( img,Accum_buf[out_buf],ncol,nrow);
	else					/* this is move, initialize sum counter */
		moveimage ( img,Accum_buf[out_buf],ncol,nrow);

#ifdef BUG
	(void) time(&curtime);
	(void) ctime_r(&curtime,&timebuf,30);
	fprintf(stdout,"	end doadd   : %s\n",timebuf);
#endif


	Image_header[obuf]->Xshift[isum[obuf]-1] = xshi;
	Image_header[obuf]->Yshift[isum[obuf]-1] = yshi;
	Image_header[obuf]->Diam[isum[obuf]-1] = diam;
	Image_header[obuf]->Nsum = isum[obuf];

/* send message for WaitAccum */

	sprintf ( outstr,"%d %d",in_buf,out_buf );
	sendmessage ( WAIT_ACCUM,outstr );

/* Send a done to Status */

	stat_buf[0] = ACCUM_STATUS;
	stat_buf[1] = 1;
	stat_buf[2] = in_buf;
	stat_buf[3] = out_buf;
	sendstat();

/* check for another message, wait here until we get one */

	instr = receivemessage ( ACCUM_COMMAND,0 );
	code = (int)instr;
	sscanf ( instr,"%d %d %d %d %d %d",
		&in_buf,&out_buf,&do_add,&do_dark,&do_gain,&do_center );
	fprintf ( stdout,"Accumulate: got %d %d %d %d %d %d\n",
			in_buf,out_buf,do_add,do_dark,do_gain,do_center );
	}


closestat();
fprintf ( stderr,"Accumulate exit\n" );
exit ( 1 );

}


/**************** findshift ***********************/

/*   Code to find x and y shift to "center" any "disk" type image.     */
/*   This is iterative, recalculating the centers until the difference */
/*       from the last calculation is less than 0.5 pixels.            */

/*   Pass this code --> img     -- your short image.
                       numx        the x-dimension of your "img"
                       numy        the y-dimension of your "img"
                       xshi        the x shift in fractional pixels 
                                   found by this code
                       yshi        the y shift in fractional pixels 
                                   found by this code
                       diam        the true diameter found by this code.

    Updated 10/21/97 to offset from the top   and bottom
                                   and  right and left
    to get past "edge" problems.  (xoff and yoff)

    Also, narrowed the diameter fractional acceptance limit to
    0.10 from 0.36.    0.1 is still around 65 pixels, pretty big. (dlim)
    0.36 is kept for CHIP data.  0.1 is used for PICS data.
    --Alice

*/
/******************************************************************************/

#include <stdio.h>
#define ILIMIT      (10)  /* Iterations before failure */
#define NSPOTS      (10)
#define LIMIT       (0.5)  /* pixel separation between previous and present
                              center computation. */
#define MNSOB    ( -500.) /* The sobel minimum must be less than this,
                             or it isn't a real edge. was -1500. */


float   ledge[NSPOTS], redge[NSPOTS], *le, *re;
float     bot[NSPOTS],   top[NSPOTS], *be, *te;



void findshift(img,numx,numy,xshi,yshi,diam)

    short *img;
    float *xshi,*yshi, *diam;
    int numx,numy;
{

    float xcen,ycen, sepx,sepy, XCEN,YCEN, goodx,goody, goodt,goodb;
    float lastx,lasty;
    float *ximg,*yimg,*sximg,*syimg;
    float sxmn,sxmx,symx,symn;
    float dlim;  /* Fraction of how big the diameter can be off. */
    float sobel[11]={ -6.,-2.,0.,2.,4.,4.,4.,2.,0.,-2.,-6.};

    int   hs=5, whs=11;
    int   yoff , xoff;
    int notgoodenough;
    float RE,LE,TE,BE;
    float xshift,yshift,t_diam;
    double yedge;
    double xdiff, ydiff, xydiff, fabs();
    register float 	*qq, *pp, *ss;
    register short 	*iq;
    register	int	 ii,jj,kk,iter,strt;

    notgoodenough=1;
    iter=0;
    xoff = 100;
    yoff = 50;
    dlim = 0.36;
    dlim = 0.10;

    ximg  = (float *) malloc ( numx*     sizeof ( float )) ;
    sximg = (float *) malloc ( numx*     sizeof ( float )) ;
    yimg  = (float *) malloc ( numy*     sizeof ( float )) ;
    syimg = (float *) malloc ( numy*     sizeof ( float )) ;

    XCEN = xcen = lastx = (numx-1.)*0.5;
    YCEN = ycen = lasty = (numy-1.)*0.5;

    while(notgoodenough) {

        pp= sximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp=  ximg; for(ii=0;ii<numx;ii++) *pp++=0.;
        pp= syimg; for(ii=0;ii<numy;ii++) *pp++=0.;
        pp=  yimg; for(ii=0;ii<numy;ii++) *pp++=0.;

        /* Find the left and right edges. */
        strt = (int)(ycen-NSPOTS*0.5);  /* Get to the "middle" of the image. */
        re=redge; le=ledge;
        for(kk=0;kk<NSPOTS;kk++){

            /**** Zero the x-sobel area, and grab a x row from the image. */

            pp= sximg;             for(ii=0;ii<numx;ii++) *pp++=0.;
            qq=  ximg; 
            iq=img+(strt+kk)*numx; for(ii=0;ii<numx;ii++) *qq++ = *iq++;


            /**** Apply the sobel. */
            pp=sximg+hs;
            for(ii=hs;ii<numx-hs;ii++){
                ss=sobel;
                qq=ximg+ii-hs;
                for(jj=0;jj<whs;jj++) { *pp += (*ss) * (*qq) ; ss++; qq++; }
                pp++;
            }

            /**** Find the minimum (the edge) on the right          */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+whs+xoff;
            sxmn=0.;
            for(ii=xoff;ii<XCEN;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp++; }

            ii=whs+xoff; pp=sximg+whs+xoff;
		while( *pp++ > sxmn ) ii++;
		while( *pp++ <    0 ) ii++;
            *le++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));

#ifdef BUG
            fprintf(stdout,"left %10.2f   sxmn=%10.2f\t",*(le-1), sxmn);
#endif
 
            /**** Find the minimum (the edge) on the right          */
            /**** Gotta go backwards.                               */
            /**** Then find the zero crossing as a fracional pixel. */
            pp=sximg+numx-1-whs-xoff;
            sxmn=0.;
            for(ii=xoff;ii<XCEN;ii++){ sxmn= (sxmn < *pp) ? sxmn : *pp; pp--; }

            ii=numx-1-whs-xoff; pp=sximg+numx-1-whs-xoff;
		while( *pp-- > sxmn ) ii--;
		while( *pp-- <    0 ) ii--;
            *re++=(float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));

#ifdef BUG
            fprintf(stdout,"right %10.2f  sxmn=%10.2f\n",*(re-1),sxmn);
#endif
        }





        /* Find the top and bottom edges. */

        strt = (int)(xcen-NSPOTS*0.5) ;
        be=bot; te=top;
        for(kk=0;kk<NSPOTS;kk++){

            pp= syimg;      for(ii=0;ii<numy;ii++)  *pp++=0.;
            qq=  yimg; 
            iq=img+strt+kk; for(ii=0;ii<numy;ii++){ *qq = *iq; qq++; iq+=numx; }
            pp=syimg+hs;

            for(ii=hs;ii<numy-hs;ii++){ 
                ss=sobel;
                qq=yimg+ii-hs;
                for(jj=0;jj<whs;jj++) *pp += *ss++ * *qq++ ;
                pp++;
            }

            pp=syimg+whs+yoff;
            symn= MNSOB;  /*If it isn't less than this, it isn't a real edge*/
            for(ii=yoff;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp++; }
            if (symn < MNSOB)  {
                ii=whs+yoff; pp=syimg+whs+yoff;
		    while( *pp++ > symn ) ii++;
		    while( *pp++ <    0 ) ii++;
                *be++=(float)(ii-1) + (0.-*(pp-1))/((float)(*pp-*(pp-1)));
            }
            else *be++ = numy;

#ifdef BUG
            fprintf(stdout,"bottom %10.2f  symn=%10.2f\t",*(be-1),symn);
#endif
 
            pp=syimg+numy-1-whs-yoff;
            symn= MNSOB;
            for(ii=yoff;ii<YCEN;ii++){ symn=(symn < *pp) ? symn : *pp; pp--; }
            if (symn < MNSOB)  {
                ii=numy-1-whs-yoff; pp=syimg+numy-1-whs-yoff;
		    while( *pp-- > symn ) ii--;
		    while( *pp-- <    0 ) ii--;
                *te++= (float)(ii) + (0.-*pp)/((float)(*(pp+1)-*(pp)));
            }
            else *te++ = 0.;

#ifdef BUG
            fprintf(stdout,"top %10.2f   symn=%10.2f\n",*(te-1),symn);
#endif
        }

        /*  Check the separations between the top and bottom
            and the right and left edges.  The separations should be close 
            to the known diameter of the sun for the instrument.  If not
            the edge was found incorrectly.  In particular, this happens
            for the bottom and top edges when the image of the sun is 
            moved around in the fov to find the flat field. 
        */
        RE=LE=BE=TE=0.; 
        re=redge; le=ledge; be=bot; te=top;

#ifdef BUG
        fprintf(stdout,"\n");
#endif

        goodx=goody=0.;
        for (ii=0;ii<NSPOTS;ii++){
#ifdef BUG
            fprintf(stdout,"%10.2f %10.2f \t %10.2f %10.2f",*le,*re,*be,*te);
#endif
            sepx= *re - *le;
            if(sepx < (*diam)+dlim*(*diam) && sepx > (*diam)-dlim*(*diam)) {
                LE += *le ; RE += *re ;
                goodx++;
#ifdef BUG
		fprintf(stdout," goodx ");
#endif
            }
            sepy= *te - *be;
            if(sepy < (*diam)+dlim*(*diam) && sepy > (*diam)-dlim*(*diam)) {
                BE += *be ; TE += *te ;
                goody++;
#ifdef BUG
		fprintf(stdout," goody ");
#endif
            }
		le++ ; re++ ; be++ ; te++ ;
#ifdef BUG
		fprintf(stdout,"\n");
#endif
        }
	if(goodx != 0. ) {  LE /= goodx; RE /= goodx; }
	else             {  LE = 0.;     RE =  0.;    }
	if(goody != 0. ) {  BE /= goody; TE /= goody; }
	else             {  BE = 0.;     TE = 0.;     }
        xcen  = (RE-LE)*0.5 +LE;
        ycen  = (TE-BE)*0.5 +BE;
                         
                        /* If the y separations were hideous, go back and */
        if(goody==0) {  /* find a good top **or** bottom edge. */
            BE=TE=0.; 
            goodt=goodb=0.;
            be=bot; te=top;
            for (ii=0;ii<NSPOTS;ii++){
                if((*be>whs ) && (*be<YCEN)    ) { BE += *be++ ; goodb++; }
                if((*te>YCEN) && (*te<numy-whs)) { TE += *te++ ; goodt++; }
            }
            if      (goodb>0.) { 
                BE /= goodb; yedge=(double)BE; BE=0.; 
                goodb=0.;
                be=bot;
                for (ii=0;ii<NSPOTS;ii++){
                    ydiff=yedge- (double)*be;
                    if( fabs(ydiff) < 0.1*numy ) { BE += *be++ ; goodb++; }
                }
                BE /= goodb;  TE=BE+(*diam);  ycen=BE+0.5*(*diam);
            }
            else if (goodt>0.) {
                TE /= goodt; yedge=(double)TE; TE=0.; 
                goodt=0.;
                te=top;
                for (ii=0;ii<NSPOTS;ii++){
                    ydiff=yedge- (double)*te;
                    if( fabs(ydiff) < 0.1*numy ) { TE += *te++ ; goodt++; }
                }
                TE /= goodt;  BE=TE-(*diam);  ycen=TE-0.5*(*diam);
            }
            else {
#ifdef BUF
                fprintf(stdout,"Neither a good top or bottom was found.\n");
#endif
                TE=YCEN+(*diam)*0.5 + 2.*LIMIT*(float)iter;
                BE=YCEN-(*diam)*0.5 + 2.*LIMIT*(float)iter;
                ycen  = (TE-BE)*0.5 +BE;
            }
        }

        t_diam= ((RE-LE) + (TE-BE))*0.5;
        *xshi = xshift = XCEN-xcen;
        *yshi = yshift = YCEN-ycen;

#ifdef BUG
        fprintf(stdout,"Iteration: %d\n",iter);
        fprintf(stdout,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
        fprintf(stdout,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
        fprintf(stdout,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
#endif
      

        /* Check to see how close the centers are to the previous iteration.
           Note, the while loop will happen at least twice...
        */
        xdiff = lastx-xcen;  xdiff = fabs(xdiff);
        ydiff = lasty-ycen;  ydiff = fabs(ydiff);
        xydiff = xdiff + ydiff;
        if (xydiff < LIMIT) notgoodenough=0;

        if (iter   > ILIMIT) 
			{
			*xshi = 0.;
			*yshi = 0.;
			return;
			}

        lastx=xcen; lasty=ycen;

	iter++;
    }

    *diam = t_diam;

#ifdef BUG
    fprintf(stdout,"findshift: iterations: %d\n",iter);
    fprintf(stdout,"\tExpected centers %10.2f %10.2f \n",XCEN,YCEN);
    fprintf(stdout,"\tFound    centers %10.2f %10.2f \n",xcen,ycen);
    fprintf(stdout,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
    fprintf(stdout,"\tShifts: %10.2f %10.2f \n",xshift,yshift);
#endif

    free( (char *) ximg );
    free( (char *)sximg );
    free( (char *) yimg );
    free( (char *)syimg );
}
/********************** doshift ***********************/

/*   Code to "center" any "disk" type image.  This code applies the x and y
     shifts found by routine "findshift()". */

/*   Pass this code --> img     -- your short image.
                       numx        the x-dimension of your "img"
                       numy        the y-dimension of your "img"
                       xshi        the x shift in fractional pixels 
                                       found by findshift()
                       yshi        the y shift in fractional pixels 
                                       found by findshift()
*/

#include <stdio.h>

void doshift(img,numx,numy,xshi,yshi)

    short *img;
    float xshi,yshi;
    int numx,numy;
{

    long imsz;
    float xx,xxm1,yy,yym1;
    float *timg;
    int shift, off;
    float xshift,yshift;
    register short *iq;
    register float *tt,*t1,*t2,*t3,*t4;
	register float	xy1,xy2,xy3,xy4,ftmp;
    register int	ii,jj,kk,strt;

    imsz=numx*numy;

    /* Copy the short image to a another */

    timg = (float *) malloc ( imsz*sizeof ( float )) ; 
    tt= timg; iq=img; ii = imsz>>2;
	while ( ii-- > 0 )
		{*tt++ = *iq++; *tt++ = *iq++; *tt++ = *iq++; *tt++ = *iq++;}

    /* Get the fractional and integer portions of the x and y shift. */

    xshift = xshi ;
    yshift = yshi ;
    if(xshift < 0.0) {
        xx = 1. + (xshift - (int)xshift); xxm1=1.-xx;
        xshift -= 1.0;
    }
    else {
        xx = xshift-(int)xshift; xxm1=1.-xx;
    }
    if(yshift < 0.0) {
        yy = 1. + (yshift - (int)yshift); yym1=1.-yy;
        yshift -= 1.0;
    }
    else {
        yy = yshift-(int)yshift; yym1=1.-yy;
    }
    shift = (int)yshift*numx+(int)xshift;  /* This is how much to shift
                                              the entire array in whole
                                              integer dimensions, looking
                                              at the array as if it was 1-d. */

    /*   Position to the integer portion of the shift. */

    if(shift>=0) { off=shift;        iq=img+off; tt=timg;     }
    else         { off = shift * -1; iq=img;     tt=timg+off; }

    tt += numx; iq += numx;
    t1=tt-1;
    t2=tt;
    t3=tt-1-numx;
    t4=tt  -numx;

	xy1 = xx*yym1;
	xy2 = xxm1*yym1;
	xy3 = xx*yy;
	xy4 = xxm1*yy;
    strt = imsz-off-numx;                  /* Apply the fractional portion  */
	ii = strt;
	while ( ii-- > 0 )
		{
		*iq++ = (short)(xy1 * *t1++ + 
					xy2 * *t2++ + 
					xy3 * *t3++ + 
					xy4 * *t4++ +
					0.5);
		}

    free( (char *) timg );
}

moveimage ( inimg,outimg,ncol,nrow )

int		*inimg,*outimg;
int		ncol,nrow;
{
register	int	*ip,*op;
register	int		i;

/* Do move on longs (2) and do 4 at a time */
i = (nrow*ncol)>>3;
ip = inimg;
op = outimg;
while ( i-- > 0 )
	{
	*op++ = *ip++;
	*op++ = *ip++;
	*op++ = *ip++;
	*op++ = *ip++;
	}
}

addimage ( inimg,outimg,ncol,nrow )

short	*inimg,*outimg;
int		ncol,nrow;
{
register	short	*ip,*op;
register	int		i;

i = nrow*ncol>>2;
ip = inimg;
op = outimg;
while ( i-- > 0 )
	{
	*op = *op + *ip++; op++;
	*op = *op + *ip++; op++;
	*op = *op + *ip++; op++;
	*op = *op + *ip++; op++;
	}
}

/*
 * Subtract bias image as well as
 * intensity to bias problem
 */

subdark ( inimg,dark,ncol,nrow)

short	*inimg,*dark;
int		ncol,nrow;
{
register	short	*idp,*ip,*op,*dp;
register	short	sb;
register	int		i,j,b;

op = inimg;
dp = dark;
j = nrow;
while ( j-- > 0 )
	{
	ip = op;
	ip += 4;
	idp = dp;
	idp += 4;
	i = 32;
	b = 0;
	while ( i-- > 0 )
		{
		b += *ip++ - *idp++;
		b += *ip++ - *idp++;
		}
	b = b >> 5;
	sb = b;
	ip = op;
	i = ncol>>1;
	while ( i-- > 0 )
		{
/******
		*op++ = *ip++ - *dp++ - sb;
		*op++ = *ip++ - *dp++ - sb;
*****/
/**** Modification 3/8/01 DFE to fix LSD offsets *****/
		*op++ = *ip++ - *dp++ ;
		*op++ = *ip++ - *dp++ ;
		}
	}
}

timesgain ( inimg,gain,ncol,nrow)

short	*inimg,*gain;
int		ncol,nrow;
{
register	short	*ip,*op,*gp;
register	int		itmp,i;

ip = inimg;
op = inimg;
gp = gain;
i = nrow*ncol >> 2;
while ( i-- > 0 )
	{
	itmp = *ip++;
	itmp *= *gp++;
	itmp = itmp >> 13;
	*op++ = itmp;
	itmp = *ip++;
	itmp *= *gp++;
	itmp = itmp >> 13;
	*op++ = itmp;
	itmp = *ip++;
	itmp *= *gp++;
	itmp = itmp >> 13;
	*op++ = itmp;
	itmp = *ip++;
	itmp *= *gp++;
	itmp = itmp >> 13;
	*op++ = itmp;
	}
}


