/* chip2acos.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert binary CHIP image to ACOS format (add header).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   27 Nov 1995
 * 28 Nov 1995: Add Diam, Pos, Drive, Analog.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "acos.h"
#include "acoslib.h"
#include "device.h"
#define MAXFILES 250

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern char *malloc () ;		/* Function to allocate memory. */
   static char flist [MAXFILES][80] ;
   static char namimg [80] ;
   static char namlst [80] ;
   static char namout [80] ;
   unsigned char *acosbuf ;
   int argp = 1 ;
   int fd = -1 ;
   int findex = 0 ;
   int hdrsiz = 165 ;
   int i = 0 ;
   int nelem = 0 ;
   int nbyimg = -1 ;
   int nf = 0 ;
   int nread = 0 ;
   int numfil = 0 ;
   int pmode = 1 ;
   int seq = 0 ;
   FILE *FD, *LIST ;


   if (argc == 1)
      {
      fprintf (stderr, "usage: %s {chip_file_name[s] | @namelist}\n",
               argv [0]) ;
      exit (0) ;
      }
   else
   if (argc > MAXFILES)
      {
      fprintf (stderr, "Maximum number of input files: %d\n", MAXFILES) ;
      exit (-1) ;
      }
   else
   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
   if (argc >= 3)
      {
      nf  = 0 ;
      seq = 1 ;
      while (argp < argc)
	 {
         strcpy (&flist [nf++][0], argv [argp]) ;
	 argp++ ;
	 }
      numfil = nf ;
      }

   /*** Filename list. ***/

   if (namimg [0] == '@')
      {
      seq = 1 ;

      strcpy (namlst, &namimg [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst) ;
         exit (-2) ;
         }
      i = 0 ;
      while (fscanf (LIST, "%s", namimg) != EOF)
         {
         strcpy (&flist [i++][0], namimg) ;
         }
      numfil = i ;
      fclose (LIST) ;
      }

loop:

   if (findex > numfil)  exit (0) ;
   if (seq)
      strcpy (namimg, &flist [findex][0]) ;
   findex++ ;

      sprintf (namout, "%s.acos", namimg) ;

   /*** Read CHIP image from disk file. ***/

   FD = fopen (namimg, "r") ;
/*
   printf ("namimg: %s\n", namimg) ;
   if (fd = open (namimg, 0) < 0)
      {
      fprintf (stderr, "Unable to open image file: %s\n", namimg) ;
      exit (-1) ;
      }
 */
   nelem = 767 * 516 ;
   nbyimg = nelem * sizeof (short) ;
   printf ("fd: %d   nbyimg: %d\n", fd, nbyimg ) ;
   acosbuf = (unsigned char *) malloc (nbyimg) ;

   fread (acosbuf, sizeof (short), nelem, FD) ;
   fclose (FD) ;

/*   nread = read (fd, acosbuf, nbyimg) ; */	/* Read ACOS image. */
   printf ("nread: %d\n", nread) ;
   close (fd) ;

   ACOSH->Version   = IS_CHIP ;
   ACOSH->Year      = 1995;
   ACOSH->Month     =    9 ;
   ACOSH->Day       =   22 ;
   ACOSH->Hour      =   18 ;
   ACOSH->Minute    =   00 ;
   ACOSH->Sec       =   00 ;

   ACOSH->Xdim      =  767 ;
   ACOSH->Ydim      =  516 ;
   ACOSH->Nsum      =    1 ;
   ACOSH->Exposure  =  200 ;
   ACOSH->Gain      =    1 ;
   ACOSH->Odisk     = 1920 ;
   ACOSH->Type      = CHIP_MAJOR ;
   ACOSH->Wave      = 1083.029 ;	/* Helium. */

   ACOSH->Pangle    = 0.0 ;		/* Solar P-angle (deg). */
   ACOSH->Bangle    = 1.0 ;		/* Solar B-angle (deg). */
   ACOSH->Langle    = 2.0 ;		/* Solar L-angle (deg). */
   ACOSH->HA        = 3.0 ;		/* Hour angle (deg). */
   ACOSH->Elevation = 4.0 ;		/* Solar elevation (deg). */
   ACOSH->Azimuth   = 5.0 ;		/* Solar azimuth (deg). */
   ACOSH->Amass     = 6.0 ;		/* Air mass range: 1 -> infinity. */
   ACOSH->Radius    = 960.0 ;		/* Solar Radius (arcsec) */

   /* X-axis shift applied to exposures summed to make this image. */
   /* Nominal: 0 -> 100.0 */

   ACOSH->Xshift [0]   = 0.0 ;
   ACOSH->Xshift [1]   = 1.0 ;
   ACOSH->Xshift [2]   = 2.0 ;
   ACOSH->Xshift [3]   = 3.0 ;
   ACOSH->Xshift [4]   = 4.0 ;
   ACOSH->Xshift [5]   = 5.0 ;
   ACOSH->Xshift [6]   = 6.0 ;
   ACOSH->Xshift [7]   = 7.0 ;
   ACOSH->Xshift [8]   = 8.0 ;
   ACOSH->Xshift [9]   = 9.0 ;
   ACOSH->Xshift [10]  = 10.0 ;
   ACOSH->Xshift [11]  = 11.0 ;
   ACOSH->Xshift [12]  = 12.0 ;
   ACOSH->Xshift [13]  = 13.0 ;
   ACOSH->Xshift [14]  = 14.0 ;
   ACOSH->Xshift [15]  = 15.0 ;

   /* Y-axis shift applied to exposures summed to make this image. */
   /* Nominal: 0 -> 100.0 */

   ACOSH->Yshift [0]   = 100.0 ;
   ACOSH->Yshift [1]   = 101.0 ;
   ACOSH->Yshift [2]   = 102.0 ;
   ACOSH->Yshift [3]   = 103.0 ;
   ACOSH->Yshift [4]   = 104.0 ;
   ACOSH->Yshift [5]   = 105.0 ;
   ACOSH->Yshift [6]   = 106.0 ;
   ACOSH->Yshift [7]   = 107.0 ;
   ACOSH->Yshift [8]   = 108.0 ;
   ACOSH->Yshift [9]   = 109.0 ;
   ACOSH->Yshift [10]  = 110.0 ;
   ACOSH->Yshift [11]  = 111.0 ;
   ACOSH->Yshift [12]  = 112.0 ;
   ACOSH->Yshift [13]  = 113.0 ;
   ACOSH->Yshift [14]  = 114.0 ;
   ACOSH->Yshift [15]  = 115.0 ;

   /* Solar Diameter of exposures summed to make this image. */
   /* Nominal: 420.0 pixels */

   ACOSH->Diam [0]   = 420.0 ;
   ACOSH->Diam [1]   = 421.0 ;
   ACOSH->Diam [2]   = 422.0 ;
   ACOSH->Diam [3]   = 423.0 ;
   ACOSH->Diam [4]   = 424.0 ;
   ACOSH->Diam [5]   = 425.0 ;
   ACOSH->Diam [6]   = 426.0 ;
   ACOSH->Diam [7]   = 427.0 ;
   ACOSH->Diam [8]   = 428.0 ;
   ACOSH->Diam [9]   = 429.0 ;
   ACOSH->Diam [10]  = 430.0 ;
   ACOSH->Diam [11]  = 431.0 ;
   ACOSH->Diam [12]  = 432.0 ;
   ACOSH->Diam [13]  = 433.0 ;
   ACOSH->Diam [14]  = 434.0 ;
   ACOSH->Diam [15]  = 435.0 ;

   /* Positions of ACOS optical devices. */

   ACOSH->Pos [0]   = 1 ;	/* Filter Stage In. */
   ACOSH->Pos [1]   = 2 ;	/* O2 Stage dPMon. */
   ACOSH->Pos [2]   = 2 ;	/* CHIP cover OUT. */
   ACOSH->Pos [3]   = 2 ;	/* CHIP diffuser Out. */
   ACOSH->Pos [4]   = 1 ;	/* CHIP strobe CHIP. */
   ACOSH->Pos [5]   = 0 ;	/* DPM/LowK strobe None. */
   ACOSH->Pos [6]   = 0 ;	/* Occulting Limits Unknown. */
   ACOSH->Pos [7]   = 2 ;	/* Cover Out. */
   ACOSH->Pos [8]   = 2 ;	/* Polarizer Stage Out. */
   ACOSH->Pos [9]   = 6 ;	/* Filter Wheel 656.3. */
   ACOSH->Pos [10]  = 0 ;	/* Polarizer Wheel 0 .*/
   ACOSH->Pos [11]  = 0 ;	/* Objective Wheel Clear. */
   ACOSH->Pos [12]  = 5 ;	/* Occulting Wheel Occult. */
   ACOSH->Pos [13]  = 0 ;
   ACOSH->Pos [14]  = 0 ;
   ACOSH->Pos [15]  = 0 ;
   ACOSH->Pos [16]  = 0 ;
   ACOSH->Pos [17]  = 0 ;
   ACOSH->Pos [18]  = 0 ;
   ACOSH->Pos [19]  = 0 ;
   ACOSH->Pos [20]  = 0 ;
   ACOSH->Pos [21]  = 0 ;
   ACOSH->Pos [22]  = 0 ;
   ACOSH->Pos [23]  = 0 ;
   ACOSH->Pos [24]  = 0 ;
   ACOSH->Pos [25]  = 0 ;
   ACOSH->Pos [26]  = 0 ;
   ACOSH->Pos [27]  = 0 ;
   ACOSH->Pos [28]  = 0 ;
   ACOSH->Pos [29]  = 0 ;
   ACOSH->Pos [30]  = 0 ;
   ACOSH->Pos [31]  = 0 ;

   /* Drive Requests for ACOS optical devices. */

   ACOSH->Drive [0]   = 3 ;		/* Filter Stage Stop. */
   ACOSH->Drive [1]   = 3 ;		/* O2 Stage Stop. */
   ACOSH->Drive [2]   = 3 ;		/* CHIP Cover Stop. */
   ACOSH->Drive [3]   = 3 ;		/* CHIP Diffuser Stop. */
   ACOSH->Drive [4]   = 3 ;		/* CHIP Strobe 0. */
   ACOSH->Drive [5]   = 3 ;		/* DPM/LowK Strobe 0. */
   ACOSH->Drive [6]   = 3 ;		/* Occulting Limits 0. */
   ACOSH->Drive [7]   = 3 ;		/* Cover Stop. */
   ACOSH->Drive [8]   = 3 ;		/* Polarizer Stage Stop. */
   ACOSH->Drive [9]   = 3 ;		/* Filter Wheel Stop. */
   ACOSH->Drive [10]  = 3 ;		/* Polarizer Wheel Stop. */
   ACOSH->Drive [11]  = 3 ;		/* Objective Wheel Stop. */
   ACOSH->Drive [12]  = 3 ;		/* Occulting Wheel Stop. */
   ACOSH->Drive [13]  = 3 ;
   ACOSH->Drive [14]  = 3 ;
   ACOSH->Drive [15]  = 3 ;
   ACOSH->Drive [16]  = 3 ;
   ACOSH->Drive [17]  = 3 ;
   ACOSH->Drive [18]  = 3 ;
   ACOSH->Drive [19]  = 3 ;
   ACOSH->Drive [20]  = 3 ;
   ACOSH->Drive [21]  = 3 ;
   ACOSH->Drive [22]  = 3 ;
   ACOSH->Drive [23]  = 3 ;
   ACOSH->Drive [24]  = 3 ;
   ACOSH->Drive [25]  = 3 ;
   ACOSH->Drive [26]  = 3 ;
   ACOSH->Drive [27]  = 3 ;
   ACOSH->Drive [28]  = 3 ;
   ACOSH->Drive [29]  = 3 ;
   ACOSH->Drive [30]  = 3 ;
   ACOSH->Drive [31]  = 3 ;

   /* Analog readings (interpreted) */

   ACOSH->Analog [0]   =  20.0 ;	/* DPM/LowK Plate Temp:	  20.0 */
   ACOSH->Analog [1]   =  25.0 ;	/* DPM/LowK Air Temp:     25.0 */
   ACOSH->Analog [2]   =  35.0 ;	/* DPM/LowK Filter Temp:  35.0 */
   ACOSH->Analog [3]   =  20.0 ;	/* DPM/LowK Spare Temp:   20.0 */
   ACOSH->Analog [4]   =  25.0 ;	/* LowK PBS Temp:         25.0 */
   ACOSH->Analog [5]   =  25.0 ;	/* LowK Modulator:        25.0 */
   ACOSH->Analog [6]   = 400.0 ;	/* Sky Transmission:     400.0 */
   ACOSH->Analog [7]   =  -1.0 ;	/* Spare Voltage:        -10 to +10 */
   ACOSH->Analog [8]   =  35.0 ;	/* CHIP Filter Temp:      35.0 */
   ACOSH->Analog [9]   =  20.0 ;	/* CHIP Plate Temp:       20.0 */
   ACOSH->Analog [10]  =  25.0 ;	/* CHIP Air Temp:         25.0 */
   ACOSH->Analog [11]  =   1.0 ;	/* CHIP Spare Temp:        0.0 */
   ACOSH->Analog [12]  =   2.0 ;	/* Guider X Error:       -10 to +10 */
   ACOSH->Analog [13]  =   3.0 ;	/* Guider Y Error:       -10 to +10 */
   ACOSH->Analog [14]  =   4.0 ;	/* O1 X Error:           -10 to +10 */
   ACOSH->Analog [15]  =   5.0 ;	/* O1 Y Error:           -10 to +10 */

   /* Liquid Crystal millivoltages. */

   ACOSH->Lcvr [0]     = 0.0 ;		/* CHIP Filter Stage 4 */
   ACOSH->Lcvr [1]     = 1.0 ;		/* CHIP Filter Stage 3 */
   ACOSH->Lcvr [2]     = 2.0 ;		/* CHIP Filter Stage 2 */
   ACOSH->Lcvr [3]     = 3.0 ;		/* CHIP Filter Stage 1 */
   ACOSH->Lcvr [4]     = 4.0 ;		/* Spare LCVR 3 */
   ACOSH->Lcvr [5]     = 5.0 ;		/* Spare LCVR 2 */
   ACOSH->Lcvr [6]     = 6.0 ;		/* Spare LCVR 1 */
   ACOSH->Lcvr [7]     = 7.0 ;		/* LowK Modulator Stage 1 */

   /* Motor Micrometer Readings */
   /* Range: 0-25,000; Nominal: 12,500 */

   ACOSH->Mic  [0]     = 12500.0 ;	/* CHIP Camera Z   */
   ACOSH->Mic  [1]     = 12501.0 ;	/* CHIP O1 X       */
   ACOSH->Mic  [2]     = 12502.0 ;	/* CHIP O1 Y       */
   ACOSH->Mic  [3]     = 12503.0 ;	/* dPMon/LowK C1 X */
   ACOSH->Mic  [4]     = 12504.0 ;	/* dPMon/LowK C1 Y */
   ACOSH->Mic  [5]     = 12505.0 ;	/* dPMon/LowK C1 Z */

   /*** Write header & image to disk file. ***/

   if ((FD = fopen (namout, "w")) == NULL)	/* Open ACOS image file. */
      {
      fprintf (stderr, "Unable to write file: %s\n", namout) ;
      exit (-2) ;
      };

   fwrite ((char *) &ACOSHS, sizeof (ACOSHS), 1, FD) ;	/* Write ACOS header. */
   fwrite (acosbuf, sizeof (char), nbyimg, FD) ;	/* Write ACOS image. */
   fclose (FD) ;					/* Close ACOS file. */

   /*** Print header. ***/

   if (pmode)
      {
      fprintf (stdout, "ACOSH->Version:		%10d\n",	ACOSH->Version) ;
      fprintf (stdout, "ACOSH->Year   :		%10d\n",	ACOSH->Year   ) ;
      fprintf (stdout, "ACOSH->Month  :		%10d\n",	ACOSH->Month  ) ;
      fprintf (stdout, "ACOSH->Day    :		%10d\n",	ACOSH->Day    ) ;
      fprintf (stdout, "ACOSH->Hour   :		%10d\n",	ACOSH->Hour   ) ;
      fprintf (stdout, "ACOSH->Minute :		%10d\n",	ACOSH->Minute ) ;
      fprintf (stdout, "ACOSH->Sec    :		%10d\n",	ACOSH->Sec    ) ;
      fprintf (stdout, "ACOSH->Xdim   :		%10d\n",	ACOSH->Xdim   ) ;
      fprintf (stdout, "ACOSH->Ydim   :		%10d\n",	ACOSH->Ydim   ) ;
      fprintf (stdout, "ACOSH->Nsum   :		%10d\n",	ACOSH->Nsum   ) ;
      fprintf (stdout, "ACOSH->Exposure:	%10d\n",	ACOSH->Exposure) ;
      fprintf (stdout, "ACOSH->Gain   :		%10d\n",	ACOSH->Gain   ) ;
      fprintf (stdout, "ACOSH->Odisk  :		%10d\n",	ACOSH->Odisk  ) ;
      fprintf (stdout, "ACOSH->Type   :		%10d\n",	ACOSH->Type   ) ;
	  fprintf (stdout, "ACOSH->Wave   :		%10d\n",	ACOSH->Wave   ) ;
      fprintf (stdout, "ACOSH->Pangle :		%10.4f\n",	ACOSH->Pangle ) ;
      fprintf (stdout, "ACOSH->Bangle :		%10.4f\n",	ACOSH->Bangle ) ;
      fprintf (stdout, "ACOSH->Langle :		%10.4f\n",	ACOSH->Langle ) ;
      fprintf (stdout, "ACOSH->HA     :		%10.4f\n",	ACOSH->HA     ) ;
      fprintf (stdout, "ACOSH->Elevation:	%10.4f\n",	ACOSH->Elevation) ;
      fprintf (stdout, "ACOSH->Azimuth:		%10.4f\n",	ACOSH->Azimuth) ;
      fprintf (stdout, "ACOSH->Amass  :		%10.4f\n",	ACOSH->Amass  ) ;
      fprintf (stdout, "ACOSH->Radius :		%10.4f\n",	ACOSH->Radius ) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Xshift [%2d]:	%10.3f\n",
                           i, ACOSH->Xshift [i]) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Yshift [%2d]:	%10.3f\n",
                           i, ACOSH->Yshift [i]) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Diam [%2d]:	%10.3f\n",
                           i, ACOSH->Diam [i]) ;
      for (i = 0; i < 32; i++)
         fprintf (stdout, "ACOSH->Pos [%2d]:	%10d\n",
                           i, ACOSH->Pos [i]) ;
      for (i = 0; i < 32; i++)
         fprintf (stdout, "ACOSH->Drive [%2d]:	%10d\n",
                           i, ACOSH->Drive [i]) ;
      for (i = 0; i < 16; i++)
         fprintf (stdout, "ACOSH->Analog [%2d]:	%10.3f\n",
                           i, ACOSH->Analog [i]) ;
      for (i = 0; i <  8; i++)
         fprintf (stdout, "ACOSH->Lcvr [%2d]:	%10.3f\n",
                           i, ACOSH->Lcvr [i]) ;
      for (i = 0; i <  6; i++)
         fprintf (stdout, "ACOSH->Mic [%2d]:	%10.3f\n",
                           i, ACOSH->Mic [i]) ;
      fprintf (stdout, "\n") ;
      printf ("namimg: %s   nbyimg: %d\n", namimg, nbyimg ) ;
      free (acosbuf) ;
      }

   goto loop ;
}
