/* fblint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * blint returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float blint ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, or floating-point images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 29 March 2000: Add nbypix parameter (instead of include statement).
 *                Add xdim & ydim parameters.
 *                Replace "npixl" with "xdim".
 * 11 Apr 2002 : Modify to work only with floating-point images.
 *               Eliminate nbypix argument.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
fblint (imgbuf, xdim, ydim, x, y)

float	*imgbuf;		/* Pointer to image array. */
int	xdim ;			/* X-axis dimension. */
int	ydim ;			/* Y-axis dimension. */
float	x, y;			/* Coordinates of scan point in image. */

{
   register float *imf;
   int	 ix, iy;
   float fx, fy;
   float f00, f10, f01, f11;
   float fx0, fx1, fxy;
  
/*
 * fprintf (stderr, "fblint> xdim/ydim: %4d %4d  x/y: %7.2f %7.2f\n",
 *	    xdim, ydim, x, y) ;
 */

   if (x < 0.0 || x > (float) xdim)
      {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
      return (-1.0) ;
      }

   if (y < 0.0 || y > (float) ydim)
      {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
      return (-2.0) ;
      }

   imf = (float *) imgbuf;

   ix = (int) x;
   iy = (int) y;

   fx = x - (float) ix;
   fy = y - (float) iy;

   /*** Select 4 nearest neighbor elements in 'imgbuf' array. ***/

   f00 = (float) *(imf +  ix    +  iy*xdim);
   f10 = (float) *(imf + (ix+1) +  iy*xdim);
   f01 = (float) *(imf +  ix    + (iy+1)*xdim);
   f11 = (float) *(imf + (ix+1) + (iy+1)*xdim);

   /*** Bilinear interpolation. ***/

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}
