/* fixchiphd.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fix CHIP binary header.
 * Native ifile syntax: yyyy_mm_dd_hh_mm_ss.wsr[.][1|2|3|4]
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 May 2002
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

#define ACOS
#include "/home/cordyn/fits/acos.h"
#include "/home/cordyn/fits/chip.h"
#include "/home/stanger/lib/src/acos/acoslib.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
    static char ifile [80] ;
    static char ofile [80] ;
    static char wave_name   [4] ;
    static char source_name [4] ;
    static char reduce_name [4] ;
    static char expose_name [4] ;
    static char type_obs [80] ;
    static char wsr [8] ;
    unsigned char *padbuf ;
    unsigned char *imgbuf ;

    int argnum = 0 ;
    int pmode = 0 ;
    int year ;
    int month ;
    int day ;
    int hour ;
    int minute ;
    int second ;
    int type ;
    int version ;
    int wave_num ;
    int source_num ;
    int reduce_num ;
    int wave_code ;
    int source_code ;
    int reduce_code ;
    int wave_char ;
    int source_char ;
    int reduce_char ;
    int header_len ;
    int padlen ;
    int xdim, ydim ;

    float wavelen ;
    float wavewid ;

    FILE *IFD, *OFD ;

    if (argc < 2)
        {
        fprintf (stderr, "usage: %s file1 file2 file3 ...\n", argv [0]) ;
	fprintf (stderr, "Fix wavelength & source fields in binary header.\n") ;
	fprintf (stderr, "Input file syntax: YYYY_MM_DD_HH_MM_SS.wsr.1\n") ;
	fprintf (stderr, "w=wavelength, s=source, r=reduction.\n") ;
	fprintf (stderr, "Input & output files are native CHIP format.\n") ;
	fprintf (stderr, "Output file has the date & time from the header.\n") ;
        exit (0) ;
        }
    else
    while (++argnum < argc)
	{
	printf ("argc: %d  argnum: %d\n", argc, argnum) ;

        strcpy (ifile, argv [argnum]) ;
	fprintf (stdout, "\n") ;
	fprintf (stdout, "ifile: %s\n", ifile) ;

	strncpy (wave_name,   &ifile [20], 1) ;
	strncpy (source_name, &ifile [21], 1) ;
	strncpy (reduce_name, &ifile [22], 1) ;
	strncpy (expose_name, &ifile [24], 1) ;

        if (pmode)
	    fprintf (stdout, "wave_name: %d  %o  %x\n",
		     wave_name [0], wave_name [0], wave_name [0]) ;

	wave_char   = (int) wave_name [0] ;
	source_char = (int) source_name [0] ;
	reduce_char = (int) reduce_name [0] ;

        if (pmode)
	    {
	    fprintf (stdout, "wave_name: %s   wave_char: %d\n",
		     wave_name, wave_char) ;
	    fprintf (stdout, "source_name: %s   source_char: %d\n",
		     source_name, source_char) ;
	    fprintf (stdout, "reduce_name: %s   reduce_char: %d\n",
		     reduce_name, reduce_char) ;
	    }

        /*** Read binary header & decode it. ***/

        acoshskp (ifile, pmode) ;

	/*** Extract selected items from the header. ***/

        version = ACOSH->Version ;
	if (version != IS_CHIP)
	    {
	    fprintf (stdout, "Invalid image type (%d) . . . Not CHIP.\n",
		     version) ;
	    exit (-1) ;
	    }
        year    = ACOSH->Year ;
	month   = ACOSH->Month ;
	day     = ACOSH->Day ;
	hour    = ACOSH->Hour ;
	minute  = ACOSH->Minute ;
	second  = ACOSH->Sec ;
	xdim    = ACOSH->Xdim ;
	ydim    = ACOSH->Ydim ;

	type    = ACOSH->Type ;		/* WSR numerical code */
	wave_num = type / 100 ;
	source_num = (type - wave_num * 100) / 10 ;
	reduce_num = (type - wave_num * 100 - source_num * 10) ;

	fprintf (stdout, "Type: %d  Wave: %d  Source: %d  Reduce: %d\n",
		 type, wave_num, source_num, reduce_num) ;

	/*** Re-define wavelength code using information in file name. ***/

	switch (wave_char)
	    {
	    case 'x': /* x */
		      wave_code = CWAVELENGTH_x ;
		      sprintf (type_obs, "UNKNOWN") ;
		      wavelen = 0.0 ;
		      wavewid = 0.0 ;
		      break ;
	    case 'h': /* h */
		      wave_code = CWAVELENGTH_h ;
		      sprintf (type_obs, "HELIUM-I") ;
		      wavelen = 1083.03 ;
		      wavewid =    0.14 ;
		      break ;
	    case 'd': /* d */
		      wave_code = CWAVELENGTH_d ;
		      sprintf (type_obs, "FAR-BLUE") ;
		      wavelen = 1082.74 ;
		      wavewid =    0.14 ;
		      break ;
	    case 't': /* t */
		      wave_code = CWAVELENGTH_t ;
		      sprintf (type_obs, "FAR-RED") ;
		      wavelen = 1083.31 ;
		      wavewid =    0.14 ;
		      break ;
	    case 'c': /* c */
		      wave_code = CWAVELENGTH_c ;
		      sprintf (type_obs, "MID BLUE") ;
		      wavelen = 1082.85 ;
		      wavewid =    0.14 ;
		      break ;
	    case 's': /* s */
		      wave_code = CWAVELENGTH_s ;
		      sprintf (type_obs, "MID RED") ;
		      wavelen = 1083.21 ;
		      wavewid =    0.14 ;
		      break ;
	    case 'b': /* b */
		      wave_code = CWAVELENGTH_b ;
		      sprintf (type_obs, "NEAR BLUE") ;
		      wavelen = 1082.96 ;
		      wavewid =    0.14 ;
		      break ;
	    case 'r': /* r */
		      wave_code = CWAVELENGTH_r ;
		      sprintf (type_obs, "NEAR RED") ;
		      wavelen = 1083.10 ;
		      wavewid =    0.14 ;
		      break ;
	    case 'k': /* k */
		      wave_code = CWAVELENGTH_k ;
		      sprintf (type_obs, "K-CORONA") ;
		      wavelen =  656.28 ;
		      wavewid =    0.80 ;
		      break ;
	    case 'o': /* o */
		      wave_code = CWAVELENGTH_o ;
		      sprintf (type_obs, "OTHER") ;
		      wavelen = 0.0 ;
		      wavewid = 0.0 ;
		      break ;
	    default:  
		      wave_code = CWAVELENGTH_x ;
		      sprintf (type_obs, "UNKNOWN") ;
		      wave_code = 0 ;
		      wavelen = 0.0 ;
		      wavewid = 0.0 ;
	    }

	/*** Re-define source code using information in file name. ***/

	switch (source_char)
	    {
	    case 'x': source_code = 0 ;  break ;
	    case 's': source_code = 1 ;  break ;
	    case 'l': source_code = 2 ;  break ;
	    case 'o': source_code = 3 ;  break ;
	    case 'd': source_code = 4 ;  break ;
	    case 'b': source_code = 5 ;  break ;
	    case 'c': source_code = 6 ;  break ;
	    case 'f': source_code = 7 ;  break ;
	    default:  source_code = 0 ;
	    }

	/*** Re-define reduction code using information in file name. ***/

	switch (reduce_char)
	    {
	    case 'x': reduce_code = 0 ;  break ;
	    case 'r': reduce_code = 1 ;  break ;
	    case 'c': reduce_code = 2 ;  break ;
	    case 'l': reduce_code = 3 ;  break ;
	    case 'd': reduce_code = 4 ;  break ;
	    case 'm': reduce_code = 5 ;  break ;
	    case 't': reduce_code = 6 ;  break ;
	    case 'h': reduce_code = 7 ;  break ;
	    default:  reduce_code = 0 ;
	    }

        if (wave_num != wave_code)
	    {
	    fprintf (stdout, "wave [file]: %d  != wave [header]: %d\n",
		     wave_num, wave_code) ;
	    }

        if (source_num != source_code)
	    {
	    fprintf (stdout, "source [file]: %d  != source [header]: %d\n",
		     source_num, source_code) ;
	    }

        if (reduce_num != reduce_code)
	    {
	    fprintf (stdout, "reduce [file]: %d  != reduce [header]: %d\n",
		     reduce_num, reduce_code) ;
	    }

	/*** Update information in binary header. ***/

        ACOSH->Type = wave_code * 100 + source_code * 10 + reduce_code ;
	ACOSH->Wave = wavelen ;
	fprintf (stdout, "Type: %d  Wave: %f\n", ACOSH->Type, ACOSH->Wave) ;

        /*** Create output file name. ***/

        sprintf (wsr, "%s%s%s%s", wave_name, source_name, reduce_name,
	    expose_name) ;
	sprintf (ofile, "%04d%02d%02d.%02d%02d%02d.chp.%s",
            year, month, day, hour, minute, second, wsr) ;
        fprintf (stdout, "ofile: %s\n", ofile) ;

        /*** Write file with repaired header to disk. ***/

	header_len = sizeof (ACOSHS) ;
	padlen = xdim - header_len ;
	padbuf = (unsigned char *) malloc (padlen) ;
	imgbuf = (unsigned char *) malloc (xdim * ydim * sizeof (short)) ;

	IFD = fopen (ifile, "r") ;
	fread (imgbuf, sizeof (short), xdim * ydim, IFD) ;
	fclose (IFD) ;

        OFD = fopen (ofile, "w") ;
	fwrite (&ACOSHS,    header_len,               1, OFD) ;
	fwrite (padbuf,              1,          padlen, OFD) ;
	fwrite (imgbuf, sizeof (short), xdim * (ydim-1), OFD) ;
	fclose (OFD) ;
	}

    exit (0) ;
}

