/* dPcenter.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Author: Alice Lecinski
 * Modified 9 Sep 97 by Andrew Stanger
 * Changed ledge to Ledge, redge to Redge, top to Top, bot to Bot.
 * lines 254 & 255: changed aspot to UI to prevent int/pointer conflict.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 */
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include "center.h"


extern int fts;

char  *hh, hdr[HDSZ] ;
short *qq, *pp;
short  img [IMSZ], yimg [YDIM], syimg [YDIM], orig_yimg [YDIM];

float xcens[NSPOTS], *xc;
float sepy;
int   Ledge [NSPOTS], Redge [NSPOTS], *le, *re;
int   sepX[NSPOTS], *sX;
int   ODISK,BKRND,YCEN,H_BKRND;
int   B_BKRND,T_BKRND;

float numx, count;
float avgsepX, hisepX, losepX, Top, Bot;


dPcenter (filenm, xcen, ycen, radius, avgI) 
    char *filenm; float *xcen, *ycen, *radius, *avgI;
{
    char *w;
    short *real_img;
    int ii, jj, aspot;
    int isdisk=0, islimb=0;
    int cen_err;
    int spt;
    FILE *infid ;
    extern char *malloc();

    real_img = (short *)malloc (XDIM*YDIM*sizeof(short) );


    YCEN = (int)(YDIM*0.5);
    cen_err=0;


    w=filenm;


    spt  = (fts==1) ? 20   : 6 ;
    if ( *(w+strlen(filenm)-spt) == 'd' )  { isdisk=1; islimb=0; }
    if ( *(w+strlen(filenm)-spt) == 's' )  { isdisk=1; islimb=0; }
    if ( *(w+strlen(filenm)-spt) == 'l' )  { isdisk=0; islimb=1; }

    hh=hdr;

    infid = fopen(filenm, "r");
    fprintf(stderr, "Opening %s\n", filenm);
    ii=fread(hdr, 1, HDSZ, infid); 
    if (ii==HDSZ) jj=fread(real_img, 2, IMSZ, infid); 
    else          jj= -1;
    fclose(infid);
    if(jj!=IMSZ) {
	fprintf(stderr, "Full image not found, bytes read %d \n", jj);
    }
    else {
	ii  =  dPcenter_nofile(real_img, islimb, xcen, ycen, radius);
	if (ii==0) cen_err=1;

	/* Compute and avgI near the center of the *disk* image. */
	if(isdisk){
	    for(jj=-50;jj<50;jj++) {
		qq=real_img+ ((int)*ycen+jj) * XDIM + (int)*xcen-50;
		for(ii=-50;ii<50;ii++) *avgI += *qq++;
	    }
	    *avgI *= 0.0001;
	}

	if(islimb) { /* Get avgI in the background. */
	    for(jj=50;jj<150;jj++) {
		qq=real_img+ jj*XDIM + 50;
		for(ii=0;ii<100;ii++) *avgI += *qq++;
	    }
	    *avgI *= 0.0001;
	}
    }
    return(cen_err);
}
int 
statistics( pimg , isz , sigma ) short *pimg ; int  isz , *sigma;
{
    double dtmp , sx, sxx, sam_sz, davg, dsigma;
    int ii, jj, kk, hisz , iavg;
    short *pq ;

    hisz = isz/2;
    sx=sxx=sam_sz=0. ;
    for( jj=0;jj<isz;jj++) {
	pq = pimg + jj*XDIM  - ( XDIM*hisz + hisz ) ;
	for( ii=0;ii<isz;ii++) {
	    dtmp  =  (double) ( *pq );   pq++;
	    sx   +=  dtmp;
	    sxx  +=  dtmp*dtmp;
	    sam_sz++;
	}
    }
    davg  = sx/sam_sz;
    dsigma= ( sam_sz*sxx - sx*sx ) / ( sam_sz*(sam_sz-1.) ) ;
    dsigma= sqrt(dsigma);
    *sigma = (int)(dsigma + 0.5);
    iavg =   (int)(davg   + 0.5);
    return(iavg);

}




int
dPcenter_nofile (orig_img, type, xcen, ycen, radius)
    short *orig_img ; int type; float *xcen, *ycen, *radius;
{
    char *w;
    short *USE_IMG, *UI;
    int ii, jj, aspot;
    int isdisk=0, islimb=0, dosobel=0;
    int symx, symn;
    int *ss, sobel[11]={ -6, -2, 0, 2, 4, 4, 4, 2, 0, -2, -6}, hs=5, whs=11;
    int centerr;
    int doprint;
    int trial;
    int ulimit, llimit;
    int ckspill=1, spillpix;
    float DDIAM, newycen, lastycen;
    int   statistics(), disk_sigma, bkrnd_sigma;
    int   smooth_im();

    DDIAM = (fts==1) ? 656. : DIAM ;

    YCEN = (int)(YDIM*0.5);
    centerr=1;  /* successful center found, centerr=1 */
    doprint=1;

    trial=0;

    if (type==0)  { isdisk=1; islimb=0; }
    else          { isdisk=0; islimb=1; }

    qq=syimg; for(ii=0;ii<YDIM;ii++) *qq++=0;

    if(islimb) {
	qq=img;   for(ii=0;ii<IMSZ;ii++) *qq++=0;
	smooth_im(orig_img, img, 20);
    }
    else {
	qq=img; pp=orig_img;  for(ii=0;ii<IMSZ;ii++) *qq++=*pp++;
    }

    if(islimb) { USE_IMG=img;                   }
    else       { USE_IMG=orig_img; dosobel=1;   }

	ODISK=   statistics( (USE_IMG+400*XDIM+767) , 50 , &disk_sigma );
	H_BKRND= statistics( (USE_IMG+100*XDIM+100) , 50 , &bkrnd_sigma  );
	BKRND=   statistics( (USE_IMG+ 35*XDIM+400) , 50 , &bkrnd_sigma  );
	if(BKRND == -1) {
	    *xcen=1534.*0.5;
	    *ycen=1029.*0.5;
	    *radius=333.;
	    return(centerr);
	}
	if(doprint==1)
	fprintf(stderr, "BKRND %d +- %d  ODISK %d +- %d \n", BKRND, bkrnd_sigma, 
							   ODISK, disk_sigma);

	aspot=DROP*XDIM;
	qq=USE_IMG+aspot;  /* Go up a third of the rows. */
	le=Ledge;     re=Redge;        xc=xcens;
	sX=sepX;
	if (islimb) {
	    ulimit = (BKRND < ODISK+6*disk_sigma) ? BKRND : ODISK+6*disk_sigma;
	    llimit = ODISK;
	    for(jj=0;jj<NSPOTS;jj++){
		ii=767; qq+=767;
		while(*qq < ulimit           && ii>   0){ ii--; qq--; }
		*le = (ii==   0) ? -1 : ii ;
		ii+=(int)DDIAM-100; qq+=(int)DDIAM-100;
		while(*qq < ulimit           && ii<XDIM){ ii++; qq++; }
		*re = (ii==XDIM) ? -1 : ii ;
		*xc = ((*re - *le) *0.5 ) + *le;
		*sX =  (*re - *le) ;
		*sX =  (*sX < XDIM) ? *sX : 32767;
		re++; le++; xc++; sX++;
		qq = USE_IMG+aspot+jj*XDIM;
	    }
	}
	if (isdisk) {
	    llimit = BKRND +20*bkrnd_sigma ;
	    ulimit = ODISK;
	    for(jj=0;jj<NSPOTS;jj++){
		ii=10; qq+=10;
		while(*qq<llimit && ii<XDIM){ ii++; qq++; }
		*le = (ii==XDIM) ? -1 : ii ;
		ii+=(int)DDIAM-100; qq+=(int)DDIAM-100;
		while(*qq>llimit && ii<XDIM){ ii++; qq++; }
		*re = (ii==XDIM) ? -1 : ii ;
		*xc = ((*re - *le) *0.5 ) + *le;
		*sX =  (*re - *le) ;
		*sX =  (*sX < XDIM) ? *sX : 32767;
		re++; le++; xc++; sX++;
		qq = USE_IMG+aspot+jj*XDIM;
	    }
	}

	/* Search through the separations to toss the too little ones. */
	sX=sepX;
	avgsepX=DDIAM;
	hisepX=avgsepX+0.10*avgsepX; losepX=avgsepX-0.10*avgsepX;
	for(jj=0;jj<NSPOTS;jj++) {
	    *sX =  (*sX < hisepX && *sX > losepX ) ? *sX : 32767;
	    sX++;
	}

	xc=xcens;
	sX=sepX;
	numx=0.;  *xcen=0.; count=0.;
	for(jj=0;jj<NSPOTS;jj++){
	    if ( *sX < XDIM )  {
		*xcen += *xc;
		numx++;
	    }
	    xc++; sX++;
	}
	if ( numx != 0. ) *xcen = *xcen / numx;
	else {
	    centerr=0;
	    return(centerr);
	}

	sX=sepX;
	*radius=0.;
	for(jj=0;jj<NSPOTS;jj++) {
	    if(*sX<XDIM) *radius = (*radius > *sX*0.5) ? *radius : *sX*0.5;
	    sX++;
	}

	/* make a y row at xcen, and find the edges here. */
	if(dosobel) { USE_IMG=orig_img;  }
	else        { USE_IMG=img;       }

	UI = (USE_IMG + 100 * XDIM + (int)(*xcen) - (int)(DDIAM * 0.5)) ;
	BKRND=   statistics( UI , 50 , &bkrnd_sigma  );

	if(doprint==1)
	    fprintf(stderr, "Xcen adjusted BKRND %d +- %d \n", 
	    BKRND, bkrnd_sigma);


tryagain:
	jj = (islimb) ? (int)*xcen -80 +trial : (int)*xcen - 10 + 2*trial ;

	qq=yimg; pp=USE_IMG+jj;
	for(ii=0;ii<YDIM;ii++) { *qq = *pp;   qq++;   pp+=XDIM; }

	qq=orig_yimg; pp=orig_img+jj;
	for(ii=0;ii<YDIM;ii++) { *qq = *pp;   qq++;   pp+=XDIM; }

	if(dosobel) {
	    /* Apply a "fake" sobel to the line. */
	    pp=syimg+hs;
	    for(ii=hs;ii<YDIM-hs;ii++){
		ss=sobel;
		qq=yimg+ii-hs;
		for(jj=0;jj<whs;jj++) *pp += *ss++ * *qq++ ;
		pp++;
	    }
	    if(isdisk){
		/* Get min near bottom of the filtered line. */
		pp=syimg+hs+1+100;
		symn=0; 
		for(ii=0;ii<350;ii++){ symn= (symn < *pp) ? symn : *pp; pp++; }
		symn *= 0.9;
		ii=hs+1+100; pp=syimg+hs+1+100;   while( *pp++ > symn ) ii++;
						  while( *pp++ <    0 ) ii++; 
		Bot=(float)ii;

		/* Get min near top of the filtered line. */
		pp=syimg+YDIM-1-hs-1-100;
		symn=0; 
		for(ii=0;ii<300;ii++){ symn= (symn < *pp) ? symn : *pp; pp--; }
		symn *= 0.9;
		ii=YDIM-1-100; pp=syimg+YDIM-1-100;  while( *pp-- > symn ) ii--;
						     while( *pp-- <    0 ) ii--; 
		Top=(float)ii;
	    }
	    else      {
		/* Get min near bottom of the filtered line. */
		pp=syimg+hs+5+300;
		symn=0; 
		for(ii=300;ii>20;ii--){ symn= (symn < *pp) ? symn : *pp; pp--; }
		symn += 0.5;
		ii=hs+5+300; pp=syimg+hs+5+300;   while( *pp-- > symn ) ii--;
						  while( *pp-- <    0 ) ii--; 
		Bot=(float)ii;

		/* Get min near top of the filtered line. */
		pp=syimg+YDIM-hs-5-300;
		symn=0; 
		for(ii=0;ii<300;ii++){ symn= (symn < *pp) ? symn : *pp; pp++; }
		symn += 0.5;
		ii=YDIM-hs-5-300; pp=syimg+YDIM-hs-5-300;
					      while( *pp++ > symn ) ii++;
					      while( *pp++ <    0 ) ii++; 
		Top=(float)ii;
	    }
	}
	else {
	    qq=yimg;
	    if (islimb) {
		ulimit = (BKRND < ODISK+50*disk_sigma) ? 
			  BKRND : ODISK+50*disk_sigma;
		ulimit = (BKRND-6*bkrnd_sigma < ODISK+50*disk_sigma) ? 
			  BKRND-6*bkrnd_sigma : ODISK+50*disk_sigma;
		
		if(doprint==1) fprintf(stderr, "ulimit %d\n", ulimit);

		    ii=515; qq+=515;
		    while(*qq < ulimit           && ii>   0){ ii--; qq--; }
		    Bot = (ii==   0) ? -1 : ii ;
		    ii+=(int)DDIAM-100; qq+=(int)DDIAM-100;
		    while(*qq < ulimit           && ii<YDIM){ ii++; qq++; }
		    Top = (ii==YDIM) ? -1 : ii ;

		    if(ckspill==1) {
			if(doprint==1)fprintf(stderr, "B T %.1f %.1f\n", Bot, Top);
			qq=orig_yimg+(int)Bot+50;
			for(ii=Bot+50;ii>5;ii--) {
			    if(*qq-- > 1000) Bot=ii;
			}
			if(doprint==1) fprintf(stderr, "B spill %.1f  ", Bot);

			qq=orig_yimg+(int)Top-50;
			for(ii=Top-50;ii<YDIM-5;ii++) {
			    if(*qq++ > 1000) Top=ii;
			}
			if(doprint==1) fprintf(stderr, "T spill %.1f\n", Top);
		    }
	    }
	    else {
		llimit = BKRND +20*bkrnd_sigma ;
		ulimit = ODISK;

		    ii=10; qq+=10;
		    while(*qq<llimit && ii<XDIM){ ii++; qq++; }
		    Bot = (ii==XDIM) ? -1 : ii ;
		    ii+=(int)DDIAM-100; qq+=(int)DDIAM-100;
		    while(*qq>llimit && ii<XDIM){ ii++; qq++; }
		    Top = (ii==XDIM) ? -1 : ii ;
	    }
	}
	sepy=(Top-Bot);
	newycen  = (Top-Bot)*0.5 + Bot;
	if(sepy > DDIAM+0.06*DDIAM || sepy < DDIAM-0.06*DDIAM || trial==0 ) {
	    lastycen = (Top-Bot)*0.5 + Bot;
	    if(doprint==1 && trial==0 ) 
		fprintf(stderr, "Trying again   ");
	    if(doprint==1 ) 
		fprintf(stderr, "Trial %d -- ycen %.1f - %.1f %.1f\n", 
		trial, newycen, Top, Bot);
	    if(trial < 10 ) { trial++; goto tryagain;}
	    if(doprint==1) fprintf(stderr, 
	    "Rats, y-diam %.1f vs %.1f, Top %.1f Bot %.1f at xcen %.1f\n"
		, sepy, DDIAM, Top, Bot, *xcen);

	    centerr=0;
	}
	if(newycen >= lastycen+2.  || newycen <= lastycen-2. ) {
	    if(doprint==1 ) 
		fprintf(stderr, "TrialDDD %d -- ycen %.1f\n", trial, newycen);
	    if(trial < 10 ) { lastycen = newycen;  trial++; goto tryagain;}
	    if(doprint==1) fprintf(stderr, 
	    "Rats, ycens %.1f %.1f differ, Top %.1f Bot %.1f at xcen %.1f\n"
		, lastycen, newycen, Top, Bot, *xcen);

	    centerr=0;
	}

	*ycen=(Top-Bot)*0.5 + Bot;

	if(doprint==1) fprintf(stderr, "xcen %.1f ycen %.1f \n", *xcen, *ycen);

    return(centerr);
}
int
h_bkrnd(pimg, limit) short *pimg , limit;
{
    int ii;
    for(ii=0;ii<IMSZ;ii++) *pimg++ = (*pimg > limit) ? limit : *pimg;
}

int
smooth_im(o_im, s_im, slen) short *o_im, *s_im; int slen;
{
    register short *tt;
    register int ii , jj , kk ;
    int start, end ;
    float ovslen;
    int printme = 0 ;

    if(printme) fprintf(stderr, "In smooth_im %d\n", slen);

    /* slen *must* be even */
    ovslen = 1. / (float)(slen) ;

    start =          XDIM + slen/2  ;
    end   = IMSZ - ( XDIM + slen/2 );
    pp=o_im;  qq=s_im+start;
    for( kk=start;kk<end;kk++) {
	
	tt=pp; for(ii=0;ii<slen;ii++) *qq += *tt++;
	*qq *= ovslen ;
	qq++ ; pp++;
    }
}
