/******************************************************************************
    Name:    device.h

    Purpose:    Structure to register where to drive a device,
        how to stop it, what is its timeout length in
        1/125 sec increments, and what are names of
        postions for the device.

    Author:    David Elmore

    Date:    Mon Aug 21 13:26:28 MDT 1995

******************************************************************************/

/* Position is used to set desired position, direction, and names for them */

struct device {
    int     goal;            /* Target position to which to drive */
    int     dir;             /* Direction we are driving */
    int     num_pos;         /* Number of positions for device */
    int     stop;            /* Stop bits to set upon arrival */
    int     wrap;            /* Can we wrap around through 0 pos? */
    int     timeout;         /* Number of 1/100s of a second to wait */
    int     stat_map[16];    /* Map of real bits to logical stat bits */
    int     drive_map[4];    /* Map of real bits to logical drv bits */
			     /* If in is out or drive is backward */
			     /*          change the above tables */
    char     *name;          /* The name of the device */
    char     *stat_name[9];  /* Names of the 8 possible positions skipping 0*/
    char     *drive_name[4]; /* Names of the 4 possible drive types */
    };

#ifdef START

struct device dev[NUM_DEV] = {
        0,0,2,COAST,NO_ZERO,1250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,2,1,3,
	    "Filter Stage",
	    "Unknown","In","Out","Error","?","?","?","?","?",
	    "Stop","In","Out","Stop",
        0,0,2,COAST,NO_ZERO,1250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,2,1,3,
	    "O2 Stage",
	    "Unknown","LowK","dPMon","Error","?","?","?","?","?",
	    "Stop","Lowk","dPMon","Stop",
        0,0,2,COAST,NO_ZERO,1500,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,2,1,3,
	    "CHIP Cover",
	    "Unknown","In","Out","Error","?","?","?","?","?",
	    "Stop","In","Out","Stop",
        0,0,2,COAST,NO_ZERO,1500,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,2,1,3,
	    "CHIP Diffuser",
	    "Unknown","In","Out","Error","?","?","?","?","?",
	    "Stop","In","Out","Stop",

        0,0,2,COAST,NO_ZERO,1250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,1,2,3,
	    "Camera Strobe",
	    "None","CHIP","dPMon/LowK","Both","?","?","?","?","?",
	    "?","?","?","?",
        0,0,6,COAST,TIMER,250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,1,2,3,
	    "dPMon/LowK O1-S/N",
	    "?","VVV","VV","V","^","^^","^^^","?","?",
	    "Stop","South","North","Stop",
        0,0,6,COAST,TIMER,250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,1,2,3,
	    "dPMon/LowK O1-E/W",
	    "?","<<<","<<","<",">",">>",">>>","?","?",
	    "Stop","East","West","Stop",
        0,0,2,COAST,NO_ZERO,1250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,2,1,3,
	    "Cover",
	    "Unknown","In","Out","Error", "?","?","?","?","?",
	    "Stop","In","Out","Stop",
        0,0,2,COAST,NO_ZERO,1250,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,
	    0,2,1,3,
	    "Polarizer Stage",
	    "Unknown","In","Out","Error", "?","?","?","?","?",
	    "Stop","In","Out","Stop",

        0,0,8,COAST,EITHER,1250,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,
	    0,1,2,3,
	    "Filter Wheel",
	    "?","654.0","Dark","Dark","Dark",
		"657.1","655.5","656.3","654.7",
	    "Stop","CCW","CW","Stop",
        0,0,8,COAST,POS_ONLY,1250,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,
	    0,1,2,3,
	    "Polarizer Wheel",
	    "?","0","45","90","135", "180","225","270","315",
	    "Stop","CCW","CW","Stop",

        0,0,4,COAST,EITHER,1250,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,
	    0,1,2,3,
	    "Objective Wheel",
	    "?","Opal","Diffuser","Clear","Clear",
		"Error","Error","Error","Error",
	    "Stop","CCW","CW","Stop",
        0,0,6,COAST,EITHER,2000,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,
	    0,1,2,3,
	    "Occulting Wheel",
	    "?","Wedge","Lowk","Clear","Offset", "Occult","Reticule",
		"Error","Error",
	    "Stop","CCW","CW","Stop"
        };

int    net_wait[NUM_DEV] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
#else

extern    struct    device dev[NUM_DEV];
extern    int    net_wait[NUM_DEV];

#endif

