/* dpmtofits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert MLSO Digital Prominence Monitor (DPM) image to FITS format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   1 March 1994
 *  4 Sep 1997: change cdelt1/2 from 3.006 to 2.90
 * 16 Sep 1997: derive wmin & wmax values as a function of vmin/vmax. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include "/home/stanger/lib/src/dpmnew/dpmobs.h"
#include "/home/stanger/lib/src/dpmnew/dpmlib.h"
#define MAXLIN 180	/* Maximum # lines allocated for FITS header. */

int
dpmtofits (namimg, namext, hduflag, pmode, nodel)

   char namimg [] ;		/* Name of primary file. */

   char namext [] ;		/* Name of extension file. */

   int hduflag ;		/* 1: primary image ; no extension.        */
				/* 2: primary image ; extension exists.    */
				/* 3: extension image ; append to namfits. */

   int pmode ;			/* Print mode. */

   int nodel ;                  /* Whether or no to delete the file after
				 * conversion to fits. */

{
   extern char *malloc () ;		/* Function to allocate memory. */
   extern unsigned char *rddpm () ;	/* Function to read DPM image. */
   extern int ckifexists () ;		/* Routine  to check file existence. */
   extern int day_of_year () ;		/* Returns DOY. */
   extern int cdatim () ;		/* Returns current date & time. */
   extern int rot180 () ;		/* Rotate image 180 degrees. */
   int dpmext () ;			/* DPM extension. */

   unsigned char *padbuf, *qq ;			/* Pointers. */
   unsigned char *dpmbuf ;		/* MLSO DPM file  array pointer. */
   unsigned char *imgbuf ;		/* MLSO DPM image array pointer. */

   unsigned char *cibuf ;		/* unsigned char image pointers. */
   unsigned char cimin, cimax ;		/* Data min/max values. */
   unsigned char cival ;		/* Data value. */

   int do_yrddoy = 1 ;   /* Set this to 1 if you want yrddoy names,
                          * Set it to   0 if you do not.
                          */
   int do_rot180 = 1 ;   /* Set this to 1 if you need to rotate an image
                          * by 180 degrees.  (dPmon)
                          * Set it to   0 if you do not.
                          */

   static char fitbuf    [MAXLIN][80] ;		/* FITS header buffer. */
   static char kairmass  [12] = "AIRMASS " ;
   static char kangnorth [12] = "ANGNORTH" ;
   static char kazimuth  [12] = "AZIMUTH " ;
   static char kbitpix   [12] = "BITPIX  " ;
   static char kbscale   [12] = "BSCALE  " ;
   static char kbzero    [12] = "BZERO   " ;
   static char kbunit    [12] = "BUNIT   " ;
   static char kcdelt1   [12] = "CDELT1  " ;
   static char kcdelt2   [12] = "CDELT2  " ;
   static char kcode_obs [12] = "CODE-OBS" ;
   static char kcrota1   [12] = "CROTA1  " ;
   static char kcrota2   [12] = "CROTA2  " ;
   static char kcrpix1   [12] = "CRPIX1  " ;
   static char kcrpix2   [12] = "CRPIX2  " ;
   static char kcrradius [12] = "CRRADIUS" ;
   static char kcrval1   [12] = "CRVAL1  " ;
   static char kcrval2   [12] = "CRVAL2  " ;
   static char kctype1   [12] = "CTYPE1  " ;
   static char kctype2   [12] = "CTYPE2  " ;
   static char kcunit1   [12] = "CUNIT1  " ;
   static char kcunit2   [12] = "CUNIT2  " ;
   static char kcoordnam [12] = "COORDNAM" ;
   static char kdatamax  [12] = "DATAMAX " ;
   static char kdatamin  [12] = "DATAMIN " ;
   static char kdate     [12] = "DATE    " ;
   static char kdate_cal [12] = "DATE-CAL" ;
   static char kdate_obs [12] = "DATE-OBS" ;
   static char kocc_disc [12] = "OCC-DISC" ;	/* ? */
   static char kdataform [12] = "DATAFORM" ;
   static char kdatatype [12] = "DATATYPE" ;	/* ? */
   static char kdisp_dir [12] = "DISP-DIR" ;
   static char kdispmax  [12] = "DISPMAX " ;
   static char kdispmin  [12] = "DISPMIN " ;
   static char kelev     [12] = "ELEV    " ;
   static char kend      [12] = "END     " ;
   static char kexpdur   [12] = "EXPDUR  " ;
   static char kextend   [12] = "EXTEND  " ;
   static char kgcount   [12] = "GCOUNT  " ;
   static char khour_ang [12] = "HOUR-ANG" ;
   static char kinstrume [12] = "INSTRUME" ;
   static char knaxis    [12] = "NAXIS   " ;
   static char knaxis1   [12] = "NAXIS1  " ;
   static char knaxis2   [12] = "NAXIS2  " ;
   static char knimsum   [12] = "NIMSUM  " ;
   static char kobject   [12] = "OBJECT  " ;
   static char korigin   [12] = "ORIGIN  " ;
   static char kpcount   [12] = "PCOUNT  " ;
   static char krsun     [12] = "RSUN    " ;
   static char ksimple   [12] = "SIMPLE  " ;
   static char ksolar_b0 [12] = "SOLAR_B0" ;
   static char ksolar_l0 [12] = "SOLAR_L0" ;
   static char ksolar_p0 [12] = "SOLAR_P0" ;
   static char ktelescop [12] = "TELESCOP" ;
   static char ktime     [12] = "TIME    " ;
   static char ktime_obs [12] = "TIME-OBS" ;
   static char ktype_dp  [12] = "TYPE-DP " ;
   static char ktype_obs [12] = "TYPE-OBS" ;
   static char kvers_sw  [12] = "VERS-SW " ;
   static char kwavelen  [12] = "WAVELEN " ;
   static char kwavewid  [12] = "WAVEWID " ;
   static char kxtension [12] = "XTENSION" ;
   static char krcode    [4]  = "?" ;
   static char kscode    [4]  = "?" ;
   static char kwcode    [4]  = "?" ;

   static char atype1   [20] ;			/* Coord type axis 1. */
   static char atype2   [20] ;			/* Coord type axis 2. */
   static char bunit    [ 8] = "CCD" ;		/* Brightness unit. */
   static char code_obs [ 4] = "xxx" ;		/* Observation code. */
   static char ctype1   [12] = "X" ;		/* Coord type axis 1. */
   static char ctype2   [12] = "Y" ;		/* Coord type axis 2. */
   static char ctypenam [16] = "GEOCENTRIC" ;	/* Coordinate system name. */
   static char cunit1   [ 8] = "ARCSEC" ;	/* Coord units axis 1. */
   static char cunit2   [ 8] = "ARCSEC" ;	/* Coord units axis 2. */
   static char datcom   [ 8] = "CCD" ;		/* Data min/max comment. */
   static char date     [12] = "DD/MM/YY" ;	/* File creation date. */
   static char date_cal [12] = "DD/MM/YY" ;	/* Calibration date. */
   static char date_obs [12] = "DD/MM/YY" ;	/* Observation date. */
   static char disp_dir [ 4] = "RU" ;	/* Display direction: R/L, U/D, O/I. */
   static char dform    [16] = "UNKNOWN" ;	/* Data form. */
   static char eq       [ 4] = "= " ;		/* Equal sign + space. */
   static char instrume [ 4] = "CCD" ;		/* Instrument. */
   static char namfit   [80] = "img.fts" ;	/* FITS file name. */
   static char object   [12] = "SUN" ;		/* Object. */
   static char origin   [12] = "MLSO" ;		/* Data origin. */
   static char origincom[52] ;			/* Data origin comment. */
   static char outbuf   [84] ;			/* Line buffer. */
   static char pixcom   [24] ;			/* Data type comment. */
   static char refcom   [12] = "SUN CENTER" ;	/* Coord ref comment. */
   static char telescop [ 4] = "DPM" ;		/* Telescope name. */
   static char time     [12] = "HH:MM:SS" ;     /* File creation time. */
   static char time_obs [12] = "HH:MM:SS" ;	/* Observation time. */
   static char true     [ 4] = "T" ;		/* True. */
   static char type_dp  [ 4] = "RAW" ;		/* Type of data processing. */
   static char type_obs [12] = "HALPHA" ;	/* Type of observation. */
   static char vers_sw  [12] = "DPM V3.00" ;	/* S/W version. */
   static char wave     [12] ;
   static char source   [12] ;
   static char reduce   [12] ;
   static int ic [7] ;                  /* Array containing date & time:
                                         * year, doy, day, month,
                                         * hour, minute, second.
                                         */

   short *sibuf ;			/* Short image pointer. */
   short simax	= 0 ;			/* display max value. */
   short simin	= 0 ;			/* display min value. */
   short sival	= 0 ;			/* temporary storage. */

   int bitpix = 16 ;			/* # bits per pixel. */
   int doy_obs ;			/* Observation day-of-year. */
   int duplicate_hdr ;   /* Some times the header does not get refreshed (ick).*/
   int gcount = 1 ;			/* gcount. */
   int iyear ;				/* year - 1900. */
   int i, ii, ix, iy ;			/* Indices. */
   int icyear ;				/* year. */
   int icdoy ;				/* day-of-year. */
   int icday ;				/* day-of-month. */
   int icmonth ;			/* month. */
   int ichour ;				/* hour. */
   int icmin ;				/* minute. */
   int icsec ;				/* second. */
   int ival ;				/* Integer value. */
   int ndim	= 0 ;			/* # dimensions for image (2=XY). */
   int nhdu ;                           /* # HDU blocks to write. */
   int nbyimg ;				/* # bytes in image array. */
   int nbyfits	= 2880 ;		/* # bytes per FITS logical record. */
   int npximg ;				/* # pixels in image. */
   int npad ;				/* # bytes to pad file. */
   int pcount	= 0 ;			/* pcount. */
   int nrec ;				/* # fits records for image. */
   int nelwr	= 0 ;
   int rcode	= 0 ;			/* Reduction code: Undefined. */
   int scode	= 0 ;			/* Source code:    Undefined. */
   int wcode	= 0 ;			/* Wavelength code: Undefined. */
   int year_obs ;			/* Observation year. */
   int znum ;				/* # lines in FITS header. */
   int zz ;				/* fitbuf index. */

   float *fibuf ;			/* Float image pointer. */
   float fmin, fmax ;			/* Data min/max values. */
   float fval ;				/* Data value. */

   float angnorth	=  90.0 ;	/* solar north w.r.t. +X axis. */
   float cdelt1		=  0.0 ;	/* Physical coord: increment X-axis. */
   float cdelt2		=  0.0 ;	/* Physical coord: increment Y-axis. */
   float crpix1		=  0.0 ;	/* Physical coord: index     X-axis. */
   float crpix2		=  0.0 ;	/* Physical coord: index     Y-axis. */
   float crota1		=  0.0 ;	/* Physical coord: rotation  X-axis. */
   float crota2		=  0.0 ;	/* Physical coord: rotation  Y-axis. */
   float crval1		=  0.0 ;	/* Physical coord: value     X-axis. */
   float crval2		=  0.0 ;	/* Physical coord: value     Y-axis. */
   float crradius	=  0.0 ;	/* Solar radius [pixels]. */
   float expdur		=  0.0 ;	/* Exposure duration (seconds). */
   float occ_disc	=  0.0 ;	/* Occulting disc diameter. */
   float vmax		=  0.0 ;	/* Data    max value. */
   float vmin		=  0.0 ;	/* Data    min value. */
   float wmax		=  0.0 ;	/* Display max value. */
   float wmin		=  0.0 ;	/* Display min value. */
   float wavelen	=  0.0 ;	/* Wavelength (nm). */
   float wavewid	=  0.0 ;	/* Bandpass   (nm). */
   float xcenter	= -1.0 ;	/* X center. */
   float ycenter	= -1.0 ;	/* Y center. */
   float diam		=  0.0 ;	/* Disk diameter. */

   FILE *FF ;				/* FITS file ID. */

   sprintf (namfit, "%s.fts", namimg) ;

   /*** Read MLSO DPM image/data from disk into memory array. ***/

   if (hduflag == 1 || hduflag == 2)
      dpmbuf = rddpm (namimg, pmode) ;	/* Read DPM dataset into memory. */
   else
      dpmbuf = rddpm (namext, pmode) ;	/* Read DPM dataset into memory. */

/*   strcpy (namfit, ((do_yrddoy) ? dpm_yrddoy_name () : dpm_name ())) ; */

   if (pmode)
      {
      fprintf (stdout, "dpmtofits> namimg: %s  namext: %s   hduflag: %d\n",
	       namimg, namext, hduflag) ;

      fprintf (stdout, "\n------- dpmtofits> %s -> %s -------\n\n",
	       namimg, namfit) ;
      }
 
/*
 *   duplicate_hdr = ckifexists (namimg, namfit) ;
 */

/*
 *   if ((duplicate_hdr == 0) && ( nodel == 0 )) remove_file (namimg) ;
 *
 *   if ( duplicate_hdr == 1)
 *      {
 *      fprintf (stderr, "\tNo fits file will be produced.\n") ;
 *      return (-1) ;
 *      }
 */

   npximg = DPMI->xdim * DPMI->ydim ;	/* # pixels in image. */
   nbyimg = npximg * DPMI->nbypx ;	/* # bytes in image. */

   if (pmode)
      fprintf (stdout, "xdim:%3d  ydim:%3d  nbypx:%d  npximg:%6d  nbyimg:%6d\n",
			DPMI->xdim, DPMI->ydim, DPMI->nbypx, npximg, nbyimg) ;

   /*** Find min/max data values. ***/
   /*** Find disk center for short integer images. ***/

   imgbuf = dpmbuf ;

   switch (DPMI->nbypx)
      {
      case 1:
         cibuf = (unsigned char *) imgbuf ;
         cimin  = 255 ;
         cimax  =   0 ;
         for (iy = 0 ; iy < DPMI->ydim ; iy++)
            {
            for (ix = 0 ; ix < DPMI->xdim ; ix++)
               {
               cival = *(cibuf + ix + iy*DPMI->xdim) ;
               if (cival < cimin)  cimin = cival ;
               if (cival > cimax)  cimax = cival ;
               }
            }
         vmin = (float) cimin ;
         vmax = (float) cimax ;
         break ;

      case 2:
         sibuf = (short *) imgbuf ;

         simin  =  32767 ;
         simax  = -32767 ;

	 /* Exclude perimeter zone (100 pixels wide) in computing min/max. */

         sibuf = (short *) imgbuf ;
         for (iy = 100 ; iy < DPMI->ydim-100 ; iy++)
            {
            for (ix = 100 ; ix < DPMI->xdim-100 ; ix++)
               {
               sival = *(sibuf + ix + iy*DPMI->xdim) ;
               if (sival < simin)  simin = sival ;
               if (sival > simax)  simax = sival ;
/*
 *               simin = ( sival < simin && sival >  -1000 ) ? sival : simin ;
 *               simax = ( sival > simax && sival <      0 ) ? sival : simax ;
 */
               }
            }

         vmin = (float) simin ;
         vmax = (float) simax ;
         break ;

      case 4:
         fibuf = (float *) imgbuf ;
         fmin  =  1.0e+30 ;
         fmax  = -1.0e+30 ;
         for (iy = 0 ; iy < DPMI->ydim ; iy++)
            {
            for (ix = 0 ; ix < DPMI->xdim ; ix++)
               {
               fval = *(fibuf + ix + iy*DPMI->xdim) ;
               if (fval < fmin)  fmin = fval ;
               if (fval > fmax)  fmax = fval ;
               }
            }
         vmin = fmin ;
         vmax = fmax ;
         break ;

      default:
         fprintf (stderr, 
	    "Invalid DPM image . . . (numnber of bytes per pixel) nbypx: %d\n",
	    DPMI->nbypx) ;
         exit (-1) ;
      }
   vmin = vmin * DPMI->bscale + DPMI->bzero ;
   vmax = vmax * DPMI->bscale + DPMI->bzero ;
   DPMI->dmin = (int) vmin ;
   DPMI->dmax = (int) vmax ;

   /*** Set image attributes based upon data type. ***/

   switch (DPMI->dtype)
      {
      case  0: /* other */
               strcpy (object, "UNDEF") ;   
               strcpy (dform,  "OTHER") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax = 4000.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 0.0 ;
	       wavewid = 0.0 ;
	       wcode = 9 ;		/* WAVELENGTH_s */
	       scode = 0 ;		/* SOURCE_x     */
	       rcode = 0 ;		/* REDUCTION_x  */
	       strcpy (code_obs, "sxx") ;
               break ;
      case  1: /* disc */
               strcpy (object, "SUN") ;   
               strcpy (dform,  "DISC") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax = 1500.0 ;
               wmin    = 2.00 * vmin ;   wmax = vmax * 1.0 ;
	       wavelen = 653.6 ;
	       wavewid =   0.07 ;	/* Daystar filter [nm units]. */
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 1 ;		/* SOURCE_s     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "asr") ;
               break ;
      case  2: /* limb */
               strcpy (object, "LIMB") ;   
               strcpy (dform,  "LIMB") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax =  4000.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 0.75 ;
	       wavelen = 653.6 ;
	       wavewid =   0.8 ;	/* nm units. */
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 2 ;		/* SOURCE_l     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "alr") ;
               break ;
      case  3: /* ref */
               strcpy (object, "SKY") ;   
               strcpy (dform,  "REF") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax = 4000.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 658.1 ;
	       wavewid =   0.8 ;	/* Continuum. */
	       wcode = 9 ;		/* WAVELENGTH_s */
	       scode = 2 ;		/* SOURCE_l     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "slr") ;
               break ;
      case  4: /* diffuser */
               strcpy (object, "DIFFUSER") ;   
               strcpy (dform,  "DIFFUSER") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax = 4000.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 653.6 ;
	       wavewid =   0.8 ;
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 4 ;		/* SOURCE_d     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "adr") ;
               break ;
      case  5: /* disc_dark */
               strcpy (object, "DARK") ;   
               strcpy (dform,  "DISC_DARK") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax =  500.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 653.6 ;
	       wavewid =   0.07 ;
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 5 ;		/* SOURCE_b     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "abr") ;
               break ;
      case  6: /* limb_dark */
               strcpy (object, "DARK") ;   
               strcpy (dform,  "LIMB_DARK") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax =  500.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 653.6 ;
	       wavewid =   0.8 ;
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 5 ;		/* SOURCE_b     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "abr") ;
               break ;
      case  7: /* diffuser_dark */
               strcpy (object, "DARK") ;   
               strcpy (dform,  "DIFFUSER_DARK") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax =  500.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 653.6 ;
	       wavewid =   0.8 ;
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 5 ;		/* SOURCE_b     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "abr") ;
               break ;
      case  8: /* ref_dark */
               strcpy (object, "DARK") ;   
               strcpy (dform,  "REF_DARK") ;
               strcpy (pixcom, "2-BYTE SIGNED PIXELS") ;
               wmin    = 0.0 ;   wmax = 4000.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 653.6 ;
	       wavewid =   0.8 ;
	       wcode = 5 ;		/* WAVELENGTH_a */
	       scode = 5 ;		/* SOURCE_b     */
	       rcode = 1 ;		/* REDUCTION_r  */
	       strcpy (code_obs, "abr") ;
               break ;
      default:
               strcpy (object, "UNKNOWN") ;   
               strcpy (dform,  "UNKNOWN") ;
               wmin =    0.0 ;   wmax = 1024.0 ;
               wmin    = 1.00 * vmin ;   wmax = vmax * 1.00 ;
	       wavelen = 0.0 ;
	       wavewid = 0.0 ;
	       wcode = 0 ;		/* WAVELENGTH_x */
	       scode = 0 ;		/* SOURCE_x     */
	       rcode = 0 ;		/* REDUCTION_x  */
	       strcpy (code_obs, "xxx") ;
	       fprintf (stderr, "Invalid dtype: %d\n", DPMI->dtype) ;
	       exit (-1) ;
      }
   wmin = wmin * DPMI->bscale + DPMI->bzero ;
   wmax = wmax * DPMI->bscale + DPMI->bzero ;
   DPMI->smin = (int) wmin ;
   DPMI->smax = (int) wmax ;

   /* Rotate the image by 180 degrees. */

   if (DPMI->nbypx == 2 && do_rot180 && (rcode != REDUCTION_h))
      {
      sibuf = (short *) imgbuf ;
      rot180 (sibuf, DPMI->xdim, DPMI->ydim) ;
      }

   iyear = DPMI->year ;
   if (iyear > 1900)  iyear -= 1900 ;
   year_obs = iyear + 1900 ;
   doy_obs  = day_of_year (iyear, DPMI->month, DPMI->day) ;

   sprintf (date_obs, "%02d/%02d/%02d", DPMI->day, DPMI->month, iyear) ;
   sprintf (time_obs, "%02d:%02d:%02d",
	    DPMI->hour, DPMI->minute, DPMI->second) ;

   /*** Get current date & time. ***/

   cdatim (ic) ;
   icyear       = ic [0] ;
   icdoy        = ic [1] ;
   icday        = ic [2] ;
   icmonth      = ic [3] ;
   ichour       = ic [4] ;
   icmin        = ic [5] ;
   icsec        = ic [6] ;
   sprintf (date, "%02d/%02d/%02d", icday,  icmonth, icyear) ;
   sprintf (time, "%02d:%02d:%02d", ichour, icmin,   icsec) ;

   iyear = DPMI->cal_year ;
   if (iyear > 1900)  iyear -= 1900 ;
   sprintf (date_cal, "%02d/%02d/%02d", DPMI->cal_day, DPMI->cal_month, iyear) ;

   /*** Set number of dimensions. ***/

   ndim = 0 ;
   if (DPMI->xdim > 0)
      ndim++ ;
   if (DPMI->ydim > 1)
      ndim++ ;

   /*
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    * Set parameters.
    * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    */

   strcpy (atype1,   "X [EAST->WEST]") ;
   strcpy (atype2,   "Y [SOUTH->NORTH]") ;
   strcpy (origincom, "MAUNA LOA SOLAR OBSERVATORY") ;

   crpix1 = -1.0 ;				/* Sun center (X pixels). */
   crpix2 = -1.0 ;				/* Sun center (Y pixels). */
   crval1 = 0.0 ;				/* Sun center (X arcsec). */
   crval2 = 0.0 ;				/* Sun center (Y arcsec). */
   cdelt1 = 2.900 ;				/* arcsec/pixel. */
   cdelt2 = 2.900 ;				/* arcsec/pixel. */
   crota1 = 0.0 ;				/* Rotation angle: X-axis. */
   crota2 = 0.0 ;				/* Rotation angle: Y-axis. */

   crradius = DPMI->rsun / cdelt1 ;		/* Solar radius (pixels). */
   expdur   = (float) DPMI->expdur / 100.0 ;	/* Exposure duration (sec). */
   occ_disc = (float) DPMI->odisk ;		/* Occulting disk (arcsec). */

   /*** Set image type and miscellaneous information codes according to the
    *** DPM image type parameter.
    ***/

   switch (wcode)
      {
      case  WAVELENGTH_x:  strcpy (wave, "UNDEF") ;      strcpy (kwcode, "x") ;
			   break ;
      case  WAVELENGTH_h:  strcpy (wave, "HELIUM-I") ;   strcpy (kwcode, "h") ;
			   break ;
      case  WAVELENGTH_c:  strcpy (wave, "HE-CONT") ;    strcpy (kwcode, "c") ;
			   break ;
      case  WAVELENGTH_m:  strcpy (wave, "HE-MINOR") ;   strcpy (kwcode, "m") ;
			   break ;
      case  WAVELENGTH_w:  strcpy (wave, "H2O") ;        strcpy (kwcode, "w") ;
			   break ;
      case  WAVELENGTH_a:  strcpy (wave, "HALPHA") ;     strcpy (kwcode, "a") ;
			   break ;
      case  WAVELENGTH_b:  strcpy (wave, "HA-BLUE") ;    strcpy (kwcode, "b") ;
			   break ;
      case  WAVELENGTH_r:  strcpy (wave, "HA-RED") ;     strcpy (kwcode, "r") ;
			   break ;
      case  WAVELENGTH_k:  strcpy (wave, "KCORONA") ;    strcpy (kwcode, "k") ;
			   break ;
      case  WAVELENGTH_s:  strcpy (wave, "OTHER") ;      strcpy (kwcode, "s") ;
			   break ;
      default:             strcpy (wave, "UNDEF") ;      strcpy (kwcode, "?") ;
      }

   switch (scode)
      {
      case  SOURCE_x:  strcpy (source, "UNDEF") ;    strcpy (kscode, "x") ;
		       break ;
      case  SOURCE_s:  strcpy (source, "SUN") ;      strcpy (kscode, "s") ;
		       break ;
      case  SOURCE_l:  strcpy (source, "LIMB") ;     strcpy (kscode, "l") ;
		       break ;
      case  SOURCE_o:  strcpy (source, "OPAL") ;     strcpy (kscode, "o") ;
		       break ;
      case  SOURCE_d:  strcpy (source, "DIFFUSER") ; strcpy (kscode, "d") ;
		       break ;
      case  SOURCE_b:  strcpy (source, "DARK") ;     strcpy (kscode, "b") ;
		       break ;
      case  SOURCE_c:  strcpy (source, "CAL-OPT") ;  strcpy (kscode, "c") ;
		       break ;
      case  SOURCE_f:  strcpy (source, "FLAT-FLD") ; strcpy (kscode, "f") ;
		       break ;
      default:         strcpy (source, "UNKNOWN") ;  strcpy (kscode, "?") ;
      }

   switch (rcode)
      {
      case  REDUCTION_x:  strcpy (reduce, "UNDEF") ;    strcpy (krcode, "x") ;
                strcpy (bunit,  "CCD") ;       break ;
      case  REDUCTION_r:  strcpy (reduce, "RAW") ;      strcpy (krcode, "r") ;
                strcpy (bunit,  "CCD") ;       break ;
      case  REDUCTION_c:  strcpy (reduce, "CAL") ;      strcpy (krcode, "c") ;
                strcpy (bunit,  "PB") ;        break ;
      case  REDUCTION_l:  strcpy (reduce, "LD-COR") ;   strcpy (krcode, "l") ;
                strcpy (bunit,  "PB") ;        break ;
      case  REDUCTION_d:  strcpy (reduce, "RATIO") ;    strcpy (krcode, "d") ;
                strcpy (bunit,  "NONE") ;      break ;
      case  REDUCTION_m:  strcpy (reduce, "CONTRAST") ; strcpy (krcode, "m") ;
                strcpy (bunit,  "PB") ;        break ;
      case  REDUCTION_h:  strcpy (reduce, "LD-COR") ;   strcpy (krcode, "h") ;
                strcpy (bunit,  "PB") ;        
	        strcpy (ctypenam, "HELIOCENTRIC") ;
		break ;
      default:  strcpy (reduce, "UNKNOWN") ;  strcpy (krcode, "?") ;
                strcpy (bunit,  "CCD") ;
      }

   sprintf (code_obs, "%s%s%s", kwcode, kscode, krcode) ;
   strcpy (object,   source) ;
   strcpy (type_obs, wave) ;
   strcpy (type_dp,  reduce) ;

   /*** Get wavelength from position of optical device #4. ***/

   sscanf (&DPMI->optpos [3][0], "%f", &wavelen) ;


    /********************************************************************/
    /*** Find disk center.  Exclude housekeeping line. ***/
    /*** Alice put the housekeeping line back in -- just to keep
         things consistent with the calibration code. */
    /*** It looks like after Jan 1, 1997, the dPmon disk images are
	 centered on the fly at MLSO when they are captured.
	 So, the findcen below is skipped for these images,
	 and the image center set to 767.5,514.5                   */
    /*** Ha! But it turns out that the on the fly code fails on sharp dust
	 bit, so I have to redo it and try to do it better aDPMI->ydimway! */

	 if (rcode != REDUCTION_h &&
	    (scode != SOURCE_d && scode != SOURCE_b && scode != SOURCE_c))
	    {
            sibuf = (short *) imgbuf ;
            diam = crradius * 2.0 ;

            findcen (sibuf, DPMI->xdim, DPMI->ydim, &xcenter, &ycenter, &diam, scode) ;
            }
         else
            {
	    xcenter = -1.0 ;
	    ycenter = -1.0 ;
            }
    crpix1 = xcenter ;
    crpix2 = ycenter ;
    /********************************************************************/

   /*** Get wavelength from position of optical device #4. ***/

   sscanf (&DPMI->optpos [3][0], "%f", &wavelen) ;

   /*** Create FITS file. ***/

   if (hduflag == 3)				/* Extension dataset. */
      {
      if ((FF = fopen (namfit, "a")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }
   else						/* Primary dataset. */
      {
      if ((FF = fopen (namfit, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namfit) ;
         exit (-2) ;
         }
      }

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < MAXLIN ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- PRIMARY HDU ---\n") ;

   /*** Fill FITS primary header. ***/

   zz = 0 ;

   /*** Required FITS keywords. ***/

   if (hduflag == 1 || hduflag == 2)
      {
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20s", ksimple  , eq, true) ;
      strcpy  (&fitbuf [zz++][31], "/ FITS FILE: HAO/MLSO DIGITAL PROMINENCE MONITOR") ;
      }
   else
      {
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kxtension, eq, dform) ;
      strcpy  (&fitbuf [zz++][31], "/ MLSO DPM %s EXTENSION", dform) ;
      }

   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, DPMI->xdim, ctype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, DPMI->ydim, ctype2) ;
/*
 *   else
 *      strcpy (&fitbuf  [zz++][ 0], "                                                                                ") ;
 */

   /*** Extension information. ***/

   if (hduflag == 3)
      {
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / EXTENSION PCOUNT",
	                          kpcount, eq, pcount) ;
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / EXTENSION GCOUNT",
				  kgcount, eq, gcount) ;
      }
   else
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20s", kextend, eq, true) ;
      sprintf (&fitbuf [zz++][31], "/ EXTENSION(S) POSSIBLE") ;

   /*** Date & Time of observation. ***/

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [zz++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",                                doy_obs, year_obs) ;
   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   strcpy  (&fitbuf [zz++][31], "/ OBSERVATION TIME: HH/MM/SS [UT]") ;

   /*** Telescope information. ***/

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [zz++][31], "/ %s", origincom) ;
   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", ktelescop, eq, telescop) ;
   sprintf (&fitbuf [zz++][31], "/ DIGITAL PROMINENCE MONITOR") ;
   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kinstrume, eq, instrume) ;
   sprintf (&fitbuf [zz++][31], "/ KODAK MEGAPLUS 1.6 CCD") ;

   /*** Observation source & type. ***/

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kcode_obs, eq, code_obs) ;
   sprintf (&fitbuf [zz++][31], "/ TYPE OF OBSERVATION [CHARACTER CODE]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [zz++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kobject  , eq, object) ;
   sprintf (&fitbuf [zz++][31], "/ SOURCE OBJECT") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", ktype_dp , eq, type_dp) ;
   sprintf (&fitbuf [zz++][31], "/ DATA PROCESSING TYPE") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kdataform, eq, dform) ;
   strcpy  (&fitbuf [zz++][31], "/ DATA FORM") ;

   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20d / 1: DISC 2: LIMB 3: REF 4: DIFU 5-8: DARK + 1-4",
			      kdatatype, eq, DPMI->dtype) ;


   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kwavelen , eq, wavelen) ;
   sprintf (&fitbuf [zz++][31], "/ CENTRAL WAVE LENGTH OF OBSERVATION [NM]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f",
                                kwavewid,  eq, wavewid) ;
   sprintf (&fitbuf [zz++][31], "/ BANDPASS OF FILTER [NM]") ;

/*
 *   sprintf (&fitbuf [zz++][ 0], "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;

 */
   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",  kctype1  , eq, ctype1) ;
   sprintf (&fitbuf [zz++][31], "/ AXIS 1 TYPE: '%-8s'", atype1) ;
   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",  kcunit1  , eq, cunit1) ;
   sprintf (&fitbuf [zz++][31], "/ AXIS 1 UNITS: '%s'", cunit1) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / %s %s [INDEX]",
			      kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / DPM +X=%7.2f DEGREES CCW FROM ARRAY +X AXIS",
			      kcrota1  , eq, crota1, crota1) ;

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",  kctype2  , eq, ctype2) ;
      sprintf (&fitbuf [zz++][31], "/ AXIS 2 TYPE: %s", atype2) ;
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",  kcunit2  , eq, cunit2) ;
      sprintf (&fitbuf [zz++][31], "/ AXIS 2 UNITS: '%s'", cunit1) ;
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / %s %s [INDEX]",
			           kcrpix2  , eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / %s %s [%s]",
			           kcrval2  , eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2, eq, cdelt2, atype2, cunit2) ;
   sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f / DPM +Y=%7.2f DEGREES CCW FROM ARRAY +Y AXIS",
			        kcrota2 , eq, crota2, crota2) ;
      }

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [zz  ][ 0], "%-8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [zz++][31], "/ COORDINATE SYSTEM NAME") ;

   sprintf (&fitbuf [zz  ][ 0], "%-8s%2s%20.2f", kcrradius, eq, crradius) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR RADIUS [PIXELS]") ;

   sprintf (&fitbuf [zz  ][ 0], "%-8s%2s%20.2f", krsun    , eq, DPMI->rsun) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR RADIUS [ARCSEC]") ;

   sprintf (&fitbuf [zz  ][ 0], "%-8s%2s%20.2f", kocc_disc, eq, occ_disc) ;
   sprintf (&fitbuf [zz++][31], "/ OCCULTING DISC DIAMETER [ARCSECONDS]") ;

/*
 *   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kangnorth, eq, angnorth) ;
 *   sprintf (&fitbuf [zz++][31], "/ DIR OF SOLAR NORTH W.R.T. +X AXIS [CCW DEG]") ; 
 */

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", ksolar_p0, eq, DPMI->pangle) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR P-ANGLE   [DEGREES]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", ksolar_b0, eq, DPMI->bangle) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR B-ANGLE   [DEGREES]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", ksolar_l0, eq, DPMI->langle) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR L-ANGLE   [DEGREES]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", khour_ang, eq, DPMI->hangle) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR H-ANGLE   [DEGREES]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kelev    , eq, DPMI->elev) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR ELEVATION [DEGREES]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kazimuth , eq, DPMI->azimuth) ;
   sprintf (&fitbuf [zz++][31], "/ SOLAR AZIMUTH   [DEGREES]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kairmass , eq, DPMI->airmass) ;
   sprintf (&fitbuf [zz++][31], "/ AIRMASS         [ATMOSPHERES]") ;

/*
 *   sprintf (&fitbuf [zz++][ 0], "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
 */

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20d", knimsum  , eq, DPMI->nsum) ;
   sprintf (&fitbuf [zz++][31], "/ NUMBER OF IMAGES SUMMED") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kexpdur  , eq, expdur) ;
   sprintf (&fitbuf [zz++][31], "/ EXPOSURE DURATION [SECONDS]") ;

   /*** Magnitude scaling. ***/

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kbunit, eq, bunit) ;
   sprintf (&fitbuf [zz++][31], "/ MAGNITUDE UNITS") ;

   if (fabs ((double) DPMI->bscale) < 10000.0)
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kbscale, eq, DPMI->bscale) ;
   else
      sprintf (&fitbuf [zz  ][0], "%8s%2s%20.2f", kbscale, eq, DPMI->bscale) ;
   sprintf (&fitbuf [zz++][31], "/ PHYSICAL = DATA * BSCALE + BZERO") ;

   if (fabs ((double) DPMI->bzero) < 10000.0)
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2f", kbzero , eq, DPMI->bzero ) ;
   else
      sprintf (&fitbuf [zz++][ 0], "%8s%2s%20.2e", kbzero , eq, DPMI->bzero ) ;

   if (fabs ((double) vmin) < 10000.0)
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kdatamin , eq, vmin) ;
   else
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2e", kdatamin , eq, vmin) ;
   sprintf (&fitbuf [zz++][31], "/ MINIMUM DATA VALUE [%s]", datcom) ;

   if (fabs ((double) vmax) < 10000.0)
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kdatamax , eq, vmax) ;
   else
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2e", kdatamax , eq, vmax) ;
   sprintf (&fitbuf [zz++][31], "/ MAXIMUM DATA VALUE [%s]", datcom) ;

   if (fabs ((double) wmin) < 10000.0)
      sprintf (&fitbuf [zz  ][0], "%8s%2s%20.2f", kdispmin , eq, wmin) ;
   else
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2e", kdispmin , eq, wmin) ;
   sprintf (&fitbuf [zz++][31], "/ MIN VALUE FOR DISPLAY [%s]", datcom) ;

   if (fabs ((double) wmax) < 10000.0)
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2f", kdispmax , eq, wmax) ;
   else
      sprintf (&fitbuf [zz  ][ 0], "%8s%2s%20.2e", kdispmax , eq, wmax) ;
   sprintf (&fitbuf [zz++][31], "/ MAX VALUE FOR DISPLAY [%s]", datcom) ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'", kdisp_dir,  eq, disp_dir) ;
   sprintf (&fitbuf [zz++][31], "/ DISPLAY DIRECTION (AXIS 1 2 3): R/L  U/D  I/O") ;

/*
 *   sprintf (&fitbuf [zz++][ 0], "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
 */

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",     kdate_cal, eq, date_cal) ;
   sprintf (&fitbuf [zz++][31],"/ DPM CALIBRATION DATE: DD/MM/YY [UT]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",     kdate, eq, date) ;
   sprintf (&fitbuf [zz++][31], "/ FITS FILE CREATION DATE: DD/MM/YY [UT]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",     ktime, eq, time) ;
   sprintf (&fitbuf [zz++][31], "/ FITS FILE CREATION TIME: HH:MM:SS [UT]") ;

   sprintf (&fitbuf [zz  ][ 0], "%8s%2s'%-8s'",
			        kvers_sw,  eq, vers_sw) ;
   sprintf (&fitbuf [zz++][31], "/ DPM SOFTWARE VERSION") ;

   sprintf (&fitbuf [zz++][ 0], "          ----------------------------------------------------------------------") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   THE MAUNA LOA SOLAR OBSERVATORY (MLSO) IS LOCATED ON THE ISLAND") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   OF HAWAII, AND IS OPERATED BY THE HIGH ALTITUDE OBSERVATORY (HAO),") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   A DIVISION OF THE NATIONAL CENTER FOR ATMOSPHERIC RESEARCH (NCAR),") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   UNDER SPONSORSHIP OF THE NATIONAL SCIENCE FOUNDATION (NSF).") ;
   sprintf (&fitbuf [zz++][ 0], "          ----------------------------------------------------------------------") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   THE DIGITAL PROMINENCE MONITOR (DPM) OBSERVES THE SOLAR DISK & LIMB") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   WITH AN APPROXIMATE 1.6 x 2.2 SOLAR RADIUS FIELD OF VIEW, USING A") ;
   sprintf (&fitbuf [zz++][ 0], "COMMENT   KODAK CCD DETECTOR (1534x1032 PIXELS, OF WHICH 1534x1029 ARE USED).") ;
   if (strncmp (type_dp, "RAW", 3) == 0)
      {
      sprintf (&fitbuf [zz++][ 0], "COMMENT   BRIGHTNESS MEASUREMENTS ARE SPECIFIED IN RAW CCD UNITS.") ;
      }
   else
      {
      sprintf (&fitbuf [zz++][ 0], "COMMENT   BRIGHTNESS MEASUREMENTS ARE SPECIFIED IN UNITS OF BSUN-HALPHA,") ;
      sprintf (&fitbuf [zz++][ 0], "COMMENT   WHICH IS A RATIO OF PIXEL BRIGHTNESS TO BRIGHTNESS OF SOLAR DISK.") ;
      }
   sprintf (&fitbuf [zz++][ 0], "COMMENT   IMAGE ORIGIN IS THE LOWER LEFT CORNER; X: LEFT->RIGHT, Y: BOTTOM->TOP.") ;
   sprintf (&fitbuf [zz++][ 0], "          ----------------------------------------------------------------------") ;

   /*** Terminate FITS HDU. ***/

   sprintf (&fitbuf [zz++][0], "%8s", kend) ;

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   znum = zz ;
   nhdu = znum / 36 ;
   if (nhdu * 36 != znum) nhdu++ ;
   for (iy = 0 ; iy < nhdu*36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
         {
         if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
         ival = (int) fitbuf [iy][ix] ;
         fitbuf [iy][ix] = (char) toupper (ival) ;
         }

   if (pmode)
      fprintf (stdout, "dpmtofits> znum: %d  nhdu: %d\n", znum, nhdu) ;

   if (pmode)
      for (i = 0 ; i < nhdu*36 ; i++)
         {
         strncpy (outbuf, &fitbuf [i][0], 80) ;
         outbuf [80] = '\0' ;
         fprintf (stdout, "%s\n", outbuf) ;
         }

   /*** Write PRIMARY FITS header to disk file. ***/

   nelwr = fwrite (fitbuf, sizeof (char), nhdu*nbyfits, FF) ;

   /*** Write pixel data. ***/

   nelwr = fwrite (imgbuf, DPMI->nbypx, npximg, FF) ;

   /*** If image size is not an integer multiple of 2880 bytes,
    *** pad FITS file with zeroes.
    ***/

   nrec = nbyimg / nbyfits ;
   if ((nrec * nbyfits) < nbyimg)
      {
      nrec++ ;
      npad = (nrec * nbyfits) - nbyimg ;
      padbuf = (unsigned char *) malloc ((size_t) npad) ;
      qq = padbuf ;
      for (i = 0 ; i < npad ; i++)  *qq++ = '\0' ;
      nelwr = fwrite (padbuf, 1, npad, FF) ;
      free (padbuf) ;
      }
   if (pmode)
      fprintf (stdout, "nrec:%4d  npad:%6d  nbyimg:%6d\n", nrec, npad, nbyimg) ;

   /* Add header info extension. */

   dpmext (FF, object, type_obs, date_obs, time_obs, pmode) ;

   free (imgbuf) ;
   close (FF) ;
   return (0) ;
}
