/* findcen
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find solar disk center & diameter.
 *
 * Adapted from "find_shift.c" in "/home/puka/mrchip/chipcal/src".
 * Calls Alice's subroutine findshift () to find shift values for given image.
 *
 * Adapted by Andrew L. Stanger   15 March 1996
 *
 * Andy was using doing a couple of things which I fixed.
 *  1) taking out the housekeeping line, then shoving it back in
 *  2) Using the 20 lines of padding and then not compensating for it
 *     in the center calculations.
 *  So, I took 1) out  and made padding and option, and called my routine
 *  "findshift" directly and calculate the centers properly from that.
 *  --Alice 4/22/96
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */


#include        <stdio.h>
#include        <stdlib.h>
#include        <errno.h>
#include        <math.h>

#include "/home/cordyn/mlsofits/lib/acos/acos.h"

int fts = 1 ;

int
findcen (imgbuf, xdim, ydim, xcenter, ycenter, diam, scode)

short *imgbuf ;				/* Image array. */
int xdim, ydim ;			/* Image dimensions. */
float *xcenter, *ycenter, *diam ;	/* Sun center & diameter. */
int scode ;				/* Source code: see "acos.h". */
{
   extern int     findshift () ;
   extern int     findshiftL () ;
   extern int     dPcenter_nofile () ;
   int            ishift, icen, ix, iy, code, read (), write () ;
   short          *img ;
   float          XCEN, YCEN, xshi, yshi , radius ;
   register short *op ;
   register short *ip ;
   int            nrow, ncol, record ;
   int            dopad, pad, hpad ;
   int            doprint = 1 ;

   dopad=1 ; pad=20 ; hpad=10 ;  /* Add padding to the y-dimension just in case
				    disk is too close to the edge.  */
   dopad=0 ; pad= 0 ; hpad= 0 ;  /* Don't need no stinkin' padding! */

   ncol = xdim ;			/* # columns. */
   nrow = ydim ;			/* # rows.    */

   XCEN = (xdim - 1.0) * 0.5 ;		/* x-axis center of image array. */
   YCEN = (ydim - 1.0) * 0.5 ;		/* y-axis center of image array. */
   
   if (doprint) fprintf (stderr, "find_shift: XDIM %d YDIM %d RECORD %d\n",
                       ncol, nrow, record) ;

   if ( dopad ) 			/* Add padding to image. */
      {
      img = (short *) malloc ( sizeof (short) * ncol * (nrow + pad) ) ;

      ip = img ;
      ip += hpad * ncol ;

      for (iy = 0 ; iy < nrow ; iy++)
         for (ix = 0 ; ix < ncol ; ix++)
            {
            *(ip++) = *(imgbuf + ix + iy * xdim) ;
            }

      /*** Pad top and bottom in case Y is too close to edge ***/

      for ( iy = 0 ; iy < hpad ; iy++ )
         {
         ip = img ;
         ip += hpad * ncol ;
         op = img ;
         op += iy * ncol ;
         for ( ix = 0 ; ix < ncol ; ix++ )   *op++ = *ip++ ;
         }

      for ( iy = nrow - hpad ; iy < nrow ; iy++ )
         {
         ip = img ;
         ip += (nrow - 11) * ncol ;
         op = img ;
         op += iy * ncol ;
         for ( ix = 0 ; ix < ncol ; ix++ )   *op++ = *ip++ ;
         }

      }
   else
      {
      img = imgbuf ;
      }

/*
 *   if (ncol ==  767) *diam = 335.0 ;
 *   if (ncol == 1534) *diam = 335.0 * 2.0 ;
 */

   if (scode == SOURCE_l)		/* Limb image. */
      {
      if (doprint) fprintf ( stderr, "findcen> calling findshiftL.\n") ;
      ishift = findshiftL (img+3*ncol, ncol-6, nrow+pad, &xshi, &yshi, diam) ;
      }
   else					/* Non-limb image. */
      {
      if (doprint) fprintf ( stderr, "findcen> calling findshift.\n") ;
      ishift = findshift (img+3*ncol, ncol-6, nrow+pad, &xshi, &yshi, diam) ;
      }

   if (ishift == 1)			/* findshift was successful. */
      {
      *xcenter = XCEN - xshi ;
      *ycenter = YCEN - yshi ;
      if (scode == SOURCE_s)		/* Disc image. */
	 {
         /* Try it again via the less accurate method.
          * If the answers differ by more than 5 pixels, then the less accurate
	  * method, which is more robust, is the result to use.
          */
         if (doprint) fprintf ( stderr,
			       "findcen> calling dPcenter_nofile.  ishift:%d\n",
                                ishift) ;
         icen        = dPcenter_nofile (img, 0, &XCEN, &YCEN, &radius) ;
         *diam       = radius * 2.0 ;
         xshi = yshi = 0.0 ;
         if (icen == 1)
	    {
            if (*xcenter > XCEN + 5.0 || *xcenter < XCEN - 5) *xcenter = XCEN ;
            if (*ycenter > YCEN + 5.0 || *ycenter < YCEN - 5) *ycenter = YCEN ;
	    }
         }
      }

   else					/* findshift failed. */

      { 
      if (scode == SOURCE_l)		/* Limb image. */
	 {
         if (doprint) fprintf ( stderr,
			       "findcen> calling dPcenter_nofile.  icen:%d\n",icen) ;
         icen        = dPcenter_nofile (img, 1, &XCEN, &YCEN, &radius) ;
         *diam       = radius * 2.0 ;
         xshi = yshi = 0.0 ;
         }
      if (scode == SOURCE_s)		/* Disc image. */
	 {
         icen        = dPcenter_nofile (img, 0, &XCEN, &YCEN, &radius) ;
         *diam       = radius * 2.0 ;
         xshi = yshi = 0.0 ;
         }

      if (icen == 1)			/* dPcenter was successful. */
	 { 
         *xcenter = XCEN - xshi ;
         *ycenter = YCEN - yshi ;
         }
      else				/* dPcenter failed. */
         {
         *xcenter = -999999.0 ;
         *ycenter = -999999.0 ;
	 fprintf (stderr, ">>> findcen FAILED <<<\n") ;
         }
      }

   if (dopad) free (img) ;
   return (0) ;
}
