/* rddpm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read MLSO Digital Prominence Monitor (DPM) image from disk to memory.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 February 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/src/dpm/dpmlib.ext"

unsigned char *
rddpm (namimg, pmode)

char namimg [] ;		/* DPM file name. */
int pmode ;			/* Print mode: 0=silent, 1=verbose. */

{
   extern int dpmhskp () ;		/* DPM housekeeping. */
   extern char *malloc () ;		/* Memory allocation. */
   FILE *FD ;
   int ix, iy ;
   int nbyimg ;
   int nbypix ;
   int nelrd ;
   int npximg ;
   int nx, ny ;
   int xdim, ydim ;
   unsigned char  *cbuf ;		/* Short int buffer pointer. */

   dpmhskp (namimg, pmode) ;

   /*** Open DPM file. ***/

   FD = fopen (namimg, "r") ;

   /*** Read DPM header into a structure. ***/

   fread ((char *) &DPMHS, sizeof (DPMHS), 1, FD);

   if (pmode)
      {
      printf ("Version:    %8d\n",   DPMH->Version) ;
      printf ("Year:       %8d\n",   DPMH->Year) ;
      printf ("Month:      %8d\n",   DPMH->Month) ;
      printf ("Day:        %8d\n",   DPMH->Day) ;
      printf ("Hour:       %8d\n",   DPMH->Hour) ;
      printf ("Minute:     %8d\n",   DPMH->Minute) ;
      printf ("Sec:        %8d\n",   DPMH->Sec) ;
      printf ("Xdim:       %8d\n",   DPMH->Xdim) ;
      printf ("Ydim:       %8d\n",   DPMH->Ydim) ;
      printf ("Nsum:       %8d\n",   DPMH->Nsum) ;
      printf ("Exposure:   %8d\n",   DPMH->Exposure) ;
      printf ("Gain:       %8d\n",   DPMH->Gain) ;
      printf ("Odisk:      %8d\n",   DPMH->Odisk) ;
      printf ("Type:       %8d\n",   DPMH->Type) ;
      printf ("Optics[0]:  %8d\n",   DPMH->Optics[0]) ;
      printf ("Optics[1]:  %8d\n",   DPMH->Optics[1]) ;
      printf ("Optics[2]:  %8d\n",   DPMH->Optics[2]) ;
      printf ("Optics[3]:  %8d\n",   DPMH->Optics[3]) ;
      printf ("Optics[4]:  %8d\n",   DPMH->Optics[4]) ;
      printf ("Optics[5]:  %8d\n",   DPMH->Optics[5]) ;
      printf ("Optics[6]:  %8d\n",   DPMH->Optics[6]) ;
      printf ("Optics[7]:  %8d\n",   DPMH->Optics[7]) ;
      printf ("Pangle:     %8.2f\n", DPMH->Pangle) ;
      printf ("Bangle:     %8.2f\n", DPMH->Bangle) ;
      printf ("Langle:     %8.2f\n", DPMH->Langle) ;
      printf ("HA:         %8.2f\n", DPMH->HA) ;
      printf ("Elevation:  %8.2f\n", DPMH->Elevation) ;
      printf ("Azimuth:    %8.2f\n", DPMH->Azimuth) ;
      printf ("Amass:      %8.2f\n", DPMH->Amass) ;
      printf ("Radius:     %8.2f\n", DPMH->Radius) ;
      }

   /*** Read pixel data. ***/

   npximg = DPMH->Xdim * DPMH->Ydim ;		/* # pixels in image. */
   nbypix = sizeof (unsigned short) ;		/* # bytes  in image. */
   nbyimg = npximg * nbypix ;
   if (pmode)
      {
      fprintf (stdout, "rddpm> DPMH->Xdim: %d  DPMH->Ydim: %d\n",
	       DPMH->Xdim, DPMH->Ydim) ;
      fprintf (stdout, "rddpm> npximg: %d  nbyimg: %d\n", npximg, nbyimg) ;
      }
   cbuf = (unsigned char *) malloc (nbyimg) ;	/* Allocate memory for image. */

   nelrd = fread (cbuf, 1, nbypix * npximg, FD) ;	/* Read image. */
   if (pmode)
      fprintf (stdout, "rddpm> nelrd: %d\n", nelrd) ;

   fclose (FD) ;
   return (cbuf) ;			/* Return pointer to image buffer.*/
}
