/* mk3sk.c
 * -----------------------------------------------------------------------------
 * Read a special ASCII Mk3 header output file.
 * Get the date, time, and sky transmission.
 * Compute the Airmass.
 * Write an output file with the following information:
 *    date
 *    time
 *    sky transmisson
 *    airmass
 * -----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   18 July 2002
 * -----------------------------------------------------------------------------
 */

#define START

#include <stdio.h>
#include <math.h>
#include "mk4.h"
#include "ephem.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
    static char filein  [84] ;
    static char fileout [84] ;
    static char line [84] ;
    static char s1 [84] ;
    static char s2 [84] ;
    static char s3 [84] ;
    static char s4 [84] ;

    int i ;
    int year ;
    int month ;
    int day ;
    int hour ;
    int minute ;
    int second ;
    int sk ;

    float skytx ;
    float airmass ;

    FILE *fdin, *fdout ;

    printf ("siteLatitude: %7.2f [deg]       siteLongitude: %7.2f [deg]\n",
	    siteLatitude, siteLongitude) ;
    siteLongitude *= d2r ;
    siteLatitude  *= d2r ;
    sin_Lat = sin (siteLatitude) ;
    cos_Lat = cos (siteLatitude) ;
    printf ("siteLatitude: %7.2f [radians]   siteLongitude: %7.2f [radians]\n",
	    siteLatitude, siteLongitude) ;

    if (argc < 2)
	{
	fprintf (stdout, "usage: %s mk3_hdr_file\n", argv [0]) ;
	exit (-1) ;
	}

    strcpy (filein, argv [1]) ;		/* Get Mk3 "header" file name */

    fdin  = fopen (filein, "r") ;

    i = 0 ;

    while (fgets (line, 80, fdin) != NULL)
	{
	sscanf (line, "%s %s %s %s", s1, s2, s3, s4) ;
	printf ("%s\n", line) ;
	if (strncmp (s1, "No", 2) == 0) goto done ;
	if (strncmp (s1, "Image", 5) == 0)
	    {
	    fgets  (line, 80, fdin) ;
	    sscanf (line, "%s", s1) ;	/* Read 'record length' line. */
	    printf ("%s\n", s1) ;

	    fgets  (line, 80, fdin) ;
	    sscanf (line, "%d/%d/%d %d_%d_%d",
		    &month, &day, &year,
		    &hour, &minute, &second) ;	/* Read 'date & time' line. */
	    printf ("%d %d %d   %d %d %d\n",
	            month, day, year, hour, minute, second) ;

	    fgets  (line, 80, fdin) ;
	    sscanf (line, "%s", s1) ;	/* Read 'nray' line. */
	    printf ("%s\n", s1) ;

	    fgets  (line, 80, fdin) ;
	    sscanf (line, "%s %s %s %d",
		    s1, s2, s3, &sk) ;	/* Read 'bright' line. */
	    printf ("%s %s %s %d\n", s1, s2, s3, sk) ;

	    fgets  (line, 80, fdin) ;
	    sscanf (line, "%s", s1) ;	/* Read 'pangle' line. */
	    printf ("%s\n", s1) ;

	    year += 1900 ;
	    skytx = (float) sk ;

	    Eph_year   = year ;
	    Eph_month  = month ;
	    Eph_day    = day ;
	    Eph_hour   = hour ;
	    Eph_minute = minute ;
	    Eph_second = second ;

	    MRSUN_Isat () ;
	    airmass = ephAmass ;

	    printf ("B0:%7.2f     L0:%7.2f      P:%7.2f\n",
		    ephBangle, ephLangle, ephPangle) ;
	    printf ("Ro:%7.2f     RA:%7.2f    DEC:%7.2f\n",
		     ephSRadius, ephRA, ephDEC ) ;
	    printf ("ST:%7.2f     HA:%7.2f \n", ephST, ephHA) ;
	    printf ("Az:%7.2f   Elev:%7.2f \n", ephAz, ephElev) ;
	    printf ("Amass:%8.4f\n", ephAmass) ;

	    i += 1 ;
	    if (i == 1)
		{
		sprintf (fileout, "%04d%02d%02d.%02d%02d%02d.mk3.stxam.txt",
			 year, month, day, hour, minute, second) ;
		printf ("fileout: %s\n", fileout) ;
                fdout = fopen (fileout, "w") ;
		}

	    fprintf (fdout, "%04d/%02d/%02d %02d:%02d:%02d %8.2f %8.4f\n",
		     year, month, day, hour, minute, second,
		     skytx, airmass) ;
	    }

	}

done:

    fclose (fdin) ;
    fclose (fdout) ;
    exit (0) ;
}
