/* mk4.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * MLSO Mark-IV K-coronameter
 * Constants & Definitions.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define  NAZ     (720)
#define  NPX     (384)
#define  CY_IMG  (1080*384) /* Have to keep this big to hold raw data. */
#define  VECTOR  (1080)     /* Have to keep this big to hold raw data. */

/* Can go up to SQ==700 , with TRY==2 , and not suffer any "pixelation" */
/* With TRY==2, it takes  8 sec to do an image. */
/* With TRY==3, it takes 12 sec to do an image. */

#define  SQ   (512)
#define  TRY    (2)             /* Expansion factor */

#ifdef   BIGSCALE
#undef   SQ
#undef   TRY
#define  SQ   (960)
#define  TRY    (5)             /* Expansion factor */
#endif

#define  RSIZE  (SQ*SQ*2)       /* byte size of rectangular array */
#define  SIZE   (SQ*SQ)         /* size of rect array as shorts */
#define  XSIZE  (SQ)            /* x size of rect array */

#define  LOWPIX     (70)
#define  HIGHPIX   (250)
#define  SCALEPIX  ( 80)

/* These are the values gleaned from 99d274.22_44.mk4.i data */
/* #define  HAIRLO  (107.0)      */
/* #define  HAIRHI  (277.0)      */
/* #define  RZERO   (0.868645)   */
/* #define  RDELT   (0.00546721) */

#define  HAIRLO  (103.0)        /* 28 Apr 2001 */
#define  HAIRHI  (273.0)        /* 28 Apr 2001 */

#define  RZERO   (0.9386)       /* 28 Apr 2001 01d118.20_36 */
#define  RDELT   (0.005265)     /* 28 Apr 2001 01d118.20_36 */

/* #define LOZERO (48) */
#define  LOZERO  (43)
#define  HIZERO  (NPX-1-10)

/*  Bottom and top pixels that *may* be excused from careful correction. */

/* #define BOTPIX  (38) */
#define BOTPIX  (33) 
#define TOPPIX  (NPX)

#define BACKLASH  (0.5)         /* In degrees */

#define SKYOFF   (0)
#define ZEROSKY  (376.0)
#define DEFSKY   (350)
#define MAXSKY   ZEROSKY
#define MINSKY   (250)
#define BADSKY   (200)

/* Observation Type (what) == mk4h->ObsType */

#define IS_cal0        0
#define IS_cal15     150
#define IS_dark     9999
#define IS_coronal  1000

/* Data Type (who) mk4h->DataType  */

#define IS_i                   0
#define IS_q                   1
#define IS_u                   2
#define IS_ba                  3
#define IS_sk                  4
#define IS_st                  5
#define IS_tscp                6
#define IS_drift               7

/* Processing steps (howfar) mk4h->ProcSteps */

#define IS_raw                10
#define IS_raw_spln           11
#define IS_raw_rect           12
#define IS_cal                13
#define IS_cal_slit           14
#define IS_cal_rect           15
#define IS_cal_rect_slit      16
#define IS_cal_rect_byte      17 /* rpB image scaled from 0-255 for display.*/
#define IS_cal_avg            18
#define IS_cal_avg_slit       19
#define IS_cal_avg_rect       20
#define IS_cal_avg_rect_slit  21

#define IS_raw_avg            24
#define IS_raw_avg_spln       25
#define IS_raw_avg_rect       26

#define IS_UNKNOWN            99

#ifdef START
    double twopi  =  3.14159265358979 *2.;
    double pi     =  3.14159265358979;
    double r2d    = 57.29577951308232;
    double d2r    =  0.01745329251994;
    double piov2  = 1.57079632679490;
    double piov4  = 0.78539816339745;
    double ba2rad = 0.000383495196971410;     /* 2 * pi / 16384 */
    double ba2deg = 0.02197265625;            /* 360.0  / 16384 */
    double deg2ba = 45.511111111;             /* 16384  / 360.0 */

    char *prtime ();

    char picnm [64], *piceos ;
    int mon, day, year, doy ;
    int dayflag, timeflag;
    int Secflag ;
    int nameflag;
    int doyflag ;
    unsigned int tim;

    float badval = -9999.0;

#else
    extern double        twopi ;
    extern double           pi ;
    extern double        r2d ;
    extern double        d2r ;
    extern double        piov2  ;
    extern double        piov4  ;
    extern double        ba2rad ;
    extern double        ba2deg ;
    extern double        deg2ba ;

    extern char          *prday () ;
    extern char          *prtime () ;
    extern int           get_hrminsec () ;
    extern double        pangle () ;
    extern double        phi () ;
    extern int           idoy () ;
    extern int           get_doy () ;
    extern int           jday () ;
    extern int           jdate () ;
    extern int           idate () ;
    extern int           isleap () ;

    extern char          picnm [64], *piceos ;
    extern int           mon, day, year, doy ;
    extern int           dayflag, timeflag;
    extern int           Secflag ;
    extern int           nameflag ;
    extern int           doyflag ;
    extern unsigned int  tim ;

    extern float badval ;

#endif
