/* mk3syn1_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display MLSO Mark-III synoptic map in SGI/GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Assumes that a 512x512x8-bit window has been opened using winopen ().
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR
 *  3 Dec 93
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/glcp_color.h"

mk3syn1_view (namfil)

char namfil [];

{
   static char slimb [4];			/* Solar limb [e,w]. */
   static char limb;				/* Solar limb [e,w]. */
   static char namout [80];
   static short sbuf [181][360];		/* Synoptic map buffer. */
   static unsigned short smap [181][360];	/* Synoptic map buffer. */
   unsigned short *sptr;
   short bdoy, byr;				/* Begin DOY, year. */
   short edoy, eyr;				/* End   DOY, year. */
   short smax =     0;				/* Data maximum. */
   short smin = 65535;				/* Data minimum. */
   unsigned short soff = 512;			/* Offset for LUT #2. */
   short sval;					/* Short integer. */
   short offset = 0;				/* Data offset. */

   int crot;					/* Carrington rotation. */
   int ix, iy;					/* indices. */
   int nlat  = 181;				/* # latitudes. */
   int nlong = 360;				/* # longitudes. */
   int xll, yll, xur, yur;			/* rectwrite coordinates. */
   int wxsize = 512;				/* window x size. */
   int wysize = 512;				/* window y size. */

   float fval;
   float radius;				/* height of synoptic scan. */
   float slope;					/* Scaling factor. */

   FILE *FD;

   gl_wclear (glcp_white);
   sptr = (unsigned short *) smap;
   sscanf (namfil, "%4d%1s%4.2f", &crot, &slimb, &radius);
   sscanf (&namfil [5], "%f", &radius);

   limb = (char) slimb [0];
   fprintf (stdout, "crot:%4d   limb:%c   radius:%4.2f\n", crot, limb, radius);
   sprintf (namout, "%s.map", namfil);

   /*** Open synoptic data file. ***/

   if ((FD = fopen (namfil, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namfil);
      exit (-1);
      }

   /*** Read data from synoptic file. ***/

   fread (sbuf, sizeof (short), nlat*nlong, FD);
   fread (&bdoy, sizeof (short), 1, FD);
   fread (&byr,  sizeof (short), 1, FD);
   fread (&edoy, sizeof (short), 1, FD);
   fread (&eyr,  sizeof (short), 1, FD);
   fclose (FD);

   xll = (wxsize - nlong) / 2;
   yll = (wysize - nlat) / 2;
   xur = xll + nlong - 1;
   yur = yll + nlat - 1;

   /*** Find data min/max. ***/

   smax =     0;
   smin = 65535;
   for (iy = 0; iy < nlat; iy++)
      for (ix = 0; ix < nlong; ix++)
	 {
	 if (smax < sbuf [iy][ix])  smax = sbuf [iy][ix];
	 if (smin > sbuf [iy][ix])  smin = sbuf [iy][ix];
	 }

/*
   if (radius <= 1.3)
      smax = 2500;
   else
   if (radius > 1.3 && radius <= 1.7)
      smax =  2500;
   if (radius > 1.7 && radius <= 2.0)
      smax = 2500;
 */

/*   if (smin < 0)  smin = 0; */
   if (smax == 0)
      {
      fprintf (stderr, "Max pixel value is 0.\n");
      exit (-1);
      }

   offset = 0;
   slope  = 250.0 / (float) (smax - smin);
   printf ("smin:%4d   smax:%4d\n", smin, smax);
   printf ("offset: %4d   slope: %9.4f\n", offset, slope);

   /*** Apply scaling factor & add display offset. ***/
   /*** Also reverse image top to bottom. ***/

   for (iy = 0; iy < nlat; iy++)
      for (ix = 0; ix < nlong; ix++)
	 {
	 sval = (short) sbuf [nlat - iy - 1][ix];
	 sval = (short) (float) (sval - offset) * slope;
	 if (sval < 0)    sval =   0; 
	 if (sval > 249)  sval = 249; 
	 smap [iy][ix] = (unsigned short) (sval + soff);
	 }
   rectwrite (xll, yll, xur, yur, sptr);

   mk3syn1_label (crot, byr, bdoy, eyr, edoy, radius, limb);

   mk3syn_save (namout);

   return (0);
}
