/* mk3syn3.c
 * * * ** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an MLSO Mark-III synoptic map (or a sequence of maps)
 * on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 April 1991: use onemap color mode.
 *  7 December 1992: add sequence + forward/backward options.
 *  8 December 1992: use mouse instead of keyboard; add wrap-around option (^).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#define MAXIMG 250
int pmode = 0;			/* Print mode. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
main (argc, argv)

int argc;
char *argv [];

{
   extern int pmode;
   FILE *LIST;
   short val;

   static char fliste [MAXIMG][80];	/* east filelist. */
   static char flistw [MAXIMG][80];	/* west filelist. */
   static char nampgm [80];	/* Program file name. */
   static char eastfile [80];	/* East synoptic data file name. */
   static char westfile [80];	/* West synoptic data file name. */
   static char namfil [80];	/* Synoptic data file name. */
   static char namlst [80];	/* Namelist file name. */

   int mk3syn3_view ();		/* Image display function. */
   int findex = 0;
   int i;			/* Index. */
   int ierr;			/* Error status from cpview(). */
   int lastimg = 0;		/* Last image index value. */
   int seq = 0;			/* Sequence indicator
				 * =0 single image
				 * =1 normal sequence
				 * =2 wrap-around sequence
				 * =3 continuous loop.
				 */
   int wxsize = 512;		/* Window width. */
   int wysize = 512;		/* Window height. */
   int xll, yll;		/* Lower left  window coordinates. */
   int xur, yur;		/* Upper right window coordinates. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   /*** Single argument: must be filename. ***/

   if (argc == 2)
      {
      strcpy (namfil, argv [1]);
      }

   /*** Two arguments: must be east & west filenames. ***/

   else
   if (argc == 3)
      {
      strcpy (namfil,   argv [1]);
      strcpy (eastfile, argv [1]);
      strcpy (westfile, argv [2]);
      printf ("arg3: eastfile: %s   westfile: %s\n", eastfile, westfile);
      }

   /*** Invalid number of command line arguments. ***/

   else
      {
      fprintf (stderr, "usage: %s [eastname westname | @namelist | ^namelist | %namelist]\n", argv [0]);
      exit (-1);
      }

   printf ("mk3syn3> A eastfile: %s   westfile: %s\n", eastfile, westfile);

   /*** Sequence of images ? ***/

   if (namfil [0] == '@' || namfil [0] == '^')
      {
      if (namfil [0] == '@')
         seq = 1;
      else
      if (namfil [0] == '^')
	 seq = 2;

      /*** Open namelist file & read contents to flist array. ***/

      strcpy (namlst, &namfil [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst);
         exit (-2);
         }
      i = -1;
      while (fscanf (LIST, "%s", namfil) != EOF)
         {
	 i++;
	 if (i > MAXIMG)
	    {
	    fprintf (stderr, "Too many images (%d) in namelist sequence.\n");
	    exit (-1);
	    }
         strcpy (&fliste [i][0], eastfile);
         strcpy (&flistw [i][0], westfile);
         }
      lastimg = i;
      fclose (LIST);
      }
   printf ("mk3syn3> B eastfile: %s   westfile: %s\n", eastfile, westfile);

   /*** Initialize GL software. ***/
   /* Put synoptic map window in upper right corner of screen.*/

   xur = 1271;
   yur =  991;
   xll = xur - wxsize;
   yll = yur - wysize;

   foreground ();
   prefposition (xll, xur, yll, yur);
   fminit ();			/* Initialize SGI font manager. */

   winopen (argv [0]);		/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

   doublebuffer ();		/* Use double buffer mode. */
   gconfig ();

   /*** Declare devices. ***/

   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   fprintf (stderr, "Position pointer in image window using mouse.\n");
   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n");
      fprintf (stderr, "click `left mouse'   to go forward.\n");
      fprintf (stderr, "click `middle mouse' to go backward.\n");
      }
   fprintf (stderr, "type  `ESC'          to quit.\n");

loop:

   printf ("mk3syn3> C eastfile: %s   westfile: %s\n", eastfile, westfile);
/*
   printf ("seq: %d\n", seq);
   if (seq)
      strcpy (eastfile, &fliste [findex][0]);
      strcpy (westfile, &flistw [findex][0]);
 */

   printf ("mk3syn3> D eastfile: %s   westfile: %s\n", eastfile, westfile);

   /*** Display image on IRIS screen in GL window. ***/

   mk3syn3_view (wxsize, wysize, eastfile, westfile);

   swapbuffers ();

   /*** Window remains on screen until terminated by mouse. ***/

   while (1)
   {
   while (qtest ())		/* Check device queue. */
      {
      switch (qread (&val))	/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++;
               if (findex > lastimg)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.\n");
                     findex = lastimg;
		     }
                  else
		  if (seq == 2)
		     findex = 0;
                  }
               goto loop;
               }
            break;

         case MIDDLEMOUSE:
            if (val == 1 && seq > 0)	/* Middle mouse button is down. */
               {
               findex--;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.\n");
                     findex = 0;
		     }
                  else
		  if (seq == 2)
		     findex = lastimg;
                  }
               goto loop;
               }
            break;

         case REDRAW:
	    mk3syn3_view (wxsize, wysize, eastfile, westfile);
            break;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               gexit ();
               exit (0);
               }
         }
      qreset ();
      }
   }

quit:
   gexit ();
   exit (0);
}
