/* mk3syn3_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display MLSO Mark-III synoptic map in SGI/GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Assumes that a 512x512x8-bit window has been opened using winopen ().
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR
 *  3 Dec 93
 * 10 Dec 93: Display BOTH east & west limbs in one window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/glcp_color.h"
#include "mk3syn.h"

mk3syn3_view (wxsize, wysize, eastfile, westfile)

int wxsize, wysize;				/* Window size. */
char eastfile [];				/* East synoptic map file. */
char westfile [];				/* West synoptic map file. */

{
   extern int pmode;
   static char slimb [4];			/* Solar limb [e,w]. */
   static char limb;				/* Solar limb [e,w]. */
   static char namout [80];
   static short sbufe [181][360];		/* Synoptic east map buffer. */
   static short sbufw [181][360];		/* Synoptic west map buffer. */
   static unsigned short smape [181][360];	/* Synoptic east map buffer. */
   static unsigned short smapw [181][360];	/* Synoptic west map buffer. */
   unsigned short *sptre, *sptrw;
   unsigned short begdoye, begyre;		/* East begin DOY, year. */
   unsigned short enddoye, endyre; 		/* East end   DOY, year. */
   unsigned short begdoyw, begyrw; 		/* West begin DOY, year. */
   unsigned short enddoyw, endyrw; 		/* West end   DOY, year. */
   short smaxe =     0;				/* Data maximum. */
   short smine= 65535;				/* Data minimum. */
   short smaxw =     0;				/* Data maximum. */
   short sminw= 65535;				/* Data minimum. */
   unsigned short soff = 512;			/* Offset for LUT #2. */
   short svale, svalw;				/* Short integer. */
   short offset = 0;				/* Data offset. */

   int ibegdoye, ibegyre;			/* East begin DOY, year. */
   int ienddoye, iendyre;			/* East end   DOY, year. */
   int ibegdoyw, ibegyrw;			/* West begin DOY, year. */
   int ienddoyw, iendyrw;			/* West end   DOY, year. */
   int crot;					/* Carrington rotation. */
   int ix, iy;					/* indices. */
   int nlat  = 181;				/* # latitudes. */
   int nlong = 360;				/* # longitudes. */
   int xlle, ylle, xure, yure;			/* rectwrite coordinates. */
   int xllw, yllw, xurw, yurw;			/* rectwrite coordinates. */

   float fval;
   float radius;				/* height of synoptic scan. */
   float slope;					/* Scaling factor. */

   FILE *FD;

   gl_wclear (glcp_white);
   sptre = (unsigned short *) smape;
   sptrw = (unsigned short *) smapw;

   if (pmode)
    printf ("mk3syn3_view> eastfile: %s   westfile: %s\n", eastfile, westfile);
   sscanf (eastfile, "%4d%1s%4.2f", &crot, &slimb, &radius);
   sscanf (&eastfile [5], "%f", &radius);
   limb = (char) slimb [0];
   if (pmode)
    fprintf (stdout, "crot:%4d   limb:%c   radius:%4.2f\n", crot, limb, radius);

   sscanf (westfile, "%4d%1s%4.2f", &crot, &slimb, &radius);
   sscanf (&westfile [5], "%f", &radius);
   limb = (char) slimb [0];
   if (pmode)
    fprintf (stdout, "crot:%4d   limb:%c   radius:%4.2f\n", crot, limb, radius);

   sprintf (namout, "%04d.r%03d.map", crot, (int) (radius*100.0 + 0.5));

   /*** Open east synoptic data file. ***/

   if ((FD = fopen (eastfile, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", eastfile);
      exit (-1);
      }

   /*** Read data from synoptic file. ***/

   fread (sbufe, sizeof (short), nlat*nlong, FD);
   fread (&begdoye, sizeof (short), 1, FD);
   fread (&begyre,  sizeof (short), 1, FD);
   fread (&enddoye, sizeof (short), 1, FD);
   fread (&endyre,  sizeof (short), 1, FD);
   fclose (FD);
   ibegdoye = (int) begdoye;
   ibegyre  = (int) begyre;
   ienddoye = (int) enddoye;
   iendyre  = (int) endyre;
   if (pmode)
    printf ("mk3syn3_view> begdoye:%6d   begyre:%4d   enddoye:%6d   endyre:%4d\n",
	   begdoye, begyre, enddoye, endyre);

   /*** Open west synoptic data file. ***/

   if ((FD = fopen (westfile, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", westfile);
      exit (-1);
      }

   /*** Read data from synoptic file. ***/

   fread (sbufw, sizeof (short), nlat*nlong, FD);
   fread (&begdoyw, sizeof (short), 1, FD);
   fread (&begyrw,  sizeof (short), 1, FD);
   fread (&enddoyw, sizeof (short), 1, FD);
   fread (&endyrw,  sizeof (short), 1, FD);
   fclose (FD);
   ibegdoyw = (int) begdoyw;
   ibegyrw  = (int) begyrw;
   ienddoyw = (int) enddoyw;
   iendyrw  = (int) endyrw;
   if (pmode)
    printf ("mk3syn3_view> begdoyw:%6d   begyrw:%4d   enddoyw:%6d   endyrw:%4d\n",
	   begdoyw, begyrw, enddoyw, endyrw);

   xllw = (wxsize - nlong) / 2;
   yllw = bfy * 3;
/*
 *   yllw = (wysize/2 - nlat) / 2;
 */
   xurw = xllw + nlong - 1;
   yurw = yllw + nlat - 1;

   xlle = (wxsize - nlong) / 2;
   ylle = yurw + bfy * 2;
/*
 *   ylle = (wysize/2 - nlat) / 2 + wysize/2;
 */
   xure = xlle + nlong - 1;
   yure = ylle + nlat - 1;

   /*** Find data min/max. ***/

   smaxe =     0;
   smine = 65535;
   for (iy = 0; iy < nlat; iy++)
      for (ix = 0; ix < nlong; ix++)
	 {
	 if (smaxe < sbufe [iy][ix])  smaxe = sbufe [iy][ix];
	 if (smine > sbufe [iy][ix])  smine = sbufe [iy][ix];
	 if (smaxw < sbufw [iy][ix])  smaxw = sbufw [iy][ix];
	 if (sminw > sbufw [iy][ix])  sminw = sbufw [iy][ix];
	 }

/*
   if (radius <= 1.3)
      {
      smaxe = 2500;
      smaxw = 2500;
      }
   else
   if (radius > 1.3 && radius <= 1.7)
      {
      smaxe =  2500;
      smaxw =  2500;
      }
   if (radius > 1.7 && radius <= 2.0)
      {
      smaxe = 2500;
      smaxw = 2500;
      }
 */

/*   if (smine < 0)  smine = 0; */
/*   if (sminw < 0)  sminw = 0; */
   if (smaxe == 0)
      {
      fprintf (stderr, "Max E pixel value is 0.\n");
      exit (-1);
      }

   if (smaxw == 0)
      {
      fprintf (stderr, "Max W pixel value is 0.\n");
      exit (-1);
      }

   offset = 0;
   slope  = 250.0 / (float) (smaxe - smine);
   if (pmode)
      {
      printf ("smine:%4d   smaxe:%4d\n", smine, smaxe);
      printf ("offset: %4d   slope: %9.4f\n", offset, slope);
      }

   /*** Apply scaling factor & add display offset. ***/
   /*** Also reverse image top to bottom. ***/

   for (iy = 0; iy < nlat; iy++)
      for (ix = 0; ix < nlong; ix++)
	 {
	 /*** East. ***/

	 svale = (short) sbufe [nlat - iy - 1][ix];
	 svale = (short) (float) (svale - offset) * slope;
	 if (svale < 0)    svale =   0; 
	 if (svale > 249)  svale = 249; 
	 smape [iy][ix] = (unsigned short) (svale + soff);

	 /*** West. ***/

	 svalw = (short) sbufw [nlat - iy - 1][ix];
	 svalw = (short) (float) (svalw - offset) * slope;
	 if (svalw < 0)    svalw =   0; 
	 if (svalw > 249)  svalw = 249; 
	 smapw [iy][ix] = (unsigned short) (svalw + soff);
	 }
   rectwrite (xlle, ylle, xure, yure, sptre);
   rectwrite (xllw, yllw, xurw, yurw, sptrw);

   mk3syn3_label (wxsize, wysize,
		  xlle, ylle, xure, yure, xllw, yllw, xurw, yurw,
		  ibegyre, ibegdoye, iendyre, ienddoye,
		  ibegyrw, ibegdoyw, iendyrw, ienddoyw,
		  crot, radius);

   mk3syn_save (namout);

   return (0);
}
