/* mk3syn5_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display MLSO Mark-III synoptic map in SGI/GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Assumes that a 512x512x8-bit window has been opened using winopen ().
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR
 *  3 Dec 93
 * 10 Dec 93: Display BOTH east & west limbs in one window.
 * 24 Jan 94: begdoye, enddoye, begdoyw, enddoyw changed from short to int.
 * 19 Aug 96: rasterfile output name format: mk3.CCCC.rXX, CCCC=C.R., XX=Rsun.
 * 21 Aug 96: multiple rotation version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/glcp_color.h"
#include "mk3syn.h"

mk3syn_view (wxsize, wysize, begrot, endrot, radius)

int wxsize, wysize ;			/* Window size. */
int begrot, endrot ;			/* Beginning & ending rotations. */
float radius ;				/* Radius (Rsun). */

{
   extern int pmode ;
   static char slimb [4] ;		/* Solar limb [e,w]. */
   static char limb ;			/* Solar limb [e,w]. */
   static char eastfile [80] ;		/* East limb file name. */
   static char westfile [80] ;		/* West limb file name. */
   static char namout [80] ;
   static short sbufe [181][360] ;	/* Synoptic east map buffer. */
   static short sbufw [181][360] ;	/* Synoptic west map buffer. */
   unsigned short *smape, *smapw ;
   short smaxe =     0 ;		/* Data maximum. */
   short smine= 65535 ;			/* Data minimum. */
   short smaxw =     0 ;		/* Data maximum. */
   short sminw= 65535 ;			/* Data minimum. */
   unsigned short soff = 512 ;		/* Offset for LUT #2. */
   short svale, svalw ;			/* Short integer. */
   short offset = 0 ;			/* Data offset. */

   int begdoye, begyre ;		/* East begin DOY, year. */
   int enddoye, endyre ;		/* East end   DOY, year. */
   int begdoyw, begyrw ;		/* West begin DOY, year. */
   int enddoyw, endyrw ;		/* West end   DOY, year. */
   int crot ;				/* Carrington rotation. */
   int irot ;				/* Rotation index. */
   int ix, iy, ixo ;			/* indices. */
   int nlat  = 181 ;			/* # latitudes. */
   int nlong = 360 ;			/* # longitudes. */
   int nlongo ;				/* # longitude pixels output. */
   int nrot ;				/* # rotations to map. */
   int ipass = 0 ;			/* Loop counter. */
   int xlle0, ylle0, xure0, yure0 ;	/* rectwrite coordinates. */
   int xllw0, yllw0, xurw0, yurw0 ;	/* rectwrite coordinates. */
   int xlle, ylle, xure, yure ;		/* rectwrite coordinates. */
   int xllw, yllw, xurw, yurw ;		/* rectwrite coordinates. */

   float fval ;
   float slope ;			/* Scaling factor. */

   FILE *FD ;

   printf ("begrot: %4d  endrot: %4d  radius:%7.2f\n", begrot, endrot, radius) ;

   gl_wclear (glcp_white) ;

   nrot = endrot - begrot + 1 ;
   nlongo = nlong / nrot ;
   smape = (unsigned short *) malloc (nlat * nlongo * sizeof (unsigned short)) ;
   smapw = (unsigned short *) malloc (nlat * nlongo * sizeof (unsigned short)) ;
   if (pmode)
      {
      printf ("nlat: %d   nlong : %d\n", nlat, nlong) ;
      printf ("nrot: %d   nlongo: %d\n", nrot, nlongo) ;
      }

   /*** Create output name for sun rasterfile. ***/

   if (begrot == endrot)
      sprintf (namout, "mk3.%04d.r%03d.sun",
	       begrot, (int) (radius*100.0 + 0.5)) ;
   else
      sprintf (namout, "mk3.%04d-%04d.r%03d.sun",
	       begrot, endrot,  (int) (radius*100.0 + 0.5)) ;

   /*** Define lower left and upper right corner pixel coordinates
    *** for synoptic array placement in map frame buffer.
    ***/

      xllw0 = (wxsize - nlong) / 2 ;
      yllw0 = bfy * 3 ;
/*
 *      yllw0 = (wysize/2 - nlat) / 2 ;
 */
      xurw0 = xllw0 + nlong - 1 ;
      yurw0 = yllw0 + nlat - 1 ;

      xlle0 = (wxsize - nlong) / 2 ;
      ylle0 = yurw0 + bfy * 2 ;
/*
 *      ylle0 = (wysize/2 - nlat) / 2 + wysize/2 ;
 */
      xure0 = xlle0 + nlong - 1 ;
      yure0 = ylle0 + nlat - 1 ;

      if (pmode)
	 {
         printf ("xlle0/ylle0: %4d %4d   xure0/yure0: %4d %4d\n",
                  xlle0, ylle0, xure0, yure0) ;
         printf ("xllw0/yllw0: %4d %4d   xurw0/yurw0: %4d %4d\n",
                  xllw0, yllw0, xurw0, yurw0) ;
         }

   ipass = 0 ;

   for (irot = begrot; irot <= endrot; irot++)
      {
      ipass++;

      /*** Adjust lower left & upper right coordinates for each rotation. ***/

      xlle = xlle0 + (nrot-ipass)   * nlongo ;
      ylle = ylle0 ;
      xure = xlle0 + (nrot-ipass+1) * nlongo - 1 ;
      yure = yure0 ;
      xllw = xllw0 + (nrot-ipass)   * nlongo ;
      yllw = yllw0 ;
      xurw = xllw0 + (nrot-ipass+1) * nlongo - 1 ;
      yurw = yurw0 ;

      if (pmode)
	 {
         printf ("xlle/ylle: %4d %4d   xure/yure: %4d %4d\n",
                  xlle, ylle, xure, yure) ;
         printf ("xllw/yllw: %4d %4d   xurw/yurw: %4d %4d\n",
                  xllw, yllw, xurw, yurw) ;
         }

      /*** Define input binary file name for East & West synoptic maps. ***/

      sprintf (eastfile, "mk3.cr%04d.e%d", irot, (int) (radius * 100)) ;
      sprintf (westfile, "mk3.cr%04d.w%d", irot, (int) (radius * 100)) ;
      if (pmode)
         printf ("mk3syn_view> eastfile: %s   westfile: %s\n", eastfile, westfile) ;
      crot = irot ;
/*
      sscanf (eastfile, "%4d%1s%4.2f", &crot, &slimb, &radius) ;
      sscanf (&eastfile [5], "%f", &radius) ;
 */
      limb = (char) slimb [0] ;
      if (pmode)
         fprintf (stdout, "crot:%4d   limb:%c   radius:%4.2f\n",
	          crot, limb, radius) ;

/*
      sscanf (westfile, "%4d%1s%4.2f", &crot, &slimb, &radius) ;
      sscanf (&westfile [5], "%f", &radius) ;
 */
      limb = (char) slimb [0] ;
      if (pmode)
         fprintf (stdout, "crot:%4d   limb:%c   radius:%4.2f\n",
                  crot, limb, radius) ;

      /*** Open East synoptic data file. ***/

      if ((FD = fopen (eastfile, "r")) == NULL)
         {
         fprintf (stderr, "mk3syn_view> Unable to open file: %s\n", eastfile) ;
         exit (-1) ;
         }

      /*** Read data from synoptic file. ***/

      fread (sbufe, sizeof (short), nlat*nlong, FD) ;
      fread (&begdoye, sizeof (int), 1, FD) ;
      fread (&begyre,  sizeof (int), 1, FD) ;
      fread (&enddoye, sizeof (int), 1, FD) ;
      fread (&endyre,  sizeof (int), 1, FD) ;
      fclose (FD) ;
      if (pmode)
         printf ("mk3syn_view> begdoye:%6d   begyre:%4d   enddoye:%6d   endyre:%4d\n",
	         begdoye, begyre, enddoye, endyre) ;

      /*** Open West synoptic data file. ***/

      if ((FD = fopen (westfile, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file: %s\n", westfile) ;
         exit (-1) ;
         }

      /*** Read data from synoptic file. ***/

      fread (sbufw, sizeof (short), nlat*nlong, FD) ;
      fread (&begdoyw, sizeof (int), 1, FD) ;
      fread (&begyrw,  sizeof (int), 1, FD) ;
      fread (&enddoyw, sizeof (int), 1, FD) ;
      fread (&endyrw,  sizeof (int), 1, FD) ;
      fclose (FD) ;
      if (pmode)
         printf ("mk3syn_view> begdoyw:%6d   begyrw:%4d   enddoyw:%6d   endyrw:%4d\n",
	         begdoyw, begyrw, enddoyw, endyrw) ;

      /*** Find data min/max. ***/

      smaxe =     0 ;
      smine = 65535 ;
      smaxw =     0 ;
      sminw = 65535 ;
      for (iy = 0 ; iy < nlat ; iy++)
         for (ix = 0 ; ix < nlong ; ix++)
	    {
	    if (smaxe < sbufe [iy][ix])  smaxe = sbufe [iy][ix] ;
	    if (smine > sbufe [iy][ix])  smine = sbufe [iy][ix] ;
	    if (smaxw < sbufw [iy][ix])  smaxw = sbufw [iy][ix] ;
	    if (sminw > sbufw [iy][ix])  sminw = sbufw [iy][ix] ;
	    }

      printf ("smine:%4d   smaxe:%4d  sminw:%4d  smaxw:%4d\n",
	       smine, smaxe, sminw, smaxw) ;
      smine =     0.0 ;
      sminw =     0.0 ;

      if (radius <  1.2)
         {
         smaxe = 10000 ;
         smaxw = 10000 ;
         }
      else
      if (radius > 1.2 && radius <= 1.35)
	 {
         smaxe =  8000 ;
         smaxw =  8000 ;
	 }
      else
      if (radius > 1.35 && radius <= 1.5)
	 {
         smaxe =  2500 ;
         smaxw =  2500 ;
	 }
      else
      if (radius > 1.5 && radius <= 1.7)
         {
         smaxe =  2000 ;
         smaxw =  2000 ;
         }
      else
      if (radius > 1.7 && radius <= 1.9)
         {
         smaxe =  1000 ;
         smaxw =  1000 ;
         }
      else
         {
         smaxe =   500 ;
         smaxw =   500 ;
         }

/*      if (smine < 0)  smine = 0 ; */
/*      if (sminw < 0)  sminw = 0 ; */
      if (smaxe == 0)
         {
         fprintf (stderr, "Max E pixel value is 0.\n") ;
         exit (-1) ;
         }

      if (smaxw == 0)
         {
         fprintf (stderr, "Max W pixel value is 0.\n") ;
         exit (-1) ;
         }

      offset = 0 ;
      slope  = 250.0 / (float) (smaxe - smine) ;
      if (pmode)
         {
         printf ("smine:%4d   smaxe:%4d\n", smine, smaxe) ;
         printf ("offset: %4d   slope: %9.4f\n", offset, slope) ;
         }

      /*** Apply scaling factor & add display offset. ***/
      /*** Also reverse image top to bottom. ***/

      for (iy = 0 ; iy < nlat ; iy++)
	 {
	 ixo = 0 ;
         for (ix = 0 ; ix < nlong ; ix = ix + nrot)
	    {
	    /*** East. ***/

	    svale = (short) sbufe [nlat - iy - 1][ix] ;
	    svale = (short) (float) (svale - offset) * slope ;
	    if (svale <= 0)   svale =   0 ; 
	    if (svale > 249)  svale = 249 ; 
	    *(smape + ixo +iy*nlongo) = (unsigned short) (svale + soff) ;

	    /*** West. ***/

	    svalw = (short) sbufw [nlat - iy - 1][ix] ;
	    svalw = (short) (float) (svalw - offset) * slope ;
	    if (svalw <= 0)   svalw =   0 ; 
	    if (svalw > 249)  svalw = 249 ; 
	    *(smapw + ixo + iy*nlongo) = (unsigned short) (svalw + soff) ;
/*
 *            if (iy == 0)
 *            printf ("ixo/iy: %3d %3d   ix/iy: %3d %3d\n", ixo, iy, ix, iy) ;
 */
	    ixo++ ;
            }
         }
      rectwrite (xlle, ylle, xure, yure, smape) ;
      rectwrite (xllw, yllw, xurw, yurw, smapw) ;
      }
   free (smape) ;
   free (smapw) ;

   mk3syn_label (wxsize, wysize,
		  xlle0, ylle0, xure0, yure0,
		  xllw0, yllw0, xurw0, yurw0,
		  begyre, begdoye, endyre, enddoye,
		  begyrw, begdoyw, endyrw, enddoyw,
		  begrot, endrot, radius) ;

   mk3syn_save (namout) ;

   return (0) ;
}
