/* mk3syn_label.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label a MLSO Mark-III synoptic map.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   3 Dec 93
 * 10 Dec 93: layout includes both east & west limbs.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/glcp_color.h"
#include "mk3syn.h"
#include "ephemeris.h"
float syndoy ();
float syndur ();

mk3syn_label (wxsize, wysize,
	       xlle, ylle, xure, yure, xllw, yllw, xurw, yurw,
               begyre, begdoye, endyre, enddoye,
               begyrw, begdoyw, endyrw, enddoyw,
	       ncrot, radius)

int wxsize, wysize;		/* Window size. */
int xlle, ylle;			/* Lower left  east map. */
int xure, yure;			/* Lower left  east map. */
int xllw, yllw;			/* Upper right west map. */
int xurw, yurw;			/* Upper right west map. */
int begyre;			/* Begin Year. */
int begdoye;			/* Begin DOY. */
int endyre;			/* End   Year. */
int enddoye;			/* End   DOY. */
int begyrw;			/* Begin Year. */
int begdoyw;			/* Begin DOY. */
int endyrw;			/* End   Year. */
int enddoyw;			/* End   DOY. */
int ncrot;			/* Carrington Rotation. */
float radius;	 		/* Radius. */

{
   extern int pmode;
   int xl1, xr1;
   int yb1, yt1;
   int xl2, xr2;
   int yb2, yt2;

   extern fmfonthandle gl_setfont ();
   fmfonthandle afh, bfh, cfh;
   static char title1 [80];
   static char title2 [80];
   static char title3 [80];
   static char titdoy [80];
   static char label  [80];
   static char lablong [] = "LONGITUDE";
   static char crad [4];
   static char crot [8];
   static char cbegmon [4], cendmon [4];
   static char mon_name [13][4] =
		{
		"***", "JAN", "FEB", "MAR", "APR", "MAY", "JUN",
		       "JUL", "AUG", "SEP", "OCT", "NOV", "DEC" };

   static int xbox1 [5], ybox1 [5], xbox2 [5], ybox2 [5];
  
   int ipixrot;				/* # pixels per rotation. */

   int  ibegdoy, ienddoy;
   int  ibegday, iendday;
   int  ibegmon, iendmon;
   int  ibegyr,  iendyr;
   int  i;
   int  lenlat, lenlong, lendoy;
   int	lenstr;
   int  lentit1, lentit2, lentit3;
   int	nlat = 181;	/* # latitude pixels. */
   int	nlong = 360;	/* # longitude pixels. */
   int  nxmap;		/* # X pixels in map. */
   int  nymap;		/* # Y pixels in map. */
   int  x0;		/* X position for 0 degrees longitude. */
   int	xd1;	/* x east.          */
   int  xxl;	/*                  */
   int  xxr;	/*                  */
   int	xx;	/* general purpose x-location variable. */
   int	yy;	/* general purpose y-location variable. */
   int	yd1;	/* y east.          */
   int	xd2;	/* x west.          */
   int	yd2;	/* y west.          */
   int	xwidth; /* # pixels width of synoptic strips for individual rotations.*/
   int	xw2;	/* xwidth / 2 */
   int	xspan;	/* # additional pixels width needed to span a gap. */
   int	yz1, yz2;	/* 0 degree latitude for east & west maps. */

   float xtrarot = 0.0;	/* Extra display beyond limits of a given rotation. */
   float fbegday, fendday;
   float fbegdoy, fenddoy;
   float fbegdoye, fenddoye;
   float fbegdoyw, fenddoyw;
static	float	crdur [16];	/* Duration of carrington rotations. */
float	crbeg, crend;	/* Rotation begin/end for synoptic map display. */
float   doybeg, doyend;			/* C.R. begin/end DOY. */
float	doye, doyw, doybege, doybegw, doymax, doydel;
float	fcrot;
float	filedoy0;	/* DOY at start of synoptic map data file. */
float	filerot0;	/* Rotation (cmp) at start of synoptic map data file. */
float	doy0;		/* DOY for first sample in data file. */
float	doymap;		/* Map duration in units of days. */
float	doyrot;		/* rotation duration in units of days. */
float	pixdoy;		/* # pixels per DOY. */
float	pixsam;		/* # pixels per sample. */
float	posdoye, posdoyw;
static	char sdoy [4];
int	locdoye, locdoyw;	/* screen coordinates for DOY tick marks. */
int	lxdoye, lydoye;		/* screen coordinates for east DOY string. */
int	lxdoyw, lydoyw;		/* screen coordinates for west DOY string. */
int	ibege, ibegw;	/* start index for east/west DOY loops. */
int	iende, iendw;	/* end index for east/west DOY loops. */
int	idoye, idoyw;	/* Indices for DOY annotation loops. */
int	ndoye, ndoyw;	/* Truncated (integer) DOY for right map edge. */
int	samdoy = 2;	/* # samples per DOY. */
int	NROT = 1;	/* # Carrington Rotations to display. */

   ipixrot = nlong;
   nxmap = (int) ((1.0 + xtrarot * 2.0) * ipixrot);
   nymap = nlat;
/*
 *   printf ("nxmap:%3d   nymap:%d\n", nxmap, nymap);
 */

   xl1 = xlle - (int) (xtrarot * ipixrot);
   yt1 = yure;
   xr1 = xure + (int) (xtrarot * ipixrot);
   yb1 = ylle;

   xl2 = xllw - (int) (xtrarot * ipixrot);
   yt2 = yurw;
   xr2 = xurw + (int) (xtrarot * ipixrot);
   yb2 = yllw;

   if (pmode)
      printf ("xr1:%3d   yt1:%3d   xl1:%3d   yb1:%3d\n", xr1, yt1, xl1, yb1);

   yz1 = yt1 -  90;		/* Zero degrees latitude (east map). */

   yz2 = yt2 -  90;		/* Zero degrees latitude (west map). */

   /*** Write annotation in GL image window. ***/

   bfh = gl_setfont (bfont, bsize);	/* Medium font & size. */

   /*** Titles. ***/

   sprintf (title1, "NCAR/HAO/MLSO Mark-III K-CORONAMETER");
   lentit1 = fmgetstrwidth (bfh, title1);
   xx = ((xr1 - xl1) / 2 + xl1) - (lentit1 / 2) - 1;   if (xx < 0)  xx = 2;
   yy = wysize - 1 - (int) (bfy * 0.75);
   gl_wstr (title1, xx, yy, glcp_black);

   sprintf (title2, "CORONAL pB SYNOPTIC MAP @ %4.2f Rsun", radius);
   lentit2 = fmgetstrwidth (bfh, title2);
   xx = ((xr1 - xl1) / 2 + xl1) - (lentit2 / 2) - 1;   if (xx < 0)  xx = 2;
   yy = wysize - 1 - (int) (bfy * 1.75);
   gl_wstr (title2, xx, yy, glcp_black);

   fbegdoye = (float) begdoye / 100.0;
   fenddoye = (float) enddoye / 100.0;
   fbegdoyw = (float) begdoyw / 100.0;
   fenddoyw = (float) enddoyw / 100.0;
/*
 *   printf ("begyre:%d   fbegdoye:%06.2f   endyre:%d   fenddoye:%06.2f\n",
 *	    begyre, fbegdoye, endyre, fenddoye);
 */

   doybege = syndoy (begyre, ncrot) - syndur (ncrot) * (0.25 + xtrarot);
   doybegw = syndoy (begyre, ncrot) + syndur (ncrot) * (0.25 - xtrarot);

   for (i = 0; i < NUMROT; i++)
      {
      if (CRtable [i][0] == (float) ncrot)
	 {
	 fbegdoy =       CRtable [i][1];
	 ibegyr  = (int) CRtable [i][2];

	 fenddoy =      CRtable [i+1][1];
	 iendyr = (int) CRtable [i+1][2];
	 break;
	 }
      }

   if (pmode)
      printf ("mk3syn_label> fbegdoy:%6.2f  ibegyr:%4d  fenddoy:%6.2f  iendyr:%4d\n",
	      fbegdoy, ibegyr, fenddoy, iendyr);
   ibegdoy = (int) fbegdoy;
   ienddoy = (int) fenddoy;

   month_day (ibegyr, ibegdoy, &ibegmon, &ibegday);
   month_day (iendyr, ienddoy, &iendmon, &iendday);
   strcpy (cbegmon, &mon_name [ibegmon][0]);
   strcpy (cendmon, &mon_name [iendmon][0]);
   fbegday = (float) ibegday + fbegdoy - (float) ((int) fbegdoy);
   fendday = (float) iendday + fenddoy - (float) ((int) fenddoy);
   ibegyr  = begyre;
   iendyr  = endyre;

   sprintf (crot, "%d", ncrot);
   sprintf (title3, "CARRINGTON ROTATION # %1c%1c%1c%1c   %5.2f %s %d -> %5.2f %s %d",
            crot [0], crot [1], crot [2], crot [3],
            fbegday, cbegmon, ibegyr, fendday, cendmon, iendyr);
   lentit3 = fmgetstrwidth (bfh, title3);
   sprintf (crot, "%d", ncrot);
   xx = ((xr1 - xl1) / 2 + xl1) - (lentit3 / 2) - 1;   if (xx < 0)  xx = 2;
   yy = wysize - 1 - (int) (bfy * 2.75);
   gl_wstr (title3, xx, yy, glcp_black);

   /*** Latitude labels. ***/

   afh = gl_setfont (afont, asize);	/* Small font & size. */
   xx = xl1 - afx * 3 - 10;
   gl_wstr ("+90", xx, yt1    -afy/2, glcp_black);
   gl_wstr ("  0", xx, yt1- 90-afy/2, glcp_black);
   gl_wstr ("-90", xx, yt1-180-afy/2, glcp_black);

   gl_wstr ("+90", xx, yt2    -afy/2, glcp_black);
   gl_wstr ("  0", xx, yt2- 90-afy/2, glcp_black);
   gl_wstr ("-90", xx, yt2-180-afy/2, glcp_black);

   xx = xx - afx;
   if (xx < 0)  xx = 2;
   yy = yt1 - 90;
   gl_wstr ("L", xx, yy + afy * 7, glcp_black);
   gl_wstr ("A", xx, yy + afy * 5, glcp_black);
   gl_wstr ("T", xx, yy + afy * 3, glcp_black);
   gl_wstr ("I", xx, yy + afy * 1, glcp_black);
   gl_wstr ("T", xx, yy - afy * 1, glcp_black);
   gl_wstr ("U", xx, yy - afy * 3, glcp_black);
   gl_wstr ("D", xx, yy - afy * 5, glcp_black);
   gl_wstr ("E", xx, yy - afy * 7, glcp_black);

   yy = yt2 - 90;
   gl_wstr ("L", xx, yy + afy * 7, glcp_black);
   gl_wstr ("A", xx, yy + afy * 5, glcp_black);
   gl_wstr ("T", xx, yy + afy * 3, glcp_black);
   gl_wstr ("I", xx, yy + afy * 1, glcp_black);
   gl_wstr ("T", xx, yy - afy * 1, glcp_black);
   gl_wstr ("U", xx, yy - afy * 3, glcp_black);
   gl_wstr ("D", xx, yy - afy * 5, glcp_black);
   gl_wstr ("E", xx, yy - afy * 7, glcp_black);

   xxl = xl1 - 6;
   xxr = xl1 - 2;
   gl_wvec (xxl, yt1,     xxr, yt1,     glcp_black);  /*Index mark +90 degrees.*/
   gl_wvec (xxl, yt1- 30, xxr, yt1-30,  glcp_black);  /*Index mark +60 degrees.*/
   gl_wvec (xxl, yt1- 60, xxr, yt1-60,  glcp_black);  /*Index mark +30 degrees.*/
   gl_wvec (xxl, yt1- 90, xxr, yt1-90,  glcp_black);  /*Index mark   0 degrees.*/
   gl_wvec (xxl, yt1-120, xxr, yt1-120, glcp_black);  /*Index mark -30 degrees.*/
   gl_wvec (xxl, yt1-150, xxr, yt1-150, glcp_black);  /*Index mark -60 degrees.*/
   gl_wvec (xxl, yt1-180, xxr, yt1-180, glcp_black);  /*Index mark -90 degrees.*/

   gl_wvec (xxl, yt2,     xxr, yt2,     glcp_black);  /*Index mark +90 degrees.*/
   gl_wvec (xxl, yt2- 30, xxr, yt2-30,  glcp_black);  /*Index mark +60 degrees.*/
   gl_wvec (xxl, yt2- 60, xxr, yt2-60,  glcp_black);  /*Index mark +30 degrees.*/
   gl_wvec (xxl, yt2- 90, xxr, yt2-90,  glcp_black);  /*Index mark   0 degrees.*/
   gl_wvec (xxl, yt2-120, xxr, yt2-120, glcp_black);  /*Index mark -30 degrees.*/
   gl_wvec (xxl, yt2-150, xxr, yt2-150, glcp_black);  /*Index mark -60 degrees.*/
   gl_wvec (xxl, yt2-180, xxr, yt2-180, glcp_black);  /*Index mark -90 degrees.*/

   /*** Limb labels. ***/

   xx = xr1 + (wxsize - xr1 - 1) / 2 - afx * 2 + 1;
   if ((xx + 4 * afx) > wxsize - 1) xx = wxsize - 1 - 4 * afx;

   yy = ((yb1 - yt1) / 2 + yt1) - afy/2;
   gl_wstr ("EAST", xx, yz1+afy,   glcp_black);
   gl_wstr ("LIMB", xx, yz1-afy*2, glcp_black);

   yy = ((yb2 - yt2) / 2 + yt2) - afy/2;
   gl_wstr ("WEST", xx, yz2+afy,   glcp_black);
   gl_wstr ("LIMB", xx, yz2-afy*2, glcp_black);

   /*** Draw boxes around synoptic map image area. ***/

   xbox1 [0] = xr1 + 2;
   xbox1 [1] = xr1 - nxmap;
   xbox1 [2] = xbox1 [1];
   xbox1 [3] = xbox1 [0];
   xbox1 [4] = xbox1 [0];

   ybox1 [0] = yt1 + 1;
   ybox1 [1] = ybox1 [0];
   ybox1 [2] = yt1 - nymap;
   ybox1 [3] = ybox1 [2];
   ybox1 [4] = ybox1 [0];
   gl_wlvec (xbox1, ybox1, 5, glcp_green);	/* Box frame. */

   if (pmode)
      {
      printf ("xbox1/ybox1:\n");
      printf ("%3d,%3d  %3d,%3d  %3d,%3d  %3d,%3d  %3d,%3d\n",
	   xbox1 [0],  ybox1 [0],
	   xbox1 [1],  ybox1 [1],
	   xbox1 [2],  ybox1 [2],
	   xbox1 [3],  ybox1 [3],
	   xbox1 [4],  ybox1 [4]);
      }

   xbox2 [0] = xr2 + 2;
   xbox2 [1] = xr2 - nxmap;
   xbox2 [2] = xbox2 [1];
   xbox2 [3] = xbox2 [0];
   xbox2 [4] = xbox2 [0];

   ybox2 [0] = yt2 + 1;
   ybox2 [1] = ybox2 [0];
   ybox2 [2] = yt2 - nymap;
   ybox2 [3] = ybox2 [2];
   ybox1 [4] = ybox2 [0];
   if (pmode)
      {
      printf ("xbox2/ybox2:\n");
      printf ("%3d,%3d  %3d,%3d  %3d,%3d  %3d,%3d  %3d,%3d\n",
	   xbox2 [0],  ybox2 [0],
	   xbox2 [1],  ybox2 [1],
	   xbox2 [2],  ybox2 [2],
	   xbox2 [3],  ybox2 [3],
	   xbox2 [4],  ybox2 [4]);
      }
/*   gl_wlvec (xbox2, ybox2, 5, glcp_green); */	/* Box frame. */
   gl_wvec (xr2 + 1,     yt2 + 1,     xr2 - nxmap, yt2 + 1,     glcp_green);
   gl_wvec (xr2 - nxmap, yt2 + 1,     xr2 - nxmap, yt2 - nymap, glcp_green);
   gl_wvec (xr2 - nxmap, yt2 - nymap, xr2 + 1,     yt2 - nymap, glcp_green);
   gl_wvec (xr2 + 1,     yt2 - nymap, xr2 + 1,     yt2 + 1,     glcp_green);

   gl_wvec (xl1, yt1-90, xr1, yt1-90, glcp_blue);/* Equatorial line.*/
   gl_wvec (xl1, yt2-90, xr1, yt2-90, glcp_blue);/* Equatorial line.*/

   /*** Longitude tick marks. ***/

   x0 = xl1 + (int) (xtrarot * (float) ipixrot + 0.5);
   bfh = gl_setfont (bfont, bsize);	/* Medium font & size. */

   yy = (yb1 - yt2) / 2 + yt2 - (int) (bfy*0.3);
/*   yy = yb1 - 3 - bfy; */

   for (i = 0; i <= 360; i+=10)
      {
      xx = x0 + i;
      gl_wvec (xx, yb1-2, xx, yb1-6, glcp_red);	/* longitude tick mark.*/
      gl_wvec (xx, yt2+2, xx, yt2+6, glcp_red);	/* longitude tick mark.*/
      sprintf (label, "%d", i);
      if (i % 30 == 0)				/* Write thick tick mark. */
	 {
	 xxl = xx - 1;
	 xxr = xx + 1;
         gl_wvec (xxl, yb1-2, xxl, yb1-6, glcp_red);   /* longitude tick mark.*/
         gl_wvec (xxr, yb1-2, xxr, yb1-6, glcp_red);   /* longitude tick mark.*/

         gl_wvec (xxl, yt2+2, xxl, yt2+6, glcp_red);   /* longitude tick mark.*/
         gl_wvec (xxr, yt2+2, xxr, yt2+6, glcp_red);   /* longitude tick mark.*/
	 }
      if (i % 90 == 0 && i != 180)		/* Write longitude labels. */
	 {
         lenstr = fmgetstrwidth (bfh, label);
         xx -= lenstr / 2;
         gl_wstr (label, xx, yy, glcp_red);
/*
 *         gl_wstr (label, xx, yy, glcp_red);
 *         gl_wstr (label, xx, yt2 +9, glcp_red);
 */
	 }
      }
   xx = x0;
   gl_wvec (xx, yt1, xx, yb1-6, glcp_red);   /* longitude tick mark.*/
   gl_wvec (xx, yb2, xx, yt2+6, glcp_red);   /* longitude tick mark.*/
   xx = x0 + 360;
   gl_wvec (xx, yt1, xx, yb1-6, glcp_red);   /* longitude tick mark.*/
   gl_wvec (xx, yb2, xx, yt2+6, glcp_red);   /* longitude tick mark.*/

   /*** Longitude label. ***/

   bfh = gl_setfont (bfont, bsize);	/* Medium font & size. */
   lenlong = fmgetstrwidth (bfh, lablong);
   xx = (int) (xl1 + (float) ipixrot * (0.5 + xtrarot) + 0.5)
	       - (lenlong / 2) - 2;
   yy = (yb1 - yt2) / 2 + yt2 - (int) (bfy*0.3);
   gl_wstr (lablong, xx, yy, glcp_red);

   /*** DOY labels. ***/

   /* * * * * * * * * * * * * * * * * * * * * * * * *
    *   Set up starting DOY values and x locations. *
    * * * * * * * * * * * * * * * * * * * * * * * * */

/*
 *   printf ("begyre: %d   ncrot: %d\n", begyre, ncrot);
 */
   crbeg = (float) ncrot - xtrarot;
   crend = (float) ncrot + xtrarot + 1.0;
   doyrot = syndur (ncrot);		/* # days in rotation `ncrot'. */
   pixdoy = 360.0 / syndur (ncrot);

/*
 *   printf ("syndoy(%d,%d): %6.2f   syndur(%d): %6.2f\n",
 *	    begyre, ncrot, syndoy (begyre, ncrot),
 *	            ncrot, syndur (begyre, ncrot));
 *   printf ("doyrot:%6.2f   pixdoy:%6.2f\n", doyrot, pixdoy);
 */

   if ((begyre/4) * 4 == begyre)  doymax = 366.0;  else doymax = 365.0;
   if (doybege > doymax)  doybege -= doymax;
   if (doybegw > doymax)  doybegw -= doymax;
/*
 *   printf ("doybege: %f   doybegw: %f\n", doybege, doybegw);
 */

   bfh = gl_setfont (bfont, bsize);	/* Medium font & size. */

   /*** DOY label. ***/

   strcpy (titdoy, "IMAGE DAY-OF-YEAR");
   lendoy = fmgetstrwidth (bfh, titdoy);
   xx = xr1 - (int) ((xtrarot + 0.5) * (float) ipixrot + 0.5) - lendoy/2;
   yy = yt1 + 15 + bfy;
/*   gl_wstr (titdoy, xx, yy, glcp_green); */
   yy = yb2 - 6 - bfy * 2;
   if (yy < 2)
      yy = 2;
   gl_wstr (titdoy, xx, yy, glcp_green);

   /*** East DOY initialization. ***/

   ndoye = (int) doybege;
   ibege = ndoye + 1;
   iende = ndoye + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
   posdoye = (float) xr1 + pixdoy * (doybege - (float) ndoye);
/*
 *   printf ("ibege: %3d   iende: %3d   NROT: %d   xtrarot: %f   doyrot: %f\n",
 *            ibege, iende, NROT, xtrarot, doyrot);
 */

   /*** West DOY initialization. ***/

   ndoyw = (int) doybegw;
   ibegw = ndoyw + 1;
   iendw = ndoyw + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
   posdoyw = (float) xr1 + pixdoy * (doybegw - (float) ndoyw);

   /*** Compute East DOYs and annotate Grinnell. ***/

   for (idoye=ibege; idoye<iende; idoye++) /* Days increase right to left. */
      {
      posdoye -= pixdoy;
      locdoye = (int) (posdoye + 0.5);
/*
 *      printf ("idoye:%d   posdoye:%6.2f  locdoye:%d   xl1:%d   xr1:%d\n",
 *               idoye, posdoye, locdoye, xl1, xr1);
 */
      if (locdoye >= xl1 && locdoye <= xr1)
         {
         gl_wvec (locdoye, yt1+2, locdoye, yt1+6, glcp_green);
         if (idoye/5*5 == idoye)   /* Write 2 more tick marks every 5 deg. */
            {
	    gl_wvec (locdoye-1, yt1+2, locdoye-1, yt1+6, glcp_green);
	    gl_wvec (locdoye+1, yt1+2, locdoye+1, yt1+6, glcp_green);
	    sprintf (sdoy, "%d", idoye);
/*
 *	    printf ("sdoy: %s\n", sdoy);
 */
            lenstr = fmgetstrwidth (bfh, sdoy);
            lxdoye = locdoye - lenstr / 2;
	    lydoye = yt1 + 11;
	    if (idoye/10*10 == idoye)	/* Write DOY every 10 days.*/
	       gl_wstr (sdoy, lxdoye, lydoye, glcp_green);
            }
         }
      }

   /*** Compute West DOYs and annotate Grinnell. ***/

   for (idoyw=ibegw; idoyw<iendw; idoyw++) /* Days increase right to left. */
      {
      posdoyw -= pixdoy;
      locdoyw = (int) (posdoyw + 0.5);
/*
 *      printf ("idoyw:%d   posdoyw:%6.2f  locdoyw:%d   xl1:%d   xr1:%d\n",
 *                idoyw, posdoyw, locdoyw, xl1, xr1);
 */

      if (locdoyw >= xl1 && locdoyw <= xr1)
         {
         gl_wvec (locdoyw, yb2-2, locdoyw, yb2-6, glcp_green);
         if (idoyw % 5 == 0)   /* Write 2 more tick marks every 5 deg. */
            {
	    gl_wvec (locdoyw-1, yb2-2, locdoyw-1, yb2-6, glcp_green);
	    gl_wvec (locdoyw+1, yb2-2, locdoyw+1, yb2-6, glcp_green);
	    sprintf (sdoy, "%d", idoyw);
            lenstr = fmgetstrwidth (bfh, sdoy);
            lxdoyw = locdoyw - lenstr / 2;
	    lydoyw = yb2 - 3 - bfy;
	    if (idoyw/10*10 == idoyw)	/* Write DOY every 10 days.*/
	       gl_wstr (sdoy, lxdoyw, lydoyw, glcp_green);
            }
         }
      }

   return (0);
}
