/* mk3synras.c
 * * * ** * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an MLSO Mark-III synoptic map (or a sequence of maps)
 * on the SGI IRIS screen using GL library software.
 * A GL window is automatically created on the IRIS screen.
 * Uses onemap color mode, with an index offset of 512.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P   mk3syn.c
 * 26 April 1991: use onemap color mode.
 *  7 December 1992: add sequence + forward/backward options.
 *  8 December 1992: use mouse instead of keyboard; add wrap-around option (^).
 * 19 August 1996: Version to offer 1,2,3,4,5,6 rotations per map.
 * 18 Mar 2002: renamed "mk3synras.c".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#define MAXIMG 250
int pmode = 0;			/* Print mode. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];

{
   FILE *LIST;
   short val;

   static char fliste [MAXIMG][80];	/* east filelist. */
   static char flistw [MAXIMG][80];	/* west filelist. */
   static char nampgm [80];	/* Program file name. */
   static char eastfile [80];	/* East synoptic data file name. */
   static char westfile [80];	/* West synoptic data file name. */
   static char namfil [80];	/* Synoptic data file name. */
   static char namlst [80];	/* Namelist file name. */

   int mk3syn5_view ();		/* Image display function. */
   int irot ;			/* Rotation index. */
   int nrot ;			/* # rotations to display. */
   int begrot, endrot ;		/* Beginning & Ending rotations to display. */
   int findex = 0;
   int i;			/* Index. */
   int ierr;			/* Error status from cpview(). */
   int lastimg = 0;		/* Last image index value. */

   int wxsize = 512;		/* Window width. */
   int wysize = 512;		/* Window height. */
   int xll, yll;		/* Lower left  window coordinates. */
   int xur, yur;		/* Upper right window coordinates. */

   float radius ;		/* Radial height at which synoptic scan is taken. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   strcpy (nampgm, argv [0]);

   /*** Request information if not enough arguments are provided. ***/

   /*** Three arguments: #rotations, first rotation, radius. ***/

   if (argc == 4)
      {
      sscanf (argv [1], "%d", &nrot) ;
      sscanf (argv [2], "%d", &begrot) ;
      sscanf (argv [3], "%f", &radius) ;
      endrot = begrot + nrot - 1 ;
      }

   /*** Invalid number of command line arguments. ***/

   else
      {
      fprintf (stderr, "usage: %s  #_rotations  first_rotation_# radius\n",
	       argv [0]);
      exit (-1);
      }

   if (pmode)
      printf ("mk3synras> nrot: %d   begrot: %d  endrot: %d\n",
              nrot, begrot, endrot);

   /*** Initialize GL software. ***/
   /* Put synoptic map window in upper right corner of screen.*/

   xur = 1271;
   yur =  991;
   xll = xur - wxsize;
   yll = yur - wysize;

   foreground ();
   prefposition (xll, xur, yll, yur);
   fminit ();			/* Initialize SGI font manager. */

   winopen (argv [0]);		/* Create GL window. */
   gl_wclear (BLACK);		/* Clear window to black. */

   doublebuffer ();		/* Use double buffer mode. */
   gconfig ();

   /*** Display image on IRIS screen in GL window. ***/

   mk3syn_view (wxsize, wysize, begrot, endrot, radius) ;

quit:
   gexit ();
   exit (0);
}
