        /*  This is the standard mk4 definitions and
	    global parameters file.

            Written by Alice Lecinski, June 2001.

            I strongly suggest *not* editting this code at all, unless
            you are absolutely sure you've found an error that
            needs fixing.

            If you edit it, you risk destroying the integrity of the
            code.  So if you do edit it, insure results pre and post
            actually match.                                            */


#define TRUE        (1) 
#define FALSE       (0) 

#define JULD_REF    (2415021)  /* 2415021 Juld reference date, 1 Jan 1900. */
#define MAY01_2001  (37010)
#define MAY14_2001  (37023)

#define SHORTS    ( 16)
#define FLOATS    (-32)

#define NAZ      (720)
#define NPX      (384)
#define CY_IMG   (1080*384) /* Have to keep this big to hold raw data. */
#define VECTOR   (1080)     /* Have to keep this big to hold raw data. */

        /*  SQ  is the dimension  of the square image produced
            when converting to rectangular coordinates.
            TRY is how big to expand the NAZxNPX cylindrical image
            before doing the conversion.
            Can go up to SQ==700 , with TRY==2 , and not
            suffer too badly from  "pixelation" problems.
            With TRY==2, it takes  8 sec to do an image.
            With TRY==3, it takes 12 sec to do an image. */

#define SQ       (512)
#define TRY      (2)

        /*  If you need to go make a square image bigger than
            512x512, redefine
            SQ and TRY */

#ifdef BIGSCALE
#undef           SQ
#undef           TRY
#define          SQ ( 960)
#define          TRY   (5)
#endif

#define RSIZE    (SQ*SQ*2)        /* byte size of rectangular array */
#define SIZE     (SQ*SQ)          /* size of rect array as shorts */
#define XSIZE    (SQ)             /* x size of rect array */

        /*  LOWPIX and HIGHPIX are used only when sampling the image
            to determine mins and maxes when rescaling to a byte
            image in "scale2byte".                               */

#define LOWPIX   (70)
#define HIGHPIX  (250)

        /*  Bottom and top pixels used in cy2rec4 when scaling to
            a byte image in scale2byte.  In that routine,
            pixels below and above are set to zero.               */

#define BOTPIX   (33)
#define TOPPIX   (NPX)

        /*  The backlash tries to compensate for backlash
            in the barrel when going from a forward to
            reverse direction.
            a reverse scan gets a -BACKLASH
            a forward scan gets a +BACKLASH
            After all is said and done in "fshiftimg"
            the signs of the above are reversed.                   */

#define BACKLASH (0.5) /* In degrees */


        /*  During the May 1, 2001 refurb, David found out
 	    some interesting things about the phasing of
	    the data collection involved in the skytx data.
	    Prior, the skytx could not be guarenteed to be
	    the maximum for a "ray".
	    Post, now it is!  Yay David!
	    AMASS_SLOPE and AMASS_YCEPT are used to compute a SkyTx
	    to compensate for the bad values prior to the refurb
	    and for a little while there after (until May 14, 2001).
	    These values were computed from data gathered 
	    by David Elmore using May 14-17, 2001 skytx's.
            */

#define SKYOFF   (0.)
#define MINSKY   (200)
#define LOWSKY   (300)
#define AMASS_SLOPE  (-11.0)
#define AMASS_YCEPT  (376.0)


/* Observation Type (what) == mk4h->ObsType */
#define IS_cal0               0
#define IS_cal15              150
#define IS_dark               9999
#define IS_coronal            1000

/* Data Type (who) mk4h->DataType  */
#define IS_i                  0
#define IS_q                  1
#define IS_u                  2
#define IS_ba                 3
#define IS_sk                 4
#define IS_st                 5
#define IS_tscp               6
#define IS_drift              7
#define IS_tscp_inv           8

/* Processing steps (howfar) mk4h->ProcSteps */
#define IS_raw                10
#define IS_raw_spln           11
#define IS_raw_rect           12
#define IS_cal                13
#define IS_cal_slit           14
#define IS_cal_rect           15
#define IS_cal_rect_slit      16
#define IS_cal_rect_byte      17  /* rpB image scaled from 0-255 for display. */
#define IS_cal_avg            18
#define IS_cal_avg_slit       19
#define IS_cal_avg_rect       20
#define IS_cal_avg_rect_slit  21

#define IS_raw_avg            24
#define IS_raw_avg_spln       25
#define IS_raw_avg_rect       26

#define IS_UNKNOWN            99

#ifdef START
    double twopi  =  3.14159265358979 *2.;
    double pi     =  3.14159265358979;
    double r2d    = 57.29577951308232;
    double d2r    =  0.01745329251994;
    double piov2  = 1.57079632679490;
    double piov4  = 0.78539816339745;
    double ba2rad = 0.000383495196971410;     /* 2 pi   / 16384 */
    double ba2deg = 0.02197265625;            /* 360.   / 16384 */
    double deg2ba = 45.511111111;             /* 16384  / 360. */

    char *prtime();

    char picnm[64] , *piceos ;
    int mon, day, year, doy ;
    int dayflag, timeflag;
    int Secflag ;
    int nameflag;
    int doyflag ;
    unsigned int tim;

    float badval = -9999.;

#else
    extern double twopi ;
    extern double    pi ;
    extern double r2d ;
    extern double d2r ;
    extern double piov2  ;
    extern double piov4  ;
    extern double ba2rad ;
    extern double ba2deg ;
    extern double deg2ba ;

    extern char *prday();
    extern char *prtime();
    extern int  get_hrminsec();
    extern double pangle();
    extern double phi();
    extern int    idoy() ;
    extern int    get_doy() ;
    extern int    jday() ;
    extern int    jdate() ;
    extern int    idate() ;
    extern int    isleap() ;

    extern char picnm[64] , *piceos ;
    extern int mon, day, year, doy ;
    extern int dayflag, timeflag;
    extern int Secflag ;
    extern int nameflag;
    extern int doyflag ;
    extern unsigned int tim;

    extern float badval;

#endif
