/* fixbias.c
 *------------------------------------------------------------------------------
 * Fix Mk4 bias error.
 *------------------------------------------------------------------------------
 * Author: Tony Darnell   HAO/NCAR   18 May 2001
 *------------------------------------------------------------------------------
 */

#include "mk4.h"		/* NAZ, NPX definitions. */

int
FixBias (img, gain)      

  float *img;		/* float  img [NPX][NAZ]   (polar coord. image) */
  float *gain;		/* float gain [NPX]        (from corrector table) */
{
    int it, ir;
    float sum = 0.0, biascor [NAZ];

    /* Correct gain for large numbers */

    for (it = 0; it < NPX ; it++)
        {
        *gain /= 65536.0;
        if (*gain > 1.0) {*gain = 1.0;}
        gain++;
        }

    /* Compute the average of the first 10 pitels
     * along the radial dimension of "img".
     */

    for (it = 0; it < NAZ; it++)
	{

	/* Compute a bias correction for each azimuth by
         * finding the average of the first 10 pitels
         * along the radial dimension of "img".
         */

	sum = 0.0; 
        for (ir = 0; ir < 10; ir++)
	    {
            sum += *(img + it + ir * NAZ);
            }
        sum /= 10.0;
	biascor [it] = sum;

        /* Apply the bias correction to the image. */

        for (ir = 0; ir < NPX; ir++)
	    {
            *(img + it + ir * NAZ) =
		*(img + it + ir * NAZ) - (biascor [it] * *(gain + ir));
            }
        }

    return (0) ;
}

