#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/*
  Every program which uses the CFITSIO interface must include the
  the fitsio.h header file.  This contains the prototypes for all
  the routines and defines the error status values and other symbolic
  constants used in the interface.  
*/
#include "fitsio.h"

int main( void );
void writeimage( void );
void writeascii( void );
void writebintable( void );
void copyhdu( void );
void selectrows( void );
void readheader( void );
void readimage( void );
void readtable( void );
void printerror( int status);

int main ()
{
/*******************************************************************************
   This is a simple main program that calls the following routines:

    copyhdu       - copy a header/data unit from one FITS file to another

*******************************************************************************/
    copyhdu ();

    return (0);
}

/*----------------------------------------------------------------------------*/

void copyhdu (void)
{
    /********************************************************************/
    /* copy the 1st and 3rd HDUs from the input file to a new FITS file */
    /********************************************************************/
    fitsfile *infptr;      /* pointer to the FITS file, defined in fitsio.h */
    fitsfile *outfptr;                 /* pointer to the new FITS file      */

    char infilename[]  = "20000331.180645.mk4.cpb.fts";  /* existing file */
    char outfilename[] = "20000331.180645.mk4.cpb.vig.fts";  /* new file  */

    int status, morekeys, hdutype;

    status = 0;

    remove (outfilename);            /* Delete old file if it already exists */

    /* Open the existing FITS file */

    if ( fits_open_file (&infptr, infilename, READONLY, &status) ) 
         printerror ( status );

    if (fits_create_file (&outfptr, outfilename, &status)) /*create FITS file*/
         printerror ( status );           /* call printerror if error occurs */

    /* Copy the primary array from the input file to the output file */

    morekeys = 0;     /* don't reserve space for additional keywords */
    if ( fits_copy_hdu (infptr, outfptr, morekeys, &status) )
         printerror ( status );

    /* Move to the 2nd HDU in the input file */

    if ( fits_movabs_hdu (infptr, 2, &hdutype, &status) )
         printerror ( status );

    /* Copy 2nd HDU from the input file to the output file (to 2nd HDU) */

    if ( fits_copy_hdu (infptr, outfptr, morekeys, &status) )
         printerror ( status );

    if (fits_close_file (outfptr, &status) ||
        fits_close_file (infptr, &status)) /* close files */
         printerror ( status );

    return;
}

/*--------------------------------------------------------------------------*/
