/* nextday.c
 * -----------------------------------------------------------------------------
 * Construct the next day's date.
 *
 * syntax: nextday yy_mm_dd
 * e.g.,   nextday 01_02_03	(2001 Feb 3)
 * -----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR    3 Jan 2003
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern int isleap () ;

    int year1;
    int y1 ;
    int m1 ;

    int d1 ;
    int y2 ;
    int m2 ;
    int d2 ;

    int leap ;
    int maxday ;

    int days_in_month [2][12] =
	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31,
         31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31} ;

    if (argc < 2)
	{
	fprintf (stderr, "usage: %s yy_mm_dd\n", argv [0]) ;
	exit (0) ;
	}

    sscanf (argv [1], "%d_%d_%d", &y1, &m1, &d1) ;

    year1 = y1 ;
    if (y1 < 50)  year1 += 2000 ;
    if (y1 > 50)  year1 += 1900 ;

    leap = isleap (y1) ; 

    maxday = days_in_month [leap][m1-1] ;

    y2 = y1 ;
    m2 = m1 ;
    d2 = d1 + 1 ;

    /*** If next day exceeds the last day of the month,
     *** it must be the first day of the next month.
     ***/

    if (d2 > maxday)
	{
	m2 = m1 + 1 ;
	d2 = 1 ;

	/*** If the next day is beyond the last day of the year,
	 *** it must the the first day of the next year.
	 ***/

	if (m2 > 12)
	    {
	    y2 = y1 + 1 ;
	    m2 = 1 ;
	    d2 = 1 ;
	    }
	}

    fprintf (stdout, "%02d_%02d_%02d\n", y2, m2, d2) ;

    exit (0) ;
}

/* -------------------------------------------------------------------------- */

/* Gregorian leap year calculation */

int
isleap (yr)

int yr;
{
    if ((yr & 3) != 0)
	return (0);		/* not divisible by 4, not leap */
    if ((yr % 100) != 0)
	return(1);		/* not divisible by 100 but by 4, a leap */
    if ((yr % 400) != 0)
	return(0);		/* not divisible by 400 but by 100, not a leap*/
    if ((yr % 4000) != 0)
	return(1);		/* not divisible by 4000, a leap */
    return (0);			/* divisible by 4000, not a leap */
}

/* -------------------------------------------------------------------------- */
