/* polvig.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Apply an artificial 'vignetting' to polar coordinate mk4 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adapted from 'cy_slit4.c', authored by Alice Lecinski
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   4 April 2000
 * 29 Nov 2000: Replace "dofn" with dofname4 (use "vig" instead of "slit").
 * 16 May 2001: Add David Elmore's May 2001 model.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define START

#include "mk4.h"
#include "mk4head.h"
#include "model4.h"
#include "model4_May2001.h"
#include "crunch.h"
#include "telscp4.h"

#define MODEL  1
#define CORONA 2
#define TSCP   3
#define MAY2001MODEL  4

char *filenm, picnm [64], *piceos ;
char tscpfile [80] ;

short  MyIm [CY_IMG] ;
short  MyBA [VECTOR] ;

float Qim [CY_IMG] ;
float Aim [CY_IMG] ;
float meanQ [VECTOR] ;

/* -------------------------------------------------------------------------- */

main (an, av)

char *av [] ;
{
    FILE *fid ;

    register char *w ;

    long begtime, endtime ;
    long btime, etime ;
    int ii ;
    int howfar ;
    int dotest ;
    int method ;
    int doelmore ;
    int lozero ;
    int lspot ;
    float *pf, ftemp ;
    float MyOffset ;
    double dtmp ;
    struct tm *T, *localtime () ;
    char *ctime_r (), buf [30] ;


    if ( an <= 1 ) { prsum () ; exit (3) ; }

    (void) time (&begtime) ;
    T = (struct tm *) malloc (sizeof (struct tm)) ;
    T = localtime (&begtime) ;
    T->tm_mon += 1 ;

    (void) ctime_r (&begtime, &buf, 30) ;
    fprintf (stderr, "Started polvig:      %s\n", buf) ;

    timeflag = 1 ;
    dotest = 0 ;
    method = MODEL ;
    lspot = 32 ;

    while (--an)
	{		/* Read in the files and perform the calibration */
	w = *++av ;
	if (*w == '?') { prsum () ; exit (2) ; }
	if (*w == '-')
	    {				/* Check for flag arguments. */
	    switch (w [1])
		{
		case 'L': lspot = LOZERO ;
			  break ;
		case 'C': method = CORONA ; 
			  break ;
		case 'T': method = TSCP ; 
			  an-- ;   w = *++av ;   sscanf (w, "%s", tscpfile) ;
			  readtscp (tscpfile, thd, 1) ;
			  pf = thd->DriftRefVect ;
			  ftemp  = *(pf + 45) ;
			  ftemp += 400.0 ;
			  for (ii = 0; ii < 45; ii++)
			      {
			      *pf++ = ftemp ;
			      }
			  break ;
		case 'M': method = MAY2001MODEL ;
			  break ;
		case 'O': an-- ; w = *++av ; sscanf (w, "%f", &MyOffset) ;
			  break ;
                default:  fprintf (stderr, " unknown flag %s\n", *av) ;
                          prsum () ; exit (1) ;
                }
	    continue ;
	    }
	filenm = w ;

	(void) time (&btime) ;


	/* Check on the file naming convention used on the file */

	if ((*w == '0') || (*w >= '8' && *w <= '9') || *(w + 2) == 'd' )
	    {
	    if ( *(w + 3) >='0' && *(w+3) <='9' )  doyflag++ ;
	    else                                   nameflag++ ;
	    }
	if (nameflag || doyflag) { dayflag++ ; timeflag=1 ; }


	/* Read mk4 image & convert to floating point. */

        readmk4 (filenm, mk4h, MyIm, 0) ; 
	Short2Float (MyIm, Qim, mk4h->Xdim, mk4h->Ydim) ;

	HandleNeg_pB (Qim, mk4h->Xdim, mk4h->Ydim, lspot) ;

	switch (method)
	    {
	    case MAY2001MODEL:
		VVcopy (May2001model, meanQ, NPX);
		ftemp = MyOffset ;
		Vplus (meanQ, ftemp, NPX);
		ftemp = mk4h->data_offset;
		Vminus (Qim, ftemp, mk4h->Xdim * mk4h->Ydim);
		break ;
	    case MODEL:
		model_RadVect (meanQ) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi_preM", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi_pre write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof (float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
		break ;
	    case CORONA:
		VVcopy (mymodel, meanQ, NPX) ;
		ftemp = mk4h->data_offset - 50.0 ;
		Vminus (meanQ, ftemp, NPX) ;
		ftemp = mk4h->data_offset - 100.0 ;
		Vminus (Qim, ftemp, mk4h->Xdim * mk4h->Ydim) ;
		break ;
	    case TSCP:
		VVcopy (thd->DriftRefVect, meanQ, NPX) ;
		ftemp = mk4h->data_offset - 10.0 ;
		Vminus (Qim, ftemp, mk4h->Xdim * mk4h->Ydim) ;
		break ;
	    default:
		Im2AvgRadVect_Culled (Qim, meanQ, mk4h->Xdim, mk4h->Ydim) ;
		fixLO (meanQ, 8000.0, -1.0) ;
		fixHI (meanQ,  200.0) ;
		smoothie (meanQ, mk4h->Ydim, 13) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi_pre", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi_pre write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof (float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
	    }
	MVdivide (Qim, meanQ, Qim, mk4h->Xdim, mk4h->Ydim) ;


	switch (method)
	    {
	    case MODEL:
		model_Resid_orig2 (meanQ) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfiM", "w"))<(FILE*)0)
			{
			fprintf (stderr, "barfi write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof (float), mk4h->Ydim, fid) ;
		    fclose (fid) ;
		    }
		break ;
	    case CORONA:
		Vinitialize (meanQ, 0.0, NPX) ;
		break ;
	    case TSCP:
		Vinitialize (meanQ, 0.0, NPX) ;
		break ;
	    case MAY2001MODEL:
		Vinitialize (meanQ, 0.0, NPX) ;
		break ;
	    default:
		VVcopy       (Qim, Aim,    mk4h->Xdim * mk4h->Ydim) ;
		get_min_Vect (Aim, meanQ,  mk4h->Xdim, mk4h->Ydim) ;
		fixLO (meanQ, 0.0, 0.6) ;
		fixHI (meanQ, 0.9) ;
		smoothie (meanQ, mk4h->Ydim, 13) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof(float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
	        }
	MVminus (Qim, meanQ, Qim, mk4h->Xdim, mk4h->Ydim) ;

	switch (method)
	    {
	    case CORONA:
		Vinitialize (meanQ, 100.0, NPX) ;
		break ;
	    case TSCP:
		Vinitialize (meanQ, 180.0, NPX) ;
		break ;
	    case MODEL:
	    default:
		/* Put a little extra zing into the outer field. */
		pf = meanQ ;
		for (ii = 0; ii < mk4h->Ydim; ii++)
		    {
/*
		    dtmp  = (double) ii ;
		    ftemp = (float) (pow (dtmp, 1.35)) ;
		    ftemp = (float) (pow (dtmp, 1.20)) ;
		    ftemp = (float) (pow (dtmp, 1.01)) ;
		    if (ii > 180)
			{
			dtmp  -= 280.0 ;
			ftemp -= (float) (pow (dtmp, 1.60)) ;
			dtmp  -= 180.0 ;
			ftemp -= (float) (pow (dtmp, 1.30)) ;
			dtmp  -= 180.0 ;
			ftemp -= (float) (pow (dtmp, 1.00)) ;
		        }
*/
		    ftemp = 100.0 + 2.0 * ((float) ii) ;
		    ftemp = 100.0 + 1.0 * ((float) ii) ;
		    if (ii > 372) ftemp = 0.01 ;
		    *pf++ = ftemp ;
		    }
	    }
	MVmult (Qim, meanQ, Qim, mk4h->Xdim, mk4h->Ydim) ;



	/* Fix and fill in the headers with new info. */

	switch (mk4h->ProcSteps)
	    {
	    case IS_cal               : howfar = IS_cal_slit ; break ;
	    case IS_cal_avg           : howfar = IS_cal_avg_slit ; break ;
	     
            default                   : howfar = IS_UNKNOWN ; break ;

	    }
	mk4h->ProcSteps = howfar ;
	mk4h->data_scale  = badval ;
	mk4h->data_offset = badval ;


        lozero = LOZERO ;
        zminlo (Qim, mk4h->Xdim, lozero) ;


	/* Convert floating point image to short integer. */
	/* Write 'vignetted' image to disk. */

        Float2Short (Qim, MyIm, mk4h->Xdim, mk4h->Ydim) ;
	dopicnm (mk4h) ;
/*
 *        dofnm (mk4h->DataType, mk4h->ProcSteps) ;
 */
	dofname4 (mk4h->DataType, mk4h->ProcSteps) ;
	writemk4 (picnm, mk4h, MyIm, 1) ;

	(void) time (&etime) ;
	fprintf (stderr, 
	"\t\t\tSingle image time used: %ld seconds\n\n", etime-btime) ;
        }

    (void) time (&endtime) ;
    fprintf (stderr, "\nTotal time used  :    %ld seconds\n", endtime-begtime) ;

    exit (0) ;

}

/* -------------------------------------------------------------------------- */

prsum () {
    static char help [] = "\
polvig [-L] [-C] [-T tscpfile] [-M [-O offset]] [polar file1] [polar file2] ... [polar fileN]\n\
\n\
Example: polvig 92d004.17_28.mk4.pB  92d004.17_32.mk4.pB\n\
Example: polvig -L -M -O 100. `ls -1 *.mk4.pB`\n\
\n\
\n\
Flags          Description\n\
-----          -----------\n\
-L             Only fixes neg pB values down to pixel LOZERO instead of\n\
	       pixel 32 (the default).\n\
-C             Will use a coronal model as the vignetting function\n\
-T tscpfile    Will use the DriftRefVect from the tscp file at the\n\
	       vignetting function.\n\
-M             Will use David Elmore\'s May 2001 coronal model for the\n\
               vignetting function.\n\
-O offset      The offset to use in the May 2001 model, where:\n\
	       pBvig = pB / (May_2001_model + offset).\n\
\n\
" ;

       fputs (help, stderr) ;
}


/* -------------------------------------------------------------------------- */

int fixLO (Vect, flim, frepla)

float *Vect ;
float flim ;
float frepla ;
{
    int ii ;
    float *pf, fval ;

    ii   = 50 ;
    pf   = Vect+ii ;
    while ((ii > 0) && (*pf > flim)) { ii-- ; fval = *pf ; pf-- ; }

    if (frepla > 0.0) fval = frepla ;

    while (ii > 0)                { ii-- ; *pf = fval ; pf-- ; }


}

/* -------------------------------------------------------------------------- */

int fixHI (Vect, flim)

float *Vect ;
float flim ;
{
    int ii ;
    float *pf, fval ;

    ii   = 260 ;
    pf   = Vect + ii ;
    while ((ii < 384) && (*pf > flim)) { ii++ ; fval = *pf ; pf++ ; }

    while (ii < 384)                { ii++ ; *pf = fval ; pf++ ; }

}

/* -------------------------------------------------------------------------- */

int
model_RadVect_table (Vect)

float *Vect ;
{
    int ii ;
    float *pf, *pm, fval ;
    float hlim, llim ;

    hlim = 10000. ;
    llim = 1040. ;

    pf = Vect ;
    pm = mymodel ;
    for (ii = 0; ii < NPX ; ii++)
	{
	fval  = *pm ;
	fval *= 0.3 ;
	if     (fval <=    0.0) fval = hlim ;
	else if (fval >  hlim ) fval = hlim ;
	else if (fval <  llim ) fval = llim ;
	else ;
	*pf = fval ;
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

int
model_RadVect (Vect)

float *Vect ;
{
    int ii ;
    float *pf, fval, tval, slp, ycpt, y2, y1, x2, x1 ;
    float hlim, llim ;

    hlim = 10000.0 ;
    llim = 1040.0 ;
    llim = 1000.0 ;
    llim = 1040.0 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	tval = (float) ii ;
	fval = tval * 0.1 - 4.2 ;
	fval = 500.0 + 12000.0 / fval ;
	if (ii >= 40 && ii <= 70) fval *= 1.0 * ( tval) / ( 70.0) ;
	if      (fval <=    0.0) fval = hlim ;
	else if (fval >   hlim ) fval = hlim ;
	else if (fval <   llim ) fval = llim ;
	else ;
	*pf = fval ;
	pf++ ;
        }


    x1 = 220.0 ;
    ii = (int) x1 ;
    y2   = *(Vect + 383) ;   x2 = 383.0 ;
    y1   = *(Vect + ii ) +10.0 ;
    slp  = (y2 - y1) / (x2 - x1) ;
    ycpt = y1 - slp*x1 ;
    ii = (int) x1 ;
    pf = Vect + ii ;
    for (ii = (int) x1; ii < NPX; ii++)
	{
	tval = (float) ii ;
	fval = slp * tval + ycpt ;
	fval = (fval - (tval-x1) * 1.00) ;
	if (fval < llim ) fval = llim ;
	*pf = fval ;
	pf++ ;
        }
}


/* -------------------------------------------------------------------------- */

int
model_Resid_orig (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    double dval ;
    double dlim, hlim, ov600 ;

    hlim= 1.0 ;
    ov600 = 1.0 / 600.0 ;
    ov600 = 1.0 / 900.0 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	dval = (float) ii * pi * ov600 ;
	/* dval = sin (dval) * 0.5 + 0.4 ; */
	dval = sin (dval) * 0.6 + 0.5 ;
	if (dval > hlim ) dval = hlim ;
	*pf = (float) dval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

int
model_Resid_orig2 (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    float fval ;


    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	fval = ((float) ii -380.0) * 0.0025 ;
	fval = ((float) ii -400.0) * 0.0025 ;
	fval = 1.0 - fval * fval ;
	*pf = fval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

int
model_Resid (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    float fval ;
    float hlim ;

    hlim = 1.2 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	fval = (float) ii * 0.0015 + 0.5 ;
	if (fval > hlim ) fval = hlim ;
	*pf = fval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

