/*
                readmk4  (filenm,img,doprint)
                writemk4 (filenm,img,doprint)

		Reads/Writes a mk4 header and img to/from
		    the file "filenm".

		"img" is assumed to be a short array.
*/

#include                <stdio.h>
#include                "mk4v2.h"
#include                "mk4headv2.h"
#include                "telscp4v2.h"

int
printit(astring) char *astring;
{
    fprintf(stderr,"\t%s\t",astring);
}

int
readmk4head(filenm,hd,doprint) 
    char *filenm; 
    struct mk4_header *hd; 
    int doprint;
{
    int ii,hdsz;
    FILE *fid;

    if(doprint>0) fprintf(stderr,"\tReading header from: %s \n",filenm);

    if ( ( fid = fopen(filenm,"r") ) == NULL  ) {
	fprintf(stderr,"Error opening %s for reading.\n",filenm);
	exit(-90);
    }

    hdsz = sizeof(mk4head);

    if( ( ii = fread(hd, 1, hdsz ,fid)) != hdsz ) {
	fprintf(stderr,"Header read error:\n\
	    Only read %d bytes from %s\nExpected %d", ii,filenm,hdsz);
	fclose(fid);
	exit(-92);
    }
    fclose(fid);
}

int
readmk4(filenm,hd,img,doprint) 
    char *filenm; 
    struct mk4_header *hd; 
    short *img; 
    int doprint;
{
    int ii,imsz,hdsz;
    FILE *fid;

    if(doprint>0) fprintf(stderr,"\tReading %s\n",filenm);

    if ( ( fid = fopen(filenm,"r") ) == NULL  ) {
	fprintf(stderr,"Error opening %s for reading.\n",filenm);
	exit(-90);
    }

    hdsz = sizeof(mk4head);

    if( ( ii = fread(hd, 1, hdsz ,fid)) != hdsz ) {
	fprintf(stderr,"Header read error:\n\
	Only read %d bytes from %s\nExpected %d",
		        ii,filenm,hdsz);
	fclose(fid);
	exit(-92);
    }

    imsz = (hd->Xdim)*(hd->Ydim);

    if( ( ii = fread (img,sizeof(short),imsz,fid)) != imsz ) {
	fprintf(stderr,"Image read error:\n\
	Only read %d shorts from %s\nExpected %d",
		        ii,filenm,imsz);
	fclose(fid);
	exit(-94);
    }
    fclose(fid);

    if(doprint==1) {
	/* Print out the sumry info. */
	printf ("\t    File: %s\n",filenm);
	printf ("\t        %d %d \n",       hd->Xdim,hd->Ydim);
	printf ("\t        %s %s \n",       prday (hd->Image.Juld),
				            prtime(hd->Image.Time));

	switch (hd->DataType) {  
	    case IS_i:  printf("\t\tThis image is i data.\n") ; break;
	    case IS_q:  printf("\t\tThis image is q data.\n") ; break;
	    case IS_u:  printf("\t\tThis image is u data.\n") ; break;
	    case IS_ba: printf("\t\tThis image is ba data.\n") ; break;
	    case IS_sk: printf("\t\tThis image is sk data.\n") ; break;
	    case IS_st: printf("\t\tThis image is st data.\n") ; break;
	    default:    printf("\t\tThis image is UNKNOWN.\n") ;
	} 
    }
}

int
writemk4(filenm,hd,img,doprint) 
    char *filenm; 
    struct mk4_header *hd; 
    short *img; 
    int doprint;
{
    long  ii,imsz,hdsz;
    FILE *fid;

    if(doprint) fprintf(stderr,"\tWriting %s\n",filenm);
    if ( ( fid = fopen(filenm,"w") ) < (FILE*)0 ) {
	fprintf(stderr,"Error opening %s for writing. Error %d\n",filenm,ii);
	exit(-90);
    }

    hdsz = sizeof(mk4head);

    if( ( ii = fwrite(hd, 1, hdsz ,fid)) != hdsz ) {
	fprintf(stderr,"Header write error:\n\
	Only wrote %d bytes from %s\nExpected %d",
		        ii,filenm,hdsz);
	fclose(fid);
	exit(-92);
    }

    imsz = (hd->Xdim)*(hd->Ydim);

    if( ( ii = fwrite (img,sizeof(short),imsz,fid)) != imsz ) {
	fprintf(stderr,"Image write error:\n\
	Only wrote %d shorts from %s\nExpected %d",
		        ii,filenm,imsz);
	fclose(fid);
	exit(-94);
    }
    fclose(fid);
    if(doprint) fprintf(stderr,"\n");
}

int
readtscp(filenm,Thd,doprint)
    char *filenm; 
    struct telhd *Thd; 
    int doprint;
{
    long  ii,Thdsz;
    FILE *fid;

    if(doprint) fprintf(stderr,"\tReading %s\n",filenm);
    if ( ( fid = fopen(filenm,"r") ) < (FILE*)0 ) {
	fprintf(stderr,"Error opening %s for reading. Error %d\n",filenm,ii);
	exit(-90);
    }

    Thdsz = sizeof(telscp);

    if( ( ii = fread(Thd, 1, Thdsz ,fid)) != Thdsz ) {
	fprintf(stderr,"Telescope read error:\n\
	Only read %d bytes from %s\nExpected %d",
		        ii,filenm,Thdsz);
	fclose(fid);
	exit(-92);
    }

    fclose(fid);
    if(doprint) fprintf(stderr,"\n");

    /*
	fprintf(stderr,
	"\t\t\tIn readtscp: Thd->hdCi.orig_hairspot[0] %.2f\n",
                            Thd->hdCi.orig_hairspot[0] );
    */


}

int
writetscp(filenm,Thd,doprint)
    char *filenm; 
    struct telhd *Thd; 
    int doprint;
{
    long  ii,Thdsz;
    FILE *fid;

    if(doprint) fprintf(stderr,"\tWriting %s\n",filenm);
    if ( ( fid = fopen(filenm,"w") ) < (FILE*)0 ) {
	fprintf(stderr,"Error opening %s for writing. Error %d\n",filenm,ii);
	exit(-90);
    }

    Thdsz = sizeof(telscp);

    if( ( ii = fwrite(Thd, 1, Thdsz ,fid)) != Thdsz ) {
	fprintf(stderr,"Telescope write error:\n\
	Only wrote %d bytes from %s\nExpected %d",
		        ii,filenm,Thdsz);
	fclose(fid);
	exit(-92);
    }

    fclose(fid);
    if(doprint) fprintf(stderr,"\n");

    /*
	fprintf(stderr,
	"\t\t\tIn writetscp: Thd->hdCi.orig_hairspot[0] %.2f\n",
                             Thd->hdCi.orig_hairspot[0] );
    */

}


int
read_long(filenm,img,imsz,doprint) 
    char *filenm; 
    long *img; 
    int  imsz;
    int doprint;
{
    int ii,hdsz;
    FILE *fid;

    if(doprint>0) fprintf(stderr,"\tReading %s\n",filenm);

    if ( ( fid = fopen(filenm,"r") ) == NULL  ) {
	fprintf(stderr,"Error opening %s for reading.\n",filenm);
	exit(-90);
    }

    if( ( ii = fread (img,sizeof(long),imsz,fid)) != imsz ) {
	fprintf(stderr,"Image read error:\n\
	Only read %d longs from %s\nExpected %d",
		        ii,filenm,imsz);
	fclose(fid);
	exit(-94);
    }
    fclose(fid);

}
