/* rotate_cpbimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate a Mk4 cpb image [cylindrical/polar coordinates].
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   28 Aug 2003
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

short *
rotate_cpbimg (imgbuf, xdim, ydim, rotang)

short *imgbuf ;		/* Mk4 cpb image array. */
int xdim, ydim ;	/* Image dimensions. */
float rotang ;		/* Rotation angle [CCW degrees]. */

{
    short *imgrot ;
    int ir ;
    int ix, iy ;
    int nrot ;

    if (rotang == 0.0)
	{
	fprintf (stderr, "No rotation performed.  Rotation angle is zero.\n") ;
	return (imgbuf) ;
	}

    /*** Allocate storage for rotated image array. ***/

    imgrot = (short *) malloc (xdim * ydim * sizeof (short)) ;

    /*** Convert rotation angle [degrees] into x-axis pixels. ***/
    /*** Note: the angular increment for cpb images is 0.5 degrees. ***/

    if (rotang > 0.0)
        nrot = (int) ((rotang / 0.5) + 0.5) ;
    else
        nrot = (int) ((rotang / 0.5) - 0.5) ;

    for (iy = 0; iy < ydim; iy++)
	for (ix = 0; ix < xdim; ix++)
	    {
	    ir = ix + nrot ;
	    if (ir >= xdim)  ir -= xdim ;
	    if (ir <     0)  ir += xdim ;
	    *(imgrot + ir + iy*xdim) = *(imgbuf + ix + iy*xdim) ;
	    }

    return (imgrot) ;
}
