/* sfi.c
 * -----------------------------------------------------------------------------
 * Spline Fit a native format Mk4 "i" file.
 * -----------------------------------------------------------------------------
 * Author: Andrew L. Stanger   HAO/NCAR   30 July 2002 
 * -----------------------------------------------------------------------------
 */

/*
 * This program was uses software written by Alice Lecinski, September 2001.
 */

#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define IS_DEPOL     (2)
#define IS_DEPOLSCAN (1)


#include "mk4v2.h"
#include "mk4headv2.h"
#include "telscp4v2.h"

float iscale = 1.0e+2;
float qscale = 1.0e+4;
float uscale = 1.0e+4;

float ioffset =    0.;
float qoffset = 1000.;
float uoffset =    0.;


char *filenm, picnm[64], *piceos;
char tscpfile [64];
char reffile  [64];

short  MyIm[CY_IMG];
short  MyBA[VECTOR] ;

float Iim [CY_IMG], Qim [CY_IMG], Uim [CY_IMG], ba [VECTOR];
float Aim [CY_IMG];
float  costh [VECTOR],sinth [VECTOR],cos2th [VECTOR],sin2th [VECTOR];
double theta [VECTOR], Xval [VECTOR], Phi [VECTOR], Amp [VECTOR];
float tmpV [VECTOR];
float Copy [VECTOR];

char noIUExpla[] ="\
No calibrated I or U data will be created.\n";

char CinIExpla [] ="\
The I data will be filtered to obtain the coronal signal.\n";

char FloatExpla [] ="\
Output data will be floats.\n";

char HairExpla [] ="\
Hair line resizing will be attempted.  But if it fails,\n\
\ta non-adjusted image will be created.\n";

/* -------------------------------------------------------------------------- */

main (an, av)

int argc;
char *av [];
{
    FILE *fid;
    int do_cksz;
    int do_depol = 1;
    int do_depolscan = 1;

    register char *w;
    unsigned short appdate;
    FH  hdi, hdq, hdu, hd;

    long begtime, endtime;
    long btime, etime;
    int ii,jj;
    int imsz;
    int numGim, numBim;
    int gotMYcalib = 0;
    int already_rescaled_cal;
    int dotest;
    int ckme;
    int FixBA;
    int hairdo, toupee;
    int dozeroLO, dozeroHI;
    int lozero, hizero;
    int noIU;
    int getCinI;
    int floatout;
    double *pd, myang, dtmp;
    float  *pf, *p2f, *pba;
    float  cal_hairLo, cal_hairHi;
    float  std_hairLo, std_hairHi;
    float  hairLo, hairHi, Lo, Hi, mean, stddev;
    float  xcen, ycen, new_xcen, new_ycen, fx, fy;
    float  *myDAT [3];
    float  *myCAL2d [3];
    float  *myCAL1d [4];
    double diffLo, diffHi;
    double shift_angle;
    double pphase;
    struct tm *TS, *localtimel ();
    char *ctime_r (), buf [30];


    if ( an <= 2 ) { prsum () ; exit (3) ; }

    (void) time (&begtime);
    TS = (struct tm *) malloc (sizeof (struct tm));
    TS = localtime (&begtime);
    TS->tm_mon += 1 ;

    (void) ctime_r (&begtime,&buf,30);
    fprintf (stderr,"Started appcal4v2:      %s\n",buf);



    hd  = (FH) malloc (sizeof (mk4head));
    hdi = (FH) malloc (sizeof (mk4head));
    hdq = (FH) malloc (sizeof (mk4head));
    hdu = (FH) malloc (sizeof (mk4head));


    timeflag = 1;
    hairdo = 1;
    toupee = 0;
    dozeroLO = dozeroHI = 0;
    lozero = 0; hizero = NPX-1;
    noIU   = 0;
    getCinI  = 0;
    floatout = 0;
    already_rescaled_cal = 0;
    imsz = NAZ * NPX;

    myDAT [0] = Iim;
    myDAT [1] = Qim;
    myDAT [2] = Uim;

    myCAL2d [0] = thd->Gain;
    myCAL2d [1] = thd->x21a;
    myCAL2d [2] = thd->x31a;
    myCAL1d [0] = thd->x22;
    myCAL1d [1] = thd->x23;
    myCAL1d [2] = thd->x32;
    myCAL1d [3] = thd->x33;


    pd  = theta;
    pf  = costh;
    p2f = cos2th;

    for (ii=0; ii<NAZ; ii++)
	{
        myang  = (double) ii * AngSep * d2r ;
	*pd    = myang;
        dtmp   = cos (myang);
        *pf    = (float) dtmp;
        myang *= 2.0;
        dtmp   = cos (myang);
        *p2f   = (float) dtmp;
        pd++; pf++; p2f++;
        }  

    pf  = sinth;
    p2f = sin2th;

    for (ii=0; ii<NAZ; ii++)
	{
        myang  = (double) ii * AngSep * d2r ;
        dtmp   = sin (myang);
        *pf    = (float) dtmp;
        myang *= 2.0;
        dtmp   = sin (myang);
        *p2f   = (float) dtmp;
        pf++; 
	p2f++;
        }

    numGim = 0;
    numBim = 0;

    while (--an)
	{		/* Read in the files and perform the calibration */
	w = *++av ;
	if (*w == '?') { prsum (); exit (2); }
	if (*w == '-')
	    {			/* Check for flag arguments. */
	    switch (w[1]) {
                case 'f': 
			 floatout = 1;
			 ioffset = qoffset = uoffset = 0.0;
			 fprintf (stderr, FloatExpla);
			 break;
                case 'g': 
			 switch (w [2])
			     {
			     case 'c':
			     case 'C':
				 fprintf (stderr, CinIExpla);
				 getCinI = 1;
				 break;
			     default: fprintf (stderr," unknown flag %s\n",*av);
				 prsum (); exit (1);
			     }
			 break;
                case 'n': 
			 switch (w [2])
			     {
			     case 'i':
			     case 'I':
				 fprintf (stderr,noIUExpla);
				 noIU = 1;
				 break;
			     case 'h':
			     case 'H':
				 fprintf (stderr,HairExpla);
				 toupee = 1;
				 break;
			     default: fprintf (stderr," unknown flag %s\n",*av);
				 prsum (); exit (1);
			     }
			 break;
                case 'Z': 
			 switch (w [2])
			     {
			     case 'l':
			     case 'L':
				 an--; w = *++av; sscanf (w, "%d", &lozero);
				 fprintf (stderr,
				 "Lowest %d pixels will be zeroed\n\n",lozero);
				 dozeroLO = 1;
				 break;
			     case 'h':
			     case 'H':
				 an--; w = *++av; sscanf(w,"%d",&hizero);
				 fprintf (stderr,
				 "Top %d pixels will be zeroed\n\n",
				 NPX-hizero-1);
				 dozeroHI = 1;
				 break;
			     default: fprintf (stderr," unknown flag %s\n",*av);
				 prsum (); exit (1);
			     }
			 break;
			    
		
                default: fprintf (stderr, " unknown flag %s\n", *av);
                         prsum (); exit (1);
                }
	    continue;
	    }
	filenm = w;

	/*  Read in the tscp or the tscpinv file.
	 *  If it is the "tscp" file, create inverse X matrix and inverse Gain.
	 *  If this is the "tscpinv" file,
	 *  it already contains the inverse X matrix and inverse Gain.
	 */

	if ( (*(w + strlen (filenm) - 1) == 'p' ) ||
             (*(w + strlen (filenm) - 1) == 'v' )    ) 
	    {
	    gotMYcalib = 1;
	    sprintf (tscpfile, "%s", filenm);
	    readtscp (filenm, thd, 1);

	    fprintf (stderr, "\t    Calibration was created: %s\n", 
		 prday (thd->tdysdate));

	    if ( *(w + strlen (filenm) - 1) == 'p' )
		{
		ComputeInverse (thd);
	        }

            doyflag = dayflag = 1;
	    dopicnm (&(thd->hdCi));
	    dofnm   (IS_q, thd->hdCi.ProcSteps); 
            sprintf (reffile, "%s", picnm);

	    doyflag = dayflag = 0;

	    /* The hair spots are really from the cal0 image, 
	     * but they've been copied into all headers in the
	     * tscp file.
	     *
	     * The tscp file should not have been adjusted for
	     * height in getcal4v2.  It will be adjusted to the
	     * correct height based on the date of collection
	     * of the coronal data.
	     */

	    cal_hairLo           = thd->hd0i.hairspot [0];
	    cal_hairHi           = thd->hd0i.hairspot [1];


	    continue;
	    }


	if (!gotMYcalib)
	    {
	    fprintf (stderr, 
	    "Sorry, the calibration \".tscp\" file must come first.\n");
	    fprintf (stderr, "Exitting\n");
	    exit (-10);
	    }

	(void) time (&btime);

	/* Check on the file naming convention used on the file */

	if ((*w == '0') || (*w >= '8' && *w <= '9') || *(w+2) == 'd' )
	    {
	    if ( *(w + 3) >='0' && *(w + 3) <='9' )  doyflag++ ;
	    else                                     nameflag++ ;
	    }
	if (nameflag || doyflag) { dayflag++ ; timeflag = 1; }


	readmk4head (filenm, mk4h, 1);

        /* Get the i q u data. */

        dopicnm (mk4h);
        dofnm (IS_i, mk4h->ProcSteps);
	readmk4 (picnm, hdi, MyIm, 0);
	Short2Float (MyIm, Iim, hdi->Xdim, hdi->Ydim);

        dofnm (IS_q, mk4h->ProcSteps);
	readmk4 (picnm, hdq, MyIm, 0);
	Short2Float (MyIm, Qim, hdq->Xdim, hdq->Ydim);

        dofnm (IS_u, mk4h->ProcSteps);
	readmk4 (picnm, hdu, MyIm, 0);
	Short2Float (MyIm, Uim, hdu->Xdim, hdu->Ydim);

        dofnm (IS_ba, IS_raw);
	readmk4 (picnm, hd, MyBA, 0);
	Short2Float (MyBA, ba, hd->Xdim, hd->Ydim);


	dotest = 0;
	if (dotest == 0)
	    {
	    fprintf (stderr, "\t    Doing \"fixmk4\".\n");
	    fixmk4 (Iim, hdi, Qim, hdq, Uim, hdu, ba, hd , 0);
	    }
	dotest = 0;

	ckme = 0;
	if (ckme == 1)
	    {
	    if ( ( fid = fopen ("q_after_fixmk4", "w") ) < (FILE*) 0 )
		{
		fprintf (stderr, "q_after_fixmk4 writing error %d\n", ii);
		exit (-90);
	        }
	    fwrite (Qim, sizeof (float), hdi->Xdim*hdi->Ydim , fid);
	    fclose (fid);
	    }

	dotest = 0;
	if (dotest == 0)	/* Spline it, save it as an theta by R array. */
	    {
	    fprintf (stderr, "\t    Doing \"fcyspline\".\n");
	    FixBA= 1;
	    fcyspline (hdi, Iim, ba, FixBA);   FixBA = 0;
	    fcyspline (hdq, Qim, ba, FixBA);
	    fcyspline (hdu, Uim, ba, FixBA);
	    hdi->Xdim      = hdq->Xdim      = hdu->Xdim      =     NAZ;
	    hdi->Ydim      = hdq->Ydim      = hdu->Ydim      =     NPX;
	    }

	ckme = 0;
	if (ckme == 1)
	    {
	    if ( ( fid = fopen ("q_after_fcyspline", "w") ) < (FILE*) 0 )
		{
		fprintf (stderr, "q_after_fcyspline writing error %d\n", ii);
		exit (-90);
	        }
	    fwrite (Qim, sizeof (float), imsz , fid);
	    fclose (fid);
	    }

	dotest = 0;
        if (dotest == 0)
	    {
	    fprintf (stderr, "\tAdjust for phase and backlash\n");
            shift_angle = (double) (hdq->Phase + hdq->backlash );
	    for (jj=0; jj<3; jj++)
		{
		fshiftimg (myDAT [jj], shift_angle);
	        }
            }

	dotest = 0;

	dotest = 0;
	if ((hairdo == 1) && (dotest == 0))
	    {
            if ( hd->Image.Juld > 36429 )
		{ /* 99d271 */
		fprintf (stderr, "\tGetting coronal HairLine positions.\n");

		getHair (Iim, NAZ, &hairLo, &hairHi);

		if (OkHair (hd, hairLo, hairHi))
		    {

		    hdi->orig_hairspot [0] =
		    hdq->orig_hairspot [0] =
		    hdu->orig_hairspot [0] = hairLo;
		    hdi->orig_hairspot [1] =
		    hdq->orig_hairspot [1] =
		    hdu->orig_hairspot [1] = hairHi;

		    std_hairLo = hdi->ID.hairspot [0];
		    std_hairHi = hdi->ID.hairspot [1];

		    if ((cal_hairLo != std_hairLo)
		       ||
		         cal_hairHi != std_hairHi) 
		        {
			fprintf (stderr, 
			"\n\tAdjusting tscp size to standard HairLines.\n");
			do_cksz = 0;
			if (do_cksz == 1)
			    {
			    if ((fid = fopen("g_pre_resize", "w")) < (FILE*)0 )
				{
				fprintf (
				stderr, "g_pre_resize writing error %d\n", ii);
				exit (-90);
			        }
			    fwrite (myCAL2d[0], sizeof(float), imsz , fid);
			    fclose (fid);
			    }

			if (already_rescaled_cal == 1)
			    {
			    /* already did this once
			     * -- who knows what got rescaled?
			     */
			    fprintf (stderr, 
			    "Different eras in the data mess things up... \n");
			    exit (-99);
			    }

			if (cal_hairLo <= 0.0)
			    {
			    /* The tscp file has never been adjusted for size
			     * and is in the scaling of the original data.
			     */
			    cal_hairLo = thd->hdCi.orig_hairspot [0];
			    cal_hairHi = thd->hdCi.orig_hairspot [1];
			    }
			fprintf (stderr, 
			    "\t\tstd_hairHi %.2f std_hairLo %.2f\n", 
			    std_hairHi, std_hairLo);
			fprintf (stderr, 
			    "\t\tcal_hairHi %.2f cal_hairLo %.2f\n", 
			    cal_hairHi, cal_hairLo);


			fx = 1.0;
			xcen = new_xcen = (NAZ - 1.0) * 0.5;
			ycen     =  cal_hairLo;
			new_ycen =  std_hairLo;
			fy       =   (std_hairHi - std_hairLo) 
				   / (cal_hairHi - cal_hairLo);


			for (jj=0; jj<3; jj++)
			    {			/* rescale Gain x21a, x31a */
			    fexpcon (myCAL2d [jj], NAZ, NPX, 
				     Aim,          NAZ, NPX, 
				    fx, fy, xcen, ycen, new_xcen, new_ycen);
			    VVcopy (Aim, myCAL2d [jj], imsz);
			    }
			for (jj=0; jj<4; jj++)
			    {			/* rescale x22, x23, x32, x33 */
			    fexpconVec (myCAL1d [jj], NPX, 
				        Aim,        NPX,    fy, ycen, new_ycen);
			    VVcopy    (Aim, myCAL1d[jj], NPX);
			    }

			/* set cal_hairLo == std_hairLo so resizing doesn't
			 * happen again.
			 */

			cal_hairLo = std_hairLo;
			cal_hairHi = std_hairHi;

			/* Flag that the rescale of the cal has been done
			 * so that it can't happen again.
			 */
			already_rescaled_cal = 1;

			do_cksz = 0;
			if (do_cksz == 1)
			    {
			    if ((fid=fopen("g_post_resize", "w")) < (FILE*)0 )
				{
				fprintf (
				stderr, "g_post_resize writing error %d\n", ii);
				exit (-90);
			        }
			    fwrite (myCAL2d[0], sizeof(float), imsz , fid);
			    fclose (fid);
			    }

		        }


		    do_cksz = 0;
		    if (do_cksz == 1)
			{
			if ( ( fid = fopen("i_pre_resize", "w") ) < (FILE*)0 )
			    {
			    fprintf (
			    stderr, "i_pre_resize writing error %d\n", ii);
			    exit (-90);
			    }
			fwrite (Iim, sizeof (float), imsz , fid);
			fclose( fid);
		        }

		    /* Now rescale the coronal images. */

		    fprintf (stderr, 
		    "\n\tAdjusting the size and getting *new* HairLines.\n");
		    fx = 1.0;
		    xcen = new_xcen = (NAZ - 1.0) * 0.5;
		    ycen     =      hairLo;
		    new_ycen =  std_hairLo;
		    fy       = (std_hairHi - std_hairLo) / (hairHi - hairLo);

		    for (jj=0; jj<3; jj++)
			{

			fexpcon (myDAT [jj], NAZ, NPX, 
				 Aim,        NAZ, NPX, 
				fx, fy, xcen, ycen, new_xcen, new_ycen);

			if (jj == 0)
			    {			/* re-check the I data. */
			    getHair (Aim, NAZ, &Lo, &Hi);
			    diffLo = Lo - std_hairLo;  diffLo = fabs (diffLo);
			    diffHi = Hi - std_hairHi;  diffHi = fabs (diffHi);
			    if ((diffLo > 0.5) || (diffHi > 0.5))
				{
				fprintf (stderr, 
				    "Couldn\'t match std and image hairs\n");
				numBim++;
				if (toupee == 1) goto BadHair;
				else             goto Bald;
			        }
			    }
			VVcopy (Aim, myDAT [jj], imsz);
		        }

		    hdi->hairspot [0]= hdq->hairspot [0]= hdu->hairspot[0]= Lo;
		    hdi->hairspot [1]= hdq->hairspot [1]= hdu->hairspot[1]= Hi;


		    do_cksz = 0;
		    if (do_cksz == 1)
			{
			if ( ( fid = fopen("i_post_resize", "w") ) < (FILE*)0 )
			    {
			    fprintf (stderr, "i_post_resize writing error %d\n",
				     ii);
			    exit (-90);
			    }
			fwrite (Iim, sizeof (float), imsz , fid);
			fclose (fid);
		        }
		    }
		else
		    {

		    if (toupee == 1) goto BadHair;
		    else             goto Bald;

		    }
	        }
	    }

BadHair:


	dotest = 0;
	if (dotest == 0)
	    {
	    fprintf (stderr, "\tCalibrate using the invX matrix and ovGain\n");
	    Recrunch (   thd, 
			 Iim, Qim, Uim);
	    }
	dotest = 0;

	dotest = 0;
        if ((do_depol == 1) && (dotest == 0))
	    {
            fprintf (stderr, "\tDepolarizing (depol).\n");
	    depol    (Iim, Qim, Uim, IS_DEPOL);
	    }

	dotest = 0;
        if ((do_depolscan == 1) && (dotest == 0))
	    {
            fprintf (stderr, "\tDepolarizing (depolscan).\n");
	    depol    (Iim, Qim, Uim, IS_DEPOLSCAN);
            }
	dotest = 0;

	dotest = 0;
	if (dotest == 0)
	    {
            fprintf (stderr, "\tFinding and removing rings.\n");
	    FindRing ( Qim, Aim , NAZ, NPX , hdq->ID.RingSearch [0], 
					     hdq->ID.RingSearch [1], 
					     hdq->ID.RingFixPix);
	    VVcopy  ( Aim, Qim, imsz);
	    }
	dotest = 0;

	dotest = 0;
	if ((dotest == 0) && (noIU == 0) && (getCinI == 1))
	    {
            fprintf (stderr, "\tGetting corona from Intensity.\n");
	    FindCinI ( Iim, Aim , tmpV, NAZ, NPX , sinth, costh, theta);
	    VVcopy  ( Aim, Iim, imsz);
	    }
	dotest = 0;

	dotest = 0;
	if (dotest == 0)		/* Get to heliocentric coordinates. */
	    {
            fprintf (stderr, "\tShifting to heliocentric coordinates.\n");
	    shift_angle = (double) (180. + hdq->Pangle 
				         - hdq->ID.BarrelOffset
				         - thd->hd0i.InstrPhs);
	    if (noIU == 0)
		{
		for (jj=0; jj<3; jj++)
		    {
		    fshiftimg (myDAT [jj], shift_angle);
		    }
	        }
	    else
		{
		fshiftimg (myDAT [ 1], shift_angle);
	        }
	    }
	dotest = 0;

	dotest = 0;
	if (dotest == 0)
	    {
            fprintf (stderr, "\tFinding and removing neg. pB near poles.\n");
	    FixNegpBatP ( Qim, Aim , NAZ, NPX );
	    VVcopy     ( Aim, Qim, imsz);
	    }
	dotest = 0;

	dotest = 0;
	if (dotest == 0)
	    {
	    fprintf (stderr, "\tRescaling data.\n");
	    if (noIU == 0) Vmult (Iim, iscale, imsz);
			   Vmult (Qim, qscale, imsz);  
	    if (noIU == 0) Vmult (Uim, uscale, imsz);

	    if ((ioffset != 0.0) && (noIU == 0)) Vplus (Iim, ioffset, imsz);
	    if ( qoffset != 0.0)                 Vplus (Qim, qoffset, imsz);
	    if ((uoffset != 0.0) && (noIU == 0)) Vplus (Uim, uoffset, imsz);
	    }

	if (dozeroLO == 1)
	    {
	    if (noIU == 0)
		{
		for (jj=0; jj<3; jj++)
		    {
		    zeroLO (myDAT [jj], NAZ, lozero);
		    }
	        }
	    else
		{
		zeroLO (myDAT [ 1], NAZ, lozero);
	        }
	    }

	if (dozeroHI == 1)
	    {
	    if (noIU == 0)
		{
		for (jj=0; jj<3; jj++)
		    {
		    zeroHI (myDAT [jj], NAZ, NPX, hizero);
		    }
	        }
	    else
		{
		zeroHI (myDAT [ 1], NAZ, NPX, hizero);
	        }
	    }


	/* Fix and fill in the headers with new info. */

	UpdateHeader (thd, hdi, hdq, hdu, TS, floatout);

	if (noIU == 0)
	    {
	    dopicnm (hdi);
	    dofnm (hdi->DataType, hdi->ProcSteps);

	    if (hdi->ImgDataType == SHORTS)
		{
		Float2Short (Iim, MyIm, hdi->Xdim, hdi->Ydim);
		writemk4 (picnm, hdi, MyIm, 1);
	        }
	    else
		{
		writemk4 (picnm, hdi, (short *) Iim, 1);
	        }
	    }

	    {
	    dopicnm (hdq);
	    dofnm (hdq->DataType, hdq->ProcSteps);

	    if (hdq->ImgDataType == SHORTS)
		{
		Float2Short (Qim, MyIm, hdq->Xdim, hdq->Ydim);
		writemk4 (picnm, hdq, MyIm, 1);
	        }
	    else
		{
		writemk4 (picnm, hdq, (short *)Qim, 1);
	        }
	    }

	if (noIU == 0)
	    {
	    dopicnm (hdu);
	    dofnm (hdu->DataType, hdu->ProcSteps);

	    if (hdu->ImgDataType == SHORTS)
		{
		Float2Short (Uim, MyIm, hdu->Xdim, hdu->Ydim);
		writemk4 (picnm, hdu, MyIm, 1);
	        }
	    else
		{
		writemk4 (picnm, hdu, (short *)Uim, 1);
	        }
	    }

        numGim++;

Bald:

	(void) time (&etime);
	get_mean (Qim + 152 * NAZ, NAZ, &mean, &stddev);
	fprintf (stderr, "\t\tMean at pixel 152 is %.2f\n", mean);
	fprintf (stderr, 
	"\t\t\tSingle image time used: %ld seconds\n\n", etime-btime);

        }

    (void) time (&endtime);
    fprintf (stderr, "\nTotal time used  :    %ld seconds\n", endtime-begtime);
    fprintf (stderr, "Total good images:    %d \n", numGim);
    fprintf (stderr, "Total bad  images:    %d \n", numBim);

    exit (0);

}

/* -------------------------------------------------------------------------- */

prsum ()
{
    static char help[] = "\
appcal4v2 -f -gC -nIU -nH -ZL zlopix -ZH zhipix [[tscpfile] [tscpinvfile]] [cylindrical file1] [cylindrical file2] ... [cylindrical fileN]\n\
\n\
Example: appcal4v2 -ZL 40 -ZH 380 92d004.20_04.mk4.tscp    92d004.17_28.mk4.q  92d004.17_32.mk4.q\n\
Example: appcal4v2 92d004.20_04.mk4.tscp    92d004.17_28.mk4.q  92d004.17_32.mk4.q\n\
Example: appcal4v2 92d004.20_04.mk4.tscpinv 92d004.17_28.mk4.q  92d004.17_32.mk4.q\n\
\n\
\n\
Note:  This progam requires the \".i\" \".q\" \".u\" and \".ba\" files \n\
       for a given image time.\n\
       The program automatically builds the filenames of \n\
       the \".i\" \".q\" \".u\" and \".ba\" files \n\
       files using the .q data filenames as a template.\n\
\n\
Flags          Description\n\
-----          -----------\n\
\n\
-f             output images will be floating point\n\
-gC            will filter the calibrated I data to get at any coronal \n\
	       signal\n\
-nIU           will *not* produce calibrated I or U data\n\
-nH            will still produce an image, even if no hairlines\n\
	       can be found, or if they in the wrong places.\n\
-ZL zlopix     will zero     pixels  zlopix and below\n\
-ZH zhipix     will zero     pixels  zhipix and above\n\
\n\
";

       fputs (help, stderr);
}

/* -------------------------------------------------------------------------- */

int
Recrunch ( Thd, 
	   Iim, Qim, Uim)

struct telhd *Thd;
float        *Iim, *Qim, *Uim;
{
    int ii, dd , mee;
    float  tmpI, tmpQ, tmpU;
    float *pIim, *pQim , *pUim;
    float *povGain;
    float *povx21 ;
    float *povx31 ;
    float *povx22 ;
    float *povx23 ;
    float *povx32 ;
    float *povx33 ;
    float calI, calQ, calU;

    pIim = Iim;
    pQim = Qim;
    pUim = Uim;

    povGain = Thd->Gain;
    povx21  = Thd->x21a;
    povx31  = Thd->x31a;
    povx22  = Thd->x22;
    povx23  = Thd->x23;
    povx32  = Thd->x32;
    povx33  = Thd->x33;

    for (dd = 0; dd < NPX; dd++)
	{

	for (ii=0; ii<NAZ; ii++)
	    {

	    tmpI = *pIim ;
	    tmpQ = *pQim ;
	    tmpU = *pUim  ;

	    /* Divide out the Gain */

	    tmpI *= (*povGain) ;
	    tmpQ *= (*povGain) ;
	    tmpU *= (*povGain) ;

	    /* Multiply by the X inverse matrix. */

	    calI = tmpI;
	    calQ = (*povx21)*tmpI  +(*povx22)*tmpQ  +(*povx23)*tmpU;
	    calU = (*povx31)*tmpI  +(*povx32)*tmpQ  +(*povx33)*tmpU;

	    *pIim = calI ;
	    *pQim = calQ ;
	    *pUim = calU ;

	    pIim++;  pQim++;  pUim++;  
	    
	    povGain++ ;
	    povx21++  ;
	    povx31++  ;
	    }
	povx22++  ;
	povx23++  ;
	povx32++  ;
	povx33++  ;
        }
}

/* -------------------------------------------------------------------------- */

int
depol (I, Q, U, which)

    float *I, *Q, *U;
    int      which;
{
    int   ii, jj;
    int   pixStart, pixEnd, GoodPix;
    float  *pI, *pQ, *pU, *pUovI, *pV, *pW;
    float sprod, cprod;
    double Correl_limit;
    double AmpA, AmpB, AmpR;
    double PhiA, PhiB, PhiR;
    double *pX, *pTh, *pPhi, *pAmp, myQang, myUang , mpiov2, Pfac;
    double Smean, Cmean, SumSq;
    double sphiQ, sphiU, PskyQ, PskyU;
    double atan2 (), sqrt ();

    mpiov2       = -1.0 * piov2 ;
    Correl_limit = 0.1;
    GoodPix      = 0;

    /* Use only the pixels 150 to 300 */

    pixStart = 150;
    pixEnd   = 301;

    if (which == IS_DEPOL) Pfac = 2.0;
    else                   Pfac = 1.0;


    VVdivide(U, I, Aim, NAZ*NPX);


    pUovI = Aim + pixStart*NAZ;
    pPhi  = Phi;
    pAmp  = Amp;
    pX    = Xval;

    for (jj = pixStart; jj<pixEnd; jj++)
	{
	*pPhi = 0.0;
	*pAmp = 0.0;
	*pX   = -1.0;

	if (which == IS_DEPOL) pW = sin2th;
	else                   pW = sinth;

	VVmult  (pUovI, pW,  tmpV, NAZ);
	get_mean_fast (      tmpV, NAZ, &sprod);

	if (which == IS_DEPOL) pW = cos2th;
	else                   pW = costh;

	VVmult  (pUovI, pW, tmpV, NAZ);
	get_mean_fast (    tmpV, NAZ, &cprod);

	Smean = (double) sprod;
	Cmean = (double) cprod;
	     
	SumSq = Smean*Smean + Cmean*Cmean;
		  
	*pX   = (double) jj;
	*pPhi = atan2 (Smean, Cmean);
	*pAmp = 2. * sqrt (SumSq);

	if (which == IS_DEPOLSCAN)
	    {
	    if ( *pPhi < 0.0 ) *pPhi += twopi ;
	    }

	pX++;
	pPhi++;
	pAmp++;

	GoodPix++;

	/*
	 * fprintf (stderr, "%d %f %f \n", jj, *pPhi, *pAmp);
	 */

	pUovI += NAZ;

        }



    /* Get a least squares fit for the Amp
     * for the pixel range above.  
     * Then replace Amp with the least squares
     * fit values.
     *
     * If the correlation coefficient is lousy, 
     * fake it so that we use the mean.
     *
     * For Phi -- always compute the mean.
     */


    Ddo_lst (&AmpA, &AmpB, &AmpR,   Xval, Amp, GoodPix, 1);
    if (AmpR < Correl_limit)
	{ 
	AmpB = 0.0;
	Dget_mean_fast (Amp, GoodPix, &AmpA);
	if (which==IS_DEPOL)
	    fprintf(stderr, "\t\t\t\tDepol: using Amp mean %g\n", AmpA);
	else
	    fprintf(stderr, "\t\t\t\tDepolscan: using Amp mean %g\n", AmpA);
        }

    Ddo_lst (&PhiA, &PhiB, &PhiR,   Xval, Phi, GoodPix, 1);
    PhiR = 0.0;
    if (PhiR < Correl_limit)
	{ 
	PhiB = 0.;
	Dget_mean_fast (Phi, GoodPix, &PhiA);
	if (which == IS_DEPOL)
	    fprintf (stderr, "\t\t\t\tDepol: using Phi mean %g\n", PhiA);
	else
	    fprintf (stderr, "\t\t\t\tDepolscan: using Phi mean %g\n", PhiA);
        }


    pAmp  = Amp;
    pPhi  = Phi;
    for (ii=0; ii<NPX; ii++)
	{
	*pAmp = AmpA + AmpB*(double)ii;
	*pPhi = PhiA + PhiB*(double)ii;
	pAmp++;
	pPhi++;
        }




    pI    = I  ;
    pQ    = Q  ;
    pU    = U  ;
    pPhi  = Phi;
    pAmp  = Amp;
    for (jj=0; jj<NPX; jj++)
	{
	pTh = theta;
	for (ii=0; ii<NAZ; ii++)
	    {
	    myQang = *pTh * Pfac   - *pPhi - piov2;
	    myUang = *pTh * Pfac   - *pPhi        ;
	    sphiQ = cos ( myQang );
	    sphiU = cos ( myUang );
	    PskyQ = (float) (sphiQ * (*pAmp));
	    PskyU = (float) (sphiU * (*pAmp));

	    PskyQ *= *pI;
	    PskyU *= *pI;

	    *pQ -= PskyQ;
	    *pU -= PskyU;

	    pI++; pQ++; pU++; pTh++;
	}
	pPhi++;
	pAmp++;
    }
}

/* -------------------------------------------------------------------------- */

int
UpdateHeader (Thd , hdi, hdq, hdu, TS, floatout)

    struct telhd *Thd;
    FH  hdi, hdq, hdu;
    struct tm *TS;
    int floatout;
{

	hdi->data_scale = iscale * 1.0e+6;
	hdq->data_scale = qscale * 1.0e+6;
	hdu->data_scale = uscale * 1.0e+6;

	hdi->data_offset = ioffset ;
	hdq->data_offset = qoffset ;
	hdu->data_offset = uoffset ;

	if (floatout == 0 )
	    {
	    hdi->ImgDataType = hdq->ImgDataType = hdu->ImgDataType = SHORTS;
	    }
	else
	    {
	    hdi->ImgDataType = hdq->ImgDataType = hdu->ImgDataType = FLOATS;
	    }

	hdi->ProcSteps = hdq->ProcSteps = hdu->ProcSteps =  IS_cal;
	hdi->theta_min = hdq->theta_min = hdu->theta_min =     0.0;
	hdi->theta_max = hdq->theta_max = hdu->theta_max =   359.5;
	hdi->Xdim      = hdq->Xdim      = hdu->Xdim      =     NAZ;
	hdi->Ydim      = hdq->Ydim      = hdu->Ydim      =     NPX;

	hdi->InstrPhs  = hdq->InstrPhs  = hdu->InstrPhs  = Thd->hd0i.InstrPhs;

        sprintf (hdi->tscpfile, "%s", tscpfile );
        sprintf (hdq->tscpfile, "%s", tscpfile );
        sprintf (hdu->tscpfile, "%s", tscpfile );
        sprintf (hdi->reffile,  "%s", reffile  );
        sprintf (hdq->reffile,  "%s", reffile  );
        sprintf (hdu->reffile,  "%s", reffile  );

	hdi->Date_calibrated.Year   = 
	hdq->Date_calibrated.Year   = 
	hdu->Date_calibrated.Year   = TS->tm_year+1900; /* Ok even after 2000 */
	hdi->Date_calibrated.Month  = 
	hdq->Date_calibrated.Month  = 
	hdu->Date_calibrated.Month  = TS->tm_mon;
	hdi->Date_calibrated.Day    = 
	hdq->Date_calibrated.Day    = 
	hdu->Date_calibrated.Day    = TS->tm_mday;
	hdi->Date_calibrated.Doy    = 
	hdq->Date_calibrated.Doy    = 
	hdu->Date_calibrated.Doy    = TS->tm_yday;
	hdi->Date_calibrated.Hour   = 
	hdq->Date_calibrated.Hour   = 
	hdu->Date_calibrated.Hour   = TS->tm_hour;
	hdi->Date_calibrated.Minute = 
	hdq->Date_calibrated.Minute = 
	hdu->Date_calibrated.Minute = TS->tm_min;
	hdi->Date_calibrated.CentiSec = 
	hdq->Date_calibrated.CentiSec = 
	hdu->Date_calibrated.CentiSec = TS->tm_sec * 100 ;

	hdi->Date_calibrated.Juld = 
	hdq->Date_calibrated.Juld = 
	hdu->Date_calibrated.Juld = jday(TS->tm_mon, 
					 TS->tm_mday, TS->tm_year+1900) 
				    - JULD_REF;

	hdi->Date_calibrated.Time = 
	hdq->Date_calibrated.Time = 
	hdu->Date_calibrated.Time =  TS->tm_hour * 60*60*60
				    +TS->tm_min  * 60*60
				    +TS->tm_sec  * 60       ;

}

/* -------------------------------------------------------------------------- */

