/* This files contains --all-- the comments for mkMKfile.c
   and the general idea for the synoptic codes.
   I have also written documentation for this stuff. It is in 
   the sub-directory "Doc"
*/

/*  This program "mkMKfile" makes a MK____.[EW] file (just like the ones
    Michele uses for all of the mk3 history tape programs) from
    the output of the "synoptc" CRAY program on ha.  
    pB units are 10^-10 B/Bsun
    
    (The CRAY output is in /hao/d/cordyn/Synoptc/SYN___ )

    Since this CRAY output
    contains all heights, at one degree increments, the height desired
    to create the [EW] files must be indicated (else 1.3 is the default) and 
    each 5 degrees are averaged to produce the "Michele" output.
    The MK____.E files go from NORTH to SOUTH, from 0 degrees position angle
    to 180 degrees position angle (37 points)
    The MK____.W files go from SOUTH to NORTH, from 180 degrees position angle
    to 360 degrees position angle (37 points)

    The MK____.[EW] files produced by this code are kept in a 
    subdirectory encoded by height.
    (/cordyn/d/mk3S/DATA/EW_files/height_.__/MK____.[EW] )

    Recall the MK____.[EW] files must be processed with
    the code that Michele created
    to get the data into the final form of the [EW]ifib files.
    These are like: mkinterp
		    mkfillin
		    mkinterp
		    MKasc_bin
    These programs are in /home/yoda/mk3_history/MK 
*/

/* This program can also create "cmap", 
             an array of shorts pB[360][360] (pB[PA][longs] longs goes
	     fastest) that begins at Central Meridian passage.

   and, well really or, "imap", and east or west limb map of 
             an array of shorts pB[181][360] , suitable for the Grinnell
	     or idl.
*/
/* For both Bmap and BCmap, (imap and cmap) 
       longitude goes from 0 to 360 degrees, 
       with 0 degrees on the left and 360 degrees on the right.
       Time is reversed, so that the earliest day is on the right,
       and the latest day is on the left.
   For Bmap (imap) the data goes from north to south (PA 0 to 180 degrees)
   For BCmap (cmap) the data goes from north to south to north 
       (PA 0 to 360 degrees)
   In idl, cmap can be displayed with: a=intary(360,360)
					tvscl,a,order=1
   In idl, imap can be displayed with: a=intary(360,181)
					tvscl,a,order=1
*/

 
    /* Below is the structure of the data in the SYN___ file...

      pos(32) reclen;\n
      pos(32) recoff;\n
      string(256) def;\n
      npos(13) flags;\n
      pos(3) chan;\n
      pos(16) juld;\n
      pos(32) time;\n
      int(16) phase;\n
      pos(16) bright;\n
      int(16) pangle;\n
      pos(16) nray;\n
      int(16) qmax, qmin;\n
      int(16) didxx, didxn;\n
      int(16)[MMMM] b-angle;\n
      int(16)[360,128] qqsyn;\n\0
 */

 /*   My idea to calculate the beginning of the east or west limb maps is to
      take each individual carrington rotation, divide it into quarters,
      and then either subtract or add a quarter of that rotation for 
      determining where the east or west limb map begins.  The quarter
      in question is calculated in the rotation that is is in.
      (East goes to previous rotation, west to the present.)

      The end of the rotation is calculated similarily, finding the 
      quarter that it is in, and then subtracting or adding accordingly.
      The east uses the present rotation, the west the future one.

      I've used julian days and fractional days to calculate longitudes,
      since julian days don't care about years, and with fractional
      days, I can be more accurate in determining longitudes.

      The longitudes are also floats.

      However, when I fill up the maps, I am constrained to make the
      indicies of the maps integers.  Hence when the final
      map is created, I've truncated (NOT rounded) the longitudes.

      Hence data collected at longitude 349.62  get placed on the map
      at longitude 349.

*/
/* 
I have discerned from the Forth printouts from MLSO the the history tape
data is taken at diodes  22 , 58 , 104 (0 is first diode)
in the original scaling system, 
    r = 1.03 + diode*1.3/127  (diodes begin at 0)
these heights would be:  1.255 , 1.624 , 2.095

---> However, the Tech notes always say the first height was at 1.36 Rsun,
which is more like diode 32!  <--- (This is a 12/12/90 scaling of 1.46)


in the 12/12/90 system ,        
    r = 1.1224 + diode* 0.0104244  (diodes begin at 0)
these heights would be:  1.352 , 1.727 , 2.207

(1.5Rsun is diode 36 in 12/12/90 scaling)

To make the "history" synoptic maps, the first height (diode 22) is used.

Michele then tries to adjust this height to 1.5 Rsun with routine
/home/yoda/cordyn/mk3_history/MK/fixdat.c

*/
