/* ck_getv2.c
 *------------------------------------------------------------------------------
 * ck_get_day ()
 * ck_get_long ()
 *------------------------------------------------------------------------------
 * Author:  Alice Lecinski   HAO/NCAR    3 Nov 1997   [ck_get.c]
 * Updates: Andrew Stanger   HAO/NCAR
 * 14 Nov 2002: Cosmetic changes.  Add comments.
 *------------------------------------------------------------------------------
 */
#include "EWmap4.h"

extern long  iFbegEj, iFendEj, iFbegWj, iFendWj ;
extern double  begEj,  endEj,  begWj,  endWj ;
extern double  begCj,  endCj ;
extern double slopeE, slopeW, slopeC ;

/* -------------------------------------------------------------------------- */

/*
 * iFbegEj & iFendEj: East Limb Begin/End Julian date for Carrington Rotation.
 * iFbegWj & iFendWj: West Limb Begin/End Julian date for Carrington Rotation.
 */

void
ck_get_day (juld, time, eday, wday, fjuld) 

    unsigned short juld;		/* Image Julian date (less offset). */
    unsigned time;			/* Time of day. */
    int *eday ; 			/* East Limb day index for map. */
    int *wday ; 			/* East Limb day index for map. */
    double *fjuld ;			/* Julian date + time. */
{
    int itmp;
    double jdt ;
    long ijdt ;

    /*** Julian date. ***/

    itmp = time ;

    *fjuld = (double) (juld) + 2415021 + ((double) itmp ) / ONEDAY;
    jdt = *fjuld ;
    ijdt = (long) jdt ;

    /*** East Limb. ***/

    if ( (ijdt >= iFbegEj) && (ijdt <= iFendEj) )
	*eday = (int) (ijdt - iFbegEj ) ;
    else
	*eday = -9999 ;

    /*** West Limb. ***/

    if ( ijdt >= iFbegWj && ijdt <= iFendWj )
	*wday = (int) (ijdt - iFbegWj ) ;
    else
	*wday = -9999 ;

/*
 *    fprintf(stderr, " eday %d , wday %d , juld %.1lf \n", *eday, *wday, jdt);
 */
}

/* -------------------------------------------------------------------------- */

/*
 * begEj & endEj: East Limb Begin/End Julian dates for Carrington Rotation.
 * begWj & endWj: West Limb Begin/End Julian dates for Carrington Rotation.
 * begCj & endCj: Central Meridian Begin/End Julian dates for C. R.
 */

void
ck_get_long (juld, time, elong, wlong, clong, fjuld) 

    unsigned short juld;		/* Julian date (less offset). */
    unsigned time;			/* Time of day. */
    float *elong ;			/* East Limb Longitude. */
    float *wlong ;			/* West Limb Longitude. */
    float *clong ;			/* Central Meridian Longitude. */
    double *fjuld ;			/* Julian date + time. */
{
    int itmp;
    double jdt ;

    /*** Julian date. ***/

    itmp = time ;

    *fjuld = (double) (juld) + 2415021 + ((double) itmp ) / ONEDAY;
    jdt = *fjuld ;

    /*** East Limb Longitude. ***/

    if ( jdt > begEj - OVLAP && jdt < endEj + OVLAP )
	*elong =  (360.0 - (jdt - begEj) * slopeE  + 0.0  ) ;
    else
	*elong = -9999 ;

    /*** West Limb Longitude. ***/

    if ( jdt > begWj - OVLAP && jdt < endWj + OVLAP )
	*wlong = (360.0 - (jdt - begWj) * slopeW  +  0.0  ) ;
    else
	*wlong = -9999 ;

    /*** Central Meridian Longitude. ***/

    if ( jdt > begCj - OVLAP && jdt < endCj + OVLAP )
	*clong =  (360.0 - (jdt - begCj) * slopeC  + 0.0  ) ;
    else
	*clong = -9999 ;
}

/* -------------------------------------------------------------------------- */
