/* mk3_azscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Azimuthal scan using cylindrical pB image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Author:  Alice Lecinski      HAO/NCAR   3 Nov 1997   [fillsyarry.c]
 * Updates: Andrew L. Stanger   HAO/NCAR
 * 13 Feb 2002: cosmetic changes only.
 *  8 Nov 2002: Apply opal correction in fillsyarry.
 * 14 Nov 2002: Rename from fillsyarry to mk3_azscan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/cordyn/mk3S/EWmap.h"

/* -------------------------------------------------------------------------- */

int
mk3_azscan (sbuf, pBim, fba, nn, diode)

    short *sbuf;		/* Azimuth vector array. */
    short *pBim ;		/* Cylindrical image array. */
    float *fba ;		/* Barrel Angle vector array. */
    int nn ;			/* Y-axis dimension [azimuth] of image array. */
    int diode ;			/* Height [pixels]. */
{
    float *pfba ;
    float opalcor = 1.556873 ;			/* Opal correction. */
    double tstang, PosA, fabs () ;
    short *sy ;
    short sval ;
    int ix, iy ;
    static int ndiode = 128 ;			/* Dimension of radial axis. */

    pfba = fba ;
    sy   = sbuf ;

    for (ix = 0; ix < 360; ix++)		/* Azimuth Loop. */
	{
	PosA = (double) ix ;
	pfba = fba ;

	/*** Find barrel angle that matches desired angle within 0.5 deg. ***/
	/*** Copy pixel intensity into output array. ***/

	for (iy = 0; iy < nn; iy++)
	    {
	    tstang = (double) (*pfba) - PosA ; /* Barrel Angle - Output Angle.*/
	    pfba++ ;

	    if ( fabs (tstang) < 0.5 )		/* Angle within 0.5 degrees ? */
		{
		sval = *(pBim + diode + iy * ndiode) ;	/* pixel intensity*/

		if (sval > 0)
		    *(sy + ix) = sval ;
		else
		    *(sy + ix) = 0 ;	/* Set negative values to zero. */

		*(sy + ix) = (short) (*(sy + ix) * opalcor) ; /* opal corr. */
		break ;
	        }
	    }
        }
}

/* -------------------------------------------------------------------------- */
