/* mk4_azscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Azimuthal scan using cylindrical pB image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Author:  Alice Lecinski      HAO/NCAR   3 Nov 1997   [fillsyarry.c]
 * Updates: Andrew L. Stanger   HAO/NCAR
 * 13 Feb 2002: cosmetic changes only.
 * 20 Nov 2002: modify for Mk4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "EWmap4.h"

/* -------------------------------------------------------------------------- */

int
mk4_azscan (sbuf, pBim, fba, nang, rpix)

    short *sbuf;		/* Azimuth vector array. */
    short *pBim ;		/* Cylindrical image array. */
    float *fba ;		/* Barrel Angle vector array. */
    int nang ;			/* X-axis dimension [azimuth] of image array. */
    int rpix ;			/* Height [pixels]. */
{
    float *pfba ;
    double tstang, PosA, fabs () ;
    short *sy ;
    short sval ;
    int ix, ia ;

    pfba = fba ;
    sy   = sbuf ;

    for (ix = 0; ix < 360; ix++)	/* Azimuth Loop. */
	{
	PosA = (double) ix ;
	pfba = fba ;

	/*** Find barrel angle that matches desired angle within 0.5 deg. ***/
	/*** Copy pixel intensity into output array. ***/

	for (ia = 0; ia < nang; ia++)
	    {
	    tstang = (double) (*pfba) - PosA ; /* Barrel Angle - Output Angle.*/
	    pfba++ ;

	    if ( fabs (tstang) < 0.5 )		/* Angle within 0.5 degrees ? */
		{
		sval = *(pBim + ia + rpix * nang) ;	/* pixel intensity*/

		if (sval > 0)
		    *(sy + ix) = sval ;
		else
		    *(sy + ix) = 0 ;	/* Set negative values to zero. */

		break ;
	        }
	    }
        }
    return (0) ;
}

/* -------------------------------------------------------------------------- */
