/*
    Below are various 1-d and 2-d filters.

    Some are modeled after idl code.

    All the below were written by Alice Lecinski, June 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the 
    code.  So if you do edit it, insure results pre and post
    actually match.

*/

#include <stdio.h>
#include <math.h>

#include "mk4v2.h"

float  Copy[2*VECTOR];
float  Mean[VECTOR];
float  MeanSq[VECTOR];
float  Var_X [VECTOR];

float *Copy2d  ;
float *Mean2d  ;
float *MeanSq2d;
float *Var_X2d ;

/* Since the vector is copied to a vector twice
   it's original size, this routine can do wrap around
   to handle the edges.
*/
int
MedianFilt( Vect , neigh , npts , wrap, doprint) 
    float *Vect;
    int   neigh;
    int   npts;
    int   wrap;
    int   doprint;
{
    int ii;
    int hsz;
    int mpos;
    float *pV, *pC;

    hsz = (neigh-1) >> 1 ;

    pV = Vect; pC=Copy;   for(ii=0;ii<npts;ii++){ *pC++ = *pV++; }
    pV = Vect;            for(ii=0;ii<npts;ii++){ *pC++ = *pV++; }

    if(wrap==1) {
	/* This part skips over the beginning,
	   and then handles everything after that, including
	   the end. 
	*/
        pV = Vect+hsz;
        pC = Copy;
        for(ii=hsz;ii<npts;ii++){
             fmedian(pC,neigh,pV,&mpos,doprint);
             pV++;
             pC++;
        }
	/* This part fills in the beginning. */
        pV = Vect;
        for(ii=0;ii<hsz;ii++){
             fmedian(pC,neigh,pV,&mpos,doprint);
             pV++;
             pC++;
        }
    }
    else {
        pV = Vect+hsz;
        pC = Copy;
        for(ii=hsz;ii<npts-hsz;ii++){
             fmedian(pC,neigh,pV,&mpos,doprint);
             pV++;
             pC++;
        }
    }
}

int
MedianFilter2d(Image,FilteredImage,xdim,ydim,neigh)
    float *Image;
    float *FilteredImage;
    int xdim,ydim,neigh;
{
    float *pI, *pF, *pSub , *SubImg;
    int ii,jj;
    int start,stop;
    int hsz, subsz;
    int mpos;

    subsz  = neigh*neigh;
    SubImg = (float *)malloc( (subsz*sizeof(float)) ) ;
    pSub   = SubImg;

    hsz = (neigh-1) >> 1 ;

    start = hsz*xdim + hsz;
    stop  = xdim*ydim - (hsz*xdim+hsz);

    /* This handles the center section of the image.
       It doesn't handle the very bottom or the very top.
       Edges are wrapped automatically.
    */
    pSub = SubImg;
    pI   = Image;
    pF   = FilteredImage + start;
    for(ii=start;ii<stop;ii++){
	GetSubImgf(pI,  pSub, xdim,ydim,neigh,neigh,0,0);
	fmedian   (pSub,subsz,pF,&mpos,0);
	pI++;
	pF++;
    }

    free((void *)SubImg);
}

/* Lee filter derived from idl Function Lee_filter_fast
; Faster, but less accurate Lee filter. Only recomended when N < 7.
;
;
; Function Lee_filter_exact    is
; Slow, but accurate Lee filter. Derived from
; Jong-Sen Lee, Optical Engineering 25(5), 636-643 (May 1986)
;
This is the 1-d version -- 2-d is below...
*/

/* This is the "fast" version.  */
int
LeeFilt( Vect, npts, hsz, wrap)
    float *Vect;
    int   npts;
    int   hsz;
    int   wrap;
{
    int ii;
    int neigh;
    float *pV , *pC, *pM, *pMS, *pX;
    float sigma, SigSq, ovSigSqP1;
    float ftemp, tmp;

    neigh = hsz*2+1;
    sigma = 5.;
    SigSq = sigma * sigma;
    ovSigSqP1 = 1./(sigma * sigma + 1.);

    pM=Mean; pV=Vect;   for(ii=0;ii<npts;ii++) { *pM++ = *pV++ ;  }

    if(wrap==1) Smooth_Wrap  (Mean,neigh,npts);
    else        Smooth_NoWrap(Mean,neigh,npts);

    pMS=MeanSq;  pM=Mean;   
    for(ii=0;ii<npts;ii++) { 
        ftemp = *pM * (*pM);
        *pMS  = ftemp;
        pMS++;
        pM++;
    }
    
    pV=Vect;    pM=Mean;
    for(ii=0;ii<npts;ii++) { 
        ftemp = *pV - *pM;
        *pV   = ftemp;
        pV++;
        pM++;
    }

    pX=Var_X;   pV=Vect;
    for(ii=0;ii<npts;ii++) {
        ftemp = *pV * (*pV);
        *pX   = ftemp;
        pX++;
        pV++;
    }

    if(wrap==1) Smooth_Wrap  (Var_X,neigh,npts);
    else        Smooth_NoWrap(Var_X,neigh,npts);

    pX=Var_X;    pMS=MeanSq;
    for(ii=0;ii<npts;ii++) { 
        ftemp  = *pX + *pMS;
        ftemp *= ovSigSqP1;
        ftemp -= *pMS;
        ftemp  = (ftemp >= 0.) ? ftemp : 0.;
        *pX    = ftemp;
        pX++;
        pMS++;
    }

    pX=Var_X;    pMS=MeanSq;  pM=Mean;     pV=Vect;
    for(ii=0;ii<npts;ii++) { 
        ftemp  = *pMS * SigSq;
        ftemp += *pX;
        tmp = (ftemp != 0.) ? *pX / ftemp : 0.;
        tmp *= *pV;
        tmp += *pM;
        *pV  = tmp;
        pX++;
        pMS++;
        pM++;
        pV++;
    }
}

/* This is the "fast" version... */
int
LeeFilt2d( Matrx, xdim, ydim, hsz , wrap)
    float *Matrx;
    int   xdim;
    int   ydim;
    int   hsz;
    int   wrap;
{
    int ii;
    int neigh;
    float *pV , *pC, *pM, *pMS, *pX;
    float sigma, SigSq, ovSigSqP1;
    float ftemp, tmp;
    int   Msz;
    int   npts;
    int   checkme=1;

    neigh = (hsz*2) +1;
    sigma = 5.;
    SigSq = sigma * sigma;
    npts  = xdim*ydim;
    Msz   = xdim * ydim * sizeof(float);
    ovSigSqP1 = 1./(sigma * sigma + 1.);

    Mean2d  = (float *)malloc( Msz ) ;
    MeanSq2d= (float *)malloc( Msz ) ;
    Var_X2d = (float *)malloc( Msz ) ;

    pM=Mean2d; pV=Matrx;   for(ii=0;ii<npts;ii++) { *pM++ = *pV++ ;  }

    if(wrap==1) Smooth_Wrap2d  (Mean2d,neigh,xdim,ydim);
    else        Smooth_NoWrap2d(Mean2d,neigh,xdim,ydim);

if(checkme==1) {
    pM=Mean2d; pV=Matrx;   for(ii=0;ii<npts;ii++) { *pV++ = *pM++ ;  }
}
else {
  
    pMS=MeanSq2d;  pM=Mean2d;   
    for(ii=0;ii<npts;ii++) { 
        ftemp = *pM * (*pM);
        *pMS  = ftemp;
        pMS++;
        pM++;
    }
    
    pV=Matrx;    pM=Mean2d;
    for(ii=0;ii<npts;ii++) { 
        ftemp = *pV - *pM;
        *pV   = ftemp;
        pV++;
        pM++;
    }

    pX=Var_X2d;   pV=Matrx;
    for(ii=0;ii<npts;ii++) {
        ftemp = *pV * (*pV);
        *pX   = ftemp;
        pX++;
        pV++;
    }

    if(wrap==1) Smooth_Wrap2d  (Var_X2d,neigh,xdim,ydim);
    else        Smooth_NoWrap2d(Var_X2d,neigh,xdim,ydim);

    pX=Var_X2d;    pMS=MeanSq2d;
    for(ii=0;ii<npts;ii++) { 
        ftemp  = *pX + *pMS;
        ftemp *= ovSigSqP1;
        ftemp -= *pMS;
        ftemp  = (ftemp >= 0.) ? ftemp : 0.;
        *pX    = ftemp;
        pX++;
        pMS++;
    }

    pX=Var_X2d;    pMS=MeanSq2d;  pM=Mean2d;     pV=Matrx;
    for(ii=0;ii<npts;ii++) { 
        ftemp  = *pMS * SigSq;
        ftemp += *pX;
        tmp = (ftemp != 0.) ? *pX / ftemp : 0.;
        tmp *= *pV;
        tmp += *pM;
        *pV  = tmp;
        pX++;
        pMS++;
        pM++;
        pV++;
    }


}

    free((void *)Mean2d);
    free((void *)MeanSq2d);
    free((void *)Var_X2d);
}

int
DSmooth_NoWrap(Vect,neigh,npts)
    double *Vect;
    int   neigh;
    int   npts;
{
    int ii;
    int hsz;
    double *pV;
    float  *pC;
    float mean , stddev;

    hsz = (neigh-1) >> 1 ;

    pV = Vect; pC=Copy;   for(ii=0;ii<npts;ii++){ *pC++ = (float)(*pV); pV++; }

    pV = Vect+hsz;
    pC = Copy;
    for(ii=hsz;ii<npts-hsz;ii++){
         get_mean(pC,neigh,&mean,&stddev);
         *pV = (double)mean;
         pV++;
         pC++;
    }

}

int
Smooth_NoWrap(Vect,neigh,npts)
    float *Vect;
    int   neigh;
    int   npts;
{
    int ii;
    int hsz;
    float *pV, *pC;
    float mean ;

    hsz = (neigh-1) >> 1 ;

    pV = Vect; pC=Copy;   for(ii=0;ii<npts;ii++){ *pC++ = *pV++; }
    pV = Vect+npts-1;     for(ii=0;ii<npts;ii++){ *pC++ = *pV--; }

    pV = Vect+hsz;
    pC = Copy;
    for(ii=hsz;ii<npts-hsz;ii++){
         get_mean_fast(pC,neigh,&mean);
         *pV = mean;
         pV++;
         pC++;
    }

}

int
Smooth_Wrap(Vect,neigh,npts)
    float *Vect;
    int   neigh;
    int   npts;
{
    int ii;
    int hsz;
    float *pV, *pC;

    hsz = (neigh-1) >> 1 ;

    pV = Vect; pC=Copy;   for(ii=0;ii<npts;ii++){ *pC++ = *pV++; }
    pV = Vect;            for(ii=0;ii<npts;ii++){ *pC++ = *pV++; }

    pV = Vect+hsz;
    pC = Copy;
    for(ii=hsz;ii<npts;ii++){
         get_mean_fast(pC,neigh,pV);
         pV++;
         pC++;
    }
    pV = Vect;
    for(ii=0;ii<hsz;ii++){
         get_mean_fast(pC,neigh,pV);
         pV++;
         pC++;
    }

}




int
Smooth_NoWrap2d(Matx,neigh,xdim,ydim)
    float *Matx;
    int   neigh;
    int   xdim;
    int   ydim;
{
    int ii, jj;
    int hsz, Msz;
    float *pV, *pC, *ExtraRoom;

    hsz     = (neigh-1) >> 1 ;
    Msz     = xdim * ydim * sizeof(float);
    Copy2d  = (float *)malloc( Msz+Msz );

    /* Make a copy of the original NOT simulating azimuthal wrap. */
    pC = Copy2d;
    for(jj=0;jj<ydim;jj++){
        /* Duplicate the 1st data point over and over. */
            pV = Matx   + jj*xdim ;
            for(ii=0; ii<hsz ;ii++){ *pC++ = *pV  ; }

        /* Copy the middle. */
            pV = Matx   + jj*xdim ;
            for(ii=0; ii<xdim;ii++){ *pC++ = *pV++; }

        /* Duplicate the 1st data point over and over. */
            pV = Matx   + jj*xdim + (xdim-1);
            for(ii=0; ii<hsz ;ii++){ *pC++ = *pV  ; }
    }

    ExtraRoom = Copy2d;
    Smoothie2d(Matx,ExtraRoom,neigh,xdim,ydim);

    free((void *)Copy2d);
       

}

int
Smooth_Wrap2d(Matx,neigh,xdim,ydim)
    float *Matx;
    int   neigh;
    int   xdim;
    int   ydim;
{
    int ii, jj;
    int hsz, Msz;
    float *pV, *pC, *ExtraRoom;

    hsz     = (neigh-1) >> 1 ;
    Msz     = xdim * ydim * sizeof(float);
    Copy2d  = (float *)malloc( Msz+Msz );
 
    /* Make a copy of the original simulating azimuthal wrap. */
    pC = Copy2d;
    for(jj=0;jj<ydim;jj++){
        /* Copy the end to the start. */
            pV = Matx   + jj*xdim + (xdim-1) - (hsz-1); 
            for(ii=0; ii<hsz ;ii++){ *pC++ = *pV++; }

        /* Copy the middle. */
            pV = Matx   + jj*xdim ;
            for(ii=0; ii<xdim;ii++){ *pC++ = *pV++; }

        /* Copy the start to the end. */
            pV = Matx   + jj*xdim ;
            for(ii=0; ii<hsz ;ii++){ *pC++ = *pV++; }
    }
    ExtraRoom = Copy2d;
    Smoothie2d(Matx,ExtraRoom,neigh,xdim,ydim);

    free((void *)Copy2d);
}

int
Smoothie2d(Matx,Room,neigh,xdim,ydim)
    float *Matx;
    float *Room;
    int   neigh;
    int   xdim;
    int   ydim;
{
    int ii, jj, kk, ll;
    int Start,Stop;
    int hsz;
    int Exsz;
    int neighSq;
    float *pV, *pR, *pN, *ppR;
    float MySum, ftemp, ovneighSq;

    hsz     = (neigh-1) >> 1 ;
    neighSq = neigh*neigh;
    ovneighSq = 1./(float)neighSq;
    Exsz    = (xdim + 2*hsz);
 

    { /* Try to handle the bottom  */
	Start = 0;
	Stop  = neigh;
	pV    = Matx + xdim*Start;
	pR    = Room + Exsz*Start;
	for(jj=Start;jj<Stop;jj++){
	    for(ii=0;ii<xdim;ii++){
		MySum = 0.;
		for(kk=0;kk<neigh;kk++) {
		    ppR = pR + kk*Exsz;
		    for(ll=0;ll<neigh;ll++) {
		       ftemp  = *ppR;
		       MySum += ftemp;
		       ppR++;
		    }
		}
		*pV = MySum * ovneighSq;
		pV++;
		pR++;
	    }
	    pR = Room +jj*Exsz;
	}
    }

    { /* Try to handle the top */
	Start = ydim - neigh;
	Stop  = ydim;
	pV    = Matx + xdim*Start;
	pR    = Room + Exsz*Start;
	for(jj=Start;jj<Stop;jj++){
	    for(ii=0;ii<xdim;ii++){
		MySum = 0.;
		for(kk=0;kk<neigh;kk++) {
		    ppR = pR + kk*Exsz;
		    for(ll=0;ll<neigh;ll++) {
		       ftemp  = *ppR;
		       MySum += ftemp;
		       ppR++;
		    }
		}
		*pV = MySum * ovneighSq;
		pV++;
		pR++;
	    }
	    pR = Room +jj*Exsz;
	}
    }


    /* Get the middle */
    Stop  = ydim - 2*hsz ;
    pV    = Matx + xdim*(hsz-1);
    pR    = Room;
    for(jj=0;jj<Stop;jj++){
        for(ii=0;ii<xdim;ii++){
            MySum = 0.;
            for(kk=0;kk<neigh;kk++) {
                ppR = pR + kk*Exsz;
                for(ll=0;ll<neigh;ll++) {
		   ftemp  = *ppR;
                   MySum += ftemp;
                   ppR++;
                }
            }
            *pV = MySum * ovneighSq;
            pV++;
            pR++;
        }
        pR = Room +jj*Exsz;
    }

}

int
smoothie(V,npts,smoolen)
    float *V;
    int  npts;
    int  smoolen;
{
    int wrap;
    int doprint;
    wrap=0;
    doprint=0;
    MedianFilt (V,smoolen,npts,  wrap,doprint);
    LeeFilt    (V,        npts,5,wrap);
}



int
replace_bad(V,npts,limit,replacement)
    float *V;
    int npts;
    float limit;
    float replacement;
{
    int ii;
    float *pV;
    float  ftmp, ftmp2;
    double dtmp, dlimit;
    int doprint=0;

    dlimit = (double) (limit);

    if(doprint==1)
        fprintf(stderr,"limit %f  replacement %f\n",limit,replacement);

    pV=V;
    for(ii=0;ii<npts;ii++){
        dtmp = (double) *pV;
        ftmp = ((fabs(dtmp)>dlimit)||(isnan(dtmp)==1)) ? replacement : *pV ;
        *pV  = ftmp;
        pV++;
    }
}

DSmooth_LeastSq(X,Y,npts,Rlimit)
  double *X;
  double *Y;
  int npts;
  float Rlimit;
{
    double *pX,  *pY;
    float  *pGX, *pGY;
    float  a,b,R;
    float  testa,testb;
    int ii, numgood;

    /* Rebuild the arrays to use, skipping those points flagged by a -1. */

    numgood=0;
    pX = X;
    pY = Y;
    pGX = Var_X;
    pGY = Mean;
    for(ii=0;ii<npts;ii++){
        if ( *pX >= 0. ) {
            numgood++;
            *pGX = (float)(*pX);
            *pGY = (float)(*pY);
            pGX++;
            pGY++;
        }
        pX++;
        pY++;
    }

    pGX = Var_X; 
    pGY = Mean;
    do_lst(&a,&b,&R,pGX,pGY,numgood,1);   testa = ( a >= 0. ) ? a : -1.*a ;
                                          testb = ( b >= 0. ) ? b : -1.*b ;

    if(R > Rlimit) {
        pY = Y;
        for(ii=0;ii<npts;ii++){
            *pY = (double)( a + b *(float)ii );
            pY++;
        }
        fprintf(stderr,"\t\tGood correlation and can replace.\n");
        return(1);
    }
    else if((testb*100.<testa) && (R > 0.9*Rlimit)) {
        pY = Y;
        for(ii=0;ii<npts;ii++){
            *pY = (double)( a + b *(float)ii );
            pY++;
        }
        fprintf(stderr,"\t\tAcceptable correlation and can replace.\n");
        return(0);
    }
    else {
        fprintf(stderr,"\t\tBAD correlation and can NOT replace.\n");
        return(-1);
    }
}

int
zeroLO( im , xdim , spot ) float *im; int xdim; int spot;
{
    int ii;
    float *pf;

    fprintf(stderr,"\t\tZeroing LOW\n");

    pf  = im ;
    for(ii=0;ii<spot*xdim;ii++){
        *pf = 0.;
        pf++;
    }
}

int
zeroHI( im , xdim , ydim , spot ) float *im; int xdim; int ydim; int spot;
{
    int ii, imsz;
    float *pf;

    fprintf(stderr,"\t\tZeroing HIGH\n");

    imsz=xdim*ydim;

    pf  = im +spot*xdim;
    for(ii=spot*xdim;ii<imsz;ii++){
        *pf = 0.;
        pf++;
    }
}

