/* polvigfv2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Apply an artificial 'vignetting' to polar coordinate mk4 images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adapted from 'cy_slit4.c', authored by Alice Lecinski
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   4 April 2000
 * 29 Nov 2000: Replace "dofn" with dofname4 (use "vig" instead of "slit").
 * 16 May 2001: Add David Elmore's May 2001 model.
 * 19 Oct 2001: Use FITS format (instead of native Mk4 format).
 *  2 Nov 2001: Use version 2 include files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <ctype.h>
#include <math.h>
#include <time.h>

#define START

#include "/opt/share/PUBLIC/include/fitsio.h"
#include "mk4v2.h"
#include "mk4headv2.h"
#include "telscp4v2.h"
#include "/home/cordyn/mk4/src/model4.h"
#include "/home/cordyn/mk4/src/model4_May2001.h"
#include "/home/cordyn/mk4/src/crunch.h"

#define MODEL  1
#define CORONA 2
#define TSCP   3
#define MAY2001MODEL  4

char *fits_file, picnm [64], *piceos ;
char chd [1024] ;
char new_file [80] ;
char vig_file [80] ;
char tscpfile [80] ;

short  MyIm [CY_IMG] ;
short  MyBA [VECTOR] ;

float Kim [CY_IMG] ;
float Qim [CY_IMG] ;
float Aim [CY_IMG] ;
float meanQ [VECTOR] ;

fitsfile *fpimg, *fpvig ;
void prsum () ;
void fixLO () ;
void fixHI () ;
void model_RadVect_table () ;
void model_RadVect () ;
void model_Resid_orig () ;
void model_Resid_orig2 () ;
void model_Resid () ;

/* -------------------------------------------------------------------------- */

main (an, av)

char *av [] ;
{
    FILE *fid ;
    extern char *mk4name () ;

    register char *w ;
    static char bunit     [24] ;
    static char bunitv    [24] ;
    static char comment   [48] ;
    static char coordnam  [24] ;
    static char date_obs  [24] ;
    static char time_obs  [24] ;
    static char instrume  [12] ;
    static char telescop  [12] ;
    static char code_obs  [12] ;
    static char type_obs  [24] ;
    static char type_obsv [24] ;

    static char cbscale   [48] ;
    static char cbzero    [48] ;
    static char cbunit    [48] ;
    static char ccoordnam [48] ;
    static char cnaxis1   [48], cnaxis2 [48] ;

    static char xtension  [12] ;
    static char extname   [12] ;

    static char anynullc ;

    long begtime, endtime ;
    long btime, etime ;

    int anynull ;
    int bitpix ;
    int bitpix1, bitpix2 ;
    int dotest ;
    int extdim = 1024 ;
    int hipix ;
    int howfar ;
    int hdunum ;
    int hdutype ;
    int i ;
    int ii ;
    int ival ;
    int ix, iy ;
    int iyear, imonth, iday ;
    int ihour, iminute, isecond ;
    int method ;
    int doelmore ;
    int lozero ;
    int lopix ;
    int lspot ;
    int morekeys = 0 ;
    int ndims = 2 ;
    int ncode ;
    int nbypix ;
    int npix ;
    int nullval = 0 ;
    int status = 0 ;
    int xdim, ydim ;

    static int naxis [2] ;
    static long fpiximg [2] ;
    long fpixext = 1 ;
    static long naxes [2] ;

    float *pf, ftemp ;
    float MyOffset ;
    float bscale ;
    float bzero ;
    float bscale_inp ;
    float bzero_inp ;
    float datamin ;
    float datamax ;
    float dispmin ;
    float dispmax ;
    float fmin, fmax ;

    double dtmp ;
    double scale = 1.0 ;
    double zero  = 0.0 ;

    struct tm *T, *localtime () ;
    char *ctime_r () ;
    static char buf [30] ;

    short *simg ;
    float *fimg ;
    float *vimg ;

    if ( an <= 1 ) { prsum () ; exit (3) ; }

    (void) time (&begtime) ;
    T = (struct tm *) malloc (sizeof (struct tm)) ;
    T = localtime (&begtime) ;
    T->tm_mon += 1 ;

    (void) ctime_r (&begtime, &buf, 30) ;
    fprintf (stderr, "Started polvigf:      %s\n", buf) ;

    timeflag = 1 ;
    dotest = 0 ;
    method = MODEL ;
    lspot = 32 ;

    while (--an)
	{		/* Read in the files and perform the calibration */
	w = *++av ;
	if (*w == '?') { prsum () ; exit (2) ; }
	if (*w == '-')
	    {				/* Check for flag arguments. */
	    switch (w [1])
		{
		case 'L': lspot = BOTPIX ;
			  break ;
		case 'C': method = CORONA ; 
			  break ;
/*
		case 'T': method = TSCP ; 
			  an-- ;   w = *++av ;   sscanf (w, "%s", tscpfile) ;
			  readtscp (tscpfile, thd, 1) ;
			  pf = thd->DriftRefVect ;
			  ftemp  = *(pf + 45) ;
			  ftemp += 400.0 ;
			  for (ii = 0; ii < 45; ii++)
			      {
			      *pf++ = ftemp ;
			      }
			  break ;
 */
		case 'M': method = MAY2001MODEL ;
			  break ;
		case 'O': an-- ; w = *++av ; sscanf (w, "%f", &MyOffset) ;
			  break ;
                default:  fprintf (stderr, " unknown flag %s\n", *av) ;
                          prsum () ; exit (1) ;
                }
	    continue ;
	    }
	fits_file = w ;

	(void) time (&btime) ;


	/* Check on the file naming convention used on the file */

	if ((*w == '0') || (*w >= '8' && *w <= '9') || *(w + 2) == 'd' )
	    {
	    if ( *(w + 3) >='0' && *(w+3) <='9' )  doyflag++ ;
	    else                                   nameflag++ ;
	    }
	if (nameflag || doyflag) { dayflag++ ; timeflag=1 ; }

        /*** Open FITS file. ***/

	if ( fits_open_file (&fpimg, fits_file, READONLY, &status) )
	    printerror ( status );

	if (fits_get_num_hdus (fpimg, &hdunum, &status)) printerror (status) ;
	printf ("#hdus: %d\n", hdunum) ;

	if (fits_get_hdu_num (fpimg, &hdunum))   printerror (status) ;

	if (fits_get_hdu_type (fpimg, &hdutype, &status)) printerror (status) ;

	printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;


        /*** Primary FITS header. ***/

	if (fits_read_key (fpimg, TINT, "BITPIX", &bitpix1,
			   comment, &status))  printerror (status) ;
	nbypix = abs (bitpix1) / 8 ;

	if (fits_read_key (fpimg, TINT, "NAXIS1", &xdim,
			   cnaxis1, &status))  printerror (status) ;

	if (fits_read_key (fpimg, TINT, "NAXIS2", &ydim,
			   cnaxis2, &status))  printerror (status) ;

	fprintf (stdout, "[1] xdim/ydim/bitpix: %d %d %d\n",
		 xdim, ydim, bitpix1) ;

	if (fits_read_key (fpimg, TSTRING, "DATE-OBS", date_obs,
			   comment, &status))  printerror (status) ;

	sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
	fprintf (stdout, "iyear: %d  imonth: %d  iday: %d\n",
		 iyear, imonth, iday) ;

	if (fits_read_key (fpimg, TSTRING, "TIME-OBS", time_obs,
			   comment, &status))  printerror (status) ;

	sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;
	fprintf (stdout, "ihour: %d  iminute: %d  isecond: %d\n",
			 ihour, iminute, isecond) ;

        if (fits_read_key (fpimg, TSTRING, "TELESCOP", telescop,
			   comment, &status))  printerror (status) ;
	printf ("telescop: %s\n", telescop) ;

        if (fits_read_key (fpimg, TSTRING, "INSTRUME", instrume,
			   comment, &status))  printerror (status) ;
	printf ("instrume: %s\n", instrume) ;

        if (fits_read_key (fpimg, TSTRING, "TYPE-OBS", type_obs,
			   comment, &status))  printerror (status) ;
	printf ("type_obs: %s\n", type_obs) ;

        if (fits_read_key (fpimg, TSTRING, "COORDNAM", coordnam,
			   ccoordnam, &status))  printerror (status) ;
	printf ("coordnam: %s\n", coordnam) ;

        if (fits_read_key (fpimg, TSTRING, "BUNIT", bunit,
			   cbunit, &status))  printerror (status) ;

	if (fits_read_key (fpimg, TFLOAT, "BSCALE", &bscale_inp,
			   cbscale, &status))  printerror (status) ;

	if (fits_read_key (fpimg, TFLOAT, "BZERO", &bzero_inp,
			   cbzero, &status))  printerror (status) ;

	printf ("bscale_inp: %12.4g  bzero_inp: %12.4g\n",
		bscale_inp, bzero_inp) ;

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpimg, hdunum, &hdutype, &status))
	    printerror (status) ;

	fprintf (stdout, "fits_movabs_hdu [2] . . . status: %d\n", status) ;

	if (fits_get_hdu_num (fpimg, &hdunum))   printerror (status) ;

	if (fits_get_hdu_type (fpimg, &hdutype, &status)) printerror (status) ;

	printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;

	/*** Read Mk4 binary header (stored as a FITS extension). ***/

	fpixext = 1 ;
        if ( fits_read_img (fpimg, TBYTE, fpixext, extdim, &nullval, mk4h,
			    &anynull, &status) )
	    printerror ( status ) ;

        /*** Create output file name. ***/

	dayflag = 1 ;
	doyflag = 0 ;
        strcpy (new_file, mk4name (mk4h)) ;

	sprintf (vig_file, "%s.vig.fts", new_file) ;
	fprintf (stdout, "vig_file: %s\n", vig_file) ;

	remove (vig_file) ;		/* Delete pre-existing file. */

	/*** Create output file. ***/

	if ( fits_create_file (&fpvig, vig_file, &status) )
	    printerror ( status ) ;

	/*** Move to FITS primary HDU [input file]. ***/

	hdunum = 1 ;
	if (fits_movabs_hdu (fpimg, hdunum, &hdutype, &status))
	    printerror (status) ;

	/*** Copy 1st HDU to output file. ***/

	morekeys = 0 ;
	if ( fits_copy_hdu (fpimg, fpvig, morekeys, &status) )
	    printerror ( status );

	/*** Read FITS image data. ***/

        npix    = xdim * ydim ;
	simg    = (short *) malloc (xdim * ydim * sizeof (short)) ;
	fimg    = (float *) malloc (xdim * ydim * sizeof (float)) ;
	vimg    = (float *) malloc (xdim * ydim * sizeof (float)) ;
	nullval = 0 ;          /* Don't check for null values in the image. */

	scale = 1.0 ;
	zero  = 0.0 ;
	if (fits_set_bscale (fpimg, scale, zero, &status)) printerror (status) ;

	fpiximg [0] = 1 ;
	fpiximg [1] = 1 ;
        if ( fits_read_img (fpimg, TFLOAT, fpiximg [0], npix, &nullval, Qim,
			    &anynull, &status) )
	    printerror ( status ) ;

	printf ("After fits_read_img\n") ;
	ix = 0 ;
	for (iy = 0; iy < ydim; iy+=10)
	    printf ("ix/iy: %3d %3d  Qim: %12.4f\n",
		    ix, iy, *(Qim + ix + iy*VECTOR));

 	HandleNeg_pB (Qim, xdim, ydim, lspot) ;

	printf ("After HandleNeg_pB\n") ;

	ix = 0 ;
	for (iy = 0; iy < ydim; iy+=10)
	    printf ("ix/iy: %3d %3d  Qim: %12.4f\n",
		    ix, iy, *(Qim + ix + iy*VECTOR));

	/*** Copy to Kim array. ***/

	for (iy = 0; iy < ydim; iy++)
	    for (ix = 0; ix < xdim; ix++)
		{
		*(Kim + ix + iy*VECTOR) = *(Qim + ix + iy*VECTOR) ;
		}

/*
	iy = ydim / 2 ;
	for (ix = 0; ix < xdim; ix++)
	    printf ("Qim [%3d][%3d]: %8.2e\n",
		    iy, ix, *(Qim + ix + iy*VECTOR)) ;
 */

	/*** Move to FITS extension HDU [input file]. ***/

	hdunum = 2 ;
	if (fits_movabs_hdu (fpimg, hdunum, &hdutype, &status))
	    printerror (status) ;

	fprintf (stdout, "fits_movabs_hdu [2] . . . status: %d\n", status) ;

	/*** Copy HDU2 to output file. ***/

	morekeys = 1 ;
	if (fits_copy_hdu (fpimg, fpvig, morekeys, &status) )
	    printerror (status);

	printf ("Copy HDU2 to output file . . . status: %d\n", status) ;

	if (fits_get_hdu_num  (fpimg, &hdunum))  printerror (status) ;

	if (fits_get_hdu_type (fpimg, &hdutype, &status)) printerror (status) ;

	printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;

	/*** Read keywords from FITS extension header. ***/

	if (fits_read_key (fpimg, TINT, "BITPIX", &bitpix2,
			   comment, &status))  printerror (status) ;
	nbypix = abs (bitpix2) / 8 ;

	if (fits_read_key (fpimg, TINT, "NAXIS1", &extdim,
			   cnaxis1, &status))  printerror (status) ;

	fprintf (stdout, "[2] extdim/bitpix: %d %d\n", extdim, bitpix2);

	/*** Read Mk4 binary header (stored as a FITS extension). ***/

	fpixext = 1 ;
        if ( fits_read_img (fpimg, TBYTE, fpixext, extdim, &nullval, mk4h,
			    &anynull, &status) )
	    printerror ( status ) ;

/*
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Year) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Month) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Day) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Hour) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Minute) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.CentiSec) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Juld) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Image.Doy) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Xdim) ;
	printf ("mk4h->Image.Year: %d\n", mk4h->Ydim) ;
 */

/*
 *	fprintf (stdout, "mk4h->Image.Year: %4d\n", mk4h->Image.Year) ;
 */

	switch (method)
	    {
	    case MAY2001MODEL:
		VVcopy (May2001model, meanQ, NPX);
		ftemp = MyOffset ;
		Vplus (meanQ, ftemp, NPX);
		ftemp = mk4h->data_offset;
		Vminus (Qim, ftemp, xdim * ydim);
		break ;
	    case MODEL:
		model_RadVect (meanQ) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi_preM", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi_pre write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof (float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
		break ;
	    case CORONA:
		VVcopy (mymodel, meanQ, NPX) ;
		ftemp = mk4h->data_offset - 50.0 ;
		Vminus (meanQ, ftemp, NPX) ;
		ftemp = mk4h->data_offset - 100.0 ;
		Vminus (Qim, ftemp, xdim * ydim) ;
		break ;
/*
	    case TSCP:
		VVcopy (thd->DriftRefVect, meanQ, NPX) ;
		ftemp = mk4h->data_offset - 10.0 ;
		Vminus (Qim, ftemp, xdim * ydim) ;
		break ;
 */
	    default:
		Im2AvgRadVect_Culled (Qim, meanQ, xdim, ydim) ;
		fixLO (meanQ, 8000.0, -1.0) ;
		fixHI (meanQ,  200.0) ;
		smoothie (meanQ, mk4h->Ydim, 13) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi_pre", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi_pre write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof (float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
	    }
	MVdivide (Qim, meanQ, Qim, xdim, ydim) ;


	switch (method)
	    {
	    case MODEL:
		model_Resid_orig2 (meanQ) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfiM", "w"))<(FILE*)0)
			{
			fprintf (stderr, "barfi write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof (float), mk4h->Ydim, fid) ;
		    fclose (fid) ;
		    }
		break ;
	    case CORONA:
		Vinitialize (meanQ, 0.0, NPX) ;
		break ;
/*
	    case TSCP:
		Vinitialize (meanQ, 0.0, NPX) ;
		break ;
 */
	    case MAY2001MODEL:
		Vinitialize (meanQ, 0.0, NPX) ;
		break ;
	    default:
		VVcopy       (Qim, Aim,  xdim * ydim) ;
		get_min_Vect (Aim, meanQ, xdim, ydim) ;
		fixLO (meanQ, 0.0, 0.6) ;
		fixHI (meanQ, 0.9) ;
		smoothie (meanQ, mk4h->Ydim, 13) ;
		if (dotest == 1)
		    { 
		    if ((fid = fopen ("barfi", "w")) < (FILE*) 0)
			{
			fprintf (stderr, "barfi write error %d\n", ii) ;
			exit (-90) ;
		        }
		    fwrite (meanQ, sizeof(float), mk4h->Ydim , fid) ;
		    fclose (fid) ;
		    }
	        }
	MVminus (Qim, meanQ, Qim, xdim, ydim) ;

	switch (method)
	    {
	    case CORONA:
		Vinitialize (meanQ, 100.0, NPX) ;
		break ;
/*
	    case TSCP:
		Vinitialize (meanQ, 180.0, NPX) ;
		break ;
 */
	    case MODEL:
	    default:
		/* Put a little extra zing into the outer field. */
		pf = meanQ ;
		for (ii = 0; ii < mk4h->Ydim; ii++)
		    {
/*
		    dtmp  = (double) ii ;
		    ftemp = (float) (pow (dtmp, 1.35)) ;
		    ftemp = (float) (pow (dtmp, 1.20)) ;
		    ftemp = (float) (pow (dtmp, 1.01)) ;
		    if (ii > 180)
			{
			dtmp  -= 280.0 ;
			ftemp -= (float) (pow (dtmp, 1.60)) ;
			dtmp  -= 180.0 ;
			ftemp -= (float) (pow (dtmp, 1.30)) ;
			dtmp  -= 180.0 ;
			ftemp -= (float) (pow (dtmp, 1.00)) ;
		        }
*/
		    ftemp = 100.0 + 2.0 * ((float) ii) ;
		    ftemp = 100.0 + 1.0 * ((float) ii) ;
		    if (ii > 372) ftemp = 0.01 ;
		    *pf++ = ftemp ;
		    }
	    }
	MVmult (Qim, meanQ, Qim, xdim, ydim) ;


	/*** Set pixels inside 1.0 Rsun to a minimum value. ***/

        lozero = BOTPIX ;
	lozero = (1.06 - mk4h->r0) / mk4h->dr ; 
	printf ("r0: %10.6g  dr: %12.8g  lozero: %d\n",
		mk4h->r0, mk4h->dr, lozero) ;
        zminlo (Qim, xdim, lozero) ;

	printf ("Vignetting computation done.\n") ;
	ix = 0 ;
	for (iy = 0; iy < ydim; iy+=10)
	    printf ("ix/iy: %3d %3d  Kim: %12.4f  Qim: %12.4f\n",
		    ix, iy, *(Kim + ix + iy*VECTOR), *(Qim + ix + iy*VECTOR));

	/* Convert floating point image to short integer. */
	/* Write 'vignetted' image to disk. */

        Float2Short (Qim, MyIm, xdim, ydim) ;

	lopix = (1.10 - mk4h->r0) / mk4h->dr ; 
	hipix = (2.25 - mk4h->r0) / mk4h->dr ; 
	printf ("lopix [1.10 Rsun]: %d  hipix [2.25 Rsun]: %d\n",
		lopix, hipix) ;
	fmin =  1.e+30 ;
	fmax = -1.e+30 ;
	for (iy = 0; iy < ydim; iy++)
	    for (ix = lopix; ix < hipix; ix++)
		{
		if (*(Qim + ix + iy*VECTOR) < fmin &&
		    *(Qim + ix + iy*VECTOR) > -3.0e-6) 
		    fmin = *(Qim + ix + iy*VECTOR) ;

		if (*(Qim + ix + iy*VECTOR) > fmax &&
		    *(Qim + ix + iy*VECTOR) < 3.0e-6) 
		    fmax = *(Qim + ix + iy*VECTOR) ;
/*
 *		*(MyIm + ix + iy*VECTOR) = (short) *(Qim + ix + iy*VECTOR) ;
 */
		}

	printf ("fmin/fmax: %8.2e  %8.2e\n", fmin, fmax) ;

	/*** Move to FITS primary HDU [output file]. ***/

	hdunum = 1 ;
	if (fits_movabs_hdu (fpvig, hdunum, &hdutype, &status))
	    printerror (status) ;

	/*** Update keywords in primary HDU header. ***/

	bitpix = bitpix1 ;
	if (fits_update_key (fpvig, TINT, "BITPIX", &bitpix, 0, &status))
	     printerror ( status );

	sprintf (type_obsv, "%s-VIG", type_obs) ;
	if (fits_update_key (fpvig, TSTRING, "TYPE-OBS", type_obsv, 0, &status))
	     printerror ( status );

	sprintf (bunitv, "%s-VIG", bunit) ;
	if (fits_update_key (fpvig, TSTRING, "BUNIT", bunitv, cbunit, &status))
		     printerror ( status );

	bscale = 1.0 ;
	bzero  = 0.0 ;
	if (fits_update_key (fpvig, TFLOAT, "BSCALE", &bscale, 0, &status))
	     printerror ( status );

	if (fits_update_key (fpvig, TFLOAT, "BZERO", &bzero, 0, &status))
	     printerror ( status );

        datamin = fmin ;
	if (fits_update_key (fpvig, TFLOAT, "DATAMIN", &datamin, 0, &status))
	     printerror( status );
	     printerror( status );

	datamax = fmax ;
	if (fits_update_key (fpvig, TFLOAT, "DATAMAX", &datamax, 0, &status))
	     printerror( status );

	dispmin = fmin ;
	dispmin = -100.0 * bscale_inp + bzero_inp ;
	dispmin = -100.0 * 1.0e-10 ;
	dispmin = 0.0 ;
	if (fits_update_key (fpvig, TFLOAT, "DISPMIN", &dispmin, 0, &status))
	     printerror( status );

	dispmax = fmax ;
	dispmax = 650.0 * bscale_inp + bzero_inp ;
	dispmax = 750.0 * 1.0e-10 ;
	if (fits_update_key (fpvig, TFLOAT, "DISPMAX", &dispmax, 0, &status))
	     printerror( status );

	if (ffgidt (fpvig, &bitpix, &status)) printerror (status) ; ;
	printf ("fpvig: bitpix = %d\n", bitpix) ;

	if (ffgidm (fpvig, naxis, &status)) printerror (status) ; ;
	printf ("fpvig: naxis = %d %d\n", naxis [0], naxis [1]) ;

	/*** Write vignetted image array into the FITS data array. ***/

	npix = xdim * ydim ;
	printf ("xdim/ydim: %d %d\n", xdim, ydim) ;

	fpiximg [0] = 1 ;
	fpiximg [1] = 1 ;

/*
	if (ffppx (fpvig, TSHORT, fpiximg, npix, MyIm, &status) )
	    printerror ( status ) ;
 */

	ix = 0 ;
	for (iy = 0; iy < ydim; iy+=10)
	    printf ("ix/iy: %3d %3d  Qim: %12.4f  MyIm: %6d\n",
		    ix, iy, *(Qim + ix + iy*VECTOR), *(MyIm + ix + iy*VECTOR));

	scale = 1.0 ;
	zero  = 0.0 ;
	if (fits_set_bscale (fpvig, scale, zero, &status)) printerror (status) ;

/*
	if (fits_write_img (fpvig, TSHORT, fpiximg [0], npix, MyIm, &status) )
	    printerror ( status ) ;
 */

	if (fits_write_img (fpvig, TFLOAT, fpiximg [0], npix, Qim, &status) )
	    printerror ( status ) ;

	printf ("After fits_write_img [fpvig, hdu1]\n") ;

	/*** Fix and fill in the headers with new info. ***/

	switch (mk4h->ProcSteps)
	    {
	    case IS_cal               : howfar = IS_cal_slit ; break ;
	    case IS_cal_avg           : howfar = IS_cal_avg_slit ; break ;
	     
            default                   : howfar = IS_UNKNOWN ; break ;

	    }
	mk4h->ProcSteps   = howfar ;
	mk4h->data_scale  = badval ;
	mk4h->data_offset = badval ;
	mk4hdp (mk4h) ;

	/*** Move to FITS extension HDU [output file]. ***/

	printf ("Update keywords [fpvig, hdu2]\n") ;

	hdunum = 2 ;
	if (fits_movabs_hdu (fpvig, hdunum, &hdutype, &status))
	    printerror (status) ;

	sprintf (xtension, "IMAGE   ") ;
	if (fits_update_key (fpvig, TSTRING, "XTENSION", &xtension, 0, &status))
	     printerror ( status );

	if (fits_update_key (fpvig, TINT, "PROCSTEP", &howfar, 0, &status))
	     printerror ( status );

	sprintf (extname, "MK4_HSKP") ;
	if (fits_update_key (fpvig, TSTRING, "EXTNAME", &extname, 0, &status))
	     printerror ( status );

	/*** Write binary header into the FITS extension [output file]. ***/

	printf ("fits_write_img [fpvig, hdu2]\n") ;
	npix = 1024 ;
	if ( fits_write_img (fpvig, TBYTE, fpixext, npix, mk4h, &status) )
	    printerror ( status ) ;

	/*** Close both input & output FITS files. ***/

	printf ("fits_close_file [fpimg, fpvig]\n") ;

	if (fits_close_file (fpimg, &status) ||
	    fits_close_file (fpvig, &status))		/* close files */
	    printerror ( status );

	(void) time (&etime) ;
	fprintf (stderr, 
	"\t\t\tSingle image time used: %ld seconds\n\n", etime-btime) ;
        }

    (void) time (&endtime) ;
    fprintf (stderr, "\nTotal time used  :    %ld seconds\n", endtime-begtime) ;

    exit (0) ;

}

/* -------------------------------------------------------------------------- */

void
prsum ()
{
/*
    static char help [] = "\
polvigf [-L] [-C] [-T tscpfile] [-M [-O offset]] [polar file1] [polar file2] ... [polar fileN]\n\
\n\
Example: polvigf 92d004.17_28.mk4.pB  92d004.17_32.mk4.pB\n\
Example: polvigf -L -M -O 100. `ls -1 *.mk4.pB`\n\
\n\
\n\
Flags          Description\n\
-----          -----------\n\
-L             Only fixes neg pB values down to pixel BOTPIX instead of\n\
	       pixel 32 (the default).\n\
-C             Will use a coronal model as the vignetting function\n\
-T tscpfile    Will use the DriftRefVect from the tscp file at the\n\
	       vignetting function.\n\
-M             Will use David Elmore\'s May 2001 coronal model for the\n\
               vignetting function.\n\
-O offset      The offset to use in the May 2001 model, where:\n\
	       pBvig = pB / (May_2001_model + offset).\n\
\n\
" ;
 */

    static char help [] = "\
polvigf [-L] [-C] [-M [-O offset]] [polar file1] [polar file2] ... [polar fileN]\n\
\n\
Example: polvigf 92d004.17_28.mk4.pB  92d004.17_32.mk4.pB\n\
Example: polvigf -L -M -O 100. `ls -1 *.mk4.pB`\n\
\n\
\n\
Flags          Description\n\
-----          -----------\n\
-L             Only fixes neg pB values down to pixel BOTPIX instead of\n\
	       pixel 32 (the default).\n\
-C             Will use a coronal model as the vignetting function\n\
-M             Will use David Elmore\'s May 2001 coronal model for the\n\
               vignetting function.\n\
-O offset      The offset to use in the May 2001 model, where:\n\
	       pBvig = pB / (May_2001_model + offset).\n\
\n\
" ;

       fputs (help, stderr) ;
}

/* -------------------------------------------------------------------------- */

void
fixLO (Vect, flim, frepla)

float *Vect ;
float flim ;
float frepla ;
{
    int ii ;
    float *pf, fval ;

    ii   = 50 ;
    pf   = Vect+ii ;
    while ((ii > 0) && (*pf > flim)) { ii-- ; fval = *pf ; pf-- ; }

    if (frepla > 0.0) fval = frepla ;

    while (ii > 0)                { ii-- ; *pf = fval ; pf-- ; }


}

/* -------------------------------------------------------------------------- */

void
fixHI (Vect, flim)

float *Vect ;
float flim ;
{
    int ii ;
    float *pf, fval ;

    ii   = 260 ;
    pf   = Vect + ii ;
    while ((ii < 384) && (*pf > flim)) { ii++ ; fval = *pf ; pf++ ; }

    while (ii < 384)                { ii++ ; *pf = fval ; pf++ ; }

}

/* -------------------------------------------------------------------------- */

void
model_RadVect_table (Vect)

float *Vect ;
{
    int ii ;
    float *pf, *pm, fval ;
    float hlim, llim ;

    hlim = 10000. ;
    llim = 1040. ;

    pf = Vect ;
    pm = mymodel ;
    for (ii = 0; ii < NPX ; ii++)
	{
	fval  = *pm ;
	fval *= 0.3 ;
	if     (fval <=    0.0) fval = hlim ;
	else if (fval >  hlim ) fval = hlim ;
	else if (fval <  llim ) fval = llim ;
	else ;
	*pf = fval ;
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

void
model_RadVect (Vect)

float *Vect ;
{
    int ii ;
    float *pf, fval, tval, slp, ycpt, y2, y1, x2, x1 ;
    float hlim, llim ;

    hlim = 10000.0 ;
    llim = 1040.0 ;
    llim = 1000.0 ;
    llim = 1040.0 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	tval = (float) ii ;
	fval = tval * 0.1 - 4.2 ;
	fval = 500.0 + 12000.0 / fval ;
	if (ii >= 40 && ii <= 70) fval *= 1.0 * ( tval) / ( 70.0) ;
	if      (fval <=    0.0) fval = hlim ;
	else if (fval >   hlim ) fval = hlim ;
	else if (fval <   llim ) fval = llim ;
	else ;
	*pf = fval ;
	pf++ ;
        }


    x1 = 220.0 ;
    ii = (int) x1 ;
    y2   = *(Vect + 383) ;   x2 = 383.0 ;
    y1   = *(Vect + ii ) +10.0 ;
    slp  = (y2 - y1) / (x2 - x1) ;
    ycpt = y1 - slp*x1 ;
    ii = (int) x1 ;
    pf = Vect + ii ;
    for (ii = (int) x1; ii < NPX; ii++)
	{
	tval = (float) ii ;
	fval = slp * tval + ycpt ;
	fval = (fval - (tval-x1) * 1.00) ;
	if (fval < llim ) fval = llim ;
	*pf = fval ;
	pf++ ;
        }
}


/* -------------------------------------------------------------------------- */

void
model_Resid_orig (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    double dval ;
    double dlim, hlim, ov600 ;

    hlim= 1.0 ;
    ov600 = 1.0 / 600.0 ;
    ov600 = 1.0 / 900.0 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	dval = (float) ii * pi * ov600 ;
	/* dval = sin (dval) * 0.5 + 0.4 ; */
	dval = sin (dval) * 0.6 + 0.5 ;
	if (dval > hlim ) dval = hlim ;
	*pf = (float) dval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

void
model_Resid_orig2 (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    float fval ;


    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	fval = ((float) ii -380.0) * 0.0025 ;
	fval = ((float) ii -400.0) * 0.0025 ;
	fval = 1.0 - fval * fval ;
	*pf = fval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

void
model_Resid (Vect)

float *Vect ;
{
    int ii ;
    float *pf ;
    float fval ;
    float hlim ;

    hlim = 1.2 ;

    pf = Vect ;
    for (ii = 0; ii < NPX; ii++)
	{
	fval = (float) ii * 0.0015 + 0.5 ;
	if (fval > hlim ) fval = hlim ;
	*pf = fval ; 
	pf++ ;
        }
}

/* -------------------------------------------------------------------------- */

