/*
; Intensity to polarization
;
; Smooth corona.  Smooth opal.  Take the difference
; between smoothed corona and corona and add to opal
; This produces X21 and X31 with gain set by opal, but
; ring structure from coronal data
; Also throw in an attempt to correct for polarized diffraction
; around occulter.  Absolute pixel numbers may need to be put
; into settings file.
;
; Period of Lee filter is important.  Too high and ring structure not
; compensated.  Too low and calibration introduces noise into the data.
; As now set, old camera data with clean lens is noisy.
;
*/

#include <stdio.h>
#include <ctype.h>
#include <time.h>
#include <math.h>

#include "mk4v2.h"
#include "mk4headv2.h"
#include "telscp4v2.h"

extern float XimA[CY_IMG];
extern float XimB[CY_IMG];
extern float XimC[CY_IMG];
extern float Xspace[VECTOR];
extern float Avg [VECTOR];
extern float StdD[VECTOR];



int
x321 (ovopalI,opalQ,opalU,
      ovcornI,cornQ,cornU,
      Thd, is_x321)

    float *ovopalI, *opalQ, *opalU;
    float *ovcornI, *cornQ, *cornU;
    struct telhd *Thd; 
{
    int   doprint;
    int   ii,jj;
    int   isz;
    int   lpx,hpx;
    float *pxxx;
    float *cxxx, *oxxx, *ocxxx;
    float *pX, *pZ;
    float *pAvg,*pStdD;
    float Sum,mean,stddev;
    float rat;

    int   dome=0;


    isz   = NAZ*NPX;


    /* Recall, ovopalI and ovcornI are 1./opalI and 1./cornI  */

	if(is_x321 == 21 )   pxxx  = Thd->x21a;
	else                 pxxx  = Thd->x31a;
	oxxx  = XimA;
	cxxx  = XimB;
	ocxxx = XimC;

    /* Here is the first cut X21 or X31 */
	if(is_x321 == 21 )   
	    VVmult(opalQ,ovopalI,pxxx,isz);  /* x21 = opalQ/opalI */
	else
	    VVmult(opalU,ovopalI,pxxx,isz);  /* x31 = opalU/opalI */

    
    /* Remove sine waves.
    */
	lpx=100; 
	hpx=300;
	GetSignal     (pxxx,Xspace,lpx,hpx,NAZ,NPX);
	get_mean_fast (Xspace,NAZ,&mean);
	Vminus        (Xspace,mean,NAZ);
	MVminusReplace(pxxx,Xspace,NAZ,NPX,  NAZ);



    /*      ; cal0_q is really noisy, the median helps the leefilt
	    ; remove this noise
    */
	VVcopy        (pxxx,oxxx,isz);
	Smooth_Wrap2d (oxxx,   7,NAZ,NPX);



    /* Do a similar thing for xxx based on the coronal data.
    */

	if(is_x321 == 21 )
	    VVmult  (cornQ,ovcornI,cxxx,isz);  /* cx21 = cornQ/cornI */
	else
	    VVmult  (cornU,ovcornI,cxxx,isz);  /* cx31 = cornU/cornI */


	/* Save the avg and stddev of the initial cxxx */
	pX    = cxxx;
	pAvg  = Avg;
	pStdD = StdD;
	for(ii=0;ii<NPX;ii++){
	    VVcopy   (pX,Xspace,NAZ);
	    ReplaceLE(Xspace,NAZ,1,-10000.,-9999. , 0);
	    ReplaceGE(Xspace,NAZ,1, 10000., 9999. , 0);
	    get_mean (Xspace,NAZ,&mean,&stddev);
	    *pAvg  = mean;
	    *pStdD = stddev;
	    pAvg++;
	    pStdD++;
	    pX += NAZ;
	}

	lpx=300;
	hpx=350;
	GetSignal     (cxxx,Xspace,lpx,hpx,NAZ,NPX);
	get_mean_fast (Xspace,NAZ,&mean);
	Vminus        (Xspace,mean,NAZ);
	MVminusReplace(cxxx,Xspace,NAZ,NPX,  NAZ);


	VVcopy        (cxxx ,ocxxx,isz);
	Smooth_Wrap2d (ocxxx,    7,NAZ,NPX);



	VVplus (oxxx, cxxx,pxxx,isz);
	VVminus(pxxx,ocxxx,pxxx,isz);


	if( is_x321 == 21 ) {
	    /*      ; Slowly varying component from cal0
		    ; Structure using coronal image with corona removed
		    ; except where smoothed corona is < smoothed cal0
	    */
	    lpx=340;
	    hpx=350;
	    GetSignal (ocxxx,Xspace,lpx,hpx,NAZ,NPX);
	    Total     (Xspace,NAZ,&Sum);
	    rat = Sum;
	    GetSignal (oxxx, Xspace,lpx,hpx,NAZ,NPX);
	    Total     (Xspace,NAZ,&Sum);
	    if ( Sum != 0. ) rat /= Sum;
	    else             rat  =  1.0;

	    fprintf(stderr,"\t    rat is %f\n",rat);

	    Vmult  (pxxx,  rat,     isz);
	    pX =  pxxx + NAZ*345;
	    pZ = ocxxx + NAZ*345;
	    VVcopy(pZ,pX,(NPX-345)*NAZ);
	}


    /*      ; Polarized diffraction around occulter shows up only in corona
	    ; data.
	    ; Emperical determination of amount of corona variation.
	    ; Note: hardwired pixels here, should be in some define
	    ;
    */
	Vmult  (StdD, Thd->hd0i.ID.DiffractFac,      NPX);
	VVminus(Avg ,                     StdD, Avg, NPX);

	pX =  pxxx;
	pZ =  Avg;
	for(ii=0;ii<=Thd->hd0i.ID.DiffractPix;ii++){
	    Vinitialize(pX,*pZ,NAZ);
	    pX += NAZ;
	    pZ++;
	}



    /*      ; Note: leefilter at each height so not to smear rings.
	    ; Frequency of ring variations at a given height is >> 19
    */


    pX = pxxx;
    for (ii=0;ii<NPX;ii++){
	Smooth_Wrap(  pX,   19, NAZ);
	pX += NAZ;
    }

    /* That's it!! */

}
