/*  fixmk4v2.c
 *
 /
/*  Fix various "problems" in the mk4 raw data.

    This code was written by Alice Lecinski, September 2001.

    It is modeled after David Elmore's mk4 calibraton
    routines written as idl procedures.  David developed these
    procedures in fall of 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/
/*
 * ALS 11 Mar 2002: cosmetic changes only.
 */

#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "mk4v2.h"
#include "mk4headv2.h"


int    indx [VECTOR];
float findx [VECTOR];
float fyval [VECTOR];
float CorFac  [VECTOR];
float ovCorFac [VECTOR];
float BiasArray [VECTOR];

/*
char CorrectorTablePathHAO [80]  = "/thisbe/e/elmore/MkIV/Correct/";
*/
char CorrectorTablePathHAO [80]  = "/puu/f/cordyn/mk4_cal/";
char CorrectorTablePathMLSO [80] = "/local/d/Data/Cal/";
char CorrectorTablePath [80] ;

int
fixmk4 (Iim, hdi, Qim, hdq, Uim, hdu, fba, hd, doprint)

    float *Iim;
    FH hdi;
    float *Qim;
    FH hdq;
    float *Uim;
    FH hdu;
    float *fba;
    FH hd;
    int    doprint;
{
    int ii, jj, kk;
    int pix, lpx, hpx;
    int imsz, xdim, ydim;
    float    *pf, ftemp;
    int HaveU = 1;
    int OldDalsa;

    OldDalsa = FALSE;
    if(hd->Image.Juld < MAY01_2001 ) OldDalsa = TRUE;

    imsz  = hd->Xdim * hd->Ydim;

    if (hdq->ObsType == IS_coronal )
	{
        fixnegQU (Qim, Uim, hdq->Xdim, hdq->Ydim, HaveU,  hd);
        }

    if ( hdi->ObsType != IS_dark)
	{
	lpx = hdi->ID.NegPix [0];
	hpx = hdi->ID.NegPix [1];
        fixneg (Iim,     hdi->Xdim, hdi->Ydim, lpx, hpx);

	if (hdq->ObsType == IS_coronal )
	    {
	    lpx = hdi->ID.WrapPix [0];
	    hpx = hdi->ID.WrapPix [1];
	    if(OldDalsa == TRUE)
		{
		fixneg     (Iim,    hdi->Xdim, hdi->Ydim,   lpx, hpx);
		fixneg     (Qim,    hdq->Xdim, hdq->Ydim,   lpx, hpx);
	        }
	    else
		{
		fixnegWrap (Iim,    hdi->Xdim, hdi->Ydim,   lpx, hpx);
		fixnegWrap (Qim,    hdq->Xdim, hdq->Ydim,   lpx, hpx);
	        }
	    }
        }


    /*
    * Remove the corrector table gain from the image
    *
    * Correct bias error on a ray by ray basis
    * using offset of specified range of low pixels
    *
    * if old Dalsa camera apply non linear gain
    * Intensity corrected by kappa function
    * Polarization corrected by derivative of intensity
    *
    * Re-apply corrector table gain
    */

    if ( hdi->ObsType != IS_dark)
	{				/* Get the corrector table gain. */
	GetCorFac (hdi);

	/* Multiply the I data by the corrector table. */

	MVmultReplace (Iim, ovCorFac, hdi->Xdim, hdi->Ydim, hdi->Ydim);

	/* Get the bias estimate from the modified I data. */

	lpx = hdi->ID.biasspot [0];
	hpx = hdi->ID.biasspot [1];
	GetBiasEstimate (Iim, hdi->Xdim, hdi->Ydim, lpx, hpx );

	if (OldDalsa == TRUE)
	    {
	    MVmultReplace (Qim, ovCorFac, hdq->Xdim, hdq->Ydim, hdq->Ydim);
	    MVmultReplace (Uim, ovCorFac, hdu->Xdim, hdu->Ydim, hdu->Ydim);

	    MVminusReplace (Iim, BiasArray, hdi->Xdim, hdi->Ydim, hdi->Xdim);

	    DoKappaFix (Iim, Qim, Uim, hdi->Xdim, hdi->Ydim, hdi->ID.kappa);

	    MVmultReplace (Iim, CorFac,   hdi->Xdim, hdi->Ydim, hdi->Ydim);
	    MVmultReplace (Qim, CorFac,   hdq->Xdim, hdq->Ydim, hdq->Ydim);
	    MVmultReplace (Uim, CorFac,   hdu->Xdim, hdu->Ydim, hdu->Ydim);
	    }
	else
	    {
	    MVminusReplace (Iim, BiasArray, hdi->Xdim, hdi->Ydim, hdi->Xdim);
	    MVmultReplace  (Iim, CorFac,    hdi->Xdim, hdi->Ydim, hdi->Ydim);
	    }
        }

    ReplaceNonFinite (Iim, hdi->Xdim, hdi->Ydim, 1.0 , 1);
    ReplaceNonFinite (Qim, hdq->Xdim, hdq->Ydim, 0.0 , 1);
    ReplaceNonFinite (Uim, hdu->Xdim, hdu->Ydim, 0.0 , 1);

    if ( hdi->ObsType != IS_dark)
	{			/* don't let intensity be 0 or negative */
	ReplaceLE (Iim,    hdi->Xdim, hdi->Ydim,   0.0, 1.0, 1);
        }


    UnGlytchBaaaa (fba, hd->Xdim);

    if (hdi->ObsType != IS_dark)
	{
	NormBySky (Iim, hdi);
	NormBySky (Qim, hdq);
	NormBySky (Uim, hdu);
        }
    
}

/* -------------------------------------------------------------------------- */

int
NormBySky ( im, hd )

float *im; FH hd;
{
    float sky;
    float *pf;
    int ii, imsz;

    imsz = hd->Xdim * hd->Ydim;

    sky = (hd->ID.ZeroSky - SKYOFF)  / (hd->SkyTx - SKYOFF);

    /*
     * fprintf (stderr, "sky is %f\n", sky);
     */

    pf = im;
    for (ii = 0; ii < imsz; ii++)
	{
	*pf *= sky ;
	pf++;
        }
}

int
fixnegI_fancy ( Iim , xdim , ydim )

float *Iim; int xdim, ydim;
{
    int ii, jj;
    int lspot, hspot, spot;
    float *pf, ftemp, prev;
    double diff, ddiff, dlimit;

    lspot  =    32;
    hspot  =    95;
    hspot  =   120;
    dlimit = 40000.0;

    fprintf (stderr, "\t\tFixing negI the fancy way.\n");

    for (ii = 0; ii < xdim; ii++)
	{
	jj = hspot + 1;
	pf = Iim + xdim*jj + ii;
	prev  = *pf;
	for (jj = hspot; jj > lspot; jj--)
	    {
	    pf = Iim + xdim*jj + ii;
	    ftemp = *pf;
	    diff  = (double) (ftemp - prev);
	    ddiff = fabs (diff);
	    if (ddiff > dlimit ) *pf += 65536.0 ;
	    prev = *pf;
	    }
        }
}

/* -------------------------------------------------------------------------- */

int
fixneg ( AnImg, xdim, ydim, lspot, hspot)

    float *AnImg;
    int xdim, ydim;
    int lspot, hspot;
{
    int ii;
    int strt, stp;
    float *pf, ftemp;

    fprintf (stderr, "\t\tFixing negative data between %d %d\n", lspot, hspot);

    strt = (lspot  )   * xdim;
    stp  = (hspot + 1) * xdim;
    pf   = AnImg + strt;
    for (ii = strt; ii < stp; ii++)
	{
	ftemp = ( *pf < 0.0) ? *pf + 65536.0 : *pf;
	*pf = ftemp;
	pf++;
        }
}

/* -------------------------------------------------------------------------- */

int
fixnegWrap ( AnImg, xdim, ydim, lspot, hspot)

    float *AnImg;
    int xdim, ydim;
    int lspot, hspot;
{
    int ii, jj;
    int strt;
    float *pf, ftemp, wrap;

    fprintf (stderr, "\t\tWrap Fixing negative data between %d %d\n", 
	     lspot, hspot);

    strt = hspot * xdim;
    for(ii = 0; ii < xdim; ii++)
	{
	pf   = AnImg + strt + ii;
	wrap = 0.0;
	for (jj = hspot; jj >= lspot; jj--)
	    {
	    ftemp = *pf + wrap;
	    if ( ftemp < 0.0)
		{
		wrap  += 65536.0;
		ftemp += 65536.0;
	        }
	    *pf = ftemp;
	    pf -= xdim;
	    }
        }
}

/* -------------------------------------------------------------------------- */

int
fixnegQU ( Qim, Uim, xdim, ydim, HaveU, hd)

    float *Qim;
    float *Uim; 
    int xdim;
    int ydim;
    int HaveU;
    FH  hd;
{
    float *pf ,  ftemp;
    int testpix;
    int ii, jj;
    int spot , *dx;
    int doprint = 1;
    float neglim;

    fprintf (stderr, "\t\tFixing negQU\n");

    ii = (int) (hd->ID.hairspot [0]);	/* The low hairline position, ~103. */
    if (ii == 0)  ii = 103;		/* For when there were no hairlines */
    testpix  =  ii - hd->ID.PolarityCkPix ;
    neglim   =  0.0;


    dx = indx;
    for (ii = 0; ii < xdim; ii++) { *dx++ = -1 ; }

    /* Record the azimuths where in Qim 
     * the value at testpix is < 0
     */

    dx  = indx;
    pf  = Qim + testpix * xdim;
    for (ii = 0; ii < xdim; ii++)
	{
	ftemp = *pf;
	if ( ftemp < neglim ) { *dx = ii; dx++; }	/* Record the bad az. */
	pf++;
        }

    /* Fix the Qim at the "bad" azimuths. */

    dx  = indx;
    while ( *dx != -1 )
	{
	spot = *dx;

	if (doprint==1)
	    fprintf (stderr, "\t\tFixing spot %d\n", spot);

	pf = Qim + spot ;
	for (ii = 0; ii < ydim; ii++)
	    {
	    ftemp = *pf;
	    *pf   = -1.0 * ftemp;
	    pf   += xdim;
	    }
	dx++;
        }

    if ( HaveU == 1 )
	{			/* Fix the Uim at the "bad" azimuths. */
	dx  = indx;
	while ( *dx != -1 )
	    {
	    spot = *dx;

	    if (doprint >= 1)
		fprintf (stderr, "\t\tFixing spot %d\n", spot);

	    pf = Uim + spot ;
	    for (ii = 0; ii < ydim; ii++)
		{
		ftemp = *pf;
		*pf   = -1.0 * ftemp;
		pf   += xdim;
	        }
	    dx++;
	    }
        }
}

/* -------------------------------------------------------------------------- */

int
fixnegQU_cal0 ( Qim, Uim, xdim, ydim, HaveU )

    float *Qim;
    float *Uim; 
    int xdim;
    int ydim;
    int HaveU;
{
    float *pf, *paz, ftemp;
    int testpix1 = 300;
    int testpix2 = 330;
    int ii, jj;
    int spot , *dx;
    int doprint = 0;

    fprintf (stderr, "\t\tFixing negQU_cal0\n");


    dx = indx;
    for (ii = 0; ii < xdim; ii++) { *dx++ = -1 ; }

    /* Record the azimuths where in Qim 
     * the sum of values from testpix1 to testpix2 is > 0
     */

    dx  = indx;
    paz = Qim + testpix1 * xdim;
    for (ii = 0; ii < xdim; ii++)
	{
	pf    = paz;
	ftemp = 0.0;
	for (jj = testpix1; jj < testpix2; jj++)
	    {
	    ftemp += *pf;
	    pf    += xdim;                    /* Run through the heights */
	    }
	if ( ftemp > 0.0) { *dx = ii; dx++; } /* Record the bad one. */
	paz++;                                /* Step to the next az. */
        }

    /* Fix the Qim at the "bad" azimuths. */

    dx  = indx;
    while ( *dx != -1 )
	{
	spot = *dx;

	if (doprint == 1) fprintf (stderr, "\t\tFixing Q spot %d\n", spot);

	pf = Qim + spot ;
	for (ii = 0; ii < ydim; ii++)
	    {
	    ftemp = *pf;
	    *pf  = -1.0 * ftemp;
	    pf += xdim;
	    }
	dx++;
        }

    if ( HaveU == 1 )
	{			/* Fix the Uim at the "bad" azimuths. */
	dx  = indx;
	while ( *dx != -1 )
	    {
	    spot = *dx;

	    if (doprint == 1) fprintf (stderr, "\t\tFixing U spot %d\n", spot);

	    pf = Uim + spot ;
	    for (ii = 0; ii < ydim; ii++)
		{
		ftemp = *pf;
		*pf  = -1.0 * ftemp;
		pf += xdim;
	        }
	    dx++;
	    }
        }
}


/* -------------------------------------------------------------------------- */

int
UnGlytchBaaaa (fba, xdim)

float *fba; int xdim;
{
    float ewe, ram, lamb, wolf, bigbadwolf, stray, eaten;
    float *px , *py, *pf;
    double diff;
    int ii, ngood;

    /* First fix any 360 degree wraps. */

    eaten =  360.0 * deg2ba;
    stray =   90.0 * deg2ba;
    pf = fba;
    lamb = *pf - stray ;
    pf++;
    for (ii = 1; ii < xdim; ii++)
	{
	ewe = *pf;
	if ( (*pf) < lamb )
	    { 
	    *pf += eaten; 
	    fprintf (stderr, "%d %f to %f because %f\n", 
	             ii, ewe * ba2deg, *pf * ba2deg, lamb * ba2deg);
	    }
	lamb= *pf - stray;
	pf++;
        }

    /* Next cull any off looking values. */

    stray =   1.0 * deg2ba;
    pf    = fba;
    lamb  = *pf;
    px    = findx;   *px = 0.0;
    py    = fyval;   *py = *pf;		/* Hope the first one is good! */

    ngood = 1 ;   pf++;   px++;   py++;
    for (ii = 1; ii < xdim; ii++)
	{
	diff = (double) ( (*pf) - lamb );
	if ( fabs (diff) < deg2ba )
	    {
	    *px  = (float) ii;
	    *py  = *pf;
	    lamb = *pf ;
	    ngood++;
	    px++;
	    py++;
	    }
	else
	    {
	    fprintf (stderr, "\t\t    Culling ba %d %f (deg)\n",
		     ii, *pf * ba2deg);
	    }
	pf++;
        }

    do_lst (&ewe, &ram, &lamb, findx, fyval, ngood, 1);

    /* In do_lst, 
     * fbaaaa = ewe + ram * findx 
     * and      "lamb" is the "correlation coefficient".
     */

    wolf = 17.0 ;			/* 17 is about 0.34 deg. */
    bigbadwolf = -1.0 * wolf ;

    if ( lamb > 0.90 )
	{
	pf = fba;
	for (ii = 0; ii < xdim; ii++)
	    {
	    eaten = ewe + ram * (float) ii;
	    stray = eaten - *pf;
	    if ( (stray > wolf) || (stray < bigbadwolf) )
		{
		*pf = eaten;
		fprintf (stderr, "\t\tFixed ba-ewe %d\n", ii);
	        }
	    pf++;
	    } 
        }
    else
	{
	fprintf (stderr, "Looks like the barrel angles are totally hosed!\n");
        }
}

/* -------------------------------------------------------------------------- */

int
HandleNeg_pB ( im, xdim, ydim, lspot )

float *im;
int xdim, ydim;
int lspot;
{
    int ii, jj;
    int hspot, spot;
    float *pf, ftemp, prev;
    double diff, ddiff, dlimit;

    hspot  =   120;
    dlimit = 40000.0;

    fprintf (stderr, "\t\tFixing neg pB\n");

    for (ii = 0; ii < xdim; ii++)
	{
	jj = hspot + 1;
	pf = im + xdim * jj + ii;
	prev  = *pf;
	for (jj = hspot; jj > lspot; jj--)
	    {
	    pf = im + xdim * jj + ii;
	    ftemp = *pf;
	    diff  = (double) (ftemp - prev);
	    ddiff = fabs (diff);
	    if (ddiff > dlimit ) *pf += 65536.0;
	    prev = *pf;
	    }
        }
}

/* -------------------------------------------------------------------------- */

int
HandleNeg_pB_orig (im, xdim, ydim)

float *im;
int xdim, ydim;
{
    int ii, jj;
    int spot = 100;
    float *pf, ftemp;

    fprintf (stderr, "\t\tHandle neg pB the original way.\n");

    /* pB is unsigned below pixel spot */

    pf  = im ;
    for (ii = 0; ii < spot * xdim; ii++)
	{
	ftemp = ( *pf < 0.0 ) ? *pf + 65536.0 : *pf;
	*pf = ftemp;
	pf++;
        }
}

/* -------------------------------------------------------------------------- */

int
GetCorFac (hd)      

    FH hd;
{
    int  ii, ydim;
    char filenm [200];
    long corgain0 [NPX], corgain1 [NPX];
    long *pcg0, *pcg1;
    float *pcf, *povcf, cg, cg0, cg1;

    ydim = NPX;

    /* Open appropriate correct tables. */
    /* gain_[01].YYYYMMDDHHMM */

    WhereAmI ();
    sprintf  (filenm, "%s%s", CorrectorTablePath, 
	  		    hd->ID.CorrectFile0);
    read_long (filenm, corgain0, ydim, 0);

    sprintf (filenm, "%s%s", CorrectorTablePath, hd->ID.CorrectFile1);
    read_long (filenm, corgain1, ydim, 0);

    /* Average the two gains */

    pcg0  = corgain0;
    pcg1  = corgain1;
    pcf   = CorFac;
    povcf = ovCorFac;
    for (ii = 0; ii < ydim; ii++)
	{
	cg0    = (float) (*pcg0);
	cg1    = (float) (*pcg1);
	cg     = (cg0 + cg1) * 0.5;
	*pcf   = cg;
	*povcf = (cg != 0.0) ?  1.0 / cg : 1.0;
	/*
	 * fprintf (stderr, "ii %d %g %g\n", ii, *pcf, *povcf);
	 */
	pcg0++;
	pcg1++;
	pcf++;
	povcf++;
        }

}

/* -------------------------------------------------------------------------- */

int
GetBiasEstimate ( AnImg , xdim , ydim , lspot, hspot)

    float *AnImg;
    int xdim, ydim;
    int lspot, hspot;
{
    int ii, jj;
    int strt, stp;
    float *pf, *pb, *pv, ovNum, ftemp;


    fprintf(stderr, "\t\tGetting bias estimate between %d %d\n", lspot, hspot);

    ovNum = 1.0 / (1.0 + (float) (hspot-lspot)) ;

    pb   = BiasArray;  /* The Bias is an xdim array. */
    strt = lspot * xdim;
    for (ii = 0; ii < xdim; ii++)
	{
	pf   = AnImg + strt + ii;
	ftemp = 0.0;
	for (jj = lspot; jj <= hspot; jj++)
	    {
	    ftemp += *pf ;
	    pf    += xdim;
	    }
	*pb = ftemp * ovNum;
	pb++;
        }
}

/* -------------------------------------------------------------------------- */

int
DoKappaFix (Iimg, Qimg, Uimg, xdim, ydim, fkappa)

    float *Iimg;
    float *Qimg;
    float *Uimg;
    int   xdim, ydim;
    float fkappa;
{
    int ii, imsz;
    float  *pI, *pQ, *pU, myI, myQ, myU, fred;
    float  prevI;
    float  mysign;
    double pow (), dI, d1,  kappaM1, kappa;
    FILE *fid;
    int do_ck = 0;

    imsz = xdim * ydim;

    kappa   = (double) fkappa;
    kappaM1 = kappa - 1.0;
    prevI   = 5.e-5;

    if(do_ck == 1)
	{
	if ( ( fid = fopen ("i_pre_kappa", "w") ) < (FILE*)0 )
	    {
	    fprintf (stderr, "i_pre_kappa writing error %d\n", ii);
	    exit (-90);
	    }
	fwrite (Iimg, sizeof (float), imsz  , fid);
	fclose (fid);
        }

    pI = Iimg;
    pQ = Qimg;
    pU = Uimg;
    for (ii = 0; ii < imsz; ii++)
	{
	myI   = *pI;
	myQ   = *pQ;
	myU   = *pU;

	if ( myI <= 0.0)
	    {
	    myI = 1.0;
	    myQ = 0.0;
	    myU = 0.0;
	    }
	else
	    {

	    dI    = (double) myI;
	    d1    = pow (dI, kappaM1);
	    d1   *= kappa;
	    fred  = (float) d1;

	    myQ  *= fred;
	    myU  *= fred;

	    d1    = pow (dI, kappa);
	    myI   = (float) d1;
	    }

	*pI   = myI;
	*pQ   = myQ;
	*pU   = myU;

	pI++;
	pQ++;
	pU++;
        }

    if (do_ck == 1)
	{
	if ( ( fid = fopen ("i_post_kappa", "w") ) < (FILE*) 0 )
	    {
	    fprintf (stderr, "i_post_kappa writing error %d\n", ii);
	    exit (-90);
	    }
	fwrite (Iimg, sizeof (float), imsz, fid);
	fclose (fid);
        }

}

/* -------------------------------------------------------------------------- */

int
WhereAmI ()
{
    register FILE *pid;
    char *fgetl () ;
    char cmnd [120], line [120];
    int   ii;

    sprintf (cmnd, "uname -n");
    pid = popen (cmnd, "r+");
    sprintf (line, "") ;			/* Zero out the line. */
    fgetl (line, sizeof (line), pid);  /* See if the uname returned anything. */
		      
     /* ii == 0 if successful  ii != 0 error.*/

    ii = pclose (pid) ;

    if ( strcmp ("nahenahe", line ) == 0)
	{
	sprintf (CorrectorTablePath, "%s", CorrectorTablePathMLSO) ;
        }
    else
	{
	sprintf (CorrectorTablePath, "%s", CorrectorTablePathHAO) ;
        }

}

/* -------------------------------------------------------------------------- */
