/* acos_blint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : character pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * acos_blint returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float acos_blint ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, or floating-point images. 
 * The file 'cpsize.h' contains the variable 'nbypix' which specifies whether
 * the image is * a byte (nbypix=1), short integer (nbypix=2), or
 * floating-point (nbypix=4) * array. 
 * 'cpsize.h' also contains the variable 'npixl', which specifies the number
 * of pixels per horizontal line in the image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language   IRIX 5.2 operating system   SGI 4D/25 G
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 27 February 1984
 * 6 November 1986: add capability of handling floating point images.
 * 10 October 1995: use unsigned char for nbypix=1, unsigned short for nbypix=2.
 *                  add nbypx argument & delete nbypix reference.
 *                  add dx, dy arguments & delete npixl reference.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "libacos.h"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
acos_blint (float *imgbuf, int nbypx, int dx, int dy, float *x, float *y){
register unsigned char	*imc;
register unsigned short	*ims;
register float	*imf;
int	ix, iy;
float	fx, fy;
float	f00, f10, f01, f11;
float	fx0, fx1, fxy;

   imc = (unsigned char *) imgbuf;
   ims = (unsigned short *) imgbuf;
   imf = (float *) imgbuf;

   ix = (int) *x;
   iy = (int) *y;

   fx = *x - (float) ix;
   fy = *y - (float) iy;

   /*** Select 4 nearest neighbor elements in 'imgbuf' array. ***/

   /*** 1 byte per pixel. ***/

   if (nbypx == 1)
      {
      f00 = (float) (*(imc +  ix    +  iy*dx) & 0377);
      f10 = (float) (*(imc + (ix+1) +  iy*dx) & 0377);
      f01 = (float) (*(imc +  ix    + (iy+1)*dx) & 0377);
      f11 = (float) (*(imc + (ix+1) + (iy+1)*dx) & 0377);
      }
   else

   /*** 2 bytes per pixel (short integer). ***/

   if (nbypx == 2)
      {
      f00 = (float) *(ims +  ix    +  iy*dx);
      f10 = (float) *(ims + (ix+1) +  iy*dx);
      f01 = (float) *(ims +  ix    + (iy+1)*dx);
      f11 = (float) *(ims + (ix+1) + (iy+1)*dx);
      }
   else

   /*** 4 bytes per pixel (floating point). ***/

   if (nbypx == 4)
      {
      f00 = (float) *(imf +  ix    +  iy*dx);
      f10 = (float) *(imf + (ix+1) +  iy*dx);
      f01 = (float) *(imf +  ix    + (iy+1)*dx);
      f11 = (float) *(imf + (ix+1) + (iy+1)*dx);
      }

   /*** Bilinear interpolation. ***/

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}
