/*frotimg.c
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Rotate a floating-point image.
* Use bilinear interpolation.
*
* syntax:
*    float *frotimg () ;
*    float imginp [ydim][xdim] ;
*    imgout = frotimg (imginp, xdim, ydim, xcen, ycen, angdeg, oval) ;
*
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Andrew Stanger   HAO/NCAR   28 June 2000   rotate_img.c
* 11 Apr 2002 : Modify to only work with floating-point images.
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*/
#include<stdio.h>
#include<stdlib.h>
#include<math.h>

/*-------------------------------------------------------------------------- */

float *
frotimg (imginp, xdim, ydim, xcen, ycen, angdeg, oval) 

  unsigned char *imginp ;	/* Input image array pointer. */
  int nbypix ;			/* # bytes per pixel. */
  int xdim, ydim ;		/* Dimensions of image. */
  float xcen, ycen ;		/* Center of rotation. */
  float  angdeg ;		/* Rotation angle in degrees (+ = CCW). */
  float oval ;			/* Value for magnitude of missing data. */

{
   extern void  *malloc () ;
   extern float blintf () ;

   char *parg ;

   float *imgrot ;

   int ix, iy ;
   int nbypix = 4 ;

   float bval ;
   float fox, foy ;
   float cosrot, sinrot ;
   float fx, fy ;
   float fysin, fycos ;
   float *finp, *fout ;
   float fval ;

   double angrad ;		/* Rotation angle in radians. */
   double raddeg ;		/* radians/degree. */

   FILE *angfd ;

   /*** If center coordinates are negative,
    *** set image center = array center.
    ***/

   if (xcen < 0.0)  xcen = xdim / 2.0 - 0.5 ;
   if (ycen < 0.0)  ycen = ydim / 2.0 - 0.5 ;

   fprintf (stderr, "frotimg> xdim/ydim:%3d %3d  xcen/ycen:%7.2f %7.2f\n",
	    xdim, ydim, xcen, ycen) ;

   /*** Compute constants. ***/
   /*** Need to reverse the sign of angdeg for proper rotation direction. ***/

   raddeg = acos (-1.0) / 180.0 ;		/* Radians / degree. */
   angrad = (double) -angdeg * raddeg ;		/* Convert degrees to radians.*/
   sinrot = (float) sin ( angrad ) ;
   cosrot = (float) cos ( angrad ) ;

   fprintf (stderr,
	    "angdeg: %6.2f  angrad: %7.4f  sinrot: %8.4f  cosrot: %8.4f\n",
            angdeg, angrad, sinrot, cosrot) ;

   /*** Allocate storage for both input & output arrays. ***/

   imgrot = (float *) malloc ( xdim * ydim * sizeof (float) ) ;

   /*** Rotate image. ***/

   finp = (float *) imginp ;
   fout = (float *) imgrot ;

   for ( iy = 0 ; iy < ydim ; iy++ )
      {
      fy = (float) (iy) - ycen ;	/* y-coordinate relative to center. */
      fysin = fy * sinrot - xcen ;
      fycos = fy * cosrot + ycen ;

      for ( ix = 0 ; ix < xdim ; ix++)
         {
         fx = (float) (ix) - xcen ;	/* x-coordinate relative to center. */
         fox = fx * cosrot - fysin ;	/* rotated x coordinate. */
         foy = fycos + fx * sinrot ;	/* rotated y coordinate. */

         /*** Use bilinear interpolation to compute pixel intensity. ***/

         if ((fox < 0 || fox > xdim-1) || (foy < 0 || foy > ydim-1))
	    bval = oval ;		/* magnitude for missing data. */
	 else
	    bval = (float) fblint (imginp, xdim, ydim, fox, foy) ;

	 fval = bval ;

         *fout++ = fval ;
         }
      }

   return (imgrot) ;
}

/*-------------------------------------------------------------------------- */

