/* picsum2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute the sum of all of the pixels in a PICS image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 December 1998
 * Limit integration to 50x50 pixel box centered on the solar disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/fitslib.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern char *rdfits () ;
   float imgsum ;
   short *img ;
   int ix, iy ;
   int ixcen, iycen ;
   int nxbeg, nxend ;
   int nybeg, nyend ;
   int nx, ny ;
   int nbypix ;
   int pixcnt ;
   static char namimg [80] ;
   static char namlst [80] ;
   FILE *FD ;

   if (argc < 2)
      {
      fprintf (stdout, "usage: picsum fits_image_list\n") ;
      exit (0) ;
      }
   else
      strcpy (namlst, argv [1]) ;

   FD = fopen (namlst, "r") ;

   while (fscanf (FD, "%s", namimg) != EOF)
      {

      /*** Read FITS image. ***/

      img = (short *) rdfits (namimg) ;

      nx = naxis1 ;
      ny = naxis2 ;
      nbypix = bitpix / 8 ;
      ixcen = (int) crpix1 ;
      iycen = (int) crpix2 ;
      nxbeg = ixcen - 25 ;
      nxend = nxbeg + 50 ;
      nybeg = iycen - 25 ;
      nyend = nybeg + 50 ;

      imgsum = 0.0 ;
      pixcnt = 0 ;
      for (iy = nybeg ; iy < nyend ; iy++) /* skip 1st line [binary header]. */
         for (ix = nxbeg ; ix < nxend ; ix++)
            {
            imgsum += *(img + ix + iy * nx) ;
	    pixcnt += 1 ;
            }

         fprintf (stdout, "%s   %10.4e   %8d\n", namimg, imgsum, pixcnt) ;
      free (img) ;
      }

   fclose (FD) ;
   exit (0) ;
}
