/* glimg_view.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display an image ('filename') in SGI IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 24 May 1991: use onemap color mode.
 * 12 Dec 1991: add hdrsiz argument for header to skip at BOF.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include "/home/stanger/lib/include/gl_color.h"


/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

glimg_view (filename, hdrsiz, nbypx, nx, ny, dir2, mapoff)

char *filename;		/* Pointer to filename array. */
int hdrsiz;		/* # bytes to skip at BOF. */
int nbypx;		/* # bytes per pixel. */
int nx;			/* X dimension. */
int ny;			/* Y dimension. */
char dir2 [];		/* Scan direction: rd or ru. */
int mapoff;		/* Color map offset (usually 512: color map #2). */

{
   extern char *malloc ();
   float pbfac = 2.5 ;
   int fd;
   int i, ix, iy, iiy;
   unsigned int imgsiz;
   int nelem;
   int nread;
   int ul = 0 ;			/* Upper left origin ? =0 no, =1 yes. */

   unsigned short *imgbuf;
   unsigned char *cbuf;
   short *sbuf;
   float *fbuf;

   float fval ;
   float fmin, fmax ;
   float slope, offset ;

   if (strcmp (dir2, "rd") == 0)
      ul = 1 ;
   else
   if (strcmp (dir2, "ru") == 0)
      ul = 0 ;
   else
      {
      fprintf (stderr, "Unsupported scan direction (%s); must be ru or rd.\n",
	       dir2) ;
      return (-1) ;
      }


   /*** Open image file. ***/

   if ((fd = open (filename, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open file %s ... fd: %d\n", filename, fd);
      exit (-1);
      }
   lseek (fd, hdrsiz, 0);

   /*** Read image into memory. ***/
   /*** Convert to short integer if input image is either byte or float. ***/
   /*** Flip image top-to-bottom for rectwrite. ***/

   nelem = nx * ny;
   imgsiz = nelem * nbypx;
   cbuf   = (unsigned char *)  malloc (imgsiz);
   imgbuf = (unsigned short *) malloc (nelem * sizeof (short));

   if (nbypx == 1)
      {
      if ((nread = read (fd, cbuf, imgsiz)) != imgsiz)
         {
         fprintf (stderr, "nread: %d  != imgsiz: %d\n", nread, imgsiz);
         return (-2);
         }

      for (iy = 0; iy < ny; iy++)
	 {
	 if (ul)
	    iiy = ny - 1 - iy ;
         else
	    iiy = iy ;
         for (ix = 0; ix < nx; ix++)
            {
            *(imgbuf + ix + iiy*nx) =
	       (unsigned short) *(cbuf + ix + iy*nx) + mapoff ;
            }
         }
      }

   if (nbypx == 2)
      {
      sbuf = (short *) cbuf ;
      if (read (fd, cbuf, imgsiz) != imgsiz)
         {
         fprintf (stderr, "nread: %d  != imgsiz: %d\n", nread, imgsiz);
         return (-3);
         }

      for (iy = 0; iy < ny; iy++)
	 {
	 if (ul)
	    iiy = ny - 1 - iy ;
         else
	    iiy = iy ;
         for (ix = 0; ix < nx; ix++)
            {
            *(imgbuf + ix + iiy*nx) =
	       (unsigned short) *(sbuf + ix + iy*nx) / 4 + mapoff ;
            }
         }
      }

   if (nbypx == 4)
      {
      fmin =  1.e+30 ;
      fmax = -1.e+30 ;
      fbuf = (float *) cbuf ;
      if (read (fd, cbuf, imgsiz) != imgsiz)
         {
         fprintf (stderr, "nread: %d  != imgsiz: %d\n", nread, imgsiz);
         return (-4);
         }

      slope = 250.0 / pbfac ;
      offset = 0.0 ;

      for (iy = 0; iy < ny; iy++)
	 {
	 if (ul)
	    iiy = ny - 1 - iy ;
         else
	    iiy = iy ;

         for (ix = 0; ix < nx; ix++)
            {
	    fval = *(fbuf + ix + iiy*nx) ;
	    if (fval < fmin)  fmin = fval ;
	    if (fval > fmax)  fmax = fval ;
	    if (fval > 0.0)
	       fval = (fval -  offset) * slope ;
            else
	       fval = 0.0 ;
            *(imgbuf + ix + iiy*nx) = (unsigned short) fval + mapoff ;
            }
         }
      }
   fprintf (stdout, "fmin: %8.2e  fmax: %8.2e\n", fmin, fmax) ;

   /*** Write image to GL window. ***/
   
   rectwrite (0, 0, nx-1, ny-1, imgbuf);

   free (cbuf);
   free (imgbuf);
   close (fd);
   return (0);
}
