;+
; pmfits.pro   IDL procedure
;
; PURPOSE
;	Read a raw MLSO Prominence Monitor image (digitized from film)
;	and convert it to a file in FITS format.
;
; SYNTAX
;	pmfits, 'pm.img'
;
; PROCEDURES USED
;	fxhmake
;	fxaddpar
;	writefits
;
; HISTORY
;	Author: Andrew L. Stanger   1 October 1997
;-

pro pmfits, pmon_list

pmon_name = ''				; Define PM file name variable.
imgbuf = BYTARR (680, 900)		; Define image array.

CLOSE, 1
CLOSE, 2

OPENR, 1, pmon_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, pmon_name		; Read PM file name from list file.
      OPENR, 2, pmon_name		; Open PM raw image file.
      READU, 2, imgbuf			; Read PM pixels into buffer.
      CLOSE, 2				; Close PM raw image file.

      ; -- Invert intensity values (since raw image is a "negative".

      imgbuf (*,*) = 255 - imgbuf (*,*)
      dmin = float (MIN (imgbuf))
      dmax = float (MAX (imgbuf))
      print, 'dmin/dmax: ', dmin, dmax
      print, 'sizeof (dmin): ', sizeof (dmin)

      ; -- Create FITS header.

      fxhmake, hdu, imgbuf		; Make simple FITS header for PM image.
      fxaddpar, hdu, 'ORIGIN',   'MLSO',   'MAUNA LOA SOLAR OBSERVATORY'
      fxaddpar, hdu, 'TELESCOP', 'PMON',   'Prominence Monitor'
      fxaddpar, hdu, 'INSTRUME', 'FILM',   'B/W film camera'
      fxaddpar, hdu, 'OBJECT',   'SUN'
      fxaddpar, hdu, 'TYPE-OBS', 'H-ALPHA'
      date = '89/02/17'				; DOY 48 1989
      hour   = strmid (pmon_name, 0, 2)
      minute = strmid (pmon_name, 2, 2)
      second = strmid (pmon_name, 4, 2)
      time = hour + ':' + minute + ':' + second
      fxaddpar, hdu, 'DATE-OBS',  date,    'YY/MM/DD'
      fxaddpar, hdu, 'TIME-OBS',  time,    'HH:MM:SS'
      fxaddpar, hdu, 'DATAMIN',  dmin,  'MINIMUM INTENSITY OF DATA'
      fxaddpar, hdu, 'DATAMAX',  dmax,  'MAXIMUM INTENSITY OF DATA'
      fxaddpar, hdu, 'DISPMIN',  dmin,  'MINIMUM INTENSITY FOR DISPLAY'
      fxaddpar, hdu, 'DISPMAX',  dmax,  'MAXIMUM INTENSITY FOR DISPLAY'

      ; -- Write FITS image to disk.

      pmon_fits = pmon_name + '.fts'	; Create output file name.
      writefits, pmon_fits, imgbuf, hdu

   END
CLOSE, 1

END
