/* pmhdr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read byte image file, pre-pend header, write to disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

int          Nyear , Ndoy , Nhrs , Nmins , Nsecs ,
	     Nxdim , Nydim , Nxcenter , Nycenter ,
             Nhdrlen ;

float        Noffang , Nradius ;

char         *Nheader;


int          year , doy , hrs , mins , secs ,
	     xdim , ydim , xcenter , ycenter ,
             hdrlen ;

float        offang , radius ;          

char         *header ;

int  Y , D , h , m , s , r , o, c ;
unsigned char *imgbuf;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
main (argc, argv)

int argc;
char *argv [];
{

   char *p;				/* pointer to command line arguments */
   char filename [80], efilenm [80] ;
   char ans [2] ;
   int ii , n ;
   long lsize , ftell () ;
   FILE *fid , *fopen () ;

   Y=0 ; D=0 ; h=0 ; m=0 ; s=0 ; r=0 ; o=0 ;

   ii = argc ;
   if (argc == 2)
      {
      strcpy (filename, argv [1]);
      fprintf (stderr, "Enter x/y dimensions [xdim,ydim]: ");
      fscanf  (stdin,  "%d,%d", &xdim, &ydim);
      }
   else
   if (argc == 4)
      {
      strcpy (filename, argv [1]);
      sscanf (argv [2], "%d", &xdim);
      sscanf (argv [3], "%d", &ydim);
      }
   else
      {
      fprintf (stderr, "usage: %s filename [xdim ydim]\n", argv [0]);
      exit (0);
      }

   /*** Initialize variables. ***/

   xcenter = xdim / 2;
   ycenter = ydim / 2;
   offang = 0.0;
   radius = 230.0;
   imgbuf = (unsigned char *)  malloc (xdim*ydim);
   hdrlen  = xdim;
   header = (char *)  malloc (hdrlen);
   Nhdrlen = xdim;
   Nheader = (char *) malloc (Nhdrlen);

   fprintf (stderr, "xdim: %d   ydim: %d\n", xdim, ydim);
   fprintf (stderr, "xcenter: %d  ycenter: %d  offang: %6.2f  radius: %6.2f\n",
           xcenter, ycenter, offang, radius);

   /*** Open input file. ***/

   if ((fid = fopen (filename, "r")) == NULL)
      {
      fprintf (stderr, "can't open %s\n", filename) ; exit(-1) ;
      }
   rewind (fid) ;

   /*** Read input image into memory. ***/

   if ( ( n = fread ( imgbuf, 1, xdim*ydim, fid )) != xdim*ydim)
      {
      fprintf (stderr, "Error reading image of file %s\n",filename) ;
      exit (-1) ;
      }
   fclose (fid) ;

   /*** Open output file. ***/

   sprintf (efilenm, "%s.eikp", filename);
   if ((fid = fopen (efilenm, "w")) == NULL)
      {
      fprintf (stderr, "can't open %s\n", efilenm) ;
      exit (-2) ;
      }
   rewind (fid) ;

   /*** Get header parameters. ***/

   if ( argc < 8 )
      {
      prhelp () ;
      fprintf (stderr, "--> "); scanf ("%s", ans) ;
      while ( *ans != 'R' )
         {
         switch ( *ans )
            {
            case 'Q':  exit (0) ;
	    case 'H':  prhelp () ; break ;
	    case 'Y':  Y++ ; scanf ("%d", &Nyear);  break;
	    case 'D':  D++ ; scanf ("%d", &Ndoy);  break;
	    case 'h':  h++ ; scanf ("%d", &Nhrs);  break;
	    case 'm':  m++ ; scanf ("%d", &Nmins);  break;
	    case 's':  s++ ; scanf ("%d", &Nsecs);  break;
	    case 'r':  r++ ; scanf ("%f", &Nradius); break;
	    case 'o':  o++ ; scanf ("%f", &Noffang); break;
	    case 'c':  c++ ; scanf ("%d %d", &Nxcenter, &Nycenter); break;
	    default:   fprintf (stderr, "Unknown request %s\n", *ans);
                       prhelp ();
                       break ;
	    }
	 fprintf (stderr,"--> "); scanf ("%s", ans) ;
         }
    }

   rewind (fid) ;

   /*** Write output image. ***/

   rewritehead () ;
   fwrite (header, 1, xdim, fid) ;
   fwrite (imgbuf, 1, xdim*ydim, fid);
   rewind (fid) ;  
   fclose (fid) ;

   exit (0);
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
rewritehead () 	 /* Subroutine to write header string */
{
   if ( Y )    year = Nyear ;
   if ( D )    doy  = Ndoy ;
   if ( h )    hrs  = Nhrs ;
   if ( m )    mins = Nmins ;
   if ( s )    secs = Nsecs ;
   if ( r )    radius = Nradius ;
   if ( o )    offang = Noffang ;
   if ( c )    { xcenter = Nxcenter ;  ycenter = Nycenter; }

   sprintf (header," Header length %3d", hdrlen);
   sprintf (header + 18, " Year %2d", year);
   sprintf (header + 26, " DOY %3d", doy);
   sprintf (header + 34, " Hr %2d", hrs);
   sprintf (header + 40, " Min %2d", mins);
   sprintf (header + 47, " Sec %2d", secs);
   sprintf (header + 54, " xdim %3d", xdim);
   sprintf (header + 63, " ydim %3d", ydim);
   sprintf (header + 72, " xcenter %3d", xcenter);
   sprintf (header + 84, " ycenter %3d", ycenter);
   sprintf (header + 96, " radius %4.1f", radius);
   sprintf (header + 109, " offang %5.2f", offang) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
prsum ()

{
static char summary [] = "\
\n\
pmhdr \n\
\n\
Example:             pmhdr\n\
\n\
Flags		Description\n\
-----		----------------------------------------------------\n\
[-Y yr]        the year should be changed to \"yr\" \n\
[-D DOY]       the Day of Year should be changed to \"DOY\"  \n\
[-h hr]        the hour should be changed to \"hr\"  \n\
[-m min]       the minutes should be changed to \"min\"  \n\
[-s sec]       the seconds should be changed to \"sec\"  \n\
[-r rad]       the radius should be changed to \"rad\"  \n\
[-o offang]    the offset angle should be changed to \"offang\"  \n\
[-c xcen ycen] the x & y center coordinates should be changed to \"xcen ycen\"  \n\
\n\
\n" ;

   fputs (summary, stderr);
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
prhelp ()

{
static char help [] = "\
\n\
Enter in all of the following to putin the header. \n\
    R     when you are ready to save the header.\n\
    Q     to quit this program gracefully, saving NO changes.\n\
    H     to get this help message.\n\
    Y yr         the year should be changed to \"yr\" \n\
    D DOY        the Day of Year should be changed to \"DOY\"  \n\
    h hr         the hour should be changed to \"hr\"  \n\
    m min        the minutes should be changed to \"min\"  \n\
    s sec        the seconds should be changed to \"sec\"  \n\
    r rad        the radius should be changed to \"rad\"  \n\
    o offang     the offset angle should be changed to \"offang\"  \n\
    c xcen ycen  the x & y center coordinates should be changed to \"xcen ycen\" \n\
\n";

   fputs (help, stderr);
}
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -  */
