
#include <stdio.h>
#include "fitsio.h"
#include "libacos.h"

main (int argc, char *argv[]){

  fitsfile *diffptr, *darkptr, *outfptr;
  int status = 0;
  int anynull = -1, nullval, i;
  int bitpix = -32;
  long fpixel = 1, nelements;
  long xsz, ysz;
  long naxes[2] = {0,0}, naxis = 2;
  float *diffuser, *dark;
  short *sdiff;
  float brtpct = 0.05, brtavg, crradius;
  char datestr[11];
  int year, month, day;
  float avg, adev, sdev, var, skew, curt;

  char *diffname = "", *sfitname = "", *darkname = "", *outname = "";

  if (argc != 4){
    printf("usage:  mkpicsflat diffuser dark outputname\n");
    exit(1);
  }

  diffname = argv[1];
  darkname = argv[2];
  outname  = argv[3];

  fits_open_file(&diffptr, diffname, READONLY, &status);
  fits_open_file(&darkptr, darkname, READONLY, &status);
  
  /* Get today's date */
  fits_get_system_date(&day, &month, &year, &status);
  fits_date2str(year, month, day, datestr, &status);

  fits_read_key(diffptr, TINT, "NAXIS1", &xsz, NULL, &status);
  fits_read_key(diffptr, TINT, "NAXIS2", &ysz, NULL, &status);
  fits_read_key(diffptr, TFLOAT, "CRRADIUS", &crradius, NULL, &status);
  naxes[0] = xsz;
  naxes[1] = ysz;

  if (status) {
    fits_report_error(stderr, status); /* print error message */
    return(status);
  }

  nelements = xsz * ysz;
  sdiff    = (short *) malloc (nelements * sizeof (short));
  diffuser = (float *) malloc (nelements * sizeof (float));
  dark     = (float *) malloc (nelements * sizeof (float));

  fits_read_img(diffptr, TFLOAT, fpixel, nelements, &nullval, diffuser, &anynull, &status);
  fits_read_img(darkptr, TFLOAT, fpixel, nelements, &nullval, dark, &anynull, &status);

  fits_close_file(diffptr, &status);
  fits_close_file(darkptr, &status);

  /* Clean up raw image of hot pixels and negative values
   *  due to wraparound of hot pixels
   */

  acos_moment(diffuser, nelements, &avg, &adev, &sdev, &var, &skew, &curt);
  printf("data mean:  %0.2f\n",avg);
  printf("data sigma:  %0.2f\n",sdev);

  for (i = 0; i < nelements; i++){
    if (diffuser[i] < 0) diffuser[i] = 0;
  }

  /* Subtract the dark from the diffuser first */

  /* printf ("Subtracting dark...\n"); */
  for (i = 0; i < nelements; i++){
    diffuser[i] -= dark[i];
    if (diffuser[i] < 0) diffuser[i] = 1;
  }

  /* Normalize the diffuser to the brighest brtpct of the diffuser */

  for (i = 0; i < nelements; i++)
    sdiff[i] = (short)diffuser[i];

  brtavg = acos_brtpix(sdiff, crradius, xsz, ysz, 767, 515, brtpct);
  printf ("brtavg:  %f\n",brtavg);
  for (i = 0; i < nelements; i++){
    diffuser[i] /= brtavg;
  }

  acos_moment(diffuser, nelements, &avg, &adev, &sdev, &var, &skew, &curt);
  for (i = 0; i < nelements; i++){
    if (diffuser[i] > 4 * sdev) diffuser[i] = 1;
  }

  /* Get rid of outfile if it already exists */
  acos_remove_file(outname);
  
  fits_create_template(&outfptr,outname,diffname,&status);
  fits_update_key(outfptr, TINT, "BITPIX", &bitpix, "4 BYTE Float", &status); 
  fits_set_hdustruc(outfptr, &status);

  fits_update_key(outfptr, TSTRING, "DATE-CAL", &datestr, NULL, &status); 
  fits_update_key(outfptr, TSTRING, "CODE-OBS", "AFC",NULL, &status); 

  /* Write the corrected diffuser image to disk */
  fits_write_img(outfptr, TFLOAT, fpixel, nelements, diffuser, &status);
  
  fits_close_file(outfptr, &status);

}
