#include <stdio.h>
#include "fitsio.h"
#include "libacos.h"

main (int argc, char *argv[]){

  fitsfile *rawptr, *outfptr;
  int status = 0;
  int anynull = -1, nullval, i;
  int bitpix, ix, iy;
  long fpixel = 1, nelements;
  long xsz, ysz;
  long naxes[2] = {0,0}, naxis = 2;
  float *rawimg;
  short *calimg;
  float brtpct = 0.05, brtavg, crradius;
  char datestr[11];
  int year, month, day;
  float xcen, ycen, xdist, ydist, radius;
  float avg, adev, sdev, var, skew, curt;

  char *rawname = "", *outname = "";

  if (argc != 5){
    printf("usage:  picscal_nodiffs raw_image outputname\n");
    exit(1);
  }

  rawname  = argv[1];
  outname  = argv[2];

  fits_open_file(&rawptr, rawname, READONLY, &status);
  
  /* Get today's date */
  fits_get_system_date(&day, &month, &year, &status);
  fits_date2str(year, month, day, datestr, &status);

  fits_read_key(rawptr, TINT, "NAXIS1", &xsz, NULL, &status);
  fits_read_key(rawptr, TINT, "NAXIS2", &ysz, NULL, &status);
  fits_read_key(rawptr, TINT, "BITPIX", &bitpix, NULL, &status);
  fits_read_key(rawptr, TFLOAT, "CRRADIUS", &crradius, NULL, &status);
  fits_read_key(rawptr, TFLOAT, "CRPIX1",&xcen, NULL, &status);
  fits_read_key(rawptr, TFLOAT, "CRPIX2",&ycen, NULL, &status);

  naxes[0] = xsz;
  naxes[1] = ysz;

  if (xcen < 0) xcen = xsz / 2.0;
  if (ycen < 0) ycen = ysz / 2.0;

  if (status) {
    fits_report_error(stderr, status); /* print error message */
    return(status);
  }

  nelements = xsz * ysz;
  rawimg   = (float *) malloc (nelements * sizeof (float));
  calimg   = (short *) malloc (nelements * sizeof (short));

  fits_read_img(rawptr, TFLOAT, fpixel, nelements, &nullval, rawimg, &anynull, &status);

  fits_close_file(rawptr, &status);

  /* Clean up raw image of hot pixels and negative values
   *  due to wraparound of hot pixels
   */

  acos_moment(rawimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);
  for (i = 0; i < nelements; i++){
    if (rawimg[i] < 0) rawimg[i] = 0;
    if (rawimg[i] > 7 * sdev) rawimg[i] = 0;
  }

  acos_moment(rawimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);

  for (i = 0; i < nelements; i++)
    calimg[i] = (short) rawimg[i];

  /* Get rid of outfile if it already exists */
  /* acos_remove_file(outname); */

  fits_create_template(&outfptr,outname,rawname,&status);
  fits_update_key(outfptr, TINT, "BITPIX", &bitpix, "4 BYTE Float", &status); 
  fits_set_hdustruc(outfptr, &status);

  fits_update_key(outfptr, TSTRING, "DATE-CAL", &datestr, NULL, &status); 
  fits_update_key(outfptr, TSTRING, "CODE-OBS", "AFC",NULL, &status); 

  /* Write the corrected diffuser image to disk */
  fits_write_img(outfptr, TSHORT, fpixel, nelements, calimg, &status);
  
  fits_close_file(outfptr, &status);

}
