#!/opt/local/bin/perl

use strict;
use Time::JulianDay;
use mlso_navbar;
use mlso;

my $movietxt = "(Not implemented yet.  Need to copy files from /toshi/d5)";
my $images = "http://download.hao.ucar.edu/d5/www/images";
my $styles = "http://download.hao.ucar.edu/d5/www/styles";
my $rootdir = $ENV{DOCUMENT_ROOT};
my $query = $ENV{QUERY_STRING};
my @query = split(/&/,$query);
my $year  = $query[0];
my $month = $query[1];
my $day   = $query[2];
my $date_str = "$year-$month-$day";
my $inst  = $query[3];
my $logdir = "http://download.hao.ucar.edu/d5/mlso/log/observer/$year";
my $getdata_cmnd="mlso_getdata.cgi";
my @filestats = stat "mlso_datasum.cgi";
my @gm_arr = localtime($filestats[9]);
my $create_str = sprintf("%02d/%02d/%02d %02d:%02d:%02d",
        $gm_arr[4]+1,$gm_arr[3],$gm_arr[5]-100,$gm_arr[2],$gm_arr[1],$gm_arr[0]);

my ($doy,$yrddoy,$processed, $logpath);
my %instruments = ('CHIP',"chp",'PICS',"dpm",'MKIV',"mk4","ACOS",'mk4',"MKIII",'mk3');
my %logs = ('CHIP',"chiasLogs",'PICS',"pciasLogs",'MKIV',"midx","MKIII","midx");

my @months =("January","February","March","April","May","June","July",
		"August","September","October","November","December");
my %months = ("Jan",1,"Feb",2,"Mar",3,"Apr",4,"May",5,"Jun",6,
              "Jul",7,"Aug",8,"Sep",9,"Oct",10,"Nov",11,"Dec",12);

sub get_timestr;
sub get_dutycycle;
sub where;

my $a = mlso->getnumimgs(Year => $year, Month => $month, Day => $day);
my $numchipint = $a->{numchipint};
my $numchipvel = $a->{numchipvel};
my $numdisk = $a->{numlimb};
my $numasc = $a->{numasc};
my $numlimb = $a->{numlimb};
my $numcpb = $a->{numcpb};
my $numrpb = $a->{numrpb};
my $numvig = $a->{numvig};

print <<END_OF_PAGE;
Content-type: text/html

<html>
<head>
<title>Data Summary of $inst data</title>

<link rel="stylesheet" type="text/css"
href="$styles/mlsohome.css" />

END_OF_PAGE

PrintJavaScript();
PrintACOSbar("mlso_banner3.sml.gif");

# Monthly summary page output
if ($day =~ /Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec/){
  my $month = $day;
  print "<h1 class=\"datatitle\">Monthly Summary of $inst data for $months[where(\@months,$day)], $year</h1>";

# Yearly summary page output
} elsif ($day > 1000){
  print "<h1 class=\"datatitle\">Yearly Summary of $inst data for $year</h1>";

# Daily summary page output
} else {
  print "<h1 class=\"datatitle\">Daily Summary of $inst data for $months[$month-1] $day, $year</h1>";

  PrintDataTable($inst,$year,$month,$day,$a);

}

print <<END_OF_PAGE;

<div align="center">
<img src='$images/ncartext.gif'>
</div>

<address>
<p>Webpage written and maintained by: <br>
<a href="mailto:tdarnell\@ucar.edu">Tony Darnell</a> <br>
Last modified:  $create_str
</p>
</address>

</body> 
</html>

END_OF_PAGE

sub get_timestr{
  my $seconds = shift;

  my $secondsinaday = 3600*24;
  my $days     = int $seconds/$secondsinaday;
  my $hours    = int (($seconds - ($days * $secondsinaday))/3600);
  my $minutes  = int (($seconds - ($days * $secondsinaday) - ($hours*3600))/60);
  my $seconds  = $seconds - ($days * $secondsinaday) - ($hours*3600) - ($minutes*60);
  #my $seconds  = $seconds % 60;

  return sprintf("%d Days %d Hours %d Minutes %d Seconds",$days,$hours,$minutes,$seconds);

}

#
#  Modeled after the IDL where function
#
sub where {
  my($arrayRef, $target) = @_;
  my(@array)             = @{$arrayRef};
  my($numElements)       = scalar(@array);
  my(@indexes)           = (0..$numElements);
  my(@elements);

  @elements = grep @{$arrayRef}[$_] =~ /$target/, @indexes;
  return(wantarray ? @elements : $elements[0]);
} 

sub get_dutycycle{

  my @logfiles = @_;
  my $logdir = "/toshi/ftp/d5/mlso/log/observer/$year";
  my $nummlsodays = @logfiles;
  my $numdays   = 0;
  my $elapsed_time = 0;

  foreach my $f (@logfiles){
    open(LOG, "< $logdir/$f");
    my @log = <LOG>;
    close(LOG);

    my @startimes = grep(/Start Patrol|Start 7 Passband Patrol/,@log);
    my @endtimes  = grep(/End Patrol/,@log);
    my (@startlines, @endlines,$numstarttimes,$numendtimes);

    if ($inst eq "ACOS"){
      my @chipstart = grep(/CHIP/,@startimes);
      my $numchiptimes = @chipstart;
      my @picsstart = grep(/PICS/,@startimes);
      my $numpicstimes = @picsstart;
      my @mk4start  = grep(/MKIV/,@startimes);
      my $nummk4times = @mk4start;
      my @chipend   = grep(/CHIP/,@endtimes);
      my @picsend   = grep(/PICS/,@endtimes);
      my @mk4end    = grep(/MKIV/,@endtimes);
      #
      #  In order to calculate the duty cycle for MLSO,
      #	 I'm just going to use the first MK4 time and the
      #  last PICS time.  This seems representative of when
      #  the observatory was open.
      if (@mk4start[0]){
        @startlines = @mk4start[0];
      } elsif (@picsstart[0]){
	@startlines = @picsstart[0];
      } elsif (@chipstart[0]){
	@startlines = @chipstart[0];
      } else {
	next;
      }
      @endlines   = @picsend [$#picsend];
      $numstarttimes = @startlines;
      $numdays = $nummlsodays;
    } else {
      @startlines = grep(/$inst/,@startimes);
      @endlines = grep(/$inst/,@endtimes);
      $numstarttimes = @startlines;
      if ($numstarttimes >= 1){$numdays += 1;}
    }

    $numendtimes = @endlines;
    my ($mon, $starthr, $startmin, $startsec, $endhr, $endmin, $endsec);

    if ($numstarttimes == $numendtimes){
      my (@temp, $startday, $endday);
      my $index=0;
      foreach (@startlines){
	 if ($startlines[$index] =~ /Mon|Tue|Wed|Thu|Fri|Sat|Sun/){
	    @temp = split(/\s{1,}/,$startlines[$index]);
	    $startday = $temp[2];
	    #print "Got start day: $startday ";
	  }
	  if ($endlines[$index] =~ /Mon|Tue|Wed|Thu|Fri|Sat|Sun/){
	    @temp = split(/\s{1,}/,$endlines[$index]);
	    $endday = $temp[2];
	    #print " Got end day: $endday ";
	  }

	  if (/(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)/){
	    $mon = where(\@months,$1);
	    #print " Got mon: $mon ";
	  }
	  if (/(..):(..):(..)/){
	    $starthr = $1; $startmin = $2; $startsec = $3; 
	  }
	  if ($endlines[$index] =~ /(..):(..):(..)/){
	    $endhr = $1; $endmin = $2; $endsec = $3;
	  }

	  #elapsed_time is in seconds.
	  if ($startday && $starthr && $startmin && $startsec && $endday && $endhr && $endmin && $endsec){
	    $elapsed_time += jd_timegm($endsec,$endmin,$endhr,$endday,$mon,$year) - 
	  		jd_timegm($startsec,$startmin,$starthr,$startday,$mon,$year);
	  } else {
	    #print "<pre>Something's missing</pre>";
	    $elapsed_time += 0;
	  }

	  #print "<pre>$_</pre>";
	  #print "<pre>$endlines[$index]</pre>";
	  #print "<pre>$numstarttimes $numendtimes $starthr:$startmin:$startsec $endhr:$endmin:$endsec $elapsed_time</pre>";
	  #print "<pre>$startday $starthr:$startmin:$startsec $endday $endhr:$endmin:$endsec $elapsed_time</pre>";
	  $index++;
      }
     }

  }
  print "<pre>There are $numdays days of $inst data available</pre>";
  return $elapsed_time;
}
