/* agex6c.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Autograph v2.00 Aug 87   Example #6   page 150
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 21 June 1991: Converted from F77 to C.
 * 16 Sep 1997:  Use C-language calls to NCAR Graphics.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

main (argc, argv)

int argc;
char *argv [];

{
   static float xbuf [501], ybuf [501];
   static float gwin [4][4] = { 0.0, 0.5, 0.5, 1.0,
                                0.5, 1.0, 0.5, 1.0,
                                0.0, 0.5, 0.0, 0.5,
                                0.5, 1.0, 0.0, 0.5 };
   static char glab [36];
   static char back [4][24] = { "Perimeter Background$",
                                "Grid Background$",
                                "Half-axis Background$",
                                "No Background$"         };
   static char linlog [4][14] = {"LINEAR$", "LOGARITHMIC$"};
   static char param [80];
   static char cstr  [80];

   double theta;

   int i;
   int igrf;
   int ival;
   int lx, ly;
   int nval = 501;
   int ngraphs = 4;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   c_opngks ();

   for (i=0; i<nval; i++)
      {
      theta = 0.031415926535898 * (double) i;
      xbuf [i] = (float) (nval-1 + 0.9 * (double) i * cos (theta));
      ybuf [i] = (float) (nval-1 + 0.9 * (double) i * sin (theta));
      }

   /*** Suppress frame advance. ***/

   c_agseti ("FRAME.", 2);

   /*** Draw 4 graphs on same frame, using different backgrounds. ***/

   for (i = 0; i < ngraphs; i++)
      {
      igrf = i + 1;

      /*** Position the graph window. ***/

      c_agsetp ("GRAPH WINDOW.", &gwin [i][0], 4);

      /*** Declare the background type. ***/

      c_agseti ("BACKGROUND TYPE.", igrf);

      /*** Turn Informational Labels on
       *** (in case they were turned off by setting the background).
       ***/

      if (igrf == 4)
         c_agseti ("LABEL/CONTROL.", 2);

      /*** Set up parameters determining linear/log nature of axes. ***/

      lx = i / 2;
      ly = i % 2;

      /*** Declare the linear/log nature of the graph. ***/

      c_agseti ("X/LOGARIGHTMIC.", lx);
      c_agseti ("Y/LOGARIGHTMIC.", ly);

      /*** Change the X/Y axis labels to reflect linear/log nature of graph.***/

      c_agsetc ("LABEL/NAME.", "B");
      c_agseti ("LINE/NUMBER.", -100);
      c_agsetc ("LINE/TEXT.", &linlog [lx][0]);

      c_agsetc ("LABEL/NAME.", "L");
      c_agseti ("LINE/NUMBER.", 100);
      c_agsetc ("LINE/TEXT.", &linlog [ly][0]);

      /*** Set up the label for the top of the graph. ***/

      sprintf (glab, "Example 6-%d %s", igrf, &back [i][0]);

      /*** Draw the graph, using EZXY. ***/

      c_ezxy (xbuf, ybuf, &nval, glab, strlen (glab));
      }

      /*** Draw a boundard around the edge of the plotter frame. ***/

/*      bndary_ (); */

      c_frame  ();
      c_clsgks ();
      exit (0);
}
