/* agex7.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Autograph v2.00 Aug 87   Example #7   page 161
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 21 June 1991: Converted from F77 to C.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

main (argc, argv)

int argc;
char *argv [];

{
   static float xbuf [101], ybuf [9][101];
   static char gdash [20];
   static char dshape [9][32];

   int i, j;
   int nval = 101;
   int ncurves = 1;
   int zero = 0;

   float fj;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   opngks_ ();

   /*** Fill data arrays. ***/

   for (i=0; i<nval; i++)
      {
      xbuf [i] = -90.0 + 1.8 * (float) (i);
      }

   for (j=0; j<ncurves; j++)
      {
      fj = (float) j;
      sprintf (&dshape [j][0], "$$$$$$$$$$$$$$$$$$$$$ J=%d", j);
      for (i=0; i<nval; i++)
         ybuf [j][i] = 3.0 * fj - (fj / 2700.0) * xbuf [i] * xbuf [i];
      }

   /*** Turn on windowing. ***/

   agseti ("WINDOWING.", 1);

   /*** Moove edges of curve window (grid). ***/

   agsetf ("GRID/LEFT.",   0.10);
   agsetf ("GRID/RIGHT.",  0.90);
   agsetf ("GRID/BOTTOM.", 0.10);
   agsetf ("GRID/TOP.",     0.85);

   /*** Set X/Y min & max. ***/

   agsetf ("X/MINIMUM.", -90.0);
   agsetf ("X/MAXIMUM.",  90.0);
   agsetf ("Y/MINIMUM.",   0.0);
   agsetf ("Y/MAXIMUM.",  18.0);

   /*** Set left axis parameters. ***/

   agseti ("LEFT/MAJOR/TYPE.", 1);
   agsetf ("LEFT/MAJOR/BASE.", 3.0);
   agseti ("LEFT/MINOR/SPACING.", 2);

   /*** Set right axis parameters. ***/

   agseti ("RIGHT/FUNCTION.", 1);
   agsetf ("RIGHT/NUMERIC/TYPE.", 1.E36);

   /*** Set bottom axis parameters. ***/

   agseti ("BOTTOM/MAJOR/TYPE.", 1);
   agsetf ("BOTTOM/MAJOR/BASE.", 15.0);
   agseti ("BOTTOM/MINOR/SPACING.", 2);

   /*** Set top axis parameters. ***/

   agseti ("TOP/FUNCTION.", 2);
   agsetf ("TOP/NUMERIC/TYPE.", 1.E36);

   /*** Set up dash patterns to be used. ***/

/*
 *   agseti ("DASH/SELECTOR.", ncurves);
 *   agseti ("DASH/LENGTH.", 28);
 */

/*
 *   agsetc (gdash, dshape);
 */

   /*** Set up left label. ***/

   agsetc ("LABEL/NAME.", "L");
   agseti ("LINE/NUMBER.", 100);
   agsetc ("LINE/TEXT.", "HEIGHT (KILOMETERS)$");

   /*** Set up right label. ***/


   agsetc ("LABEL/NAME.", "R");
   agseti ("LINE/NUMBER.", -100);
   agsetc ("LINE/TEXT.", "PRESSURE (TONS/SQUARE FURLONG)$");

   /*** Set up bottom labels. ***/

   agsetc ("LABEL/NAME.", "B");
   agseti ("LINE/NUMBER.", -100);
   agsetc ("LINE/TEXT.", "LATITUDE (DEGREES)$");

   agsetc ("LABEL/NAME.", "SP");
   agsetf ("LABEL/BASEPOINT/X.", 0.000001);
   agsetf ("LABEL/BASEPOINT/Y.", 0.0);
   agsetf ("LABEL/OFFSET/Y.", -0.015);
   agseti ("LINE/NUMBER.", -100);
   agsetc ("LINE/TEXT.", "SP$");
   
   agsetc ("LABEL/NAME.", "NP");
   agsetf ("LABEL/BASEPOINT/X.", 0.999999);
   agsetf ("LABEL/BASEPOINT/Y.", 0.0);
   agsetf ("LABEL/OFFSET/Y.", -0.015);
   agseti ("LINE/NUMBER.", -100);
   agsetc ("LINE/TEXT.", "NP$");

   /*** Set up top label. ***/

   agsetc ("LABEL/NAME.", "T");
   agseti ("LINE/NUMBER.", 80);
   agsetc ("LINE/TEXT.", "DISTANCE FROM EQUATOR (MILES)$");
   agseti ("LINE/NUMBER.", 90);
   agsetc ("LINE/TEXT.", " $");
   agseti ("LINE/NUMBER.", 100);
   agsetc ("LINE/TEXT.", "LINES OF CONSTANST INCRUDESCENCES$");
   agseti ("LINE/NUMBER.", 110);
   agsetc ("LINE/TEXT.", "EXAMPLE 7 (EZXY)$");

   /*** Set up centered box. ***/

   agsetc ("LABEL/NAME.", "EQUATOR");
   agseti ("LABEL/ANGLE.", 90);
   agseti ("LINE/NUMBER.", 0);
   agsetc ("LINE/TEXT.", "EQUATOR$");

   /*** Draw boundard around edge of plotter frame. ***/

/*
 *   bndary_ ();
 */

   /*** Draw the graph, using EZMXY. ***/

/*
 *   ezmxy (xbuf, ybuf, &nval, &ncurves, &nval, &zero);
 */
   ezxy (xbuf, &ybuf [0][0], &nval);

   /*** Close GKS. ***/

   frame_ ();
   clsgks_ ();
   exit (0);
}
   
