;+
; pmfitsname.pro   IDL procedure
;
; PURPOSE
;	Read a FITS image and change the file name. 
;
; SYNTAX
;	pmfitsname, 'img.fts'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   10 October 1997
;-

pro pmfitsname, fits_list

fits_name = ''				; Define FITS file name variable.

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      date   = fxpar (hdu, 'DATE-OBS')
      time   = fxpar (hdu, 'TIME-OBS')
      day    = strmid (date, 0, 2)
      month  = strmid (date, 3, 2)
      year   = strmid (date, 6, 2)
      hour   = strmid (time, 0, 2)
      minute = strmid (time, 3, 2)
      second = strmid (time, 6, 2)
      new_fits_name = 'pm.' + year + month + day + '.' + $
			      hour + minute + second + '.fts'

      fxaddpar, hdu, 'TELESCOP', 'PMON',     'PROMINENCE MONITOR'
      fxaddpar, hdu, 'INSTRUME', 'FILM',     '35MM FILM CAMERA'
      fxaddpar, hdu, 'OBJECT',   'SUN-LIMB', 'SOLAR LIMB USING OCCULTING DISC'
      fxaddpar, hdu, 'TYPE-OBS', 'H-ALPHA',  'H-ALPHA FILTER'
;      fxaddpar, hdu, 'COMMENT', 'DIGITIZED FILM IMAGE FROM MLSO PROMINENCE MONITOR.'
;      fxaddpar, hdu, 'COMMENT', 'IMAGE ARRAY ORIGIN IS AT THE LOWER LEFT CORNER.'
;      fxaddpar, hdu, 'HISTORY', 'SUN CENTER DETERMINED USING SOHO IMAGE_TOOL SOFTWARE.'
;      fxaddpar, hdu, 'HISTORY', 'ORIENTATION BASED UPON P-ANGLE OF -18.08 DEGREES.'

      ; -- Write FITS image to disk.

      writefits, new_fits_name, img, hdu

   END
CLOSE, 1

END
