#!/opt/local/bin/perl
#-------------------------------------------------------------------------------
# days.pl
#-------------------------------------------------------------------------------
# This program returns variously formatted date strings for the
# date a specified number of days ago or forward from today
#
# if five arguements are supplied, the last two are 
# interpreted as the reference year and day of year, and
# the date is calculated relative to that particlar date
#-------------------------------------------------------------------------------

# Import modules from the time package.

  use Time::Local;

# The program pathname.

  $progpath = $0;

# Parse the program name.

  $pathlength = length ($progpath);
  @parts      = split (/\//,$progpath);
  $nparts     = scalar (@parts);
  $drivename  = $parts[0];
  $imgsource  = $parts[$nparts-2];
  $progname   = $parts[$nparts-1];
  $proglength = length ($progname);
  $loc_dir    = substr ($progpath, 0, $pathlength-$proglength-1);
  $scriptlib  = "$drivename\\imagescripts\\lib";

# Directory to include in search path.

  unshift (@INC, "$scriptlib");

# How many command arguements ?

  my $args = scalar (@ARGV);

# Store the arguements in named variables: 
# $dir for direction, $dd for number of days
# $fmt for format, 
# $nyear for reference year (default current year)
# $ndoy for reference day of year (default current doy)

  my ($dir, $dd, $fmt, $nyear, $ndoy) = @ARGV;

# declare some local variables

  my ($nmonth,$nyr,$nep,$dd_sec,$qep,$qyear,$qmonth,$qday,$qdoy,$res);

# Modify delta days ($dd) depending on direction

  if ($dir eq 'b') { $dd *= -1; }

# Provide a help message if there are less than two arguements

  if ($args < 2)
    {
    print "Arguments:     1:  b (Back) or f (Forward)\n";
    print "               2:  Number of days back or forward\n";
    print "           [   3:  Format (1 = DD Mon YYYY, e.g. 24 Jul 1996\n";
    print "                           2 = YYYY DOY\n";
    print "                           3 = YYYY_MM_DD   (Used by pmapw)\n";
    print "                           4 = YYYY,MM,DD)  (Used by tiger)\n";
    print "                           5 = YYYY Mon DD HHMM UT\n";
    print "                           6 = YYYY (Used by process.ksh)\n";
    print "                           7 = DOY\n";
    print "                           8 = YYYYMMDD (Used by GetAcdMag1s)\n";
    print "                           9 = YYYY MM DD (Used by ACE scripts)\n";
    print "                          10 = YYDOY (Used by tiger)\n";        
    print "             4-5:  Year \"now\" and Day of year \"now\"   ]\n";
    }

# A string array with the month names.

  my @mon = (Jan, Feb, Mar, Apr, May, Jun, Jul, Aug, Sep, Oct, Nov, Dec);

# If the fourth and fifth arguements were not supplied then
# get today's year and day of year (and additional time informatino).

  if ($args < 5)
    { 
    ($sec,$min,$hr,$mday,$mon,$nyear,$wday,$ndoy,$isdst) = gmtime(time);
    $ndoy++;
    }

# If there was no third arguement, specify default format.

  if ($args < 3) { $fmt = 1;}		# Default format selection

# Using the day of year and the year, find the year, month, and day of month.

  ($nyr, $nmonth, $nday) = &Monday ($ndoy, $nyear);

# Convert - just give the last two digits of the year

  $nyr = substr ($nyr, -2);

# System epoch in seconds for this year, month, day at 00:00:00 UTC.

  $nep = timegm (0, 0, 0, $nday, $nmonth-1, $nyr);

# Now to the differencing using $dd (delta days).

  $dd_sec = $dd * 1440 * 60;
  $qep = $nep + $dd_sec;

# Use gmtime to convert and store in array.

  @qdate = gmtime ($qep);

# The following 2 statements will be ok until the year 2070 
# (or until PERL starts using 4-digit years!!!).

  $qyear = $qdate[5] + 2000;
  if ( $qyear > 2070 ) { $qyear = $qyear - 100; }
  $qmonth = $qdate[4] + 1;
  $qday   = $qdate[3];
  $qdoy   = $qdate[7] + 1;

# Put output into desired format and print it.

  if ($fmt == 1)
    {
    $res = sprintf ("%02d %s $qyear\n", $qday, @mon[$qmonth-1]);
    }
  elsif ($fmt == 2)
    {
    $res = sprintf ("$qyear %03d\n", $qdoy);
    }
  elsif ($fmt == 3)
    {
    $res = sprintf ("%4d_%02d_%02d\n", $qyear, $qmonth, $qday);
    }
  elsif ($fmt == 4)
    {
    $res = sprintf("%4d, %02d, %02d\n", $qyear, $qmonth, $qday);
    }
  elsif ($fmt == 5)
    {
    $res = sprintf ("%4d %s %02d %02d%02d UT\n",
		$qyear, $mon[$qmonth-1], $qday, $hr, $min);                    
    }
  elsif ($fmt == 6)
    {
    $res = sprintf ("%04d\n", $qyear);
    }
  elsif ($fmt == 7)
    {
    $res = sprintf ("%03d\n", $qdoy);
    }
  elsif ($fmt == 8)
    {
    $res = sprintf ("%04d%02d%02d\n", $qyear, $qmonth, $qday);
    }
  elsif ($fmt == 9)
    { 
    $res = sprintf ("%04d %02d %02d\n", $qyear, $qmonth, $qday);
    }
  else
    {
    $res = sprintf ("%2s%03d\n", substr ($qyear, 2, 2), $qdoy);
    }
  print $res;

# ------------------------------------------------------------------------------

sub Monday
    {

#    require ("Leap_year.pl");
    my ($i, $m, $doy, $year, $month, $day, $leap);
    my @monend = (0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365);
    $doy  = shift (@_);
    $year = shift (@_);
    $leap = &Leap_year ($year);
    if ($leap) { foreach $m (@monend[2..12]) {$m++;} }
    $i = 0;

    while ($i <= 12 )
	{
        if ($doy <= $monend[$i]) { last; }
        $i++;
        }
    
  # Now $i is the day ending the high month.

    $day = $doy - $monend[$i-1];
    $month = $i;
    return ($year, $month, $day);
}

# ------------------------------------------------------------------------------

# Leap_year.pl -- Returns 1 if input 4-digit year is a ly year, 0 if not.

sub Leap_year
    {

    $qyear = shift;    # Get input year
    
    $ly = 0;
    
    if ( $qyear % 400 == 0 )		# Divisible by 400.
	{
        $ly = 1;
        }
    else
	{
        if ( $qyear % 4 == 0 )		# Divisible by 4.
	    {
            if ( $qyear % 100 != 0 )	# NOT divisible by 100
		{
                $ly = 1;
                }
            }
        }
    return $ly;
    }
1;
# ------------------------------------------------------------------------------
