#!/opt/local/bin/perl -w
#-------------------------------------------------------------------------------
# mlsoimages.pl
#-------------------------------------------------------------------------------
# Written 13 Nov 2002
# Automatic ftp script 
#
# Get images from Mauna Loa
# (H-alpha and 10830)
#
# Modified 22 April 2003 to use 
# library for common scripts
#
# Modified  7 January 2004 
# Make sure we don't transfer files that
# are older than the ndays retention interval
#
# Modified 13 September 2005
# add Mk4 image data
#
#-------------------------------------------------------------------------------
use Cwd;

# --- Some specifics.

$ndays     = 27; 
$fileext   = "gif";
@typenames = ("asr", "hsl", "rawmk4");
$imgsource = 'mlso';

# --- Remote server information.

$host_name  = 'ftp.hao.ucar.edu';
@remote_dir = ('/d5/dpm/raw_daily_image','/d5/chip/raw_daily_image','/d5/mk4/raw_daily_image/');
$port       = 122;
$uid        = "anonymous";
$upwd       = 'swo\@noaa.gov';
$upwd       = "stanger";
print ("port: $port\n") ;
print ("uid:  $uid\n") ;
print ("upwd: $upwd\n") ;

# --- Make note of the start time.

$startTime = localtime ();			# string format.

# --- Turn off buffering of STDOUT.

$| = 1;   

# --- The program pathname.

$progpath = $0;

# --- Parse the program name.

$pathlength = length ($progpath);
@parts      = split (/\//,$progpath);
$nparts     = scalar (@parts);
$drivename  = $parts[0];
$progname   = $parts[$nparts-1];
$proglength = length ($progname);
$loc_dir    = substr ($progpath, 0, $pathlength - $proglength - 1);
$scriptlib  = "$drivename\\imagescripts\\lib";
$scriptlib  = "/picard/d/img/noaa";

#print ("pathlength: $pathlength\n") ;
#print ("parts:      @parts\n") ;
#print ("parts [0]:  $parts[0]\n") ;
#print ("parts [1]:  $parts[1]\n") ;
#print ("parts [2]:  $parts[2]\n") ;
#print ("parts [3]:  $parts[3]\n") ;
#print ("progpath:   $progpath\n") ;
#print ("progname:   $progname\n") ;
#print ("drivename:  $drivename\n") ;
#print ("loc_dir:    $loc_dir\n") ;
#print ("scriptlib:  $scriptlib\n") ;

# --- Location of the image data.

#$imagedir = "i:\\imagedata\\$imgsource"; 
#$imagedir = "imagedata\/$imgsource"; 
$imagedir = "$imgsource"; 
#print ("imagedir: $imagedir\n") ;
#print ("\n") ;

# --- Add the scripts library directory to the PATH environment variable.

# set PATH = e:\imagescripts\lib;%PATH%

$ENV{PATH} = "$ENV{PATH};$scriptlib";

# --- Tell me when you started.

print "$progname started: $startTime\n";

# --- Get start time - seconds into the system epoch.

$startTime = time ();			# --- long integer.

# --- Go to that directory.

$result = chdir ("$loc_dir");

if (!$result)
    {
    $cwd = cwd ();
    print "current directory is $cwd\n";
    die "Can't change to directory $loc_dir\n";
    }

# --- Use the Net::FTP package.

use Net::FTP;

# --- If this directory doesn't exist, then create it.

$result = chdir ("$imagedir");

if (!$result)
    { 
    `mkdir $imagedir`;
    }
chdir ("$loc_dir");

# --- Get a list of files in the image directory.

@haveimages = <$imagedir\/*.$fileext>;
$numhave    = scalar (@haveimages);

print ("\n") ;
print "There are currently $numhave images stored locally.\n\n";

# --- Get a string that gives the date $ndays days ago.
# --- Format is yyyymmdd - this defines 'old' files
# --- (i.e. any files with yyyymmdd less than $old 
# ---  are deleted and should not be transferred).

$old = `days.pl b $ndays 8`;
chomp $old;

# --- Keep track of the number of files that are removed.

$rm_num = 0;

# --- Run through all the files, and delete the old ones.
# --- Also delete incomplete files.

foreach $file (@haveimages)
    {

    # --- Remove the EOL character.

    chomp $file;

    # --- Extract the yyyymmdd part of the filename.

    $yearstr  = substr ($file, length ($imagedir) + 1,     2);
    $monthstr = substr ($file, length ($imagedir) + 1 + 3, 2);
    $daystr   = substr ($file, length ($imagedir) + 1 + 6, 2);
    $test_str = "20$yearstr$monthstr$daystr";

#    print ("imagedir:  $imagedir\n") ;
#    print ("yearstr:   $yearstr\n") ;
#    print ("monthstr:  $monthstr\n") ;
#    print ("daystr:    $daystr\n") ;
#    print ("test_str:   $test_str\n") ;

    # --- Is the file old ?

    $res = $test_str lt $old;

    # --- If it is old, or it is incomplete delete it.

    if ($res || -s $file < 10)
        { 
        `del $file`;
        $rm_num++;
        print "deleted $file\n";
        }  # --- if ($res)

    # --- Strip off the directory information.

    $file = substr($file,length($imagedir)+1,80);    

    }  # foreach

# --- Tell me about it.

print ("Removed $rm_num local files as old as or older than $old\n");
print ("\n") ;

# --- Delete log files older than one year.

`rm_oldlogs.pl mlsodata 365`;
`rm_oldlogs.pl mlsodata 31`;

# --- Change working directory to the image directory.

$result = chdir ("$imagedir");
if (!$result) { die "Can't change to directory $imagedir\n"; }

# --- Connect to the site to get new image files.

print "Connecting to ftp site:...\n";
#$ftp = Net::FTP->new("$host_name", "Port", $port, "Passive", 1);

#$ftp = Net::FTP->new("$host_name", "Port", $port);

$ftp = Net::FTP->new("$host_name", "Passive", 1);
$ftp = Net::FTP->new("$host_name");
#print ("ftp  $ftp\n") ;

# --- Login.

$ier = $ftp->login($uid, $upwd);

# --- Check for failure.

if (!$ier)
    {

    # --- Login failed.

    print "Login failed\n";
    $| = 0;  # --- Turn STDOUT buffering back on
    exit;
    }  # End of block for 'if ($@ ne "");

# --- Make sure we are in binary mode.

$ftp->binary;

# --- Loop on each type of image.

print ("typenames: @typenames\n") ;

$itype = 0;
foreach $imgtype (@typenames)
    {
  
    # --- Diagnostic.
  
    print ("\n") ;
    print "Processing for image type: $imgtype\n";
  
    # --- Get list of image files we have in $imagedir.
    # --- These images are in gif format.
    # --- Store in @haveimage array.
  
    @haveimages = <*$typenames[$itype].$fileext>;
    $nimages = scalar (@haveimages);
    print "There are $nimages locally for this file type\n";
  
    # --- Chop of the end of line characters.
  
    foreach $file (@haveimages) { chomp $file; }
  
    # --- Change remote directory to appropriate location.
  
    $ftp->cwd ("$remote_dir[$itype]");
#    print ("ftp> cwd $remote_dir[$itype]\n") ;
  
    # --- Get a list of files on the remote site.
  
    $curtemp = "*$imgtype.gif";
    @images1 = $ftp->ls("$curtemp");
    $rimg    = $images1[0];

#    print ("images1:\n");
#    print ("@images1\n");

    if (! $rimg) {print "I got an empty list !!!\n";}
    next if (! $rimg); 
    $numimages = scalar (@images1);
    print "There are $numimages remote images for this file type\n";
  
    # --- Keep count of the number of new images
    # --- that need to be transferred.
  
    $j = 0;
  
    # --- Loop through each image file on the remote site.
  
    for ($i=0; $i<$numimages; $i++)
        {
    
        # --- Variable to hold the filename on the server.
    
        $file = $images1[$i];
    
        # --- Extract the yyyymmdd part of the filename.
    
        $yearstr  = substr ($file,0,2);
        $monthstr = substr ($file,3,2);
        $daystr   = substr ($file,6,2);
        $test_str = "20$yearstr$monthstr$daystr";
    
        # --- Is the file old ?
     
        $res = $test_str lt $old;
        next if $res;
    
        # --- Assume that it is new until proven otherwise.
    
        $new = 1;
    
        # --- Now compare this file with what we already have.
        # --- if we have it, $new is set to zero; otherwise, 
        # --- we know it is a new file and will want to get it.
    
        foreach $haveimage (@haveimages)
    	    {
            if ($haveimage eq $file)
    	        {
                $new = 0;  
                last; 
                }   
            }
    
        # --- If this is a new file that we don't have
        # --- (check for bad files too).
    
        if ($new == 1 && index ($images1[$i],'.gif',0) > 1)
    	    {      
            $getimages[$j] = $file;
            $j++;        
            }
        }  # End of block for 'for ($i=0; $i<$numimages; $i++)'
  
    # --- Get each file.
  
    $i = 1;
    for ($jj=0; $jj<$j; $jj++)
        {
        $animage = $getimages[$jj];
        $ftp->get ($animage);
        print "transferring $animage\n";
        $i++;        
        }
  
    # --- Increment itype.
  
    $itype++;
  
    }  # On to the next type of file...
 
# --- Log off the ftp session.

$ftp->quit;

# --- Get end time.

$endTime = time ();
printf "Elapsed time: %ld seconds\n",$endTime-$startTime;
chdir ("$loc_dir");

# --- Turn STDOUT buffering back on.

$| = 0;  

# --- DONE !!!
