#include <stdio.h>
#include "fitsio.h"
#include "libacos.h"

main (int argc, char *argv[]){

  fitsfile *rawptr, *diffptr, *darkptr, *outfptr;
  int status = 0;
  int anynull = -1, nullval, i;
  int bitpix, ix, iy;
  long fpixel = 1, nelements;
  long xsz, ysz;
  long naxes[2] = {0,0}, naxis = 2;
  float *rawimg, *diffuser, *darkimg;
  short *calimg;
  float brtpct = 0.05, brtavg, crradius;
  char datestr[11];
  int year, month, day;
  float xcen, ycen, xdist, ydist, radius;
  float avg, adev, sdev, var, skew, curt;

  char *diffname = "", *rawname = "", *darkname = "", *outname = "";

  if (argc != 5){
    printf("usage:  picscal raw_image diffuser dark outputname\n");
    exit(1);
  }

  rawname  = argv[1];
  diffname = argv[2];
  darkname = argv[3];
  outname  = argv[4];

  fits_open_file(&rawptr, rawname, READONLY, &status);
  fits_open_file(&diffptr, diffname, READONLY, &status);
  fits_open_file(&darkptr, darkname, READONLY, &status);
  
  /* Get today's date */
  fits_get_system_date(&day, &month, &year, &status);
  fits_date2str(year, month, day, datestr, &status);

  fits_read_key(rawptr, TINT, "NAXIS1", &xsz, NULL, &status);
  fits_read_key(rawptr, TINT, "NAXIS2", &ysz, NULL, &status);
  fits_read_key(rawptr, TINT, "BITPIX", &bitpix, NULL, &status);
  fits_read_key(rawptr, TFLOAT, "CRRADIUS", &crradius, NULL, &status);
  fits_read_key(rawptr, TFLOAT, "CRPIX1",&xcen, NULL, &status);
  fits_read_key(rawptr, TFLOAT, "CRPIX2",&ycen, NULL, &status);

  naxes[0] = xsz;
  naxes[1] = ysz;

  if (xcen < 0) xcen = xsz / 2.0;
  if (ycen < 0) ycen = ysz / 2.0;

  if (status) {
    fits_report_error(stderr, status); /* print error message */
    return(status);
  }

  nelements = xsz * ysz;
  rawimg   = (float *) malloc (nelements * sizeof (float));
  calimg   = (short *) malloc (nelements * sizeof (short));
  diffuser = (float *) malloc (nelements * sizeof (float));
  darkimg  = (float *) malloc (nelements * sizeof (float));

  fits_read_img(rawptr, TFLOAT, fpixel, nelements, &nullval, rawimg, &anynull, &status);
  fits_read_img(diffptr, TFLOAT, fpixel, nelements, &nullval, diffuser, &anynull, &status);
  fits_read_img(darkptr, TFLOAT, fpixel, nelements, &nullval, darkimg, &anynull, &status);

  fits_close_file(rawptr, &status);
  fits_close_file(diffptr, &status);
  fits_close_file(darkptr, &status);

  /* Clean up raw image of hot pixels and negative values
   *  due to wraparound of hot pixels
   */

  acos_moment(darkimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);
  for (i = 0; i < nelements; i++){
    if (darkimg[i] < 0) darkimg[i] = 0;
    if (darkimg[i] > 6 * sdev) darkimg[i] = 0;
  }

  acos_moment(rawimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);
  for (i = 0; i < nelements; i++){
    if (rawimg[i] < 0) rawimg[i] = 0;
    if (rawimg[i] > 7 * sdev) rawimg[i] = 0;
  }

  /* Subtract the dark from the diffuser first */

  /* printf ("Subtracting dark...\n"); */
  /*
  for (i = 0; i < nelements; i++){
    rawimg[i] -= darkimg[i];
    if (rawimg[i] < 0) rawimg[i] = 0;
  }
  */

  /* Apply the corrected diffuser */

  /*
  for (i = 0; i < nelements; i++){
      if (diffuser[i] > 0.05)
        rawimg[i] /= diffuser[i];
  }
  */

  acos_moment(rawimg, nelements, &avg, &adev, &sdev, &var, &skew, &curt);
  /*
  for (iy = 0; iy < ysz; iy++)
    for (ix = 0; ix < xsz; ix++){
      xdist = (float) ix - xcen ;
      ydist = (float) iy - ycen ;
      radius = sqrt ((xdist * xdist) + (ydist * ydist));
      if ((radius/crradius) > 1.2) 
      if (*(rawimg + ix + iy * xsz) < 20)
        *(rawimg + ix + iy * xsz) = 1.; 
    }
  */

  for (i = 0; i < nelements; i++)
    calimg[i] = (short) rawimg[i];

  /* Get rid of outfile if it already exists */
  /* acos_remove_file(outname); */

  fits_create_template(&outfptr,outname,rawname,&status);
  fits_update_key(outfptr, TINT, "BITPIX", &bitpix, "4 BYTE Float", &status); 
  fits_set_hdustruc(outfptr, &status);

  fits_update_key(outfptr, TSTRING, "DATE-CAL", &datestr, NULL, &status); 
  fits_update_key(outfptr, TSTRING, "CODE-OBS", "AFC",NULL, &status); 

  /* Write the corrected diffuser image to disk */
  fits_write_img(outfptr, TSHORT, fpixel, nelements, calimg, &status);
  
  fits_close_file(outfptr, &status);

}
