;
;
; Routine to calibrate a list of pics images
;
;	For each position in the field
;	s = Xinv t
;	Where s is the coronal stokes vector, Xinv
;	is the inverse polarimeter response matrix
;	and t is the measured stokes vector.
;
;	A list of files is read one at a time and calibrated.
;	Each measurement consitst of a pair of files
;	one for modulation states 1 and 3, the other for
;	modulation states 2 and 4.
;	For each file, bias is computed and removed using
;	a dark region in the field.  Note:  if the image
;	alignment changes, the position of a dark region
;	may change.
;	For the upper and lower beams, create a measured
;	stokes vector at each position.
;	I = ( m1 + m2 + m4 + m4 )/4
;	Q = ( m1 + m2 - m3 - m4 )/4.
;	U = ( m1 - m2 - m3 + m4 )/4.
;	Apply the inverse polarimeter response to each
;	vector and apply the gain.
;	
;	Correct the centering for each image
;	Note: Centering values change with optical alignment
;	such as the one at the end of April 1999.
;
;	Add the calibrated S to the calibrated P
;	to get the resulting inferred stokes input.
;
;	David Elmore
;	1/5/99
;
;window,0,xsize=384,ysize=256,xpos=768,ypos=660,title="I"
;window,1,xsize=384,ysize=256,xpos=768,ypos=374,title="Q"
;window,2,xsize=384,ysize=256,xpos=768,ypos= 88,title="U"

file0 = 'fred'
file1 = 'fred'

loadct,3

xpix = 767
ypix = 2060
xsiz = xpix
ysiz = ypix/4

shortimg = intarr(xpix,ypix)
hdr0 = intarr(xpix)
img = fltarr(xpix,ypix)
hdr1 = intarr(xpix)
sx = fltarr(4,4,xsiz,ysiz)
px = fltarr(4,4,xsiz,ysiz)
gains = fltarr(xsiz,ysiz)
gainp= fltarr(xsiz,ysiz)

;
;  Four elements to measured vector
;
s1 = fltarr(xsiz,ysiz)
p1 = fltarr(xsiz,ysiz)
s2 = fltarr(xsiz,ysiz)
p2 = fltarr(xsiz,ysiz)
s3 = fltarr(xsiz,ysiz)
p3 = fltarr(xsiz,ysiz)
s4 = fltarr(xsiz,ysiz)
p4 = fltarr(xsiz,ysiz)

;
; Three Stokes parameters in calibrated vector
;
pci = intarr(xsiz,ysiz)
pcq = intarr(xsiz,ysiz)
pcu = intarr(xsiz,ysiz)
sci = intarr(xsiz,ysiz)
scq = intarr(xsiz,ysiz)
scu = intarr(xsiz,ysiz)

x=intarr(2)
y=intarr(2)

sxinv = fltarr(3,3,xsiz,ysiz)
pxinv = fltarr(3,3,xsiz,ysiz)
gains = fltarr(xsiz,ysiz)
gainp = fltarr(xsiz,ysiz)

; offsets lower half of image vs upper half
; the two beams are called s and p
;
;  sxoff, syoff, pxoff, and pyoff need to be trimmed up so
;  that all images register correctly.
;  Not only the relaive values need to be adjusted, but the
;  absolute values which will put the image in the center of
;  the field need to be determined.  7/3/99 DFE
;
sxoff = -40
syoff = 0.
sxcen = 383 + sxoff
sycen = 257 + syoff

pxoff = -30.7
pyoff = -1.5
pxcen = 383 + pxoff
pycen = 257 + pyoff

xdim = xsiz
ydim = ysiz
x0 = 1.*xsiz/2.
y0 = 1.*ysiz/2.

;
; where to find frame bias
; These rows were OK 7/3/99 DFE
;
bxmin = xpix-6
bxmax = xpix-1
bymin = ypix/4
bymax = ypix/2 -1
btot = (bxmax-bxmin)*(bymax-bymin)

;
; Read in the inverse p and s matrices
;
print,'sxinv'
close,1 & openr,1,'sxinv' & readu,1,sxinv
print,'pxinv'
close,1 & openr,1,'pxinv' & readu,1,pxinv
print,'gains'
close,1 & openr,1,'gains' & readu,1,gains
print,'gainp'
close,1 & openr,1,'gainp' & readu,1,gainp


close,13 & openr,13,'picslist'

i = 0
while ( not eof(13) ) do begin ;{
	readf,13,file0
	readf,13,file1

!p.title=file0
;	wset,0

	print,file0
	close,1 & openr,1,file0 & readu,1,shortimg
	hdr0 = shortimg(*,0)
	img = float(shortimg)

;
; Extract header
; Remove header from image
; Remove bias
; Normalize by exposure time
;
	exposure = float(hdr0(21))
	bias = total(img(bxmin:bxmax,bymin:bymax))/btot
	img(*,0) = img(*,2)
	img(*,1) = img(*,2)
	img(*,ypix/2) = img(*,ypix/2+2)
	img(*,ypix/2+1) = img(*,ypix/2+2)
	img = img - bias
	img = img/exposure

	p1 = img(0:xsiz-1,ysiz:2*ysiz-1)
	s1 = img(0:xsiz-1,0:ysiz-1)

	p3 = img(0:xsiz-1,3*ysiz:4*ysiz-1)
	s3 = img(0:xsiz-1,2*ysiz:3*ysiz-1)

	print,file1
	close,1 & openr,1,file1 & readu,1,shortimg 
	hdr1 = shortimg(*,0)
	exposure = float(hdr1(21))
	img = float(shortimg)
	bias = total(img(bxmin:bxmax,bymin:bymax))/btot
	img(*,0) = img(*,2)
	img(*,1) = img(*,2)
	img(*,ypix/2) = img(*,ypix/2+2)
	img(*,ypix/2+1) = img(*,ypix/2+2)
	img = img - bias
	img = img/exposure

	p2 = img(0:xsiz-1,ysiz:2*ysiz-1)
	s2 = img(0:xsiz-1,0:ysiz-1)

	p4 = img(0:xsiz-1,3*ysiz:4*ysiz-1)
	s4 = img(0:xsiz-1,2*ysiz:3*ysiz-1)

	pi = ( p1 + p2 + p3 + p4 )/4.
	si = ( s1 + s2 + s3 + s4 )/4.
	pq = ( p1 + p2 - p3 - p4 )/4.
	sq = ( s1 + s2 - s3 - s4 )/4.
	pu = ( p1 - p2 - p3 + p4 )/4.
	su = ( s1 - s2 - s3 + s4 )/4.

;
; Apply inverse X matrix to infer I, Q, and U
;

;sci = sxinv(0,0,*,*)*si + sxinv(1,0,*,*)*sq + sxinv(2,0,*,*)*su
;scq = sxinv(0,1,*,*)*si + sxinv(1,1,*,*)*sq + sxinv(2,1,*,*)*su
;scu = sxinv(0,2,*,*)*si + sxinv(1,2,*,*)*sq + sxinv(2,2,*,*)*su

;pci = pxinv(0,0,*,*)*pi + pxinv(1,0,*,*)*pq + pxinv(2,0,*,*)*pu
;pcq = pxinv(0,1,*,*)*pi + pxinv(1,1,*,*)*pq + pxinv(2,1,*,*)*pu
;pcu = pxinv(0,2,*,*)*pi + pxinv(1,2,*,*)*pq + pxinv(2,2,*,*)*pu

sci = sxinv(0,0,*,*)*si + sxinv(0,1,*,*)*sq + sxinv(0,2,*,*)*su
scq = sxinv(1,0,*,*)*si + sxinv(1,1,*,*)*sq + sxinv(1,2,*,*)*su
scu = sxinv(2,0,*,*)*si + sxinv(2,1,*,*)*sq + sxinv(2,2,*,*)*su

pci = pxinv(0,0,*,*)*pi + pxinv(0,1,*,*)*pq + pxinv(0,2,*,*)*pu
pcq = pxinv(1,0,*,*)*pi + pxinv(1,1,*,*)*pq + pxinv(1,2,*,*)*pu
pcu = pxinv(2,0,*,*)*pi + pxinv(2,1,*,*)*pq + pxinv(2,2,*,*)*pu

sci = reform(sci)/gains
scq = reform(scq)/gains
scu = reform(scu)/gains

pci = reform(pci)/gainp
pcq = reform(pcq)/gainp
pcu = reform(pcu)/gainp

;
; Now once calibrated it is OK to shift the images
; to sum and difference
;
sci = rot(sci,0.,1.,sxcen,sycen,/interp)
scq = rot(scq,0.,1.,sxcen,sycen,/interp)
scu = rot(scu,0.,1.,sxcen,sycen,/interp)

pci = rot(pci,0.,1.,pxcen,pycen,/interp)
pcq = rot(pcq,0.,1.,pxcen,pycen,/interp)
pcu = rot(pcu,0.,1.,pxcen,pycen,/interp)

ci = (sci+pci)/2.
cq = (scq+pcq)/2.
cu = (scu+pcu)/2.

!y.style=1
!y.range=[0,1.1e-5]
wset,0
tv,bytscl(congrid(ci,384,258),0,4.e-5)
;plot,ci(*,258)
wset,1
!y.range=[-5.e-6,5.e-6]
tv,bytscl(congrid(cq,384,258),-2.e-6,2.e-6)
;plot,cq(*,258)
wset,2
!y.range=[-5.e-6,5.e-6]
tv,bytscl(congrid(cu,384,258),-2.e-6,2.e-6)
;plot,cu(*,258)

;
; Here is where one would like to do a coordinate transformation
; into the local solar radius coordinate frame.
; Centering the images and then using a cos 2 theta and sin 2 theta
; map of the field is a good way to transform q and u in this frame
; to the solar one.
; Once in the solar reference frame, U should be near zero.  If not
; there is an error in the calibration.
; In the solar reference frame, Q should have a better signal to noise
; than pol does.  7/3/99 DFE
; This would also be a good place to correct for P angle.
; Finally, one might want to trim off the portions of the field
; with no signal to noise.  A square 512x512 sample might be
; perfect.

pol = sqrt(cq*cq+cu*cu)
;tv,bytscl(congrid(pol,384,258),-2.e-6,3.e-6)


stop
;
; Here is where one might want to pick up the header values
; which were stripped off and pack them along with the
; calibrated data into a FITS file
;;;;;;

;;;;;;

end ;}

stop

end

