#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <values.h>

#include "SciCPP.hpp"
#include "fitsio.h"
#include "libacos.h"
#include "libldc.hpp"

main(int argc, char *argv[]){

  char Fname[80], ImName[80], MaskName[80];
  char datestr[11];

  int ip, i, j, k, ii, jj, count, status = 0;
  int day, month, year, bitpix = 16, nc;
  long naxes[2], fpixel = 1, nelements, naxis = 2;
  
  fitsfile *fptr;
  float xc, yc, rx, ry, avg, bg_amp, maxR, fitmax, crradius;
  float xshft, yshft;
  float radius, xdist, ydist;
  float hours, mins, secs;
  double seconds;
  int datamin, datamax, dispmin, dispmax, xdim, ydim;
  float *img;
  short *simg;
  float pangle;

  FILE *fp;

  Matrix im, mask, fit, invfit, flat;
  Vector dr, di, dmu, MedianCoef;
  CoefLabel *coef;
  time_t start, stop;

  if(argc < 2){
    printf("\nusage:  picsldc image [image image...]\n\n");
    exit(1);
  }

  printf ("\nPICSLDC:  There are %d file(s) to process\n\n",argc - 1);

  for(ip = 1; ip < argc; ip++){
    strcpy(ImName, argv[ip]);
    printf("\nProcessing %s\n\n", ImName);
    start = time(NULL);

    im = readfits(ImName);
    fits_open_file(&fptr, ImName, READONLY, &status);
    fits_read_key(fptr, TFLOAT, "CRRADIUS", &crradius, NULL, &status);
    fits_read_key(fptr, TFLOAT, "CROTA1", &pangle, NULL, &status);
    fits_close_file(fptr, &status);

    count = 0;
    avg = 0.0;
    for(ii = 757; ii <= 777; ii++)
      for(jj = 505; jj <= 525; jj++){
	avg += im(ii, jj);
	count++;
      }
    avg /= count;

    printf("Average disk intensity:  %f  (%d)\n", avg, count);
    count = 0;
    bg_amp=0.0;
    for(ii = 100; ii <= 200; ii++)
      for(jj = 100; jj <= 200; jj++){
	bg_amp += im(ii, jj);
	count++;
      }
    bg_amp /= count;
    if (bg_amp < 0.0)
      bg_amp = 1.0;
    printf("Average background intensity:  %f  (%d)\n", bg_amp, count);

    xdim = im.Rows();
    ydim = im.Columns();

    solargeom(im, &xc, &yc, &rx, &ry);

    avg = 0.0;
    count = 0;
    for(ii = xc-10; ii <= xc+9; ii++)
      for(jj = yc-10; jj <= yc+9; jj++){
	avg += im(ii, jj);
	count++;
      }
    avg /= count;
    printf("Disk center average intensity:  %f  (%d)\n", avg, count);

    mask = quartile_profile(im, xc, yc, Min(rx, ry), 1.2, -1.2, di, dr);
    coef = legendre_fourier(im, xc, yc, Min(rx, ry), mask, 1, 4, &nc);
    fit  = make_fit_image(xdim, ydim, xc, yc, Min(rx, ry),1,4,coef);
    fitmax = fit.Max();
    
    strcpy(MaskName,ImName);
    //strcpy(strstr(MaskName,"asc"),"mask.fts");
    strcpy(strstr(MaskName,"fts"),"mask.fts");
    writefits("quartile_fit.fts",fit);
    writefits(MaskName,mask);
    ChangeDimensions(xdim, ydim, &flat);
    for(j = 1; j <= ydim; j++)
      for (i = 1; i <= xdim; i++){
	xdist = (float) i - xc;
	ydist = (float) j - yc;
	radius = sqrt ((xdist * xdist) + (ydist * ydist)) ;
	radius /= crradius ;
	if (radius <= 1.0)
	  /* flat(i,j) = (im(i,j) / fit(i,j)) * fitmax; */
	  flat(i,j) = (im(i,j) / fit(i,j)) * 1000;
        else
	  flat(i,j) = im(i,j);
      }

    /*
     *  I had to write my own writefits here in order to
     *  preserve the header info that is necessary later
     *  in the processing.
     */

    naxes[0] = flat.Rows();
    naxes[1] = flat.Columns();
    datamin = (int) flat.Min();
    datamax = (int) flat.Max();
    dispmin = 800;
    dispmax = 920;
    nelements = naxes[0] * naxes[1];
    img = new float[nelements];
    simg = (short *) malloc (nelements * sizeof(short));
    k = 0;
    for(j = 1; j <= naxes[1]; j++)
      for(i = 1; i <= naxes[0]; i++){
        img[k] = flat(i, j);
	simg[k] = (short) img[k];
        k++;
      }

    strcpy(Fname, ImName);
    //strcpy(strstr(Fname,"sc"),"sl.fts");
    strcpy(strstr(Fname,"fts"),"test.fts");
    /* Get rid of outfile if it already exists */
    fits_open_file(&fptr, Fname, READWRITE, &status);
    if(status == 0){
      printf("%s exists.  Deleting...\n",Fname);
      fits_delete_file(fptr, &status);
    } else {
      status = 0;
      printf("%s doesn't exist.\n",Fname);
    }
    printf("Writing file:  %s\n",Fname);

    fits_get_system_date(&day, &month, &year, &status);
    fits_date2str(year, month, day, datestr, &status);

    /* Write the image and update the header.
     * This is a good time to convert the image into a
     * short for the rest of the processing.
     */
    //xshft = (xdim/2) - xc;
    //yshft = (ydim/2) - yc;
    //simg = (short *) malloc (nelements * sizeof(short));
    //printf ("Rotating for a pangle of:  %f\n",pangle);
    //simg = acos_frotimg(img, xdim, ydim, xc, yc, pangle, 0.0);
    //pangle = 0.00;
    //acos_shift(simg,img,xdim,ydim,xshft, yshft);

    //xc = xdim/2;
    //yc = ydim/2;
    // acos_sharpen(simg,xdim,ydim,50);

    fits_create_template(&fptr, Fname, ImName, &status);
    fits_update_key(fptr, TINT, "BITPIX", &bitpix, "16-bit SIGNED INTEGER", &status);
    fits_update_key(fptr, TFLOAT, "CRPIX1", &xc, NULL, &status);
    fits_update_key(fptr, TFLOAT, "CRPIX2", &yc, NULL, &status);
    //fits_update_key(fptr, TFLOAT, "CROTA1", &pangle, "CORRECTED FOR P-ANGLE", &status);
    //fits_update_key(fptr, TFLOAT, "CROTA2", &pangle, "CORRECTED FOR P-ANGLE", &status);
    fits_update_key(fptr, TSTRING, "TYPE-DP", "LDC", "FLAT FIELDED, LD CORRECTED", &status);
    fits_update_key(fptr, TSTRING, "CODE-OBS", "HSL", NULL, &status);
    fits_update_key(fptr, TFLOAT, "CRRADIUS", &rx, NULL, &status);
    fits_update_key(fptr, TSTRING, "DATE-CAL", &datestr, NULL, &status);
    //fits_update_key(fptr, TSTRING, "COORDNAM", "HELIOCENTRIC", NULL, &status);

    fits_set_hdustruc(fptr, &status);
    fits_write_img(fptr, TSHORT, fpixel, nelements, simg, &status);
    /*
    fits_update_key(fptr, TINT, "DATAMIN", &datamin, NULL, &status);
    fits_update_key(fptr, TINT, "DATAMAX", &datamax, NULL, &status);
    fits_update_key(fptr, TINT, "DISPMIN", &dispmin, NULL, &status);
    fits_update_key(fptr, TINT, "DISPMAX", &dispmax, NULL, &status);
    */
    fits_close_file(fptr, &status);

    fits_report_error(stderr, status);

    stop = time(NULL);

    seconds = difftime(stop,start);
    hours = seconds/3600.;
    if (hours < 1) {hours = 0.0;}
    mins=seconds/60.;
    if (mins < 1) { mins = 0.0;}
    secs = (int) seconds % 60;
    printf("\n\tTime Elapsed: %02.0f:%02.0f:%05.2f\n\n",hours,mins,secs);

 }//endfor

 printf("<========= Finished ==========>\n\n");
}
