;+
; pmdate1.pro   IDL procedure
;
; PURPOSE
;	Read a FITS image and change the date. 
;
; SYNTAX
;	pmdate1, 'fits_list'
;
; PROCEDURES USED
;	readfits
;	writefits
;	fxaddpar
;
; HISTORY
;	Author: Andrew L. Stanger   16 October 1997
;-

pro pmdate1, fits_list

fits_name = ''				; Define FITS file name variable.

CLOSE, 1

print, 'fits_list: ', fits_list

OPENR, 1, fits_list			; Open list file.

WHILE (NOT EOF (1)) DO	$		; File processing loop.
   BEGIN
      READF, 1, fits_name		; Read FITS file name from list file.
      img = readfits (fits_name, hdu)	; Read FITS image & header.

      date = '17/02/89'				; 17 Feb 1989 = DOY 48 1989
      pmon_time = fxpar (hdu, 'TIME-OBS')
      hour   = strmid (pmon_time, 0, 2)
      minute = strmid (pmon_time, 3, 2)
      second = strmid (pmon_time, 6, 2)
      time = hour + ':' + minute + ':' + second
      date_time = date + ' ' + time

      ang = PB0R (date_time, /ARCSEC) ;
      rsun = ang (2)
      cdelt = 4.10
      pixrs = rsun / cdelt
      print, 'Rsun: ', rsun, '   [arcsec]'

      ; -- Modify FITS header.

      fxaddpar, hdu, 'CDELT1',   cdelt,      'SCALE SIZE: X AXIS [ARCSEC/PIXEL]'
      fxaddpar, hdu, 'CDELT2',   cdelt,      'SCALE SIZE: Y AXIS [ARCSEC/PIXEL]'
      fxaddpar, hdu, 'DATE-OBS', date,       'DAY/MONTH/YEAR'
      fxaddpar, hdu, 'TIME-OBS', pmon_time,  'HOUR:MINUTE:SECOND'
      fxaddpar, hdu, 'CRRADIUS', pixrs,      'SOLAR RADIUS [PIXELS]'
      fxaddpar, hdu, 'RSUN',     rsun,       'SOLAR RADIUS [ARCSEC]'

      ; -- Write FITS image to disk.

      writefits, fits_name, img, hdu

   END
CLOSE, 1

END
