/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * polyfill : fill polygon.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

short *
polyfill (nampgm, xocp, yocp, poly, nver, cmap, nwp, region)

char nampgm [];			/* Program name. */
int xocp, yocp;			/* C/P image origin. */
int poly [][2];			/* Polygon vertex array. */
int nver;			/* # vertices. */
int cmap;			/* Color map index. */
int *nwp;			/* # pixels inside polygon. */
int region;			/* Region index number. */

{
   short *wpos;			/* pixel array pointer. */
   static char namcpv [80];	/* C/P vertex file. */
   static char namver [80];	/* Window vertex file. */
   static char namper [80];	/* Perimeter file. */
   static char namyx  [80];	/* yxsort file. */
   static char namfil [80];	/* Fill file. */
   static char op [80];		/* String for system operation. */
   static char smap [4];	/* String for cmap. */

   int xcp, ycp;
   int xwin, ywin;
   int i;
   int ipix;
   int np;

   FILE *fd;			/* File descriptor. */
   FILE *fdver;			/* Vertex file descriptor. */

   sprintf (namcpv, "%s.cpv%d.ov", nampgm, region);
   sprintf (namver, "%s.ver%d.ov", nampgm, region);
   sprintf (namper, "%s.per%d.ov", nampgm, region);
   sprintf (namyx,  "%s.yx%d.ov",  nampgm, region);
   sprintf (namfil, "%s.fil%d.ov", nampgm, region);

   unlink (namcpv);
   unlink (namver);
   unlink (namper);
   unlink (namyx);
   unlink (namfil);

   color (cmap);		/* Set current color for drawing. */

/*
 *   bgnpolygon ();
 *      for (i=0; i<nver; i++)  v2i (poly [i]);
 *      v2i (poly [0]);
 *   endpolygon ();
 */

   if ((fd = fopen (namcpv, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namcpv);
      return (NULL);
      }

   if ((fdver = fopen (namver, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namver);
      return (NULL);
      }

   /*** Write polygon vertices to vertex file. ***/

   for (i=0; i<nver; i++)
      {
      xcp = poly [i][0];
      ycp = poly [i][1];

      xwin = xcp + xocp;
      ywin = yocp - ycp;

      fprintf (fd,    "%3d %3d\n", xcp,  ycp);
      fprintf (fdver, "%3d %3d\n", xwin, ywin);
      }

   xcp = poly [0][0] + 1;
   ycp = poly [0][1];

   xwin = xcp + xocp;
   ywin = yocp - ycp;

   fprintf (fd,    "%3d %3d\n", xcp,  ycp);	/* Repeat first vertex. */
   fprintf (fdver, "%3d %3d\n", xwin, ywin);	/* Repeat first vertex. */

   fclose (fd);
   fclose (fdver);

   /*** Open/close perimeter file. ***/

   if ((fd = fopen (namper, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namper);
      return (NULL);
      }
   fclose (fd);

   /*** Open/close yxsort file. ***/

   if ((fd = fopen (namyx, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namyx);
      return (NULL);
      }
   fclose (fd);

   /*** Open/close fill file. ***/

   if ((fd = fopen (namfil, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namfil);
      return (NULL);
      }
   fclose (fd);

   /*** Execute "makeov" command (WHIP). ***/

   sprintf (op, "makeov < %s > %s", namver, namper);
   system (op);
   fprintf (stderr, "makeov done\n");

   /*** Execute "drawov" command (WHIP). ***/

   sprintf (op, "drawov -g %d %s", cmap, namper);
   system (op);
   fprintf (stderr, "drawov done\n");

   /*** Execute "yxsort" command (WHIP). ***/

   sprintf (op, "yxsort %s %s", namper, namyx);
   system (op);
   fprintf (stderr, "yxsort done\n");

/*
 *   sprintf (op, "fill -g %d %s", cmap, namyx);
 *   system (op);
 *   fprintf (stderr, "fill   done\n");
 */

   /*** Execute "fill" command (WHIP). ***/

   sprintf (op, "fill -xy -g %d %s > %s", cmap, namyx, namfil);
   system (op);
   fprintf (stderr, "fill   done\n");

   /*** Read "fill" file to count the number of coordinate pairs. ***/

   fd = fopen (namfil, "r");
   *nwp = 0;
   while (fscanf (fd, "%d %d", &xwin, &ywin) != EOF) (*nwp)++;
   fclose (fd);
   printf ("polyfill> nver: %d   nwp: %d\n", nver, *nwp);

   /*** Reopen "fill" file and read pixel coordinates into array "wpos". ***/

   wpos = (short *) malloc (2 * *nwp * sizeof (short));
   fd = fopen (namfil, "r");
   for (ipix =  0; ipix < *nwp; ipix++)
      {
      fscanf (fd, "%d %d", &xwin, &ywin);
      *(wpos + ipix * 2) = (short) xwin;
      *(wpos + ipix * 2 + 1) = (short) ywin;
      }
   fclose (fd);

   return (wpos);
}
