/*  This is a program to separate out i, q, u data from a raw mk4 files.
    See mk4head_raw.h for a raw data record description. 
    See mk4headv2.h for the reformatted header.
    See mk4v2.h for various definitions.

    This code was written by Alice Lecinski, September 2001.

    I strongly suggest *not* editting this code at all, unless
    you are absolutely sure you've found an error that
    needs fixing.

    If you edit it, you risk destroying the integrity of the
    code.  So if you do edit it, insure results pre and post
    actually match.

*/



#include <stdio.h>
#include <ctype.h>
#include <math.h>

#include "mk4v2.h"
#include "ephem.h"
#include "mk4headv2.h"
#include "mk4head_raw.h"

#include "mk4InsDesc.h"


char usage[] = "\
Gets i,q,u data from a \"raw\" mk4 file...\n\
\n\
The   -R    flag will leave the data RAW, NOT REVERSING negative scans.\n\
\n\
rawt4v2 -R rawfile1 rawfile2 ... rawfileN \n\
rawt4v2    rawfile1 rawfile2 ... rawfileN \n\
rawt4v2    99d124.17_45.rawmk4 ... 99d124.22_10.rawmk4 \n\
";

extern char *malloc();

int direction;
int thetamin,     thetamax;
int raw_thetamin, raw_thetamax;
float SkyTx;
float stmin,stmax;

short buf[RECSZ];   /* 1184 numbers  , 2368 bytes */

short i0[CY_IMG] ;
short q0[CY_IMG] ;
short u0[CY_IMG] ;

short rev[CY_IMG];

short *psk , sk[VECTOR] ;
short *pba , ba[VECTOR] ;
short *pst , st[VECTOR] ;

int recno;

int   skyarray[VECTOR] ;
int   indx    [VECTOR] ;

main (an, av) char *av[]; {
    register int i, j , ii,jj,kk ;
    register char *p ;
    register TH thd ;
    short *a , *b , *c , *bb , *qq, *rr;
    int nray;
    short     *myDAT[3];  /* 3 data types (iqu) */
    int who,howfar;
    int doREVERSE;
    FILE *ifid ;

    howfar   = IS_raw;
    doREVERSE=1; /* Normally, reverse negative scans. */

    dayflag=doyflag=timeflag=1;
    siteLongitude *= d2r;
    siteLatitude  *= d2r;


    if ( an <= 1 ) { prsum() ; exit(-5) ; }
    myDAT[0]=i0; 
    myDAT[1]=q0;
    myDAT[2]=u0;

    while (--an) {
	if (*(p = *++av) == '-') {	/* flag arguments */
	    switch (p[1]) {
		case 'R':  /* Leave data RAW, don't reverse negative scans */
                           doREVERSE=0; 
	   	           break;
		default:   fprintf (stderr," unknown flag %s\n",*av);
			   prsum(); exit(1);
	    }
	    continue;
	}
	else {
	    if (*p == '?') { prsum(); exit(2); }
	    if ((ifid = fopen (p, "r")) == NULL) {
		fprintf (stderr,"Could not open tape file %s\n",p);
		exit (1);
	    }
	    fprintf(stderr,"\nReading %s\n",p);
	}
	psk = sk ;
	pba = ba ;
	pst = st ;
	recno=2;
	for (i = 0 ;i<=recno; i++) {
	    if ((j = fread (buf, sizeof (short), RECSZ , ifid)) > 0) {
		thd = (struct tapehead *) buf; 
		if (j != (thd->comprs ? 2776  : RECSZ ))
		    fprintf (stderr,"Wrong size read record %d %d bytes\n",
			     i,  j);

		if ( i == 0 )     recno = thd->rec_no;
		if ( recno == 0 ) recno = 985;
		if ( i == recno ) goto breakout;

                /* Re-assemble the time, leave it as Hz. */
                tim = thd->tod_msw << 16 | (unsigned short) thd->tod_lsw;

		bb = &buf[32] ;
		a = i0+i*NPX ; b = q0+i*NPX ; c = u0+i*NPX ;
		for(ii=0;ii<NPX;ii++){ *a++ =*bb++; *b++ =*bb++; *c++ =*bb++;}

		*psk++ = thd->sky;
		*pba++ = thd->b_ang;
		*pst++ = thd->scan_time;

	    }
	    else if (j != 0) {
		fprintf (stderr,"error %d  on record %d \n",j,i);
		exit (3); }
	    else {
breakout:
		fprintf(stderr,"    Number of records --> %d\n",i);

		nray=i; direction=0;

		getsky(&SkyTx,nray,sk, thd,mk4h);
		stmin = *st;  
		stmax = *(st+nray-1);
		thetamin     = *ba          *ba2deg;
		thetamax     = *(ba+nray-1) *ba2deg;
		raw_thetamin = *ba          *ba2deg;
		raw_thetamax = *(ba+nray-1) *ba2deg;


		if(*ba < *(ba+4))  { 
		    direction =  1;
		}
		else               {
		    direction = -1;
		    if(doREVERSE==1) {/*Reverse the order of backward images. */
			for(jj=0;jj<3;jj++){
			    ReverseScan(NPX,nray,myDAT[jj]);
			}
		        /* Reverse the sk , ba , and st */
			ReverseScan(1,nray,sk);
			ReverseScan(1,nray,ba);
			ReverseScan(1,nray,st);

		        thetamin     = *ba           *ba2deg;
		        thetamax     = *(ba+nray-1)  *ba2deg;
		    }
		}

		/* Make "i" "q" "u" images Theta by R arrays */
		for(jj=0;jj<3;jj++){
		    Rtheta2thetaR(nray,myDAT[jj]);
		}


		for(jj=0;jj<3;jj++){
		    qq=myDAT[jj];
		    who   =IS_i+jj;
		    fillmk4hdr(thd,mk4h,nray,who,howfar);
		    dopicnm (mk4h);
		    dofnm (who,howfar);
		    writemk4(picnm,mk4h,qq,1);
		}
		fprintf (stderr,"\t    SkyTx  = %7.2f\n",mk4h->SkyTx);
		fprintf (stderr,"\t    pangle = %7.2f\n",mk4h->Pangle);

		who   = IS_sk;
		fillmk4hdr(thd,mk4h,nray, who, howfar);
		dofnm(who,howfar);
		writemk4(picnm,mk4h,sk,1);

		who   = IS_ba;
		fillmk4hdr(thd,mk4h,nray, who, howfar);
		dofnm(who,howfar);
		writemk4(picnm,mk4h,ba,1);

		who   = IS_st;
		fillmk4hdr(thd,mk4h,nray, who, howfar);
		dofnm(who,howfar);
		writemk4(picnm,mk4h,st,1);

		break;
	    }
	}
	fclose(ifid) ;
    }
    exit (0);
}
prsum () {
    fprintf (stderr,usage);
}

int
fillmk4hdr(thd,mk4h,nray, who, howfar)
   register TH  thd;
   register FH  mk4h;
   int nray;
   int who, howfar;
{
    float sec;
    double dr , r0 , Rratio;
    struct InstrDescrip *idd;
    int doprint=0;

    filltime   (thd,mk4h);
    fillephem  (    mk4h);
    fillInsDesc(thd,mk4h);

    idd = &(mk4h->ID);

    /* Determine pixel scaling. */
    dr = idd->dr ;
    r0 = idd->r0 ;  
    Rratio = 960. / ephSRadius ;
    r0 *= Rratio;
    dr *= Rratio;
 
    mk4h->Xdim     = nray;
    switch ( who ) {
        case IS_ba:
        case IS_st:
        case IS_sk: mk4h->Ydim=  1; break;
        default:    mk4h->Ydim=NPX; 
    }
    mk4h->Duration    = stmax-stmin;
    mk4h->JuldRef     = JULD_REF;
    mk4h->ImgDataType = SHORTS;
    mk4h->Pangle      = (float)ephPangle;
    mk4h->Bangle      = (float)ephBangle;
    mk4h->Langle      = (float)ephLangle;
    mk4h->RA          = (float)ephRA;
    mk4h->Dec         = (float)ephDEC;
    mk4h->HA          = (float)ephHA;
    mk4h->Elevation   = (float)ephElev;
    mk4h->Azimuth     = (float)ephAz;
    mk4h->Amass       = (float)ephAmass;
    mk4h->Radius      = (float)ephSRadius;
    mk4h->SkyTx       = SkyTx;

    /* For both old and new camera, MAY01_2001,
       InstrPhs is calculated in getcal4v2.c    
    */
    mk4h->InstrPhs =     0.0; 


    mk4h->spare1               = 0.     ;
    mk4h->data_scale           = 1.     ;
    mk4h->data_offset          = 0.     ;
    mk4h->spare2               = 0.     ;
    mk4h->orig_hairspot[0] = mk4h->orig_hairspot[1] = badval ;
    mk4h->hairspot     [0] = mk4h->hairspot     [1] = badval ;

    mk4h->Xcen     = badval ;
    mk4h->Ycen     = badval ;
    mk4h->pixscl   = badval ;
    mk4h->r0       = r0;
    mk4h->dr       = dr;
    mk4h->rmax     = r0 + (NPX-1.)*dr;
    mk4h->theta_min=thetamin;
    mk4h->theta_max=thetamax;
    mk4h->raw_theta_min=raw_thetamin;
    mk4h->raw_theta_max=raw_thetamax;
    
    if(direction== -1) mk4h->backlash = -1.*BACKLASH;
    if(direction==  1) mk4h->backlash =     BACKLASH;

    mk4h->Phase    =  30.0;
    
    mk4h->ObsType  =thd->tilt; /* calib. or coronal */
    mk4h->DataType =who;       /* IS_i , IS_q , IS_u  etc */
    mk4h->ProcSteps=howfar;    /* IS_raw through IS_cal_avg_rect_slit */
    mk4h->Direction=direction; /* "direction" of original raw data
                                   -1==reverse, +1 forward.*/
    mk4h->Nray     =nray;    /* Number of barrel angle positions (rays)
                                in original raw data. */
    mk4h->Pixels   =NPX;     /* Number of pixels in original raw data. */
 
    mk4h->Date_calibrated.Year    =0;
    mk4h->Date_calibrated.Month   =0;
    mk4h->Date_calibrated.Day     =0;
    mk4h->Date_calibrated.Hour    =0;
    mk4h->Date_calibrated.Minute  =0;
    mk4h->Date_calibrated.CentiSec=0;  
    mk4h->Date_calibrated.Juld    =0;   /* Julian day - JULD_REF */
    mk4h->Date_calibrated.Doy     =0;
    mk4h->Date_calibrated.Time    =0;
    sprintf(mk4h->tscpfile, "\0");  /* filename of tscp file. */
    sprintf(mk4h->reffile,  "\0");  /* filename of ref file. */
    sprintf(mk4h->version,  "Version 2");  /* Code version */
    sprintf(mk4h->AvgTimes, "\0");  /* If avg img, these are the img times
                                       used to make the avg. */
}


/* Gets the most common reasonable sky transmission.  */
/* Same as David's "skyis.pro" as of Jan 18, 2000 */

/* Turns out that the sky transmission reported in the raw mk4 "tape" header
   used to be sampled funny.  It would not report the
   maximum, but an accending or decending value.
   It would get phased locked, so each 
   barrel angle sould report the misleading value.

   So, prior to May 14, 2001 compute the SkyTx from the airmass.
   (David Elmore computed the conversion based on data 14-17 May 2001)
   After May 14 2001 (inclusive), use the maximum value found from all
   barrel angles.

   Eventually, the skytx instrument at MLSO will be fixed to 
   obtain the real skytx at each ba, and then, this skytx vector
   will be used to refine the calibration further.  As presently
   the calibration just used the one skytx point obtained
   below.

   Also, if the sampled SkyTx is < LOWSKY (300)
   replace it with that calculated from the AirMass.

*/
   

int
getsky  (sky , nray, sk, thd, mk4h) 
   float *sky; 
   int nray; 
   short *sk;
   register TH  thd;
   register FH  mk4h;
{
    int   ii;
    int  *dx, sdim, histmax, mxsky;
    int   isky , *ps;
    int   UseMax;
    float Amass_slope, Amass_ycept, AmassSkyTx;

    UseMax   = 1 ;  /* If ==1,  use the maximum skytx found. */
		    /* If ==0,  use the most frequent skytx found. */

    Amass_slope = AMASS_SLOPE;
    Amass_ycept = AMASS_YCEPT;


    filltime (thd,mk4h);
    fillephem(    mk4h);

    fprintf(stderr,"\t Image.Juld: %d\n",mk4h->Image.Juld);

    AmassSkyTx = Amass_slope * ephAmass  + Amass_ycept;

    if (mk4h->Image.Juld < MAY14_2001 ) {
	/* Before May 14, 2001 use skytx computed from airmass. */
	*sky = AmassSkyTx;
	fprintf(stderr,"\t SkyTx : %8.3f  Amass : %8.3f\n",
	    *sky,ephAmass);
    }
    else {


	/* Make a copy if the sky data. */
	ps = skyarray;
	for(ii=0;ii<nray;ii++){
	   *ps = (int)*sk;
	   ps++;
	   sk++;
	}

	/* Fix any negative sky values. */
	/* Get the maximum along the way. */
	ps    = skyarray;
	mxsky = 0;
	for(ii=0;ii<nray;ii++){
	    isky  = (*ps  <    0  ) ? MINSKY : *ps   ;
	    mxsky = (isky > mxsky ) ? isky   : mxsky ;
	    *ps  = isky ;
	    ps++;
	}

	if(UseMax==1) {
	    isky = mxsky;
	}
	else {
	    /* The dimension of the indx array, rounded up */
	    sdim = mxsky+1 ;

	    /* Zero the index array which will hold the number
	       of occurrences of each sky value. */
	    dx = indx;
	    for (ii=0;ii<sdim;ii++) { *dx++ = 0 ; }


	    /* Create a histogram of the skyarray. */
	    ps = skyarray;
	    for(ii=0;ii<nray;ii++){
		*(indx + (*ps)) += 1;
		ps++;
	    }
	    /* Find the position of the maximum of the histogram. */
	    dx      = indx;
	    histmax = 0;
	    for(ii=0;ii<sdim;ii++){
		if ( *dx > histmax ) { 
		    histmax = *dx;
		    isky  = ii;
		}
		dx++;
	    }
	}

	*sky = (float)isky;

	if (*sky<LOWSKY) {
	    *sky = AmassSkyTx;
	    fprintf(stderr,"\t SkyTx : %8.3f  Amass : %8.3f\n",
		*sky,ephAmass);
	}
	else {
	    fprintf(stderr,"\t SkyTx : %8.3f\n",*sky);
	}

    }

}



int
Rtheta2thetaR(mylen,psq)
    int  mylen;
    short *psq;
{
    int ii,jj;
    short *pq;
    short *pu;


    /* Rotate it to a theta by R array. */
    pq=psq; pu=rev;
    for(ii=0;ii<NPX;ii++) {  
	pq = psq + ii;
	for(jj=0;jj<mylen;jj++) { *pu++ = *pq; pq+=NPX; }
    }
    /* Copy it back. */
    pq = psq; pu=rev;
    for(ii=0;ii<NPX*mylen;ii++)   *pq++ = *pu++; 
}

int
ReverseScan(pixl,rays,qq)
    int  pixl,rays;
    short *qq;
{
    int ii,kk;
    short *pq;
    short *rr;

    if ( pixl==1 ) {
	pq=qq; rr=rev+rays-1; for(ii=0;ii<rays;ii++) *rr-- = *pq++;
    }
    else {
	pq=qq;
	rr=rev+rays*pixl -pixl;
	for(ii=0;ii<rays;ii++){
	    for(kk=0;kk<pixl;kk++) *rr++ = *pq++;
	    rr-=pixl;
	    rr-=pixl;
	}   
    }

    pq=qq;
    rr=rev;
    for(ii=0;ii<rays*pixl;ii++) *pq++ = *rr++;

}

int
filltime(thd,mk4h)
   register TH  thd;
   register FH  mk4h;
{
    float sec;
    int doprint=0;

    sin_Lat = sin(siteLatitude);
    cos_Lat = cos(siteLatitude);
 
    mk4h->Image.Year     = thd->yr +1900; /* Ok even after 2000 */
    mk4h->Image.Doy      = thd->doy;
    idate (  mk4h->Image.Doy,
           &(mk4h->Image.Month),&(mk4h->Image.Day),
             mk4h->Image.Year);
    get_hrminsec(tim,&(mk4h->Image.Hour),&(mk4h->Image.Minute),&sec);
    mk4h->Image.CentiSec = (int)(sec*100.);
    mk4h->Image.Juld     = ((jday (mk4h->Image.Month, 
				   mk4h->Image.Day, 
				   mk4h->Image.Year) - JULD_REF));
    mk4h->Image.Time     = tim;

}

int
fillephem(mk4h)
   register FH  mk4h;
{
    int doprint=0;

    sin_Lat = sin(siteLatitude);
    cos_Lat = cos(siteLatitude);
 
    Eph_year  =mk4h->Image.Year;
    Eph_month =mk4h->Image.Month;
    Eph_day   =mk4h->Image.Day; 
    Eph_hour  =mk4h->Image.Hour;
    Eph_minute=mk4h->Image.Minute;

    Eph_second=(int)(0.5005 +  0.01 * ((float)mk4h->Image.CentiSec));

    if (doprint==1)
	fprintf(stderr," %d %d %d %d %d %d\n\n",
	Eph_year,Eph_month ,Eph_day ,Eph_hour,Eph_minute,Eph_second);

    MRSUN_Isat();

    if (doprint==1)
	fprintf(stderr,"%lf B0, %lf L0, %lf P, %lf DIAM , %lf RA , %lf DEC \n",
	ephBangle, ephLangle , ephPangle, ephSRadius*2. , ephRA,ephDEC ) ;

}


int
fillInsDesc(thd,mk4h)
   register TH  thd;
   register FH  mk4h;
{
    struct InstrDescrip *pID;
    struct InstrDescrip *pid;

    double myjuld, ov60, ov24;
    int ii;
    int doprint=0;

    ov60 = 1./60. ;
    ov24 = 1./24. ;



    myjuld = (double)((mk4h->Image.Juld) 
	             + ov24*(            (double)(mk4h->Image.Hour)
	                     +      ov60*(double)(mk4h->Image.Minute)
	                     + ov60*ov60*(double)(mk4h->Image.CentiSec)*0.01
		            )
	     );
     

    pID = &(mk4h->ID);
    pid = InDe;

    ii=0;
    for(ii=0;ii<NUM_IDS;ii++) {
	if (  ( myjuld <  pid->JuldRange[0] ) 
	    ||( myjuld >  pid->JuldRange[1] )) {
	    pid++;
	}
	else {
	    pID->JuldRange[0] = pid->JuldRange[0] ;
	    pID->JuldRange[1] = pid->JuldRange[1] ;
	    pID->Bopal        = pid->Bopal        ;
	    pID->trt          = pid->trt          ;
	    pID->fpt          = pid->fpt          ;
	    pID->IDspare      = pid->IDspare      ;
	    pID->kappa        = pid->kappa        ;
	    pID->BarrelOffset = pid->BarrelOffset ;
	    pID->hairspot[0]  = pid->hairspot[0]  ;
	    pID->hairspot[1]  = pid->hairspot[1]  ;
	    pID->r0           = pid->r0           ;
	    pID->dr           = pid->dr           ;
	    pID->goodRmin     = pid->goodRmin     ;
	    pID->goodRmax     = pid->goodRmax     ;
	    pID->biasspot[0]  = pid->biasspot[0]  ;
	    pID->biasspot[1]  = pid->biasspot[1]  ;

	    pID->ZeroSky       = pid->ZeroSky       ;
	    pID->VenusCalFac   = pid->VenusCalFac   ;
	    pID->ExtraCalFac   = pid->ExtraCalFac   ;
	    pID->DiffractFac   = pid->DiffractFac   ;
	    pID->DiffractPix   = pid->DiffractPix   ;
	    pID->PhiPix        = pid->PhiPix        ;
	    pID->PolarityCkPix = pid->PolarityCkPix ;
	    pID->NegPix[0]     = pid->NegPix[0]     ;
	    pID->NegPix[1]     = pid->NegPix[1]     ;
	    pID->WrapPix[0]    = pid->WrapPix[0]    ;
	    pID->WrapPix[1]    = pid->WrapPix[1]    ;

	    pID->RingSearch[0] = pid->RingSearch[0] ;
	    pID->RingSearch[1] = pid->RingSearch[1] ;

	    pID->RingFixPix    = pid->RingFixPix    ;


	    sprintf(pID->CorrectFile0,"%s",pid->CorrectFile0);
	    sprintf(pID->CorrectFile1,"%s",pid->CorrectFile1);

	    break;
	}
    }
    if (ii==NUM_IDS) {
	fprintf(stderr,"Never found the correct InsDesc\n");
	exit(-92);
    }


    if (doprint==1) {
	fprintf(stderr,"\n") ;
    }

}
