/* rdacos.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read MLSO/ACOS image from disk to memory.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   27 November 1995
 * 8 Mar 96: Remove first line of image, which contains binary header.
 *           Oops.  Keep binary header as top line of image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/puu/cordyn/mlsofits/lib/acos/acos.h"
#include "/home/puu/cordyn/mlsofits/lib/acos/acos_info.h"
#include "/home/puu/cordyn/mlsofits/lib/acos/acos_header.h"
#include "/home/puu/cordyn/mlsofits/lib/acos/room.h"

unsigned short *
rdacos (namimg, pmode)

char namimg [] ;		/* ACOS file name. */
int pmode ;			/* Print mode: 0=silent, 1=verbose. */

{
   extern int acoshskp () ;		/* ACOS housekeeping. */
   extern char *malloc () ;		/* Memory allocation. */
   FILE *FD ;
   int ix, iy ;
   int nbyimg ;
   int nbypix ;
   int npximg ;
   int nx, ny ;
   int xdim, ydim ;
   unsigned short *sbuf ;		/* Short int buffer pointer. */

   /* Alice has chaged this a bit... Since  the file is opened in
      acoshskp, and then never closed (ooops) might as well truely leave it
      open, and pass FD (the file descriptor) back and forth.
      There's no reason to open the file twice, and no reason to read 
      the header twice.

      OLDWAY ----in this code
	    acoshskp (namimg, pmode) ;
	    FD = fopen (namimg, "r") ;
	    fread ((char *) &ACOSHS, sizeof (ACOSHS), 1, FD);
   */

   
   /* Read the ACOSHS header and then fill, up the ACOSIS  (ACOSI) structure. */
   /*** ACOSHS is where the memory is allocated, ACOSH is the pointer. */
   /*** ACOSIS is where the memory is allocated, ACOSI is the pointer. */

   /* NEWWAY */
   /*** Open ACOS file. ***/

   if ((FD = fopen (namimg, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namimg) ;
      }

   acoshskp (FD, namimg, pmode) ;

   if (pmode)
      {
      printf ("Version:     %10d\n",   ACOSH->Version) ;
      printf ("Year:        %10d\n",   ACOSH->Year) ;
      printf ("Month:       %10d\n",   ACOSH->Month) ;
      printf ("Day:         %10d\n",   ACOSH->Day) ;
      printf ("Hour:        %10d\n",   ACOSH->Hour) ;
      printf ("Minute:      %10d\n",   ACOSH->Minute) ;
      printf ("Sec:         %10d\n",   ACOSH->Sec) ;
      printf ("Xdim:        %10d\n",   ACOSH->Xdim) ;
      printf ("Ydim:        %10d\n",   ACOSH->Ydim) ;
      printf ("Nsum:        %10d\n",   ACOSH->Nsum) ;
      printf ("Exposure:    %10d\n",   ACOSH->Exposure) ;
      printf ("Gain:        %10d\n",   ACOSH->Gain) ;
      printf ("Odisk:       %10d\n",   ACOSH->Odisk) ;
      printf ("Type:        %10d\n",   ACOSH->Type) ;
      printf ("Pangle:      %10.3f\n", ACOSH->Pangle) ;
      printf ("Bangle:      %10.3f\n", ACOSH->Bangle) ;
      printf ("Langle:      %10.3f\n", ACOSH->Langle) ;
      printf ("HA:          %10.3f\n", ACOSH->HA) ;
      printf ("Elevation:   %10.3f\n", ACOSH->Elevation) ;
      printf ("Azimuth:     %10.3f\n", ACOSH->Azimuth) ;
      printf ("Amass:       %10.3f\n", ACOSH->Amass) ;
      printf ("Radius:      %10.3f\n", ACOSH->Radius) ;
      printf ("Xshift[0]:   %10.3f\n",   ACOSH->Xshift[0]) ;
      printf ("Xshift[1]:   %10.3f\n",   ACOSH->Xshift[1]) ;
      printf ("Xshift[2]:   %10.3f\n",   ACOSH->Xshift[2]) ;
      printf ("Xshift[3]:   %10.3f\n",   ACOSH->Xshift[3]) ;
      printf ("Xshift[4]:   %10.3f\n",   ACOSH->Xshift[4]) ;
      printf ("Xshift[5]:   %10.3f\n",   ACOSH->Xshift[5]) ;
      printf ("Xshift[6]:   %10.3f\n",   ACOSH->Xshift[6]) ;
      printf ("Xshift[7]:   %10.3f\n",   ACOSH->Xshift[7]) ;
      printf ("Xshift[8]:   %10.3f\n",   ACOSH->Xshift[8]) ;
      printf ("Xshift[9]:   %10.3f\n",   ACOSH->Xshift[9]) ;
      printf ("Xshift[10]:  %10.3f\n",   ACOSH->Xshift[10]) ;
      printf ("Xshift[11]:  %10.3f\n",   ACOSH->Xshift[11]) ;
      printf ("Xshift[12]:  %10.3f\n",   ACOSH->Xshift[12]) ;
      printf ("Xshift[13]:  %10.3f\n",   ACOSH->Xshift[13]) ;
      printf ("Xshift[14]:  %10.3f\n",   ACOSH->Xshift[14]) ;
      printf ("Xshift[15]:  %10.3f\n",   ACOSH->Xshift[15]) ;
      printf ("Yshift[0]:   %10.3f\n",   ACOSH->Yshift[0]) ;
      printf ("Yshift[1]:   %10.3f\n",   ACOSH->Yshift[1]) ;
      printf ("Yshift[2]:   %10.3f\n",   ACOSH->Yshift[2]) ;
      printf ("Yshift[3]:   %10.3f\n",   ACOSH->Yshift[3]) ;
      printf ("Yshift[4]:   %10.3f\n",   ACOSH->Yshift[4]) ;
      printf ("Yshift[5]:   %10.3f\n",   ACOSH->Yshift[5]) ;
      printf ("Yshift[6]:   %10.3f\n",   ACOSH->Yshift[6]) ;
      printf ("Yshift[7]:   %10.3f\n",   ACOSH->Yshift[7]) ;
      printf ("Yshift[8]:   %10.3f\n",   ACOSH->Yshift[8]) ;
      printf ("Yshift[9]:   %10.3f\n",   ACOSH->Yshift[9]) ;
      printf ("Yshift[10]:  %10.3f\n",   ACOSH->Yshift[10]) ;
      printf ("Yshift[11]:  %10.3f\n",   ACOSH->Yshift[11]) ;
      printf ("Yshift[12]:  %10.3f\n",   ACOSH->Yshift[12]) ;
      printf ("Yshift[13]:  %10.3f\n",   ACOSH->Yshift[13]) ;
      printf ("Yshift[14]:  %10.3f\n",   ACOSH->Yshift[14]) ;
      printf ("Yshift[15]:  %10.3f\n",   ACOSH->Yshift[15]) ;
      printf ("Diam[0]:     %10.3f\n",   ACOSH->Diam[0]) ;
      printf ("Diam[1]:     %10.3f\n",   ACOSH->Diam[1]) ;
      printf ("Diam[2]:     %10.3f\n",   ACOSH->Diam[2]) ;
      printf ("Diam[3]:     %10.3f\n",   ACOSH->Diam[3]) ;
      printf ("Diam[4]:     %10.3f\n",   ACOSH->Diam[4]) ;
      printf ("Diam[5]:     %10.3f\n",   ACOSH->Diam[5]) ;
      printf ("Diam[6]:     %10.3f\n",   ACOSH->Diam[6]) ;
      printf ("Diam[7]:     %10.3f\n",   ACOSH->Diam[7]) ;
      printf ("Diam[8]:     %10.3f\n",   ACOSH->Diam[8]) ;
      printf ("Diam[9]:     %10.3f\n",   ACOSH->Diam[9]) ;
      printf ("Diam[10]:    %10.3f\n",   ACOSH->Diam[10]) ;
      printf ("Diam[11]:    %10.3f\n",   ACOSH->Diam[11]) ;
      printf ("Diam[12]:    %10.3f\n",   ACOSH->Diam[12]) ;
      printf ("Diam[13]:    %10.3f\n",   ACOSH->Diam[13]) ;
      printf ("Diam[14]:    %10.3f\n",   ACOSH->Diam[14]) ;
      printf ("Diam[15]:    %10.3f\n",   ACOSH->Diam[15]) ;
      printf ("Pos[0]:      %10.3f\n",   ACOSH->Pos[0]) ;
      printf ("Pos[1]:      %10.3f\n",   ACOSH->Pos[1]) ;
      printf ("Pos[2]:      %10.3f\n",   ACOSH->Pos[2]) ;
      printf ("Pos[3]:      %10.3f\n",   ACOSH->Pos[3]) ;
      printf ("Pos[4]:      %10.3f\n",   ACOSH->Pos[4]) ;
      printf ("Pos[5]:      %10.3f\n",   ACOSH->Pos[5]) ;
      printf ("Pos[6]:      %10.3f\n",   ACOSH->Pos[6]) ;
      printf ("Pos[7]:      %10.3f\n",   ACOSH->Pos[7]) ;
      printf ("Pos[8]:      %10.3f\n",   ACOSH->Pos[8]) ;
      printf ("Pos[9]:      %10.3f\n",   ACOSH->Pos[9]) ;
      printf ("Pos[10]:     %10.3f\n",   ACOSH->Pos[10]) ;
      printf ("Pos[11]:     %10.3f\n",   ACOSH->Pos[11]) ;
      printf ("Pos[12]:     %10.3f\n",   ACOSH->Pos[12]) ;
      printf ("Pos[13]:     %10.3f\n",   ACOSH->Pos[13]) ;
      printf ("Pos[14]:     %10.3f\n",   ACOSH->Pos[14]) ;
      printf ("Pos[15]:     %10.3f\n",   ACOSH->Pos[15]) ;
      printf ("Pos[16]:     %10.3f\n",   ACOSH->Pos[16]) ;
      printf ("Pos[17]:     %10.3f\n",   ACOSH->Pos[17]) ;
      printf ("Pos[18]:     %10.3f\n",   ACOSH->Pos[18]) ;
      printf ("Pos[19]:     %10.3f\n",   ACOSH->Pos[19]) ;
      printf ("Pos[20]:     %10.3f\n",   ACOSH->Pos[20]) ;
      printf ("Pos[21]:     %10.3f\n",   ACOSH->Pos[21]) ;
      printf ("Pos[22]:     %10.3f\n",   ACOSH->Pos[22]) ;
      printf ("Pos[23]:     %10.3f\n",   ACOSH->Pos[23]) ;
      printf ("Pos[24]:     %10.3f\n",   ACOSH->Pos[24]) ;
      printf ("Pos[25]:     %10.3f\n",   ACOSH->Pos[25]) ;
      printf ("Pos[26]:     %10.3f\n",   ACOSH->Pos[26]) ;
      printf ("Pos[27]:     %10.3f\n",   ACOSH->Pos[27]) ;
      printf ("Pos[28]:     %10.3f\n",   ACOSH->Pos[28]) ;
      printf ("Pos[29]:     %10.3f\n",   ACOSH->Pos[29]) ;
      printf ("Pos[30]:     %10.3f\n",   ACOSH->Pos[30]) ;
      printf ("Pos[31]:     %10.3f\n",   ACOSH->Pos[31]) ;
      printf ("Drive[0]:      %10.3f\n",   ACOSH->Drive[0]) ;
      printf ("Drive[1]:      %10.3f\n",   ACOSH->Drive[1]) ;
      printf ("Drive[2]:      %10.3f\n",   ACOSH->Drive[2]) ;
      printf ("Drive[3]:      %10.3f\n",   ACOSH->Drive[3]) ;
      printf ("Drive[4]:      %10.3f\n",   ACOSH->Drive[4]) ;
      printf ("Drive[5]:      %10.3f\n",   ACOSH->Drive[5]) ;
      printf ("Drive[6]:      %10.3f\n",   ACOSH->Drive[6]) ;
      printf ("Drive[7]:      %10.3f\n",   ACOSH->Drive[7]) ;
      printf ("Drive[8]:      %10.3f\n",   ACOSH->Drive[8]) ;
      printf ("Drive[9]:      %10.3f\n",   ACOSH->Drive[9]) ;
      printf ("Drive[10]:     %10.3f\n",   ACOSH->Drive[10]) ;
      printf ("Drive[11]:     %10.3f\n",   ACOSH->Drive[11]) ;
      printf ("Drive[12]:     %10.3f\n",   ACOSH->Drive[12]) ;
      printf ("Drive[13]:     %10.3f\n",   ACOSH->Drive[13]) ;
      printf ("Drive[14]:     %10.3f\n",   ACOSH->Drive[14]) ;
      printf ("Drive[15]:     %10.3f\n",   ACOSH->Drive[15]) ;
      printf ("Drive[16]:     %10.3f\n",   ACOSH->Drive[16]) ;
      printf ("Drive[17]:     %10.3f\n",   ACOSH->Drive[17]) ;
      printf ("Drive[18]:     %10.3f\n",   ACOSH->Drive[18]) ;
      printf ("Drive[19]:     %10.3f\n",   ACOSH->Drive[19]) ;
      printf ("Drive[20]:     %10.3f\n",   ACOSH->Drive[20]) ;
      printf ("Drive[21]:     %10.3f\n",   ACOSH->Drive[21]) ;
      printf ("Drive[22]:     %10.3f\n",   ACOSH->Drive[22]) ;
      printf ("Drive[23]:     %10.3f\n",   ACOSH->Drive[23]) ;
      printf ("Drive[24]:     %10.3f\n",   ACOSH->Drive[24]) ;
      printf ("Drive[25]:     %10.3f\n",   ACOSH->Drive[25]) ;
      printf ("Drive[26]:     %10.3f\n",   ACOSH->Drive[26]) ;
      printf ("Drive[27]:     %10.3f\n",   ACOSH->Drive[27]) ;
      printf ("Drive[28]:     %10.3f\n",   ACOSH->Drive[28]) ;
      printf ("Drive[29]:     %10.3f\n",   ACOSH->Drive[29]) ;
      printf ("Drive[30]:     %10.3f\n",   ACOSH->Drive[30]) ;
      printf ("Drive[31]:     %10.3f\n",   ACOSH->Drive[31]) ;
      printf ("Analog[0]:     %10.3f\n",   ACOSH->Analog[0]) ;
      printf ("Analog[1]:     %10.3f\n",   ACOSH->Analog[1]) ;
      printf ("Analog[2]:     %10.3f\n",   ACOSH->Analog[2]) ;
      printf ("Analog[3]:     %10.3f\n",   ACOSH->Analog[3]) ;
      printf ("Analog[4]:     %10.3f\n",   ACOSH->Analog[4]) ;
      printf ("Analog[5]:     %10.3f\n",   ACOSH->Analog[5]) ;
      printf ("Analog[6]:     %10.3f\n",   ACOSH->Analog[6]) ;
      printf ("Analog[7]:     %10.3f\n",   ACOSH->Analog[7]) ;
      printf ("Analog[8]:     %10.3f\n",   ACOSH->Analog[8]) ;
      printf ("Analog[9]:     %10.3f\n",   ACOSH->Analog[9]) ;
      printf ("Analog[10]:    %10.3f\n",   ACOSH->Analog[10]) ;
      printf ("Analog[11]:    %10.3f\n",   ACOSH->Analog[11]) ;
      printf ("Analog[12]:    %10.3f\n",   ACOSH->Analog[12]) ;
      printf ("Analog[13]:    %10.3f\n",   ACOSH->Analog[13]) ;
      printf ("Analog[14]:    %10.3f\n",   ACOSH->Analog[14]) ;
      printf ("Analog[15]:    %10.3f\n",   ACOSH->Analog[15]) ;
      printf ("Lcvr[0]:       %10.3f\n",   ACOSH->Lcvr[0]) ;
      printf ("Lcvr[1]:       %10.3f\n",   ACOSH->Lcvr[1]) ;
      printf ("Lcvr[2]:       %10.3f\n",   ACOSH->Lcvr[2]) ;
      printf ("Lcvr[3]:       %10.3f\n",   ACOSH->Lcvr[3]) ;
      printf ("Lcvr[4]:       %10.3f\n",   ACOSH->Lcvr[4]) ;
      printf ("Lcvr[5]:       %10.3f\n",   ACOSH->Lcvr[5]) ;
      printf ("Lcvr[6]:       %10.3f\n",   ACOSH->Lcvr[6]) ;
      printf ("Lcvr[7]:       %10.3f\n",   ACOSH->Lcvr[7]) ;
      printf ("Mic[0]:        %10.3f\n",   ACOSH->Mic[0]) ;
      printf ("Mic[1]:        %10.3f\n",   ACOSH->Mic[1]) ;
      printf ("Mic[2]:        %10.3f\n",   ACOSH->Mic[2]) ;
      printf ("Mic[3]:        %10.3f\n",   ACOSH->Mic[3]) ;
      printf ("Mic[4]:        %10.3f\n",   ACOSH->Mic[4]) ;
      printf ("Mic[5]:        %10.3f\n",   ACOSH->Mic[5]) ;
      }

   /*** Change Ydim to reflect removal of binary header. ***/

/*
 *   ACOSH->Ydim -= 1 ;
 *   if (pmode)
 *      printf ("Ydim:        %10d   (removed 1st line of image)\n",
 *		    ACOSH->Ydim) ;
 */

   /*** Allocate storage for image array. ***/

   npximg = ACOSH->Xdim * ACOSH->Ydim ;		/* # pixels in image. */
   nbypix = sizeof (unsigned short) ;		/* # bytes  in image. */
   nbyimg = npximg * nbypix ;
   sbuf = (unsigned short *) malloc (nbyimg) ;	/* Allocate memory for image. */

   (void) fseek (FD, 0L, SEEK_SET) ;		/* Rewind image file. */

   /*** Skip binary header (first "line" in image). ***/

/*   (void) fseek (FD, (long) ACOSH->Xdim*nbypix, SEEK_SET) ; */

   /*** Read pixel data. ***/

   fread (sbuf, nbypix, npximg, FD) ;		/* Read image. */

   fclose (FD) ;
   return (sbuf) ;			/* Return pointer to image buffer.*/
}
