/* rdimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 4 February 1985
 * 11 November 1986: modify to accomodate floating-point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char	*
rdimg (namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
   register unsigned char *cpbuf;	/* Pointer to image array. */
   register int	 cpfd;		/* File descriptor for image disk file. */
   register int  imgsiz;	/* Image size (bytes) (pixel data only). */
   register long nread;
   int nrechd;			/* # records for header. */

   /*** Decommutate Housekeeping. ***/

   cphskp (namimg);	

   /*** Open image file. ***/

   if ((cpfd = open (namimg, 0)) < 0)
      {
      fprintf (stderr,"rdimg> Can't open file %s\n", namimg);
      return (NULL);
      }

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

   if (nbypix == 4)
      nrechd = 1;
   else
      nrechd = 2;
   if (lseek (cpfd, (long)(npix * nbypix * nrechd), 0) == -1)
      {
      fprintf (stderr, "rdimg> seek error.\n");
      return (NULL);
      }

/*   fprintf (stdout, "rdimg> npix: %d   npixl: %d   nbypix: %d\n",
 *            npix, npixl, nbypix);
 */

   /*** Allocate memory for image buffer. ***/

   imgsiz = (npixl * npixl * nbypix);
   if ((cpbuf = (unsigned char *) malloc (imgsiz)) == NULL)
      {
      fprintf (stderr, "rdimg> malloc error.");
      exit (-1);
      }

   if ((nread = read (cpfd, cpbuf, imgsiz)) <= 0)
      {
      fprintf (stderr, "rdimg> image read error.\n");
      return (NULL);
      }

   if (nread != imgsiz)
      {
      fprintf (stderr,"rdimg> nread:%d < imgsiz:%d\n", nread, imgsiz);
      return (NULL);
      }

   /*** Reverse bytes for Vax-calibrated images. ***/

   if (nbypix == 2 && bytord == 1)  srevbyt (cpbuf, nbypix, npixl*npixl);

   if (close (cpfd) == -1)
      {
      fprintf (stderr, "rdimg> file close error.%s\n", namimg);
      return (NULL);
      }

  return (cpbuf);
}
