/* rdmk3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read MLSO Mark-III image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Mauna Loa Solar Observatory   Mark-III K-coronameter
 * Andrew L. Stanger   3 February 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/cordyn/mlsofits/lib/mk3/mk3lib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char *
rdmk3 (namimg, pmode)

char  *namimg ;		/* Pointer to image disk file name. */
int pmode ;		/* Print mode: 0=silent, 1=verbose. */

{
register unsigned char *imgbuf;	/* Pointer to image array. */
register int	imgsiz;		/* Image size (bytes) (excludes hskp/tailer). */
register long	nread;
int nrechd;			/* # records for header. */
FILE *imgFD ;			/* File structure for image disk file. */

   /*** Decommutate Housekeeping. ***/

   mk3hskp (namimg, pmode);	

   /*** Open image file. ***/

   if ((imgFD = fopen (namimg, "r")) == NULL)
      {
      fprintf (stderr, "rdmk3> Can't open file %s\n", namimg);
      return (NULL);
      }

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

   if (fseek (imgFD, (long) 0, 0) == -1)
      {
      fprintf (stderr, "rdmk3> seek error.\n");
      return (NULL);
      }

/*   fprintf (stdout, "rdmk3> xdim: %d   ydim: %d   nbypx: %d\n",
 *            MK3I->xdim, MK3I->ydim, MK3I->nbypx);
 */

   imgsiz = (MK3I->xdim * MK3I->ydim * MK3I->nbypx);
   if ((imgbuf = (unsigned char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr, "rdmk3> malloc error.");
      exit (1);
     }

   if ((nread = fread (imgbuf, sizeof (char), imgsiz, imgFD)) <= 0)
      {
      fprintf (stderr, "rdmk3> image read error.\n");
      return (NULL);
      }

   if (nread != imgsiz)
      {
      fprintf (stderr, "rdmk3> nread:%d < imgsiz:%d\n", nread, imgsiz);
      return (NULL);
      }

   if (fclose (imgFD) != 0)
      {
      fprintf (stderr, "rdmk3> file close error.%s\n", namimg);
      return (NULL);
      }
   
  return (imgbuf);
}
