/* rscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Radial Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984
 * 18 July 1984: added option to remove stray light & vignetting.
 * 26 July 1984: added calibration correction.
 *               added option to remove F-corona.
 * 7 February 1985: change double to float.
 * 10 December 1985: Apply fixcal to all calibrated images.
 * 15 May 1986: Use ATM/WLC F-corona function.
 * 14 November 1986: use 'gres' instead of 'nres' for Grinnell.
 * 25 November 1986: use 'rcoord' instead of 'coord' for roll compensation.
 * 22 March 1988: remove Grinnell references for use on Sun-4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

rscan (namimg, imgbuf, rmin, rmax, rinc, theta, scntyp, scan, scandx, ns)

char	namimg [];	/* Image file name. */
char	*imgbuf;	/* Pointer to image array. */
float	rmin, rmax;	/* Solar radius limits for radial scan. */
float	rinc;		/* Radial increment for radial scan. */
float	theta;		/* Theta (degrees). */
int	*scntyp;	/* Pointer to scan type (1=radial,2=line). */
float	scan [];	/* Dependent axis scan array. */
float	scandx [];	/* Independent axis scan array. */
int	*ns;		/* Number of values in scan. */

{
extern	int rcoord ();	/* Function to convert between polar & cartesian. */
extern	int fixcal ();	/* Function which applies calibration correction. */
extern	int remstr ();	/* Function which removes stray light. */
extern	int remvig ();	/* Function which removes vignetting. */
extern	int rmfc ();	/* Function which removes F-corona. */
extern  float blint ();	/* Bilinear interpolation. */

float	dstsun;
float	radinc;
float   Bexp;		/* "Generic" magnitude scaling factor. */
float	x, y;
float	r;
float	th;

int	irad;
int	ixg, iyg;
int	ierr;
int	nscan;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Set scale factor. ***/

   Bexp = pscale;

   /*** Radial increment: set to 0.1 Rsun if 'rinc' <= 0. ***/

   radinc = rinc;
   if (rinc <= 0.0)  radinc = 0.1;
   th = theta;

   if (*scntyp == 2)  dstsun = -radinc;

   /* ****** Radial Scan ****** */

   for (irad = 0; irad < MAXSCN; irad++) *(scan + irad) = *(scandx + irad) = 0.;
 
   /* ....... Radial scan loop ....... */

   *ns = 0;
   irad = -1;
   for (r = rmin; r <= rmax; r += radinc)
      {
      irad++;
 
      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = rcoord (&r, &th, &x, &y, 1, scroll)) != 0)  return (ierr);

      /*** Draw "X" to mark theta scan point on Grinnell display. ***/
 
      if (igr > 0)
         {
         ixg = (int) x/gres + 63;
         iyg = 510 - (int) y/gres;
         wvec (ixg, iyg, ixg+1, iyg+1);
         wvec (ixg+1, iyg, ixg, iyg+1);
         }
 
      /* 
       * Perform bilinear interpolation to compute scan point value from image.
       * Use 4 nearest neighbors.
       */
      dstsun += radinc;
      if (*scntyp == 1)  *(scandx + irad) = (float) r;
      else if (*scntyp == 2)  *(scandx + irad) = (float) dstsun;

      *(scan + irad) = (float) (blint (imgbuf, &x, &y) * Bexp);

      /*
       * Divide scan values by 5 for equivalent second data type.
       * (numbers in file are 5 * equivalent second units.)
       */
 
      if (idtype == 2) *(scan + irad) /= 5.0;
      }

   *ns = irad + 1;

   /*** Smooth data using 'nave' point average. ***/
 
   faverage (nave, scan, *ns);
 
/* Apply Munro calibration correction. */
   /* Remove stray light & vignetting. */
   /* Remove F-corona. */
 
   nscan = *ns;
/*   if (idtype == 3)  fixcal (scandx, nscan, scan, nscan); */
   if (idtype == 3 && lsv == 1)
      {
      remstr (scandx, nscan, scan, nscan);
      remvig (scandx, nscan, scan, nscan);
      if (lfc == 1)  rmfc (scandx, nscan, scan, nscan, th, 1);
      }

/*
 *   if (igterm == 0)
 *      for (irad=0; irad<*ns; irad++)
 *         fprintf (stdout, "r:%7.3f  th:%7.2f  scan:%9.2e\n",
 *                  scandx [irad], th, scan [irad]);
 */

   return (0);
}
