/* sfvcor.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Remove stray light and/or vignetting and/or F-corona from a C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 * 17 February 1987
 * 1 December 1987: Make calibration correction removal an OPTION.
 * 30 March 1988: Sun-4/280
 * 30 September 1991: SGI IRIS
 * 10 June 1992: accept image types short & float.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"

main (argc, argv)

int argc;
char *argv [];
{
extern char *malloc ();
extern char *rdimg ();
extern int rcoord ();
extern float calcor ();
extern float stray ();
extern float vignet ();
extern float fcorona ();

float r, th;
float x, y;
float cc   = 1.0;
float str  = 0.0;
float vig  = 1.0;
float fcor = 0.0;
float fv    = 1.0;
float vinv = 1.0;
float pixval;

static char buf [1792];
static char namcor [80];
static char namimg [80];
static char answer [2];

register char  *imc;
register short *image;
register short *imgcor;
char  *imb;
short *ims;
float *imf;
char  *corb;
short *cors;
float *corf;

int idot;
int nbylin;
int nbyimg;
int nread, nwrite, nclose;
int i;
register int ix, iy;
int mcc = 0;
int mstr = 0;
int mvig = 0;
int mfc = 0;
int lunimg, luncor;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s cp_image_filename\n", argv [0]);
      fprintf (stderr, "Enter image file name : ");
      fscanf  (stdin,  "%s", namimg);
      }

   printf ("Apply Munro calibration correction factor [y/n]? ");
   scanf  ("%s", answer);
   if (strncmp (answer, "y", 1) == 0)  mcc = 1;

   printf ("Remove stray light [y/n] ? ");
   scanf  ("%s", answer);
   if (strncmp (answer, "y", 1) == 0)  mstr = 1;

   printf ("Remove vignetting [y/n] ? ");
   scanf  ("%s", answer);
   if (strncmp (answer, "y", 1) == 0)  mvig = 1;

   printf ("Remove F-corona [y/n] ? ");
   scanf  ("%s", answer);
   if (strncmp (answer, "y", 1) == 0)  mfc = 1;


   /*** If any of the corrections have previously been applied,
    *** do NOT remove them a second time.
    ***/

   if (H.cc)   mcc  = 0;
   if (H.str)  mstr = 0;
   if (H.vig)  mvig = 0;
   if (H.f)    mfc  = 0;

   /*** Derive output file name from input image name. ***/

   idot = -1;
   for (i = 0; i < strlen (namimg); i++) if (namimg [i] == '.')  idot = i;
   if (idot < 0)  idot = strlen (namimg) - 1;
   strcpy (namcor, namimg);

   /*** If no vignetting removed previously (or now),
    *** append "V" to filename.
    ***/

   if (!H.vig && mvig == 0)
      {
      namcor [idot] = 'V';
      idot++;
      }
   namcor [idot] = '.';

   if (mcc  == 1)   namcor [idot+1] = 'c';  else namcor [idot+1] = 'x';
   if (mstr == 1)   namcor [idot+2] = 's';  else namcor [idot+2] = 'x';
   if (mvig == 1)   namcor [idot+3] = 'v';  else namcor [idot+3] = 'x';
   if (mfc  == 1)   namcor [idot+4] = 'f';  else namcor [idot+4] = 'x';
   if (H.cc  == 1)  namcor [idot+1] = 'C';
   if (H.str == 1)  namcor [idot+2] = 'S';
   if (H.vig == 1)  namcor [idot+3] = 'V';
   if (H.f   == 1)  namcor [idot+4] = 'F';
   namcor [idot+5] = '\0';
   printf ("namcor: %s\n", namcor);

   /*** Read image into memory. ***/

   imb = (char *) rdimg (namimg);
   ims = (short *) imb;
   imf = (float *) imb;

   nbylin = nbypix * npix;
   nbyimg = nbylin * npix;
   corb = (char *) malloc (nbyimg);
   cors = (short *) corb;
   corf = (float *) corb;

   /*** Open output image file & copy housekeeping from input image. ***/

   lunimg = open (namimg, 0);
   luncor = creat (namcor, 0644);
   nclose = close (luncor);
   luncor = open (namcor, 1);

   /*** Copy first record of image disk file. ***/

   nread  = read  (lunimg, buf, nbylin);
   if (mstr) buf [223] = buf [223] |   1;
   if (mvig) buf [223] = buf [223] |   2;
   if (mfc ) buf [223] = buf [223] |   8;
   if (mcc)  buf [223] = buf [223] | 128;
   nwrite = write (luncor, buf, nbylin);

   if (nbypix != 4)	/* Second record is bitmap only for 16-bit images. */
      {
      nread  = read  (lunimg, buf, nbylin);

/*  Vax-11/750 byte order.
 *
 *   *(buf+70) = ((((int) *(buf+70)) & 3) | 4) & 0377;
 */

      nwrite = write (luncor, buf, nbylin);
      }
   nclose = close (lunimg);

   /*** Remove stray light, vignetting & F-corona. ***/

   for (iy = 0; iy < npix; iy++)
      for (ix = 0; ix < npix; ix++)
         {
         x = (float) ix;
         y = (float) iy;
         rcoord (&r, &th, &x, &y, -1, scroll);

         if (mcc)  cc = calcor (r);			/* Munro cal.corr. */
         if (mstr)  str = stray (r);			/* Stray light. */
         if (mvig) { vig = vignet (r);  vinv = 1.0 / vig; }	/* Vignetting.*/
         if (mfc)  fcor = fcorona (r,th);		/* F-corona. */

         /*** If Vignetting was not previously removed, and it is
          *** not going to be removed now, then the resultant image
          *** will be K*V instead of K.
          *** In such a case, the K-corona correction must be vignetted.
          ***/

         if (!H.vig && mvig == 0)
            fv = vignet (r);
         else
            fv = 1.0;

         i = x + y * npix;

         if (nbypix == 2)		/* Short integer images. */
            {
            pixval = (*(ims + i) * cc - str) * vig - fcor * fv;
            if (pixval < 0.0)  pixval = 0.0;
            *(cors + i) = (short) (pixval) & 0177777;
            }
         else
         if (nbypix == 4)		/* Floating point images. */
            {
            pixval = (*(imf + i) * cc - str) * vig - fcor * fv;
            if (pixval < 0.)  pixval = 0.0;
            *(corf + i) = (float) (pixval);
            }
         }

   /*** Write image pixels to disk file. ***/

/*   srevbyt (imc, 2, npix*npix); */
   nwrite = write (luncor, corb, nbyimg);
   nclose = close (luncor);
   exit (0);
}
