/* sfvimgv.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create stray light and/or vignetting and/or F-corona images.
 * Apply vignetting to stray & F-corona images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 * 26 February 1987
 * 27 September 1991: High resolution version
 *  3 June 1992: floating point image version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#define MAXBUF 1792

main ()

{
extern char *malloc ();
extern int rcoord ();
extern float calcor ();
extern float stray ();
extern float vignet ();
extern float fcorona ();

static	float	pi = 3.141592654;
static	float	secrs = 961.0;	   /* arcseconds per solar radius. */
static	float	secpix = 11.866;   /* arcseconds per pixel (4 quad, hi res). */
static float Cinv = 1.e+11;
float r, th;
float x, y;
float cc, str, vig, fcor;
float viginv;

static char hk [224] =
{0120,    0, 0055, 0045, 0345, 0130,    0,    0,    0, 0377,
    0,    0,    0,    0, 0202, 0226, 0224, 0261, 0260, 0240,
 0014,    0, 0200, 0250,    0, 0001, 0077, 0010, 0100, 0001,
 0200, 0120, 0224, 0262, 0260, 0237, 0040, 0040, 0040, 0040,
 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0224, 0261,
 0260, 0240,    0, 0337, 0042,    0, 0345, 0336, 0334, 0040,
 0350, 0026, 0200, 0177, 0224, 0262, 0260, 0240, 0142, 0220,
 0001, 0335, 0226, 0206, 0335, 0040, 0350, 0322, 0363, 0144, 
 0224, 0262, 0260, 0240, 0335, 0042, 0314, 0245, 0177, 0177,
 0361, 0314, 0352, 0017, 0342, 0374, 0224, 0261, 0260, 0240,
 0070, 0062, 0342, 0040, 0055, 0344, 0177, 0343, 0177, 0343,
 0003, 0200, 0224, 0262, 0260, 0240, 0260, 0335, 0352, 0040,
 0040, 0040, 0040, 0040, 0040, 0120,    0, 0055, 0040, 0040,
 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040,
    0, 0120,    0, 0055, 0060, 0061, 0060, 0062, 0061, 0062,
 0060, 0060, 0060, 0060, 0056, 0106, 0103, 0126, 0040, 0040,
 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040,
 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040, 0040,
 0040, 0040, 0116, 0117, 0116, 0123, 0124, 0122, 0116, 0117,
 0116, 0126, 0111, 0107, 0047, 0020,    0,    0,    0,    0,
    0,    0,    0,    0, 0034, 0022, 0032, 0211, 0034, 0022,
 0032, 0211, 0034, 0022, 0032, 0211,    0, 0144, 0001, 0104,
 0013,    0, 0200,    0,
};

static char hkbuf [MAXBUF];
static char bmbuf [MAXBUF];
static char namcor [] = {"cc.img"};
static char namstr [] = {"sv.img"};
static char namvig [] = {"v.img"};
static char namfc []  = {"fv.img"};
static char namfs []  = {"s+fv.img"};
static char namimg [80];
static char answer [2];

register float *lincor;
register float *linstr;
register float *linvig;
register float *linfc;
register float *linfs;

int nbylin;
int nbypix = 4;
int npix = 896;
int nread, nwrite, nclose;
int i;
int ires;
int ival;
register int ix, iy;
int lcor = 0;
int lstr = 0;
int lvig = 0;
int lfc = 0;
int lfs = 0;
int luncor, lunstr, lunvig, lunfc;
int lunfs;

   /*** Set up scale variables. ***/

   raddeg = pi / 180.;
   degrad = 180. / pi;
   pixrs = secrs / secpix;
   rspix = 1. / pixrs;

   scroll = 0;
   xcen = 447;
   ycen = 447;

   fprintf (stderr, "Create calibration correction image [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)  lcor = 1;

   fprintf (stderr, "Create stray light image [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)  lstr = 1;

   fprintf (stderr, "Create vignetting image [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)  lvig = 1;

   fprintf (stderr, "Create F-corona image [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)  lfc = 1;

   fprintf (stderr, "Create F+stray image [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   if (strncmp (answer, "y", 1) == 0)  lfs = 1;

   /*** Create housekeeping and bitmap record arrays. ***/

   for (i = 0; i < 224; i++)
      {
      hkbuf [i] = hk[i];
      }
   for (i = 224; i < MAXBUF; i++)
      {
      hkbuf [i] = '\0';;
      }

   for (i = 0; i < npix; i++)
      {
      bmbuf[i] = (char) 0377;
      }
   for (i = npix; i < MAXBUF; i++)
      {
      bmbuf [i] = '\0';;
      }

   /*** Set hkbuf to high resolution. ***/

   hkbuf [3] = hkbuf [3] & 0373;


   nbylin = nbypix * npix;
   printf ("nbypix: %3d   npix: %3d   nbylin: %4d\n", nbypix, npix, nbylin);

   /*** Open output image file & copy housekeeping from array hkbuf. ***/

   if (lcor == 1)				/* [M] */
      {
      fprintf (stdout, "namcor: %s\n", namcor);
      luncor = creat (namcor, 0644);
      nclose = close (luncor);
      luncor = open (namcor, 1);
      hkbuf [223] = 128;			/* Munro cal. corr. applied. */
      nwrite = write (luncor, hkbuf, MAXBUF);
      nwrite = write (luncor, bmbuf, MAXBUF);
      lincor = (float *) malloc (nbylin);
      }
   if (lstr == 1)				/* [S * V] */
      {
      fprintf (stdout, "namstr: %s\n", namstr);
      lunstr = creat (namstr, 0644);
      nclose = close (lunstr);
      lunstr = open (namstr, 1);
      hkbuf [223] = 4 + 8 + 16;			/* F + K removed. */
      nwrite = write (lunstr, hkbuf, MAXBUF);
      nwrite = write (lunstr, bmbuf, MAXBUF);
      linstr = (float *) malloc (nbylin);
      }
   if (lvig == 1)				/* [V] */
      {
      fprintf (stdout, "namvig: %s\n", namvig);
      lunvig = creat (namvig, 0644);
      nclose = close (lunvig);
      lunvig = open (namvig, 1);
      hkbuf [223] = 1 + 4 + 8 + 16;		/* S + F + K removed. */
      nwrite = write (lunvig, hkbuf, MAXBUF);
      nwrite = write (lunvig, bmbuf, MAXBUF);
      linvig = (float *) malloc (nbylin);
      }
   if (lfc == 1)				/* [F * V] */
      {
      fprintf (stdout, "namfc: %s\n", namfc);
      lunfc = creat (namfc, 0644);
      nclose = close (lunfc);
      lunfc  = open (namfc, 1);
      hkbuf [223] = 1 + 4 + 16;			/* S + K removed. */
      nwrite = write (lunfc, hkbuf, MAXBUF);
      nwrite = write (lunfc, bmbuf, MAXBUF);
      linfc = (float *) malloc (nbylin);
      }
   if (lfs == 1)				/* [(S + F) * V] */
      {
      fprintf (stdout, "namfs: %s\n", namfs);
      lunfs = creat (namfs, 0644);
      nclose = close (lunfs);
      lunfs  = open (namfs, 1);
      hkbuf [223] = 4 + 16;			/* K removed. */
      nwrite = write (lunfs, hkbuf, MAXBUF);
      nwrite = write (lunfs, bmbuf, MAXBUF);
      linfs = (float *) malloc (nbylin);
      }

   /*** Compute calibration correction, stray light, vignetting & F-corona. ***/

   for (iy = 0; iy < npix; iy++)
      {
      for (ix = 0; ix < npix; ix++)
         {
         x = (float) ix;
         y = (float) iy;
         rcoord (&r, &th, &x, &y, -1, scroll);
         vig = vignet (r);
         if (r <= 1.0)  vig = 0.0;
         if (vig != 0.0)  viginv = 1.0 / vig;  else viginv = 1.0;
         if (lcor)
            {
            cc = calcor (r);
            if (r <= 1.5)  cc = 0.0;
            *(lincor + ix) = ((float) (cc));
            }
         if (lstr)
            {
            str = stray (r);
            if (r <= 1.5)  str = 0.0;
            *(linstr + ix) = ((float) (str * viginv));
            }
         if (lvig)
            {
            vig = vignet (r);
            if (r <= 1.0)  vig = 0.0;
            *(linvig + ix) = ((float) (vig));
            }
         if (lfc)
            {
            fcor = fcorona (r, th);
            if (r <= 1.5)  fcor = 0.0;
            *(linfc + ix) = ((float) (fcor * viginv));
            }
         if (lfs)
            {
            fcor = fcorona (r, th);
            str = stray (r);
            if (r <= 1.5)  {fcor = str = 0.0;}
            *(linfs + ix) = (float) ((fcor * viginv) + str);
            }
         }
      if (lcor == 1)  nwrite = write (luncor, lincor, nbylin);
      if (lstr == 1)  nwrite = write (lunstr, linstr, nbylin);
      if (lvig == 1)  nwrite = write (lunvig, linvig, nbylin);
      if (lfc  == 1)  nwrite = write (lunfc,  linfc,  nbylin);
      if (lfs  == 1)  nwrite = write (lunfs,  linfs,  nbylin);
      }

   nclose = close (luncor);
   nclose = close (lunstr);
   nclose = close (lunvig);
   nclose = close (lunfc);
   nclose = close (lunfs);
   exit (0);
}
