/* sfvplot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Plot stray light, vignetting, & F-corona.
 * Use ATM/WLCE F-corona function (Saito equation).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 25 February 1987
 * 19 June 1990: add electron/Bsun plot.
 * 10 May 1993: stray light inherently includes the effects of vignetting.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.h"

/* Standard C system library and Math functions library. */

#include <stdio.h>
#include <math.h>

int pcount = 0;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];

{

   extern float calcor ();	/* Munro calibration correction. */
   extern float elec ();	/* Electrons per B/Bsun. */
   extern float stray ();	/* Stray light. */
   extern float vignet ();	/* Vignetting. */
   extern float fcorona ();	/* F-corona function. */
   extern xyplot ();		/* XY plot using NCAR system plot package. */

   static char buff [84];
   static char namdat [80] = "sfv.dat";	/* Scan data file name. */
   static char answer [2];		/* Storage for y/n response. */
   static char gksout [80];		/* NCARG_GKS_OUTPUT */
   static char namemc [80] = "sfv.cgm";	/* Metacode file name. */
   static char namgks [80] = "gks.cgm";
   static char labx [40];		/* Label for X-axis. */
   static char laby [40];		/* Label for Y-axis. */
   static char title [40];		/* Graph title. */
   static int iasf [13] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1};

   static float cc [MAXSCN];	/* Pointer to calib. correction scan array. */
   static float ebsun [MAXSCN];	/* Pointer to electron/Bsun scan array. */
   static float str [MAXSCN];	/* Pointer to stray light scan array. */
   static float fc0 [MAXSCN];	/* Pointer to F-corona scan array. */
   static float fc90 [MAXSCN];	/* Pointer to F-corona scan array. */
   static float sf0 [MAXSCN];	/* Pointer to stray + F-corona scan array. */
   static float sf90 [MAXSCN];	/* Pointer to stray + F-corona scan array. */
   static float sfv0 [MAXSCN];	/* Pointer to stray + F-corona *vignet array. */
   static float sfv90 [MAXSCN];	/* Pointer to stray + F-corona *vignet array. */
   static float vig [MAXSCN];	/* Pointer to vignetting scan array. */
   static float viginv [MAXSCN];/* Pointer to inverse vignetting scan array. */
   static float scandx [MAXSCN];/* Pointer to dependent scan array. */

   float	r;		/* Radial value. */
   float	rmin, rmax;	/* Radial limits (Rsun). */
   float	rinc;		/* Radial increment. */
   float	theta = 0.0;	/* Solar position angle (degrees). */
   float	th0 = 0.0;	/* 0 degrees. */
   float	th90 = 90.0;	/* 90 degrees. */
   float f0 = 0.0, f1 = 1.0;	/* Values for GKS subroutines. */

   int	scntyp = 1;	/* Scan type: 1=radial, 2=line. */
   int	nval;		/* # values in scan. */
   int	i;
   int i0 = 0;			/* Value for use with GKS subroutines. */
   int i1 = 1;			/* Value for use with GKS subroutines. */
   int i2 = 2;			/* Value for use with GKS subroutines. */
   int i3 = 3;			/* Value for use with GKS subroutines. */
   int i4 = 4;			/* Value for use with GKS subroutines. */
   int i5 = 5;			/* Value for use with GKS subroutines. */
   int i6 = 6;			/* Value for use with GKS subroutines. */
   int i7 = 7;			/* Value for use with GKS subroutines. */
   int	idot;		/* Index pointer to "." in image filename.*/
   int	ixscale = 0;	/* x-axis scaling type; 0=linear. */
   int	iyscale = 0;	/* y-axis scaling type; 0=linear. */
   int	ibkg = 0;	/* Background flag; 0=tick marks only. */
   int	plotit = 1;
   int	noplot = 0;
   int	clip = 0;
   int	isiz = 0;
   int	wksid = 1;
   int	lunit = 8;
   int	wkstype = 1;
   int	lunerr;

   FILE	*lunsfv;	/* File descriptor for scan data file. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

/*
 *   printf ("Are you using a graphics terminal (y/n) ? ");
 *   scanf ("%s", answer);
 *   if (strncmp ("y", answer, 1) == 0)
 *      {
 *      igterm = 1;
 *      gksini_ (&plotit, namgks);
 *      }
 *   else
 *      {
 *      igterm = 0;
 *      gksini_ (&noplot, namgks);
 *      }
 */
   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
   putenv (gksout);
   opngks_ ();
   gsclip_ (&clip);

   gsasf_ (iasf);			/* Set aspect source flags. */
   gscr_ (&i1, &i0, &f1, &f1, &f1);	/* Background color: white. */
   gscr_ (&i1, &i1, &f0, &f0, &f0);	/* Foreground color: black. */
   gscr_ (&i1, &i2, &f1, &f0, &f0);	/* color: red. */
   gscr_ (&i1, &i3, &f0, &f1, &f0);	/* color: green. */
   gscr_ (&i1, &i4, &f0, &f0, &f1);	/* color: blue. */
   gscr_ (&i1, &i5, &f1, &f1, &f0);	/* color: yellow. */
   gscr_ (&i1, &i6, &f1, &f0, &f1);	/* color: magenta. */
   gscr_ (&i1, &i7, &f0, &f1, &f1);	/* color: cyan. */

   plotit_ (&i0, &i0, &i0);
   plotit_ (&i0, &i0, &i2);
   plotif_ (&f0, &f0, &i2);
   sflush_ ();				/* Flush polyline buffer in SPPS. */

   gsplci_ (&i1);			/* polyline color index. */
   gspmci_ (&i1);			/* polymarker color index. */
   gstxci_ (&i1);			/* text color index. */
   gsfaci_ (&i1);			/* fill area color index. */

   /* Request input information. */

/*
 *   printf ("Enter solar position angle (deg.): ");
 *   scanf  ("%f", &theta);
 */
   fprintf (stderr, "Enter solar radius min, max, & increment (Rsun): ");
   fscanf  (stdin,  "%f,%f,%f", &rmin, &rmax, &rinc);

   /*** Get correction data from interpolation tables. ***/

   nval = (int) ((rmax - rmin ) / rinc) + 2;
   r = rmin - rinc;

   for (i=0; i<nval; i++)
       {
	r += rinc;
	scandx [i] = r;
	cc [i]     = calcor (r);
	ebsun [i]  = elec (r);
	str [i]    = stray (r);
	fc0 [i]    = fcorona (r, th0);
	fc90 [i]   = fcorona (r, th90);
	sf0 [i]    = stray (r) + fcorona (r, th0);
	sf90 [i]   = stray (r) + fcorona (r, th90);
	sfv0 [i]   = stray (r) + (fcorona (r, th0) / vignet (r));
	sfv90 [i]  = stray (r) + (fcorona (r, th90) / vignet (r));
	vig [i]    = vignet (r);
	viginv [i] = 1.0 / vignet (r);
	}

   /*** Print scan values. ***/

   strcpy (namemc, namgks);
   printf ("corrections data file:  %s\n", namdat);
   printf ("metacode plot file: %s\n", namemc);
   lunsfv = fopen (namdat, "w");

   fprintf (lunsfv, "SMM C/P Corrections\n");
   fprintf (lunsfv, "Radial scan  %6.2f to %6.2f Rsun  %7.2f degrees\n\n",
            rmin, rmax, theta);
   fprintf (lunsfv, "        position   Munro     stray             Fcorona    Fcorona    electrons\n");
   fprintf (lunsfv, "radius   angle   cal.corr    light   vignet  0 degrees 90 degrees    /Bsun\n");

   for (i=0; i<nval; i++)
      fprintf (lunsfv,"%5.2f  %7.2f   %7.2f   %8.2e   %5.2f   %8.2e   %8.2e   %9.3e\n",
        scandx[i], theta, cc[i], str[i], vig[i], fc0[i], fc90[i], ebsun[i]);

   close (lunsfv);

   /*** Plot "scans". ***/
 
   sprintf (labx, "solar radius (Rsun)");

   sprintf (title, "SMM C/P Munro Calibration Correction");
   sprintf (laby, "Calibration Correction Factor");
/*   gets (buff); */
   xyplot (scandx, cc, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "Electrons/Bsun");
   sprintf (laby, "# Electrons per B/Bsun");
/*   gets (buff); */
   xyplot (scandx, ebsun, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "SMM C/P Vignetting Correction");
   sprintf (laby, "Vignetting Correction Factor");
/*   gets (buff); */
   xyplot (scandx, vig, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "SMM C/P Vignetting Attenuation");
   sprintf (laby, "Vignetting Attenuation Factor");
/*   gets (buff); */
   xysplot (scandx, viginv, 0.0, 0.0, 0.0, 1.2, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "SMM C/P Stray Light");
   sprintf (laby, "Stray Light (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, str, 0., 0., 0., 2e-9, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th0;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P F-corona", theta);
   sprintf (laby, "F-corona (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, fc0, 0., 0., 0., 2.2e-8, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th90;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P F-corona", theta);
   sprintf (laby, "F-corona (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, fc90, 0., 0., 0., 2.2e-8, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th0;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P Stray + F-corona", theta);
   sprintf (laby, "F-corona (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, sf0, 0., 0., 0., 2.2e-8, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th90;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P Stray + F-corona", theta);
   sprintf (laby, "F-corona (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, sf90, 0., 0., 0., 2.2e-8, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th0;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P  S + F * V", theta);
   sprintf (laby, "F-corona (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, sfv0, 0., 0., 0., 2.8e-9, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th90;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P  S + F * V", theta);
   sprintf (laby, "F-corona (B/Bsun)");
/*   gets (buff); */
   xysplot (scandx, sfv90, 0., 0., 0., 2.8e-9, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   clsgks_ ();
   exit (0);
}
