; atmvig.pro
;
;+
; Skylab ATM/WLCE coronagraph
;
; Re-Apply Vignetting function to Intensity image to reduce
; the dynamic range of brightness.
;
; Reads & Writes FITS formatted images.
;
; Andrew L. Stanger   HAO/NCAR   10 March 1998
; 
;-
PRO atmvig, namfts

   ; ifram : frame number
   ; iex   : exposure duration (seconds)
   ; ipol  : polaroid filter
   ; iweg  : wedge
   ; kswt  :
   ; istry : stray light flag
   ; jstry : stray light
   ; jswt  :
   ; pes   : pointing error sensor
   ; ias   : internal alignment sensor

   lfram= intarr (9)	; End frames for 35mm camera loads.
   lfram (0) =  4355	; End Camera Load 1
   lfram (1) =  8293	; End Camera Load 2A
   lfram (2) = 12327	; End Camera Load 2B
   lfram (3) = 16142	; End Camera Load 3A
   lfram (4) = 20105	; End Camera Load 3B
   lfram (5) = 24087	; End Camera Load 4A
   lfram (6) = 27839	; End Camera Load 4B
   lfram (7) = 32039	; End Camera Load 5A
   lfram (8) = 35798	; End Camera Load 5B

   xvin = 12234
   yvin = 12440
   xsun = 12075
   ysun = 12479
   pes1 = fltarr (2)
   pes1 (0) = 17.0
   pes1 (1) = 11.0
   fix = 0.92

   rad  = fltarr (50)	; Table of distances (microns) from vignet center
   trad = fltarr (50)	; Table of % transmission (vignetting) Munro 7 May 1976.

   img = READFITS (namfts, hdu)

   filmload = fxpar (hdu, 'FILMLOAD')
   previgx  = fxpar (hdu, 'PREVIGX')
   previgy  = fxpar (hdu, 'PREVIGY')

   irec = 0 ;
   FOR i = 0, 8 DO	$
   BEGIN
      IF (ifram LT lfram (i)) break 
      irec = irec + 3
   END

   IF (iex EQ 9)  THEN irec = irec + 1
   IF (iex EQ 27) THEN irec = irec + 2

   i = 0
   WHILE (ifram GT lfram (i)) DO i = i + 1

   ;--- Calculate vignetting & sun centers in microns.

   xcnn = xvin - dx
   ycnn = yvin - dy
   xscen = xsun - dx * 2.19 * pes (1)
   yscen = ysun - dy - 1.86 * pes (0)

   print, 'Vignetting center: ', xcnn, ycnn
   print, 'Sun center :       ', xscen, yscen

   ;--- Vignetting center in index units:

   xcen = xcnn / scfct + 0.5
   ycen = ycnn / scfct + 0.5

   IF (kswt) THEN			$
   BEGIN
      ;--- Remove the instrument vignetting function if kwst=1.

      ;--- Set up limits for vignetting function removal.

      ylim = (double) (9900.0 / scfct)
      ylim = ylim * ylim ;
      yl = (double) (i - ycen)
      y1 = yl * yl

      IF (y1 LE ylim) THEN		$
      BEGIN
         FOR j = 0, nwds - 1  DO	$
	 BEGIN
            xl = j - xcen
            x1 = xl * xl
            ir = (int) sqrt (x1 + y1)
            IF ( (j EQ 289 AND i EQ 161)  OR (j EQ 243 AND i EQ 338) )

;            print, 'trecsav (%3d, %3d) * vfunc= %g * %g  = %g\n', 
;			j, i, *recsav, vten [ir], *recsav * vten [ir])

            *recsav = *recsav * vten (ir)
            recsav++
         END 	; for j
      END 	; y1 <= ylim
   END		; kswt

END
