/* fitslib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * FITS keyword variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 March 1994
 * 22 April 1994: add variables: type_obs, etc., for disk images (BBSO, KPNO)
 * 2 June 1994: change:     bangle   -> solar_b0, hangle   -> solar_H0,
 *                          langle   -> solar_l0, pangle   -> solar_p0
 * 18 August 1994: change : solar_h0 -> hour_ang                      
 *                          scndir   -> scan_dir, scndur   -> scan_dur
 *                          angnorth -> northang, scroll   -> sc_roll
 *		   add    : code_obs, crradius, disp_dir, occ_disc
 *                 remove : cpver, dpmver, mk3ver
 * 22 August 1994: add    : CODE-CP, CODE-DPM, CODE-MK3
 * 19 June 1997: change "end" to "end_fts" & "time" to "time_fts".
 * 22 December 1997: Initialize naxis1=naxis2=naxis3=naxis4=1 (instead of zero).
 * 11 March 1998: Add variables for Skylab ATM/WLCE coronagraph.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

int	simple   = 0 ;	/* SIMPLE ? = 1, T; = 0, F; = -1, not defined. */
int	extend   = 0 ;	/* EXTEND ? = 1, T, = 0, F; = -1, not defined. */
int	blocked  = 0 ;	/* =0, NO; =1, YES. */
int	end_fts  = 0 ;	/* END encountered ? =0, NO; =1, YES. */

int	bitpix   = 0 ;	/* # bitx/pixel: 8,16,32: int; -16,-32: float. */
int	naxis    = 0 ;		/* # axes. */
int	naxis1   = 1 ;		/* # elements along axis 1. */
int	naxis2   = 1 ;		/* # elements along axis 2. */
int	naxis3   = 1 ;		/* # elements along axis 3. */
int	naxis4   = 1 ;		/* # elements along axis 4. */

int	pcount   = 0 ;		/* pcount. */
int	gcount   = 0 ;		/* gcount. */

char	xtension [12] ;		/* Extension name. */

char	date     [12] = "" ;	/* DD/MM/YY : file created */
char	date_obs [12] = "" ;	/* DD/MM/YY : observation */
char	date_cal [12] = "" ;	/* DD/MM/YY : calibration */
char	date_opc [12] = "" ;	/* DD/MM/YY : MK3 cal 0 */

char	time_fts [12] = "" ;	/* HH/MM/SS : file created */
char	time_obs [12] = "" ;	/* HH/MM/SS : observation */
char	time_cal [12] = "" ;	/* HH/MM/SS : calibration */
char	time_opc [12] = "" ;	/* HH/MM/SS : MK3 cal 0 */

char	origin   [72] = "" ;	/* Institution name. */
char	author   [72] = "" ;	/* Compilation author's name. */
char	referenc [72] = "" ;	/* Reference where assoc. data are published. */

char	telescop [72] = "" ;	/* Telescope   name. */
char	instrume [72] = "" ;	/* Instrument  name. */
char	observer [72] = "" ;	/* Observer's  name. */
char	object   [72] = "" ;	/* Object      name. */

float	equinox  = 0 ;	/* Equinox (year) for celestial coord system. */
float	epoch    = 0 ;	/* Epoch   (year) for celestial coord system. */

char	extname  [72] = "" ;	/* Extension name.	*/
int	extver   = 0 ;		/* Extension version.	*/
int	extlevel = 0 ;		/* Extension level.	*/

int	blank    = 0 ;		/* Blank value. */

float	b_scale  = 1.0 ;	/* Multiplicative scaling value. */
float	b_zero   = 0.0 ;	/* Additive scaling value. */

char	bunit    [12] = "" ;	/* Name of brightness (magnitude) units. */

char	ctype1   [72] = "" ;	/* Name of coordinate axis 1. */
char	ctype2   [72] = "" ;	/* Name of coordinate axis 2. */
char	ctype3   [72] = "" ;	/* Name of coordinate axis 3. */
char	ctype4   [72] = "" ;	/* Name of coordinate axis 4. */

float	crpix1   = 0.0 ;/* Location of reference point axis 1 [index]. */
float	crpix2   = 0.0 ;/* Location of reference point axis 2 [index]. */
float	crpix3   = 0.0 ;/* Location of reference point axis 3 [index]. */
float	crpix4   = 0.0 ;/* Location of reference point axis 4 [index]. */

float	crradius = 0.0 ;/* Rsun [pixels]. */

float	crval1   = 0.0 ;/* Location of reference point axis 1 [coord 1]. */
float	crval2   = 0.0 ;/* Location of reference point axis 2 [coord 2]. */
float	crval3   = 0.0 ;/* Location of reference point axis 3 [coord 3]. */
float	crval4   = 0.0 ;/* Location of reference point axis 4 [coord 4]. */

float	cdelt1   = 0.0 ;/* Increment between pixels axis 1 [coord 1]. */
float	cdelt2   = 0.0 ;/* Increment between pixels axis 2 [coord 2]. */
float	cdelt3   = 0.0 ;/* Increment between pixels axis 3 [coord 3]. */
float	cdelt4   = 0.0 ;/* Increment between pixels axis 4 [coord 4]. */

float	crota1   = 0.0 ;/* Rotation angle of axis 1 [coord 1]. */
float	crota2   = 0.0 ;/* Rotation angle of axis 2 [coord 2]. */
float	crota3   = 0.0 ;/* Rotation angle of axis 3 [coord 3]. */
float	crota4   = 0.0 ;/* Rotation angle of axis 4 [coord 4]. */

float	datamin  = 0.0 ;/* Minimum physical value in data array. */
float	datamax  = 0.0 ;/* Maximum physical value in data array. */

float	dispmin  = 0.0 ;/* Minimum display value in data array. */
float	dispmax  = 0.0 ;/* Maximum display value in data array. */

char	disp_dir [4] ;	/* Display direction: R/L U/D O/I */

char    diraxis1 [8] = "" ;	/* Scan direction axis 1: "left" or "right". */
char    diraxis2 [8] = "" ;	/* Scan direction axis 2: "up"   or "down".  */
char    diraxis3 [8] = "" ;	/* Scan direction axis 3. "in"   or "out"    */
char    diraxis4 [8] = "" ;	/* Scan direction axis 4. "plus" or "minus"  */

char	dataform [72] = "" ;	/* Data type description. */

int	code_cp  = 0 ;		/* SMM C/P data code. */
int	code_dpm = 0 ;		/* MLSO DPM data code. */
int	code_mk3 = 0 ;		/* MLSO MK3 data code. */
int	code_obs = 0 ;		/* observation code. */

char	type_cp  [72] ;		/* SMM C/P data type. */
char	type_dpm [72] ;		/* MLSO DPM data type. */
char	type_mk3 [72] ;		/* MLSO MK3 data type. */
char	type_obs [72] ;		/* Type of observation. */
char	type1 [72] = "" ;	/* Type 1. */
char	type2 [72] = "" ;	/* Type 2. */
char	type3 [72] = "" ;	/* Type 3. */
char	type4 [72] = "" ;	/* Type 4. */
char	type5 [72] = "" ;	/* Type 5. */
char	type6 [72] = "" ;	/* Type 6. */
char	type7 [72] = "" ;	/* Type 7. */
char	type8 [72] = "" ;	/* Type 8. */

int	nimavg   = 0 ;		/* # images averaged. */
float	expdur   = 0 ;		/* Exposure duration. */
char	expunits [12] = "" ;	/* Exposure duration units. */

float	rmin     = 0.0 ;	/* Minimum radius. */
float	rmax     = 0.0 ;	/* Maximum radius. */
float	rsun     = 0.0 ;	/* pixels per solar radius. */

float	tmin     = 0.0 ;	/* Minimum angle. */
float	tmax     = 0.0 ;	/* Maximum angle. */

float	scan_dur   = 0.0 ;	/* MLSO MK3 scan duration. */
char	scan_dir [4] = "" ;	/* MLSO MK3 scan direction: CW,CCW. */

float	northang = 0.0 ;	/* Angle of solar north w.r.t. +X axis. */
float	angle    = 0.0 ;	/* Angle. */

float	bias     = 0.0 ;	/* Bias. */
float	gain     = 0.0 ;	/* Gain. */
float	deltat   = 0.0 ;	/* Delta Time. */
float	phi      = 0.0 ;	/* Phi angle. */
float	theta    = 0.0 ;	/* Theta angle. */
float	radius   = 0.0 ;	/* Radius. */
float	rangvalu = 0.0 ;	/* Range value. */
float	zerovalu = 0.0 ;	/* Zero value. */
float	tilt     = 0.0 ;	/* Tilt angle. */
float	wave     = 0.0 ;	/* Wavelength. */
float	wavelen  = 0.0 ;	/* Wavelength. */
float	wavelnth = 0.0 ;	/* Wavelength. */

char	calibrat  [4] = "" ;	/* Calibrated. */
char	camera    [4] = "" ;	/* Camera name. */
char	filter   [24] = "" ;	/* Filter name. */
char	int_mode [24] = "" ;	/* Integration mode. */
char	runit    [24] = "" ;	/* Radius units. */
char	punit    [24] = "" ;	/* Phi angle units. */
char	tunit    [24] = "" ;	/* Theta angle units. */
char	seeing   [24] = "" ;	/* Seeing.  */
char	quality  [24] = "" ;	/* Quality. */
char	sky      [24] = "" ;	/* Sky.     */
char	utstart  [24] = "" ;	/* UT start time. */
char	utstop   [24] = "" ;	/* UT stop  time. */
char	version  [24] = "" ;	/* Version name.  */

int	frames   = 0 ;		/* # frames. */

float	occ_disc = 0.0 ;	/* occulting disk diameter [arcsec]. */

float	solar_b0 = 0.0 ;	/* B-angle. */
float	solar_l0 = 0.0 ;	/* L-angle. */
float	solar_p0 = 0.0 ;	/* P-angle. */
float	hour_ang = 0.0 ;	/* H-angle. */

float	elev     = 0.0 ;	/* Elevation (degrees). */
float	azimuth  = 0.0 ;	/* Azimuth   (degrees). */
float	airmass  = 0.0 ;	/* Airmass. */

char	optdev1 [16] = "" ;	/* MLSO DPM optical device #1 */
char	optdev2 [16] = "" ;	/* MLSO DPM optical device #2 */
char	optdev3 [16] = "" ;	/* MLSO DPM optical device #3 */
char	optdev4 [16] = "" ;	/* MLSO DPM optical device #4 */
char	optdev5 [16] = "" ;	/* MLSO DPM optical device #5 */
char	optdev6 [16] = "" ;	/* MLSO DPM optical device #6 */
char	optdev7 [16] = "" ;	/* MLSO DPM optical device #7 */
char	optdev8 [16] = "" ;	/* MLSO DPM optical device #8 */

char	optpos1 [16] = "" ;	/* MLSO DPM optical position #1 */
char	optpos2 [16] = "" ;	/* MLSO DPM optical position #2 */
char	optpos3 [16] = "" ;	/* MLSO DPM optical position #3 */
char	optpos4 [16] = "" ;	/* MLSO DPM optical position #4 */
char	optpos5 [16] = "" ;	/* MLSO DPM optical position #5 */
char	optpos6 [16] = "" ;	/* MLSO DPM optical position #6 */
char	optpos7 [16] = "" ;	/* MLSO DPM optical position #7 */
char	optpos8 [16] = "" ;	/* MLSO DPM optical position #8 */

int	calmir	= 0 ;		/* SMM C/P calibration mirror position.	*/
int	colorfil= 0 ;		/* SMM C/P color filter position.	*/
int	frame	= 0 ;		/* SMM C/P frame number.		*/
int	glfilter= 0 ;		/* SMM C/P Green Line filter position.	*/
int	polaroid= 0 ;		/* SMM C/P polaroid filter position.	*/
int	sector	= 0 ;		/* SMM C/P sector mirror position.	*/
int	sc_roll	= 0 ;		/* SMM C/P spacecraft roll (degrees).	*/

/*** GSFC variables: ***/

float	cd001001 = 0 ;		/* Rotation Matrix. */
float	cd001002 = 0 ;		/* Rotation Matrix. */
float	cd001003 = 0 ;		/* Rotation Matrix. */
float	cd002001 = 0 ;		/* Rotation Matrix. */
float	cd002002 = 0 ;		/* Rotation Matrix. */
float	cd002003 = 0 ;		/* Rotation Matrix. */
float	cd003001 = 0 ;		/* Rotation Matrix. */
float	cd003002 = 0 ;		/* Rotation Matrix. */
float	cd003003 = 0 ;		/* Rotation Matrix. */
char	creator [48] = "" ;	/* Creator: program that creates FITS file. */
float	longpole = 0.0 ;	/* Coordinate Rotation. */
float	mjd_obs  = 0 ;		/* Julian Date of Observation. */
float	projp1   = 0.0 ;	/* Projection parameter 1. */
float	projp2   = 0.0 ;	/* Projection parameter 2. */
char	radecsys [12] = "" ;	/* Reference Frame. */

/*** Skylab variables: ***/

short	doy_obs  = 0 ;		/* DAY-OF-YEAR : Universal Time (UT). */
char	strayin  [4]  = "" ;	/* Stray Light: 'IN' or 'OUT'. */
char	vignetin [4]  = "" ;	/* Vignetting : 'IN' or 'OUT'. */
short	exposure = 0 ;		/* Exposure Duration [seconds]. */
char	filmload [4]  = "" ;	/* Film Load: 1,2A,2B,3A,3B,4A,4B,5A,5B. */
float	rsun_arc = 0.0 ;	/* Radius of sun [arcsec]. */
float	rsun_mic = 0.0 ;	/* Radius of sun [microns]. */
float	rsun_pix = 0.0 ;	/* Radius of sun [pixels]. */
float	film_scl = 0.0 ;	/* Microns per pixel on film. */
char	fidum_lr [12] = "" ;	/* x,y position of lower right fiducial [mic].*/
char	fidup_lr [12] = "" ;	/* x,y position of lower right fiducial [pix].*/
char	fidum_ll [12] = "" ;	/* x,y position of lower left  fiducial [mic].*/
char	fidup_ll [12] = "" ;	/* x,y position of lower left  fiducial [pix].*/
char	fidum_ur [12] = "" ;	/* x,y position of upper right fiducial [mic].*/
char	fidup_ur [12] = "" ;	/* x,y position of upper right fiducial [pix].*/
char	fidum_ul [12] = "" ;	/* x,y position of upper left  fiducial [mic].*/
char	fidup_ul [12] = "" ;	/* x,y position of upper left  fiducial [pix].*/
int	roll = 0 ;		/* Direction of solar north w.r.t. -Y [arcmin]*/
int	presunx = 0 ;		/* Pre-flight X sun center [microns]. */
int	presuny = 0 ;		/* Pre-flight Y sun center [microns]. */
int	previgx = 0 ;		/* Pre-flight X vignet center [microns]. */
int	previgy = 0 ;		/* Pre-flight Y vignet center [microns]. */
int	strycenx = 0 ;		/* Pre-flight X stray light center [microns]. */
int	stryceny = 0 ;		/* Pre-flight Y stray light center [microns]. */
float	pesudok = 0.0 ;		/* PES UD error. */
float	pesrlok = 0.0 ;		/* PES RD error. */
float	peslim = 0.0 ;		/* PES errors are OK within +- PESLIM. */
float	iasudok = 0.0 ;		/* IAS UD error. */
float	iasrlok = 0.0 ;		/* IAS RL error. */
float	iaslim = 0.0 ;		/* IAS errors are OK within +- IASLIM. */
float	pesud = 0.0 ;		/* PES UP-DOWN    (Y-axis) measurement. */
float	pesrl = 0.0 ;		/* PES RIGHT-LEFT (X-axis) measurement. */
float	iasud = 0.0 ;		/* IAS UP-DOWN    (Y-axis) measurement. */
float	iasrl = 0.0 ;		/* IAS RIGHT-LEFT (X-axis) measurement. */
float	sunoffdx = 0.0 ;	/* X center offset for film load [microns]. */
float	sunoffdy = 0.0 ;	/* Y center offset for film load [microns]. */
float	suncenx = 0.0 ;		/* True X sun center [microns]. */
float	sunceny = 0.0 ;		/* True Y sun center [microns]. */
float	vigcenx = 0.0 ;		/* Vignetting X center [microns]. */
float	vigceny = 0.0 ;		/* Vignetting Y center [microns]. */
