/* ftsaddpar.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Add/Change keyword parameter in a FITS header.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   12 March 1998
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

ftsaddpar (hdu, key, param, param_type, comment)

char hdu [][80] ;		/* FITS Header buffer. */
char key [] ;			/* Keyword. */
char param [] ;			/* Parameter. */
char param_type [] ;		/* Parameter type: char, short, long, float. */
char comment [] ;		/* Comment. */

{
   static char eq [4] = "= " ;
   static char hkey [12] ;
   static char line [84] ;
   char   *cparam ;
   short  *sparam ;
   long   *lparam ;
   int    *iparam ;
   float  *fparam ;
   double *dparam ;

   int i ;
   int ik ;
   int nchlin ;
   int nbyhdu ;
   int nkey ;
   int nlines ;
   int ptype ;

   cparam = (char   *) param ;
   sparam = (short  *) param ;
   lparam = (long   *)  param ;
   iparam = (int    *) param ;
   fparam = (float  *) param ;
   dparam = (double *) param ;

   if (strcmp (param_type, "char")   == 0)  ptype = 1 ;
   if (strcmp (param_type, "short")  == 0)  ptype = 2 ;
   if (strcmp (param_type, "int")    == 0)  ptype = 3 ;
   if (strcmp (param_type, "long")   == 0)  ptype = 4 ;
   if (strcmp (param_type, "float")  == 0)  ptype = 5 ;
   if (strcmp (param_type, "double") == 0)  ptype = 6 ;

   fprintf (stdout, "param_type: %s  ptype: %d\n", param_type, ptype) ;
/*
   switch (param_type) 
      {
      case "char":   ptype = 1 ;
		     break ;
      case "short":  ptype = 2 ;
		     break ;
      case "int":    ptype = 3 ;
		     break ;
      case "long":   ptype = 4 ;
		     break ;
      case "float":  ptype = 5 ;
		     break ;
      case "double": ptype = 6 ;
		     break ;
      default:       ptype = 0 ;
      }
 */

   nbyhdu = strlen (hdu) ;		/* Size of HDU in bytes. */
   nlines = nbyhdu / 80 ;		/* Size of HDU in 80-char lines. */
   fprintf (stdout, "nbyhdu: %d  nlines: %d\n", nbyhdu, nlines) ;
   nkey = strlen (key) ;

   ik = 0 ;
   for (i = 0; i < nlines; i++)
      {
      if (strncmp (&hdu [i][0], key, nkey) == 0)
	 {
	 /*** Found keyword in FITS header. ***/
	 strncpy (line, &hdu [i][0], 80) ;
	 fprintf (stdout, "Found keyword: %s  line: %d\n", line, i) ;
	 ik = i ;
	 break ;
	 }
/*
      else
	 {
	 strncat (hkey, &hdu [i][0], nkey) ;
	 hkey [nkey] = '\0' ;
	 fprintf (stdout, "hkey: %s\n", hkey) ;
	 }
 */
      }

   if (ik == 0)
      {
      fprintf (stdout, "Unable to find keyword (%s)\n", key) ;
      return (-1) ;
      }


   if (ptype == 1)			/* Character parameter */
      {
      sprintf (line, "%-8s%2s'%-8s'", key, eq, cparam) ;
      nchlin = strlen (line) ;
      fprintf (stdout, "nchlin: %d\n", nchlin) ;
      for (i = nchlin; i < 80; i++)
      line [i] = ' ' ;
      sprintf (&line [31], "/ %s", comment) ;
      }

   if (ptype == 2)			/* Short Integer parameter */
      sprintf (line, "%-8s%2s%20d / %s", key, eq, sparam, comment) ;

   if (ptype == 3)			/* Integer parameter */
      sprintf (line, "%-8s%2s%20d / %s", key, eq, *iparam, comment) ;

   if (ptype == 4)			/* Long Integer parameter */
      sprintf (line, "%-8s%2s%20d / %s", key, eq, *lparam, comment) ;

   if (ptype == 5)			/* Floating Point parameter */
      {
      if (*fparam < 1.0e-2 && *fparam != 0.0)
      sprintf (line, "%-8s%2s%20.4e / %s", key, eq, *fparam, comment) ;
      else
      sprintf (line, "%-8s%2s%20.4f / %s", key, eq, *fparam, comment) ;
      }

   if (ptype == 6)			/* Double parameter */
      sprintf (line, "%-8s%2s%20.4g / %s", key, eq, *dparam, comment) ;

   /*** Fill remainder of line with blanks. ***/

   fprintf (stdout, "new line contents:\n%s\n", line) ;
   nchlin = strlen (line) ;
   fprintf (stdout, "nchlin: %d\n", nchlin) ;
   for (i = nchlin; i < 80; i++)
      line [i] = ' ' ;

   /*** Copy modified line into FITS header. ***/

   for (i = 0; i < 80; i++)
      {
      hdu [ik][i] = line [i] ;
      }
}

