/* hdu_clear.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Set FITS header parameters back to default values.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 22 Dec 1997: Initialize naxis1=naxis2=naxis3=naxis4=1 (instead of zero).
 * 12 Mar 1998: Add GSFC & Skylab variables.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "fitslib.ext"

hdu_clear ()

{
simple   = 0 ;  /* SIMPLE ? = 1, T; = 0, F; = -1, not defined. */
extend   = 0 ;  /* EXTEND ? = 1, T, = 0, F; = -1, not defined. */
blocked  = 0 ;  /* =0, NO; =1, YES. */
end_fts  = 0 ;  /* END encountered ? =0, NO; =1, YES. */

bitpix   = 0 ;  /* # bitx/pixel: 8,16,32: int; -16,-32: float. */
naxis    = 0 ;          /* # axes. */
naxis1   = 1 ;          /* # elements along axis 1. */
naxis2   = 1 ;          /* # elements along axis 2. */
naxis3   = 1 ;          /* # elements along axis 3. */
naxis4   = 1 ;          /* # elements along axis 4. */

pcount   = 0 ;          /* pcount. */
gcount   = 0 ;          /* gcount. */

strcpy (xtension, "");         /* Extension name. */

strcpy (date    , "") ;    /* DD/MM/YY : file created */
strcpy (date_obs, "") ;    /* DD/MM/YY : observation */
strcpy (date_cal, "") ;    /* DD/MM/YY : calibration */
strcpy (date_opc, "") ;    /* DD/MM/YY : MK3 cal 0 */

strcpy (time_fts, "") ;    /* HH/MM/SS : file created */
strcpy (time_obs, "") ;    /* HH/MM/SS : observation */
strcpy (time_cal, "") ;    /* HH/MM/SS : calibration */
strcpy (time_opc, "") ;    /* HH/MM/SS : MK3 cal 0 */

strcpy (origin  , "") ;    /* Institution name. */
strcpy (author  , "") ;    /* Compilation author's name. */
strcpy (referenc, "") ;    /* Reference where assoc. data are published. */
strcpy (telescop, "") ;    /* Telescope   name. */
strcpy (instrume, "") ;    /* Instrument  name. */
strcpy (observer, "") ;    /* Observer's  name. */
strcpy (object  , "") ;    /* Object      name. */

equinox  = 0.0 ;  /* Equinox (year) for celestial coord system. */
epoch    = 0.0 ;  /* Epoch   (year) for celestial coord system. */

strcpy (extname, "") ;    /* Extension name.      */
extver   = 0 ;          /* Extension version.   */
extlevel = 0 ;          /* Extension level.     */

blank    = 0 ;          /* Blank value. */

b_scale  = 1.0 ;        /* Multiplicative scaling value. */
b_zero   = 0.0 ;        /* Additive scaling value. */

strcpy (bunit , "") ;    /* Name of brightness (magnitude) units. */

strcpy (ctype1, "") ;    /* Name of coordinate axis 1. */
strcpy (ctype2, "") ;    /* Name of coordinate axis 2. */
strcpy (ctype3, "") ;    /* Name of coordinate axis 3. */
strcpy (ctype4, "") ;    /* Name of coordinate axis 4. */

crpix1   = 0.0 ;	/* Location of reference point axis 1 [index]. */
crpix2   = 0.0 ;	/* Location of reference point axis 2 [index]. */
crpix3   = 0.0 ;	/* Location of reference point axis 3 [index]. */
crpix4   = 0.0 ;	/* Location of reference point axis 4 [index]. */

crradius = 0.0 ;	/* Rsun [pixels]. */

crval1   = 0.0 ;	/* Location of reference point axis 1 [coord 1]. */
crval2   = 0.0 ;	/* Location of reference point axis 2 [coord 2]. */
crval3   = 0.0 ;	/* Location of reference point axis 3 [coord 3]. */
crval4   = 0.0 ;	/* Location of reference point axis 4 [coord 4]. */

cdelt1   = 0.0 ;	/* Increment between pixels axis 1 [coord 1]. */
cdelt2   = 0.0 ;	/* Increment between pixels axis 2 [coord 2]. */
cdelt3   = 0.0 ;	/* Increment between pixels axis 3 [coord 3]. */
cdelt4   = 0.0 ;	/* Increment between pixels axis 4 [coord 4]. */

crota1   = 0.0 ;	/* Rotation angle of axis 1 [coord 1]. */
crota2   = 0.0 ;	/* Rotation angle of axis 2 [coord 2]. */
crota3   = 0.0 ;	/* Rotation angle of axis 3 [coord 3]. */
crota4   = 0.0 ;	/* Rotation angle of axis 4 [coord 4]. */

datamin  = 0.0 ;	/* Minimum physical value in data array. */
datamax  = 0.0 ;	/* Maximum physical value in data array. */

dispmin  = 0.0 ;	/* Minimum display value in data array. */
dispmax  = 0.0 ;	/* Maximum display value in data array. */

strcpy (disp_dir, "" );	/* Display direction: R/L U/D O/I */

strcpy (diraxis1, "") ;	/* Scan direction axis 1: "left" or "right". */
strcpy (diraxis2, "") ;	/* Scan direction axis 2: "up"   or "down".  */
strcpy (diraxis3, "") ;	/* Scan direction axis 3. "in"   or "out"    */
strcpy (diraxis4, "") ;	/* Scan direction axis 4. "plus" or "minus"  */

strcpy (dataform, "") ;	/* Data type description. */

code_cp  = 0 ;          /* SMM C/P data code. */
code_dpm = 0 ;          /* MLSO DPM data code. */
code_mk3 = 0 ;          /* MLSO MK3 data code. */
code_obs = 0 ;          /* observation code. */

strcpy (type_cp , "") ;/* SMM C/P data type. */
strcpy (type_dpm, "") ;	/* MLSO DPM data type. */
strcpy (type_mk3, "") ;	/* MLSO MK3 data type. */
strcpy (type_obs, "") ;	/* Type of observation. */
strcpy (type1,    "") ;	/* Type 1. */
strcpy (type2,    "") ;	/* Type 2. */
strcpy (type3,    "") ;	/* Type 3. */
strcpy (type4,    "") ;	/* Type 4. */
strcpy (type5,    "") ;	/* Type 5. */
strcpy (type6,    "") ;	/* Type 6. */
strcpy (type7,    "") ;	/* Type 7. */
strcpy (type8,    "") ;	/* Type 8. */

nimavg   = 0 ;          /* # images averaged. */
expdur   = 0 ;          /* Exposure duration. */
strcpy (expunits, "") ;	/* Exposure duration units. */

rmin     = 0.0 ;        /* Minimum radius. */
rmax     = 0.0 ;        /* Maximum radius. */
rsun     = 0.0 ;        /* pixels per solar radius. */

tmin     = 0.0 ;        /* Minimum angle. */
tmax     = 0.0 ;        /* Maximum angle. */

scan_dur   = 0.0 ;      /* MLSO MK3 scan duration. */
strcpy (scan_dir, "") ;     /* MLSO MK3 scan direction: CW,CCW. */

northang = 0.0 ;        /* Angle of solar north w.r.t. +X axis. */
angle    = 0.0 ;        /* Angle. */

bias     = 0.0 ;        /* Bias. */
gain     = 0.0 ;        /* Gain. */
deltat   = 0.0 ;        /* Delta Time. */
phi      = 0.0 ;        /* Phi angle. */
theta    = 0.0 ;        /* Theta angle. */
radius   = 0.0 ;        /* Radius. */
rangvalu = 0.0 ;        /* Range value. */
zerovalu = 0.0 ;        /* Zero value. */
tilt     = 0.0 ;        /* Tilt angle. */
wave     = 0.0 ;        /* Wavelength. */
wavelen  = 0.0 ;        /* Wavelength. */
wavelnth = 0.0 ;        /* Wavelength. */

strcpy (calibrat, "") ;    /* Calibrated. */
strcpy (camera  , "") ;    /* Camera name. */
strcpy (filter  , "") ;    /* Filter name. */
strcpy (int_mode, "") ;    /* Integration mode. */
strcpy (runit   , "") ;    /* Radius units. */
strcpy (punit   , "") ;    /* Phi angle units. */
strcpy (tunit   , "") ;    /* Theta angle units. */
strcpy (seeing  , "") ;    /* Seeing.  */
strcpy (quality , "") ;    /* Quality. */
strcpy (sky     , "") ;    /* Sky.     */
strcpy (utstart , "") ;    /* UT start time. */
strcpy (utstop  , "") ;    /* UT stop  time. */
strcpy (version , "") ;    /* Version name.  */

frames   = 0 ;          /* # frames. */

occ_disc = 0.0 ;        /* occulting disk diameter [arcsec]. */

solar_b0 = 0.0 ;        /* B-angle. */
solar_l0 = 0.0 ;        /* L-angle. */
solar_p0 = 0.0 ;        /* P-angle. */
hour_ang = 0.0 ;        /* H-angle. */

elev     = 0.0 ;        /* Elevation (degrees). */
azimuth  = 0.0 ;        /* Azimuth   (degrees). */
airmass  = 0.0 ;        /* Airmass. */

strcpy (optdev1 , "") ;     /* MLSO DPM optical device #1 */
strcpy (optdev2 , "") ;     /* MLSO DPM optical device #2 */
strcpy (optdev3 , "") ;     /* MLSO DPM optical device #3 */
strcpy (optdev4 , "") ;     /* MLSO DPM optical device #4 */
strcpy (optdev5 , "") ;     /* MLSO DPM optical device #5 */
strcpy (optdev6 , "") ;     /* MLSO DPM optical device #6 */
strcpy (optdev7 , "") ;     /* MLSO DPM optical device #7 */
strcpy (optdev8 , "") ;     /* MLSO DPM optical device #8 */

strcpy (optpos1 , "") ;     /* MLSO DPM optical position #1 */
strcpy (optpos2 , "") ;     /* MLSO DPM optical position #2 */
strcpy (optpos3 , "") ;     /* MLSO DPM optical position #3 */
strcpy (optpos4 , "") ;     /* MLSO DPM optical position #4 */
strcpy (optpos5 , "") ;     /* MLSO DPM optical position #5 */
strcpy (optpos6 , "") ;     /* MLSO DPM optical position #6 */
strcpy (optpos7 , "") ;     /* MLSO DPM optical position #7 */
strcpy (optpos8 , "") ;     /* MLSO DPM optical position #8 */

calmir  = 0 ;		/* SMM C/P calibration mirror position. */
colorfil= 0 ;		/* SMM C/P color filter position.       */
frame   = 0 ;		/* SMM C/P frame number.                */
glfilter= 0 ;		/* SMM C/P Green Line filter position.  */
polaroid= 0 ;		/* SMM C/P polaroid filter position.    */
sector  = 0 ;		/* SMM C/P sector mirror position.      */
sc_roll = 0 ;		/* SMM C/P spacecraft roll (degrees).   */

/*** GSFC variables: ***/

cd001001 = 0 ;		/* Rotation Matrix. */
cd001002 = 0 ;		/* Rotation Matrix. */
cd001003 = 0 ;		/* Rotation Matrix. */
cd002001 = 0 ;		/* Rotation Matrix. */
cd002002 = 0 ;		/* Rotation Matrix. */
cd002003 = 0 ;		/* Rotation Matrix. */
cd003001 = 0 ;		/* Rotation Matrix. */
cd003002 = 0 ;		/* Rotation Matrix. */
cd003003 = 0 ;		/* Rotation Matrix. */
strcpy (creator, "") ;	/* Creator: program that creates FITS file. */
longpole = 0.0 ;	/* Coordinate Rotation. */
mjd_obs  = 0 ;		/* Julian Date of Observation. */
projp1   = 0.0 ;	/* Projection parameter 1. */
projp2   = 0.0 ;	/* Projection parameter 2. */
strcpy (radecsys, "") ;	/* Reference Frame. */

/*** Skylab variables: ***/

doy_obs  = 0 ;		/* DAY-OF-YEAR : Universal Time (UT). */
strcpy (strayin,  "") ;	/* Stray Light: 'IN' or 'OUT'. */
strcpy (vignetin, "") ;	/* Vignetting : 'IN' or 'OUT'. */
exposure = 0 ;		/* Exposure Duration [seconds]. */
strcpy (filmload, "") ;	/* Film Load: 1,2A,2B,3A,3B,4A,4B,5A,5B. */
rsun_arc = 0.0 ;	/* Radius of sun [arcsec]. */
rsun_mic = 0.0 ;	/* Radius of sun [microns]. */
rsun_pix = 0.0 ;	/* Radius of sun [pixels]. */
film_scl = 0.0 ;	/* Microns per pixel on film. */
strcpy (fidum_lr, "") ;	/* x,y position of lower right fiducial [mic].*/
strcpy (fidup_lr, "") ;	/* x,y position of lower right fiducial [pix].*/
strcpy (fidum_ll, "") ;	/* x,y position of lower left  fiducial [mic].*/
strcpy (fidup_ll, "") ;	/* x,y position of lower left  fiducial [pix].*/
strcpy (fidum_ur, "") ;	/* x,y position of upper right fiducial [mic].*/
strcpy (fidup_ur, "") ;	/* x,y position of upper right fiducial [pix].*/
strcpy (fidum_ul, "") ;	/* x,y position of upper left  fiducial [mic].*/
strcpy (fidup_ul, "") ;	/* x,y position of upper left  fiducial [pix].*/
roll = 0 ;		/* Direction of solar north w.r.t. -Y [arcmin]*/
presunx = 0 ;		/* Pre-flight X sun center [microns]. */
presuny = 0 ;		/* Pre-flight Y sun center [microns]. */
previgx = 0 ;		/* Pre-flight X vignet center [microns]. */
previgy = 0 ;		/* Pre-flight Y vignet center [microns]. */
strycenx = 0 ;		/* Pre-flight X stray light center [microns]. */
stryceny = 0 ;		/* Pre-flight Y stray light center [microns]. */
pesudok = 0.0 ;		/* PES UD error. */
pesrlok = 0.0 ;		/* PES RD error. */
peslim = 0.0 ;		/* PES errors are OK within +- PESLIM. */
iasudok = 0.0 ;		/* IAS UD error. */
iasrlok = 0.0 ;		/* IAS RL error. */
iaslim = 0.0 ;		/* IAS errors are OK within +- IASLIM. */
pesud = 0.0 ;		/* PES UP-DOWN    (Y-axis) measurement. */
pesrl = 0.0 ;		/* PES RIGHT-LEFT (X-axis) measurement. */
iasud = 0.0 ;		/* IAS UP-DOWN    (Y-axis) measurement. */
iasrl = 0.0 ;		/* IAS RIGHT-LEFT (X-axis) measurement. */
sunoffdx = 0.0 ;	/* X center offset for film load [microns]. */
sunoffdy = 0.0 ;	/* Y center offset for film load [microns]. */
suncenx = 0.0 ;		/* True X sun center [microns]. */
sunceny = 0.0 ;		/* True Y sun center [microns]. */
vigcenx = 0.0 ;		/* Vignetting X center [microns]. */

return (0) ;
}
