/* hdu_decode.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decode FITS Header Data Unit record.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   7 March 1994
 * 22 April 1994: add keywords: TYPE-OBS, etc., for disk images (BBSO, KPNO)
 * 2 June 1994: change names: bangle->solar_b0, hangle->solar_h0,
 *                            langle->solar_l0, pangle->solar_p0.
 * 18 August 1994: change : SOLAR_H0 -> HOUR-ANG
 *                          SCNDIR   -> SCAN-DIR, SCNDUR   -> SCAN-DUR
 *                          ANGNORTH -> NORTHANG, SCROLL   -> SC-ROLL
 *		   add    : CODE-OBS, CRRADIUS, DISP-DIR, NIMAVG, OCC-DISC
 *                 remove : CPVER, DPMVER, MK3VER
 * 22 August 1994: add    : CODE-CP, CODE-DPM, CODE-MK3
 * 11 March 1998: Add keywords for Skylab ATM/WLCE.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "fitslib.ext"
#include "fitskey.h"

hdu_decode (linbuf)

char *linbuf ;

{
   int i ;
   int ibeg ;
   int ikey ;				/* Keyword index. */
   static char key [12] ;		/* Keyword [ASCII string]. */
   static char cbuf [84];		/* line buffer. */
   static char cparam  [72] ;		/* Parameter [ASCII string]. */
   static char cparam1 [72] ;		/* Parameter [ASCII string]. */
   short sparam ;			/* Short parameter. */
   long  lparam ;			/* Long  parameter. */
   float fparam ;			/* Float parameter. */

   strncpy (cbuf, linbuf, 80) ;
   cbuf [80] = '\0' ;			/* Terminate string. */
   strncpy (cparam, &cbuf [10], 70) ;	/* Character parameter. */
   cparam [70] = '\0' ;			/* Terminate string. */

   /*** Discard comment. ***/

   for (i = 20; i < strlen (cparam) - 1; i++)
/*      if (cparam [i] == '/' && cparam [i+1] == ' ') */
      if (cparam [i] == '/')
	 {
	 cparam [i] = '\0' ;
	 break ;
	 }

   squeeze (cparam, 047) ;		/* Remove all quotes from cparam. */

   /*** Replace trailing blanks with NULL. ***/

   for (i = strlen (cparam) - 1; i >= 0; i--)
      {
      if (cparam [i] != ' ')  break ;
      cparam [i] = '\0' ;
      }

/*
   for (i = 0; i = strlen (cparam) - 1; i++)
      {
      if (cparam [i] != ' ')  
	 {
	 ibeg = i ;
	 break ;
	 }
      }
   if (ibeg > 0)
      {
      strcpy (cparam1, cparam + ibeg) ;
      strcpy (cparam, cparam1) ;
      }
 */

/*   squeeze (cparam, ' ') ; */		/* Remove all blanks from cparam. */

   sparam = (short) atoi (cparam) ;		/* Short int parameter. */
   lparam = (int)   atol (cparam) ;		/* Long  int parameter. */
   fparam = (float) atof (cparam) ;		/* Floating point parameter. */

   sscanf (cparam, "%hd", &sparam) ;
   sscanf (cparam, "%d", &lparam) ;
   sscanf (cparam, "%e", &fparam) ;

   strncpy (key, cbuf, 8) ;
   squeeze (key, ' ') ;			/* remove all blanks from key. */

   ikey = -1 ;
   for (i = 0; i < numkey; i++)
      {
      if (strcmp (key, &fitskey [i][0]) == 0)
         ikey = i ;
      }
/*   fprintf (stdout, "key:|%s|cparam:|%s|ikey:%3d\n", key, cparam, ikey) ; */

   switch (ikey)
      {
      case   0:	/*          */
						break ;
      case   1:	/* AIRMASS  */
		airmass  = fparam ;		break ;	/* MLSO DPM  */
      case   2:	/* ANGLE    */
		angle    = fparam ;		break ;	/*           */
      case   3:	/* AUTHOR   */
		strcpy (author,   cparam) ;	break ;
      case   4:	/* AZIMUTH  */
		azimuth  = fparam ;		break ;	/* MLSO DPM  */

      case   5:	/* BIAS     */
		bias     = fparam ;		break ;	/* KPNO      */
      case   6:	/* BITPIX   */
		bitpix   = sparam ;		break ;
      case   7:	/* BLANK    */
		blank    = lparam ;		break ;
      case   8:	/* BLOCKED  */
		blocked  = 1 ;			break ;
      case   9:	/* BSCALE   */
		b_scale   = fparam ;		break ;
      case  10:	/* BUNIT    */
		strcpy (bunit,    cparam) ;	break ;
      case  11:	/* BZERO    */
		b_zero    = fparam ;		break ;

      case  12:	/* CALIBRAT */
		strcpy (calibrat, cparam) ;	break ;
      case  13:	/* CALMIR   */
		calmir   = sparam ;		break ;
      case  14:	/* CAMERA   */
		strcpy (camera,   cparam) ;	break ;
      case  15: /* CD001001 */
		strcpy (cd001001, cparam) ;	break ;
      case  16: /* CD001002 */
		strcpy (cd001002, cparam) ;	break ;
      case  17: /* CD001003 */
		strcpy (cd001003, cparam) ;	break ;
      case  18: /* CD002001 */
		strcpy (cd002001, cparam) ;	break ;
      case  19: /* CD002002 */
		strcpy (cd002002, cparam) ;	break ;
      case  20: /* CD002003 */
		strcpy (cd002003, cparam) ;	break ;
      case  21: /* CD003001 */
		strcpy (cd003001, cparam) ;	break ;
      case  22: /* CD003002 */
		strcpy (cd003002, cparam) ;	break ;
      case  23: /* CD003003 */
		strcpy (cd003003, cparam) ;	break ;
      case  24:	/* CDELT1   */
		cdelt1   = fparam ;		break ;
      case  25:	/* CDELT2   */
		cdelt2   = fparam ;		break ;
      case  26:	/* CDELT3   */
		cdelt3   = fparam ;		break ;
      case  27:	/* CODE-CP  */
		code_cp  = sparam ;		break ;	/* SMM C/P data code. */
      case  28:	/* CODE-DPM */
		code_dpm = sparam ;		break ;	/* MLSO DPM data code.*/
      case  29:	/* CODE-MK3 */
		code_mk3 = sparam ;		break ;	/* MLSO MK3 data code.*/
      case  30:	/* CODE-OBS */
		code_obs = sparam ;		break ;	/* observation code. */
      case  31:	/* COLORFIL */
		colorfil = sparam ;		break ;
      case  32:	/* COMMENT  */
						break ;
      case  33:	/* CREATOR  */
		strcpy (creator, cparam) ;	break ;
      case  34:	/* CROTA1   */
		crota1   = fparam ;		break ;
      case  35:	/* CROTA2   */
		crota2   = fparam ;		break ;
      case  36:	/* CROTA3   */
		crota3   = fparam ;		break ;
      case  37:	/* CRPIX1   */
		crpix1   = fparam ;		break ;
      case  38:	/* CRPIX2   */
		crpix2   = fparam ;		break ;
      case  39: /* CRPIX3   */
		crpix3   = fparam ;		break ;
      case  40: /* CRRADIUS */
		crradius = fparam ;		break ;	/* Rsun [pixels]. */
      case  41:	/* CRVAL1   */
		crval1   = fparam ;		break ;
      case  42:	/* CRVAL2   */
		crval2   = fparam ;		break ;
      case  43:	/* CRVAL3   */
		crval3   = fparam ;		break ;
      case  44:	/* CTYPE1   */
		strcpy (ctype1,   cparam) ;	break ;
      case  45:	/* CTYPE2   */
		strcpy (ctype2,   cparam) ;	break ;
      case  46:	/* CTYPE3   */
		strcpy (ctype3,   cparam) ;	break ;

      case  47:	/* DATAFORM */
		strcpy (dataform, cparam) ;	break ; /* HAO       */
      case  48:	/* DATAMAX  */
		datamax  = fparam ;		break ;
      case  49:	/* DATAMIN  */
		datamin  = fparam ;		break ;
      case  50:	/* DATE     */
		strcpy (date,     cparam) ;	break ;
      case  51:	/* DATE-CAL */
		strcpy (date_cal, cparam) ;	break ;	/* HAO       */
      case  52:	/* DATE-OBS */
		strcpy (date_obs, cparam) ;	break ;	/* HAO       */
      case  53:	/* DATE-OPC */
		strcpy (date_opc, cparam) ;	break ;	/* MLSO MK3  */
      case  54:	/* DELTAT   */
		deltat   = fparam ;		break ;
      case  55:	/* DIRAXIS1 */
		strcpy (diraxis1, cparam) ;	break ;	/* direction axis 1 */
      case  56:	/* DIRAXIS2 */
		strcpy (diraxis2, cparam) ;	break ;	/* direction axis 2 */
      case  57:	/* DIRAXIS3 */
		strcpy (diraxis3, cparam) ;	break ;	/* direction axis 3 */
      case  58: /* DISP-DIR */
		strcpy (disp_dir, cparam) ;	break ;	/* display direction. */
      case  59:	/* DISPMAX  */
		dispmax  = fparam ;		break ;	/* HAO       */
      case  60:	/* DISPMIN  */
		dispmin  = fparam ;		break ;	/* HAO       */
      case  61:	/* DOY-OBS  */
		doy_obs = sparam ;		break ;	/* Day-Of-Year [UT]. */

      case  62:	/* ELEV     */
		elev     = fparam ;		break ;	/* MLSO DPM  */
      case  63:	/* END      */
		end_fts  = 1 ;			break ;
      case  64: /* EPOCH    */
		epoch    = fparam ;		break ;
      case  65:	/* EQUINOX  */
		equinox  = fparam ;		break ;
      case  66:	/* EXPOSURE */
		exposure = fparam ;		break ;	/* HAO       */
      case  67:	/* EXPDUR   */
		expdur   = fparam ;		break ;	/* HAO       */
      case  68:	/* EXPUNITS */
		strcpy (expunits, cparam) ;	break ;	/* HAO       */
      case  69:	/* EXTEND   */
		extend   = 1 ;			break ;
      case  70:	/* EXTLEVEL */
		extlevel = lparam ;		break ;
      case  71:	/* EXTNAME  */
		strcpy (extname,  cparam) ;	break ;
      case  72:	/* EXTVER   */
		extver   = lparam ;		break ;

      case  73:	/* FIDUM-LL */
      		strcpy (fidum_ll, cparam) ;	break ;
      case  74:	/* FIDUM-LR */
		strcpy (fidum_lr, cparam) ;	break ;
      case  75:	/* FIDUM-UL */
		strcpy (fidum_ul, cparam) ;	break ;
      case  76:	/* FIDUM-UR */
		strcpy (fidum_ur, cparam) ;	break ;
      case  77:	/* FIDUP-LL */
      		strcpy (fidum_ll, cparam) ;	break ;
      case  78:	/* FIDUP-LR */
		strcpy (fidum_lr, cparam) ;	break ;
      case  79:	/* FIDUP-UL */
		strcpy (fidum_ul, cparam) ;	break ;
      case  80:	/* FIDUP-UR */
		strcpy (fidum_ur, cparam) ;	break ;
      case  81:	/* FILMLOAD */
		strcpy (filmload, cparam) ;	break ;
      case  82:	/* FILM-SCL */
		film_scl = fparam ;		break ;
      case  83:	/* FILTER   */
		strcpy (filter,   cparam) ;	break ;
      case  84:	/* FRAME    */
		frame    = lparam ;		break ;
      case  85:	/* FRAMES   */
		frames   = lparam ;		break ;

      case  86:	/* GAIN     */
		gain     = fparam ;		break ;
      case  87:	/* GCOUNT   */
		gcount   = lparam ;		break ;
      case  88:	/* GLFILTER */
		glfilter = sparam ;		break ;

      case  89:	/* HISTORY  */
						break ;
      case  90:	/* HOUR-ANG */
		hour_ang   = fparam ;		break ;	/* MLSO DPM  */

      case  91:	/* IASLIM   */
		iaslim  = fparam ;		break ;
      case  92:	/* IASRL    */
		iasrl   = fparam ;		break ;
      case  93:	/* IASRLOK  */
		iasrlok = fparam ;		break ;
      case  94:	/* IASUD    */
		iasud   = fparam ;		break ;
      case  95:	/* IASUDOK  */
		iasudok = fparam ;		break ;
      case  96:	/* INSTRUME */
		strcpy (instrume, cparam) ;	break ;
      case  97:	/* INT-MODE */
		strcpy (int_mode, cparam) ;	break ;

      case  98:	/* LONGPOLE */
		longpole = fparam ;		break ;

      case  99:	/* MJD-OBS  */
		mjd_obs  = fparam ;		break ;

      case 100:	/* NAXIS    */
		naxis    = sparam ;		break ;
      case 101:	/* NAXIS1   */
		naxis1   = sparam ;		break ;
      case 102:	/* NAXIS2   */
		naxis2   = sparam ;		break ;
      case 103:	/* NAXIS3   */
		naxis3   = sparam ;		break ;
      case 104:	/* NIMAVG   */
		nimavg   = sparam ;		break ;	/* # images averaged. */
      case 105:	/* NORTHANG */
		northang = fparam ;		break ;	/* HAO       */

      case 106:	/* OBJECT   */
		strcpy (object,   cparam) ;	break ;
      case 107:	/* OBSERVER */
		strcpy (observer, cparam) ;	break ;
      case 108:	/* OCC-DISC */
		occ_disc = fparam ;		break ;	/* MLSO DPM  */
      case 109:	/* OPTDEV1  */
		strcpy (optdev1,  cparam) ;	break ; /* MLSO DPM  */
      case 110:	/* OPTDEV2  */
		strcpy (optdev2,  cparam) ;	break ; /* MLSO DPM  */
      case 111:	/* OPTDEV3  */
		strcpy (optdev3,  cparam) ;	break ; /* MLSO DPM  */
      case 112:	/* OPTDEV4  */
		strcpy (optdev4,  cparam) ;	break ; /* MLSO DPM  */
      case 113:	/* OPTDEV5  */
		strcpy (optdev5,  cparam) ;	break ; /* MLSO DPM  */
      case 114:	/* OPTDEV6  */
		strcpy (optdev6,  cparam) ;	break ; /* MLSO DPM  */
      case 115:	/* OPTDEV7  */
		strcpy (optdev7,  cparam) ;	break ; /* MLSO DPM  */
      case 116:	/* OPTDEV8  */
		strcpy (optdev8,  cparam) ;	break ; /* MLSO DPM  */
      case 117:	/* OPTPOS1  */
		strcpy (optpos1,  cparam) ;	break ; /* MLSO DPM  */
      case 118:	/* OPTPOS2  */
		strcpy (optpos2,  cparam) ;	break ; /* MLSO DPM  */
      case 119:	/* OPTPOS3  */
		strcpy (optpos3,  cparam) ;	break ; /* MLSO DPM  */
      case 120:	/* OPTPOS4  */
		strcpy (optpos4,  cparam) ;	break ; /* MLSO DPM  */
      case 121:	/* OPTPOS5  */
		strcpy (optpos5,  cparam) ;	break ; /* MLSO DPM  */
      case 122:	/* OPTPOS6  */
		strcpy (optpos6,  cparam) ;	break ; /* MLSO DPM  */
      case 123:	/* OPTPOS7  */
		strcpy (optpos7,  cparam) ;	break ; /* MLSO DPM  */
      case 124:	/* OPTPOS8  */
		strcpy (optpos8,  cparam) ;	break ; /* MLSO DPM  */
      case 125:	/* ORIGIN   */
		strcpy (origin,   cparam) ;	break ;

      case 126:	/* PESLIM   */
		peslim   = fparam ;	break ;
      case 127:	/* PESRL    */
		pesrl    = fparam ;	break ;
      case 128:	/* PESRLOK  */
		pesrlok  = fparam ;	break ;
      case 129:	/* PESUD    */
		pesud    = fparam ;	break ;
      case 130:	/* PESUDOK  */
		pesudok  = fparam ;	break ;
      case 131:	/* PCOUNT   */
		pcount   = lparam ;		break ;
      case 132:	/* PHI      */
		phi      = fparam ;		break ;
      case 133:	/* POLAROID */
		polaroid = sparam ;		break ;	/* HAO SMM C/P  */
      case 134: /* PRESUNX  */
		presunx  = sparam ;		break ;
      case 135: /* PRESUNY  */
		presuny  = sparam ;		break ;
      case 136: /* PREVIGX  */
		previgx  = sparam ;		break ;
      case 137: /* PREVIGY  */
		previgy  = sparam ;		break ;
      case 138:	/* PROJP1   */
		projp1 =  fparam ;		break ;
      case 139:	/* PROJP2   */
		projp2 =  fparam ;		break ;
      case 140:	/* PUNIT    */
		strcpy (punit,    cparam) ;	break ;

      case 141:	/* QUALITY  */
		strcpy (quality,  cparam) ;	break ;

      case 142:	/* RADECSYS */
		strcpy (radecsys, cparam) ;	break ;
      case 143:	/* RADIUS   */
		radius   = fparam ;		break ;
      case 144:	/* RANGVALU */
		rangvalu = fparam ;		break ;
      case 145:	/* REFERENC */
		strcpy (referenc, cparam) ;	break ;
      case 146:	/* RMAX     */
		rmax     = fparam ;		break ;	/* MLSO MK3  */
      case 147:	/* RMIN     */
		rmin     = fparam ;		break ;	/* MLSO MK3  */
      case 148:	/* ROLL     */
		roll     = fparam ;		break ;	/* MLSO MK3  */
      case 149:	/* RSUN     */
		rsun     = fparam ;		break ;	/* HAO       */
      case 150:	/* RSUN-ARC */
		rsun_arc = fparam ;		break ;	/* HAO WLCE  */
      case 151:	/* RSUN-MIC */
		rsun_mic = fparam ;		break ;	/* HAO WLCE  */
      case 152:	/* RSUN-PIX */
		rsun_pix = fparam ;		break ;	/* HAO WLCE  */
      case 153:	/* RUNIT    */
		strcpy (runit,    cparam) ;	break ;

      case 154:	/* SECTOR   */
		sector   = sparam ;		break ;	/* HAO SMM C/P  */
      case 155:	/* SEEING   */
		strcpy (seeing,   cparam) ;	break ;
      case 156:	/* SCAN-DIR */
		strcpy (scan_dir, cparam) ;	break ; /* MLSO MK3  */
      case 157:	/* SCAN_DUR */
		scan_dur = fparam ;		break ;	/* MLSO MK3  */
      case 158:	/* SC_ROLL  */
		sc_roll  = sparam ;		break ;	/* HAO SMM C/P */
      case 159:	/* SIMPLE   */
		if (cparam [0] == 'T') simple =1; else simple = 0; break;
      case 160:	/* SKY      */
		strcpy (sky,      cparam) ;	break ; /*           */
      case 161:	/* SOLAR_B0 */
		solar_b0 = fparam ;		break ;	/* MLSO DPM  */
      case 162:	/* SOLAR_L0 */
		solar_l0 = fparam ;		break ;	/* MLSO DPM  */
      case 163:	/* SOLAR_P0 */
		solar_p0 = fparam ;		break ;	/* MLSO DPM  */
      case 164:	/* STRAYIN  */
		strcpy (strayin, cparam) ;	break ;	/* HAO WLCE  */
      case 165:	/* STRYCENX */
		strycenx = fparam ;		break ;	/* HAO WLCE  */
      case 166:	/* STRYCENY */
		stryceny = fparam ;		break ;	/* HAO WLCE  */
      case 167:	/* SUNCENX  */
		suncenx  = fparam ;		break ;	/* HAO WLCE  */
      case 168:	/* SUNCENY  */
		sunceny  = fparam ;		break ;	/* HAO WLCE  */
      case 169:	/* SUNOFFDX */
		sunoffdx = fparam ;		break ;	/* HAO WLCE  */
      case 170:	/* SUNOFFDY */
		sunoffdy = fparam ;		break ;	/* HAO WLCE  */

      case 171:	/* TELESCOP */
		strcpy (telescop, cparam) ;	break ;
      case 172:	/* THETA    */
		theta    = fparam ;		break ;	/* MLSO MK3  */
      case 173:	/* TIME     */
		strcpy (time_fts, cparam) ;	break ;
      case 174:	/* TIME-CAL */
		strcpy (time_cal, cparam) ;	break ;	/* HAO       */
      case 175:	/* TIME-OBS */
		strcpy (time_obs, cparam) ;	break ;
      case 176:	/* TIME-OPC */
		strcpy (time_opc, cparam) ;	break ;	/* MLSO MK3  */
      case 177:	/* TILT     */
		tilt     = fparam ;		break ;	/*           */
      case 178:	/* TMAX     */
		tmax     = fparam ;		break ;	/* MLSO MK3  */
      case 179:	/* TMIN     */
		tmin     = fparam ;		break ;	/* MLSO MK3  */
      case 180:	/* TUNIT    */
		strcpy (tunit,    cparam) ;	break ;	/*           */
      case 181:	/* TYPE-CP
		strcpy (type_cp,  cparam) ;	break ;	/* SMM  C/P  */
      case 182:	/* TYPE-DPM */
		strcpy (type_dpm, cparam) ;	break ;	/* MLSO DPM  */
      case 183:	/* TYPE-MK3 */
		strcpy (type_mk3, cparam) ;	break ;	/* MLSO MK3  */
      case 184:	/* TYPE-OBS */
		strcpy (type_obs, cparam) ;	break ;	/*           */
      case 185:	/* TYPE1    */
		strcpy (type1,    cparam) ;	break ;	/*           */
      case 186:	/* TYPE2    */
		strcpy (type2,    cparam) ;	break ;	/*           */
      case 187:	/* TYPE3    */
		strcpy (type3,    cparam) ;	break ;	/*           */
      case 188:	/* TYPE4    */
		strcpy (type4,    cparam) ;	break ;	/*           */
      case 189:	/* TYPE5    */
		strcpy (type5,    cparam) ;	break ;	/*           */
      case 190:	/* TYPE6    */
		strcpy (type6,    cparam) ;	break ;	/*           */
      case 191:	/* TYPE7    */
		strcpy (type7,    cparam) ;	break ;	/*           */
      case 192:	/* TYPE8    */
		strcpy (type8,    cparam) ;	break ;	/*           */

      case 193:	/* UTSTART  */
		strcpy (utstart,  cparam) ;	break ;	/*           */
      case 194:	/* UTSTOP   */
		strcpy (utstop,   cparam) ;	break ;	/*           */

      case 195:	/* VERSION  */
		strcpy (version,  cparam) ;	break ;	/*           */
      case 196:	/* VIGCENX  */
		vigcenx  = fparam ;		break ;	/* HAO WLCE  */
      case 197:	/* VIGCENY  */
		vigceny  = fparam ;		break ;	/* HAO WLCE  */
      case 198:	/* VIGNETIN */
		strcpy (vignetin, cparam) ;	break ;	/* HAO WLCE  */

      case 199:	/* WAVE     */
		wave     = fparam ;		break ;	/* wavelength */
      case 200:	/* WAVELEN  */
		wavelen  = fparam ;		break ;	/* wavelength */
      case 201:	/* WAVELNTH */
		wavelnth = fparam ;		break ;	/* wavelength */

      case 202:	/* XTENSION */
		strcpy (xtension, cparam) ;	break ;	/* FITS extension */

      case 203:	/* ZEROVALU */
		zerovalu = fparam ;		break ;	/*            */
      }
}
