/* rdfits.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read FITS file: HDU & data.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 March 1994
 * 22 Dec 1997: Add naxis3 & naxis4 to computation for nbyimg.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "fitslib.ext"

#define NBYFIT 2880
#define NCHLIN 80
#define NLINES 36

unsigned char *
rdfits (namfit)

char namfit [] ;		/* Name of FITS file. */
{
   FILE *FD ;			/* file descriptor. */
   unsigned char *imgbuf ;	/* Image buffer. */
   int nbyhdu ;			/* # bytes in FITS hdu(s). */
   int nbyimg ;			/* # bytes in image. */

   /*** Read FITS header data unit(s). ***/

   nbyhdu = rdhdu (namfit) ;

   /*** Open FITS file. ***/

   if ((FD = fopen (namfit, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open FITS file: %s\n", namfit) ;
      exit (-1) ;
      }
   fseek (FD, nbyhdu, 0) ;

   nbyimg = naxis1 * naxis2 * naxis3 * naxis4 * bitpix/8 ;
   imgbuf = (unsigned char *) malloc (nbyimg) ;
   fread (imgbuf, sizeof (char), nbyimg, FD) ;

   fclose (FD) ;

   return (imgbuf) ;
}
