/* wlcevigfts.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Re-apply vignetting to Skylab/ATM WLCE coronagraph images (FITS format).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   11 March 1998
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include "fitslib.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern char *gethdu () ;
   extern unsigned char *rdfits () ;
   extern float wlce_vignet () ;
   static char namimg [80] ;
   static char namout [80] ;
   unsigned char *imgbuf ;
   unsigned char *padbuf ;
   unsigned short *sbuf ;
   unsigned short sval ;
   unsigned short smin, smax ;
   FILE *FDOUT ;

   char *hdu ;
   int i ;
   int idot ;
   int ix, iy ;
   int nx, ny ;
   int nbypad ;
   int nbypix ;
   int nbyhdu ;
   int nbyimg ;
   int nrec ;

   float pixrs ;
   float rpix ;
   float rsun ;
   float skyroll ;
   float vigfac ;
   float xvcen, yvcen ;
   float x, y ;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stdout, "%s : Re-apply vignetting to calibrated WLCE image.\n",
	       argv [0]) ;
      fprintf (stdout, "usage: %s calibrated_wlce_fits_file\n", argv [0]) ;
      exit (-1) ;
      }

   /*** Read FITS header. ***/

   hdu = gethdu (namimg, &nbyhdu) ;		/* Get FITS header. */
   fprintf (stdout, "nbyhdu: %d\n", nbyhdu) ;
/*
 *   xvcen = (naxis1 - 1) - suncenx / 50.0 ;
 *   yvcen = sunceny / 50.0 ;
 */
   fprintf (stdout, "xvcen: %7.2f  yvcen: %7.2f\n", xvcen, yvcen) ;
   xvcen = (naxis1 - 1) - vigcenx / 50.0 ;
   yvcen = vigceny / 50.0 ;
   fprintf (stdout, "xvcen: %7.2f  yvcen: %7.2f\n", xvcen, yvcen) ;

   /*** Read FITS image. ***/

   imgbuf = rdfits (namimg) ;
   nx = naxis1 ;
   ny = naxis2 ;
   nbypix = bitpix / 8 ;
   pixrs = 40.0 ;
   sbuf = (unsigned short *) imgbuf ;

   idot = strlen (namimg) ;
   for (i = 0; i < strlen (namimg); i++)
      {
      if (namimg [i] == '.')  idot = i ;
      }

   strncpy (namout, namimg, idot) ;
   strcat (namout, ".vig.fts") ;
   fprintf (stdout, "namout: %s\n", namout) ;

   /*** Apply WLCE Vignetting to image. ***/

   smin = 65535 ;
   smax =     0 ;
   for (iy = 0 ; iy < ny ; iy++)
      {
      y = (float) (iy) - yvcen ;
      for (ix = 0 ; ix < nx ; ix++)
	 {
	 x = (float) (ix) - xvcen ;
         rpix = fsqrt (x * x + y * y) ;
	 rsun = rpix / pixrs ;
	 vigfac = wlce_vignet (rsun) ;
	 sval = *(sbuf + ix + iy * nx) *= vigfac ;
	 if (sval < smin)  smin = sval ;
	 if (sval > smax)  smax = sval ;
	 if (ix == iy)
	    {
	    fprintf (stdout, "ix:%3d  rpix:%5.2f  rsun:%7.3f  vignet:%7.3f\n",
		     ix, rpix, rsun, vigfac) ;
	    }
	 }
      }
   fprintf (stdout, "smin: %d  smax: %d\n", smin, smax) ;

   ftsaddpar (hdu, "VIGNETIN", "Y", "char", "VIGNETTING FUNCTION IS STILL IN IF 'Y'") ;

   /*** Write vignetted WLCE image to disk in FITS format. ***/

   FDOUT = fopen (namout, "w") ;
   fwrite (hdu, nbyhdu, 1, FDOUT) ;
   nbyimg = naxis1 * naxis2 * bitpix / 8 ;
   fwrite (imgbuf, nbyimg, 1, FDOUT) ;

   if (((nbyimg / 2880) * 2880) != nbyimg)
      {
      nrec = (nbyimg / 2880) + 1 ;
      nbypad = nrec * 2880 - nbyimg ;
      padbuf = (unsigned char *) malloc (nbypad) ;
      bzero (padbuf, nbypad) ;
      fwrite (padbuf, nbypad, 1, FDOUT) ;
      free (padbuf) ;
      }

   fclose (FDOUT) ;
   free (imgbuf) ;

   exit (0) ;
}
