/* slint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Interpolation [horizontal/vertical] across a scan line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Given the end point coordinates, generate a line scan,
 * then interpolate, either horizontally or vertically, over the scan line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 December 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

slint (cbuf, nbypx, xd, yd, xbeg, ybeg, xend, yend, nint, dirint)

unsigned char *cbuf;		/* Pointer to 1 byte image array. */
int   nbypx;			/* # bytes per pixel. */
int   xd, yd;			/* Dimensions of cbuf. */
int   xbeg, ybeg, xend, yend ;	/* x/y coordinate scan limits. */
int   nint;			/* # pixels over which to interpolate. */
char  dirint [];		/* Direction of interpolation [h,v]. */
{
   int   i ;			/* index. */
   int   *xit ;			/* X coordinates (top hemisphere). */
   int   *yit ;			/* Y coordinates (top hemisphere). */
   int   *xib ;			/* X coordinates (bottom hemisphere). */
   int   *yib ;			/* Y coordinates (bottom hemisphere). */

   int   intdim;		/* Dimension of interpolation arrays. */
   int   ns;			/* Number of values in scan. */
   int   ds;			/* Distance along scan line. */
   int   scandim;		/* Dimension of scan arrays. */
   int   xs, ys;		/* Differences: xend - xbeg, yend - ybeg. */

   float pi = 3.14159265 ;
   float *scanpix ;		/* scan pixel intensity. */
   int   *scanx ;		/* x-axis scan position. */
   int   *scany ;		/* y-axis scan position. */

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*** Allocate storage for scan arrays. ***/

   xs = xend - xbeg + 1;
   ys = yend - ybeg + 1;
   ds = (int) ( sqrt ( (double) (xs * xs + ys * ys) ) + 0.5);
   ns = scandim = ds;

   scanpix = (float *)  malloc (scandim * sizeof (float)) ;
   scanx   = (int *)    malloc (scandim * sizeof (int)) ;
   scany   = (int *)    malloc (scandim * sizeof (int)) ;

   /*** Do line scan & interpolate either horizontally or vertically
    *** along a line centered at each scan pixel.
    ***/

   clscan (cbuf, nbypx, xbeg, ybeg, xend, yend, scanpix, scanx, scany, &ns) ;

   scanint (cbuf, nbypx, xd, yd, scanx, scany, ns, nint, dirint) ;

   free (scanpix) ;
   free (scanx) ;
   free (scany) ;
   return (0);
}
