/* imgave.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average multiple images together.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 31 October 1985
 * 11 February 1991: modify for use on Yoda (Sun-4/280).
 * 11 June 1991: modify for SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables */

#include "/local/home/stanger/lib/include/cplib.h"
#include "/local/home/stanger/lib/include/img_scale.h"

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <sys/types.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];

{

char	*malloc();	/* Memory allocation routine. */
char	*rdimg();	/* Read image from disk to memory. */
char	*outname();	/* Create new file name for output image. */
char	*imgbuf;	/* Pointer to input image array. */
char	*imgsum;	/* Pointer to summation image array. */
float	*imgadd;	/* Pointer to summation image array. */
char	*hkbuf;		/* Pointer to housekeeping array. */
char	*bitmap;	/* Pointer to bit map array. */
char	*imgbyt;	/* Pointer to output image (1 byte per pixel). */
short	*imgint;	/* Pointer to output image (2 bytes per pixel). */
float	*imgflt;	/* Pointer to output image (4 bytes per pixel). */
static	char	namlst[80];	/* File containing image list. */
static	char	namimg[80];	/* Input image file name. */
static	char	namout[80];	/* Output image file name. */
static	char	answer[2];	/* Storage for y/n response. */
int imgfd;		/* File descriptor for first image. */
int avgfd;		/* File descriptor for output image. */
int nread, nclose;	/* Status variables. */
int nwrite;		/* Counter for # bytes written. */
int nave;		/* Number of images to be averaged. */
int imgsiz;		/* # bytes in summation array. */
int nelem;		/* # pixels in image array. */
int nbyhk;		/* # bytes in housekeeping record. */
int nbylin;		/* # bytes per line in input image. */
int nbymap;		/* # bytes in bitmap record. */
int nbyimg;		/* Size of output image. */
int nfblin;
int nrechd;		/* # logical records in header. */
int i;			/* General purpose index. */
float invnum;		/* Reciprocal of # of images to be averaged. */
FILE *LIST;

   if (argc == 3)
      {
      strcpy (namlst, argv [1]);
      strcpy (namout, argv [2]);
      }
   else
   if (argc == 2)
      {
      fprintf (stderr, "usage: %s [filename_list] [outfile_name]\n", argv [0]);
      strcpy (namlst, argv [1]);
      fprintf (stderr, "Enter output file name: ");
      fscanf  (stdin,  "%s", namout);
      }
   else
      {
      fprintf (stderr, "usage: %s [filename_list] [outfile_name]\n", argv [0]);
      fprintf (stderr, "Enter name of file containing images to be averaged : ");
      fscanf  (stdin,  "%s", namlst);
      fprintf (stderr, "Enter output file name: ");
      fscanf  (stdin,  "%s", namout);
      }

   /*** Open file containing image list. ***/

   if ((LIST = fopen (namlst,"r"))==NULL)
      {
      fprintf (stderr,"cannot open %s\n", namlst);
      exit (1);
      }

   /*** Open input image disk file and read housekeeping record. ***/

   if (fscanf (LIST,"%s", namimg) == EOF)
	{
	fprintf (stderr, "Empty file: %s\n", namlst);
	exit(-1);
	}
/*
 *   strcpy (namout, outname (namimg));
*/
   nave ++;
   cphskp (namimg);
   imgfd = open (namimg, 0);
   fprintf (stdout, "%s  npix: %d  nbypix: %d\n", namimg, npix, nbypix);

   nbylin = npixl * nbypix;
   nfblin = npixl * sizeof (float);
   if (nbypix == 4)
      {
      nbyhk = npixl;
      nbymap = nfblin - npixl;
      nrechd = 1;
      }
   else
      {
      nbyhk = nbymap = nbylin;
      nrechd = 2;
      }

   hkbuf = malloc (nbyhk);
   bitmap = malloc (nbymap);
   imgbuf = malloc (npix*npix*nbypix);

   imgbyt = (char *) imgbuf;
   imgint = (short *) imgbuf;
   imgflt = (float *) imgbuf;

   nread = read (imgfd, hkbuf, nbyhk);
   nread = read (imgfd, bitmap, nbymap);
   nread = read (imgfd, imgbuf, npix*npix*nbypix);
   nclose = close (imgfd);

   /*** Convert image to low res. ***/

/*
   if ( H.ires == 0 )
	{
	smm_avg (imgbuf);
	npix /= 2;
	}
 */
   hkline (stdout);

   printf ("ih,iv: %d %d\n", H.ih, H.iv);
   printf ("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
   printf ("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
   printf ("xcen,ycen: %f %f\n", xcen, ycen);

   /*** Allocate memory storage for summation array. ***/

   imgsiz = (npix * npix * sizeof (float));
   nelem = (npix * npix);
   printf ("nelem: %d\n", nelem);
   if ((imgsum = malloc (imgsiz)) == NULL)
     {  fprintf (stderr,"imgsum: malloc error.\n");
        exit (-1);
     }
   imgadd = (float *) imgsum;

   /*** Copy first image into summation array. ***/

   fprintf (stdout, "Copy %s into summation array.\n", namimg);

   if (nbypix == 1)
      for (i=0; i<nelem; i++)  *(imgadd + i) = (float) (*(imgbyt + i) & 0377);
   if (nbypix == 2)
      for (i=0; i<nelem; i++)  *(imgadd + i) = (float) (*(imgint + i) & 0177777);
   if (nbypix == 4)
      for (i=0; i<nelem; i++)  *(imgadd + i) = (float) *(imgflt + i);
   free (imgbuf);

   /*** Fetch images and compute image sum. ***/

   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      cphskp (namimg);
      imgbuf = rdimg (namimg);
      imgbyt = (char *)  imgbuf;
      imgint = (short *) imgbuf;
      imgflt = (float *) imgbuf;

      /*** Convert image to low res. ***/

/*
      if ( H.ires == 0 )
         {
         smm_avg (imgbuf);
         npix /= 2;
         }
 */
      hkline (stdout);

      /*** Add image to summation array. ***/

      nave ++;
      fprintf (stdout, "Add %s to summation array.\n", namimg);
      if (nbypix == 1)
         for (i=0; i<nelem; i++)
            *(imgadd + i) += (float) (*(imgbyt + i) & 0377);
      if (nbypix == 2)
         for (i=0; i<nelem; i++)
            *(imgadd + i) += (float) (*(imgint + i) & 0177777);
      if (nbypix == 4)
         for (i=0; i<nelem; i++)
            *(imgadd + i) += (float) *(imgflt + i);

      /*** Relinquish storage for input image array. ***/

      free (imgbuf);
      }

   /*** Compute image average. ***/

   invnum = 1.0 / (float) nave;
   imgbuf = malloc (nelem*nbypix);
   imgbyt = (char *)  imgbuf;
   imgint = (short *) imgbuf;
   imgflt = (float *) imgbuf;
   nbyimg = npix * npix * nbypix;
   fprintf (stderr, "Compute average.\n");
   if (nbypix == 1)
   for (i=0; i<nelem; i++)  *(imgbyt + i) = (char) (*(imgadd + i) * invnum);
   if (nbypix == 2)
   for (i=0; i<nelem; i++)  *(imgint + i) = (int) (*(imgadd + i) * invnum);
   if (nbypix == 4)
   for (i=0; i<nelem; i++)  *(imgflt + i) = (float) (*(imgadd + i) * invnum);

   /*** Copy averaged image to a disk file. ***/

   fprintf (stderr, "Write average image to disk file %s.\n", namout);
   fprintf (stderr, "nelem: %d  nbypix: %d\n", nelem, nbypix);
   avgfd = creat (namout, 0644);
   nclose = close (avgfd);
   avgfd = open (namout, 1);
/*   hkbuf [3] = hkbuf [3] | 004; */		/* Set resolution bit to low. */
   nwrite = write (avgfd, hkbuf, nbyhk);
   nwrite = write (avgfd, bitmap, nbymap);
   nwrite = write (avgfd, imgbuf, nelem*nbypix);
   nclose = close (avgfd);
   free (hkbuf);
   free (bitmap);
   free (imgbuf);

   exit (0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
outname (namvid)

char *namvid;

{
int lenvid;
int idot;
int i;
char newnam [80];

   lenvid = strlen (namvid);
   idot = -1;
   for (i=0; i<lenvid; i++) 
      {
      if (namvid [i] == '.') idot = i;
      newnam [i] = namvid [i];
      }
   if (idot == -1)
      {
      fprintf (stderr, "outname: invalid file name: %s\n", namvid);
      exit (-1);
      }
   newnam [idot+2] = 'a';
   newnam [idot+3] = 'v';
   newnam [lenvid] = '\0';

   return (newnam);
}
