/* xysplot.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 *   User specifies fxmin, fxmax, fymin, fymax.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   9 February 1987
 *   12 May 1989: remove cpgraf.h include file; make ig an argument of xyplot.
 *   25 March 1992: set format of numeric labels; remove ig argument.
 *                  Plot points only.  No curve plotted.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   fxmin, fxmax : min & max values for x-axis.
 *   fymin, fymax : min & max values for y-axis.
 *   nval : # values of x = # values of y.
 *   namemc : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   ig = 0 for non-graphics terminal, =1 for graphics terminal.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   ltype	x-axis	y-axis
 *   -----      ------  ------
 *	1	linear	linear
 *	2	linear	log
 *	3	log	linear
 *	4	log	log
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
extern int pcount;

xysplot (x, y, fxmin, fxmax, fymin, fymax, nval, namemc, title, labx, laby,
        ixscale, iyscale, ibkg)

float	x[], y[];
float	fxmin, fxmax;
float	fymin, fymax;	
int	nval;
char	namemc[];
char	title[], labx[], laby[];
int	ixscale, iyscale;
int	ibkg;

{
static char ip [4];
static char fmtx [8];
static char fmty [8];
int lfmtx = 0, lfmty = 0;
int ltype;
int i;
int ich = 42;
int il  = 0;
int is = 2;
int io = 0;
int ic = 0;
int ll = 1;
int lentit;
int lenlabx;
int lenlaby;
int lp;
static int igph = 5;
static int majrx = 10;
static int majry = 10;
static int ixlab = 1, iylab= 1;
int minrx, minry;
int numx = 0, numy = 0;
int iszx = 12, iszy = 12;
int ixdc = 20, iydc = 20;
int ixor = 0;

float dxx, dxn;
float dyy, dyn;
float fl = 0.0, fr = 1.0, fb = 0.0, ft = 1.0;
float ul = 0.0, ur = 1.0, ub = 0.0, ut = 1.0;
float fval;
float px, py;
float xmin, xmax, ymin, ymax;
float xxmin, xxmax, yymin, yymax;
float chsize, angd, cntr;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   pcount ++;
/*   if (pcount == 1)  gkschnam_ (namemc); */	/* Change metacode file name. */

   /*** Set scaling type. ***/

   if (ixscale == 0 && iyscale == 0)  ltype = 1;	/* x linear, y linear */
   if (ixscale == 0 && iyscale == 1)  ltype = 2;	/* x linear, y log    */
   if (ixscale == 1 && iyscale == 0)  ltype = 3;	/* x log,    y linear */
   if (ixscale == 1 && iyscale == 1)  ltype = 4;	/* x log,    y log    */
   if (ltype < 1 || ltype > 4)  ltype = 1;

   /*** Determine min & max limits for axes ranges. ***/

   xxmin =  1.0e+30;
   xxmax = -1.0e+30;
   yymin =  1.0e+30;
   yymax = -1.0e+30;

   for (i=0; i<nval; i++)
      {
      if (xxmin > x[i])  xxmin = x[i];
      if (xxmax < x[i])  xxmax = x[i];
      if (yymin > y[i])  yymin = y[i];
      if (yymax < y[i])  yymax = y[i];
      }

   /*** Set x/y min & max values. ***/

   if (fxmin == 0.0 && fxmax == 0.0)	/* Set min/max from array hi/lo. */
      {
      xmin = xxmin;
      xmax = xxmax;
      }
      else				/* Set min/max values from arg list. */
      {
      xmin = fxmin;
      xmax = fxmax;
      }
   if (fymin == 0.0 && fymax == 0.0)	/* Set min/max from array hi/lo. */
      {
      ymin = yymin;
      ymax = yymax;
      }
      else				/* Set min/max values from arg list. */
      {
      ymin = fymin;
      ymax = fymax;
      }

   /*** Write title & X/Y axis labels. ***/

   lentit = strlen (title);
   lenlabx = strlen (labx);
   lenlaby = strlen (laby);
   is = 2;     io = 0;     ic = 0;
   fl = 0.0;   fr = 1.0;   fb = 0.0;   ft = 1.0;
   ul = 0.0;   ur = 1.0;   ub = 0.0;   ut = 1.0;
   ll = 1;
   set_ (&fl, &fr, &fb, &ft, &ul, &ur, &ub, &ut, &ll);

   chsize = 14.0;   angd = 0.0;   cntr = 0.0;
   px = 0.565;   py = 0.985;
   strcpy (ip, "HW");
   fval = 3.0;
   pcsetr_ (ip, &fval, strlen (ip));
   plchmq_ (&px, &py, title, &chsize, &angd, &cntr, strlen (title));

   px = 0.6;   py = 0.02;   chsize = 16.0;
   fval = 2.0;
   pcsetr_ (ip, &fval, strlen (ip));
   plchmq_ (&px, &py, labx,  &chsize, &angd, &cntr, strlen (labx));

   px = 0.01;   py = 0.5;   angd = 90.0;
   plchmq_ (&px, &py, laby,  &chsize, &angd, &cntr, strlen (laby));

   /*** Determine minor & major scaling values for axes. ***/

   nicexr (&xmin, &xmax, &majrx, &minrx, &dxx, &dxn);

   nicexr (&ymin, &ymax, &majry, &minry, &dyy, &dyn);

   fl = 0.12;   fr = 0.98;   fb = 0.09;   ft = 0.95;
   set_ (&fl, &fr, &fb, &ft, &xmin, &xmax, &ymin, &ymax, &ltype);

   /*** Background : perimeter, grid, half axis. ***/

   switch (ibkg)
      {
      case 0: break;			/* No change. */
      default:
      case 1: igph = 5;  break;		/* perimeter with tick marks. */
      case 2: igph = 0;  break;		/* full grid. */
      case 3: igph = 10; break;		/* half axes. */
      }

/*
 *   periml_ (&majrx, &minrx, &majry, &minry);
 */
   
   /*** Set up numeric label formats. ***/

   strcpy (fmtx, "(I4)");
   strcpy (fmty, "(F6.1)");
   lfmtx = strlen (fmtx);
   lfmty = strlen (fmty);

   labmod_ (fmtx, fmty, &numx, &numy, &iszx, &iszy, &ixdc, &iydc, &ixor,
            lfmtx, lfmty);

   /*** Draw grid. ***/

   gridal_(&majrx, &minrx, &majry, &minry, &ixlab, &iylab, &igph, &xmin, &ymin);

   /*** Draw curve through data points. ***/

/*   curve_ (x, y, &nval); */

   points_ (x, y, &nval, &ich, &il);
      
/*   frame_ (); */

   return (0);
}
