/* dnadj89.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adjust DN (raw vidicon intensity) for predicted value at DOY 205.   *
 * The transfer curve for year 1989 is on DOY 205.                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	               *
 * High Altitude Observatory (HAO)			               *
 * National Center for Atmospheric Research (NCAR) 	               *
 * SGI 4D/25 G   UNIX IRIX 4.0.5   C programming language              *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Andrew L. Stanger				 	               *
 * 20 July 1995                                                        *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dnadj89 (fdoy, dn, dn205)

float fdoy ;			/* Image day-of-year (includes fraction). */
int dn ;			/* pixel DN intensity value. */
int *dn205 ;			/* pixel intensity at DOY 205. */

{
   static float dnoff [15]   = {   5.9,  22.4,  45.5,  68.4,  89.1,
				 106.6, 123.0, 149.6, 163.9, 174.8,
				 186.4, 202.0, 212.3, 222.4, 230.2} ;

   static float dnslope [15] = {  0.0068, -0.0160, -0.0503, -0.0855, -0.1130,
				 -0.1316, -0.1529, -0.1838, -0.1791, -0.1432,
				 -0.1363, -0.1487, -0.1399, -0.1073, -0.1088} ;

   float dnf, dno, dns ;

   int idn1, idn2 ;

   /*** Compute DOY fraction (fractional difference from lower index DOY). ***/

   ddoy = (fdoy - tcdoy [idoy1]) / (tcdoy [idoy2] - tcdoy [idoy1]) ;

   /*** Interpolate to derive DN offset and slope for a given image date. ***/

   for (i = 0; i < 15; i++)
      {
      if (dn < dnoff [i])  idn2 = i ;
      }

   /*** Interpolate to determine DN slope and intercept. ***/
   /*** Use input DN and compare with DN offset (intercept). ***/

   dnf = (dn -dnoff [idn1]) / (dnoff [idn2] - dnoff [idn1]) ;
   dno = dnoff [idn1] + dnf * (dnoff [idn2] = dnoff [idn1]) ;
   dns = dnslope [idn1] + dnf * (dnslope [idn2] = dnslope [idn1]) ;

/*
 *   dn1 = slope * t1 + off
 *   dn2 = slope * t2 + off
 *   ----------------------
 *   dn2 = dn1 + slope * (t2 - t1)
 */

   *dn205 = (int) (dn + dns * (205.0 - fdoy)) ;

   return (0) ;
}
