/* radj.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Actual Earth-sun distance due to Earth's elliptical orbit about the sun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 1 November 1996
 * 2 March 1999
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h> 

float
radj (iyear, idoy, ihour)

int iyear ;		/* Year. */
int idoy ;		/* Day of year. */
int ihour ;		/* Hour. */

{
   int iy ;

   double A = 1.0 ;		/* Semi-major axis of ellipse (AU units). */
   double ecc = 0.01675 ;	/* Eccentricity of Earth's orbit. */
   double period = 365.2596 ;	/* Earth's orbital period. */
   double PI = 3.14159265 ;	/* PI. */
   double radius ;		/* Orbital radius (earth to sun). */
   double th ;
   double theta ;		/* Angle of Earth-sun line w.r.t. perihelion. */
   double doyo ;		/* Day-of-year of observation. */
   double doyp ;		/* Perihelion day-of-year. */

   /* Perihelion DOY & hour (as a function of year: 80-89). */

   static float Pdoy  [10] = { 3,  2,  4,  2,  3,  3,  2,  4,  4,  1} ;
   static float Phour [10] = {15,  2, 11, 16, 22, 20,  5, 23,  0, 22} ;

   /*** Convert year to array index. ***/

   iy = iyear ;
   if (iyear > 1900)  iy -= 1900 ;
   if (iyear >= 80)   iy -=   80 ;
   if (iy < 0 || iy > 9)
      {
      fprintf (stderr, "Year not between 1980 & 1989.\n") ;
      exit (-1.0) ;
      }

   doyp = (double) (Pdoy [iy] + Phour [iy] / 24.0) ;	/* perihelion DOY. */

   doyo = (double) (idoy + ihour / 24.0) ;		/* image DOY. */

   theta = (2.0 * PI * (doyo - doyp)) / period ; /* Angle w.r.t. perihelion. */

   /*** Compute angle w.r.t. perihelion. ***/

   th = (2.0 * PI * (doyo - doyp) / period) +
	 2.0 * ecc * sin (2.0 * PI * (doyo - doyp) / period) ;

   radius = (A * (1.0 - ecc*ecc)) / (1.0 + ecc * cos (th)) ;

   fprintf (stderr,
      "radj> iy: %2d  doyo: %7.3f  theta: %7.3f  th: %7.3f  radius:%7.3f\n",
      iy, doyo, theta, th, radius) ;

   return (radius) ;
}
