/* boxcm11c.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extract values within 11 boxes from a series of C/P cal reticle image files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 15 March 1985
 * 26 March 1992: use NCAR system plot package instead of autograph.
 * 20 October 1992: remove fixed Y-axis scaling.
 * 29 October 1992: apply R**2 luminosity correction for Earth-Sun distance.
 *  3 November 1992: apply DOY slope correction for tube sensitivity loss.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 11
#define MAXBOX 64
#define MAXIMG 750
#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
#include "/usr/people/smm/lib/include/gl_color.h"
int DSP = 0;
int pcount = 0;

static float mean [NBOX];
static float pctdev [NBOX];
static float stddev [NBOX];
static float temp;
static float ymean [MAXIMG];
static float sigp [MAXIMG];
static float sigm [MAXIMG];
static float xline [MAXIMG];

main (argc, argv)

int argc;
char *argv [];
{
extern int cphskp ();		/* C/P Housekeeping function. */
extern int boxval ();		/* Function to extract box values from image. */
static char gksout [40];	/* NCARG_GKS_OUTPUT env. variable value. */
static char namcor [80];	/* Name of luminosity correction output file. */
static char namdat [80];	/* Name of data output file. */
static char namemc [80];	/* Name of metacode file. */
static char namimg [80];	/* Name of image file. */
static char namlst [80];	/* List of image file names. */
static char nampgm [80];	/* Program name. */
static char namstat [80];	/* Name of statistics file. */
static char atitle [60];	/* Adjusted plot title (Solar luminosity). */
static char ctitle [60];	/* Corrected plot title (luminosity + tube). */
static char ltitle [60];	/* Uncorrected plot title. */
static char labx [40] = {"D a y - o f - Y e a r   ( 19xx )"} ;
static char laby [40];

static int box [MAXBOX*MAXBOX];	/* Array containing box values. */
int nave = 16;			/* Size of one dimension of box. */
int i, j, k, l;			/* Loop indices. */
int ibkg = 0;
int imgnum;			/* Image index. */
int ixscale = 0;
int iyscale = 0;
int lsv;			/* Stray light/vignetting flag: =1 remove. */
int lxmin = 0;
int lxmax = 0;
int lymin = 0;
int lymax = 0;
int maxpix;			/* # values in box. */
int nimg;			/* # images. */
int ndoyyr;			/* # days in year. */
int p;				/* box index. */
int pmode = 0;			/* Print mode. */
int tcdoy = 205;		/* DOY for transfer curve images. */
int x,y;			/* Coordinates of center of box. */
/*
 * static int xtab [NBOX] = { 56,168,280,392, 56,168,280,392, 56,168,280,392};
 * static int ytab [NBOX] = {112,112,112,112,224,224,224,224,336,336,336,336};
 */
static int xtab [NBOX] = { 56,168,280,392, 56,168,280,392, 56,280,392};
static int ytab [NBOX] = {112,112,112,112,224,224,224,224,336,336,336};

float SMA = 1.0;		/* Semi-major axis of Earth-Sun orbit (1 AU). */
float DOYP = 3.0;		/* Day-of-year corresponding to perihelion. */
float R2;			/* Earth-Sun distance squared. */
float ecc  = 0.017;		/* Eccentricity of Earth's orbit about Sun. */
float esd;			/* Earth-Sun distance. */
float dncor = 0;		/* DN correction. */
float dnslope = 0.1;		/* DOY versus DN inverse slope. */
float imsum;			/* Sum of box averages. */
float imsumc;			/* Corrected sum of box averages. */
static float ave  [NBOX][MAXIMG];  /* Average of values within box. */
static float avec [NBOX][MAXIMG];  /* Corrected average of values within box. */
static float imgave  [MAXIMG];	/* Average of all boxes in image. */
static float imgavec [MAXIMG];	/* Corrected average of all boxes in image. */
static float imgndx [MAXIMG];	/* Index array for images. */
float pxmin = 0.0;
float pxmax = 0.0;
float pymin = 0.0;
float pymax = 0.0;
float sum;			/* Sum of values within box. */
float theta;			/* Angle w.r.t. Earth-Sun angle at perihelion.*/

FILE *LIST, *IMG, *corp, *datp, *outp ;


   strcpy (nampgm, argv [0]);

   if (argc == 2)
      strcpy (namlst, argv [1]);
   else
   if (argc > 2)
      {
      for (i = 1; i < (argc - 1); i++)
         {
/*         printf ("i: %d   argv: %s\n", i, argv [i]); */
         if (strcmp (argv [i], "-s") == 0)
            {
            i++;
            nave = atoi (argv [i]);
            }
         else
         if (strcmp (argv [i], "-g") == 0)
            {
            DSP = 1;
            igr = 1;
            }
         else
         if (strcmp (argv [i], "-x") == 0)
            {
            i++;
            pxmin = atof (argv [i]);
            i++;
            pxmax = atof (argv [i]);
            }
         else
         if (strcmp (argv [i], "-y") == 0)
            {
            i++;
            pymin = atof (argv [i]);
            i++;
            pymax = atof (argv [i]);
            }
         else
            {
            fprintf (stderr, "usage: %s [-g] [-s box_size] [-x xmin xmax] [-y ymin ymax] filelist_name\n",
                     argv[0]);
            exit (-1);
            }
         }
      strcpy (namlst, argv [argc-1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-g] [-s box_size] [-x xmin xmax] [-y ymin ymax] filelist_name\n",
               argv[0]);
      exit (-1);
      }

   /*** Open name-list file. ***/

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Cannot open %s", namlst);
      exit (-1);
      }

   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n",  nave,  MAXBOX);
      exit (-1);
      }

   /*** Read first image to get exposure duration. ***/

   fscanf (LIST, "%s", namimg);
   cphskp (namimg);
   rewind (LIST);

   sprintf (namcor,  "%s.%ds.cor",  nampgm, H.exp);
   sprintf (namdat,  "%s.%ds.dat",  nampgm, H.exp);
   sprintf (namemc,  "%s.%ds.cgm",  nampgm, H.exp);
   sprintf (namstat, "%s.%ds.stat", nampgm, H.exp);
   printf ("namdat: %s  namemc: %s  namstat: %s\n\n", namdat, namemc, namstat);
   sprintf (ltitle, "RAW DN: CM Exp: %ds %d-box average %dx%d",
            H.exp, NBOX, nave, nave);
   sprintf (atitle, "LUM fix: CM Exp: %ds %d-box average %dx%d",
            H.exp, NBOX, nave, nave);
   sprintf (ctitle, "LUM+DOY fix: CM Exp: %ds %d-box average %dx%d",
            H.exp, NBOX, nave, nave);

   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
   putenv (gksout);

   opngks_ ();				/* Open GKS. */

   /*** Open an output file for statistical information for each box. ***/

   outp = fopen (namstat, "w") ;
   if ( outp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namstat) ;
      exit (-1) ;
      }

   /***	Open an output file for the DOY and DN information ***/

   datp = fopen (namdat, "w") ;
   if ( datp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namdat) ;
      exit (-1) ;
      }
   fprintf (datp, "%s\n\n", ctitle);

   if (DSP)		/* Display option selected. */
      {
      /*** Initialize GL software. ***/

      foreground ();
      prefposition (760, 1271, 480, 991);   /* window in upper right corner.*/
      fminit ();			    /* Initialize SGI font manager. */

      winopen (nampgm);			/* Create GL window. */
      gl_wclear (BLACK);		/* Clear window to black. */
      }

   /***	Open an output file for the Solar Luminosity correction. ***/

   corp = fopen (namcor, "w") ;
   if ( corp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namcor) ;
      exit (-1) ;
      }
   fprintf (corp, "%s  Luminosity + DOY Correction\n\n", ctitle);
   fprintf (corp, "image name       imgndx     theta   Earth-Sun     R**2\n");

   /*** Image loop. ***/

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit (-6);
         }
      imgnum++;
      imsum  = 0.0;
      imsumc = 0.0;

      cphskp (namimg);
      if (H.year == 89)
         dncor = (float) ((H.doy - tcdoy) * dnslope);

      if (imgnum == 0)
         sprintf (labx, "D a y - o f - Y e a r   ( 19%2d )", H.year);

      if (DSP)
	{
	glcp_view (namimg, 1, pmode);
	}

      /*** Compute time index in units of DOY. ***/

      imgndx [imgnum] = H.doy + H.hour/24.0 + H.minute/1440.0;

      /*** Compute luminosity correction (R2 = Earth-Sun distance squared). ***/
      /*** orbital distance is computed in AU units. ***/

      if (H.year/4 * 4 == H.year)  ndoyyr = 366;  else ndoyyr = 365;
      theta = ((imgndx [imgnum] - DOYP) / (float) ndoyyr) * 360.0;
      esd = SMA * (1.0 - ecc * ecc) /
                  (1.0 + ecc * cos ((double) (raddeg * theta)));
      R2 = esd * esd;
      fprintf (corp, "%s  %7.2f  %7.2f  %9.4f  %9.4f\n",
               namimg, imgndx [imgnum], theta, esd, R2);

      /*** Box loop : ***/

      for ( i = 0; i < NBOX; i++ )
         {
	 x = xtab [i] * (float) nres;
	 y = ytab [i] * (float) nres;
	 boxval (namimg, x, y, nave, lsv, box, i, gl_red);

	 /*** Average value of box : ***/

	 sum = 0.0;
	 maxpix = nave * nave;
	 for ( p = 0; p < maxpix; p++ )
            {
            sum += (float) box[p];
            }
	 ave  [i][imgnum] = sum / (float) maxpix;
         avec [i][imgnum] = ave [i][imgnum] * R2 + dncor;
         imsum  += ave  [i][imgnum];
         imsumc += avec [i][imgnum];
         }

      if (DSP)  sleep (1);
      imgave  [imgnum] = imsum  / NBOX;
      imgavec [imgnum] = imsumc / NBOX;
      fprintf (datp, "%s   %d-box average: %7.2f\n",
               namimg, NBOX, imgave [imgnum]);
      fclose (IMG);
      }
/*
 *	mean and standard deviation calculation
 *	added by Michele D. De La Pena
 *	25 April 1985
 */
   nimg = imgnum + 1;
   fprintf (outp, "%s\n\n", ctitle);
   fprintf (outp, "NBOX   LOCATION    SIZE      MEAN     STD.DEV.   std.dev (%%)\n\n") ;
   fprintf (stderr, "NBOX   LOCATION    SIZE      MEAN     STD.DEV.   std.dev (%%)\n") ;
   temp = 0.0 ;

   for ( i = 0; i < NBOX; i++ )
      {
      for ( j = 0 ; j < nimg ; j++ )
         {
         temp += avec [i][j] ;
         }
      mean [i] = temp / ((double) nimg) ;
      temp = 0.0 ;

      for ( k = 0 ; k < nimg ; k++ )
         {
         temp += (avec [i][k] - mean [i]) * (avec [i][k] - mean [i]) ;
         }

      stddev [i] = sqrt (temp / ((double) (nimg-1))) ;
      pctdev [i] = (stddev [i] / mean [i]) * 100.0;
      temp = 0.0 ;

      fprintf (outp,   " %2d   (%3d,%3d)   %2dx%2d   %7.2f    %7.2f    %7.2f\n",
               i, xtab [i], ytab [i], nave, nave,
               mean [i], stddev [i], pctdev [i]);
      fprintf (stderr, " %2d   (%3d,%3d)   %2dx%2d   %7.2f    %7.2f    %7.2f\n",
               i, xtab [i], ytab [i], nave, nave,
               mean [i], stddev [i], pctdev [i]);

      for ( l = 0 ; l < (imgnum+1) ; l++ )
         {
         ymean [l] = mean [i] ;
         sigp [l] = mean [i] + stddev [i] ;
         sigm [l] = mean [i] - stddev [i] ;
         }

      sprintf (ltitle, "RAW DN: CM Exp: %ds %d-box average %dx%d",
               H.exp, NBOX, nave, nave);
      sprintf (atitle, "LUM fix: CM Exp: %ds %d-box average %dx%d",
               H.exp, NBOX, nave, nave);
      sprintf (ctitle, "LUM+DOY fix: CM Exp: %ds %d-box average %dx%d",
               H.exp, NBOX, nave, nave);
      sprintf (laby, "Vidicon Digital Number (DN)");

      }  /*  end i loop  */

   fclose (outp) ;
   fclose (datp) ;

   /*** Plot average DN level versus DOY. ***/

   lymin = lymax = 1;
/*
 *   pxmin =   0.0;
 *   pxmax = 366.0;
 *   pymin =  85.0;
 *   pymax = 130.0;
 */
   printf ("pxmin: %7.2f  pxmax: %7.2f  pymin: %7.2f  pymax:  %7.2f\n",
            pxmin, pxmax, pymin, pymax);


   xysplot (imgndx, imgave, pxmin, pxmax, pymin, pymax, nimg, namemc,
            ltitle, labx, laby, ixscale, iyscale, ibkg);


   xysplot (imgndx, imgavec, pxmin, pxmax, pymin, pymax, nimg, namemc,
            ctitle, labx, laby, ixscale, iyscale, ibkg);

   clsgks_ ();				/* Close GKS. */
   sprintf (gksout, "NCARG_GKS_OUTPUT=gks.cgm");
   putenv (gksout);
   if (DSP)
      gexit ();
}
