/* boxcm12.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extract values within 12 boxes from a series of C/P cal reticle image files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 15 March 1985
 * 26 March 1992: use NCAR system plot package instead of autograph.
 * 20 October 1992: remove fixed Y-axis scaling.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 12
#define MAXBOX 64
#define MAXIMG 750
#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
#include "/usr/people/smm/lib/include/gl_color.h"
int DSP = 0;
int pcount = 0;

static float mean [NBOX];
static float pctdev [NBOX];
static float stddev [NBOX];
static float temp;
static float ymean [MAXIMG];
static float sigp [MAXIMG];
static float sigm [MAXIMG];
static float xline [MAXIMG];

main (argc, argv)

int argc;
char *argv [];
{
extern int cphskp ();		/* C/P Housekeeping function. */
extern int boxval ();		/* Function to extract box values from image. */
static char gksout [40];	/* NCARG_GKS_OUTPUT env. variable value. */
static char namdat [80];		/* Name of data output file. */
static char namemc [80];		/* Name of metacode file. */
static char namimg [80];		/* Name of image file. */
static char namlst [80];		/* List of image file names. */
static char nampgm [80];		/* Program name. */
static char namstat [80];		/* Name of statistics file. */
static char ltitle [40];
static char labx [40] = {"D a y - o f - Y e a r   ( 1986 )"} ;
static char laby [40];
int ixscale = 0;
int iyscale = 0;
int ibkg = 0;
int x,y;			/* Coordinates of center of box. */
int nave = 16;			/* Size of one dimension of box. */
int lsv;			/* Stray light/vignetting flag: =1 remove. */
static int box [MAXBOX*MAXBOX];	/* Array containing box values. */
int i;				/* General purpose index. */
int imgnum;			/* Image index. */
int lxmin = 0;
int lxmax = 0;
int lymin = 0;
int lymax = 0;
int nimg;			/* # images. */
int p;				/* box index. */
int pmode = 0;			/* Print mode. */
int maxpix;			/* # values in box. */
float imsum;			/* Sum of box averages. */
float sum;			/* Sum of values within box. */
static float ave [NBOX][MAXIMG];	/* Average of values within box. */
static float imgave [MAXIMG];	/* Average of all boxes in image. */
static float imgndx [MAXIMG];	/* Index array for images. */
float pxmin = 0.0;
float pxmax = 0.0;
float pymin = 0.0;
float pymax = 0.0;
/*
 * static int xtab [NBOX] = {186} ;
 * static int ytab [NBOX] = {13} ;
 */
static int xtab [NBOX] = { 56,168,280,392, 56,168,280,392, 56,168,280,392};
static int ytab [NBOX] = {112,112,112,112,224,224,224,224,336,336,336,336};
static char answer [2];
static char ovrcolor [2];
FILE *LIST, *IMG, *outp, *datp ;

double sqrt (), fabs () ;
int j, k, l ;

   strcpy (nampgm, argv [0]);

   if (argc == 2)
      strcpy (namlst, argv [1]);
   else
   if (argc == 3 || argc == 4 || argc == 5)
      {
      for (i = 1; i < (argc - 1); i++)
         {
         printf ("i: %d   argv: %s\n", i, argv [i]);
         if (strcmp (argv [i], "-s") == 0)
            {
            i++;
            nave = atoi (argv [i]);
            }
         else
         if (strcmp (argv [i], "-g") == 0)
            {
            DSP = 1;
            igr = 1;
            }
         else
            {
            fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
                     argv[0]);
            exit (-1);
            }
         }
      strcpy (namlst, argv [argc-1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
               argv[0]);
      exit (-1);
      }


   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Cannot open %s", namlst);
      exit (-1);
      }

   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n",  nave,  MAXBOX);
      exit (-1);
      }

   /*** Read first image to get exposure duration. ***/

   fscanf (LIST, "%s", namimg);
   cphskp (namimg);
   rewind (LIST);

   sprintf (gksout, "NCARG_GKS_OUTPUT=%s.%ds.cgm", argv [0], H.exp);
   putenv (gksout);
   opngks_ ();				/* Open GKS. */

   sprintf (namstat, "%s.stat", nampgm);
   sprintf (namdat, "%s.dat", nampgm);
   sprintf (namemc, "%s.cgm", nampgm);
   printf ("namdat: %s  namemc: %s  namstat: %s\n\n", namdat, namemc, namstat);

   outp = fopen (namstat, "w") ;
   if ( outp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namstat) ;
      exit (-1) ;
      }

   /***	Open an output file for the DOY and DN information ***/

   datp = fopen (namdat, "w") ;
   if ( datp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namdat) ;
      exit (-1) ;
      }
   fscanf (LIST, "%s", namimg);
   cphskp (namimg);
   rewind (LIST);
   sprintf (ltitle, "CM Exp: %ds 12-box average %dx%d",
            H.exp, nave, nave);
   fprintf (datp, "%s\n\n", ltitle);

   if (DSP)
      {
      /*** Initialize GL software. ***/

      foreground ();
      prefposition (760, 1271, 480, 991);   /* window in upper right corner.*/
      fminit ();			    /* Initialize SGI font manager. */

      winopen (nampgm);			/* Create GL window. */
      gl_wclear (BLACK);		/* Clear window to black. */
      }

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit (-6);
         }
      imgnum++;
      imsum = 0.0;

      cphskp (namimg);
      if (imgnum == 0)
         sprintf (labx, "D a y - o f - Y e a r   ( 19%2d )", H.year);

      if (DSP)
	{
	glcp_view (namimg, 1, pmode);
	}

      imgndx [imgnum] = H.doy + H.hour/24. + H.minute/1440.;
/*      if ( H.year == 85 ) imgndx [imgnum] += 366.0; */

      /*** Box loop : ***/

      for (i=0; i < NBOX; i++)
         {
	 x = xtab [i] * (float) nres;
	 y = ytab [i] * (float) nres;
	 boxval (namimg, x, y, nave, lsv, box, i, gl_red);

	 /*** Average value of box : ***/

	 sum = 0.0;
	 maxpix = nave*nave;
	 for (p=0; p<maxpix; p++)
            {
            sum += (float) box[p];
            }
	 ave [i][imgnum] = sum / (float) maxpix;
         imsum += ave [i][imgnum];
         }

      if (DSP)  sleep (2);
      imgave [imgnum] = imsum / NBOX;
      fprintf (datp, "%s   12-box average: %7.2f\n", namimg, imgave [imgnum]);
      fclose (IMG);
      }
/*
 *	mean and standard deviation calculation
 *	added by Michele D. De La Pena
 *	25 April 1985
 */
   nimg = imgnum + 1;
   fprintf (outp, "%s\n\n", ltitle);
   fprintf (outp, "NBOX   LOCATION    SIZE      MEAN     STD.DEV.   std.dev (%%)\n\n") ;
   fprintf (stderr, "NBOX   LOCATION    SIZE      MEAN     STD.DEV.   std.dev (%%)\n") ;
   temp = 0.0 ;

   for (i=0; i < NBOX; i++)
      {
      for ( j=0 ; j<nimg ; j++ )
         {
         temp += ave [i][j] ;
         }
      mean [i] = temp / ((double) nimg) ;
      temp = 0.0 ;

      for ( k=0 ; k<nimg ; k++ )
         {
         temp += (ave [i][k] - mean [i]) * (ave [i][k] - mean [i]) ;
         }

      stddev [i] = sqrt (temp / ((double) (nimg-1))) ;
      pctdev [i] = (stddev [i] / mean [i]) * 100.0;
      temp = 0.0 ;

      fprintf (outp,   " %2d   (%3d,%3d)   %2dx%2d   %7.2f    %7.2f    %7.2f\n",
               i, xtab [i], ytab [i], nave, nave,
               mean [i], stddev [i], pctdev [i]);
      fprintf (stderr, " %2d   (%3d,%3d)   %2dx%2d   %7.2f    %7.2f    %7.2f\n",
               i, xtab [i], ytab [i], nave, nave,
               mean [i], stddev [i], pctdev [i]);

      for ( l = 0 ; l < (imgnum+1) ; l++ )
         {
         ymean [l] = mean [i] ;
         sigp [l] = mean [i] + stddev [i] ;
         sigm [l] = mean [i] - stddev [i] ;
         }

      sprintf (ltitle, "CM Exp: %ds 12-box average %dx%d",
               H.exp, nave, nave);
      sprintf (laby, "Vidicon Digital Number (DN)");

      }  /*  end i loop  */

   fclose (outp) ;
   fclose (datp) ;

   /*** Plot average DN level versus DOY. ***/

   lymin = lymax = 1;
   pxmin =   0.0;
   pxmax = 366.0;
   pymin =  85.0;
   pymax = 130.0;

/*
 *   ncarplot (imgndx, imgave, nimg, namemc, ltitle, labx, laby,
 *             ixscale, iyscale, ibkg, ymean, sigp, sigm,
 *             lxmin, lxmax, lymin, lymax, pxmin, pxmax, pymin, pymax);
 */
   xysplot (imgndx, imgave, pxmin, pxmax, pymin, pymax, nimg, namemc,
            ltitle, labx, laby,
            ixscale, iyscale, ibkg);

   clsgks_ ();				/* Close GKS. */
   sprintf (gksout, "NCARG_GKS_OUTPUT=gks.cgm");
   putenv (gksout);

   if (DSP)
      gexit ();
}
