/* boxcm3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extract values within a box from a series of C/P cal reticle image files.
 * 3 boxes: upper right (#4), near center (#7), lower left (#9).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 15 March 1985
 * 26 September 1988
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 3
#define MAXBOX 64
#define MAXIMG 250
#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
#include "/usr/people/smm/lib/include/gl_color.h"
int DSP = 0;
int pcount = 0;

static float mean[NBOX], stdev[NBOX], temp ;
static float ymean[MAXIMG], sigp[MAXIMG], sigm[MAXIMG], xline[MAXIMG] ;

main (argc,  argv)

int argc;
char *argv [];
{
extern int cphskp ();		/* C/P Housekeeping function. */
extern int boxval ();		/* Function to extract box values from image. */
static char gksout [40];	/* NCARG_GKS_OUTPUT env. variable value. */
static char namdat [80];	/* Name of data output file. */
static char namlst [80];	/* List of image file names. */
static char namimg [80];	/* Name of image file. */
static char namemc [80];
static char nampgm [80];	/* Name of image file. */
static char namstat [80];	/* Name of statistics file. */
static char ltitle [40];
static char labx [40];
static char laby [40];
int ixscale = 0;
int iyscale = 0;
int ibkg = 0;
int x, y;			/* Coordinates of center of box. */
int nave = 16;			/* Size of one dimension of box. */
int lsv;			/* Stray light/vignetting flag: =1 remove. */
static int box [MAXBOX*MAXBOX];	/* Array containing box values. */
int i;				/* General purpose index. */
int imgnum;			/* Image index. */
int maxpix;			/* # values in box. */
int nimg;			/* # images. */
int p;				/* box index. */
int pmode = 0;			/* print mode. */
float imsum;			/* Sum of box averages. */
float sum;			/* Sum of values within box. */
static float ave [NBOX][MAXIMG];/* Average of values within box. */
static float imgave [MAXIMG];	/* Average of all boxes in image. */
static float imgndx [MAXIMG];	/* Index array for images. */
static int xtab [NBOX] = {392,  280,   56};
static int ytab [NBOX] = {112,  224,  336};
/*
 * static int xtab [NBOX] = { 56, 168, 280, 392,  56, 168, 280, 392,  56, 168, 280, 392};
 * static int ytab [NBOX] = {112, 112, 112, 112, 224, 224, 224, 224, 336, 336, 336, 336};
 */
static char answer [2];
static char ovrcolor [2];
FILE *LIST,  *IMG,  *outp,  *datp ;

   double sqrt (), fabs () ;
   int j, k, l ;

   /*** Process command line arguments. ***/

   strcpy (nampgm, argv [0]);

   if (argc == 2)
      strcpy (namlst, argv [1]);
   else
   if (argc == 3 || argc == 4 || argc == 5)
      {
      for (i = 1; i < (argc - 1); i++)
         {
         printf ("i: %d   argv: %s\n", i, argv [i]);
         if (strcmp (argv [i], "-s") == 0)
            {
            i++;
            nave = atoi (argv [i]);
            }
         else
         if (strcmp (argv [i], "-g") == 0)
            {
            DSP = 1;
            igr = 1;
            }
         else
            {
            fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
                     argv[0]);
            exit (-1);
            }
         }
      strcpy (namlst, argv [argc-1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
               argv[0]);
      exit (-1);
      }

   /*** Read list of images; process one image at a time. ***/

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Cannot open %s", namlst);
      exit (-1);
      }

   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n",  nave,  MAXBOX);
      exit (-1);
      }

   /*** Read first image to get exposure duration. ***/

   fscanf (LIST, "%s", namimg);
   cphskp (namimg);
   rewind (LIST);

   sprintf (gksout, "NCARG_GKS_OUTPUT=%s.%ds.cgm", argv [0], H.exp);
   putenv (gksout);

   opngks_ ();
   sprintf (namstat, "%s.stat", argv [0]);
   sprintf (namdat, "%s.dat", argv [0]);
   sprintf (namemc, "%s.cgm", argv [0]);

   outp = fopen (namstat, "w") ;
   if (outp == NULL)
      {
      fprintf (stderr,  "Error opening file.\n\n") ;
      exit (-1) ;
      }

   /*** Open an output file for the DOY and DN information. ***/

   datp = fopen (namdat, "w") ;
   if ( datp == NULL)
      {
      fprintf (stderr, "Error opening file.\n\n") ;
      exit (-1) ;
      }

   fprintf (datp,  "INDEX    DOY    DN\n\n") ;

   /*** Initialize GL software. ***/

   if (DSP)
      {
      foreground ();
      prefposition (760, 1271, 480, 991);   /* window in upper right corner.*/
      fminit ();			    /* Initialize SGI font manager. */

      winopen (nampgm);			/* Create GL window. */
      gl_wclear (BLACK);		/* Clear window to black. */
      }

/*
   printf ("Remove stray light & vignetting (y/n) ? ");
   scanf ("%s",  answer);
   lvs = 0;
   if (strncmp ("y",  answer,  1) == 0)  lvs = 1;
*/

   imgnum = -1;
   while (fscanf (LIST,  "%s",  namimg) != EOF)
      {
      if ((IMG = fopen (namimg, "r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s", namimg);
         exit (-6);
         }
      imgnum++;
      imsum = 0.0;

      cphskp (namimg);
      sprintf (labx, "D a y   O f   Y e a r  ( 1 9 8 %1d)", H.year - 80);

      if (DSP)
	{
	glcp_view (namimg, 1, pmode);
	}

      imgndx [imgnum] = H.doy + H.hour/24. + H.minute/1440.;
      if ( H.year == 85 ) imgndx [imgnum] += 366.0 ;

      /*** Box loop : ***/

      for (i=0; i < NBOX; i++)
         {
         x = xtab [i] * (float) nres;
         y = ytab [i] * (float) nres;
         boxval (namimg, x, y, nave, lsv, box, i, gl_red);

	 /*** Average value of box : ***/

         sum = 0.0;
         maxpix = nave*nave;
         for (p=0; p<maxpix; p++)
            {
            sum += (float) box[p];
            }
         ave [i][imgnum] = sum / (float) maxpix;
         imsum += ave [i][imgnum];
/*
 *	printf ("box: %d  x: %d  y: %d  size: %d  ave: %f\n", 
 *		i,  x,  y,  nave,  ave[i][imgnum]);
 */
         }
      if (DSP)  sleep (2);
      imgave [imgnum] = imsum / NBOX;
      fclose (IMG);
      }
/*
 *	mean and standard deviation calculation
 *	added by Michele D. De La Pena
 *	25 April 1985
 */
      nimg = imgnum + 1;
      fprintf (outp, "NBOX   (x, y)   MEAN      STANDARD DEVIATION\n\n") ;
      temp = 0.0 ;
      for (i=0; i < NBOX; i++)
         {
	 for ( j=0 ; j<nimg ; j++ )
	    {
	    temp += ave[i][j] ;
	    }
	 mean [i] = temp / ((double) nimg) ;
	 temp = 0.0 ;

	 for ( k=0 ; k<nimg ; k++ )
	    {
	    temp += (ave [i][k] - mean [i]) * (ave [i][k] - mean [i]) ;
	    }
	 stdev [i] = sqrt (temp / ((double) (nimg-1))) ;
	 temp = 0.0 ;

	 fprintf (outp, " %2d   (%03d, %03d)    %7.3f       %7.3f\n", 
                  i,  xtab [i],  ytab [i],  mean [i],  stdev [i]) ;
	 fprintf (stderr, " %2d   (%03d, %03d)       %7.3f       %7.3f\n", 
                  i,  xtab[i],  ytab[i],  mean[i],  stdev[i]) ;

	 for ( l=0 ; l<(imgnum+1) ; l++ )
	    {
	    ymean [l] = mean [i] ;
	    sigp [l] = mean [i] + stdev [i] ;
	    sigm [l] = mean [i] - stdev [i] ;
	    }

/*
 *	 sprintf (ltitle,  "CM exp:%d Box:(%03d, %03d) %dx%d$", 
 *		  H.exp,  xtab[i],  ytab[i],  nave,  nave);
 */

 	 sprintf (ltitle, "CM Exp: %ds Box:(%d, %d) %dx%d", 
                  H.exp,  xtab [i],  ytab [i],  nave,  nave) ; 
/*
 *         sprintf (ltitle,  "Green Images - Exp: %ds 12-box average",  H.exp);
 */
         sprintf (laby,  "Vidicon Digital Number (DN)$");
/*
 *         sprintf (laby,  "B/Bsun (x10E-11)$") ;
 */
	 fprintf(stderr, "To the plotting routine.\n") ;

	 for ( j=0 ; j<(imgnum+1) ; j++)
	     fprintf (datp, "%d    %6.2f     %6.2f\n", j, imgndx[j], ave[i][j]) ;
	 fprintf (datp, "\n") ;

      ncarplot (imgndx,  &ave [i][0],  nimg,  namemc,  ltitle,  labx,  laby, 
                ixscale,  iyscale,  ibkg,  ymean,  sigp,  sigm);
      }  /*  end i loop  */
   fclose (outp) ;
   fclose (datp) ;

   clsgks_ ();
   sprintf (gksout, "NCARG_GKS_OUTPUT=gks.cgm");
   putenv (gksout);

   if (DSP)
      gexit ();
}
