/* boxcmy.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extract values within 12 boxes from a series of C/P cal reticle image files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 15 March 1985
 * 26 March 1992: use NCAR system plot package instead of autograph.
 * 20 October 1992: remove fixed Y-axis scaling.
 * 29 October 1992: apply R**2 luminosity correction for Earth-Sun distance.
 *                  imgndx has year units.
 * 4 October 1994: use tables for perihelion.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 12
#define MAXBOX 16
#define MAXIMG 4000
#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
#include "/usr/people/smm/lib/include/gl_color.h"
int DSP = 0;
int pcount = 0;

static float mean [NBOX];
static float pctdev [NBOX];
static float stddev [NBOX];
static float temp;
static float ymean [MAXIMG];
static float sigp [MAXIMG];
static float sigm [MAXIMG];
static float xline [MAXIMG];

main (argc, argv)

int argc;
char *argv [];
{
   extern int cphskp ();	/* C/P Housekeeping function. */
   extern int boxval ();	/* Function to extract box values from image. */
   static char namdat [80];	/* Name of data output file. */
   static char namemc [80];	/* Name of metacode file. */
   static char namimg [80];	/* Name of image file. */
   static char namlst [80];	/* List of image file names. */
   static char nampgm [80];	/* Program name. */
   static char namstat [80];	/* Name of statistics file. */
   static char ctitle [40];	/* Corrected plot title. */
   static char ltitle [40];	/* Uncorrected plot title. */
   static char labx [40] = {"D a y - o f - Y e a r   ( 19xx )"} ;
   static char laby [40];

   static int box [MAXBOX*MAXBOX];	/* Array containing box values. */
   int nave = 16;			/* Size of one dimension of box. */
   int i, j, k, l;			/* Loop indices. */
   int ibkg = 0;
   int imgnum;				/* Image index. */
   int iy ;
   int ixscale = 0;
   int iyscale = 0;
   int lsv;			/* Stray light/vignetting flag: =1 remove. */
   int lxmin = 0;
   int lxmax = 0;
   int lymin = 0;
   int lymax = 0;
   int maxpix;				/* # values in box. */
   int nimg;				/* # images. */
   int ndoyyr;				/* # days in year. */
   int p;				/* box index. */
   int pmode = 0;			/* Print mode. */
   int x,y;				/* Coordinates of center of box. */
   static    int xtab [NBOX] = { 56,168,280,392, 56,168,280,392, 56,168,280,392};
   static    int ytab [NBOX] = {112,112,112,112,224,224,224,224,336,336,336,336};

   static float ave  [NBOX][MAXIMG];	/* Average of values within box. */
   static float avec [NBOX][MAXIMG];	/* Corrected ave of values within box.*/
   static float imgave  [MAXIMG];	/* Average of all boxes in image. */
   static float imgavec [MAXIMG];	/* Corrected ave of boxes in image. */
   static float imgndx [MAXIMG];	/* Index array for images. */

   /* Perihelion DOY & hour. */

   /* year:                    80  81  82  83  84  85  86  87  88  89 */
   static double Pdoy  [10] = { 3,  2,  4,  2,  3,  3,  2,  4,  4,  1} ;
   static double Phour [10] = {15,  2, 11, 16, 22, 20,  5, 23,  0, 22} ;
   double PI = 3.14159265 ;     /* PI. */

   float SMA = 1.0;		/* Semi-major axis of Earth-Sun orbit (1 AU). */
   float DOYP = 3.0;		/* Day-of-year corresponding to perihelion. */
   float R2;			/* Earth-Sun distance squared. */
   float doyndx;		/* DOY index. */
   float doyp ;			/* Day-Of-Year corresponding to perihelion. */
   float doyo ;			/* Day-Of-Year corresponding to observation. */
   float ecc  = 0.017;		/* Eccentricity of Earth's orbit about Sun. */
   float esd;			/* Earth-Sun distance. */
   float imsum;			/* Sum of box averages. */
   float imsumc;		/* Corrected sum of box averages. */
   float pxmin = 0.0;
   float pxmax = 0.0;
   float pymin = 0.0;
   float pymax = 0.0;
   float sum;			/* Sum of values within box. */
   float theta;			/* Angle w.r.t. Earth-Sun angle at perihelion.*/

   FILE *LIST, *IMG, *outp, *datp ;


   strcpy (nampgm, argv [0]);

   if (argc == 2)
      strcpy (namlst, argv [1]);
   else
   if (argc == 3 || argc == 4 || argc == 5)
      {
      for (i = 1; i < (argc - 1); i++)
         {
         printf ("i: %d   argv: %s\n", i, argv [i]);
         if (strcmp (argv [i], "-s") == 0)
            {
            i++;
            nave = atoi (argv [i]);
            }
         else
         if (strcmp (argv [i], "-g") == 0)
            {
            DSP = 1;
            igr = 1;
            }
         else
            {
            fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
                     argv[0]);
            exit (-1);
            }
         }
      strcpy (namlst, argv [argc-1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
               argv[0]);
      exit (-1);
      }


   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Cannot open %s", namlst);
      exit (-1);
      }

   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n",  nave,  MAXBOX);
      exit (-1);
      }

   opngks_ ();				/* Open GKS. */

   sprintf (namstat, "%s.stat", nampgm);
   sprintf (namdat, "%s.dat", nampgm);
   sprintf (namemc, "%s.cgm", nampgm);
   printf ("namdat: %s  namemc: %s  namstat: %s\n\n", namdat, namemc, namstat);

   outp = fopen (namstat, "w") ;
   if ( outp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namstat) ;
      exit (-1) ;
      }

   /***	Open an output file for the DOY and DN information ***/

   datp = fopen (namdat, "w") ;
   if ( datp == NULL)
      {
      fprintf (stderr, "Error opening file: %s\n\n", namdat) ;
      exit (-1) ;
      }
   fscanf (LIST, "%s", namimg);
   cphskp (namimg);
   rewind (LIST);
   sprintf (ltitle, "CM Exp: %ds 12-box average %dx%d",
            H.exp, nave, nave);
   sprintf (ctitle, "ADJ CM Exp: %ds 12-box average %dx%d",
            H.exp, nave, nave);
   fprintf (datp, "%s\n\n", ctitle);

   if (DSP)
      {
      /*** Initialize GL software. ***/

      foreground ();
      prefposition (760, 1271, 480, 991);   /* window in upper right corner.*/
      fminit ();			    /* Initialize SGI font manager. */

      winopen (nampgm);			/* Create GL window. */
      gl_wclear (BLACK);		/* Clear window to black. */
      }

   printf ("image name       imgndx     theta   Earth-Sun\n");

   /*** Image loop. ***/

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit (-6);
         }
      imgnum++;
      imsum  = 0.0;
      imsumc = 0.0;

      cphskp (namimg);
      if (imgnum == 0)
         strcpy (labx, "Y e a r");

      if (DSP)
	{
	glcp_view (namimg, 1, pmode);
	}

      /*** Compute time index in units of year. ***/

      if (H.year/4 * 4 == H.year)  ndoyyr = 366;  else ndoyyr = 365;
      doyndx = H.doy + H.hour/24.0 + H.minute/1440.0;
      imgndx [imgnum] = H.year + doyndx / (float) ndoyyr;

      iy = H.year ;
      if (iy > 1900)  iy -= 1900 ;
      if (iy >= 80)   iy -=   80 ;
      if (iy < 0 || iy > 9)
         {
         fprintf (stderr, "valid only for years 1980-1989.\n") ;
         exit (-1) ;
         }

      degrad = 180.0 / PI ;

      /*
      fprintf (stdout, "iyear:%2d  idoy:%3d  ihour:%2d  iy:%2d  Pdoy:%6.2f
      Phour:%6.2f\n",
      iyear, idoy, ihour, iy, Pdoy [iy], Phour [iy]) ;
      */

      doyp = (double) (Pdoy [iy] + Phour [iy] / 24.0) ;

      doyo = (double) (H.doy + (double) H.hour / 24.0) ;


      /*** Compute luminosity correction (R2 = Earth-Sun distance squared). ***/
      /*** orbital distance is computed in AU units. ***/

      theta = ((doyo - doyp) / (float) ndoyyr) * 360.0;
      esd = SMA * (1.0 - ecc * ecc) /
                  (1.0 + ecc * cos ((double) (raddeg * theta)));
      R2 = esd * esd;
      printf ("%s  %7.2f  %7.2f  %9.4f\n", namimg, imgndx [imgnum], theta, esd);

      /*** Box loop : ***/

      for ( i = 0; i < NBOX; i++ )
         {
	 x = xtab [i] * (float) nres;
	 y = ytab [i] * (float) nres;
	 boxval (namimg, x, y, nave, lsv, box, i, gl_red);

	 /*** Average value of box : ***/

	 sum = 0.0;
	 maxpix = nave * nave;
	 for ( p = 0; p < maxpix; p++ )
            {
            sum += (float) box[p];
            }
	 ave  [i][imgnum] = sum / (float) maxpix;
         avec [i][imgnum] = ave [i][imgnum] * R2;
         imsum  += ave  [i][imgnum];
         imsumc += avec [i][imgnum];
         }

      if (DSP)  sleep (1);
      imgave  [imgnum] = imsum  / NBOX;
      imgavec [imgnum] = imsumc / NBOX;
      fprintf (datp, "%s   12-box average: %7.2f\n", namimg, imgave [imgnum]);
      fclose (IMG);
      }
/*
 *	mean and standard deviation calculation
 *	added by Michele D. De La Pena
 *	25 April 1985
 */
   nimg = imgnum + 1;
   fprintf (outp, "%s\n\n", ctitle);
   fprintf (outp, "NBOX   LOCATION    SIZE      MEAN     STD.DEV.   std.dev (%%)\n\n") ;
   fprintf (stderr, "NBOX   LOCATION    SIZE      MEAN     STD.DEV.   std.dev (%%)\n") ;
   temp = 0.0 ;

   for ( i = 0; i < NBOX; i++ )
      {
      for ( j = 0 ; j < nimg ; j++ )
         {
         temp += avec [i][j] ;
         }
      mean [i] = temp / ((double) nimg) ;
      temp = 0.0 ;

      for ( k = 0 ; k < nimg ; k++ )
         {
         temp += (avec [i][k] - mean [i]) * (avec [i][k] - mean [i]) ;
         }

      stddev [i] = sqrt (temp / ((double) (nimg-1))) ;
      pctdev [i] = (stddev [i] / mean [i]) * 100.0;
      temp = 0.0 ;

      fprintf (outp,   " %2d   (%3d,%3d)   %2dx%2d   %7.2f    %7.2f    %7.2f\n",
               i, xtab [i], ytab [i], nave, nave,
               mean [i], stddev [i], pctdev [i]);
      fprintf (stderr, " %2d   (%3d,%3d)   %2dx%2d   %7.2f    %7.2f    %7.2f\n",
               i, xtab [i], ytab [i], nave, nave,
               mean [i], stddev [i], pctdev [i]);

      for ( l = 0 ; l < (imgnum+1) ; l++ )
         {
         ymean [l] = mean [i] ;
         sigp [l] = mean [i] + stddev [i] ;
         sigm [l] = mean [i] - stddev [i] ;
         }

      sprintf (ltitle, "CM Exp: %ds 12-box average %dx%d",
               H.exp, nave, nave);
      sprintf (ctitle, "ADJ CM Exp: %ds 12-box average %dx%d",
               H.exp, nave, nave);
      sprintf (laby, "Vidicon Digital Number (DN)");

      }  /*  end i loop  */

   fclose (outp) ;
   fclose (datp) ;

   /*** Plot average DN level versus DOY. ***/

   lymin = lymax = 1;
   pxmin =  84.0;
   pxmax =  90.0;
   pymin =  85.0;
   pymax = 130.0;

   xysplot (imgndx, imgave, pxmin, pxmax, pymin, pymax, nimg, namemc,
            ltitle, labx, laby,
            ixscale, iyscale, ibkg);

   xysplot (imgndx, imgavec, pxmin, pxmax, pymin, pymax, nimg, namemc,
            ctitle, labx, laby,
            ixscale, iyscale, ibkg);

   clsgks_ ();				/* Close GKS. */
   if (DSP)
      gexit ();
}
