/* boxval1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extract values within a box from a C/P image file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Note: 'cphskp ()' should be called prior to boxval.
 *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 14 December 1984
 * 13 February 1987: change coord to rcoord.
 * 18 March 1987: label box with index number.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include "/usr/people/smm/lib/include/cplib.ext"

boxval (namfil, x, y, nbox, lsv, box, ibox, mapcol)

char namfil [];		/* Name of image file. */
int x, y;		/* Coordinates of center of box. */
int nbox;		/* Size of box in one dimension. */
int lsv;		/* Stray light/vignetting flag; =1 remove. */
int box [];		/* Array containing box values. */
int ibox;		/* Box number. */
int mapcol;		/* Color LUT color index. */

{
   extern int rcoord ();/* Polar/cartesian coordinate transformation function.*/
   extern float stray ();	/* Stray light function. */
   extern float vignet ();	/* Vignetting function. */

   static char nambox [6];
   int imgfd;
   int ip, ir;
   int ip1, ip2, ir1, ir2;
   int nhalf;
   int ib, il;
   int ierr;
   int ips, irs;
   float radius, theta;
   static short line [896];
   char *linbyt;

   linbyt  = (char *) line;

   /* Define box surrounding specified point (x,y) of size nbox*nbox. */

   nhalf = nbox / 2;
   ip1 = x - nhalf;
   ip2 = ip1 + nbox - 1;
   ir1 = y - nhalf;
   ir2 = ir1 + nbox - 1;

   sprintf (nambox, "%d", ibox);
   if (strlen (nambox) == 1) ips = x/nres + 62 - 3;
   else  ips = x/nres + 62 - 7;
   if (ips < 63)  ips = 63;
   irs = 511 - y/nres + nhalf + 1;
   if (irs > 500)  irs = 511 - y/nres - nhalf - 19;
/*
   printf ("ip1: %d  ir1: %d    ip2: %d  ir2: %d\n", ip1, ir1, ip2, ir2);
   printf ("npix: %d  nbypix: %d  nbox: %d  nres: %d\n",
	    npix, nbypix, nbox, nres);
 */

   /*** Draw box on Grinnell in overlay channel 'ovrch'. ***/

   if (igr == 1)
      {

      /*** Connect 4 box vertices on Grinnell. ***/

      gl_wvec (ip1/nres+62, 511-ir1/nres, ip2/nres+62, 511-ir1/nres, mapcol);
      gl_wvec (ip2/nres+62, 511-ir1/nres, ip2/nres+62, 511-ir2/nres, mapcol);
      gl_wvec (ip2/nres+62, 511-ir2/nres, ip1/nres+62, 511-ir2/nres, mapcol);
      gl_wvec (ip1/nres+62, 511-ir2/nres, ip1/nres+62, 511-ir1/nres, mapcol);
      gl_wstr (nambox, ips, irs, 1, mapcol);
      }

   /* Open image file. */

   imgfd = open (namfil, 0);
   ib = -1;

   /* Record loop : */

   for (ir = ir1; ir <= ir2; ir++)
      {
      lseek (imgfd, (ir + 2) * npix * nbypix, 0);
      read (imgfd, line, npix * nbypix);
      il = ip1 - 1;

      /* Extract image data from box area. */

      for (ip=ip1; ip<=ip2; ip++)
         {
         ib++;
         il++;
         if (nbypix == 1)  box [ib] = (int) (*(linbyt + il) & 0377);
         else  box[ib] = (int) line[il];
/*   printf ("box [%d] = line [%d][%d] = %d\n", ib, ir, ip, line [il]); */

         /* Remove stray light & vignetting (if flag set). */

         if (lsv == 1)
            {
            rcoord (&radius, &theta, &ip, &ir, -1, scroll);
            box [ib] = (int) (box [ib] - stray (radius)) * vignet (radius);
            }
         }
      }

   close (imgfd);
   return (0);
}
