/* ncarplot (x, y, nval, namfil, title, labx, laby, ixscale, iyscale,
 *          ibkg, ym, sp, sm)
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   15 March 1985
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namfil : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
extern int pcount;

ncarplot (x, y, nval, namfil, title, labx, laby, ixscale, iyscale, ibkg,
	  ym, sp, sm,
          lxmin, lxmax, lymin, lymax, pxmin, pxmax, pymin, pymax)

float	x [], y [];
int	nval;
char	namfil [];
char	title [40], labx [40], laby [40];
int	ixscale, iyscale;
int	ibkg;
float	ym [], sp [], sm [] ;
int	lxmin, lxmax;
int	lymin, lymax;
float	pxmin, pxmax;
float	pymin, pymax;

{
   static char cxmin [12] = "X/MINIMUM.";
   static char cxmax [12] = "X/MAXIMUM.";
   static char cymin [12] = "Y/MINIMUM.";
   static char cymax [12] = "Y/MAXIMUM.";
   static int ichar = '*' ;
   static int iset  = 0;
   static int ldash = 0;
   static int ndash = 0;
   int lcxmin = 0;
   int lcxmax = 0;
   int lcymin = 0;
   int lcymax = 0;
   int ll = 1;
   int llabx = 0;
   int llaby = 0;
   int ltit = 0;

   float xmm [2], ymm [2] ;
   float fzero = 0.0;

   float fl = 0.0, fr = 1.0, fb = 0.0, ft = 1.0;
   float ul = 0.0, ur = 1.0, ub = 0.0, ut = 1.0;

   xmm [0] = 110.0 ;
   xmm [1] = 117.0 ;
   ymm [0] = 100.0 ;
   ymm [1] = 150.0 ;

   pcount ++;
/*   if (pcount == 1)  q8qini (namfil, 0); */
/*
 *	anotat (labx, laby, 1, -1, 0, 0.0);
 *	displa (2, 0, 1);
 *
 *	ezxy (xmm, ymm, 2, title);
 *	curve(x,y,nval) ;
 */
   fl = 0.0;   fr = 1.0;   fb = 0.0;   ft = 1.0;
   ul = 0.0;   ur = 1.0;   ub = 0.0;   ut = 1.0;
   ll = 1;
   set_ (&fl, &fr, &fb, &ft, &ul, &ur, &ub, &ut, &ll);

   llabx = strlen (labx);
   llaby = strlen (laby);
   anotat_ (labx, laby, &ibkg, &iset, &ndash, &ldash, &llabx, &llaby);

   if (lxmin)
      {
      lcxmin = strlen (cxmin);
      agsetf_ (cxmin, &pxmin, &lcxmin);
      }
   if (lxmax)
      {
      lcxmax = strlen (cxmax);
      agsetf_ (cxmax, &pxmax, &lcxmax);
      }
   if (lymin)
      {
      lcymin = strlen (cymin);
      agsetf_ (cymin, &pymin, &lcymin);
      }
   if (lymax)
      {
      lcymax = strlen (cymax);
      agsetf_ (cymax, &pymax, &lcymax);
      }

   printf ("lxmin: %d   lxmax: %d   lymin: %d   lymax: %d\n",
            lxmin, lxmax, lymin, lymax);
   printf ("pxmin: %7.2f   pxmax: %7.2f   pymin: %7.2f   pymax: %7.2f\n",
            pxmin, pxmax, pymin, pymax);
   printf ("lcxmin: %d   lcxmax: %d   lcymin: %d   lcymax: %d\n",
            lcxmin, lcxmax, lcymin, lcymax);

   displa_ (&fzero, &fzero, &fzero);
   ltit = strlen (title);
   ezxy_ (x, y, &nval, title, &ltit);
/*
 *   points (x, y, nval, ichar, 1) ;
 *   curve (x, ym, nval) ;
 *   curve (x, sp, nval) ;
 *   curve (x, sm, nval) ;
 *   frame () ;
 */

   return (0);
}
