      SUBROUTINE LSPOLY (N,M,X,Y,W,C,WRK,IER)
C
C DIMENSION OF           X(M),Y(M),W(M),C(N),WRK(N**2+N+M)
C ARGUMENTS
C
C LATEST REVISION        FEBRUARY 1985
C
C PURPOSE                GIVEN A SET OF DATA (X(I),Y(I)), I = 1,...,M,
C                        LSPOLY CALCULATES A SET OF COEFFICIENTS FOR A
C                        WEIGHTED LEAST SQUARES POLYNOMIAL FIT TO THE
C                        GIVEN DATA.  IT IS NECESSARY THAT M (THE NUMBER
C                        OF DATA POINTS) BE GREATER THAN OR EQUAL TO N
C                        (THE NUMBER OF COEFFICIENTS).
C
C USAGE                  CALL LSPOLY (N,M,X,Y,W,C,WRK,IER)
C
C ARGUMENTS
C
C ON INPUT               N
C                          AN INTEGER VARIABLE SET EQUAL TO THE NUMBER
C                          OF COEFFICIENTS DESIRED (I.E., N-1 WILL BE
C                          THE DEGREE OF THE POLYNOMIAL).
C
C                        M
C                          AN INTEGER VARIABLE SET EQUAL TO THE NUMBER
C                          OF DATA POINTS.  M MUST BE GREATER THAN OR
C                          EQUAL TO N.
C
C                        X
C                          A REAL ARRAY WITH DIMENSION AT LEAST M.  ON
C                          INPUT, X CONTAINS THE ABSCISSA VALUES OF THE
C                          DATA POINTS.
C
C                        Y
C                          A REAL ARRAY WITH DIMENSION AT LEAST M.  ON
C                          INPUT, Y CONTAINS THE ORDINATE VALUES OF THE
C                          DATA POINTS.
C
C                        W
C                          A REAL ARRAY WITH DIMENSION AT LEAST M.  ON
C                          INPUT, W IS A VECTOR OF WEIGHTS FOR A
C                          WEIGHTED LEAST SQUARES MODEL.  IF NO
C                          WEIGHTING IS DESIRED, THE ELEMENTS OF W
C                          SHOULD BE SET TO 1 .
C
C                        WRK
C                          A REAL ARRAY WITH DIMENSION AT LEAST N*N+N+M.
C                          WRK IS USED INTERNALLY FOR WORKING STORAGE.
C
C ON OUTPUT              C
C                          A REAL ARRAY WITH DIMENSION AT LEAST N.  ON
C                          OUTPUT, C CONTAINS THE COEFFICIENTS FOR THE
C                          POLYNOMIAL.  C(1) WILL CONTAIN THE CONSTANT
C                          TERM.
C
C                        IER
C                          AN INTEGER ERROR FLAG
C                          = 1  IF M .LT. N.
C                          = 2  IF N .GT. 5 (NON-FATAL).
C                          = 3  IF THE LINEAR SYSTEM IS SINGULAR.
C                          = 0  OTHERWISE.
C
C I/O                    ALL MESSAGES ARE PRINTED USING THE NCAR SYSTEM
C                        ROUTINE ULIBER.
C
C                        FOR IER = 1, THE FOLLOWING MESSAGE IS PRINTED.
C                            M LESS THAN N IN LSPOLY
C
C                        FOR IER = 2, THE FOLLOWING MESSAGE IS PRINTED.
C                            **WARNING** DEGREE OF POLYNOMIAL GREATER
C                                        THAN 4 IN LSPOLY
C
C                        FOR IER = 3, THE FOLLOWING MESSAGE IS PRINTED.
C                            *MATRIX SINGULAR IN LSPOLY*
C
C PRECISION              SINGLE
C
C REQUIRED LIBRARY       ULIBER AND Q8QST4, WHICH ARE AUTOMATICALLY
C FILES                  LOADED ON NCAR'S CRAY MACHINES.
C
C LANGUAGE               FORTRAN
C
C HISTORY                LSPOLY WAS WRITTEN IN THE EARLY 1970'S
C                        BY MEMBERS OF NCAR'S SCIENTIFIC COMPUTING
C                        DIVISION.  LSPOLY REPLACES FORMER ULIB
C                        ROUTINE LSTSQR.
C
C PORTABILITY            FORTRAN 66.  LSPOLY USES THE NCAR ROUTINE
C                        ULIBER TO PRINT MESSAGES, BUT OTHERWISE DOES
C                        NOT CONTAIN MACHINE DEPENDENT CONSTANTS.
C
C ALGORITHM              LSPOLY FORMS THE NORMAL EQUATIONS AND SOLVES
C                        THE RESULTING SQUARE LINEAR SYSTEM USING
C                        GAUSSIAN ELIMINATION WITH FULL PIVOTING.
C
C ACCURACY               FOR LOWER ORDER POLYNOMIALS (N .LE. 5), LSPOLY
C                        CAN BE EXPECTED TO GIVE SATISFACTORY RESULTS.
C                        AS THE DEGREE OF THE FITTING POLYNOMIAL IS
C                        INCREASED, THE PROBLEM BECOMES NUMERICALLY
C                        UNSTABLE.  IF A HIGHER DEGREE POLYNOMIAL FIT
C                        IS DESIRED, THEN THE ULIB ROUTINE RGRSN1 SHOULD
C                        BE USED.
C
C TIMING                 NO CRAY TIMING TESTS HAVE BEEN PERFORMED.
C **********************************************************************
      DIMENSION       X(M)       ,Y(M)       ,W(M)       ,C(N)       ,
     1                WRK(1)
C
C
      I1 = N*N+1
      I2 = N*N+M+1
      CALL LSPLY2 (M,N,X,Y,C,W,WRK,WRK(I1),WRK(I2),IER)
      RETURN
      END
      SUBROUTINE LSPLY2 (M,N,X,Y,C,W,WRK,WRK2,IC,IER)
C
      INTEGER         M          ,N          ,IC(N)      ,IER
      REAL            X(M)       ,Y(M)       ,C(N)       ,W(M)       ,
     1                WRK(N,1)   ,WRK2(M)
C
C TEST INPUT PARAMETERS
C
      IER = 0
      IF (M .GE. N) GO TO  10
      IER = 1
      CALL ULIBER (33,24H M LESS THAN N IN LSPOLY,24)
      RETURN
   10 IF (N .LE. 5) GO TO  20
      IER = 2
      CALL ULIBER (31,59H **WARNING**  DEGREE OF POLYNOMIAL GREATER THAN
     1 4 IN LSPOLY ,59)
   20 CONTINUE
C
C
C FORM NORMAL EQUATIONS
C
      DO  30 I=1,M
         WRK2(I) = W(I)
   30 CONTINUE
      DO  50 J=1,N
         WRK(1,J) = 0.
         C(J) = 0.
         DO  40 I=1,M
            WRK(1,J) = WRK(1,J)+WRK2(I)
            C(J) = C(J)+WRK2(I)*Y(I)
            WRK2(I) = WRK2(I)*X(I)
   40    CONTINUE
   50 CONTINUE
      IF (N .EQ. 1) GO TO 100
      DO  70 J=2,N
         WRK(J,N) = 0.
         DO  60 I=1,M
            WRK(J,N) = WRK(J,N)+WRK2(I)
            WRK2(I) = WRK2(I)*X(I)
   60    CONTINUE
   70 CONTINUE
      DO  90 I=2,N
         DO  80 J=2,N
            WRK(I,J-1) = WRK(I-1,J)
   80    CONTINUE
   90 CONTINUE
  100 NM1 = N-1
      IF (N .EQ. 1) GO TO 240
C
C DECOMPOSE MATRIX USING GAUSSIAN ELIMINATION WITH FULL PIVOTING
C
      NP1 = N+1
      DO 230 K=1,NM1
         KP1 = K+1
         AMAX = 0.
         DO 130 J=K,N
            DO 120 I=K,N
               A = ABS(WRK(I,J))
               IF (AMAX-A) 110,120,120
  110          AMAX = A
               IRT = I
               ICT = J
  120       CONTINUE
  130    CONTINUE
         IC(K) = ICT
         IF (IRT-K) 140,160,140
  140    DO 150 J=1,N
            HOLD = WRK(IRT,J)
            WRK(IRT,J) = WRK(K,J)
            WRK(K,J) = HOLD
  150    CONTINUE
         HOLD = C(IRT)
         C(IRT) = C(K)
         C(K) = HOLD
  160    IF (ICT-K) 170,190,170
  170    DO 180 I=1,N
            HOLD = WRK(I,ICT)
            WRK(I,ICT) = WRK(I,K)
            WRK(I,K) = HOLD
  180    CONTINUE
  190    IF (K .EQ. 1) AMAXFS = ABS(AMAX)
         IF (ABS(AMAX/FLOAT(K))+AMAXFS .EQ. AMAXFS) GO TO 290
         C1 = 1./AMAX
         DO 200 J=KP1,N
            WRK(K,J) = WRK(K,J)*C1
  200    CONTINUE
         C(K) = C(K)*C1
         DO 220 I=KP1,N
            DO 210 J=KP1,N
               WRK(I,J) = WRK(I,J)-WRK(I,K)*WRK(K,J)
  210       CONTINUE
            C(I) = C(I)-WRK(I,K)*C(K)
  220    CONTINUE
  230 CONTINUE
C
C SOLVE FOR COEFFICIENTS USING BACK SUBSTITUTION
C
      IF (ABS(WRK(N,N)/FLOAT(N))+AMAXFS .EQ. AMAXFS) GO TO 290
  240 C(N) = C(N)/WRK(N,N)
      IF (N .EQ. 1) RETURN
      DO 260 I=1,NM1
         NMI = N-I
         DO 250 K=NMI,NM1
            C(NMI) = C(NMI)-WRK(NMI,K+1)*C(K+1)
  250    CONTINUE
  260 CONTINUE
C
C PERMUTE SOLUTION VECTOR
C
      DO 280 I=1,NM1
         NMI = N-I
         ICNMI = IC(NMI)
         IF (ICNMI-NMI) 270,280,270
  270    HOLD = C(ICNMI)
         C(ICNMI) = C(NMI)
         C(NMI) = HOLD
  280 CONTINUE
      RETURN
  290 IER = 3
      CALL ULIBER (33,28H *MATRIX SINGULAR IN LSPOLY*,28)
      RETURN
C
C
C
C
C REVISION HISTORY---
C
C APRIL 1977     FIRST ADDED TO NSSL FROM XLIB
C
C JANUARY 1978     MOVED THE REVISION HISTORY TO APPEAR
C                  BEFORE THE FINAL END CARD
C-----------------------------------------------------------------------
      END
