/* nl.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Test nllst2 with tangent fit.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 5 March 1987
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define MAXIMG 7 
#define MAXPAR 3

#include <math.h>
#include <stdio.h>
int pcount = 0;

main()
{
static char namdat[] = "nl.dat";
static char namemc[] = "nl.mc";
static char ltitle[40];
static char labx[40];
static char laby[40];
int ixscale = 0;
int iyscale = 0;
int ibkg = 0;
int i, ii;			/* General purpose index. */
int nimg = MAXIMG;		/* # images. */
int npar = MAXPAR;		/* # parameters in tangent equation. */
int ierr;			/* error value for nllst2. */
float sum;			/* Sum of values within box. */
static float expose[MAXIMG] = {1.,3.,5.,8.,12.,17.,25.};
static float ybuf[MAXIMG] = {40.,80.,115.,150.,180.,200.,210.};

static float yfit[MAXIMG];	/* Fitted values for 'ybuf'. */
static float alphin = 24.;
static float betain = 177.;
static float offin = 5.;
static float par[MAXPAR];
static float sig[MAXPAR*2];
static char fix[MAXPAR];
float alpha, beta, offset, xerr;
float chisq;
static char answer[2];

int j ;
FILE *datp ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/*
	Open the output data file
*/
   datp = fopen(namdat,"w") ;
   if ( datp == NULL ) 
   {
      fprintf(stdout,"Error in opening output file.\n") ;
      exit(-1) ;
   }
   fprintf(stdout,"Output file %s is opened for writing.\n\n", namdat) ;
   fprintf(datp,"Exp.Time    DN     fit(DN)\n\n") ;


      fix[0] = '\0';
      fix[1] = '\0';
      fix[2] = '\0';

/* Initial values for non-linear least-squares fit. */

         par[0] = alphin;
         par[1] = betain;
         par[2] = offin;

/* Non-linear least squares fit. */

         nllst2_ (ybuf,expose,par,sig,fix,&npar,&nimg,&chisq,&ierr);
/*
      subroutine nllst2(x,y,c,sig,fix,na,m,chisq,ier)
c
c        x         = array of independent quantities
c        y         = array of dependent quantities
c        c         = estimates of the  fitting parameters
c        fix       - logical array
c                  if fix(i) = .true. then c(i) is not corrected
c        na        = number of parameters in fit (including those fixed)
c        m         = number of data points
c
c     User supplied subroutines :
c        func(x,c,na)        func = value at x given parameters c
c        deriv(x,c,na,f,df)  f = value at x given parameters c
c                            df(i) = df/dc(i)
c
c     output :
c        c         = least squares fitted parameters
c        sig       = estimate of standard error in c.
c        chisq     = estimate of unit weight variance of fit
c        ier       = 1 - - > fit is ok
c                  = 2 - - > fit is suspicious
c                  = 3 - - > more than 25 iterations required
c                  = 4 - - > disaster, no fit found
 */
         alpha = par[0];
         beta = par[1];
         offset = par[2];

         for (ii=0; ii<nimg; ii++)
            {
            yfit[ii] = beta * (float) atan2 ((double) expose[ii], (double) alpha) + offset;
            }

/***/
   fprintf (datp, "alpha:%f  beta:%f  offset:%f\n", alpha, beta, offset);
   fprintf (datp, "sig: %f   %f   %f\n", sig[0], sig[1], sig[2]);
   fprintf (datp, "fix: %o   %o   %o\n", fix[0], fix[1], fix[2]);
   fprintf (datp, "chisq: %g   ierr:%d\n", chisq, ierr);
/***/
	 sprintf (ltitle, "Tangent Fit$");
         sprintf (labx, "E x p o s u r e (*1.024 sec)$");
         sprintf (laby, "Vidicon Digital Number (DN)$");

         tanplot (expose, ybuf, yfit, nimg, namemc, ltitle, labx, laby,
	  	   ixscale, iyscale, ibkg);

	 for ( j=0 ; j<nimg ; j++ )
	     fprintf(datp,"%6.3f     %5.1f   %5.1f\n",expose[j], ybuf[j], yfit[j]);

	 fprintf(datp,"\n") ;

	 fclose(datp) ;

}
