/* nlc.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Test nllst2 with tangent fit.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 13 March 1987
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#include <math.h>
#define MAXPAR 3
#define MAXIMG 7

int pcount = 0;

main()
{
   static char namemc[] = "nlc.mc";

   static char ltitle[] = "Tangent Fit$";
   static char labx[] = "E x p o s u r e (*1.024 sec)$";
   static char laby[] = "Vidicon Digital Number (DN)$";
   int ixscale = 0;
   int iyscale = 0;
   int ibkg = 0;
   int i;
   int nimg = MAXIMG;
   int npar = MAXPAR;
   int ierr;
   static float expose[MAXIMG] = {1.,3.,5.,8.,12.,17.,25.};
   static float ybuf[MAXIMG] = {40.,80.,115.,150.,180.,200.,210.};

   static float yfit[MAXIMG];
   float alphin = 24.;
   float betain = 177.;
   float offin = 5.;
   static float par[MAXPAR];
   static float sig[6];
   static int fix[MAXPAR];
   float alpha, beta, offset;
   float chisq;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   fix[0] = 0;
   fix[1] = 0;
   fix[2] = 0;

/* Initial values for non-linear least-squares fit. */

   par[0] = alphin;
   par[1] = betain;
   par[2] = offin;

/* Non-linear least squares fit. */

   nllst2 (ybuf,expose,par,sig,fix,&npar,&nimg,&chisq,&ierr);

/*
********************************************************************************
   nllst2(dn,expose,c,sig,fix,na,m,chisq,ier)
   
   float dn[nval], expose[nval];
   float c[npar], sig[npar], sig[npar];
   float sig[6];
   int fix[npar];
   int *npar, *nval;
   float *chisq;
   int *ier;

     dn        = array of independent quantities
     expose    = array of dependent quantities
     c         = estimates of the  fitting parameters
     fix       - logical array
               if fix(i) = .true. then c(i) is not corrected
     npar      = number of parameters in fit (including those fixed)
     nval      = number of data points

  User supplied functions :
     func(x,c,na)        func = value at x given parameters c
     deriv(x,c,na,f,df)  f = value at x given parameters c
                         df(i) = df/dc(i)

  output :
     c         = least squares fitted parameters
     sig       = estimate of standard error in c.
     chisq     = estimate of unit weight variance of fit
     ier       = 1 - - > fit is ok
               = 2 - - > fit is suspicious
               = 3 - - > more than 25 iterations required
               = 4 - - > disaster, no fit found
********************************************************************************
 */

   alpha = par[1];
   beta  = par[2];
   offset = par[3];

   for (i=0; i<nimg; i++)
      yfit[i] = beta * (float) atan2 ((double) expose[i], (double) alpha) + offset;

   printf ("alpha:%f   beta:%f   offset:%f\n", alpha, beta, offset);
   printf ("sig: %f   %f   %f\n", sig[0], sig[1], sig[2]);
   printf ("fix: %o   %o   %o\n", fix[0], fix[1], fix[2]);
   printf ("chisq:%f   ierr:%d\n ", chisq, ierr);

   tanplot (expose, ybuf, yfit, nimg, namemc, ltitle, labx, laby,
       	   ixscale, iyscale, ibkg);

   printf ("Exp.Time    DN     fit(DN)\n");

   for (i=0; i<nimg; i++)
      printf ("%7.2f   %7.2f   %7.2f\n", expose[i], ybuf[i], yfit[i]);
}

