/* pfplotj.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   22 March 1987
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namfil : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   param: parameters for polynomial equation.
 *   npar: # parameters in polynomial equation.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <math.h>
extern int pcount;

pfplotj (x,y,nval,namfil,title,labx,laby,ixscale,iyscale,ibkg,param,npar)

float	x[], y[];
int	nval;
char	namfil[];
char	title[40], labx[40], laby[40];
int	ixscale, iyscale;
int	ibkg;
float	param[];
int	npar;

{
extern char *malloc();
extern float fpow();
static int iset  = 0;
static int ndash = 0;
static int ldash = 0;
static int ichar = '*' ;

int i, ip;
static int nfit = 64;
static float xmm[2],ymm[2] ;
static float dn[65], expfit[65];
float d;
float fit;

   xmm[0] = 0.0 ;
   xmm[1] = 256.0 ;
   ymm[0] = 0.0 ;
   ymm[1] = 100.0 ;

   pcount ++;
   if(pcount == 1)  q8qini (namfil, 0);
   anotat (labx, laby, 1, -1, 0, 0.0);
   displa (2, 0, 1);
   ezxy (ymm, xmm, 2, title);
   points(y, x, nval, ichar, 0) ;

/* Compute fitted function curve. */

   d = -0.04;
   for (i=0; i<=nfit; i++)
      {
      d += 0.04;
      dn[i] = d * 100.;
      fit = 0.;
      for (ip=0; ip<npar; ip++)
         fit += param[ip] * fpow(d,ip);
      expfit[i] = fit;
      }
   curve (expfit, dn, nfit);
   
   frame() ;

   return(0);
}

float
fpow (x, n)

float x;
int n;

{
float xp;
int i;

   if (n < 0)  return (-1.);
   xp = 1.;
   for (i = 1; i<=n; i++)  xp *= x;
   return (xp);
}
