      subroutine pvalue(l,nder,x,yfit,yp,a)                             
c***begin prologue  pvalue                                              
c***date written   740601   (yymmdd)                                    
c***revision date  820801   (yymmdd)                                    
c***category no.  k6                                                    
c***keywords  curve fitting,least squares,polynomial approximation      
c***author  shampine, l. f., (snla)                                     
c           davenport, s. m., (snla)                                    
c***purpose  to use the coefficients generated by polfit to evaluate the
c            polynomial fit of degree l, along with the first nder of   
c            its derivatives, at a specified point.                     
c***description                                                         
c                                                                       
c     written by l. f. shampine and s. m. davenport.                    
c                                                                       
c     abstract                                                          
c                                                                       
c     the subroutine  pvalue  uses the coefficients generated by  polfit
c     to evaluate the polynomial fit of degree  l , along with the first
c     nder  of its derivatives, at a specified point.  computationally  
c     stable recurrence relations are used to perform this task.        
c                                                                       
c     the parameters for  pvalue  are                                   
c                                                                       
c     input --                                                          
c         l -      the degree of polynomial to be evaluated.  l  may be 
c                  any non-negative integer which is less than or equal 
c                  to  ndeg , the highest degree polynomial provided    
c                  by  polfit .                                         
c         nder -   the number of derivatives to be evaluated.  nder     
c                  may be 0 or any positive value.  if nder is less     
c                  than 0, it will be treated as 0.                     
c         x -      the argument at which the polynomial and its         
c                  derivatives are to be evaluated.                     
c         a -      work and output array containing values from last    
c                  call to  polfit .                                    
c                                                                       
c     output --                                                         
c         yfit -   value of the fitting polynomial of degree  l  at  x  
c         yp -     array containing the first through  nder  derivatives
c                  of the polynomial of degree  l .  yp  must be        
c                  dimensioned at least  nder  in the calling program.  
c***references  shampine l.f., davenport s.m., huddleston r.e., *curve  
c                 fitting by polynomials in one variable*, sla-74-0270, 
c                 sandia laboratories, june 1974.                       
c***routines called  xerror,xerrwv                                      
c***end prologue  pvalue                                                
      dimension yp(1),a(1)                                              
c***first executable statement  pvalue                                  
      if (l .lt. 0) go to 12                                            
      ndo = max0(nder,0)                                                
      ndo = min0(ndo,l)                                                 
      maxord = a(1) + 0.5                                               
      k1 = maxord + 1                                                   
      k2 = k1 + maxord                                                  
      k3 = k2 + maxord + 2                                              
      nord = a(k3) + 0.5                                                
      if (l .gt. nord) go to 11                                         
      k4 = k3 + l + 1                                                   
      if (nder .lt. 1) go to 2                                          
      do 1 i = 1,nder                                                   
 1      yp(i) = 0.0                                                     
 2    if (l .ge. 2) go to 4                                             
      if (l .eq. 1) go to 3                                             
c                                                                       
c l is 0                                                                
c                                                                       
      val = a(k2+1)                                                     
      go to 10                                                          
c                                                                       
c l is 1                                                                
c                                                                       
 3    cc = a(k2+2)                                                      
      val = a(k2+1) + (x-a(2))*cc                                       
      if (nder .ge. 1) yp(1) = cc                                       
      go to 10                                                          
c                                                                       
c l is greater than 1                                                   
c                                                                       
 4    ndp1 = ndo + 1                                                    
      k3p1 = k3 + 1                                                     
      k4p1 = k4 + 1                                                     
      lp1 = l + 1                                                       
      lm1 = l - 1                                                       
      ilo = k3 + 3                                                      
      iup = k4 + ndp1                                                   
      do 5 i = ilo,iup                                                  
 5      a(i) = 0.0                                                      
      dif = x - a(lp1)                                                  
      kc = k2 + lp1                                                     
      a(k4p1) = a(kc)                                                   
      a(k3p1) = a(kc-1) + dif*a(k4p1)                                   
      a(k3+2) = a(k4p1)                                                 
c                                                                       
c evaluate recurrence relations for function value and derivatives      
c                                                                       
      do 9 i = 1,lm1                                                    
        in = l - i                                                      
        inp1 = in + 1                                                   
        k1i = k1 + inp1                                                 
        ic = k2 + in                                                    
        dif = x - a(inp1)                                               
        val = a(ic) + dif*a(k3p1) - a(k1i)*a(k4p1)                      
        if (ndo .le. 0) go to 8                                         
        do 6 n = 1,ndo                                                  
          k3pn = k3p1 + n                                               
          k4pn = k4p1 + n                                               
 6        yp(n) = dif*a(k3pn) + float(n)*a(k3pn-1) - a(k1i)*a(k4pn)     
c                                                                       
c save values needed for next evaluation of recurrence relations        
c                                                                       
        do 7 n = 1,ndo                                                  
          k3pn = k3p1 + n                                               
          k4pn = k4p1 + n                                               
          a(k4pn) = a(k3pn)                                             
 7        a(k3pn) = yp(n)                                               
 8      a(k4p1) = a(k3p1)                                               
 9      a(k3p1) = val                                                   
c                                                                       
c normal return or abort due to error                                   
c                                                                       
 10   yfit = val                                                        
      return                                                            
11     continue
      call xerror( 'pvalue-the order of polynomial evaluation, l(i1), r
     1equested exceeds the highest order fit, nord(i2), computed by polf
     2it -- execution terminated.',144,8,2,2)          
c
c**********************************************
c  commented out call to xerrwv and used call to xerror instead
c-------------------------------------------------------------
c
c   11 call xerrwv ( 'pvalue-the order of polynomial evaluation, l(i1), r
c     1equested exceeds the highest order fit, nord(i2), computed by polf
c     2it -- execution terminated.',144,8,2,2,l,nord,0,0.0,0.0)          
c*******************************************************
c
      return                                                            
   12 call xerror ( 'pvalue-invalid input parameter.  order of polynomia
     1l evaluation requested is negative -- execution terminated.',110, 
     2 2,2)                                                             
      return                                                            
      end                                                               
