/* tcfit1a.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average values within a set of boxes, and plot transfer curve.
 * Use raw DN calibration mirror images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 20 February 1985
 * 19 March 1987: fit tangent curve to transfer curve data points.
 * 30 August 1988: add epsilon variable to tangent equation.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 16
#define MAXBOX 64
#define MAXIMG 100
#define MAXPAR 4

#include <math.h>
#include <stdio.h>
#include "/u/stanger/smm/cpgrin.h"
#include "/u/stanger/smm/cpgraf.h"
#include "/u/stanger/smm/hskp.h"
struct	hskp	H;
int GRN = 0;
int pcount = 0;

main()
{
extern int cphskp();		/* C/P Housekeeping function. */
extern int boxval();		/* Function to extract box values from image. */
static char namlst[80];		/* List of image file names. */
static char namimg[80];		/* Name of image file. */
static char namdat[] = "tcfit1a.dat";
static char namemc[] = "tcfit1a.mc";
static char ltitle[40];
static char labx[40];
static char laby[40];
int ixscale = 0;
int iyscale = 0;
int ibkg = 0;
int x,y;			/* Coordinates of center of box. */
int nave;			/* Size of one dimension of box. */
int lsv;			/* Stray light/vignetting flag: =1 remove. */
static int box[MAXBOX*MAXBOX];	/* Array containing box values. */
int i, ii;			/* General purpose index. */
int imgnum;			/* Image index. */
int nimg;			/* # images. */
int nval;			/* # images to use for fit. */
int p;				/* box index. */
int maxpix;			/* # values in box. */
int npar = MAXPAR;		/* # parameters in tangent equation. */
int ierr;			/* error value for nllst2. */
float sum;			/* Sum of values within box. */
static float ave[NBOX][MAXIMG];	/* Average of values within box. */
static float dn[MAXIMG];	/* Raw digital values from image. */
static float yfit[MAXIMG];	/* Fitted values for 'ave'. */
static float expose[MAXIMG];	/* Index array for images. */
static float alphain = 24.;
static float betain = 177.;
static float gammain = 5.;
static float epsin = 0.0;
static float par[MAXPAR];
static float sig[MAXPAR*2];
static int fix[MAXPAR];
float alpha, beta, gamma, eps, xerr;
float chisq;
static int xtab[NBOX] = {336,112, 56,168,280,392, 56,168,280,392, 56,280,392, 16,280,432};
static int ytab[NBOX] = { 28, 56,112,112,112,112,224,224,224,224,336,336,336,420,420,420};
static char answer[2];
static char ovrcolor[2];
FILE *LIST, *IMG;

int j ;
int ibox;
static int nbox = NBOX;
FILE *datp ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/*
	Open the output data file
*/
   datp = fopen(namdat, "w") ;
   if ( datp == NULL ) 
   {
      fprintf(stdout,"Error in opening output file.\n") ;
      exit(-1) ;
   }
   fprintf(stdout,"Output data file: %s\n", namdat);
   fprintf(stdout,"Output metacode file: %s\n", namemc) ;
   fprintf (datp, "******* Transfer Curve Fits *******\n\n");
   fprintf (datp, "exposure = epsilon + alpha * [(DN - gamma) / beta]\n\n");

/* Read list of images; process one image at a time. */

   printf ("Enter name of file containing list of images : ");
   scanf  ("%s", namlst);
   if ((LIST = fopen (namlst,"r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s",namlst);
      exit(-1);
      }

   printf ("Enter size of box : ");
   scanf  ("%d", &nave);
   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n", nave, MAXBOX);
      exit(-1);
      }

   printf ("Do you wish to use the Grinnell (y/n) ? ");
   scanf ("%s", answer);
   igr = 0;
   GRN = 0;
   if (strncmp("y",answer,1) == 0)
	{
	igr = 1;
	GRN = 1;
	printf ("Enter Grinnell device [0,1] :");
	scanf  ("%d", &ngr);
	if (ngr == 0)  printf ("Enter Grinnell channel [0,1,2] : ");
	else   printf ("Enter Grinnell channel [0,1,2,3] : ");
	scanf  ("%d", &imgch);
	printf ("Enter overlay channel color [r,g,b,w] : ");
	scanf  ("%1s", ovrcolor);
	if (strncmp("r", ovrcolor, 1) == 0)  ovrch = 10;
	if (strncmp("g", ovrcolor, 1) == 0)  ovrch =  9;
	if (strncmp("b", ovrcolor, 1) == 0)  ovrch =  8;
	if (strncmp("w", ovrcolor, 1) == 0)  ovrch = 11;
	gr_start (ngr);
	}

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit(-6);
         }
      imgnum++;
      if (GRN)
	{
	cpview (namimg, imgch, 1);
	rwech(ovrch);
	disp (ovrch);
	gr_erase (ovrch);
	}

      cphskp(namimg);
      fprintf (datp, "Year: 19%2d   DOY: %03d   Time: %02d:%02d   Exposure: %d\n",
               H.year, H.doy, H.hour, H.minute, H.exp);

      expose[imgnum] = (float) H.exp;

/* Box loop : */

      for (i=0; i < nbox; i++)
	{
	x = xtab[i];
	y = ytab[i];
	ibox = i + 1;
	boxval (namimg, x, y, nave, lsv, box, ibox);

	/* Average value of box : */

	sum = 0.;
	maxpix = nave*nave;
	for (p=0; p<maxpix; p++)
		{
		sum += (float) box[p];
		}
	ave[i][imgnum] = sum / (float) maxpix;
 	printf ("box: %d  x: %d  y: %d  size: %d  ave: %f\n",
 		i, x, y, nave, ave[i][imgnum]);
	}
      fclose (IMG);
      }
   fprintf (datp, "\n");

   fix[0] = 0;
   fix[1] = 0;
   fix[2] = 0;
   nimg = imgnum + 1;

   printf ("nimg: %d   nbox:%d\n", nimg, nbox);

   for (i=0; i < nbox; i++)
      {
      for (ii = 0; ii < nimg; ii++)
         {
         dn[ii] = ave[i][ii];
         if (ii > 0 && dn[ii] <= dn[ii-1])
            {
            nval = ii;
            break;
            }
         nval = ii + 1;
         }
      printf ("nval: %d\n", nval);

/* Initial values for non-linear least-squares fit. */

      par[0] = alphain;
      par[1] = betain;
      par[2] = gammain;
      par[3] = epsin;

/* Non-linear least squares fit. */

      nllst2_ (dn, expose, par, sig, fix, &npar, &nval, &chisq, &ierr);
/*
 *    subroutine nllst2(y,x,c,sig,fix,na,m,chisq,ier)
 *
 *       x         = array of independent quantities
 *       y         = array of dependent quantities
 *       c         = estimates of the  fitting parameters
 *       fix       - logical array
 *                 if fix(i) = .true. then c(i) is not corrected
 *       na        = number of parameters in fit (including those fixed)
 *       m         = number of data points
 *
 *    User supplied subroutines :
 *       func(x,c,na)        func = value at x given parameters c
 *       deriv(x,c,na,f,df)  f = value at x given parameters c
 *                           df(i) = df/dc(i)
 *
 *    output :
 *       c         = least squares fitted parameters
 *       sig       = estimate of standard error in c.
 *       chisq     = estimate of unit weight variance of fit
 *       ier       = 1 - - > fit is ok
 *                 = 2 - - > fit is suspicious
 *                 = 3 - - > more than 25 iterations required
 *                 = 4 - - > disaster, no fit found
 */
      alpha  = par[0];
      beta   = par[1];
      gamma  = par[2];
      eps    = par[3];

/***/
      fprintf (datp, "nval: %d\n", nval);
      fprintf (datp, "alpha:%8.2f  beta:%8.2f  gamma:%8.2f  eps:%9.3f\n",
               alpha, beta, gamma, eps);
      fprintf (datp, "sig: %9.3f   %9.3f   %9.3f   %9.3f\n",
               sig[0], sig[1], sig[2], sig[3]);
      fprintf (datp, "fix: %o   %o   %o\n", fix[0], fix[1], fix[2]);
      fprintf (datp, "chisq: %g   ierr:%d\n", chisq, ierr);
      fprintf (datp, "\n");
/***/
      sprintf (ltitle, "CM 19%d:%03d Box#%d: (%d,%d) %dx%d$",
		  H.year, H.doy, i+1, xtab[i], ytab[i], nave, nave);
      sprintf (labx, "E x p o s u r e (*1.024 sec)$");
      sprintf (laby, "Vidicon Digital Number (DN)$");

      tfplota (expose, &ave[i][0], nimg, namemc, ltitle, labx, laby,
		ixscale, iyscale, ibkg, alpha, beta, gamma, eps);

      fprintf(datp,"BOX   Exp.Time    DN     fit(DN)\n") ;
      for ( j=0 ; j<nimg ; j++ )
         {
         yfit[j] = gamma +
            beta * (float) atan2 ((double) (expose[j] - eps), (double) alpha);
         fprintf(datp,"%2d    %6.3f     %5.1f   %5.1f\n",
		i+1, expose[j], ave[i][j],  yfit[j]);
         }

      fprintf(datp,"\n") ;
      }

   fclose(datp) ;
}
