/* tcfit3j.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average values within a set of boxes, and plot transfer curve.
 * Use raw DN calibration mirror images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 7 September 1988: linear fit to 5th order polynomial curve.
 * Linear least-squares fit from book by Bevington:
 *    "Data Reduction for the Physical Sciences".
 * 18 May 1989: One box only for Jupiter offpoint position.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 2
#define MAXBOX 32
#define MAXIMG 25 
#define MAXPAR 4

#include <math.h>
#include <stdio.h>
#include "/u/stanger/bin/cplib.h"
int GRN = 0;
int pcount = 0;

main()
{
extern int cphskp();		/* C/P Housekeeping function. */
extern int boxval();		/* Function to extract box values from image. */
static char namlst[80];		/* List of image file names. */
static char namimg[80];		/* Name of image file. */
static char namdat[] = "tcfit3j.dat";
static char namemc[] = "tcfit3j.mc";
static char ltitle[40];
static char labx[40];
static char laby[40];
int ixscale = 0;
int iyscale = 0;
int ibkg = 0;
int x,y;			/* Coordinates of center of box. */
int nave;			/* Size of one dimension of box. */
int lsv;			/* Stray light/vignetting flag: =1 remove. */
static int box[MAXBOX*MAXBOX];	/* Array containing box values. */
int i, ii;			/* General purpose index. */
int imgnum;			/* Image index. */
int nimg;			/* # images. */
int nval;			/* # images to use for fit. */
int p;				/* box index. */
int maxpix;			/* # values in box. */
int mode = 0;
int npar = MAXPAR;		/* # parameters in polynomial equation. */
float delta;
float chisqr;
float sum;			/* Sum of values within box. */
float yfit;			/* Fitted values for 'ave'. */
float z, z2, z3;
static float ave[NBOX][MAXIMG];	/* Average of values within box. */
static float dn[MAXIMG];	/* Raw digital values from image. */
static float expose[MAXIMG];	/* Index array for images. */
static float sigmay[MAXIMG];	/* Index array for images. */
static float a0in = 1.;
static float a1in = 1.;
static float a2in = 1.;
static float a3in = 1.;
static float par[MAXPAR];
float a0, a1, a2, a3;
static int xtab[NBOX] = {320, 320};
static int ytab[NBOX] = {250, 230};
static char answer[2];
static char ovrcolor[2];
FILE *LIST, *IMG;

int j ;
int ibox;
int ndeg;
static int nbox = NBOX;
FILE *datp ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/*
	Open the output data file
*/
   datp = fopen(namdat, "w") ;
   if ( datp == NULL ) 
   {
      fprintf(stdout,"Error in opening output file.\n") ;
      exit(-1) ;
   }
   fprintf (datp, "******* Transfer Curve Fit *******\n\n");
   fprintf (datp, "Linear Least Squares: PFIT (Bevington).\n\n");
   fprintf (datp, "exposure = a0 + a1 * z + a2 * z**2 + a3 * z**3\n\n");
   fprintf(stdout,"Output data file: %s\n", namdat);
   fprintf(stdout,"Output metacode file: %s\n", namemc) ;

/* Read list of images; process one image at a time. */

   printf ("Enter name of file containing list of images : ");
   scanf  ("%s", namlst);
   if ((LIST = fopen (namlst,"r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s",namlst);
      exit(-1);
      }

   printf ("Enter size of box : ");
   scanf  ("%d", &nave);
   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n", nave, MAXBOX);
      exit(-1);
      }

   printf ("Do you wish to use the Grinnell (y/n) ? ");
   scanf ("%s", answer);
   igr = 0;
   GRN = 0;
   if (strncmp("y",answer,1) == 0)
	{
	igr = 1;
	GRN = 1;
	printf ("Enter Grinnell device [0,1] :");
	scanf  ("%d", &ngr);
	if (ngr == 0)  printf ("Enter Grinnell channel [0,1,2] : ");
	else   printf ("Enter Grinnell channel [0,1,2,3] : ");
	scanf  ("%d", &imgch);
	printf ("Enter overlay channel color [r,g,b,w] : ");
	scanf  ("%1s", ovrcolor);
	if (strncmp("r", ovrcolor, 1) == 0)  ovrch = 10;
	if (strncmp("g", ovrcolor, 1) == 0)  ovrch =  9;
	if (strncmp("b", ovrcolor, 1) == 0)  ovrch =  8;
	if (strncmp("w", ovrcolor, 1) == 0)  ovrch = 11;
	gr_start (ngr);
	}

   /*** Image Loop. ***/

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit(-6);
         }
      imgnum++;
      if (GRN)
	{
	cpview (namimg, imgch, 1);
	rwech(ovrch);
	disp (ovrch);
	gr_erase (ovrch);
	}

      cphskp(namimg);
      fprintf (datp, "Year: 19%2d   DOY: %03d   Time: %02d:%02d   Exposure: %d\n",
               H.year, H.doy, H.hour, H.minute, H.exp);

      expose[imgnum] = (float) H.exp;

      /*** Box loop : pixel value extraction. ***/

      for (i=0; i < nbox; i++)
	{
	x = xtab[i];
	y = ytab[i];
	ibox = i + 1;
	boxval (namimg, x, y, nave, lsv, box, ibox);

	/* Average value of box : */

	sum = 0.;
	maxpix = nave*nave;
	for (p=0; p<maxpix; p++)
		{
		sum += (float) box[p];
		}
	ave[i][imgnum] = (sum / (float) maxpix);
 	printf ("box: %d  x: %d  y: %d  size: %d  ave: %f\n",
 		i, x, y, nave, ave[i][imgnum]);
	}
      fclose (IMG);
      }

   nimg = imgnum + 1;
   printf ("nimg: %d   nbox:%d\n", nimg, nbox);

   /*** Box curve fitting loop. ***/

   for (i=0; i < nbox; i++)
      {
      for (ii = 0; ii < nimg; ii++)
         {
         dn[ii] = ave[i][ii] * 0.01;
/*         if (ii > 0 && dn[ii] <= dn[ii-1]) */
         if (ave[i][ii] > 220.0)
            {
            nval = ii;
            break;
            }
         nval = ii + 1;
         }
      printf ("nval: %d\n", nval);

/* Initial values for non-linear least-squares fit. */

      par[0] = a0in;
      par[1] = a1in;
      par[2] = a2in;
      par[3] = a3in;

      for (ii=0; ii<MAXIMG; ii++)  sigmay[ii] = 0.0;

/* Linear least squares fit. */

      pfit_ (dn, expose, sigmay, &nval, &npar, &mode, par, &chisqr);

      a0  = par[0];
      a1  = par[1];
      a2  = par[2];
      a3  = par[3];

      /*** Plot data points and fitted curve. ***/

      sprintf (ltitle, "CM 19%d:%d Box#%d:(%d,%d) %dx%d PF3$",
		  H.year, H.doy, i+1, xtab[i], ytab[i], nave, nave);
      sprintf (labx, "E x p o s u r e (*1.024 sec)$");
      sprintf (laby, "Vidicon Digital Number (DN)$");

      pfplotj (&ave[i][0], expose, nimg, namemc, ltitle, labx, laby,
		ixscale, iyscale, ibkg, par, npar);

      /*** Report results of curve fits. ***/

      fprintf (datp, "\nBox # %d   center coordinates: (%3d,%3d)",
               i+1, xtab[i], ytab[i]); 
      fprintf (datp, "   #images used: %d\n\n", nval);
      fprintf (datp, "a0:%8.2f  a1:%8.2f  a2:%8.2f  a3:%8.2f\n",
          a0, a1, a2, a3);
      fprintf (datp, "chi-squared: %12.5f\n", chisqr);
      fprintf (datp, "\n");
      fprintf(datp," Exp.Time    fit(exp)   delta(exp)%%         DN\n") ;
      for ( j=0 ; j<nimg ; j++ )
         {
         z = dn[j];
         z2 = z * z;
         z3 = z2 * z;
         yfit = a0 + a1*z + a2*z2 + a3*z3;
         delta = ((yfit - expose[j]) / expose[j]) * 100.0;

         fprintf(datp,"%9.3f   %9.3f    %9.3f      %6.2f\n",
		 expose[j], yfit, delta, ave[i][j]);
         }
      }

   fclose(datp) ;
}
