/* tcfit5.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average values within a set of boxes, and plot transfer curve.
 * Use raw DN calibration mirror images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 23 March 1987: fit 5th order polynomial curve to transfer exposure data points.
 * Non-linear least squares fit (nllst2.f).
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 16
#define MAXBOX 64
#define MAXIMG 100
#define MAXPAR 6

#include <math.h>
#include <stdio.h>
#include "/u/stanger/smm/cpgrin.h"
#include "/u/stanger/smm/cpgraf.h"
#include "/u/stanger/smm/hskp.h"
struct	hskp	H;
int GRN = 0;
int pcount = 0;

main()
{
extern int cphskp();		/* C/P Housekeeping function. */
extern int boxval();		/* Function to extract box values from image. */
static char namlst[80];		/* List of image file names. */
static char namimg[80];		/* Name of image file. */
static char namdat[] = "tcfit5.dat";
static char namemc[] = "tcfit5.mc";
static char ltitle[40];
static char labx[40];
static char laby[40];
int ixscale = 0;
int iyscale = 0;
int ibkg = 0;
int x,y;			/* Coordinates of center of box. */
int nave;			/* Size of one dimension of box. */
int lsv;			/* Stray light/vignetting flag: =1 remove. */
static int box[MAXBOX*MAXBOX];	/* Array containing box values. */
int i, ii;			/* General purpose index. */
int imgnum;			/* Image index. */
int nimg;			/* # images. */
int nval;			/* # images to use for fit. */
int p;				/* box index. */
int maxpix;			/* # values in box. */
int npar = MAXPAR;		/* # parameters in tangent equation. */
int ierr;			/* error value for nllst2. */
float sum;			/* Sum of values within box. */
static float ave[NBOX][MAXIMG];	/* Average of values within box. */
static float dn[MAXIMG];	/* Raw digital values from image. */
static float yfit[MAXIMG];	/* Fitted values for 'ave'. */
static float expose[MAXIMG];	/* Index array for images. */
static float a0in = 1.;
static float a1in = 1.;
static float a2in = 1.;
static float a3in = 1.;
static float a4in = 1.;
static float a5in = 1.;
static float par[MAXPAR];
static float sig[MAXPAR*2];
static int fix[MAXPAR];
float a0, a1, a2, a3, a4, a5;
float chisq;
static int xtab[NBOX] = {336,112, 56,168,280,392, 56,168,280,392, 56,280,392, 16,280,432};
static int ytab[NBOX] = { 28, 56,112,112,112,112,224,224,224,224,336,336,336,420,420,420};
static char answer[2];
static char ovrcolor[2];
FILE *LIST, *IMG;

int j ;
int ibox;
static int nbox = NBOX;
FILE *datp ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/*
	Open the output data file
*/
   datp = fopen(namdat, "w") ;
   if ( datp == NULL ) 
   {
      fprintf(stdout,"Error in opening output file.\n") ;
      exit(-1) ;
   }
   fprintf(stdout,"Output data file: %s\n", namdat);
   fprintf(stdout,"Output metacode file: %s\n", namemc) ;

/* Read list of images; process one image at a time. */

   printf ("Enter name of file containing list of images : ");
   scanf  ("%s", namlst);
   if ((LIST = fopen (namlst,"r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s",namlst);
      exit(-1);
      }

   printf ("Enter size of box : ");
   scanf  ("%d", &nave);
   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n", nave, MAXBOX);
      exit(-1);
      }

   printf ("Do you wish to use the Grinnell (y/n) ? ");
   scanf ("%s", answer);
   igr = 0;
   GRN = 0;
   if (strncmp("y",answer,1) == 0)
	{
	igr = 1;
	GRN = 1;
	printf ("Enter Grinnell device [0,1] :");
	scanf  ("%d", &ngr);
	if (ngr == 0)  printf ("Enter Grinnell channel [0,1,2] : ");
	else   printf ("Enter Grinnell channel [0,1,2,3] : ");
	scanf  ("%d", &imgch);
	printf ("Enter overlay channel color [r,g,b,w] : ");
	scanf  ("%1s", ovrcolor);
	if (strncmp("r", ovrcolor, 1) == 0)  ovrch = 10;
	if (strncmp("g", ovrcolor, 1) == 0)  ovrch =  9;
	if (strncmp("b", ovrcolor, 1) == 0)  ovrch =  8;
	if (strncmp("w", ovrcolor, 1) == 0)  ovrch = 11;
	gr_start (ngr);
	}

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit(-6);
         }
      imgnum++;
      if (GRN)
	{
	cpview (namimg, imgch, 1);
	rwech(ovrch);
	disp (ovrch);
	gr_erase (ovrch);
	}

      cphskp(namimg);

      expose[imgnum] = (float) H.exp;

/* Box loop : */

      for (i=0; i < nbox; i++)
	{
	x = xtab[i];
	y = ytab[i];
	ibox = i + 1;
	boxval (namimg, x, y, nave, lsv, box, ibox);

	/* Average value of box : */

	sum = 0.;
	maxpix = nave*nave;
	for (p=0; p<maxpix; p++)
		{
		sum += (float) box[p];
		}
	ave[i][imgnum] = sum / (float) maxpix;
 	printf ("box: %d  x: %d  y: %d  size: %d  ave: %f\n",
 		i, x, y, nave, ave[i][imgnum]);
	}
      fclose (IMG);
      }

   fix[0] = 0;
   fix[1] = 0;
   fix[2] = 0;
   fix[3] = 0;
   nimg = imgnum + 1;

   printf ("nimg: %d   nbox:%d\n", nimg, nbox);

   for (i=0; i < nbox; i++)
      {
      for (ii = 0; ii < nimg; ii++)
         {
         dn[ii] = ave[i][ii];
         if (ii > 0 && dn[ii] <= dn[ii-1])
            {
            nval = ii;
            break;
            }
         nval = ii + 1;
         }
      printf ("nval: %d\n", nval);

/* Initial values for non-linear least-squares fit. */

      par[0] = a0in;
      par[1] = a1in;
      par[2] = a2in;
      par[3] = a3in;
      par[4] = a4in;
      par[5] = a5in;

/* Non-linear least squares fit. */

      nllst2_ (dn, expose, par, sig, fix, &npar, &nval, &chisq, &ierr);
/*
      subroutine nllst2(y,x,c,sig,fix,na,m,chisq,ier)
c
c        x         = array of independent quantities
c        y         = array of dependent quantities
c        c         = estimates of the  fitting parameters
c        fix       - logical array
c                  if fix(i) = .true. then c(i) is not corrected
c        na        = number of parameters in fit (including those fixed)
c        m         = number of data points
c
c     User supplied subroutines :
c        func(x,c,na)        func = value at x given parameters c
c        deriv(x,c,na,f,df)  f = value at x given parameters c
c                            df(i) = df/dc(i)
c
c     output :
c        c         = least squares fitted parameters
c        sig       = estimate of standard error in c.
c        chisq     = estimate of unit weight variance of fit
c        ier       = 1 - - > fit is ok
c                  = 2 - - > fit is suspicious
c                  = 3 - - > more than 25 iterations required
c                  = 4 - - > disaster, no fit found
 */
      a0  = par[0];
      a1  = par[1];
      a2  = par[2];
      a3  = par[3];
      a4  = par[4];
      a5  = par[5];

/***/
      fprintf (datp, "nval: %d\n", nval);
      fprintf (datp, "a0:%8.2f  a1:%8.2f  a2:%8.2f  a3:%8.2f   a4:%8.2f   a5:%8.2f\n",
          a0, a1, a2, a3, a4, a5);
      fprintf (datp, "sig: %8.2f   %8.2f   %8.2f   %8.2f   %8.2f\n",
          sig[0], sig[1], sig[2], sig[3], sig[4], sig[5]);
      fprintf (datp, "fix: %o   %o   %o   %o   %o\n",
         fix[0], fix[1], fix[2], fix[3], fix[4], fix[5]);
      fprintf (datp, "chisq: %g   ierr:%d\n", chisq, ierr);
      fprintf (datp, "\n");
/***/
      sprintf (ltitle, "CM 19%d:%d Box#%d: (%d,%d) %dx%d P5$",
		  H.year, H.doy, i+1, xtab[i], ytab[i], nave, nave);
      sprintf (labx, "E x p o s u r e (*1.024 sec)$");
      sprintf (laby, "Vidicon Digital Number (DN)$");

      pfplot (expose, &ave[i][0], nimg, namemc, ltitle, labx, laby,
		ixscale, iyscale, ibkg, par, npar);

      fprintf(datp,"BOX   Exp.Time    DN     fit(DN)\n\n") ;
      for ( j=0 ; j<nimg ; j++ )
         fprintf(datp,"%2d    %6.3f     %5.1f   %5.1f\n",
		i, expose[j], ave[i][j],  yfit[j]);

      fprintf(datp,"\n") ;
      }

   fclose(datp) ;
}
