/* tcfit3g.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average values within a set of boxes, and plot transfer curve.
 * Use raw DN calibration mirror images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 7 September 1988: linear fit to 5th order polynomial curve.
 * Linear least-squares fit from book by Bevington:
 *    "Data Reduction for the Physical Sciences".
 * 10 September 1991: SGI/IRIS version.
 * 16 October 1992: increase chisqr tolerance to 0.2
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 16
#define MAXBOX 32
#define MAXIMG 25 
#define MAXPAR 4
#define cp_black 0
#define cp_yellow 250
#define cp_grey 251
#define cp_blue 252
#define cp_green 253
#define cp_red 254
#define cp_white 255

#include <math.h>
#include <stdio.h>
#include <whip.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/grscale.h"
int DSP = 0;

main (argc, argv)

int argc;
char *argv [];
{
   extern int cphskp ();	/* C/P Housekeeping function. */
   extern int cp_view();	/* Image display function. */
   extern int boxval ();	/* Function to extract box values from image. */

   static char answer [2];
   static char gksout [40];	/* NCARG_GKS_OUTPUT env. variable value. */
   static char labx [40];
   static char laby [40];
   static char ltitle [40];
   static char namlst [80];	/* List of image file names. */
   static char namimg [80];	/* Name of image file. */
   static char namdat [80];
   static char namemc [80];
   static char ovrcolor [2];
   static char dropch;

   static int box [MAXBOX*MAXBOX];	/* Array containing box values. */
   static int drop [NBOX];
   static int xtab [NBOX] = {336,112, 56,168,280,392, 56,168,280,392, 56,280,392, 16,280,432};
   static int ytab [NBOX] = { 28, 56,112,112,112,112,224,224,224,224,336,336,336,420,420,420};

   int FB = 0;		/* default: 512x512 byte frame buffer (colormap mode).*/
   int fast = 1;	/* Parameter for fb_clear. */
   int i, ii;			/* General purpose index. */
   int ibkg = 0;
   int ibox;
   int ndeg;
   int imgnum;			/* Image index. */
   int ixscale = 0;
   int iyscale = 0;
   int j ;
   int lsv;			/* Stray light/vignetting flag: =1 remove. */
   int maxpix;			/* # values in box. */
   int mode = 0;
   int nave = 16;		/* Size of one dimension of box. */
   int nbox = NBOX;
   int nimg;			/* # images. */
   int nval;			/* # images to use for fit. */
   int npar = MAXPAR;		/* # parameters in polynomial equation. */
   int ovrcol = cp_red;		/* color map index. */
   int p;			/* box index. */
   int x,y;			/* Coordinates of center of box. */

   float a0, a1, a2, a3;
   float bkgcol = 0.0;		/* Grey level for fb_clear. */
   float chisqr;
   float delta;			/* (fit - expose) / expose */
   float diff;			/* expose - fit */
   float ratio;			/* fit / expose */
   float sum;			/* Sum of values within box. */
   float yfit;			/* Fitted values for 'ave'. */
   float z, z2, z3;

   static float a0in = 1.0;
   static float a1in = 1.0;
   static float a2in = 1.0;
   static float a3in = 1.0;
   static float ave [NBOX][MAXIMG];	/* Average of values within box. */
   static float dn [MAXIMG];		/* Raw digital values from image. */
   static float expose [MAXIMG];	/* Index array for images. */
   static float sigmay [MAXIMG];	/* Index array for images. */
   static float par [MAXPAR];

   FILE *LIST, *IMG;
   FILE *datp ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   sprintf (namdat, "%s.dat", argv [0]);
   sprintf (namemc, "%s.cgm", argv [0]);

   if (argc == 2)
      strcpy (namlst, argv [1]);
   else
   if (argc == 3 || argc == 4 || argc == 5)
      {
      for (i = 1; i < (argc - 1); i++)
         {
         printf ("i: %d   argv: %s\n", i, argv [i]);
         if (strcmp (argv [i], "-s") == 0)
            {
            i++;
            nave = atoi (argv [i]);
            }
         else
         if (strcmp (argv [i], "-g") == 0)
            {
            DSP = 1;
            igr = 1;
            }
         else
            {
            fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
                     argv[0]);
            exit (-1);
            }
         }
      strcpy (namlst, argv [argc-1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
               argv[0]);
      exit (-1);
      }

   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
   putenv (gksout);
   opngks_ ();				/* Initialize NCAR graphics (GKS). */

   if ((LIST = fopen (namlst,"r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s",namlst);
      exit(-1);
      }

   /*** Open the output data file ***/

   datp = fopen (namdat, "w") ;
   if ( datp == NULL ) 
      {
      fprintf (stdout, "Error in opening output file.\n") ;
      exit (-1) ;
      }
   fprintf (datp, "******* Transfer Curve Fit *******\n\n");
   fprintf (datp, "Linear Least Squares: PFIT (Bevington).\n\n");
   fprintf (datp, "exposure = a0 + a1 * z + a2 * z**2 + a3 * z**3\n\n");
   fprintf (stdout, "Output data file: %s\n", namdat);
   fprintf (stdout, "Output metacode file: %s\n", namemc) ;

   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n", nave, MAXBOX);
      exit (-1);
      }

   if (DSP == 1)
      {
      printf ("Enter Frame Buffer [0-35] :");
      scanf  ("%d", &FB);
      printf ("Enter overlay color [r,g,b,w] : ");
      scanf  ("%1s", ovrcolor);
      if (strncmp ("r", ovrcolor, 1) == 0)  ovrcol = cp_red;
      if (strncmp ("g", ovrcolor, 1) == 0)  ovrcol = cp_green;
      if (strncmp ("b", ovrcolor, 1) == 0)  ovrcol = cp_blue;
      if (strncmp ("w", ovrcolor, 1) == 0)  ovrcol = cp_white;

      /*** Initialize WHIP software. ***/

      fb_open ();
      openpl ();
      fb_setfb (FB);
      fb_clear (FB, &bkgcol, fast);
      }

   /*** Image Loop. ***/

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr,"Cannot open %s",namimg);
         exit (-6);
         }
      imgnum++;
      if (DSP)
	{
	cp_view (namimg, imgch, 1);
	}

      cphskp (namimg);
      fprintf (datp, "Year: 19%2d   DOY: %03d   Time: %02d:%02d   Exposure: %d\n",
               H.year, H.doy, H.hour, H.minute, H.exp);

      expose [imgnum] = (float) H.exp;

      /*** Box loop : pixel value extraction. ***/

      for (i=0; i < nbox; i++)
         {
         x = xtab [i];
         y = ytab [i];
         ibox = i + 1;
         boxval (namimg, x, y, nave, lsv, box, ibox, ovrcol);

         /*** Average value of box : ***/

         sum = 0.0;
         maxpix = nave*nave;
         for (p=0; p<maxpix; p++)
            {
            sum += (float) box[p];
            }
         ave [i][imgnum] = (sum / (float) maxpix);
/*
 *         printf ("box: %d  x: %d  y: %d  size: %d  ave: %f\n",
 *                 i, x, y, nave, ave[i][imgnum]);
 */
         }
      fclose (IMG);
      }

   nimg = imgnum + 1;
   printf ("nimg: %d   nbox:%d\n", nimg, nbox);

   /*** Box curve fitting loop. ***/

   for (i=0; i < nbox; i++)
      {
      for (ii = 0; ii < nimg; ii++)   dn [ii] = ave [i][ii];

      for (ii = 0; ii < nimg; ii++)
         {
         if ((ii > 0 && dn [ii] <= dn [ii-1])
         || (ave [i][ii] > 245.0))
            {
            nval = ii;
            break;
            }
         nval = ii + 1;
         }

      /*** Initial values for non-linear least-squares fit. ***/

      par [0] = a0in;
      par [1] = a1in;
      par [2] = a2in;
      par [3] = a3in;

      for (ii=0; ii<MAXIMG; ii++)  sigmay [ii] = 0.0;

      /*** Linear least squares fit. ***/

      pfit_ (dn, expose, sigmay, &nval, &npar, &mode, par, &chisqr);

      while (chisqr > 1.25)
	 {
	 if (nval-- < 4)
	    break;
         pfit_ (dn, expose, sigmay, &nval, &npar, &mode, par, &chisqr);
         }

      printf ("box # %2d   nval: %2d\n", i+1, nval);

      a0  = par [0];
      a1  = par [1];
      a2  = par [2];
      a3  = par [3];

      /*** Plot data points and fitted curve. ***/

      sprintf (ltitle, "CM 19%d:%d Box#%d:(%d,%d) %dx%d PF3$",
		  H.year, H.doy, i+1, xtab [i], ytab [i], nave, nave);
      sprintf (labx, "E x p o s u r e (*1.024 sec)$");
      sprintf (laby, "Vidicon Digital Number (DN)$");

      pfplot5 (&ave [i][0], expose, nimg, namemc, ltitle, labx, laby,
		ixscale, iyscale, ibkg, par, npar, nval);

      /*** Report results of curve fits. ***/

      fprintf (datp, "Box # %d   center coordinates: (%3d,%3d)   #images used: %d\n",
               i+1, xtab [i], ytab [i], nval); 
      fprintf (datp, " a0: %8.2e  a1: %8.2e  a2: %8.2e  a3: %8.2e   chi-square: %7.5f\n",
          a0, a1, a2, a3, chisqr);
      fprintf (datp, "\n");
      fprintf (datp," Exp.Time    fit (exp)      fit/exp    (fit-exp)    (fit-exp)/exp       DN\n") ;
      for ( j=0 ; j<nimg ; j++ )
         {
         z = dn [j];
         z2 = z * z;
         z3 = z2 * z;
         yfit = a0 + a1*z + a2*z2 + a3*z3;
         delta = ((yfit - expose [j]) / expose [j]) * 100.0;
         diff = yfit - expose [j];
         ratio = yfit / expose [j];

         if (j < nval)  dropch = ' ';  else dropch = '*';
         fprintf (datp,"%9.3f   %9.3f    %9.3f    %9.3f      %9.3f %%      %6.2f %c\n",
		 expose [j], yfit, ratio, diff, delta, dn [j], dropch);
         }
      fprintf (datp, "\n* indicates images not used for curve fit.\n");
      }

   fb_touch ();
   closepl ();
   fb_close ();
   fclose (datp) ;
   clsgks_ ();
   sprintf (gksout, "NCARG_GKS_OUTPUT=gks.cgm");
   putenv (gksout);
   exit (0);
}
