/* tctan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average values within a set of boxes, and plot transfer curve.
 * Use raw DN calibration mirror images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 20 February 1985
 * 19 March 1987: fit tangent curve to transfer curve data points.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#define NBOX 16
#define MAXBOX 64
#define MAXIMG 100
#define MAXPAR 3
#define cp_black 0
#define cp_yellow 250
#define cp_grey 251
#define cp_blue 252
#define cp_green 253
#define cp_red 254
#define cp_white 255

#include <math.h>
#include <stdio.h>
#include <whip.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/grscale.h"

int DSP = 0;
int pcount = 0;

main (argc, argv)

int argc;
char *argv [];
{
   extern int cphskp();		/* C/P Housekeeping function. */
   extern int boxval();		/* Function to extract box values from image. */

   static char answer [2];
   static char gksout [80];		/* NCARG_GKS_OUTPUT */
   static char namlst [80];		/* List of image file names. */
   static char namimg [80];		/* Name of image file. */
   static char namdat [] = "TCtan.dat";
   static char namemc [] = "TCtan.cgm";
   static char ltitle [40];
   static char labx [40];
   static char laby [40];
   static char ovrcolor [2];

   static int box [MAXBOX*MAXBOX];	/* Array containing box values. */
   static int fix [MAXPAR];
   static int xtab [NBOX] = {336,112, 56,168,280,392, 56,168,280,392, 56,280,392, 16,280,432};
   static int ytab [NBOX] = { 28, 56,112,112,112,112,224,224,224,224,336,336,336,420,420,420};

   int FB = 0;		/* default: 512x512 byte frame buffer (colormap mode).*/
   int fast = 1;	/* Parameter for fb_clear. */
   int i, ii;			/* General purpose index. */
   int ibkg = 0;
   int ibox;
   int ierr;			/* error value for nllst2. */
   int imgnum;			/* Image index. */
   int ixscale = 0;
   int iyscale = 0;
   int j ;
   int lsv;			/* Stray light/vignetting flag: =1 remove. */
   int maxpix;			/* # values in box. */
   int nave = 8;		/* Size of one dimension of box. */
   int nbox = NBOX;
   int nimg;			/* # images. */
   int nval;			/* # images to use for fit. */
   int npar = MAXPAR;		/* # parameters in tangent equation. */
   int ovrcol = cp_red;		/* color map index. */
   int p;			/* box index. */
   int x,y;			/* Coordinates of center of box. */

   static float ave [NBOX][MAXIMG];	/* Average of values within box. */
   static float dn [MAXIMG];		/* Raw digital values from image. */
   static float yfit [MAXIMG];		/* Fitted values for 'ave'. */
   static float expose [MAXIMG];	/* Index array for images. */
   static float alphain = 24.0;
   static float betain = 177.0;
   static float gammain = 5.0;
   static float par [MAXPAR];
   static float sig [MAXPAR*2];

   float alpha, beta, gamma, xerr;
   float bkgcol = 0.0;		/* Grey level for fb_clear. */
   float chisq;
   float sum;				/* Sum of values within box. */

   FILE *LIST, *IMG;
   FILE *datp ;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   if (argc == 2)
      strcpy (namlst, argv [1]);
   else
   if (argc == 3 || argc == 4 || argc == 5)
      {
      for (i = 1; i < (argc - 1); i++)
         {
         if (strcmp (argv [i], "-s") == 0)
            {
            i++;
            nave = atoi (argv [i]);
            }
         else
         if (strcmp (argv [i], "-g") == 0)
            {
            DSP = 1;
            igr = 1;
            }
         else
            {
            fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
                     argv [0]);
            exit (-1);
            }
         }
      strcpy (namlst, argv [argc-1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-g] [-s box_size] filelist_name\n",
               argv[0]);
      exit (-1);
      }

   if (nave > MAXBOX)
      {
      printf ("nave:%d > MAXBOX:%d\n", nave, MAXBOX);
      exit (-1);
      }

   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
   putenv (gksout);
   opngks_ ();				/* Initialize NCAR graphics (GKS). */

   /*** Open the output data file ***/

   datp = fopen (namdat, "w") ;
   if ( datp == NULL ) 
      {
      fprintf (stderr, "Error in opening output file.\n") ;
      exit(-1) ;
      }
   fprintf (stdout, "Output data file: %s\n", namdat);
   fprintf (stdout, "Output metacode file: %s\n", namemc) ;
   fprintf (datp, "******* %s Transfer Curve Fits *******\n\n", argv [0]);
   fprintf (datp, "exposure = alpha * [(DN - gamma) / beta]\n\n");

   /*** Initialize WHIP frame buffer. ***/

   if (DSP == 1)
      {
      printf ("Enter Frame Buffer [0-35] :");
      scanf  ("%d", &FB);
      printf ("Enter overlay color [r,g,b,w] : ");
      scanf  ("%1s", ovrcolor);
      if (strncmp ("r", ovrcolor, 1) == 0)  ovrcol = cp_red;
      if (strncmp ("g", ovrcolor, 1) == 0)  ovrcol = cp_green;
      if (strncmp ("b", ovrcolor, 1) == 0)  ovrcol = cp_blue;
      if (strncmp ("w", ovrcolor, 1) == 0)  ovrcol = cp_white;

      /*** Initialize WHIP software. ***/

      fb_open ();
      openpl ();
      fb_setfb (FB);
      fb_clear (FB, &bkgcol, fast);
      }

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open list file: %s\n", namlst);
      exit (-1);
      }

   /*** Read list of images; process one image at a time. ***/

   imgnum = -1;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open image file: %s", namimg);
         exit (-6);
         }
      imgnum++;

      if (DSP)
         cp_view (FB, namimg, 1);		/* Write image to FB. */
      cphskp (namimg);
      fprintf (datp, "Year: 19%2d   DOY: %03d   Time: %02d:%02d   Exposure: %d\n",
               H.year, H.doy, H.hour, H.minute, H.exp);

      expose [imgnum] = (float) H.exp;

      /*** Box loop : ***/

      for (i = 0; i < nbox; i++)
         {
         x = xtab [i];
         y = ytab [i];
         ibox = i + 1;
         boxval (namimg, x, y, nave, lsv, box, ibox);

         /*** Average value of box : ***/

         sum = 0.0;
         maxpix = nave*nave;
         for (p = 0; p < maxpix; p++)
            {
            sum += (float) box [p];
            }
         ave [i][imgnum] = sum / (float) maxpix;
/* 
 *         fprintf (stdout, "box: %d  x: %d  y: %d  size: %d  ave: %f\n",
 *                  i, x, y, nave, ave [i][imgnum]);
 */
	}
      fclose (IMG);
      }
   fprintf (datp, "\n");

   fix [0] = 0;
   fix [1] = 0;
   fix [2] = 0;
   nimg = imgnum + 1;

/*   fprintf (stdout, "nimg: %d   nbox:%d\n", nimg, nbox); */

   for (i = 0; i < nbox; i++)
      {
      for (ii = 0; ii < nimg; ii++)
         {
         dn [ii] = ave [i][ii];
         if ((ii > 0 && dn [ii] <= dn [ii-1])
            || dn [ii] > 248.0)
            {
            nval = ii;
            break;
            }
         nval = ii + 1;
         }
      fprintf (stdout, "box:%3d   nval:%3d\n", i+1, nval);

   /*** Initial values for non-linear least-squares fit. ***/

      par [0] = alphain;
      par [1] = betain;
      par [2] = gammain;

   /*** Non-linear least squares fit. ***/

      nllst2_ (dn, expose, par, sig, fix, &npar, &nval, &chisq, &ierr);
/*
 *   subroutine nllst2(y,x,c,sig,fix,na,m,chisq,ier)
 *
 *      x         = array of independent quantities
 *      y         = array of dependent quantities
 *      c         = estimates of the  fitting parameters
 *      fix       - logical array
 *                  if fix(i) = .true. then c(i) is not corrected
 *      na        = number of parameters in fit (including those fixed)
 *      m         = number of data points
 *
 *   User supplied subroutines :
 *      func(x,c,na)        func = value at x given parameters c
 *      deriv(x,c,na,f,df)  f = value at x given parameters c
 *                          df(i) = df/dc(i)
 *
 *   output :
 *      c         = least squares fitted parameters
 *      sig       = estimate of standard error in c.
 *      chisq     = estimate of unit weight variance of fit
 *      ier       = 1 - - > fit is ok
 *                = 2 - - > fit is suspicious
 *                = 3 - - > more than 25 iterations required
 *                = 4 - - > disaster, no fit found
 */
      alpha  = par [0];
      beta   = par [1];
      gamma  = par [2];

/***/
      fprintf (datp, "nval: %d\n", nval);
      fprintf (datp, "alpha:%f  beta:%f  gamma:%f\n", alpha, beta, gamma);
      fprintf (datp, "sig: %f   %f   %f\n", sig [0], sig [1], sig [2]);
      fprintf (datp, "fix: %o   %o   %o\n", fix [0], fix [1], fix [2]);
      fprintf (datp, "chisq: %g   ierr:%d\n", chisq, ierr);
      fprintf (datp, "\n");
/***/
      sprintf (ltitle, "%s CM 19%d:%d Box#%d: (%d,%d) %dx%d$",
		  argv [0], H.year, H.doy, i+1, xtab[i], ytab[i], nave, nave);
      sprintf (labx, "E x p o s u r e (*1.024 sec)$");
      sprintf (laby, "Vidicon Digital Number (DN)$");

      tfplot1 (&ave [i][0], expose, nimg, namemc, ltitle, labx, laby,
               ixscale, iyscale, ibkg, par, npar, nval);

      fprintf (datp, "BOX   Exp.Time    DN     fit(DN)\n\n") ;
      for ( j = 0 ; j < nimg ; j++ )
         {
         yfit [j] = beta * (float) atan2 ((double) expose [j], (double) alpha)
		+ gamma;
         fprintf (datp, "%2d    %6.3f     %5.1f   %5.1f\n",
		i+1, expose [j], ave [i][j],  yfit [j]);
         }

      fprintf (datp,"\n") ;
      }

   closepl ();
   fb_close ();
   fclose (datp) ;
   sprintf (gksout, "NCARG_GKS_OUTPUT=gks.cgm");
   putenv (gksout);
   clsgks_ ();
   exit (0);
}
