/* tfplot (x, y, nval, namfil, title, labx, laby, ixscale, iyscale, ibkg,
           alpha, beta, gamma)
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   19 March 1987
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namfil : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   alpha, beta, gamma : tangent equation parameters for transfer curve fit.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <math.h>
extern int pcount;

tfplot (x, y, nval, namfil, title, labx, laby, ixscale, iyscale, ibkg,
         alpha, beta, gamma)

float	x [], y [];
int	nval;
char	namfil [];
char	title [40], labx [40], laby [40];
int	ixscale, iyscale;
int	ibkg;
float	alpha;
float	beta;
float	gamma;

{
extern char *malloc();
static int iset  = 0;
static int ndash = 0;
static int ldash = 0;
static int ichar = '*' ;

int i;
static int nfit = 64;
static float xmm [2], ymm [2] ;
static float expose [65], dnfit [65];
float exp;

float f1  =  1.0;
float fm1 = -1.0;
float f2  =  2.0;
float fzero = 0.0;

   xmm [0] = 0.0 ;
   xmm [1] = 32.0 ;
   ymm [0] = 0.0 ;
   ymm [1] = 256.0 ;

   pcount ++;
   anotat_ (&labx, &laby, &f1, &fm1, fzero, fzero, strlen (labx), strlen (laby));
   displa_ (&f2, &fzero, &f1);
   ezxy_ (xmm, ymm, &f2, title, strlen (title));
   points_ (x, y, &nval, &ichar, 0) ;

   /* Compute fitted function curve. */

   exp = 0.0;
   for (i = 0; i < nfit; i++)
      {
      exp += 0.5;
      expose [i] = exp;
      dnfit [i] = gamma +
         beta * (float) atan2 ((double) expose[i], (double) alpha);
      }
   curve_ (expose, dnfit, &nfit);
   
   frame_ () ;

   return (0);
}
