/* tfplot1.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nexp : # values of x = # values of y.
 *   namfil : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   param: parameters for polynomial equation.
 *   npar: # parameters in polynomial equation.
 *   nuse: # points used in fit.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 22 March 1987
 * 12 November 1992: SGI/IRIS version.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <math.h>
extern int pcount;

tfplot1 (x, y, nexp, namfil, title, labx, laby, ixscale, iyscale, ibkg,
	 param, npar, nuse)

float	x [], y [];
int	nexp;					/* # total points. */
char	namfil [];
char	title [40], labx [40], laby [40];
int	ixscale, iyscale;
int	ibkg;
float	param [];
int	npar;
int	nuse;					/* # points used in fit. */

{
extern char *malloc ();
extern float fpow ();

static char agparam [40];
static char buf1 [84], buf2 [84];
static char dashch [4];
static char namsav [80] = "pfplot5.cgm";

int i, ip;
int iiex = 1, iiey = 1;
int iivx = 1, iivy = 1;
int ival;
int istar= 42;		/* ASCII character '*'. */
int iplus = 43;		/* ASCII character '+'. */
int iO = 79;		/* ASCII character 'O'. */
int io = 111;		/* ASCII character 'o'. */
int kdash = 0;
int lback;
int lenparam;
int lenx, leny;
int lendash;
int lset;
int ndash;
int nfit = 64;
int npts;
int nvix = 1, nviy = 1;
int nxtra;
int pval;
int one = 1;
int two = 2;
int zero = 0;

static float xmm [2], ymm [2] ;
static float dn [65], expfit [65];
static float dnfit [65], expose [65];
float d;
float fit;
float expval;
float alpha, beta, gamma;

   npts = nuse;
   nxtra = nexp - nuse;
   xmm [0] = 0.0 ;
   xmm [1] = 256.0 ;
   ymm [0] = 0.0 ;
   ymm [1] = 32.0 ;

/*   if (strcmp (namsav, namfil) != 0)  gkschnam_ (namfil); */

   strcpy (agparam, "FRAME.");
   lenparam = strlen (agparam);
   pval = 2;
   agseti_ (agparam, &pval, &lenparam);		/* Suppress frame advance. */

   strcpy (agparam, "SET.");
   lenparam = strlen (agparam);
   pval = -1;
   agseti_ (agparam, &pval, &lenparam);    /* Suppress curve drawing by EZXY. */

   agstup_ (ymm, &nviy, &iivy, &two, &iiey, xmm, &nvix, &iivx, &two, &iiex);

   /*** Title ***/

   strcpy (buf1, "LABEL/NAME.");
   strcpy (buf2, "T");
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   strcpy (buf1, "LINE/MAXIMUM.");
   ival = 60;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/NUMBER.");
   ival = 100;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/TEXT.");
   sprintf (buf2, "%s$", title);
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   /*** X-axis label. ***/

   strcpy (buf1, "LABEL/NAME.");
   strcpy (buf2, "B");
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   strcpy (buf1, "LINE/NUMBER.");
   ival = -100;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/TEXT.");
   sprintf (buf2, "%s$", labx);
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   /*** Y-axis label. ***/

   strcpy (buf1, "LABEL/NAME.");
   strcpy (buf2, "L");
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   strcpy (buf1, "LINE/NUMBER.");
   ival = 100;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/TEXT.");
   sprintf (buf2, "%s$", laby);
   agsetc_ (buf1, buf2);

/*   agback_ (); */

   lenparam = strlen (title);
   ezxy_ (ymm, xmm, &two, title, &lenparam);


   /*** Compute fitted function curve. ***/

/*
 *   d = -4.0;
 *   for (i=0; i<=nfit; i++)
 *      {
 *      d += 4.0;
 *      dn [i] = d;
 *      fit = 0.0;
 *      for (ip = 0; ip < npar; ip++)
 *         fit += param [ip] * fpow (d, ip);
 *      expfit [i] = fit;
 *      }
 *
 *   points_ (y, x, &npts, &istar, &zero) ;
 *   points_ (&y [npts], &x [npts], &nxtra, &iO, &zero) ;
 *
 *   agcurv_ (expfit, &iiex, dn, &iiey, &nfit, &kdash);
 */

   /* Compute fitted function curve. */

   expval = -0.4;
   alpha = param [0];
   beta  = param [1];
   gamma = param [2];

   for (i = 0; i < nfit; i++)
      {
      expval += 0.5;
      expose [i] = expval;
      dnfit [i] = gamma +
         beta * (float) atan2 ((double) expose[i], (double) alpha);
      }

   points_ (y, x, &npts, &istar, &zero) ;
   points_ (&y [npts], &x [npts], &nxtra, &iO, &zero) ;

   agcurv_ (expose, &iiex, dnfit, &iiey, &nfit, &kdash);
   
   frame_ () ;

   strcpy (namsav, namfil);
   return (0);
}


/* -------------------------------------------------------------------------- */
float
fpow (x, n)

float x;
int n;

{
float xp;
int i;

   if (n < 0)  return (-1.);
   xp = 1.0;
   for (i = 1; i<=n; i++)  xp *= x;
   return (xp);
}
