/* cflo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This program will create curve for translating DN to radiance
 * using the given set of reference frames.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *    The input images are averaged.
 *    Both high and low resolution input images yield high resolution
 *    output arrays alpha, beta, gamma , and error.
 *
 *    Inputs are:
 *          [input data,alpha,beta,gamma ,error] volume names  5(4x,a6)
 *          number of input frames (i5)
 *          input file numbers to use (20i5)
 *          number of records and pixels (2i5)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *    avefit written by Tom Blaschko, > Jan - March 1980.
 *    calfit adapted from avefit by Andrew L. Stanger, 24 July 1981.
 *    calfit converted to VAX Fortran-77 by A. Stanger, 10 August 1983.
 *     3 September 1985: use cleaned-up calibration reticle images.
 *    23 March 1987: Remove running average feature; set npts.
 *    11 March 1988: Add 1 second image (011317092.vgr).
 *     8 September 1988: convert program from F77 to C language.
 *    14 September 1988: do not use exposure levels above 220 unless necessary.
 *     1 November 1988: use multiple-image averaged frames for each exposure.
 *     2 November 1988: use only images from DOY 75-130 of 1980.
 *    28 February 1989: generate low resolution output files from low resolution
 *                      input files.
 *    17 March 1989: Read input file containing extension for output files
 *                   and names of input images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define nf 15 
#define nplo 448
#define numpar 4

main()
{
static float time [nf];
static float a0 [nplo], a1 [nplo], a2 [nplo], a3 [nplo];
static float pixval [nf], par[numpar];
static float sigmay [nf];
static int line [nf][nplo];
static int lun [nf];
static int idata [nplo];
static unsigned char bbuf [nplo];
static char namimg [nf][14];
static char nama0 [20];
static char nama1 [20];
static char nama2 [20];
static char nama3 [20];
static char namext [20];
static char namfil [80];
static char namio [80];

int i;
int idat1, idat2, itmp1, itmp2, itime, ires, jres;
int ifile;
int iorec;
int ip;
int iprev, ipodd, ipeven;
int irec;
int lenave = 16;
int luna0, luna1, luna2, luna3;
int mode = 0;
int nclose;
int nimg = nf;
int npar = numpar;
long noff;
int npix = 448;
int npout;
int npts;
int nread;
int nrec;
int nseek;
int nwrite;
float a0in = 1.0;
float a1in = 1.0;
float a2in = 1.0;
float a3in = 1.0;
float a0max = 100.0;
float a1max = 100.0;
float a2max = 100.0;
float a3max = 100.0;
float a0min = -100.0;
float a1min = -100.0;
float a2min = -100.0;
float a3min = -100.0;
float a0sav, a1sav, a2sav, a3sav;
float chisqr;
float fval;
FILE *IO;
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   /*** Request name of file specifying output & input names. ***/

   fprintf (stderr, "Enter name of file containing output/input file names: ");
   fscanf (stdin, "%s", namio);
   if ((IO = fopen (namio, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %s\n", namio);
      exit (-1);
      }

   i = 0;
   fscanf (IO, "%s", namext);
   while (fscanf (IO, "%s", namfil) != EOF)
      {
      printf ("namfil: %s\n", namfil);
      nimg = i + 1;
      if (nimg > nf)  break;
      strcpy (&namimg[i][0], namfil);
      i++;
      }
   fclose (IO);
   printf ("namext: %s\n", namext);

/*** Initialization. ***/

   a0sav = a0in;
   a1sav = a1in;
   a2sav = a2in;
   a3sav = a3in;
   iorec = 0;

   printf ("*** SMM Coronagraph/Polarimeter ***\n");
   printf ("photometric transfer curve fitting\n");

   /* Open calibration mirror image files.
    *
    *  namimg : names of input images.
    *  nimg  : # images to process.
    */

   sprintf (nama0, "a0.%s", namext);
   sprintf (nama1, "a1.%s", namext);
   sprintf (nama2, "a2.%s", namext);
   sprintf (nama3, "a3.%s", namext);
   fprintf (stdout, "parameter files: %s  %s  %s  %s\n",
	    nama0, nama1, nama2, nama3);

   luna0 = creat (nama0, 0644);
   luna1 = creat (nama1, 0644);
   luna2 = creat (nama2, 0644);
   luna3 = creat (nama3, 0644);

   /*  npix : # pixels per line (record).
    *  nrec : # records per image (file).
    */
   nrec = npix;

   /*
    * Read housekeeping record.
    * Extract resolution and exposure time.
    */

   for (ifile = 0; ifile < nimg; ifile++)
      {
      lun[ifile] = open (&namimg[ifile][0], 0);
      printf ("lun: %d   namimg: %s\n", lun[ifile], &namimg[ifile][0]);
      if (lun[ifile] < 0)  exit (-1);
      nread = read (lun[ifile], bbuf, npix);
      if (nread <= 0)  printf ("hskp read error...nread: %d\n", nread);

      /*** Copy byte data into integer array. ***/

      for (i=0; i<npix; i++)  idata[i] = ((int) bbuf[i] & 0377);

      idat1 = (int) idata[10];
      idat2 = (int) idata[11];
      itmp1 = (idat1 & 0377) * 16;
      itmp2 = (idat2 & 0360) / 16;
      itime = (itmp1 | itmp2);
      idat1 = (int) idata[3];
      ires = (idat1 & 4) / 4;
      time[ifile] = itime;
      printf ("exposure: %d   ires: %d\n", itime, ires);

      npout = npix * sizeof(int); 
      nread = read (lun[ifile], bbuf, npix);	/* Read bit-map record. */
      } /* End of image loop. */



   /* >   >   >   >   >   >   >   >   >   >   >   >   >   >   >   >  >  >  >  >
    *
    *    Do the transfer curves one line at a time, storing the data into
    *    the big array named 'line'.
    */

   for (irec=0; irec<nrec; irec++)
      {
         for (ifile = 0; ifile<nimg; ifile++)
	    {
            nread = read (lun[ifile], bbuf, npix); /* Read record from image. */

            if (nread <= 0)
	       {
	       printf ("read error...nread: %d\n", nread);
	       exit (-1);
	       }

            for (ip=0; ip<npix; ip++)
	       {
               line [ifile][ip] = ((int) bbuf[ip] & 0377);
	       }
	 } /* End of image loop. */



      /*** - - - - - - Do curve-fitting pixel by pixel. - - - - - - - ***/

      for (ip=0; ip<npix; ip++)
         {
         for (ifile=0; ifile<nimg; ifile++)
	    {
            fval = (float) line[ifile][ip];
            if (ifile > 0 && fval < (pixval[ifile-1] + 3))  break;
	    if (npts > 4 && line[ifile][ip] > 220)  break;
            pixval[ifile] = fval;
            npts = ifile + 1;
	    } /* End of image loop. */

	 if (ip == irec)
	   printf ("(%03d,%03d) %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f npts: %d\n",
                   ip, irec,
                   pixval[0], pixval[1], pixval[2], pixval[3], pixval[4],
                   pixval[5], pixval[6], pixval[7], pixval[8], npts);

         if (npts > 4)	/* Do curve fit if > 4 points are used. */
	    {
            par[0] = a0in;
            par[1] = a1in;
            par[2] = a2in;
            par[3] = a3in;

            for (ifile=0; ifile<npts; ifile++)  sigmay[ifile] = 0.0;
	    pfit_ (pixval, time, sigmay, &npts, &npar, &mode, par, &chisqr);

            /*** - - - Store the parameters. ***/

            a0[ip] = par[0];
            a1[ip] = par[1];
            a2[ip] = par[2];
            a3[ip] = par[3];

            if (ip == irec)
	       {
               printf ("(%03d,%03d) a0:%9.5f  a1:%9.5f  a2:%9.5f  a3:%9.6f  chisqr:%7.4f\n",
               ip, irec, a0[ip], a1[ip], a2[ip], a3[ip], chisqr);
	       }

            /*** If parameter fit is ok, then save a0, a1, a2, & a3 values. ***/

            if ( (a0 [ip] < a0min) || (a0 [ip] > a0max)
              || (a1 [ip] < a1min) || (a1 [ip] > a1max)
	      || (a2 [ip] < a2min) || (a2 [ip] > a2max)
              || (a3 [ip] < a3min) || (a3 [ip] > a3max))
		 {

                 /*** If parameter fit is bad, use last good  parameters. ***/

                 a0[ip] = a0sav;
	         a1[ip] = a1sav;
	         a2[ip] = a2sav;
	         a3[ip] = a3sav;
		 }
	    }

	    a0sav = a0[ip];
	    a1sav = a1[ip];
	    a2sav = a2[ip];
	    a3sav = a3[ip];

         } /* End of pixel loop. */


      /*** - - - Now store the parameter data. - - - ***/

      nwrite = write (luna0, a0, npout);
      nwrite = write (luna1, a1, npout);
      nwrite = write (luna2, a2, npout);
      nwrite = write (luna3, a3, npout);
      iorec = iorec + 1;
      }

   /* <   <   <   <   <   <   <   <   <   <   <   <   <   <   <   <   <  <  < */

   nclose = close (luna0);
   nclose = close (luna1);
   nclose = close (luna2);
   nclose = close (luna3);
   for (ifile = 0; ifile < nimg; ifile++)  nclose = close (lun[ifile]);


   exit(0);
}
