/* cftan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This program will create curve for translating DN to radiance
 * using the given set of reference frames.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *    The input images are averaged.
 *    Both high and low resolution input images yield high resolution
 *    output arrays alpha, beta, gamma , and error.
 *
 *    Inputs are:
 *          [input data,alpha,beta,gamma ,error] volume names  5(4x,a6)
 *          number of input frames (i5)
 *          input file numbers to use (20i5)
 *          number of records and pixels (2i5)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *    avefit written by Tom Blaschko, > Jan - March 1980.
 *    calfit adapted from avefit by Andrew L. Stanger, 24 July 1981.
 *    calfit converted to VAX Fortran-77 by A. Stanger, 10 August 1983.
 *     3 September 1985: use cleaned-up calibration reticle images.
 *    23 March 1987: Remove running average feature; set npts.
 *    11 March 1988: Add 1 second image (011317092.vgr).
 *     8 September 1988: convert program from F77 to C language.
 *    14 September 1988: do not use exposure levels above 220 unless necessary.
 *     1 November 1988: use multiple-image averaged frames for each exposure.
 *     2 November 1988: use only images from DOY 75-130 of 1980.
 *    28 February 1989: generate low resolution output files from low resolution
 *                      input files.
 *    17 March 1989: Read input file containing extension for output files
 *                   and names of input images.
 *    26 March 1991: adapt for SGI IRIS.
 *    12 February 1993: apply luminance correction for earth-sun distance.
 *    26 March 1993: use tangent equation for fit.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.h"
#define MAXPAR 3
#define nf 15 
#define nplo 448
#define numpar 3

main (argc, argv)

int argc;
char *argv [];
{
   float lumadj ();

   static unsigned char bbuf [nplo];
   static char namimg  [nf][80];
   static char nama0   [80];
   static char nama1   [80];
   static char nama2   [80];
   static char namext  [80];
   static char namfil  [80];
   static char namio   [80];

   static int fix [MAXPAR];
   static int line [nf][nplo];
   static int lun [nf];
   static int idata [nplo];

   static float time [nf];
   static float a0 [nplo], a1 [nplo], a2 [nplo];
   static float pixval [nf], par[numpar];
   static float sigmay [nf];
   static float sig [MAXPAR*2];

   int i;
   int idat1, idat2, itmp1, itmp2, itime, ires, jres;
   int ierr;
   int ifile;
   int iorec;
   int ip;
   int iprev, ipodd, ipeven;
   int irec;
   int lenave = 16;
   int luna0, luna1, luna2;
   int mode = 0;
   int nclose;
   int nimg = nf;
   int npar = numpar;
   long noff;
   int npix = 448;
   int npout;
   int npts;
   int nread;
   int nrec;
   int nseek;
   int nwrite;

   float a0in = 24.0;
   float a1in = 177.0;
   float a2in = 5.0;
   float a0max =  50.0;
   float a1max = 300.0;
   float a2max = 255.0;
   float a0min =   0.0;
   float a1min =   0.0;
   float a2min = -25.0;
   float a0sav, a1sav, a2sav;
   float errlim = 0.25;
   float chisqr = 0.0;
   float fval;

   float brtcor [nf];	/* Brightness correction for earth-sun distance. */

   FILE *IO;
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   /*** Request name of file specifying output & input names. ***/

   if (argc == 1)
      {
      fprintf (stderr, "Enter name of file containing output/input file names: ");
      fscanf (stdin, "%s", namio);
      }

   else
   if (argc == 2)
      strcpy (namio, argv [1]);
   else
      {
      fprintf (stderr, "Usage: %s filelist_name\n", argv [0]);
      exit (-1);
      }

   if ((IO = fopen (namio, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file %s\n", namio);
      exit (-1);
      }

   i = 0;
   fscanf (IO, "%s", namext);
   while (fscanf (IO, "%s", namfil) != EOF)
      {
      fprintf (stdout, "namfil: %s\n", namfil);
      nimg = i + 1;
      if (nimg > nf)  break;
      strcpy (&namimg [i][0], namfil);
      i++;
      }
   fclose (IO);
   fprintf (stdout, "namext: %s\n", namext);

/*** Initialization. ***/

   a0sav = a0in;
   a1sav = a1in;
   a2sav = a2in;
   iorec = 0;
   fix [0] = 0;
   fix [1] = 0;
   fix [2] = 0;

   fprintf (stdout, "*** SMM Coronagraph/Polarimeter ***\n");
   fprintf (stdout, "photometric transfer curve fitting\n");

   /* Open calibration mirror image files.
    *
    *  namimg : names of input images.
    *  nimg  : # images to process.
    */

   sprintf (nama0, "a0.%s", namext);
   sprintf (nama1, "a1.%s", namext);
   sprintf (nama2, "a2.%s", namext);
   fprintf (stdout, "parameter files: %s  %s  %s\n",
	    nama0, nama1, nama2);

   luna0 = creat (nama0, 0644);
   luna1 = creat (nama1, 0644);
   luna2 = creat (nama2, 0644);

   /*  npix : # pixels per line (record).
    *  nrec : # records per image (file).
    */
   nrec = npix;

   /*
    * Read housekeeping record.
    * Extract resolution and exposure time.
    */

   for (ifile = 0; ifile < nimg; ifile++)
      {
      cphskp (&namimg [ifile][0]);
      brtcor [ifile] = lumadj (H.year, H.doy, H.hour);/* Luminance adjustment.*/
      lun [ifile] = open (&namimg [ifile][0], 0);
      fprintf (stdout, "lun: %d   namimg: %s\n",
	       lun [ifile], &namimg [ifile][0]);
      if (lun [ifile] < 0)  exit (-1);
      nread = read (lun [ifile], bbuf, npix);
      if (nread <= 0)  fprintf (stdout, "hskp read error...nread: %d\n", nread);


      /*** Copy byte data into integer array. ***/

      for (i = 0; i < npix; i++)  idata [i] = ((int) bbuf [i] & 0377);

      idat1 = (int) idata [10];
      idat2 = (int) idata [11];
      itmp1 = (idat1 & 0377) * 16;
      itmp2 = (idat2 & 0360) / 16;
      itime = (itmp1 | itmp2);
      idat1 = (int) idata [3];
      ires = (idat1 & 4) / 4;
      time [ifile] = itime;
      fprintf (stdout, "exposure: %d   ires: %d\n", itime, ires);

      npout = npix * sizeof  (int); 
      nread = read (lun [ifile], bbuf, npix);	/* Read bit-map record. */
      } /* End of image loop. */



   /* >   >   >   >   >   >   >   >   >   >   >   >   >   >   >   >  >  >  >  >
    *
    *    Do the transfer curves one line at a time, storing the data into
    *    the big array named 'line'.
    */

   for (irec = 0; irec < nrec; irec++)
      {
         for (ifile = 0; ifile < nimg; ifile++)
	    {
            nread = read (lun [ifile], bbuf, npix);/* Read record from image. */

            if (nread <= 0)
	       {
	       fprintf (stdout, "read error...nread: %d\n", nread);
	       exit (-1);
	       }

            for (ip = 0; ip < npix; ip++)
	       {
               line [ifile][ip] = ((int) bbuf [ip] & 0377);
	       }
	 } /* End of image loop. */



      /*** - - - - - - Do curve-fitting pixel by pixel. - - - - - - - ***/

      for (ip = 0; ip < npix; ip++)
         {
         for (ifile = 0; ifile < nimg; ifile++)
	    {
            fval = (float) line [ifile][ip] * brtcor [ifile];
            if (ifile > 0 && fval < (pixval [ifile-1] + 3))  break;
	    if (npts > 5 && line  [ifile][ip] > 250)  break;
            pixval [ifile] = fval;
            npts = ifile + 1;
	    } /* End of image loop. */

/***	 if (ip == irec)
	   fprintf (stdout, "(%03d,%03d) %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f %3.0f npts: %d\n",
                   ip, irec,
                   pixval[0], pixval[1], pixval[2], pixval[3], pixval[4],
                   pixval[5], pixval[6], pixval[7], pixval[8], npts);
***/

         if (npts > 5)	/* Do curve fit if > 5 points are used. */
	    {
            par [0] = a0in;
            par [1] = a1in;
            par [2] = a2in;

            for (ifile = 0; ifile < npts; ifile++)  sigmay [ifile] = 0.0;

            nllst2_ (pixval, time, par, sig, fix, &npar, &npts, &chisqr, &ierr);

            /*** Check to make sure the chi-square is less than max limit. ***/
            
            while (chisqr > errlim)
               {
               if (npts-- < 5)
                  break;
               nllst2_ (pixval, time, par, sig, fix, &npar, &npts, &chisqr, &ierr);
               }

/*
 *         if ((ip == 276 && irec == 90) ||
 *            (ip == 276 && irec == 91) ||
 *            (ip == 276 && irec == 92))
 *
 *            printf ("(ip = %d,irec = %d)  npts:%d  chisqr:%5.3f  a0:%g  a1:%g  a2:%g\n",
 *               ip, irec, npts, chisqr, par [0], par [1], par [2]);
 *
 */
            /*** - - - Store the parameters. ***/

            a0 [ip] = par [0];
            a1 [ip] = par [1];
            a2 [ip] = par [2];

/***
 *         if (ip == irec)
 *            {
 *            fprintf (stdout, "(%03d,%03d) a0:%9.5f  a1:%9.5f  a2:%9.5f  chisqr:%7.4f\n",
 *            ip, irec, a0 [ip], a1 [ip], a2 [ip], chisqr);
 *            }
***/

            /*** If parameter fit is ok, then save a0, a1, & a2 values. ***/
            /*** If parameter fit is bad, use last good  parameters. ***/

            if ( (a0 [ip] < a0min) || (a0 [ip] > a0max)
              || (a1 [ip] < a1min) || (a1 [ip] > a1max)
	      || (a2 [ip] < a2min) || (a2 [ip] > a2max))
		 {
                 a0 [ip] = a0sav;
	         a1 [ip] = a1sav;
	         a2 [ip] = a2sav;
		 }
	    }

	    a0sav = a0 [ip];
	    a1sav = a1 [ip];
	    a2sav = a2 [ip];

         } /* End of pixel loop. */


      /*** - - - Now store the parameter data. - - - ***/

      nwrite = write (luna0, a0, npout);
      nwrite = write (luna1, a1, npout);
      nwrite = write (luna2, a2, npout);
      iorec = iorec + 1;
      }

   /* <   <   <   <   <   <   <   <   <   <   <   <   <   <   <   <   <  <  < */

   nclose = close (luna0);
   nclose = close (luna1);
   nclose = close (luna2);
   for (ifile = 0; ifile < nimg; ifile++)  nclose = close (lun [ifile]);

   exit (0);
}
