/* esdist.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Earth-sun distance.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Returns earth-sun distance squared in AU units.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

float
esdist (cpyear, doy)

int cpyear;		/* Year. */
int doy;		/* Day-of-year. */

{
   float DOYP = 3.0;		/* Day-of-year corresponding to perihelion. */
   float R2;			/* Earth-Sun distance squared. */
   float SMA = 1.0;		/* Semi-major axis of Earth-Sun orbit (1 AU). */
   float ecc  = 0.017;		/* Eccentricity of Earth's orbit about Sun. */
   float esd;			/* Earth-Sun distance. */
   float raddeg;		/* radians/degree. */
   float theta;			/* Angle of earth-sun line w.r.t. perihelion. */

   int DOYP [10] = {};	/* Perihelion DOY for years 1980-89. */
   int ndoyyr;
   int iyear;
   int year;

   year = cpyear;
   if (year > 1900)  year -= 1900;
   iyear = year - 80;

   raddeg = (float) acos (-1.0) / 180.0;

   /*** Compute luminosity correction (R2 = Earth-Sun distance squared). ***/
   /*** orbital distance is computed in AU units. ***/

   if (year/4 * 4 == year)  ndoyyr = 366;  else ndoyyr = 365;

   theta = ((doy - DOYP [iyear]) / (float) ndoyyr) * 360.0;

   esd = SMA * (1.0 - ecc * ecc) /
               (1.0 + ecc * cos ((double) (raddeg * theta)));

   R2 = esd * esd;

   fprintf (corp, "%s  %7.2f  %7.2f  %9.4f  %9.4f\n",
            namimg, imgndx [imgnum], theta, esd, R2);

   return (R2);
}
