/* lumadj.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Correction for solar luminance due to Earth's elliptical orbit about the sun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 12 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h> 

float
lumadj (iyear, idoy, ihour)

int iyear;		/* Year. */
int idoy;		/* Day of year. */
int ihour;		/* Hour. */

{
   int iy;

   double A = 1.0;		/* Semi-major axis of ellipse (AU units). */
   double ecc = 0.01675;	/* Eccentricity of Earth's orbit. */
   double period = 365.2596;	/* Earth's orbital period. */
   double PI = 3.14159265;	/* PI. */
   float  r2;
   double radius;		/* Orbital radius (earth to sun). */
   double theta;		/* Angle of Earth-sun line w.r.t. perihelion. */
   double doyo;			/* Day-of-year of observation. */
   double doyp;			/* Perihelion day-of-year. */

   /* Perihelion DOY & hour. */

   static float Pdoy [10] = {3, 2, 4, 2, 3, 3, 2, 4, 4, 1};
   static float Phour [10] = {15,  2, 11, 16, 22, 20,  5, 23,  0, 22};

   iy = iyear;
   if (iyear > 1900)  iy -= 1900;
   if (iyear >= 80)   iy -=   80;
   doyp = (double) (Pdoy [iy] + Phour [iy] / 24.0);

   doyo = (double) (idoy + ihour / 24.0);

   theta = (2.0 * PI * (doyo - doyp)) / period;	/* Angle w.r.t. perihelion. */

   radius = (A * (1.0 - ecc*ecc)) / (1.0 + ecc * cos (theta));

   /*** Since luminosity diminishes as 1/r^2, correction is r^2. ***/

   r2 = radius * radius;
   printf ("iy: %2d  doyo: %7.3f  theta: %7.3f  radius:%7.3f   r2: %7.3f\n",
	    iy, doyo, theta, radius, r2);
   return (r2);
}
