/* cpcen.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate C/P disk center by fitting an arc
 * to second diffraction ring (1.7 Rsun).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 31 July 1984
 * 25 March 1991: modify for use on SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>

/* Global variables. */

#include "/home/riker/stanger/lib/src/cp/cplib.h"
#include "/home/riker/stanger/lib/src/ip/grscale.h"
#define Red 1
#define Green 2
#define Blue 3
#define ESC 27

int GRN = 1;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

int    cpview();		/* Display image on Grinnell. */
char   *rdimg();		/* Read image from disk to memory. */
FILE   *LIST, *OUT;
char   *imgbuf;			/* Pointer to image array. */
static char flag [8];
static char inline [80];	/* Input line buffer. */
static char namimg [80];	/* Image file name. */
static char namlst [80];	/* Name of file containing image list. */
static char namout [80]";	/* Output file name. */
static char nampgm [80];	/* Program file name. */
static char answer [2];		/* Storage for y/n response. */
char key;

static short r [512], g [512], b [512];
short icol;
int index;
int i, ioff;
float offset, slope;

static float scan [MAXSCN];	/* Pointer to independent scan array. */
static float scandx [MAXSCN];	/* Pointer to dependent scan array. */
static char namsmf [8][3] = {"sw","ss","se","ee","ne","nn","nw","ww"};
static float tmin [8] = {135.,100., 45., 35.,315.,285.,230.,210.};
static float tmax [8] = {315.,260.,235.,155.,135., 85., 45.,330.};

float  thmin, thmax;	/* Theta limits (deg.). */
float  thinc = 5.0;	/* Theta increment (deg.). */
float  radius = 1.7;	/* Solar radius (Rsun) for 2nd diffraction ring. */
float  xxcen, yycen;	/* Hi resolution center coordinates. */
float  dx, dy;		/* Delta x & y. */

int ierr;
int it;			/* Theta index variable. */
int nval;		/* # values in scan. */
int out = 0;
int seq = 0;
int xl = 760;
int yl = 480;
int xu = 1271;
int yu = 991;

   strcpy (nampgm, argv [0]);
   sprintf (namout, "%s.log", nampgm);

   fprintf (stderr, "%s: locate disk center by visually fitting an arc\n",
            nampgm);
   fprintf (stderr, "         to the second diffraction ring.\n");
   fprintf (stderr, "Origin for SMM C/P image is the upper left corner.\n");
   fprintf (stderr, "Direction for +x is to the right;  +y is down.\n");


   if (argc == 1)		/* No arguments. */
      {
      fprintf (stderr, "usage: %s [image_filename | @filename_list] [outfile]\n", argv [0]);
      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);
      fprintf (stderr, "Enter output filename ['none' to omit]: ");
      fscanf  (stdin, "%s", namout);
      if (strncmp (namout, "none") == 0)
         out = 0;
      else
         out = 1;
      }
   else
   if (argc == 2)		/* 1 argument: must be image filename. */
      {
      strcpy (namimg, argv [1]);
      out = 0;
      }
   else
   if (argc == 3)		/* 2 arguments: 1st must be image filename. */
      {
      strcpy (namimg, argv [1]);		/* Input image file. */
      strcpy (namout, argv [2]);		/* Output file name. */
      out = 1;
      }

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

   if (strncmp (namout, "none") == 0)
      out = 0;
   else
      out = 1;
   if (out)
      if ((OUT = fopen (namout, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create output file: %s\n", namout);
         exit (-2);
         }

   igr = 1;
   igterm = 1;
   lsv = 0;
   lfc = 0;

   /*** Save current color map. ***/

/*
   for (i=0; i<256; i++)
      {
      index = i + 512;
      getmcolor (index, r[i], g[i], b[i]);
      }
 */

   /*** GL window initialization. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen (nampgm);
   gl_clear (BLACK);

   icol = 0;
   ioff = 50;
   for (i=0; i<ioff; i++)  mapcolor (i+512, icol, icol, icol);

   offset = (float) ioff;
   slope = 256.0 / (256.0-offset);
   for (i=ioff; i<256; i++)
      {
      index = 512 + i;
      icol = (short) ( ((float) i - offset) * slope);
      mapcolor (index, icol, icol, icol);
      }

   multimap ();
   gconfig ();
   setmap (2);

newimg:
   drawmode (NORMALDRAW);		/* Set overlay mode for drawing. */
   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }
   fprintf (stderr, "Displaying image %s\n", namimg);
   if ((ierr = cpwgl (namimg)) != 0)
      {
      fprintf (stderr, "Error (%d) in cpwfb.\n", ierr);
      exit (-3);
      }

   /*** Overlay set-up. ***/

   overlay (2);			/* Establish two bitplanes for overlay. */
   gconfig ();

   drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
   mapcolor (0,   0,   0,   0);	/* color 0 -> black. */
   mapcolor (1, 255,   0,   0);	/* color 0 -> red.   */
   mapcolor (2,   0, 255,   0);	/* color 0 -> green. */
   mapcolor (3,   0,   0, 255);	/* color 0 -> blue.  */
   gl_clear (0);

   /*** Open input image disk file and read housekeeping record. ***/

   fprintf (stderr, "Reading image %s into memory.\n", namimg);
   imgbuf = rdimg (namimg);		/* Read image into memory array. */
   fprintf (stdout,"     G M T          Q(%%)  IMG  IH-IV RES  EXP  FW     PW  5303  SM     CM  ROLL\n");
   hkline (stdout);
   xxcen = xcen * (H.ires+1);
   yycen = ycen * (H.ires+1);

/*
 * printf("ih,iv: %d %d\n", H.ih, H.iv);
 * printf("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
 * printf("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
 */

   /*** Theta scan limits. ***/

   thmin = tmin[H.ism] - (float) scroll;
   thmax = tmax[H.ism] - (float) scroll;
   printf ("ism: %d   thmin: %f   thmax: %f\n", H.ism, thmin, thmax);

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red);

next:
   fflush  (stdin);
   fprintf (stderr, "disk center . . . x: %6.2f   y: %6.2f\n", xxcen, yycen);
   fprintf (stderr, "%s> ", nampgm);
   fprintf(stderr, "Action: delta x,y (hi res) | n (next image) | q (quit) : ");
   fgets (inline, 80, stdin);
   key = inline [0];
   if (inline [0] == 'n')
      {
      if (out)
         {
         fprintf (OUT, "%s %3d %3d\n", namimg, (int) xxcen, (int) yycen);
         }
      goto newimg;
      }
   else
   if (inline [0] == 'q' || inline [0] == ESC)
      {
      if (out)
         {
         fprintf (OUT, "%s %3d %3d\n", namimg, (int) xxcen, (int) yycen);
         fclose (OUT);
         }
      fclose (LIST);
      goto quit;
      }
   else
   switch (key)
      {
      case '0':
      case '1':
      case '2':
      case '3':
      case '4':
      case '5':
      case '6':
      case '7':
      case '8':
      case '9':
      case '-':
      case '+':
         break;
      default:
         fprintf (stderr, "Invalid response.\n");
         goto next;
      }

   sscanf (inline, "%f,%f", &dx, &dy);
   xxcen += dx;
   yycen += dy;
   xcen = xxcen / (float) (H.ires+1);
   ycen = yycen / (float) (H.ires+1);
   gl_clear (0);

   /*** Perform theta scan. ***/

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red);

   goto next;

   /*** Relinquish storage for image array. ***/

quit:

   free (imgbuf);

   /*** Clean up overlay bitplanes. ***/
   
   drawmode (OVERDRAW);
   gl_clear (0);

   drawmode (NORMALDRAW);
   greset ();

   gconfig ();
   gexit();
   exit (0);
}


gl_clear (level)

Colorindex level;

{
   color (level);
   clear ();

   return (0);
}
