/* ringfit.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate C/P disk center by fitting an arc
 * to second diffraction ring (1.7 Rsun).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Opens a GL window for image display.
 * Writes a "dotted" arc using overlay planes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 31 July 1984
 * 18 March 1991: modify for use on SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>

/* Global variables. */

#include "/home/riker/stanger/lib/src/cp/cplib.h"
#include "/home/riker/stanger/lib/src/ip/grscale.h"
#define Red 1
#define Green 2
#define Blue 3

int GRN = 1;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

int    cpview();		/* Display image on Grinnell. */
char   *rdimg();		/* Read image from disk to memory. */
char   *imgbuf;			/* Pointer to image array. */
static char namimg [80];	/* Image file name. */
static char answer [2];		/* Storage for y/n response. */
static short r [512], g [512], b [512];
short icol;
int index;
int i, ioff;
float offset, slope;

static float scan [MAXSCN];	/* Pointer to independent scan array. */
static float scandx [MAXSCN];	/* Pointer to dependent scan array. */
static char namsmf [8][3] = {"sw","ss","se","ee","ne","nn","nw","ww"};
static float tmin [8] = {135.,100., 45., 35.,315.,285.,230.,210.};
static float tmax [8] = {315.,260.,235.,155.,135., 85., 45.,330.};
float  thmin, thmax;	/* Theta limits (deg.). */
float  thinc = 5.0;	/* Theta increment (deg.). */
float  radius = 1.7;	/* Solar radius (Rsun) for 2nd diffraction ring. */
float  xxcen, yycen;	/* Hi resolution center coordinates. */
float  dx, dy;		/* Delta x & y. */

int nval;		/* # values in scan. */
int it;			/* Theta index variable. */
int nfb = 6;		/* Frame buffer number. */
int xl = 760;
int yl = 480;
int xu = 1271;
int yu = 991;

   fprintf (stderr, "RINGFIT: locate disk center by visually fitting an arc\n");
   fprintf (stderr, "         to the second diffraction ring.\n");
   fprintf (stderr, "Origin for SMM C/P image is the upper left corner.\n");
   fprintf (stderr, "Direction for +x is to the right;  +y is down.\n");

   if (argc == 1)
      {
      fprintf (stderr, "usage: ringfit [image_filename]\n");
      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);
      }
   else
   if (argc == 2)
      {
      strcpy (namimg, argv [1]);
      }

   igr = 1;
   igterm = 1;
   lsv = 0;
   lfc = 0;

   /*** Save current color map. ***/

/*
   for (i=0; i<256; i++)
      {
      index = i + 512;
      getmcolor (index, r[i], g[i], b[i]);
      }
 */

   /*** GL window initialization. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen ("ringfit");
   color (BLACK);
   clear ();

   icol = 0;
   ioff = 50;
   for (i=0; i<ioff; i++)  mapcolor (i+512, icol, icol, icol);

   offset = (float) ioff;
   slope = 256.0 / (256.0-offset);
   for (i=ioff; i<256; i++)
      {
      index = 512 + i;
      icol = (short) ( ((float) i - offset) * slope);
      mapcolor (index, icol, icol, icol);
      }

   multimap ();
   gconfig ();
   setmap (2);

   fprintf (stderr, "Displaying image %s\n", namimg);
   cpwgl (namimg);
   sleep (5);

   /*** Overlay set-up. ***/

   overlay (2);			/* Establish two bitplanes for overlay. */
   gconfig ();

   drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
   mapcolor (0,   0,   0,   0);	/* color 0 -> black. */
   mapcolor (1, 255,   0,   0);	/* color 0 -> red.   */
   mapcolor (2,   0, 255,   0);	/* color 0 -> green. */
   mapcolor (3,   0,   0, 255);	/* color 0 -> blue.  */

   /*** Open input image disk file and read housekeeping record. ***/

   fprintf (stderr, "Reading image %s into memory.\n", namimg);
   imgbuf = rdimg (namimg);		/* Read image into memory array. */
   fprintf (stdout,"     G M T          Q(%%)  IMG  IH-IV RES  EXP  FW     PW  5303  SM     CM  ROLL\n");
   hkline (stdout);
   xxcen = xcen * (H.ires+1);
   yycen = ycen * (H.ires+1);

/*
   printf("ih,iv: %d %d\n", H.ih, H.iv);
   printf("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
   printf("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
 */

   /*** Theta scan limits. ***/

   thmin = tmin[H.ism] - (float) scroll;
   thmax = tmax[H.ism] - (float) scroll;
   printf ("ism: %d   thmin: %f   thmax: %f\n", H.ism, thmin, thmax);
/*
 *  printf("Enter solar radius (Rsun) for theta scan : ");
 *  scanf("%f", &radius);
 *  if (radius <= 0.) goto quit;
 *  printf("Enter solar position angle min, max, & increment (deg.): ");
 *  scanf("%f,%f,%f", &thmin, &thmax, &thinc);
 */

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red);

next:
   fprintf (stderr, "disk center . . . x: %6.2f   y: %6.2f\n", xxcen, yycen);
   fprintf (stderr, "Change disk center (y/n/e(xit)) ? ");
   fflush  (stdin);
   fscanf  (stdin,  "%s", answer);
   if (strncmp ("y", answer, 1) == 0)
      {
      fprintf(stderr, "Enter delta x,y (Hi res) : ");
      fscanf (stdin, "%f,%f", &dx, &dy);
      xxcen += dx;
      yycen += dy;
      xcen = xxcen / (float) (H.ires+1);
      ycen = yycen / (float) (H.ires+1);
      }
   if (strncmp ("e", answer, 1) == 0)  goto quit;
   color (0);
   clear ();

   /*** Perform theta scan. ***/

   gl_tscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, &nval,
             Red);

   goto next;

   /*** Relinquish storage for image array. ***/

quit:

   free (imgbuf);

   /*** Clean up overlay bitplanes. ***/
   
   color (0);
   clear (0);

   drawmode (NORMALDRAW);
   greset ();

   gconfig ();
   gexit();
   exit (0);
}
