/* tscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Theta Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 2 March 1984: Re-created on Vax-11/750; C language; UNIX 4.2 bsd.
 * 18 July 1984: Add stray light & vignetting removal capabilities.
 * 26 July 1984: Add calibration correction.  Add F-corona removal option.
 * 8 February 1985: Add average routine.
 * 6 December 1985: Apply fixcal to all calibrated images.
 * 16 December 1985: Replace 'coord' with 'rcoord'.
 * 17 November 1986: Replace 'nres' by 'gres' for Grinnell display.
 * 22 March 1988: remove Grinnell references for use on Sun-4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/src/cp/cplib.h"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

tscan (namimg, image, thmin, thmax, thinc, radius, scan, scandx, ns)

char	namimg [];	/* Image file name. */
char	*image;		/* Pointer to image array. */

float	thmin, thmax;	/* Solar position angle limits for theta scan. */
float	thinc;		/* Angular increment for theta scan. */
float	radius;		/* Radius (R/Rsun units). */
float	scan [];	/* Dependent axis scan array. */
float	scandx [];	/* Independent axis scan array. */

int	*ns;		/* Number of values in scan. */

{
extern int fixcal ();	/* Function which applies calibration correction. */
extern int remstr ();	/* Function which removes stray light. */
extern int remvig ();	/* Function which removes vignetting. */
extern int rmfc ();	/* Function which removes F-corona. */
extern float blint ();	/* Bilinear interpolation function. */
extern int rcoord ();	/* Coordinate transformation: polar <--> cartesian. */

float	x, y;
float	r, th;
float	tinc;
float	rbuf[1];
register float Bexp;	/* "Generic" scale factor. */
register int it;
int	ixg, iyg;
int	ierr;
int	nr;
int	nscan;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   /*** Set scale factor. ***/

   Bexp = pscale;

   /* Theta increment: set default to angle separating two adjacent pixels. */

   tinc = thinc;
   if (thinc <= 0.0)  tinc = degrad / (radius*pixrs);

   r = radius;

   /*** Clear scan arrays. ***/

   for (it = 0; it < MAXSCN; it++)  *(scan + it) = *(scandx+ it) = 0.0;
 
   /* ******* Theta scan loop ******* */
 
   if(thmax < thmin)  thmax += 360.0;

   *ns = 0;
   it = -1;
   for (th = thmin; th <= thmax; th += tinc)
     {
      it++;
 
   /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      if ((ierr = rcoord (&r, &th, &x, &y, 1, scroll)) != 0)  return (ierr);

   /*** Draw "X" to mark theta scan point on Grinnell display. ***/
 
      if (igr > 0)
         {
         ixg = (int) x/gres + 63;
         iyg = 510 - (int) y/gres;
         wvec (ixg,   iyg, ixg+1, iyg+1);
         wvec (ixg+1, iyg, ixg,   iyg+1);
         }

   /*** Perform bilinear interpolation to compute scan point from image. ***/

      *(scandx + it) = (float) th;

      *(scan + it) = (float) (blint (image, &x, &y) * Bexp);

   }
   *ns = it + 1;

   /*** Smooth data using 'nave' point average. ***/
 
   faverage (nave, scan, *ns);
 
/* Apply calibration correction. */
   /* Remove stray light & vignetting. */
   /* Remove F-corona. */
 
   rbuf [0] = r;
   nr = 1;
   nscan = *ns;
/*   if(idtype == 3)  fixcal (rbuf, nr, scan, nscan); */
   if (idtype == 3 && lsv == 1)
      {
      remstr (rbuf, nr, scan, nscan);
      remvig (rbuf, nr, scan, nscan);
      if (lfc == 1)  rmfc (rbuf, nr, scan, nscan, scandx, *ns);
      }

   /* Divide scan values by 5 for equivalent second data type.
    * (numbers in file are 5 * equivalent second units.)
    */
 
   if (idtype == 2)  for (it = 0; it < *ns; it++) scan[it] /= 5.0;
 
   if (igterm == 0)
      for (it=0; it<*ns; it++)
         fprintf (stdout, "r:%7.3f  th:%7.2f  scan:%9.2e\n",
		  r, scandx [it], scan [it]);
   return (0);
}
