/*-------------------------------------------------------------------------
 * 
 * Given a file containing a list of SMM calibrated images, this program
 * finds the center of each image and creates a new file containing the
 * image name and the proposed center of that image.
 * The basic premise is to "scan" out to a given bright point on the corona,
 * defined by BRLIMIT, and determine the number of pixels from the 'center'
 * of the image to this point.  The calculated radius is compared against
 * a given radius, RADIUS, the program determines a new 'center' based on
 * this comparison and the process repeats until the difference between
 * RADIUS and the calculated number of pixels is zero.  This happens in
 * three directions: x, y and z (the diagonal between x and y).  The z scan
 * is basically used on north images to combat the black line down the center.
 *
 *-------------------------------------------------------------------------
 * Ron Decker ASAP/CU  SMM C/P March 2, 1992
 * Initial creation.
 */

#include <stdio.h>
#include <math.h>
#include <string.h>
#include "center.h"
#include "/usr/people/smm/lib/include/cplib.h"

main(argc, argv)
int             argc;
char           *argv[];

{
    extern char    *rdimg();         /* read in the image */
    /*- The following scan variables are used to scan in a "piece" of
     *  of the images.  xscan in the x direction, y in the y direction,
     *  z between x and y. dxscan and dyscan are both experiment variables,
     *  not being used at present.
     */
    int             xscan[(SCANS * 2) + 1][RADELTA*2];
    int             yscan[(SCANS * 2) + 1][RADELTA*2];
    int             zscan[(SCANS * 2) + 1][RADELTA*2];
    int             dxscan[(SCANS * 2) + 1][RADELTA*2];
    int             dyscan[(SCANS * 2) + 1][RADELTA*2];
    char            quad1, quad2;	    /*- quads such as '.nn' */  
    char           *cursmm;                 /*- current smm file name */
    char           *iname, *indup, *outname;/*- names of the in and out files */
    short          *image;                  /*- contain the actual images */
    char	   *quadp;		    /*- point to quadrant in name */
    int             list;                   /*- is there a list of images? */
    int             x, y, z;                /*- used for debugging */
    int             xpix, ypix;             /*- pixels in the x & y directions*/
    int             zpix;                   /*- corrected # of pixels in z dir*/
    int             zrpix;                  /*- actual # of pixels in z dir */  
    int             xoff, yoff;             /*- more x and y this much */
    int             minradz;                /*- corrected radius for z */
    int             minrad, maxrad;         /*- real radius for x and y */
    int             xstart, ystart;         /*- initial x and y positions */
    int             loopcount;              /*- loop count variable */
    double          zdpix;                  /*- help to calculate zpix */
    FILE           *infile ,*outfile; /*- in and out file pointers */
    FILE           *file;                   /*- help in debugging, not used */

    if (argc < 2)
    {
	fprintf(stderr, "Usage: %s {smmfile.cxx | @filename_list}\n", argv[0]);
	exit(1);
    }

    /*-
     * Calculate min and max radius values.  Allocated space for reading
     * in the smm file name.
     */
    minrad = RADIUS - RADELTA;
    maxrad = RADIUS + RADELTA;
    cursmm = (char *) malloc(50);

    /*-
     * Determine if the given file contains file names or is an actual
     * image file itself.  If it contains file names, create a new file
     * to hold the output, be sure infile isn't empty and set list to 1.
     */
    if (argv[1][0] == '@')
    {
	argv[1]++;

	if ((iname = strdup(argv[1])) == NULL) {
	   fprintf(stderr, "%s: iname strdup failed\n", argv[0]);
	   exit(1);
        }

	if ((indup = strrchr(iname, '/')) == NULL) {
	   indup = iname;
        }

	if ((outname = strdup(indup)) == NULL) {
	   fprintf(stderr, "%s: outname strdup failed\n", argv[0]);
	   exit(1);
        }

	if ((indup = strcat("./", outname)) == NULL) {
	   fprintf(stderr, "%s: indup strcat failed\n", argv[0]);
	   exit(1);
        }

	if ((outname = strcat(indup, ".cen")) == NULL) {
	   fprintf(stderr, "%s: outname strcat failed\n", argv[0]);
	   exit(1);
        }

	if ((outfile = fopen(outname, "w")) == NULL) {
	   fprintf(stderr, "%s: Can't open output file\n", argv[0]);
	   exit(1);
        }

	if ((infile = fopen(iname, "r")) == NULL) {
	   fprintf(stderr, "%s: %s doesn't exist\n",argv[0], iname);
	   exit(1);
        }
	else if (fscanf(infile, "%s", cursmm) == EOF)
	{
	    fprintf(stderr, "%s: %s: empty file\n", argv[0], iname);
	    exit(1);
	}

/*	printf("cursmm is ***%s***\n", cursmm);*/
	list = 1;
    }
    else
    /*-
     * If the file is an smm image file, make the necessary arrangements.
     */
    {
	outfile = stdout;
	strcpy(cursmm, argv[1]);
	list = 0;
    }

    /*-********************************************************
     * This next portion was used simply to debug the code.
     * It can be uncommented to examine the scan results.
    image = (short *) rdimg(cursmm);
    file = fopen("array.out", "w");
    for (x = 0;
	x < 448; x++)
    {
	fprintf(file, "row %d\n", x);
	for (y = 0; y < 448; y++)
	{
	    fprintf(file, "%d ", *(image + npixl * x + y));
	} fprintf(file, "\n");
    }
    fclose(file);

    quad1 = cursmm[11];
    quad2 = cursmm[12];
    xyzscan(image, xscan, yscan, zscan, quad2);

    for (x = 0; x < (SCANS * 2) + 1; x++)
    {
	printf("xscan row %d:\n", x);
	for (y = 0; y
	    <= maxrad - minrad - 1; y++)
	{
	    printf("%d ", xscan[x][y]);
	} printf("\n");
    }

    for (x = 0; x < (SCANS * 2) + 1; x++)
    {
	printf("yscan row %d:\n", x);
	for (y = 0;
	    y <= maxrad - minrad - 1; y++)
	{
	    printf("%d ", yscan[x][y]);
	} printf("\n");
    }


    for (x = 0; x < (SCANS * 2) + 1; x++)
    {
	printf("zscan row %d:\n", x);
	for (y = 0;
	    y <= maxrad - minrad - 1; y++)
	{
	    printf("%d ", zscan[x][y]);
	} printf("\n");
    }

    ******End of debug portion**************/

    /*-
     * While there is a list of file names, get the quadrant of each
     * file,  read the image into memory, and begin processing.
     */
    do
    {
	if ((quadp = strchr(cursmm, '.')) == NULL) {
	   fprintf(stderr, "File name must be in the form xxx.cxx\n");
	   exit(1);
        }
	else {
	   quad1 = quadp[2];
	   quad2 = quadp[3];
        }

	image = (short *) rdimg(cursmm);

	xstart = xcen;
	ystart = ycen;
	loopcount = 0;

        /*-
	 * When xoff and yoff both equal 0, the image is centered.
	 */
	xoff = yoff = 1;
	while ((xoff != 0) || (yoff != 0))
	{
	    /*-
	     * Scan in the x, y and z directions.
	     */
	    xyzscan(image, xscan, yscan, zscan, quad2);

            /*-
	     * Calculate the number of pixels in the x and y directions.
	     */
	    xpix = RADIUS - (minrad + average(xscan, "xscan"));
	    ypix = RADIUS - (minrad + average(yscan, "yscan"));

	    /*- 
	     * Calculate the corrected minimum radius and difference in
	     * the radius in the z direction using the right triangle law.
	     */
	    minradz = (int) sqrt((double) ((minrad * minrad) / 2));
	    zrpix = minradz + average(zscan, "zscan");
	    zdpix = sqrt((double) ((zrpix * zrpix) * 2.0));
	    /*-
	     * Round up if necessary and get the final zpix result.
	     */
	    zpix = ((zdpix - (int) zdpix) < abs(.5) ? (int) zdpix : (int) zdpix + 1);
	    zpix = RADIUS - zpix;

            /*-
	     * Switch on the quadrant to determine what to add and what to
	     * subtract from the x and y offsets.
	     */
	    switch (quad1)
	    {
		case 'n':
		    xoff = zpix;
		    yoff = ypix;
		    break;
		case 'e':
		    xoff = xpix;
		    yoff = -ypix;
		    break;
		case 's':
		    xoff = -xpix;
		    yoff = -ypix;
		    break;
		case 'w':
		    xoff = -xpix;
		    yoff = ypix;
		    break;
                case 'p':
		    xoff = -zpix;
		    yoff = -ypix;
		    break;
	    }

	    xcen += xoff;
	    ycen += yoff;
	    loopcount++;

            /*-
	     * If we can't find the given BRLIMIT, then give up
	     * and return the default center value... :(
	     */
	    if (xcen > xstart + STOPSCAN) {
	       fprintf(stderr, "%s: %s: ", argv[0], cursmm);
	       fprintf(stderr, "xscan has exceeded STOPSCAN value.\n");
	       fprintf(stderr, "Using default center for %s.\n", cursmm);
	       xcen = xstart;
	       ycen = ystart;
	       xoff = yoff = 0;
            }

	    if (ycen > ystart + STOPSCAN) {
	       fprintf(stderr, "%s: %s: ", argv[0], cursmm);
	       fprintf(stderr, "yscan has exceeded STOPSCAN value.\n");
	       fprintf(stderr, "Using default center for %s.\n", cursmm);
	       xcen = xstart;
	       ycen = ystart;
	       xoff = yoff = 0;
            }

            /*-
	     * Are we spinning our wheels moving the center back and forth?
	     * If so, give up and return the default center value... :(
	     */
	    if (loopcount > LOOPLIMIT) {
	       fprintf(stderr, "%s: %s: ", argv[0], cursmm);
	       fprintf(stderr, "loopcount has exceeded LOOPLIMIT value.\n");
	       fprintf(stderr, "Using default center for %s.\n\n", cursmm);
	       xcen = xstart;
	       ycen = ystart;
	       xoff = yoff = 0;
            }

            /*-
	     * Print some info to the screen.
	     *
	    printf("xoff is %d, yoff is %d\n", xoff, yoff);

	    printf("average of xscan  is %d\n", average(xscan, "xscan"));
	    printf("average of yscan  is %d\n", average(yscan, "yscan"));
	    printf("average of zscan  is %d\n", average(zscan, "zscan"));
	    */

	}
	/*-
	 * Print the information to the outfile.
	 */
        if (list) 
           fprintf(stderr, "%s %d %d\n", cursmm, (int)xcen*2, (int)ycen*2);

	fprintf(outfile, "%s %d %d\n", cursmm, (int) xcen * 2, (int) ycen * 2);

    /*
     * diamscan(image, dxscan, dyscan, quad2); printf("dxscan is %d, dyscan
     * is %d\n", average(dxscan, "dxscan"), average(dyscan, "dyscan"));
     * 
     */
        /*-
	 * Get the next file out of the infile (if that was the input) and
	 * center it. Otherwise, we're done! :)
	 */
	if (list)
	{
	    if (fscanf(infile, "%s", cursmm) == EOF)
	    {
		list = 0;
	    }

	}
	/*printf("cursmm is ***%s***\n", cursmm);*/
	printf("------------------------------------\n\n");

    } while (list);

    free(cursmm);
    fclose(infile);
    fclose(outfile);
    return (0);
}
