/*-------------------------------------------------------------------------
 * xzyscan() scans values out of an image along the x, y and z axis.  
 *-------------------------------------------------------------------------
 * Ron Decker ASAP/CU  SMM C/P March 2, 1992
 * Initial Creation.
 */
#include <stdio.h>
#include <math.h>
#include "center.h"
#include "/usr/people/smm/lib/include/cplib.ext"

xyzscan(image, xscan, yscan, zscan, quad)
short *image;         			/*- image array to scan */
int xscan[][RADELTA*2];    		/*- results of x scan */
int yscan[][RADELTA*2];    		/*- results of y scan */
int zscan[][RADELTA*2];			/*- results of z scan */
char quad;				/*- image quadrant */
{
   int x, y, z;				/*- loop variables */
   int dx, dy;				/*- determine how to scan image */ 
   int xc, yc;				/*- x and y centers */
   int lowz, highz;			/*- corrected min & max z radius */
   int maxrad, minrad;               	/*- find true min & max radius */ 

   minrad = RADIUS - RADELTA;
   maxrad = RADIUS + RADELTA;

   /*-
    * Determine in which direction to scan image.
    */
   switch (quad) {
      case 'n': dx = -1; dy = -1; break;
      case 'w': dx =  1; dy = -1; break;
      case 's': dx =  1; dy =  1; break;
      case 'e': dx = -1; dy =  1; break;
      default: 
	 fprintf(stderr, "xyzscan: Unknown quadrant given\n");
	 exit (1);
   }

   /*-
    * Find integer x and y centers.
    */ 
   xc = (int)xcen;
   yc = (int)ycen;
   /*printf("xcen is %f, xc is %d\n",xcen,xc);
   printf("ycen is %f, yc is %d\n",ycen,yc);*/

   /*-
    * Scan from the image in the x direction.
    */
   for (x = xc-SCANS; x <= xc+SCANS; x++)
      {
      for (y = yc + (dy*minrad); y != yc + (dy*(maxrad+1)); y += dy)
         {
	 yscan [x-xc+SCANS][abs(yc+(dy*minrad)-y)] = (*(image + x + y*npixl));
         }
      }

   /*-
    * Scan from the image in the y direction.
    */
   for (y = yc-SCANS; y <= yc+SCANS; y++) {
      for (x = xc + (dx*minrad); x != xc + (dx*(maxrad+1)); x += dx) {
	 xscan [y-yc+SCANS][abs(xc+(dx*minrad)-x)] = (*(image + x + y*npixl));
      }
   }

   /*-
    * Determine the min and max radius to use, then scan from the image
    * in the z direction.
    */
   lowz = (int)sqrt((double)((minrad*minrad)/2));
   highz = lowz + (maxrad-minrad);
   for (z = -SCANS; z <= SCANS; z++) {
      y = yc + (dy*lowz) + z;
      for (x = xc + (dx*lowz); x != xc + (dx*(highz+1)); x += dx) {
	 zscan [z+SCANS][abs(xc+(dx*lowz)-x)] = (*(image + x + y*npixl));
	 y += dy;
      }
   }
	 

   return (0);
}
