
/* nicexr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Find a nice range for plotting
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * On Input:
 *    xmin	minimum x value
 *    xmax	maximum x value
 *    mjrx	desired number of major intervals
 *
 * On Output:
 *    xmin	a nice minimum x value
 *    xmax	a nice maximum x value
 *    mjrx	number of major intervals
 *    mnrx	number of minor intervals
 *    dxx	x interval between major intervals
 *    dxn	x interval between minor intervals
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <math.h>

nicexr (xmin, xmax, mjrx, mnrx, dxx, dxn)

float *xmin, *xmax ;
int   *mjrx, *mnrx ;
float *dxx, *dxn ;
{
   extern double floor (), ceil () ;
   static float ddx [4] = {2.0, 2.5, 5.0, 10.0} ;
   static int ndx [4] = {2, 5, 2, 4} ;
   float dx, dz ;
   float p10 ;
   int i ;
   int ipow ;

   if (*xmin > *xmax)
      {
      dx = *xmin ;
      *xmin = *xmax ;
      *xmax = dx ;
      }
   if (*mjrx <= 0) *mjrx = 10 ;
   dx = (*xmax - *xmin) / *mjrx ;
   if (dx <= 0.0)
      {
      dx = 1.0 ;
      *xmax = *xmin + 1.0 ;
      }
   ipow = (int) floor (log10 ((double) dx)) ;
   p10 = (float) pow ((double) 10.0, (double) ipow) ;
   dz = dx / p10 ;
   for (i = 0 ; i < 4 ; i++)
      if (dz <= ddx [i]) break ;

   dx = ddx [i] * p10 ;
   *dxx = dx ;
   *xmin = (float) (dx * floor ((double) (*xmin / dx))) ;
   *xmax = (float) (dx * ceil ((double) (*xmax / dx))) ;
   *mjrx = ceil ((double) ((*xmax - *xmin) / dx)) ;
   *mnrx = ndx [i] ;
   *dxn = *dxx / *mnrx ;

   return (0) ;
}
