/* plscan(image, ip1, ir1, ip2, ir2, scntyp, scan, scandx, ns)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Line Scan procedure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extracts values from an image along a line between two arbitrary end points.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 20 April 1984
 * 18 July 1984: add option to remove stray light & vignetting.
 * 31 July 1984: add option to remove F-corona; remove printf for scan values.
 * 7 February 1985: change double to float.
 * 7 November 1986: fix calibration correction logic.
 * 5 December 1986: use Saito F-corona.
 * 10 June 1991: adapt for SGI/IRIS GL windows.
 *  1 April 1999: replace stray & vignet with cpstray & cpvignet functions.
 * 15 April 1999: replace calcor with jupcor (Jupiter cal. correction).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>

#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/ov_color.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

plscan (image, ip1, ir1, ip2, ir2, scntyp, scan, scandx, ns)

char	*image;			/* Pointer to 1 byte image array. */
int	ip1, ir1, ip2, ir2;	/* Pixel,line coordinate scan limits. */
int	*ns;			/* Number of values in scan. */
int	*scntyp;		/* Scan type: radial=1 or line=2. */
float	scan [];		/* Dependent axis scan array. */
float	scandx [];		/* Independent axis scan array. */

{

extern float polcor;		/* Polaroid transmission correction factor. */
extern float cpstray ();	/* Stray Light function. */
extern float cpvignet ();	/* Vignetting function. */
extern float fcorona ();	/* Function to provide F-corona value. */
extern float jupcor ();		/* Jupiter calibation correction factor. */
short *ims;			/* Pointer to 2 byte image array. */
float x, y, slope, yoff;
float x1, x2, y1, y2;
float dist, dstinc, dstsun;
float r1, r2, th1, th2;
float r, th;
float pix1, rec1, pix2, rec2;
float Cexp = 1.e-11;		/* Scaling factor for calibrated intensities. */
float Bexp;
float str, vig;		/* Storage for stray light & vignetting. */
float jc;			/* Storage for calibration correction factor. */
int i;
int ierr;			/* Error. */
int ival1, ival2;
int ix, ixinc, iy, iyinc;
int ix1, ix2, iy1, iy2;
int ixg, iyg;
int nx, nxdif;
int ny, nyinc, nydif;

   jc = 1.0;
   str = 0.0;
   vig = 1.0;

   ims = (short *) image;
   if (idtype == 3)
      Bexp = Cexp;
   else
      Bexp = 1.0;

   /*** Clear scan arrays. ***/

   for (i=0; i < MAXSCN; i++)  scandx[i] = scan[i] = 0.;

   /*** Check scan size. ***/

   ierr = 0;
   *ns = max (abs(ip2-ip1), abs(ir2-ir1)) + 1;

   if (*ns > MAXSCN)
      {
      fprintf (stderr, "linscn> error...# scan values:%d > MAXSCN:%d\n",
               *ns, MAXSCN);
      return (-1);
      }

   /*** Compute distance (in Rsun units) between scan values. ***/
   /*   (line scans only) */

/*   if (*scntyp == 2) */
      {
      dist = sqrt ( (float)(ip2-ip1)*(ip2-ip1) + (float)(ir2-ir1)*(ir2-ir1) );
      dstinc = dist / ((*ns-1) * pixrs);
      dstsun = -dstinc;
      }

   /*** Orient start/end points so that start point is closest to disk. ***/
   /*   (radial scans only) */

   pix1 = (float) ip1;
   rec1 = (float) ir1;
   pix2 = (float) ip2;
   rec2 = (float) ir2;

   if ((ierr = rcoord (&r1, &th1, &pix1, &rec1, -1, scroll)) != 0)
      return (ierr);
   if ((ierr = rcoord (&r2, &th2, &pix2, &rec2, -1, scroll)) != 0)
      return (ierr);

   /*** Reverse start/end points. ***/

   if (r1 > r2 && *scntyp == 1)
      {
      ix1 = ip2;
      ix2 = ip1;
      iy1 = ir2;
      iy2 = ir1;
      }

   else

   /*** Leave start/end points as given. ***/
 
      {
      ix1 = ip1;
      ix2 = ip2;
      iy1 = ir1;
      iy2 = ir2;
      }

   /*** Equation of a line:   y = slope * x + yoff. ***/

   x1 = (float) ix1;
   x2 = (float) ix2;
   y1 = (float) iy1;
   y2 = (float) iy2;
   slope = -1.0e+9;
   yoff = 0.0;
   if (ix1 != ix2)  slope = (y2 - y1) / (x2 - x1);
   if (ix1 != ix2)  yoff  = (x2*y1 - x1*y2) / (x2 - x1);

/*
   printf("ix1,iy1,ix2,iy2: %d %d %d %d\n", ix1, iy1, ix2, iy2);
   printf("slope,yoff: %f %f\n", slope, yoff);
 */

   if (fabs ((float) slope) < 1.0)
      {
/*      printf("pixel-driven scan\n"); */

      /* ...... pixel-driven loop : */

      nxdif = ix2 - ix1;
      if (nxdif < 0)  nxdif = -nxdif;
      nx = nxdif + 1;
      ix = ix1 - 1;
      ixinc = 1;
      if (ix1 > ix2)
         {
         ix = ix1 + 1;
         ixinc = -1;
	 }

      for (i = 0; i < nx; i++)
         {
         ix = ix + ixinc;
         x = (float) ix;
         y = (float) iy1;
         if (iy1 != iy2)  y = slope * x + yoff;
         iy = (int) y;

         /* 1 byte per pixel:
          *  clear all high order bits above low-order byte.
          */

         if (nbypix == 1)
            {
            ival1 = (*(image + (ix-ip) + (iy-ir)*npix) & 0377);
            ival2 = (*(image + (ix-ip) + (iy-ir+1)*npix) & 0377);
            }
         else

         /*** 2 bytes per pixel. ***/

            {
            ival1 = *(ims + (ix-ip) + (iy-ir)*npix) & 0177777;
            ival2 = *(ims + (ix-ip) + (iy-ir+1)*npix) & 0177777;
            }

         /*** Convert (x,y) coordinates into (r,th). ***/

         if ((ierr = rcoord (&r, &th, &x, &y, -1, scroll)) != 0)
            return (ierr);

         /*** Linear interpolation in vertical axis. ***/

         if (ljc == 1)  jc = jupcor ();
         scan [i] = (float) ((ival1 + (ival2-ival1) * (y-iy)) * Bexp) * jc;

         /*** Remove stray light & vignetting. ***/

/*         if (idtype == 3) */
            {
	    if (lstr == 1)  str = cpstray  (r * H.rsun);
	    if (lvig == 1)  vig = cpvignet (r * H.rsun);
	    scan [i] = (scan [i] * polcor - str) * vig;
            if (lfc == 1)  scan [i] -= fcorona (r, th);
	    }

         /*** Determine independent axis coordinate value. ***/

         dstsun = dstsun + dstinc;
         if (*scntyp == 1)  scandx [i] = (float) r;
         if (*scntyp == 2)  scandx [i] = (float) dstsun;
/*
 *         if (igterm == 0)
 *            {
 *	      if (idtype == 3)
 *	      printf ("ix:%3d iy:%3d r:%7.3f th:%7.2f length(Rsun):%7.3f Bsun:%9.2e\n",
 *                    ix, iy, r, th, dstsun, scan[i]);
 *	      else
 *	      printf ("ix:%3d iy:%3d r:%7.3f th:%7.2f length(Rsun):%7.3f intensity:%7.2f\n",
 *                    ix, iy, r, th, dstsun, scan[i]);
 *	      }
 */

         /*** Plot 2 nearest points on IRIS in graphics overlay. ***/

         if (igr == 1)
            {
            ixg = ix/nres + gMINCOL;
            iyg = gMAXROW - iy/nres;
	    gl_wvec (ixg, iyg, ixg, iyg+1, ov_red);
	    }
         }
      *ns = nx;
      faverage (nave, scan, *ns);
      }

   else

      {

/* ...... line-driven loop : */

/*       printf ("line-driven scan\n"); */

      nydif = iy2 - iy1;
      if (nydif < 0)  nydif = -nydif;
      ny = nydif + 1;
      iy = iy1 - 1;
      iyinc = 1;
      if (iy1 > iy2)
         {
         iy = iy1 +1;
         iyinc = -1;
         }

      for (i = 0; i < ny; i++)
         {
         iy = iy + iyinc;
         x = (float) ix1;
         y = (float) iy;
         if (ix1 != ix2)  x = (y-yoff) / slope;
         ix = (int) x;

         /*** Extract 2 adjacent pixels (in same line) from image. ***/

         /* 1 byte per pixel:  clear all high order bits above low-order byte.*/

         if (nbypix == 1)
            {
            ival1 = (*(image + (ix-ip)   + (iy-ir)*npix) & 0377);
            ival2 = (*(image + (ix-ip+1) + (iy-ir)*npix) & 0377);
            }
         else

         /* 2 bytes per pixel. */

            {
            ival1 = *(ims + (ix-ip)   + (iy-ir)*npix) & 0177777;
            ival2 = *(ims + (ix-ip+1) + (iy-ir)*npix) & 0177777;
            }

         /*** Convert (x,y) coordinates into (r,th). ***/

         if ((ierr = rcoord (&r, &th, &x, &y, -1, scroll)) != 0)
            return (ierr);

         /*** Linear interpolation in horizonal axis. ***/

         if (ljc == 1)  jc = jupcor ();
         scan [i] = (float) ((ival1 + (ival2-ival1) * (x-ix)) * Bexp) * jc;

         /*** Remove stray light & vignetting. ***/

         if (idtype == 3)
            {
	    if (lstr == 1)  str = cpstray  (r * H.rsun);
	    if (lvig == 1)  vig = cpvignet (r * H.rsun);
	    scan [i] = (scan [i] * polcor - str) * vig;
            if (lfc == 1)  scan[i] -= fcorona (r, th);
	    }

         /*** Determine independent coordinate value for scan. ***/

         dstsun = dstsun + dstinc;
         if (*scntyp == 1)  scandx [i] = (float) r;
         if (*scntyp == 2)  scandx [i] = (float) dstsun;
/*
 *         if (igterm == 0)
 *            {
 *            if (idtype == 3)
 *               printf ("ix:%3d iy:%3d r:%6.2f th:%7.2f length(Rsun):%6.2f Bsun:%9.2e\n",
 *                       ix, iy, r, th, dstsun, scan[i]);
 *            else
 *               printf("ix:%3d iy:%3d r:%6.2f th:%7.2f length(Rsun):%7.2f intensity:%7.2f\n",
 *                       ix, iy, r, th, dstsun, scan[i]);
 *	      }
 */

         /*** Plot 2 nearest points on Grinnell in graphics overlay. ***/

         if (igr == 1)
            {
            ixg = ix/nres + gMINCOL;
            iyg = gMAXROW - iy/nres;
            gl_wvec (ixg, iyg, ixg+1, iyg, ov_red);
            }
         }
      *ns = ny;
      faverage (nave, scan, *ns);

      }
   return (ierr);
}
