/* rcoord.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P coordinate transformation.
 * rcoord (radius, theta, x, y, isign, roll)
 * isign = 1 :  (radius,theta) --> (x,y).
 *       =-1 :  (x,y) --> (radius,theta).
 * Image coordinate origin (x=0,y=0): upper left corner.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 16 December 1985
 * 29 August 1991: ensure that theta is between 0 and 360 degrees.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include "/home/stanger/lib/include/cplib.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

rcoord (radius, theta, x, y, isign, roll)

float	*radius, *theta;/* Radius (Rsun) & Position Angle (degrees). */
float	*x, *y;		/* Cartesian coordinates:  (0,0)=upper left.*/
int	isign;		/* =1: (r,th) --> (x,y). otherwise, vice-versa.*/
int	roll;		/* Roll angle (degrees). */

{
int	ierr;			/* error indicator. */
float	smmth;			/* angle in degrees w.r.t. +x axis. */
double	angle;			/* angle in radians. */
double r2;			/* radius^2 */
double deltax, deltay;

   degnor = 133.0 + (float) roll;	/* Orientation of solar north. */
   ierr = 0;

   if (isign == 1)		/*** (radius,theta) --> (x,y) ***/
      {
      smmth = *theta + degnor;
      while (smmth > 360.0)  smmth -= 360.0;
      angle = (float) (raddeg * smmth);
         *x = (float) (xcen + *radius * pixrs * cos (angle));
         *y = (float) (ycen - *radius * pixrs * sin (angle));

      if ((*x < ip || *x > lp) || (*y < ir || *y > lr))
	 {
         ierr = -1;
	 fprintf (stderr, "rcoord> ip,lp:%3d,%3d  ir/lr:%3d,%3d  ",
		  ip, lp, ir, lr);
	 fprintf (stderr, "raddeg:%9.4f  pixrs:%9.4f\n",
		  raddeg, pixrs);
	 fprintf (stderr, "rcoord> x/ycen:%7.2f %7.2f\n", xcen, ycen);
         fprintf (stderr, "rcoord> radius:%5.2f  theta:%7.2f --> ",
		  *radius, *theta);
         fprintf (stderr, "x:%7.2f  y:%7.2f\n", *x, *y);
	 }
      }
   else				/*** (x,y) --> (r,theta) ***/
      {
      deltax = *x - xcen;
      deltay = ycen - *y;
      r2 = deltax * deltax + deltay * deltay;

      *radius = sqrt (r2) * rspix;
      *theta = atan2 (deltay, deltax) * degrad - degnor;
      while (*theta < 0.0)  *theta += 360.0;
      }
   return (ierr);
}
