/* scs.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Intensity scans (radial & theta): bilinear interpolation algorithm.
 * Reverse order in applying corrections: F-corona, vignet, stray, Munro.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 * 24 March 1987
 * 2 April 1987: change polaroid transmission factor (0.38 to 0.76).
 * 3 April 1987: add point-point scan.
 * 27 March 1990: convert to GKS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include <stdio.h>
#include "/u/rose/svgrin2/grin.h"
#include "/u/smm/lib/src/cp/cplib.h"
#include "/u/smm/lib/src/grin/grscale.h"

/* Standard C system library and Math functions library. */

#include <stdio.h>
#include <math.h>

int GRN = 0;
int pcount = 0;
float polcor = 1.;		/* Polaroid transmission correction factor. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main()

{

extern	int prscan();	/* Radial intensity scan function. */
extern	int ptscan();	/* Theta intensity scan function. */
extern  int hkline();	/* Housekeeping information display on terminal. */
extern  int xyplot();	/* XY plot using NCAR system plot package. */
extern  int xysplot();	/* XY plot using NCAR system plot package. */
/* extern	int ncarplot(); */
extern char *rdimg();	/* Read image from disk to memory. */

char	*image;		/* Pointer to image array. */
static char buff [80];
static char namimg[80];	/* Image file name. */
static char namdat[80];	/* Scan data file name. */
static char answer[2];	/* Storage for y/n response. */
static char ovrcolor[2];/* Storage for overlay color. */
static char namemc[80]; /* Metacode file name. */
static char labx[40];	/* Label for X-axis. */
static char laby[40];	/* Label for Y-axis. */
static char title[60];	/* Graph title. */
static char labtyp [30];/* Data type description. */
static char gksnam [80] = "scs.cgm";

static float scan[MAXSCN];	/* Scan magnitude array. */
static float scandx[MAXSCN];	/* Scan index array. */
/*
 * Polaroid wheel :
 * p0 : clear			    telemetry value = 0
 * p1 : +60 degrees (w.r.t. pylon)  telemetry value = 1
 * p2 :   0 degrees		    telemetry value = 3
 * p3 : -60 degrees		    telemetry value = 2
 */
static float pctab[4] = {1.,0.4077,0.3942,0.4214}; /* order: p0, p1, p3, p2. */

float	rmin, rmax;	/* Radial limits (Rsun). */
float	rinc;		/* Radial increment. */
float	theta;		/* Solar position angle (degrees). */
float	th;		/* Solar position angle (degrees). */
float	tmin, tmax;	/* Angular limits (degrees). */
float	tinc;		/* Angular increment. */
float	radius;		/* Radius (R/Rsun). */
float	fymin, fymax;	/* Min/max values for y-axis plot scaling. */

int	scntyp = 2;	/* Scan type: 1=radial, 2=line. */
int	nval;		/* # values in scan. */
int	ndata;
int	i;
int	ierr;
int	ip1, ir1;
int	ip2, ir2;
int	idot;		/* Index pointer to "." in image filename.*/
int	ixscale = 0;	/* x-axis scaling type; 0=linear. */
int	iyscale = 0;	/* y-axis scaling type; 0=linear. */
int	ibkg = 1;	/* Background flag; 0=tick marks only. */
int	ig;		/* Graphics terminal; 0=no, 1=yes. */
int	ismooth = 0;
int	plotit = 1;
int	clip = 0;

FILE	*datfd;		/* File descriptor for scan data file. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   lcc  = 0;
   lstr = 0;
   lvig = 0;
   lfc  = 0;

   fprintf (stderr, "ps: C/P point-point scans in intensity.\n");
   fprintf (stderr, "Enter C/P image file name: ");
   fscanf  (stdin,  "%s", namimg);

   fprintf (stderr, "Are you using a graphics terminal [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   igterm = 0;
   if (strncmp ("y",answer,1) == 0)
      {
      igterm = 1;
      gksini_ (&plotit, gksnam);
      opngks_ ();
      gsclip_ (&clip);
      }
   ig = igterm;

   fprintf (stderr, "Do you wish to use the SunView Grinnell emulator [y/n] ? ");
   fscanf  (stdin,  "%s", answer);
   igr = 0;
   if (strncmp ("y",answer,1) == 0)
	{
	GRN = 1;
	igr = 1;
	ngr = 0;
	imgch = 0;
	ovrch = 10;
/*
 *	fprintf (stderr, "Enter Grinnell device [0,1] :");
 *	fscanf  (stdin,  "%d", &ngr);
 *	if (ngr == 0)  fprintf (stderr, "Enter Grinnell channel [0,1,2] : ");
 *		else   fprintf (stderr, "Enter Grinnell channel [0,1,2,3] : ");
 *	fscanf  (stdin,  ""%d", &imgch);
 *	fprintf (stderr, "Enter overlay channel color [r,g,b,w] : ");
 *	fscanf  (stdin,  "%1s", ovrcolor);
 *	if (strncmp("r", ovrcolor, 1) == 0)  ovrch = 10;
 *	if (strncmp("g", ovrcolor, 1) == 0)  ovrch =  9;
 *	if (strncmp("b", ovrcolor, 1) == 0)  ovrch =  8;
 *	if (strncmp("w", ovrcolor, 1) == 0)  ovrch = 11;
 */
	}

/* Grinnell initialization. */

   if (igr == 1)
      {
      gr_start (ngr);
      init (1);
      zcursor (OFF);
      rwech_batch_on ();
      cpview (namimg, imgch, 1);
      rwech_batch_off ();
      rwech (ovrch);
      disp (ovrch);
      gr_erase (ovrch);
      }

/* Open input image disk file and read housekeeping record. */

   image = rdimg (namimg);
   hkline ();
   if (idtype == 1)  strcpy (labtyp, "vidicon DN value   ");
   if (idtype == 3)  strcpy (labtyp, "brightness(B/Bsun) ");
   if (idtype == 4)  strcpy (labtyp, "density (elec/cm2) ");
   if (idtype == 5)  strcpy (labtyp, "B2 - B1   (B/Bsun) ");
   if (idtype == 6)  strcpy (labtyp, "     Bk   (B/Bsun) ");
   if (idtype == 7)  strcpy (labtyp, "(Be-Bb)/Bk (B/Bsun)");
   if (idtype == 8)  strcpy (labtyp, "(Be-Bb)/Bb (B/Bsun)");
   if (idtype == 9)  strcpy (labtyp, " polarization frac.");
   if (idtype == 10)  strcpy (labtyp, "      pB  (B/Bsun) ");
   if (idtype == 11)  strcpy (labtyp, "     Br+t (B/Bsun) ");
   if (idtype == 12)  strcpy (labtyp, "     Btan (B/Bsun) ");
   if (idtype == 13)  strcpy (labtyp, "     Brad (B/Bsun) ");

/* Create file names for scan data and plot. */

   for (i=0; i<strlen(namimg); i++)  if (namimg[i] == '.')  idot = i;
   strncpy (namdat, namimg, idot);
   strncpy (namemc, namimg, idot);
   strcat (namdat, ".dat");
   strcat (namemc, ".cgm");
   printf ("data file:  %s\n", namdat);
   printf ("metacode plot file: %s\n", namemc);
   datfd = fopen (namdat, "w");

   fprintf (stderr, "Enter # points to average in scans : ");
   fscanf  (stdin,  "%d", &nave);

/******* Scan loop. *******/

next:
   fflush (stdout);
   fprintf (stderr, "Quit [y/n] ? ");

   fscanf  (stdin,  "%s", answer);
   if (strncmp ("y", answer, 1) == 0)
      {
      fclose (datfd);
      free (image);
      if (igterm)
	 {
	 clsgks_ ();
	 unlink (gksnam);
	 }
      exit(0);
      }

   fprintf (stderr, "Enter the type of scan [r=radial, t=theta, p=point]: ");
   fscanf  (stdin,  "%s", answer);
   scntyp = 0;
   if (strncmp("r", answer, 1) == 0)  scntyp = 1;
   else
   if (strncmp("p", answer, 1) == 0)  scntyp = 2;
   else
   if (strncmp("t", answer,1) == 0)   scntyp = 3;

   if (idtype == 3)
      {
      fprintf (stderr, "Apply calibration correction [y,n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp("y", answer, 1) == 0)  lcc = 1;
      else lcc = 0;

      fprintf (stderr, "Remove stray light [y,n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lstr = 1;
      else lstr = 0;

      fprintf (stderr, "Remove vignetting [y,n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lvig = 1;
      else lvig = 0;

      fprintf (stderr, "Remove F-corona [y,n] ? ");
      fscanf  (stdin,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lfc = 1;
      else lfc = 0;

      if (H.ipw != 0)
         {
         fprintf (stderr, "Apply polaroid transmission correction [y,n] ? ");
         fscanf  (stdin,  "%s", answer);
         if ((strncmp ("y", answer, 1) == 0))
            polcor = 1.0 / (2.0 * pctab[H.ipw]);
         else polcor = 1.;
         }
      fprintf (stderr, "polcor: %f\n", polcor);
      }

/* Check calibration mirror telemetry. */

   if (H.icm == 1)	/* Calibration Mirror is OUT. */
      {
      if (lsv == 1)
         {
         fprintf (stderr, "Remove F-corona (y/n) ? ");
         fscanf  (stdin,  "%s", answer);
         if (strncmp ("y", answer, 1) == 0)  lfc = 1;
         else lfc = 0;
         }
      }
   else	/* Assume calibration mirror is IN. */
      {
      fprintf (stdout,
         "*** cal. mirror IN: no removal of stray,vignet, or F-corona. ***\n");
      lsv = 0;
      lfc = 0;
      }

   if (nbypix == 1)
   sprintf (laby, "%s pc=%4.2f", labtyp, polcor);
   else
   if (nbypix == 2)
   sprintf (laby, "%s pc=%4.2f", labtyp, polcor);
   if (nbypix == 4)
   sprintf (laby, "%s", labtyp);

   fprintf (datfd, "Image file name: %s\n", namimg);
   if (idtype == 3 && lcc == 1)
   fprintf (datfd,
      "*** WARNING *** Munro calibration correction has been applied.\n");
   if (H.icm != 1)
   fprintf (datfd,
      "*** cal. mirror IN: no removal of stray,vignet, or F-corona. ***\n");
   if (lstr == 1)  fprintf (datfd,"*** Stray light removed. ***\n");
   if (lvig == 1)  fprintf (datfd,"*** Vignetting removed. ***\n");
   if (lfc == 1)   fprintf (datfd,"*** F-corona removed. ***\n");

/* Radial scan. */

   if (scntyp == 1)
      {
      fprintf (stderr, "Enter solar position angle for radial scan (deg.): ");
      fscanf  (stdin,  "%f", &theta);
      fprintf (stderr, "Enter solar radius min, max, & increment (Rsun): ");
      fscanf  (stdin,  "%f,%f,%f", &rmin, &rmax, &rinc);

      if (igr == 1)  rwech_batch_on ();
         prscan (namimg, image, rmin, rmax, rinc, theta, &scntyp,
                             scan, scandx, &nval);
      if (igr == 1)  rwech_batch_off ();

      /* Write scan values to output data file. */

      fprintf (datfd, "Radial scan  %6.2f to %6.2f Rsun  %7.2f degrees\n",
              rmin, rmax, theta);
      for (i=0; i<nval; i++)
         {
         fprintf (datfd,"%3d  r:%7.3f  th:%7.2f %s:%9.2e\n",
                  i, scandx [i], theta, labtyp, scan [i]);
         }

      /* Plot scan. */
 
      if (igterm == 1)
         {
         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s +csvf", theta, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -s", theta, namimg);
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -v", theta, namimg);
         else
         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -f", theta, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -sv", theta, namimg);
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -vf", theta, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -sf", theta, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -svf", theta, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -c", theta, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -cs", theta, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -cv", theta, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -cf", theta, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 0)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -csv", theta, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -csf", theta, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -cvf", theta, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 1)
         sprintf (title, "Radial Scan %5.1f deg. %14.14s -csvf", theta, namimg);
         else
         sprintf (title, "Radial Scan %5.1f deg. %14.14s ????", theta, namimg);
         sprintf (labx,"solar radius (Rsun)");

         if (lvig == 0)
            {
            fymin = 5.e-10;
            fymax = 5.e-9;
            }
         else
            {
            fymin = 5.e-10;
            fymax = 5.e-8;
            }
         if (nbypix == 4)
	    {
	    fymin = -1.0;
	    fymax =  1.0;
	    }
         gets (buff);
/*
 *         xysplot (scandx, scan, rmin, rmax, fymin, fymax, nval,
 *		 namemc, title, labx, laby, ixscale, iyscale, ibkg, ig);
 */
         xyplot (scandx, scan, nval, namemc, title, labx, laby,
                  ixscale, iyscale, ibkg, ig);
         }
      }

   else
   if (scntyp == 2)

      {
      fprintf (stderr, "Enter starting pixel & line (x,y): ");
      fscanf  (stdin,  "%d,%d", &ip1, &ir1);
      fprintf (stderr, "Enter ending pixel & line (x,y): ");
      fscanf  (stdin,  "%d,%d", &ip2, &ir2);

      if (igr == 1)  rwech_batch_on ();
         if ((ierr = plscan (image, ip1, ir1, ip2, ir2, &scntyp,
	      scan, scandx, &nval))!= 0)
            fprintf (stderr, "scs> linscn error: %d\n", ierr);
      if (igr == 1)  rwech_batch_off ();

      /* Write scan values to output data file. */

      fprintf (datfd,"Line Scan  (%d,%d) - (%d,%d)\n",
              ip1, ir1, ip2, ir2);
      for (i=0; i<nval; i++)
         {
         fprintf (datfd, "%3d  distance:%7.2f Rsun  %s:%9.2e\n",
	          i, scandx[i], labtyp, scan[i]);
         }

      /* Plot scan. */
 
      if (igterm == 1)
         {
         sprintf (title,"Scan %d,%d-%d,%d %14.14s", ip1,ir1,ip2,ir2,namimg);

         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 0)
         strcat (title, " +csvf");
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 0)
         strcat (title, " -s");
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 0)
         strcat (title, " -v");
         else
         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 1)
         strcat (title, " -f");
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 0)
         strcat (title, " -sv");
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 1)
         strcat (title, " -vf");
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 1)
         strcat (title, " -sf");
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 1)
         strcat (title, " -svf");
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 0)
         strcat (title, " -c");
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 0)
         strcat (title, " -cs");
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 0)
         strcat (title, " -cv");
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 1)
         strcat (title, " -cf");
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 0)
         strcat (title, " -csv");
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 1)
         strcat (title, " -csf");
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 1)
         strcat (title, " -cvf");
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 1)
         strcat (title, " -csvf");
         else
         strcat (title, " ????");

         sprintf (labx,"distance along line (Rsun)");

         gets (buff);
         xyplot (scandx, scan, nval, namemc, title, labx, laby,
                  ixscale, iyscale, ibkg, ig);
/*
 *         ncarplot (scandx, scan, nval, namemc, title, labx, laby,
 *		   ixscale, iyscale, ibkg, ismooth, ig);
 */

         }
      }

   else

   if (scntyp == 3)
      {
      fprintf (stderr, "Enter solar radius for theta scan (R/Rsun): ");
      fscanf (stdin,  "%f", &radius);
      fprintf (stderr, "Enter position angle min, max, & increment (degrees): ");
      fscanf (stdin,  "%f,%f,%f", &tmin, &tmax, &tinc);

      if (igr == 1)  rwech_batch_on ();
         ptscan (namimg, image, tmin, tmax, tinc, radius, scan, scandx,
		    &nval);
      if (igr == 1)  rwech_batch_off ();

      /* Print scan values. */

      fprintf (datfd,"Theta scan  %6.2f to %6.2f degrees  %7.2f Rsun\n",
              tmin, tmax, radius);
      for (i=0; i<nval; i++)
         {
         th = scandx[i];
         if (th > 360.)  th -= 360.;
            fprintf (datfd,"%3d  r:%7.3f  th:%7.2f %s:%9.2g\n",
                    i, radius, th, labtyp, scan[i]);
         }

      /* Plot scan. */
 
      if(igterm == 1)
         {
         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s +csvf", radius, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -s", radius, namimg);
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -v", radius, namimg);
         else
         if (lcc == 0 && lstr == 0 && lvig == 0 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -f", radius, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 1 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -sv", radius, namimg);
         else
         if (lcc == 0 && lstr == 0 && lvig == 1 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -vf", radius, namimg);
         else
         if (lcc == 0 && lstr == 1 && lvig == 0 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -sf", radius, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -c", radius, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -cs", radius, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -cv", radius, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 0 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -cf", radius, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 0)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -csv", radius, namimg);
         else
         if (lcc == 1 && lstr == 0 && lvig == 1 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -cvf", radius, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 0 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -csf", radius, namimg);
         else
         if (lcc == 1 && lstr == 1 && lvig == 1 && lfc == 1)
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s -csvf", radius, namimg);
         else
         sprintf (title,"Theta Scan %5.2f Rsun %14.14s ????", radius, namimg);

         sprintf (labx,"position angle (degrees)");

	 gets (buff);
         xyplot (scandx, scan, nval, namemc, title, labx, laby,
                   ixscale, iyscale, ibkg, ig);
/*
 *         ncarplot (scandx, scan, nval, namemc, title, labx, laby,
 *                ixscale, iyscale, ibkg, ig);
 */

         }
      }

   fprintf (datfd, "\n");
   goto next;
}
