/* gmthk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Modifies housekeeping format of a C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 10 August 1989
 * 5 September 1989: file name format is "YYDDDHHMM.vXX".
 * 12 February 1990: reformatted year & DOY should be copied from bytes 125-127.
 * 13 February 1990: reformatted year & DOY are already in bytes 55,62,63 for
 *                   images in 1989 after DOY 239.
 * 25 June 1996: update directory name for "cplib.ext".
 *  2 August 1996: clear housekeeping bytes 222 & 223 to zero.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/riker/stanger/lib/include/cplib.ext"

gmthk (imgbuf, imgdir, nameid, iyear, idoy, ihour, imin, isec, ivers)

char imgbuf [];		/* raw image memory array. */
char imgdir [];		/* image disk directory. */
char nameid [];		/* image file name. */
int iyear, idoy, ihour, imin, isec;	/* image GMT */
int ivers;		/* MET --> GMT conversion index. */

{
   static char nammonth[12][4] = {"jan","feb","mar","apr","may","jun",
				  "jul","aug","sep","oct","nov","dec"};
   static int ic [7];
   int icyear, icdoy, icday, icmonth, ichour, icmin, icsec;
   int i;

   imgbuf [0] = (char) (iyear & 0377);		/* GMT year. */

   imgbuf [1] = (char) ((idoy / 256) & 0377);	/* GMT DOY (MSB). */
   imgbuf [2] = (char) (idoy & 0377);		/* GMT DOY (LSB). */

   imgbuf [20] = (char) (ihour & 0377);		/* GMT hour. */
   imgbuf [21] = (char) (imin & 0377);		/* GMT minutes. */

   imgbuf [24] = (char) (isec & 0377);		/* GMT seconds. */
   imgbuf [25] = (char) (ivers & 0377);		/* MET --> GMT conversion. */

   strncpy (&imgbuf[36], "yodacp", 6);	/* raw image volume name. */
   strncpy (&imgbuf[42], imgdir, 6);	/* raw image dataset name. */

   imgbuf [52] = (char) 0;	/* byte order: 0 -> Sun-4/280; 1 -> DEC. */

   /*** For 1989, DOY > 239, hskp info is already correct. ***/

   if (iyear != 89 || idoy < 240)
      {
      imgbuf [55] = imgbuf [125];/* Year for reformatted image creation. */
      imgbuf [62] = imgbuf [126];/* [MSB] DOY for reformatted image creation. */
      imgbuf [63] = imgbuf [127];/* [LSB] DOY for reformatted image creation. */
      }

   imgbuf [70] |= 020;		/* Sun-4 processor. */

/*   imgbuf [79] */		/* Image quantity [0 -> 100 %]. */

   rollcp (imgbuf);		/* Copy roll to bytes 88 & 89 [degree units]. */

   imgbuf [103] = (char) 8;	/* bits per pixel. */

   imgbuf [110] = (char) (npix / 256) & 0377;	/* # pixels per line. */
   imgbuf [111] = (char) npix & 0377;

   strncpy (&imgbuf[119], nameid, 13);	/* image filename. */

   for (i=132; i<137; i++)  imgbuf[i] = '\0';	/* spares for filename. */

   strncpy (&imgbuf[137], "      ", 6);	/* calibrated image volume name. */
   strncpy (&imgbuf[143], "      ", 6);	/* calibrated image dataset name. */

   imgbuf [149] = '\0';		/* Year image was calibrated. */
   imgbuf [150] = '\0';		/* Day-Of-Year image was calibrated (MSB). */	
   imgbuf [151] = '\0';		/* Day-Of-Year image was calibrated (LSB). */	

   for (i=152; i<158; i++)  imgbuf [i] = ' ';	/* a0 parameter name */
   for (i=158; i<164; i++)  imgbuf [i] = ' ';	/* a0 parameter name */
   for (i=164; i<170; i++)  imgbuf [i] = ' ';	/* a0 parameter name */
   for (i=170; i<176; i++)  imgbuf [i] = ' ';	/* a0 parameter name */

   for (i=176; i<182; i++)  imgbuf [i] = ' ';	/* NAMCOR file name */
   for (i=182; i<188; i++)  imgbuf [i] = ' ';	/* NAMSTR file name */
   for (i=188; i<194; i++)  imgbuf [i] = ' ';	/* NAMVIG file name */

   for (i=194; i<196; i++)  imgbuf [i] = '\0';	/* FWMAG */
   for (i=196; i<200; i++)  imgbuf [i] = '\0';	/* PWCORX, PWCORY */
   for (i=200; i<204; i++)  imgbuf [i] = '\0';	/* SMCORX, SMCORY */
   for (i=204; i<208; i++)  imgbuf [i] = '\0';	/* CXCEN, CYCEN */
   for (i=208; i<212; i++)  imgbuf [i] = '\0';	/* DXCEN, DYCEN */
   for (i=212; i<216; i++)  imgbuf [i] = '\0';	/* SXCEN, SYCEN */
   for (i=216; i<218; i++)  imgbuf [i] = '\0';	/* ESCORR */
   for (i=218; i<220; i++)  imgbuf [i] = '\0';	/* RADFAC */

   imgbuf [220] = '\0';		/* CALEXP = cal scale factor, exponent of 10. */
   imgbuf [221] = (char) 1;
   imgbuf [222] = '\0';
   imgbuf [223] = '\0';

   return (0);
}
