/* gmtmv1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Recompute MET for SMM C/P images.
 * Read image, recompute MET, write new file, delete original image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 4 August 1989
 * 5 September 1989: file name format is "YYDDDHHMM.vXX".
 * 27 December 1989: Create new directory if image belongs in another DOY.
 * 22 February 1990: Do NOT modify housekeeping if it was modified previously.
 * 25 June 1996: update directory name for "cplib.ext".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.ext"

extern FILE *LOG;

/* Standard C system I/O library and Math functions library. */

#include <sys/types.h>
#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

gmtmv (olddir, oldimg)

char olddir [];			/* DOY directory pathname. */
char oldimg [];			/* Image file name. */
{

extern u_char	*rdsimg();	/* Read image from disk to memory. */
extern int	wrimg();	/* Write image from memory to disk. */
u_char	*imgbuf;		/* Pointer to image array. */
static char	buff [80];	/* Storage for terminal message. */
static char	cpdir [6];	/* Sub-directory. */
static char	cpyrdoy [6];	/* Image year/DOY. */
static char	oldpath [80];	/* Input image path name. */
static char	newdir [80];	/* Alternate output image directory. */
static char	newimg [80];	/* Output image file name. */
static char	newpath [80];	/* Output image path name. */

int	i, n;	/* index variable. */
int	idot;	/* index for "period" character in file name. */
int	islash;	/* index for last "slash" character in directory pathname. */
int	ivers;	/* MET --> GMT conversion index. */

int iyear0, idoy0, ihour0, imin0;	/* Initial GMT. */
float fsec0;				/* Initial seconds. */
int sct0 = 0;				/* Initial MET. */
int met, met0, met1, met2, met3;	/* Image MET. */
int kyear, kdoy, khour, kmin, ksec;	/* Image GMT. */

static int izyear[7] = {80,  80,  80,  80,  84,  78, 78};
static int izdoy[7]  = {45,  45,  45,  95, 102, 121, 78};
static int izhour[7] = {12,  12,  12,   5,  14,   0,  0};
static int izmin[7]  = { 0,  0,   17,  20,  01,   0,  0};
static double zsec[7] = {0.0, 3.324, 30.873, 20.799, 29.500, 0.0, 0.0};


   /*** Open input image disk file and read housekeeping record. ***/

   sprintf (oldpath, "%s/%s", olddir, oldimg);
   imgbuf = rdsimg (oldpath);

   /*** Exit if image year/DOY does not match directory name. ***/

   islash = -1;
   for (i=0; i<80; i++)
      {
      if (olddir [i] == '\0')  break;
      if (olddir [i] == '/')  islash = i;
      }
   strncpy (cpdir, &olddir [islash+1], 5);
   cpdir [5] = '\0';
   sprintf (cpyrdoy, "%02d%03d", H.year, H.doy);
   cpyrdoy [5] = '\0';

/*
   if (strcmp (cpdir, cpyrdoy) != 0)
      {
      fprintf (LOG, "gmtmv> directory: %s  doesn't match image yr/doy: %s\n",
	       cpdir, cpyrdoy);
      fprintf (stderr, "gmtmv> directory: %s  doesn't match image yr/doy: %s\n",
	       cpdir, cpyrdoy);
      exit (-211);
      }
 */

   ghkline ();
   if (H.year == 80)
      {
      ivers = 1;
      if (H.doy < 94)  ivers = 2;
      if (H.doy >= 94 && H.doy < 119)  ivers = 3;
      if (H.doy >= 119)  ivers = 4;
      iyear0 = izyear [ivers-1];
      idoy0  = izdoy  [ivers-1];
      ihour0 = izhour [ivers-1];
      imin0  = izmin  [ivers-1];
      fsec0  = (float) zsec [ivers-1];
      }
   else
      {
      ivers = 5;
      iyear0 = 84;
      idoy0  = 102;
      ihour0 = 14;
      imin0  = 1;
      fsec0  = 29.5;
      }

   /*** Find position of period (.) delimiter in input file name. ***/

   for (i=0; i < strlen (oldimg); i++)
      {
      if (oldimg[i] == '.')  {idot = i; break;}
      }

/*
 * printf ("ih,iv: %d %d\n", H.ih, H.iv);
 * printf ("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
 * printf ("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
 * printf ("xcen,ycen: %f %f\n", xcen, ycen);
 */

/*** Extract MET from housekeeping. ***/

   met0 = 0;
   if (ivers >= 5)  met0 = (*(imgbuf+12) & 077) * 16777216;

   met1 = *(imgbuf+6) & 0377;
   met1 = met1 * 65536;
   met2 = *(imgbuf+7) & 0377;
   met2 = met2 * 256;
   met3 = *(imgbuf+8) & 0377;
   met = met0 | met1 | met2 | met3;
   metgmt (iyear0, idoy0, ihour0, imin0, fsec0, sct0, met,
          &kyear,  &kdoy,  &khour,  &kmin,  &ksec);

   /*** If computed DOY differs from original DOY, create new directory. ***/

   strcpy (newdir, olddir);
   if (H.doy != kdoy)
      {
      islash = -1;
      for (i=0; i<80; i++)
         {
         if (newdir [i] == '\0')  break;
         if (newdir [i] == '/')  islash = i;
         }
      newdir [islash+1] = '\0';
      sprintf (cpyrdoy, "%02d%03d", kyear, kdoy);
      cpyrdoy [5] = '\0';
      strcat (newdir, cpyrdoy);

      /*** Create data directory for images. ***/

      if (mkdir (newdir, 0755))
         {
         fprintf (stderr, "*** UNABLE TO CREATE DIRECTORY %s ***", newdir);
         fprintf (LOG, "*** UNABLE TO CREATE DIRECTORY %s ***", newdir);
         exit (-212);
         }
      else
         {
         fprintf (stderr, ">>> Directory %s created <<< ", newdir);
         }
      }

/*
 * fprintf (LOG, "gmtmv> met: %d   met0: %d   met1: %d   met2: %d   met3: %d\n",
 *          met, met0, met1, met2, met3);
 * fprintf (LOG, "gmtmv> iyear0: %d   idoy0: %d   ihour0: %d   imin0: %d\n",
 *	    iyear0, idoy0, ihour0, imin0);
 */

   sprintf (newimg, "%02d%03d%02d%02d\0", kyear, kdoy, khour, kmin);
   strncat (newimg, &oldimg [idot], 4);
   sprintf (newpath, "%s/%s", newdir, newimg);

   fprintf (LOG, "19%2d %03d: %02d:%02d:%02d\n",
	    kyear, kdoy, khour, kmin, ksec);

   /*** Delete original image from disk. ***/

   unlink (oldpath);		/* Delete old image from disk. */

   /*** Rearrange housekeeping format, update GMT. ***/

   /* If first two characters of file name have NOT changed,
    * do NOT modify housekeeping.
    */

   for (i=0; i<strlen(newdir); i++)  if (newdir[i] == '/')  n = i;
   if (strncmp (oldimg, newimg, 2) != 0)
      gmthk (imgbuf, &newdir[n], newimg, kyear, kdoy, khour, kmin, ksec, ivers);
   else
      fprintf (LOG, "*** %s : Housekeeping NOT modified. ***\n", newimg);

   /*** Write modified image to disk. ***/

   wrimg (newpath, imgbuf);	/* Write new image to disk. */

   /*** Relinquish storage for image array. ***/

   free (imgbuf);
   return (0);
}
