/* rdsimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array,
 * INCLUDING housekeeping and bitmap records.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 21 August 1985
 * 19 September 1989: use sqlhskp instead of cphskp.
 * 19 December 1989: Use updated (19 Dec 89) sqlhskp.
 * 25 June 1996: update directory name for "cplib.ext".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

char	*
rdsimg (namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
register   char	*image;	/* Pointer to image array. */
register   int	imgfd;	/* File descriptor for image disk file. */
register   int	imgsiz; /* Image size (bytes) (excluding hskp & bitmap). */
register   long	nread;

/* Decommutate Housekeeping. */

   sqlhskp (namimg);	

/*   printf ("rdsimg> npix: %d nbypix: %d\n", npix, nbypix); */

/* Open image file. */

   if ((imgfd = open (namimg, 0)) < 0)
     { fprintf (stderr,"rdsimg> Can't open file %s\n", namimg);  return(NULL); }

/* Allocate storage in memory for image array. */

   imgsiz = (npix * (npix + 2) * nbypix);
   if ((image = malloc (imgsiz)) == NULL)
     {  fprintf (stderr, "rdsimg> malloc error.");
        exit (1);
     }

   if ((nread = read (imgfd, image, imgsiz)) <= 0)
      { fprintf (stderr,"rdsimg> image read error.\n");  return (NULL); }

   if (nbypix == 2)
      {
      imgsiz = (npix * npix);
      srevbyt (image + npix * nbypix * 2, nbypix, imgsiz);
      }

/*   printf ("rdsimg> nread: %d\n", nread); */

   if (close (imgfd) == -1)
      {
      fprintf (stderr,"rdsimg> file close error.%s\n", namimg);
      return (NULL);
      }
  return (image);
}
