/* cphskp3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Decodes housekeeping and stores results in the stucture CPH [smmcplib.h].
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew Stanger  27 September 2002
 * 16 Jan 2006: Update lcc, lfc, lreg, lvig, lstr for CPH structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define	MAXHSK	224
#define	ZC	'\0'

#include <stdio.h>
#include "/home/stanger/lib/include/smmcplib.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

int
cphskp3 (namimg)			/* SMM C/P Housekeeping Decoder */

char	namimg [] ;			/* Image file name. */

{

   /*** Arrays for housekeeping entries. ***/

   extern float pow10 () ;
   extern float sunrad () ;	/* Solar radius [arcsec]. */
   extern int   pb0r () ;	/* Solar p-angle, b-angle, Rsun. */

   static char  namext [80] ;	/* File name extension. */
   static char  cglstw [2] ;	/* Storage for green line stow status. */

   static char  namcm  [4][2] = {"m", "o", "i", "b"} ;
   static char  namfw  [6][3] = {"r", "Ha", "op", "b", "wb", "g"} ;
   static char  namgls [4][2] = {"m", "a", "s", "f"} ;
   static char  nampw  [4][4] = {"CLR", "+60", "-60", "0dg"} ;
   static char  namres [2][2] = {"H","L"} ;
   static char  namsm  [9][3] = {"sw","s", "se","e", "ne","n", "nw","w", "4q"} ;
   static char  namsmf [9][3] = {"sw","ss","se","ee","ne","nn","nw","ww","4q"} ;
   static char  namtype [4] = {'v', 't', 'c', '\0'} ;
   static int   lih [4], liv [4] ;   /* Binary components of IH & IV. */

   /*** Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). ***/
   /* (No geometric correction applied.) */

   static float vidcen [9][2] = { 36.0, 500.0,		/* sw */
				 214.0, 194.0,		/* s  */
				 540.0,   4.0,		/* se */
				 820.0,  88.0,		/* e  */
				 892.0, 362.0,		/* ne */
				 718.0, 680.0,		/* n  */
				 386.0, 856.0,		/* nw */
				 104.0, 792.0,		/* w  */
				 447.5, 447.5} ;	/* n+e+s+w */

   /*** Disk centers (vs. Sector Mirror) for calibrated data (C34520). ***/
   /* (Processed with geometric correction WW6COR). */

   static float ww6cen [9][2] = { 48.0, 508.0,		/* sw */
				 222.0, 206.0,		/* s  */
				 530.0,  40.0,		/* se */
				 814.0,  86.0,		/* e  */
				 884.0, 354.0,		/* ne */
				 714.0, 672.0,		/* n  */
				 394.0, 824.0,		/* nw */
				 114.0, 786.0,		/* w  */
				 447.0, 447.0} ;	/* n+e+s+w */

   static double secrs = 960.0 ;	/* arcseconds per solar radius.   */
   static float secpix = 5.933 ;	/* arcseconds per pixel (Hi res). */
   double pangle, bangle ;

   double PI = 3.141592654 ;
   double froll, radroll, rollwt ;

   int day ;
   int i ;
   int icode1, icode2, icode3 ;
   int idot ;
   int iext ;
   int iexp = 11 ;
   int imgoff ;
   int ires, ipw, ifw, ism, icm, iglstw, iglpos ;
   int isign ;
   int jsm ;
   int kres, ihs, ivs ;
   int lenimg ;
   int lres ;
   int month ;
   int nbtpix ;
   int roll1, roll2, cproll, iroll, troll, tlmroll, rolldif ;
   int segcnt1, segcnt2 ;

   short *whiphd ;

   int  *hk ;			/* Pointer to housekeeping array (integer).*/
   char *cbuf ;			/* Pointer to housekeeping array (character). */
   char *calloc () ;		/* system allocation routine. */
   char *malloc () ;		/* system allocation routine. */

   FILE *imgfd ;		/* File pointer for image disk file. */

   for (i = 0 ; i < strlen (namimg) ; i++)
      {
      if (namimg [i] == '.')  idot = i ;
      }
   strcpy (namext, &namimg [idot + 1]) ;

   if (strcmp (namext, "pic") == 0)
      {
      imgoff = 24 ;
      }
   else
      imgoff = 0 ;

   /*** Open image file. ***/

   if ((imgfd = fopen (namimg,"r")) == NULL)
      {
      fclose (imgfd) ;
      fprintf (stdout, "Unable to open file: %s\n", namimg) ;
      return (-1) ;
      }

   /*** Allocate storage for housekeeping array. ***/

   if ( ( (hk = (int *) calloc (MAXHSK, sizeof (int))) == NULL ) ||
        ( cbuf = malloc (MAXHSK) ) == NULL )
      {
      fprintf (stderr,"cphskp> malloc error.\n") ;
      fflush (stderr) ;
      fclose (imgfd) ;
      return (-2) ;
      }

   if (imgoff > 0)
      {
      whiphd = (short *) malloc (imgoff) ;
      if (fread (whiphd, sizeof (short), imgoff/2, imgfd) == NULL)
         {
         fclose (imgfd) ;
         free (cbuf) ;
         free ((char *) hk) ;
         return (-3) ;
         }
      }

   /*** Read housekeeping from C/P image file into character array 'cbuf'. ***/

   if (fread (cbuf, sizeof (char), MAXHSK, imgfd) == NULL)
      {
      fclose (imgfd) ;
      free (cbuf) ;
      free ((char *) hk) ;
      if (imgoff > 0)  free (whiphd) ;
      return (-3) ;
      }
   fclose (imgfd) ;

   /*** Copy housekeeping from `char' to `int' array. ***/

   for (i = 0 ; i < MAXHSK ; i++)  *(hk + i) = (int) *(cbuf + i) & 0377;
   free (cbuf) ;

/*
 *   fprintf (stdout, "hk:\n") ;
 *   fprintf (stdout, "%d %d %d %d %d %d %d %d\n",
 *      *(hk), *(hk+1), *(hk+2), *(hk+3), *(hk+4), *(hk+5), *(hk+6), *(hk+7)) ;
 */

   /*** Year and Day-of-Year. ***/

   CPH.year = *hk ;		/* year */

   CPH.doy  = *(hk + 1) << 8 ;
   CPH.doy += *(hk + 2) ;	/* day-of-year */

   /*** Polaroid Filter: p0=clear, p1=+60, p2=0 degrees, p3=-60. ***/

   CPH.pwpos = (*(hk + 3) & 0300) >> 6 ;

   /*** HV command status. ***/

   CPH.hvstat = (*(hk + 3) & 040) >> 5 ;

   /*** Instrument ID. ***/

   CPH.smmid = (*(hk + 3) & 030) >> 3 ;

   /*** Resolution: high or low. ***/

   CPH.resolut = (*(hk + 3) & 04) >> 2 ;

   /*** Data Mode. ***/

   CPH.datamode = (*(hk + 3) & 02) >> 1 ;

   /*** Housekeeping Frozen. ***/

   CPH.hkfrozen = (*(hk + 3) & 01) ;

   /*** Segment count. ***/

   CPH.segcnt =  (*(hk + 4) & 0377) * 32 +
		((*(hk + 5) & 0370) >> 3) ;

   /*** Exposure Inhibit. ***/

   CPH.expinh = (*(hk + 5) & 04) >> 2 ;

   /*** Read Inhibit. ***/

   CPH.readinh = (*(hk + 5) & 02) > 1 ;

   /*** Premature Exposure Inhibit. ***/

   CPH.prexpinh = (*(hk + 5) & 01) ;

   /*** Mission Elapsed Time. ***/

   CPH.met = (*(hk + 6) & 0377) * 65536 +
	     (*(hk + 7) & 0377) * 256 +
	     (*(hk + 8) & 0377) ;

   /*** IH,IV: image format parameters (horizontal, vertical). ***/

   CPH.ih = (*(hk + 9) & 0360) >> 4 ;
   CPH.iv = (*(hk + 9) & 017) ;

   /*** Exposure duration (units of 1.024 seconds). ***/

   CPH.expose  = (*(hk + 10) & 0377) << 4 ;
   CPH.expose += (*(hk + 11) & 0360) >> 4 ;

   /*** Spacecraft roll. ***/

   /*** Compute roll using bytes 11 & 12 (from SMM on-board computer [OBC]).***/

   roll1 = (int) (*(hk + 11) & 017) ;
   roll2 = (int) (*(hk + 12) & 0377) ;
   isign =  (roll1 & 8) / 8 ;		/* Sign bit. */

   if (CPH.year == 80)	/* 1980: Least significant roll bit = 0.111905819 deg.*/
      {
      iroll = (int) ((roll1 & 7) * 256 + (roll2 & 0377)) ;
      if (isign == 1)
	 radroll = - (float) (((~(iroll) + 1) & 03777)) ;
      else
         radroll = (float) iroll ;

      rollwt = 0.111905819 ;
      froll = radroll * rollwt ;
      if (froll > 0)
	 tlmroll = (int) (froll + 0.5) ;
      else
	 tlmroll = (int) froll ;
      }
   else
   if (CPH.year > 80)	/* 1984-89: Least sig. roll bit = 7.161972438 deg. */
      {
      iroll = (int) ((roll1 & 7) * 4 + (roll2 & 0300) / 64) ;
      if (isign == 1)
	 radroll = - (float) (((~(iroll) + 1) & 037)) ;
      else
         radroll = (float) iroll ;

      rollwt = 7.161972438 ;
      tlmroll = (int) (radroll * rollwt) ;
      }

   /*** troll is ``best-guess'' roll in degrees derived from C/P telemetry. ***/

   if (CPH.year == 80)
      troll = tlmroll ;
   else
   if (CPH.year > 80)	/* For 1984-1989, set troll to closest 10 degrees. */
      {
      if (isign == 1)
         troll = tlmroll ;
      else
         troll = tlmroll + 7 ;
      troll = (troll / 10) * 10 ;
      }
   else
      troll = 0 ;

   /*** Adjust troll so that -180 < cproll < +180 ***/

   while (troll >  180)  troll -= 360 ;
   while (troll < -180)  troll += 360 ;

   /*** Compute roll using hskp bytes 88 & 89 (valid if between 1 & 360). ***/

   roll1 = (int) (*(hk + 88) & 0377) ;
   roll2 = (int) (*(hk + 89) & 0377) ;
   cproll = roll1 * 256 + roll2 ;

   /*** Compare troll and cproll. ***/

   if (cproll < 1 || cproll > 360)	/* Invalid value in bytes 88 & 89. */
      scroll = troll ;			/* Use troll for S/C roll. */

   else					/* Use cproll for S/C roll. */
      {

      /*** Adjust cproll so that -180 < cproll < +180 ***/

      while (cproll >  180)  cproll -= 360 ;
      while (cproll < -180)  cproll += 360 ;

      scroll = cproll ;

      rolldif = abs (troll - cproll) ;
      if (rolldif > 10)
	 {
	 fprintf (stderr,
	    "*** roll (bytes 11&12): %d disagrees with roll (bytes 88&89): %d\n",
            troll, cproll) ;
         fprintf (stderr, "*** Check records for correct roll, then run fixroll.\n") ;
/*	 exit (-1) ; */
	 }
      }

   CPH.smmrollr = radroll ;
   CPH.smmrolld = scroll ;

   /*** Green line flter tilt position and stow status. ***/

   iglpos = *(hk + 13) & 0377 ;
   CPH.glpos = iglpos ;

   iglstw = (*(hk + 14) & 0300) >> 6 ;
   CPH.glstow = iglstw ;
   strcpy (cglstw, &namgls [iglstw][0]) ;

   /*** Sector mirror position: sw, s, se, e, ne, n, nw, w. ***/

   CPH.smpos = (*(hk + 14) & 074) >> 2 ;

   /*** Internal Shutter. ***/

   CPH.ispos = (*(hk + 14) & 03) ;

   /*** Aperture Door. ***/

   CPH.adpos = (*(hk + 15) & 0300) >> 6 ;

   /*** Calibration mirror position: move, out, in, between. ***/

   CPH.cmpos = (*(hk + 15) & 060) >> 4 ;

   /*** Color filter wheel position: red|H-alpha|open|blue|wideband|green. ***/

   CPH.fwpos = (*(hk + 15) & 016) >> 1 ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1a = (*(hk + 16) & 0377) ;
   CPH.pes2a = (*(hk + 17) & 0377);
   CPH.pes3a = (*(hk + 18) & 0377);
   CPH.pes4a = (*(hk + 19) & 0377);

   /*** Hours and minutes [at beginning of exposure]. ***/

   CPH.hour   = (*(hk + 20) & 0377) ;
   CPH.minute = (*(hk + 21) & 0377) ;

   /*** PCS Y Offset. ***/

   CPH.pcsyoff = (*(hk + 22) & 0374) >> 2 ;

   /*** PCS Z Offset. ***/

   CPH.pcszoff = (*(hk + 23) & 374) >> 2 ;

   /*** Seconds [at beginning of exposure]. ***/

   CPH.second = *(hk + 24) ;

   /*** MET -> GMT conversion index [1-5]. ***/

   CPH.metconv = (*(hk + 25) & 0377) ;

   /*** PCS Threshold. ***/

   CPH.pcsth = (*(hk + 26) & 0360) >> 4 ;

   /*** DAS Threshold. ***/

   CPH.dasth = (*(hk + 26) & 017) ;

   /*** PCS Override Status. ***/

   CPH.pcsovr = (*(hk + 27) & 0200) >> 7 ;

   /*** DAS Override Status. ***/

   CPH.dasovr = (*(hk + 27) & 0100) >> 6 ;

   /*** PCS Threshold Exceeded. ***/

   CPH.pcsthexc = (*(hk + 27) & 040) >> 5 ;

   /*** DAS Threshold Exceeded. ***/

   CPH.dasthexc = (*(hk + 27) & 020) >> 4 ;

   /*** 8 Volt Status. ***/

   CPH.stat8v = (*(hk + 27) & 010) >> 3 ;

   /*** PCS Enable/Disable Status. ***/

   CPH.pcsen = (*(hk + 27) & 04) >> 2 ;

   /*** SEC Erase Mode. ***/

   CPH.secerase = (*(hk + 28) & 0200) >> 7 ;

   /*** SEC Expose Mode. ***/

   CPH.secexpos = (*(hk + 28) & 0100) >> 6 ;

   /*** SEC Read Mode. ***/

   CPH.secread = (*(hk + 28) & 040) >> 5 ;

   /*** Image Size Override. ***/

   CPH.imsizovr = (*(hk + 28) & 020) >> 4 ;

   /*** Flood Status. ***/

   CPH.flood = (*(hk + 28) & 010) >> 3 ;

   /*** MEB Sequencer Reset ***/

   CPH.mebseq = (*(hk + 28) & 04) >> 2 ;

   /*** Internal Shutter Redundant Enable. ***/

   CPH.isredun = (*(hk + 29) & 020) >> 4 ;

   /*** Aperture Door Redundant Enable. ***/

   CPH.adredun = (*(hk + 29) & 010) >> 3 ;

   /*** Calibration Mirror Redundant Enable. ***/

   CPH.cmredun = (*(hk + 29) & 04) >> 2 ;

   /*** 5303 Redundant Enable. ***/

   CPH.glredun = (*(hk + 29) & 02) >> 1 ;

   /*** Floop Lamp Power. ***/

   CPH.flpower = (*(hk + 29) & 01) ;

   /*** Image sequence counter (aka: 12-bit operational status). ***/

   CPH.imgcnt  = (*(hk + 30) & 0377) * 16 ;
   CPH.imgcnt += (*(hk + 31) & 0360) / 16 ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1b = (*(hk + 32) & 0377) ;
   CPH.pes2b = (*(hk + 33) & 0377) ;
   CPH.pes3b = (*(hk + 34) & 0377) ;
   CPH.pes4b = (*(hk + 35) & 0377) ;

   /*** Reformatted Volume ID [ASCII]. ***/

   CPH.refvolid [0] = *(hk + 36) ;
   CPH.refvolid [1] = *(hk + 37) ;
   CPH.refvolid [2] = *(hk + 38) ;
   CPH.refvolid [3] = *(hk + 39) ;
   CPH.refvolid [4] = *(hk + 40) ;
   CPH.refvolid [5] = *(hk + 41) ;
   CPH.refvolid [6] = '\0' ;

   /*** Reformatted Dataset ID [ASCII]. ***/

   CPH.refdsid [0] = *(hk + 42) ;
   CPH.refdsid [1] = *(hk + 43) ;
   CPH.refdsid [2] = *(hk + 44) ;
   CPH.refdsid [3] = *(hk + 45) ;
   CPH.refdsid [4] = *(hk + 46) ;
   CPH.refdsid [5] = *(hk + 47) ;
   CPH.refdsid [6] = '\0' ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1c = (*(hk + 48) & 0377) ;
   CPH.pes2c = (*(hk + 49) & 0377) ;
   CPH.pes3c = (*(hk + 50) & 0377) ;
   CPH.pes4c = (*(hk + 51) & 0377) ;

   /*** Byte Order [0=sun,sgi,mac, 1=DEC/IBM]. ***/

   CPH.bytorder = (*(hk + 52) & 377) ;

   /*** SCB Volts [+15]. ***/

   CPH.scbvp15 = (*(hk + 53) & 0377) ;

   /*** SCB Volts [-15]. ***/

   CPH.scbvm15 = (*(hk + 54) & 0377) ;

   /*** Reformatted Year [year - 1900]. ***/

   CPH.refyear = (*(hk + 55) & 0377) ;

   /*** SCB Volts [+10]. ***/

   CPH.scbvp10 = (*(hk + 56) & 0377) ;

   /*** SCB Volts [+12]. ***/

   CPH.scbvp12 = (*(hk + 57) & 0377) ;

   /*** PCB Volts [+15]. ***/

   CPH.pcbvp15 = (*(hk + 58) & 0377) ;

   /*** PCB Volts [-15]. ***/

   CPH.pcbvm15 = (*(hk + 59) & 0377) ;

   /*** SCB Volts [+22]. ***/

   CPH.scbvp22 = (*(hk + 60) & 0377) ;

   /*** SCB Volts [-22]. ***/

   CPH.scbvm22 = (*(hk + 61) & 0377) ;

   /*** Reformatted Data DOY. ***/

   CPH.refdoy = (*(hk + 62) & 0377) * 256 + (*(hk + 63) & 0377) ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1d = (*(hk + 64) & 0377) ;
   CPH.pes2d = (*(hk + 65) & 0377) ;
   CPH.pes3d = (*(hk + 66) & 0377) ;
   CPH.pes4d = (*(hk + 67) & 0377) ;

   /*** PCS Y Position. ***/

   CPH.pcsypos = (*(hk + 68) & 0377) ;

   /*** PCS Z Position. ***/

   CPH.pcszpos = (*(hk + 69) & 0377) ;

   /*** Source Processor [1=pdp11/34, 2=IPD, 4=cray, 8=vax-11/750, 16=sun]. ***/

   CPH.src_cpu = (*(hk + 70) & 0377) ;

   /*** MCB Volts {+5]. ***/

   CPH.mcbvp5 = (*(hk + 71) & 0377) ;

   /*** DAS Y Position. ***/

   CPH.dasypos = (*(hk + 72) & 0377) ;

   /*** DAS Z Position. ***/

   CPH.daszpos = (*(hk + 73) & 0377) ;

   /*** MEB Volts [+15]. ***/

   CPH.mebvp15 = (*(hk + 74) & 0377) ;

   /*** MEB Volts [-15]. ***/

   CPH.mebvm15 = (*(hk + 75) & 0377) ;

   /*** MEB Volts [+12]. ***/

   CPH.mebvp12 = (*(hk + 76) & 0377) ;

   /*** MEB Volts [+5]. ***/

   CPH.mebvp5 = (*(hk + 77) & 0377) ;

   /*** MEB Volts [+6.4]. ***/

   CPH.mebvp64 = (*(hk + 78) & 0377) ;

   /*** Quantity: % image segments captured (i.e., NOT zero-filled). ***/

   CPH.imgquan = *(hk + 79) & 0377 ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1e = (*(hk + 80) & 0377) ;
   CPH.pes2e = (*(hk + 81) & 0377) ;
   CPH.pes3e = (*(hk + 82) & 0377) ;
   CPH.pes4e = (*(hk + 83) & 0377) ;

   /*** SDIB Volts [+15]. ***/

   CPH.sdibvp15 = (*(hk + 84) & 0377) ;

   /*** SDIB Volts [-15]. ***/

   CPH.sdibvm15 = (*(hk + 85) & 0377) ;

   /*** SDIB Volts [+5]. ***/

   CPH.sdibvp5 = (*(hk + 86) & 0377) ;

   /*** SEC Temperature. ***/

   CPH.sectemp = (*(hk + 87) & 0377) ;

   /*** VPB Volts [+6.4]. ***/

   CPH.vpbvp64 = (*(hk + 90) & 0377) ;

   /*** VPB Volts [+26]. ***/

   CPH.vpbvp26 = (*(hk + 91) & 0377) ;

   /*** VPB Volts [+20]. ***/

   CPH.vpbvp20 = (*(hk + 92) & 0377) ;

   /*** VPB Volts [-10]. ***/

   CPH.vpbvm10 = (*(hk + 93) & 0377) ;

   /*** Filament Volts. ***/

   CPH.filvolt = (*(hk + 94) & 0377) ;

   /*** Filament Current. ***/

   CPH.filcur = (*(hk + 95) & 0377) ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1f = (*(hk + 96) & 0377) ;
   CPH.pes2f = (*(hk + 97) & 0377) ;
   CPH.pes3f = (*(hk + 98) & 0377) ;
   CPH.pes4f = (*(hk + 99) & 0377) ;

   /*** Photocathode Volts. ***/

   CPH.pcathv = (*(hk + 100) & 0377) ;

   /*** G1 Volts. ***/

   CPH.g1v = (*(hk + 101) & 0377) ;

   /*** G2 Volts. ***/

   CPH.g2v = (*(hk + 102) & 0377) ;

   /*** Bits Per Pixel [8=byte, 16=int, 32=float]. ***/

   CPH.nbitpix = *(hk + 103) ;
   if (CPH.nbitpix == 8 || CPH.nbitpix == 16 || CPH.nbitpix == 32)
      nbypix = CPH.nbitpix / 8 ;

   /*** Cathode Volts [high]. ***/

   CPH.cathvh = (*(hk + 104) & 0377) ;

   /*** Cathode Volts [low]. ***/

   CPH.cathvl = (*(hk + 105) & 0377) ;

   /*** G3 Volts [high]. ***/

   CPH.g3vh = (*(hk + 106) & 0377) ;

   /*** G3 Volts [low]. ***/

   CPH.g3vl = (*(hk + 107) & 0377) ;

   /*** G4 Volts [high]. ***/

   CPH.g4vh = (*(hk + 108) & 0377) ;

   /*** G4 Volts [low]. ***/

   CPH.g4vl = (*(hk + 109) & 0377) ;

   /*** Actual # pixels per line. ***/

   CPH.npixrow = (*(hk + 110) & 0377) * 256 + (*(hk + 111) & 0377) ;

   /*** PES #1/#2/#3/#4. ***/

   CPH.pes1g = (*(hk + 112) & 0377) ;
   CPH.pes2g = (*(hk + 113) & 0377) ;
   CPH.pes3g = (*(hk + 114) & 0377) ;
   CPH.pes4g = (*(hk + 115) & 0377) ;

   /*** SEC Alignment Current [vertical]. ***/

   CPH.secalcv = (*(hk + 116) & 0377) ;

   /*** SEC Alignment Current [horizontal]. ***/

   CPH.secalch = (*(hk + 117) & 0377) ; 

   /*** SEC Alignment Volts [+15]. ***/

   CPH.secalv = (*(hk + 118) & 0377) ;

   /*** File Name Identifier [YYDDDHHMM.vXX]. ***/

   for (i = 0; i < 18; i++)
      CPH.fileid [i] = (char) *(hk + 119 + i) ;
   CPH.fileid [18] = '\0' ;

   /*** Calibrated Image Volume Name [ASCII]. ***/

   for (i = 0; i < 6; i++)
      CPH.calvolid [i] = (char) (*(hk + 137 + i) & 0377) ;
   CPH.calvolid [6] = '\0' ;

   /*** Calibrated Image Dataset Name [ASCII]. ***/

   for (i = 0; i < 6; i++)
      CPH.caldsid [i] = (char) (*(hk + 143 + i) & 0377) ;
   CPH.caldsid [6] = '\0' ;

   /*** Year when image was calibrated [year - 1900]. ***/

   CPH.calyear = (*(hk + 249) & 0377) ;

   /*** Day-Of-Year when image was calibrated. ***/

   CPH.caldoy = (*(hk + 250) & 0377) * 256 + (*(hk + 251) & 0377) ;

   /*** A0 parameter array name. ***/

   for (i = 0; i < 6; i++)
      CPH.namea0 [i] = (char) (*(hk + 152 + i) & 0377) ;
   CPH.namea0 [6] = '\0' ;

   /*** A1 parameter array name. ***/

   for (i = 0; i < 6; i++)
      CPH.namea1 [i] = (char) (*(hk + 158 + i) & 0377) ;
   CPH.namea1 [6] = '\0' ;

   /*** A2 parameter array name. ***/

   for (i = 0; i < 6; i++)
      CPH.namea2 [i] = (char) (*(hk + 164 + i) & 0377) ;
   CPH.namea2 [6] = '\0' ;

   /*** A3 parameter array name. ***/

   for (i = 0; i < 6; i++)
      CPH.namea3 [i] = (char) (*(hk + 170 + i) & 0377) ;
   CPH.namea3 [6] = '\0' ;

   /*** Geometric Correction Array Name. ***/

   for (i = 0; i < 6; i++)
      CPH.namecor [i] = (char) (*(hk + 176 + i) & 0377) ;
   CPH.namecor [6] = '\0' ;

   /*** Stray Light Mask Name. ***/

   for (i = 0; i < 6; i++)
      CPH.namestr [i] = (char) (*(hk + 182 + i) & 0377) ;
   CPH.namestr [6] = '\0' ;

   /*** Vignetting Factor Mask Name. ***/

   for (i = 0; i < 6; i++)
      CPH.namevig [i] = (char) (*(hk + 188 + i) & 0377) ;
   CPH.namevig [6] = '\0' ;

   /*** Filter Magnification Factor [x 10,000]. ***/

   CPH.fwmag = (*(hk + 194) & 0377) * 256 + (*(hk + 195) & 0377) ;

   /*** Polaroid Shift, no magnification [x 10]. ***/

   CPH.pwcorx = (*(hk + 196) & 0377) * 256 + (*(hk + 197) & 0377) ;
   CPH.pwcory = (*(hk + 198) & 0377) * 256 + (*(hk + 199) & 0377) ;

   /*** Sector Mirror Shift, no registration [x 10]. ***/

   CPH.smcorx = (*(hk + 200) & 0377) * 256 + (*(hk + 201) & 0377) ;

   /*** Color Filter  Shift, no registration [x 10]. ***/

   CPH.fwcorx = (*(hk + 202) & 0377) * 256 + (*(hk + 203) & 0377) ;

   /*** Computed Disk Center before registration [x 10]. ***/

   CPH.cxcen  = (*(hk + 204) & 0377) * 256 + (*(hk + 205) & 0377) ;
   CPH.cycen  = (*(hk + 206) & 0377) * 256 + (*(hk + 207) & 0377) ;

   /*** Standard Disk Center [x 10]. ***/

   CPH.dxcen  = (*(hk + 208) & 0377) * 256 + (*(hk + 209) & 0377) ;
   CPH.dycen  = (*(hk + 210) & 0377) * 256 + (*(hk + 211) & 0377) ;

   /*** Standard Sun  Center [x 10]. ***/

   CPH.sxcen  = (*(hk + 212) & 0377) * 256 + (*(hk + 213) & 0377) ;
   CPH.sycen  = (*(hk + 214) & 0377) * 256 + (*(hk + 215) & 0377) ;

   /*** Equivalent Second color correction [x 10,000]. ***/

   CPH.escorr = (*(hk + 216) & 0377) * 256 + (*(hk + 217) & 0377) ;

   /*** Calflux Radiance Factor, [x 1.0e+11]. ***/

   CPH.radfac = (*(hk + 218) & 0377) * 256 + (*(hk + 219) & 0377) ;

   /*** Photometric scaling factor, exponent of 10 [=11]. ***/

   CPH.calexp = (int) (*(hk + 220) & 0377) ;

   /* IDTYPE: data type.
    *		=1 raw vidicon data (DN units, 8 bits).
    *		=2 Equivalent seconds * 5, 8 bits.
    *		=3 Calibrated data (10^-11 B/Bsun units, 16 bits). 
    * Data type is inferred from the first letter of the filename extension.
    *
    * A C/P filename has the following format :
    * yydddhhmm.vxx :	type 1
    *          .txx :	     2
    *          .cxx :	     3
    *          .dxx :        4
    * where yy = year - 1900.
    *       ddd = day-of-year, hh = hour, mm = minute.
    *       xx = sector mirror position (typically).
    */

   lenimg = strlen (namimg) ;
   iext = -1 ;
   for (i = 0 ; i < lenimg ; i++)
      {
      if (namimg [i] == '.') iext = i+1 ;
      }

/*
 *   if (namimg [iext] == 'v' || namimg [iext] == 'V')  {idtype = 1 ; nbypix=1;}
 *   if (namimg [iext] == 't' || namimg [iext] == 'T')  {idtype = 2 ; nbypix=2;}
 *   if (namimg [iext] == 'c' || namimg [iext] == 'C')  {idtype = 2 ; nbypix=3;}
 *   if (namimg [iext] == 'd' || namimg [iext] == 'D')  {idtype = 4 ; nbypix=4;}
 */

   /*** Decipher numerical codes. ***/

/*
   H.str  = 0 ;
   H.vig  = 0 ;
   H.reg  = 0 ;
   H.f    = 0 ;
   H.k    = 0 ;
   H.bit5 = 0 ;
   H.bit6 = 0 ;
   H.cc   = 0 ;
 */

   icode1 = *(hk + 221) & 0377 ;
   icode2 = *(hk + 222) & 0377 ;
   icode3 = *(hk + 223) & 0377 ;

   if (namimg [iext] == 'v' || namimg [iext] == 'V')
      {
      idtype = 1 ;
      nbypix = 1 ;
/*      H.str = H.vig = H.reg = H.f = H.k = H.bit5 = H.bit6 = H.cc = 0 ; */
      lsv = 0 ;
      }
   else
      {
      if (icode1 &   1) idtype =  1 ;	/* raw vidicon DN values (0-255). */
      if (icode1 &   2) idtype =  2 ;	/* equivalent second (10-bit log). */
      if (icode1 &   4) idtype =  3 ;	/* calibrated radiance (B/Bsun). */
      if (icode1 &   8) idtype =  4 ;	/* electron density data (elec/cm2). */
      if (icode1 &  16) idtype =  5 ;	/* B1 - B2 (deltab B). */
      if (icode1 &  32) idtype =  6 ;	/* Bk (K-corona). */
      if (icode1 &  64) idtype =  7 ;	/* (Bcme - Bbkg) / Bk. */
      if (icode1 & 128) idtype =  8 ;	/* (Bcme - Bbkg) / Bbkg. */

      if (icode2 &   1) idtype =  9 ;	/* polarization fraction. */
      if (icode2 &   2) idtype = 10 ;	/* pB. */
      if (icode2 &   4) idtype = 11 ;	/* B radial+tangential. */
      if (icode2 &   8) idtype = 12 ;	/* Btangential. */
      if (icode2 &  16) idtype = 13 ;	/* Bradial. */
      if (icode2 &  32) idtype = 14 ;	/* Bf + Bs. */
      if (icode2 &  64) idtype = 15 ;	/* B1 / B2. */
      if (icode2 & 128) idtype = 16 ;	/* Miscellaneous. */

      if (icode3 &   1) H.str  = 1 ;	/* stray light removed. */
      if (icode3 &   2) H.vig  = 1 ;	/* vignetting removed. */
      if (icode3 &   4) H.reg  = 1 ;	/* disk registered. */
      if (icode3 &   8) H.f    = 1 ;	/* F-corona removed. */
      if (icode3 &  16) H.k    = 1 ;	/* K-corona removed. */
      if (icode3 &  32) H.bit5 = 1 ;	/* not used. */
      if (icode3 &  64) H.bit6 = 1 ;	/* not used. */
      if (icode3 & 128) H.cc   = 1 ;	/* Munro calib. correction applied. */
      if (lstr == 2 && lvig == 2)  lsv = 2 ;
      }
/*
   H.idtype = idtype ;
   H.nbypix = nbypix ;
 */

/*
 *   fprintf (stdout, "idtype: %d\n", idtype) ;
 */

   CPH.datatype = idtype ;

   CPH.lcc  = H.cc ;
   CPH.lfc  = H.f  ;
   CPH.lreg = H.reg ;
   CPH.lvig = H.vig ;
   CPH.lstr = H.str ;

   /*** calyear & caldoy fields should be zero for raw vidicon data. ***/

   if (idtype == 1)
      {
      CPH.calyear = 0 ;
      CPH.caldoy  = 0 ;
      strcpy (CPH.calvolid, "NONE") ;
      strcpy (CPH.caldsid,  "NONE") ;
      strcpy (CPH.namea0,   "NONE") ;
      strcpy (CPH.namea1,   "NONE") ;
      strcpy (CPH.namea2,   "NONE") ;
      strcpy (CPH.namea3,   "NONE") ;
      strcpy (CPH.namecor,  "NONE") ;
      strcpy (CPH.namestr,  "NONE") ;
      strcpy (CPH.namevig,  "NONE") ;
      }

   /*** Extract reference image year, DOY, hour, minute & second. ***/

   switch (idtype)
      {
      case 5:				/* B2 - B1 */
      case 7:				/* (Bk   - Bref) / Bk   */
      case 8:				/* (Bcme - Bref) / Bref */
      case 15:				/* B2 / B1 */
               year_zero   =  *(hk + 143) ;
               doy_zero    = (*(hk + 144) << 8) + *(hk + 145) ;
               hour_zero   =  *(hk + 146) ;
               minute_zero =  *(hk + 147) ;
               second_zero =  *(hk + 148) ;
      default:
               break ;
      }

/*
 *   fprintf (stderr, "cphskp> icode1/2/3: %3.3o  %3.3o   %3.3o   idtype: %d\n",
 *      icode1, icode2, icode3, idtype) ;
 */

   /*** Convert Green Line Filter position & stow status into characters. ***/

   sprintf (H.glc, "%03d%s", iglpos, cglstw) ;


   /*** Convert date & time into characters [yydddhhmm]. ***/

/*
   sprintf (CPH.fileid, "%02d%03d%02d%02d",
            CPH.year, CPH.doy, CPH.hour, CPH.minute) ;
 */

   /*** Image filename. ***/

/*
   strncat (CPH.fileid, ".", 1) ;
   strncat (CPH.fileid, &namtype [idtype-1], 1) ;
   if (icm == 1)  strncat (CPH.fileid, &namsm [ism][0], 2) ;
   else
   if (icm == 2)  strcat (CPH.fileid, "cm") ;
   else
      strcat (CPH.fileid, "XX") ;
   lenimg = strlen (CPH.fileid) ;
   CPH.fileid [lenimg] = '\0' ;
 */

   free (hk) ;					/* Discard storage for `h'. */
   if (imgoff > 0)  free (whiphd) ;
   return (0) ;
}
