/* fcphksumg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Produce a summary of items from an SMM C/P FITS file.
 * libbcfitsio.a
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   23 October 2001   [ftschk.c]
 *  4 Mar 2002: add file loop.
 * 21 Jan 2006: Customize for SMM C/P.
 *  2 Mar 2006: List FITS file name instead of date & time.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include "fitsio.h"

fcphksumg (filename)

char filename [80] ;			/* FITS file name. */

{
    extern int day_of_year () ;
    extern int time1, time2 ;
    extern int imgnum ;

    static char namfc  ;
    static char namreg ;
    static char namvig ;
    static char namstr ;
    static char namcc  ;
    static char nampei ;
    static char calmir   [12] ;
    static char colorfil [12] ;
    static char comment  [80] ;
    static char date_obs [12] ;
    static char glfilter [12] ;
    static char polaroid [12] ;
    static char res      [12] ;
    static char sector   [12] ;
    static char time_obs [12] ;
    static char namwave   [4] ;
    static char namfw    [6][4] = {"R", "Ha", "OP", "B", "WB", "G"} ;
    static char nampw    [4][4] = {"CL", "P1", "P3", "P2"} ;
    static char namcm    [4][4] = {"M", "O", "I", "B"} ;
    static char namgls   [4][4] = {"M", "I", "S", "F"} ;

    int bitpix ;
    int cmpos ;
    int expose ;
    int frame ;
    int fwpos ;
    int glstw ;
    int hdunum ;
    int hdutype ;
    int idoy ;
    int iglstw ;
    int ihour, iminute, isecond ;
    int ih, iv ;
    int iyear, imonth, iday ;
    int lfc ;
    int lreg ;
    int lvig ;
    int lstr ;
    int lcc ;
    int ndoyyr ;
    int pmode = 0 ;
    int prexpinh ;
    int pwpos ;
    int quantity ;
    int resolut ;
    int scroll ;
    int status ;
    int timedif ;
    int xdim, ydim ;

    float expdur ;

    fitsfile *fp ;

/*
    fprintf (stdout, "   DATE      TIME   CSVFR  Q%% IMG  IH-IV RES EXP I FW GL PW SM CM ROLL\n") ;
    fprintf (stdout, "---------- -------- ----- --- ---- -- -- --- --- - -- -- -- -- -- ----\n") ;
 */
/*
    fprintf (stdout, "      File Name                  Q%% IMG  IH-IV RES EXP I FW GL PW SM CM ROLL\n") ;
    fprintf (stdout, "-------------------------------- --- ---- -- -- --- --- - -- -- -- -- -- ----\n") ;
 */


    status = 0 ;

    /*** Open input FITS file. ***/

    if ( fits_open_file (&fp, filename, READONLY, &status) )
	printerror     ( status );

    if (pmode)
	  {
	  printf ("\n") ;
      printf ("fits_open_file [%s]   status: %d\n", filename, status) ;
	  }

    if (fits_get_num_hdus (fp, &hdunum, &status)) printerror (status) ;

    if (pmode)
	  printf ("#hdus: %d\n", hdunum) ;

    if (fits_get_hdu_num (fp, &hdunum))   printerror (status) ;

    if (fits_get_hdu_type (fp, &hdutype, &status)) printerror (status) ;

    if (pmode)
      printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;

    /*** Retrieve keyword parameters in primary HDU. ***/

    if (fits_read_key (fp, TINT, "NAXIS1", &xdim, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "NAXIS2", &ydim, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "BITPIX", &bitpix, comment, &status))
	printerror (status) ;

    if (fits_read_key (fp, TSTRING, "DATE-OBS", date_obs, comment,
	&status))
	printerror     (status) ;

    sscanf (date_obs, "%04d-%02d-%02d", &iyear, &imonth, &iday) ;
    idoy = day_of_year (iyear, imonth, iday) ;

    if (pmode)
	  printf ("iyear: %d  imonth: %d  iday: %d  idoy: %d\n",
		   iyear, imonth, iday, idoy) ;

    if (fits_read_key (fp, TSTRING, "TIME-OBS", time_obs, comment,
	&status))
	printerror     (status) ;

    sscanf (time_obs, "%2d:%2d:%2d", &ihour, &iminute, &isecond) ;

    if (pmode)
      fprintf (stdout, "[1] xdim/ydim/bitpix: %d %d %d\n",
		   xdim, ydim, bitpix) ;

    if (fits_read_key (fp, TINT, "QUANTITY", &quantity, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "FRAME", &frame, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TFLOAT, "EXPDUR", &expdur, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TSTRING, "POLAROID", polaroid, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TSTRING, "COLORFIL", colorfil, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TSTRING, "GLFILTER", glfilter, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TSTRING, "SECTOR", sector, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TSTRING, "CALMIR", calmir, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "SCROLL", &scroll, comment, &status))
	printerror     (status) ;

    /*** Move to FITS extension HDU. ***/

    hdunum = 2 ;
    if (fits_movabs_hdu (fp, hdunum, &hdutype, &status))
	printerror     (status) ;

    if (pmode)
      printf ("fits_movabs_hdu [2] . . . status: %d\n", status) ;

    if (fits_get_hdu_num  (fp, &hdunum))  printerror (status) ;

    if (fits_get_hdu_type (fp, &hdutype, &status)) printerror (status) ;

    if (pmode)
      printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;

    /*** Retrieve keyword parameters in extension HDU. ***/

    if (fits_read_key (fp, TINT, "NAXIS1", &xdim, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "BITPIX", &bitpix, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "PWPOS", &pwpos, comment, &status))
	printerror     (status) ;
    if (pmode)
	  {
      printf ("pwpos: %d\n", pwpos) ;
	  printf ("nampw: %s\n", &nampw [0][pwpos]) ;
	  printf ("nampw [0][0]: %s\n", &nampw [0][0]) ;
	  printf ("nampw [1][0]: %s\n", &nampw [1][0]) ;
	  printf ("nampw [2][0]: %s\n", &nampw [2][0]) ;
	  printf ("nampw [3][0]: %s\n", &nampw [3][0]) ;
	  }


    if (fits_read_key (fp, TINT, "RESOLUT", &resolut, comment, &status))
	printerror     (status) ;

    if (resolut == 0) strcpy (res, "H"); else strcpy (res, "L") ;

    if (fits_read_key (fp, TINT, "PREXPINH", &prexpinh, comment, &status))
	printerror     (status) ;
    if (prexpinh == 1) nampei = 'Y'; else nampei = 'N';

    if (fits_read_key (fp, TINT, "IH", &ih, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "IV", &iv, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "EXPOSE", &expose, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "GLSTOW", &iglstw, comment, &status))
	printerror     (status) ;
    if (pmode)
	  {
      printf ("iglstw: %d\n", iglstw) ;
	  printf ("namgls: %s\n", &namgls [0][iglstw]) ;
	  printf ("namgls [0][0]: %s\n", &namgls [0][0]) ;
	  printf ("namgls [1][0]: %s\n", &namgls [1][0]) ;
	  printf ("namgls [2][0]: %s\n", &namgls [2][0]) ;
	  printf ("namgls [3][0]: %s\n", &namgls [3][0]) ;
	  }

    if (fits_read_key (fp, TINT, "CMPOS", &cmpos, comment, &status))
	printerror     (status) ;
    if (pmode)
	  {
      printf ("cmpos: %d\n", cmpos) ;
	  printf ("namcm: %s\n", &namcm [0][cmpos]) ;
	  printf ("namcm [0][0]: %s\n", &namcm [0][0]) ;
	  printf ("namcm [1][0]: %s\n", &namcm [1][0]) ;
	  printf ("namcm [2][0]: %s\n", &namcm [2][0]) ;
	  printf ("namcm [3][0]: %s\n", &namcm [3][0]) ;
	  }


    if (fits_read_key (fp, TINT, "FWPOS", &fwpos, comment, &status))
	printerror     (status) ;

    strcpy (namwave, &namfw [fwpos-1][0]) ;
    if (fwpos == 3 && iglstw == 1)  strcpy (namwave, "GL") ;
    if (pmode)
	  {
	  printf ("fwpos       : %1d   iglstw:  %1d\n", fwpos, iglstw) ;
	  printf ("namfw:        %s\n", &namfw [fwpos-1][0]) ;
	  printf ("namfw [0][0]: %s\n", &namfw [0][0]) ;
	  printf ("namfw [1][0]: %s\n", &namfw [1][0]) ;
	  printf ("namfw [2][0]: %s\n", &namfw [2][0]) ;
	  printf ("namfw [3][0]: %s\n", &namfw [3][0]) ;
	  printf ("namfw [4][0]: %s\n", &namfw [4][0]) ;
	  printf ("namfw [5][0]: %s\n", &namfw [5][0]) ;
	  printf ("namwave:      %s\n",  namwave) ;
	  }


    if (fits_read_key (fp, TINT, "LFC", &lfc, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "LREG", &lreg, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "LVIG", &lvig, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "LSTR", &lstr, comment, &status))
	printerror     (status) ;

    if (fits_read_key (fp, TINT, "LCC", &lcc, comment, &status))
	printerror     (status) ;

	namcc  = '_' ;
	namstr = '_' ;
	namvig = '_' ;
    namfc  = '_' ;
	namreg = '_' ;
	if (lcc  == 1) namcc  = 'c' ;
	if (lstr == 1) namstr = 's' ;
	if (lvig == 1) namvig = 'v' ;
    if (lfc  == 1) namfc  = 'f' ;
	if (lreg == 1) namreg = 'r' ;

    if (pmode)
      fprintf (stdout, "[2] xdim/bitpix: %d %d\n", xdim, bitpix) ;

    if (fits_close_file (fp, &status))  printerror (status) ;

    /*** Skip a line if abs (time2 - time1) > 40. ***/

    if (iyear/4 * 4 == iyear)  ndoyyr = 366;  else ndoyyr = 365;
    time2 = (int) ((iyear - 1980) * ndoyyr + idoy * 1440 +
			ihour * 60 + iminute);
    timedif = abs (time2 - time1) ;
    if ((timedif) > 40 && imgnum != 0) fprintf (stdout," \n");

    if (pmode)
        printf ("time1: %d   time2: %d  timedif: %d  imgnum: %d\n",
	         time1, time2, timedif, imgnum) ;
    imgnum += 1 ;
    time1 = time2;

    fprintf (stdout,"%s %3d %4d %2d %2d  %1s %4d %c %2s %2s %2s %2s %4d\n",
    filename, quantity, frame, ih, iv, res, expose, nampei, namwave, &nampw [pwpos][0], sector, &namcm [cmpos][0], scroll);

    return (0) ;
}
