/* fcpquan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Print the FITS file name and quantity.
 * Requires an image name, a directory, or a file containing a list of
 * SMM C/P image file names.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P
 * Andrew L. Stanger
 * UNIX 4.2 BSD   C programming language
 * 25 Oct 1984
 * 18 Dec 1984 : Add Roll.
 * 22 Apr 1985: Change housekeeping structure.  Add filename to output line.
 *  4 Jan 1992: write delimiter line when time gap is > 40 minutes.
 * 24 Jan 2006: Adapt for FITS file format.
 * 22 Feb 2006: Print out filename and quantity ONLY.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.h"

#define MAXHSK 224

int time1  = 0 ;
int time2  = 0 ;
int imgnum = 0 ;

main (argc, argv)

int argc;
char *argv [];

{
   extern int fcp_quan () ;

   static char	namlst   [80];	/* Name of file containing list of images. */
   static char	namimg   [80];	/* C/P image disk file name. */
   static char	lsdir    [80] = "ls ";
   static char	imdir    [80] = ".";
   static char	flag     [80];
   static char	filename [80];
   static char	name     [80];

   int		idot;
   int		i, lenfile;
   int		ndoyyr = 365;
   int		time1 = 0;
   int		time2 = 0;
   FILE		*LIST, *IMG, *DIR, *popen ();


   /*** 3 arguments: "fcplog -d directory"   or   "fcplog -f filelistname". ***/

   if (argc == 3)
      {
      strcpy (flag, argv [1]);
      switch (flag [1])
         {
         case 'd' :	strcpy (imdir, argv [2]);
			goto directory;
         case 'f' :	strcpy (namlst, argv [2]);
			goto file;
         default   :	fprintf (stderr, "Usage: cplog [image_name] [-d directory] [-f filelistname]\n");
         }
   }

   /*
    * 2 arguments: fcplog image_name.
    * Interpret housekeeping for a single image.
    */

   else if (argc == 2)
      {
/*
    fprintf (stdout, "   DATE      TIME   CSVFR  Q%% IMG  IH-IV RES EXP I FW GL PW SM CM ROLL\n") ;
    fprintf (stdout, "---------- -------- ----- --- ---- -- -- --- --- - -- -- -- -- -- ----\n") ;
 */

      fcp_quan (argv [1]);
      exit (0);
      }

   /*
    * 1 argument: "fcplog".
    *  By default, interpret contents of current directory.
    */

   else if (argc == 1)
      {
      fprintf (stderr, "Usage: fcplog [image_name] [-d directory] [-f filelistname]\n");
      strcpy (imdir, ".");
      }

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/

directory:

   strcat (lsdir, imdir);

   /*** Get directory. ***/

   DIR = popen (lsdir,"r");

/*
    fprintf (stdout, "   DATE      TIME   CSVFR  Q%% IMG  IH-IV RES EXP I FW GL PW SM CM ROLL\n") ;
    fprintf (stdout, "---------- -------- ----- --- ---- -- -- --- --- - -- -- -- -- -- ----\n") ;
 */

   /*** Start going through data. ***/

   imgnum = 0;
   while (fscanf (DIR, "%s", filename) != EOF)
      {
      lenfile = strlen (filename);
      for (i = 0; i < lenfile; i++) if (filename [i] == '.')  idot = i;
      if (idot < 0 || (lenfile <= idot+2) ||
         (filename [idot+1] != 'v' && filename [idot+1] != 'c'))
         {
         fprintf (stderr, "Invalid file name: %s\n", filename);
         goto loop;
         }
      sprintf (name, "%s/%s", imdir, filename);	/* Add directory. */

      /*** Display housekeeping summary on terminal screen. ***/

      fcp_quan (name);

loop:
      i++;
      }
   exit (0);

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/

file:

   /*** Open file containing list of SMM C/P images. ***/

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Cannot open %s\n", namlst);
      exit (-2);
      }

/*
    fprintf (stdout, "   DATE      TIME   CSVFR  Q%% IMG  IH-IV RES EXP I FW GL PW SM CM ROLL\n") ;
    fprintf (stdout, "---------- -------- ----- --- ---- -- -- --- --- - -- -- -- -- -- ----\n") ;
 */

   /*** Read list of images; process one image at a time. ***/

   imgnum = 0;
   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s\n", namimg);
         exit (-6);
         }
      fclose (IMG);

      /*** Display housekeeping summary on terminal screen. ***/

      fcp_quan (namimg);
      }
   fclose (LIST);
   exit (0);
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * E R R O R ( )	Print error message and exit.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
void 
error (str1, str2)

char *str1, *str2;
{
   fprintf (stderr, str1, str2);
   fprintf (stderr, "\n");
   exit (1);
}
