/* ftschk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Test libcfitsio.a
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   23 October 2001
 *  4 Mar 2002: add file loop.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "fitsio.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
    static char filename [80] ;
    static char cnaxis1 [80] ;
    static char cnaxis2 [80] ;
    static char comment [80] ;
    static char chd [1024] ;

    short *simg ;

    int hdunum ;
    int hdutype ;
    int status ;
    int xdim, ydim ;
    int bitpix ;

    fitsfile *fp ;


    status = 0 ;

    while (--argc)
	{
	strcpy (filename, argv [argc]) ;

        /*** Open input FITS file. ***/

        if ( fits_open_file (&fp, filename, READONLY, &status) )
	    printerror     ( status );

	printf ("\n") ;
        printf ("fits_open_file [%s]   status: %d\n", filename, status) ;

        if (fits_get_num_hdus (fp, &hdunum, &status)) printerror (status) ;

	printf     ("#hdus: %d\n", hdunum) ;

        if (fits_get_hdu_num (fp, &hdunum))   printerror (status) ;

        if (fits_get_hdu_type (fp, &hdutype, &status)) printerror (status) ;

        printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;

        /*** Retrieve keyword parameters in primary HDU. ***/

        if (fits_read_key (fp, TINT, "NAXIS1", &xdim, cnaxis1, &status))
	    printerror     (status) ;

        if (fits_read_key (fp, TINT, "NAXIS2", &ydim, cnaxis2, &status))
	    printerror     (status) ;

        if (fits_read_key (fp, TINT, "BITPIX", &bitpix, comment, &status))
	printerror     (status) ;

        fprintf (stdout, "[1] xdim/ydim/bitpix: %d %d %d\n",
		 xdim, ydim, bitpix) ;

        /*** Move to FITS extension HDU. ***/

        hdunum = 2 ;
        if (fits_movabs_hdu (fp, hdunum, &hdutype, &status))
	    printerror     (status) ;

        printf ("fits_movabs_hdu [2] . . . status: %d\n", status) ;

        if (fits_get_hdu_num  (fp, &hdunum))  printerror (status) ;

        if (fits_get_hdu_type (fp, &hdutype, &status)) printerror (status) ;

        printf ("hdunum: %d  hdutype: %d\n", hdunum, hdutype) ;

        /*** Retrieve keyword parameters in extension HDU. ***/

        if (fits_read_key (fp, TINT, "NAXIS1", &xdim, cnaxis1, &status))
	    printerror     (status) ;

        if (fits_read_key (fp, TINT, "BITPIX", &bitpix, comment, &status))
	    printerror     (status) ;

        fprintf (stdout, "[2] xdim/bitpix: %d %d\n", xdim, bitpix) ;

        if (fits_close_file (fp, &status))  printerror (status) ;
        }

    exit (0) ;
}
