/* rdcpimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array,
 * INCLUDING housekeeping and bitmap records.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   Solar Maximum Mission   Coronagraph/Polarimeter
 * 21 August 1985
 * 20 Sep 2002: Clear image array to zero before reading data.
 * 17 Mar 2006: Use "cphskp3.c".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/smmcplib.h"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char	*
rdcpimg (namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
   unsigned char *cpbuf;	/* Pointer to image array. */
   int  cpfd;			/* File descriptor for image disk file. */
   int  imgsiz;			/* Image size (bytes) (pixel data only). */
   int  i;
   int  nrechd;
   long nread;

   /*** Decommutate Housekeeping. ***/

   cphskp_v3 (namimg);	

   /*** Open image file. ***/

   if ((cpfd = open (namimg, 0)) < 0)
     {
     fprintf (stderr,"rdcpimg> Can't open file %s\n", namimg);
     return (NULL);
     }

   /*** Allocate storage in memory for image array. ***/

/*   fprintf (stdout, "rdcpimg> npix: %d   npixl: %d   nbypix: %d\n",
 *            npix, npixl, nbypix);
 */

   if (nbypix == 4)			/* Floating point images. */
      nrechd = 1;
   else
      nrechd = 2;
   imgsiz = (npixl * nbypix * (npixl + nrechd));

   if ((cpbuf = (unsigned char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr,"rdcpimg> malloc error.");
     exit (1);
     }

   /*** Clear image array to all zeroes. ***/

   for (i = 0; i < imgsiz; i++)  *(cpbuf + i) = 0 ;

   /*** Read houskeeping, bitmap & pixel data. ***/

   if ((nread = read (cpfd, cpbuf, imgsiz)) <= 0)
      {
      fprintf (stderr,"rdcpimg> image read error.\n");
      return (NULL);
      }

   /*** Reverse bytes for Vax-calibrated images. ***/

   if (idtype == 3 && bytord == 1)  srevbyt (cpbuf, nbypix, npixl*npixl);

   if (close (cpfd) == -1)
      {
      fprintf (stderr,"rdcpimg> file close error.%s\n", namimg);
      return (NULL);
      }

  return (cpbuf);
}
