/* rdfimg3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array,
 * INCLUDING housekeeping and bitmap records.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   Solar Maximum Mission   Coronagraph/Polarimeter
 * 21 August 1985
 * 20 Sep 2002: Clear image array to zero before reading data.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char	*
rdfimg3 (namimg)

char  *namimg;			/* Pointer to image disk file name. */

{
   extern int cphskp3 () ;	/* SMM C/P housekeeping decoder. */
   unsigned char *cpbuf;	/* Pointer to image array. */
   int  cpfd;			/* File descriptor for image disk file. */
   int  imgsiz;			/* Image size (bytes) (pixel data only). */
   long nread;
   int i;
   int nrechd;

   /*** Decommutate Housekeeping. ***/

   cphskp3 (namimg);	

   /*** Open image file. ***/

   if ((cpfd = open (namimg, 0)) < 0)
     {
     fprintf (stderr,"rdfimg> Can't open file %s\n", namimg);
     return (NULL);
     }

   /*** Allocate storage in memory for image array. ***/

   fprintf (stdout, "rdfimg> npix: %d   npixl: %d   nbypix: %d\n",
            npix, npixl, nbypix);


   if (nbypix == 4)
      nrechd = 1;
   else
      nrechd = 2;
   imgsiz = (npixl * nbypix * (npixl + nrechd));

   if ((cpbuf = (unsigned char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr,"rdfimg> malloc error.");
     exit (1);
     }

   /*** Clear image array to all zeroes. ***/

   for (i = 0; i < imgsiz; i++)  *(cpbuf + i) = 0 ;

   /*** Read houskeeping, bitmap & pixel data. ***/

   if ((nread = read (cpfd, cpbuf, imgsiz)) <= 0)
      {
      fprintf (stderr, "rdfimg> image read error.\n");
      fprintf (stderr, "rdfimg> nread: %d\n", nread) ;
      return (NULL);
      }

   /*** Reverse bytes for Vax-calibrated images. ***/

   if (idtype == 3 && bytord == 1)  srevbyt (cpbuf, nbypix, npixl*npixl);

   if (close (cpfd) == -1)
      {
      fprintf (stderr,"rdfimg> file close error.%s\n", namimg);
      return (NULL);
      }

  return (cpbuf);
}
