/* smmfitsname.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Create FITS filename from a native file.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   HAO/NCAR    6 Jan 2006
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#include <stdio.h>
#define START
#include "/home/stanger/lib/include/smmcplib.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern int cphskp3 () ;
   extern unsigned char *rdfimg () ;

   int i;
   int iyear ;

   unsigned char *cpbuf ;               /* SMM C/P file  array pointer. */

   static char namimg [80] ;
   static char namfit [80] ;
   static char namsrc   [4] = {'u', '1', '3', '2'} ;	/* Polaroid Filter. */
   static char namwave  [4] ;
   static char namfw    [6] = {'r', 'a', 'o', 'b', 'w', 'g'};
   static char namfil [6][12] = {"RED", "H-ALPHA", "OPEN", "BLUE", "WIDEBAND",
				 "GREEN"} ;
   static char namcm  [4][8]  = {"MOVE", "OUT", "IN", "BAD"} ;
   static char namgls [4][8]  = {"MOVE", "FE-XIV", "STOW", "FAIL"} ;
   static char namobj [4][8]  = {"UNKNOWN", "CORONA", "CALIB", "UNKNOWN"};
   static char nampol [4][8]  = {"CLEAR", "+60 DEG", "-60 DEG", "  0 DEG"} ;
/*   static char nampol [4][8]  = {"P0", "P1", "P3", "P2"} ; */
   static char namsec [9][3] = {"SW","S", "SE","E", "NE","N", "NW","W", "4Q"} ;

   static char namsmf  [9][3] = {"sw","ss","se","ee","ne","nn","nw","ww","4q"} ;
   static char namsmhi [9][3] = {"SW","SS","SE","EE","NE","NN","NW","WW","4Q"} ;
   static char namtype  [ 4] = {'v', 't', 'c', '\0'} ;
   static char namredlo [ 4] = {'r', 'e', 'i', 'x'} ;
   static char namredhi [ 4] = {'R', 'E', 'I', 'X'} ;
   static char namreduce [16] = {'r', 'e', 'c', 'n', 'd', 'k', 'h', 'g', 'f', 'p', 's', 't', 'r', 'b', 'q', 'm'} ;
   static char namcor   [ 8];
   static char reduce   [16] ;

   strcpy (namimg, argv [1]) ;

/*   printf ("namimg: %s\n", namimg) ; */

   cpbuf = rdfimg (namimg) ;            /* Read C/P image into memory. */

   cphskp3 (namimg) ;                   /* Decode ALL HK items. */

/*   printf ("H.year: %d  H.month: %d  H.day: %d\n", H.year, H.month, H.day) ;*/

   /*** Create output FITS file name. ***/

   if (H.year < 1900)
      iyear = 1900 + H.year ;
   else
      iyear = H.year ;

   /*** Set namwave field to "i" if FW=open and GL is in optical path. ***/

   strcpy (namwave, &namfw [H.ifw]);
   if (H.ifw == 2 && H.iglstw == 1)
      {
      strcpy (namwave, "i");
      }
/*   printf ("H.ifw: %d  namwave: %s\n", H.ifw, namwave) ; */

   /*** Create the "corrections" string. ***/

   sprintf (namcor, "%s", "xxxx");
   if (H.cc  == 1)  namcor [0] = 'c';
   if (H.str == 1)  namcor [1] = 's';
   if (H.vig == 1)  namcor [2] = 'v';
   if (H.f   == 1)  namcor [3] = 'f';

   for (i = 0; i < 16; i++)  reduce [i] = namreduce [i] ;
   if (H.cc == 1 && H.str == 0 && H.vig == 0 && H.f == 0)  reduce [2] = 'j' ;
   if (H.cc == 1 && H.str == 1 && H.vig == 0 && H.f == 0)  reduce [2] = 'i' ;
   if (H.cc == 1 && H.str == 1 && H.vig == 1 && H.f == 0)  reduce [2] = 'b' ;
   if (H.cc == 1 && H.str == 1 && H.vig == 1 && H.f == 1)  reduce [2] = 'k' ;

   /*** Create FITS file name. ***/

   if (idtype > 0 & idtype < 17)
      {
      if (H.ires == 1)
         sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.fts",
	          iyear, H.month, H.day, H.hour, H.minute, H.second,
	          namwave [0], namsrc [H.ipw], reduce [H.idtype-1],
	          &namsmf [H.ism][0]);
      if (H.ires == 0)
         sprintf (namfit, "%04d%02d%02d.%02d%02d%02d.smmcp.%c%c%c.%s.fts",
	          iyear, H.month, H.day, H.hour, H.minute, H.second,
	          namwave [0], namsrc [H.ipw], reduce [H.idtype-1],
	          &namsmhi [H.ism][0]);
      }
   else
      sprintf (namfit, "smmcp.%s.fts", namimg) ;

   printf ("%s\n", namfit) ;

 return (0) ;
 }
