/* chsm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Change sector mirror position of input image in housekeeping record.
 * Read files of images, change sector telemetry in housekeeping,
 * and output new file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adapted by Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 28 November 1984
 * 6 February 1985: upgrade to handle all image sizes/resolutions, cal or raw.
 * 7 November 1985: Allow user to specify any sector position.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define  U	"./"
#include <stdio.h>
#include "/local/home/stanger/lib/include/cplib.h"

/* Define global variable storage. */

FILE	*Names ;

/* Standard disk centers (low resolution (n,e,s,w). */

int	cx [] =	{357, 407, 111,  57} ;
int	cy [] =	{336,  43, 103, 393} ;

int	cxn, cyn, cxe, cye, cxs, cys, cxw, cyw ;

char	smid [4] ;
char	smtab [9][3] = {"sw", "ss", "se", "ee", "ne", "nn", "nw", "ww", "4q"} ;
int	smhk [9]     = {004,   014,  024,  034,  044,  054,  064,  074,  0} ;
int	masksm ;
int	infd, outfd ;


main ()
{
   /* Declare variables. */

   extern	char	*malloc() ;	/* System storage allocation routine. */
   extern  char    *outname() ;	/* Convert input name to output name.*/
   extern	FILE	*Image ;
   char	*img ;
   static	char	f1[80], F1[80] ;
   static	char	f5[80], F5[80] ;
   char	*hk ;
   int	hk14 ;
   register int i ;
   int	nread, nwrite ;
   int	imgsiz ;

   printf ("chsm\n V01.00\n") ;

   printf ("Enter file containing image names\n>") ;
   scanf ("%s", f1) ;

   /* Open file containing list of image file names. */

   if ((Names = fopen (f1, "r"))==NULL)
      {
      fprintf (stderr, "cannot open %s\n", f1) ;
      exit (1) ;
      }

   /* Read filenames: one file name per line. */

   while (fscanf (Names, "%s %s", f1,  smid) != EOF)
      {

      /* Append directory pathname onto filenames. */

      sprintf (F1, "%s%s", U, f1) ;

      /* Open input file. */

      if ((infd = open (F1,  0)) <= 0)
         {
         fprintf (stderr,  "Unable to open file %s\n",  F1) ;
         exit (-1) ;
         }

      /* Open output file. */

      strcpy (f5,  outname (f1)) ;
      sprintf (F5, "%s%s", U, f5) ;
      printf ("%s\n", F5) ;
      if ((outfd = creat (F5, 0644)) <= 0)
         {
         fprintf (stderr, "Cannot create %s\n", F5) ;
         exit (-2) ;
         }

      /* Read Housekeeping. */

      cphskp (F1) ;
      printf ("npix: %d   nbypix: %d\n",  npix,  nbypix) ;

      imgsiz = (npix + 2) * npix * nbypix ;
      img = malloc (imgsiz) ;

      if ((nread = read (infd,  img,  imgsiz)) != imgsiz)
         {
         fprintf (stderr, "read: only %d bytes read\n", nread) ;
         exit (-3) ;
         }

      /* Change sector position (sm = 0 => NESW). */

      masksm = -1 ;
      for (i=0 ; i<9; i++)
         {
         printf ("smtab: %s %o\n",  &smtab[i][0],  &smtab[i][0]) ;
         if (strncmp (smid,  &smtab[i][0],  2) == 0) masksm = smhk[i] ;
         }
      printf ("i: %d  smid: %s  masksm: %di\n",  i,  smid,  masksm) ;
      if (masksm == -1)
         {
         fprintf (stderr,  "Invalid sector ID: %s\n",  smid) ;
         exit (-1) ;
         }
      hk = (char *) img ;
      hk14 = (int) hk[14] & 0377 ;
      printf ("hk14: %o\n",  hk14) ;
      hk14 = (int) hk[14] & 0303 ;
      hk14 |= masksm ;
      printf ("hk14: %o\n",  hk14) ;
      img[14] = (char) (hk14 & 0377) ;

      if ((nwrite = write (outfd,  img,  imgsiz)) != imgsiz)
         {
         fprintf (stderr,  "write: invalid # bytes written: %d\n", nwrite) ;
         exit (-4) ;
         }
      }
   close (infd) ;
   close (outfd) ;
   exit (0) ;
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
outname (namvid)

char *namvid ;

{
   int lenvid ;
   int idot ;
   int i ;
   char paknam [80] ;

   lenvid = strlen (namvid) ;
   idot = -1 ;
   for (i=0 ; i<lenvid; i++) 
      {if (namvid [i] == '.') idot = i ; paknam [i] = namvid [i];}
   if (idot == -1)
      {
      printf ("outname: invalid file name: %s\n",  namvid) ;
      exit (-1) ;
      }
   paknam [idot+2] = smid [0] ;
   paknam [idot+3] = smid [1] ;
   paknam [lenvid] = '\0' ;

   return (paknam) ;
}
