/* cproll.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Roll SMM C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   27 April 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/local/home/stanger/lib/include/cplib.h"

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern char *malloc () ;
   extern unsigned char *rdfimg () ;
   FILE *FD ;
   static char namimg [80] ;
   static char namout [80] ;
   static char namsm [8][4] = {"sw", "ss", "se", "ee", "ne", "nn", "nw", "ww"};
   unsigned char *imgbuf ;
   unsigned char *outbuf ;
   unsigned char  *cibuf ;
   unsigned char  *cobuf ;
   unsigned short *sibuf ;
   unsigned short *sobuf ;
   float          *fibuf ;
   float          *fobuf ;
   int iroll ;
   int ix, iy ;
   int ox, oy ;
   int nx, ny ;
   int maxx, maxy ;
   int nrechd, nbyhdr, nbyimg ;
   int osm, ksm ;
   int troll ;
   int roll1, roll2 ;
   float rollwt ;

   if (argc == 3)
      {
      strcpy (namimg, argv [1]) ;
      sscanf (argv [2], "%d", &iroll) ;
      }
   else
      {
      fprintf (stderr, "usage: %s {smmcp_filename | @smmcp_filelist} roll\n",
	       argv [0]);
      exit (0) ;
      }

   imgbuf = rdfimg (namimg) ;
   if (nbypix == 4)  nrechd = 1 ;
   else              nrechd = 2 ;
   maxx = maxy = npix - 1 ;
   nx = ny = npix ;
   nbyimg = nx * ny * nbypix ;
   outbuf = (unsigned char *) malloc (nbyimg) ;

   printf ("maxx/maxy: %3d %3d   nbyimg: %d   nx/ny: %3d %3d   nbypix: %d\n",
	   maxx, maxy, nbyimg, nx, ny, nbypix) ;

   nbyhdr = nrechd * npix * nbypix ;

   switch (nbypix)
      {
      case 1 :	cibuf = (unsigned char *)  (imgbuf + nbyhdr) ;
		cobuf = (unsigned char *)  outbuf ;		break ;
      case 2 :	sibuf = (unsigned short *) (imgbuf + nbyhdr) ;
		sobuf = (unsigned short *) outbuf ;		break ;
      case 4 :	fibuf = (float *)          (imgbuf + nbyhdr) ;
		fobuf = (float *)          outbuf ;		break ;
      default :	fprintf (stderr, "Invalid image . . . nbypix: %d\n", nbypix);
		exit (0);
      }

   /*** Adjust sector mirror designation. ***/

   switch (iroll)
      {
      case  90:	osm = H.ism - 2 ;		break ;
      case -90:	osm = H.ism + 2 ;		break ;
      case 180:	osm = H.ism + 4 ;		break ;
      default :	fprintf (stderr, "Invalid roll; must be 90, 180, or -90.\n");
		exit (0) ;
      }

   if (osm < 0)  osm += 8 ;
   if (osm > 7)  osm -= 8 ;
   ksm = osm * 2 + 1 ;
   *(imgbuf + 14) = *(imgbuf + 14) & 0303 ;		/* Clear sector bits. */
   *(imgbuf + 14) = *(imgbuf + 14) | (ksm << 2) ;	/* Set new sector. */

   /*** Adjust S/C roll. ***/

   scroll += iroll ;
   while (scroll <   0)  scroll += 360 ;
   while (scroll > 360)  scroll -= 360 ;

   if (H.year == 80)
      rollwt = 0.111905819 ;
   else
      rollwt = 7.161972438  ;

   troll = (int) ((float) scroll / rollwt) ;
   printf ("scroll: %4d  rollwt: %11.9f  troll: %4d\n", scroll, rollwt, troll) ;

   if (H.year == 80)
      {
      roll1 = (troll >> 8) & 017 ;
      roll2 = (troll & 0377) ;
      *(imgbuf + 11) = *(imgbuf + 11) & 0360 ;	/* Clear rollbits. */
      *(imgbuf + 11) = *(imgbuf + 11) | roll1 ;
      *(imgbuf + 12) = roll2 ;
      }
   else
      {
      roll1 = (troll >> 2) & 017 ;
      roll2 = (troll & 0003) << 6 ;
      *(imgbuf + 11) = *(imgbuf + 11) & 0360 ;	/* Clear rollbits. */
      *(imgbuf + 11) = *(imgbuf + 11) | roll1 ;
      *(imgbuf + 12) = *(imgbuf + 12) & 0077 ;	/* Clear rollbits. */
      *(imgbuf + 12) = *(imgbuf + 12) | roll2 ;
      printf ("roll1/2: %3o %3o   hk11/12: %3o %3o\n",
	       roll1, roll2, *(imgbuf + 11), *(imgbuf + 12)) ;
      }

   *(imgbuf + 88) = (scroll >> 8) & 0377 ;
   *(imgbuf + 89) =  scroll & 0377 ;

   sprintf (namout, "%s.%s", namimg, &namsm [osm][0]) ;

   if (iroll == 90)
      {
      if (nbypix == 1)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    ox = iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       oy = maxx - ix ;
	       *(cobuf + ox + oy * nx) = *(cibuf + ix + iy * nx) ;
	       }
	    }
         }
      else
      if (nbypix == 2)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    ox = iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       oy = maxx - ix ;
	       *(sobuf + ox + oy * nx) = *(sibuf + ix + iy * nx) ;
	       }
	    }
         }
      else
      if (nbypix == 4)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    ox = iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       oy = maxx - ix ;
	       *(fobuf + ox + oy * nx) = *(fibuf + ix + iy * nx) ;
	       }
	    }
         }
      }

   else
   if (iroll == -90)
      {
      if (nbypix == 1)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    ox = maxy - iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       oy = ix ;
	       *(cobuf + ox + oy * nx) = *(cibuf + ix + iy * nx) ;
	       }
	    }
         }
      else
      if (nbypix == 2)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    ox = maxy - iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       oy = ix ;
	       *(sobuf + ox + oy * nx) = *(sibuf + ix + iy * nx) ;
	       }
	    }
         }
      else
      if (nbypix == 4)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    ox = maxy - iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       oy = ix ;
	       *(fobuf + ox + oy * nx) = *(fibuf + ix + iy * nx) ;
	       }
	    }
         }
      }

   else
   if (iroll == 180)
      {
      if (nbypix == 1)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    oy = maxy - iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       ox = maxx - ix ;
	       *(cobuf + ox + oy * nx) = *(cibuf + ix + iy * nx) ;
	       }
	    }
         }
      else
      if (nbypix == 2)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    oy = maxy - iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       ox = maxx - ix ;
	       *(sobuf + ox + oy * nx) = *(sibuf + ix + iy * nx) ;
	       }
	    }
         }
      else
      if (nbypix == 4)
         {
         for (iy = 0; iy < nx ; iy++)
	    {
	    oy = maxy - iy ;
            for (ix = 0; ix < nx ; ix++)
	       {
	       ox = maxx - ix ;
	       *(fobuf + ox + oy * nx) = *(fibuf + ix + iy * nx) ;
	       }
	    }
         }
      }

   /*** Open output file. ***/

   if ((FD = fopen (namout, "w")) == NULL)
      {
      fprintf (stderr, "Unable to create file: %s\n", namout) ;
      exit (-1) ;
      }

   /*** Write image header & pixels. ***/

   fwrite (imgbuf, 1, nbyhdr, FD) ;

   fwrite (outbuf, 1, nbyimg, FD) ;

   fclose (FD) ;

   free (outbuf) ;

   exit (0) ;
}
