/* quad3.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Build a composite N+E+W image.
 * Read files of images, insert into packed file, and output file of packed
 * images.
 * Disk center position is required on input.
 * v1.00 for calibrated images (short not char) 6/1/84
 * Works only for low resolution images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * IDTYPE: data type.
 *	=1 raw vidicon data (DN units, 8 bits).
 *	=2 Equivalent seconds * 5, 8 bits.
 *	=3 Calibrated data (10^-11 B/Bsun units, 16 bits). 
 * Data type is inferred from the first letter of the filename extension.
 *
 * A C/P filename has the following format :
 * ydddhhmms.vxx :	type 1
 *          .txx :	     2
 *          .cxx :	     3
 * where y = year -1980.
 *       ddd = day-of-year, hh = hour, mm = minute, s = sec/10.
 *       x = arbitrary character.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adapted by Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 27 November 1984
 * 21 July 1987: Set # pixels per line in bytes 110 & 111 of housekeeping.
 * 10 September 1987
 * 18 November 1988: north + east + west only; no south sector.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include	<stdio.h>
#include	"/local/home/stanger/lib/include/cplib.h"
#define		U	"./"

/* Define global variable storage. */

int	idtype, nbypix;
char 	pakraw [896][896];
short	pakcal [896][896];
FILE	*Names,*Image;

main (argc, argv)

int argc;
char *argv [];
{
   /* Declare variables. */

   extern char    *outname();	/* Convert north name to output name.*/
   extern FILE	*Image;
   extern char 	pakraw [896][896];
   extern short	pakcal [896][896];
   static char	hskp [896], bitmap [896], htmp [896];
   static char	f1 [80], f2 [80], f3 [80], f4 [80];
   static char	F1 [80], F2 [80], F3 [80], F4 [80];
   static char	f5 [80], F5 [80];
   char	*hk;
   int	cxn, cyn, cxe, cye, cxs, cys, cxw, cyw;
   int	hk14;
   int	iext, lenimg;
   int	ires, jsm, ism;
   int	nbylin, nbyout;
   int	npixl = 896;
   int	nwrite;
   int	sec;
   register i, n, ret;

/* Disk centers (vs. Sector Mirror) for raw vidicon data (V34520). */
/* (No geometric correction applied.) */

static	float	vidcen [9][2] = { 36.0, 500.0,	/* sw */
				 214.0, 194.0,	/* s  */
				 540.0,   4.0,	/* se */
				 820.0,  88.0,	/* e  */
				 892.0, 362.0,	/* ne */
				 718.0, 680.0,	/* n  */
				 386.0, 856.0,	/* nw */
				 104.0, 792.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

/* Disk centers (vs. Sector Mirror) for calibrated data (C34520). */
/* (Processed with geometric correction WW6COR). */

static	float	ww6cen [9][2] = { 48.0, 508.0,	/* sw */
				 216.0, 206.0,	/* s  */
				 530.0,  40.0,	/* se */
				 814.0,  86.0,	/* e  */
				 884.0, 354.0,	/* ne */
				 714.0, 672.0,	/* n  */
				 394.0, 824.0,	/* nw */
				 114.0, 786.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

   printf ("%s: create composite image using 3 quadrants (n,e,w)\n", argv [0]);

        
   if (argc == 2)
      strcpy (f1, argv [1]);
   else
      {
      fprintf (stderr, "usage: %s [new_list]\n", argv [0]);
      fprintf (stderr, "Enter file containing image names : ");
      fscanf  (stdin,  "%s",f1);
      }

   /* Open file containing image names. */

   if ((Names = fopen (f1, "r")) == NULL)
      {
      fprintf (stderr, "cannot open %s\n", f1);
      exit (1);
      }

   /* Read filenames & disk centers; line order is : N,E,W. */

   while (fscanf (Names, "%s %d %d", f1, &cxn, &cyn) != EOF)
      {
      fscanf (Names,"%s %d %d", f2, &cxe, &cye);
      fscanf (Names,"%s %d %d", f4, &cxw, &cyw);
      sec = 0;						/*sector counter. */

      /* Append directory pathname onto filenames. */

      sprintf (F1, "%s%s", U, f1);
      sprintf (F2, "%s%s", U, f2);
      sprintf (F4, "%s%s", U, f4);

      /* Open North image, store houskeeping and bitmap, and insert. */

      printf ("%s\n", F1);
      cphskp (F1);
      if ((Image = fopen (F1, "r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s\n", F1);
         exit (1);
         }

      /* Determine Data type. */

      lenimg = strlen (F1);
      iext = -1;
      for (i = 0; i < lenimg; i++) if (F1 [i] == '.') iext = i + 1;
      idtype = 3;
      if (F1 [iext] == 'v' || F1 [iext] == 'V')  idtype = 1;
      if (F1 [iext] == 't' || F1 [iext] == 'T')  idtype = 2;
      nbypix = 2;
      if (idtype == 1 || idtype == 2)  nbypix = 1;
      nbylin = 448 * nbypix;

      if ((ret = read (fileno (Image), hskp, nbylin)) != nbylin)
         {
         if (ret == 0)
            fprintf(stderr, "Unexpected EOF (hskp) on %s\n", F1);
         else if (ret > 0)
           fprintf (stderr, "short read: only %d bytes read\n", ret);
         else
            perror ("read");
         exit (1);
         }
      ires = (*(hskp+3) & 04) >> 2;
      if (ires == 0)
         { printf ("High Resolution not accomodated\n"); exit (-1); }
      jsm = (*(hskp+14) & 074) >> 2;
      ism = (jsm-1)/2;			/* Decode to array position. */
      if (jsm == 0)  ism = 8;		/* Special case for NESW. */

      if ((ret = read (fileno (Image), bitmap, nbylin)) != nbylin)
         {
         if (ret == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", F1);
         else if (ret > 0)
           fprintf (stderr,"short read: only %d bytes read\n", ret);
         else
            perror ("read");
         exit (1);
         }
      if (idtype == 1)
         {
         if (cxn == 0)  cxn = vidcen [ism][0];
         if (cyn == 0)  cyn = vidcen [ism][1];
         }
      else
         {
         if (cxn == 0)  cxn = ww6cen [ism][0];
         if (cyn == 0)  cyn = ww6cen [ism][1];
         }
      cxn = (int) (cxn * 0.5);
      cyn = (int) (cyn * 0.5);
      quadin (cxn,cyn);
      sec++;
      fclose (Image);

      /* Reopen with east. */

      printf ("%s\n", F2);
      cphskp (F2);
      if ((Image = fopen (F2, "r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s\n", F2);
         exit (1);
         }

      /* Determine Data type. */

      lenimg = strlen (F2);
      iext = -1;
      for (i = 0; i < lenimg; i++) if (F2 [i] == '.') iext = i + 1;
      idtype = 3;
      if (F2 [iext] == 'v' || F2 [iext] == 'V')  idtype = 1;
      if (F2 [iext] == 't' || F2 [iext] == 'T')  idtype = 2;
      nbypix = 2;
      if (idtype == 1 || idtype == 2)  nbypix = 1;
      nbylin = 448 * nbypix;

      if ((ret = read (fileno (Image), htmp, nbylin)) != nbylin)
         {
         if (ret == 0)
            fprintf (stderr, "Unexpected EOF (htmp) on %s\n", F2);
         else if (ret > 0)
            fprintf (stderr, "short read: only %d bytes read\n", ret);
         else
            perror ("read");
         exit (1);
         }
      ires = (*(htmp+3) & 04) >> 2;
      if (ires == 0)
         { printf ("High Resolution not accomodated\n"); exit (-1); }
      jsm = (*(htmp+14) & 074) >> 2;
      ism = (jsm-1)/2;			/* Decode to array position. */
      if (jsm == 0)  ism = 8;		/* Special case for NESW. */

      if ((ret = read (fileno (Image), htmp, nbylin)) != nbylin)
         {
         if (ret == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", F2);
         else if (ret > 0)
            fprintf (stderr, "short read: only %d bytes read\n", ret);
         else
            perror ("read");
         exit (1);
         }
      if (idtype == 1)
         {
         if (cxe == 0)  cxe = vidcen [ism][0];
         if (cye == 0)  cye = vidcen [ism][1];
         }
      else
         {
         if (cxe == 0)  cxe = ww6cen [ism][0];
         if (cye == 0)  cye = ww6cen [ism][1];
         }
      cxe = (int) (cxe * 0.5);
      cye = (int) (cye * 0.5);
      quadin (cxe,cye);
      sec++;
      fclose (Image);

      /* Reopen with west. */

      printf ("%s\n", F4);
      cphskp (F4);
      if ((Image = fopen (F4, "r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s\n", F4);
         exit (1);
         }

      /* Determine Data type. */

      lenimg = strlen (F4);
      iext = -1;
      for (i = 0; i < lenimg; i++) if (F4 [i] == '.') iext = i + 1;
      idtype = 3;
      if (F4 [iext] == 'v' || F4 [iext] == 'V')  idtype = 1;
      if (F4 [iext] == 't' || F4 [iext] == 'T')  idtype = 2;
      nbypix = 2;
      if (idtype == 1 || idtype == 2)  nbypix = 1;
      nbylin = 448 * nbypix;

      if ((ret = read (fileno (Image), htmp, nbylin)) != nbylin)
         {
         if (ret == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", F4);
         else if (ret > 0)
           fprintf (stderr, "short read: only %d bytes read\n", ret);
         else
            perror ("read");
         exit (1);
         }
      ires = (*(htmp+3) & 04) >> 2;
      if (ires == 0)
         { printf ("High Resolution not accomodated\n"); exit (-1); }
      jsm = (*(htmp+14) & 074) >> 2;
      ism = (jsm-1)/2;			/* Decode to array position. */
      if (jsm == 0)  ism = 8;		/* Special case for NESW. */

      if ((ret = read (fileno (Image), htmp, nbylin)) != nbylin)
         {
         if (ret == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", F4);
         else if (ret > 0)
            fprintf (stderr, "short read: only %d bytes read\n", ret);
         else
            perror ("read");
         exit (1);
         }
      if (idtype == 1)
         {
         if (cxw == 0)  cxw = vidcen [ism][0];
         if (cyw == 0)  cyw = vidcen [ism][1];
         }
      else
         {
         if (cxw == 0)  cxw = ww6cen [ism][0];
         if (cyw == 0)  cyw = ww6cen [ism][1];
         }
      cxw = (int) (cxw * 0.5);
      cyw = (int) (cyw * 0.5);
      quadin (cxw,cyw);
      sec++;
      fclose (Image);

      /* Output packed image. */

over:

      /* First set hskp to high resolution. */

      hskp [3] = hskp [3] & 0373;

      /* Change sector position (sm = 0 => NESW). */

      hk = (char *) hskp;
      hk14 = (int) hk [14] & 0377;
      printf ("hk14: %o\n", hk14);
      hk14 = (int) hk [14] & 0303;
      printf ("hk14: %o\n", hk14);
      hskp [14] = (char) (hk14 & 0377);

      /* Set # pixels per line to 896. */

      hskp [110] = (char) ((npixl & 0177400) >> 8);
      hskp [111] = (char) (npixl & 0377);

      /* Write housekeeping & bitmap. */

      strcpy (f5, outname (f1));
      sprintf (F5, "%s%s", U, f5);
      printf ("%s\n", F5);
      if ((Image = fopen (F5, "w")) == NULL)
         {
	 fprintf (stderr, "Cannot create %s\n", F5);
         exit (1);
         }
      if (fwrite (hskp, nbypix, npixl, Image) != npixl)
         {
         perror ("fwrite");
         exit (1);
         }
      if (fwrite (bitmap, nbypix, npixl, Image) != npixl)
         {
         perror ("fwrite");
         exit (1);
         }

      /* Write out image data. */

      n = 896*896;
      if (nbypix == 1) nwrite = fwrite ((char *)  pakraw, 1, n, Image);
      if (nbypix == 2) nwrite = fwrite ((short *) pakcal, 2, n, Image);
      if (nwrite != n)
         {
         perror ("fwrite (packimg)");
         exit (1);
         }
      fclose (Image);
      }		/*end of while Names	*/
   exit (0);
}


/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
outname (namvid)

char *namvid;

{
   int lenvid;
   int idot;
   int i;
   char paknam [80];

   lenvid = strlen (namvid);
   idot = -1;
   for (i = 0; i < lenvid; i++) 
      { if (namvid [i] == '.') idot = i; paknam [i] = namvid [i]; }

   if (idot == -1)
      {
      printf ("outname: invalid file name: %s\n", namvid);
      exit (-1);
      }
   paknam [idot+2] = '3';
   paknam [idot+3] = 'q';
   paknam [lenvid] = '\0';

   return (paknam);
}
