/* quadin.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Routine to insert picture at xoff,yoff offset from 0,0 of big picture,
 * with center defined as cx,cy.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * nbypix: # bytes per pixel (=1 for raw images, =2 for calibrated images).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adapted & maintained by Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 30 December 1986: set yoff = 0.
 * 10 September 1987: Adapt for use with both raw & calibrated images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/local/home/stanger/lib/include/cplib.ext"

#define maxpix 896

quadin (cx, cy)

int cx, cy;

{
   extern  FILE	*Image;
   extern  char  pakraw [maxpix][maxpix];
   extern  short pakcal [maxpix][maxpix];
   extern int  nbypix;
   extern int  idtype;
   extern int  bytord;

   int	ix0, ix1;	/* X-axis start/stop limits. */
   int  iy0, iy1;	/* Y-axis start/stop limits. */
   int  ix, iy;		/* X & Y axis indices */
   int  xoff, yoff;	/* X & Y axis offsets. */
   int  nx;		/* # pixels per line. */
   int  nx2;		/* nx / 2 */
   short line [maxpix];
   char	*linbyt;

   nx = npixl;
   nx2 = nx / 2;
   linbyt = (char *) line;
   printf ("quadin> idtype: %d  bytord: %d\n", idtype, bytord);
   printf ("quadin> npixl:  %3d   nx2: %3d\n", npixl, nx2);

   /* Find limits of picture. */

   ix0 = (cy < nx2) ? cy  : 0;	/* if cy<224, start at center.   */
   ix1 = (cy < nx2) ? 447 : cy;	/* if cy<224, end at bottom.     */
   iy0 = (cx < nx2) ? cx  : 0;	/* if cx<224, start at center.   */
   iy1 = (cx < nx2) ? 447 : cx;	/* if cx<224, end at right edge. */

   xoff = (cx > nx2) ? nx-cx-iy0 : nx-iy0;
   yoff = (cy > nx2) ? nx-cy-ix0 : nx-ix0;

   printf ("cx, cy=  %d, %d\n", cx, cy);
   printf ("ix0, ix1=  %d, %d\n", ix0, ix1);
   printf ("iy0, iy1=  %d, %d\n", iy0, iy1);
   printf ("xoff, yoff=  %d, %d\n", xoff, yoff);

   /* Read and insert. */

   if (nbypix == 1)	/* Raw images. */
      {
      for (iy = 0; iy < ix0; iy++)
         fread (linbyt, nbypix, nx, Image);   /* Skip to beginning ! */

      for (iy = ix0; iy <= ix1; iy++)
         {
         if (fread (linbyt, nbypix, nx, Image) != nx)
            {
            if (feof (Image))
               fprintf (stderr, "Unexpected EOF (file) iy:%d\n", iy);
            else
               perror ("fread");
            exit (1);
            }

         for (ix = iy0; ix < iy1; ix++) pakraw [yoff+iy][xoff+ix] = linbyt [ix];
         }
      }
   else			/* nbypix = 2  (Calibrated images). */
      {
      for (iy = 0; iy < ix0; iy++)
      fread (line, nbypix, nx, Image);   /* Skip to beginning ! */

      for (iy = ix0; iy <= ix1; iy++)
         {
         if (fread (line, nbypix, nx, Image) !=nx)
            {
            if (feof (Image))
               fprintf (stderr, "Unexpected EOF (file) iy:%d\n", iy);
	    else
	       perror ("fread");
	     exit (1);
             }

          /*** Reverse bytes for Vax-calibrated images. ***/

          if (idtype == 3 && bytord == 1)
          srevbyt (linbyt, nbypix, nx);
	  for (ix = iy0; ix < iy1; ix++) pakcal [yoff+iy][xoff+ix] = line [ix]; 
	  }
       }
   return (0);
}
