/* rotatec.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate a calibrated C/P image about standard disk center.
 * Low resolution, 6 Rsun images only.
 * Output file name is "yydddhhmm.rxx".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 29 January 1990
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#define	U "/d/stanger/slow/"
#define NPIX 448

static  short  cp [NPIX][NPIX];
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
main ()
{
   extern	unsigned short *rotatecp ();

   char	*outname();	/* Function: converts input name into output name. */
   FILE	*names;		/* File handle for list of images. */
   int	image;		/* File descriptor for input image. */
   unsigned short  *cpout;
   static  short  hskp [NPIX], bitmap [NPIX], tmp [NPIX];
   static  char	f1 [80], f2 [80], F1 [80], F2 [80];
   char	*hk;

   register i, n, stat;
   int	nbypix = 2;	/* # bytes per pixel. */
   int	nbyimg;		/* # bytes per image. */
   int	np;		/* # bytes per line. */
   int	xmin, xmax;	/* minimum & maximum pixels for input image. */
   int	ymin, ymax;	/* minimum & maximum lines for input image. */
   int	x, y;		/* coordinates for input image. */
   int	xx, yy;		/* coordinates for output image. */
   int	dx, dy;		/* Delta x & y (amount of image shift. */
   int	nread;		/* # bytes read from disk. */
   int	ism;		/* Sector mirror array index. */
   int	jsm;		/* Sector mirror telemetry. */

   /* Disk centers (vs. Sector Mirror) for calibrated data (C34520). */
   /* (Processed with geometric correction WW6COR). */

static	float	ww6cen [9][2] = { 48.0, 508.0,	/* sw */
				 216.0, 206.0,	/* s  */
				 530.0,  40.0,	/* se */
				 814.0,  86.0,	/* e  */
				 884.0, 354.0,	/* ne */
				 714.0, 672.0,	/* n  */
				 394.0, 824.0,	/* nw */
				 114.0, 786.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

   int	cx = 0;		/* Standard X-coordinate for center. */
   int	cy = 0;		/* Standard Y-coordinate for center. */

   int	cxm = 0;	/* Measured X-coordinate for center. */
   int	cym = 0;	/* Measured Y-coordinate for center. */
   int	pmin;		/* Minimum for pixel/line coordinate. */
   int	pmax;		/* Maximum for pixel/line coordinate. */

   float	angle = 0.0;	/* Number of degrees to rotate image. */

   hk = (char *) hskp;
   pmin = 0;
   pmax = NPIX - 1;

   printf ("rotatec\n V01.00\n");

   printf ("Enter file containing list of image names: ");
   scanf ("%s", f1);

   /* Open file containing list of images. */

   if ((names = fopen (f1, "r")) == NULL)
      {
      fprintf(stderr, "Cannot open %s\n",f1);
      exit (1);
      }

   /* Read filenames: order per line is : image, angle (degrees). */

   while (fscanf (names,"%s %f", f1, &angle) != EOF)
      {

      /* Append directory pathname onto filenames. */

/*	sprintf (F1,"%s%s",U,f1); */

      sprintf (F1, "%s", f1);

      np = NPIX * nbypix;
      nbyimg = NPIX * NPIX * nbypix;

      /* Open image file. */

      printf ("Image: %s\n", F1);
      if ((image = open (F1, 0)) < 0)
         {
         fprintf (stderr,"Cannot open %s\n",F1);
         exit (1);
         }

      if ((stat = read (image, hskp, np)) != np)
         {
         if (stat == 0)
            fprintf (stderr,"Unexpected EOF (bitmap) on %s\n",F1);
         else if (stat > 0)
            fprintf (stderr,"read: only %d bytes read\n", stat);
         else
            perror ("read");
         exit (1);
         }

      if ((stat = read (image, bitmap, np)) != np)
         {
         if (stat == 0)
            fprintf (stderr, "Unexpected EOF (bitmap) on %s\n", F1);
         else if (stat > 0)
            fprintf (stderr, "read: only %d bytes read\n", stat);
         else
            perror ("read");
         exit (1);
         }

      /* Decode sector position. */

      hk = (char *) hskp;
      jsm = (*(hk + 14) & 074) >> 2;
      ism = (jsm - 1) / 2;
      if (jsm == 0)  ism = 8;
      cx = (int) ((ww6cen [ism][0] + 0.5) * 0.5);
      cy = (int) ((ww6cen [ism][1] + 0.5) * 0.5);
      printf ("ism: %d\n", ism);
      printf ("cx:%d cy:%d cxm:%d cym:%d\n", cx, cy, cxm, cym);

      /* Read image into memory array 'cp'. */

      nread = read (image, cp, nbyimg);

      dx = dy = NPIX;
      cpout = rotatecp (cp, dx, dy, cx, cy, angle);

      close (image);

      /* Output rotated image. */

over:
      strcpy (F2, outname (F1));
      printf ("shifted image: %s\n", F2);
      if ((image = creat (F2, 0664)) < 0)
         {
         fprintf (stderr, "Cannot create %s\n", F2);
         exit (1);
         }

      if (write (image, hskp, np) != np)
         {
         perror ("write");
         exit (1);
         }
      for (i=0; i<3; i++)
         if (write (image, bitmap, np) != np)
            {
            perror ("write");
            exit (1);
            }

      n = NPIX*NPIX*nbypix;
      if (write (image, cpout, n) != n)
         {
         perror ("write (cpout)");
         exit (1);
         }
      close (image);
      }						/*end of while names	*/
   exit (0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
outname (namvid)

char *namvid;

{
int lenvid;
int idot;
int i;
char paknam [80];

   lenvid = strlen (namvid);
   idot = -1;
   for (i = 0; i < lenvid; i++) 
      {if(namvid [i] == '.') idot = i; paknam [i] = namvid [i];}
   if (idot == -1)
      {
      printf ("outname: invalid file name: %s\n", namvid);
      exit (-1);
      }
   paknam [idot + 1] = 'r';
   paknam [lenvid] = '\0';

   return (paknam);
}

/* rotatecp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an unsigned short integer image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rose Reynolds
 * Adapted by Andrew Stanger
 * 22 September 1987
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* #include "/u/rose/grin2/grin.h" */
#include <math.h>

unsigned short *
rotatecp (oldimg, dx, dy, cx, cy, angdeg)

unsigned short *oldimg;	/* Input image array pointer. */
int dx, dy;		/* Dimensions of image. */
int cx, cy;		/* Center of rotation. */
float angdeg;		/* Rotation angle in degrees. */

{
   extern char *malloc();
   register int nx, ny, ox, oy, xdim, ydim;
   float fox, foy, cose, sine, fnx, fny, fnysin, fnycos;
   float fcx, fcy;
   double angle;
   unsigned short *newimg;
   register unsigned short *rold, *rnew;

   xdim = dx;   ydim = dy;
   fcx = (float) cx;   fcy = (float) cy;
   angle = (double) angdeg * (3.14159265/180.);/* convert deg to rads. */
   sine = (float) sin( angle );
   cose = (float) cos( angle );
printf ("angdeg: %6.2f   angle: %f   sine: %6.2f   cose: %6.2f\n",
         angdeg, angle, sine, cose);
   newimg = (unsigned short *) malloc( dx * dy * sizeof(short) );
   rold = oldimg;
   rnew = newimg;

   for( ny = 0; ny < ydim; ny++ )
      {
      fny = (float)(ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *rnew++ = 0;
         else *rnew++ = *(rold + (oy*xdim) + ox);
         }
      }
   return ( newimg );
}
