/* shiftv.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Shift raw C/P images by moving measured center to a "standard" center.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 19 November 1984 : Low resolution, 6 Rsun images only.
 * 25 November 1984 : Output filename is ydddhhmms(s).dxx.
 * 4 February 1985: Remove default pathname from in/out files.
 * 9 March 1988: modify to handle raw images instead of calibrated images.
 */

#include <stdio.h>
#define	U "/d/stanger/299/"
#define NPIX 448

static	char  cpout[NPIX][NPIX];
static  char  cp[NPIX][NPIX];
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
main()
{
char	*outname();	/* Function: converts input name into output name. */
FILE	*names;		/* File handle for list of images. */
int	image;		/* File descriptor for input image. */
extern	char  cpout[NPIX][NPIX];
extern  char  cp[NPIX][NPIX];
static  char  hskp [NPIX], bitmap[NPIX], tmp[NPIX];
static  char	f1[80], f2[80], F1[80], F2[80];
char	*hk;

register i, n, stat;
int	nbypix = 1;	/* # bytes per pixel. */
int	nbyimg;		/* # bytes per image. */
int	np;		/* # bytes per line. */
int	xmin, xmax;	/* minimum & maximum pixels for input image. */
int	ymin, ymax;	/* minimum & maximum lines for input image. */
int	x, y;		/* coordinates for input image. */
int	xx, yy;		/* coordinates for output image. */
int	dx, dy;		/* Delta x & y (amount of image shift. */
int	nread;		/* # bytes read from disk. */
int	ism;		/* Sector mirror array index. */

/* Disk centers (vs. Sector Mirror) for raw data (R34520). */

static	float	vidcen [9][2] = { 36.0, 500.0,	/* sw */
				 214.0, 194.0,	/* s  */
				 540.0,   4.0,	/* se */
				 820.0,  88.0,	/* e  */
				 892.0, 362.0,	/* ne */
				 718.0, 680.0,	/* n  */
				 386.0, 856.0,	/* nw */
				 104.0, 792.0,	/* w  */
				 447.5, 447.5};	/* n+e+s+w */

int	cx = 0;		/* Standard X-coordinate for center. */
int	cy = 0;		/* Standard Y-coordinate for center. */

int	cxm = 0;	/* Measured X-coordinate for center. */
int	cym = 0;	/* Measured Y-coordinate for center. */
int	pmin;		/* Minimum for pixel/line coordinate. */
int	pmax;		/* Maximum for pixel/line coordinate. */

	hk = (char *) hskp;
	pmin = 0;
	pmax = NPIX - 1;

	printf ("shiftv\n V01.00\n");

	printf ("Enter file containing list of image names: ");
	scanf ("%s", f1);

/* Open file containing list of images. */

	if((names = fopen(f1,"r")) == NULL)
		{
		fprintf(stderr,"Cannot open %s\n",f1);
		exit(1);
		}

/* Read filenames: order per line is : image, x-center, y-center. */

	while (fscanf (names,"%s %d %d", f1, &cxm, &cym) != EOF)
	{

/* Append directory pathname onto filenames. */

/*	sprintf (F1,"%s%s",U,f1); */
	sprintf (F1,"%s",f1);

	np = NPIX * nbypix;
	nbyimg = NPIX * NPIX * nbypix;


/* Open image file. */

	printf ("Image: %s\n", F1);
	if((image = open(F1, 0)) < 0)
		{
		fprintf(stderr,"Cannot open %s\n",F1);
		exit(1);
		}

	if((stat = read(image, hskp, np)) != np)
		{
		if(stat == 0)
		    fprintf(stderr,"Unexpected EOF (bitmap) on %s\n",F1);
		else if (stat > 0)
		    fprintf(stderr,"read: only %d bytes read\n", stat);
		else
		    perror("read");
		exit(1);
		}

	if((stat = read(image, bitmap, np)) != np)
		{
		if(stat == 0)
		    fprintf(stderr,"Unexpected EOF (bitmap) on %s\n",F1);
		else if (stat > 0)
		    fprintf(stderr,"read: only %d bytes read\n", stat);
		else
		    perror("read");
		exit(1);
		}

/* Decode sector position. */

	hk = (char *) hskp;
	ism = (*(hk+14) & 074)>>2;
	ism = (ism-1)/2;
	cx = (int) ((vidcen[ism][0]+0.5) * 0.5);
	cy = (int) ((vidcen[ism][1]+0.5) * 0.5);
	printf ("ism: %d\n", ism);
	printf ("cx:%d cy:%d cxm:%d cym:%d\n", cx, cy, cxm, cym);

/* Read image into memory array 'cp'. */

	nread = read (image, cp, nbyimg);

	xmin = pmin;
	xmax = pmax;
	ymin = pmin;
	ymax = pmax; 
	dx = cxm - cx;
	dy = cym - cy;
printf ("xmin: %d  xmax: %d  ymin: %d  ymax: %d\n", xmin, xmax, ymin, ymax);
printf ("dx: %d  dy: %d\n", dx, dy);

	for (yy = ymin; yy <= ymax; yy++)
		{
		y = yy + dy;
		for (xx = xmin; xx <= xmax; xx++)
			{
			x = xx + dx;
			if ((x < pmin || x > pmax) ||
			    (y < pmin || y > pmax))  cpout[yy][xx] = 0;
			else
				cpout[yy][xx] = cp[y][x];
			}
		}
	printf ("xx: %d  yy: %d\n", xx, yy);

	close(image);


/* Output shifted image. */

over:
	strcpy (F2, outname(F1));
	printf ("shifted image: %s\n", F2);
	if((image = creat(F2, 0664)) < 0)
		{
		fprintf(stderr,"Cannot create %s\n",F2);
		exit(1);
		}

	if(write(image, hskp, np) != np)
		{
		perror("write");
		exit(1);
		}
	for (i=0; i<3; i++)
		if(write(image, bitmap, np) != np)
			{
			perror("write");
			exit(1);
			}

		n = NPIX*NPIX*nbypix;
		if(write(image, cpout, n) != n)
			{
			perror("write (cpout)");
			exit(1);
			}
		close(image);
	}		/*end of while names	*/
	exit(0);
}
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
char *
outname (naminp)

char *naminp;

{
int leninp;
int idot;
int i;
static char namout[80];

   strcpy (namout, naminp);
   strcat (namout, "s");

   return (namout);
}
