;* ilscan.pro
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* Line Scan procedure.
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* Extracts values from an image along a line between two arbitrary end points.
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* Solar Maximum Mission   Coronagraph/Polarimeter
;* Andrew L. Stanger
;* 20 April 1984
;* 18 July 1984: add option to remove stray light & vignetting.
;* 31 July 1984: add option to remove F-corona; remove printf for scan values.
;* 7 February 1985: change double to float.
;* 7 November 1986: fix calibration correction logic.
;* 5 December 1986: use Saito F-corona.
;* 10 June 1991: adapt for SGI/IRIS GL windows.
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;#include <stdio.h>
;#include <math.h>

;#include "/usr/people/smm/lib/include/cplib.ext"
;#include "/usr/people/smm/lib/include/img_scale.ext"
;#include "/usr/people/smm/lib/include/ov_color.h"

;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

PRO ilscan, imgbuf, ip1, ir1, ip2, ir2, scntyp, scan, scandx, ns, ierr

COMMON cpgrin, igr, ngr, nres, imgch, ovrch, gres
COMMON cpgraf, igterm, lcc, lden, lfc, lstr, lsv, lvig, lreg, ldif, modeg, $
rawtocal
COMMON cpscale, pixrs,  rspix, degrad, raddeg
COMMON cpsize,  bytord, npixl, npix,   nbypix, ip, ir, lp, lr, idtype, nave, $
                icyear, icdoy, pscale
COMMON cpsun, scroll, degnor, xcen, ycen
COMMON img_format, gMINROW, gMAXROW, gMINCOL, gMAXCOL

;char	*imgbuf;		/* Pointer to 1 byte image array. */
;int	ip1, ir1, ip2, ir2;	/* Pixel,line coordinate scan limits. */
;int	*ns;			/* Number of values in scan. */
;int	*scntyp;		/* Scan type: radial=1 or line=2. */
;float	scan[];			/* Dependent axis scan array. */
;float	scandx[];		/* Independent axis scan array. */

;{

;extern float polcor;		/* Polaroid transmission correction factor. */
;extern float cpstray();		/* Stray Light function. */
;extern float vignet();		/* Vignetting function. */
;extern float fcorona();		/* Function to provide F-corona value. */
;extern float calcor();		/* Munro calibation correction factor. */
;short *ims;			/* Pointer to 2 byte image array. */
;float x, y, slope, yoff;
;float x1, x2, y1, y2;
;float dist, dstinc, dstsun;
;float r1, r2, th1, th2;
;float r, th;
;float pix1, rec1, pix2, rec2;
;float Cexp = 1.e-11;		/* Scaling factor for calibrated intensities. */
;float Bexp;
;float str, vig;		/* Storage for stray light & vignetting. */
;float cc;			/* Storage for calibration correction factor. */
;int i;
;int ierr;			/* Error. */
;int ival1, ival2;
;int ix, ixinc, iy, iyinc;
;int ix1, ix2, iy1, iy2;
;int ixg, iyg;
;int nx, nxdif;
;int ny, nyinc, nydif;
;
   MAXSCN = 3600
   cc = 1.0
   str = 0.0
   vig = 1.0

   IF (idtype EQ 3) THEN Bexp = Cexp $
                    ELSE Bexp = 1.0

;  *** Clear scan arrays. ***

;   FOR (i = 0, MAXSCN-1 DO
;      scandx (i) = 0.0
;      scan (i) = 0.
;   ENDFOR

;   *** Check scan size. ***

   ierr = 0

   ns = imax (abs (ip2-ip1), abs (ir2-ir1)) + 1

   IF (ns GT MAXSCN) THEN $
   BEGIN
      PRINT, "linscn> error...# scan values:", ns, " > MAXSCN:", MAXSCN
      ierr = -1
      RETURN
   END

;  *** Compute distance (in Rsun units) between scan values. ***
;  *   (line scans only) *

   IF (scntyp EQ 2) THEN $
   BEGIN
      d = float (ip2 - ip1) * (ip2 - ip1) + float (ir2 - ir1) * (ir2 - ir1)
      dist = sqrt (d)
;      dist = sqrt (float ((ip2-ip1)*(ip2-ip1)) + float ((ir2-ir1)*(ir2-ir1)))

      dstinc = dist / ((ns-1) * pixrs)
      dstsun = -dstinc
   END

;  *** Orient start/end points so that start point is closest to disk. ***
;  *   (radial scans only) *

   pix1 = float (ip1)
   rec1 = float (ir1)
   pix2 = float (ip2)
   rec2 = float (ir2)

   rcoord, r1, th1, pix1, rec1, -1, scroll, ierr
   IF (ierr NE 0) THEN RETURN
   rcoord, r2, th2, pix2, rec2, -1, scroll, ierr
   IF (ierr NE 0) THEN RETURN

;  *** Reverse start/end points. ***

   IF (r1 GT r2 AND scntyp EQ 1) THEN $
   BEGIN
      ix1 = ip2
      ix2 = ip1
      iy1 = ir2
      iy2 = ir1
   END $

   ELSE $

;  *** Leave start/end points as given. ***
 
   BEGIN
      ix1 = ip1
      ix2 = ip2
      iy1 = ir1
      iy2 = ir2
   END
      

;  *** Equation of a line:   y = slope * x + yoff. ***

   x1 = float (ix1)
   x2 = float (ix2)
   y1 = float (iy1)
   y2 = float (iy2)
   slope = -1.0e+9
   yoff = 0.0
   IF (ix1 NE ix2) THEN slope = (y2 - y1) / (x2 - x1)
   IF (ix1 NE ix2) THEN yoff  = (x2*y1 - x1*y2) / (x2 - x1)

;   printf ("ix1,iy1,ix2,iy2: %d %d %d %d\n", ix1, iy1, ix2, iy2)
;   printf ("slope,yoff: %f %f\n", slope, yoff)

   IF (abs (slope) LT 1.0) THEN $
   BEGIN
;  * printf("pixel-driven scan\n")

;     * ...... pixel-driven loop : *

      nxdif = ix2 - ix1
      IF (nxdif LT 0) THEN nxdif = -nxdif
      nx = nxdif + 1
      ix = ix1 - 1
      ixinc = 1
      scan   = fltarr (nx)
      scandx = fltarr (nx)

      IF (ix1 GT ix2) THEN $
      BEGIN
         ix = ix1 + 1
         ixinc = -1
      END

      FOR i = 0, nx-1 DO BEGIN
         ix = ix + ixinc
         x = float (ix)
         y = float (iy1)
         IF (iy1 NE iy2) THEN y = slope * x + yoff
         iy = long (y)

;        * 1 byte per pixel:
;        *  clear all high order bits above low-order byte.

         IF (nbypix EQ 1) THEN $
         BEGIN
            ival1 = imgbuf ( (ix-ip) , (iy-ir)  ) AND 0377
            ival2 = imgbuf ( (ix-ip) , (iy-ir+1)) AND 0377
         END  $
         ELSE $

;        *** 2 bytes per pixel. ***

         BEGIN
            ival1 = imgbuf ( (ix-ip) , (iy-ir)  ) AND 0177777
            ival2 = imgbuf ( (ix-ip) , (iy-ir+1)) AND 0177777
         END

;        *** Convert (x,y) coordinates into (r,th). ***

         rcoord, r, th, x, y, -1, scroll, ierr
         IF (ierr NE 0) THEN RETURN

;        *** Linear interpolation in vertical axis. ***

         IF (lcc EQ 1) THEN cc = calcor (r)
         scan (i) = ((ival1 + (ival2-ival1) * (y-iy)) * Bexp) * cc

;        *** Remove stray light & vignetting. ***

         IF (idtype EQ 3) THEN $
         BEGIN
	    IF (lstr EQ 1) THEN str = cpstray (r)
	    IF (lvig EQ 1) THEN vig = vignet (r)
	    scan (i) = (scan (i) * polcor - str) * vig
            IF (lfc EQ 1) THEN scan (i) = scan (i) - fcorona (r, th)
	 END

;        *** Determine independent axis coordinate value. ***

         dstsun = dstsun + dstinc
         IF (scntyp EQ 1) THEN scandx (i) = float (r)
         IF (scntyp EQ 2) THEN scandx (i) = float (dstsun)

;*         if (igterm EQ 0)
;*            {
;*	      if (idtype EQ 3)
;*	      printf ("ix:%3d iy:%3d r:%7.3f th:%7.2f length(Rsun):%7.3f Bsun:%9.2e\n",
;*                    ix, iy, r, th, dstsun, scan[i])
;*	      else
;*	      printf ("ix:%3d iy:%3d r:%7.3f th:%7.2f length(Rsun):%7.3f intensity:%7.2f\n",
;*                    ix, iy, r, th, dstsun, scan[i])
;*	      }

;        *** Plot 2 nearest points on IRIS in graphics overlay. ***

         IF (igr EQ 1) THEN $
         BEGIN
            ixg = ix/nres + gMINCOL
            iyg = gMAXROW - iy/nres
	    plots, [ixg, ixg], [iyg, iyg+1], color=0, /DEVICE
	 END
      ENDFOR
      ns = nx
;     faverage (nave, scan, ns)
   END $

   ELSE $

   BEGIN 

;* ...... line-driven loop : *

;      printf ("line-driven scan\n")

      nydif = iy2 - iy1
      IF (nydif LT 0) THEN nydif = -nydif
      ny = nydif + 1
      iy = iy1 - 1
      iyinc = 1
      scan   = fltarr (ny)
      scandx = fltarr (ny)

      IF (iy1 GT iy2) THEN $
      BEGIN
         iy = iy1 +1
         iyinc = -1
      END

      FOR i = 0, ny-1 DO BEGIN
         iy = iy + iyinc
         x = float (ix1)
         y = float (iy)
         IF (ix1 NE ix2) THEN x = (y-yoff) / slope
         ix = long (x)

;        *** Extract 2 adjacent pixels (in same line) from image. ***

;        *** 1 byte per pixel: clear all high order bits above low-order byte.**

         IF (nbypix EQ 1) THEN $
         BEGIN
            ival1 = imgbuf ((ix-ip)   , (iy-ir)) AND 0377
            ival2 = imgbuf ((ix-ip+1) , (iy-ir)) AND 0377
	 END  $
         ELSE $

;        *** 2 bytes per pixel. ***

         BEGIN
            ival1 = imgbuf ( (ix-ip)   , (iy-ir)) AND 0177777
            ival2 = imgbuf ( (ix-ip+1) , (iy-ir)) AND 0177777
         END

;        *** Convert (x,y) coordinates into (r,th). ***

         rcoord, r, th, x, y, -1, scroll, ierr
         IF (ierr NE 0) THEN RETURN

;        *** Linear interpolation in horizonal axis. ***

         IF (lcc EQ 1) THEN cc = calcor (r)
         scan (i) = ((ival1 + (ival2-ival1) * (x-ix)) * Bexp) * cc

;        *** Remove stray light & vignetting. ***

         IF (idtype EQ 3) THEN $
         BEGIN
	    IF (lstr EQ 1) THEN str = cpstray (r)
	    IF (lvig EQ 1) THEN  vig = vignet (r)
	    scan (i) = (scan (i) * polcor - str) * vig
            IF (lfc EQ 1)  THEN scan (i) = scan (i) - fcorona (r, th)
	 END

;        *** Determine independent coordinate value for scan. ***

         dstsun = dstsun + dstinc
         IF (scntyp EQ 1) THEN scandx (i) = float (r)
         IF (scntyp EQ 2) THEN scandx (i) = float (dstsun)

; *         if (igterm EQ 0)
; *            {
; *            if (idtype EQ 3)
; *               printf ("ix:%3d iy:%3d r:%6.2f th:%7.2f length(Rsun):%6.2f Bsun:%9.2e\n",
; *                       ix, iy, r, th, dstsun, scan[i])
; *            else
; *               printf("ix:%3d iy:%3d r:%6.2f th:%7.2f length(Rsun):%7.2f intensity:%7.2f\n",
; *                       ix, iy, r, th, dstsun, scan[i])
; *	      }

;        *** Plot 2 nearest points on display in graphics overlay. ***

         IF (igr EQ 1) THEN $
         BEGIN
            ixg = ix/nres + gMINCOL
            iyg = gMAXROW - iy/nres
	    plots, [ixg, ixg+1], [iyg, iyg], COLOR=0, /DEVICE
         END
      ENDFOR
      ns = ny
;      faverage (nave, scan, ns)

   END
   RETURN
END
