; irscan.pro : IDL procedure
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Radial Scan.
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Solar Maximum Mission   Coronagraph/Polarimeter
; Andrew L. Stanger
; 27 January 1993: IDL procedure
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; namimg: image file name
; imgbuf: image memory array (byte)
; rmin, rmax: min/max radii (Rsun)
; rinc: radial increment (Rsun)
; theta: position angle (degrees)
; scan:  array of scan intensity values.
; scandx: independent (x-axis) array = Rsun distance.
; ns : # values in scan & scandx.
; ierr: error code.
;. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .

PRO irscan, namimg, image, rmin, rmax, rinc, theta, scan, scandx, ns, ierr

;  *** The following variables are ordinarily found in a common block :

   MAXSCN = 3600
   gMINCOL = 0
   gMAXCOL = 447
   gMINROW = 0
   gMAXROW = 447

;  * The following variables are ordinarily set by cphskp (housekeeping
;  * decoder) and depend upon the image resolution

   idtype = 1	; 1=raw data (byte), 2=equiv.sec (SHORT), 3=calibrated (SHORT)
   igr = 1	; 0=no display, 1=display image
   gres = 1	; 1=low resolution 2=high resolution
   pscale = 1.0	; photometric brightness scaling factor
   scroll = 0	; S/C roll

;  *** Set photometric scale factor. ***

   Bexp = pscale

;  *** Initialize variables. 

   str = 0.0	; stray light
   vig = 1.0	; vignetting
   fcc = 0.0	; F-corona
   cc = 1.0	; Munro calibration correction
   pc = 1.0	; polaroid filter correction
   polcor = 1.0

;  *** Radial increment: set to 0.1 Rsun if 'rinc' <= 0.

   radinc = rinc
   IF (rinc LE 0.0) THEN radinc = 0.1
   th = theta

;  * ******* Radial Scan ******* *

   ns = 0
   r = rmin - rinc
   irmax = long ((rmax - rmin) / rinc) + 1
   scan = fltarr (irmax)
   scandx = fltarr (irmax)
   FOR ir = 0, irmax-1 DO BEGIN
      scan (ir) = 0.0
      scandx (ir) = 0.0
   ENDFOR
 
;   ....... Radial scan loop .......

   FOR ir = 0, irmax-1 DO BEGIN

      r = r + rinc
 
;     *** Coordinate transformation :  (radius,angle) --> (pixel,line). ***

      rcoord, r, th, x, y, 1, scroll, ierr
      IF (ierr NE 0) THEN RETURN

;     *** Compute stray light, vignetting & F-corona values. ***

      IF (idtype EQ 3) THEN $
      BEGIN
         IF (lcc  EQ 1)  THEN cc  = calcor (r)
         IF (lstr EQ 1)  THEN str = cpstray (r) * pc
         IF (lfc  EQ 1)  THEN fcc = fcorona (r) * pc
         IF (lvig EQ 1)  THEN vig = vignet (r)
      END

;     *** Draw "X" to mark scan point on IRIS display. ***
 
      IF (igr GT 0) THEN $
      BEGIN
         ixg = long (x/gres) + gMINCOL
         iyg = gMAXROW - long (y/gres)
	 plots, [ixg, ixg+1], [iyg, iyg+1], COLOR=0, /DEVICE
	 plots, [ixg+1, ixg], [iyg, iyg+1], COLOR=1, /DEVICE
	 PRINT, ixg, iyg
      END

;     * Perform bilinear interpolation to compute scan point value from image.
;     * Use 4 nearest neighbors.

      scandx (ir) = float (r)

;      cval = (Blint (image, x, y) * Bexp) * cc		;bi-linear interpolation

      cval = (image (x,y) * Bexp) * cc			; single pixel.
      scan (ir) = (cval - str) * vig - fcc

;     * Divide scan values by 5 for equivalent second data type.
;     * (numbers in file are 5 * equivalent second units.)
 
      IF (idtype EQ 2) THEN scan (ir) = scan (ir) / 5.0
   ENDFOR

   ns = ir

;   *** Smooth data using 'nave' point average. ***
 
;   Faverage, nave, scan, ns
 
   nscan = ns
;   IF (igterm EQ 0) THEN $
      FOR ir = 0, ns-1 DO $
         PRINT, scandx (ir), th, scan (ir)
   RETURN
END
