; itscan.pro
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* Theta Scan.
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* Solar Maximum Mission   Coronagraph/Polarimeter
;* Andrew L. Stanger
;* 28 January 1993: IDL version.
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

;#include <stdio.h>
;#include "/usr/people/smm/lib/include/cplib.ext"
;#include "/usr/people/smm/lib/include/img_scale.ext"
;#include "/usr/people/smm/lib/include/ov_color.h"

;/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

PRO itscan, namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, ns, ierr

;ptscan (namimg, imgbuf, thmin, thmax, thinc, radius, scan, scandx, ns)

;char	namimg[];	/* Image file name. */
;char	*imgbuf;	/* Pointer to image array. */

;float	thmin, thmax;	/* Solar position angle limits for theta scan. */
;float	thinc;		/* Angular increment for theta scan. */
;float	radius;		/* Radius (R/Rsun units). */
;float	scan[];		/* Dependent axis scan array. */
;float	scandx[];	/* Independent axis scan array. */

;int	*ns;		/* Number of values in scan. */

;{
;extern float calcor();	/* Munro Calbiration correction function. */
;extern float cpstray();	/* Stray light function. */
;extern float vignet();	/* Vignetting function. */
;extern float fcorona();	/* F-corona function. */
;extern int faverage();	/* Scan average routine. */
;extern float blint();	/* Bilinear interpolation function. */
;extern int rcoord();	/* Coordinate transformation: polar <--> cartesian. */
;extern float polcor;	/* Polaroid transmission correction factor. */

;float	x, y;
;float	r, th;
;float	tinc;
;float	Cexp = 1.e-11;		/* Scale factor for calibrated images. */
;register float Bexp;		/* "Generic" scale factor. */
;float	cc, str, vig, fcc, pc;
;float	cval;
;register int it;
;int	ixg, iyg;
;int	ierr;
;int	nscan;
;/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

;  *** The following variables are ordinarily found in a common block :

   MAXSCN = 3600
   gMINCOL = 0
   gMAXCOL = 447
   gMINROW = 0
   gMAXROW = 447
   lfc  = 0
   lcc  = 0
   lstr = 0
   lvig = 0

;  * The following variables are ordinarily set by cphskp (housekeeping
;  * decoder) and depend upon the image resolution

   idtype = 1	; 1=raw data (byte), 2=equiv.sec (SHORT), 3=calibrated (SHORT)
   igr = 1	; 0=no display, 1=display image
   gres = 1	; 1=low resolution 2=high resolution
   pscale = 1.0	; photometric brightness scaling factor
   scroll = 0	; S/C roll
   polcor = 1.0	; Polaroid correction

;  *** Set photometric scale factor. ***

   Bexp = pscale

;  *** Initialize variables. 

   str = 0.0	; stray light
   vig = 1.0	; vignetting
   fcc = 0.0	; F-corona
   cc = 1.0	; Munro calibration correction
   pc = 1.0	; polaroid filter correction
   polcor = 1.0

   IF (idtype EQ 3 AND lcc EQ 1) THEN cc = calcor (r)

;  *  IF (polcor == 1.0) THEN pc = 1.0 ELSE pc = 0.5

;  *** Theta increment: set default to angle separating two adjacent pixels. ***

   tinc = thinc
   IF (thinc LE 0.0) THEN tinc = degrad / (radius*pixrs)
   r = radius


;   /*** Clear scan arrays. ***/

   IF (thmax LT thmin) THEN thmax = thmax + 360.0
   t = thmin - tinc
   itmax = long ((thmax - thmin) / tinc) + 1
   scan = fltarr (itmax)
   scandx = fltarr (itmax)

   FOR it = 0, itmax-1 DO BEGIN
      scan (it) = 0.0
      scandx (it) = 0.0
   ENDFOR
 
;   /*** Compute stray light, vignetting & F-corona values. ***/

   IF (idtype EQ 3) THEN $
   BEGIN 
      IF (lstr EQ 1) THEN str = cpstray (r) * pc
      IF (lfc  EQ 1) THEN fcc = fcorona (r) * pc
      IF (lvig EQ 1) THEN vig = vignet (r)
   END

;   * ******* Theta scan loop ******* *

   ns = 0
   th = thmin - tinc
   FOR it = 0, itmax-1 DO BEGIN
      th = th + tinc
 
;      /*** Coordinate transformation :  (radius,angle) --> (pixel,line). ***/

      rcoord, r, th, x, y, 1, scroll, ierr
      IF (ierr NE 0) THEN RETURN

;      /*** Draw "X" to mark theta scan point on display. ***/
 
      IF (igr GT 0) THEN $
      BEGIN 
         ixg = long (x) / gres + gMINCOL
         iyg = gMAXROW - long (y) / gres
         plots, [ixg, ixg+1], [iyg, iyg+1], COLOR=0, /DEVICE
         plots, [ixg+1, ixg], [iyg, iyg+1], COLOR=1, /DEVICE
      END
 

;      /*** Perform bilinear interpolation to compute scan point from image. ***/

      scandx (it) = float (th)

;      cval = (blint (imgbuf, x, y) * Bexp) * cc * polcor

      cval = (imgbuf (x, y) * Bexp) * cc * polcor
      scan (it) = (cval - str) * vig - fcc

   ENDFOR
   ns = it

;   /*** Smooth data using 'nave' point average. ***/
 
;   faverage (nave, scan, *ns)
 
;   /* Apply calibration correction. */
;   /* Remove stray light & vignetting. */
;   /* Remove F-corona. */
 
   nscan = ns

;   /* Divide scan values by 5 for equivalent second data type.
;    *(numbers in file are 5 * equivalent second units.)
;    */
 
   IF (idtype EQ 2)  THEN FOR it = 0, ns-1 DO scan (it) = scan (it) / 5.0
 
;   IF (igterm EQ 0) THEN $
      FOR it = 0, ns-1 DO $
         PRINT, "r:", r, "  scandx:", scandx (it), "  scan:", scan (it)
   PRINT, "ns: ", ns
   RETURN
END
;}
