; rcoord.pro
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* SMM C/P coordinate transformation.
;* rcoord (radius, theta, x, y, isign, roll)
;* isign = 1 :  (radius,theta) --> (x,y).
;*       =-1 :  (x,y) --> (radius,theta).
;* ierr: error code.
;* Image coordinate origin (x=0,y=0): upper left corner.
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;* Solar Maximum Mission   Coronagraph/Polarimeter
;* Andrew L. Stanger
;* 27 January 1993: IDL procedure
;* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

PRO rcoord, radius, theta, x, y, isign, roll, ierr

;  *** Define constants.

   pixrs = 961.0 / (5.933 * 2.0)
   rspix = 1.0 / pixrs
   degrad = 180.0 / 3.14159265
   raddeg = 3.14159265 / 180.0
   xcen = 357.0
   ycen = 340.0

   degnor = 133.0 + float (roll)	;/* Orientation of solar north. */
   ierr = 0

   IF (isign EQ 1) THEN $		;/*** (radius,theta) --> (x,y) ***/
   BEGIN
      smmth = theta + degnor
      while (smmth GT 360.0) DO smmth = smmth - 360.0
      angle = float (raddeg * smmth)
         x = float (xcen + radius * pixrs * cos (angle))
         y = float (ycen - radius * pixrs * sin (angle))

;      IF ((x LT ip OR x GT lp) OR (y LT ir OR y GT lr)) THEN ierr = -1
   END $
   ELSE	$				;/*** (x,y) --> (r,theta) ***/
   BEGIN
      deltax = x - xcen
      deltay = ycen - y
      r2 = deltax * deltax + deltay * deltay

      radius = sqrt (r2) * rspix
      theta = atan (deltay, deltax) * degrad - degnor
      while (theta LT 0.0) DO theta = theta + 360.0
   END
   RETURN
END
