/* chroll.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Change roll of input image in housekeeping record.
 * Read files of images, change roll telemetry in housekeeping,
 * and output new file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 16 April 1987
 *  2 December 1989: Write roll as 16-bit integer into hskp bytes 88 & 89.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define  U	"./"
#include <stdio.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"

/* Define global variable storage. */

FILE	*Names;

int	infd, outfd;

main ()
{
/* Declare variables. */

   extern char	*malloc();	/* System storage allocation routine. */
   extern FILE	*Image;
   char	 *img;
   static char	f1 [80], F1 [80];
   static char	f2 [80], F2 [80];
   static char	sroll [10];
   char *hk;
   int newroll;
   int hkroll;
   int	hk11, hk12;
   register int i;
   int nread, nwrite;
   int imgsiz;
   int lenvid, idot;
   float xroll;

   printf ("chsm\n V01.00\n");

   printf ("Enter file containing image names\n>");
   scanf ("%s",f1);

/* Open file containing list of image file names. */

   if ((Names = fopen (f1,"r"))==NULL)
      {
      fprintf (stderr,"cannot open %s\n",f1);
      exit (1);
      }

/* Read filenames: one file name per line. */

   while (fscanf (Names,"%s %d", f1, &newroll) != EOF)
      {

/* Append directory pathname onto filenames. */

      sprintf (F1,"%s%s",U,f1);

/* Open input file. */

      if ((infd = open (F1, 0)) <= 0)
         {
         fprintf (stderr, "Unable to open file %s\n", F1);
         exit (-1);
         }

/* Open output file. */

      strcpy (f2, f1);
      lenvid = strlen (f2);
      idot = -1;
      for (i=0; i<lenvid; i++) if (f2 [i] == '.') idot = i;
      if (idot == -1)
         {
         fprintf (stderr, "invalid file name: %s\n", f2);
         exit (-1);
         }
      sprintf (f2, "%s%d", f1, newroll);

      sprintf(F2, "%s%s", U, f2);
      printf ("%s\n", F2);
      if ((outfd = creat (F2, 0644)) <= 0)
         {
         fprintf (stderr,"Cannot create %s\n",F2);
         exit (-2);
         }

/* Read Housekeeping. */

      cphskp (F1);
      printf ("npix: %d   nbypix: %d\n", npix, nbypix);

      imgsiz = (npix + 2) * npix * nbypix;
      img = malloc (imgsiz);

      if ((nread = read (infd, img, imgsiz)) != imgsiz)
         {
         fprintf (stderr,"read: only %d bytes read\n", nread);
         exit (-3);
         }

/* Change roll in housekeeping. */

      hk = (char *) img;
      hk11 = (int) hk[11] & 0360;
      hk12 = (int) hk[12] & 0077;
      xroll = (float) newroll * 3.14159 / 22.5;
      hkroll = ((int) xroll) & 077;
      hk11 |= ((hkroll & 074) >> 2) &  017;
      hk12 |= ((hkroll & 003) << 6) & 0300;
      hk[11] = (char) (hk11 & 0377);
      hk[12] = (char) (hk12 & 0377);

/* Write roll as 16-bit integer into housekeeping bytes 88 and 89. */

      putroll (img, newroll);
		
      if ((nwrite = write (outfd, img, imgsiz)) != imgsiz)
         {
         fprintf (stderr, "write: invalid # bytes written: %d\n", nwrite);
         exit (-4);
         }

      }
   close(infd);
   close(outfd);
}
