/* cp_svfcor.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Remove stray light and/or vignetting and/or F-corona from a C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 * 17 February 1987
 * 1 December 1987: Make calibration correction removal an OPTION.
 * 30 March 1988: Sun-4/280
 * 30 September 1991: SGI IRIS
 * 28 April 1997: buf changed from "char" to "unsigned char".
 *                Stray Light, Vignetting, Registration, F-corona flags set.
 *                Modify to also accomodate floating point images.
 * 16 June 1999: replace stray & vignet with cpstray & cpvignet.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"

cp_svfcor (kcc, kstr, kvig, kfc, namimg)

int kcc  ;		/* =1: Apply  Calibration Correction. */
int kstr ;		/* =1: Remove Stray Light.            */
int kvig ;		/* =1: Remove Vignetting.             */
int kfc  ;		/* =1: Remove F-corona.               */
char *namimg ;		/* SMM C/P image. */

{
extern char  *malloc (); 
extern char  *rdimg (); 
extern int   rcoord (); 
extern float jupcor (); 
extern float cpstray (); 
extern float cpvignet (); 
extern float fcorona (); 

static float Cinv = 1.e+11; 
float r, th; 
float x, y; 
float cc   = 1.0; 
float str  = 0.0; 
float vig  = 1.0; 
float fcor = 0.0; 
float vinv = 1.0; 
float pixval; 

static unsigned char buf [3584]; 
static char namcor [80]; 
static char answer [2]; 

unsigned char  *imgbuf ; 
unsigned char  *cibuf ; 
unsigned short *sibuf ; 
float          *fibuf ; 
unsigned char  *outbuf ; 
unsigned char  *cobuf ; 
unsigned short *sobuf ; 
float          *fobuf ; 

unsigned char  cmin, cmax ;
unsigned short smin, smax ;
float          fmin, fmax ;

register char  *imc; 

int day;
int idot; 
int nbyhdr; 
int nbyimg; 
int nread, nwrite, nclose; 
int i; 
register int ix, iy; 
int lcor = 0; 
int lunimg, luncor; 
int month ;
int nrechd ; 

   /*** Derive output file name from input image name. ***/

   idot = -1; 
   for (i = 0; i < strlen (namimg); i++) if (namimg [i] == '.')  idot = i; 
   if (idot < 0)  idot = strlen (namimg) - 1; 
   idot = strlen (namimg) ;

   strcpy (namcor, namimg); 
   namcor [idot] = '.'; 
   if (kcc  == 1)  namcor [idot+1] = 'c';  else namcor [idot+1] = 'x'; 
   if (kstr == 1)  namcor [idot+2] = 's';  else namcor [idot+2] = 'x'; 
   if (kvig == 1)  namcor [idot+3] = 'v';  else namcor [idot+3] = 'x'; 
   if (kfc  == 1)  namcor [idot+4] = 'f';  else namcor [idot+4] = 'x'; 
   namcor [idot+5] = '\0'; 

   printf ("%s --> %s\n", namimg, namcor) ;

   /*** Read image into memory. ***/

   imgbuf = (unsigned char *) rdimg (namimg); 

   /*** Do NOT apply corrections to RAW images. ***/

   if (nbypix == 1)
      {
      fprintf (stderr, "Unable to apply corrections to RAW images.\n") ;
      return (-1) ;
      }

   /*** If any of the corrections has been applied previously,
    *** then do NOT apply it again.
    ***/

   if (H.cc  == 1 && kcc  == 1)  kcc  = 0;
   if (H.str == 1 && kstr == 1)  kstr = 0;
   if (H.vig == 1 && kvig == 1)  kvig = 0;
   if (H.f   == 1 && kfc  == 1)  kfc  = 0;

/* printf ("H.pscale: %e\n", H.pscale); */

   /*** Set up array pointers. ***/

   cibuf = (unsigned char  *) imgbuf ;
   sibuf = (unsigned short *) imgbuf ;
   fibuf = (float          *) imgbuf ;

   if (nbypix == 4)  nrechd = 1 ; 
   else              nrechd = 2 ; 

   nbyhdr = nbypix * nrechd * npixl ;
   nbyimg = nbypix * npixl  * npixl ;
/*   printf ("nbypix: %d  nbyhdr: %d  nbyimg: %d\n", nbypix, nbyhdr, nbyimg) ;*/

   outbuf = (unsigned char *) malloc (nbyimg); 
   cobuf = (unsigned char  *) outbuf ;
   sobuf = (unsigned short *) outbuf ;
   fobuf = (float          *) outbuf ;

   /*** Open output image file & copy housekeeping from input image. ***/

   lunimg = open (namimg, 0); 
   luncor = creat (namcor, 0644); 
   nclose = close (luncor); 
   luncor = open (namcor, 1); 

   nread  = read  (lunimg, buf, nbyhdr); 	/* Housekeeping. */
/*   printf ("buf [223]: %o (octal)\n", buf [223]) ; */

   /*** Set appropriate flags in image housekeeping. ***/

   buf [223] = buf [223] | (kstr *   1) ; 	/* Set Stray Light Flag. */
   buf [223] = buf [223] | (kvig *   2) ; 	/* Set Vignetting Flag. */
   buf [223] = buf [223] | (kfc  *   8) ; 	/* Set F-corona removal Flag. */   buf [223] = buf [223] | (kcc  * 128) ;	/* Set Calibration Corr. Flag.*/

/*
 *   printf ("kstr: %d  kvig: %d  kfc: %d  kcc: %d\n", kstr, kvig, kfc, kcc) ;
 *   printf ("buf [223]: %o (octal)\n", buf [223]) ;
 */

   /*** Write houskeeping record. ***/

   nwrite = write (luncor, buf, nbyhdr); 
   nclose = close (lunimg); 

   /*** Remove stray light, vignetting & F-corona. ***/

   if (nbypix == 1)
   {
   for (iy = 0; iy < npix; iy++)
      for (ix = 0; ix < npix; ix++)
         {
         x = (float) ix; 
         y = (float) iy; 
         rcoord (&r, &th, &x, &y, -1, scroll); 
         if (kcc)    cc  = jupcor   (r); 
         if (kstr)   str = cpstray  (r * H.rsun) * Cinv; 
         if (kvig) { vig = cpvignet (r * H.rsun);  vinv = 1.0 / vig; }
         if (kfc)   fcor = fcorona  (r,th) * Cinv; 
         i = x + y * npix; 
         pixval = (*(cibuf + i) * cc - str) * vig - fcor; 
         if (pixval < 0.0)  pixval = 0.0; 
         *(cobuf + i) = (unsigned char) (pixval) & 0377; 
         }
      }
   else
   if (nbypix == 2)
   {
   smin = 32767 ;
   smax =     0 ;
   fmin = 0.0 ;
   fmax = -1.0e+30;
   for (iy = 0; iy < npix; iy++)
      for (ix = 0; ix < npix; ix++)
         {
         x = (float) ix; 
         y = (float) iy; 
         rcoord (&r, &th, &x, &y, -1, scroll); 		/* Get radius [Rsun]. */
         if (kcc)    cc  = jupcor   (r) ; 
         if (kstr)   str = cpstray  (r * H.rsun) * Cinv; 
         if (kvig) { vig = cpvignet (r * H.rsun);  vinv = 1.0 / vig; }
         if (kfc)   fcor = fcorona  (r,th) * Cinv; 
         i = x + y * npix; 
         pixval = (*(sibuf + i) * cc - str) * vig - fcor; 

         if (pixval < 0.0)  pixval = 0.0; 

	 if (pixval < fmin)  fmin = pixval ;
	 if (pixval > fmax)  fmax = pixval ;
         *(sobuf + i) = (unsigned short) (pixval) & 0177777; 
	 if (*(sobuf + i) < smin)  smin = *(sobuf + i) ;
	 if (*(sobuf + i) > smax)  smax = *(sobuf + i) ;
         }
/*
 *      fprintf (stdout, "fmin, fmax: %e %e\n", fmin, fmax) ;
 *      fprintf (stdout, "smin, smax: %d %d\n", smin, smax) ;
 */
      }
   else
   if (nbypix == 4)
   {
   for (iy = 0; iy < npix; iy++)
      for (ix = 0; ix < npix; ix++)
         {
         x = (float) ix; 
         y = (float) iy; 
         rcoord (&r, &th, &x, &y, -1, scroll); 
         if (kcc)    cc  = jupcor   (r); 
         if (kstr)   str = cpstray  (r * H.rsun) * Cinv; 
         if (kvig) { vig = cpvignet (r * H.rsun);  vinv = 1.0 / vig; }
         if (kfc)   fcor = fcorona  (r,th) * Cinv; 
         i = x + y * npix; 
         pixval = (*(fibuf + i) * cc * H.pscale - str) * vig - fcor; 
         if (pixval < 0.0)  pixval = 0.0; 
         *(fobuf + i) = (float) (pixval); 
         }
      }


   nwrite = write (luncor, outbuf, nbyimg); 
   nclose = close (luncor); 
   return (0); 
}
