/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * cpjfsv.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Subtract Stray Light and/or Vignetting and/or F-corona from a C/P image.
 * Output difference image is floating point.
 * Output image is designated the same as the input image.
 * Input image may be short integer or floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.h"

main (argc, argv)

int argc;
char *argv [];

{
   char *parg ;			/* Argument pointer. */
   char *name ;			/* File name. */
   static char namimg [80] ;	/* Image file name. */
   static char namlst [80] ;	/* List  file name. */
   static char nampgm [80] ;	/* Command file name. */

   int kcc  ;			/* =1: Apply  Calibration Correction. */
   int kfc  ;			/* =1: Remove F-corona.    */
   int kstr ;			/* =1: Remove Stray Light. */
   int kvig ;			/* =1: Remove Vignetting.  */

   FILE *LIST ;

   kcc  = 0 ;
   kfc  = 0 ;
   kstr = 0 ;
   kvig = 0 ;
   sprintf (namimg, "%s", "smmcp.img") ;

   strcpy (nampgm, argv [0]) ;
   if (argc < 2)		/* Need at least 2 arguments: option & file. */
      {
      printf ("usage arg<2\n") ;
      usage (nampgm) ;
      exit (0) ;
      }

   while (--argc)		/* Argument processing loop. */
      {
      parg = *++argv ;
      printf ("parg: %s\n", parg) ;

      if (*(parg) == '?')	/* Help. */
	 {
	 printf ("usage ?\n") ;
	 usage (nampgm) ;
	 exit (0) ;
	 }

     if (*(parg) == '-')		/* Check for option arguments. */
         {
         switch (parg [1])
	    {
	    case 'c': kcc  = 1 ;  break ;  /* Set Jupiter Correction  flag. */
	    case 'f': kfc  = 1 ;  break ;  /* Set F-corona    removal flag. */
	    case 's': kstr = 1 ;  break ;  /* Set stray light removal flag. */
	    case 'v': kvig = 1 ;  break ;  /* Set vignetting  removal flag. */
	    default: printf ("usage ?\n") ;  usage (nampgm);
	    }
         continue ;
         }
      else		/* Get the filename. */
         {
	 name = parg ;
         if (name [0] == '@')           /* Read list file. */
            {
            strcpy (namlst, &name [1]) ;
            if ((LIST = fopen (namlst, "r")) == NULL)
               {
               fprintf (stderr, "Unable to open file: %s\n", namlst) ;
               exit (-2) ;
	       }

            while (fscanf (LIST, "%s", namimg) != EOF)
	       {
	       printf ("namimg: %s  kcc:%d  kfc:%d  kstr:%d  kvig:%d\n",
			namimg, kcc, kfc, kstr, kvig) ;
               cp_jfsv (kcc, kfc, kstr, kvig, namimg) ;
               }
            fclose (LIST) ;
            }
         else
	    {
	    strcpy (namimg, name) ;
	    printf ("namimg: %s  kcc:%d  kfc:%d  kstr:%d  kvig:%d\n",
		     namimg, kcc, kfc, kstr, kvig) ;
	    cp_jfsv (kcc, kfc, kstr, kvig, namimg) ;
	    }
         }
      }
   exit (0) ;
}

usage (command)

char *command ;
{
   fprintf (stderr, "%s  [-j][-f][-s][-v] file1 file2 file ...\n", command) ;
   return (0) ;
}
