/* cplog4.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Housekeeping display routine.
 * Requires an image name, a directory, or a file containing a list of
 * SMM C/P image file names.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P
 * Andrew L. Stanger
 * UNIX 4.2 BSD   C programming language
 * 25 October 1984
 * 18 December 1984 : Add Roll.
 * 22 April 1985: Change housekeeping structure.  Add filename to output line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.h"

#define MAXHSK 224

char pei;			/* Pre-exposure inhibit. */

main (argc, argv)

int argc;
char *argv [];

{
static  char	namlst [32];	/* Name of file containing list of images. */
static	char	namimg [32];	/* C/P image disk file name. */
static	char lsdir [80] = "ls ";
static	char imdir [80] = ".";
static	char flag [80];
static	char filename [80];
char	name [80];
int	idot;
int	i, lenfile;
FILE *LIST, *IMG, *DIR, *popen ();


   /*** 3 arguments: "cplog -d directory"   or   "cplog -f filelistname". ***/

   if (argc == 3)
      {
      strcpy (flag, argv [1]);
      switch (flag [1])
         {
         case 'd' :	strcpy (imdir, argv [2]);
			goto directory;
         case 'f' :	strcpy (namlst, argv [2]);
			goto file;
         default   :	fprintf (stderr, "Usage: cplog [image_name] [-d directory] [-f filelistname]\n");
         }
   }

   /*
    * 2 arguments: cplog image_name.
    * Interpret housekeeping for a single image.
    */

   if (argc == 2)
      {
      smmhk (argv [1]);
      fprintf (stdout,"   filename           G M T             MET    Q%% IMG RES EXP FW  PW  SM CM ROLL\n");
      fprintf (stdout, "---------------- ----------------- --------- --- ---- - ---- -- ---  -- -- ----\n");

      fhkline ();
      exit (0);
      }

   /*
    * 1 argument: "cplog".
    *  By default, interpret contents of current directory.
    */

   if (argc == 1)
      {
      fprintf (stderr, "Usage: cplog [image_name] [-d directory] [-f filelistname]\n");
      strcpy (imdir, ".");
      }

directory:

   strcat (lsdir, imdir);

   /*** Get directory. ***/

   DIR = popen(lsdir,"r");

   fprintf (stdout,"   filename           G M T            MET    Q%% IMG RES EXP FW  PW  SM CM ROLL\n");
   fprintf (stdout,"---------------- ----------------- --------- --- ---- - ---- -- ---  -- -- ----\n");

   /*** Start going through data. ***/

   while (fscanf (DIR, "%s", filename) != EOF)
      {
      lenfile = strlen (filename);
      for (i=0; i<lenfile; i++) if (filename [i] == '.')  idot = i;
      if (idot < 0 || (lenfile <= idot+2) ||
         (filename [idot+1] != 'v' && filename [idot+1] != 'c'))
         {
         fprintf (stderr, "Invalid file name: %s\n", filename);
         goto loop;
         }
      sprintf (name, "%s/%s", imdir, filename);	/* Add directory. */

      /*** Extract housekeeping and write to screen. ***/

      smmhk (name);

      /*** Display housekeeping summary on terminal screen. ***/

      fhkline ();

loop:
      i++;
      }
   exit (0);

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/

file:

   /*** Open file containing list of SMM C/P images. ***/

   if ((LIST = fopen (namlst, "r")) == NULL)
      {
      fprintf (stderr, "Cannot open %s\n", namlst);
      exit (-2);
      }

   fprintf (stdout, "   filename           G M T            MET    Q%% IMG RES EXP FW  PW  SM CM ROLL\n");
   fprintf (stdout, "---------------- ----------------- --------- --- ---- - ---- -- ---  -- -- ----\n");

   /*** Read list of images; process one image at a time. ***/

   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg,"r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s\n", namimg);
         exit (-6);
         }
      fclose (IMG);

      /*** Decommutate Housekeeping. ***/

      smmhk (namimg);

      /*** Display housekeeping summary on terminal screen. ***/

      fhkline ();
      }
   fclose (LIST);
   exit (0);
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * E R R O R ( )	Print error message and exit.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
error (str1, str2)

char *str1, *str2;
{
   fprintf (stderr, str1, str2);
   fprintf (stderr, "\n");
   exit (1);
}
