/* fixrollb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Change roll of input image in housekeeping record.
 * Read files of images, change roll telemetry in housekeeping, rewrite file.
 * NOTE: changes housekeeping bytes 88 & 89.
 * WARNING: Overwrites original images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 1980: 12-bit roll occupies least significant 4 bits of byte 11 and
 *       all 8 bits of byte 12.  Least Significant bit weight is 0.1119 degrees.
 *
 * 1984-89: 6-bit roll occupies least significant 4 bits of byte 11 and
 *          2 most significant bits of byte 12.
 *          Least Significant bit weight is 7.16197 degrees.
 *          The lower 6 bits of byte 12 are the 6 most significant bits
 *          for MET (mission elapsed time, or spacecraft time).
 *
 * Bytes 11 & 12: will contain roll as specified by user, except in
 *                appropriate scaled integer radian units.
 *
 * Bytes 88 & 89: will contain roll as a positive integer in the range
 *                of 1-360 degrees.  Note that 0 degrees is stored as 360.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 20 December 1989
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#define  U	"."
#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"

/* Define global variable storage. */

FILE	*Names;

int	fd;

main()
{
   /*** Declare variables. ***/

   extern char	*malloc();	/* System storage allocation routine. */
   extern FILE	*Image;
   char	 *img;
   static char	f1[80], F1[80];
   static char	f2[80], F2[80];
   static char	sroll[10];
   int hkroll;
   int	hk11, hk12;
   int newroll;
   int posroll;
   register int i;
   int nread, nwrite;
   int imgsiz;
   int lenvid, idot;
   float xroll;
   double rollres80 = 0.111905819;	/* pi / (22.5 * 64) */
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   fprintf (stderr, "modroll\n V01.00: change roll in hskp bytes 11&12, 88&89.\n");
   fprintf (stderr, "WARNING : Original images will be rewritten with revised roll.\n");

   fprintf (stderr, "Enter roll value (integer): ");
   fscanf  (stdin, "%d", &newroll);
   fprintf (stderr, "Enter name of file containing image names > ");
   fscanf (stdin, "%s", f1);

   if (newroll <= 0)  posroll = newroll + 360;	/* Range: 1-360 degrees. */
   else posroll = newroll;

   /*** Open file containing list of image file names. ***/

   if ((Names = fopen (f1,"r"))==NULL)
      {
      fprintf (stderr,"cannot open %s\n",f1);
      exit (1);
      }

   /*** Read filenames: one file name per line. ***/

   while (fscanf (Names,"%s", f1) != EOF)
      {
      fprintf (stdout, "%s   roll: %d\n", f1, posroll);       

      /*** Append directory pathname onto filenames. ***/

      sprintf (F1,"%s/%s",U,f1);

      /*** Open input file. ***/

      if ((fd = open (F1, 2)) <= 0)
         {
         fprintf (stderr, "Unable to open file %s\n", F1);
         exit (-1);
         }

      /*** Read image into memory. ***/

      cphskp (F1);

      imgsiz = (npix + 2) * npix * nbypix;
      img = malloc (imgsiz);

      if ((nread = read (fd, img, imgsiz)) != imgsiz)
         {
         fprintf (stderr,"read: only %d bytes read\n", nread);
         exit (-2);
         }

      /*** Store roll [degree units] into bytes #88 & 89 of housekeeping. ***/

      img [88] = (char) ((posroll &0177400) >> 8);
      img [89] = (char)  (posroll &   0377);

      /*** Rewrite image back to original disk file. ***/

      lseek (fd, 0L, 0);
      if ((nwrite = write (fd, img, imgsiz)) != imgsiz)
         {
         fprintf (stderr, "write: invalid # bytes written: %d\n", nwrite);
         exit (-3);
         }
      close (fd);
      }
   fclose (Names);
}
