/* fwlist.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Generates a list of images having a common, specified filter position.
 * Requires an image name, a directory, or a file containing a list of
 * SMM C/P image file names.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P
 * Andrew L. Stanger
 *  6 January 1993: SGI version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

#define MAXHSK 224

/* Housekeeping structure. */

#include "/usr/people/smm/lib/include/cplib.h"

int pei = 0;

main (argc, argv)

int argc;
char *argv[];

{
   static  char	namlst[32];	/* Name of file containing list of images. */
   static	char	namimg[32];	/* C/P image disk file name. */
   static	char lsdir[81] = "ls ";
   static	char imdir[81] = ".";
   static	char flag[81];
   static	char filename[81];
   char	name[81];
   int	idot;
   int	i, lenfile;
   char	namefw [8];
   FILE *LIST, *IMG, *DIR, *popen();

   fprintf (stderr,"Enter filter: ");
   scanf  ("%s", namefw);
   strcat (namefw, " ");

   /* 3 arguments: "fwlist -d directory"   or   "fwlist -f filelistname". */

   if (argc == 3)
   {
	strcpy (flag, argv[1]);
	switch (flag[1])
	{
	case 'd' :	strcpy (imdir, argv[2]);
			goto directory;
	case 'f' :	strcpy (namlst, argv[2]);
			goto file;
	default   :	fprintf (stderr, "usage: %s [image_name] [-d directory] [-f filelistname]\n", argv [0]);
	}
   }

   /* 2 arguments: fwlist image_name.  Interpret housekeeping for a single image. */

   else if (argc == 2)
      {
      cphskp (argv [1]);

      /* Display image name on terminal screen if exposure matches given value. */

      if (strncmp (H.fil, namefw, 2) == 0)  fprintf (stdout, "%s\n", namimg);
      exit (0);
      }

   /* 1 argument: "fwlist".  By default, interpret contents of current directory. */

   else if (argc == 1)
      {
      fprintf (stderr, "usage: %s [image_name] [-d directory] [-f filelistname]\n", argv [0]);
      strcpy (imdir, ".");
      }

directory:

   strcat (lsdir, imdir);

   /* Get directory. */

   DIR = popen( lsdir, "r");

   /* Start going through data. */

   while (fscanf (DIR, "%s", filename) != EOF)
      {
      lenfile = strlen (filename);
      for (i=0; i<lenfile; i++) if (filename[i] == '.')  idot = i;
      if (idot < 0 || (lenfile <= idot+2) ||
         (filename[idot+1] != 'v' && filename[idot+1] != 'c'))
         {
         fprintf (stderr, "Invalid file name: %s\n", filename);
         goto loop;
         }
      sprintf (name,"%s/%s",imdir,filename);	/* Add directory. */

      /* Extract housekeeping and write to screen. */

      cphskp (name);

      /* Display image name on terminal screen if exposure matches given value. */

      if (strncmp (H.fil, namefw, 2) == 0)  fprintf (stdout,  "%s\n",  namimg);

loop:
      i++;
      }
   exit (0);

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .*/

file:

   /* Open file containing list of SMM C/P images. */

   if ((LIST = fopen (namlst,"r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s\n", namlst);
      exit(-2);
      }

      /* Read list of images; process one image at a time. */

   while (fscanf (LIST, "%s", namimg) != EOF)
      {
      if ((IMG = fopen (namimg, "r")) == NULL)
         {
         fprintf (stderr, "Cannot open %s\n", namimg);
         exit (-6);
         }
      fclose (IMG);

      /* Decommutate Housekeeping. */

      cphskp (namimg);

      /* Display image name on terminal screen if exposure matches given value. */

      if (strncmp (H.fil, namefw, 2) == 0)  fprintf (stdout,  "%s\n",  namimg);
      }
   fclose (LIST);
   exit (0);
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * E R R O R ( )	Print error message and exit.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
error (str1, str2)

char *str1, *str2;
{
   fprintf(stderr, str1, str2);
   fprintf(stderr, "\n");
   exit(1);
}
