/* mergercp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create one image from fragments.
 * Accepts either low or high resolution SMM C/P images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 October 1989
 * 17 August 1990: Improve logic to determine "imgsiz".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#define IMGSIZ 802816
#define MAXSEG 7168
#define MAXBYLIN 896 

main()
{
   extern char *newname ();
   static char namlst [80];
   static char namimg [80];
   static char namout [80];
   static char hskp [MAXBYLIN];
   static char linbyt [MAXBYLIN];
   static char imgbuf [IMGSIZ];
   static char imfrag [IMGSIZ];
   static short bitmap [MAXSEG];
   static int imgsiz = IMGSIZ;
   unsigned short value, bit;
   static int i, im, ib, is;
   static int khdr = 0;
   int nread, nwrite, nclose;
   int fdimg, fdout;
   int imgcnt;
   int imgpct;
   int iseg;
   int nbylin;
   int nbymap;
   int nseg;
   FILE *list;

   fprintf (stderr, "Enter file containing the list of images to be merged: ");
   fscanf  (stdin,  "%s", namlst);

   /*** Open file containing image names. ***/

   if ((list = fopen (namlst,"r")) == NULL)
      {
      fprintf (stderr,"Cannot open %s\n", namlst);
      exit (-1);
      }

   /*** Read image name from list in file. ***/

   for (i = 0; i < MAXSEG; i++)  bitmap [i] = 0;/* Clear bitmap to all zeroes.*/
   npix = 0;
   imgcnt = 1;
   while (fscanf (list, "%s", namimg) != EOF)
      {
      fprintf (stderr, "input %s\n", namimg);
      fdimg = open (namimg, 0);

      /*** Read and copy housekeeping for first image only. ***/

      if (imgcnt++ == 1)
         {
	 cphskp (namimg);
	 nseg = npix * (npix/112);
	 nbylin = npix;
	 nbymap = nseg / 8;
         strcpy (namout, newname (namimg));
         fdout = creat (namout, 0644);
         nclose = close (fdout);
         fdout = open (namout, 2);
         }

      /*** Save housekeeping record of first image whose hskp is OK. ***/

      if (khdr == 0)
         {
         nread = read (fdimg, hskp, nbylin);
         if (hskp [16] != '\0')
	    {
	    khdr = 1;
	    fprintf (stderr, " . . . Save housekeeping.\n");
	    }
         else
	    fprintf (stderr, "\n");
         }
      else
	 {
         lseek (fdimg, nbylin, 0);
	 fprintf (stderr, "\n");
	 }

      /*** Read bitmap record. ***/

      nread = read (fdimg, linbyt, nbylin);

      /*** Read input image into memory array. ***/

      imgsiz = read (fdimg, imfrag, IMGSIZ);
      if (npix = 0) 
	 {
	 if (nread == 200704)  npix = 448;
	 else
	 if (nread == 802816)  npix = 896;
	 else
	    {
	    fprintf (stderr, "Image is not full size (IH=IV=1111)\n");
	    exit (-2);
	    }
	 }

      /* Unpack bitmap record.
       * Copy current image segment to new image array
       * if current image bitmap value is 1
       * and corresponding new bitmap value is zero.
       */

      i = -1;
      for (im = 0; im < nbymap; im++)
         {
         value = ((unsigned short) linbyt [im] & 0377);
         for (ib = 0; ib < 8; ib++)
            {
            i++;
            bit =  ((value >> ib) & 1);
            if (bit == 1 && *(bitmap + i) == 0)
               {
/*
 *               fprintf (stderr, "im: %d   i:%d   imfrag:%o\n",
 *                        im, i, *(imfrag + 56 + i*112));
 */
               for (is = 0; is < 112; is++)
                  *(imgbuf + is + i*112) = *(imfrag + is + i*112);
               *(bitmap + i) = 1;
               }
            }
         }
	 close (fdimg);
      }

   /*** Check for valid housekeeping. ***/

   if (khdr == 0)
      {
      fprintf (stderr, "******* WARNING: Incomplete housekeeping.\n");
/*      exit (-3); */
      };

   /*** Pack new bit map. ***/

   printf ("nbymap: %d   nseg: %d   npix: %d\n", nbymap, nseg, npix);
   iseg = 0;
   for (im=0; im<nbymap; im++)
      {
      for (ib=0; ib<8; ib++)
         {
         iseg += (int) *(bitmap + ib + im*8);
         value = (*(bitmap + ib + im*8) << ib);
         *(linbyt + im) = (char) ((unsigned short) *(linbyt + im) | value);
         }
      }

   /*** Write housekeeping to file. ***/
   /* imgpct: % non-zero segments in image. */

   imgpct = (int) (((float) iseg / (float) nseg) * 100.0);
   hskp [79] = (char) imgpct;
   fprintf (stderr, "iseg:%d   imgpct:%d\n", iseg, imgpct);
   nwrite = write (fdout, hskp, nbylin);
      
   /*** Write bit-map to file. ***/

   nwrite = write (fdout, linbyt, nbylin);
   fclose (list);

   /*** Write image data to file. ***/

   fprintf (stderr, "output %s\n", namout);
   nwrite = write (fdout, imgbuf, imgsiz);
   nclose = close (fdimg);
   nclose = close (fdout);
}

/* outname.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert 'namvid' filename into a new filename with a different extension.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 19 November 1984
 */

char *
newname (namvid)

char *namvid;

{
int lenvid;
int idot;
int i;
char namfil[80];

   lenvid = strlen(namvid);
   idot = -1;
   for (i = 0; i < lenvid; i++) 
      { if (namvid [i] == '.')  idot = i;  namfil [i] = namvid [i]; }
   if (idot == -1)
      {
      printf ("newname: invalid file name: %s\n", namvid);
      exit (-4);
      }
/*
 *   namfil [idot+1] = 'n';
 *   namfil [idot+2] = 'e';
 *   namfil [idot+3] = 'w';
 */
   namfil [idot+4] = 'm';
   namfil [idot+5] = '\0';

   return (namfil);
}
