/* rdimg(namimg)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 4 February 1985
 * 11 November 1986: modify to accomodate floating-point images.
 * 18 March 1988: swap bytes for images calibrated on Sun-4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/u/stanger/smm/cpgrin.h"
#include "/u/stanger/smm/cpsize.h"
#include "/u/stanger/smm/hskp.h"

struct	hskp	H;

char	*malloc();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

char	*
rdimg(namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
extern int srevbyt();	/* Reverse bytes of Sun-4 calibrated images. */
register   char	*image;	/* Pointer to image array. */
register   int	imgfd;	/* File descriptor for image disk file. */
register   int	imgsiz; /* Image size (bytes) (excluding hskp & bitmap). */
register   long	nread;
int nbyhdp;		/* Bytes per pixel in housekeeping & bitmap. */

/*** Decommutate Housekeeping. ***/

   cphskp(namimg);	

/*** Open image file. ***/

   if((imgfd = open(namimg, 0)) < 0)
     { fprintf(stderr,"rdimg> Can't open file %s\n", namimg);  return(NULL); }

/*** Position I/O pointer to beginning of video data in image disk file. ***/

   if (nbypix == 4) nbyhdp = 1;
   else
      nbyhdp = nbypix;
   if (lseek(imgfd,(long)(npix*nbyhdp*2),0) == -1)
      { fprintf(stderr,"rdimg> seek error.\n");  return(NULL); }

/*** Allocate storage for image array. ***/

   printf("rdimg> npix: %d   npixl: %d   nbypix: %d\n", npix, npixl, nbypix);
   imgsiz = (npixl*npixl*nbypix);
   if((image = malloc(imgsiz)) == NULL)
     {  fprintf(stderr,"rdimg> malloc error.");
        exit(1);
     }

/*** Read image into memory. ***/

   if ((nread = read(imgfd, image, imgsiz)) <= 0)
      { fprintf(stderr,"rdimg> image read error.\n");  return(NULL); }

   if (nread != imgsiz)  fprintf(stderr,"rdimg> nread:%d < imgsiz:%d\n",
				 nread, imgsiz);

   if (close(imgfd) == -1)
      { fprintf(stderr,"rdimg> file close error.%s\n", namimg);  return(NULL);}

/*** Reverse bytes for images calibrated on Sun-4. ***/

   if (nbypix == 2 && idtype == 5)  srevbyt (image, nbypix, npixl*npixl);
  return(image);
}


/* srevbyt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse byte order for a short integer image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 14 March 1988
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

srevbyt (image, nbytpx, nelem)

char *image;		/* Image array. */
int nbytpx;		/* # bytes per pixel. */
int nelem;		/* # elements in array. */

{
   register char *cimage;
   register char ctemp;
   register int i;

fprintf (stderr, "srevbyt> nbytpx: %d   nelem: %d\n", nbytpx, nelem);
   if (nbytpx != 2)
      {
      sprintf (stderr, "Image pixels are NOT short integers.\n");
      return (-1);
      }

   cimage = (char *) image;

/*** Reverse odd & even bytes in image array. ***/

   for (i=0; i<nelem; i++)
      {
      ctemp = *(cimage + 2*i);
      *(cimage + 2*i)   = *(cimage + 2*i+1);
      *(cimage + 2*i+1) = ctemp;
      }
   return (0);
}
