/* svfcor.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Remove stray light and/or vignetting and/or F-corona from a C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 * 17 February 1987
 * 1 December 1987: Make calibration correction removal an OPTION.
 * 30 March 1988: Sun-4/280
 * 30 September 1991: SGI IRIS
 * 28 April 1997: buf changed from "char" to "unsigned char".
 *                Stray Light, Vignetting, Registration, F-corona flags set.
 *                Modify to also accomodate floating point images.
 * 16 June 1999: replace stray & vignet with cpstray & cpvignet.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/cplib.h"

main (argc, argv)

int argc; 
char *argv []; 
{
extern char  *malloc (); 
extern char  *rdimg (); 
extern int   rcoord (); 
extern float jupcor (); 
extern float cpstray (); 
extern float cpvignet (); 
extern float fcorona (); 

static float Cinv = 1.e+11; 
float r, th; 
float x, y; 
float cc   = 1.0; 
float str  = 0.0; 
float vig  = 1.0; 
float fcor = 0.0; 
float vinv = 1.0; 
float pixval; 

static unsigned char buf [3584]; 
static char namcor [80]; 
static char namimg [80]; 
static char answer [2]; 

unsigned char  *imgbuf ; 
unsigned char  *cibuf ; 
unsigned short *sibuf ; 
float          *fibuf ; 
unsigned char  *outbuf ; 
unsigned char  *cobuf ; 
unsigned short *sobuf ; 
float          *fobuf ; 

unsigned char  cmin, cmax ;
unsigned short smin, smax ;
float          fmin, fmax ;

register char  *imc; 

int day;
int idot; 
int nbyhdr; 
int nbyimg; 
int nread, nwrite, nclose; 
int i; 
register int ix, iy; 
int lcor = 0; 
int lstr = 0; 
int lvig = 0; 
int lfc = 0; 
int lunimg, luncor; 
int month ;
int nrechd ; 

   if (argc == 2)
      {
      strcpy (namimg, argv [1]); 
      }
   else
      {
      fprintf (stderr, "usage: %s cp_image_filename\n", argv [0]); 
      fprintf (stderr, "Enter image file name : "); 
      fscanf  (stdin,  "%s", namimg); 
      }

   printf ("Apply Jupiter calibration correction factor [y/n]? "); 
   scanf  ("%s", answer); 
   if (strncmp (answer, "y", 1) == 0)  ljc = 1; 

   printf ("Remove stray light [y/n] ? "); 
   scanf  ("%s", answer); 
   if (strncmp (answer, "y", 1) == 0)  lstr = 1; 

   printf ("Remove vignetting [y/n] ? "); 
   scanf  ("%s", answer); 
   if (strncmp (answer, "y", 1) == 0)  lvig = 1; 

   printf ("Remove F-corona [y/n] ? "); 
   scanf  ("%s", answer); 
   if (strncmp (answer, "y", 1) == 0)  lfc = 1; 

   /* Derive output file name from input image name. */

   idot = -1; 
   for (i = 0; i < strlen (namimg); i++) if (namimg [i] == '.')  idot = i; 
   if (idot < 0)  idot = strlen (namimg) - 1; 
   strcpy (namcor, namimg); 
   namcor [idot] = '.'; 
   if (ljc  == 1)  namcor [idot+1] = 'j';  else namcor [idot+1] = 'x'; 
   if (lstr == 1)  namcor [idot+2] = 's';  else namcor [idot+2] = 'x'; 
   if (lvig == 1)  namcor [idot+3] = 'v';  else namcor [idot+3] = 'x'; 
   if (lfc  == 1)  namcor [idot+4] = 'f';  else namcor [idot+4] = 'x'; 
   namcor [idot+5] = '\0'; 
   printf ("namcor: %s\n", namcor); 

   /* Read image into memory. */

   imgbuf = (unsigned char *) rdimg (namimg); 

   printf ("H.pscale: %e\n", H.pscale);

   cibuf = (unsigned char  *) imgbuf ;
   sibuf = (unsigned short *) imgbuf ;
   fibuf = (float          *) imgbuf ;

   if (nbypix == 4)  nrechd = 1 ; 
   else              nrechd = 2 ; 

   nbyhdr = nbypix * nrechd * npixl ;
   nbyimg = nbypix * npixl  * npixl ;
   printf ("nbypix: %d  nbyhdr: %d  nbyimg: %d\n", nbypix, nbyhdr, nbyimg) ;

   outbuf = (unsigned char *) malloc (nbyimg); 
   cobuf = (unsigned char  *) outbuf ;
   sobuf = (unsigned short *) outbuf ;
   fobuf = (float          *) outbuf ;

   /* Open output image file & copy housekeeping from input image. */

   lunimg = open (namimg, 0); 
   luncor = creat (namcor, 0644); 
   nclose = close (luncor); 
   luncor = open (namcor, 1); 

   nread  = read  (lunimg, buf, nbyhdr); 	/* Housekeeping. */
   printf ("buf [223]: %o (octal)\n", buf [223]) ;

   buf [223] = buf [223] | (lstr * 1) ; 	/* Set Stray Light Flag. */
   buf [223] = buf [223] | (lvig * 2) ; 	/* Set Vignetting Flag. */
   buf [223] = buf [223] | (lfc  * 8) ; 	/* Set F-corona removal Flag. */

   printf ("lstr: %d  lvig: %d  lfc: %d\n", lstr, lvig, lfc) ;
   printf ("buf [223]: %o (octal)\n", buf [223]) ;

   nwrite = write (luncor, buf, nbyhdr); 
   nclose = close (lunimg); 

   /* Remove stray light, vignetting & F-corona. */

   if (nbypix == 1)
   {
   for (iy=0; iy<npix; iy++)
      for (ix=0; ix<npix; ix++)
         {
         x = (float) ix; 
         y = (float) iy; 
         rcoord (&r, &th, &x, &y, -1, scroll); 
         if (ljc)    cc  = jupcor (r); 
         if (lstr)   str = cpstray  (r * H.rsun) * Cinv; 
         if (lvig) { vig = cpvignet (r * H.rsun);  vinv = 1.0 / vig; }
         if (lfc)  fcor = fcorona (r,th) * Cinv; 
         i = x + y * npix; 
         pixval = (*(cibuf + i) * cc - str) * vig - fcor; 
         if (pixval < 0.0)  pixval = 0.0; 
         *(cobuf + i) = (unsigned char) (pixval) & 0377; 
         }
      }
   else
   if (nbypix == 2)
   {
   smin = 32767 ;
   smax =     0 ;
   fmin = 0.0 ;
   fmax = -1.0e+30;
   for (iy = 0; iy < npix; iy++)
      for (ix = 0; ix < npix; ix++)
         {
         x = (float) ix; 
         y = (float) iy; 
         rcoord (&r, &th, &x, &y, -1, scroll); 		/* Get radius [Rsun]. */
         if (ljc)    cc  = jupcor (r) ; 
         if (lstr)   str = cpstray  (r * H.rsun) * Cinv; 
         if (lvig) { vig = cpvignet (r * H.rsun);  vinv = 1.0 / vig; }
         if (lfc)  fcor = fcorona (r,th) * Cinv; 
         i = x + y * npix; 
         pixval = (*(sibuf + i) * cc - str) * vig - fcor; 

         if (pixval < 0.0)  pixval = 0.0; 

	 if (pixval < fmin)  fmin = pixval ;
	 if (pixval > fmax)  fmax = pixval ;
         *(sobuf + i) = (unsigned short) (pixval) & 0177777; 
	 if (*(sobuf + i) < smin)  smin = *(sobuf + i) ;
	 if (*(sobuf + i) > smax)  smax = *(sobuf + i) ;
         }
      fprintf (stdout, "fmin, fmax: %e %e\n", fmin, fmax) ;
      fprintf (stdout, "smin, smax: %d %d\n", smin, smax) ;
      }
   else
   if (nbypix == 4)
   {
   for (iy=0; iy<npix; iy++)
      for (ix=0; ix<npix; ix++)
         {
         x = (float) ix; 
         y = (float) iy; 
         rcoord (&r, &th, &x, &y, -1, scroll); 
         if (ljc)    cc = jupcor (r); 
         if (lstr)   str = cpstray  (r * H.rsun) * Cinv; 
         if (lvig) { vig = cpvignet (r * H.rsun);  vinv = 1.0 / vig; }
         if (lfc)  fcor = fcorona (r,th) * Cinv; 
         i = x + y * npix; 
         pixval = (*(fibuf + i) * cc - str) * vig - fcor; 
         if (pixval < 0.0)  pixval = 0.0; 
         *(fobuf + i) = (float) (pixval); 
         }
      }


   nwrite = write (luncor, outbuf, nbyimg); 
   nclose = close (luncor); 
   exit (0); 
}
