/* tarhdr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Extract file name and size from tar header.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 8 January 1987
 */
#define HDRSIZ 512
#define NAMSIZ 100

char *
tarhdr (buf, filsiz)

char buf[512];
long *filsiz;

{
static char name[NAMSIZ];	/* file name. */
static char mode[8];	/* file mode. */
static char userid[8];	/* user ID. */
static char groupid[8];	/* group ID. */
static char size[12];	/* size of file in octal bytes. */
static char mtime[12];	/* file modification time in octal bytes. */
static char chksum[8];	/* header checksum. */
static char linkflag;  /* ASCII 0: normal file; 1:hard link; 2:symbolic link. */
static char linkname[NAMSIZ];	/* name of linked file, if any. */

int imode;	/* decimal integer value for 'mode'. */
int iuid;	/* decimal integer value for 'uid'. */
int igid;	/* decimal integer value for 'gid'. */
long isize;	/* decimal integer value for 'size'. */
long imtime;	/* decimal integer value for 'mtime'. */

/* Check for zero-filled record, which indicates end-of-tape. */

   if (buf[0] == '\0')  exit(0);

/* Assign field names to header information. */

   buf[135] = '\0';	/* set null to terminate string for size. */
   buf[147] = '\0';	/* set null to terminate string for mtime. */

   strcpy (name,      &buf[0]);
   strcpy (mode,      &buf[100]);
   strcpy (userid,    &buf[108]);
   strcpy (groupid,   &buf[116]);
   strcpy (size,      &buf[124]);
   strcpy (mtime,     &buf[136]);
   strcpy (chksum,    &buf[148]);
   linkflag = buf[156];
   strcpy (linkname,  &buf[157]);

/*
   printf ("name: %s\n", name);
   printf ("length(name): %d\n", strlen(name));
   printf ("mode: %s  userid: %s  groupid: %s\n", mode, userid, groupid); 
   printf ("length(mode): %d\n", strlen(mode));
   printf ("length(userid): %d\n", strlen(userid));
   printf ("length(groupid): %d\n", strlen(groupid));
   printf ("size: %s  mtime: %s\n", size, mtime);
   printf ("length(size): %d\n", strlen(size));
   printf ("chksum: %s   linkflag: %c\n", chksum, linkflag);
   printf ("length(chksum): %d\n", strlen(chksum));
   printf ("linkname: %s\n", linkname);
   printf ("length(linkname): %d\n", strlen(linkname));
 */

/* Convert ASCII octal numerical fields into integers. */

   imode = aotoi (mode);
   iuid = aotoi (userid);
   igid = aotoi (groupid);
   isize = aotoi (size);
   imtime = aotoi (mtime);

/*
   printf ("mode: %s   imode: %d\n", mode, imode);
   printf ("userid: %s   iuid: %d\n", userid, iuid);
   printf ("groupid: %s   igid: %d\n", groupid, igid);
   printf ("size: %s   isize: %d\n", size, isize);
   printf ("mtime: %s   imtime: %d\n", mtime, imtime);
 */

   *filsiz = isize;

   return(name);
}
