/* tarlog.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P image summary of files on a 'tar' tape.
 * Output is written to terminal screen.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * HAO/NCAR   Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 12 January 1987
 */

#include <stdio.h>

#define BLKSIZ	10240
#define RECSIZ	512
#define NAMSIZ	100

#include "/u/stanger/hk/smmhk.h"
struct	smmhk	H;

int irec = 0;		/* counter for record # within a tape block. */
int iblk;		/* Tape block counter. */
int nrecbl;		/* # records per tape block. */

main()

{
extern char *tarhdr();		/* Function to decode tar header. */
extern cphdr();			/* Housekeeping decoder. */
extern cpskip();		/* Skip over image data. */
extern aotoi();			/* Convert ASCII octal to decimal integer. */
extern readblock();		/* Read a tape block. */

static char blkbuf[BLKSIZ];	/* Tape block buffer. */
static char recbuf[RECSIZ];	/* Record buffer. */
static char tapein[11];		/* Input tape device name. */
static char delimiter[2];	/* Storage for delimiter between drive #'s. */
static char answer[9];		/* Storage for user response. */
static char namefile[NAMSIZ];	/* file name. */

int fdin;		/* File descriptor for tape device. */
long imgsiz;		/* Size of data file. */
int nread;		/* # bytes read. */
int nwrite;		/* # bytes written. */
int indrive = 8;	/* Input tape drive number. */
int ncmp;
int mterr;
int nbyblk;		/* # bytes in physical tape block. */
int nrskip;		/* # logical records to skip. */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   fprintf (stderr, "program TARLOG : SMM C/P TAR tape log.\n");
   fprintf (stderr, "Mount input tape on either drive.\n");
   fprintf (stderr, "Input tape may be 1600 or 6250 BPI.\n");
   fprintf (stderr, "Enter input source [0 or 1] : ");
   fscanf (stdin, "%s", answer);
   if ((ncmp = strncmp("0",answer,1)) == 0)
      {
      indrive = 8;
      }
   else
   if ((ncmp = strncmp("1",answer,1)) == 0)
      {
      indrive = 9;
      }
   else
	{
	indrive = -1;
	sprintf (tapein, "%s", answer);
	}

   if (indrive >= 0)  sprintf(tapein,"/dev/rmt%d",indrive);

/* Open magnetic tape device file. */

   if((fdin = open(tapein, 0)) < 0)
     { fprintf(stderr,"tarlog> Can't open file %s\n", tapein);  return(NULL); }

/* Read first block from tape.  Determine block size. */

   if((nread = read (fdin, blkbuf, BLKSIZ)) == BLKSIZ)
	nbyblk = BLKSIZ;
   else if (nread >= RECSIZ)
	nbyblk = nread;
   else if (nread > 0)
	{
	fprintf (stderr,"Tape block size (%d) is too small.\n", nread);
	exit(-1);
	}
   else
	{
	fprintf (stderr,"EOF detected at beginning of tape.\n");
	exit (-1);
	}
   iblk++;
   nrecbl = nbyblk / RECSIZ;
   fprintf (stdout,"   filename         G M T        Q(%%)  IMG IH-IV RES EXP FW  PW  5303 SM CM ROLL\n");

/* Extract file name and size from header. */

next:
   if (irec > nrecbl)  readblock (fdin, blkbuf, nbyblk);
   strcpy (namefile, tarhdr (blkbuf+irec*RECSIZ, &imgsiz));
   irec++;

   nrskip = (int) ( ((float) imgsiz / (float) RECSIZ) + 0.99);

/* Write image summary line to output file. */

   if (irec > nrecbl)  readblock (fdin, blkbuf, nbyblk);
   cphdr (namefile, blkbuf+irec*RECSIZ);
   fhkline();

/* Skip remainder of data file. */

   cpskip (fdin, nrskip, blkbuf, nbyblk);
   goto next;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * R E A D B L O C K ( fdin, buf, nbyblk)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

readblock (fdin, buf, nbyblk)

int fdin;
char buf[BLKSIZ];
int nbyblk;

{
   int nread;

   if((nread = read (fdin, buf, nbyblk)) == 0)
	{
	fprintf (stderr,"readblock> EOF encountered.\n");
	exit (-1);
	}
   else
	{
	fprintf (stderr,"readblock> Illegal block size (%d).\n", nread);
	exit (-1);
	}

   iblk++;
   irec = 0;
   return(0);
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C P S K I P ( fd, nrskip )
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

cpskip (fd, nrskip, blkbuf, nbyblk)

int fd;		/* file descriptor for tape device. */
int nrskip;	/* # logical records to skip. */
char blkbuf[];	/* Buffer for tape block data. */
int nbyblk;	/* # bytes per tape block. */
{
   int nbskip;
   int i;
   int nread;

   if (nrskip < (nrecbl - irec))
	{
	irec += nrskip;
	return(0);
	}
   else
	{
	nrskip -= (nrecbl - irec);
	nbskip = (nrskip / nrecbl) + 1;
	for (i=0; i<nbskip; i++)
		{
		nread = read (fd, blkbuf, nbyblk);
		iblk++;
		}
	irec = nrskip - ((nbskip-1) * nrecbl);
	return(0);
	}
}
