/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * cp_jfsv.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Subtract Stray Light and/or Vignetting and/or F-corona from a C/P image.
 * Output difference image is floating point.
 * Output image is designated the same as the input image.
 * Input image may be short integer or floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 3 October 1991
 * 16 June 1999: Replace calcor (Munro calibration correction)
 *               with jupcor (Jupiter calibration correction).
 * 19 Dec 2005:  Convert main routine to sub-routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"
#define maxbyt 3584
#define maxhk   896
#define maxmap 2688

cp_jfsv (kcc, kfc, kstr, kvig, namimg)

int kcc ;		/* Apply Jupiter calibration correction. */
int kfc ;		/* Remove F-corona. */
int kstr ;		/* Remove stray light. */
int kvig ;		/* Remove vignetting. */
char *namimg ;		/* Input image file name. */

{
   float jupcor (), cpstray (), cpvignet (), fcorona ();

   static char namcor [80];
   static char hk [maxhk];
   static char bitmap [maxmap];

   unsigned char *buf1, *buf3;
   unsigned short *sbuf1;
   float *fbuf1, *fbuf3;

   int col;
   int idot;
   int fdin;
   int fdout;
   int i;
   int nbylin;			/* # bytes per line in input image. */
   int nbyimg, nbyout;
   int nbyhk;
   int nbymap;
   int nfblin;
   int row;

   float cc;
   float cexp;
   float fc;
   float r, th;
   float rsec ;
   float str;
   float subval;
   float vig;
   float x, y;

   /* Derive output file name from input image name. */

   idot = -1;
   for (i = 0; i < strlen (namimg); i++) if (*(namimg + i) == '.')  idot = i;
   if (idot < 0)  idot = strlen (namimg) - 1;
   idot = strlen (namimg);
   strcpy (namcor, namimg);
   namcor [idot] = '.';
   if (kcc  == 1 || H.cc )  namcor [idot+1] = 'j';  else namcor [idot+1] = 'x';
   if (kstr == 1 || H.str)  namcor [idot+2] = 's';  else namcor [idot+2] = 'x';
   if (kvig == 1 || H.vig)  namcor [idot+3] = 'v';  else namcor [idot+3] = 'x';
   if (kfc  == 1 || H.f  )  namcor [idot+4] = 'f';  else namcor [idot+4] = 'x';
   namcor [idot+5] = '\0';
   printf ("corrected image filename . . . namcor: %s\n", namcor);
   printf ("kcc: %d  kstr: %d  kvig: %d  kfc: %d\n", kcc, kstr, kvig, kfc) ;

   /*** Decode housekeeping records. ***/

   cphskp (namimg);
   nbylin = npix * nbypix;
   nfblin = npixl * sizeof (float);
   if (nbypix == 4)
      {
      nbyhk = npixl;
      nbymap = nfblin - npixl;
      }
   else
      nbyhk = nbymap = nbylin;
   cexp = pscale;
   printf ("H.rsun: %f\n", H.rsun) ;

   nbyimg = npix * npix * nbypix;
   nbyout = npix * npix * sizeof (float);
   buf1 = (unsigned char *) malloc (nbyimg);
   buf3 = (unsigned char *) malloc (nbyout);

   sbuf1 = (unsigned short *) buf1;
   fbuf1 = (float *) buf1;
   fbuf3 = (float *) buf3;

   /*** Check input image flags. ***/

   printf ("H.str: %d   H.vig: %d   H.reg: %d   H.f: %d\n",
           H.str, H.vig, H.reg, H.f);

   if (H.cc  == 1 && kcc  == 1)  kcc  = 0; /* Cal. correction already applied.*/
   if (H.f   == 1 && kfc  == 1)  kfc  = 0; /* F-corona already removed.       */
   if (H.str == 1 && kstr == 1)  kstr = 0; /* Stray Light already removed.    */
   if (H.vig == 1 && kvig == 1)  kvig = 0; /* Vignetting  already removed.    */

   if (kcc == 0 && kfc == 0 && kstr == 0 && kvig == 0)
      {
      fprintf (stderr, "No corrections need to be applied.\n") ;
      return (-1) ;
      }

   printf ("npix: %d   nbypix: %d   nbylin: %d\n", npix, nbypix, nbylin);
   printf ("nbyhk: %d   nbymap: %d\n",
	    nbyhk, nbymap);

   /*** Open input & output image files. ***/

    fdin = open (namimg, 0);
    if (fdin < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg);
       exit (-1);
       }
    fdout = creat (namcor, 0644);
    if (fdout < 0)
       {
       fprintf (stderr, "Unable to create %s\n", namcor);
       exit (-3);
       }

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin, hk, nbyhk);
   hk [52] = 0;				/* Sun byte order. */
   hk [103] = (char) 32;		/* # bits per pixel. */
   hk [110] = (npixl & 0177400) / 256;	/* # pixels per line (m.s.byte). */
   hk [111] = (npixl &    0377);	/* # pixels per line (l.s.byte). */
   hk [220] = (char) 0;			/* Scaling factor = 10^0 = 1.0 */
   hk [221] = (char) 32;		/* Bk image. */
   hk [222] = (char) 0;			/* Mask off data types 9-16. */
   hk [223] = hk [223] | (kstr *   1) ; /* Set Stray Light Flag. */
   hk [223] = hk [223] | (kvig *   2) ; /* Set Vignetting Flag. */
   hk [223] = hk [223] | (kfc  *   8) ; /* Set F-corona removal Flag. */
   hk [223] = hk [223] | (kcc  * 128) ; /* Set F-corona removal Flag. */

   write (fdout, hk, nbyhk);

   for (i=0; i<maxmap; i++) bitmap [i] = 0;
   read  (fdin, bitmap, nbymap);
   write (fdout, bitmap, nfblin-nbyhk);

   /*** Read input image into memory array. ***/

   read (fdin, buf1, nbyimg);
   if (nbypix == 2 && bytord == 1) srevbyt (sbuf1, nbypix, nbyimg);

   /*** Subtract stray light from image. ***/

   cc  = 1.0;
   str = 0.0;
   vig = 1.0;
   fc  = 0.0;

   for (row = 0; row < npix; row++)
      {

      /*** Short Integer Image. ***/

      if (nbypix == 2)
         for (col=0; col<npix; col++)
            {
            i = col + row * npix;
            x = (float) col;
            y = (float) row;
            rcoord (&r, &th, &x, &y, -1, scroll);	/* Get radius [Rsun]. */
	    rsec = r * H.rsun ;				/* Radius [arcsec]    */
            if (kcc  == 1)                cc  = jupcor (r) * H.pscale;
            if (kstr == 1 && H.str == 0)  str = cpstray  (rsec);
            if (kvig == 1 && H.vig == 0)  vig = cpvignet (rsec);
            if (kfc  == 1 && H.f  == 0)   fc  = fcorona (r);
            if (col == row)
	       {
               fprintf (stdout,
                  "r: %6.2f   rsec: %f  str: %g   vig: %g   cc: %g   fc: %g\n",
                   r, rsec, str, vig, cc, fc);
               }

            if (*(sbuf1 + i) != 0)
               *(fbuf3 + i) = (((float) *(sbuf1 + i) * cc - str) * vig) - fc;
            else
               *(fbuf3 + i) = 0;
            if ((*fbuf3 + i) < 0.0)  *(fbuf3 + i) = 0.0 ;
            }

      /*** Floating Point Image. ***/

      else
      if (nbypix == 4)
         {
         for (col=0; col<npix; col++)
            {
            i = col + row * npix;
            x = (float) col;
            y = (float) row;
            rcoord (&r, &th, &x, &y, -1, scroll);
	    rsec = r * H.rsun ;
            if (kcc  == 1)                cc  = jupcor (r) * H.pscale;
            if (kstr == 1 && H.str == 0)  str = cpstray  (rsec);
            if (kvig == 1 && H.vig == 0)  vig = cpvignet (rsec);
            if (kfc  == 1 && H.f   == 0)  fc  = fcorona (r);
            if (col == row)
               fprintf (stdout,
                  "r: %6.2f   str: %g   vig: %g   cc: %g   fc: %g\n",
                   r, str, vig, cc, fc);

            if (*(fbuf1 + i) != 0.0)
               *(fbuf3 + i) = (((float) *(fbuf1 + i) * cc - str) * vig) - fc;
            else
               *(fbuf3 + i) = 0.0;
            if ((*fbuf3 + i) < 0.0)  *(fbuf3 + i) = 0.0 ;
            }
/*
 *         fprintf (stdout, "row: %3d   fbuf1: %g   fbuf3: %g\n",
 *                  row, *(fbuf1 + row + row*npix), *(fbuf3 + row + row*npix));
 */

         }
      }
   write (fdout, fbuf3, nbyout);

   close (fdin);
   close (fdout);
   return (0);
}
