/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * cpjfsv.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Subtract Stray Light and/or Vignetting and/or F-corona from a C/P image.
 * Output difference image is floating point.
 * Output image is designated the same as the input image.
 * Input image may be short integer or floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.h"

main (argc, argv)

int argc;
char *argv [];

{
   char *parg ;			/* Argument pointer. */
   static char name [80] ;	/* File name. */
   static char namimg [80] ;	/* Image file name. */
   static char namlst [80] ;	/* List  file name. */

   int kcc  ;			/* =1: Apply  Calibration Correction. */
   int kfc  ;			/* =1: Remove F-corona.    */
   int kstr ;			/* =1: Remove Stray Light. */
   int kvig ;			/* =1: Remove Vignetting.  */

   FILE *LIST ;

   if (argc < 2)		/* Need at least 2 arguments: option & file. */
      {
      usage (argv [0]) ;
      exit (0) ;
      }

   while (--argc)		/* Argument processing loop. */
      {
      parg = *++argv ;

      if (*(parg) == '?')	/* Help. */
	 {
	 usage (argv [0]) ;
	 exit (0) ;
	 }

      if (*(parg) == '-')		/* Check for option arguments. */
         {
         switch (parg [1])
	    {
	    case 'c': kcc  = 1 ;   break ;	/* Set Jupiter Correction  flag. */
	    case 'f': kfc  = 1 ;	break ;	/* Set F-corona    removal flag. */
	    case 's': kstr = 1 ;	break ;	/* Set stray light removal flag. */
	    case 'v': kvig = 1 ;	break ;	/* Set vignetting  removal flag. */
	    default:  usage (argv [0]);
	    }
         continue ;
         }
      else		/* Get the filename. */
         {
         strcpy (name, parg) ;
         if (name [0] == '@')           /* Read list file. */
            {
            strcpy (namlst, &name [1]) ;
            if ((LIST = fopen (namlst, "r")) == NULL)
               {
               fprintf (stderr, "Unable to open file: %s\n", namlst) ;
               exit (-2) ;
	       }

            while (fscanf (LIST, "%s", namimg) != EOF)
	       {
               cp_jfsv (kcc, kfc, kstr, kvig) ;
               }
            fclose (LIST) ;
            }
         else
	    {
	    cp_jfsv (kcc, kfc, kstr, kvig) ;
	    }
         }
      }
   exit (0) ;
}

usage (nampgm)

char *nampgm ;
{
   fprintf (stderr, "%s  [-j][-f][-s][-v] file1 file2 file ...\n", nampgm) ;
   return (0) ;
}
