/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fdeltab.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute (B - Bzero) / Bzero.
 * B =     brightness image (event).
 * Bzero = brightness image (background).
 * Input images must be calibrated (short int), C/P format.
 * Output is a floating point image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 January 1989
 * 17 April 1989
 * 15 February 1990: Set bit #0 in byte 222 to indicate delta B / B.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxhk 896 
#define maxmap 2688
#define maxpix 896 

main (argc, argv)

int argc ;
char *argv [] ;
{
   static char namcme [80];
   static char nambkg [80];
   static char namout [80];
   static char hk [maxhk];
   static char bitmap [maxmap];

   static short ibufe [maxpix], ibufb [maxpix];
   static float linbuf [maxpix];

   int bytordb, bytorde;
   int fdout;
   int fdcme, fdbkg;
   int nbylin;			/* # bytes per line in input images. */
   int nfblin;			/* # bytes per line in output image. */
   int nbypixb, nbypixe;
   int nbyhkb, nbyhke;
   int nbymapb, nbymape;

   float x, y, radius, theta;
   float difabs;
   float eps = 1.e-11;

   register short *lincme, *linbkg;
   register int i, line;
   register float difval, *lindif;

   lincme = ibufe;
   linbkg = ibufb;
   lindif = linbuf;

   fprintf (stdout, "fdeltab: [B(cme) - B(bkg)] / B(bkg).\n\n");

   if (argc == 3)
      {
      strcpy (namcme, argv [1]) ;
      strcpy (nambkg, argv [2]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s  cme_name  bkg_name\n", argv [0]) ;
      exit (0) ;
      }

   strcpy (namout, namcme);
   strcat (namout, ".d");
   if (strlen (nambkg) >= 8)  strncat (namout, &nambkg [5], 4);
      else strcat (namout, "rt");
   fprintf (stderr, "Output file name: %s\n", namout);

   /*** Open input & output image files. ***/

   cphskp (nambkg);
   bytordb = bytord;
   nbypixb = nbypix;
   nbylin = npixl * nbypix;
   nfblin = npixl * sizeof (float);
   if (nbypix == 4)
      {
      nbyhkb = npixl;
      nbymapb = nfblin - npixl;
      }
   else
      nbyhkb = nbymapb = nbylin;

   cphskp (namcme);
   bytorde = bytord;
   nbypixe = nbypix;
   nbylin = npixl * nbypix;
   nfblin = npixl * sizeof (float);
   if (nbypix == 4)
      {
      nbyhke = npixl;
      nbymape = nfblin - npixl;
      }
   else
      nbyhke = nbymape = nbylin;

   printf ("npixl: %d   nbypix: %d   nbylin: %d\n", npixl, nbypix, nbylin);

   fdcme = open (namcme, 0);
   fdbkg = open (nambkg, 0);
   fdout = creat (namout, 0644);
   if (fdcme < 0)
   {fprintf (stderr, "Unable to open %s\n", namcme);  exit (-1);}
   fdbkg = open (nambkg, 0);
   if (fdbkg < 0)
   {fprintf (stderr, "Unable to open %s\n", nambkg);  exit (-2);}
   fdout = creat (namout, 0644);
   if (fdout < 0)
   {fprintf (stderr, "Unable to creat %s\n", namout);  exit (-3);}

   /*** Copy housekeeping & bitmap records. ***/

   for (i=0; i<maxhk; i++)  hk [i] = (char) 0;
   for (i=0; i<maxmap; i++)  bitmap [i] = (char) 0;
   read (fdcme, hk, nbyhke);
   read (fdcme, bitmap, nbymape);
   hk [52] = 0;				/* Sun byte order. */
   hk [103] = (char) 32;		/* # bits per pixel. */
   hk [110] = (npix & 0177400) / 256;
   hk [111] = (npix &    0377);
   hk [220] = '\0';
   hk [221] = (char) 128;		/* Set bit to indicate delta B / B */
   hk [222] = '\0';
   write (fdout, hk, npixl);
   write (fdout, bitmap, (nfblin - npixl));

   read (fdbkg, hk, nbyhkb);
   read (fdbkg, bitmap, nbymapb);

   /*** Subtract background frame from event frame. ***/
   /*** Divide result by event frame and store as a short integer image. ***/

   for (line=0; line<npixl; line++)
      {
      read (fdcme, ibufe, nbylin);
      read (fdbkg, ibufb, nbylin);
      if (nbypixe == 2 && bytorde == 1)
         srevbyt (ibufe, nbypix, npixl);
      if (nbypixb == 2 && bytordb == 1)
         srevbyt (ibufb, nbypix, npixl);

      for (i=0; i<npixl; i++)
	 {
	 difval = (float) (*(lincme + i) - *(linbkg + i));
	 if (difval < 0.0)
	    difabs = -difval;
         else
	    difabs = difval;

         if ((float) *(linbkg + i) > eps && difabs > eps)
	     *(lindif + i) =
	       (float) (difval / (float) *(linbkg + i));
         else
	    *(lindif + i) = 0.0;

         if (*(lindif + i) > 3.0)
	    {
            rcoord (&radius, &theta, &x, &y, -1, scroll);
	    if (radius > 4.0)  *(lindif + i) = 3.0;
            }
         if (*(lindif + i) < -3.0)
	    {
            rcoord (&radius, &theta, &x, &y, -1, scroll);
	    if (radius > 4.0)  *(lindif + i) = -3.0;
	    }
         }

/*
 *      fprintf (stdout, "line %3d:   cme: %d   bkg: %d   delta: %f\n",
 *	       line, *(lincme + line), *(linbkg + line), *(lindif + line));
 */
      write (fdout, linbuf, nfblin);
      }

   close (fdcme);
   close (fdbkg);
   close (fdout);
}
