/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fdeltabk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute (Bcme - Bbkg) / Bbkg[k].
 * Bcme    = brightness image (event).
 * Bbkg    = brightness image (background).
 * Bbkg[k] = K-corona brightness of background image.
 * All input images must be calibrated (short int), C/P format.
 * NOTE::> Output is a floating point image !
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 January 1989
 * 17 April 1989
 * 15 February 1990: Set bit #0 in byte 222 to indicate delta B / B.
 * 18 April 1990: divide by Bk image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxhk 1792
#define maxmap 2688
#define maxpix 896

main (argc, argv)

int argc ;
char *argv [] ;
{
   static char namcme [80];
   static char nambkg [80];
   static char nambk  [80];
   static char namout [80];
   static char hk [maxhk];
   static char bitmap [maxmap];

   static short ibufe [maxpix], ibufb [maxpix], ibufk [maxpix];
   static float linbuf [maxpix];

   int bytordb, bytorde;
   int fdout;
   int fdcme, fdbkg;
   int nbylin;			/* # bytes per line in input images. */
   int nfblin;			/* # bytes per line in output image. */
   int nbypixb, nbypixe;
   int bytordk, nbypixk, fdbk;

   float x, y, radius, theta;

   register short *lincme, *linbkg, *linbk;
   register int i, line;
   register float difval, *lindif;

   lincme = ibufe;
   linbkg = ibufb;
   linbk  = ibufk;
   lindif = linbuf;

   fprintf (stdout, "deltab: [B(cme) - B(bkg)] / B(cme).\n\n");

   /*** Request names of image files. ***/

   if (argc == 4)
      {
      strcpy (namcme, argv [1]) ;
      strcpy (nambkg, argv [2]) ;
      strcpy (nambk , argv [3]) ;
      }

   else
      {
      fprintf (stderr, "usage: %s  cme_name  bkg_name  Bk_name\n", argv [0]) ;
      exit  (0) ;
      }

   strcpy (namout, namcme);
   strcat (namout, ".k");
   if (strlen (nambkg) >= 9)  strncat (namout, &nambkg [5], 4);
      else strcat (namout, "corona");
   fprintf (stderr, "Output file name: %s\n", namout);

   /*** Decode housekeeping records. ***/

   cphskp (nambkg);
   bytordb = bytord;
   nbypixb = nbypix;

   cphskp (namcme);
   bytorde = bytord;
   nbypixe = nbypix;

   cphskp (nambk);
   bytordk = bytord;
   nbypixk = nbypix;

   nbylin = npixl * nbypix;
   nfblin = npixl * sizeof (float);
   printf ("npixl: %d   nbypix: %d   nbylin: %d\n", npixl, nbypix, nbylin);

   /*** Open input & output image files. ***/

   fdcme = open (namcme, 0);
   if (fdcme < 0)
      {fprintf (stderr, "Unable to open %s\n", namcme);  exit (-1);}
   fdbkg = open (nambkg, 0);
   if (fdbkg < 0)
      {fprintf (stderr, "Unable to open %s\n", nambkg);  exit (-2);}
   fdout = creat (namout, 0644);
   if (fdout < 0)
      {fprintf (stderr, "Unable to creat %s\n", namout);  exit (-3);}
   fdbk = open (nambk, 0);
   if (fdbk < 0)
      {fprintf (stderr, "Unable to open %s\n", nambk);  exit (-4);}

   fprintf (stderr, "fdcme: %d   fdbkg: %d   fdout: %d   fdbk: %d\n",
	    fdcme, fdbkg, fdout, fdbk);

   /*** Copy housekeeping & bitmap records. ***/

   for (i=0; i<maxhk; i++)  hk [i] = (char) 0;
   for (i=0; i<maxmap; i++)  bitmap [i] = (char) 0;
   read (fdcme, hk, nbylin);
   read (fdcme, bitmap, nbylin);
   hk [52] = 0;				/* Sun byte order. */
   hk [103] = (char) 32;		/* # bits per pixel. */
   hk [110] = (npix & 0177400) / 256;
   hk [111] = (npix &    0377);
   hk [220] = (char) 0;			/* Scaling factor = 1.0 */
   hk [221] = (char) 64;		/* Set bit to indicate delta B / Bk */
   hk [222] = (char) 0;			/* Clear all data types 9-16. */
   write (fdout, hk, npixl);
   write (fdout, bitmap, (nfblin - npixl));

   read (fdbkg, hk, nbylin);
   read (fdbkg, bitmap, nbylin);

   read (fdbk, hk, nbylin);
   read (fdbk, bitmap, nbylin);

   /***
    *** Subtract background image from event image.
    *** Divide result by background K-corona image and store
    *** as a short integer image.
    ***/

   for (line=0; line<npixl; line++)
      {
      y = (float) line;
      read (fdcme, lincme, nbylin);	/* Read one line of CME image. */
      read (fdbkg, linbkg, nbylin);	/* Read one line of BKG image. */
      read (fdbk,  linbk,  nbylin);	/* Read one line of Bk  image. */
      if (nbypixe == 2 && bytorde == 1)	/* Reverse byte order if needed. */
         srevbyt (lincme, nbypix, npixl);
      if (nbypixb == 2 && bytordb == 1)
         srevbyt (linbkg, nbypix, npixl);
      if (nbypixk == 2 && bytordk == 1)
         srevbyt (linbk,  nbypix, npixl);

      for (i=0; i<npixl; i++)
	 {
	 x = (float) i;
	 difval = (float) (*(lincme + i) - *(linbkg + i));
         if (*(linbk  + i) > 1.e-11)
            *(lindif + i) = (float) (difval / (float) *(linbk  + i));
         else
	    *(lindif + i) = 0.0;

         if (*(lindif + i) > 3.0)
	    {
            rcoord (&radius, &theta, &x, &y, -1, scroll);
	    if (radius > 4.0)  *(lindif + i) = 3.0;
            }
         if (*(lindif + i) < -3.0)
	    {
            rcoord (&radius, &theta, &x, &y, -1, scroll);
	    if (radius > 4.0)  *(lindif + i) = -3.0;
	    }
         }

/*
 *      fprintf (stdout, "line %3d:   cme: %d   bkg: %d   delta: %f\n",
 *	       line, *(lincme + line), *(linbkg + line), *(lindif + line));
 */
      write (fdout, lindif, nfblin);
      }

   close (fdcme);
   close (fdbkg);
   close (fdbk);
   close (fdout);
}
