/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fdeltai.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute fractional difference between two C/P images.
 * Output image is floating point.
 * Either input image may be char, short, or float.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 24 Jan 1991
 * 12 Feb 1991: change divisor from image 1 to image 2.
 *  5 August 1992: store ref image date & time in bytes 143-148.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxbyt 3584
#define maxhk   896
#define maxmap 2688

main (argc, argv)

int argc ;
char *argv [] ;
{
   static char namimg1 [80];
   static char namimg2 [80];
   static char namout  [80];
   static char hk1 [maxhk], hk2 [maxhk];
   static char bitmap [maxmap];

   static unsigned char buf1 [maxbyt], buf2 [maxbyt], buf3 [maxbyt];
   static unsigned char *cbuf1, *cbuf2;
   static unsigned short *sbuf1, *sbuf2;
   static float *fbuf1, *fbuf2, *fbuf3;

   int bytord1, bytord2;
   int idot;
   int fdin1, fdin2;
   int fdout;
   int nbylin1, nbylin2;		/* # bytes per line in input images. */
   int nfblin;			/* # bytes per line in output image. */
   int nbypix1, nbypix2;
   int npixl1, npixl2;
   int nbyhk1, nbyhk2, nbyhk3;
   int nbymap1, nbymap2, nbymap3;

   register int i, line;
   register float subval;

   float cexp1, cexp2;

   cbuf1 = (unsigned char  *) buf1;
   cbuf2 = (unsigned char  *) buf2;
   sbuf1 = (unsigned short *) buf1;
   sbuf2 = (unsigned short *) buf2;
   fbuf1 = (float *) buf1;
   fbuf2 = (float *) buf2;
   fbuf3 = (float *) buf3;

   fprintf (stdout,
      "fdeltai> fractional difference: [img1 - img2] / img2.\n\n");

   if (argc == 3)
      {
      strcpy (namimg1, argv [1]) ;
      strcpy (namimg2, argv [2]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s  img1_name  img2_name\n", argv [0]) ;
      exit (0) ;
      }

/*
   strcpy (namout, namimg1);
   for (i = 0; i < strlen (namimg1); i++)
      {
      idot = i;
      if (namimg1 [i] == '.')  break;
      }
   namout [idot+1] = '\0';
   strcat (namout, "f");
   if (strlen (namimg2) >= 9)  strncat (namout, &namimg2 [5], 4);
      else strcat (namout, "db");
   printf ("namimg1: %s   namimg2: %s\n", namimg1, namimg2);
 */

   strcpy (namout, namimg1);
   strcat (namout, ".i");
   if (strlen (namimg2) >= 9)  strncat (namout, &namimg2 [5], 4);
      else strcat (namout, "deltab");

   fprintf (stdout, "Output image file name : %s\n", namout);

   /*** Decode housekeeping image 1. ***/

   cphskp (namimg1);
   bytord1 = bytord;
   nbypix1 = nbypix;
   npixl1 = npixl;
   nbylin1 = npixl1 * nbypix1;
   nfblin = npixl1 * sizeof (float);
   nbyhk3  = npixl ;
   nbymap3 = nfblin - nbyhk3 ;

   if (nbypix == 4)
      {
      nbyhk1 = npixl;
      nbymap1 = nfblin - npixl;
      }
   else
      nbyhk1 = nbymap1 = nbylin1;
   cexp1 = pscale;

   if (nbypix1 != 1 && nbypix1 != 2 && nbypix1 != 4)
      {
      fprintf (stderr, "Invalid image (%s): nbypix1=%d\n",
               namimg1, nbypix1);
      exit (-1);
      }

   /*** Decode housekeeping image 2. ***/

   cphskp (namimg2);
   bytord2 = bytord;
   nbypix2 = nbypix;
   npixl2 = npixl;
   nbylin2 = npixl2 * nbypix2;

   if (nbypix == 4)
      {
      nbyhk2 = npixl;
      nbymap2 = nfblin - npixl;
      }
   else
      nbyhk2 = nbymap2 = nbylin2;
   cexp2 = pscale;

   if (nbypix2 != 1 && nbypix2 != 2 && nbypix2 != 4)
      {
      fprintf (stderr, "Invalid image (%s): nbypix1=%d\n",
               namimg2, nbypix2);
      exit (-2);
      }

   if (npixl1 != npixl2)
      {
      fprintf (stderr, "size of %s (%d) != size of %s (%d)\n",
	       namimg1, npixl1, namimg2, npixl2);
      exit (-3);
      }
   fprintf (stdout, "npixl1: %d  nbypix1: %d  nbylin1: %d\n",
	    npixl1, nbypix1, nbylin1);
   fprintf (stdout, "npixl2: %d  nbypix2: %d  nbylin2: %d\n",
	    npixl2, nbypix2, nbylin2);
   fprintf (stdout, "nbyhk1: %d  nbymap1: %d  nbyhk2:  %d  nbymap2: %d\n",
	    nbyhk1, nbymap1, nbyhk2, nbymap2);

   /*** Open input & output image files. ***/

    fdin1 = open (namimg1, 0);
    if (fdin1 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg1);
       exit (-4);
       }
    fdin2 = open (namimg2, 0);
    if (fdin2 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg2);
       exit (-5);
       }
    fdout = creat (namout, 0644);
    if (fdout < 0)
       {
       fprintf (stderr, "Unable to create %s\n", namout);
       exit (-6);
       }

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin1, hk1, nbyhk1);
   read (fdin2, hk2, nbyhk2);
   hk1 [52] = 0;			/* Sun byte order. */
   hk1 [103] = (char) 32;		/* # bits per pixel. */
   hk1 [110] = (npixl & 0177400) / 256;	/* # pixels per line (m.s.byte). */
   hk1 [111] = (npixl &    0377);	/* # pixels per line (l.s.byte). */
   hk1 [220] = (char) 0;		/* Scaling factor = 1.0 */
   hk1 [221] = (char) 128;		/* Delta B (frac. difference image). */
   hk1 [222] = (char) 0;;		/* Mask off data types 9-16. */

   /*** Copy date & time of image #2 into hskp bytes #143-148 for dif image. */

   hk1 [143] = hk2 [0];			/* Store year2 in byte #143 of hk1. */
   hk1 [144] = hk2 [1];			/* Store doy2a in byte #144 of hk1. */
   hk1 [145] = hk2 [2];			/* Store doy2b in byte #145 of hk1. */
   hk1 [146] = hk2 [20];		/* Store hour2 in byte #146 of hk1. */
   hk1 [147] = hk2 [21];		/* Store min2  in byte #147 of hk1. */
   hk1 [148] = hk2 [24];		/* Store sec2  in byte #148 of hk1. */

   write (fdout, hk1, nbyhk3);

   read  (fdin1, bitmap, nbymap1);
   write (fdout, bitmap, nbymap3);

   read  (fdin2, bitmap, nbymap2);

   /*** Fractional difference between image #2 and image #1. ***/
   /*** Store result as a floating point image. ***/

   for (line = 0; line < npixl1; line++)
      {
      read (fdin1, buf1, nbylin1);
      read (fdin2, buf2, nbylin2);
      if (nbypix1 == 2 && bytord1 == 1) srevbyt (sbuf1, nbypix1, npixl1);
      if (nbypix2 == 2 && bytord2 == 1) srevbyt (sbuf2, nbypix2, npixl2);

      /*** Both images are characters. ***/

      if (nbypix1 == 1 && nbypix2 == 1)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(cbuf1 + i) == 0 || *(cbuf2 + i) == 0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(cbuf1 + i)) * cexp1)
	                  - (((float) *(cbuf2 + i)) * cexp2))
                          / (((float) *(cbuf2 + i)) * cexp2);
	 }

      /*** Image #1 is character, image #2 is short integer. ***/

      else
      if (nbypix1 == 1 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(cbuf1 + i) == 0 || *(sbuf2 + i) == 0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(cbuf1 + i)) * cexp1)
	                  - (((float) *(sbuf2 + i)) * cexp2))
		          / (((float) *(cbuf2 + i)) * cexp2);
	 }

      /*** Image #1 is character, image #2 is floating point. ***/

      else
      if (nbypix1 == 1 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(cbuf1 + i) == 0 || *(fbuf2 + i) == 0.0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(cbuf1 + i)) * cexp1)
	                  - (((float) *(fbuf2 + i)) * cexp2))
		          / (((float) *(cbuf2 + i)) * cexp2);
	 }

      /*** Image #1 is short integer, image #2 is character. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 1)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(sbuf1 + i) == 0 || *(cbuf2 + i) == 0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(sbuf1 + i)) * cexp1)
	                  - (((float) *(cbuf2 + i)) * cexp2))
		          / (((float) *(sbuf2 + i)) * cexp2);
         }

      /*** Both images are short integers. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(sbuf1 + i) == 0 || *(sbuf2 + i) == 0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(sbuf1 + i)) * cexp1)
	                  - (((float) *(sbuf2 + i)) * cexp2))
		          / (((float) *(sbuf2 + i)) * cexp2);
         }

      /*** Image #1 is short integer, image #2 is floating point. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(sbuf1 + i) == 0  || *(fbuf2 + i) == 0.0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(sbuf1 + i)) * cexp1)
	                  - (((float) *(fbuf2 + i)) * cexp2))
		          / (((float) *(sbuf2 + i)) * cexp2);
         }

      /*** Image #1 is floating point, image #2 is character. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 1)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(fbuf1 + i) == 0.0 || *(cbuf2 + i) == 0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(fbuf1 + i)) * cexp1)
	                  - (((float) *(cbuf2 + i)) * cexp2))
		          / (((float) *(fbuf2 + i)) * cexp2);
         }

      /*** Image #1 is floating point, image #2 is short integer. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(fbuf1 + i) == 0.0 || *(sbuf2 + i) == 0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(fbuf1 + i)) * cexp1)
	                  - (((float) *(sbuf2 + i)) * cexp2))
		          / (((float) *(fbuf2 + i)) * cexp2);
         }

      /*** Image #1 is floating point, image #2 is floating point. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
	 {
	 if (*(fbuf1 + i) == 0.0 || *(fbuf2 + i) == 0.0)
	    *(fbuf3 + i) = 0.0;
          else
            *(fbuf3 + i) = ((((float) *(fbuf1 + i)) * cexp1)
	                  - (((float) *(fbuf2 + i)) * cexp2))
		          / (((float) *(fbuf2 + i)) * cexp2);
         }

/*      fprintf (stdout, "fbuf3 [%3d]: %d\n", line, *(fbuf3 + line)); */
      write (fdout, fbuf3, nfblin);
      }

   close (fdin1);
   close (fdin2);
   close (fdout);
}
