/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * ffdelbk.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute (Bcme - Bbkg) / Bbkg[k].
 * Bcme    = brightness image (event).
 * Bbkg    = brightness image (background).
 * Bbkg[k] = K-corona brightness of background image.
 * Input images must be in floating point, calibrated, C/P format.
 * NOTE::> Output is a floating point image !
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 19 April 1990
 *  5 August 1992: store ref image date & time in bytes 143-148.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxhk  1792
#define maxmap 2688
#define maxpix 896

main (argc, argv)

int argc ;
char *argv [] ;
{
   static char namcme [80];
   static char nambkg [80];
   static char nambk  [80];
   static char namout [80];
   static char hke [maxhk];
   static char hkb [maxhk];
   static char bitmap [maxmap];

   static float ibufe [maxpix], ibufb [maxpix], ibufk [maxpix];
   static float linbuf [maxpix];

   int bytordb, bytorde;
   int fdout;
   int fdcme, fdbkg;
   int nbylin;		/* # bytes per line in input images. */
   int nbylin2;		/* record size for hskp & bitmap for float images. */
   int nfblin;		/* # bytes per line in output image. */
   int nbypixb, nbypixe;
   int bytordk, nbypixk;

   float x, y, radius, theta;

   register float *lincme, *linbkg, *linbk;
   register int i, line;
   register float difval, *lindif;

   lincme = ibufe;
   linbkg = ibufb;
   linbk  = ibufk;
   lindif = linbuf;

   fprintf (stdout, "ffdelbk: [B(cme) - B(bkg)] / B(bkg).\n\n");

   if (argc == 3)
      {
      strcpy (namcme, argv [1]) ;
      strcpy (nambkg, argv [2]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s  cme_name  bkg_name\n", argv [0]) ;
      exit (0) ;
      }

   strcpy (namout, namcme);
   strcat (namout, ".k");
   if (strlen (nambkg) >= 9)  strncat (namout, &nambkg [5], 4);
      else strcat (namout, "deltab");

   /*** Decode housekeeping records. ***/

   cphskp (nambkg);
   bytordb = bytord;
   nbypixb = nbypix;

   cphskp (namcme);
   bytorde = bytord;
   nbypixe = nbypix;

/*
 *   cphskp (nambk);
 *   bytordk = bytord;
 *   nbypixk = nbypix;
 */

   nbylin = npixl * nbypix;
   nbylin2 = nbylin / 2;
   nfblin = npixl * sizeof (float);
   printf ("npixl: %d   nbypix: %d   nbylin: %d\n", npixl, nbypix, nbylin);

   /*** Open input & output image files. ***/

   if (fdout = creat (namout, 0644) < 0)
      {
      fprintf (stderr, "Unable to create %s\n", namout);
      exit (-1);
      }
   fdcme = open (namcme, 0);
   if (fdcme < 0)
      {
      fprintf (stderr, "Unable to open %s\n", namcme);
      exit (-1);
      }
   fdbkg = open (nambkg, 0);
   if (fdbkg < 0)
      {
      fprintf (stderr, "Unable to open %s\n", namcme);
      exit (-2);
      }
   fdout = creat (namout, 0644);
   if (fdout < 0)
      {
      fprintf (stderr, "Unable to create %s\n", namout);
      exit (-3);
      }

   /*** Copy housekeeping & bitmap records. ***/

   for (i=0; i<maxhk; i++)  hke [i] = (char) 0;		/* Clear buffer. */
   for (i=0; i<maxhk; i++)  hkb [i] = (char) 0;		/* Clear buffer. */
   for (i=0; i<maxmap; i++)  bitmap [i] = (char) 0;	/* Clear buffer. */
   read (fdcme, hke, npixl);
   read (fdcme, bitmap, (nfblin - npixl));
   read (fdbkg, hkb, npixl);
   hke [52] = 0;			/* Sun byte order. */
   hke [103] = (char) 32;		/* # bits per pixel. */
   hke [110] = (npixl & 0177400) / 256;
   hke [111] = (npixl &    0377);
   hke [220] = (char) 0;		/* Scaling factor = 1.0 */
   hke [221] = (char) 64;		/* Set bit to indicate delta B / Bk */

   /*** Copy date & time of image #2 into hskp bytes #143-148 for dif image. */

   hke [143] = hkb [0];			/* Store year2 in byte #143 of hke. */
   hke [144] = hkb [1];			/* Store doy2a in byte #144 of hke. */
   hke [145] = hkb [2];			/* Store doy2b in byte #145 of hke. */
   hke [146] = hkb [20];		/* Store hour2 in byte #146 of hke. */
   hke [147] = hkb [21];		/* Store min2  in byte #147 of hke. */
   hke [148] = hkb [24];		/* Store sec2  in byte #148 of hke. */

   write (fdout, hke, npixl);
   write (fdout, bitmap, (nfblin - npixl));

   read (fdbkg, bitmap, (nfblin - npixl));

   /***
    *** Subtract background image from event image.
    *** Divide result by background K-corona image and store
    *** as a short integer image.
    ***/

   for (line=0; line<npixl; line++)
      {
/*      printf ("line: %d\n", line); */
      y = (float) line;
      read (fdcme, lincme, nbylin);	/* Read one line of CME image. */
      read (fdbkg, linbkg, nbylin);	/* Read one line of BKG image. */

      for (i=0; i<npixl; i++)
	 {
	 x = (float) i;
	 difval = *(lincme + i) - *(linbkg + i);
         if (*(linbkg + i) != 0)
            *(lindif + i) = difval / *(linbkg  + i);
         else
	    *(lindif + i) = 0.0;

         if (*(lindif + i) > 1.0)
	    {
            rcoord (&radius, &theta, &x, &y, -1, scroll);
	    if (radius > 4.0)  *(lindif + i) = 1.0;
            }
         if (*(lindif + i) < -1.0)
	    {
            rcoord (&radius, &theta, &x, &y, -1, scroll);
	    if (radius > 4.0)  *(lindif + i) = -1.0;
	    }
         }

/*
 *      fprintf (stdout, "line %3d:   cme: %d   bkg: %d   delta: %f\n",
 *	       line, *(lincme + line), *(linbkg + line), *(lindif + line));
 */
      write (fdout, lindif, nfblin);
      }

   close (fdcme);
   close (fdbkg);
   close (fdout);
}
