/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fmassdif.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute the excess mass of two SMM C/P images.
 * Output image is floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR   11 May 1999 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxbyt 3584
#define maxhk   896
#define maxmap 2688

main (argc, argv)

int argc;
char *argv [];
{
   static char namimg1 [80];
   static char namimg2 [80];
   static char namout [80];
   static char namtmp [80];
   static char hk1 [maxhk], hk2 [maxhk];
   static char bitmap [maxmap];

   static unsigned char buf1 [maxbyt], buf2 [maxbyt], buf3 [maxbyt];
   static unsigned char *cbuf1, *cbuf2;
   static unsigned short *sbuf1, *sbuf2;
   static float *fbuf1, *fbuf2, *fbuf3;

   int bytord1, bytord2;
   int idot;
   int fdin1, fdin2;
   int fdout;
   int nbylin1, nbylin2;	/* # bytes per line in input images. */
   int nfblin;			/* # bytes per line in output image. */
   int nbypix1, nbypix2;
   int npixl1, npixl2;
   int nbyhk1, nbyhk2;
   int nbymap1, nbymap2;

   register int i, line;
   register float subval;
   float cexp1, cexp2;

   cbuf1 = (unsigned char  *) buf1;
   cbuf2 = (unsigned char  *) buf2;
   sbuf1 = (unsigned short *) buf1;
   sbuf2 = (unsigned short *) buf2;
   fbuf1 = (float *) buf1;
   fbuf2 = (float *) buf2;
   fbuf3 = (float *) buf3;

   if (argc == 3)
      {
      strcpy (namimg1, argv [1]);
      strcpy (namimg2, argv [2]);
      }
   else
      {
      fprintf (stderr, "%s: subtract one C/P image from another [img1 - img2].\n\n",
               argv [0]);
      fprintf (stderr, "usage: %s image1 image2\n", argv [0]);
      exit (0);
      }

/*
   strcpy (namout, namimg1);
   for (i = 0; i < strlen (namimg1); i++)
      {
      idot = i;
      if (namimg1[i] == '.')  break;
      }
   namout [idot+1] = '\0';

   strcat (namout, "s");
   if (strlen (namimg2) >= 9)  strncat (namout, &namimg2 [5], 4);
      else strcat (namout, "ub");
   printf ("namimg1: %s   namimg2: %s\n", namimg1, namimg2);

   fprintf (stdout, "Difference image file name : %s\n", namout);
 */

   /*** Decode housekeeping records. ***/

   cphskp (namimg1);
   bytord1 = bytord;
   nbypix1 = nbypix;
   npixl1 = npixl;
   nbylin1 = npixl1 * nbypix1;
   nfblin = npixl1 * sizeof (float);
   if (nbypix == 4)
      {
      nbyhk1 = npixl;
      nbymap1 = nfblin - npixl;
      }
   else
      nbyhk1 = nbymap1 = nbylin1;
   cexp1 = pscale;
   if (nbypix1 != 1 && nbypix1 != 2 && nbypix1 != 4)
      {
      fprintf (stderr, "Invalid image (%s): nbypix1=%d\n",
               namimg1, nbypix1);
      exit (-1);
      }
   sprintf (namtmp, "%02d%03d.%02d%02d-", H.year, H.doy, H.hour, H.minute);

   cphskp (namimg2);
   bytord2 = bytord;
   nbypix2 = nbypix;
   npixl2 = npixl;
   nbylin2 = npixl2 * nbypix2;
   if (nbypix == 4)
      {
      nbyhk2 = npixl;
      nbymap2 = nfblin - npixl;
      }
   else
      nbyhk2 = nbymap2 = nbylin2;
   cexp2 = pscale;

   if (nbypix2 != 1 && nbypix2 != 2 && nbypix2 != 4)
      {
      fprintf (stderr, "Invalid image (%s): nbypix1=%d\n",
               namimg2, nbypix2);
      exit (-2);
      }

   if (npixl1 != npixl2)
      {
      fprintf (stderr, "size of %s (%d) != size of %s (%d)\n",
	       namimg1, npixl1, namimg2, npixl2);
      exit (-3);
      }
   printf ("npixl1: %d   nbypix1: %d   nbylin1: %d\n", npixl1, nbypix1, nbylin1);
   printf ("npixl2: %d   nbypix2: %d   nbylin2: %d\n", npixl2, nbypix2, nbylin2);
   printf ("nbyhk1: %d   nbymap1: %d   nbyhk2: %d   nbymap2: %d\n",
	    nbyhk1, nbymap1, nbyhk2, nbymap2);

   sprintf (namout, "%s%02d%02d.img", namtmp, H.hour, H.minute);
   fprintf (stdout, "Difference image file name : %s\n", namout);

   /*** Open input & output image files. ***/

    fdin1 = open (namimg1, 0);
    if (fdin1 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg1);
       exit (-4);
       }
    fdin2 = open (namimg2, 0);
    if (fdin2 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg2);
       exit (-5);
       }
    fdout = creat (namout, 0644);
    if (fdout < 0)
       {
       fprintf (stderr, "Unable to create %s\n", namout);
       exit (-6);
       }

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin1, hk1, nbyhk1);
   read (fdin2, hk2, nbyhk2);
   hk1 [52] = 0;			/* Sun byte order. */
   hk1 [103] = (char) 32;		/* # bits per pixel. */
   hk1 [110] = (npixl & 0177400) / 256;	/* # pixels per line (m.s.byte). */
   hk1 [111] = (npixl &    0377);	/* # pixels per line (l.s.byte). */
   hk1 [220] = (char) 0;		/* Scaling factor = 1.0 */
   hk1 [221] = (char) 16;		/* Delta B (difference image). */
   hk1 [222] = (char) 0;;		/* Mask off data types 9-16. */
   hk1 [223] |= (char) (1 + 8);		/* S & F removed by subtraction. */

   /*** Copy date & time of image #2 into hskp bytes #143-148 for dif image. */

   hk1 [143] = hk2 [0];			/* Store year2 in byte #143 of hk1. */
   hk1 [144] = hk2 [1];			/* Store doy2a in byte #144 of hk1. */
   hk1 [145] = hk2 [2];			/* Store doy2b in byte #145 of hk1. */
   hk1 [146] = hk2 [20];		/* Store hour2 in byte #146 of hk1. */
   hk1 [147] = hk2 [21];		/* Store min2  in byte #147 of hk1. */
   hk1 [148] = hk2 [24];		/* Store sec2  in byte #148 of hk1. */

   write (fdout, hk1, nbyhk1);

   for (i=0; i<maxmap; i++)  bitmap [i] = 0;
   read  (fdin1, bitmap, nbymap1);
   write (fdout, bitmap, nfblin-nbyhk1);

   read  (fdin2, bitmap, nbymap2);

   /*** Subtract image #2 from image #1. ***/
   /*** Store result as a floating point image. ***/

   for (line = 0; line < npixl1; line++)
      {
      read (fdin1, buf1, nbylin1);
      read (fdin2, buf2, nbylin2);
      if (nbypix1 == 2 && bytord1 == 1) srevbyt (sbuf1, nbypix1, npixl1);
      if (nbypix2 == 2 && bytord2 == 1) srevbyt (sbuf2, nbypix2, npixl2);

      /*** Both images are characters. ***/

      if (nbypix1 == 1 && nbypix2 == 1)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(cbuf1 + i)) * cexp1)
	              - (((float) *(cbuf2 + i)) * cexp2);

      /*** Image #1 is character, image #2 is short integer. ***/

      else
      if (nbypix1 == 1 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(cbuf1 + i)) * cexp1)
	              - (((float) *(sbuf2 + i)) * cexp2);

      /*** Image #1 is character, image #2 is floating point. ***/

      else
      if (nbypix1 == 1 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(cbuf1 + i)) * cexp1)
	              - (((float) *(fbuf2 + i)) * cexp2);

      /*** Image #1 is short integer, image #2 is character. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 1)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(sbuf1 + i)) * cexp1)
	              - (((float) *(cbuf2 + i)) * cexp2);

      /*** Both images are short integers. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(sbuf1 + i)) * cexp1)
	              - (((float) *(sbuf2 + i)) * cexp2);

      /*** Image #1 is short integer, image #2 is floating point. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(sbuf1 + i)) * cexp1)
	              - (((float) *(fbuf2 + i)) * cexp2);

      /*** Image #1 is floating point, image #2 is character. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 1)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(fbuf1 + i)) * cexp1)
	              - (((float) *(cbuf2 + i)) * cexp2);

      /*** Image #1 is floating point, image #2 is short integer. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(fbuf1 + i)) * cexp1)
	              - (((float) *(sbuf2 + i)) * cexp2);

      /*** Image #1 is floating point, image #2 is floating point. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
         *(fbuf3 + i) = (((float) *(fbuf1 + i)) * cexp1)
	              - (((float) *(fbuf2 + i)) * cexp2);

/*      fprintf (stdout, "fbuf3 [%3d]: %d\n", line, *(fbuf3 + line)); */
      write (fdout, fbuf3, nfblin);
      }

   close (fdin1);
   close (fdin2);
   close (fdout);
}
