/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fmassimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Subtract Stray Light and/or Vignetting and/or F-corona from a C/P image.
 * Output image is floating point.
 * Output image name is the same as the input image with suffix: ".jsvfm".
 * Input image may be short integer or floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR   12 May 1999
 * 19 May 1999: cc applied ONLY to coronal signal, NOT to cpstray () value.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.h"
#define maxbyt 3584
#define maxhk   896
#define maxmap 2688

main (argc, argv)

int argc ;
char *argv [] ;

{
   extern float jupcor (), cpstray (), cpvignet (), fcorona () ;
   extern double medsun () ;

   static char answer [4] ;
   static char namimg [80] ;
   static char namcor [80] ;
   static char hk [maxhk] ;
   static char bitmap [maxmap] ;

   unsigned char *buf1, *buf3 ;
   unsigned short *sbuf1 ;
   float *fbuf1, *fbuf3 ;

   int bytord ;
   int col ;
   int idot ;
   int fdin ;
   int fdout ;
   int i ;
   int nbylin ;			/* # bytes per line in input image. */
   int nbyimg, nbyout ;
   int nbyhk ;
   int nbymap ;
   int nfblin ;
   int row ;

   float cc ;
   float cexp ;
   float fc ;
   float r, th ;
   float str ;
   float subval ;
   float vig ;
   float x, y ;

   float  cbrite ;
   float  cmass ;
   float  gcm2 ;
   float  cm2pix ;
   float  emass  = 2.0e-24 ;	/* mass (g) per light-scattering electron. */
   float  RSUNCM = 6.96e+10 ;	/* Average solar radius (cm). */

   double pangle, bangle, rsun ;

   fprintf (stdout,
      "%s: remove stray light/vignetting/Fcorona from an SMM C/P image & compute mass.\n\n",
       argv [0]) ;

   ljc  = 0 ;
   lstr = 0 ;
   lvig = 0 ;
   lfc  = 0 ;

   if (argc == 2)
      {
      strcpy (namimg, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s cp_image_filename\n", argv [0]) ;
      fprintf (stderr, "Enter image file name : ") ;
      fscanf  (stdin,  "%s", namimg) ;
      }

   fprintf (stderr, "Apply Jupiter calibration correction factor [y/n]? ") ;
   fscanf  (stdin,  "%s", answer) ;
   if (strncmp (answer, "y", 1) == 0)  ljc = 1 ;

   fprintf (stderr, "Remove stray light [y/n] ? ") ;
   fscanf  (stdin,  "%s", answer) ;
   if (strncmp (answer, "y", 1) == 0)  lstr = 1 ;

   fprintf (stderr, "Remove vignetting [y/n] ? ") ;
   fscanf  (stdin,  "%s", answer) ;
   if (strncmp (answer, "y", 1) == 0)  lvig = 1 ;

   fprintf (stderr, "Remove F-corona [y/n] ? ") ;
   fscanf  (stdin,  "%s", answer) ;
   if (strncmp (answer, "y", 1) == 0)  lfc = 1 ;


   /* Derive output file name from input image name. */

   idot = -1 ;
   for (i = 0 ; i < strlen (namimg) ; i++) if (namimg [i] == '.')  idot = i ;
   if (idot < 0)  idot = strlen (namimg) - 1 ;
   idot = strlen (namimg) ;
   strcpy (namcor, namimg) ;
   namcor [idot] = '.' ;
   if (ljc  == 1)          namcor [idot+1] = 'c' ;  else namcor [idot+1] = 'x' ;
   if (lstr == 1 || H.str) namcor [idot+2] = 's' ;  else namcor [idot+2] = 'x' ;
   if (lvig == 1 || H.vig) namcor [idot+3] = 'v' ;  else namcor [idot+3] = 'x' ;
   if (lfc  == 1 || H.f)   namcor [idot+4] = 'f' ;  else namcor [idot+4] = 'x' ;
   namcor [idot+5] = 'm' ;
   namcor [idot+6] = '\0' ;
   printf ("corrected image filename . . . namcor: %s\n", namcor) ;
   printf ("H.str: %d   H.vig: %d   H.reg: %d   H.f: %d\n",
           H.str, H.vig, H.reg, H.f) ;

   /*** Decode housekeeping records. ***/

   cphskp (namimg) ;
   nbylin = npix * nbypix ;
   nfblin = npixl * sizeof (float) ;
   if (nbypix == 4)
      {
      nbyhk = npixl ;
      nbymap = nfblin - npixl ;
      }
   else
      nbyhk = nbymap = nbylin ;
   cexp = pscale ;

   nbyimg = npix * npix * nbypix ;
   nbyout = npix * npix * sizeof (float) ;
   buf1 = (unsigned char *) malloc (nbyimg) ;
   buf3 = (unsigned char *) malloc (nbyout) ;

   sbuf1 = (unsigned short *) buf1 ;
   fbuf1 = (float *) buf1 ;
   fbuf3 = (float *) buf3 ;


   printf ("npix: %d   nbypix: %d   nbylin: %d\n", npix, nbypix, nbylin) ;
   printf ("nbyhk: %d   nbymap: %d\n",
	    nbyhk, nbymap) ;

   /*** Open input & output image files. ***/

    fdin = open (namimg, 0) ;
    if (fdin < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg) ;
       exit (-1) ;
       }
    fdout = creat (namcor, 0644) ;
    if (fdout < 0)
       {
       fprintf (stderr, "Unable to create %s\n", namcor) ;
       exit (-3) ;
       }

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin, hk, nbyhk) ;
   hk [52] = 0 ;			/* Sun byte order. */
   hk [103] = (char) 32 ;		/* # bits per pixel. */
   hk [110] = (npixl & 0177400) / 256 ;	/* # pixels per line (m.s.byte). */
   hk [111] = (npixl &    0377) ;	/* # pixels per line (l.s.byte). */
   hk [220] = (char) 0 ;		/* Scaling factor = 1.0 */
   hk [221] = (char) 0 ;		/* Mask off data types 1-8. */
   hk [222] = (char) 128 ;		/* Set Mass data type. */
   if (lstr) hk [223] |= 1 ;		/* Set Stray Light removal flag. */
   if (lvig) hk [223] |= 2 ;		/* Set Vignettting removal flag. */
   if (lfc ) hk [223] |= 8 ;  		/* Set F-corona removal flag. */

   write (fdout, hk, nbyhk) ;

   for (i = 0 ; i < maxmap ; i++) bitmap [i] = 0 ;
   read  (fdin,  bitmap, nbymap) ;
   write (fdout, bitmap, nfblin-nbyhk) ;

   pb0r (H.year, H.month, H.day, H.hour, H.minute, H.second,
	 &pangle, &bangle, &rsun) ;

   /*** Compute the cross-sectional area (square cm) for one pixel. ***/

   cm2pix = (RSUNCM*rspix) * (RSUNCM*rspix) ;

   /*** Read input image into memory array. ***/

   read (fdin, buf1, nbyimg) ;
   if (nbypix == 2 && bytord == 1) srevbyt (sbuf1, nbypix, nbyimg) ;

   /*** Subtract stray light from image. ***/

   cc  = pscale ;
   str = 0.0 ;
   vig = 1.0 ;
   fc  = 0.0 ;
   for (row = 0 ; row < npix ; row++)
      {

      /*** Image is short integer. ***/

      if (nbypix == 2)
         for (col=0 ; col<npix ; col++)
            {
            i = col + row * npix ;
            x = (float) col ;
            y = (float) row ;
            rcoord (&r, &th, &x, &y, -1, scroll) ;
	    gcm2 = medsun (r) ;
            if (ljc  == 1)  cc = jupcor () * pscale ;
            if (lstr == 1 && H.str == 0)  str = cpstray  (r * rsun) ;
            if (lvig == 1 && H.vig == 0)  vig = cpvignet (r * rsun) ;
            if (lfc  == 1 && H.f  == 0)   fc  = fcorona  (r, th) ;
/*
 *            if (col == row)
 *               fprintf (stdout,
 *                  "r: %6.2f   str: %g   vig: %g   cc: %g   fc: %g\n",
 *                   r, str, vig, cc, fc) ;
 */
            cbrite = (((float) *(sbuf1 + i) * cc - str) * vig) - fc ;
	    cmass = cbrite * gcm2 * cm2pix ;
	    if (r < 1.5)  cmass = 0.0 ;		/* Set mass to 0 in occ.disk. */

            if (*(sbuf1 + i) != 0)
               *(fbuf3 + i) = cmass ;
            else
               *(fbuf3 + i) = 0.0 ;
            }

      /*** Image is floating point. ***/

      else
      if (nbypix == 4)
         {
         for (col = 0 ; col < npix ; col++)
            {
            i = col + row * npix ;
            x = (float) col ;
            y = (float) row ;
            rcoord (&r, &th, &x, &y, -1, scroll) ;
	    gcm2 = medsun (r) ;
            if (ljc  == 1)  cc = jupcor () * pscale ;
            if (lstr == 1 && H.str == 0)  str = cpstray  (r * rsun) ;
            if (lvig == 1 && H.vig == 0)  vig = cpvignet (r * rsun) ;
            if (lfc  == 1 && H.f   == 0)  fc  = fcorona  (r, th) ;
/*
 *            if (col == row)
 *               fprintf (stdout,
 *                  "r: %6.2f   str: %g   vig: %g   cc: %g   fc: %g\n",
 *                   r, str, vig, cc, fc) ;
 */

            cbrite = (((float) *(fbuf1 + i) * cc - str) * vig) - fc ;
	    cmass  = cbrite * gcm2 * cm2pix ;

            if (*(fbuf1 + i) != 0.0)
               *(fbuf3 + i) = cmass ;
            else
               *(fbuf3 + i) = 0.0 ;
            }
/*
         fprintf (stdout, "row: %3d   fbuf1: %g   fbuf3: %g\n",
                  row, *(fbuf1 + row + row*npix), *(fbuf3 + row + row*npix)) ;
 */

         }
      }
   write (fdout, fbuf3, nbyout) ;

   x = 0.0 ;
   y = 0.0 ;
   printf ("x,y: %7.2f  %7.2f  scroll: %d\n", x, y, scroll) ;
   rcoord (&r, &th, &x, &y, -1, scroll) ;
   printf ("[0,0] -> [%7.2f,%7.2f]\n", r, th) ;
   printf ("cm2pix: %10.3e\n", cm2pix) ;
   printf ("hk [222]: %d\n", hk [222]) ;

   close (fdin) ;
   close (fdout) ;
   exit (0) ;
}
