/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * frmstr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Subtract Stray Light from a C/P image
 * Output difference image is floating point.
 * Output image is designated the same as the input image.
 * Input image may be short integer or floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 27 September 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxbyt 3584
#define maxhk   896
#define maxmap 2688

main (argc, argv)

int argc;
char *argv [];

{
   float stray (), vignet ();

   static char namimg [80];
   static char namout [80];
   static char hk [maxhk];
   static char bitmap [maxmap];

   unsigned char *buf1, *buf3;
   unsigned short *sbuf1;
   float *fbuf1, *fbuf3;

   int bytord;
   int col;
   int idot;
   int fdin;
   int fdout;
   int i;
   int nbylin;			/* # bytes per line in input image. */
   int nbyimg, nbyout;
   int nbyhk;
   int nbymap;
   int nfblin;
   int row;

   float cexp;
   float r, th;
   float str;
   float subval;
   float vig;
   float x, y;

   fprintf (stdout,
      "frmstr: remove stray light from an SMM C/P image.\n\n");

   if (argc == 2)
      strcpy (namimg, argv [1]);
   else
      {
      fprintf (stderr, "usage: %s image_filename\n", argv [0]);
      fprintf (stderr, "Enter name of input image file:               ");
      fscanf  (stdin,  "%s", namimg);
      }

   strcpy (namout, namimg);
   for (i=0; i<strlen(namimg); i++)
      {
      idot = i;
      if (namimg [i] == '.')  break;
      }
   namout [idot+1] = '\0';
   if (H.vig == 0)
      strcat (namout, "bfv");
   else
   if (H.vig == 1)
      strcat (namout, "bf");
   fprintf (stdout, "Bf image file name : %s\n", namout);

   /*** Decode housekeeping records. ***/

   cphskp (namimg);
   nbylin = npix * nbypix;
   nfblin = npixl * sizeof (float);
   if (nbypix == 4)
      {
      nbyhk = npixl;
      nbymap = nfblin - npixl;
      }
   else
      nbyhk = nbymap = nbylin;
   cexp = pscale;

   nbyimg = npix * npix * nbypix;
   nbyout = npix * npix * sizeof (float);
   buf1 = (unsigned char *) malloc (nbyimg);
   buf3 = (unsigned char *) malloc (nbyout);

   sbuf1 = (unsigned short *) buf1;
   fbuf1 = (float *) buf1;
   fbuf3 = (float *) buf3;


   printf ("npix: %d   nbypix: %d   nbylin: %d\n", npix, nbypix, nbylin);
   printf ("nbyhk: %d   nbymap: %d\n",
	    nbyhk, nbymap);

   /*** Open input & output image files. ***/

    fdin = open (namimg, 0);
    if (fdin < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg);
       exit (-1);
       }
    fdout = creat (namout, 0644);
    if (fdout < 0)
       {
       fprintf (stderr, "Unable to create %s\n", namout);
       exit (-3);
       }

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin, hk, nbyhk);
   hk [52] = 0;			/* Sun byte order. */
   hk [103] = (char) 32;		/* # bits per pixel. */
   hk [110] = (npixl & 0177400) / 256;	/* # pixels per line (m.s.byte). */
   hk [111] = (npixl &    0377);	/* # pixels per line (l.s.byte). */
   hk [220] = (char) 0;		/* Scaling factor = 1.0 */
   hk [221] = (char) 32;		/* Bk image. */
   hk [222] = (char) 0;;		/* Mask off data types 9-16. */
   hk [223] |= 1;			/* Set Stray Light removal flag. */
/*   hk [223] |= 8; */	 		/* Set F-corona removal flag. */

   write (fdout, hk, nbyhk);

   for (i=0; i<maxmap; i++) bitmap [i] = 0;
   read  (fdin, bitmap, nbymap);
   write (fdout, bitmap, nfblin-nbyhk);

   /*** Read input image into memory array. ***/

   read (fdin, buf1, nbyimg);
   if (nbypix == 2 && bytord == 1) srevbyt (sbuf1, nbypix, nbyimg);

   /*** Subtract stray light from image. ***/

   vig = 1.0;
   for (row = 0; row < npix; row++)
      {

      /*** Image is short integer. ***/

      if (nbypix == 2)
         for (col=0; col<npix; col++)
            {
            i = col + row * npix;
            x = (float) col;
            y = (float) row;
            rcoord (&r, &th, &x, &y, -1, scroll);
            str = stray (r);
            if (H.vig == 0)
               vig = vignet (r);

            if (*(sbuf1 + i) != 0)
               *(fbuf3 + i) = (float) *(sbuf1 + i) - str / vig;
            else
               *(fbuf3 + i) = 0;
            }

      /*** Image is floating point. ***/

      else
      if (nbypix == 4)
         {
         for (col=0; col<npix; col++)
            {
            i = col + row * npix;
            x = (float) col;
            y = (float) row;
            rcoord (&r, &th, &x, &y, -1, scroll);
            str = stray (r);
            if (H.vig == 0)
               vig = vignet (r);
            if (col == row)
               fprintf (stdout, "r: %6.2f   str: %g   vig: %g\n", r, str, vig);

            if (*(fbuf1 + i) != 0.0)
               *(fbuf3 + i) = (float) *(fbuf1 + i) - str / vig;
            else
               *(fbuf3 + i) = 0.0;
            }
/*
         fprintf (stdout, "row: %3d   fbuf1: %g   fbuf3: %g\n",
                  row, *(fbuf1 + row + row*npix), *(fbuf3 + row + row*npix));
 */

         }
      }
   write (fdout, fbuf3, nbyout);

   close (fdin);
   close (fdout);
   exit (0);
}
