/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fsmaskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create a Bfcorona + Bstray image mask.
 * Bk/V = [pB/V] / pol.
 * [Bf/V + Bs] = Btotal - Bk/V.
 * Output difference image is floating point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 19 April 1990
 * 25 April 1990: Store date & time of image #2 in hskp of output image.
 *  2 May 1990: Either image may be short integer or floating point.
 *  7 November 1990: modify sfsubimg.c to create fsmask.c program.
 * 11 February 1991: request polarization percentage from user.
 *  5 August 1992: store ref image date & time in bytes 143-148.
 * 30 March 1994: polarization fraction is read from .pol image file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define maxbyt 3584
#define maxhk   896
#define maxmap 2688

main (argc, argv)

int argc;
char *argv [];
{
static char bitmap [maxmap];
static char cpol [4];
static char hk1 [maxhk], hk2 [maxhk], hk3 [maxhk];
static char namimg1 [80];
static char namimg2 [80];
static char namimg3 [80];
static char namout [80];
static char namgmt [80] = "yydddhhmm";

static unsigned char buf1 [maxbyt], buf2 [maxbyt], buf3 [maxbyt], buf4 [maxbyt];
static unsigned short *sbuf1, *sbuf2;

int bytord1, bytord2, bytord3;
int i;
int idot;
int fdin1, fdin2, fdin3;
int line;
int fdout;
int nbylin1, nbylin2, nbylin3;		/* # bytes per line in input images. */
int nfblin;				/* # bytes per line in output image. */
int nbypix1, nbypix2, nbypix3;
int npixl1, npixl2, npixl3;
int nbyhk1, nbyhk2, nbyhk3;
int nbymap1, nbymap2, nbymap3;
int pol;

static float *fbuf1, *fbuf2, *fbuf3, *fbuf4;
float cexp1, cexp2, cexp3;
float polpct = 60.0;
float polfac;
float subval;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   sbuf1 = (unsigned short *) buf1;
   sbuf2 = (unsigned short *) buf2;
   fbuf1 = (float *) buf1;
   fbuf2 = (float *) buf2;
   fbuf3 = (float *) buf3;
   fbuf4 = (float *) buf4;

   fprintf (stdout, "fsmask: create Bfcorona + Bstray image mask.\n\n");

   if (argc == 4)
      {
      strcpy (namimg1, argv [1]);
      strcpy (namimg2, argv [2]);
      strcpy (namimg3, argv [3]);
      }
   else
      {
      fprintf (stderr, "usage: %s   B_total   pB   polarization_image\n",
	       argv [0]);
      exit (0) ;
      }

   pol   = (int) (polpct + 0.5);
   polfac = 100.0 / polpct;

   strcpy (namgmt, namimg1);
   for (i=0; i<strlen(namimg1); i++)
      {
      idot = i;
      if (namimg1[i] == '.')  break;
      }
   namgmt [idot+1] = '\0';
   sprintf (namout, "%sfspol", namgmt);
   printf ("namimg1: %s   namimg2: %s  namimg3: %s\n",
	   namimg1, namimg2, namimg3);
   fprintf (stdout, "F-corona + Stray Light mask image file name : %s\n",
	    namout);

   /*** Decode housekeeping records. ***/

   cphskp (namimg1);
   bytord1 = bytord;
   nbypix1 = nbypix;
   npixl1 = npixl;
   nbylin1 = npixl1 * nbypix1;
   nfblin = npixl1 * sizeof (float);
   if (nbypix == 4)
      {
      nbyhk1 = npixl;
      nbymap1 = nfblin - npixl;
      }
   else
      nbyhk1 = nbymap1 = nbylin1;
   cexp1 = pscale;

   cphskp (namimg2);
   bytord2 = bytord;
   nbypix2 = nbypix;
   npixl2 = npixl;
   nbylin2 = npixl2 * nbypix2;
   if (nbypix == 4)
      {
      nbyhk2 = npixl;
      nbymap2 = nfblin - npixl;
      }
   else
      nbyhk2 = nbymap2 = nbylin2;
   cexp2 = pscale;

   if (npixl1 != npixl2)
      {
      fprintf (stderr, "size of %s (%d) != size of %s (%d)\n",
	       namimg1, npixl1, namimg2, npixl2);
      exit (-1);
      }

   cphskp (namimg3);
   bytord3 = bytord;
   nbypix3 = nbypix;
   npixl3 = npixl;
   nbylin3 = npixl3 * nbypix3;
   if (nbypix == 4)
      {
      nbyhk3 = npixl;
      nbymap3 = nfblin - npixl;
      }
   else
      nbyhk3 = nbymap3 = nbylin3;
   cexp3 = pscale;

   if (npixl1 != npixl3)
      {
      fprintf (stderr, "size of %s (%d) != size of %s (%d)\n",
	       namimg1, npixl1, namimg3, npixl3);
      exit (-1);
      }

   fprintf (stdout, "npixl1: %d   nbypix1: %d   nbylin1: %d\n",
	    npixl1, nbypix1, nbylin1);
   fprintf (stdout, "npixl2: %d   nbypix2: %d   nbylin2: %d\n",
	    npixl2, nbypix2, nbylin2);
   fprintf (stdout, "nbyhk1: %d   nbymap1: %d   nbyhk2: %d   nbymap2: %d\n",
	    nbyhk1, nbymap1, nbyhk2, nbymap2);
   fprintf (stdout, "cexp1: %8.2e   cexp2: %8.2e   cexp3: %8.2e\n",
	    cexp1, cexp2, cexp3) ;

   /*** Open input & output image files. ***/

    fdin1 = open (namimg1, 0);
    if (fdin1 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg1);
       exit (-1);
       }
    fdin2 = open (namimg2, 0);
    if (fdin2 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg2);
       exit (-2);
       }
    fdin3 = open (namimg3, 0);
    if (fdin3 < 0)
       {
       fprintf (stderr, "Unable to open %s\n", namimg3);
       exit (-3);
       }
    fdout = creat (namout, 0644);
    if (fdout < 0)
       {
       fprintf (stderr, "Unable to create %s\n", namout);
       exit (-3);
       }

   /*** Copy housekeeping & bitmap records. ***/

   for (i=0; i<maxmap; i++)  bitmap [i] = 0;
   read (fdin1, hk1, nbyhk1);
   read (fdin1, bitmap, nbymap1);
   read (fdin2, hk2, nbyhk2);
   read (fdin3, hk3, nbyhk3);
   hk1 [52] = 0;			/* Sun byte order. */
   hk1 [103] = (char) 32;		/* # bits per pixel. */
   hk1 [110] = (npixl & 0177400) / 256;	/* # pixels per line (m.s.byte). */
   hk1 [111] = (npixl &    0377);	/* # pixels per line (l.s.byte). */
   hk1 [220] = (char) 0;		/* Scaling factor = 1.0 */
   hk1 [221] = (char) 0;		/* Set to zero. */
   hk1 [222] = (char) 32;		/* Mask off data types 9-16. */
   hk1 [223] = (char) 16;		/* 16 => K corona removed. */

   /*** Copy date & time of image #2 into hskp bytes #143-148 for dif image. */

   hk1 [143] = hk2 [0];			/* Store year2 in byte #143 of hk1. */
   hk1 [144] = hk2 [1];			/* Store doy2a in byte #144 of hk1. */
   hk1 [145] = hk2 [2];			/* Store doy2b in byte #145 of hk1. */
   hk1 [146] = hk2 [20];		/* Store hour2 in byte #146 of hk1. */
   hk1 [147] = hk2 [21];		/* Store min2  in byte #147 of hk1. */
   hk1 [148] = hk2 [24];		/* Store sec2  in byte #148 of hk1. */

   write (fdout, hk1, nbyhk1);
   write (fdout, bitmap, nfblin - nbyhk1);

   read  (fdin2, bitmap, nbymap2);
   read  (fdin3, bitmap, nbymap3);

   /*** Subtract image #2 from image #1. ***/
   /*** Store result as a short integer image. ***/

   for (line = 0; line < npixl1; line++)
      {
      read (fdin1, buf1, nbylin1);
      read (fdin2, buf2, nbylin2);
      read (fdin3, buf3, nbylin3);
      if (nbypix1 == 2 && bytord1 == 1) srevbyt (sbuf1, nbypix1, npixl1);
      if (nbypix2 == 2 && bytord2 == 1) srevbyt (sbuf2, nbypix2, npixl2);

      /*** Both images are short integers. ***/

      if (nbypix1 == 2 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
         *(fbuf4 + i) = (((float) *(sbuf1 + i)) * cexp1)
	                - (((float) *(sbuf2 + i)) * cexp2 / *(fbuf3 + i));

      /*** Image #1 is short integer, image #2 is floating point. ***/

      else
      if (nbypix1 == 2 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
         *(fbuf4 + i) = (((float) *(sbuf1 + i)) * cexp1)
                                -(*(fbuf2 + i) / *(fbuf3 + i));

      /*** Image #1 is floating point, image #2 is short integer. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 2)
      for (i=0; i<npixl1; i++)
         *(fbuf4 + i) = *(fbuf1 + i)
            - (((float) *(fbuf2 + i)) * cexp2 / *(fbuf3 + i));

      /*** Both Images are floating point. ***/

      else
      if (nbypix1 == 4 && nbypix2 == 4)
      for (i=0; i<npixl1; i++)
         *(fbuf4 + i) = *(fbuf1 + i) - (*(fbuf2 + i) / *(fbuf3 + i));

/*      fprintf (stdout, "fbuf4 [%3d]: %d\n", line, *(fbuf4 + line)); */
/*
 *      for (i=0; i<npixl1; i++)
 *         if (*(fbuf4 + i) < 0.0)  *(fbuf4 + i) = 0.0;
 */
      write (fdout, fbuf4, nfblin);
      }

   close (fdin1);
   close (fdin2);
   close (fdin3);
   close (fdout);
}
