/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sdeltab.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute (B - Bzero) / B.
 * B =     brightness image (event).
 * Bzero = brightness image (background).
 * Input images must be calibrated, C/P format.
 * Output image pixels are short integers.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 January 1989
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#define maxbyt 1792
#define maxpix 896

main ()
{
static char namcme [80];
static char nambkg [80];
static char namout [80];
static char hk [maxbyt];
static char bitmap [maxbyt];

static short ibufe [maxpix], ibufb [maxpix];
static short linbuf [maxpix];

int bytordb, bytorde;
int fdout;
int fdcme, fdbkg;
int nbylin;
int nbypixb, nbypixe;

register short *lincme, *linbkg;
register int i, line;
register float difval;

   lincme = ibufe;
   linbkg = ibufb;

   fprintf (stdout, "deltab: [B(cme) - B(bkg)] / B(cme).\n\n");

   fprintf (stdout, "Enter name of CME image file: ");
   fscanf  (stdin,  "%s", namcme);
   fprintf (stdout, "Enter name of BKG image file: ");
   fscanf  (stdin,  "%s", nambkg);
   strcpy (namout, namcme);
   strcat (namout, ".b");
   if (strlen(nambkg) >= 8)  strncat (namout, &nambkg[4], 4);
      else strcat (namout, "rt");

   /*** Open input & output image files. ***/

   cphskp (nambkg);
   nbypixb = nbypix;
   bytordb = bytord;

   cphskp (namcme);
   nbypixe = nbypix;
   bytorde = bytord;
   nbylin = npixl * nbypix;
   printf ("npixl: %d   nbypix: %d   nbylin: %d\n", npixl, nbypix, nbylin);
   fdcme = open (namcme, 0);
   fdbkg = open (nambkg, 0);
   fdout = creat (namout, 0644);

   /*** Copy housekeeping & bitmap records. ***/

   read (fdcme, hk, nbylin);
   read (fdcme, bitmap, nbylin);
   hk [52] = 0;				/* Sun byte order. */
   hk [103] = 16;			/* # bits per pixel. */
   hk [220] = (2 & 0377);		/* Scaling factor (10**3). */
   write (fdout, hk, nbylin);
   write (fdout, bitmap, nbylin);

   read (fdbkg, hk, nbylin);
   read (fdbkg, bitmap, nbylin);

   /*** Subtract background frame from event frame. ***/
   /*** Divide result by event frame and store as a short integer image. ***/

   for (line=0; line<npixl; line++)
      {
      read (fdcme, lincme, nbylin);
      read (fdbkg, linbkg, nbylin);
      if (nbypixe == 2 && bytorde == 1)
         srevbyt (lincme, nbypix, npixl);
      if (nbypixb == 2 && bytordb == 1)
         srevbyt (linbkg, nbypix, npixl);

      for (i=0; i<448; i++)
	 {
	 difval = (float) (*(lincme + i) - *(linbkg + i));
         if (difval != 0.0)
	    *(linbuf + i) =
	       (short) ((difval / (float) *(linbkg+ i)) * 100. + 125.);
         else
	    *(linbuf + i) = 125;
         if (*(linbuf + i) > 250.0)  *(linbuf + i) = 250.0;
         if (*(linbuf + i) <   0.0)  *(linbuf + i) =   0.0;
         }

      fprintf (stdout, "line %3d:   cme: %d   bkg: %d   delta: %d\n",
	       line, *(lincme + line), *(linbkg + line), *(linbuf + line));
      write (fdout, linbuf, nbylin);
      }

   close (fdcme);
   close (fdbkg);
   close (fdout);
}
