/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * sdivimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Divide a calibrated, low resolution C/P image by a second C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 24 Jan 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/usr/people/smm/lib/src/cp/cplib.h"
#define maxpix 896
#define maxbyt 1792

main ()
{
char *image;
static char namimg1 [80];
static char namimg2 [80];
static char namout [80];
static char hk [maxbyt];
static char bitmap [maxbyt];
static unsigned short ibuf1 [maxpix], ibuf2 [maxpix];
static unsigned short linbuf [maxpix];

int bytord1, bytord2;
int fdin1, fdin2;
int fdout;
int i, line;
int nbypix1, nbypix2;
int nbylin;

   fprintf (stdout, "sdivimg: divide one short C/P image by another.\n\n");

   fprintf (stdout, "Enter name of numerator   image file: ");
   fscanf  (stdin,  "%s", namimg1);
   fprintf (stdout, "Enter name of denominator image file: ");
   fscanf  (stdin,  "%s", namimg2);
   strcpy (namout, namimg1);
   strcat (namout, ".d");
   if (strlen (namimg2) >= 8)  strncat (namout, &namimg2 [4], 4);
      else strcat (namout, "db");

   /*** Open input & output image files. ***/

   cphskp (namimg2);
   bytord2 = bytord;
   nbypix2 = nbypix;

   cphskp (namimg1);
   bytord1 = bytord;
   nbypix1 = nbypix;
   nbylin = npixl * nbypix;
   fdin1 = open (namimg1, 0);
   fdin2 = open (namimg2, 0);
   fdout = creat (namout, 0644);

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin1, hk, nbylin);
   read (fdin1, bitmap, nbylin);
   write (fdout, hk, nbylin);
   write (fdout, bitmap, nbylin);
   hk [52] = 0;				/* Sun byte order. */
   hk [220] = (char) (-2 & 0377);

   read (fdin2, hk, nbylin);
   read (fdin2, bitmap, nbylin);

   /*** Divide image #1 by image #2. ***/
   /*** Multiply result by 100 and store as a short integer. ***/

   for (line=0; line<npixl; line++)
      {
      read (fdin1, ibuf1, nbylin);
      read (fdin2, ibuf2, nbylin);
      if (nbypix1 == 2 && bytord1 == 1)
         srevbyt (ibuf1, nbypix, npixl);
      if (nbypix2 == 2 && bytord2 == 2)
         srevbyt (ibuf2, nbypix, npixl);

      for (i=0; i<npixl; i++)
	 {
         if (*(ibuf1 + i) > 0)
	    *(linbuf + i) =
	       (unsigned short) (((float) *(ibuf1 + i) / (float) *(ibuf2 + i)) * 100.0);
         else
	    *(linbuf + i) = 0;
         }

      fprintf (stdout, "linbuf [%3d]: %d\n", line, *(linbuf + line));
      write (fdout, linbuf, nbylin);
      }

   close (fdin1);
   close (fdin2);
   close (fdout);
}
