/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * subimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Subtract a calibrated, low resolution C/P image by a second C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 January 1989
 * 25 April 1990: Store date & time of image #2 in hskp of output image.
 *  5 August 1992: store ref image date & time in bytes 143-148.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/src/cp/cplib.h"
#define maxbyt 1792
#define maxpix 896

main()
{
static char namimg1 [80];
static char namimg2 [80];
static char namout [80];
static char hk1 [maxbyt], hk2 [maxbyt];
static char bitmap [maxbyt];

static unsigned short ibuf1 [maxpix], ibuf2 [maxpix];
static unsigned short linbuf [maxpix];

int bytord1, bytord2;
int idot;
int fdin1, fdin2;
int fdout;
int nbylin;			/* # bytes per line in input images. */
int nsblin;			/* # bytes per line in output image. */
int nbypix1, nbypix2;
int npixl1, npixl2;

register int i, line;
register int subval;

   fprintf (stdout, "subimg: subtract one C/P image from another [img1 - img2].\n\n");

   fprintf (stdout, "Enter name of first  image file: ");
   fscanf  (stdin,  "%s", namimg1);
   fprintf (stdout, "Enter name of second image file: ");
   fscanf  (stdin,  "%s", namimg2);
   strcpy (namout, namimg1);
   for (i=0; i<strlen(namimg1); i++)
      {
      idot = i;
      if (namimg1[i] == '.')  break;
      }
   namout[idot+1] = '\0';
   strcat (namout, "s");
   if (strlen (namimg2) >= 9)  strncat (namout, &namimg2[5], 4);
      else strcat (namout, "ub");
   fprintf (stderr, "Difference image file name : %s\n", namout);

   /*** Decode housekeeping records. ***/

   cphskp (namimg1);
   bytord1 = bytord;
   nbypix1 = nbypix;
   npixl1 = npixl;

   cphskp (namimg2);
   bytord2 = bytord;
   nbypix2 = nbypix;
   npixl2 = npixl;

   if (npixl1 != npixl2)
      {
      fprintf (stderr, "size of %s (%d) != size of %s (%d)\n",
	       npixl1, namimg1, npixl2, namimg2);
      exit (-1);
      }
   nbylin = npixl1 * nbypix1;
   nsblin = npixl1 * sizeof (short);
   printf ("npixl1: %d   nbypix1: %d   nbylin: %d\n", npixl1, nbypix1, nbylin);

   /*** Open input & output image files. ***/

   fdin1 = open (namimg1, 0);
   if (fdin1 < 0)
   {fprintf (stderr, "Unable to open %s\n", namimg1);  exit (-1);}
   fdin2 = open (namimg2, 0);
   if (fdin2 < 0)
   {fprintf (stderr, "Unable to open %s\n", namimg2);  exit (-2);}
   fdout = creat (namout, 0644);
   if (fdout < 0)
   {fprintf (stderr, "Unable to creat %s\n", namout);  exit (-3);}

   /*** Copy housekeeping & bitmap records. ***/

   read (fdin1, hk1, nbylin);
   read (fdin1, bitmap, nbylin);
   read (fdin2, hk2, nbylin);
   hk1 [52] = 0;			/* Sun byte order. */
   hk1 [221] = (char) 16;		/* Delta B image. */

   /*** Copy date & time of image #2 into hskp bytes #143-148 for dif image. */

   hk1 [143] = hk2 [0];			/* Store year2 in byte #143 of hk1. */
   hk1 [144] = hk2 [1];			/* Store doy2a in byte #144 of hk1. */
   hk1 [145] = hk2 [2];			/* Store doy2b in byte #145 of hk1. */
   hk1 [146] = hk2 [20];		/* Store hour2 in byte #146 of hk1. */
   hk1 [147] = hk2 [21];		/* Store min2  in byte #147 of hk1. */
   hk1 [148] = hk2 [24];		/* Store sec2  in byte #148 of hk1. */

   write (fdout, hk1, nbylin);
   write (fdout, bitmap, nbylin);
   read (fdin2, bitmap, nbylin);

   /*** Subtract image #2 from image #1. ***/
   /*** Store result as a short integer image. ***/

   for (line=0; line<448; line++)
      {
      read (fdin1, ibuf1, 896);
      read (fdin2, ibuf2, 896);
      if (nbypix1 == 2 && bytord1 == 1)
         srevbyt (ibuf1, nbypix, 448);
      if (nbypix2 == 2 && bytord2 == 1)
         srevbyt (ibuf2, nbypix, 448);

      for (i=0; i<448; i++)
	 {
	 subval = (int) *(ibuf1 + i) - (int) *(ibuf2 + i);
	 if (subval < 0)
	    {
/*
 *	    fprintf (stdout, "Difference (%d,%d) = %d set to zero.\n",
 *                   i, line, subval);
 */
            subval = 0;
            }
         *(linbuf + i) = (unsigned short) subval;
	 }

/*      fprintf (stdout, "linbuf[%3d]: %d\n", line, *(linbuf + line)); */
      write (fdout, linbuf, 896);
      }

   close (fdin1);
   close (fdin2);
   close (fdout);
}
