/* avelin.c
 *--------------------------------------------------------------------
 *** Averaging routine for an array of numbers.
 *--------------------------------------------------------------------
 *    Andrew L. Stanger   HAO/NCAR   SMM C/P
 *    4 March 1983
 *    6 August 1984: converted from 11/34 (f4p) to VAX 750 (C).
 *    15 November 1990: correct temp index computation.
 *    21 May 1992: improve logic for boundary cases.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *
 *    nave = # elements to average.
 *    buf  = input line array.
 *    nval = number of elements in array buf.
 *
 */
#include <stdio.h>
#include <sys/types.h>
#include "/usr/people/smm/lib/include/cplib.ext"

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

avelin (nave, buf, nval)

int nave;		/* # points to average. */
u_char buf [];		/* Image array. */
int nval;		/* # values in array buf. */

{

#define NT 896
   float	temp [NT];	/* Temporary storage array. */
   float	sum;		/* Storage for sum of pixel intensities. */
   int		ibeg, iend;	/* Begin/End indices for smoothing. */
   int		ip;		/* Pixel index. */
   int		is;		/* Summation index. */
   int		it;		/* Temporary array index. */
   int		navehalf;	/* Half of averaging range. */
   int		ns;		/* # pixels averaged. */
   short	*sp;		/* Pointer to short integer array. */
   float	*fp;		/* Pointer to short integer array. */


   if (nave <= 1)  return;
   if (nval > NT)
      {
      fprintf (stderr, "avelin> Array too large (%d) for smoothing routine.\n",
               nval);
      return (0);
      }

   navehalf = nave / 2;
   ibeg = 0;
   iend = nval;

   if (nbypix == 1)		/* 1-byte/pixel. */
      {
      /*** Copy input array into temp buffer. ***/

      for (ip = 0; ip < nval; ip++)  temp [ip] = (float) buf [ip];

      /*** Average over 'nave' points. ***/

      for (ip = ibeg; ip < iend; ip++)
         {
         sum = 0.0;
         ns = 0;
         for (is = 0; is < nave; is++)
            {
            it = ip - navehalf + is;
            if (it >= 0 && it < npix)
               {
               ns++;
               sum = sum + temp [it];
               }
            }
         *(buf + ip) = (u_char) (sum / (float) ns);
         }
      }

   else
   if (nbypix == 2)		/* 2-bytes/pixel. */
      {
      sp = (short *) buf;
      for (ip = 0; ip < nval; ip++)  temp [ip] = (float) *(sp + ip);

      /*** Average over 'nave' points. ***/

      for (ip = ibeg; ip < iend; ip++)
         {
         sum = 0.0;
         ns = 0;
         for (is = 0; is < nave; is++)
            {
            it = ip - navehalf + is;
            if (it >= 0 && it < npix)
               {
               ns++;
               sum = sum + temp [it];
               }
            }
         *(sp + ip) = (short) (sum / (float) ns);
         }
      }
   else
   if (nbypix == 4)		/* 4-bytes/pixel. */
      {
      fp = (float *) buf;
      for (ip = 0; ip < nval; ip++)  temp [ip] = (float) *(fp + ip);

      /*** Average over 'nave' points. ***/

      for (ip = ibeg; ip < iend; ip++)
         {
         sum = 0.0;
         ns = 0;
         for (is = 0; is < nave; is++)
            {
            it = ip - navehalf + is;
            if (it >= 0 && it < npix)
               {
               ns++;
               sum = sum + temp [it];
               }
            }
         *(fp + ip) = (short) (sum / (float) ns);
         }
      }

   return (0);
}
