/* bprms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bright point removal.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 7 September 1985
 * 26 December 1985: Search for brightest point within a box centered around
 *                   approximate location of bright point.
 * 30 July 1986
 * 7 March 1988: convert to function instead of main program.
 * 17 July 1992: SGI GL version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define NBOX 10 
#define NSEARCH 16 
#define NINT 12

bprms (imgbuf, xdim, ydim)

unsigned short *imgbuf;		/* Image buffer. */
int xdim;			/* X-dimension. */
int ydim;			/* Y-dimension. */
{
   static unsigned short pixval [NSEARCH][NSEARCH];

   static char dir2 [] = "ru";

   int bpx, bpy;
   int dxx = 25;
   int dyy = 25;
   int ibox;
   int level = -1;
   int pixmax;
   int x, y;
   int xmin = 0;
   int ymin = 0;
   int xp, yp;
   int xul = 85;
   int yul = 345;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;
   static int xb [NBOX] = {208, 226, 215, 253, 250, 371, 494, 497, 325, 414};
   static int yb [NBOX] = {504, 504, 493, 494, 418, 340, 303, 228, 159, 130};
   static int xbrt [NBOX];
   static int ybrt [NBOX];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*** Bright Point Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {

      /*** Read box centered about estimated location of bright point. ***/

      xmin = xb [ibox] - dxs/2;
      ymin = yb [ibox] - dys/2;
      mr_box (imgbuf, xdim, ydim, pixval, xmin, ymin, dxs, dys, dir2);

      /*** Initial estimate of bright point location and intensity. ***/

      bpx = xb [ibox];
      bpy = yb [ibox];
      pixmax = pixval [yb[ibox]] [xb[ibox]];

      /*** Search each box for brightest pixel. ***/

      for (y = 0; y < dys; y++)
         {
         yp = ymin + y;
         if (yp >= 0 && yp < ydim)

            for (x = 0; x < dxs; x++)
               {
               xp = xmin + x;
               if ((xp >= 0 && xp < xdim) && (pixval [y][x] > pixmax))
                  {
		  pixmax = pixval [y][x];
		  bpx = xp;
		  bpy = yp;
		  }
               }
         }

   /* Redefine lower left corner of box so that it is centered about the
    * brightest pixel found.
    */

      xbrt [ibox] = bpx - dx/2;
      ybrt [ibox] = bpy - dy/2;

/*
 *      fprintf (stdout, "box: %2d   pixmax: %3d   bpx: %3d   bpy:%3d   ",
 *              ibox,pixmax,bpx,bpy);
 *      fprintf (stdout, "xbrt: %3d   ybrt: %3d\n", xbrt [ibox], ybrt [ibox]);
 */
      }

   /*** Perform horizontal interpolation for first two bright points. ***/

   hint (imgbuf, xdim, ydim, level,      2, xbrt, ybrt, dx, dy, dir2);

   /*** Perform vertical interpolation for remaining bright points. ***/

   vint (imgbuf, xdim, ydim, level, nbox-2, &xbrt [2], &ybrt [2], dx, dy, dir2);

   /*** Vertical interpolation for upper left black/white spot. ***/

   vints (imgbuf, xdim, ydim, level, 1, &xul, &yul, &dxx, &dyy, dir2);

   return (0);
}
