/* cirimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create 2D image & draw circle in image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 13 November 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

main (argc, argv)

int argc;
char *argv [];

{
   unsigned char *imgbuf;
   static char namimg [80];	/* Image file name. */
   int *xit;
   int *yit;
   int *xib;
   int *yib;

   int i;
   int imgfd;			/* Image file descriptor. */
   int ix, iy;			/* X/Y indices. */
   int ixb, iyb;		/* Bottom x/y indices. */
   int ixt, iyt;		/* Top x/y indices. */
   int nbyimg;			/* # bytes in imgbuf. */
   int nbyper;			/* # bytes in xit/yit/xib/yib. */
   int nperim;			/* # values in perimeter hemisphere. */
   int npximg;			/* # pixels in image. */
   int rc = 1;			/* circle Radius. */
   int xc = 0;			/* circle X center. */
   int yc = 0; 			/* circle Y center. */
   int xd = 112;		/* X dimension. */
   int yd = 112;		/* Y dimension. */


   strcpy (namimg, "circle.img");
   xc = xd / 2;
   yc = yd / 2;
   rc = xd / 4;

   if (argc == 2)
      {
      sscanf (argv [1], "%d", &rc);
      }
   else
   if (argc == 4)
      {
      sscanf (argv [1], "%d", &rc);
      sscanf (argv [2], "%d", &xc);
      sscanf (argv [3], "%d", &yc);
      }
   else
   if (argc == 6)
      {
      sscanf (argv [1], "%d", &rc);
      sscanf (argv [2], "%d", &xc);
      sscanf (argv [3], "%d", &yc);
      sscanf (argv [4], "%d", &xd);
      sscanf (argv [5], "%d", &yd);
      }
   else
      {
      fprintf (stderr, "usage: %s radius [xcenter ycenter [xdim ydim]]\n", argv [0]);
      exit (0);
      }

   printf ("rc:%3d  xc:%3d  yc:%3d  xd:%3d  yd:%3d\n", rc, xc, yc, xd, yd);

   /*** Open output image file. ***/

   if ((imgfd = creat (namimg, 0644)) < 0)
      {
      fprintf (stderr, "Unable to open output file: %s\n", namimg);
      exit (-1);
      }

   /*** Allocate storage for arrays. ***/

   nbyimg = xd * yd * sizeof (unsigned char);
   imgbuf = (unsigned char *) malloc (nbyimg);

   nbyper = (rc * 2 + 1) * sizeof (int);
   xit = (int *) malloc (nbyper);
   yit = (int *) malloc (nbyper);
   xib = (int *) malloc (nbyper);
   yib = (int *) malloc (nbyper);

   /*** Computer circle perimeter pixel locations. ***/

   circle_perim (rc, xc, yc, xit, yit, xib, yib, &nperim);

   /*** Initialize image array to all zeroes. ***/

   npximg = xd * yd;
   for (i = 0; i < npximg; i++) 
      *(imgbuf + i) = (char) 0;

   /*** Write value 127 into perimeter pixel locations in image array. ***/

   for (iy = 0; iy < nperim; iy++)
      {
      iyt = *(yit + iy);
      iyb = *(yib + iy);

/*      for (ix = 0; ix < nperim; ix++) */
         {
         ixt = *(xit + iy);
         ixb = *(xib + iy);
         *(imgbuf + ixt + iyt*xd) = (char) 127;
         *(imgbuf + ixb + iyb*xd) = (char) 127;
         }
      }

   /*** Write image to disk file. ***/

   write (imgfd, imgbuf, npximg);
   close (imgfd);

   exit (0);
}
