/* cirimgf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create 2D image & draw circle in image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 13 November 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/stanger/lib/include/img_scale.h"
#include "/usr/people/stanger/lib/include/ov_color.h"

main (argc, argv)

int argc;
char *argv [];

{
   unsigned char *imgbuf;
   unsigned char col_perim = 251;
   unsigned char col_fill  = 254;

   static char namimg [80];	/* Image file name. */

   int *xit;
   int *yit;
   int *xib;
   int *yib;

   int i;			/* General purpose index. */
   int imgfd;			/* Image file descriptor. */
   int is;			/* Scan index. */
   int ix, iy;			/* X/Y indices. */
   int ixb, iyb;		/* Bottom x/y indices. */
   int ixt, iyt;		/* Top x/y indices. */
   int nbyimg;			/* # bytes in imgbuf. */
   int nbypx = 1;		/* # bytes per pixel. */
   int nbyper;			/* # bytes in xit/yit/xib/yib. */
   int nperim;			/* # values in perimeter hemisphere. */
   int npximg;			/* # pixels in image. */
   int ns;			/* # scan values. */
   int rc = 1;			/* circle Radius. */
   int xc = 0;			/* circle X center. */
   int yc = 0; 			/* circle Y center. */
   int xd = 512;		/* X dimension. */
   int yd = 512;		/* Y dimension. */

   static float scanpix [MAXSCN];
   static int scanx [MAXSCN];
   static int scany [MAXSCN];
   double pval;
   double raddeg;

   raddeg = acos (-1.0) / 180.0;

   strcpy (namimg, "circle.img");
   xc = xd / 2;
   yc = yd / 2;
   rc = xd / 4;

   if (argc == 2)
      {
      sscanf (argv [1], "%d", &rc);
      }
   else
   if (argc == 4)
      {
      sscanf (argv [1], "%d", &rc);
      sscanf (argv [2], "%d", &xc);
      sscanf (argv [3], "%d", &yc);
      }
   else
   if (argc == 6)
      {
      sscanf (argv [1], "%d", &rc);
      sscanf (argv [2], "%d", &xc);
      sscanf (argv [3], "%d", &yc);
      sscanf (argv [4], "%d", &xd);
      sscanf (argv [5], "%d", &yd);
      }
   else
      {
      fprintf (stderr, "usage: %s radius [xcenter ycenter [xdim ydim]]\n", argv [0]);
      exit (0);
      }

   printf ("rc:%3d  xc:%3d  yc:%3d  xd:%3d  yd:%3d\n", rc, xc, yc, xd, yd);

   /*** Open output image file. ***/

   if ((imgfd = creat (namimg, 0644)) < 0)
      {
      fprintf (stderr, "Unable to open output file: %s\n", namimg);
      exit (-1);
      }

   /*** Allocate storage for arrays. ***/

   nbyimg = xd * yd * sizeof (unsigned char);
   imgbuf = (unsigned char *) malloc (nbyimg);

   nbyper = 2 * (rc * 2 + 1) * sizeof (int);
   xit = (int *) malloc (nbyper);
   yit = (int *) malloc (nbyper);
   xib = (int *) malloc (nbyper);
   yib = (int *) malloc (nbyper);

   /*** Computer circle perimeter pixel locations. ***/

   circle_perim (rc, xc, yc, xit, yit, xib, yib, &nperim);

   /*** Initialize image array. ***/

   npximg = xd * yd;
   for (iy = 0; iy < yd; iy++) 
      for (ix = 0; ix < xd; ix++) 
	 {
	 pval = (double) (sin ((double)(ix*360.0*raddeg)/1024.0)) * 255.0;
	 while (pval > 255.0)  pval -= 255.0;
	 while (pval < 255.0)  pval += 255.0;
         *(imgbuf + ix + iy * xd) = (unsigned char) pval;
	 }

   /*** Write col_perim into perimeter pixel locations in image array. ***/

   for (i = 0; i < nperim; i++)
      {
      iyt = *(yit + i);
      iyb = *(yib + i);

      ixt = *(xit + i);
      ixb = *(xib + i);
      *(imgbuf + ixt + iyt*xd) = (unsigned char) (1);
      *(imgbuf + ixb + iyb*xd) = (unsigned char) (254);

/*
      *(imgbuf + ixt + 1 + iyt*xd) = (unsigned char) (1);
      *(imgbuf + ixb + 1 + iyb*xd) = (unsigned char) (254);
 */
      }

   /*** Do diameter line scans & fill scan pixels with col_fill. ***/

   for (i = 0; i < nperim; i++)
      {
      ns = MAXSCN;
      clscan (imgbuf, xit [i], yit [i], xib [i], yib [i],
              scanpix, scanx, scany, &ns);
      clinterp (imgbuf, xd, yd, scanx, scany, ns, nbypx);

      /*** Repeat scan & fill with +1 pixel shift in X axis. ***/

      ns = MAXSCN;
      clscan (imgbuf, xit [i] + 1, yit [i], xib [i], yib [i] + 1,
              scanpix, scanx, scany, &ns);
      clinterp (imgbuf, xd, yd, scanx, scany, ns, nbypx);
/*
      for (is = 1; is < ns-1; is++)
         {
         *(imgbuf + scanx [is] + scany [is] * xd) = (unsigned char) col_fill;
         }
 */
      }

   /*** Write image to disk file. ***/

   write (imgfd, imgbuf, npximg);
   close (imgfd);

   exit (0);
}
