/* cirint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Circular interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 9 December 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#define MAXSCN 3600

cirint (imgbuf, nbypx, xd, yd, rc, xc, yc)

char  *imgbuf;			/* Pointer to 1 byte image array. */
int   nbypx;			/* # bytes per pixel. */
int   xd, yd;			/* Dimensions of imgbuf. */
int   rc ;			/* Circle radius (pixel units). */
int   xc, yc ;			/* Circle center coordinates (pixel units). */
{
   int   i ;			/* index. */
   int   *xit ;			/* X coordinates (top hemisphere). */
   int   *yit ;			/* Y coordinates (top hemisphere). */
   int   *xib ;			/* X coordinates (bottom hemisphere). */
   int   *yib ;			/* Y coordinates (bottom hemisphere). */
   int   nperim ;		/* # of perimeter values in xit/yit/xib/yib. */

   int   ip1, ir1, ip2, ir2;	/* Pixel,line coordinate scan limits. */
   int   perdim;		/* Dimension of cirle perimeter arrays. */
   int   ns;			/* Number of values in scan. */
   int   scandim;		/* Dimension of scan arrays. */

   float pi = 3.14159265 ;
   float *scanpix ;		/* scan pixel intensity. */
   int   *scanx ;		/* x-axis scan position. */
   int   *scany ;		/* y-axis scan position. */

   /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   /*** Allocate storage for perimeter arrays. ***/

   perdim = 2 * (rc * 2 + 1) * sizeof (int) ;
   xit = (int *) malloc (perdim) ;
   yit = (int *) malloc (perdim) ;
   xib = (int *) malloc (perdim) ;
   yib = (int *) malloc (perdim) ;

   /*** Allocate storage for scan arrays. ***/

   scandim = (int) ((rc * 2 * pi) + 0.5) ;
   ns = scandim = MAXSCN;
   scanpix = (float *)  malloc (scandim * sizeof (float)) ;
   scanx   = (int *)    malloc (scandim * sizeof (int)) ;
   scany   = (int *)    malloc (scandim * sizeof (int)) ;

   /*** Compute circle perimeter pixel locations.
    *** (xit,yit) & (xib,yib) are pairs of points on circle perimeter.
    ***/

   circle_perim (rc, xc, yc, xit, yit, xib, yib, &nperim) ;

   /*** Do diameter line scans & interpolate between perimeter points. ***/

   for (i = 0; i < nperim; i++)
      {
      clscan (imgbuf, xit [i], yit [i], xib [i], yib [i],
              scanpix, scanx, scany, &ns) ;

      circle_int (imgbuf, xd, yd, scanx, scany, ns, nbypx) ;
      }

   free (xit) ;
   free (xib) ;
   free (yit) ;
   free (yib) ;

   free (scanpix) ;
   free (scanx) ;
   free (scany) ;
   return (0);
}
