/* dfill.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill diagonal (l,u) image elements which are less than 'level' with
 * horizontally-interpolated values.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 11 August 1986
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dfill (imgbuf, nbypx, xsize, ysize, level, x, y, dx, dy, dir2, ddir)


unsigned char *imgbuf;	/* Image buffer. */
int nbypx;		/* # bytes per pixel. */
int xsize, ysize;	/* Image dimensions. */
int level;		/* Intensity threshold level. */
int x, y;		/* Corner coordinates of rectangular image area. */
int dx, dy;		/* Dimensions of area. */
char *dir2;		/* Direction [l,r][u,d] for array storage. */
char *ddir;		/* Direction [l,r] for diagonal (up assumed for vert.)*/

{
   int ix, iy;			/* Indices. */
   int xx, yy;			/* Index. */
   int ival;
   int ierr;
   unsigned char *dimg;	/* Character pointer to diagonal image array. */

   /*** If needed, adjust coordinates so that array order is right and up. ***/

   if ( (*dir2 == 'l') || (*(dir2+1) == 'l') )  x = x - dx + 1;
   if ( (*dir2 == 'd') || (*(dir2+1) == 'd') )  y = y - dy + 1;


   /* Allocate storage for rectangular area of image.
    * Read from `imgbuf' into memory array `dimg'.
    */

   dimg = (unsigned char *) malloc (dx * dy * sizeof (char));

   /*** Read a r->l, d->u diagonal image segment into a rectangular array. ***/

   yy = y - 1;

   for (iy = 0; iy < dy; iy++)
      {
      yy += 1;
      if (*ddir == 'l')  xx = x - iy - 1;
      if (*ddir == 'r')  xx = x + iy - 1;

      for (ix = 0; ix < dx; ix++)
         {
         xx += 1;
         *(dimg + ix + iy*dx) = *(imgbuf + xx + yy * xsize);
         }
      }

   /*** Do horizontal interpolation/fill. ***/

   hfill (dimg, nbypx, dx, dy, level);

   /*** Write rectangular array into a r->l, d->u diagonal image segment. ***/

   yy = y - 1;

   for (iy = 0; iy < dy; iy++)
      {
      yy += 1;
      if (*ddir == 'l')  xx = x - iy - 1;
      if (*ddir == 'r')  xx = x + iy - 1;

      for (ix = 0; ix < dx; ix++)
         {
         xx += 1;
         *(imgbuf + xx + yy * xsize) = *(dimg + ix + iy * dx);
         }
      }

   free (dimg);
   return (0);
}
