/* dkrms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Dark point removal (reseau dots).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * 7 September 1985
 * 26 December 1985: Search for darkest point within a box centered around
 *                   approximate location of bright point.
 * 30 July 1986
 *  7 March 1988: convert to function from main program.
 *  1 March 1991: revise reseau dot locations from measurements on DOY 100 1980.
 * 17 July 1992: SGI GL version.
 * 16 October 1992: add dark point near bullet hole: 179,148 (window coords).
 *  7 June 1993: UNIX version; no screen display.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

#define NBOX 26 
#define NSEARCH 16
#define NINT 11

dkrms (imgbuf, nbypx, xdim, ydim)

unsigned char *imgbuf;		/* Image buffer. */
int nbypx;			/* # bytes per pixel. */
int xdim;			/* X-dimension. */
int ydim;			/* Y-dimension. */
{
   static char dir2 [] = "ru";
   static unsigned char pixval [NSEARCH][NSEARCH];

   int cmoff =   0;			/* Colormap index offset. */
   int darkpx, darkpy;
   int i;
   int ibox;
   int ierr;
   int level = -1;
   int pixmin;
   int x, y;
   int xdp, ydp;
   int ximax = 512, yimax = 512;
   int ximin =   0, yimin =   0;
   int xmin = 0, ymin = 0;
   int xp, yp;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;

   static int xx [NBOX] = {112, 207, 306, 410,       108, 207, 306, 407, 508, 
                           107, 208, 308, 407, 505,  105, 210, 310, 408, 502, 
                           100, 209, 310, 407, 494,  422, 179};

   static int yy [NBOX] = {497, 491, 489, 487,       390, 387, 388, 389, 387, 
                           285, 284, 286, 288, 286,  185, 183, 183, 184, 181, 
                            92,  84,  81,  78,  72,  227, 146};

   static int xd [NBOX];
   static int yd [NBOX];
   static int xdll [NBOX];
   static int ydll [NBOX];
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   ierr = 0;

   for (i = 0; i < NBOX; i++)
      {
      xd [i] = xx [i] -  63;
      yd [i] = 510 - yy [i];
      }

   if (xdim == 448)
      {
      ximin = yimin =   1;
      ximax = yimax = 446;
      }

   /*** Reseau Dot Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {
      xmin = xd [ibox] - dxs/2;		/* Lower left corner of box. */
      ymin = yd [ibox] - dys/2;

      if  (xmin < ximin)  xmin = ximin;
      if  (ymin < yimin)  ymin = yimin;
      if ((xmin + dxs) > ximax)  xmin = ximax - dxs;
      if ((ymin + dys) > yimax)  ymin = yimax - dys;

      /*** Read box centered about estimated reseau dot location. ***/

      ierr = mr_box (imgbuf, nbypx, xdim, ydim, pixval,
		     xmin, ymin, dxs, dys, dir2);
      if (ierr < 0)
	 {
	 fprintf (stderr, "dkrms> mr_box error: %d  x:%3d  y:%3d\n",
		  ierr, xmin, ymin);
         return (ierr);
	 }

      /*** Initial estimate for reseau dot location and intensity. ***/

      xdp = xd [ibox];
      ydp = yd [ibox];      
      pixmin = 255 + cmoff;

      /*** Search each box for darkest pixel. ***/

      for (y = 0; y < dys; y++)
         {
         yp = ymin + y;
         if (yp > yimin && yp < yimax)

            for (x = 0; x < dxs; x++)
               {
               xp = xmin + x;
               if ((xp > ximin && xp < ximax) && (pixval [y][x] < pixmin))
                  {
                  pixmin = pixval [y][x];
                  xdp = xp;
                  ydp = yp;
                  darkpx = x;
                  darkpy = y;
                  }
              }
         }

      /* Redefine lower left corner of box so that it is centered about the
       * darkest pixel found.
       */

        xdp -= 1;				/* Fudge adjustment. */
        ydp += 2;				/* Fudge adjustment. */
	xdll [ibox] = xdp - dx/2;		/* Lower left corner of box. */
	ydll [ibox] = ydp - dy/2;

	if (xdll [ibox] < ximin)  xdll [ibox] = ximin;
	if (ydll [ibox] < yimin)  ydll [ibox] = yimin;
	if (xdll [ibox] + dx > ximax)  xdll [ibox] = ximax - dx;
	if (ydll [ibox] + dy > yimax)  ydll [ibox] = yimax - dy;

/*
 *   printf ("box:%2d  x/yd:%3d  %3d  x/ydp:%3d  %3d  x/ydll:%3d  %3d  pixval: %3d  %3d\n",
 *           ibox, xd [ibox], yd [ibox], xdp, ydp, xdll [ibox], ydll [ibox],
 *           pixval [darkpy][darkpx]-cmoff, *(imgbuf + xdp + ydp*xdim)-cmoff);
 */
      }

   /*** Perform horizontal interpolation for each box. ***/

   ierr = vint (imgbuf, nbypx, xdim, ydim, level,
		nbox, xdll, ydll, dx, dy, dir2);
   if (ierr < 0)
      {
      fprintf (stderr, "dkrms> vint  error: %d.\n", ierr);
      return (ierr);
      }

   return (0);
}
