/* hint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill image elements which are less than 'level' with
 * horizontally-interpolated values.
 * For MULTIPLE box areas.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 17 July 1992: SGI GL version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

hint (imgbuf, nbypx, xsize, ysize, level, nbox, xx, yy, dx, dy, dir2)

unsigned char *imgbuf;	/* Image buffer. */
int nbypx;		/* # bytes per pixel. */
int xsize;		/* X dimension of image. */
int ysize;		/* Y dimension of image. */
int level;		/* Intensity threshold level. */
int nbox;		/* Number of values in arrays x & y. */
int *xx, *yy;		/* Arrays: corner coordinates of rectangular image. */
int dx, dy;		/* Dimensions of area. */
char *dir2;		/* Direction [l,r][u,d] for array storage. */
{
   int ibox;
   int ierr;
   int x, y;
   unsigned char *aimg;	/* Pointer to image array. */
   int ix, iy;

   ierr = 0;

   /* Allocate storage for rectangular area of image.
    * Read from image array `imgbuf' into temporary memory array `aimg'.
    */

   aimg = (unsigned char *)  malloc (dx * dy * sizeof (char));

   /*** Box loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {
      x = xx [ibox];
      y = yy [ibox];

      /* If needed, adjust coordinates so that array order is right and up. */

      if ( (*dir2 == 'l') || (*(dir2+1) == 'l') )  x = x - dx + 1;
      if ( (*dir2 == 'd') || (*(dir2+1) == 'd') )  y = y - dy + 1;

      /* Read subset of image. */

      ierr = mr_box (imgbuf, nbypx, xsize, ysize, aimg, x, y, dx, dy, "ru");
      if (ierr < 0)
	 {
	 fprintf (stderr, "hint> mr_box error: %d  x:%3d  y:%3d\n",
		  ierr, x, y);
         return (ierr);
         }

      /*** Do interpolation/fill. ***/


/*
 *      for (iy = 0; iy < dy; iy++)
 *         for (ix = 0; ix < dx; ix++)
 *            *(aimg + ix + iy * dx) = 1;
 */

      hfill (aimg, nbypx, dx, dy, level);

      /*** Write resultant image to array `imgbuf'. */

      ierr = mw_box (imgbuf, nbypx, xsize, ysize, aimg, x, y, dx, dy, "ru");
      if (ierr < 0)
	 {
	 fprintf (stderr, "hint> mw_box error: %d  x:%3d  y:%3d\n",
		  ierr, x, y);
         return (ierr);
         }
      }

   free (aimg);
   return (0);
}
