/* scrmss.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Calibration reticle scratch mark removal.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define NBOX 4

scrms (cbuf, nbypx, xdim, ydim)

unsigned char	*cbuf;		/* Image buffer. */
int		nbypx;		/* # bytes per pixel. */
int		xdim;		/* X-dimension. */
int		ydim;		/* Y-dimension. */
{
   static char dir2 [] = "ru";
   unsigned char level = 0;

   float pixsum = 0.0;
   int pixave = 0;
   int pixcnt = 0;

   int darkpx, darkpy;
   int i;
   int ix, iy;
   int xmax = 0, ymax = 0;
   int xmin = 0, ymin = 0;

   static int xs  [NBOX] = {247, 317, 352, 385};   /* Minimum corner of box. */
   static int ys  [NBOX] = {284, 284, 284, 284};
   static int dxs [NBOX] = { 70,  35,  33,  63};
   static int dys [NBOX] = { 48,  21,  21,  21};
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   if (nbypx != 1)
      {
      fprintf (stderr, "scrms> Invalid image type. Must be byte image.\n");
      return (-1);
      }

   for (i = 0; i < NBOX; i++)
      {

      /* Minimum corner of box. */

      xmin = xs [i];
      ymin = ys [i];
      if (xmin < 0)  xmin = 0;
      if (ymin < 0)  ymin = 0;

      /* Maximum corner of box. */

      xmax = xmin + dxs [i];
      ymax = ymin + dys [i];
      if (xmax > xdim)  xmax = xdim - dxs [i];
      if (ymax > ydim)  ymax = ydim - dys [i];

      pixsum = 0.0;
      pixave = 0;
      pixcnt = 0;

      /*** Find average pixel intensity within box. ***/

      for (iy = ymin; iy < ymax; iy++)
         for (ix = xmin; ix < xmax; ix++)
            {
            pixsum += (float) *(cbuf + ix + iy*xdim);
            pixcnt++;
            }

      pixave = (int) (pixsum / (float) pixcnt);
      level = (unsigned char) ((int) (pixave * 1.05) & 0377);
/*
 *      printf ("pixave: %d   pixcnt: %d   pixsum: %6.2f\n",
 *              pixave, pixcnt, pixsum);
 */

      /*** Replace any pixel whose value is greater than level with pixave. ***/

      for (iy = ymin; iy < ymax; iy++)
         for (ix = xmin; ix < xmax; ix++)
            {
            if (*(cbuf + ix + iy*xdim) > level)
               *(cbuf + ix + iy*xdim) = pixave;
            }
      }

   return (0);
}
