/* cpclean84.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Interpolate over reseau dots and bright points in a sequence of C/P images.
 * Also remove vidicon blemishes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 28 September 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
int DSP = 1;

main (argc, argv)

int argc;
char *argv [];

{
   extern char *malloc ();
   FILE *LIST;

   static int xbrt [1] = 100;
   static int ybrt [1] = 200;
   static int dxb = 50;
   static int dyb = 50;

   static char namimg [80];
   static char namlst [80];
   unsigned short *sbuf;

   int linit = 1;
   int maxpix;
   int maxlin;
   int nbypx;
   int seq = 0;			/* Flag to indicate sequence of images. */
   int xll, yll;		/* GL window: lower left coordinates. */
   int xur, yur;		/* GL window: upper right coordinates. */

   if (argc == 2)
      strcpy (namimg, argv [1]);
   else
      {
      fprintf (stderr, "usage: %s cp_filename | @cp_filename_list\n", argv [0]);
      exit (0);
      }

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

loop:

   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }
   /*** Decode C/P image housekeeping. ***/

   ldisk = 1;
   cphskp (namimg);

   /*** Initialize GL software. ***/

   foreground ();
   if (H.ires == 1)		/* Low resolution image (512x512) window. */
      {
      xll = 760;
      yll = 480;
      xur = 1271;
      yur = 991;
      }
   else				/* High resolution (1024x1024) image window. */
      {
      xll = 266;
      yll =   0;
      xur = 1279;
      yur = 1023;
      }

   maxpix = xur - xll + 1;
   maxlin = yur - yll + 1;

   prefposition (xll, xur, yll, yur);
   fminit ();				/* Initialize SGI font manager. */

   winopen (argv [0]);			/* Create GL window. */
   gl_wclear (BLACK);			/* Clear window to black. */

   /*** Display C/P image in GL window. ***/

   glcp_view (namimg, linit, 0);	/* Display image in GL window. */

   sbuf = (unsigned short *) malloc (maxpix * maxlin * sizeof (short));

   /*** Read C/P image from GL window. ***/

   rectread (0, 0, maxpix-1, maxlin-1, sbuf);
   nbypx = sizeof (short);

   /*** Remove bright points (pin holes). ***/

   bprms (sbuf, maxpix, maxlin);
   rectwrite (0, 0, maxpix-1, maxlin-1, sbuf);

   /*** Remove dark points (reseau dots). ***/

   dkrms (sbuf, maxpix, maxlin);
   rectwrite (0, 0, maxpix-1, maxlin-1, sbuf);

   blrms (sbuf, maxpix, maxlin);
   rectwrite (0, 0, maxpix-1, maxlin-1, sbuf);

   /*** Remove fiber in upper right quadrant of image. ***/

/*
 *   fibrms (sbuf, maxpix, maxlin);
 *   rectwrite (0, 0, maxpix-1, maxlin-1, sbuf);
 */

   /*** Store interpolated image into a disk file in C/P image format. ***/

   gltofs (namimg);

   sleep (1);
   if (seq)  goto loop;

quit:

   sleep (3);
   gexit ();
   exit (0);

   /*** Window remains on screen until terminated by mouse. ***/

/*
   qdevice (ESCKEY);
   qdevice (WINCLOSE);
   qdevice (WINQUIT);
   for (;;)
      {
      short val;
      switch (qread (&val))
         {
         case REDRAW:
            glcp_view (namimg, linit, 0);
            break;
         case ESCKEY:
         case WINCLOSE:
         case WINQUIT:
            gexit ();
            exit (0);
         }
      }
 */
}
