/* cperim.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Circle perimeter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Locate the x/y coordinates of the circle perimeter,
 * given the radius of the circle, and the center coordinates of the circle.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 13 November 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

main (argc, argv)

int argc;
char *argv [];

{
   int i;
   int rc;			/* Radius of circle. */
   int xc = 0, yc = 0;		/* Center coordinates of circle. */
   int *xit, *yit;
   int *xib, *yib;
   int nperim;
   int nbytes;

   double degrad;
   double angle;
   double xt, yt;

   if (argc == 2)
      {
      sscanf (argv [1], "%d", &rc);
      }
   else
   if (argc == 4)
      {
      sscanf (argv [1], "%d", &rc);
      sscanf (argv [2], "%d", &xc);
      sscanf (argv [3], "%d", &yc);
      }
   else
      {
      fprintf (stderr, "usage: %s radius [xcenter ycenter]\n", argv [0]);
      exit (0);
      }

   degrad = 180.0 / acos (0.0);
   nbytes = (rc * 2 + 1) * sizeof (int) ;
   xit = (int *) malloc (nbytes);
   yit = (int *) malloc (nbytes);
   xib = (int *) malloc (nbytes);
   yib = (int *) malloc (nbytes);

   circle_perim (rc, xc, yc, xit, yit, xib, yib, &nperim);

   fprintf (stdout, "nbytes: %3d  nperim:%3d  rc:%3d  xc:%3d  yc:%3d\n",
            nbytes, nperim, rc, xc, yc);
   fprintf (stdout, "  i    angle      xt      yt   xit   yit   xib   yib\n");

   for (i = 0; i < nperim; i++)
      {
      xt = (double) (xit [i] - xc);
      yt = (double) (yit [i] - yc);
      angle = atan2 (yt, xt) * degrad;
/*
      fprintf (stdout, "%3d   %6.2f   %5.1f   %5.1f   %3d   %3d   %3d   %3d\n",
               i, angle, xt, yt, *(xit + i), *(yit + i), *(xib + i), *(yib + i));
 */
      fprintf (stdout, "%3d   %6.2f   %5.1f   %5.1f   %3d   %3d   %3d   %3d\n",
               i, angle, xt, yt, xit [i], yit [i], xib [i], yib [i]);
      }


   exit (0);
}
