/* dvfill.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill diagonal (l,u) image elements which are less than 'level' with
 * vertically-interpolated values.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 11 August 1986
 *  6 October 1992: vertical interpolation on a diagonal image segment.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dvfill (imgbuf, xsize, ysize, level, x, y, dx, dy, slope, dir2, ddir)


unsigned short *imgbuf;	/* Image buffer. */
int xsize, ysize;	/* Image dimensions. */
int level;		/* Intensity threshold level. */
int x, y;		/* Corner coordinates of rectangular image area. */
int dx, dy;		/* Dimensions of area. */
float slope;		/* delta y / delta x : slope of "line" artifact. */
char *dir2;		/* Direction [l,r][u,d] for array storage. */
char *ddir;		/* Direction [l,r] for diagonal (up assumed for vert.)*/

{
   int ix, iy;			/* Indices. */
   int xx, yy;			/* Index. */
   int ival;
   int ierr;
   unsigned short *dimg;	/* Character pointer to diagonal image array. */

/*   printf ("\n*** dvfill ******************************\n"); */

   /*** If needed, adjust coordinates so that array order is right and up. ***/

   if ( (*dir2 == 'l') || (*(dir2+1) == 'l') )  x = x - dx + 1;
   if ( (*dir2 == 'd') || (*(dir2+1) == 'd') )  y = y - dy + 1;

   /* Allocate storage for rectangular area of image.
    * Read from `imgbuf' into memory array `dimg'.
    */

   dimg = (unsigned short *) malloc (dx * dy * sizeof (short));

   /*** Read a d->u, l->r diagonal image segment into a rectangular array. ***/

   for (iy = 0; iy < dx; iy++)
      {
      if (*ddir == 'l')  xx = x - iy - 1;
      if (*ddir == 'r')  xx = x + iy - 1;
      yy = (int) ((float) y + iy * slope) - 1;

      for (ix = 0; ix < dy; ix++)
         {
         yy += 1;
         *(dimg + ix + iy*dy) = *(imgbuf + xx + yy * xsize);


/*
 *         printf ("dvfill> ix: %3d   iy: %3d   xx: %3d  yy: %3d\n",
 *                 ix, iy, xx, yy);
 */

         }
      }

   /*** Do horizontal interpolation/fill. ***/

   hfill (dimg, dy, dx, level);

/*   printf ("hfill (%3d,%3d)\n", dy, dx); */

   /*** Write rectangular array into a d->u, l->r diagonal image segment. ***/

   for (iy = 0; iy < dx; iy++)
      {
      if (*ddir == 'l')  xx = x - iy - 1;
      if (*ddir == 'r')  xx = x + iy - 1;
      yy = (int) ((float) y + iy * slope) - 1;

      for (ix = 0; ix < dy; ix++)
         {
         yy += 1;
         *(imgbuf + xx + yy * xsize) = *(dimg + ix + iy * dy);

/*
 *         printf ("dvfill> ix: %3d   iy: %3d   xx: %3d  yy: %3d\n",
 *                 ix, iy, xx, yy);
 */
         }
      }

   free (dimg);
   return (0);
}
