/* gl_rbox.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read an SMM C/P image from an SGI IRIS GL window to a memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 16 July 1992: all image orientations supported.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
typedef unsigned char u_char;
typedef unsigned short u_short;

gl_rbox (imgbuf, nbypx, xdim, ydim, xs, ys, dx, dy, dir2, soff, pmode)

u_char *imgbuf;			/* Image memory buffer. */
int nbypx;			/* # bytes per pixel. */
int xdim, ydim;			/* X & Y dimensions of imgbuf. */
int xs, ys;			/* Coordinates for sub-image origin in imgbuf.*/
int dx, dy;			/* Size of sub-image. */
char *dir2;			/* Direction to read sub-image: l/r, u/d. */
short soff;			/* Subtract 512 to each pixel for onemap mode.*/
int pmode;			/* Print mode. */
{
   extern char *malloc ();	/* Memory allocation. */
   char xdir, ydir;
   int bytpix = 2;		/* # bytes per pixel for output image. */
   int col;			/* Column index counter. */
   int i;			/* Loop index. */
   int ix, iy;
   int ymax, yhalf, ny;
   short sval;
   int nbyimg;			/* # bytes in output image. */
   int nelem;			/* # pixels in image. */
   int rel = 0;			/* Relative flag for fb_setaddr. */
   int row;			/* Row index counter. */
   int xll, yll;		/* Lower left corner. */
   int xur, yur;		/* Upper right corner. */
   u_char  *cimgbuf;
   u_short *simgbuf;
   u_short *sbuf;
   float *fimgbuf;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   if (pmode)
      fprintf (stderr, "gl_wbox> nbypx: %d  xdim: %d  ydim: %d  xs: %d  ys: %d  dx: %d  dy: %d  dir2: %s\n",
               nbypx, xdim, ydim, xs, ys, dx, dy, dir2);

   /*** Determine storage orientation of image. ***/
   /*
    * GL uses an origin at lower left corner of frame buffer.
    * Therefore, convert any input image so that increasing X is to right,
    * and increasing Y is upward.
    */

   xdir = 'x';
   ydir = 'y';

   for (i=0; i<2; i++)
      switch (*(dir2 + i))
         {
         case 'l':
         case 'r': xdir = *(dir2 + i);
                   break;
         case 'u':
         case 'd': ydir = *(dir2 + i);
                   break;
         }

   if (xdir == 'x' || ydir == 'y')
      {
      fprintf (stderr, "***gl_rbox> Illegal direction (%s).\n", dir2); 
      return (-1);
      }

   /*** Set lower left and upper right x & y rectangle coordinates
    *** for compatibility with readrect.
    ***/

   if (xdir == 'r')
      xll = xs;
   else
      xll = xdim - xs - 1;

   xur = xll + xdim - 1;

   if (ydir == 'r')
      yll = ys;
   else
      yll = ydim - ys - 1;

   yur = yll + ydim - 1;

   if (pmode)
      {
      fprintf (stderr, "   xdir: %c   ydir: %c\n", xdir, ydir);
      fprintf (stderr,
              "   xll: %d   yll: %d   xur: %d   yur: %d\n", xll, yll, xur, yur);
      }


   /*** Allocate memory for temporary image buffer & define pointer values. ***/

   nelem = dx * dy;
   nbyimg = nelem * sizeof (short);
   sbuf = (u_short *) malloc (nbyimg);

   cimgbuf = (u_char *) imgbuf;
   simgbuf = (u_short *) imgbuf;
   fimgbuf = (float *) imgbuf;

   /*** Copy image into temporary array. ***/

   rectread (xll, yll, xur, yur, sbuf);

   /*** Manipulate image as needed for user's image orientation. ***/

   if (xdir == 'l' || xdir == 'r')	/* Horizontally-scanned image. */
      {
      if (xdir == 'l')			/* Image scanned right to left. */
         {
         xs -= (xdim - 1);
         swapcols (sbuf, bytpix, dx, dy, pmode); /* Reverse column order. */
         }
      if (ydir == 'd')			/* First scan line is at top of image.*/
         {
         ys -= (ydim - 1);
         swaprows (sbuf, bytpix, dx, dy, pmode);   /* Reverse row order. */
         }
      }

   else					/* Vertically-scanned image. */
      {
      if (xdir == 'd')			/* Image is scanned top to bottom. */
         {
         ys -= (ydim - 1);
         swaprows (sbuf, bytpix, dx, dy, pmode);   /* Reverse row order. */
         }
      if (ydir == 'l')			/* First column is at right of image. */
         {
         xs -= (xdim - 1);
         swapcols (sbuf, bytpix, dx, dy, pmode); /* Reverse column order. */
         }
      }

   /*** Write image to user's memory array. ***/

   if (nbypx == 1)
      for (i = 0; i < nelem; i++)
         *(cimgbuf + i) = (char) (*(sbuf + i) & 0377) - soff;
   else
   if (nbypx == 2)
      for (i = 0; i < nelem; i++)
         *(simgbuf + i) = (u_short) (*(sbuf + i) & 0377) - soff;
   else
   if (nbypx == 4)
      for (i = 0; i < nelem; i++)
         *(fimgbuf + i) = (float) *(fbuf + i) & 0377 - soff;

   free (sbuf);
   return (0);
}

/* swaprows
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse image top to bottom.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

swaprows (ibuf, nbytpx, xsize, ysize, pmode)

char *ibuf;
int nbytpx;
int xsize, ysize;
int pmode;

{
int ix;
int iy, ny;
int yhalf, ymax;
char cval;
short sval;
float fval;
short *sibuf;
float *fibuf;

   if (pmode)
      fprintf (stderr,
           "   swaprows> nbytpx: %d   xsize: %d   ysize: %d\n", nbtypx, xsize, ysize);
   sibuf = (short *) ibuf;
   fibuf = (float *) ibuf;

   /*** Flip image so that origin is lower left instead of upper left. ***/

   ymax = ysize - 1;
   yhalf = ysize / 2;

   if (nbytpx == 1)				/* Byte image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xsize; ix++)
            {
            cval = *(ibuf + ix + iy*xsize);
            *(ibuf + ix + iy*xsize) = *(ibuf + ix + ny*xsize);
            *(ibuf + ix + ny*xsize) = cval;
            }
         }
      }
   else
   if (nbytpx == 2)				/* Short integer image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xsize; ix++)
            {
            sval = *(sibuf + ix + iy*xsize);
            *(sibuf + ix + iy*xsize) = *(sibuf + ix + ny*xsize);
            *(sibuf + ix + ny*xsize) = sval;
            }
         }
      }

   else
   if (nbytpx == 4)				/* Floating point image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xsize; ix++)
            {
            fval = *(fibuf + ix + iy*xsize);
            *(fibuf + ix + iy*xsize) = *(fibuf + ix + ny*xsize);
            *(fibuf + ix + ny*xsize) = fval;
            }
         }
      }

   return (0);
}


/* swapcols
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse image left to right.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

swapcols (ibuf, nbytpx, xsize, ysize, pmode)

char *ibuf;
int nbytpx;
int xsize, ysize;
int pmode;

{
int ix, nx;
int iy;
int xhalf, xmax;
char cval;
short sval;
float fval;
short *sibuf;
float *fibuf;

   if (pmode)
      fprintf (stderr,
           "   swapcols> nbytpx: %d   xsize: %d   ysize: %d\n", nbytpx, xsize, ysize);
   sibuf = (short *) ibuf;
   fibuf = (float *) ibuf;

   /*** Flip image left to right. ***/

   xmax = xsize - 1;
   xhalf = xsize / 2;

   if (nbytpx == 1)				/* Byte image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ysize; iy++)
            {
            cval = *(ibuf + ix + iy*xsize);
            *(ibuf + ix + iy*xsize) = *(ibuf + nx + iy*xsize);
            *(ibuf + nx + iy*xsize) = cval;
            }
         }
      }

   else
   if (nbytpx == 2)				/* Short image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ysize; iy++)
            {
            sval = *(sibuf + ix + iy*xsize);
            *(sibuf + ix + iy*xsize) = *(sibuf + nx + iy*xsize);
            *(sibuf + nx + iy*xsize) = sval;
            }
         }
      }

   else
   if (nbytpx == 4)				/* Floating point image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ysize; iy++)
            {
            fval = *(fibuf + ix + iy*xsize);
            *(fibuf + ix + iy*xsize) = *(fibuf + nx + iy*xsize);
            *(fibuf + nx + iy*xsize) = fval;
            }
         }
      }

   return (0);
}
