/* gltofs.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Save GL image as an SMM image (1 byte/pixel) on disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 23 July 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include "/usr/people/smm/lib/include/cplib.ext"

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>


/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

gltofs (namimg)

char *namimg;		/* Image file name. */
{

   extern unsigned char *rdfimg ();	/* Read image from disk to memory. */
   unsigned char *cbuf;		/* Pointer to image array. */
   static char answer [2];	/* Storage for y/n response. */
   static char namout [80];

   int fd;
   int i, idot;
   int imgsiz;
   int len;

   /*** Open input image disk file and read housekeeping record. ***/

   cbuf = rdfimg (namimg);
   hkline (stdout);
   if (H.ires == 0)
      {
      fprintf (stderr, "Invalid image; high resolution.\n");
      exit (-1);
      }

/*
   printf ("ih,iv: %d %d\n", H.ih, H.iv);
   printf ("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
   printf ("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
   printf ("xcen,ycen: %f %f\n", xcen, ycen);
 */

   cbuf [70] = 16;				/* Sun-4 byte order. */
   cbuf [103] = 8;				/* # bits per pixel. */
   cbuf [110] = (char) ((npix / 256) & 0377);	/* # pixels per line. */
   cbuf [111] = (char) (npix & 0377);

   /*** Copy GL image into image array. ***/

   cpgltois (cbuf + npix*2, npix, npix);

   /*** Write image to disk file. ***/

   idot = -1;
   len = strlen (namimg);
   strcpy (namout, namimg);

   for (i=0; i<len; i++)
	{
	if (namimg [i] == '.')  {idot = i;  break;}
	}
   namout [idot+1] = 'g';
   if (idot < 0)
      {
      fprintf (stderr, "Invalid image name: %s\n", namimg);
      exit (-1);
      }

   imgsiz = npix * (npix + 2) * sizeof (char);
   fd = creat (namout, 0644);
   write (fd, cbuf, imgsiz);
   close (fd);

   /*** Relinquish storage for image array. ***/

   free (cbuf);
   return (0);
}
