/* gtof.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Save Grinnell image as an SMM image on disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 21 August 1985
 * 11 August 1987: set # bits per pixel to 8 in housekeeping.
 * 7 March 1988: convert to function from a main program.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include "/usr/people/smm/lib/include/cplib.ext"

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>


/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

gtofs (namimg, lgrdev, chan)

char *namimg;		/* Image file name. */
int lgrdev;
int chan;
{

char	*rdfimg();	/* Read image from disk to memory. */
char	*image;		/* Pointer to image array. */
static char namsav[] = "Gr.sav";
static	char	answer[2];	/* Storage for y/n response. */
static	char	namout[80];
int i, idot;
int fd;
int imgsiz;
int len;

   /*** Display Grinnell channel. ***/

   imgch = chan;
   disp (chan);

   /*** Open input image disk file and read housekeeping record. ***/

   image = rdfimg (namimg);
   hkline ();
   if (H.ires == 0)
      {
      fprintf (stderr, "Invalid image; high resolution.\n");
      exit (-1);
      }

/*
   printf ("ih,iv: %d %d\n", H.ih, H.iv);
   printf ("ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
   printf ("npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
   printf ("xcen,ycen: %f %f\n", xcen, ycen);
 */

   image [70] = 16;				/* Sun-4 byte order. */
   image [103] = 8;				/* # bits per pixel. */
   image [110] = (char) ((npix / 256) & 0377);	/* # pixels per line. */
   image [111] = (char) (npix & 0377);

   /*** Copy Grinnell image into image array. ***/

   cpgtois (image);

   /*** Write image to disk file. ***/

   idot = -1;
   len = strlen (namimg);
   for (i=0; i<len; i++)
	{
	if (namimg[i] == '.')  idot = i;
	namout [i] = namimg [i];
	}
   namout [idot+1] = 'g';
   if (idot < 0)
      {
      fprintf (stderr, "Invalid image name: %s\n", namimg);
      exit (-1);
      }

   imgsiz = npix * (npix + 2);
   fd = creat (namout, 0644);
   write (fd, image, imgsiz);
   close (fd);

   /*** Relinquish storage for image array. ***/

   free (image);
   return (0);
}
