/* hints.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill image elements which are less than 'level' with
 * horizontally-interpolated values.
 * For MULTIPLE box areas & variable sizes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 30 July 1986
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

hints (imgbuf, xsize, ysize, level, nbox, xx, yy, dxx, dyy, dir2)


unsigned short *imgbuf;	/* Image buffer. */
int xsize;		/* X dimension of image. */
int ysize;		/* Y dimension of image. */
int level;		/* Intensity threshold level. */
int nbox;		/* Number of values in arrays x & y. */
int *xx, *yy;		/* Arrays: corner coordinates of rectangular image. */
int *dxx, *dyy;		/* Dimensions of area. */
char *dir2;		/* Direction [l,r][u,d] for array storage. */
{
   int ibox;
   int ix, iy;
   int x, y;
   int dx, dy;
   unsigned short *aimg;	/* Pointer to image array. */

/*   fprintf (stdout, "x/ysize:%4d  level:%3d  nbox:%3d  dir2: %s\n",
 *            xsize, ysize, level, nbox, dir2);
 */

   /*** Box loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {
      x = xx [ibox];
      y = yy [ibox];
      dx = dxx [ibox];
      dy = dyy [ibox];

      /* Allocate storage for rectangular area of image.
       * Read from Grinnell channel 'a' into memory array.
       */

      aimg = (unsigned short *)  malloc (dx * dy * sizeof (short));

      /* If needed, adjust coordinates so that array order is right and up. */

      if ( (*dir2 == 'l') || (*(dir2+1) == 'l') )  x = x - dx + 1;
      if ( (*dir2 == 'd') || (*(dir2+1) == 'd') )  y = y - dy + 1;

/*      fprintf (stdout, "hints> x/y:%4d %4d  dx/y:%4d %4d\n", x, y, dx, dy);
 */

      mr_box (imgbuf, xsize, ysize, aimg, x, y, dx, dy, "ru");

      /*** Do interpolation/fill. ***/

/*
 *      for (iy = 0; iy < dy; iy++)
 *         for (ix = 0; ix < dx; ix++)
 *            *(aimg + ix + iy * dx) = 2;
 */

      hfill (aimg, dx, dy, level);

      /*** Write resultant image to array `imgbuf'. ***/

      mw_box (imgbuf, xsize, ysize, aimg, x, y, dx, dy, "ru");

      free (aimg);
      }

   return (0);
}
