/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 * XCAT - Coronal Analysis Tool - Mike Hoswell - Solar Maximum Mission
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 * FILENAME: hinterp.c
 *
 * DESCRIPTION: Horizontal linear interpolation for SMM C/P images.
 *
 * UPDATES:
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 2 November 1990
 * $Log: hinterp.c,v $
 * Revision 3.1  1992/06/22  17:55:35  hoswell
 * New version - with overlays.
 *
 */

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 * hinterp
 */
void
hinterp (ibuf, npxlin, x1, x2, y1, y2)

unsigned char  *ibuf;			/* Image array (2D). */
int             npxlin;			/* # pixels per line. */
int             x1, x2;			/* Minimum & maximum x coordinates. */
int             y1, y2;			/* Minimum & maximum y coordinates. */

{
   int    ih1, ih2;
   int    iv1, iv2;
   int    ix, iy;
   int    ival;
   int    lx, hx;
   float  vallo, valhi;
   float  frac;

   ih1 = x1;
   ih2 = x2;
   iv1 = y1;
   iv2 = y2;

   lx = ((ih1 > 0) ? ih1 - 1 : 0);			/* Set min x coord. */
   hx = ((ih2 < npxlin - 1) ? ih2 + 1 : npxlin - 1);	/* Set max x coord. */

   /*** Interpolation Loop. ***/

   for (iy = iv1; iy < iv2; iy++)
      {
      for (ix = ih1; ix < ih2; ix++)
         {
	 vallo = *(ibuf + lx + iy * npxlin);
	 valhi = *(ibuf + hx + iy * npxlin);

	 frac = (float) (ix - ih1) / (float) (ih2 - ih1);

	 ival = (int) (frac * (valhi - vallo) + vallo);

	 *(ibuf + ix + iy * npxlin) = ival;
         }
      }
}
