/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 * XCAT - Coronal Analysis Tool - Mike Hoswell - Solar Maximum Mission
 *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 * FILENAME: vinterp.c
 *
 * DESCRIPTION: Vertical linear interpolation for SMM C/P images.
 *
 * UPDATES:
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 2 November 1990
 * $Log: vinterp.c,v $
 * Revision 3.1  1992/06/22  17:55:35  hoswell
 * New version - with overlays.
 *
 */

/*-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 * vinterp
 */

void
vinterp (ibuf, npxlin, x1, x2, y1, y2)

unsigned char  *ibuf;		/* Image array (2D). */
int             npxlin;		/* # pixels per line. */
int             x1, x2;		/* Minimum & maximum x coordinates. */
int             y1, y2;		/* Minimum & maximum y coordinates. */

{
   int    ih1, ih2;
   int    iv1, iv2;
   int    ix, iy;
   int    ival;
   int    ly, hy;
   float  vallo, valhi;
   float  frac;

   ih1 = x1;
   ih2 = x2;
   iv1 = y1;
   iv2 = y2;

   ly = ((iv1 > 0) ? iv1 - 1 : 0);			/* Set min y coord. */
   hy = ((iv2 < npxlin - 1) ? iv2 + 1 : npxlin - 1);	/* Set max y coord. */

   /*** Interpolation Loop. ***/

   for (ix = ih1; ix < ih2; ix++)
      {
      for (iy = iv1; iy < iv2; iy++)
         {
         vallo = *(ibuf + ix + ly * npxlin);
         valhi = *(ibuf + ix + hy * npxlin);

         frac = (float) (iy - iv1) / (float) (iv2 - iv1);

         ival = (int) (frac * (valhi - vallo) + vallo);

         *(ibuf + ix + iy * npxlin) = ival;
         }
      }
}
