/* mw_box.c
 ****************************************************************************
 * Write values within a box to a C/P image memory array.
 ****************************************************************************
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 17 July 1992
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
#include <stdio.h>

mw_box (cpdat, nbypx, xdim, ydim, cpbox, xs, ys, dx, dy, dir2)

unsigned short *cpdat;	/* Full image array. */
int nbypx;		/* # bytes per pixel. */
int xdim, ydim;		/* Size of full image. */
unsigned short *cpbox;	/* Subset image array. */
int xs, ys;		/* Box origin of subset image in full image. */
int dx, dy;		/* X/Y dimensions of subset image. */
char dir2 [];		/* Read direction: l(eft)/r(ight),u(p)/d(own). */

{
   int ix, iy;
   int sx, sy;
   int xend, yend;
   int xmax, ymax;

   xmax = xs + dx;
   ymax = ys + dy;
   xend = xmax -1;
   yend = ymax -1;

   /*** Check validity of min/max coordinates. ***/

   if (xs < 0 || xmax > dx || ys < 0 || ymax > dy)
      {
      fprintf (stderr,
               "mr_box> invalid coordinates: xs:%d  ys:%d  xmax:%d  ymax:%d\n",
               xs, ys, xmax, ymax);
      return (-1);
      }


   /*** Horizontally-scanned image.***/

   if (dir2 [0] == 'l' || dir2 [0] == 'r')
      {
      if (dir2 [0] == 'r' && dir2 [1] == 'u')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + ix + iy*xdim) = *(cpbox + sx + sy*dx);
               }
            }
         }
      else
      if (dir2 [0] == 'l' && dir2 [1] == 'u')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + (xend-ix) + iy*xdim) = *(cpbox + sx + sy*dx);
               }
            }
         }
      else
      if (dir2 [0] == 'r' && dir2 [1] == 'd')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + ix + (yend-iy)*xdim) = *(cpbox + sx + sy*dx);
               }
            }
         }
      else
      if (dir2 [0] == 'l' && dir2 [1] == 'd')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + (xend-ix) + (yend-iy)*xdim) = *(cpbox + sx + sy*dx);
               }
            }
         }
      }

   /*** Vertically-scanned image. ***/

   else
   if (dir2 [0] == 'u' || dir2 [0] == 'd')
      {
      if (dir2 [0] == 'u' && dir2 [1] == 'r')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + ix + iy*xdim) = *(cpbox + sy + sx*dy);
               }
            }
         }
      else
      if (dir2 [0] == 'd' && dir2 [1] == 'r')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + ix + (xend-iy)*xdim) = *(cpbox + sy + sx*dy);
               }
            }
         }
      else
      if (dir2 [0] == 'u' && dir2 [1] == 'l')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + (xend-ix) + iy*xdim) = *(cpbox + sy + sx*dy);
               }
            }
         }
      else
      if (dir2 [0] == 'd' && dir2 [1] == 'l')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;

            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpdat + (xend-ix) + (xend-iy)*xdim) = *(cpbox + sy + sx*dy);
               }
            }
         }
      }

   else
      {
      fprintf (stderr, "mr_box> invalid dir2: %s\n", dir2);
      return (-2);
      }

   return (0);
}
