/* avimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average SMM C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 6 August 1984
 * 1 October 1987: Allow different averaging values in x & y dimensions.
 * 11 April 1988: Remove Grinnell code for Sun-4/280.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <sys/types.h>
#include <math.h>

/* Global variables. */

#include "/usr/people/smm/lib/include/cplib.h"
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)
int argc;
char *argv [];

{

extern u_char	*rdwimg ();	/* Read image from disk to memory. */
extern int	wrimg ();	/* Write image from memory to disk. */
u_char	*filbuf;		/* Pointer to file array. */
u_char	*imgbuf;		/* Pointer to image array. */
u_char	*imgptr;		/* Pointer to start of a line within image. */
static u_char	ybuf [1792];	/* Y-axis buffer. */
static char	namimg [80];	/* Input image file name. */
static char	namave [80];	/* Output image file name. */
static char	answer [2];	/* Storage for y/n response. */

short	*ys;		/* Short integer pointer to ybuf. */
short	*is;		/* Short integer pointer to image. */
int	ix, iy;		/* horizontal & vertical indices. */
int	length;		/* # characters in namimg. */
int	navex, navey;	/* # points in average for x & y dimensions. */
int	ybeg, yend;	/* begin & end indices for vertical axis. */
int	ny;		/* # vertical pixels in buffer. */

   if (argc == 4)
      {
      strcpy (namimg, argv [1]);
      sscanf (argv [2], "%d", &navex);
      sscanf (argv [3], "%d", &navey);
      }
   else
      {
      fprintf (stderr, "usage: %s cp_filename x_pixel_ave y_pixel_ave\n",
               argv [0]);
      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);

      /*** Request # points to average in each dimension. ***/

      fprintf (stderr, "Enter number of points in average (x,y): ");
      fscanf  (stdin,  "%d,%d", &navex, &navey);
      }

   length = strlen (namimg);
/*
 *   strcpy (namave, namimg);
 *   namave [length] = '.';
 *   namave [length+1] = 'a';
 *   namave [length+2] = 'v';
 *   namave [length+3] = '\0';
 */
   sprintf (namave, "%s.x%dy%d", namimg, navex, navey);

   /*** Open input image disk file and read housekeeping record. ***/

   filbuf = rdwimg (namimg);
   imgbuf = filbuf + npix * nbypix * 2;
   hkline (stdout);

/*
 *   fprintf (stderr, "ih,iv: %d %d\n", H.ih, H.iv);
 *   fprintf (stderr, "ip,ir,lp,lr: %d %d %d %d\n", ip,ir,lp,lr);
 *   fprintf (stderr, "npix,nbypix,idtype: %d %d %d\n", npix, nbypix, idtype);
 *   fprintf (stderr, "xcen,ycen: %f %f\n", xcen, ycen);
 */

   /*** X-axis averaging : ***/

   for (iy = 0; iy < npix; iy++)
      {
      imgptr = imgbuf + iy*npix*nbypix;
      avelin (navex, imgptr, npix);
      }

   /*** Y-axis averaging : ***/

   ybeg = 0;
   yend = npix - 1;
   ny = yend - ybeg + 1;

   if (nbypix == 1)			/* 1-byte/pixel. */
      {
      for (ix = 0; ix < npix; ix++)
         {
         for (iy = ybeg; iy <= yend; iy++)
            *(ybuf + iy) = *(imgbuf + iy*npix + ix);

         avelin (navey, &ybuf [ybeg], ny);

         for (iy = ybeg; iy <= yend; iy++)
            *(imgbuf + iy*npix + ix) = *(ybuf + iy);
         }
      }

   else
   if (nbypix == 2)			/* 2-bytes/pixel. */
      {
      ys = (short *) ybuf;
      is = (short *) imgbuf;
      for (ix = 0; ix < npix; ix++)
         {
         for (iy = ybeg; iy <= yend; iy++)  *(ys + iy) = *(is + iy*npix + ix);

         avelin (navey, &ybuf [ybeg], ny);

         for (iy = ybeg; iy <= yend; iy++)  *(is + iy*npix + ix) = *(ys + iy);
         }
      }

   wrimg (namave, filbuf);

   /*** Relinquish storage for image array. ***/

   free (filbuf);
}
