/* perint.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Interpolation over a scan line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 December 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

perint (cbuf, nbypx, nx, ny, xbuf, ybuf, nbuf, nint, dirint)

unsigned char *cbuf;		/* image buffer. */
int nbypx;			/* # bytes per element in lbuf. */
int nx, ny;			/* dimensions of image buffer. */
int xbuf [];			/* x location array. */
int ybuf [];			/* y location array. */
int nbuf;			/* # elements in lbuf. */
int nint;			/* # pixels to interpolate. */
char dirint [];			/* interpolation direction [h,v]. */

{
   extern char  cpixave ();
   extern short spixave ();
   extern float fpixave ();
   int is;
   int ix, iy;
   int nint2;
   int xmin, xmax;
   int ymin, ymax;

   short *sbuf;
   float *fbuf;

   float frange;
   float pv;
   float pvdif, pvmin, pvmax;

   nint2 = nint / 2;

   frange = (float) ( 2 * nint2 + 1 );

   if (nbypx == 1)
      {
      if (dirint [0] == 'h')
	 {
	 for (is = 0; is < ns; is++)
	    {
	    xmin = xbuf [is - nint2] ;
	    xmax = xbuf [is + nint2] ;
	    ymin = ybuf [is] ;
	    ymax = ybuf [is] ;
            pvmin = (float) cpixave (cbuf, nx, ny, xmin, ymin) ;
            pvmax = (float) cpixave (cbuf, nx, ny, xmax, ymax) ;
            pvdif = (float) ( pvmax - pvmin );

            for (ix = xmin + 1; ix < xmax; i++)
	       {
	       pv = (unsigned char)
		    ( ((float) (ix - xmin) / frange) * pvdif + pvmin );
	       *(cbuf + xbuf [ix] + ybuf [is] * nx) = (char) pv;
	       }
	    }
	 }
      else
      if (dirint [0] == 'v')
	 {
	 for (is = 0; is < ns; is++)
	    {
	    xmin = xbuf [is] ;
	    xmax = xbuf [is] ;
	    ymin = ybuf [is - nint2] ;
	    ymax = ybuf [is + nint2] ;
            pvmin = (float) cpixave (cbuf, nx, ny, xmin, ymin) ;
            pvmax = (float) cpixave (cbuf, nx, ny, xmax, ymax) ;
            pvdif = (float) ( pvmax - pvmin );

            for (iy = ymin + 1; iy < ymax; iy++)
	       {
	       pv = (unsigned char)
		    ( ((float) (iy - ymin) / frange) * pvdif + pvmin );
	       *(cbuf + xbuf [is] + ybuf [iy] * nx) = (char) pv;
	       }
	    }
	 }
      }

   else
   if (nbypx == 2)
      {
      sbuf = (short *) cbuf;
      if (dirint [0] == 'h')
	 {
	 for (is = 0; is < ns; is++)
	    {
	    xmin = xbuf [is - nint2] ;
	    xmax = xbuf [is + nint2] ;
	    ymin = ybuf [is] ;
	    ymax = ybuf [is] ;
            pvmin = (float) cpixave (sbuf, nx, ny, xmin, ymin) ;
            pvmax = (float) cpixave (sbuf, nx, ny, xmax, ymax) ;
            pvdif = (float) ( pvmax - pvmin );

            for (ix = xmin + 1; ix < xmax; i++)
	       {
	       pv = (unsigned char)
		    ( ((float) (ix - xmin) / frange) * pvdif + pvmin );
	       *(sbuf + xbuf [ix] + ybuf [is] * nx) = (char) pv;
	       }
	    }
	 }
      else
      if (dirint [0] == 'v')
	 {
	 for (is = 0; is < ns; is++)
	    {
	    xmin = xbuf [is] ;
	    xmax = xbuf [is] ;
	    ymin = ybuf [is - nint2] ;
	    ymax = ybuf [is + nint2] ;
            pvmin = (float) cpixave (sbuf, nx, ny, xmin, ymin) ;
            pvmax = (float) cpixave (sbuf, nx, ny, xmax, ymax) ;
            pvdif = (float) ( pvmax - pvmin );

            for (iy = ymin + 1; iy < ymax; iy++)
	       {
	       pv = (unsigned char)
		    ( ((float) (iy - ymin) / frange) * pvdif + pvmin );
	       *(sbuf + xbuf [is] + ybuf [iy] * nx) = (char) pv;
	       }
	    }
	 }
      }
   else
   if (nbypx == 4)
      {
      fbuf = (float *) cbuf;
      if (dirint [0] == 'h')
	 {
	 for (is = 0; is < ns; is++)
	    {
	    xmin = xbuf [is - nint2] ;
	    xmax = xbuf [is + nint2] ;
	    ymin = ybuf [is] ;
	    ymax = ybuf [is] ;
            pvmin = (float) fpixave (fbuf, nx, ny, xmin, ymin) ;
            pvmax = (float) fpixave (fbuf, nx, ny, xmax, ymax) ;
            pvdif = (float) ( pvmax - pvmin );

            for (ix = xmin + 1; ix < xmax; i++)
	       {
	       pv = (unsigned char)
		    ( ((float) (ix - xmin) / frange) * pvdif + pvmin );
	       *(fbuf + xbuf [ix] + ybuf [is] * nx) = (char) pv;
	       }
	    }
	 }
      else
      if (dirint [0] == 'v')
	 {
	 for (is = 0; is < ns; is++)
	    {
	    xmin = xbuf [is] ;
	    xmax = xbuf [is] ;
	    ymin = ybuf [is - nint2] ;
	    ymax = ybuf [is + nint2] ;
            pvmin = (float) fpixave (fbuf, nx, ny, xmin, ymin) ;
            pvmax = (float) fpixave (fbuf, nx, ny, xmax, ymax) ;
            pvdif = (float) ( pvmax - pvmin );

            for (iy = ymin + 1; iy < ymax; iy++)
	       {
	       pv = (unsigned char)
		    ( ((float) (iy - ymin) / frange) * pvdif + pvmin );
	       *(fbuf + xbuf [is] + ybuf [iy] * nx) = (char) pv;
	       }
	    }
	 }
      }

   return (0);
}
