/* rdfimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array,
 * INCLUDING housekeeping and bitmap records.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   Solar Maximum Mission   Coronagraph/Polarimeter
 * 21 August 1985
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char	*
rdfimg (namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
   register unsigned char *cpbuf;	/* Pointer to image array. */
   register int  cpfd;		/* File descriptor for image disk file. */
   register int  imgsiz;	/* Image size (bytes) (pixel data only). */
   register long nread;
   int nrechd;

   /*** Decommutate Housekeeping. ***/

   cphskp (namimg);	

   if (npixl == 0 || npixl > 896)  npixl = npix ;

   /*** Open image file. ***/

   if ((cpfd = open (namimg, 0)) < 0)
     {
     fprintf (stderr,"rdfimg> Can't open file %s\n", namimg);
     return (NULL);
     }

   /*** Allocate storage in memory for image array. ***/

/*
 *   fprintf (stdout, "rdfimg> npix: %d   npixl: %d   nbypix: %d\n",
 *	    npix, npixl, nbypix);
 */
   if (nbypix == 4)
      nrechd = 1;
   else
      nrechd = 2;
   imgsiz = (npixl * nbypix * (npixl + nrechd));

   if ((cpbuf = (unsigned char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr,"rdfimg> malloc error.");
     exit (1);
     }

   if ((nread = read (cpfd, cpbuf, imgsiz)) <= 0)
      {
      fprintf (stderr,"rdfimg> image read error.\n");
      return (NULL);
      }

   /*** Reverse bytes for Vax-calibrated images. ***/

   if (idtype == 3 && bytord == 1)  srevbyt (cpbuf, nbypix, npixl*npixl);

   if (close (cpfd) == -1)
      {
      fprintf (stderr,"rdfimg> file close error.%s\n", namimg);
      return (NULL);
      }

  return (cpbuf);
}
