/* rdwimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 25 April 1984
 * 7 February 1990: Set hskp byte 52 to indicate Sun-4 byte order.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <sys/types.h>
#include "/usr/people/smm/lib/include/cplib.ext"

char	*malloc();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

u_char	*
rdwimg (namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
   register   u_char *imgbuf;	/* Pointer to image array. */
   register   int imgfd;	/* File descriptor for image disk file. */
   register   int imgsiz;	/* Image size (bytes) (excludes hskp, bitmap).*/
   register   long nread;

   /*** Decommutate Housekeeping. ***/

   cphskp (namimg);	

   /*   printf("rdimg> npix: %d nbypix: %d\n", npix, nbypix); */

   /*** Open image file. ***/

   if ((imgfd = open (namimg, 0)) < 0)
      {
      fprintf (stderr, "rdwimg> Unable to open file : %s\n", namimg);
      return (NULL);
      }

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

   /*
    *   if (imgfd <= 0)
    *      { fprintf (stderr, "rdwimg> seek error.\n");  return (NULL); }
    */

   lseek (imgfd, 0L, 0);

   imgsiz = ((npix+2) * npix * nbypix);
   if ((imgbuf = (u_char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr, "rdwimg> malloc error.");
     exit (1);
     }

   if ((nread = read (imgfd, imgbuf, imgsiz)) <= 0)
      {
      fprintf (stderr, "rdwimg> image read error.\n");
      return (NULL);
      }

   /*** Set byte order in housekeeping to Sun-4 convention. ***/

   *(imgbuf + 52) = 0;

   if (nbypix == 2 && bytord == 1)
      {
      imgsiz = (npix * npix);
      srevbyt ((imgbuf + npix * nbypix * 2), nbypix, imgsiz); 
      }

/*   printf("rdimg> nread: %d\n", nread); */

   if (close (imgfd) == -1)
      {
      fprintf (stderr, "rdwimg> file close error : %s\n", namimg);
      return (NULL);
      }

  return (imgbuf);
}
