/* svfill.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Vertical interpolation: fill dns < level with interpolated value.
 * Case level >   0: intepolate over pixel if magnitude < level.
 * Case level == -1: compute average of end points for interpolation.
 * Case level <  -1: scan line in sections of length `-level'.  If midpoint
 *                   pixel << endpoints, interpolate across that section.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <sys/types.h>

svfill (imgs, dx, dy, level)

u_short *imgs;
int dx, dy, level;
{
   register u_short *ladd, *radd, *rradd, *xadd;
   int y, wid, mid;
   float factor, midint, minint;
   int n, left, right;
   float fright, fleft;

   /*** Line loop. ***/

   for ( y = 0; y < dy; y++)
      {
      ladd = imgs + y*dx;
      rradd = ladd + dx;

      /*** Interpolate over all pixels with magnitude less than
       *** or equal to `level'.
       ***/

      if ( level >= 0 )
         {
         while ( (ladd < rradd) && (*ladd < level) ) *ladd++ = level;
         if ( ladd < rradd )
            {
            do
               {
               while ( (ladd < rradd) && (*ladd >= level) ) ladd++;
               if ( ladd >= rradd ) break;
               radd = ladd--;
               while ( (radd < rradd) && (*radd < level) ) radd++;
               if ( radd >= rradd )
                  {
                  ladd++;
                  while ( ladd < rradd ) *ladd++ = level;
                  break;
                  }
               factor = (float) (*radd-*ladd) / (float) (radd-ladd);
               for ( xadd = ladd+1; xadd < radd; xadd++ )
               *xadd = *ladd + (u_short) (factor * (float) (xadd-ladd));
               ladd = radd;
               } while ( ladd < rradd );
            } /*** end-if ***/
         } /*** end-if(level>=0) ***/

      /*** Interpolate between end points. ***/

      if ( level == -1 )
         {
         radd = rradd-1;
         n = 0;  left = 0;  right = 0;

         /*** Use an average end point. ***/

         if ( y>0 )
            {
            left += *(ladd-dx) + *(ladd-dx+1);
            right += *(radd-dx) + *(radd-dx-1);
            n += 2;
            }
         left += *(ladd) + *(ladd+1);
         right += *(radd) + *(radd-1);
         n += 2;
         if ( y<(dy-1) )
            {
            left += *(ladd+dx) + *(ladd+dx+1);
            right += *(radd+dx) + *(radd+dx-1);
            n += 2;
            }
         ladd++; radd--;
         fleft  = (float) (left) / (float) (n);
         fright = (float) (right) / (float) (n);
         factor = (float) (fright-fleft) / (float) (radd-ladd);
         for ( xadd = ladd+1; xadd < radd; xadd++ )
         *xadd = (u_short) (fleft + factor * (float) (xadd-ladd));
         }  /*** end-if (level == -1) ***/

      /*** Pixels with magnitude < -1.
       *** Scan line in sections of width `-level'.
       *** If midpoint of a section is << than endpoints,  interpolate over
       *** that section.
       ***/

      if ( level < (-1) )
         {
         wid = -level;
         mid = wid/2;
         radd = ladd + wid;
         while ( radd<rradd )
            {
            midint = (float) ( *(ladd+mid) );
            if ( *ladd < *radd ) minint = (float) (*ladd);
            else minint = (float) (*radd);
            if ( midint <  0.75 * minint )
               {
               factor = (float) (*radd-*ladd) / (float) (radd-ladd);
               for (xadd = ladd+1;  xadd<radd;  xadd++)
               *xadd = *ladd + (u_short) (factor * (float) (xadd-ladd));
               ladd = radd;
               radd = ladd + wid;
               }
            else
               {
               ladd++;  radd++;
               }
            } /*** end-while ***/
         }  /*** end-if (level < -1) ***/
      }  /*** end-for ***/
   return (0);
}
