/* transpose.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Transpose (interchange columns & rows) a 2-dimensional array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 16 July 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

transpose (imbuf, tpbuf, nbypx, xsize, ysize);

unsigned char *imbuf;		/* Input image buffer. */
unsigned char *tpbuf;		/* Transposed image buffer. */
int nbypx;			/* # bytes per pixel. */
int xsize;			/* X-dimension. */
int ysize;			/* Y-dimension. */
{
   int ix, iy;
   unsigned char *cbuf, ctbuf;
   unsigned short *sbuf, stbuf;
   float *fbuf, ftbuf;

   if (nbypx == 1)
      {
      cbuf = (unsigned char *) imbuf;
      ctbuf = (unsigned char *) tpbuf;
      for (iy = 0; iy < ysize;  iy++)
         for (ix = 0; ix < xsize;  ix++)
            *(ctbuf + iy + ix * dy) = *(cbuf + ix + iy * dx);
      }

   else
   if (nbypx == 2)
      {
      sbuf = (unsigned short *) imbuf;
      stbuf = (unsigned short *) tpbuf;
      for (iy = 0; iy < ysize;  iy++)
         for (ix = 0; ix < xsize;  ix++)
            *(ctbuf + iy + ix * dy) = *(cbuf + ix + iy * dx);
      }

   else
   if (nbypx == 4)
      {
      fbuf = (unsigned float *) imbuf;
      ftbuf = (unsigned float *) tpbuf;
      for (iy = 0; iy < ysize;  iy++)
         for (ix = 0; ix < xsize;  ix++)
            *(ctbuf + iy + ix * dy) = *(cbuf + ix + iy * dx);
      }

   else
      fprintf (stdout, "transpose> Invalid image type; nbypx != 1, 2, or 4.\n");
      
   return (0);
}
