/* wboxb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write byte image array from memory to SGI IRIS frame buffer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 March 1991: Currently only supports dir2 = "rd" or "ur".
 * 15 July 1991: Add FB argument.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <whip.h>

wboxb (FB, imgbuf, xs, ys, xdim, ydim, dir2)

int FB;				/* Byte frame buffer (colormap). */
char *imgbuf;			/* Image memory buffer. */
int xs, ys;			/* Coordinates for image origin in FB. */
int xdim, ydim;			/* X & Y dimensions of image. */
char *dir2;			/* Direction to write image: l/r, u/d. */
{
   extern char *malloc ();	/* Memory allocation function. */
   pichdr imghdr;		/* WHIP image header. */
   char xdir, ydir;
   char namimg [80];		/* Image file name. */
   int col;			/* Column index counter. */
   int i;			/* Loop index. */
   int mode = 0;		/* Read only. */
   int nread;
   int rel = 0;			/* Relative flag for fb_setaddr. */
   int row;			/* Row index counter. */
   int xpos, ypos;
   int fast = 1;		/* Parameter for fb_clear. */
   int z = 0;
   float grey = 0.0;		/* Grey level for fb_clear. */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   xdir = 'x';
   ydir = 'y';
   for (i=0; i<2; i++)
      switch (*(dir2 + i))
         {
         case 'l':
         case 'r': xdir = *(dir2 + i); break;
         case 'u':
         case 'd': ydir = *(dir2 + i); break;
         }
   if (xdir == 'x' || ydir == 'y')
      {
      fprintf (stderr, "wboxb> Illegal direction (%s).\n", dir2); 
      return (-1);
      }
   if (strcmp (dir2, "rd") != 0 && strcmp (dir2, "ur") != 0)
      {
      fprintf (stderr, "wboxb> Unsupported direction (%s).\n", dir2);
      return (-2);
      }

   xpos = xs;
   if (xdir == 'r')	/* Horizontally-scanned image. */
      {
      fb_xstep ();			/* Auto increment in x axis. */
      if (ydir == 'd')
         {
         for (row = 0; row < ydim; row++)
            {
            ypos = ys - row;
            fb_setaddr (FB, rel, xpos, ypos, z);	/* Set address in FB. */
            fb_write (imgbuf + row * xdim, xdim);	/* Write row to FB.   */
            }
         }
      else
      if (ydir == 'u')
         {
         for (row = 0; row < ydim; row++)
            {
            ypos = ys + row;
            printf ("xpos: %d   ypos: %d   dir2: %s\n", xpos, ypos, dir2);
            fb_setaddr (FB, rel, xpos, ypos, z);	/* Set address in FB. */
            fb_write (imgbuf + row * xdim, xdim);	/* Write row to FB.   */
            }
         }
      }

   else					/* Vertically-scanned image. */
      {
      ypos = ys;
      fb_ystep ();			/* Auto increment in y axis. */
      for (col = 0; col < xdim; col++)
         {
         xpos = xs + col;
         fb_setaddr (FB, rel, xpos, ypos, z);	/* Set address in FB. */
         fb_write (imgbuf + col, ydim);		/* Write column to FB.*/
         }
      }
   return (0);
}
