/* rvfill.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rectangular vertical fill.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

rvfill (ibuf, nbypx, xdim, ydim, xs, ys, dxs, dys)

unsigned char  *ibuf;		/* Image array (2D). */
int		nbypx;		/* # bytes per pixel. */
int		xdim, ydim;	/* Image size. */
int		xs, ys;		/* Minimum x,y coordinates. */
int		dxs, dys;	/* Size of rectangle. */

{
   unsigned char	*cbuf;
   unsigned short	*sbuf;
   float		*fbuf;

   int    ih1, ih2;
   int    iv1, iv2;
   int    ix, iy;
   int    ival;
   int    ly, hy;
   float  vallo, valhi;
   float  frac;

   cbuf = (unsigned char *)  ibuf;
   sbuf = (unsigned short *) ibuf;
   fbuf = (float *)          ibuf;

   ih1 = xs;
   ih2 = xs + dxs;
   iv1 = ys;
   iv2 = ys + dys;

   ly = ((iv1 > 0) ? iv1 - 1 : 0);			/* Set min y coord. */
   hy = ((iv2 < xdim - 1) ? iv2 + 1 : xdim - 1);	/* Set max y coord. */

   /*** Interpolation Loop. ***/

   if (nbypx == 1)
   for (ix = ih1; ix < ih2; ix++)
      {
      for (iy = iv1; iy < iv2; iy++)
         {
         vallo = (float) (*(cbuf + ix + ly * xdim) & 0377);
         valhi = (float) (*(cbuf + ix + hy * xdim) & 0377);

         frac = (float) (iy - iv1) / (float) (iv2 - iv1);

         ival = (int) (frac * (valhi - vallo) + vallo);

         *(cbuf + ix + iy * xdim) = (unsigned char) (ival & 0377);
         }
      }

   else
   if (nbypx == 2)
   for (ix = ih1; ix < ih2; ix++)
      {
      for (iy = iv1; iy < iv2; iy++)
         {
         vallo = (float) (*(sbuf + ix + ly * xdim) & 0177777);
         valhi = (float) (*(sbuf + ix + hy * xdim) & 0177777);

         frac = (float) (iy - iv1) / (float) (iv2 - iv1);

         ival = (int) (frac * (valhi - vallo) + vallo);

         *(sbuf + ix + iy * xdim) = (unsigned short) (ival & 0177777);
         }
      }

   else
   if (nbypx == 4)
   for (ix = ih1; ix < ih2; ix++)
      {
      for (iy = iv1; iy < iv2; iy++)
         {
         vallo = *(fbuf + ix + ly * xdim);
         valhi = *(fbuf + ix + hy * xdim);

         frac = (float) (iy - iv1) / (float) (iv2 - iv1);

         *(fbuf + ix + iy * xdim) = (frac * (valhi - vallo) + vallo);
         }
      }

   return (0);
}
