/* wbox.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Move SMM C/P image from disc file to SGI IRIS frame buffer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 March 1991: Currently only supports dir2 = "rd" or "ur".
 * 6 November 1991: Add FB argument.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <whip.h>

wbox (FB, xsize, ysize, imgbuf, nbypix, xdim, ydim, dir2)

int FB;				/* Frame buffer. */
int xsize, ysize;		/* FB size. */
char *imgbuf;			/* Image memory buffer. */
int nbypix;			/* # bytes per pixel. */
int xdim, ydim;			/* X & Y dimensions of image. */
char *dir2;			/* Direction to write image: l/r, u/d. */
{
   extern char *malloc ();	/* Memory allocation function. */
   pichdr imghdr;		/* WHIP image header. */
   static char namimg [80];	/* Image file name. */
   char xdir, ydir;
   int col;			/* Column index counter. */
   int fast = 1;		/* Parameter for fb_clear. */
   int i;			/* Loop index. */
   int mode = 0;		/* Read only. */
   int nread;
   int rel = 0;			/* Relative flag for fb_setaddr. */
   int row;			/* Row index counter. */
   int xstart;			/* X location for writing. */
   int ystart;			/* Y location for writing. */
   int z = 0;
   float grey = 0.0;		/* Grey level for fb_clear. */
   FILE *IMG;			/* File descriptor for image file. */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   xstart = xsize - xdim - 1;
   xdir = 'x';
   ydir = 'y';
   for (i=0; i<2; i++)
      switch (*(dir2 + i))
         {
         case 'l':
         case 'r': xdir = *(dir2 + i); break;
         case 'u':
         case 'd': ydir = *(dir2 + i); break;
         }
   if (xdir == 'x' || ydir == 'y')
      {
      fprintf (stderr, "wbox> Illegal direction (%s).\n", dir2); 
      return (-1);
      }
   if (strcmp (dir2, "rd") != 0 && strcmp (dir2, "ur") != 0)
      {
      fprintf (stderr, "wbox> Unsupported direction (%s).\n", dir2);
      return (-2);
      }

   if (xdir == 'l' || xdir == 'r')	/* Horizontally-scanned image. */
      {
      fb_xstep ();			/* Auto increment in x axis. */
      for (row = 0; row < xdim; row++)
         {
         ystart = ysize - ydim - 1;
         fb_setaddr (FB, rel, xstart, ystart, z);	/* Set address in FB. */
         fb_write (imgbuf + row * xdim, xdim);		/* Write row to FB.   */
         }
      }

   else					/* Vertically-scanned image. */
      {
      fb_ystep ();			/* Auto increment in y axis. */
      for (col=0; col<xdim; col++)
         {
         ystart = ysize - ydim - 1;
         fb_setaddr (FB, rel, xstart, ystart, z);	/* Set address in FB. */
         fb_write (imgbuf + col, xdim);			/* Write column to FB.*/
         }
      }
   fclose (IMG);
   fb_close ();					/* Terminate WHIP software. */
}
