/* bprms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bright point removal.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

#define NBOX 10 
#define NSEARCH 16 
#define NINT 12

bprms (cbuf, nbypx, xdim, ydim)

unsigned char *cbuf;		/* Image buffer. */
int nbypx;			/* # bytes per pixel. */
int xdim;			/* X-dimension. */
int ydim;			/* Y-dimension. */
{
   static char dir2 [] = "ru";
   unsigned char pixmax;

   int bpx, bpy;
   int dxx = 25;
   int dyy = 25;
   int ibox;
   int ix, iy;
   int level = -1;
   int x, y;
   int xmin = 0, ymin = 0;
   int xmax = 0, ymax = 0;
   int xp, yp;
   int xul = 22;
   int yul = 140;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;
   static int xb [NBOX] = {208, 226, 215, 253, 250, 371, 494, 497, 325, 414};
   static int yb [NBOX] = {504, 504, 493, 494, 418, 340, 303, 228, 159, 130};
   static int xbrt [NBOX];
   static int ybrt [NBOX];

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   if (nbypx != 1)
      {
      fprintf (stderr, "bprms> Invalid image type. Must be byte image.\n");
      return (-1);
      }

   /*** Bright Point Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {

      /*** Read box centered about estimated location of bright point. ***/

      xmin = xb [ibox] - 63 - dxs/2;
      ymin = 510 - yb [ibox] - dys/2;
      if (xmin < 0)  xmin = 0;
      if (ymin < 0)  ymin = 0;
      xmax = xmin + dxs;
      ymax = ymin + dys;
      printf ("bprms> xmin: %3d   ymin: %3d\n", xmin, ymin);

      /*** Initial estimate of bright point location and intensity. ***/

      bpx = xmin;
      bpy = ymin;
      pixmax = 0;

      /*** Search each box for brightest pixel. ***/

      for (iy = ymin; iy < ymax; iy++)
         for (ix = xmin; ix < xmax; ix++)
            {
            if (*(cbuf + ix + iy*xdim) > pixmax)
               {
               pixmax = *(cbuf + ix + iy*xdim);
               bpx = ix;
               bpy = iy;
               }
            }

      /* Redefine lower left corner of box so that it is centered about the
       * brightest pixel found.
       */

      xbrt [ibox] = bpx - dx/2;
      ybrt [ibox] = bpy - dy/2;

      if (xbrt [ibox] < 0)  xbrt [ibox] = 0;
      if (ybrt [ibox] < 0)  ybrt [ibox] = 0;
      if (xbrt [ibox] + dx > xdim)  xbrt [ibox] = xdim - dx;
      if (ybrt [ibox] + dy > ydim)  ybrt [ibox] = ydim - dy;

      rhfill (cbuf, nbypx, xdim, ydim, xbrt [ibox], ybrt [ibox], dx, dy);
      }

   /*** Vertical interpolation for upper left black/white spot. ***/

   rvfill (cbuf, nbypx, xdim, ydim, xul, yul, dxx, dyy);

   return (0);
}
