/* circle_int.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Line interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  8 December 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

circle_int (cbuf, nx, ny, xbuf, ybuf, nbuf, nbypx)

unsigned char *cbuf;		/* image buffer. */
int nx, ny;			/* dimensions of image buffer. */
int xbuf [];			/* x location array. */
int ybuf [];			/* y location array. */
int nbuf;			/* # elements in lbuf. */
int nbypx;			/* # bytes per element in lbuf. */

{
   extern char  cpixave ();
   extern short spixave ();
   extern float fpixave ();
   int i;
   int imin;
   int imax;

   short *sbuf;
   float *fbuf;

   float frange;
   float pvdif, pvmin, pvmax;

   imin = 0;
   imax = nbuf - 1;

   /*** character image: 1 byte/pixel. ***/

   if (nbypx == 1)	
      {
      frange = (float) ( nbuf );

      pvmin = (float) cpixave (cbuf, nx, ny, xbuf [imin], ybuf [imin]);
      pvmax = (float) cpixave (cbuf, nx, ny, xbuf [imax], ybuf [imax]);
      pvdif = (float) ( pvmax - pvmin );

      for (i = 1; i < imax; i++)
	 {
	 *(cbuf + xbuf [i] + (ybuf [i]) * nx) =
	    (char) ( ((float) (i - imin) / frange) * pvdif + pvmin );
	 }
      }

   /*** short integer image: 2 bytes/pixel. ***/

   else
   if (nbypx == 2)
      {
      sbuf = (short *) cbuf;

      frange = (float) ( nbuf );

      pvmin = (float) spixave (sbuf, nx, ny, xbuf [imin], ybuf [imin]);
      pvmax = (float) spixave (sbuf, nx, ny, xbuf [imax], ybuf [imax]);
      pvdif = (float) ( pvmax - pvmin );

      for (i = 1; i < imax - 1; i++)
	 {
	 *(sbuf + xbuf [i] + ybuf [i] * nx) =
	    (short) ( ( (float) (i - imin) / frange ) * pvdif + pvmin );
	 }
      }

   /*** floating point image: 4 bytes/pixel. ***/

   else
   if (nbypx == 4)
      {
      fbuf = (float *) cbuf;

      frange = (float) ( nbuf );

      pvmin = (float) fpixave (fbuf, nx, ny, xbuf [imin], ybuf [imin]);
      pvmax = (float) fpixave (fbuf, nx, ny, xbuf [imax], ybuf [imax]);
      pvdif = pvmax - pvmin;

      for (i = 1; i < imax - 1; i++)
	 {
	 *(fbuf + xbuf [i] + ybuf [i] * nx) =
	    ( (float) (i - imin) / frange ) * pvdif + pvmin;
	 }
      }

   return (0);
}
