/* clean.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Interpolate over reseau dots and bright points in a sequence of C/P images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 23 July 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/img_scale.h"
int DSP = 1;

main (argc, argv)

int argc;
char *argv [];

{
   extern char *malloc ();
   char *rdfimg ();
   FILE *LIST;

   static int xbrt [1] = 100;
   static int ybrt [1] = 200;
   static int dxb = 50;
   static int dyb = 50;

   static char namimg [80];	/* YYDDDHHMM.vXX */
   static char namout [80];	/* YYDDDHHMM.gXX */
   static char namlst [80];	/* Namelist file. */
   unsigned short *sbuf;	/* Short integer array pointer. */
   char *ibuf;			/* Byte image pointer (includes hk + bitmap). */
   char *cbuf;			/* Byte image pointer (pixel data only). */

   int fd = 0;
   int i;
   int idot = -1;
   int linit = 1;
   int maxpix;
   int maxlin;
   int nbyimg;
   int npximg;
   int seq = 0;			/* Flag to indicate sequence of images. */
   int xll, yll;		/* GL window: lower left coordinates. */
   int xur, yur;		/* GL window: upper right coordinates. */

   if (argc == 2)
      strcpy (namimg, argv [1]);
   else
      {
      fprintf (stderr, "usage: %s cp_filename | @cp_filename_list\n", argv [0]);
      exit (0);
      }

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

loop:

   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }

   /*** Read image from disk. ***/

   ibuf = rdfimg (namimg);

   /*** Copy byte image into short integer array. ***/

   npximg = npix * npix;
   cbuf = ibuf + npix * 2;
   sbuf = (unsigned short *) malloc (npix * npix * sizeof (short));

   /*** Remove dark points (reseau dots). ***/

   dkrms (cbuf, nbypix, npix, npix);

   /*** Remove bright points (pin holes). ***/

   bprms (cbuf, nbypix, npix, npix);

   /*** Remove blemishses. ***/

   blrms (cbuf, nbypix, npix, npix);

   /*** Write interpolated image to disk. ***/

   idot = -1;
   strcpy (namout, namimg);

   for (i = 0; i < strlen (namimg); i++)
	if (namimg [i] == '.')  {idot = i;  break;}
   namout [idot+1] = 'g';
   if (idot < 0)
      {
      fprintf (stderr, "Invalid image name: %s\n", namimg);
      exit (-1);
      }

   nbyimg = npix * (npix + 2) * sizeof (char);
   fd = creat (namout, 0644);
   write (fd, ibuf, nbyimg);
   close (fd);
   free (ibuf);
   free (sbuf);

   if (seq)  goto loop;

quit:

   exit (0);
}
