/* dkrms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Dark point removal (reseau dots).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Search for darkest point within a box centered around
 * approximate location of dark point.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

#define NBOX 26 
#define NSEARCH 16
#define NINT 11

dkrms (cbuf, nbypx, xdim, ydim)

unsigned char *cbuf;		/* Image buffer. */
int nbypx;			/* # bytes per pixel. */
int xdim;			/* X-dimension. */
int ydim;			/* Y-dimension. */
{
   static char dir2 [] = "ru";
   unsigned char pixmin;

   int darkpx, darkpy;
   int ibox;
   int ix, iy;
   int level = -1;
   int x, y;
   int xdp, ydp;
   int xmin = 0, ymin = 0;
   int xmax = 448, ymax = 448;
   int xp, yp;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;

   static int xd [NBOX] = {112, 207, 306, 410, 108, 207, 306, 407, 508, 107,
			   208, 308, 407, 505, 105, 210, 310, 408, 502, 100,
			   209, 310, 407, 494, 422, 179};

   static int yd [NBOX] = {497, 491, 489, 487, 390, 387, 388, 389, 387, 285,
			   284, 286, 288, 286, 185, 183, 183, 184, 181, 92,
			    84,  81,  78,  72, 227, 146};

   static int xdll [NBOX];
   static int ydll [NBOX];
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   if (nbypx != 1)
      {
      fprintf (stderr, "dkrms> Invalid image type. Must be byte image.\n");
      return (-1);
      }

   /*** Reseau Dot Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {
      xmin = xd [ibox] - 63 - dxs/2;			/* Lower left corner of box. */
      ymin = 510 - yd [ibox] - dys/2;
      xmax = xmin + dxs;
      ymax = ymin + dys;

      if  (xmin < 0)  xmin = 0;
      if  (ymin < 0)  ymin = 0;
      if ((xmin + dxs) > xdim)  xmin = xdim - dxs;
      if ((ymin + dys) > ydim)  ymin = ydim - dys;

      printf ("dkrms> xmin: %3d   ymin: %3d\n", xmin, ymin);

      /*** Initial estimate for reseau dot location and intensity. ***/

      pixmin = 255;

      /*** Search each box for darkest pixel. ***/

      for (iy = ymin; iy < ymax; iy++)
         for (ix = xmin; ix < xmax; ix++)
            {
            if (*(cbuf + ix + iy*xdim) < pixmin)
               {
               pixmin = *(cbuf + ix + iy*xdim);
               xdp = ix;
               ydp = iy;
               }
           }

      /* Redefine lower left corner of box so that it is centered about the
       * darkest pixel found.
       */

	xdll [ibox] = xdp - dx/2;		/* Lower left corner of box. */
	ydll [ibox] = ydp - dy/2;

	if (xdll [ibox] < 0)  xdll [ibox] = 0;
	if (ydll [ibox] < 0)  ydll [ibox] = 0;
	if (xdll [ibox] + dx > xdim)  xdll [ibox] = xdim - dx;
	if (ydll [ibox] + dy > ydim)  ydll [ibox] = ydim - dy;

      /*** Perform horizontal interpolation for each box. ***/

      rvfill (cbuf, nbypx, xdim, ydim, xdll [ibox], ydll [ibox], dx, dy);

      }

   return (0);
}
