/* fibrms.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fiber removal (1989: upper right).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

#define NBOX 1 
#define NSEARCH 50
#define NINT 20 

fibrms (cbuf, nbypx, xdim, ydim)

unsigned char	*cbuf;		/* Image buffer. */
int		nbypx;		/* # bytes per pixel. */
int		xdim;		/* X-dimension. */
int		ydim;		/* Y-dimension. */
{
   static char dir2 [] = "ru";
   unsigned char level = 0;

   float pixsum = 0.0;
   int pixave = 0;
   int pixcnt = 0;

   int darkpx, darkpy;
   int dpx, dpy;
   int ibox;
   int ix, iy;
   int pixmin;
   int x, y;
   int xmax = 0, ymax = 0;
   int xmin = 0, ymin = 0;
   int xp, yp;

   static int nbox = NBOX;
   static int dx =  NINT;
   static int dy =  NINT;
   static int dxs = NSEARCH;
   static int dys = NSEARCH;

   static int xd [NBOX] = {277};	/* Lower left corner of box. */
   static int yd [NBOX] = {110};
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
   if (nbypx != 1)
      {
      fprintf (stderr, "fibrms> Invalid image type. Must be byte image.\n");
      return (-1);
      }

   /*** Fiber Loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {

      /* Minimum corner of box. */

      xmin = xd [ibox];
      ymin = yd [ibox];
      if (xmin < 0)  xmin = 0;
      if (ymin < 0)  ymin = 0;

      /* Maximum corner of box. */

      xmax = xmin + dxs;
      ymax = ymin + dys;
      if (xmax > xdim)  xmax = xdim - dxs;
      if (ymax > ydim)  ymax = ydim - dys;

      pixsum = 0.0;
      pixave = 0;
      pixcnt = 0;

      /*** Find average pixel intensity within box. ***/

      for (iy = ymin; iy < ymax; iy++)
         for (ix = xmin; ix < xmax; ix++)
            {
            pixsum += (float) *(cbuf + ix + iy*xdim);
            pixcnt++;
            }

      pixave = (int) (pixsum / (float) pixcnt);
      level = (unsigned char) ((int) (pixave * 0.95) & 0377);
      printf ("pixave: %d   pixcnt: %d   pixsum: %6.2f\n",
              pixave, pixcnt, pixsum);

      /*** Replace any pixel whose value is less than level with pixave. ***/

      for (iy = ymin; iy < ymax; iy++)
         for (ix = xmin; ix < xmax; ix++)
            {
            if (*(cbuf + ix + iy*xdim) < level)
               *(cbuf + ix + iy*xdim) = pixave;
            }
      }

   return (0);
}
