/* mr_box.c
 ****************************************************************************
 * Extract values within a box from a C/P image memory array.
 ****************************************************************************
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 17 July 1992
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */

#include <stdio.h>

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

mr_box (cpdat, xdim, ydim, cpbox, xs, ys, dx, dy, dir2)

unsigned short *cpdat;	/* Full image array. */
int xdim, ydim;		/* Size of full image. */
unsigned short *cpbox;	/* Subset image array. */
int xs, ys;		/* Box origin of subset image in full image. */
int dx, dy;		/* X/Y dimensions of subset image. */
char dir2 [4];		/* Read direction: l(eft)/r(ight),u(p)/d(own). */

{
   int ix, iy;
   int sx, sy;
   int xend, yend;
   int xmax, ymax;

   xmax = xs + dx;
   ymax = ys + dy;
   xend = xmax -1;
   yend = ymax -1;

/*
 *   fprintf (stdout,
 *      "mr_box> x/ydim:%4d %4d  x/ys:%4d %4d  dx/y:%4d %4d  x/ymax:%4d %4d\n",
 *       xdim, ydim, xs, ys, dx, dy, xmax, ymax);
 */

   /*** Check validity of min/max coordinates. ***/

   if (xs < 0 || xmax > xdim || ys < 0 || ymax > ydim)
      {
      fprintf (stderr,
               "mr_box> invalid coordinates: xs:%d  ys:%d  xmax:%d  ymax:%d\n",
               xs, ys, xmax, ymax);
      return (-1);
      }

   /*** Horizontally-scanned image.***/

   if (dir2 [0] == 'l' || dir2 [0] == 'r')
      {
      if (dir2 [0] == 'r' && dir2 [1] == 'u')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sx + sy*dx)  = *(cpdat + ix + iy*xdim);
/*
 *   printf ("mr_box> sx/y: %3d  %3d    ix/y: %3d  %3d   DN: %3d  %3d\n",
 *           sx, sy, ix, iy,
 *           *(cpbox + sx + sy*dx) - 512, *(cpdat + ix + iy*xdim) - 512);
 */
               }
            }
         }
      else
      if (dir2 [0] == 'l' && dir2 [1] == 'u')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sx + sy*dx)  = *(cpdat + (xend-ix) + iy*xdim);
               }
            }
         }
      else
      if (dir2 [0] == 'r' && dir2 [1] == 'd')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sx + sy*dx)  = *(cpdat + ix + (yend-iy)*xdim);
               }
            }
         }
      else
      if (dir2 [0] == 'l' && dir2 [1] == 'd')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sx + sy*dx)  = *(cpdat + (xend-ix) + (yend-iy)*xdim);
               }
            }
         }
      }

   /*** Vertically-scanned image. ***/

   else
   if (dir2 [0] == 'u' || dir2 [0] == 'd')
      {
      if (dir2 [0] == 'u' && dir2 [1] == 'r')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sy + sx*dy)  = *(cpdat + ix + iy*xdim);
               }
            }
         }
      else
      if (dir2 [0] == 'd' && dir2 [1] == 'r')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sy + sx*dy)  = *(cpdat + ix + (xend-iy)*xdim);
               }
            }
         }
      else
      if (dir2 [0] == 'u' && dir2 [1] == 'l')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sy + sx*dy)  = *(cpdat + (xend-ix) + iy*xdim);
               }
            }
         }
      else
      if (dir2 [0] == 'd' && dir2 [1] == 'l')
         {
         sy = -1;
         for (iy = ys; iy < ymax; iy++)
            {
            sy++;
            sx = -1;
            for (ix = xs; ix < xmax; ix++)
               {
               sx++;
               *(cpbox + sy + sx*dy)  = *(cpdat + (xend-ix) + (xend-iy)*xdim);
               }
            }
         }
      }

   else
      {
      fprintf (stderr, "mr_box> invalid dir2: %s\n", dir2);
      return (-2);
      }

   return (0);
}
