/* rhfill.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rectangular horizontal fill.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 10 February 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

rhfill (ibuf, nbypx, xdim, ydim, xs, ys, dxs, dys)

unsigned char  *ibuf;		/* Image array (2D). */
int		nbypx;		/* # bytes per pixel. */
int		xdim, ydim;	/* Image size. */
int		xs, ys;		/* Minimum x,y coordinates. */
int		dxs, dys;	/* Size of rectangle. */

{
   unsigned char	*cbuf;
   unsigned short	*sbuf;
   float		*fbuf;

   int    ih1, ih2;
   int    iv1, iv2;
   int    ix, iy;
   int    ival;
   int    lx, hx;

   float  vallo, valhi;
   float  frac;

   cbuf = (unsigned char  *) ibuf;
   sbuf = (unsigned short *) ibuf;
   fbuf = (float *)          ibuf;

   ih1 = xs;
   ih2 = xs + dxs;
   iv1 = ys;
   iv2 = ys + dys;

   lx = ((ih1 > 0) ? ih1 - 1 : 0);			/* Set min x coord. */
   hx = ((ih2 < xdim - 1) ? ih2 + 1 : xdim - 1);	/* Set max x coord. */

   /*** Interpolation Loop. ***/

   if (nbypx == 1)
      for (iy = iv1; iy < iv2; iy++)
         {
         for (ix = ih1; ix < ih2; ix++)
            {
	    vallo = (float) (*(cbuf + lx + iy * xdim) & 0377);
	    valhi = (float) (*(cbuf + hx + iy * xdim) & 0377);

	    frac = (float) (ix - ih1) / (float) (ih2 - ih1);

	    ival = (int) (frac * (valhi - vallo) + vallo);

	    *(cbuf + ix + iy * xdim) = (unsigned char) (ival & 0377);
            }
         }
   else
   if (nbypx == 2)
      for (iy = iv1; iy < iv2; iy++)
         {
         for (ix = ih1; ix < ih2; ix++)
            {
	    vallo = (float) (*(sbuf + lx + iy * xdim) & 0177777);
	    valhi = (float) (*(sbuf + hx + iy * xdim) & 0177777);

	    frac = (float) (ix - ih1) / (float) (ih2 - ih1);

	    ival = (int) (frac * (valhi - vallo) + vallo);

	    *(sbuf + ix + iy * xdim) = (unsigned short) (ival & 0177777);
            }
         }
   else
   if (nbypx == 4)
      for (iy = iv1; iy < iv2; iy++)
         {
         for (ix = ih1; ix < ih2; ix++)
            {
	    vallo = *(fbuf + lx + iy * xdim);
	    valhi = *(fbuf + hx + iy * xdim);

	    frac = (float) (ix - ih1) / (float) (ih2 - ih1);

	    *(fbuf + ix + iy * xdim) = (frac * (valhi - vallo) + vallo);
            }
         }

   return (0);
}
