/* vints.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill image elements which are less than 'level' with
 * vertically-interpolated values.
 * For MULTIPLE box areas & variable sizes.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 30 July 1986
 * 17 July 1992: SGI GL version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

vints (sbuf, xsize, ysize, level, nbox, xx, yy, dxx, dyy, dir2)

unsigned short *sbuf;	/* Image buffer. */
int xsize;		/* X dimension of image. */
int ysize;		/* Y dimension of image. */
int level;		/* Intensity threshold level. */
int nbox;		/* Number of values in arrays x & y. */
int *xx, *yy;		/* Arrays: corner coordinates of rectangular image. */
int *dxx, *dyy;		/* Dimensions of area. */
char *dir2;		/* Direction [l,r][u,d] for array storage. */
{
   int i;
   int ibox;
   int ix, iy;
   int x, y;
   int dx, dy;
   unsigned short *aimg;	/* Pointer to image array. */

/*   fprintf (stdout, "vints> x/ysize:%4d %4d  level:%3d  nbox:%3d  dir2: %s\n",
 *            xsize, ysize, level, nbox, dir2);
 *
 *   for (i = 0; i < nbox; i++)
 *      fprintf (stdout, "vints> xx:%d   yy:%d   dxx:%d   dyy:%d\n",
 *               xx [i], yy [i], dxx [i], dyy [i]);
 */

   /*** Box loop. ***/

   for (ibox = 0; ibox < nbox; ibox++)
      {
      x = xx [ibox];
      y = yy [ibox];
      dx = dxx [ibox];
      dy = dyy [ibox];

      /* Allocate storage for rectangular area of image.
       * Read from image array `sbuf' into temporary memory array `aimg'.
       */

      aimg = (unsigned short *)  malloc (dx * dy * sizeof (short));

      /* If needed, adjust coordinates so that array order is right and up. */

      if ( (*dir2 == 'l') || (*(dir2+1) == 'l') )  x = x - dx + 1;
      if ( (*dir2 == 'd') || (*(dir2+1) == 'd') )  y = y - dy + 1;

/*      fprintf (stdout, "vints> ibox:%2d x/y:%4d %d  dx/y:%4d %4d\n",
 *               ibox, x, y, dx, dy);
 */

      /* Read subset of image. */

      mr_box (sbuf, xsize, ysize, aimg, x, y, dx, dy, "ur");

      /*** Do interpolation/fill. ***/

/*
 *      for (iy = 0; iy < dy; iy++)
 *         for (ix = 0; ix < dx; ix++)
 *            *(aimg + ix + iy * dx) = 4;
 *
 */
      hfill (aimg, dy, dx, level);

      /*** Write resultant image to array `sbuf'. */

      mw_box (sbuf, xsize, ysize, aimg, x, y, dx, dy, "ur");
      free (aimg);
      }

   return (0);
}
