/* cpxm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Absolute/excess mass.
 * SGI/IRIS version.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * HAO/NCAR   Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 21 March 1984
 * 21 December 1984: Add brightness threshold.
 * 31 October 1985: Accept overlay color, not number.
 * 14 November 1985: Display comparison image.  Draw wedge before computation.
 * 5 December 1986: Use 'rcoord' instead of 'coord'.
 * 9 July 1990: Add cursor selection of wedge vertices.
 * 25 April 1991: adapt for SGI/IRIS; not able to read mouse position.
 * 23 October 1991: add logic to read SGI mouse position.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <fmclient.h>
#include <gl/gl.h>
#include <gl/device.h>

/* Global variables. */

#include "/usr/people/smm/lib/src/whip/grscale.h"
#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/whip/ov_color.h"
#include "cpwedge.h"

#define maxline 84
int DSP = 0;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{
FILE *pgmout;

   static char line [maxline];		/* Array to store keyboard input. */
   static char namlog [80];		/* Log file name. */
   static char nampgm [80];		/* Program name. */
   static char namopt [80];		/* Option argument. */
   static char namref [80] = "none";	/* Reference image file name. */
   static char namcmp [80] = "CMP";	/* Comparison image file name. */
   static char answer [4];		/* Storage for y/n response. */
   static char namovr [12];		/* Name for overlay color. */
   static char corner [4][24] = {"minimum position angle",
                              "maximum position angle",
                              "minimum radius",
                              "maximum radius"};

 
   static int isx [4], isy [4];		/* Image domain coordinates. */
   static float wradius [4], wangle [4];/* Wedge radius & angle coordinates. */

   float	p11, p12, p21, p22;		/* Pixel limits for wedge. */
   float	r11, r12, r21, r22;		/* Line limits for wedge. */
   float	cbth = 0.0;			/* Brightness threshold. */
   float	wedger, wedgeth, wthdif;   /* Temporary wedge radius & angle. */
   float	wx, wy;

   int	c;				/* Storage for keyboard response. */
   int	i;				/* Index variable. */
   int	icursor = 0;			/* Cursor flag: =1 use cursor. */
   int	ip11, ip12, ip21, ip22;		/* Pixel limits for wedge. */
   int	ir11, ir12, ir21, ir22;		/* Line limits for wedge. */
   int	mx;		/* Mass: =0 Absolute; otherwise, Excess CMP - REF. */
   int	ref_hour, ref_min;		/* Reference hour & minute. */
   int	zx, zy;				/* Cursor position. */

   long xs, ys;			/* x/y size of current window. */
   long xo, yo;			/* x/y origin of current window. */
   int xocp = 63, yocp = 510;	/* C/P image origin in window coordinates. */
   long xscrnsize;		/* size of screen in x to set globals.*/
   long xmaxscrn, ymaxscrn;	/* max size of screen in x & y. */
   float xcp, ycp;		/* x/y position in C/P image coords. */
   float r, th;			/* radius & theta of C/P position. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   i = 1;
   igr = 1;			/* Display option on by default. */
   lcc  = 0;			/* Munro correction off by default. */
   lstr = 0;
   lvig = 0;
   lfc  = 0;
   lsv  = 0;
   mx = 0;
   
   strcpy (nampgm, argv [0]);
   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-cd] [namref] [namcmp]\n", nampgm);

      fprintf (stderr,
              "Do you wish to apply Munro calibration correction [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lcc = 1;  else lcc = 0;

      fprintf (stderr, "Do you wish to display image [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  igr = 1;  else igr = 0;

      fprintf (stderr,
             "Enter reference C/P image file name ['none' if non-existent]:  ");
      fgets (line, maxline, stdin);
      sscanf  (line, "%s", namref);

      fprintf (stderr, "Enter comparison C/P image file name:  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", namcmp);
      }
   else
   if (argc == 2)	/* One argument: must be namcmp. */
      {
      igr = 1;
      mx = 0;
      strcpy (namcmp, argv [1]);
      }
   else
   if (argc == 3)	/* Two arguments: namref & namcmp or -d & namcmp. */
      {
      strcpy (namref, argv [1]);
      if (namref [0] == '-')
         {
         if (namref [1] == 'c' || namref [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namref [1] == 'd' || namref [2] == 'd')  igr = 0;  else igr = 1;
         strcpy (namref, "none");
         igr = 0;
         mx = 0;
         }
      else
         mx = 1;
      strcpy (namcmp, argv [2]);
      printf ("argc=3   namref: %s   namcmp: %s   mx: %d\n", namref, namcmp, mx);
      }
   else
   if (argc == 4)	/* Three arguments: -cd namref namcmp. */
      {
      strcpy (namopt, argv [1]);
      if (namopt [0] == '-')
         {
         if (namopt [1] == 'c' || namopt [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namopt [1] == 'd' || namopt [2] == 'd')  igr = 0;  else igr = 1;
         }
      strcpy (namref, argv [2]);
      if (strcmp (namref, "none") == 0)  mx = 0;  else mx = 1;
      strcpy (namcmp, argv [3]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-cd] [namref] [namcmp]\n", nampgm);
      exit (-1);
      }

   if (strncmp (namref, "none", 4) == 0)  mx = 0;  else mx = 1;
   if (mx)
      {
      cphskp (namref);
      ref_hour = H.hour;
      ref_min  = H.minute;
      }
   cphskp (namcmp);

   if (H.ires == 0)		/* Change window image offsets for Hi res. */
      {
      xocp *= 2;
      yocp *= 2;
      }

   /*** Open log file. ***/

   if (mx)
      sprintf (namlog, "%02d%03d.%02d%02d-%02d%02d.xm",
               H.year, H.doy, H.hour, H.minute, ref_hour, ref_min);
   else
      sprintf (namlog, "%02d%03d%02d%02d.xm",
               H.year, H.doy, H.hour, H.minute);
   printf ("namlog: %s\n", namlog);

   if ((pgmout = fopen (namlog, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namlog);
      exit (-1);
      }

   fprintf (pgmout, "--- %s : SMM C/P coronal mass program ---\n", argv [0]);

   if (mx  ==  0)
      fprintf (pgmout, "\n*** absolute mass computation : %s ***\n\n", namcmp);
   if (mx  >  0)
      fprintf (pgmout,
              "\n*** excess mass computation : %s - %s ***\n", namcmp,namref);
   fprintf (pgmout, "--------------------------------------------------------------------------------\n");
   if (mx) 
      {
      fprintf (pgmout, "bkg   : ");
      hkline (pgmout);
      }
   fprintf (pgmout, "event : ");
   hkline (pgmout);
   fprintf (pgmout, "--------------------------------------------------------------------------------\n");

   /*** Initialize GL software. ***/

   if (igr)
      {
      DSP = 1;
      win_init (nampgm, H.ires, &xo, &yo, &xs, &ys, &xmaxscrn, &ymaxscrn);

      glcp_view (namcmp, 1);

      /*** Overlay set-up. ***/

      overlay (2);		/* Establish two bitplanes for overlay. */
      gconfig ();

      drawmode (OVERDRAW);		     /* Set overlay mode for drawing. */
      mapcolor (ov_black,   0,   0,   0);	/* color 0 -> black. */
      mapcolor (ov_red,   255,   0,   0);	/* color 0 -> red.   */
      mapcolor (ov_green,   0, 255,   0);	/* color 0 -> green. */
      mapcolor (ov_blue,    0,   0, 255);	/* color 0 -> blue.  */
      gl_clear (ov_black);
      }

next:
   if (igr)
   fprintf (stderr, "Select wedge with cursor [c] or keyboard [k] or Quit [q] ? ");
   else
   fprintf (stderr, "Select wedge with keyboard [k] or Quit [q] ? ");
   fgets (line, maxline, stdin);
   sscanf (line, "%s", answer);
   if (strncmp ("c", answer, 1) == 0)
      icursor = 1;
   else if (strncmp ("k", answer, 1) == 0)
      icursor = 0;
   else if (strncmp ("q", answer, 1) == 0)
      {
      drawmode (OVERDRAW);	/* Clean up overlay bitplanes. */
      gl_clear (ov_black);

      drawmode (NORMALDRAW);
      onemap ();		/* Set single-map mode for color table. */
      gconfig ();
      gexit();
      exit (0);
      }

   /*** Excess/Absolute Mass. ***/

wedge:
   if (icursor == 0)
      {
      fprintf (stderr, "Enter solar position angle min & max (deg.):  ");
      fgets (line, maxline, stdin);
      sscanf (line, "%f,%f", &wthmin, &wthmax);
      fprintf (stderr, "Enter solar radius min, max (Rsun):  ");
      fgets (line, maxline, stdin);
      sscanf (line, "%f,%f", &wrmin, &wrmax);
      }

   /*** Convert wedge corners to cartesian coordinates. ***/

   if (igr == 1)
      {
      gl_clear (BLACK);
      if (icursor == 1)
	 {
	 for (i=0; i<4; i++)
            {
            fprintf (stderr, "Position cursor at %s.\n", &corner [i][0]);
	    fprintf (stderr, "Click Right mouse button.\n");
            rdmpos (xmaxscrn, ymaxscrn, &xo, &yo, &xs, &ys, &zx, &zy);

	    gl_wvec (zx-1, zy-1, zx+1, zy+1, ov_red);/* Draw 3x3 pixel dot. */
	    gl_wvec (zx+1, zy-1, zx-1, zy+1, ov_red);/* Draw 3x3 pixel dot. */
            isx [i] = zx   - xocp;	/* x coordinate in image domain. */
            isy [i] = yocp - zy;	/* y coordinate in image domain. */
/*
 *            fprintf (stderr, "zx: %3d   zy: %3d      isx: %3d   isy: %3d\n",
 *	  	       zx, zy, isx[i], isy[i]);
 */

            wx = (float) isx [i];
	    wy = (float) isy [i];
            if (rcoord (&wradius [i], &wangle [i], &wx, &wy, -1, scroll) != 0)
               {
               geoerr (wradius [i], wangle [i], wx, wy);
               goto next;
               }
/*
 *	    fprintf (stderr, "radius: %5.2f   position angle: %6.2f\n",
 *                   wradius [i], wangle [i]);
 */
	    }

         wthmin = wangle [0];
	 wthmax = wangle [1];
	 wrmin = wradius [2];
	 wrmax = wradius [3];
         }

      fprintf (stderr,
              "wrmin : %6.2f   wthmin: %6.2f   wrmax : %6.2f   wthmax: %6.2f\n",
	       wrmin, wthmin, wrmax, wthmax);

      if (wrmin   > wrmax)		/* Reverse min/max radii. */
	 {wedger = wrmin;   wrmin = wrmax;   wrmax = wedger;}

      while ((wthdif = wthmax - wthmin) < 0.0)  wthmax += 360;
      if (wthdif > 120.0)		/* reverse min/max angles. */
	 {
	 wedgeth = wthmin;  wthmin = wthmax;  wthmax = wedgeth;
         wthdif = wthmax - wthmin;
	 }
      fprintf (stdout,
              "wrmin : %6.2f   wthmin: %6.2f   wrmax : %6.2f   wthmax: %6.2f\n",
	       wrmin, wthmin, wrmax, wthmax);

      /*** Convert min/max (r,th) values to wedge vertices. ***/

      if (rcoord (&wrmin, &wthmin, &p11, &r11, 1, scroll) != 0)
         {
         geoerr (wrmin, wthmin, p11, r11);
         goto next;
         }
      if (rcoord (&wrmin, &wthmax, &p12, &r12, 1, scroll) != 0)
         {
         geoerr (wrmin, wthmax, p12, r12);
         goto next;
         }
      if (rcoord (&wrmax, &wthmin, &p21, &r21, 1, scroll) != 0)
         {
         geoerr (wrmax, wthmin, p21, r21);
         goto next;
         }
      if (rcoord (&wrmax, &wthmax, &p22, &r22, 1, scroll) != 0)
         {
         geoerr (wrmax, wthmax, p22, r22);
         goto next;
         }

      ip11 = (int) p11 / nres + xocp;
      ip12 = (int) p12 / nres + xocp;
      ip21 = (int) p21 / nres + xocp;
      ip22 = (int) p22 / nres + xocp;
      ir11 = yocp - (int) r11 / nres;
      ir12 = yocp - (int) r12 / nres;
      ir21 = yocp - (int) r21 / nres;
      ir22 = yocp - (int) r22 / nres;

/*
 *      fprintf (stdout, "ip11:%d  ir11:%d  ip12:%d  ir12:%d\n",
 *               ip11, ir11, ip12, ir12);
 *      fprintf (stdout, "ip21:%d  ir21:%d  ip22:%d  ir22:%d\n",
 *               ip21, ir21, ip22, ir22);
 */

      glcp_arc (wthmin, wthmax, wrmin, ov_red);	/* Draw arc at wrmin. */
      glcp_arc (wthmin, wthmax, wrmax, ov_red);	/* Draw arc at wrmax. */
      gl_wvec (ip11, ir11, ip21, ir21, ov_red);	/* Draw radial line at wthmin.*/
      gl_wvec (ip12, ir12, ip22, ir22, ov_red);	/* Draw radial line at wthmax.*/

      fprintf (stderr, "Is the mass integration area acceptable [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp("y",answer,1) != 0)  goto wedge;
      }

/*
 *   fprintf (stderr, "Enter coronal brightness threshold (B/Bsun):  ");
 *   fgets (line, maxline, stdin);
 *   sscanf  (line,  "%e", &cbth);
 *   if (cbth > 1.e-6)
 *      {
 *      fprintf (stderr, "Threshold must be in the range 1.e-11 to 1.e-7\n");
 *      exit(-1);
 *      }
 */

   i++;

/* Execute mass computation subroutine. */

   cpmass (namref, namcmp, mx, cbth, pgmout);

   goto next;
}


/* geoerr
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Report coordinate range error.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

geoerr (r, th, x, y)

float r, th, x, y;

{
   fprintf (stderr, "Invalid coordinate range: r:%5.2f   th:%7.2f   x:%6.2f   y:%6.2f\n",
        r, th, x, y);
   return (0);
}


/* win_init
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Initialize GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 23 October 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

win_init (nampgm, res, xorig, yorig, xsize, ysize, xmaxscrn, ymaxscrn)

char *nampgm;			/* Program name. */
int res;			/* C/P resolution [0=hi, 1=lo]. */
long *xorig, *yorig;		/* window x/y origin. */
long *xsize, *ysize;		/* window x/y size. */
long *xmaxscrn, *ymaxscrn;     /* maximum size of screen in x and y       */

{
   static Cursor plus1 = {0x0100, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0380, 0xFEFE,
                          0x0380, 0x0100, 0x0100, 0x0100,
                          0x0100, 0x0100, 0x0100, 0x0000};
 
   long xscrnsize;              /* size of screen in x used to set globals */
   Colorindex dummy;		/* For strict ANSI C proto-typing. */

   xscrnsize = getgdesc (GD_XPMAX);       /* get/set screen size */
   if (xscrnsize == 1280)
      {
      *xmaxscrn = 1279;
      *ymaxscrn = 1023;
      }
   else if (xscrnsize == 1024)
      {
      *xmaxscrn = 1023;
      *ymaxscrn = 767;
      }
   else
      {
      fprintf (stderr, "Something's EXTREMELY wrong:  ");
      fprintf (stderr, "xscrnsize=%d\n", xscrnsize);
      exit (-1) ;
      }

   foreground ();

   if (res == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (266, 1279,   0, 1023);/* 1Kx1K window in upper right corner.*/

   fminit ();			/* Initialize SGI font manager. */

   winopen (nampgm);
   getorigin (xorig, yorig);
   getsize (xsize, ysize);

   color (BLACK);
   clear ();
   color (RED);

   onemap ();
   gconfig ();
   pixmode (PM_ADD24, 512);		/* i.e., effectively use color map #2.*/

   curstype (C16X1);			/* Modified small crosshair cursor. */
   defcursor (1, plus1);
   curorigin (1, 7, 7);
   setcursor (1, dummy, dummy);
  
   return (0);
}


/* rdmpos
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Read mouse position.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#define XY 2
#define X 0
#define Y 1

rdmpos (xmaxscrn, ymaxscrn, xo, yo, xs, ys, xwin, ywin)

long xmaxscrn, ymaxscrn;     /* maximum size of screen in x and y       */
long *xo, *yo;                 /* x/y origin of current window */
long *xs, *ys;                 /* x/y size of current window   */
long *xwin, *ywin;

{
   Device mdev [XY];
   short mval [XY];
   short val;
   long dev;

   mdev [X] = MOUSEX;
   mdev [Y] = MOUSEY;

   qreset ();
   qdevice (ESCKEY);
   qdevice (LEFTMOUSE);
   qdevice (MIDDLEMOUSE);

   while (1)
      {
      while (qtest ())
         {
         dev = qread (&val);
         switch (dev)
            {
            case ESCKEY:
                 setvaluator (MOUSEX, (short)(*xs/2), 0, xmaxscrn);
                 setvaluator (MOUSEY, (short)(*ys/2), 0, ymaxscrn);
                 gexit ();
                 exit (1);
                 break;
            case REDRAW:
                 reshapeviewport ();
                 getsize (xs, ys);
                 getorigin (xo, yo);
                 color (CYAN);
                 clear ();
                 break;
            case MIDDLEMOUSE:	/* Read winition of mouse. */
            case LEFTMOUSE:	/* Read winition of mouse. */
                 if (val == 1)
                    {
                    getdev (XY, mdev, mval);
                    *xwin = mval [X] - *xo;
                    *ywin = mval [Y] - *yo;
                    printf ("mval: %3d %3d   xo/yo: %3d %3d   xwin/ywin: %3d %3d\n",
                             mval [0], mval [1], *xo, *yo, *xwin, *ywin);
                    qreset ();
                    return (0);
                    }
                 break;
            default:
                 break;
            }
         }
      }
}
