/* xm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Absolute/excess mass.
 * SGI/IRIS version.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * HAO/NCAR   Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 21 March 1984
 * 21 December 1984: Add brightness threshold.
 * 31 October 1985: Accept overlay color, not number.
 * 14 November 1985: Display comparison image.  Draw wedge before computation.
 * 5 December 1986: Use 'rcoord' instead of 'coord'.
 * 9 July 1990: Add cursor selection of wedge vertices.
 * 25 April 1991: adapt for SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>

/* Global variables. */

#include "/usr/people/smm/lib/src/ip/grscale.h"
#include "/usr/people/smm/lib/src/cp/cplib.h"
#include "/usr/people/smm/lib/src/ip/ov_color.h"
#include "cpwedge.h"

#define maxline 84
int GRN = 0;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

static char line [maxline];
static char nampgm [80];
static char namopt [32];		/* Option argument. */
static char namref [32] = "none"; /* Reference image file name. */
static char namcmp [32] = "CMP"; /* Comparison image file name. */
static char answer [4];		/* Storage for y/n response. */
static char namovr [12];	/* Name for overlay color. */
static char corner [4][24] = {"minimum position angle",
                              "maximum position angle",
                              "minimum radius",
                              "maximum radius"};

static int ipx [4], ipy [4];
static int isx [4], isy [4];
static float wradius [4], wangle [4];

float	p11, p12, p21, p22;	/* Pixel limits for wedge. */
float	r11, r12, r21, r22;	/* Line limits for wedge. */
float	cbth = 0.0;		/* Brightness threshold. */
float	thinc = 1.0;
float	wedger, wedgeth, wthdif;
float	wrmin1, wrmin2;
float	wrmax1, wrmax2;
float	wthmin1, wthmin2;
float	wthmax1, wthmax2;
float	wx, wy;

int	c;
int	i;
int	icursor = 0;
int	ip11, ip12, ip21, ip22;	/* Pixel limits for wedge. */
int	ir11, ir12, ir21, ir22;	/* Line limits for wedge. */
int	mx;		/* Mass: =0 Absolute; otherwise, Excess CMP - REF. */
int	zx, zy;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   i = 1;
   igr = 1;			/* Display option on by default. */
   lcc  = 0;			/* Munro correction off by default. */
   lstr = 0;
   lvig = 0;
   lfc  = 0;
   lsv  = 0;
   mx = 0;
   
   strcpy (nampgm, argv [0]);
   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-d] [namref] [namcmp]\n", nampgm);

      fprintf (stderr,
              "Do you wish to apply Munro calibration correction [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  lcc = 1;  else lcc = 0;

      fprintf (stderr, "Do you wish to display image [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp ("y", answer, 1) == 0)  igr = 1;  else igr = 0;

      fprintf (stderr,
             "Enter reference C/P image file name ['none' if non-existent]:  ");
      fgets (line, maxline, stdin);
      sscanf  (line, "%s", namref);

      fprintf (stderr, "Enter comparison C/P image file name:  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", namcmp);
      }
   else
   if (argc == 2)	/* One argument: must be namcmp. */
      {
      igr = 1;
      mx = 0;
      strcpy (namcmp, argv [1]);
      }
   else
   if (argc == 3)	/* Two arguments: namref & namcmp or -d & namcmp. */
      {
      strcpy (namref, argv [1]);
      if (namref[0] == '-')
         {
         if (namref [1] == 'c' || namref [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namref [1] == 'd' || namref [2] == 'd')  igr = 0;  else igr = 1;
         strcpy (namref, "none");
         igr = 0;
         mx = 0;
         }
      else
         mx = 1;
      strcpy (namcmp, argv [2]);
      printf ("argc=3   namref: %s   namcmp: %s   mx: %d\n", namref, namcmp, mx);
      }
   else
   if (argc == 4)	/* Three arguments: -cd namref namcmp. */
      {
      strcpy (namopt, argv [1]);
      if (namopt[0] == '-')
         {
         if (namref [1] == 'c' || namref [2] == 'c')  lcc = 1;  else lcc = 1;
         if (namref [1] == 'd' || namref [2] == 'd')  igr = 0;  else igr = 1;
         }
      strcpy (namref, argv [2]);
      if (strcmp (namref, "none") == 0)  mx = 0;
      strcpy (namcmp, argv [3]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-d] [namref] [namcmp]\n", nampgm);
      exit (-1);
      }

   if (strncmp (namref, "none", 4) == 0)  mx = 0;  else mx = 1;
   cphskp (namcmp);

   /*** Initialize GL software. ***/

   if (igr)
      {
      GRN = 1;
      foreground ();
      prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
      fminit ();			/* Initialize SGI font manager. */

      winopen (nampgm);
      gl_clear (BLACK);
      onemap ();
      gconfig ();
      pixmode (PM_ADD24, 512);		/* i.e., effectively use color map #2.*/

      glcp_view (namcmp, 1);
      sleep (5);
      }

   /*** Overlay set-up. ***/

   overlay (2);			/* Establish two bitplanes for overlay. */
   gconfig ();

   drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
   mapcolor (ov_black,   0,   0,   0);	/* color 0 -> black. */
   mapcolor (ov_red,   255,   0,   0);	/* color 0 -> red.   */
   mapcolor (ov_green,   0, 255,   0);	/* color 0 -> green. */
   mapcolor (ov_blue,    0,   0, 255);	/* color 0 -> blue.  */
   gl_clear (ov_black);

next:
   if (igr)
   fprintf (stderr, "Select wedge with cursor [c] or keyboard [k] or Quit [q] ? ");
   else
   fprintf (stderr, "Select wedge with keyboard [k] or Quit [q] ? ");
   fgets (line, maxline, stdin);
   sscanf (line, "%s", answer);
   if (strncmp ("c", answer, 1) == 0)
      icursor = 1;
   else if (strncmp ("k", answer, 1) == 0)
      icursor = 0;
   else if (strncmp ("q", answer, 1) == 0)
      {
		/* Turn off zoom cursor. */
/*
      pan (OFF);
      zcursor (OFF);
 */
      /*** Clean up overlay bitplanes. ***/
   
      drawmode (OVERDRAW);
      gl_clear (ov_black);

      drawmode (NORMALDRAW);
      onemap ();		/* Set single-map mode for color table. */
      gconfig ();
      gexit();
      exit (0);
      }

/* Excess/Absolute Mass. */

wedge:
   if (icursor == 0)
      {
      fprintf (stderr, "Enter solar position angle min & max (deg.):  ");
      fgets (line, maxline, stdin);
      sscanf (line, "%f,%f", &wthmin, &wthmax);
      fprintf (stderr, "Enter solar radius min, max (Rsun):  ");
      fgets (line, maxline, stdin);
      sscanf (line, "%f,%f", &wrmin, &wrmax);
      }

/* Convert wedge corners to cartesian coordinates. */

   if (igr == 1)
      {
      gl_clear (BLACK);
      if (icursor == 1)
	 {
/*
          pan (OFF);
          zcursor (ON);
          szcur (287,287);
 */
	 for (i=0; i<4; i++)
            {
            fprintf (stderr, "Position cursor at %s.\n", &corner [i][0]);
	    fprintf (stderr, "Click Right mouse button.\n");
            fprintf (stderr, "Type any character + <RETURN> to continue . . .\n");
	    fflush (stderr);
	    fflush (stdin);
	    c = getc (stdin);

/*            rzcur (); */ /* Read x & y coordinates of zoom cursor position. */
/*            zx = Gr.Zcx; */	/* x coordinate of zoom cursor position. */
/*            zy = Gr.Zcy; */	/* y coordinate of zoom cursor position. */
	    gl_wvec (zx-1, zy-1, zx+1, zy+1, ov_red);/* Draw 2x2 pixel dot. */
	    gl_wvec (zx+1, zy-1, zx-1, zy+1, ov_red);/* Draw 2x2 pixel dot. */
            isx [i] = zx  - 63;	/* x coordinate in image domain. */
            isy [i] = 510 - zy;	/* y coordinate in image domain. */
/*
 *            fprintf (stderr, "zx: %3d   zy: %3d      isx: %3d   isy: %3d\n",
 *	  	     zx, zy, isx[i], isy[i]);
 */

         wx = (float) isx[i];
	 wy = (float) isy [i];
         if (rcoord (&wradius[i], &wangle[i], &wx, &wy, -1, scroll) != 0)  goto geoerr;
/*
 *	 fprintf (stderr, "radius: %5.2f   position angle: %6.2f\n", wradius[i], wangle[i]);
 */
	    }

         wthmin = wangle [0];
	 wthmax = wangle [1];
	 wrmin = wradius [2];
	 wrmax = wradius [3];
/*         zcursor (OFF); */
         }

      fprintf (stderr,
              "wrmin : %6.2f   wthmin: %6.2f   wrmax : %6.2f   wthmax: %6.2f\n",
	       wrmin, wthmin, wrmax, wthmax);

      if (wrmin   > wrmax)		/* Reverse min/max radii. */
	 {wedger = wrmin;   wrmin = wrmax;   wrmax = wedger;}

      while ((wthdif = wthmax - wthmin) < 0.0)  wthmax += 360;
      if (wthdif > 120.0)		/* reverse min/max angles. */
	 {
	 wedgeth = wthmin;  wthmin = wthmax;  wthmax = wedgeth;
         wthdif = wthmax - wthmin;
	 }
      fprintf (stderr,
              "wrmin : %6.2f   wthmin: %6.2f   wrmax : %6.2f   wthmax: %6.2f\n",
	       wrmin, wthmin, wrmax, wthmax);

      /*** Convert min/max (r,th) values to wedge vertices. ***/

      if (rcoord (&wrmin, &wthmin, &p11, &r11, 1, scroll) != 0)  goto geoerr;
      if (rcoord (&wrmin, &wthmax, &p12, &r12, 1, scroll) != 0)  goto geoerr;
      if (rcoord (&wrmax, &wthmin, &p21, &r21, 1, scroll) != 0)  goto geoerr;
      if (rcoord (&wrmax, &wthmax, &p22, &r22, 1, scroll) != 0)  goto geoerr;

      ip11 = (int) p11 / nres + 63;
      ip12 = (int) p12 / nres + 63;
      ip21 = (int) p21 / nres + 63;
      ip22 = (int) p22 / nres + 63;
      ir11 = 510 - (int) r11 / nres;
      ir12 = 510 - (int) r12 / nres;
      ir21 = 510 - (int) r21 / nres;
      ir22 = 510 - (int) r22 / nres;

/*
 *	printf ("ip11:%d  ir11:%d  ip12:%d  ir12:%d\n", ip11, ir11, ip12, ir12);
 *	printf ("ip21:%d  ir21:%d  ip22:%d  ir22:%d\n", ip21, ir21, ip22, ir22);
 */

      glcp_arc (wthmin, wthmax, wrmin, ov_red);	/* Draw arc at wrmin. */
      glcp_arc (wthmin, wthmax, wrmax, ov_red);	/* Draw arc at wrmax. */
      gl_wvec (ip11, ir11, ip21, ir21, ov_red);	/* Draw radial line at wthmin.*/
      gl_wvec (ip12, ir12, ip22, ir22, ov_red);	/* Draw radial line at wthmax.*/

      fprintf (stderr, "Is the mass integration area acceptable [y/n] ?  ");
      fgets (line, maxline, stdin);
      sscanf  (line,  "%s", answer);
      if (strncmp("y",answer,1) != 0)  goto wedge;
      }

/*
 *   fprintf (stderr, "Enter coronal brightness threshold (B/Bsun):  ");
 *   fgets (line, maxline, stdin);
 *   sscanf  (line,  "%e", &cbth);
 *   if (cbth > 1.e-6)
 *      {
 *      fprintf (stderr, "Threshold must be in the range 1.e-11 to 1.e-7\n");
 *      exit(-1);
 *      }
 */

   i++;

/* Execute mass computation subroutine. */

   mass (namref, namcmp, mx, cbth);

   goto next;

geoerr: fprintf (stderr, "Invalid range: radius or theta");
        exit (-10);
}
