/* cpovfb.c
 * usage: cpovfb [image_filename] [ovfile] [color_index]
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write pixels (specified by input file coordinates) into image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Works only for calibrated (short integer) C/P images.
 * Generates its own greyscale color table (saves & restores pre-existing LUT).
 * Writes to current WHIP frame buffer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 13 July 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Standard C system I/O library and Math functions library. */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>

/* Global variables. */

#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/grscale.h"
#define Black 0
#define Red 254
#define Green 253
#define Grey 251
#define Yellow 250
#define Blue 252
#define White 255 
#define ESC 27

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main (argc, argv)

int argc;
char *argv [];
{

   extern int cp_view ();	/* Image display function. */
   extern char *rdimg ();	/* Read image from disk to memory. */

   FILE   *LIST, *OV;
   char   *imgbuf;		/* Pointer to image array. */
   char   key;
   static char flag [8];
   static char inline [80];	/* Input line buffer. */
   static char namimg [80];	/* Image file name. */
   static char namlst [80];	/* Name of file containing image list. */
   static char namov [80];	/* Overlay file name. */
   static char nampgm [80];	/* Program file name. */
   static char answer [2];	/* Storage for y/n response. */

   static short r [512], g [512], b [512];
   short icol;
   unsigned short cindex;
   int Cval;
   int FB = 0;		/* default: 512x512 byte frame buffer (colormap mode).*/
   int fast = 1;	/* Parameter for fb_clear. */
   int i, ioff;
   float offset, slope;
   float bkgcol = 0.0;	/* Grey level for fb_clear. */

   int ierr;		/* Error variable. */
   int imgcnt;		/* Image counter. */
   int linit = 1;	/* Display initialization flag. */
   int log = 0;		/* Flag to indicate existence of output log file. */
   int nval;		/* # values in scan. */
   int ovcnt;		/* Overlay file line counter. */
   int seq = 0;		/* Flag to indicate sequence of images. */
   int xl = 760;	/* Specification of GL window position & size. */
   int yl = 480;
   int xu = 1271;
   int yu = 991;
   int xpos, ypos;
   int xpos1, ypos1;
   int xpos2, ypos2;

   /*** Process input parameter(s). ***/

   strcpy (nampgm, argv [0]);
   sprintf (namov, "cpov.pix");
   Cval = Green;

   fprintf (stderr, "%s: draw specified pixels in overlay\n",
            nampgm);

   if (argc == 1)		/* No arguments. */
      {
      fprintf (stderr,
         "usage: %s [image_filename | @filename_list] [overlay] [color]\n",
          argv [0]);
      fprintf (stderr, "Enter C/P image file name: ");
      fscanf  (stdin,  "%s", namimg);
      fprintf (stderr, "Enter overlay filename ['none' to omit]: ");
      fscanf  (stdin, "%s", namov);
      if (strncmp (namov, "none") == 0)
         log = 0;
      else
         log = 1;
      fprintf (stderr, "Enter color index level [254=red,253=green,250=yellow,252=blue,251=grey,255=white]: ");
      fscanf  (stdin,  "%d", &Cval);
      }
   else
   if (argc == 2)		/* 1 argument: must be image filename. */
      {
      strcpy (namimg, argv [1]);
      log = 0;
      }
   else
   if (argc == 3)		/* 2 arguments: 1st must be image filename. */
      {
      strcpy (namimg, argv [1]);		/* Input image file. */
      strcpy (namov, argv [2]);			/* Overlay file name. */
      log = 1;
      }
   else
   if (argc == 4)		/* 3 arguments: 1st must be image filename. */
      {
      strcpy (namimg, argv [1]);		/* Input image file. */
      strcpy (namov, argv [2]);			/* Overlay file name. */
      sscanf (argv [3], "%d", &Cval);		/* Color index value. */
      log = 1;
      }

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

   if (strncmp (namov, "none") == 0)
      log = 0;
   else
      {
      log = 1;
      if ((OV = fopen (namov, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open overlay file: %s\n", namov);
         exit (-2);
         }
      }

   igr = 1;
   igterm = 1;
   lsv = 0;
   lfc = 0;
   imgcnt = 0;

   /*** Initialize WHIP software. ***/

   fb_open ();
   fb_setfb (FB);
   openpl ();
   fb_clear (FB, &bkgcol, fast);

   /*** Invoke function to transfer image into frame buffer FB. ***/

   fprintf (stdout, "Displaying image: %s\n", namimg);
   if ((ierr = cp_view (namimg, FB, linit)) < 0)
      {
      fprintf (stderr, "cp_view error: %d\n", ierr);
      exit (-4);
      }

newimg:
   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }

   imgcnt++;

   /*** Display C/P image in WHIP frame buffer. ***/

   if ((ierr = cp_view (namimg, FB, linit)) < 0)
      {
      fprintf (stderr, "cp_view error: %d\n", ierr);
      exit (-4);
      }

   /*** Open input image disk file and read housekeeping record. ***/

   fprintf (stderr, "Reading image %s into memory.\n", namimg);

   imgbuf = rdimg (namimg);		/* Read image into memory array. */

   fprintf (stdout,"     G M T          Q(%%)  IMG  IH-IV RES  EXP  FW     PW  5303  SM     CM  ROLL\n");
   hkline (stdout);

   /*** Read overlay file & draw overlay perimeter. ***/

   ovcnt = 0;
   while (fscanf (OV, "%d %d", &xpos, &ypos) != EOF)
      {
      xpos += 63;
      ypos = 510 - ypos;
      if (ovcnt++ == 0)  {xpos1 = xpos;  ypos1 = ypos;  continue;}

      xpos2 = xpos;
      ypos2 = ypos;
      wvec (xpos1, ypos1, xpos2, ypos2, Cval);

      printf ("x/ypos1: %3d %3d   x/ypos2: %3d %3d\n",
              xpos1, ypos1, xpos2, ypos2);

      xpos1 = xpos2;
      ypos1 = ypos2;
      }

   /*** Relinquish storage for image array. ***/

quit:

   free (imgbuf);

   /*** Exit program. ***/
   
   fb_touch ();
   closepl ();
   fb_close ();

   exit (0);
}
