/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * glsrf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read byte image from GL window & convert it to a sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>

glsrf (namsrf, xdim, ydim)

char *namsrf;			/* Sun rasterfile name. */
int xdim, ydim;			/* X/Y dimensions. */
{
   extern char *malloc ();
   unsigned char *cbuf;
   unsigned short *sbuf;
   static unsigned char red [256], green [256], blue [256];

   int dx, dy, dz;		/* Frame Buffer size. */
   int i;
   int imin, imax;
   int ix, iy;			/* Indices for image array. */
   int nbypx;			/* # bytes per pixel. */
   int nbyimg;
   int row;			/* row index counter. */
   int startx = 0;		/* X location for writing. */
   int sy;			/* Y-dimension index. */
   int winsize;			/* Window size (short int). */
   int xll, yll;		/* Lower left  corner of image in window. */
   int xur, yur;		/* Upper right corner of image in window. */
   unsigned short off = 512;	/* Color level offset. */
   float frange;

   /*** Allocate memory for image array. ***/

   nbypx = 1;
   nbyimg = xdim * ydim * nbypx;
   winsize = xdim * ydim * sizeof (short);

   cbuf = (unsigned char  *)  malloc (nbyimg);
   sbuf = (unsigned short *)  malloc (winsize);

   /*** Copy image from frame buffer to memory. ***/

   xll = 0;
   yll = 0;
   xur = xdim - 1;
   yur = ydim - 1;
   rectread (xll, yll, xur, yur, sbuf);

   /*** Invert image top to bottom & convert from short to char. ***/

   for (iy = 0; iy < ydim;  iy++)
      {
      sy = ydim - iy;
      for (ix = 0; ix < xdim; ix++)
         *(cbuf + ix + iy*xdim) = (unsigned char) *(sbuf + ix + sy*xdim) - 512;
      }

   /*** Read color table. ***/

   cp_cmap (off, red, green, blue);

   /*** Create sun rasterfile and write to disk. ***/

   bytsrf (namsrf, cbuf, xdim, ydim, red, green, blue);

   free (cbuf);
   free (sbuf);
   return (0);
}


/* cp_cmap
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copy color map to memory arrays.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
cp_cmap (offset, red, green, blue)

unsigned short offset;
unsigned char red [];
unsigned char green [];
unsigned char blue [];

{
   unsigned short ic, im;
   short rcomp, gcomp, bcomp;

   for (ic = 0; ic < 256; ic++)
      {
      im = ic + offset;
      getmcolor (im, &rcomp, &gcomp, &bcomp);

      red [ic]   = (unsigned char) rcomp;
      green [ic] = (unsigned char) gcomp;
      blue [ic]  = (unsigned char) bcomp;
      }

   return (0);
}

