/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * kb_wedge : Define wedge using keyboard input.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 12 November 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/ov_color.h"
#include "cp_wedge.ext"
#define maxline 84
extern FILE *pgmlog;

kb_wedge (xocp, yocp, xo, yo, xs, ys, xmaxscrn, ymaxscrn, poly, nver)

int xocp, yocp;			/* C/P image origin. */
long *xo, *yo;			/* Window origin. */
long *xs, *ys;			/* Window size. */
short xmaxscrn, ymaxscrn;	/* Screen size. */
int poly [][2];			/* Polynomial vertices. */
int *nver;			/* # vertices in polygon. */


{
   static char answer [4];
   static char line [maxline];
   int i, it;
   int ip11, ip12, ip21, ip22;		/* Pixel limits for wedge. */
   int ir11, ir12, ir21, ir22;		/* Line limits for wedge. */
   float p11, p12, p21, p22;		/* Pixel limits for wedge. */
   float r11, r12, r21, r22;		/* Line limits for wedge. */
   float theta;				/* Theta angle. */
   float wedger, wedgeth, wthdif;	/* Temporary wedge radius & angle. */
   float xp, yp;			/* x/y coordinates. */

   /*** Request wedge limits from keyboard. ***/

boundary:

   fprintf (stderr, "Enter solar position angle min & max (deg.):  ");
   fgets   (line, maxline, stdin);
   sscanf  (line, "%f,%f", &wthmin, &wthmax);

   fprintf (stderr, "Enter solar radius min, max (Rsun):  ");
   fgets   (line, maxline, stdin);
   sscanf  (line, "%f,%f", &wrmin, &wrmax);

/*
 *   fprintf (stdout,
 *            "wrmin : %6.2f   wthmin: %6.2f   wrmax : %6.2f   wthmax: %6.2f\n",
 *             wrmin, wthmin, wrmax, wthmax);
 */

   if (wrmin   > wrmax)		/* Reverse min/max radii. */
      {wedger = wrmin;   wrmin = wrmax;   wrmax = wedger;}

   while ((wthdif = wthmax - wthmin) < 0.0)  wthmax += 360;
   if (wthdif > 120.0)		/* reverse min/max angles. */
      {
      wedgeth = wthmin;  wthmin = wthmax;  wthmax = wedgeth;
      wthdif = wthmax - wthmin;
      }
/*
 *   fprintf (stdout,
 *            "wrmin : %6.2f   wthmin: %6.2f   wrmax : %6.2f   wthmax: %6.2f\n",
 *             wrmin, wthmin, wrmax, wthmax);
 */


   /*** Convert min/max (r,th) values to wedge vertices. ***/

   if (rcoord (&wrmin, &wthmin, &p11, &r11, 1, scroll) != 0)
      {
      geoerr (wrmin, wthmin, p11, r11);
      goto boundary;
      }
   if (rcoord (&wrmin, &wthmax, &p12, &r12, 1, scroll) != 0)
      {
      geoerr (wrmin, wthmax, p12, r12);
      goto boundary;
      }
   if (rcoord (&wrmax, &wthmin, &p21, &r21, 1, scroll) != 0)
      {
      geoerr (wrmax, wthmin, p21, r21);
      goto boundary;
      }
   if (rcoord (&wrmax, &wthmax, &p22, &r22, 1, scroll) != 0)
      {
      geoerr (wrmax, wthmax, p22, r22);
      goto boundary;
      }

   ip11 = (int) p11 / nres + xocp;
   ip12 = (int) p12 / nres + xocp;
   ip21 = (int) p21 / nres + xocp;
   ip22 = (int) p22 / nres + xocp;
   ir11 = yocp - (int) r11 / nres;
   ir12 = yocp - (int) r12 / nres;
   ir21 = yocp - (int) r21 / nres;
   ir22 = yocp - (int) r22 / nres;

/*
 *   fprintf (stdout, "ip11:%d  ir11:%d  ip12:%d  ir12:%d\n",
 *            ip11, ir11, ip12, ir12);
 *   fprintf (stdout, "ip21:%d  ir21:%d  ip22:%d  ir22:%d\n",
 *            ip21, ir21, ip22, ir22);
 */

   glcp_arc (wthmin, wthmax, wrmin, ov_red);	/* Draw arc at wrmin. */
   glcp_arc (wthmin, wthmax, wrmax, ov_red);	/* Draw arc at wrmax. */
   gl_wvec (ip11, ir11, ip21, ir21, ov_red);	/* Draw radial line at wthmin.*/
   gl_wvec (ip12, ir12, ip22, ir22, ov_red);	/* Draw radial line at wthmax.*/

   /*** Create vetex array representing polygon approximating wedge. */

   /*** wrmin to wrmax @ wthmin. ***/

   rcoord (&wrmin, &wthmin, &xp, &yp, 1, scroll);
   poly [0][0] = (int) xp;
   poly [0][1] = (int) yp;
   rcoord (&wrmax, &wthmin, &xp, &yp, 1, scroll);
   poly [1][0] = (int) xp;
   poly [1][1] = (int) yp;

   /*** wthmin to wthmax @ wrmax. ***/

   i = 1;
   for (it = (int) wthmin + 5; it < (int) wthmax - 4; it += 5)
      {
      i++;
      theta = (float) it;
/*      fprintf (stdout, "kb_wedge> i: %3d   theta: %7.2f\n", i, theta); */
      rcoord (&wrmax, &theta, &xp, &yp, 1, scroll);
      poly [i][0] = (int) xp;
      poly [i][1] = (int) yp;
      }

   /*** wrmax to wrmin @ wthmax. ***/

   i++;
   rcoord (&wrmax, &wthmax, &xp, &yp, 1, scroll);
   poly [i][0] = (int) xp;
   poly [i][1] = (int) yp;
   i++;
   rcoord (&wrmin, &wthmax, &xp, &yp, 1, scroll);
   poly [i][0] = (int) xp;
   poly [i][1] = (int) yp;

   /*** wthmax to wthmin @ wrmin. ***/

   for (it = (int) wthmax - 5; it > (int) wthmin + 4; it -= 5)
      {
      i++;
      theta = (float) it;
/*      fprintf (stdout, "kb_wedge> i: %3d   theta: %7.2f\n", i, theta); */
      rcoord (&wrmin, &theta, &xp, &yp, 1, scroll);
      poly [i][0] = (int) xp;
      poly [i][1] = (int) yp;
      }
   *nver = i + 1;
   fprintf (stdout, "kb_wedge> nver: %d\n", *nver);
     
   fprintf (stderr, "Is the mass integration area acceptable [y/n/q] ?  ");
   fgets (line, maxline, stdin);
   sscanf  (line,  "%s", answer);
   if (strncmp ("n", answer, 1) == 0)  goto boundary;
   if (strncmp ("q", answer, 1) == 0)  return (-1);

   fprintf (pgmlog,
      "Wedge area : %4.2f - %4.2f Rsun & %6.2f - %-6.2f degrees",
          wrmin, wrmax, wthmin, wthmax);

   return (0);
}
