/* blints.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation: unsigned short integer images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language   UNIX 4.2 bsd operating system   VAX-11/750
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * image : unsigned short integer pointer to array containing image.
 * x,y:    floating point pointers to x & y position in image.
 * `blints' returns a floating point value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    float blints();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept unsigned short integer images. 
 * The variable 'npxlin' specifies the number of pixels per horizontal line
 * in the image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 22 September 1987
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

float
blints (image, x, y, npxlin)

unsigned short	*image;		/* Pointer to image array. */
float	x, y;			/* Coordinates of scan point in image. */
int	npxlin;			/* # pixels per horizontal line. */

{
register unsigned short	*ims;
int	ix, iy;
float	fx, fy;
float	f00, f10, f01, f11;
float	fx0, fx1, fxy;

	ims = (unsigned short *) image;

	ix = (int) x;
	iy = (int) y;

	fx = x - (float) ix;
	fy = y - (float) iy;

/* Select 4 nearest neighbor elements in 'image' array. */

	f00 = (float) *(ims +  ix    +  iy*npxlin);
	f10 = (float) *(ims + (ix+1) +  iy*npxlin);
	f01 = (float) *(ims +  ix    + (iy+1)*npxlin);
	f11 = (float) *(ims + (ix+1) + (iy+1)*npxlin);

/* Bilinear interpolation. */

	fx0 = f00 + fx * (f10 - f00);
	fx1 = f01 + fx * (f11 - f01);
	fxy = fx0 + fy * (fx1 - fx0);

	return(fxy);
}
