/* cor4qvid.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a 4-quadrant SMM C/P image oriented so that solar north is UP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Input C/P image must be low resolution, and calibrated.
 * Output image is 640x480 pixels in Sun rasterfile format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 *  2 June 1989: Use Eikonix-digitized images (512x512 byte array).
 *  3 August 1989: Use Eikonix-digitized images (with header).
 *                 Version for sunview.
 * 21 August 1990: adapt for use with C/P 4-quadrant images.
 *                 Rotate C/P image so that solar north is along +y axis.
 * 13 August 1991: adapt for SGI/IRIS.
 *  1 October 1997: output sun rasterfile.
 * 28 October 1998: change frame size from 512x512 to 640x480.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/grscale.h"
#include "/home/stanger/lib/include/cp_color.h"

main (argc, argv)

int argc ;
char *argv [] ;
{

   extern fmfonthandle gl_setfont () ;
   fmfonthandle cfh ;
   unsigned short *rdpmon() ;
   unsigned short *rdmk3() ;
   unsigned short *rdc4q() ;
   unsigned short *rotate_img() ;
   unsigned short *magimg() ;
   unsigned short *merge3s() ;

   int mskimg () ;
   int rescale () ;
   int wdisk () ;

   static unsigned char red [256], green [256], blue [256] ;
   static char namepm[80] = "1711.eikp" ;	/* Pmon image file name. */
   static char namemk[80] = "1708.srq" ;	/* Mark-III image file name. */
   static char namecp[80] = "801050544.c4q" ;	/* C/P image file name. */
   static char cptime[6] = "HH:MM" ;
   static char pmontime[6] = "HH:MM" ;
   static char mk3time[6] = "HH:MM" ;
   static char nampgm [80] ;
   static char namout [80] = "cor4q.img" ;

   unsigned short *pmims ;	/* Pointer to Prominence Monitor image array. */
   unsigned short *mkims ;	/* Pointer to Mark-III image array. */
   unsigned short *cpims ;	/* Pointer to SMM C/P image array. */

   unsigned short *pmimr ;	/* Pointer to Prominence Monitor image array. */
   unsigned short *mkimr ;	/* Pointer to Mark-III image array. */
   unsigned short *cpimr ;	/* Pointer to SMM C/P image array. */

   unsigned short *pmimg ;	/* Pointer to Prominence Monitor image array. */
   unsigned short *img3 ;	/* Pointer to composite image array. */
   unsigned char  *cimg ;	/* Pointer to output image array. */

   int i ;
   int ix, iy, iiy ;
   int iix ;
   int lfull = 1 ;
   int lbottom = 0 ;
   int nbypx = 2 ;
   int wx = 640, wy = 480 ;
   int npximg ;
   int xl = 632 ;	/* GL window position & size (640x480). */
   int yl = 512 ;
   int xu = 1271 ;
   int yu = 991 ;

   int cpdx = 448, cpdy = 448 ;
   int cpnx = 400, cpny = 400 ;
   int cpcx, cpcy, icpdsk, icprad ;

   static int xgs = 200, ygs = 40 ;
   static int xs = 0, ys = 0 ;
   static int ithresh = 175 ;
   static int level = 0 ;
   static int pmmin1 = 0, pmmax1 = 250, pmmin2 = 0, pmmax2 = 250 ;
   static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250 ;
   static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250 ;
   static int dsplo = 0, dsphi = 249 ;
   int sroll ;

   /* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

   static float cpcxnew[] = {  7., 100., 225., 398., 441., 348., 225., 50., 223.5} ;
   static float cpcynew[] = {225., 348., 441., 398., 258., 100.,   7., 50., 223.5} ;
   static float fcpcx, fcpcy ;
   static float cpfx, cpfy, mkfx, mkfy, pmfx, pmfy ;
   static float cprot = 43.0 ;		/* Angle of C/P north w.r.t. +Y axis. */
   static float fx, fy ;
   static float pmfac = 1.0 ;
   static float pmdsk = 0.99,  pmrad = 1.5 ;
   static float mkdsk = 1.08, mkrad = 1.8 ;
   static float cpdsk = 1.5,  cprad1 = 1.5, cprad2 = 4.15 ;
   static float pmang =  0.0, mkang =  0.0, cpang = 0.0 ;
   static float pmpxrs = 117.5, mkpxrs = 104.4435, cppxrs = 80.9877 ;
   static float impxrs = 54.0 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* Request input information. */

   if (argc == 2)
      {
      strcpy (namecp, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s [cp_radius] cp_filename\n", argv [0]) ;
      exit (0) ;
      }

   strcpy (nampgm, argv [0]) ;

   /*** Initialize SGI Graphics Library. ***/

   foreground () ;
   prefposition (xl, xu, yl, yu) ;
   winopen (nampgm) ;
   gl_wclear (BLACK) ;
   onemap () ;			/* Set single-map mode for color table. */
   get_cmap () ;		/* Save color map. */

   multimap () ;		/* Set multi-map mode for color table. */
   gconfig () ;
   setmap (2) ;			/* Select color map #2. */

   fminit () ;			/* Initialize SGI font manager. */

   /*** Read SMM C/P image file. ***/

   printf ("Read C/P image: %s\n", namecp) ;
   cpims = rdc4q (namecp, &cpdx, &cpdy, &cpcx, &cpcy) ;

   sprintf (namout, "%02d%03d%02d%02d", H.year, H.doy, H.hour, H.minute) ;
   fcpcx = cpcxnew[H.ism] ;
   fcpcy = cpcynew[H.ism] ;
   cppxrs = H.pixrs ;
   icprad = (int) (cprad1 * cppxrs) ;
   icpdsk = (int) (cpdsk * cppxrs) ;
   printf ("fcpcx: %7.2f   fcpcy: %7.2f\n", fcpcx, fcpcy) ;

   /*** Rescale intensity levels. ***/

   printf ("Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2) ;
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, dsplo, dsphi-1) ;

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Write black disk over occulting disk. ***/

   printf ("Write disk into C/P image... cpcx: %3d   cpcy: %3d   icprad: %3d\n",
            cpcx, cpcy, icprad) ;
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level) ;

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Rotate image so that solar north is UP. ***/

   cpang = cprot - (float) scroll ;
   scroll -= cprot ;
   printf ("Rotate SMM C/P image . . . cpang: %7.2f\n", cpang) ;
   cpimr = rotate_img (cpims, cpdx, cpdy, cpcx, cpcy, cpang) ;
   scroll = 0 ;
   free (cpims) ;
   imgwgl (cpimr, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Mask off image beyond "cprad2" radii. ***/

   impxrs = 54.0 ;		  /* Output image scale size (pixels/Rsun).*/
   impxrs = (cpnx / 2.0) / 4.15 ;	/* FOV = 4.15 Rsun */
   printf ("impxrs: %7.2f\n", impxrs) ;
   cprad2 = (cpnx / impxrs) / 2.0 ;
   icprad = (int) (cprad2 * cppxrs) ;
   printf ("Mask SMM C/P image beyond %7.2f Rsun.\n", cprad2) ;
   mskimg (cpimr, 0, 0, cpdx, cpdy, cpcx, cpcy, icprad) ;
   imgwgl (cpimr, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Magnify image. ***/

   printf ("Magnify C/P image.\n") ;
   cpfx = cpfy = impxrs / cppxrs ; /* Set FOV to 8.30 Rsun. */
   cpims = (unsigned short *) malloc (cpnx * cpny * sizeof(short)) ;

   /*** Crop image by selecting centered cpnx*cpny array. ***/

/*
   for (iy = 0; iy < cpny; iy++)
      {
      iiy = iy + (cpdy - cpny) / 2 ;
      for (ix = 0; ix < cpnx; ix++)
	 {
	 iix = ix + (cpdx - cpnx) / 2 ;
	 *(cpims + ix + iy*cpnx) = *(cpimr + iix + iiy*cpdx) ;
	 }
      }
 */

   printf ("xcen/ycen: %7.2f %7.2f\n", xcen, ycen) ;
   xcen -= (cpdx - cpnx) / 2 ;
   ycen -= (cpdy - cpny) / 2 ;
   fcpcx = xcen ;
   fcpcy = ycen ;
   ycen = cpny - 1 - ycen ;
   printf ("xcen/ycen: %7.2f %7.2f\n", xcen, ycen) ;
   expcon (cpimr, cpdx, cpdy, cpims, cpnx, cpny, cpfx, cpfy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy) ;
   free (cpimr) ;

   imgwgl (cpims, nbypix, xs, ys, cpnx, cpny, "ru") ;
   sleep (1) ;

/******************************************************************************/

   /*** Display image with annotation. ***/

   gl_wclear (BLACK) ;
   ldisk = 1 ;

   gMINCOL = xgs ;
   gMAXCOL = xgs + cpnx - 1 ;
   gMINROW = ygs ;
   gMAXROW = ygs + cpny - 1 ;

   printf ("gMINCOL/gMAXCOL: %4d %4d  gMINROW/gMAXROW: %4d %4d\n",
	    gMINCOL, gMAXCOL, gMINROW, gMAXROW) ;

   img3 = cpims ;
   imgwgl (img3, nbypx, xgs, ygs, cpnx, cpny, "ru") ;/* Write merged image.*/
   free (img3) ;


   fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy) ;
   xcen = fcpcx ;
   ycen = cpny - 1 - fcpcy ;
   cpfx = cpfy = impxrs / cppxrs ;
   pixrs = pixrs * cpfx ;
   printf ("pixrs: %7.2f\n", pixrs) ;

   glcp_label_vid (lfull, lbottom, cp_grey, cp_green, cp_yellow) ;
   glcp_annot_vid (lbottom, cp_red, cp_grey, cp_green, cp_yellow) ;

   scroll = -43 ;
   glcp_north ((int) xcen, (int) ycen, 0.1, 0.3, cp_yellow) ;
   glcp_scalers (0, cp_yellow) ;
   glcp_suncir (0.0, 0.0, 0.0, 90.0, 1.0, 1.0, 0.2, 30.0, cp_yellow) ;
/*   glcp_suncir (0.6, 1.2, 0.2, 90.0, 1.0, 1.0, 0.2, 30.0, cp_yellow) ; */
/*   glcp_suncir (0.4, 1.5, 0.1, 90.0, 1.0, 1.5, 0.2, 30.0, cp_yellow) ; */
   sleep (1) ;

   /*** Read image from frame buffer. ***/

   npximg = wx * wy ;
   img3 = (unsigned short *) malloc (npximg * sizeof (short)) ;
   cimg = (unsigned char *) malloc (npximg) ;
   rectread (0, 0, wx-1, wy-1, img3) ;

   /*** Flip image top to bottom for Sun Rasterfile. ***/

   for (iy = 0 ; iy < wy ; iy++)
      {
      iiy = wy - 1 - iy ;
      for (ix = 0 ; ix < wx ; ix++)
	 {
	 *(cimg + ix + iy*wx) = (unsigned char) (*(img3 + ix + iiy*wx) & 0377) ;
	 }
      }

   /*** Read color table. ***/

   get_scmap (512, 256, red, green, blue) ;

   /*** Write image to disc as a sun rasterfile. ***/

   sprintf (namout, "%s.sun", namecp) ;
   bytsrf (namout, cimg, wx, wy, red, green, blue) ;

   fprintf (stdout, "\n") ;
   gexit () ;
   exit (0) ;
}
