/* cp4q.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a 4-quadrant SMM C/P image oriented so that solar north is UP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Input C/P image must be low resolution, and calibrated.
 * Output image is in Sun rasterfile format.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 *  2 June 1989: Use Eikonix-digitized images (512x512 byte array).
 *  3 August 1989: Use Eikonix-digitized images (with header).
 *                 Version for sunview.
 * 21 August 1990: adapt for use with C/P 4-quadrant images.
 *                 Rotate C/P image so that solar north is along +y axis.
 * 13 August 1991: adapt for SGI/IRIS.
 *  1 October 1997: output sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/grscale.h"
#include "/home/stanger/lib/include/cp_color.h"

main (argc, argv)

int argc;
char *argv [];
{

   extern fmfonthandle gl_setfont ();
   fmfonthandle cfh;
   unsigned short *rdpmon();
   unsigned short *rdmk3();
   unsigned short *rdc4q();
   unsigned short *rotate_img();
   unsigned short *magimg();
   unsigned short *merge3s();

   int mskimg ();
   int rescale ();
   int wdisk ();

   static unsigned char red [256], green [256], blue [256];
   static char namepm[80] = "1711.eikp";	/* Pmon image file name. */
   static char namemk[80] = "1708.srq";	/* Mark-III image file name. */
   static char namecp[80] = "801050544.c4q";	/* C/P image file name. */
   static char cptime[6] = "HH:MM";
   static char pmontime[6] = "HH:MM";
   static char mk3time[6] = "HH:MM";
   static char nampgm [80];
   static char namout [80] = "cor4q.img";

   unsigned short *pmims;	/* Pointer to Prominence Monitor image array. */
   unsigned short *mkims;	/* Pointer to Mark-III image array. */
   unsigned short *cpims;	/* Pointer to SMM C/P image array. */

   unsigned short *pmimr;	/* Pointer to Prominence Monitor image array. */
   unsigned short *mkimr;	/* Pointer to Mark-III image array. */
   unsigned short *cpimr;	/* Pointer to SMM C/P image array. */

   unsigned short *pmimg;	/* Pointer to Prominence Monitor image array. */
   unsigned short *img3;	/* Pointer to composite image array. */
   unsigned char  *cimg;	/* Pointer to output image array. */

   int i;
   int ix, iy, iiy ;
   int lfull = 1;
   int lbottom = 0;
   int nbypx = 2;
   int nx = 512, ny = 512 ;
   int npximg;
   int xl = 760;	/* Specification of GL window position & size. */
   int yl = 480;
   int xu = 1271;
   int yu = 991;

   static    int ngr = 0;
   static    int imgch0 = 0, imgch1 = 1, imgch2 = 2, imgch3 = 3;

   int pmdx, pmdy, pmnx, pmny;
   int pmcx, pmcy, ipmdsk, ipmrad;

   int mkdx, mkdy, mknx, mkny;
   int mkcx, mkcy, imkdsk, imkrad;

   int cpdx = 448, cpdy = 448, cpnx = 448, cpny = 448;
   int cpcx, cpcy, icpdsk, icprad;

   static int xgs = 63, ygs = 63;
   static int xs = 0, ys = 0;
   static int ithresh = 175;
   static int level = 0;
   static int pmmin1 = 0, pmmax1 = 250, pmmin2 = 0, pmmax2 = 250;
   static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250;
   static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250;
   static int grnlo = 0, grnhi = 249;
   int sroll;

   /* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

   static float cpcxnew[] = {  7., 100., 225., 398., 441., 348., 225., 50., 223.5};
   static float cpcynew[] = {225., 348., 441., 398., 258., 100.,   7., 50., 223.5};
   static float fcpcx, fcpcy;
   static float cpfx, cpfy, mkfx, mkfy, pmfx, pmfy;
   static float cprot = 43.0;		/* Angle of C/P north w.r.t. +Y axis. */
   static float fx, fy;
   static float pmfac = 1.0;
   static float pmdsk = 0.99,  pmrad = 1.5;
   static float mkdsk = 1.08, mkrad = 1.8;
   static float cpdsk = 1.5,  cprad1 = 1.5, cprad2 = 4.15;
   static float pmang =  0.0, mkang =  0.0, cpang = 0.0;
   static float pmpxrs = 117.5, mkpxrs = 104.4435, cppxrs = 80.9877;
   static float impxrs = 54.0;
   mkpxrs = (511.0 / (2.0 * (1.1224 + 127.0 * 0.0104244)));/* scale 13 Dec 90 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* Request input information. */

   if (argc == 2)
      {
      strcpy (namecp, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [cp_radius] cp_filename\n", argv [0]);
      exit (0);
      }

/*
 *   fprintf (stderr, "Enter file name of SMM C/P image: ");
 *   fscanf  (stdin, "%s", namecp);
 */
/*
 *   fprintf (stderr, "Enter minimum radius for SMM C/P image: ");
 *   fscanf  (stdin, "%f", &cprad);
 */
/*
 *   fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ");
 *   fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2);
 */

   strcpy (nampgm, argv [0]);

   /*** Initialize SGI Graphics Library. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen (nampgm);
   gl_wclear (BLACK);
   onemap ();			/* Set single-map mode for color table. */
   get_cmap ();			/* Save color map. */

   multimap ();			/* Set multi-map mode for color table. */
   gconfig ();
   setmap (2);			/* Select color map #2. */

   fminit ();			/* Initialize SGI font manager. */

/*
 * Read SMM C/P image file.
 * Write black disk.
 * Magnify image.
 */

   printf ("Read C/P image: %s\n", namecp);
   cpims = rdc4q (namecp, &cpdx, &cpdy, &cpcx, &cpcy);

   sprintf (namout, "%02d%03d%02d%02d", H.year, H.doy, H.hour, H.minute) ;
   fcpcx = cpcxnew[H.ism];
   fcpcy = cpcynew[H.ism];
   cppxrs = H.pixrs;
   icprad = (int) (cprad1 * cppxrs);
   icpdsk = (int) (cpdsk * cppxrs);
   printf ("fcpcx: %7.2f   fcpcy: %7.2f\n", fcpcx, fcpcy);

   printf ("Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2);
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1);

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   printf ("Write disk into C/P image... cpcx: %3d   cpcy: %3d   icprad: %3d\n",
            cpcx, cpcy, icprad);
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level);

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   cpang = cprot - (float) scroll;
   scroll -= cprot;
   printf ("Rotate SMM C/P image . . . cpang: %7.2f\n", cpang);
   cpimr = rotate_img (cpims, cpdx, cpdy, cpcx, cpcy, cpang);
   free (cpims);
   imgwgl (cpimr, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   impxrs = 54.0;		  /* Composite image scale size (pixels/Rsun).*/
   cprad2 = (448.0 / impxrs) / 2.0;
   icprad = (int) (cprad2 * cppxrs);
   printf ("Mask SMM C/P image beyond %7.2f Rsun.\n", cprad2);
   mskimg (cpimr, 0, 0, cpdx, cpdy, cpcx, cpcy, icprad);
   imgwgl (cpimr, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   printf ("Magnify C/P image.\n");
   cpfx = cpfy = impxrs / cppxrs; /* Set FOV to 8.30 Rsun. */
   cpims = (unsigned short *) malloc (cpdx * cpdy * sizeof(short));
   expcon (cpimr, cpdx, cpdy, cpims, cpdx, cpdy, cpfx, cpfy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy);
   free (cpimr);

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

/******************************************************************************/

   img3 = cpims;

   /*** Display composite image with annotation. ***/

   gl_wclear (BLACK);
   ldisk = 1;
   glcp_label (lfull, lbottom, cp_grey, cp_green, cp_yellow) ;

   glcp_annot (lbottom, cp_red, cp_grey, cp_green, cp_yellow) ;

   imgwgl (img3, nbypx, xgs, ygs, cpnx, cpny, "ru");/* Write merged image.*/
   free (img3);


/*
 *   CharterBT-Black, 16x17.
 *
 *   cfh = gl_setfont ("CharterBT-Black", 12);
 *   gl_wstr ("HAO/SMM CORONAGRAPH/POLARIMETER", 63, 38, cp_green);
 */
/*
 *   gl_wstr ("MLSO MARK-III K-CORONAMETER    ", 63, 20, cp_green);
 *   gl_wstr ("MLSO PROMINENCE MONITOR        ", 63,  2, cp_green);
 *   gl_wstr (pmontime, 444,  2, cp_red);
 *   gl_wstr (mk3time,  444, 20, cp_red);
 */
/*
 *   sprintf (cptime, "%02d:%02d", H.hour, H.minute);
 *   gl_wstr (cptime,   444, 38, cp_red);
 */

   fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
   xcen = fcpcx;
   ycen = 447 - fcpcy;
   cpfx = cpfy = impxrs / cppxrs; /* Set FOV to 8.30 Rsun. */
   pixrs = pixrs * cpfx;
   glcp_north ((int) fcpcx, (int) fcpcy, 0.1, 0.3, cp_yellow);
   glcp_scalers (0, cp_yellow);
   glcp_suncir (0.4, 1.5, 0.1, 90.0, 1.0, 1.5, 0.2, 30.0, cp_yellow);

   
   /*** Read image from frame buffer. ***/

   npximg = 512 * 512;
   img3 = (unsigned short *) malloc (npximg * sizeof (short));
   cimg = (unsigned char *) malloc (npximg);
   rectread (0, 0, 511, 511, img3);

   /*** Flip image top to bottom for Sun Rasterfile. ***/

   for (iy = 0; iy < ny; iy++)
      {
      iiy = 511 - iy ;
      for (ix = 0; ix < nx; ix++)
	 {
	 *(cimg + ix + iy*nx) = (unsigned char) (*(img3 + ix + iiy*nx) & 0377) ;
	 }
      }

   /*** Read color table. ***/

   get_scmap (512, 256, red, green, blue);

   /*** Write image to disc as a sun rasterfile. ***/

   sprintf (namout, "%s.sun", namecp) ;
   bytsrf (namout, cimg, 512, 512, red, green, blue) ;

   fprintf (stdout, "\n") ;
   gexit ();
   exit (0);
}
