/* cprot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adjust for S/C roll by rotating C/P image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C/P image must be low resolution, and calibrated.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 12 July 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#include "/home/riker/stanger/lib/include/grscale.h"

main (argc, argv)

int argc;
char *argv [];
{

   FILE *FD ;
   unsigned char *rdfimg ();
   unsigned char *rotimg ();

   unsigned char *cpimg, *cpimr ;

   static char namecp [80];		/* C/P image file name. */
   static char nampgm [80];
   static char namout [80];

   static char namsmf [8][3] = {"sw", "ss", "se", "ee", "ne", "nn", "nw", "ww"};

   int cpcx, cpcy ;
   int cpdx, cpdy ;
   int i ;
   int idot ;
   int is ;
   int jsm ;
   int nbypx ;
   int nrechd ;
   int hdrsiz ;
   int imgsiz ;

   static float cpang = 0.0;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   /*** Get command line argument. ***/

   if (argc == 2)
      {
      strcpy (namecp, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s cp_filename\n", argv [0]);
      exit (0);
      }

   strcpy (nampgm, argv [0]);

   /*** Read SMM C/P image file. ***/

   printf ("Read C/P image.\n");
   cpimg = rdfimg (namecp);

   if (nbypix == 4)
      nrechd = 1;
   else
      nrechd = 2;
   hdrsiz = nrechd * npixl * nbypix ;
   imgsiz = (npixl * npixl * nbypix);

   nbypx = H.nbypix ;
   cpdx = cpdy = npix ;
   cpcx = (int) H.xcen / nres ;
   cpcy = (int) H.ycen / nres ;

   cpang = - (float) scroll;
   printf ("Rotate SMM C/P image . . . cpang: %f\n", cpang);
   cpimr = rotimg (cpimg + hdrsiz, nbypx, cpdx, cpdy, cpcx, cpcy, cpang);

   /*** Change sector mirror telemetry value in housekeeping. ***/

   jsm = 2 * H.ism + 1 ;
   printf ("hk [14]: %3o   jsm: %3d\n", *(cpimg + 14), jsm) ;
   is = (H.ism + scroll / 45) % 8 ;

   *(cpimg + 14) &= 0303 ;		/* Mask off sector bits. */
   printf ("hk [14]: %3o   jsm: %3d\n", *(cpimg + 14), jsm) ;

   jsm = 2 * is + 1 ;
   *(cpimg + 14) |= (jsm << 2) ;
   printf ("hk [14]: %3o   jsm: %3d\n", *(cpimg + 14), jsm) ;
   printf ("is: %3d   jsm: %3d\n", is, jsm) ;

   /*** Change S/C roll to zero. ***/

   *(cpimg + 11) &= 0360 ;		/* Mask off roll bits. */

   if (H.year == 80)
      *(cpimg + 12) &= 0000 ;		/* Mask off roll bits. */
   else
      *(cpimg + 12) &= 0077 ;		/* Mask off roll bits. */

   *(cpimg + 88) = 0 ;
   *(cpimg + 89) = 0 ;

   /*** Create output file name. ***/

   idot = -1 ;
   for (i = 0; i < strlen (namecp); i++)
      {
      if (namecp [i] == '.')  idot = i ;
      }

   strncpy (namout, namecp, idot + 1) ;
   namout [idot + 1] = 'r' ;
   strcat (namout, &namsmf [is][0]) ;

   FD = fopen (namout, "w") ;
   fwrite (cpimg, 1, hdrsiz, FD) ;
   fwrite (cpimr, 1, imgsiz, FD) ;
   fclose (FD) ;

   exit (0);
}
