/* cpvf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *  Display an SMM C/P image ('filename') on Grinnell channel 'ch'.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Created by adapting recycled parts from 'seesmm.c' (Rose Reynolds)
 *   and 'ied_disp.c' (Kevin Cole).
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * NOTE: image intensity levels 251 through 254 are used for annotation
 *       and graphics.  Their colors will depend upon what hues are assigned
 *       to those intensity levels.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 19 March 1985
 * 26 August 1985: Send 'wsolid' command prior to 'wbox'.
 * 17 October 1985: Allow photometric rescaling for raw images.
 *  1 November 1985: Draw pointer which is correct for all roll angles.
 *  7 November 1985: Change pointer: circle, center dot, north triangle.
 *  5 September 1986: Add option to eliminate sun circle, logo, & color bar.
 * 15 January 1987: Add ability to display floating-point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/u/rose/grin2/grin.h"
#include "/u/smm/lib/src/cp/cplib.h"
#include "/u/smm/lib/src/grin/grscale.ext"

#define white	255
#define color4	254	/* red. */
#define color3	253	/* green. */
#define color2	252	/* blue. */
#define color1	251	/* gray. */
#define color0  250	/* yellow. */
#define black	0

int SMM_imgno,SMM_dim;
int gpixrs;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

cpview (filename, imgch, linit)

char *filename;		/* Pointer to filename array. */
int imgch;		/* Grinnell image channel number (0,1,2,3). */
int linit;		/* Image label flag; ON if =1. */

{
  extern int scalers ();	/* Function to draw solar radius scale. */
  extern int crosshair ();	/* Function to draw "+" at sun center. */
  extern int suncir ();		/* Function to draw circle at 1.0 Rsun. */
  extern int smm_label ();	/* Function to label SMM C/P frame. */
  extern int smm_annot ();	/* Function to write SMM C/P annotation. */
  extern int wdisk ();		/* Function to write black disk. */
  extern int wdiskb ();		/* Function to write black disk. */
  extern int srevbyt ();	/* Reverses bytes (short int images)*/
  extern int GRN;		/* Grinnell usage flag: =0 off, =1 on. */
  int cx, cy, radius;
  int level;
  int nx = 448, ny = 448;
  int fin, x, y, dx, len;
  char *image;

   if (!GRN)
      {
      fprintf (stderr, "cpview> Grinnell option is not selected.\n");
      rwech (DEF);
      return (-1);
      }
   zoom (1);
   zcursor (OFF);

   if ( (fin = smm_open (filename)) == -1 )	/* Open C/P image. */
      {
      fprintf (stderr, "cpview> Unable to open file: %s\n", filename);
      rwech (DEF);
      return (-2);
      }

   disp (imgch);		   /* Display Grinnell image channel 'imgch'. */
   if (imgch != DEF ) rwech (imgch); /* Set read/write/erase channel. */
   if (linit)  gr_erase (imgch);     /* Erase Grinnell image channel 'imgch'. */

/*** Set origin coordinates of image. ***/

   switch (SMM_imgno)
      {
      case 1: x=63;   y=510; dx=448; break;
      case 2: x=63;   y=510; dx=336; break;
      case 3: x=175;  y=510; dx=336; break;
      case 4: x=63;   y=398; dx=336; break;
      case 5: x=175;  y=398; dx=336; break;
      case 6: x=63;   y=510; dx=224; break;
      case 7: x=175;  y=510; dx=224; break;
      case 8: x=287;  y=510; dx=224; break;
      case 9: x=63;   y=398; dx=224; break;
      case 10: x=175; y=398; dx=224; break;
      case 11: x=287; y=398; dx=224; break;
      case 12: x=63;  y=286; dx=224; break;
      case 13: x=175; y=286; dx=224; break;
      case 14: x=287; y=286; dx=224; break;
      case 15: x=63;  y=510; dx=112; break;
      case 16: x=175; y=510; dx=112; break;
      case 17: x=287; y=510; dx=112; break;
      case 18: x=399; y=510; dx=112; break;
      case 19: x=63;  y=398; dx=112; break;
      case 20: x=175; y=398; dx=112; break;
      case 21: x=287; y=398; dx=112; break;
      case 22: x=399; y=398; dx=112; break;
      case 23: x=63;  y=286; dx=112; break;
      case 24: x=175; y=286; dx=112; break;
      case 25: x=287; y=286; dx=112; break;
      case 26: x=399; y=286; dx=112; break;
      case 27: x=63;  y=174; dx=112; break;
      case 28: x=175; y=174; dx=112; break;
      case 29: x=287; y=174; dx=112; break;
      case 30: x=399; y=174; dx=112; break;
      }

/*** Read image into memory. ***/

   len = SMM_dim * SMM_dim * H.nbypix;
   image = malloc ( len );
   if (read (fin,image,len) <= 0 )
      {
      fprintf (stderr,"cpview> read error.");
      rwech (DEF);
      return (-3);
      }
   close ( fin );

/*
 * Clear area below image.
 * If image is high resolution, convert to low resolution.
 */

   printf ("nbypix: %d   bytord: %d\n", nbypix, bytord);
   if (H.ih < 15)  wsolid (0, 63, 63, 448, 448, "ru");
   if( H.ires == 0 ) smm_avg (image);	/* Convert image to low res. */

/*** Write disk in occulting disk area. ***/

   cx = (int) xcen / nres;
   cy = (int) ycen / nres;
   level = 0;
   gpixrs = pixrs / nres;
   radius = (int) (1.4 * gpixrs);
   printf ("cx: %3d   cy: %3d   xcen: %3d   ycen: %3d   nres: %d  radius: %d\n",
           cx, cy, (int) xcen, (int) ycen, nres, radius);

/*** Display image in Grinnell channel 'imgch'. ***/

   if ( H.nbypix == 1 )
      {
      wboxb (image, x, y, dx, dx, "rd");
      wdiskb (image, nx, ny, cx, cy, radius, level);
      }
   else if (H.nbypix == 2)
      {	
      oldmin =  0;  oldmax =  498;
      newmin =  0;  newmax =  249;
      clipmin = 0;  clipmax = 249;
      set_resc_parms (oldmin, oldmax, newmin, newmax, clipmin, clipmax);
      if (H.ipw != 0)  set_resc_parms (0, 198, 0, 249, 0, 249);
      if (lscale == 1) set_resc_parms (oldmin, oldmax, newmin, newmax,
                                       clipmin, clipmax);
      if (idtype == 3 && bytord == 1 && H.ires == 1)
	 srevbyt (image, nbypix, dx*dx);
      gr_resc (image, dx*dx);
      wbox (image, x, y, dx, dx, "rd");
      wdisk (image, nx, ny, cx, cy, radius, level);
      }
   else if (H.nbypix == 4)
      dfimg (image, x, y, dx);
   else
      {
      fprintf (stderr, "Invalid image . . . nbypix: %d\n", nbypix);
      return (-4);
      }
   free (image);

/*** Compute disk center in Grinnell coordinates. ***/

   cx = (int) xcen / nres + gMINROW;
   cy = gMAXCOL - ((int) ycen / nres);
   printf ("cx: %3d   cy: %3d   xcen: %3d   ycen: %3d   nres: %d\n",
           cx, cy, (int) xcen, (int) ycen, nres);

/*** Draw solar radius scale. ***/
/*** Draw pointer to mark center of sun & indicate north direction. ***/

   nres = 1;
   if (H.ires == 0)  nres = 2;
   x = (int) xcen / nres + gMINROW;
   y = gMAXCOL - ((int) ycen / nres);
   if (H.icm == 1 && ldisk == 0)  
      {
      crosshair (cx, cy, 10, 10, color0);
      scalers (color0);
      suncir (color0);
      }

/*** Write annotation labels if linit = 1. ***/
/*** Write annotation information. ***/

   if (linit)  smm_label(linit, color1);	/* Add labels for annotation. */
   smm_annot (color4);				/* Annotate image. */
   rwech (DEF);
   return (0);
}


/* smm_open (filename)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Open SMM C/P image file &  decommutate selected items from housekeeping.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (Extracted from 'smmsubs.c'.)
 * 29 August 1984
 * 1 November 1984: set 0-255 -> 0-255 scaling for calibrated polaroid images.
 * 9 November 1984: use hskp.h structure and cphskp.c function.
 * 15 January 1987: adapt to handle floating-point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

smm_open (filename)

char *filename;

{
int cphskp ();			/* Function to decode housekeeping. */
int fdimg;
int nrechd;
char *it;
long offset;

   cphskp (filename);	/* Decommutate selected housekeeping items. */

/*** Open image disk file. ***/

   if ((fdimg = open (filename,0)) == -1)
      {
      fprintf (stderr, "smm_open> can't open file: %s\n", filename);
      return (-1);
      }

   if ( smm_dim(H.ih) != smm_dim(H.iv) )
      {
      SMM_dim = 0;
      SMM_imgno = 0;
      return (-2);
      }
   else
      {
      SMM_dim = smm_dim(H.ih);
      if ( H.ires == 1 ) SMM_dim = SMM_dim /2;

      switch (H.iv)
         {
         case 017: SMM_imgno = 1; break;
         case 016: switch (H.ih)
	    {
            case 016: SMM_imgno = 2; break;
            case 07:  SMM_imgno = 3; break;
            }
         break;
         case 07: switch (H.ih)
	    {
            case 016: SMM_imgno = 4; break;
            case 07:  SMM_imgno = 5; break;
            }
         break;
         case 014: switch (H.ih)
	    {
            case 014: SMM_imgno = 6; break;
            case 06:  SMM_imgno = 7; break;
            case 03:  SMM_imgno = 8; break;
            }
         break;
         case 06:  switch (H.ih)
	    {
            case 014: SMM_imgno = 9; break;
            case 06:  SMM_imgno = 10; break;
            case 03:  SMM_imgno = 11; break;
            }
         break;
         case 03:  switch (H.ih)
	    {
            case 014: SMM_imgno = 12; break;
            case 06:  SMM_imgno = 13; break;
            case 03:  SMM_imgno = 14; break;
            }
         break;
         case 010: switch (H.ih)
	    {
            case 010: SMM_imgno = 15; break;
            case 04:  SMM_imgno = 16; break;
            case 02:  SMM_imgno = 17; break;
            case 01:  SMM_imgno = 18; break;
            }
         break;
         case 04:  switch (H.ih)
	    {
            case 010: SMM_imgno = 19; break;
            case 04:  SMM_imgno = 20; break;
            case 02:  SMM_imgno = 21; break;
            case 01:  SMM_imgno = 22; break;
            }
         break;
         case 02:  switch (H.ih)
	    {
            case 010: SMM_imgno = 23; break;
            case 04:  SMM_imgno = 24; break;
            case 02:  SMM_imgno = 25; break;
            case 01:  SMM_imgno = 26; break;
            }
         break;
         case 01:  switch (H.ih)
	    {
            case 010: SMM_imgno = 27; break;
            case 04:  SMM_imgno = 28; break;
            case 02:  SMM_imgno = 29; break;
            case 01:  SMM_imgno = 30; break;
            }
         break;
         default:  SMM_imgno = 0;  return (-3);  break;
         }
      }

   if( (it=rindex (filename,'.'))==(char *) 0)
      {
      H.nbypix = -1;
      SMM_dim = 0;
      SMM_imgno = 0;
      return (-4);
      }
   it++;

/*** Position file pointer at beginning of image data (at end of bit-map). ***/

   printf ("idtype: %d  nbypix: %d\n", H.idtype, H.nbypix);
   if (nbypix == 1 || nbypix == 2)
      nrechd = 2;
   else
      nrechd = 1;
   offset = (long) SMM_dim * H.nbypix * 2;
   if ( lseek (fdimg, offset, 0) == -1 )
      {
      fprintf (stderr,"smm_hskp> seek error.\n");
      return (-5);
      }
   return (fdimg);
}


/* smm_avg (image)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert an SMM C/P image from high to low resolution.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c'.)
 * 29 August 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

smm_avg (image)

char *image;	/* Pointer to image array. */

{
register char *bl, *bl1, *rbim;
register u_short  *il, *il1, *rim;
int i,j,size;
int temp;

/*** One byte per pixel. ***/

   if ( H.nbypix == 1 )
      {
      bl = image;
      rbim = image;
      for (i=0; i<SMM_dim; i+=2)
         {
         bl1 = bl + SMM_dim;
         for (j=0; j<SMM_dim; j+=2)
            {
            temp = ((*bl++)&0377) + ((*bl1++)&0377) +
                   ((*bl++)&0377) + ((*bl1++)&0377);
            *rbim++ =  temp/4 ;
            }
         bl = bl1;
         }
      return( 0 );
      }

   else 

/*** 2 bytes per pixel. ***/

      {
      size = SMM_dim * SMM_dim;
      if (bytord == 1)  srevbyt (image, nbypix, size);
      il = (u_short *) image;
      rim = (u_short *) image;
      for (i=0; i<SMM_dim; i+=2)
         {
         il1 = il + SMM_dim;
         for (j=0; j<SMM_dim; j+=2)
            {
            *rim++ = (*il++ + *il++ + *il1++ + *il1++)/4;
            }
         il = il1;
         }
      return ( 0 );
      }
}


/* smm_label (linit, subch)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label SMM C/P image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Kevin Cole   HAO/NCAR   SMM C/P   (extracted from 'ied_disp.c'.)
 * Adapted by Andrew L. Stanger
 * 14 February 1985
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

smm_label (linit, subch)

int linit;	/* Initialization flag. */
int subch;	/* Grinnell sub-channel mask. */

{
   short int cbar[512];		/* Color bar array. */
   int charheight, startrow, startcol, endcol, i;
   int colormax = 255;			/* Maximum color value. */
   int barlength = 512, barwidth = 12;	/* color bar dimensions. */
   int ppc = 2;	/* # pixels per color in color bar. */

   if (!GRN) return (0);	/* If image display option is off. */
   doublew (OFF);   doubleh (OFF);

   startrow = 17 + barwidth;
   startcol = 0;
   endcol = startcol + barlength - 1;
   charheight = 43;
   scmask(subch);		/* Set sub-channel mask. */

/*** Draw box around image window. ***/

   if (linit)
      {
      scmask(color1);		/* Set sub-channel mask. */
      wvec ( 62,  62,  62, 511);
      wvec ( 62, 511, 511, 511);
      wvec (511, 511, 511,  62);
      wvec (511,  62,  62,  62);

/*** Write annotation labels into Grinnell channel. ***/

      scmask (subch);		/* Set sub-channel mask. */
      wstr ("DATE",       1, 496,              7, 0);
      wstr ("TIME",       1, 496-1*charheight, 7, 0);
      wstr ("FILTER",     1, 475-3*charheight, 7, 0);
      wstr ("POLAROID",   1, 475-4*charheight, 7, 0);
      wstr ("SECTOR",     1, 475-5*charheight, 7, 0);
      wstr ("EXPOSURE",   1, 475-6*charheight, 7, 0);
      wstr ("RESOLUTION", 1, 475-7*charheight, 6, 0);
      }

   if (H.icm == 2)
      wstr ("CAL MIRROR", 1, 475-8*charheight, 6, 0);
   else
      wstr ("ROLL      ", 1, 475-8*charheight, 6, 0);

   if (linit)
      {
      wstr ("FRAME  ", 1, 475-9*charheight,7, 0);

/*** Write HAO logo & color bar only if 'ldisk = 1'. ***/

      if (ldisk == 1)  return (0);

      scmask (color3);		/* Set sub-channel mask. */
      doublew (ON);  doubleh (OFF);
      /*	revback (ON); */
      wstr (" HAO/SMM   CORONAGRAPH / POLARIMETER ", 64, 34, 6, 0);

      scmask (subch);		/* Set sub-channel mask. */
      doublew (OFF);  doubleh (OFF);  revback(OFF);

/*** Add annotation for the color bar ***/

      wstr ("0"  , (startcol+1),          1, 7, 0);
      wstr ("64" , (startcol+64*ppc-7),   1, 7, 0);
      wstr ("128", (startcol+128*ppc-10), 1, 7, 0);
      wstr ("192", (startcol+192*ppc-10), 1, 7, 0);
      wstr ("255", (endcol-21),           1, 7, 0);

/*** Add tick marks to color bar ***/

      wvec (startcol,         11, (startcol),         16);
      wvec (startcol+32*ppc , 11, (startcol+32*ppc),  16);
      wvec (startcol+64*ppc , 11, (startcol+64*ppc),  16);
      wvec (startcol+96*ppc , 11, (startcol+96*ppc),  16);
      wvec (startcol+128*ppc, 11, (startcol+128*ppc), 16);
      wvec (startcol+160*ppc, 11, (startcol+160*ppc), 16);
      wvec (startcol+192*ppc, 11, (startcol+192*ppc), 16);
      wvec (startcol+224*ppc, 11, (startcol+224*ppc), 16);
      wvec (startcol+255*ppc, 11, (startcol+255*ppc), 16);

      scmask (DEF);
      doublew (OFF); doubleh (OFF);  revback(OFF);

/*** Draw the color bar (2 pixels wide per color). ***/

      for (i = 0; i <= colormax; i++)
         {
         cbar[2*i] = i & 0377;
         cbar[2*i+1] = i & 0377;
         }

      for (i = 0; i < barwidth; i++)
         wlin (cbar, startcol, startrow--, barlength, 'r');
      }
}

/* smm_annot (subch)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write SMM C/P image annotation to Grinnell channel.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c').
 * Adapted by Andrew L. Stanger
 * 14 February 1985
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

smm_annot (subch)

int subch;			/* Grinnell sub-channel mask. */

{
  int month_day();		/* Function to compute month & day from DOY. */
  char *month_name();		/* Function to look up ASCII month name. */
  static char month[4];
  static char wbuf[10];
  int annot_state = 1;
  int charheight = 43;
  int offset = -21;
  int year, doy;
  int imonth, iday;

   if (!GRN) return;	/* If image display option is off. */

   scmask (subch);		/* Set sub-channel mask. */
   doublew (OFF);  doubleh (ON);

/*** Year and Day-Of-Year: ***/

   (void) sprintf (wbuf, "%d: %03d", H.year, H.doy);
   wbuf[7] = '\0';
   wstr (wbuf, 3, 476, 7, 0);

/*** Time: ***/

   (void) sprintf (wbuf, "%02d:%02d:%02d\0", H.hour, H.minute, H.second);
   wstr (wbuf, 3, 433, 7,0);

/*** Day and month. ***/

   year = H.year + 1900;
   doy = H.doy;
   (void) month_day (year, doy, &imonth, &iday);
   strcpy (month, month_name (imonth));
   sprintf (wbuf, "%d %s ", iday, month);
   wstr (wbuf, 3, 400, 7, 0);
   sprintf (wbuf, "%d", year);
   wstr (wbuf, 3, 375, 7, 0);

/*** Spectral filter: ***/

   switch (H.ifw)
      {
      case 2: sprintf(wbuf,"OPEN  "); break;
      case 6: sprintf(wbuf,"GL    "); break;
      case 0: sprintf(wbuf,"RED   "); break;
      case 1: sprintf(wbuf,"HALPHA"); break;
      case 3: sprintf(wbuf,"BLUE  "); break;
      case 4: sprintf(wbuf,"WIDEBD"); break;
      case 5: sprintf(wbuf,"GREEN "); break;
      default: sprintf(wbuf,"******"); break;
      }
   if (H.iglstw == 1)  sprintf(wbuf,"%s  ", H.glc);
   wstr (wbuf, 3, 475-3*charheight+offset, 8, 0);

/*** Polaroid filter: ***/

   switch (H.ipw)
      {
      case 0: sprintf(wbuf,"CLR"); break;
      case 1: sprintf(wbuf,"+60"); break;
      case 2: sprintf(wbuf,"-60"); break;
      case 3: sprintf(wbuf,"0DG"); break;
      default: sprintf(wbuf,"***"); break;
      }
   wstr (wbuf, 3, 475-4*charheight+offset, 8, 0);

/*** Sector mirror: ***/

   switch (H.ism)
      {
      case 5:  sprintf (wbuf,"N   "); break;
      case 4:  sprintf (wbuf,"NE  "); break;
      case 3:  sprintf (wbuf,"E   "); break;
      case 2:  sprintf (wbuf,"SE  "); break;
      case 1:  sprintf (wbuf,"S   "); break;
      case 0:  sprintf (wbuf,"SW  "); break;
      case 7:  sprintf (wbuf,"W   "); break;
      case 6:  sprintf (wbuf,"NW  "); break;
      case 8:  sprintf (wbuf,"NESW"); break;
      default: sprintf (wbuf,"****"); break;
      }
   wstr (wbuf, 3, 475-5*charheight+offset, 8, 0);

/*** Exposure (1.024 second units) : ***/

   (void) sprintf (wbuf, "%-4u", H.exp);
   wstr (wbuf, 3, 475-6*charheight+offset, 8, 0);

/*** Resolution : ***/

   if( H.ires == 0 ) (void) sprintf (wbuf, "HIGH");
   else (void) sprintf(wbuf,"LOW ");
   wstr (wbuf, 3, 475-7*charheight+offset, 8, 0);

/*** S/C Roll : ***/

   if (H.icm == 1)
      {
      if (annot_state) 
         {
         smm_label (0, color1);
         scmask (subch);		/* Set sub-channel mask. */
         doublew (OFF);  doubleh (ON);
         }
      (void) sprintf (wbuf, "%-4d", scroll);
      wstr (wbuf, 3, 475-8*charheight+offset, 8, 0);

      annot_state = 1;
      }

/*** Calibration Mirror: ***/

   else
      {
      switch (H.icm)
         {
         case 0: (void) sprintf(wbuf,"MOVE"); break;
         case 1: (void) sprintf(wbuf,"OUT "); break;
         case 2: (void) sprintf(wbuf,"IN  "); break;
         case 3: (void) sprintf(wbuf,"BAD "); break;
         default: (void) sprintf(wbuf,"****"); break;
         }
      if (annot_state) 
         {
         smm_label (0, color1);
         scmask (subch);		/* Set sub-channel mask. */
         doublew (OFF);  doubleh (ON);
         }
      wstr (wbuf, 3, 475-8*charheight+offset, 8, 0);
      annot_state = 0;
      }

/*** Image count (frame number) : ***/

   (void) sprintf (wbuf, "%-4u", H.frame);
   wstr (wbuf, 3, 475-9*charheight+offset, 8, 0);

   scmask(color1);		/* Set sub-channel mask. */
   wvec ( 62,  62,  62, 511);
   wvec ( 62, 511, 511, 511);
   wvec (511, 511, 511,  62);
   wvec (511,  62,  62,  62);

   wvec ( 63,  63,  63, 510);
   wvec ( 63, 510, 510, 510);
   wvec (510, 510, 510,  63);
   wvec (510,  63,  63,  63);

   scmask (DEF);		/* Set sub-channel mask. */
   doublew (OFF);  doubleh (OFF);
}


/* smm_dim (ihv)
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Return the dimension of image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c'.)
 * 29 August 1984
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

smm_dim (ihv)

int ihv;
{
   switch (ihv)
      {
      case 0:
      case 5:
      case 9:
      case 10:
      case 11:
      case 13: return(0);
      case 1:
      case 2:
      case 4:
      case 8: return(224);
      case 3:
      case 6:
      case 12: return(448);
      case 7:
      case 14: return(672);
      case 15: return(896);
      default : return(-1);
      }
}
