/* glcp_suncir.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C P _ S U N C I R ( )	- Draw a dotted circular pattern at 1.0 Rsun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 July 1990
 * 29 August 1990: make dots 2 pixels high.
 *  4 March 1991: make "dots" plus marks.
 *  4 April 1991: SGI/GL version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"

glcp_suncir (r1min, r1max, r1inc, ang1inc, r2min, r2max, r2inc, ang2inc, cindex)

int cindex;		/* Color LUT index. */

float r1min, r1max;	/* radial limits for major dotted lines. */
float r1inc;		/* radial increment for major dotted lines. */
float ang1inc;		/* angular increment between minor lines.  */
float r2min, r2max;	/* radial limits for minor dotted lines. */
float r2inc;		/* radial increment for minor dotted lines. */
float ang2inc;		/* angular increment between minor lines.  */
{
extern int nres;
float x, y;		/* (x,y) coordinates corresponding to (r,th). */
float r = 1.0;		/* Radius in R/Rsun units. */
float th;		/* Position angle in degrees. */
int xg, yg;		/* Grinnell coordinates. */
int xg1, yg1, xg2, yg2;

float radius;
float x1, y1, x2, y2;
float r1, r2;

   /*** Draw radial scans (r=r1min -> r1max) every ang1inc degrees. ***/

   for (th = 0.0; th < 360.0; th += ang1inc)
      {
      if (th == 180.0)  r1min = 0.2;  else r1min = 0.4;
      for (radius = r1min; radius < r1max; radius += r1inc)
         {
	 rcoord (&radius, &th, &x, &y, 1, scroll);
         xg = ((x / nres) + 0.5) + gMINCOL;
         yg = gMAXROW - ((y / nres) + 0.5);
         if (xg >= gMINCOL && xg <= gMAXCOL &&
	     yg >= gMINROW && yg <= gMAXROW)
            {
            gl_wvec (xg,   yg-1, xg,   yg+1, cindex);
            gl_wvec (xg-1, yg,   xg+1, yg,   cindex);
            }
	 }
      }

   /*** Draw radial scans (r=r2min -> r2max) every ang2inc degrees. ***/

   for (th = 0.0; th < 360.0; th += ang2inc)
      for (radius = r2min; radius < r2max; radius += r2inc)
         {
	 rcoord (&radius, &th, &x, &y, 1, scroll);
         xg = ((x / nres) + 0.5) + gMINCOL;
         yg = gMAXROW - ((y / nres) + 0.5);
         if (xg >= gMINCOL && xg <= gMAXCOL &&
	     yg >= gMINROW && yg <= gMAXROW)
            {
            gl_wvec (xg,   yg-1, xg,   yg+1, cindex);
            gl_wvec (xg-1, yg,   xg+1, yg,   cindex);
            }
	 }
/*
 *   r1 = 0.12;
 *   r2 = 0.9;
 *   for (th = 0.0; th < 360.0; th += 90.0)
 *      {
 *      rcoord (&r1, &th, &x1, &y1, 1, scroll);
 *      rcoord (&r2, &th, &x2, &y2, 1, scroll);
 *      xg1 = ((x1 / nres) + 0.5) + gMINROW;
 *      yg1 = gMAXCOL - ((y1 / nres) + 0.5);
 *      xg2 = ((x2 / nres) + 0.5) + gMINROW;
 *      yg2 = gMAXCOL - ((y2 / nres) + 0.5);
 *      if (xg1 >= gMINCOL && xg1 <= gMAXCOL &&
 *          yg1 >= gMINROW && yg1 <= gMAXROW &&
 *          xg2 >= gMINCOL && xg2 <= gMAXCOL &&
 *          yg2 >= gMINROW && yg2 <= gMAXROW)
 *         {
 *         gl_wvec (xg1,   yg1-1, xg2,   yg2+1, cindex);
 *         gl_wvec (xg1-1, yg1,   xg2+1, yg2,   cindex);
 *         }
 *      }
 */

   /*** Draw sun circle at 1.0 Rsun.  One dot every 10 degrees. ***/

   for (th = 0; th < 360; th += 10)
      {
      rcoord (&r, &th, &x, &y, 1, scroll);
      xg = ((x / nres) + 0.5) + gMINCOL;
      yg = gMAXROW - ((y / nres) + 0.5);
      if (xg >= gMINCOL && xg <= gMAXCOL &&
	  yg >= gMINROW && yg <= gMAXROW)
         {
         gl_wvec (xg,   yg-1, xg,   yg+1, cindex);
         gl_wvec (xg-1, yg,   xg+1, yg,   cindex);
         }
      }

   return (0);
}
