/* imgpic.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert a single image (in a memory array) to WHIP pic format.
 * Write output image to disc.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  4 April 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
/* #include "/usr/whip/h/whip.h" */
#include "/home/stanger/lib/include/cplib.ext"

imgpic (cbuf, nbypx, xdim, ydim, dir2, namimg)

char *cbuf;
int nbypx;
int xdim, ydim;
char *dir2;
char *namimg;

{
   extern char *newname ();
   extern char *malloc ();
   pichdr imghdr;
   FILE *INFD, *OUTFD;
   char *cpbuf;
   char xdir, ydir;
   static char namout [80];

   int i;
   int nbyimg;
   int nelem;
   int nrechd;
   int nwrite;
   static short hdrbuf [12];

   xdir = 'x';
   ydir = 'y';
   for (i=0; i<2; i++)
      switch (*(dir2 + i))
         {
         case 'l':
         case 'r': xdir = *(dir2 + i); break;
         case 'u':
         case 'd': ydir = *(dir2 + i); break;
         }
   if (xdir == 'x' || ydir == 'y')
      {
      fprintf (stderr, "***imgpic> Illegal direction (%s).\n", dir2); 
      return (-1);
      }
   fprintf (stdout, "   xdir: %c   ydir: %c\n", xdir, ydir);

   if (xdir == 'l')
         swapcols (cbuf, nbypx, xdim, ydim);  /* Reverse image left to right. */

   if (ydir == 'u')
         swaprows (cbuf, nbypx, xdim, ydim);  /* Reverse image top to bottom. */

   /*** Create pic header. ***/

   imghdr.id    = hdrbuf [0]  = 0;
   imghdr.lcom  = hdrbuf [1]  = 0;
   imghdr.type  = hdrbuf [2]  = ROW_MODE;
   imghdr.dims  = hdrbuf [3]  = 2;
   imghdr.hpls  = hdrbuf [4]  = 1;
   imghdr.plns  = hdrbuf [5]  = 1;
   imghdr.rows  = hdrbuf [6]  = xdim;
   imghdr.cols  = hdrbuf [7]  = ydim;
   imghdr.bp    = hdrbuf [8]  = nbypx*8;
   imghdr.sbp   = hdrbuf [9]  = nbypx*8;
   imghdr.xstrt = hdrbuf [10] = 0;
   imghdr.ystrt = hdrbuf [11] = 0;

   strcpy (namout, newname (namimg));		/* Create output image name. */
   fprintf (stdout, "   namout: %s\n", namout);

   /*** id ***/

   switch (nbypx)
      {
      case 1:	imghdr.id = BYTE;   break;
      case 2:	imghdr.id = SHORT;  break;
      case 4:	imghdr.id = FLOAT;  break;
      default:	imghdr.id = 0;
      };
   hdrbuf [0] = imghdr.id;

   /*** lcom ***/

   imghdr.lcom = hdrbuf [1] = 0;

   /*** Allocate memory for output image buffer. ***/

   nelem = xdim * ydim;
   nbyimg = nbypix * nelem;
   fprintf (stdout,"   nbypx: %d   xdim: %d   ydim: %d   nelem: %d   nbyimg: %d\n",
           nbypx, xdim, ydim, nelem, nbyimg);

   if ((OUTFD = fopen (namout, "w")) == NULL)
      {
      fprintf (stderr, "***imgpic> Unable to create output file %s\n", namout);
      return (-2);
      }

   nwrite = fwrite (hdrbuf, 2, 12, OUTFD);
   fprintf (stdout, "   %s   nwrite: %d\n", namout, nwrite);
   nwrite = fwrite (cbuf, nbypx, nelem, OUTFD);
   fprintf (stdout, "   %s   nbypx: %d   nelem: %d\n", namout, nbypx, nelem);

   free (cbuf);
   fclose (OUTFD);
   return (0);
}


/* newname.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create new name for cptopic.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 4 February 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

char *
newname (cp_name)

char *cp_name;
{
static char pic_name [80];

   sprintf (pic_name, "%s.pic", cp_name);
   return (pic_name);
}
