/* merge4s.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Merge 4 images together and store in a new image array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 25 September 1987
 * 10 February 1992: add 4th image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

unsigned short *
merge4s (image1, image2, image3, image4, imdx, imdy, ithresh)

unsigned short *image1;		/* HAO/SMM C/P image array. */
unsigned short *image2;		/* MLSO Mark-III image array. */
unsigned short *image3;		/* MLSO Prominence Monitor image array. */
unsigned short *image4;		/* Solar Disk image array. */
int imdx, imdy;			/* Dimensions of images (same for all 3). */
int ithresh;			/* Intensity threshold for image #3. */

{
extern char *malloc ();
register unsigned short int *im1, *im2, *im3, *im4, *im5;
register int i;
register int maxbrt;
register int threshold;
int npximg;
int nbyimg;

   /* Initialize variables. */

   maxbrt = 249;
   threshold = ithresh;
   npximg = imdx * imdy;
   nbyimg = npximg * sizeof(short);
   im1 = (unsigned short int *) image1;
   im2 = (unsigned short int *) image2;
   im3 = (unsigned short int *) image3;
   im4 = (unsigned short int *) image4;
   im5 = (unsigned short int *) malloc (nbyimg);

   /* Combine 4 images together: use the highest value at each pixel location.*/

   fprintf (stderr, "Merging 4 image channels . . . Please wait.\n");
   for (i=0; i<npximg; i++)
      {
      *(im5 + i) = (*(im1 + i) > *(im2 + i)) ? *(im1 + i) : *(im2 + i);

      if (*(im3 + i) > threshold)  *(im3 + i) = maxbrt;

      *(im5 + i) = (*(im3 + i) > *(im5 + i)) ? *(im3 + i) : *(im5 + i);

      *(im5 + i) = (*(im4 + i) > *(im5 + i)) ? *(im4 + i) : *(im5 + i);

      if (*(im5 + i) > maxbrt)  *(im5 + i) = maxbrt;
      }

   return (im5);
}
