/* mkc4qn.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Merge Mark-III & SMM C/P images together.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C/P image must be 4-quadrant, low resolution, and calibrated.
 * Mark III image is of the type "srq".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 17 August 1992
 * 15 November 1994: rotate image so that solar north is UP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/grscale.h"
#include "/usr/people/smm/lib/include/cp_color.h"

#define color0 250	/* yellow	*/
#define color1 251	/* gray		*/
#define color2 252	/* blue		*/
#define color3 253	/* green	*/
#define color4 254	/* red		*/

main (argc, argv)

int argc;
char *argv [];
{

unsigned short *rdpmon ();
unsigned short *rdmk3 ();
unsigned short *rdc4q ();
unsigned short *rotate ();
unsigned short *merge2s ();

int mskimg ();
int rescale ();
int wdisk ();

static char namemk [80] = "19:13.srq";	/* Mark-III image file name. */
static char namecp [80] = "021819155.c04";	/* C/P image file name. */
static char pmontime [6] = "HH:MM";
static char cptime[6] = "HH:MM";
static char mk3time [6] = "HH:MM";
static char nampgm [80];
static char namout [80] = "mkcp.img";

unsigned char  *cimg;	/* Pointer to output image array. */
unsigned short *cpimr;	/* Pointer to SMM C/P image array. */
unsigned short *cpims;	/* Pointer to SMM C/P image array. */

unsigned short *img2;	/* Pointer to composite image array. */

unsigned short *mkimr;	/* Pointer to Mark-III image array. */
unsigned short *mkims;	/* Pointer to Mark-III image array. */

static int DSP = 1 ;

static int cpcx, cpcy, icpdsk, icprad;
static int cpdx = 448, cpdy = 448, cpnx = 448, cpny = 448;
static int cpmin1 = 0, cpmax1 = 125, cpmin2 = 0, cpmax2 = 250;

static int grnlo = 0, grnhi = 249;
static int i ;
static int imgch0 = 0, imgch1 = 1, imgch2 = 2, imgch3 = 3;
static int ithresh = 255;
static int level = 0;

static int mkcx, mkcy, imkdsk, imkrad;
static int mkdx, mkdy, mknx, mkny;
static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250;

static int nbypx = 2 ;
static int npximg ;

static int xgs = 63, ygs = 63;
static int xl =  760;	/* Specification of GL window position & size. */
static int yl =  480;
static int xu = 1271;
static int yu =  991;

/* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

static float cpcxnew[] = {  7., 100., 225., 398., 441., 348., 225., 50., 223.5};
static float cpcynew[] = {225., 348., 441., 398., 258., 100.,   7., 50., 223.5};
static float cpdsk = 1.5,  cprad = 1.8;
static float cpfx, cpfy, mkfx, mkfy;

static float degnor = 43.0 ;	/* Angle CCW of north for C/P images, roll=0. */
static float fcpcx, fcpcy;
static float fx, fy;

static float mkdsk = 1.08, mkrad = 1.8;
static float mkang =  0.0, cpang = 43.0;
static float mkpxrs = 104.4435, cppxrs = 80.9877, impxrs = 54.0;

mkpxrs = (511.0 / (2.0 * (1.1224 + 127.0 * 0.0104244)));   /* Scale 13 Dec 90 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* Request input information. */

   fprintf (stderr, "Enter file name of SMM C/P image: ");
   fscanf  (stdin, "%s", namecp);
/*
 *   fprintf (stderr, "Enter minimum radius for SMM C/P image: ");
 *   fscanf  (stdin, "%f", &cprad);
 */

   fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ");
   fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2);

   fprintf (stderr, "Enter file name of MLSO Mark-III image: ");
   fscanf  (stdin, "%s", namemk);
   fprintf (stderr, "Enter time (HR:MIN) of MLSO Mark-III image: ");
   fscanf  (stdin,  "%s", mk3time);

   fprintf (stderr, "Enter maximum radius for MLSO Mark-III image: ");
   fscanf  (stdin, "%f", &mkrad);
   cprad = mkrad;

   imkrad = (int) (mkrad * mkpxrs);
   imkdsk = (int) (mkdsk * mkpxrs);

   /*** Initialize SGI Graphics Library. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen (nampgm);
   gl_wclear (BLACK);
   onemap ();			/* Set single-map mode for color table. */
   get_cmap ();			/* Save color map. */

   multimap ();			/* Set multi-map mode for color table. */
   gconfig ();
   setmap (2);			/* Select color map #2. */

   fminit ();			/* Initialize SGI font manager. */

   /*
    * Read SMM C/P image file.
    * Write black disk.
    * Rotate image so that solar north is UP.
    * Magnify image.
    */

   printf ("Read C/P image.\n");
   cpims = rdc4q (namecp, &cpdx, &cpdy, &cpcx, &cpcy);
   sprintf (cptime, "%02d:%02d", H.hour, H.minute);
   fcpcx = cpcxnew[H.ism];
   fcpcy = cpcynew[H.ism];
   cppxrs = H.pixrs;
   icprad = (int) (cprad * cppxrs);
   icpdsk = (int) (cpdsk * cppxrs);
   printf ("fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);

   printf ("Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2);
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1);

   if (DSP)
      {
      printf ("xgs:%d   ygs:%d   cpnx:%d   cpny:%d\n", xgs,ygs,cpnx,cpny);
      imgwgl (cpims, nbypx, xgs, ygs, cpdx, cpdy, "ru");
      }

   printf ("Write disk into C/P image . . . cpcx:%d   cpcy:%d   icprad:%d\n",
            cpcx, cpcy, icprad);
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level);

   if (DSP)
      {
      printf ("xgs:%d   ygs:%d   cpnx:%d   cpny:%d\n", xgs,ygs,cpnx,cpny);
      imgwgl (cpims, nbypx, xgs, ygs, cpdx, cpdy, "ru");
      }

/*** Rotate C/P image so that solar north is UP. ********************/
/*
   cpang = (float) -(scroll + degnor);
   printf ("Rotate SMM C/P  image . . . cpang: %f\n", cpang);
   cpimr = rotate (cpims, cpdx, cpdy, cpcx, cpcy, cpang);
   free (cpims);
   if (DSP)
      imgwgl (cpimr, nbypx, 0, 0, cpdx, cpdy, "ru");
 */
   cpimr = cpims ;
   free (cpims) ;
/************************/

   printf ("Magnify C/P image.\n");
   impxrs = 54.0;		  /* Composite image scale size (pixels/Rsun).*/
   cpfx = cpfy = impxrs / cppxrs; /* Set FOV to 9.33 Rsun. */
   cpims = (unsigned short *) malloc (cpdx * cpdy * sizeof (short));
   expcon (cpimr, cpdx, cpdy, cpims, cpdx, cpdy, cpfx, cpfy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy);
   free (cpimr);

   if (DSP)
      {
      printf ("xgs:%d   ygs:%d   cpnx:%d   cpny:%d\n", xgs,ygs,cpnx,cpny);
      imgwgl (cpims, nbypx, xgs, ygs, cpnx, cpny, "ru");
      }

   /*
    * Read MLSO Mark-III image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Rescale intensity levels.
    */

   printf ("Read Mark-III image.\n");
   mkims = rdmk3 (namemk, &mkdx, &mkdy, &mkcx, &mkcy);
   printf ("mkdx:%d   mkdy:%d   mkcx:%d   mkcy:%d\n", mkdx, mkdy, mkcx, mkcy);
   if (DSP)
      {
      imgwgl (mkims, nbypx, 0, 0, mkdx, mkdy, "ru");
      }

   printf ("Write disk in Mark-III image . . . radius:%f\n", mkdsk);
   wdisk (mkims, mkdx, mkdy, mkcx, mkcy, imkdsk, level);
   if (DSP)  imgwgl (mkims, nbypx, 0, 0, mkdx, mkdy, "ru");

   mkang = mkang - cpang;
   mkang -= (float) scroll;
   printf ("Rotate Mark-III image . . . mkang: %f\n", mkang);
   mkimr = rotate (mkims, mkdx, mkdy, mkcx, mkcy, mkang);
   free (mkims);
   if (DSP)  imgwgl (mkimr, nbypx, 0, 0, mkdx, mkdy, "ru");

   printf ("Mask Mark-III image beyond %f Rsun.\n", mkrad);
   mskimg (mkimr, 0, 0, mkdx, mkdy, mkcx, mkcy, imkrad);
   if (DSP)  imgwgl (mkimr, nbypx, 0, 0, mkdx, mkdy, "ru");

   printf ("Magnify Mark-III image.\n");
   mknx = cpnx;
   mkny = cpny;
   mkfx = mkfy = impxrs / mkpxrs;
   mkims = (unsigned short *) malloc (mknx * mkny * sizeof(short));
   expcon (mkimr, mkdx, mkdy, mkims, mknx, mkny, mkfx, mkfy,
           (float) mkcx, (float) mkcy, fcpcx, fcpcy);
   if (DSP)
      {
      imgwgl (mkims, nbypx, xgs, xgs, mknx, mkny, "ru");
      }

   printf ("Rescale Mark-III image.\n");
   rescale (mkims, mknx, mkny, mkmin1, mkmax1, mkmin2, mkmax2, grnlo, grnhi-1);
   if (DSP)  imgwgl (mkims, nbypx, xgs, ygs, mknx, mkny, "ru");

   /*** Add two images together. ***/

   img2 = merge2s (cpimr, mkims, cpnx, cpny, ithresh);

   /*** Display composite image with annotation. ***/

   if (DSP)
      {
      gl_wclear (BLACK);
      ldisk = 1;
      glcp_label (0, 1, 1, 1, cp_grey, 1);
      glcp_annot (0, 1);

      imgwgl (img2, nbypx, xgs, ygs, cpnx, cpny, "ru");/* Write merged image.*/
      free (img2) ;
/***************/
      select_font ("Charter-Black", 12);	/* Helvetica-Bold, 16x17 characters. */
      gl_wstr ("HAO/SMM CORONAGRAPH/POLARIMETER", 63, 38, cp_green);
      gl_wstr ("MLSO MARK-III K-CORONAMETER    ", 63, 20, cp_green);
      gl_wstr (mk3time,  444, 20, cp_red);
      gl_wstr (cptime,   444, 38, cp_red);

/***************/

      fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
      xcen = fcpcx;
      ycen = 447 - fcpcy;
      fx = fy = mkpxrs / cppxrs;
      pixrs = pixrs * fx;
      glcp_north ((int) fcpcx, (int) fcpcy, 0.1, 0.3, cp_yellow);
      glcp_scalers (0, cp_yellow);
      glcp_suncir (0.4, 1.0, 0.1, 90.0, 0.4, 1.0, 0.2, 30.0, cp_yellow);

   
      /*** Write composite image to disc file (WHIP pic format). ***/

      npximg = 512 * 512;
      img2 = (unsigned short *) malloc (npximg * sizeof (short));
      cimg = (unsigned char *)  malloc (npximg);
      rectread (0, 0, 511, 511, img2);
      for (i = 0; i < npximg; i++)
         *(cimg + i) = (unsigned char) (*(img2 + i) & 0377);
      imgpic (cimg, 1, 512, 512, "ru", namout);

      }
   sleep (3);
   gexit ();
   exit (0);
}
