/* mkcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Merge Mark-III & SMM C/P images together.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C/P image must be low resolution, and calibrated.
 * Mark III image is of the type "srq".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 * 16 March 1989: eliminate pmon image.
 * 20 February 1991: Update Mark-III geometric scale (mkpxrs).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include "/u/rose/grin2/grin.h"
#include "/u/smm/lib/src/cp/cplib.h"
#include "grscale.h"
int GRN = 0;
#define cred   254
#define cgreen 253
#define cblue  252
#define cgray  251
#define cyellow 250

main()
{

unsigned short *rdpmon2();
unsigned short *rdmk3();
unsigned short *rdcp();
unsigned short *rotate();
unsigned short *magimg();
unsigned short *merge2s();

int mskimg();
int rescale();
int wdisk();

static char namemk[80] = "19:13.srq";	/* Mark-III image file name. */
static char namecp[80] = "021819155.c04";	/* C/P image file name. */
static char mk3time[6] = "HH:MM";

unsigned short *mkims;	/* Pointer to Mark-III image array. */
unsigned short *cpims;	/* Pointer to SMM C/P image array. */

unsigned short *mkimr;	/* Pointer to Mark-III image array. */
unsigned short *cpimr;	/* Pointer to SMM C/P image array. */

unsigned short *img2;	/* Pointer to composite image array. */

static int ngr = 0;
static int imgch0 = 0, imgch1 = 1, imgch2 = 2, imgch3 = 3;

static int mkdx, mkdy, mknx, mkny;
static int mkcx, mkcy, imkdsk, imkrad;

static int cpdx = 448, cpdy = 448, cpnx = 448, cpny = 448;
static int cpcx, cpcy, icpdsk, icprad;

static int xgs = 63, ygs = 63;
static int ithresh = 249;
static int level = 0;
static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250;
static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250;
static int grnlo = 0, grnhi = 250;

/* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

static float cpcxnew[] = {  7., 100., 225., 398., 441., 348., 225., 50., -1.};
static float cpcynew[] = {225., 348., 441., 398., 258., 100.,   7., 50., -1.};
static float cpdsk = 1.5,  cprad = 1.8;
static float cppxrs = 80.9877;		/* 961.0 / (5.933 * 2.0) [low res] */
static float fcpcx, fcpcy;
static float fx, fy;
static float mkang =  -46.;
static float mkdsk = 1.10, mkrad = 1.8;
static float mkpxrs = 104.4435;		/* pixels/Rsun: old scale size. */
mkpxrs = (511.0 / (2.0 * (1.1224 + 127.0 * 0.0104244)));   /* scale 13 Dec 90 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* Request input information. */

   fprintf (stderr, "Enter file name of SMM C/P image: ");
   fscanf  (stdin, "%s", namecp);
/*
 *   fprintf (stderr, "Enter minimum radius for SMM C/P image: ");
 *   fscanf  (stdin, "%f", &cprad);
 *   fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ");
 *   fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2);
 */
   fprintf (stderr, "Enter file name of MLSO Mark-III image: ");
   fscanf  (stdin, "%s", namemk);
   fprintf (stderr, "Enter time (HR:MIN) of MLSO Mark-III image: ");
   fscanf  (stdin,  "%s", mk3time);
   fprintf (stderr, "Enter maximum radius for MLSO Mark-III image: ");
   fscanf  (stdin, "%f", &mkrad);

   cprad = mkrad;
   icprad = (int) (cprad * cppxrs);
   imkrad = (int) (mkrad * mkpxrs);
   icpdsk = (int) (cpdsk * cppxrs);
   imkdsk = (int) (mkdsk * mkpxrs);

   fprintf (stderr, "Enter Grinnell device [0,1]: ");
   fscanf  (stdin,  "%d", &ngr);

/* Initialize Grinnell. */

   if (ngr >= 0)
      {
      GRN = 1;
      gr_start (ngr);
      init();
      if (ngr == 0)  imgch3 = imgch0;  else imgch3 = 0;
      if (ngr == 1)
         {imgch0 = 1;  imgch1 = 1;  imgch2 = 1;}   /* Channels 2 & 3 N/A. */
      disp(imgch0);
      gr_erase (imgch0);
      gr_erase (7);
      }

/*
 * Read SMM C/P image file.
 * Write black disk.
 * Magnify image.
 */

   if (GRN)
      {
      rwech (imgch0);
      disp (imgch0);
      }

   printf ("Read C/P image.\n");
   cpims = rdcp (namecp, &cpdx, &cpdy, &cpcx, &cpcy);
   fcpcx = cpcxnew[H.ism];
   fcpcy = cpcynew[H.ism];

   printf ("Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2);
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1);

   if (GRN)
      {
      printf ("wbox> xgs:%d   ygs:%d   cpnx:%d   cpny:%d\n", xgs,ygs,cpnx,cpny);
      wbox (cpims, xgs, ygs, cpdx, cpdy, "ru");
      }

   printf ("Write disk into C/P image . . . cpcx:%d   cpcy:%d   icprad:%d\n",
            cpcx, cpcy, icprad);
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level);

   if (GRN)
      {
      printf ("wbox> xgs:%d   ygs:%d   cpnx:%d   cpny:%d\n", xgs,ygs,cpnx,cpny);
      wbox (cpims, xgs, ygs, cpdx, cpdy, "ru");
      }

   printf ("Magnify C/P image.\n");
   fx = fy = mkpxrs / cppxrs;
   cpimr = (unsigned short *) malloc (cpdx * cpdy * sizeof(short));
   printf ("fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
   expcon (cpims, cpdx, cpdy, cpimr, cpdx, cpdy, fx, fy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy);
   free (cpims);

   if (GRN)
      {
      gr_erase (imgch0);
      printf ("wbox> xgs:%d   ygs:%d   cpnx:%d   cpny:%d\n", xgs,ygs,cpnx,cpny);
      wbox (cpimr, xgs, ygs, cpnx, cpny, "ru");
      }

/*
 * Read MLSO Mark-III image file.
 * Write black disk.
 * Rotate image.
 * Mask image beyond maximum radius.
 * Rescale intensity levels.
 */

   printf ("Read Mark-III image.\n");
   mkims = rdmk3 (namemk, &mkdx, &mkdy, &mkcx, &mkcy);
   printf ("mkdx:%d   mkdy:%d   mkcx:%d   mkcy:%d\n", mkdx, mkdy, mkcx, mkcy);
   if (GRN)
      {
      rwech (imgch1);
      gr_erase (imgch1);
      disp(imgch1);
      wbox (mkims, 0, 0, mkdx, mkdy, "ru");
      }

   printf ("Write disk in Mark-III image . . . radius:%f\n", mkdsk);
   wdisk (mkims, mkdx, mkdy, mkcx, mkcy, imkdsk, level);
   if (GRN)  wbox (mkims, 0, 0, mkdx, mkdy, "ru");

   mkang -= (float) scroll;
   printf ("Rotate Mark-III image . . . mkang: %f\n", mkang);
   mkimr = rotate (mkims, mkdx, mkdy, mkcx, mkcy, mkang);
   free (mkims);
   if (GRN)  wbox (mkimr, 0, 0, mkdx, mkdy, "ru");

   printf ("Mask Mark-III image beyond %f Rsun.\n", mkrad);
   mskimg (mkimr, 0, 0, mkdx, mkdy, mkcx, mkcy, imkrad);
   if (GRN)  wbox (mkimr, 0, 0, mkdx, mkdy, "ru");

   printf ("Magnify Mark-III image.\n");
   mknx = cpnx;
   mkny = cpny;
   fx = 1.0;
   fy = 1.0;
   mkims = (unsigned short *) malloc (mknx * mkny * sizeof(short));
   expcon (mkimr, mkdx, mkdy, mkims, mknx, mkny, fx, fy,
           (float) mkcx, (float) mkcy, fcpcx, fcpcy);
   if (GRN)
      {
      gr_erase (imgch1);
      wbox (mkims, xgs, xgs, mknx, mkny, "ru");
      }

   printf ("Rescale Mark-III image.\n");
   rescale (mkims, mknx, mkny, mkmin1, mkmax1, mkmin2, mkmax2, grnlo, grnhi-1);
   if (GRN)  wbox (mkims, xgs, ygs, mknx, mkny, "ru");

/* Add two images together. */

   img2 = merge2s (cpimr, mkims, cpnx, cpny, ithresh);

/* Display composite image with annotation. */

   if (GRN)
      {
      rwech (imgch2);
      disp (imgch2);
      gr_erase (imgch2);
      cpview (namecp, imgch2, 1);	      /* Display original C/P image.*/
      wbox (img2, xgs, ygs, cpnx, cpny, "ru");/* Overwrite with merged image.*/
      scmask(253);
      doubleh (OFF);
      doublew (ON);
      wsolid (imgch2, 0, 0, 512, 61, "ru");	/* Erase area below image. */
      fx = fy = mkpxrs / cppxrs;
      pixrs = pixrs * fx;
      printf ("fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
      crosshair ((int) fcpcx, (int) fcpcy, 10, 10, 253);
      suncir(253);
      scalers (253);
      scmask(253);
      wstr ("HAO/SMM CORONAGRAPH/POLARIMETER", 38, 40, 7, 0);
      wstr ("MLSO MARK-III K-CORONAMETER    ", 38, 20, 7, 0);
      scmask(252);
      wstr (mk3time,  444, 20, 7, 0);
      }
}
