/* mkcpn.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Merge Mark-III & SMM C/P images together.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C/P image must be low resolution, and calibrated.
 * Mark III image is of the type "srq".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 *  2 June 1989: Use Eikonix-digitized images (512x512 byte array).
 *  3 August 1989: Use Eikonix-digitized images (with header).
 * 13 December 1990: change Mark-III geometric scale to 104.4435 pixels/Rsun.
 *  2 January 1991: add prompt for Pmon intensity rescaling.
 * 28 March 1991: adapt for SGI IRIS.
 * 16 November 1994: eliminate Pmon image & orient so solar north is UP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <whip.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/cp_color.h"
#include "/usr/people/stanger/lib/include/img_scale.h"
#include "/usr/people/stanger/lib/include/mk3lib.h"

main (argc, argv)

int argc ;
char *argv [] ;
{

unsigned short *rdpmon ();
unsigned short *rdmk3s ();
unsigned short *rdcp ();
unsigned short *rotate_img ();
unsigned short *magimg ();
unsigned short *merge3s ();

int mskimg ();
int rescale ();
int wdisk ();

static char answer [80];
static char namepm [80] = "pmon.218-80.191310";	/* Pmon image file name. */
static char namemk [80] = "19:13.srq";	/* Mark-III image file name. */
static char namecp [80] = "021819155.c04";	/* C/P image file name. */
static char pmontime [6] = "HH:MM";
static char mk3time  [6] = "HH:MM";
static char fbname [] = "mulimg";

unsigned short *pmims;	/* Pointer to Prominence Monitor image array. */
unsigned short *mkims;	/* Pointer to Mark-III image array. */
unsigned short *cpims;	/* Pointer to SMM C/P image array. */

unsigned short *pmimr;	/* Pointer to Prominence Monitor image array. */
unsigned short *mkimr;	/* Pointer to Mark-III image array. */
unsigned short *cpimr;	/* Pointer to SMM C/P image array. */

unsigned short *pmimg;	/* Pointer to Prominence Monitor image array. */
unsigned short *img3;	/* Pointer to composite image array. */

unsigned char *imgbuf ;

static int FB = 23;
static int type = FB_MEM;
static int kind = SHORT;
static int rows = 512;
static int cols = 512;
static int fast = 1;
static int fin ;
static int nbypx = 2;
static int nbyimg ;
static float bkgcol = 0.0;

static int pmdx, pmdy, pmnx, pmny;
static int pmcx, pmcy, ipmdsk, ipmrad;

static int mkdx, mkdy, mknx, mkny;
static int mkcx, mkcy, imkdsk, imkrad;

static int cpdx = 448, cpdy = 448, cpnx = 448, cpny = 448;
static int cpcx, cpcy, icpdsk, icprad;

static int xs = 0, ys = 0;
static int xgs = 63, ygs = 63;
static int ithresh = 175;
static int level = 0;
static int pmmin1 = 0, pmmax1 = 250, pmmin2 = 0, pmmax2 = 250;
static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250;
static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250;
static int grnlo = 0, grnhi = 249;

/* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

static float cpcxnew[] = {  7., 100., 225., 398., 441., 348., 225., 50., -1.};
static float cpcynew[] = {225., 348., 441., 398., 258., 100.,   7., 50., -1.};
static float fcpcx, fcpcy;
static float fx, fy;
static float pmfac = 1.0;
static float pmdsk = 0.99,  pmrad = 1.5;
static float mkdsk = 1.08, mkrad = 1.8;
static float cpdsk = 1.5,  cprad = 1.8;
static float pmang =  0.0, mkang =  0.0, cpang = 43.0;
static float pmpxrs = 117.5, mkpxrs = 104.4435, cppxrs = 80.9877;
mkpxrs = (511.0 / (2.0 * (1.1224 + 127.0 * 0.0104244)));   /* scale 13 Dec 90 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   /* Request input information. */

   fprintf (stderr, "Enter file name of SMM C/P image: ");
   fscanf  (stdin, "%s", namecp);
/*
 *   fprintf (stderr, "Enter minimum radius for SMM C/P image: ");
 *   fscanf  (stdin, "%f", &cprad);
 */
   fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ");
   fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2);

   fprintf (stderr, "Enter file name of MLSO Mark-III image: ");
   fscanf  (stdin, "%s", namemk);
   fprintf (stderr, "Enter time (HR:MIN) of MLSO Mark-III image: ");
   fscanf  (stdin,  "%s", mk3time);

   fprintf (stderr, "Enter maximum radius for MLSO Mark-III image: ");
   fscanf  (stdin, "%f", &mkrad);
   cprad = mkrad;

   fprintf (stderr, "Enter file name of MLSO Prominence Monitor image: ");
   fscanf  (stdin, "%s", namepm);

   fprintf (stderr, "Enter time (HR:MIN) of Prominence Monitor image: ");
   fscanf  (stdin,  "%s", pmontime);
/*
 *   fprintf (stderr, "Enter maximum radius for Prominence Monitor image: ");
 *   fscanf  (stdin, "%f", &pmrad);
 */

   fprintf (stderr, "Do you wish to rescale Pmon image intensities [y/n] ? ");
   fscanf  (stdin, "%s", answer);
   if (strncmp (answer, "y", 1) == 0)
      {
      fprintf (stderr, "Enter Pmon oldmin, oldmax, newmin, newmax: ");
      fscanf  (stdin, "%d,%d,%d,%d", &pmmin1, &pmmax1, &pmmin2, &pmmax2);
      }

   fprintf (stderr, "Enter threshold DN level for Pmon image: ");
   fscanf  (stdin, "%d", &ithresh);

   icprad = (int) (cprad * cppxrs);
   imkrad = (int) (mkrad * mkpxrs);
   ipmrad = (int) (pmrad * pmpxrs);
   icpdsk = (int) (cpdsk * cppxrs);
   imkdsk = (int) (mkdsk * mkpxrs);
   ipmdsk = (int) (pmdsk * pmpxrs);


   /*** Initialize WHIP. ***/

   foreground ();
   fb_open ();			/* Initialize WHIP software. */
/*  FB = fb_create (fbname, type, kind, rows, cols); */	/* Create Frame Buffer*/
   fb_setfb (FB);
   openpl ();
   foreground ();
   fb_clear (FB, &bkgcol, fast);	/* Clear frame buffer. */

   /*
    * Read SMM C/P image file.
    * Write black disk.
    * Magnify image.
    */

   fprintf (stdout, "<--Read C/P image.\n");
   cpims = rdcp (namecp, &cpdx, &cpdy, &cpcx, &cpcy);
   fcpcx = cpcxnew [H.ism];
   fcpcy = cpcynew [H.ism];
   fprintf (stdout, "   fcpcx: %6.2f   fcpcy: %6.2f\n", fcpcx, fcpcy);
   wboxfb (FB, cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (2);

   /*** Rescale brightness of C/P image. ***/

   fprintf (stdout, "~~~Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2);
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1);

   fprintf (stdout, "   pmmkcp> xs:%d   ys:%d   cpnx:%d   cpny:%d\n",
            xs, ys, cpnx, cpny);
   wboxfb (FB, cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (2);

   /*** Write black disk in C/P occulting disk. ***/

   fprintf (stdout, "-0-Write disk into C/P image . . . cpcx:%d   cpcy:%d   icprad:%d\n",
            cpcx, cpcy, icprad);
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level);

   fprintf (stdout, "   pmmkcp> xs:%d   ys:%d   cpnx:%d   cpny:%d\n",
            xs, ys, cpnx, cpny);
   wboxfb (FB, cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (2);

   /*** Magnify C/P image. ***/

   fprintf (stdout, "<->Magnify C/P image.\n");
   fx = fy = mkpxrs / cppxrs;
   cpimr = (unsigned short *) malloc (cpdx * cpdy * sizeof(short));
   expcon (cpims, cpdx, cpdy, cpimr, cpdx, cpdy, fx, fy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy);
   free (cpims);

   fprintf (stdout, "   pmmkcp> xs:%d   ys:%d   cpnx:%d   cpny:%d\n",
            xs, ys, cpnx, cpny);
   wboxfb (FB, cpimr, nbypix, xs, ys, cpnx, cpny, "ru");
   sleep (2);

   /*
    * Read MLSO Mark-III image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Rescale intensity levels.
    */

   fprintf (stdout, "<--Read Mark-III image.\n");

   mkims = rdmk3s (namemk, &mkdx, &mkdy, &mkcx, &mkcy, 0);
   fprintf (stdout, "   mkdx:%d   mkdy:%d   mkcx:%d   mkcy:%d\n",
            mkdx, mkdy, mkcx, mkcy);
   wboxfb (FB, mkims, nbypx, xs, ys, mkdx, mkdy, "ru");
   sleep (2);

   /*** Write black disk into Mark-III occulting disk. ***/

   fprintf (stdout, "-0-Write disk in Mark-III image . . . radius:%f\n", mkdsk);
   wdisk (mkims, mkdx, mkdy, mkcx, mkcy, imkdsk, level);
   wboxfb (FB, mkims, nbypx, xs, ys, mkdx, mkdy, "ru");
   sleep (2);

   /*** Rotate Mark-III image. ***/

   mkang -= cpang;
   mkang -= (float) scroll;
   fprintf (stdout, "~^~Rotate Mark-III image . . . mkang: %f\n", mkang);
   mkimr = rotate_img (mkims, mkdx, mkdy, mkcx, mkcy, mkang);
   free (mkims);
   wboxfb (FB, mkimr, nbypx, xs, ys, mkdx, mkdy, "ru");
   sleep (2);

   /*** Mask Mark-III image beyond specified radius. ***/

/*
   fprintf (stdout, "|-|Mask Mark-III image beyond %f Rsun.\n", mkrad);
   mskimg (mkimr, 0, 0, mkdx, mkdy, mkcx, mkcy, imkrad);
   wboxfb (FB, mkimr, nbypx, xs, ys, mkdx, mkdy, "ru");
   sleep (2);
 */

   /*** Magnify Mark-III image. ***/

   fprintf (stdout, "<->Magnify Mark-III image.\n");
   mknx = cpnx;
   mkny = cpny;
   fx = 1.0;
   fy = 1.0;
   mkims = (unsigned short *) malloc (mknx * mkny * sizeof(short));
   expcon (mkimr, mkdx, mkdy, mkims, mknx, mkny, fx, fy,
           (float) mkcx, (float) mkcy, fcpcx, fcpcy);
   free (mkimr);
 
   wboxfb (FB, mkims, nbypx, xs, xs, mknx, mkny, "ru");
   sleep (2);

   /*** Rescale brightness of Mark-III image. ***/

   fprintf (stdout, "~~~Rescale Mark-III image.\n");
   rescale (mkims, mknx, mkny, mkmin1, mkmax1, mkmin2, mkmax2, grnlo, grnhi-1);
   wboxfb (FB, mkims, nbypx, xs, ys, mknx, mkny, "ru");
   sleep (2);

   /*** Mask Mark-III image beyond specified radius. ***/

   mkcx = (int) fcpcx;
   mkcy = (int) fcpcy;
   fprintf (stdout, "|-|Mask Mark-III image beyond %f Rsun.\n", mkrad);
   fprintf (stdout, "mknx: %d  mkny: %d  mkcx: %d  mkcy: %d  imkrad: %d\n",
            mknx, mkny, mkcx, mkcy, imkrad);
   mskimg (mkims, 0, 0, mknx, mkny, mkcx, mkcy, imkrad);
   wboxfb (FB, mkims, nbypx, xs, ys, mknx, mkny, "ru");
   sleep (2);

   /*
    * Read MLSO Prominence Monitor image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Minify image.
    * Rescale intensity levels.
    */

   fprintf (stdout, "<--Read Pmon image.\n");
   pmims = rdpmon (namepm, &pmdx, &pmdy, &pmcx, &pmcy, pmontime, &pmpxrs, &pmang);
   pmang -=  cpang;
   ipmdsk = (int) (pmdsk * pmpxrs);
   ipmrad = pmrad * pmpxrs;
   fprintf (stdout, "   pmdx: %d   pmdy: %d   pmcx: %d   pmcy: %d\n",
            pmdx, pmdy, pmcx, pmcy);
   wboxfb (FB, pmims, nbypx, xs, ys, pmdx, pmdy, "ru");
   sleep (2);

   /*** Rescale brightness of Pmon image. ***/

   fprintf (stdout, "~~~Rescale Pmon image.\n");
   pmscale (pmims, pmdx, pmdy, pmmin1, pmmax1, pmmin2, pmmax2, grnlo, grnhi);

   wboxfb (FB, pmims, nbypx, xs, ys, pmdx, pmdy, "ru");
   sleep (2);

   /*** Write black disk at center of Pmon occulting disk. ***/

   fprintf (stdout, "-0-Write %6.2f Rsun disk into Pmon image.\n", pmdsk);
   wdisk (pmims, pmdx, pmdy, pmcx, pmcy, ipmdsk, level);
   wboxfb (FB, pmims, nbypx, xs, ys, pmdx, pmdy, "ru");
   sleep (2);

   /*** Rotate Pmon image. ***/

   pmang -= (float) scroll;
   fprintf (stdout, "~^~Rotate Pmon image . . . pmang: %f\n", pmang);
   pmimr = rotate_img (pmims, pmdx, pmdy, pmcx, pmcy, pmang);
   free (pmims);
   wboxfb (FB, pmimr, nbypx, xs, ys, pmdx, pmdy, "ru");
   sleep (2);

   /*** Magnify Pmon image. ***/

   fprintf (stdout, "<->Magnify Pmon image.\n");
   pmnx = cpnx; pmny = cpny;
   fx = fy = mkpxrs / pmpxrs;
   pmimg = (unsigned short *) malloc (pmnx * pmny * sizeof(short));
   expcon (pmimr, pmdx, pmdy, pmimg, pmnx, pmny, fx, fy,
           (float) pmcx, (float) pmcy, fcpcx, fcpcy);
   free (pmimr);
   wboxfb (FB, pmimg, nbypx, xs, ys, pmnx, pmny, "ru");
   sleep (2);

   /*** Mask Pmon image beyond specified radius. ***/

   fprintf (stdout, "|-|Mask Pmon image beyond %4.2f Rsun.\n", pmrad);
   pmfac = fx;
   ipmrad = pmrad * pmfac * pmpxrs;
   fprintf (stdout, "   pmfac: %6.2f   pmpxrs: %6.2f   ipmrad: %d\n",
            pmfac, pmpxrs, ipmrad);
   pmcx = (int) fcpcx;
   pmcy = (int) fcpcy;
   mskimg (pmimg, xs, ys, pmnx, pmny, pmcx, pmcy, ipmrad);
   wboxfb (FB, pmimg, nbypx, xs, ys, pmnx, pmny, "ru");
   sleep (2);

   /*
    *   fprintf (stdout, "~~~Rescale Pmon image.\n");
    *   rescale (pmimg, pmnx, pmny, pmmin1, pmmax1, pmmin2, pmmax2);
    *   wboxfb (FB, pmimg, nbypx, xs, ys, pmnx, pmny, "ru");
    *   sleep (2);
    */

   /*** Add three images together. ***/

   img3 = merge3s (cpimr, mkims, pmimg, cpnx, cpny, ithresh);
   free (cpimr);
   free (mkims);
   free (pmimg);

   /*** Display composite image with annotation. ***/
   /* Clear frame buffer. */
   /* Add labels for annotation. */
   /* Annotate image. */

   fb_clear (FB, &bkgcol, fast);
   wboxfb (FB, img3, nbypx, xgs, ygs, cpnx, cpny, "ru");/* Write merged image.*/
   sleep (2);
   cp_label (FB, 1, cp_grey, 1);
   cp_annot (FB, cp_red);
/*   wsolid (imgch3, 0, 0, 512, 61, "ru"); */	/* Erase area below image. */
/*
   wstr ("HAO/SMM CORONAGRAPH/POLARIMETER", 38, 40, 2, cp_green);
   wstr ("MLSO MARK-III K-CORONAMETER    ", 38, 20, 2, cp_green);
   wstr ("MLSO PROMINENCE MONITOR        ", 38,  0, 2, cp_green);
   wstr (pmontime, 444,  0, 2, cp_red);
   wstr (mk3time,  444, 20, 2, cp_red);

   fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
   xcen = fcpcx;
   ycen = 447 - fcpcy;
   fx = fy = mkpxrs / cppxrs;
   pixrs = pixrs * fx;
   cp_north ((int) fcpcx, (int) fcpcy, 10, 10, cp_yellow);
   cp_scalers (cp_yellow);
 */
/*   cp_suncir (cp_yellow); */
   
   closepl ();
   fb_close ();
   exit (0);
}
