/* glcp_annot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write SMM C/P image annotation to Iris Frame Buffer.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger  HAO/NCAR   SMM C/P
 *  3 April 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/usr/people/smm/lib/include/cp_color.h"
#include "/usr/people/smm/lib/include/cplib.ext"

glcp_annot (hires, see)

int hires;		/* =0 for 512x512 display, =1 for 1024x1024 display. */
int see;		/* Display flag: =0 no, =1 yes. */

{
   int month_day();		/* Function to compute month & day from DOY. */
   char *month_name();		/* Function to look up ASCII month name. */
   static char month [4];
   static char wbuf [10];

   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    */
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
  
   int annot_state = 1;
   int charheight = 43;
   int offset = -21;
   int xstart = 2;
   int ystart = 485;
   int xc, yc;			/* height & width for character annotation. */
   int xs, ys;			/* Start position for character annotation. */
   int year, doy;
   int imonth, iday;
   int fsize;

   if (!see) return (0);	/* If image display option is off. */
   if (hires)
      {
      xs = xstart * 2;
      ys = ystart * 2;
      yc = charheight * 2;
      fsize = 19;
      }
   else
      {
      xs = xstart;
      ys = ystart;
      yc = charheight;
      fsize = 10;
      }

   select_font (font4, fsize);		/* CharterBT-Black font. */

   /*** Year and Day-Of-Year: ***/

   (void) sprintf (wbuf, "%d: %03d", H.year, H.doy);
   wbuf [7] = '\0';
   glwstr (wbuf, xs, ys, cp_red);

   /*** Time: ***/

   (void) sprintf (wbuf, "%02d:%02d:%02d\0", H.hour, H.minute, H.second);
   glwstr (wbuf, xs, ys-yc, cp_red);

   /*** Day and month. ***/

   year = H.year + 1900;
   doy = H.doy;
   (void) month_day (year, doy, &imonth, &iday);
   strcpy (month, month_name (imonth));
   sprintf (wbuf, "%d %s ", iday, month);
   glwstr (wbuf, xs, ys-(int)(2.0*yc), cp_red);
   sprintf (wbuf, "%d", year);
   glwstr (wbuf, xs, ys-(int)(2.5*yc), cp_red);

   /*** Spectral filter: ***/

   switch (H.ifw)
      {
      case 2: sprintf (wbuf,"OPEN  "); break;
      case 6: sprintf (wbuf,"Fe-XIV"); break;
      case 0: sprintf (wbuf,"RED   "); break;
      case 1: sprintf (wbuf,"Halpha"); break;
      case 3: sprintf (wbuf,"BLUE  "); break;
      case 4: sprintf (wbuf,"WIDEBD"); break;
      case 5: sprintf (wbuf,"GREEN "); break;
      default: sprintf (wbuf,"******"); break;
      }
   if (H.iglstw == 1)  sprintf (wbuf,"%s  ", H.glc);
   glwstr (wbuf, xs, ys-(int)(3.5*yc), cp_red);

   /*** Polaroid filter: ***/

   switch (H.ipw)
      {
      case 0: sprintf (wbuf,"CLEAR "); break;
      case 1: sprintf (wbuf,"+60deg"); break;
      case 2: sprintf (wbuf,"-60deg"); break;
      case 3: sprintf (wbuf," 0 deg"); break;
      default: sprintf (wbuf,"***"); break;
      }
   glwstr (wbuf, xs, ys-(int)(4.5*yc), cp_red);

   /*** Sector mirror: ***/

   switch (H.ism)
      {
      case 5:  sprintf (wbuf, "N   "); break;
      case 4:  sprintf (wbuf, "NE  "); break;
      case 3:  sprintf (wbuf, "E   "); break;
      case 2:  sprintf (wbuf, "SE  "); break;
      case 1:  sprintf (wbuf, "S   "); break;
      case 0:  sprintf (wbuf, "SW  "); break;
      case 7:  sprintf (wbuf, "W   "); break;
      case 6:  sprintf (wbuf, "NW  "); break;
      case 8:  sprintf (wbuf, "NESW"); break;
      default: sprintf (wbuf, "****"); break;
      }
   glwstr (wbuf, xs, ys-(int)(5.5*yc), cp_red);

   /*** Exposure (1.024 second units) : ***/

   (void) sprintf (wbuf, "%-4u", H.exp);
   glwstr (wbuf, xs, ys-(int)(6.5*yc), cp_red);

   /*** Resolution : ***/

   if( H.ires == 0 ) (void) sprintf (wbuf, "HIGH");
   else (void) sprintf (wbuf, "LOW ");
   glwstr (wbuf, xs, ys-(int)(7.5*yc), cp_red);

   /*** S/C Roll : ***/

   if (H.icm == 1)
      {
      if (annot_state) 
         {
         glcp_label (hires, 0, 0, see, cp_grey);
         }
      (void) sprintf (wbuf, "%-4d", scroll);
      select_font (font4, fsize);	/* CharterBT-Black font. */
      glwstr (wbuf, xs, ys-(int)(8.5*yc), cp_red);

      annot_state = 1;
      }

   /*** Calibration Mirror: ***/

   else
      {
      switch (H.icm)
         {
         case 0: (void) sprintf (wbuf, "MOVE"); break;
         case 1: (void) sprintf (wbuf, "OUT "); break;
         case 2: (void) sprintf (wbuf, "IN  "); break;
         case 3: (void) sprintf (wbuf, "BAD "); break;
         default: (void) sprintf (wbuf, "****"); break;
         }
      if (annot_state) 
         {
         glcp_label (hires, 0, 0, see, cp_grey);
         }
      select_font (font4, fsize);	/* CharterBT-Black font. */
      glwstr (wbuf, xs, ys-(int)(8.5*yc), cp_red);
      annot_state = 0;
      }

   /*** Image count (frame number) : ***/

   (void) sprintf (wbuf, "%-4u", H.frame);
   glwstr (wbuf, xs, ys-(int)(9.5*yc), cp_red);

   return (0);
}
