/* glcp_label.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label SMM C/P image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  3 April 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/usr/people/smm/lib/include/cplib.ext"
#include "/usr/people/smm/lib/include/cp_color.h"
#include "/usr/people/smm/lib/include/grscale.ext"

glcp_label (hires, linit, lbottom, see, colorval)

int hires;	/* =0 for 512x512 resolution, =1 for 1024x1024 resolution. */
int linit;	/* Initialization flag. */
int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int see;	/* Display flag: =0 no, =1 yes. */
int colorval;	/* Color level. */

{
   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    */
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
   static char fonts [32];			/* Large font. */
   static char fontl [32];			/* Small font. */
  
   static short *cbar;		/* Color bar array. */
   int charheight = 43;
   int sizes, sizel;
   int xstart = 2;
   int ystart = 501;
   int startrow, startcol, endcol, i;
   int step;
   int colormax = 255;			/* Maximum color value. */
   int barlength = 512, barwidth = 12;	/* color bar dimensions. */
   int ppc = 2;		/* # pixels per color in color bar. */
   int csx = 6, csy = 7;/* # pixels in character for screen.7 font. */
   int cby0, cby1;	/* color bar y-axis locations. */
   int cwx, cwx15;	/* # pixels for character width. */
   int xc, yc;
   int xs, ys;
   int x1 = 0, x2 = 62, x3 = 511;
   int y1 = 0, y2 = 62, y3 = 511;
   int xwl, xil, xwr;
   int ywb, yib, ywt;

   if (!see) return (0);	/* If image display option is off. */

   startrow = 16 + barwidth;
   startcol = 0;
   endcol = startcol + barlength - 1;
   charheight = 43;

   if (hires)
      {
      xs = xstart * 2;
      ys = ystart * 2;
      yc = charheight * 2;
      strcpy (fonts, font1);
      sizes = 10;
      strcpy (fontl, font4);
      sizel = 25;
      xwl = x1 * 2;  xil = x2 * 2;  xwr = x3 * 2;
      ywb = y1 * 2;  yib = y2 * 2;  ywt = y3 * 2;
      csx = 14;   csy = 15;
      ppc = 4;
      barlength = 1024;
      barwidth = 24;
      }
   else
      {
      xs = xstart;
      ys = ystart;
      yc = charheight;
      strcpy (fonts, font4);
      sizes = 7;
      strcpy (fontl, font4);
      sizel = 14;
      xwl = x1;  xil = x2;  xwr = x3;
      ywb = y1;  yib = y2;  ywt = y3;
      csx = 6;   csy = 7;
      ppc = 2;
      barlength = 512;
      barwidth = 12;
      }

   if (linit)
      {		/*** Draw box around image window. ***/

      glwvec (xwl, ywb, xwr, ywb, colorval);
      glwvec (xwr, ywb, xwr, ywt, colorval);
      glwvec (xwr, ywt, xwl, ywt, colorval);
      glwvec (xwl, ywt, xwl, ywb, colorval);
      glwvec (xwl, yib, xwr, yib, colorval);
      glwvec (xil, yib, xil, ywt, colorval);

      /*** Write annotation labels into frame buffer. ***/

      select_font (fonts, sizes);	/* Small font & size. */
      glwstr ("DATE",       xs, ys,              colorval);
      glwstr ("TIME",       xs, ys-1*yc, colorval);
      glwstr ("FILTER",     xs, ys-(int)(3.5*yc), colorval);
      glwstr ("POLAROID",   xs, ys-(int)(4.5*yc), colorval);
      glwstr ("SECTOR",     xs, ys-(int)(5.5*yc), colorval);
      glwstr ("EXPOSURE",   xs, ys-(int)(6.5*yc), colorval);
      glwstr ("RESOLUTION", xs, ys-(int)(7.5*yc), colorval);
      }

   select_font (fonts, sizes);	/* Small font & size. */
   if (H.icm == 2)
      glwstr ("CAL MIRROR", xs, ys-(int)(8.5*yc), colorval);
   else
      glwstr ("ROLL      ", xs, ys-(int)(8.5*yc), colorval);

   if (linit)
      {
      glwstr ("FRAME  ", xs, ys-(int)(9.5*yc), colorval);

      /*** Write HAO logo & colorval bar only if 'lbottom = 0'. ***/

      if (lbottom == 1)  return (0);

      /*** Add annotation for the color bar ***/

      select_font (fonts, sizes);	/* Small font & size. */
      cby0 = 1 * (hires+1);
      cwx = csx * (hires+1);
      cwx15 = (int) ((cwx * 1.5) + 0.5);
      glwstr ("0"  , (startcol+1),             cby0, colorval);
      glwstr ("64" , (startcol+64*ppc-cwx),    cby0, colorval);
      glwstr ("128", (startcol+128*ppc-cwx15), cby0, colorval);
      glwstr ("192", (startcol+192*ppc-cwx15), cby0, colorval);
      glwstr ("255", (endcol-cwx*3),           cby0, colorval);

      /*** Add tick marks to color bar ***/

      cby0 += (csy * (hires+1)) + 2;
      cby1 = cby0 + 4 * (hires+1);
      glwvec (startcol,         cby0, (startcol),         cby1, colorval);
      glwvec (startcol+32*ppc , cby0, (startcol+32*ppc),  cby1, colorval);
      glwvec (startcol+64*ppc , cby0, (startcol+64*ppc),  cby1, colorval);
      glwvec (startcol+96*ppc , cby0, (startcol+96*ppc),  cby1, colorval);
      glwvec (startcol+128*ppc, cby0, (startcol+128*ppc), cby1, colorval);
      glwvec (startcol+160*ppc, cby0, (startcol+160*ppc), cby1, colorval);
      glwvec (startcol+192*ppc, cby0, (startcol+192*ppc), cby1, colorval);
      glwvec (startcol+224*ppc, cby0, (startcol+224*ppc), cby1, colorval);
      glwvec (startcol+255*ppc, cby0, (startcol+255*ppc), cby1, colorval);

      /*** Draw the color bar (2 pixels wide per color). ***/

      barlength = colormax * ppc;
      cbar = (short *) malloc (barlength);
      for (step = 0; step <= colormax; step+=ppc)
         {
         for (i=0; i<ppc; i++)
            {
            cbar [step+i] = (short) (i & 0377);
            }
         }
      free (cbar);

      startrow = cby1 + 1;
      for (i = 0; i < barwidth; i++)
         {
         rectwrite (startcol, startrow, endcol, startrow, cbar);
         startrow++;
         }

      /*** Write HAO ID. ***/

      select_font (fontl, sizel);	/* Large font & size. */
      glwstr (" HAO/SMM   CORONAGRAPH / POLARIMETER ",
              xil+((hires+1)*2), startrow+2*(hires+1), cp_green);
      }
   return (0);
}
