/* pmcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Merge Prominence Monitor & SMM C/P images together.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C/P image must be low resolution, and calibrated.
 * Pmon image is an Eikonix-digitized image file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 *  2 June 1989: Use Eikonix-digitized images (512x512 byte array).
 * 12 July 1989: Use Eikonix-digitized images (450x450 + 128-byte header).
 * 30 August 1989: version for Grinnell-274.
 * 19 October 1989: use rdpmon6.c
 *  2 January 1991: add prompt for Pmon intensity rescaling.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/usr/people/smm/lib/include/cplib.h"
#include "/usr/people/smm/lib/include/grscale.h"
#include "/usr/people/smm/lib/include/cp_color.h"

main (argc, argv)

int argc;
char *argv [];
{

unsigned short *rdpmon ();
unsigned short *rdmk3 ();
unsigned short *rdcp ();
unsigned short *rotate_img ();
unsigned short *magimg ();
unsigned short *merge2s ();

int mskimg ();
int rescale ();
int wdisk ();

static char answer [80];
static char namepm [80] = "pmon.218-80.191310";	/* Pmon image file name. */
static char namemk [80] = "19:13.srq";	/* Mark-III image file name. */
static char namecp [80] = "021819155.c04";	/* C/P image file name. */
static char cptime[6] = "HH:MM";
static char pmontime [6] = "HH:MM";
static char mk3time [6] = "HH:MM";
static char nampgm [80];
static char namout [80] = "cor.img";

unsigned short *pmims;	/* Pointer to Prominence Monitor image array. */
unsigned short *mkims;	/* Pointer to Mark-III image array. */
unsigned short *cpims;	/* Pointer to SMM C/P image array. */

unsigned short *pmimr;	/* Pointer to Prominence Monitor image array. */
unsigned short *mkimr;	/* Pointer to Mark-III image array. */
unsigned short *cpimr;	/* Pointer to SMM C/P image array. */

unsigned short *pmimg;	/* Pointer to Prominence Monitor image array. */
unsigned short *img2;	/* Pointer to composite image array. */
unsigned char  *cimg;	/* Pointer to output image array. */

int i;
int nbypx = 2;
int npximg;
int xl = 760;		/* Specification of GL window position & size. */
int yl = 480;
int xu = 1271;
int yu = 991;

static int ngr = 0;
static int imgch0 = 0, imgch1 = 1, imgch2 = 2, imgch3 = 3;

static int pmdx, pmdy, pmnx, pmny;
static int pmcx, pmcy, ipmdsk, ipmrad;

static int mkdx, mkdy, mknx, mkny;
static int mkcx, mkcy, imkdsk, imkrad;

static int cpdx = 448, cpdy = 448, cpnx = 448, cpny = 448;
static int cpcx, cpcy, icpdsk, icprad;

static int xgs = 63, ygs = 63;
static int xs = 0, ys = 0;
static int ithresh = 175;
static int level = 0;
static int pmmin1 = 0, pmmax1 = 250, pmmin2 = 0, pmmax2 = 250;
static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250;
static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250;
static int grnlo = 0, grnhi = 249;

/* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

static float cpcxnew [] = {  7., 100., 225., 398., 441., 348., 225., 50., -1.};
static float cpcynew [] = {225., 348., 441., 398., 258., 100.,   7., 50., -1.};
static float fcpcx, fcpcy;
static float fx, fy;
static float pmfac = 1.0;
static float pmdsk = 0.99,  pmrad = 2.0;
static float mkdsk = 1.08, mkrad = 1.8;
static float cpdsk = 1.5,  cprad = 1.6;
static float pmang =  0.0, mkang =  0.0, cpang = 43.0;
static float pmpxrs = 117.5, mkpxrs = 104.4435, cppxrs = 80.9877, impxrs = 54.0;
mkpxrs = (511.0 / (2.0 * (1.1224 + 127.0 * 0.0104244)));   /* scale 13 Dec 90 */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   /* Request input information. */

   fprintf (stderr, "Enter file name of SMM C/P image: ");
   fscanf  (stdin, "%s", namecp);
/*
 *   fprintf (stderr, "Enter minimum radius for SMM C/P image: ");
 *   fscanf  (stdin, "%f", &cprad);
 *
 *   fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ");
 *   fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2);
 */

/*
 *   fprintf (stderr, "Enter file name of MLSO Mark-III image: ");
 *   fscanf  (stdin, "%s", namemk);
 *   fprintf (stderr, "Enter time (HR:MIN) of MLSO Mark-III image: ");
 *   fscanf  (stdin,  "%s", mk3time);
 *
 *   fprintf (stderr, "Enter maximum radius for MLSO Mark-III image: ");
 *   fscanf  (stdin, "%f", &mkrad);
 */
    fprintf (stderr, "Enter file name of MLSO Prominence Monitor image: ");
    fscanf  (stdin, "%s", namepm);
/* 
 *    fprintf (stderr, "Enter time (HR:MIN) of Prominence Monitor image: ");
 *    fscanf  (stdin,  "%s", pmontime);
 * 
 *    fprintf (stderr, "Enter maximum radius for Prominence Monitor image: ");
 *    fscanf  (stdin, "%f", &pmrad);
 */
   fprintf (stderr, "Do you wish to rescale Pmon image intensities [y/n] ? ");
   fscanf  (stdin, "%s", answer);
   if (strncmp (answer, "y", 1) == 0)
      {
      fprintf (stderr, "Enter Pmon oldmin, oldmax, newmin, newmax: ");
      fscanf  (stdin, "%d,%d,%d,%d", &pmmin1, &pmmax1, &pmmin2, &pmmax2);
      }

   fprintf (stderr, "Enter threshold DN level for Pmon image: ");
   fscanf  (stdin, "%d", &ithresh);

   icprad = (int) (cprad * cppxrs);
   imkrad = (int) (mkrad * mkpxrs);
   icpdsk = (int) (cpdsk * cppxrs);
   imkdsk = (int) (mkdsk * mkpxrs);

   /*** Initialize SGI Graphics Library. ***/

   foreground ();
   prefposition (xl, xu, yl, yu);
   winopen (nampgm);
   gl_wclear (BLACK);
   onemap ();			/* Set single-map mode for color table. */
   get_cmap ();			/* Save color map. */

   multimap ();			/* Set multi-map mode for color table. */
   gconfig ();
   setmap (2);			/* Select color map #2. */

   fminit ();			/* Initialize SGI font manager. */

   /*
    * Read SMM C/P image file.
    * Write black disk.
    * Magnify image.
    */

   printf ("Read C/P image.\n");
   cpims = rdcp (namecp, &cpdx, &cpdy, &cpcx, &cpcy);
   sprintf (cptime, "%02d:%02d", H.hour, H.minute);
   fcpcx = cpcxnew[H.ism];
   fcpcy = cpcynew[H.ism];
   printf ("fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   printf ("Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2);
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1);
   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   printf ("Write disk into C/P image . . . cpcx:%d   cpcy:%d   icprad:%d\n",
            cpcx, cpcy, icprad);
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level);
   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru");
   sleep (1);

   printf ("Magnify C/P image.\n");
   fx = fy = mkpxrs / cppxrs;
   cpimr = (unsigned short *) malloc (cpdx * cpdy * sizeof(short));
   expcon (cpims, cpdx, cpdy, cpimr, cpdx, cpdy, fx, fy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy);
   free (cpims);
   imgwgl (cpimr, nbypix, xs, ys, cpnx, cpny, "ru");
   sleep (1);

   /*
    * Read MLSO Prominence Monitor image file.
    * Write black disk.
    * Rotate image.
    * Mask image beyond maximum radius.
    * Minify image.
    * Rescale intensity levels.
    */

   printf ("Read Pmon image.\n");
   pmims = rdpmon (namepm, &pmdx, &pmdy, &pmcx, &pmcy, pmontime, &pmpxrs, &pmang);
   pmang -= cpang;
   ipmdsk = (int) (pmdsk * pmpxrs);
   ipmrad = pmrad * pmpxrs;
   printf ("pmdx: %d   pmdy: %d   pmcx: %d   pmcy: %d\n", pmdx,pmdy,pmcx,pmcy);

   printf ("Rescale Pmon image.\n");
   rescale (pmims, pmdx, pmdy, pmmin1, pmmax1, pmmin2, pmmax2, grnlo, grnhi);

   gl_wclear (BLACK);
   imgwgl (pmims, nbypx, 0, 0, pmdx, pmdy, "ru");
   sleep (1);

   printf ("Write %6.2f Rsun disk into Pmon image.\n", pmdsk);
   wdisk (pmims, pmdx, pmdy, pmcx, pmcy, ipmdsk, level);

   imgwgl (pmims, nbypx, 0, 0, pmdx, pmdy, "ru");
   sleep (1);

   pmang -= (float) scroll;
   printf ("Rotate Pmon image . . . pmang: %f\n", pmang);
   pmimr = rotate_img (pmims, pmdx, pmdy, pmcx, pmcy, pmang);
   free (pmims);

   imgwgl (pmimr, nbypx, 0, 0, pmdx, pmdy, "ru");
   sleep (1);

   printf ("Magnify Pmon image.\n");
   pmnx = cpnx; pmny = cpny;
   fx = fy = mkpxrs / pmpxrs;
   pmimg = (unsigned short *) malloc (pmnx * pmny * sizeof(short));
   expcon (pmimr, pmdx, pmdy, pmimg, pmnx, pmny, fx, fy,
           (float) pmcx, (float) pmcy, fcpcx, fcpcy);
   free (pmimr);

   imgwgl (pmims, nbypx, xs, ys, pmnx, pmny, "ru");
   sleep (1);

   printf ("Mask Pmon image beyond %4.2f Rsun.\n", pmrad);
   pmfac = fx;
   ipmrad = pmrad * pmfac * pmpxrs;
   printf ("pmfac: %6.2f   pmpxrs: %6.2f   ipmrad: %d\n", pmfac,pmpxrs,ipmrad);
   pmcx = (int) fcpcx;
   pmcy = (int) fcpcy;
   mskimg (pmimg, 0, 0, pmnx, pmny, pmcx, pmcy, ipmrad);

   gl_wclear (BLACK);
   imgwgl (pmims, nbypx, xs, ys, pmnx, pmny, "ru");
   sleep (1);

/*
 *   printf ("Rescale Pmon image.\n");
 *   rescale (pmimg, pmnx, pmny, pmmin1, pmmax1, pmmin2, pmmax2);
 */

    /* Add two images together. */

   img2 = merge2s (cpimr, pmimg, cpnx, cpny, ithresh);

   /*** Display composite image with annotation. ***/

   gl_wclear (BLACK);
   ldisk = 1;
   glcp_label (0, 1, 1, 1, cp_grey, 1);
   glcp_annot (0, 1);

   imgwgl (img2, nbypx, xgs, ygs, cpnx, cpny, "ru");/* Write merged image.*/
   free (img2);

   select_font ("Charter-Black", 12);	/* Helvetica-Bold, 16x17 characters. */
   glwstr ("HAO/SMM CORONAGRAPH/POLARIMETER", 63, 38, cp_green);
/*
 *   glwstr ("MLSO MARK-III K-CORONAMETER    ", 63, 20, cp_green);
 */
   glwstr ("MLSO PROMINENCE MONITOR        ", 63,  2, cp_green);

   glwstr (pmontime, 444,  2, cp_red);
/*
 *   glwstr (mk3time,  444, 20, cp_red);
 */
   glwstr (cptime,   444, 38, cp_red);

   fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy);
   xcen = fcpcx;
   ycen = 447 - fcpcy;
   fx = fy = mkpxrs / cppxrs;
   pixrs = pixrs * fx;
   glcp_north ((int) fcpcx, (int) fcpcy, 0.1, 0.3, cp_yellow);
   glcp_scalers (0, cp_yellow);
   glcp_suncir (0.4, 1.0, 0.1, 90.0, 0.4, 1.0, 0.2, 30.0, cp_yellow);

   /*** Write composite image to disc file (WHIP pic format). ***/

   npximg = 512 * 512;
   img2 = (unsigned short *) malloc (npximg * sizeof (short));
   cimg = (unsigned char *) malloc (npximg);
   rectread (0, 0, 511, 511, img2);
   for (i = 0; i < npximg; i++)
      *(cimg + i) = (unsigned char) (*(img2 + i) & 0377);
   imgpic (cimg, 1, 512, 512, "ru", namout);

   sleep (10);
   gexit ();
   exit (0);
}
