/* rdcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 23 September 1987
 */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned short *
rdcp (namimg, dx, dy, cx, cy)

char  *namimg;		/* Pointer to image disk file name. */
int   *dx, *dy;		/* Dimensions of image array. */
int   *cx, *cy;		/* Disk center coordinates. */

{
   extern char *malloc ();	/* System memory allocation function. */
   register unsigned short *ims;/* Pointer to short integer image array. */
   register int imgfd;		/* File descriptor for image disk file. */
   register int ix, iy;
   register unsigned short *sline;
   register char *bline;

   int iybeg;
   int nbyhdp;			/* Bytes per pixel in housekeeping & bitmap. */
   int nbylin;			/* # bytes per line. */

   /* Sector mirror order for xoff/yoff : sw, s, se, e, ne, n, nw, w, nesw. */

   static int xoff [9] = {0, 0, 0, 0, 0, 0, 0, 0, -1};
   static int yoff [9] = {0, 0, 0, 0, 0, 0, 0, 0, -1};

   /* Decommutate Housekeeping. */

   cphskp (namimg);	
   if (H.ires == 0)
      {
      fprintf (stderr, "C/P image is high resolution: not allowed.\n");
      exit (-1);
      }
   *dx = H.lp - H.ip + 1;
   *dy = H.lr - H.ir + 1;
   *cx = (int) (H.xcen + 0.5) + xoff [H.ism];
   *cy = (int) (447.0 - H.ycen + 0.5) + yoff [H.ism];

   /* Open image file. */

   if ((imgfd = open (namimg, 0)) < 0)
     { fprintf (stderr, "rdcp> Can't open file %s\n", namimg);  return (NULL); }

   /* Position I/O pointer to beginning of video data in image disk file. */

   if (nbypix == 4) nbyhdp = 1;
   else
      nbyhdp = nbypix;
   if (lseek (imgfd, (long) (npix*nbyhdp*2), 0) == -1)
      { fprintf (stderr, "rdcp> seek error.\n");  return (NULL); }

   printf ("rdcp> npix: %d   npixl: %d   nbypix: %d\n", npix, npixl, nbypix);
   if ( (ims = (unsigned short *) malloc (npixl*npixl*nbypix)) == NULL )
     {  fprintf (stderr, "rdcp> malloc error.");
        exit (1);
     }

   /* Read image, one line at a time, and invert it vertically. */

   nbylin = npixl * nbypix;
   iybeg = npixl - 1;

   if (nbypix == 2)
      {
      sline = (unsigned short *) malloc (npixl * sizeof (short));
      for (iy = iybeg; iy >= 0; iy--)
         {
         if (read (imgfd, sline, nbylin) != nbylin)
            fprintf (stderr, "rdcp> nread != nbylin:%d\n", nbylin);
         if (bytord == 1)  srevbyt (sline, nbypix, nbylin);
         for (ix = 0; ix < npixl; ix++)
            *(ims + ix + iy * npixl) = *(sline + ix);
         }
      free (sline);
      }
   else
      {
      bline = (char *) malloc (npixl * sizeof(char));
      for (iy = iybeg; iy >= 0; iy--)
         {
         if (read (imgfd, bline, nbylin) != nbylin)
            fprintf (stderr, "rdcp> nread != nbylin:%d\n", nbylin);
         for (ix = 0; ix < npixl; ix++)
            *(ims + ix + iy * npixl) = (unsigned short) *(bline + ix);
         }
      free (bline);
      }

   if (close (imgfd) == -1)
      {
      fprintf (stderr, "rdcp> file close error.%s\n", namimg);
      return (NULL);
      }

  return (ims);
}
