/* rdfcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 23 September 1987
 * 18 August 1993: adapt to accept short AND floating point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.ext"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned short *
rdfcp (namimg, dx, dy, cx, cy)

char  *namimg;		/* Pointer to image disk file name. */
int   *dx, *dy;		/* Dimensions of image array. */
int   *cx, *cy;		/* Disk center coordinates. */

{
   extern char *malloc ();	/* System memory allocation function. */
   register float *imf;		/* Pointer to floating point image array. */
   register unsigned short *ims;/* Pointer to short integer image array. */
   register int imgfd ;		/* File descriptor for image disk file. */
   register int ix, iy;
   register unsigned short *sline;
   register char *bline;
   register float *fline;

   int iybeg;
   int nbyhdp;			/* Bytes per pixel in housekeeping & bitmap. */
   int nbylin;			/* # bytes per line. */

   /*** Sector mirror order for x/yoff: sw, s, se, e, ne, n, nw, w, nesw. ***/

   static int xoff [9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};
   static int yoff [9] = {0, 0, 0, 0, 0, 0, 0, 0, 0};

   float dmin, dmax, slope ;		/* Data min/max & slope. */
   float fgmin = 0.0, fgmax = 249.0;	/* Min/Max limits allowed for display.*/
   float newval;			/* Pixel display value. */

   /*** Decommutate Housekeeping. ***/

   cphskp (namimg);	
   if (H.ires == 0)
      {
      fprintf (stderr, "C/P image is high resolution: not allowed.\n");
      exit (-1);
      }

   /*** Determine data minimum & maximum for scaling display image. ***/

   switch (idtype)
      {
      case 1:  dmin =     0.0 ;	dmax =   250.0 ; break;	/* raw DN     */
      case 2:  dmin =     0.0 ;	dmax =   250.0 ; break;	/* Eq. Sec.   */
      case 3:  dmin =     0.0 ;	dmax =   500.0 ; break;	/* Cal. Btotal*/
      case 4:  dmin =     0.0 ;	dmax = 5.0e+15 ; break;	/* elec/cm2   */
      case 5:  dmin = -2.0e-9 ;	dmax = 5.0e+15 ; break;	/* B2 - B1    */
      case 6:  dmin =     0.0 ;	dmax = 2.5e-09 ; break;	/* Bk         */
      case 7:  dmin =    -1.0 ;	dmax =     1.0 ; break;	/* (Bk-B1)/Bk */
      case 8:  dmin =    -1.0 ;	dmax =     1.0 ; break;	/* (B2-B1)/B1 */
      case 9:  dmin =     0.0 ;	dmax =     1.0 ; break;	/* pol. frac. */
      case 10: dmin =     0.0 ;	dmax = 5.0e-09 ; break;	/* pB         */
      case 11: dmin =     0.0 ;	dmax = 5.0e-09 ; break;	/* Br+t       */
      case 12: dmin =     0.0 ;	dmax = 5.0e-09 ; break;	/* Bt         */
      case 13: dmin =     0.0 ;	dmax = 5.0e-09 ; break;	/* Br         */
      case 14: dmin =     0.0 ;	dmax = 2.5e-09 ; break;	/* Bf + Bs    */
      case 15: dmin =     0.0 ;	dmax =     2.0 ; break;	/* B2 / B1    */
      case 16: dmin =     0.0 ;	dmax = 2.0e-09 ; break;	/* B misc     */
      default: dmin =   0.0 ;	dmax =   250.0 ;
      }
   slope = 250.0 / (dmax - dmin) ;
   fprintf (stderr, "rdfcp> idtype: %d  dmin:%8.2g  dmax:%8.2e  slope:%8.2g\n",
	    idtype, dmin, dmax, slope);

   *dx = H.lp - H.ip + 1;
   *dy = H.lr - H.ir + 1;
   *cx = (int) (H.xcen + 0.5) + xoff [H.ism];
   *cy = (int) (447.0 - H.ycen + 0.5) + yoff [H.ism];
   printf ("H.xcen: %6.2f   H.ycen: %6.2f   H.ism: %d\n", H.xcen, H.ycen, H.ism);

   /*** Open image file. ***/

   if ((imgfd = open (namimg, 0)) < 0)
      {fprintf (stderr, "rdfcp> Can't open file %s\n", namimg);  return (NULL);}

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

   if (nbypix == 4)
      nbyhdp = 1;
   else
      nbyhdp = nbypix;

   if (lseek (imgfd, (long) (npix * nbyhdp * nbypix), 0) == -1)
      {
      fprintf (stderr, "rdfcp> seek error.\n");
      return (NULL);
      }

   printf ("rdfcp> npix: %d   npixl: %d   nbypix: %d\n", npix, npixl, nbypix);

   if ( (ims =
	(unsigned short *) malloc (npixl * npixl * sizeof (short))) == NULL )
      {
      fprintf (stderr, "rdfcp> malloc error.");
      exit (-2);
      }

   /*** Read image, one line at a time, and invert it vertically. ***/

   nbylin = npixl * nbypix;
   iybeg = npixl - 1;

   if (nbypix == 1)
      {
      bline = (char *) malloc (npixl * sizeof (char));
      for (iy = iybeg; iy >= 0; iy--)
         {
         if (read (imgfd, bline, nbylin) != nbylin)
            fprintf (stderr, "rdfcp> nread != nbylin:%d\n", nbylin);
         for (ix = 0; ix < npixl; ix++)
            *(ims + ix + iy * npixl) = (unsigned short) *(bline + ix);
         }
      free (bline);
      }
   else
   if (nbypix == 2)
      {
      sline = (unsigned short *) malloc (npixl * sizeof (short));
      for (iy = iybeg; iy >= 0; iy--)
         {
         if (read (imgfd, sline, nbylin) != nbylin)
            fprintf (stderr, "rdfcp> nread != nbylin:%d\n", nbylin);
         if (bytord == 1)  srevbyt (sline, nbypix, nbylin);
         for (ix = 0; ix < npixl; ix++)
            {
            newval = (float) ((*(sline + ix) - dmin) * slope);
            if (newval < fgmin)  newval = fgmin;
            if (newval > fgmax)  newval = fgmax;
            *(ims + ix + iy * npixl) = ((unsigned short) newval) & 0177777;
	    }
         }
      free (sline);
      }
   else
   if (nbypix == 4)
      {
      fline = (float *) malloc (npixl * sizeof (float));
      for (iy = iybeg; iy >= 0; iy--)
         {
         if (read (imgfd, fline, nbylin) != nbylin)
            fprintf (stderr, "rdfcp> nread != nbylin:%d\n", nbylin);
         for (ix = 0; ix < npixl; ix++)
            {
            newval = (float) ((*(fline + ix) - dmin) * slope);
            if (newval < fgmin)  newval = fgmin;
            if (newval > fgmax)  newval = fgmax;
            *(ims + ix + iy * npixl) = ((unsigned short) newval) & 0177777;
	    }
         }
      free (fline);
      }

   if (close (imgfd) == -1)
      {
      fprintf (stderr, "rdfcp> file close error.%s\n", namimg);
      return (NULL);
      }

  return (ims);
}
