/* rdmk3s.c
 * *****************************************************************************
 * Read MLSO Mark-III image into a short int array.
 * *****************************************************************************
 */

/* -------------------------------------------------------------------------- */
#include "/usr/people/stanger/lib/include/mk3lib.ext"

unsigned short *
rdmk3s (filename, mkdx, mkdy, mkcx, mkcy, pmode)

char filename [] ;			/* MK3 image file name. */
int *mkdx, *mkdy ;			/* MK3 image dimensions. */
int *mkcx, *mkcy ;			/* MK3 disc center coordinates. */
int pmode ;				/* Diagnostic print mode. */

{
   unsigned char *imgbuf ;
   unsigned short *sbuf ;

   int fin ;
   int idtype ;
   int imin, imax ;
   int nbyimg ;
   int nbypix ;
   int nread ;
   int xdim, ydim ;

   float smin, smax ;

   /*** Read image into memory. ***/

   mk3hskp (filename, pmode);
   xdim = MK3I->xdim ;
   ydim = MK3I->ydim ;
   *mkdx = xdim ;
   *mkdy = ydim ;
   *mkcx = xdim / 2 ;
   *mkcy = ydim / 2 ;
   nbypix = MK3I->nbypx ;
   nbyimg = MK3I->xdim * MK3I->ydim * MK3I->nbypx;
   imgbuf = malloc ( nbyimg );
   if ((nread = read (fin, imgbuf, nbyimg)) <= 0 )
      {
      fprintf (stderr, "mk3 read error.");
      return (-3);
      }
   close ( fin );

   if ( nbypix == 1 )           /* 1 byte per pixel. */
      {
      if (lscale == 1)          /* User-supplied scaling parameters. */
         {
	 smin = (float) oldmin;
         smax = (float) oldmax;
         imin = newmin;
         imax = newmax;
         }
      else
         {
         smin =   0.0;
         smax = 250.0;

         if (idtype ==  2)   {smin =     0.0; smax =   250.0;}  /* srq  */
         if (idtype ==  5)   {smin =     0.0; smax =   250.0;}  /* sri  */
         if (idtype == 11)   {smin =     0.0; smax =   250.0;}  /* srpB */

         smin = (float) MK3I->smin ;
         smax = (float) MK3I->smax ;
         imin =   0;
         imax = 250;
         }
      MK3I->smin = smin;
      MK3I->smax = smax;
      mk3scale (imgbuf, xdim, ydim, nbypix, smin, smax, imin, imax);
      }
   else if (nbypix == 2)	/* 2 bytes per pixel (short integer). */
      {
      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 smin = (float) oldmin;
	 smax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else			/* Default short integer intensity scaling. */
         {
	 smin =   0.0;
	 smax = 500.0;

	 if (idtype ==  0)   {smin = - 500.0; smax =  5000.0;}  /*   q  */
	 if (idtype ==  1)   {smin = - 500.0; smax =  5000.0;}  /*  rq  */

	 if (idtype ==  3)   {smin = - 500.0; smax =  5000.0;}  /*   i  */
	 if (idtype ==  4)   {smin = - 500.0; smax =  5000.0;}  /*  ri  */

	 if (idtype ==  6)   {smin = -5000.0; smax =  5000.0;}  /*   u  */
	 if (idtype ==  7)   {smin = -5000.0; smax =  5000.0;}  /*  ru  */

	 if (idtype ==  9)   {smin = - 250.0; smax =  2500.0;}  /*  pB */
	 if (idtype == 10)   {smin = - 500.0; smax =  7500.0;}  /*  rpB */

         smin = (float) MK3I->smin ;
         smax = (float) MK3I->smax ;
	 imin =   0;
	 imax = 250;
	 if (pmode)
	    {
            fprintf (stdout, "rdmk3s> xs: %d  ys: %d  xdim: %d  ydim: %d  nbypix: %d  idtype: %d\n",
	             xs, ys, xdim, ydim, nbypix, idtype);
	    fprintf (stdout, "rdmk3s> smin/max: %7.2f %7.2f  imin/max: %3d %3d\n",
		     smin, smax, imin, imax);
            }
         }
      MK3I->smin = smin;
      MK3I->smax = smax;
      mk3scale (imgbuf, xdim, ydim, nbypix, smin, smax, imin, imax);
      }
   else if (nbypix == 4)	/* 4 bytes per pixel (floating point). */
      {
      if (lscale == 1)
	 {
	 smin = (float) oldmin;
	 smax = (float) oldmax;
	 imin = newmin;
	 imax = newmax;
	 }
      else
	 {
         smin = MK3I->smin ;
         smax = MK3I->smax ;
	 imin = 0;
	 imax = 250;
	 }
      MK3I->smin = smin;
      MK3I->smax = smax;
      mk3scale (imgbuf, xdim, ydim, nbypix, smin, smax, imin, imax);
      }
   else
      {
      fprintf (stderr, "rdmk3s> Invalid image . . . nbypix: %d\n", nbypix);
      return (-4);
      }

   sbuf = (unsigned short *) imgbuf ;
   free (imgbuf);

   return (sbuf) ;
}



/* -----------------------------------------------------------------------------
 * Rescale MLSO MK3 image to 8-bit integer stored in unsigned short.
 * -----------------------------------------------------------------------------
 */
unsigned short *
mk3scale (imgbuf, xdim, ydim, nbypix, smin, smax, imin, imax) ;

unsigned char *imgbuf ;		/* Image array. */
int xdim, ydim ;		/* Image dimensions. */
int nbypix ;			/* # bytes per pixel. */
float smin, smax ;		/* Input min/max. */
int imin, imax ;		/* Output min/max. */

{
   unsigned short *outbuf ;
   unsigned char  *cbuf ;
   unsigned short *sbuf ;
   float          *fbuf ;
   float offset = 0.0 ;
   float slope  = 1.0 ;
   float fval ;
   unsigned short usval ;
   int ix ;
   int iy ;

   offset = smin ;
   slope  = (smax - smin) / (float) (imax - imin) ;

   if (nbypix ==  1)
      {
      for (iy = 0; iy < ydim ; iy++)
         for (ix = 0; ix < xdim ; ix++)
            {
	    fval = (float) (*(cbuf + ix + iy*xdim) - offset) * slope ;
	    usval = (unsigned short) fval  & 0377;
	    *(usbuf + ix + iy*xdim) = (unsigned short *) usval ;
	    }
   else
   if (nbypix ==  1)
      {
      for (iy = 0; iy < ydim ; iy++)
         for (ix = 0; ix < xdim ; ix++)
            {
	    fval = (float) (*(sbuf + ix + iy*xdim) - offset) * slope ;
	    usval = (unsigned short) fval  & 0377;
	    *(usbuf + ix + iy*xdim) = (unsigned short *) usval ;
	    }
   if (nbypix ==  1)
      {
      for (iy = 0; iy < ydim ; iy++)
         for (ix = 0; ix < xdim ; ix++)
            {
	    fval = (float) (*(fbuf + ix + iy*xdim) - offset) * slope ;
	    usval = (unsigned short) fval  & 0377;
	    *(usbuf + ix + iy*xdim) = (unsigned short *) usval ;
	    }
   else
      return (NULL) ;      

   return (outbuf) ;
}
