/* rdpmon.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read MLSO Prominence Monitor image (Eikonix) from disk into memory.
 * Image has a 452-byte header.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 12 July 1989
 * 18 September 1989: revise to accomodate new Pmon format (452x453).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

unsigned short *
rdpmon (namimg, dx, dy, cx, cy, pmtime, pixrs, ang)

char *namimg;		/* Name of image file. */
int *dx, *dy;		/* Dimensions of Prominence Monitor image. */
int *cx, *cy;		/* Center coordinates. */
char *pmtime;
float *pixrs, *ang;

{
   extern char *malloc();
   char *linbuf;
   unsigned short *ims;
   int x, y;
   int hdrsiz = 452;
   int imgsiz, linsiz;
   int imgfd, nread;
   int pmyear, pmdoy, pmhour, pmmin, pmsec;	/* Image date & time. */
   int pmdx, pmdy;				/* Image dimensions. */
   int pmcx, pmcy;				/* Disk center coordinates. */
   int xdim, ydim;
   float pmpixrs, pmang;			/* pixel scale size & angle. */

   /*** Open disk file and read image into memory. ***/

   if ((imgfd = open (namimg, 0)) <= 0)
      {
      fprintf (stderr, "***rdpmon> Unable to open Pmon image: %s\n", namimg);
      exit(-1);
      }
   linbuf = malloc (hdrsiz+1);
   nread = read (imgfd, linbuf, hdrsiz);
   *(linbuf + hdrsiz) = '\0';
   fprintf (stdout, "   pm header:\n   %s\n", linbuf);

   /*** Extract information from Prominence monitor header. ***/

   rdpmhead (linbuf, &pmdx, &pmdy, &pmcx, &pmcy, &pmyear, &pmdoy, &pmhour, &pmmin, &pmpixrs, &pmang);
   free (linbuf);
   sprintf (pmtime, "%02d:%02d", pmhour, pmmin);

   *dx = xdim = pmdx;
   *dy = ydim = pmdy;
   *cx = pmcx;
   *cy = pmcy;
   *pixrs = pmpixrs;
   *ang = pmang;

   imgsiz = xdim * ydim * sizeof(unsigned short);
   linsiz = xdim * sizeof (char);
   linbuf = (char *) malloc (linsiz);
   ims   = (unsigned short *) malloc (imgsiz);
   fprintf (stdout, "   rdpmon> xdim: %d   ydim: %d   imgsiz: %d\n",
            xdim, ydim, imgsiz);

   /*** Read image into short integer memory array. ***/
   /*** Reverse image top to bottom. ***/

   for (y=ydim-1; y>=0; y--)
      {
      nread = read (imgfd, linbuf, linsiz);
      for (x = 0; x < xdim; x++)
         *(ims + x + y * xdim) = (unsigned short) (*(linbuf + x)) & 0377;
      }

   close (imgfd);
   free (linbuf);
   return (ims);
}
