/* rotate_img.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an unsigned short integer image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rose Reynolds
 * Adapted by Andrew Stanger
 * 22 September 1987
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <math.h>

unsigned short *
rotate_img (oldimg, dx, dy, cx, cy, angdeg)

unsigned short *oldimg;	/* Input image array pointer. */
int dx, dy;		/* Dimensions of image. */
int cx, cy;		/* Center of rotation. */
float angdeg;		/* Rotation angle in degrees. */

{
   extern char *malloc();
   register int nx, ny, ox, oy, xdim, ydim;
   float fox, foy, cose, sine, fnx, fny, fnysin, fnycos;
   float fcx, fcy;
   double angle;
   unsigned short *newimg;
   register unsigned short *rold, *rnew;

   xdim = dx;   ydim = dy;
   fcx = (float) cx;   fcy = (float) cy;
   angle = (double) angdeg * (3.14159265/180.);/* convert deg to rads. */
   sine = (float) sin( angle );
   cose = (float) cos( angle );
printf ("angdeg: %6.2f   angle: %f   sine: %6.2f   cose: %6.2f\n",
         angdeg, angle, sine, cose);
   newimg = (unsigned short *) malloc( dx * dy * sizeof(short) );
   rold = oldimg;
   rnew = newimg;

   for( ny = 0; ny < ydim; ny++ )
      {
      fny = (float)(ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *rnew++ = 0;
         else *rnew++ = *(rold + (oy*xdim) + ox);
         }
      }
   return ( newimg );
}
