/* rotate_img.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an unsigned short integer image.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Original Author: Rose Reynolds
 * Adapted by Andrew Stanger   22 September 1987
 * 27 June 2000: Use stdin & stdout.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* -------------------------------------------------------------------------- */

main (argc, argv)

int argc ;
char *argv [] ;

{
   register unsigned short *rold, *rnew ;
   register int nx, ny, ox, oy, xdim, ydim ;

   FILE *angfd ;
   double angle ;
   double deg2rad ;

   int cx, cy ;				/* Center of rotation. */
   int dx, dy ;				/* Dimensions of image. */
   int nbypix = 2 ;			/* Short integer images. */

   float angdeg ;			/* Rotation angle in degrees. */
   float cose, sine ;
   float fcx, fcy ;
   float fox, foy ;
   float fnx, fny ;
   float fnysin, fnycos ;

   unsigned short *oldimg ;		/* Input image array pointer. */
   unsigned short *newimg ;

   char *parg ;
   static char nampgm [80] ;


   strcpy (nampgm, argv [0]) ;

   /*** Get rotation angle. ***/

   if ( argc == 2 )
      {
      parg = *++argv ;
      if ( ( *parg != '+' ) && ( *parg != '-' ) )
         {
	 if ( ( angfd = fopen ( parg, "r" ) ) < (FILE*)0 )
            {
            fprintf ( stderr," open error %d %s\n", angfd, parg) ;
            usage (nampgm) ;
	    exit (-1) ;
            }
         fscanf ( angfd, "%f", &angdeg ) ;
         fclose ( angfd ) ;
         }
      else
         sscanf ( parg, "%f", &angdeg ) ;
      }
   else
      {
      usage (nampgm) ;
      exit (-2) ;
      }

   /*** Get array dimensions ***/

   sscanf ( getenv ( "XDIM" ), "%d", &dx ) ;
   sscanf ( getenv ( "YDIM" ), "%d", &dy ) ;
   sscanf ( getenv ( "XCEN" ), "%d", &cx ) ;
   sscanf ( getenv ( "YCEN" ), "%d", &cy ) ;

   deg2rad = 3.14159265 / 180.0 ;
   xdim = dx ;   ydim = dy ;
   fcx = (float) cx ;   fcy = (float) cy ;
   angle = (double) angdeg * deg2rad ;		/* Convert degees to radians. */
   sine = (float) sin ( angle ) ;
   cose = (float) cos ( angle ) ;

   fprintf (stderr, "angdeg: %6.2f   angle: %f   sine: %6.2f   cose: %6.2f\n",
            angdeg, angle, sine, cose) ;

   /*** Allocate storage for both input & output arrays. ***/

   oldimg = (unsigned short *) malloc ( dx * dy * sizeof (short) ) ;
   newimg = (unsigned short *) malloc ( dx * dy * sizeof (short) ) ;

   /*** Read input image. ***/

   read ( 0, oldimg, dx * dy * nbypix ) ;

   /*** Rotate image. ***/

   rold = oldimg ;
   rnew = newimg ;

   for ( ny = 0 ; ny < ydim ; ny++ )
      {
      fny = (float) (ny) - fcy ;
      fnysin = fny * sine - fcx ;
      fnycos = fny * cose + fcy ;

      for ( nx = 0 ; nx < xdim ; nx++)
         {
         fnx = (float) (nx) - fcx ;
         fox = fnx * cose - fnysin ;
         foy = fnycos + fnx * sine ;
         ox = (int) fox ;
         oy = (int) foy ;
         if( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *rnew++ = 0 ;
         else *rnew++ = *(rold + (oy * xdim) + ox) ;
         }
      }

   write ( 1, newimg, dx * dy * nbypix) ;

   exit (0) ;
}

/* -------------------------------------------------------------------------- */

usage (nampgm )

char *nampgm ;

{
   fprintf ( stderr, "%s  angle < infile > outfile\n", nampgm) ;
   fprintf ( stderr, " angle is either a float value or a file name\n" );
   return (0) ;
}

/* -------------------------------------------------------------------------- */

