/* rotimg90.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an image 90 degrees.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * original author: Rose Reynolds
 * Adapted by Andrew Stanger   HAO/NCAR   12 July 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <math.h>

unsigned char *
rotimg90 (imginp, nbypx, dx, dy, cx, cy, angdeg)

unsigned char *imginp;	/* Input image array pointer. */
int nbypx ;		/* # bytes per pixel. */
int dx, dy ;		/* Dimensions of image. */
int cx, cy ;		/* Center of rotation. */
float angdeg ;		/* Rotation angle in degrees (+ = CCW). */

{
   extern char *malloc ();
   register int nx, ny, ox, oy, xdim, ydim;
   int rx, ry ;
   float fox, foy, cose, sine, fnx, fny, fnysin, fnycos;
   float fcx, fcy;
   double angle;
   unsigned char  *imgrot;
   unsigned char  *cinp, *crot ;
   unsigned short *sinp, *srot ;
   float          *finp, *frot ;

   printf ("cprot> nbypx:%3d  dx/dy:%3d %3d  cx/cy:%3d %3d\n",
	    nbypx, dx, dy, cx, cy) ;

   xdim = dx;   ydim = dy;
   fcx = (float) cx;   fcy = (float) cy;
   angle = (double) angdeg * (3.14159265/180.);/* convert deg to rads. */
   sine = (float) sin ( angle );
   cose = (float) cos ( angle );
   printf ("angdeg: %6.2f   angle: %f   sine: %6.2f   cose: %6.2f\n",
            angdeg, angle, sine, cose);
   imgrot = (unsigned char *) malloc ( dx * dy * nbypx );

   if (nbypx == 1)				/* unsigned char pixels. */
   for ( ny = 0; ny < ydim; ny++ )
      {
      cinp = imginp ;
      crot = imgrot ;
      fny = (float) (ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for ( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *crot++ = 0;
         else *crot++ = *(cinp + (oy*xdim) + ox);
         }
      }

   else
   if (nbypx == 2)				/* unsigned short int pixels. */
      {
      sinp = (unsigned short *) imginp ;
      srot = (unsigned short *) imgrot ;

      for ( ny = 0; ny < ydim; ny++ )
         {
         for ( nx = 0; nx < xdim; nx++)
            {
	    *(srot + (dy - ny - 1) + nx*dx) = *(sinp + nx + ny*dx) ;
            }
         }
      }
   else
   if (nbypx == 4)				/* floating point pixels. */
   for ( ny = 0; ny < ydim; ny++ )
      {
      finp = (float *) imginp ;
      frot = (float *) imgrot ;
      fny = (float) (ny) - fcy;
      fnysin = fny*sine - fcx;
      fnycos = fny*cose + fcy;

      for ( nx = 0; nx < xdim; nx++)
         {
         fnx = (float) (nx) - fcx;
         fox = fnx * cose - fnysin;
         foy = fnycos + fnx * sine;
         ox = (int) fox;
         oy = (int) foy;
         if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
            *frot++ = 0;
         else *frot++ = *(finp + (oy*xdim) + ox);
         }
      }

   return ( imgrot );
}
