/* glcp_label.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label SMM C/P image in SGI/IRIS GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  3 April 1991
 * 26 April 1991: use onemap color mode.
 * 10 July 1991: Write border around bottom portion AFTER color bar is written.
 * 6 August 1992: modify format to accomodate difference images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"
#include "/home/stanger/lib/include/gl_color.h"

glcp_label (lfull, lbottom, cval2, cval3, cval4)

int lfull;	/* Flag for full redrawing of labels. */
int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int cval2;	/* Color level for labels. */
int cval3;	/* Color level for HAO logo. */
int cval4;	/* Color level for miscellaneous labels. */

{
   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    */
   extern fmfonthandle gl_setfont ();
   fmfonthandle afh, bfh, cfh;
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
   static char afont [32];			/* Small font. */
   static char bfont [32];			/* Medium font. */
   static char cfont [32];			/* Large font. */
  
   static char logo [] = " HAO/SMM CORONAGRAPH/POLARIMETER ";
   static char pixval [12];		/* Pixel value mode string. */
   static short *cbar;			/* Color bar array. */

   int asize, bsize, csize;		/* Font point sizes. */
   int afx, afy, bfx, bfy, cfx, cfy;	/* Font x/y sizes. */
   int afx15;

   int i;
   int lxstr;
   int pxstr;
   int xempty;
   int xlogo;
   int ylogo;
   int xstart = 1;
   int ystart = 501;
   int begrow, begcol, endrow, endcol, ix, iy;
   int step;

   int colormax = 255;			/* Maximum color value. */
   int barwidth = 512, barheight = 12;	/* color bar dimensions. */
   int ppc = 2;		/* # pixels per color in color bar. */
   int cby0, cby1;	/* color bar y-axis locations. */
   int startcol;

   int xs, ys;
   int x1 = 0, x2 = 62, x3 = 511;
   int y1 = 0, y2 = 62, y3 = 511;
   int xwl, xil, xwr;
   int ywb, yib, ywt;

   if (H.ires == 1)		/* Low resolution. */
      {
      ystart = 501;
      xs = xstart;
      ys = ystart;
      strcpy (afont, font1);
      asize = 7;
      strcpy (bfont, font4);
      bsize = 10;
      strcpy (cfont, font4);
      csize = 12;
      afx =  6;   afy =  7;
      bfx = 14;   bfy = 15;
      cfx = 16;   cfy = 17;
      xwl = x1;  xil = x2;  xwr = x3;
      ywb = y1;  yib = y2;  ywt = y3;
      ppc = 2;
      startcol  =   1;
      barwidth  = 512;
      barheight =  12;
      }
   else
   if (H.ires == 0)		/* High resolution. */
      {
      ystart = 1008;
      xs = xstart * 2;
      ys = ystart * 2;
      strcpy (afont, font4);
      asize = 10;
      strcpy (bfont, font4);
      bsize = 19;
      strcpy (cfont, font4);
      csize = 25;
      afx = 14;   afy = 15;
      bfx = 28;   bfy = 30;
      cfx = 38;   cfy = 40;
      xwl = x1 * 2;  xil = x2 * 2;  xwr = x3 * 2;
      ywb = y1 * 2;  yib = y2 * 2;  ywt = y3 * 2;
      ppc = 4;
      startcol  =    2;
      barwidth  = 1024;
      barheight =   24;
      }
   ys = ywt - afy;
   begcol = 0;
   endcol = begcol + barwidth - 1;

   if (lfull)
      {		/*** Draw box around image window. ***/

      gl_wvec (xwl, yib, xwr, yib, cval2);
      gl_wvec (xwr, yib, xwr, ywt, cval2);
      gl_wvec (xwr, ywt, xwl, ywt, cval2);
      gl_wvec (xwl, ywt, xwl, yib, cval2);
      gl_wvec (xil, yib, xil, ywt, cval2);
      gl_wvec (xwl, ys-4*afy-10*bfy, xil, ys-4*afy-10*bfy, cval2); 

      /*** Write annotation labels into frame buffer. ***/

      afh = gl_setfont (afont, asize);	/* Small font & size. */
      gl_wstr ("DATE    ",   xs, ys,                   cval2);
      gl_wstr ("YEAR:DOY",   xs, ys -  2*afy -  3*bfy, cval2);
      gl_wstr ("TIME",       xs, ys -  4*afy -  6*bfy, cval2);

      gl_wstr ("FILTER",     xs, ys -  6*afy - 10*bfy, cval2);
      gl_wstr ("POLAROID",   xs, ys -  7*afy - 12*bfy, cval2);
      gl_wstr ("SECTOR",     xs, ys -  8*afy - 14*bfy, cval2);
      gl_wstr ("EXPOSURE",   xs, ys -  9*afy - 16*bfy, cval2);
      gl_wstr ("RESOLUTION", xs, ys - 10*afy - 18*bfy, cval2);
      }

   gl_setfont (afont, asize);	/* Small font & size. */
   if (H.icm == 2)
      gl_wstr ("CAL MIRROR", xs, ys - 11*afy - 20*bfy, cval2);
   else
      gl_wstr ("ROLL      ", xs, ys - 11*afy - 20*bfy, cval2);

   if (lfull)
      {
      gl_wstr ("FRAME  ", xs, ys - 12*afy - 22*bfy, cval2);

      /*** Write HAO logo & cval2 bar only if 'lbottom = 0'. ***/

      if (lbottom == 1)  return (0);

      /*** Add annotation for the color bar ***/

      gl_setfont (afont, asize);	/* Small font & size. */
      cby0 = 1 * nres;
      afx15 = (int) ((afx * 1.5) + 0.5);
      gl_wstr ("0"  , (startcol+1),               cby0, cval2);
      gl_wstr ("64" , (startcol + 64 *ppc-afx),   cby0, cval2);
      gl_wstr ("128", (startcol + 128*ppc-afx15), cby0, cval2);
      gl_wstr ("192", (startcol + 192*ppc-afx15), cby0, cval2);
      gl_wstr ("255", (endcol-afx*3),             cby0, cval2);

      /*** Add tick marks to color bar ***/

      cby0 += afy + 2;
      cby1 = cby0 + 4 * nres;
      gl_wvec (startcol,         cby0, (startcol),         cby1, cval2);
      gl_wvec (startcol+32*ppc , cby0, (startcol+32*ppc),  cby1, cval2);
      gl_wvec (startcol+64*ppc , cby0, (startcol+64*ppc),  cby1, cval2);
      gl_wvec (startcol+96*ppc , cby0, (startcol+96*ppc),  cby1, cval2);
      gl_wvec (startcol+128*ppc, cby0, (startcol+128*ppc), cby1, cval2);
      gl_wvec (startcol+160*ppc, cby0, (startcol+160*ppc), cby1, cval2);
      gl_wvec (startcol+192*ppc, cby0, (startcol+192*ppc), cby1, cval2);
      gl_wvec (startcol+224*ppc, cby0, (startcol+224*ppc), cby1, cval2);
      gl_wvec (startcol+255*ppc, cby0, (startcol+255*ppc), cby1, cval2);

      /*** Draw the color bar. ***/

      barwidth = (colormax + 1) * ppc;
      cbar = (short *) malloc (barwidth * barheight * sizeof (short));

      for (iy = 0; iy < barheight; iy++)
         for (step = 0; step <= colormax; step++)
            for (ix = 0; ix < ppc; ix++)
               *(cbar + (step*ppc+ix) + (iy*barwidth)) =
                (short) (step & 0377) + cmapoff;

      begrow = cby1 + 1;
      endrow = begrow + barheight - 1;
      rectwrite (begcol, begrow, endcol, endrow, cbar);
      free (cbar);

      /*** Write HAO ID. ***/

      for (i = 0; i < 12; i++)  pixval [i] = '\0';

      if ( H.k == 1)
         {
         if ( H.str &&  H.f &&  H.vig)  strcpy (pixval, "         ");
         if (!H.str && !H.f && !H.vig)  strcpy (pixval, "S+F*V    ");

         if ( H.str &&  H.f && !H.vig)  strcpy (pixval, "    V    ");
         if ( H.str && !H.f && !H.vig)  strcpy (pixval, "  F*V    ");
         if (!H.str &&  H.f && !H.vig)  strcpy (pixval, "S  *V    ");

         if (!H.str &&  H.f &&  H.vig)  strcpy (pixval, "S        ");
         if (!H.str && !H.f &&  H.vig)  strcpy (pixval, "S+F      ");

         if ( H.str && !H.f &&  H.vig)  strcpy (pixval, "  F      ");
         }
      else
         {
         if ( H.str &&  H.f &&  H.vig)  strcpy (pixval, "     K   ");
         if (!H.str && !H.f && !H.vig)  strcpy (pixval, "S+(F+K)*V");

         if ( H.str &&  H.f && !H.vig)  strcpy (pixval, "     K *V");
         if ( H.str && !H.f && !H.vig)  strcpy (pixval, "  (F+K)*V");
         if (!H.str &&  H.f && !H.vig)  strcpy (pixval, "S+   K *V");

         if (!H.str &&  H.f &&  H.vig)  strcpy (pixval, "S+   K   ");
         if (!H.str && !H.f &&  H.vig)  strcpy (pixval, "S+(F+K)  ");

         if ( H.str && !H.f &&  H.vig)  strcpy (pixval, "  (F+K)  ");
         }
 
      cfh = gl_setfont (cfont, csize);	/* Large font & size. */
      pxstr = fmgetstrwidth (cfh, pixval);
      lxstr = fmgetstrwidth (cfh, logo);

      xempty = (xwr - xwl) - pxstr - lxstr;
      if (xempty > 0)  xlogo = xs + pxstr + xempty/2;
      ylogo = endrow + 7*nres;

      gl_wstr (pixval, xs,    ylogo, cval4);
      gl_wstr (logo,   xlogo, ylogo, cval3);

     /*** Write border around lower portion of window. ***/

      gl_wvec (xwl, yib, xwl, ywb, cval2);
      gl_wvec (xwl, ywb, xwr, ywb, cval2);
      gl_wvec (xwr, ywb, xwr, yib, cval2);
      }
   return (0);
}
