/* smm4qs.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display a 4-quadrant SMM C/P image oriented so that solar north is UP.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Input C/P image must be low resolution, and calibrated.
 * Output image is in Sun rasterfile format.
 * Output image size: 256x224 pixels
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 29 September 1987
 *  4 May 1988: change pmon angle for DOY 314 1987
 *  2 June 1989: Use Eikonix-digitized images (512x512 byte array).
 *  3 August 1989: Use Eikonix-digitized images (with header).
 *                 Version for sunview.
 * 21 August 1990: adapt for use with C/P 4-quadrant images.
 *                 Rotate C/P image so that solar north is along +y axis.
 * 13 August 1991: adapt for SGI/IRIS.
 *  1 October 1997: output sun rasterfile.
 *  2 March 1998: Small output image size for Web pages.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/grscale.h"
#include "/home/stanger/lib/include/cp_color.h"

main (argc, argv)

int argc ;
char *argv [] ;
{

   extern fmfonthandle gl_setfont () ;
   fmfonthandle cfh ;
   unsigned short *rdc4q () ;
   unsigned short *rotate_img () ;
   unsigned short *magimg () ;
   unsigned short *merge3s () ;

   int mskimg () ;
   int rescale () ;
   int wdisk () ;

   static unsigned char red [256], green [256], blue [256] ;
   static char namecp [80] = "801050544.c4q" ;	/* C/P image file name. */
   static char cptime [6]  = "HH:MM" ;
   static char nampgm [80] ;
   static char namout [80] = "cor4q.img" ;

   unsigned short *cpims ;	/* Pointer to SMM C/P image array. */
   unsigned short *cpimr ;	/* Pointer to SMM C/P image array. */
   unsigned short *img3 ;	/* Pointer to composite image array. */

   unsigned char  *cimg ;	/* Pointer to output image array. */

   int i ;
   int ix, iy, iiy ;
   int lfull = 1 ;
   int lbottom = 0 ;
   int nbypx = 2 ;
   int nx = 512, ny = 512 ;
   int npximg ;
   int xl = 760 ;	/* Specification of GL window position & size. */
   int yl = 480 ;
   int xu = 1271 ;
   int yu = 991 ;

   static    int ngr = 0 ;
   static    int imgch0 = 0, imgch1 = 1, imgch2 = 2, imgch3 = 3 ;

   int cpdx = 448, cpdy = 448, cpnx = 224, cpny = 224 ;
   int cpcx, cpcy, icpdsk, icprad ;

   static int xgs = 0, ygs = 0 ;
   static int xs = 0, ys = 0 ;
   static int ithresh = 175 ;
   static int level = 0 ;
   static int pmmin1 = 0, pmmax1 = 250, pmmin2 = 0, pmmax2 = 250 ;
   static int mkmin1 = 0, mkmax1 = 250, mkmin2 = 0, mkmax2 = 250 ;
   static int cpmin1 = 0, cpmax1 = 500, cpmin2 = 0, cpmax2 = 250 ;
   static int grnlo = 0, grnhi = 249 ;
   int sroll ;

   /* Sector mirror order: sw, s, se, e, ne, n, nw, w, nesw. */

   static float cpcxnew [] = {  7., 100., 225., 398., 441., 348., 225., 50., 223.5} ;
   static float cpcynew [] = {225., 348., 441., 398., 258., 100.,   7., 50., 223.5} ;
   static float fcpcx, fcpcy ;
   static float cpfx, cpfy ;
   static float cprot = 43.0 ;		/* Angle of C/P north w.r.t. +Y axis. */
   static float fx, fy ;
   static float cpdsk = 1.5,  cprad1 = 1.5, cprad2 = 4.15 ;
   static float cpang = 0.0 ;
   static float cppxrs = 80.9877 ;	/* Low resolution, single quadrant. */
   static float impxrs = 54.0 ;

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
/* Request input information. */

   if (argc == 2)
      {
      strcpy (namecp, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s [cp_radius] cp_filename\n", argv [0]) ;
      exit (0) ;
      }

/*
 *   fprintf (stderr, "Enter file name of SMM C/P image: ") ;
 *   fscanf  (stdin, "%s", namecp) ;
 */
/*
 *   fprintf (stderr, "Enter minimum radius for SMM C/P image: ") ;
 *   fscanf  (stdin, "%f", &cprad) ;
 */
/*
 *   fprintf (stderr, "Enter C/P oldmin, oldmax, newmin, newmax: ") ;
 *   fscanf  (stdin, "%d,%d,%d,%d", &cpmin1, &cpmax1, &cpmin2, &cpmax2) ;
 */

   strcpy (nampgm, argv [0]) ;

   /*** Initialize SGI Graphics Library. ***/

   foreground () ;
   prefposition (xl, xu, yl, yu) ;
   winopen (nampgm) ;
   gl_wclear (BLACK) ;
   onemap () ;			/* Set single-map mode for color table. */
   get_cmap () ;		/* Save color map. */

   multimap () ;		/* Set multi-map mode for color table. */
   gconfig () ;
   setmap (2) ;			/* Select color map #2. */

   fminit () ;			/* Initialize SGI font manager. */

   /*** Read SMM C/P image file. ***/

   printf ("Read C/P image: %s\n", namecp) ;
   cpims = rdc4q (namecp, &cpdx, &cpdy, &cpcx, &cpcy) ;

   sprintf (namout, "%02d%03d%02d%02d", H.year, H.doy, H.hour, H.minute) ;
   fcpcx = cpcxnew [H.ism] ;			/* New x sun center. */
   fcpcy = cpcynew [H.ism] ;			/* New y sun center. */
   fcpcx = 111.5 ;
   fcpcy = 111.5 ;
   cppxrs = H.pixrs ;				/* pixels/Rsun. */
   icprad = (int) (cprad1 * cppxrs) ;		/* Inner radius. */
   icpdsk = (int) (cpdsk * cppxrs) ;
   printf ("fcpcx: %7.2f   fcpcy: %7.2f\n", fcpcx, fcpcy) ;

   /*** Rescale image intensities. ***/

   printf ("Rescale C/P image . . . %d-%d --> %d-%d\n",
            cpmin1, cpmax1, cpmin2, cpmax2) ;
   rescale (cpims, cpdx, cpdy, cpmin1, cpmax1, cpmin2, cpmax2, grnlo, grnhi-1) ;

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Write black disk over occulting disk. ***/

   printf ("Write disk into C/P image... cpcx: %3d   cpcy: %3d   icprad: %3d\n",
            cpcx, cpcy, icprad) ;
   wdisk (cpims, cpdx, cpdy, cpcx, cpcy, icprad, level) ;

   imgwgl (cpims, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Rotate image so that north is up. ***/

   cpang = cprot - (float) scroll ;
   scroll -= cprot ;
   printf ("Rotate SMM C/P image . . . cpang: %7.2f\n", cpang) ;
   cpimr = rotate_img (cpims, cpdx, cpdy, cpcx, cpcy, cpang) ;
   free (cpims) ;
   imgwgl (cpimr, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Mask image beyond specified radius. ***/

   icprad = (int) (cprad2 * cppxrs) ;		/* Outer radius [pixels]. */
   printf ("Mask SMM C/P image beyond %7.2f Rsun.\n", cprad2) ;
   mskimg (cpimr, 0, 0, cpdx, cpdy, cpcx, cpcy, icprad) ;
   imgwgl (cpimr, nbypix, xs, ys, cpdx, cpdy, "ru") ;
   sleep (1) ;

   /*** Magnify/Shrink image. ***/

   printf ("Magnify C/P image.\n") ;
   impxrs = cpnx / (cprad2 * 2.0) ;	/* Set resolution: new_size / FOV */ 
   cpfx = cpfy = impxrs / cppxrs ;	/* Set FOV to 8.30 Rsun. */
   fprintf (stdout, "impxrs: %7.2f\n", impxrs) ;
   fprintf (stdout, "cpfx/cpfy: %7.2f  %7.2f\n", cpfx, cpfy) ;
   cpims = (unsigned short *) malloc (cpdx * cpdy * sizeof (short)) ;
   expcon (cpimr, cpdx, cpdy, cpims, cpnx, cpny, cpfx, cpfy,
           (float) cpcx, (float) cpcy, fcpcx, fcpcy) ;
   free (cpimr) ;

   imgwgl (cpims, nbypix, xs, ys, cpnx, cpny, "ru") ;
   sleep (1) ;

/******************************************************************************/

   img3 = cpims ;

   /*** Display composite image with annotation. ***/

/*
   gl_wclear (BLACK) ;
   ldisk = 1 ;
   glcp_label (lfull, lbottom, cp_grey, cp_green, cp_yellow) ;

   glcp_annot (lbottom, cp_red, cp_grey, cp_green, cp_yellow) ;
 */

   imgwgl (img3, nbypx, xgs, ygs, cpnx, cpny, "ru") ;   /* Write output image.*/
   free (img3) ;

/*
   fprintf (stdout, "   fcpcx: %f   fcpcy: %f\n", fcpcx, fcpcy) ;
   xcen = fcpcx ;
   ycen = cpny - 1 - fcpcy ;
   pixrs = pixrs * cpfx ;
   glcp_north ((int) fcpcx, (int) fcpcy, 0.1, 0.3, cp_yellow) ;
   glcp_scalers (0, cp_yellow) ;
   glcp_suncir (0.4, 1.5, 0.1, 90.0, 1.0, 1.5, 0.2, 30.0, cp_yellow) ;
 */
   
   /*** Read image from frame buffer. ***/

   npximg = cpnx * cpny ;
   fprintf (stdout, "cpnx/cpny: %3d %3d  npximg: %d\n", cpnx, cpny, npximg) ;
   img3 = (unsigned short *) malloc (npximg * sizeof (short)) ;
   cimg = (unsigned char *) malloc (npximg) ;
   rectread (0, 0, cpnx-1, cpny-1, img3) ;

   /*** Flip image top to bottom for Sun Rasterfile. ***/

   for (iy = 0 ; iy < cpny ; iy++)
      {
      iiy = cpny - 1 - iy ;
      for (ix = 0 ; ix < cpnx ; ix++)
	 {
	 *(cimg + ix + iy*nx) = (unsigned char) (*(img3 + ix + iiy*nx) & 0377) ;
	 }
      }

   /*** Read color table. ***/

   get_scmap (512, 256, red, green, blue) ;

   /*** Write image to disc as a sun rasterfile. ***/

   sprintf (namout, "%s.sun", namecp) ;
   bytsrf (namout, cimg, cpnx, cpny, red, green, blue) ;

   fprintf (stdout, "\n") ;
   gexit () ;
   exit (0) ;
}
