/* wboxfb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Move SMM C/P image from disc file to SGI IRIS frame buffer.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  1 April 1991
 * Currently only supports dir2 = "rd" or "ur".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <whip.h>

wboxfb (fb, imgbuf, nbypx, xs, ys, xdim, ydim, dir2)

int fb;				/* Frame buffer descriptor. */
char *imgbuf;			/* Image memory buffer. */
int nbypx;			/* # bytes per pixel. */
int xs, ys;			/* Coordinates for image origin in FB. */
int xdim, ydim;			/* X & Y dimensions of image. */
char *dir2;			/* Direction to write image: l/r, u/d. */
{
   extern char *malloc ();	/* Memory allocation. */
   char xdir, ydir;
   int col;			/* Column index counter. */
   int i;			/* Loop index. */
   int nbyimg;
   int rel = 0;			/* Relative flag for fb_setaddr. */
   int row;			/* Row index counter. */
   int startx = 63;		/* X location for writing. */
   int xsize, ysize, zsize;	/* Sizes for each dimension of FB. */
   int z = 0;
   char *cimgbuf;
   short *simgbuf;
   float *fimgbuf;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

/*
 *   fb_getsize (fb, &xsize, &ysize, &zsize);
 */

   /*** Determine storage orientation of image. ***/
   /*
    * WHIP uses an origin at lower left corner of frame buffer.
    * Therefore, convert any input image so that increasing X is to right,
    * and increasing Y is upward.
    */

   xdir = 'x';
   ydir = 'y';
   for (i=0; i<2; i++)
      switch (*(dir2 + i))
         {
         case 'l':
         case 'r': xdir = *(dir2 + i); break;
         case 'u':
         case 'd': ydir = *(dir2 + i); break;
         }
   if (xdir == 'x' || ydir == 'y')
      {
      fprintf (stderr, "***wboxfb> Illegal direction (%s).\n", dir2); 
      return (-1);
      }
   fprintf (stdout, "   xdir: %c   ydir: %c\n", xdir, ydir);

   nbyimg = xdim * ydim * nbypx;
   cimgbuf = (char *) malloc (nbyimg);
   simgbuf = (short *) cimgbuf;
   fimgbuf = (float *) cimgbuf;

   /*** Copy image into temporary array. ***/

   for (i = 0; i < nbyimg; i++)  *(cimgbuf + i) = *(imgbuf + i);


   if (xdir == 'l' || xdir == 'r')	/* Horizontally-scanned image. */
      {
      fb_xstep (fb);			/* Auto increment in x axis. */
      if (xdir == 'l')			/* Image scanned right to left. */
         {
         xs -= (xdim - 1);
         swapcols (cimgbuf, nbypx, xdim, ydim);	/* Reverse row order. */
         }
      if (ydir == 'd')			/* First scan line is at top of image.*/ 
         {
         ys -= (ydim - 1);
         }

      if (nbypx == 1)
      for (row = 0; row < ydim; row++)
         {
         fb_setaddr (fb, rel, xs, ys+row, z);		/* Set address in FB. */
         fb_write (cimgbuf + row * xdim, xdim);		/* Write row to FB.   */
         }
      else
      if (nbypx == 2)
      for (row = 0; row < ydim; row++)
         {
/*
         printf ("   wboxfb> rel: %d   xs: %d   ys-row-1: %d   z: %d\n",
                 rel, xs, ys-row-1, z);
 */
         fb_setaddr (fb, rel, xs, ys+row, z);		/* Set address in FB. */
         fb_write (simgbuf + row * xdim, xdim);		/* Write row to FB.   */
         }
      else
      if (nbypx == 4)
      for (row = 0; row < ydim; row++)
         {
         fb_setaddr (fb, rel, xs, ys+row, z);		/* Set address in FB. */
         fb_write (fimgbuf + row * xdim, xdim);		/* Write row to FB.   */
         }
      }

   else					/* Vertically-scanned image. */
      {
      fb_ystep (fb);			/* Auto increment in y axis. */
      if (xdir == 'd')			/* Image is scanned top to bottom. */
         {
         xs -= (xdim - 1);
         }
      if (ydir == 'l')			/* First column is at right of image. */
         {
         ys -= (ydim - 1);
         swaprows (cimgbuf, nbypx, xdim, ydim);	/* Reverse column order. */
         }
      if (nbypx == 1)
      for (col = 0; col < xdim; col++)
         {
         fb_setaddr (fb, rel, xs+col, ys, z);		/* Set address in FB. */
         fb_write (cimgbuf + col * ydim, ydim);		/* Write column to FB.*/
         }
      else
      if (nbypx == 2)
      for (col = 0; col < xdim; col++)
         {
         fb_setaddr (fb, rel, xs+col, ys, z);	/* Set address in FB. */
         fb_write (simgbuf + col * ydim, ydim);		/* Write column to FB.*/
         }
      else
      if (nbypx == 4)
      for (col = 0; col < xdim; col++)
         {
         fb_setaddr (fb, rel, xs+col, ys, z);	/* Set address in FB. */
         fb_write (fimgbuf + col * ydim, ydim);		/* Write column to FB.*/
         }
      }
   fb_touch ();					/* Terminate WHIP software. */
   free (cimgbuf);
   return (0);
}

/* swaprows
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse image top to bottom.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

swaprows (imgbuf, nbypx, xdim, ydim)

char *imgbuf;
int nbypx;
int xdim, ydim;

{
int ix;
int iy, ny;
int yhalf, ymax;
char cval;
short sval;
float fval;
short *simgbuf;
float *fimgbuf;

   printf ("   swaprows> nbypx: %d   xdim: %d   ydim: %d\n", nbypx, xdim, ydim);
   simgbuf = (short *) imgbuf;
   fimgbuf = (float *) imgbuf;

   /*** Flip image so that origin is lower left instead of upper left. ***/

   ymax = ydim - 1;
   yhalf = ydim / 2;

   if (nbypx == 1)				/* Byte image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xdim; ix++)
            {
            cval = *(imgbuf + ix + iy*xdim);
            *(imgbuf + ix + iy*xdim) = *(imgbuf + ix + ny*xdim);
            *(imgbuf + ix + ny*xdim) = cval;
            }
         }
      }
   else
   if (nbypx == 2)				/* Short integer image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xdim; ix++)
            {
            sval = *(simgbuf + ix + iy*xdim);
            *(simgbuf + ix + iy*xdim) = *(simgbuf + ix + ny*xdim);
            *(simgbuf + ix + ny*xdim) = sval;
            }
         }
      }

   else
   if (nbypx == 4)				/* Floating point image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xdim; ix++)
            {
            fval = *(fimgbuf + ix + iy*xdim);
            *(fimgbuf + ix + iy*xdim) = *(fimgbuf + ix + ny*xdim);
            *(fimgbuf + ix + ny*xdim) = fval;
            }
         }
      }

   return (0);
}


/* swapcols
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse image left to right.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

swapcols (imgbuf, nbypx, xdim, ydim)

char *imgbuf;
int nbypx;
int xdim, ydim;

{
int ix, nx;
int iy;
int xhalf, xmax;
char cval;
short sval;
float fval;
short *simgbuf;
float *fimgbuf;

   printf ("   swapcols> nbypx: %d   xdim: %d   ydim: %d\n", nbypx, xdim, ydim);
   simgbuf = (short *) imgbuf;
   fimgbuf = (float *) imgbuf;

   /*** Flip image left to right. ***/

   xmax = xdim - 1;
   xhalf = xdim / 2;

   if (nbypx == 1)				/* Byte image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ydim; iy++)
            {
            cval = *(imgbuf + ix + iy*xdim);
            *(imgbuf + ix + iy*xdim) = *(imgbuf + nx + iy*xdim);
            *(imgbuf + nx + iy*xdim) = cval;
            }
         }
      }

   else
   if (nbypx == 2)				/* Short image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ydim; iy++)
            {
            sval = *(simgbuf + ix + iy*xdim);
            *(simgbuf + ix + iy*xdim) = *(simgbuf + nx + iy*xdim);
            *(simgbuf + nx + iy*xdim) = sval;
            }
         }
      }

   else
   if (nbypx == 4)				/* Floating point image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ydim; iy++)
            {
            fval = *(fimgbuf + ix + iy*xdim);
            *(fimgbuf + ix + iy*xdim) = *(fimgbuf + nx + iy*xdim);
            *(fimgbuf + nx + iy*xdim) = fval;
            }
         }
      }

   return (0);
}
