/* stokes1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute Stokes vector from 3 polaroid intensity images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 27 January 1987
 * 14 November 1988: reverse bytes from input images if not processed on Sun-4.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/include/cplib.ext"
#define Cinv 1.e+11

stokes (namp0, namp1, namp2, namp3, rinv)

char *namp0;	/* Name of clear image. */
char *namp1;	/* Name of +60 degree polaroid image. */
char *namp2;	/* Name of   0 degree polaroid image. */
char *namp3;	/* Name of -60 degree polaroid image. */
float rinv[3][3];

{
extern FILE *lun;	/* File descriptor for data output file. */
extern char *malloc();	/* Storage Allocation function. */
extern int mkname();	/* Create new file name. */
extern int rcoord();	/* Coordinate transformation: (r,th) <---> (x,y). */
extern float calcor();	/* Munro calibration correction. */
extern float stray();	/* Stray Light function. */
extern float vignet();	/* Vignetting function. */
extern float fcorona(); /* F-corona function. */
extern int rdlinp();	/* Read one line of intensity data from 3 polaroids. */
extern int wrlinp();	/* Read one line of intensity data from 3 polaroids. */
extern int mxm_();	/* Matrix multiplication routine. */
extern double sqrt(), fabs(), atan2();	/* Math routines. */

static char namebr[80];	/* Name of Br image. */
static char namebt[80];	/* Name of Bt image. */
static char nampol[80];	/* Name of polarization fraction image. */
static char namepb[80];	/* Name of pB image. */
static char namprt[80];	/* Name of total brightness image. */

int icpu;
int i, j;
int nbylin;
int lunp0;
int lunp1, lunp2, lunp3;/* File descriptors for +60, 0, -60 polaroids.*/
int lunbr, lunbt;
int lunp, lunpb;
int lunrt;
int nread, nwrite;
static int n1 = 1;
static int n3 = 3;

static short linbuf[896];	/* Line buffer. */
short *p1, *p2, *p3;		/* Line arrays for polaroid images. */
short *br;
short *bt;
short *p;
short *pb;
short *p123;

static float s[3];
static float ss[3];
float cc, str, vig;
float si, sq, su;
float ci, cq, cu;
float brad, btan;
float pol, polbrt;
float clear;
float alpha, delang, fsign;
float x, y;
float radius, theta;
float fc;
float vinv;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

/* Allocate storage for line arrays. */

   nbylin = npix * nbypix;
   br = (short *) malloc (nbylin);
   bt = (short *) malloc (nbylin);
   p  = (short *) malloc (nbylin);
   pb = (short *) malloc (nbylin);
   p123 = (short *) malloc (nbylin);

/* Open polaroid image files. */

   lunp0 = open (namp0, 0);
   lunp1 = open (namp1, 0);
   lunp2 = open (namp2, 0);
   lunp3 = open (namp3, 0);

/* Create names for output images. */

   strcpy (namebr, mkname (namp0, "pbr"));
   strcpy (namebt, mkname (namp0, "pbt"));
   strcpy (nampol, mkname (namp0, "pol"));
   strcpy (namepb, mkname (namp0, "ppb"));
   strcpy (namprt, mkname (namp0, "prt"));
   lunbr = creat (namebr, 0644);
   lunbt = creat (namebt, 0644);
   lunp  = creat (nampol, 0644);
   lunpb = creat (namepb, 0644);
   lunrt = creat (namprt, 0644);
   close (lunbr);
   close (lunbt);
   close (lunp);
   close (lunpb);
   close (lunrt);
   lunbr = open (namebr, 1);
   lunbt = open (namebt, 1);
   lunp  = open (nampol, 1);
   lunpb = open (namepb, 1);
   lunrt = open (namprt, 1);

/* Copy housekeeping record from clear image to output image files. */

   nread = read (lunp0, linbuf, nbylin);
   icpu = (int) *(linbuf + 70) & 0377;		/* CPU designation: 8=Vax. */

   nwrite = write (lunbr, linbuf, nbylin);
   nwrite = write (lunbt, linbuf, nbylin);
   nwrite = write (lunp,  linbuf, nbylin);
   nwrite = write (lunpb, linbuf, nbylin);
   nwrite = write (lunrt, linbuf, nbylin);

/* Copy bitmap record from clear image to output image files. */

   nread = read (lunp0, linbuf, nbylin);

   nwrite = write (lunbr, linbuf, nbylin);
   nwrite = write (lunbt, linbuf, nbylin);
   nwrite = write (lunp,  linbuf, nbylin);
   nwrite = write (lunpb, linbuf, nbylin);
   nwrite = write (lunrt, linbuf, nbylin);

/* Position pointer to start of image data within files. */

   lseek (lunp1, 2*nbylin, 0);
   lseek (lunp2, 2*nbylin, 0);
   lseek (lunp3, 2*nbylin, 0);

/* Allocate storage for polaroid line arrays. */

   p1 = (short *) malloc (nbylin);
   p2 = (short *) malloc (nbylin);
   p3 = (short *) malloc (nbylin);

   for (j=0; j<npix; j++)
      {
      y = (float) j;
      rdlinp (lunp1, lunp2, lunp3, p1, p2, p3);
      if (icpu != 16)		/* Reverse bytes if not processed by Sun-4. */
	 {
	 srevbyt (p1, nbypix, npix);
	 srevbyt (p2, nbypix, npix);
	 srevbyt (p3, nbypix, npix);
	 }
      for (i=0; i<npix; i++)
         {
         x = (float) i;
         rcoord (&radius, &theta, &x, &y, -1, scroll);

         *(s)   = (float) *(p1+i);
         *(s+1) = (float) *(p2+i);
         *(s+2) = (float) *(p3+i);

         /* Compute ss : the resultant Stokes vector. */

         mxm_(rinv,&n3,s,&n3,ss,&n1);
         si = ss[0];
         sq = ss[1];
         su = ss[2];

         /* Remove stray light & vignetting. */

         cc = calcor (radius);		/* Calibration correction. */
         vig = vignet(radius);		/* Vignetting correction. */
         vinv = 1. / vig;		/* Inverse vignetting. */
         str = stray(radius) * Cinv;	/* Stray Light. */
         if (lfc == 1)  fc = fcorona (radius, theta) * Cinv;  else fc = 0.;

      /* Corrected Stokes intensity derived from 3 polaroids (+60,0,-60 deg). */

         ci = (si * cc - str) * vig - fc;
         cq =  sq * cc * vig;
         cu =  su * cc * vig;

/*
 *         if (i==j)  fprintf (stdout, "ci:%g   cq:%g   cu:%g\n", ci, cq, cu);
 */

         /* Clear polaroid intensity. */

/*
 *         clear = (p0[i] * cc - str) * vig;
 *         p0[i] = clear;
 */
         if (ci < 0.)  ci = 0.;
         p123[i] = (short) (ci * vinv) & 0177777;
/*
 *         if (ci == 0.)
 *            p0123[i] = 0.;
 *         else
 *            p0123[i] = clear / ci;
 */

         /* Polarization * Brightness. */

         polbrt = sqrt ((double) (cq*cq + cu*cu));
         if (polbrt < 0.)  polbrt = 0.;
         if (polbrt > 32767.)  polbrt = 32767.;
         pb[i] = ((short) (polbrt * vinv)) & 0177777;

         /* Polarization fraction. */

         pol = 0.;
         if(ci > 0.)  pol = polbrt / ci;
         p[i] = ((short) (pol * 500.)) & 0177777;

         /* Radial & tangential brightness components. */

         brad = ((ci - polbrt) / 2.) * vinv;
         btan = ((ci + polbrt) / 2.) * vinv;
         if (brad < 0.)  brad = 0.;
         if (btan < 0.)  btan = 0.;
         br[i] = ((short) brad) & 0177777;
         bt[i] = ((short) btan) & 0177777;

         /* Polarization deviation angle. */

         alpha = 0.5 * degrad * (float) atan2 ((double) cu, (double) cq);
         alpha = alpha - 90.;
         if(alpha < 0.)    alpha = alpha + 360.;
         if(alpha > 360.)  alpha = alpha - 360.;

         delang = alpha - theta;
         fsign = 1.;
         if(delang != 0.) fsign = delang / fabs ((double) delang);
         while (fabs((double) delang) > 45.) delang -= 90. * fsign;

         }
      fprintf (stdout, "line:%d  br:%d  bt:%d  p:%d  pb:%d  p123:%d\n",
                        j, *(br+j), *(bt+j), *(p+j), *(pb+j), *(p123+j));
      wrlinp (lunbr, lunbt, lunp, lunpb, br, bt, p, pb);
      srevbyt (p123, nbypix, npix);		/* Reverse bytes. */
      nwrite = write (lunrt, p123, nbylin);
      }

   close (lunp1);
   close (lunp2);
   close (lunp3);
   close (lunbr);
   close (lunbt);
   close (lunp);
   close (lunpb);
   close (lunrt);

   return(0);
}
