/* stokes6.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Compute Stokes vector from 3 polaroid intensity images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 27 January 1987
 * 9 December 1987: Set Munro calibration correction to 1.
 * 9 December 1987: Set Stray light correction to 0.
 * 14 November 1988: reverse bytes from input images if not processed on Sun-4.
 * 15 November 1988: use atan function instead of atan2.
 * 27 February 1989: Polarization output image consists of floats, not shorts.
 * 21 November 1989: use Sun byte order for output images.
 * 12 April 1990: Set appropriate bits in byte #222 of housekeeping.
 * 19 April 1990: revise numerically-encoded housekeeping bytes (#221-223).
 *  2 May 1990: hk + bitmap occupy a single, full-sized record for float images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/include/cplib.ext"
#define Cinv 1.e+11
#define maxbyt 3584

stokes (namp0, namp1, namp2, namp3, rinv)

char *namp0;	/* Name of clear image. */
char *namp1;	/* Name of +60 degree polaroid image. */
char *namp2;	/* Name of   0 degree polaroid image. */
char *namp3;	/* Name of -60 degree polaroid image. */
float rinv[3][3];

{
   extern FILE *lun;	    /* File descriptor for data output file. */
   extern char *malloc();   /* Storage Allocation function. */
   extern int mkname ();    /* Create new file name. */
   extern int rcoord ();    /* Coordinate transformation: (r,th) <---> (x,y). */
   extern float calcor ();  /* Munro calibration correction. */
   extern float stray ();   /* Stray Light function. */
   extern float vignet ();  /* Vignetting function. */
   extern float fcorona (); /* F-corona function. */
   extern int rdlinp ();    /* Read line of intensity data from 3 polaroids. */
   extern int wrlinp ();    /* Read line of intensity data from 3 polaroids. */
   extern int mxm_ ();	    /* Matrix multiplication routine. */
   extern double sqrt (), fabs (), atan2 ();	/* Math routines. */

   static char nampol [80];	/* Name of polarization fraction image. */
   static char namepb [80];	/* Name of pB image. */
   static char nambrt [80];	/* Name of total brightness image. */
   static char namebt [80];	/* Name of Bt image. */
   static char namebr [80];	/* Name of Br image. */

   int icpu;
   int i, j;
   int nbylin;
   int nfblin;
   int lunp0;
   int lunp1, lunp2, lunp3;	/* File descriptors for +60, 0, -60 polaroids.*/
   int lunbr, lunbt;
   int lunpol, lunpb;
   int lunbrt;
   int nread, nwrite;
   static int n1 = 1;
   static int n3 = 3;

   static char bytbuf [maxbyt];	/* Line buffer. */
   short *p1, *p2, *p3;		/* Line arrays for polaroid images. */
   short *pb;
   short *brt;
   short *bt;
   short *br;
   float *pol;

   static float s [3];
   static float ss [3];
   float alpha, delang, fsign;
   float brad, btan;
   float cc = 1.0, str = 0.0, vig = 1.0;
   float ci, cq, cu, cuq;
   float clear;
   float fc;
   float polfrac, polbrt;
   float pi = 3.14159;
   float radius, theta;
   float si, sq, su;
   float vinv = 1.0;
   float x, y;
   /*. . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */
   lcc = 0;
   lstr = 0;
   lvig = 1;			/* Remove vignetting. */

   /*** Allocate storage for line arrays. ***/

   nbylin = npixl * nbypix;
   nfblin = npixl * sizeof (float);
   pol = (float *) malloc (nfblin);
   pb  = (short *) malloc (nbylin);
   brt = (short *) malloc (nbylin);
   bt  = (short *) malloc (nbylin);
   br  = (short *) malloc (nbylin);

   /*** Open polaroid image files. ***/

   lunp0 = open (namp0, 0);
   lunp1 = open (namp1, 0);
   lunp2 = open (namp2, 0);
   lunp3 = open (namp3, 0);

   /*** Create names for output images. ***/

   strcpy (nampol, mkname (namp0, "pol"));
   strcpy (namepb, mkname (namp0, "ppb"));
   strcpy (nambrt, mkname (namp0, "prt"));
   strcpy (namebt, mkname (namp0, "pbt"));
   strcpy (namebr, mkname (namp0, "pbr"));
   lunpol= creat (nampol, 0644);
   lunpb = creat (namepb, 0644);
   lunbrt = creat (nambrt, 0644);
   lunbt = creat (namebt, 0644);
   lunbr = creat (namebr, 0644);
   close (lunpol);
   close (lunpb);
   close (lunbrt);
   close (lunbr);
   close (lunbt);
   lunpol = open (nampol, 1);
   lunpb  = open (namepb, 1);
   lunbrt = open (nambrt, 1);
   lunbt  = open (namebt, 1);
   lunbr  = open (namebr, 1);

   /*** Copy housekeeping record from clear image to output image files. ***/

   for (i=0; i<maxbyt; i++)  bytbuf [i] = (char) 0;	/* Clear buffer. */
   nread = read (lunp0, bytbuf, nbylin);
   icpu = (int) *(bytbuf + 70) & 0377;		/* CPU designation: 8=Vax. */
   fprintf (stdout, "icpu: %d\n", icpu);

   *(bytbuf + 52) = '\0';		/* Sun Microsystems byte order. */
   *(bytbuf + 70) = (char) (*(bytbuf + 70) & 07);
   *(bytbuf + 70) = (char) (*(bytbuf + 70) | 020);
   *(bytbuf + 103) = (char) 16;		/* # bits per pixel. */
   *(bytbuf + 110) = (npix & 0177400) / 256;
   *(bytbuf + 111) = (npix &    0377);

   *(bytbuf + 221) = (char) 0;
   if (lstr)  *(bytbuf + 223) = (*(bytbuf + 223) |  1) & 0377;
   if (lvig)  *(bytbuf + 223) = (*(bytbuf + 223) |  2) & 0377;
   if (lreg)  *(bytbuf + 223) = (*(bytbuf + 223) |  4) & 0377;
   if (lfc)   *(bytbuf + 223) = (*(bytbuf + 223) |  8) & 0377;

   *(bytbuf + 222) =  (char) 2;
   nwrite = write (lunpb,  bytbuf, nbylin);
   *(bytbuf + 222) =  (char) 4;
   nwrite = write (lunbrt,  bytbuf, nbylin);
   *(bytbuf + 222) =  (char) 8;
   nwrite = write (lunbt,  bytbuf, nbylin);
   *(bytbuf + 222) = (char) 16;
   nwrite = write (lunbr,  bytbuf, nbylin);

   *(bytbuf + 103) = (char) 32;		/* 4 bytes per pixel. */
   *(bytbuf + 220) = '\0';		/* CALEXP: photometric scaling. */
   *(bytbuf + 221) = '\0';
   *(bytbuf + 222) = (char) 1;
   nwrite = write (lunpol, bytbuf, nfblin/2);	/* Floating Point image. */
   fprintf (stdout, "npix: %d   nbypix: %d   npixl: %d   nwrite: %d\n",
	    npix, nbypix, npixl, nwrite);

   /*** Copy bitmap record from clear image to output image files. ***/

   for (i=0; i<maxbyt; i++)  bytbuf [i] = (char) 0;	/* Clear buffer. */
   nread = read (lunp0, bytbuf, nbylin);

   nwrite = write (lunpb,  bytbuf, nbylin);
   nwrite = write (lunbrt, bytbuf, nbylin);
   nwrite = write (lunbr,  bytbuf, nbylin);
   nwrite = write (lunbt,  bytbuf, nbylin);

   nwrite = write (lunpol, bytbuf, (nfblin/2));	/* Floating Point image. */

   /*** Position pointer to start of image data within files. ***/

   lseek (lunp1, 2*nbylin, 0);
   lseek (lunp2, 2*nbylin, 0);
   lseek (lunp3, 2*nbylin, 0);

   /*** Allocate storage for polaroid line arrays. ***/

   p1 = (short *) malloc (nbylin);
   p2 = (short *) malloc (nbylin);
   p3 = (short *) malloc (nbylin);

   for (j=0; j<npix; j++)
      {
      y = (float) j;
      rdlinp (lunp1, lunp2, lunp3, p1, p2, p3);

      /*** Reverse bytes if byte order not compatible with Sun-4. ***/

      if (nbypix == 2 && bytord == 1)
	 {
	 srevbyt (p1, nbypix, npix);
	 srevbyt (p2, nbypix, npix);
	 srevbyt (p3, nbypix, npix);
	 }

      for (i=0; i<npix; i++)
         {
         x = (float) i;
         rcoord (&radius, &theta, &x, &y, -1, scroll);

         /*** Compute Munro correction, stray light, vignetting, F-corona. ***/

         if (lcc == 1)  cc = calcor (radius);
         else cc = 1.0;			/* Munro calibration correction. */

         if (lvig == 1)  vig = vignet (radius);
         else vig = 1.0;			/* Vignetting correction. */
/*         vinv = 1.0 / vig; */			/* Inverse vignetting. */

         if (lstr)  str = stray (radius) * Cinv;
         else str = 0.0;				/* Stray Light. */

         if (lfc == 1)  fc = fcorona (radius, theta) * Cinv;
         else fc = 0.0;						/* F-corona. */

         /*** Compute ss : the resultant Stokes vector. ***/

         *(s)     = (float) *(p1 + i);
         *(s + 1) = (float) *(p2 + i);
         *(s + 2) = (float) *(p3 + i);

         mxm_ (rinv, &n3, s, &n3, ss, &n1);

         si = ss [0];
         sq = ss [1];
         su = ss [2];

         /*** Corrected Stokes intensity
          *** derived from 3 polaroids (+60, 0, -60 deg).
          ***/

         ci = (si * cc - str) * vig - fc;
         cq =  sq * cc * vig;
         cu =  su * cc * vig;

/*
 *         if (i==j)  fprintf (stdout, "ci:%g   cq:%g   cu:%g\n", ci, cq, cu);
 */

         /*** Clear polaroid intensity. ***/

/*
 *         clear = (p0 [i] * cc - str) * vig;
 *         p0 [i] = clear;
 */
         if (ci < 0.0)  ci = 0.0;
         brt [i] = (short) (ci * vinv) & 0177777;

         /*** Polarization * Brightness. ***/

         polbrt = sqrt ((double) (cq*cq + cu*cu));
         if (polbrt < 0.0)  polbrt = 0.0;
         if (polbrt > 32767.0)  polbrt = 32767.0;
         pb [i] = ((short) (polbrt * vinv)) & 0177777;

         /*** Polarization fraction. ***/

         polfrac = 0.0;
         if (ci > 0.0)  polfrac = polbrt / ci;
         pol [i] = polfrac;
	 if (polfrac > 1.0)  pol [i] = 1.0;

         /*** Tangential & radial brightness components. ***/

         btan = ((ci + polbrt) / 2.0) * vinv;
         brad = ((ci - polbrt) / 2.0) * vinv;
         if (btan < 0.0)  btan = 0.0;
         if (brad < 0.0)  brad = 0.0;
         bt [i] = ((short) btan) & 0177777;
         br [i] = ((short) brad) & 0177777;

         /*** Polarization deviation angle. ***/

         if (cq == 0.0)
            cuq = 0.0;
         else
            cuq = cu / cq;
	 while (cuq < -pi)  cuq += 2 * pi;
	 while (cuq >  pi)  cuq -= 2 * pi;
	 alpha = 0.5 * degrad * (float) atan ((double) cuq);

/*         alpha = 0.5 * degrad * (float) atan2 ((double) cu, (double) cq); */

         alpha = alpha - 90.0;
         if (alpha <   0.0)  alpha = alpha + 360.0;
         if (alpha > 360.0)  alpha = alpha - 360.0;

         delang = alpha - theta;
         fsign = 1.0;
         if (delang != 0.0) fsign = delang / fabs ((double) delang);
         while (fabs ((double) delang) > 45.0) delang -= 90.0 * fsign;

         }
/*
 *      fprintf (stdout, "line:%d  p:%5.2f  pb:%3d  brt:%3d  bt:%3d  br: %3d\n",
 *                        j, *(pol+j), *(pb+j), *(brt+j), *(bt+j), *(br+j));
 */
      wrlinp (lunpb, lunbrt, lunbt, lunbr, pb, brt, bt, br);

      nwrite = write (lunpol, pol, nfblin);
      }

   close (lunp1);
   close (lunp2);
   close (lunp3);
   close (lunbr);
   close (lunbt);
   close (lunpol);
   close (lunpb);
   close (lunbrt);

   return(0);
}
