/* coord.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P coordinate transformation.
 * isign = 1 :  (radius,theta) --> (x,y).
 *       =-1 :  (x,y) --> (radius,theta).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter   HAO/NCAR
 * Andrew L. Stanger
 *  2 March 1984
 * 17 June 1991: Adapt for SGI/IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * (xcen, ycen) specify sun center coordinates; found in 'cpsun.h'.
 */

#include <stdio.h>
#include <math.h>
#include "/usr/people/smm/lib/src/cplib.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

coord (radius, theta, x, y, isign)

float	*radius, *theta; /* Radius (Rsun) & Position Angle (degrees). */
float	*x, *y;		 /* Cartesian coordinates:  (0,0)=upper left.*/
int	isign;		 /* =1: (r,th) --> (x,y). otherwise, vice-versa.*/

{
int	ierr;
float	smmth;
float	angle;

   degnor = 133.0;
   ierr = 0;


   if (isign == 1)		/*** (radius,theta) --> (x,y) ***/
      {
      smmth = *theta + degnor;
      if (smmth > 360.0) smmth -= 360.0;
      angle = (float) (raddeg * smmth);
      *x = (float) (xcen + *radius * pixrs * cos(angle));
      *y = (float) (ycen - *radius * pixrs * sin(angle));

/*
 *     printf ("radius:%f theta:%f\n", *radius, *theta);
 *     printf ("raddeg:%f degrad:%f\n", raddeg, degrad);
 *     printf ("xcen:%f ycen:%f pixrs:%f\n", xcen, ycen, pixrs);
 *     printf ("smmth:%f angle(rad):%f\n", smmth, angle);
 *     printf ("cos(angle):%f sin(angle):%f\n", cos(angle), sin(angle));
 *     printf ("(radius,theta)->(x,y): %f %f %f %f\n", *radius,*theta,*x,*y);
 */

      if ((*x < ip || *x > lp) || (*y < ir || *y > lr))
         {
         ierr = -1;
/*         fprintf (stderr,"(radius,theta)->(x,y): %f %f %f %f\n",
 *                  *radius,*theta,*x,*y);
 */
         }
      }
   else				/*** (x,y) --> (r,theta) ***/
      {
      *radius = sqrt ((*x-xcen) * (*x-xcen) + (*y-ycen) * (*y-ycen)) * rspix;
      *theta = atan2 ((ycen-*y), (*x-xcen)) * degrad - degnor;
      if (*theta < 0) *theta += 360.0;
/*
 *       printf ("(x,y)->(radius,theta): %f %f %f %f\n",
 *               *x,*y,*radius,*theta);
 */
      }
   return (ierr);
}
