      subroutine matinv(a,b,c,n)
c---------------------------------------------------------------------
c   A matrix inverse routine for square matrices using the gauss-jordon
c   ellimination technique.
c---------------------------------------------------------------------
c... Programmer:   Kevin Cole, programmer II
c                  HAO/NCAR - Solar Maximum Mission
c                  Coronagraph/Polarimeter experiment
c                  April 1982
c
c... Hardware:     DEC VAX 11/750
c                  32 bit word length
c    14 July 1984: converted from 11/34 to VAX 750
c---------------------------------------------------------------------
c... dictionary.
c      n           - order of the matrix
c      a(,)        - n x n input matrix
c      b(,)        - n x 2*n working storage.
c      c(,)        - n x n output matrix; the inverse of a(,).
c---------------------------------------------------------------------
c
      dimension a(n,n), b(n,2*n), c(n,n)
c
c
c---------------------------------------------------------------------
c... Create matrix with a on the left side and identity on right.
c---------------------------------------------------------------------
c
      do 10 i=1,n
        do 5 j=1,n
          b(i,j+n) = 0
          b(i,j) = a(i,j)
 5      continue
        b(i,i+n) = 1
10    continue
c
c---------------------------------------------------------------------
c... Do row operation to convert the left hand side of b(,) to the 
c     identity matrix. The inverse of a will then be on the right.
c---------------------------------------------------------------------
c
      do 100 k=1,n
        if (k .eq. n) go to 500
        m = k
c
c... Find maximum element.
c
        do 25 i = k+1, n
          if (abs( b(i,k) ) .gt. abs( b(m,k) )  ) m = i
   25   continue
c
        if (k .eq. m) go to 500
c
        do 30 j = k, 2*n
          bsav = b(k,j)
          b(k,j) = b(m,j)
          b(m,j) = bsav
   30   continue
c
c... Divide row k.
c
  500   do 40 j = k+1, 2*n
          b(k,j) = b(k,j) / b(k,k)
   40   continue
c
        if (k .eq. 1) go to 510
c
        do 60 i = 1, k-1
          do 50 j = k+1, 2*n
            b(i,j) = b(i,j) - b(i,k)*b(k,j)
   50     continue
   60   continue
c
        if (k .eq. n) go to 520
c
  510   do 80 i = k+1, n
          do 70 j = k+1, 2*n
            b(i,j) = b(i,j) - b(i,k) * b(k,j)
   70     continue
   80   continue
  100 continue
c
c... Retrieve inverse from right hand side of b(,).
c
  520 do 120 i = 1, n
        do 110 j = 1, n
          c(i,j) = b(i,j+n)
  110   continue
  120 continue
c
      return
      end
