/*  matinvc.c
 *---------------------------------------------------------------------
 *   A matrix inverse routine for square matrices using the gauss-jordon
 *   ellimination technique.
 *---------------------------------------------------------------------
 *... Programmer:   Kevin Cole, programmer II
 *                  HAO/NCAR - Solar Maximum Mission
 *                  Coronagraph/Polarimeter experiment
 *                  April 1982
 *
 *... Hardware:     DEC VAX 11/750
 *                  32 bit word length
 * 15 July 1984: converted from 11/34 (f4p) to VAX 750 (C).
 * 18 June 1985
 *---------------------------------------------------------------------
 *... dictionary.
 *      n           - order of the matrix
 *      a[n][n]     - n x n input matrix
 *      b[n][2*n]   - n x 2*n working storage.
 *      c[n][n]     - n x n output matrix; the inverse of a(,).
 *---------------------------------------------------------------------
 *
 */

#define n 3

matinvc (a,b,c)

float a[n][n];
float b[n][2*n];
float c[n][n];

{
int i, j, k, m;
float bsav;

/* --------------------------------------------------------------------- */
/* Create matrix with a on the left side and identity on right. */
/* --------------------------------------------------------------------- */

   for (i=0; i<n; i++)
     {
      for (j=0; j<n; j++)
        {
         b[i][j+n] = 0;
         b[i][j] = a[i][j];
        }
        b[i][i+n] = 1.;
     }

/* --------------------------------------------------------------------- */

/*
   Do row operation to convert the left hand side of b(,) to the
     identity matrix. The inverse of a will then be on the right.
 */

/* --------------------------------------------------------------------- */
   for (k=0; k<n; k++)
     {
      if (k == n) goto s500;
      m = k;

/* Find maximum element. */
 
      for (i=k+1; i<n; i++)
        {
         if (abs((double) b[i][k]) > abs((double) b[m][k]) ) m = i;
        }

      if (k != m)
        {
         for (j=k; j<2*n; j++)
           {
            bsav = b[k][j];
            b[k][j] = b[m][j];
            b[m][j] = bsav;
           }
       }

/* Divide row k. */

s500:
     for (j=k+1; j<2*n; j++)   b[k][j] /=  b[k][k];

     if (k != 1)
       {
        for (i=0; i<k-1; i++)
          {
           for (j=k+1; j<2*n; j++) b[i][j] -= b[i][k] * b[k][j];
          }
        if (k == n) goto s520;
       }

     for (i=k+1; i<n; i++)
       {
        for (j=k+1; j<2*n; j++) b[i][j] -= b[i][k] * b[k][j];
       }
    }

/* Retrieve inverse from right hand side of b[][]. */

s520:
     for (i=0; i<n; i++)
        {
         for (j=0; j<n; j++)  c[i][j] = b[i][j+n];
        }

   return(0);
}
