/* ncarplot (x, y, nval, namfil, title, labx, laby, ixscale, iyscale, ibkg,
 *           ismooth)
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   2 December 1986
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namfil : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   ismooth : smoothing flag.
 *             = 0 no smoothing of curve through points.
 *             = 1 smooth curve through points.
 *   igterm = 0 for non-graphics terminal, =1 for graphics terminal.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include "/u/smm/lib/src/cp/cplib.h"

extern struct st
   {
   int ioffs;
   }
   dashd0_;

extern int pcount;

ncarplot (x, y, nval, namfil, title, labx, laby, ixscale, iyscale, ibkg,ismooth)

float	x[], y[];
int	nval;
char	namfil[];
char	title[40], labx[40], laby[40];
int	ixscale, iyscale;
int	ibkg;
int	ismooth;

{
static int iset  = 0;
static int ndash = 0;
static int ldash = 0;
int nosmooth;

	if (ismooth == 0)  nosmooth = 1;  else nosmooth = 0;
	dashd0_.ioffs = ismooth;	/* Set smoothing option. */
	pcount ++;
	if(pcount == 1)  q8qini (namfil, igterm);
	anotat (labx, laby, ibkg, iset, ndash, ldash);
	displa (0., 0., 0.);
	ezxy (x, y, nval, title);

	return(0);
}
