/* poldef.c
 *--------------------------------------------------------------------
 * Defines polaroid image and matrix files for scnpol.
 * Andrew L. Stanger   programmer III   NCAR/HAO   SMM C/P
 * 15 July 1984:  conversion from 11/34 (f4p) to VAX 750 (C).
 * 17 June 1985:  use F77 subroutine 'matinv' for matrix inversion.
 * 2 August 1985: add clear frame.
 * 24 October 1985: Read 'polmtx.dat' file from '/u/stanger/polc' directory.
 * 17 June 1991: Modify for SGI/IRIS GL windows.
 *--------------------------------------------------------------------
 *  r    : mueller matrix (3x3)
 *  rinv : inverse mueller matrix (3x3)
 *--------------------------------------------------------------------
 * IRIX 3.3.1   C programming language
 *--------------------------------------------------------------------
 */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.ext"

/* -------------------------------------------------------------------- */

poldef (namp0, namp1, namp2, namp3, rinv)

char *namp0;		/* File name for clear (NON-polaroid)image. */
char *namp1;		/* File name for +60 degree polaroid image. */
char *namp2;		/* File name for   0 degree polaroid image. */
char *namp3;		/* File name for -60 degree polaroid image. */
float rinv [3][3];

{
   extern int matinv_ ();
   extern int ixc0, iyc0, ixc1, iyc1, ixc2, iyc2, ixc3, iyc3;

   static char namout [80];
   static char answer [2];

   static float r [3][3];
   static float rr [3][3];
   static float rwork [6][3];
   float xc, yc;

   int i, j;
   int idot;
   int n3 = 3;
   int ifd;
   int kres;
   extern FILE *lun;
   FILE *MTX;
/* -------------------------------------------------------------------- */

   /*** Read Mueller matrix from disk file. ***/

   MTX = fopen ("/usr/people/stanger/pol/scan/polmtx.dat", "r");
   fscanf (MTX, "%f%f%f%f%f%f%f%f%f",
	  &r [0][0], &r [0][1], &r [0][2],
	  &r [1][0], &r [1][1], &r [1][2],
	  &r [2][0], &r [2][1], &r [2][2]);
/*
 *  printf ("polmtx.dat matrix  r:\n %10.3f %10.3f %10.3f\n %10.3f %10.3f %10.3f\n %10.3f %10.3f %10.3f\n",
 *	    r [0][0], r [0][1], r [0][2],
 *	    r [1][0], r [1][1], r [1][2],
 *	    r [2][0], r [2][1], r [2][2]);
 */
   fclose (MTX);

   /*** Request polaroid file names. ***/

   /***  p0: clear image. ***/

   fprintf (stderr, "Enter file name of clear (NON-polaroid) image : ");
   fscanf  (stdin,  "%s", namp0);

   cphskp (namp0);

   fprintf (stderr, "Enter Hi res disk center [x,y] (0 if unknown) :");
   fscanf  (stdin,  "%f,%f", &xc, &yc);

   kres = H.ires + 1;
   ixc0 = (int) (xc) / kres;
   iyc0 = (int) (yc) / kres;

   /***  p1: +60 degrees. ***/

   fprintf (stderr, "Enter file name of +60 degree polaroid image : ");
   fscanf  (stdin,  "%s", namp1);

   cphskp (namp1);

   fprintf (stderr, "Enter Hi res disk center [x,y] (0 if unknown) :");
   fscanf  (stdin,  "%f,%f", &xc, &yc);

   ixc1 = (int) (xc) / kres;
   iyc1 = (int) (yc) / kres;

   /***  p2: 0 degrees. ***/

   fprintf (stderr, "Enter file name of zero degree polaroid image : ");
   fscanf  (stdin,  "%s", namp2);

   cphskp (namp2);
   fprintf (stderr, "Enter Hi res disk center [x,y] (0 if unknown) :");
   fscanf  (stdin,  "%f,%f", &xc, &yc);

   ixc2 = (int) (xc) / kres;
   iyc2 = (int) (yc) / kres;

   /***  p3: -60 degrees. ***/

   fprintf (stderr, "Enter file name of -60 degree polaroid image : ");
   fscanf  (stdin,  "%s", namp3);

   cphskp (namp3);

   fprintf (stderr, "Enter Hi res disk center [x, y] (0 if unknown) :");
   fscanf  (stdin,  "%f, %f",  &xc,  &yc);

   ixc3 = (int) (xc) / kres;
   iyc3 = (int) (yc) / kres;

   kres = H.ires + 1;

   /*** 4-quadrant composite image ? ***/

   fprintf (stderr, "4-quadrant composite images [360 degree view] (y/n) ?");
   fscanf  (stdin,  "%s", answer);

   ifd = 1;
   if (strncmp ("y", answer, 1) == 0)  ifd = 2;

   /*** Determine disk center coordinate indices. ***/

   if (ifd == 2)
      {
      ixc0 = (447 / kres);
      iyc0 = (447 / kres);
      ixc1 = (447 / kres);
      iyc1 = (447 / kres);
      ixc2 = (447 / kres);
      iyc2 = (447 / kres);
      ixc3 = (447 / kres);
      iyc3 = (447 / kres);
      }
   if (ixc0 == 0)  ixc0 = (int) xcen;
   if (iyc0 == 0)  iyc0 = (int) ycen;
   if (ixc1 == 0)  ixc1 = (int) xcen;
   if (iyc1 == 0)  iyc1 = (int) ycen;
   if (ixc2 == 0)  ixc2 = (int) xcen;
   if (iyc2 == 0)  iyc2 = (int) ycen;
   if (ixc3 == 0)  ixc3 = (int) xcen;
   if (iyc3 == 0)  iyc3 = (int) ycen;

   fprintf (stderr, "ifd:%d  pixrs:%f  rspix:%f\n", ifd, pixrs, rspix);
   fprintf (stderr, "xcen:%f  ycen:%f\n", xcen, ycen);
   fprintf (stderr, "ixc0:%d  iyc0:%d\n", ixc0, iyc0);
   fprintf (stderr, "ixc1:%d  iyc1:%d\n", ixc1, iyc1);
   fprintf (stderr, "ixc2:%d  iyc2:%d\n", ixc2, iyc2);
   fprintf (stderr, "ixc3:%d  iyc3:%d\n", ixc3, iyc3);

   /*** Open output file to store polarization results. ***/

   strcpy (namout, namp0);
   for (i=0; i<strlen(namp0); i++)  if (namout[i] == '.')  idot = i;
   namout[idot] = '\0';
   strcat (namout, ".log");
   fprintf (stderr, "Output file name: %s\n", namout);
   lun = fopen (namout, "w");
   fprintf (lun, "******* P O L S C N . C *******\n");

   /*** Invert [polaroid]*[optical] matrix. ***/

   matinv_ (r, rwork, rr, &n3);

   fprintf (stderr, "input matrix  r:\n");
   fprintf (stderr, "%10.3f %10.3f %10.3f\n%10.3f %10.3f %10.3f\n%10.3f %10.3f %10.3f\n",
           r [0][0], r [0][1], r [0][2], 
           r [1][0], r [1][1], r [1][2], 
           r [2][0], r [2][1], r [2][2]);
   fprintf(lun, "input matrix  r:\n");
   fprintf(lun, "%10.3f %10.3f %10.3f\n%10.3f %10.3f %10.3f\n%10.3f %10.3f %10.3f\n",
           r [0][0], r [0][1], r [0][2], 
           r [1][0], r [1][1], r [1][2], 
           r [2][0], r [2][1], r [2][2]);

/*
   rinv [0][0] = .855;
   rinv [0][1] = .780;
   rinv [0][2] = .855;
   rinv [1][0] = -.863;
   rinv [1][1] = 1.652;
   rinv [1][2] = -.863;
   rinv [2][0] = 1.449;
   rinv [2][1] = 0.;
   rinv [2][2] = -1.449;
 */

   fprintf (stderr, "inverse matrix  rr:\n");
   fprintf (stderr, "%10.3f %10.3f %10.3f\n%10.3f %10.3f %10.3f\n%10.3f %10.3f %10.3f\n",
           rr [0][0],rr [0][1],rr [0][2],
           rr [1][0],rr [1][1],rr [1][2],
           rr [2][0],rr [2][1],rr [2][2]);
   fprintf (lun, "inverse matrix  rr:\n");
   fprintf (lun, "%f %f %f\n%f %f %f\n%f %f %f\n",
            rr [0][0],rr [0][1],rr [0][2],
            rr [1][0],rr [1][1],rr [1][2],
            rr [2][0],rr [2][1],rr [2][2]);

   /*** Return transpose of matrix 'rr' in array 'rinv'. ***/

   for (i=0; i<3; i++)
      for (j=0; j<3; j++)
         rinv [i][j] = rr [j][i];
}
