/* svf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Plot stray light, vignetting, & F-corona.
 * Use ATM/WLCE F-corona function (Saito equation).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 25 February 1987
 * 19 June 1990: add electron/Bsun plot.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

/* Global variables. */

#include <stdio.h>
#include "/usr/people/smm/lib/include/cplib.h"

/* Standard C system library and Math functions library. */

#include <stdio.h>
#include <math.h>

int pcount = 0;
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

main ()

{

extern	float calcor ();	/* Munro calibration correction. */
extern	float elec ();		/* Electrons per B/Bsun. */
extern	float stray ();		/* Stray light. */
extern	float vignet ();	/* Vignetting. */
extern	float fcorona ();	/* F-corona function. */
extern  xyplot ();		/* XY plot using NCAR system plot package. */

static char buff [84];
static char namimg [80] = "svf.plt";	/* Base name for output files. */
static char namdat [80];		/* Scan data file name. */
static char answer [2];			/* Storage for y/n response. */
static char namemc [80];		/* Metacode file name. */
static char namgks [] = "gks.cgm";
static char labx [40];			/* Label for X-axis. */
static char laby [40];			/* Label for Y-axis. */
static char title [40];			/* Graph title. */

static float cc [MAXSCN];	/* Pointer to calib. correction scan array. */
static float ebsun [MAXSCN];	/* Pointer to electron/Bsun scan array. */
static float str [MAXSCN];	/* Pointer to stray light scan array. */
static float vig [MAXSCN];	/* Pointer to vignetting scan array. */
static float fcor0 [MAXSCN];	/* Pointer to F-corona scan array. */
static float fcor90 [MAXSCN];	/* Pointer to F-corona scan array. */
static float scandx [MAXSCN];	/* Pointer to dependent scan array. */

float	r;		/* Radial value. */
float	rmin, rmax;	/* Radial limits (Rsun). */
float	rinc;		/* Radial increment. */
float	theta = 0.0;	/* Solar position angle (degrees). */
float	th0 = 0.0;	/* 0 degrees. */
float	th90 = 90.0;	/* 90 degrees. */

int	scntyp = 1;	/* Scan type: 1=radial, 2=line. */
int	nval;		/* # values in scan. */
int	i;
int	idot;		/* Index pointer to "." in image filename.*/
int	ixscale = 0;	/* x-axis scaling type; 0=linear. */
int	iyscale = 0;	/* y-axis scaling type; 0=linear. */
int	ibkg = 0;	/* Background flag; 0=tick marks only. */
int	plotit = 1;
int	noplot = 0;
int	clip = 0;
int	isiz = 0;
int	wksid = 1;
int	lunit = 8;
int	wkstype = 1;
int	lunerr;

FILE	*lunsvf;	/* File descriptor for scan data file. */
/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

   printf ("Are you using a graphics terminal (y/n) ? ");
   scanf ("%s", answer);
   if (strncmp ("y", answer, 1) == 0)
      {
      igterm = 1;
/*      gksini_ (&plotit, namgks); */
      }
   else
      {
      igterm = 0;
/*      gksini_ (&noplot, namgks); */
      }
   opngks_ ();
/*   gsclip_ (&clip); */

   /* Request input information. */

/*
 *   printf ("Enter solar position angle (deg.): ");
 *   scanf  ("%f", &theta);
 */
   printf ("Enter solar radius min, max, & increment (Rsun): ");
   scanf  ("%f,%f,%f", &rmin, &rmax, &rinc);

   /* Get correction data from interpolation tables. */

   nval = (int) ((rmax - rmin ) / rinc) + 2;
   r = rmin - rinc;
   for (i=0; i<nval; i++)
       {
	r += rinc;
	scandx [i] = r;
	cc [i] = calcor (r);
	ebsun [i] = elec (r);
	str [i] = stray (r);
	vig [i] = vignet (r);
	fcor0 [i] = fcorona (r, th0);
	fcor90 [i] = fcorona (r, th90);
	}

   /* Print scan values. */

   for (i=0; i<strlen (namimg); i++)  if (namimg[i] == '.')  idot = i;
   strncpy (namdat, namimg, idot);
   strncpy (namemc, namimg, idot);
   strcat (namdat, ".dat");
   strcat (namemc, ".cgm");
   printf ("corrections data file:  %s\n", namdat);
   printf ("metacode plot file: %s\n", namemc);
   lunsvf = fopen (namdat, "w");

   fprintf (lunsvf, "SMM C/P Corrections\n");
   fprintf (lunsvf, "Radial scan  %6.2f to %6.2f Rsun  %7.2f degrees\n\n",
            rmin, rmax, theta);
   fprintf (lunsvf, "        position    Munro       stray              Fcorona   Fcorona      electrons\n");
   fprintf (lunsvf, "radius   angle     cal.corr     light    vignet   0 degrees  90 degrees    /Bsun\n");
   for (i=0; i<nval; i++)
      fprintf (lunsvf,"%6.2f  %8.2f   %8.2f   %8.2e   %6.2f   %9.2e   %10.2e   %9.3e\n",
        scandx[i], theta, cc[i], str[i], vig[i], fcor0[i], fcor90[i], ebsun[i]);

   /* Plot "scans". */
 
   sprintf (labx, "solar radius (Rsun)");

   sprintf (title, "Radial Scan %5.1f deg. SMM C/P Cal.Corr.", theta);
   sprintf (laby, "Calibration Correction Factor");
   gets (buff);
   xyplot (scandx, cc, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "Radial Scan %5.1fdeg Electrons/Bsun", theta);
   sprintf (laby, "# Electrons per B/Bsun");
   gets (buff);
   xyplot (scandx, ebsun, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "Radial Scan %5.1fdeg SMM C/P Stray Light", theta);
   sprintf (laby, "Stray Light (B/Bsun)");
   gets (buff);
   xyplot (scandx, str, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   sprintf (title, "Radial Scan %5.1fdeg. SMM C/P Vignetting", theta);
   sprintf (laby, "Vignetting Correction Factor");
   gets (buff);
   xyplot (scandx, vig, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th0;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P F-corona", theta);
   sprintf (laby, "F-corona (B/Bsun)");
   gets (buff);
   xyplot (scandx, fcor0, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);

   theta = th90;
   sprintf (title, "Radial Scan %5.1f deg. SMM C/P F-corona", theta);
   sprintf (laby, "F-corona (B/Bsun)");
   gets (buff);
   xyplot (scandx, fcor90, nval, namemc, title, labx, laby,
	  ixscale, iyscale, ibkg);
   clsgks_ ();
   exit (0);
}
