/* ag_xyplot.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   9 February 1987
 *  27 March 1990: convert to GKS.
 *   6 April 1990: use Autograph subroutines.
 *  15 October 1990: specify format for numeric tick-mark labels.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namemc : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   ltype	x-axis	y-axis
 *   -----      ------  ------
 *	1	linear	linear
 *	2	linear	log
 *	3	log	linear
 *	4	log	log
 */
xyplot (x, y, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg)

float	x[], y[];
int	nval;
char	namemc[];
char	title[60], labx[40], laby[40];
int	ixscale, iyscale;
int	ibkg;

{
static int ixlab = 1, iylab= 1;
int majrx = 10;
int majry = 10;

int i;
int is = 2;
int io = 0;
int ic = 0;
int ll = 1;
int lentit;
int lenlabx;
int lenlaby;
int ltype;
int minrx, minry;
int npts;
int lbac;
int lset = 0;
int ndsh = 0;
int iiex = 1;	/* Increment between successive X values to be plotted. */
int iiey = 1;	/* Increment between successive Y values to be plotted. */
int kdsh = 0;	/* Dash-line pattern. */
int ival;

float dxx, dxn;
float dyy, dyn;
float fl = 0.0, fr = 1.0, fb = 0.0, ft = 1.0;
float ul = 0.0, ur = 1.0, ub = 0.0, ut = 1.0;
float px, py;
float xmin, xmax, ymin, ymax;
float size, angd, cntr;
static float gwin [4] = {0.0, 1.0, 0.0, 1.0};
static char namsav [80] = "xyplot.cgm";
static char buf1 [84], buf2 [84];
static char dshc [4];
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   if (strcmp (namemc, namsav) != 0)  gkschnam_ (namemc);

/* Set scaling type. */

   if (ixscale == 0 && iyscale == 0)  ltype = 1;	/* x linear, y linear */
   if (ixscale == 0 && iyscale == 1)  ltype = 2;	/* x linear, y log    */
   if (ixscale == 1 && iyscale == 0)  ltype = 3;	/* x log,    y linear */
   if (ixscale == 1 && iyscale == 1)  ltype = 4;	/* x log,    y log    */

/* Determine minimum & maximum values for axes ranges. */

   xmin =  1.e+30;
   xmax = -1.e+30;
   ymin =  1.e+30;
   ymax = -1.e+30;

   npts = nval;
   for (i=0; i<npts; i++)
      {
      if (xmin > x[i])  xmin = x[i];
      if (xmax < x[i])  xmax = x[i];
      if (ymin > y[i])  ymin = y[i];
      if (ymax < y[i])  ymax = y[i];
      }

/* Write title & axes labels. */

   lentit = strlen (title);
   lenlabx = strlen (labx);
   lenlaby = strlen (laby);
   is = 2;     io = 0;     ic = 0;
   fl = 0.0;   fr = 1.0;   fb = 0.0;   ft = 1.0;
   ul = 0.0;   ur = 1.0;   ub = 0.0;   ut = 1.0;
   ll = 1;
   set_ (&fl, &fr, &fb, &ft, &ul, &ur, &ub, &ut, &ll);

   px = 0.5;   py = 0.95;   size = 14.0;   angd = 0.0;   cntr = 0.0;
/*   pwrit_ (&px, &py, title, &lentit, &is, &io, &ic); */
/*   plchhq_ (&px, &py, title, &size, &angd, &cntr, strlen (title)); */

   px = 0.6;   py = 0.03;   size = 12.0;   cntr = 0.0;
/*   pwrit_ (&px, &py, labx, &lenlabx, &is, &io, &ic); */
/*   plchhq_ (&px, &py, labx, &size, &angd, &cntr, strlen (labx)); */

   px = 0.05;   py = 0.55;   io = 90;   size = 12.0;   angd = 90.0;   cntr = 0.0;
/*   pwrit_ (&px, &py, laby, &lenlaby, &is, &io, &ic); */
/*   plchhq_ (&px, &py, laby, &size, &angd, &cntr, strlen (laby)); */

/* Determine minor * major scaling values for axes. */
/*
 *   nicexr (&xmin, &xmax, &majrx, &minrx, &dxx, &dxn);
 *   nicexr (&ymin, &ymax, &majry, &minry, &dyy, &dyn);
 */

   fl = 0.2;   fr = 1.00;   fb = 0.1;   ft = 0.9;
   set_ (&fl, &fr, &fb, &ft, &xmin, &xmax, &ymin, &ymax, &ltype);

/*
 *  periml_ (&majrx, &minrx, &majry, &minry);
 *
 *
 *   gridal_(&majrx, &minrx, &majry, &minry, &ixlab, &iylab, &igph, &xmin, &ymin);
 *
 *
 *   lbac = ibkg;
 *   lbac = 2;
 *   anotat_ (labx, laby, &lbac, &lset, &ndsh, dshc,
 *             strlen (labx), strlen(laby), strlen (dshc));
 */

/*   agstup_ (x, &npts, &iivx, &nevx, &iiex, y, &npts, &iivx, &nevx, &iiex); */

   strcpy (buf1, "FRAME.");
   ival = 2;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "GRAPH WINDOW.");
   ival = 4;
   agsetp_ (buf1, gwin, &ival, strlen (buf1));	/* Set graph frame. */

   strcpy (buf1, "BACKGROUND TYPE.");
   ival = ibkg;
   agseti_ (buf1, &ival, strlen (buf1));	/* Background pattern. */

   strcpy (buf1, "LABEL/CONTROL.");
   ival = 2;
   if (ibkg == 4)
      agseti_ (buf1, &ival, strlen (buf1));	/* Turn on info for lablels. */

   ival = 0;
   strcpy (buf1, "X/LOGARITHMIC.");
   agseti_ (buf1, &ival, strlen (buf1));	/* X axis --> linear. */

   ival = 0;
   strcpy (buf1, "Y/LOGARITHMIC.");
   agseti_ (buf1, &ival, strlen (buf1));	/* Y axis --> linear. */

   /* X axis --> float format. */
   ival = 3;
   strcpy (buf1, "X/TYPE.");
/*   agseti_ (buf1, &ival, strlen (buf1)); */

   /* X axis --> omit fraction. */
   ival = 0;
   strcpy (buf1, "X/FRAC.");
/*   agseti_ (buf1, &ival, strlen (buf1)); */

   /* Y axis --> scientific format. */
   ival = 1;
   strcpy (buf1, "Y/TYPE.");
/*   agseti_ (buf1, &ival, strlen (buf1)); */

   /* Y axis --> 1 digit coef.format.*/
   ival = 1;
   strcpy (buf1, "Y/EXPO.");
/*   agseti_ (buf1, &ival, strlen (buf1)); */

   /*** Title ***/

   strcpy (buf1, "LABEL/NAME.");
   strcpy (buf2, "T");
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   strcpy (buf1, "LINE/MAXIMUM.");
   ival = 60;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/NUMBER.");
   ival = 100;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/TEXT.");
   sprintf (buf2, "%s$", title);
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   /*** X-axis label. ***/

   strcpy (buf1, "LABEL/NAME.");
   strcpy (buf2, "B");
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   strcpy (buf1, "LINE/NUMBER.");
   ival = -100;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/TEXT.");
   sprintf (buf2, "%s$", labx);
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   /*** Y-axis label. ***/

   strcpy (buf1, "LABEL/NAME.");
   strcpy (buf2, "L");
   agsetc_ (buf1, buf2, strlen (buf1), strlen (buf2));

   strcpy (buf1, "LINE/NUMBER.");
   ival = 100;
   agseti_ (buf1, &ival, strlen (buf1));

   strcpy (buf1, "LINE/TEXT.");
   sprintf (buf2, "%s$", laby);
   agsetc_ (buf1, buf2);

/*   agback_ ();    */

/* Draw curve through data points. */

/*   agcurv_ (x, &iiex, y, &iiey, &npts, &kdsh); */

/*   boundary (); */	 		/* Draw boundary box. */

   sprintf (buf2, "%s$", title);
   ezxy_ (x, y, &npts, buf2, strlen (buf2));
      
   frame_ ();

   strcpy (namsav, namemc);

   return (0);
}
