/* calcor.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P calibration correction.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 18 November 1985: use correction factors from MacQueen memo (29 Aug 85).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

float
calcor (r)

float	r;	/* Radius (Rsun). */

{
#define NTAB 85

extern float lint();
float	cc;		/* Calibration correction. */

/* Radial values in units of R/Rsun corresponding to cal. correction table. */

static float rtab[NTAB] = 
     { 0.00,0.10,0.20,0.30,0.40,0.50,0.60,0.70,0.80,0.90,
       1.00,1.10,1.20,1.30,1.40,1.50,1.60,1.70,1.80,1.90,
       2.00,2.10,2.20,2.30,2.40,2.50,2.60,2.70,2.80,2.90,
       3.00,3.10,3.20,3.30,3.40,3.50,3.60,3.70,3.80,3.90,
       4.00,4.10,4.20,4.30,4.40,4.50,4.60,4.70,4.80,4.90,
       5.00,5.10,5.20,5.30,5.40,5.50,5.60,5.70,5.80,5.90,
       6.00,6.10,6.20,6.30,6.40,6.50,6.60,6.70,6.80,6.90,
       7.00,7.10,7.20,7.30,7.40,7.50,7.60,7.70,7.80,7.90,
       8.00,8.10,8.20,8.30,8.40};

/* Calibration correction table. */
/* Extrapolated guess from Munro's calibration study:
 *    a comparison between SMM C/P and the 16 Feb 85 eclipse.
 * Values below are taken from a memo from Bob MacQueen dated 29 Aug 85.
 */

static float cctab[NTAB] =
	{  0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0.,   0., 
	   0.,   0.,   0.,   0.,   0., 3.50, 3.35, 3.23, 3.03, 2.82, 
	 2.70, 2.56, 2.50, 2.38, 2.33, 2.25, 2.20, 2.13, 2.08, 2.06, 
	 2.04, 2.00, 1.98, 1.98, 1.98, 1.96, 1.96, 1.96, 1.96, 1.96, 
	 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 
	 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 
	 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 
	 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 1.96, 
	 1.96, 1.96, 1.96, 1.96, 1.96};
/* ----------------------------------------------------------------------- */

   cc = lint(r,rtab,cctab,NTAB);

   return(cc);

}
