/* linescan.c
 *--------------------------------------------------------------------------
 * L I N E S C A N ()	- Do line scans on image data sets.
 *--------------------------------------------------------------------------
 * Kevin Cole	Programmer II
 * Creation Date: 9/12/84
 *--------------------------------------------------------------------------
 * Major Variables:
 *	row,col		- current row and column position. (0 <= row <= 24)
 *			  (0 <= col <= 24).
 *	cmd		- current character command from keyboard.
 *	L_line[]	- Menu lines 
 *
 * Libraries required:
 * 	libcurses.
 *----------------------------------------------------------------------------
 */
#include "scan.ext"

linescan(icb,scan,scandx,nval,brow)

struct IMAGE *icb[];
int *nval,brow;
float scan[], scandx[];
{
   float  dummy, line1, pixel1, line2, pixel2; 
   int    i, WRITEFILE, tabstop;
   char    cmd, scanid[20];
   char    buff[82];
   FILE *fd, *fopen();
   WINDOW *rwin, *newwin();

   static char *L_line[] =
      {
      "LINE SCAN: ",
      " Line 1  [deg]    = ",
      " Pixel 1 [rsun]   = ",
      " Line 2  [rsun]   = ",
      " Pixel 2 [rsun]   = "  
      };

   WRITEFILE = 0;
   if (strcmp(scans->linefile, "off") != 0)
      {
      if ((fd = fopen (scans->linefile, "a")) == NULL)
    	 {
	 sprintf (buff, "Unable to open %s      ", scans->linefile);
	 disp_err (buff);
         }
      else
         WRITEFILE = 1;
      }
   if (scans->GRN)			/* Initialize overlay for radial scan */
      {
      rwech (scans->RSCANOVL);
      disp(scans->RSCANOVL);
      zcursor(ON);			/* Display the zoom cursor	      */
      }
   tabstop = strlen (L_line[1]);
   strcpy(scanid, L_line[0]);

   /* Make an SMM image control block. */

   if ((icb[scans->index]->addr == NULL) || (scans->index > scans->maxindex))
      {
      if (WRITEFILE) fclose (fd);
         return (-1);
      }

   rwin = newwin (9, 45, brow, 5);
   wpaint (rwin);

L_Begin:
   revon ();
   hion (); 
   (void) sprintf (buff,"%s %s ", scanid, icb[scans->index]->name);
   mvwaddstr (rwin, 0, 0, buff); 
   wrefresh (rwin);
   hioff ();
   revon (); 

   wdisplines (rwin, L_line, 5);
/*-----------------------------------------------------------*/

   line1 = scans->line1;
   pixel1 = scans->pixel1;
   line2 = scans->line2;
   pixel2 = scans->pixel2;
   wdispdefs (rwin, 2, tabstop, line1, pixel1, line2, pixel2);
	
/* Now, get the info from the screen. */

   wmove (rwin, 2,tabstop); wrefresh(rwin);

   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC) 
      {
      cleanup (rwin);
      if (WRITEFILE) fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto L_Begin;
   if (cmd == ':')
      {
      command ();
      }

/* Process Ctrl-G :   Get coordinates from current cursor position. */

   if (( cmd == 7) && (scans->GRN) ) 
      {
      if (wgetzoomf (icb[scans->index],rwin,tabstop,&line1,&pixel1,2,2)<0)
	 {
 	 if (WRITEFILE) fclose (fd);
            return (-1);
         }
      goto skip1;
      }

/* Get line1. */

   if (strlen (buff) > 0)
      (void) sscanf(buff,"%f",&line1);
   hion (); 
   sprintf (buff, "%7.2f__", line1); 
   hioff ();
   revon (); mvwaddstr (rwin, 2, tabstop, buff);

   wmove (rwin, 3, tabstop);  wrefresh (rwin);

   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC)
      {
      cleanup (rwin);
      if (WRITEFILE)  fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto L_Begin;
   if (cmd == ':')
      {
      command ();
      }

/* Get pixel1. */

   if (strlen (buff) > 0)
      (void) sscanf (buff, "%f", &pixel1);
   sprintf (buff, "%7.2f______", pixel1);
   revon ();  mvwaddstr (rwin, 3, tabstop, buff);
   wrefresh (rwin);

skip1:
   wmove (rwin, 4,tabstop); wrefresh(rwin);
   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC) 
      {
      cleanup (rwin);
      if (WRITEFILE)  fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto L_Begin;
   if (cmd == ':')
      {
      command ();
      }

   if (( cmd == 7) && scans->GRN)  
      {
      if (wgetzoomf (icb[scans->index],rwin,tabstop,&line2,&pixel2,4,0) < 0)
	 {
         if (WRITEFILE)  fclose (fd);
         return (-1);
	 }
      goto skip2;
      }

/* Get line2. */

   if (strlen (buff) > 0)
      (void) sscanf(buff,"%f",&line2);
   sprintf (buff, "%7.2f______", line2);
   mvwaddstr (rwin, 4, tabstop, buff);
   revon ();  wrefresh (rwin);
skip2:
   wmove (rwin, 5,tabstop);   wrefresh (rwin);
   if ((cmd = (char) readline(rwin, buff, 64, '-')) == ESC) 
      {
      cleanup (rwin);
      if (WRITEFILE)  fclose (fd);
      return (0);
    }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto L_Begin;
   if (cmd == ':')
      {
      command ();
      }

/* Get pixel2. */

   if (strlen (buff) > 0)
      (void) sscanf (buff, "%f",&pixel2);
   sprintf (buff, "%7.3f______", pixel2);
   mvwaddstr (rwin, 5, tabstop, buff);
   revon ();  wrefresh (rwin);

   lscan (icb[scans->index],line1,pixel1,line2,pixel2,2,scan,scandx,nval);
   scans->line1 = line1;
   scans->pixel1 = pixel1;
   scans->line2 = line2;
   scans->pixel2 = pixel2;

   (void) sprintf (buff," Points: %3d",*nval);
   if (WRITEFILE)
      {

    /* Write the header record. */

      fprintf (fd, "\n");
      fprintf (fd,
"-------------------------------------------------------------------------------\n");
      fprintf (fd,"File: %s  TIME: %d:%d:%d  DOY: %d  YEAR: %d\n",
              icb[scans->index]->name,   icb[scans->index]->hour,
	      icb[scans->index]->minute, icb[scans->index]->sec,
	      icb[scans->index]->doy,    icb[scans->index]->year);
      fprintf (fd,"Type: %s   Brightness Scale: %f    Center x: %f y: %f\n",
		icb[scans->index]->type ,icb[scans->index]->scale,
		icb[scans->index]->xcen ,icb[scans->index]->ycen);
       fprintf (fd,
"-------------------------------------------------------------------------------\n");

     /* write the data arrays. */

      fprintf (fd, "  LINE SCAN     BRIGHTNESS [BSUN]\n");
      for (i=0; i < *nval; i++)
      fprintf (fd, "    %6.4f          %.4e\n", scandx[i], scan[i]);
      }

   revon ();
   mvwaddstr (rwin, 7,0, buff);  
   wrefresh (rwin);

   goto L_Begin;
}
