/* pixave.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Average over box of size (nb*nb).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 22 July 1985
 * 16 February 1990: add ability to average floating point arrays.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include "/u/smm/lib/src/ip/new/iproc.ext"

float
pixave (image, x, y, nb)

struct IMAGE *image;		/* Pointer to IMAGE structure. 		*/
float	*x, *y;			/* Coordinates of scan point in image. 	*/
int	*nb;			/* size of box over which to average. */
{
   register unsigned char	*im;
   register unsigned short	*ims;
   register float *imf;
   register int ix, iy;
   register float sum, num;
   int	xx, yy;
   int	ix1, ix2, iy1, iy2;
   int	nb2;

   im  = (unsigned char *) image->addr;
   ims = (unsigned short *) image->addr;
   imf = (float *) image->addr;

/* Set up box limits. */

   nb2 = *nb / 2;
   xx = (int) *x;
   yy = (int) *y;
   ix1 = xx - nb2;
   ix2 = xx + nb2;
   iy1 = yy - nb2;
   iy2 = yy + nb2;

/* Clip the box to the edges of the image. */

   if (ix1 < 0) ix1 = 0;
   if (ix2 > (image->dx - 1))  ix2 = image->dx - 1;
   if (iy1 < 0) iy1 = 0;
   if (iy2 > (image->dy - 1))  iy2 = image->dy - 1;

/* Sum up intensities of values within the box. */

   sum = 0.;
   num = 0.;
   if (image->bppx == 1)	/* 1 byte per pixel. */
      {
      for (iy=iy1; iy<=iy2; iy++)
         {
         for (ix=ix1; ix<=ix2; ix++)
            {
            num++;
            sum += (float) (*(im + ix + iy*image->dx) & 0377);
            }
         }
      }

   else
   if (image->bppx == 2)	/* 2 bytes per pixel. */
      {
      for (iy=iy1; iy<=iy2; iy++)
         {
         for (ix=ix1; ix<=ix2; ix++)
            {
            num++;
            sum += (float) (*(ims + ix + iy*image->dx) & 0177777);
            }
         }
      }

   else
   if (image->bppx == 4)	/* 4 bytes per pixel. */
      {
      for (iy=iy1; iy<=iy2; iy++)
         {
         for (ix=ix1; ix<=ix2; ix++)
            {
            num++;
            sum += *(imf + ix + iy*image->dx);
            }
         }
      }

/* Average. */

   sum /= num;

   return (sum);
}
