/* remfc(rsun,nr,angle,na,data,ndata)
 *----------------------------------------------------------------------
 * Removes F-corona from array data.
 *----------------------------------------------------------------------
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 4 February 1983
 * 26 July 1984: conversion from 11/34 (f4p) to VAX 750 (C).
 * 11 December 1985: return -1 in case of error.
 * 17 March 1986: use F-corona function dependent upon angle as well as radius.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * rsun  = array of radial values (units of rsun). 
 * nr = # elements in rsun array (=1 for theta scan, =ndata for radial scan).
 * angle = array of theta values (units of degrees: position angle).
 * na = # elements in angle array (=1 for radial scan, =ndata for theta scan).
 * data  = array of intensity values (units of 10**-11 b/bsun).
 * ndata = # elements in array data.
 *----------------------------------------------------------------------
 */

#include <stdio.h>

/* ********************************************************************** */

remfc(rsun,nr,angle,na,data,ndata)

float	*rsun, *angle, *data;
int	nr, na, ndata;

{

extern	float	fcorona();
float	r, th;
int	i;

printf ("rsun:%f   angle:%f   data:%e\n", rsun[0], angle[0], data[0]);
printf ("nr:%d   na:%d   ndata:%d\n", nr, na, ndata);

/* Radial scans. */

   if (nr == ndata)
     {
      th = angle[0];
      for (i=0; i<ndata; i++)
        {
         r = rsun[i];
         data[i] -= fcorona(r,th);
        }
      return(0);
     }

/* Theta scans. */

   else

   if (nr == 1)
     {
      r = rsun[0];
      for (i=0; i<ndata; i++)  data[i] -= fcorona(r,angle[i]);
      return(0);
     }

   else

/* Error. */

     {
      fprintf(stderr,"# radii: %d < # data values: %d\n", nr, ndata);
      return(-1);
     }
}
