/* rscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Radial Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 21 February 1985
 * 4 April 1985: add box average feature.
 * 17 March 1986: add stray light/vignetting * F-corona removal features.
 * 22 January 1990: Munro calibration correction removed.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Modified by Kevin Cole  10/31/84 to include structure definitions
 * for image library iproc.a .
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "scan.ext"
#include "/u/smm/lib/src/ip/cp.h"
extern int QUAD4;		/* 4 quadrant composite  SMM images */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

rscan(icb, rmin, rmax, rinc, theta, scntyp, scan, scandx, ns)

struct IMAGE  *icb;     /* Pointer to image structure.			*/
float  rmin, rmax;     /* Solar radius limits for radial scan. 	*/
float  rinc;           /* Radial increment for radial scan. 		*/
float  theta;          /* Theta (degrees). 				*/
int    *scntyp;        /* Pointer to scan type (1=radial,2=line). 	*/
float  scan[];          /* Dependent axis scan array. 			*/
float  scandx[];        /* Independent axis scan array. 		*/
int    *ns;            /* Number of values in scan. 			*/
{
   extern float blint();
   extern float pixave();
   float x, y, r, th, dstsun, radinc, temp;
   register int  ir;
   int ix, iy, ixg, iyg, ierr;
   int nave, nave2;

/* Radial increment: set to 0.1 Rsun if 'rinc' <= 0. */

   radinc = rinc;
   if(rinc <= 0.)  radinc = 0.1;
   if(rmax < rmin)
      {
      temp = rmax;
      rmax = rmin;
      rmin = temp;
      }
   th = theta;

   dstsun = -radinc;

   /* ****** Radial Scan ****** */

   for (ir = 0; ir < MAXSCN; ir++)  *(scan + ir) = *(scandx + ir) = 0.0;
 

   /* ....... Radial scan loop ....... */

   *ns = 0;
   ir = -1;
   for (r = rmin; r <= rmax; r += radinc)
      {
      ir++;
 
      /* Coordinate transformation :  (theta,angle) --> (pixel,line). */

      if((ierr = coord(icb,&r,&th,&x,&y,1)) != 0)  return(ierr);

      /* Draw "X" to mark scan point on Grinnell display. */
 
      nave = scans->nave;
      nave2 = nave / 2;
      nres = 1;  			     /* initialize for low res images */
      if (icb->ires == 0) nres = 2;     /* if hi res image               */
      if (scans->GRN && ((strcmp(icb->type,"SMM")== 0) || QUAD4))
         {
         ixg = (int) (x/nres + 0.5) + ginitx();
         iyg = ginity() - (int) (y/nres + 0.5);
	 if (nave == 1)
            {
            if (((ixg > 62) && (ixg < 511)) && (iyg > 62) && (iyg < 511))
               wvec (ixg-1, iyg-1, ixg+1, iyg+1);
            if (((ixg > 62) && (ixg < 511)) && (iyg > 62) && (iyg < 511))
               wvec (ixg+1, iyg-1, ixg-1, iyg+1);
            }
         else
         if (nave > 1)
            {
            if (((ixg > 62) && (ixg < 511)) && (iyg > 62) && (iyg < 511))
               wvec (ixg - nave2, iyg - nave2, ixg + nave2, iyg + nave2);
            if (((ixg > 62) && (ixg < 511)) && (iyg > 62) && (iyg < 511))
               wvec (ixg + nave2, iyg - nave2, ixg - nave2, iyg + nave2);
            }
         }
 
/* Perform bilinear interpolation to compute scan point value from image. */
/* Use 4 nearest neighbors. */

      dstsun += radinc;
      if(*scntyp == 1)  *(scandx + ir) = (float) r;
      else
      if(*scntyp == 2)  *(scandx + ir) = (float) dstsun;

      ix = (int) (x + 0.5);
      iy = (int) (y + 0.5);

      /* convert to frame coordinates */

      x = x - icb->ip;  y = y - icb->ir;

      /* clip the coordinates */

      if((x >= 0.0) && (x <= icb->dx) && (y >= 0.0) && (y <= icb->dy))
         {
         if (nave == 1)  *(scan+ir)=(float) (blint(icb,&x,&y)*icb->scale);
         else
         if (nave > 1) *(scan+ir) = (float) pixave(icb,&x,&y,&nave)*icb->scale;
         }
      else
         *(scan+ir)= 0.0;

      /*
       * Divide scan values by 5 for equivalent second data type.
       * (numbers in file are 5 * equivalent second units.)
       */
 
      if(icb->dtype == 2) *(scan + ir) /= 5.;
      }
   *ns = ir + 1;

   /*
    * Remove stray light & vignetting.
    * Remove F-corona.
    */
 
   if(icb->dtype == 3)
      {
      if(scans->lsv == 1)
         {
         remstr (scandx, *ns, scan, *ns);
         remvig (scandx, *ns, scan, *ns);
         if(scans->lfc == 1)  remfc (scandx, *ns, &th, 1, scan, *ns);
         }
      }
   return(0);
}
