/*------------------------------------------------------------------------
 * utility.c	- A set of UNIX utilities for use by scan.c
 *
 * routines included here;
 *    readline(win,s,lim,fill)
 *    ztosmm()
 *-----------------------------------------------------------------------
 */
#include "scan.ext"

/*---------------------------------------------------------------------------
 * R E A D L I N E ()		- read a line of input from a terminal 
 *			operating in C-Break mode. This is a special version 
 *			for SCAN.C which checks for Ctrl-F and Ctrl-B.
 *---------------------------------------------------------------------------
 */

readline(win, s, lim, fill) 
WINDOW *win;			/* the window to get the line through       */
char s[];			/* the line read from the terminal          */
char fill;			/* the fill character for deleting backward */
int lim;			/* max characters to read -NOT IMPLEMENTED  */
{
    extern struct SCAN *scans;
    int i,c, row, col, start_row, start_col;

    i = 0; 
    /* Allow newline processing, but do not echo the user's input to the
     * screen, I'll handle that.
     */
    nl();  noecho();
	
    /* Preserve the initial screen location in this window.
     */
    getyx(win, start_row, start_col);

    while((c = wgetch(win)) != EOF && c != '\n')  
    {
	switch(c)  
	{
            case ESC: return(ESC);
	    case '?': return('?');

	    case 7:	/* Ctrl-G --- access the Grinnell */
		return(7);
		break;
	    case CTRL_F: 	/* forward one file */
		scans->index++;		/* point to next file    */
		if(scans->index > scans->maxindex) scans->index = 0;
		if((scans->index > -1) && (scans->index < 4) && scans->GRN)
		     disp(scans->index);
		return((int) CTRL_F);
		break;

	    case CTRL_B:    /* backward one file */
		scans->index--;		/* point to previous file */
		if(scans->index < 0) 
		    scans->index = scans->maxindex;
		if((scans->index > -1) && (scans->index < 4) && scans->GRN)
		    disp(scans->index);
		return((int) CTRL_B);
		break;	/* seems silly having a break after a return */

	     case ':':
		return((int) ':');
		break;

	    /* here I'll handle a delete or backspace (Hey, I'm easy )
 	     */

	    case '\177':	/* delete    (del) */
	    case '\010': 	/* backspace (del) */
		getyx(win, row,col);	    /* where are we on screen */
		if(--col < start_col)       /* decrement column and   */
		    col = start_col;	    /* clip to initial loc.   */
		wmove(win, row,col);	    /* move back one space    */
		wdelch(win);		    /* delete the character   */
		winsch(win, fill);	    /* insert fill character  */
		wrefresh(win);
		i--;
		break;

            default:    /* if we get here, then it is input to the buffer */
		if(i < 0) i = 0;	/* clip i	  */
                s[i++] = c;		/* add to buffer  */
                waddch(win, c);		/* echo the input */
                wrefresh(win);
	 	break;
        }
    }
    wmove(win,start_row, start_col);
    wrefresh(win);
    s[i] = '\0';		        /* close out the buffer   	*/
    nonl();			        /* turn newline processing off 	*/
    return(0);		                /* return A-OK			*/
}


/*-----------------------------------------------------------------
 * Z T O S M M   - Grinnell zoom cursor coords to Solar Max (448x448) coords
 *		  entry - (Global) Zcx, Zcy  == Zoom cursor coords.
 *		  exit  - x,y == Solar Max (448x448) coords.  
 *------------------------------------------------------------------
 */
ztosmm (image,x, y)
struct IMAGE *image;
float *x, *y;   
{
	rzcur ();			/* Read the zoom cursor. */
	*x = (float) ((Gr.Zcx - (ginitx () + image->ip)));
	*y = (float) (((ginity () - image->ir) - Gr.Zcy));
/*
 *	sprintf (wbuff, "Gr.Zcx: %d   Gr.Zcy: %d", Gr.Zcx, Gr.Zcy);
 *	disp_err (wbuff);
 */
}
