/* iscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * SMM C/P Radial and Theta scans.  Also does position measurement.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Version for SGI IRIS workstations.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <fmclient.h>
#include <curses.h>
#include <errno.h>
#include <signal.h>
#include "/usr/people/stanger/lib/include/gl_color.h"
#include "/usr/people/stanger/lib/include/ov_color.h"

/* If TRAPSIGNALS is defined, the bus errors, segmentation violation etc are
 * caught. When debugging this program it is better to remove the definition.
 */
#define TRAPSIGNALS
 
/*---------------------------------------------------------------------
 * MAIN:  SCAN          -Position measurement and image scanning
 *---------------------------------------------------------------------
 * Kevin Cole
 * Programmer II        Solar Maximum Mission           4/23/84
 * modified by Andrew L. Stanger
 * 21 February 1985: End plot title & labels with '$'.
 * 22 February 1985: Display each new image on Grinnell (if DSP=1) in loadfile.
 * 25 February 1985: Add option for linescan file name.
 * 3 April 1985: add box average feature for scans.
 * 18 March 1986: Add stray light / vignetting removal option.
 * 23 July 1986: Allow 4 images for Grinnell #1.
 * 4 December 1986: Allow scans w/o use of Grinnell.
 * 25 April 1989: Add slide_batch_on/off for svgrin library usage.
 * 22 January 1990: Remove Munro calibration correction.
 * 28 May 1991: adapt for SGI/IRIS.
 *---------------------------------------------------------------------
 */

#include "scan.h"
#include "/usr/people/stanger/lib/include/iproc.h"

/*
 * tlib.c contains the enhanced terminal library, revon(), hion() etc.
 */
#include "tlib.c"

WINDOW  *stdscr, *Comm, *Head, *newwin ();

struct IMAGE *imgst [9];
struct SCAN  *scnst;

float datay [MAXSCN], datax [MAXSCN];

char buff [84];
char fname [20];
char labx [40];
char laby [40];
char nampgm [16];
char term_name [64];
char title [64];
char wbuff [80], wbuff2 [80];

int  Max_Images;
int ibkg = 1;
int ixscale = 0;
int iyscale = 0;
int lymin, lymax;
int lfull = 1, lbottom = 1;
int nres;
int plotit = 1;			/* Plot flag. =1: plot on terminal screen. */
int plot_index = -1;

float pymin=0.0;	/* Fix y-scaling in plot. This is minimum value */
float pymax=5.0e-9;	/* Fix y-scaling in plot. This is maximum value */

/* ................................................................. */

main (argc, argv)

int argc;
char *argv [];
{
   char c;
   int row, col;
   static char rope [] =
   "-------scan-------------loadfiles-------------graphics------------options-------";

   scan_init ();        /* Initialize scan program.  */
   Max_Images = 4;	/* Maximum number of images that the program can use. */
   if ((scnst->DSP == 1) && (scnst->DSPDEV == 0))  Max_Images = 3;
   display (rope, 0);    /* Display command window.   */

   strcpy (nampgm, argv [0]);
   sprintf (fname, "%s.cgm", argv [0]);

   /*** Get input from the user.  ***/

   while ((c = wgetch (stdscr)) != 'q')  
      {
      switch (c)  
         {
         case 's':				/* SCAN command */
         case 'S': urope (rope, "scan", 'U');	/* Update the command line. */
                   display (rope, 1);
                   procscan (rope);
                   display (rope, 0);
                   urope (rope, "scan", 'L');
                   break;

         case 'l':			/* LOAD command */
         case 'L': urope (rope, "loadfiles", 'U');
                   display (rope, 1);
                   procfile (imgst, rope);
                   display (rope, 0);
                   urope (rope, "loadfiles", 'L');
                   break;

         case 'o':			/* OPTIONS command */
         case 'O': urope (rope, "options", 'U');
                   display (rope, 1);
                   procoptions ();
                   display (rope, 0);
                   urope (rope, "options", 'L');
                   break;

         case 'g':			/* GRAPHICS command */
         case 'G': urope (rope, "graphics", 'U');
                   display (rope, 1);
                   procgraphics (imgst [scnst->index]);
                   display (rope, 0);
                   urope (rope, "graphics", 'L');
                   break;

         case ':':  			/* enter command level */
                   (void) command ();
                   display (rope, 0);
                   break;

         case (char) CTRL_R:		/* refresh the screen */
                   wclear (stdscr);
                   wrefresh (stdscr);
                   wclear (Head);
                   wrefresh (Head);
                   wclear (Comm);
                   wrefresh (Comm);
                   display (rope, 0);
                   break;

         default:			/* display everything else */
                  getyx (stdscr, row, col);		/* get current y,x    */
		  (void) sprintf (wbuff, "%s", unctrl (c));/* in case of ctrl ch */
		  mvwaddstr (stdscr, 23, 40, wbuff);	/* add to window      */
		  wmove (stdscr, row, col);		/* return to y,x      */
		  cwrefresh (stdscr);			/* display it	      */
		  display (rope, 0);
                  break;

         }   /* end case */
      }    /* end while */

   die ();
}


/*------------------------------------------------------------------------------
 * P R O C G R A P H I C S ()  -- Process a request for the Graphics sub-menu.
 *------------------------------------------------------------------------------
 */
procgraphics (icb)

struct IMAGE *icb;
{
int row;
static char *Mlines[] = 
   {
   "----------------",
   "_grids__________",
   "_plots__________",
   "_output_________",
   "----------------"
   };
   if ((row = getmenu (Mlines, 3, 1, 45)) > 0)
      {
      switch (row)
         {
         case 1:			/* grids */
            grids (icb);
            procgraphics (icb);
            break;
         case 2:			/* plots */
            break;
         case 3:			/* output */
            break;
         }
      }
}


/*------------------------------------------------------------------------------
 * G R I D S ()  --  Process request for grids sub-sub menu.
 *------------------------------------------------------------------------------
 */
grids (icb)

struct IMAGE *icb;
{
   WINDOW *iowin, *newwin ();
   int  row, column;
   float p1, p2, p3, p4, p5, p6;
   static char *GridLines[] = 
      {
      ".-------------.",
      ":___polar_____:",
      ":___radial____:",
      ":___theta_____:",
      ":.............:"
      };

   if ((row = getmenu (GridLines, 3, 3, 45)) > 0)
      {
      switch (row)
         {
         case 1:	/*** polar ***/
            iowin   = newwin (11, 32, 4, 46);
            mvwaddstr (iowin, 0, 0, ".--------------------------.");
            mvwaddstr (iowin, 1, 0, ":--------POLAR GRID--------:");
            mvwaddstr (iowin, 2, 0, ":--------------------------:");
            mvwaddstr (iowin, 3, 0, ": *tmin = _________ [deg.] :");
            mvwaddstr (iowin, 4, 0, ": *tmax = _________ [deg.] :");
            mvwaddstr (iowin, 5, 0, ": tstep = _________ [deg.] :");
            mvwaddstr (iowin, 6, 0, ": *rmin = _________ [rsun] :");
            mvwaddstr (iowin, 7, 0, ": *rmax = _________ [rsun] :");
            mvwaddstr (iowin, 8, 0, ": rstep = _________ [rsun] :");
            mvwaddstr (iowin, 9, 0, ":__________________________:");

            /* The following two lines are a fudge factor. Curses refuses
             * to update this position and hence will not draw the space
             * in reverse background. If REVERSE BACKGROUND is deleted
             * from this program, these two lines can be removed.
             */
            mvwaddstr (iowin, 6, 2, " ");
            mvwaddstr (iowin, 7, 2, " ");
            mvwaddstr (iowin, 3, 2, " ");
            mvwaddstr (iowin, 4, 2, " ");

            wrefresh (iowin);
            row = 3; column = 10;

            wmove (iowin, row,column);
            wrefresh (iowin);
            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p1);

               wmove (iowin, ++row, column);  wrefresh (iowin);
               if (readline (iowin, buff, 80, '_') == ESC)
                  {
                  wclear (iowin);
                  wrefresh (iowin);
                  delwin (iowin);
                  break;
                  }
               else
                  (void) sscanf (buff, "%f", &p2);

                  wmove (iowin, ++row, column);  wrefresh (iowin);
                  if (readline (iowin, buff, 80, '_') == ESC)
                     {
                     wclear (iowin);
                     wrefresh (iowin);
                     delwin (iowin);
                     break;
                     }
                  else
                     (void) sscanf (buff, "%f", &p3);

                  wmove (iowin, ++row, column);  wrefresh (iowin);
                  if (readline (iowin, buff, 80, '_') == ESC)
                     {
                     wclear (iowin);
                     wrefresh (iowin);
                     delwin (iowin);
                     break;
                     }
                  else
	   	    (void) sscanf (buff, "%f", &p4);

                  wmove (iowin, ++row, column);  wrefresh (iowin);
                  if (readline (iowin, buff, 80, '_') == ESC)
                     {
                     wclear (iowin);
                     wrefresh (iowin);
                     delwin (iowin);
                     break;
                     }
                  else
                     (void) sscanf (buff, "%f", &p5);

                  wmove (iowin, ++row, column);  wrefresh (iowin);
                  if (readline (iowin, buff, 80, '_') == ESC)
                     {
                     wclear (iowin);
                     wrefresh (iowin);
                     delwin (iowin);
                     break;
                     }
                  else
                     (void) sscanf (buff, "%f", &p6);

                  sprintf (wbuff, "p1:%f   p2:%f   p3:%f", p1, p2, p3);
/*		    disp_err (wbuff); */
                  sprintf (wbuff, "p4:%f   p5:%f   p6:%f", p4, p5, p6);
/*		    disp_err (wbuff); */
                  if (scnst->DSP)
		     {
                     pgrid (icb, p4, p5, p6, p1, p2, p3);
		     }
                  wclear (iowin);
                  wrefresh (iowin);
                  delwin (iowin);
                  break;

         case 2:	/*** radial ***/
            iowin   = newwin (9, 32, 4, 46);
            mvwaddstr (iowin, 0, 0, ".--------------------------.");
            mvwaddstr (iowin, 1, 0, ":------ RADIAL GRID -------:");
            mvwaddstr (iowin, 2, 0, ":--------------------------:");
            mvwaddstr (iowin, 3, 0, ":  tmin = _________ [deg.] :");
            mvwaddstr (iowin, 4, 0, ":  tmax = _________ [deg.] :");
            mvwaddstr (iowin, 5, 0, ": tstep = _________ [deg.] :");
            mvwaddstr (iowin, 6, 0, ":__________________________:");
            wrefresh (iowin);
            row = 3;  column = 10;

            /*** Move to the initial row and column position (tmin) ***/

            wmove (iowin, row, column);
            wrefresh (iowin);

            /*** Get a line of input from the user. If escape is hit, exit. ***/

            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p1);

            /*** Move to the next row and column for input of tmax. ***/

            wmove (iowin, ++row, column);  wrefresh (iowin);
            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p2);

            wmove (iowin, ++row, column);  wrefresh (iowin);
            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p3);
	     
            /*** Draw the grid, since we now have all the parameters. ***/

            if (scnst->DSP)
	       {
               rgrid (icb, p1, p2, p3);
	       }
            wclear (iowin);
            wrefresh (iowin);
            delwin (iowin);
            break;

         case 3:	/*** theta ***/
            iowin   = newwin (9, 32, 4, 46);
            mvwaddstr (iowin, 0, 0, ".--------------------------.");
            mvwaddstr (iowin, 1, 0, ":------ THETA  GRID -------:");
            mvwaddstr (iowin, 2, 0, ":--------------------------:");
            mvwaddstr (iowin, 3, 0, ":  rmin = _________ [Rsun] :");
            mvwaddstr (iowin, 4, 0, ":  rmax = _________ [Rsun] :");
            mvwaddstr (iowin, 5, 0, ": rstep = _________ [Rsun] :");
            mvwaddstr (iowin, 6, 0, ":__________________________:");
            wrefresh (iowin);
            row = 3;  column = 10;

            /*** Move to the initial row and column position (tmin) ***/

            wmove (iowin, row, column);
            wrefresh (iowin);

            /*** Get a line of input from the user, if escape is hit, exit. ***/

            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p1);

            /*** Move to the next row and column for input of tmax. ***/

            wmove (iowin, ++row, column);  wrefresh (iowin);
            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p2);

            wmove (iowin, ++row, column);  wrefresh (iowin);
            if (readline (iowin, buff, 80, '_') == ESC)
               {
               wclear (iowin);
               wrefresh (iowin);
               delwin (iowin);
               break;
               }
            else
               (void) sscanf (buff, "%f", &p3);
	     
            /*** Draw the grid, since we now have all the parameters. ***/

            if (scnst->DSP)
	       {
               tgrid (icb, p1, p2, p3);
	       }
            wclear (iowin);
            wrefresh (iowin);
            delwin (iowin);
            break;
         }
      }
}



/*------------------------------------------------------------------------------
 * P R O C O P T I O N S ()  --  Process program options. (NOT IMPLEMENTED!)
 *------------------------------------------------------------------------------
 */
procoptions ()
{
   char cmd;
   int row;
   static char *Mlines[] = 
      {
      "---------------------",
      "_Display  #_______0__",
      "_grid color_____red__",
      "_rscancolor____green_",
      "_tscancolor____blue__",
      "---------------------"
      };
   if ((row = getmenu (Mlines, 4, 1, 56)) > 0)
      {
      switch (row)
         {
         case 1:					/* display */
	         while ((cmd = wgetch (stdscr)) == CR)
                    {
                    }
		    break;
         case 2:					 /* image_format */
		    break;
	 }
      }
}



/*------------------------------------------------------------------------------
 * P R O C F I L E ()  --  Process a request to load a file.
 *------------------------------------------------------------------------------
 */
procfile (imgst, rope)

struct IMAGE *imgst[];
char rope [];
{
   int row, nval;

   /* NOTE: It was my intention to add some additional functions to this
    *       menu, for example, 'load and diff' would load two images and
    *       display the difference.
    */

   static char *Mlines[] = 
      {
      "----------------------",
      "_load_and_display_____",
      "----------------------"
      };

   if ((row = getmenu (Mlines, 1, 1, 24)) > 0)
      {
      switch (row)
         {
         case 1:			/* call `load file' routine */
                 loadfile (imgst, rope);
                 break;
         case 2:			/* call `load and diff' routine */
                 break;
         }
      }
}


/*----------------------------------------------------------------------------
 * L O A D F I L E ()		- Get filename from the user, open them and
 *				if image display option has been selected I'll
 *				even display them for you.
 *----------------------------------------------------------------------------
 */

loadfile (imgst, rope)

struct IMAGE *imgst [];
char rope [];
{
   WINDOW *swin, *newwin ();
   extern int Max_Images;
   char filename [64], listfile [64];
   int  i, count;
   FILE *LIST;

   /*** Create a new window. ***/

   swin  = newwin (9, 54, 6, 30);

   mvwaddstr (swin, 0, 0, "-CHAN--------FILENAME----------------------------");
   mvwaddstr (swin, 1, 0, "_0_______________________________________________");
   mvwaddstr (swin, 2, 0, "_1_______________________________________________");
   mvwaddstr (swin, 3, 0, "_2_______________________________________________");
   mvwaddstr (swin, 4, 0, "_3_______________________________________________");
   mvwaddstr (swin, 5, 0, "|||||||||||||||||||||||||||||||||||||||||||||||||");
   mvwaddstr (swin, 6, 0, ".................................................");
   mvwaddstr (swin, 7, 0, ".................................................");
   mvwaddstr (swin, 8, 0, "-------------------------------------------------");
   wmove (swin, 1, 3);
   wrefresh (swin);

   /* If the scan index is greater than 1, then at least one file has been 
    * previously opened.  We'll display that filename on the loadfiles menu.
    */
   count = 0;
   if (scnst->index > -1)
      {
      i = 1;
      while (count <= scnst->maxindex)
         {
         mvwaddstr (swin, i+count, 3, imgst [count]->name);
         wrefresh (swin);
         count++;
         }
      }
   display (rope, 2);

   /* This loop will continue until the user types an escape character or until
    * an error occurs when trying to open the image file.
    */
   while (1)
      {
      revon ();
      mvwaddstr (swin, 6, 0,"Enter Filename:.................................");
      wrefresh (swin);

      wmove (swin, 6, 16);
      wrefresh (swin);

      if (((char) readline (swin, buff, 64, '.')) == ESC) 
         {
         werase (swin);
         wrefresh (swin);
         scnst->maxindex = scnst->index;
         delwin (swin);
         return (0);
         }
      (void) sscanf (buff, "%s", filename);

      if (*buff == '@')			/* Read in list of files. */
	 {
	 sscanf (buff, "%*c%s", listfile);
	 if ((LIST = fopen (listfile, "r")) == NULL)
	    {
	    scnst->maxindex = scnst->index;
	    (void) sprintf (buff, "UNABLE TO OPEN %s ,return. ", listfile);
	    mvwaddstr (swin, 7, 0, buff);
	    wrefresh (swin);
	    wclear (swin);
	    wrefresh (swin);
	    delwin (swin);
	    return (-1);
	    }
         else
  	    while (fscanf (LIST, "%s", filename) != EOF)
	       if (readfile (filename, imgst, swin) == -1)  return (-1);
         (void) fclose (LIST);
         }
      else
         if (readfile (filename, imgst, swin) == -1)  return (-1);
      }
}

/*----------------------------------------------------------------------------
 * R E A D F I L E ()	- Read image from disk file and store in memory array.
 *                        If image display option has been selected, the image
 *			  will be displayed as well.
 *----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 21 September 1990: Isolated as a C function by Mike Hoswell.
 *----------------------------------------------------------------------------
 */

readfile ( filename, imgst, swin )

char filename [];
struct IMAGE *imgst [];
WINDOW *swin;
{
   hion ();
      (void) sprintf (buff, "READING FILE: %s    ", filename);
      mvwaddstr (swin, 6, 0, buff);
      wrefresh (swin);
   hioff ();

   /* Return cursor to beginning of next line.  This is to allow space for 
    * output which may occur from smm_resc() when the image is a high 
    * resolution frame.
    */
   wmove (swin, 7, 0);
   wrefresh (swin);

   /*** Increment the file structure array index and open the new file. ***/

   scnst->index++;
   if (scnst->index > (Max_Images-1)) scnst->index = 0;
   (void) sprintf (buff, "scnst->index: %d    ", scnst->index);
   mvwaddstr (swin, 8, 0, buff);
   wrefresh (swin);
   wmove (swin, 7, 0);
   wrefresh (swin);
   imgst [scnst->index] = NULL;

   if ((imgst [scnst->index] = make (filename, "SMM")) == NULL) 
      {
      scnst->index--;		/* Open failed; reset index back 1. */
      scnst->maxindex = scnst->index;
      (void) sprintf (buff, "UNABLE TO OPEN %s <return> ", filename);
      mvwaddstr (swin, 7,0, buff);
      wrefresh (swin);
      sleep (5);
      wclear (swin);
      wrefresh (swin);
      delwin (swin);
      return (-1);
      }
   sprintf (buff, "After make> filename: %s  index: %d  bppx: %d  dx: %d",
            filename, scnst->index, imgst [scnst->index]->bppx,
            imgst [scnst->index]->dx);
   disp_err (buff);
   nres = 1;			/* Low resolution image. */
   if (imgst [scnst->index]->ires == 0) nres = 2;/* High resolution image. */
   scnst->OPENFLAG = 1;		/* An image file is now available    */
/*
 *	if (strcmp (imgst [scnst->index]->type, "QUAD4") == 0)
 *		{
 *		SMM = 0;
 *		QUAD4 = 1;
 *		ATM = 0;
 *		}
 *	display (rope, 1);
 */

   /* If the image display option is active,
    * then display this image with the appropriate grid.
    */
   if (scnst->DSP)
      {
      if (scnst->ERASEOVL == 1) gl_wclear (ov_black);	/* Erase overlay. */
      drawmode (NORMALDRAW);		/* Set normal mode for drawing. */

      sprintf (buff, "Before dspimg> index: %d  bppx: %d  dx: %d",
            scnst->index, imgst [scnst->index]->bppx, imgst [scnst->index]->dx);
      disp_err (buff);
      dspimg (imgst [scnst->index]);
      labelsmm (imgst [scnst->index], lfull, lbottom, gl_grey, gl_green);
      annotsmm (imgst [scnst->index], scnst->index);
      annotsmm (imgst [scnst->index], lbottom,
                gl_grey, gl_red, gl_yellow, gl_green);

      /*** Overlay set-up. ***/

      overlay (2);		/* Establish two bitplanes for overlay. */
      gconfig ();		/* Make it so. */

      drawmode (OVERDRAW);		/* Set overlay mode for drawing. */
      mapcolor (ov_black,   0,   0,   0);	/* color 0 -> black. */
      mapcolor (ov_red,   255,   0,   0);	/* color 0 -> red.   */
      mapcolor (ov_green,   0, 255,   0);	/* color 0 -> green. */
      mapcolor (ov_blue,    0,   0, 255);	/* color 0 -> blue.  */
      if (scnst->ERASEOVL == 1) gl_wclear (ov_black);	/* Erase overlay. */

      (void) sprintf (buff, "display image...scnst->index: %d ", scnst->index);
      mvwaddstr (swin, 7,0, buff);
      wrefresh (swin);

      /*** if this is a (896x896) image, then average to (448x448). ***/

/*      if (imgst [scnst->index]->ires == 0)  smm_avg (imgst [scnst->index]); */

      /* grid (imgst [scnst->index], "SMM"); */
      }
   revon ();
   mvwaddstr (swin, 1+scnst->index, 3, filename);
   wrefresh (swin);
   mvwaddstr (swin, 6, 0, ".................................................");
/*
 *   mvwaddstr (swin, 7, 0, ".................................................");
 */
   wrefresh (swin);

}

/*------------------------------------------------------------------------------
 * P R O C S C A N ()  -  Process a request for image scanning.
 *------------------------------------------------------------------------------
 */
procscan (rope)

char *rope;
{
   extern struct SCAN *scnst;
   extern float datay[], datax[];
   int row, nval;
   static char *Mlines[] = 
      {
      "----------------",
      "_radial_________",
      "_theta__________",
      "_line___________",
      "_point__________",
      "----------------"
      };

   if (imgst [scnst->index]->dtype == 1)
      (void) sprintf (laby,"Intensity  (DN)");
   else
   if (imgst [scnst->index]->dtype == 2)
      (void) sprintf (laby,"Equivalent Seconds Exposure");
   else
   if (imgst [scnst->index]->dtype == 3)
      {
      (void) sprintf (laby,"B [total]  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 4)
      {
      (void) sprintf (laby,"density  (elec/cm**2)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 5)
      {
      (void) sprintf (laby,"Bcme - Bbkg difference  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 6)
      {
      (void) sprintf (laby,"B [K-corona]  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 7)
      {
      (void) sprintf (laby,"(Bcme - Bbkg) / Bk  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 8)
      {
      (void) sprintf (laby,"(Bcme - Bbkg) / Bbkg  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 9)
      {
      (void) sprintf (laby,"polarization fraction");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 10)
      {
      (void) sprintf (laby,"pB  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 11)
      {
      (void) sprintf (laby,"B [radial+tangential]  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 12)
      {
      (void) sprintf (laby,"B [tangential]  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
   else
   if (imgst [scnst->index]->dtype == 13)
      {
      (void) sprintf (laby,"B [radial]  (B/Bsun)");
      if (scnst->lsv)  strcat (laby," (-sv)");
      if (scnst->lfc)  strcat (laby," (-fc))");
      }
/*   strcat (laby, "$"); */
   sprintf (buff, "bppx:%d   pixsr:%f   scale:%g   dtype:%d   nave:%d",
            imgst [scnst->index]->bppx, imgst [scnst->index]->pixsr,
            imgst [scnst->index]->scale, imgst [scnst->index]->dtype,
	    scnst->nave);
/*   disp_err (buff); */
		
   if ((row = getmenu (Mlines, 4, 1, 0)) > 0)
      {
      switch (row)
         {
         case 1:	/* Execute radial scan routine. */
            if (scnst->ERASEOVL && scnst->DSP)
               gl_wclear (ov_black);			/* EraSE overlay. */

            if (rtscan (imgst, datay, datax, &nval, row+2, 'R', rope) == 0)
               {
               if (scnst->GRAPH)
                  {
                  (void) sprintf (title, "%s Radial Scan %6.2f deg.",
				  imgst [scnst->index]->name, scnst->rangle);
		  strcpy (labx, "Radius [Rsun]");
                  blinkon ();
	          mvwaddstr (stdscr, 6, 15, "P L O T T I N G !!");
                  wrefresh (stdscr);
                  (void) strcpy (wbuff,  imgst [scnst->index]->name);
                  (void) strcat (wbuff, ".cgm");
		  gets (buff);
/*
                  xyplot (datax, datay, nval, wbuff, title, labx, laby, ixscale, iyscale, ibkg);
 */
                  blinkoff ();
                  display (rope, 1);
                  }
               }
            procscan (rope);
            break;

         case 2: /* Execute theta scan routine. */

            if (scnst->ERASEOVL && scnst->DSP)
               gl_wclear (ov_black);			/* EraSE overlay. */

            if (rtscan (imgst, datay, datax, &nval, row+2, 'T', rope) == 0)
               {
               if (scnst->GRAPH)
                  {
                  (void) sprintf (title, "%s Theta Scan %4.2f Rsun",
				  imgst [scnst->index]->name, scnst->thradius);
                  strcpy (labx, "Position Angle [degrees]");
                  blinkon ();
                  mvwaddstr (stdscr, 6, 15, "P L O T T I N G !!");
                  wrefresh (stdscr);
                  (void) strcpy (wbuff,  imgst [scnst->index]->name);
                  (void) strcat (wbuff, ".cgm");
		  gets (buff);
/*
                  xyplot (datax, datay, nval, wbuff, title, labx, laby, ixscale, iyscale, ibkg);
 */
                  blinkoff ();
                  display (rope, 1);
                  }
               }
            procscan (rope);
            break;

         case 3: /* Perform line scan. */
            if (linescan (imgst, datay, datax, &nval, row+3))
               fprintf (stderr,"ERROR IN LINE SCAN\n");
            procscan (rope);
            break;

         case 4: /* Perform position measurement. */ 

            if (scnst->DSP)
	       {
               if (scnst->ERASEOVL)  gl_wclear (ov_black);   /* Erase overlay.*/
               point (imgst, row+2);
	       }
            else
               point (imgst, row+2);
            procscan (rope);
            break; 
         }
      }
}


/*------------------------------------------------------------------------------
 * S C A N _ I N I T ()	--  Do any initialization required for the scan program.
 *------------------------------------------------------------------------------
 * 17 March 1986: Add options to remove stray light/vignetting and F-corona.
 */
scan_init () 
{
   extern struct SCAN *scnst;
   FILE *fd, *fopen();
   int chsize;
   int clip = 0;
   int row = 6;
   int numave;
   char keybuff [20], op [10], name [64];

   revoff ();		/* initialize to reverse background off */
   winit ();		/* initialize curses windows package	*/
   gMINROW = 127;
   gMAXROW = 1023;
   gMINCOL = 127;
   gMAXCOL = 1023;
   /*  gMINROW = 63;  */
   /*  gMINCOL = 63;  */
   /*  gMAXROW = 510; */
   /*  gMAXCOL = 510; */

   /* Allocate space for the scan record structure, the radius/theta scan file
    * name, and the position measurement file name.
    */
   scnst    = (struct SCAN *) malloc (sizeof (struct SCAN));
   scnst->rtfile   = (char *) malloc (64);
   scnst->rttitle  = (char *) malloc (81);
   scnst->posfile  = (char *) malloc (64);
   scnst->linefile = (char *) malloc (64);

   /*** Set default values. ***/

   strcpy (scnst->rtfile,   "rtscan.dat");
   strcpy (scnst->posfile,  "pscan.dat");
   strcpy (scnst->linefile, "linescan.dat");
   strcpy (scnst->rttitle,  "Radial/Theta scans");
   scnst->OPENFLAG = 0;		/* No image file is available	      */
   scnst->DSP = 0;		/* Display is off 		      */
   scnst->DSPDEV = 0;		/* Device is Display 0 	      */
   scnst->DSPCHA = 0;		/* Default channel is 0		      */
   scnst->DSPOVL = 8;		/* Default overlay is 8 (blue).       */
   scnst->GRAPH = 0;		/* Graphics are off 		      */
   scnst->GRIDOVL = 9;		/* Overlay color for grids	      */
   scnst->RSCANOVL = 9;		/* Overlay color for radial scan      */
   scnst->TSCANOVL = 9;		/* Overlay color for theta scans      */
   scnst->ERASEOVL = 0;		/* Erase overlays: true if = 1.       */
   scnst->nave = 1;		/* box size (nave*nave) for average.  */
   scnst->lsv = 0;		/* No stray light/vignetting removal. */
   scnst->lfc = 0;		/* No F-corona removal. 	      */
   scnst->rmin = 1.5;		/* Default min. for radial scan       */
   scnst->rmax = 3.0;		/* Default max. for radial scan       */
   scnst->rstep = 0.1;		/* Default step for radial scan       */
   scnst->rangle = 90;		/* Default angle for rad. scan        */
   scnst->thmin = 45.0;		/* Default min. for theta scan        */
   scnst->thmax = 90.0;		/* Default max. for theta scan        */
   scnst->thstep = 1.0;		/* Default step for theta scan        */
   scnst->thradius = 2.0;	/* Default radius for theta scan      */
   scnst->index = -1;

   /*** Read user programming.  ***/

   if ((fd = fopen ("config.scan", "r")) == NULL)
      {
      /* If the open fails on 'config.scan',
       * all parameters are have default values.
       */
      }
   else
      {
      while (fgetline (fd, buff) != EOF)
         {
         mvwaddstr (stdscr, row++, 5, buff);
         wrefresh (stdscr);
         (void) sscanf (buff, "%s %s %s", keybuff, op, name);
         squeeze (name, '\n');

         /*** Set up filename and/or title for radial/theta scans. ***/

         if (strcmp (keybuff, "rtscan") == 0)
            {
            if (strcmp (op, "=") == 0)
            strcpy (scnst->rtfile, name);
            if (strcmp (op, "title:") == 0);
            strcpy (scnst->rttitle, &buff[14]);
            }

         /*** Set up filename for line scan. ***/

         if (strcmp (keybuff, "linescan") == 0)
         strcpy (scnst->linefile, name);

         /*** Set up filename for position measurement. ***/

         if (strcmp (keybuff, "position") == 0)
         strcpy (scnst->posfile, name);

         /*** Set appropriate Display device [0,1]. ***/

         if (strncmp (keybuff, "display", 4) == 0)
            {
            if (strcmp (name, "off") == 0) 
               scnst->DSP = 0;
            else
               scnst->DSP = 1;

            if (strcmp (name, "1") == 0)
               scnst->DSPDEV = 1;
            if (strcmp (name, "0") == 0)
               scnst->DSPDEV = 0;
            }
	    
            /*** Setup overlay (color) to use in drawing grids. ***/

         if (strcmp (keybuff, "gridcolor") == 0)
            {
            (void) sscanf (name, "%d", &scnst->GRIDOVL);
            if (strncmp (name, "blue", 1) == 0)
               scnst->GRIDOVL = 8;
            if (strncmp (name, "green", 2) == 0)
               scnst->GRIDOVL = 9; 
            if (strncmp (name, "red", 1) == 0)
               scnst->GRIDOVL  = 10;
            if (strncmp (name, "white", 2) == 0)
               scnst->GRIDOVL = 11;
            }

         /*** Setup overlay (color) to use in drawing radial scans. ***/

         if (strcmp (keybuff, "rscancolor") == 0)
            {
            (void) sscanf (name, "%d", &scnst->RSCANOVL);
            if (strncmp (name, "blue", 1) == 0)
               scnst->RSCANOVL = 8;
            if (strncmp (name, "green", 2) == 0)
               scnst->RSCANOVL = 9; 
            if (strncmp (name, "red", 1) == 0)
               scnst->RSCANOVL  = 10;
            if (strncmp (name, "white", 2) == 0)
               scnst->RSCANOVL = 11;
            }

         /*** Setup overlay (color) to use in drawing theta scans. ***/

         if (strcmp (keybuff, "tscancolor") == 0)
            {
            (void) sscanf (name,"%d", &scnst->TSCANOVL);
            if (strncmp (name, "blue", 1) == 0)
               scnst->TSCANOVL = 8;
            if (strncmp (name, "green", 2) == 0)
               scnst->TSCANOVL = 9; 
            if (strncmp (name, "red", 1) == 0)
               scnst->TSCANOVL  = 10;
            if (strncmp (name, "white", 2) == 0)
               scnst->TSCANOVL = 11;
            }

         /*** Setup the default overlay (color). ***/

         if (strcmp (keybuff, "overlay") == 0)
            {
            (void) sscanf (name, "%d", &scnst->DSPOVL);
            if (strncmp (name, "blue", 1) == 0)
               scnst->DSPOVL = 8;
            if (strncmp (name, "green", 2) == 0)
               scnst->DSPOVL = 9; 
            if (strncmp (name, "red", 1) == 0)
               scnst->DSPOVL = 10;
            if (strncmp (name, "white", 2) == 0)
               scnst->DSPOVL = 11;
            }


         /*** Erase overlay flag. ***/

         if (strncmp (keybuff, "eraseov", 5) == 0)
            {
            if (strcmp (name, "on") == 0) 
               scnst->ERASEOVL = 1;
            else
               scnst->ERASEOVL = 0;
            }

         /*** Process 'draw-graphs' switch. ***/

         if (strcmp (keybuff, "graphs") == 0)
            {
            if (strcmp (name, "off") == 0)
               scnst->GRAPH = 0;
            else
               scnst->GRAPH = 1;
            }

         /*** Process 'box average' switch. ***/

         if (strcmp (keybuff, "average") == 0)
            {
            sscanf (name, "%d", &numave);
            if (numave > 1 && numave < 113)  scnst->nave = numave;
            }

         /*** Process 'stray light / vignetting' switch. ***/

         if (strcmp (keybuff, "strayvignet") == 0)
            {
            if (strcmp(name, "remove") == 0)  scnst->lsv = 1;
            }

         /*** Process 'F-corona' flag. ***/

         if (strcmp (keybuff, "fcorona") == 0)
            {
            if (strcmp(name, "remove") == 0)  scnst->lfc = 1;
            }

         /*** `ymin' option: set maximum value for y-axis in plots. ***/

         if (strcmp (keybuff, "ymin") == 0)
            {
            if (strcmp (name, "off") == 0)  lymin = 0;
	    else
	       {
	       lymin = 1;
               sscanf (name, "%g", &pymin);
	       }
            }

         /*** `ymax' option: set maximum value for y-axis in plots. ***/

         if (strcmp (keybuff, "ymax") == 0)
            {
            if (strcmp (name, "off") == 0)  lymax = 0;
	    else
	       {
	       lymax = 1;
               sscanf (name, "%g", &pymax);
	       }
            }

         /*** Initialize image type. ***/

         if (strcmp(keybuff, "image") == 0)
            {	
            if (strcmp(name, "SMM") == 0)
               {
               SMM = 1;
               ATM = 0;
               QUAD4 = 0;
               }
            if (strcmp(name, "ATM") == 0)
               {
               SMM = 0;
               ATM = 1;
               QUAD4 = 0;
               }
            }
         }
      (void) fclose(fd);
      }

   /*
    * Initialize the image display device if the image display option is active.
    */

   if (scnst->DSP) 
      {
      ginitscr (scnst->DSPDEV,scnst->DSPCHA,scnst->DSPOVL);
      gsetdef (scnst->DSPDEV, scnst->DSPCHA, scnst->DSPOVL, 63, 510, 448, 448); 
      gerase ();		/* erase the default screen   */
      gl_wclear (ov_black);	/* erase all overlay channels */
      }

   if (scnst->GRAPH)
      {
/*      gksini_ (&plotit, fname); */
/*
      opngks_ ();
 */
/*      gsclip_ (&clip); */
      }

   /* Set attribute byte for various terminals. 
    * NOTE: Only a GraphOn terminal is currently supported by the enhanced
    *       terminal library.
    */
   if (isgraphon ())
      do_graphon ();
   else
      do_dumb ();
}


/*------------------------------------------------------------------------------
 * W I N I T ()   --  Initialize the windows 'curses' package.
 *------------------------------------------------------------------------------
 */
winit ()                         /* Initialize windows package. */
{
   WINDOW *newwin ();
   int die ();

   if (initscr () == NULL) 	/* Initialize the curses package. */
      {
      disp_err ("UNABLE TO INITIALIZE WINDOWS PACKAGE");
      die ();
      };
   nonl (); 		/* Do not allow new line (nl) processing. */
   noecho ();		/* Do not echo user input to the screen. */

   /* Set up user screen windows. Note that most windows are allocated 
    * dynamically by the individual functions.
    */
   Comm    = newwin (2, 80, 22, 0);
   Head    = newwin (2, 80, 0,  0);

   crmode ();		    /* Operate in c-break mode. */

   /*** Always trap Control-C. ***/

   (void) signal (SIGINT, die);  /* Trap ctrl-c interrupt to routine die().*/

#ifdef TRAPSIGNALS
   (void) signal (SIGBUS, die);  /* Trap bus error. */
   (void) signal (SIGSEGV, die); /* Trap segmentation violation. */
#endif TRAPSIGNALS
}


/*------------------------------------------------------------------------------
 * D I E ()  --  This is the normal exit routine; where good programs go to die.
 *------------------------------------------------------------------------------
 */
die ()    
{
   extern struct SCAN *scnst;
   extern char wbuff[], wbuff2[];
   FILE *pfd, *popen();

   (void) signal (SIGINT,  SIG_IGN);
#ifdef TRAPSIGNALS
   (void) signal (SIGBUS,  SIG_IGN);
   (void) signal (SIGSEGV, SIG_IGN);
#endif TRAPSIGNALS

   sprintf (buff, "die> index: %d  bppx: %d  dx: %d",
            scnst->index, imgst [scnst->index]->bppx,
            imgst [scnst->index]->dx);
   disp_err (buff);

   /*** Clean up overlay bitplanes. ***/
   
   drawmode (OVERDRAW);
   gl_wclear (ov_black);

   drawmode (NORMALDRAW);
   onemap ();			/* Set single-map mode for color table. */
   gconfig ();
   pixmode (PM_ADD24, 0);	/* i.e., effectively use color map #0.*/

   gexit ();			/* Terminate SGI GL graphics. */

   revoff ();			/* Make sure reverse background is off. */
   wclear (stdscr);		/* Clear the screen. */
   wrefresh (stdscr);
   stopwin ();			/* End windows package. */
   printf ("\n");

   printf ("\0332");		/* ESC 2 puts terminal in alphanumeric mode. */
	
/*
   if (scnst->GRAPH)  (void) clsgks_ ();
 */
   (void) unlink (fname);
   exit (0);
}


/*------------------------------------------------------------------------------
 * S T O P W I N ()	- Shut off the curses windows package.
 *------------------------------------------------------------------------------
 */
stopwin ()
{
   noraw ();		/* End raw terminal mode. */
   endwin ();		/* End windows package. */
}


/*
 *------------------------------------------------------------------------------
 * D I S P L A Y ( )
 *------------------------------------------------------------------------------
 * Handles the dynamics of the menu line at the top of the screen and the
 * status line at the base of the screen.
 *------------------------------------------------------------------------------
 */
display (rope, sw)

char *rope;
int sw;
{
   static char SPB[]   = "|---Spacebar to select---|";
   static char FCH[]   = "|---s,l,g,o  to select---|";
   static char EXESC[] = "|---ESC to exit menu---|";
   static char FILECH[]= "|----ENTER FILENAME----|";
   static char SCOPT[] = "|---ENTER OR CTRL+G----|";
   static char QT[]  = "|---q quit---|";
   static char HLP[]  = "|--------|";
   static char ATYP[] = "|--ATM--|";
   static char STYP[] = "|--SMM--|";
   static char QTYP[] = "|-QUAD4-|";
   int lSPB, lQT, lHLP, lSTYP, lQTYP, lEXESC;

   lSPB = strlen (SPB);
   lQT = strlen (QT);
   lHLP = strlen (HLP);
   lSTYP = strlen (STYP);
   lQTYP = strlen (QTYP);
   lEXESC = strlen (EXESC);
   disp_head (rope);

   revon ();
   if (SMM)
      mvwaddstr (Comm, 1, 0, STYP);
   if (ATM)
      mvwaddstr (Comm, 1, 0, ATYP);
   if (QUAD4)
      mvwaddstr (Comm, 1, 0, QTYP);

   /* Different points in the program will pass the value of 'sw'. This
    * is used to change the status line at the base of the screen.
    */
   if (sw == 4)
      mvwaddstr (Comm, 1, lSTYP-1, EXESC);
   if (sw == 3)
      mvwaddstr (Comm, 1, lSTYP-1, SCOPT);
   if (sw == 2)
      mvwaddstr (Comm, 1, lSTYP-1, FILECH);
   if (sw == 1)
      mvwaddstr (Comm, 1, lSTYP-1, SPB);
   if (sw == 0)
      mvwaddstr (Comm, 1, lSTYP-1, FCH);
   mvwaddstr (Comm, 1, lSTYP+lSPB-2, QT);
   mvwaddstr (Comm, 1, lSTYP+lSPB+lQT-3, HLP);
   mvwaddstr (Comm, 1, lSTYP+lSPB+lQT+lHLP-4, EXESC);
   wrefresh (Comm);
   revoff ();
}


/*-----------------------------------------------------------------------------
 * W D I S P L ()   --  Display menu lines.
 *-----------------------------------------------------------------------------
 */
wdispl (win, lines, nlines)

WINDOW *win;
char *lines[];
int nlines;
{
   int i;
   extern char *lower();

   /*** Display remaining lines. ***/

   for (i = 0; i < nlines; i++)
   mvwaddstr (win, i,0, lower(lines[i]));
   cwrefresh (win);
}


/*
 *------------------------------------------------------------------------------
 * G E T M E N U ( )
 *------------------------------------------------------------------------------
 * This routine handles the pull down menus. It allows the user to make a
 * selection via the space bar and returns the row position of that selection.
 *------------------------------------------------------------------------------
 * input:
 *	Mlines[]	- the menu lines.
 *	nlines		- number of menu items.
 *	initrow		- initial row position of the menu.
 *	initcol		- initial column position of the menu.
 *------------------------------------------------------------------------------
 */
getmenu (Mlines, nlines, initrow, initcol)

char *Mlines[];				/* The menu lines. */
int nlines, initrow, initcol;
{
   int ncols, row, i;
   char cmd, faster[40];
   extern char tolower (), *upper (), *lower ();
   WINDOW *pwin;
	
   ncols = strlen (Mlines[1]);
   pwin = newwin (nlines+2, ncols, initrow, initcol);
   leaveok (pwin, 1);
   wpaint (pwin);
   wdispl (pwin, Mlines, nlines+2);
   mvcur (0, 0, 23, 79); 

   row = 0;
   while ((cmd = wgetch (stdscr)) == ' ')
      {
      if (++row > nlines) row = 1;
      if (row == 1)
         {
         hion ();
         mvwaddstr (pwin,row,0, upper(Mlines[row]));
         cwrefresh (pwin); 
         hioff ();  revon ();
         if (nlines != 1)
         mvwaddstr (pwin, nlines, 0, lower (Mlines[nlines]));
         wmove (stdscr, 1,0);  cwrefresh (stdscr);  wrefresh (pwin);
         mvcur (0, 0, 23, 79); 
         }
      else
         {
         hion ();
         mvwaddstr (pwin, row, 0, upper (Mlines[row]));
         cwrefresh (pwin);
         hioff ();  revon ();
         mvwaddstr (pwin, row-1, 0, lower (Mlines[row-1]));
         wmove (stdscr,1,0);  cwrefresh (stdscr);  wrefresh (pwin);
         mvcur (0, 0, 23, 79); 
         }
      }
   for (i = 0; i < nlines; i++) 
      if((char) tolower (cmd) == faster[i]) row = i + 1;

   wclear (pwin);
   cwrefresh (pwin);
   revoff ();
   delwin (pwin);
   revon ();
   wrefresh (stdscr);

   if (cmd == ESC)
      return (-1);
   return (row);
}

/*-----------------------------------------------------------------------------
 * C W R E F R E S H ()	- Rewrite curses window.
 *-----------------------------------------------------------------------------
 */
cwrefresh (win)

WINDOW *win;
{
   move (23, 45);
   refresh ();
   wrefresh (win);
}

/*-----------------------------------------------------------------------------
 * H O M E C U R S O R ()	- Put cursor at "home" position.
 *-----------------------------------------------------------------------------
 */
homecursor (loc)

char *loc;
{
}


/*-----------------------------------------------------------------------------
 * U R O P E ()		- maintain the command bar at the top of the screen
 *-----------------------------------------------------------------------------
 */
urope (rope, cstr, uplow)

char rope [], uplow;
char cstr [];
{
   extern char toupper(), tolower();	/* from clib.a */
   extern char *lower();
   int i, count;
   int j = 0;

   count = 1; i = 0;
   strcpy (rope, lower (rope));
   while (rope [i++] != '\0')
      {
      if (strncmp (&rope [i], lower (cstr), strlen (cstr)) == 0)
         {
         while (cstr [j] != '\0')
            {
            if (uplow == 'U')
               rope [i+j] = (char) toupper (rope [i+j]);
            else
               rope [i+j] = (char) tolower (rope [i+j]);
            j++;
            }
         }
      }
   disp_head (rope);
   /* mvcur (0, 0, 23, 79); */
}
