/*--------------------------------------------------------------------------
 * point()	- Extracts positional information from SMM C/P image data sets
 *--------------------------------------------------------------------------
 * Written by Kevin Cole 12/17/84  		
 * 8 August 1986: Open posfil wil 'append', instead of 'write' attribute.
 *--------------------------------------------------------------------------
 * Major Variables:
 *	row,col		- current row and column position. (0 <= row <= 24)
 *			  (0 <= col <= 24).
 *	cmd		- current character command from keyboard.
 *	P_line[]	- Menu lines 
 *
 * Libraries required:
 *	curses and termlib.
 *----------------------------------------------------------------------------
 */
#include "scan.ext"

point (imgst, brow)

struct IMAGE *imgst [];
int brow;			/* beginning row for window */
{
   extern struct SCAN *scnst;
   FILE *fd, *fopen ();
   WINDOW *pwin, *newwin ();
   float radius = 1.5, theta = 0.0, xpixel, ypixel;
   int  row=0, ctab1=4, ctab2=14, ctab3=28, ctab4=39;
   int  WRITEFILE, DATABASEFILE;
   char cmd, filename[64], buff[80];

   static char *P_line[] =
      {
      "-----------------------Position Measurements-------------------------",
      "InFile:..............................................................",
      "OutFile:.............................................................",
      "---------------------------------------------------------------------",
      ".....POLAR COORDINATES....FILE COORDINATES...........................",
      ".....Angle......Radius........X...........Y..........Comments........."
      };

   /* If scnst->index is negative then no image files have been opened...
    * Print error and return.
    */
   if ((scnst->index < 0) || (scnst->OPENFLAG == 0)) 
      {
      disp_err ("Image file is undefined");
      return (-1);
      }

   /* We'll open a window here :
    * 10 rows, 70 columns beginning at row 8 column 5
    */
   pwin = newwin (10, 70, 8, 5);
   scrollok (pwin, TRUE);		/* this window will scroll */

   if (scnst->DSP)		/* if grinnell access requested then init */
      {
      gl_setcur (63, 63);	/* initialize zoom cursor location */
      gl_cursor (1);		/* display the zoom cursor	*/
      }

   /* This routine is necessary if inverse video is desired, otherwise you
    * can safely remove it and the routines revon() and revoff().
    */
   wpaint (pwin);

   /*** If there is no image is available, then we're out of here. ***/

   if(strlen (imgst[scnst->index]->name) < 1) return (-1);

   /* Let's see if there is a filename available for storing results.
    * If the filename begins with the letters 'db:', then treat it as
    * a database file.
    */

   if (strncmp (scnst->posfile, "db:", 3) == 0)
      {
      DATABASEFILE = 1;
      (void) strcpy (filename, &scnst->posfile[3]);
      }
   else
      {
      DATABASEFILE = 0;
      if ((fd = fopen (scnst->posfile, "a")) == NULL)
         {
         /* if the image is not available, display an error message and 
          * and return to caller.
          */
         (void) sprintf (buff, "Unable to open %s      ", scnst->posfile);
         disp_err (buff);
         WRITEFILE = 0;
         }
      else
         {
         /*** Print a header record and the file name ***/

         WRITEFILE = 1;
         fprintf (fd,
"-------------------------------------------------------------------------------\n");
         fprintf (fd,"File: %s  TIME: %d:%d:%d  DOY: %d  YEAR: %d\n",
                  imgst[scnst->index]->name,imgst[scnst->index]->hour,
                  imgst[scnst->index]->minute, imgst[scnst->index]->sec,
                  imgst[scnst->index]->doy, imgst[scnst->index]->year);
         fprintf (fd,"Type: %s   Brightness Scale: %f    Center x: %f y: %f\n",
                  imgst[scnst->index]->type, imgst[scnst->index]->scale,
                  imgst[scnst->index]->xcen, imgst[scnst->index]->ycen);
         printf (fd,
"-------------------------------------------------------------------------------\n");
         }
      }
   revon ();
   wdisptable (pwin, P_line, 6, WRITEFILE);

   /*** Now, get the info from the screen. ***/

loop1:
   wmove (pwin, row, ctab1);
   wrefresh (pwin);

   /*** Read a command from the user. If escape is pressed, then exit. ***/

   if ((cmd = (char) readline (pwin, buff, 64,'-')) == ESC)  
      {
      cleanup (pwin);
      removetable (pwin);
      if (WRITEFILE)
         {
         if (DATABASEFILE) 
            {sleep (1);}
         else
            (void) fclose (fd);
         }
      return (0);
      }
   if (cmd == 7) 
      {
      /*
       * This is the response to the user pressing control-g. Reads the 
       * zoom cursor location and uses those coordinates.
       */
      gltosmm (imgst[scnst->index], &xpixel, &ypixel);
      xpixel += imgst[scnst->index]->ip;
      ypixel += imgst[scnst->index]->ir;
      if (coord (imgst[scnst->index], &radius, &theta, &xpixel, &ypixel, -1))
         {
         printf ("position measurement error: xpix=%f, ypix=%f\n",
                 xpixel, ypixel);
         goto loop1;
         }
      (void) sprintf (buff,"%8.3f",theta);
      mvwaddstr (pwin, row,ctab1, buff);
      if (WRITEFILE) fprintf (fd, "  %s", buff);
      (void) sprintf (buff,"%8.3f",radius);
      mvwaddstr (pwin, row,ctab2, buff);
      if (WRITEFILE) fprintf (fd, "  %s", buff);

      (void) sprintf (buff, "%6.2f", xpixel);
      mvwaddstr (pwin, row,ctab3, buff);
      if (WRITEFILE) fprintf (fd, "  %s", buff);

      (void) sprintf (buff, "%6.2f", ypixel);
      mvwaddstr (pwin, row,ctab4, buff);
      if (WRITEFILE) fprintf (fd, "  %s", buff);

      wrefresh (pwin);
      }
   if (WRITEFILE) fprintf (fd, "\n");
   row++;
   goto loop1;
}


/*
 *--------------------------------------------------------------------------
 * W D I S P T A B L E ( )	- 
 *--------------------------------------------------------------------------
 */
wdisptable (win, lines, nlines, WRITEFILE)

WINDOW *win;					/* the window               */
char *lines[];					/* the menu lines           */
int nlines;					/* number of menu lines     */
int WRITEFILE;					/* to write or not to write */
{
   extern struct IMAGE *imgst[];
   extern struct SCAN *scan;		/* program constants	    */
   extern WINDOW *stdscr;			/* standard screen	    */
   int rowoff,coloff;

   rowoff = 2;				/* row offset		   */
   coloff = win->_begx;			/* get first window column */

   mvwaddstr (stdscr,rowoff,coloff, lines[0]); 
   wrefresh (stdscr);

   /*** Fill in the image file name and position save file name. ***/

   mvwaddstr (stdscr, rowoff+1,coloff, lines[1]);
   if (WRITEFILE)
      mvwaddstr (stdscr, rowoff+1,coloff+9,imgst[scnst->index]->name);
   else
      mvwaddstr (stdscr, rowoff+1,coloff+9,"FILE NOT DEFINED");
      mvwaddstr (stdscr, rowoff + 2,coloff, lines[2]);
      mvwaddstr (stdscr, rowoff + 2,coloff+9, scnst->posfile);

      mvwaddstr (stdscr, rowoff + 3,coloff, lines[3]);
      mvwaddstr (stdscr, rowoff + 4,coloff, lines[4]);
      mvwaddstr (stdscr, rowoff + 5,coloff, lines[5]);
      wrefresh (stdscr);
}


/*
 *--------------------------------------------------------------------------
 * R E M O V E T A B L E ( )	-
 *--------------------------------------------------------------------------
 */
removetable (win)

WINDOW *win;
{
   extern WINDOW *stdscr;
   int rowoff, coloff;

   rowoff = 2;
   coloff = win->_begx;
   revoff ();
   mvwaddstr (stdscr, rowoff, coloff, "\n");
   mvwaddstr (stdscr, rowoff+1,coloff,"\n");
   mvwaddstr (stdscr, rowoff+2,coloff,"\n");
   mvwaddstr (stdscr, rowoff+3,coloff,"\n");
   mvwaddstr (stdscr, rowoff+4,coloff,"\n");
   mvwaddstr (stdscr, rowoff+5,coloff,"\n");
   wrefresh (stdscr);
}
