/* rtscan.c
 *--------------------------------------------------------------------------
 * R T S C A N ( )	- Do radial & theta scans on SMM C/P image data sets.
 *--------------------------------------------------------------------------
 * Creation Date: 9/12/84  - Kevin Cole   Programmer II
 * Modified by Andrew L. Stanger
 * 22 February 1985: Modify output format for scan file.
 * 8 April 1985: Change 'g' format to 'f' and 'e' for scan report file.
 * 13 December 1985: Added # points in scan to output report file.
 * 18 March 1986: Add notice of Munro calibration correction, and
 *		  stray light/vignetting and F-corona removal.
 * 22 January 1990: Munro calibration correction removed.
 * 18 April 1990: add logic to support various data types (dtype).
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Major Variables:
 *	row,col		- current row and column position. (0 <= row <= 24)
 *			  (0 <= col <= 24).
 *	cmd		- current character command from keyboard.
 *	R_line[]	- Menu lines 
 *
 * Libraries required:
 * 	libcurses.
 *----------------------------------------------------------------------------
 */

#include "scan.ext"
#include "/usr/people/stanger/lib/src/gl/ov_color.h"

rtscan (icb, scan, scandx, nval, brow, scantype, rope)

struct IMAGE *icb [];
char scantype;
int *nval, brow;
float scan [], scandx [];
char rope [];
{
   extern display();
   float  dummy, theta, step, radius, min, max; 
   int    i, WRITEFILE, tabstop, scntyp = 1;
   char    cmd, scanid [20];
   char    buff [84];
   FILE *fd, *fopen ();
   WINDOW *rwin, *newwin ();

   static char *R_line[] =
      {
      "RADIAL SCAN: ",
      " Position angle  [deg]    = ",
      " Minimum radius  [rsun]   = ",
      " Maximum radius  [rsun]   = ",
      " Radial stepsize [rsun]   = "  
      };

   static char *T_line[] =
      {
      "THETA SCAN: ",
      " Radius          [rsun] = ",
      " Minimum theta   [deg]  = ",
      " Maximum theta   [deg]  = ",
      " Stepsize        [deg]  = "
      };

   if (scnst->OPENFLAG == 0)
      {
      revoff ();
      disp_err ("YOU HAVE NOT OPENED AN IMAGE FILE");
      return (-1);
      }

   rwin = newwin (9, 45, brow, 5);
   wpaint (rwin);

   WRITEFILE = 0;
   if (strcmp (scnst->rtfile,"off") != 0)
      {
      if ((fd = fopen (scnst->rtfile, "a")) == NULL)
         {
	 (void) sprintf (buff, "Unable to open %s      ", scnst->posfile);
	 disp_err (buff);
         }
      else
         WRITEFILE = 1;
      }

   if (scantype == 'R')
      {
      tabstop = strlen (R_line[1]);
      (void) strcpy (scanid, R_line[0]);
      }

   if(scantype == 'T')
      {
      tabstop = strlen (T_line[1]);
      (void) strcpy (scanid, T_line[0]);
      }

   if (scnst->DSP)
      {
      gl_setcur (0, 0);		/* Set zoom cursor position: center of image. */
      gl_cursor (1);		/* Display the zoom cursor.	      */
      }

   /*** Make an SMM image control block. ***/

   if ((icb[scnst->index]->addr == NULL) || (scnst->index > scnst->maxindex))
      {
      if (WRITEFILE) (void) fclose (fd);
      return (-1);
      }

R_Begin:
   revon ();
   hion (); 
   mvwaddstr (rwin, 0,0,"________________________________________");
   wrefresh (rwin);
   (void) sprintf (buff,"%s %s ", scanid, icb[scnst->index]->name);
   mvwaddstr (rwin, 0,0, buff); 
   wrefresh (rwin);
   hioff ();
   revon (); 

   /* If this was called as a radial scan, then display radial scans menu, else
    * display theta scans menu.  Anything else would be silly.
    */
   if (scantype == 'R')
      wdisplines (rwin, R_line, 5);
   if (scantype == 'T')
      wdisplines (rwin, T_line, 5);
/*-----------------------------------------------------------*/

   if (scantype == 'T')
      {
      radius = theta = scnst->thradius;
      min = scnst->thmin;
      max = scnst->thmax;
      step = scnst->thstep;
      wdispdefs (rwin, 2, tabstop, radius, min, max, step);
      }
   else
      {
      theta = scnst->rangle;
      min = scnst->rmin;
      max = scnst->rmax;
      step = scnst->rstep;
      wdispdefs (rwin, 2, tabstop, theta, min, max, step);
      }

   /*** Now, get the info from the screen. ***/

   wmove (rwin, 2,tabstop); wrefresh(rwin);

   display (rope,3);
   revon ();
   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC) 
      {
      cleanup (rwin);
      if(WRITEFILE) (void) fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto R_Begin;
   if (cmd == ':')
      {
      revoff ();
      command ();
      revon ();
      goto R_Begin;
      }

   /*** Process Ctrl-G	- Get coordinates from current cursor position. ***/

   if (( cmd == 7) && (scnst->DSP) ) 
      {
      if (wgetzoomf (icb[scnst->index],rwin,tabstop,&min,&theta,2,2,scantype)<0)
         {
         if (WRITEFILE) (void) fclose (fd);
         return (-1);
	 }
      goto skip1;
      }

   /*** Get THETA or RADIUS. ***/

   if (strlen (buff) > 0)
      (void) sscanf (buff,"%f",&theta);

   /*** If this is a theta scan.... ***/

   if (scantype == 'T')
      radius = theta;
   hion (); 
   (void) sprintf (buff,"%7.2f__", theta); 
   hioff ();
   revon (); mvwaddstr (rwin, 2, tabstop, buff);

   wmove (rwin, 3,tabstop);  wrefresh (rwin);

   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC)
      {
      cleanup (rwin);
      if (WRITEFILE) (void) fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto R_Begin;
   if (cmd == ':')
      {
      revoff ();
      command ();
      revon ();
      goto R_Begin;
      }

   /*** Get RADIUS MIN or THETA MIN. ***/

   if (strlen (buff) > 0)
      (void) sscanf (buff,"%f",&min);
   (void) sprintf (buff,"%7.2f______", min);
   revon(); mvwaddstr (rwin, 3, tabstop, buff);
   wrefresh (rwin);

skip1:
   wmove (rwin, 4,tabstop);  wrefresh (rwin);
   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC) 
      {
      cleanup (rwin);
      if (WRITEFILE) (void) fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto R_Begin;
   if (cmd == ':')
      {
      revoff ();
      command ();
      revon ();
      goto R_Begin;
      }

   /*** if Ctrl-G pressed, read the grinnell zoom cursor. ***/

   if (( cmd == 7) && scnst->DSP)  
      {
      if (wgetzoomf (icb[scnst->index], rwin, tabstop, &max, &dummy, 4, 0, 
	  scantype) < 0)
	 {
         if (WRITEFILE) (void) fclose (fd);
         return (-1);
         }
      goto skip2;
      }

   /*** Get RMAX or THETA MAX. ***/

   if (strlen(buff) > 0) (void) sscanf (buff, "%f", &max);
   (void) sprintf (buff, "%7.2f______", max);
   mvwaddstr (rwin, 4, tabstop, buff);
   revon ();  wrefresh (rwin);
skip2:
   wmove (rwin, 5, tabstop);   wrefresh (rwin);
   if ((cmd = (char) readline (rwin, buff, 64, '-')) == ESC) 
      {
      cleanup (rwin);
      if (WRITEFILE) (void) fclose (fd);
      return (0);
      }
   if ((cmd == CTRL_F) || (cmd == CTRL_B)) goto R_Begin;
   if (cmd == ':')
      {
      revoff ();
      command ();
      revon ();
      goto R_Begin;
      }

   /*** Get STEPSIZE. ***/

   if (strlen (buff) > 0)
   (void) sscanf (buff, "%f", &step);
   (void) sprintf (buff, "%7.3f______", step);
   mvwaddstr (rwin, 5, tabstop, buff);
   revon ();  wrefresh (rwin);

   /*** RADIAL scan. ***/

   if (scantype == 'R')
      {
      if (scnst->DSP)
         {
/*         if (scnst->ERASEOVL)  gl_wclear (ov_black); */
         rscan (icb[scnst->index],min,max,step,theta,&scntyp,scan,scandx,nval);
         }
      else
         rscan (icb[scnst->index],min,max,step,theta,&scntyp,scan,scandx,nval);
      scnst->rangle = theta;
      scnst->rmin = min;
      scnst->rmax = max;
      scnst->rstep = step;
      }

   /*** THETA scan. ***/

   if (scantype == 'T')
      {
      if (scnst->DSP)
         {
/*         if (scnst->ERASEOVL)  gl_wclear (ov_black); */
         tscan (icb[scnst->index], min, max, step, radius, scan, scandx, nval);
         }
      else
         tscan (icb[scnst->index], min, max, step, radius, scan, scandx, nval);
      scnst->thradius = radius;
      scnst->thmin = min;
      scnst->thmax = max;
      scnst->thstep = step;
      }

   (void) sprintf (buff," Points: %3d",*nval);
   if (WRITEFILE)
      {
      /* write the header record */
      fprintf (fd,"\n");
      fprintf (fd,
"-------------------------------------------------------------------------------\n");
      fprintf (fd,"File: %s  TIME: %02d:%02d:%02d  DOY: %03d  YEAR: %d\n",
              icb[scnst->index]->name,   icb[scnst->index]->hour,
              icb[scnst->index]->minute, icb[scnst->index]->sec,
              icb[scnst->index]->doy,    icb[scnst->index]->year);
      fprintf (fd,"Type: %s   Brightness Scale: %.4e   Center x: %6.2f y: %6.2f\n",
              icb[scnst->index]->type, icb[scnst->index]->scale,
	      icb[scnst->index]->xcen, icb[scnst->index]->ycen);
      fprintf (fd, "Number of points in scan : %d\n", *nval);
      if (icb[scnst->index]->dtype == 3)
      if (scnst->lsv)  fprintf (fd, "Stray Light/Vignetting removed.  ");
      if (scnst->lfc)  fprintf (fd, "F-corona removed.");
      fprintf (fd, "\n");
      fprintf (fd,
"-------------------------------------------------------------------------------\n");

   /*** Write the data arrays. ***/

      if (icb[scnst->index]->dtype == 1)
      strcat (wbuff2, "BRIGHTNESS [DN]");
      else
      if (icb[scnst->index]->dtype == 3)
      strcat (wbuff2, "B [total] [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 4)
      strcat (wbuff2, "DENSITY [elec/cm**2]");
      else
      if (icb[scnst->index]->dtype == 5)
      strcat (wbuff2, "Bcme-Bbkg  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 6)
      strcat (wbuff2, "B (K corona)  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 7)
      strcat (wbuff2, "(Bcme-Bbkg)/Bk  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 8)
      strcat (wbuff2, "(Bcme-Bbkg)/Bbkg  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 9)
      strcat (wbuff2, "polarization fraction");
      else
      if (icb[scnst->index]->dtype == 10)
      strcat (wbuff2, "pB  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 11)
      strcat (wbuff2, "B tangential+radial  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 12)
      strcat (wbuff2, "B tangential  [B/Bsun]");
      else
      if (icb[scnst->index]->dtype == 13)
      strcat (wbuff2, "B radial  [B/Bsun]");

   if (scantype == 'T')
      {
      sprintf (wbuff, "INDEX  RADIUS [Rsun]   THETA [degrees]");
      fprintf (fd, "%s %s\n", wbuff, wbuff2);

      for (i=0; i < *nval; i++)
      if (icb[scnst->index]->dtype == 1)
         fprintf (fd, " %3d        %6.4f         %6.4f         %10.0f\n",
                  i, scnst->thradius, scandx[i], scan[i]);
         else
            if (icb[scnst->index]->dtype == 3)
            fprintf (fd, " %3d        %6.4f         %6.4f         %10.2e\n",
                     i, scnst->thradius, scandx[i], scan[i]);
         else
            fprintf (fd, " %3d        %6.4f         %6.4f         %g\n",
                     i, scnst->thradius, scandx[i], scan[i]);
      }

   if (scantype == 'R')
      {
      sprintf (wbuff, "INDEX  RADIUS [Rsun]   THETA [Degrees]");
      fprintf (fd, "%s %s\n", wbuff, wbuff2);

      for (i=0; i < *nval; i++)
         if (icb[scnst->index]->dtype == 1)
            fprintf (fd, " %3d        %6.4f         %6.4f         %10.0f\n",
                     i, scandx[i], scnst->rangle, scan[i]);
         else
         if (icb[scnst->index]->dtype == 3)
            fprintf (fd, " %3d        %6.4f         %6.4f         %10.2e\n",
                     i, scandx[i], scnst->rangle, scan[i]);
         else
            fprintf (fd, " %3d        %6.4f         %6.4f         %g\n",
                     i, scandx[i], scnst->rangle, scan[i]);
      }
   }

   revon ();
   mvwaddstr (rwin, 7, 0, buff);  
   wrefresh (rwin);
   display (rope, 4);

   goto R_Begin;
}

/*------------------------------------------------------------------------------
 * W D I S P L I N E S ( )
 *------------------------------------------------------------------------------
 */
wdisplines (win, lines, nlines)
WINDOW *win;
char *lines[];
int nlines;
{
   int i;

   nlines++;

/* Expect first line to be a title line... previously displayed.
 * ... Display remaining lines. 
 */
   for (i = 2; i < nlines; i++)
      mvwaddstr (win, i,0, lines[i-1]);
   touchwin (win);
   wrefresh (win);
}
	
/*------------------------------------------------------------------------------
 * W P A I N T ( )
 *------------------------------------------------------------------------------
 */
wpaint (win)

WINDOW *win;
{
   int i, j;

   for (i = 0; i < win->_maxy; i++)
      {
      for (j = 0; j < win->_maxx-1; j++)
         {
         wmove (win, i, j);
         waddch (win, '_');
         }   	
      }
   revon ();
   wmove (win, 0,0);
   wrefresh (win);
}

/*------------------------------------------------------------------------------
 * W R E V O N ( )
 *------------------------------------------------------------------------------
 */
wrevon (win)

WINDOW *win;
{
   touchwin (win);
   revon ();
   wrefresh (win);
}


/*------------------------------------------------------------------------------
 * M V W S P A C E ( )
 *------------------------------------------------------------------------------
 */
mvwspace (win, row,col, nchar)

WINDOW *win;
short int row,col,nchar;
{
   int i;

   revoff ();
   wmove (win, (int) row, (int) col);
   touchwin (win);  wrefresh (win);
   wrevon (win);

   for (i = 0; i < nchar; i++)
      {
      wmove (win, row, col+i);
      waddch (win, '.');
      wrefresh (win);
      wmove (win, row, col+i);
      waddch (win, '_');
      wrefresh (win);
      }
}


/*------------------------------------------------------------------------------
 * W G E T Z O O M F ( )	- read the zoom cursor and convert to (r,theta)
 *				  coordinates.
 *------------------------------------------------------------------------------
 */
wgetzoomf (fcb, win, tabstop, radius, theta, row, nvar, scantype)

struct IMAGE *fcb;
WINDOW *win;
int tabstop, row, nvar;
float *radius, *theta;
char scantype;
{
   float zoomx, zoomy;
   char buff [84];

   revon ();
   gltosmm (fcb, &zoomx, &zoomy); 	  /* get zoom cursor coords. */
   if (coord (fcb, radius, theta, &zoomx, &zoomy, -1))
      {
      disp_err ("scan coordinate error ");
      revoff ();
      return (-1);
      }
   if (nvar == 0)
      {
      if (scantype == 'T')
         (void) sprintf (buff,"__%7.2f", *theta);
      if (scantype == 'R')
         (void) sprintf (buff,"__%7.2f", *radius);
	 mvwaddstr (win,row,tabstop,buff);
      }
   if (nvar)
      {
      if (scantype == 'R')
         {
         (void) sprintf (buff,"__%7.2f", *theta);
         mvwaddstr (win,row,tabstop,buff);
         (void) sprintf (buff,"__%7.2f", *radius);
         ++row;
         mvwaddstr (win,row,tabstop,buff);
         }
      if (scantype == 'T')
         {
         (void) sprintf (buff, "__%7.2f", *radius);
         mvwaddstr (win, row, tabstop, buff);
	 (void) sprintf (buff,"__%7.2f", *theta);
         ++row;
         mvwaddstr (win,row,tabstop,buff);
         }
      }
   wrefresh (win);
   revoff ();
   return (0);
}

/*------------------------------------------------------------------------------
 * C L E A N U P ( )	- Clean up radial/theta scan windows and attributes.
 *------------------------------------------------------------------------------
 */

cleanup (rwin)
WINDOW *rwin;
{
   wclear (rwin);
   revoff ();
   wrefresh (rwin);
   delwin (rwin);
}
	

/*------------------------------------------------------------------------------
 * W D I S P D E F S ( )
 *------------------------------------------------------------------------------
 */
wdispdefs (win,row,col, first,second,third,fourth)

WINDOW *win;
int row, col;
float first, second;
float third, fourth;
{
   char buff [20];

   (void) sprintf (buff, "__%7.2f", first);
   mvwaddstr (win, row++, col, buff);

   (void) sprintf (buff, "__%7.2f", second);
   mvwaddstr (win, row++, col, buff);

   (void) sprintf (buff, "__%7.2f", third);
   mvwaddstr (win, row++, col, buff);

   (void) sprintf (buff, "__%7.2f", fourth);
   mvwaddstr (win, row++, col, buff);

   wrefresh (win);
}
