
/*---------------------------------------------------------------------
 * Header file:  scan.h
 *
 *---------------------------------------------------------------------
 */

#include "cp.h"
#include "ncarplot.h"

#define ESC     27
#define CTRL_K  11
#define CTRL_N 	14
#define CTRL_B	 2
#define CTRL_F   6
#define CTRL_R  18
#define BEEP	 7
#define CR	13

#define MAXBUF 896 

char *string();

int   SMM = 1; 		/* true if this is Solar Max image */
int   ATM = 0;		/* true if this is an ATM image */
int   QUAD4 = 0;	/* true if this is Solar Max 4-quadrant image */

struct SCAN
   {	
   char *posfile;	/* default position measurement file  	    */
   char *rtfile;	/* default r,theta scan file         	    */
   char *rttitle;	/* title on r, theta graphics files         */
   char *linefile;	/* default line scan file		    */
   int  OPENFLAG;	/* True (=1) if an image file is opened.    */
   int  DSP;		/* Display on/off switch		    */
   int  DSPDEV;		/* Display device number		    */
   int  DSPCHA;		/* Display default image channel	    */
   int  DSPOVL;		/* Display default overlay channel         */
   int  GRIDOVL;	/* Default overlay for grids		    */
   int  LSCANOVL;	/* Default overlay for line scans	    */
   int  RSCANOVL;	/* Default overlay for radial scans	    */
   int  TSCANOVL;	/* Default overlay for theta scan	    */
   int  ERASEOVL;	/* Erase overlay flag: true if = 1.         */
   int  ITYPE;		/* Image type   (see header file)	    */
   int  GRAPH;		/* Graph generation can be either on or off */
   int  index;		/* index to structure IMAGE		    */
   int  maxindex;	/* maximum number of active images	    */
   int  nave;		/* box size (nave*nave) for scan average.   */
   int  lsv;		/* Flag for stray light & vignetting removal*/
   int  lfc;		/* Flag for F-corona removal.		    */
   float rmin;		/* Default minimum for radial scans	    */
   float rmax;		/* Default maximum for radial scans	    */
   float rstep;		/* Default stepsize for radial scans	    */
   float rangle;	/* Default angle [deg] for radial scans     */
   float thmin;		/* Default min theta for theta scan [deg]   */
   float thmax;		/* Default max theta for theta scan [deg]   */
   float thstep;	/* Default theta stepsize [deg]		    */
   float thradius;	/* Default radius for theta scan [rsun]     */
   float line1;		/* Default first line no. in linescan       */
   float pixel1;	/* Default first pixel no. in linescan      */
   float line2;		/* Default second line no. in linescan      */
   float pixel2;	/* Default second pixel no. in linescan     */
   };

/* The plotter arrays structure. */

struct PLOT
{
    char  title[80];
    char  ylab[25];
    char  xlab[80];
    float *x;
    float *y;
    int   npts;
};
