/* tscan.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Theta Scan.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 21 February 1985
 * 3 April 1985: add box average feature.
 * 17 March 1986: add stray light/vignetting and F-corona removal features.
 * 22 January 1990: Munro calibration correction removed.
 *  3 June 1991: Modified for SGI IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
extern int SMM;
extern int ATM;
extern int QUAD4;

#include <stdio.h>
#include <errno.h>
#include "scan.ext"
#include "/usr/people/smm/lib/src/ip/cp.h"

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

tscan (icb, thmin, thmax, thinc, radius, scan, scandx, ns)

struct IMAGE *icb;	/* Pointer to image array. */
float   thmin, thmax;   /* Solar position angle limits for theta scan. */
float   thinc;          /* Angular increment for theta scan. */
float   radius;         /* Radius (R/Rsun units). */
float   scan[];         /* Dependent axis scan array. */
float   scandx[];       /* Independent axis scan array. */
int     *ns;            /* Number of values in scan. */
{
   extern float blint();
   extern float pixave();
   extern int ginitx(), ginity();
   float x, y;
   float r, th;
   float tinc;
   static float rbuf[1];

   float   Cexp = 1.e-11;	/* Scale factor for calibrated images. */
/*    float   Cexp = 1.00; */	/* temporary fix for autograph  */
   register float  Bexp;	/* "Generic" scale factor. */
   register int it;
   int nres, ierr, ixg, iyg;
   int ineg, ipos;
   static char buff [80];

/* Set scale factor. */

/*   if(icb->dtype == 3)  Bexp = Cexp;   else   Bexp = 1.; */

/* Theta increment: set default to angle separating two adjacent pixels. */

   tinc = thinc;
   if(thinc <= 0.)  tinc = degrad / (radius*pixrs);

   r = radius;
   nres = 1;				/* Initialize for low res images. */
   if(icb->ires == 0) nres = 2;		/* If hi res image. */

/* Clear scan arrays. */

   for (it = 0; it < MAXSCN; it++)  *(scan + it) = *(scandx+ it) = 0.;
 
/*
 *   sprintf (buff, "bppx: %d   dtype: %d   scale: %g   nave: %d",
 *	    icb->bppx, icb->dtype, icb->scale, scnst->nave);
 *   disp_err (buff);
 */

/* ******* Theta scan loop ******* */
 
   if(thmax < thmin)  thmax += 360.;

   if (scnst->nave > 1)
      {
      ipos = scnst->nave / 2;
      ineg = scnst->nave - ipos;
      }
   else
      ipos = ineg = 1;

   *ns = 0;
   it = -1;
   for (th = thmin; th <= thmax; th += tinc)
      {
      it++;
 
/* Coordinate transformation :  (radius,angle) --> (pixel,line). */

      if((ierr = coord(icb, &r,&th,&x,&y,1)) != 0)  return(ierr);

/* Draw "X" to mark theta scan point on Grinnell display. */

      if(scnst->DSP && ((strcmp(icb->type,"SMM")== 0) || QUAD4))
         {
         ixg = (int) x/nres + ginitx();
         iyg = ginity() - (int) y/nres;
	 if (((ixg > 62) && (ixg < 511)) && (iyg > 62) && (iyg < 511))
            gl_wvec (ixg-ineg, iyg-ineg, ixg+ipos, iyg+ipos, scnst->TSCANOVL);
	 if (((ixg > 62) && (ixg < 511)) && (iyg > 62) && (iyg < 511))
            gl_wvec (ixg-ineg, iyg+ipos, ixg+ipos, iyg-ineg, scnst->TSCANOVL);
	 }
      if(scnst->DSP && ATM)
	 {
	 ixg = (int) x + 13;
	 iyg = 510 - (int) y;
	 gl_wvec (ixg,   iyg, ixg+1, iyg+1, scnst->TSCANOVL);
	 gl_wvec (ixg+1, iyg, ixg,   iyg+1, scnst->TSCANOVL);
         }
 
/* Perform bilinear interpolation to compute scan point from image. */

      *(scandx + it) = (float) th;
      x = x - icb->ip;  y = y - icb->ir;
      if ((x >= 0.0) && (x <= icb->dx) && (y >= 0.0) && (y <= icb->dy))
         {
         if (scnst->nave == 1)
	    *(scan+it) = (float) (blint (icb, &x, &y)*icb->scale);
         else
         if (scnst->nave > 1)
	    *(scan+it) = (float) pixave (icb, &x, &y, &scnst->nave)*icb->scale;
         }
      else
         *(scan+it) = 0.0;
      }
   *ns = it + 1;

/* Remove stray light & vignetting. */
/* Remove F-corona. */
 
   if (icb->dtype == 3)
      {
      rbuf[0] = r;
      if (scnst->lsv == 1)
         {
         remstr (scandx, *ns, scan, *ns);
         remvig (scandx, *ns, scan, *ns);
         if (scnst->lfc == 1)  remfc (r, 1, scandx, *ns, scan, *ns);
         }
      }
 
/*
 * Divide scan values by 5 for equivalent second data type.
 * (numbers in file are 5 * equivalent second units.)
 */
 
   if (icb->dtype == 2)
      for (it = 0; it < *ns; it++)  scan[it] /= 5.;

   return (0);
}
