/*------------------------------------------------------------------------
 * utility.c	- A set of UNIX utilities for use by scan.c
 *
 * routines included here;
 *    readline(win,s,lim,fill)
 *    ztosmm()
 *-----------------------------------------------------------------------
 */
#include "scan.ext"
struct Gr_sav
   {
   int Zcx;
   int Zcy;
   };
struct Gr_sav Gr;

/*---------------------------------------------------------------------------
 * R E A D L I N E ()		- read a line of input from a terminal 
 *			operating in C-Break mode. This is a special version 
 *			for SCAN.C which checks for Ctrl-F and Ctrl-B.
 *---------------------------------------------------------------------------
 */

readline (win, s, lim, fill) 
WINDOW *win;			/* the window to get the line through       */
char s [];			/* the line read from the terminal          */
char fill;			/* the fill character for deleting backward */
int lim;			/* max characters to read -NOT IMPLEMENTED  */
{
    int i,c, row, col, start_row, start_col;

    i = 0; 
    /* Allow newline processing, but do not echo the user's input to the
     * screen, I'll handle that.
     */
    nl();  noecho();
	
    /* Preserve the initial screen location in this window.
     */
    getyx (win, start_row, start_col);

   while((c = wgetch (win)) != EOF && c != '\n')  
      {
      switch (c)  
         {
         case ESC: return(ESC);
         case '?': return('?');

         case 7:	/* Ctrl-G --- access the Display */
            return(7);
            break;
         case CTRL_F: 	/* forward one file */
            scnst->index++;		/* point to next file    */
            if (scnst->index > scnst->maxindex) scnst->index = 0;
            if ((scnst->index > -1) && (scnst->index < 4) && scnst->DSP)
               sleep (1);
            return ((int) CTRL_F);
            break;

         case CTRL_B:    /* backward one file */
            scnst->index--;		/* point to previous file */
            if (scnst->index < 0) 
               scnst->index = scnst->maxindex;
            if ((scnst->index > -1) && (scnst->index < 4) && scnst->DSP)
               sleep (1);
            return((int) CTRL_B);
            break;	/* seems silly having a break after a return */

         case ':':
            return((int) ':');
            break;

         /*** Here I'll handle a delete or backspace (Hey, I'm easy ). ***/

         case '\177':	/* delete    (del) */
         case '\010': 	/* backspace (del) */
            getyx (win, row, col);	    /* where are we on screen */
            if (--col < start_col)	/* decrement column and   */
               col = start_col;		/* clip to initial loc.   */
            wmove (win, row, col);	/* move back one space    */
            wdelch (win);		/* delete the character   */
            winsch (win, fill);		/* insert fill character  */
            wrefresh (win);
            i--;
            break;

         default:    /* if we get here, then it is input to the buffer */
            if (i < 0) i = 0;	/* clip i	  */
            s [i++] = c;	/* add to buffer  */
            waddch (win, c);	/* echo the input */
            wrefresh (win);
            break;
         }
      }
    wmove (win, start_row, start_col);
    wrefresh (win);
    s [i] = '\0';		        /* close out the buffer   	*/
    nonl ();			        /* turn newline processing off 	*/
    return (0);		                /* return A-OK			*/
}


/*-----------------------------------------------------------------
 * G L T O S M M   - Zoom cursor coords to Solar Max (448x448) coords
 *		  entry - (Global) Zcx, Zcy  == Zoom cursor coords.
 *		  exit  - x,y == Solar Max (448x448) coords.  
 *------------------------------------------------------------------
 */
gltosmm (imgst, x, y)
struct IMAGE *imgst;
float *x, *y;   
{
   gl_rdcur ();			/* Read the zoom cursor. */
   *x = (float) ((Gr.Zcx - (ginitx () + imgst->ip)));
   *y = (float) (((ginity () - imgst->ir) - Gr.Zcy));
/*
 *	sprintf (wbuff, "Gr.Zcx: %d   Gr.Zcy: %d", Gr.Zcx, Gr.Zcy);
 *	disp_err (wbuff);
 */
}
