/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fixcal (rsun, nr, data, ndata)                                    *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Applies calibration correction to array 'data'.
 *----------------------------------------------------------------------
 *    Andrew L. Stanger   NCAR/HAO   SMM C/P
 *    26 July 1984
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *    rsun  = array of radial values (units of Rsun). 
 *    nr = # elements in rsun array (=1 for radial scan, =ndata for theta scan).
 *    data  = array of intensity values (units of 10**-11 B/Bsun).
 *    ndata = # elements in array data.
 *----------------------------------------------------------------------
 */

#include <stdio.h>

fixcal (rsun, nr, data, ndata)

int nr, ndata;
float rsun[];
float data[];

{

int i;
float r;
float cc;
extern float calcor();

	if (nr > 1 && nr < ndata)
	  {
	   fprintf(stdout,"# rsun values:%d less than # data values:%d\n",
                   nr, ndata);
	   return(-1);
	  }

	else

/* Radial scans. */

	if (nr > 1)
	  {
	   for (i=0; i<ndata; i++)
		{
		r = rsun[i];
	cc = calcor(r);
/*
 * printf("cal.correction: %f at %f radii\n", cc, r);
 */
		data[i] *= calcor(r);
		}
	   return(0);
	  }

	else

/* Theta scans. */

	if (nr == 1)
	  {
	   r = rsun[0];
	   cc = calcor(r);
/*
 * printf("cal.correction: %f at %f radii\n", cc, r);
 */
	   for (i=0; i<ndata; i++)  data[i] *= cc;
	   return(0);
	  }
}
