/* scpview.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display SMM C/P image on Grinnell-274.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#define color1 251	/* gray. */
#define color2 254	/* red. */
#define color3 250	/* yellow. */
#define color4 253	/* green. */
#define gminrow 63
#define gmincol 63
#define gmaxrow 510
#define gmaxcol 510

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Label SMM C/P image.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Kevin Cole   HAO/NCAR   SMM C/P   (extracted from 'ied_disp.c'.)
 * Adapted by Andrew L. Stanger
 * 22 February 1985
 * 24 April 1989: modified for Sun-4/280 (Yoda).
 * 20 October 1989: revised iproc.ext file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/u/rose/svgrin2/grin.h"
#include "/u/smm/lib/src/ip/new/iproc.ext"

labelsmm (ifc, linit)

struct IMAGE *ifc;
int linit;	/* Initialization flag. */
{
   short int cbar[512];			/* Color bar array. */
   int charheight, startrow;
   int startcol, endcol;
   int i;
   int colormax = 255;			/* Maximum color value. */
   int barlength = 512, barwidth = 12;	/* color bar dimensions. */
   int ppc = 2;			/* # pixels per color in color bar. */
   int cal;

   scmask (color1);
   doubleh (OFF);   doublew (OFF);

   startrow = 17 + barwidth;
   startcol = 0;
   endcol = startcol + barlength - 1;
   charheight = 43;

   if (ifc->icm == 2)
      wstr ("CAL MIRROR", 0, 475-8*charheight, 6, 0);
   else
      wstr("ROLL      ", 0, 475-8*charheight, 6, 0);

/* Draw box around image window. */

   if (linit)
      {
      wvec ( 62,  62,  62, 511);
      wvec ( 62, 511, 511, 511);
      wvec (511, 511, 511,  62);
      wvec (511,  62,  62,  62);

/* Write annotation labels into Grinnell channel. */ 

      wstr ("DATE",       1, 496,              7, 0);
      wstr ("TIME",       1, 496-1*charheight, 7, 0);
      wstr ("FILTER",     1, 475-3*charheight, 7, 0);
      wstr ("POLAROID",   1, 475-4*charheight, 7, 0);
      wstr ("SECTOR",     1, 475-5*charheight, 7, 0);
      wstr ("EXPOSURE",   1, 475-6*charheight, 7, 0);
      wstr ("RESOLUTION", 1, 475-7*charheight, 6, 0);

      wstr ("FRAME  ", 1, 475-9*charheight,7, 0);

/* Add annotation for the color bar */

      wstr ("0"  , (startcol+1),          1, 7, 0);
      wstr ("64" , (startcol+64*ppc-7),   1, 7, 0);
      wstr ("128", (startcol+128*ppc-10), 1, 7, 0);
      wstr ("192", (startcol+192*ppc-10), 1, 7, 0);
      wstr ("255", (endcol-21),           1, 7, 0);

/* Add tick marks to color bar */

      wvec (startcol,         11, (startcol),         16);
      wvec (startcol+32*ppc , 11, (startcol+32*ppc),  16);
      wvec (startcol+64*ppc , 11, (startcol+64*ppc),  16);
      wvec (startcol+96*ppc , 11, (startcol+96*ppc),  16);
      wvec (startcol+128*ppc, 11, (startcol+128*ppc), 16);
      wvec (startcol+160*ppc, 11, (startcol+160*ppc), 16);
      wvec (startcol+192*ppc, 11, (startcol+192*ppc), 16);
      wvec (startcol+224*ppc, 11, (startcol+224*ppc), 16);
      wvec (startcol+255*ppc, 11, (startcol+255*ppc), 16);

/* Draw the color bar (2 pixels wide per color). */

      scmask (DEF);
      for (i = 0; i <= colormax; i++)
         {
         cbar[2*i] = i & 0377;
         cbar[2*i+1] = i & 0377;
         }

      for (i = 0; i < barwidth; i++)
         wlin (cbar, startcol, startrow--, barlength, 'r');

/* Write logo. */

      scmask (color4);		/* Set sub-channel mask. */
      doublew (ON);
      wstr (" HAO/SMM   CORONAGRAPH / POLARIMETER ", 64, 34, 6,0);
      doublew (OFF);
      scmask(DEF);
      }
}

/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write SMM C/P image annotation to Grinnell channel.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Rose Reynolds   HAO/NCAR   SMM C/P   (extracted from 'smmsubs.c').
 * Adapted by Andrew L. Stanger
 * 22 February 1985
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

annotsmm (ifc)
struct IMAGE *ifc;	/* Pointer to image info structure. */
{
  extern int nres;
  static char wbuf[11];
  static char month[4]; 
  static char namgls[] = "MASF";
  char *month_name();		/* Function to look up ASCII month name. */
  int month_day();		/* Function to compute mont & day from DOY. */
  int annot_state = 1;
  int charheight = 43;
  int offset = -21;
  int year,doy,hr,min,sec,met;
  int sectr,filter,pol,n53,s53,cal,exp,ops,roll;
  int x,y,yy,imonth, iday;
  int xc, yc;

   doubleh (ON);
   scmask (color2);

/* Year and Day-Of-Year: */

   (void) sprintf (wbuf, "%d: %3d", ifc->year, ifc->doy);
   wbuf[7] = '\0';
   wstr (wbuf, 3, 476, 7, 0);

/* Time: */
   (void) sprintf (wbuf,"%02d:%02d:%02d\0",ifc->hour,ifc->minute,ifc->sec);
   wstr (wbuf, 3, 433, 7,0);

/* Day and month. */

   year = ifc->year + 1900;
   (void) month_day (year, ifc->doy, &imonth, &iday);
   strcpy (month, month_name(imonth));
   sprintf (wbuf, "%2d %s ", iday, month);
   wstr (wbuf, 3, 400, 7, 0);
   sprintf (wbuf, "%d", year);
   wstr (wbuf, 3, 375, 7, 0);

/* Spectral filter: */

   switch (ifc->ifw) 
      {
      case 0: sprintf(wbuf,"RED   "); break;
      case 1: sprintf(wbuf,"HALPHA"); break;
      case 2: sprintf(wbuf,"OPEN  "); break;
      case 3: sprintf(wbuf,"BLUE  "); break;
      case 4: sprintf(wbuf,"WIDEBD"); break;
      case 5: sprintf(wbuf,"GREEN "); break;
      case 6: sprintf(wbuf,"GL    "); break;
      default: sprintf(wbuf,"******"); break;
      }
   if (ifc->iglstw == 1)  sprintf(wbuf,"%03d %c ",
			  ifc->iglpos, namgls[ifc->iglstw]);
   wstr(wbuf, 3, 475-3*charheight+offset, 8, 0);


/* Polaroid filter: */

   switch (ifc->ipw) 
      {
      case 0: sprintf(wbuf,"CLR"); break;
      case 1: sprintf(wbuf,"+60"); break;
      case 2: sprintf(wbuf,"-60"); break;
      case 3: sprintf(wbuf,"0DG"); break;
      default: sprintf(wbuf,"***"); break;
      }
   wstr (wbuf, 3, 475-4*charheight+offset, 8, 0);


/* Sector mirror: */

   switch (ifc->ism) 
      {
      case 0:  sprintf(wbuf,"SW  "); break;
      case 1:  sprintf(wbuf,"S   "); break;
      case 2:  sprintf(wbuf,"SE  "); break;
      case 3:  sprintf(wbuf,"E   "); break;
      case 4:  sprintf(wbuf,"NE  "); break;
      case 5:  sprintf(wbuf,"N   "); break;
      case 6:  sprintf(wbuf,"NW  "); break;
      case 7:  sprintf(wbuf,"W   "); break;
      case 8:  sprintf(wbuf,"NESW"); break;
      default: sprintf(wbuf,"****");
      }
   wstr (wbuf, 3, 475-5*charheight+offset, 8, 0);


/* Exposure (1.024 second units) : */

   (void) sprintf(wbuf,"%-4u",ifc->exp);
   wstr (wbuf, 3, 475-6*charheight+offset, 8, 0);


/* Resolution : */

   if ( ifc->ires == 0 ) (void) sprintf(wbuf,"HIGH");
   else (void) sprintf(wbuf,"LOW ");
   wstr (wbuf, 3, 475-7*charheight+offset, 8, 0);

/* S/C Roll : */

   if (ifc->icm == 1)
      {
      if (annot_state) 
         {
         labelsmm (ifc, 0);
         doubleh (ON);
         scmask (color2);
         }

      (void) sprintf(wbuf,"%-4d",ifc->roll);
      wstr (wbuf, 3, 475-8*charheight+offset, 8, 0);

      annot_state = 1;
      }

/* Calibration Mirror: */

   else
      {
      switch (ifc->icm)
         {
         case 0: (void) sprintf(wbuf,"MOVE"); break;
         case 1: (void) sprintf(wbuf,"OUT "); break;
         case 2: (void) sprintf(wbuf,"IN  "); break;
         case 3: (void) sprintf(wbuf,"BAD "); break;
         default: (void) sprintf(wbuf,"****"); break;
         }
      if (annot_state) 
         {
         labelsmm (ifc, 0);
         doubleh (ON);
         }

      wstr (wbuf, 3, 478-8*charheight+offset, 8, 0);
      annot_state = 0;
      }

/* Image count (frame number) : */

   (void) sprintf (wbuf,"%-4u",ifc->frameno);
   wstr (wbuf, 3, 475-9*charheight+offset, 8, 0);

   doubleh (OFF);
   scmask (DEF);

   xc = (int) ifc->xcen;
   yc = (int) ifc->ycen;

   pointer (ifc, xc, yc, 10, 10, color3); /* Draw pointer at disk center. */
   scalers (ifc, color3);		  /* Draw solar radius scale. */
   suncir (ifc, color3);		  /* Draw sun circle at 1.0 Rsun. */
}


/*-----------------------------------------------------------------------------
 * P O I N T E R ()   - Draws a cross hair with center (screen coordinates)
 *		        at x,y with lengths dx and dy.
 *
 * If the coordinates fall outside the displayable screen area. The the 
 * coordinates are clipped to the screen boundary.
 *
 * gminrow, gmaxrow  - integers. Minimum and maximum vertical pixels.
 * gmincol, gmaxcol  - integers. Minimum and maximum horizontal pixels.
 *
 * externals required;
 *    wvec()	- draw a vector on the display screen.
 *-----------------------------------------------------------------------------
 */

pointer (ifc, x, y, dx, dy, subch)

struct IMAGE *ifc;
int x, y;				/* cross hair center position */
int dx, dy;				/* cross hair extent in pixels */
int subch;				/* Grinnell image channel. */
{
extern int nres;
float r, th, rfactor;
float xx, yy;
int xg, yg;	/* Inner circle. */
int xcn, ycn;	/* North of center. */
int xcs, ycs;	/* South of center. */
int xce, yce;	/* East of center. */
int xcw, ycw;	/* West of center. */
int xpn, ypn;	/* North pointer tip. */
int xwe, ywe;	/* Left wing tip. */
int xww, yww;	/* Right wing tip. */

   scmask (subch);			/* Set sub-channel mask. */

   if (ifc->ism == 8)  rfactor = 2.0;
   else
   rfactor = 1.0;
 
/* North pointer tip coordinates. */

   r = 0.17 * rfactor;
   th = 0.0;
   coord (ifc, &r, &th, &xx, &yy, 1);
   xpn = ((xx / nres) + 0.5) + gminrow;
   ypn = gmaxcol - ((yy / nres) + 0.5);

/* East wing tip coordinates. */

   r = 0.12 * rfactor;
   th = 45.0;
   coord (ifc, &r, &th, &xx, &yy, 1);
   xwe = ((xx / nres) + 0.5) + gminrow;
   ywe = gmaxcol - ((yy / nres) + 0.5);

/* West wing tip coordinates. */

   r = 0.12 * rfactor;
   th = 315.0;
   coord (ifc, &r, &th, &xx, &yy, 1);
   xww = ((xx / nres) + 0.5) + gminrow;
   yww = gmaxcol - ((yy / nres) + 0.5);

/* Draw dot at disk center. */

   r = 0.0;
   th = 0.0;
   coord (ifc, &r, &th, &xx, &yy, 1);
   xg = ((xx / nres) + 0.5) + gminrow;
   yg = gmaxcol - ((yy / nres) + 0.5);
   if (xg  >= 0 && xg  <= 511 && yg  >= 0 && yg  <= 511)
	wvec (xg, yg, xg, yg);

/* Draw circle around disk center. */

   r = 0.06 * rfactor;
   for (th=0.0; th<=360.0; th += 10.0)
      {
      coord (ifc, &r, &th, &xx, &yy, 1);
      xg = ((xx / nres) + 0.5) + gminrow;
      yg = gmaxcol - ((yy / nres) + 0.5);
      if (xg  >= 0 && xg  <= 511 && yg  >= 0 && yg  <= 511)
         wvec (xg, yg, xg, yg);
      }
/* Draw line from circle to north pointer tip. */

   r = 0.06 * rfactor;
   th = 0.0;
   coord (ifc, &r, &th, &xx, &yy, 1);
   xg = ((xx / nres) + 0.5) + gminrow;
   yg = gmaxcol - ((yy / nres) + 0.5);
   if (xpn >= 0 && xpn <= 511 && ypn >= 0 && ypn <= 511 &&
       xg  >= 0 && xg  <= 511 && yg  >= 0 && yg  <= 511)
	wvec (xg, yg, xpn, ypn);
 
/* Draw north pointer. */

   if (xpn >= 0 && xpn <= 511 && ypn >= 0 && ypn <= 511 &&
       xwe >= 0 && xwe <= 511 && ywe >= 0 && ywe <= 511)
	wvec (xpn, ypn, xwe, ywe);   /* North pointer tip to east wing tip. */

   if (xpn >= 0 && xpn <= 511 && ypn >= 0 && ypn <= 511 &&
       xww >= 0 && xww <= 511 && yww >= 0 && yww <= 511)
	wvec (xpn, ypn, xww, yww);   /* North pointer tip to west wing tip. */

   scmask (DEF);		/* Set sub-channel mask. */
}

/* scalers.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Draw solar radius measuring scale.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
scalers(ifc,  subch)

struct IMAGE *ifc;
int subch;		/* Grinnell sub-channel mask. */
{
extern int nres;
int x;
int ix = 63;
int iy = 63;
int gpixrs;
int dx = 4;	/* Size of 1/2 of tick mark. */
int dxg = 448;
int yt;		/* Grinnell y-coordinate for top of tick mark. */
int ym;		/* Grinnell y-coordinate for middle of tick mark. */
int yb = 52;	/* Grinnell y-coordinate for bottom of tick mark. */

   wsolid (0, ix, yb, 448, 2*dx+1, "ru"); /* Write black in Rsun scale box. */
   ym = yb + dx;
   yt = ym + dx;
   scmask (subch);		/* Set sub-channel mask. */

/* Draw ticks along the bottom of the image. */

   gpixrs = ifc->pixsr / nres;
   x = (int) ifc->xcen / nres + gminrow;
   wvec (x-dx, ym, x+dx, ym);
   wvec (x, yt, x, yb);

   while(((x += gpixrs) > ix) && (x < (ix + dxg)))
      {
      wvec(x, yt, x, yb);
      }

   x = (int) ifc->xcen / nres + gminrow;
   while(((x -= gpixrs) > ix) && (x < (ix + dxg)))
      {
      wvec(x, yt, x, yb);
      }
   scmask(DEF);			/* Set sub-channel mask. */
}



/*
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * S U N C I R ( )	- Draw a dotted circular pattern at 1.0 Rsun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
suncir (ifc, subch)

struct IMAGE *ifc;
int subch;		/* Image channel. */

{
extern int nres;
float x, y;		/* (x,y) coordinates corresponding to (r,th). */
float r = 1.0;		/* Radius in R/Rsun units. */
float th;		/* Position angle in degrees. */
int xg, yg;		/* Grinnell coordinates. */

float radius;
float x1, y1, x2, y2;
float r1, r2;
int xg1, yg1, xg2, yg2;

   scmask (subch);			/* Set sub-channel mask. */

   /*** Draw radial scans (r=0.2 ->1.4; th= 0,90,180,270). ***/

   for (th = 0.0; th < 360.0; th += 90.0)
      for (radius = 0.2; radius < 1.4; radius += 0.1)
	 {
	 coord (ifc, &radius, &th, &x, &y, 1);
         xg = ((x / nres) + 0.5) + gminrow;
         yg = gmaxcol - ((y / nres) + 0.5);
         if (xg >= gmincol && xg <= gmaxcol &&
	     yg >= gminrow && yg <= gmaxrow)  wvec (xg, yg, xg, yg);
	 }
/*
    r1 = 0.12;
    r2 = 0.9;
    for (th = 0.0; th < 360.0; th += 90.0)
       {
	 coord (ifc, &r1, &th, &x1, &y1, 1);
	 coord (ifc, &r2, &th, &x2, &y2, 1);
         xg1 = ((x1 / nres) + 0.5) + gminrow;
         yg1 = gmaxcol - ((y1 / nres) + 0.5);
         xg2 = ((x2 / nres) + 0.5) + gminrow;
         yg2 = gmaxcol - ((y2 / nres) + 0.5);
         if (xg1 >= gmincol && xg1 <= gmaxcol &&
	     yg1 >= gminrow && yg1 <= gmaxrow &&
             xg2 >= gmincol && xg2 <= gmaxcol &&
	     yg2 >= gminrow && yg2 <= gmaxrow)
	    wvec (xg1, yg1, xg2, yg2);
       }
 */

   /*** Draw sun circle at 1.0 Rsun.  One dot every 10 degrees. ***/

   for (th = 0; th < 360; th += 10)
      {
      coord (ifc, &r, &th, &x, &y, 1);
      xg = ((x / nres) + 0.5) + gminrow;
      yg = gmaxcol - ((y / nres) + 0.5);
      if (xg >= gmincol && xg <= gmaxcol &&
	  yg >= gminrow && yg <= gmaxrow)  wvec (xg, yg, xg, yg);
      }

   scmask (DEF);			/* Set sub-channel mask. */
   return (0);
}
