/* cpsyn1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Create a pixel file containing SMM C/P Synoptic Map data.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Use with synoptic map files containing photometrically calibrated data.
 * Companion program for use with files generated by `sg'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Original code written by Rainer Illing.
 * Revisions by Andrew L. Stanger
 * 29 April 1987
 * 5 May 1987: Fix for 1980 synoptic map.
 * 14 January 1988: Revise to be compatible with `sg3.c'.
 * 25 May 1988: modify for compatibility with `sg3a.c'.
 * 2 June 1988: non-existent data bins have times > +1000.
 * 3 August 1988: Remove photometric scaling adjustments.  All synoptic files
 *                should contain photometrically calibrated data.
 * 23 June 1989: Sun-4 HIPS version.
 * 10 September 1990 : For single-rotation maps, use square aspect ratio.
 *  9 April 1991: adapt for SGI IRIS.
 *  7 January 1992: adapt from "sd.c"
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include	<stdio.h>
#include	<math.h>
#include	"synst.h"
#define		nxsyn	364
#define		nysyn	184

struct S STT;

main (argc, argv)

int argc;
char *argv [];
{
extern float synrot ();	/* Synodic rotation for a given DOY. */
extern float syndoy ();	/* Synodic day-of-year for start of a given rotation. */
extern float syndur ();	/* Synodic rotation duration. */
register struct	S	*ST = &STT;	/* pointer to synoptic record. */
FILE	*SYN;				/* file pointer. */
static	char	dir1[2] = "d";
static  char	crad [3];
static  char	ibuf [364];		/* Intensity buffer. */
static  char	indiv [2];		/* Individual maps. */
static	char	irismin = 0;
static	char	irismax = 249;
static  char	namfil [80];
static  char	nameast [80];		/* East output file name. */
static  char	namwest [80];		/* West output file name. */
static	char	eastbuf [nysyn][nxsyn];
static	char	westbuf [nysyn][nxsyn];

static	float	crdur [16];	/* Duration of carrington rotations. */
float	crbeg, crend;	/* Rotation begin/end for synoptic map display. */
float	doybege, doybegw, doymax;
float	filedoy0;	/* DOY at start of synoptic map data file. */
float	filerot0;	/* Rotation (cmp) at start of synoptic map data file. */
float	doymap;		/* Map duration in units of days. */
float	doyrot;		/* rotation duration in units of days. */
float	pixsam;		/* # pixels per sample. */
float	posdoye, posdoyw;
float	radius;
float	rotbeg, rotend;	/* Rotation begin/end for synoptic data file. */
float	samrot;		/* # samples per rotation. */
float	zmax = 250.0;
float	zscale;
float	xtrarot = 0.0;	/* Extra display beyond limits of a given rotation. */

static int locbeg [16];
int	MSROT = 55;	/* Maximum # samples per rotation. */
int	MAXROT = 15;	/* Maximum # carrington rotations in syngen data file.*/
int	NROT = 6;	/* # carrington rotations to be displayed in map. */

int	fdeast;		/* File descriptor for east output file. */
int	fdwest;		/* File descriptor for west output file. */
int	idoy0;		/* DOY for first sample in data file. */
int	i;
int	ibege, ibegw;	/* start index for east/west DOY loops. */
int	iende, iendw;	/* end index for east/west DOY loops. */
int	icr;		/* Carrington rotation index. */
int	idot;
int	ieast, iwest;
int	ibeg, iend;	/* start/stop index limits for c.r. data loop. */
int	irad1, irad2;
int	irec;		/* Data file record pointer. */
int	ival;		/* Temporary brightness variable. */
int	iyear;		/* Year for synoptic map display. */
int	jj;		/* General purpose index. */
int	ncr;			/* Carrington rotation number. */
int	ncrot = 0;		/* First carrington rotation in display. */
int	ncrot0;		/* First carrington rotation in data file. */
int	ndoye, ndoyw;	/* Truncated (integer) DOY for right map edge. */
int	nlat = 180;	/* # latitude pixels. */
int	nlong;		/* # longitude pixels. */
int	numtry = 0;	/* # retries to select # legal Carrington rotations. */
int	nwrite;		/* # bytes written to east/west synoptic files. */
long	offset;		/* # logical records to skip in synoptic data file. */
int	ipixsam;	/* # pixels per sample. */
int	ipixdoy;	/* # pixels per DOY. */
int	ipixrot;	/* # pixels per rotation. */
int	samdoy = 2;	/* # samples per DOY. */
int	sammap;		/* # samples per map. */
int	sizST;		/* Size (bytes) of data record. */
int	xstart, xstop;		/* start/stop x-location for synoptic strips. */
int	x1prev = 0;		/* Previous IRIS x-coordinate of data. */
int	x2prev = 0;		/* Previous IRIS x-coordinate of data. */
int	xd1;	/* x east.          */
int	xr1;	/* Right x east.    */
int	xl1;	/* Left  x east.    */
int	yd1;	/* y east.          */
int	yt1;	/* Top y east.      */
int	yb1;	/* Bottom y east.   */
int	xd2;	/* x west.          */
int	xr2;	/* Right x west.    */
int	xl2;	/* Left  x west.    */
int	yd2;	/* y west.          */
int	yt2;	/* Top y west.      */
int	yb2;	/* Bottom y west.   */
int	xloc1;	/* Grinnell x location for a given DOY in a rotation. */
int	xloc2;	/* Grinnell x location for a given DOY in a rotation. */
int	xwidth; /* # pixels width of synoptic strips for individual rotations.*/

char	*index (), *p8;			/* to find year of file	*/
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   /*** Request synoptic map data file name. ***/

   fprintf (stderr,"%s  Version 3.00\n", argv [0]);

   if (argc == 2)
      {
      strcpy (namfil, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [synoptic_filename]\n", argv [0]);
      fprintf (stderr,"enter synoptic file name: ");

      fscanf (stdin, "%s", namfil);
      }

   /*** Open SMM C/P synoptic data file. ***/

   if ((SYN = fopen (namfil,"r")) == NULL)
      {
      fprintf (stderr, "cannot open synoptic file: %s\n", namfil);
      exit (-1);
      }

   /*** Read in first record. ***/

   sizST = sizeof (STT);	/* Size (bytes) of synoptic data structure. */
   if (fseek (SYN, 0L, 0) == -1)
      {
      fprintf (stderr, "seek error: BOF\n");
      exit (-1);
      }
   if (fread ((char *) ST, sizST, 1, SYN) == 0)
      {
      fprintf (stderr, "read error: BOF\n");
      exit (-2);
      }
   /*
    * Find starting rotation number from file name.
    * Fix this later in rational rewrite of whole business.
    */
   p8 = index (namfil, '8');
   if (*(p8+1) == '0')  iyear = 80;
   if (*(p8+1) == '4')  iyear = 84;
   if (*(p8+1) == '5')  iyear = 85;
   if (*(p8+1) == '6')  iyear = 86;
   if (*(p8+1) == '7')  iyear = 87;
   if (*(p8+1) == '8')  iyear = 88;
   if (*(p8+1) == '9')  iyear = 89;

   for (i = 0; i < strlen (namfil); i++)  if (namfil [i] == '.')  idot = i;
   strncat (crad, &namfil [idot+2], 2);
   irad1 = ((int) (crad [0]) & 0377) - 060;
   irad2 = ((int) (crad [1]) & 0377) - 060;
   radius = (float) irad1 + ((float) irad2) * 0.1;
   fprintf (stdout, "crad:%s   idot:%d\n", crad, idot);
   fprintf (stdout, "irad1:%d   irad2:%d   radius: %5.2f\n", irad1, irad2, radius);
   fprintf (stdout, "iyear:%d   ST->et: %f\n", iyear, ST->et);

   if (ST->et < 1000.)  filedoy0 = ST->et;
   if (ST->et >= 1000.)  filedoy0 = ST->et - 2000.;
   filerot0 = synrot (iyear, filedoy0) + 0.25;
   ncrot0 = (int) filerot0;

   /*** Get scaling value. ***/

   fprintf (stderr, "Enter maximum value for scale:  ");
   fscanf  (stdin,  "%f", &zmax);
   zscale = (float) irismax / zmax;
   numtry = 0;

   fprintf (stderr, "First rotation in map: %d\n", ncrot0);

maptype:

   fprintf (stderr, "Enter C.R. or # rotations to be displayed [1-7]: ");
   fscanf  (stdin,  "%d", &NROT);
   if (NROT > 1000)
      {
      ncrot = NROT;
      NROT = 1;
      indiv [0] = 'y';
      ipixsam = 5;
      }
   else
   switch (NROT)
      {
      case 1: indiv [0] = 'y';  ipixsam = 5;  break;
      case 2: indiv [0] = 'n';  ipixsam = 3;  break;
      case 3: indiv [0] = 'n';  ipixsam = 2;  break;
      case 4: indiv [0] = 'n';  ipixsam = 1;  break;
      case 5: indiv [0] = 'n';  ipixsam = 1;  break;
      case 6: indiv [0] = 'n';  ipixsam = 1;  break;
      case 7: indiv [0] = 'n';  ipixsam = 1;  break;
      default:
               fprintf (stderr, "Invalid # rotations.  Try again.");
               numtry++;
               if (numtry < 3)  goto maptype;
               else exit (-1);
      }

   if (indiv [0] != 'y')
      {

/* * * * * * * * * * * * * Multiple Rotation Map. * * * * * * * * * * * * * * */

      fprintf (stderr, "First rotation in map: %d\n", ncrot0);
      fprintf (stderr, "Enter starting carring rotation number : ");
      fscanf  (stdin,  "%d", &ncrot);
/*
 *      fprintf (stderr, "Enter radius [Rsun]: ");
 *      fscanf  (stdin,  "%f", &radius);
 */

      /*** Initialization. ***/

      doymap = 0;
      for (i=0; i<NROT; i++)
         {
         crdur [i] = syndur (ncrot + i); 
         doymap += crdur [i]; 
         }
      doymap /= (float) NROT;
      samrot = doymap * (float) samdoy;
      sammap = doymap * samdoy;
      ipixrot = ipixsam * samrot;
      ipixdoy = ipixsam * samdoy;
      nlong = (int) (NROT);		/* # longitudinal pixel slices. */
      xr1 = 182 + nlong/2;		/* Upper right corner of east map. */
      yt1 = 0;
      xl1 = xr1 - nlong + 1;		/* Upper left corner of east map. */
      yb1 = yt1 - 179;

      xr2 = xr1;			/* Upper right corner of west map. */
      yt2 = 511 - 231;
      xl2 = xr2 - nlong + 1;		/* Upper left corner of west map. */
      yb2 = yt2 - 179;

      xd1 = xr1; yd1 = yt1;
      xd2 = xr2; yd2 = yt2;

      /*** Map initialization for multiple rotation synoptic map. ***/

      xwidth = ipixsam * 2;
      xtrarot = 0.0;

      rotbeg = (float) ncrot0;
      rotend = rotbeg + (float) MAXROT;
      fprintf (stdout, "rotbeg: %7.2f   rotend: %7.2f\n", rotbeg, rotend);

      crbeg = (float) ncrot - xtrarot;
      crend = (float) ncrot + xtrarot + NROT;
      fprintf (stdout, "crbeg: %8.3f   crend: %8.3f   xtrarot: %9.4f\n",
	       crbeg, crend, xtrarot);

      if (crbeg < rotbeg)  crbeg = rotbeg;
      if (crend > rotend)  crend = rotend;

      ibeg = (int) ((crbeg - filerot0) * MSROT) - 2;
      iend = (int) ((crend - filerot0 + 0.5) * MSROT) + 2;
      if (ibeg < 0)  ibeg = 0;
      if (iend > MSROT * MAXROT)  iend = MSROT * MAXROT;
      fprintf (stdout, "ibeg: %d   iend:%d\n", ibeg, iend);

      ieast = 0;
      iwest = 0;

      /*** Data loop for multiple rotation synoptic map. ***/

      x1prev = xr1;
      for (irec=ibeg; irec<iend; irec++)
         {

         /*** Read in record ***/

         offset = (long) (irec * sizST);
         if (fseek (SYN, offset, 0) == -1)
            {
            fprintf (stderr, "seek error: EOF at record %d\n", irec);
            goto endmult;
	    }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
	    {
            fprintf (stderr, "read error: EOF at record %d\n", irec);
            goto endmult;
            }
         fprintf (stdout, "irec: %3d   net:%7.2f   et:%7.2f   set:%7.2f\n",
                  irec, ST->net, ST->et, ST->set);
         fprintf (stdout, "            swt:%7.2f   wt:%7.2f   nwt:%7.2f\n",
                  ST->swt, ST->wt, ST->nwt);

         /*** East ok?  Then insert. ***/

         if ((ST->ec >= crbeg) && (ST->ec < crend))
            {
            ncr = (int) ST->ec;
            icr = ncr - ncrot;
            xloc1 = locbeg [icr]
		    - (int) ((ST->ec - (float) ncr) * crdur [icr] * ipixdoy);
/*
 *	    if (ieast > 0)  xwidth = x1prev - xloc1;  else  xwidth = ipixsam + 1;
 */
            printf("ec: %7.2f   icr: %d   locbeg: %d   xloc1: %d  x1prev: %d\n",
                   ST->ec, icr, locbeg [icr], xloc1, x1prev);
	    ieast += 1;
	    x1prev = xloc1;

            /*** Screen index of slot. ***/

            for (jj=0; jj<180; jj++)
               {
               ival = (int) ST->th [jj] * zscale;
               if (ival > irismax)  ival = irismax;
               ibuf [jj] = (char) ival;
               }

            xstart = xloc1;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl1 || jj > xr1) continue;
               wcol (eastbuf, nxsyn, nysyn, ibuf, jj, yt1, nlat, dir1);
               }
            }

         /*** West ok?  Then insert. ***/

         if ((ST->wc >= crbeg) && (ST->wc < crend))
            {
            ncr = (int) ST->wc;
            icr = ncr - ncrot;
            xloc2 = locbeg [icr]
		    - (int) ((ST->wc - (float) ncr) * crdur [icr] * ipixdoy);
            printf("wc: %7.2f  icr: %d  locbeg: %d  xloc2: %d  x2prev: %d\n",
                   ST->wc, icr, locbeg [icr], xloc2, x2prev);
	    iwest += 1;
	    x2prev = xloc2;

            /*** Screen index of slot. ***/

            for (jj=181; jj<361; jj++)
               {
               ival = (int) ST->th [jj] * zscale;
               if (ival > irismax)  ival = irismax;
               ibuf [541 - jj] = (char) ival;
               }

            xstart = xloc2;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl2 || jj > xr1)  continue;
               wcol (westbuf, nxsyn, nysyn, &ibuf [180], jj, yt2, nlat, dir1);
               }
            }	/*** End of west limb loop. ***/
         }	/*** End of multiple rotation data loop. ***/

endmult:
      fprintf (stderr, "End of multiple rotation map.\n");
      }
   else



/* * * * * * * * * * * * * * * Single Rotation Map * * * * * * * * * * * * * */
      {

      /*** Request starting Carrington rotation. ***/

      if (ncrot < 1000)
         {
         fprintf (stderr, "First rotation in map data file: %d\n", ncrot0);
         fprintf (stderr, "Enter rotation number : ");
         fscanf  (stdin,  "%d", &ncrot);
         }
      fprintf (stdout, "ncrot = %d start = %d\n", ncrot, ncrot0);

      /*** Create East & West output files. ***/

      sprintf (nameast, "cp%04de.syn", ncrot);
      sprintf (namwest, "cp%04dw.syn", ncrot);

      if ((fdeast = creat (nameast, 0644)) <= 0)
         {
         fprintf (stderr, "Unable to create file: %s\n", nameast);
         exit (-1);
         }
printf ("fdeast: %d\n", fdeast);
      close (fdeast);
      fdeast = open (nameast, 1);

      if ((fdwest = creat (namwest, 0644)) <= 0)
         {
         fprintf (stderr, "Unable to create file: %s\n", namwest);
         exit (-1);
         }
printf ("fdwest: %d\n", fdwest);
      close (fdwest);
      fdwest = open (namwest, 1);
printf ("fdeast: %d   fdwest: %d\n", fdeast, fdwest);

/*
 *      fprintf (stderr, "Enter radius [Rsun]: ");
 *      scanf ("%f", &radius);
 */

      doyrot = syndur (ncrot); 	/* # days in rotation `ncrot'. */
      samrot = doyrot * samdoy;	/* # samples in rotation `ncrot'. */
      ipixrot = 360;		/* # pixels per rotation (one per degree). */
      pixsam = ((float) ipixrot / samrot);		/* # pixels / sample. */
      ipixsam = (int) ((float) ipixrot / samrot);	/* # pixels / sample. */

      xtrarot = 0.0;
      doymap = doyrot + 2 * xtrarot * samrot;
      nlong = (int) ((NROT + xtrarot * 2) * ipixrot);
      xr1 = 182 + nlong/2;	/* Upper right corner of east map. */
      yt1 = 0;
      xl1 = xr1 - nlong + 1;	/* Upper left corner of east map. */
      yb1 = yt1 - 179;

      xr2 = 182 + nlong/2;	/* Upper right corner of west map. */
      yt2 = 0;
      xl2 = xr2 - nlong + 1;	/* Upper left corner of west map. */
      yb2 = yt2 - 179;

      xwidth = ipixsam * 2;
      xd1 = xr1; yd1 = yt1;
      xd2 = xr2; yd2 = yt2;

      /* * * * * * * * * * * * * * * * * * * * * * * * *
       *   Set up starting DOY values and x locations. *
       * * * * * * * * * * * * * * * * * * * * * * * * */

      crbeg = (float) ncrot - xtrarot;
      crend = (float) ncrot + xtrarot + 1.0;

      doybege = syndoy (iyear, ncrot) - syndur (ncrot) * (0.25 + xtrarot);
      doybegw = syndoy (iyear, ncrot) + syndur (ncrot) * (0.25 - xtrarot);

      if ((iyear/4) * 4 == iyear)  doymax = 366.0;  else doymax = 365.0;
      if (doybege > doymax)  doybege -= doymax;
      if (doybegw > doymax)  doybegw -= doymax;
      printf ("doybege: %f   doybegw: %f\n", doybege, doybegw);

      /*** East DOY initialization. ***/

      ndoye = (int) doybege;
      ibege = ndoye + 1;
      iende = ndoye + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
      posdoye = (float) xr1 + pixsam * samdoy * (doybege - (float) ndoye);
      printf ("ibege: %3d   iende: %3d   NROT: %d   xtrarot: %f   doyrot: %f\n",
               ibege, iende, NROT, xtrarot, doyrot);

      /*** West DOY initialization. ***/

      ndoyw = (int) doybegw;
      ibegw = ndoyw + 1;
      iendw = ndoyw + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
      posdoyw = (float) xr1 + pixsam * samdoy * (doybegw - (float) ndoyw);

      /*** Map initialization. ***/

      fprintf (stdout, "crbeg: %8.3f   crend: %8.3f   xtrarot: %9.4f\n",
	       crbeg, crend, xtrarot);

      if (fseek (SYN, 0L, 0) == -1)
         {
         fprintf (stderr, "seek error: BOF\n");
         exit (-1);
         }
      if (fread ((char *) ST, sizST, 1, SYN) == 0)
         {
         fprintf (stderr, "read error: BOF\n");
         exit (-2);
         }

      if (ST->et < 1000.)  filedoy0 = ST->et;
      if (ST->et >= 1000.)  filedoy0 = ST->et - 2000.;
      idoy0 = (int) filedoy0;

      ibeg = ((int) doybege - idoy0) * samdoy
	     - 4;	/* Start record in data file. */
      if (ibeg < 0)  ibeg = 0;
      iend = ((int) doybegw - idoy0) * samdoy + (1.0 + xtrarot * 2.0) * samrot
	     + 4;	/* End record in data file. */

      printf ("crbeg : %8.3f   crend : %8.3f\n", crbeg, crend);
      printf ("idoy0: %d   ibeg: %d   iend: %d\n", idoy0, ibeg, iend);

      /*** Data loop for single rotation maps. ***/

      for (irec=ibeg; irec<iend; irec++)
         {

         /*** Read in record ***/

         offset = (long) (irec * sizST);
         if (fseek (SYN, offset, 0) == -1)
            {
            fprintf (stderr, "seek error: EOF at record %d\n", irec);
            goto endmap1;
	    }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
	    {
            fprintf (stderr, "read error: EOF at record %d\n", irec);
            goto endmap1;
            }
         fprintf (stdout, "irec: %3d   net:%7.2f   et:%7.2f   set:%7.2f\n",
                  irec, ST->net, ST->et, ST->set);
         fprintf (stdout, "           swt:%7.2f   wt:%7.2f   nwt:%7.2f\n",
                  ST->swt, ST->wt, ST->nwt);

         /*** East ok?  Then insert. ***/

         if ((ST->ec >= crbeg) && (ST->ec < crend))
            {
            xloc1 = xr1 - (int) ((ST->ec - crbeg) * ipixrot);
            printf("ec = %8.3f   xloc1 = %d  x1prev = %d\n", ST->ec, xloc1, x1prev);
	    x1prev = xloc1;

            /*** Screen index of slot. ***/

            for (jj=0; jj<180; jj++)
               {
               ival = (int) ST->th [jj] * zscale;
               if (ival > irismax)  ival = irismax;
               ibuf [jj] = (char) ival;
               }

            xstart = xloc1;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl1 || jj > xr1) continue;
               wcol (eastbuf, nxsyn, nysyn, ibuf, jj, yt1, nlat, dir1);
               }
            }

         /*** West ok?  Then insert. ***/

         if ((ST->wc >= crbeg) && (ST->wc < crend))
            {
            xloc2 = xr2 - (int) ((ST->wc - crbeg) * ipixrot);
            fprintf (stdout, "wc = %8.3f   xloc2 = %d  x2prev = %d\n",
		     ST->wc, xloc2, x2prev);
	    x2prev = xloc2;

            /*** Screen index of slot. ***/

            for (jj=180; jj<361; jj++)
               {
               ival = (int) ST->th [jj] * zscale;
               if (ival > irismax)  ival = irismax;
               ibuf [541 - jj] = (char) ival;
               }

            xstart = xloc2;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl2 || jj > xr1)  continue;
               wcol (westbuf, nxsyn, nysyn, &ibuf [180], jj, yt2, nlat, dir1);
               }
            }  /* West Limb. */
         } /* Data loop. */

endmap1:
      fprintf (stderr, "End of single rotation map.\n");

      }		/* End of single rotation map. */
   nwrite = write (fdeast, eastbuf, nxsyn * nysyn);
   printf ("fdeast> nwrite: %d\n", nwrite);

   nwrite = write (fdwest, westbuf, nxsyn * nysyn);
   printf ("fdwest> nwrite: %d\n", nwrite);

   close (fdeast);
   close (fdwest);
   exit (0);
}
