/* dnadj89.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Adjust DN (raw vidicon intensity) for predicted value at DOY 205.   *
 * The transfer curve for year 1989 is on DOY 205.                     *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Solar Maximum Mission       Coronagraph/Polarimeter	               *
 * High Altitude Observatory (HAO)			               *
 * National Center for Atmospheric Research (NCAR) 	               *
 * SGI 4D/25 G   UNIX IRIX 4.0.5   C programming language              *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - *
 * Andrew L. Stanger				 	               *
 * 20 July 1995                                                        *
 * 26 July 1995: compute DN for fdoy for each cal.mir. exposure.       *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

dnadj89 (fdoy, dn, tcdn)

float fdoy ;			/* Image day-of-year (includes fraction). */
int dn ;			/* pixel DN intensity value in coronal image. */
int *tcdn ;			/* pixel intensity for transfer curve date. */

{
   /*** offset and slope of image average DN level for Cal.Mir. exposures.
    *** Corresponding Exposure durations:
    *** min,0,1,2,3,4,5,6,8,10,12,14,17,20,25,30,max
    ***/

   static float dnoff [17]   = {   0.0,   5.9,  22.4,  45.5,  68.4,
				  89.1, 106.6, 123.0, 149.6, 163.9,
				 174.8, 186.4, 202.0, 212.3, 222.4,
				 230.2, 256.0} ;

   static float dnslope [17] = {  0.0068,  0.0068, -0.0160, -0.0503, -0.0855,
				 -0.1130, -0.1316, -0.1529, -0.1838, -0.1791,
				 -0.1432, -0.1363, -0.1487, -0.1399, -0.1073,
				 -0.1088, -0.1088} ;

   float fdn ;
   float dnf, dno, dns ;
   float tcdn1, tcdn2 ;
   float tcdoy = 205.3 ;	/*** DOY date of 1989 transfer curve. ***/

   int i, idn1, idn2 ;

   fdn = (float) dn ;

   if (dn <= 0 || fdoy >= tcdoy)
      {
/*      printf ("dn: %3d\n", dn) ; */
      *tcdn = dn ;
      return (-1) ;
      }

   /*** Find two table entries between which the vidicon DN level falls. ***/

   idn1 = 0 ;
   idn2 = 1 ;
   for (i = 0; i < 16; i++)
      {
      tcdn1 = dnslope [i] * fdoy + dnoff [i] ;		/* calmir DN for fdoy.*/
      tcdn2 = dnslope [i+1] * fdoy + dnoff [i+1] ;
      if (fdn > tcdn1  && fdn < tcdn2)  idn1 = i ;
      }
   idn2 = idn1 + 1 ;

   /*** Interpolate to derive DN offset and slope for a given image date. ***/

   /*** Interpolate to determine DN slope and intercept. ***/
   /*** Use input DN and interpolate between two calmir curves (DN vs. DOY).***/

   dnf = (fdn - dnoff [idn1]) / (dnoff [idn2] - dnoff [idn1]) ;
   dno = dnoff [idn1] + dnf * (dnoff [idn2] - dnoff [idn1]) ;
   dns = dnslope [idn1] + dnf * (dnslope [idn2] - dnslope [idn1]) ;

/*
 *   dn1 = slope * t1 + off
 *   dn2 = slope * t2 + off
 *   ----------------------
 *   dn2 = dn1 + slope * (t2 - t1)
 */

   /*** Now, compute predicted value of DN level for the date
    *** of the calibration mirror transfer curve images.
    ***/

   *tcdn = (int) (fdn + dns * (tcdoy - fdoy)) ;

/*
   printf ("idn1/2: %2d %2d  dnf: %7.2f  dns: %7.4f  fdoy: %7.2f  dn: %3d  tcdn: %3d\n",
	    idn1, idn2, dnf, dns, fdoy, dn, *tcdn) ;
 */

   return (0) ;
}
