/* gl_whdash
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write a horizontal dashed line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_whdash (xbeg, xend, yloc, dashlen, cmap)

int xbeg, xend;			/* Start/End x location. */
int yloc;			/* y location. */
int dashlen; 			/* Dash length (pixels). */
int cmap;			/* Color map index. */

{
   int x1, x2;
/*
 *   printf ("gl_whdash> xbeg:%3d   xend:%3d   yloc:%3d   dashlen:%d\n",
 *            xbeg, xend, yloc, dashlen);
 */

   /*** Write short vertical lines at left & right of CME line. ***/

   gl_wvec (xbeg-2, yloc, xbeg+2, yloc, cmap);
   gl_wvec (xend-2, yloc, xend+2, yloc, cmap);

   /*** Write horizontal dashed line to mark CME. ***/

   for (x1 = xbeg+2; x1 < xend; x1 += dashlen*2)
      {
      x2 = x1 + dashlen - 1;
      if (x2 > (xend - 2))  x2 = xend - 2;
      gl_wvec (x1, yloc, x2, yloc, cmap);
/*
 *      printf ("whdash> x1:%3d   x2:%3d   yloc:%3d\n", x1, x2, yloc);
 */
      }

   return (0);
}



/* gl_wvdash
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write a vertical dashed line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_wvdash (xloc, ybeg, yend, dashlen, cmap)

int xloc;			/* x location. */
int ybeg, yend;			/* Start/End y location. */
int dashlen; 			/* Dash length (pixels). */
int cmap;			/* Color map index. */

{
   int y1, y2;

   /*** Write short horizontal lines at top and bottom of CME line. ***/

   gl_wvec (xloc-2, ybeg, xloc+2, ybeg, cmap);
   gl_wvec (xloc-2, yend, xloc+2, yend, cmap);

   /*** Write vertical dashed line to mark CME. ***/

   for (y1 = ybeg+2; y1 < yend; y1 += dashlen*2)
      {
      y2 = y1 + dashlen - 1;
      if (y2 > (yend - 2))  y2 = yend - 2;
      gl_wvec (xloc, y1, xloc, y2, cmap);
      }

   return (0);
}
