/* gl_wlin.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write a line of pixel data in current GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 19 November 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

gl_wlin (buf, xloc, yloc, numpix, dir1)

unsigned char *buf;	/* Character string to be written. */
int xloc, yloc;		/* Pixel location for start of pixel data. */
int numpix;		/* # pixels to write. */
char *dir1;		/* direction of write: 'r', 'l', 'u', 'd'. */

{
   unsigned short *sbuf;	/* Short int buffer. */
   unsigned short soff = 512;	/* Use 3rd logical colormap. */
   int i;
   int nbypx = 1;
   int xll, yll;
   int xur, yur;

/*
 *   fprintf (stdout, ":-:xloc: %3d  yloc: %3d  dir1: %s\n", xloc, yloc, dir1);
 *   fflush (stdout);
 */

   switch ((int) *dir1)
      {
      case 'r':						/* rightward. */
		xll = xloc;
		yll = yloc;
		xur = xloc + numpix - 1;
		yur = yloc;
		break;
      case 'l':						/* leftward. */
		xll = xloc - numpix + 1;
		yll = yloc;
		xur = xloc;
		yur = yloc;
		revorder (buf, nbypx, numpix);
		break;
      case 'u':						/* upward. */
		xll = xloc;
		yll = yloc;
		xur = xloc;
		yur = yloc + numpix - 1;
                break;
      case 'd':						/* downward. */
		xll = xloc;
		yll = yloc - numpix + 1;
		xur = xloc;
		yur = yloc;
		revorder (buf, nbypx, numpix);
		break;
      default:
		fprintf (stderr, "Invalid direction: %s\n", dir1);
		return (-1);
      }


   /*** Copy byte buffer to short int buffer. ***/

   sbuf = (unsigned short *)  malloc (numpix * sizeof (short));
   for (i = 0; i < numpix; i++)
      *(sbuf + i) = (unsigned short) *(buf + i) + soff;

   /*** Write pixel data. ***/

   rectwrite (xll, yll, xur, yur, sbuf);

   free (sbuf);
   return (0);
}


/* revorder
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse order of elements in an array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
revorder (buf, nbypx, nval)

char *buf;		/* Pointer to array. */
int nbypx;		/* # bytes per element. */
int nval;		/* # elements in array. */

{
   int i, ni;
   int ihalf, imax;
   char cval;
   short sval;
   float fval;
   short *sbuf;
   float *fbuf;

   printf ("   revorder> nbypx: %d   nval: %d\n", nbypx, nval);
   sbuf = (short *) buf;
   fbuf = (float *) buf;

   /*** Reverse order of elements of array. ***/

   imax = nval - 1;
   ihalf = nval / 2;

   if (nbypx == 1)				/* Byte image. */
      {
      for (i = 0; i < ihalf; i++)
         {
         ni = imax - i; 
         cval = *(buf + i);
         *(buf + i) = *(buf + ni);
         *(buf + ni) = cval;
/*
 *      printf ("***revorder> i: %3d   buf[%3d]: %3d   buf[%3d]: %3d\n",
 *              i, i, *(buf+i), ni, *(buf+ni));
 */
         }
      }
   else
   if (nbypx == 2)				/* Short integer image. */
      {
      for (i = 0; i < ihalf; i++)
         {
         ni = imax - i; 
         sval = *(sbuf + i);
         *(sbuf + i) = *(sbuf + ni);
         *(sbuf + ni) = sval;
         }
      }

   else
   if (nbypx == 4)				/* Floating point image. */
      {
      for (i = 0; i < ihalf; i++)
         {
         ni = imax - i; 
         fval = *(fbuf + i*nval);
         *(fbuf + i) = *(fbuf + ni);
         *(fbuf + ni) = fval;
         }
      }

   return (0);
}
