/* synexam.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Synoptic data file examiner.
 * Also allows user to clear a record.
 * Uses shorter synoptic data files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 2 June 1988
 * 4 August 1988: Change angles reported; start with 30 instead of 0 degrees.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "synst.h"

struct S STT;

main()
{
static char namfil[80];
static char answer [80];
static char zip[2];
FILE *SYN, *fopen();
register struct S *ST = &STT;
int cmd;
int doy0;
int doybeg, doyend;
int i;
int irec, irec1, irec2;
int sizST;
long offset;
float doy;

   sizST = sizeof (STT);

   fprintf (stderr, "Enter synoptic data file name: ");
   fscanf  (stdin, "%s", namfil);

   /*** Open synoptic data file, read first record. ***/

   if ((SYN = fopen (namfil, "r+")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namfil);
      exit (-1);
      }

   if (fread ((char *) ST, sizST, 1, SYN) == 0)
      {
      fprintf (stderr, "Unable to read 1st record in file.\n");
      exit (-1);
      }

   doy0 = (int) ST->et;
   if (doy0 > 1000)  doy0 -= 2000;
   fprintf (stderr, "doy0: %d\n", doy0);

   /*** Get option. ***/

start:

   fprintf (stderr, "Choose one: e [examine], c [clear], l [list], or q [quit]: ");
   fscanf  (stdin,  "%s", answer);
   cmd = (int) answer[0];

   switch (cmd)
      {

      /*** Examine specified range of records. ***/

      case 'e':
	 fprintf (stderr, "Enter start & stop DOY's: ");
	 fscanf  (stdin,  "%d%c%d", &doybeg, zip, &doyend);
	 irec1 = (doybeg - doy0) * 2;
	 irec2 = (doyend - doy0) * 2 + 1;
	 fprintf (stdout, "----------------------------------------------------------------------------------------------------------------------------\n");
         fprintf (stdout, "rec      net      et     set     swt      wt     nwt     nec      ec     sec     swc      wc     nwc  30  90 150 210 270 330\n");
	 fprintf (stdout, "----------------------------------------------------------------------------------------------------------------------------\n");
	 offset = irec1 * sizST;
	 fprintf (stderr, "irec1:%d   irec2:%d   offset:%d\n", irec1, irec2, offset);
	 if (fseek (SYN, offset, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # %d  offset: %d\n",
		     irec1, offset);
	    break;
	    }

	 for (irec=irec1; irec<=irec2; irec++)
	    {
            if (fread ((char *) ST, sizST, 1, SYN) == 0)
               {
               fprintf (stderr, "Unable to read record # %d in file.\n", irec);
               exit (-1);
               }
            fprintf (stdout, "%3d %8.2f%8.2f%8.2f%8.2f%8.2f%8.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
	             irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                   ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
		           ST->th[30],   ST->th[90],  ST->th[150],
		           ST->th[210], ST->th[270], ST->th[330]);
	    }
          break;

      /*** Clear a record. ***/

      case 'c':
	 fprintf (stderr, "Enter record to be cleared: ");
	 fscanf  (stdin, "%d", &irec);

         /*** Read record to be cleared and display its contents. ***/

	 offset = irec * sizST;
	 if (fseek (SYN, offset, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # %d  offset: %d\n",
		     irec, offset);
	    break;
	    }
         fread ((char *) ST, sizST, 1, SYN);
         fprintf (stdout, "%3d %8.2f%8.2f%8.2f%8.2f%8.2f%8.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
                  irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
                        ST->th[30],  ST->th[90],  ST->th[150],
                        ST->th[210], ST->th[270], ST->th[330]);

         /*** Clear  record to empty state. ***/

         doy = (float) irec * 0.5 + (float) doy0; 
	 doy = doy + 2000.0;
	 ST->net = ST->et = ST->set = ST->et = ST->swt = ST->wt = ST->nwt = doy;
	 ST->nec = ST->ec = ST->sec = ST->ec = ST->swc = ST->wc = ST->nwc = 0.0;
	 for (i=0; i<361; i++)  ST->th[i] = 0;
	 fseek (SYN, offset, 0);
	 fwrite ((char *) ST, sizST, 1, SYN);

         /*** Read record which has been cleared and display its contents. ***/

	 fseek (SYN, offset, 0);
         fread ((char *) ST, sizST, 1, SYN);
         fprintf (stdout, "%3d %8.2f%8.2f%8.2f%8.2f%8.2f%8.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
	             irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                   ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
		           ST->th[30],  ST->th[90],  ST->th[150],
		           ST->th[210], ST->th[270], ST->th[330]);
	 break;

      /*** List entire file. ***/

      case 'l':
         irec = -1;


	 if (fseek (SYN, 0L, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # 0  offset: 0\n");
	    break;
	    }
         while (fread ((char *) ST, sizST, 1, SYN) != 0)
            {
            irec++;
            if (((irec/50)*50) == irec)
               {
	       fprintf (stdout, "----------------------------------------------------------------------------------------------------------------------------\n");
               fprintf (stdout, "rec      net      et     set     swt      wt     nwt     nec      ec     sec     swc      wc     nwc  30  90 150 210 270 330\n");
	       fprintf (stdout, "----------------------------------------------------------------------------------------------------------------------------\n");
	       }
            fprintf (stdout, "%3d %8.2f%8.2f%8.2f%8.2f%8.2f%8.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
	             irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                   ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
		           ST->th[30],  ST->th[90],  ST->th[150],
		           ST->th[210], ST->th[270], ST->th[330]);
            }
         break;

      /*** Quit. ***/

      case 'q':
	 exit(0);

      default:
	 break;
      }
   goto start;

}
