/* sd2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display SMM C/P Synoptic Map data on SGI IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Use with synoptic map files containing photometrically calibrated data.
 * Companion program for use with files generated by `sg'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Original code written by Rainer Illing.
 * Revisions by Andrew L. Stanger
 * 29 April 1987
 * 5 May 1987: Fix for 1980 synoptic map.
 * 14 January 1988: Revise to be compatible with `sg3.c'.
 * 25 May 1988: modify for compatibility with `sg3a.c'.
 * 2 June 1988: non-existent data bins have times > +1000.
 * 3 August 1988: Remove photometric scaling adjustments.  All synoptic files
 *                should contain photometrically calibrated data.
 * 23 June 1989: Sun-4 HIPS version.
 * 10 September 1990 : For single-rotation maps, use square aspect ratio.
 *  9 April 1991: adapt for SGI IRIS.
 *  8 January 1992: minor change to ibuf saturation logic.
 *  4 November 1993: add zmin for intensity scaling.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include	<stdio.h>
#include	<math.h>
#include	<whip.h>
#include	"/usr/people/smm/lib/include/cplib.h"
#include	"/usr/people/smm/lib/include/cp_color.h"
#include	"synst.h"

struct S STT;

main (argc, argv)

int argc;
char *argv [];
{
extern float synrot ();	/* Synodic rotation for a given DOY. */
extern float syndoy ();	/* Synodic day-of-year for start of a given rotation. */
extern float syndur ();	/* Synodic rotation duration. */
register struct	S	*ST = &STT;	/* pointer to synoptic record. */
FILE	*SYN;				/* file pointer. */
static	char	dir1[2] = "d";
static  char	crad [3];
static	char	crot [8];
static	char	crs [6], sdoy [4];
static  char	namfil [80], answer [16];
static  char	indiv [2];		/* Individual maps. */
static	char	labdoy [] = "I M A G E   D A Y - O F - Y E A R";
static	char	lablong [] = "L O N G I T U D E";
static	char	labrot [] = "C A R R I N G T O N   R O T A T I O N";
static  char	title [40] = "198x CORONAL SYNOPTIC MAP : x.xx Rsun";
static  char	subtit [80];		/* Sub-title character string. */
static  char	ibuf [364];		/* Intensity buffer. */
static	char	irismin = 0;
static	char	irismax = 249;

static	float	crdur [16];	/* Duration of carrington rotations. */
float	crbeg, crend;	/* Rotation begin/end for synoptic map display. */
float	doye, doyw, doybege, doybegw, doymax, doydel;
float	fcrot;
float	filedoy0;	/* DOY at start of synoptic map data file. */
float	filerot0;	/* Rotation (cmp) at start of synoptic map data file. */
float	doy0;		/* DOY for first sample in data file. */
float	doymap;		/* Map duration in units of days. */
float	doyrot;		/* rotation duration in units of days. */
float	pixsam;		/* # pixels per sample. */
float	posdoye, posdoyw;
float	radius;
float	rotbeg, rotend;	/* Rotation begin/end for synoptic data file. */
float	samrot;		/* # samples per rotation. */
float	zmax = 250.0;
float	zt = 1.0 / 80.0;
float	zm = 200.0;
float	zscale;
float	xtrarot = 0.0;	/* Extra display beyond limits of a given rotation. */

static int xbox1 [5], ybox1 [5], xbox2 [5], ybox2 [5], xl [2],yl [2];
static int locbeg [16];
int	MSROT = 55;	/* Maximum # samples per rotation. */
int	MAXROT = 15;	/* Maximum # carrington rotations in syngen data file.*/
int	NROT = 6;	/* # carrington rotations to be displayed in map. */

int FB = 0;		/* default: 512x512 byte frame buffer (colormap mode).*/
int fast = 1;		/* Parameter for fb_clear. */
float bkgcol = 0.0;	/* Grey level for fb_clear. */

int	chang = 0;	/* character string angle (w.r.t. +x axis). */
int	idoy0;		/* DOY for first sample in data file. */
int	i;
int	ibege, ibegw;	/* start index for east/west DOY loops. */
int	iende, iendw;	/* end index for east/west DOY loops. */
int	icr;		/* Carrington rotation index. */
int	idot;
int	ieast, iwest;
int	ibeg, iend;	/* start/stop index limits for c.r. data loop. */
int	imgch;		/* image channel.   */
int	irad1, irad2;
int	irec;		/* Data file record pointer. */
int	ival;		/* Temporary brightness variable. */
int	iyear;		/* Year for synoptic map display. */
int	jj;		/* General purpose index. */
int	lxcre, lycre;	/* screen coordinates of east Carrington rotations. */
int	lxcrw, lycrw;	/* screen coordinates of west Carrington rotations. */
int	locdoye, locdoyw;	/* screen coordinates for DOY tick marks. */
int	lxdoye, lydoye;		/* screen coordinates for east DOY string. */
int	lxdoyw, lydoyw;		/* screen coordinates for west DOY string. */
int	ncr;		/* Carrington rotation number. */
int	ncrot = 0;		/* First carrington rotation in display. */
int	ncrot0;		/* First carrington rotation in data file. */
int	ndoye, ndoyw;	/* Truncated (integer) DOY for right map edge. */
int	nlat = 180;	/* # latitude pixels. */
int	nlong;		/* # longitude pixels. */
int	numecar;	/* East Carrington rotation number. */
int	numwcar;	/* West Carrington rotation number. */
int	numtry = 0;	/* # retries to select # legal Carrington rotations. */
int	idoye, idoyw;	/* Indices for DOY annotation loops. */
long	offset;		/* # logical records to skip in synoptic data file. */
int	ipixsam;	/* # pixels per sample. */
int	ipixdoy;	/* # pixels per DOY. */
int	ipixrot;	/* # pixels per rotation. */
int	samdoy = 2;	/* # samples per DOY. */
int	sammap;		/* # samples per map. */
int	size = 1;	/* character size. */
int	swshx =  6;	/* horizontal size 1. */
int	swshy =  9;	/* vertical   size 1. */
int	swdhx =  6;	/* horizontal size 1. */
int	swdhy =  9;	/* vertical   size 1. */
int	dwshx = 12;	/* horizontal size 2. */
int	dwshy = 18;	/* vertical   size 2. */
int	dwdhx = 12;	/* horizontal size 2. */
int	dwdhy = 18;	/* vertical   size 2. */
int	sizST;		/* Size (bytes) of data record. */
int	xinc = 7;	/* pixel increment for character strings. */
int	xx = 0, xxl = 0, xxr = 0;	/* general x-location variables. */
int	xstart, xstop;		/* start/stop x-location for synoptic strips. */
int	x1prev, x2prev;		/* Previous IRIS x-coordinate of data. */
int	xd1;	/* x east.          */
int	xr1;	/* Right x east.    */
int	xl1;	/* Left  x east.    */
int	yy;	/* general purpose y-location variable. */
int	yd1;	/* y east.          */
int	yt1;	/* Top y east.      */
int	yb1;	/* Bottom y east.   */
int	xd2;	/* x west.          */
int	xr2;	/* Right x west.    */
int	xl2;	/* Left  x west.    */
int	yd2;	/* y west.          */
int	yt2;	/* Top y west.      */
int	yb2;	/* Bottom y west.   */
int	xloc1;	/* Grinnell x location for a given DOY in a rotation. */
int	xloc2;	/* Grinnell x location for a given DOY in a rotation. */
int	xwidth; /* # pixels width of synoptic strips for individual rotations.*/
int	xw2;	/* xwidth / 2 */
int	xspan;	/* # additional pixels width needed to span a gap. */
int	yz1, yz2;	/* 0 degree latitude for east & west maps. */

FILE	*NLN, *fopen();			/*   "         file  */
char	*index (), *p8;			/* to find year of file	*/
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int yellow, grey, blue, green, red;
yellow = cp_yellow;
grey   = cp_grey;
blue   = cp_blue;
green  = cp_green;
red    = cp_red;
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   /*** Request synoptic map data file name. ***/

   fprintf (stderr,"%s  Version 3.00\n", argv [0]);

   if (argc == 2)
      {
      strcpy (namfil, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [synoptic_filename]\n", argv [0]);
      fprintf (stderr,"enter synoptic file name: ");

      fscanf (stdin, "%s", namfil);
      }

   /*** Open SMM C/P synoptic data file. ***/

   if ((SYN = fopen (namfil,"r")) == NULL)
      {
      fprintf (stderr, "cannot open synoptic file: %s\n", namfil);
      exit (-1);
      }

   /*** Read in first record. ***/

   sizST = sizeof (STT);	/* Size (bytes) of synoptic data structure. */
   if (fseek (SYN, 0L, 0) == -1)
      {
      fprintf (stderr, "seek error: BOF\n");
      exit (-1);
      }
   if (fread ((char *) ST, sizST, 1, SYN) == 0)
      {
      fprintf (stderr, "read error: BOF\n");
      exit (-2);
      }
   /*
    * Find starting rotation number from file name.
    * Fix this later in rational rewrite of whole business.
    */
   p8 = index (namfil, '8');
   if (*(p8+1) == '0')  iyear = 80;
   if (*(p8+1) == '4')  iyear = 84;
   if (*(p8+1) == '5')  iyear = 85;
   if (*(p8+1) == '6')  iyear = 86;
   if (*(p8+1) == '7')  iyear = 87;
   if (*(p8+1) == '8')  iyear = 88;
   if (*(p8+1) == '9')  iyear = 89;

   for (i = 0; i < strlen (namfil); i++)  if (namfil [i] == '.')  idot = i;
   strncat (crad, &namfil [idot+2], 2);
   irad1 = ((int) (crad [0]) & 0377) - 060;
   irad2 = ((int) (crad [1]) & 0377) - 060;
   radius = (float) irad1 + ((float) irad2) * 0.1;
   fprintf (stdout, "crad:%s   idot:%d\n", crad, idot);
   fprintf (stdout, "irad1:%d   irad2:%d   radius: %5.2f\n", irad1, irad2, radius);
   fprintf (stdout, "iyear:%d   ST->et: %f\n", iyear, ST->et);

   if (ST->et < 1000.)  filedoy0 = ST->et;
   if (ST->et >= 1000.)  filedoy0 = ST->et - 2000.;
   filerot0 = synrot (iyear, filedoy0) + 0.25;
   ncrot0 = (int) filerot0;

   /*** Set up IRIS WHIP software. ***/

   fb_open ();				/* Initialize WHIP. */
   fb_setfb (FB);			/* Select frame buffer. */
   openpl ();
   fb_clear (FB, &bkgcol, fast);	/* Clear frame buffer. */

   /*** Get scaling value. ***/

   fprintf (stderr, "Enter maximum value for scale:  ");
   fscanf  (stdin,  "%f", &zmax);
   zscale = (float) (250.0 / zmax);
   numtry = 0;

   fprintf (stderr, "First rotation in map: %d\n", ncrot0);

maptype:

   fprintf (stderr, "Enter C.R. or # rotations to be displayed [1-7]: ");
   fscanf  (stdin,  "%d", &NROT);
   if (NROT > 1000)
      {
      ncrot = NROT;
      NROT = 1;
      indiv [0] = 'y';
      ipixsam = 5;
      }
   else
   switch (NROT)
      {
      case 1: indiv [0] = 'y';  ipixsam = 5;  break;
      case 2: indiv [0] = 'n';  ipixsam = 3;  break;
      case 3: indiv [0] = 'n';  ipixsam = 2;  break;
      case 4: indiv [0] = 'n';  ipixsam = 1;  break;
      case 5: indiv [0] = 'n';  ipixsam = 1;  break;
      case 6: indiv [0] = 'n';  ipixsam = 1;  break;
      case 7: indiv [0] = 'n';  ipixsam = 1;  break;
      default:
               fprintf (stderr, "Invalid # rotations.  Try again.");
               numtry++;
               if (numtry < 3)  goto maptype;
               else exit (-1);
      }

   if (indiv [0] != 'y')
      {

/* * * * * * * * * * * * * Multiple Rotation Map. * * * * * * * * * * * * * * */

      fprintf (stderr, "First rotation in map: %d\n", ncrot0);
      fprintf (stderr, "Enter starting carring rotation number : ");
      fscanf  (stdin,  "%d", &ncrot);
/*
 *      fprintf (stderr, "Enter radius [Rsun]: ");
 *      fscanf  (stdin,  "%f", &radius);
 */

      /*** Initialization. ***/

      doymap = 0;
      for (i=0; i<NROT; i++)
         {
         crdur [i] = syndur (ncrot + i); 
         doymap += crdur [i]; 
         }
      doymap /= (float) NROT;
      samrot = doymap * (float) samdoy;
      sammap = doymap * samdoy;
      ipixrot = ipixsam * samrot;
      ipixdoy = ipixsam * samdoy;
      nlong = (int) (NROT * ipixrot + 0.5); /* # longitudinal pixel slices. */
      xr1 = 256 + nlong/2;		   /* Upper right corner of east map. */
      yt1 = 440;
      xl1 = xr1 - nlong + 1;	/* Upper left corner of east map. */
      yb1 = yt1 - 179;
      yz1 = yt1 -  90;		/* Zero degrees latitude (east map). */

      xr2 = xr1;		/* Upper right corner of west map. */
      yt2 = 231;
      xl2 = xr2 - nlong + 1;	/* Upper left corner of west map. */
      yb2 = yt2 - 179;
      yz2 = yt2 -  90;		/* Zero degrees latitude (west map). */

      xd1 = xr1; yd1 = yt1;
      xd2 = xr2; yd2 = yt2;

   /*** Write annotation on IRIS display. ***/

      size = 2;
      if (namfil [0] == 'v')
         sprintf (title, "19%d CORONAL SYNOPTIC MAP : %3.1f Rsun RAW",
                  iyear, radius);
      else
         sprintf (title, "19%d CORONAL SYNOPTIC MAP : %3.1f Rsun CAL",
                  iyear, radius);
      xx = 256 - (strlen (title) * dwdhx / 2);
      yy = 511;
      wstr (title, xx, yy, size, yellow);

      size = 2;
      sprintf (subtit, "NCAR / HAO :   SMM  CORONAGRAPH");
      xx = 256 - (strlen (subtit) * dwdhx / 2);
      yy = 511 - 1.25 * dwdhy;
      wstr (subtit, xx, yy, size, red);

      size = 1;
      xx = xl1 - swshx * 3 - 10;
      if (xx < 0) xx = 0;

      wstr ("+90", xx, yt1    +swdhy/2, size, grey);
      wstr ("  0", xx, yt1- 90+swdhy/2, size, grey);
      wstr ("-90", xx, yt1-180+swdhy/2, size, grey);

      wstr ("+90", xx, yt2    +swdhy/2, size, grey);
      wstr ("  0", xx, yt2- 90+swdhy/2, size, grey);
      wstr ("-90", xx, yt2-180+swdhy/2, size, grey);

      xx = xx / 2 - swdhx;
      if (xx < 0)  xx = 0;
      yy = ((yb1 - yt2) / 2 + yt2) + swdhy/2;
      wstr ("L", xx, yy + swdhy * 8, size, grey);
      wstr ("A", xx, yy + swdhy * 6, size, grey);
      wstr ("T", xx, yy + swdhy * 4, size, grey);
      wstr ("I", xx, yy + swdhy * 2, size, grey);
      wstr ("T", xx, yy - swdhy * 0, size, grey);
      wstr ("U", xx, yy - swdhy * 2, size, grey);
      wstr ("D", xx, yy - swdhy * 4, size, grey);
      wstr ("E", xx, yy - swdhy * 6, size, grey);

      xxl = xl1 - 10;
      xxr = xl1 -  4;
      if (xxl < 0) xxl = 0;
      if (xxr < 0) xxr = 0;
      wvec (xxl,yt1,xxr,yt1,grey);	/* Index mark +90 degrees, east limb.  */
      wvec(xxl,yt1-30,xxr,yt1-30,grey);  /* Index mark +60 degrees, east limb.*/
      wvec(xxl,yt1-60,xxr,yt1-60,grey);  /* Index mark +30 degrees, east limb.*/
      wvec(xxl,yt1-90,xxr,yt1-90,grey);  /* Index mark   0 degrees, east limb.*/
      wvec(xxl,yt1-120,xxr,yt1-120,grey);/* Index mark -30 degrees, east limb.*/
      wvec(xxl,yt1-150,xxr,yt1-150,grey);/* Index mark -60 degrees, east limb.*/
      wvec(xxl,yt1-180,xxr,yt1-180,grey);/* Index mark -90 degrees, east limb.*/

      wvec(xxl,yt2,xxr,yt2,grey);	 /* Index mark +90 degrees, west limb.*/
      wvec(xxl,yt2-30,xxr,yt2-30,grey);  /* Index mark +60 degrees, west limb.*/
      wvec(xxl,yt2-60,xxr,yt2-60,grey);  /* Index mark +30 degrees, west limb.*/
      wvec(xxl,yt2-90,xxr,yt2-90,grey);  /* Index mark   0 degrees, west limb.*/
      wvec(xxl,yt2-120,xxr,yt2-120,grey);/* Index mark -30 degrees, west limb.*/
      wvec(xxl,yt2-150,xxr,yt2-150,grey);/* Index mark -60 degrees, west limb.*/
      wvec(xxl,yt2-180,xxr,yt2-180,grey);/* Index mark -90 degrees, west limb.*/

      xx = xr1 + (511 - xr1) / 2 - swdhx * 2;
      if (xx < 0) xx = 0;
      fprintf (stdout, "EAST LIMB  xx: %d   yy: %d\n", xx, yy);
      wstr ("EAST", xx, yz1+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz1-swdhy,   size, grey);
      wstr ("WEST", xx, yz2+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz2-swdhy,   size, grey);

      xx = xl2 + (xr2 - xl2) / 2 - (strlen (labdoy) * swshx) / 2 - 1;
      wstr (labrot, xx, swdhy + 7, size, green);

      /*** Draw boxes around synoptic map image area. ***/

      xbox1[0] = xr1 + 4;
      xbox1[1] = xr1 - nlong - 3;
      xbox1[2] = xbox1[1];
      xbox1[3] = xbox1[0];
      xbox1[4] = xbox1[0];
      ybox1[0] = yt1 + 1;
      ybox1[1] = ybox1[0];
      ybox1[2] = yt1 - 181;
      ybox1[3] = ybox1[2];
      ybox1[4] = ybox1[0];
      wlvec (xbox1, ybox1, 5, grey);

      xbox2[0] = xr2 + 4;
      xbox2[1] = xr2 - nlong - 3;
      xbox2[2] = xbox2[1];
      xbox2[3] = xbox2[0];
      xbox2[4] = xbox2[0];
      ybox2[0] = yt2 + 1;
      ybox2[1] = ybox2[0];
      ybox2[2] = yt2 - 181;
      ybox2[3] = ybox2[2];
      ybox2[4] = ybox2[0];
      wlvec (xbox2, ybox2, 5, grey);

      /*** Draw lines to separate Carrington rotations. ***/

      doy0 = syndoy (iyear, ncrot);
      for (i=0; i<=NROT; i++)
         {
	 xx = xr1 - (int)(syndoy (iyear, ncrot+i) - doy0) * ipixdoy + 1;
         wvec (xx, yt1+1, xx, yt1+swdhy+5, red);
         wvec (xx, yb1-1, xx, yb1-6, red);
         wvec (xx, yt2+1, xx, yt2+6, red);
         wvec (xx, yb2-1, xx, yb2-swdhy-5, red);
         locbeg [i] = xx;
         }

      /*** Write Carrington rotation numbers on Grinnell. ***/

      lxcre = xr1 - (int) (ipixrot / 2.) - xinc * 2 - 1;
      lycre = yt1 + 5 + swdhy;
      lxcrw = xr2 - (int) (ipixrot / 2.) - xinc * 2 - 1;
      lycrw = yb2 - 7;

      numecar = ncrot;
      numwcar = ncrot;

      for (i=0; i<NROT; i++)
	 {
	 printf ("east/west c.r.: %d %d  lxcre:%d lxcrw:%d\n",
		  numecar, numwcar, lxcre, lxcrw);
         sprintf (crs, "%d", numecar);
         wstr (crs, lxcre, lycre, size, green);
         wstr (crs, lxcrw, lycrw, size, green);
	 numecar += 1;
	 numwcar += 1;
	 lxcre -= ipixrot;
	 lxcrw -= ipixrot;
	 }

      /*** Map initialization for multiple rotation synoptic map. ***/

      xwidth = ipixsam * 2;
      xtrarot = 0.0;

      rotbeg = (float) ncrot0;
      rotend = rotbeg + (float) MAXROT;
      fprintf (stdout, "rotbeg: %7.2f   rotend: %7.2f\n", rotbeg, rotend);

      crbeg = (float) ncrot - xtrarot;
      crend = (float) ncrot + xtrarot + NROT;
      fprintf (stdout, "crbeg: %8.3f   crend: %8.3f   xtrarot: %9.4f\n",
	       crbeg, crend, xtrarot);

      if (crbeg < rotbeg)  crbeg = rotbeg;
      if (crend > rotend)  crend = rotend;

      ibeg = (int) ((crbeg - filerot0) * MSROT) - 2;
      iend = (int) ((crend - filerot0 + 0.5) * MSROT) + 2;
      if (ibeg < 0)  ibeg = 0;
      if (iend > MSROT * MAXROT)  iend = MSROT * MAXROT;
      fprintf (stdout, "ibeg: %d   iend:%d\n", ibeg, iend);

      ieast = 0;
      iwest = 0;

      /*** Data loop for multiple rotation synoptic map. ***/

      x1prev = xr1;
      for (irec=ibeg; irec<iend; irec++)
         {

         /*** Read in record ***/

         offset = (long) (irec * sizST);
         if (fseek (SYN, offset, 0) == -1)
            {
            fprintf (stderr, "seek error: EOF at record %d\n", irec);
            goto endmult;
	    }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
	    {
            fprintf (stderr, "read error: EOF at record %d\n", irec);
            goto endmult;
            }
         fprintf (stdout, "irec: %3d   net:%7.2f   et:%7.2f   set:%7.2f\n",
                  irec, ST->net, ST->et, ST->set);
         fprintf (stdout, "            swt:%7.2f   wt:%7.2f   nwt:%7.2f\n",
                  ST->swt, ST->wt, ST->nwt);

         /*** East ok?  Then insert. ***/

         if ((ST->ec >= crbeg) && (ST->ec < crend))
            {
            ncr = (int) ST->ec;
            icr = ncr - ncrot;
            xloc1 = locbeg [icr]
		    - (int) ((ST->ec - (float) ncr) * crdur [icr] * ipixdoy);
/*
 *	    if (ieast > 0)  xwidth = x1prev - xloc1;  else  xwidth = ipixsam + 1;
 */
            printf("ec: %7.2f   icr: %d   locbeg: %d   xloc1: %d  x1prev: %d\n",
                   ST->ec, icr, locbeg [icr], xloc1, x1prev);
	    ieast += 1;
	    x1prev = xloc1;

            /*** Screen index of slot. ***/

            for (jj=0; jj<180; jj++)
               {
               ival = (int) (atan ((float)(ST->th [jj] * zscale * zt)) * zm);
               if (ival > irismax)  ival = irismax;
               ibuf [jj] = (char) ival;
               }

            xstart = xloc1;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl1 || jj > xr1) continue;
               wlin (FB, ibuf, jj, yt1-1, nlat, dir1);
               }
            }

         /*** West ok?  Then insert. ***/

         if ((ST->wc >= crbeg) && (ST->wc < crend))
            {
            ncr = (int) ST->wc;
            icr = ncr - ncrot;
            xloc2 = locbeg [icr]
		    - (int) ((ST->wc - (float) ncr) * crdur [icr] * ipixdoy);
            printf("wc: %7.2f  icr: %d  locbeg: %d  xloc2: %d  x2prev: %d\n",
                   ST->wc, icr, locbeg [icr], xloc2, x2prev);
	    iwest += 1;
	    x2prev = xloc2;

            /*** Screen index of slot. ***/

            for (jj=181; jj<361; jj++)
               {
               ival = (int) (atan ((float)(ST->th [jj] * zscale * zt)) * zm);
               if (ival > irismax)  ival = irismax;
               ibuf [541 - jj] = (char) ival;
               }

            xstart = xloc2;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl2 || jj > xr1)  continue;
               wlin (FB, &ibuf [180], jj, yt2-1, nlat, dir1);
               }
            }	/*** End of west limb loop. ***/
         }	/*** End of multiple rotation data loop. ***/

endmult:

      /*** Draw boxes around synoptic map image area. ***/

      wlvec (xbox1, ybox1, 5, green);	/* Box frame for east limb. */
      wvec (xl1, yt1-90, xr1, yt1-90, green);/* Equatorial line for east limb.*/

      wlvec (xbox2, ybox2, 5, green);	/* Box frame for west limb. */
      wvec (xl2, yt2-90, xr2, yt2-90, green);/* Equatorial line for east limb.*/

      /*** Draw lines to separate Carrington rotations. ***/

      for (i=0; i<=NROT; i++)
         {
	 xx = xr1 - (int)(syndoy (iyear, ncrot+i) - doy0) * ipixdoy + 1;
         wvec (xx, yb1-5, xx, yt1+15, red);
         wvec (xx, yt2+5, xx, yb2-15, red);
         }

      }
   else



/* * * * * * * * * * * * * * * Single Rotation Map * * * * * * * * * * * * * */
      {

      /*** Request starting Carrington rotation. ***/

      if (ncrot < 1000)
         {
         fprintf (stderr, "First rotation in map data file: %d\n", ncrot0);
         fprintf (stderr, "Enter rotation number : ");
         fscanf  (stdin,  "%d", &ncrot);
         }
      fprintf (stdout, "ncrot = %d start = %d\n", ncrot, ncrot0);

/*
 *      fprintf (stderr, "Enter radius [Rsun]: ");
 *      scanf ("%f", &radius);
 */

      doyrot = syndur (ncrot); 	/* # days in rotation `ncrot'. */
      samrot = doyrot * samdoy;	/* # samples in rotation `ncrot'. */
      ipixrot = 360;		/* # pixels per rotation (one per degree). */
      pixsam = ((float) ipixrot / samrot);		/* # pixels / sample. */
      ipixsam = (int) ((float) ipixrot / samrot);	/* # pixels / sample. */

      xtrarot = 0.10;
      doymap = doyrot + 2 * xtrarot * samrot;
      nlong = (int) ((NROT + xtrarot * 2) * ipixrot);
      xr1 = 256 + nlong/2;	/* Upper right corner of east map. */
      yt1 = 442;
      xl1 = xr1 - nlong + 1;	/* Upper left corner of east map. */
      yb1 = yt1 - 179;
      yz1 = yt1 -  90;		/* Zero degrees latitude (east map). */

      xr2 = 256 + nlong/2;	/* Upper right corner of west map. */
      yt2 = 228;
      xl2 = xr2 - nlong + 1;	/* Upper left corner of west map. */
      yb2 = yt2 - 179;
      yz2 = yt2 -  90;		/* Zero degrees latitude (west map). */

      xwidth = ipixsam * 2;
      xd1 = xr1; yd1 = yt1;
      xd2 = xr2; yd2 = yt2;

      /*** Set up boxes and display upper right corner of synoptic plot. ***/

      xbox1[0] = xr1 + 1;
      xbox1[1] = xr1 - nlong - 1;
      xbox1[2] = xbox1[1];
      xbox1[3] = xbox1[0];
      xbox1[4] = xbox1[0];
      ybox1[0] = yt1+1;
      ybox1[1] = ybox1[0];
      ybox1[2] = yt1 - nlat - 1;
      ybox1[3] = ybox1[2];
      ybox1[4] = ybox1[0];
      for (i=0; i<=4; i++) printf("i,x,y = %d %d %d\n",i,xbox1[i],ybox1[i]);
      wlvec (xbox1, ybox1, 5, grey);
	
      /*** Upper right corner of synoptic plot. ***/

      xbox2[0] = xr2+1;
      xbox2[1] = xr2-nlong-1;
      xbox2[2] = xbox2[1];
      xbox2[3] = xbox2[0];
      xbox2[4] = xbox2[0];
      ybox2[0] = yt2 + 1;
      ybox2[1] = ybox2[0];
      ybox2[2] = yt2 - nlat - 1;
      ybox2[3] = ybox2[2];
      ybox2[4] = ybox2[0];
      printf ("\n");
      for (i=0; i<=4; i++) printf("i,x,y = %d %d %d\n",i,xbox2[i],ybox2[i]);
      wlvec (xbox2, ybox2, 5, grey);

      /*** Write annotation in Grinnell image channel. ***/

      size = 2;
      if (namfil [0] == 'v')
         sprintf (title, "19%d CORONAL SYNOPTIC MAP : %3.1f Rsun RAW",
                  iyear, radius);
      else
         sprintf (title, "19%d CORONAL SYNOPTIC MAP : %3.1f Rsun CAL",
                  iyear, radius);
      xx = 256 - (strlen (title) * dwdhx / 2);
      yy = 511;
      wstr (title, xx, yy, size, yellow);

      size = 2;
      sprintf (crot, "%d", ncrot);
      sprintf (subtit,
	"NCAR/HAO: SMM C/P -- ROTATION # %1c%1c%1c%1c",
	 crot [0], crot [1], crot [2], crot [3]);
      xx = ((xr1 - xl1) / 2 + xl1) - ((strlen (subtit) * dwdhx) / 2) - 1;
      if (xx < 0)  xx = 0;
      yy = 511 - (int) (1.25 * dwdhy);
      wstr (subtit, xx, yy, size, red);

      size = 1;
      xx = (int) (xl1 + (float) ipixrot * (0.5 + xtrarot) + 0.5)
           - ((strlen (lablong) * swshx) / 2) - 2;
      yy = ((yb1 - yt2) / 2 + yt2) + swshy/2;
      wstr (lablong, xx, yy, size, red);

      xx = (int) (xl1 + (float) ipixrot * xtrarot + 0.5) - swshx/2;
      wstr ("0", xx, yy, size, red);
      xx = (int) (xl1 + (float) ipixrot * (1.0 + xtrarot) + 0.5) - (swshx*3)/2;
      wstr ("360", xx, yy, size, red);

      xx = xl1 - swdhx * 3 - 10;
      wstr ("+90", xx, yt1    +swshy/2, size, grey);
      wstr ("  0", xx, yt1- 90+swshy/2, size, grey);
      wstr ("-90", xx, yt1-180+swshy/2, size, grey);

      wstr ("+90", xx, yt2    +swshy/2, size, grey);
      wstr ("  0", xx, yt2- 90+swshy/2, size, grey);
      wstr ("-90", xx, yt2-180+swshy/2, size, grey);

      xx = xx / 2 - swdhx;
      if (xx < 0)  xx = 0;
      yy = ((yb1 - yt2) / 2 + yt2) - swdhy/2;
      wstr ("L", xx, yy + swshy * 8, size, grey);
      wstr ("A", xx, yy + swshy * 6, size, grey);
      wstr ("T", xx, yy + swshy * 4, size, grey);
      wstr ("I", xx, yy + swshy * 2, size, grey);
      wstr ("T", xx, yy + swshy * 0, size, grey);
      wstr ("U", xx, yy - swshy * 2, size, grey);
      wstr ("D", xx, yy - swshy * 4, size, grey);
      wstr ("E", xx, yy - swshy * 6, size, grey);

      xxl = xl1 - 6;
      xxr = xl1 - 2;
      wvec(xxl,yt1,xxr,yt1,grey);	/* Index mark +90 degrees, east limb. */
      wvec(xxl,yt1-30,xxr,yt1-30,grey); /* Index mark +60 degrees, east limb. */
      wvec(xxl,yt1-60,xxr,yt1-60,grey); /* Index mark +30 degrees, east limb. */
      wvec(xxl,yt1-90,xxr,yt1-90,grey); /* Index mark   0 degrees, east limb. */
      wvec(xxl,yt1-120,xxr,yt1-120,grey);/* Index mark -30 degrees, east limb.*/
      wvec(xxl,yt1-150,xxr,yt1-150,grey);/* Index mark -60 degrees, east limb.*/
      wvec(xxl,yt1-180,xxr,yt1-180,grey);/* Index mark -90 degrees, east limb.*/

      wvec(xxl,yt2,xxr,yt2,grey);	/* Index mark +90 degrees, west limb. */
      wvec(xxl,yt2-30,xxr,yt2-30,grey); /* Index mark +60 degrees, east limb. */
      wvec(xxl,yt2-60,xxr,yt2-60,grey); /* Index mark +30 degrees, east limb. */
      wvec(xxl,yt2-90,xxr,yt2-90,grey); /* Index mark   0 degrees, west limb. */
      wvec(xxl,yt2-120,xxr,yt2-120,grey);/* Index mark -30 degrees, west limb.*/
      wvec(xxl,yt2-150,xxr,yt2-150,grey);/* Index mark -60 degrees, west limb.*/
      wvec(xxl,yt2-180,xxr,yt2-180,grey);/* Index mark -90 degrees, west limb.*/

      xx = xr1 + (511 - xr1) / 2 - swdhx * 2 + 1;
      if ((xx + 4 * swdhx) > 511) xx = 511 - 4 * swdhx;
      yy = ((yb1 - yt2) / 2 + yt2) - swdhy/2;
      wstr ("EAST", xx, yz1+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz1-swdhy,   size, grey);
      wstr ("WEST", xx, yz2+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz2-swdhy,   size, grey);

      xx = xl2 + (xr2 - xl2) / 2 - (strlen (labdoy) * swshx) / 2 - 1;
      wstr (labdoy, xx, swshy + 7, size, green);

      /* * * * * * * * * * * * * * * * * * * * * * * * *
       *   Set up starting DOY values and x locations. *
       * * * * * * * * * * * * * * * * * * * * * * * * */

      crbeg = (float) ncrot - xtrarot;
      crend = (float) ncrot + xtrarot + 1.0;

      doybege = syndoy (iyear, ncrot) - syndur (ncrot) * (0.25 + xtrarot);
      doybegw = syndoy (iyear, ncrot) + syndur (ncrot) * (0.25 - xtrarot);

      if ((iyear/4) * 4 == iyear)  doymax = 366.0;  else doymax = 365.0;
      if (doybege > doymax)  doybege -= doymax;
      if (doybegw > doymax)  doybegw -= doymax;
      printf ("doybege: %f   doybegw: %f\n", doybege, doybegw);

      /*** East DOY initialization. ***/

      ndoye = (int) doybege;
      ibege = ndoye + 1;
      iende = ndoye + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
      posdoye = (float) xr1 + pixsam * samdoy * (doybege - (float) ndoye);
      printf ("ibege: %3d   iende: %3d   NROT: %d   xtrarot: %f   doyrot: %f\n",
               ibege, iende, NROT, xtrarot, doyrot);

      /*** West DOY initialization. ***/

      ndoyw = (int) doybegw;
      ibegw = ndoyw + 1;
      iendw = ndoyw + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
      posdoyw = (float) xr1 + pixsam * samdoy * (doybegw - (float) ndoyw);

      /*** Compute East DOYs and annotate Grinnell. ***/

      for (idoye=ibege; idoye<iende; idoye++) /* Days increase right to left. */
	 {
         posdoye -= pixsam * samdoy;
	 locdoye = (int) (posdoye + 0.5);
	 printf ("idoye:%d   posdoye:%6.2f  locdoye:%d   xl1:%d   xr1:%d\n",
		  idoye, posdoye, locdoye, xl1, xr1);
	 if (locdoye >= xl1 && locdoye <= xr1)
            {
	    wvec (locdoye, yt1+2, locdoye, yt1+6, green);
	    if (idoye/5*5 == idoye)   /* Write 2 more tick marks every 5 deg. */
               {
	       wvec (locdoye-1, yt1+2, locdoye-1, yt1+6, green);
	       wvec (locdoye+1, yt1+2, locdoye+1, yt1+6, green);
	       sprintf (sdoy, "%d", idoye);
	       printf ("sdoy: %s\n", sdoy);
	       lxdoye = locdoye - (strlen(sdoy) * swdhx)/2;
	       lydoye = yt1+9+swdhy;
	       if (idoye/10*10 == idoye)	/* Write DOY every 10 degrees.*/
	          wstr (sdoy, lxdoye, lydoye, size, green);
               }
	    }
         }

      /*** Compute West DOYs and annotate Grinnell. ***/

      for (idoyw=ibegw; idoyw<iendw; idoyw++) /* Days increase right to left. */
	 {
         posdoyw -= pixsam * samdoy;
	 locdoyw = (int) (posdoyw + 0.5);
	 printf ("idoyw:%d   posdoyw:%6.2f  locdoyw:%d   xl1:%d   xr1:%d\n",
		  idoyw, posdoyw, locdoyw, xl1, xr1);

	 if (locdoyw >= xl1 && locdoyw <= xr1)
            {
	    wvec (locdoyw, yb2-2, locdoyw, yb2-6, green);
	    if (idoyw/5*5 == idoyw)   /* Write 2 more tick marks every 5 deg. */
               {
	       wvec (locdoyw-1, yb2-2, locdoyw-1, yb2-6, green);
	       wvec (locdoyw+1, yb2-2, locdoyw+1, yb2-6, green);
	       sprintf (sdoy, "%d", idoyw);
	       lxdoyw = locdoyw - (strlen(sdoy) * swdhx)/2;
	       lydoyw = yb2-11;
	       if (idoyw/10*10 == idoyw)	/* Write DOY every 10 degrees.*/
	          wstr (sdoy, lxdoyw, lydoyw, size, green);
               }
	    }
         }

      /*** Map initialization. ***/

      fprintf (stdout, "crbeg: %8.3f   crend: %8.3f   xtrarot: %9.4f\n",
	       crbeg, crend, xtrarot);

      if (fseek (SYN, 0L, 0) == -1)
         {
         fprintf (stderr, "seek error: BOF\n");
         exit (-1);
         }
      if (fread ((char *) ST, sizST, 1, SYN) == 0)
         {
         fprintf (stderr, "read error: BOF\n");
         exit (-2);
         }

      if (ST->et < 1000.)  filedoy0 = ST->et;
      if (ST->et >= 1000.)  filedoy0 = ST->et - 2000.;
      idoy0 = (int) filedoy0;

      ibeg = ((int) doybege - idoy0) * samdoy
	     - 4;	/* Start record in data file. */
      if (ibeg < 0)  ibeg = 0;
      iend = ((int) doybegw - idoy0) * samdoy + (1.0 + xtrarot * 2.0) * samrot
	     + 4;	/* End record in data file. */

      printf ("crbeg : %8.3f   crend : %8.3f\n", crbeg, crend);
      printf ("idoy0: %d   ibeg: %d   iend: %d\n", idoy0, ibeg, iend);

      /*** Data loop for single rotation maps. ***/

      for (irec=ibeg; irec<iend; irec++)
         {

         /*** Read in record ***/

         offset = (long) (irec * sizST);
         if (fseek (SYN, offset, 0) == -1)
            {
            fprintf (stderr, "seek error: EOF at record %d\n", irec);
            goto endmap1;
	    }
         if (fread ((char *) ST, sizST, 1, SYN) == 0)
	    {
            fprintf (stderr, "read error: EOF at record %d\n", irec);
            goto endmap1;
            }
         fprintf (stdout, "irec: %3d   net:%7.2f   et:%7.2f   set:%7.2f\n",
                  irec, ST->net, ST->et, ST->set);
         fprintf (stdout, "           swt:%7.2f   wt:%7.2f   nwt:%7.2f\n",
                  ST->swt, ST->wt, ST->nwt);

         /*** East ok?  Then insert. ***/

         if ((ST->ec >= crbeg) && (ST->ec < crend))
            {
            xloc1 = xr1 - (int) ((ST->ec - crbeg) * ipixrot);
            printf("ec = %8.3f   xloc1 = %d  x1prev = %d\n", ST->ec, xloc1, x1prev);
	    x1prev = xloc1;

            /*** Screen index of slot. ***/

            for (jj=0; jj<180; jj++)
               {
               ival = (int) (atan ((float)(ST->th [jj] * zscale * zt)) * zm);
               if (ival > irismax)  ival = irismax;
               ibuf [jj] = (char) ival;
               }

            xstart = xloc1;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl1 || jj > xr1) continue;
               wlin (FB, ibuf, jj, yt1-1, nlat, dir1);
               }
            }

         /*** West ok?  Then insert. ***/

         if ((ST->wc >= crbeg) && (ST->wc < crend))
            {
            xloc2 = xr2 - (int) ((ST->wc - crbeg) * ipixrot);
            fprintf (stdout, "wc = %8.3f   xloc2 = %d  x2prev = %d\n",
		     ST->wc, xloc2, x2prev);
	    x2prev = xloc2;

            /*** Screen index of slot. ***/

            for (jj=180; jj<361; jj++)
               {
               ival = (int) (atan ((float)(ST->th [jj] * zscale * zt)) * zm);
               if (ival > irismax)  ival = irismax;
               ibuf [541 - jj] = (char) ival;
               }

            xstart = xloc2;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl2 || jj > xr1)  continue;
               wlin (FB, &ibuf [180], jj, yt2-1, nlat, dir1);
               }
            }  /* West Limb. */
         } /* Data loop. */

endmap1:

      /*** Draw boxes around synoptic map image area. ***/

      wlvec (xbox1, ybox1, 5, green);	/* Box frame for east limb. */
      wvec (xl1, yt1-90, xr1, yt1-90, green);/* Equatorial line for east limb.*/

      wlvec (xbox2, ybox2, 5, green);	/* Box frame for west limb. */
      wvec (xl2, yt2-90, xr2, yt2-90, green);/* Equatorial line for west limb.*/

      xx = xr1 - (int) (xtrarot * (float) ipixrot + 0.5);
      wvec (xx, yt1,    xx, yb1-5, red);/* Index mark   0 degrees longitude. */
      wvec (xx, yt2+5,  xx, yb2, red);  /* Index mark   0 degrees longitude. */

      xx = xr1 - (int) ((xtrarot + 0.5) * (float) ipixrot + 0.5);
      wvec (256, yb1-1, 256, yb1-5, red);/* Index mark 180 degrees longitude.*/
      wvec (256, yt2+5, 256, yt2+1, red);/* Index mark 180 degrees longitude.*/

      xx = xr1 - (int) ((xtrarot + 1.0) * (float) ipixrot + 0.5);
      wvec (xx, yt1,    xx, yb1-5, red); /* Index mark 360 degrees longitude. */
      wvec (xx, yt2+5,  xx, yb2, red);	 /* Index mark 360 degrees longitude. */

      }		/* End of single rotation map. */
   closepl ();
   fb_close ();
}
