/* sdcme.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Display SMM C/P Synoptic Map data on SGI IRIS.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Use with synoptic map files containing photometrically calibrated data.
 * Companion program for use with files generated by `sg'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Original code written by Rainer Illing.
 * Revisions by Andrew L. Stanger
 * 29 April 1987
 * 5 May 1987: Fix for 1980 synoptic map.
 * 14 January 1988: Revise to be compatible with `sg3.c'.
 * 25 May 1988: modify for compatibility with `sg3a.c'.
 * 2 June 1988: non-existent data bins have times > +1000.
 * 3 August 1988: Remove photometric scaling adjustments.  All synoptic files
 *                should contain photometrically calibrated data.
 * 23 June 1989: Sun-4 HIPS version.
 * 10 September 1990 : For single-rotation maps, use square aspect ratio.
 *  9 April 1991: adapt for SGI IRIS.
 *  7 August 1991: add logic to read CME info file & draw "bugle" lines.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include	<stdio.h>
#include	<math.h>
#include	<whip.h>
#include	"/usr/people/smm/lib/src/cp/cplib.h"
#include	"/usr/people/smm/lib/src/whip/cp_color.h"
#include	"synst.h"

struct S STT;

main (argc, argv)

int argc;
char *argv [];
{
extern float synrot ();	/* Synodic rotation for a given DOY. */
extern float syndoy ();	/* Synodic day-of-year for start of a given rotation. */
extern float syndur ();	/* Synodic rotation duration. */
register struct	S	*ST = &STT;	/* pointer to synoptic record. */
FILE	*SYN;				/* file pointer. */
FILE	*CME;
static	char	dir1[2] = "d";
static  char	crad [3];
static	char	crot [8];
static	char	crs [6], sdoy [4];
static  char	namsyn [80], answer [16];
static  char	namcme [80] = "cpsyn.cme";
static  char	indiv [2];		/* Individual maps. */
static	char	labdoy [] = "I M A G E   D A Y - O F - Y E A R";
static	char	lablong [] = "L O N G I T U D E";
static	char	labrot [] = "C A R R I N G T O N   R O T A T I O N";
static  char	title [40] = "198x CORONAL SYNOPTIC MAP : x.xx Rsun";
static  char	subtit [80];		/* Sub-title character string. */
static  char	ibuf [364];		/* Intensity buffer. */
static	char	irismin = 0;
static	char	irismax = 249;

static	float	crdur [16];	/* Duration of carrington rotations. */
float	cmedoy;		/* CME Day-Of-Year. */
float	cmedoy1, cmedoy2;/* Begin/End CME Day-Of-Year. */
float	cmedur;		/* CME duration (in DOY units). */
float	crbeg, crend;	/* Rotation begin/end for synoptic map display. */
float	doybege, doybegw, doyende, doyendw, doymax;
float	doy0;		/* DOY for first sample in data file. */
float	doymap;		/* Map duration in units of days. */
float	doyrot;		/* rotation duration in units of days. */
float	fcrot;
float	filedoy0;	/* DOY at start of synoptic map data file. */
float	filerot0;	/* Rotation (cmp) at start of synoptic map data file. */
float	pixsam;		/* # pixels per sample. */
float	posdoye, posdoyw;
float	radius;
float	rotbeg, rotend;	/* Rotation begin/end for synoptic data file. */
float	samrot;		/* # samples per rotation. */
float	zmax = 250.0;
float	zscale;
float	xtrarot = 0.0;	/* Extra display beyond limits of a given rotation. */

static int xbox1 [5], ybox1 [5], xbox2 [5], ybox2 [5], xl [2],yl [2];
static int locbeg [16];
int	MSROT = 55;	/* Maximum # samples per rotation. */
int	MAXROT = 15;	/* Maximum # carrington rotations in syngen data file.*/
int	NROT = 6;	/* # carrington rotations to be displayed in map. */

int FB = 0;		/* default: 512x512 byte frame buffer (colormap mode).*/
int fast = 1;		/* Parameter for fb_clear. */
float bkgcol = 0.0;	/* Grey level for fb_clear. */

int	colans;		/* Color answer. */
int	cenang, widang;	/* Central angle & width. */
int	begang, endang;	/* Begin/End angles for CME lines. */
int	chang = 0;	/* character string angle (w.r.t. +x axis). */
int	cmecol;		/* CME color. */
int	cmehr, cmemin;	/* CME hour & minute. */
int	cmehr1, cmehr2;	/* CME begin/end hour. */
int	cmemin1, cmemin2;/* CME begin/end minute. */
int	cmeyr;		/* CME year. */
int	idoy0;		/* DOY for first sample in data file. */
int	i;
int	ibege, ibegw;	/* start index for east/west DOY loops. */
int	iende, iendw;	/* end index for east/west DOY loops. */
int	icr;		/* Carrington rotation index. */
int	idot;
int	ieast, iwest;
int	ibeg, iend;	/* start/stop index limits for c.r. data loop. */
int	imgch;		/* image channel.   */
int	irad1, irad2;
int	irec;		/* Data file record pointer. */
int	iyear;		/* Year for synoptic map display. */
int	jj;		/* General purpose index. */
int	ldash;		/* Flag to specify dashed CME line. */
int	lxcre, lycre;	/* screen coordinates of east Carrington rotations. */
int	lxcrw, lycrw;	/* screen coordinates of west Carrington rotations. */
int	locdoye, locdoyw;	/* screen coordinates for DOY tick marks. */
int	lxdoye, lydoye;		/* screen coordinates for east DOY string. */
int	lxdoyw, lydoyw;		/* screen coordinates for west DOY string. */
int	ncr;		/* Carrington rotation number. */
int	ncrot;		/* First carrington rotation in display. */
int	ncrot0;		/* First carrington rotation in data file. */
int	ndoye, ndoyw;	/* Truncated (integer) DOY for right map edge. */
int	nlat = 181;	/* # latitude pixels. */
int	nlong;		/* # longitude pixels. */
int	numecar;	/* East Carrington rotation number. */
int	numwcar;	/* West Carrington rotation number. */
int	numtry = 0;	/* # retries to select # legal Carrington rotations. */
int	idoye, idoyw;	/* Indices for DOY annotation loops. */
long	offset;		/* # logical records to skip in synoptic data file. */
int	ipixsam;	/* # pixels per sample. */
int	ipixdoy;	/* # pixels per DOY. */
int	ipixrot;	/* # pixels per rotation. */
int	samdoy = 2;	/* # samples per DOY. */
int	sammap;		/* # samples per map. */
int	size = 1;	/* character size. */
int	swshx =  6;	/* horizontal size 1. */
int	swshy =  9;	/* vertical   size 1. */
int	swdhx =  6;	/* horizontal size 1. */
int	swdhy =  9;	/* vertical   size 1. */
int	dwshx = 12;	/* horizontal size 2. */
int	dwshy = 18;	/* vertical   size 2. */
int	dwdhx = 12;	/* horizontal size 2. */
int	dwdhy = 18;	/* vertical   size 2. */
int	sizST;		/* Size (bytes) of data record. */

int	x1prev, x2prev;	/* Previous IRIS x-coordinate of data. */
int	xbeg;	/* x begin location. */
int	xend;	/* x end   location. */
int	xd1;	/* x east.          */
int	xd2;	/* x west.          */
int	xinc=7;	/* pixel increment for character strings. */
int	xr1;	/* Right x east.    */
int	xr2;	/* Right x west.    */
int	xl1;	/* Left  x east.    */
int	xl2;	/* Left  x west.    */
int	xloc1;	/* Grinnell x location for a given DOY in a rotation. */
int	xloc2;	/* Grinnell x location for a given DOY in a rotation. */
int	xspan;	/* # additional pixels width needed to span a gap. */
int	xstart, xstop;	/* start/stop x-location for synoptic strips. */
int	xwidth; /* # pixels width of synoptic strips for individual rotations.*/
int	xw2;	/* xwidth / 2 */
int	xx = 0, xxl = 0, xxr = 0;	/* general x-location variables. */

int	yb1;	/* Bottom y east.   */
int	yb2;	/* Bottom y west.   */
int	ybeg;	/* Y begin location. */
int	yend;	/* Y end location. */
int	yd1;	/* y east.          */
int	yd2;	/* y west.          */
int	yt1;	/* Top y east.      */
int	yt2;	/* Top y west.      */
int	yy;	/* general purpose y-location variable. */
int	yz1, yz2;/* 0 degree latitude for east & west maps. */

FILE	*NLN, *fopen();			/*   "         file  */
char	*index(), *p8;			/* to find year of file	*/
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
int yellow, grey, blue, green, red;
yellow = color0;
grey   = color1;
blue   = color2;
green  = color3;
red    = color4;
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

   /*** Request synoptic map data file name. ***/

   fprintf (stderr,"%s  Version 3.00\n", argv [0]);
   fprintf (stderr,"enter synoptic file name: ");

   sizST = sizeof(STT);	/* Size (bytes) of synoptic data structure. */

   scanf ("%s", namsyn);
   if ((SYN = fopen (namsyn, "r")) == NULL)
      {
      fprintf (stderr, "cannot open synoptic file: %s\n", namsyn);
      exit (-1);
      }

   /*** Read in first record. ***/

   if (fseek (SYN, 0L, 0) == -1)
      {
      fprintf (stderr, "seek error: BOF\n");
      exit (-1);
      }
   if (fread ((char *) ST, sizST, 1, SYN) == 0)
      {
      fprintf (stderr, "read error: BOF\n");
      exit (-2);
      }
   /*
    * Find starting rotation number from file name.
    * Fix this later in rational rewrite of whole business.
    */
   p8 = index (namsyn, '8');
   for (i = 0; i < strlen (namsyn); i++)  if (namsyn[i] == '.')  idot = i;
   if (*(p8+1) == '0')  iyear = 80;
   if (*(p8+1) == '4')  iyear = 84;
   if (*(p8+1) == '5')  iyear = 85;
   if (*(p8+1) == '6')  iyear = 86;
   if (*(p8+1) == '7')  iyear = 87;
   if (*(p8+1) == '8')  iyear = 88;
   if (*(p8+1) == '9')  iyear = 89;

   strncat (crad, &namsyn [idot+2], 2);
   irad1 = ((int) (crad[0]) & 0377) - 060;
   irad2 = ((int) (crad[1]) & 0377) - 060;
   radius = (float) irad1 + ((float) irad2) * 0.1;
   printf ("crad:%s   idot:%d\n", crad, idot);
   printf ("irad1:%d   irad2:%d   radius: %f\n", irad1, irad2, radius);
   printf ("iyear:%d   ST->et: %f\n", iyear, ST->et);

   if (ST->et < 1000.)  filedoy0 = ST->et;
   if (ST->et >= 1000.)  filedoy0 = ST->et - 2000.;
   filerot0 = synrot (iyear, filedoy0) + 0.25;
   ncrot0 = (int) filerot0;

   /*** Set up IRIS WHIP software. ***/

   fb_open ();				/* Initialize WHIP. */
   fb_setfb (FB);			/* Select frame buffer. */
   openpl ();
   fb_clear (FB, &bkgcol, fast);	/* Clear frame buffer. */

   /*** Get scaling value. ***/

   fprintf (stderr, "Enter maximum value for scale:  ");
   fscanf  (stdin,  "%f", &zmax);
   zscale = (float) irismax / zmax;
   numtry = 0;

maptype:

   fprintf (stderr, "Enter # rotations to be displayed [1-7]: ");
   fscanf  (stdin,  "%d", &NROT);
   switch (NROT)
      {
      case 1: indiv [0] = 'y';  ipixsam = 5;  break;
      case 2: indiv [0] = 'n';  ipixsam = 3;  break;
      case 3: indiv [0] = 'n';  ipixsam = 2;  break;
      case 4: indiv [0] = 'n';  ipixsam = 1;  break;
      case 5: indiv [0] = 'n';  ipixsam = 1;  break;
      case 6: indiv [0] = 'n';  ipixsam = 1;  break;
      case 7: indiv [0] = 'n';  ipixsam = 1;  break;
      default:
               printf (stderr, "Invalid # rotations.  Try again.");
               numtry++;
               if (numtry < 3)  goto maptype;
               else exit (-1);
      }

   if (indiv [0] != 'y')
      {

/* * * * * * * * * * * * * Multiple Rotation Map. * * * * * * * * * * * * * * */

      fprintf (stderr, "First rotation in map: %d\n", ncrot0);
      fprintf (stderr, "Enter starting carring rotation number : ");
      fscanf  (stdin,  "%d", &ncrot);

      fprintf (stderr, "Enter radius [Rsun]: ");
      fscanf  (stdin,  "%f", &radius);

      /*** Initialization. ***/

      doymap = 0;
      for (i=0; i<NROT; i++)
         {
         crdur [i] = syndur (ncrot + i); 
         doymap += crdur [i]; 
         }
      doymap /= (float) NROT;
      samrot = doymap * (float) samdoy;
      sammap = doymap * samdoy;
      ipixrot = ipixsam * samrot;
      ipixdoy = ipixsam * samdoy;
      nlong = (int) (NROT * ipixrot + 0.5); /* # longitudinal pixel slices. */
      xr1 = 256 + nlong/2;		   /* Upper right corner of east map. */
      yt1 = 440;
      xl1 = xr1 - nlong + 1;	/* Upper left corner of east map. */
      yb1 = yt1 - nlat + 1;
      yz1 = yt1 -  90;		/* Zero degrees latitude (east map). */

      xr2 = xr1;		/* Upper right corner of west map. */
      yt2 = 231;
      xl2 = xr2 - nlong + 1;	/* Upper left corner of west map. */
      yb2 = yt2 - nlat + 1;
      yz2 = yt2 -  90;		/* Zero degrees latitude (west map). */

      xd1 = xr1; yd1 = yt1;
      xd2 = xr2; yd2 = yt2;

   /*** Write annotation on IRIS display. ***/

      size = 2;
      sprintf (title, "19%d CORONAL SYNOPTIC MAP : %3.1f Rsun", iyear, radius);
      xx = 256 - (strlen (title) * dwdhx / 2);
      yy = 511;
      wstr (title, xx, yy, size, yellow);

      size = 2;
      sprintf (subtit, "NCAR / HAO :   SMM  CORONAGRAPH");
      xx = 256 - (strlen (subtit) * dwdhx / 2);
      yy = 511 - 1.25 * dwdhy;
      wstr (subtit, xx, yy, size, red);

      size = 1;
      xx = xl1 - swshx * 3 - 10;
      if (xx < 0) xx = 0;

      wstr ("+90", xx, yt1    +swdhy/2, size, grey);
      wstr ("  0", xx, yt1- 90+swdhy/2, size, grey);
      wstr ("-90", xx, yt1-180+swdhy/2, size, grey);

      wstr ("+90", xx, yt2    +swdhy/2, size, grey);
      wstr ("  0", xx, yt2- 90+swdhy/2, size, grey);
      wstr ("-90", xx, yt2-180+swdhy/2, size, grey);

      xx = xx / 2 - swdhx;
      if (xx < 0)  xx = 0;
      yy = ((yb1 - yt2) / 2 + yt2) + swdhy/2;
      wstr ("L", xx, yy + swdhy * 8, size, grey);
      wstr ("A", xx, yy + swdhy * 6, size, grey);
      wstr ("T", xx, yy + swdhy * 4, size, grey);
      wstr ("I", xx, yy + swdhy * 2, size, grey);
      wstr ("T", xx, yy - swdhy * 0, size, grey);
      wstr ("U", xx, yy - swdhy * 2, size, grey);
      wstr ("D", xx, yy - swdhy * 4, size, grey);
      wstr ("E", xx, yy - swdhy * 6, size, grey);

      xxl = xl1 - 10;
      xxr = xl1 -  4;
      if (xxl < 0) xxl = 0;
      if (xxr < 0) xxr = 0;
      wvec (xxl,yt1,   xxr,yt1,grey);	 /* Index mark +90 degrees, east limb.*/
      wvec(xxl,yt1-30,xxr,yt1-30,grey);  /* Index mark +60 degrees, east limb.*/
      wvec(xxl,yt1-60,xxr,yt1-60,grey);  /* Index mark +30 degrees, east limb.*/
      wvec(xxl,yt1-90,xxr,yt1-90,grey);  /* Index mark   0 degrees, east limb.*/
      wvec(xxl,yt1-120,xxr,yt1-120,grey);/* Index mark -30 degrees, east limb.*/
      wvec(xxl,yt1-150,xxr,yt1-150,grey);/* Index mark -60 degrees, east limb.*/
      wvec(xxl,yt1-180,xxr,yt1-180,grey);/* Index mark -90 degrees, east limb.*/

      wvec(xxl,yt2,   xxr,yt2,grey);	 /* Index mark +90 degrees, west limb.*/
      wvec(xxl,yt2-30,xxr,yt2-30,grey);  /* Index mark +60 degrees, west limb.*/
      wvec(xxl,yt2-60,xxr,yt2-60,grey);  /* Index mark +30 degrees, west limb.*/
      wvec(xxl,yt2-90,xxr,yt2-90,grey);  /* Index mark   0 degrees, west limb.*/
      wvec(xxl,yt2-120,xxr,yt2-120,grey);/* Index mark -30 degrees, west limb.*/
      wvec(xxl,yt2-150,xxr,yt2-150,grey);/* Index mark -60 degrees, west limb.*/
      wvec(xxl,yt2-180,xxr,yt2-180,grey);/* Index mark -90 degrees, west limb.*/

      xx = xr1 + (511 - xr1) / 2 - swdhx * 2;
      if (xx < 0) xx = 0;
      fprintf (stdout, "EAST LIMB  xx: %d   yy: %d\n", xx, yy);
      wstr ("EAST", xx, yz1+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz1-swdhy,   size, grey);
      wstr ("WEST", xx, yz2+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz2-swdhy,   size, grey);

      xx = xl2 + (xr2 - xl2) / 2 - (strlen (labdoy) * swshx) / 2 - 1;
      wstr (labrot, xx, swdhy + 7, size, green);

      /*** Draw boxes around synoptic map image area. ***/

      xbox1[0] = xr1 + 4;
      xbox1[1] = xr1 - nlong - 3;
      xbox1[2] = xbox1[1];
      xbox1[3] = xbox1[0];
      xbox1[4] = xbox1[0];
      ybox1[0] = yt1 + 1;
      ybox1[1] = ybox1[0];
      ybox1[2] = yt1 - nlat;
      ybox1[3] = ybox1[2];
      ybox1[4] = ybox1[0];
      wlvec (xbox1, ybox1, 5);

      xbox2[0] = xr2 + 4;
      xbox2[1] = xr2 - nlong - 3;
      xbox2[2] = xbox2[1];
      xbox2[3] = xbox2[0];
      xbox2[4] = xbox2[0];
      ybox2[0] = yt2 + 1;
      ybox2[1] = ybox2[0];
      ybox2[2] = yt2 - nlat;
      ybox2[3] = ybox2[2];
      ybox2[4] = ybox2[0];
      wlvec (xbox2, ybox2, 5);

      /*** Draw lines to separate Carrington rotations. ***/

      doy0 = syndoy (iyear, ncrot);
      for (i=0; i<=NROT; i++)
         {
	 xx = xr1 - (int)(syndoy (iyear, ncrot+i) - doy0) * ipixdoy + 1;
         wvec (xx, yt1+1, xx, yt1+swdhy+6, red);
         wvec (xx, yb1-1, xx, yb1-6, red);
         wvec (xx, yt2+1, xx, yt2+6, red);
         wvec (xx, yb2-1, xx, yb2-swdhy-6, red);
         locbeg [i] = xx;
         }

      /*** Write Carrington rotation numbers on Grinnell. ***/

      lxcre = xr1 - (int) (ipixrot / 2.0) - xinc * 2 - 1;
      lycre = yt1 + 5 + swdhy;
      lxcrw = xr2 - (int) (ipixrot / 2.0) - xinc * 2 - 1;
      lycrw = yb2 - 7;

      numecar = ncrot;
      numwcar = ncrot;

      for (i=0; i<NROT; i++)
	 {
	 printf ("east/west c.r.: %d %d  lxcre:%d lxcrw:%d\n",
		  numecar, numwcar, lxcre, lxcrw);
         sprintf (crs, "%d", numecar);
         wstr (crs, lxcre, lycre, size, green);
         wstr (crs, lxcrw, lycrw, size, green);
	 numecar += 1;
	 numwcar += 1;
	 lxcre -= ipixrot;
	 lxcrw -= ipixrot;
	 }

      /*** Map initialization for multiple rotation synoptic map. ***/

      xwidth = ipixsam * 2;
      xtrarot = 0.0;

      rotbeg = (float) ncrot0;
      rotend = rotbeg + (float) MAXROT;
      fprintf (stdout, "rotbeg: %7.2f   rotend: %7.2f\n", rotbeg, rotend);

      crbeg = (float) ncrot - xtrarot;
      crend = (float) ncrot + xtrarot + NROT;
      fprintf (stdout, "crbeg: %8.3f   crend: %8.3f   xtrarot: %9.4f\n",
	       crbeg, crend, xtrarot);

      if (crbeg < rotbeg)  crbeg = rotbeg;
      if (crend > rotend)  crend = rotend;

      ibeg = (int) ((crbeg - filerot0) * MSROT) - 2;
      iend = (int) ((crend - filerot0 + 0.5) * MSROT) + 2;
      if (ibeg < 0)  ibeg = 0;
      if (iend > MSROT * MAXROT)  iend = MSROT * MAXROT;
      fprintf (stdout, "ibeg: %d   iend:%d\n", ibeg, iend);

      ieast = 0;
      iwest = 0;

      /*** Data loop for multiple rotation synoptic map. ***/

      x1prev = xr1;
      for (irec=ibeg; irec<iend; irec++)
         {

         /*** Read in record ***/

         offset = (long) (irec * sizST);
         if (fseek (SYN, offset, 0) == -1)
            {
            fprintf(stderr,"seek error: loop  %d\n",i);
            exit (-1);
	    }
         if (fread ((char *)ST,sizST,1,SYN) == 0)
	    {
            fprintf(stderr,"read error: loop  %d\n",i);
            exit (-2);
            }
         fprintf (stdout, "irec: %3d   net:%7.2f   et:%7.2f   set:%7.2f\n",
                  irec, ST->net, ST->et, ST->set);
         fprintf (stdout, "            swt:%7.2f   wt:%7.2f   nwt:%7.2f\n",
                  ST->swt, ST->wt, ST->nwt);

         /*** East ok?  Then insert. ***/

         if ((ST->ec >= crbeg) && (ST->ec < crend))
            {
            ncr = (int) ST->ec;
            icr = ncr - ncrot;
            xloc1 = locbeg [icr]
		    - (int) ((ST->ec - (float) ncr) * crdur [icr] * ipixdoy);
/*
 *	    if (ieast > 0)  xwidth = x1prev - xloc1;  else  xwidth = ipixsam + 1;
 */
            printf("ec: %7.2f   icr: %d   locbeg: %d   xloc1: %d  x1prev: %d\n",
                   ST->ec, icr, locbeg [icr], xloc1, x1prev);
	    ieast += 1;
	    x1prev = xloc1;

            /*** Screen index of slot. ***/

            for (jj=0; jj<180; jj++)
               ibuf [jj] = (char) (ST->th [jj] * zscale);

	    for (jj=0; jj<180; jj++)
	       if (ibuf [jj] > irismax)  ibuf [jj] = irismax;

            xstart = xloc1;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl1 || jj > xr1) continue;
               wlin (FB, ibuf, jj, yt1, nlat, dir1);
               }
            }

         /*** West ok?  Then insert. ***/

         if ((ST->wc >= crbeg) && (ST->wc < crend))
            {
            ncr = (int) ST->wc;
            icr = ncr - ncrot;
            xloc2 = locbeg [icr]
		    - (int) ((ST->wc - (float) ncr) * crdur [icr] * ipixdoy);
            printf("wc: %7.2f  icr: %d  locbeg: %d  xloc2: %d  x2prev: %d\n",
                   ST->wc, icr, locbeg [icr], xloc2, x2prev);
	    iwest += 1;
	    x2prev = xloc2;

            /*** Screen index of slot. ***/

            for (jj=181; jj<361; jj++)
               ibuf [541-jj] = (char) ( ST->th [jj] * zscale );

	    for (jj=180; jj<361; jj++)
	       if (ibuf [jj] > irismax)  ibuf [jj] = irismax;

            xstart = xloc2;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl2 || jj > xr1)  continue;
               wlin (FB, &ibuf [180], jj, yt2, nlat, dir1);
               }
            }	/*** End of west limb loop. ***/
         }	/*** End of multiple rotation data loop. ***/

      /*** Draw boxes around synoptic map image area. ***/

      wlvec (xbox1, ybox1, 5, green);	/* Box frame for east limb. */
      wvec (xl1, yt1-90, xr1, yt1-90, green);/* Equatorial line for east limb.*/

      wlvec (xbox2, ybox2, 5, green);	/* Box frame for west limb. */
      wvec (xl2, yt2-90, xr2, yt2-90, green);/* Equatorial line for east limb.*/

      /*** Draw lines to separate Carrington rotations. ***/

      for (i=0; i<=NROT; i++)
         {
	 xx = xr1 - (int)(syndoy (iyear, ncrot+i) - doy0) * ipixdoy + 1;
         wvec (xx, yb1-6, xx, yt1+15, red);
         wvec (xx, yt2+6, xx, yb2-15, red);
         }

      }
   else



/* * * * * * * * * * * * * * * Single Rotation Map * * * * * * * * * * * * * */
      {

      /*** Request starting Carrington rotation. ***/

      fprintf (stderr, "First rotation in map data file: %d\n", ncrot0);
      fprintf (stderr, "Enter rotation number : ");
      scanf ("%d", &ncrot);
      printf ("ncrot = %d start = %d\n", ncrot, ncrot0);

      fprintf (stderr, "Enter radius [Rsun]: ");
      scanf ("%f", &radius);

      fprintf (stderr, "CME vertical lines [y,n]: ");
      fscanf  (stdin,  "%s", answer);
      if (answer [0] == 'y' || answer [0] == 'Y')
         {
         fprintf (stderr, "CME line color [r,g,b,w,y]: ");
         fscanf (stdin,  "%s", answer);
         colans = answer [0];
         switch (colans)
            {
            case 'r': cmecol = red;     break;
            case 'g': cmecol = green;   break;
            case 'b': cmecol = blue;    break;
            case 'y': cmecol = yellow;  break;
            case 'w':
            default:  cmecol = white;
            }
         fprintf (stderr, "CME line pattern [dash,solid]: ");
         fscanf  (stdin,  "%s", answer);
         if (answer [0] == 'd' || answer [0] == 'D')
            ldash = 1;
         else
            ldash = 0;
         }

      doyrot = syndur (ncrot); 	/* # days in rotation `ncrot'. */
      samrot = doyrot * samdoy;	/* # samples in rotation `ncrot'. */
      ipixrot = 360;		/* # pixels per rotation (one per degree). */
      pixsam = ((float) ipixrot / samrot);		/* # pixels / sample. */
      ipixsam = (int) ((float) ipixrot / samrot);	/* # pixels / sample. */

      xtrarot = 0.10;
      doymap = doyrot + 2 * xtrarot * samrot;
      nlong = (int) ((NROT + xtrarot * 2) * ipixrot);
      xr1 = 256 + nlong/2;	/* Upper right corner of east map. */
      yt1 = 442;
      xl1 = xr1 - nlong + 1;	/* Upper left corner of east map. */
      yb1 = yt1 - nlat + 1;
      yz1 = yt1 -  90;		/* Zero degrees latitude (east map). */

      xr2 = 256 + nlong/2;	/* Upper right corner of west map. */
      yt2 = 228;
      xl2 = xr2 - nlong + 1;	/* Upper left corner of west map. */
      yb2 = yt2 - nlat + 1;
      yz2 = yt2 -  90;		/* Zero degrees latitude (west map). */

      xwidth = ipixsam * 2;
      xd1 = xr1; yd1 = yt1;
      xd2 = xr2; yd2 = yt2;

      /*** Set up boxes and display upper right corner of synoptic plot. ***/

      xbox1[0] = xr1 + 1;
      xbox1[1] = xr1 - nlong - 1;
      xbox1[2] = xbox1[1];
      xbox1[3] = xbox1[0];
      xbox1[4] = xbox1[0];
      ybox1[0] = yt1+1;
      ybox1[1] = ybox1[0];
      ybox1[2] = yt1 - nlat;
      ybox1[3] = ybox1[2];
      ybox1[4] = ybox1[0];
      for (i=0; i<=4; i++) printf("i,x,y = %d %d %d\n",i,xbox1[i],ybox1[i]);
      wlvec (xbox1,ybox1,5);
	
      /*** Upper right corner of synoptic plot. ***/

      xbox2[0] = xr2+1;
      xbox2[1] = xr2-nlong-1;
      xbox2[2] = xbox2[1];
      xbox2[3] = xbox2[0];
      xbox2[4] = xbox2[0];
      ybox2[0] = yt2 + 1;
      ybox2[1] = ybox2[0];
      ybox2[2] = yt2 - nlat;
      ybox2[3] = ybox2[2];
      ybox2[4] = ybox2[0];
      printf ("\n");
      for (i=0; i<=4; i++) printf("i,x,y = %d %d %d\n",i,xbox2[i],ybox2[i]);
      wlvec (xbox2,ybox2,5);

      /*** Write annotation in Grinnell image channel. ***/

      size = 2;
      sprintf (title, "19%d CORONAL SYNOPTIC MAP : %3.1f Rsun", iyear, radius);
      xx = 256 - (strlen (title) * dwdhx / 2);
      yy = 511;
      wstr (title, xx, yy, size, yellow);

      size = 2;
      sprintf (crot, "%d", ncrot);
      sprintf (subtit,
	"NCAR/HAO: SMM C/P -- ROTATION # %1c%1c%1c%1c",
	 crot [0], crot [1], crot [2], crot [3]);
      xx = ((xr1 - xl1) / 2 + xl1) - ((strlen (subtit) * dwdhx) / 2) - 1;
      if (xx < 0)  xx = 0;
      yy = 511 - (int) (1.25 * dwdhy);
      wstr (subtit, xx, yy, size, red);

      size = 1;
      xx = (int) (xl1 + (float) ipixrot * (0.5 + xtrarot) + 0.5)
           - ((strlen (lablong) * swshx) / 2) - 2;
      yy = ((yb1 - yt2) / 2 + yt2) + swshy/2;
      wstr (lablong, xx, yy, size, red);

      xx = (int) (xl1 + (float) ipixrot * xtrarot + 0.5) - swshx/2;
      wstr ("0", xx, yy, size, red);
      xx = (int) (xl1 + (float) ipixrot * (1.0 + xtrarot) + 0.5) - (swshx*3)/2;
      wstr ("360", xx, yy, size, red);

      xx = xl1 - swdhx * 3 - 10;
      wstr ("+90", xx, yt1    +swshy/2, size, grey);
      wstr ("  0", xx, yt1- 90+swshy/2, size, grey);
      wstr ("-90", xx, yt1-180+swshy/2, size, grey);

      wstr ("+90", xx, yt2    +swshy/2, size, grey);
      wstr ("  0", xx, yt2- 90+swshy/2, size, grey);
      wstr ("-90", xx, yt2-180+swshy/2, size, grey);

      xx = xx / 2 - swdhx;
      if (xx < 0)  xx = 0;
      yy = ((yb1 - yt2) / 2 + yt2) - swdhy/2;
      wstr ("L", xx, yy + swshy * 8, size, grey);
      wstr ("A", xx, yy + swshy * 6, size, grey);
      wstr ("T", xx, yy + swshy * 4, size, grey);
      wstr ("I", xx, yy + swshy * 2, size, grey);
      wstr ("T", xx, yy + swshy * 0, size, grey);
      wstr ("U", xx, yy - swshy * 2, size, grey);
      wstr ("D", xx, yy - swshy * 4, size, grey);
      wstr ("E", xx, yy - swshy * 6, size, grey);

      xxl = xl1 - 6;
      xxr = xl1 - 2;
      wvec(xxl,yt1,   xxr,yt1,grey);	/* Index mark +90 degrees, east limb. */
      wvec(xxl,yt1-30,xxr,yt1-30,grey); /* Index mark +60 degrees, east limb. */
      wvec(xxl,yt1-60,xxr,yt1-60,grey); /* Index mark +30 degrees, east limb. */
      wvec(xxl,yt1-90,xxr,yt1-90,grey); /* Index mark   0 degrees, east limb. */
      wvec(xxl,yt1-120,xxr,yt1-120,grey);/* Index mark -30 degrees, east limb.*/
      wvec(xxl,yt1-150,xxr,yt1-150,grey);/* Index mark -60 degrees, east limb.*/
      wvec(xxl,yt1-180,xxr,yt1-180,grey);/* Index mark -90 degrees, east limb.*/

      wvec(xxl,yt2,   xxr,yt2,grey);	/* Index mark +90 degrees, west limb. */
      wvec(xxl,yt2-30,xxr,yt2-30,grey); /* Index mark +60 degrees, east limb. */
      wvec(xxl,yt2-60,xxr,yt2-60,grey); /* Index mark +30 degrees, east limb. */
      wvec(xxl,yt2-90,xxr,yt2-90,grey); /* Index mark   0 degrees, west limb. */
      wvec(xxl,yt2-120,xxr,yt2-120,grey);/* Index mark -30 degrees, west limb.*/
      wvec(xxl,yt2-150,xxr,yt2-150,grey);/* Index mark -60 degrees, west limb.*/
      wvec(xxl,yt2-180,xxr,yt2-180,grey);/* Index mark -90 degrees, west limb.*/

      xx = xr1 + (511 - xr1) / 2 - swdhx * 2 + 1;
      if ((xx + 4 * swdhx) > 511) xx = 511 - 4 * swdhx;
      yy = ((yb1 - yt2) / 2 + yt2) - swdhy/2;
      wstr ("EAST", xx, yz1+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz1-swdhy,   size, grey);
      wstr ("WEST", xx, yz2+swdhy*2, size, grey);
      wstr ("LIMB", xx, yz2-swdhy,   size, grey);

      xx = xl2 + (xr2 - xl2) / 2 - (strlen (labdoy) * swshx) / 2 - 1;
      wstr (labdoy, xx, swshy + 7, size, green);

      /* * * * * * * * * * * * * * * * * * * * * * * * *
       *   Set up starting DOY values and x locations. *
       * * * * * * * * * * * * * * * * * * * * * * * * */

      crbeg = (float) ncrot - xtrarot;
      crend = (float) ncrot + xtrarot + 1.0;

      doybege = syndoy (iyear, ncrot) - syndur (ncrot) * (0.25 + xtrarot);
      doybegw = syndoy (iyear, ncrot) + syndur (ncrot) * (0.25 - xtrarot);

      if ((iyear/4) * 4 == iyear)  doymax = 366.0;  else doymax = 365.0;
      if (doybege > doymax)  doybege -= doymax;
      if (doybegw > doymax)  doybegw -= doymax;
      printf ("doybege: %f   doybegw: %f\n", doybege, doybegw);

      /*** East DOY initialization. ***/

      ndoye = (int) doybege;
      ibege = ndoye + 1;
      iende = ndoye + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
      posdoye = (float) xr1 + pixsam * samdoy * (doybege - (float) ndoye);
      printf ("ibege: %3d   iende: %3d   NROT: %d   xtrarot: %f   doyrot: %f\n",
               ibege, iende, NROT, xtrarot, doyrot);

      /*** West DOY initialization. ***/

      ndoyw = (int) doybegw;
      ibegw = ndoyw + 1;
      iendw = ndoyw + (int) (((float) NROT + 2.0 * xtrarot) * doyrot) + 2;
      posdoyw = (float) xr1 + pixsam * samdoy * (doybegw - (float) ndoyw);

      /*** Compute East DOYs and annotate Grinnell. ***/

      for (idoye=ibege; idoye<iende; idoye++) /* Days increase right to left. */
	 {
         posdoye -= pixsam * samdoy;
	 locdoye = (int) (posdoye + 0.5);
	 printf ("idoye:%d   posdoye:%6.2f  locdoye:%d   xl1:%d   xr1:%d\n",
		  idoye, posdoye, locdoye, xl1, xr1);
	 if (locdoye >= xl1 && locdoye <= xr1)
            {
	    wvec (locdoye, yt1+2, locdoye, yt1+6, green);
	    wvec (locdoye, yb1-2, locdoye, yb1-6, green);
	    if (idoye/5*5 == idoye)   /* Write 2 more tick marks every 5 deg. */
               {
	       wvec (locdoye-1, yt1+2, locdoye-1, yt1+6, green);
	       wvec (locdoye+1, yt1+2, locdoye+1, yt1+6, green);
	       wvec (locdoye-1, yb1-2, locdoye-1, yb1-6, green);
	       wvec (locdoye+1, yb1-2, locdoye+1, yb1-6, green);
	       sprintf (sdoy, "%d", idoye);
	       printf ("sdoy: %s\n", sdoy);
	       lxdoye = locdoye - (strlen(sdoy) * swdhx)/2;
	       lydoye = yt1+9+swdhy;
	       if (idoye/10*10 == idoye)	/* Write DOY every 10 degrees.*/
	          wstr (sdoy, lxdoye, lydoye, size, green);
               }
	    }
         }

      /*** Compute West DOYs and annotate Grinnell. ***/

      for (idoyw=ibegw; idoyw<iendw; idoyw++) /* Days increase right to left. */
	 {
         posdoyw -= pixsam * samdoy;
	 locdoyw = (int) (posdoyw + 0.5);
	 printf ("idoyw:%d   posdoyw:%6.2f  locdoyw:%d   xl1:%d   xr1:%d\n",
		  idoyw, posdoyw, locdoyw, xl1, xr1);

	 if (locdoyw >= xl1 && locdoyw <= xr1)
            {
	    wvec (locdoyw, yb2-2, locdoyw, yb2-6, green);
	    wvec (locdoyw, yt2+2, locdoyw, yt2+6, green);
	    if (idoyw/5*5 == idoyw)   /* Write 2 more tick marks every 5 deg. */
               {
	       wvec (locdoyw-1, yb2-2, locdoyw-1, yb2-6, green);
	       wvec (locdoyw+1, yb2-2, locdoyw+1, yb2-6, green);
	       wvec (locdoyw-1, yt2+2, locdoyw-1, yt2+6, green);
	       wvec (locdoyw+1, yt2+2, locdoyw+1, yt2+6, green);
	       sprintf (sdoy, "%d", idoyw);
	       lxdoyw = locdoyw - (strlen(sdoy) * swdhx)/2;
	       lydoyw = yb2-11;
	       if (idoyw/10*10 == idoyw)	/* Write DOY every 10 degrees.*/
	          wstr (sdoy, lxdoyw, lydoyw, size, green);
               }
	    }
         }

      /*** Map initialization. ***/

      fprintf (stdout, "crbeg: %8.3f   crend: %8.3f   xtrarot: %9.4f\n",
	       crbeg, crend, xtrarot);

      if (fseek (SYN, 0L, 0) == -1)
         {
         fprintf (stderr, "seek error: BOF\n");
         exit (-1);
         }
      if (fread ((char *) ST, sizST, 1, SYN) == 0)
         {
         fprintf (stderr, "read error: BOF\n");
         exit (-2);
         }

      if (ST->et < 1000.)  filedoy0 = ST->et;
      if (ST->et >= 1000.)  filedoy0 = ST->et - 2000.;
      idoy0 = (int) filedoy0;

      ibeg = ((int) doybege - idoy0) * samdoy
	     - 4;	/* Start record in data file. */
      if (ibeg < 0)  ibeg = 0;
      iend = ((int) doybegw - idoy0) * samdoy + (1.0 + xtrarot * 2.0) * samrot
	     + 4;	/* End record in data file. */

      printf ("crbeg : %8.3f   crend : %8.3f\n", crbeg, crend);
      printf ("idoy0: %d   ibeg: %d   iend: %d\n", idoy0, ibeg, iend);

      /*** Data loop for single rotation maps. ***/

      for (irec=ibeg; irec<iend; irec++)
         {

         /*** Read in record ***/

         offset = (long) (irec * sizST);
         if (fseek(SYN,offset,0) == -1)
            {
            fprintf(stderr,"seek error: loop  %d\n",i);
            exit (-1);
	    }
         if (fread ((char *)ST,sizST,1,SYN) == 0)
	    {
            fprintf(stderr,"read error: loop  %d\n",i);
            exit (-2);
            }
         fprintf (stdout, "irec: %3d   net:%7.2f   et:%7.2f   set:%7.2f\n",
                  irec, ST->net, ST->et, ST->set);
         fprintf (stdout, "           swt:%7.2f   wt:%7.2f   nwt:%7.2f\n",
                  ST->swt, ST->wt, ST->nwt);

         /*** East ok?  Then insert. ***/

         if ((ST->ec >= crbeg) && (ST->ec < crend))
            {
            xloc1 = xr1 - (int) ((ST->ec - crbeg) * ipixrot);
            printf("ec = %8.3f   xloc1 = %d  x1prev = %d\n", ST->ec, xloc1, x1prev);
	    x1prev = xloc1;

            /*** Screen index of slot. ***/

            for (jj=0; jj<180; jj++)
	       ibuf [jj] = (char) (ST->th [jj] * zscale);

	    for (jj=0; jj<nlat; jj++)
	       if (ibuf [jj] > irismax)  ibuf [jj] = irismax;

            xstart = xloc1;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl1 || jj > xr1) continue;
               wlin (FB, ibuf, jj, yt1, nlat, dir1);
               }
            }

         /*** West ok?  Then insert. ***/

         if ((ST->wc >= crbeg) && (ST->wc < crend))
            {
            xloc2 = xr2 - (int) ((ST->wc - crbeg) * ipixrot);
            fprintf (stdout, "wc = %8.3f   xloc2 = %d  x2prev = %d\n",
		     ST->wc, xloc2, x2prev);
	    x2prev = xloc2;

            /*** Screen index of slot. ***/

            for (jj=180; jj<361; jj++)
               ibuf [541-jj] = (char) (ST->th [jj] * zscale);

	    for (jj=180; jj<361; jj++)
	       if (ibuf [jj] > irismax)  ibuf [jj] = irismax;

            xstart = xloc2;
	    xstop  = xstart - xwidth;
            for (jj=xstart; jj >= xstop; jj--)
               {
               if (jj < xl2 || jj > xr1)  continue;
               wlin (FB, &ibuf [180], jj, yt2, nlat, dir1);
               }
            }  /* West Limb. */
         } /* Data loop. */

      /*** Draw boxes around synoptic map image area. ***/

      wlvec (xbox1, ybox1, 5, green);	/* Box frame for east limb. */
      wvec (xl1, yt1-90, xr1, yt1-90, green);/* Equatorial line for east limb.*/

      wlvec (xbox2, ybox2, 5, green);	/* Box frame for west limb. */
      wvec (xl2, yt2-90, xr2, yt2-90, green);/* Equatorial line for west limb.*/

      xx = xr1 - (int) (xtrarot * (float) ipixrot + 0.5);
      wvec (xx, yt1,    xx, yb1-6, red);/* Index mark   0 degrees longitude. */
      wvec (xx, yt2+6,  xx, yb2,   red);/* Index mark   0 degrees longitude. */

      xx = xr1 - (int) ((xtrarot + 0.5) * (float) ipixrot + 0.5);
      wvec (256, yb1-1, 256, yb1-6, red);/* Index mark 180 degrees longitude.*/
      wvec (256, yt2+6, 256, yt2+1, red);/* Index mark 180 degrees longitude.*/

      xx = xr1 - (int) ((xtrarot + 1.0) * (float) ipixrot + 0.5);
      wvec (xx, yt1,    xx, yb1-6, red); /* Index mark 360 degrees longitude. */
      wvec (xx, yt2+6,  xx, yb2, red);	 /* Index mark 360 degrees longitude. */

      /*** Draw bugle lines to mark Coronal Mass Ejections. ***/


      printf ("^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n");
      doybege = syndoy (iyear, ncrot) - syndur (ncrot) * (0.25 + xtrarot);
      doybegw = syndoy (iyear, ncrot) + syndur (ncrot) * (0.25 - xtrarot);
      doyende = doybege + syndur (ncrot) * (1.0 + xtrarot * 2.0);
      doyendw = doybegw + syndur (ncrot) * (1.0 + xtrarot * 2.0);
      printf ("pixsam: %7.2f   samdoy: %d\n", pixsam, samdoy);
      printf ("xr1:%3d   yb1:%3d   xr2:%3d   yb2:%3d\n", xr1, yb1, xr2, yb2);
      printf ("doybege:%7.2f   doyende:%7.2f   doybegw:%7.2f   doyendw:%7.2f\n",
               doybege, doyende, doybegw, doyendw);

      if ((CME = fopen (namcme, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open CME file: %s\n", namcme);
         exit (-1);
         }
      for (i=0; i<5; i++)  fgetline (CME, ibuf); /* Skip first 5 lines. */

      while (fscanf (CME, "%d %f %f %d:%d %d:%d %d %d",
             &cmeyr, &cmedoy1, &cmedoy2, &cmehr1, &cmemin1, &cmehr2, &cmemin2,
             &cenang, &widang) != EOF)
         
         {
         cmedoy1 += cmehr1 / 24.0 + cmemin1 / 3600.0;	/* CME begin DOY. */
         cmedoy2 += cmehr2 / 24.0 + cmemin2 / 3600.0;	/* CME end   DOY. */
         cmedur = cmedoy2 - cmedoy1;			/* CME duration (DOY).*/

         printf ("cmeyr:%2d   iyear:%2d\n", cmeyr, iyear);
         if (cmeyr > 1900)  cmeyr -= 1900;

         if (cmeyr != iyear)  continue;
         if (cmeyr >  iyear)  break;
         if (cmedoy2 > doyendw)  break;

         printf ("<---> cmedoy1:%7.2f  %02d:%02d   cenang:%3d   widang:%3d\n",
                  cmedoy1, cmehr1, cmemin1, cenang, widang);
         printf ("<---> cmedoy2:%7.2f  %02d:%02d\n",
                  cmedoy2, cmehr2, cmemin2);
         begang = cenang - widang / 2;
         endang = cenang + widang / 2;
         printf ("<---> cmedoy:%7.2f,%7.2f          begang:%3d   endang:%3d\n",
                  cmedoy1, cmedoy2, begang, endang);

         /*** Draw East Limb CME box. ***/

         if (cmedoy1 >= doybege && cmedoy1 <= doyende ||
             cmedoy2 >= doybege && cmedoy2 <= doyende)
            {
            xbeg = (cmedoy1 - doybege) * pixsam * samdoy;
            xend = (cmedoy2 - doybege) * pixsam * samdoy;

            ybeg = 180 - endang;
            yend = 180 - begang;
            if (ybeg < 0)  ybeg = 0;
            if (yend < 0)  yend = 0;

            printf ("east> cmedoy:%7.2f-%7.2f   xloc1:%3d   ybeg:%3d   yend:%3d\n",
                     cmedoy1, cmedoy2, xloc1, ybeg, yend);
            ybeg += yb1 - 1;
            yend += yb1 - 1;
            xbeg = xr1 - xbeg;
            xend = xr1 - xend;

            /*** Clip x/y coordinates at synoptic map boundaries. ***/

            if (xbeg < xl1)  xbeg = xl1;
            if (xbeg > xr1)  xbeg = xr1;
            if (xend < xl1)  xend = xl1;
            if (xend > xr1)  xend = xr1;
            if (ybeg < yb1)  ybeg = yb1;
            if (ybeg > yt1)  ybeg = yt1;
            if (yend < yb1)  yend = yb1;
            if (yend > yt1)  yend = yt1;
            printf ("east> cmedoy:%7.2f-%7.2f   xbeg:%3d   xend:%3d   ybeg:%3d   yend:%3d\n",
                     cmedoy1, cmedoy2, xbeg, xend, ybeg, yend);

            if (ybeg != yend)
               {
               if (ldash)				/* Dashed lines. */
                  {
                  wvdash (xbeg, ybeg, yend, 3, cmecol);
                  wvdash (xend, ybeg, yend, 3, cmecol);
/*
 *                  whdash (xend, xbeg, ybeg, 3, cmecol);
 *                  whdash (xend, xbeg, yend, 3, cmecol);
 */
                  }
               else					/* Solid lines. */
                  {
                  wvec (xbeg, ybeg, xbeg, yend, cmecol);
                  wvec (xend, ybeg, xend, yend, cmecol);
/*
 *                  wvec (xbeg, ybeg, xend, ybeg, cmecol);
 *                  wvec (xbeg, yend, xend, yend, cmecol);
 */
                  }
               wvec (xbeg, ybeg, xend, ybeg, cmecol);
               wvec (xbeg, yend, xend, yend, cmecol);
               }
            }

         /*** Draw West Limb CME box. ***/

         if (cmedoy1 >= doybegw && cmedoy1 <= doyendw ||
             cmedoy2 >= doybegw && cmedoy2 <= doyendw)
            {
            xbeg = (cmedoy1 - doybegw) * pixsam * samdoy;
            xend = (cmedoy2 - doybegw) * pixsam * samdoy;

            if (begang < 0)
               {
               begang += 360;
               endang += 360;
               }
            ybeg = begang - 180;
            yend = endang - 180;
            if (ybeg < 0)  ybeg  =   0;
            if (yend < 0)  yend  =   0;

            printf ("west> cmedoy:%7.2f-%7.2f   xbeg:%3d   xend:%3d   ybeg:%3d   yend:%3d\n",
                     cmedoy1, cmedoy2, xbeg, xend, ybeg, yend);
            ybeg += yb2 - 1;
            yend += yb2 - 1;
            xbeg = xr2 - xbeg;
            xend = xr2 - xend;

            /*** Clip x/y coordinates at synoptic map boundaries. ***/

            if (xbeg < xl2)  xbeg = xl2;
            if (xbeg > xr2)  xbeg = xr2;
            if (xend < xl2)  xend = xl2;
            if (xend > xr2)  xend = xr2;
            if (ybeg < yb2)  ybeg = yb2;
            if (ybeg > yt2)  ybeg = yt2;
            if (yend < yb2)  yend = yb2;
            if (yend > yt2)  yend = yt2;
            printf ("west> cmedoy:%7.2f-%7.2f   xbeg:%3d   xend:%3d   ybeg:%3d   yend:%3d\n",
                     cmedoy1, cmedoy2, xbeg, xend, ybeg, yend);

            if (ybeg != yend)
               {
               if (ldash)				/* Dashed lines. */
                  {
                  wvdash (xbeg, ybeg, yend, 3, cmecol);
                  wvdash (xend, ybeg, yend, 3, cmecol);
/*
 *                  whdash (xend, xbeg, ybeg, 3, cmecol);
 *                  whdash (xend, xbeg, yend, 3, cmecol);
 */
                  }
               else					/* Solid lines. */
                  {
                  wvec (xbeg, ybeg, xbeg, yend, cmecol);
                  wvec (xend, ybeg, xend, yend, cmecol);
/*
 *                  wvec (xbeg, ybeg, xend, ybeg, cmecol);
 *                  wvec (xbeg, yend, xend, yend, cmecol);
 */
                  }
               wvec (xbeg, ybeg, xend, ybeg, cmecol);
               wvec (xbeg, yend, xend, yend, cmecol);
               }
            }
         } 

      }		/* End of single rotation map. */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   closepl ();
   fb_close ();
}


/* whdash
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write a horizontal dashed line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
whdash (xbeg, xend, yloc, dashlen, cmap)

int xbeg, xend;			/* Start/End x location. */
int yloc;			/* y location. */
int dashlen; 			/* Dash length (pixels). */
int cmap;			/* Color map index. */

{
   int x1, x2;
   printf ("whdash> xbeg:%3d   xend:%3d   yloc:%3d   dashlen:%d\n",
            xbeg, xend, yloc, dashlen);

   /*** Write short vertical lines at left & right of CME line. ***/

   wvec (xbeg-2, yloc, xbeg+2, yloc, cmap);
   wvec (xend-2, yloc, xend+2, yloc, cmap);

   /*** Write horizontal dashed line to mark CME. ***/

   for (x1 = xbeg+2; x1 < xend; x1 += dashlen*2)
      {
      x2 = x1 + dashlen - 1;
      if (x2 > (xend - 2))  x2 = xend - 2;
      wvec (x1, yloc, x2, yloc, cmap);
      printf ("whdash> x1:%3d   x2:%3d   yloc:%3d\n", x1, x2, yloc);
      }

   return (0);
}



/* wvdash
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write a vertical dashed line.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
wvdash (xloc, ybeg, yend, dashlen, cmap)

int xloc;			/* x location. */
int ybeg, yend;			/* Start/End y location. */
int dashlen; 			/* Dash length (pixels). */
int cmap;			/* Color map index. */

{
   int y1, y2;

   /*** Write short horizontal lines at top and bottom of CME line. ***/

   wvec (xloc-2, ybeg, xloc+2, ybeg, cmap);
   wvec (xloc-2, yend, xloc+2, yend, cmap);

   /*** Write vertical dashed line to mark CME. ***/

   for (y1 = ybeg+2; y1 < yend; y1 += dashlen*2)
      {
      y2 = y1 + dashlen - 1;
      if (y2 > (yend - 2))  y2 = yend - 2;
      wvec (xloc, y1, xloc, y2, cmap);
      }

   return (0);
}
