/* synmap.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P synoptic data file (csyn8X.rNN)
 * and generate X-Y array (longitude-latitude): 361x181 pixels.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 3 November 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>
#include <gl/gl.h>
#include <device.h>
#include <fmclient.h>
#include "synst.h"

struct S STT;

#define DSX	 64
#define DSY	181
#define DCX	361
#define DCY	181
main (argc, argv)

int argc;
char *argv [];
{
   float syndur ();
   float synrot ();
   register struct S *ST = &STT;	/* pointer to synoptic record. */
   FILE	*SYN;				/* file pointer. */

   static double nec [DSX];
   static char namout [80];	/* Name of output C/P synoptic map file. */
   static char namsyn [80];	/* Name of C/P synoptic data file. */
   static Colorindex simg [DSY][DSX];		/* 2D synoptic map array. */
   static unsigned short smap [DCY][DCX];	/* 2D synoptic map array. */
   static unsigned char cmap [DCY][DCX];	/* 2D synoptic map array. */

   double crrot;	/* Carrington rotation value. */
   double crfrac;	/* Carrington rotation fraction. */
   double rotmax;	/* Limit for interpolation array. */

   float filedoy;	/* DOY at start of synoptic map data file. */
   float filedoy0;	/* DOY at start of synoptic map data file. */
   float filerot0;	/* Rotation (cmp) at start of synoptic map data file. */
   float rrot;		/* record carrington rotation. */
   float radius;	/* Radius at which synoptic scan is performed. */
   float frac, fix;
   float slope;
   float smax = 650.0;		/* Maximum calibrated pixel value allowed. */

   int crot;			/* Carrington rotation of current record. */
   int fd;
   int i;
   int idot;
   int icr;
   int irad;
   int irec;
   int irot;
   int ix, iy;
   int iyear;
   int maxrot;
   int ncrot0;			/* First Carrington rotation in data file. */
   int nlong;			/* # longitude values for interpolation. */
   int nread;
   int sizST;			/* Size (bytes) of data record. */
   int xll, yll, xur, yur;

   unsigned short soff = 512;	/* color index offset for colormap #2. */
   unsigned short sval;
   unsigned short *si, *sm;

   /* WHIP header. */

   static unsigned short pichdr [12] =
   {18768, 0, 0, 2, 1, 1, DCY, DCX, sizeof (char) * 8, sizeof (char) * 8, 0, 0};

   char	*index (), *p8;			/* to find year of file	*/
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   si = (unsigned short *) simg;
   sm = (unsigned short *) smap;
   slope = 250.0 / smax;
   irec = 0;
   xll = 0;
   yll = 0;

   /*** Get synoptic map data file name. ***/

/*
   fprintf (stderr,"%s  Version 1.00\n", argv [0]);
 */

   if (argc == 2)
      {
      strcpy (namsyn, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s [synoptic_filename]\n", argv [0]);
      exit (0);
      }

   /*** Open SMM C/P synoptic data file. ***/

   if ((SYN = fopen (namsyn,"r")) == NULL)
      {
      fprintf (stderr, "cannot open synoptic file: %s\n", namsyn);
      exit (-1);
      }

   /*** Read in first record. ***/

   sizST = sizeof (STT);	/* Size (bytes) of synoptic data structure. */
   if (fseek (SYN, 0L, 0) == -1)
      {
      fprintf (stderr, "seek error: BOF\n");
      exit (-1);
      }
   if (fread ((char *) ST, sizST, 1, SYN) == 0)
      {
      fprintf (stderr, "read error: BOF\n");
      exit (-2);
      }

   /*** Read until a non-zero carrington rotation is found. ***/

   while (fread ((char *) ST, sizST, 1, SYN) == 1)
      {
      if ((int) ST->nec > 0)  break;
      }

   /*
    * Find starting rotation number from file name.
    */

   p8 = index (namsyn, '8');
   if (*(p8+1) == '0')  iyear = 80;
   if (*(p8+1) == '4')  iyear = 84;
   if (*(p8+1) == '5')  iyear = 85;
   if (*(p8+1) == '6')  iyear = 86;
   if (*(p8+1) == '7')  iyear = 87;
   if (*(p8+1) == '8')  iyear = 88;
   if (*(p8+1) == '9')  iyear = 89;

   for (i = 0; i < strlen (namsyn); i++)  if (namsyn [i] == '.')  idot = i;
   sscanf (&namsyn [idot+2], "%2d", &irad);
   radius = (float) (irad) / 10.0;

/*
   fprintf (stdout, "iyear:%d   ST->et: %f   radius: %5.2f\n",
	    iyear, ST->et, radius);
 */

   if (ST->et <  1000.0)  filedoy0 = ST->et;
   if (ST->et >= 1000.0)  filedoy0 = ST->et - 2000.0;
   filerot0 = synrot (iyear, filedoy) + 0.25;
   ncrot0 = (int) filerot0;

   irot = ncrot0;
   irot = (int) ST->nec;
   rotmax = (double) (irot + 1) + 1.0 / 55.0;
   maxrot = irot + 14;

   /*** Create GL image window. ***/

   foreground ();
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   winopen (argv [0]);

   /*** Carrington Rotation loop. ***/

   for (icr = 0; icr < 14; icr++)
      {
      nlong = 0;
/*
      printf ("icr:%2d   irot:%d   irec:%d   nread:%d   ST->nec:%9.4f\n",
               icr, irot, irec, nread, ST->nec);
 */

      /*** Clear synoptic map array to all zeroes. ***/

      for (ix = 0; ix < DSX; ix++)
	 {
	 nec [ix] = 0.0;
         for (iy = 0; iy < DSY; iy++)
	    simg [iy][ix] = 0;
         }

      for (ix = 0; ix < DCX; ix++)
         for (iy = 0; iy < DCY; iy++)
	    {
	    cmap [iy][ix] = 0;
	    smap [iy][ix] = 0;
	    }

      /*** Ready synoptic file records. ***/

      ix = -1;
      while (nread = fread ((char *) ST, sizST, 1, SYN) == 1)
         {

	 /*** Determine carrington rotation number. ***/

         if (ST->et <  1000.0)  filedoy = ST->et;
         if (ST->et >= 1000.0)  filedoy = ST->et - 2000.0;
         rrot = synrot (iyear, filedoy) + 0.25;
	 crot = (int) rrot;
/*
         printf ("ST->et:%9.4f   filedoy:%6.2f   rrot:%9.4f   crot:%4d   irot:%4d\n",
		  ST->et, filedoy, rrot, crot, irot);
 */


         if (rrot > rotmax)  break;		/* Quit loop. */

/*
         printf ("irec:%4d   rrot:%9.3f   rotmax:%9.3f\n", irec, rrot, rotmax);
 */
         irec++;
         ix++;

/*
	 frac = rrot / (float) irot;
	 fix = frac * 56.0;
	 ix = (int) fix;
 */
/*
	 printf ("ST->nec:%9.4f   irot:%4d   frac:%6.4f   fix:%7.2f   ix:%4d\n",
		  ST->nec, irot, frac, fix, ix);
 */

         nec [ix] = rrot;
	 nlong++;

/*
	 printf ("store: ix:%3d  nec:%9.3f\n", ix, nec [ix]);
 */


         /* Copy scan values into XY array. */
	 /* Reverse array left <--> right.*/

         for (iy = 0; iy < 181; iy++)
	    {
	    simg [iy][ix] = ST->th [iy];
/*	    smap [iy][ix] = (unsigned short) (ST->th [iy] * slope) + soff; */
            }
         }

         /*** --- --- --- --- --- End of rotation loop --- --- --- --- --- ***/

/*
      xur = DCX - 1;
      yur = DCY - 1;
      rectwrite (xll, yll, xur, yur, sm);
 */

      for (ix = 0; ix < DCX; ix++)		/* Longitude loop. */
	 {
	 /*** Locate indices in nec array to use for interpolation. ***/

         crrot = (float) (360 - ix) / 360.0 + (float) irot;
	 for (i = 0; i < nlong; i++)
	    {
	    if (crrot < nec [i])  break;
	    }
/*
         printf ("interp: ix:%3d  i:%3d  crday:%9.3f\n", ix, i, crrot);
 */

         /*** Perform image interpolation: [181][60] --> [181][361]. ***/

         crfrac = (crrot - nec [i]) / (nec [i] - nec [i-1]);
	 for (iy = 0; iy < 181; iy++)		/* Latitude loop. */
	    {
	    sval = (unsigned short)
	       (crfrac * (simg [iy][i] - simg [iy][i-1])) + simg [iy][i];
            sval *= slope ;
	    if (sval > 249) sval = 249;
	    smap [iy][ix] = sval + soff;

	    cmap [iy][ix] = (unsigned char) (sval & 0377);
/*
	    if (ix == iy)
	    printf ("ix:%3d  iy:%3d  simg:%3d   smap:%3d  cmap:%3d\n",
		    ix, iy, simg [iy][i], smap [iy][ix], cmap [iy][ix]);
 */
            }
	 }

      xur = DCX - 1;
      yur = DCY - 1;
      rectwrite (xll, yll, xur, yur, sm);

      /*** Reposition file pointer backwards one record. ***/

      if (fseek (SYN, -sizST*3, SEEK_CUR) == -1)
         {
         fprintf (stderr, "seek error: BOF\n");
         exit (-1);
         }

      /*** Quit if EOF reached. ***/

      if (nread != 1)
	 {
	 gexit ();
	 exit (0);
	 }

   /*** Write Carrington Rotation map array to disk file. ***/

   sprintf (namout, "cpsyn%04d.%d.r%02d.pic", irot, iyear, irad);
   fd = creat (namout, 0644);
   close (fd);
   fd = open (namout, 1);
   write (fd, pichdr, sizeof (pichdr));
   write (fd, cmap, DCX*DCY);
   close (fd);
   irot = crot;
   rotmax = (double) (irot + 1) + 1.0 / 55.0;
   }
}
