/* synplot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Synoptic data file plotter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 2 June 1988
 * 4 August 1988: Change angles reported; start with 30 instead of 0 degrees.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "synst.h"
int pcount = 0 ;

struct S STT ;

main ()
{
   static char namfil [80] ;
   static char answer [80] ;
   static char zip [2] ;
   static char namemc [80] = "synplot.cgm" ;
   static char title [60] ;
   static char labx [40] ;
   static char laby [40] ;
   static char desig [2][4] = {"A","B"} ;
   static char gksout [80] ;

   FILE *SYN, *fopen() ;

   register struct S *ST = &STT ;

   int cmd ;
   int doy0 ;
   int doybeg, doyend ;
   int hdoy ;
   int i ;
   int ig = 1 ;
   int irec, irec1, irec2 ;
   int iyear, idoy, irad ;
   int ixscale = 0, iyscale = 0, ibkg = 0 ;
   int nval = 361 ;
   int sizST ;
   int ipa ;

   long offset ;

   float doy ;
   float frad ;
   float fpa ;
   float fdoybeg, fdoyend ;
   float fxmin = 0.0, fxmax = 0.0 ;
   float fymin = 0.0, fymax = 0.0 ;

   static float posang [361], brite [361] ;
   static float fdoy [750], tbright [750] ;

   sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc) ;
   putenv (gksout) ;
   opngks_ () ;

   for (i = 0; i < 361; i++)
      {
      posang [i] = (float) i ;
      brite [i]  = 0.0 ;
      }


   sizST = sizeof (STT) ;

   fprintf (stderr, "Enter synoptic year (80,84-89) : ") ;
   fscanf  (stdin, "%d", &iyear) ;

   fprintf (stderr, "Enter radius (2, 2.8, 3.4): ") ;
   fscanf  (stdin, "%f", &frad) ;

   irad = (int) (frad * 10.0 + 0.5) ;
   sprintf (namfil, "csyn%02d.r%02d", iyear, irad) ;

   printf ("Synoptic Data File: %s\n", namfil) ;

   fprintf (stderr, "Enter synoptic data file name: ") ;
   fscanf  (stdin, "%s", namfil) ;

   /*** Open synoptic data file, read first record. ***/

   if ((SYN = fopen (namfil, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", namfil) ;
      exit (-1) ;
      }

   if (fread ((char *) ST, sizST, 1, SYN) == 0)
      {
      fprintf (stderr, "Unable to read 1st record in file.\n") ;
      exit (-1) ;
      }

   doy0 = (int) ST->et ;
   if (doy0 > 1000)  doy0 -= 2000 ;
   fprintf (stderr, "doy0: %d\n", doy0) ;

   /*** Get option. ***/

next:

   fprintf (stderr, "Choose one: e [examine], l [list], i [intensity report] a [angular scan]  t [time plot] or q [quit]: ") ;
   fscanf  (stdin,  "%s", answer) ;
   cmd = (int) answer[0] ;

   switch (cmd)
      {

      /*** Examine specified range of records. ***/

      case 'e':
	 fprintf (stderr, "Enter start & stop DOY's: ") ;
	 fscanf  (stdin,  "%d%c%d", &doybeg, zip, &doyend) ;
	 irec1 = (doybeg - doy0) * 2 ;
	 irec2 = (doyend - doy0) * 2 + 1 ;
	 offset = irec1 * sizST ;
	 fprintf (stderr, "irec1:%d   irec2:%d   offset:%d\n", irec1, irec2, offset) ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;
         fprintf (stdout, "rec     net      et     set     swt      wt     nwt  Position Angle Data:\n        nec      ec     sec     swc      wc     nwc  30  90 150 210 270 330\n") ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;
	 if (fseek (SYN, offset, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # %d  offset: %d\n",
		     irec1, offset) ;
	    break ;
	    }

	 for (irec=irec1 ; irec<=irec2 ; irec++)
	    {
            if (fread ((char *) ST, sizST, 1, SYN) == 0)
               {
               fprintf (stderr, "Unable to read record # %d in file.\n", irec) ;
               exit (-1) ;
               }
            fprintf (stdout, "%3d %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f\n    %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
	             irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                   ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
		           ST->th[30],   ST->th[90],  ST->th[150],
		           ST->th[210], ST->th[270], ST->th[330]) ;
	    }
          break ;

      /*** List entire file. ***/

      case 'l':
         irec = -1 ;

	 if (fseek (SYN, 0L, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # 0  offset: 0\n") ;
	    break ;
	    }
         while (fread ((char *) ST, sizST, 1, SYN) != 0)
            {
            irec++ ;
            if (((irec/50)*50) == irec)
               {
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;
         fprintf (stdout, "rec     net      et     set     swt      wt     nwt  Position Angle Data:\n        nec      ec     sec     swc      wc     nwc  30  90 150 210 270 330\n") ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;
	       }
            fprintf (stdout, "%3d %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f\n    %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
	             irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                   ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
		           ST->th[30],  ST->th[90],  ST->th[150],
		           ST->th[210], ST->th[270], ST->th[330]) ;
            }
         break ;

      /*** Intensity Report. ***/

      case 'i':
	 
         irec = -1 ;

	 if (fseek (SYN, 0L, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # 0  offset: 0\n") ;
	    break ;
	    }

         fprintf (stdout, "SMM C/P Calibrated Synoptic Data\n") ;
	 fprintf (stdout, "Brightness = V * (B[k] + B[f]) + Stray Light   (coefficient of 10^-11 Bsun)\n") ;
	 fprintf (stdout, "Year: 19%2d   Height: %7.2f Rsun\n", iyear, frad) ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;
         fprintf (stdout, "CME DOY     0    30    60    90   120   150   180   210   240   270   300   330\n") ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;

         while (fread ((char *) ST, sizST, 1, SYN) != 0)
            {
            irec++ ;
            if (ST->net < 1000.0)
            fprintf (stdout, "%7.2f  %4d  %4d  %4d  %4d  %4d  %4d  %4d  %4d  %4d  %4d  %4d  %4d\n",
	             ST->net,
		     ST->th [  0], ST->th [ 30], ST->th [ 60], ST->th [ 90],
		     ST->th [120], ST->th [150], ST->th [180], ST->th [210],
		     ST->th [240], ST->th [270], ST->th [300], ST->th [330]) ;
            }
         break ;


      /*** Angular Scan Plot. ***/

      case 'a':

         sprintf (gksout, "NCARG_GKS_OUTPUT=%s", namemc);
         putenv (gksout);

	 fprintf (stderr, "Enter DOY: ") ;
	 fscanf  (stdin,  "%d", &doybeg) ;
	 doyend = doybeg ;
	 irec1 = (doybeg - doy0) * 2 ;
	 irec2 = (doyend - doy0) * 2 + 1 ;
	 fprintf (stderr, "doybeg: %3d  doyend: %3d %3d\n", doybeg, doyend) ;
	 offset = irec1 * sizST ;
	 fprintf (stderr, "irec1:%d   irec2:%d   offset:%d\n", irec1, irec2, offset) ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;
         fprintf (stdout, "rec     net      et     set     swt      wt     nwt  Position Angle Data:\n        nec      ec     sec     swc      wc     nwc  30  90 150 210 270 330\n") ;
	 fprintf (stdout, "--------------------------------------------------------------------------------\n") ;

	 if (fseek (SYN, offset, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # %d  offset: %d\n",
		     irec1, offset) ;
	    break ;
	    }

	 for (irec=irec1 ; irec<=irec2 ; irec++)
	    {
            if (fread ((char *) ST, sizST, 1, SYN) == 0)
               {
               fprintf (stderr, "Unable to read record # %d in file.\n", irec) ;
               exit (-1) ;
               }
            fprintf (stdout, "%3d %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f\n    %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %3d %3d %3d %3d %3d %3d\n",
	             irec, ST->net, ST->et, ST->set, ST->swt, ST->wt, ST->nwt,
	                   ST->nec, ST->ec, ST->sec, ST->swc, ST->wc, ST->nwc,
		           ST->th[30],   ST->th[90],  ST->th[150],
		           ST->th[210], ST->th[270], ST->th[330]) ;

            for (i = 0; i < nval; i++)
               {
               brite [i]  = (float) ST->th [i] * 1.0e-11 ;
               }
            hdoy = (irec / 2) * 2 ;
	    if (hdoy == irec) hdoy = 0 ;  else hdoy = 1 ;
            sprintf (title, "Synoptic Map Theta Scan: DOY %03d-%s 19%2d %3.1f Rsun",
		     doybeg, &desig [hdoy][0], iyear, frad) ;

            sprintf (labx, "Position Angle [degrees]") ;
            sprintf (laby, "Brightness [B/Bsun]") ;

/*
 *	    ezxy_ (posang, brite, &nval, title, strlen (title)) ;
 */

            xyplot (posang, brite, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg) ;

	    }
          break ;
   case 't':
	 fprintf (stderr, "Enter start & stop DOY's: ") ;
	 fscanf  (stdin,  "%d%c%d", &doybeg, zip, &doyend) ;
         fdoybeg = (float) doybeg ;
         fdoyend = (float) doyend ;
	 fprintf (stderr, "fdoybeg/fdoyend: %7.2f  %7.2f\n", fdoybeg, fdoyend) ;

	 fprintf (stderr, "Enter Position Angle [0-360 degrees]: ") ;
	 fscanf  (stdin,  "%f", &fpa) ;
	 ipa = (int) (fpa + 0.5) ;

         irec = -1 ;
	 i = 0 ;

	 if (fseek (SYN, 0L, 0) == -1)
	    {
	    fprintf (stderr, "Seek error: record # 0  offset: 0\n") ;
	    break ;
	    }
         while (fread ((char *) ST, sizST, 1, SYN) != 0)
            {
            irec++ ;
            if (ST->net < fdoybeg || ST->net > fdoyend)  continue ;
	    if (ST->th [ipa] == 0.0)  continue ;

            fdoy [i] = ST->net ;
            tbright [i]  = (float) ST->th [ipa] * 1.0e-11 ;
	    printf ("i: %6d   fdoy: %8.3f   tbright: %8.2e\n",
		     i, fdoy [i], tbright [i]) ;
	    i++ ;
	    }
         nval = i ;

         sprintf (title, "SMM C/P Time Plot: 19%2d %3d Deg. %3.1f Rsun",
                  iyear, ipa, frad) ;
         sprintf (labx, "19%02d Day-Of-Year", iyear) ;
         sprintf (laby, "Brightness [B/Bsun]") ;


/*
 *	 ezxy_ (fdoy, tbright, &nval, title, strlen (title)) ;
 */

         fxmin = 0.0 ;
	 fxmax = 366.0 ;

	 fxmin = (float) doybeg ;
	 fxmax = (float) doyend ;

	 fymin = fymax = 0.0 ;

	 if (irad == 20)
	    {
	    fymin = 0.1e-9 ;
	    fymax = 7.5e-9 ;
	    }
         else
	 if (irad == 28)
	    {
	    fymin = 2.0e-10 ;
	    fymax = 5.0e-9  ;
	    }
         else
	 if (irad == 34)
	    {
	    fymin = 2.0e-10 ;
	    fymax = 3.0e-9  ;
	    }

         printf ("fxmin/fxmax: %8.2e   %8.2e\n", fxmin, fxmax) ;
         printf ("fymin/fymax: %8.2e   %8.2e\n", fymin, fymax) ;

/*
         xyplot (fdoy, tbright, nval, namemc, title, labx, laby,
		 ixscale, iyscale, ibkg) ;
 */

         xysplot (fdoy, tbright, fxmin, fxmax, fymin, fymax, nval, namemc,
		  title, labx, laby, ixscale, iyscale, ibkg, ig) ;

         break ;


      /*** Quit. ***/

      case 'q':

         clsgks_ () ;
	 exit (0) ;

      default:
	 break ;
      }
   goto next ;

}
