/* wlin.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write a line of pixel data in current FB using WHIP functions
 * fb_setaddr & fb_write.
 * Assumes fb_open () has been called and a frame buffer is selected.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 26 February 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

wlin (fb, buf, xloc, yloc, numpix, dir1)

int fb;			/* Frame buffer. */
char *buf;		/* Character string to be written. */
int xloc, yloc;		/* Pixel location for start of pixel data. */
int numpix;		/* # pixels to write. */
char *dir1;		/* direction of write: 'r', 'l', 'u', 'd'. */

{
   int xpos, ypos;
   int nbypx = 1;
   int zloc = 0;
   int rel = 0;

/*
 *   fprintf (stdout, ":-:xloc: %3d   yloc: %3d   dir1: %s\n",
 *                     xloc, yloc, dir1);
 *   fflush (stdout);
 */

   switch ((int) *dir1)
      {
      case 'r':						/* rightward. */
		xpos = xloc;
		ypos = yloc;
		fb_xstep (fb);
/*
 *   fprintf (stdout, ":r:xpos: %3d   ypos: %3d   dir1: %s\n",
 *                     xpos, ypos, dir1);
 */
		break;
      case 'l':						/* leftward. */
		xpos = xloc - numpix + 1;
		ypos = yloc;
		xloc = xloc;
		fb_xstep (fb);
		revorder (buf, nbypx, numpix);
/*
 *   fprintf (stdout, ":l:xpos: %3d   ypos: %3d   dir1: %s\n",
 *                     xpos, ypos, dir1);
 */
		break;
      case 'd':						/* upward. */
		xpos = xloc;
		ypos = yloc;
		fb_ystep (fb);
/*
 *   fprintf (stdout, ":d:xpos: %3d   ypos: %3d   dir1: %s\n",
 *                     xpos, ypos, dir1);
 */
                break;
      case 'u':						/* downward. */
		xpos = xloc;
		ypos = yloc + numpix - 1;
		fb_ystep (fb);
		revorder (buf, nbypx, numpix);
/*
 *   fprintf (stdout, ":u:xpos: %3d   ypos: %3d   dir1: %s\n",
 *                     xpos, ypos, dir1);
 */
		break;
      default:
/*
 *		fprintf (stderr, "Invalid direction: %s\n", dir1);
 */
		return (-1);
      }


   /*** Set current FB position (xloc, yloc, zloc). ***/

   fb_setaddr (fb, rel, xpos, ypos, zloc);

   /*** Write pixel data. ***/

   fb_write (buf, numpix);

   return (0);
}


/* revorder
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse order of elements in an array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
revorder (buf, nbypx, nval)

char *buf;		/* Pointer to array. */
int nbypx;		/* # bytes per element. */
int nval;		/* # elements in array. */

{
   int i, ni;
   int ihalf, imax;
   char cval;
   short sval;
   float fval;
   short *sbuf;
   float *fbuf;

/*
 *   printf ("   revorder> nbypx: %d   nval: %d\n", nbypx, nval);
 */
   sbuf = (short *) buf;
   fbuf = (float *) buf;

   /*** Reverse order of elements of array. ***/

   imax = nval - 1;
   ihalf = nval / 2;

   if (nbypx == 1)				/* Byte image. */
      {
      for (i = 0; i < ihalf; i++)
         {
         ni = imax - i; 
         cval = *(buf + i);
         *(buf + i) = *(buf + ni);
         *(buf + ni) = cval;
/*
 *      printf ("***revorder> i: %3d   buf[%3d]: %3d   buf[%3d]: %3d\n",
 *               i, i, *(buf+i), ni, *(buf+ni));
 */
         }
      }
   else
   if (nbypx == 2)				/* Short integer image. */
      {
      for (i = 0; i < ihalf; i++)
         {
         ni = imax - i; 
         sval = *(sbuf + i);
         *(sbuf + i) = *(sbuf + ni);
         *(sbuf + ni) = sval;
         }
      }

   else
   if (nbypx == 4)				/* Floating point image. */
      {
      for (i = 0; i < ihalf; i++)
         {
         ni = imax - i; 
         fval = *(fbuf + i*nval);
         *(fbuf + i) = *(fbuf + ni);
         *(fbuf + ni) = fval;
         }
      }

   return (0);
}
