/*
 * solar.c - solar ephemeris calculations.  Produces sun's
 * ra, dec, az, el, p, b0, sradius calculations
 *
 * SEE ALSO: astrotime.c for time calculations, jday, and last for
 * unix commands, astro.h for miscellaneous definitions.
 */
#include "astro.h"

main(argc,argv)
int argc;
char **argv;
{
	long gmt;
	long jday;
	ANGLE Az, El, Sradius, P, B0, L0, Last, Ra, Dec, Ha, Airmass;
	long hh,mm,ss;

	argc--;argv++;
	if(argc < 1)
	{
		printf("usage: solar jday hh mm ss - print solar ephemeris for day/time\n");
		printf("       jday = julian day; use jday command\n");
		printf("       hh mm ss = greenwich time\n");
		exit (1);
	}

	sscanf(*argv++,"%ld",&jday);argc--;
	gmt = 0;
	sscanf(*argv++,"%ld",&hh);
	sscanf(*argv++,"%ld",&mm);
	sscanf(*argv++,"%ld",&ss);

	gmt = hmstosec(hh,mm,ss);

	Last = last(jday,gmt);			/* radians -> degrees */
	Last = fr2d(Last) * 24./ 360.;	/* degrees -> hours */
	Last = fmod(fmod(Last,24.)+24.,24.);	/* positive angle */

	azel((long)jday,(long)gmt,&Az,&El,&Sradius,&P,&B0,&L0);
	solarEphm((long)jday,(long)gmt,&Ra,&Dec,&Sradius,&P,&B0,&L0);
	printf("gmt %lg hrs, last %lg hrs, Az %lg deg., El %lg deg.\n",
				(double)gmt/3600.,Last,fr2d(Az),fr2d(El));
	printf("sradius %lg, P %lg, B0 %lg, L0 %lg\n",
		fr2d(Sradius)*3600.,fr2d(P),fr2d(B0),fr2d(L0));

	Ra = fr2d(Ra) * 24./ 360.;	/* degrees -> hours */
	Ra = fmod(fmod(Ra,24.)+24.,24.);	/* positive angle */
	Dec = fr2d(Dec);
	Dec = fmod(Dec,360.);	/* positive angle */
	Ha = (Last-Ra)*360./24.;
	Airmass = 1./cos(1.5708-El);
	printf("Ra %lg hours., Dec %lg deg. H.A. %lg  Airmass %lg\n",
		Ra,Dec,Ha,Airmass );
 }
