/* engfmt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-03 Engineering Data Formatter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 Sep 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp201lib.h"

engfmt (pmode)

int pmode ;				/* Print mode. */

{
   extern int engmet_prev ;
   extern int ir ;			/* ir: current block (record) index. */
   extern int ib ;			/* ib: current byte index. */
   extern int blkcnt ;				/* # data blocks in tape/file.*/
   extern unsigned char  hkbuf [46] ;		/* Housekeeping buffer. */
   extern unsigned short imgbuf [256][320] ;	/* Image 2D buffer. */
   extern unsigned char  blkbuf [8200] ;	/* Data Block buffer. */
   extern unsigned char *blkhdr ;		/* Block header data. */
   extern unsigned char *dbuf ;			/* Data. */
   extern int dblknum  ;			/* Data block number. */
   extern int derrcnt ;				/* Data block error count. */
   extern int npxlin ;			/* # pixels per line. */
   extern int nbyeng;			/* # bytes per eng frame = 128 */
   extern int nbypix ;			/* # bytes per pixel. */
   extern int npximg ;			/* # pixels per image  = 256 * 320. */
   extern FILE *FD_RAW, *FD_IMG, *FD_DLOG, *FD_ILOG, *FD_ELOG, *FD_MLOG ;

   static unsigned char ebuf [128] ;	/* Engineering telemetry buffer. */

   int i ;
   int is ;
   int engmet_new = 0 ;
   int nbyend = 0 ;		/* # eng bytes remaining at end of block. */
   int nbybeg = 0 ;		/* # eng bytes remaining at start of block.*/

   /*** If Engineering data extends beyond end of block, read in new block. ***/

   if ((ib + nbyeng) <= 8192)	/* Engr frame contained within current block. */
      {
      nbyend = ib + nbyeng ;
      for (i = ib; i < nbyend; i++)  ebuf [i-ib] = dbuf [i] ;
      is = ib + nbyeng ;
      }
   else				/* Engr frame extends beyond end of block. */
      {
      nbyend = 8192 - ib ;
      nbybeg = nbyeng - nbyend ;
/*
 *      fprintf (stderr, "imgfmt> ir: %4d  ib: %4d  nbyend: %4d  nbybeg: %4d\n",
 *			ir, ib, nbyend, nbybeg) ;
 */
      fprintf (FD_DLOG, "engfmt> ir: %4d  ib: %4d  nbyend: %4d  nbybeg: %4d\n",
			ir, ib, nbyend, nbybeg) ;
      for (i = ib; i < 8192; i++)   ebuf [i-ib] = dbuf [i] ;

      if (ir + 1 < blkcnt)
	 {
	 newblk () ;   
         for (i = 0 ; i < nbybeg; i++) ebuf [nbyend+i] = dbuf [i] ;
         is = nbybeg - 1 ;
	 }
      else
	 {
	 is = -1 ;
	 }
      }

   fprintf (FD_DLOG, "engfmt>sync: %3o %3o %3o %3o %3o %3o\n",
      ebuf [0], ebuf [1], ebuf [2], ebuf [3], ebuf [4], ebuf [5]) ;

   /*** Decode MET. ***/

   engmet_new      = int4 (&ebuf [6]) ;

   /*** Decode Engineering Housekeeping. ***/

   spetlm (ebuf, pmode) ;
   fprintf (FD_DLOG, "engfmt> met:%10d  orbit:%4d   image:%4d\n",
		     SPEH->met, SPEH->orbit_num, SPEH->image_num) ;

   /*** Write Engineering data to disk if MET has changed. ***/

   if (engmet_new != engmet_prev) /* New Eng data, since MET != previous MET. */
      {

      /*** Write image to disk in FITS format. ***/

      if (engmet_prev != 0)
         neweng (ebuf, pmode) ;

      bzero  (ebuf, nbyeng) ;			  /* Clear image buffer. */
      engmet_prev = engmet_new ;
      }

   if (is < 0)
      {
      neweng (ebuf, pmode) ;
      exit (0) ;
      }
   else
      {
      ib = is ;
      return (0) ;
      }
}
