/* imgfmt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-03 Image Data Formatter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 Sep 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp201lib.h"

imgfmt (pmode)

int pmode ;				/* Print mode. */
{
   extern int imgmet_prev ;
   extern int ir ;			/* ir: current block (record) index. */
   extern int ib ;			/* ib: current byte index. */
   extern int blkcnt ;				/* # data blocks in tape/file.*/
   extern unsigned char  hkbuf [46] ;		/* Housekeeping buffer. */
   extern unsigned short imgbuf [256][320] ;	/* Image 2D buffer. */
   extern unsigned char  blkbuf [8200] ;	/* Data Block buffer. */
   extern unsigned char *blkhdr ;		/* Block header data. */
   extern unsigned char *dbuf ;			/* Data. */
   extern int dblknum  ;			/* Data block number. */
   extern int derrcnt ;				/* Data block error count. */
   extern int npxlin ;			/* # pixels per line. */
   extern int nbyimg ;			/* # bytes per image = 256 * 320 * 2. */
   extern int nbypix ;			/* # bytes per pixel. */
   extern int npximg ;			/* # pixels per image  = 256 * 320. */
   extern FILE *FD_RAW, *FD_IMG, *FD_DLOG, *FD_ILOG, *FD_ELOG, *FD_MLOG ;

   static unsigned char ibuf [686] ;		/* Image line buffer. */
   static unsigned char *hdbuf ;		/* Image Housekeeping data. */
   static unsigned char *pxbuf ;		/* Image Pixel data. */
   unsigned char *cbuf ;

   int hduflag = 1 ;
   int i ;
   int is ;
   int met_new = 0 ;
   int new_ccd_line = 0 ;
   int nimend = 0 ;		/* # img bytes remaining at end of block. */
   int nimbeg = 0 ;		/* # img bytes remaining at start of block.*/

   /*** If Image data extends beyond end of block, read in new block. ***/

   if ((ib + 686) <= 8192)	/* Image line contained within current block. */
      {
      nimend = ib + 686 ;
      for (i = ib; i < nimend; i++)  ibuf [i-ib] = dbuf [i] ;
      hdbuf = ibuf ;
      pxbuf = ibuf + 46 ;
      is = ib + 685 ;
      }
   else				/* Image data extends beyond end of block. */
      {
      nimend = 8192 - ib ;
      nimbeg = 686 - nimend ;
/*
 *      fprintf (stderr, "imgfmt> ir: %4d  ib: %4d  nimend: %4d  nimbeg: %4d\n",
 *			ir, ib, nimend, nimbeg) ;
 */
      fprintf (FD_DLOG, "imgfmt> ir: %4d  ib: %4d  nimend: %4d  nimbeg: %4d\n",
			ir, ib, nimend, nimbeg) ;
      for (i = ib; i < 8192; i++)   ibuf [i-ib] = dbuf [i] ;

      if (ir + 1 < blkcnt)
	 {
	 newblk () ;   
         for (i = 0 ; i < nimbeg; i++) ibuf [nimend+i] = dbuf [i] ;
         hdbuf = ibuf ;
         pxbuf = ibuf + 46 ;
         is = nimbeg - 1 ;
	 }
      else
	 {
	 hdbuf = ibuf ;
	 pxbuf = ibuf + 46 ;
	 is = -1 ;
	 }
      }

   /*** Decode MET & CCD line number. ***/

   met_new      = int4 (&hdbuf [6]) ;
   new_ccd_line = int2 (&hdbuf [44]) ;

   /*** Write image to disk if MET has changed. ***/

   if (met_new != imgmet_prev)	/* New image, since MET != previous MET. */
      {

      /*** Write image to disk in FITS format. ***/

      if (imgmet_prev != 0)
         newimg (pmode) ;

      bzero  (imgbuf, nbyimg) ;			  /* Clear image buffer. */
      imgmet_prev = met_new ;

      for (i = 0; i < 46; i++)  hkbuf [i] = hdbuf [i] ;
      }

   if (new_ccd_line > 255)
      {
      fprintf (FD_DLOG, "* * * Invalid CCD line number (%4d)\n",
			new_ccd_line) ;
      return (-1) ;
      }

   /*** Decode Image Housekeeping. ***/

   sphskp (hdbuf, pmode) ;
   fprintf (FD_DLOG, "imgfmt> met:%10d  orbit:%4d   image:%4d  line:%4d\n",
		     SPIH->met, SPIH->orbit_num, SPIH->image_num,
		     SPIH->cur_ccd_line) ;

   /*** Fill image 2D buffer. ***/

   int2arr (pxbuf, &imgbuf [SPII->ydim - SPIH->cur_ccd_line - 1][0], npxlin) ;

   if (is < 0)
      {
      newimg (pmode) ;
      exit (0) ;
      }
   else
      {
      ib = is ;
      return (0) ;
      }
}
