/* newimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write Spartan image to disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   20 September 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp201lib.h"

newimg (pmode)

int pmode ;				/* Print mode. */

{
   extern int rel_pitch_prev ;		/* Previous relative pitch angle. */
   extern int ir ;			/* ir: current block (record) index. */
   extern int ib ;			/* ib: current byte index. */
   extern int blkcnt ;			/* # data blocks in raw tape or file. */
   extern unsigned char  fitbuf [36][80] ;	/* FITS HDU buffer. */
   extern unsigned char  hkbuf [46] ;		/* Housekeeping buffer. */
   extern unsigned short imgbuf [256][320] ;	/* Image 2D buffer. */
   extern unsigned char  blkbuf [8200] ;	/* Data Block buffer. */
   extern unsigned char *blkhdr ;		/* Block header data. */
   extern unsigned char *dbuf ;			/* Data. */
   extern int dblknum  ;		/* Data block number. */
   extern int derrcnt ;			/* Data block error count. */
   extern int npxlin ;			/* # pixels per line. */
   extern int nbyimg ;			/* # bytes per image = 256 * 320 * 2. */
   extern int nbypix ;			/* # bytes per pixel. */
   extern int npximg ;			/* # pixels per image  = 256 * 320. */
   extern FILE *FD_RAW, *FD_IMG, *FD_DLOG, *FD_ILOG, *FD_ELOG, *FD_MLOG ;

   static unsigned char padbuf [320] ;
   static char namfts [80] = "sp201.fts";
   static char namext [80] = "SPARTAN_CMAP";
   int hduflag = 1 ;
   int kyear, kdoy, khour, kmin ;
   float ksec ;

   sprintf (namfts, "sp201.%04d%02d%02d.%02d%02d%02d.fts",
	    SPII->year, SPII->month,  SPII->day,
	    SPII->hour, SPII->minute, SPII->second) ;

   fprintf (FD_DLOG, "---> FITS image: %s  met:%8d  orbit:%3d  image:%3d\n",
		     namfts, SPIH->met, SPIH->orbit_num, SPIH->image_num) ;
   fprintf (stdout, "%s met:%8d orb/img:%2d-%2d rel_pitch:%3d roll:%7.2f\n",
		     namfts, SPIH->met, SPIH->orbit_num, SPIH->image_num,
		     SPIH->rel_pitch_ang, SPII->nangle) ;

   metconv (SPIH->met, &kyear, &kdoy, &khour, &kmin, &ksec) ;

   fprintf (FD_ILOG, "%25s %02d-%02d:%02d:%02.2f %2d - %2d %5.2f %2d  %3.3s  %3.3s  %2.2s   %-2.2s %7.2f %7.2f %7.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f\n",
            namfts, kdoy, khour, kmin, ksec,
	    SPII->orbit_num, SPII->image_num, SPII->expdur,
	    SPII->nimsum, &SPII->optpos[0], &SPII->optpos[1], &SPII->optpos[2],
	    SPII->optpos[3], SPII->ccd_temp_beg, SPII->ccd_temp_end,
	    SPII->nangle,
	    SPII->pes_yaw_ave, SPII->pes_yaw_min, SPII->pes_yaw_max,
	    SPII->pes_pitch_ave, SPII->pes_pitch_min, SPII->pes_pitch_max) ;
   fflush (FD_ILOG) ;

   /*** Write image to disk in FITS format. ***/

   sptofits (hkbuf, imgbuf, namfts, namext, hduflag, pmode) ;

   return (0) ;
}
