/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * NAME		csi.c
 *
 * PURPOSE	Check header information on a Spartan 201-3 raw data file.
 * 
 * SYNTAX	csi tape1.raw
 *
 * HISTORY	Created 16 September 1996 by Andrew L. Staner, HAO/NCAR
 *
 * + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
#include <stdio.h>

/*** Define tape header structure. ***/

struct tape_header
{
unsigned long blkcnt ;
unsigned short blksize ;
unsigned short tapenum ;
unsigned short uerrcnt ;
} ;

struct tape_header hdr ;

/*** Define UART structure array. ***/

struct uart
{
unsigned long  blknum ;
unsigned short bytnum ;
unsigned char err ;
unsigned char z ;
};

struct uart uarts [512] ;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

main (argc, argv)

int argc ;
char *argv [] ;
{


extern unsigned short int2rev () ;
extern unsigned int   int4rev () ;
/*** Define byte arrays. ***/

static char ifile [80] ;
static char logfile [80] ;
static unsigned char taphdr [10] ;
static unsigned char blkhdr [8] ;
static unsigned char ubuf [512][8] ;
static unsigned char dbuf [8192] ;
unsigned char *data ;
int i, ii ;
int blkcnt  ;
int blksize ;
int tapenum ;
int uerrcnt ;
int ublknum  ;
int bytnum ;
int uerr ;
int z ;
int dblknum  ;
int derrcnt ;
int engprev = 0 ;
int imgprev = 0 ;
int msgprev = 0 ;
int engsize = 0 ;
int imgsize = 0 ;
int msgsize = 0 ;
FILE *FDI, *FDO, *FDLOG ;

   if (argc < 2)
      {
      fprintf (stderr, "Usage: %s input_file_name\n", argv [0]) ;
      exit (0) ;
      }
   else
      {
      sprintf (ifile, "%s", argv [1]) ;
      }

   sprintf (logfile, "%s.chk", ifile) ;

   if (( FDLOG = fopen (logfile, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", logfile) ;
      exit (-1) ;
      }

   fprintf (FDLOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fprintf (FDLOG, "Log File for Spartan Data File: %s\n", ifile) ;

   /*** Open input file & read header information. ***/

   if (( FDI = fopen (ifile, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", ifile) ;
      exit (-2) ;
      }

   fread ((char *) taphdr, 10, 1, FDI) ;

   /*** Convert header information from bytes to integers. ***/

   blkcnt  = int4rev ( &taphdr [0] ) ;		/* Number of blocks. */
   blksize = int2rev ( &taphdr [4] ) ;		/* Block size. */
   tapenum = int2rev ( &taphdr [6]) ;		/* Tape number. */
   uerrcnt = int2rev ( &taphdr [8]) ;		/* UART error count. */

   fprintf (FDLOG, "TAPE HEADER:\n") ;
   fprintf (FDLOG, "   blkcnt : %d\n", blkcnt ) ;
   fprintf (FDLOG, "   blksize: %d\n", blksize) ;
   fprintf (FDLOG, "   tapenum: %d\n", tapenum) ;
   fprintf (FDLOG, "   uerrcnt: %d\n", uerrcnt) ;

   /*** Read UART structure. ***/

   fread (ubuf, 1, 8*512, FDI) ;

   fprintf (FDLOG, "UART Structure:\n") ;

   for (i = 0; i < 512; i++)
      {
      ublknum = (int4rev (&ubuf [i][0])) ;
      bytnum = (int2rev (&ubuf [i][4])) ;
      uerr   = ubuf [i][6] ;
      z      = ubuf [i][7] ;
      uarts [i].blknum = ublknum ;
      uarts [i].bytnum = bytnum ;
      uarts [i].err    = uerr ;
      if (uerr != 0)
         fprintf (FDLOG, "   i: %3d  ublknum: %4d  bytnum: %4d  uerr: %4d  z: %4d\n",
                             i, ublknum, bytnum, uerr, z) ;
      }

   fseek (FDI, 8200L, 0) ;		/* Position file after tape header. */

   /*** Read data blocks. ***/

   fprintf (FDLOG, "Block Information:\n") ;

   for (i = 0; i < blkcnt ; i++)
      {
      fread (blkhdr, 1, 8, FDI) ;
      dblknum = int4rev (&blkhdr [0]) ;
      derrcnt = int4rev (&blkhdr [4]) ;
      if (derrcnt != 0)
         fprintf (FDLOG, "   dblknum : %4d  derrcnt: %4d\n", dblknum, derrcnt) ;

      fread (dbuf, 1, 8192, FDI) ;

      engprev =  0 ;
      imgprev =  0 ;
      msgprev =  0 ;
      for (ii = 0; ii < 8186; ii++)
	 {
/*
	 if (dbuf [ii] == 063)
	    fprintf (FDLOG, "ii: %4d dbuf [ii:ii+5]: %4d  %4d  %4d  %4d  %4d  %4d\n",
			     ii, dbuf [ii], dbuf [ii+1], dbuf [ii+2], dbuf [ii+3], dbuf [ii+4], dbuf [ii+5]) ;
 */

         if (dbuf [ii]   ==  063 && dbuf [ii+1] ==  052 && dbuf [ii+2] == 0146
	 &&  dbuf [ii+3] == 0125 && dbuf [ii+4] ==    0 && dbuf [ii+5] ==    0)
	    {
	    engsize = ii - engprev ;
	    fprintf (FDLOG, "Engineering ... byte# %4d  size: %6d\n", ii, engsize) ;
	    engprev = ii ;
	    }
         if (dbuf [ii]   ==  063 && dbuf [ii+1] ==  052 && dbuf [ii+2] == 0146
	 &&  dbuf [ii+3] == 0125 && dbuf [ii+4] ==    0 && dbuf [ii+5] ==    4)
	    {
	    imgsize = ii - imgprev ;
	    fprintf (FDLOG, "Image ... byte# %4d  size: %6d\n", ii, imgsize) ;
	    imgprev = ii ;
	    }
         if (dbuf [ii]   ==  063 && dbuf [ii+1] ==  052 && dbuf [ii+2] == 0146
	 &&  dbuf [ii+3] == 0125 && dbuf [ii+4] ==    0 && dbuf [ii+5] ==    8)
	    {
	    msgsize = ii - msgprev ;
	    fprintf (FDLOG, "Message ... byte# %4d  size: %6d\n", ii, msgsize) ;
	    msgprev = ii ;
	    }
	 }
      }

   fprintf (FDLOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fclose (FDI) ;
   fclose (FDLOG) ;

}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2rev (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   sval = bbuf [1] * 256 + bbuf [0] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4rev (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   ival = bbuf [3] * 16777216 + bbuf [2] * 65536 + bbuf [1] * 256 + bbuf [0] ;

   return (ival) ;
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short*
int2arr (bbuf, n_elem)		/*  Convert byte array to 16-bit integers. */

char bbuf [] ;
int n_elem ;
{
   unsigned short *sbuf ;
   int i ;

   sbuf = (unsigned short *) malloc (n_elem/2 * sizeof (int)) ;

   for (i = 0; i < n_elem/2; i++)
      {
      *(sbuf + i) = *(bbuf + i*2 + 0) * 256  + *(bbuf + i*2 + 1) ;
      }

   return (sbuf) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int *
int4arr (bbuf, n_elem)		/*  Convert byte array to 32-bit integers. */

char bbuf [] ;
int n_elem ;
{
   unsigned int *ibuf ;
   int i ;

   ibuf = (unsigned int *) malloc (n_elem/4 * sizeof (int)) ;

   for (i = 0; i < n_elem/4; i++)
   {
      *(ibuf + i) = *(bbuf + i*4) * 16777216 + *(bbuf + i*4 + 1) * 65536 +
                  + *(bbuf + i*4 + 2) * 256  + *(bbuf + i*4 + 3) ;
   }

   return (ibuf) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
rev4byt (inbuf, outbuf)

unsigned char inbuf  [] ;
unsigned char outbuf [] ;

{
   outbuf [0] = inbuf [3] ;
   outbuf [1] = inbuf [2] ;
   outbuf [2] = inbuf [1] ;
   outbuf [3] = inbuf [0] ;

   return (0) ;
}
