/* imgfmt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-03 Image Data Formatter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 Sep 1996
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>

imgfmt ()

{
   extern int met_prev ;
   extern int ir ;			/* ir: current block (record) index. */
   extern int ib ;			/* ib: current byte index. */
   extern int blkcnt ;				/* # data blocks in tape/file.*/
   extern unsigned short imgbuf [256][320] ;	/* Image 2D buffer. */
   extern unsigned char  blkbuf [8200] ;	/* Data Block buffer. */
   extern unsigned char *blkhdr ;		/* Block header data. */
   extern unsigned char *dbuf ;			/* Data. */
   extern int dblknum  ;			/* Data block number. */
   extern int derrcnt ;				/* Data block error count. */
   extern int npxlin ;			/* # pixels per line. */
   extern int nbyimg ;			/* # bytes per image = 256 * 320 * 2. */
   extern int nbypix ;			/* # bytes per pixel. */
   extern int npximg ;			/* # pixels per image  = 256 * 320. */
   extern FILE *FD_RAW, *FD_IMG, *FD_LOG ;

   static unsigned char ibuf [686] ;		/* Image line buffer. */
   static unsigned char *hkbuf ;		/* Image Housekeeping data. */
   static unsigned char *pxbuf ;		/* Image Pixel data. */
   unsigned char *cbuf ;

   /*** Data Block Housekeeping: ***/
   /* sync pattern: bytes  0 -  5 */

   int met ;			/* bytes  6 -  9     */
   int rel_pitch_ang ;		/* bytes 10 - 11     */
   int mech_pos1 ;		/* byte  12          */
   int fold_mir_pos ;		/* byte  12 bit  0   */
   int hdump_mir_pos ;		/* byte  12 bit  1   */
   int pw_pos ;			/* byte  12 bits 2-3 */
   int half_wave_pos ;		/* byte  12 bits 4-5 */
   int mech_pos2 ;		/* byte  13          */
   int num_img_sum ;		/* byte  13 bits 0-3 */
   int stim_lamp_stat ;		/* byte  13 bit  5   */
   int ccd_expdur ;		/* bytes 14 - 15     */
   int orbit_num ;		/* byte  16          */
   int image_num ;		/* byte  17          */
   int ccd_temp_beg ;		/* bytes 18 - 19     */
   int ccd_temp_end ;		/* bytes 20 - 21     */
   int pes_yaw_mean ;		/* bytes 22 - 23     */
   int pes_yaw_min ;		/* bytes 24 - 25     */
   int pes_yaw_max ;		/* bytes 26 - 27     */
   int pes_pitch_mean ;		/* bytes 28 - 29     */
   int pes_pitch_min ;		/* bytes 30 - 31     */
   int pes_pitch_max ;		/* bytes 32 - 33     */
   int spare1 ;			/* bytes 34 - 35     */
   int spare2 ;			/* byte  36          */
   int spare3 ;			/* byte  37          */
   int spare4 ;			/* byte  38          */
   int spare5 ;			/* byte  39          */
   int spare6 ;			/* byte  40          */
   int spare7 ;			/* byte  41          */
   int spare8 ;			/* byte  42          */
   int spare9 ;			/* byte  43          */
   int cur_ccd_line ;		/* bytes 44 - 45     */

   /*** Image data: bytes 46 - 685. ***/

   int i ;
   int is ;
   int nimend = 0 ;		/* # img bytes remaining at end of block. */
   int nimbeg = 0 ;		/* # img bytes remaining at start of block.*/

   /*** If Image data extends beyond end of block, read in new block. ***/

   if ((ib + 686) <= 8192)	/* Image line contained within current block. */
      {
      nimend = ib + 686 ;
      for (i = ib; i < nimend; i++)  ibuf [i-ib] = dbuf [i] ;
      hkbuf = ibuf ;
      pxbuf = ibuf + 46 ;
      is = ib + 685 ;
      }
   else				/* Image data extends beyond end of block. */
      {
      nimend = 8192 - ib ;
      nimbeg = 686 - nimend ;
/*
 *      fprintf (stderr, "imgfmt> ir: %4d  ib: %4d  nimend: %4d  nimbeg: %4d\n",
 *			ir, ib, nimend, nimbeg) ;
 */
      fprintf (FD_LOG, "imgfmt> ir: %4d  ib: %4d  nimend: %4d  nimbeg: %4d\n",
			ir, ib, nimend, nimbeg) ;
      for (i = ib; i < 8192; i++)   ibuf [i-ib] = dbuf [i] ;

      if (ir + 1 < blkcnt)
	 {
	 newblk () ;   
         for (i = 0 ; i < nimbeg; i++) ibuf [nimend+i] = dbuf [i] ;
         hkbuf = ibuf ;
         pxbuf = ibuf + 46 ;
         is = nimbeg - 1 ;
	 }
      else
	 {
	 hkbuf = ibuf ;
	 pxbuf = ibuf + 46 ;
	 is = -1 ;
	 }
      }

   /*** Decode Image Housekeeping. ***/

   met = int4 (&hkbuf [6]) ;
   rel_pitch_ang = int2 (&hkbuf [10]) ;
   mech_pos1 = hkbuf [12] ;
   fold_mir_pos = hkbuf [12] & 001 ;
   hdump_mir_pos = (hkbuf [12] & 002) >> 1 ;
   pw_pos = (hkbuf [12] & 014) >> 2 ;
   half_wave_pos = (hkbuf [12] & 060) >> 4 ;
   num_img_sum = (hkbuf [13] & 017) ;
   stim_lamp_stat = (hkbuf [13] & 040) >> 5 ;
   ccd_expdur = int2 (&hkbuf [14]) ;
   orbit_num = hkbuf [16] ;
   image_num - hkbuf [17] ;
   ccd_temp_beg = int2 (&hkbuf [18]) ;
   ccd_temp_end = int2 (&hkbuf [20]) ;
   pes_yaw_mean = int2 (&hkbuf [22]) ;
   pes_yaw_min  = int2 (&hkbuf [24]) ;
   pes_yaw_max  = int2 (&hkbuf [26]) ;
   pes_pitch_mean = int2 (&hkbuf [28]) ;
   pes_pitch_min  = int2 (&hkbuf [30]) ;
   pes_pitch_max  = int2 (&hkbuf [32]) ;
   cur_ccd_line = int2 (&hkbuf [44]) ;

   /*** Write image to disk if MET has changed. ***/

   if (met != met_prev)		/* New image, since MET != previous MET. */
      {

      /*** Write image to disk & close file. ***/

      if (met_prev != 0)
	 newimg () ;

      bzero  (imgbuf, nbyimg) ;			  /* Clear image buffer. */
      met_prev = met ;

      /*** Print Housekeeping. ***/

      fprintf (FD_LOG, "> fmet          : %12.3f (sec)\n",
					        (float) met * 0.008) ;
      fprintf (FD_LOG, "> met           : %8d     (8 msec)\n", met) ;
      fprintf (FD_LOG, "> rel_pitch_ang : %8d\n", rel_pitch_ang ) ;
      fprintf (FD_LOG, "> fold_mir_pos  : %8d\n", fold_mir_pos  ) ;
      fprintf (FD_LOG, "> hdump_mir_pos : %8d\n", hdump_mir_pos ) ;
      fprintf (FD_LOG, "> pw_pos        : %8d\n", pw_pos        ) ;
      fprintf (FD_LOG, "> half_wave_pos : %8d\n", half_wave_pos ) ;
      fprintf (FD_LOG, "> num_img_sum   : %8d\n", num_img_sum   ) ;
      fprintf (FD_LOG, "> stim_lamp_stat: %8d\n", stim_lamp_stat) ;
      fprintf (FD_LOG, "> ccd_expdur    : %8d\n", ccd_expdur    ) ;
      fprintf (FD_LOG, "> orbit_num     : %8d\n", orbit_num     ) ;
      fprintf (FD_LOG, "> image_num     : %8d\n", image_num     ) ;
      fprintf (FD_LOG, "> ccd_temp_beg  : %8d\n", ccd_temp_beg  ) ;
      fprintf (FD_LOG, "> ccd_temp_end  : %8d\n", ccd_temp_end  ) ;
      fprintf (FD_LOG, "> pes_yaw_mean  : %8d\n", pes_yaw_mean  ) ;
      fprintf (FD_LOG, "> pes_yaw_min   : %8d\n", pes_yaw_min   ) ;
      fprintf (FD_LOG, "> pes_yaw_max   : %8d\n", pes_yaw_max   ) ;
      fprintf (FD_LOG, "> pes_pitch_mean: %8d\n", pes_pitch_mean) ;
      fprintf (FD_LOG, "> pes_pitch_min : %8d\n", pes_pitch_min ) ;
      fprintf (FD_LOG, "> pes_pitch_max : %8d\n", pes_pitch_max ) ;
      fprintf (FD_LOG, "> cur_ccd_line  : %8d\n", cur_ccd_line) ;
      }

   if (cur_ccd_line > 255)
      {
      fprintf (FD_LOG, "* * * Invalid CCD line number (%4d)\n", cur_ccd_line) ;
      return (-1) ;
      }

   /*** Fill image 2D buffer. ***/

   int2arr (pxbuf, &imgbuf [cur_ccd_line][0], npxlin) ;

   if (is < 0)
      {
      newimg () ;
      exit (0) ;
      }
   else
      {
      ib = is ;
      return (0) ;
      }
}
