/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * NAME		si.c
 *
 * PURPOSE	Extract Spartan 201-3 images from a raw data file.
 * 
 * SYNTAX	si tape1.raw
 *
 * HISTORY	Created 16 September 1996 by Andrew L. Staner, HAO/NCAR
 *
 * + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
#include <stdio.h>

FILE *FD_RAW, *FD_IMG, *FD_LOG ;

int met_prev = 0 ;
int ir = 0;			/* ir: current block (record) index. */
int ib = 0;			/* ib: current byte index. */
int blkcnt  ;			/* # data blocks in raw tape raw file. */
int dblknum  ;			/* Data block number. */
int derrcnt ;			/* Data block error count. */
int npxlin = 320 ;		/* # pixels per line. */
int nbyimg = 163840 ;		/* # bytes per image = 256 * 320 * 2. */
int nbypix = 2 ;		/* # bytes per pixel. */
int npximg =  81920 ;		/* # pixels per image  = 256 * 320. */
unsigned short imgbuf [256][320] ;
unsigned char  blkbuf [8200] ;
unsigned char  *blkhdr ;
unsigned char  *dbuf ;

/*** Define tape header structure. ***/

struct tape_header
{
unsigned long blkcnt ;
unsigned short blksize ;
unsigned short tapenum ;
unsigned short uerrcnt ;
} ;

struct tape_header hdr ;

/*** Define UART structure array. ***/

struct uart
{
unsigned long  blknum ;
unsigned short bytnum ;
unsigned char err ;
unsigned char z ;
};

struct uart uarts [512] ;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

main (argc, argv)

int argc ;
char *argv [] ;
{


extern unsigned short int2rev () ;
extern unsigned int   int4rev () ;

/*** Define byte arrays. ***/

static char ifile [80] ;
static char logfile [80] ;
static unsigned char taphdr [10] ;
static unsigned char ubuf [512][8] ;

int i, ii ;
int blksize ;
int tapenum ;
int uerrcnt ;
int ublknum  ;
int bytnum ;
int uerr ;
int z ;

   if (argc < 2)
      {
      fprintf (stderr, "Usage: %s input_file_name\n", argv [0]) ;
      exit (0) ;
      }
   else
      {
      sprintf (ifile, "%s", argv [1]) ;
      }

   sprintf (logfile, "%s.log", ifile) ;

   if (( FD_LOG = fopen (logfile, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", logfile) ;
      exit (-1) ;
      }

   fprintf (FD_LOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fprintf (FD_LOG, "Log File for Spartan Data File: %s\n", ifile) ;

   /*** Open input file & read header information. ***/

   if (( FD_RAW = fopen (ifile, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", ifile) ;
      exit (-2) ;
      }

   fread ((char *) taphdr, 10, 1, FD_RAW) ;

   /*** Convert header information from bytes to integers. ***/

   blkcnt  = int4rev ( &taphdr [0] ) ;		/* Number of blocks. */
   blksize = int2rev ( &taphdr [4] ) ;		/* Block size. */
   tapenum = int2rev ( &taphdr [6]) ;		/* Tape number. */
   uerrcnt = int2rev ( &taphdr [8]) ;		/* UART error count. */

   fprintf (FD_LOG, "TAPE HEADER:\n") ;
   fprintf (FD_LOG, "  blkcnt : %d\n", blkcnt ) ;
   fprintf (FD_LOG, "  blksize: %d\n", blksize) ;
   fprintf (FD_LOG, "  tapenum: %d\n", tapenum) ;
   fprintf (FD_LOG, "  uerrcnt: %d\n", uerrcnt) ;

   /*** Read UART structure. ***/

   fread (ubuf, 1, 8*512, FD_RAW) ;

   fprintf (FD_LOG, "UART Structure:\n") ;

   for (i = 0; i < 512; i++)
      {
      ublknum = (int4rev (&ubuf [i][0])) ;
      bytnum = (int2rev (&ubuf [i][4])) ;
      uerr   = ubuf [i][6] ;
      z      = ubuf [i][7] ;
      uarts [i].blknum = ublknum ;
      uarts [i].bytnum = bytnum ;
      uarts [i].err    = uerr ;
      if (uerr != 0)
         fprintf (FD_LOG, "  i: %3d  ublknum: %4d  bytnum: %4d  uerr: %4d  z: %4d\n",
                             i, ublknum, bytnum, uerr, z) ;
      }

   fseek (FD_RAW, 8200L, 0) ;		/* Position file after tape header. */

   /*** Read data blocks. ***-***-***-***-***-***-***-***-***-***-***-***-***-*/

   fprintf (FD_LOG, "Block Information:\n") ;

/*   for (ir = 0; ir < 200 ; ir++) */

   for (ir = 0; ir < blkcnt ; ir++)
      {
      fread (blkbuf, 1, 8200, FD_RAW) ;
      blkhdr = blkbuf ;
      dbuf   = blkbuf + 8 ;

      dblknum = int4rev (&blkhdr [0]) ;
      derrcnt = int4rev (&blkhdr [4]) ;
      if (derrcnt != 0)
         fprintf (FD_LOG, "  dblknum : %4d  derrcnt: %4d\n", dblknum, derrcnt) ;

      /* --- Find Sync Pattern. --- --- --- --- --- --- --- --- --- --- --- - */

      for (ib = 0; ib < 8186; ib++)
	 {
	 /*** Engineering Data. ***/

         if (dbuf [ib]   ==  063 && dbuf [ib+1] ==  052 && dbuf [ib+2] == 0146
	 &&  dbuf [ib+3] == 0125 && dbuf [ib+4] ==    0 && dbuf [ib+5] ==    0)
	    {
	    fprintf (FD_LOG, "Engineering ... ir: %4d  ib: %4d\n", ir, ib) ;
	    engfmt (dbuf) ;
	    }

         /*** Image Data. ***/

         if (dbuf [ib]   ==  063 && dbuf [ib+1] ==  052 && dbuf [ib+2] == 0146
	 &&  dbuf [ib+3] == 0125 && dbuf [ib+4] ==    0 && dbuf [ib+5] ==    4)
	    {
	    fprintf (FD_LOG, "Image ... ir: %4d  ib: %4d\n", ir, ib) ;
	    imgfmt (dbuf) ;
	    }

         /*** Message Data. ***/

         if (dbuf [ib]   ==  063 && dbuf [ib+1] ==  052 && dbuf [ib+2] == 0146
	 &&  dbuf [ib+3] == 0125 && dbuf [ib+4] ==    0 && dbuf [ib+5] ==    8)
	    {
	    fprintf (FD_LOG, "Message ... ir: %4d  ib: %4d\n", ir, ib) ;
	    msgfmt (dbuf) ;
	    }
	 }
      }

   fprintf (FD_LOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fclose (FD_RAW) ;
   fclose (FD_LOG) ;

}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2rev (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   sval = bbuf [1] * 256 + bbuf [0] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4rev (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   ival = bbuf [3] * 16777216 + bbuf [2] * 65536 + bbuf [1] * 256 + bbuf [0] ;

   return (ival) ;
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2 (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   sval = bbuf [0] * 256 + bbuf [1] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4 (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   ival = bbuf [0] * 16777216 + bbuf [1] * 65536 + bbuf [2] * 256 + bbuf [3] ;

   return (ival) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int2arr (bbuf, sbuf, n_elem)	/*  Convert byte array to 16-bit integers. */

unsigned char  bbuf [] ;	/* Input  buffer. */
unsigned short sbuf [] ;	/* Output buffer. */
int n_elem ;			/* # pixels in output array. */
{
   int i ;

   for (i = 0; i < n_elem; i++)
      {

/*fprintf (stderr, "int2arr> i: %4d  i*2: %4d  i*2+1: %4d\n", i, i*2, i*2+1) ;*/

      *(sbuf + i) = *(bbuf + i*2 + 0) * 256  + *(bbuf + i*2 + 1) ;
      }

   return (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int4arr (bbuf, ibuf, n_elem)	/*  Convert byte array to 32-bit integers. */

unsigned char bbuf [] ;		/* Input  buffer. */
int ibuf [] ;			/* Output buffer. */
int n_elem ;			/* # pixels in output array. */
{
   int i ;

   for (i = 0; i < n_elem; i++)
   {
      *(ibuf + i) = *(bbuf + i*4) * 16777216 + *(bbuf + i*4 + 1) * 65536 +
                  + *(bbuf + i*4 + 2) * 256  + *(bbuf + i*4 + 3) ;
   }

   return (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
