/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * NAME		si.c
 *
 * PURPOSE	Extract Spartan 201-3 images from a raw data file.
 * 
 * SYNTAX	si tape1.raw
 *
 * HISTORY	Created 16 September 1996 by Andrew L. Staner, HAO/NCAR
 *
 * + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
#include <stdio.h>
#define SPARTAN
#include "sp201lib.h"

FILE *FD_RAW, *FD_IMG, *FD_DLOG, *FD_ILOG, *FD_ELOG, *FD_MLOG, *FD_ETLM ;

int engmet_prev = 0 ;
int imgmet_prev = 0 ;
int msgmet_prev = 0 ;
int ir = 0;			/* ir: current block (record) index. */
int ib = 0;			/* ib: current byte index. */
int blkcnt  ;			/* # data blocks in raw tape raw file. */
int dblknum  ;			/* Data block number. */
int derrcnt ;			/* Data block error count. */
int npxlin = 320 ;		/* # pixels per line. */
int nbyimg = 163840 ;		/* # bytes per image = 256 * 320 * 2. */
int nbyeng = 128 ;		/* # bytes in engineering telemetry frame. */
int nbypix = 2 ;		/* # bytes per pixel. */
int npximg =  81920 ;		/* # pixels per image  = 256 * 320. */
int rel_pitch_prev = -1 ;	/* Previous Relative Pitch Angle. */
int rel_pitch_new  =  0 ;	/* Current  Relative Pitch Angle. */
int pitch_orb_prev = -1 ;	/* Previous Pitch orbit. */
int pitch_img_prev = -1 ;	/* Previous Pitch image. */
unsigned char  hkbuf [46] ;		/* Housekeeping buffer. */
unsigned short imgbuf [256][320] ;	/* Image 2D buffer. */
unsigned char  blkbuf [8200] ;		/* Block buffer. */
unsigned char  *blkhdr ;
unsigned char  *dbuf ;

/*** Define tape header structure. ***/

struct tape_header
{
unsigned long blkcnt ;
unsigned short blksize ;
unsigned short tapenum ;
unsigned short uerrcnt ;
} ;

struct tape_header hdr ;

/*** Define UART structure array. ***/

struct uart
{
unsigned long  blknum ;
unsigned short bytnum ;
unsigned char err ;
unsigned char z ;
};

struct uart uarts [512] ;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

main (argc, argv)

int argc ;
char *argv [] ;
{


extern unsigned short int2rev () ;
extern unsigned int   int4rev () ;

/*** Define arrays. ***/

static char ifile [80] ;
static char dlognam [80] ;
static char elognam [80] ;
static char ilognam [80] ;
static char mlognam [80] ;
static char etlmnam [80] ;
static unsigned char taphdr [10] ;
static unsigned char ubuf [512][8] ;

int i, ii ;
int blksize ;
int tapenum ;
int uerrcnt ;
int ublknum  ;
int bytnum ;
int uerr ;
int z ;
int pmode = 0 ;
int nbyblk = 8192 ;
int nbyend ;

   if (argc < 2)
      {
      fprintf (stderr, "Usage: %s input_file_name\n", argv [0]) ;
      exit (0) ;
      }
   else
      {
      sprintf (ifile, "%s", argv [1]) ;
      }

   sprintf (dlognam, "%s.dlog", ifile) ;
   sprintf (elognam, "%s.elog", ifile) ;
   sprintf (ilognam, "%s.ilog", ifile) ;
   sprintf (mlognam, "%s.mlog", ifile) ;
   sprintf (etlmnam, "%s.etlm", ifile) ;

   if (( FD_DLOG = fopen (dlognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", dlognam) ;
      exit (-1) ;
      }

   if (( FD_ELOG = fopen (elognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", elognam) ;
      exit (-1) ;
      }

   if (( FD_ILOG = fopen (ilognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", ilognam) ;
      exit (-1) ;
      }

   if (( FD_MLOG = fopen (mlognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", mlognam) ;
      exit (-1) ;
      }

   if (( FD_ETLM = fopen (etlmnam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", etlmnam) ;
      exit (-1) ;
      }


   fprintf (FD_DLOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fprintf (FD_DLOG, "Log File for Spartan Data File: %s\n", ifile) ;

            fprintf    (FD_ELOG,    "  ir   ib -- MET --- ddd-hh:mm:ss.sss    +28v     +5v    +12v    TEDv    BATv    c+5v    c+15    c-15     csp    fmII     ptr     yfp     rfp    yoff    roff    ppit      pyaw     fbt     o1t     fwt\n")    ;

   fprintf (FD_ILOG, "%s\n", ifile) ;
   fprintf (FD_ILOG, "------- FILE NAME ------- ---MET----- orb/img  EXP SUM FOLD HEAT POL 1/2   ccd     ccd     Actual ------ Yaw ------  ----- Pitch ----\n");
   fprintf (FD_ILOG, "                                               sec     MIR  MIR  WHL WAVE  T-beg   T-end    Pitch   Ave   Min   Max   Ave   Min   Max\n");

   /*** Open input file & read header information. ***/

   if (( FD_RAW = fopen (ifile, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", ifile) ;
      exit (-2) ;
      }

   fread ((char *) taphdr, 10, 1, FD_RAW) ;

   /*** Convert header information from bytes to integers. ***/

   blkcnt  = int4rev ( &taphdr [0] ) ;		/* Number of blocks. */
   blksize = int2rev ( &taphdr [4] ) ;		/* Block size. */
   tapenum = int2rev ( &taphdr [6]) ;		/* Tape number. */
   uerrcnt = int2rev ( &taphdr [8]) ;		/* UART error count. */

   fprintf (FD_DLOG, "TAPE HEADER:\n") ;
   fprintf (FD_DLOG, "  blkcnt : %d\n", blkcnt ) ;
   fprintf (FD_DLOG, "  blksize: %d\n", blksize) ;
   fprintf (FD_DLOG, "  tapenum: %d\n", tapenum) ;
   fprintf (FD_DLOG, "  uerrcnt: %d\n", uerrcnt) ;

   /*** Read UART structure. ***/

   fread (ubuf, 1, 8*512, FD_RAW) ;

   fprintf (FD_DLOG, "UART Structure:\n") ;

   for (i = 0; i < 512; i++)
      {
      ublknum = (int4rev (&ubuf [i][0])) ;
      bytnum = (int2rev (&ubuf [i][4])) ;
      uerr   = ubuf [i][6] ;
      z      = ubuf [i][7] ;
      uarts [i].blknum = ublknum ;
      uarts [i].bytnum = bytnum ;
      uarts [i].err    = uerr ;
      if (uerr != 0)
         fprintf (FD_DLOG, "  i: %3d  ublknum: %4d  bytnum: %4d  uerr: %4d  z: %4d\n",
                             i, ublknum, bytnum, uerr, z) ;
      }

   fseek (FD_RAW, 8200L, 0) ;		/* Position file after tape header. */

   /*** Read data blocks. ***-***-***-***-***-***-***-***-***-***-***-***-***-*/

   fprintf (FD_DLOG, "Block Information:\n") ;

/*   for (ir = 0; ir < 200 ; ir++) */

   for (ir = 0; ir < blkcnt ; ir++)
      {
      fread (blkbuf, 1, 8200, FD_RAW) ;
      blkhdr = blkbuf ;
      dbuf   = blkbuf + 8 ;

      dblknum = int4rev (&blkhdr [0]) ;
      derrcnt = int4rev (&blkhdr [4]) ;
      if (derrcnt != 0)
         fprintf (FD_DLOG, "  dblknum : %4d  derrcnt: %4d\n", dblknum, derrcnt) ;

      nbyend = nbyblk - 6 ;

      /* --- Find Sync Pattern. --- --- --- --- --- --- --- --- --- --- --- - */

      for (ib = 0; ib < nbyend; ib++)
	 {
	 /*** Engineering Data. ***/

         if (dbuf [ib]   ==  063 && dbuf [ib+1] ==  052 && dbuf [ib+2] == 0146
	 &&  dbuf [ib+3] == 0125 && dbuf [ib+4] ==    0 && dbuf [ib+5] ==    0)
	    {
	    fprintf (FD_DLOG, "Engineering ... ir: %4d  ib: %4d\n", ir, ib) ;
	    engfmt (pmode) ;
	    }

         /*** Image Data. ***/

         if (dbuf [ib]   ==  063 && dbuf [ib+1] ==  052 && dbuf [ib+2] == 0146
	 &&  dbuf [ib+3] == 0125 && dbuf [ib+4] ==    0 && dbuf [ib+5] ==    4)
	    {
	    fprintf (FD_DLOG, "Image ... ir: %4d  ib: %4d\n", ir, ib) ;
	    imgfmt (pmode) ;
	    }

         /*** Message Data. ***/

         if (dbuf [ib]   ==  063 && dbuf [ib+1] ==  052 && dbuf [ib+2] == 0146
	 &&  dbuf [ib+3] == 0125 && dbuf [ib+4] ==    0 && dbuf [ib+5] ==    8)
	    {
	    fprintf (FD_DLOG, "Message ... ir: %4d  ib: %4d\n", ir, ib) ;
	    msgfmt (pmode) ;
	    }
	 }
      }

   newimg (pmode) ;

   fprintf (FD_DLOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fclose (FD_RAW) ;
   fclose (FD_DLOG) ;
   fclose (FD_ELOG) ;
   fclose (FD_ILOG) ;
   fclose (FD_MLOG) ;
   fclose (FD_ETLM) ;
   exit (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2rev (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   sval = bbuf [1] * 256 + bbuf [0] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4rev (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   ival = bbuf [3] * 16777216 + bbuf [2] * 65536 + bbuf [1] * 256 + bbuf [0] ;

   return (ival) ;
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2 (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   sval = bbuf [0] * 256 + bbuf [1] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4 (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   ival = bbuf [0] * 16777216 + bbuf [1] * 65536 + bbuf [2] * 256 + bbuf [3] ;

   return (ival) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int2arr (bbuf, sbuf, n_elem)	/*  Convert byte array to 16-bit integers. */

unsigned char  bbuf [] ;	/* Input  buffer. */
unsigned short sbuf [] ;	/* Output buffer. */
int n_elem ;			/* # pixels in output array. */
{
   int i ;

   for (i = 0; i < n_elem; i++)
      {

/*fprintf (stderr, "int2arr> i: %4d  i*2: %4d  i*2+1: %4d\n", i, i*2, i*2+1) ;*/

      *(sbuf + i) = *(bbuf + i*2 + 0) * 256  + *(bbuf + i*2 + 1) ;
      }

   return (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int4arr (bbuf, ibuf, n_elem)	/*  Convert byte array to 32-bit integers. */

unsigned char bbuf [] ;		/* Input  buffer. */
int ibuf [] ;			/* Output buffer. */
int n_elem ;			/* # pixels in output array. */
{
   int i ;

   for (i = 0; i < n_elem; i++)
   {
      *(ibuf + i) = *(bbuf + i*4) * 16777216 + *(bbuf + i*4 + 1) * 65536 +
                  + *(bbuf + i*4 + 2) * 256  + *(bbuf + i*4 + 3) ;
   }

   return (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
