/* sphskp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Fill HAO/SPARTAN housekeeping structure
 * using information in SPARTAN image header information.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    24 September 1996
 * 1 Oct 96: Add gyro_pitch_set for roll angle specification versus orbit_num.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp201lib.h"

sphskp (hkbuf, pmode)

unsigned char hkbuf [] ;		/* Spartan image housekeeping array. */
int pmode ;				/* Print mode: 0=silent, 1=verbose. */

{
   extern FILE *FD_ELOG ;
   extern int rel_pitch_prev ;
   extern int rel_pitch_new  ;
   extern int pitch_orb_prev ;
   extern int pitch_img_prev ;
   static int irel = 0 ;
   int iyear ;
   int idoy  ;
   int imonth ;
   int iday ;
   int ihour ;
   int iminute ;
   float isecond ;
   int iyear0 =  95 ;
   int idoy0  = 251 ;
   int ihour0 =  15 ;
   int imin0  =  47 ;
   int sct0   =   0 ;
   float sec0  = 35.0 ;

   /*** Data Block Housekeeping: ***/
   /* sync pattern: bytes  0 -  5 */

   int met ;			/* bytes  6 -  9     */
   int rel_pitch_ang ;		/* bytes 10 - 11     */
   int mech_pos1 ;		/* byte  12          */
   int fold_mir_pos ;		/* byte  12 bit  0   */
   int hdump_mir_pos ;		/* byte  12 bit  1   */
   int pw_pos ;			/* byte  12 bits 2-3 */
   int half_wave_pos ;		/* byte  12 bits 4-5 */
   int mech_pos2 ;		/* byte  13          */
   int num_img_sum ;		/* byte  13 bits 0-3 */
   int stim_lamp_stat ;		/* byte  13 bit  5   */
   int ccd_expdur ;		/* bytes 14 - 15     */
   int orbit_num ;		/* byte  16          */
   int image_num ;		/* byte  17          */
   int ccd_temp_beg ;		/* bytes 18 - 19     */
   int ccd_temp_end ;		/* bytes 20 - 21     */
   int pes_yaw_mean ;		/* bytes 22 - 23     */
   int pes_yaw_min ;		/* bytes 24 - 25     */
   int pes_yaw_max ;		/* bytes 26 - 27     */
   int pes_pitch_mean ;		/* bytes 28 - 29     */
   int pes_pitch_min ;		/* bytes 30 - 31     */
   int pes_pitch_max ;		/* bytes 32 - 33     */
   int spare1 ;			/* bytes 34 - 35     */
   int spare2 ;			/* byte  36          */
   int spare3 ;			/* byte  37          */
   int spare4 ;			/* byte  38          */
   int spare5 ;			/* byte  39          */
   int spare6 ;			/* byte  40          */
   int spare7 ;			/* byte  41          */
   int spare8 ;			/* byte  42          */
   int spare9 ;			/* byte  43          */
   int cur_ccd_line ;		/* bytes 44 - 45     */

   FILE *FD ;
   int doy ;
   int i ;
   int ipos ;
   float roll_ang ;
   float pitch_id, pitch_beg, pitch_end ;

   static char namdev [5][12] =
      {"FOLD-MIR","HEAT-MIR","POL-WHL","HALFWAVE","STIMLAMP"} ;

   static char nampos [5][4][12] =
      {
      "CAL",        "CORONA",     "?",          "?",

      "CORONA",     "CAL",        "?",          "?",

      "CLEAR",      "0 DEG",      "30 DEG",     "60 DEG",

      "90 DEG",     "60 DEG",     "30 DEG",     "0 DEG",

      "OFF",        "ON",         "?",          "?",
      } ;

   /*** "Roll" angle set by ground command. ***/
   /*** Specified for each orbit. ***/

   static float gyro_pitch_set [28] =
       {264.61, 264.61, 264.61, 264.61, 264.61, 264.61,  12.74,  12.74,  12.74,
	 12.74,  12.74,  12.74,  12.74,  12.74,  12.74,  12.74,  85.87,  85.87,
	 85.87,  85.87,  38.00,  38.00,  38.00,  38.00,  38.00,  38.00,  38.00
       } ;

   static float rel_pitch_adj [] =
      {
         0.0,   5.0,  -5.0, -10.0, -15.0, -20.0,  15.0, -10.0,
         0.0, -10.0, -14.0,  10.0,   0.0,
         0.0,  21.0,  14.0,   7.0,  -7.0, -14.0,
	 0.0,  10.0,   5.0,   0.0,  -5.0, -10.0, -14.0,   0.0, 10.0, -10.0,  0.0
      } ;

/*
        0.0  :  0- 1
        5.0  :  2-20 
      355.0  :  3- 1
      350.0  :  3-16
      345.0  :  4- 2
      340.0  :  4-17
       10.0  :  5- 1
	5.0  :  5-15
	0.0  :  6- 1
      350.0  :  9- 2
      345.0  : 10- 2
	5.0  : 13- 1
      355.0  : 14- 1
	0.0  : 16- 1
       20.0  : 17- 1
       13.5  : 17-16
	7.0  : 18- 1
      353.0  : 18-16
      346.5  : 19- 1
	0.0  : 20- 1
       10.0  : 22-10
	5.0  : 22-14
        0.0  : 22-16
      355.0  : 22-18
      350.0  : 22-20
      345.0  : 23- 1
      359.0  : 25- 1
 */

   static int pitch_orb [28] = 
      {
        0,  2,  3,  3,  4,  4,  5,  5,  6,  9,
       10, 13, 14, 16, 17, 17, 18, 18, 19, 20,
       22, 22, 22, 22, 22, 23, 25, 99
      } ;

   static int pitch_img [28] = 
      {
	1, 20,  1, 16,  2, 17,  1, 15,  1,  2,
	2,  1,  1,  1,  1, 16,  1, 16,  1,  1,
       10, 14, 16, 18, 20,  1,  1, 99
      } ;

   static float abs_roll_ang [27] = 
      {
       264.61, 269.61, 259.61, 254.61, 249.61, 244.61, 279.61, 274.61,  12.74,
	 2.74, 358.74,  22.74,  12.74,  85.87, 106.87,  99.87,  92.87,  78.87,
        71.87,  38.00,  48.00,  43.00,  38.00,  33.00,  28.00,  24.00,  38.00
      } ;

   static float abs_pitch_set [] =
      {
         0.0,   5.0,  -5.0, -10.0, -15.0, -20.0,  15.0, -10.0,
         0.0, -10.0, -14.0,  10.0,   0.0,
         0.0,  21.0,  14.0,   7.0,  -7.0, -14.0,
	 0.0,  10.0,   5.0,   0.0,  -5.0, -10.0, -14.0,   0.0, 10.0, -10.0,  0.0
      } ;


   /*** Decode Image Housekeeping. ***/
   /*** Fill Raw Spartan housekeeping structure. ***/

   for (i = 0; i < 6; i++)
      SPIH->sync [i] = hkbuf [i] ;
   SPIH->met = int4 (&hkbuf [6]) ;
   SPIH->rel_pitch_ang = int2 (&hkbuf [10]) ;

   SPIH->fold_mir_pos = hkbuf [12] & 001 ;
   SPIH->heat_mir_pos = (hkbuf [12] & 002) >> 1 ;
   SPIH->pw_pos = (hkbuf [12] & 014) >> 2 ;
   SPIH->half_wave_pos = (hkbuf [12] & 060) >> 4 ;
   SPIH->num_img_sum = (hkbuf [13] & 017) ;
   SPIH->stim_lamp_stat = (hkbuf [13] & 040) >> 5 ;

   SPIH->pos [0] = SPIH->fold_mir_pos ;
   SPIH->pos [1] = SPIH->heat_mir_pos ;
   SPIH->pos [2] = SPIH->pw_pos ;
   SPIH->pos [3] = SPIH->half_wave_pos ;
   SPIH->pos [4] = SPIH->stim_lamp_stat ;

   SPIH->ccd_expdur = int2 (&hkbuf [14]) ;
   SPIH->orbit_num = hkbuf [16] ;
   SPIH->image_num = hkbuf [17] ;
   SPIH->ccd_temp_beg = int2 (&hkbuf [18]) ;
   SPIH->ccd_temp_end = int2 (&hkbuf [20]) ;
   SPIH->pes_yaw_mean = int2 (&hkbuf [22]) ;
   SPIH->pes_yaw_min  = int2 (&hkbuf [24]) ;
   SPIH->pes_yaw_max  = int2 (&hkbuf [26]) ;
   SPIH->pes_pitch_mean = int2 (&hkbuf [28]) ;
   SPIH->pes_pitch_min  = int2 (&hkbuf [30]) ;
   SPIH->pes_pitch_max  = int2 (&hkbuf [32]) ;
   SPIH->cur_ccd_line = int2 (&hkbuf [44]) ;

   /*** Print header. ***/

   if (pmode)
      {
      fprintf (stdout, "SPIH->met           : %10d\n", SPIH->met           ) ;
      fprintf (stdout, "SPIH->rel_pitch_ang : %10d\n", SPIH->rel_pitch_ang ) ;
      fprintf (stdout, "SPIH->fold_mir_pos  : %10d\n", SPIH->fold_mir_pos  ) ;
      fprintf (stdout, "SPIH->heat_mir_pos  : %10d\n", SPIH->heat_mir_pos ) ;
      fprintf (stdout, "SPIH->pw_pos        : %10d\n", SPIH->pw_pos        ) ;
      fprintf (stdout, "SPIH->half_wave_pos : %10d\n", SPIH->half_wave_pos ) ;
      fprintf (stdout, "SPIH->num_img_sum   : %10d\n", SPIH->num_img_sum   ) ;
      fprintf (stdout, "SPIH->stim_lamp_stat: %10d\n", SPIH->stim_lamp_stat) ;
      fprintf (stdout, "SPIH->ccd_expdur    : %10d\n", SPIH->ccd_expdur    ) ;
      fprintf (stdout, "SPIH->orbit_num     : %10d\n", SPIH->orbit_num     ) ;
      fprintf (stdout, "SPIH->image_num     : %10d\n", SPIH->image_num     ) ;
      fprintf (stdout, "SPIH->ccd_temp_beg  : %10d\n", SPIH->ccd_temp_beg  ) ;
      fprintf (stdout, "SPIH->ccd_temp_end  : %10d\n", SPIH->ccd_temp_end  ) ;
      fprintf (stdout, "SPIH->pes_yaw_mean  : %10d\n", SPIH->pes_yaw_mean  ) ;
      fprintf (stdout, "SPIH->pes_yaw_min   : %10d\n", SPIH->pes_yaw_min   ) ;
      fprintf (stdout, "SPIH->pes_yaw_max   : %10d\n", SPIH->pes_yaw_max   ) ;
      fprintf (stdout, "SPIH->pes_pitch_mean: %10d\n", SPIH->pes_pitch_mean) ;
      fprintf (stdout, "SPIH->pes_pitch_min : %10d\n", SPIH->pes_pitch_min ) ;
      fprintf (stdout, "SPIH->pes_pitch_max : %10d\n", SPIH->pes_pitch_max ) ;
      fprintf (stdout, "SPIH->cur_ccd_line  : %10d\n", SPIH->cur_ccd_line  ) ;
      }

   /*** MET = 0 corresponds to 1995 DOY 251 15:42:25 + 00:05:10 = 15:47:35 ***/

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPIH->met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   month_day (iyear, idoy, &imonth, &iday) ;

   SPII->swvers		= 1.00 ;
   SPII->dtype		= 1 ;
   SPII->xdim		= 320 ;
   SPII->ydim		= 256 ;
   SPII->zdim		= 1 ;
   SPII->nbypx		= sizeof (short) ;
   SPII->dmin		= 0 ;
   SPII->dmax		= 0 ;
   SPII->smin		= 0 ;
   SPII->smax		= 0 ;
   SPII->year		= iyear ;
   if (SPII->year < 1900)   SPII->year += 1900 ;
   SPII->month		= imonth ;
   SPII->day		= iday ;
   SPII->doy		= idoy ;
   SPII->hour		= ihour ;
   SPII->minute		= iminute ;
   SPII->second		= (int) isecond ;
   SPII->cal_year	= 0 ;
   SPII->cal_month	= 0 ;
   SPII->cal_day	= 0 ;
   SPII->nimsum		= SPIH->num_img_sum ;
   SPII->orbit_num	= SPIH->orbit_num ;
   SPII->image_num	= SPIH->image_num ;
   SPII->expdur	= (float) SPIH->ccd_expdur * 0.008 ;	/* seconds. */
   SPII->ccd_temp_beg   = ((SPIH->ccd_temp_beg * 36680.0) / 4096.0 - 13570.0)
			   / 100.0 ;
   SPII->ccd_temp_end   = ((SPIH->ccd_temp_end * 36680.0) / 4096.0 - 13570.0)
			   / 100.0 ;
   SPII->pes_yaw_ave    = (SPIH->pes_yaw_mean   * 5.0 ) / 4096.0 ;
   SPII->pes_yaw_min    = (SPIH->pes_yaw_min    * 5.0 ) / 4096.0 ;
   SPII->pes_yaw_max    = (SPIH->pes_yaw_max    * 5.0 ) / 4096.0 ;
   SPII->pes_pitch_ave  = (SPIH->pes_pitch_mean * 5.0 ) / 4096.0 ;
   SPII->pes_pitch_min  = (SPIH->pes_pitch_min  * 5.0 ) / 4096.0 ;
   SPII->pes_pitch_max  = (SPIH->pes_pitch_max  * 5.0 ) / 4096.0 ;

   SPII->odisk		= 1.25 ;			/* solar radii */
   SPII->bscale		= 1.0 ;
   SPII->bzero		= 0.0 ;
   SPII->pixrs		= 0.0 ;
   SPII->pangle		= 0.0 ;
   SPII->bangle		= 0.0 ;
   SPII->langle		= 0.0 ;
   SPII->ra		= 0.0 ;
   SPII->dec		= 0.0 ;
   SPII->hangle		= 0.0 ;

   /*** If rel_pitch_ang has changed, get new value from look-up table. ***/

/*
   if (SPIH->rel_pitch_ang != rel_pitch_prev)
      {
      rel_pitch_new = rel_pitch_adj [irel++] ;
      fprintf (FD_ELOG, "orb/img: %2d-%2d  irel:%2d  gyro_pitch:%7.2f  new_pitch:%3d  nangle:%7.2f\n",
	       SPII->orbit_num, SPII->image_num,
	       irel,
	       gyro_pitch_set [SPIH->orbit_num],  rel_pitch_new,
	       gyro_pitch_set [SPIH->orbit_num] + rel_pitch_new) ;
      fflush (FD_ELOG) ;
      rel_pitch_prev = SPIH->rel_pitch_ang ;
      }
 */
   pitch_id = (float) SPIH->orbit_num + (float) SPIH->image_num / 100.0 ;
   for (i = 0; i < 27; i++)
      {
      pitch_beg = (float) pitch_orb [i]   + (float) pitch_img [i]   / 100.0 ;
      pitch_end = (float) pitch_orb [i+1] + (float) pitch_img [i+1] / 100.0 ;
      if (pitch_id >= pitch_beg && pitch_id < pitch_end)
         {
	 roll_ang = abs_roll_ang [i] ;
	 break ;
         }
      }

   if (roll_ang > 360.0)  roll_ang -= 360.0 ;
   if (roll_ang <   0.0)  roll_ang += 360.0 ;
/*
 *   fprintf (FD_ELOG, "orb/img: %2d-%2d  i:%2d  nangle:%7.2f\n",
 *      SPII->orbit_num, SPII->image_num, i, roll_ang) ;
 *   fflush (FD_ELOG) ;
 */

/*
 *   SPII->nangle		= gyro_pitch_set [SPIH->orbit_num] + rel_pitch_new ;
 */
   SPII->nangle		= roll_ang ;
   if (SPII->nangle > 360.0)  SPII->nangle -= 360.0 ;
   if (SPII->nangle <   0.0)  SPII->nangle += 360.0 ;
   SPII->elev		= 0.0 ;
   SPII->azimuth	= 0.0 ;
   SPII->rsun		= 960.75 ;	/* Solar Radius (arcsec). */
   SPII->xcen		= 0.0 ;
   SPII->ycen		= 0.0 ;
   SPII->rmin		= 0.0 ;
   SPII->rmax		= 0.0 ;
   SPII->wavelen	= 477.0 ;	/* Central Wavelength (nm). */
   SPII->wavewid	=  60.0 ;	/* Filter bandpass (nm), FWHM +/-6 nm.*/

   for (i = 0; i < 5; i++)
      {
      ipos = SPIH->pos [i] ;
      strcpy (&SPII->optdev [i][0], &namdev [i][0]) ;		/* Dev. Name. */
      SPII->optdev [i][11] = '\0';

      strcpy (&SPII->optpos [i][0], &nampos [i][ipos][0]) ;	/* Position. */
      SPII->optpos [i][11] = '\0';
      }

   /*** Print housekeeping. ***/

   if (pmode)
      {
      fprintf (stdout, "SPII->swvers: 		%10d\n",	SPII->swvers) ;
      fprintf (stdout, "SPII->dtype:		%10d\n",	SPII->dtype) ;
      fprintf (stdout, "SPII->xdim:		%10d\n",	SPII->xdim) ;
      fprintf (stdout, "SPII->ydim:		%10d\n",	SPII->ydim) ;
      fprintf (stdout, "SPII->zdim:		%10d\n",	SPII->zdim) ;
      fprintf (stdout, "SPII->nbypx:		%10d\n",	SPII->nbypx) ;
      fprintf (stdout, "SPII->dmin:		%10d\n",	SPII->dmin) ;
      fprintf (stdout, "SPII->dmax:		%10d\n",	SPII->dmax) ;
      fprintf (stdout, "SPII->smin:		%10d\n",	SPII->smin) ;
      fprintf (stdout, "SPII->smax:		%10d\n",	SPII->smax) ;
      fprintf (stdout, "SPII->year:		%10d\n",	SPII->year) ;
      fprintf (stdout, "SPII->month:		%10d\n",	SPII->month) ;
      fprintf (stdout, "SPII->day:		%10d\n",	SPII->day) ;
      fprintf (stdout, "SPII->doy:		%10d\n",	SPII->doy) ;
      fprintf (stdout, "SPII->hour:		%10d\n",	SPII->hour) ;
      fprintf (stdout, "SPII->minute:		%10d\n",	SPII->minute) ;
      fprintf (stdout, "SPII->second:		%10d\n",	SPII->second) ;
      fprintf (stdout, "SPII->cal_year:		%10d\n",	SPII->cal_year) ;
      fprintf (stdout, "SPII->cal_month:	%10d\n",	SPII->cal_month) ;
      fprintf (stdout, "SPII->cal_day:		%10d\n",	SPII->cal_day) ;
      fprintf (stdout, "SPII->nimsum:		%10d\n",	SPII->nimsum) ;


      fprintf (stdout, "SPII->expdur:		%10.4\n",	SPII->expdur) ;
      fprintf (stdout, "SPII->ccd_temp_beg:	%10.4\n",	SPII->ccd_temp_beg) ;
      fprintf (stdout, "SPII->ccd_temp_end:	%10.4\n",	SPII->ccd_temp_end) ;
      fprintf (stdout, "SPII->pes_yaw_ave:	%10.4\n",	SPII->pes_yaw_ave) ;
      fprintf (stdout, "SPII->pes_yaw_min:	%10.4\n",	SPII->pes_yaw_min) ;
      fprintf (stdout, "SPII->pes_yaw_max:	%10.4\n",	SPII->pes_yaw_max) ;
      fprintf (stdout, "SPII->pes_pitch_ave:	%10.4\n",	SPII->pes_yaw_ave) ;
      fprintf (stdout, "SPII->pes_pitch_min:	%10.4\n",	SPII->pes_yaw_min) ;
      fprintf (stdout, "SPII->pes_pitch_max:	%10.4\n",	SPII->pes_yaw_max) ;

      fprintf (stdout, "SPII->odisk:		%10.4\n",	SPII->odisk) ;
      fprintf (stdout, "SPII->bscale:		%10.4f\n",	SPII->bscale) ;
      fprintf (stdout, "SPII->bzero:		%10.4f\n",	SPII->bzero) ;
      fprintf (stdout, "SPII->pixrs:		%10.4f\n",	SPII->pixrs) ;
      fprintf (stdout, "SPII->pangle:		%10.4f\n",	SPII->pangle) ;
      fprintf (stdout, "SPII->bangle:		%10.4f\n",	SPII->bangle) ;
      fprintf (stdout, "SPII->langle:		%10.4f\n",	SPII->langle) ;
      fprintf (stdout, "SPII->hangle:		%10.4f\n",	SPII->hangle) ;
      fprintf (stdout, "SPII->nangle:		%10.4f\n",	SPII->nangle) ;
      fprintf (stdout, "SPII->elev:		%10.4f\n",	SPII->elev) ;
      fprintf (stdout, "SPII->azimuth:		%10.4f\n",	SPII->azimuth) ;
      fprintf (stdout, "SPII->rsun:		%10.4f\n",	SPII->rsun) ;
      fprintf (stdout, "SPII->xcen:		%10.4f\n",	SPII->xcen) ;
      fprintf (stdout, "SPII->ycen:		%10.4f\n",	SPII->ycen) ;
      fprintf (stdout, "SPII->rmin:		%10.4f\n",	SPII->rmin) ;
      fprintf (stdout, "SPII->rmax:		%10.4f\n",	SPII->rmax) ;
      fprintf (stdout, "SPII->wavelen:		%10.4f\n",	SPII->wavelen) ;
      fprintf (stdout, "SPII->wavewid:		%10.4f\n",	SPII->wavewid) ;
      for (i = 0; i < 5; i++)
	 {
         fprintf (stdout, "SPII->optdev[%d]: %-16s	SPII->optpos[%d]: %s\n",
		  i, &SPII->optdev [i][0], i, &SPII->optpos[i][0]) ;
	 }
      }

   return (0) ;
}
