/* find_sync.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-4 Mission
 * Find sync pattern in telemetry stream.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Function value = array byte position for the start of the sync pattern.
 * Return the telemetry type as a parameter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   13 May 1998
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2014lib.h"

int
find_sync ()

{
   extern FILE *FP_RAW ;
   extern long ifloc ;		/* Byte position within telemetry file. */
   extern char tlmbuf [] ;	/* Telemetry buffer. */
   int i ;			/* Loop index. */
   int nelrd ;			/* # elements read from telemetry file. */
   int tlmtype ;		/* Data Frame Type: 0=eng, 4=img, 8=msg. */
   long filepos ;

   /*** Read a block of data from input file. ***/

   nelrd = fread ((char *) tlmbuf, 1, BLKSIZ, FP_RAW) ;
   fseek (FP_RAW, (long) (-nelrd), SEEK_CUR) ;

   /*** Find sync pattern. */

   filepos = -1 ;

   for (i = 0 ; i < BLKSIZ-6 ;  i++)
      if ( tlmbuf [i]   == 0x33 && tlmbuf [i+1] == 0x2a &&
           tlmbuf [i+2] == 0x66 && tlmbuf [i+3] == 0x55 &&
	   tlmbuf [i+4] == 0x00 )
	 {
         filepos = (long) i ;
	 tlmtype = tlmbuf [i+5] ;   /* Data Frame type: 0=eng, 4=img, 8=msg. */
         break ;
         }

   ifloc += filepos ;			/* Update file position indicator. */
   fseek (FP_RAW, filepos, SEEK_CUR) ;	/* Move file pointer to sync location.*/

   return (tlmtype) ;
}
