/* mkengfiles.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read Spartan 201-4 Engineering Telemetry structure from a disk file.
 * Write individual Spartan 201-3 Engineering Telemetry items to separate files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   9 October 1996
 * 12 May 1998: Modify by adding uvcs_yfp, uvcs_rfp, wlc_yfp, wlc_rfp.
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2014lib.h"
unsigned char ebuf [NBYEDF] ;		/* Engineering telemetry buffer. */

main (argc, argv)

int argc ;
char *argv [] ;

{
   FILE *FP_ETLM ;
   FILE *FP_MET ;
   FILE *FP_V28 ;
   FILE *FP_V5 ;
   FILE *FP_V12 ;
   FILE *FP_TED_V ;
   FILE *FP_CCD_BAT ;
   FILE *FP_CCD_5 ;
   FILE *FP_CCD_P15 ;
   FILE *FP_CCD_M15 ;
   FILE *FP_UVCS_YFP ;
   FILE *FP_UVCS_RFP ;
   FILE *FP_CSP ;
   FILE *FP_FMII ;
   FILE *FP_TRG ;
   FILE *FP_YFP ;
   FILE *FP_RFP ;
   FILE *FP_PESP ;
   FILE *FP_PESY ;
   FILE *FP_FBLK ;
   FILE *FP_O1 ;
   FILE *FP_FIL ;
   FILE *FP_ELC ;
   FILE *FP_DET ;
   FILE *FP_WLC_YFP ;
   FILE *FP_WLC_RFP ;
   FILE *FP_TED ;
   FILE *FP_P5V ;
   FILE *FP_EP2 ;
   FILE *FP_K68 ;
   FILE *FP_PIA0AR ;
   FILE *FP_PIA0BR ;
   FILE *FP_PIA1AR ;
   FILE *FP_PIA1BR ;
   FILE *FP_RSTS ;
   FILE *FP_VER ;
   FILE *FP_RELP ;
   FILE *FP_ORB ;
   FILE *FP_IMG ;
   FILE *FP_ACSBP ;
   FILE *FP_PIA1BW ;
   FILE *FP_MSGS ;
   FILE *FP_ACSB07 ;
   FILE *FP_OPTDAT ;
   FILE *FP_LSR ;
   FILE *FP_LSS ;
   FILE *FP_TBL ;
   FILE *FP_PRG ;
   FILE *FP_EOM ;
   FILE *FP_UPCHG ;
   FILE *FP_UVRAST ;
   FILE *FP_WLCCAL ;
   FILE *FP_LWLC ;
   FILE *FP_CCDMD ;
   FILE *FP_CCDER ;
   FILE *FP_CCDEXP ;
   static char file_eng [80] ;
   static char file_met [] = "T97_MET.dat" ;
   static char file_v28 [] = "T97_V28.dat" ;
   static char file_v5  [] = "T97_V5.dat" ;
   static char file_v12  [] = "T97_V12.dat" ;
   static char file_ted_v [] = "T97_TED_V.dat" ;
   static char file_ccd_bat [] = "T97_CCD_BAT.dat" ;
   static char file_ccd_5   [] = "T97_CCD_5.dat" ;
   static char file_ccd_p15 [] = "T97_CCD_P15.dat" ;
   static char file_ccd_m15 [] = "T97_CCD_M15.dat" ;
   static char file_uvcs_yfp [] = "T97_UVCS_YFP.dat" ;
   static char file_uvcs_rfp [] = "T97_UVCS_RFP.dat" ;
   static char file_csp [] = "T97_CSP.dat" ;
   static char file_fmii [] = "T97_FMII.dat" ;
   static char file_trg [] = "T97_TRG.dat" ;
   static char file_yfp [] = "T97_YFP.dat" ;
   static char file_rfp [] = "T97_RFP.dat" ;
   static char file_yaw_off [] = "T97_YAW_OFF.dat" ;
   static char file_roll_off [] = "T97_ROLL_OFF.dat" ;
   static char file_pesp [] = "T97_PESP.dat" ;
   static char file_pesy [] = "T97_PESY.dat" ;
   static char file_fblk [] = "T97_FBLK.dat" ;
   static char file_o1 [] = "T97_O1.dat" ;
   static char file_fil [] = "T97_FIL.dat" ;
   static char file_elc [] = "T97_ELC.dat" ;
   static char file_det [] = "T97_DET.dat" ;
   static char file_wlc_yfp [] = "T97_WLC_YFP.dat" ;
   static char file_wlc_rfp [] = "T97_WLC_RFP.dat" ;
   static char file_ted [] = "T97_TED.dat" ;
   static char file_p5v [] = "T97_P5V.dat" ;
   static char file_ep2 [] = "T97_EP2.dat" ;
   static char file_k68 [] = "T97_K68.dat" ;
   static char file_pia0ar [] = "T97_PIA0AR.dat" ;
   static char file_pia0br [] = "T97_PIA0BR.dat" ;
   static char file_pia1ar [] = "T97_PIA1AR.dat" ;
   static char file_pia1br [] = "T97_PIA1BR.dat" ;
   static char file_rsts [] = "T97_RSTS.dat" ;
   static char file_ver [] = "T97_VER.dat" ;
   static char file_relp [] = "T97_RELP.dat" ;
   static char file_orb [] = "T97_ORB.dat" ;
   static char file_img [] = "T97_IMG.dat" ;
   static char file_acsbp [] = "T97_ACSBP.dat" ;
   static char file_pia1bw [] = "T97_PIA1BW.dat" ;
   static char file_msgs [] = "T97_MSGS.dat" ;
   static char file_acsb07 [] = "T97_ACSB07.dat" ;
   static char file_optdat [] = "T97_OPTDAT.dat" ;
   static char file_lsr [] = "T97_LSR.dat" ;
   static char file_lss [] = "T97_LSS.dat" ;
   static char file_tbl [] = "T97_TBL.dat" ;
   static char file_prg [] = "T97_PRG.dat" ;
   static char file_eom [] = "T97_EOM.dat" ;
   static char file_upchg [] = "T97_UPCHG.dat" ;
   static char file_uvrast [] = "T97_UVRAST.dat" ;
   static char file_wlccal [] = "T97_WLCCAL.dat" ;
   static char file_lwlc [] = "T97_LWLC.dat" ;
   static char file_ccdmd [] = "T97_CCDMD.dat" ;
   static char file_ccder [] = "T97_CCDER.dat" ;
   static char file_ccdexp [] = "T97_CCDEXP.dat" ;
   int irec = 0 ;
   int size_SPEH ;

   size_SPEH = sizeof (SPEH) ;

   if (argc == 2)
      strcpy (file_eng, argv [1]) ;
   else
      {
      fprintf (stderr, "usage: %s eng_telem_name\n", argv [0]) ;
      exit (0) ;
      }

   if (( FP_ETLM = fopen (file_eng, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_eng) ;
      exit (-1) ;
      }

   if (( FP_MET = fopen (file_met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_met) ;
      exit (-1) ;
      }

   if (( FP_V28 = fopen (file_v28, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_v28) ;
      exit (-1) ;
      }

   if (( FP_V5 = fopen (file_v5, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_v5) ;
      exit (-1) ;
      }

   if (( FP_V12 = fopen (file_v12, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_v12) ;
      exit (-1) ;
      }

   if (( FP_TED_V = fopen (file_ted_v, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ted_v) ;
      exit (-1) ;
      }

   if (( FP_CCD_BAT = fopen (file_ccd_bat, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccd_bat) ;
      exit (-1) ;
      }

   if (( FP_CCD_5 = fopen (file_ccd_5, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccd_5) ;
      exit (-1) ;
      }

   if (( FP_CCD_P15 = fopen (file_ccd_p15, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccd_p15) ;
      exit (-1) ;
      }

   if (( FP_CCD_M15 = fopen (file_ccd_m15, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccd_m15) ;
      exit (-1) ;
      }

   if (( FP_UVCS_YFP = fopen (file_uvcs_yfp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_uvcs_yfp) ;
      exit (-1) ;
      }

   if (( FP_UVCS_RFP = fopen (file_uvcs_rfp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_uvcs_rfp) ;
      exit (-1) ;
      }

   if (( FP_CSP = fopen (file_csp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_csp) ;
      exit (-1) ;
      }

   if (( FP_FMII = fopen (file_fmii, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_fmii) ;
      exit (-1) ;
      }

   if (( FP_TRG = fopen (file_trg, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_trg) ;
      exit (-1) ;
      }

   if (( FP_YFP = fopen (file_yfp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_yfp) ;
      exit (-1) ;
      }

   if (( FP_RFP = fopen (file_rfp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_rfp) ;
      exit (-1) ;
      }

   if (( FP_PESP = fopen (file_pesp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pesp) ;
      exit (-1) ;
      }

   if (( FP_PESY = fopen (file_pesy, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pesy) ;
      exit (-1) ;
      }

   if (( FP_FBLK = fopen (file_fblk, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_fblk) ;
      exit (-1) ;
      }

   if (( FP_O1 = fopen (file_o1, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_o1) ;
      exit (-1) ;
      }

   if (( FP_FIL = fopen (file_fil, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_fil) ;
      exit (-1) ;
      }

   if (( FP_ELC = fopen (file_elc, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_elc) ;
      exit (-1) ;
      }

   if (( FP_DET = fopen (file_det, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_det) ;
      exit (-1) ;
      }

   if (( FP_WLC_YFP = fopen (file_wlc_yfp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_wlc_yfp) ;
      exit (-1) ;
      }

   if (( FP_WLC_RFP = fopen (file_wlc_rfp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_wlc_rfp) ;
      exit (-1) ;
      }

   if (( FP_TED = fopen (file_ted, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ted) ;
      exit (-1) ;
      }

   if (( FP_P5V = fopen (file_p5v, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_p5v) ;
      exit (-1) ;
      }

   if (( FP_EP2 = fopen (file_ep2, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ep2) ;
      exit (-1) ;
      }

   if (( FP_K68 = fopen (file_k68, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_k68) ;
      exit (-1) ;
      }

   if (( FP_PIA0AR = fopen (file_pia0ar, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pia0ar) ;
      exit (-1) ;
      }

   if (( FP_PIA0BR = fopen (file_pia0br, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pia0br) ;
      exit (-1) ;
      }

   if (( FP_PIA1AR = fopen (file_pia1ar, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pia1ar) ;
      exit (-1) ;
      }

   if (( FP_PIA1BR = fopen (file_pia1br, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pia1br) ;
      exit (-1) ;
      }

   if (( FP_RSTS = fopen (file_rsts, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_rsts) ;
      exit (-1) ;
      }

   if (( FP_VER = fopen (file_ver, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ver) ;
      exit (-1) ;
      }

   if (( FP_RELP = fopen (file_relp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_relp) ;
      exit (-1) ;
      }

   if (( FP_ORB = fopen (file_orb, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_orb) ;
      exit (-1) ;
      }

   if (( FP_IMG = fopen (file_img, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_img) ;
      exit (-1) ;
      }

   if (( FP_ACSBP = fopen (file_acsbp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_acsbp) ;
      exit (-1) ;
      }

   if (( FP_PIA1BW = fopen (file_pia1bw, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pia1bw) ;
      exit (-1) ;
      }

   if (( FP_MSGS = fopen (file_msgs, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msgs) ;
      exit (-1) ;
      }

   if (( FP_ACSB07 = fopen (file_acsb07, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_acsb07) ;
      exit (-1) ;
      }

   if (( FP_OPTDAT = fopen (file_optdat, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_optdat) ;
      exit (-1) ;
      }

   if (( FP_LSR = fopen (file_lsr, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_lsr) ;
      exit (-1) ;
      }

   if (( FP_LSS = fopen (file_lss, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_lss) ;
      exit (-1) ;
      }

   if (( FP_TBL = fopen (file_tbl, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_tbl) ;
      exit (-1) ;
      }

   if (( FP_PRG = fopen (file_prg, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_prg) ;
      exit (-1) ;
      }

   if (( FP_EOM = fopen (file_eom, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_eom) ;
      exit (-1) ;
      }

   if (( FP_UPCHG = fopen (file_upchg, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_upchg) ;
      exit (-1) ;
      }

   if (( FP_UVRAST = fopen (file_uvrast, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_uvrast) ;
      exit (-1) ;
      }

   if (( FP_WLCCAL = fopen (file_wlccal, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_wlccal) ;
      exit (-1) ;
      }

   if (( FP_LWLC = fopen (file_lwlc, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_lwlc) ;
      exit (-1) ;
      }

   if (( FP_CCDMD = fopen (file_ccdmd, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccdmd) ;
      exit (-1) ;
      }

   if (( FP_CCDER = fopen (file_ccder, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccder) ;
      exit (-1) ;
      }

   if (( FP_CCDEXP = fopen (file_ccdexp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccdexp) ;
      exit (-1) ;
      }

   /*** Read Engineering Telemetry to a disk file. ***/

/*
   while (fread (SPEH, size_SPEH, 1, FP_ETLM) == 1)
      {
      fwrite (SPEH->met, sizeof (int), 1, FP_MET) ;
      fwrite (SPEH->p28v, sizeof (int), 1, FP_V28) ;
      fwrite (SPEH->p5v, sizeof (int), 1, FP_V5) ;
      fwrite (SPEH->p12v, sizeof (int), 1, FP_V12) ;
      fwrite (SPEH->rs232_ted_volt, sizeof (int), 1, FP_TED_V) ;
      fwrite (SPEH->ccd_bat_p6v, sizeof (int), 1, FP_CCD_BAT) ;
      fwrite (SPEH->ccd_p5v, sizeof (int), 1, FP_CCD_5) ;
      fwrite (SPEH->ccd_p15v, sizeof (int), 1, FP_CCD_P15) ;
      fwrite (SPEH->ccd_m15v, sizeof (int), 1, FP_CCD_M15) ;
      fwrite (SPEH->uvcs_yfp, sizeof (int), 1, FP_UVCS_YFP) ;
      fwrite (SPEH->uvcs_rfp, sizeof (int), 1, FP_UVCS_RFP) ;
      fwrite (SPEH->sun_pres, sizeof (int), 1, FP_CSP) ;
      fwrite (SPEH->fine_mode2, sizeof (int), 1, FP_FMII) ;
      fwrite (SPEH->pitch_trig_rate, sizeof (int), 1, FP_TRG) ;
      fwrite (SPEH->yaw_fine_pos, sizeof (int), 1, FP_YFP) ;
      fwrite (SPEH->roll_fine_pos, sizeof (int), 1, FP_RFP) ;
      fwrite (SPEH->pes_pitch, sizeof (int), 1, FP_PESP) ;
      fwrite (SPEH->pes_yaw, sizeof (int), 1, FP_PESY) ;
      fwrite (SPEH->front_bulk_temp, sizeof (int), 1, FP_FBLK) ;
      fwrite (SPEH->o1_temp, sizeof (int), 1, FP_O1) ;
      fwrite (SPEH->fw_temp, sizeof (int), 1, FP_FIL) ;
      fwrite (SPEH->ccd_elec_temp, sizeof (int), 1, FP_ELC) ;
      fwrite (SPEH->ccd_det_temp, sizeof (int), 1, FP_DET) ;
      fwrite (SPEH->wlc_yfp, sizeof (int), 1, FP_WLC_YFP) ;
      fwrite (SPEH->wlc_rfp, sizeof (int), 1, FP_WLC_RFP) ;
      fwrite (SPEH->ted_hot_temp, sizeof (int), 1, FP_TED) ;
      fwrite (SPEH->p5v_supply_temp, sizeof (int), 1, FP_P5V) ;
      fwrite (SPEH->elec_pkg2_temp, sizeof (int), 1, FP_EP2) ;
      fwrite (SPEH->m68000_temp, sizeof (int), 1, FP_K68) ;
      fwrite (SPEH->, sizeof (int), 1, FP_PIA0AR) ;
      fwrite (SPEH->, sizeof (int), 1, FP_PIA0BR) ;
      fwrite (SPEH->, sizeof (int), 1, FP_PIA1AR) ;
      fwrite (SPEH->, sizeof (int), 1, FP_PIA1BR) ;
      fwrite (SPEH->, sizeof (int), 1, FP_RSTS) ;
      fwrite (SPEH->, sizeof (int), 1, FP_VER) ;
      fwrite (SPEH->, sizeof (int), 1, FP_RELP) ;
      fwrite (SPEH->, sizeof (int), 1, FP_ORB) ;
      fwrite (SPEH->, sizeof (int), 1, FP_IMG) ;
      fwrite (SPEH->acs_pointer, sizeof (int), 1, FP_ACSBP) ;
      fwrite (SPEH->, sizeof (int), 1, FP_PIA1BW) ;
      fwrite (SPEH->msg_flag_sot, sizeof (int), 1, FP_MSGS) ;
      fwrite (SPEH->acs_buf, sizeof (int), 1, FP_ACSB07) ;
      fwrite (SPEH->optoo_data_id, sizeof (int), 1, FP_OPDAT) ;
      fwrite (SPEH->last_sunrise, sizeof (int), 1, FP_LSR) ;
      fwrite (SPEH->last_sunset, sizeof (int), 1, FP_LSS) ;
      fwrite (SPEH->table_num, sizeof (int), 1, FP_TBL) ;
      fwrite (SPEH->prog_num, sizeof (int), 1, FP_PRG) ;
      fwrite (SPEH->eom_flag, sizeof (int), 1, FP_EOM) ;
      fwrite (SPEH->undet_pitch_flag, sizeof (int), 1, FP_UPCHG) ;
      fwrite (SPEH->uvcs_raster_flag, sizeof (int), 1, FP_UVRAST) ;
      fwrite (SPEH->wlc_cal_flag, sizeof (int), 1, FP_WLCCAL) ;
      fwrite (SPEH->wlc_liss_flag, sizeof (int), 1, FP_LWLC) ;
      fwrite (SPEH->ccd_mode, sizeof (int), 1, FP_CCDMD) ;
      fwrite (SPEH->ccd_erase, sizeof (int), 1, FP_CCDER) ;
      fwrite (SPEH->ccd_expdur, sizeof (int), 1, FP_CCDEXP
      }
 */

   while (fread (ebuf, 128, 1, FP_ETLM) == 1)
      {
      fwrite (&ebuf [  6], 4, 1, FP_MET) ;

      fwrite (&ebuf [ 12], 2, 1, FP_V28) ;
      fwrite (&ebuf [ 14], 2, 1, FP_V5) ;
      fwrite (&ebuf [ 16], 2, 1, FP_V12) ;

      fwrite (&ebuf [ 20], 2, 1, FP_TED_V) ;
      fwrite (&ebuf [ 22], 2, 1, FP_CCD_BAT) ;
      fwrite (&ebuf [ 24], 2, 1, FP_CCD_5) ;
      fwrite (&ebuf [ 26], 2, 1, FP_CCD_P15) ;
      fwrite (&ebuf [ 28], 2, 1, FP_CCD_M15) ;

      fwrite (&ebuf [ 30], 2, 1, FP_UVCS_YFP) ;
      fwrite (&ebuf [ 32], 2, 1, FP_UVCS_RFP) ;
      fwrite (&ebuf [ 34], 2, 1, FP_CSP) ;
      fwrite (&ebuf [ 36], 2, 1, FP_FMII) ;
      fwrite (&ebuf [ 38], 2, 1, FP_TRG) ;
      fwrite (&ebuf [ 40], 2, 1, FP_YFP) ;
      fwrite (&ebuf [ 42], 2, 1, FP_RFP) ;

      fwrite (&ebuf [ 48], 2, 1, FP_PESP) ;
      fwrite (&ebuf [ 50], 2, 1, FP_PESY) ;
      fwrite (&ebuf [ 52], 2, 1, FP_FBLK) ;
      fwrite (&ebuf [ 54], 2, 1, FP_O1) ;
      fwrite (&ebuf [ 56], 2, 1, FP_FIL) ;
      fwrite (&ebuf [ 58], 2, 1, FP_ELC) ;
      fwrite (&ebuf [ 60], 2, 1, FP_DET) ;

      fwrite (&ebuf [ 62], 2, 1, FP_WLC_YFP) ;
      fwrite (&ebuf [ 64], 2, 1, FP_WLC_RFP) ;
      fwrite (&ebuf [ 66], 2, 1, FP_TED) ;
      fwrite (&ebuf [ 68], 2, 1, FP_P5V) ;
      fwrite (&ebuf [ 70], 2, 1, FP_EP2) ;
      fwrite (&ebuf [ 72], 2, 1, FP_K68) ;

      fwrite (&ebuf [ 74], 1, 1, FP_PIA0AR) ;
      fwrite (&ebuf [ 75], 1, 1, FP_PIA0BR) ;
      fwrite (&ebuf [ 76], 1, 1, FP_PIA1AR) ;
      fwrite (&ebuf [ 77], 1, 1, FP_PIA1BR) ;
      fwrite (&ebuf [ 78], 1, 1, FP_RSTS) ;
      fwrite (&ebuf [ 79], 1, 1, FP_VER) ;
      fwrite (&ebuf [ 80], 2, 1, FP_RELP) ;
      fwrite (&ebuf [ 82], 1, 1, FP_ORB) ;
      fwrite (&ebuf [ 83], 1, 1, FP_IMG) ;
      fwrite (&ebuf [ 84], 1, 1, FP_ACSBP) ;
      fwrite (&ebuf [ 85], 1, 1, FP_PIA1BW) ;
      fwrite (&ebuf [ 86], 1, 1, FP_MSGS) ;
      fwrite (&ebuf [ 87], 8, 1, FP_ACSB07) ;

      fwrite (&ebuf [ 95], 1, 1, FP_OPTDAT) ;
      fwrite (&ebuf [ 96], 4, 1, FP_LSR) ;
      fwrite (&ebuf [100], 4, 1, FP_LSS) ;

      fwrite (&ebuf [112], 1, 1, FP_TBL) ;
      fwrite (&ebuf [113], 1, 1, FP_PRG) ;
      fwrite (&ebuf [114], 1, 1, FP_EOM) ;
      fwrite (&ebuf [115], 1, 1, FP_UPCHG) ;

      fwrite (&ebuf [117], 1, 1, FP_UVRAST) ;
      fwrite (&ebuf [118], 1, 1, FP_WLCCAL) ;
      fwrite (&ebuf [119], 1, 1, FP_LWLC) ;
      fwrite (&ebuf [114], 1, 1, FP_CCDMD) ;
      fwrite (&ebuf [125], 1, 1, FP_CCDER) ;
      fwrite (&ebuf [126], 2, 1, FP_CCDEXP) ;
      }

   fclose (FP_ETLM) ;
   fclose (FP_MET) ;
   fclose (FP_V28) ;
   fclose (FP_V5) ;
   fclose (FP_V12) ;
   fclose (FP_TED_V) ;
   fclose (FP_CCD_BAT) ;
   fclose (FP_CCD_5) ;
   fclose (FP_CCD_P15) ;
   fclose (FP_CCD_M15) ;
   fclose (FP_UVCS_YFP) ;
   fclose (FP_UVCS_RFP) ;
   fclose (FP_CSP) ;
   fclose (FP_FMII) ;
   fclose (FP_TRG) ;
   fclose (FP_YFP) ;
   fclose (FP_RFP) ;
   fclose (FP_PESP) ;
   fclose (FP_PESY) ;
   fclose (FP_O1) ;
   fclose (FP_FIL) ;
   fclose (FP_ELC) ;
   fclose (FP_DET) ;
   fclose (FP_WLC_YFP) ;
   fclose (FP_WLC_RFP) ;
   fclose (FP_TED) ;
   fclose (FP_P5V) ;
   fclose (FP_EP2) ;
   fclose (FP_K68) ;
   fclose (FP_PIA0AR) ;
   fclose (FP_PIA0BR) ;
   fclose (FP_PIA1AR) ;
   fclose (FP_PIA1BR) ;
   fclose (FP_RSTS) ;
   fclose (FP_VER) ;
   fclose (FP_RELP) ;
   fclose (FP_ORB) ;
   fclose (FP_IMG) ;
   fclose (FP_ACSBP) ;
   fclose (FP_PIA1BW) ;
   fclose (FP_MSGS) ;
   fclose (FP_ACSB07) ;
   fclose (FP_OPTDAT) ;
   fclose (FP_LSR) ;
   fclose (FP_LSS) ;
   fclose (FP_TBL) ;
   fclose (FP_PRG) ;
   fclose (FP_EOM) ;
   fclose (FP_UPCHG) ;
   fclose (FP_UVRAST) ;
   fclose (FP_WLCCAL) ;
   fclose (FP_LWLC) ;
   fclose (FP_CCDMD) ;
   fclose (FP_CCDER) ;
   fclose (FP_CCDEXP) ;

   return (0) ;
}
