/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * NAME		si.c
 *
 * PURPOSE	Extract Spartan 201-4 images from a raw data file.
 * 
 * SYNTAX	si4 tape1.raw
 *
 * HISTORY	si.c SP201-3 16 September 1996 by Andrew L. Stanger, HAO/NCAR
 *		si.c SP201-4 12 May 1998
 *			Insert nbyidf=688 & nbymdf=40 in "sp2014lib.h".
 *
 * + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 */

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
#include <stdio.h>
#define SPARTAN
#include "sp2014lib.h"

FILE *FP_RAW, *FP_IMG, *FP_DLOG, *FP_ILOG, *FP_ELOG, *FP_MLOG, *FP_ETLM,
     *FP_ITLM, *FP_MTLM ;

int engmet_prev = 0 ;
int imgmet_prev = 0 ;
int msgmet_prev = 0 ;
int idf = 0;			/* idf: current data frame index. */
int ib = 0;			/* ib: current byte index. */
int blkcnt  ;			/* # data blocks in raw tape raw file. */
int dblknum  ;			/* Data block number. */
int derrcnt ;			/* Data block error count. */
int npxlin = NX ;		/* # pixels per line. */
int nbyimg = NX * NY * NBYPIX ;	/* # bytes per image = 256 * 320 * 2. */
int nbyeng = NBYEDF ;		/* # bytes in engineering telemetry frame. */
int nbymsg = NBYMDF ;		/* # bytes in message     telemetry frame. */
int nbypix = NBYPIX ;		/* # bytes per pixel. */
int npximg = NX * NY ;		/* # pixels per image  = 256 * 320. */
int rel_pitch_prev = -1 ;	/* Previous Relative Pitch Angle. */
int rel_pitch_new  =  0 ;	/* Current  Relative Pitch Angle. */
int pitch_orb_prev = -1 ;	/* Previous Pitch orbit. */
int pitch_img_prev = -1 ;	/* Previous Pitch image. */
int dftype = -1 ;		/* Data Frame type: 0=eng, 4=img, 8=msg. */
int dfprev = -1 ;		/* Data Frame type (previous). */
long ifloc ;			/* File pointer. */
char dfname  [80] ;
unsigned char  hkbuf  [NBYHK] ;		/* Housekeeping buffer. */
unsigned short imgbuf [NY][NX] ;	/* Image 2D buffer. */
unsigned char  tlmbuf [BLKSIZ] ;	/* Block buffer. */
unsigned char  *blkhdr ;
unsigned char  *dbuf ;

/*** Define tape header structure. ***/

struct tape_header
{
unsigned long blkcnt ;
unsigned short blksize ;
unsigned short tapenum ;
unsigned short uerrcnt ;
} ;

struct tape_header hdr ;

/*** Define UART structure array. ***/

struct uart
{
unsigned long  blknum ;
unsigned short bytnum ;
unsigned char err ;
unsigned char z ;
};

struct uart uarts [512] ;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

main (argc, argv)

int argc ;
char *argv [] ;
{


extern unsigned short int2rev () ;
extern unsigned int   int4rev () ;
extern int find_sync () ;

/*** Define arrays. ***/

static char ifile   [80] ;
static char dlognam [80] ;
static char elognam [80] ;
static char ilognam [80] ;
static char mlognam [80] ;
static char etlmnam [80] ;
static char itlmnam [80] ;
static char mtlmnam [80] ;
static unsigned char taphdr [10] ;
static unsigned char ubuf   [512][8] ;

int i, ii ;
int ierr ;
int blksize ;
int tapenum ;
int uerrcnt ;
int ublknum  ;
int bytnum ;
int uerr ;
int z ;
int pmode = 0 ;
int nbyblk = 8192 ;
int nbyend ;
int nelrd ;

   if (argc < 2)
      {
      fprintf (stderr, "Usage: %s input_file_name\n", argv [0]) ;
      exit (0) ;
      }
   else
      {
      sprintf (ifile, "%s", argv [1]) ;
      }

   sprintf (dlognam, "%s.dlog", ifile) ;
   sprintf (elognam, "%s.elog", ifile) ;
   sprintf (ilognam, "%s.ilog", ifile) ;
   sprintf (mlognam, "%s.mlog", ifile) ;
   sprintf (itlmnam, "%s.itlm", ifile) ;
   sprintf (etlmnam, "%s.etlm", ifile) ;
   sprintf (mtlmnam, "%s.mtlm", ifile) ;

   if (( FP_DLOG = fopen (dlognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", dlognam) ;
      exit (-1) ;
      }

   if (( FP_ELOG = fopen (elognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", elognam) ;
      exit (-2) ;
      }

   if (( FP_ILOG = fopen (ilognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", ilognam) ;
      exit (-3) ;
      }

   if (( FP_MLOG = fopen (mlognam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", mlognam) ;
      exit (-4) ;
      }

   if (( FP_ITLM = fopen (itlmnam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", itlmnam) ;
      exit (-5) ;
      }

   if (( FP_ETLM = fopen (etlmnam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", etlmnam) ;
      exit (-6) ;
      }

   if (( FP_MTLM = fopen (mtlmnam, "w")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", mtlmnam) ;
      exit (-7) ;
      }


   fprintf (FP_DLOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fprintf (FP_DLOG, "Log File for Spartan Data File: %s\n", ifile) ;

            fprintf    (FP_ELOG,    "  ifd  ifloc  -- MET --- ddd-hh:mm:ss.sss    +28v     +5v    +12v    TEDv    BATv    c+5v    c+15    c-15     csp    fmII     ptr     yfp     rfp    yoff    roff    ppit      pyaw     fbt     o1t     fwt\n")    ;

   fprintf (FP_ILOG, "%s\n", ifile) ;
   fprintf (FP_ILOG, "------- FILE NAME ------- ------MET----- orb/img  EXP SUM FOLD HEAT POL 1/2   ---- ccd -----  Actual ------ Yaw ------  ----- Pitch ----\n");
   fprintf (FP_ILOG, "                                                  sec     MIR  MIR  WHL WAVE  T-beg   T-end    Pitch   Ave   Min   Max   Ave   Min   Max\n");

   /*** Open telemetry file. ***/

   if (( FP_RAW = fopen (ifile, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", ifile) ;
      exit (-9) ;
      }

   /*** Find sync pattern & move file pointer to start of sync pattern. ***/

   dftype = find_sync () ;

   if (dftype < 0)
      {
      fprintf (stderr, "Unable to find valid sync pattern.\n") ;
      exit (-8) ;
      }

   /*** Process new data frame. ***/

   while (1)
      {
      ierr = new_frame (pmode) ;
      if (ierr < 0)  break ;
      }

/*
 *   newimg (pmode) ;
 */

   fprintf (FP_DLOG, "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * \n") ;
   fclose (FP_RAW) ;
   fclose (FP_IMG) ;
   fclose (FP_DLOG) ;
   fclose (FP_ELOG) ;
   fclose (FP_ILOG) ;
   fclose (FP_MLOG) ;
   fclose (FP_ETLM) ;
   fclose (FP_ITLM) ;
   fclose (FP_MTLM) ;
   exit (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2rev (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   sval = bbuf [1] * 256 + bbuf [0] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4rev (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   /*** Invert byte order, effectively converting from VAX to SGI/Sun order.***/

   ival = bbuf [3] * 16777216 + bbuf [2] * 65536 + bbuf [1] * 256 + bbuf [0] ;

   return (ival) ;
}


/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned short
int2 (bbuf)			/*  Convert byte array to a 16-bit integer. */

unsigned char bbuf [] ;

{
   unsigned short sval ;

   sval = bbuf [0] * 256 + bbuf [1] ;

   return (sval) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
unsigned int
int4 (bbuf)			/*  Convert byte array to a 32-bit integer. */

unsigned char bbuf [] ;

{
   unsigned int ival ;

   ival = bbuf [0] * 16777216 + bbuf [1] * 65536 + bbuf [2] * 256 + bbuf [3] ;

   return (ival) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int2arr (bbuf, sbuf, n_elem)	/*  Convert byte array to 16-bit integers. */

unsigned char  bbuf [] ;	/* Input  buffer. */
unsigned short sbuf [] ;	/* Output buffer. */
int n_elem ;			/* # pixels in output array. */
{
   int i ;

   for (i = 0; i < n_elem; i++)
      {

/*fprintf (stderr, "int2arr> i: %4d  i*2: %4d  i*2+1: %4d\n", i, i*2, i*2+1) ;*/

      *(sbuf + i) = *(bbuf + i*2 + 0) * 256  + *(bbuf + i*2 + 1) ;
      }

   return (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
int4arr (bbuf, ibuf, n_elem)	/*  Convert byte array to 32-bit integers. */

unsigned char bbuf [] ;		/* Input  buffer. */
int ibuf [] ;			/* Output buffer. */
int n_elem ;			/* # pixels in output array. */
{
   int i ;

   for (i = 0; i < n_elem; i++)
   {
      *(ibuf + i) = *(bbuf + i*4) * 16777216 + *(bbuf + i*4 + 1) * 65536 +
                  + *(bbuf + i*4 + 2) * 256  + *(bbuf + i*4 + 3) ;
   }

   return (0) ;
}

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
