/* spetlm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-4 Engineering Telemetry Decoder.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   3 October 1996
 * 12 May 1998: Modify by adding uvcs_yfp, uvcs_rfp, wlc_yfp, wlc_rfp.
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2014lib.h"

spetlm (ebuf, pmode)

unsigned char ebuf [] ;			/* Spartan image housekeeping array. */
int pmode ;				/* Print mode. */

{
   extern int idf ;
   extern long ifloc ;
   extern FILE *FP_ELOG ;
   int i ;
   int iyear, idoy, imonth, iday, ihour, iminute ;
   int kyear, kdoy, khour, kmin ;
   float isecond ;
   float ksec ;

   for (i = 0; i < 6; i++)
	 SPEH->sync [i] = ebuf [i] ;
   SPEH->met             = int4 (&ebuf [6]) ;
   SPEH->p28v            = int2 (&ebuf [12]) ;
   SPEH->p5v             = int2 (&ebuf [14]) ;
   SPEH->rs232_p12v      = int2 (&ebuf [16]) ;
   SPEH->ccd_ted_volt    = int2 (&ebuf [20]) ;
   SPEH->ccd_bat_p6v     = int2 (&ebuf [22]) ;
   SPEH->ccd_p5v         = int2 (&ebuf [24]) ;
   SPEH->ccd_p15v        = int2 (&ebuf [26]) ;
   SPEH->ccd_m15v        = int2 (&ebuf [28]) ;
   SPEH->uvcs_yfp        = int2 (&ebuf [30]) ;
   SPEH->uvcs_rfp        = int2 (&ebuf [32]) ;
   SPEH->sun_pres        = int2 (&ebuf [34]) ;
   SPEH->fine_mode2      = int2 (&ebuf [36]) ;
   SPEH->pitch_trig_rate = int2 (&ebuf [38]) ;
   SPEH->yaw_fine_pos    = int2 (&ebuf [40]) ;
   SPEH->roll_fine_pos   = int2 (&ebuf [42]) ;
   SPEH->yaw_offset      = int2 (&ebuf [44]) ;
   SPEH->roll_offset     = int2 (&ebuf [46]) ;
   SPEH->pes_pitch       = int2 (&ebuf [48]) ;
   SPEH->pes_yaw         = int2 (&ebuf [50]) ;
   SPEH->front_bulk_temp = int2 (&ebuf [52]) ;
   SPEH->o1_temp         = int2 (&ebuf [54]) ;
   SPEH->fw_temp         = int2 (&ebuf [56]) ;
   SPEH->ccd_elec_temp   = int2 (&ebuf [58]) ;
   SPEH->ccd_det_temp    = int2 (&ebuf [60]) ;
   SPEH->wlc_yfp         = int2 (&ebuf [62]) ;
   SPEH->wlc_rfp         = int2 (&ebuf [64]) ;
   SPEH->ted_hot_temp    = int2 (&ebuf [66]) ;
   SPEH->p5v_supply_temp = int2 (&ebuf [68]) ;
   SPEH->elec_pkg2_temp  = int2 (&ebuf [70]) ;
   SPEH->m68000_temp     = int2 (&ebuf [72]) ;
   SPEH->tx_start        = (ebuf [74] & 0040) >> 5 ;
   SPEH->tx_done         = (ebuf [74] & 0100) >> 6 ;
   SPEH->m5v_eom         = (ebuf [74] & 0200) >> 7 ;
   SPEH->acs_byte        = (ebuf [75] & 0077)      ;
   SPEH->wlc_sun_pres    = (ebuf [75] & 0100) >> 6 ;
   SPEH->pes_ok          = (ebuf [75] & 0200) >> 7 ;
   SPEH->fold_mir_pos    = (ebuf [76] & 0001)      ;
   SPEH->heat_mir_pos    = (ebuf [76] & 0002) >> 1 ;
   SPEH->pw_pos          = (ebuf [76] & 0014) >> 2 ;
   SPEH->half_wave_pos   = (ebuf [76] & 0060) >> 4 ;
   SPEH->half_wave_io    = (ebuf [77] & 0001)      ;
   SPEH->fold_mir_io     = (ebuf [77] & 0004) >> 2 ;
   SPEH->heat_mir_io     = (ebuf [77] & 0010) >> 3 ;
   SPEH->pw_io           = (ebuf [77] & 0020) >> 4 ;
   SPEH->stim_lamp_io    = (ebuf [77] & 0040) >> 5 ;
   SPEH->wlc_mode        = (ebuf [77] & 0200) >> 6 ;
   SPEH->reset_counter   = ebuf [78] ;
   SPEH->flt_sw_ver      = ebuf [79] ;
   SPEH->rel_pitch_ang   = int2 (&ebuf [80]) ;
   SPEH->orbit_num       = ebuf [82] ;
   SPEH->image_num       = ebuf [83] ;
   SPEH->acs_pointer     = ebuf [84] ;
   SPEH->half_wave_cmd   = (ebuf [85] & 0001)      ;
   SPEH->fold_mir_cmd    = (ebuf [85] & 0004) >> 2 ;
   SPEH->heat_mir_cmd    = (ebuf [85] & 0010) >> 3 ;
   SPEH->pw_cmd          = (ebuf [85] & 0020) >> 4 ;
   SPEH->stim_lamp_cmd   = (ebuf [85] & 0040) >> 5 ;
   SPEH->msg_flag_sot    = ebuf [86] ;
   SPEH->acs_buf [0]     = ebuf [88] ;
   SPEH->acs_buf [1]     = ebuf [89] ;
   SPEH->acs_buf [2]     = ebuf [90] ;
   SPEH->acs_buf [3]     = ebuf [91] ;
   SPEH->acs_buf [4]     = ebuf [92] ;
   SPEH->acs_buf [5]     = ebuf [93] ;
   SPEH->acs_buf [6]     = ebuf [94] ;
   SPEH->opt_data_id     = ebuf [95] ;
   SPEH->last_sunrise    = int4 (&ebuf [96]) ;
   SPEH->last_sunset     = int4 (&ebuf [100]) ;
   SPEH->time_avail      = int4 (&ebuf [108]) ;
   SPEH->table_num       = ebuf [112] ;
   SPEH->prog_num        = ebuf [113] ;
   SPEH->eom_flag        = ebuf [114] ;
   SPEH->undet_pitch_flag = ebuf [115] ;
   SPEH->testmode_flag   = ebuf [116] ;
   SPEH->uvcs_raster_flag = ebuf [117] ;
   SPEH->wlc_cal_flag    = ebuf [118] ;
   SPEH->wlc_liss_flag   = ebuf [119] ;
   SPEH->base_add_gse    = int4 (&ebuf [120]) ;
   SPEH->ccd_mode        = ebuf [124] ;
   SPEH->ccd_erase       = ebuf [125] ;
   SPEH->ccd_expdur      = int2 (&ebuf [126]) ;

   metconv (SPEH->met, &kyear, &kdoy, &khour, &kmin, &ksec) ;

/*
   fprintf (FP_ELOG, "%02d-%02d:%02d:%06.3f %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
      kdoy, khour, kmin, ksec,
      SPEH->p28v, SPEH->p5v, SPEH->rs232_p12v, SPEH->ccd_ted_volt,
      SPEH->ccd_bat_p6v, SPEH->ccd_p5v, SPEH->ccd_p15v, SPEH->ccd_m15v,
      SPEH->uvcs_yfp, SPEH->uvcs_rfp,
      SPEH->sun_pres, SPEH->fine_mode2, SPEH->pitch_trig_rate,
      SPEH->yaw_fine_pos, SPEH->roll_fine_pos, SPEH->yaw_offset,
      SPEH->roll_offset, SPEH->pes_pitch, SPEH->pes_yaw, SPEH->front_bulk_temp,
      SPEH->o1_temp, SPEH->fw_temp) ;
 */

/*
   fprintf (stdout, "%02d-%02d:%02d:%06.3f %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
      kdoy, khour, kmin, ksec,
      SPEH->p28v, SPEH->p5v, SPEH->rs232_p12v, SPEH->ccd_ted_volt,
      SPEH->ccd_bat_p6v, SPEH->ccd_p5v, SPEH->ccd_p15v, SPEH->ccd_m15v,
      SPEH->uvcs_yfp, SPEH->uvcs_rfp,
      SPEH->sun_pres, SPEH->fine_mode2, SPEH->pitch_trig_rate,
      SPEH->yaw_fine_pos, SPEH->roll_fine_pos, SPEH->yaw_offset,
      SPEH->roll_offset, SPEH->pes_pitch, SPEH->pes_yaw, SPEH->front_bulk_temp,
      SPEH->o1_temp, SPEH->fw_temp) ;
 */

   /*** MET = 0 corresponds to 1997 DOY 325 21:09:10 ***/

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPEH->met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   month_day (iyear, idoy, &imonth, &iday) ;

   /*** Fill Engineering Information structure. ***/
   
   SPEI->year   = iyear ;
   SPEI->doy    = idoy ;
   SPEI->month  = imonth ;
   SPEI->day    = iday   ;
   SPEI->doy    = idoy ;
   SPEI->hour   = ihour ;
   SPEI->minute = iminute ;
   SPEI->second = isecond ;

   SPEI->p28v         = ((SPEH->p28v         * 500000.0) / 409500.0) ;
   SPEI->p5v          = ((SPEH->p5v          * 100000.0) / 409500.0) ;
   SPEI->rs232_p12v   = ((SPEH->rs232_p12v   * 246300.0) / 409500.0) ;
   SPEI->ccd_ted_volt = ((SPEH->ccd_ted_volt * 100000.0) / 409500.0) ;
   SPEI->ccd_bat_p6v  = ((SPEH->ccd_bat_p6v  * 100000.0) / 409500.0) ;
   SPEI->ccd_p5v      = ((SPEH->ccd_p5v      * 100000.0) / 409500.0) ;
   SPEI->ccd_p15v     = ((SPEH->ccd_p15v     * 246900.0) / 409500.0) ;
   SPEI->ccd_m15v     = ((SPEH->ccd_m15v     * 100000.0) / 409500.0) ;

   SPEI->uvcs_yfp         = (SPEH->uvcs_yfp  * 0.251) - 289.2 ;
   SPEI->uvcs_rfp         = (SPEH->uvcs_rfp  * 0.251) - 289.2 ;

   SPEI->p28v         = (SPEI->p28v         +   0.0) / 100.0 ;
   SPEI->p5v          = (SPEI->p5v          +   0.0) / 100.0 ;
   SPEI->rs232_p12v   = (SPEI->rs232_p12v   +   0.0) / 100.0 ;
   SPEI->ccd_ted_volt = (SPEI->ccd_ted_volt +   0.0) / 100.0 ;
   SPEI->ccd_bat_p6v  = (SPEI->ccd_bat_p6v  +   0.0) / 100.0 ;
   SPEI->ccd_p5v      = (SPEI->ccd_p5v      +   0.0) / 100.0 ;
   SPEI->ccd_p15v     = (SPEI->ccd_p15v     +   0.0) / 100.0 ;
   SPEI->ccd_m15v     = (SPEI->ccd_m15v     + 900.0) / 100.0 ;

   if (SPEH->sun_pres > 1024)
      strcpy (SPEI->sun_pres, "Y") ;
   else
      strcpy (SPEI->sun_pres, "N") ;

   if (SPEH->fine_mode2 > 1024)
      {
      strcpy (SPEI->fine_mode2, "ON") ;
      SPEI->pitch_trig_rate = ((SPEH->pitch_trig_rate * -0.0296) + 29.89) ;
      }
   else
      {
      strcpy (SPEI->fine_mode2, "OFF") ;
      SPEI->pitch_trig_rate = ((SPEH->pitch_trig_rate * -0.0049) + 4.917) ;
      }

   SPEI->yaw_fine_pos    = (SPEH->yaw_fine_pos  * 0.251)  - 289.2 ;
   SPEI->roll_fine_pos   = (SPEH->roll_fine_pos * 0.251)  - 289.2 ;
   SPEI->yaw_offset      = (SPEH->yaw_offset    * 0.0882) - 180.00 ;
   SPEI->roll_offset     = (SPEH->roll_offset   * 0.0882) - 180.00 ;
   SPEI->pes_pitch       = ((SPEH->pes_pitch     * 5.0) / 4095.0) ;
   SPEI->pes_yaw         = ((SPEH->pes_yaw       * 5.0) / 4095.0) ;
   SPEI->front_bulk_temp = (((SPEH->front_bulk_temp * 38460.0)/4096.0)-14248.0)*0.01 ;
   SPEI->o1_temp         = (((SPEH->o1_temp*38460.0)/4096.0)-14248.0)*0.01 ;
   SPEI->fw_temp         = (((SPEH->fw_temp*38460.0)/4096.0)-14248.0)*0.01 ;
   SPEI->ccd_elec_temp   = (((SPEH->ccd_elec_temp*24972.0)/4096.0)-9478.0)*0.01;
   SPEI->ccd_det_temp    = (((SPEH->ccd_det_temp*36680.0)/4096.0)-13570.0)*0.01;

   SPEI->wlc_yfp         = (SPEH->wlc_yfp  * 0.251) - 289.2 ;
   SPEI->wlc_rfp         = (SPEH->wlc_rfp  * 0.251) - 289.2 ;

   SPEI->ted_hot_temp    = (((SPEH->ted_hot_temp*38413.0)/4096.0)-13904.0)*0.01;
   SPEI->p5v_supply_temp = (((SPEH->p5v_supply_temp*30831.0)/4096.0)-11271.0)*0.01;
   SPEI->elec_pkg2_temp  = (((SPEH->elec_pkg2_temp*24730.0)/4096.0)-8722.0)*0.01;
   SPEI->m68000_temp     = (((SPEH->m68000_temp*22724.0)/4096.0)-7692.0)*0.01 ;
   SPEI->tx_start        = SPEH->tx_start ;
   SPEI->tx_done         = SPEH->tx_done ;

   if (SPEH->m5v_eom == 0)
      strcpy (SPEI->m5v_eom, "RECEIVED");
   else
      strcpy (SPEI->m5v_eom, "NO");

   SPEI->acs_byte        = SPEH->acs_byte ;

   if (SPEH->wlc_sun_pres == 0)
      strcpy (SPEI->wlc_sun_pres, "Y") ;
   else
      strcpy (SPEI->wlc_sun_pres, "N") ;

   if (SPEH->pes_ok == 1)
      strcpy (SPEI->pes_ok, "Y") ;
   else
      strcpy (SPEI->pes_ok, "N") ;

   if (SPEH->fold_mir_pos == 1)
      strcpy (SPEI->fold_mir_pos, "CAL") ;
   else
      strcpy (SPEI->fold_mir_pos, "COR") ;

   if (SPEH->heat_mir_pos == 1)
      strcpy (SPEI->heat_mir_pos, "CAL") ;
   else
      strcpy (SPEI->heat_mir_pos, "COR") ;

   if (SPEH->pw_pos == 0)
      strcpy (SPEI->pw_pos, "CLEAR") ;
   else
   if (SPEH->pw_pos == 1)
      strcpy (SPEI->pw_pos, " 0 DEG") ;
   else
   if (SPEH->pw_pos == 2)
      strcpy (SPEI->pw_pos, "30 DEG") ;
   else
   if (SPEH->pw_pos == 3)
      strcpy (SPEI->pw_pos, "60 DEG") ;

   if (SPEH->half_wave_pos == 0)
      strcpy (SPEI->half_wave_pos, "90 DEG") ;
   else
   if (SPEH->half_wave_pos == 1)
      strcpy (SPEI->half_wave_pos, "60 DEG") ;
   else
   if (SPEH->half_wave_pos == 2)
      strcpy (SPEI->half_wave_pos, "30 DEG") ;
   else
   if (SPEH->half_wave_pos == 3)
      strcpy (SPEI->half_wave_pos, " 0 DEG") ;

   if (SPEH->half_wave_io == 1)
      strcpy (SPEI->half_wave_io, "IN") ;
   else
      strcpy (SPEI->half_wave_io, "OUT") ;

   if (SPEH->fold_mir_io == 1)
      strcpy (SPEI->fold_mir_io, "CAL") ;
   else
      strcpy (SPEI->fold_mir_io, "COR") ;

   if (SPEH->heat_mir_io == 1)
      strcpy (SPEI->heat_mir_io, "COR") ;
   else
      strcpy (SPEI->heat_mir_io, "CAL") ;

   if (SPEH->pw_io == 1)
      strcpy (SPEI->pw_io, "IN") ;
   else
      strcpy (SPEI->pw_io, "OUT") ;

   if (SPEH->stim_lamp_io == 1)
      strcpy (SPEI->stim_lamp_io, "ON") ;
   else
      strcpy (SPEI->stim_lamp_io, "OFF") ;

   SPEI->wlc_mode        = SPEH->wlc_mode ;
   SPEI->reset_counter   = SPEH->reset_counter ;
   SPEI->flt_sw_ver      = (float) SPEH->flt_sw_ver / 10.0 ;
   SPEI->rel_pitch_ang   = (float) SPEH->rel_pitch_ang * 0.5 ;
   SPEI->orbit_num       = SPEH->orbit_num ;
   SPEI->image_num       = SPEH->image_num ;
   SPEI->acs_pointer     = SPEH->acs_pointer ;

   if (SPEH->half_wave_cmd  == 1)
      strcpy (SPEI->half_wave_cmd, "GO") ;
   else
      strcpy (SPEI->half_wave_cmd, "STOP") ;

   if (SPEH->fold_mir_cmd  == 1)
      strcpy (SPEI->fold_mir_cmd, "GO") ;
   else
      strcpy (SPEI->fold_mir_cmd, "STOP") ;

   if (SPEH->heat_mir_cmd  == 1)
      strcpy (SPEI->heat_mir_cmd, "GO") ;
   else
      strcpy (SPEI->heat_mir_cmd, "STOP") ;

   if (SPEH->pw_cmd  == 1)
      strcpy (SPEI->pw_cmd, "GO") ;
   else
      strcpy (SPEI->pw_cmd, "STOP") ;

   if (SPEH->stim_lamp_cmd  == 1)
      strcpy (SPEI->stim_lamp_cmd, "ON") ;
   else
      strcpy (SPEI->stim_lamp_cmd, "OFF") ;

   if (SPEH->msg_flag_sot == 0)
      strcpy (SPEI->msg_flag_sot, "NO MSG") ;
   else
      strcpy (SPEI->msg_flag_sot, "MSG WAITING") ;

   for (i = 0; i < 7; i++)  SPEI->acs_buf [i] =  SPEH->acs_buf [i] ;
   SPEI->opt_data_id     = SPEH->opt_data_id ;
   SPEI->last_sunrise    = SPEH->last_sunrise * 0.008 ;
   SPEI->last_sunset     = SPEH->last_sunset  * 0.008 ;
   SPEI->time_avail      = SPEH->time_avail   * 0.008 ;
   SPEI->table_num       = SPEH->table_num ;
   SPEI->prog_num        = SPEH->prog_num ;

   if (SPEH->eom_flag >> 0)
      strcpy (SPEI->eom_flag, "Y") ;
   else
      strcpy (SPEI->eom_flag, "N") ;

   SPEI->undet_pitch_flag = SPEH->undet_pitch_flag ;
   SPEI->testmode_flag   = SPEH->testmode_flag ;

   if (SPEH->uvcs_raster_flag >> 0)
      strcpy (SPEI->uvcs_raster_flag, "Y") ;
   else
      strcpy (SPEI->uvcs_raster_flag, "N") ;

   if (SPEH->wlc_cal_flag >> 0)
      strcpy (SPEI->wlc_cal_flag, "Y") ;
   else
      strcpy (SPEI->wlc_cal_flag, "N") ;

   if (SPEH->wlc_liss_flag >> 0)
      strcpy (SPEI->wlc_liss_flag, "Y") ;
   else
      strcpy (SPEI->wlc_liss_flag, "N") ;

   SPEI->base_add_gse    = SPEH->base_add_gse ;
   if (SPEH->ccd_mode == 0)
      strcpy (SPEI->ccd_mode, "INACTIVE") ;
   else
   if (SPEH->ccd_mode == 255)
      strcpy (SPEI->ccd_mode, "EXPOSE") ;
   else
      strcpy (SPEI->ccd_mode, "WAIT") ;

   SPEI->ccd_erase       = SPEH->ccd_erase ;

   SPEI->ccd_expdur      = SPEH->ccd_expdur * 0.008 ;

/*
   fprintf (FP_ELOG, "%4d-%8d %10d %02d-%02d:%02d:%06.3f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.7s %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %9.2f %7.2f %7.2f %7.2f\n",
      idf, ifloc, SPEH->met, idoy, ihour, iminute, isecond,
      SPEI->p28v, SPEI->p5v, SPEI->rs232_p12v, SPEI->ccd_ted_volt,
      SPEI->ccd_bat_p6v, SPEI->ccd_p5v, SPEI->ccd_p15v, SPEI->ccd_m15v,
      SPEI->sun_pres, SPEI->fine_mode2, SPEI->pitch_trig_rate,
      SPEI->yaw_fine_pos, SPEI->roll_fine_pos, SPEI->yaw_offset,
      SPEI->roll_offset, SPEI->pes_pitch, SPEI->pes_yaw, SPEI->front_bulk_temp,
      SPEI->o1_temp, SPEI->fw_temp) ;
 */
   fprintf (FP_ELOG, "%4d-%8d %10d %02d-%02d:%02d:%06.3f",
      idf, ifloc, SPEH->met, idoy, ihour, iminute, isecond) ;

   fprintf (FP_ELOG, " %7.2f %7.2f %7.2f %7.2f",
      SPEI->p28v, SPEI->p5v, SPEI->rs232_p12v, SPEI->ccd_ted_volt) ;
   fprintf (FP_ELOG, " %7.2f %7.2f %7.2f %7.2f",
      SPEI->ccd_bat_p6v, SPEI->ccd_p5v, SPEI->ccd_p15v, SPEI->ccd_m15v) ;
   fprintf (FP_ELOG, " %7.7s %7.2f %7.2f",
      SPEI->sun_pres, SPEI->fine_mode2, SPEI->pitch_trig_rate) ;
   fprintf (FP_ELOG, " %7.2f %7.2f %7.2f",
      SPEI->yaw_fine_pos, SPEI->roll_fine_pos, SPEI->yaw_offset) ;
   fprintf (FP_ELOG, " %7.2f %7.2f %9.2f %7.2f",
      SPEI->roll_offset, SPEI->pes_pitch, SPEI->pes_yaw, SPEI->front_bulk_temp);
   fprintf (FP_ELOG, " %7.2f %7.2f\n",
      SPEI->o1_temp, SPEI->fw_temp) ;


   return (0) ;
}
