/* engfmt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-04 Engineering Data Formatter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 Sep 1996
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"

engfmt (pmode)

int pmode ;				/* Print mode. */

{
   extern int engmet_prev ;
   extern int ifd ;			/* ifd: current block (record) index. */
   extern int ib ;			/* ib: current byte index. */
   extern int blkcnt ;				/* # data blocks in tape/file.*/
   extern unsigned char  hkbuf  [NBYHK] ;	/* Housekeeping buffer. */
   extern unsigned short imgbuf [NY][NX] ;	/* Image 2D buffer. */
   extern unsigned char  blkbuf [BLKSIZ] ;	/* Data Block buffer. */
   extern unsigned char *blkhdr ;		/* Block header data. */
   extern unsigned char *dbuf ;			/* Data. */
   extern int dblknum  ;			/* Data block number. */
   extern int derrcnt ;				/* Data block error count. */
   extern long ifloc ;			/* Byte position within telem. file. */
   extern int npxlin ;			/* # pixels per line. */
   extern int nbyeng;			/* # bytes per eng frame = 128 */
   extern int nbypix ;			/* # bytes per pixel. */
   extern int npximg ;			/* # pixels per image  = 256 * 320. */
   extern FILE *FP_RAW, *FP_IMG, *FP_DLOG, *FP_ILOG, *FP_ELOG, *FP_MLOG ;

   static unsigned char ebuf [NBYEDF] ;	/* Engineering telemetry buffer. */

   int i ;
   int is ;
   int engmet_new = 0 ;
   int nbyend = 0 ;		/* # eng bytes remaining at end of block. */
   int nbybeg = 0 ;		/* # eng bytes remaining at start of block.*/
   int nelrd ;

   nelrd = fread (ebuf, 1, NBYEDF, FP_RAW) ;
   ifloc += nelrd ;

   /*** Decode MET. ***/

   engmet_new      = int4 (&ebuf [6]) ;
/*
 *   printf ("ebuf [6-9]: %2x %2x %2x %2x\n",
 *	   ebuf [6], ebuf [7], ebuf [8], ebuf [9]) ;
 */

   /*** Decode Engineering Housekeeping. ***/

   spetlm (ebuf, pmode) ;
   fprintf (FP_DLOG, "engfmt> met:%10d  orbit:%4d   image:%4d\n",
		     SPEH->met, SPEH->orbit_num, SPEH->image_num) ;

   /*** Write Engineering data to disk if MET has changed. ***/

   if (engmet_new != engmet_prev) /* New Eng data, since MET != previous MET. */
      {

      /*** Write Engineering Data to disk. ***/

      if (engmet_prev != 0)
         neweng (ebuf, pmode) ;

      bzero  (ebuf, nbyeng) ;			  /* Clear image buffer. */
      engmet_prev = engmet_new ;
      }

   if (is < 0)
      {
      neweng (ebuf, pmode) ;
      exit (0) ;
      }
   else
      {
/*      ifloc = is ; */
      return (0) ;
      }
}
