/* imgfmt.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-04 Image Data Formatter.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 Sep 1996
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * 13 Nov 1998: Write image to disk if either the orb or img # has changed.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"

imgfmt (pmode)

int pmode ;				/* Print mode. */
{
   extern int imgmet_prev ;
   extern int orb_prev ;
   extern int img_prev ;
   extern int idf ;			/* idf: current data frame index. */
   extern int ib ;			/* ib: current byte index. */
   extern int blkcnt ;				/* # data blocks in tape/file.*/
   extern unsigned char  hkbuf  [NBYHK] ;	/* Housekeeping buffer. */
   extern unsigned short imgbuf [NY][NX] ;	/* Image 2D buffer. */
   extern unsigned char  blkbuf [BLKSIZ] ;	/* Data Block buffer. */
   extern unsigned char *blkhdr ;		/* Block header data. */
   extern unsigned char *dbuf ;			/* Data. */
   extern int dblknum  ;			/* Data block number. */
   extern int derrcnt ;				/* Data block error count. */
   extern long ifloc ;                  /* Byte position within telem. file. */
   extern int npxlin ;			/* # pixels per line. */
   extern int nbyimg ;			/* # bytes per image = 256 * 320 * 2. */
   extern int nbypix ;			/* # bytes per pixel. */
   extern int npximg ;			/* # pixels per image  = 256 * 320. */
   extern FILE *FP_RAW, *FP_DLOG, *FP_ILOG, *FP_ITLM ;

   static unsigned char ibuf [NBYIDF] ;		/* Image line buffer. */
   static unsigned char *hdbuf ;		/* Image Housekeeping data. */
   static unsigned char *pxbuf ;		/* Image Pixel data. */
   unsigned char *cbuf ;

   int hduflag = 1 ;
   int i ;
   int ierr ;
   int is ;
   int met_new = 0 ;
   int orb_new = 0 ;
   int img_new = 0 ;
   int new_ccd_line = 0 ;
   int nimend = 0 ;		/* # img bytes remaining at end of block. */
   int nimbeg = 0 ;		/* # img bytes remaining at start of block.*/
   int nelrd ;

   /*** Read an image data frame. ***/

   nelrd = fread (ibuf, 1, NBYIDF, FP_RAW) ;
   if (nelrd != NBYIDF)
      {
      fprintf (stderr, "imgfmt> Read Error . . . nelrd: %d != NBYIDF: %d\n",
		       nelrd, NBYIDF) ;
      return (-1) ;
      }
   if (ierr = ferror (FP_RAW))
      {
      fprintf (stderr, "imgfmt> ferror: %d\n", ierr) ;
      return (-2) ;
      }
   if (ierr = feof (FP_RAW))
      {
      fprintf (stderr, "imgfmt> feof: %d\n", ierr) ;
      return (-3) ;
      }

   ifloc += nelrd ;

   hdbuf = ibuf ;
   pxbuf = ibuf + 46 ;
   is = 687 ;

   /*** Decode MET & CCD line number. ***/

   met_new      = int4 (&hdbuf [6]) ;
   orb_new      = hdbuf [16] ;
   img_new      = hdbuf [17] ;
   new_ccd_line = int2 (&hdbuf [44]) ;

   fprintf (FP_DLOG, "imgfmt> idf: %4d  ifloc: %4d  met_new: %d\n",
            idf, ifloc, met_new) ;
   fflush (FP_DLOG) ;

   /*** Write image to disk if either the orb or img # has changed. ***/

/*   if (orb_new != orb_prev || img_new != img_prev) */

  if (met_new != imgmet_prev)
      {
      bzero  (imgbuf, nbyimg) ;			  /* Clear image buffer. */
      imgmet_prev = met_new ;

      for (i = 0; i < NBYHK; i++)  hkbuf [i] = hdbuf [i] ;
      }

   if (new_ccd_line >= NY)
      {
      fprintf (FP_DLOG, "* * * Invalid CCD line number (%4d)\n",
			new_ccd_line) ;
      return (-1) ;
      }

   /*** Decode Image Housekeeping. ***/

   sphskp (hdbuf, pmode) ;
   fprintf (FP_DLOG, "imgfmt> met:%10d  orbit:%4d   image:%4d  line:%4d\n",
		     SPIH->met, SPIH->orbit_num, SPIH->image_num,
		     SPIH->cur_ccd_line) ;

   /*** Fill image 2D buffer. ***/
   /* CCD image origin is at the upper left corner. */
   /* Flip image top to bottom so that the origin is the lower left corner. */

   int2arr (pxbuf, &imgbuf [SPII->ydim - SPIH->cur_ccd_line - 1][0], npxlin) ;

   if (is < 0)
      {
      fprintf (FP_DLOG, "imgfmt> call newimg * * * * * * *\n") ;
      newimg (pmode) ;
      exit (0) ;
      }

   /* Write Image telemetry to disk. */

/*
 * Move statements below to "newimg.c", since the image header only
 * needs to be written once per image, not once per image line.
 *
 *   else
 *      {
 *      if (fwrite (ibuf, 1, NBYHK, FP_ITLM) == 0)
 *         {
 *         fprintf (stderr, "imgfmt> write error... idf: %d  ifloc: %d\n",
 *		  idf, ifloc) ;
 *         return (-2) ;
 *         }
 *
 *      return (0) ;
 *      }
 */

   return (0) ;
}
