/* mkimgfiles.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read Spartan 201-4 Image Telemetry structure from a disk file.
 * Write individual Spartan 201-4 Image Telemetry items to separate files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   25 April 1997
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * 13 Oct 1998: Changer "I97_*.dat" files to "I98_*.dat".
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"
unsigned char ibuf [NBYHK] ;		/* Image Telemetry Buffer. */

main (argc, argv)

int argc ;
char *argv [] ;

{
   FILE *FP_ITLM ;
   FILE *FP_MET ;
   FILE *FP_RELP ;
   FILE *FP_PIA1AR10 ;
   FILE *FP_MPII ;
   FILE *FP_CCDEXP ;
   FILE *FP_ORB ;
   FILE *FP_IMG ;
   FILE *FP_CCDT0 ;
   FILE *FP_CCDTEND ;
   FILE *FP_PYMEAN ;
   FILE *FP_PYMIN ;
   FILE *FP_PYMAX ;
   FILE *FP_PPMEAN ;
   FILE *FP_PPMIN ;
   FILE *FP_PPMAX ;

   static char file_itlm [80] ;
   static char file_met       [] = "I98_MET.dat" ;
   static char file_relp      [] = "I98_RELP.dat" ;
   static char file_pia1ar10  [] = "I98_PIA1AR10.dat" ;
   static char file_mpii      [] = "I98_MPII.dat" ;
   static char file_ccdexp    [] = "I98_CCDEXP.dat" ;
   static char file_orb       [] = "I98_ORB.dat" ;
   static char file_img       [] = "I98_IMG.dat" ;
   static char file_ccdt0     [] = "I98_CCDT0.dat" ;
   static char file_ccdtend   [] = "I98_CCDTEND.dat" ;
   static char file_pymean    [] = "I98_PYMEAN.dat" ;
   static char file_pymin     [] = "I98_PYMIN.dat" ;
   static char file_pymax     [] = "I98_PYMAX.dat" ;
   static char file_ppmean    [] = "I98_PPMEAN.dat" ;
   static char file_ppmin     [] = "I98_PPMIN.dat" ;
   static char file_ppmax     [] = "I98_PPMAX.dat" ;
   int irec = 0 ;
   int size_SPIH ;

   size_SPIH = sizeof (SPIH) ;

   if (argc == 2)
      strcpy (file_itlm, argv [1]) ;
   else
      {
      fprintf (stderr, "usage: %s img_telem_name\n", argv [0]) ;
      exit (0) ;
      }

   if (( FP_ITLM = fopen (file_itlm, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_itlm) ;
      exit (-1) ;
      }

   if (( FP_MET = fopen (file_met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_met) ;
      exit (-1) ;
      }

   if (( FP_RELP = fopen (file_relp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_relp) ;
      exit (-1) ;
      }

   if (( FP_PIA1AR10 = fopen (file_pia1ar10, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pia1ar10) ;
      exit (-1) ;
      }

   if (( FP_MPII = fopen (file_mpii, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_mpii) ;
      exit (-1) ;
      }

   if (( FP_CCDEXP = fopen (file_ccdexp, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccdexp) ;
      exit (-1) ;
      }

   if (( FP_ORB = fopen (file_orb, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_orb) ;
      exit (-1) ;
      }

   if (( FP_IMG = fopen (file_img, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_img) ;
      exit (-1) ;
      }

   if (( FP_CCDT0 = fopen (file_ccdt0, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccdt0) ;
      exit (-1) ;
      }

   if (( FP_CCDTEND = fopen (file_ccdtend, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ccdtend) ;
      exit (-1) ;
      }

   if (( FP_PYMEAN = fopen (file_pymean, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pymean) ;
      exit (-1) ;
      }

   if (( FP_PYMIN = fopen (file_pymin, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pymin) ;
      exit (-1) ;
      }

   if (( FP_PYMAX = fopen (file_pymax, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_pymax) ;
      exit (-1) ;
      }

   if (( FP_PPMEAN = fopen (file_ppmean, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ppmean) ;
      exit (-1) ;
      }

   if (( FP_PPMIN = fopen (file_ppmin, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ppmin) ;
      exit (-1) ;
      }

   if (( FP_PPMAX = fopen (file_ppmax, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_ppmax) ;
      exit (-1) ;
      }

   /*** Read Image Telemetry to a disk file. ***/

   while (fread (ibuf, 46, 1, FP_ITLM) == 1)
      {
      fwrite (&ibuf [  6], 4, 1, FP_MET) ;

      fwrite (&ibuf [ 10], 2, 1, FP_RELP) ;
      fwrite (&ibuf [ 12], 1, 1, FP_PIA1AR10) ;
      fwrite (&ibuf [ 13], 1, 1, FP_MPII) ;

      fwrite (&ibuf [ 14], 2, 1, FP_CCDEXP) ;
      fwrite (&ibuf [ 16], 1, 1, FP_ORB) ;
      fwrite (&ibuf [ 17], 1, 1, FP_IMG) ;

      fwrite (&ibuf [ 18], 2, 1, FP_CCDT0) ;
      fwrite (&ibuf [ 20], 2, 1, FP_CCDTEND) ;

      fwrite (&ibuf [ 22], 2, 1, FP_PYMEAN) ;
      fwrite (&ibuf [ 24], 2, 1, FP_PYMIN) ;
      fwrite (&ibuf [ 26], 2, 1, FP_PYMAX) ;

      fwrite (&ibuf [ 28], 2, 1, FP_PPMEAN) ;
      fwrite (&ibuf [ 30], 2, 1, FP_PPMIN) ;
      fwrite (&ibuf [ 32], 2, 1, FP_PPMAX) ;
      }

   fclose (FP_ITLM) ;
   fclose (FP_MET) ;
   fclose (FP_RELP) ;
   fclose (FP_PIA1AR10) ;
   fclose (FP_MPII) ;
   fclose (FP_CCDEXP) ;
   fclose (FP_ORB) ;
   fclose (FP_IMG) ;
   fclose (FP_CCDT0) ;
   fclose (FP_CCDTEND) ;
   fclose (FP_PYMEAN) ;
   fclose (FP_PYMIN) ;
   fclose (FP_PYMAX) ;
   fclose (FP_PPMEAN) ;
   fclose (FP_PPMIN) ;
   fclose (FP_PPMAX) ;

   return (0) ;
}
