/* mkmsgfiles.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read Spartan 201-4 Message Telemetry structure from a disk file.
 * Write individual Spartan 201-4 Message Telemetry items to separate files.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   25 April 1997
 * 23 June 1997: Fixed message indices.
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * 13 Oct 1998: Changer "I97_*.dat" files to "I98_*.dat".
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"
unsigned char mbuf [NBYMDF] ;		/* Message Telemetry Buffer. */

main (argc, argv)

int argc ;
char *argv [] ;

{
   FILE *FP_MTLM ;
   FILE *FP_MSG1 ;
   FILE *FP_MSG1MET ;
   FILE *FP_MSG2 ;
   FILE *FP_MSG2MET ;
   FILE *FP_MSG3 ;
   FILE *FP_MSG3MET ;
   FILE *FP_MSG4 ;
   FILE *FP_MSG4MET ;
   FILE *FP_MSG5 ;
   FILE *FP_MSG5MET ;
   FILE *FP_MSGPTR ;

   static char file_mtlm [80] ;

   static char file_msg1      [] = "M98_MSG1.dat" ;
   static char file_msg1met   [] = "M98_MSG1MET.dat" ;
   static char file_msg2      [] = "M98_MSG2.dat" ;
   static char file_msg2met   [] = "M98_MSG2MET.dat" ;
   static char file_msg3      [] = "M98_MSG3.dat" ;
   static char file_msg3met   [] = "M98_MSG3MET.dat" ;
   static char file_msg4      [] = "M98_MSG4.dat" ;
   static char file_msg4met   [] = "M98_MSG4MET.dat" ;
   static char file_msg5      [] = "M98_MSG5.dat" ;
   static char file_msg5met   [] = "M98_MSG5MET.dat" ;
   static char file_msgptr    [] = "M98_MSGPTR.dat" ;
   int irec = 0 ;
   int size_SPMH ;

   size_SPMH = sizeof (SPMH) ;

   if (argc == 2)
      strcpy (file_mtlm, argv [1]) ;
   else
      {
      fprintf (stderr, "usage: %s msg_telem_name\n", argv [0]) ;
      exit (0) ;
      }

   if (( FP_MTLM = fopen (file_mtlm, "r")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_mtlm) ;
      exit (-1) ;
      }

   if (( FP_MSG1 = fopen (file_msg1, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg1) ;
      exit (-1) ;
      }

   if (( FP_MSG1MET = fopen (file_msg1met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg1met) ;
      exit (-1) ;
      }

   if (( FP_MSG2 = fopen (file_msg2, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg2) ;
      exit (-1) ;
      }

   if (( FP_MSG2MET = fopen (file_msg2met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg2met) ;
      exit (-1) ;
      }

   if (( FP_MSG3 = fopen (file_msg3, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg3) ;
      exit (-1) ;
      }

   if (( FP_MSG3MET = fopen (file_msg3met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg3met) ;
      exit (-1) ;
      }

   if (( FP_MSG4 = fopen (file_msg4, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg4) ;
      exit (-1) ;
      }

   if (( FP_MSG4MET = fopen (file_msg4met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg4met) ;
      exit (-1) ;
      }

   if (( FP_MSG5 = fopen (file_msg5, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg5) ;
      exit (-1) ;
      }

   if (( FP_MSG5MET = fopen (file_msg5met, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msg5met) ;
      exit (-1) ;
      }

   if (( FP_MSGPTR = fopen (file_msgptr, "a")) == NULL)
      {
      fprintf (stderr, "Unable to open file: %s\n", file_msgptr) ;
      exit (-1) ;
      }

   /*** Read Image Telemetry to a disk file. ***/

   while (fread (mbuf, NBYMDF, 1, FP_MTLM) == 1)
      {
      fwrite (&mbuf [  6], 2, 1, FP_MSG1) ;
      fwrite (&mbuf [  8], 4, 1, FP_MSG1MET) ;

      fwrite (&mbuf [ 12], 2, 1, FP_MSG2) ;
      fwrite (&mbuf [ 14], 4, 1, FP_MSG2MET) ;

      fwrite (&mbuf [ 18], 2, 1, FP_MSG3) ;
      fwrite (&mbuf [ 20], 4, 1, FP_MSG3MET) ;

      fwrite (&mbuf [ 24], 2, 1, FP_MSG4) ;
      fwrite (&mbuf [ 26], 4, 1, FP_MSG4MET) ;

      fwrite (&mbuf [ 30], 2, 1, FP_MSG5) ;
      fwrite (&mbuf [ 32], 4, 1, FP_MSG5MET) ;

      fwrite (&mbuf [ 36], 2, 1, FP_MSGPTR) ;
      }

   fclose (FP_MTLM) ;
   fclose (FP_MSG1) ;
   fclose (FP_MSG1MET) ;
   fclose (FP_MSG2) ;
   fclose (FP_MSG2MET) ;
   fclose (FP_MSG3) ;
   fclose (FP_MSG3MET) ;
   fclose (FP_MSG4) ;
   fclose (FP_MSG4MET) ;
   fclose (FP_MSG5) ;
   fclose (FP_MSG5MET) ;
   fclose (FP_MSGPTR) ;

   return (0) ;
}
