/* new_frame.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read new data frame.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 May 1998
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"

new_frame (pmode)

int pmode ;			/* Print mode. */

{
   extern int imgmet_prev ;	/* MET for previous image data frame. */ 
   extern int orb_prev ;	/* Orbit number for previous image data frame.*/
   extern int img_prev ;	/* Image number for previous image data frame.*/
   extern int	idf ;		/* idf: current data frame index. */
   extern long	ifloc ;		/* ifloc: current byte index. */
   extern char	dfname [] ;	/* Data frame name. */
   extern int dftype ;		/* Data Frame type: 0=eng, 4=img, 8=msg. */
   extern int dfprev ;		/* Data Frame type (previous). */
   extern FILE *FP_RAW, *FP_DLOG ;

   static unsigned char tbuf [6] ;
   int ib ;
   int met_new = 0 ;
   int orb_new = 0 ;
   int img_new = 0 ;
   size_t nelrd ;

   if (pmode)
      fprintf (stdout, "new_frame> idf: %4d  ifloc: %d\n", idf, ifloc) ;

   ib = 0 ;
   strcpy (dfname, "") ;

   /*** Read 48 bytes of data from telemetry file. ***/

   nelrd = fread (tbuf, 1, 48, FP_RAW) ;

   if (nelrd != 48)
      return (-1) ;
   if (nelrd == EOF)
      return (-2) ;

   fseek (FP_RAW, -48L, SEEK_CUR) ;

   /*** Look for sync pattern. ***/

   /*** Engineering Data. ***/

   if (tbuf [ib]   == 0x33 && tbuf [ib+1] == 0x2a && tbuf [ib+2] == 0x66
   &&  tbuf [ib+3] == 0x55 && tbuf [ib+4] == 0x00 && tbuf [ib+5] == 0x00)
      {
      idf++ ;
      dftype = tbuf [ib+5] ;
      if (dfprev == 4)
	 {
         fprintf (FP_DLOG, "new_frame> ENG newimg * * * * * * *\n") ;
	 newimg (pmode) ;
	 }
      strcat (dfname, "Engineering") ;
/*
 *      fprintf (FP_DLOG, "ENG ... idf: %4d  ifloc: %4d\n", idf, ifloc) ;
 */
      engfmt (pmode) ;
      dfprev = dftype ;
      return (0) ;
      }

   /*** Image Data. ***/

   if (tbuf [ib]   == 0x33 && tbuf [ib+1] == 0x2a && tbuf [ib+2] == 0x66
   &&  tbuf [ib+3] == 0x55 && tbuf [ib+4] == 0x00 && tbuf [ib+5] == 0x04)
      {
      idf++ ;
      dftype = tbuf [ib+5] ;		/* Data Frame type. */
      if (dfprev == 4)			/* Previous Data Frame was image type.*/
	 {
	 met_new = int4 (&tbuf [6]) ;
	 orb_new = tbuf [16] ;		/* Orbit number. */
         img_new = tbuf [17] ;		/* Image number. */

         /* Write image if MET has changed. */

         if (imgmet_prev != met_new)  newimg (pmode) ;

         /* Write image if orbit or image number have changed. */

/*
         if (orb_new != orb_prev || img_new != img_prev)
	    {
            fprintf (FP_DLOG, "new_frame> IMG newimg * * * * * * *\n") ;
	    newimg (pmode) ;
	    }
 */
	 }
      strcat (dfname, "Image") ;
/*
 *      fprintf (FP_DLOG, "IMG ... idf: %4d  ifloc: %4d\n", idf, ifloc) ;
 */
      dfprev = dftype ;
      orb_prev = orb_new ;
      img_prev = img_new ;
      imgfmt (pmode) ;
      return (4) ;
      }

   /*** Message Data. ***/

   if (tbuf [ib]   == 0x33 && tbuf [ib+1] == 0x2a && tbuf [ib+2] == 0x66
   &&  tbuf [ib+3] == 0x55 && tbuf [ib+4] == 0x00 && tbuf [ib+5] == 0x08)
      {
      idf++ ;
      dftype = tbuf [ib+5] ;
      if (dfprev == 4)
	 {
         fprintf (FP_DLOG, "new_frame> MSG newimg * * * * * * *\n") ;
	 newimg (pmode) ;
	 }
      strcat (dfname, "Message") ;
/*
 *      fprintf (FP_DLOG, "MSG ... idf: %4d  ifloc: %4d\n", idf, ifloc) ;
 */
      msgfmt (pmode) ;
      dfprev = dftype ;
      return (8) ;
      }

   fprintf (FP_DLOG, "new_frame> Invalid sync pattern ... idf: %d  ifloc: %d\n",
	    idf, ifloc) ;
   fflush (FP_DLOG) ;

   fprintf (stderr, "new_frame> Invalid sync pattern ... idf: %d  ifloc: %d\n",
	    idf, ifloc) ;
   fflush (stderr) ;

   /*** Find next sync pattern. ***/

   dfprev = dftype ;
   dftype = -1 ;

   find_sync (pmode) ;

   /*** Return error code if a valid sync pattern is NOT found. ***/

   if (dfname == NULL)
      return (-2) ;
   else
      return (0) ;
}
