/* newimg.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write Spartan image to disk.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   20 September 1996
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 *  1 Dec 1998: Add ncp_ang = ocp_ang + rcp_ang + wlc_ang, solar_p0, nhp_ang.
 *  3 Dec 1998: Add ILOG2 output (comma-delimited image catalog file).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"

newimg (pmode)

int pmode ;				/* Print mode. */

{
   extern int rel_pitch_prev ;		/* Previous relative pitch angle. */
   extern unsigned char  hkbuf  [NBYHK] ;	/* Housekeeping buffer. */
   extern unsigned short imgbuf [NY][NX] ;	/* Image 2D buffer. */
   extern FILE *FP_DLOG, *FP_ILOG, *FP_ILOG2 ;

   static char namfts [80] = "sp201.fts";
   static char namext [80] = "SPARTAN_CMAP";
   int hduflag = 1 ;
   int kyear, kdoy, khour, kmin ;
   float ksec ;
   float ncp_ang, nhp_ang ;

   ncp_ang = SPII->ocp_ang - SPII->rcp_ang + SPII->wlc_ang ;   /* Geocentric */
   nhp_ang = ncp_ang - SPII->pangle ;		/* Heliocentric */

   sprintf (namfts, "sp.%04d%02d%02d.%02d%02d%02d.fts",
	    SPII->year, SPII->month,  SPII->day,
	    SPII->hour, SPII->minute, SPII->second) ;

   fprintf (FP_DLOG, "---> FITS image: %s  met:%8d  orbit:%3d  image:%3d\n",
		     namfts, SPIH->met, SPIH->orbit_num, SPIH->image_num) ;
   fprintf (stdout, "%s met:%8d orb/img:%2d-%2d ocp:%7.2f rcp:%7.2f ncp:%7.2f pangle:%7.2f nhp:%7.2f\n",
		     namfts, SPIH->met, SPIH->orbit_num, SPIH->image_num,
		     SPII->ocp_ang, SPII->rcp_ang,
		     ncp_ang, SPII->pangle, nhp_ang) ;

   metconv (SPIH->met, &kyear, &kdoy, &khour, &kmin, &ksec) ;

   fprintf (FP_ILOG, "%25s %02d-%02d:%02d:%05.2f %2d - %2d %5.2f %2d  %3.3s  %3.3s  %2.2s   %2.2s %7.2f %7.2f %7.2f %5.2f %5.2f %5.2f %5.2f %5.2f %5.2f %7.2f %7.2f %7.2f %7.2f\n",
            namfts, kdoy, khour, kmin, ksec,
	    SPII->orbit_num, SPII->image_num, SPII->expdur,
	    SPII->nimsum, &SPII->optpos[0], &SPII->optpos[1], &SPII->optpos[2],
	    SPII->optpos[3], SPII->ccd_temp_beg, SPII->ccd_temp_end,
	    SPII->nangle,
	    SPII->pes_yaw_ave, SPII->pes_yaw_min, SPII->pes_yaw_max,
	    SPII->pes_pitch_ave, SPII->pes_pitch_min, SPII->pes_pitch_max,
	    SPII->ocp_ang, SPII->rcp_ang, SPII->pangle, SPII->nangle) ;
   fflush (FP_ILOG) ;

   fprintf (FP_ILOG2, "%25s,%02d-%02d:%02d:%05.2f,%02d-%03d,%5.2f,%2d,%3.3s,%3.3s,%2.2s,%2.2s,%7.2f,%7.2f,%7.2f,%5.2f,%5.2f,%5.2f,%5.2f,%5.2f,%5.2f,%7.2f,%7.2f,%7.2f,%7.2f\n",
            namfts, kdoy, khour, kmin, ksec,
	    SPII->orbit_num, SPII->image_num, SPII->expdur,
	    SPII->nimsum, &SPII->optpos[0], &SPII->optpos[1], &SPII->optpos[2],
	    SPII->optpos[3], SPII->ccd_temp_beg, SPII->ccd_temp_end,
	    SPII->nangle,
	    SPII->pes_yaw_ave, SPII->pes_yaw_min, SPII->pes_yaw_max,
	    SPII->pes_pitch_ave, SPII->pes_pitch_min, SPII->pes_pitch_max,
	    SPII->ocp_ang, SPII->rcp_ang, SPII->pangle, SPII->nangle) ;
   fflush (FP_ILOG2) ;

   /*** Write image to disk in FITS format. ***/

   sptofits (hkbuf, imgbuf, namfts, namext, hduflag, pmode) ;

   return (0) ;
}
