/* sp2015lib.h
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-4 & 201-5 structures.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   23 September 1996
 * 12 May 1998: Modify by adding uvcs_yfp, uvcs_rfp, wlc_yfp, wlc_rfp.
 *  9 Nov 1998: Modify Initial start time for STS95 Discovery Mission.
 *  1 Dec 1998: add rcp_ang, ncp_ang & wlc_ang to spartan_img_info structure.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*/
#define BLKSIZ 8200		/* # bytes in a block. */
#define NBYEDF 128		/* # bytes per Engineering Data Frame. */
#define NBYIDF 688		/* # bytes per Image       Data Frame. */
#define NBYMDF  40		/* # bytes per Message     Data Frame. */
#define NX     320		/* # pixels in horizontal axis of image. */
#define NY     256		/* # pixels in horizontal axis of image. */
#define NBYPIX   2		/* # bytes per pixel. */
#define NBYHK   46		/* # bytes for housekeeping. */

struct spartan_eng_hk
   {
   char sync [6] ;
   int met ;
   int spare_ad ;
   int p28v ;
   int p5v ;
   int rs232_p12v ;
   int ccd_ted_volt ;
   int ccd_bat_p6v ;
   int ccd_p5v ;
   int ccd_p15v ;
   int ccd_m15v ;
   int uvcs_yfp ;
   int uvcs_rfp ;
   int sun_pres ;
   int fine_mode2 ;
   int pitch_trig_rate ;
   int yaw_fine_pos ;
   int roll_fine_pos ;
   int yaw_offset ;
   int roll_offset ;
   int pes_pitch ;
   int pes_yaw ;
   int front_bulk_temp ;
   int o1_temp ;
   int fw_temp ;
   int ccd_elec_temp ;
   int ccd_det_temp ;
   int wlc_yfp ;
   int wlc_rfp ;
   int ted_hot_temp ;
   int p5v_supply_temp ;
   int elec_pkg2_temp ;
   int m68000_temp ;
   int tx_start ;
   int tx_done ;
   int m5v_eom ;
   int acs_byte ;
   int wlc_sun_pres ;
   int pes_ok ;
   int fold_mir_pos ;
   int heat_mir_pos ;
   int pw_pos ;
   int half_wave_pos ;
   int half_wave_io ;
   int fold_mir_io ;
   int heat_mir_io ;
   int pw_io ;
   int stim_lamp_io ;
   int wlc_mode ;
   int reset_counter ;
   int flt_sw_ver ;
   int rel_pitch_ang ;
   int orbit_num ;
   int image_num ;
   int acs_pointer ;
   int half_wave_cmd ;
   int fold_mir_cmd ;
   int heat_mir_cmd ;
   int pw_cmd ;
   int stim_lamp_cmd ;
   int msg_flag_sot ;
   int acs_buf [7] ;
   int opt_data_id ;
   int last_sunrise ;
   int last_sunset ;
   int time_avail ;
   int table_num ;
   int prog_num ;
   int eom_flag ;
   int undet_pitch_flag ;
   int testmode_flag ;
   int uvcs_raster_flag ;
   int wlc_cal_flag ;
   int wlc_liss_flag ;
   int base_add_gse ;
   int ccd_mode ;
   int ccd_erase ;
   int ccd_expdur ;
   } ;

struct spartan_eng_info
   {
   char  sync [6] ;
   int   met ;
   int   year ;
   int   doy ;
   int   month ;
   int   day ;
   int   hour ;
   int   minute ;
   float second ;
   int   spare_ad ;
   float p28v ;
   float p5v ;
   float rs232_p12v ;
   float ccd_ted_volt ;
   float ccd_bat_p6v ;
   float ccd_p5v ;
   float ccd_p15v ;
   float ccd_m15v ;
   float uvcs_yfp ;
   float uvcs_rfp ;
   char  sun_pres [4] ;
   char  fine_mode2 [4] ;
   float pitch_trig_rate ;
   float yaw_fine_pos ;
   float roll_fine_pos ;
   float yaw_offset ;
   float roll_offset ;
   float pes_pitch ;
   float pes_yaw ;
   float front_bulk_temp ;
   float o1_temp ;
   float fw_temp ;
   float ccd_elec_temp ;
   float ccd_det_temp ;
   float wlc_yfp ;
   float wlc_rfp ;
   float ted_hot_temp ;
   float p5v_supply_temp ;
   float elec_pkg2_temp ;
   float m68000_temp ;
   int   tx_start ;
   int   tx_done ;
   char  m5v_eom [12] ;
   int   acs_byte ;
   char  wlc_sun_pres [4] ;
   char  pes_ok [4] ;
   char  fold_mir_pos [4] ;
   char  heat_mir_pos [4] ;
   char  pw_pos [8] ;
   char  half_wave_pos [8] ;
   char  half_wave_io [4] ;
   char  fold_mir_io [4] ;
   char  heat_mir_io [4] ;
   char  pw_io [4] ;
   char  stim_lamp_io [4] ;
   int   wlc_mode ;
   int   reset_counter ;
   float flt_sw_ver ;
   float rel_pitch_ang ;
   int   orbit_num ;
   int   image_num ;
   int   acs_pointer ;
   char  half_wave_cmd [8] ;
   char  fold_mir_cmd [8] ;
   char  heat_mir_cmd [8] ;
   char  pw_cmd [8] ;
   char  stim_lamp_cmd [8] ;
   char  msg_flag_sot [12] ;
   int   acs_buf [7] ;
   int   opt_data_id ;
   float last_sunrise ;
   float last_sunset ;
   float time_avail ;
   int   table_num ;
   int   prog_num ;
   char  eom_flag [4] ;
   int   undet_pitch_flag ;
   int   testmode_flag ;
   char  uvcs_raster_flag [4] ;
   char  wlc_cal_flag [4] ;
   char  wlc_liss_flag [4] ;
   int   base_add_gse ;
   char  ccd_mode [8];
   int   ccd_erase ;
   float ccd_expdur ;
   } ;

struct spartan_img_hk		/* HAO/Spartan housekeeping data. */
   {
   char sync [6] ;		/* Sync pattern.     */
   int met ;			/* bytes  6 -  9     */
   int rel_pitch_ang ;		/* bytes 10 - 11     */
   int fold_mir_pos ;		/* byte  12 bit  0   */
   int heat_mir_pos ;		/* byte  12 bit  1   */
   int pw_pos ;			/* byte  12 bits 2-3 */
   int half_wave_pos ;		/* byte  12 bits 4-5 */
   int num_img_sum ;		/* byte  13 bits 0-3 */
   int stim_lamp_stat ;		/* byte  13 bit  5   */
   int ccd_expdur ;		/* bytes 14 - 15     */
   int orbit_num ;		/* byte  16          */
   int image_num ;		/* byte  17          */
   int ccd_temp_beg ;		/* bytes 18 - 19     */
   int ccd_temp_end ;		/* bytes 20 - 21     */
   int pes_yaw_mean ;		/* bytes 22 - 23     */
   int pes_yaw_min ;		/* bytes 24 - 25     */
   int pes_yaw_max ;		/* bytes 26 - 27     */
   int pes_pitch_mean ;		/* bytes 28 - 29     */
   int pes_pitch_min ;		/* bytes 30 - 31     */
   int pes_pitch_max ;		/* bytes 32 - 33     */
   int spare1 ;			/* bytes 34 - 35     */
   int spare2 ;			/* byte  36          */
   int spare3 ;			/* byte  37          */
   int spare4 ;			/* byte  38          */
   int spare5 ;			/* byte  39          */
   int spare6 ;			/* byte  40          */
   int spare7 ;			/* byte  41          */
   int spare8 ;			/* byte  42          */
   int spare9 ;			/* byte  43          */
   int cur_ccd_line ;		/* bytes 44 - 45     */
   int pos [6] ;
   } ;

struct spartan_img_info		/* HAO/Spartan housekeeping info. */
   {
   int   swvers 	;	/* Software version number.	*/
   int   dtype		;	/* Data type [1-8]		*/
   int   xdim		;	/* Image width			*/
   int   ydim		;	/* Image height			*/
   int   zdim		;	/* Image depth			*/
   int   nbypx		;	/* # bytes/pixel		*/
   int   dmin		;	/* Data minimum value		*/
   int   dmax		;	/* Data maximum value		*/
   int   smin		;	/* Display min value		*/
   int   smax		;	/* Display max value		*/
   int   year		;	/* Image year			*/
   int   month		;	/* Image month			*/
   int   day		;	/* Image day (of month)		*/
   int   doy		;	/* Image day-of-year		*/
   int   hour		;	/* Image hour			*/
   int   minute		;	/* Image minute			*/
   int   second		;	/* Image second			*/
   int   cal_year 	;	/* Calibration year.		*/
   int   cal_month 	;	/* Calibration month.		*/
   int   cal_day 	;	/* Calibration day.		*/
   int   nimsum		;	/* # images summed.		*/
   int   orbit_num	;	/* Orbit number.		*/
   int   image_num	;	/* Image number.		*/

   char optdev [5][12]	;	/* Names of optical devices	*/
   char optpos [5][12]	;	/* Positions of optical devices	*/

   float expdur		;	/* Exposure duration (sec)	*/
   float ccd_temp_beg	;	/* CCD temp @ First Image.	*/
   float ccd_temp_end	;	/* CCD temp @ Last  Image.	*/
   float pes_yaw_ave	;	/* PES Yaw Mean.		*/
   float pes_yaw_min	;	/* PES Yaw Minimum.		*/
   float pes_yaw_max	;	/* PES Yaw Maximum.		*/
   float pes_pitch_ave	;	/* PES Pitch Mean.		*/
   float pes_pitch_min	;	/* PES Pitch Minimum.		*/
   float pes_pitch_max	;	/* PES Pitch Maximum.		*/

   float odisk  	;	/* Occulting disk (arcsec)      */
   float bscale 	;	/* Brightness scaling factor.	*/
   float bzero  	;	/* Brightness scaling offset.	*/
   float pixrs		;	/* # pixels/Rsun		*/
   float pangle		;	/* P-angle (degrees)		*/
   float bangle		;	/* B-angle (degrees)		*/
   float langle		;	/* L-angle (degrees)		*/
   float ra     	;	/* Right Ascension (degrees)    */
   float dec    	;	/* Declination (degrees)        */
   float hangle		;	/* Hour-angle (degrees)		*/
   float rcp_ang	;	/* Relative Celestial Pole Angle (deg). */
   float ocp_ang	;	/* North Celestial Pole Angle offset (deg). */
   float wlc_ang	;	/* WLC Angle w.r.t. Spartan S/C (deg). */
   float nangle 	;	/* Angle (CCW) of north w.r.t. +X axis. */
   float elev   	;	/* Solar elevation (degrees)	*/
   float azimuth	;	/* Solar azimuth (degrees)	*/
   float airmass	;	/* Airmass.			*/
   float rsun   	;	/* Solar radius (arcsec)	*/
   float xcen		;	/* X-axis sun center		*/
   float ycen		;	/* Y-axis sun center		*/
   float rmin   	;	/* Inner FOV radius (Rsun)      */
   float rmax   	;	/* Outer FOV radius (Rsun)      */
   float wavelen	;	/* Filter Wavelength (nm)       */
   float wavewid	;	/* Filter Banpass (nm)          */
   } ;

struct spartan_msg_hk
   {
   char sync [6] ;              /* Sync pattern.     */
   int msg1 ;
   int msg1_met ;
   int msg2 ;
   int msg2_met ;
   int msg3 ;
   int msg3_met ;
   int msg4 ;
   int msg4_met ;
   int msg5 ;
   int msg5_met ;
   int msg_pointer ;
   } ;

struct spartan_msg_info
   {
   int msg1 ;
   int msg1_met ;
   int msg2 ;
   int msg2_met ;
   int msg3 ;
   int msg3_met ;
   int msg4 ;
   int msg4_met ;
   int msg5 ;
   int msg5_met ;
   int msg_pointer ;
   } ;

#ifdef SPARTAN 

/* Initial GMT for MET = 0.  Test 1: 28 Jul 1998 */

/*
   int iyear0 =  98 ;
   int idoy0  = 209 ;
   int ihour0 =  19 ;
   int imin0  =  43 ;
   int sct0   =   0 ;
   float sec0  =  0.0 ;
 */

/* Initial GMT for MET = 0.  Test 2: 10 Aug 1998 */

/*
   int iyear0 =  98 ;
   int idoy0  = 222 ;
   int ihour0 =  20 ;
   int imin0  =  18 ;
   int sct0   =   0 ;
   float sec0  =  0.0 ;
 */

/* Initial GMT for MET = 0.  STS95 Discovery Mission. */

   int iyear0 =  98 ;
   int idoy0  = 305 ;
   int ihour0 =  19 ;
   int imin0  =  04 ;
   int sct0   =   0 ;
   float sec0  = 44.0 ;

   struct spartan_eng_hk  SPEHS ;
   struct spartan_eng_hk *SPEH = &SPEHS ;

   struct spartan_eng_info  SPEIS ;
   struct spartan_eng_info *SPEI = &SPEIS ;

   struct spartan_img_hk  SPIHS ;
   struct spartan_img_hk *SPIH = &SPIHS ;

   struct spartan_img_info  SPIIS ;
   struct spartan_img_info *SPII = &SPIIS ;

   struct spartan_msg_hk  SPMHS ;
   struct spartan_msg_hk *SPMH = &SPMHS ;

   struct spartan_msg_info  SPMIS ;
   struct spartan_msg_info *SPMI = &SPMIS ;
#else
   extern int iyear0 ;
   extern int idoy0  ;
   extern int ihour0 ;
   extern int imin0  ;
   extern int sct0   ;
   extern float sec0 ;

   extern struct spartan_eng_hk  SPEHS ;
   extern struct spartan_eng_hk *SPEH  ;

   extern struct spartan_eng_info  SPEIS ;
   extern struct spartan_eng_info *SPEI  ;

   extern struct spartan_img_hk  SPIHS ;
   extern struct spartan_img_hk *SPIH  ;

   extern struct spartan_img_info  SPIIS ;
   extern struct spartan_img_info *SPII  ;

   extern struct spartan_msg_hk  SPMHS ;
   extern struct spartan_msg_hk *SPMH  ;

   extern struct spartan_msg_info  SPMIS ;
   extern struct spartan_msg_info *SPMI  ;
#endif
