/* spmtlm.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Spartan 201-4 Message Telemetry Decoder.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   25 April 1997
 * 13 May 1998: "sp201lib.h" replaced by "sp2014lib.h".
 * 14 Oct 1998: "sp2014lib.h" replaced by "sp2015lib.h".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "sp2015lib.h"

spmtlm (mbuf, pmode)

unsigned char mbuf [] ;			/* Spartan Message array. */
int pmode ;				/* Print mode. */

{
   extern int idf ;
   extern long ifloc ;
   extern FILE *FP_MLOG ;
   int i ;
   int iyear, idoy, imonth, iday, ihour, iminute ;
   int kyear, kdoy, khour, kmin ;
   float isecond ;
   float ksec ;

   for (i = 0; i < 6; i++)
	 SPMH->sync [i] = mbuf [i] ;
   SPMH->msg1            = int2 (&mbuf [ 6]) ;
   SPMH->msg1_met        = int4 (&mbuf [ 8]) ;
   SPMH->msg2            = int2 (&mbuf [12]) ;
   SPMH->msg2_met        = int4 (&mbuf [14]) ;
   SPMH->msg3            = int2 (&mbuf [18]) ;
   SPMH->msg3_met        = int4 (&mbuf [20]) ;
   SPMH->msg4            = int2 (&mbuf [24]) ;
   SPMH->msg4_met        = int4 (&mbuf [26]) ;
   SPMH->msg5            = int2 (&mbuf [30]) ;
   SPMH->msg5_met        = int4 (&mbuf [32]) ;
   SPMH->msg_pointer     = int2 (&mbuf [36]) ;


   metconv (SPMH->msg1_met, &kyear, &kdoy, &khour, &kmin, &ksec) ;
   metconv (SPMH->msg2_met, &kyear, &kdoy, &khour, &kmin, &ksec) ;
   metconv (SPMH->msg3_met, &kyear, &kdoy, &khour, &kmin, &ksec) ;
   metconv (SPMH->msg4_met, &kyear, &kdoy, &khour, &kmin, &ksec) ;
   metconv (SPMH->msg5_met, &kyear, &kdoy, &khour, &kmin, &ksec) ;

/*
   fprintf (FP_MLOG, "%02d-%02d:%02d:%06.3f %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d %4d\n",
      kdoy, khour, kmin, ksec,
      SPMH->p28v, SPMH->p5v, SPMH->rs232_p12v, SPMH->ccd_ted_volt,
      SPMH->ccd_bat_p6v, SPMH->ccd_p5v, SPMH->ccd_p15v, SPMH->ccd_m15v,
      SPMH->sun_pres, SPMH->fine_mode2, SPMH->pitch_trig_rate,
      SPMH->yaw_fine_pos, SPMH->roll_fine_pos, SPMH->yaw_offset,
      SPMH->roll_offset, SPMH->pes_pitch, SPMH->pes_yaw, SPMH->front_bulk_temp,
      SPMH->o1_temp, SPMH->fw_temp) ;
 */

   /*** MET = 0 corresponds to 1997 DOY 325 21:09:10 ***/

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPMH->msg1_met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPMH->msg2_met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPMH->msg3_met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPMH->msg4_met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   metgmt (iyear0, idoy0, ihour0, imin0, sec0, sct0, SPMH->msg5_met,
	  &iyear,  &idoy,  &ihour,  &iminute,  &isecond) ;

   month_day (iyear, idoy, &imonth, &iday) ;

   /*** Fill Engineering Information structure. ***/
   
/*
   SPMI->year   = iyear ;
   SPMI->doy    = idoy ;
   SPMI->month  = imonth ;
   SPMI->day    = iday   ;
   SPMI->doy    = idoy ;
   SPMI->hour   = ihour ;
   SPMI->minute = iminute ;
   SPMI->second = isecond ;
 */

   SPMI->msg1     = ((SPMH->msg1)) ;
   SPMI->msg1_met = ((SPMH->msg1_met)) ;
   SPMI->msg2     = ((SPMH->msg2)) ;
   SPMI->msg2_met = ((SPMH->msg2_met)) ;
   SPMI->msg3     = ((SPMH->msg3)) ;
   SPMI->msg3_met = ((SPMH->msg3_met)) ;
   SPMI->msg4     = ((SPMH->msg4)) ;
   SPMI->msg4_met = ((SPMH->msg4_met)) ;
   SPMI->msg5     = ((SPMH->msg5)) ;
   SPMI->msg5_met = ((SPMH->msg5_met)) ;


   fprintf (FP_MLOG, "%4d-%8d %10d %02d %10d %02d %10d %02d %10d %02d %10d %02d %10d\n",
      idf, ifloc,
      SPMH->msg1, SPMH->msg1_met,
      SPMH->msg2, SPMH->msg2_met,
      SPMH->msg3, SPMH->msg3_met,
      SPMH->msg4, SPMH->msg4_met,
      SPMH->msg5, SPMH->msg5_met,
      SPMH->msg_pointer) ;

   return (0) ;
}
