;From: lika@sdac.gsfc.nasa.gov
;Date: Thu, 6 Apr 1995 16:27:33 -0400
;To: card@hao.ucar.edu
;Subject: Reformat, windowless

;-------------------------------------------------------------------------------
; reformat_spartan.pro
;-------------------------------------------------------------------------------
; REFORMAT_SPARTAN reads SPARTAN 201 WLC tape records dumped to a disk file, 
; in an attempt to reformat image data.
; D.M. fecit, 1993.
;-------------------------------------------------------------------------------   
FUNCTION spartan_int, byte_array

n_byte = N_ELEMENTS (byte_array)
n_word = n_byte / 2
b = BYTARR (2, n_word)
b (0) = byte_array (0 : 2 * n_word - 1)
a = REFORM (b (0, *) * 256 + b (1, *))
RETURN, a
END

;-------------------------------------------------------------------------------   
FUNCTION exp_time, time_bytes

c = 256l ^ (3 - indgen (4))
t_sec = 0.008 * total (c * time_bytes)
RETURN, t_sec
END

;-------------------------------------------------------------------------------   
FUNCTION time_string, t_sec

h = fix (t_sec / 3600.0)
t_rem = t_sec - 3600.0 * h
m = fix (t_rem / 60.0)
s = t_rem - 60.0 * m

h = STRTRIM (h, 2)
IF (STRLEN (h) EQ 1) THEN h = '0' + h

m = STRTRIM (m, 2)
IF (STRLEN (m) EQ 1) THEN m = '0' + m

s = STRTRIM (STRING (s, FORMAT = '$(f6.3)'), 2)
IF (STRLEN (s) EQ 5) THEN s = '0' + s

met = h + ':' + m + ':' + s
RETURN, met
END

;-------------------------------------------------------------------------------   
PRO decom_params, b, met, pitch, yaw, roll, exposure, pol, hwp, cal, n_sum, $
                  ccd_t

pol_pos = ['clear', '    0', '   30', '   60']
hwp_pos = 30 * (3 - indgen (4))
ad25 = [36680.0, 4096.0, -13570.0]
cal_string = [' ', 'C']

start_time = exp_time (b (6 : 9))
met   = time_string (start_time)		; mission elapsed time
pitch = 0.5 * (b (10) * 256 + b (11))		; relative pitch angle
yaw   = 256 * b (22) + b (23)			; mean PES yaw
roll  = 256 * b (28) + b (29)			; mean PES roll
exposure = 0.008 * (256 * b (14) + b (15))	; exp. time (s)
pol = pol_pos (ishft (b (12), -2) and 3)		; polarizer wheel position
hwp = hwp_pos (ishft (b (12), -4) and 3)		; half-wave plate position
cal = cal_string (ISHFT (b (13), -7))		; calibration mirror
n_sum = b (13) AND 'f'xb			; number of summed exposures
ccd_t = spartan_int (b (18 : 21))		; CCD initial, final temps.
ccd_t = 0.01 * (ccd_t * ad25 (0) / ad25 (1) + ad25 (2))

RETURN
END

;-------------------------------------------------------------------------------   
;@sys$user:[gurman.spartan]find_image

@find_image

; device = !d.name & IF (STRUPCASE (device) NE 'X') THEN SET_PLOT, 'X'

journal_format = $
   '$(a11, 2x, a5, 1x, a12, 1x, 3(i6, 1x), f7.3, 2x, a5, 3x, i2, 5x, a1, 3x, i3, 2(2x, f5.1))'

;stop

file = '' & extension = '9404'

;--- NOTE: Change the extension for future flights.

WHILE file EQ '' DO BEGIN
;    read, 'File name [SYS$SYSDEVICE:[spartan.sept94]]      ', file

    READ, 'File name     ', file
END

;--- Find out how big the sucker is.

;spawn, 'directory /size = all /out = check.size sys$sysdevice:[spartan.sept94]' + file

SPAWN, 'directory /size = all /out = check.size ' + file
OPENR, size_unit, 'check.size', /get_lun
s = ''
WHILE strpos (s, '/') LT 0	$
   DO READF, size_unit, s
CLOSE, size_unit
FREE_LUN, size_unit

SPAWN, 'delete check.size;'
ich          = STRPOS (s, '/')
n_block      = LONG (STRMID (s, ich - 8, 8))
n_tape_block = (n_block / 16) - 1
n_800        = n_tape_block / 800
n_extra      = n_tape_block MOD 800
n_VLA        = n_800 + (n_extra NE 0)

IF (n_extra EQ 0) THEN block_size = INTARR (n_800) + 800 ELSE BEGIN
   IF (n_800 GT 0) THEN block_size = [INTARR (n_800) + 800, n_extra] ELSE $
      block_size = INTARR (1) + n_extra
END

PRINT, '%REFORMAT_SPARTAN-D-BLOCKS, block_size ='
PRINT, block_size

;openr, in_unit, '$1$dia5:[spartan]' + file, /get_lun 

OPENR, in_unit,  file, /get_lun 

PRINT, ' '
PRINT, '%SPARTAN-I-DOZE, setting up VLA to read in up to 6.4 Mbyte from'
PRINT, '                 disk dump of tape.'
PRINT, '%SPARTAN-I-TIME, time now is ' + !stime

vla_size = 8192L * 800
a = BYTARR (vla_size)
w = ASSOC (in_unit, a)
PRINT, '%SPARTAN-I-ALLOC, initial memory allocation completed.'
PRINT, '%SPARTAN-I-TIME, time now is ' + !stime & print, ' '

;stop

journal_file = 'journal.dat'
ich = STRPOS (STRUPCASE (file), 'TAPE_')
IF (ich GE 0) THEN journal_file = 'journal_' + STRMID (file, 5, 2) + '.dat'

OPENW, journal_unit, journal_file, /get_lun
PRINTF, journal_unit, 'Journal for file ' + file
PRINTF, journal_unit, 'Run starting at ' + !stime
FOR i = 0, 1 DO PRINTF, journal_unit, ' '
PRINTF, journal_unit, $
   '             Image                                             Pol.' + $
   '  Half  Cal.  Sum  T_CCD  T_CCD'
PRINTF, journal_unit, $
   '   File       ID        MET      Pitch  <Yaw>  <Roll>  Exp.    pos.' + $
   '  wave  mirr  exp  first  last'
PRINTF, journal_unit, ' '

;--- Get the next file number to use by checking for other SPARTAN files on
;--- this directory. (This could be replaced by checking a catalog.)

;s = findfile ('$1$dia5:[spartan]S*.' + extension)

s = FINDFILE ('S*.' + extension)
n_file = N_ELEMENTS (s)

IF (N_ELEMENTS (s) EQ 1) AND (s (0) EQ '') THEN BEGIN
   n_file = 0
   i_file = 1
ENDIF ELSE BEGIN
   s = s (n_file - 1)
   i_right_bracket = STRPOS (s, ']')
   nch = STRLEN (s)
   i_file = fix (STRMID (s, i_right_bracket + 2, 5)) + 1
END

file_string = STRTRIM (i_file, 2)
WHILE (STRLEN (file_string) LT 5) DO file_string = '0' + file_string

file_name = 'S' + file_string + '.' + extension

;OPENW, out_unit, 'sys$sysdevice:[spartan.sept94]' + file_name, 640, /get_lun

OPENW, out_unit, file_name, 640, /get_lun
w_out = ASSOC (out_unit, INTARR (320))
image = INTARR (320, 256)
first_line = 1

n_byte_remaining = 0
mem_alloc = 0

FOR i_VLA = 0, n_VLA - 1 DO BEGIN

   PRINT, '%SPARTAN-I-BEGIN_READ, time now is ' + !stime

   IF block_size (i_VLA) EQ 800 THEN BEGIN
      a = w (i_VLA)
   ENDIF ELSE BEGIN
      vla_size = 8192L * n_extra
      w = ASSOC (in_unit, BYTARR (8192))
      offset = n_800 * 800L
      FOR i_extra = 0, n_extra - 1 DO a (8192L * i_extra) = w (offset + i_extra)
   END

   PRINT, '%SPARTAN-I-END_READ, time now is ' + !stime

;--- If only part of an image line remained in the previous VLA, read the rest
;--- and write the complete image line to disk before proceding.

   IF (n_byte_remaining NE 0) THEN BEGIN
      n_word_written = 320 - n_byte_remaining / 2
      image (0, image_line) = [image (0 : n_word_written - 1, image_line), $
         spartan_int (a (0 : n_byte_remaining - 1))]
      w_out (image_line) = image (*, image_line)
      n_byte_remaining = 0
   END

   PRINT, '%SPARTAN-I-INDICES, calling FIND_IMAGE to locate image data frames.'

   find_image, a (0 : vla_size - 1), frame_index

   n_index = N_ELEMENTS (frame_index)

   FOR i_index = 0, n_index - 1 DO BEGIN

;--- I know I'm going to regret using records of length 320 on a system
;--- with 512-byte disk blocks....

      index = frame_index (i_index)
;
;--- If there's even one word of image data, get it.
;
      n_max = vla_size - index
      IF (n_max GE 48) THEN BEGIN

;--- Figure out how many data frames are needed from the next
;--- very large array's worth of data.

         IF (n_max LT 686) THEN n_byte_remaining = 686 - n_max ELSE $
            n_byte_remaining = 0
         b = a (index : index + MIN ([n_max - 1, 685]))

;--- Check to see if matching only the first two bytes of the synch pattern
;--- and the data frame descriptor byte was enough. If so, start turning the
;--- crank.

         IF ((b (2) eq '66'xb) AND (b (3) EQ '55'xb) AND (b (4) EQ 0b)) THEN $
	 BEGIN
            image_id = b (16 : 17)
	    image_line = 256 * b (44) + b (45)

;--- If it's the very first image line we've read, decommutate some
;--- observing parameters.

            IF (first_line) THEN BEGIN
               last_id = image_id
	       first_line = 0
	       n_line = 0
               decom_params, b, met, pitch, yaw, roll, exposure, pol, hwp, $
                             cal, n_sum, ccd_t
;
               PRINT, '%SPARTAN-D-FRAME, beginning frame ', image_id
            ENDIF ELSE BEGIN
               same_image = $
                 (image_id (0) EQ last_id (0)) AND (image_id (1) EQ last_id (1))

;--- If a new image has started, CLOSE the image file, ....

               IF (NOT same_image) THEN BEGIN
                  PRINT, '%SPARTAN-D-FILE, closing file ' + file_name + $
                     ' with ' + STRTRIM (n_line, 2) + ' lines.'
                  CLOSE, out_unit
                  image_id_string = STRING (last_id (0), FORMAT='$(i2)') + '/' $
		                  + STRING (last_id (1), FORMAT='$(i2)')

;--- display the image, ....

;                 WINDOW, 0, xsize = 320, ysize = 256, $
;                    title = 'SPARTAN 201 image ' + image_id_string
;                 display_image = ((image / n_sum) > 0) < 4095
;                 color, 10 & tvscl, sqrt (display_image) , order = 1
                  PRINT, 'Total (image) = ', LONG (total (image))

;--- initialize a new file and some variables, ....

                  i_file = i_file + 1
		  file_string = STRTRIM (i_file, 2)
                  WHILE (STRLEN (file_string) LT 5) DO $
                     file_string = '0' + file_string
                  file_name = 'S' + file_string + '.' + extension

;                   OPENW, out_unit, 'sys$sysdevice:[spartan.sept94]' + file_name, 640
                  OPENW, out_unit,  file_name, 640
                  image = INTARR (320, 256)
		  last_id = image_id
		  n_line = 0
                  w_out (255) = TEMPORARY (INTARR (320))

;--- write the catalog information, ....

                  PRINT, format = journal_format, $
                     file_name, image_id_string, met, pitch, yaw, roll, $
                     exposure, pol, hwp, cal, n_sum, ccd_t
                  PRINTF, journal_unit, format = journal_format, $
                     file_name, image_id_string, met, pitch, yaw, roll, $
                     exposure, pol, hwp, cal, n_sum, ccd_t

;--- and decommutate the new observing parameters.

                  decom_params, b, met, pitch, yaw, roll, exposure, pol, hwp, $
                                cal, n_sum, ccd_t
               END
            END

;--- In any case, write out an image line to disk.

            image (0, image_line) = spartan_int (b (46 : *))
            w_out (image_line)    = image (*, image_line)
	    n_line = n_line + 1
         END
      ENDIF ELSE BEGIN

;--- If there's not enough data remaining to read in image data, 
;--- close the last image file and write the catalog information.

         CLOSE, out_unit
         PRINT, format = journal_format, $
            file_name, image_id_string, met, pitch, yaw, roll, exposure, $
            pol, hwp, cal, n_sum, ccd_t
         PRINTF, journal_unit, format = journal_format, $
            file_name, image_id_string, met, pitch, yaw, roll, exposure, $
            pol, hwp, cal, n_sum, ccd_t
      END
   END
END

PRINTF, journal_unit, '' & PRINTf, journal_unit, 'Run finished at ' + !stime
CLOSE, /all

END
;-------------------------------------------------------------------------------   
