; Routine to read raw mk4 files and output 
; files in a format for further processing
;
; INPUT:	The name of a file contained in the file ~/mk4slist
; OUTPUT:	input_file_name_I
; 		input_file_name_Q
; 		input_file_name_U
;
; The input file is read, header information 
; current barrel angle is extracted.
; The I, Q, and U raw data are extracted.
;
; The files are 'cleaned' of any point defects
; There is a hokey section which corrects for intensity digital
; wraparounds and a factor of two gain change at elevated pixels
;
; The data are spline fit to regularly spaced points, 1080 of them
; in a circle.
; The data are offset by the channel 4 offset (30 deg)
; Calibrations must be performed in the instrument coordinate
; frame.
; Later one can correct calibrated corona by the P-angle
;
; Files are written out
;
; David Elmore 11/5/98
;

debug = 0
NUMPIX = 384
NUMAZ = 1080
window,0,xsize=512,ysize=512
window,1,xsize=512,ysize=512

imgi=fltarr(NUMPIX,NUMAZ)
imgq=fltarr(NUMPIX,NUMAZ)
imgu=fltarr(NUMPIX,NUMAZ)
data=intarr(3,NUMPIX)
header=intarr(32)
filename='fred'
angle=fltarr(NUMAZ)
st = intarr(NUMAZ)


;
; while there are still filenames to read from mk4slist
; read and process them
;
close,2
openr,2,'mk4slist'
while ( not eof(2) ) do begin
	readf,2,filename
	print,filename

	close,1
	openr,1,filename
	i = 0
	while ( not eof(1) ) do begin
		readu,1,header,data
		if ( i gt 0 ) then begin
			if ( header(19) eq 0 ) then goto, endofrecords
		endif
		angle(i) = header(10)*180./8192.
		st(i) = header(2)

		imgi(*,i)=float(data(0,*))
		imgq(*,i)=float(data(1,*))
		imgu(*,i)=float(data(2,*))
		i = i + 1
	endwhile

	endofrecords:

wset,0
!y.range=[300,400]
plot,st,psym=3

wset,1
!y.range=[-200,200]
plot,angle,psym=3

goto, nospline
	pangle = header(14)/60.
	close,1 & openw,1,'Pangle' & writeu,1,pangle

	numaz = i
	numazm1 = numaz - 1
	ang=angle(0:numazm1)

	;
	; Remove point defects
	; Following fix of data will create streaks if they remain
	; It also confuses the boundaries between digital steps,
	; but the former is worse
	;
	imi = imgi(*,0:numazm1)
	imi = cleanmk4(imi)
	imq = imgq(*,0:numazm1)
	imq = cleanmk4(imq)
	imu = imgu(*,0:numazm1)
	imu = cleanmk4(imu)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; The following code should vanish once instrument 'correct' table values
; are right.
; Try to fix up data
;	The 300 value will change when proper corrector values
;	have been found
;  A magic pixel value of 354 is appropriate for SPARTAN days
;  The value of 327 is good for
; Intensity data has digital wraps...

	for jj = 300,NUMPIX-1 do begin
		addval = 0
		if ( imi(jj,0) lt 0 ) then addval = 32768
		imi(jj,0) = imi(jj,0)+addval
		oldpix = imi(jj,0)
		for ii=1,NUMAZ-1 do begin
			imi(jj,ii) = imi(jj,ii) + addval
			if ( (imi(jj,ii)-oldpix ) gt 16384 ) then begin
				addval = addval - 32768
				imi(jj,ii) = imi(jj,ii) - 32768
			endif 
			if ( (imi(jj,ii)-oldpix) lt -16384 ) then begin
				addval = addval + 32768
				imi(jj,ii) = imi(jj,ii) + 32768
			endif
			oldpix=imi(jj,ii)
		endfor
	endfor
	;
	; Gain above magic pixel is off by 2
	;
	imi(327:383,*)=imi(327:383,*)/2.
	imq(327:383,*)=imq(327:383,*)/2.
	imu(327:383,*)=imu(327:383,*)/2.

; end of digital 'correct' problem fix
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	imis=fltarr(NUMPIX,3*360)
	imqs=fltarr(NUMPIX,3*360)
	imus=fltarr(NUMPIX,3*360)

;
; Make scan go the proper direction
;
	!y.range=0
	a = where ( ang gt 200 )
	if ( a(0) ne -1 ) then ang(a) = ang(a)-360.
	if ( ang(0) gt 0 ) then begin
		imi = reverse(imi,2)
		imq = reverse(imq,2)
		imu = reverse(imu,2)
		ang = reverse(ang)
	endif

	;
;
; Spline fit to 1080 points
;
	newang = -180 + findgen(3*360)/3

	for ii=0,NUMPIX-1 do begin
		print,ii
		imis(ii,*) = spline(ang,imi(ii,*),newang)
		imqs(ii,*) = spline(ang,imq(ii,*),newang)
		imus(ii,*) = spline(ang,imu(ii,*),newang)
	endfor

;
; Offset between channel 4 and geocentric coordinates
; Shift from scan direction to scan direction should
; be inserted here
;
	shftpix = -30*(NUMAZ/360.)
	imis = shift (imis,0,shftpix)
	imqs = shift (imqs,0,shftpix)
	imus = shift (imus,0,shftpix)

;
; Look at resulting Q image
;
	wdelete,0
	window,0,xsize=3*360,ysize=NUMPIX
	tv,bytscl(transpose(imqs),-1000,4000)

;
; Write out I, Q, and U images
;
	str = string(filename+'_I')
	close,1 & openw,1,str & writeu,1,imis & close,1
	str = string(filename+'_Q')
	close,1 & openw,1,str & writeu,1,imqs & close,1
	str = string(filename+'_U')
	close,1 & openw,1,str & writeu,1,imus & close,1

nospline:

endwhile

stop

end
