PRO SPMAKEBUFRS, fmt, ibuf, obuf, dbuf, ifstat
; procedure to create the structures necessary for the reading and writing 
; of the data from the input raw unsorted files and the output of the 
; sorted files. The obuf is an intermediate step for accumulating a) only 
; those items desired (as outlined in fmt) and b) storing those items into
; the appropriate member of obuf (and thus not requiring a write-to-file
; for every frame found. dbuf is the decommutation buffer, outlining the
; bytes to be extracted from ibuf and stored in obuf.dat

nfmts = n_elements(fmt)
maxnbytes = 0

; set up the input buffer data structure
maxflen = max(fmt(*).flen)	; get the maximum format length in bytes
niframes = 6	; number of max fmt lengths to make the input stream buffer

ibuf = {ibufr, ibpos:0L, ibvalid:0L,ibnmax:maxflen*niframes, $
		ib:bytarr(maxflen*niframes), syncfnd:-1L, $
		tbrd: maxflen*niframes}

;	ibpos	lwInt	current position in the ib vector. During sync-search
;			this will be changing rapidly. Once a sync is found
;			this will be the index of the byte following
;			the sync pattern, the sub-vector of desired items 
;			is then	extracted and stored in the obuf.dat. ibpos
;			will then take on the value of the latest_sync_pos +
;			fmt.slen and another sync search will commence if 
;			the eofflag is not set and (ibvalid ne ibpos)
;	ibvalid	lwInt	number of bytes of valid data in ib: 
;			end of data read into the ib fron the input data
;			buffer. this will differ from the size of the ib
;			only when eofflag is set
;	ibnmax	lwInt	maximum buffer size for ib
;	ib	bytarr	input data buffer: raw unsorted data
;	syncfnd	lwInt	position in ib of the 'latest sync' found
;	tbrd	lwInt	number of bytes to be read into the ib on next
;			read cycle
;
; set up for the output buffers (really an intermediate form)
; find the maximum number of to-be-extracted bytes for any of the formats
for i = 0,nfmts-1 do begin
	nbytes = total(fmt(i).ilen(where(fmt(i).ilen gt 0)))
	if nbytes gt maxnbytes then maxnbytes = nbytes
endfor
naframes = 8192L	; number of frames (occurances of the format)
		; to accummulate in obufr before writing to 
		; the output files

obuf = replicate({obufr, ndat:0L , nmax:naframes, $
		dat:bytarr(maxnbytes,naframes)}, nfmts)
;	ndat	lwInt	number of format occurances found and stored in 
;			obuf(i).dat
;	nmax	lwInt	number of rows available to be filled
;			(max(ndat) = nmax)
;	dat	bytarr	contains array of bytes: rows will be the selected
;			items out of a particular occurance of the format and
;			columns will be the item stored (in byte form) for
;			different MET's. Not all columns will have valid 
;			data for all formats as the array is over-sized
;			for all but one fmt, the largest one.

; set up the localized input buffer indices for all formats eg
; (sync position + indices of desired bytes to be extracted from ib)

dbuf = replicate({dbufr, nind:0L, ind:lonarr(maxnbytes)-1L},nfmts)
;	nind	lwInt	number of bytes desired in format
;	ind	lonarr	localized indices in input stream for format
;			of the desired items. the 0-byte is taken to be the
;			1st byte following the format's sync pattern

; now to initialize the dbuf.ind & dbuf.nind
for i=0,nfmts-1 do begin
	synclen = n_elements(fmt(i).sync)
	dbuf(i).nind = total(fmt(i).ilen(where(fmt(i).ilen gt 0)))
	bbyte = 0
	for j=0,fmt(i).nitems-1 do begin
		dbuf(i).ind(bbyte:(bbyte+fmt(i).ilen(j)-1L)) = $
			fmt(i).ipos(j)+lindgen(fmt(i).ilen(j)) - synclen
		bbyte = bbyte+fmt(i).ilen(j)
	endfor
endfor

; input file status structure
ifstat = {ifstatus, eofflag:0}
;	
;	eofflag	Int	flag for eof detection on the input data stream file
;			will trigger a "sort ib" event, close input file,
;			open new input file (if available)

return
end
