/* cpcrop.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Squeeze/Stretch an SMM C/P image and display in a GL window.
 * Crop image to yield straight edges.
 * Save result as a sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 26 March 1998   Adapted from smmcp4.c.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/img_scale.h"
#include "/home/stanger/lib/include/cp_color.h"
char *malloc () ;
int pmode = 1 ;

main (argc, argv)
int argc ;
char *argv [] ;
{

   extern fmfonthandle gl_setfont () ;
   extern unsigned char *rdcp () ;
   extern unsigned char *sqimg () ; /* Squeeze/Stretch function. */
   extern int month_day () ;	/* Function to compute month & day from DOY. */
   extern char *month_name () ;	/* Function to look up ASCII month name. */

   static char namfw  [6][12] =
		       {"RED", "H-Alpha", "OPEN", "BLUE", "WIDEBAND", "GREEN"} ;
   static char namgls [4][2] = {"m", "a", "s", "f"} ;
   static char namsm  [9][12] = {"SOUTHWEST","SOUTH", "SOUTHEAST","EAST",
				 "NORTHEAST","NORTH", "NORTHWEST","WEST", "4Q"};
   static char afont [32]  = "Times-Bold" ;
   static char cpname [80] ;			/* Image file name. */
   static char label [20] ;			/* Label for images. */
   static char listname [80] ;			/* List file name. */
   static char month [4] ;
   static char namsrf   [80] = "smmcp.crop.sun" ;   /* Sun rasterfile name. */
   static char nampgm   [80] ;			/* Program name. */
   static char winname  [80] ;			/* Window name. */
   static int xtab [4] = {  0, 266,   0, 266} ;
   static int ytab [4] = {511, 511, 255, 255} ;
   static unsigned char red [256], green [256], blue [256] ;
   unsigned char  *cbuf ;
   unsigned short *sbuf ;
   unsigned char *imgbuf ;		/* Pointer to input image array. */
   unsigned char *imgout ;		/* Pointer to output image array. */
   char *parg ;			/* Argument pointer. */

   int asize = 12 ;
   int i, idot ;
   int loop ;
   int x = 63 ;
   int y = 510 ;
   int dx = 248 ;		/* Output image size (x-axis). */
   int dy = 248 ;		/* Output image size (y-axis). */
   int cx ;			/* Cropped output image size (x-axis). */
   int cy ;			/* Cropped output image size (y-axis). */
   int bx = 12 ;		/* Border size (x-axis). */
   int by = 12 ;		/* Border size (y-axis). */
   int cval1 = cp_grey ;
   int fd ;
   int gwid = -1 ;		/* SGI/GL window ID. */
   int imgsiz ;
   int imin =   0 ;
   int imax = 249 ;
   int ix, iy ;
   int lfw = 0;			/* =0 no fw display; =1 display filter name. */
   int lsm = 0;			/* =0 no sm display; =1 display sector name. */
   int npxwin ;
   int sy ;
   int xdim, ydim ;
   int xll, yll, xur, yur ;
   int year, doy, imonth, iday ;

   long offset ;
   float dmin =   0.0 ;
   float dmax = 255.0 ;
   float bmin, bmax, emin, emax ;

   fmfonthandle afh ;
   FILE *List ;

   strcpy (nampgm, argv [0]) ;

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   /*** Request information if not enough arguments are provided. ***/

   if ( argc < 2 ) { usage (nampgm) ; exit (-1) ; }

   while (--argc)
      {
      if (*(parg = *++argv) == '-')		/* Check for flag arguments */
	 {
	 switch (parg [1])
	    {
	    case 's':  lscale = 1 ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmin) ;
	               --argc ; parg = *++argv ; sscanf (parg, "%f", &foldmax) ;
		       break ;
            case 'f':  lfw = 1 ; 	/* Filter label. */
                       break ;
            case 'q':  lsm = 1 ; 	/* Sector Mirror (quadrant) label. */
                       break ;
            default:   fprintf (stderr," unknown flag %s\n", *argv) ;
		       usage (nampgm) ; exit (-2) ;
            }
         }
      else					/* Get the image filename. */
         {
         if (*parg == '?') { usage (nampgm) ; exit (-3) ; }
         strcpy (listname, parg) ; 
	 }
      }

   /*** Set intensity scaling values if lscale > 0. ***/

   if (lscale == 1)
      {
      oldmin =  (int) foldmin ;    oldmax  = (int) foldmax ;
      newmin =  (int) fnewmin ;    newmax  = (int) fnewmax ;
      clipmin = (int) fclipmin ;   clipmax = (int) fclipmax ;
      }

   /* Size of cropped output image. */

   cx = dx - bx * 2 ;
   cy = dy - by * 2 ;

   /* Initialize GL software. */

   sprintf (winname, "%s\n", nampgm) ;

   foreground () ;
   prefposition ( 20,  275,  20, 275) ;/* 256x256 window in lower left corner.*/
   fminit () ;                   /* Initialize SGI font manager. */
   gwid = winopen (winname) ;     /* Create GL window. */
   gl_wclear (BLACK) ;           /* Clear window to black. */
   afh = gl_setfont (afont, asize) ; /* Small font & size. */

   if ((List = fopen (listname, "r"))==NULL)
      {
      fprintf (stderr,"Cannot open %s\n", listname) ;
      exit (-1) ;
      }

   /* Read filenames (one per line). */

   loop = -1 ;
   while (fscanf (List,"%s", cpname) != EOF)
      {
      gl_wclear (BLACK) ;           /* Clear window to black. */
      idot = strlen (cpname) ;
      for (i = 0; i < strlen (cpname); i++)
	 {
	 if (cpname [i] == '.') idot = i ;
	 }
      strcpy (namsrf, cpname) ;
      strcat (namsrf, ".crop.sun") ;
      fprintf (stdout, "namsrf: %s\n", namsrf) ;

      loop++ ;
      if (loop > 3) loop = 0 ;

      /* Decommutate housekeeping. */

      cphskp (cpname) ;

      /*** Allocate memory for image array. ***/

      if (H.ires == 0)
      xdim = ydim = 512 ;
      else
      xdim = ydim = 256 ;

      /* Read image into memory. */

      imgbuf = rdcp (cpname) ;

      /* Rescale pixels if image is calibrated. */

      if (H.nbypix == 2)
         {
	 dmin =   0.0 ;
	 dmax = 500.0 ;
         if (H.ipw != 0)
	    {
	    dmin =   0.0 ;
	    dmax = 255.0 ;
	    }
         }

      set_scale (&bmin, &bmax, &emin, &emax) ;
      dmin = bmin ;
      dmax = bmax ;
      imin = (int) emin ;
      imax = (int) emax ;

      printf ("dmin/dmax: %8.2e %8.2e   imin/imax: %4d %4d\n",
      dmin, dmax, imin, imax) ;

      /* Squeeze/Stretch image. */

       imgout = sqimg (imgbuf, H.nbypix, npix, npix, dx, dy) ; 

      /* Display image in GL window. */

      x =   0 ;
      y = 255 ;
      gl_wbox (imgout, H.nbypix, x, y, dx, dy, "rd", pmode) ;
      gl_wimg (imgout, x, y, dx, dy, H.nbypix, dmin, dmax, imin, imax, pmode) ;

      /* Crop image to make edges square. */

      color (BLACK) ;
      xll = x ;       yll = y ;        xur = xll+dx-1 ;  yur = yll-by+1 ;
      rectfi (xll, yll, xur, yur) ;

      xll = x ;       yll = y -dy+1 ;  xur = xll+dx-1 ;  yur = yll+by-1 ;
      rectfi (xll, yll, xur, yur) ;

      xll = x ;       yll = y ;        xur = xll+bx-1 ;  yur = yll-dy+1 ;
      rectfi (xll, yll, xur, yur) ;

      xll = x+dx-1 ;  yll = y ;        xur = xll-bx+1 ;  yur = yll-dy+1 ;
      rectfi (xll, yll, xur, yur) ;

      /* Day and month. */

      year = H.year + 1900 ;
      doy = H.doy ;
      (void) month_day (year, doy, &imonth, &iday) ;
      strcpy (month, month_name (imonth)) ;

      /* Label image with date and time. */

      if (lfw == 1 && lsm == 0)
         sprintf (label, "%2d %3s %2d  %02d:%02d  %s filter",
                  iday, month, year, H.hour, H.minute, &namfw [H.ifw][0]) ;
      else
      if (lsm == 1 && lfw == 0)
         sprintf (label, "%2d %3s %2d  %02d:%02d  %s",
                  iday, month, year, H.hour, H.minute, &namsm [H.ism][0]) ;
      else
      if (lfw == 1 && lsm == 1)
         sprintf (label, "%2d %3s %2d  %02d:%02d  %s  %s",
                         iday, month, year, H.hour, H.minute,
		         &namfw [H.ifw][0], &namsm [H.ism][0]) ;
      else
         sprintf (label, "%2d %3s %2d  %02d:%02d UT",
                   iday, month, year, H.hour, H.minute) ;
      xll = x + bx ;
      yll = y - by - cy - 16 ;
      gl_wstr (label, xll, yll, cval1) ;

      /* Draw border around image. */

      gl_wvec (x+bx-1,  y-by+1,  x+bx+cx, y-by+1,  cval1) ;
      gl_wvec (x+bx+cx, y-by+1,  x+bx+cx, y-by-cy, cval1) ;
      gl_wvec (x+bx+cx, y-by-cy, x+bx-1 , y-by-cy, cval1) ;
      gl_wvec (x+bx-1,  y-by-cy, x+bx-1,  y-by+1,  cval1) ;

      free (imgout) ;
      free (imgbuf) ;

      npxwin  = xdim * ydim ;

      cbuf = (unsigned char  *)  malloc (npxwin * sizeof (unsigned char)) ;
      sbuf = (unsigned short *)  malloc (npxwin * sizeof (short)) ;

      /*** Copy image from frame buffer to memory. ***/

      xll = yll = 0 ;
      xur = xdim - 1 ;
      yur = ydim - 1 ;
      rectread (xll, yll, xur, yur, sbuf) ;

      /*** Invert image top to bottom & convert from short to char. ***/

      for (iy = 0 ; iy < ydim ;  iy++)
         {
         sy = ydim - iy - 1 ;
         for (ix = 0 ; ix < xdim ; ix++)
            *(cbuf + ix + iy*xdim) = (unsigned char) *(sbuf + ix + sy*xdim) ;
         }

      /*** Read color table. ***/

      get_scmap (512, 256, red, green, blue) ;

      /*** Create sun rasterfile and write to disk. ***/

      bytsrf (namsrf, cbuf, xdim, ydim, red, green, blue) ;
      free (cbuf) ;
      free (sbuf) ;
      }
/*
   winclose (gwid) ;
 */
   exit (0) ;
}

usage (nampgm)

char nampgm [] ;
{
   fprintf (stdout, "%s: Accepts SMM C/P low res images & resizes each\n",
	    nampgm) ;
   fprintf (stdout, "to fit into a 256x256 pixel window.  Each image is\n") ;
   fprintf (stdout, "labeled with date & time.\n") ;
   fprintf (stdout, "Filter name may also included.\n") ;
   fprintf (stdout, "Sector Mirror name may also included.\n") ;
   fprintf (stdout, "Result is saved as a Sun Rasterfile (smmcp4.sun),\n") ;
   fprintf (stdout, "using the current colormap set by the user.\n") ;
   fprintf (stdout, "\n") ;
   fprintf (stdout, "usage: %s [-f] [-q] [-s oldmin oldmax] file_list_name\n",
	    nampgm) ;

   fprintf (stdout, "-f                 Display filter name.\n") ;
   fprintf (stdout, "-q                 Display sector mirror name.\n") ;
   fprintf (stdout, "-s oldmin oldmax   Scaling min/max values.\n") ;
   fprintf (stdout, "file_list_name     Name of file containing a list of\n") ;
   fprintf (stdout, "                   Each image to be displayed in a\n") ;
   fprintf (stdout, "                   256x256 pixel window & saved as a\n") ;
   fprintf (stdout, "                   Sun rasterfile.\n") ;
   return (0) ;
}
