/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * cpglsrf2.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read C/P image, display in GL window & convert it to a sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 20 August 1992
 *  5 August 1996: Change output file name extension from "srf" to "sun".
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <fmclient.h>
#include "/home/stanger/lib/include/cplib.h"
#include "/home/stanger/lib/include/img_scale.h"
int DSP = 1;

main (argc, argv)

int argc;
char *argv [];

{
   extern char *malloc ();
   extern int glcp_view2 ();
   FILE *LIST;

   unsigned char *cbuf;
   unsigned short *sbuf;
   unsigned short smin, smax ;

   static unsigned char red [256], green [256], blue [256];
   static char namimg [80];		/* C/P image file name. */
   static char namlst [80];		/* C/P image namelist file name. */
   static char namopt [80] = "";	/* Command line option. */
   static char namsrf [80];		/* Sun rasterfile name. */

   int i;
   int idot;
   int ierr;			/* Error flag. */
   int imgcnt;
   int ix, iy;			/* X/Y indices. */
   int linit = 1;		/* Image label initialization flag. */
   int nbypx;			/* # bytes per pixel. */
   int nbyimg;
   int pmode = 0;		/* print mode for glcp_view2 (). */
   int seq = 0;			/* =0 single image; =1 sequence of images. */
   int sy;			/* Window y index. */
   int winsize;			/* Window size (short int units). */
   int xdim, ydim;		/* Window X/Y dimensions. */
   int xll, yll, xur, yur;	/* Lower left & upper right window coords. */

   long gwid;			/* GL window ID. */

   /*** Initialize intensity scaling variables. ***/

   foldmin  =   0.0 ;
   foldmax  =   0.0 ;
   fnewmin  =   0.0 ;
   fnewmax  = 250.0 ;
   fclipmin =   0.0 ;
   fclipmax = 249.0 ;

   if (argc == 2)		/* One argument: filename. */
      {
      strcpy (namimg, argv [1]);
      }
   else
   if (argc == 3)		/* Two arguments: option & filename. */
      {
      strcpy (namopt, argv [1]);
      strcpy (namimg, argv [2]);
      }
   else
      {
      fprintf (stderr, "usage: %s [-s] cp_image_file_name | @cp_filename_list\n",
               argv [0]);
      exit (-1);
      }

   /*** Request intensity scaling values if lscale > 0. ***/

   fclipmin =   0.0;
   fclipmax = 249.0;
   printf ("namopt: %s\n", namopt);
   if (strcmp (namopt, "-s") == 0)
      {
      lscale = 1;
      fprintf (stderr,
               "Enter oldmin, oldmax, newmin, newmax: ");
      fscanf  (stdin,  "%f %f %f %f",
               &foldmin, &foldmax, &fnewmin, &fnewmax);
      oldmin =  (int) foldmin;    oldmax  = (int) foldmax;
      newmin =  (int) fnewmin;    newmax  = (int) fnewmax;
      clipmin = (int) fclipmin;   clipmax = (int) fclipmax;
      }
        
   /*** If filename begins with '@', it contains a list of image names. ***/

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

   /*** Initialize GL software. ***/

   foreground ();
   fminit ();			/* Initialize SGI font manager. */
   imgcnt = 0;

loop:

   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }

   /*** Create Sun Rasterfile image name. ***/

/*
 *   idot = 0;
 *   for (i = 0; i < strlen (namimg); i++)
 *      {
 *      if (namimg [i] == '.')  idot = i;
 *      }
 *   strcpy (namsrf, namimg);
 *   if (idot > 0)
 *      namsrf [idot] = '\0';
 *   strcat (namsrf, ".sun");
 */

   sprintf (namsrf, "%s.sun", namimg) ;
   
   cphskp (namimg);			/* Decode SMM C/P image housekeeping. */

   /*** Open GL window.. ***/

   if (H.ires == 1)
   prefposition (760, 1271, 480, 991);/* 512x512 window in upper right corner.*/
   else
   prefposition (266, 1279,   0, 1023);/*1024x1024 window: upper right corner.*/

/*   if (imgcnt++ == 0) */
      {
      noborder () ;
      gwid = winopen (argv [0]);	/* Create GL window. */
      gl_wclear (BLACK);		/* Clear window to black. */
      }

   /*** Display image on IRIS screen in GL window. ***/

   fprintf (stdout, "C/P image: %s  Sun rasterfile: %s\n", namimg, namsrf);
   if ((ierr = glcp_view2 (namimg, linit, pmode)) < 0)
      {
      fprintf (stderr, "cpview error: %d\n", ierr);
      exit (-4);
      }

   /*** Allocate memory for image array. ***/

   if (H.ires == 0)
      xdim = ydim = 1024;
   else
      xdim = ydim = 512;

   nbypx = 1;
   nbyimg = xdim * ydim * nbypx;
   winsize = xdim * ydim * sizeof (short);

   cbuf = (unsigned char  *)  malloc (nbyimg);
   sbuf = (unsigned short *)  malloc (winsize);

   /*** Copy image from frame buffer to memory. ***/

   xll = yll = 0;
   xur = xdim - 1;
   yur = ydim - 1;
   rectread (xll, yll, xur, yur, sbuf);

   /*** Invert image top to bottom & convert from short to char. ***/

   smin = 32767 ;
   smax = 0 ;
   for (iy = 0; iy < ydim;  iy++)
      {
      sy = ydim - iy;
      if (*(sbuf + ix + sy * xdim) < smin) smin = *(sbuf + ix + sy * xdim) ;
      if (*(sbuf + ix + sy * xdim) > smax) smax = *(sbuf + ix + sy * xdim) ;
      for (ix = 0; ix < xdim; ix++)
         *(cbuf + ix + iy*xdim) = (unsigned char) *(sbuf + ix + sy*xdim);
      }

   fprintf (stdout, "smin: %d  smax: %d\n", smin, smax) ;

   /*** Read color table. ***/

   get_scmap (512, 256, red, green, blue);

   /*** Create sun rasterfile and write to disk. ***/

   bytsrf (namsrf, cbuf, xdim, ydim, red, green, blue);

   free (cbuf);
   free (sbuf);
   winclose (gwid);
   if (seq) goto loop;

quit:

   gexit ();
   return (0);
}
