/* dofits2srf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read FITS file, write Sun Rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 March 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <string.h>

#include "/home/stanger/lib/include/fitslib.h"

dofits2srf (namfit)

char namfit [] ;		/* FITS file name. */

{
   unsigned char *rdfits () ;
   static unsigned char red   [256] ;
   static unsigned char green [256] ;
   static unsigned char blue  [256] ;
   static char namsrf [84] ;			/* Sun rasterfile name. */
   unsigned char *imgbuf ;			/* Pointer to image array. */
   unsigned char *srfbuf ;			/* Pointer to image array. */
   char *ext ;					/* Extension pointer. */

   int gwid ;					/* GL window ID. */
   int ix, iy, ky ;
   int nbyimg ;

   /*** Create output file name. ***/

   printf ("namfit: %s\n", namfit) ;
   strcpy (namsrf, namfit) ;

   if ((ext = strstr (namsrf, ".fits")) != NULL && ext != namsrf)
      *ext = '\0' ;
   strcat (namsrf, ".sun") ;

   printf ("namsrf: %s\n", namsrf) ;

   /*** Read FITS HDU; read image into memory array. ***/

   if ((imgbuf = rdfits (namfit)) == NULL)
      {
      fprintf (stderr, "Unable to open FITS file: %s\n", namfit) ;
      exit (-1) ;
      }

   /*** Reverse image top-to-bottom. ***/
   /*** Sun rasterfile scans left-to-right, top-to-bottom. ***/
   /*** FITS      file scans left-to-right, bottom-to-top? ***/

   nbyimg = naxis1 * naxis2 * sizeof (unsigned char) ;
   srfbuf = (unsigned char *) malloc (nbyimg) ;
   for (iy = 0; iy < naxis2; iy++)
      {
      ky = naxis2 - iy - 1 ;
      for (ix = 0; ix < naxis1; ix++)
	 *(srfbuf + ix + iy*naxis1) = *(imgbuf + ix + ky*naxis1) ; 
      }

   /*** Read color table. ***/

   foreground () ;
   noport () ;
   gwid = winopen ("cmap") ;
   get_scmap (512, 256, red, green, blue) ;

   /*** Write Sun rasterfile. ***/

   bytsrf (namsrf, srfbuf, naxis1, naxis2, red, green, blue) ;

   free (imgbuf) ;
   free (srfbuf) ;
   winclose (gwid) ;

   return (0) ;
}
