/* expcon.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Expand/contract an image (of type unsigned short).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Written by Rose Reynolds
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
typedef unsigned short u_short;

expcon (a, dxa, dya, b, dxb, dyb, fx, fy, oax, oay, obx, oby)
u_short *a;		/* Input image. */
u_short *b;		/* Output image. */
int dxa, dya;		/* Size of input image. */
int dxb, dyb;		/* Size of output image. */
float fx, fy;		/* Magnification/minification factors.*/
float oax, oay;		/* Center for expansion/contraction in input image. */
float obx, oby;		/* Center for expansion/contraction in output image. */
{
   int bi, bj, Ai, Aj;
   register u_short *rb, *aj0, *aj1;
   float ai, aj, fdxa, fdya, factorx, factory, P1, P2, P3, P4, P5, P6, P7;

   fdxa = (float) dxa;  fdya = (float) dya;	/* Float variables: `a' size. */
   if ( fx < 0.0 )  fx = (float) dxb / fdxa;	/* X magnification. */
   if ( fy < 0.0 )  fy = (float) dyb / fdya;	/* Y magnification. */
   rb = b;					/* Pointer to array `b'. */
   factorx = oax - obx / fx;			/* X center offset. */
   factory = oay - oby / fy;			/* Y center offset. */

   for ( bj = 0; bj < dyb; bj++ )	/* Y-axis loop. */
      {
      aj = (float) bj / fy + factory;	/* Corresponding `a' Y coordinate. */
      if ( aj < 0.0 )			/* Left boundary case. */
	 aj = 0.0;
      else
      if ( aj > (fdya - 1.0) )		/* Right boundary case. */
	 aj = (fdya - 1.0);
      Aj = (int) aj;			/* Integer form of `aj'. */
      aj0 = a + dxa * Aj;		/* `a' array value at a [Aj]   [0]. */
      aj1 = aj0 + dxa;			/* `a' array value at a [Aj+1] [0]. */

      for ( bi = 0; bi < dxb; bi++ )	/* X-axis loop. */
	 {
         ai = (float) bi / fx + factorx;/* Corresponding `a' X coord. */
         if ( ai < 0.0 )		/* Bottom boundary case. */
	    ai = 0.0;
         else
	 if ( ai > (fdxa - 1.0) )	/* Top boundary case. */
	    ai = (fdxa - 1.0);
         Ai = (int) ai;			/* Integer form of `ai'. */
         P1 = (float) ( *(aj0 + Ai) );		/* a [Aj]  [Ai].   */
         P2 = (float) ( *(aj0 + Ai + 1) );	/* a [Aj]  [Ai+1]. */
         P3 = (float) ( *(aj1 + Ai) );		/* a [Aj+1][Ai].   */
         P4 = (float) ( *(aj1 + Ai + 1) );	/* a [Aj+1][Ai+1]. */

	 /*** Bilinear interpolation. ***/

         P5 = (ai - (float) Ai) * (P2 - P1) + P1; /* X-axis interp: row j. */
         P6 = (ai - (float) Ai) * (P4 - P3) + P3; /* X-axis interp: row j+1. */
         P7 = (aj - (float) Aj) * (P6 - P5) + P5; /* Y-axis interpolation. */
	 
         *rb++ = (u_short) (P7 + 0.5);	/* Convert float to short int. */
         }  /* end-for bi=0 to dxb */

      }  /* end-for bj=0 to dyb */

   return (0);
}
