/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * fbsrf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read byte image from SGI/WHIP frame buffer & convert it to a sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 26 May 1993
 *  6 April 1994: add switch statement for FB.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <whip.h>

main (argc, argv )

int argc;
char *argv [];
{
   extern char *malloc ();
   unsigned char *imgbuf;
   unsigned char *cbuf;
   unsigned short *sbuf;
   float *fbuf;
   static unsigned char red [256], green [256], blue [256];

   static char namsrf [80];
   int FB = 0;			/* Frame Buffer number. */
   int dx, dy, dz;		/* Frame Buffer size. */
   int fbd;			/* Frame Buffer descriptor. */
   int i;
   int imin, imax;
   int nbypx;			/* # bytes per pixel. */
   int nbyimg;
   int rel = 0;			/* relative flag for fb_setaddr. */
   int row;			/* row index counter. */
   int startx = 0;		/* X location for writing. */
   float frange;

   if (argc == 2)
      {
      FB = 0 ;
      strcpy (namsrf, argv [1]);
      }
   else
   if (argc == 3)
      {
      sscanf (argv [1], "%d", &FB) ;
      strcpy (namsrf, argv [2]);
      }
   else
      {
      fprintf (stderr, "usage: %s [FB] sun_rasterfile_name\n", argv [0]);
      exit (0) ;
      }

   /*** Fetch current frame buffer number. ***/

/*
   fbd = fb_getfb ();
   fb_getsize (fbd, &dx, &dy, &dz);
   nbypx = fb_bytes (fb_kind (FB));
 */
   switch (FB)
      {
      case  0:
      case  1:
      case  2:
      case  3:
	 dx = dy =  512 ;  nbypx = 1 ; break;
      case  4:
	 dx = dy =  512 ;  nbypx = 2 ; break ;
      case  5:
	 dx = dy =  512 ;  nbypx = 4 ; break ;
      case  6:
	 dx = dy =  512 ;  nbypx = 3 ; break ;
      case  7:
      case  8:
      case  9:
	 dx = dy =  512 ;  nbypx = 1 ; break ;
      case 10:
      case 11:
      case 12:
      case 13:
	 dx = dy = 1024 ;  nbypx = 1 ; break ;
      case 14:
	 dx = dy =  512 ;  nbypx = 2 ; break ;
      case 15:
	 dx = dy =  512 ;  nbypx = 4 ; break ;
      case 16:
	 dx = dy = 1024 ;  nbypx = 3 ; break ;
      case 17:
      case 18:
      case 19:
	 dx = dy = 1024 ;  nbypx = 1 ; break ;
      case 20:
	 dx = dy = 1024 ;  nbypx = 1 ; break ;
      case 21:
	 dx = dy = 1024 ;  nbypx = 2 ; break ;
      case 22:
	 dx = dy = 1024 ;  nbypx = 4 ; break ;
      case 23:
	 dx = dy = 1024 ;  nbypx = 3 ; break ;
      case 24:
      case 25:
      case 26:
	 dx = dy = 1024 ;  nbypx = 1 ; break ;
      case 27:
      case 28:
      case 29:
      case 30:
	 dx = dy =  512 ;  nbypx = 1 ; break ;
      case 31:
	 dx = dy =  512 ;  nbypx = 3 ; break ;
      case 32:
      case 33:
      case 34:
	 dx = dy =  512 ;  nbypx = 1 ; break ;
      case 35:
	 dx =  900 ;  dy =  750 ;  nbypx = 1 ; break ;
      default:
	 dx = dy =  512 ;  nbypx = 1 ; break ;
      }

   /*** Allocate memory for image array. ***/

   nbyimg = dx * dy * nbypx;
   imgbuf = (unsigned char *) malloc (nbyimg);
   switch (nbypx)
      {
      case 1: cbuf = (unsigned char *)  imgbuf;  break;
      default: fprintf (stderr, "Invalid FB kind; nbypx = %d\n", nbypx);
               exit (-1);
      }


   /*** Copy image from frame buffer to memory. ***/

   fb_open ();					/* Initialize WHIP software. */
   fb_setfb (FB);				/* Set FB. */
   fb_xstep ();					/* Auto increment in x axis. */

   for (row = 0; row < dy; row++)
      {
      fb_setaddr (FB, rel, 0, dy - row - 1, 0);	/* Set read address in FB. */
      fb_read (imgbuf + row*dx, dx);	/* Read one row from FB. */
/*      fb_getline (rel, dx, 0, row, 0, imgbuf+row*dx); */
      }
   fb_close ();					/* Terminate WHIP software. */

   /*** Read color table. ***/

   noport ();
   winopen (argv [0]);
   get_scmap (512, 256, red, green, blue);

   bytsrf (namsrf, imgbuf, dx, dy, red, green, blue);

   free (imgbuf);
   exit (0);
}
