/* gl_wbox1.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Move SMM C/P image from disc file to SGI IRIS GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 12 April 1991: Currently only supports dir2 = "rd" or "ur".
 * 26 April 1991: Add 512 offset for onemap mode.
 *  3 September 1992: add lutoff argument.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
typedef unsigned char u_char;
typedef unsigned short u_short;

gl_wbox1 (imgbuf, lutoff, nbypx, xs, ys, xdim, ydim, dir2, pmode)

u_char *imgbuf;			/* Image memory buffer. */
int lutoff;			/* LUT offset. */
int nbypx;			/* # bytes per pixel. */
int xs, ys;			/* Coordinates for image origin in FB. */
int xdim, ydim;			/* X & Y dimensions of image. */
char *dir2;			/* Direction to write image: l/r, u/d. */
int pmode;			/* Print mode. */
{
   extern char *malloc ();	/* Memory allocation. */
   char xdir, ydir;
   int bytpix = 2;		/* # bytes per pixel for output image. */
   int col;			/* Column index counter. */
   int i;			/* Loop index. */
   int ix, iy;
   int ymax, yhalf, ny;
   short sval;
   short soff = 512;		/* Add 512 to each pixel for onemap mode. */
   int nbyimg;			/* # bytes in output image. */
   int nelem;			/* # pixels in image. */
   int rel = 0;			/* Relative flag for fb_setaddr. */
   int row;			/* Row index counter. */
   int xll, yll;		/* Lower left corner. */
   int xur, yur;		/* Upper right corner. */
   int xsize, ysize, zsize;	/* Sizes for each dimension of FB. */
   int z = 0;
   u_char  *cimgbuf;
   u_short *simgbuf;
   u_short *sbuf;
   float *fimgbuf;

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
   soff = (short) lutoff;
   if (pmode)
      fprintf (stderr, "gl_wbox> nbypx: %d  xs: %d  ys: %d  xdim: %d  ydim: %d  dir2: %s\n",
               nbypx, xs, ys, xdim, ydim, dir2);
   sbuf = (u_short *) imgbuf;

/*
 *   fb_getsize (fb, &xsize, &ysize, &zsize);
 */

   /*** Determine storage orientation of image. ***/
   /*
    * GL uses an origin at lower left corner of frame buffer.
    * Therefore, convert any input image so that increasing X is to right,
    * and increasing Y is upward.
    */

   xdir = 'x';
   ydir = 'y';
   for (i=0; i<2; i++)
      switch (*(dir2 + i))
         {
         case 'l':
         case 'r': xdir = *(dir2 + i); break;
         case 'u':
         case 'd': ydir = *(dir2 + i); break;
         }
   if (xdir == 'x' || ydir == 'y')
      {
      fprintf (stderr, "***wboxfb> Illegal direction (%s).\n", dir2); 
      return (-1);
      }
   if (pmode)
      fprintf (stderr, "   xdir: %c   ydir: %c\n", xdir, ydir);

   nelem = xdim * ydim;
   nbyimg = nelem * sizeof (short);
   sbuf = (u_short *) malloc (nbyimg);
   cimgbuf = (u_char *) imgbuf;
   simgbuf = (u_short *) imgbuf;
   fimgbuf = (float *) imgbuf;

   /*** Copy image into temporary array. ***/

   if (nbypx == 1)
      for (i = 0; i < nelem; i++)
         *(sbuf + i) = (u_short) (*(cimgbuf + i) & 0377) + soff;
   else
   if (nbypx == 2)
      for (i = 0; i < nelem; i++)
         *(sbuf + i) = (u_short) (*(simgbuf + i) & 0377) + soff;
   else
   if (nbypx == 4)
      for (i = 0; i < nelem; i++)
         *(sbuf + i) = (u_short) *(fimgbuf + i) & 0377 + soff;

   /*** Manipulate image as needed for standard rectwrite orientation. ***/

   if (xdir == 'l' || xdir == 'r')	/* Horizontally-scanned image. */
      {
      if (xdir == 'l')			/* Image scanned right to left. */
         {
         xs -= (xdim - 1);
         swapcols (sbuf, bytpix, xdim, ydim, pmode); /* Reverse column order. */
         }
      if (ydir == 'd')			/* First scan line is at top of image.*/
         {
         ys -= (ydim - 1);
         swaprows (sbuf, bytpix, xdim, ydim, pmode);   /* Reverse row order. */
         }
      }

   else					/* Vertically-scanned image. */
      {
      if (xdir == 'd')			/* Image is scanned top to bottom. */
         {
         ys -= (ydim - 1);
         swaprows (sbuf, bytpix, xdim, ydim, pmode);   /* Reverse row order. */
         }
      if (ydir == 'l')			/* First column is at right of image. */
         {
         xs -= (xdim - 1);
         swapcols (sbuf, bytpix, xdim, ydim, pmode); /* Reverse column order. */
         }
      }

   /*** Write image to GL window. ***/

   xll = xs;
   yll = ys;
   xur = xll + xdim - 1;
   yur = yll + ydim - 1;
   if (pmode)
      fprintf (stderr,
              "   xll: %d   yll: %d   xur: %d   yur: %d\n", xll, yll, xur, yur);
   rectwrite (xll, yll, xur, yur, sbuf);

   free (sbuf);
   return (0);
}

/* swaprows
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse image top to bottom.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

swaprows (imgbuf, nbypx, xdim, ydim, pmode)

char *imgbuf;
int nbypx;
int xdim, ydim;
int pmode;

{
int ix;
int iy, ny;
int yhalf, ymax;
char cval;
short sval;
float fval;
short *simgbuf;
float *fimgbuf;

   if (pmode)
      fprintf (stderr,
           "   swaprows> nbypx: %d   xdim: %d   ydim: %d\n", nbypx, xdim, ydim);
   simgbuf = (short *) imgbuf;
   fimgbuf = (float *) imgbuf;

   /*** Flip image so that origin is lower left instead of upper left. ***/

   ymax = ydim - 1;
   yhalf = ydim / 2;

   if (nbypx == 1)				/* Byte image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xdim; ix++)
            {
            cval = *(imgbuf + ix + iy*xdim);
            *(imgbuf + ix + iy*xdim) = *(imgbuf + ix + ny*xdim);
            *(imgbuf + ix + ny*xdim) = cval;
            }
         }
      }
   else
   if (nbypx == 2)				/* Short integer image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xdim; ix++)
            {
            sval = *(simgbuf + ix + iy*xdim);
            *(simgbuf + ix + iy*xdim) = *(simgbuf + ix + ny*xdim);
            *(simgbuf + ix + ny*xdim) = sval;
            }
         }
      }

   else
   if (nbypx == 4)				/* Floating point image. */
      {
      for (iy = 0; iy < yhalf; iy++)
         {
         ny = ymax - iy; 
         for (ix = 0; ix < xdim; ix++)
            {
            fval = *(fimgbuf + ix + iy*xdim);
            *(fimgbuf + ix + iy*xdim) = *(fimgbuf + ix + ny*xdim);
            *(fimgbuf + ix + ny*xdim) = fval;
            }
         }
      }

   return (0);
}


/* swapcols
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Reverse image left to right.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

swapcols (imgbuf, nbypx, xdim, ydim, pmode)

char *imgbuf;
int nbypx;
int xdim, ydim;
int pmode;

{
int ix, nx;
int iy;
int xhalf, xmax;
char cval;
short sval;
float fval;
short *simgbuf;
float *fimgbuf;

   if (pmode)
      fprintf (stderr,
           "   swapcols> nbypx: %d   xdim: %d   ydim: %d\n", nbypx, xdim, ydim);
   simgbuf = (short *) imgbuf;
   fimgbuf = (float *) imgbuf;

   /*** Flip image left to right. ***/

   xmax = xdim - 1;
   xhalf = xdim / 2;

   if (nbypx == 1)				/* Byte image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ydim; iy++)
            {
            cval = *(imgbuf + ix + iy*xdim);
            *(imgbuf + ix + iy*xdim) = *(imgbuf + nx + iy*xdim);
            *(imgbuf + nx + iy*xdim) = cval;
            }
         }
      }

   else
   if (nbypx == 2)				/* Short image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ydim; iy++)
            {
            sval = *(simgbuf + ix + iy*xdim);
            *(simgbuf + ix + iy*xdim) = *(simgbuf + nx + iy*xdim);
            *(simgbuf + nx + iy*xdim) = sval;
            }
         }
      }

   else
   if (nbypx == 4)				/* Floating point image. */
      {
      for (ix = 0; ix < xhalf; ix++)
         {
         nx = xmax - ix; 
         for (iy = 0; iy < ydim; iy++)
            {
            fval = *(fimgbuf + ix + iy*xdim);
            *(fimgbuf + ix + iy*xdim) = *(fimgbuf + nx + iy*xdim);
            *(fimgbuf + nx + iy*xdim) = fval;
            }
         }
      }

   return (0);
}
