/* glcp_suncir.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C P _ S U N C I R ( )	- Draw a dotted circular pattern at 1.0 Rsun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 July 1990
 * 29 August 1990: make dots 2 pixels high.
 *  4 March 1991: make "dots" plus marks.
 *  4 April 1991: SGI/GL version.
 * 10 October 1995: xg computations should use gMINCOL, not gMINROW.
 *                  yg computations should use gMAXROW, not gMAXCOL.
 * 17 October 1995: replace xg with xw, yg with yw.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"

glcp_suncir (r1min, r1max, r1inc, ang1inc, r2min, r2max, r2inc, ang2inc, cindex)

int cindex;		/* Color LUT index. */

float r1min, r1max;	/* radial limits for major dotted lines. */
float r1inc;		/* radial increment for major dotted lines. */
float ang1inc;		/* angular increment between minor lines.  */
float r2min, r2max;	/* radial limits for minor dotted lines. */
float r2inc;		/* radial increment for minor dotted lines. */
float ang2inc;		/* angular increment between minor lines.  */
{
extern int nres;
float x, y;		/* (x,y) coordinates corresponding to (r,th). */
float r = 1.0;		/* Radius in R/Rsun units. */
float rmin ;
float th;		/* Position angle in degrees. */
int xw, yw;		/* Window coordinates. */
int xw1, yw1, xw2, yw2;

float radius;
float x1, y1, x2, y2;
float r1, r2;

   /*** Draw radial scans (r=r1min -> r1max) every ang1inc degrees. ***/

   for (th = 0.0; th < 360.0; th += ang1inc)
      {
      rmin = r1min ;
/*      if (th == 180.0)  rmin = 0.4;  else rmin = r1min ; */
      for (radius = rmin; radius <= r1max; radius += r1inc)
         {
	 rcoord (&radius, &th, &x, &y, 1, scroll);
         xw = ((x / nres) + 0.5) + gMINCOL;
         yw = gMAXROW - ((y / nres) + 0.5);
         if (xw >= gMINCOL && xw <= gMAXCOL &&
	     yw >= gMINROW && yw <= gMAXROW)
            {
            gl_wvec (xw-1, yw+1, xw+1, yw+1, cindex);
            gl_wvec (xw-1, yw,   xw+1, yw,   cindex);
            gl_wvec (xw-1, yw-1, xw+1, yw-1, cindex);
            }
	 }
      }

   /*** Draw radial scans (r=r2min -> r2max) every ang2inc degrees. ***/

   for (th = 0.0; th < 360.0; th += ang2inc)
      for (radius = r2min; radius < r2max; radius += r2inc)
         {
	 rcoord (&radius, &th, &x, &y, 1, scroll);
         xw = ((x / nres) + 0.5) + gMINCOL;
         yw = gMAXROW - ((y / nres) + 0.5);
         if (xw >= gMINCOL && xw <= gMAXCOL &&
	     yw >= gMINROW && yw <= gMAXROW)
            {
            gl_wvec (xw-1, yw+1, xw+1, yw+1, cindex);
            gl_wvec (xw-1, yw,   xw+1, yw,   cindex);
            gl_wvec (xw-1, yw-1, xw+1, yw-1, cindex);
            }
	 }
/*
 *   r1 = 0.12;
 *   r2 = 0.9;
 *   for (th = 0.0; th < 360.0; th += 90.0)
 *      {
 *      rcoord (&r1, &th, &x1, &y1, 1, scroll);
 *      rcoord (&r2, &th, &x2, &y2, 1, scroll);
 *      xw1 = ((x1 / nres) + 0.5) + gMINCOL;
 *      yw1 = gMAXROW - ((y1 / nres) + 0.5);
 *      xw2 = ((x2 / nres) + 0.5) + gMINCOL;
 *      yw2 = gMAXROW - ((y2 / nres) + 0.5);
 *      if (xw1 >= gMINCOL && xw1 <= gMAXCOL &&
 *          yw1 >= gMINROW && yw1 <= gMAXROW &&
 *          xw2 >= gMINCOL && xw2 <= gMAXCOL &&
 *          yw2 >= gMINROW && yw2 <= gMAXROW)
 *         {
 *         gl_wvec (xw1,   yw1-1, xw2,   yw2+1, cindex);
 *         gl_wvec (xw1-1, yw1,   xw2+1, yw2,   cindex);
 *         }
 *      }
 */

   /*** Draw sun circle at 1.0 Rsun.  One dot every 10 degrees. ***/

   radius = 1.0 ;
   for (th = 0; th < 360; th += 10)
      {
      rcoord (&radius, &th, &x, &y, 1, scroll);
      xw = ((x / nres) + 0.5) + gMINCOL;
      yw = gMAXROW - ((y / nres) + 0.5);
      if (xw >= gMINCOL && xw <= gMAXCOL &&
	  yw >= gMINROW && yw <= gMAXROW)
         {
         gl_wvec (xw-1, yw+1, xw+1, yw+1, cindex);
         gl_wvec (xw-1, yw,   xw+1, yw,   cindex);
         gl_wvec (xw-1, yw-1, xw+1, yw-1, cindex);
         }
      }

   return (0);
}
