/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * grn3srf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read 3 Grinnell images from files & convert it to a 24-bit sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  1 November 1990
 * 11 December 1991: add command line arguments.
 * 18 August 1993: adapt to accomodate 24/32-bit color (in addition to 8-bit).
 * 17 April 1995: adapted from 'rgbsrf.c'.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl.h>

main (argc, argv)

int argc ;
char *argv [] ;
{
extern char *malloc () ;
unsigned char *imgbuf ;
unsigned char *rbuf, *gbuf, *bbuf, *abuf ;
static unsigned char rlut [256], glut [256], blut [256] ;

static char nambyt1 [80] ;
static char nambyt2 [80] ;
static char nambyt3 [80] ;
static char nambyt4 [80] ;
static char namsrf [80] ;
int dx = 512, dy = 512 ;
int fd1, fd2, fd3, fd4 ;
int hdrsiz = 0 ;
int i ;
int nbyimg ;
int nbypx = 3 ;
int npximg ;
int nread ;

   if (argc == 5)
      {
      strcpy (nambyt1, argv [1]) ;
      strcpy (nambyt2, argv [2]) ;
      strcpy (nambyt3, argv [3]) ;
      strcpy (nambyt4, argv [4]) ;
      }
   if (argc == 4)
      {
      strcpy (nambyt1, argv [1]) ;
      strcpy (nambyt2, argv [2]) ;
      strcpy (nambyt3, argv [3]) ;
      }

   printf ("argc: %d\n", argc) ;

   nbypx = argc - 1 ;		/*** # bytes/pixel = # input images. ***/

   printf ("nambyt1/2/3: %s %s %s\n", nambyt1, nambyt2, nambyt3) ;

   if (nbypx == 3)
      strcpy (namsrf, "rgb.srf") ;
   else
      strcpy (namsrf, "rgba.srf") ;

   /*** Open input images. ***/

   if ((fd1 = open (nambyt1, 0)) <= 0)
      {
      printf ("fd1: %s\n", fd1) ;
      fprintf (stderr, "Unable to open file : %s\n", nambyt1) ;
      exit (-1) ;
      }
   lseek (fd1, (long) hdrsiz, 0) ;		/* Skip over image header. */

   if ((fd2 = open (nambyt2, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open file : %s\n", nambyt2) ;
      exit (-1) ;
      }
   lseek (fd2, (long) hdrsiz, 0) ;		/* Skip over image header. */

   if ((fd3 = open (nambyt3, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open file : %s\n", nambyt3) ;
      exit (-1) ;
      }
   lseek (fd3, (long) hdrsiz, 0) ;		/* Skip over image header. */

   if (nbypx == 4)
      {
      if ((fd4 = open (nambyt4, 0)) <= 0)
         {
         fprintf (stderr, "Unable to open file: %s\n", nambyt4) ;
         exit (-1) ;
         }
      lseek (fd4, (long) hdrsiz, 0) ;		/* Skip over image header. */
      }

   npximg = dx * dy ;
   nbyimg = npximg * nbypx ;

   /*** 24/32-bit color image. ***/

   rbuf = (unsigned char *) malloc (npximg) ;
   gbuf = (unsigned char *) malloc (npximg) ;
   bbuf = (unsigned char *) malloc (npximg) ;
   if (nbypx == 4)
      abuf = (unsigned char *) malloc (npximg) ;

   read (fd1, rbuf, npximg) ;		/* Red   channel. */
   read (fd2, gbuf, npximg) ;		/* Green channel. */
   read (fd3, bbuf, npximg) ;		/* Blue  channel. */
   if (nbypx == 4)
      read (fd4, abuf, npximg) ;		/* Alpha channel. */

   wrgbsrf (namsrf, dx, dy, nbypx, rbuf, gbuf, bbuf, abuf) ;

   free (rbuf) ;
   free (gbuf) ;
   free (bbuf) ;
   if (nbypx == 4)
      free (abuf) ;

   close (fd1) ;
   close (fd2) ;
   close (fd3) ;
   if (nbypx == 4)
      close (fd4) ;

   exit (0) ;
}
