/* srfsgi1k.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read Sun rasterfile image, load colormap to LUT, load image to FB.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 8 March 1991
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define RAS_MAGIC 0x59a66a95

main (argc, argv)

int argc;
char *argv [];
{
static char namsrf [80];
static int srfhdr [8];
static unsigned short int ired [256], igreen [256], iblue [256];
static char lutbuf [768];
static char imgbuf [262144];

int begindex = 512, numindex = 256;
int dxg = 1024, dyg = 1024;
int fd;
int i;
int ngr = 0, imgch = 0;
int nread;
int xs = 0, ys = 1023;

int magic;
int dxs, dys;
int bitpix;
int imgsiz, imgtyp;
int coltyp, colsiz;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   if (argc == 1)
      {
      fprintf (stderr, "Enter Sun rasterfile image name: ");
      fscanf  (stdin,  "%s", namsrf);
      }
   else
   if (argc == 2)
      strcpy (namsrf, argv [1]);

   /*** Read Sun rasterfile. ***/

   fd = open (namsrf, 0);
   nread = read (fd, srfhdr, 32);

   /*** Examine header. ***/

   magic  = srfhdr [0];
   dxs    = srfhdr [1];
   dys    = srfhdr [2];
   bitpix = srfhdr [3];
   imgsiz = srfhdr [4];
   imgtyp = srfhdr [5];
   coltyp = srfhdr [6];
   colsiz = srfhdr [7];

   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "Not a sun rasterfile.\n");
      exit (-2);
      }

   if (dxs > dxg || dys > dyg)
      {
      fprintf (stderr, "Invalid image size . . . dx: %3d . . . dy: %3d\n",
	       dxs, dys);
      exit (-3);
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz);
      exit (-4);
      }

   /*** Copy RGB lookup tables into unsigned short int arrays. ***/

   if (colsiz > 0)
      {
      nread = read (fd, lutbuf, 768);
      for (i=0; i<256; i++)
         {
         ired [i]   = (unsigned short int) (lutbuf [i]       & 0377);
         igreen [i] = (unsigned short int) (lutbuf [i + 256] & 0377);
         iblue [i]  = (unsigned short int) (lutbuf [i + 512] & 0377);
         }

      /*** Write color map to appropriate look-up tables. ***/

      loadlut (begindex, numindex, ired, igreen, iblue);
      }

   nread = read (fd, imgbuf, imgsiz);
/*
 *   for (i=0; i<512; i++)
 *      printf ("%d   %d\n", i, *(imgbuf + i + i*512));
 */
   close (fd);


   /*** Write image to IRIS frame buffer. ***/

   wboxb (imgbuf, xs, ys, dxs, dys, "rd");
}
