/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * rgbsrf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read byte image from file & convert it to a sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  1 November 1990
 * 11 December 1991: add command line arguments.
 * 18 August 1993: adapt to accomodate 24/32-bit color (in addition to 8-bit).
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl.h>

main (argc, argv)

int argc;
char *argv [];
{
extern char *malloc ();
unsigned char *imgbuf;
unsigned char *rbuf, *gbuf, *bbuf, *abuf;
static unsigned char rlut [256], glut [256], blut [256];

static char nambyt [80];
static char namsrf [80];
int dx, dy;
int fd;
int hdrsiz = 0;
int i;
int imin, imax;
int mode = 0;		/* packed mode; =0 interleaved, =1 serial channels. */
int nbyimg;
int nbypix;
int nbypx = 8;
int npximg;
int nread;
float frange;

   if (argc == 7)
      {
      strcpy (nambyt, argv [1]);
      sscanf (argv [2], "%d", &dx);
      sscanf (argv [3], "%d", &dy);
      sscanf (argv [4], "%d", &hdrsiz);
      sscanf (argv [5], "%d", &nbypx);
      sscanf (argv [6], "%d", &mode);
      }
   else
   if (argc == 6)
      {
      strcpy (nambyt, argv [1]);
      sscanf (argv [2], "%d", &dx);
      sscanf (argv [3], "%d", &dy);
      sscanf (argv [4], "%d", &hdrsiz);
      sscanf (argv [5], "%d", &nbypx);
      }
   else
   if (argc == 5)
      {
      strcpy (nambyt, argv [1]);
      sscanf (argv [2], "%d", &dx);
      sscanf (argv [3], "%d", &dy);
      sscanf (argv [4], "%d", &hdrsiz);
      }
   else
   if (argc == 4)
      {
      strcpy (nambyt, argv [1]);
      sscanf (argv [2], "%d", &dx);
      sscanf (argv [3], "%d", &dy);
      }
   else
      {
      fprintf (stderr, "usage: %s byte_image_name x_size y_size [hdr_siz [bytes_pixel mode]]\n",
	       argv [0]);
      exit (0);
      }

/*
      fprintf (stderr, "Enter name of input byte image: ");
      fscanf  (stdin,  "%s", nambyt);

      fprintf (stderr, "header size: ");
      fscanf  (stdin,  "%d", &hdrsiz);

      fprintf (stderr, "Enter size in x & y: ");
      fscanf  (stdin,  "%d,%d", &dx, &dy);

      fprintf (stderr, "Enter # bytes per pixel [1,3]: ");
      fscanf  (stdin,  "%d", &nbypx);
 */

   sprintf (namsrf, "%s.srf", nambyt);
   printf ("nambyt: %s   dx:%d   dy:%d   hdrsiz:%d   nbypx:%d\n",
	   nambyt, dx, dy, hdrsiz, nbypx);

   if ((fd = open (nambyt, 0)) <= 0)
      {
      fprintf (stderr, "Unable to open %s\n", nambyt);
      exit (-1);
      }

   lseek (fd, (long) hdrsiz, 0);		/* Skip over image header. */

   npximg = dx * dy;
   nbyimg = npximg * nbypx;
   imgbuf = (unsigned char *) malloc (nbyimg);

   /*** 8-bit color image. ***/

   if (nbypx == 1)
      {
      noport ();
      winopen (argv [0]);
      get_scmap (512, 256, rlut, glut, blut);	/* Read SGI color LUT. */
      nread = read (fd, imgbuf, nbyimg);
      bytsrf (namsrf, imgbuf, dx, dy, rlut, glut, blut);
      free (imgbuf);
      }

   /*** 24/32-bit color image. ***/

   if (nbypx == 3 || nbypx == 4)
      {
      if (nbypx == 4)
         abuf = (unsigned char *) malloc (npximg);
      else
	 abuf = (unsigned char *) malloc (4);
      rbuf = (unsigned char *) malloc (npximg);
      gbuf = (unsigned char *) malloc (npximg);
      bbuf = (unsigned char *) malloc (npximg);

      if (mode == 0)				/* Interleaved channels. */
	 {
         nread = read (fd, imgbuf, nbyimg);
	 if (nbypx == 3)
            for (i = 0; i < npximg; i++)
               {
	       *(bbuf + i) = *(imgbuf + i*3);		/* Blue channel.  */
	       *(gbuf + i) = *(imgbuf + i*3 + 1);	/* Green channel. */
	       *(rbuf + i) = *(imgbuf + i*2 + 2);	/* Red channel.   */
               }
	 else 
	 if (nbypx == 4)
            for (i = 0; i < npximg; i++)
               {
	       *(abuf + i) = *(imgbuf + i*4);		/* Alpha channel. */
	       *(rbuf + i) = *(imgbuf + i*4 + 1);	/* Blue channel.  */
	       *(gbuf + i) = *(imgbuf + i*4 + 2);	/* Green channel. */
	       *(bbuf + i) = *(imgbuf + i*4 + 3);	/* Red channel.   */
               }
         free (imgbuf);
	 }
      else
      if (mode == 1)
	 {
         if (nbypx == 4)
	    read (fd, abuf, npximg);		/* Alpha channel. */
         read (fd, bbuf, npximg);		/* Blue  channel. */
         read (fd, gbuf, npximg);		/* Green channel. */
         read (fd, rbuf, npximg);		/* Red   channel. */
         }
      wrgbsrf (namsrf, dx, dy, nbypx, rbuf, gbuf, bbuf, abuf);
      free (rbuf);
      free (gbuf);
      free (bbuf);
      if (nbypx == 4)
         free (abuf);
      }

   exit (0);
}
