/* set_scale.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * For SMM C/P images.
 * Set dmin & dmax values, based upon image type.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   14 August 1997
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"
#include "/home/stanger/lib/include/img_scale.ext"

set_scale (begmin, begmax, endmin, endmax)

float *begmin, *begmax;
float *endmin, *endmax;
{
   float min1, max1 ;
   float min2, max2 ;
   
   printf ("set_scale> lscale: %d\n", lscale) ;
   printf ("set_scale> foldmin/foldmax: %8.2e %8.2e\n", foldmin, foldmax) ;
   printf ("set_scale> fnewmin/fnewmax: %8.2e %8.2e\n", fnewmin, fnewmax) ;
   printf ("set_scale> idtype: %d\n", idtype) ;

   if ( H.nbypix == 1 )		/* 1 byte per pixel. */
      {
      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 min1 = foldmin ;
	 max1 = foldmax ;
	 min2 = newmin ;
	 max2 = newmax ;
	 }
      else
         {
	 min1 =   0.0 ;
	 max1 = 250.0 ;
	 min2 =   0 ;
	 max2 = 250 ;
         }
      }

   else if (H.nbypix == 2)	/* 2 bytes per pixel (short integer). */
      {
      if (lscale == 1)		/* User-supplied scaling parameters. */
	 {
	 min1 = foldmin ;
	 max1 = foldmax ;
	 min2 = newmin ;
	 max2 = newmax ;
	 }
      else
         if (H.ipw != 0)	/* Polaroid filter. */
         {
	 min1 =   0.0 ;
	 max1 = 200.0 ;
	 min2 =   0 ;
	 max2 = 250 ;
         }
      else			/* Default short integer intensity scaling. */
         {
	 min1 =   0.0 ;
	 max1 = 500.0 ;
	 min2 =   0 ;
	 max2 = 250 ;
         }
      }

   else if (H.nbypix == 4)	/* 4 bytes per pixel (floating point). */
      {
      if (lscale == 1)
	 {
	 min1 = foldmin ;
	 max1 = foldmax ;
	 min2 = newmin ;
	 max2 = newmax ;
	 }
      else
	 {
	 if (idtype == 4)  {min1 =  0.0 ;    max1 = 5.e+15 ;}/* elec/cm2 */
	 if (idtype == 5)  {min1 = -2.0e-9 ; max1 = 2.e-9 ;} /* B1 - B2 */
	 if (idtype == 6)  {min1 =  0.0 ;    max1 = 3.e-9 ;} /* Bk */
	 if (idtype == 7)  {min1 = -1.0 ;    max1 = 1.0 ;}  /* Bcme-Bbkg/Bk */
	 if (idtype == 8)  {min1 = -1.0 ;    max1 = 1.0 ;}  /* Bcme-Bbkg/Bbkg */
	 if (idtype == 9)  {min1 =  0.0 ;    max1 = 1.0 ;}  /* polarization */
	 if (idtype == 10) {min1 =  0.0 ;    max1 = 5.e-9 ;} /* pB */
	 if (idtype == 11) {min1 =  0.0 ;    max1 = 5.e-9 ;} /* Br+t */
	 if (idtype == 12) {min1 =  0.0 ;    max1 = 5.e-9 ;} /* Bt */
	 if (idtype == 13) {min1 =  0.0 ;    max1 = 5.e-9 ;} /* Br */
	 if (idtype == 14) {min1 =  0.0 ;    max1 = 2.5e-9 ;}/* Bf + Bs */
	 if (idtype == 15) {min1 =  0.0 ;    max1 = 2.0 ;}   /* B1 / B2 */
	 if (idtype == 16) {min1 =  0.0 ;    max1 = 2e-9 ;}  /* B misc */
	 min2 =   0 ;
	 max2 = 250 ;
	 }
      }

   else
      {
      fprintf (stderr, "glcp_view> Invalid image . . . nbypix: %d\n", nbypix) ;
      return (-4) ;
      }

   *begmin = min1 ;
   *begmax = max1 ;
   *endmin = min2 ;
   *endmax = max2 ;

   return (0) ;
}
