/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * srf3srf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read 3 sun rasterfile byte images & convert it to a 24-bit sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  1 November 1990
 * 11 December 1991: add command line arguments.
 * 18 August 1993: adapt to accomodate 24/32-bit color (in addition to 8-bit).
 * 18 April 1995: adapted from 'rgbsrf.c'
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl.h>
#include "rasterfile.h"

struct rasterfile rss1, rss2, rss3 ;	/* Define rasterfile structure. */
struct rasterfile *rs1, *rs2, *rs3 ;

main (argc, argv)

int argc ;
char *argv [] ;
{
extern char *malloc () ;
extern unsigned char *readsrf () ;
unsigned char *rbuf, *gbuf, *bbuf ;
static unsigned char abuf [4] ;
static unsigned char rlut [256], glut [256], blut [256] ;
static char namsrf1 [80] ;
static char namsrf2 [80] ;
static char namsrf3 [80] ;
static char namrgb  [80] ;
int nbypx ;
int nread1, nread2, nread3 ;

   if (argc == 5)
      {
      strcpy (namsrf1, argv [1]) ;
      strcpy (namsrf2, argv [2]) ;
      strcpy (namsrf3, argv [3]) ;
      strcpy (namrgb,  argv [4]) ;
      }
   else
   if (argc == 4)
      {
      strcpy (namsrf1, argv [1]) ;
      strcpy (namsrf2, argv [2]) ;
      strcpy (namsrf3, argv [3]) ;
      sprintf (namrgb, "%rgb.srf") ;
      }
   else
      {
      fprintf (stderr, "usage: %s namsrf1 namsrf2 namsrf3 nameRGB\n", argv [0]) ;
      exit (0) ;
      }

   /*** Open input sun rasterfiles. ***/

   rs1 = &rss1 ;
   rs2 = &rss2 ;
   rs3 = &rss3 ;

   if ((rbuf = readsrf (namsrf1, rs1, rlut, glut, blut)) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namsrf1) ;
      exit (-1) ;
      }

   if ((gbuf = readsrf (namsrf2, rs2, rlut, glut, blut)) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namsrf2) ;
      exit (-2) ;
      }

   if ((bbuf = readsrf (namsrf3, rs3, rlut, glut, blut)) == NULL)
      {
      fprintf (stderr, "Unable to open file : %s\n", namsrf3) ;
      exit (-3) ;
      }

   /*** Verify that widths, heights, and depths all agree. ***/

   if (rs1->ras_width != rs2->ras_width ||
       rs1->ras_width != rs3->ras_width)
      {
      fprintf (stderr, "Image widths not equal : %d %d %d\n",
	       rs1->ras_width, rs2->ras_width, rs3->ras_width) ;
      exit (-4) ;
      }

   if (rs1->ras_height != rs2->ras_height ||
       rs1->ras_height != rs3->ras_height)
      {
      fprintf (stderr, "Image heights not equal : %d %d %d\n",
	       rs1->ras_height, rs2->ras_height, rs3->ras_height) ;
      exit (-5) ;
      }

   if (rs1->ras_depth != rs2->ras_depth ||
       rs1->ras_depth != rs3->ras_depth)
      {
      fprintf (stderr, "Image depths not equal : %d %d %d\n",
	       rs1->ras_depth, rs2->ras_depth, rs3->ras_depth) ;
      exit (-6) ;
      }

   if (rs1->ras_depth != 8 && rs2->ras_depth != 8 && rs3->ras_depth != 8)
      {
      fprintf (stderr, "Image depths not 8 bits : %d %d %d\n",
	       rs1->ras_depth, rs2->ras_depth, rs3->ras_depth) ;
      exit (-7) ;
      }

   /*** Create 24-bit color image. ***/

   nbypx = 3 ;
   wrgbsrf (namrgb, rs1->ras_width, rs1->ras_height, nbypx,
	    rbuf, gbuf, bbuf, abuf) ;

   free (rbuf) ;
   free (gbuf) ;
   free (bbuf) ;

   exit (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * srfopen : open sun rasterfile.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
struct rasterfile rss ;		/* Define rasterfile structure. */

char *
readsrf (namfil, rs, rlut, glut, blut)

char *namfil ;
struct rasterfile *rs ;			/* Define rasterfile structure. */
unsigned char rlut [] ;			/* Red   look-up table. */
unsigned char glut [] ;			/* Green look-up table. */
unsigned char blut [] ;			/* Blue  look-up table. */

{
   extern char *malloc () ;
   FILE *FD ;
   unsigned char *cbuf ;
   int nbyimg ;
   int ncm ;
   int nread ;
   int rslen ;

   /*** Define sun rasterfile structure. ***/

/*
 *   rs = &rss ;
 */
   rslen = sizeof (rss) ;		/* Size of rasterfile structure. */
    
   if ((FD = fopen (namfil, "r")) == NULL)	/* Open file. */
      {
      fprintf (stderr, "Unable to open file: %s\n", namfil) ;
      return (NULL) ;
      }

   nread = fread ((char *) rs, rslen, 1, FD) ;	/* Read rasterfile header. */

   if (rs->ras_magic != RAS_MAGIC)
      {
      fprintf (stderr, "file %s is NOT a sun rasterfile!\n", namfil) ;
      return (NULL) ;
      }

   /*** Allocate storage. ***/

   ncm = rs->ras_maplength / 3 ;
   nbyimg = rs->ras_width * rs->ras_height * rs->ras_depth / 8 ;
   cbuf = malloc (nbyimg) ;

   /*** Read color look-up tables. ***/

   nread = fread (rlut, 1, ncm, FD) ;
   nread = fread (glut, 1, ncm, FD) ;
   nread = fread (blut, 1, ncm, FD) ;

   /*** Read image. ***/

   nread = fread (cbuf, 1,  nbyimg, FD) ;

   fclose (FD) ;

   return (cbuf) ;
}
