/* srffb.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read Sun rasterfile image, load colormap to LUT, load image to FB.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 8 March 1991
 * 12 December 1991: automatically select FB based upon image size.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <whip.h>
#define RAS_MAGIC 0x59a66a95

main (argc, argv)

int argc;
char *argv [];
{
   extern char *malloc ();
   FILE *LIST;
   static char namlst [80];
   static char namsrf [80];
   static char namfb [] = "srf";
   static int srfhdr [8];
   static unsigned short int ired [256], igreen [256], iblue [256];
   static char lutbuf [768];
   char *imgbuf, *ibuf, *linbuf;

   int begindex = 512, numindex = 256;
   int colmax = 0;
   int dxg = 2048, dyg = 2048;
   int dzs = 0;
   int fb = 0;				/* Frame buffer number. */
   int fbkind = BYTE;
   int fbsize;				/* FB size. */
   int fbtype = FB_SHMEM;
   int fd;				/* Image file descriptor. */
   int i;
   int ians;			/* Holds keyboard character input. */
   int maxdim;
   int nbypix;				/* # bytes per pixel. */
   int nbylini, nbylinf;		/* # bytes per line (image, file). */
   int ngr = 0, imgch = 0;
   int nread;
   int nxs;				/* # pixels per line in file. */
   int seq;				/* sequence ?  =0 no, =1 yes. */
   int xs, ys;

   int magic;
   int dxs, dys;
   int bitpix;
   int imgsiz, imgtyp;
   int coltyp, colsiz;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   if (argc == 2)
      {
      strcpy (namsrf, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s sun_rasterfile_name | @filename_list\n",
               argv [0]);
      exit (0);
      }

   if (namsrf [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namsrf [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      fprintf (stderr, "Enter `q <return>' to quit, `<return>' to continue.\n");
      }
   else
      seq = 0;


   /*** Set up GL in order to rewrite color map. ***/

   noport ();
   foreground ();
   winopen ("map");

loop:

   if (seq)
      if (fscanf (LIST, "%s", namsrf) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }

   /*** Read Sun rasterfile. ***/

   fd = open (namsrf, 0);
   nread = read (fd, srfhdr, 32);

   /*** Examine header. ***/

   magic  = srfhdr [0];
   dxs    = srfhdr [1];
   dys    = srfhdr [2];
   bitpix = srfhdr [3];
   imgsiz = srfhdr [4];
   imgtyp = srfhdr [5];
   coltyp = srfhdr [6];
   colsiz = srfhdr [7];

   nbypix = bitpix / 8 ;
   printf ("dxs: %3d  dys: %3d  bitpix: %3d  imgsiz: %d\n",
	  dxs, dys, bitpix, imgsiz) ;
   nxs = imgsiz / (dys * nbypix) ;
   nbylini = dxs * nbypix ;
   nbylinf = nxs * nbypix ;
   printf ("nxs: %3d  nbylinf: %4d  nbylini: %4d\n", nxs, nbylinf, nbylini) ;

   xs = 0;
   ys = dys - 1;
   fbsize = dxs * dys;
/*   fprintf (stderr, "fbsize: %d\n", fbsize); */

   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "Not a sun rasterfile.\n");
      exit (-2);
      }

   if (dxs > dxg || dys > dyg)
      {
      fprintf (stderr, "Invalid image size . . . dx: %3d . . . dy: %3d\n",
	       dxs, dys);
      exit (-3);
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz);
      exit (-4);
      }

   /*** Copy RGB lookup tables into unsigned short int arrays. ***/

   if (colsiz > 0)
      {
      nread = read (fd, lutbuf, colsiz);
      colmax = colsiz / 3;
      for (i = 0; i < colmax; i++)
         {
         ired [i]   = (unsigned short int) (lutbuf [i]            & 0377);
         igreen [i] = (unsigned short int) (lutbuf [i + colmax]   & 0377);
         iblue [i]  = (unsigned short int) (lutbuf [i + colmax*2] & 0377);
         }

      /*** Write color map to appropriate look-up tables. ***/

      loadcm (begindex, numindex, ired, igreen, iblue);
      }

   /*** Read image from file into memory. ***/

   imgbuf = malloc (fbsize);
   linbuf = malloc (nbylinf) ;
   ibuf = imgbuf ;
   for (i = 0; i < dys; i++)
      {
      nread = read (fd, linbuf, nbylinf);
      bcopy (linbuf, ibuf, nbylini) ;
      ibuf += dxs ;
      }
   close (fd);

   if (dxs > dys)  maxdim = dxs;  else maxdim = dys;
   if (maxdim <= 512)  fb = 0;
   else
   if (maxdim <= 1024)  fb = 10;
   else
   if (maxdim <= 2048)  fb = 20;
/*   fprintf (stderr, "fb: %d\n", fb); */

   /*** Create WHIP frame buffer which is compatible with image. ***/

   fb_open ();

   /*** Write image to IRIS frame buffer. ***/

   wimgfb (fb, imgbuf, xs, ys, dxs, dys);

   free (imgbuf);
   fb_close ();

   if (seq)
      {
      fprintf (stderr, "%s . . . ", namsrf);
      ians = getchar ();
      if (ians == 0161)
         goto quit;		/* q <return> */
      else
         goto loop;
      }
quit:

   gexit ();
   exit (0);
}
