/* srfinfo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read Sun rasterfile header and print information on screen.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR    5 August 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#define RAS_MAGIC 0x59a66a95

main (argc, argv)

int argc ;
char *argv [] ;
{
   FILE *LIST ;
   static char namlst [80] ;
   static char namsrf [80] ;
   static char rastyp [6][15] = {"RT_OLD", "RT_STANDARD", "RT_BYTE_ENCODED",
				 "RT_FORMAT_RGB", "RT_FORMAT_TIFF",
				 "RT_FORMAT_IFF"};
   static char maptyp [3][16] = {"RMT_NONE", "RMT_EQUAL_RGB", "RMT_RAW"} ;

   static int  srfhdr [8] ;

   int fd ;			/* Image file descriptor. */
   int i ;
   int argp = 0 ;		/* argument pointer. */
   int ians ;			/* Holds keyboard character input. */
   int nread ;
   int seq ;			/* sequence ?  =0 no, =1 yes. */

   int magic ;
   int dxs, dys ;
   int bitpix ;
   int imgsiz, imgtyp ;
   int coltyp, colsiz ;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   if (argc >= 2)
      {
      strcpy (namsrf, argv [1]) ;
      }
   else
      {
      fprintf (stderr, "usage: %s rasterfile1 [rasterfile2 ...] | @filename_list\n",
               argv [0]) ;
      exit (0) ;
      }

   if (namsrf [0] == '@')
      {
      seq = 1 ;
      strcpy (namlst, &namsrf [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst) ;
         exit (-1) ;
         }
      fprintf (stderr, "Enter `q <return>' to quit, `<return>' to continue.\n") ;
      }
   else
   if (argc > 2)
      seq = 2 ;
   else
      seq = 0 ;

loop:

   if (seq == 1)
      {
      if (fscanf (LIST, "%s", namsrf) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst) ;
         goto quit ;
         }
      }
   else
   if (seq == 2)
      {
      if (++argp >= argc)
	 goto quit ;
      sscanf (argv [argp], "%s", namsrf) ;
      }

   /*** Read Sun rasterfile. ***/

   fd = open (namsrf, 0) ;
   nread = read (fd, srfhdr, 32) ;
   close (fd) ;

   /*** Examine header. ***/

   magic  = srfhdr [0] ;
   dxs    = srfhdr [1] ;
   dys    = srfhdr [2] ;
   bitpix = srfhdr [3] ;
   imgsiz = srfhdr [4] ;
   imgtyp = srfhdr [5] ;
   coltyp = srfhdr [6] ;
   colsiz = srfhdr [7] ;

   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "Not a sun rasterfile.\n") ;
      exit (-2) ;
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz) ;
      exit (-4) ;
      }

   fprintf (stdout, "%s   Sun Rasterfile header information:\n", namsrf) ;
   fprintf (stdout, "width: %3d   height: %3d   depth [bits]: %3d\n",
		     srfhdr [1], srfhdr [2], srfhdr [3]) ;
   fprintf (stdout, "length [bytes]: %6d   type: %s\n",
		     srfhdr [4], &rastyp [srfhdr [5]][0]) ;
   fprintf (stdout, "maptype: %s   maplength: %3d   [=%3d R/G/B]\n",
		    &maptyp [srfhdr [6]][0], srfhdr [7], srfhdr [7] / 3) ;
   fprintf (stdout, "\n") ;

   if (seq)
      goto loop ;
quit:

   exit (0) ;
}
