/* srfw.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a sequence Sun rasterfile images,
 * load colormap to LUT, load image to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Supports only 8-bit greyscale, 8-bit colormap, and 24-bit RGB image types.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 28 January 92
 * 20 August 92: add ability to display a sequence of images using keyboard.
 *  3 September 92: add forward/backward control.
 *                  add option to use selectable colormap.
 * 14 October 92: use mouse buttons to go forward or backward ; use ESC to quit.
 * 17 March 1994: close old window if size changes.
 * 16 May 1994: compute nx: # bytes in x-dimension of sun rasterfile image.
 *  3 March 1995: add ability to accept wild card (*) in input filename.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <gl/gl.h>
#include <gl/device.h>
#include "/home/riker/stanger/lib/include/rasterfile.h"
#define MAXFILES 250
extern char *malloc () ;

main (argc, argv)

int argc ;
char *argv [] ;
{
   extern int gl_wbox1 () ;		/* Write colormap image to GL window. */
   struct stat *fs ;			/* Pointer to stat structure. */
   struct stat FS ;			/* Pointer to stat structure. */
   FILE *LIST ;
   short val ;

   int gl_wrgb () ;			/* Write RGB image to GL window. */

   static char dir2 [4] = "rd" ;
   static char namlst [80] ;
   static char namsrf [80] ;
   static char lutbuf [768] ;

   static char flist [MAXFILES][80] ;
   unsigned char *imgbuf ;
   unsigned char *srfbuf ;
   static unsigned short int sred [256], sgreen [256], sblue [256] ;

   static int srfhdr [8] ;

   int argp = 1 ;
   int begindex = 512, numindex = 256 ;
   int colmax = 0 ;
   int dxg = 1280, dyg = 1024 ;
   int dzs = 0 ;
   int fd ;				/* Image file descriptor. */
   int findex = 0 ;
   int gwid = -1 ;			/* SGI/GL window ID. */
   int i ;
   int ix, iy ;
   int ians ;				/* KB response. */
   int imgsize ;
   int lutnum = 2 ;			/* LUT number. */
   int lutoff = 512 ;			/* LUT offset. */
   int maxdim ;
   int nbypx ;
   int nf = 0 ;				/* File index. */
   int ngr = 0, imgch = 0 ;
   int npximg ;
   int nread ;
   int numfil ;				/* # images in file list. */
   int prevdxs = 0, prevdys = 0 ;	/* Previous image size. */
   int seq = 0 ;
   int xs, ys ;

   int magic ;
   int dxs, dys ;
   int bitpix ;
   int hdrsiz = 32 ;			/* Size of sun rasterfile header. */
   int imgsiz, imgtyp ;
   int coltyp, colsiz ;
   int nxf;			/* # bytes in x-dimension of sun rasterfile. */
   int nxs ;			/* # bytes/line. */
   int xll, yll ;
   int xur, yur ;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   fs = &FS ;

   /*** Get sun rasterfile from command line. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s [-lutnum] sun_rasterfile_name[s] | @namelist | ^namelist\n",
               argv [0]) ;
      exit (0) ;
      }
   else
   if (argc > MAXFILES)
      {
      fprintf (stderr, "Maximum number of input files: %d\n", MAXFILES) ;
      exit (-1) ;
      }
   else
   if (argc == 2)
      {
      strcpy (namsrf, argv [1]) ;
      }
   else
   if (argc >= 3)
      {
      if (*argv [argp] == '-')
	 {
         sscanf (argv [argp], "%d", &lutnum) ;
	 lutnum = - lutnum ;
	 argp++ ;
	 }
      nf  = 0 ;
      seq = 2 ;
      while (argp < argc)
	 {
         strcpy (&flist [nf++][0], argv [argp]) ;
	 argp++ ;
	 }
      numfil = nf ;
      }

   lutoff = lutnum * 256 ;
   begindex = lutoff ;

   if (namsrf [0] == '@' || namsrf [0] == '^')
      {
      if (namsrf [0] == '@')
         seq = 1 ;
      else
      if (namsrf [0] == '^')
         seq = 2 ;

      strcpy (namlst, &namsrf [1]) ;
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open list file %s\n", namlst) ;
         exit (-2) ;
         }
      i = 0 ;
      while (fscanf (LIST, "%s", namsrf) != EOF)
         {
         strcpy (&flist [i++][0], namsrf) ;
         }
      numfil = i ;
      fclose (LIST) ;
      }

   if (seq)
      {
      fprintf (stderr, "After each image is displayed,\n") ;
      fprintf (stderr, "position cursor within %s window.\n", argv [0]) ;
      fprintf (stderr, "click `left  mouse' to go forward.\n") ;
      fprintf (stderr, "click `right mouse' to go backward.\n") ;
      fprintf (stderr, "type  `ESC'         to quit.\n") ;
      }
   else
      {
      fprintf (stderr, "position cursor within %s window.\n", argv [0]) ;
      fprintf (stderr, "type  `ESC'         to quit.\n") ;
      }

   /*** Initialize GL software. ***/

   foreground () ;

loop:

   if (seq)
      strcpy (namsrf, &flist [findex][0]) ;

   /*** Read Sun rasterfile header. ***/

   if ((fd = open (namsrf, 0)) < 0)
      {
      fprintf (stderr, "Unable to open image file: %s\n", namsrf) ;
      exit (-3) ;
      }
   fstat (fd, fs) ;			/* Get file structure 'stat' (fs). */
   nread = read (fd, srfhdr, hdrsiz) ;	/* Read sun rasterfile header. */

   /*** Decode Sun rasterfile header. ***/

   magic  = srfhdr [0] ;		/* Sun rasterfile magic number. */
   dxs    = srfhdr [1] ;		/* Image width (pixels). */
   dys    = srfhdr [2] ;		/* Image height (pixels). */
   bitpix = srfhdr [3] ;		/* Image depth (bits/pixel). */
   imgsiz = srfhdr [4] ;		/* Image size (bytes). */
   imgtyp = srfhdr [5] ;		/* Image type. */
   coltyp = srfhdr [6] ;		/* Colormap type. */
   colsiz = srfhdr [7] ;		/* Colormap size (bytes). */

   xs = 0 ;
   ys = dys - 1 ;
   nbypx = bitpix / 8 ;
   npximg = dxs * dys ;
   nxs = dxs * nbypx ;
   imgsiz = fs->st_size - hdrsiz  - colsiz ;	/* Actual size of image. */
   nxf = imgsiz / dys ;
/*   if (imgsiz > (fs->st_size - hdrsiz))  imgsiz = fs->st_size - hdrsiz ; */
   imgsize = npximg * nbypx ;

/*
 *   fprintf (stdout, "dxs: %4d   dys: %4d   bitpix: %2d   imgsiz: %7d\n",
 *            dxs, dys, bitpix, imgsiz) ;
 *   fprintf (stdout, "imgtyp: %d   coltyp: %d   colsiz: %d\n",
 *            imgtyp, coltyp, colsiz) ;
 *   fprintf (stdout, "nxf: %d\n", nxf) ;
 */


   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "%s: Not a sun rasterfile.\n", namsrf) ;
      exit (-4) ;
      }

   if (dxs > dxg || dys > dyg)
      {
      fprintf (stderr, "Invalid image size . . . dxs: %3d . . . dys: %3d\n",
	       dxs, dys) ;
      exit (-5) ;
      }

   if (imgtyp != RT_STANDARD)
      {
      fprintf (stderr, "Only RT_STANDARD type sun rasterfiles are supported.\n") ;
      exit (-6) ;
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz) ;
      exit (-7) ;
      }

   /*** Copy RGB lookup tables into unsigned short int arrays. ***/

   if (colsiz > 0)
      {
      nread = read (fd, lutbuf, colsiz) ;
      colmax = colsiz / 3 ;
      numindex = 256 ;
      for (i = 0 ; i < colmax; i++)
         {
         sred   [i] = (unsigned short int) (lutbuf [i]            & 0377) ;
         sgreen [i] = (unsigned short int) (lutbuf [i + colmax]   & 0377) ;
         sblue  [i] = (unsigned short int) (lutbuf [i + colmax*2] & 0377) ;
         }
      if (colsiz < 256)
	 for (i = colsiz ; i < 256 ; i++)
	    {
            sred [i]   = (unsigned short int) 0 ;
            sgreen [i] = (unsigned short int) 0 ;
            sblue [i]  = (unsigned short int) 0 ;
	    }
      }

   /*** Read image from file into memory. ***/

   imgbuf = (unsigned char *)  malloc (imgsize) ;
   srfbuf = (unsigned char *)  malloc (imgsiz ) ;
   if (imgsiz != imgsize)
      {
      nread = read (fd, srfbuf, imgsiz) ;
      for (iy = 0 ; iy < dys; iy++)
         for (ix = 0 ; ix < nxs; ix++)
	    *(imgbuf + ix + iy*nxs) = *(srfbuf + ix + iy*nxf) ;
      }
   else
      nread = read (fd, imgbuf, imgsiz) ;
   close (fd) ;

   /*** Compute lower left and upper right coordinates for GL window. ***/

   if (dxs <= 1270)  xur = 1270 ; else xur = 1279;
   if (dys <=  995)  yur =  995 ; else yur = 1023;
   xll = xur - dxs + 1 ;
   yll = yur - dys + 1 ;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n") ;
      exit (-1) ;
      }
/*
 *   fprintf (stdout, "xll: %4d  yll: %4d  xur: %4d  yur: %4d\n",
 *            xll, yll, xur, yur) ;
 */

   /*** Open GL window. ***/

   prefposition (xll, xur, yll, yur) ;

   if (dxs != prevdxs || dys != prevdys)
      {
/*
 *      printf ("winopen . . . prevdxs: %4d  prevdys: %4d   dxs: %4d  dys: %4d\n",
 *              prevdxs, prevdys, dxs, dys) ;
 */
      if (gwid > 0)  winclose (gwid) ;
/*      noborder () ; */
      gwid = winopen (argv [0]) ;
/*
 *      doublebuffer () ;
 *      gconfig () ;
 */
      }
   prevdxs = dxs ;
   prevdys = dys ;

   /*** Write greyscale or colormap (pseudo-color) image to GL window. ***/

   if (imgtyp == RT_STANDARD && bitpix == 8)
      {
      cmode () ;
      gconfig () ;
      gl_wbox1 (imgbuf, lutoff, nbypx, xs, ys, dxs, dys, dir2, 0) ;
      }

   /*** Write RGB image to GL window. ***/
   /*
    * RT_STANDARD RGB sun rasterfile:
    * Each triplet (adjacent 3 bytes) contains
    * Blue, Green & Red pixel intensities, in that order.
    */

   if (imgtyp == RT_STANDARD && bitpix == 24)
      gl_wrgb (imgbuf, nxs, dxs, dys) ;

/*
   swapbuffers () ;
 */

   /*** Write color map (if present) to appropriate look-up tables. ***/

   if (colsiz > 0)
      loadcm (begindex, numindex, sred, sgreen, sblue) ;

   fprintf (stderr, "%s > ", namsrf) ;

   /*** Window remains on screen until terminated by mouse. ***/

   /*** Declare devices. ***/

   qdevice (WINCLOSE) ;
   qdevice (WINQUIT) ;
   qdevice (ESCKEY) ;
   qdevice (LEFTMOUSE) ;
   qdevice (MIDDLEMOUSE) ;
   qdevice (RIGHTMOUSE) ;

   /*** Respond to mouse buttons. ***/

   while (1)
   {
   while (qtest ())			/* Check device queue. */
      {
      switch (qread (&val))		/* Read entry at top of device queue. */
         {
         case LEFTMOUSE:
            if (val == 1 && seq > 0)	/* Left mouse button is down. */
               {
               findex++ ;
               free (imgbuf) ;
               if (findex >= numfil)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to advance beyond end of list.") ;
                     findex = numfil - 1 ;
		     }
                  else
		  if (seq == 2)
		     findex = 0 ;
                  }
               fprintf (stderr, "\n") ; 
               goto loop ;
               }
            break ;

         case RIGHTMOUSE:
            if (val == 1 && seq > 0)	/* Right mouse button is down. */
               {
               findex-- ;
               free (imgbuf) ;
               if (findex < 0)
                  {
		  if (seq == 1)
		     {
                     fprintf (stderr,
                              "*** Unable to back up beyond start of list.") ;
                     findex = 0 ;
		     }
                  else
		  if (seq == 2)
		     findex = numfil - 1 ;
                  }
               fprintf (stderr, "\n") ; 
               goto loop ;
               }
            break ;

         case MIDDLEMOUSE:		/* Middle mouse button is down. */
         case REDRAW:
            if (imgtyp == RT_STANDARD && bitpix == 8)
	       {
               cmode () ;
               gconfig () ;
               gl_wbox1 (imgbuf, lutoff, nbypx, xs, ys, dxs, dys, dir2, 0) ;
	       }
            else
            if (imgtyp == RT_STANDARD && bitpix == 24)
               gl_wrgb (imgbuf, nxs, dxs, dys) ;
            break ;

         case ESCKEY:		/* ESCape key. */
         case WINCLOSE:
         case WINQUIT:
            if (val == 1)
               {
               free (imgbuf) ;
               gexit () ;
               exit (0) ;
               }
         }
      qreset () ;
      }
   }

quit:
   gexit () ;
   exit (0) ;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * gl_wrgb: Write RGB image (standard Sun rasterfile format) to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
gl_wrgb (imgbuf, nxf, dxs, dys)

unsigned char *imgbuf ;
int nxf ;
int dxs, dys ;

{
   unsigned long *imgrgb ;
   unsigned long ired, igreen, iblue ;
   int dx3 ;
   int ix, iy, iiy ;

   /*** Allocate storage for GL output array. ***/

   dx3 = nxf ;
/*   dx3 = nxf * 3 ; */
/*   printf ("gl_wrgb> dx3: %d\n", dx3) ; */
   imgrgb = (unsigned long *) malloc (dxs * dys * sizeof (unsigned long)) ;

   /*** Rewrite Sun rasterfile data into GL lrectwrite form. ***/

   for (iy = 0 ; iy < dys; iy++)
      {
      iiy = dys - iy - 1 ;
      for (ix = 0 ; ix < dxs; ix++)
         {
         iblue  = (unsigned long) *(imgbuf + ix * 3     + iy * dx3) & 0377 ;
         igreen = (unsigned long) *(imgbuf + ix * 3 + 1 + iy * dx3) & 0377 ;
         ired   = (unsigned long) *(imgbuf + ix * 3 + 2 + iy * dx3) & 0377 ;

         *(imgrgb + ix + iiy * dxs) = iblue << 16 | igreen << 8 | ired ;
         }
      }

   /*** Write RGB image to GL window. ***/

   RGBmode () ;
   gconfig () ;

   lrectwrite (0, 0, dxs-1, dys-1, imgrgb) ;

   return (0) ;
}
