/* srfwlook.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a sequence Sun rasterfile images,
 * load colormap to LUT, load image to GL window.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Supports only 8-bit greyscale, 8-bit colormap, and 24-bit RGB image types.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 28 January 92
 * 20 August 92: add ability to display a sequence of images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include "/usr/people/stanger/lib/include/rasterfile.h"

main (argc, argv)

int argc;
char *argv [];
{
   extern char *malloc ();
   extern int gl_wbox ();		/* Write colormap image to GL window. */
   FILE *LIST;

   int gl_wrgb ();			/* Write RGB image to GL window. */

   static char dir2 [4] = "rd";
   static char namlst [80];
   static char namsrf [80];
   static char namgl [] = "srf";
   static char lutbuf [768];

   unsigned char *imgbuf;
   unsigned char *imgr, *imgg, *imgb;
   static unsigned short int sred [256], sgreen [256], sblue [256];

   static int srfhdr [8];

   int begindex = 512, numindex = 256;
   int dxg = 1280, dyg = 1024;
   int dzs = 0;
   int fd;				/* Image file descriptor. */
   int i;
   int ians;				/* KB response. */
   int imgsize;
   int maxdim;
   int nbypx;
   int ngr = 0, imgch = 0;
   int npximg;
   int nread;
   int prevdxs = 0, prevdys = 0;		/* Previous image size. */
   int seq;
   int xs, ys;

   int magic;
   int dxs, dys;
   int bitpix;
   int imgsiz, imgtyp;
   int coltyp, colsiz;
   int xll, yll;
   int xur, yur;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   /*** Get sun rasterfile from command line. ***/

   if (argc == 2)
      {
      strcpy (namsrf, argv [1]);
      }
   else
   if (argc == 2)
      {
      fprintf (stderr, "usage: %s sun_rasterfile_name | @namelist\n", argv [0]);
      exit (-1);
      }

   if (namsrf [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namsrf [1]);
      if ((LIST = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr, "Unable to open file %s\n", namlst);
         exit (-1);
         }
      }

   /*** Initialize GL software. ***/

   foreground ();

/*
 *   fprintf (stderr, "After each image is displayed,\n");
 *   fprintf (stderr, "Enter `q <return>' to quit, `<return>' to continue.\n");
 */

loop:

   if (seq)
      if (fscanf (LIST, "%s", namsrf) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }

   /*** Read Sun rasterfile header. ***/

   fd = open (namsrf, 0);
   nread = read (fd, srfhdr, 32);

   /*** Decode header. ***/

   magic  = srfhdr [0];
   dxs    = srfhdr [1];
   dys    = srfhdr [2];
   bitpix = srfhdr [3];
   imgsiz = srfhdr [4];
   imgtyp = srfhdr [5];
   coltyp = srfhdr [6];
   colsiz = srfhdr [7];
   xs = 0;
   ys = dys - 1;
   nbypx = bitpix / 8;
   npximg = dxs * dys;
   imgsize = npximg * nbypx;

/*
 *   fprintf (stdout, "dxs: %4d   dys: %4d   bitpix: %2d   imgsiz: %7d\n",
 *            dxs, dys, bitpix, imgsiz);
 *   fprintf (stdout, "imgtyp: %d   coltyp: %d   colsiz: %d\n",
 *            imgtyp, coltyp, colsiz);
 */

   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "Not a sun rasterfile.\n");
      exit (-2);
      }

   if (dxs > dxg || dys > dyg)
      {
      fprintf (stderr, "Invalid image size . . . dxs: %3d . . . dys: %3d\n",
	       dxs, dys);
      exit (-3);
      }

   if (imgtyp != RT_STANDARD)
      {
      fprintf (stderr, "Only RT_STANDARD type sun rasterfiles are supported.\n");
      exit (-4);
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz);
      exit (-5);
      }

   /*** Copy RGB lookup tables into unsigned short int arrays. ***/

   if (colsiz > 0)
      {
      nread = read (fd, lutbuf, 768);
      for (i=0; i<256; i++)
         {
         sred [i]   = (unsigned short int) (lutbuf [i]       & 0377);
         sgreen [i] = (unsigned short int) (lutbuf [i + 256] & 0377);
         sblue [i]  = (unsigned short int) (lutbuf [i + 512] & 0377);
         }
      }

   /*** Read image from file into memory. ***/

   imgbuf = (unsigned char *)  malloc (imgsize);
   nread = read (fd, imgbuf, imgsize);
   close (fd);

   /*** Compute lower left and upper right coordinates for GL window. ***/

   if (dxs <= 1270)  xur = 1270; else xur = 1279;
   if (dys <=  995)  yur =  995; else yur = 1023;
   xll = xur - dxs + 1;
   yll = yur - dys + 1;
   if (xll < 0 || yll < 0)
      {
      fprintf (stderr, "Image too large for screen\n");
      exit (-1);
      }
/*
 *   fprintf (stdout, "xll: %4d  yll: %4d  xur: %4d  yur: %4d\n",
 *            xll, yll, xur, yur);
 */

   /*** Open GL window. ***/

/*
 *   foreground ();
 */
   prefposition (xll, xur, yll, yur);

   if (dxs != prevdxs || dys != prevdys)
      {
printf ("winopen . . . prevdxs: %4d  prevdys: %4d    dxs: %4d  dys: %4d\n",
         prevdxs, prevdys, dxs, dys);
      winopen ("srfw");
      doublebuffer ();
      gconfig ();
      }
   prevdxs = dxs;
   prevdys = dys;

   /*** Write color map (if present) to appropriate look-up tables. ***/

   if (colsiz > 0)
      loadcm (begindex, numindex, sred, sgreen, sblue);

   /*** Write greyscale or colormap (pseudo-color) image to GL window. ***/

   if (imgtyp == RT_STANDARD && bitpix == 8)
      gl_wbox (imgbuf, nbypx, xs, ys, dxs, dys, dir2);

   /*** Write RGB image to GL window. ***/
   /*
    * RT_STANDARD RGB sun rasterfile:
    * Each triplet (adjacent 3 bytes) contains
    * Blue, Green & Red pixel intensities, in that order.
    */

   if (imgtyp == RT_STANDARD && bitpix == 24)
      gl_wrgb (imgbuf, dxs, dys);

   swapbuffers ();

/*
 *   fprintf (stderr, "%s > ", namsrf);
 *   ians = getchar ();
 *   if (ians == 0161)  goto quit;
 */

   if (seq)
      {
      sleep (1);
      goto loop;
      }

   else		/*** Window remains on screen until terminated by mouse. ***/
      {
      qdevice (WINCLOSE);
      qdevice (WINQUIT);
      qdevice (ESCKEY);
      for (;;)
         {
         short val;
         switch (qread (&val))
            {
            case REDRAW:
               gl_wbox (imgbuf, nbypx, xs, ys, dxs, dys, dir2);
               break;
            case WINCLOSE:
            case WINQUIT:
            case ESCKEY:
               free (imgbuf);
               gexit ();
               exit (0);
            }
         }
      }

quit:
   gexit ();
   exit (0);

}


gl_wrgb (imgbuf, dxs, dys)

unsigned char *imgbuf;
int dxs, dys;

{
   unsigned char *ibuf;
   long *irgb, *imgrgb;
   int i, iy;
   int ired, igreen, iblue;
   int npximg;

   npximg = dxs * dys;
   imgrgb = (long *) malloc (dxs * dys * sizeof (long));

   for (i = 0; i < npximg; i++)
      {
      irgb = imgrgb + i;
      ibuf = imgbuf + i * 3;

      iblue  = (long) *ibuf & 0377;
      igreen = (long) *(ibuf + 1) & 0377;
      ired   = (long) *(ibuf + 2) & 0377;
      *irgb = iblue << 16 | igreen << 8 | ired;
      }

   RGBmode ();
   gconfig ();

   for (i = 0; i < dys; i++)
      {
      iy = dys - i - 1;
      lrectwrite (0, iy, dxs-1, iy, imgrgb + i * dxs);
      }

   return (0);
}
