/* stripsrfhdr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Remove header from Sun Rasterfile.
 * Read Sun rasterfile image,
 * save colormap to a colormap file,
 * write image to new data file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Supports only 8-bit greyscale, 8-bit colormap, and 24-bit RGB image types.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 *  6 February 1992
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <gl/gl.h>
#include <device.h>
#include "/usr/people/stanger/lib/include/rasterfile.h"

main (argc, argv)

int argc;
char *argv [];
{
   extern char *malloc ();
   extern int gl_wbox ();		/* Write colormap image to GL window. */
   int gl_wrgb ();			/* Write RGB image to GL window. */
   FILE *FDLUT;			/* Colormap file descriptor. */

   static char dir2 [4] = "rd";
   static char namlut [80];
   static char namimg [80];
   static char namsrf [80];
   static char lutbuf [768];

   unsigned char *imgbuf;
   unsigned char *imgr, *imgg, *imgb;
   static unsigned short int sred [256], sgreen [256], sblue [256];

   static int srfhdr [8];

   int begindex = 512, numindex = 256;
   int colmax = 0;
   int dxg = 1280, dyg = 1024;
   int dzs = 0;
   int fd;			/* Sun Rasterfile image file descriptor. */
   int fdimg;			/* Binary image file descriptor. */
   int i;
   int idot;
   int imgsize;
   int maxdim;
   int nbypx;
   int ngr = 0, imgch = 0;
   int npximg;
   int nread;
   int xs, ys;

   int magic;
   int dxs, dys;
   int bitpix;
   int imgsiz, imgtyp;
   int coltyp, colsiz;
   int xll, yll;
   int xur, yur;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   /*** Get sun rasterfile from command line. ***/

   if (argc == 2)
      {
      strcpy (namsrf, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s sun_rasterfile_name\n", argv [0]);
      fprintf (stderr, "Enter Sun rasterfile image name: ");
      fscanf  (stdin,  "%s", namsrf);
      }

   /*** Create binary image file and LUT file names. ***/

   idot = -1;
   for (i = 0; i < strlen (namsrf); i++)
      {
      if (namsrf [i] == '.')  idot = i;
      }
   strncpy (namimg, namsrf, idot);
   strcat (namimg, ".img");
   strncpy (namlut, namsrf, idot);
   strcat (namlut, ".lut");

   /*** Create output image file. ***/

   if ((fdimg = creat (namimg, 0644)) <= 0)
      {
      fprintf (stderr, "Unable to create file: %s\n", namimg);
      exit (-1);
      }

   /*** Read Sun rasterfile header. ***/

   fd = open (namsrf, 0);
   nread = read (fd, srfhdr, 32);

   /*** Decode header. ***/

   magic  = srfhdr [0];
   dxs    = srfhdr [1];
   dys    = srfhdr [2];
   bitpix = srfhdr [3];
   imgsiz = srfhdr [4];
   imgtyp = srfhdr [5];
   coltyp = srfhdr [6];
   colsiz = srfhdr [7];
   xs = 0;
   ys = dys - 1;
   nbypx = bitpix / 8;
   npximg = dxs * dys;
   imgsize = npximg * nbypx;

   fprintf (stdout, "dxs: %4d   dys: %4d   bitpix: %2d   imgsiz: %7d\n",
            dxs, dys, bitpix, imgsiz);
   fprintf (stdout, "imgtyp: %d   coltyp: %d   colsiz: %d\n",
            imgtyp, coltyp, colsiz);

   if (magic != RAS_MAGIC)
      {
      fprintf (stderr, "Not a sun rasterfile.\n");
      exit (-2);
      }

   if (dxs > dxg || dys > dyg)
      {
      fprintf (stderr, "Invalid image size . . . dxs: %3d . . . dys: %3d\n",
	       dxs, dys);
      exit (-3);
      }

   if (imgtyp != RT_STANDARD)
      {
      fprintf (stderr, "Only RT_STANDARD type sun rasterfiles are supported.\n");
      exit (-4);
      }

   if (colsiz > 768)
      {
      fprintf (stderr, "Invalid color table . . . colsiz: %d\n", colsiz);
      exit (-5);
      }

   /*** Copy RGB lookup tables into unsigned short int arrays. ***/

   if (colsiz > 0)
      {
      if ((FDLUT = fopen (namlut, "w")) == NULL)
         {
         fprintf (stderr, "Unable to create file: %s\n", namlut);
         exit (-1);
         }

      nread = read (fd, lutbuf, colsiz);
      colmax = colsiz / 3;
      for (i = 0; i < colmax; i++)
         {
         sred [i]   = (unsigned short int) (lutbuf [i]          & 0377);
         sgreen [i] = (unsigned short int) (lutbuf [i + colmax] & 0377);
         sblue [i]  = (unsigned short int) (lutbuf [i + colmax] & 0377);
         fprintf (FDLUT, "%4d %3d %3d %3d\n",
                  i, sred [i], sgreen [i], sblue [i]);
         }
      fclose (FDLUT);
      }

   /*** Read image from file into memory. ***/

   imgbuf = (unsigned char *)  malloc (imgsize);
   nread = read (fd, imgbuf, imgsize);
   close (fd);

   write (fdimg, imgbuf, imgsize);
   close (fdimg);

   exit (0);
}
