/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * wrgbsrf.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Convert raw RGB (or RGBA) image to a Sun Rasterfile format file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   SMM C/P   HAO/NCAR
 * 18 August 1993
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include "/usr/people/stanger/lib/include/rasterfile.h"

wrgbsrf (namsrf, dx, dy, nbypx, rbuf, gbuf, bbuf, abuf)

char *namsrf;				/* Name of ouput Sun rasterfile. */
int dx, dy;				/* Image width & height (pixels). */
int nbypx;				/* # bytes per pixel. */
unsigned char *rbuf, *gbuf, *bbuf, *abuf;	/* R/G/B/A buffers. */

{
   struct rasterfile sun;
   FILE *fp;
   unsigned char *imgbuf, *tmp, *end;
   int i;
   int nbyimg;
   int npximg;

   if ( (fp = fopen (namsrf, "w")) == NULL )
      {
      perror ("wrgbsrf> fopen error:");
      return (-1);
      }

   sun.ras_magic = RAS_MAGIC;
   sun.ras_width = ( dx % 2 == 0 ) ? dx : dx + 1;
   sun.ras_height = dy;
   sun.ras_depth = nbypx * 8;
   sun.ras_length = sun.ras_width * sun.ras_height * sun.ras_depth/8;
   sun.ras_type = RT_STANDARD;
   sun.ras_maptype = RMT_NONE;
   sun.ras_maplength = 0;

   /*** Write Sun rasterfile header. ***/

   if ( fwrite ((char *) (&sun), sizeof (sun), 1, fp) != 1 )
      {
      perror ("wrgbsrf> header fwrite error:");
      return (-1);
      }

   /*** Pack image buffer. ***/

   npximg = dx * dy;
   nbyimg = npximg * nbypx;
   imgbuf = (unsigned char *) malloc (npximg * nbypx * sizeof (char));

   if (nbypx  == 3)
      for (i = 0; i < npximg; i++)
         {
         *(imgbuf + i*3)     = *(bbuf + i);
         *(imgbuf + i*3 + 1) = *(gbuf + i);
         *(imgbuf + i*3 + 2) = *(rbuf + i);
         }

   if (nbypx  == 4)
      for (i = 0; i < npximg; i++)
         {
         *(imgbuf + i*4)     = *(abuf + i);
         *(imgbuf + i*4 + 1) = *(bbuf + i);
         *(imgbuf + i*4 + 2) = *(gbuf + i);
         *(imgbuf + i*4 + 3) = *(rbuf + i);
         }

   /*** Write image pixel data. ***/

   if ( sun.ras_width == dx )
      {
      if ( fwrite ((char *) imgbuf, 1, nbyimg, fp) != nbyimg )
	 {
	 perror ("wrgbsrf> image fwrite error:");
	 return (-1);
         }
      }
   else 
      {
      end = (unsigned char *) imgbuf + nbyimg;
      for (tmp = (unsigned char *) imgbuf; tmp < end; tmp += dx*3)
	 {
         if ( fwrite (tmp, sun.ras_width, 1, fp) != 1 )
	    {
	    perror ("wrgbsrf> image fwrite error:");
	    return (-1);
             }
          }
      }

   free (imgbuf);

   fclose (fp);
   return (0);
}
