/* acosext.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * ACOS FITS header extension header.
 * Contains real-time information and instrument housekeeping.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * ACOS : Advanced Coronal Observing System, includes these instruments :
 *        DPM: Digital Prominence Monitor
 *        Low K
 *        CHIP: Chromospheric Helium-1 Imaging Photometer
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   17 November 1995
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
 */

acosext ()

{
   static char eq       [4]  = "= " ;
   static char fitbuf   [36][80] ;		/* FITS header buffer. */

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;

   if (pmode)
      printf (" --- PRIMARY HDU ---\n") ;

   /*** Fill FITS primary header. ***/

   iy = 0 ;

   /*** Required FITS keywords. ***/

   sprintf (&fitbuf [iy++][0],  "XTENSION= 'IMAGE   '           / MLSO/ACOS %s EXTENSION", type_obs) ;

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / %s", kbitpix, eq, bitpix, pixcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / NUMBER OF DIMENSIONS",
	                       knaxis, eq, ndim) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                       knaxis1, eq, nx, atype1) ;
   if (ndim >= 2)
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / SIZE OF %s DIMENSION",
	                          knaxis2, eq, ny, atype2) ;
   else
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;

   /*** Extension information. ***/

   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION PCOUNT",
				kpcount, eq, pcount) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20d / EXTENSION GCOUNT",
				kgcount, eq, gcount) ;

   /*** Telescope information. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kobject,   eq, object) ;
   sprintf (&fitbuf [iy++][31], "/ Mauna Loa Solar Observatory (MLSO)") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     korigin,   eq, origin) ;
   sprintf (&fitbuf [iy++][31], "/ High Altitude Observatory (HAO), DIV. OF NCAR") ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-9s'",     ktelescop, eq, telescop) ;
   sprintf (&fitbuf [iy++][31], "/ MLSO %s", &tscopcom [ACOSI->ttype][0]) ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kinstrume, eq, instrume) ;
   sprintf (&fitbuf [iy++][31], "/ %s", &instcom [ACOSH->ttype][0]) ;


   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     kdate_obs, eq, date_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION DATE: DD/MM/YY [UT] DOY %03d %d",
				doy_obs, year_obs) ;
   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",     ktime_obs, eq, time_obs) ;
   sprintf (&fitbuf [iy++][31], "/ OBSERVATION TIME: HH.MM.SS [UT]") ;

   /*** AXIS 1 COORDINATE SYSTEM. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype1, eq, ctype1) ;
   sprintf (&fitbuf [iy++][31], "/ AXIS 1 TYPE: %s %s", atype1, ctypenam) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [INDEX]",
			      kcrpix1  , eq, crpix1, atype1, refcom) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [%s]",
			      kcrval1  , eq, crval1, atype1, refcom, cunit1) ;
   sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			      kcdelt1  , eq, cdelt1, atype1, cunit1) ;

   /*** AXIS 2 COORDINATE SYSTEM. ***/

   if (ndim >= 2)
      {
      sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kctype2, eq, ctype2) ;
      sprintf (&fitbuf [iy++][31], "/ AXIS 2 TYPE: %s %s", atype2, ctypenam) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [INDEX]",
			           kcrpix2, eq, crpix2, atype2, refcom) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.2f / %s %s [%s]",
			           kcrval2, eq, crval2, atype2, refcom, cunit2) ;
      sprintf (&fitbuf [iy++][0],  "%8s%2s%20.4f / %s COORD INCREMENT [%s/PIXEL]",
			           kcdelt2  , eq, cdelt2, atype2, cunit2) ;
      }
   else
      {
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;
      }

   /*** Coordinate System Name: geocentric or heliocentric. ***/

   sprintf (&fitbuf [iy  ][0],  "%8s%2s'%-8s'",  kcoordnam, eq, ctypenam) ;
   sprintf (&fitbuf [iy++][31], "/ COORDINATE SYSTEM NAME") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", kdisp_dir,  eq, disp_dir) ;
   sprintf (&fitbuf [iy++][31], "/ DISPLAY DIRECTION (AXIS 1 2 3): R/L  U/D  O/I") ;

   sprintf (&fitbuf [iy  ][0],  "%8s%2s%20.2f", kcrradius, eq, crradius) ;
   sprintf (&fitbuf [iy++][31], "/ SOLAR RADIUS [PIXELS] (DEFAULT GUESS)") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR RADIUS [ARCSEC]",
			      krsun    , eq, ACOSI->rsun) ;

   if (ACOSI->ttype .eq. IS_DPMON || ACOSI->ttype .eq. IS_LOWK)
      {
      sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / OCCULTING DISC DIAMETER [ARCSECONDS]",
				   kocc_disc, eq, odisk) ;
      }
   else
      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;

   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR P-ANGLE [DEGREES]",
			      ksolar_p0, eq, ACOSI->pangle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR B-ANGLE [DEGREES]",
			      ksolar_b0, eq, ACOSI->bangle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR L-ANGLE [DEGREES]; CARRINGTON LONGITUDE",
			      ksolar_l0, eq, ACOSI->langle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR HOUR ANGLE [DEGREES] (W.R.T.ZENITH E- W+)",
			      khour_ang, eq, ACOSI->hangle) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR ELEVATION [DEGREES]",
			      kelev    , eq, ACOSI->elev) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / SOLAR AZIMUTH [DEGREES]",
			      kazimuth , eq, ACOSI->azimuth) ;
   sprintf (&fitbuf [iy++][0], "%8s%2s%20.2f / AIRMASS [ATMOSPHERES]",
			      kairmass , eq, ACOSI->airmass) ;


   sprintf (&fitbuf [iy  ][0], "%8s%2s'%-8s'", ktype_obs, eq, type_obs) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [DESCRIPTION]") ;

   sprintf (&fitbuf [iy  ][0], "%8s%2s%20d",
			      kcode_obs, eq, ACOSI->dtype) ;
   sprintf (&fitbuf [iy++][31], "/ TYPE OF OBSERVATION [NUMERICAL CODE] ") ;

      sprintf (&fitbuf  [iy++][0],  "          ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^") ;

/* -------------------------------------------------------------------------- */

   /*** Set string's null terminator to blank. ***/
   /*** Convert all characters to upper case. ***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 {
	 if (fitbuf [iy][ix] == '\0')  fitbuf [iy][ix] = ' ' ;
	 ival = (int) fitbuf [iy][ix] ;
	 fitbuf [iy][ix] = (char) toupper (ival) ;
	 }

   for (i = 0 ; i < 36 ; i++)
      {
      strncpy (outbuf, &fitbuf [i][0], 80) ;
      outbuf [80] = '\0' ;
      if (pmode)
         fprintf (stdout, "%s\n", outbuf) ;
      }

   /*** Write first record of PRIMARY FITS header. ***/

   nelwr = fwrite (fitbuf, sizeof (char), 2880, FF) ;

   /*** Fill FITS buffer with ASCII blanks.***/

   for (iy = 0 ; iy < 36 ; iy++)
      for (ix = 0 ; ix < 80 ; ix++)
	 fitbuf [iy][ix] = ' ' ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   ACOS_HEADER STRUCTURE: ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   VERSION                                                         ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   YEAR                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   MONTH                                                           ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   DAY                                                             ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   HOUR                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   MINUTE                                                          ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   SECOND                                                          ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   XDIM                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   YDIM                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   NSUM                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   EXPOSURE                                                        ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   GAIN                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   ODISK                                                           ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   TYPE                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   WAVE                                                            ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT PANGLE                                                          ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT BANGLE                                                          ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT LANGLE                                                          ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT HA                                                              ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT ELEVATION                                                       ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT AZIMUTH                                                         ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT AMASS                                                           ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT RADIUS                                                          ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT XSHIFT [16]                                                     ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT YSHIFT [16]                                                     ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT DIAM [16]                                                       ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   POS [32]                                                        ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   INT   DRIVE [32]                                                      ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT ANALOG [16]                                                     ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT LCVR [8]                                                        ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT   FLOAT MIC [6]                                                         ") ;
   sprintf (&fitbuf [iy++][0],  "COMMENT                                                                         ") ;
}
