/* cpremstr.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Removes stray light from array 'data'.
 *----------------------------------------------------------------------
 * rsun  = array of radial values (units of rsun). 
 * nr = # elements in rsun array (=1 for radial scan, =ndata for theta scan).
 * data  = array of intensity values (units of 10**-11 b/bsun).
 * ndata = # elements in array data.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   programmer III   NCAR/HAO   SMM C/P
 *  3 February 1983: PDP-11/34 Fortran-4+ RSX-11M
 * 18 July 1984: conversion to VAX 750 (C).
 *  1 April 1999: replace stray () function with cpstray ().
 *----------------------------------------------------------------------
 */

#include <stdio.h>

cpremstr (rsun, nr, data, ndata)

int nr, ndata ;
float rsun [] ;				/* arcsec units */
float data [] ;				/* Brightness (Bsun) */

{

int i ;
int ierr ;
float r ;
float s ;
extern float cpstray () ;

   if (nr > 1 && nr < ndata)
      {
      fprintf (stderr, "# rsun values:%d less than # data values:%d\n",
               nr, ndata) ;
      ierr = -1 ;
      }

   else

      {

      if (nr > 1)		/* Radial scans. */
         {
         for (i = 0 ; i < ndata ; i++)
            {
            r = rsun [i] ;
            data [i] -= cpstray (r) ;
            }
         ierr = 0 ;
         }

      else

      if (nr == 1)		/* Theta scans. */
         {
          r = rsun [0] ;
          s = cpstray (r) ;
          for (i = 0 ; i < ndata ; i++)  data [i] -= s ;
          ierr = 0 ;
          }
      }

   return (ierr) ;
}
