/* rdcp.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read SMM C/P image from disk file into memory array.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Solar Maximum Mission   Coronagraph/Polarimeter
 * Andrew L. Stanger
 * 4 February 1985
 * 11 November 1986: modify to accomodate floating-point images.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>
#include "/home/stanger/lib/include/cplib.ext"

char	*malloc ();	/* System allocation function. */

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

unsigned char *
rdcp (namimg)

char  *namimg;	/* Pointer to image disk file name. */

{
register unsigned char *imgbuf;	/* Pointer to image array. */
register int	imgfd;		/* File descriptor for image disk file. */
register int	imgsiz;		/* Image size (bytes) (excludes hskp/bitmap). */
register long	nread;
int nrechd;			/* # records for header. */

   /*** Decommutate Housekeeping. ***/

   cphskp (namimg);	

   /*** Open image file. ***/

   if ((imgfd = open (namimg, 0)) < 0)
      {
      fprintf (stderr, "rdcp> Can't open file %s\n", namimg);
      return (NULL);
      }

   /*** Position I/O pointer to beginning of video data in image disk file. ***/

   if (nbypix == 4) nrechd = 1;	/* hk + bitmap reside in a single record. */
   else
      nrechd = 2;		/* hk & bitmap reside in separate records. */
   if (lseek (imgfd, (long) (npix * nbypix * nrechd), 0) == -1)
      {
      fprintf (stderr, "rdcp> seek error.\n");
      return (NULL);
      }

/*   fprintf (stdout, "rdcp> npix: %d   npixl: %d   nbypix: %d\n",
 *            npix, npixl, nbypix);
 */

   imgsiz = (npixl * npixl * nbypix);
   if ((imgbuf = (unsigned char *) malloc (imgsiz)) == NULL)
     {
     fprintf (stderr, "rdcp> malloc error.");
      exit (1);
     }

   if ((nread = read (imgfd, imgbuf, imgsiz)) <= 0)
      {
      fprintf (stderr, "rdcp> image read error.\n");
      return (NULL);
      }

   if (nread != imgsiz)
      {
      fprintf (stderr, "rdcp> nread:%d < imgsiz:%d\n", nread, imgsiz);
      return (NULL);
      }

   /*** Reverse bytes for Vax-calibrated images. ***/

   if (idtype == 3 && bytord == 1)  srevbyt (imgbuf, nbypix, npixl*npixl);

   if (close (imgfd) == -1)
      {
      fprintf (stderr, "rdcp> file close error.%s\n", namimg);
      return (NULL);
      }
  return (imgbuf);
}
