/* rmfc.c
 *------------------------------------------------------------------------------
 * Removes F-corona from array data.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   NCAR/HAO   SMM C/P
 * 4 February 1983
 * 26 July 1984: conversion from 11/34 (f4p) to VAX 750 (C).
 * 11 December 1985: return -1 in case of error.
 * 15 May 1986: Use ATM/WLC F-corona function.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * rsun  = array of radial values (units of rsun). 
 * nr = # elements in rsun array (=1 for radial scan, =ndata for theta scan).
 * data  = array of intensity values (units of 10**-11 b/bsun).
 * ndata = # elements in array data.
 *------------------------------------------------------------------------------
 */

#include <stdio.h>

/* ********************************************************************** */

rmfc (rsun, nr, data, ndata, ang, nang)

float	*rsun ;		/* Array of radial values. */
float	*data ;		/* Array of intensity values. */
float	*ang ;		/* Array of position angle values. */
int	nr ;		/* # items in rsun. */
int	ndata ;		/* # items in data. */
int	nang ;		/* # items in ang. */

{

extern	float	fcorona () ;
float	r ;
int	i ;

   if (nr == ndata)		/* Radial scans. */
      {
      for (i = 0 ; i < ndata ; i++)
         {
         r = rsun [i] ;
         data [i] -= fcorona (r, ang [0]) ;
         }
      return (0) ;
      }

   else
   if (nr == 1)			/* Theta scans. */
      {
      r = rsun [0] ;
      for (i = 0 ; i < ndata ; i++)  data [i] -= fcorona (r, ang [i]) ;
      return (0) ;
      }

   else				/* Error. */
      {
      fprintf (stderr, "# radii: %d < # data values: %d\n", nr, ndata) ;
      return (-1) ;
      }
}
