/* glmk3_annot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Write SMM C/P image annotation to Iris GL window.
 * . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . .
 * Andrew L. Stanger  HAO/NCAR   3 April 1991
 *  7 Jan 94: adapt for MLSO Mark-III.
 * 25 Feb 94: add data types 12-23.
 * 25 Apr 94: dtype 23 is now 24 (avgrpB); dtype 23 is avgpB.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include "/home/stanger/lib/include/mk3lib.ext"
#include "/home/stanger/lib/include/gl_color.h"

glmk3_annot (lbottom, cval1, cval2, cval3, cval4)

int lbottom;	/* Bottom flag: =0 normal; =1 no information below image. */
int cval1;	/* Color level for annotation. */
int cval2;	/* Color level for labels. */
int cval3;	/* Color level for HAO logo. */
int cval4;	/* Color level for miscellaneous labels. */

{
   static char Month_name [12][4] = {"jan", "feb", "mar", "apr", "may", "jun",
				     "jul", "aug", "sep", "oct", "nov", "dec"};
   static char month [4];
   static char wbuf [12];
   static char imop [4];

   /* `Screen' font sizes:
    * 7 => 6x7
    * 11,13,15 => 8x15
    *
    * `Screen-Bold' font sizes:
    * 13,15 => 9x15
    *
    * `Helvetica-Bold' font sizes:
    *  8 => 11x13
    * 10 => 14x15
    * 12 => 18x20
    * 14 => 20x22
    * 18 => 27x29
    * 24 => 37x38
    *
    * `CharterBT-Black' font sizes:
    *  8 => 12x13
    * 10 => 14x15
    * 12 => 16x17
    * 14 => 22x23
    * 19 => 28x30
    * 25 => 38x40
    */
   static char font1 [] = "Screen";
   static char font2 [] = "Screen-Bold";
   static char font3 [] = "Helvetica-Bold";
   static char font4 [] = "CharterBT-Black";
   static char afont [32];
   static char bfont [32];
  
   static int afx, afy, bfx, bfy;
   int annot_state = 1;
   int charheight = 43;
   int lref;
   int offset = -21;
   int xstart = 2;
   int ystart;
   int xc, yc;			/* height & width for character annotation. */
   int xs, ys;			/* Start position for character annotation. */
   int year, doy;
   int imonth, iday;
   int asize, bsize;

   ystart = 573;
   xs = xstart;
   yc = charheight;
   strcpy (afont, font1);
   strcpy (bfont, font3);
   asize =  7;
   bsize = 11;
   afx =  6;
   afy =  7;
   bfx = 14;
   bfy = 15;
   ys = ystart - afy - 1;

   gl_setfont (bfont, bsize);		/* Select font. */

   /*** Define image operator (if applicable). ***/

   switch (MK3I->dtype)
      {
      case 5:  strcpy (imop, "- "); lref = 1; break;
      case 7:  strcpy (imop, "-/"); lref = 1; break;
      case 8:  strcpy (imop, "-/"); lref = 1; break;
      case 15: strcpy (imop, " /"); lref = 1; break;
      default: strcpy (imop, "  "); lref = 0; break;
      }

   /*** Day and month. ***/

   year = MK3I->year;
   if (MK3I->year < 1900)  year = MK3I->year + 1900;
   doy    = MK3I->doy;
   imonth = MK3I->month;
   iday   = MK3I->day;
   strcpy (month, &Month_name [imonth-1][0]);
   sprintf (wbuf, "%d %s ", iday, month);
   gl_wstr (wbuf, xs, ys - 1*bfy, cval1);
   sprintf (wbuf, "%d", year);
   gl_wstr (wbuf, xs, ys - 2*bfy, cval1);

   /*** Year and Day-Of-Year: ***/

   if (MK3I->year > 1900)  year = MK3I->year - 1900;
   (void) sprintf (wbuf, "%02d: %03d", year, doy);
   gl_wstr (wbuf, xs, ys - 2*afy - 4*bfy, cval1);

   /*** Reference image year & DOY (if applicable). ***/

/*
   (void) sprintf (wbuf, "         ");
   if (lref)
      {
      if (MK3I->year != year_zero)
         (void) sprintf (wbuf, "%s%02d:%03d", imop, year_zero, doy_zero);
      else
      if (MK3I->year == year_zero && MK3I->doy != doy_zero)
         (void) sprintf (wbuf, "   %s %03d ", imop, doy_zero);
      else
         (void) sprintf (wbuf, "         ");
      }
   gl_wstr (wbuf, xs, ys - 2*afy - 5*bfy, cval1);
 */

   /*** Time: ***/

   (void) sprintf (wbuf, "%02d:%02d:%02d", MK3I->hour, MK3I->minute, MK3I->second);
   gl_wstr (wbuf, xs, ys - 4*afy - 7*bfy, cval1);

   /*** Reference image time (if applicable). ***/

/*
   if (lref)
      {
      (void) sprintf (wbuf, "    %s", imop);
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 8*bfy), cval1);
      (void) sprintf (wbuf, "%02d:%02d:%02d",
                      hour_zero, minute_zero, second_zero);
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 9*bfy), cval1);
      }
   else
      {
      (void) sprintf (wbuf, "         ");
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 8*bfy), cval1);
      gl_wstr (wbuf, xs, (int) (ys - 4*afy - 9*bfy), cval1);
      }
 */

   /*** Data type ***/

   switch (MK3I->dtype)
      {
      case  0: sprintf (wbuf,"  q    ");   break;
      case  1: sprintf (wbuf," rq    ");   break;
      case  2: sprintf (wbuf,"srq    ");   break;
      case  3: sprintf (wbuf,"  i    ");   break;
      case  4: sprintf (wbuf," ri    ");   break;
      case  5: sprintf (wbuf,"sri    ");   break;
      case  6: sprintf (wbuf,"  u    ");   break;
      case  7: sprintf (wbuf," ru    ");   break;
      case  8: sprintf (wbuf,"sru    ");   break;
      case  9: sprintf (wbuf,"  pB   ");   break;
      case 10: sprintf (wbuf," rpB   ");   break;
      case 11: sprintf (wbuf,"srpB   ");   break;
      case 12: sprintf (wbuf,"  Ne   ");   break;
      case 13: sprintf (wbuf," rNe   ");   break;
      case 14: sprintf (wbuf,"srNe   ");   break;
      case 15: sprintf (wbuf," aba   ");   break;
      case 16: sprintf (wbuf,"  ba   ");   break;
      case 17: sprintf (wbuf," ast   ");   break;
      case 18: sprintf (wbuf,"  st   ");   break;
      case 19: sprintf (wbuf,"calib  ");   break;
      case 20: sprintf (wbuf,"tscp   ");   break;
      case 21: sprintf (wbuf,"   I   ");   break;
      case 22: sprintf (wbuf,"   U   ");   break;
      case 23: sprintf (wbuf," avgpB ");   break;
      case 24: sprintf (wbuf," avgrpB");   break;
      default: sprintf (wbuf,"*******");   break;
      }
   gl_wstr (wbuf, xs, ys - 6*afy - 11*bfy, cval1);

   /*** Channel. ***/


   sprintf (wbuf, "%2d", MK3I->channel);
   gl_wstr (wbuf, xs, ys - 7*afy - 13*bfy, cval1);

   /*** Min/Max data range: ***/

   sprintf (wbuf, "%5d", MK3I->dmin);
   gl_wstr (wbuf, xs, ys - 8*afy - 15*bfy, cval1);

   sprintf (wbuf, "%5d", MK3I->dmax);
   gl_wstr (wbuf, xs, ys - 8*afy - 16*bfy, cval1);

   /*** Min/Max display range: ***/

   sprintf (wbuf, "%5d", MK3I->smin);
   gl_wstr (wbuf, xs, ys - 9*afy - 18*bfy, cval1);

   sprintf (wbuf, "%5d", MK3I->smax);
   gl_wstr (wbuf, xs, ys - 9*afy - 19*bfy, cval1);

   /*** Scan Direction: ***/

   switch (MK3I->scndir)
      {
      case -1:  sprintf (wbuf, "-  CW "); break;
      case  1:  sprintf (wbuf, "+ CCW "); break;
      default: sprintf (wbuf, "*****"); break;
      }
   gl_wstr (wbuf, xs, ys - 10*afy - 21*bfy, cval1);

   /*** Scan duration (seconds) : ***/

   (void) sprintf (wbuf, "%6.2f", MK3I->scndur);
   gl_wstr (wbuf, xs, ys - 11*afy - 23*bfy, cval1);

   /*** Solar Radius scale size (pixels) : ***/

   (void) sprintf (wbuf, "%6.2f", MK3I->pixrs);
   gl_wstr (wbuf, xs, ys - 12*afy - 25*bfy, cval1);

   /*** P-angle: ***/

   sprintf (wbuf, "%6.2f", MK3I->pangle);
   gl_wstr (wbuf, xs, ys - 13*afy - 27*bfy, cval1);

   return (0);
}
