/* mk3flags.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Set MLSO Mark-III flags by decoding information in flagch bits.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   6 January 1994
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

mk3flags (flagch, quickflag, pbflag, qdflag, udflag, idflag,
		  posflag, negflag, channel)

int flagch ;			/* Header flagch short integer. */
int *quickflag ;		/*				*/
int *pbflag ;			/* pB data			*/
int *qdflag ;			/* Q  data			*/
int *udflag ;			/* U  data			*/
int *idflag ;			/* I  data			*/
int *posflag ;			/* CCW scan +			*/
int *negflag ;			/*  CW scan -			*/
int *channel ;			/* data channel [0,1]		*/

{
   int COMPRESS_FLAG =    1 ;
   int CALIB0_FLAG   =    2 ;
   int CALIB15_FLAG  =    4 ;
   int SPEED_FLAG    =    8 ;
   int CLEAN_FLAG    =   16 ;
   int TIDY_FLAG     =   32 ;
   int QUICK_FLAG    =   64 ;	/* Xfer to rect coord done quickly/inelegantly*/
   int PB_FLAG       =  128 ;	/* pB data	*/
   int Q_FLAG        =  256 ;	/* Q  data	*/
   int U_FLAG        =  512 ;	/* U  data	*/
   int I_FLAG        = 1024 ;	/* I  data	*/
   int DIR_FLAG      = 2048 ;	/* 0 => - scan (CW); 1 => + scan (CCW) */
   int NE_FLAG       = 4096 ;	/* Nelectrons, scaled by 1.e+33 */

   /*** Initialize variables. ***/

   *quickflag = 0 ;
   *pbflag    = 0 ;
   *qdflag    = 0 ;
   *udflag    = 0 ;
   *idflag    = 0 ;
   *posflag   = 0 ;
   *negflag   = 0 ;
   *channel   = -1 ;

   *channel = (flagch & 0000007) ;	/* In least significant 3 bits. */
   flagch = flagch >> 3 ;		/* Shift right 3 bits. */

   if ((flagch & QUICK_FLAG) > 0)  *quickflag = 1 ;
   if ((flagch & PB_FLAG)    > 0)  *pbflag    = 1 ;
   if ((flagch & Q_FLAG)     > 0)  *qdflag    = 1 ;
   if ((flagch & U_FLAG)     > 0)  *udflag    = 1 ;
   if ((flagch & I_FLAG)     > 0)  *idflag    = 1 ;
   if ((flagch & DIR_FLAG)   > 0)  *posflag   = 1 ;
   if ((flagch & DIR_FLAG)  == 0)  *negflag   = 1 ;


   return (0) ;
}
