/*	sun_eph.c	sun's coordinates				*/
/*									*/
/*	13 FEB 85: phw							*/
/*	21 FEB 86: phw	 move def of fd2r to math.h			*/
/************************************************************************/

#ifdef	VxWorks
#include "vxWorks.h"
#include	"../h/math.h"
#include	"../h/time.h"
#include	"../h/astro.h"
#else
#include	"astro.h"
#endif

sun_eph (n,ut, ra,dec, diam,p,b,l0)
long	n,				/* days since EPOCH */
	ut;				/* and TICKS into day 'n' */
ANGLE	*ra, *dec;			/* sun's R.A. and Dec */ 
ANGLE	*diam;				/* solar semi-diameter, as angle */
ANGLE	*p,				/* position angle of solar pole */
	*b, *l0;			/* center disk heliographic coords. */
{
#define	sin_I	((AMP) (.126198969))/* inclination of solar equator */
#define cos_I	((AMP) (.992004950))/* is 7.25 degrees */
#define	J1854	2398220			/* julian date for 1854 Jan 1.5 */
  ANGLE	L, g, lambda, xi, two_g;
  AMP	sin_l, cos_l, sin_xi, cos_xi;
  ANGLE	Omega, LmO;
  AMP	sin_LmO, cos_LmO, r, temp;

		/* (Follows low precision method in		*/
		/* "The Astronomical Almanac", 1984, p C24.	*/
  L = fd2r(280.460) + fd2r(.9856474)*n;	/* aberration corrected mean long */
  g = fd2r(357.528) + fd2r(.9856003)*n;	/* mean anomaly */
  two_g = g*2;
  lambda = L + fd2r(1.915)*SIN(g)
	+ fd2r(.02)*SIN(two_g);		/* ecliptic longitude */
  sin_l = SIN(lambda); cos_l = COS(lambda);
  xi = fd2r(23.439) - fd2r(.0000004)*n;	/* obliquity of ecliptic */
  sin_xi = SIN(xi); cos_xi = COS(xi);
  *ra = ATAN2(cos_xi*sin_l,cos_l);	/* right ascension */
  temp = sin_xi*sin_l;
  *dec= ASIN(temp);			/* declination */
  r = .00014 - .01671*COS(g)
	- .00014*COS(two_g);		/* R = earth/sun distance = 1 + r */
					/* 1/(1+r) = 1 - r + r**2 - r**3 ... */
  *diam = fd2r(0.2666)*(1./(1.+r)); /* solar semi-diameter, as ANGLE */

		/* Follows method in "Explanatory Supplement to		*/
		/* the Astronomical Ephemeris", 1961, pp 307-310.	*/
		/* Omega is longitude of solar equator's ascending node */
  Omega = fd2r(73.666667)		/* epoch of 1850 */
	+ fd2r((EPOCH-1850.)*50.25/3600.)/* current EPOCH */
	+ fd2r(50.*50.25/3600)*(AMP) (n*2)/36525.;	
/* change in 1/2 century */
/* fraction of 1/2 century */
  LmO = lambda - Omega;
  sin_LmO = SIN(LmO); cos_LmO = COS(LmO);
  *p = ATAN2(-cos_l*sin_xi,cos_xi)
	+ ATAN2(-cos_LmO*sin_I,cos_I);
  temp = sin_LmO*sin_I;
  *b = ASIN(temp);			/* latitude center of sun */
  *l0 = ATAN2(sin_LmO*cos_I,cos_LmO)
	+ fd2r((J1854-JD_EPOCH)*360./25.38+180)
	- fd2r(360./25.38)*n;		/* Carrington longitude */
#ifdef	DEBUG0
#define	DEGREES(x)	(((double) x)*180/PI)
  printf("L       =%12.4f\ng       =%12.4f\nlambda  =%12.4f\nxi      =%12.4f\n",
	fr2d(L), fr2d(g), fr2d(lambda), fr2d(xi));
  printf("Omega   =%12.4f\nLmO     =%12.4f\nsin_LmO =%f\n",
	fr2d(Omega), fr2d(LmO), ((double) sin_LmO));
#endif
  }
