/* suncir4.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * S U N C I R 4 ( )	- Draw a dotted circular pattern at 1.0 Rsun.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 26 Jul 1990: original program: suncir.c
 * 29 Aug 1990: make dots 2 pixels high.
 *  4 Mar 1991: make "dots" plus marks.
 *  4 Apr 1991: SGI/GL version.
 *  5 Apr 2000: Adapt for image array (instead of SGI window).
 *                For short integer images.
 *                Mark-IV version.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */

#include <stdio.h>

   int xbeg, xend ;
   int ybeg, yend ;

suncir4 (sbuf, xdim, ydim, xcen, ycen, rspix, roll)

short	*sbuf ;		/* Short integer array pointer. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
float	xcen ;		/* X-axis center. */
float	ycen ;		/* Y-axis center. */
float	rspix ;		/* Rsun/pixel. */
float	roll ;		/* Roll (degrees from north). */

{
   extern int pcoord () ;
   short smax = 2000 ;
   int xg, yg ;			/* Pixel coordinates. */
   int xg1, yg1, xg2, yg2 ;

   float r1min, r1max ;	/* radial limits for major dotted lines. */
   float r1inc ;	/* radial increment for major dotted lines. */
   float ang1inc ;	/* angular increment between minor lines.  */
   float r2min, r2max ;	/* radial limits for minor dotted lines. */
   float r2inc ;	/* radial increment for minor dotted lines. */
   float ang2inc ;	/* angular increment between minor lines.  */

   float radius ;
   float th ;		/* Position angle in degrees. */
   float x, y ;		/* (x,y) coordinates corresponding to (r,th). */

   float r = 1.0 ;	/* Radius in R/Rsun units. */

   float x1, y1, x2, y2 ;
   float r1, r2 ;

   r1min = 0.2 ;		/* Radial  inner limit for major lines. */
   r1max = 1.0  ;		/* Radial  outer limit for major lines. */
   r1inc = 0.1  ;		/* Radial  increment for major lines. */
   ang1inc = 90.0 ;		/* Angular increment between major lines. */

   r2min = 0.0 ;		/* Radial  inner limit for minor lines. */
   r2max = 1.0 ;		/* Radial  outer limit for minor lines. */
   r2inc = 0.2 ;		/* Radial  increment for minor lines. */
   ang2inc = 30.0 ;		/* Angular increment between minor lines. */

   xbeg = 0 ;
   xend = xdim - 1 ;
   ybeg = 0 ;
   yend = ydim - 1 ;

   /*** Draw "+" mark at sun center. ***/

   xg = (int) (xcen + 0.5) ;
   yg = (int) (ycen + 0.5) ;
   *(sbuf + xg   + yg*xdim) = smax ;
   *(sbuf + xg-1 + yg*xdim) = smax ;
   *(sbuf + xg-2 + yg*xdim) = smax ;
   *(sbuf + xg+1 + yg*xdim) = smax ;
   *(sbuf + xg+2 + yg*xdim) = smax ;
   *(sbuf + xg   + (yg)  *xdim) = smax ;
   *(sbuf + xg   + (yg-1)*xdim) = smax ;
   *(sbuf + xg   + (yg-2)*xdim) = smax ;
   *(sbuf + xg   + (yg+1)*xdim) = smax ;
   *(sbuf + xg   + (yg+2)*xdim) = smax ;

   /*** Draw radial scans (r=r1min -> r1max) every ang1inc degrees. ***/

   for (th = 0.0 ; th < 360.0 ; th += ang1inc)
      {
/*      if (th == 180.0)  r1min = 0.2 ;  else r1min = 0.4 ; */
      for (radius = r1min ; radius <= r1max * 1.01 ; radius += r1inc)
         {
	 pcoord (&radius, &th, &x, &y, xdim, ydim, xcen, ycen, rspix, roll, 1) ;
	 xg = (int) (x + 0.5) ;
	 yg = (int) (y + 0.5) ;
/*
 *         if (xg >= xbeg && xg <= xend &&
 *             yg >= ybeg && yg <= yend)
 */
            {
	    *(sbuf + xg   + yg*xdim) = smax ;
/*
	    *(sbuf + xg-1 + yg*xdim) = smax ;
	    *(sbuf + xg+1 + yg*xdim) = smax ;
	    *(sbuf + xg   + (yg-1)*xdim) = smax ;
	    *(sbuf + xg   + (yg+1)*xdim) = smax ;
 */
            }
	 }
      }

   /*** Draw radial scans (r=r2min -> r2max) every ang2inc degrees. ***/

   for (th = 0.0 ; th < 360.0 ; th += ang2inc)
      for (radius = r2min ; radius < r2max ; radius += r2inc)
         {
	 pcoord (&radius, &th, &x, &y, xdim, ydim, xcen, ycen, rspix, roll, 1) ;
	 xg = (int) (x + 0.5) ;
	 yg = (int) (y + 0.5) ;
/*
 *         if (xg >= xbeg && xg <= xend &&
 *             yg >= ybeg && yg <= yend)
 */
            {
	    *(sbuf + xg + yg*xdim) = smax ;
            }
	 }

   /*** Draw sun circle at 1.0 Rsun.  One dot every 10 degrees. ***/

   for (th = 0 ; th < 360 ; th += 10)
      {
      pcoord (&r, &th, &x, &y, xdim, ydim, xcen, ycen, rspix, roll, 1) ;
      xg = (int) (x + 0.5) ;
      yg = (int) (y + 0.5) ;
/*
 *      if (xg >= xbeg && xg <= xend &&
 *          yg >= ybeg && yg <= yend)
 */
         {
	 *(sbuf + xg + yg*xdim) = smax ;
         }
      }

   /*** Draw sun circle at 1.0 Rsun.  One BOLD dot every 30 degrees. ***/

   for (th = 0 ; th < 360 ; th += 30)
      {
      pcoord (&r, &th, &x, &y, xdim, ydim, xcen, ycen, rspix, roll, 1) ;
      xg = (int) (x + 0.5) ;
      yg = (int) (y + 0.5) ;
/*
 *      if (xg >= xbeg && xg <= xend &&
 *          yg >= ybeg && yg <= yend)
 */
/*
         {
	 *(sbuf + xg + yg*xdim) = smax ;
         *(sbuf + xg-1 + yg*xdim) = smax ;
         *(sbuf + xg+1 + yg*xdim) = smax ;
         *(sbuf + xg   + (yg-1)*xdim) = smax ;
         *(sbuf + xg   + (yg+1)*xdim) = smax ;
         }
 */
      }

   return (0) ;
}
