/* blintd.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * img : character pointer to array containing image.
 * x,y:  floating point pointers to x & y position in image.
 * wint returns a double precision value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    double wint ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, floating-point,
 * or double images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 *  6 Mar 2003 : Add double precision image type.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

double
blintd (img, nbypix, xdim, ydim, x, y)

unsigned char *img;	/* Pointer to image array. */
int	nbypix ;	/* # bytes per pixel. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
double	x, y;		/* Coordinates of scan point in image. */

{
   char   *im;
   short  *ims;
   float  *imf;
   double *imd;
   int	 ix, iy;
   double fx, fy;
   double f00, f10, f01, f11;
   double fx0, fx1, fxy;
  
/*
   fprintf (stderr, "blint> nbypix: %d  xdim/ydim: %4d %4d  x/y: %7.2f %7.2f\n",
	    nbypix, xdim, ydim, x, y) ;
 */

   if (x < 0.0 || x > (double) xdim)
      {
/*      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ; */
      return (-1.0) ;
      }

   if (y < 0.0 || y > (double) ydim)
      {
/*      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ; */
      return (-2.0) ;
      }


   ix = (int) x;
   iy = (int) y;

   fx = x - (double) ix;
   fy = y - (double) iy;

   /*** Select 4 nearest neighbor elements in 'img' array. ***/

   /*** 1 byte per pixel. ***/

   if (nbypix == 1)
      {
      im  = (char *) img;
      f00 = (double) (*(im +  ix    +  iy*xdim) & 0377);
      f10 = (double) (*(im + (ix+1) +  iy*xdim) & 0377);
      f01 = (double) (*(im +  ix    + (iy+1)*xdim) & 0377);
      f11 = (double) (*(im + (ix+1) + (iy+1)*xdim) & 0377);
      }
   else

   /*** 2 bytes per pixel (short integer). ***/

   if (nbypix == 2)
      {
      ims = (short *) img;
      f00 = (double) *(ims +  ix    +  iy*xdim);
      f10 = (double) *(ims + (ix+1) +  iy*xdim);
      f01 = (double) *(ims +  ix    + (iy+1)*xdim);
      f11 = (double) *(ims + (ix+1) + (iy+1)*xdim);
      }
   else

   /*** 4 bytes per pixel (doubleing point). ***/

   if (nbypix == 4)
      {
      imf = (float *) img;
      f00 = (double) *(imf +  ix    +  iy*xdim);
      f10 = (double) *(imf + (ix+1) +  iy*xdim);
      f01 = (double) *(imf +  ix    + (iy+1)*xdim);
      f11 = (double) *(imf + (ix+1) + (iy+1)*xdim);
      }

   if (nbypix == 8)
      {
      imd = (double *) img;
      f00 = (double) *(imd +  ix    +  iy*xdim);
      f10 = (double) *(imd + (ix+1) +  iy*xdim);
      f01 = (double) *(imd +  ix    + (iy+1)*xdim);
      f11 = (double) *(imd + (ix+1) + (iy+1)*xdim);
      }

   /*** Bilinear interpolation. ***/

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}
