/* blintd.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * img : character pointer to array containing image.
 * x,y:  floating point pointers to x & y position in image.
 * wint returns a double precision value representing the intensity
 *    value at position (x,y).
 * Remember to include the statement
 *    double blintd ();
 * in the calling routine.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This routine will accept byte, short integer, floating-point,
 * or double images. 
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * C programming language
 * Andrew L. Stanger
 *  8 Mar 2003 : Use double precision image type.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

/* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

double
blintd (img, xdim, ydim, x, y)

double *img;		/* Pointer to image array. */
int	xdim ;		/* X-axis dimension. */
int	ydim ;		/* Y-axis dimension. */
double	x, y;		/* Coordinates of scan point in image. */

{
   int	 ix, iy;
   int pmode = 0 ;
   double fx, fy;
   double f00, f10, f01, f11;
   double fx0, fx1, fxy;
  
   if (pmode)
      fprintf (stderr,
	       "blintd> xdim/ydim: %4d %4d  x/y: %7.2f %7.2f\n",
	       xdim, ydim, x, y) ;

   if (x < 0.0 || x > (double) xdim)
      {
      fprintf (stderr, "Invalid X-axis coordinate: %f\n", x) ;
      return (-1.0e+30) ;
      }

   if (y < 0.0 || y > (double) ydim)
      {
      fprintf (stderr, "Invalid Y-axis coordinate: %f\n", y) ;
      return (-2.0e+30) ;
      }

   /*** Get integer component of each coordinate. ***/

   ix = (int) x;
   iy = (int) y;

   /*** Compute fractional distance of point from lower left neighbor. ***/

   fx = x - (double) ix;
   fy = y - (double) iy;

   /*** Select 4 nearest neighbor elements in 'img' array. ***/

   f00 = *(img +  ix    +  iy    * xdim);
   f10 = *(img + (ix+1) +  iy    * xdim);
   f01 = *(img +  ix    + (iy+1) * xdim);
   f11 = *(img + (ix+1) + (iy+1) * xdim);

   /*** Bilinear interpolation. ***/

   fx0 = f00 + fx * (f10 - f00);
   fx1 = f01 + fx * (f11 - f01);
   fxy = fx0 + fy * (fx1 - fx0);

   return (fxy);
}
