/* carhg3.c
 *------------------------------------------------------------------------------
 * Convert FITS Cartesian Coordinate images to Heliographic Coordinates.
 *
 * Uses cfitsio library for FITS operations.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   20 Nov 2002
 * Use interpolation = 3 (modified bilinear).
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>
#include "/opt/share/PUBLIC/include/fitsio.h"
#define MAXFILES 250
#define NLONG 720 
#define NLAT  720
extern char *malloc () ;

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern int strtrim () ;
    extern int lowercase () ;

    static char ksimple   [12] = "SIMPLE"   ;
    static char kbitpix   [12] = "BITPIX"   ;
    static char knaxis    [12] = "NAXIS"    ;
    static char knaxis1   [12] = "NAXIS1"   ;
    static char knaxis2   [12] = "NAXIS2"   ;
    static char kdate_obs [12] = "DATE-OBS" ;
    static char ktime_obs [12] = "TIME-OBS" ;
    static char ktimesys  [12] = "TIMESYS"  ;
    static char korigin   [12] = "ORIGIN"   ;
    static char ktelescop [12] = "TELESCOP" ;
    static char kinstrume [12] = "INSTRUME" ;
    static char kcode_obs [12] = "CODE-OBS" ;
    static char ktype_obs [12] = "TYPE-OBS" ;
    static char kobject   [12] = "OBJECT"   ;
    static char ktype_dp  [12] = "TYPE-DP"  ;
    static char kctype1   [12] = "CTYPE1"   ;
    static char kcrpix1   [12] = "CRPIX1"   ;
    static char kcrval1   [12] = "CRVAL1"   ;
    static char kcdelt1   [12] = "CDELT1"   ;
    static char kcrota1   [12] = "CROTA1"   ;
    static char kctype2   [12] = "CTYPE2"   ;
    static char kcrpix2   [12] = "CRPIX2"   ;
    static char kcrval2   [12] = "CRVAL2"   ;
    static char kcdelt2   [12] = "CDELT2"   ;
    static char kcrota2   [12] = "CROTA2"   ;
    static char kcoordnam [12] = "COORDNAM" ;
    static char kcarr_rot [12] = "CARR_ROT" ;
    static char ksolar_b0 [12] = "SOLAR_B0" ;
    static char ksolar_l0 [12] = "SOLAR_L0" ;
    static char kdatamin  [12] = "DATAMIN"  ;
    static char kdatamax  [12] = "DATAMAX"  ;
    static char kdispmin  [12] = "DISPMIN"  ;
    static char kdispmax  [12] = "DISPMAX"  ;

    static char comment   [80] ;
    static char csimple   [50] = "FITS FILE" ;
    static char cbitpix   [50] = "BITS/PIXEL: 8, 16, 32, -32, -64" ;
    static char cnaxis    [50] = "NUMBER OF AXES" ;
    static char cnaxis1   [50] = "LONGITUDE DIMENSION" ;
    static char cnaxis2   [50] = "LATITUDE  DIMENSION" ;
    static char cdate_obs [50] = "OBSERVATION DATE: CCYY-MM-DD [UTC]" ;
    static char ctime_obs [50] = "OBSERVATION TIME: HH:MM:SS [UTC]" ;
    static char ctimesys  [50] = "COORDINATED UNIVERSAL TIME" ;
    static char corigin   [50] = "MAUNA LOA SOLAR OBSERVARORY" ;
    static char ctelescop [50] = "SOLAR INSTRUMENT" ;
    static char cinstrume [50] = "KODAK MEGAPLUS 1.6 CCD" ;
    static char ccode_obs [50] = "TYPE OF OBSERVATION [CHARACTER CODE]" ;
    static char ctype_obs [50] = "TYPE OF OBSERVATION [DESCRIPTION]" ;
    static char cobject   [50] = "SOURCE OBJECT" ;
    static char ctype_dp  [50] = "DATA PROCESSING TYPE" ;
    static char cctype1   [50] = "AXIS 1 TYPE: [LONGITUDE: 0 -> 360]" ;
    static char ccrpix1   [50] = "LONGITUDE ORIGIN [PIXELS]" ;
    static char ccrval1   [50] = "LONGITUDE ORIGIN [DEGREES]" ;
    static char ccdelt1   [50] = "LONGITUDE INCREMENT [DEG/PIXEL]" ;
    static char ccrota1   [50] = "LONGITUDE ROTATION [DEG]" ;
    static char cctype2   [50] = "AXIS 2 TYPE: [SIN (LATITUDE): -1 -> +1]" ;
    static char ccrpix2   [50] = "SIN (LATITUDE)  ORIGIN [PIXELS]" ;
    static char ccrval2   [50] = "SIN (LATITUDE)  ORIGIN [DEG]" ;
    static char ccdelt2   [50] = "SIN (LATITUDE)  INCREMENT [SIN/PIXEL]" ;
    static char ccrota2   [50] = "SIN (LATITUDE)  ROTATION [DEG]" ;
    static char ccoordnam [50] = "HELIOGRAPHIC: [LONGITUDE, SIN (LATITUDE)]" ;
    static char ccarr_rot [50] = "CARRINGTON ROTATION NUMBER" ;
    static char csolar_b0 [50] = "CARRINGTON  LATITUDE [DEG]" ;
    static char csolar_l0 [50] = "CARRINGTON LONGITUDE [DEG]" ;
    static char cdatamin  [50] = "DATA MINIMUM" ;
    static char cdatamax  [50] = "DATA MAXIMUM" ;
    static char cdispmin  [50] = "DISPLAY MINIMUM" ;
    static char cdispmax  [50] = "DISPLAY MAXIMUM" ;

    static char code_obs  [24] ;
    static char date_obs  [24] ;
    static char imgdate   [24] ;
    static char imgtime   [24] ;
    static char time_obs  [24] ;
    static char timesys   [24] ;
    static char origin    [24] ;
    static char telescop  [24] ;
    static char instrume  [24] ;
    static char type_obs  [24] ;
    static char object    [24] ;
    static char type_dp   [24] ;
    static char ctype1    [24] = "LONGITUDE" ;
    static char ctype2    [24] = "SIN (LATITUDE)" ;
    static char coordnam  [24] ;
    static char out_name  [80] ;
    static char filename  [80] ;
    static char file      [80] ;
    static char filelist  [80] ;
    static char flist     [MAXFILES][80] ;
    static char orient    [8] = "SESW" ;
    static char code_tscp [12] ;   /* Telescope name [3-characters]. */
    static char wsr       [12] ;   /* wsr: wavelength/source/reduction code. */

    float *imgbuf1 ;
    static float imgbuf2 [NLAT][NLONG];

    int argp = 1 ;
    int bitpix = -32 ;
    int carr_rot = 0;			/* Carrington Rotation number. */
    int i ;
    int ierr ;
    int ix, iy ;
    int interp = 3 ;	/* Interpolation: 1=cubic, 2=bilinear, 3=modified.*/
    int status, nfound, anynull ;
    int naxis = 2 ;
    int naxis1, naxis2 ;
    int nf ;
    int numfil ;			/* # images in file list. */
    int pmode = 1 ;			/* Print mode: =1 verbose. */
    int seq = 0 ;
    int true = 1 ;

    long fpixel = 1 ;
    long naxes [2] = {NLONG, NLAT} ;
    long npix1, npix2 ;
    long xdim1, ydim1 ;			/* Dimensions: Cartesian image. */
    long xdim2, ydim2 ;			/* Dimensions: Heliographic image. */

    float crpix1 = 0.0 ;		/* Longitude origin [pixels]. */
    float crval1 = 0.0 ;		/* Longitude origin [degrees]. */
    float cdelt1 = 0.0 ;		/* Longitude increment [deg]. */
    float crota1 = 0.0 ;		/* Longitude rotation [deg]. */

    float crpix2 = 0.0 ;		/* Latitude  origin [pixels]. */
    float crval2 = 0.0 ;		/* Latitude  origin [sin]. */
    float cdelt2 = 0.0 ;		/* Latitude  increment [sin]. */
    float crota2 = 0.0 ;		/* Latitude  rotation [deg]. */

    float datamin, datamax ;
    float dispmin, dispmax ;
    float fmin1, fmax1 ;
    float fmin2, fmax2 ;
    float fval ;
    float minint ;
    float nullval  = 0.0 ;	/* Don't check for null values in the image. */
    float solar_p0 ;		/* P-angle  [rotation].  */
    float solar_b0 ;		/* B0 angle [tilt].      */
    float solar_l0 ;		/* L0 angle [longitude]. */
    float crradius ;		/* Solar radius [pixels]. */
    float rsun ;		/* Solar radius [arcsec]. */
    float xcen, ycen ;		/* Sun center   [pixels]. */

    double pi = 3.14159265 ;	/* pi */
    double deg_arcsec ;		/* Degrees per arcsec. */
    double rad_deg ;		/* Radians per degree. */
    double deg_rad ;		/* Degrees per radian. */

    double Bo ;
    double Po ;

    /*** Longitude range: -90 to +90 degrees [-pi/2 to +pi/2 radians] ***/
    /*** sin (Lat) range: -1  to +1 [effectively, -90 to +90 degrees. ***/
    /*** Note: all angle parameter inputs to obs2helio () are in RADIANS ! ***/

    double Lmin      =  -1.56207 ;/* Start of Longitude range [pi*(-89.5/180)]*/
    double Ldelta    =   0.01768 ;/* Longitude increment per pixel [pi/180]   */
    double sinBdelta =   0.01111 ;/* sin Latitude increment per pixel [1/90]  */
    double rsunrad ;		/* Solar radius [radians]. */
    double rsunpix ;		/* Solar radius [pixels].  */
    double Rmax   = 0.98 ;	/* Maximum disc radius to use. */
    double sangle = 0.0 ;	/* Position angle of semi-major axis. */
    double smajor = 1.0 ;	/* Scale for semi-major axis. */
    double sminor = 1.0 ;	/* Scale for semi-minor axis. */
    double xscale = 2.29 ;	/* arcsec/pixel */
    double yscale = 2.29 ;	/* arcsec/pixel */
    double x0, y0 ;		/* Sun center coordinates Helio image.*/

    fitsfile *fp1, *fp2 ;   /* Pointer to the FITS file, defined in fitsio.h */
    FILE *FD ;

    /*** Constants. ***/

    xdim2 = NLONG ;
    ydim2 = NLAT  ;
    npix2 = xdim2 * ydim2 ;
    
    deg_arcsec = 1.0 / 3600.0 ;			/* degrees/arcsec */
    rad_deg = pi / 180.0 ;			/* radians/degree */
    deg_rad = 180.0 / pi ;			/* degrees/radian */

    /*** Lmin      = pi * (-89.5 / NLONG) ***/
    /*** Ldelta    = pi / NLONG           ***/
    /*** sinBdelta = 2.0 / NLAT           ***/

    Lmin      =    pi * (-89.5 / 180.0) ;
    Lmin      =    pi * (-90.0 / 180.0) ;
    Ldelta    =    pi / NLONG ;
    sinBdelta =   2.0 / NLAT ;

    /*** Get the FITS file name[s] from the command line. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s fits_file_name[s]\n",
               argv [0]) ;
      exit (0) ;
      }
   else
   if (argc > MAXFILES)
      {
      fprintf (stderr, "Maximum number of input files: %d\n", MAXFILES) ;
      exit (-1) ;
      }
   else
   if (argc == 2)
      {
      strcpy (filename, argv [1]) ;
      strcpy (&flist [0][0], filename) ;
      numfil = 1 ;
      }
   else
   if (argc >= 3)
      {
      nf  = 0 ;
      seq = 1 ;
      while (argp < argc)
	 {
         strcpy (&flist [nf++][0], argv [argp]) ;
	 argp++ ;
	 }
      numfil = nf ;
      }

    /*** File loop. ***/

    for (i = 0; i < numfil; i++)
	{
	strcpy (filename, &flist [i][0]) ;
	if (pmode)
            printf ("filename: %s\n", filename) ;

        status = 0 ;
    
        /*** Open FITS file. ***/
    
        if ( fits_open_file (&fp1, filename, READWRITE, &status) )
             printerror ( status ) ;
    
        /*** Read the NAXIS1 and NAXIS2 keyword to get image size. ***/
    
        if ( fits_read_key (fp1, TINT, "NAXIS1", &xdim1, comment, &status) )
             printerror ( status ) ;
        if ( fits_read_key (fp1, TINT, "NAXIS2", &ydim1, comment, &status) )
             printerror ( status ) ;

        npix1 = xdim1 * ydim1 ;

	/*** Read date & time. ***/

        if ( fits_read_key (fp1, TSTRING, "DATE-OBS", date_obs, cdate_obs,
	     &status) ) printerror ( status ) ;
	strtrim (date_obs, '-', imgdate) ;

        if ( fits_read_key (fp1, TSTRING, "TIME-OBS", time_obs, ctime_obs,
	     &status) ) printerror ( status ) ;
	strtrim (time_obs, ':', imgtime) ;

        if ( fits_read_key (fp1, TSTRING, "TIMESYS", timesys, ctimesys,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "ORIGIN", origin, corigin,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "TELESCOP", telescop, ctelescop,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "INSTRUME", instrume, cinstrume,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "CODE-OBS", code_obs, ccode_obs,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "TYPE-OBS", type_obs, ctype_obs,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "OBJECT", object, cobject,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "TYPE-DP", type_dp, ctype_dp,
	     &status) ) printerror ( status ) ;

        if ( fits_read_key (fp1, TSTRING, "COORDNAM", coordnam, ccoordnam,
	     &status) ) printerror ( status ) ;

	strtrim (telescop, ' ', code_tscp) ;
	if (strcmp (code_tscp, "chip") == 0) sprintf (code_tscp, "chp") ;
	if (strcmp (code_tscp, "CHIP") == 0) sprintf (code_tscp, "chp") ;
	if (pmode)
	    printf ("code_tscp: %s\n", code_tscp) ;

	strtrim (code_obs, ' ', wsr) ;
	wsr [2] = 'g' ;	/* reduction = "g" => heliographic coordinates. */
	if (pmode)
	    printf ("wsr: %s\n", wsr) ;
	sprintf (out_name, "!%s.%s.%s.%s3.fts",
		 imgdate, imgtime, code_tscp, wsr) ;
        if (pmode)
	    printf ("out_name: %s\n", out_name) ;

	lowercase (out_name) ;

	printf ("in/out: %s --> %s\n", filename, out_name) ;

/*
 *        if ( fits_read_key (fp1, TSTRING, "CTYPE1", ctype1, comment,
 *	     &status) ) printerror ( status ) ;
 *
 *        if ( fits_read_key (fp1, TSTRING, "CTYPE2", ctype2, comment,
 *	     &status) ) printerror ( status ) ;
 */

	/*** Read CRPIX1 & CRPIX2 keyword values [sun center]. ***/

        if ( fits_read_key (fp1, TFLOAT, "CRPIX1", &crpix1, comment, &status) )
             printerror ( status ) ;
        if ( fits_read_key (fp1, TFLOAT, "CRPIX2", &crpix2, comment, &status) )
             printerror ( status ) ;
    
	/*** Read CDELT1 & CDELT2 keyword values [pixel scale in arcsec]. ***/

        if ( fits_read_key (fp1, TFLOAT, "CDELT1", &cdelt1, comment, &status) )
             printerror ( status ) ;
        if ( fits_read_key (fp1, TFLOAT, "CDELT2", &cdelt2, comment, &status) )
             printerror ( status ) ;

	/*** Read CROTA1 & CROTA2 keyword values [rotation angle] ***/

        if ( fits_read_key (fp1, TFLOAT, "CROTA1", &crota1, comment, &status) )
             printerror ( status ) ;
        if ( fits_read_key (fp1, TFLOAT, "CROTA2", &crota2, comment, &status) )
             printerror ( status ) ;

	xscale = (double) cdelt1 ;		/* arcsec/pixel */
	yscale = (double) cdelt2 ;		/* arcsec/pixel */
	xscale = 1.0 ;
	yscale = 1.0 ;
    
        /*** If CRPIX1 & CRPIX2 are positive, sun center is already known. ***/
    
	if (pmode)
            printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;

        if (crpix1 > 0.0 && crpix2 > 0.0)
    	    {
	    if (pmode)
                printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
	    xcen = crpix1 ;
	    ycen = crpix2 ;
    	    }
        else		/* Assume sun center is at array center. */
	    {
	    xcen = (xdim1 -1) / 2.0 ;
	    ycen = (ydim1 -1) / 2.0 ;
	    }

	x0 = (double) xcen ;
	y0 = (double) ycen ;

	/*** Read solar radius [pixels]. */

        if ( fits_read_key (fp1, TFLOAT, "CRRADIUS", &crradius, comment,
	     &status) )
             printerror ( status ) ;
	rsunpix = crradius ;

	/*** Read solar radius [arcsec]. ***/

        if ( fits_read_key (fp1, TFLOAT, "RSUN", &rsun, comment,
	     &status) )
             printerror ( status ) ;

	rsunrad = (rsun * deg_arcsec) * rad_deg ;   /* Solar radius [radians] */

        if (pmode)
	    printf ("rsun: %8.4f  deg_arcsec: %8.4f  rad_deg: %8.4f\n",
		 rsun, deg_arcsec, rad_deg) ;

	/*** Read P-angle, Bo angle, Lo angle keyword values. ***/

        if ( fits_read_key (fp1, TFLOAT, "SOLAR_P0", &solar_p0, comment,
	     &status) )
             printerror ( status ) ;

	if (pmode)
	    printf ("solar_p0: %9.4f\n", solar_p0) ;
	fflush (stdout) ;

        if ( fits_read_key (fp1, TFLOAT, "SOLAR_B0", &solar_b0, comment,
	     &status) )
             printerror ( status ) ;

	if (pmode)
	    printf ("solar_b0: %9.4f\n", solar_b0) ;
	fflush (stdout) ;

        if ( fits_read_key (fp1, TFLOAT, "SOLAR_L0", &solar_l0, comment,
	     &status) )
             printerror ( status ) ;

	if (pmode)
	    printf ("solar_l0: %9.4f\n", solar_l0) ;
	fflush (stdout) ;

        if ( fits_read_key (fp1, TINT,   "CARR_ROT", &carr_rot, comment,
	     &status) )
             printerror ( status ) ;

	if (pmode)
	    printf ("carr_rot: %4d\n", carr_rot) ;
	fflush (stdout) ;

	Bo = (double) (solar_b0 * rad_deg) ;		/* radians. */

	Po = (double) (solar_p0 * rad_deg) ;		/* radians. */
	Po = -(double) (crota1 * rad_deg) ;		/* radians. */

	while (solar_l0 <   0.0)  solar_l0 += 360.0 ;
	while (solar_l0 > 360.0)  solar_l0 -= 360.0 ;

	if (pmode)
	    {
            printf ("xdim1: %4d  ydim1: %4d\n", xdim1, ydim1) ;

	    printf ("x0: %7.2f  y0: %7.2f\n", x0, y0) ;

	    printf ("solar_p0: %7.2f  solar_b0: %7.2f  solar_l0: %7.2f\n",
		    solar_p0, solar_b0, solar_l0) ;
	    printf ("Bo: %7.2f  Po: %7.2f\n", Bo, Po) ;

	    printf ("rsunrad: %10.6f  rsunpix: %8.4f  Rmax: %7.2f\n",
		     rsunrad, rsunpix, Rmax) ;

	    printf ("interp: %1d\n", interp) ;

	    printf ("xdim2: %3d  ydim2: %3d\n", xdim2, ydim2) ;

	    printf ("Lmin: %8.4f  Ldelta: %8.4f  sinBdelta: %8.4f\n",
		    Lmin, Ldelta, sinBdelta) ;

            printf ("smajor: %7.2f  sminor: %7.2f  sangle: %7.2f\n",
		     smajor, sminor, sangle) ;

	    printf ("xscale: %7.2f  yscale: %7.2f\n", xscale, yscale) ;

	    printf ("orientation: %s\n", orient) ;
	    }

        /* Note that even though the FITS images contains unsigned integer */
        /* pixel values (or more accurately, signed integer pixels with    */
        /* a bias of 32768),  this routine is reading the values into a    */
        /* float array.   Cfitsio automatically performs the datatype      */
        /* conversion in cases like this.                                  */
    
        imgbuf1   = (float *) malloc (npix1 * sizeof (float)) ;
        if ( fits_read_img (fp1, TFLOAT, fpixel, npix1, &nullval,
                            imgbuf1, &anynull, &status) )
             printerror ( status ) ;

	/*** Find min/max values of input array. ***/

	fmin1 =  1.0e+30 ;
	fmax1 = -1.0e+30 ;
	for (iy = 0; iy < ydim1; iy++)
	    for (ix = 0; ix < xdim1; ix++)
		{
		fval = *(imgbuf1 + ix + iy * xdim1) ;
		if (fval < fmin1)  fmin1 = fval ;
		if (fval > fmax1)  fmax1 = fval ;
		}
	if (pmode)
	    printf ("fmin1: %9.2e  fmax1: %9.2e\n", fmin1, fmax1) ;
    
        /*** Close FITS input file. ***/
    
        if ( fits_close_file (fp1, &status) )  printerror ( status ) ;

	for (iy = 0; iy < ydim2; iy++)
	    for (ix = 0; ix < xdim2; ix++)
		{
		imgbuf2 [iy][ix] = 0.0 ;
		}
	
	/*** Convert image from Cartesian to Heliographics Coordinates. */

	ierr = obs2helio3 (imgbuf1, imgbuf2, xdim1, ydim1, x0, y0,
			   Bo, Po,
			   rsunrad, rsunpix, Rmax, interp, xdim2, ydim2,
			   Lmin, Ldelta, sinBdelta,
			   smajor, sminor, sangle, xscale, yscale,
			   orient) ;

	if (ierr != 0)
	    {
	    printf ("obs2helio error: %d\n", ierr) ;
	    exit (ierr) ;
	    }

	/*** Find min/max values of output array. ***/

	fmin2 =  1.0e+30 ;
	fmax2 = -1.0e+30 ;
	for (iy = 0; iy < ydim2; iy++)
	    for (ix = 0; ix < xdim2; ix++)
		{
		fval = imgbuf2 [iy][ix] ;
		if (fval < fmin2)  fmin2 = fval ;
		if (fval > fmax2)  fmax2 = fval ;
		}
	if (pmode)
	    printf ("fmin2: %9.2e  fmax2: %9.2e\n", fmin2, fmax2) ;
	datamin = fmin2 ;
	datamax = fmax2 ;
	dispmin = fmin2 ;
	dispmax = fmax2 ;

/*
	FD = fopen ("fh1.dat", "w") ;
	fwrite (imgbuf1, sizeof (float), npix1, FD) ;
	fclose (FD) ;

	FD = fopen ("fh2.dat", "w") ;
	fwrite (imgbuf2, sizeof (float), npix2, FD) ;
	fclose (FD) ;
 */
    
	/*** Create output file. ***/

	if ( fits_create_file (&fp2, out_name, &status) )
	    printerror ( status ) ;

	/*** Write required keywords. ***/

	if (fits_create_img (fp2, bitpix, naxis, naxes, &status))
	    printerror (status) ;

/*
	if (fits_write_key (fp2, TLOGICAL, ksimple, &true, csimple, &status)) ;
	    printerror (status) ;

	if (fits_write_key (fp2, TINT, kbitpix, &bitpix, cbitpix, &status))
	    printerror (status) ;

	if (fits_write_key (fp2, TINT, knaxis,  &naxis,  cnaxis,  &status))
	    printerror (status) ;

	if (fits_write_key (fp2, TINT, knaxis1, &xdim2,  cxdim2,  &status))
	    printerror (status) ;

	if (fits_write_key (fp2, TINT, knaxis2, &ydim2,  cydim2,  &status))
	    printerror (status) ;
 */

	/*** Write additional keywords. ***/

        crval1 = 0.0 ;			/* Longitude origin [deg]. */
        cdelt1 = Ldelta * deg_rad ;	/* Longitude increment [deg/pixel]. */
        crpix1 = (xdim2 - 1) / 2.0 ;	/* Longitude origin [pixels]. */

        crval2 = 0.0 ;			/* Latitude origin [sin].*/
        cdelt2 = sinBdelta ;		/* Latitude increment [sin/pixel]. */
        crpix2 = (ydim2 - 1) / 2.0 
           + sin (solar_b0 * rad_deg) / cdelt2 ; /* Latitude origin [pixels]. */

	if (pmode)
	    {
	    printf ("solar_b0: %9.4f  sin (solar_b0): %9.4f\n",
		     solar_b0, sin (solar_b0 * rad_deg)) ;
	    printf ("sin (solar_b0) / cdelt2: %9.4e\n",
		     sin (solar_b0 * rad_deg) / cdelt2) ;
	    }

	if (fits_write_key (fp2, TSTRING, kdate_obs, date_obs, cdate_obs,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, ktime_obs, time_obs, ctime_obs,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, ktimesys, timesys, ctimesys, 
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, korigin, origin, corigin,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, ktelescop, telescop, ctelescop,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, kinstrume, instrume, cinstrume,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, kcode_obs, code_obs, ccode_obs,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, ktype_obs, type_obs, ctype_obs,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, kobject, object, cobject,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, ktype_dp, type_dp, ctype_dp,
	    &status)) printerror (status) ;

	strcpy (coordnam, "HELIOGRAPHIC") ;
	if (fits_write_key (fp2, TSTRING, kcoordnam, coordnam, ccoordnam,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, kctype1, ctype1, cctype1,   
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcrpix1, &crpix1, ccrpix1,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcrval1, &crval1, ccrval1,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcdelt1, &cdelt1, ccdelt1,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcrota1, &crota1, ccrota1,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TSTRING, kctype2, ctype2, cctype2,   
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcrpix2, &crpix2, ccrpix2,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcrval2, &crval2, ccrval2,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcdelt2, &cdelt2, ccdelt2,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kcrota2, &crota2, ccrota2,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TINT,   kcarr_rot, &carr_rot,  ccarr_rot,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, ksolar_l0, &solar_l0,  csolar_l0,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, ksolar_b0, &solar_b0,  csolar_b0,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kdatamin, &datamin,  cdatamin,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kdatamax, &datamax,  cdatamax,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kdispmin, &dispmin,  cdispmin,
	    &status)) printerror (status) ;

	if (fits_write_key (fp2, TFLOAT, kdispmax, &dispmax,  cdispmax,
	    &status)) printerror (status) ;

	/*** Write output image array. ***/

	if (fits_write_img (fp2, TFLOAT, fpixel, npix2, imgbuf2,
	    &status)) printerror (status) ;

	fits_close_file (fp2, &status) ;
        }					/* End of file loop. */
    
    exit (0) ;
}
    
/* -------------------------------------------------------------------------- */

/*** Eliminate the character specified by "c" from a string. ***/

int
strtrim (s, c, t)

char s [] ;
char c ;
char t [] ;
{
    int is, it ;
    int nc ;
    int ns ;
    int nt ;
    int nrem ;
    int npad ;
/*    char *t ; */

    ns = strlen (s) ;

    nc = 0 ;
    for (is = 0; is < ns; is++)
	{
	if (s [is] != c)  nc++ ;
	}

/*
 *    printf ("strlen (%s): %d\n", s, ns) ;
 */

    nt = nc + 1 ;		
    nrem = nt % 4 ;
    if (nrem != 0) npad = 4 - nrem ;
    else           npad = 0 ;
    nt += npad ;

/*
 *    printf ("nc: %d  nrem: %d  npad: %d  nt: %d\n", nc, nrem, npad, nt) ;
 */

/*    t = (char *) malloc (nc) ; */

    it = 0 ;
    for (is = 0; is < ns; is++)
	if (s [is] != c)  t [it++] = s [is] ;

    for (it = nc; it < nt; it++)
       t [it] = '\0' ;

/*
 *    printf ("s: %s  t: %s\n", s, t) ;
 */
/*    return (t) ; */
    return (0) ;
}

/* -------------------------------------------------------------------------- */

/*** Convert string to lower case. ***/

int
lowercase (s)

char s [] ;

{
    int i ;
    int ns ;

    ns = strlen (s) ;

    for (i = 0; i < ns; i++)
       s [i] = (char) tolower ((int) s [i]) ;

    return (0) ;
}

/* -------------------------------------------------------------------------- */
