/* carr_jd_fn.c
 *
 * Convert Carrington Rotation & Latitude to Julian Date.
 *
 * Nick Arge   NOAA   20 Nov 2002   [carr_jd_fn.pro]
 *
 * Andrew Stanger   HAO/NCAR
 * 2 Dec 2002: Conversion to C-language.
 */

#include <stdio.h>
#include <math.h>

double
carr_jd_fn (ca, lon)

double ca ;
double lon ;

{

    /* Convert from Carrington Longitude and Latitude to (Julian Date -2440000)
     * Modified Nov. 20 2002
     */

    double pi = 3.14159265358979323846264338327 ;
    double JD_DD   = 0.0 ;
    double JD_low  = 0.0 ;
    double JD_orig = 0.0 ;
    double carrot  = 0.0 ;
    double lonmax  = 0.0 ;
    double incl    = 7.25 ;
    double Bo      = 0.0 ;
    double Lo      = 0.0 ;
    double nn ;
    double incr ;
    double ww ;
    double LL ;
    double dayfr ;
    double gg ;
    double sun ;
    double omega ;
    double Bosin ;
    double Bocos ;
    double LoWsin ;
    double LoWcos ;
    double long_diff ;
    double LoW ;

    long MM ;
    
/*
 *    printf ("carr_jd_fn> ca: %7.2f  lon: %9.4f\n", ca, lon) ;
 */

    carrot = (double) ca ;		/* Carrington Rotation.  */
    lonmax = (double) lon ;		/* Carrington Longitude. */
    
    if (lonmax == 0.0)
	{
        lonmax = 360.0 ;
        carrot = carrot + 1.0 ;
	}
    
    incl    = 7.25 ;
    JD_DD   = 0.0 ;
    JD_orig = 0.0 ;
    Lo      = 0.0 ;

    JD_DD = carrot + (360.0 - lonmax) / 360.0 ;
    JD_DD = JD_DD - 1750.0 ;
    JD_DD = JD_DD * 27.27530 ;
    JD_DD = JD_DD + 5871.91 ;
    
    JD_orig = JD_DD ;
    
    JD_low = JD_DD + 3.0 ;
    JD_DD  = JD_DD - 3.0 ;
    
    /*** For each of the dates labeling the bottom x-axis of the 1st plot. ***/
    
    incr = (double) (0.1) ;
    
    for (MM = 0; MM <= 10000; MM++)
	{
        JD_DD = JD_DD + incr ;

/*
 *        printf ("incr: %7.2e  JD_DD: %9.4f  long_diff: %9.4f\n",
 *		incr, JD_DD, long_diff) ;
 */
    
	/*
         * nn = JD_DD - 2451545.0
         * 2440000 - 2451545 = 11545
	 */
    
        nn = JD_DD - (double) 11545.0 ;
        LL = 280.472 + 0.9856474 * nn ;
        gg = 357.528 + 0.9856003 * nn ;
        sun = LL + 1.915 * sin (gg * pi / 180.0)
    	    + 0.020 * sin (2.0 * gg * pi / 180.0) ;
    
	/*
         * Explains where the number 43243.25 comes from
         * omega = 73.0 + 40.0/60.0+ 50.25*((JD_DD+2440000.0 - 
         * 2396756.75)/365.25)/3600.0 ;
	 */
    
        omega = 73.0 + 40.0 / 60.0 +
                50.25 * ((JD_DD + 43243.25) / 365.25) / 3600.0 ;
    
       /*
        * Explains where the number 41780.0 comes from
        * ww = 360.0 / 25.38 * (JD_DD + 2440000.0 - 2398220.0) ;
	*/
    
        ww = 360.0 / 25.38 * (JD_DD + 41780.0) ;
        ww = (ww / 360.0 - (long) (ww / 360.0)) * 360.0 ;
    
        Bosin = sin ((sun - omega) * pi / 180.0) * sin (incl * pi / 180.0) ;
        Bocos = sqrt (1.0 - Bosin * Bosin) ;
    
        Bo = atan2 (Bosin, Bocos) * 180.0 / pi ;
    
/*
 *	printf ("carr_jd_fn> Bo: %9.4f\n", Bo) ;
 *	printf ("carr_jd_fn> cos (Bo * pi / 180): %f\n", cos (Bo * pi / 180.0));
 *	fflush (stdout) ;
 */
    
        LoWsin = cos (incl * pi / 180.0) * sin ((omega - sun) * pi / 180.0)
    	       / cos (Bo * pi / 180.0) ;
        LoWcos = -cos ((omega - sun) * pi / 180.0) / cos (Bo * pi / 180.0) ;
        LoW = atan2 (LoWsin, LoWcos) * 180.0 / pi ;

/*
 *	printf ("LoWsin: %9.4f  LoWcos: %9.4f  LoW: %9.4f\n",
 *		 LoWsin, LoWcos, LoW) ;
 */
    
        if (LoW < 0.0)  LoW = LoW + 360.0 ;
        Lo = LoW - ww ;
    
        if (Lo < 0.0)  Lo = Lo + 360.0 ;
    
	long_diff = Lo - lonmax ;
	    
/*
 *        printf ("long_diff: %9.4f  Lo: %9.4f  lonmax: %9.4f  JD_DD: %9.4f\n",
 *		 long_diff, Lo, lonmax, JD_DD) ;
 */

    
        if ( (long_diff < 0.0) && (labs ((long) long_diff) < 100))
	    {
            JD_DD = JD_DD - incr ;
            incr = incr / 10.0 ;
            if (incr < 1.0e-6) goto DONE ;
	    }
    
        if ( (long_diff > 0.0) && (labs ((long) long_diff) > 100))
            { 
            JD_DD = JD_DD - incr ;
            incr = incr / 10.0 ;
            if (incr < 1.0e-6) goto DONE ;
            } 
        }
    
    DONE:
    
/*
 *    caldat ((JD_DD + 2440000.0), &mmm, &ddd, &yyy, &hour, &minute, &second) ;
 */

    dayfr = JD_DD - (long) (JD_DD) + 0.5 ;
    if (dayfr >= 1.0)  dayfr = dayfr - 1.0 ;
    
    return (JD_DD) ;
}
