/* carr_rotation.c
 *------------------------------------------------------------------------------
 * Convert Carrington Rotation to Julian Date.
 *
 * Nick Arge   NOAA   20 Nov 2002   [jd_carr.pro]
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR
 * 2 Dec 2002: Converted to C-language.
 *------------------------------------------------------------------------------
 */

#include <stdio.h>

double
carr_rotation (JD_temp)

double JD_temp ;

{
    extern double carr_jd () ;
    extern double helio_lat () ;
    extern double helio_long () ;

    double CARR_CM     = 0.0 ;
    double carr_increm = 0.0 ;
    double carr_test   = 0.0 ;
    double crdur       = 27.2753 ;		/* Average C.R. duration. */

    double JD_mod      = 0.0 ;			/* Julian Date. */
    double JD_CR0      = -41859.899931 ;
    double JD_test     = 0.0 ;
    double jd_offset   = 2440000.0 ;		/* Julian Date offset. */

/*
 *    double Bo          = 0.0 ;
 */
    double Lo          = 0.0 ;

    double num_rot     = 0.0 ;
    double pred_carr   = 0.0 ;
    double del_JD      = 0.0 ;
    double dayfr       = 0.0 ;

    int year, month, day ;
    int hour, minute, second ;


    JD_mod = JD_temp - jd_offset ;

    num_rot = (JD_mod - JD_CR0) / crdur ;
/*
 *    printf ("num_rot: %7.2f\n", num_rot) ;
 */

    if (num_rot >= 1000.0) carr_increm = 1000.0 ;
    else
    if (num_rot >=  100.0) carr_increm =  100.0 ;
    else
    if (num_rot >=   10.0) carr_increm =   10.0 ;
    else
    if (num_rot >=    1.0) carr_increm =    1.0 ;
    else
        CARR_CM = 0.0 ;

/*
 *    printf ("carr_increm: %9.4f\n", carr_increm) ;
 */

    carr_test = carr_increm ;
    JD_test = carr_jd (carr_test, (double) 0.0) ;

/*
 *    printf ("carr_test: %9.4f  JD_test: %9.4f\n", carr_test, JD_test) ;
 */

    while (JD_test <= JD_mod)
        {
        num_rot = (JD_mod - JD_test) / crdur ;


/*
 *        printf ("num_rot: %7.2f\n", num_rot) ;
 */


        if (num_rot >= 1000.0) carr_increm = 1000.0 ;
        else
        if (num_rot >=  100.0) carr_increm =  100.0 ;
        else
        if (num_rot >=   10.0) carr_increm =   10.0 ;
        else
        if (num_rot >=    0.0) carr_increm =    1.0 ;
  
/*
 *        printf ("carr_increm: %9.4f\n", carr_increm) ;
 */

        carr_test = carr_test + carr_increm ;
        JD_test   = carr_jd (carr_test, (double) 0.0) ;

/*
 *        printf ("carr_test: %9.4f  JD_test: %9.4f\n", carr_test, JD_test) ;
 */

        }

/*
 *    Bo        = helio_lat  (JD_mod + jd_offset) ;
 */

    Lo        = helio_long (JD_mod + jd_offset) ;
    pred_carr = carr_jd (carr_test, Lo) ;

    del_JD = labs ((long) (JD_mod - pred_carr)) ;
    if (del_JD > 20.0) 
	{
        if ((JD_mod - pred_carr) < 0.0)
	    carr_test = carr_test - 1.0 ;
	else
            carr_test = carr_test + 1.0 ;
        }

/*
 *    printf ("carr_test: %f  Lo: %f  Bo: %f\n", carr_test, Lo, Bo) ;
 */

    /*** Convert Julian day to calendar date. ***/

    caldat ((long)(JD_mod + jd_offset),
	    &month, &day, &year, &hour, &minute, &second) ;

    dayfr = JD_mod - (long) JD_mod + (double) 0.5 ;
    if (dayfr >= 1.0) dayfr = dayfr - 1.0 ;

    printf ("JD_mod: %14.7f JD_mod+offset: %14.5f\n",
	     JD_mod, JD_mod + jd_offset) ;
    printf ("month: %2d day: %2d year: %4d dayfr: %10.3f\n",
	     month, day, year, dayfr ) ;

/*
 *    printf ("carr_rotation: %f  Lo: %f  Bo: %f\n", carr_test, Lo, Bo) ;
 */
    printf ("carr_rotation: %f  Lo: %f\n", carr_test, Lo) ;

    return (carr_test) ;
}
