/* dwint2d.c
 * -----------------------------------------------------------------------------
 * 2-dimensional linear interpolation.
 * -----------------------------------------------------------------------------
 * C language
 * Usage example :
 * -------------
 * #define xdim 1024		Example value: x dimension original image.
 * #define ydim 1024		Example value: y dimension original image.
 * #define xidim 512		Example value: x dimension output image.
 * #define yidim 512		Example value: y dimension output image.
 *
 * extern double *int2d () ;
 * double img    [ydim][xdim] ;
 * double ximg   [yidim][xidim] ;
 * double yimg   [yidim][xidim] ;
 * double imgint [yidim][xidim] ;
 * double xcen, ycen ;			Sun center in original image.
 * double rsunpix ;			Solar radius (pixels) in original image.
 * double rmax ;
 * int ix, iy ;
 * int minpix ;
 *
 * for (iy = 0 ; iy < yidim; iy++)
 *     for (ix = 0 ; ix < xidim; ix++)
 *         {
 *         ximg [iy][ix] = ix * 2.0 + 0.5 ;	Example ximg equation.
 *         yimg [iy][ix] = iy * 2.0 + 0.5 ;	Example yimg equation.
 *         }
 *
 * Example ouput image is 1/2 the size of original image.
 *
 * imgint = fwint2d (img, &xdim, &ydim, &xcen, &ycen, &rsunpix, &rmax,
 *                   imgint, ximg, yimg, &xidim, &yidim, &minpix, &bkg) ;
 *
 * -----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   10 Mar 2003
 * 12 Mar 2003 : Convert from function to subroutine.
 * 13 Mar 2003 : Adapt for IDL [CALL_EXTERNAL].
 * 27 Mar 2003 : Double precision.
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>

/*
 * void
 * wint2d (img,                xdim,  ydim, xcen, ycen, rsunpix, rmax,
 *         imgint, ximg, yimg, xidim, yidim, minpix, bkg)
 */

void
fwint2d (argc, argv)

int argc ;
char *argv [] ;

{
    double  *img ;	/* Original image. */
    int     *xd ;	/* Original image x-dimension. */
    int     *yd ;	/* Original image y-dimension. */
    double  *xc ;	/* Original image x sun center. */
    double  *yc ;	/* Original image y sun center. */
    double  *rspix ;	/* Solar radius (pixels). */
    double  *radmax  ;	/* Maximum solar radius to use [Rsun]. */

    double  *imgint ;	/* Interpolated image. */
    double  *ximg ;	/* Array of x-axis coordinates for interpolation. */
    double  *yimg ;	/* Array of y-axis coordinates for interpolation. */
    int     *xid ;	/* New image x-dimension. */
    int     *yid ;	/* New image y-dimension. */
    int     *mpix ;	/* Minimum # pixels to use for interpolation. */
    double  *bkglev ;	/* Background intensity level to use. */

    int     xdim, ydim ;
    double  xcen, ycen ;
    double  rsunpix ;
    double  rmax ;
    int     xidim, yidim ;
    int     minpix ;
    double  bkg ;
    
    extern double fwintrb () ;
    double  xi, yi ;
    int     ix, iy ;

    /*** Get arguments from command line. ***/

    img    = (double *) argv [0] ;
    xd     = (int *)   argv [1] ;
    yd     = (int *)   argv [2] ;
    xc     = (double *) argv [3] ;
    yc     = (double *) argv [4] ;
    rspix  = (double *) argv [5] ;
    radmax = (double *) argv [6] ;
    imgint = (double *) argv [7] ;
    ximg   = (double *) argv [8] ;
    yimg   = (double *) argv [9] ;
    xid    = (int *)   argv [10] ;
    yid    = (int *)   argv [11] ;
    mpix   = (int *)   argv [12] ;
    bkglev = (double *) argv [13] ;

    xdim    = *xd ;
    ydim    = *yd ;
    xcen    = *xc ;
    ycen    = *yc ;
    rsunpix = *rspix ;
    rmax    = *radmax ;
    xidim   = *xid ;
    yidim   = *yid ;
    minpix  = *mpix ;
    bkg     = *bkglev ;
   
    for (iy = 0; iy < yidim; iy++)
	{
	for (ix = 0; ix < xidim; ix++)
	    {
	    xi = *(ximg + ix + iy * xidim) ;
	    yi = *(yimg + ix + iy * xidim) ;
	    *(imgint + ix + iy * xidim) =
		fwintrb (img, xdim, ydim, xi, yi, xcen, ycen,
			 rsunpix, rmax, bkg, minpix) ;
	    }
	}
}
