/* fh.c
 *------------------------------------------------------------------------------
 * Convert FITS 2D images to Heliographic Coordinates.
 *
 * Uses cfitsio library for FITS operations.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   20 Nov 2002
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include "/opt/share/PUBLIC/include/fitsio.h"
#define MAXFILES 250
extern char *malloc () ;

main (argc, argv)

int argc ;
char *argv [] ;

{
    extern float *rd_fits () ;

    static char comment  [80] ;
    static char filename [80] ;
    static char file [80] ;
    static char filelist [80] ;
    static char flist [MAXFILES][80] ;

    int argp = 1 ;
    int i ;
    int status, nfound, anynull ;
    int nf ;
    int numfil ;                         /* # images in file list. */
    int seq = 0 ;

    long xdim, ydim ;
    long fpixel, npix ;

    float crpix1, crpix2 ;
    float *imgbuf ;
    float minint ;
    float nullval ;
    float xrad, yrad ;
    float xcen, ycen ;

    fitsfile *fptr ;	/* Pointer to the FITS file, defined in fitsio.h */


    /*** Get FITS file name from command line. ***/

   if (argc == 1)
      {
      fprintf (stderr, "usage: %s fits_file_name[s]\n",
               argv [0]) ;
      exit (0) ;
      }
   else
   if (argc > MAXFILES)
      {
      fprintf (stderr, "Maximum number of input files: %d\n", MAXFILES) ;
      exit (-1) ;
      }
   else
   if (argc == 2)
      {
      strcpy (filename, argv [1]) ;
      strcpy (&flist [0][0], filename) ;
      numfil = 1 ;
      }
   else
   if (argc >= 3)
      {
      nf  = 0 ;
      seq = 1 ;
      while (argp < argc)
	 {
         strcpy (&flist [nf++][0], argv [argp]) ;
	 argp++ ;
	 }
      numfil = nf ;
      }

    for (i = 0; i < numfil; i++)
	{
	strcpy (filename, &flist [i][0]) ;
        printf ("filename: %s\n", filename) ;

        status = 0 ;
    
        /*** Open fits file. ***/
    
        if ( fits_open_file (&fptr, filename, READWRITE, &status) )
             printerror ( status ) ;
    
        /*** Read the NAXIS1 and NAXIS2 keyword to get image size. ***/
    
        if ( fits_read_key (fptr, TINT, "NAXIS1", &xdim, comment, &status) )
             printerror ( status ) ;
        if ( fits_read_key (fptr, TINT, "NAXIS2", &ydim, comment, &status) )
             printerror ( status ) ;
        
        printf ("xdim: %4d  ydim: %4d\n", xdim, ydim) ;
    
        if ( fits_read_key (fptr, TFLOAT, "CRPIX1", &crpix1, comment, &status) )
             printerror ( status ) ;
        if ( fits_read_key (fptr, TFLOAT, "CRPIX2", &crpix2, comment, &status) )
             printerror ( status ) ;
    
    
    
        /*** If CRPIX1 & CRPIX2 are positive, disc center is already known. ***/
    
            printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
    /*
        if (crpix1 > 0.0 && crpix2 > 0.0)
    	{
            printf ("crpix1/crpix2: %8.2f  %8.2f\n", crpix1, crpix2) ;
            printf ("Disc center is already known.  No need to recompute & update.\n") ;
    	exit (0) ;
    	}
     */
    
    
        /* Read FITS image data. */
    
        /* Note that even though the FITS images contains unsigned integer */
        /* pixel values (or more accurately, signed integer pixels with    */
        /* a bias of 32768),  this routine is reading the values into a    */
        /* float array.   Cfitsio automatically performs the datatype      */
        /* conversion in cases like this.                                  */
    
        npix     = xdim * ydim ;
        imgbuf   = (float *) malloc (npix * sizeof (float)) ;
        fpixel   = 1 ;
        nullval  = 0 ;               /* Don't check for null values in the image. */
        if ( fits_read_img (fptr, TFLOAT, fpixel, npix, &nullval,
                            imgbuf, &anynull, &status) )
             printerror ( status ) ;
    
    
        /*** Compute limb intensity value for solargeom routine. ***/
    
	limb_int_ (imgbuf, &xdim, &ydim, &minint) ;
     
        printf ("minint: %f\n", minint) ;

/*
 *        minint = *(imgbuf + xdim / 2 + ydim / 2 * xdim) / 8.0 ;
 *        minint = *(imgbuf + xdim / 2 + ydim / 2 * xdim) / 4.0 ;
 *        minint = *(imgbuf + xdim / 2 + ydim / 2 * xdim) / 2.0 ;
 */
    
        printf ("minint: %7.2f\n", minint) ;
    
        /*** Compute sun center & radius. ***/
    
        solargeom_(imgbuf, &minint, &xdim, &ydim, &xrad, &yrad, &xcen, &ycen);
    
    
        /*** Correct for origin at [0,0] instead of [1,1] (C <--> Fortran). ***/
    
        xcen -= 1.0 ;
        ycen -= 1.0 ;
    
        /*** Limit precision to 2 decimal places. ***/
    
        xcen = ((int)(xcen * 100.0) ) / 100.0 ;
        ycen = ((int)(ycen * 100.0) ) / 100.0 ;
        xrad = ((int)(xrad * 100.0) ) / 100.0 ;
        yrad = ((int)(yrad * 100.0) ) / 100.0 ;
    
        printf ("xcen/ycen: %7.2f %7.2f  xrad/yrad: %7.2f %7.2f\n",
    	     xcen, ycen, xrad, yrad) ;
    
        /*** Update CRPIX1 & CRPIX2. ***/
    
        if ( fits_update_key (fptr, TFLOAT, "CRPIX1", &xcen, comment, &status) )
    	printerror (status) ;
        if ( fits_update_key (fptr, TFLOAT, "CRPIX2", &ycen, comment, &status) )
    	printerror (status) ;
    
        /*** Close FITS file. ***/
    
        if ( fits_close_file (fptr, &status) )  printerror ( status ) ;
        }
    
    exit (0) ;
}
    
