/* int2d.c
 * -----------------------------------------------------------------------------
 * 2-dimensional linear interpolation.
 * -----------------------------------------------------------------------------
 * Usage example :
 * -------------
 * #define xdim 1024		Example value: x dimension original image.
 * #define ydim 1024		Example value: y dimension original image.
 * #define xidim 512		Example value: x dimension output image.
 * #define yidim 512		Example value: y dimension output image.
 *
 * extern double *int2d () ;
 * double img [ydim][xdim] ;
 * double ximg [yidim][xidim] ;
 * double yimg [yidim][xidim] ;
 * double *imgint ;
 * int ix, iy ;
 *
 * for (iy = 0 ; iy < yidim; iy++)
 *     for (ix = 0 ; ix < xidim; ix++)
 *         {
 *         ximg [iy][ix] = ix * 2.0 + 0.5 ;	Example ximg equation.
 *         yimg [iy][ix] = iy * 2.0 + 0.5 ;	Example yimg equation.
 *         }
 *
 * Example ouput image is 1/2 the size of original image.
 *
 * imgint = int2d (img, xdim, ydim, ximg, yimg, xidim, yidim) ;
 *
 * -----------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR   10 Mar 2003
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>

double *
int2d (img, xdim, ydim, xcen, ycen, rsunpix, ximg, yimg, xidim, yidim)

double *img ;		/* Original image. */
int xdim, ydim ;	/* Original image dimensions. */
double xcen, ycen ;	/* Original image sun center. */
double rsunpix ;	/* Solar radius (pixels). */
double *ximg ;		/* Array of x-axis coordinates for interpolation. */
double *yimg ;		/* Array of y-axis coordinates for interpolation. */
int xidim, yidim ;	/* New image dimensions. */

{
    extern double wintdr () ;
    double *imgint ;
    double xi, yi ;
    int ix, iy ;

    imgint= (double *) malloc (xidim * yidim * sizeof (double)) ;

    for (iy = 0; iy < yidim; iy++)
	{
	for (ix = 0; ix < xidim; ix++)
	    {
	    xi = *(ximg + ix + iy * xidim) ;
	    yi = *(yimg + ix + iy * xidim) ;
	    *(imgint + ix + iy * xidim) =
		wintdr (img, xdim, ydim, xi, yi, xcen, ycen, rsunpix) ;
	    }
	}
    return (imgint) ;    
}
