/* jd_carr.pro
 *------------------------------------------------------------------------------
 * Convert Julian Date to Carrington Rotation.
 *
 * Nick Arge   NOAA   20 Nov 2002   [jd_carr.pro]
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR
 * 2 Dec 2002: Converted to C-language.
 *------------------------------------------------------------------------------
 */

#include <stdio.h>

double
jd_carr (JD_temp)

double JD_temp ;

{
    extern double carr_jd_fn () ;
    extern double jd_bo_fn () ;
    extern double jd_lo_fn () ;

    double CARR_CM     = 0.0 ;
    double carr_increm = 0.0 ;
    double carr_test   = 0.0 ;
    double crdur       = 27.2753 ;		/* Average C.R. duration. */

    double JD          = 0.0 ;
    double JD_CR0      = -41859.899931 ;
    double JD_test     = 0.0 ;
    double jdoffset    = 2440000.0 ;

    double Bo          = 0.0 ;
    double Lo          = 0.0 ;

    double num_rot     = 0.0 ;
    double pred_carr   = 0.0 ;
    double del_JD      = 0.0 ;
    double dayfr       = 0.0 ;

    int year, month, day ;
    int hour, minute, second ;


    JD = JD_temp - jdoffset ;

    num_rot = (JD - JD_CR0) / crdur ;
/*
 *    printf ("num_rot: %7.2f\n", num_rot) ;
 */

    if (num_rot >= 1000.0) carr_increm = 1000.0 ;
    else
    if (num_rot >=  100.0) carr_increm =  100.0 ;
    else
    if (num_rot >=   10.0) carr_increm =   10.0 ;
    else
    if (num_rot >=    1.0) carr_increm =    1.0 ;
    else
        CARR_CM = 0.0 ;

/*
 *    printf ("carr_increm: %9.4f\n", carr_increm) ;
 */

    carr_test = carr_increm ;
    JD_test = carr_jd_fn (carr_test, (double) 0.0) ;

/*
 *    printf ("carr_test: %9.4f  JD_test: %9.4f\n", carr_test, JD_test) ;
 */

    while (JD_test <= JD)
        {
        num_rot = (JD - JD_test) / crdur ;


/*
 *        printf ("num_rot: %7.2f\n", num_rot) ;
 */


        if (num_rot >= 1000.0) carr_increm = 1000.0 ;
        else
        if (num_rot >=  100.0) carr_increm =  100.0 ;
        else
        if (num_rot >=   10.0) carr_increm =   10.0 ;
        else
        if (num_rot >=    0.0) carr_increm =    1.0 ;
  
/*
 *        printf ("carr_increm: %9.4f\n", carr_increm) ;
 */

        carr_test = carr_test + carr_increm ;
        JD_test   = carr_jd_fn (carr_test, (double) 0.0) ;

/*
 *        printf ("carr_test: %9.4f  JD_test: %9.4f\n", carr_test, JD_test) ;
 */

        }

    Lo        = jd_lo_fn (JD) ;
    Bo        = jd_bo_fn (JD) ;
    pred_carr = carr_jd_fn (carr_test, Lo) ;

    del_JD = labs ((long) (JD - pred_carr)) ;
    if (del_JD > 20.0) 
	{
        if ((JD - pred_carr) < 0.0)
	    carr_test = carr_test - 1.0 ;
	else
            carr_test = carr_test + 1.0 ;
        }

/*
 *    printf ("carr_test: %f  Lo: %f  Bo: %f\n", carr_test, Lo, Bo) ;
 */

    /*** Convert Julian day to calendar date. ***/

    caldat ((long)(JD + 2440000.0),
	    &month, &day, &year, &hour, &minute, &second) ;

    dayfr = JD - (long) JD + (double) 0.5 ;
    if (dayfr >= 1.0) dayfr = dayfr - 1.0 ;

    printf ("JD: %14.7f JD+offset: %14.5f\n", JD, JD + 2440000.0) ;
    printf ("month: %2d day: %2d year: %4d dayfr: %10.3f\n",
	     month, day, year, dayfr ) ;

    printf ("carr_rot: %f  Lo: %f  Bo: %f\n", carr_test, Lo, Bo) ;

    return (carr_test) ;
}
