/* jd_lat.c
 *------------------------------------------------------------------------------
 * PURPOSE	Convert Julian Date to Heliographic Longitude.
 *
 * AUTHOR	Nick Arge   NOAA   20 Nov 2002   [jd_bo_fn.pro]
 *
 * HISTORY	Modified 27 Apr 2001
 *		Modified 20 Nov 2002
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR
 *  3 Dec 2002: Conversion from IDL to C-language. [jd_lat.c]
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>

  /*** Converts from (Julian date - 2440000) to Helographic longitude. ***/

double
jd_lat (jdmod)

double jdmod ;			/* Modified Julian day. */

{
  double pi = 3.14159265358979323846264338327 ;

  double incl  = 7.25 ;
  double JD_DD  = 0.0 ;
  double nn     = 0.0 ;
  double LL     = 0.0 ;
  double gg     = 0.0 ;
  double sun    = 0.0 ;
  double omega  = 0.0 ;
  double Bosin  = 0.0 ;
  double Bocos  = 0.0 ;
  double Bo     = 0.0 ;
  double Lo     = 0.0 ;
  double LoWsin = 0.0 ;
  double LoWcos = 0.0 ;
  double LoW    = 0.0 ;
  double ww     = 0.0 ;

  JD_DD = (double) (jdmod) ;

/*
 *  nn = JD_DD - 2451545.0
 *  2440000 - 2451545 = 11545
 */

  nn = JD_DD - 11545.0 ;
  LL = 280.472 + 0.9856474 * nn ;
  gg = 357.528 + 0.9856003 * nn ;
  sun = LL + 1.915 * sin (gg * pi / 180.0)
      + 0.020 * sin (2.0 * gg * pi / 180.0) ;

/*
 * Explains where the number 43243.25 comes from.
 * omega = 73.0 + 40.0 / 60.0 + 50.25 * (JD_DD + 2440000.0 - $
 * 2396756.75) / 365.25) / 3600.0
 */

  omega = 73.0 + 40.0 / 60.0
        + 50.25 * ((JD_DD + 43243.25) / 365.25) / 3600.0 ;

/*
 * Explains where the number 41780.0 comes from
 * ww = 360.0 / 25.38 * (JD_DD + 2440000.0 - 2398220.0)
 */

  ww = 360.0 / 25.38 * (JD_DD + 41780.0) ;
  ww = (ww / 360.0 - (long) (ww / 360.0)) * 360.0 ;

  Bosin = sin ((sun - omega) * pi / 180.0) * sin (incl * pi / 180.0) ;
  Bocos = sqrt (1.0 - Bosin * Bosin) ;

  Bo = atan2 (Bosin, Bocos) * 180.0 / pi ;

  LoWsin = cos (incl * pi / 180.0) * sin ((omega - sun) * pi / 180.0)
	 / cos (Bo * pi / 180.0) ;
  LoWcos = -cos ((omega - sun) * pi / 180.0) / cos (Bo * pi / 180.0) ;
  LoW = atan2 (LoWsin, LoWcos) * 180.0 / pi ;

  if (LoW < 0.0)  LoW = LoW + 360.0 ;
  Lo = LoW - ww ;

  if (Lo < 0.0)  Lo = Lo + 360.0 ;

  return (Bo) ;
}
