/* jdate.c
 *------------------------------------------------------------------------------
 * Compute Julian Date.
 *------------------------------------------------------------------------------
 * Author:  David Elmore        HAO/NCAR   3 Feb 2000
 * Modified Andrew L. Stanger   HAO/NCAR   5 Dec 2002
 *  5 Dec 2002: Modified to use hour:min:sec, providing fractional day result.
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>

/* -------------------------------------------------------------------------- */

double
jdate (year, month, day, hour, minute, second)

int year ;
int month ;
int day ;
int hour ;
int minute ;
int second ;
{
    extern int day_of_year () ;
    int doy         = 0 ;
    int  i          = 0 ;
    long leap       = 0 ;
    long EPOCH      = 2000 ;	    /* Reference date: 2000 Jan 1 00:00:00 UT */
    double JD_EPOCH = 2451544.5 ;   /* Julian date for 2000 Jan 1 00:00:00 UT */
    double jd ;

    /*** Count the leap years between EPOCH and specified year. ***/

    if (year > EPOCH)
        {
        for (i = EPOCH; i <= year; i++)
	   if (isleap (i)) leap++ ;
        }
    else
	{
	for (i = year; i < EPOCH; i++)
	    if (isleap (i)) leap-- ;
        }

    /*** Compute day-of-year. ***/

    doy = day_of_year (year, month, day) ;
    printf ("doy: %d\n", doy) ;

    /*** Compute Julian date. ***/

    jd = JD_EPOCH + (year - EPOCH) * 365.0 + leap + doy - 1.0 ;

    jd += (double) hour / 24.0 + (double) minute / 14400.0
	+ (double) second / 86400.0 ;		/* Add fraction of day. */

    return (jd) ;
}

/* -------------------------------------------------------------------------- */

/*** Returns Day-Of-Year corresponding to a year, month & day. ***/

int
day_of_year (year, month, day)

int year ;
int month ;
int day ;
{
    extern int isleap () ;
    int doy ;
    int leap ;
    static int mday [2][13] =
                   {
		   0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365,
		   0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366
		   };

    if (isleap (year))
	leap = 1 ;
    else
	leap = 0 ;

    doy = day ;
    doy += mday [leap][month - 1] ;

    return (doy) ;
}
/* -------------------------------------------------------------------------- */

/*** Leap Year: 0=no, 1=yes. ***/

int
isleap (year)                     /* Gregorian leap year calculation */

int year;
{
    if ((year & 3) != 0)
	return (0);             /* not divisible by 4, not leap */
    if ((year % 100) != 0)
	return (1);             /* not divisible by 100 but by 4, a leap */
    if ((year % 400) != 0)
	return (0);             /* not divisible by 400 but by 100, not leap */
    if ((year % 4000) != 0)
	return (1);             /* not divisible by 4000, a leap */

    return (0);                 /* divisible by 4000, not a leap */
}

/* -------------------------------------------------------------------------- */
