/*
 *  Bilinear interpolation, based on pipeLNU remap.
 *
 *  Reference: 
 *     Abramowitz, Milton, and Stegun, Irene, "Handbook of Mathematical
 *     Functions," Dover Publications, New York, 1964.
 *
 *  Usage:
 *    double linintd (double *f, int nx, int ny, double x, double y)
 *
 *  Bugs:
 *    Only works for double data.
 */

double
linintd (double *f, int nx, int ny, double x, double y)
{

   double p0, p1, q0, q1 ;	/* weights                              */
   int ilow, jlow ;		/* selected CCD pixels                  */
   double *fptr ;		/* input array temp                     */
   double u ;                        

   ilow = floor (x) ;
   jlow = floor (y) ;

   if (ilow < 0)       ilow  = 0 ;
   if (ilow + 1 >= nx) ilow -= 1 ;
   if (jlow < 0)       jlow  = 0 ;
   if (jlow + 1 >= ny) jlow -= 1 ;

   p1 = x - ilow ;
   p0 = 1.0 - p1 ;
   q1 = y - jlow ;
   q0 = 1.0 - q1 ;

   /* Bilinear interpolation from four data points. */

   u = 0.0 ;

   fptr = f + jlow * nx + ilow ;
   u += p0 * q0 * *fptr++ ;
   u += p1 * q0 * *fptr ;

   fptr += nx - 1 ;
   u += p0 * q1 * *fptr++ ;
   u += p1 * q1 * *fptr ;

   return u ;
}
