/* se.c
 *------------------------------------------------------------------------------
 * Compute Solar ephemeris parameters.
 *------------------------------------------------------------------------------
 * Andrew L. Stanger   HAO/NCAR    6 Dec 2002
 *------------------------------------------------------------------------------
 */

#include <stdio.h>
#include <math.h>

main (argc, argv)

int argc ;
char *argv [] ;
{
    extern double jdate () ;
    extern int caldat () ;
    extern double helio_lat () ;
    extern double helio_long () ;
    extern double carr_rotation () ;

    int year ;
    int month ;
    int day ;
    int hour ;
    int minute ;
    int second ;

    double jd ;				/* Juldian Date. */
    double jdmod ;			/* Juldian Date - 2440000.0 */
    double jdoffset = 2440000.0 ;	/* Modified Julian Date. */
    double Lo ;				/* Heliographic Longitude */		
    double Bo ;				/* Heliographic Latitude. */
    double crot ;			/* Carrington Rotation. */

    if (argc < 7)
	{
	fprintf (stderr, "Usage: %s year month day hour minute second\n",
		 argv [0]) ;
        exit (0) ;
	}

    sscanf (argv [1], "%d", &year) ;
    sscanf (argv [2], "%d", &month) ;
    sscanf (argv [3], "%d", &day) ;
    sscanf (argv [4], "%d", &hour) ;
    sscanf (argv [5], "%d", &minute) ;
    sscanf (argv [6], "%d", &second) ;

    jd = jdate (year, month, day, hour, minute, second) ;

    jdmod = jd - jdoffset ;

    Lo = helio_long (jd) ;

    Bo = helio_lat (jd) ;

    crot = carr_rotation (jd) ;

    printf ("date: %4d-%02d-%02d  %02d:%02d:%02d\n",
	     year, month, day, hour, minute, second) ;
    printf ("jd   : %12.4f\n", jd) ;
    printf ("jdmod: %12.4f\n", jdmod) ;
    printf ("Lo:    %12.4f\n", Lo) ;
    printf ("Bo:    %12.4f\n", Bo) ;
    printf ("crot:  %12.4f\n", crot) ;

    exit (0) ;
}
