/* fts2jpg.c
 * -----------------------------------------------------------------------------
 * Convert FITS image(s) into JPEG format.
 * -----------------------------------------------------------------------------
 * Requires cfitsio and ImageMagick libraries.
 * -----------------------------------------------------------------------------
 * Tony Darnell   HAO/NCAR   5 Mar 2002
 * 31 May 2002: [ALS] cosmetic changes.
 * -----------------------------------------------------------------------------
 */

#include <stdio.h>
#include "fitsio.h"
#include "math.h"
#include "string.h"
#include "magick/magick.h"
#include "magick/api.h"

int scaleimg (int *image, int xsz, int ysz, int imin, int imax, int top);
int scalechip (int *image, int xsz, int ysz, int imin, int imax, int top,
               float crradius, float xcen, float ycen);

main (int argc, char *argv[])
{

  FILE     *fptr;
  fitsfile *fileptr;
  int      *image;

  static char filename [80];
  static char outname [80];
  static char tmparr [80];
  static char origin [20], instrument [20] ;
  static char dateobs [20], timeobs [20];
  static char typeobs [20], object [20], typedata[20];
  static char colorpath[21] = "/home/cordyn/color/";
  static char *geometry;

  static int index [255], red [255], green [255], blue [255], ind, r, g, b;

  int status = 0, num_hdus, value, mystat;
  int i, iy, ix, ky, fpixel=1, nelements, xsz, ysz;
  int indx, c, loflag, hiflag, topflag, colorflag, cropflag=0, scalelimb = 1;
  int xpos, ypos, newxsz, newysz;
  int nbyimg, nullval, anynull = -1;
  int scalemin, scalemax, top;

  float crradius, xcen, ycen;

  unsigned char *imgbuf;
  unsigned char *newimgbuf;
  unsigned char *colorbuf;

  /* The following are structures specific to the ImageMagick library */

  Image *jpegimage;
  ImageInfo *image_info;
  DrawInfo *info;
  ExceptionInfo exception;
  PixelPacket textcolor;

  /*** Parse command line ***/

  loflag = hiflag = topflag = colorflag = 1;
  top = 250;
  opterr = 0;

  while ((c = getopt (argc, argv, "l:h:t:c:s:")) != -1)
    switch (c)
      {
      case 'l':
        scalemin = atoi(optarg);
	loflag = 0;
        break;
      case 'h':
	scalemax = atoi(optarg);
	hiflag = 0;
	break;
      case 't':
	top = atoi(optarg);
	topflag = 0;
	break;
      case 'c':
        fptr = fopen (strcat(colorpath,optarg),"r");
	colorflag = 0;
	break;
      case 's':
	geometry = optarg;
	cropflag = 1;
	break;
      default:
	abort();
      }

/*** File Loop. ***/

for (indx = optind; indx < argc; indx++)
  {

/*
 *  printf ("Start of loop\n");
 *  fflush (stdout);
 */

  /*** Open fits file and get image info ***/

  strcpy (filename, argv [indx]);

  fits_open_file (&fileptr, filename, READONLY, &status);

  fits_read_key  (fileptr, TINT,    "NAXIS1",   &xsz,        NULL, &status);
  fits_read_key  (fileptr, TINT,    "NAXIS2",   &ysz,        NULL, &status);
  fits_read_key  (fileptr, TSTRING, "ORIGIN",   &origin,     NULL, &status);
  fits_read_key  (fileptr, TSTRING, "TELESCOP", &instrument, NULL, &status);
  fits_read_key  (fileptr, TSTRING, "DATE-OBS", &dateobs,    NULL, &status);
  fits_read_key  (fileptr, TSTRING, "TIME-OBS", &timeobs,    NULL, &status);
  fits_read_key  (fileptr, TSTRING, "TYPE-OBS", &typeobs,    NULL, &status);
  fits_read_key  (fileptr, TSTRING, "OBJECT",   &object,     NULL, &status);

  if (strstr (instrument, "CHIP")){
    fits_read_key  (fileptr, TSTRING, "TYPE-DP", &typedata,     NULL, &status);
    fits_read_key  (fileptr, TFLOAT, "CRRADIUS", &crradius,     NULL, &status);
    fits_read_key  (fileptr, TFLOAT, "CRPIX1",  &xcen,   NULL, &status);
    fits_read_key  (fileptr, TFLOAT, "CRPIX2",  &ycen,   NULL, &status);
  }

/*
 *  printf ("instrument: %s\n", instrument) ;
 *  fflush (stdout) ;
 */

  nelements = xsz*ysz;
  image = (int *) malloc (nelements * sizeof (int));

  /* fits_read_img(fileptr,TINT,fpixel,nelements,&nullval,image,
                &anynull,&status); */

  if (fits_read_img (fileptr, TINT, fpixel, nelements, &nullval, image, 
		&anynull, &status)) fits_report_error (stderr, status);

  fits_close_file (fileptr, &status);

  /* fits_report_error (stderr, status); */

  /*** Assign color table according to instrument. ***/

  if (strstr (instrument, "CHIP"))
    {
    if (colorflag)
      {
      fptr = fopen ("/home/cordyn/color/RED_TEMPERATURE.lut","r");
      }
    if (strstr (typedata, "VEL"))
      {
      textcolor.red   = 255;
      textcolor.green = 255;
      textcolor.blue  = 255;
      scalelimb       = 0;
      }
    else
      {
      textcolor.red   = 0;
      textcolor.green = 0;
      textcolor.blue  = 0;
      }
    if (loflag){ scalemin = -1200;}
    if (hiflag){ scalemax = 50;}
    }
  else if (strstr (instrument, "MK4")) 
    {
    if (colorflag)
      {
      fptr = fopen ("/home/cordyn/color/qual.lut","r");
      }
    textcolor.red   =255;
    textcolor.green =255;
    textcolor.blue  =255;
    if (loflag){ scalemin = -350; }
    if (hiflag){ scalemax = 1200; }
    }
  else if (strstr (instrument, "MK3"))
    {
    if (colorflag)
      {
      fptr = fopen ("/home/cordyn/color/qual.lut","r");
      }
    textcolor.red   =255;
    textcolor.green =255;
    textcolor.blue  =255;
    if (loflag){ scalemin = -1100; }
    if (hiflag){ scalemax = 4000; }
    }
  else if (strstr (instrument, "DPM"))
    {
    if (colorflag)
      {
      fptr = fopen ("/home/cordyn/color/RED_TEMPERATURE.lut","r");
      }
    textcolor.red   =255;
    textcolor.green =255;
    textcolor.blue  =255;

    if (loflag)
      {
      if (strstr (object, "SUN"))        { scalemin = 0; } 
      else if (strstr (object, "LIMB"))  { scalemin = 0; }
      }
    if (hiflag)
      {
      if (strstr(object,"SUN")){ scalemax = 600; } 
      else if (strstr(object,"LIMB")){ scalemax = 200; }
      }
    }

  /*** Read color table. ***/

  i=0;
  while (fscanf (fptr, "%3d %3d %3d %3d", &ind, &r, &g, &b) != EOF)
    {
    red [i] = r;   green [i] = g;   blue [i] = b;   index [i] = ind;
    i++;
    }
  fclose (fptr);

  /*** Perform image scaling and create image buffers ***/

  if (strstr (instrument, "CHIP") && scalelimb) 
    scalechip(image, xsz, ysz, scalemin, scalemax, top,
	      crradius, xcen, ycen);
  else 
    scaleimg (image, xsz, ysz, scalemin, scalemax, top);

  nbyimg = xsz * ysz * sizeof (unsigned char);

  imgbuf    = (unsigned char *) malloc (nbyimg);
  newimgbuf = (unsigned char *) malloc (nbyimg);
  colorbuf  = (unsigned char *) malloc (nbyimg * 3);

  /*** Convert int array to byte array. ***/

  for (i=0; i<nelements; i++)
     imgbuf [i] = (unsigned char) image [i];

  /*** Invert image top to bottom. ***/

  for (iy = 0; iy < ysz; iy++)
    {
    ky = ysz - iy - 1 ;
    for (ix = 0; ix < xsz; ix++)
	*(newimgbuf + ix + (iy*xsz)) = *(imgbuf + ix + (ky*xsz));
    }

  /*** RGB interlacing. ***/

  for (i=0;i < (nelements); i++)
    {
    colorbuf [i*3]   = (unsigned char) red   [newimgbuf [i]];
    colorbuf [i*3+1] = (unsigned char) green [newimgbuf [i]];
    colorbuf [i*3+2] = (unsigned char) blue  [newimgbuf [i]];
    }



  /*** Create ImageMagick image, annotate, apply filters and resize ***/

  GetExceptionInfo (&exception);
  image_info = CloneImageInfo  ((ImageInfo *) NULL);
  jpegimage  = ConstituteImage (xsz,ysz,"RGB",CharPixel,colorbuf,&exception);
  info       = CloneDrawInfo   (image_info, (DrawInfo *) NULL);

  if (!(cropflag))
    {
    if (xsz == ysz) 
      {
      newxsz = 512; newysz = 512;
      jpegimage = ScaleImage (jpegimage, 512, 512, &exception);
      xpos = 256; ypos = 256;
      }
    else
      {
      newxsz = xsz/2; newysz = ysz/2;
      jpegimage = MinifyImage (jpegimage, &exception);
      xpos = newxsz/2; ypos = newysz/2;
      }
    }
  else
    {
    printf ("Cropping image...\n"); 
    newxsz = 512; newysz = 512;
    xpos = 256; ypos = 256;
    TransformImage (&jpegimage,geometry,"512x512");
    }

/*
  NormalizeImage (jpegimage);
  jpegimage = EnhanceImage (jpegimage, &exception);
  ContrastImage (jpegimage, 1);
  info.stroke = textcolor;
  GetDrawInfo (image_info, &info);
  info.fill = textcolor;
  info.box = textcolor;
  info.text_antialias = 1;
  info.pointsize = 18;
  info.border_color = textcolor;
  info.box = textcolor;
  info.primitive = tmparr;
  info.primitive = tmparr;
  info.primitive = tmparr;
  status = DrawImage (jpegimage, info);
  mystat = AnnotateImage (jpegimage, info);
  printf ("%d\n", mystat);
  info->pointsize = 18;
 */

  info->fill.opacity = 255;
  info->fill = textcolor;
  info->font = "-*-screen-medium-r-*-*-16-*-*-*-*-*-iso8859-1";

  sprintf (tmparr,"text 5,20 '%s %s' text 5,40 '%s' text 5,60 '%s'",
		origin, instrument, typeobs, dateobs);
  info->primitive = tmparr;
  mystat = DrawImage (jpegimage, info); 

  sprintf (tmparr, "text 5, 80 '%s UT' text %d, 10 'N' text %d, %d 'W'", 
	timeobs, xpos, newxsz-10, ypos);
  info->primitive = tmparr;
  mystat = DrawImage (jpegimage, info);

  sprintf (tmparr, "text %d,%d 'S' text 5,%d 'E'",
	xpos, newysz-10, ypos);
  info->primitive = tmparr;
  mystat = DrawImage (jpegimage, info);

  sprintf (tmparr, "text 5,%d 'Scaling: %d to %d'",
	newysz-10, scalemin, scalemax);
  info->primitive = tmparr;
  mystat = DrawImage (jpegimage, info);



  /*** Save image as desired filetype ***/

  strcpy (outname, filename);
  strcpy (strstr (outname, "fts"), "jpg\0");
  printf ("%s\n", outname);
  (void) strcpy (jpegimage->filename, outname);

  WriteImage (image_info, jpegimage);

/*
 *  printf ("End of loop\n");
 *  fflush (stdout);
 */

  free (image);
  }  /* File Loop */
}  /* Main */



/* -------------------------------------------------------------------------- */

/*
 * scaleimg applies a linear scaling to the pixel data
 * from supplied data min, max values as well as a top.
 * Works just like IDL's bytscl routine.
 */

int scaleimg (int *image, int xsz, int ysz, int imin, int imax, int top)

{
  int nelements, omin=0, omax=top, offset=imin;
  int i; 
  float bscale,newpixel;

  /*
  printf ("scaleimg> xsz:%4d  ysz:%4d  imin:%5d  imax:%5d  top:%5d\n",
	  xsz, ysz, imin, imax, top) ;
  */
  nelements = xsz*ysz;

  bscale = (float) (omax - omin) / (float) (imax - imin);

  for (i=0;i < nelements;i++) 
      {
      if (*(image+i) <= -30000)
	  {
          *(image+i) = 1;
          }
      else if (*(image+i) == -3000)
	  {
          *(image+i) = 1;
          }
      else if (*(image+i) == 2000)
	  {
          *(image + (i - 1)) = top;
          *(image +  i)      = top;
          *(image + (i + 1)) = top;
          }
      else
	  {
          newpixel = (float) (*(image + i) - offset) * bscale;
          if (newpixel > omax){ newpixel = omax;}
          if (newpixel < omin){ newpixel = omin;}
          *(image + i)=(int) newpixel;
          }
      }

  return (0);
}


/*
 * scalechip applies different scaling to the limb than it does to the
 * disk because we see the disk in absorption and the limb in emmission.
 * The scaling values are currently hard-coded in and may need to be
 * added to the command line at some point in the future.
 *
 */
int scalechip (int *image, int xsz, int ysz, int imin, int imax, int top, 
	       float crradius, float xcen, float ycen)
{

  int nelements, omin=0, omax=top, offset=imin, 
      limb_imin = -250, limb_imax = 150, limb_offset = limb_imin;
  int i, ix, iy; 
  float xdist, ydist, radius, bscale, limb_bscale, newpixel;

  nelements = xsz*ysz;

  bscale =      (float) (imax - imin) / (float) (omax-omin);
  limb_bscale = (float) (limb_imax - limb_imin) / (float) (omax-omin);
  for (iy = 0; iy < ysz; iy++)
    for (ix = 0; ix < xsz; ix++)
      {
      xdist = (float) ix - xcen ;
      ydist = (float) iy - ycen ;
      radius = sqrt ((xdist * xdist) + (ydist * ydist));
      if ((radius/crradius) < 1.0)
	{
	if (*(image + ix + iy * xsz) < imin)
	  *(image + ix + iy * xsz) = imin;
	if (*(image + ix + iy * xsz) > imax)
	  *(image + ix + iy * xsz) = imax;

        newpixel = (float) (*(image + ix + iy * xsz) - offset)/bscale;
        if (newpixel > omax){ newpixel = omax;}
        *(image + ix + iy * xsz)=(int) newpixel;
        }
      else
	{
	if (*(image + ix + iy * xsz) == -1000.)
	  {*(image + ix + iy * xsz) = limb_imax;}
        newpixel = (float) (*(image + ix + iy * xsz) - limb_offset)/limb_bscale;
        if (newpixel > omax){ newpixel = omax;}
        *(image + ix + iy * xsz)=(int) newpixel;
        }

      }
  return (0);
}
