#include <stdio.h>
#include <stdlib.h>
#include "fitsio.h"
#include "string.h"

float *magimg (float *,int ,int ,int ,int ,int );

main(int argc, char *argv[]){

  fitsfile *fileptr, *ofileptr;
  float    *image, *outimg;
  int      nullval;
  int 	   indx, xsz, ysz, nelements, status, c, anynull = -1;
  int      nbypx = sizeof(float);
  int     fpixel = 1;
  long     naxes[2];
  long     naxis = 2;

  static char filename [80];
  static char outname [80];
  static char temp [80];
  static char type [4];
  float version;
  float mag = 1.0;

  /*** Parse command line arguments ***/

  if (argc < 2){
    printf("Usage:  rsize [-m magnification -h] fitsfile outimage\n");
    exit(1);
  }

  while ((c = getopt (argc, argv, "m:h")) != -1)
    switch (c) {
      case 'm':
        mag = atof(optarg);
        break;
      case 'h':
        printf("Usage:  rsize [-m magnification -h] fitsfile outimage\n");
	exit(1);
        break;
      default:
        mag = 1.0;
    }

  /*
  fits_get_version(&version);
  printf ("Using CFITSIO Version:  %f\n",version);
  */
  printf ("Mag factor:  %f\n", mag);
  /*** Read in fits file ***/

  indx = optind;
  
  strcpy (filename, argv [indx]);
  strcpy (outname, argv [indx+1]);
  fits_open_file (&fileptr, filename, READONLY, &status);
  fits_read_key  (fileptr, TINT,"NAXIS1",&xsz,NULL,&status);
  fits_read_key  (fileptr, TINT,"NAXIS2", &ysz, NULL, &status);

  nelements = xsz*ysz;
  image = (float *) malloc (nelements * sizeof (float));
  outimg = (float *) malloc (nelements * sizeof (float));
  fits_read_img (fileptr, TFLOAT, fpixel, nelements, &nullval, image, 
	  &anynull, &status);
  fits_close_file (fileptr, &status);

  /*** Apply magimg to resize image ***/

  outimg = magimg(image, nbypx, xsz, ysz, xsz * mag, ysz * mag);

  /*** Write newly resized image to new fitsfile ***/

  naxes[0] = xsz * mag;
  naxes[1] = ysz * mag;
  nelements = (naxes[0]) * (naxes[1]);
  printf ("new xsz:  %d\n",naxes[0]);
  printf ("new ysz:  %d\n",naxes[1]);
  /*
  strncpy(temp,filename,strlen(filename)-4);
  temp [strlen (filename) - 4] = '\0' ;
  sprintf(type,"%dx%d",naxes[0], naxes[1]);
  sprintf(outname, "%s.%s.fts",temp,type);
  */
  printf("Writing resized image to %s.\n\n", outname);

  fits_open_file(&ofileptr, outname, READWRITE, &status);
  if(status == 0){
    fits_delete_file(ofileptr, &status);
  } else {
    status = 0;
  }

  fits_open_file (&fileptr, filename, READONLY, &status);

  fits_create_file(&ofileptr, outname, &status);
  fits_create_img(ofileptr, FLOAT_IMG, naxis, naxes, &status);
  fits_write_img(ofileptr, TFLOAT, fpixel, nelements, outimg, &status);
  fits_close_file (ofileptr, &status);
  fits_close_file (fileptr, &status);
  /*
  fits_copy_header(fileptr, ofileptr, &status);
  fits_resize_img(fileptr, TFLOAT, naxis, naxes, &status);
  fits_create_template(&ofileptr, outname, filename, &status);
  fits_movabs_hdu(fileptr,0,NULL,&status);
  fits_movabs_hdu(ofileptr,0,NULL,&status);
  fits_update_key(ofileptr, TLONG, "NAXIS1", naxes, NULL, &status);
  fits_update_key(ofileptr, TLONG, "NAXIS2", naxes+1, NULL, &status);
  */

}
