/* chilo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a High res calibrated video image file (16 bits/pixel) on disk
 * and rewrite as a Low res image (16 bits/pixel).
 * Process a list of images contained in a file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * input: High resolution, 6 Rsun image (ydddhhmmss.v0n).
 *  output: Low resolution, 6 Rsun image (ydddhhmmss.vlo).
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 16 June 1983
 * 24 July 1984: converted from 11/34 (f4p) to VAX-11/750 (f77).
 * 31 October 1984: Accomodates all vidicon image sizes.
 * 4 October 1985: Eliminate output message for each vidicon line processed.
 * 14 November 1988: rewrite in C programming language.
 * 30 October 1989: Use byte #52 of housekeeping to determine byte order.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <stdio.h>

main ()
{
   char namlst [80], namimg [80], namout [80];
   char linbyt [1792];
   char *cline1, *cline2;
   unsigned short line1 [896], line2 [896], ibuf [896];

   register int i, ii, j;
   register unsigned short *ib;
   int icpu;
   int idot, iext;
   int ih, iv, ires, ival;
   int ldec;
   int nplo = 448, nphi = 896;
   int nbylo = 896, nbyhi = 1792;
   int nbypix = 2;			/* # bytes per pixel. */
   int npix;
   int nread, nwrite, nseek, nclose;
   int num;
   int lunlst, lunhi, lunlo;
   static int npixhi [15] = {224,224,448,224,0,448,672,224,0,0,0,448,0,672,896};
   register float bval, bval11, bval12, bval21, bval22;
   FILE *Names;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   cline1 = (char *) line1;
   cline2 = (char *) line2;
   ib = (unsigned short *) ibuf;

   fprintf (stdout, "Enter name of file containing list of images: ");
   fscanf  (stdin, "%s", namlst);

   lunlst = open (namlst, 0);

   /*** Open file containing image names. ***/

   if ((Names = fopen (namlst, "r")) == NULL)
      {
      fprintf(stderr,"cannot open %s\n", namlst);
      exit(-1);
      }

   /*** Read filenames of high resolution images. ***/

   while (fscanf (Names, "%s", namimg) != EOF)
      {
      idot = -1;
      for (i = 0; i < strlen (namimg); i++)
         {
         if (namimg  [i] == '.')
            {
            idot = i;
            break;
            }
         }

      if (idot == -1)
         {
         fprintf (stderr, "Invalid file name; must end with '.cXX'\n");
         exit (-1);
         }

      iext = idot + 1;
      if (namimg[iext] == 'v' || namimg[iext] == 'V')
         {
         fprintf (stderr, "Invalid file name; must be calibrated data.\n");
         exit (-1);
         }

      strcpy (namout, namimg);
      strcat (namout, ".lo");
      fprintf (stdout, "Output file name : %s\n", namout);

      /*** Open input C/P video file on disk. ***/

      lunhi = open (namimg, 0);

      lunlo = creat (namout, 0644);
      nclose = close (lunlo);
      lunlo = open (namout, 1);

      /*
       *** Copy housekeeping record.
       *** Change resolution from high to low.
       */

      nread = read (lunhi, linbyt, nbylo);
      ires = (((int) linbyt [3]) & 0004) / 4 + 1;
      ival = linbyt [3];
      ival = ival | 0004;
      linbyt [3] = (char) ival;

      /*
       * Store computer used to produce calibrated image.
       *
       * bit 0: pdp-11/34
       * bit 1: IPD (Univac 1108)
       * bit 2: Cray
       * bit 3: Vax-11/750 ---> pretend image was calibrated on Vax.
       * bit 4: Sun-4/280
       * bit 5: SGI 4D IRIS
       */

      /*
       * icpu = ((int) linbyt [70] & 0020);
       * linbyt [70] = ((char) (8 & 0377));
       */
      ldec = ((int) linbyt [52] & 0377);
      linbyt [52] = '\0';

      ih = ((int) linbyt [9] & 0360) / 16;
      iv = ((int) linbyt [9] & 0017);
      npix = npixhi [ih-1];
      nphi = npix;
      nplo = npix / 2;
      linbyt [110] = (char) ((nplo / 256) & 0377);
      linbyt [111] = (char) (nplo & 0377);

      /*
       * fprintf (stdout, "ih: %d   iv: %d   ires: %d   npix: %d\n",
       *         ih, iv, ires, npix);
       */

      if (ires != 1)
	 {
         fprintf (stderr, "Image is not HIGH resolution.\n");
         exit (-1);  
	 }

      nwrite = write (lunlo, linbyt, nbylo);

      /*** Copy bit-map record to output volume. ***/

      nseek = lseek (lunhi, (long) nbyhi, 0);
      nread = read (lunhi, linbyt, nbylo);
      nwrite = write (lunlo, linbyt, nbylo);

      /*** Now read each record (two at a time). ***/

      nseek = lseek (lunhi, (long) (nbyhi*2), 0);

      for (j = 0; j < nphi; j = j + 2)

         {
         nread = read (lunhi, cline1, nbyhi);	/* Even numbered lines. */
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j);
            exit (-1);
	    }
         if (ldec == 1)  srevbyt (cline1, nbypix, nphi);   /* Swap bytes. */

         nread = read (lunhi, cline2, nbyhi);	/* Odd numbered lines. */
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j+1);
            exit (-1);
	    }
         if (ldec == 1)  srevbyt (cline2, nbypix, nphi);   /* Swap bytes. */

        /*** Convert from high to low resolution by averaging 2x2 pixels. ***/

         for (i = 0; i < nphi; i = i + 2)
	    {
            ii = i / 2;
            bval11 = (float) (line1 [i]   & 0177777);
            bval12 = (float) (line1 [i+1] & 0177777);
            bval21 = (float) (line2 [i]   & 0177777);
            bval22 = (float) (line2 [i+1] & 0177777);
            bval = (bval11 + bval12 + bval21 + bval22) / 4.;
            num = (int) bval;
            if (num > 32767) num = 32767 ;
            *(ib+ii) = (unsigned short) (num & 0177777);
            /*
             * if (i == j)
	     * {
             * printf ("i=j:%3d  num:%3d  bval11:%5.0f  bval12:%5.0f  ",
             *          i, num, bval11, bval12);
             * printf ("bval21:%5.0f  bval22:%5.0f\n", bval21, bval22);
	     * }
             */
	    }


         /*** Write low resolution record to output file. ***/

         /* srevbyt (ibuf, nbypix, nplo); */

         nwrite = write (lunlo, ibuf, nbylo);

	 }
         nclose = close (lunlo);
         nclose = close (lunhi);
      }
   fprintf (stdout, "chilo finished.\n");

   exit (0);
}
