/* cphilo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a High res calibrated video image file (16 bits/pixel) on disk
 * and rewrite as a Low res image (16 bits/pixel).
 * Process a list of images contained in a file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * input: High resolution, 6 Rsun image (ydddhhmmss.v0n).
 *  output: Low resolution, 6 Rsun image (ydddhhmmss.vlo).
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 16 June 1983
 * 24 July 1984: converted from 11/34 (f4p) to VAX-11/750 (f77).
 * 31 October 1984: Accomodates all vidicon image sizes.
 * 4 October 1985: Eliminate output message for each vidicon line processed.
 * 14 November 1988: rewrite in C programming language.
 * 30 October 1989: Use byte #52 of housekeeping to determine byte order.
 * 18 May 1992: rewrite original file with low resolution image.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <stdio.h>
#include "/home/stanger/lib/include/cplib.h"
#define MAXHI 896
#define MAXLO 448

main (argc, argv)

int argc;
char *argv [];
{
   extern char *malloc ();
   static char hkbuf  [MAXHI];
   static char bitmap [MAXHI];
   static char namimg [80];
   static char namlst [80] = "none";
   char *cline1, *cline2;
   unsigned short line1 [MAXHI], line2 [MAXHI], ibuf [MAXHI];
   unsigned short *sbuf;

   register int i, ii, j;
   register unsigned short *ib;
   int icpu;
   int ival;
   int ix, iy;
   int ldec;
   int List = 0;
   int nplo = MAXLO, nphi = MAXHI;
   int nbylo = MAXLO * sizeof (short), nbyhi = MAXHI * sizeof (short);
   int nbyimg;
   int nread, nwrite, nseek, nclose;
   int num;
   int lun;
   register float bval, bval11, bval12, bval21, bval22;
   FILE *Names;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   cline1 = (char *) line1;
   cline2 = (char *) line2;
   ib = (unsigned short *) ibuf;

   if (argc == 2)
      strcpy (namimg, argv [1]);
   else
      {
      fprintf (stderr, "usage: %s {cp_hires_filename | @filename_list}\n",
               argv [0]);
      exit (0);
      }

   if (namimg [0] == '@')
      {
      List = 1;
      strcpy (namlst, &namimg [1]);

      /*** Open file containing image names. ***/

      if ((Names = fopen (namlst, "r")) == NULL)
         {
         fprintf (stderr,"cannot open %s\n", namlst);
         exit (-1);
         }
      }
   /*
    *   fprintf (stdout, "Enter name of file containing list of images: ");
    *   fscanf  (stdin, "%s", namlst);
    */

   /*** Read filenames of high resolution images. ***/

   if (List == 1)
   while (fscanf (Names, "%s", namimg) != EOF)
      {
      cphskp (namimg);
      if (idtype != 3)
         {
         fprintf (stderr, "idtype: %d  Invalid image type.\n", idtype);
         exit (-1);
         }

      nphi = npix;
      nplo = npix / 2;
      nbyhi = nphi * sizeof (short);
      nbylo = nplo * sizeof (short);
      nbyimg = nplo * nplo * sizeof (short);
      sbuf = (unsigned short *) malloc (nbyimg);

      /*** Open input C/P video file on disk. ***/

      lun = open (namimg, 2);

      /*
       *** Copy housekeeping record.
       *** Change resolution from high to low.
       */

      nread = read (lun, hkbuf, nbylo);
      ival = hkbuf [3];
      ival = ival | 0004;
      hkbuf [3] = (char) ival;

      ldec = ((int) hkbuf [52] & 0377);
      hkbuf [52] = '\0';

      hkbuf [110] = (char) ((nplo / 256) & 0377);
      hkbuf [111] = (char) (nplo & 0377);

      if (H.ires != 0)
	 {
         fprintf (stderr, "Image is not HIGH resolution.\n");
         exit (-1);  
	 }


      /*** Copy bit-map record to output volume. ***/

      nseek = lseek (lun, (long) nbyhi, 0);
      nread = read (lun, bitmap, nbylo);

      /*** Now read each record (two at a time). ***/

      nseek = lseek (lun, (long) (nbyhi*2), 0);

      iy = -1;
      for (j = 0; j < nphi; j = j + 2)
         {
         iy++;
         nread = read (lun, cline1, nbyhi);	/* Even numbered lines. */
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j);
            exit (-1);
	    }
         if (ldec == 1)  srevbyt (cline1, nbypix, nphi);   /* Swap bytes. */

         nread = read (lun, cline2, nbyhi);	/* Odd numbered lines. */
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j+1);
            exit (-1);
	    }
         if (ldec == 1)  srevbyt (cline2, nbypix, nphi);   /* Swap bytes. */

        /*** Convert from high to low resolution by averaging 2x2 pixels. ***/

         for (i = 0; i < nphi; i = i + 2)
	    {
            ii = i / 2;
            bval11 = (float) (line1[i]   & 0177777);
            bval12 = (float) (line1[i+1] & 0177777);
            bval21 = (float) (line2[i]   & 0177777);
            bval22 = (float) (line2[i+1] & 0177777);
            bval = (bval11 + bval12 + bval21 + bval22) / 4.;
            num = (int) bval;
            if (num > 32767) num = 32767 ;
            *(ib+ii) = (unsigned short) (num & 0177777);
            /*
             * if (i == j)
	     * {
             * printf ("i=j:%3d  num:%3d  bval11:%5.0f  bval12:%5.0f  ",
             *          i, num, bval11, bval12);
             * printf ("bval21:%5.0f  bval22:%5.0f\n", bval21, bval22);
	     * }
             */
	    }


         /*** Copy low resolution record to output buffer. ***/

         /* srevbyt (ibuf, nbypix, nplo); */


         for (ix = 0; ix < nplo; ix++)
            *(sbuf + ix + iy * nplo) = *(ib + ix);

	 }

      /*** Write low resolution image to original disk file. ***/

      nclose = close (lun);
      unlink (namimg);
      lun = creat (namimg, 0644);
      nseek = lseek (lun, 0L, 0);
      nwrite = write (lun, hkbuf, nbylo);
      nwrite = write (lun, bitmap, nbylo);
      nwrite = write (lun, sbuf, nbyimg);
      nclose = close (lun);
      free (sbuf);
      fprintf (stderr, "%s rewritten as low resolution file.\n", namimg);
      }
   fclose (Names);

   fprintf (stdout, "cphilo finished.\n");

   exit (0);
}
