/* rawhilo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a High res raw video image file (8 bits/pixel) on disk
 * and rewrite as a Low res image(8 bits/pixel).
 * Process a list of images contained in a file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * input: High resolution, 6 Rsun image (ydddhhmmss.v0n).
 *  output: Low resolution, 6 Rsun image (ydddhhmmss.vlo).
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 16 June 1983
 * 24 July 1984: converted from 11/34 (f4p) to VAX-11/750 (f77).
 * 31 October 1984: Accomodates all vidicon image sizes.
 * 4 October 1985: Eliminate output message for each vidicon line processed.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <stdio.h>
#include "/home/riker/stanger/lib/include/cplib.h"
#define MAXHI 896
#define MAXLO 448

main (argc, argv)

int argc;
char *argv [];
{
   char namlst [80], namimg [80];
   char hkbuf [896];
   char bitmap [896];
   char line1 [896], line2 [896], ibuf [448];
   unsigned char *cbuf;

   int i, ii, idot, iext;
   int ih, iv, imgres, ival;
   int ix, iy;
   int j;
   int nbyimg;
   int nplo, nphi;
   int nbylo, nbyhi;
   int npxlin;
   int nread, nwrite, nseek, nclose;
   int num;
   int lun;
   int seq;
   static int npixhi [15] = {224,224,448,224,0,448,672,224,0,0,0,448,0,672,896};
   float bval, bval11, bval12, bval21, bval22;
   FILE *LIST;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   if (argc == 2)
      {
      strcpy  (namimg, argv [1]);
      }
   else
      {
      fprintf (stderr, "usage: %s cp_raw_filename | @cp_filelist\n", argv [0]);
      exit (0);
      }

/*
 *   fprintf (stdout, "Enter name of file containing list of images: ");
 *   fscanf  (stdin, "%s", namlst);
 */

   if (namimg [0] == '@')
      {
      seq = 1;
      strcpy (namlst, &namimg [1]);
      if ((LIST = fopen (namlst, "r"))==NULL)
         {
         fprintf(stderr,"cannot open %s\n", namlst);
         exit (-1);
         }
      }
   else
      seq = 0;


   /* Read filenames of high resolution images. */

loop:

   if (seq)
      if (fscanf (LIST, "%s", namimg) == EOF)
         {
         fprintf (stderr, "End of file: %s\n", namlst);
         goto quit;
         }

      cphskp (namimg);
      hkline (stdout);
      nphi = npix;
      nplo = npix / 2;
      nbyhi = nphi * sizeof (char);
      nbylo = nplo * sizeof (char);
      nbyimg = nplo * nplo * sizeof (char);
      cbuf = (unsigned char *) malloc (nbyimg);

      idot = -1;
      for (i=0; i<strlen(namimg); i++)
         {
         if (namimg[i] == '.')
            {
            idot = i;
            break;
            }
         }

      if (idot == -1)
         {
         fprintf (stderr, "Invalid file name; must end with '.vXX'\n");
         exit (-1);
         }

      iext = idot + 1;
      if (namimg[iext] != 'v' && namimg[iext] != 'V')
         {
         fprintf (stderr, "Invalid file name; must be raw DN data.\n");
         exit (-1);
         }

      /*** Open input C/P video file on disk. ***/

      lun = open (namimg, 0);

      /*
       *** Copy housekeeping record.
       *** Change resolution from high to low.
       */

      nread = read (lun, hkbuf, nplo);
      imgres = (((int) hkbuf [3]) & 0004) / 4 + 1;
      ival = hkbuf [3];
      ival = ival | 0004;
      hkbuf [3] = (char) ival;

      ih = ((int) hkbuf [9] & 0360) / 16;
      iv = ((int) hkbuf [9] & 0017);
      npxlin = npixhi [ih-1];
      hkbuf [110] = (char) ((nplo / 256) & 0377);
      hkbuf [111] = (char) (nplo & 0377);
/*
 *      fprintf (stdout, "ih: %d   iv: %d   imgres: %d   npxlin: %d\n",
 *	       ih, iv, imgres, npxlin);
 */

      if (imgres != 1)
	 {
         fprintf (stderr, "Image is not HIGH resolution.\n");
         exit (-1);  
	 }

      /*** Copy bit-map record to output volume. ***/

      nseek = lseek (lun, (long) nphi, 0);
      nread = read (lun, bitmap, nplo);

      /*** Now read each record (two at a time). ***/

      nseek = lseek (lun, (long) (nphi*2), 0);

      iy = -1;
      for (j = 0; j < nphi; j = j + 2)

         {
         iy++;
         nread = read (lun, line1, nphi);
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j);
            exit (-1);
	    }
         nread = read (lun, line2, nphi);
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j+1);
            exit (-1);
	    }

        /*** Convert from high to low resolution by averaging 2x2 pixels. ***/

         for (i = 0; i < nphi; i = i + 2)
	    {
            ii = i / 2;
            bval11 = (float) (line1 [i]   & 0377);
            bval12 = (float) (line1 [i+1] & 0377);
            bval21 = (float) (line2 [i]   & 0377);
            bval22 = (float) (line2 [i+1] & 0377);
            bval = (bval11 + bval12 + bval21 + bval22) / 4.;
            num = (int) bval;
            if (num > 255) num = 255 ;
            ibuf [ii] = (char) (num & 0377);
/*
 *	    if (i == j)  printf ("i=j:%3d  num:%3d  bval11:%5.0f  bval12:%5.0f  bval21:%5.0f  bval22:%5.0f\n",
 *				 i, num, bval11, bval12, bval21, bval22);
 */
	    }

         /*** Copy low resolution record to output buffer. ***/

         for (ix = 0; ix < nplo; ix++)
            *(cbuf + ix + iy * nplo) = *(ibuf + ix);

	 }

      /*** Write low resolution image to original disk file. ***/

      nclose = close (lun);
      unlink (namimg);
      lun = creat (namimg, 0644);
      nseek = lseek (lun, 0L, 0);
      nwrite = write (lun, hkbuf, nbylo);
      nwrite = write (lun, bitmap, nbylo);
      nwrite = write (lun, cbuf, nbyimg);
      nclose = close (lun);
      free (cbuf);
      fprintf (stderr, "%s rewritten as low resolution file.\n", namimg);

   if (seq)  goto loop;

quit:

   fprintf (stdout, "rawhilo finished.\n");

   exit (0);
}
