
/* rhilo.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Read a High res raw video image file (8 bits/pixel) on disk
 * and rewrite as a Low res image(8 bits/pixel).
 * Process a list of images contained in a file.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * input: High resolution, 6 Rsun image (ydddhhmmss.v0n).
 *  output: Low resolution, 6 Rsun image (ydddhhmmss.vlo).
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 * Andrew L. Stanger   HAO/NCAR   SMM C/P
 * 16 June 1983
 * 24 July 1984: converted from 11/34 (f4p) to VAX-11/750 (f77).
 * 31 October 1984: Accomodates all vidicon image sizes.
 * 4 October 1985: Eliminate output message for each vidicon line processed.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
#include <stdio.h>

main()
{
   char namlst[80], namimg[80], namout[80];
   char linbyt[896];
   char line1[896], line2[896], ibuf[448];

   int i, ii, idot, iext;
   int ih, iv, ires, ival;
   int j;
   int nphi = 896, nplo = 448;
   int npix;
   int nread, nwrite, nseek, nclose;
   int num;
   int lunlst, lunhi, lunlo;
   static int npixhi[15] = {224,224,448,224,0,448,672,224,0,0,0,448,0,672,896};
   float bval, bval11, bval12, bval21, bval22;
   FILE *Names;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   fprintf (stdout, "Enter name of file containing list of images: ");
   fscanf  (stdin, "%s", namlst);

   lunlst = open (namlst, 0);

   /* Open file containing image names. */

   if((Names = fopen (namlst,"r"))==NULL)
      {
      fprintf(stderr,"cannot open %s\n", namlst);
      exit(-1);
      }

   /* Read filenames of high resolution images. */

   while (fscanf (Names,"%s", namimg) != EOF)
      {
      idot = -1;
      for (i=0; i<strlen(namimg); i++)
         {
         namout[i] = namimg[i];
         if (namimg[i] == '.')
            {
            idot = i;
            break;
            }
         }

      if (idot == -1)
         {
         fprintf (stderr, "Invalid file name; must end with '.vXX'\n");
         exit (-1);
         }

      iext = idot + 1;
      if (namimg[iext] != 'v' && namimg[iext] != 'V')
         {
         fprintf (stderr, "Invalid file name; must be raw DN data.\n");
         exit (-1);
         }

      namout[iext] = '\0';
      strcat (namout, "vlo");
      fprintf (stdout, "Output file name : %s\n", namout);

      /*** Open input C/P video file on disk. ***/

      lunhi = open (namimg, 0);

      lunlo = creat (namout, 0644);
      nclose = close (lunlo);
      lunlo = open (namout, 1);

      /*
       *** Copy housekeeping record.
       *** Change resolution from high to low.
       */

      nread = read (lunhi, linbyt, nplo);
      ires = (((int)linbyt[3]) & 0004) / 4 + 1;
      ival = linbyt[3];
      ival = ival | 0004;
      linbyt[3] = (char) ival;

      ih = ((int) linbyt[9] & 0360) / 16;
      iv = ((int) linbyt[9] & 0017);
      npix = npixhi[ih-1];
      nphi = npix;
      nplo = npix / 2;
      linbyt[110] = (char) ((nplo / 256) & 0377);
      linbyt[111] = (char) (nplo & 0377);
/*
 *      fprintf (stdout, "ih: %d   iv: %d   ires: %d   npix: %d\n",
 *	       ih, iv, ires, npix);
 */

      if (ires != 1)
	 {
         fprintf (stderr, "Image is not HIGH resolution.\n");
         exit (-1);  
	 }

      nwrite = write (lunlo, linbyt, nplo);

      /*** Copy bit-map record to output volume. ***/

      nseek = lseek (lunhi, (long) nphi, 0);
      nread = read (lunhi, linbyt, nplo);
      nwrite = write (lunlo, linbyt, nplo);

      /*** Now read each record (two at a time). ***/

      nseek = lseek (lunhi, (long) (nphi*2), 0);

      for (j = 0; j < nphi; j = j + 2)

         {
         nread = read (lunhi, line1, nphi);
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j);
            exit (-1);
	    }
         nread = read (lunhi, line2, nphi);
	 if (nread < 1)
	    {
	    fprintf (stderr, "%d bytes read from file %s, record #%d\n",
		     nread, namimg, j+1);
            exit (-1);
	    }

        /*** Convert from high to low resolution by averaging 2x2 pixels. ***/

         for (i = 0; i < nphi; i = i + 2)
	    {
            ii = i / 2;
            bval11 = (float) (line1[i]   & 0377);
            bval12 = (float) (line1[i+1] & 0377);
            bval21 = (float) (line2[i]   & 0377);
            bval22 = (float) (line2[i+1] & 0377);
            bval = (bval11 + bval12 + bval21 + bval22) / 4.;
            num = (int) bval;
            if (num > 255) num = 255 ;
            ibuf[ii] = (char) (num & 0377);
/*
 *	    if (i == j)  printf ("i=j:%3d  num:%3d  bval11:%5.0f  bval12:%5.0f  bval21:%5.0f  bval22:%5.0f\n",
 *				 i, num, bval11, bval12, bval21, bval22);
 */
	    }

         /*** Write low resolution record to output filet file. ***/

         nwrite = write (lunlo, ibuf, nplo);

	 }
         nclose = close (lunlo);
         nclose = close (lunhi);
      }
   fprintf (stdout, "rhilo finished.\n");

   exit (0);
}
