/* rot.c
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Rotate an image.
 * Use bilinear interpolation.
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Andrew Stanger   HAO/NCAR   28 June 2000
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 */
#include <stdio.h>
#include <math.h>

unsigned char *
rot (imginp, nbypx, dx, dy, cx, cy, angdeg)

unsigned char *imginp ;	/* Input image array pointer. */
int nbypx ;		/* # bytes per pixel. */
int dx, dy ;		/* Dimensions of image. */
int cx, cy ;		/* Center of rotation. */
float angdeg ;		/* Rotation angle in degrees (+ = CCW). */

{
   extern void  *malloc () ;
   extern float blint () ;
   register int nx, ny, ox, oy, xdim, ydim ;

   float fox, foy, cose, sine, fnx, fny, fnysin, fnycos ;
   float fcx, fcy ;
   float fval, *frot ;

   double angle ;

   unsigned char  *imgrot ;
   unsigned char cval, *crot ;

   unsigned short sval, *srot ;

   fprintf (stdout, "rot> nbypx:%3d  dx/dy:%3d %3d  cx/cy:%3d %3d\n",
	    nbypx, dx, dy, cx, cy) ;

   xdim = dx ;   ydim = dy ;
   fcx = (float) cx ;   fcy = (float) cy ;
   angle = (double) angdeg * (3.14159265/180.) ;/* convert deg to rads. */
   sine = (float) sin ( angle ) ;
   cose = (float) cos ( angle ) ;

   fprintf (stdout, "angdeg: %6.2f   angle: %f   sine: %6.2f   cose: %6.2f\n",
            angdeg, angle, sine, cose) ;

   imgrot = (unsigned char *) malloc ( dx * dy * nbypx ) ;

   if (nbypx == 1)				/* unsigned char pixels. */
      {
      crot = (unsigned char *) imgrot ;

      for ( ny = 0 ; ny < ydim ; ny++ )
         {
         fny = (float) (ny) - fcy ;
         fnysin = fny * sine - fcx ;
         fnycos = fny * cose + fcy ;

         for ( nx = 0 ; nx < xdim ; nx++)
            {
            fnx = (float) (nx) - fcx ;
            fox = fnx * cose - fnysin ;
            foy = fnycos + fnx * sine ;
            ox = (int) fox ;
            oy = (int) foy ;
	    cval = (unsigned char) blint (imginp, nbypx, nx, ny, fox, foy) ;
            if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
               *crot++ = 0 ;
            else *crot++ = cval ;
            }
         }
      }

   else
   if (nbypx == 2)				/* unsigned short int pixels. */
      {
      srot = (unsigned short *) imgrot ;

      for ( ny = 0 ; ny < ydim ; ny++ )
         {
         fny = (float) (ny) - fcy ;
         fnysin = fny*sine - fcx ;
         fnycos = fny*cose + fcy ;

         for ( nx = 0 ; nx < xdim ; nx++)
            {
            fnx = (float) (nx) - fcx ;
            fox = fnx * cose - fnysin ;
            foy = fnycos + fnx * sine ;
            ox = (int) fox ;
            oy = (int) foy ;
	    sval = (unsigned short) blint (imginp, nbypx, nx, ny, fox, foy) ;
            if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
               *srot++ = 0 ;
            else *srot++ = sval ;
            }
         }
      }
   else
   if (nbypx == 4)				/* floating point pixels. */
      {
      frot = (float *) imgrot ;

      for ( ny = 0 ; ny < ydim ; ny++ )
         {
         fny = (float) (ny) - fcy ;
         fnysin = fny*sine - fcx ;
         fnycos = fny*cose + fcy ;

         for ( nx = 0 ; nx < xdim ; nx++)
            {
            fnx = (float) (nx) - fcx ;
            fox = fnx * cose - fnysin ;
            foy = fnycos + fnx * sine ;
            ox = (int) fox ;
            oy = (int) foy ;
	    fval = (float) blint (imginp, nbypx, nx, ny, fox, foy) ;
            if ( (ox < 0) || (ox >= xdim) || (oy < 0) || (oy >= ydim) )
               *frot++ = 0 ;
            else *frot++ = fval ;
            }
         }
      }

   return ( imgrot ) ;
}
