/************************************************************************
*                         W H I P . H                                   *
*                                                                       *
*           Copyright (C) 1984-90, GW Hannaway & Associates             *
*                                                                       *
*  These coded instructions, statements, and computer programs  contain *
*  unpublished  proprietary  information of  GW Hannaway & Assoc.,  and *
*  are protected by Federal copyright law.  They  may  not be disclosed *
*  to  third  parties  or copied or duplicated in any form, in whole or *
*  in part, without the prior written consent of GW Hannaway & Assoc.   *
*                                                                       *
*       Author: William R. Bishop      09/84                            *
*                                                                       *
************************************************************************/
/*
 * Tip - when using vi, set ts=4
*/

# include <fbconfig.h>

/*
 * Entries consist of 2 byte integer short words.
 * A variable length comments section may follow this header and
 * before the image data (lcom specifies length).
*/
struct pic_hdr 
{
        short   id; 	/* Image Identifier; BYTE, SHORT, etc	*/
        short   lcom; 	/* Length of comments before image data */
        short   type;	/* Type of pixel encoding; ROW_MODE, etc*/
        short   dims; 	/* Dimensions (default = 2) 			*/
        short   hpls;	/* hyperplanes (default = 1) 			*/
        short   plns;	/* Planes (default = 1) 				*/
        short   rows; 	/* Number of rows 						*/
        short   cols; 	/* Number of columns 					*/
        short   bp;   	/* Bits/pixel - must be multiple of 8	*/
        short   sbp;  	/* Significant bits/pixel				*/
        short   xstrt;	/* X starting address of image			*/
        short   ystrt;	/* Y starting address of image			*/
};
typedef struct pic_hdr pichdr;

/*
* Immediately following the header structure in
* oldest-to-newest-order, are ASCII title and comments 
* consisting of lines of less than 80 characters
* each ended by a newline (012).
* New comments would be added after the last comment 
* and the comment length field of the header would be incremented.
*/
/*
* The image data which follows is deemed to be in row mode if 
* the image data was acquired by columns, and proceeds right across
* the image.  Image data taken in column mode is assumed to be interlaced.
* Row mode is the common mode which rasters across the top row from left
* to right and then downward by rows.
* The type of acquisition is store in pic_hdr.type.
*/

/*
 * Definitions for different image formats for pic_hdr.type
*/
#define NO_MODE         -1
#define ROW_MODE        0
#define COL_MODE        1
#define RUNLEN          2
#define CONOGRAPH       3
#define VECTOR          4
#define SCANLINE		5

#define Z_MODE			6	/* Special value for 3D framebuffers */

/* 
 * Defines for pic_hdr.id (and framebuffer kind)
*/
#define	BIT			000011	/* Bit images treated as byte	*/
#define SIGNED_BYTE	000012	/* For those needing signed math*/
#define BYTE    	044520	/* 4950 hex, or "PI" in ASCII	*/
#define	SHORT		000001	/* 16 Bit signed data			*/
#define	LONG		000010	/* 32 bit signed data			*/
#define	FLOAT		000007	/* 32 bit floating point data	*/
#define	CMPLX		046503	/* 4D43 hex, Type "CM" (complex)*/
#define	RGB			000002	/* RGB data, 8 bits/component	*/
#define	RGBA		000003	/* RGBA data, 8 bits/component	*/
#define	ABGR		000004	/* ABGR data, 8 bits/component	*/
#define	RGBZ		000005	/* RGBZ data, 24 bits + 32bits	*/
#define	RGBAZ		000006	/* RGBAZ data, 32 bits + 32bits	*/

/* 
 * Local RAM structures, created and maintained by image(3) subroutines.
*/
struct str
{
	struct	str	*nx;
	char		*p;
};

struct image
{
	struct image *nx;			/* Struct maint, must be first */
	struct image *pr;

	struct	pic_hdr	hdr;		/* Image header			*/
	struct	str		*com[2];	/* Image Comments		*/
	short			type;		/* Type of image data	*/
	union
	{
		unsigned char	*cp;	/* Char (8 bit) data	*/
		short			*sp;	/* Short (16 bit) data	*/
		long			*lp;	/* Long (32 bit) data	*/
		float			*fp;	/* Float (32 bit) data	*/
	} d;
};

/*
 * Current Types of framebuffers. Note that some may or may not exist
 * in your system depending upon the configuration purchased.
*/
#define    FB_MONO	 0	/* Monochrome frame buffer		*/
#define    FB_RGB	 1	/* RGB frame buffer				*/
#define    FB_IRIS	 2	/* Iris frame buffer			*/
#define    FB_IRISGL 3	/* Iris in single buffer mode	*/
#define    FB_150	 4	/* 150 series frame buffer		*/
#define    FB_FILE	 5	/* Frame buffer is a data file 	*/
#define    FB_SHMEM	 6	/* Frame buffer is shared mem fb*/
#define    FB_RGBA	 7	/* Frame buffer is RGBA fb		*/
#define    FB_COMP	 8	/* Multi-component Framebuffer	*/
#define    FB_CMPLX	 9	/* Two component floating pt. fb*/
#define    FB_MEM	 10	/* Local memory RAM framebuffer	*/
#define    FB_PXM	 11	/* Pixel machine framebuffer	*/
#define    FB_3D     12 /* 3D Shared memory framebuffer	*/
#define    FB_SCSI   13 /* Raw SCSI framebuffer			*/

/*
 * Some global variables.  I should probably hide them and make
 * access to them via subroutine calls (sigh).
*/
extern int NUMALU;		/* Number of ALU's in system 			*/
extern int NUMFBS;		/* Number of frame buffers				*/
extern int NUMAPS;		/* Number of AP's						*/
extern int NUMRTC;		/* Number of RTC's						*/
extern int NUMHF;		/* Number of HF's						*/
extern int fb_xmax;		/* Max x of current fb					*/
extern int fb_ymax;		/* Max y of current fb					*/
extern int fb_zmax;		/* Max z of current fb					*/
extern int stepmode;	/* Rows/cols, etc						*/
extern char errbuf[];	/* Tmp buffer for error msg generation	*/

/*
 * For those crummy compilers that can't have more than 8 character
 * function names (Toy Computers -- GRRR).
*/
# ifdef IBM_AT
# define it_putap	ITPUTAP
# define it_putalu	ITPUTALU
# define it_getap	ITGETAP
# define it_getalu	ITGETALU
# define it_setap	ITSETAP
# define it_setalu	ITSETALU
# endif

# ifdef IT_150
# include <it150.h>
# endif

# ifdef PIXEL_MACHINE
# include <pxm.h>
# endif
