/* xyplot.c
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Calls NCAR graphics routines to generate a 2-dimensional plot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   Andrew L. Stanger
 *   NCAR/HAO   SMM C/P
 *   9 February 1987
 *   12 May 1989: remove cpgraf.h include file; make ig an argument of xyplot.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   x, y : real arrays to be plotted.
 *   nval : # values of x = # values of y.
 *   namemc : metacode file name.
 *   title : graph title.
 *   labx, laby : labels for x & y axes.
 *   ixscale, iyscale : integers specifying scale type (0=linear, 1=log).
 *   ibkg : integer specifying graph background 
 *          = 0  do not change default.
 *          = 1  tick marks only.
 *          = 2  full grid.
 *          = 3  half axis.
 *          = 4  no background.
 *   ig = 0 for non-graphics terminal, =1 for graphics terminal.
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 *   ltype	x-axis	y-axis
 *   -----      ------  ------
 *	1	linear	linear
 *	2	linear	log
 *	3	log	linear
 *	4	log	log
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 */
extern int pcount;

xyplot (x, y, nval, namemc, title, labx, laby, ixscale, iyscale, ibkg, ig)

float	x [], y [];
int	nval;
char	namemc [];
char	title [60], labx [40], laby [40];
int	ixscale, iyscale;
int	ibkg;
int	ig;

{
static int igph = 5;
static int ixlab = 1, iylab= 1;
int majrx = 10;
int majry = 10;

int i;
int lentit;
int lenlabx;
int lenlaby;
int ltype;
int minrx, minry;

float dxx, dxn;
float dyy, dyn;
float xmin, xmax, ymin, ymax;
/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

   pcount ++;
   if (pcount == 1)  q8qini (namemc, ig);

   /*** Set scaling type. ***/

   if (ixscale == 0 && iyscale == 0)  ltype = 1;	/* x linear, y linear */
   if (ixscale == 0 && iyscale == 1)  ltype = 2;	/* x linear, y log    */
   if (ixscale == 1 && iyscale == 0)  ltype = 3;	/* x log,    y linear */
   if (ixscale == 1 && iyscale == 1)  ltype = 4;	/* x log,    y log    */

   /*** Determine minimum & maximum values for axes ranges. ***/

   xmin = 1.e+30;
   xmax = -1.e+30;
   ymin = 1.e+30;
   ymax = -1.e+30;

   for (i=0; i<nval; i++)
      {
      if (xmin > x [i])  xmin = x [i];
      if (xmax < x [i])  xmax = x [i];
      if (ymin > y [i])  ymin = y [i];
      if (ymax < y [i])  ymax = y [i];
      }
   printf ("xmin:%f  xmax:%f  ymin:%f  ymax:%f\n", xmin, xmax, ymin, ymax);

   /*** Write title & axes labels. ***/

   lentit = strlen (title);
   lenlabx = strlen (labx);
   lenlaby = strlen (laby);
   set (0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 0.0, 1.0, 1);
   pwrit (0.6,  0.9,  title, lentit,  2,  0, 0);
   pwrit (0.6,  0.08, labx,  lenlabx, 2,  0, 0);
   pwrit (0.1,  0.5,  laby,  lenlaby, 2, 90, 0);

   /*** Determine minor * major scaling values for axes. ***/

   nicexr_ (&xmin, &xmax, &majrx, &minrx, &dxx, &dxn);
   printf ("majrx:%d  minrx:%d  dxx:%f  dxn:%f\n", majrx, minrx, dxx, dxn);

   nicexr_ (&ymin, &ymax, &majry, &minry, &dyy, &dyn);
   printf ("majry:%d  minry:%d  dyy:%f  dyn:%f\n", majry, minry, dyy, dyn);

   set (0.25, 0.95, 0.15, 0.85, xmin, xmax, ymin, ymax, ltype);

   switch (ibkg)
      {
      case 0: break;			/* Do not change. */
      default:
      case 1: igph = 5;  break;		/* perimeter with tick marks. */
      case 2: igph = 0;  break;		/* full grid. */
      case 3: igph = 10; break;		/* half axes. */
      }
/*
 *    periml (majrx, minrx, majry, minry);
 */
   gridal (majrx, minrx, majry, minry, ixlab, iylab, igph, xmin, ymin);

   /*** Draw curve through data points. ***/

   curve (x, y, nval);
   frame ();

   return (0);
}
