c* nlf.f
c* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c* Test nllst2 with tangent fit.
c* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
c* Andrew L. Stanger   NCAR/HAO   SMM C/P
c* 5 March 1987
c* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c*/

      parameter (MAXIMG=7, MAXPAR=3)
      integer pcount

      character*40  namemc, title, labx, laby
      integer ixscale
      integer iyscale
      integer ibkg
      integer i
      integer nimg
      integer npar
      integer ierr
      real expose(MAXIMG)
      real ybuf(MAXIMG)

      real yfit(MAXIMG)
      real alphin
      real betain
      real offin
      real par(MAXPAR)
      real sig(6)
      logical fix(MAXPAR)
      real alpha, beta, offset
      real chisq

      data nimg /MAXIMG/
      data npar /MAXPAR/
      data pcount /0/
      data ixscale /0/, iyscale /0/,ibkg /0/
      data expose /1.,3.,5.,8.,12.,17.,25./
      data ybuf /40.,80.,115.,150.,180.,200.,210./
      data alphin /24./, betain /177./, offin /5./

      data namemc/'nlf.mc'/
      data title/'Tangent Fit$'/
      data labx /'E x p o s u r e (*1.024 sec)$'/
      data laby /'Vidicon Digital Number (DN)$'/
c* . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . . */

      fix(1) = .false.
      fix(2) = .false.
      fix(3) = .false.

c* Initial values for non-linear least-squares fit. */

         par(1) = alphin
         par(2) = betain
         par(3) = offin

c* Non-linear least squares fit. */

      call nllst2 (ybuf,expose,par,sig,fix,npar,nimg,chisq,ierr)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c      subroutine nllst2(y,x,c,sig,fix,na,m,chisq,ier)
c
c        y         = array of independent quantities
c        x         = array of dependent quantities
c        c         = estimates of the  fitting parameters
c        fix       - logical array
c                  if fix(i) = .true. then c(i) is not corrected
c        na        = number of parameters in fit (including those fixed)
c        m         = number of data points
c
c     User supplied subroutines :
c        func(x,c,na)        func = value at x given parameters c
c        deriv(x,c,na,f,df)  f = value at x given parameters c
c                            df(i) = df/dc(i)
c
c     output :
c        c         = least squares fitted parameters
c        sig       = estimate of standard error in c.
c        chisq     = estimate of unit weight variance of fit
c        ier       = 1 - - > fit is ok
c                  = 2 - - > fit is suspicious
c                  = 3 - - > more than 25 iterations required
c                  = 4 - - > disaster, no fit found
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      alpha = par(1)
      beta = par(2)
      offset = par(3)

      do 10 i = 1, nimg
         yfit(i) = beta * atan2 (expose(i), alpha) + offset
   10 continue

      write (*,*) 'alpha, beta, offset: ', alpha, beta, offset
      write (*,*) 'sig: ', sig(0), sig(1), sig(2)
      write (*,*) 'fix: ', fix(0), fix(1), fix(2)
      write (*,*) 'chisq, ierr: ', chisq, ierr

      call tanplt (expose, ybuf, yfit, nimg, namemc, ltitle, labx, laby,
     .  	   ixscale, iyscale, ibkg, pcount)

      write (*,*) 'Exp.Time    DN     fit(DN)' 
      do 20 i = 1, nimg
         write (*,15) expose(i), ybuf(i), yfit(i)
   15    format (1x,f5.1,3x,f5.1,3x,f5.1)
   20 continue

      stop
      end
